// Type definitions for yFiles for HTML 2.0.1
// Project: http://www.yworks.com/products/yfiles-for-html
// Definitions by: yWorks GmbH HTML Team <https://github.com/yGuy>
// Definitions: https://github.com/yWorks/DefinitelyTyped

/****************************************************************************
 **
 ** This file is part of yFiles for HTML 2.0.
 **
 ** yWorks proprietary/confidential. Use is subject to license terms.
 **
 ** Copyright (c) 2017 by yWorks GmbH, Vor dem Kreuzberg 28,
 ** 72070 Tuebingen, Germany. All rights reserved.
 **
 ***************************************************************************/

declare namespace yfiles{
  namespace lang{ 
  interface ClassConstructor<T> {
      new ():T;
  }
  function BaseClass<T>(...base:any[]):yfiles.lang.ClassConstructor<T>;
  }
}

declare namespace yfiles{
  export namespace lang{
    /**
     * The root of the class hierarchy of the yFiles for HTML class framework.
     * <p>
     * Two of these methods are actually defined on the native JavaScript <code>object</code> prototype:
     * </p>
     * <ul>
     * <li>{@link yfiles.lang.Object_Interface#hashCode}</li>
     * <li>{@link yfiles.lang.Object_Interface#equals}</li>
     * </ul>
     * <p>
     * The other methods are part of the {@link yfiles.lang.Object_Interface} type, only.
     * </p>
     * @class
     */
    export interface Object_Interface extends Object{}
    export class Object_Interface {
      /**
       * Tests whether this object and the other object are equal.
       * @param other The tested object.
       * @returns <code>true</code>, if the objects are equal; <code>false</code> otherwise.
       */
      equals(other:Object):boolean;
      /**
       * Tests if two objects are equal.
       * @param a One of the objects. Can be null or of any type.
       * @param b One of the objects. Can be null or of any type.
       * @returns <code>true</code>, if the objects are equal; <code>false</code> otherwise.
       * @static
       */
      static equals(a:Object,b:Object):boolean;
      /**
       * Returns the type of this object.
       * @returns The type of this object.
       */
      getClass():yfiles.lang.Class;
      /**
       * Returns an object with a get and/or a set method that is optionally bound to this instance.
       * <p>
       * The requested <code>name</code> must be a property (with getter and/or setter).
       * </p>
       * @param name The name of the property.
       * @param bound If <code>true</code>, then the get/set methods of the returned object will be bound to this instance.
       * @returns An object with a get and/or set method that reads/writes to the specified field.
       */
      getOwnProperty(name:string,bound:boolean):Object;
      /**
       * Returns the most recently defined property descriptor in the prototype chain of the given object.
       * @param o The object that the property belongs to
       * @param name The name of the property.
       * @returns The most recently defined property descriptor in the prototype chain of the given object.
       * @static
       */
      static getPropertyDescriptor(o:Object,name:string):Object;
      /**
       * Returns the hash code of this object.
       * <p>
       * If two objects are {@link yfiles.lang.Object_Interface#equals equal}, then they must have the same hash code.
       * </p>
       * @returns The hash code of this object.
       */
      hashCode():number;
      /**
       * Shallow clones an object.
       * <p>
       * It will also clone any value types.
       * </p>
       * @returns A cloned instance of the same type.
       */
      memberwiseClone():Object;
      /**
       * Tests if two objects are identical.
       * @param a One of the objects.
       * @param b One of the objects.
       * @returns <code>true</code>, if the objects are the identical; <code>false</code> otherwise.
       * @static
       */
      static referenceEquals(a:Object,b:Object):boolean;
      /**
       * Displays the textual representation of the object.
       * @returns The textual representation of this object.
       */
      toString():string;
      static $class:yfiles.lang.Class;
    }
    /**
     * This type represents an abstract value, that is a function that is not implemented.
     * @class
     */
    export interface Abstract extends Object{}
    export class Abstract {
      /**
       * Tests whether the given object or property descriptor is a representation of an abstract value.
       * @param o The object to test.
       * @returns <code>true</code>, if the object is abstract; <code>false</code> otherwise.
       * @static
       */
      static isAbstract(o:Object):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A function that transforms a class definition object into a class type.
     * <p>
     * The class definition object may contain any of the following special identifiers:
     * </p>
     * <ul>
     * <li><code>$extends</code>: An object reference that points to the parent object.</li>
     * <li><code>$with</code>: An array of traits this class implements.</li>
     * <li><code>$abstract</code>: A boolean value that indicates whether or not this class is abstract.</li>
     * <li>
     * <code>$static</code>: An object that contains methods, fields and properties that should be statically available and will be
     * registered on the constructor function instead of its prototype.
     * </li>
     * <li>
     * <code>$meta</code>: An array of {@link yfiles.lang.Attribute}s or a function that returns an array of
     * {@link yfiles.lang.Attribute}s that describe the class.
     * </li>
     * </ul>
     * <p>
     * Additionally, it may contain methods, properties and fields that should be added to the constructor functions prototype.
     * If the value of an field is an object with a <code>$meta</code> property, then it will be used as a property descriptor for the field
     * (meaning that <code>writable</code>, <code>enumerable</code> and <code>configurable</code>, <code>value</code> and <code>get</code>, <code>set</code> can be used as in a property descriptor for <code>Object.defineProperty</code>
     * and the value of the <code>$meta</code> property is stored for usage in the reflection API. If the object has a <code>get</code> and/or <code>set</code>
     * property, then it will also be used as a property descriptor.
     * </p>
     * <pre>
     * var MyList = yfiles.lang.Class('MyList', {
     *     $extends: my.AbstractList,
     *     $with: [my.Iterable, my.Collection],
     * 
     *     constructor: function() { // the constructor function
     *     },
     * 
     *     add: function(obj) { ... },
     *     size: { // transformed into a property
     *      'get': function() { return ...; }
     *     },
     * 
     *     $static: {
     *      create: function() { ... }
     *     }
     * });
     * // usage
     * var list = new MyList();
     * list.add("hello");
     * list.size === 1; // true
     * </pre>
     * 
     * <p>
     * The constructor property can be either a function (in case of a single constructor) or an object that is transformed
     * into named constructors.
     * </p>
     * <pre>
     * var Rect = yfiles.lang.Class('Rect', {
     *  constructor: {
     *      'default': function(x, y, width, height) { ... },
     *      'FromPoints': function(topLeft, bottomRight) { ... },
     *      'FromPoint': function(topLeft, width, height) { ... }
     *  }
     * });
     * // usage:
     * var myrect = new Rect(10, 10, 50, 50);
     * myrect instanceof Rect; // true
     * myrect = new Rect.FromPoints({x: 10, y: 10}, {x: 60, y: 60});
     * myrect instanceof Rect; // true
     * myrect = new Rect.FromPoint({x: 10, y: 10}, 50, 50);
     * myrect instanceof Rect; // true
     * </pre>
     * 
     * <p>
     * Named constructors can call other constructors and can be written like any other constructor, i.e. <code>this</code> will be a new
     * instance of the class that is being constructed.
     * </p>
     * <p>
     * To access super methods, you can use the static <code>$super</code> property that is added to all classes.
     * </p>
     * <p>
     * Example (inside a method or property of the previously defined Rect class):
     * </p>
     * <pre>
     * Rect.$super.toString.call(this);
     * </pre>
     * 
     * <p>
     * If you do not specify the parent class of a new class, then {@link yfiles.lang.Object_Interface} will be used as its base type. The parent
     * class of {@link yfiles.lang.Object_Interface} is the standard JavaScript <code>Object.prototype</code>.
     * </p>
     * <p>
     * Every class has a couple of static methods and properties:
     * </p>
     * <ul>
     * <li>{@link yfiles.lang.Class#isInstance}: tests whether an object is an instance of this class.</li>
     * <li><code>$class</code>: a reference to the {@link yfiles.lang.Class} instance that provides reflective information about this type.</li>
     * <li><code>$super</code>: the parent class.</li>
     * </ul>
     * @class
     */
    export interface Class extends Object{}
    export class Class {
      /**
       * Returns the class object for the given name, or <code>null</code> if no such class exists.
       * @param name The name of the requested class.
       * @returns The class object or <code>null</code>, if no such class exists.
       * @static
       */
      static forName(name:string):yfiles.lang.Class;
      /**
       * Returns all attributes that belong to this type.
       * @param [inherited=true] If <code>true</code>, then the attributes of the parent types are also added to the list. Default value is <code>false</code>.
       * @returns An array of attributes that belong to this type.
       */
      getAttributes(inherited?:boolean):yfiles.lang.Attribute[];
      /**
       * Returns an array of all properties of the type.
       * @param [inherited=true]
       * @param [staticProperties=false]
       * @returns An array of properties.
       */
      getProperties(inherited?:boolean,staticProperties?:boolean):yfiles.lang.PropertyInfo[];
      /**
       * Returns an array of all properties of the type.
       * @param {Object} options The parameters to pass.
       * @param [options.inherited=true]
       * @param [options.staticProperties=false]
       * @returns An array of properties.
       */
      getProperties(options:{inherited?:boolean,staticProperties?:boolean}):yfiles.lang.PropertyInfo[];
      /**
       * Modify an existing object <code>obj</code> that is not created with the help of the yfiles class framework to be usable with our
       * library functions.
       * <p>
       * This method modifies a given object <code>obj</code> in the following ways:
       * </p>
       * <ul>
       * <li>Methods and properties that implement member declarations of the <code>traits</code> are callable from yfiles library code.</li>
       * <li>For each interface <code>trait</code> in <code>traits</code>, <code>trait.isInstance(obj)</code> is true.</li>
       * </ul>
       * <p>
       * Note that this method does not enable extended usages, e.g. <code>obj</code> does not implement the full reflection API necessary for
       * GraphML serialization, and it is not possible to inject actual classes as opposed to interfaces.
       * </p>
       * <p>
       * To modify an object on an instance level:
       * </p>
       * <pre>
       * //Create a plain JavaScript object
       * var myHitTestable = {};
       * myHitTestable.isHit = function(p, ctx) { return false; };
       * myHitTestable.lookup = function(clazz) { return null; };
       * //Modify
       * yfiles.lang.Class.injectInterfaces(myHitTestable, [yfiles.drawing.IHitTestable, yfiles.support.ILookup]);
       * //These are equally valid:
       * //yfiles.lang.Class.injectInterfaces(myHitTestable, ["yfiles.drawing.IHitTestable", "yfiles.support.ILookup"]);
       * //yfiles.lang.Class.injectInterfaces(myHitTestable, [yfiles.drawing.IHitTestable.$class, yfiles.support.ILookup.$class]);
       * //As well as using a variable number of arguments, such as:
       * //yfiles.lang.Class.injectInterfaces(myHitTestable, yfiles.drawing.IHitTestable, yfiles.support.ILookup);
       * 
       * //This is true now:
       * yfiles.drawing.IHitTestable.isInstance(myHitTestable);
       * //And the object can be used where an interface instance is expected:
       * graphEditorInputMode.clickInputMode.validClickHitTestable = myHitTestable;
       * </pre>
       * 
       * <p>
       * It is also possible to modify an object prototype, e.g. in TypeScript: Class definition:
       * </p>
       * <pre>
       * export class MyHitTestable implements yfiles.drawing.IHitTestable {
       *    isHit(p: yfiles.geometry.PointD, ctx: yfiles.canvas.ICanvasContext): boolean {
       *        return false;
       *    }
       * }
       * </pre>
       * 
       * <p>
       * Usage:
       * </p>
       * <pre>
       * //This is false:
       * //yfiles.drawing.IHitTestable.isInstance(new MyHitTestable());
       * //Modify
       * yfiles.lang.Class.injectInterfaces(MyHitTestable.prototype, [yfiles.drawing.IHitTestable]);
       * //Now it is true
       * var myHitTestable = new MyHitTestable();
       * yfiles.drawing.IHitTestable.isInstance(myHitTestable);
       * //And the object can be used where an interface instance is expected:
       * graphEditorInputMode.clickInputMode.validClickHitTestable = myHitTestable;
       * </pre>
       * 
       * @param obj The object to modify.
       * @param traits Array or variable number of {@link }s, {@link } objects of traits or fully qualified string names of
       * traits.
       * @static
       */
      static injectInterfaces(obj:Object,...traits:Object[]):void;
      /**
       * Tests whether an object is an instance of this type.
       * <p>
       * This is the only way to test if an object implements a {@link yfiles.lang.Trait}.
       * </p>
       * @param o The object that may be an instance of this type.
       * @returns <code>true</code>, if the object is an instance of this type; <code>false</code> otherwise.
       */
      isInstance(o:Object):boolean;
      /**
       * Creates a new instance of this type using its default constructor.
       * @returns A new instance of this type using its default constructor.
       */
      newInstance():Object;
      /**
       * <code>true</code>, if this type represents an enum; <code>false</code> otherwise.
       * @type {boolean}
       */
      isEnum:boolean;
      /**
       * <code>true</code>, if this type represents a primitive type; <code>false</code> otherwise.
       * <p>
       * The primitive types are: number, string and boolean.
       * </p>
       * @type {boolean}
       */
      isPrimitive:boolean;
      /**
       * <code>true</code>, if this type represents a value type; <code>false</code> otherwise.
       * <p>
       * A value type is either a primitive or an instance of a {@link yfiles.lang.Struct}.
       * </p>
       * @type {boolean}
       */
      isValueType:boolean;
      /**
       * <code>true</code>, if this type represents a regular class; <code>false</code> otherwise.
       * @type {boolean}
       */
      isClass:boolean;
      /**
       * The short name of this class.
       * @type {string}
       */
      name:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * Wrapper class for the JavaScript <code>number</code> primitive type.
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface Number extends Object,yfiles.lang.IComparable{}
    export class Number {
      /**
       * Compares this number with another one.
       * @param obj The other number to compare with.
       * @returns <code>0</code> if both numbers are equal, <code>-1</code> if this number is smaller than the other one, <code>1</code> otherwise.
       */
      compareTo(obj:Object):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Wrapper class for the JavaScript <code>string</code> primitive type.
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface String extends Object,yfiles.lang.IComparable{}
    export class String {
      /**
       * Compares this string with another one.
       * @param obj The other string to compare with.
       * @returns <code>0</code> if both strings are equal, <code>-1</code> if this string comes before the other one in lexicographic sort order, <code>1</code> otherwise.
       */
      compareTo(obj:Object):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Wrapper class for the JavaScript <code>boolean</code> primitive type.
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface Boolean extends Object,yfiles.lang.IComparable{}
    export class Boolean {
      /**
       * Compares this boolean with another one.
       * @param obj The other boolean to compare with.
       * @returns <code>0</code> if both booleans are equal, <code>-1</code> if this boolean is <code>false</code> and the other is <code>true</code>, <code>1</code> otherwise.
       */
      compareTo(obj:Object):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base type for types that provide value type semantics, which means that they are cloned when their containing type is
     * cloned.
     * <p>
     * They provide a static {@link yfiles.lang.Struct#createDefault} method that can be used to create an instance of an struct where
     * all of its values are set to their default without using any of its constructors.
     * </p>
     * <p>
     * Besides those properties, they can be treated like any other class, i.e. they can implement {@link yfiles.lang.Trait Traits} and can be tested
     * with <code>a instanceof A</code> and <code>A.$class.isInstance(a)</code>.
     * </p>
     * @see yfiles.lang.Struct#createDefault
     * @class
     */
    export interface Struct extends Object{}
    export class Struct {
      /**
       * This methods creates an instance of the struct in which all of its values are set to their default values without using
       * any of its constructors.
       * @returns 
       * @static
       */
      static createDefault():Object;
      /**
       * Tests whether another object is equal to this struct.
       * <p>
       * As opposed to the implementation of
       * {@link yfiles.lang.Object_Interface#equals}, this implementation tests the members of both objects to determine equality.
       * </p>
       * @param other The other struct.
       * @returns <code>true</code>, if both objects are equal; <code>false</code> otherwise.
       */
      equals(other:Object):boolean;
      /**
       * Calculates the hash code of this struct based on its members.
       * @returns The hash code.
       */
      hashCode():number;
      static $class:yfiles.lang.Class;
    }
    /**
     * The base type of an enumeration. It is not type safe.
     * <p>
     * All enums extend {@link yfiles.lang.Enum} and should be used instead of "magic values".
     * </p>
     * <p>
     * The members of the enum type definition will be static and an enum can not be constructed.
     * </p>
     * <pre>
     * var RequestState = yfiles.lang.Enum('RequestState', {
     *     SEND: 1,
     *     PROCESSED: 2,
     *     SHIPPED: 3,
     *     FAILED: 4,
     *    });
     *    if(state === RequestState.FAILED) { ... }
     * </pre>
     * 
     * @class
     */
    export interface Enum extends Object{}
    export class Enum {
      /**
       * Returns the name of an enum value.
       * @param type The type of the enum.
       * @param value The value of the field.
       * @returns The name of the field.
       * @static
       */
      static getName(type:yfiles.lang.Class,value:yfiles.lang.Enum):string;
      /**
       * Returns a list of all field names that represent an enum value.
       * @param type The type of the enum.
       * @returns A list of all field names that represent an enum value.
       * @static
       */
      static getValueNames(type:yfiles.lang.Class):yfiles.collections.IEnumerable<Object>;
      /**
       * Returns a list of all numeric values.
       * @param type The type of the enum.
       * @returns A list of all numeric values.
       * @static
       */
      static getValues(type:yfiles.lang.Class):yfiles.collections.IEnumerable<Object>;
      /**
       * Returns the value of the field with the given ID.
       * @param type The type of the enum.
       * @param id The name of the field.
       * @param ignoreCase If <code>true</code>, then the case of the field will be ignored and the search is case-insensitive.
       * @returns The value of the field with the given ID.
       * @static
       */
      static parse(type:yfiles.lang.Class,id:string,ignoreCase:boolean):yfiles.lang.Enum;
      static $class:yfiles.lang.Class;
    }
    /**
     * Wrapper around JavaScript function objects.
     * <p>
     * Compared with plain JavaScript functions,
     * </p>
     * <ul>
     * <li>delegates can be bound to a different target, which acts as <code>this</code> in the delegate function body,</li>
     * <li>multiple delegates can be combined into a single delegate instance, which provides multicast functionality.</li>
     * </ul>
     * @class
     */
    export interface delegate extends Object{}
    export class delegate {
      /**
       * Combines two delegates into a single one.
       * @param d1 One of the delegates to combine.
       * @param d2 One of the delegates to combine.
       * @returns A delegate that combines both original delegates.
       * @static
       */
      static combine(d1:Function,d2:Function):Function;
      /**
       * Creates a new delegate from the given <code>functions</code> and the execution target.
       * <p>
       * When the delegate is executed, the <code>functions</code> will be <code>applied</code> on
       * <code>target</code>, independent from the current closure. If multiple functions are specified, they will all be applied in order.
       * </p>
       * @param functions The functions from which the delegate should be created.
       * @param target The target object on which the <code>functions</code> should be applied.
       * @returns A new delegate.
       * @static
       */
      static createDelegate(functions:Function[],target:Object):Function;
      /**
       * Apply <code>function</code> in the <b>current</b> execution context.
       * @param function2 The function to execute.
       * @param args Optional function arguments.
       * @returns The result from invoking <code>function</code>.
       * @static
       */
      static dynamicInvoke(function2:Function,...args:Object[]):Object;
      /**
       * Return a list of all delegates that are combined into <code>del</code>.
       * @param del The delegate to test.
       * @returns A list of all delegates that are combined into <code>del</code>.
       * @static
       */
      static getInvocationList(del:Function):Function[];
      /**
       * Removes the first occurrence of <code>d2</code> from the {@link yfiles.lang.delegate#getInvocationList invocation list} of <code>d1</code>.
       * @param d1 The delegate that should be modified.
       * @param d2 The delegate that should be removed.
       * @returns The modified delegate <code>d1</code>.
       * @static
       */
      static remove(d1:Function,d2:Function):Function;
      /**
       * Removes all occurrences of <code>d2</code> from the {@link yfiles.lang.delegate#getInvocationList invocation list} of <code>d1</code>.
       * @param d1 The delegate that should be modified.
       * @param d2 The delegate that should be removed.
       * @returns The modified delegate <code>d1</code>.
       * @static
       */
      static removeAll(d1:Function,d2:Function):Function;
      static $class:yfiles.lang.Class;
    }
    /**
     * A trait is an interface that may or may not have default implementations of its methods, properties and fields.
     * <p>
     * Traits have very specific rules in case two traits with the same member are implemented by a subtype:
     * </p>
     * <ul>
     * <li>If both method definitions are abstract, then the method is abstract.</li>
     * <li>If one method definition is abstract and the other concrete, then the concrete method is used.</li>
     * <li>If both method definitions are concrete and the traits are implemented at the same level, then the method is abstract.</li>
     * <li>
     * If both method definitions are concrete and one of the traits is a sub-type of the other, then its method definition
     * will be used.
     * </li>
     * </ul>
     * <p>
     * In all of those cases, the rule is that a subtype overrides the parent types' definitions. Example: Given the traits A,
     * B, C and D where A defines a method "a" to be abstract and B and C extending trait A with concrete implementations of
     * "a", when D is a trait that implements both B and C, its version of "a" will be abstract, unless it defines its own
     * implementation of "a", in which case that definition is used.
     * </p>
     * <p>
     * All trait types have a <code>$class</code> property, which contains their {@link yfiles.lang.Class type}, and a static {@link yfiles.lang.Trait#isTrait} method that
     * can be used to test whether an object implements the trait.
     * </p>
     * @class
     */
    export interface Trait extends Object{}
    export class Trait {
      /**
       * Tests whether the given object is a trait.
       * <p>
       * This method is equivalent to using <code>obj.$class.isTrait</code> but safe in case the object does not have a <code>$class</code> property.
       * </p>
       * @param o The trait candidate.
       * @returns <code>true</code>, if the object is a trait; <code>false</code> otherwise.
       * @static
       */
      static isTrait(o:Object):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specialized type that can be used to describe meta information about other types, methods, fields, properties and
     * constructors.
     * <p>
     * All attributes extend {@link yfiles.lang.Attribute} or a subtype of an attribute. They can also implement {@link yfiles.lang.Trait Traits}.
     * </p>
     * <p>
     * The actual creation of an attribute is delayed to the point in time when it is needed by the reflection API. Therefore,
     * the attribute constructors are wrapped and replaced by a function that creates an instance of the attribute.
     * </p>
     * <pre>
     * var MyAttribute = yfiles.lang.Attribute('MyAttribute', {
     *  constructor: function(message) {
     *      this.message = message;
     *  }
     * });
     * var MyClass = yfiles.lang.Class('MyClass', {
     *  $meta: [MyAttribute('hello').init(recipient: 'world')]
     * });
     * var attr = yfiles.lang.Attribute.getCustomAttribute(MyClass.$class, MyAttribute.$class);
     * console.log(attr.message + ' ' + attr.recipient); // 'hello world'
     * </pre>
     * 
     * <p>
     * Even though MyAttribute is invoked at the definition of MyClass, no attribute is created until the reflection API is
     * used (in the above example: using getCustomAttribute).
     * </p>
     * <p>
     * The result of the constructor function of an attribute is a function, that, when invoked, returns an instance of the
     * attribute. It also provides an init method that adds the properties of the given object to the attribute.
     * </p>
     * <p>
     * <b>Note:</b> Attribute constructors are always called <b>without</b> the <code>new</code> keyword.
     * </p>
     * @class
     */
    export interface Attribute extends Object{}
    export class Attribute {
      static $class:yfiles.lang.Class;
    }
    /**
     * A lazy class type which delays the creation of the class until it is used.
     * @class
     */
    export interface ClassDefinition extends Object{}
    export class ClassDefinition {
      /**
       * A lazy class type which delays the creation of the class until it is used.
       * @param callback Returns the class definition as required by {@link }.
       */
      constructor(callback:()=>Object);
      static $class:yfiles.lang.Class;
    }
    /**
     * A lazy interface type which delays the creation of the interface until it is used.
     * @class
     */
    export interface InterfaceDefinition extends Object{}
    export class InterfaceDefinition {
      /**
       * A lazy interface type which delays the creation of the interface until it is used.
       * @param callback Returns the interface definition as required by {@link }.
       */
      constructor(callback:()=>Object);
      static $class:yfiles.lang.Class;
    }
    /**
     * A lazy struct type which delays the creation of the struct until it is used.
     * @class
     */
    export interface StructDefinition extends Object{}
    export class StructDefinition {
      /**
       * A lazy struct type which delays the creation of the struct until it is used.
       * @param callback Returns the struct definition as required by {@link }.
       */
      constructor(callback:()=>Object);
      static $class:yfiles.lang.Class;
    }
    /**
     * A lazy enum type which delays the creation of the enum until it is used.
     * @class
     */
    export interface EnumDefinition extends Object{}
    export class EnumDefinition {
      /**
       * A lazy enum type which delays the creation of the enum until it is used.
       * @param callback Returns the enum definition as required by {@link }.
       */
      constructor(callback:()=>Object);
      static $class:yfiles.lang.Class;
    }
    /**
     * A lazy attribute definition which delays the creation of the attribute until it is used.
     * @class
     */
    export interface AttributeDefinition extends Object{}
    export class AttributeDefinition {
      /**
       * A lazy attribute definition which delays the creation of the attribute until it is used.
       * @param callback Returns the attribute definition as required by {@link }.
       */
      constructor(callback:()=>Object);
      static $class:yfiles.lang.Class;
    }
    /**
     * Subclasses of {@link yfiles.lang.EventArgs} are passed to event handlers to provide the information they need to handle an
     * event.
     * @class
     */
    export interface EventArgs extends Object{}
    export class EventArgs {
      /**
       * An empty {@link yfiles.lang.EventArgs} object that can be used when no information needs to be passed to the event handler.
       * @const
       * @static
       * @type {yfiles.lang.EventArgs}
       */
      static EMPTY:yfiles.lang.EventArgs;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments which contain information for a {@link yfiles.lang.IPropertyObservable#addPropertyChangedListener PropertyChanged}
     * event.
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface PropertyChangedEventArgs extends yfiles.lang.EventArgs{}
    export class PropertyChangedEventArgs {
      /**
       * Creates a new instance.
       * @param propertyName The name of the changed property.
       */
      constructor(propertyName:string);
      /**
       * The name of the changed property.
       * @type {string}
       */
      propertyName:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * Defines an event which is dispatched when a property changes.
     * @interface
     */
    export interface IPropertyObservable extends Object{
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property changes.
       * @param listener The listener to add.
       * @see yfiles.lang.IPropertyObservable#removePropertyChangedListener
       * @abstract
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property changes.
       * @param listener The listener to remove.
       * @see yfiles.lang.IPropertyObservable#addPropertyChangedListener
       * @abstract
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
    }
    var IPropertyObservable:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Contains information about a property.
     * @class
     * @final
     */
    export interface PropertyInfo extends Object{}
    export class PropertyInfo {
      /**
       * Gets the attributes associated with this property.
       * @param [inherit=true] if set to <code>true</code> inherited attributes will also be returned.
       * @returns The attributes or an empty array.
       */
      getAttributes(inherit?:boolean):yfiles.lang.Attribute[];
      /**
       * Gets the value of the property from the given object.
       * @param obj The object which has the property, <code>null</code> for static properties.
       * @returns The value of the property from the given object.
       */
      getValue(obj:Object):Object;
      /**
       * Sets the value of the property on the given object.
       * @param obj The object whose property should be written, <code>null</code> for static properties.
       * @param value
       */
      setValue(obj:Object,value:Object):void;
      /**
       * Gets a value indicating whether the property can be read.
       * @type {boolean}
       */
      canRead:boolean;
      /**
       * Gets a value indicating whether the property can be written.
       * @type {boolean}
       */
      canWrite:boolean;
      /**
       * Gets a value indicating whether the property is a static property, rather than declared on an instance.
       * @type {boolean}
       */
      isStatic:boolean;
      /**
       * Gets the type of the property.
       * @type {yfiles.lang.Class}
       */
      propertyType:yfiles.lang.Class;
      /**
       * Gets the declaring type, i.e. the type where the member was declared.
       * @type {yfiles.lang.Class}
       */
      declaringType:yfiles.lang.Class;
      /**
       * Gets the name of the member.
       * @type {string}
       */
      name:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides a method by which implementors can be compared to each other.
     * @interface
     */
    export interface IComparable extends Object{
      /**
       * Compares this object to the given object of the same type.
       * @param obj The object to compare this to.
       * @returns 
       * <ul>
       * <li>-1: this is less than <code>obj</code></li>
       * <li>0: this is equal to <code>obj</code></li>
       * <li>1: this is greater than <code>obj</code></li>
       * </ul>
       * @abstract
       */
      compareTo(obj:Object):number;
    }
    var IComparable:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Implemented by objects which can create a clone of themselves.
     * @interface
     */
    export interface ICloneable extends Object{
      /**
       * Create a clone of this object.
       * @returns A clone of this object.
       * @abstract
       */
      clone():Object;
    }
    var ICloneable:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A structure which represents a time span.
     * @class
     * @implements {yfiles.lang.IComparable}
     * @final
     */
    export interface TimeSpan extends yfiles.lang.Struct,yfiles.lang.IComparable{}
    export class TimeSpan {
      /**
       * Creates a new time span with a given length.
       * @param days The days part of the time span.
       * @param hours The hour part of the time span.
       * @param minutes The minute part of the time span.
       * @param seconds The seconds part of the time span.
       * @param millis The milliseconds part of the time span.
       */
      constructor(days:number,hours:number,minutes:number,seconds:number,millis:number);
      /**
       * Creates a new time span which is <code>ticks</code> milliseconds long.
       * @param ticks The length of the time span in milliseconds.
       */
      constructor(ticks:number);
      /**
       * 
       * @param obj
       * @returns 
       */
      compareTo(obj:Object):number;
      /**
       * Creates a new time span with the given length in milliseconds.
       * @param milliseconds The length of the time span in milliseconds.
       * @returns A newly created time span.
       * @static
       */
      static fromMilliseconds(milliseconds:number):yfiles.lang.TimeSpan;
      /**
       * Creates a new time span with the given length in minutes.
       * @param minutes The length of the time span in minutes.
       * @returns A newly created time span.
       * @static
       */
      static fromMinutes(minutes:number):yfiles.lang.TimeSpan;
      /**
       * Creates a new time span with the given length in seconds.
       * @param seconds The length of the time span in seconds.
       * @returns A newly created time span.
       * @static
       */
      static fromSeconds(seconds:number):yfiles.lang.TimeSpan;
      /**
       * Empty time span.
       * @const
       * @static
       * @type {yfiles.lang.TimeSpan}
       */
      static ZERO:yfiles.lang.TimeSpan;
      /**
       * The total milliseconds (ticks) of this time span.
       * @type {number}
       */
      totalMilliseconds:number;
      /**
       * The total seconds of this time span.
       * @type {number}
       */
      totalSeconds:number;
      /**
       * The total seconds of this time span.
       * @type {number}
       */
      totalMinutes:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Extension of the native JavaScript <code>Error</code> type that includes a stack trace.
     * <p>
     * yFiles includes the following specialized exception types indicated by the {@link yfiles.lang.Exception#name}:
     * </p>
     * <ul>
     * <li><b>ArgumentError</b> – Indicates that the value of a parameter of the current method is illegal.</li>
     * <li><b>ArgumentNullError</b> – Indicates that the value of a parameter of the current method is <code>null</code> but a non-<code>null</code> value was expected.</li>
     * <li><b>ArgumentOutOfRangeError</b> – Indicates that the value of a parameter of the current method is not in the legal range of values.</li>
     * <li><b>IndexOutOfRangeError</b> – Indicates that an array index is not in the legal range.</li>
     * <li><b>NullReferenceError</b> – Indicates that a value is <code>null</code> when it shouldn't.</li>
     * <li><b>InvalidOperationError</b> – Indicates that the current operation is not valid in the instances's current state.</li>
     * <li><b>NotImplementedError</b> – Indicates that the called method is not implemented.</li>
     * <li><b>NotSupportedError</b> – Indicates that the operation is not supported.</li>
     * <li><b>ArithmeticError</b> – Indicates an error in an arithmetic operation.</li>
     * <li><b>IOError</b> – Indicates an error during an IO operation.</li>
     * <li><b>KeyNotFoundError</b> – Indicates that a mapping does not contain the given key.</li>
     * <li><b>AlgorithmAbortedError</b> – Indicates that an immediate termination request was detected by an algorithm.</li>
     * <li>
     * <b>InvalidGraphStructureError</b> – Indicates that a graph cannot be used with a certain algorithm. Some graph algorithms require a specific graph
     * structure, for example trees or planar graph. These algorithms throw this error if a non-matching graph is used.
     * </li>
     * </ul>
     * <p>
     * This type also provides the static {@link yfiles.lang.Exception#catchErrors} property, which when enabled, causes most entry points (e.g. event listeners) to use
     * try/catch blocks to catch possible errors that could be thrown during event handling. The errors are handled by the {@link yfiles.lang.Exception#handleError}
     * function, which calls the {@link yfiles.lang.Exception#handler} function, if it has been set. Note that for some browsers/plugins, try/catch blocks prevent
     * error debugging functionality. Hence, {@link yfiles.lang.Exception#catchErrors} should be turned off while debugging.
     * </p>
     * @class
     * @extends {Error}
     */
    export interface Exception extends Error{}
    export class Exception {
      /**
       * Creates a new exception with the given {@link yfiles.lang.Exception#message} and an optional error {@link yfiles.lang.Exception#name}.
       * @param [message=null] The exception message.
       * @param [name=null] The exception name. This name can be used in try...catch statements to distinguish between different types of errors.
       * @param [innerException=null] The inner exception, if another exception was wrapped.
       * @see yfiles.lang.Exception#message
       */
      constructor(message?:string,name?:string,innerException?:yfiles.lang.Exception);
      /**
       * Creates a new exception with the given {@link yfiles.lang.Exception#message} and an optional error {@link yfiles.lang.Exception#name}.
       * @param {Object} options The parameters to pass.
       * @param [options.message=null] The exception message.
       * @param [options.name=null] The exception name. This name can be used in try...catch statements to distinguish between different types of errors.
       * @param [options.innerException=null] The inner exception, if another exception was wrapped.
       * @see yfiles.lang.Exception#message
       */
      constructor(options:{message?:string,name?:string,innerException?:yfiles.lang.Exception});
      /**
       * This function is called in catch blocks in the library if {@link yfiles.lang.Exception#catchErrors} is enabled.
       * <p>
       * If a {@link yfiles.lang.Exception#handler} is registered, the error is passed to the error handler, otherwise if {@link yfiles.lang.Exception#catchErrors} is set to <code>true</code> the error is logged to the
       * console using <code>console.warn</code> if available.
       * </p>
       * @param error The error object that has been caught
       * @static
       */
      static handleError(error:Object):void;
      /**
       * Gets the exception message.
       * @type {string}
       */
      message:string;
      /**
       * Gets the exception name.
       * @type {string}
       */
      name:string;
      /**
       * The inner exception, if another exception was wrapped.
       * @type {yfiles.lang.Exception}
       */
      innerException:yfiles.lang.Exception;
      /**
       * Whether to use try/catch blocks when invoking event handlers.
       * <p>
       * If this property is enabled, try/catch blocks are used when invoking event handlers. As most browsers provide a stack
       * trace with Error objects, this property can be used to provide more detailed error information. If this property is set,
       * but no {@link yfiles.lang.Exception#handler} is registered, a warning will be logged to the console instead.
       * </p>
       * @static
       * @type {boolean}
       */
      static catchErrors:boolean;
      /**
       * If {@link yfiles.lang.Exception#catchErrors} is enabled, any caught errors are passed to this function.
       * <p>
       * Setting a handler via this property automatically turns on the {@link yfiles.lang.Exception#catchErrors} flag!
       * </p>
       * @static
       * @type {function(Object): void}
       */
      static handler:(obj:Object)=>void;
      static $class:yfiles.lang.Class;
    }
  }export namespace collections{
    /**
     * The default implementation for {@link yfiles.collections.IMap.<TKey,TValue>}.
     * Type parameter <code>TKey</code>: The type of the key elements of this dictionary.
     * Type parameter <code>TValue</code>: The type of the value elements of this dictionary.
     * @class
     * @implements {yfiles.collections.IMap.<TKey,TValue>}
     * @implements {yfiles.collections.ICollection.<yfiles.collections.MapEntry.<TKey,TValue>>}
     * @implements {yfiles.collections.IEnumerable.<yfiles.collections.MapEntry.<TKey,TValue>>}
     * @template TKey,TValue
     */
    export interface Map<TKey,TValue> extends Object,yfiles.collections.IMap<TKey,TValue>,yfiles.collections.ICollection<yfiles.collections.MapEntry<TKey,TValue>>,yfiles.collections.IEnumerable<yfiles.collections.MapEntry<TKey,TValue>>{}
    export class Map<TKey,TValue> {
      /**
       * Creates a new instance.
       * @param {Object} [options=null] The parameters to pass.
       * @param options.entries An array of items which are added to this map. Each item will be added via the {@link #add} method.
       */
      constructor(options?:{entries?:Array<any>});
      /**
       * 
       * @param item
       */
      add(item:yfiles.collections.MapEntry<TKey,TValue>):void;
      /**
       * 
       */
      clear():void;
      /**
       * Whether this dictionary contains the given <code>value</code>.
       * @param value The value to search for.
       * @returns <code>true</code> if this dictionary contains the given value.
       */
      containsValue(value:TValue):boolean;
      /**
       * 
       * @param array
       * @param arrayIndex
       */
      copyTo(array:yfiles.collections.MapEntry<TKey,TValue>[],arrayIndex:number):void;
      /**
       * 
       * @param key
       * @returns 
       */
      delete(key:TKey):boolean;
      /**
       * 
       * @param key
       * @returns 
       */
      get(key:TKey):TValue;
      /**
       * 
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<yfiles.collections.MapEntry<TKey,TValue>>;
      /**
       * 
       * @param key
       * @returns 
       */
      has(key:TKey):boolean;
      /**
       * 
       * @param item
       * @returns 
       */
      includes(item:yfiles.collections.MapEntry<TKey,TValue>):boolean;
      /**
       * 
       * @param item
       * @returns 
       */
      remove(item:yfiles.collections.MapEntry<TKey,TValue>):boolean;
      /**
       * 
       * @param key
       * @param value
       */
      set(key:TKey,value:TValue):void;
      /**
       * 
       * @type {number}
       */
      size:number;
      /**
       * 
       * @type {boolean}
       */
      isReadOnly:boolean;
      /**
       * 
       * @type {yfiles.collections.ICollection.<TKey>}
       */
      keys:yfiles.collections.ICollection<TKey>;
      /**
       * 
       * @type {yfiles.collections.ICollection.<TValue>}
       */
      values:yfiles.collections.ICollection<TValue>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Defines methods to manipulate a collection of objects with the same type.
     * Type parameter <code>T</code>: The type of the elements in the collection.
     * @interface
     * @implements {yfiles.collections.IEnumerable.<T>}
     * @template T
     */
    export interface ICollection<T> extends Object,yfiles.collections.IEnumerable<T>{
      /**
       * Adds the given <code>item</code> to the collection.
       * @param item
       * @abstract
       */
      add(item:T):void;
      /**
       * Removes all items from this collection.
       * @abstract
       */
      clear():void;
      /**
       * Copies all elements of this collection into the given array.
       * @param array The array to copy the elements to.
       * @param arrayIndex The index in the given array where the first element should be copied to.
       * @abstract
       */
      copyTo(array:T[],arrayIndex:number):void;
      /**
       * Whether the given <code>item</code> is contained in this collection.
       * @param item The item to search for.
       * @returns <code>true</code> if the given <code>item</code> is contained in this collection.
       * @abstract
       */
      includes(item:T):boolean;
      /**
       * Removes the given <code>item</code> from this collection.
       * @param item The item to remove.
       * @returns Whether the item was removed from the collection.
       * @abstract
       */
      remove(item:T):boolean;
      /**
       * The number of elements in this collection.
       * @abstract
       * @type {number}
       */
      size:number;
      /**
       * Whether this collection is read-only.
       * @abstract
       * @type {boolean}
       */
      isReadOnly:boolean;
    }
    var ICollection:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A collection of value objects which are mapped to key objects and can be queried using their keys.
     * <p>
     * The keys must have a well defined <code>hashCode</code> method that must not change between registering a value for the key and
     * retrieving a value using the key. Thus it is advisable to use stable, i.e. immutable, objects as the key.
     * </p>
     * Type parameter <code>TKey</code>: The type of the keys.
     * Type parameter <code>TValue</code>: The type of the values.
     * @interface
     * @implements {yfiles.collections.ICollection.<yfiles.collections.MapEntry.<TKey,TValue>>}
     * @template TKey,TValue
     */
    export interface IMap<TKey,TValue> extends Object,yfiles.collections.ICollection<yfiles.collections.MapEntry<TKey,TValue>>{
      /**
       * Removes the key/value pair which is represented by the given <code>key</code>.
       * @param key The key of the key/value pair to remove.
       * @returns <code>true</code> if a key/value pair with the given key was removed from this collection.
       * @abstract
       */
      delete(key:TKey):boolean;
      /**
       * Gets the value of the key/value pair with the given key.
       * <p>
       * Setter: if there is already a key/value pair with the given key in the dictionary its value will be overridden. If not a
       * new key/value pair will be added.
       * <br />
       * Getter: if there is no key/value pair with the given key in this dictionary <code>null</code> will be returned.
       * </p>
       * @param key
       * @returns 
       * @see yfiles.collections.IMap.<TKey,TValue>#set
       * @abstract
       */
      get(key:TKey):TValue;
      /**
       * Whether this dictionary's key collection contains the given key.
       * @param key The key to search for.
       * @returns <code>true</code> if this dictionary contains the given <code>key</code>.
       * @abstract
       */
      has(key:TKey):boolean;
      /**
       * Sets the value of the key/value pair with the given key.
       * <p>
       * Setter: if there is already a key/value pair with the given key in the dictionary its value will be overridden. If not a
       * new key/value pair will be added.
       * <br />
       * Getter: if there is no key/value pair with the given key in this dictionary <code>null</code> will be returned.
       * </p>
       * @param key
       * @param value
       * @see yfiles.collections.IMap.<TKey,TValue>#get
       * @abstract
       */
      set(key:TKey,value:TValue):void;
      /**
       * A {@link yfiles.collections.ICollection.<T>} of the keys of this dictionary.
       * @abstract
       * @type {yfiles.collections.ICollection.<TKey>}
       */
      keys:yfiles.collections.ICollection<TKey>;
      /**
       * A {@link yfiles.collections.ICollection.<T>} of the values of this dictionary.
       * @abstract
       * @type {yfiles.collections.ICollection.<TValue>}
       */
      values:yfiles.collections.ICollection<TValue>;
    }
    var IMap:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Defines ways to iterate over the items contained in this type by providing a method to get an {@link yfiles.collections.IEnumerator.<T>}.
     * Type parameter <code>T</code>: The type of the enumerable objects.
     * @interface
     * @template T
     */
    export interface IEnumerable<T> extends Object{
      /**
       * 
       * @param enumerable
       * @returns 
       */
      concat?(enumerable:yfiles.collections.IEnumerable<T>):yfiles.collections.IEnumerable<T>;
      /**
       * Returns the element at the given <code>index</code> in the enumerable.
       * @param index The index of the element to return.
       * @returns The element at the given <code>index</code>.
       * @throws {Stubs.Exceptions.ArgumentError} If the <code>index</code> is not within the enumerable's range.
       */
      elementAt?(index:number):T;
      /**
       * Whether all elements of the enumerable match the given <code>predicate</code>.
       * @param predicate A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the element matches a condition.
       * @param [thisArg=null]
       * @returns Whether all elements of the enumerable match the given <code>predicate</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>e</code> is <code>null</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>predicate</code> is <code>null</code>.
       */
      every?(predicate:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object):boolean;
      /**
       * Returns an enumerable which only contains elements of the source enumerable which match a given <code>predicate</code>.
       * @param predicate A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be included in
       * the resulting enumerable.
       * @param [thisArg=null]
       * @returns An enumerable which contains the elements of the source enumeration which match the given <code>predicate</code>.
       */
      filter?(predicate:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object):yfiles.collections.IEnumerable<T>;
      /**
       * Returns the first element of the enumerable which matches a given filter.
       * @param [predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be returned
       * @param [thisArg=null]
       * @returns The first matching element of the enumerable.
       * @throws {Stubs.Exceptions.ArgumentError} If the given enumerable is <code>null</code>.
       * @throws {Stubs.Exceptions.InvalidOperationError} If the enumerable does not contain a matching element.
       */
      first?(predicate?:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object):T;
      /**
       * Returns the first element of the enumerable which matches a given filter.
       * @param {Object} options The parameters to pass.
       * @param [options.predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be returned
       * @param [options.thisArg=null]
       * @returns The first matching element of the enumerable.
       * @throws {Stubs.Exceptions.ArgumentError} If the given enumerable is <code>null</code>.
       * @throws {Stubs.Exceptions.InvalidOperationError} If the enumerable does not contain a matching element.
       */
      first?(options:{predicate?:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object}):T;
      /**
       * Returns the first element of the enumerable, the first element of the enumerable which matches a given filter if
       * sepcified, or <code>null</code> if there is no such element.
       * @param [predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be returned
       * @param [thisArg=null]
       * @returns The first element of the enumerable or <code>null</code>.
       * @throws {Stubs.Exceptions.ArgumentError} If the given enumerable is <code>null</code>.
       */
      firstOrDefault?(predicate?:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object):T;
      /**
       * Returns the first element of the enumerable, the first element of the enumerable which matches a given filter if
       * sepcified, or <code>null</code> if there is no such element.
       * @param {Object} options The parameters to pass.
       * @param [options.predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the given element should be returned
       * @param [options.thisArg=null]
       * @returns The first element of the enumerable or <code>null</code>.
       * @throws {Stubs.Exceptions.ArgumentError} If the given enumerable is <code>null</code>.
       */
      firstOrDefault?(options:{predicate?:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object}):T;
      /**
       * Iterates over the enumerable and invokes the action for each element passing the value and index as arguments.
       * @param action The delegate to call.
       * @param [thisArg=null]
       */
      forEach?(action:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>void,thisArg?:Object):void;
      /**
       * Gets an {@link yfiles.collections.IEnumerator.<T>} which can be used to iterate over the items in this instance
       * @returns The {@link } which can be used to iterate over the items in this instance
       * @abstract
       */
      getEnumerator():yfiles.collections.IEnumerator<T>;
      /**
       * Whether the given value is contained in the enumerable.
       * @param value The value to search for.
       * @returns <code>true</code> if the enumerable contains the given value.
       */
      includes?(value:T):boolean;
      /**
       * Returns the last element of the enumerable.
       * @returns The last element of the enumerable.
       * @throws {Stubs.Exceptions.ArgumentError} If the given enumerable is <code>null</code>.
       * @throws {Stubs.Exceptions.InvalidOperationError} If the enumerable is empty.
       */
      last?():T;
      /**
       * Returns the last element of the enumerable or <code>null</code> if the enumerable is empty.
       * @returns The last element of the enumerable or <code>null</code>.
       * @throws {Stubs.Exceptions.ArgumentError} If the given enumerable is <code>null</code>.
       */
      lastOrDefault?():T;
      /**
       * Maps each element of the enumerable into a new element in a new enumerable.
       * Type parameter <code>TResult</code>: The type of the mapped elements.
       * @param selector A function with the signature <code>function(element, index):TResult</code> which converts each element into a new element of the
       * type <code>TResult</code> depending on the element's index.
       * @param [thisArg=null] The optional value to use as <code>this</code> when executing the <code>selector</code>.
       * @returns A projection of the original enumerable.
       * @template TResult
       */
      map?<TResult>(selector:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>TResult,thisArg?:Object):yfiles.collections.IEnumerable<TResult>;
      /**
       * Converts a untyped enumerable into an enumerable with the given type.
       * <p>
       * The source enumerable may contain elements which are not of the given type. Those elements will not be contained in the
       * returned enumerable.
       * </p>
       * Type parameter <code>TResult</code>.
       * @returns A typed enumerable.
       * @template TResult
       */
      ofType?<TResult>(resultType:yfiles.lang.Class):yfiles.collections.IEnumerable<TResult>;
      /**
       * Applies an <code>func</code> function over the enumerable's elements.
       * Type parameter <code>TAccumulate</code>: The type of the accumulated value.
       * @param seed The start value for the accumulator.
       * @param func A function with the signature <code>function(seed,element):TAccumulate</code> which "adds" (accumulates) a value depending on the
       * element to the seed value and returns the result.
       * @returns The final value of the accumulator.
       * @template TAccumulate
       */
      reduce?<TAccumulate>(seed:TAccumulate,func:(arg1:TAccumulate,arg2:T,arg3:number,arg4:yfiles.collections.IEnumerable<T>)=>TAccumulate):TAccumulate;
      /**
       * Creates a flattened view of a given enumerable using the given <code>selector</code> function which returns an enumerable for each
       * element of the source enumerable.
       * Type parameter <code>TResult</code>.
       * @param selector A function with the signature <code>function(element):IEnumerable</code> which returns an enumerable for each element of the source
       * enumerable.
       * @param [thisArg=null]
       * @returns A flattened view of the given enumerable. The enumerables which are returned by the <code>selector</code> method for each element are
       * concatenated in the result.
       * @template TResult
       */
      selectMany?<TResult>(selector:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>yfiles.collections.IEnumerable<TResult>,thisArg?:Object):yfiles.collections.IEnumerable<TResult>;
      /**
       * Whether the enumerable contains any elements matching the given <code>predicate</code>.
       * @param [predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the element matches a condition.
       * @param [thisArg=null]
       * @returns Whether the enumerable contains any elements matching the given <code>predicate</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>e</code> is <code>null</code>.
       */
      some?(predicate?:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object):boolean;
      /**
       * Whether the enumerable contains any elements matching the given <code>predicate</code>.
       * @param {Object} options The parameters to pass.
       * @param [options.predicate=null] A function with the signature <code>function(element):boolean</code> which returns <code>true</code> if the element matches a condition.
       * @param [options.thisArg=null]
       * @returns Whether the enumerable contains any elements matching the given <code>predicate</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>e</code> is <code>null</code>.
       */
      some?(options:{predicate?:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object}):boolean;
      /**
       * Calculates the sum of the elements of the enumerable.
       * @param selector A function with the signature <code>function(element):Number</code> which returns a numeric value for the given element.
       * @param [thisArg=null]
       * @returns The sum of the elements of the enumerable.
       */
      sum?(selector:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>number,thisArg?:Object):number;
      /**
       * Returns elements from an enumerable as long as the given <code>predicate</code> is <code>true</code>.
       * @param predicate A function with the signature <code>function(element):boolean</code> which returns <code>true</code> as long as the elements should be added to
       * the returned enumerable.
       * @param [thisArg=null]
       * @returns A subset of the original enumerable.
       */
      takeWhile?(predicate:(arg1:T,arg2:number,arg3:yfiles.collections.IEnumerable<T>)=>boolean,thisArg?:Object):yfiles.collections.IEnumerable<T>;
      /**
       * Creates an array with the values of the enumerable.
       * @returns An array with the enumerable's elements.
       */
      toArray?():T[];
      /**
       * Creates a {@link yfiles.collections.List.<T>} with the values of the enumerable.
       * @returns An {@link } with the enumerable's elements.
       */
      toList?():yfiles.collections.List<T>;
      /**
       * Returns an enumerable which contains the elements of the original enumerable in reverse order.
       * @returns An enumerable which contains the elements of the original enumerable in reverse order.
       */
      toReversed?():yfiles.collections.IEnumerable<T>;
      /**
       * Returns the number of elements contained in the enumerable.
       * @type {number}
       */
      size?:number;
    }
    var IEnumerable:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides methods to iterate over a collection or list of elements of the same type.
     * <pre>
     * var e = someList.getEnumerator();
     * // always move before you access
     * while(e.moveNext()) {
     *   // get the current element
     *   var current = e.current;
     * }
     * </pre>
     * 
     * Type parameter <code>T</code>.
     * @interface
     * @template T
     */
    export interface IEnumerator<T> extends Object{
      /**
       * Moves this enumerator to the next element.
       * <p>
       * A newly created enumerator's {@link yfiles.collections.IEnumerator.<T>#current current position} is before the first element. Thus, this method must be called before first
       * access to the {@link yfiles.collections.IEnumerator.<T>#current} property. The same applies to the state after calling {@link yfiles.collections.IEnumerator.<T>#reset}.
       * </p>
       * @returns <code>true</code> if the enumerator was moved to a valid position
       * @abstract
       */
      moveNext():boolean;
      /**
       * Resets the enumerator to its starting state.
       * <p>
       * In other words: sets the current position before the first element.
       * </p>
       * @abstract
       */
      reset():void;
      /**
       * Gets the element at the enumerator's current position.
       * <p>
       * This value is undefined if the enumerator is in initial state (after creation or
       * {@link yfiles.collections.IEnumerator.<T>#reset}) or has been moved past the end of the represented collection.
       * </p>
       * @abstract
       * @type {T}
       */
      current:T;
    }
    var IEnumerator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A read-write collection of objects of the same type.
     * Type parameter <code>T</code>: The type of objects in the list.
     * @interface
     * @implements {yfiles.collections.ICollection.<T>}
     * @template T
     */
    export interface IList<T> extends Object,yfiles.collections.ICollection<T>{
      /**
       * Gets the item at the given index.
       * @param index The index of the item to access.
       * @returns The item at the given index.
       * @see yfiles.collections.IList.<T>#set
       * @abstract
       */
      get(index:number):T;
      /**
       * The index of the given item in the list.
       * @param item The item to search for.
       * @returns The index of the given item in the list. <code>-1</code> if the item is not in the list.
       * @abstract
       */
      indexOf(item:T):number;
      /**
       * Inserts the given <code>item</code> at the given <code>index</code>.
       * @param index The index to insert the item at.
       * @param item The item to insert.
       * @abstract
       */
      insert(index:number,item:T):void;
      /**
       * Removes the item at the given index from the list.
       * @param index The index of the item to remove.
       * @abstract
       */
      removeAt(index:number):void;
      /**
       * Sets the item at the given index.
       * @param index The index of the item to access.
       * @param value The item at the given index.
       * @see yfiles.collections.IList.<T>#get
       * @abstract
       */
      set(index:number,value:T):void;
    }
    var IList:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The key/value pair of a {@link yfiles.collections.IMap.<TKey,TValue>}.
     * Type parameter <code>TKey</code>: The type of the key element.
     * Type parameter <code>TValue</code>: The type of the value element.
     * @class
     * @template TKey,TValue
     * @final
     */
    export interface MapEntry<TKey,TValue> extends Object{}
    export class MapEntry<TKey,TValue> {
      /**
       * Creates a new instance with the given <code>key</code> and <code>value</code>.
       * @param key The key for this pair.
       * @param value The value for this pair.
       */
      constructor(key:TKey,value:TValue);
      /**
       * The key for this key/value pair.
       * @type {TKey}
       */
      key:TKey;
      /**
       * The value for this key/value pair.
       * @type {TValue}
       */
      value:TValue;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default implementation of {@link yfiles.collections.IList.<T>}.
     * <p>
     * This implementation provides fast random indexed access. However, removing or adding elements from the beginning or in
     * the middle of the list will slower.
     * </p>
     * Type parameter <code>T</code>: The type of the list's elements.
     * @class
     * @implements {yfiles.collections.IList.<T>}
     * @template T
     */
    export interface List<T> extends Object,yfiles.collections.IList<T>{}
    export class List<T> {
      /**
       * Creates a new instance and fills it with the elements of the given <code>enumerable</code>.
       * @param enumerable The enumerable to fill the list with.
       */
      constructor(enumerable:yfiles.collections.IEnumerable<T>);
      /**
       * Creates a new instance.
       * @param {Object} [options=null] The parameters to pass.
       * @param options.items An array of items which are added to this list. Each item will be added via the {@link #add} method.
       */
      constructor(options?:{items?:Array<any>});
      /**
       * 
       * @param item
       */
      add(item:T):void;
      /**
       * Adds the elements of the given collection to this list.
       * @param enumerable The collection to add.
       */
      addRange(enumerable:yfiles.collections.IEnumerable<T>):void;
      /**
       * Searches the (sorted) list for the given <code>item</code>.
       * <p>
       * This search is done as a binary search, therefore the list must be sorted. The given <code>comparer</code> is used to compare the
       * elements to each other.
       * </p>
       * @param item The item to search for.
       * @param comparer The comparer to use for comparing the elements.
       * @returns The index of the item in the list. -1 if the item is not found.
       */
      binarySearch(item:T,comparer:yfiles.collections.IComparer<T>):number;
      /**
       * 
       */
      clear():void;
      /**
       * 
       * @param array
       * @param arrayIndex
       */
      copyTo(array:T[],arrayIndex:number):void;
      /**
       * Searches the list for the first element for which <code>match</code> returns <code>true</code>.
       * @param match A predicate function with the signature <code>function(o:Object):boolean</code> which returns <code>true</code> if o matches the conditions of the
       * element to search for.
       * @returns The first element for which <code>match</code> returns <code>true</code> or <code>null</code> is no such element in the list.
       */
      find(match:(obj:T)=>boolean):T;
      /**
       * Searches the list for the first element for which <code>match</code> returns <code>true</code> and returns its index.
       * @param match A predicate function with the signature <code>function(o:Object):boolean</code> which returns <code>true</code> if o matches the conditions of the
       * element to search for.
       * @returns The index of the first element for which <code>match</code> returns <code>true</code> or -1 if there is no such element in the list.
       */
      findIndex(match:(obj:T)=>boolean):number;
      /**
       * Creates a new list with the given array used as the live backing store.
       * <p>
       * Modifying the list or the array will modify both instances.
       * </p>
       * @param array The array to create a list view over from.
       * @returns A list backed by the given array.
       * @static
       */
      static fromArray<T>(array:T[]):yfiles.collections.List<T>;
      /**
       * 
       * @param index
       * @returns 
       */
      get(index:number):T;
      /**
       * 
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<T>;
      /**
       * 
       * @param item
       * @returns 
       */
      includes(item:T):boolean;
      /**
       * 
       * @param item
       * @returns 
       */
      indexOf(item:T):number;
      /**
       * 
       * @param index
       * @param item
       */
      insert(index:number,item:T):void;
      /**
       * 
       * @param item
       * @returns 
       */
      remove(item:T):boolean;
      /**
       * Removes all elements for which <code>match</code> returns <code>true</code>.
       * @param match A predicate function with the signature <code>function(o:Object):boolean</code> which returns <code>true</code> if o matches the conditions of the
       * element to search for.
       * @returns The number of elements which were removed.
       */
      removeAll(match:(obj:T)=>boolean):number;
      /**
       * 
       * @param index
       */
      removeAt(index:number):void;
      /**
       * Removes a given <code>count</code> of elements starting from the given <code>index</code>.
       * @param index The index of the first element to remove.
       * @param count The number of elements to remove.
       */
      removeRange(index:number,count:number):void;
      /**
       * Reverses the order of the elements in the list.
       */
      reverse():void;
      /**
       * 
       * @param index
       * @param value
       */
      set(index:number,value:T):void;
      /**
       * Sorts all elements in the list using the given <code>comparison</code> function.
       * @param comparison The comparison function to use.
       */
      sort(comparison:(arg1:Object,arg2:Object)=>number):void;
      /**
       * Sorts all elements in the list using the given <code>comparer</code>.
       * @param comparer The comparer to use.
       */
      sort(comparer:yfiles.collections.IComparer<T>):void;
      /**
       * Sorts all elements in list using the default comparer.
       */
      sort():void;
      /**
       * Copies the elements of this list into an array.
       * @returns An array containing the elements of this list.
       */
      toArray():T[];
      /**
       * 
       * @type {number}
       */
      size:number;
      /**
       * 
       * @type {boolean}
       */
      isReadOnly:boolean;
      capacity:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides a method to compare two objects of the same type.
     * Type parameter <code>T</code>: The type of the objects to compare.
     * @interface
     * @template T
     */
    export interface IComparer<T> extends Object{
      /**
       * Compares two objects of type T.
       * @param x The first object.
       * @param y The second object.
       * @returns 
       * <ul>
       * <li>-1: <code>x</code> is less than <code>y</code></li>
       * <li>0: <code>x</code> is equal to <code>y</code></li>
       * <li>1: <code>x</code> is greater than <code>y</code></li>
       * </ul>
       * @abstract
       */
      compare(x:T,y:T):number;
    }
    var IComparer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A simple default implementation of an {@link yfiles.collections.IObservableCollection.<T>} that is backed by an ordinary {@link yfiles.collections.ICollection.<T>}
     * Type parameter <code>T</code>: The type of the elements contained in this collection.
     * @class
     * @implements {yfiles.collections.IObservableCollection.<T>}
     * @template T
     */
    export interface ObservableCollection<T> extends Object,yfiles.collections.IObservableCollection<T>{}
    export class ObservableCollection<T> {
      /**
       * Creates a new instance using a provided collection as the backing store.
       * @param [items=null] The collection to use as the backing store. If no collection is provided a {@link } will be used.
       */
      constructor(items?:yfiles.collections.ICollection<T>);
      /**
       * Adds the item the the collection and triggers the {@link yfiles.collections.ObservableCollection.<T>#addItemAddedListener ItemAdded} event.
       * <p>
       * This implementation calls the {@link yfiles.collections.ObservableCollection.<T>#onItemAdded} method.
       * </p>
       * @param item The item to add to the collection.
       */
      add(item:T):void;
      /**
       * Clears this collection, triggering an {@link yfiles.collections.ObservableCollection.<T>#addItemRemovedListener ItemRemoved} event for each element removed.
       * @see yfiles.collections.ObservableCollection.<T>#onItemRemoved
       */
      clear():void;
      /**
       * Copies the items to the given array.
       * @param array
       * @param arrayIndex
       */
      copyTo(array:T[],arrayIndex:number):void;
      /**
       * Returns an enumerator over the elements in this collection.
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<T>;
      /**
       * Checks for containment of the item in this collection.
       * @param item The item to check.
       * @returns Whether the item is contained in this collection.
       */
      includes(item:T):boolean;
      /**
       * Callback method that will trigger the {@link yfiles.collections.ObservableCollection.<T>#addItemAddedListener ItemAdded} event.
       * @param item The item that has just been added to the collection.
       * @protected
       */
      onItemAdded(item:T):void;
      /**
       * Callback method that will trigger the {@link yfiles.collections.ObservableCollection.<T>#addItemChangedListener ItemChanged} event.
       * @param item The item that has changed.
       * @protected
       */
      onItemChanged(item:T):void;
      /**
       * Callback method that will trigger the {@link yfiles.collections.ObservableCollection.<T>#addItemRemovedListener ItemRemoved} event.
       * @param item The item that has just been removed from the collection.
       * @protected
       */
      onItemRemoved(item:T):void;
      /**
       * Method that may be called by client code to trigger an {@link yfiles.collections.ObservableCollection.<T>#addItemChangedListener ItemChanged} event.
       * @param item The item to trigger the change event for.
       * @see yfiles.collections.ObservableCollection.<T>#onItemChanged
       */
      publishItemChanged(item:T):void;
      /**
       * Tries to remove the item from the collection and triggers the {@link yfiles.collections.ObservableCollection.<T>#addItemRemovedListener ItemRemoved} event.
       * @param item The item to remove.
       * @returns Whether the item has been removed.
       * @see yfiles.collections.ObservableCollection.<T>#onItemRemoved
       */
      remove(item:T):boolean;
      /**
       * Returns the number of elements in this collection.
       * @type {number}
       */
      size:number;
      /**
       * Checks whether this collection is read only.
       * @type {boolean}
       */
      isReadOnly:boolean;
      /**
       * Gets the collection used internally as the backing store.
       * <p>
       * Normally code should not use this method to gain access to the collection in order to modify it.
       * </p>
       * @returns The backing store of this observable collection.
       * @type {yfiles.collections.ICollection.<T>}
       */
      items:yfiles.collections.ICollection<T>;
      /**
       * Adds the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
       * @param listener The listener to add.
       * @see yfiles.collections.ObservableCollection.<T>#removeItemAddedListener
       */
      addItemAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
       * @param listener The listener to remove.
       * @see yfiles.collections.ObservableCollection.<T>#addItemAddedListener
       */
      removeItemAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
       * @param listener The listener to add.
       * @see yfiles.collections.ObservableCollection.<T>#removeItemRemovedListener
       */
      addItemRemovedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
       * @param listener The listener to remove.
       * @see yfiles.collections.ObservableCollection.<T>#addItemRemovedListener
       */
      removeItemRemovedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemChanged</code> event that occurs when an item has been changed inside this collection.
       * @param listener The listener to add.
       * @see yfiles.collections.ObservableCollection.<T>#removeItemChangedListener
       */
      addItemChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemChanged</code> event that occurs when an item has been changed inside this collection.
       * @param listener The listener to remove.
       * @see yfiles.collections.ObservableCollection.<T>#addItemChangedListener
       */
      removeItemChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.collections.IMapper.<K,V>} implementation backed by a {@link yfiles.collections.IMap.<TKey,TValue>}.
     * <p>
     * A {@link yfiles.collections.Mapper.<K,V>#defaultValue} can be set that is returned for keys that don't have an explicit value set.
     * </p>
     * <p>
     * It is legal to use <code>null</code> as key, which can be assigned a value just like other keys.
     * </p>
     * Type parameter <code>K</code>.
     * Type parameter <code>V</code>.
     * @class
     * @implements {yfiles.collections.IMapper.<K,V>}
     * @template K,V
     * @final
     */
    export interface Mapper<K,V> extends Object,yfiles.collections.IMapper<K,V>{}
    export class Mapper<K,V> {
      /**
       * Creates an instance using the provided {@link yfiles.collections.Map.<K,V>} as the backing store.
       * <p>
       * The value for the null item is stored separately.
       * </p>
       * @param dictionary
       */
      constructor(dictionary:yfiles.collections.IMap<K,V>);
      /**
       * Creates an instance using the provided {@link yfiles.collections.Map.<K,V>} as the backing store.
       * <p>
       * The value for the null item is stored separately.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.dictionary
       * @param {V} options.defaultValue The default value to return if no specific value has been set for a given key.
       * <p>
       * This option sets the {@link yfiles.collections.Mapper.<K,V>#defaultValue} property on the created object.
       * </p>
       */
      constructor(options:{dictionary:yfiles.collections.IMap<K,V>,defaultValue?:V});
      /**
       * Creates an instance using a {@link yfiles.collections.Map.<K,V>} as the backing store.
       * <p>
       * The value for the null item is stored separately.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {V} options.defaultValue The default value to return if no specific value has been set for a given key.
       * <p>
       * This option sets the {@link yfiles.collections.Mapper.<K,V>#defaultValue} property on the created object.
       * </p>
       */
      constructor(options?:{defaultValue?:V});
      /**
       * Calls {@link yfiles.collections.Map.<TKey,TValue>#clear} on the backing implementation.
       */
      clear():void;
      /**
       * 
       * @param key
       */
      delete(key:K):void;
      /**
       * 
       * @param key
       * @returns 
       */
      get(key:K):V;
      /**
       * 
       * @param key
       * @param value
       */
      set(key:K,value:V):void;
      /**
       * Gets an enumerable over all entries in this (dictionary based) mapper.
       * @type {yfiles.collections.IEnumerable.<yfiles.collections.MapEntry.<K,V>>}
       */
      entries:yfiles.collections.IEnumerable<yfiles.collections.MapEntry<K,V>>;
      /**
       * Gets or sets the default value to return if no specific value has been set for a given key.
       * @type {V}
       */
      defaultValue:V;
      static $class:yfiles.lang.Class;
    }
    /**
     * The interface for a collection that will notify registered event handlers of changes to its contents.
     * Type parameter <code>T</code>: The type of the items in the collection.
     * @see yfiles.collections.ObservableCollection.<T>
     * @interface
     * @implements {yfiles.collections.ICollection.<T>}
     * @template T
     */
    export interface IObservableCollection<T> extends Object,yfiles.collections.ICollection<T>{
      /**
       * Adds the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
       * @param listener The listener to add.
       * @see yfiles.collections.IObservableCollection.<T>#removeItemAddedListener
       * @abstract
       */
      addItemAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemAdded</code> event that occurs when an item has been added to this collection.
       * @param listener The listener to remove.
       * @see yfiles.collections.IObservableCollection.<T>#addItemAddedListener
       * @abstract
       */
      removeItemAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
       * @param listener The listener to add.
       * @see yfiles.collections.IObservableCollection.<T>#removeItemRemovedListener
       * @abstract
       */
      addItemRemovedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemRemoved</code> event that occurs when an item has been removed from this collection.
       * @param listener The listener to remove.
       * @see yfiles.collections.IObservableCollection.<T>#addItemRemovedListener
       * @abstract
       */
      removeItemRemovedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemChanged</code> event that occurs when an item in this collection has changed
       * significantly.
       * <p>
       * It is up to the implementation whether and when to trigger this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.collections.IObservableCollection.<T>#removeItemChangedListener
       * @abstract
       */
      addItemChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemChanged</code> event that occurs when an item in this collection has changed
       * significantly.
       * <p>
       * It is up to the implementation whether and when to trigger this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.collections.IObservableCollection.<T>#addItemChangedListener
       * @abstract
       */
      removeItemChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
    }
    var IObservableCollection:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An extension to the {@link yfiles.collections.IEnumerable.<T>} interface that allows for indexed access and retrieval of the element count.
     * <p>
     * This interface is like a rudimentary read-only {@link yfiles.collections.IList.<T>}
     * </p>
     * Type parameter <code>T</code>: The type of the contained elements.
     * @interface
     * @implements {yfiles.collections.IEnumerable.<T>}
     * @template T
     */
    export interface IListEnumerable<T> extends Object,yfiles.collections.IEnumerable<T>{
      /**
       * Gets the <code>i</code>-th element in the collection.
       * @param i the zero-based index of the item in this collection
       * @returns the item for the given index
       * @throws {Stubs.Exceptions.ArgumentError} <code>i</code> is not a valid index in the {@link yfiles.collections.IListEnumerable.<T>}.
       * @abstract
       */
      get(i:number):T;
      /**
       * Gets the number of elements in this collection.
       * @abstract
       * @type {number}
       */
      size:number;
    }
    var IListEnumerable:{
      /**
       * An empty, immutable instance of {@link yfiles.collections.IListEnumerable.<T>}.
       * @const
       * @static
       * @type {yfiles.collections.IListEnumerable.<any>}
       */
      EMPTY?:yfiles.collections.IListEnumerable<any>;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Wraps a simple {@link yfiles.collections.IEnumerable.<T>} into a {@link yfiles.collections.IListEnumerable.<T>}.
     * <p>
     * This implementation is optimized for {@link yfiles.collections.IList.<T>} and {@link yfiles.collections.ICollection.<T>}, as well as {@link Object} {@link yfiles.collections.IEnumerable.<T>}s.
     * </p>
     * Type parameter <code>T</code>: The type of the enumerable.
     * @class
     * @implements {yfiles.collections.IListEnumerable.<T>}
     * @template T
     * @final
     */
    export interface ListEnumerable<T> extends Object,yfiles.collections.IListEnumerable<T>{}
    export class ListEnumerable<T> {
      /**
       * Creates a new instance using the given backing enumerable.
       * @param backingEnumerable The enumerable to back this instance.
       */
      constructor(backingEnumerable:yfiles.collections.IEnumerable<T>);
      /**
       * Creates a new instance with a backing {@link yfiles.collections.List.<T>} as the backing store.
       */
      constructor();
      /**
       * 
       * @param i
       * @returns 
       */
      get(i:number):T;
      /**
       * 
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<T>;
      /**
       * Gets the backing enumerable.
       * @type {yfiles.collections.IEnumerable.<T>}
       */
      backingEnumerable:yfiles.collections.IEnumerable<T>;
      /**
       * 
       * @type {number}
       */
      size:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event argument class used by {@link yfiles.collections.IObservableCollection.<T>} and similar that holds a specific item that is related to the event.
     * Type parameter <code>T</code>: The type of the item contained in the event.
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @template T
     */
    export interface ItemEventArgs<T> extends yfiles.lang.EventArgs{}
    export class ItemEventArgs<T> {
      /**
       * Creates a new event argument for the given item.
       * @param item The item which is the subject of the event.
       */
      constructor(item:T);
      /**
       * Gets the item that is the subject of the event.
       * @type {T}
       */
      item:T;
      static $class:yfiles.lang.Class;
    }
    /**
     * An interface for simple typed dictionary-like implementations.
     * <p>
     * {@link yfiles.collections.IMapper.<K,V>}s are most prominently used when interfacing with {@link yfiles.layout.ILayoutAlgorithm layout algorithms} and {@link yfiles.graphml.GraphMLIOHandler GraphML}. {@link yfiles.graph.IGraph} also provides the {@link yfiles.graph.IGraph#mapperRegistry}
     * to add additional information to a graph.
     * </p>
     * Type parameter <code>K</code>: The type of element this interface accepts as keys for the mapping.
     * Type parameter <code>V</code>: The type of element this interface accepts as values for the mapping.
     * <p>
     * Using {@link yfiles.collections.IMapper.<K,V>} is described in the section {@link @DGUIDE_PREFIX@/customizing_graph-binding_data.html Associating Data with Graph Elements}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: DragAndDropForm, demo.yfiles.input.draganddrop</li>
     * </ul>
     * @see yfiles.collections.Mapper.<K,V>
     * @interface
     * @template K,V
     */
    export interface IMapper<K,V> extends Object{
      /**
       * Gets the value for a given key in the mapping.
       * <p>
       * Setting a value for an already existing key overwrites the previous value.
       * </p>
       * <p>
       * It depends on the specific implementation of the mapper whether a mapping can be removed. If an implementation provides
       * a way to remove a mapping, that should be preferred over setting a <code>null</code> value. For example, {@link yfiles.collections.Mapper.<K,V>} provides the {@link yfiles.collections.Mapper.<K,V>#delete}
       * method.
       * </p>
       * @param key The key of the mapping.
       * @returns The value.
       * @see yfiles.collections.IMapper.<K,V>#set
       * @abstract
       */
      get(key:K):V;
      /**
       * Sets the value for a given key in the mapping.
       * <p>
       * Setting a value for an already existing key overwrites the previous value.
       * </p>
       * <p>
       * It depends on the specific implementation of the mapper whether a mapping can be removed. If an implementation provides
       * a way to remove a mapping, that should be preferred over setting a <code>null</code> value. For example, {@link yfiles.collections.Mapper.<K,V>} provides the {@link yfiles.collections.Mapper.<K,V>#delete}
       * method.
       * </p>
       * @param key The key of the mapping.
       * @param value The value.
       * @see yfiles.collections.IMapper.<K,V>#get
       * @abstract
       */
      set(key:K,value:V):void;
    }
    var IMapper:{
      /**
       * Creates a constant mapper that will always return the <code>constant</code>.
       * Type parameter <code>K</code>: The type of the key to use.
       * Type parameter <code>V</code>: The type of the value to use.
       * @param constant The constant to return in {@link #get}
       * @returns A new instance that will always return the same constant.
       * @template K,V
       * @static
       */
      fromConstant?<K,V>(constant:V):yfiles.collections.IMapper<K,V>;
      /**
       * Create an implementation of {@link yfiles.collections.IMapper.<K,V>} that delegates {@link yfiles.collections.IMapper.<K,V>#get} getter calls to the provided handler.
       * <p>
       * The implementation returned silently ignores calls to the {@link yfiles.collections.IMapper.<K,V>#get} setter.
       * </p>
       * Type parameter <code>K</code>: The type of the key to use.
       * Type parameter <code>V</code>: The type of the value to use.
       * @param getter The delegate to delegate {@link #get} getter calls to.
       * @returns An implementation that delegates to <code>getter</code>.
       * @template K,V
       * @static
       */
      fromDelegate?<K,V>(getter:(key:K)=>V):yfiles.collections.IMapper<K,V>;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
  }export namespace graphml{
    export interface XmlName extends Object{}
    export class XmlName {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.XmlName} class.
       * @param localName The local name of the element.
       * @param ns The namespace it belongs to.
       */
      constructor(localName:string,ns:string);
      /**
       * Performs a conversion from {@link string} to {@link yfiles.graphml.XmlName}.
       * <p>
       * The name is made up of at most two parts: the namespace and the local name. When given an input like <code>"http://www.w3.org/2000/xmlns/}local"</code>
       * it will be split at the "}" character and the first part will be used as the namespace URI, the second part as the local
       * name. If no "}" character is used in the input then it will be assumed that the input consists of the local name only.
       * </p>
       * @param expandedName Name of the expanded.
       * @returns The result of the conversion.
       * @static
       */
      static convertFrom(expandedName:string):yfiles.graphml.XmlName;
      /**
       * Gets or sets the local name.
       * @type {string}
       */
      localName:string;
      /**
       * Gets the namespace.
       * @type {string}
       */
      namespace:string;
      static $class:yfiles.lang.Class;
    }
    export enum XamlAttributeWritePolicy{
      /**
       * Specifies that the write mode of a member should be determined automatically according to the current property value and
       * the presence of {@link yfiles.graphml.ValueSerializer}s or implementations of {@link yfiles.graphml.IMarkupExtensionConverter}s.
       * <p>
       * Usually, an implementation would write the member as attribute if the content can somehow be (directly or indirectly)
       * converted to a valid XML attribute content (i.e. a simple string)
       * </p>
       */
      AUTO,
      /**
       * Never write as a XML attribute, even if a possible conversion exists.
       */
      NEVER,
      /**
       * Always write as XML attribute.
       * <p>
       * This can be used for special cases where the default implementation does not detect a valid conversion, but some part of
       * the (de)serialization is handled manually.
       * </p>
       */
      ALWAYS
    }
    export enum GraphMLSharingPolicy{
      /**
       * Specifies that the share mode of a member should be determined automatically according to the current property value.
       */
      AUTO,
      /**
       * Never share the object.
       */
      NEVER,
      /**
       * Always share the object.
       * <p>
       * This will also always place the object in the resources section, even if it only occurs once in the document.
       * </p>
       */
      ALWAYS
    }
    export enum GraphMLMemberVisibility{
      /**
       * The member is treated according to the normal XAML serialization rules.
       */
      VISIBLE,
      /**
       * Only the content of a member value is read or written, without an enclosing explicit container object.
       */
      CONTENT,
      /**
       * The member is ignored by GraphML.
       */
      HIDDEN
    }
    export enum UndefinedHandling{
      /**
       * Explicitly read and write undefined values.
       */
      EXPLICIT,
      /**
       * Throw an exception when undefined values are encountered.
       */
      THROW,
      /**
       * Treat undefined values like null values.
       */
      AS_NULL,
      /**
       * Treat undefined values as default values.
       * <p>
       * Default values are never written.
       * </p>
       */
      AS_DEFAULT
    }
    /**
     * Custom attribute to control specific aspects of GraphML deserialization.
     * <p>
     * The specialized {@link yfiles.graphml.XamlSerializer} will evaluate this attribute to override its built in serialization rules.
     * The main use cases for this attribute are to override object reference sharing, configure consistent member naming and
     * handle legacy serialization.
     * </p>
     * @see yfiles.graphml.XamlSerializer
     * @class
     * @extends {yfiles.lang.Attribute}
     */
    export interface GraphMLAttribute extends yfiles.lang.Attribute{}
    export class GraphMLAttribute {
      constructor();
      /**
       * Gets or sets the name of the type or member where this attribute is used.
       * <p>
       * This will be used as the XML element or XML attribute name.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @type {string}
       */
      name:string;
      /**
       * Gets or sets the XML namespace URI for the member where this attribute is used.
       * <p>
       * This will be used as the XML namespace URI for the XML element or XML attribute.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @type {string}
       */
      xmlNamespace:string;
      /**
       * Gets or sets whether an enclosing tag for the property itself is written.
       * <p>
       * If <code>true</code>, the resulting XML will look like
       * </p>
       * <pre>
       * &lt;ParentObjectName.Name>
       *    &lt;PropertyTypeName>
       *       content
       *    &lt;/PropertyTypeName>
       * &lt;/ParentObjectName.Name>
       * </pre>
       * 
       * <p>
       * Default value is <code>true</code>.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @type {boolean}
       */
      writeEnclosingTag:boolean;
      /**
       * Gets or sets whether the member should be written as an XML element or as an XML attribute.
       * <p>
       * Default value is {@link yfiles.graphml.XamlAttributeWritePolicy#AUTO}.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @see yfiles.graphml.XamlAttributeWritePolicy
       * @type {yfiles.graphml.XamlAttributeWritePolicy}
       */
      writeAsAttribute:yfiles.graphml.XamlAttributeWritePolicy;
      /**
       * Gets or sets whether the object reference identity of the property values should be kept intact.
       * <p>
       * Default value is {@link yfiles.graphml.GraphMLSharingPolicy#AUTO}.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @type {yfiles.graphml.GraphMLSharingPolicy}
       */
      shareable:yfiles.graphml.GraphMLSharingPolicy;
      /**
       * Gets or sets how a member should be visible to GraphML.
       * <p>
       * Default value is {@link yfiles.graphml.GraphMLMemberVisibility#VISIBLE}.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @type {yfiles.graphml.GraphMLMemberVisibility}
       */
      visibility:yfiles.graphml.GraphMLMemberVisibility;
      /**
       * Specifies the property of a class that is used as the content property in XAML.
       * <p>
       * The value of the content property can appear in XAML as a direct child of the class element, without the need to wrap it
       * into a <code>&lt;Class.Property></code> element.
       * </p>
       * <p>
       * This property is ignored if the attribute targets a member declaration.
       * </p>
       * @type {string}
       */
      contentProperty:string;
      /**
       * Gets or sets the type of the {@link yfiles.graphml.GraphMLAttribute#valueSerializer} implementation.
       * @type {yfiles.lang.Class}
       */
      valueSerializer:yfiles.lang.Class;
      /**
       * Gets or sets a custom {@link yfiles.graphml.IMarkupExtensionConverter} for attribute serialization.
       * <p>
       * The specialized {@link yfiles.graphml.XamlSerializer} will evaluate this attribute to override its built in mechanism to lookup
       * an {@link yfiles.graphml.IMarkupExtensionConverter} for an object. The specified type must implement
       * {@link yfiles.graphml.IMarkupExtensionConverter}.
       * </p>
       * @see yfiles.graphml.XamlSerializer
       * @see yfiles.graphml.IMarkupExtensionConverter
       * @type {yfiles.lang.Class}
       */
      markupExtensionConverter:yfiles.lang.Class;
      /**
       * Types that should be searched for static field or property members of the annotated type.
       * <p>
       * When a type is annotated with this attribute, all types in the {@link yfiles.graphml.GraphMLAttribute#singletonContainers} array are searched for public static fields or
       * properties that have the annotated type. The GraphML writer will write these members as StaticExtensions instead of the
       * usual serialization. This property is ignored if the attribute targets a member declaration.
       * </p>
       * <p>
       * Using the following annotation
       * </p>
       * <p>
       * has the effect that all references to field <code>ContainedTypeSingleton</code> will be written as static extensions, e.g.
       * </p>
       * <pre>
       * {x:Static myNSDeclarationPrefix:ContainingType.ContainedTypeSingleton}
       * </pre>
       * 
       * @type {Array.<yfiles.lang.Class>}
       */
      singletonContainers:yfiles.lang.Class[];
      /**
       * Gets or sets the default value of the property this attribute is bound to.
       * <p>
       * This property is ignored if the attribute targets a type declaration.
       * </p>
       * @type {Object}
       */
      defaultValue:Object;
      static $class:yfiles.lang.Class;
    }
    export enum KeyType{
      /**
       * Identifier for attribute of type <code>int</code>.
       */
      INT,
      /**
       * Identifier for attribute of type <code>long</code>.
       */
      LONG,
      /**
       * Identifier for attribute of type <code>float</code>.
       */
      FLOAT,
      /**
       * Identifier for attribute of type <code>double</code>.
       */
      DOUBLE,
      /**
       * Identifier for attribute of type <code>string</code> that denotes simple string content as opposed to
       * {@link yfiles.graphml.KeyType#COMPLEX}.
       */
      STRING,
      /**
       * Identifier for attribute of type <code>boolean</code>.
       */
      BOOLEAN,
      /**
       * Identifier for arbitrary complex xml content
       */
      COMPLEX
    }
    export enum KeyScope{
      /**
       * Unscoped for key definition
       */
      ALL,
      /**
       * Node-Scope for key definition.
       */
      NODE,
      /**
       * Edge-Scope for key definition.
       */
      EDGE,
      /**
       * Graph-Scope for key definition.
       */
      GRAPH,
      /**
       * Port-Scope for key definition.
       */
      PORT,
      /**
       * Graphml-Scope for key definition.
       */
      GRAPHML
    }
    /**
     * Interface that defines the context of a GraphML parsing context.
     * <p>
     * All state data that is needed for the parsing of a concrete input should go here, allowing all handlers and
     * deserializers to be implemented stateless.
     * </p>
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface IParseContext extends Object,yfiles.graph.ILookup{
      /**
       * Deserialize the object representation in <code>targetNode</code>.
       * <p>
       * This method calls {@link yfiles.graphml.IParseContext#deserializeCore} with <code>context</code> as first argument and should be used in
       * virtually all cases.
       * </p>
       * Type parameter <code>T</code>: The desired target type.
       * @param targetType The desired target type.
       * @param targetNode The XML node that should be deserialized.
       * @returns an instance of <code>T</code> or <code>null</code>.
       * @throws {yfiles.graphml.DeserializationNotSupportedError}
       * @see yfiles.graphml.IParseContext#deserializeCore
       * @template T
       */
      deserialize?<T>(targetType:yfiles.lang.Class,targetNode:Node):T;
      /**
       * Deserialize the object representation in <code>targetNode</code>
       * <p>
       * Client code should usually use the extension method {@link yfiles.graphml.IParseContext#deserialize} instead to ensure the the
       * correct <code>context</code> instance is used.
       * </p>
       * Type parameter <code>T</code>: The expected return type.
       * @param targetType The expected return type.
       * @param context The current parse context.
       * @param targetNode The XML node that should be deserialized.
       * @returns The deserialized object from <code>targetNode</code> or an {@link } if <code>targetNode</code> could not be deserialized.
       * @see yfiles.graphml.IParseContext#deserialize
       * @template T
       * @abstract
       */
      deserializeCore<T>(targetType:yfiles.lang.Class,context:yfiles.graphml.IParseContext,targetNode:Node):T;
      /**
       * Returns the most current (the last element) within the container hierarchy as it is returned by {@link yfiles.graphml.IParseContext#objectStack} or <code>default(T)</code> if the
       * most current element is not of the desired target type.
       * Type parameter <code>T</code>: The desired target type.
       * @param itemType The desired target type.
       * @returns the most current element within the container hierarchy or <code>default(T)</code>.
       * @template T
       * @abstract
       */
      getCurrent<T>(itemType:yfiles.lang.Class):T;
      /**
       * Get a property value that specifies information about how to handle specific cases.
       * @param key The identifier for the property
       * @returns The property value, or <code>null</code> if no such property exists.
       * @abstract
       */
      getDeserializationProperty(key:string):Object;
      /**
       * Returns the current nesting of created graphs and graph elements.
       * <p>
       * The list contains the user objects which correspond to the GraphML elements which are ancestors of the current node in
       * the DOM tree.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<Object>}
       */
      objectStack:yfiles.collections.IListEnumerable<Object>;
      /**
       * Returns an implementation of {@link yfiles.graphml.IParseEvents} that allows to subscribe to various events in the parse
       * process.
       * @abstract
       * @type {yfiles.graphml.IParseEvents}
       */
      parseEvents:yfiles.graphml.IParseEvents;
      /**
       * Returns the currently active graph object
       * @abstract
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
    }
    var IParseContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface of the event trigger class that is used by the {@link yfiles.graphml.IParseContext#parseEvents} property.
     * <p>
     * This interface defines the available events that client code can register to during the parsing of {@link yfiles.graphml.GraphMLParser GraphML}.
     * </p>
     * @see yfiles.graphml.ParseEventArgs
     * @see yfiles.graphml.GraphMLParser
     * @interface
     */
    export interface IParseEvents extends Object{
      /**
       * Adds the given listener for the <code>DocumentParsing</code> event that occurs when the document is about to be parsed.
       * <p>
       * This event is triggered when the document element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeDocumentParsingListener
       * @abstract
       */
      addDocumentParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DocumentParsing</code> event that occurs when the document is about to be parsed.
       * <p>
       * This event is triggered when the document element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addDocumentParsingListener
       * @abstract
       */
      removeDocumentParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DocumentParsed</code> event that occurs when the document has been parsed.
       * <p>
       * This event is triggered when the document has been fully parsed.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeDocumentParsedListener
       * @abstract
       */
      addDocumentParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DocumentParsed</code> event that occurs when the document has been parsed.
       * <p>
       * This event is triggered when the document has been fully parsed.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addDocumentParsedListener
       * @abstract
       */
      removeDocumentParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeParsing</code> event that occurs when a node element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'node' element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeNodeParsingListener
       * @abstract
       */
      addNodeParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>NodeParsing</code> event that occurs when a node element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'node' element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addNodeParsingListener
       * @abstract
       */
      removeNodeParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeParsed</code> event that occurs after a node element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'node' closing tag has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeNodeParsedListener
       * @abstract
       */
      addNodeParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>NodeParsed</code> event that occurs after a node element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'node' closing tag has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addNodeParsedListener
       * @abstract
       */
      removeNodeParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeParsing</code> event that occurs when an edge element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'edge' element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeEdgeParsingListener
       * @abstract
       */
      addEdgeParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeParsing</code> event that occurs when an edge element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'edge' element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addEdgeParsingListener
       * @abstract
       */
      removeEdgeParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeParsed</code> event that occurs after a edge element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'edge' closing tag has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeEdgeParsedListener
       * @abstract
       */
      addEdgeParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeParsed</code> event that occurs after a edge element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'edge' closing tag has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addEdgeParsedListener
       * @abstract
       */
      removeEdgeParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PortParsing</code> event that occurs when a port element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'port' element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removePortParsingListener
       * @abstract
       */
      addPortParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PortParsing</code> event that occurs when a port element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'port' element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addPortParsingListener
       * @abstract
       */
      removePortParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PortParsed</code> event that occurs after a port element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'port' closing tag has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removePortParsedListener
       * @abstract
       */
      addPortParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PortParsed</code> event that occurs after a port element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'port' closing tag has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addPortParsedListener
       * @abstract
       */
      removePortParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphParsing</code> event that occurs when a graph element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'graph' element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeGraphParsingListener
       * @abstract
       */
      addGraphParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphParsing</code> event that occurs when a graph element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'graph' element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addGraphParsingListener
       * @abstract
       */
      removeGraphParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphParsed</code> event that occurs after a graph element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'graph' closing tag has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeGraphParsedListener
       * @abstract
       */
      addGraphParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphParsed</code> event that occurs after a graph element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'graph' closing tag has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addGraphParsedListener
       * @abstract
       */
      removeGraphParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphMLParsing</code> event that occurs when the graphml element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'graphml' root-element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeGraphMLParsingListener
       * @abstract
       */
      addGraphMLParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphMLParsing</code> event that occurs when the graphml element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'graphml' root-element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addGraphMLParsingListener
       * @abstract
       */
      removeGraphMLParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphMLParsed</code> event that occurs after the graphml element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'graphml' closing tag has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeGraphMLParsedListener
       * @abstract
       */
      addGraphMLParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphMLParsed</code> event that occurs after the graphml element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'graphml' closing tag has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addGraphMLParsedListener
       * @abstract
       */
      removeGraphMLParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DataParsing</code> event that occurs when a data element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'data' element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeDataParsingListener
       * @abstract
       */
      addDataParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DataParsing</code> event that occurs when a data element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'data' element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addDataParsingListener
       * @abstract
       */
      removeDataParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DataParsed</code> event that occurs after a data element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'data' element has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeDataParsedListener
       * @abstract
       */
      addDataParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DataParsed</code> event that occurs after a data element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'data' element has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addDataParsedListener
       * @abstract
       */
      removeDataParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyParsing</code> event that occurs when a key element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'key' element has been encountered.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeKeyParsingListener
       * @abstract
       */
      addKeyParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyParsing</code> event that occurs when a key element is about to be parsed.
       * <p>
       * This event is triggered when a GraphML 'key' element has been encountered.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addKeyParsingListener
       * @abstract
       */
      removeKeyParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyParsed</code> event that occurs after a key element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'key' element has been handled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IParseEvents#removeKeyParsedListener
       * @abstract
       */
      addKeyParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyParsed</code> event that occurs after a key element has been fully parsed.
       * <p>
       * This event is triggered when a GraphML 'key' element has been handled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IParseEvents#addKeyParsedListener
       * @abstract
       */
      removeKeyParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
    }
    var IParseEvents:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is the event argument that is used by the {@link yfiles.graphml.IParseEvents} interface that can be queried from the {@link yfiles.graphml.IParseContext#parseEvents}
     * property.
     * <p>
     * This class holds a reference to the {@link yfiles.graphml.ParseEventArgs#context parse context} and the {@link yfiles.graphml.ParseEventArgs#element} that is currently being parsed.
     * </p>
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface ParseEventArgs extends yfiles.lang.EventArgs{}
    export class ParseEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.ParseEventArgs} class.
       * @param context The context to assign to {@link #context}.
       * @param element The XML element to assign to {@link #element}.
       */
      constructor(context:yfiles.graphml.IParseContext,element:Element);
      /**
       * Gets the context that is being used for parsing.
       * @type {yfiles.graphml.IParseContext}
       */
      context:yfiles.graphml.IParseContext;
      /**
       * Gets the XML element that is currently parsed.
       * @type {Element}
       */
      element:Element;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data holder for use in the {@link yfiles.graphml.IOutputHandler} interface that encapsulates an XML attribute.
     * @class
     */
    export interface GraphMLXmlAttribute extends Object{}
    export class GraphMLXmlAttribute {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.GraphMLXmlAttribute} class.
       * @param name The attribute name.
       * @param value The value.
       */
      constructor(name:yfiles.graphml.XmlName,value:string);
      /**
       * Gets the fully qualified name of the attribute.
       * @type {yfiles.graphml.XmlName}
       */
      name:yfiles.graphml.XmlName;
      /**
       * Gets the value of the attribute.
       * @type {string}
       */
      value:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * The interface used by the {@link yfiles.graphml.GraphMLIOHandler} for writing data to the GraphML.
     * @interface
     */
    export interface IOutputHandler extends Object{
      /**
       * Determines whether in the current context, the value is the default value and therefore no data element needs to be
       * written.
       * @param context The context.
       * @returns <code>true</code> if for the current context the default value applies and therefore no data element needs to be written.
       * @abstract
       */
      isDefaultValue(context:yfiles.graphml.IWriteContext):boolean;
      /**
       * Writes the contents of the key definition.
       * <p>
       * At the time this method is called, the surrounding 'key' element has already been written. However no 'default' element
       * is written by the framework.
       * </p>
       * @param context The context.
       * @abstract
       */
      writeKeyDefinitionContent(context:yfiles.graphml.IWriteContext):void;
      /**
       * Actually writes the value for the current context.
       * <p>
       * At the time this method is called, the surrounding 'data' element has already been written.
       * </p>
       * @param context The context.
       * @abstract
       */
      writeValue(context:yfiles.graphml.IWriteContext):void;
      /**
       * Gets the write precedence that is associated with this instance.
       * <p>
       * Registered output handlers are written either before or after the (optional) child content of the governing GraphML
       * element.
       * </p>
       * @see yfiles.graphml.WritePrecedence
       * @abstract
       * @type {yfiles.graphml.WritePrecedence}
       */
      precedence:yfiles.graphml.WritePrecedence;
      /**
       * Gets the XML attributes that should be added to the key definition in the GraphML key element.
       * @abstract
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.GraphMLXmlAttribute>}
       */
      keyDefinitionAttributes:yfiles.collections.IEnumerable<yfiles.graphml.GraphMLXmlAttribute>;
      /**
       * Gets the XML attributes that should be added to the data element.
       * @abstract
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.GraphMLXmlAttribute>}
       */
      dataTagAttributes:yfiles.collections.IEnumerable<yfiles.graphml.GraphMLXmlAttribute>;
    }
    var IOutputHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum WritePrecedence{
      /**
       * The handler writes its content before the child nodes of the governing GraphML element.
       */
      BEFORE_CHILDREN,
      /**
       * Default precedence, the order in relation to child content is unspecified.
       */
      DEFAULT,
      /**
       * The handler writes its content after the child nodes of the governing GraphML element.
       */
      AFTER_CHILDREN
    }
    /**
     * Interface that provides the context of a GraphML write process.
     * <p>
     * All state data that is needed for the writing of a concrete graph instance should go here, allowing all handlers and
     * deserializers to be implemented stateless.
     * </p>
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface IWriteContext extends Object,yfiles.graph.ILookup{
      /**
       * Returns the most current (the last element) within the container hierarchy as it is returned by {@link yfiles.graphml.IWriteContext#objectStack} or <code>default(T)</code> if the
       * most current element is not of the desired target type.
       * Type parameter <code>T</code>: The desired target type.
       * @param itemType The desired target type.
       * @returns the most current element within the container hierarchy or <code>default(T)</code>.
       * @template T
       * @abstract
       */
      getCurrent<T>(itemType:yfiles.lang.Class):T;
      /**
       * Get a property value that specifies information about how to handle specific cases
       * @param key The identifier for the property
       * @returns The property value, or <code>null</code> if no such property exists
       * @abstract
       */
      getSerializationProperty(key:string):Object;
      /**
       * Serializes the specified item using the provided type information.
       * <p>
       * <code>item</code> should be assignable to type <code>t</code>. This method simply delegates to
       * {@link yfiles.graphml.IWriteContext#serializeCore}, using the correct type parameter.
       * </p>
       * @param item The item to serialize.
       * @param t The type the item is known to be of at deserialization time.
       */
      serialize?(item:Object,t:yfiles.lang.Class):void;
      /**
       * Serialize the <code>item</code>.
       * <p>
       * This method calls {@link yfiles.graphml.IWriteContext#serializeCore} with <code>context</code> as first argument and should be used in
       * virtually all cases.
       * </p>
       * Type parameter <code>T</code>: The type of the object that is serialized.
       * @param targetType The type of the object that is serialized.
       * @param item The object that should be serialized.
       * @throws {yfiles.graphml.SerializationNotSupportedError}
       * @see yfiles.graphml.IWriteContext#serializeCore
       * @template T
       */
      serialize?<T>(targetType:yfiles.lang.Class,item:T):void;
      /**
       * Serialize <code>item</code> to an XML representation.
       * <p>
       * Client code should usually use the extension method {@link yfiles.graphml.IWriteContext#serialize} instead to ensure the the
       * correct <code>context</code> instance is used.
       * </p>
       * Type parameter <code>T</code>: The type of the object that is serialized.
       * @param targetType The type of the object that is serialized.
       * @param context The current write context.
       * @param item The object that should be serialized.
       * @throws {yfiles.graphml.SerializationNotSupportedError}
       * @see yfiles.graphml.IWriteContext#serialize
       * @template T
       * @abstract
       */
      serializeCore<T>(targetType:yfiles.lang.Class,context:yfiles.graphml.IWriteContext,item:T):void;
      /**
       * Serialize an replacement object <code>replacement</code> instead of the original <code>originalItem</code>.
       * <p>
       * Calling this method allows to remember the original object instance, even if the object is really written through a {@link yfiles.graphml.MarkupExtension}
       * or similar replacement mechanisms.
       * </p>
       * Type parameter <code>T</code>: The type of the object that is serialized.
       * @param originalItem The original object that should be serialized.
       * @param replacement The replacement object that will actually be serialized instead.
       * @throws {yfiles.graphml.SerializationNotSupportedError}
       * @see yfiles.graphml.IWriteContext#serializeCore
       * @template T
       */
      serializeReplacement?<T>(targetType:yfiles.lang.Class,originalItem:Object,replacement:T):void;
      /**
       * Returns the current nesting of graphs and graph elements.
       * <p>
       * The list contains the user objects which correspond to the GraphML elements.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<Object>}
       */
      objectStack:yfiles.collections.IListEnumerable<Object>;
      /**
       * Returns an implementation of {@link yfiles.graphml.IWriteEvents} that allows to subscribe to various events in the write
       * process.
       * @abstract
       * @type {yfiles.graphml.IWriteEvents}
       */
      writeEvents:yfiles.graphml.IWriteEvents;
      /**
       * Returns the current {@link yfiles.graphml.IXmlWriter} implementation.
       * @abstract
       * @type {yfiles.graphml.IXmlWriter}
       */
      writer:yfiles.graphml.IXmlWriter;
      /**
       * Returns the currently active graph object
       * @abstract
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
    }
    var IWriteContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface of the event trigger class that is used by the {@link yfiles.graphml.IWriteContext#writeEvents} property.
     * <p>
     * This interface defines the available events that client code can register to during the writing of {@link yfiles.graphml.GraphMLWriter GraphML}.
     * </p>
     * @see yfiles.graphml.WriteEventArgs
     * @see yfiles.graphml.GraphMLWriter
     * @interface
     */
    export interface IWriteEvents extends Object{
      /**
       * Adds the given listener for the <code>DocumentWriting</code> event that occurs just after the {@link yfiles.graphml.IXmlWriter#writeStartDocument}
       * has been called.
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeDocumentWritingListener
       * @abstract
       */
      addDocumentWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DocumentWriting</code> event that occurs just after the {@link yfiles.graphml.IXmlWriter#writeStartDocument}
       * has been called.
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addDocumentWritingListener
       * @abstract
       */
      removeDocumentWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DocumentWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndDocument}
       * will be called.
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeDocumentWrittenListener
       * @abstract
       */
      addDocumentWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DocumentWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndDocument}
       * will be called.
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addDocumentWrittenListener
       * @abstract
       */
      removeDocumentWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'node' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeNodeWritingListener
       * @abstract
       */
      addNodeWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>NodeWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'node' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addNodeWritingListener
       * @abstract
       */
      removeNodeWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'node' element will be called.
       * <p>
       * This can be used to append more custom elements to the node element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeNodeWrittenListener
       * @abstract
       */
      addNodeWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>NodeWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'node' element will be called.
       * <p>
       * This can be used to append more custom elements to the node element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addNodeWrittenListener
       * @abstract
       */
      removeNodeWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'edge' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeEdgeWritingListener
       * @abstract
       */
      addEdgeWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'edge' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addEdgeWritingListener
       * @abstract
       */
      removeEdgeWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'edge' element will be called.
       * <p>
       * This can be used to append more custom elements to the edge element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeEdgeWrittenListener
       * @abstract
       */
      addEdgeWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'edge' element will be called.
       * <p>
       * This can be used to append more custom elements to the edge element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addEdgeWrittenListener
       * @abstract
       */
      removeEdgeWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PortWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'port' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removePortWritingListener
       * @abstract
       */
      addPortWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PortWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'port' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addPortWritingListener
       * @abstract
       */
      removePortWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PortWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'port' element will be called.
       * <p>
       * This can be used to append more custom elements to the port element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removePortWrittenListener
       * @abstract
       */
      addPortWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PortWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'port' element will be called.
       * <p>
       * This can be used to append more custom elements to the port element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addPortWrittenListener
       * @abstract
       */
      removePortWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'graph' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeGraphWritingListener
       * @abstract
       */
      addGraphWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'graph' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addGraphWritingListener
       * @abstract
       */
      removeGraphWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'graph' element will be called.
       * <p>
       * This can be used to append more custom elements to the graph element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeGraphWrittenListener
       * @abstract
       */
      addGraphWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'graph' element will be called.
       * <p>
       * This can be used to append more custom elements to the graph element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addGraphWrittenListener
       * @abstract
       */
      removeGraphWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphMLWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'graphml' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeGraphMLWritingListener
       * @abstract
       */
      addGraphMLWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphMLWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'graphml' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addGraphMLWritingListener
       * @abstract
       */
      removeGraphMLWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GraphMLWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'graphml' element will be called.
       * <p>
       * This can be used to append more custom elements to the graphml element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeGraphMLWrittenListener
       * @abstract
       */
      addGraphMLWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GraphMLWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'graphml' element will be called.
       * <p>
       * This can be used to append more custom elements to the graphml element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addGraphMLWrittenListener
       * @abstract
       */
      removeGraphMLWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DataWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'data' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeDataWritingListener
       * @abstract
       */
      addDataWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DataWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'data' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addDataWritingListener
       * @abstract
       */
      removeDataWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DataWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'data' element will be called.
       * <p>
       * This can be used to append more custom elements to the data element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeDataWrittenListener
       * @abstract
       */
      addDataWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DataWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'data' element will be called.
       * <p>
       * This can be used to append more custom elements to the data element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addDataWrittenListener
       * @abstract
       */
      removeDataWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'key' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeKeyWritingListener
       * @abstract
       */
      addKeyWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyWriting</code> event that occurs right after the {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * method for writing a GraphMl 'key' element has been called.
       * <p>
       * At that state, callees can use the {@link yfiles.graphml.IXmlWriter#writeAttributeNS} methods to add custom attributes to the
       * XML element or write a first custom inner element using {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addKeyWritingListener
       * @abstract
       */
      removeKeyWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'key' element will be called.
       * <p>
       * This can be used to append more custom elements to the key element.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.IWriteEvents#removeKeyWrittenListener
       * @abstract
       */
      addKeyWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyWritten</code> event that occurs just before the {@link yfiles.graphml.IXmlWriter#writeEndElement}
       * for the GraphML 'key' element will be called.
       * <p>
       * This can be used to append more custom elements to the key element.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.IWriteEvents#addKeyWrittenListener
       * @abstract
       */
      removeKeyWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
    }
    var IWriteEvents:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Abstraction of XML output classes that provide a consistent way to write XML elements to some output
     * @interface
     */
    export interface IXmlWriter extends Object{
      /**
       * Close the document.
       * <p>
       * This is typically used to actually flush the document to an output stream
       * </p>
       * @abstract
       */
      flushDocument():void;
      /**
       * Gets a text representation of all content that has been written so far.
       * @returns 
       * @abstract
       */
      getXmlString():string;
      /**
       * Write an XML attribute with the given parameters.
       * <p>
       * The attribute is not bound to a XML namespace.
       * </p>
       * @param localName The name of the attribute.
       * @param value The value of the attribute.
       * @returns The writer instance <code>writer</code> for writer chaining.
       */
      writeAttribute?(localName:string,value:string):yfiles.graphml.IXmlWriter;
      /**
       * Write a XML attribute node.
       * <p>
       * The attribute is attached to the closest open XML element that has been started with {@link yfiles.graphml.IXmlWriter#writeStartElement}
       * or {@link yfiles.graphml.IXmlWriter#writeStartElement}. Note that namespace declarations should not be written directly as
       * attributes. Instead the {@link yfiles.graphml.IXmlWriter#namespaceManager} should be used.
       * </p>
       * @param prefix The namespace prefix of the attribute
       * @param localName The local name of the attribute
       * @param ns The namespace URI of this attribute
       * @param value The value of this attribute
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeAttributeNS(prefix:string,localName:string,ns:string,value:string):yfiles.graphml.IXmlWriter;
      /**
       * Write a CDATA section.
       * @param content The content of the CDATA section
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeCData(content:string):yfiles.graphml.IXmlWriter;
      /**
       * Write a XML comment node
       * @param comment The content of the comment
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeComment(comment:string):yfiles.graphml.IXmlWriter;
      /**
       * Write a document fragment
       * @param fragment
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeDocumentFragment(fragment:Document):yfiles.graphml.IXmlWriter;
      /**
       * Close the output.
       * <p>
       * Attempts to write after this method has been called will have undefined results.
       * </p>
       * @abstract
       */
      writeEndDocument():void;
      /**
       * Close a XML element previously opened with {@link yfiles.graphml.IXmlWriter#writeStartElement} or
       * {@link yfiles.graphml.IXmlWriter#writeStartElement}.
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeEndElement():yfiles.graphml.IXmlWriter;
      /**
       * Write an XML attribute from the given <code>attribute</code> struct.
       * @param attribute Encapsulates the current attribute values.
       * @returns The writer instance <code>writer</code> for writer chaining.
       */
      writeGraphMLXmlAttribute?(attribute:yfiles.graphml.GraphMLXmlAttribute):yfiles.graphml.IXmlWriter;
      /**
       * Write a xml processing instruction.
       * @param target The target of the PI
       * @param data The data of the PI
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeProcessingInstruction(target:string,data:string):yfiles.graphml.IXmlWriter;
      /**
       * Begin the output process
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeStartDocument():yfiles.graphml.IXmlWriter;
      /**
       * Begin a new XML element with given parameters
       * <p>
       * All subsequent output is attached to this element until {@link yfiles.graphml.IXmlWriter#writeEndElement} has been called or a
       * new element has been started.
       * </p>
       * @param prefix The namespace prefix to use for this element
       * @param localName The local name of this element
       * @param ns The namespace of this element
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeStartElement(prefix:string,localName:string,ns:string):yfiles.graphml.IXmlWriter;
      /**
       * Begin a new XML element with given parameters
       * <p>
       * All subsequent output is attached to this element until {@link yfiles.graphml.IXmlWriter#writeEndElement} has been called or a
       * new element has been started.
       * </p>
       * <p>
       * This acts like {@link yfiles.graphml.IXmlWriter#writeStartElement}, but uses the default prefix for the namespace
       * </p>
       * @param localName The local name of this element
       * @param ns The namespace of this element
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeStartElement(localName:string,ns:string):yfiles.graphml.IXmlWriter;
      /**
       * Write a text node.
       * <p>
       * The string value of <code>s</code> will be correctly escaped
       * </p>
       * @param s The string that gets written as XML text
       * @returns The writer instance for further chaining
       * @abstract
       */
      writeString(s:string):yfiles.graphml.IXmlWriter;
      /**
       * Gets the namespace manager associated with this writer.
       * @abstract
       * @type {yfiles.graphml.IXmlNamespaceManager}
       */
      namespaceManager:yfiles.graphml.IXmlNamespaceManager;
    }
    var IXmlWriter:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Helper class to facilitate namespace handling together with {@link yfiles.graphml.IXmlWriter} instances.
     * @interface
     */
    export interface IXmlNamespaceManager extends Object{
      /**
       * Returns a unique namespace prefix, or creates and registeres one.
       * @param ns The namespace URI for which a prefix is searched.
       * @returns a unique namespace prefix for <code>ns</code>.
       * @abstract
       */
      getOrCreateUniqueMapping(ns:string):string;
      /**
       * Pops a current declaration scope.
       * @abstract
       */
      popScope():void;
      /**
       * Pushes a current declaration scope.
       * @abstract
       */
      pushScope():void;
      /**
       * Register a local namespace/prefix mapping
       * @param nsUri The namespace URI
       * @param prefix The preferred namespace prefix or <code>null</code> if any prefix may be used.
       * @returns The prefix that finally has been used for the namespace URI.
       * @abstract
       */
      registerLocalMapping(nsUri:string,prefix:string):string;
      /**
       * Seals the current declaration scope.
       * <p>
       * No further namespace mappings may be added to a sealed scope.
       * </p>
       * @abstract
       */
      sealScope():void;
      /**
       * Return a list of all namespace declarations that should be declared at the document's root element.
       * @abstract
       * @type {yfiles.collections.IMap.<string,string>}
       */
      globalNamespaces:yfiles.collections.IMap<string,string>;
    }
    var IXmlNamespaceManager:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Constant class for common serialization properties.
     * @see yfiles.graphml.IWriteContext#getSerializationProperty
     * @class
     * @static
     */
    export interface SerializationProperties extends Object{}
    export class SerializationProperties {
      /**
       * Property key to control user tag serialization and deserialization
       * <p>
       * This property has a value of type {@link yfiles.graph.GraphItemTypes}.
       * </p>
       * <p>
       * Setting this property as a {@link yfiles.graphml.GraphMLIOHandler#serializationPropertyOverrides serialization property} disables tag serialization for all of the provided
       * {@link yfiles.graph.GraphItemTypes}. Setting this property as a {@link yfiles.graphml.GraphMLIOHandler#deserializationPropertyOverrides deserialization property} disables tag deserialization only for
       * nodes, edges and/or ports. By default, this property is set to {@link yfiles.graph.GraphItemTypes#NONE}
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_USER_TAGS:string;
      /**
       * Property key to control graph element style serialization and deserialization
       * <p>
       * This property has a value of type {@link yfiles.graph.GraphItemTypes}.
       * </p>
       * <p>
       * Setting this property as a {@link yfiles.graphml.GraphMLIOHandler#serializationPropertyOverrides serialization property} disables style serialization for all of the provided
       * {@link yfiles.graph.GraphItemTypes}. Setting this property as a {@link yfiles.graphml.GraphMLIOHandler#deserializationPropertyOverrides deserialization property} disables style deserialization only for
       * nodes, edges and/or ports. By default, this property is set to {@link yfiles.graph.GraphItemTypes#NONE}
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_STYLES:string;
      /**
       * Property key to control graph elements serialization and deserialization
       * <p>
       * This property has a value of type {@link yfiles.graph.GraphItemTypes}.
       * </p>
       * <p>
       * Setting this property disables serialization and deserialization for the provided {@link yfiles.graph.GraphItemTypes}. By
       * default, this property is set to {@link yfiles.graph.GraphItemTypes#NONE}.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_ITEMS:string;
      /**
       * Property key to control graph element geometry serialization and deserialization
       * <p>
       * This property has a value of type {@link yfiles.graph.GraphItemTypes}.
       * </p>
       * <p>
       * Setting this property as a {@link yfiles.graphml.GraphMLIOHandler#serializationPropertyOverrides serialization property} disables style serialization for all of the provided
       * {@link yfiles.graph.GraphItemTypes}. Setting this property as a {@link yfiles.graphml.GraphMLIOHandler#deserializationPropertyOverrides deserialization property} disables style deserialization only for
       * nodes, edges and/or ports. By default, this property is set to {@link yfiles.graph.GraphItemTypes#NONE}.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_GEOMETRY:string;
      /**
       * Property key to control user tag serialization and deserialization of {@link yfiles.graph.IStripe}s.
       * <p>
       * This property has a value of type {@link yfiles.graph.StripeTypes}.
       * </p>
       * <p>
       * Setting this property disables tag serialization for the provided {@link yfiles.graph.StripeTypes}. By default, this property is
       * set to {@link yfiles.graph.StripeTypes#NONE}.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_STRIPE_USER_TAGS:string;
      /**
       * Property key to control style serialization and deserialization of {@link yfiles.graph.IStripe}s.
       * <p>
       * This property has a value of type {@link yfiles.graph.StripeTypes}.
       * </p>
       * <p>
       * Setting this property disables style serialization for the provided {@link yfiles.graph.StripeTypes}. By default, this property
       * is set to {@link yfiles.graph.StripeTypes#NONE}.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_STRIPE_STYLES:string;
      /**
       * Property key to control label serialization of {@link yfiles.graph.IStripe}s.
       * <p>
       * This property has a value of type {@link yfiles.graph.StripeTypes}.
       * </p>
       * <p>
       * Setting this property disables label serialization for the provided {@link yfiles.graph.StripeTypes}. By default, this property
       * is set to {@link yfiles.graph.StripeTypes#NONE}.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_STRIPE_LABELS:string;
      /**
       * Property key to omit serialization and deserialization of the {@link yfiles.graph.IGraph}'s default settings.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * If this property is set to <code>true</code> in {@link yfiles.graphml.GraphMLIOHandler#serializationPropertyOverrides} or the write context, the current graph defaults are not written.
       * </p>
       * <p>
       * If this property is set to <code>true</code> in {@link yfiles.graphml.GraphMLIOHandler#deserializationPropertyOverrides} or the parse context, any graph defaults present in the input are not parsed and
       * applied to the current graph.
       * </p>
       * <p>
       * Default for this property is <code>true</code> for writing and <code>false</code> for parsing.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static DISABLE_GRAPH_SETTINGS:string;
      /**
       * Property key for alllowing to parse member names (properties and enum values) case-insensitively during XAML
       * deserialization.
       * <p>
       * This will convert property and member names to lower-case and remove underscores to be able to compare things like <code>SOME_CONSTANT_NAME</code>
       * and <code>SomeConstantName</code> as well as <code>somePropertyName</code> and <code>SomePropertyName</code>, respectively.
       * </p>
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * Default for this property is <code>false</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static IGNORE_PROPERTY_CASE:string;
      /**
       * Property key to control how undefined values are serialized.
       * <p>
       * This property has a value of type {@link yfiles.graphml.UndefinedHandling}.
       * </p>
       * <p>
       * The default for this property is {@link yfiles.graphml.UndefinedHandling#EXPLICIT }.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static UNDEFINED_HANDLING:string;
      /**
       * Property key whether the XML output should be properly indented.
       * <p>
       * This property has a boolean values.
       * </p>
       * <p>
       * Default for this property is <code>true</code>
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static INDENT_OUTPUT:string;
      /**
       * The current {@link yfiles.graphml.KeyScope} in which an {@link yfiles.graphml.IOutputHandler},
       * {@link yfiles.graphml.IInputHandler}, or custom serialization code is called.
       * <p>
       * This property has a value of type {@link yfiles.graphml.KeyScope}
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static CURRENT_KEY_SCOPE:string;
      /**
       * The edge that is currently being represented by the dummy edge that is written instead.
       * <p>
       * This property has a value of type {@link yfiles.graph.IEdge}.
       * </p>
       * <p>
       * If this value is set, then the edge currently being serialized represents the given edge in reality.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static REPRESENTED_EDGE:string;
      /**
       * Property key to specify whether to write the default node style in the &lt;default> element of the key definition.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * Default value is <code>false</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static WRITE_NODE_STYLE_DEFAULT:string;
      /**
       * Property key to specify whether to write the default edge style in the &lt;default> element of the key definition.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * Default value is <code>false</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static WRITE_EDGE_STYLE_DEFAULT:string;
      /**
       * Property key to specify whether to write the default port style in the &lt;default> element of the key definition.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * Default value is <code>false</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static WRITE_PORT_STYLE_DEFAULT:string;
      /**
       * Property key to specify whether the stripe defaults of a table should be written.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * If set to <code>true</code> the {@link yfiles.graph.ITable#columnDefaults} and {@link yfiles.graph.ITable#rowDefaults} will be written. Default value is <code>true</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static WRITE_STRIPE_DEFAULTS:string;
      /**
       * Property key to provide a predicate that determines if the preferred size of a label should be written.
       * <p>
       * If this property is not set, the label size is always written.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static WRITE_LABEL_SIZE_PREDICATE:string;
      /**
       * Property key to specify whether the preferred size of labels should be parsed.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * If this property is set to <code>true</code>, the preferred size provided by GraphML is used, otherwise, the preferred size is
       * determined automatically. This property is mainly meant for interoperability usecases where the text size calculations
       * can be quite different for different platforms. Default value is <code>true</code>
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static PARSE_LABEL_SIZE:string;
      /**
       * The current {@link string} base URI which is used for the GraphML file being parsed or written.
       * <p>
       * This property has a string value.
       * </p>
       * <p>
       * If set, this can be used to resolve or create relative paths.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static BASE_URI:string;
      /**
       * Property key to specify whether to rewrite relative URIs relative to the current document location.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * If set, relative URIs for images and other resources are rewritten relative to
       * {@link yfiles.graphml.SerializationProperties#BASE_URI}. If this property is <code>false</code>, or if {@link yfiles.graphml.SerializationProperties#BASE_URI}
       * is not set (either explicitly or implicitly) or the resource URI is not relative, the resource URI will not be
       * rewritten.
       * </p>
       * <p>
       * Default value is <code>true</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static REWRITE_RELATIVE_RESOURCE_URIS:string;
      /**
       * Property key to specify whether to ignore errors during XAML deserialization will be ignored and instead return <code>null</code>.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * Default value is <code>false</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static IGNORE_XAML_DESERIALIZATION_ERRORS:string;
      /**
       * Property key to control caching for external references.
       * <p>
       * This property has a boolean value.
       * </p>
       * <p>
       * If this property is set to <code>false</code> in the write context, the writer will not cache remember <b>external</b> reference ids provided via
       * {@link yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId}, i.e. the event will be
       * raised whenever an instance of the external object is encountered during the write process. This allows to provide
       * different values for the same object, depending on the current context. Default for this property is <code>true</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static CACHE_EXTERNAL_REFERENCES:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is the event argument that is used by the {@link yfiles.graphml.IWriteEvents} interface that can be queried from the {@link yfiles.graphml.IWriteContext#writeEvents}
     * property.
     * <p>
     * This class holds a reference to the {@link yfiles.graphml.WriteEventArgs#context write context} and the {@link yfiles.graphml.WriteEventArgs#item} that is currently being written.
     * </p>
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface WriteEventArgs extends yfiles.lang.EventArgs{}
    export class WriteEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.WriteEventArgs} class.
       * @param context The context to assign to {@link #context}.
       * @param item The item to assign to {@link #item}.
       */
      constructor(context:yfiles.graphml.IWriteContext,item:Object);
      /**
       * Gets the context that is being used for writing.
       * @type {yfiles.graphml.IWriteContext}
       */
      context:yfiles.graphml.IWriteContext;
      /**
       * Gets the item that is currently being written or <code>null</code> if nothing is applicable.
       * @type {Object}
       */
      item:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * An interface for converters that can convert an object to a corresponding {@link yfiles.graphml.MarkupExtension}.
     * <p>
     * This interface is only used for GraphML serialiaztion.
     * </p>
     * @interface
     */
    export interface IMarkupExtensionConverter extends Object{
      /**
       * Returns if the converter can convert the <code>value</code> to a {@link yfiles.graphml.MarkupExtension} in the current write context.
       * @param context The current write context.
       * @param value The object to convert.
       * @returns Whether the converter can convert the passed value.
       * @abstract
       */
      canConvert(context:yfiles.graphml.IWriteContext,value:Object):boolean;
      /**
       * Converts the <code>value</code> to a {@link yfiles.graphml.MarkupExtension} using the current write context.
       * @param context The current write context.
       * @param value The object to convert.
       * @returns The {@link } the value was converted to.
       * @abstract
       */
      convert(context:yfiles.graphml.IWriteContext,value:Object):yfiles.graphml.MarkupExtension;
    }
    var IMarkupExtensionConverter:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Utility class that facilitates determining type information from XML elements and writing of markup extensions as
     * attributes.
     * @interface
     */
    export interface IXamlNameMapper extends Object{
      /**
       * Returns a {@link yfiles.lang.Class} that matches the given <code>fqTagName</code> of an XML element.
       * <p>
       * <code>fqTagName</code> must follow the usual XAML naming rules.
       * </p>
       * @param context The current parse context.
       * @param fqTagName The fully qualified XML element name that must describe a valid type in this CLR namespace.
       * @returns The first matching type, if any.
       * @abstract
       */
      getClassForName(context:yfiles.graphml.IParseContext,fqTagName:yfiles.graphml.XmlName):yfiles.lang.Class;
      /**
       * Returns the XML tag name that belongs to type <code>type</code>
       * @param context The current write context.
       * @param type The type for which an XML namespace mapping is searched.
       * @returns the XML tag name that belongs to type <code>type</code>
       * @abstract
       */
      getNameForClass(context:yfiles.graphml.IWriteContext,type:yfiles.lang.Class):yfiles.graphml.XmlName;
    }
    var IXamlNameMapper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Wraps a property together with default values and meta data.
     * @class
     */
    export interface Property extends Object{}
    export class Property {
      /**
       * Gets the value the wrapped property has on the given instance.
       * @param ownerInstance The instance to get the property value from.
       * @returns The value the wrapped property has on the given instance.
       */
      getValue(ownerInstance:Object):Object;
      /**
       * Sets the value to the wrapped property on the given instance.
       * @param ownerInstance The instance to set the property value on.
       * @param value The value to set.
       */
      setValue(ownerInstance:Object,value:Object):void;
      /**
       * Returns <code>true</code> iff the property has a <b>public</b> getter.
       * @type {boolean}
       */
      canRead:boolean;
      /**
       * Returns <code>true</code> iff the property has a <b>public</b> setter.
       * @type {boolean}
       */
      canWrite:boolean;
      /**
       * Returns <code>true</code> iff this is a static property.
       * @type {boolean}
       */
      isStatic:boolean;
      /**
       * Gets the serialization visibility.
       * @type {yfiles.graphml.GraphMLMemberVisibility}
       */
      serializationVisibility:yfiles.graphml.GraphMLMemberVisibility;
      /**
       * Gets the {@link yfiles.graphml.Property#valueSerializer}.
       * @type {yfiles.graphml.ValueSerializer}
       */
      valueSerializer:yfiles.graphml.ValueSerializer;
      /**
       * Gets the MarkupExtensionConverter.
       * @type {yfiles.graphml.IMarkupExtensionConverter}
       */
      markupExtensionConverter:yfiles.graphml.IMarkupExtensionConverter;
      /**
       * Gets the {@link yfiles.graphml.Property#graphMLAttribute}.
       * @type {yfiles.graphml.GraphMLAttribute}
       */
      graphMLAttribute:yfiles.graphml.GraphMLAttribute;
      /**
       * Gets a value indicating whether this instance is the one defined in the {@link yfiles.graphml.GraphMLAttribute}'s {@link yfiles.graphml.GraphMLAttribute#contentProperty}.
       * @type {boolean}
       */
      isContentProperty:boolean;
      /**
       * Gets the type of the property.
       * @type {yfiles.lang.Class}
       */
      type:yfiles.lang.Class;
      /**
       * Gets the type of the owner.
       * @type {yfiles.lang.Class}
       */
      ownerType:yfiles.lang.Class;
      /**
       * Gets the default value.
       * @type {Object}
       */
      defaultValue:Object;
      /**
       * Gets a value indicating whether this instance has a default value.
       * @type {boolean}
       */
      hasDefaultValue:boolean;
      /**
       * Gets the name of the property, as it will appear in GraphML.
       * @type {string}
       */
      name:string;
      static $class:yfiles.lang.Class;
    }
    export enum SliderParameterLocation{
      /**
       * Left of the edge.
       */
      LEFT,
      /**
       * Right of the edge.
       */
      RIGHT,
      /**
       * Measured from the source end.
       */
      FROM_SOURCE,
      /**
       * Measured from the target end.
       */
      FROM_TARGET,
      /**
       * "Center" segment.
       */
      CENTER
    }
    /**
     * Support interface used by {@link yfiles.graphml.ValueSerializer}.
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface IValueSerializerContext extends Object,yfiles.graph.ILookup{
      /**
       * Gets the value serializer for a specific {@link yfiles.graphml.Property}.
       * @param property The descriptor of the property.
       * @returns A serializer or <code>null</code>.
       * @abstract
       */
      getValueSerializerFor(property:yfiles.graphml.Property):yfiles.graphml.ValueSerializer;
      /**
       * Gets the value serializer for a specific type.
       * @param type The type to obtain a serializer for.
       * @returns A serializer or <code>null</code>.
       * @abstract
       */
      getValueSerializerFor(type:yfiles.lang.Class):yfiles.graphml.ValueSerializer;
    }
    var IValueSerializerContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides a base class for all XAML markup extensions.
     * <p>
     * Markup extensions return objects based on string attribute values or markup elements in XAML and/or can be used to
     * serialize classes that do not fulfil XAML serialization constraints.
     * </p>
     * @class
     */
    export interface MarkupExtension extends Object{}
    export class MarkupExtension {
      /**
       * Returns an object that is set as the value of the target property for this markup extension.
       * @param serviceProvider Object that can provide services for the markup extension.
       * @returns The object value to set on the property where the extension is applied.
       * @abstract
       */
      provideValue(serviceProvider:yfiles.graph.ILookup):Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * Helper classes used during the XAML serialization of instances.
     * <p>
     * Using this class instances can be written in xml attributes as strings.
     * </p>
     * @class
     */
    export interface ValueSerializer extends Object{}
    export class ValueSerializer {
      /**
       * Determines whether this instance can convert the specified value from the given string.
       * @param value The string value to convert.
       * @param context The context.
       * @returns <code>true</code> if this instance can convert the specified value from the string; otherwise, <code>false</code>.
       */
      canConvertFromString(value:string,context:yfiles.graphml.IValueSerializerContext):boolean;
      /**
       * Determines whether this instance can convert the specified value to a string.
       * @param value The value.
       * @param context The context.
       * @returns <code>true</code> if this instance can convert the specified value to a string; otherwise, <code>false</code>.
       */
      canConvertToString(value:Object,context:yfiles.graphml.IValueSerializerContext):boolean;
      /**
       * Converts the string to an appropriate instance.
       * @param value The value to convert to an object.
       * @param context The context.
       * @returns The converted object
       */
      convertFromString(value:string,context:yfiles.graphml.IValueSerializerContext):Object;
      /**
       * Converts the given value to string.
       * @param o The value.
       * @param context The context.
       * @returns The value as string representation.
       */
      convertToString(o:Object,context:yfiles.graphml.IValueSerializerContext):string;
      /**
       * Gets the {@link yfiles.graphml.ValueSerializer} implementation for the given type.
       * @param type The type.
       * @param context The context.
       * @returns The serializer to use for the type.
       * @static
       */
      static getSerializerFor(type:yfiles.lang.Class,context:yfiles.graphml.IValueSerializerContext):yfiles.graphml.ValueSerializer;
      /**
       * Gets the {@link yfiles.graphml.ValueSerializer} implementation for the given descriptor.
       * @param property The descriptor.
       * @param context The context.
       * @returns The serializer to use for the descriptor.
       * @static
       */
      static getSerializerFor(property:yfiles.graphml.Property,context:yfiles.graphml.IValueSerializerContext):yfiles.graphml.ValueSerializer;
      /**
       * Gets the {@link yfiles.graphml.ValueSerializer} implementation for the given descriptor.
       * @param property The descriptor.
       * @returns The serializer to use for the type.
       * @static
       */
      static getSerializerFor(property:yfiles.graphml.Property):yfiles.graphml.ValueSerializer;
      /**
       * Gets the {@link yfiles.graphml.ValueSerializer} implementation for the given type.
       * @param type The type.
       * @returns The serializer to use for the type.
       * @static
       */
      static getSerializerFor(type:yfiles.lang.Class):yfiles.graphml.ValueSerializer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A helper holder class that can contain a {@link yfiles.graphml.Future.<T>#value} of a specific type.
     * <p>
     * Typically {@link yfiles.graphml.Future.<T>#hasValue} will be <code>false</code> until at a later point in time {@link yfiles.graphml.Future.<T>#onValueDetermined} is called and the {@link yfiles.graphml.Future.<T>#value} and {@link yfiles.graphml.Future.<T>#hasValue} properties are being
     * updated and the {@link yfiles.graphml.Future.<T>#addPropertyChangedListener PropertyChanged} event is triggered.
     * </p>
     * Type parameter <code>T</code>: The type of the {@link yfiles.graphml.Future.<T>#value}
     * @class
     * @implements {yfiles.lang.IPropertyObservable}
     * @template T
     * @final
     */
    export interface Future<T> extends Object,yfiles.lang.IPropertyObservable{}
    export class Future<T> {
      /**
       * Called when the {@link yfiles.graphml.Future.<T>#value} property of this future has been determined.
       * <p>
       * This will set the {@link yfiles.graphml.Future.<T>#hasValue} property to <code>true</code> and assign the {@link yfiles.graphml.Future.<T>#value} property. Also the corresponding {@link yfiles.graphml.Future.<T>#addPropertyChangedListener PropertyChanged} events
       * will be triggered if necessary.
       * </p>
       * @param value The value.
       */
      onValueDetermined(value:T):void;
      /**
       * The value of this future, which may only be queried if {@link yfiles.graphml.Future.<T>#hasValue} yields <code>true</code>.
       * @throws {Stubs.Exceptions.InvalidOperationError} If this property is queried although {@link yfiles.graphml.Future.<T>#hasValue} returns <code>false</code>.
       * @type {T}
       */
      value:T;
      /**
       * Gets a value indicating whether this instance has been {@link yfiles.graphml.Future.<T>#onValueDetermined assigned} a {@link yfiles.graphml.Future.<T>#value}.
       * @type {boolean}
       */
      hasValue:boolean;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * <p>
       * This event will be triggered for {@link yfiles.graphml.Future.<T>#hasValue} changes and changes of the {@link yfiles.graphml.Future.<T>#value} property. Note that if the determined future value is the
       * same as the default, the event will only be triggered for the {@link yfiles.graphml.Future.<T>#hasValue} property.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.Future.<T>#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * <p>
       * This event will be triggered for {@link yfiles.graphml.Future.<T>#hasValue} changes and changes of the {@link yfiles.graphml.Future.<T>#value} property. Note that if the determined future value is the
       * same as the default, the event will only be triggered for the {@link yfiles.graphml.Future.<T>#hasValue} property.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.Future.<T>#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Class for reading/writing graphs from/to GraphML format.
     * <p>
     * For more information about the GraphML file format please refer to the <a href="http://graphml.graphdrawing.org/primer/graphml-primer.html" target="_blank">GraphML Primer</a>.
     * </p>
     * @class
     */
    export interface GraphMLIOHandler extends Object{}
    export class GraphMLIOHandler {
      /**
       * Creates a new instance of {@link yfiles.graphml.GraphMLIOHandler}.
       */
      constructor();
      /**
       * Create a {@link yfiles.graphml.Future.<T>} instance that wraps an {@link yfiles.graphml.IInputHandler} that is dynamically created by <code>factory</code>.
       * <p>
       * The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link yfiles.graphml.Future.<T>#hasValue}.
       * The validity is determined by <code>keyDefinitionPredicate</code>. This methods creates and configures the necessary handler itself.
       * </p>
       * @param keyDefinitionPredicate The predicate that matches the key definition.
       * @param factory A factory to create {@link } instances.
       * @returns A {@link } instance that wraps an {@link } that actually parses the data.
       */
      addInputHandlerFactory(keyDefinitionPredicate:(obj:Element)=>boolean,factory:yfiles.graphml.IGenericInputHandlerFactory):yfiles.graphml.Future<yfiles.graphml.IInputHandler>;
      /**
       * Register a {@link yfiles.collections.IMapper.<K,V>} instance for use as an input data target.
       * <p>
       * This method uses a custom event handler for the actual deserialization.
       * </p>
       * <p>
       * This methods creates and configures the necessary handler itself.
       * </p>
       * Type parameter <code>TKey</code>: The type of the mapper keys
       * Type parameter <code>TData</code>: The type of the mapper values
       * @param dataType The type of the mapper values
       * @param keyType The type of the mapper keys
       * @param keyDefinitionPredicate The predicate function that will determine whether to create an {@link } for the given key
       * definition.
       * @param mapper the mapper that serves as data source
       * @param deserializationCallback custom deserialization callback.
       * @template TKey,TData
       */
      addInputMapper<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,keyDefinitionPredicate:(obj:Element)=>boolean,mapper:yfiles.collections.IMapper<TKey,TData>,deserializationCallback:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):void;
      /**
       * Register a {@link yfiles.collections.IMapper.<K,V>} instance for use as an input data target.
       * <p>
       * This methods creates and configures the necessary handler itself.
       * </p>
       * Type parameter <code>TKey</code>: The type of the mapper keys
       * Type parameter <code>TData</code>: The type of the mapper values
       * @param dataType The type of the mapper values
       * @param keyType The type of the mapper keys
       * @param name The name of the attribute
       * @param mapper the mapper that serves as data source
       * @template TKey,TData
       */
      addInputMapper<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,name:string,mapper:yfiles.collections.IMapper<TKey,TData>):void;
      /**
       * Create a {@link yfiles.graphml.Future.<T>} instance that wraps an {@link yfiles.collections.IMapper.<K,V>} for use as an input data target.
       * <p>
       * The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link yfiles.graphml.Future.<T>#hasValue}
       * This methods creates and configures the necessary handler itself.
       * </p>
       * Type parameter <code>TKey</code>: The type of the mapper keys
       * Type parameter <code>TData</code>: The type of the mapper values
       * @param dataType The type of the mapper values
       * @param keyType The type of the mapper keys
       * @param deserializationCallback custom deserialization callback.
       * @param keyDefinitionPredicate The predicate function that will determine whether to create an {@link } for the given key
       * definition.
       * @returns A {@link } instance that wraps an {@link } that actually holds the data.
       * @template TKey,TData
       */
      addInputMapperFuture<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,keyDefinitionPredicate:(obj:Element)=>boolean,deserializationCallback:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):yfiles.graphml.Future<yfiles.collections.IMapper<TKey,TData>>;
      /**
       * Create a {@link yfiles.graphml.Future.<T>} instance that wraps an {@link yfiles.collections.IMapper.<K,V>} for use as an input data target.
       * <p>
       * The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link yfiles.graphml.Future.<T>#hasValue}
       * This methods creates and configures the necessary handler itself.
       * </p>
       * Type parameter <code>TKey</code>: The type of the mapper keys
       * Type parameter <code>TData</code>: The type of the mapper values
       * @param dataType The type of the mapper values
       * @param keyType The type of the mapper keys
       * @param name The name of the attribute
       * @returns A {@link } instance that wraps an {@link } that actually holds the data.
       * @template TKey,TData
       */
      addInputMapperFuture<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,name:string):yfiles.graphml.Future<yfiles.collections.IMapper<TKey,TData>>;
      /**
       * Adds a namespace to the header of a GraphML file.
       * @param namespaceURI The namespace URI
       * @param shortName The namespace prefix
       */
      addNamespace(namespaceURI:string,shortName:string):void;
      /**
       * Register a {@link yfiles.collections.IMapper.<K,V>} instance for use as an output data source.
       * <p>
       * This method uses a custom event handler for the actual serialization.
       * </p>
       * <p>
       * This methods creates and configures the necessary handler itself.
       * </p>
       * Type parameter <code>TModelItem</code>: The type of the mapper keys
       * Type parameter <code>TValue</code>: The type of the mapper values
       * @param dataType The type of the mapper values
       * @param modelItemType The type of the mapper keys
       * @param name The name of the attribute
       * @param uri The URI to add to the key definition to disambiguate keys.
       * @param mapper the mapper that serves as data source
       * @param serializationCallback custom serialization callback.
       * @param declaredKeyType The key type to write in the declaration.
       * @template TModelItem,TValue
       */
      addOutputMapper<TModelItem,TValue>(modelItemType:yfiles.lang.Class,dataType:yfiles.lang.Class,name:string,uri:string,mapper:yfiles.collections.IMapper<TModelItem,TValue>,serializationCallback:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void,declaredKeyType:yfiles.graphml.KeyType):void;
      /**
       * Register a {@link yfiles.collections.IMapper.<K,V>} instance for use as an output data source
       * <p>
       * This methods creates and configures the necessary handler itself
       * </p>
       * Type parameter <code>TModelItem</code>: The type of the mapper keys
       * Type parameter <code>TValue</code>: The type of the mapper values
       * @param dataType The type of the mapper values
       * @param modelItemType The type of the mapper keys
       * @param name The name of the attribute
       * @param mapper the mapper that serves as data source
       * @template TModelItem,TValue
       */
      addOutputMapper<TModelItem,TValue>(modelItemType:yfiles.lang.Class,dataType:yfiles.lang.Class,name:string,mapper:yfiles.collections.IMapper<TModelItem,TValue>):void;
      /**
       * Create a {@link yfiles.graphml.Future.<T>} instance that wraps an {@link yfiles.graphml.IInputHandler} that stores the data in an {@link yfiles.collections.IMapper.<K,V>} that will be {@link yfiles.graphml.GraphMLIOHandler#createMapper created} in the
       * graph's {@link yfiles.graph.IMapperRegistry}.
       * <p>
       * The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link yfiles.graphml.Future.<T>#hasValue}.
       * This methods creates and configures the necessary handler itself and {@link yfiles.graphml.GraphMLIOHandler#createMapper creates} an {@link yfiles.collections.IMapper.<K,V>} in the {@link yfiles.graph.IMapperRegistry}
       * with name <code>tag</code>.
       * </p>
       * Type parameter <code>TKey</code>.
       * Type parameter <code>TData</code>.
       * @param dataType
       * @param keyType
       * @param tag The name of the key in the {@link }.
       * @param keyDefinitionPredicate A predicate to determine whether the key definition is valid.
       * @param deserializationCallback The custom deserialization callback, can be <code>null</code> which will result in default deserialization.
       * @returns A {@link } instance that wraps an {@link } that actually holds the data.
       * @template TKey,TData
       */
      addRegistryInputMapper<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,tag:Object,keyDefinitionPredicate:(obj:Element)=>boolean,deserializationCallback:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):yfiles.graphml.Future<yfiles.collections.IMapper<TKey,TData>>;
      /**
       * Create a {@link yfiles.graphml.Future.<T>} instance that wraps an {@link yfiles.graphml.IInputHandler} that stores the data in an {@link yfiles.collections.IMapper.<K,V>} that will be {@link yfiles.graphml.GraphMLIOHandler#createMapper created} in the
       * graph's {@link yfiles.graph.IMapperRegistry}.
       * <p>
       * The return value will only contain a value if a matching GraphML attribute has been read, which can be queried with {@link yfiles.graphml.Future.<T>#hasValue}.
       * The attribute must be registered under the attribute name <code>name</code>. This methods creates and configures the necessary
       * handler itself and {@link yfiles.graphml.GraphMLIOHandler#createMapper creates} an {@link yfiles.collections.IMapper.<K,V>} in the {@link yfiles.graph.IMapperRegistry} with name <code>name</code>.
       * </p>
       * Type parameter <code>TKey</code>.
       * Type parameter <code>TData</code>.
       * @param dataType
       * @param keyType
       * @param name The name of the GraphML attribute and the key in the {@link }.
       * @returns A {@link } instance that wraps an {@link } that actually holds the data.
       * @template TKey,TData
       */
      addRegistryInputMapper<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,name:string):yfiles.graphml.Future<yfiles.collections.IMapper<TKey,TData>>;
      /**
       * Use a registered {@link yfiles.collections.IMapper.<K,V>} instance as source for the given GraphML attribute.
       * Type parameter <code>TModelItem</code>.
       * Type parameter <code>TValue</code>.
       * @param valueType
       * @param modelItemType
       * @param name The GraphML name of the attribute
       * @param uri The URI to add to the key definition to disambiguate keys.
       * @param tag The tag name of the mapper in the {@link }
       * @param serializationCallback custom serialization callback.
       * @param declaredKeyType The key type to write in the declaration.
       * @template TModelItem,TValue
       */
      addRegistryOutputMapper<TModelItem,TValue>(modelItemType:yfiles.lang.Class,valueType:yfiles.lang.Class,name:string,uri:string,tag:Object,serializationCallback:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void,declaredKeyType:yfiles.graphml.KeyType):void;
      /**
       * Use a registered {@link yfiles.collections.IMapper.<K,V>} instance as source for the given GraphML attribute.
       * Type parameter <code>TModelItem</code>.
       * Type parameter <code>TValue</code>.
       * @param valueType
       * @param modelItemType
       * @param name The GraphML name of the attribute
       * @param tag The tag name of the mapper in the {@link }
       * @template TModelItem,TValue
       */
      addRegistryOutputMapper<TModelItem,TValue>(modelItemType:yfiles.lang.Class,valueType:yfiles.lang.Class,name:string,tag:Object):void;
      /**
       * Use a registered {@link yfiles.collections.IMapper.<K,V>} instance as source for the given GraphML attribute.
       * <p>
       * The type arguments of the mapper are determined automatically.
       * </p>
       * @param name The GraphML name of the attribute
       * @param uri The optional URI of the attribute
       * @param tag The tag name of the mapper in the {@link }
       * @param serializationCallback An optional serialization handler
       */
      addRegistryOutputMapper(name:string,uri:string,tag:Object,serializationCallback:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void):void;
      /**
       * Use a registered {@link yfiles.collections.IMapper.<K,V>} instance as source for the given GraphML attribute.
       * <p>
       * The type arguments of the mapper are determined automatically.
       * </p>
       * @param name The GraphML name of the attribute
       * @param tag The tag name of the mapper in the {@link }
       */
      addRegistryOutputMapper(name:string,tag:Object):void;
      /**
       * Adds a schema location to the header of a GraphML file.
       * @param schemaNamespace The namespace URI for this schema location
       * @param schemaLocation The schema location
       */
      addSchemaLocation(schemaNamespace:string,schemaLocation:string):void;
      /**
       * Add a namespace mapping between a fully qualified XML name (consisting of <code>xmlNamespace</code> and <code>tagName</code> and a type.
       * <p>
       * Either of <code>xmlNamespace</code> or <code>tagName</code> may be <code>null</code>, meaning that the default rules for determining the namespace and/or tag
       * name of <code>type</code> are used.
       * </p>
       * <p>
       * This is used both for writing and parsing objects.
       * </p>
       * @param xmlNamespace The XML namespace that corresponds to <code>type</code>. If <code>null</code>, the namespace will be determined based on the JavaScript module
       * that contains <code>type</code>
       * @param tagName The XML tag name that corresponds to <code>type</code>. If <code>null</code>, the name will be determined from the type name automatically.
       * @param type The type that is mapped to the fully qualified XML name.
       * @see yfiles.graphml.IXamlNameMapper
       * @see yfiles.graphml.GraphMLIOHandler#createXamlNameMapper
       * @see yfiles.graphml.GraphMLIOHandler#addXamlNamespaceMapping
       */
      addXamlNamespaceMapping(xmlNamespace:string,tagName:string,type:yfiles.lang.Class):void;
      /**
       * Add a namespace mapping between an XML namespace and a JavaScript <code>module</code>.
       * <p>
       * This is used both for writing and parsing objects.
       * </p>
       * @param xmlNamespace The XML namespace that corresponds to <code>module</code>.
       * @param module The module that is mapped to the namespace.
       * @see yfiles.graphml.IXamlNameMapper
       * @see yfiles.graphml.GraphMLIOHandler#createXamlNameMapper
       * @see yfiles.graphml.GraphMLIOHandler#addXamlNamespaceMapping
       */
      addXamlNamespaceMapping(xmlNamespace:string,module:Object):void;
      /**
       * Configure the predefined handlers for the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization}
       * event.
       * <p>
       * Except for text content and primitive types, this implementation delegates all work to
       * {@link yfiles.graphml.GraphMLIOHandler#handleXamlDeserialization}
       * </p>
       * <p>
       * This method is called for each parse process.
       * </p>
       * @param graph The graph that will be parsed.
       * @param parser The parser to configure.
       * @see yfiles.graphml.GraphMLIOHandler#handlePrimitivesDeserialization
       * @see yfiles.graphml.GraphMLIOHandler#handleXamlDeserialization
       * @see yfiles.graphml.GraphMLIOHandler#handleTextNodeDeserialization
       * @protected
       */
      configureDeserializationHandlers(graph:yfiles.graph.IGraph,parser:yfiles.graphml.GraphMLParser):void;
      /**
       * Configures the {@link yfiles.graphml.GraphMLIOHandler#deserializationPropertyOverrides deserialization property overrides} for the given parser.
       * <p>
       * This method is called for each parse process.
       * </p>
       * @param parser The parser to set the deserialization properties.
       * @protected
       */
      configureDeserializationPropertyOverrides(parser:yfiles.graphml.GraphMLParser):void;
      /**
       * Configures a given instance of {@link yfiles.graphml.GraphMLParser} that can handle the given graph.
       * <p>
       * This method is called by {@link yfiles.graphml.GraphMLIOHandler#createGraphMLParser}.
       * </p>
       * @param parser The parser to configure
       * @param graph The graph that should be filled by the parser
       * @protected
       */
      configureGraphMLParser(parser:yfiles.graphml.GraphMLParser,graph:yfiles.graph.IGraph):void;
      /**
       * Configures a given {@link yfiles.graphml.GraphMLWriter} that is used for writing <code>graph</code>.
       * <p>
       * This method is called by {@link yfiles.graphml.GraphMLIOHandler#createGraphMLWriter}.
       * </p>
       * @param writer The writer to configure
       * @param graph The graph that will be written.
       * @protected
       */
      configureGraphMLWriter(writer:yfiles.graphml.GraphMLWriter,graph:yfiles.graph.IGraph):void;
      /**
       * Configures default event handlers for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event.
       * <p>
       * This implementation configures several input handlers for the standard yFiles for HTML GraphML attributes (styles,
       * folding state, labels, geometry).
       * </p>
       * <p>
       * This method is called for each parse process.
       * </p>
       * @param parser The parser that should be configured.
       * @protected
       */
      configureInputHandlers(parser:yfiles.graphml.GraphMLParser):void;
      /**
       * Configures <code>writer</code> to use the namespace declarations set with {@link yfiles.graphml.GraphMLIOHandler#addNamespace}.
       * <p>
       * In addition to the namespace declarations provided with {@link yfiles.graphml.GraphMLIOHandler#addNamespace}, the default {@link yfiles.graphml.GraphMLWriter}
       * implementation also automatically places all namespace declarations that have occurred
       * </p>
       * <p>
       * This method is called for each write process.
       * </p>
       * @param writer The writer to configure.
       * @protected
       */
      configureNamespaces(writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configures default event handlers for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event.
       * <p>
       * This implementation configures several output handlers for the standard yFiles GraphML attributes (styles, folding
       * state, labels, geometry).
       * </p>
       * <p>
       * This method is called for each write process.
       * </p>
       * @param graph The graph that will be written.
       * @param writer The writer that should be configured.
       * @protected
       */
      configureOutputHandlers(graph:yfiles.graph.IGraph,writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configures the lookup of the given parser.
       * <p>
       * This method is called for each parse process.
       * </p>
       * @param parser The parser to configure.
       * @protected
       */
      configureParserLookup(parser:yfiles.graphml.GraphMLParser):void;
      /**
       * Configures <code>writer</code> to use the schema locations set with {@link yfiles.graphml.GraphMLIOHandler#addSchemaLocation}.
       * <p>
       * This method is called for each write process.
       * </p>
       * @param writer The writer to configure.
       * @protected
       */
      configureSchemaLocations(writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configures the predefined handlers for the {@link yfiles.graphml.GraphMLWriter#addHandleSerializationListener HandleSerialization}
       * event.
       * <p>
       * This method is called for each write process.
       * </p>
       * @param graph The graph that will be written.
       * @param writer The writer to configure.
       * @protected
       */
      configureSerializationHandlers(graph:yfiles.graph.IGraph,writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configure important serialization properties from properties of {@link yfiles.graphml.GraphMLIOHandler}.
       * <p>
       * This method is called for each write process.
       * </p>
       * @param graph The graph to write.
       * @param writer The writer to configure.
       * @see yfiles.graphml.GraphMLWriter#setSerializationProperty
       * @protected
       */
      configureSerializationProperties(graph:yfiles.graph.IGraph,writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configures the {@link yfiles.graphml.GraphMLIOHandler#serializationPropertyOverrides serialization property overrides} for the given writer.
       * <p>
       * This method is called for each write process.
       * </p>
       * @param writer The writer to set the serialization properties.
       * @protected
       */
      configureSerializationPropertyOverrides(writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configure the lookup map of <code>writer</code>.
       * <p>
       * This method is called for each write process.
       * </p>
       * @param graph The graph that will be written.
       * @param writer The writer to configure.
       * @see yfiles.graphml.GraphMLWriter#setLookup
       * @protected
       */
      configureWriterLookup(graph:yfiles.graph.IGraph,writer:yfiles.graphml.GraphMLWriter):void;
      /**
       * Configure <code>writer</code>.
       * <p>
       * This method is called for each write process.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @param writer The writer to configure.
       * @protected
       */
      configureXMLWriter(writer:yfiles.graphml.IXmlWriter):void;
      /**
       * Creates the default value serializer context and returns it.
       * <p>
       * This factory method can be used to provide custom {@link yfiles.graphml.ValueSerializer}s in cases where no {@link yfiles.graphml.GraphMLAttribute#valueSerializer} can be used.
       * </p>
       * @returns An implementation of the {@link } interface.
       */
      createDefaultValueSerializerContext():yfiles.graphml.IValueSerializerContext;
      /**
       * Create an {@link yfiles.graphml.IGraphElementFactory} instance that creates the graph elements for the <code>graph</code> instance.
       * <p>
       * This method is called for each parse process.
       * </p>
       * @param parser The parser to use.
       * @param graph The graph that is populated.
       * @returns An {@link } instance that is configured for the given parameters.
       * @protected
       */
      createGraphElementFactory(parser:yfiles.graphml.GraphMLParser,graph:yfiles.graph.IGraph):yfiles.graphml.IGraphElementFactory;
      /**
       * Create a suitable instance of {@link yfiles.graphml.GraphMLParser} that can handle the given graph.
       * <p>
       * This method is called for each parse process.
       * </p>
       * <p>
       * It delegates the configuration of the parser to {@link yfiles.graphml.GraphMLIOHandler#configureGraphMLParser}.
       * </p>
       * @param graph The graph that should be filled by this instance
       * @returns a suitable instance of {@link } that can handle the given graph
       * @protected
       */
      createGraphMLParser(graph:yfiles.graph.IGraph):yfiles.graphml.GraphMLParser;
      /**
       * Create and configure a new {@link yfiles.graphml.GraphMLWriter} that is used for writing <code>graph</code>.
       * <p>
       * This method is called for each write process. It delegates the configuration of the writer to
       * {@link yfiles.graphml.GraphMLIOHandler#configureGraphMLWriter}.
       * </p>
       * @param graph The graph that will be written.
       * @returns A new {@link } instance that is configured for <code>graph</code>.
       * @protected
       */
      createGraphMLWriter(graph:yfiles.graph.IGraph):yfiles.graphml.GraphMLWriter;
      /**
       * Callback used by {@link yfiles.graphml.GraphMLIOHandler#addRegistryInputMapper} and its overloads to creates the mapper
       * instance.
       * <p>
       * This method creates the mapper instance, unless there is an existing mapper in the target graph with a matching type. In
       * this case the existing mapper is returned. In all other cases a new mapper will be created and returned. If a mapper is
       * already registered with the same <code>registryTag</code> but wrong types, it will be replaced.
       * </p>
       * Type parameter <code>TKey</code>: The type of the keys.
       * Type parameter <code>TData</code>: The type of the data.
       * @param dataType The type of the data.
       * @param keyType The type of the keys.
       * @param context
       * @param registryTag The tag for the mapper in the registry.
       * @returns The mapper instance to use.
       * @protected
       * @template TKey,TData
       */
      createMapper<TKey,TData>(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,context:yfiles.graphml.IParseContext,registryTag:Object):yfiles.collections.IMapper<TKey,TData>;
      /**
       * Convenience factory method that can create an instance of several predefined {@link yfiles.graphml.IInputHandler}
       * implementations that match the provided <code>TKey</code> and <code>TData</code>.
       * Type parameter <code>TKey</code>: The type of the mapper keys
       * Type parameter <code>TData</code>: The type of the mapper entries
       * @param dataType The type of the mapper entries
       * @param modelItemType The type of the mapper keys
       * @param mapper The mapper to use.
       * @param deserializationCallback Optional deserialization callback for complex data.
       * @returns A suitable {@link } implementation
       * @template TKey,TData
       * @static
       */
      static createMapperInputHandler<TKey,TData>(modelItemType:yfiles.lang.Class,dataType:yfiles.lang.Class,mapper:yfiles.collections.IMapper<TKey,TData>,deserializationCallback:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):yfiles.graphml.MapperInputHandler<TKey,TData>;
      /**
       * Convenience factory method that can create an instance of several predefined {@link yfiles.graphml.IOutputHandler}
       * implementations that match the provided <code>TKey</code> and <code>TData</code>.
       * Type parameter <code>TKey</code>: The type of the mapper keys
       * Type parameter <code>TData</code>: The type of the mapper entries
       * @param dataType The type of the mapper entries
       * @param modelItemType The type of the mapper keys
       * @param name The name of the graphml attribute
       * @param mapper The mapper to use.
       * @param serializationCallback Optional serialization callback for complex data.
       * @returns A suitable {@link } implementation
       * @template TKey,TData
       * @static
       */
      static createMapperOutputHandler<TKey,TData>(modelItemType:yfiles.lang.Class,dataType:yfiles.lang.Class,name:string,mapper:yfiles.collections.IMapper<TKey,TData>,serializationCallback:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void):yfiles.graphml.MapperOutputHandler<TKey,TData>;
      /**
       * Configure mappings from CLR to XML names.
       * <p>
       * This implementation scans all loaded assemblies for XmlnsDefinitionAttributes and returns a suitable instance of
       * {@link yfiles.graphml.IXamlNameMapper}.
       * </p>
       * <p>
       * This method is called for each read or write process.
       * </p>
       * @returns 
       * @protected
       */
      createXamlNameMapper():yfiles.graphml.IXamlNameMapper;
      /**
       * Configure mappings from XML namespaces to prefixes.
       * <p>
       * This method is called for each write process.
       * </p>
       * @returns 
       * @protected
       */
      createXamlPrefixMapper():yfiles.graphml.IXamlPrefixMapper;
      /**
       * Factory method that creates and configures a suitable {@link yfiles.graphml.IXmlWriter} implementation.
       * <p>
       * This method is called for each write process. The {@link yfiles.graphml.IXmlWriter} implementation is used by default for all
       * low-level write operations.
       * </p>
       * @returns A suitable {@link } instance for the serialization
       * @see yfiles.graphml.IWriteContext#writer
       * @protected
       */
      createXMLWriter():yfiles.graphml.IXmlWriter;
      /**
       * Handle the serialization of arrays.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is an array. This handler is registered by default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleArraySerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization for enums.
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleEnumSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of types that extend FrameworkTemplate.
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleJSONDeserialization(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Handle the serialization of plain JavaScript object that can be serialized to JSON.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} can be converted to JSON data. This serializer is only called from element property syntax and
       * is registered by default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleJSONSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of {@link yfiles.collections.IEnumerable.<T>} instances.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is an IEnumerable instance. This handler is registered by default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleListSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of {@link yfiles.graphml.MarkupExtension} convertible instances.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} can be converted to a {@link yfiles.graphml.MarkupExtension} instance. This handler is registered by
       * default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleMarkupExtensionBasedSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of <code>null</code> references.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is a <code>null</code> reference. This handler is registered by default and is only used to write <code>null</code>
       * references in XML element syntax.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleNullSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of primitive types, enums and strings.
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handlePrimitivesDeserialization(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Handle the serialization of primitive types, enums and strings.
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handlePrimitivesSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of {@link yfiles.graphml.IReferenceHandle} instances.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is a {@link yfiles.graphml.IReferenceHandle} instance. This handler is registered by default and should
       * not be removed if {@link yfiles.graphml.GraphMLIOHandler#writeSharedReferences} is enabled.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleReferenceHandleSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of instances that can be serialized through a {@link yfiles.graphml.ValueSerializer}.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} can be converted to a string with a
       * {@link yfiles.graphml.ValueSerializer}. This serializer is only called from element property syntax and is registered by
       * default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleTextConvertibleSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Callback for the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
       * @param source The source of the event.
       * @param evt The event arguments.
       * @see yfiles.graphml.HandleDeserializationEventArgs
       * @protected
       */
      handleTextNodeDeserialization(source:Object,evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Handle the serialization of TypeExtensions.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is an array. This handler is registered by default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleTypeExtensionSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of Types.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is an array. This handler is registered by default.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleTypeSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Handle the serialization of <code>undefined</code> references.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is an <code>undefined</code> reference. This handler is registered by default and is only used to write <code>undefined</code>
       * references in XML element syntax.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleUndefinedSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Callback for the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
       * @param source The source of the event.
       * @param evt The event arguments.
       * @see yfiles.graphml.HandleDeserializationEventArgs
       * @protected
       */
      handleXamlDeserialization(source:Object,evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Handle the serialization of complex objects that are not handled by other callbacks.
       * <p>
       * This sets {@link yfiles.graphml.HandleSerializationEventArgs#handled} to <code>true</code> iff {@link yfiles.graphml.HandleSerializationEventArgs#item} is not a <code>null</code> reference. This handler is registered by default and uses {@link yfiles.graphml.XamlSerializer}
       * to create XAML output.
       * </p>
       * @param sender The origin of the event.
       * @param evt The event handler arguments.
       * @protected
       */
      handleXamlSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Checks whether the <code>name</code> attribute in <code>definition</code> matches <code>name</code>
       * @param definition The key definition to check
       * @param name The name that should be matched
       * @returns <code>true</code> if the name attribute in <code>definition</code> matches <code>name</code>
       * @static
       */
      static matchesName(definition:Element,name:string):boolean;
      /**
       * Checks whether the <code>scope</code> attribute in <code>definition</code> matches <code>scope</code>
       * @param definition The key definition to check
       * @param scope The scope that should be matched
       * @returns <code>true</code> if the scope attribute in <code>definition</code> matches <code>scope</code>
       * @static
       */
      static matchesScope(definition:Element,scope:yfiles.graphml.KeyScope):boolean;
      /**
       * Checks whether the <code>type</code> attribute in <code>definition</code> matches <code>type</code>
       * @param definition The key definition to check
       * @param type The type that should be matched
       * @returns <code>true</code> if the type attribute in <code>definition</code> matches <code>type</code>
       * @static
       */
      static matchesType(definition:Element,type:yfiles.graphml.KeyType):boolean;
      /**
       * Checks whether the <code>uri</code> attribute in <code>definition</code> matches <code>uri</code>
       * @param definition The key definition to check
       * @param uri The uri that should be matched
       * @returns <code>true</code> if the uri attribute in <code>definition</code> matches <code>uri</code>
       * @static
       */
      static matchesUri(definition:Element,uri:string):boolean;
      /**
       * Event handler for the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
       * <p>
       * This event handler is used to propagate the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization}
       * event to subscribers of
       * {@link yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}. This implementation
       * just calls {@link yfiles.graphml.GraphMLIOHandler#onHandleDeserialization}, which in turn raises the {@link yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}
       * event.
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      onGraphMLParserHandleDeserialization(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Event handler for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers} event.
       * <p>
       * This event handler is used to propagate the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event to subscribers of {@link yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers}.
       * This implementation just calls {@link yfiles.graphml.GraphMLIOHandler#onQueryInputHandlers}, which in turn raises the {@link yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers}
       * event.
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      onGraphMLParserQueryInputHandlers(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Event handler for the {@link yfiles.graphml.GraphMLWriter#addHandleSerializationListener HandleSerialization} event.
       * <p>
       * This event handler is used to propagate the {@link yfiles.graphml.GraphMLWriter#addHandleSerializationListener HandleSerialization}
       * event to subscribers of {@link yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener HandleSerialization}.
       * This implementation just calls {@link yfiles.graphml.GraphMLIOHandler#onHandleSerialization}, which in turn raises the {@link yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener HandleSerialization}
       * event.
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      onGraphMLWriterHandleSerialization(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Event handler for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers} event.
       * <p>
       * This event handler is used to propagate the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event to subscribers of {@link yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}.
       * This implementation just calls {@link yfiles.graphml.GraphMLIOHandler#onQueryOutputHandlers}, which in turn raises the {@link yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}
       * event.
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      onGraphMLWriterQueryOutputHandlers(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization} event.
       * @param evt The arguments for the {@link #addHandleDeserializationListener HandleDeserialization}
       * event.
       * @protected
       */
      onHandleDeserialization(evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener HandleSerialization} event.
       * @param evt The arguments for the {@link #addHandleSerializationListener HandleSerialization}
       * event.
       * @protected
       */
      onHandleSerialization(evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Fires the {@link yfiles.graphml.GraphMLIOHandler#addOverrideResolveReferenceListener OverrideResolveReference} event.
       * @param evt The arguments that get passed to the {@link #addOverrideResolveReferenceListener OverrideResolveReference}
       * event.
       * @protected
       */
      onOverrideResolveReference(evt:yfiles.graphml.ResolveReferenceEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addParsedListener Parsed} event.
       * <p>
       * This method is called when the document has been parsed and after {@link yfiles.graphml.IParseEvents#addDocumentParsedListener DocumentParsed}
       * is called.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLIOHandler#addParsedListener
       * @protected
       */
      onParsed(evt:yfiles.graphml.ParseEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addParsingListener Parsing} event.
       * <p>
       * This method is called just before the parsing starts and before {@link yfiles.graphml.IParseEvents#addDocumentParsingListener DocumentParsing}
       * is called.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLIOHandler#addParsingListener
       * @protected
       */
      onParsing(evt:yfiles.graphml.ParseEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener QueryInputHandlers} event.
       * @param evt The arguments for the {@link #addQueryInputHandlersListener QueryInputHandlers}
       * event.
       * @protected
       */
      onQueryInputHandlers(evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers} event.
       * @param evt The arguments for the {@link #addQueryOutputHandlersListener QueryOutputHandlers}
       * event.
       * @protected
       */
      onQueryOutputHandlers(evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Fires the {@link yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId} event.
       * @param evt The arguments that get passed to the {@link #addQueryReferenceIdListener QueryReferenceId}
       * event.
       * @throws {Stubs.Exceptions.ArgumentError} <code>args</code> is null.
       * @see yfiles.graphml.GraphMLWriter#onQueryReferenceId
       * @protected
       */
      onQueryReferenceId(evt:yfiles.graphml.QueryReferenceIdEventArgs):void;
      /**
       * Fires the {@link yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener ResolveReference} event.
       * @param evt The arguments that get passed to the {@link #addResolveReferenceListener ResolveReference}
       * event.
       * @protected
       */
      onResolveReference(evt:yfiles.graphml.ResolveReferenceEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addWritingListener Writing} event.
       * <p>
       * This method is called just before the writing starts and before {@link yfiles.graphml.IWriteEvents#addDocumentWritingListener DocumentWriting}
       * is called.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLIOHandler#addWritingListener
       * @protected
       */
      onWriting(evt:yfiles.graphml.WriteEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLIOHandler#addWrittenListener Written} event.
       * <p>
       * This method is called just after the writing finishes and after {@link yfiles.graphml.IWriteEvents#addDocumentWrittenListener DocumentWritten}
       * is called.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLIOHandler#addWrittenListener
       * @protected
       */
      onWritten(evt:yfiles.graphml.WriteEventArgs):void;
      /**
       * Read GraphML from an existing XML document.
       * @param graph The Graph object that is to be populated with nodes and edges as read from the document.
       * @param document The XML document. <b>Note</b>: If you are using Internet Explorer 9 and {@link }s to retrieve the document, you may not use
       * the value from the {@link #responseXML} property. Please parse {@link #responseText} instead and use the
       * result. For further reference see {@link http://msdn.microsoft.com/en-us/library/ie/hh180177.aspx}.
       * @returns 
       * @throws {Stubs.Exceptions.ArgumentError} <code>document</code> is null.
       * @throws {Stubs.Exceptions.ArgumentError} <code>graph</code> is null.
       */
      readFromDocument(graph:yfiles.graph.IGraph,document:Document):Promise<yfiles.graph.IGraph>;
      /**
       * Convenience method that imports the <code>graph</code> from an XML data provided in a string <code>data</code>.
       * @param graph The Graph object that is to be populated with nodes and edges as read from the GraphML data.
       * @param data A string that contains GraphML data.
       * @returns 
       */
      readFromGraphMLText(graph:yfiles.graph.IGraph,data:string):Promise<yfiles.graph.IGraph>;
      /**
       * Reads a GraphML file from the given GraphML file and populates the Graph object.
       * <p>
       * Note that this is an asynchronous method that returns immediately. Use the returned Promise to get notified when the
       * actual parsing has completed.
       * </p>
       * @param graph The Graph object that is to be populated with nodes and edges as read from the GraphML file.
       * @param url The url of the file to be read.
       * @returns 
       * @throws {Stubs.Exceptions.ArgumentError} <code>url</code> is null.
       * @throws {Stubs.Exceptions.ArgumentError} <code>graph</code> is null.
       */
      readFromURL(graph:yfiles.graph.IGraph,url:string):Promise<yfiles.graph.IGraph>;
      /**
       * Predefined input handler that reads edge bends.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerEdgeBendsInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes edge bends.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerEdgeBendsOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads edge labels.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerEdgeLabelsInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes edge labels.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerEdgeLabelsOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads edge styles.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerEdgeStyleInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes edge styles.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerEdgeStyleOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads {@link yfiles.graph.IGraph} graph default settings.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerGraphSettingsInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes the settings of a graph.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event.
       * </p>
       * @param sender
       * @param evt
       * @see yfiles.graphml.SerializationProperties#DISABLE_GRAPH_SETTINGS
       * @protected
       */
      registerGraphSettingsOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads node labels.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerNodeLabelsInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes node labels.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerNodeLabelsOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads node layouts.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerNodeLayoutInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes node layouts.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerNodeLayoutOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads node styles.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerNodeStyleInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes node styles.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerNodeStyleOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads port locations.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerPortLocationInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes port locations.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerPortLocationOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads port styles.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerPortStyleInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes port styles.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerPortStyleOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Predefined input handler that reads {@link yfiles.graph.ITagOwner} tags.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerTagInputHandler(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Predefined output handler that writes {@link yfiles.graph.ITagOwner} tags.
       * <p>
       * This handler is by default registered for the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      registerTagOutputHandler(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Writes the given graph object to a GraphML file with a given filename.
       * @param graph The Graph object that is to be written to a GraphML file.
       * @returns A string containing the GraphML data.
       */
      write(graph:yfiles.graph.IGraph):Promise<string>;
      /**
       * <code>true</code> if schema descriptions should be written.
       * @type {boolean}
       */
      writeXMLSchema:boolean;
      /**
       * Whether to support object sharing on output.
       * <p>
       * If enabled, multiple object references will be correctly represented in the output document, otherwise, each object
       * reference will be serialized independently.
       * </p>
       * <p>
       * <b>Note: Reading</b> of shared references is always supported
       * </p>
       * @type {boolean}
       */
      writeSharedReferences:boolean;
      /**
       * Whether to clear an existing graph instance before reading
       * <p>
       * If set
       * to<code>true</code>, an existing graph instance will be cleared before the graph is read into this instance, otherwise the loaded
       * graph will be merged into the existing one.
       * </p>
       * @type {boolean}
       */
      clearGraphBeforeRead:boolean;
      /**
       * Adds the given listener for the <code>OverrideResolveReference</code> event that occurs before inline references are evaluated.
       * <p>
       * Subscribe to this event to customize resolving of GraphML object references. This event is fired before the framework
       * tries to resolve internal references and can therefore be used to override internal references, for example.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener
       * @see yfiles.graphml.GraphMLIOHandler#removeOverrideResolveReferenceListener
       */
      addOverrideResolveReferenceListener(listener:(sender:Object,evt:yfiles.graphml.ResolveReferenceEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>OverrideResolveReference</code> event that occurs before inline references are evaluated.
       * <p>
       * Subscribe to this event to customize resolving of GraphML object references. This event is fired before the framework
       * tries to resolve internal references and can therefore be used to override internal references, for example.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener
       * @see yfiles.graphml.GraphMLIOHandler#addOverrideResolveReferenceListener
       */
      removeOverrideResolveReferenceListener(listener:(sender:Object,evt:yfiles.graphml.ResolveReferenceEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>ResolveReference</code> event that occurs after references are evaluated.
       * <p>
       * Subscribe to this event to customize resolving of GraphML object references. This event is fired when the framework
       * could not resolve an object reference by an internal one.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addOverrideResolveReferenceListener
       * @see yfiles.graphml.GraphMLIOHandler#removeResolveReferenceListener
       */
      addResolveReferenceListener(listener:(sender:Object,evt:yfiles.graphml.ResolveReferenceEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ResolveReference</code> event that occurs after references are evaluated.
       * <p>
       * Subscribe to this event to customize resolving of GraphML object references. This event is fired when the framework
       * could not resolve an object reference by an internal one.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addOverrideResolveReferenceListener
       * @see yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener
       */
      removeResolveReferenceListener(listener:(sender:Object,evt:yfiles.graphml.ResolveReferenceEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
       * <p>
       * This event can be used to provide names of external or internal references for objects.
       * </p>
       * <p>
       * The reference targets are not serialized to the GraphML file if they are set to
       * {@link yfiles.graphml.GraphMLReferenceType#EXTERNAL}. To resolve these references when parsing, you typically have to subscribe
       * to the {@link yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener ResolveReference} event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.QueryReferenceIdEventArgs#referenceId
       * @see yfiles.graphml.QueryReferenceIdEventArgs#referenceType
       * @see yfiles.graphml.GraphMLIOHandler#removeQueryReferenceIdListener
       */
      addQueryReferenceIdListener(listener:(sender:Object,evt:yfiles.graphml.QueryReferenceIdEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
       * <p>
       * This event can be used to provide names of external or internal references for objects.
       * </p>
       * <p>
       * The reference targets are not serialized to the GraphML file if they are set to
       * {@link yfiles.graphml.GraphMLReferenceType#EXTERNAL}. To resolve these references when parsing, you typically have to subscribe
       * to the {@link yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener ResolveReference} event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.QueryReferenceIdEventArgs#referenceId
       * @see yfiles.graphml.QueryReferenceIdEventArgs#referenceType
       * @see yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener
       */
      removeQueryReferenceIdListener(listener:(sender:Object,evt:yfiles.graphml.QueryReferenceIdEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Written</code> event that occurs when the writing of a document has been finished.
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#onWritten
       * @see yfiles.graphml.GraphMLWriter#addWrittenListener
       * @see yfiles.graphml.GraphMLIOHandler#removeWrittenListener
       */
      addWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Written</code> event that occurs when the writing of a document has been finished.
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#onWritten
       * @see yfiles.graphml.GraphMLWriter#addWrittenListener
       * @see yfiles.graphml.GraphMLIOHandler#addWrittenListener
       */
      removeWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.WriteEventArgs#context}'s {@link yfiles.graphml.IWriteContext#writeEvents} and to configure the writing process
       * on a per write case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#onWriting
       * @see yfiles.graphml.GraphMLWriter#addWritingListener
       * @see yfiles.graphml.GraphMLIOHandler#removeWritingListener
       */
      addWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.WriteEventArgs#context}'s {@link yfiles.graphml.IWriteContext#writeEvents} and to configure the writing process
       * on a per write case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#onWriting
       * @see yfiles.graphml.GraphMLWriter#addWritingListener
       * @see yfiles.graphml.GraphMLIOHandler#addWritingListener
       */
      removeWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Gets the mapper for the overrides of the serialization properties.
       * <p>
       * Using this mapper {@link yfiles.graphml.IWriteContext#getSerializationProperty serialization properties} can be specified.
       * </p>
       * @see yfiles.graphml.GraphMLIOHandler#configureSerializationPropertyOverrides
       * @type {yfiles.collections.IMap.<string,Object>}
       */
      serializationPropertyOverrides:yfiles.collections.IMap<string,Object>;
      /**
       * Gets the mapper for the overrides of the deserialization properties.
       * <p>
       * Using this mapper {@link yfiles.graphml.IParseContext#getDeserializationProperty deserialization properties} can be specified.
       * </p>
       * @see yfiles.graphml.GraphMLIOHandler#configureDeserializationPropertyOverrides
       * @type {yfiles.collections.IMap.<string,Object>}
       */
      deserializationPropertyOverrides:yfiles.collections.IMap<string,Object>;
      /**
       * Adds the given listener for the <code>QueryType</code> event that occurs when a type is queried.
       * <p>
       * This event can be used to override type resolution for XML names in XAML.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.QueryTypeEventArgs
       * @see yfiles.graphml.GraphMLIOHandler#removeQueryTypeListener
       */
      addQueryTypeListener(listener:(sender:Object,evt:yfiles.graphml.QueryTypeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryType</code> event that occurs when a type is queried.
       * <p>
       * This event can be used to override type resolution for XML names in XAML.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.QueryTypeEventArgs
       * @see yfiles.graphml.GraphMLIOHandler#addQueryTypeListener
       */
      removeQueryTypeListener(listener:(sender:Object,evt:yfiles.graphml.QueryTypeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>QueryName</code> event that occurs when a name is queried.
       * <p>
       * This event can be used to override name mappings to XML names in XAML.
       * </p>
       * <p>
       * If the event handler determines that it map resolve {@link yfiles.graphml.QueryNameEventArgs#type} to a xml name, it should place the result into the {@link yfiles.graphml.QueryNameEventArgs#result} property and
       * thus mark the event as {@link yfiles.graphml.QueryNameEventArgs#handled}.
       * </p>
       * <p>
       * It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
       * determination rules are used for that part.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#removeQueryNameListener
       */
      addQueryNameListener(listener:(sender:Object,evt:yfiles.graphml.QueryNameEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryName</code> event that occurs when a name is queried.
       * <p>
       * This event can be used to override name mappings to XML names in XAML.
       * </p>
       * <p>
       * If the event handler determines that it map resolve {@link yfiles.graphml.QueryNameEventArgs#type} to a xml name, it should place the result into the {@link yfiles.graphml.QueryNameEventArgs#result} property and
       * thus mark the event as {@link yfiles.graphml.QueryNameEventArgs#handled}.
       * </p>
       * <p>
       * It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
       * determination rules are used for that part.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryNameListener
       */
      removeQueryNameListener(listener:(sender:Object,evt:yfiles.graphml.QueryNameEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
       * <p>
       * Subscribe to this event to provide custom serialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current item is provided by {@link yfiles.graphml.HandleSerializationEventArgs#item}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#removeHandleSerializationListener
       */
      addHandleSerializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
       * <p>
       * Subscribe to this event to provide custom serialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current item is provided by {@link yfiles.graphml.HandleSerializationEventArgs#item}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener
       */
      removeHandleSerializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>QueryOutputHandlers</code> event that occurs when a GraphML attribute is about to be written.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IOutputHandler} instance(s) for a given
       * GraphML attribute key definition.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * If you want to register an {@link yfiles.graphml.IOutputHandler} for this GraphML attribute, you can add the handler with
       * {@link yfiles.graphml.QueryOutputHandlersEventArgs#addOutputHandler}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#removeQueryOutputHandlersListener
       */
      addQueryOutputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryOutputHandlers</code> event that occurs when a GraphML attribute is about to be
       * written.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IOutputHandler} instance(s) for a given
       * GraphML attribute key definition.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * If you want to register an {@link yfiles.graphml.IOutputHandler} for this GraphML attribute, you can add the handler with
       * {@link yfiles.graphml.QueryOutputHandlersEventArgs#addOutputHandler}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener
       */
      removeQueryOutputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#onParsed
       * @see yfiles.graphml.GraphMLParser#addParsedListener
       * @see yfiles.graphml.GraphMLIOHandler#removeParsedListener
       */
      addParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#onParsed
       * @see yfiles.graphml.GraphMLParser#addParsedListener
       * @see yfiles.graphml.GraphMLIOHandler#addParsedListener
       */
      removeParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Parsing</code> event that occurs when the parsing of a document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.ParseEventArgs#context}'s {@link yfiles.graphml.IParseContext#parseEvents} and to configure the parsing process
       * on a per parse case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#onParsing
       * @see yfiles.graphml.GraphMLParser#addParsingListener
       * @see yfiles.graphml.GraphMLIOHandler#removeParsingListener
       */
      addParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Parsing</code> event that occurs when the parsing of a document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.ParseEventArgs#context}'s {@link yfiles.graphml.IParseContext#parseEvents} and to configure the parsing process
       * on a per parse case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#onParsing
       * @see yfiles.graphml.GraphMLParser#addParsingListener
       * @see yfiles.graphml.GraphMLIOHandler#addParsingListener
       */
      removeParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>QueryInputHandlers</code> event that occurs when a GraphML attribute is about to be read.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IInputHandler} instance(s) for a given GraphML
       * attribute key definition.
       * </p>
       * <p>
       * The current key definition is provided by {@link yfiles.graphml.QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link yfiles.graphml.IInputHandler} for this GraphML
       * attribute, you can add the handler with {@link yfiles.graphml.QueryInputHandlersEventArgs#addInputHandler} method.
       * Implementations should also consider the {@link yfiles.graphml.QueryInputHandlersEventArgs#handled} property.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLParser#addQueryInputHandlersListener
       * @see yfiles.graphml.GraphMLIOHandler#removeQueryInputHandlersListener
       */
      addQueryInputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryInputHandlers</code> event that occurs when a GraphML attribute is about to be read.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IInputHandler} instance(s) for a given GraphML
       * attribute key definition.
       * </p>
       * <p>
       * The current key definition is provided by {@link yfiles.graphml.QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link yfiles.graphml.IInputHandler} for this GraphML
       * attribute, you can add the handler with {@link yfiles.graphml.QueryInputHandlersEventArgs#addInputHandler} method.
       * Implementations should also consider the {@link yfiles.graphml.QueryInputHandlersEventArgs#handled} property.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLParser#addQueryInputHandlersListener
       * @see yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener
       */
      removeQueryInputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be deserialized.
       * <p>
       * Subscribe to this event to provide custom deserialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current XML content is provided by {@link yfiles.graphml.HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link yfiles.graphml.HandleDeserializationEventArgs#result} property.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLParser#addHandleDeserializationListener
       * @see yfiles.graphml.GraphMLIOHandler#removeHandleDeserializationListener
       */
      addHandleDeserializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be
       * deserialized.
       * <p>
       * Subscribe to this event to provide custom deserialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current XML content is provided by {@link yfiles.graphml.HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link yfiles.graphml.HandleDeserializationEventArgs#result} property.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLParser#addHandleDeserializationListener
       * @see yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener
       */
      removeHandleDeserializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for factory classes for {@link yfiles.graphml.IInputHandler} creation.
     * @interface
     */
    export interface IGenericInputHandlerFactory extends Object{
      /**
       * Create an {@link yfiles.graphml.IInputHandler} instance based on the given <code>args</code>.
       * @param keyType The key type as inferred by the caller from the GraphML contents.
       * @param valueType The type of the value as inferred by the caller from the GraphML contents.
       * @param evt The event argument that is used like in
       * {@link #addQueryInputHandlersListener QueryInputHandlers}
       * @returns The specific input handler to use.
       * @abstract
       */
      createInputHandler(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,evt:yfiles.graphml.QueryInputHandlersEventArgs):yfiles.graphml.IInputHandler;
    }
    var IGenericInputHandlerFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * 
     * @class
     */
    export interface GraphMLSupport extends Object{}
    export class GraphMLSupport {
      /**
       * Creates a new instance of this class.
       * <p>
       * Optionally calls {@link yfiles.graphml.GraphMLSupport#registerCommands}, if <code>graphControl</code> is given.
       * </p>
       * @param [graphComponent=null] The {@link } that the {@link #OPEN} and {@link #SAVE} commands are
       * registered on.
       */
      constructor(graphComponent?:yfiles.view.GraphComponent);
      /**
       * Creates a new instance of this class.
       * <p>
       * Optionally calls {@link yfiles.graphml.GraphMLSupport#registerCommands}, if <code>graphControl</code> is given.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.graphComponent=null] The {@link } that the {@link #OPEN} and {@link #SAVE} commands are
       * registered on.
       * @param {yfiles.graphml.StorageLocation} options.storageLocation A value indicating the type of storage used by the commands Open/Save/Save As.
       * <p>
       * This option sets the {@link yfiles.graphml.GraphMLSupport#storageLocation} property on the created object.
       * </p>
       * @param {yfiles.graphml.GraphMLIOHandler} options.graphMLIOHandler The {@link yfiles.graphml.GraphMLSupport#graphMLIOHandler} to use for the various I/O methods in this instance.
       * <p>
       * This option sets the {@link yfiles.graphml.GraphMLSupport#graphMLIOHandler} property on the created object.
       * </p>
       */
      constructor(options:{graphComponent?:yfiles.view.GraphComponent,storageLocation?:yfiles.graphml.StorageLocation,graphMLIOHandler?:yfiles.graphml.GraphMLIOHandler});
      /**
       * Helper callback method that creates the {@link HTMLInputElement} for use in the handling of the {@link yfiles.input.ICommand#OPEN}
       * command.
       * <p>
       * The open command that is enabled if {@link yfiles.graphml.GraphMLSupport#storageLocation} is set to a value different from {@link yfiles.graphml.StorageLocation#NONE} will use this
       * method to create the input element for opening the files.
       * </p>
       * @returns The preconfigured element to use.
       * @protected
       */
      createOpenFileInputElement():HTMLInputElement;
      /**
       * Helper callback method that creates the {@link HTMLElement} for use in the handling of the
       * {@link yfiles.input.ICommand#SAVE}and SaveAs commands.
       * <p>
       * If {@link yfiles.graphml.GraphMLSupport#storageLocation} is set to {@link yfiles.graphml.StorageLocation#FILE_SYSTEM} but saving to a file is not supported by the user's browser,
       * this method is called to create the element to which the GraphML content is appended.
       * </p>
       * @returns The preconfigured element to use. Null if element creation failed.
       * @protected
       */
      createSaveFileTextElement():HTMLElement;
      /**
       * Gets the {@link yfiles.graphml.GraphMLSupport#graphMLIOHandler} to use for the various I/O methods in this instance.
       * <p>
       * This method will first query the {@link yfiles.graph.ILookup#lookup} of the <code>graph</code>,
       * </p>
       * @param graph
       * @returns The instance to use for IO.
       * @protected
       */
      getGraphMLIOHandler(graph:yfiles.graph.IGraph):yfiles.graphml.GraphMLIOHandler;
      /**
       * Default implementation of {@link yfiles.input.ICommand#OPEN} that shows a {@link yfiles.graphml.GraphMLSupport#createOpenFileInputElement open file dialog} and imports the GraphML.
       * <p>
       * This method is called in response to the {@link yfiles.input.ICommand#OPEN} command if {@link yfiles.graphml.GraphMLSupport#storageLocation} is set to a value different from
       * {@link yfiles.graphml.StorageLocation#NONE}.
       * </p>
       * <p>
       * Any exception thrown during loading of the GraphML file will reject the promise.
       * </p>
       * @param graphComponent
       * @param [storageLocation=null]
       * @returns A promise that will resolve to the loaded graph instance or reject upon exceptions during loading. Note that if the file
       * open dialog is cancelled by the user, the promise will stay unresolved because there is no API available that can be
       * used to determine whether the user has pressed cancel. Depending on the UA implementation it may be possible to listen
       * to focus or other input device events on the page to detect when the user has closed the file open dialog.
       * @see yfiles.graphml.GraphMLSupport#storageLocation
       * @see yfiles.graphml.GraphMLSupport#saveFile
       */
      openFile(graphComponent:yfiles.view.GraphComponent,storageLocation?:yfiles.graphml.StorageLocation):Promise<yfiles.graph.IGraph>;
      /**
       * Default implementation of {@link yfiles.input.ICommand#OPEN} that shows a {@link yfiles.graphml.GraphMLSupport#createOpenFileInputElement open file dialog} and imports the GraphML.
       * <p>
       * This method is called in response to the {@link yfiles.input.ICommand#OPEN} command if {@link yfiles.graphml.GraphMLSupport#storageLocation} is set to a value different from
       * {@link yfiles.graphml.StorageLocation#NONE}.
       * </p>
       * <p>
       * Any exception thrown during loading of the GraphML file will be caught and an error message displayed to the user. For
       * more control over error behaviour it is recommended to use {@link yfiles.graphml.GraphMLSupport#graphMLIOHandler} directly, e.g. from a custom command handler.
       * </p>
       * @param graph
       * @param [storageLocation=null]
       * @returns A promise that will resolve to the loaded graph instance or reject upon exceptions during loading. Note that if the file
       * open dialog is cancelled by the user, the promise will stay unresolved because there is no API available that can be
       * used to determine whether the user has pressed cancel. Depending on the UA implementation it may be possible to listen
       * to focus or other input device events on the page to detect when the user has closed the file open dialog.
       * @see yfiles.graphml.GraphMLSupport#storageLocation
       * @see yfiles.graphml.GraphMLSupport#saveFile
       */
      openFile(graph:yfiles.graph.IGraph,storageLocation?:yfiles.graphml.StorageLocation):Promise<yfiles.graph.IGraph>;
      /**
       * Registers bindings for {@link yfiles.input.ICommand#OPEN} and {@link yfiles.input.ICommand#SAVE} on the given <code>graphControl</code>.
       * @param graphComponent The {@link } to register the command bindings on.
       */
      registerCommands(graphComponent:yfiles.view.GraphComponent):void;
      /**
       * Convenience method that exports the graph as GraphML and saves it to a storage location.
       * <p>
       * This method is called in response to the {@link yfiles.input.ICommand#SAVE} command if {@link yfiles.graphml.GraphMLSupport#storageLocation} is set to a value different from
       * {@link yfiles.graphml.StorageLocation#NONE}.
       * </p>
       * <p>
       * Any exception thrown during loading of the GraphML file will be caught and an error message displayed to the user. For
       * more control over error behaviour it is recommended to use {@link yfiles.graphml.GraphMLSupport#graphMLIOHandler} directly, e.g. from a custom command handler.
       * </p>
       * @param graph
       * @param [storageLocation=null]
       * @returns A promise that will resolve to the saved string representation of the graph or reject upon exceptions during the save
       * operation.
       * @see yfiles.graphml.GraphMLSupport#storageLocation
       * @see yfiles.graphml.GraphMLSupport#saveFile
       * @see yfiles.graphml.GraphMLSupport#openFile
       */
      saveFile(graph:yfiles.graph.IGraph,storageLocation?:yfiles.graphml.StorageLocation):Promise<string>;
      /**
       * The {@link yfiles.graphml.GraphMLSupport#graphMLIOHandler} to use for the various I/O methods in this instance.
       * @type {yfiles.graphml.GraphMLIOHandler}
       */
      graphMLIOHandler:yfiles.graphml.GraphMLIOHandler;
      /**
       * Gets or sets a value indicating the type of storage used by the commands Open/Save/Save As.
       * <p>
       * By default this feature is set to {@link yfiles.graphml.StorageLocation#NONE}.
       * </p>
       * <p>
       * This property specifies the storage type of the following commands:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.ICommand#OPEN GraphCommands.Open}</li>
       * <li>{@link yfiles.input.ICommand#SAVE GraphCommands.Save}</li>
       * <li>GraphCommands.SaveAs</li>
       * </ul>
       * <p>
       * Note that this doesn't affect the convenience methods ImportFromGraphML(string,EventHandler),
       * ExportToGraphML(TextWriter) and their variants.
       * </p>
       * <p>
       * The default implementations delegate to OpenFile, {@link yfiles.graphml.GraphMLSupport#saveFile} and {@link yfiles.graphml.GraphMLSupport#saveFile}
       * methods respectively.
       * </p>
       * @see yfiles.graphml.GraphMLSupport#createOpenFileInputElement
       * @see yfiles.graphml.GraphMLSupport#createSaveFileTextElement
       * @see yfiles.graphml.GraphMLSupport#saveFile
       * @see yfiles.graphml.GraphMLSupport#saveFile
       * @type {yfiles.graphml.StorageLocation}
       */
      storageLocation:yfiles.graphml.StorageLocation;
      static $class:yfiles.lang.Class;
    }
    export enum StorageLocation{
      /**
       * No location at all, effectively disabling storage operations.
       */
      NONE,
      /**
       * The local storage as described by the W3C Web Storage API.
       */
      LOCAL_STORAGE,
      /**
       * The session storage as described by the W3C Web Storage API.
       */
      SESSION_STORAGE,
      /**
       * The file system.
       * <p>
       * Opening a file from the file system works in all supported browsers. Saving a file is supported in Firefox 28+, Chrome
       * 38+, Opera 25+, IE 10+, recent versions of the related mobile browsers and Android Browser 4.4.4. If saving to a file is
       * not supported, the file contents is shown in a new browser window instead.
       * </p>
       */
      FILE_SYSTEM
    }
    /**
     * An implementation of {@link yfiles.graphml.IParseContext} that delegates most calls to a {@link yfiles.graphml.ChildParseContext#parentContext}, but adds the capability to locally
     * add {@link yfiles.graphml.ChildParseContext#lookup} results and {@link yfiles.graphml.ChildParseContext#getDeserializationProperty deserialization properties}
     * <p>
     * Whenever it is necessary to locally modify some context properties, a new instance of this class should be created that
     * wraps an existing one.
     * </p>
     * @see yfiles.graphml.ChildParseContext#setDeserializationProperty
     * @see yfiles.graphml.ChildParseContext#setLookup
     * @class
     * @implements {yfiles.graphml.IParseContext}
     */
    export interface ChildParseContext extends Object,yfiles.graphml.IParseContext{}
    export class ChildParseContext {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.ChildParseContext} class that delegates to the provided context and adds
       * the <code>currentObject</code> to the {@link yfiles.graphml.ChildParseContext#objectStack}.
       * @param parentContext The parent context.
       * @param currentObject The current object to add to the {@link #objectStack} and return in {@link #getCurrent}.
       */
      constructor(parentContext:yfiles.graphml.IParseContext,currentObject:Object);
      /**
       * Initializes a new instance of the {@link yfiles.graphml.ChildParseContext} class that delegates to the provided context.
       * @param parentContext The parent context.
       */
      constructor(parentContext:yfiles.graphml.IParseContext);
      /**
       * 
       * Type parameter <code>T</code>.
       * @param targetType
       * @param context
       * @param targetNode
       * @returns 
       * @template T
       */
      deserializeCore<T>(targetType:yfiles.lang.Class,context:yfiles.graphml.IParseContext,targetNode:Node):T;
      /**
       * 
       * Type parameter <code>T</code>.
       * @param itemType
       * @returns 
       * @template T
       */
      getCurrent<T>(itemType:yfiles.lang.Class):T;
      /**
       * 
       * @param key
       * @returns 
       */
      getDeserializationProperty(key:string):Object;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Removes a previously locally {@link yfiles.graphml.ChildParseContext#setDeserializationProperty set} deserialization property.
       * @param key The key for which the local property override should be removed.
       */
      removeDeserializationProperty(key:string):void;
      /**
       * Removes a local lookup override that has been added using {@link yfiles.graphml.ChildParseContext#setLookup}
       * Type parameter <code>T</code>: The type key to remove.
       * @param tType The type key to remove.
       * @template T
       */
      removeLookup<T>(tType:yfiles.lang.Class):void;
      /**
       * Sets the local deserialization property for the given key to the value.
       * @param key The key to override.
       * @param value The value.
       */
      setDeserializationProperty(key:string,value:Object):void;
      /**
       * Adds another lookup result to the local lookup override.
       * Type parameter <code>T</code>: The type to use as a key for the lookup.
       * @param tType The type to use as a key for the lookup.
       * @param value The value to provide for queries to <code>T</code>.
       * @see yfiles.graphml.ChildParseContext#removeLookup
       * @template T
       */
      setLookup<T>(tType:yfiles.lang.Class,value:T):void;
      /**
       * Gets the parent context.
       * @type {yfiles.graphml.IParseContext}
       */
      parentContext:yfiles.graphml.IParseContext;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<Object>}
       */
      objectStack:yfiles.collections.IListEnumerable<Object>;
      /**
       * 
       * @type {yfiles.graphml.IParseEvents}
       */
      parseEvents:yfiles.graphml.IParseEvents;
      /**
       * 
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable implementation of {@link yfiles.collections.IMapper.<K,V>} that is typed for {@link Object} keys and values.
     * <p>
     * This class is used by the {@link yfiles.graphml.GraphMLParser} to store properties of an item before it has been created. An
     * instance of this class can be found in the {@link yfiles.graph.ILookup#lookup} of the {@link yfiles.graphml.IParseContext} during the
     * parsing of an item. Custom parsers may populate the dictionary with various properties that can then be used by the {@link yfiles.graphml.IGraphElementFactory}
     * during the creation of the item.
     * </p>
     * @see yfiles.graphml.IGraphElementFactory
     * @see yfiles.graphml.GraphMLParser
     * @class
     * @implements {yfiles.collections.IMapper.<Object,Object>}
     */
    export interface CreationProperties extends Object,yfiles.collections.IMapper<Object,Object>{}
    export class CreationProperties {
      constructor();
      /**
       * Clears all entries from this property store.
       */
      clear():void;
      /**
       * Gets the value associated with the specified key.
       * @param key The key.
       * @returns The value stored for the key.
       * @see yfiles.graphml.CreationProperties#set
       */
      get(key:Object):Object;
      /**
       * Removes the associated entry for the given key
       * @param key The entry key to remove.
       */
      removeValue(key:Object):void;
      /**
       * Sets the value associated with the specified key.
       * @param key The key.
       * @param value The value stored for the key.
       * @see yfiles.graphml.CreationProperties#get
       */
      set(key:Object,value:Object):void;
      /**
       * Creation property key that tags group node information.
       * <p>
       * Stores values of type {@link boolean}.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory#createNode
       * @const
       * @static
       * @type {string}
       */
      static IS_GROUP_NODE:string;
      /**
       * Creation property key that tags label information.
       * <p>
       * Stores values of type {@link yfiles.collections.IEnumerable.<T>} with {@link yfiles.graph.ILabel} template instances.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory
       * @const
       * @static
       * @type {string}
       */
      static LABELS:string;
      /**
       * Creation property key that tags node layout information.
       * <p>
       * Stores values of type {@link yfiles.geometry.Rect}.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory#createNode
       * @const
       * @static
       * @type {string}
       */
      static LAYOUT:string;
      /**
       * Creation property key that tags style information.
       * <p>
       * Stores values of type {@link yfiles.styles.INodeStyle}, {@link yfiles.styles.IEdgeStyle}, and {@link yfiles.styles.IPortStyle} respectively.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory
       * @const
       * @static
       * @type {string}
       */
      static STYLE:string;
      /**
       * Creation property key that tags {@link yfiles.graph.ITagOwner#tag tag} object information.
       * <p>
       * Stores values of type {@link Object}.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory
       * @const
       * @static
       * @type {string}
       */
      static TAG:string;
      /**
       * Creation property key that tags bend information.
       * <p>
       * Stores values of type {@link yfiles.collections.IEnumerable.<T>} with {@link yfiles.graph.IBend} template instances.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory#createEdge
       * @const
       * @static
       * @type {string}
       */
      static BENDS:string;
      /**
       * Creation property key that tags {@link yfiles.graph.IPortLocationModelParameter port location} information.
       * <p>
       * Stores values of type {@link yfiles.graph.IPortLocationModelParameter}.
       * </p>
       * @see yfiles.graphml.IGraphElementFactory#addPort
       * @const
       * @static
       * @type {string}
       */
      static PORT_LOCATION_MODEL_PARAMETER:string;
      /**
       * Gets all the entries stored by this instance.
       * @type {yfiles.collections.IEnumerable.<yfiles.collections.MapEntry.<Object,Object>>}
       */
      entries:yfiles.collections.IEnumerable<yfiles.collections.MapEntry<Object,Object>>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Core parser class for GraphML.
     * <p>
     * This class allows for low-level customization of the parsing process. Usually, it is used by one of
     * {@link yfiles.graphml.GraphMLIOHandler}'s <code>readFrom</code> methods which calls one of the various <code>GraphMLParser.parseFrom</code> method
     * variants. It should rarely be necessary to use this class directly.
     * </p>
     * @see yfiles.graphml.GraphMLIOHandler
     * @class
     */
    export interface GraphMLParser extends Object{}
    export class GraphMLParser {
      constructor();
      /**
       * Create the initial {@link yfiles.graphml.IParseContext} instance.
       * @param graph The graph that is parsed.
       * @returns An {@link } instance that is suitable for <code>graph</code>.
       * @protected
       */
      createParseContext(graph:yfiles.graph.IGraph):yfiles.graphml.IParseContext;
      /**
       * Deserialize the content of <code>element</code>.
       * <p>
       * Typically, this method is called only indirectly by calls to {@link yfiles.graphml.IParseContext#deserializeCore}. This
       * implementation calls {@link yfiles.graphml.GraphMLParser#onHandleDeserialization}, which in turn raises the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization}
       * event.
       * </p>
       * Type parameter <code>T</code>: The desired type of the result.
       * @param targetType The desired type of the result.
       * @param context The current parse context.
       * @param element The XML content that should be deserialized.
       * @returns An instance of <code>T</code> that is represented by <code>element</code>.
       * @see yfiles.graphml.IParseContext#deserializeCore
       * @see yfiles.graphml.GraphMLParser#addHandleDeserializationListener
       * @template T
       */
      deserialize<T>(targetType:yfiles.lang.Class,context:yfiles.graphml.IParseContext,element:Node):T;
      /**
       * Retrieve a deserialization property that has been set by {@link yfiles.graphml.GraphMLParser#setDeserializationProperty}.
       * @param key The key for the property.
       * @returns 
       * @see yfiles.graphml.IParseContext#getDeserializationProperty
       */
      getDeserializationProperty(key:string):Object;
      /**
       * Dynamically retrieve all {@link yfiles.graphml.IInputHandler} instances that process a GraphML attribute with key definition
       * <code>keyDefinition</code>.
       * <p>
       * Typically, this method is called only indirectly by the parse process. This implementation calls
       * {@link yfiles.graphml.GraphMLParser#onQueryInputHandlers}, which in turn raises the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers}
       * event.
       * </p>
       * @param context The current parse context.
       * @param keyDefinition The key definition element of the GraphML attribute.
       * @returns A list of {@link  IInputHandlers} that claim to be responsible for handling the given GraphML attribute.
       */
      getInputHandlers(context:yfiles.graphml.IParseContext,keyDefinition:Element):yfiles.collections.IList<yfiles.graphml.IInputHandler>;
      /**
       * Dynamically retrieve an instance of <code>type</code>.
       * @param type The type for which an implementation is needed.
       * @returns An implementation of <code>type</code>, or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Fires the {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization} event.
       * @param evt The arguments that get passed to the {@link #addHandleDeserializationListener HandleDeserialization}
       * event.
       * @see yfiles.graphml.GraphMLIOHandler#onHandleDeserialization
       * @protected
       */
      onHandleDeserialization(evt:yfiles.graphml.HandleDeserializationEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLParser#addParsedListener Parsed} event.
       * <p>
       * This method is called when the document has been parsed and after {@link yfiles.graphml.IParseEvents#addDocumentParsedListener DocumentParsed}
       * is called.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLParser#addParsedListener
       * @protected
       */
      onParsed(evt:yfiles.graphml.ParseEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLParser#addParsingListener Parsing} event.
       * <p>
       * This method is called just before the parsing starts and before {@link yfiles.graphml.IParseEvents#addDocumentParsingListener DocumentParsing}
       * is called.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLParser#addParsingListener
       * @protected
       */
      onParsing(evt:yfiles.graphml.ParseEventArgs):void;
      /**
       * Fires the {@link yfiles.graphml.GraphMLParser#addQueryInputHandlersListener QueryInputHandlers} event.
       * @param evt The arguments that get passed to the {@link #addQueryInputHandlersListener QueryInputHandlers}
       * event.
       * @see yfiles.graphml.GraphMLIOHandler#onQueryInputHandlers
       * @protected
       */
      onQueryInputHandlers(evt:yfiles.graphml.QueryInputHandlersEventArgs):void;
      /**
       * Parse the XML document <code>document</code> into an <code>graph</code> instance, using <code>elementFactory</code> to create the elements.
       * <p>
       * Typically, this method is called indirectly from {@link yfiles.graphml.GraphMLIOHandler#readFromDocument}.
       * </p>
       * @param document The XML document instance to parse. <b>Note</b>: If you are using Internet Explorer 9 and {@link }s to retrieve the document,
       * you may not use the value from the {@link #responseXML} property. Please parse {@link #responseText} instead
       * and use the result. For further reference see {@link http://msdn.microsoft.com/en-us/library/ie/hh180177.aspx}.
       * @param graph The graph instance that is populated.
       * @param elementFactory The {@link } instance that is used to create the elements.
       * @returns 
       * @see yfiles.graphml.GraphMLIOHandler#readFromDocument
       */
      parseFromDocument(graph:yfiles.graph.IGraph,document:Document,elementFactory:yfiles.graphml.IGraphElementFactory):Promise<yfiles.graph.IGraph>;
      /**
       * Parse the input <code>url</code> into an <code>graph</code> instance, using <code>elementFactory</code> to create the elements.
       * <p>
       * Typically, this method is called indirectly from {@link yfiles.graphml.GraphMLIOHandler#readFromURL}.
       * </p>
       * @param url The URL to the input to parse.
       * @param graph The graph instance that is populated.
       * @param elementFactory The {@link } instance that is used to create the elements.
       * @returns 
       * @see yfiles.graphml.GraphMLIOHandler#readFromGraphMLText
       */
      parseFromURL(graph:yfiles.graph.IGraph,url:string,elementFactory:yfiles.graphml.IGraphElementFactory):Promise<yfiles.graph.IGraph>;
      /**
       * Remove a deserialization property that has been set by {@link yfiles.graphml.GraphMLParser#setDeserializationProperty}.
       * @param key The key for the property.
       * @see yfiles.graphml.IParseContext#getDeserializationProperty
       */
      removeDeserializationProperty(key:string):void;
      /**
       * Remove an implementation of <code>T</code> that has been set with {@link yfiles.graphml.GraphMLParser#setLookup}.
       * Type parameter <code>T</code>.
       * @param tType
       * @template T
       */
      removeLookup<T>(tType:yfiles.lang.Class):void;
      /**
       * Set a deserialization property that allows to fine tune the parsing process.
       * @param key The key for the property.
       * @param value The property value.
       * @see yfiles.graphml.IParseContext#getDeserializationProperty
       */
      setDeserializationProperty(key:string,value:Object):void;
      /**
       * Register an implementation of <code>T</code> for use with {@link yfiles.graphml.GraphMLParser#lookup}.
       * Type parameter <code>T</code>: The type of <code>instance</code>.
       * @param tType The type of <code>instance</code>.
       * @param instance The implementation of <code>T</code> for lookup retrieval.
       * @template T
       */
      setLookup<T>(tType:yfiles.lang.Class,instance:T):void;
      /**
       * Adds the given listener for the <code>QueryInputHandlers</code> event that occurs when {@link yfiles.graphml.IInputHandler}s are queried.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IInputHandler} instance(s) for a given GraphML
       * attribute key definition.
       * </p>
       * <p>
       * The current key definition is provided by {@link yfiles.graphml.QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link yfiles.graphml.IInputHandler} for this GraphML
       * attribute, you can add the handler with {@link yfiles.graphml.QueryInputHandlersEventArgs#addInputHandler}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener
       * @see yfiles.graphml.GraphMLParser#removeQueryInputHandlersListener
       */
      addQueryInputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryInputHandlers</code> event that occurs when {@link yfiles.graphml.IInputHandler}s are queried.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IInputHandler} instance(s) for a given GraphML
       * attribute key definition.
       * </p>
       * <p>
       * The current key definition is provided by {@link yfiles.graphml.QueryInputHandlersEventArgs#keyDefinition}. If you want to register an {@link yfiles.graphml.IInputHandler} for this GraphML
       * attribute, you can add the handler with {@link yfiles.graphml.QueryInputHandlersEventArgs#addInputHandler}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener
       * @see yfiles.graphml.GraphMLParser#addQueryInputHandlersListener
       */
      removeQueryInputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryInputHandlersEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be deserialized.
       * <p>
       * Subscribe to this event to provide custom deserialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current XML content is provided by {@link yfiles.graphml.HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link yfiles.graphml.HandleDeserializationEventArgs#result} property.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener
       * @see yfiles.graphml.GraphMLParser#removeHandleDeserializationListener
       */
      addHandleDeserializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HandleDeserialization</code> event that occurs when XML content is about to be
       * deserialized.
       * <p>
       * Subscribe to this event to provide custom deserialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current XML content is provided by {@link yfiles.graphml.HandleDeserializationEventArgs#xmlNode}. If you want to return deserialized content, set the value of the {@link yfiles.graphml.HandleDeserializationEventArgs#result} property.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener
       * @see yfiles.graphml.GraphMLParser#addHandleDeserializationListener
       */
      removeHandleDeserializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLParser#onParsed
       * @see yfiles.graphml.GraphMLParser#removeParsedListener
       */
      addParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Parsed</code> event that occurs when the the document has been parsed.
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLParser#onParsed
       * @see yfiles.graphml.GraphMLParser#addParsedListener
       */
      removeParsedListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Parsing</code> event that occurs when the parsing of the document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.ParseEventArgs#context}'s {@link yfiles.graphml.IParseContext#parseEvents} and the configure the parsing process
       * on a per parse case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLParser#onParsing
       * @see yfiles.graphml.GraphMLParser#removeParsingListener
       */
      addParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Parsing</code> event that occurs when the parsing of the document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.ParseEventArgs#context}'s {@link yfiles.graphml.IParseContext#parseEvents} and the configure the parsing process
       * on a per parse case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLParser#onParsing
       * @see yfiles.graphml.GraphMLParser#addParsingListener
       */
      removeParsingListener(listener:(sender:Object,evt:yfiles.graphml.ParseEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * The event arguments used by {@link yfiles.graphml.GraphMLIOHandler#addHandleDeserializationListener HandleDeserialization}
     * and {@link yfiles.graphml.GraphMLParser#addHandleDeserializationListener HandleDeserialization} to let registered
     * deserialization code perform the deserialization.
     * <p>
     * If the event handler determines that it can deserialize the {@link yfiles.graphml.HandleDeserializationEventArgs#xmlNode}, it should place the result into the {@link yfiles.graphml.HandleDeserializationEventArgs#result} property and thus
     * mark the event as {@link yfiles.graphml.HandleDeserializationEventArgs#handled}.
     * </p>
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @final
     */
    export interface HandleDeserializationEventArgs extends yfiles.lang.EventArgs{}
    export class HandleDeserializationEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.HandleDeserializationEventArgs} class.
       * @param context The context in which serialization should take place.
       * @param xmlNode The XML element that contains the data to deserialize.
       * @param targetType The required {@link #targetType target type} of the {@link #result}.
       */
      constructor(context:yfiles.graphml.IParseContext,xmlNode:Node,targetType:yfiles.lang.Class);
      /**
       * Gets the type of the resulting instance that is required by the requesting deserializer or <code>null</code> if the target type is
       * not known in advance.
       * @see yfiles.graphml.IParseContext#deserializeCore
       * @type {yfiles.lang.Class}
       */
      targetType:yfiles.lang.Class;
      /**
       * Holds the result of the deserialization, which is <code>null</code> initially.
       * <p>
       * Setting this property will automatically set the {@link yfiles.graphml.HandleDeserializationEventArgs#handled} property to <code>true</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} If the value is not assignable to {@link yfiles.graphml.HandleDeserializationEventArgs#targetType}
       * @type {Object}
       */
      result:Object;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.graphml.HandleDeserializationEventArgs} is handled.
       * <p>
       * Setting this property to <code>true</code> indicates to the event source that the {@link yfiles.graphml.HandleDeserializationEventArgs#result} has been assigned and that the event should not be
       * propagated to further listeners.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * Gets the context in which the {@link yfiles.graphml.HandleDeserializationEventArgs#xmlNode} shall be deserialized.
       * @type {yfiles.graphml.IParseContext}
       */
      context:yfiles.graphml.IParseContext;
      /**
       * Gets the XML node that contains the data to deserialize.
       * @type {Node}
       */
      xmlNode:Node;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is the interface for a factory that is used for creating the constituents of an {@link yfiles.graph.IGraph} that are
     * encountered during the parsing of {@link yfiles.graphml.GraphMLParser GraphML}
     * @see yfiles.graphml.GraphMLIOHandler#createGraphElementFactory
     * @interface
     */
    export interface IGraphElementFactory extends Object{
      /**
       * Adds a new port to the given {@link yfiles.graph.IPortOwner}.
       * @param context The context from which the graph, the {@link }, and the {@link #objectStack} can be queried.
       * @param owner The owner of the port.
       * @returns A newly created port.
       * @abstract
       */
      addPort(context:yfiles.graphml.IParseContext,owner:yfiles.graph.IPortOwner):yfiles.graph.IPort;
      /**
       * Creates an edge for the given context and the provided source and target nodes or ports.
       * @param context The context from which the graph, the {@link }, and the {@link #objectStack} can be queried.
       * @param sourceNode The source node for the edge.
       * @param sourcePort The source port for the edge, if specified, otherwise <code>null</code>.
       * @param targetNode The target node for the edge.
       * @param targetPort The target port for the edge, if specified, otherwise <code>null</code>.
       * @returns A newly created edge.
       * @abstract
       */
      createEdge(context:yfiles.graphml.IParseContext,sourceNode:yfiles.graph.INode,sourcePort:yfiles.graph.IPort,targetNode:yfiles.graph.INode,targetPort:yfiles.graph.IPort):yfiles.graph.IEdge;
      /**
       * Creates a node for the given context.
       * @param context The context from which the graph, the {@link }, and the {@link #objectStack} can be queried.
       * @returns A newly created node
       * @abstract
       */
      createNode(context:yfiles.graphml.IParseContext):yfiles.graph.INode;
    }
    var IGraphElementFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for classes that can store the GraphML id attribute of GraphML core elements
     * @interface
     */
    export interface IGraphElementIdAcceptor extends Object{
      /**
       * Store the value of the id attribute for the given edge.
       * @param context The current parse context.
       * @param edge The current edge element
       * @param id The id of the edge's XML representation
       * @abstract
       */
      storeEdgeId(context:yfiles.graphml.IParseContext,edge:yfiles.graph.IEdge,id:string):void;
      /**
       * Store the value of the id attribute for the given graph.
       * @param context The current parse context.
       * @param graph The current graph element
       * @param id The id of the graph's XML representation
       * @abstract
       */
      storeGraphId(context:yfiles.graphml.IParseContext,graph:yfiles.graph.IGraph,id:string):void;
      /**
       * Store the value of the id attribute for the given node.
       * @param context The current parse context.
       * @param node The current node element
       * @param id The id of the node's XML representation
       * @abstract
       */
      storeNodeId(context:yfiles.graphml.IParseContext,node:yfiles.graph.INode,id:string):void;
      /**
       * Store the value of the id attribute for the given port.
       * @param context The current parse context.
       * @param port The current port element
       * @param id The id of the port's XML representation
       * @abstract
       */
      storePortId(context:yfiles.graphml.IParseContext,port:yfiles.graph.IPort,id:string):void;
    }
    var IGraphElementIdAcceptor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Simple implementation of {@link yfiles.graphml.IGraphElementIdAcceptor} and {@link yfiles.graphml.IGraphElementIdProvider} that stores
     * all ids in {@link yfiles.collections.IMapper.<K,V>} instances.
     * @class
     * @implements {yfiles.graphml.IGraphElementIdAcceptor}
     * @implements {yfiles.graphml.IGraphElementResolver}
     */
    export interface GraphElementIdAcceptor extends Object,yfiles.graphml.IGraphElementIdAcceptor,yfiles.graphml.IGraphElementResolver{}
    export class GraphElementIdAcceptor {
      constructor();
      /**
       * Factory method to create a default implementation of {@link yfiles.graphml.IGraphElementIdProvider} that delegates to <code>fallback</code>
       * and uses the values stored by this {@link yfiles.graphml.GraphElementIdAcceptor} instance, if possible.
       * @param fallback The fallback.
       * @returns 
       */
      createIdProvider(fallback:yfiles.graphml.IGraphElementIdProvider):yfiles.graphml.IGraphElementIdProvider;
      /**
       * 
       * @param context
       * @param id
       * @returns 
       */
      resolveEdge(context:yfiles.graphml.IParseContext,id:string):yfiles.graph.IEdge;
      /**
       * 
       * @param context
       * @param id
       * @returns 
       */
      resolveGraph(context:yfiles.graphml.IParseContext,id:string):yfiles.graph.IGraph;
      /**
       * 
       * @param context
       * @param id
       * @returns 
       */
      resolveNode(context:yfiles.graphml.IParseContext,id:string):yfiles.graph.INode;
      /**
       * 
       * @param context
       * @param ownerId
       * @param id
       * @returns 
       */
      resolvePort(context:yfiles.graphml.IParseContext,ownerId:string,id:string):yfiles.graph.IPort;
      /**
       * 
       * @param context
       * @param edge
       * @param id
       */
      storeEdgeId(context:yfiles.graphml.IParseContext,edge:yfiles.graph.IEdge,id:string):void;
      /**
       * 
       * @param context
       * @param graph
       * @param id
       */
      storeGraphId(context:yfiles.graphml.IParseContext,graph:yfiles.graph.IGraph,id:string):void;
      /**
       * 
       * @param context
       * @param node
       * @param id
       */
      storeNodeId(context:yfiles.graphml.IParseContext,node:yfiles.graph.INode,id:string):void;
      /**
       * 
       * @param context
       * @param port
       * @param id
       */
      storePortId(context:yfiles.graphml.IParseContext,port:yfiles.graph.IPort,id:string):void;
      /**
       * Return a mapping between all GraphML ids for <code>&lt;graph></code> elements and the corresponding {@link yfiles.graph.IGraph} instances
       * @type {yfiles.collections.IMap.<yfiles.graph.IGraph,string>}
       */
      graphIds:yfiles.collections.IMap<yfiles.graph.IGraph,string>;
      /**
       * Return a mapping between all GraphML ids for <code>&lt;node></code> elements and the corresponding {@link yfiles.graph.INode} instances
       * @type {yfiles.collections.IMap.<yfiles.graph.INode,string>}
       */
      nodeIds:yfiles.collections.IMap<yfiles.graph.INode,string>;
      /**
       * Return a mapping between all GraphML ids for <code>&lt;edge></code> elements and the corresponding {@link yfiles.graph.IEdge} instances
       * @type {yfiles.collections.IMap.<yfiles.graph.IEdge,string>}
       */
      edgeIds:yfiles.collections.IMap<yfiles.graph.IEdge,string>;
      /**
       * Return a mapping between all GraphML ids for <code>&lt;port></code> elements and the corresponding {@link yfiles.graph.IPort} instances
       * @type {yfiles.collections.IMap.<yfiles.graph.IPort,string>}
       */
      portIds:yfiles.collections.IMap<yfiles.graph.IPort,string>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for classes can resolve graph elements based on their GraphML id attribute(s).
     * @interface
     */
    export interface IGraphElementResolver extends Object{
      /**
       * Resolve the GraphML <code>id</code> to an {@link yfiles.graph.IEdge} instance.
       * @param context The current parse context.
       * @param id The id that should be resolved.
       * @returns The {@link } that is defined by this GraphML <code>id</code>, or <code>null</code> if no such edge exists yet.
       * @abstract
       */
      resolveEdge(context:yfiles.graphml.IParseContext,id:string):yfiles.graph.IEdge;
      /**
       * Resolve the GraphML <code>id</code> to an {@link yfiles.graph.IGraph} instance.
       * @param context The current parse context.
       * @param id The id that should be resolved.
       * @returns The {@link } that is defined by this GraphML <code>id</code>, or <code>null</code> if no such graph exists yet.
       * @abstract
       */
      resolveGraph(context:yfiles.graphml.IParseContext,id:string):yfiles.graph.IGraph;
      /**
       * Resolve the GraphML <code>id</code> to an {@link yfiles.graph.INode} instance.
       * @param context The current parse context.
       * @param id The id that should be resolved.
       * @returns The {@link } that is defined by this GraphML <code>id</code>, or <code>null</code> if no such node exists yet.
       * @abstract
       */
      resolveNode(context:yfiles.graphml.IParseContext,id:string):yfiles.graph.INode;
      /**
       * Resolve the GraphML <code>id</code> to an {@link yfiles.graph.IPort} instance.
       * <p>
       * The port owner is specified by the <code>ownerId</code> attribute.
       * </p>
       * @param context The current parse context.
       * @param ownerId The GraphML id of the port owner.
       * @param id The id that should be resolved.
       * @returns The {@link } that is defined by the GraphML ids, or <code>null</code> if no such port exists yet.
       * @abstract
       */
      resolvePort(context:yfiles.graphml.IParseContext,ownerId:string,id:string):yfiles.graph.IPort;
    }
    var IGraphElementResolver:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class defines a handler for key/data elements.
     * @interface
     */
    export interface IInputHandler extends Object{
      /**
       * This method is invoked when no data tag is defined, and the default value should be applied.
       * @param context the current parse context.
       * @abstract
       */
      applyDefault(context:yfiles.graphml.IParseContext):void;
      /**
       * This method is invoked each time a data element with matching key is processed.
       * @param context the current parse context.
       * @param node the DOM node representing the data element.
       * @abstract
       */
      parseData(context:yfiles.graphml.IParseContext,node:Node):void;
      /**
       * Gets the parse precedence that is associated with this instance.
       * @abstract
       * @type {yfiles.graphml.ParsePrecedence}
       */
      precedence:yfiles.graphml.ParsePrecedence;
    }
    var IInputHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum ParsePrecedence{
      /**
       * The data is parsed as early as possible.
       */
      FIRST,
      /**
       * The data is parsed before its owner GraphML element.
       */
      BEFORE_OWNER,
      /**
       * The data is parsed after its owner GraphML element, but before any non-data child content.
       */
      BEFORE_CHILDREN,
      /**
       * Default parse priority.
       */
      DEFAULT,
      /**
       * The data is parsed after any non-data child content, but before parsing the owner GraphML element is finished.
       */
      AFTER_CHILDREN,
      /**
       * The data is parsed directly after parsing the owner GraphML element is finished.
       */
      AFTER_OWNER,
      /**
       * The data is parsed as late as possible.
       */
      LAST
    }
    /**
     * An abstract convenience implementation of the {@link yfiles.graphml.IInputHandler} interface.
     * Type parameter <code>TKey</code>: The type of the key, i.e. the items that are associated with the data in the GraphML.
     * Type parameter <code>TData</code>: The type of the data that is associated with the keys.
     * @class
     * @implements {yfiles.graphml.IInputHandler}
     * @template TKey,TData
     */
    export interface InputHandlerBase<TKey,TData> extends Object,yfiles.graphml.IInputHandler{}
    export class InputHandlerBase<TKey,TData> {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>} class.
       * <p>
       * The {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#precedence} gets the initial value assigned to <code>precedence</code>
       * </p>
       * @param keyType The type of the key, i.e. the items that are associated with the data in the GraphML.
       * @param dataType The type of the data that is associated with the keys.
       * @param precedence The precedence to assign to the {@link #precedence} property.
       * @protected
       */
      constructor(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,precedence:yfiles.graphml.ParsePrecedence);
      /**
       * Initializes a new instance of the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>} class.
       * <p>
       * The {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#precedence} has an initial value of {@link yfiles.graphml.ParsePrecedence#DEFAULT}.
       * </p>
       * @param keyType The type of the key, i.e. the items that are associated with the data in the GraphML.
       * @param dataType The type of the data that is associated with the keys.
       * @protected
       */
      constructor(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class);
      /**
       * This method is invoked when no data tag is defined, and the default value should be applied.
       * <p>
       * This implementation delegates to {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#setValue} if {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#defaultExists a default exists} using the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#defaultValue}.
       * </p>
       * @param context the current parse context.
       */
      applyDefault(context:yfiles.graphml.IParseContext):void;
      /**
       * Initializes this instance from the GraphML key definition.
       * <p>
       * This implementation looks for the GraphML default element and delegates to {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#parseDataCore} if a default is found to store
       * the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#defaultValue}.
       * </p>
       * @param context The context.
       * @param definition The definition.
       */
      initializeFromKeyDefinition(context:yfiles.graphml.IParseContext,definition:Element):void;
      /**
       * This method is invoked each time a data element with matching key is processed.
       * <p>
       * If the {@link yfiles.graphml.IParseContext#getCurrent current element} is of the same type as this <code>TKey</code>, this method delegates the actual parsing to {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#parseDataCore} and
       * {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#setValue}
       * </p>
       * @param context the current parse context.
       * @param node the DOM node representing the data element.
       */
      parseData(context:yfiles.graphml.IParseContext,node:Node):void;
      /**
       * Abstract method callback that actually parses the data.
       * @param context The context.
       * @param node The node that contains the data. This is either the GraphML 'data' element or the 'default' element.
       * @returns The parsed data.
       * @protected
       * @abstract
       */
      parseDataCore(context:yfiles.graphml.IParseContext,node:Node):TData;
      /**
       * Abstract method callback that actually stores the value with the keys.
       * @param context The context.
       * @param key The key.
       * @param data The data.
       * @protected
       * @abstract
       */
      setValue(context:yfiles.graphml.IParseContext,key:TKey,data:TData):void;
      /**
       * Gets or sets a value indicating whether the default value in the key definition should be parsed.
       * @type {boolean}
       */
      parseDefaultValue:boolean;
      /**
       * Gets or sets the default value that will be applied to the keys where the data attribute is missing.
       * <p>
       * If any default value is set (including <code>null</code>), {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#defaultExists} is automatically set to true.
       * </p>
       * @type {TData}
       */
      defaultValue:TData;
      /**
       * Gets a value indicating whether a default value has been set, either through a key definition element or explicitly by
       * assigning a value to the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#defaultValue} property.
       * <p>
       * If this property is set to <code>true</code>, the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#applyDefault} method will call the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#setValue} method using the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#defaultValue} as the value
       * parameter.
       * </p>
       * @type {boolean}
       */
      defaultExists:boolean;
      /**
       * Gets or sets the parse precedence that is associated with this handler.
       * @type {yfiles.graphml.ParsePrecedence}
       */
      precedence:yfiles.graphml.ParsePrecedence;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.graphml.IInputHandler} implementation that uses an {@link yfiles.collections.IMapper.<K,V>} instance to associate the values with the keys.
     * <p>
     * This implementation can be provided an
     * {@link yfiles.graphml.HandleDeserializationEventArgs}-handler to perform the deserialization. If no such value is provided then
     * the default deserialization is performed.
     * </p>
     * <p>
     * The parsed values are stored in a {@link yfiles.collections.IMapper.<K,V>}.
     * </p>
     * Type parameter <code>TKey</code>: The type of the key.
     * Type parameter <code>TData</code>: The type of the value.
     * @class
     * @extends {yfiles.graphml.InputHandlerBase.<TKey,TData>}
     * @template TKey,TData
     */
    export interface MapperInputHandler<TKey,TData> extends yfiles.graphml.InputHandlerBase<TKey,TData>{}
    export class MapperInputHandler<TKey,TData> {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.MapperInputHandler.<TKey,TData>} class that uses the event handler for deserialization and suggests the provided
       * target type for the deserializer.
       * @param keyType The type of the key.
       * @param dataType The type of the value.
       * @param deserializer
       * @param [deserializerTargetType=null]
       * @see yfiles.graphml.MapperInputHandler.<TKey,TData>#deserializerTargetType
       * @see yfiles.graphml.MapperInputHandler.<TKey,TData>#deserializer
       */
      constructor(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,deserializer:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void,deserializerTargetType?:yfiles.lang.Class);
      /**
       * Initializes a new instance of the {@link yfiles.graphml.MapperInputHandler.<TKey,TData>} class that uses the {@link yfiles.graphml.IParseContext} for deserialization.
       * <p>
       * The {@link yfiles.graphml.IInputHandler#precedence} has an initial value of {@link yfiles.graphml.ParsePrecedence#DEFAULT}.
       * </p>
       * @param keyType The type of the key.
       * @param dataType The type of the value.
       */
      constructor(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class);
      /**
       * Gets the node to pass to {@link yfiles.graphml.MapperInputHandler.<TKey,TData>#deserializer} or {@link yfiles.graphml.IParseContext#deserializeCore IParseContext.Deserialize} respectively, given the 'data' or 'default' element from the
       * GraphML.
       * <p>
       * This method is called by {@link yfiles.graphml.MapperInputHandler.<TKey,TData>#parseDataCore} to determine the node to pass to the deserialization mechanism.
       * </p>
       * <p>
       * This implementation returns <code>node</code> iff {@link yfiles.graphml.MapperInputHandler.<TKey,TData>#useParentElementForDeserialization} is <code>true</code> (i.e. the <code>'data'</code> or <code>'default'</code> node itself), otherwise the first child of
       * the <code>node</code> that is not a comment or processing instruction.
       * </p>
       * @param context The context within which the node is parsed.
       * @param node The 'data' or 'default' node.
       * @returns The node that should be deserialized.
       * @see yfiles.graphml.MapperInputHandler.<TKey,TData>#useParentElementForDeserialization
       * @protected
       */
      getDeserializationXmlNode(context:yfiles.graphml.IParseContext,node:Node):Node;
      /**
       * Gets or sets the mapper to use for storing the data.
       * @type {yfiles.collections.IMapper.<TKey,TData>}
       */
      mapper:yfiles.collections.IMapper<TKey,TData>;
      /**
       * Gets or sets a value indicating whether values that are applied via the {@link yfiles.graphml.InputHandlerBase.<TKey,TData>#applyDefault} mechanism and are equal to the
       * value that is already present in the mapper will explicitly be overridden in the mapper.
       * @type {boolean}
       */
      overrideDefaultValue:boolean;
      /**
       * Controls the behavior of {@link yfiles.graphml.MapperInputHandler.<TKey,TData>#getDeserializationXmlNode} to return either the <code>'data'</code>/<code>'default'</code> element itself or its
       * content.
       * <p>
       * The default value is <code>false</code>, meaning that {@link yfiles.graphml.MapperInputHandler.<TKey,TData>#getDeserializationXmlNode} should return the content of the <code>'data'</code>/<code>'default'</code>
       * node.
       * </p>
       * @see yfiles.graphml.MapperInputHandler.<TKey,TData>#getDeserializationXmlNode
       * @type {boolean}
       */
      useParentElementForDeserialization:boolean;
      /**
       * Gets or sets the deserializer.
       * @type {function(Object, yfiles.graphml.HandleDeserializationEventArgs): void}
       */
      deserializer:(sender:Object,evt:yfiles.graphml.HandleDeserializationEventArgs)=>void;
      /**
       * Gets or sets the target deserialization type for the {@link yfiles.graphml.HandleDeserializationEventArgs#targetType} property.
       * @type {yfiles.lang.Class}
       */
      deserializerTargetType:yfiles.lang.Class;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specialized {@link yfiles.lang.EventArgs} that are used for dynamic {@link yfiles.graphml.IInputHandler} registration.
     * @see yfiles.graphml.GraphMLIOHandler#addQueryInputHandlersListener
     * @see yfiles.graphml.GraphMLParser#addQueryInputHandlersListener
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @final
     */
    export interface QueryInputHandlersEventArgs extends yfiles.lang.EventArgs{}
    export class QueryInputHandlersEventArgs {
      /**
       * Create a new instance.
       * @param context The current parse context.
       * @param keyDefinition The current key definition
       * @param handlers Stores the registered {@link } instances.
       */
      constructor(context:yfiles.graphml.IParseContext,keyDefinition:Element,handlers:yfiles.collections.IList<yfiles.graphml.IInputHandler>);
      /**
       * Register an {@link yfiles.graphml.IInputHandler} instance for the current {@link yfiles.graphml.QueryInputHandlersEventArgs#keyDefinition}.
       * <p>
       * Calling this method automatically sets {@link yfiles.graphml.QueryInputHandlersEventArgs#handled} to <code>true</code>.
       * </p>
       * @param handler
       */
      addInputHandler(handler:yfiles.graphml.IInputHandler):void;
      /**
       * The key definition of the GraphML attribute.
       * <p>
       * This XML element should be queried to determine if an {@link yfiles.graphml.IInputHandler} should be registered with
       * {@link yfiles.graphml.QueryInputHandlersEventArgs#addInputHandler}
       * </p>
       * @type {Element}
       */
      keyDefinition:Element;
      /**
       * Set or query whether the current GraphML attribute has been processed.
       * <p>
       * This is automatically set by calling
       * {@link yfiles.graphml.QueryInputHandlersEventArgs#addInputHandler}, so it should be seldom necessary to set this property
       * manually.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * The {@link yfiles.graphml.IParseContext} instance that can queried for further information.
       * @type {yfiles.graphml.IParseContext}
       */
      context:yfiles.graphml.IParseContext;
      /**
       * The {@link yfiles.graphml.IInputHandler}s that have been registered on this event argument instance so far.
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.IInputHandler>}
       */
      handlers:yfiles.collections.IEnumerable<yfiles.graphml.IInputHandler>;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.graphml.IWriteContext} implementation for use in the context of {@link yfiles.graphml.GraphMLWriter} that wraps a given
     * instance and delegates most of the calls to it but allows for modifying {@link yfiles.graphml.ChildWriteContext#setLookup lookup} and {@link yfiles.graphml.ChildWriteContext#setSerializationProperty serialization properties}.
     * <p>
     * Whenever it is necessary to locally modify some context properties, a new instance of this class should be created that
     * wraps an existing one.
     * </p>
     * @class
     * @implements {yfiles.graphml.IWriteContext}
     */
    export interface ChildWriteContext extends Object,yfiles.graphml.IWriteContext{}
    export class ChildWriteContext {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.ChildWriteContext} class that delegates to the provided context and
       * returns the given value in {@link yfiles.graphml.ChildWriteContext#getCurrent}.
       * @param parentContext The parent context to delegate calls to.
       * @param currentObject The current object to return in {@link #getCurrent}.
       */
      constructor(parentContext:yfiles.graphml.IWriteContext,currentObject:Object);
      /**
       * Initializes a new instance of the {@link yfiles.graphml.ChildWriteContext} class that delegates to the given parent context.
       * @param parentContext The parent context.
       */
      constructor(parentContext:yfiles.graphml.IWriteContext);
      /**
       * 
       * Type parameter <code>T</code>.
       * @param itemType
       * @returns 
       * @template T
       */
      getCurrent<T>(itemType:yfiles.lang.Class):T;
      /**
       * 
       * @param key
       * @returns 
       */
      getSerializationProperty(key:string):Object;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Removes a local lookup override that has been added using {@link yfiles.graphml.ChildWriteContext#setLookup}
       * Type parameter <code>T</code>: The type key to remove.
       * @param tType The type key to remove.
       * @template T
       */
      removeLookup<T>(tType:yfiles.lang.Class):void;
      /**
       * Removes a previously locally {@link yfiles.graphml.ChildWriteContext#setSerializationProperty set} serialization property.
       * @param key The key for which the local property override should be removed.
       */
      removeSerializationProperty(key:string):void;
      /**
       * 
       * Type parameter <code>T</code>.
       * @param targetType
       * @param context
       * @param item
       * @template T
       */
      serializeCore<T>(targetType:yfiles.lang.Class,context:yfiles.graphml.IWriteContext,item:T):void;
      /**
       * Adds another lookup result to the local lookup override.
       * Type parameter <code>T</code>: The type to use as a key for the lookup.
       * @param tType The type to use as a key for the lookup.
       * @param value The value to provide for queries to <code>T</code>.
       * @see yfiles.graphml.ChildWriteContext#removeLookup
       * @template T
       */
      setLookup<T>(tType:yfiles.lang.Class,value:T):void;
      /**
       * Sets the local serialization property for the given key to the value.
       * @param key The key to override.
       * @param value The value.
       */
      setSerializationProperty(key:string,value:Object):void;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<Object>}
       */
      objectStack:yfiles.collections.IListEnumerable<Object>;
      /**
       * 
       * @type {yfiles.graphml.IWriteEvents}
       */
      writeEvents:yfiles.graphml.IWriteEvents;
      /**
       * Gets the parent context to which most calls are delegated.
       * @type {yfiles.graphml.IWriteContext}
       */
      parentContext:yfiles.graphml.IWriteContext;
      /**
       * Gets or sets the writer.
       * <p>
       * If a <code>null</code> writer is set, this implementation will delegate to the {@link yfiles.graphml.ChildWriteContext#parentContext}'s {@link yfiles.graphml.IWriteContext#writer}.
       * </p>
       * @type {yfiles.graphml.IXmlWriter}
       */
      writer:yfiles.graphml.IXmlWriter;
      /**
       * 
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * Core writer class for GraphML.
     * <p>
     * This class allows for low-level customization of the writing process. Usually, it is used by
     * {@link yfiles.graphml.GraphMLIOHandler}'s <code>write</code> method which calls
     * <code>GraphMLWriter.write</code>. It should rarely be necessary to use this class directly.
     * </p>
     * @see yfiles.graphml.GraphMLIOHandler
     * @class
     */
    export interface GraphMLWriter extends Object{}
    export class GraphMLWriter {
      constructor();
      /**
       * Create the initial {@link yfiles.graphml.IWriteContext} instance.
       * @param graph The graph that is written.
       * @param writer The core {@link } instance that does the actual writing of XML content.
       * @returns An {@link } instance that is suitable for <code>graph</code>.
       * @protected
       */
      createWriteContext(graph:yfiles.graph.IGraph,writer:yfiles.graphml.IXmlWriter):yfiles.graphml.IWriteContext;
      /**
       * Dynamically retrieve all {@link yfiles.graphml.IOutputHandler} that should be used for the current write process.
       * <p>
       * Typically, this method is called only indirectly by the write process. This implementation calls
       * {@link yfiles.graphml.GraphMLWriter#onQueryOutputHandlers}, which in turn raises the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers}
       * event.
       * </p>
       * @param context The current write context.
       * @param scope The scope to get output handlers for.
       * @returns A dictionary from handler definition ids to {@link  IOutputHandlers}.
       */
      getOutputHandlers(context:yfiles.graphml.IWriteContext,scope:yfiles.graphml.KeyScope):yfiles.collections.IEnumerable<yfiles.graphml.IOutputHandler>;
      /**
       * Retrieve a serialization property that has been set by {@link yfiles.graphml.GraphMLWriter#setSerializationProperty}.
       * @param key The key for the property.
       * @returns 
       * @see yfiles.graphml.IWriteContext#getSerializationProperty
       */
      getSerializationProperty(key:string):Object;
      /**
       * Dynamically retrieve an instance of <code>type</code>.
       * @param type The type for which an implementation is needed.
       * @returns An implementation of <code>type</code>, or <code>null</code>.
       * @protected
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Fires the {@link yfiles.graphml.GraphMLWriter#addHandleSerializationListener HandleSerialization} event.
       * @param evt The arguments that get passed to the {@link #addHandleSerializationListener HandleSerialization}
       * event.
       * @see yfiles.graphml.GraphMLIOHandler#onHandleSerialization
       * @protected
       */
      onHandleSerialization(evt:yfiles.graphml.HandleSerializationEventArgs):void;
      /**
       * Fires the {@link yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener QueryOutputHandlers} event.
       * @param evt The arguments that get passed to the {@link #addQueryOutputHandlersListener QueryOutputHandlers}
       * event.
       * @see yfiles.graphml.GraphMLIOHandler#onQueryOutputHandlers
       * @protected
       */
      onQueryOutputHandlers(evt:yfiles.graphml.QueryOutputHandlersEventArgs):void;
      /**
       * Fires the {@link yfiles.graphml.GraphMLWriter#addQueryReferenceIdListener QueryReferenceId} event.
       * @param evt The arguments that get passed to the {@link #addQueryReferenceIdListener QueryReferenceId}
       * event.
       * @see yfiles.graphml.GraphMLIOHandler#onQueryReferenceId
       * @protected
       */
      onQueryReferenceId(evt:yfiles.graphml.QueryReferenceIdEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLWriter#addWritingListener Writing} event.
       * <p>
       * This method is called just before the writing starts and before {@link yfiles.graphml.IWriteEvents#addDocumentWritingListener DocumentWriting}
       * is called for the first time.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLWriter#addWritingListener
       * @protected
       */
      onWriting(evt:yfiles.graphml.WriteEventArgs):void;
      /**
       * Raises the {@link yfiles.graphml.GraphMLWriter#addWrittenListener Written} event.
       * <p>
       * This method is called just before the writing has been finished and after {@link yfiles.graphml.IWriteEvents#addDocumentWrittenListener DocumentWritten}
       * is called for the last time.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.graphml.GraphMLWriter#addWrittenListener
       * @protected
       */
      onWritten(evt:yfiles.graphml.WriteEventArgs):void;
      /**
       * Remove an implementation of <code>T</code> that has been set with {@link yfiles.graphml.GraphMLWriter#setLookup}.
       * Type parameter <code>T</code>.
       * @param tType
       * @template T
       */
      removeLookup<T>(tType:yfiles.lang.Class):void;
      /**
       * Remove a serialization property that has been set by {@link yfiles.graphml.GraphMLWriter#setSerializationProperty}.
       * @param key The key for the property.
       * @see yfiles.graphml.IWriteContext#getSerializationProperty
       */
      removeSerializationProperty(key:string):void;
      /**
       * Serialize <code>item</code>.
       * <p>
       * Typically, this method is called only indirectly by calls to {@link yfiles.graphml.IWriteContext#serializeCore}. This
       * implementation calls {@link yfiles.graphml.GraphMLWriter#onHandleSerialization}, which in turn raises the {@link yfiles.graphml.GraphMLWriter#addHandleSerializationListener HandleSerialization}
       * event.
       * </p>
       * Type parameter <code>T</code>: The desired type of the result.
       * @param targetType The desired type of the result.
       * @param context The current parse context.
       * @param item The object that should be serialized.
       * @throws {yfiles.graphml.SerializationNotSupportedError} <code>SerializationNotSupportedException</code> if <code>item</code> cannot be serialized.
       * @see yfiles.graphml.IWriteContext#serializeCore
       * @see yfiles.graphml.GraphMLWriter#addHandleSerializationListener
       * @protected
       * @template T
       */
      serialize<T>(targetType:yfiles.lang.Class,context:yfiles.graphml.IWriteContext,item:T):void;
      /**
       * Register an implementation of <code>T</code> for use with {@link yfiles.graphml.GraphMLWriter#lookup}.
       * Type parameter <code>T</code>: The type of <code>instance</code>.
       * @param tType The type of <code>instance</code>.
       * @param instance The implementation of <code>T</code> for lookup retrieval.
       * @template T
       */
      setLookup<T>(tType:yfiles.lang.Class,instance:T):void;
      /**
       * Set a serialization property that allows to fine tune the write process.
       * @param key The key for the property.
       * @param value The property value.
       * @see yfiles.graphml.IWriteContext#getSerializationProperty
       */
      setSerializationProperty(key:string,value:Object):void;
      /**
       * Write the GraphML representation of <code>graph</code>, using <code>writer</code> for the actual XML generation.
       * <p>
       * Typically, this method is called indirectly from {@link yfiles.graphml.GraphMLIOHandler#write}.
       * </p>
       * @param graph The graph instance that is populated.
       * @param writer The writer that actually writes the XML content.
       * @see yfiles.graphml.GraphMLIOHandler#write
       */
      write(graph:yfiles.graph.IGraph,writer:yfiles.graphml.IXmlWriter):void;
      /**
       * Adds the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
       * <p>
       * Subscribe to this event to provide custom serialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current item is provided by {@link yfiles.graphml.HandleSerializationEventArgs#item}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener
       * @see yfiles.graphml.GraphMLWriter#removeHandleSerializationListener
       */
      addHandleSerializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HandleSerialization</code> event that occurs when XML content is about to be serialized.
       * <p>
       * Subscribe to this event to provide custom serialization handling for XML content.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * The current item is provided by {@link yfiles.graphml.HandleSerializationEventArgs#item}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener
       * @see yfiles.graphml.GraphMLWriter#addHandleSerializationListener
       */
      removeHandleSerializationListener(listener:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>QueryOutputHandlers</code> event that occurs when {@link yfiles.graphml.IOutputHandler}s are queried.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IOutputHandler} instance(s) for a given
       * GraphML attribute key definition.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * If you want to register an {@link yfiles.graphml.IOutputHandler} for this GraphML attribute, you can add the handler with
       * {@link yfiles.graphml.QueryOutputHandlersEventArgs#addOutputHandler}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener
       * @see yfiles.graphml.GraphMLWriter#removeQueryOutputHandlersListener
       */
      addQueryOutputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryOutputHandlers</code> event that occurs when
       * {@link yfiles.graphml.IOutputHandler}s are queried.
       * <p>
       * Subscribe to this event to dynamically register one or more {@link yfiles.graphml.IOutputHandler} instance(s) for a given
       * GraphML attribute key definition.
       * </p>
       * <p>
       * The event handlers added to this event are called in inverse order, i.e. handlers that have been added later are called
       * earlier.
       * </p>
       * <p>
       * If you want to register an {@link yfiles.graphml.IOutputHandler} for this GraphML attribute, you can add the handler with
       * {@link yfiles.graphml.QueryOutputHandlersEventArgs#addOutputHandler}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener
       * @see yfiles.graphml.GraphMLWriter#addQueryOutputHandlersListener
       */
      removeQueryOutputHandlersListener(listener:(sender:Object,evt:yfiles.graphml.QueryOutputHandlersEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
       * <p>
       * This event can be used to provide names of external references for objects.
       * </p>
       * <p>
       * These reference targets are not serialized to the GraphML file.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener
       * @see yfiles.graphml.GraphMLWriter#removeQueryReferenceIdListener
       */
      addQueryReferenceIdListener(listener:(sender:Object,evt:yfiles.graphml.QueryReferenceIdEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryReferenceId</code> event that occurs when a reference is queried.
       * <p>
       * This event can be used to provide names of external references for objects.
       * </p>
       * <p>
       * These reference targets are not serialized to the GraphML file.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener
       * @see yfiles.graphml.GraphMLWriter#addQueryReferenceIdListener
       */
      removeQueryReferenceIdListener(listener:(sender:Object,evt:yfiles.graphml.QueryReferenceIdEventArgs)=>void):void;
      /**
       * This property allows to access the events in the internal {@link yfiles.graphml.IWriteEvents} implementation.
       * <p>
       * You can subscribe to any of the events in the returned {@link yfiles.graphml.IWriteEvents} instance to react to various steps in
       * the writing process
       * </p>
       * @type {yfiles.graphml.IWriteEvents}
       */
      writeEvents:yfiles.graphml.IWriteEvents;
      /**
       * Adds the given listener for the <code>Written</code> event that occurs when the writing of a document has just been finished.
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLWriter#onWritten
       * @see yfiles.graphml.GraphMLWriter#removeWrittenListener
       */
      addWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Written</code> event that occurs when the writing of a document has just been finished.
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLWriter#onWritten
       * @see yfiles.graphml.GraphMLWriter#addWrittenListener
       */
      removeWrittenListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.WriteEventArgs#context}'s {@link yfiles.graphml.IWriteContext#writeEvents} and to configure the writing process
       * on a per write case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graphml.GraphMLWriter#onWriting
       * @see yfiles.graphml.GraphMLWriter#removeWritingListener
       */
      addWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Writing</code> event that occurs when the writing of a document is just about to begin.
       * <p>
       * This event can be used to register to the fine-grained events available in the {@link yfiles.graphml.WriteEventArgs#context}'s {@link yfiles.graphml.IWriteContext#writeEvents} and to configure the writing process
       * on a per write case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graphml.GraphMLWriter#onWriting
       * @see yfiles.graphml.GraphMLWriter#addWritingListener
       */
      removeWritingListener(listener:(sender:Object,evt:yfiles.graphml.WriteEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * The event arguments used by {@link yfiles.graphml.GraphMLIOHandler#addHandleSerializationListener HandleSerialization}
     * and {@link yfiles.graphml.GraphMLWriter#addHandleSerializationListener HandleSerialization} to let registered
     * serialization code perform the serialization of items.
     * <p>
     * If the event handler determines that it can serialize the {@link yfiles.graphml.HandleSerializationEventArgs#item}, it should use the {@link yfiles.graphml.HandleSerializationEventArgs#writer} for output and mark the event as {@link yfiles.graphml.HandleSerializationEventArgs#handled} so that
     * other code will not perform serialization, too.
     * </p>
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @final
     */
    export interface HandleSerializationEventArgs extends yfiles.lang.EventArgs{}
    export class HandleSerializationEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.HandleSerializationEventArgs} class using the provided context to write
       * the <code>item</code> using the given type for serialization.
       * @param context The context to use for writing.
       * @param item The item to to serialize.
       * @param [serializationType=null] The type that should be used for serialization, which needs to be a super type or interface of the <code>item</code> or <code>null</code>.
       */
      constructor(context:yfiles.graphml.IWriteContext,item:Object,serializationType?:yfiles.lang.Class);
      /**
       * Replaces the {@link yfiles.graphml.HandleSerializationEventArgs#item} using a substitute for serialization.
       * <p>
       * This method can be used to replace the {@link yfiles.graphml.HandleSerializationEventArgs#item} and {@link yfiles.graphml.HandleSerializationEventArgs#sourceType} property so that the serialization chain can be traversed with a
       * substituted item. If this method has been called the {@link yfiles.graphml.HandleSerializationEventArgs#handled} property should not be set to <code>true</code>.
       * </p>
       * @param item The substitute to assign to the {@link #item} property.
       * @param serializationType The serialization type to assign to the {@link #sourceType} property.
       */
      replaceItem(item:Object,serializationType:yfiles.lang.Class):void;
      /**
       * Gets the item that should be serialized.
       * @type {Object}
       */
      item:Object;
      /**
       * Declares the type in the inheritance of the {@link yfiles.graphml.HandleSerializationEventArgs#item} that should be used for serialization.
       * @type {yfiles.lang.Class}
       */
      sourceType:yfiles.lang.Class;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.graphml.HandleSerializationEventArgs} is handled.
       * <p>
       * Setting this property to <code>true</code> indicates to the event source that the {@link yfiles.graphml.HandleSerializationEventArgs#item} has been successfully serialized using the {@link yfiles.graphml.HandleSerializationEventArgs#writer} and no
       * other code needs to handle the serialization request, anymore.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * Gets the context to be used for writing.
       * @type {yfiles.graphml.IWriteContext}
       */
      context:yfiles.graphml.IWriteContext;
      /**
       * Gets the writer for writing the xml output.
       * @type {yfiles.graphml.IXmlWriter}
       */
      writer:yfiles.graphml.IXmlWriter;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for classes that provide custom id's for core GraphML elements
     * @interface
     */
    export interface IGraphElementIdProvider extends Object{
      /**
       * Get an ID for the specified <code>edge</code> object
       * @param context The current write context, for further information to the provider
       * @param edge The current graph object
       * @returns an ID for the specified edge object
       * @abstract
       */
      getEdgeId(context:yfiles.graphml.IWriteContext,edge:yfiles.graph.IEdge):string;
      /**
       * Get an ID for the specified <code>graph</code> object
       * @param context The current write context, for further information to the provider
       * @param graph The current graph object
       * @returns an ID for the specified graph object
       * @abstract
       */
      getGraphId(context:yfiles.graphml.IWriteContext,graph:yfiles.graph.IGraph):string;
      /**
       * Get an ID for the specified <code>node</code> object
       * @param context The current write context, for further information to the provider
       * @param node The current graph object
       * @returns an ID for the specified graph object
       * @abstract
       */
      getNodeId(context:yfiles.graphml.IWriteContext,node:yfiles.graph.INode):string;
      /**
       * Get an ID for the specified <code>port</code> object
       * @param context The current write context, for further information to the provider
       * @param port The current port object
       * @returns an ID for the specified port object
       * @abstract
       */
      getPortId(context:yfiles.graphml.IWriteContext,port:yfiles.graph.IPort):string;
    }
    var IGraphElementIdProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An implementation of the {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>} class, that uses an {@link yfiles.collections.IMapper.<K,V>} to retrieve the values to output for each item and a {@link yfiles.graphml.MapperOutputHandler.<TKey,TData>#serializer} callback for the
     * serialization or delegates to the {@link yfiles.graphml.IWriteContext#serializeCore} callback.
     * Type parameter <code>TKey</code>: The type of the keys.
     * Type parameter <code>TData</code>: The type of the data.
     * @class
     * @extends {yfiles.graphml.OutputHandlerBase.<TKey,TData>}
     * @template TKey,TData
     */
    export interface MapperOutputHandler<TKey,TData> extends yfiles.graphml.OutputHandlerBase<TKey,TData>{}
    export class MapperOutputHandler<TKey,TData> {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.MapperOutputHandler.<TKey,TData>} class using the provided key scope, the name, and the key type.
       * @param keyType The type of the keys.
       * @param dataType The type of the data.
       * @param scope The scope for the key definition attributes.
       * @param name The name for the key definition attributes.
       * @param type The type for the key definition attributes.
       */
      constructor(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,scope:yfiles.graphml.KeyScope,name:string,type:yfiles.graphml.KeyType);
      /**
       * Gets or sets the mapper metadata to serialize to the key definition.
       * @type {yfiles.graph.MapperMetadata}
       */
      mapperMetadata:yfiles.graph.MapperMetadata;
      /**
       * Gets or sets the mapper to use for querying the data for each item.
       * @type {yfiles.collections.IMapper.<TKey,TData>}
       */
      mapper:yfiles.collections.IMapper<TKey,TData>;
      /**
       * Gets or sets the serialization callback to use for serializing the value in {@link yfiles.graphml.MapperOutputHandler.<TKey,TData>#writeValueCore}.
       * @see yfiles.graphml.MapperOutputHandler.<TKey,TData>#sourceType
       * @type {function(Object, yfiles.graphml.HandleSerializationEventArgs): void}
       */
      serializer:(sender:Object,evt:yfiles.graphml.HandleSerializationEventArgs)=>void;
      /**
       * Gets or sets the source type of the serialization.
       * <p>
       * This type will be used as the {@link yfiles.graphml.HandleSerializationEventArgs#sourceType}.
       * </p>
       * @see yfiles.graphml.MapperOutputHandler.<TKey,TData>#serializer
       * @see yfiles.graphml.HandleSerializationEventArgs
       * @type {yfiles.lang.Class}
       */
      sourceType:yfiles.lang.Class;
      static $class:yfiles.lang.Class;
    }
    /**
     * Convenience abstract base implementation of the {@link yfiles.graphml.IOutputHandler} interface.
     * Type parameter <code>TKey</code>: The type of the keys.
     * Type parameter <code>TData</code>: The type of the data.
     * @class
     * @implements {yfiles.graphml.IOutputHandler}
     * @template TKey,TData
     */
    export interface OutputHandlerBase<TKey,TData> extends Object,yfiles.graphml.IOutputHandler{}
    export class OutputHandlerBase<TKey,TData> {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>} class using the provided scope, name, and type for the key definition.
       * @param keyType The type of the keys.
       * @param dataType The type of the data.
       * @param scope The scope.
       * @param name The name to use for the key definition.
       * @param type The type to use for the key definition.
       * @see yfiles.graphml.OutputHandlerBase.<TKey,TData>#setKeyDefinitionUri
       * @protected
       */
      constructor(keyType:yfiles.lang.Class,dataType:yfiles.lang.Class,scope:yfiles.graphml.KeyScope,name:string,type:yfiles.graphml.KeyType);
      /**
       * Callback method that obtains the data for the given key.
       * @param context The context.
       * @param key The key.
       * @returns The data that is associated with the key.
       * @protected
       * @abstract
       */
      getValue(context:yfiles.graphml.IWriteContext,key:TKey):TData;
      /**
       * Determines whether in the current context, the value is the default value and therefore no data element needs to be
       * written.
       * <p>
       * This implementation will return <code>false</code> if no {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#defaultExists default has been set}. Otherwise it will use {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#getValue} and compare the result
       * with the {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#defaultValue} to determine whether they are equal and therefore no data needs to be written.
       * </p>
       * @param context The context.
       * @returns <code>true</code> if for the current context the default value applies and therefore no data element needs to be written.
       */
      isDefaultValue(context:yfiles.graphml.IWriteContext):boolean;
      /**
       * Sets the name for the key definition.
       * @param name The name of the key element.
       */
      setKeyDefinitionName(name:string):void;
      /**
       * Sets the scope for the key definition.
       * <p>
       * Note that the official GraphML schema does not support the {@link yfiles.graphml.KeyScope#GRAPHML} so this implementation will
       * silently ignore this which will result in {@link yfiles.graphml.KeyScope#ALL} implicitly being defined.
       * </p>
       * @param scope The scope of the key element.
       */
      setKeyDefinitionScope(scope:yfiles.graphml.KeyScope):void;
      /**
       * Sets the key type for the key definition.
       * @param type The type of the key element.
       */
      setKeyDefinitionType(type:yfiles.graphml.KeyType):void;
      /**
       * Sets an optional URI for the key definition.
       * <p>
       * By default, no URI is set.
       * </p>
       * @param uri The URI of the key element.
       */
      setKeyDefinitionUri(uri:string):void;
      /**
       * Writes the contents of the key definition.
       * <p>
       * Depending on whether {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#writeKeyDefault} is enabled and {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#defaultExists} is <code>true</code>, this method will write the GraphML 'default' element and use {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#writeValueCore}
       * to serialize the {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#defaultValue}.
       * </p>
       * @param context The context.
       */
      writeKeyDefinitionContent(context:yfiles.graphml.IWriteContext):void;
      /**
       * Actually writes the value for the current context.
       * <p>
       * This method will use {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#getValue} to obtain the data and delegate to {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#writeValueCore} to perform the actual serialization.
       * </p>
       * @param context The context.
       */
      writeValue(context:yfiles.graphml.IWriteContext):void;
      /**
       * Callback method that performs the actual writing of the data.
       * @param context The context.
       * @param data The data.
       * @protected
       * @abstract
       */
      writeValueCore(context:yfiles.graphml.IWriteContext,data:TData):void;
      /**
       * Gets or sets the default value.
       * <p>
       * Setting this value also sets the {@link yfiles.graphml.OutputHandlerBase.<TKey,TData>#defaultExists} property.
       * </p>
       * @type {TData}
       */
      defaultValue:TData;
      /**
       * Gets or sets a value indicating whether a default value has been assigned.
       * @type {boolean}
       */
      defaultExists:boolean;
      /**
       * Gets or sets the write precedence for the handler.
       * @type {yfiles.graphml.WritePrecedence}
       */
      precedence:yfiles.graphml.WritePrecedence;
      /**
       * 
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.GraphMLXmlAttribute>}
       */
      keyDefinitionAttributes:yfiles.collections.IEnumerable<yfiles.graphml.GraphMLXmlAttribute>;
      /**
       * 
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.GraphMLXmlAttribute>}
       */
      dataTagAttributes:yfiles.collections.IEnumerable<yfiles.graphml.GraphMLXmlAttribute>;
      /**
       * Gets or sets a value indicating whether to write the key default.
       * @default true
       * @type {boolean}
       */
      writeKeyDefault:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event argument implementation that is used by the {@link yfiles.graphml.GraphMLIOHandler#addQueryOutputHandlersListener QueryOutputHandlers}
     * event.
     * <p>
     * These event arguments can be used to add more {@link yfiles.graphml.IOutputHandler} implementations to the {@link yfiles.graphml.GraphMLWriter}
     * dynamically at the time of writing, by adding them to the {@link yfiles.graphml.QueryOutputHandlersEventArgs#handlers} collection.
     * </p>
     * @see yfiles.graphml.GraphMLIOHandler
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @final
     */
    export interface QueryOutputHandlersEventArgs extends yfiles.lang.EventArgs{}
    export class QueryOutputHandlersEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.QueryOutputHandlersEventArgs} class.
       * @param context The context to assign to the {@link #context} property.
       * @param scope The scope where the handlers should be added.
       */
      constructor(context:yfiles.graphml.IWriteContext,scope:yfiles.graphml.KeyScope);
      /**
       * Adds the given output handler to the set of registered output handlers for the given scope.
       * @param handler The handler to add.
       */
      addOutputHandler(handler:yfiles.graphml.IOutputHandler):void;
      /**
       * The scope where the handlers should be added.
       * @type {yfiles.graphml.KeyScope}
       */
      scope:yfiles.graphml.KeyScope;
      /**
       * The {@link yfiles.graphml.IOutputHandler}s that have been registered on this event argument instance so far.
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.IOutputHandler>}
       */
      handlers:yfiles.collections.IEnumerable<yfiles.graphml.IOutputHandler>;
      /**
       * Gets the context for which the handlers are queried.
       * @type {yfiles.graphml.IWriteContext}
       */
      context:yfiles.graphml.IWriteContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default implementation of {@link yfiles.graphml.IXmlWriter}.
     * <p>
     * This writer does not support random access on the output document.
     * </p>
     * @class
     * @implements {yfiles.graphml.IXmlWriter}
     */
    export interface XmlWriter extends Object,yfiles.graphml.IXmlWriter{}
    export class XmlWriter {
      /**
       * Create a new instance with enabled {@link yfiles.graphml.XmlWriter#indent}.
       */
      constructor();
      /**
       * 
       * @param addEndTag
       * @protected
       */
      closeCurrentElement(addEndTag:boolean):void;
      /**
       * Factory method that creates the namespace manager for this instance.
       * @returns A new manager instance.
       * @protected
       */
      createNamespaceManager():yfiles.graphml.IXmlNamespaceManager;
      /**
       * 
       */
      flushDocument():void;
      /**
       * 
       * @returns 
       */
      getXmlString():string;
      /**
       * 
       * @param prefix
       * @param localName
       * @param ns
       * @param value
       * @returns 
       */
      writeAttributeNS(prefix:string,localName:string,ns:string,value:string):yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param content
       * @returns 
       */
      writeCData(content:string):yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param comment
       * @returns 
       */
      writeComment(comment:string):yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param fragment
       * @returns 
       */
      writeDocumentFragment(fragment:Document):yfiles.graphml.IXmlWriter;
      /**
       * 
       */
      writeEndDocument():void;
      /**
       * 
       * @returns 
       */
      writeEndElement():yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param target
       * @param data
       * @returns 
       */
      writeProcessingInstruction(target:string,data:string):yfiles.graphml.IXmlWriter;
      /**
       * 
       * @returns 
       */
      writeStartDocument():yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param prefix
       * @param localName
       * @param ns
       * @returns 
       */
      writeStartElement(prefix:string,localName:string,ns:string):yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param localName
       * @param ns
       * @returns 
       */
      writeStartElement(localName:string,ns:string):yfiles.graphml.IXmlWriter;
      /**
       * 
       * @param s
       * @returns 
       */
      writeString(s:string):yfiles.graphml.IXmlWriter;
      /**
       * This property controls whether the output should be properly indented.
       * @type {boolean}
       */
      indent:boolean;
      /**
       * Gets the namespace manager associated with this writer.
       * @type {yfiles.graphml.IXmlNamespaceManager}
       */
      namespaceManager:yfiles.graphml.IXmlNamespaceManager;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface that allows a mark and identify shared resources.
     * @see yfiles.graphml.IReferenceHandler
     * @interface
     */
    export interface IReferenceHandle extends Object{
      /**
       * The type of a resource (external or internal)
       * @abstract
       * @type {yfiles.graphml.GraphMLReferenceType}
       */
      referenceType:yfiles.graphml.GraphMLReferenceType;
      /**
       * The resource that is encapsulated by the handle.
       * @abstract
       * @type {Object}
       */
      subject:Object;
      /**
       * The id that identifies the resource in the GraphML document.
       * <p>
       * Note that this value is ignored for internal references.
       * </p>
       * @abstract
       * @type {string}
       */
      id:string;
    }
    var IReferenceHandle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum SharingState{
      /**
       * The registered object shall not be shared.
       */
      UNSHARED,
      /**
       * The object has been registered the first time.
       */
      FIRST_SEEN,
      /**
       * The object has been registered multiple times and will be shared.
       */
      DUPLICATE
    }
    /**
     * This interface defines methods to write and configure shared sections in the resulting GraphML output
     * <p>
     * This interface provides infrastructure for both external and internal (i.e. stored in the GraphML document) references.
     * </p>
     * <p>
     * Usually, it is not necessary to use this interface directly from client code.
     * </p>
     * @interface
     */
    export interface IReferenceHandler extends Object{
      /**
       * Gets the reference handle for a given subject.
       * @param subject The subject.
       * @param context The current write context.
       * @returns A handle for the resource <code>subject</code>, or <code>null</code> if the resource is not registered.
       * @see yfiles.graphml.IReferenceHandle
       * @see yfiles.graphml.IReferenceHandle#subject
       * @abstract
       */
      getHandle(context:yfiles.graphml.IWriteContext,subject:Object):yfiles.graphml.IReferenceHandle;
      /**
       * Unregister a shared resource.
       * <p>
       * This allows to explicitly mark a resource as unshared even if it is referenced multiple times in the object graph. If a
       * resource has already been registered with {@link yfiles.graphml.IReferenceHandler#registerResource}, it will be unmarked. In
       * addition, subsequent calls to {@link yfiles.graphml.IReferenceHandler#registerResource} for the same reference won't mark the
       * resource as shared again.
       * </p>
       * @param context The current write context.
       * @param subject The resource to register.
       * @see yfiles.graphml.IReferenceHandler#registerResource
       * @see yfiles.graphml.GraphMLAttribute#shareable
       * @abstract
       */
      markAsUnshared(context:yfiles.graphml.IWriteContext,subject:Object):void;
      /**
       * Register a shared resource.
       * @param context The current write context.
       * @param subject The resource to register.
       * @returns <code>true</code> iff the resource needs to be serialized; otherwise the resource is either a duplicate resource or is declared
       * externally and does not need any additional handling.
       * @abstract
       */
      registerResource(context:yfiles.graphml.IWriteContext,subject:Object):yfiles.graphml.SharingState;
      /**
       * Return a collection of all registered reference handles that should be written to the GraphML shared data section.
       * @see yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener
       * @see yfiles.graphml.GraphMLWriter#addQueryReferenceIdListener
       * @abstract
       * @type {yfiles.collections.IEnumerable.<yfiles.graphml.IReferenceHandle>}
       */
      internalHandles:yfiles.collections.IEnumerable<yfiles.graphml.IReferenceHandle>;
    }
    var IReferenceHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum GraphMLReferenceType{
      /**
       * Specifies an internal resource, that is the serialized form of the resource is stored in the GraphML document itself.
       */
      INTERNAL,
      /**
       * Specifies an external resource, that is the resource description is not stored in the GraphML document itself.
       */
      EXTERNAL
    }
    /**
     * Event arguments for calls to the {@link yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener ResolveReference}
     * and {@link yfiles.graphml.GraphMLIOHandler#addOverrideResolveReferenceListener OverrideResolveReference} events.
     * @see yfiles.graphml.GraphMLIOHandler#addResolveReferenceListener
     * @see yfiles.graphml.GraphMLIOHandler#addOverrideResolveReferenceListener
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface ResolveReferenceEventArgs extends yfiles.lang.EventArgs{}
    export class ResolveReferenceEventArgs {
      /**
       * Creates a new instance of {@link yfiles.graphml.ResolveReferenceEventArgs}.
       * <p>
       * The {@link yfiles.graphml.ResolveReferenceEventArgs#handled} property is set to <code>false</code>, the {@link yfiles.graphml.ResolveReferenceEventArgs#value} property is set to <code>null</code>
       * </p>
       * @param context The current parse context that can be used by the event handler for additional information.
       * @param referenceId The reference id that should evaluated by the event handler.
       */
      constructor(context:yfiles.graphml.IParseContext,referenceId:string);
      /**
       * The reference id that should evaluated by the event handler.
       * @type {string}
       */
      referenceId:string;
      /**
       * The reference value that is encapsulated by this instance.
       * <p>
       * Setting this property automatically sets {@link yfiles.graphml.ResolveReferenceEventArgs#handled} to <code>true</code>.
       * </p>
       * @see yfiles.graphml.ResolveReferenceEventArgs#handled
       * @type {Object}
       */
      value:Object;
      /**
       * Returns whether the last event handler invocation has actually handled the event.
       * <p>
       * This property is set implicitly by {@link yfiles.graphml.ResolveReferenceEventArgs#value}.
       * </p>
       * @see yfiles.graphml.ResolveReferenceEventArgs#value
       * @type {boolean}
       */
      handled:boolean;
      /**
       * The current parse context that can be used by the event handler for additional information.
       * @type {yfiles.graphml.IParseContext}
       */
      context:yfiles.graphml.IParseContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for calls to the {@link yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener QueryReferenceId}
     * or {@link yfiles.graphml.GraphMLWriter#addQueryReferenceIdListener QueryReferenceId} events.
     * <p>
     * Event handling code should inspect the {@link yfiles.graphml.QueryReferenceIdEventArgs#value} property and decide whether to set the {@link yfiles.graphml.QueryReferenceIdEventArgs#referenceId} and/or {@link yfiles.graphml.QueryReferenceIdEventArgs#referenceType} property in order to {@link yfiles.graphml.QueryReferenceIdEventArgs#handled handle} the
     * event.
     * </p>
     * @see yfiles.graphml.GraphMLIOHandler#addQueryReferenceIdListener
     * @see yfiles.graphml.GraphMLWriter#addQueryReferenceIdListener
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface QueryReferenceIdEventArgs extends yfiles.lang.EventArgs{}
    export class QueryReferenceIdEventArgs {
      /**
       * Creates a new instance.
       * <p>
       * The {@link yfiles.graphml.QueryReferenceIdEventArgs#handled} property is set to <code>false</code>, the {@link yfiles.graphml.QueryReferenceIdEventArgs#referenceId} is set to <code>null</code>.
       * </p>
       * @param context The current write context that can be used by the event handler for additional information.
       * @param value The referenced object instance.
       */
      constructor(context:yfiles.graphml.IWriteContext,value:Object);
      /**
       * The reference id that should be evaluated by the event handler.
       * <p>
       * Setting this property to a non-null string automatically sets {@link yfiles.graphml.QueryReferenceIdEventArgs#handled} to <code>true</code>.
       * </p>
       * @see yfiles.graphml.QueryReferenceIdEventArgs#handled
       * @type {string}
       */
      referenceId:string;
      /**
       * Determines the type of the reference being queried.
       * <p>
       * Setting this property automatically sets {@link yfiles.graphml.QueryReferenceIdEventArgs#handled} to <code>true</code>. By default this property is {@link yfiles.graphml.GraphMLReferenceType#EXTERNAL}
       * so {@link yfiles.graphml.QueryReferenceIdEventArgs#value}s that are given a {@link yfiles.graphml.QueryReferenceIdEventArgs#referenceId} are treated as external references that are not serialized to the GraphML. Setting this property
       * to {@link yfiles.graphml.GraphMLReferenceType#INTERNAL} and specifying a {@link yfiles.graphml.QueryReferenceIdEventArgs#referenceId} will make the reference mechanism use the provided
       * reference but will cause it to serialize the value to the GraphML, nevertheless.
       * </p>
       * @see yfiles.graphml.QueryReferenceIdEventArgs#referenceId
       * @see yfiles.graphml.QueryReferenceIdEventArgs#handled
       * @default yfiles.graphml.GraphMLReferenceType.EXTERNAL
       * @type {yfiles.graphml.GraphMLReferenceType}
       */
      referenceType:yfiles.graphml.GraphMLReferenceType;
      /**
       * The reference value that is encapsulated by this instance.
       * @type {Object}
       */
      value:Object;
      /**
       * Returns whether the last event handler invocation has actually handled the event.
       * <p>
       * This property is set implicitly by {@link yfiles.graphml.QueryReferenceIdEventArgs#referenceId}.
       * </p>
       * @see yfiles.graphml.QueryReferenceIdEventArgs#referenceId
       * @type {boolean}
       */
      handled:boolean;
      /**
       * The current write context that can be used by the event handler for additional information.
       * @type {yfiles.graphml.IWriteContext}
       */
      context:yfiles.graphml.IWriteContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * Special implementation of {@link yfiles.graphml.IValueSerializerContext} that should be used in a GraphML parse context.
     * @class
     * @implements {yfiles.graphml.IValueSerializerContext}
     */
    export interface GraphMLParseValueSerializerContext extends Object,yfiles.graphml.IValueSerializerContext{}
    export class GraphMLParseValueSerializerContext {
      /**
       * Creates a new instance that wraps <code>context</code>
       * @param context The current write context
       */
      constructor(context:yfiles.graphml.IParseContext);
      /**
       * 
       * @param property
       * @returns 
       */
      getValueSerializerFor(property:yfiles.graphml.Property):yfiles.graphml.ValueSerializer;
      /**
       * 
       * @param type
       * @returns 
       */
      getValueSerializerFor(type:yfiles.lang.Class):yfiles.graphml.ValueSerializer;
      /**
       * Returns the wrapped {@link yfiles.graphml.IParseContext} for this serviceType, otherwise <code>null</code>
       * @param serviceType The serviceType
       * @returns the wrapped {@link } for this serviceType, otherwise <code>null</code>
       */
      lookup(serviceType:yfiles.lang.Class):Object;
      /**
       * The parent object if such an object exists currently.
       * @type {Object}
       */
      owner:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * Special implementation of {@link yfiles.graphml.IValueSerializerContext} that should be used in a GraphML write context.
     * @class
     * @implements {yfiles.graphml.IValueSerializerContext}
     */
    export interface GraphMLWriteValueSerializerContext extends Object,yfiles.graphml.IValueSerializerContext{}
    export class GraphMLWriteValueSerializerContext {
      /**
       * Creates a new instance that wraps <code>context</code>
       * @param context The current write context
       */
      constructor(context:yfiles.graphml.IWriteContext);
      /**
       * 
       * @param property
       * @returns 
       */
      getValueSerializerFor(property:yfiles.graphml.Property):yfiles.graphml.ValueSerializer;
      /**
       * 
       * @param type
       * @returns 
       */
      getValueSerializerFor(type:yfiles.lang.Class):yfiles.graphml.ValueSerializer;
      /**
       * Returns the wrapped {@link yfiles.graphml.IWriteContext} for this serviceType, otherwise <code>null</code>
       * @param serviceType The serviceType
       * @returns the wrapped {@link } for this serviceType, otherwise <code>null</code>
       */
      lookup(serviceType:yfiles.lang.Class):Object;
      /**
       * The parent object if such an object exists currently.
       * @type {Object}
       */
      owner:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * The event arguments used by {@link yfiles.graphml.GraphMLIOHandler#addQueryNameListener QueryName} to
     * override automatic name determination in GraphML.
     * <p>
     * If the event handler determines that it map resolve {@link yfiles.graphml.QueryNameEventArgs#type} to a xml name, it should place the result into the {@link yfiles.graphml.QueryNameEventArgs#result} property and
     * thus mark the event as {@link yfiles.graphml.QueryNameEventArgs#handled}.
     * </p>
     * <p>
     * It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
     * determination rules are used for that part.
     * </p>
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface QueryNameEventArgs extends yfiles.lang.EventArgs{}
    export class QueryNameEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.QueryNameEventArgs} class.
       * @param context The context in which the lookup shall take place.
       * @param type The type that should be mapped.
       */
      constructor(context:yfiles.graphml.IWriteContext,type:yfiles.lang.Class);
      /**
       * Holds the result of the name query, which is <code>null</code> initially.
       * <p>
       * Setting this property will automatically set the {@link yfiles.graphml.QueryNameEventArgs#handled} property to <code>true</code>.
       * </p>
       * <p>
       * It is valid to provide only part of the XML name by setting the other part to null. In that case, the default name
       * determination rules are used for that part.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} If the value is not assignable to TargetType
       * @type {yfiles.graphml.XmlName}
       */
      result:yfiles.graphml.XmlName;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.graphml.QueryNameEventArgs} is handled.
       * <p>
       * Setting this property to <code>true</code> indicates to the event source that the {@link yfiles.graphml.QueryNameEventArgs#result} has been assigned and that the event should not be
       * propagated to further listeners.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * The type for which the {@link yfiles.graphml.QueryNameEventArgs#result XML name} is queried.
       * @type {yfiles.lang.Class}
       */
      type:yfiles.lang.Class;
      /**
       * Gets the context in which the {@link yfiles.graphml.QueryNameEventArgs#type} shall mapped.
       * @type {yfiles.graphml.IWriteContext}
       */
      context:yfiles.graphml.IWriteContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * The event arguments used by {@link yfiles.graphml.GraphMLIOHandler#addQueryTypeListener QueryType} to
     * override automatic type resolution in GraphML.
     * <p>
     * If the event handler determines that it can resolve {@link yfiles.graphml.QueryTypeEventArgs#xmlName} to a type, it should place the result into the {@link yfiles.graphml.QueryTypeEventArgs#result} property and thus
     * mark the event as {@link yfiles.graphml.QueryTypeEventArgs#handled}.
     * </p>
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface QueryTypeEventArgs extends yfiles.lang.EventArgs{}
    export class QueryTypeEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graphml.QueryTypeEventArgs} class.
       * @param context The context in the lookup should take place.
       * @param xName The type that should be mapped.
       */
      constructor(context:yfiles.graphml.IParseContext,xName:yfiles.graphml.XmlName);
      /**
       * Holds the result of the type query, which is <code>null</code> initially.
       * <p>
       * Setting this property will automatically set the {@link yfiles.graphml.QueryTypeEventArgs#handled} property to <code>true</code>.
       * </p>
       * @type {yfiles.lang.Class}
       */
      result:yfiles.lang.Class;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.graphml.QueryTypeEventArgs} is handled.
       * <p>
       * Setting this property to <code>true</code> indicates to the event source that the {@link yfiles.graphml.QueryTypeEventArgs#result} has been assigned and that the event should not be
       * propagated to further listeners.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * The fully qualified xml name that should be resolved to a type.
       * @type {yfiles.graphml.XmlName}
       */
      xmlName:yfiles.graphml.XmlName;
      /**
       * Gets the context in which the {@link yfiles.lang.Class} shall mapped.
       * @type {yfiles.graphml.IParseContext}
       */
      context:yfiles.graphml.IParseContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation that can read (a subset of) XAML formatted XML content.
     * @class
     * @final
     */
    export interface XamlReader extends Object{}
    export class XamlReader {
      /**
       * Deserializes the specified element node in the context to an instance.
       * @param context The context.
       * @param element The XML element node.
       * @returns The instance that is described in the XML.
       */
      deserialize(context:yfiles.graphml.IParseContext,element:Element):Object;
      /**
       * A singleton instance of this class.
       * @const
       * @static
       * @type {yfiles.graphml.XamlReader}
       */
      static INSTANCE:yfiles.graphml.XamlReader;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for mapping XAML namespaces to XML namespace prefixes and vice versa.
     * <p>
     * This interface is only used to specify mappings between namespaces that are used for serialization or deserialization
     * with classes {@link yfiles.graphml.XamlSerializer} and
     * {@link yfiles.graphml.XamlReader}. If no prefix mapping is specified for a namespace, a default XML namespace/prefix mapping is
     * used.
     * </p>
     * @interface
     */
    export interface IXamlPrefixMapper extends Object{
      /**
       * Gets the namespace for the given prefix.
       * @param prefix The prefix.
       * @returns The namespace.
       * @abstract
       */
      getNamespaceOfPrefix(prefix:string):string;
      /**
       * Gets the prefix of namespace.
       * @param ns The namespace.
       * @returns The prefix to use.
       * @abstract
       */
      getPrefixOfNamespace(ns:string):string;
    }
    var IXamlPrefixMapper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Serializer that produces XAML conforming output from the target object's reflection data.
     * <p>
     * By default, the resulting XAML output follows the rules for normal XAML output. In addition, serialization can be fine
     * tuned by applying {@link yfiles.graphml.GraphMLAttribute} to the target object's class or some of its members.
     * </p>
     * <p>
     * Note that usually the type should be excluded from obfuscation, or its name should be declared with
     * {@link yfiles.graphml.GraphMLAttribute#name}. The same holds for specific members.
     * </p>
     * @see yfiles.graphml.GraphMLAttribute
     * @class
     * @final
     */
    export interface XamlSerializer extends Object{}
    export class XamlSerializer {
      /**
       * Serialize the given object to a {@link yfiles.graphml.IXmlWriter}
       * <p>
       * This implementation already provides the toplevel XML element with an appropriate tag name.
       * </p>
       * @param context The current write context
       * @param subject The actual object
       */
      serialize(context:yfiles.graphml.IWriteContext,subject:Object):void;
      /**
       * A singleton instance of this class.
       * @const
       * @static
       * @type {yfiles.graphml.XamlSerializer}
       */
      static INSTANCE:yfiles.graphml.XamlSerializer;
      static $class:yfiles.lang.Class;
    }
  }export namespace view{
    /**
     * Runs {@link yfiles.view.IAnimation animations} in a {@link yfiles.view.CanvasComponent}.
     * <p>
     * For running {@link yfiles.view.IAnimation} instances the method {@link yfiles.view.Animator#animate} can be used. For simpler cases of
     * custom animations there are also overloads that take an animation callback.
     * </p>
     * @see yfiles.view.IAnimation
     * @class
     */
    export interface Animator extends Object{}
    export class Animator {
      /**
       * Initializes a new instance of the {@link yfiles.view.Animator} class for the given {@link yfiles.view.CanvasComponent}.
       * <p>
       * Using this constructor this instance will query an instance of {@link yfiles.input.WaitInputMode} from the {@link yfiles.view.CanvasComponent#lookup canvas's context} to
       * automatically set the {@link yfiles.input.WaitInputMode#waiting} property during animations if {@link yfiles.view.Animator#allowUserInteraction} is set to <code>false</code>. The animation is not started until {@link yfiles.view.Animator#animate}
       * or {@link yfiles.view.Animator#animate} is called.
       * </p>
       * @param canvas The canvas control.
       */
      constructor(canvas:yfiles.view.CanvasComponent);
      /**
       * Initializes a new instance of the {@link yfiles.view.Animator} class for the given {@link yfiles.view.CanvasComponent}.
       * <p>
       * Using this constructor this instance will query an instance of {@link yfiles.input.WaitInputMode} from the {@link yfiles.view.CanvasComponent#lookup canvas's context} to
       * automatically set the {@link yfiles.input.WaitInputMode#waiting} property during animations if {@link yfiles.view.Animator#allowUserInteraction} is set to <code>false</code>. The animation is not started until {@link yfiles.view.Animator#animate}
       * or {@link yfiles.view.Animator#animate} is called.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.canvas The canvas control.
       * @param {boolean} options.allowUserInteraction Determines whether user interaction should be allowed during the animation.
       * <p>
       * This option sets the {@link yfiles.view.Animator#allowUserInteraction} property on the created object.
       * </p>
       * @param {boolean} options.autoInvalidation A value indicating whether to automatically {@link yfiles.view.Animator#invalidateComponent invalidate the control} this instance has been created for.
       * <p>
       * This option sets the {@link yfiles.view.Animator#autoInvalidation} property on the created object.
       * </p>
       */
      constructor(options:{canvas:yfiles.view.CanvasComponent,allowUserInteraction?:boolean,autoInvalidation?:boolean});
      /**
       * Animates the given animation instance and triggers the callback upon completion.
       * @param animation The animation to perform.
       * @returns 
       */
      animate(animation:yfiles.view.IAnimation):Promise<yfiles.view.IAnimation>;
      /**
       * Starts animating the given animation for the specified duration.
       * @param callback The callback to use for the animation.
       * @param duration The duration in milliseconds that the animation should last.
       * @returns 
       */
      animate(callback:(time:number)=>void,duration:yfiles.lang.TimeSpan):Promise<yfiles.view.IAnimation>;
      /**
       * Stops all running animations.
       */
      destroy():void;
      /**
       * Invalidates the control if {@link yfiles.view.Animator#autoInvalidation} is set to <code>true</code>.
       * <p>
       * This implementation calls {@link yfiles.view.CanvasComponent#updateVisual }.
       * </p>
       * @param canvas The control.
       * @protected
       */
      invalidateComponent(canvas:yfiles.view.CanvasComponent):void;
      /**
       * Determines whether user interaction should be allowed during the animation.
       * <p>
       * If <code>false</code>, the {@link yfiles.input.WaitInputMode} is queried from the {@link yfiles.view.CanvasComponent} and {@link yfiles.input.WaitInputMode#waiting} is enabled during the
       * animation. The default is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      allowUserInteraction:boolean;
      /**
       * The control the animations are run on.
       * @type {yfiles.view.CanvasComponent}
       */
      canvasComponent:yfiles.view.CanvasComponent;
      /**
       * Gets or sets a value indicating whether to automatically {@link yfiles.view.Animator#invalidateComponent invalidate the control} this instance has been created for.
       * <p>
       * The default is true.
       * </p>
       * @type {boolean}
       */
      autoInvalidation:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An interface for animations that can be run by {@link yfiles.view.Animator}.
     * <p>
     * Instances of classes implementing this interface change their state according to a relative animation time. The
     * animation time is a double between 0 for the start of the animation and 1 for the end of the animation.
     * </p>
     * <ul>
     * <li>An animated object is first created using the constructor of a subclass.</li>
     * <li>Immediately before the animation a client calls {@link yfiles.view.IAnimation#initialize}.</li>
     * <li>
     * To change the state of the animated object a client does a series of calls to
     * {@link yfiles.view.IAnimation#animate}, usually with an increasing relative time parameter.
     * </li>
     * <li>If the animation is done, the client calls {@link yfiles.view.IAnimation#cleanup} once.</li>
     * <li>
     * While {@link yfiles.view.IAnimation#animate} uses a relative time scheme, there might be an absolute time value for the
     * preferred duration of the whole sequence of animation steps. Since an IAnimation instance has no control on the number
     * of {@link yfiles.view.IAnimation#animate} calls or the intervals between them, the preferred duration is only a hint on how long
     * the animation should take. Clients can ask for that hint using {@link yfiles.view.IAnimation#preferredDuration} and try to find a suitable sequence of {@link yfiles.view.IAnimation#animate}
     * calls in order to fulfill the preference.
     * </li>
     * </ul>
     * @interface
     */
    export interface IAnimation extends Object{
      /**
       * Does the animation according to the relative animation time.
       * <p>
       * The animation starts with the time 0 and ends with time 1.
       * </p>
       * @param time the animation time [0,1]
       * @abstract
       */
      animate(time:number):void;
      /**
       * Cleans up after an animation has finished.
       * @abstract
       */
      cleanup():void;
      /**
       * Creates an eased animation for the given base animation.
       * <p>
       * The animation speed linearly increases upto a <code>easeIn</code> and linearly decreases from
       * <code>easeOut</code>. Between those two points, the animation speed is constant.
       * </p>
       * <p>
       * The duration of the ease in and ease out is determined by a mapping [0,1] → [0,1]. The mapping has to be increasing to
       * preserve the order of the frames in the decorated animation. In general, we consider continuously differentiable
       * functions <i>f</i> where integral of <i>f</i>′ equals 1 a suitable choice for the mapping.
       * </p>
       * @param [easeIn=0.5] The ratio for the ease-in time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
       * @param [easeOut=0.5] The ratio for the ease-out time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
       * @returns 
       */
      createEasedAnimation?(easeIn?:number,easeOut?:number):yfiles.view.IAnimation;
      /**
       * Creates an eased animation for the given base animation.
       * <p>
       * The animation speed linearly increases upto a <code>easeIn</code> and linearly decreases from
       * <code>easeOut</code>. Between those two points, the animation speed is constant.
       * </p>
       * <p>
       * The duration of the ease in and ease out is determined by a mapping [0,1] → [0,1]. The mapping has to be increasing to
       * preserve the order of the frames in the decorated animation. In general, we consider continuously differentiable
       * functions <i>f</i> where integral of <i>f</i>′ equals 1 a suitable choice for the mapping.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.easeIn=0.5] The ratio for the ease-in time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
       * @param [options.easeOut=0.5] The ratio for the ease-out time [0,1] where 0 is the start of the animation and 1 the end. <code>0.5</code> is the default value.
       * @returns 
       */
      createEasedAnimation?(options:{easeIn?:number,easeOut?:number}):yfiles.view.IAnimation;
      /**
       * Initializes the animation.
       * <p>
       * Call this method once before subsequent calls to {@link yfiles.view.IAnimation#animate}.
       * </p>
       * @abstract
       */
      initialize():void;
      /**
       * Gets the preferred duration of the animation.
       * @abstract
       * @type {yfiles.lang.TimeSpan}
       */
      preferredDuration:yfiles.lang.TimeSpan;
    }
    var IAnimation:{
      /**
       * Creates a new {@link yfiles.view.IAnimation} that animates the given <code>edge</code>'s bends from its current shape linearly to the shape
       * given by the <code>endBends</code> and final port locations.
       * <p>
       * The bends of the edge layout are animated from their current location to their target locations. It is assumed that the
       * edge style is a polyline style and the control points are the bends accordingly. The animation works for other styles
       * too, but the effect might not be as expected. The number of bends in <code>endBends</code> can be different to the current number of
       * bends. This will result in the animation automatically removing bends at the end of the animation or inserting new bends
       * accordingly at the beginning of the animation.
       * </p>
       * <p>
       * The source and target points are also animated to new locations. However, the ports will not be moved by this animation.
       * Instead a separate {@link yfiles.view.IAnimation#createPortAnimation animation for the ports} is needed to actually move
       * {@link yfiles.graph.IPort}s at the ends of the edge if desired.
       * </p>
       * @param graph The graph the animated edge belongs to.
       * @param edge The edge layout to animate.
       * @param endBends The bend positions after the animation.
       * @param endSourceLocation The absolute position of the source port after the animation.
       * @param endTargetLocation The absolute position of the target port after the animation.
       * @param preferredDuration The preferred duration of the animation in milliseconds.
       * @returns 
       * @static
       */
      createEdgeSegmentAnimation?(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,endBends:yfiles.geometry.IPoint[],endSourceLocation:yfiles.geometry.Point,endTargetLocation:yfiles.geometry.Point,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates a new {@link yfiles.view.IAnimation} that animates the given layout of all types of graph items.
       * @param graph The graph for which the layout should be animated.
       * @param targetNodeLayouts The node layouts after the animation.
       * @param targetBendLocations The bend locations after the animation, for each edge the points are interpreted as the position of the bends along the
       * edge in the given order.
       * @param targetLayoutParameters The label model parameters for each label after the animation.
       * @param targetPortLocations The {@link }s for each {@link } in the graph that will be morphed.
       * @param preferredDuration The preferred duration of the animation in milliseconds.
       * @returns 
       * @static
       */
      createGraphAnimation?(graph:yfiles.graph.IGraph,targetNodeLayouts:yfiles.collections.IMapper<yfiles.graph.INode,yfiles.geometry.IRectangle>,targetBendLocations:yfiles.collections.IMapper<yfiles.graph.IEdge,yfiles.geometry.IPoint[]>,targetPortLocations:yfiles.collections.IMapper<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>,targetLayoutParameters:yfiles.collections.IMapper<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates a new {@link yfiles.view.IAnimation} that animates the given <code>label</code> from its current layout linearly to the layout given
       * by the <code>targetLayoutParameter</code>.
       * <p>
       * This animation will as a side effect assign the <code>targetLayoutParameter</code> to the label at the {@link Root.IG#cleanup end of the animation}.
       * </p>
       * @param graph The graph that contains the labels.
       * @param label The label to animate the parameter of.
       * @param targetLayoutParameter The target parameter of the label.
       * @param preferredDuration The preferred duration of the animation.
       * @returns A new instance of {@link }.
       * @static
       */
      createLabelAnimation?(graph:yfiles.graph.IGraph,label:yfiles.graph.ILabel,targetLayoutParameter:yfiles.graph.ILabelModelParameter,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates a new {@link yfiles.view.IAnimation} instance that animates the current graph to the layout given by a {@link yfiles.layout.CopiedLayoutGraph}
       * instance.
       * <p>
       * The returned instance is suitable for morphing an {@link yfiles.graph.IGraph} to a layout computed by one of the layout
       * algorithms.
       * </p>
       * <p>
       * While it is usually recommended to use {@link yfiles.layout.LayoutExecutor} to handle layout calculation and subsequent
       * animation, the {@link yfiles.view.IAnimation} produced by this method can be used manually, as well:
       * </p>
       * @param graph the graph for which the layout should be animated
       * @param layoutGraph the {@link } that contains all target layout information
       * @param preferredDuration the preferred duration of the animation
       * @returns 
       * @static
       */
      createLayoutAnimation?(graph:yfiles.graph.IGraph,layoutGraph:yfiles.layout.CopiedLayoutGraph,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates a new {@link yfiles.view.IAnimation} that animates the given <code>node</code> from its current layout to the new given
       * <code>targetLayout</code>.
       * @param graph The graph the animated node belongs to.
       * @param node The node whose {@link #layout} to animate.
       * @param targetLayout The expected node layout after the animation.
       * @param preferredDuration The preferred duration of the animation.
       * @returns A new instance of {@link }.
       * @static
       */
      createNodeAnimation?(graph:yfiles.graph.IGraph,node:yfiles.graph.INode,targetLayout:yfiles.geometry.IRectangle,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates an {@link yfiles.view.IAnimation} according to the composite design pattern that animates multiple animations in
       * parallel.
       * <p>
       * The animations can be synchronized to the longest running animation.
       * </p>
       * @param animations The animations to run in parallel.
       * @param [synchronized=true] Whether all child animations should be adjusted to have the same (maximum) {@link #preferredDuration}. By default this is enabled.
       * @returns A new instance that is the composite of the given animations.
       * @static
       */
      createParallelAnimation?(animations:yfiles.collections.IEnumerable<yfiles.view.IAnimation>,synchronized?:boolean):yfiles.view.IAnimation;
      /**
       * Creates a new instance of an {@link yfiles.view.IAnimation} that animates the given {@link yfiles.geometry.IMutablePoint} along a path.
       * @param path The path to animate the point along.
       * @param animationPoint The mutable point instance that will be manipulated by the animation.
       * @param preferredDuration The preferred duration of the animation.
       * @returns 
       * @static
       */
      createPathAnimation?(path:yfiles.geometry.GeneralPath,animationPoint:yfiles.geometry.IMutablePoint,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates a new {@link yfiles.view.IAnimation} that animates the given <code>port</code> from its current location to the new location given by
       * the <code>targetLocationParameter</code>.
       * <p>
       * This animation will as a side effect assign the <code>targetLocationParameter</code> to the port at the {@link Root.IG#cleanup end of the animation}.
       * </p>
       * @param graph The graph that contains the port.
       * @param port The port to animate.
       * @param targetLocationParameter The parameter to linearly interpolate to.
       * @param preferredDuration The {@link #preferredDuration preferred duration} of this animation.
       * @returns A new instance of {@link }.
       * @static
       */
      createPortAnimation?(graph:yfiles.graph.IGraph,port:yfiles.graph.IPort,targetLocationParameter:yfiles.graph.IPortLocationModelParameter,preferredDuration:yfiles.lang.TimeSpan):yfiles.view.IAnimation;
      /**
       * Creates a new {@link yfiles.view.IAnimation} for the given table that animates the table and its {@link yfiles.graph.IStripe stripes}.
       * <p>
       * Note that only the sizes of
       * <b>leaf stripes</b>, i.e. those without child stripes, are considered. The sizes of a stripe with children is implicitly determined by its
       * contents.
       * </p>
       * <p>
       * For more control of the animation, implement a custom {@link yfiles.view.TableAnimation} and override method
       * {@link yfiles.view.TableAnimation#createStripeAnimation}.
       * </p>
       * @param table The table to animate.
       * @param columnLayout The sizes of the <b>leaf</b> columns, in natural order.
       * @param rowLayout The sizes of the <b>leaf</b> rows, in natural order.
       * @returns 
       * @static
       */
      createTableAnimation?(table:yfiles.graph.ITable,columnLayout:number[],rowLayout:number[]):yfiles.view.IAnimation;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Animates changing the viewport bounds and the zoom factor for a {@link yfiles.view.CanvasComponent}.
     * @class
     * @implements {yfiles.view.IAnimation}
     */
    export interface ViewportAnimation extends Object,yfiles.view.IAnimation{}
    export class ViewportAnimation {
      /**
       * Creates a new instance of {@link yfiles.view.ViewportAnimation}.
       * @param canvas The canvas control.
       * @param targetBounds The target world bounds for the animation.
       * @param [preferredDuration=null] The preferred duration. 1 second if omitted.
       */
      constructor(canvas:yfiles.view.CanvasComponent,targetBounds:yfiles.geometry.Rect,preferredDuration?:yfiles.lang.TimeSpan);
      /**
       * Creates a new instance of {@link yfiles.view.ViewportAnimation}.
       * @param {Object} options The parameters to pass.
       * @param options.canvas The canvas control.
       * @param options.targetBounds The target world bounds for the animation.
       * @param [options.preferredDuration=null] The preferred duration. 1 second if omitted.
       * @param {boolean} options.considerViewportLimiter Whether to respect the {@link yfiles.view.CanvasComponent#viewportLimiter} of the {@link yfiles.view.CanvasComponent} whose {@link yfiles.view.CanvasComponent#viewport} is animated.
       * <p>
       * This option sets the {@link yfiles.view.ViewportAnimation#considerViewportLimiter} property on the created object.
       * </p>
       * @param {number} options.maximumTargetZoom The maximum zoom level to use after the animation.
       * <p>
       * This option sets the {@link yfiles.view.ViewportAnimation#maximumTargetZoom} property on the created object.
       * </p>
       */
      constructor(options:{canvas:yfiles.view.CanvasComponent,targetBounds:yfiles.geometry.Rect,preferredDuration?:yfiles.lang.TimeSpan,considerViewportLimiter?:boolean,maximumTargetZoom?:number});
      /**
       * 
       * @param time
       */
      animate(time:number):void;
      /**
       * Effectively applies the center point value.
       * @param focus
       * @protected
       */
      applyCenterPoint(focus:yfiles.geometry.Point):void;
      /**
       * Effectively applies the view point value.
       * @param focus
       * @protected
       */
      applyViewPoint(focus:yfiles.geometry.Point):void;
      /**
       * Effectively applies the zoom value.
       * @param value
       * @protected
       */
      applyZoom(value:number):void;
      /**
       * Cancels this viewport animation so that subsequent calls to {@link yfiles.view.ViewportAnimation#animate} or {@link yfiles.view.ViewportAnimation#cleanup}
       * won't affect the viewport anymore.
       */
      cancel():void;
      /**
       * Cleans up after the animation.
       */
      cleanup():void;
      /**
       * Initializes this animation.
       * <p>
       * This has to be called once before any calls to {@link yfiles.view.ViewportAnimation#animate}. An instance of {@link yfiles.view.ViewportAnimation}
       * can be reused after an animation by setting new {@link yfiles.view.ViewportAnimation#targetBounds} or a new {@link yfiles.view.ViewportAnimation#preferredDuration} and calling Initialize() again.
       * </p>
       */
      initialize():void;
      /**
       * Gets or sets the target world bounds.
       * @type {yfiles.geometry.Rect}
       */
      targetBounds:yfiles.geometry.Rect;
      /**
       * Whether to respect the {@link yfiles.view.CanvasComponent#viewportLimiter} of the {@link yfiles.view.CanvasComponent} whose {@link yfiles.view.CanvasComponent#viewport} is animated.
       * <p>
       * Default is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      considerViewportLimiter:boolean;
      /**
       * Gets or sets the preferred duration of the animation.
       * @type {yfiles.lang.TimeSpan}
       */
      preferredDuration:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the maximum zoom level to use after the animation.
       * <p>
       * The default is {@link number#POSITIVE_INFINITY}.
       * </p>
       * @type {number}
       */
      maximumTargetZoom:number;
      /**
       * Gets or sets the scroll bar visibility that will be used during the animation.
       * @default yfiles.view.ScrollBarVisibility.NEVER
       * @type {yfiles.view.ScrollBarVisibility}
       */
      scrollBarVisibility:yfiles.view.ScrollBarVisibility;
      /**
       * Gets or sets the margins in view coordinate dimensions to use at the target zoom level.
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      targetViewMargins:yfiles.geometry.Insets;
      static $class:yfiles.lang.Class;
    }
    export enum ScrollBarVisibility{
      /**
       * Never display the scroll bar, even if not all of the content is visible.
       */
      NEVER,
      /**
       * Display the scroll bar when needed.
       * <p>
       * This is the default behavior in most applications.
       * </p>
       */
      AS_NEEDED,
      /**
       * Always display the scroll bar.
       * <p>
       * If it is not needed, i.e. if all content is already displaying, the scroll bar will remain visible but cannot be moved.
       * </p>
       */
      ALWAYS
    }
    export enum MouseButtons{
      /**
       * No mouse button.
       */
      NONE,
      /**
       * The left mouse button.
       */
      LEFT,
      /**
       * The right mouse button.
       */
      RIGHT,
      /**
       * The middle mouse button.
       */
      MIDDLE,
      /**
       * Extra mouse button 1.
       */
      X1,
      /**
       * Extra mouse button 2.
       */
      X2
    }
    export enum MouseWheelBehaviors{
      /**
       * The mouse wheel has no effect.
       */
      NONE,
      /**
       * The mouse wheel changes the zoom factor.
       */
      ZOOM,
      /**
       * The mouse wheel scrolls the view.
       */
      SCROLL,
      /**
       * Can be combined with {@link yfiles.view.MouseWheelBehaviors#ZOOM} or {@link yfiles.view.MouseWheelBehaviors#SCROLL} to perform the
       * action only when the control has focus.
       */
      ONLY_WHEN_FOCUSED
    }
    /**
     * The event arguments for the event handler used by {@link yfiles.view.CanvasComponent#raisePrepareRenderContextEvent}
     * <p>
     * This event argument can be used to modify the {@link yfiles.view.PrepareRenderContextEventArgs#context} during {@link yfiles.view.CanvasComponent#exportContent} and
     * {@link yfiles.view.CanvasComponent#invalidate}
     * </p>
     * @see yfiles.view.CanvasComponent#addPrepareRenderContextListener
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface PrepareRenderContextEventArgs extends yfiles.lang.EventArgs{}
    export class PrepareRenderContextEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.view.PrepareRenderContextEventArgs} class.
       * @param context The initial context which can be modified by the event handlers using the {@link #context} property.
       */
      constructor(context:yfiles.view.IRenderContext);
      /**
       * Gets or sets the context.
       * @throws {Stubs.Exceptions.ArgumentError} If the argument is <code>null</code>.
       * @type {yfiles.view.IRenderContext}
       */
      context:yfiles.view.IRenderContext;
      static $class:yfiles.lang.Class;
    }
    export enum SizeChangedDetectionMode{
      /**
       * No size change detection.
       * <p>
       * Consequently, no {@link yfiles.view.CanvasComponent#addSizeChangedListener SizeChanged} events are fired.
       * </p>
       */
      NONE,
      /**
       * Size change detection based on invisible sensor elements.
       * <p>
       * This is the preferred mode since it provides
       * {@link yfiles.view.CanvasComponent#addSizeChangedListener SizeChanged}events in real time.
       * </p>
       */
      SENSOR,
      /**
       * Timer based size change detection.
       * <p>
       * A timer periodically checks the current size with the previous one. Typically, this method is used as fall back in case
       * the {@link yfiles.view.SizeChangedDetectionMode#SENSOR} mode is not available in a browser.
       * </p>
       */
      TIMER,
      /**
       * This is the default mode which uses the preferred {@link yfiles.view.SizeChangedDetectionMode#SENSOR} mode if possible,
       * otherwise the {@link yfiles.view.SizeChangedDetectionMode#TIMER} mode is used.
       */
      AUTO
    }
    /**
     * This event occurs when the size of a control is changed.
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @final
     */
    export interface SizeChangedEventArgs extends yfiles.lang.EventArgs{}
    export class SizeChangedEventArgs {
      /**
       * Gets or sets the element whose size was changed.
       * @type {HTMLElement}
       */
      source:HTMLElement;
      /**
       * Gets or sets the old size of the control.
       * @type {yfiles.geometry.Size}
       */
      oldSize:yfiles.geometry.Size;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.view.CanvasComponent} can be used to efficiently display all kinds of data.
     * <p>
     * The control supports high performance zooming and panning of visual content and provides a high level view of the visual
     * tree. The control uses double precision world-coordinates to render items that are composed using a visual tree. In its
     * most simple form the scene graph is a tree-like structure of elements that will be rendered in depth-first-search
     * order.
     * </p>
     * <p>
     * The scene graph tree is made up of {@link yfiles.view.ICanvasObject} elements. Multiple elements are grouped in {@link yfiles.view.ICanvasObjectGroup}
     * elements. An {@link yfiles.view.ICanvasObject} consists of a {@link yfiles.view.ICanvasObject#userObject user object} and an {@link yfiles.view.ICanvasObjectDescriptor} that is used
     * by the rendering engine to query interfaces that perform the actual creation of the
     * {@link yfiles.view.Visual}s, hit testing, and bounds calculation.
     * </p>
     * <p>
     * This component handles the following commands by default:
     * </p>
     * <ul>
     * <li>{@link yfiles.input.ICommand#INCREASE_ZOOM}</li>
     * <li>{@link yfiles.input.ICommand#DECREASE_ZOOM}</li>
     * <li>{@link yfiles.input.ICommand#ZOOM}</li>
     * <li>{@link yfiles.input.ICommand#FIT_CONTENT}</li>
     * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_DOWN}</li>
     * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_UP}</li>
     * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_LEFT}</li>
     * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_RIGHT}</li>
     * </ul>
     * <p>
     * The handlers for {@link yfiles.input.ICommand#INCREASE_ZOOM} and {@link yfiles.input.ICommand#DECREASE_ZOOM} use the specified command
     * parameter as the factor by which the {@link yfiles.view.CanvasComponent#zoom} value is multiplied or divided, respectively. The parameter is not required. If it
     * is not present a default value is used. The handler for the {@link yfiles.input.ICommand#ZOOM} command needs a parameter. The
     * parameter can either be a number which is interpreted as the new zoom level, or it can be a suitable object. Supported
     * objects are {@link yfiles.geometry.Rect}, {@link yfiles.geometry.Point} and {@link yfiles.graph.ILookup}. If a rectangle is specified as the
     * parameter for the {@link yfiles.input.ICommand#ZOOM} command, the handler adjusts the {@link yfiles.view.CanvasComponent#viewport} such that the rectangle tightly fits into
     * it. If a point is specified, the handler sets the {@link yfiles.view.CanvasComponent#center} to that point. If the parameter implements {@link yfiles.graph.ILookup}, then
     * the handler tries to get a {@link yfiles.view.IBoundsProvider} from the lookup and uses the bounds obtained from the bounds
     * provider as the rectangle to zoom to.
     * </p>
     * <p>
     * {@link yfiles.view.CanvasComponent} is discussed in detail in the section {@link @DGUIDE_PREFIX@/view_graphcontrol.html The Component for Displaying Graphs}.
     * </p>
     * @see yfiles.view.IVisualCreator
     * @see yfiles.input.IHitTestable
     * @see yfiles.view.IVisibilityTestable
     * @class
     * @implements {yfiles.graph.ILookup}
     */
    export interface CanvasComponent extends Object,yfiles.graph.ILookup{}
    export class CanvasComponent {
      /**
       * Creates a new instance of {@link yfiles.view.CanvasComponent} using the <code>div</code> element that is specified by the <code>selector</code>.
       * <p>
       * The canvas will be empty with no associated InputModes.
       * </p>
       * @param selector The CSS selector or id for an existing <code>div</code> element
       */
      constructor(selector:string);
      /**
       * Creates a new instance of {@link yfiles.view.CanvasComponent} for the provided <code>div</code> element.
       * <p>
       * The canvas will be empty with no associated {@link yfiles.view.CanvasComponent#inputMode}s.
       * </p>
       * @param div The existing <code>div</code> element to use for rendering the canvas into.
       */
      constructor(div:HTMLDivElement);
      /**
       * Creates a new instance of {@link yfiles.view.CanvasComponent} with default properties.
       * <p>
       * The canvas will be empty with no associated InputModes. This constructor creates a new {@link yfiles.view.CanvasComponent#div} element that needs to be added
       * to the DOM manually.
       * </p>
       */
      constructor();
      /**
       * Scrolls to the provided view point and zoom level in an animated fashion.
       * @param viewPoint The new {@link #viewPoint}
       * @param zoom The new zoom level.
       */
      animateScrollTo(viewPoint:yfiles.geometry.Point,zoom:number):void;
      /**
       * Cleans up the Div element by removing any connection from the element to the Control instance.
       */
      cleanUp():void;
      /**
       * Helper method that ensures that the view port limit as returned by the {@link yfiles.view.CanvasComponent#viewportLimiter} are obeyed.
       * <p>
       * This method will {@link yfiles.view.CanvasComponent#zoomTo} the limited bounds if necessary.
       * </p>
       * @see yfiles.view.CanvasComponent#viewportLimiter
       */
      coerceViewportLimits():void;
      /**
       * Compares two {@link yfiles.view.ICanvasObject} instances that are live in this canvas.
       * <p>
       * The comparison will yield values greater than zero if and only if the second object is rendered after the first one.
       * </p>
       * @param canvasObject1 the first object to compare
       * @param canvasObject2 the second object to compare
       * @returns 0 if canvasObject1 ==
       * canvasObject2
       * <br />
       * >0 if canvasObject1 is painted before canvasObject2 &lt;0 if canvasObject1 is painted after canvasObject2
       */
      compareRenderOrder(canvasObject1:yfiles.view.ICanvasObject,canvasObject2:yfiles.view.ICanvasObject):number;
      /**
       * Factory method for the BackgroundGroup property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#backgroundGroup} property.
       * </p>
       * @returns a new instance of ICanvasObjectGroup
       * @protected
       */
      createBackgroundGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the ContentGroup property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#contentGroup} property.
       * </p>
       * @returns a new instance of ICanvasObjectGroup
       * @protected
       */
      createContentGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the {@link yfiles.view.CanvasComponent#focusGroup} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#focusGroup} property.
       * </p>
       * @returns a new instance of ICanvasObjectGroup
       * @protected
       */
      createFocusGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the HighlightGroup property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#highlightGroup} property.
       * </p>
       * @returns a new instance of ICanvasObjectGroup
       * @protected
       */
      createHighlightGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the {@link yfiles.view.CanvasComponent#inputModeContext} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#inputModeContext} property.
       * </p>
       * @returns a new instance of {@link #inputModeContext} that has this instance set as it's {@link #canvasComponent} and uses this instance's {@link #inputModeContextLookup}
       * callback to satisfy requests.
       * @protected
       */
      createInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Factory method for the InputModeGroup property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#inputModeGroup} property.
       * </p>
       * @returns a new instance of ICanvasObjectGroup
       * @protected
       */
      createInputModeGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Creates an appropriate {@link yfiles.view.IRenderContext render context} that can be used to {@link yfiles.view.IVisualCreator#createVisual create visuals} using {@link yfiles.view.IVisualCreator}
       * implementations.
       * <p>
       * This method is a convenience method to obtain a context for special visual creator implementations and normally needs
       * not be used by application developers.
       * </p>
       * @returns A new context instance that is bound to this instance.
       */
      createRenderContext():yfiles.view.IRenderContext;
      /**
       * Factory method for the SelectionGroup property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.CanvasComponent#selectionGroup} property.
       * </p>
       * @returns a new instance of ICanvasObjectGroup
       * @protected
       */
      createSelectionGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Ensures that the provided bounds are visible in this control by adjusting the viewport correspondingly.
       * @param bounds The bounds to make visible.
       */
      ensureVisible(bounds:yfiles.geometry.Rect):void;
      /**
       * Exports the graphical content for this {@link yfiles.view.CanvasComponent}.
       * <p>
       * This method will create the visual content using the world coordinate system.
       * </p>
       * @param context The context to use.
       * @returns A composite Visual that represents the contents of this control.
       */
      exportContent(context:yfiles.view.IRenderContext):Element;
      /**
       * Fires the size changed event.
       * @param oldSize
       */
      fireSizeChanged(oldSize:yfiles.geometry.Size):void;
      /**
       * Adjusts the view port to fully encompass the {@link yfiles.view.CanvasComponent#contentRect}.
       * <p>
       * The zoom level will not be greater than <code>1.0d</code>. If the view port is limited by the {@link yfiles.view.CanvasComponent#viewportLimiter} the zoom level might be greater than <code>1.0d</code>
       * to respect the limits.
       * </p>
       * @see yfiles.view.CanvasComponent#updateContentRect
       * @see yfiles.view.CanvasComponent#zoom
       * @see yfiles.view.CanvasComponent#zoomTo
       */
      fitContent():void;
      /**
       * Focuses the {@link yfiles.view.CanvasComponent#div} element that is backing this instance.
       */
      focus():void;
      /**
       * Convenience method that calculates the bounds for a given canvas object in the scene graph.
       * <p>
       * This method queries the descriptor for the {@link yfiles.view.IBoundsProvider} for the user object and returns the result.
       * </p>
       * @param canvasObject the canvas object to query the bounds from
       * @returns the non-<code>null</code> bounds
       */
      getBounds(canvasObject:yfiles.view.ICanvasObject):yfiles.geometry.Rect;
      /**
       * Returns the top most canvas object instance that is hit at the given coordinate set.
       * <p>
       * This will return the canvas object that is painted last at the given position.
       * </p>
       * @param location the coordinates of the query in the world coordinate system
       * @returns 
       */
      getCanvasObject(location:yfiles.geometry.Point):yfiles.view.ICanvasObject;
      /**
       * Enumerates over all possible {@link yfiles.view.ICanvasObject} instances in the tree below the given group.
       * @param group
       * @returns An enumerable for all {@link }s in the tree below the given group.
       */
      getCanvasObjects(group:yfiles.view.ICanvasObjectGroup):yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Returns a list of all canvas objects in hit order at the given world coordinate location.
       * <p>
       * The order of the elements in the list is the natural hit test order, i.e. elements painted on top of others will be
       * reported first in the list.
       * </p>
       * @param location the coordinates of the query in the world coordinate system
       * @returns an enumerable of canvas object that are hit in reverse painting order
       */
      getCanvasObjects(location:yfiles.geometry.Point):yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Enumerates over all possible {@link yfiles.view.ICanvasObject} instances in the tree.
       * @returns An enumerable for all {@link }s in the tree.
       */
      getCanvasObjects():yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Gets the {@link yfiles.view.CanvasComponent} instance that is associated with the given DOM element, or null.
       * @param element
       * @returns 
       * @static
       */
      static getComponent(element:Element):yfiles.view.CanvasComponent;
      /**
       * Gets the {@link yfiles.view.Visual} that is currently visualizing the given {@link yfiles.view.ICanvasObject}.
       * <p>
       * Note that depending on the current state of the viewport and the control no such instance may be available and instead
       * this method will yield <code>null</code>. However if a visual is currently being displayed for the given canvas object, it will be
       * returned. This method should be used with care. Manipulation of the given instance should normally be done through the
       * corresponding {@link yfiles.view.IVisualCreator} that created the visual in the first place. This method rather serves as a
       * utility method for UI testing frameworks and similar use cases.
       * </p>
       * @param canvasObject The canvas object.
       * @returns The {@link } that is currently used by the <code>canvasObject</code>.
       */
      getVisual(canvasObject:yfiles.view.ICanvasObject):yfiles.view.Visual;
      /**
       * Convenience method that retrieves the {@link yfiles.view.IVisualCreator} for a given {@link yfiles.view.ICanvasObject}.
       * @param canvasObject the canvas object to query the visual creator implementation from
       * @returns an instance of the visual creator interface
       * @see yfiles.view.ICanvasObjectDescriptor#getVisualCreator
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       */
      getVisualCreator(canvasObject:yfiles.view.ICanvasObject):yfiles.view.IVisualCreator;
      /**
       * Converts view coordinates from the document's coordinate system to the control's coordinate system.
       * @param localPoint The coordinates to convert.
       * @returns The coordinates in the control's coordinate system.
       */
      globalToLocal(localPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Assures that the {@link yfiles.view.CanvasComponent#contentRect content rectangle} encompasses the given rectangle.
       * <p>
       * Note that this will not change the view port, it will only change the scroll bars if necessary.
       * </p>
       * @param rectangle the rectangle that should be included in the content rectangle
       */
      growContentRect(rectangle:yfiles.geometry.Rect):void;
      /**
       * Enumerates all hit elements in the canvas below the given group that are accepted by a given filter using a specific {@link yfiles.view.ICanvasContext}
       * as the argument to the {@link yfiles.input.IHitTestable#isHit} method.
       * <p>
       * Hit testing is performed using the {@link yfiles.view.ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
       * {@link yfiles.view.ICanvasObject} in the current scene graph. The enumeration is performed lazily.
       * </p>
       * @param context The context instance to pass to {@link #isHit}.
       * @param location the coordinates to perform the hit test at
       * @param [root=null] the root of the scene graph to use
       * @param [filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
       * @returns a live enumeration of the elements that are hit
       * @see yfiles.input.IHitTestable
       */
      hitElementsAt(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,root?:yfiles.view.ICanvasObjectGroup,filter?:(obj:yfiles.view.ICanvasObject)=>boolean):yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Enumerates all hit elements in the canvas below the given group that are accepted by a given filter using a specific {@link yfiles.view.ICanvasContext}
       * as the argument to the {@link yfiles.input.IHitTestable#isHit} method.
       * <p>
       * Hit testing is performed using the {@link yfiles.view.ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
       * {@link yfiles.view.ICanvasObject} in the current scene graph. The enumeration is performed lazily.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.context The context instance to pass to {@link #isHit}.
       * @param options.location the coordinates to perform the hit test at
       * @param [options.root=null] the root of the scene graph to use
       * @param [options.filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
       * @returns a live enumeration of the elements that are hit
       * @see yfiles.input.IHitTestable
       */
      hitElementsAt(options:{context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,root?:yfiles.view.ICanvasObjectGroup,filter?:(obj:yfiles.view.ICanvasObject)=>boolean}):yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Enumerates all hit elements in the canvas below the given group that are accepted by a given filter.
       * <p>
       * Hit testing is performed using the {@link yfiles.view.ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
       * {@link yfiles.view.ICanvasObject} in the current scene graph. The enumeration is performed lazily.
       * </p>
       * @param location the coordinates to perform the hit test at
       * @param [root=null] the root of the scene graph to use
       * @param [filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
       * @returns a live enumeration of the elements that are hit
       * @see yfiles.input.IHitTestable
       * @see yfiles.view.CanvasComponent#hitTestRadius
       */
      hitElementsAt(location:yfiles.geometry.Point,root?:yfiles.view.ICanvasObjectGroup,filter?:(obj:yfiles.view.ICanvasObject)=>boolean):yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Enumerates all hit elements in the canvas below the given group that are accepted by a given filter.
       * <p>
       * Hit testing is performed using the {@link yfiles.view.ICanvasObjectDescriptor#getHitTestable} instance returned for each visible
       * {@link yfiles.view.ICanvasObject} in the current scene graph. The enumeration is performed lazily.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.location the coordinates to perform the hit test at
       * @param [options.root=null] the root of the scene graph to use
       * @param [options.filter=null] The predicate that decides whether a given canvas object should be considered for testing at all or <code>null</code>.
       * @returns a live enumeration of the elements that are hit
       * @see yfiles.input.IHitTestable
       * @see yfiles.view.CanvasComponent#hitTestRadius
       */
      hitElementsAt(options:{location:yfiles.geometry.Point,root?:yfiles.view.ICanvasObjectGroup,filter?:(obj:yfiles.view.ICanvasObject)=>boolean}):yfiles.collections.IEnumerable<yfiles.view.ICanvasObject>;
      /**
       * Callback that is used by the default implementation of {@link yfiles.view.CanvasComponent#inputModeContext} to resolve {@link yfiles.graph.ILookup#lookup} calls.
       * @param type The Type to query
       * @returns The result of the query.
       * @see yfiles.input.IInputModeContext
       * @protected
       */
      inputModeContextLookup(type:yfiles.lang.Class):Object;
      /**
       * Invalidates this instance and marks it as in need for an update.
       * <p>
       * Calling this method is fast and will only mark the control for a future, asynchronously executed call to UpdateVisual.
       * </p>
       */
      invalidate():void;
      /**
       * Convenience method that calculates the hit tests a given canvas object in the scene graph.
       * <p>
       * This method queries the descriptor for the {@link yfiles.input.IHitTestable} for the user object and returns the result of the
       * hit test query. If there is no <code>IHitTestable</code> returned by the descriptor this method returns <code>false</code>.
       * </p>
       * @param canvasObject the canvas object to query the bounds from
       * @param location the coordinates of the query in the world coordinate system
       * @returns whether the canvas object is hit at the given coordinates
       */
      isHit(canvasObject:yfiles.view.ICanvasObject,location:yfiles.geometry.Point):boolean;
      /**
       * Converts view coordinates from the control's coordinate system to the document's coordinate system.
       * @param localPoint The coordinates to convert.
       * @returns The coordinates in the document's coordinate system.
       */
      localToGlobal(localPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * This method will be called when the mouse wheel was turned and if the {@link yfiles.view.CanvasComponent#mouseWheelBehavior} property is set to
       * {@link yfiles.view.MouseWheelBehaviors#SCROLL}.
       * <p>
       * This method will scroll the view in vertical direction. If the Shift key modifier has been pressed this method will
       * scroll in horizontal direction.
       * </p>
       * @param evt the event describing the action
       * @see yfiles.view.CanvasComponent#mouseWheelBehavior
       * @see yfiles.view.CanvasComponent#mouseWheelScrollFactor
       * @protected
       */
      mouseWheelScroll(evt:yfiles.view.MouseEventArgs):void;
      /**
       * This method will be called when the mouse wheel was turned and if the {@link yfiles.view.CanvasComponent#mouseWheelBehavior} property is set to
       * {@link yfiles.view.MouseWheelBehaviors#ZOOM}.
       * <p>
       * This method will adjust the current zoom level. If the Control key modifier has been pressed this method will keep the
       * world coordinates at the current mouse position, i.e. the zoom will not necessarily be into the center of the canvas.
       * </p>
       * @param evt the event describing the action
       * @see yfiles.view.CanvasComponent#mouseWheelBehavior
       * @protected
       */
      mouseWheelZoom(evt:yfiles.view.MouseEventArgs):void;
      /**
       * Raises the {@link yfiles.view.CanvasComponent#addPrepareRenderContextListener PrepareRenderContext}.
       * @param evt The event arguments to raise.
       * @see yfiles.view.CanvasComponent#addPrepareRenderContextListener
       * @protected
       */
      raisePrepareRenderContextEvent(evt:yfiles.view.PrepareRenderContextEventArgs):void;
      /**
       * Raises the {@link yfiles.view.CanvasComponent#addUpdatedVisualListener UpdatedVisual}.
       * @see yfiles.view.CanvasComponent#addUpdatedVisualListener
       * @protected
       */
      raiseUpdatedVisualEvent():void;
      /**
       * Raises the {@link yfiles.view.CanvasComponent#addUpdatingVisualListener UpdatingVisual}.
       * @see yfiles.view.CanvasComponent#addUpdatingVisualListener
       * @protected
       */
      raiseUpdatingVisualEvent():void;
      /**
       * Adds a callback to the invocation chain and triggers the chain's execution.
       * <p>
       * Therefore, the given callback will be executed in sequence with previously added actions, e.g. the
       * {@link yfiles.view.CanvasComponent}'s invalidation.
       * </p>
       * @param callback The function that should be executed.
       * @param args The arguments with which the function should be called.
       */
      schedule(callback:Function,...args:Object[]):void;
      /**
       * Sets the content rectangle.
       * <p>
       * The content rectangle is the space in world coordinates that the user should at least be able to scroll to. I.e. if the
       * area currently visible in the control does not encompass the content rect, the scrollbars will be made visible, unless
       * they are disabled.
       * </p>
       * @param x
       * @param y
       * @param w
       * @param h
       * @see yfiles.view.CanvasComponent#contentRect
       * @see yfiles.view.CanvasComponent#horizontalScrollBarPolicy
       * @see yfiles.view.CanvasComponent#verticalScrollBarPolicy
       */
      setContentRect(x:number,y:number,w:number,h:number):void;
      /**
       * Stops the mouse capture and returns to normal event capturing.
       * <p>
       * When {@link yfiles.view.CanvasComponent#mouseCaptureEnabled} is set to <code>true</code> and a mouse down event is received, the control starts capturing all mouse events by registering
       * handlers on the document element. Mouse capturing is normally stopped when the mouse button is released. Mouse capture
       * can be manually stopped by calling this function.
       * </p>
       */
      stopMouseCapture():void;
      /**
       * Converts world coordinates to view coordinates expressed in the control's coordinate system.
       * @param worldPoint the coordinates in the world coordinate system
       * @returns returns the coordinates in pixels relative to the controls upper left corner
       */
      toViewCoordinates(worldPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Converts view coordinates (expressed in the control's coordinate system) to world coordinates.
       * @param viewPoint the coordinates in pixels relative to the controls upper left corner
       * @returns returns the coordinates in the world coordinate system
       */
      toWorldCoordinates(viewPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Updates the {@link yfiles.view.CanvasComponent#contentRect} to encompass the bounds by all elements in the current scene graph plus the given margins.
       * <p>
       * This method will traverse all {@link yfiles.view.ICanvasObject#visible visible} elements in the scene graph and query their {@link yfiles.view.ICanvasObjectDescriptor#getBoundsProvider bounds} The resulting bounds will be
       * set to the content rectangle plus the provided <code>margins</code>.
       * </p>
       * @param [margins=null]
       * @param [group=null]
       * @see yfiles.view.IBoundsProvider
       * @see yfiles.view.CanvasComponent#contentRect
       */
      updateContentRect(margins?:yfiles.geometry.Insets,group?:yfiles.view.ICanvasObjectGroup):void;
      /**
       * Updates the {@link yfiles.view.CanvasComponent#contentRect} to encompass the bounds by all elements in the current scene graph plus the given margins.
       * <p>
       * This method will traverse all {@link yfiles.view.ICanvasObject#visible visible} elements in the scene graph and query their {@link yfiles.view.ICanvasObjectDescriptor#getBoundsProvider bounds} The resulting bounds will be
       * set to the content rectangle plus the provided <code>margins</code>.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.margins=null]
       * @param [options.group=null]
       * @see yfiles.view.IBoundsProvider
       * @see yfiles.view.CanvasComponent#contentRect
       */
      updateContentRect(options:{margins?:yfiles.geometry.Insets,group?:yfiles.view.ICanvasObjectGroup}):void;
      /**
       * Updates the visual tree that displays the contents of this control.
       * <p>
       * Calling this method will synchronously update the visual tree and the SVG DOM and Canvas rendering.
       * </p>
       * <p>
       * This method will determine the {@link yfiles.view.ICanvasObject#dirty} canvas objects using the {@link yfiles.view.ICanvasObjectDescriptor#isDirty} method and will {@link yfiles.view.IVisualCreator#createVisual create}
       * or {@link yfiles.view.IVisualCreator#updateVisual update} the visuals that make up the visual tree.
       * </p>
       * <p>
       * Note that most of the time this method does not need to be called by client code. Instead calling {@link yfiles.view.CanvasComponent#invalidate}
       * will ultimately trigger the execution of this method. However invalidation calls will be coalesced and the actual
       * execution of the update will be delayed until the next event dispatch.
       * </p>
       */
      updateVisual():void;
      /**
       * Sets the zoom level and view port center to the given values.
       * @param center the new center of the view port in world coordinates
       * @param zoom the new zoom level
       */
      zoomTo(center:yfiles.geometry.Point,zoom:number):void;
      /**
       * Sets the zoom level and view port center so that the given rectangle in world coordinates fits the viewport.
       * @param bounds The coordinates of the rectangle to zoom to.
       */
      zoomTo(bounds:yfiles.geometry.Rect):void;
      /**
       * Gets the {@link yfiles.view.CanvasComponent#dropTarget} associated with this instance.
       * @type {yfiles.view.DropTarget}
       */
      dropTarget:yfiles.view.DropTarget;
      /**
       * Gets the <code>div</code> element that represents this instance.
       * @type {HTMLDivElement}
       */
      div:HTMLDivElement;
      /**
       * Gets or sets a value indicating whether all keyboard input is captured.
       * @type {boolean}
       */
      captureAllKeyboardInput:boolean;
      /**
       * Gets or sets a value indicating whether all pointer input is captured.
       * @type {boolean}
       */
      captureAllPointerInput:boolean;
      /**
       * Whether or not this control receives mouse input after the mouse is dragged out of its bounding area.
       * <p>
       * If enabled, this control receives mouse events when the mouse is moved out of its bounding area while a mouse button is
       * pressed, until the button is release (the mouse up event is the last event this control listens to).
       * </p>
       * @type {boolean}
       */
      mouseCaptureEnabled:boolean;
      /**
       * Gets a value indicating whether this instance currently has mouse capture.
       * @type {boolean}
       */
      hasMouseCapture:boolean;
      /**
       * Gets a value indicating whether this {@link yfiles.view.CanvasComponent} is currently focused.
       * @type {boolean}
       */
      focused:boolean;
      /**
       * Gets a value indicating whether the UI is right to left.
       * @type {boolean}
       */
      rightToLeft:boolean;
      /**
       * Gets or sets a value indicating whether to prevent scrolling if the element gets focused.
       * @type {boolean}
       */
      preventFocusScrolling:boolean;
      /**
       * Gets the control's {@link yfiles.view.CanvasComponent#svgDefsManager}
       * @type {yfiles.view.SvgDefsManager}
       */
      svgDefsManager:yfiles.view.SvgDefsManager;
      /**
       * Gets or sets whether to add a 'glass pane' overlay to the {@link yfiles.view.CanvasComponent} that acts as the source for all
       * mouse and touch events.
       * <p>
       * The glass pane can be used as a workaround for bogus mouse and touch event handling: If the touch down or mouse down
       * event occurs on an element that is removed from the DOM while the pointer is down, the up event may not bubble up to the
       * {@link yfiles.view.CanvasComponent} anymore, which leaves {@link yfiles.input.IInputMode input modes} in a bad state. Be aware that all direct touch and mouse
       * interaction with the SVG DOM is disabled if the glass pane is enabled, so this should be used with care.
       * </p>
       * @type {boolean}
       */
      useGlassPane:boolean;
      /**
       * Gets or sets the {@link yfiles.view.CanvasComponent#viewportLimiter} instance that can be used to limit the explorable region.
       * <p>
       * By default there are no limits set on the explorable region.
       * </p>
       * @type {yfiles.view.ViewportLimiter}
       */
      viewportLimiter:yfiles.view.ViewportLimiter;
      /**
       * Property that sets the editable state of this canvas.
       * <p>
       * {@link yfiles.input.IInputMode} implementations should honor this property. Events will still be delivered to the IInputModes,
       * however they should not modify the model.
       * </p>
       * @default true
       * @type {boolean}
       */
      editable:boolean;
      /**
       * Determines the behavior of the mouse wheel.
       * <p>
       * The behavior can be switched between {@link yfiles.view.MouseWheelBehaviors#ZOOM}, {@link yfiles.view.MouseWheelBehaviors#SCROLL} and
       * {@link yfiles.view.MouseWheelBehaviors#NONE}. Additionally, this property can be configured to perform the action
       * {@link yfiles.view.MouseWheelBehaviors#ONLY_WHEN_FOCUSED only when the control is focused}.
       * </p>
       * @default yfiles.view.MouseWheelBehaviors.ZOOM
       * @type {yfiles.view.MouseWheelBehaviors}
       */
      mouseWheelBehavior:yfiles.view.MouseWheelBehaviors;
      /**
       * Specifies an event recognizer that determines whether the zooming to the center of the view, rather than at the mouse
       * location should be used.
       * <p>
       * The default is {@link yfiles.input.KeyEventRecognizers#SHIFT_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      centerZoomEventRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Specifies the visibility policy for the vertical scrollbar.
       * <p>
       * Scrollbars don't need to be displayed in order to move the viewport. This can be achieved programmatically or using
       * special IInputMode instances. The default is {@link yfiles.view.ScrollBarVisibility#AS_NEEDED}.
       * </p>
       * @see yfiles.input.MoveViewportInputMode
       * @see yfiles.view.CanvasComponent#horizontalScrollBarPolicy
       * @default yfiles.view.ScrollBarVisibility.AS_NEEDED
       * @type {yfiles.view.ScrollBarVisibility}
       */
      verticalScrollBarPolicy:yfiles.view.ScrollBarVisibility;
      /**
       * Specifies the visibility policy for the vertical scrollbar.
       * <p>
       * Scrollbars don't need to be displayed in order to move the viewport. This can be achieved programmatically or using
       * special IInputMode instances. The default is {@link yfiles.view.ScrollBarVisibility#AS_NEEDED}.
       * </p>
       * @see yfiles.input.MoveViewportInputMode
       * @see yfiles.view.CanvasComponent#verticalScrollBarPolicy
       * @default yfiles.view.ScrollBarVisibility.AS_NEEDED
       * @type {yfiles.view.ScrollBarVisibility}
       */
      horizontalScrollBarPolicy:yfiles.view.ScrollBarVisibility;
      /**
       * Gets or sets a value indicating whether {@link yfiles.input.ICommand}s for scrolling like for example
       * {@link yfiles.input.ICommand#SCROLL_PAGE_DOWN}, the scrolling should be performed in animated fashion.
       * @default true
       * @type {boolean}
       */
      animateScrollCommands:boolean;
      /**
       * Gets and sets the Rectangle in world coordinates that holds the contents.
       * <p>
       * This influences the display of the scroll bars. If the content rectangle is not currently visible in the viewport,
       * scroll bars will be displayed, unless they are turned off completely. Note that in general the content rectangle is not
       * updated automatically but declared as a fixed rectangle. Rather the application programmer needs to set this value or
       * call respective automatic update methods.
       * </p>
       * @default '0,0,100,100'
       * @type {yfiles.geometry.Rect}
       */
      contentRect:yfiles.geometry.Rect;
      /**
       * Adds the given listener for the <code>ContentRectChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#contentRect content rectangle} has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeContentRectChangedListener
       */
      addContentRectChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ContentRectChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#contentRect content rectangle} has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addContentRectChangedListener
       */
      removeContentRectChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets and sets the insets in view coordinates that should be used by the {@link yfiles.view.CanvasComponent#fitContent}
       * operation.
       * <p>
       * This influences the amount of visible whitespace in the view coordinate system around the {@link yfiles.view.CanvasComponent#contentRect} after a {@link yfiles.view.CanvasComponent#fitContent}
       * operation. The default value is <code>(10,10,10,10)</code>.
       * </p>
       * <p>
       * Note limits that are enforced by the {@link yfiles.view.CanvasComponent#viewportLimiter} have a higher priority than these insets.
       * </p>
       * @default '10'
       * @type {yfiles.geometry.Insets}
       */
      fitContentViewMargins:yfiles.geometry.Insets;
      /**
       * Adds the given listener for the <code>FitContentViewMarginsChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#fitContentViewMargins margins for the fit content operation}
       * have been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeFitContentViewMarginsChangedListener
       */
      addFitContentViewMarginsChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>FitContentViewMarginsChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#fitContentViewMargins margins for the fit content operation}
       * have been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addFitContentViewMarginsChangedListener
       */
      removeFitContentViewMarginsChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseLeave</code> event that occurs when the mouse has exited the canvas.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * If the mouse leaves this canvas control with the mouse button pressed, this event is deferred until the mouse button is
       * released. Thus, this event will not interfere with the typical mouse button event cycle.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseLeaveListener
       */
      addMouseLeaveListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseLeave</code> event that occurs when the mouse has exited the canvas.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * If the mouse leaves this canvas control with the mouse button pressed, this event is deferred until the mouse button is
       * released. Thus, this event will not interfere with the typical mouse button event cycle.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseLeaveListener
       */
      removeMouseLeaveListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseEnter</code> event that occurs when the mouse has entered the canvas.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * If the mouse enters this canvas control with a mouse button pressed, this event is fired instantly but the current
       * button state reported by the {@link yfiles.view.MouseEventArgs} does not include this button unless the browser supports the
       * property <code>MouseEvent.buttons</code>. See the Known Issues for more details.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseEnterListener
       */
      addMouseEnterListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseEnter</code> event that occurs when the mouse has entered the canvas.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * If the mouse enters this canvas control with a mouse button pressed, this event is fired instantly but the current
       * button state reported by the {@link yfiles.view.MouseEventArgs} does not include this button unless the browser supports the
       * property <code>MouseEvent.buttons</code>. See the Known Issues for more details.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseEnterListener
       */
      removeMouseEnterListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseMove</code> event that occurs when the mouse has been moved in world coordinates.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * Move elements are delivered if no mouse button is pressed. This event will be fired, too, if the mouse does not move but
       * the world coordinates to which the current mouse position maps change. E.g. this will happen if the zoom level or the
       * view point is changed.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseMoveListener
       */
      addMouseMoveListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseMove</code> event that occurs when the mouse has been moved in world coordinates.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * Move elements are delivered if no mouse button is pressed. This event will be fired, too, if the mouse does not move but
       * the world coordinates to which the current mouse position maps change. E.g. this will happen if the zoom level or the
       * view point is changed.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseMoveListener
       */
      removeMouseMoveListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseDown</code> event that occurs when a mouse button has been pressed.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseDownListener
       */
      addMouseDownListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseDown</code> event that occurs when a mouse button has been pressed.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseDownListener
       */
      removeMouseDownListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseDrag</code> event that occurs when the mouse is being moved while at least one of the
       * mouse buttons is pressed.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseDragListener
       */
      addMouseDragListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseDrag</code> event that occurs when the mouse is being moved while at least one of the
       * mouse buttons is pressed.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseDragListener
       */
      removeMouseDragListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseUp</code> event that occurs when the mouse button has been released.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseUpListener
       */
      addMouseUpListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseUp</code> event that occurs when the mouse button has been released.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseUpListener
       */
      removeMouseUpListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseLostCapture</code> event that occurs when the mouse capture has been lost.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseLostCaptureListener
       */
      addMouseLostCaptureListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseLostCapture</code> event that occurs when the mouse capture has been lost.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseLostCaptureListener
       */
      removeMouseLostCaptureListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseWheel</code> event that occurs when the mouse wheel has turned.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseWheelListener
       */
      addMouseWheelListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseWheel</code> event that occurs when the mouse wheel has turned.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseWheelListener
       */
      removeMouseWheelListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MouseClick</code> event that occurs when the user clicked the mouse.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * This happens if press and release happens at the same position.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseClickListener
       */
      addMouseClickListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseClick</code> event that occurs when the user clicked the mouse.
       * <p>
       * This event delivers {@link yfiles.view.MouseEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * This happens if press and release happens at the same position.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseClickListener
       */
      removeMouseClickListener(listener:(sender:Object,evt:yfiles.view.MouseEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyDown</code> event that occurs when keys are being pressed.
       * <p>
       * This event delivers {@link yfiles.view.KeyEventArgs}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeKeyDownListener
       */
      addKeyDownListener(listener:(sender:Object,evt:yfiles.view.KeyEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyDown</code> event that occurs when keys are being pressed.
       * <p>
       * This event delivers {@link yfiles.view.KeyEventArgs}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addKeyDownListener
       */
      removeKeyDownListener(listener:(sender:Object,evt:yfiles.view.KeyEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyPressed</code> event that occurs when keys are being typed.
       * <p>
       * This event delivers {@link yfiles.view.KeyEventArgs}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeKeyPressedListener
       */
      addKeyPressedListener(listener:(sender:Object,evt:yfiles.view.KeyEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyPressed</code> event that occurs when keys are being typed.
       * <p>
       * This event delivers {@link yfiles.view.KeyEventArgs}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addKeyPressedListener
       */
      removeKeyPressedListener(listener:(sender:Object,evt:yfiles.view.KeyEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>KeyUp</code> event that occurs when keys are being released.
       * <p>
       * This event delivers {@link yfiles.view.KeyEventArgs}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeKeyUpListener
       */
      addKeyUpListener(listener:(sender:Object,evt:yfiles.view.KeyEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>KeyUp</code> event that occurs when keys are being released.
       * <p>
       * This event delivers {@link yfiles.view.KeyEventArgs}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addKeyUpListener
       */
      removeKeyUpListener(listener:(sender:Object,evt:yfiles.view.KeyEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EditableChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#editable} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeEditableChangedListener
       */
      addEditableChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EditableChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#editable} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addEditableChangedListener
       */
      removeEditableChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>ViewportChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#viewport} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeViewportChangedListener
       */
      addViewportChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ViewportChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#viewport} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addViewportChangedListener
       */
      removeViewportChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchDown</code> event that occurs when a finger has been put on the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchDownListener
       */
      addTouchDownListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchDown</code> event that occurs when a finger has been put on the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchDownListener
       */
      removeTouchDownListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchUp</code> event that occurs when a finger has been removed from the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchUpListener
       */
      addTouchUpListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchUp</code> event that occurs when a finger has been removed from the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchUpListener
       */
      removeTouchUpListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchMove</code> event that occurs when a finger has been moved on the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchMoveListener
       */
      addTouchMoveListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchMove</code> event that occurs when a finger has been moved on the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchMoveListener
       */
      removeTouchMoveListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchEnter</code> event that occurs when a finger on the touch screen has entered the canvas.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchEnterListener
       */
      addTouchEnterListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchEnter</code> event that occurs when a finger on the touch screen has entered the
       * canvas.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchEnterListener
       */
      removeTouchEnterListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchLeave</code> event that occurs when a finger on the touch screen has exited the canvas.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchLeaveListener
       */
      addTouchLeaveListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchLeave</code> event that occurs when a finger on the touch screen has exited the
       * canvas.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchLeaveListener
       */
      removeTouchLeaveListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchLostCapture</code> event that occurs when the touch capture has been lost.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchLostCaptureListener
       */
      addTouchLostCaptureListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchLostCapture</code> event that occurs when the touch capture has been lost.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchLostCaptureListener
       */
      removeTouchLostCaptureListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchClick</code> event that occurs when the user performed a tap gesture with a finger on the
       * touch screen.
       * <p>
       * An event that delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points when the
       * user performed a tap gesture with a finger on the touch screen.
       * </p>
       * <p>
       * This happens if down and up happens at the same position.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchClickListener
       */
      addTouchClickListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchClick</code> event that occurs when the user performed a tap gesture with a finger on
       * the touch screen.
       * <p>
       * An event that delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points when the
       * user performed a tap gesture with a finger on the touch screen.
       * </p>
       * <p>
       * This happens if down and up happens at the same position.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchClickListener
       */
      removeTouchClickListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>TouchLongPress</code> event that occurs when the user performed a long press gesture with a
       * finger on the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * This happens if the finger is held in the same position for the duration specified in {@link yfiles.view.CanvasComponent#longPressTime} after a
       * {@link yfiles.view.CanvasComponent#addTouchDownListener TouchDown}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeTouchLongPressListener
       */
      addTouchLongPressListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TouchLongPress</code> event that occurs when the user performed a long press gesture with a
       * finger on the touch screen.
       * <p>
       * This event delivers {@link yfiles.view.TouchEventArgs} in world coordinates using double precision floating points.
       * </p>
       * <p>
       * This happens if the finger is held in the same position for the duration specified in {@link yfiles.view.CanvasComponent#longPressTime} after a
       * {@link yfiles.view.CanvasComponent#addTouchDownListener TouchDown}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addTouchLongPressListener
       */
      removeTouchLongPressListener(listener:(sender:Object,evt:yfiles.view.TouchEventArgs)=>void):void;
      /**
       * Gets a list of the active {@link yfiles.view.TouchDevice}s at the time of invocation.
       * <p>
       * Note that this is not a live view, but rather a snapshot.
       * </p>
       * @type {yfiles.collections.IListEnumerable.<yfiles.view.TouchDevice>}
       */
      currentTouchDevices:yfiles.collections.IListEnumerable<yfiles.view.TouchDevice>;
      /**
       * Gets a value indicating whether at least one finger is on the touch screen.
       * @returns <code>true</code> if there is at least one {@link  touch device} that is down on the screen.
       * @type {boolean}
       */
      isTouchDeviceDown:boolean;
      /**
       * Gets the last {@link yfiles.view.TouchEventArgs touch event} triggered by this instance.
       * @type {yfiles.view.TouchEventArgs}
       */
      lastTouchEvent:yfiles.view.TouchEventArgs;
      /**
       * Gets the last {@link yfiles.view.MouseEventArgs mouse event} triggered by this instance.
       * @type {yfiles.view.MouseEventArgs}
       */
      lastMouseEvent:yfiles.view.MouseEventArgs;
      /**
       * Gets the last location provided by a pointing device (for instance mouse or touch).
       * <p>
       * Unlike {@link yfiles.view.CanvasComponent#lastMouseEvent} and {@link yfiles.view.CanvasComponent#lastTouchEvent} this will always contain the latest location, regardless of what input method generated it.
       * </p>
       * @see yfiles.view.CanvasComponent#lastMouseEvent
       * @see yfiles.view.CanvasComponent#lastTouchEvent
       * @type {yfiles.geometry.Point}
       */
      lastEventLocation:yfiles.geometry.Point;
      /**
       * Adds the given listener for the <code>InputModeChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#inputMode} property is changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#inputMode
       * @see yfiles.view.CanvasComponent#removeInputModeChangedListener
       */
      addInputModeChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>InputModeChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#inputMode} property is changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#inputMode
       * @see yfiles.view.CanvasComponent#addInputModeChangedListener
       */
      removeInputModeChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets or sets the single {@link yfiles.input.IInputMode} instance that shall be installed for this canvas.
       * @default null
       * @type {yfiles.input.IInputMode}
       */
      inputMode:yfiles.input.IInputMode;
      /**
       * Specifies the radius of the area around the mouse in view coordinates in which a {@link yfiles.input.IHitTestable} may lie to be
       * considered a valid hit.
       * <p>
       * This value can be queried from within the {@link yfiles.input.IHitTestable} implementation from the {@link yfiles.view.ICanvasContext#hitTestRadius} property. The default value
       * is <code>3.0d</code>
       * </p>
       * @default 3.0
       * @type {number}
       */
      hitTestRadius:number;
      /**
       * Adds the given listener for the <code>HitTestRadiusChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#hitTestRadius} property changes.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeHitTestRadiusChangedListener
       */
      addHitTestRadiusChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HitTestRadiusChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#hitTestRadius} property changes.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addHitTestRadiusChangedListener
       */
      removeHitTestRadiusChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets or sets the zoom factor for this {@link yfiles.view.CanvasComponent}.
       * <p>
       * A zoom level of <code>1.0f</code> will make each unit in world-coordinate space appear exactly one unit in screen coordinates wide.
       * The default is
       * <code>1.0f</code>. If this property is written, the canvas will zoom to the new value while keeping the center of view at the same
       * world coordinates.
       * </p>
       * <p>
       * The valid range is bounded by {@link yfiles.view.CanvasComponent#minimumZoom} and {@link yfiles.view.CanvasComponent#maximumZoom}
       * </p>
       * @see yfiles.view.CanvasComponent#addZoomChangedListener
       * @see yfiles.view.CanvasComponent#minimumZoom
       * @see yfiles.view.CanvasComponent#maximumZoom
       * @default 1.0
       * @type {number}
       */
      zoom:number;
      /**
       * Adds the given listener for the <code>ZoomChanged</code> event that occurs when the value of the {@link yfiles.view.CanvasComponent#zoom} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeZoomChangedListener
       */
      addZoomChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ZoomChanged</code> event that occurs when the value of the {@link yfiles.view.CanvasComponent#zoom} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addZoomChangedListener
       */
      removeZoomChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets or sets the minimum zoom factor for this {@link yfiles.view.CanvasComponent}.
       * <p>
       * This property sets a lower bound for the allowed values of the {@link yfiles.view.CanvasComponent#zoom} property. If this property is written, the canvas will
       * zoom to the new value while keeping the center of view at the same world coordinates. The default is <code>0.0001</code> but values
       * as small as 0.0000001d are possible.
       * </p>
       * @see yfiles.view.CanvasComponent#zoom
       * @see yfiles.view.CanvasComponent#maximumZoom
       * @default 0.0001
       * @type {number}
       */
      minimumZoom:number;
      /**
       * Gets or sets the maximum zoom value.
       * <p>
       * Default value is <code>100000</code>
       * </p>
       * @default 100000.0
       * @type {number}
       */
      maximumZoom:number;
      /**
       * Gets or sets the current view point.
       * <p>
       * The view point is the point in world coordinates that is mapped to the top left corner point in the current viewport.
       * Setting this point to another value will redispatch the last mouse event as the mouse will appear to have been moved or
       * dragged in the world coordinate system.
       * </p>
       * @default '0,0'
       * @type {yfiles.geometry.Point}
       */
      viewPoint:yfiles.geometry.Point;
      /**
       * Returns the size of the usable area in which the graph will be displayed.
       * <p>
       * This value excludes the scroll bars.
       * </p>
       * @type {yfiles.geometry.Size}
       */
      innerSize:yfiles.geometry.Size;
      /**
       * Returns the size of the HTML element.
       * @type {yfiles.geometry.Size}
       */
      size:yfiles.geometry.Size;
      /**
       * Returns the currently visible viewing region in world coordinates.
       * @see yfiles.view.CanvasComponent#viewPoint
       * @see yfiles.view.CanvasComponent#zoom
       * @type {yfiles.geometry.Rect}
       */
      viewport:yfiles.geometry.Rect;
      /**
       * Adds the given listener for the <code>GotFocus</code> event that occurs when the Control got the focus.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeGotFocusListener
       */
      addGotFocusListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GotFocus</code> event that occurs when the Control got the focus.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addGotFocusListener
       */
      removeGotFocusListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LostFocus</code> event that occurs when the Control lost the focus.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeLostFocusListener
       */
      addLostFocusListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LostFocus</code> event that occurs when the Control lost the focus.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addLostFocusListener
       */
      removeLostFocusListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Retrieves the {@link yfiles.graph.LookupChain} that can be used do decorate the {@link yfiles.graph.ILookup#lookup} call in the {@link yfiles.view.CanvasComponent#inputModeContext}.
       * @type {yfiles.graph.LookupChain}
       */
      inputModeContextLookupChain:yfiles.graph.LookupChain;
      /**
       * Returns an implementation of {@link yfiles.view.ICanvasContext} that describes the state of this {@link yfiles.view.CanvasComponent}.
       * @see yfiles.input.IHitTestable
       * @see yfiles.view.IBoundsProvider
       * @type {yfiles.view.ICanvasContext}
       */
      canvasContext:yfiles.view.ICanvasContext;
      /**
       * Gets or sets the world coordinate at the center of the control.
       * <p>
       * This can be used to move the view port.
       * </p>
       * @type {yfiles.geometry.Point}
       */
      center:yfiles.geometry.Point;
      /**
       * Gets or sets a property that causes the control to automatically pan the view when the mouse gets dragged outside of the
       * bounds of the control.
       * @default true
       * @type {boolean}
       */
      autoDrag:boolean;
      /**
       * Adds the given listener for the <code>AutoDragChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#autoDrag} property is changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeAutoDragChangedListener
       */
      addAutoDragChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>AutoDragChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#autoDrag} property is changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addAutoDragChangedListener
       */
      removeAutoDragChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets or sets the policy for caching {@link yfiles.view.Visual}s which are temporarily removed from the visual tree.
       * <p>
       * Visuals are temporarily removed when they are moved outside the visual part of the canvas (the {@link yfiles.view.CanvasComponent#viewport}) and re-added when they
       * enter the {@link yfiles.view.CanvasComponent#viewport} again. Caching prevents the need to re-build the visuals from scratch.
       * </p>
       * <p>
       * Visuals are only cached if this property is set to another value than {@link yfiles.view.VisualCachingPolicy#NEVER} and if a
       * dispose visual callback is {@link yfiles.view.IRenderContext#setDisposeCallback registered} during {@link yfiles.view.IVisualCreator#createVisual creation}.
       * </p>
       * @type {yfiles.view.VisualCachingPolicy}
       */
      visualCaching:yfiles.view.VisualCachingPolicy;
      /**
       * Adds the given listener for the <code>UpdatedVisual</code> event that occurs after the {@link yfiles.view.CanvasComponent#updateVisual visual tree has been updated}.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeUpdatedVisualListener
       */
      addUpdatedVisualListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>UpdatedVisual</code> event that occurs after the {@link yfiles.view.CanvasComponent#updateVisual visual tree has been updated}.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addUpdatedVisualListener
       */
      removeUpdatedVisualListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>UpdatingVisual</code> event.
       * <br />
       * Event that will be triggered before the {@link yfiles.view.CanvasComponent#updateVisual visual tree is updated}.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeUpdatingVisualListener
       */
      addUpdatingVisualListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>UpdatingVisual</code> event.
       * <br />
       * Event that will be triggered before the {@link yfiles.view.CanvasComponent#updateVisual visual tree is updated}.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addUpdatingVisualListener
       */
      removeUpdatingVisualListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets the root of the scene graph.
       * <p>
       * This group cannot be {@link yfiles.view.ICanvasObject#remove removed}.
       * </p>
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      rootGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets or sets the ContentGroup property.
       * <p>
       * The content group is the {@link yfiles.view.ICanvasObjectGroup} that should be used by the application code to put actual
       * content in.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createContentGroup}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addContentGroupChangedListener ContentGroupChanged}
       * event will be triggered.
       * </p>
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      contentGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Adds the given listener for the <code>ContentGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#contentGroup} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeContentGroupChangedListener
       */
      addContentGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ContentGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#contentGroup} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addContentGroupChangedListener
       */
      removeContentGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the HighlightGroup property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createHighlightGroup}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addHighlightGroupChangedListener HighlightGroupChanged}
       * event will be triggered.
       * </p>
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      highlightGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets or sets the HighlightGroup property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createHighlightGroup}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addHighlightGroupChangedListener HighlightGroupChanged}
       * event will be triggered.
       * </p>
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      focusGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Adds the given listener for the <code>FocusGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#focusGroup} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeFocusGroupChangedListener
       */
      addFocusGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>FocusGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#focusGroup} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addFocusGroupChangedListener
       */
      removeFocusGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>HighlightGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#highlightGroup} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeHighlightGroupChangedListener
       */
      addHighlightGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HighlightGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#highlightGroup} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addHighlightGroupChangedListener
       */
      removeHighlightGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the SelectionGroup property.
       * <p>
       * The selection group is the {@link yfiles.view.ICanvasObjectGroup} that should be used by the application code to put the
       * selection paintables in.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createSelectionGroup}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addSelectionGroupChangedListener SelectionGroupChanged}
       * event will be triggered.
       * </p>
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      selectionGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Adds the given listener for the <code>SelectionGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#selectionGroup} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeSelectionGroupChangedListener
       */
      addSelectionGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>SelectionGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#selectionGroup} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addSelectionGroupChangedListener
       */
      removeSelectionGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the InputModeGroup property.
       * <p>
       * This is the canvas object group where the {@link yfiles.input.IInputMode}s should add their temporary content to. This group by
       * default is in front of the {@link yfiles.view.CanvasComponent#contentGroup}.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createInputModeGroup}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addInputModeGroupChangedListener InputModeGroupChanged}
       * event will be triggered.
       * </p>
       * @see yfiles.view.CanvasComponent#contentGroup
       * @see yfiles.view.CanvasComponent#backgroundGroup
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      inputModeGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Adds the given listener for the <code>InputModeGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#inputModeGroup} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeInputModeGroupChangedListener
       */
      addInputModeGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>InputModeGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#inputModeGroup} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addInputModeGroupChangedListener
       */
      removeInputModeGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the BackgroundGroup property.
       * <p>
       * The background group is the {@link yfiles.view.ICanvasObjectGroup} that should be used by the application code to put background
       * elements in.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createBackgroundGroup}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addBackgroundGroupChangedListener BackgroundGroupChanged}
       * event will be triggered.
       * </p>
       * @see yfiles.view.CanvasComponent#contentGroup
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      backgroundGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Adds the given listener for the <code>BackgroundGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#backgroundGroup} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeBackgroundGroupChangedListener
       */
      addBackgroundGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>BackgroundGroupChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#backgroundGroup} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addBackgroundGroupChangedListener
       */
      removeBackgroundGroupChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the shape-rendering type for this visual and its children.
       * <p>
       * Shape rendering can improve performance but affects the display quality of this visual.
       * </p>
       * @type {yfiles.view.ShapeRenderingType}
       */
      shapeRendering:yfiles.view.ShapeRenderingType;
      /**
       * Gets or sets the image-rendering type for this visual and its children.
       * <p>
       * Image rendering can improve performance but affects the display quality of images.
       * </p>
       * @type {yfiles.view.ImageRenderingType}
       */
      imageRendering:yfiles.view.ImageRenderingType;
      /**
       * Returns an HTML element that can be used to show arbitrary HTML content like overlays, fly-outs, or pop-ups on top of
       * this {@link yfiles.view.CanvasComponent}.
       * <p>
       * A HTML element that was added to the overlay panel is drawn above the content of the CanvasControl but below its
       * scrollbars. In contrast to labels or other model items, the element keeps its size when the canvas is zoomed. It should
       * be placed with the CSS <code>position: absolute</code> to prevent multiple overlay elements to interfere with each other.
       * </p>
       * <p>
       * HTML elements can be added to and removed from the overlay panel at any time. In addition to this, you can use a
       * convenience feature of the constructor <code>ForDiv(div)</code> to add elements to the overlay panel. This constructor copies all
       * child elements of the given <code>div</code> element to its overlay panel.
       * </p>
       * @type {HTMLElement}
       */
      overlayPanel:HTMLElement;
      /**
       * Gets or sets the value of the drag time.
       * <p>
       * This value indicates the amount of time that may pass before a mouse movement is considered a drag when the mouse stays
       * within its {@link yfiles.view.CanvasComponent#dragSize} area. The higher the value the later a mouse movement will be recognized as a drag. This influences the
       * click-sensitivity, since a mouse button release is only considered a click if there was no drag since the last mouse
       * button press. The default value is 300 milliseconds.
       * </p>
       * @see yfiles.view.CanvasComponent#dragSize
       * @default '00:00:00.3000000'
       * @type {yfiles.lang.TimeSpan}
       */
      dragTime:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the value of the double click time.
       * <p>
       * This value indicates the amount of time that may pass within which two subsequent mouse clicks are considered multi
       * clicks. The higher the value the easier it will be to double-click. The default value is 500 milliseconds.
       * </p>
       * @see yfiles.view.CanvasComponent#doubleClickSize
       * @type {yfiles.lang.TimeSpan}
       */
      doubleClickTime:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the value of the double tap time.
       * <p>
       * This value indicates the amount of time that may pass within which two subsequent touch taps are considered multi taps.
       * The higher the value the easier it will be to double-tap. The default value is 500 milliseconds.
       * </p>
       * @see yfiles.view.CanvasComponent#doubleTapSize
       * @type {yfiles.lang.TimeSpan}
       */
      doubleTapTime:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the value of the long press time.
       * <p>
       * This value indicates the amount of time that may pass before a touch down is considered a long press when the pointer
       * stays within its {@link yfiles.view.CanvasComponent#dragSize} area. The higher the value the later a touch pointer movement will be recognized as a long press. The
       * default value is 250 milliseconds.
       * </p>
       * @see yfiles.view.CanvasComponent#dragSize
       * @type {yfiles.lang.TimeSpan}
       */
      longPressTime:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the area in view coordinates the mouse may stay in before a movement is considered a drag.
       * <p>
       * If the mouse is moved within this area longer than {@link yfiles.view.CanvasComponent#dragTime} milliseconds, the movement will be considered a drag, nevertheless.
       * The larger the area the later a mouse movement will be recognized as a drag. This influences the click-sensitivity,
       * since a mouse button release is only considered a click if there was no drag since the last mouse button press. The
       * default value is <code>(5, 5)</code>.
       * </p>
       * @see yfiles.view.CanvasComponent#dragSize
       * @default '5,5'
       * @type {yfiles.geometry.Size}
       */
      dragSize:yfiles.geometry.Size;
      /**
       * The area in view coordinates the mouse needs to stay in before multiple clicks are considered multiple single clicks
       * instead of multi-clicks.
       * <p>
       * If the mouse stays within this within than {@link yfiles.view.CanvasComponent#doubleClickTime}, multiple clicks will be considered double or multi-clicks. The default
       * value is <code>(10, 10)</code>.
       * </p>
       * @see yfiles.view.CanvasComponent#doubleClickTime
       * @type {yfiles.geometry.Size}
       */
      doubleClickSize:yfiles.geometry.Size;
      /**
       * Gets or sets the area in view coordinates the touch pointer needs to stay in before multiple taps are considered
       * multiple single taps instead of multi-taps.
       * <p>
       * If the touch pointer stays within this within than {@link yfiles.view.CanvasComponent#doubleTapTime}, multiple taps will be considered double or multi-taps. The default
       * value is <code>(20, 20)</code>.
       * </p>
       * @see yfiles.view.CanvasComponent#doubleTapTime
       * @type {yfiles.geometry.Size}
       */
      doubleTapSize:yfiles.geometry.Size;
      /**
       * Gets or sets the factor by which the zoom level changes when the mouse wheel is turned.
       * <p>
       * The default value is 1.2.
       * </p>
       * @default 1.2
       * @type {number}
       */
      mouseWheelZoomFactor:number;
      /**
       * Adds the given listener for the <code>MouseWheelZoomFactorChanged</code> event that occurs when the value of the {@link yfiles.view.CanvasComponent#mouseWheelZoomFactor} property has been
       * changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseWheelZoomFactorChangedListener
       */
      addMouseWheelZoomFactorChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseWheelZoomFactorChanged</code> event that occurs when the value of the {@link yfiles.view.CanvasComponent#mouseWheelZoomFactor} property has
       * been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseWheelZoomFactorChangedListener
       */
      removeMouseWheelZoomFactorChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets or sets a factor that controls how fast the viewport scrolls when the mouse wheel is turned.
       * <p>
       * Scrolling can work either a few lines at a time (per notch of the scroll wheel) or one page at a time. This is a system
       * setting. This property controls how many pixels represent a line if scrolling is done by line. In case scrolling is done
       * by page, this property has no effect.
       * </p>
       * <p>
       * The default value of 5 pixels. To speed up scrolling, set a larger value, to slow down scrolling, set a smaller value.
       * </p>
       * @see yfiles.view.CanvasComponent#mouseWheelZoomFactor
       * @see yfiles.view.CanvasComponent#mouseWheelBehavior
       * @see yfiles.view.MouseEventArgs#scrollAmount
       * @see yfiles.view.MouseEventArgs#deltaMode
       * @default 5
       * @type {number}
       */
      mouseWheelScrollFactor:number;
      /**
       * Adds the given listener for the <code>MouseWheelScrollFactorChanged</code> event that occurs when the value of the {@link yfiles.view.CanvasComponent#mouseWheelScrollFactor} property has been
       * changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeMouseWheelScrollFactorChangedListener
       */
      addMouseWheelScrollFactorChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MouseWheelScrollFactorChanged</code> event that occurs when the value of the {@link yfiles.view.CanvasComponent#mouseWheelScrollFactor} property has
       * been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addMouseWheelScrollFactorChangedListener
       */
      removeMouseWheelScrollFactorChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets or sets the auto drag insets.
       * <p>
       * If the mouse is being dragged within the insets of the control specified using this property, the control will
       * automatically scroll.
       * </p>
       * @see yfiles.view.CanvasComponent#autoDrag
       * @default '10'
       * @type {yfiles.geometry.Insets}
       */
      autoDragInsets:yfiles.geometry.Insets;
      /**
       * Adds the given listener for the <code>PrepareRenderContext</code> event.
       * <br />
       * Event that will be triggered before the visual tree is painted to prepare the {@link yfiles.view.IRenderContext}.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removePrepareRenderContextListener
       */
      addPrepareRenderContextListener(listener:(sender:Object,evt:yfiles.view.PrepareRenderContextEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PrepareRenderContext</code> event.
       * <br />
       * Event that will be triggered before the visual tree is painted to prepare the {@link yfiles.view.IRenderContext}.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addPrepareRenderContextListener
       */
      removePrepareRenderContextListener(listener:(sender:Object,evt:yfiles.view.PrepareRenderContextEventArgs)=>void):void;
      /**
       * Gets or sets the InputModeContext property.
       * <p>
       * This context object is passed to {@link yfiles.input.IInputMode} instances during {@link yfiles.input.IInputMode#install installation} and removal.
       * <br />
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.CanvasComponent#createInputModeContext}
       * will be called. Upon change the {@link yfiles.view.CanvasComponent#addInputModeChangedListener InputModeChanged}
       * event will be triggered.
       * </p>
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Adds the given listener for the <code>InputModeContextChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#inputModeContext} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#removeInputModeContextChangedListener
       */
      addInputModeContextChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>InputModeContextChanged</code> event that occurs when the {@link yfiles.view.CanvasComponent#inputModeContext} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#addInputModeContextChangedListener
       */
      removeInputModeContextChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets a value indicating whether navigation related command bindings are enabled.
       * <p>
       * By default this feature is enabled.
       * </p>
       * <p>
       * This property enables/disables the following commands:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.ICommand#INCREASE_ZOOM}</li>
       * <li>{@link yfiles.input.ICommand#ZOOM}</li>
       * <li>{@link yfiles.input.ICommand#DECREASE_ZOOM}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_UP}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_LEFT}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_RIGHT}</li>
       * </ul>
       * @default true
       * @type {boolean}
       */
      navigationCommandsEnabled:boolean;
      /**
       * Gets the resources for this instance.
       * <p>
       * Resources are named instances of objects that can be retrieved by other parts of the infrastructure that deal with this
       * instance, either directly or indirectly.
       * </p>
       * @type {yfiles.collections.IMap.<string,Object>}
       */
      resources:yfiles.collections.IMap<string,Object>;
      /**
       * Gets or sets the interval of the size changed timer.
       * @see yfiles.view.CanvasComponent#addSizeChangedListener
       * @see yfiles.view.CanvasComponent#sizeChangedDetection
       * @static
       * @type {number}
       */
      static sizeChangedTimerInterval:number;
      /**
       * Adds the given listener for the <code>SizeChanged</code> event that occurs when the size of this Control is changed.
       * @param listener The listener to add.
       * @see yfiles.view.CanvasComponent#sizeChangedDetection
       * @see yfiles.view.SizeChangedDetectionMode
       * @see yfiles.view.CanvasComponent#removeSizeChangedListener
       */
      addSizeChangedListener(listener:(sender:Object,evt:yfiles.view.SizeChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>SizeChanged</code> event that occurs when the size of this Control is changed.
       * @param listener The listener to remove.
       * @see yfiles.view.CanvasComponent#sizeChangedDetection
       * @see yfiles.view.SizeChangedDetectionMode
       * @see yfiles.view.CanvasComponent#addSizeChangedListener
       */
      removeSizeChangedListener(listener:(sender:Object,evt:yfiles.view.SizeChangedEventArgs)=>void):void;
      /**
       * Gets or sets how size changes of this {@link yfiles.view.CanvasComponent} will be detected.
       * <p>
       * Since there is no size changed event in HTML, {@link yfiles.view.CanvasComponent} comes with its own methods to detect whether
       * the size of its Div element has changed. This is needed to update its Size accordingly.
       * </p>
       * <p>
       * The preferred and default mode is {@link yfiles.view.SizeChangedDetectionMode#SENSOR}. If that is not available, {@link yfiles.view.SizeChangedDetectionMode#TIMER}
       * is used as fall back instead.
       * </p>
       * <p>
       * If the Div of this control is removed from the DOM and the current mode is {@link yfiles.view.SizeChangedDetectionMode#TIMER},
       * the mode is automatically changed to
       * {@link yfiles.view.SizeChangedDetectionMode#NONE}. Thus, if the element is later re-added to the DOM, the timer mode must be set
       * again.
       * </p>
       * @type {yfiles.view.SizeChangedDetectionMode}
       */
      sizeChangedDetection:yfiles.view.SizeChangedDetectionMode;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class sets up an {@link yfiles.view.IRenderContext} object for exporting the contents of a {@link yfiles.view.CanvasComponent} (see
     * Setup).
     * <p>
     * Also it provides ready-to-use methods to export the canvas contents to a standalone SVG.
     * </p>
     * <ul>
     * <li>It is possible to export only a part of the contents, see {@link yfiles.view.SvgExport#worldBounds}.</li>
     * <li>
     * The size of the target graphics is controlled by the {@link yfiles.view.SvgExport#scale}. For a given target width or height, the apropriate scale can be
     * calculated using {@link yfiles.view.SvgExport#calculateScaleForWidth} and {@link yfiles.view.SvgExport#calculateScaleForHeight}.
     * </li>
     * </ul>
     * @class
     */
    export interface SvgExport extends Object{}
    export class SvgExport {
      /**
       * Creates a new {@link yfiles.view.SvgExport} instance for the given world bounds and scale.
       * @param worldBounds the bounds of the content to export, see {@link #worldBounds}
       * @param [scale=1.0] the scale, see {@link #scale}
       */
      constructor(worldBounds:yfiles.geometry.Rect,scale?:number);
      /**
       * Calculates the value to set the {@link yfiles.view.SvgExport#scale} to in order to achieve the given target height (without {@link yfiles.view.SvgExport#margins}).
       * <p>
       * This method returns 1.0 if the {@link yfiles.view.SvgExport#worldBounds} height is zero or negative.
       * </p>
       * @param height the height of the target image
       * @returns the scale value to use
       */
      calculateScaleForHeight(height:number):number;
      /**
       * Calculates the value to set the {@link yfiles.view.SvgExport#scale} to in order to achieve the given target width (without {@link yfiles.view.SvgExport#margins}).
       * <p>
       * This method returns 1.0 if the {@link yfiles.view.SvgExport#worldBounds} width is zero or negative.
       * </p>
       * @param width the width of the target image
       * @returns the scale value to use
       */
      calculateScaleForWidth(width:number):number;
      /**
       * Returns an initialized {@link yfiles.view.IRenderContext} and changes the given transform and the clip bounds such that a part
       * of the given {@link yfiles.view.CanvasComponent} instance can be exported to an image.
       * @param canvas The {@link } instance to export.
       * @returns An {@link } to use.
       */
      createRenderContext(canvas:yfiles.view.CanvasComponent):yfiles.view.IRenderContext;
      /**
       * Encodes the given SVG document as Data URI.
       * <p>
       * This method escapes non-Latin1 characters, does a base64 encoding, and returns the result as data URI.
       * </p>
       * @param svgXml A string containing the SVG document.
       * @returns A data URI of the SVG document.
       * @static
       */
      static encodeSvgDataUrl(svgXml:string):string;
      /**
       * Exports the contents of the {@link yfiles.view.CanvasComponent} to a single, standalone SVG element.
       * <p>
       * The asynchronous version of this method,
       * {@link yfiles.view.SvgExport#exportSvgAsync}, is capable of encoding and inlining images that are embedded in the SVG.
       * </p>
       * @param canvas The canvas to be exported.
       * @returns An SVG element with the canvas contents.
       */
      exportSvg(canvas:yfiles.view.CanvasComponent):Element;
      /**
       * Exports the contents of the {@link yfiles.view.CanvasComponent} to a single, standalone SVG element.
       * <p>
       * In addition, images are inlined into the SVG using base64 encoding if {@link yfiles.view.SvgExport#encodeImagesBase64} is set to <code>true</code>. Image inlining can be controlled
       * for each image individually by overwriting the predicate method {@link yfiles.view.SvgExport#shouldEncodeImageBase64}.
       * </p>
       * <p>
       * SVG image elements are replaced with a copy of the referenced SVG element if {@link yfiles.view.SvgExport#inlineSvgImages} is set to <code>true</code>. Image encoding can be
       * controlled for each image individually by overwriting the predicate method {@link yfiles.view.SvgExport#shouldInlineSvgImage}.
       * </p>
       * @param canvas The canvas to be exported.
       * @returns 
       */
      exportSvgAsync(canvas:yfiles.view.CanvasComponent):Promise<Element>;
      /**
       * Serializes the given SVG element to a string.
       * <p>
       * This method adds SVG namespace and xlink namespace declarations and replaces <code>href</code> with <code>xlink:href</code>.
       * </p>
       * @param svg The SVG element to export.
       * @returns A string representation of the SVG.
       * @static
       */
      static exportSvgString(svg:Element):string;
      /**
       * Returns a suitable clip to use for the visual for this exporter.
       * @returns The clip.
       */
      getClip():yfiles.geometry.Rect;
      /**
       * Prepares the imported SVG element before it is placed in the DOM as a replacement for the original image.
       * <p>
       * This method can be used to modify the element is placed in the DOM. This specific implementation copies the values of
       * the x, y, width, height and preserveAspectRatio attributes from the original image element to the imported SVG element.
       * In addition, the SVG is wrapped in a <code>g</code> element to support the <code>transform</code> of the original image.
       * </p>
       * @param importedSvg The root element of the SVG file that's referenced by the original image element.
       * @param originalImage The original image element that will be replaced.
       * @returns The element that's actually placed in the DOM as a replacement of the <code>originalImage</code>.
       * @see yfiles.view.SvgExport#inlineSvgImages
       * @see yfiles.view.SvgExport#exportSvgAsync
       * @see yfiles.view.SvgExport#shouldInlineSvgImage
       * @protected
       */
      prepareInlinedImage(importedSvg:Element,originalImage:SVGImageElement):Element;
      /**
       * Specifies whether or not an image should be encoded to base64.
       * <p>
       * This implementation returns the value of {@link yfiles.view.SvgExport#encodeImagesBase64} for all values of <code>image</code>.
       * </p>
       * @param image The image to be encoded.
       * @returns Whether the given image should be encoded.
       * @protected
       */
      shouldEncodeImageBase64(image:SVGImageElement):boolean;
      /**
       * Specifies whether or not an SVG image should be inlined.
       * <p>
       * This implementation returns the value of {@link yfiles.view.SvgExport#inlineSvgImages} for all values of <code>image</code>.
       * </p>
       * @param image The image to be inlined.
       * @returns Whether the given image should be inlined.
       * @protected
       */
      shouldInlineSvgImage(image:SVGImageElement):boolean;
      /**
       * Gets or sets the bounds of the content to export in world coordinates.
       * @type {yfiles.geometry.Rect}
       */
      worldBounds:yfiles.geometry.Rect;
      /**
       * Gets or sets the scale for the export.
       * <p>
       * A scale of <code>1</code> preserves the original size, a scale of <code>0.5</code> results in a target image with half the original size and so
       * on.
       * </p>
       * <p>
       * This value has to be strictly greater than <code>0</code> and finite. Its default value is <code>1.0</code>
       * </p>
       * @type {number}
       */
      scale:number;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasContext#zoom} property to use during the creation of the visualization.
       * <p>
       * In contrast to the {@link yfiles.view.SvgExport#scale} property, which works on the output graphics, this property determines what zoom value is to be
       * assumed on the canvas when creating the visual. This can affect the rendering of zoom dependent visuals.
       * </p>
       * <p>
       * This value has to be strictly greater than <code>0</code>. Its default value is <code>1.0</code>
       * </p>
       * @type {number}
       */
      zoom:number;
      /**
       * Gets or sets the margins for the exported image.
       * <p>
       * The margins are added to the content. This means that an image with non-zero margins is larger than the {@link yfiles.view.SvgExport#worldBounds} even if the {@link yfiles.view.SvgExport#scale} is
       * <code>1.0</code>. The margins are not scaled. They are interpreted to be in units (pixels for bitmaps) for the resulting image. The
       * default is an empty margin instance.
       * </p>
       * @type {yfiles.geometry.Insets}
       */
      margins:yfiles.geometry.Insets;
      /**
       * Gets the resulting width of the target image with respect to {@link yfiles.view.SvgExport#worldBounds}, {@link yfiles.view.SvgExport#scale} and {@link yfiles.view.SvgExport#margins}.
       * @type {number}
       */
      viewWidth:number;
      /**
       * Gets the resulting height of the target image with respect to {@link yfiles.view.SvgExport#worldBounds}, {@link yfiles.view.SvgExport#scale} and {@link yfiles.view.SvgExport#margins}.
       * @type {number}
       */
      viewHeight:number;
      /**
       * Determines whether children of the {@link yfiles.view.CanvasComponent}'s defs section that aren't known to the {@link yfiles.view.SvgDefsManager}
       * should be copied to the exported SVG.
       * @type {boolean}
       */
      copyDefsElements:boolean;
      /**
       * Determines whether all images are encoded to Base64 in {@link yfiles.view.SvgExport#exportSvgAsync}.
       * <p>
       * Override {@link yfiles.view.SvgExport#shouldEncodeImageBase64} to control the encoding for each image separately.
       * </p>
       * @see yfiles.view.SvgExport#exportSvgAsync
       * @see yfiles.view.SvgExport#shouldEncodeImageBase64
       * @type {boolean}
       */
      encodeImagesBase64:boolean;
      /**
       * Determines whether all SVG images should be inlined in the SVG element in {@link yfiles.view.SvgExport#exportSvgAsync}.
       * <p>
       * Override {@link yfiles.view.SvgExport#shouldInlineSvgImage} to control the inlining for each image separately.
       * </p>
       * @see yfiles.view.SvgExport#exportSvgAsync
       * @see yfiles.view.SvgExport#shouldInlineSvgImage
       * @type {boolean}
       */
      inlineSvgImages:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Helper class for {@link yfiles.view.CanvasComponent} that manages the <code>defs</code> section of the SVG document.
     * <p>
     * This class is used internally by {@link yfiles.view.CanvasComponent} and operates on instances of {@link yfiles.view.ISvgDefsCreator}.
     * </p>
     * @see yfiles.view.CanvasComponent#svgDefsManager
     * @class
     */
    export interface SvgDefsManager extends Object{}
    export class SvgDefsManager {
      /**
       * Initializes a new instance of the {@link yfiles.view.SvgDefsManager} class.
       * @param defsElement The defs element of the svg document that will be managed by this instance.
       */
      constructor(defsElement:SVGDefsElement);
      /**
       * Triggers the clean up for svg defs on the provided control.
       * @param canvas
       */
      cleanupDefs(canvas:yfiles.view.CanvasComponent):void;
      /**
       * Generates an ID that is globally unique among all {@link yfiles.view.SvgDefsManager} instances.
       * <p>
       * This ID should be used for the "id" attribute of DOM nodes.
       * </p>
       * @returns A globally unique ID.
       */
      generateUniqueDefsId():string;
      /**
       * Gets or sets the interval for the timer to trigger the cleanup event.
       * <p>
       * By default this is 10 seconds.
       * </p>
       * @type {yfiles.lang.TimeSpan}
       */
      cleanupTimerInterval:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the prefix to use for the {@link yfiles.view.SvgDefsManager#generateUniqueDefsId unique IDs}.
       * <p>
       * By default this is <code>ygc&lt;number></code>
       * </p>
       * @type {string}
       */
      customPrefix:string;
      /**
       * Yields the <code>defs</code> element this instance is managing.
       * @type {SVGDefsElement}
       */
      defs:SVGDefsElement;
      static $class:yfiles.lang.Class;
    }
    /**
     * A component for displaying and editing an {@link yfiles.graph.IGraph}.
     * <p>
     * This is a specialized {@link yfiles.view.CanvasComponent} that can hold, display, and edit an {@link yfiles.graph.IGraph} instance. It
     * provides access to the {@link yfiles.view.GraphComponent#graph} and {@link yfiles.view.GraphComponent#selection} instances.
     * </p>
     * <p>
     * This class has bindings for the following {@link yfiles.input.ICommand}s.
     * </p>
     * <ul>
     * <li>{@link yfiles.input.ICommand#FIT_GRAPH_BOUNDS} that invokes {@link yfiles.view.GraphComponent#fitGraphBounds}</li>
     * <li>{@link yfiles.input.ICommand#SET_CURRENT_ITEM} for setting the {@link yfiles.view.GraphComponent#currentItem}</li>
     * <li>{@link yfiles.input.ICommand#ZOOM_TO_CURRENT_ITEM} for adjusting the {@link yfiles.view.CanvasComponent#viewport} to the {@link yfiles.view.GraphComponent#currentItem}</li>
     * </ul>
     * <p>
     * The {@link yfiles.input.ICommand#ZOOM_TO_CURRENT_ITEM} is enabled, if {@link yfiles.view.CanvasComponent#navigationCommandsEnabled} is <code>true</code>.
     * </p>
     * <p>
     * The components that can be used to display and edit a model are discussed in the section
     * {@link @DGUIDE_PREFIX@/view_graphcontrol.html The Component for Displaying Graphs}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: GraphViewer, Demo.yFiles.Graph.Viewer</li>
     * <li>Demo: SimpleEditorForm, Demo.yFiles.Graph.SimpleEditor</li>
     * </ul>
     * @class
     * @extends {yfiles.view.CanvasComponent}
     */
    export interface GraphComponent extends yfiles.view.CanvasComponent{}
    export class GraphComponent {
      /**
       * Creates a new instance of {@link yfiles.view.GraphComponent} using the <code>div</code> element that is specified by the <code>selector</code>.
       * <p>
       * If the {@link yfiles.view.GraphComponent#graph} and {@link yfiles.view.GraphSelection} properties are not populated externally they will be initialized with default
       * values upon first access. Thus this instance can be used right away without any further initialization.
       * </p>
       * @param selector The CSS selector or id for an existing <code>div</code> element
       */
      constructor(selector:string);
      /**
       * Creates a new instance of {@link yfiles.view.GraphComponent} using the <code>div</code> element that is specified by the <code>selector</code>.
       * <p>
       * If the {@link yfiles.view.GraphComponent#graph} and {@link yfiles.view.GraphSelection} properties are not populated externally they will be initialized with default
       * values upon first access. Thus this instance can be used right away without any further initialization.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.selector The CSS selector or id for an existing <code>div</code> element
       * @param {yfiles.input.IInputMode} options.inputMode The single {@link yfiles.input.IInputMode} instance that shall be installed for this canvas.
       * <p>
       * This option sets the {@link yfiles.view.CanvasComponent#inputMode} property on the created object.
       * </p>
       * @param {yfiles.graph.IGraph} options.graph The graph that is displayed in this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#graph} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphClipboard} options.clipboard The {@link yfiles.graph.GraphClipboard} associated with this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#clipboard} property on the created object.
       * </p>
       * @param {yfiles.view.IGraphSelection} options.selection The selection model that is used for this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#selection} property on the created object.
       * </p>
       */
      constructor(options:{selector:string,inputMode?:yfiles.input.IInputMode,graph?:yfiles.graph.IGraph,clipboard?:yfiles.graph.GraphClipboard,selection?:yfiles.view.IGraphSelection});
      /**
       * Creates a new instance of the {@link yfiles.view.GraphComponent} in the given <code>div</code> element.
       * <p>
       * If the {@link yfiles.view.GraphComponent#graph} and {@link yfiles.view.GraphSelection} properties are not populated externally they will be initialized with default
       * values upon first access. Thus this instance can be used right away without any further initialization.
       * </p>
       * @param div The existing <code>div</code> element to use for rendering the canvas into.
       */
      constructor(div:HTMLDivElement);
      /**
       * Creates a new instance of the {@link yfiles.view.GraphComponent} in the given <code>div</code> element.
       * <p>
       * If the {@link yfiles.view.GraphComponent#graph} and {@link yfiles.view.GraphSelection} properties are not populated externally they will be initialized with default
       * values upon first access. Thus this instance can be used right away without any further initialization.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.div The existing <code>div</code> element to use for rendering the canvas into.
       * @param {yfiles.input.IInputMode} options.inputMode The single {@link yfiles.input.IInputMode} instance that shall be installed for this canvas.
       * <p>
       * This option sets the {@link yfiles.view.CanvasComponent#inputMode} property on the created object.
       * </p>
       * @param {yfiles.graph.IGraph} options.graph The graph that is displayed in this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#graph} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphClipboard} options.clipboard The {@link yfiles.graph.GraphClipboard} associated with this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#clipboard} property on the created object.
       * </p>
       * @param {yfiles.view.IGraphSelection} options.selection The selection model that is used for this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#selection} property on the created object.
       * </p>
       */
      constructor(options:{div:HTMLDivElement,inputMode?:yfiles.input.IInputMode,graph?:yfiles.graph.IGraph,clipboard?:yfiles.graph.GraphClipboard,selection?:yfiles.view.IGraphSelection});
      /**
       * Creates a new instance of the {@link yfiles.view.GraphComponent}.
       * <p>
       * If the {@link yfiles.view.GraphComponent#graph} and {@link yfiles.view.GraphSelection} properties are not populated externally they will be initialized with default
       * values upon first access. Thus this instance can be used right away without any further initialization. This constructor
       * creates a new <code>div</code> element that needs to be manually added to the DOM.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.input.IInputMode} options.inputMode The single {@link yfiles.input.IInputMode} instance that shall be installed for this canvas.
       * <p>
       * This option sets the {@link yfiles.view.CanvasComponent#inputMode} property on the created object.
       * </p>
       * @param {yfiles.graph.IGraph} options.graph The graph that is displayed in this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#graph} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphClipboard} options.clipboard The {@link yfiles.graph.GraphClipboard} associated with this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#clipboard} property on the created object.
       * </p>
       * @param {yfiles.view.IGraphSelection} options.selection The selection model that is used for this control.
       * <p>
       * This option sets the {@link yfiles.view.GraphComponent#selection} property on the created object.
       * </p>
       */
      constructor(options?:{inputMode?:yfiles.input.IInputMode,graph?:yfiles.graph.IGraph,clipboard?:yfiles.graph.GraphClipboard,selection?:yfiles.view.IGraphSelection});
      /**
       * Factory method for the {@link yfiles.view.GraphComponent#focusIndicatorManager} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.GraphComponent#focusIndicatorManager} property.
       * </p>
       * @returns a new instance of {@link }.
       * @protected
       */
      createFocusIndicatorManager():yfiles.view.FocusIndicatorManager<yfiles.graph.IModelItem>;
      /**
       * Factory method for the Graph property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.GraphComponent#graph} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createGraph():yfiles.graph.IGraph;
      /**
       * Creates the default instance to use if no custom instance has been set.
       * @returns A default {@link } instance.
       * @protected
       */
      createGraphClipboard():yfiles.graph.GraphClipboard;
      /**
       * Factory method for the {@link yfiles.view.GraphComponent#graphModelManager} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.GraphComponent#graphModelManager} property.
       * </p>
       * @returns a new instance of {@link }.
       * @protected
       */
      createGraphModelManager():yfiles.view.GraphModelManager;
      /**
       * Factory method for the {@link yfiles.view.GraphComponent#highlightIndicatorManager} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.GraphComponent#highlightIndicatorManager} property.
       * </p>
       * @returns a new instance of {@link #highlightIndicatorManager}.
       * @protected
       */
      createHighlightIndicatorManager():yfiles.view.HighlightIndicatorManager<yfiles.graph.IModelItem>;
      /**
       * Factory method for the Selection property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.GraphComponent#selection} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createSelection():yfiles.view.IGraphSelection;
      /**
       * Factory method for the {@link yfiles.view.GraphComponent#selectionIndicatorManager} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.view.GraphComponent#selectionIndicatorManager} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createSelectionIndicatorManager():yfiles.view.SelectionIndicatorManager<yfiles.graph.IModelItem>;
      /**
       * {@link yfiles.view.CanvasComponent#updateContentRect Updates the content rectangle} for the {@link yfiles.view.CanvasComponent#contentGroup} considering the provided insets and
       * {@link yfiles.view.CanvasComponent#fitContent adjusts the viewport to encompass the contents}.
       * @param [insets=null]
       */
      fitGraphBounds(insets?:yfiles.geometry.Insets):void;
      /**
       * Convenience method that runs a layouter on the graph of a given graph control and animates the transition.
       * <p>
       * An event is called once the animation has finished.
       * </p>
       * <p>
       * The implementation simply delegates to a properly configured instance of {@link yfiles.layout.LayoutExecutor}. In addition to
       * the default configuration, it enables {@link yfiles.layout.LayoutExecutor#animateViewport}, {@link yfiles.layout.LayoutExecutor#easedAnimation}, and {@link yfiles.layout.LayoutExecutor#updateContentRect}. For more fine-grained control of the animation, use {@link yfiles.layout.LayoutExecutor}
       * directly.
       * </p>
       * @param layout
       * @param [morphDuration=null]
       * @param [layoutData=null]
       * @returns 
       */
      morphLayout(layout:yfiles.layout.ILayoutAlgorithm,morphDuration?:yfiles.lang.TimeSpan,layoutData?:yfiles.layout.LayoutData):Promise<Object>;
      /**
       * Convenience method that runs a layouter on the graph of a given graph control and animates the transition.
       * <p>
       * An event is called once the animation has finished.
       * </p>
       * <p>
       * The implementation simply delegates to a properly configured instance of {@link yfiles.layout.LayoutExecutor}. In addition to
       * the default configuration, it enables {@link yfiles.layout.LayoutExecutor#animateViewport}, {@link yfiles.layout.LayoutExecutor#easedAnimation}, and {@link yfiles.layout.LayoutExecutor#updateContentRect}. For more fine-grained control of the animation, use {@link yfiles.layout.LayoutExecutor}
       * directly.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.layout
       * @param [options.morphDuration=null]
       * @param [options.layoutData=null]
       * @returns 
       */
      morphLayout(options:{layout:yfiles.layout.ILayoutAlgorithm,morphDuration?:yfiles.lang.TimeSpan,layoutData?:yfiles.layout.LayoutData}):Promise<Object>;
      /**
       * Gets or sets the 'current' item.
       * @default null
       * @type {yfiles.graph.IModelItem}
       */
      currentItem:yfiles.graph.IModelItem;
      /**
       * Adds the given listener for the <code>CurrentItemChanged</code> event that occurs when the {@link yfiles.view.GraphComponent#currentItem} property has changed its value.
       * @param listener The listener to add.
       * @see yfiles.view.GraphComponent#removeCurrentItemChangedListener
       */
      addCurrentItemChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CurrentItemChanged</code> event that occurs when the {@link yfiles.view.GraphComponent#currentItem} property has changed its value.
       * @param listener The listener to remove.
       * @see yfiles.view.GraphComponent#addCurrentItemChangedListener
       */
      removeCurrentItemChangedListener(listener:(sender:Object,evt:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the {@link yfiles.view.GraphComponent#selectionIndicatorManager} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphComponent#createSelectionIndicatorManager}
       * will be called.
       * </p>
       * @type {yfiles.view.SelectionIndicatorManager.<yfiles.graph.IModelItem>}
       */
      selectionIndicatorManager:yfiles.view.SelectionIndicatorManager<yfiles.graph.IModelItem>;
      /**
       * Gets or sets the {@link yfiles.view.GraphComponent#highlightIndicatorManager} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphComponent#createHighlightIndicatorManager}
       * will be called.
       * </p>
       * @type {yfiles.view.HighlightIndicatorManager.<yfiles.graph.IModelItem>}
       */
      highlightIndicatorManager:yfiles.view.HighlightIndicatorManager<yfiles.graph.IModelItem>;
      /**
       * Gets or sets the {@link yfiles.view.GraphComponent#focusIndicatorManager} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphComponent#createFocusIndicatorManager}
       * will be called.
       * </p>
       * @type {yfiles.view.FocusIndicatorManager.<yfiles.graph.IModelItem>}
       */
      focusIndicatorManager:yfiles.view.FocusIndicatorManager<yfiles.graph.IModelItem>;
      /**
       * Gets or sets the {@link yfiles.view.GraphComponent#graphModelManager} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphComponent#createGraphModelManager}
       * will be called.
       * </p>
       * @type {yfiles.view.GraphModelManager}
       */
      graphModelManager:yfiles.view.GraphModelManager;
      /**
       * Gets or sets the graph that is displayed in this control.
       * <p>
       * Normally the {@link yfiles.view.GraphSelection} property must be adjusted accordingly if the graph instance is replaced. Also
       * depending on the {@link yfiles.input.IInputMode} implementation the instances used in this component might need to be replaced
       * or adjusted, too. If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphComponent#createGraph}
       * will be called.
       * </p>
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Adds the given listener for the <code>GraphChanged</code> event that occurs when the {@link yfiles.view.GraphComponent#graph} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.GraphComponent#removeGraphChangedListener
       */
      addGraphChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IGraph>)=>void):void;
      /**
       * Removes the given listener for the <code>GraphChanged</code> event that occurs when the {@link yfiles.view.GraphComponent#graph} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.GraphComponent#addGraphChangedListener
       */
      removeGraphChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IGraph>)=>void):void;
      /**
       * Gets or sets the selection model that is used for this control.
       * <p>
       * The selection model instance needs to be adjusted, normally if the {@link yfiles.view.GraphComponent#graph} instance is changed. If the field has not yet been
       * initialized upon first access, the factory method {@link yfiles.view.GraphComponent#createSelection} will be called.
       * </p>
       * @type {yfiles.view.IGraphSelection}
       */
      selection:yfiles.view.IGraphSelection;
      /**
       * Gets or sets the {@link yfiles.graph.GraphClipboard} associated with this control.
       * <p>
       * If no instance has been configured with this control, a default implementation will be used.
       * </p>
       * @type {yfiles.graph.GraphClipboard}
       */
      clipboard:yfiles.graph.GraphClipboard;
      static $class:yfiles.lang.Class;
    }
    /**
     * Convenience component implementation that shows an overview of an {@link yfiles.graph.IGraph} contained in another {@link yfiles.view.GraphOverviewComponent#graphComponent}.
     * <p>
     * This component uses a {@link yfiles.view.GraphOverviewComponent#graphVisualCreator} to display the graph in a preview mode and an {@link yfiles.view.GraphOverviewComponent#overviewInputMode} to allow for easily navigating in the {@link yfiles.view.GraphOverviewComponent#graphComponent}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleEditorForm, Demo.yFiles.Graph.SimpleEditor</li>
     * </ul>
     * @class
     * @extends {yfiles.view.CanvasComponent}
     */
    export interface GraphOverviewComponent extends yfiles.view.CanvasComponent{}
    export class GraphOverviewComponent {
      /**
       * Creates a new instance of {@link yfiles.view.GraphOverviewComponent} using the <code>div</code> element that is specified by the <code>selector</code> and
       * that is optionally bound to the provided {@link yfiles.view.GraphOverviewComponent#graphComponent}.
       * <p>
       * If no <code>graphControl</code> is provided, in order for this instance to be useful, either the {@link yfiles.view.GraphOverviewComponent#graphComponent} or the {@link yfiles.view.GraphOverviewComponent#graph} property need to be set.
       * </p>
       * @param [graphComponent=null] The {@link #graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
       * @param selector The CSS selector or id for an existing <code>div</code> element
       */
      constructor(selector:string,graphComponent?:yfiles.view.GraphComponent);
      /**
       * Creates a new instance of {@link yfiles.view.GraphOverviewComponent} using the <code>div</code> element that is specified by the <code>selector</code> and
       * that is optionally bound to the provided {@link yfiles.view.GraphOverviewComponent#graphComponent}.
       * <p>
       * If no <code>graphControl</code> is provided, in order for this instance to be useful, either the {@link yfiles.view.GraphOverviewComponent#graphComponent} or the {@link yfiles.view.GraphOverviewComponent#graph} property need to be set.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.graphComponent=null] The {@link #graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
       * @param options.selector The CSS selector or id for an existing <code>div</code> element
       * @param {boolean} options.svgRendering A value indicating whether the graph is rendered using SVG rendering.
       * <p>
       * This option sets the {@link yfiles.view.GraphOverviewComponent#svgRendering} property on the created object.
       * </p>
       */
      constructor(options:{selector:string,graphComponent?:yfiles.view.GraphComponent,svgRendering?:boolean});
      /**
       * Creates a new instance of {@link yfiles.view.GraphOverviewComponent} using the <code>div</code> element that is optionally bound to the
       * provided {@link yfiles.view.GraphOverviewComponent#graphComponent}.
       * <p>
       * If no <code>graphControl</code> is provided, in order for this instance to be useful, either the {@link yfiles.view.GraphOverviewComponent#graphComponent} or the {@link yfiles.view.GraphOverviewComponent#graph} property need to be set.
       * </p>
       * @param div The div element to use for holding the canvas.
       * @param [graphComponent=null] The optional graph control to {@link #graphComponent bind} this instance to. If omitted, the property needs to be set to initialize the
       * component.
       */
      constructor(div:HTMLDivElement,graphComponent?:yfiles.view.GraphComponent);
      /**
       * Creates a new instance of {@link yfiles.view.GraphOverviewComponent} using the <code>div</code> element that is optionally bound to the
       * provided {@link yfiles.view.GraphOverviewComponent#graphComponent}.
       * <p>
       * If no <code>graphControl</code> is provided, in order for this instance to be useful, either the {@link yfiles.view.GraphOverviewComponent#graphComponent} or the {@link yfiles.view.GraphOverviewComponent#graph} property need to be set.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.div The div element to use for holding the canvas.
       * @param [options.graphComponent=null] The optional graph control to {@link #graphComponent bind} this instance to. If omitted, the property needs to be set to initialize the
       * component.
       * @param {boolean} options.svgRendering A value indicating whether the graph is rendered using SVG rendering.
       * <p>
       * This option sets the {@link yfiles.view.GraphOverviewComponent#svgRendering} property on the created object.
       * </p>
       */
      constructor(options:{div:HTMLDivElement,graphComponent?:yfiles.view.GraphComponent,svgRendering?:boolean});
      /**
       * Creates a new instance that is optionally bound to the provided {@link yfiles.view.GraphOverviewComponent#graphComponent}.
       * <p>
       * This constructor creates a new {@link yfiles.view.CanvasComponent#div} element that needs to be added to the DOM manually. If no <code>graphControl</code> is provided, in
       * order for this instance to be useful, either the {@link yfiles.view.GraphOverviewComponent#graphComponent} or the {@link yfiles.view.GraphOverviewComponent#graph} property need to be set.
       * </p>
       * @param [graphComponent=null] The {@link #graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
       */
      constructor(graphComponent?:yfiles.view.GraphComponent);
      /**
       * Creates a new instance that is optionally bound to the provided {@link yfiles.view.GraphOverviewComponent#graphComponent}.
       * <p>
       * This constructor creates a new {@link yfiles.view.CanvasComponent#div} element that needs to be added to the DOM manually. If no <code>graphControl</code> is provided, in
       * order for this instance to be useful, either the {@link yfiles.view.GraphOverviewComponent#graphComponent} or the {@link yfiles.view.GraphOverviewComponent#graph} property need to be set.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.graphComponent=null] The {@link #graphComponent} to bind this instance to. If omitted, the property needs to be set to initialize the component.
       * @param {boolean} options.svgRendering A value indicating whether the graph is rendered using SVG rendering.
       * <p>
       * This option sets the {@link yfiles.view.GraphOverviewComponent#svgRendering} property on the created object.
       * </p>
       */
      constructor(options:{graphComponent?:yfiles.view.GraphComponent,svgRendering?:boolean});
      /**
       * Factory method that creates the {@link yfiles.view.IVisualCreator} that renders the preview of the graph in a HTML canvas
       * element.
       * @param graph The graph to draw.
       * @returns An {@link } that creates HTML canvas rendering.
       * @protected
       */
      createCanvasVisualCreator(graph:yfiles.graph.IGraph):yfiles.view.IVisualCreator;
      /**
       * Factory method that creates the {@link yfiles.view.IVisualCreator} that renders the preview of the graph in SVG.
       * @param graph The graph to draw.
       * @returns An {@link } that creates SVG.
       * @protected
       */
      createSvgVisualCreator(graph:yfiles.graph.IGraph):yfiles.view.IVisualCreator;
      /**
       * Gets or sets a value indicating whether the graph is rendered using SVG rendering.
       * @default false
       * @type {boolean}
       */
      svgRendering:boolean;
      /**
       * Gets the {@link yfiles.styles.GraphOverviewSvgVisualCreator} that is used to render the over view graph.
       * @type {yfiles.view.IVisualCreator}
       */
      graphVisualCreator:yfiles.view.IVisualCreator;
      /**
       * Gets or sets the {@link yfiles.input.OverviewInputMode} that is used to navigate the client control.
       * <p>
       * Setting this property will automatically update the {@link yfiles.input.OverviewInputMode#canvasComponent} property of the new {@link yfiles.input.OverviewInputMode}.
       * </p>
       * <p>
       * This property is merely a convenience alias for the {@link yfiles.view.CanvasComponent#inputMode} property. Setting either this property or the {@link yfiles.view.CanvasComponent#inputMode} property has the
       * same effect if the input mode is an {@link yfiles.input.OverviewInputMode}. On the other hand, if an input mode is set that is
       * not an {@link yfiles.input.OverviewInputMode}, then the value of this property is <code>null</code>
       * </p>
       * @type {yfiles.input.OverviewInputMode}
       */
      overviewInputMode:yfiles.input.OverviewInputMode;
      /**
       * Gets or sets the graph that is rendered in the overview.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Gets or sets the client component, that will be used to retrieve the graph from and that will be used for navigating.
       * @type {yfiles.view.GraphComponent}
       */
      graphComponent:yfiles.view.GraphComponent;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class describes a grid by providing the grid's origin as well as the horizontal and vertical spacing between the
     * grid points.
     * <p>
     * It is used by the {@link yfiles.input.IGridConstraintProvider.<T>} as well as by the <code>GridSnapResultVisualCreator&lt;T></code>
     * </p>
     * @see yfiles.input.IGridConstraintProvider.<T>
     * @class
     */
    export interface GridInfo extends Object{}
    export class GridInfo {
      /**
       * Creates a new instance using the given horizontal and vertical grid spacing as well as the given origin.
       * @param horizontalSpacing The horizontal spacing between the grid points.
       * @param verticalSpacing The vertical spacing between the grid points.
       * @param origin The canonic origin of the grid.
       */
      constructor(horizontalSpacing:number,verticalSpacing:number,origin:yfiles.geometry.Point);
      /**
       * Creates a new instance using "25" as the grid spacing.
       * @param [spacing=25] The horizontal and vertical spacing between the grid points.
       */
      constructor(spacing?:number);
      /**
       * Gets or sets the horizontal spacing between grid points in the world coordinate system.
       * @default 25
       * @type {number}
       */
      horizontalSpacing:number;
      /**
       * Gets or sets the vertical spacing between grid points in the world coordinate system.
       * @default 25
       * @type {number}
       */
      verticalSpacing:number;
      /**
       * Gets or sets the canonic origin of the grid.
       * @default '0,0'
       * @type {yfiles.geometry.Point}
       */
      origin:yfiles.geometry.Point;
      static $class:yfiles.lang.Class;
    }
    /**
     * This interface describes the context for many operations performed on items in a {@link yfiles.view.ICanvasContext#canvasComponent}.
     * <p>
     * This interface extends the {@link yfiles.graph.ILookup} interface to provide {@link yfiles.graph.ILookup#lookup dynamic lookup capability}.
     * </p>
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface ICanvasContext extends Object,yfiles.graph.ILookup{
      /**
       * Adds an element to the defs section of the document, if it has not been added yet.
       * @param defsSupport The instance that is used to create and update the element and to query if the element is still referenced
       * @returns The id of the element with which it can be referenced using an url reference.
       * @abstract
       */
      getDefsId(defsSupport:yfiles.view.ISvgDefsCreator):string;
      /**
       * Returns the current zoom level that is the ratio of world coordinates to screen pixels.
       * @abstract
       * @type {number}
       */
      zoom:number;
      /**
       * The radius for hit tests and marquee selection tests in world coordinates.
       * <p>
       * This value already takes the zoom level into account.
       * </p>
       * @abstract
       * @type {number}
       */
      hitTestRadius:number;
      /**
       * Yields the Canvas that is associated with this context.
       * <p>
       * This value may be null in those cases where no control can be associated with the current context.
       * </p>
       * @abstract
       * @type {yfiles.view.CanvasComponent}
       */
      canvasComponent:yfiles.view.CanvasComponent;
    }
    var ICanvasContext:{
      /**
       * Provides a default immutable {@link yfiles.view.ICanvasContext} with the given values.
       * <p>
       * If the values for zoom or hitTestRadius are not specified, the values will be obtained from the given canvas.
       * </p>
       * @param canvas the canvas
       * @param [zoom=0] the zoom level
       * @param [hitTestRadius=-1] the hit test radius
       * @returns 
       * @static
       */
      createCanvasContext?(canvas:yfiles.view.CanvasComponent,zoom?:number,hitTestRadius?:number):yfiles.view.ICanvasContext;
      /**
       * Provides a default immutable {@link yfiles.view.ICanvasContext} with the given values.
       * <p>
       * If the values for zoom or hitTestRadius are not specified, the values will be obtained from the given canvas.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.canvas the canvas
       * @param [options.zoom=0] the zoom level
       * @param [options.hitTestRadius=-1] the hit test radius
       * @returns 
       * @static
       */
      createCanvasContext?(options:{canvas:yfiles.view.CanvasComponent,zoom?:number,hitTestRadius?:number}):yfiles.view.ICanvasContext;
      /**
       * Provides a default immutable context with a hit radius of 3 and a zoom level of 1.
       * @const
       * @static
       * @type {yfiles.view.ICanvasContext}
       */
      DEFAULT?:yfiles.view.ICanvasContext;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is the interface implemented by the basic building blocks of a scene graph in a {@link yfiles.view.CanvasComponent}.
     * <p>
     * The interface is not meant to be implemented by application programmers. Instead, instances implementing this interface
     * can be retrieved from the factory methods in the canvas control.
     * </p>
     * @interface
     */
    export interface ICanvasObject extends Object{
      /**
       * Moves this instance exactly behind the reference item in the referenced item's group rendering list, so that it will be
       * rendered directly above the given reference.
       * <p>
       * This method returns the current instance so that calls can be chained, e.g. <code>object.Above(other).Raise()</code>.
       * </p>
       * <p>
       * Observe that this instance will move into the referenced item's group if needed.
       * </p>
       * @param reference
       * @returns <code>this</code> so that calls can be chained.
       * @abstract
       */
      above(reference:yfiles.view.ICanvasObject):yfiles.view.ICanvasObject;
      /**
       * Moves this instance exactly before the reference item in the referenced item's group rendering list, so that it will be
       * rendered directly behind the given reference.
       * <p>
       * This method returns the current instance so that calls can be chained, e.g. <code>object.Below(other).Lower()</code>.
       * </p>
       * <p>
       * Observe that this instance will move into the referenced item's group if needed.
       * </p>
       * @param reference
       * @returns <code>this</code> so that calls can be chained.
       * @abstract
       */
      below(reference:yfiles.view.ICanvasObject):yfiles.view.ICanvasObject;
      /**
       * Moves this instance one step closer to the beginning of the group rendering list, so that it will be rendered behind its
       * current predecessor.
       * <p>
       * As a convenience, this method returns the current instance so that calls can be conveniently chained, e.g.
       * <code>object.Lower().Lower()</code>
       * </p>
       * @returns <code>this</code> so that calls can be chained.
       * @abstract
       */
      lower():yfiles.view.ICanvasObject;
      /**
       * Moves this instance one step closer to the end of the group rendering list, so that it will be rendered on top of its
       * current successor.
       * <p>
       * As a convenience, this method returns the current instance so that calls can be conveniently chained, e.g.
       * <code>object.Raise().Raise().Raise()</code>
       * </p>
       * @returns <code>this</code> so that calls can be chained.
       * @abstract
       */
      raise():yfiles.view.ICanvasObject;
      /**
       * Removes this item from the scene graph it currently belongs to.
       * @abstract
       */
      remove():void;
      /**
       * Moves this instance to the beginning of the group rendering list, so that it will be rendered behind all other items in
       * the same group.
       * <p>
       * As a convenience, this method returns the current instance so that calls can be conveniently chained, e.g.
       * <code>object.ToBack().Raise().Raise()</code>
       * </p>
       * @returns <code>this</code> so that calls can be chained.
       * @abstract
       */
      toBack():yfiles.view.ICanvasObject;
      /**
       * Moves this instance to the end of the group rendering list, so that it will be rendered in front of all other items in
       * the same group.
       * <p>
       * As a convenience, this method returns the current instance so that calls can be conveniently chained, e.g.
       * <code>object.ToFront().Lower()</code>
       * </p>
       * @returns <code>this</code> so that calls can be chained.
       * @abstract
       */
      toFront():yfiles.view.ICanvasObject;
      /**
       * Gets or sets the visibility state of this instance.
       * <p>
       * If an ICanvasObject is not visible, it will not be considered for hit tests.
       * </p>
       * @abstract
       * @type {boolean}
       */
      visible:boolean;
      /**
       * Gets or sets a flag that indicates whether {@link yfiles.view.ICanvasObject} needs to be updated during the next call to
       * {@link yfiles.view.CanvasComponent#invalidate}.
       * <p>
       * Note that it is up to the implementation of the {@link yfiles.view.ICanvasObjectDescriptor} to determine the actual dirty state
       * of this instance. {@link yfiles.view.ICanvasObjectDescriptor#isDirty} can use any strategy to indicate dirtiness. The {@link yfiles.view.CanvasComponent}
       * will reset this flag during {@link yfiles.view.CanvasComponent#invalidate} to <code>false</code> if the visual is created or updated.
       * </p>
       * @abstract
       * @type {boolean}
       */
      dirty:boolean;
      /**
       * Gets or sets the parent group of this instance.
       * @abstract
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      group:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets or sets the user object associated with this instance.
       * <p>
       * The user object will be used by the {@link yfiles.view.ICanvasObject#descriptor} during rendering, hit testing, and bounds calculation
       * </p>
       * @abstract
       * @type {Object}
       */
      userObject:Object;
      /**
       * Gets or sets the descriptor instance that interprets the {@link yfiles.view.ICanvasObject#userObject} during rendering, hit testing, and bounds calculation.
       * @abstract
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      descriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets the next item in the list of rendered items that is owned by this element's {@link yfiles.view.ICanvasObject#group}.
       * <p>
       * The returned item will be rendered after this item is rendered. If this item is the last one in the list, this method
       * returns <code>null</code>.
       * </p>
       * @returns <code>null</code> if this item is last in the rendering list of its group or the next item otherwise.
       * @abstract
       * @type {yfiles.view.ICanvasObject}
       */
      nextSibling:yfiles.view.ICanvasObject;
      /**
       * Gets the previous item in the list of rendered items that is owned by this element's {@link yfiles.view.ICanvasObject#group}.
       * <p>
       * The returned item will be rendered directly before this item is rendered. If this item is the first one in the list,
       * this method returns <code>null</code>.
       * </p>
       * @returns <code>null</code> if this item is first in the rendering list of its group or the previous item otherwise.
       * @abstract
       * @type {yfiles.view.ICanvasObject}
       */
      previousSibling:yfiles.view.ICanvasObject;
    }
    var ICanvasObject:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Central interface used by the {@link yfiles.view.CanvasComponent} to render elements from the scene graph.
     * <p>
     * Implementations will be queried using the user object obtained from {@link yfiles.view.ICanvasObject#userObject} for the corresponding implementations of the
     * various aspects of the user object. Implementations of this interface need to be provided by the application programmer
     * to {@link yfiles.view.ICanvasObjectGroup}'s {@link yfiles.view.ICanvasObjectGroup#addChild} method.
     * </p>
     * @interface
     */
    export interface ICanvasObjectDescriptor extends Object{
      /**
       * Returns an implementation of {@link yfiles.view.IBoundsProvider} that can determine the visible bounds of the rendering of the
       * user object.
       * <p>
       * This method may always return the same instance. By contract, clients will not cache instances returned, but will always
       * call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
       * design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
       * each call to this method.
       * </p>
       * @param forUserObject the user object to query the bounds for
       * @returns an implementation or <code>null</code> if the bounds are not known, in which case the bounds are treated as infinite
       * @abstract
       */
      getBoundsProvider(forUserObject:Object):yfiles.view.IBoundsProvider;
      /**
       * Returns an implementation of {@link yfiles.input.IHitTestable} that can determine whether the rendering of the user object has
       * been hit at a given coordinate.
       * <p>
       * This method may always return the same instance. By contract, clients will not cache instances returned, but will always
       * call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
       * design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
       * each call to this method.
       * </p>
       * @param forUserObject the user object to do the hit testing for
       * @returns an implementation or <code>null</code> if the rendering cannot be hit tested
       * @abstract
       */
      getHitTestable(forUserObject:Object):yfiles.input.IHitTestable;
      /**
       * Returns an implementation of {@link yfiles.view.IVisibilityTestable} that can determine if the rendering of the user object
       * would be visible in a given context.
       * <p>
       * This method may always return the same instance. By contract clients will not cache instances returned but will always
       * call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
       * design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
       * each call to this method.
       * </p>
       * @param forUserObject the user object to query the bounds for
       * @returns an implementation or <code>null</code> if the bounds are not known, in which case the bounds are treated as infinite
       * @abstract
       */
      getVisibilityTestable(forUserObject:Object):yfiles.view.IVisibilityTestable;
      /**
       * Returns an implementation of {@link yfiles.view.IVisualCreator} that will create the {@link yfiles.view.Visual} tree for the user
       * object.
       * <p>
       * This method may always return the same instance. By contract, clients will not cache instances returned, but will always
       * call this method before the methods on the instance will be called. This contract enables the use of the fly-weight
       * design pattern. Implementations of this class can always return the same instance and reconfigure this instance upon
       * each call to this method.
       * </p>
       * @param forUserObject the user object to create a Visual for
       * @returns an implementation or <code>null</code> if nothing shall be rendered
       * @abstract
       */
      getVisualCreator(forUserObject:Object):yfiles.view.IVisualCreator;
      /**
       * Determines whether the given canvas object is deemed dirty and needs updating.
       * @param context The context that will be used for the update.
       * @param canvasObject The object to check.
       * @returns Whether an update is needed.
       * @abstract
       */
      isDirty(context:yfiles.view.ICanvasContext,canvasObject:yfiles.view.ICanvasObject):boolean;
    }
    var ICanvasObjectDescriptor:{
      /**
       * An implementation that returns an {@link yfiles.view.ICanvasObjectDescriptor} that does nothing.
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      VOID?:yfiles.view.ICanvasObjectDescriptor;
      /**
       * An implementation that treats {@link yfiles.view.ICanvasObject#userObject} as an instance of {@link yfiles.view.IVisualCreator} and considers the {@link yfiles.view.ICanvasObject#dirty} flag.
       * <p>
       * This descriptor tries to cast the {@link yfiles.view.ICanvasObject#userObject} to {@link yfiles.view.IVisualCreator} and use this instance to handle the visual
       * representation of the {@link yfiles.view.ICanvasObject}.
       * </p>
       * <p>
       * If the {@link yfiles.view.ICanvasObject#userObject} does not implement {@link yfiles.view.IVisualCreator} nothing will be displayed.
       * </p>
       * <p>
       * This descriptor will respect the {@link yfiles.view.ICanvasObject#dirty} flag.
       * </p>
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      DYNAMIC_DIRTY_INSTANCE?:yfiles.view.ICanvasObjectDescriptor;
      /**
       * A descriptor that treats {@link yfiles.view.ICanvasObject#userObject} as an instance of {@link yfiles.view.IVisualCreator} and ignores the {@link yfiles.view.ICanvasObject#dirty} flag.
       * <p>
       * This descriptor tries to cast the {@link yfiles.view.ICanvasObject#userObject} to {@link yfiles.view.IVisualCreator} and use this instance to handle the visual
       * representation of the {@link yfiles.view.ICanvasObject}.
       * </p>
       * <p>
       * If the {@link yfiles.view.ICanvasObject#userObject} does not implement {@link yfiles.view.IVisualCreator} nothing will be displayed.
       * </p>
       * <p>
       * This descriptor will always {@link yfiles.view.IVisualCreator#updateVisual update the visual} and thus ignore the {@link yfiles.view.ICanvasObject#dirty} flag.
       * </p>
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      ALWAYS_DIRTY_INSTANCE?:yfiles.view.ICanvasObjectDescriptor;
      /**
       * A descriptor that treats {@link yfiles.view.ICanvasObject#userObject} as an instance of {@link yfiles.graph.ILookup} to obtain suitable interface implementations and
       * considers the {@link yfiles.view.ICanvasObject#dirty} flag.
       * <p>
       * This descriptor tries to cast the {@link yfiles.view.ICanvasObject#userObject} to {@link yfiles.graph.ILookup} and obtain the appropriate interface implementations for its
       * methods from it.
       * </p>
       * <p>
       * If the {@link yfiles.view.ICanvasObject#userObject} does not implement {@link yfiles.graph.ILookup} nothing will be displayed.
       * </p>
       * <p>
       * This descriptor will respect the {@link yfiles.view.ICanvasObject#dirty} flag.
       * </p>
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      DYNAMIC_DIRTY_LOOKUP?:yfiles.view.ICanvasObjectDescriptor;
      /**
       * A descriptor that treats {@link yfiles.view.ICanvasObject#userObject} as an instance of {@link yfiles.graph.ILookup} to obtain suitable interface implementations and
       * ignores the {@link yfiles.view.ICanvasObject#dirty} flag.
       * <p>
       * This descriptor tries to cast the {@link yfiles.view.ICanvasObject#userObject} to {@link yfiles.graph.ILookup} and obtain the appropriate interface implementations for its
       * methods from it.
       * </p>
       * <p>
       * If the {@link yfiles.view.ICanvasObject#userObject} does not implement {@link yfiles.graph.ILookup} nothing will be displayed.
       * </p>
       * <p>
       * This descriptor will always {@link yfiles.view.IVisualCreator#updateVisual update the Visual} and thus ignore the {@link yfiles.view.ICanvasObject#dirty} flag.
       * </p>
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      ALWAYS_DIRTY_LOOKUP?:yfiles.view.ICanvasObjectDescriptor;
      /**
       * A descriptor that treats {@link yfiles.view.ICanvasObject#userObject} as Visual to use directly.
       * <p>
       * If the {@link yfiles.view.ICanvasObject#userObject} is not a Visual nothing will be displayed.
       * </p>
       * <p>
       * This descriptor ignores the {@link yfiles.view.ICanvasObject#dirty} flag.
       * </p>
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      VISUAL?:yfiles.view.ICanvasObjectDescriptor;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is the interface implemented by the composite building blocks of a scene graph in a {@link yfiles.view.CanvasComponent}.
     * <p>
     * It is meant to group multiple {@link yfiles.view.ICanvasObject} instances as a logic unit. The interface is not meant to be
     * implemented by application programmers. Instances implementing this interface can be retrieved from the factory methods
     * in the canvas control. The interface extends the {@link yfiles.collections.IListEnumerable.<T>} interface to allow for easy iteration and determination of the size of
     * the group.
     * </p>
     * @interface
     * @implements {yfiles.view.ICanvasObject}
     * @implements {yfiles.collections.IListEnumerable.<yfiles.view.ICanvasObject>}
     */
    export interface ICanvasObjectGroup extends Object,yfiles.view.ICanvasObject,yfiles.collections.IListEnumerable<yfiles.view.ICanvasObject>{
      /**
       * Adds a child element to the scene graph as a child of this group.
       * <p>
       * The descriptor will be queried for the various rendering related implementations for the given <code>userObject</code> at rendering
       * time.
       * </p>
       * @param userObject The user object to associate with this child element. This object will be passed to the <code>descriptor</code>'s methods.
       * @param descriptor An implementation of the {@link } interface that will be passed the <code>userObject</code> to provide the
       * various implementations that are used during rendering.
       * @returns A handle that can be used to control the rendering order and to later {@link #remove remove} the element from the scene graph, again.
       * @see yfiles.view.ICanvasObjectDescriptor
       * @see yfiles.view.ICanvasObjectDescriptor#DYNAMIC_DIRTY_INSTANCE
       * @abstract
       */
      addChild(userObject:Object,descriptor:yfiles.view.ICanvasObjectDescriptor):yfiles.view.ICanvasObject;
      /**
       * Adds a new {@link yfiles.view.ICanvasObjectGroup} to this group in the current scene graph.
       * <p>
       * This can be used to build groups of {@link yfiles.view.ICanvasObject} instances that can be moved within the scene graph or
       * whose {@link yfiles.view.ICanvasObject#visible visibility} can be controlled easily. The group will have a <code>null</code> {@link yfiles.view.ICanvasObject#userObject}.
       * </p>
       * @returns A handle to the newly created group that can be used to control the rendering order and to later {@link #remove remove} the group from
       * the scene graph, again.
       * @abstract
       */
      addGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Returns the first item in the rendering list of this group or <code>null</code>.
       * <p>
       * This is the item that will be rendered first, i.e. behind all of the other items in the rendering list. In order to
       * iterate over all immediate children, use the following idiom:
       * </p>
       * @see yfiles.collections.IListEnumerable.<T>
       * @see yfiles.collections.IEnumerable.<T>#getEnumerator
       * @abstract
       * @type {yfiles.view.ICanvasObject}
       */
      firstChild:yfiles.view.ICanvasObject;
      /**
       * Returns the last item in the rendering list of this group or <code>null</code>.
       * <p>
       * This is the item that will be on top of all other items in the list. In order to iterate over all immediate children in
       * reverse order, use the following idiom:
       * </p>
       * @abstract
       * @type {yfiles.view.ICanvasObject}
       */
      lastChild:yfiles.view.ICanvasObject;
    }
    var ICanvasObjectGroup:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface for objects that can install a visual representation of a highlight decoration of an item in the model
     * displayed in a canvas.
     * <p>
     * This interface is a tagging sub interface of the {@link yfiles.view.ICanvasObjectInstaller}. This interface is used for {@link yfiles.graph.ILookup#lookup}
     * operations.
     * </p>
     * @see yfiles.view.HighlightIndicatorManager.<T>
     * @see yfiles.view.ISelectionIndicatorInstaller
     * @interface
     * @implements {yfiles.view.ICanvasObjectInstaller}
     */
    export interface IHighlightIndicatorInstaller extends Object,yfiles.view.ICanvasObjectInstaller{
    }
    var IHighlightIndicatorInstaller:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface for objects that can install a visual focus indicator decoration of an item in the model displayed in a
     * canvas.
     * <p>
     * This interface is a tagging sub interface of the {@link yfiles.view.ICanvasObjectInstaller}. This interface is used for {@link yfiles.graph.ILookup#lookup}
     * operations.
     * </p>
     * @see yfiles.view.FocusIndicatorManager.<T>
     * @see yfiles.view.ISelectionIndicatorInstaller
     * @see yfiles.view.IHighlightIndicatorInstaller
     * @interface
     * @implements {yfiles.view.ICanvasObjectInstaller}
     */
    export interface IFocusIndicatorInstaller extends Object,yfiles.view.ICanvasObjectInstaller{
    }
    var IFocusIndicatorInstaller:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum BridgeCrossingStyle{
      /**
       * A simple gap in the path.
       */
      GAP,
      /**
       * A rectangular bridge will be inserted in the path.
       */
      RECTANGLE,
      /**
       * A rectangular bridge with a fixed aspect ratio will be inserted in the path.
       * <p>
       * This bridge will be scaled for wider bridges so that its aspect ratio remains constant.
       * </p>
       */
      RECTANGLE_SCALED,
      /**
       * A triangular bridge will be inserted in the path.
       */
      TWO_SIDES,
      /**
       * A triangular bridge with a fixed aspect ratio will be inserted in the path.
       * <p>
       * This bridge will be scaled for wider bridges so that its aspect ratio remains constant.
       * </p>
       */
      TWO_SIDES_SCALED,
      /**
       * A bridge, consisting of two quarter circle arcs connected by a straight line, will be inserted in the path.
       */
      ARC,
      /**
       * A semi-circle bridge will be inserted in the path.
       * <p>
       * This bridge will be scaled for wider bridges so that its aspect ratio remains constant.
       * </p>
       */
      ARC_SCALED,
      /**
       * Inserts a custom bridge into the path.
       * <p>
       * Custom bridges require an appropriate implementation of the {@link yfiles.view.IBridgeCreator} interface and {@link yfiles.view.IBridgeCreator#createCustomBridge}
       * method.
       * </p>
       */
      CUSTOM
    }
    export enum BridgeOrientationStyle{
      /**
       * The brige is drawn to the right of the flow of the path.
       */
      FLOW_RIGHT,
      /**
       * The brige is drawn to the left of the flow of the path.
       */
      FLOW_LEFT,
      /**
       * The brige is drawn in the direction of the positive axes.
       */
      POSITIVE,
      /**
       * The brige is drawn in the direction of the negative axes.
       */
      NEGATIVE,
      /**
       * The bridge points to the right.
       */
      LEFT,
      /**
       * The bridge points to the left.
       */
      RIGHT,
      /**
       * The bridge points upwards.
       */
      UP,
      /**
       * The bridge points downwards.
       */
      DOWN
    }
    export enum BridgeCrossingPolicy{
      /**
       * Makes horizontal segments bridge over vertical obstacles.
       * <p>
       * Horizontal segments do not cross each other, just as vertical ones will not cross each other. Horizontal segments are
       * segments with a slope between 1 and -1. All others are vertical segments. For non-parallel intersecting pairs of
       * segments there does not need to be a crossing unless only one of them is vertical.
       * </p>
       */
      HORIZONTAL_BRIDGES_VERTICAL,
      /**
       * Makes vertical segments bridge over horizontal obstacles.
       * <p>
       * Horizontal segments do not cross each other, just as vertical ones will not cross each other. Horizontal segments are
       * segments with a slope between 1 and -1. All others are vertical segments. For non-parallel intersecting pairs of
       * segments there does not need to be a crossing unless only one of them is horizontal.
       * </p>
       */
      VERTICAL_BRIDGES_HORIZONTAL,
      /**
       * Makes more horizontal edges bridge over edges which are less horizontal.
       * <p>
       * The smaller the absolute slope of an edge, the more horizontal it is. For each non-parallel intersecting pair of
       * segments, there will be a crossing.
       * </p>
       */
      MORE_HORIZONTAL_BRIDGES_LESS_HORIZONTAL,
      /**
       * Makes more vertical edges bridge over edges which are less vertical.
       * <p>
       * The larger the absolute slope of an edge, the more vertical it is. For each non-parallel intersecting pair of segments,
       * there will be a crossing.
       * </p>
       */
      MORE_VERTICAL_BRIDGES_LESS_VERTICAL
    }
    /**
     * Helper class that calculates visual hints in a path where said path crosses an obstacle.
     * <p>
     * Most notably this feature is used to render crossings of {@link yfiles.graph.IEdge}s with other edges.
     * </p>
     * <p>
     * To set up bridges in a {@link yfiles.view.GraphComponent} at the least the following is necessary:
     * </p>
     * <p>
     * Furthermore, an edge style that supports bridges is necessary; {@link yfiles.styles.PolylineEdgeStyle} is a good choice. Various
     * other properties can be set to change the defaults, e.g. {@link yfiles.view.BridgeCrossingStyle how bridges appear}, or {@link yfiles.view.BridgeOrientationStyle which way bridges point}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Bridges, Demo.yFiles.Graph.Bridges</li>
     * </ul>
     * @class
     */
    export interface BridgeManager extends Object{}
    export class BridgeManager {
      /**
       * Initializes a new instance of the {@link yfiles.view.BridgeManager} class.
       * <p>
       * Initially the {@link yfiles.view.BridgeManager#defaultBridgeCreator} will be set to an instance that will delegate to the set of <code>Default</code> properties in this class and the {@link yfiles.view.BridgeManager#insertDefaultCustomBridge}
       * method respectively.
       * </p>
       */
      constructor();
      /**
       * Core method that will add bridges to a {@link yfiles.geometry.GeneralPath} using the {@link yfiles.view.IBridgeCreator} callback to
       * determine the type of the bridges.
       * <p>
       * This method may return <code>path</code> itself if no crossings have been determined. Otherwise it will return a new path instance
       * that has the bridges added at the corresponding places.
       * </p>
       * @param context The context to use.
       * @param path The path to calculate the bridged variant for.
       * @param callback The callback or <code>null</code> that determines the appearance of the bridges.
       * @returns The path that might have been augmented by the bridges.
       */
      addBridges(context:yfiles.view.IRenderContext,path:yfiles.geometry.GeneralPath,callback:yfiles.view.IBridgeCreator):yfiles.geometry.GeneralPath;
      /**
       * Adds another {@link yfiles.view.IObstacleProvider} to query for obstacles.
       * <p>
       * The {@link yfiles.view.IObstacleProvider#getObstacles} method will be invoked during construction of the obstacles.
       * </p>
       * @param provider The provider instance to use for later queries.
       * @see yfiles.view.BridgeManager#registerObstacles
       * @see yfiles.view.BridgeManager#removeObstacleProvider
       */
      addObstacleProvider(provider:yfiles.view.IObstacleProvider):void;
      /**
       * Disposes of this instance by resetting the {@link yfiles.view.BridgeManager#canvasComponent} property to <code>null</code>.
       */
      dispose():void;
      /**
       * Gets a hash code that describes the current state of the obstacles.
       * <p>
       * This method can be used to get a concise, digest snapshot of the current obstacle locations. Storing that result and
       * comparing it to another snapshot later allows to determine quickly whether obstacles have changed since the last call.
       * This is helpful, e.g. in implementations of {@link yfiles.styles.IEdgeStyle} that support bridges to not recreate the edge path
       * (including bridges) in {@link yfiles.view.IVisualCreator#updateVisual} if no obstacles changed.
       * </p>
       * @param context The context to inspect.
       * @returns A hash of the state of the obstacles.
       */
      getObstacleHash(context:yfiles.view.IRenderContext):number;
      /**
       * Implementation that will be called by the default value of the {@link yfiles.view.BridgeManager#defaultBridgeCreator} to satisfy requests to
       * {@link yfiles.view.IBridgeCreator#createCustomBridge}.
       * <p>
       * This implementation will insert a gap style bridge by first creating a {@link yfiles.geometry.GeneralPath#lineTo line} to <code>start</code> and then {@link yfiles.geometry.GeneralPath#moveTo moving} to <code>end</code>.
       * </p>
       * @param context The context for the call.
       * @param path The path to append the next segment to.
       * @param start The coordinates of the starting point of the bridge.
       * @param end The coordinates of the ending point of the bridge.
       * @param gapLength The distance between the starting point and the end point.
       * @see yfiles.view.IBridgeCreator#createCustomBridge
       */
      insertDefaultCustomBridge(context:yfiles.view.IRenderContext,path:yfiles.geometry.GeneralPath,start:yfiles.geometry.Point,end:yfiles.geometry.Point,gapLength:number):void;
      /**
       * Installs this manager for the specified {@link yfiles.view.BridgeManager#canvasComponent}.
       * @param canvas The {@link #canvasComponent}.
       * @protected
       */
      install(canvas:yfiles.view.CanvasComponent):void;
      /**
       * Convenience method that can be called to dynamically register an obstacle path with the context during the rendering.
       * <p>
       * Normally all obstacles will be registered initially by method
       * {@link yfiles.view.BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
       * obstacles at a later point in time.
       * </p>
       * @param context The context to register the obstacles with.
       * @param path The path that describes the obstacles.
       * @see yfiles.view.BridgeManager#registerObstacleLine
       */
      registerObstacle(context:yfiles.view.IRenderContext,path:yfiles.geometry.GeneralPath):void;
      /**
       * Convenience method that can be called to dynamically register a single obstacle cubic curve with the context during the
       * rendering.
       * <p>
       * Normally all obstacles will be registered initially by method
       * {@link yfiles.view.BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
       * obstacles at a later point in time.
       * </p>
       * @param context The context to register the obstacles with.
       * @param p1 The coordinates of the starting point of the line.
       * @param cp1 The coordinates of the first control point of the curve.
       * @param cp2 The coordinates of the second control point of the curve.
       * @param p2 The coordinates of the ending point of the line.
       * @see yfiles.view.BridgeManager#registerObstacleLine
       */
      registerObstacleCubicCurve(context:yfiles.view.IRenderContext,p1:yfiles.geometry.Point,cp1:yfiles.geometry.Point,cp2:yfiles.geometry.Point,p2:yfiles.geometry.Point):void;
      /**
       * Convenience method that can be called to dynamically register a single obstacle line with the context during the
       * rendering.
       * <p>
       * Normally all obstacles will be registered initially by method
       * {@link yfiles.view.BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
       * obstacles at a later point in time.
       * </p>
       * @param context The context to register the obstacles with.
       * @param p1 The coordinates of the first point of the line.
       * @param p2 The coordinates of the second point of the line.
       * @see yfiles.view.BridgeManager#registerObstacleLine
       */
      registerObstacleLine(context:yfiles.view.IRenderContext,p1:yfiles.geometry.Point,p2:yfiles.geometry.Point):void;
      /**
       * Convenience method that can be called to dynamically register a single obstacle quadratic curve with the context during
       * the rendering.
       * <p>
       * Normally all obstacles will be registered initially by method
       * {@link yfiles.view.BridgeManager#registerObstacles}. This method can be called during the rendering phase to register additional
       * obstacles at a later point in time.
       * </p>
       * @param context The context to register the obstacles with.
       * @param p1 The coordinates of the starting point of the line.
       * @param cp The coordinates of the control point of the curve.
       * @param p2 The coordinates of the ending point of the line.
       * @see yfiles.view.BridgeManager#registerObstacleLine
       */
      registerObstacleQuadCurve(context:yfiles.view.IRenderContext,p1:yfiles.geometry.Point,cp:yfiles.geometry.Point,p2:yfiles.geometry.Point):void;
      /**
       * Called to initialize the context with the obstacles.
       * @param context The context that will be used to store the obstacles with.
       * @see yfiles.view.BridgeManager#registerObstacle
       * @see yfiles.view.BridgeManager#registerObstacleLine
       * @protected
       */
      registerObstacles(context:yfiles.view.IRenderContext):void;
      /**
       * Removes a previously {@link yfiles.view.BridgeManager#addObstacleProvider added} {@link yfiles.view.IObstacleProvider} from the list of registered providers.
       * @param provider The provider instance to remove from this instance.
       * @see yfiles.view.BridgeManager#addObstacleProvider
       */
      removeObstacleProvider(provider:yfiles.view.IObstacleProvider):void;
      /**
       * Reverts the {@link yfiles.view.BridgeManager#install} method.
       * @param canvas The {@link #canvasComponent}.
       * @protected
       */
      uninstall(canvas:yfiles.view.CanvasComponent):void;
      /**
       * Gets or sets the default implementation of the {@link yfiles.view.IBridgeCreator} that will be used if {@link yfiles.view.BridgeManager#addBridges}
       * is invoked with a <code>null</code> callback argument.
       * @see yfiles.view.BridgeManager#defaultBridgeCrossingStyle
       * @see yfiles.view.BridgeManager#defaultBridgeWidth
       * @see yfiles.view.BridgeManager#defaultBridgeHeight
       * @see yfiles.view.BridgeManager#insertDefaultCustomBridge
       * @type {yfiles.view.IBridgeCreator}
       */
      defaultBridgeCreator:yfiles.view.IBridgeCreator;
      /**
       * Gets or sets a value that determines whether curves ({@link yfiles.geometry.GeneralPath#cubicTo cubic} and {@link yfiles.geometry.GeneralPath#quadTo quadratic}) should be considered as obstacles.
       * <p>
       * This can be a costly operation so the default is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      considerCurves:boolean;
      /**
       * Gets or sets a threshold value that determines below which zoom level, there should be no more bridge calculation.
       * <p>
       * Since the calculation of the bridges is a costly operation if there are many items visible and bridges are not clearly
       * visible at low zoom levels, it is convenient to disable the calculation of bridges for these zoom levels to speed up the
       * rendering process. The default value is <code>0.3d</code>.
       * </p>
       * @type {number}
       */
      zoomThreshold:number;
      /**
       * Gets or sets the {@link yfiles.view.BridgeCrossingPolicy} mode that is used by this instance.
       * <p>
       * By default the {@link yfiles.view.BridgeCrossingPolicy#HORIZONTAL_BRIDGES_VERTICAL} mode is set.
       * </p>
       * @type {yfiles.view.BridgeCrossingPolicy}
       */
      bridgeCrossingPolicy:yfiles.view.BridgeCrossingPolicy;
      /**
       * Gets or sets the {@link yfiles.view.BridgeManager#canvasComponent} that should be managed by this instance.
       * <p>
       * This method will {@link yfiles.view.BridgeManager#install} a {@link yfiles.view.CanvasComponent#addPrepareRenderContextListener preparation stage} for the {@link yfiles.view.IRenderContext} in the
       * provided {@link yfiles.view.BridgeManager#canvasComponent} via the {@link yfiles.graph.ILookup#lookup} mechanism.
       * </p>
       * @type {yfiles.view.CanvasComponent}
       */
      canvasComponent:yfiles.view.CanvasComponent;
      /**
       * Gets or sets the default {@link yfiles.view.BridgeCrossingStyle} that will be used by the default implementation of the {@link yfiles.view.IBridgeCreator}
       * that is initially assigned to the {@link yfiles.view.BridgeManager#defaultBridgeCreator} property.
       * <p>
       * The default value is {@link yfiles.view.BridgeCrossingStyle#ARC}.
       * </p>
       * @see yfiles.view.IBridgeCreator#getCrossingStyle
       * @type {yfiles.view.BridgeCrossingStyle}
       */
      defaultBridgeCrossingStyle:yfiles.view.BridgeCrossingStyle;
      /**
       * Gets or sets the default width of a bridge that will be used by the default implementation of the {@link yfiles.view.IBridgeCreator}
       * that is initially assigned to the {@link yfiles.view.BridgeManager#defaultBridgeCreator} property.
       * <p>
       * The default value is <code>10.0d</code>. Setting this value will adjust the {@link yfiles.view.BridgeManager#clipMargin}.
       * </p>
       * @see yfiles.view.IBridgeCreator#getBridgeWidth
       * @type {number}
       */
      defaultBridgeWidth:number;
      /**
       * Gets or sets the margin by which the {@link yfiles.view.IRenderContext#clip} should be enlarged to accommodate for clipped away lines being taken into account
       * for bridge calculation.
       * <p>
       * This value should be set to at least half the value of {@link yfiles.view.IBridgeCreator#getBridgeWidth the width of the bridges} used in this context. This is because
       * obstacles might otherwise be clipped from the view and not taken into account for the calculation of the bridges. By
       * default this value will be 2 more than half of the {@link yfiles.view.BridgeManager#defaultBridgeWidth}.
       * </p>
       * @type {number}
       */
      clipMargin:number;
      /**
       * Gets or sets the default height of a bridge that will be used by the default implementation of the {@link yfiles.view.IBridgeCreator}
       * that is initially assigned to the {@link yfiles.view.BridgeManager#defaultBridgeCreator} property.
       * <p>
       * The default value is <code>5.0d</code>.
       * </p>
       * @see yfiles.view.IBridgeCreator#getBridgeHeight
       * @type {number}
       */
      defaultBridgeHeight:number;
      /**
       * Gets or sets the default {@link yfiles.view.BridgeOrientationStyle} for bridges that will be used by the default implementation
       * of the {@link yfiles.view.IBridgeCreator} that is initially assigned to the {@link yfiles.view.BridgeManager#defaultBridgeCreator} property.
       * <p>
       * The default is {@link yfiles.view.BridgeOrientationStyle#UP}.
       * </p>
       * @see yfiles.view.IBridgeCreator#getOrientationStyle
       * @type {yfiles.view.BridgeOrientationStyle}
       */
      defaultBridgeOrientationStyle:yfiles.view.BridgeOrientationStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * Allows to add bridge visualizations to a {@link yfiles.geometry.GeneralPath}.
     * <p>
     * An implementation of this interface is used by {@link yfiles.view.BridgeManager} to insert bridges into an edge path.
     * </p>
     * <p>
     * A custom implementation of this interface is necessary if a {@link yfiles.view.BridgeCrossingStyle#CUSTOM custom bridge drawing style} is used.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Bridges, Demo.yFiles.Graph.Bridges</li>
     * </ul>
     * @see yfiles.view.BridgeManager
     * @interface
     */
    export interface IBridgeCreator extends Object{
      /**
       * Callback that will be used by the {@link yfiles.view.BridgeManager} if the {@link yfiles.view.IBridgeCreator#getCrossingStyle} method
       * yields {@link yfiles.view.BridgeCrossingStyle#CUSTOM} to actually insert a bridge into the given {@link yfiles.geometry.GeneralPath}.
       * <p>
       * When this method is called the {@link yfiles.geometry.GeneralPath}'s {@link yfiles.geometry.GeneralPath#lastCoordinate last} coordinates are not yet at <code>start</code>. So most
       * implementation should first {@link yfiles.geometry.GeneralPath#moveTo} that location. Also at the end of the call,
       * implementation should make sure that the GeneralPath's end is at <code>end</code>. The <code>gapLength</code> is provided for convenience so that
       * the distance between <code>startPoint</code> and <code>endPoint</code> does not need to be calculated if it is needed for the drawing.
       * </p>
       * @param context The context for the call.
       * @param path The path to append the bridge segment to.
       * @param start The coordinates of the starting point of the bridge.
       * @param end The coordinates of the ending point of the bridge.
       * @param gapLength The distance between the starting point and the end point.
       * @abstract
       */
      createCustomBridge(context:yfiles.view.IRenderContext,path:yfiles.geometry.GeneralPath,start:yfiles.geometry.Point,end:yfiles.geometry.Point,gapLength:number):void;
      /**
       * Gets the height of the bridge for the given {@link yfiles.view.IRenderContext}.
       * <p>
       * The height is the default height that will be used for drawing the non-{@link yfiles.view.BridgeCrossingStyle#CUSTOM custom} {@link yfiles.view.BridgeCrossingStyle}s.
       * </p>
       * @param context The context where the bridge will be created for.
       * @returns The non-negative preferred basic unscaled height of the bridge.
       * @abstract
       */
      getBridgeHeight(context:yfiles.view.IRenderContext):number;
      /**
       * Gets the width of the bridge for the given {@link yfiles.view.IRenderContext}.
       * <p>
       * The width is the preferred length of the segment that will be removed from the original segment if it crosses an
       * obstacle. That bridge will be made larger if multiple obstacle bridges would otherwise overlap. The bridge will be
       * automatically made smaller if it happens to be near the end of a segment and would otherwise extend beyond the end of
       * the segment.
       * </p>
       * @param context The context where the bridge will be created for.
       * @returns The positive preferred width of the bridge.
       * @abstract
       */
      getBridgeWidth(context:yfiles.view.IRenderContext):number;
      /**
       * Gets the {@link yfiles.view.BridgeCrossingStyle} to use in the given {@link yfiles.view.IRenderContext}.
       * @param context The context in which the crossing will be used.
       * @returns The style to use, or {@link #CUSTOM} in order to let {@link } call
       * {@link #createCustomBridge}.
       * @abstract
       */
      getCrossingStyle(context:yfiles.view.IRenderContext):yfiles.view.BridgeCrossingStyle;
      /**
       * Gets the {@link yfiles.view.BridgeOrientationStyle} to use in the given {@link yfiles.view.IRenderContext}.
       * @param context The context in which the style will be used..
       * @returns The style to use. Any value can be returned and will be ignored if the {@link #getCrossingStyle crossing style} is set to
       * {@link #CUSTOM}.
       * @abstract
       */
      getOrientationStyle(context:yfiles.view.IRenderContext):yfiles.view.BridgeOrientationStyle;
    }
    var IBridgeCreator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Implementations of this interface can provide a path that is used to determine bridges by {@link yfiles.view.BridgeManager}.
     * <p>
     * Typically the {@link yfiles.graph.ILookup} implementation of styles will return an implementation of this interface.
     * </p>
     * @interface
     */
    export interface IObstacleProvider extends Object{
      /**
       * Returns a path representing obstacles that would need bridges.
       * <p>
       * For performance reasons the implementation should only return a non-<code>null</code> {@link yfiles.geometry.GeneralPath} if it might be {@link yfiles.view.IVisibilityTestable#isVisible visible}
       * in the provided {@link yfiles.view.IRenderContext#clip}.
       * </p>
       * @param context The context for which the obstacles are queried.
       * @returns A path that describes the obstacles or <code>null</code> if there are no obstacles for the given context.
       * @abstract
       */
      getObstacles(context:yfiles.view.IRenderContext):yfiles.geometry.GeneralPath;
    }
    var IObstacleProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An {@link yfiles.view.ItemModelManager.<T>} that automatically installs and removes visual representations of items in a {@link yfiles.collections.IObservableCollection.<T>} into or from the canvas.
     * Type parameter <code>T</code>: The type of items in the observable collection.
     * @class
     * @extends {yfiles.view.ItemModelManager.<T>}
     * @template T
     */
    export interface CollectionModelManager<T> extends yfiles.view.ItemModelManager<T>{}
    export class CollectionModelManager<T> {
      /**
       * Creates a new instance using the given <code>model</code>.
       * @param itemType The type of items in the observable collection.
       * @param model The model to manage.
       */
      constructor(itemType:yfiles.lang.Class,model:yfiles.collections.IObservableCollection<T>);
      /**
       * Creates a new instance.
       * @param itemType The type of items in the observable collection.
       * @see yfiles.view.CollectionModelManager.<T>#model
       */
      constructor(itemType:yfiles.lang.Class);
      /**
       * Called when this mode gets {@link yfiles.view.CollectionModelManager.<T>#enabled disabled}.
       * @protected
       */
      onDisabled():void;
      /**
       * Called when this mode gets {@link yfiles.view.CollectionModelManager.<T>#enabled enabled}.
       * @protected
       */
      onEnabled():void;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.view.CollectionModelManager.<T>} is enabled.
       * <p>
       * If the manager is disabled, it will not install the {@link yfiles.view.ICanvasObject}s into the associated
       * {@link yfiles.view.CanvasComponent}.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * Gets or sets the model of the items to display in the canvas.
       * <p>
       * Setting the model will automatically remove the items installed for a previously installed model. Also the items
       * contained in the new model will immediately be queried and installed into the canvas.
       * </p>
       * @type {yfiles.collections.IObservableCollection.<T>}
       */
      model:yfiles.collections.IObservableCollection<T>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Installs and removes visual representations of items into or from the canvas.
     * Type parameter <code>T</code>: The type of managed items.
     * @class
     * @template T
     */
    export interface ItemModelManager<T> extends Object{}
    export class ItemModelManager<T> {
      /**
       * 
       * @param itemType The type of managed items.
       */
      constructor(itemType:yfiles.lang.Class);
      /**
       * Registers and {@link yfiles.view.ItemModelManager.<T>#install installs} the given <code>item</code> into the canvas.
       * @param item The item to install.
       */
      addItem(item:T):void;
      /**
       * {@link yfiles.view.ItemModelManager.<T>#addItem Adds} all items to the canvas.
       * <p>
       * This is useful to initialize this manager with an existing collection of items.
       * </p>
       * @param items The items to add.
       */
      addItems(items:yfiles.collections.IEnumerable<T>):void;
      /**
       * Retrieves the canvas object that has been registered for the given item or <code>null</code>.
       * @param item The item to retrieve the canvas object for.
       * @returns A canvas object instance or <code>null</code>.
       * @see yfiles.view.ItemModelManager.<T>#install
       */
      getCanvasObject(item:T):yfiles.view.ICanvasObject;
      /**
       * Gets the {@link yfiles.view.ICanvasObjectGroup} used to {@link yfiles.view.ItemModelManager.<T>#install} the <code>item</code> in the canvas.
       * <p>
       * The default implementation simply returns the value of {@link yfiles.view.ItemModelManager.<T>#canvasObjectGroup}.
       * </p>
       * @param item The item to get the group for.
       * @returns The {@link } used to {@link #install} an item in the canvas.
       * @protected
       */
      getCanvasObjectGroup(item:T):yfiles.view.ICanvasObjectGroup;
      /**
       * Returns the {@link yfiles.view.ICanvasObjectDescriptor} used to {@link yfiles.view.ItemModelManager.<T>#install} the <code>item</code> in the canvas.
       * <p>
       * The default implementation simply returns the value of {@link yfiles.view.ItemModelManager.<T>#descriptor}.
       * </p>
       * @param item The item to get the descriptor for.
       * @returns The {@link } used to {@link #install} an item in the canvas.
       * @protected
       */
      getDescriptor(item:T):yfiles.view.ICanvasObjectDescriptor;
      /**
       * Tries to find the item managed by this instance that is associated with the given canvas object.
       * @param canvasObject The canvas object to query the corresponding model item for.
       * @returns The item or <code>null</code> if no such item could be found.
       */
      getItem(canvasObject:yfiles.view.ICanvasObject):T;
      /**
       * Installs a member of the collection into the canvas using the {@link yfiles.view.ItemModelManager.<T>#descriptor}.
       * @param item The member to install.
       * @returns The canvas object that has been returned by the installer.
       * @protected
       */
      install(item:T):yfiles.view.ICanvasObject;
      /**
       * Event handler for item creation events that adds the item to this manager.
       * <p>
       * This method is intended to be used to delegate item creation events from observable collections:
       * </p>
       * <p>
       * This method calls {@link yfiles.view.ItemModelManager.<T>#addItem} with {@link yfiles.collections.ItemEventArgs.<T>#item}.
       * </p>
       * @param source The instance that raised the event.
       * @param evt The event arguments.
       */
      itemAddedHandler(source:Object,evt:yfiles.collections.ItemEventArgs<T>):void;
      /**
       * Event handler for item change events that updates the visual representation for that item.
       * <p>
       * This method is intended to be used to delegate item change events from observable collections:
       * </p>
       * <p>
       * This is necessary if the group the canvas object should be added to might change after the item change. This might
       * happen, e.g. after changing an edge's port in a grouped graph.
       * </p>
       * @param source The event source.
       * @param evt The item event args which indicate the change.
       */
      itemChangedHandler(source:Object,evt:yfiles.collections.ItemEventArgs<T>):void;
      /**
       * Event handler for item removal events that removes the item from this manager.
       * <p>
       * This method is intended to be used to delegate item removal events from observable collections:
       * </p>
       * <p>
       * This method calls {@link yfiles.view.ItemModelManager.<T>#removeItem} with {@link yfiles.collections.ItemEventArgs.<T>#item}.
       * </p>
       * @param source The instance that raised the event.
       * @param evt The event arguments.
       */
      itemRemovedHandler(source:Object,evt:yfiles.collections.ItemEventArgs<T>):void;
      /**
       * Deregisters and uninstalls the given <code>item</code> from the canvas.
       * @param item The item to uninstall.
       */
      removeItem(item:T):void;
      /**
       * {@link yfiles.view.ItemModelManager.<T>#removeItem Removes} all items from the canvas.
       * <p>
       * This method can be used to clean up this manager before disposing it.
       * </p>
       * @param items The items to remove.
       */
      removeItems(items:yfiles.collections.IEnumerable<T>):void;
      /**
       * This method may be called to update the visual representation of the given item in the canvas.
       * <p>
       * If no visual representation is registered, yet, {@link yfiles.view.ItemModelManager.<T>#install} will be called.
       * </p>
       * @param item The item to update.
       */
      update(item:T):void;
      /**
       * Gets or sets an {@link yfiles.collections.IComparer.<T>} that compares the items so that they are put into the correct order in the canvas tree.
       * <p>
       * If this instance is set to <code>null</code> this feature is effectively turned off.
       * </p>
       * @type {yfiles.collections.IComparer.<T>}
       */
      comparer:yfiles.collections.IComparer<T>;
      /**
       * Gets or sets the canvas object descriptor used by {@link yfiles.view.ItemModelManager.<T>#install} when an item is {@link yfiles.view.ICanvasObjectGroup#addChild added} to an canvas object group.
       * <p>
       * Setting the descriptor to <code>null</code> will prevent this manager from installing new items.
       * </p>
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      descriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the canvas object group used by {@link yfiles.view.ItemModelManager.<T>#install}.
       * <p>
       * Setting this property to <code>null</code> will prevent this manager from installing new items.
       * </p>
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      canvasObjectGroup:yfiles.view.ICanvasObjectGroup;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default implementation of an {@link yfiles.view.ISelectionModel.<T>} that is based on an observable collection to hold the selected elements.
     * <p>
     * This selection model has a domain that's controlled with a {@link yfiles.view.DefaultSelectionModel.<T>#isInDomain predicate} which encompasses the items that can be selected.
     * If that domain changes it is important to delegate item removal events from the domain model to this selection model to
     * ensure that items that have been removed are deselected as well. A {@link yfiles.view.DefaultSelectionModel.<T>#itemRemovedFromDomainHandler convenience event handler} exists for that purpose.
     * </p>
     * Type parameter <code>T</code>: The type of the items to be selected.
     * @class
     * @implements {yfiles.view.ISelectionModel.<T>}
     * @template T
     */
    export interface DefaultSelectionModel<T> extends Object,yfiles.view.ISelectionModel<T>{}
    export class DefaultSelectionModel<T> {
      /**
       * Creates a default selection model implementation that uses a {@link yfiles.collections.ObservableCollection.<T>} to hold the selection.
       */
      constructor();
      /**
       * 
       */
      clear():void;
      /**
       * 
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<T>;
      /**
       * Determines whether an item is selected by checking for the containment in the {@link yfiles.view.DefaultSelectionModel.<T>#selection}.
       * @param item The item to check
       * @returns Whether the item is selected.
       */
      isSelected(item:T):boolean;
      /**
       * Event handler for domain item removal events that removes the item from this model.
       * <p>
       * This method is intended to be used to delegate item removal events from the respective domain collection:
       * </p>
       * <p>
       * This method calls {@link yfiles.view.DefaultSelectionModel.<T>#removeDomainItem} with {@link yfiles.collections.ItemEventArgs.<T>#item}.
       * </p>
       * @param source The instance that raised the event.
       * @param evt The event arguments.
       */
      itemRemovedFromDomainHandler(source:Object,evt:yfiles.collections.ItemEventArgs<T>):void;
      /**
       * Called when an item gets selected or deselected.
       * <p>
       * Triggers the {@link yfiles.view.DefaultSelectionModel.<T>#addItemSelectionChangedListener ItemSelectionChanged} event.
       * </p>
       * @param o The item.
       * @param selected Whether or not the item is selected after the change.
       * @protected
       */
      onItemSelectionChanged(o:T,selected:boolean):void;
      /**
       * Removes the given <code>item</code> from the selection in response to the item being removed from the domain.
       * @param item The item to remove from the considered items.
       */
      removeDomainItem(item:T):void;
      /**
       * Sets the selection state of the item.
       * @param item
       * @param selected
       */
      setSelected(item:T,selected:boolean):void;
      /**
       * Gets or sets the predicate which determines whether the queried item should be considered for this selection model.
       * @type {function(T): boolean}
       */
      isInDomain:(obj:T)=>boolean;
      /**
       * Provides an observable collection view over the selected items.
       * @type {yfiles.collections.IObservableCollection.<T>}
       */
      selection:yfiles.collections.IObservableCollection<T>;
      /**
       * 
       * @type {number}
       */
      size:number;
      /**
       * 
       * @param listener
       */
      addItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<T>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<T>)=>void):void;
      static $class:yfiles.lang.Class;
    }
    export enum ShowFocusPolicy{
      /**
       * Indicates that irrespectively of the keyboard focus, the focus should always be rendered for the {@link yfiles.view.FocusIndicatorManager.<T>#focusedItem}.
       */
      ALWAYS,
      /**
       * Indicates that depending on the value of {@link yfiles.view.FocusIndicatorManager.<T>#focused} the focus should only be rendered if this property is set to <code>true</code>.
       */
      WHEN_FOCUSED
    }
    /**
     * A {@link yfiles.view.ModelManager.<T>} implementation that manages the visual decoration of the focused element in a canvas.
     * <p>
     * For the item that should be focused this class uses the {@link yfiles.graph.ILookup} mechanism of the item to lookup an {@link yfiles.view.IFocusIndicatorInstaller}
     * implementation which will do the actual installation of the decoration in the canvas.
     * </p>
     * Type parameter <code>T</code>: The type of the items.
     * @class
     * @extends {yfiles.view.ModelManager.<T>}
     * @implements {yfiles.lang.IPropertyObservable}
     * @template T
     */
    export interface FocusIndicatorManager<T extends yfiles.graph.ILookup> extends yfiles.view.ModelManager<T>,yfiles.lang.IPropertyObservable{}
    export class FocusIndicatorManager<T extends yfiles.graph.ILookup> {
      /**
       * Creates an instance that for all focused items in the model dynamically installs a decoration.
       * @param canvas The canvas to add the focus marks to.
       */
      constructor(canvas:yfiles.view.CanvasComponent);
      /**
       * Called when the {@link yfiles.view.FocusIndicatorManager.<T>#focused} property has changed.
       * @see yfiles.view.FocusIndicatorManager.<T>#focused
       * @protected
       */
      onFocusedChanged():void;
      /**
       * Callback method that is triggered when the focused item has changed.
       * <p>
       * This method raises the {@link yfiles.view.FocusIndicatorManager.<T>#addPropertyChangedListener PropertyChanged} event. Subclasses overriding this method should make sure to invoke
       * the super class implementation.
       * </p>
       * @param evt a description of the event
       * @see yfiles.view.FocusIndicatorManager.<T>#focusedItem
       * @see yfiles.view.FocusIndicatorManager.<T>#addPropertyChangedListener
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when the {@link yfiles.view.FocusIndicatorManager.<T>#focusedItem} property has been changed.
       * @param listener The listener to add.
       * @see yfiles.view.FocusIndicatorManager.<T>#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when the {@link yfiles.view.FocusIndicatorManager.<T>#focusedItem} property has been changed.
       * @param listener The listener to remove.
       * @see yfiles.view.FocusIndicatorManager.<T>#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the focused item, updates the canvas and fires the corresponding property change event.
       * @type {T}
       */
      focusedItem:T;
      /**
       * Gets or sets the show focus policy.
       * <p>
       * The default is {@link yfiles.view.ShowFocusPolicy#WHEN_FOCUSED}.
       * </p>
       * @default yfiles.view.ShowFocusPolicy.WHEN_FOCUSED
       * @type {yfiles.view.ShowFocusPolicy}
       */
      showFocusPolicy:yfiles.view.ShowFocusPolicy;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.view.CanvasComponent container} this manager is installed in currently is focused.
       * @type {boolean}
       */
      focused:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.view.ModelManager.<T>} implementation that manages the visual decorations of highlighted elements in a canvas.
     * <p>
     * For each item that should be highlighted this class uses the {@link yfiles.graph.ILookup} mechanism of the items to lookup an {@link yfiles.view.IHighlightIndicatorInstaller}
     * implementation which will do the actual installation of the decoration in the canvas.
     * </p>
     * Type parameter <code>T</code>: The type of the items.
     * @class
     * @extends {yfiles.view.ModelManager.<T>}
     * @template T
     */
    export interface HighlightIndicatorManager<T extends yfiles.graph.ILookup> extends yfiles.view.ModelManager<T>{}
    export class HighlightIndicatorManager<T extends yfiles.graph.ILookup> {
      /**
       * Creates an instance that for all selected items in the model dynamically installs a highlight decoration.
       * @param canvas The canvas to add the selection marks to.
       * @param [selectionModel=null] The model that determines which items are highlighted.
       */
      constructor(canvas:yfiles.view.CanvasComponent,selectionModel?:yfiles.view.ISelectionModel<T>);
      /**
       * Adds another highlight.
       * @param item The item to highlight.
       */
      addHighlight(item:T):void;
      /**
       * Removes all highlights.
       */
      clearHighlights():void;
      /**
       * Removes an item from the current highlight selection.
       * @param item The item whose highlight decorator will be removed.
       */
      removeHighlight(item:T):void;
      /**
       * Gets or sets the selection model that determines which items are highlighted.
       * <p>
       * May be <code>null</code>, in which case no elements are considered highlighted.
       * </p>
       * @type {yfiles.view.ISelectionModel.<T>}
       */
      selectionModel:yfiles.view.ISelectionModel<T>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Installs visual representations of items in a {@link yfiles.view.CanvasComponent}.
     * <p>
     * This interface is most prominently used by {@link yfiles.view.ModelManager.<T>} and its descendants to manage visuals in the canvas.
     * </p>
     * @interface
     */
    export interface ICanvasObjectInstaller extends Object{
      /**
       * This the main method of the interface that performs the installation of an item's visual representation in the {@link yfiles.view.CanvasComponent canvas} by
       * adding {@link yfiles.view.ICanvasObject}s.
       * @param context The context that contains the information required to install the visual representation.
       * @param group The canvas object group to add the newly generated {@link } to.
       * @param item The item to install.
       * @returns The newly generated {@link } for the item's visual representation. <code>null</code> if nothing was installed.
       * @abstract
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
    }
    var ICanvasObjectInstaller:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface for objects that can install a visual representation of a selection decoration of an item in the model
     * displayed in a canvas.
     * <p>
     * This interface is a tagging sub interface of the {@link yfiles.view.ICanvasObjectInstaller}. This interface is used for {@link yfiles.graph.ILookup#lookup}
     * operations.
     * </p>
     * @see yfiles.view.SelectionIndicatorManager.<T>
     * @see yfiles.view.IHighlightIndicatorInstaller
     * @interface
     * @implements {yfiles.view.ICanvasObjectInstaller}
     */
    export interface ISelectionIndicatorInstaller extends Object,yfiles.view.ICanvasObjectInstaller{
    }
    var ISelectionIndicatorInstaller:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface that models the selection state of a set of items.
     * <p>
     * This interface extends the {@link yfiles.collections.IEnumerable.<T>} interface for convenient enumeration of the selected items. Use the {@link yfiles.collections.IEnumerable.<T>#getEnumerator} to get
     * all selected items.
     * </p>
     * Type parameter <code>T</code>: The type of items contained in the selection's domain.
     * @interface
     * @implements {yfiles.collections.IEnumerable.<T>}
     * @template T
     */
    export interface ISelectionModel<T> extends Object,yfiles.collections.IEnumerable<T>{
      /**
       * Clears the selection.
       * <p>
       * This is a convenience method that will set the selection state of all elements to unselected.
       * </p>
       * @see yfiles.view.ISelectionModel.<T>#addItemSelectionChangedListener
       * @abstract
       */
      clear():void;
      /**
       * Determines whether an item is currently selected.
       * @param item The item to check.
       * @returns Whether it is currently selected.
       * @abstract
       */
      isSelected(item:T):boolean;
      /**
       * Sets the selection state of an item.
       * <p>
       * If the state changes, this will trigger the {@link yfiles.view.ISelectionModel.<T>#addItemSelectionChangedListener ItemSelectionChanged} event.
       * </p>
       * @param item The item to set the selection state for.
       * @param selected Whether to select the object.
       * @abstract
       */
      setSelected(item:T,selected:boolean):void;
      /**
       * Returns the number of selected items.
       * @abstract
       * @type {number}
       */
      size:number;
      /**
       * Adds the given listener for the <code>ItemSelectionChanged</code> event that occurs when an item changed its selection state from
       * selected to unselected or vice versa.
       * @param listener The listener to add.
       * @see yfiles.view.ISelectionModel.<T>#setSelected
       * @see yfiles.view.ISelectionModel.<T>#removeItemSelectionChangedListener
       * @abstract
       */
      addItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemSelectionChanged</code> event that occurs when an item changed its selection state from
       * selected to unselected or vice versa.
       * @param listener The listener to remove.
       * @see yfiles.view.ISelectionModel.<T>#setSelected
       * @see yfiles.view.ISelectionModel.<T>#addItemSelectionChangedListener
       * @abstract
       */
      removeItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<T>)=>void):void;
    }
    var ISelectionModel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides data for the {@link yfiles.view.ISelectionModel.<T>#addItemSelectionChangedListener ItemSelectionChanged} event.
     * Type parameter <code>T</code>.
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<T>}
     * @template T
     * @final
     */
    export interface ItemSelectionChangedEventArgs<T> extends yfiles.collections.ItemEventArgs<T>{}
    export class ItemSelectionChangedEventArgs<T> {
      /**
       * Initializes a new event argument instance for the given item and its selection state.
       * @param item The item whose {@link #isSelected selection state} has changed.
       * @param selected The value indicating whether the item is selected after the change.
       */
      constructor(item:T,selected:boolean);
      /**
       * Gets a value indicating whether the item is selected after the change.
       * @type {boolean}
       */
      itemSelected:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Manages the visual representations of items in a {@link yfiles.view.CanvasComponent}.
     * Type parameter <code>T</code>: The type of items this instance deals with.
     * @class
     * @template T
     */
    export interface ModelManager<T> extends Object{}
    export class ModelManager<T> {
      /**
       * Creates a new instance that will manage the items for the given canvas.
       * @param canvas The canvas to manage.
       * @protected
       */
      constructor(canvas:yfiles.view.CanvasComponent);
      /**
       * Installs an itemm and adds it to this managers set of managed items.
       * <p>
       * If {@link yfiles.view.ModelManager.<T>#enabled} is set to false the item will not be installed.
       * </p>
       * @param item The item to add.
       * @returns The newly created canvas object or <code>null</code> if nothing was installed.
       * @protected
       */
      add(item:T):yfiles.view.ICanvasObject;
      /**
       * Callback used by {@link yfiles.view.ModelManager.<T>#install} to retrieve the canvas object group for a given item.
       * <p>
       * Returning <code>null</code> will cause {@link yfiles.view.ModelManager.<T>#install} not to install anything.
       * </p>
       * @param item The item to find a canvas object group for.
       * @returns An {@link } or <code>null</code>
       * @protected
       * @abstract
       */
      getCanvasObjectGroup(item:T):yfiles.view.ICanvasObjectGroup;
      /**
       * Callback used by {@link yfiles.view.ModelManager.<T>#install} to retrieve the installer for a given item.
       * <p>
       * Implementations may use the {@link yfiles.graph.ILookup#lookup} of the item.
       * </p>
       * <p>
       * Returning <code>null</code> will cause {@link yfiles.view.ModelManager.<T>#install} not to install anything.
       * </p>
       * @param item The item to find an installer for.
       * @returns An installer or <code>null</code>
       * @protected
       * @abstract
       */
      getInstaller(item:T):yfiles.view.ICanvasObjectInstaller;
      /**
       * Installs a member of the collection into the canvas using the {@link yfiles.view.ICanvasObjectDescriptor} which is returned by
       * {@link yfiles.view.ModelManager.<T>#getInstaller}.
       * @param item The member to install.
       * @returns The canvas object that has been returned by the installer.
       * @protected
       */
      install(item:T):yfiles.view.ICanvasObject;
      /**
       * Called when this mode gets {@link yfiles.view.ModelManager.<T>#enabled disabled}.
       * @protected
       * @abstract
       */
      onDisabled():void;
      /**
       * Called when this mode gets {@link yfiles.view.ModelManager.<T>#enabled enabled}.
       * @protected
       * @abstract
       */
      onEnabled():void;
      /**
       * Uninstalls the provided item and removes it from this managers set of managed items.
       * @param item The item to remove.
       * @protected
       */
      remove(item:T):void;
      /**
       * Simple implementation that removes the given {@link yfiles.view.ICanvasObject}.
       * @param canvasObject The canvas objects that have been associated with the descriptor.
       * @see yfiles.view.ICanvasObject#remove
       * @protected
       */
      unInstall(canvasObject:yfiles.view.ICanvasObject):void;
      /**
       * Gets the canvas instance this instance is managing.
       * @type {yfiles.view.CanvasComponent}
       */
      canvasComponent:yfiles.view.CanvasComponent;
      /**
       * Gets or sets whether this {@link yfiles.view.ModelManager.<T>} is enabled.
       * <p>
       * If the manager is disabled, it will not install the {@link yfiles.view.ICanvasObject}s into the associated
       * {@link yfiles.view.CanvasComponent}.
       * </p>
       * <p>
       * The default is <code>true</code>, i.e. this manager is enabled by default.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.view.ModelManager.<T>} implementation that manages the visual decorations of selected elements in a canvas.
     * <p>
     * For each item in the {@link yfiles.view.SelectionIndicatorManager.<T>#model} that is marked as selected by the {@link yfiles.view.SelectionIndicatorManager.<T>#selectionModel} this class uses the {@link yfiles.graph.ILookup} mechanism of the items
     * to lookup an {@link yfiles.view.ISelectionIndicatorInstaller} implementation which will do the actual installation of the
     * decoration in the canvas.
     * </p>
     * Type parameter <code>T</code>: The type of the items in the collection and selection model.
     * @class
     * @extends {yfiles.view.ModelManager.<T>}
     * @template T
     */
    export interface SelectionIndicatorManager<T extends yfiles.graph.ILookup> extends yfiles.view.ModelManager<T>{}
    export class SelectionIndicatorManager<T extends yfiles.graph.ILookup> {
      /**
       * Creates an instance that for all selected items in the model dynamically installs a selection render decoration.
       * @param canvas The canvas to add the selection marks to.
       * @param model The model that holds the domain.
       * @param selectionModel The model that determines which items are selected.
       */
      constructor(canvas:yfiles.view.CanvasComponent,model:yfiles.collections.IObservableCollection<T>,selectionModel:yfiles.view.ISelectionModel<T>);
      /**
       * Adds an item to the current selection.
       * @param item The item whose selection decorator will added.
       * @returns 
       */
      addSelection(item:T):yfiles.view.ICanvasObject;
      /**
       * Removes an item from the current selection.
       * @param item The item whose selection decorator will be removed.
       */
      removeSelection(item:T):void;
      /**
       * Gets or sets the selection model that determines which items are selected.
       * <p>
       * May be <code>null</code>, in which case no elements are considered selected.
       * </p>
       * @type {yfiles.view.ISelectionModel.<T>}
       */
      selectionModel:yfiles.view.ISelectionModel<T>;
      /**
       * Gets or sets the model that describes the domain for the selection.
       * <p>
       * Only item contained in this model can be rendered as selected.
       * </p>
       * @type {yfiles.collections.IObservableCollection.<T>}
       */
      model:yfiles.collections.IObservableCollection<T>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A container that can hold a number of {@link yfiles.view.SvgVisual}s.
     * <p>
     * Use this class in a {@link yfiles.view.IVisualCreator} implementation to group several {@link yfiles.view.SvgVisual}s.
     * </p>
     * <p>
     * The class cannot be used to group {@link yfiles.view.HtmlCanvasVisual}s.
     * </p>
     * @class
     * @extends {yfiles.view.SvgVisual}
     */
    export interface SvgVisualGroup extends yfiles.view.SvgVisual{}
    export class SvgVisualGroup {
      /**
       * Initializes a new instance of the {@link yfiles.view.SvgVisualGroup} class.
       */
      constructor();
      /**
       * Adds another {@link yfiles.view.SvgVisual} to the children collection of this instance.
       * @param child The child to add.
       */
      add(child:yfiles.view.SvgVisual):void;
      /**
       * Removes a previously added {@link yfiles.view.SvgVisual} from the children collection of this instance.
       * @param child The child to remove.
       */
      remove(child:yfiles.view.SvgVisual):void;
      /**
       * Gets or sets a transform that is applied to the children of the container.
       * @type {yfiles.geometry.Matrix}
       */
      transform:yfiles.geometry.Matrix;
      /**
       * Provides read and write access to the list of children.
       * @type {yfiles.collections.IList.<yfiles.view.SvgVisual>}
       */
      children:yfiles.collections.IList<yfiles.view.SvgVisual>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Renders a grid using various different {@link yfiles.view.GridVisualCreator#gridStyle styles}.
     * @see yfiles.view.GridInfo
     * @see yfiles.view.GridVisualCreator#gridStyle
     * @class
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface GridVisualCreator extends Object,yfiles.view.IVisualCreator{}
    export class GridVisualCreator {
      /**
       * Initializes a new instance of the {@link yfiles.view.GridVisualCreator} class.
       * @param gridInfo The grid info.
       */
      constructor(gridInfo:yfiles.view.GridInfo);
      /**
       * Initializes a new instance of the {@link yfiles.view.GridVisualCreator} class.
       * <p>
       * This initializes the {@link yfiles.view.GridVisualCreator#gridInfo} field using the given values.
       * </p>
       * @param horizontalSpacing The x spacing.
       * @param verticalSpacing The y spacing.
       */
      constructor(horizontalSpacing:number,verticalSpacing:number);
      /**
       * Initializes a new instance of the {@link yfiles.view.GridVisualCreator} class.
       * <p>
       * The spacing is initialized to 25 units.
       * </p>
       */
      constructor();
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the grid style that this implementation uses.
       * @default yfiles.view.GridStyle.DOTS
       * @type {yfiles.view.GridStyle}
       */
      gridStyle:yfiles.view.GridStyle;
      /**
       * Gets or sets the grid info that describes the geometry of the grid.
       * @type {yfiles.view.GridInfo}
       */
      gridInfo:yfiles.view.GridInfo;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.view.GridVisualCreator} is visible.
       * @default true
       * @type {boolean}
       */
      visible:boolean;
      /**
       * Gets or sets the {@link yfiles.view.GridVisualCreator#stroke} to use for painting the grid points.
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the visibility threshold.
       * <p>
       * This indicates the minimum distance in the view coordinate system between two grid points. If the distance becomes
       * smaller, the grid will not be rendered. The default value is <code>20.0d</code>.
       * </p>
       * @default 20.0
       * @type {number}
       */
      visibilityThreshold:number;
      static $class:yfiles.lang.Class;
    }
    export enum GridStyle{
      /**
       * For each grid point, there is a small dot rendered.
       */
      DOTS,
      /**
       * For each grid point, there is a small cross rendered.
       */
      CROSSES,
      /**
       * Draws horizontal and vertical lines through the grid points.
       */
      LINES
    }
    /**
     * Interface that is implemented for elements that can provide rendering bounds in the world coordinate system.
     * <p>
     * This information can be used by the rendering engine to increase rendering performance significantly.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 06 GetBounds</li>
     * </ul>
     * @see yfiles.view.IVisualCreator
     * @see yfiles.view.CanvasComponent
     * @see yfiles.view.ICanvasObjectDescriptor
     * @see yfiles.view.ICanvasObject
     * @interface
     */
    export interface IBoundsProvider extends Object{
      /**
       * Returns a tight rectangular area where the whole rendering would fit into.
       * <p>
       * If calculating the bounds is too expensive or the rendering is not bound to a certain area, this method may return
       * {@link yfiles.geometry.Rect#INFINITE}. If nothing is rendered, this method should return an empty rectangle, where either or
       * both the width and height is non-positive or {@link yfiles.geometry.Rect#EMPTY}.
       * </p>
       * @param context the context to calculate the bounds for
       * @returns the bounds or {@link #EMPTY} to indicate an unbound area
       * @abstract
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
    }
    var IBoundsProvider:{
      /**
       * Creates an instance that returns the given model as the bounds.
       * @param bounds the rectangle
       * @returns 
       * @static
       */
      fromRectangle?(bounds:yfiles.geometry.IRectangle):yfiles.view.IBoundsProvider;
      /**
       * An implementation that returns an {@link yfiles.view.IBoundsProvider} that yields an "Empty" bounds.
       * @const
       * @static
       * @type {yfiles.view.IBoundsProvider}
       */
      EMPTY?:yfiles.view.IBoundsProvider;
      /**
       * An implementation that returns an {@link yfiles.view.IBoundsProvider} that yields {@link yfiles.geometry.Rect#INFINITE "infinite"} bounds.
       * @const
       * @static
       * @type {yfiles.view.IBoundsProvider}
       */
      UNBOUNDED?:yfiles.view.IBoundsProvider;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is implemented for elements that can determine whether they might be visible in a given clipping
     * rectangle.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 09 IsVisible</li>
     * </ul>
     * @see yfiles.view.IVisualCreator
     * @see yfiles.view.CanvasComponent
     * @see yfiles.view.ICanvasObjectDescriptor
     * @see yfiles.view.ICanvasObject
     * @interface
     */
    export interface IVisibilityTestable extends Object{
      /**
       * Determines whether an element might intersect the visible region for a given context.
       * <p>
       * Conservative implementations can always return <code>true</code>.
       * </p>
       * @param context The context to determine the visibility for.
       * @param rectangle The visible region clip.
       * @returns <code>false</code> if and only if it is safe not to paint the element because it would not affect the given clipping region.
       * @abstract
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
    }
    var IVisibilityTestable:{
      /**
       * An implementation of a {@link yfiles.view.IVisibilityTestable} that always yields <code>true</code>.
       * @const
       * @static
       * @type {yfiles.view.IVisibilityTestable}
       */
      ALWAYS?:yfiles.view.IVisibilityTestable;
      /**
       * An implementation of a {@link yfiles.view.IVisibilityTestable} that always yields <code>false</code>.
       * @const
       * @static
       * @type {yfiles.view.IVisibilityTestable}
       */
      NEVER?:yfiles.view.IVisibilityTestable;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum VisualCachingPolicy{
      /**
       * Disables caching of {@link yfiles.view.Visual}s completely.
       */
      NEVER,
      /**
       * {@link yfiles.view.Visual}s will always be cached.
       * <p>
       * The cached reference will be kept until the corresponding {@link yfiles.view.ICanvasObject} is removed.
       * </p>
       */
      ALWAYS
    }
    /**
     * An interface that describes the current context for rendering routines.
     * @see yfiles.view.IVisualCreator
     * @interface
     * @implements {yfiles.view.ICanvasContext}
     */
    export interface IRenderContext extends Object,yfiles.view.ICanvasContext{
      /**
       * Notifies the context that the <code>visual</code> has been removed from the visual tree.
       * <p>
       * This has to be done in a {@link yfiles.view.IVisualCreator}'s {@link yfiles.view.IVisualCreator#updateVisual} method if a
       * {@link yfiles.view.IVisualCreator}'s {@link yfiles.view.IVisualCreator#createVisual} is called or a previously added visual will be
       * removed or replaced. Calling this method will call a previously {@link yfiles.view.IRenderContext#setDisposeCallback registered} dispose visual callback that the <code>visual</code> has
       * been removed and needs to free its resources.
       * </p>
       * <p>
       * If the <code>visual</code> has not been registered previously this method will do nothing.
       * </p>
       * @param visual The visual which has been removed.
       * @see yfiles.view.IRenderContext#setDisposeCallback
       * @see yfiles.view.IVisualCreator
       * @see yfiles.view.IVisualCreator#updateVisual
       */
      childVisualRemoved?(visual:yfiles.view.Visual):void;
      /**
       * Registers a visual if children have registered themselves for disposal.
       * <p>
       * This is necessary for visual creators which delegate to another visual creator to create one or more child visuals, like
       * style decorators.
       * </p>
       * <p>
       * Implementors have to add a call to this method at the end of their {@link yfiles.view.IVisualCreator#createVisual} and {@link yfiles.view.IVisualCreator#updateVisual}
       * methods.
       * </p>
       * <p>
       * The registered <code>callback</code> has to call {@link yfiles.view.IRenderContext#childVisualRemoved} on all child visuals of the <code>visual</code>.
       * </p>
       * @param visual The visual to be registered. This is the visual which is returned by the {@link #createVisual} or {@link #updateVisual}
       * method which calls this method.
       * @param callback A dispose visual callback which calls {@link #childVisualRemoved} on all child visuals of the
       * registered <code>visual</code>.
       */
      registerForChildrenIfNecessary?(visual:yfiles.view.Visual,callback:(context:yfiles.view.IRenderContext,removedVisual:yfiles.view.Visual,dispose:boolean)=>yfiles.view.Visual):void;
      /**
       * Registers a <code>callback</code> which will be called when the <code>visual</code> will be removed from the visual tree.
       * <p>
       * This applies to a {@link yfiles.view.IVisualCreator}'s {@link yfiles.view.IVisualCreator#createVisual} method if the created <code>visual</code>
       * allocates resources which should be freed after the visual is no longer used. In this case the visual has to be
       * registered together with a dispose visual callback which can free the allocated resources. The callback will be called
       * by the framework when the visual is removed from the visual tree.
       * </p>
       * <p>
       * The callback can also be used to support caching for temporarily removed visuals. See dispose visual callback.
       * </p>
       * @param visual The visual which needs to be disposed after removal.
       * @param callback A callback which will dispose the <code>visual</code>.
       * @see yfiles.view.IVisualCreator
       * @see yfiles.view.IVisualCreator#createVisual
       */
      setDisposeCallback?(visual:yfiles.view.Visual,callback:(context:yfiles.view.IRenderContext,removedVisual:yfiles.view.Visual,dispose:boolean)=>yfiles.view.Visual):void;
      /**
       * Converts the given set of world coordinates to a coordinate pair that can be used to render within the {@link yfiles.view.IRenderContext#viewTransform}.
       * @param worldPoint The coordinates in the world coordinate system.
       * @returns The coordinates in the view coordinate system.
       * @see yfiles.view.IRenderContext#viewTransform
       * @abstract
       */
      toViewCoordinates(worldPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * The clipping rectangle in world coordinates.
       * <p>
       * Rendering outside of this clip may be skipped altogether.
       * </p>
       * @abstract
       * @type {yfiles.geometry.Rect}
       */
      clip:yfiles.geometry.Rect;
      /**
       * A transform that can be added to the transform list of a visual to be displayed in the original coordinate system (the
       * view coordinate system).
       * <p>
       * In contrast to the world coordinate system where the logical coordinates of the elements that are managed by a {@link yfiles.view.CanvasComponent}
       * (for example, graph elements like node and edges), the view coordinate system is used to display visualizations that are
       * independent to scale (zoom) or translation transformations (like handles or selection indicators).
       * </p>
       * @abstract
       * @type {yfiles.geometry.Matrix}
       */
      viewTransform:yfiles.geometry.Matrix;
      /**
       * Provides access to the <code>defs</code> element in the SVG DOM that is managed by this instance.
       * @see yfiles.view.IRenderContext#svgDefsManager
       * @abstract
       * @type {Element}
       */
      defsElement:Element;
      /**
       * Provides access to the {@link yfiles.view.SvgDefsManager} that manages this instance.
       * @abstract
       * @type {yfiles.view.SvgDefsManager}
       */
      svgDefsManager:yfiles.view.SvgDefsManager;
    }
    var IRenderContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Factory interface which is used by implementations that know how to create {@link yfiles.view.Visual}s for rendering into a
     * given {@link yfiles.view.IRenderContext}.
     * @see yfiles.view.CanvasComponent
     * @see yfiles.view.ICanvasObjectDescriptor
     * @see yfiles.view.ICanvasObject
     * @interface
     */
    export interface IVisualCreator extends Object{
      /**
       * This method is called by the framework to create a {@link yfiles.view.Visual} that will be included into the
       * {@link yfiles.view.IRenderContext}.
       * <p>
       * {@link yfiles.view.CanvasComponent} uses this interface through the {@link yfiles.view.ICanvasObjectDescriptor} to populate the visual
       * canvas object tree.
       * </p>
       * @param context The context that describes where the visual will be used.
       * @returns The visual to include in the canvas object visual tree. This may be <code>null</code>.
       * @see yfiles.view.IVisualCreator#updateVisual
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * This method updates or replaces a previously created {@link yfiles.view.Visual} for inclusion in the
       * {@link yfiles.view.IRenderContext}.
       * <p>
       * The {@link yfiles.view.CanvasComponent} uses this method to give implementations a chance to update an existing Visual that has
       * previously been created by the same instance during a call to {@link yfiles.view.IVisualCreator#createVisual}. Implementation
       * may update the <code>oldVisual</code> and return that same reference, or create a new visual and return the new instance or <code>null</code>.
       * </p>
       * @param context The context that describes where the visual will be used in.
       * @param oldVisual The visual instance that had been returned the last time the {@link #createVisual} method was called
       * on this instance.
       * @returns <code>oldVisual</code>, if this instance modified the visual, or a new visual that should replace the existing one in the canvas
       * object visual tree.
       * @see yfiles.view.IVisualCreator#createVisual
       * @see yfiles.view.ICanvasObjectDescriptor
       * @see yfiles.view.CanvasComponent
       * @abstract
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
    }
    var IVisualCreator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A void/null implementation of the {@link yfiles.view.IVisualCreator} interface that does nothing.
     * <p>
     * Use {@link yfiles.view.VoidVisualCreator#INSTANCE} for methods that require non-<code>null</code> {@link yfiles.view.IVisualCreator} implementations
     * if you want to have no visualization.
     * </p>
     * @class
     * @implements {yfiles.view.IVisualCreator}
     * @final
     */
    export interface VoidVisualCreator extends Object,yfiles.view.IVisualCreator{}
    export class VoidVisualCreator {
      /**
       * Returns <code>null</code>.
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Returns <code>null</code>.
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * An immutable and shareable convenience instance of the {@link yfiles.view.VoidVisualCreator} class.
       * @const
       * @static
       * @type {yfiles.view.IVisualCreator}
       */
      static INSTANCE:yfiles.view.IVisualCreator;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class for the {@link yfiles.view.ISelectionIndicatorInstaller}, {@link yfiles.view.IFocusIndicatorInstaller} and {@link yfiles.view.IHighlightIndicatorInstaller}
     * implementations that draws an oriented rectangle using a {@link yfiles.view.OrientedRectangleIndicatorInstaller#template}.
     * <p>
     * In the {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject} method, this implementation will add either a fixed {@link yfiles.geometry.IOrientedRectangle}
     * instance that is passed in the constructor as a parameter or the return value of the method {@link yfiles.view.OrientedRectangleIndicatorInstaller#getRectangle}
     * if the former does not exist (i.e. the parameterless constructor was used to instantiate this class).
     * </p>
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     * @implements {yfiles.view.IHighlightIndicatorInstaller}
     * @implements {yfiles.view.IFocusIndicatorInstaller}
     */
    export interface OrientedRectangleIndicatorInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller,yfiles.view.IHighlightIndicatorInstaller,yfiles.view.IFocusIndicatorInstaller{}
    export class OrientedRectangleIndicatorInstaller {
      /**
       * Initializes a new instance of the {@link yfiles.view.OrientedRectangleIndicatorInstaller} class with fixed bounds and a {@link string}.
       * <p>
       * The given {@link yfiles.geometry.IOrientedRectangle} is used for all items as object that is added to the {@link yfiles.view.ICanvasObjectGroup}
       * in the {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject} method.
       * </p>
       * <p>
       * If the {@link yfiles.view.OrientedRectangleIndicatorInstaller#template} property is not set, the <code>templateKey</code> is used to lookup a {@link yfiles.view.IVisualTemplate} in the resources of the
       * {@link yfiles.view.CanvasComponent}. If there is none, a default templated is used.
       * </p>
       * @param fixedBounds
       * @param [templateKey=null]
       */
      constructor(fixedBounds:yfiles.geometry.IOrientedRectangle,templateKey?:string);
      /**
       * Initializes a new instance of the {@link yfiles.view.OrientedRectangleIndicatorInstaller} class without fixed bounds.
       * <p>
       * The object that is created through this constructor relies on the value returned by the {@link yfiles.view.OrientedRectangleIndicatorInstaller#getRectangle}
       * method to {@link yfiles.view.ICanvasObjectGroup#addChild add} to the {@link yfiles.view.ICanvasObjectGroup} in the {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject} method.
       * </p>
       */
      constructor();
      /**
       * 
       * @param context
       * @param group
       * @param item
       * @returns 
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Factory method that returns an {@link yfiles.geometry.IOrientedRectangle} for a given user object.
       * <p>
       * This method is called for the object to {@link yfiles.view.ICanvasObjectGroup#addChild add} to the
       * {@link yfiles.view.ICanvasObjectGroup}, if there are no fixed bounds set on this instance (i.e. the parameterless constructor
       * was used to instantiate this class).
       * </p>
       * <p>
       * This implementation returns null for every given object.
       * </p>
       * @param item
       * @returns An {@link } for the given item that is {@link #addChild added} to the {@link }.
       * @protected
       */
      getRectangle(item:Object):yfiles.geometry.IOrientedRectangle;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} for drawing the selection indicator.
       * @const
       * @static
       * @type {string}
       */
      static SELECTION_TEMPLATE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} for drawing the focus indicator.
       * @const
       * @static
       * @type {string}
       */
      static FOCUS_TEMPLATE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} for drawing the highlight indicator.
       * @const
       * @static
       * @type {string}
       */
      static HIGHLIGHT_TEMPLATE_KEY:string;
      /**
       * Gets or sets the {@link yfiles.view.IVisualTemplate} to use for this indicator.
       * <p>
       * If this property is not set, the {@link string} passed in the constructor is used to lookup a {@link yfiles.view.IVisualTemplate} in the
       * resources of the {@link yfiles.view.CanvasComponent}. If there is none, a default templated is used.
       * </p>
       * @type {yfiles.view.IVisualTemplate}
       */
      template:yfiles.view.IVisualTemplate;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple {@link yfiles.view.ISelectionIndicatorInstaller} implementation that draws an ellipse centered at a given point in
     * world coordinate space.
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     */
    export interface PointSelectionIndicatorInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller{}
    export class PointSelectionIndicatorInstaller {
      /**
       * Creates a new instance that indicates a point.
       * @param point The point to use for the center of the painting.
       */
      constructor(point:yfiles.geometry.IPoint);
      /**
       * 
       * @param context
       * @param group
       * @param item
       * @returns 
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Factory method that retrieves the center for a given user object.
       * <p>
       * This implementation simply returns the value provided to the constructor.
       * </p>
       * @param userObject
       * @returns 
       * @protected
       */
      getCenterPoint(userObject:Object):yfiles.geometry.IPoint;
      /**
       * Gets or sets the {@link yfiles.view.Stroke} to use for drawing the point in the view coordinate system.
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the {@link yfiles.view.Fill} to use for filling the point's ellipse in the view coordinate system.
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      static $class:yfiles.lang.Class;
    }
    /**
     * A basis implementation of {@link yfiles.view.ISelectionIndicatorInstaller}, {@link yfiles.view.IFocusIndicatorInstaller} and {@link yfiles.view.IHighlightIndicatorInstaller}
     * that draws a rectangle using a {@link yfiles.view.RectangleIndicatorInstaller#template}.
     * <p>
     * In the {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject} method, this implementation will add either a fixed {@link yfiles.geometry.IRectangle}
     * instance that is passed in the constructor as a parameter or the return value of the method {@link yfiles.view.RectangleIndicatorInstaller#getRectangle}
     * if the former does not exist (i.e. the parameterless constructor was used to instantiate this class).
     * </p>
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     * @implements {yfiles.view.IHighlightIndicatorInstaller}
     * @implements {yfiles.view.IFocusIndicatorInstaller}
     */
    export interface RectangleIndicatorInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller,yfiles.view.IHighlightIndicatorInstaller,yfiles.view.IFocusIndicatorInstaller{}
    export class RectangleIndicatorInstaller {
      /**
       * Initializes a new instance of the {@link yfiles.view.RectangleIndicatorInstaller} class with fixed bounds and a {@link string}.
       * <p>
       * The given {@link yfiles.geometry.IOrientedRectangle} is used for all items as object that is added to the {@link yfiles.view.ICanvasObjectGroup}
       * in the {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject} method.
       * </p>
       * <p>
       * If the {@link yfiles.view.RectangleIndicatorInstaller#template} property is not set, the <code>resourceKey</code> is used to lookup a {@link yfiles.view.IVisualTemplate} in the resources of the
       * {@link yfiles.view.CanvasComponent}. If there is none, a default templated is used.
       * </p>
       * @param fixedBounds
       * @param [resourceKey=null]
       */
      constructor(fixedBounds:yfiles.geometry.IRectangle,resourceKey?:string);
      /**
       * Initializes a new instance of the {@link yfiles.view.RectangleIndicatorInstaller} class without fixed bounds.
       * <p>
       * The object that is created through this constructor relies on the value returned by the {@link yfiles.view.RectangleIndicatorInstaller#getRectangle}
       * method to {@link yfiles.view.ICanvasObjectGroup#addChild add} to the {@link yfiles.view.ICanvasObjectGroup} in the {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject} method.
       * </p>
       */
      constructor();
      /**
       * 
       * @param context
       * @param group
       * @param item
       * @returns 
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Factory method that returns an {@link yfiles.geometry.IRectangle} for a given user object.
       * <p>
       * This method is called for the object to {@link yfiles.view.ICanvasObjectGroup#addChild add} to the
       * {@link yfiles.view.ICanvasObjectGroup}, if there are no fixed bounds set on this instance (i.e. the parameterless constructor
       * was used to instantiate this class).
       * </p>
       * <p>
       * This implementation returns null for every given object.
       * </p>
       * @param userObject
       * @returns An {@link } for the given item that is {@link #addChild added} to the {@link }.
       * @protected
       */
      getRectangle(userObject:Object):yfiles.geometry.IRectangle;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} for drawing the selection indicator.
       * @const
       * @static
       * @type {string}
       */
      static SELECTION_TEMPLATE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} for drawing the focus indicator.
       * @const
       * @static
       * @type {string}
       */
      static FOCUS_TEMPLATE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} for drawing the highlight indicator.
       * @const
       * @static
       * @type {string}
       */
      static HIGHLIGHT_TEMPLATE_KEY:string;
      /**
       * Gets or sets the {@link yfiles.view.IVisualTemplate} to use for this indicator.
       * <p>
       * If this property is not set, the {@link string} passed in the constructor is used to lookup a {@link yfiles.view.IVisualTemplate} in the
       * resources of the {@link yfiles.view.CanvasComponent}. If there is none, a default templated is used.
       * </p>
       * @type {yfiles.view.IVisualTemplate}
       */
      template:yfiles.view.IVisualTemplate;
      static $class:yfiles.lang.Class;
    }
    /**
     * Helper class used by {@link yfiles.view.CanvasComponent} to limit the interactive movement of the viewport.
     * <p>
     * The {@link yfiles.view.CanvasComponent#viewportLimiter} instance should be consulted whenever the user tries to change the viewport. Using method {@link yfiles.view.ViewportLimiter#limitViewport}
     * the desired viewport can be validated by the implementation. Note that setting the {@link yfiles.view.CanvasComponent#zoom} property or {@link yfiles.view.CanvasComponent#viewPoint} property will not be
     * influenced by implementations of this class. It is up to the caller to perform verification.
     * </p>
     * @see yfiles.view.ViewportLimiter#bounds
     * @class
     */
    export interface ViewportLimiter extends Object{}
    export class ViewportLimiter {
      /**
       * Initializes a new instance of the {@link yfiles.view.ViewportLimiter} class.
       */
      constructor();
      /**
       * Gets the to bounds that should be honored for the upcoming call.
       * <p>
       * This method is used as callback by {@link yfiles.view.ViewportLimiter#limitViewport} for each request. The default
       * implementation just yields {@link yfiles.view.ViewportLimiter#bounds}.
       * </p>
       * @param canvas The canvas control.
       * @param suggestedViewport The suggested viewport.
       * @returns The bounds to honor or <code>null</code>.
       * @protected
       */
      getCurrentBounds(canvas:yfiles.view.CanvasComponent,suggestedViewport:yfiles.geometry.Rect):yfiles.geometry.Rect;
      /**
       * Inspects the desired or suggested new viewport for the given control and returns a valid viewport to use.
       * @param canvas The canvas control on which the viewport should be applied.
       * @param suggestedViewport The suggested viewport.
       * @returns The viewport that should be used.
       */
      limitViewport(canvas:yfiles.view.CanvasComponent,suggestedViewport:yfiles.geometry.Rect):yfiles.geometry.Rect;
      /**
       * Gets or sets a value indicating whether both dimensions of {@link yfiles.view.ViewportLimiter#bounds} need to be honored.
       * @default true
       * @type {boolean}
       */
      honorBothDimensions:boolean;
      /**
       * Gets or sets the maximal allowed navigable bounds for the viewport.
       * @type {yfiles.geometry.Rect}
       */
      bounds:yfiles.geometry.Rect;
      static $class:yfiles.lang.Class;
    }
    export enum DragDropKeyStates{
      /**
       * The left mouse button is down or a touch gesture is in progress.
       */
      POINTER_DOWN,
      /**
       * No button and no key is pressed.
       */
      NONE,
      /**
       * The shift key is down.
       */
      SHIFT_KEY,
      /**
       * The control key is down.
       */
      CONTROL_KEY,
      /**
       * The alt key is down.
       */
      ALT_KEY
    }
    /**
     * A simple {@link yfiles.view.ICanvasObjectDescriptor} that can be used to draw {@link yfiles.input.IPortCandidate} instances in a
     * {@link yfiles.view.CanvasComponent}.
     * @class
     * @implements {yfiles.view.ICanvasObjectDescriptor}
     * @implements {yfiles.view.IVisualCreator}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     */
    export interface DefaultPortCandidateDescriptor extends Object,yfiles.view.ICanvasObjectDescriptor,yfiles.view.IVisualCreator,yfiles.view.IBoundsProvider,yfiles.input.IHitTestable,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable{}
    export class DefaultPortCandidateDescriptor {
      /**
       * Instantiates a new descriptor.
       */
      constructor();
      /**
       * 
       * @param forUserObject
       * @returns 
       */
      getBoundsProvider(forUserObject:Object):yfiles.view.IBoundsProvider;
      /**
       * 
       * @param forUserObject
       * @returns 
       */
      getHitTestable(forUserObject:Object):yfiles.input.IHitTestable;
      /**
       * 
       * @param forUserObject
       * @returns 
       */
      getVisibilityTestable(forUserObject:Object):yfiles.view.IVisibilityTestable;
      /**
       * 
       * @param forUserObject
       * @returns 
       */
      getVisualCreator(forUserObject:Object):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param canvasObject
       * @returns 
       */
      isDirty(context:yfiles.view.ICanvasContext,canvasObject:yfiles.view.ICanvasObject):boolean;
      /**
       * Sets the {@link yfiles.view.IVisualTemplate} to use for the given {@link string} by this instance.
       * <p>
       * This method can be used to reconfigure the visualization for the various keys that are declared by this class.
       * </p>
       * @param resourceKey The key to reconfigure. This is one of
       * {@link #CANDIDATE_DRAWING_VALID_NON_FOCUSED_KEY},
       * {@link #CANDIDATE_DRAWING_INVALID_NON_FOCUSED_KEY},
       * {@link #CANDIDATE_DRAWING_INVALID_FOCUSED_KEY},
       * {@link #CANDIDATE_DRAWING_VALID_FOCUSED_KEY}.
       * @param template The template to use instead of the default.
       */
      setTemplate(resourceKey:string,template:yfiles.view.IVisualTemplate):void;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.view.DefaultPortCandidateDescriptor} to look up a {@link yfiles.view.IVisualTemplate}
       * that will be used for rendering a valid, non-focused {@link yfiles.input.IPortCandidate}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static CANDIDATE_DRAWING_VALID_NON_FOCUSED_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.view.DefaultPortCandidateDescriptor} to look up a {@link yfiles.view.IVisualTemplate}
       * that will be used for rendering a valid, focused {@link yfiles.input.IPortCandidate}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static CANDIDATE_DRAWING_VALID_FOCUSED_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.view.DefaultPortCandidateDescriptor} to look up a {@link yfiles.view.IVisualTemplate}
       * that will be used for rendering an invalid, non-focused {@link yfiles.input.IPortCandidate}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static CANDIDATE_DRAWING_INVALID_NON_FOCUSED_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.view.DefaultPortCandidateDescriptor} to look up a {@link yfiles.view.IVisualTemplate}
       * that will be used for rendering a invalid, focused {@link yfiles.input.IPortCandidate}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static CANDIDATE_DRAWING_INVALID_FOCUSED_KEY:string;
      /**
       * Gets or sets a property that determines whether this descriptor uses the "current" candidates.
       * @type {boolean}
       */
      currentCandidate:boolean;
      /**
       * Gets or sets the logical size of the candidate.
       * <p>
       * This size is used for hit testing, visibility tests, etc. The default is <code>3</code>.
       * </p>
       * @type {number}
       */
      size:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Manages the visual appearance of an {@link yfiles.graph.IGraph} instance in a {@link yfiles.view.CanvasComponent}.
     * <p>
     * This class monitors the structural changes of an {@link yfiles.graph.IGraph} implementation and installs the necessary {@link yfiles.view.ICanvasObject}
     * instance in the canvas for each item in the graph. Note that {@link yfiles.view.GraphComponent} has an instance of this type
     * built-in already and usually it is not necessary to create an own one. This class will also {@link yfiles.view.GraphModelManager#useHierarchicNodeNesting use the node hierarchy} to
     * nest the nodes {@link yfiles.view.GraphModelManager#useHierarchicEdgeNesting and edges} accordingly.
     * </p>
     * @see yfiles.view.GraphModelManager#nodeGroup
     * @see yfiles.view.GraphModelManager#useHierarchicEdgeNesting
     * @see yfiles.view.GraphModelManager#useHierarchicNodeNesting
     * @class
     */
    export interface GraphModelManager extends Object{}
    export class GraphModelManager {
      /**
       * Creates a manager instance for the given control.
       * <p>
       * The contents of the graph will be added to given group that is must be part of the canvas.
       * </p>
       * <p>
       * In order to get anything useful out of this instance the {@link yfiles.view.GraphModelManager#graph} property should be set to an instance after construction.
       * </p>
       * @param canvas the canvas to manage the visual appearance of a graph for.
       * @param contentGroup the group to add the graph's visual content to. Can be null, in which case {@link #contentGroup} should be set afterwards or {@link #createContentGroup}
       * will automatically create a group lazily.
       */
      constructor(canvas:yfiles.view.CanvasComponent,contentGroup:yfiles.view.ICanvasObjectGroup);
      /**
       * Creates a manager instance for the given control.
       * <p>
       * The contents of the graph will be added to a newly created group that is added to the root of the scene graph.
       * </p>
       * <p>
       * In order to get anything useful out of this instance the {@link yfiles.view.GraphModelManager#graph} property should be set to an instance after construction.
       * </p>
       * @param canvas the canvas to manage the visual appearance of a graph for.
       */
      constructor(canvas:yfiles.view.CanvasComponent);
      /**
       * Compares two items with respect to their visibility order.
       * @param item1 the first item to compare
       * @param item2 the second item to compare
       * @returns an integer value indicating the relative rendering order of the two items
       * @see yfiles.view.GraphModelManager#comparer
       */
      compare(item1:yfiles.graph.IModelItem,item2:yfiles.graph.IModelItem):number;
      /**
       * Factory method that creates the {@link yfiles.view.GraphModelManager#contentGroup}
       * <p>
       * The default implementation will just add another group to the {@link yfiles.view.CanvasComponent}.
       * </p>
       * @returns The group to use for adding the content.
       * @protected
       */
      createContentGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the {@link yfiles.view.GraphModelManager#edgeGroup} property.
       * <p>
       * This method will be called upon first access to the property.
       * </p>
       * @returns a new group that has been added to the {@link #contentGroup}
       * @protected
       */
      createEdgeGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the {@link yfiles.view.GraphModelManager#edgeLabelGroup} property.
       * <p>
       * This method will be called upon first access to the property.
       * </p>
       * @returns a new group that has been added to the {@link #contentGroup}
       * @protected
       */
      createEdgeLabelGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Gets a {@link yfiles.input.IHitTester.<T>} that can be used to enumerate hits on the canvas at a given world coordinate position.
       * <p>
       * This enumerator will enumerate hits in the order so that the visually topmost element will be reported first.
       * </p>
       * Type parameter <code>T</code>.
       * @param itemType
       * @returns 
       * @template T
       */
      createHitTester<T extends yfiles.graph.IModelItem>(itemType:yfiles.lang.Class):yfiles.input.IHitTester<T>;
      /**
       * Factory method for the {@link yfiles.view.GraphModelManager#nodeGroup} property.
       * <p>
       * This method will be called upon first access to the property.
       * </p>
       * @returns a new group that has been added to the {@link #contentGroup}
       * @protected
       */
      createNodeGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the {@link yfiles.view.GraphModelManager#nodeLabelGroup} property.
       * <p>
       * This method will be called upon first access to the property.
       * </p>
       * @returns a new group that has been added to the {@link #contentGroup}
       * @protected
       */
      createNodeLabelGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Factory method for the {@link yfiles.view.GraphModelManager#portGroup} property.
       * <p>
       * This method will be called upon first access to the property.
       * </p>
       * @returns a new group that has been added to the {@link #contentGroup}
       * @protected
       */
      createPortGroup():yfiles.view.ICanvasObjectGroup;
      /**
       * Retrieves the {@link yfiles.view.ICanvasObject} for a given model item managed by this instance.
       * @param item the item
       * @returns the canvas object for the given item or <code>null</code>
       * @see yfiles.view.ItemModelManager.<T>#getCanvasObject
       */
      getCanvasObject(item:yfiles.graph.IModelItem):yfiles.view.ICanvasObject;
      /**
       * Retrieves the Canvas Object group to use for the given label.
       * <p>
       * This implementation is called by the default {@link yfiles.view.GraphModelManager#nodeLabelDescriptor} and {@link yfiles.view.GraphModelManager#edgeLabelDescriptor} instances.
       * </p>
       * @param label The label that will be installed in the canvas.
       * @returns The group to use.
       * @protected
       */
      getCanvasObjectGroup(label:yfiles.graph.ILabel):yfiles.view.ICanvasObjectGroup;
      /**
       * Retrieves the Canvas Object group to use for the given port.
       * <p>
       * This implementation is called by the default {@link yfiles.view.GraphModelManager#portDescriptor} instance.
       * </p>
       * @param port The port that will be installed in the canvas.
       * @returns The group to use.
       * @protected
       */
      getCanvasObjectGroup(port:yfiles.graph.IPort):yfiles.view.ICanvasObjectGroup;
      /**
       * Retrieves the Canvas Object group to use for the given edge.
       * <p>
       * This implementation is called by the default {@link yfiles.view.GraphModelManager#edgeDescriptor} instance.
       * </p>
       * @param edge The edge that will be installed in the canvas.
       * @returns The group to use.
       * @protected
       */
      getCanvasObjectGroup(edge:yfiles.graph.IEdge):yfiles.view.ICanvasObjectGroup;
      /**
       * Retrieves the Canvas Object group to use for the given node.
       * <p>
       * This implementation is called by the default {@link yfiles.view.GraphModelManager#nodeDescriptor} instance.
       * </p>
       * @param node The node that will be installed in the canvas.
       * @returns The group to use.
       * @protected
       */
      getCanvasObjectGroup(node:yfiles.graph.INode):yfiles.view.ICanvasObjectGroup;
      /**
       * Retrieves the corresponding {@link yfiles.graph.IModelItem} for the given canvas object.
       * @param canvasObject the instance to use for the query
       * @returns The item that is associated with the object or <code>null</code>.
       */
      getModelItem(canvasObject:yfiles.view.ICanvasObject):yfiles.graph.IModelItem;
      /**
       * Enumerates hits on the canvas at a given world coordinate position for a given context.
       * <p>
       * The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
       * enumeration is performed lazily.
       * </p>
       * @param context The context to provide to {@link #isHit}
       * @param location the world coordinates to perform the hit test
       * @param root The group to search the subtree of.
       * @returns an enumerable that will enumerate the hits at the given coordinates
       */
      hitElementsAt(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,root:yfiles.view.ICanvasObjectGroup):yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Enumerates hits on the canvas at a given world coordinate position.
       * <p>
       * The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
       * enumeration is performed lazily.
       * </p>
       * @param location the world coordinates to perform the hit test
       * @returns an enumerable that will enumerate the hits at the given coordinates
       */
      hitElementsAt(location:yfiles.geometry.Point):yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Called when the {@link yfiles.view.GraphModelManager#graph} property changes.
       * <p>
       * This method registers the collections of the graph instance with the {@link yfiles.view.CollectionModelManager.<T> mechanism} that manages the mapping between the
       * elements and the
       * {@link yfiles.view.ICanvasObject}s. Subclasses should call the base method if they want to use the default mechanisms.
       * </p>
       * @param oldGraph The old graph instance.
       * @param newGraph The new graph instance.
       * @protected
       */
      onGraphChanged(oldGraph:yfiles.graph.IGraph,newGraph:yfiles.graph.IGraph):void;
      /**
       * Enumerates hits on the canvas at a given world coordinate position in a given context.
       * <p>
       * The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
       * enumeration is performed lazily.
       * </p>
       * Type parameter <code>T</code>.
       * @param modelItemType
       * @param context The context to provide to {@link #isHit}
       * @param location the world coordinates to perform the hit test
       * @param root The group to search the subtree of.
       * @returns an enumerable that will enumerate the hits at the given coordinates
       * @template T
       */
      typedHitElementsAt<T extends yfiles.graph.IModelItem>(modelItemType:yfiles.lang.Class,context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,root:yfiles.view.ICanvasObjectGroup):yfiles.collections.IEnumerable<T>;
      /**
       * Enumerates hits on the canvas at a given world coordinate position.
       * <p>
       * The instance returned will enumerate hits in the order so that the topmost visual element will be reported first. The
       * enumeration is performed lazily.
       * </p>
       * Type parameter <code>T</code>.
       * @param itemType
       * @param location the world coordinates to perform the hit test
       * @returns an enumerable that will enumerate the hits at the given coordinates
       * @template T
       */
      typedHitElementsAt<T extends yfiles.graph.IModelItem>(itemType:yfiles.lang.Class,location:yfiles.geometry.Point):yfiles.collections.IEnumerable<T>;
      /**
       * {@link yfiles.view.CollectionModelManager.<T>#update Updates} the visual representation of the given item.
       * @param item The item to update the visual representation
       * @see yfiles.view.CollectionModelManager.<T>#update
       */
      update(item:yfiles.graph.IModelItem):void;
      /**
       * Gets the {@link yfiles.view.ICanvasObjectGroup} instance that will be used for installing node styles.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphModelManager#createNodeGroup}
       * will be called. Note that in the case of {@link yfiles.view.GraphModelManager#useHierarchicNodeNesting hierarchic node nesting}, nodes will not be installed at the top-level of this
       * group. Rather a more complicated nesting of dynamically created {@link yfiles.view.ICanvasObjectGroup}s will be used.
       * </p>
       * @see yfiles.view.GraphModelManager#getCanvasObjectGroup
       * @see yfiles.view.GraphModelManager#nodeDescriptor
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      nodeGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets the {@link yfiles.view.ICanvasObjectGroup} instance that will be used for installing edge styles.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphModelManager#createEdgeGroup}
       * will be called. Note that in the case of {@link yfiles.view.GraphModelManager#useHierarchicNodeNesting hierarchically nested graphs} edges will only be put directly in this
       * {@link yfiles.view.ICanvasObjectGroup}, if the {@link yfiles.view.GraphModelManager#useHierarchicEdgeNesting} property is set to <code>false</code>. Otherwise edges will actually be nested in the subtree
       * of the {@link yfiles.view.GraphModelManager#nodeGroup} according to the node nesting.
       * </p>
       * @see yfiles.view.GraphModelManager#getCanvasObjectGroup
       * @see yfiles.view.GraphModelManager#edgeDescriptor
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      edgeGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets the {@link yfiles.view.ICanvasObjectGroup} instance that will be used for installing edge label styles.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphModelManager#createEdgeLabelGroup}
       * will be called.
       * </p>
       * @see yfiles.view.GraphModelManager#getCanvasObjectGroup
       * @see yfiles.view.GraphModelManager#edgeLabelDescriptor
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      edgeLabelGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets the {@link yfiles.view.ICanvasObjectGroup} instance that will be used for installing node label styles.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphModelManager#createNodeLabelGroup}
       * will be called.
       * </p>
       * @see yfiles.view.GraphModelManager#getCanvasObjectGroup
       * @see yfiles.view.GraphModelManager#nodeLabelDescriptor
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      nodeLabelGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets the {@link yfiles.view.ICanvasObjectGroup} instance that will be used for installing port styles.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.view.GraphModelManager#createPortGroup}
       * will be called.
       * </p>
       * @see yfiles.view.GraphModelManager#getCanvasObjectGroup
       * @see yfiles.view.GraphModelManager#portDescriptor
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      portGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} instance used for installing an {@link yfiles.graph.IEdge} into the
       * canvas.
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      edgeDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} instance used for installing an {@link yfiles.graph.IPort} into the
       * canvas.
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      portDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} instance used for installing an {@link yfiles.graph.ILabel} of an {@link yfiles.graph.IEdge}
       * into the canvas.
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      edgeLabelDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} instance used for installing an {@link yfiles.graph.ILabel} of an {@link yfiles.graph.INode}
       * into the canvas.
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      nodeLabelDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Determines whether this instance should nest the edges accordingly if the graph is grouped.
       * <p>
       * Hierarchic nesting for edges requires that {@link yfiles.view.GraphModelManager#useHierarchicNodeNesting hierarchic nesting for nodes} is enabled as well. If both properties are
       * enabled (the default) the edges will actually be interleaved with the nodes in the nested canvas object subtree so the {@link yfiles.view.GraphModelManager#edgeGroup}
       * will be empty.
       * </p>
       * @see yfiles.view.GraphModelManager#useHierarchicNodeNesting
       * @type {boolean}
       */
      useHierarchicEdgeNesting:boolean;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} instance used for installing an {@link yfiles.graph.INode} into the
       * canvas.
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      nodeDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the content group this manager instance uses to add visual representation of the graph to.
       * @type {yfiles.view.ICanvasObjectGroup}
       */
      contentGroup:yfiles.view.ICanvasObjectGroup;
      /**
       * Gets an {@link yfiles.collections.IComparer.<T>} instance that can be used to compare two model items with respect to their visual display order.
       * <p>
       * An item is considered to be greater than another item if it is rendered on top of it.
       * </p>
       * @type {yfiles.collections.IComparer.<yfiles.graph.IModelItem>}
       */
      comparer:yfiles.collections.IComparer<yfiles.graph.IModelItem>;
      /**
       * Gets a {@link yfiles.input.IHitTester.<T>} that can be used to enumerate hits on the canvas at a given world coordinate position.
       * <p>
       * This enumerator will enumerate hits in the order so that the visually topmost element will be reported first.
       * </p>
       * @type {yfiles.input.IHitTester.<yfiles.graph.IModelItem>}
       */
      hitTester:yfiles.input.IHitTester<yfiles.graph.IModelItem>;
      /**
       * Gets or sets the graph this manager manages.
       * @see yfiles.view.GraphModelManager#onGraphChanged
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Determines whether this instance should nest the nodes {@link yfiles.view.GraphModelManager#useHierarchicEdgeNesting and optionally edges} accordingly if the graph is grouped.
       * @type {boolean}
       */
      useHierarchicNodeNesting:boolean;
      /**
       * The immutable instance of the default {@link yfiles.view.ICanvasObjectDescriptor} for {@link yfiles.graph.INode}s which simply defers to
       * the node's {@link yfiles.styles.INodeStyle}.
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      static DEFAULT_NODE_DESCRIPTOR:yfiles.view.ICanvasObjectDescriptor;
      /**
       * The immutable instance of the default {@link yfiles.view.ICanvasObjectDescriptor} for {@link yfiles.graph.IEdge}s which simply defers to
       * the edge's {@link yfiles.styles.IEdgeStyle}.
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      static DEFAULT_EDGE_DESCRIPTOR:yfiles.view.ICanvasObjectDescriptor;
      /**
       * The immutable instance of the default {@link yfiles.view.ICanvasObjectDescriptor} for {@link yfiles.graph.IPort}s which simply defers to
       * the port's {@link yfiles.styles.IPortStyle}.
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      static DEFAULT_PORT_DESCRIPTOR:yfiles.view.ICanvasObjectDescriptor;
      /**
       * The immutable instance of the default {@link yfiles.view.ICanvasObjectDescriptor} for {@link yfiles.graph.ILabel}s which simply defers
       * to the label's {@link yfiles.styles.ILabelStyle}.
       * @const
       * @static
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      static DEFAULT_LABEL_DESCRIPTOR:yfiles.view.ICanvasObjectDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.view.IObstacleProvider} interface that uses the {@link yfiles.graph.IGraph#edges edges} and {@link yfiles.graph.IGraph#nodes nodes} of an {@link yfiles.graph.IGraph}
     * as obstacles.
     * <p>
     * The {@link yfiles.view.BridgeManager} then calculates bridges for edge paths that cross these obstacles.
     * </p>
     * <p>
     * This class can be used to add bridge support via the {@link yfiles.view.BridgeManager} to a rendered {@link yfiles.graph.IGraph}.
     * </p>
     * @see yfiles.view.BridgeManager#addObstacleProvider
     * @see yfiles.view.BridgeManager
     * @see yfiles.view.IObstacleProvider
     * @class
     * @implements {yfiles.view.IObstacleProvider}
     */
    export interface GraphObstacleProvider extends Object,yfiles.view.IObstacleProvider{}
    export class GraphObstacleProvider {
      /**
       * Helper method that retrieves the {@link yfiles.graph.IGraph} to use from the <code>context</code>
       * <p>
       * This implementation uses the {@link yfiles.graph.ILookup#lookup} of the {@link yfiles.view.ICanvasContext#canvasComponent} to query the {@link yfiles.graph.IGraph} implementation.
       * </p>
       * @param context The context to retrieve the implementation from.
       * @returns The {@link } instance to query or <code>null</code>.
       * @protected
       */
      getGraph(context:yfiles.view.IRenderContext):yfiles.graph.IGraph;
      /**
       * Iterates over all {@link yfiles.graph.IGraph#edges} and {@link yfiles.graph.IGraph#nodes} to query an {@link yfiles.view.IObstacleProvider} from the {@link yfiles.graph.IModelItem}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This method will depending on the {@link yfiles.view.GraphObstacleProvider#queryEdges} and {@link yfiles.view.GraphObstacleProvider#queryNodes} property query the items for an implementation of {@link yfiles.view.IObstacleProvider}
       * and concatenate all resulting {@link yfiles.geometry.GeneralPath} obstacles into one path that will then be returned.
       * </p>
       * @param context The context where the obstacles are queried for.
       * @returns A path that is the concatenated path of all obstacles for the given context.
       */
      getObstacles(context:yfiles.view.IRenderContext):yfiles.geometry.GeneralPath;
      /**
       * Gets or sets a value indicating whether to query the {@link yfiles.graph.IGraph#edges} for an {@link yfiles.view.IObstacleProvider} implementation.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      queryEdges:boolean;
      /**
       * Gets or sets a value indicating whether to query the {@link yfiles.graph.IGraph#nodes} for an {@link yfiles.view.IObstacleProvider} implementation.
       * @type {boolean}
       */
      queryNodes:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple implementation of the {@link yfiles.view.IGraphSelection} interface.
     * @class
     * @implements {yfiles.view.IGraphSelection}
     */
    export interface GraphSelection extends Object,yfiles.view.IGraphSelection{}
    export class GraphSelection {
      /**
       * Instantiates an instance using the the graph as the model.
       * @param graph
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Creates a new instance with no initial {@link yfiles.view.GraphSelection#graph} assigned.
       * <p>
       * In order to be always in sync with the structural changes in a graph, an instance should be associated with an {@link yfiles.graph.IGraph}
       * using the {@link yfiles.view.GraphSelection#graph} property.
       * </p>
       */
      constructor();
      /**
       * 
       */
      clear():void;
      /**
       * Factory method that creates the {@link yfiles.view.DefaultSelectionModel.<T>} to use for the bends.
       * @returns 
       * @protected
       */
      createBendSelectionModel():yfiles.view.DefaultSelectionModel<yfiles.graph.IBend>;
      /**
       * Factory method that creates the {@link yfiles.view.DefaultSelectionModel.<T>} to use for the edges.
       * @returns 
       * @protected
       */
      createEdgeSelectionModel():yfiles.view.DefaultSelectionModel<yfiles.graph.IEdge>;
      /**
       * Factory method that creates the {@link yfiles.view.DefaultSelectionModel.<T>} to use for the labels.
       * @returns 
       * @protected
       */
      createLabelSelectionModel():yfiles.view.DefaultSelectionModel<yfiles.graph.ILabel>;
      /**
       * Factory method that creates the {@link yfiles.view.DefaultSelectionModel.<T>} to use for the nodes.
       * @returns 
       * @protected
       */
      createNodeSelectionModel():yfiles.view.DefaultSelectionModel<yfiles.graph.INode>;
      /**
       * Factory method that creates the {@link yfiles.view.DefaultSelectionModel.<T>} to use for the ports.
       * @returns 
       * @protected
       */
      createPortSelectionModel():yfiles.view.DefaultSelectionModel<yfiles.graph.IPort>;
      /**
       * Returns an enumerator that iterates through the collection of all selected elements.
       * @returns An {@link } object that can be used to iterate through all selected elements.
       */
      getEnumerator():yfiles.collections.IEnumerator<yfiles.graph.IModelItem>;
      /**
       * 
       * @param item
       * @returns 
       */
      isSelected(item:yfiles.graph.IModelItem):boolean;
      /**
       * Raises the {@link yfiles.view.GraphSelection#addItemSelectionChangedListener ItemSelectionChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onItemSelectionChanged(evt:yfiles.view.ItemSelectionChangedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * 
       * @param item
       * @param selected
       */
      setSelected(item:yfiles.graph.IModelItem,selected:boolean):void;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.INode>}
       */
      selectedNodes:yfiles.view.ISelectionModel<yfiles.graph.INode>;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IEdge>}
       */
      selectedEdges:yfiles.view.ISelectionModel<yfiles.graph.IEdge>;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.ILabel>}
       */
      selectedLabels:yfiles.view.ISelectionModel<yfiles.graph.ILabel>;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IPort>}
       */
      selectedPorts:yfiles.view.ISelectionModel<yfiles.graph.IPort>;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IBend>}
       */
      selectedBends:yfiles.view.ISelectionModel<yfiles.graph.IBend>;
      /**
       * 
       * @type {number}
       */
      size:number;
      /**
       * Gets or sets the {@link yfiles.graph.IGraph} this instance is using as the domain for the selection.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * 
       * @param listener
       */
      addItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Manages the selection state of items in an {@link yfiles.graph.IGraph} instance.
     * <p>
     * This interface provides access to the {@link yfiles.view.ISelectionModel.<T>} instances that manage the selection of nodes, edges, labels, ports, and bends.
     * The generic methods that use {@link yfiles.graph.IModelItem} parameters delegate to the corresponding domain-specific selection
     * models.
     * </p>
     * @see yfiles.view.ISelectionModel.<T>
     * @interface
     * @implements {yfiles.view.ISelectionModel.<yfiles.graph.IModelItem>}
     */
    export interface IGraphSelection extends Object,yfiles.view.ISelectionModel<yfiles.graph.IModelItem>{
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected nodes.
       * <p>
       * This is the node part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.INode>}
       */
      selectedNodes:yfiles.view.ISelectionModel<yfiles.graph.INode>;
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected labels.
       * <p>
       * This is the label part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.ILabel>}
       */
      selectedLabels:yfiles.view.ISelectionModel<yfiles.graph.ILabel>;
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected edges.
       * <p>
       * This is the edge part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IEdge>}
       */
      selectedEdges:yfiles.view.ISelectionModel<yfiles.graph.IEdge>;
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected ports.
       * <p>
       * This is the ports part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IPort>}
       */
      selectedPorts:yfiles.view.ISelectionModel<yfiles.graph.IPort>;
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected bends.
       * <p>
       * This is the bend part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IBend>}
       */
      selectedBends:yfiles.view.ISelectionModel<yfiles.graph.IBend>;
    }
    var IGraphSelection:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An implementation of the {@link yfiles.view.ICanvasObjectInstaller} interface that can decorate the visual appearence of an
     * {@link yfiles.graph.IEdge}.
     * <p>
     * This will install an {@link yfiles.view.ICanvasObject} during {@link yfiles.view.ICanvasObjectInstaller#addCanvasObject installation}, that will render the path and the bends of an edge
     * using a {@link yfiles.view.EdgeDecorationInstaller#getStroke pen} for the path and a {@link yfiles.view.EdgeDecorationInstaller#getBendDrawing drawing} for the bends.
     * </p>
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     * @implements {yfiles.view.IHighlightIndicatorInstaller}
     * @implements {yfiles.view.IFocusIndicatorInstaller}
     */
    export interface EdgeDecorationInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller,yfiles.view.IHighlightIndicatorInstaller,yfiles.view.IFocusIndicatorInstaller{}
    export class EdgeDecorationInstaller {
      /**
       * Installs a rendering for the <code>item</code> if it is an {@link yfiles.graph.IEdge} that will use the {@link yfiles.view.EdgeDecorationInstaller#getBendDrawing}
       * and {@link yfiles.view.EdgeDecorationInstaller#getStroke} to render the path and bends.
       * @param context The canvas context.
       * @param group The {@link } to add the new canvas object to.
       * @param item The item to install.
       * @returns The canvas object representing the item.
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Callback method that retrieves the drawing of the bend for the context.
       * <p>
       * This implementation will use the {@link yfiles.view.EdgeSelectionIndicatorInstaller#BEND_TEMPLATE_KEY} resource to find a {@link yfiles.view.IVisualTemplate}
       * instance. If <code>null</code> is yielded, a default drawing will be used.
       * </p>
       * @param canvas The canvas for which the drawing shall be returned.
       * @param edge The edge for which the rendering will be used.
       * @returns The drawing instance to use for the rendering of the bends in the given context.
       * @protected
       * @abstract
       */
      getBendDrawing(canvas:yfiles.view.CanvasComponent,edge:yfiles.graph.IEdge):yfiles.view.IVisualTemplate;
      /**
       * Callback method that retrieves the {@link yfiles.view.Stroke} for the context.
       * <p>
       * This implementation will use the {@link yfiles.view.EdgeSelectionIndicatorInstaller#STROKE_KEY} resource to find a {@link yfiles.view.Stroke}
       * instance. If <code>null</code> is yielded, a default {@link yfiles.view.Stroke} will be used.
       * </p>
       * @param edge The edge for which the rendering will be used.
       * @param canvas The canvas for which the {@link } shall be returned.
       * @returns The {@link } instance to use for the rendering of the edge in the given context.
       * @protected
       * @abstract
       */
      getStroke(canvas:yfiles.view.CanvasComponent,edge:yfiles.graph.IEdge):yfiles.view.Stroke;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.view.ICanvasObjectInstaller} interface that can install the selection outline of an
     * {@link yfiles.graph.IEdge}.
     * <p>
     * This will {@link yfiles.view.EdgeDecorationInstaller#addCanvasObject install} an {@link yfiles.view.ICanvasObject} that will render the path and the bends of an edge using a {@link yfiles.view.EdgeSelectionIndicatorInstaller#getStroke pen} for the
     * path and a {@link yfiles.view.EdgeSelectionIndicatorInstaller#getBendDrawing drawing} for the bends.
     * </p>
     * @class
     * @extends {yfiles.view.EdgeDecorationInstaller}
     */
    export interface EdgeSelectionIndicatorInstaller extends yfiles.view.EdgeDecorationInstaller{}
    export class EdgeSelectionIndicatorInstaller {
      constructor();
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.Stroke} that will be used to draw the path of the edge.
       * @const
       * @static
       * @type {string}
       */
      static STROKE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} that will be used to draw the bends of the edge.
       * @const
       * @static
       * @type {string}
       */
      static BEND_TEMPLATE_KEY:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An specialized subclass of the {@link yfiles.view.EdgeSelectionIndicatorInstaller} that can install the focus indicator of an
     * {@link yfiles.graph.IEdge}.
     * <p>
     * This will {@link yfiles.view.EdgeDecorationInstaller#addCanvasObject install} an {@link yfiles.view.ICanvasObject} that will render the path and the bends of an edge using a {@link yfiles.view.EdgeFocusIndicatorInstaller#getStroke pen} for the
     * path and a {@link yfiles.view.EdgeFocusIndicatorInstaller#getBendDrawing drawing} for the bends.
     * </p>
     * @class
     * @extends {yfiles.view.EdgeDecorationInstaller}
     */
    export interface EdgeFocusIndicatorInstaller extends yfiles.view.EdgeDecorationInstaller{}
    export class EdgeFocusIndicatorInstaller {
      constructor();
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.Stroke} that will be used to draw the path of the edge.
       * @const
       * @static
       * @type {string}
       */
      static STROKE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} that will be used to draw the bends of the edge.
       * @const
       * @static
       * @type {string}
       */
      static BEND_TEMPLATE_KEY:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An specialized subclass of the {@link yfiles.view.EdgeSelectionIndicatorInstaller} that can install the highlight rendering of
     * an {@link yfiles.graph.IEdge}.
     * <p>
     * This will {@link yfiles.view.EdgeDecorationInstaller#addCanvasObject install} an {@link yfiles.view.ICanvasObject} that will render the path and the bends of an edge using a {@link yfiles.view.EdgeHighlightIndicatorInstaller#getStroke pen} for the
     * path and a {@link yfiles.view.EdgeHighlightIndicatorInstaller#getBendDrawing drawing} for the bends.
     * </p>
     * @class
     * @extends {yfiles.view.EdgeDecorationInstaller}
     */
    export interface EdgeHighlightIndicatorInstaller extends yfiles.view.EdgeDecorationInstaller{}
    export class EdgeHighlightIndicatorInstaller {
      constructor();
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.Stroke} that will be used to draw the path of the edge.
       * @const
       * @static
       * @type {string}
       */
      static STROKE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} that will be used to draw the bends of the edge.
       * @const
       * @static
       * @type {string}
       */
      static BEND_TEMPLATE_KEY:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * Allows the use of an {@link yfiles.styles.IEdgeStyle} to render the selection, highlight or focus indicator of edges.
     * <p>
     * The {@link yfiles.view.EdgeStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
     * zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
     * default yFiles indicators.
     * </p>
     * @see yfiles.view.NodeStyleDecorationInstaller
     * @see yfiles.view.LabelStyleDecorationInstaller
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     * @implements {yfiles.view.IHighlightIndicatorInstaller}
     * @implements {yfiles.view.IFocusIndicatorInstaller}
     */
    export interface EdgeStyleDecorationInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller,yfiles.view.IHighlightIndicatorInstaller,yfiles.view.IFocusIndicatorInstaller{}
    export class EdgeStyleDecorationInstaller {
      /**
       * Create a new instance of this class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.IEdgeStyle} options.edgeStyle The style to use for the rendering.
       * <p>
       * This option sets the {@link yfiles.view.EdgeStyleDecorationInstaller#edgeStyle} property on the created object.
       * </p>
       * @param {yfiles.view.StyleDecorationZoomPolicy} options.zoomPolicy Specifies how the style is affected by the current zoom level.
       * <p>
       * This option sets the {@link yfiles.view.EdgeStyleDecorationInstaller#zoomPolicy} property on the created object.
       * </p>
       */
      constructor(options?:{edgeStyle?:yfiles.styles.IEdgeStyle,zoomPolicy?:yfiles.view.StyleDecorationZoomPolicy});
      /**
       * Installs a rendering for the <code>item</code> if it is an {@link yfiles.graph.IEdge}.
       * @param context The canvas context.
       * @param group The canvas object group to add the newly created {@link } to.
       * @param item The item to install.
       * @returns The canvas object representing the item.
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Specifies how the style is affected by the current zoom level.
       * <p>
       * Changes of the value of this property are propagated to all styles created by this instance and become immediately
       * visible.
       * </p>
       * @type {yfiles.view.StyleDecorationZoomPolicy}
       */
      zoomPolicy:yfiles.view.StyleDecorationZoomPolicy;
      /**
       * The style to use for the rendering.
       * <p>
       * Changes of the value of this property are <b>not</b> propagated to already created styles.
       * </p>
       * @type {yfiles.styles.IEdgeStyle}
       */
      edgeStyle:yfiles.styles.IEdgeStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.view.IVisualCreator} for use in a {@link yfiles.view.CanvasComponent} that renders a {@link yfiles.graph.IGraph} instance in a
     * sloppy overview style.
     * @class
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface GraphOverviewCanvasVisualCreator extends Object,yfiles.view.IVisualCreator{}
    export class GraphOverviewCanvasVisualCreator {
      /**
       * Creates a new instance for the given graph.
       * @param graph
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Callback that paints the specified <code>edge</code>.
       * @param renderContext
       * @param ctx
       * @param edge
       * @protected
       */
      paintEdge(renderContext:yfiles.view.IRenderContext,ctx:CanvasRenderingContext2D,edge:yfiles.graph.IEdge):void;
      /**
       * Callback method that paints the specified <code>node</code>.
       * @param renderContext
       * @param ctx
       * @param node
       * @protected
       */
      paintGroupNode(renderContext:yfiles.view.IRenderContext,ctx:CanvasRenderingContext2D,node:yfiles.graph.INode):void;
      /**
       * Callback method that paints the specified <code>node</code>.
       * @param renderContext
       * @param ctx
       * @param node
       * @protected
       */
      paintNode(renderContext:yfiles.view.IRenderContext,ctx:CanvasRenderingContext2D,node:yfiles.graph.INode):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Allows the use of an {@link yfiles.styles.ILabelStyle} to render the selection, highlight or focus indicator of labels.
     * <p>
     * The {@link yfiles.view.LabelStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
     * zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
     * default yFiles indicators.
     * </p>
     * @see yfiles.view.EdgeStyleDecorationInstaller
     * @see yfiles.view.NodeStyleDecorationInstaller
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     * @implements {yfiles.view.IHighlightIndicatorInstaller}
     * @implements {yfiles.view.IFocusIndicatorInstaller}
     */
    export interface LabelStyleDecorationInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller,yfiles.view.IHighlightIndicatorInstaller,yfiles.view.IFocusIndicatorInstaller{}
    export class LabelStyleDecorationInstaller {
      /**
       * Create a new instance of this class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.ILabelStyle} options.labelStyle The style to use for the rendering.
       * <p>
       * This option sets the {@link yfiles.view.LabelStyleDecorationInstaller#labelStyle} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.margins Specifies the margin around the label layout.
       * <p>
       * This option sets the {@link yfiles.view.LabelStyleDecorationInstaller#margins} property on the created object.
       * </p>
       * @param {yfiles.view.StyleDecorationZoomPolicy} options.zoomPolicy Specifies how the style is affected by the current zoom level.
       * <p>
       * This option sets the {@link yfiles.view.LabelStyleDecorationInstaller#zoomPolicy} property on the created object.
       * </p>
       */
      constructor(options?:{labelStyle?:yfiles.styles.ILabelStyle,margins?:yfiles.geometry.Insets,zoomPolicy?:yfiles.view.StyleDecorationZoomPolicy});
      /**
       * Installs a rendering for the <code>item</code> if it is an {@link yfiles.graph.ILabel}.
       * @param context The canvas context.
       * @param group The canvas object group to add the newly created canvas object to.
       * @param item The item to install.
       * @returns The canvas object representing the item.
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Specifies how the style is affected by the current zoom level.
       * <p>
       * Changes of the value of this property are propagated to all styles created by this instance and become immediately
       * visible.
       * </p>
       * @type {yfiles.view.StyleDecorationZoomPolicy}
       */
      zoomPolicy:yfiles.view.StyleDecorationZoomPolicy;
      /**
       * Specifies the margin around the label layout.
       * <p>
       * Actually, the bounds rendered by the style is the original label layout enlarged by this margin.
       * </p>
       * <p>
       * Changes of the value of this property are propagated to all styles created by this instance and become immediately
       * visible.
       * </p>
       * @type {yfiles.geometry.Insets}
       */
      margins:yfiles.geometry.Insets;
      /**
       * The style to use for the rendering.
       * <p>
       * Changes of the value of this property are <b>not</b> propagated to already created styles.
       * </p>
       * @type {yfiles.styles.ILabelStyle}
       */
      labelStyle:yfiles.styles.ILabelStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * Allows the use of an {@link yfiles.styles.INodeStyle} to render the selection, highlight or focus indicator of nodes.
     * <p>
     * The {@link yfiles.view.NodeStyleDecorationInstaller#zoomPolicy} property defines how the zoom level affects the rendering of the indicator. It can either scale according to the
     * zoom level similar to regular graph items or have always to same thickness regardless of the zoom, similar to the
     * default yFiles indicators.
     * </p>
     * @see yfiles.view.EdgeStyleDecorationInstaller
     * @see yfiles.view.LabelStyleDecorationInstaller
     * @class
     * @implements {yfiles.view.ISelectionIndicatorInstaller}
     * @implements {yfiles.view.IHighlightIndicatorInstaller}
     * @implements {yfiles.view.IFocusIndicatorInstaller}
     */
    export interface NodeStyleDecorationInstaller extends Object,yfiles.view.ISelectionIndicatorInstaller,yfiles.view.IHighlightIndicatorInstaller,yfiles.view.IFocusIndicatorInstaller{}
    export class NodeStyleDecorationInstaller {
      /**
       * Create a new instance of this class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.INodeStyle} options.nodeStyle The style to use for the rendering.
       * <p>
       * This option sets the {@link yfiles.view.NodeStyleDecorationInstaller#nodeStyle} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.margins Specifies the margin around the node layout.
       * <p>
       * This option sets the {@link yfiles.view.NodeStyleDecorationInstaller#margins} property on the created object.
       * </p>
       * @param {yfiles.view.StyleDecorationZoomPolicy} options.zoomPolicy Specifies how the style is affected by the current zoom level.
       * <p>
       * This option sets the {@link yfiles.view.NodeStyleDecorationInstaller#zoomPolicy} property on the created object.
       * </p>
       */
      constructor(options?:{nodeStyle?:yfiles.styles.INodeStyle,margins?:yfiles.geometry.Insets,zoomPolicy?:yfiles.view.StyleDecorationZoomPolicy});
      /**
       * Installs a rendering for the <code>item</code> if it is an {@link yfiles.graph.INode}.
       * @param context The canvas context.
       * @param group The canvas object group to add the newly created canvas object to.
       * @param item The item to install.
       * @returns The canvas object representing the item.
       */
      addCanvasObject(context:yfiles.view.ICanvasContext,group:yfiles.view.ICanvasObjectGroup,item:Object):yfiles.view.ICanvasObject;
      /**
       * Specifies how the style is affected by the current zoom level.
       * <p>
       * Changes of the value of this property are propagated to all styles created by this instance and become immediately
       * visible.
       * </p>
       * @type {yfiles.view.StyleDecorationZoomPolicy}
       */
      zoomPolicy:yfiles.view.StyleDecorationZoomPolicy;
      /**
       * Specifies the margin around the node layout.
       * <p>
       * Actually, the bounds rendered by the style is the original node layout enlarged by this margin.
       * </p>
       * <p>
       * Changes of the value of this property are propagated to all styles created by this instance and become immediately
       * visible.
       * </p>
       * @type {yfiles.geometry.Insets}
       */
      margins:yfiles.geometry.Insets;
      /**
       * The style to use for the rendering.
       * <p>
       * Changes of the value of this property are <b>not</b> propagated to already created styles.
       * </p>
       * @type {yfiles.styles.INodeStyle}
       */
      nodeStyle:yfiles.styles.INodeStyle;
      static $class:yfiles.lang.Class;
    }
    export enum StyleDecorationZoomPolicy{
      /**
       * The visual is rendered in the view coordinate space and doesn't scale with the zoom level.
       * <p>
       * This is similar to the default node resize handles, for example.
       * </p>
       */
      VIEW_COORDINATES,
      /**
       * The visual is rendered in the world coordinate space and scales with the zoom level like a regular graph item
       * visualization, for example a node style.
       */
      WORLD_COORDINATES,
      /**
       * Uses WorldCoordinates rendering for zoom level >= 1 and ViewCoordinates rendering for zoom level &lt; 1.
       */
      MIXED
    }
    /**
     * Represents any object which fills an area.
     * @class
     * @implements {yfiles.lang.ICloneable}
     */
    export interface Fill extends Object,yfiles.lang.ICloneable{}
    export class Fill {
      /**
       * Assigns the fill defined by this instance to the given <code>element</code>.
       * <p>
       * If this {@link yfiles.view.Fill} is a gradient and implements the {@link yfiles.view.ISvgDefsCreator} interface, it will be added to the
       * <code>defs</code> section of the SVG document. The fill of the given element will be set to a reference to the gradient element.
       * </p>
       * @param element The element whose fill should be set.
       * @param context The context.
       */
      applyTo(element:SVGElement,context:yfiles.view.ICanvasContext):void;
      /**
       * Clones this instance by returning a {@link Object#memberwiseClone} or this in case this instance is already {@link yfiles.view.Fill#isFrozen frozen}.
       * @returns An instance of the same type as this instance.
       */
      clone():Object;
      /**
       * Freezes this instance.
       * <p>
       * Freezing an object makes it immutable, which can result in performance enhancements.
       * </p>
       */
      freeze():void;
      /**
       * Determines if this instance is frozen.
       * @returns 
       */
      isFrozen():boolean;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color AliceBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ALICE_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color AntiqueWhite.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ANTIQUE_WHITE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Aqua.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static AQUA:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Aquamarine.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static AQUAMARINE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Azure.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static AZURE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Beige.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BEIGE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Bisque.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BISQUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Black.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BLACK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color BlanchedAlmond.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BLANCHED_ALMOND:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Blue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color BlueViolet.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BLUE_VIOLET:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Brown.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BROWN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color BurlyWood.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static BURLY_WOOD:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color CadetBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CADET_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Chartreuse.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CHARTREUSE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Chocolate.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CHOCOLATE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Coral.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CORAL:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color CornflowerBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CORNFLOWER_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Cornsilk.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CORNSILK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Crimson.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CRIMSON:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Cyan.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static CYAN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkCyan.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_CYAN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkGoldenrod.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_GOLDENROD:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkGray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkKhaki.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_KHAKI:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkMagenta.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_MAGENTA:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkOliveGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_OLIVE_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkOrange.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_ORANGE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkOrchid.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_ORCHID:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkRed.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_RED:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkSalmon.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_SALMON:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkSeaGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_SEA_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkSlateBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_SLATE_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkSlateGray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_SLATE_GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkTurquoise.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_TURQUOISE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DarkViolet.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DARK_VIOLET:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DeepPink.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DEEP_PINK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DeepSkyBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DEEP_SKY_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DimGray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DIM_GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color DodgerBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static DODGER_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Firebrick.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static FIREBRICK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color FloralWhite.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static FLORAL_WHITE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color ForestGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static FOREST_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Fuchsia.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static FUCHSIA:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Gainsboro.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GAINSBORO:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color GhostWhite.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GHOST_WHITE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Gold.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GOLD:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Goldenrod.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GOLDENROD:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Gray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Green.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color GreenYellow.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static GREEN_YELLOW:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Honeydew.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static HONEYDEW:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color HotPink.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static HOT_PINK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color IndianRed.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static INDIAN_RED:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Indigo.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static INDIGO:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Ivory.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static IVORY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Khaki.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static KHAKI:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Lavender.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LAVENDER:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LavenderBlush.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LAVENDER_BLUSH:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LawnGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LAWN_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LemonChiffon.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LEMON_CHIFFON:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightCoral.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_CORAL:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightCyan.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_CYAN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightGoldenrodYellow.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_GOLDENROD_YELLOW:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightGray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightPink.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_PINK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightSalmon.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_SALMON:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightSeaGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_SEA_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightSkyBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_SKY_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightSlateGray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_SLATE_GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightSteelBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_STEEL_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LightYellow.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIGHT_YELLOW:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Lime.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIME:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color LimeGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LIME_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Linen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static LINEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Magenta.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MAGENTA:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Maroon.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MAROON:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumAquamarine.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_AQUAMARINE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumOrchid.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_ORCHID:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumPurple.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_PURPLE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumSeaGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_SEA_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumSlateBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_SLATE_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumSpringGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_SPRING_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumTurquoise.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_TURQUOISE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MediumVioletRed.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MEDIUM_VIOLET_RED:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MidnightBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MIDNIGHT_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MintCream.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MINT_CREAM:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color MistyRose.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MISTY_ROSE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Moccasin.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static MOCCASIN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color NavajoWhite.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static NAVAJO_WHITE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Navy.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static NAVY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color OldLace.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static OLD_LACE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Olive.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static OLIVE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color OliveDrab.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static OLIVE_DRAB:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Orange.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ORANGE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color OrangeRed.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ORANGE_RED:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Orchid.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ORCHID:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PaleGoldenrod.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PALE_GOLDENROD:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PaleGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PALE_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PaleTurquoise.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PALE_TURQUOISE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PaleVioletRed.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PALE_VIOLET_RED:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PapayaWhip.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PAPAYA_WHIP:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PeachPuff.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PEACH_PUFF:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Peru.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PERU:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Pink.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PINK:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Plum.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PLUM:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color PowderBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static POWDER_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Purple.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static PURPLE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Red.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static RED:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color RosyBrown.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ROSY_BROWN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color RoyalBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static ROYAL_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SaddleBrown.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SADDLE_BROWN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Salmon.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SALMON:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SandyBrown.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SANDY_BROWN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SeaGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SEA_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SeaShell.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SEA_SHELL:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Sienna.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SIENNA:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Silver.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SILVER:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SkyBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SKY_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SlateBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SLATE_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SlateGray.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SLATE_GRAY:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Snow.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SNOW:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SpringGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static SPRING_GREEN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color SteelBlue.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static STEEL_BLUE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Tan.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static TAN:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Teal.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static TEAL:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Thistle.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static THISTLE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Tomato.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static TOMATO:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Transparent.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static TRANSPARENT:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Turquoise.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static TURQUOISE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Violet.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static VIOLET:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Wheat.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static WHEAT:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color White.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static WHITE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color WhiteSmoke.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static WHITE_SMOKE:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color Yellow.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static YELLOW:yfiles.view.Fill;
      /**
       * Gets a {@link yfiles.view.SolidColorFill} for the color YellowGreen.
       * <p>
       * This property will yield a {@link yfiles.view.Fill#isFrozen frozen} shared immutable value.
       * </p>
       */
      static YELLOW_GREEN:yfiles.view.Fill;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents an ARGB (alpha, red, green, blue) encoded color object.
     * @class
     * @final
     */
    export interface Color extends Object{}
    export class Color {
      /**
       * Initializes a new instance of the {@link yfiles.view.Color} class.
       * @param [a=255] The alpha value; <code>255</code> by default.
       * @param [r=0] The red value; <code>0</code> by default.
       * @param [g=0] The green value; <code>0</code> by default
       * @param [b=0] The blue value; <code>0</code> by default.
       */
      constructor(r?:number,g?:number,b?:number,a?:number);
      /**
       * Initializes a new instance of the {@link yfiles.view.Color} class.
       * @param {Object} options The parameters to pass.
       * @param [options.a=255] The alpha value; <code>255</code> by default.
       * @param [options.r=0] The red value; <code>0</code> by default.
       * @param [options.g=0] The green value; <code>0</code> by default
       * @param [options.b=0] The blue value; <code>0</code> by default.
       */
      constructor(options:{r?:number,g?:number,b?:number,a?:number});
      /**
       * Creates a new color instance from ARGB values.
       * <p>
       * All values must be between 0 and 255.
       * </p>
       * @param a The alpha value.
       * @param r The red value.
       * @param g The green value.
       * @param b The blue value.
       * @returns A new color instance with the given values.
       * @static
       */
      static fromArgb(a:number,r:number,g:number,b:number):yfiles.view.Color;
      /**
       * Gets the value of the red component of the color.
       * <p>
       * The value is between 0 and 255. A higher value represents a higher part of the color is made up of this component.
       * </p>
       * @type {number}
       */
      r:number;
      /**
       * Gets the value of the green component of the color.
       * <p>
       * The value is between 0 and 255. A higher value represents a higher part of the color is made up of this component.
       * </p>
       * @type {number}
       */
      g:number;
      /**
       * Gets the value of the blue component of the color.
       * <p>
       * The value is between 0 and 255. A higher value represents a higher part of the color is made up of this component.
       * </p>
       * @type {number}
       */
      b:number;
      /**
       * Gets the value of the alpha component of the color.
       * <p>
       * The value is between 0 and 255. If this component is 0, then the color will be transparent.
       * </p>
       * @type {number}
       */
      a:number;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name AliceBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ALICE_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name AntiqueWhite.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ANTIQUE_WHITE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Aqua.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static AQUA:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Aquamarine.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static AQUAMARINE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Azure.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static AZURE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Beige.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BEIGE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Bisque.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BISQUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Black.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BLACK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name BlanchedAlmond.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BLANCHED_ALMOND:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Blue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name BlueViolet.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BLUE_VIOLET:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Brown.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BROWN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name BurlyWood.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static BURLY_WOOD:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name CadetBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CADET_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Chartreuse.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CHARTREUSE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Chocolate.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CHOCOLATE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Coral.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CORAL:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name CornflowerBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CORNFLOWER_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Cornsilk.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CORNSILK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Crimson.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CRIMSON:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Cyan.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static CYAN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkCyan.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_CYAN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkGoldenrod.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_GOLDENROD:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkGray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkKhaki.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_KHAKI:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkMagenta.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_MAGENTA:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkOliveGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_OLIVE_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkOrange.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_ORANGE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkOrchid.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_ORCHID:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkRed.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_RED:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkSalmon.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_SALMON:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkSeaGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_SEA_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkSlateBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_SLATE_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkSlateGray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_SLATE_GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkTurquoise.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_TURQUOISE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DarkViolet.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DARK_VIOLET:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DeepPink.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DEEP_PINK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DeepSkyBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DEEP_SKY_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DimGray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DIM_GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name DodgerBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static DODGER_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Firebrick.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static FIREBRICK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name FloralWhite.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static FLORAL_WHITE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name ForestGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static FOREST_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Fuchsia.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static FUCHSIA:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Gainsboro.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GAINSBORO:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name GhostWhite.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GHOST_WHITE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Gold.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GOLD:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Goldenrod.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GOLDENROD:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Gray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Green.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name GreenYellow.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static GREEN_YELLOW:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Honeydew.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static HONEYDEW:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name HotPink.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static HOT_PINK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name IndianRed.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static INDIAN_RED:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Indigo.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static INDIGO:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Ivory.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static IVORY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Khaki.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static KHAKI:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Lavender.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LAVENDER:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LavenderBlush.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LAVENDER_BLUSH:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LawnGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LAWN_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LemonChiffon.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LEMON_CHIFFON:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightCoral.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_CORAL:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightCyan.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_CYAN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightGoldenrodYellow.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_GOLDENROD_YELLOW:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightGray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightPink.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_PINK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightSalmon.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_SALMON:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightSeaGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_SEA_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightSkyBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_SKY_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightSlateGray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_SLATE_GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightSteelBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_STEEL_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LightYellow.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIGHT_YELLOW:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Lime.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIME:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name LimeGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LIME_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Linen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static LINEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Magenta.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MAGENTA:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Maroon.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MAROON:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumAquamarine.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_AQUAMARINE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumOrchid.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_ORCHID:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumPurple.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_PURPLE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumSeaGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_SEA_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumSlateBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_SLATE_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumSpringGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_SPRING_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumTurquoise.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_TURQUOISE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MediumVioletRed.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MEDIUM_VIOLET_RED:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MidnightBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MIDNIGHT_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MintCream.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MINT_CREAM:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name MistyRose.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MISTY_ROSE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Moccasin.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static MOCCASIN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name NavajoWhite.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static NAVAJO_WHITE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Navy.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static NAVY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name OldLace.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static OLD_LACE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Olive.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static OLIVE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name OliveDrab.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static OLIVE_DRAB:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Orange.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ORANGE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name OrangeRed.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ORANGE_RED:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Orchid.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ORCHID:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PaleGoldenrod.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PALE_GOLDENROD:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PaleGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PALE_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PaleTurquoise.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PALE_TURQUOISE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PaleVioletRed.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PALE_VIOLET_RED:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PapayaWhip.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PAPAYA_WHIP:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PeachPuff.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PEACH_PUFF:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Peru.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PERU:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Pink.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PINK:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Plum.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PLUM:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name PowderBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static POWDER_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Purple.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static PURPLE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Red.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static RED:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name RosyBrown.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ROSY_BROWN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name RoyalBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static ROYAL_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SaddleBrown.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SADDLE_BROWN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Salmon.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SALMON:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SandyBrown.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SANDY_BROWN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SeaGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SEA_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SeaShell.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SEA_SHELL:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Sienna.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SIENNA:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Silver.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SILVER:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SkyBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SKY_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SlateBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SLATE_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SlateGray.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SLATE_GRAY:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Snow.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SNOW:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SpringGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static SPRING_GREEN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name SteelBlue.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static STEEL_BLUE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Tan.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static TAN:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Teal.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static TEAL:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Thistle.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static THISTLE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Tomato.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static TOMATO:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Transparent.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static TRANSPARENT:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Turquoise.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static TURQUOISE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Violet.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static VIOLET:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Wheat.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static WHEAT:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name White.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static WHITE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name WhiteSmoke.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static WHITE_SMOKE:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name Yellow.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static YELLOW:yfiles.view.Color;
      /**
       * Yields a new {@link yfiles.view.Color} for the wellknown name YellowGreen.
       * <p>
       * This method will yield the same value for every call because it is immutable.
       * </p>
       */
      static YELLOW_GREEN:yfiles.view.Color;
      static $class:yfiles.lang.Class;
    }
    /**
     * 
     * @class
     * @extends {yfiles.graphml.MarkupExtension}
     */
    export interface ColorExtension extends yfiles.graphml.MarkupExtension{}
    export class ColorExtension {
      /**
       * 
       * @param color
       */
      constructor(color:yfiles.view.Color);
      /**
       * 
       */
      constructor();
      /**
       * 
       * @type {string}
       */
      value:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a cursor object.
     * @class
     */
    export interface Cursor extends Object{}
    export class Cursor {
      /**
       * Initializes a new {@link yfiles.view.Cursor} instance using the provided url and an alternative.
       * @param url The url should point to a file in the ".cur" format with a size of at most 32x32 pixels to ensure compatibility across
       * browsers.
       * @param alternative Cursors can be chained but the last alternative needs to be one of the predefined cursors.
       */
      constructor(url:string,alternative:yfiles.view.Cursor);
      /**
       * Returns the CSS text representation for the cursor and its alternatives.
       * @returns The CSS text representation for the cursor.
       */
      getSystemCursor():string;
      /**
       * Gets the invisible cursor.
       * <p>
       * This corresponds to the <code>none</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: none;" />
       * </p>
       */
      static NONE:yfiles.view.Cursor;
      /**
       * Gets the default cursor.
       * <p>
       * This corresponds to the <code>default</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: default;" />
       * </p>
       */
      static DEFAULT:yfiles.view.Cursor;
      /**
       * Gets the cursor with a pointing hand, as used for example on a hyperlink.
       * <p>
       * This corresponds to the <code>pointer</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: pointer;" />
       * </p>
       */
      static POINTER:yfiles.view.Cursor;
      /**
       * Gets the cursor that indicates that help is available for the object under the cursor.
       * <p>
       * This corresponds to the <code>help</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: help;" />
       * </p>
       */
      static HELP:yfiles.view.Cursor;
      /**
       * Gets the cursor that indicates progress.
       * <p>
       * This corresponds to the <code>progress</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: progress;" />
       * </p>
       */
      static PROGRESS:yfiles.view.Cursor;
      /**
       * Gets the text selection cursor.
       * <p>
       * This corresponds to the <code>text</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: text;" />
       * </p>
       */
      static TEXT:yfiles.view.Cursor;
      /**
       * Gets the wait cursor.
       * <p>
       * This corresponds to the <code>wait</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: wait;" />
       * </p>
       */
      static WAIT:yfiles.view.Cursor;
      /**
       * Gets the crosshair cursor.
       * <p>
       * This corresponds to the <code>crosshair</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: crosshair;" />
       * </p>
       */
      static CROSSHAIR:yfiles.view.Cursor;
      /**
       * Gets the move cursor.
       * <p>
       * This corresponds to the <code>move</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: move;" />
       * </p>
       */
      static MOVE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that an edge at the east will be moved.
       * <p>
       * Depending on the browser this might just be a horizontal resize cursor.
       * </p>
       * <p>
       * This corresponds to the <code>e-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: e-resize;" />
       * </p>
       */
      static E_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that an edge at the north will be moved.
       * <p>
       * Depending on the browser this might just be a vertical resize cursor.
       * </p>
       * <p>
       * This corresponds to the <code>n-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: n-resize;" />
       * </p>
       */
      static N_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that edges at the north and east will be moved.
       * <p>
       * Depending on the browser this might just be the diagonal resize cursor for NE to SW resizes.
       * </p>
       * <p>
       * This corresponds to the <code>ne-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: ne-resize;" />
       * </p>
       */
      static NE_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that edges at the north and west will be moved.
       * <p>
       * Depending on the browser this might just be the diagonal resize cursor for NW to SE resizes.
       * </p>
       * <p>
       * This corresponds to the <code>nw-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: nw-resize;" />
       * </p>
       */
      static NW_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that the edge at the south will be moved.
       * <p>
       * Depending on the browser this might just be a vertical resize cursor.
       * </p>
       * <p>
       * This corresponds to the <code>s-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: s-resize;" />
       * </p>
       */
      static S_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that edges at the south and east will be moved.
       * <p>
       * Depending on the browser this might just be the diagonal resize cursor for NW to SE resizes.
       * </p>
       * <p>
       * This corresponds to the <code>se-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: se-resize;" />
       * </p>
       */
      static SE_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that edges at the south and west will be moved.
       * <p>
       * Depending on the browser this might just be the diagonal resize cursor for NE to SW resizes.
       * </p>
       * <p>
       * This corresponds to the <code>sw-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: sw-resize;" />
       * </p>
       */
      static SW_RESIZE:yfiles.view.Cursor;
      /**
       * Gets a cursor that indicates that the edge at the west will be moved.
       * <p>
       * Depending on the browser this might just be a horizontal resize cursor.
       * </p>
       * <p>
       * This corresponds to the <code>w-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: w-resize;" />
       * </p>
       */
      static W_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the vertical resize cursor.
       * <p>
       * This corresponds to the <code>ns-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: ns-resize;" />
       * </p>
       */
      static NS_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the diagonal resize cursor for NW to SE resizes.
       * <p>
       * This corresponds to the <code>nwse-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: nwse-resize;" />
       * </p>
       */
      static NWSE_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the diagonal resize cursor for NE to SW resizes.
       * <p>
       * This corresponds to the <code>nesw-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: nesw-resize;" />
       * </p>
       */
      static NESW_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the horizontal resize cursor.
       * <p>
       * This corresponds to the <code>ew-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: ew-resize;" />
       * </p>
       */
      static EW_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the cursor for resizing columns.
       * <p>
       * This corresponds to the <code>col-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: col-resize;" />
       * </p>
       */
      static COL_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the cursor for resizing rows.
       * <p>
       * This corresponds to the <code>row-resize</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: row-resize;" />
       * </p>
       */
      static ROW_RESIZE:yfiles.view.Cursor;
      /**
       * Gets the cursor where the browser determines the actual cursor to be displayed.
       * <p>
       * This corresponds to the <code>auto</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: auto;" />
       * </p>
       */
      static AUTO:yfiles.view.Cursor;
      /**
       * Gets the cursor to indicate that there is a context menu available.
       * <p>
       * This corresponds to the <code>context-menu</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: context-menu;" />
       * </p>
       */
      static CONTEXT_MENU:yfiles.view.Cursor;
      /**
       * Gets the cursor to indicate selection of a table cell..
       * <p>
       * This corresponds to the <code>cell</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: cell;" />
       * </p>
       */
      static CELL:yfiles.view.Cursor;
      /**
       * Gets the vertical text selection cursor.
       * <p>
       * This corresponds to the <code>vertical-text</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: vertical-text;" />
       * </p>
       */
      static VERTICAL_TEXT:yfiles.view.Cursor;
      /**
       * Gets the cursor to be shown when an alias or shortcut can be created.
       * <p>
       * This corresponds to the <code>alias</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: alias;" />
       * </p>
       */
      static ALIAS:yfiles.view.Cursor;
      /**
       * Gets the cursor to be shown when something can be copied (often used for drag &amp; drop).
       * <p>
       * This corresponds to the <code>copy</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: copy;" />
       * </p>
       */
      static COPY:yfiles.view.Cursor;
      /**
       * Gets the cursor to be shown where nothing can be dropped in a drag &amp; drop operation.
       * <p>
       * This corresponds to the <code>no-drop</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: no-drop;" />
       * </p>
       */
      static NO_DROP:yfiles.view.Cursor;
      /**
       * Gets the cursor to be shown when an action cannot be done.
       * <p>
       * This corresponds to the <code>not-allowed</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: not-allowed;" />
       * </p>
       */
      static NOT_ALLOWED:yfiles.view.Cursor;
      /**
       * Gets the cursor indicating that something can be scrolled or panned.
       * <p>
       * This corresponds to the <code>all-scroll</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: all-scroll;" />
       * </p>
       */
      static ALL_SCROLL:yfiles.view.Cursor;
      /**
       * Gets the cursor for zooming in.
       * <p>
       * This corresponds to the <code>zoom-in</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: zoom-in;" />
       * </p>
       */
      static ZOOM_IN:yfiles.view.Cursor;
      /**
       * Gets the cursor for zooming out.
       * <p>
       * This corresponds to the <code>zoom-out</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: zoom-out;" />
       * </p>
       */
      static ZOOM_OUT:yfiles.view.Cursor;
      /**
       * Gets the cursor indicating that something can be grabbed (usually in conjunction with {@link yfiles.view.Cursor#GRABBING}).
       * <p>
       * This corresponds to the <code>grab</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: grab;" />
       * </p>
       */
      static GRAB:yfiles.view.Cursor;
      /**
       * Gets the cursor indicating that something is currently being grabbed (usually in conjunction with {@link yfiles.view.Cursor#GRAB}).
       * <p>
       * This corresponds to the <code>grabbing</code> CSS cursor. This is how it appears in your browser (hover to see): <span style="display: block; border: black solid 1px; background: white; width: 50px; height: 50px; cursor: grabbing;" />
       * </p>
       */
      static GRABBING:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * A container for {@link yfiles.view.DashStyle#dashes} and an {@link yfiles.view.DashStyle#offset}.
     * <p>
     * All of these values are in terms of multiples of the the thickness of a
     * {@link yfiles.view.Stroke}. This is different from the corresponding properties of a SVG stroke.
     * </p>
     * @class
     */
    export interface DashStyle extends Object{}
    export class DashStyle {
      /**
       * Initializes a new instance of the {@link yfiles.view.DashStyle} class using the provided dashes and offset.
       * @param dashes The dashes.
       * @param offset The offset.
       */
      constructor(dashes:number[],offset:number);
      /**
       * Initializes a new instance of the {@link yfiles.view.DashStyle} class using an empty {@link yfiles.view.DashStyle#dashes} and zero {@link yfiles.view.DashStyle#offset}.
       */
      constructor();
      /**
       * Clones this instance by returning a {@link Object#memberwiseClone} or this in case this instance is already {@link yfiles.view.DashStyle#isFrozen frozen}.
       * @returns An instance of the same type as this instance.
       */
      clone():Object;
      /**
       * Freezes this instance.
       * <p>
       * Freezing an object makes it immutable, which can result in performance enhancements.
       * </p>
       */
      freeze():void;
      /**
       * Determines if this instance is frozen.
       * @returns 
       */
      isFrozen():boolean;
      /**
       * Gets or sets the dashes.
       * @type {yfiles.collections.List.<number>}
       */
      dashes:yfiles.collections.List<number>;
      /**
       * Gets or sets the offset.
       * @type {number}
       */
      offset:number;
      /**
       * Gets the dash style with a single dash.
       * <p>
       * This style is designed for dash caps that extend into the 'gaps' like {@link yfiles.view.LineCap#SQUARE} and
       * {@link yfiles.view.LineCap#ROUND}. A pen with this style doesn't look as expected if its dash cap is {@link yfiles.view.LineCap#FLAT}.
       * Since {@link yfiles.view.LineCap#FLAT} is the default value, you should explicitly set another value.
       * </p>
       */
      static DASH:yfiles.view.DashStyle;
      /**
       * Gets the dash style with a single dash and dot.
       * <p>
       * This style is designed for dash caps that extend into the 'gaps' like {@link yfiles.view.LineCap#SQUARE} and
       * {@link yfiles.view.LineCap#ROUND}. A pen with this style doesn't contain visible dots if its dash cap is
       * {@link yfiles.view.LineCap#FLAT}. Since {@link yfiles.view.LineCap#FLAT} is the default value, you should explicitly set another value.
       * </p>
       */
      static DASH_DOT:yfiles.view.DashStyle;
      /**
       * Gets the dash style with a single dash and two dots.
       * <p>
       * This style is designed for dash caps that extend into the 'gaps' like {@link yfiles.view.LineCap#SQUARE} and
       * {@link yfiles.view.LineCap#ROUND}. A pen with this style doesn't contain visible dots if its dash cap is
       * {@link yfiles.view.LineCap#FLAT}. Since {@link yfiles.view.LineCap#FLAT} is the default value, you should explicitly set another value.
       * </p>
       */
      static DASH_DOT_DOT:yfiles.view.DashStyle;
      /**
       * Gets the dash style with a single dot.
       * <p>
       * This style is designed for dash caps that extend into the 'gaps' like {@link yfiles.view.LineCap#SQUARE} and
       * {@link yfiles.view.LineCap#ROUND}. A pen with this style doesn't contain visible dots if its dash cap is
       * {@link yfiles.view.LineCap#FLAT}. Since {@link yfiles.view.LineCap#FLAT} is the default value, you should explicitly set another value.
       * </p>
       */
      static DOT:yfiles.view.DashStyle;
      /**
       * Gets the solid dash style with no dashes.
       */
      static SOLID:yfiles.view.DashStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * Template factory for creating visual representations of rectangular shaped objects.
     * <p>
     * Implementations of this class have the purpose of creating a {@link yfiles.view.SvgVisual} that serves as the visual
     * representation of the object associated with this instance. This special-purpose interface is only used in some places
     * where the user interface can be customized. The {@link yfiles.view.IVisualCreator} interface is the main interface used by the
     * {@link yfiles.view.CanvasComponent}.
     * </p>
     * @interface
     */
    export interface IVisualTemplate extends Object{
      /**
       * This method is called by the framework to create a {@link yfiles.view.SvgVisual} with the given <code>bounds</code>.
       * <p>
       * {@link yfiles.view.CanvasComponent} uses this interface through the {@link yfiles.view.ICanvasObjectDescriptor} to populate the visual
       * canvas object tree.
       * </p>
       * @param context The context that describes where the visual will be used.
       * @param bounds The initial bounds to use for the visual.
       * @param dataObject The data object to visualize.
       * @returns The visual to include in the canvas object visual tree. This may be <code>null</code>.
       * @see yfiles.view.IVisualTemplate#updateVisual
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext,bounds:yfiles.geometry.Rect,dataObject:Object):yfiles.view.SvgVisual;
      /**
       * This method updates or replaces a previously created {@link yfiles.view.SvgVisual} for inclusion in the
       * {@link yfiles.view.IRenderContext}.
       * <p>
       * The {@link yfiles.view.CanvasComponent} uses this method to give implementations a chance to update an existing Visual that has
       * previously been created by the same instance during a call to {@link yfiles.view.IVisualTemplate#createVisual}. Implementation
       * may update the <code>oldVisual</code> and return that same reference, or create a new visual and return the new instance or <code>null</code>.
       * </p>
       * @param context The context that describes where the visual will be used in.
       * @param oldVisual The visual instance that had been returned the last time the {@link #createVisual} method was called
       * on this instance.
       * @param bounds The initial bounds to use for the visual.
       * @param dataObject The data object to visualize.
       * @returns <code>oldVisual</code>, if this instance modified the visual, or a new visual that should replace the existing one in the canvas
       * object visual tree.
       * @abstract
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.SvgVisual,bounds:yfiles.geometry.Rect,dataObject:Object):yfiles.view.SvgVisual;
    }
    var IVisualTemplate:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The default implementation of a data object for the {@link yfiles.input.DropInputMode}.
     * @class
     * @final
     */
    export interface DragDropItem extends Object{}
    export class DragDropItem {
      /**
       * Initializes a new instance of the {@link yfiles.view.DragDropItem} class and sets the provided data item for the specified
       * format.
       * @param [format=null] The format.
       * @param [data=null] The data item.
       * @see yfiles.view.DragDropItem#setData
       */
      constructor(format?:string,data?:Object);
      /**
       * Initializes a new instance of the {@link yfiles.view.DragDropItem} class and sets the provided data item for the specified
       * format.
       * @param {Object} options The parameters to pass.
       * @param [options.format=null] The format.
       * @param [options.data=null] The data item.
       * @see yfiles.view.DragDropItem#setData
       */
      constructor(options:{format?:string,data?:Object});
      /**
       * Gets the data item for the given format.
       * @param format The format.
       * @returns The data item that is stored for this format, or <code>null</code> if no such item exists.
       * @see yfiles.view.DragDropItem#setData
       */
      getData(format:string):Object;
      /**
       * Sets the data item for the given format.
       * @param format The format.
       * @param data The data item.
       * @see yfiles.view.DragDropItem#getData
       */
      setData(format:string,data:Object):void;
      /**
       * Gets the types of the data items stored in this instance.
       * @type {Array.<string>}
       */
      types:string[];
      static $class:yfiles.lang.Class;
    }
    export enum DragDropEffects{
      /**
       * Indicates that the drop target cannot be used for dropping the data.
       */
      NONE,
      /**
       * Indicates that the data can be copied to the drop target.
       */
      COPY,
      /**
       * Indicates that the data can be moved to the drop target.
       */
      MOVE,
      /**
       * Indicates that the data can be linked to the drop target.
       */
      LINK,
      /**
       * Indicates that the data can be copied or linked to the drop target.
       */
      COPY_LINK,
      /**
       * Indicates that the data can be linked or copied to the drop target.
       */
      LINK_MOVE,
      /**
       * Indicates that the data can be copied or moved to the drop target.
       */
      COPY_MOVE,
      /**
       * Indicates that the value is uninitialized.
       */
      UNINITIALIZED,
      /**
       * A combination of all of the flags.
       */
      ALL
    }
    /**
     * The event argument that will be used by {@link yfiles.view.DropTarget} to notify handlers of the various events during a drag
     * and drop operation.
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface DragEventArgs extends yfiles.lang.EventArgs{}
    export class DragEventArgs {
      /**
       * Gets the position of the current drag operation in the coordinates system of the provided element.
       * @param element The element for which the coordinates should be used.
       * @returns The position in the coordinates system of <code>element</code>
       */
      getPosition(element:HTMLElement):yfiles.geometry.Point;
      /**
       * Gets or sets the effects that the current handler can realize.
       * <p>
       * Setting this value will automatically mark this instance as {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @type {yfiles.view.DragDropEffects}
       */
      dropEffect:yfiles.view.DragDropEffects;
      /**
       * Gets the key states relevant during the drag and drop operation.
       * @type {yfiles.view.DragDropKeyStates}
       */
      keyStates:yfiles.view.DragDropKeyStates;
      /**
       * Gets the data that is associated with the drag.
       * @see yfiles.view.DragSource#startDrag
       * @type {yfiles.view.DragDropItem}
       */
      item:yfiles.view.DragDropItem;
      /**
       * Gets the allowed effects for the current drag operation as specified by the {@link yfiles.view.DragSource}.
       * @type {yfiles.view.DragDropEffects}
       */
      effectAllowed:yfiles.view.DragDropEffects;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.view.DragEventArgs} has been handled.
       * <p>
       * This property needs to be adjusted by handlers to tell the engine that the event has been handled.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum GradientSpreadMethod{
      /**
       * The outside is filled with the terminal colors of the gradient.
       */
      PAD,
      /**
       * The outside is filled by repeating the gradient but in alternating directions, until the area is filled.
       */
      REFLECT,
      /**
       * The outside is filled by repeating the original gradient.
       */
      REPEAT
    }
    /**
     * Specifies the properties of a transition point of a gradient.
     * @class
     * @implements {yfiles.lang.ICloneable}
     */
    export interface GradientStop extends Object,yfiles.lang.ICloneable{}
    export class GradientStop {
      /**
       * Creates a new transition point of a gradient with the given rgba values and offset.
       * <p>
       * The rgba values must be between 0 and 255.
       * </p>
       * @param r The red value.
       * @param g The green value.
       * @param b The blue value.
       * @param [a=255] The alpha value, 255 by default.
       * @param [offset=0] The offset, 0 by default.
       */
      constructor(r:number,g:number,b:number,a?:number,offset?:number);
      /**
       * Creates a new transition point of a gradient with the given rgba values and offset.
       * <p>
       * The rgba values must be between 0 and 255.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.r The red value.
       * @param options.g The green value.
       * @param options.b The blue value.
       * @param [options.a=255] The alpha value, 255 by default.
       * @param [options.offset=0] The offset, 0 by default.
       */
      constructor(options:{r:number,g:number,b:number,a?:number,offset?:number});
      /**
       * Creates a new transition point of a gradient with the given color and offset.
       * @param [color=null] The stop's color.
       * @param [offset=0] The offset.
       */
      constructor(color?:yfiles.view.Color,offset?:number);
      /**
       * Creates a new transition point of a gradient with the given color and offset.
       * @param {Object} options The parameters to pass.
       * @param [options.color=null] The stop's color.
       * @param [options.offset=0] The offset.
       */
      constructor(options:{color?:yfiles.view.Color,offset?:number});
      /**
       * Clones this instance by returning a {@link Object#memberwiseClone} or this in case this instance is already {@link yfiles.view.GradientStop#isFrozen frozen}.
       * @returns An instance of the same type as this instance.
       */
      clone():Object;
      /**
       * Freezes this instance.
       * <p>
       * Freezing an object makes it immutable, which can result in performance enhancements.
       * </p>
       */
      freeze():void;
      /**
       * Determines if this instance is frozen.
       * @returns 
       */
      isFrozen():boolean;
      /**
       * Gets or sets the color of this transition point.
       * @type {yfiles.view.Color}
       */
      color:yfiles.view.Color;
      /**
       * Gets or sets the offset of this transition point.
       * @type {number}
       */
      offset:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Classes that implement this interface can be placed in the SVGs defs section by the {@link yfiles.view.SvgDefsManager}.
     * <p>
     * The {@link yfiles.view.SvgDefsManager} can then create and update an element that can be used with an SVG use element. The
     * interface is also used to help the {@link yfiles.view.SvgDefsManager} to find unused elements that can be removed from the defs
     * section.
     * </p>
     * <p>
     * When an element is created using this interface, the {@link yfiles.view.SvgDefsManager} assigns an ID to it that is required to
     * reference it through an SVG use element. The ID that was assigned to it can be queried using the {@link yfiles.view.ICanvasContext#getDefsId}
     * method when the {@link yfiles.view.ISvgDefsCreator} object is passed in that was used to create the element.
     * </p>
     * @interface
     */
    export interface ISvgDefsCreator extends Object{
      /**
       * Checks if the specified node references the element represented by this object.
       * @param context The current canvas context.
       * @param node The SVG node.
       * @param id The defs id that has been stored for this instance by the provided context.
       * @returns <code>true</code> if <code>node</code> references the element represented by this instance.
       * @abstract
       */
      accept(context:yfiles.view.ICanvasContext,node:Node,id:string):boolean;
      /**
       * Creates the element that is put into the defs section of the SVG element.
       * @param context The canvas context that can be used to register brushes etc.
       * @returns An {@link } that can be put into the defs section of the SVG element.
       * @abstract
       */
      createDefsElement(context:yfiles.view.ICanvasContext):SVGElement;
      /**
       * Updates the defs element with the current gradient data.
       * @param context
       * @param oldElement
       * @abstract
       */
      updateDefsElement(context:yfiles.view.ICanvasContext,oldElement:SVGElement):void;
    }
    var ISvgDefsCreator:{
      /**
       * Determine whether the given element has an attribute with the given name that references the id in the defs element.
       * @param element The element to inspect
       * @param attributeName The attribute to check
       * @param defsId The id in the defs section
       * @returns Whether the attribute is equal to "url(#{defsId})"
       * @static
       */
      isAttributeReference?(element:Element,attributeName:string,defsId:string):boolean;
      /**
       * Determines whether the specified element is an SVG <code>use</code> element that references the given id.
       * @param element The element to inspect.
       * @param defsId The id of the element in the defs section.
       * @returns <code>true</code> if the specified element references the element; otherwise, <code>false</code>.
       * @static
       */
      isUseReference?(element:Node,defsId:string):boolean;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum ImageRenderingType{
      /**
       * Indicates that the rendering engine should use a balanced setup with a slight preference of quality.
       */
      AUTO,
      /**
       * Indicates that the rendering speed should be optimized.
       */
      OPTIMIZE_SPEED,
      /**
       * Indicates that the rendering quality should be optimized.
       */
      OPTIMIZE_QUALITY,
      INHERIT
    }
    export enum Key{
      ESCAPE,
      DELETE,
      SHIFT,
      CTRL,
      ENTER,
      SPACE,
      NONE,
      ALT,
      PAGE_UP,
      PAGE_DOWN,
      INSERT,
      BACK,
      ADD,
      SUBTRACT,
      F1,
      F2,
      F3,
      F4,
      F5,
      F6,
      F7,
      F8,
      F9,
      F10,
      F11,
      F12,
      ARROW_LEFT,
      ARROW_UP,
      ARROW_RIGHT,
      ARROW_DOWN,
      D0,
      D1,
      D2,
      D3,
      D4,
      D5,
      D6,
      D7,
      D8,
      D9,
      A,
      B,
      C,
      D,
      E,
      F,
      G,
      H,
      I,
      J,
      K,
      L,
      M,
      N,
      O,
      P,
      Q,
      R,
      S,
      T,
      U,
      V,
      W,
      X,
      Y,
      Z,
      TAB,
      PAUSE,
      HOME,
      END,
      N0,
      N1,
      N2,
      N3,
      N4,
      N5,
      N6,
      N7,
      N8,
      N9,
      MULTIPLY,
      CAPS,
      DECIMAL,
      DIVIDE,
      SEMICOLON,
      EQUAL,
      COMMA,
      DASH,
      PERIOD,
      ACCENT,
      OPEN_BRACKET,
      BACK_SLASH,
      CLOSE_BRACKET,
      SINGLE_QUOTE
    }
    export enum KeyEventType{
      /**
       * Neither Pressed, nor Typed, nor Released.
       */
      NONE,
      /**
       * The key has been pressed.
       */
      PRESSED,
      /**
       * The character has been typed.
       */
      TYPED,
      /**
       * The key has been released.
       */
      RELEASED
    }
    /**
     * A wrapper for the native keyboard event which abstracts away incompatibilities in browser implementations.
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface KeyEventArgs extends yfiles.lang.EventArgs{}
    export class KeyEventArgs {
      /**
       * Creates a new instance of this class from a native keyboard event.
       * @param evt The native keyboard event.
       * @param isPrintable if set to <code>true</code> [is printable].
       * @param type The type of the event.
       */
      constructor(evt:KeyboardEvent,isPrintable:boolean,type:yfiles.view.KeyEventType);
      /**
       * Sets this event's {@link yfiles.view.KeyEventArgs#defaultPrevented} property to <code>true</code> signalling that the event has been properly handled and any default behaviors should
       * be prevented.
       * <p>
       * This will call the corresponding method on the {@link yfiles.view.KeyEventArgs#originalEvent original native event}.
       * </p>
       * @see yfiles.view.KeyEventArgs#defaultPrevented
       */
      preventDefault():void;
      /**
       * Gets the original DOM event associated with this event.
       * @type {Event}
       */
      originalEvent:Event;
      /**
       * Gets the original source.
       * @type {Node}
       */
      originalSource:Node;
      /**
       * Gets the type of the event.
       * @type {yfiles.view.KeyEventType}
       */
      eventType:yfiles.view.KeyEventType;
      /**
       * Gets the key.
       * @type {yfiles.view.Key}
       */
      key:yfiles.view.Key;
      /**
       * Gets the charcode of the original event, if the original event was a key press.
       * <p>
       * Otherwise returns 0.
       * </p>
       * @type {number}
       */
      charCode:number;
      /**
       * Gets the modifiers.
       * @type {yfiles.view.ModifierKeys}
       */
      modifiers:yfiles.view.ModifierKeys;
      /**
       * Gets a value indicating whether the alt modifier is pressed.
       * @type {boolean}
       */
      alt:boolean;
      /**
       * Gets a value indicating whether the shift modifier is pressed.
       * @type {boolean}
       */
      shift:boolean;
      /**
       * Gets a value indicating whether the control modifier is pressed.
       * @type {boolean}
       */
      control:boolean;
      /**
       * Gets a value indicating whether the meta modifier is pressed.
       * @type {boolean}
       */
      meta:boolean;
      /**
       * Gets a value indicating whether the default behaviors for this event have been prevented by a previous call to
       * {@link yfiles.view.KeyEventArgs#preventDefault}.
       * @type {boolean}
       */
      defaultPrevented:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Fills an area with a linear gradient.
     * @class
     * @extends {yfiles.view.Fill}
     * @implements {yfiles.view.ISvgDefsCreator}
     */
    export interface LinearGradient extends yfiles.view.Fill,yfiles.view.ISvgDefsCreator{}
    export class LinearGradient {
      /**
       * Initializes a new instance of the {@link yfiles.view.LinearGradient} class.
       * @param {Object} [options=null] The parameters to pass.
       * @param options.gradientStops An array of {@link }s. Each item will be added to the {@link #gradientStops} collection.
       * @param {yfiles.geometry.Point} options.startPoint The start point of this gradient.
       * <p>
       * This option sets the {@link yfiles.view.LinearGradient#startPoint} property on the created object.
       * </p>
       * @param {yfiles.geometry.Point} options.endPoint The end point of this gradient.
       * <p>
       * This option sets the {@link yfiles.view.LinearGradient#endPoint} property on the created object.
       * </p>
       * @param {yfiles.view.GradientSpreadMethod} options.spreadMethod The spread method of this gradient.
       * <p>
       * This option sets the {@link yfiles.view.LinearGradient#spreadMethod} property on the created object.
       * </p>
       */
      constructor(options?:{startPoint?:yfiles.geometry.Point,endPoint?:yfiles.geometry.Point,spreadMethod?:yfiles.view.GradientSpreadMethod,gradientStops?:Array<any>});
      /**
       * 
       * @param context
       * @param item
       * @param id
       * @returns 
       */
      accept(context:yfiles.view.ICanvasContext,item:Node,id:string):boolean;
      /**
       * Creates a new SVG gradient that corresponds to this gradient brush.
       * @param context
       * @returns A new SVG gradient that corresponds to this gradient brush
       */
      createDefsElement(context:yfiles.view.ICanvasContext):SVGElement;
      /**
       * Creates a new SVG gradient that corresponds to this gradient brush.
       * @returns A new SVG gradient that corresponds to this gradient brush
       */
      toSvgGradient():SVGElement;
      /**
       * Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
       * @param context The context.
       * @param oldElement The old element.
       */
      updateDefsElement(context:yfiles.view.ICanvasContext,oldElement:SVGElement):void;
      /**
       * Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
       * @param oldElement The old element.
       * @param context The context.
       */
      updateSvgGradient(oldElement:SVGElement,context:yfiles.view.ICanvasContext):void;
      /**
       * Gets or sets the spread method of this gradient.
       * @type {yfiles.view.GradientSpreadMethod}
       */
      spreadMethod:yfiles.view.GradientSpreadMethod;
      /**
       * Gets or sets the start point of this gradient.
       * @type {yfiles.geometry.Point}
       */
      startPoint:yfiles.geometry.Point;
      /**
       * Gets or sets the end point of this gradient.
       * @type {yfiles.geometry.Point}
       */
      endPoint:yfiles.geometry.Point;
      /**
       * Gets or sets the gradient stops of this brush.
       * @type {yfiles.collections.List.<yfiles.view.GradientStop>}
       */
      gradientStops:yfiles.collections.List<yfiles.view.GradientStop>;
      static $class:yfiles.lang.Class;
    }
    export enum ModifierKeys{
      /**
       * Indicates that no modifier key was pressed.
       */
      NONE,
      /**
       * Indicates that the control key was pressed.
       */
      CONTROL,
      /**
       * Indicates that the shift key was pressed.
       */
      SHIFT,
      /**
       * Indicates that the alt key was pressed.
       */
      ALT,
      /**
       * Indicates that the windows (or meta) key was pressed.
       */
      WINDOWS
    }
    /**
     * Fills an area with a SVG pattern.
     * @class
     * @extends {yfiles.view.Fill}
     * @implements {yfiles.view.ISvgDefsCreator}
     */
    export interface PatternFill extends yfiles.view.Fill,yfiles.view.ISvgDefsCreator{}
    export class PatternFill {
      /**
       * Initializes a new instance of the {@link yfiles.view.PatternFill} class.
       */
      constructor();
      /**
       * 
       * @param context
       * @param node
       * @param id
       * @returns 
       */
      accept(context:yfiles.view.ICanvasContext,node:Node,id:string):boolean;
      /**
       * 
       * @param context
       * @returns 
       */
      createDefsElement(context:yfiles.view.ICanvasContext):SVGElement;
      /**
       * 
       * @param context
       * @param oldElement
       */
      updateDefsElement(context:yfiles.view.ICanvasContext,oldElement:SVGElement):void;
      /**
       * Gets or sets the content.
       * @type {yfiles.view.SvgVisual}
       */
      content:yfiles.view.SvgVisual;
      /**
       * Gets or sets the SVG view box of this pattern brush.
       * @type {yfiles.geometry.Rect}
       */
      viewBox:yfiles.geometry.Rect;
      /**
       * Gets or sets the origin of this pattern brush.
       * @type {yfiles.geometry.Point}
       */
      origin:yfiles.geometry.Point;
      /**
       * Gets or sets the size of this pattern brush.
       * @type {yfiles.geometry.Size}
       */
      size:yfiles.geometry.Size;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that holds all properties of a {@link yfiles.view.Stroke}.
     * @class
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface Stroke extends Object,yfiles.lang.ICloneable{}
    export class Stroke {
      /**
       * Initializes a new instance of the {@link yfiles.view.Stroke} class using the given rgba values and thickness.
       * <p>
       * The rgba values must be between 0 and 255.
       * </p>
       * @param r The red value.
       * @param g The green value.
       * @param b The blue value.
       * @param [a=255] The alpha value, 255 by default.
       * @param [thickness=1.0] The thickness, 1 by default.
       */
      constructor(r:number,g:number,b:number,a?:number,thickness?:number);
      /**
       * Initializes a new instance of the {@link yfiles.view.Stroke} class using the given rgba values and thickness.
       * <p>
       * The rgba values must be between 0 and 255.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.r The red value.
       * @param options.g The green value.
       * @param options.b The blue value.
       * @param [options.a=255] The alpha value, 255 by default.
       * @param [options.thickness=1.0] The thickness, 1 by default.
       * @param {yfiles.view.LineCap} options.lineCap The dash cap.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#lineCap} property on the created object.
       * </p>
       * @param {yfiles.view.DashStyle} options.dashStyle The dash style for the {@link yfiles.view.Stroke}.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#dashStyle} property on the created object.
       * </p>
       * @param {yfiles.view.LineJoin} options.lineJoin The line join property.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#lineJoin} property on the created object.
       * </p>
       * @param {number} options.miterLimit The miter limit.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#miterLimit} property on the created object.
       * </p>
       */
      constructor(options:{r:number,g:number,b:number,a?:number,thickness?:number,lineCap?:yfiles.view.LineCap,dashStyle?:yfiles.view.DashStyle,lineJoin?:yfiles.view.LineJoin,miterLimit?:number});
      /**
       * Initializes a new instance of the {@link yfiles.view.Stroke} class using the given brush and thickness.
       * @param [fill=null] The brush.
       * @param [thickness=1.0] The thickness.
       */
      constructor(fill?:yfiles.view.Fill,thickness?:number);
      /**
       * Initializes a new instance of the {@link yfiles.view.Stroke} class using the given brush and thickness.
       * @param {Object} options The parameters to pass.
       * @param [options.fill=null] The brush.
       * @param [options.thickness=1.0] The thickness.
       * @param {yfiles.view.LineCap} options.lineCap The dash cap.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#lineCap} property on the created object.
       * </p>
       * @param {yfiles.view.DashStyle} options.dashStyle The dash style for the {@link yfiles.view.Stroke}.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#dashStyle} property on the created object.
       * </p>
       * @param {yfiles.view.LineJoin} options.lineJoin The line join property.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#lineJoin} property on the created object.
       * </p>
       * @param {number} options.miterLimit The miter limit.
       * <p>
       * This option sets the {@link yfiles.view.Stroke#miterLimit} property on the created object.
       * </p>
       */
      constructor(options:{fill?:yfiles.view.Fill,thickness?:number,lineCap?:yfiles.view.LineCap,dashStyle?:yfiles.view.DashStyle,lineJoin?:yfiles.view.LineJoin,miterLimit?:number});
      /**
       * Assigns the stroke defined by this instance to the given <code>element</code>.
       * @param element The element whose stroke should be set.
       * @param context The context.
       */
      applyTo(element:SVGElement,context:yfiles.view.ICanvasContext):void;
      /**
       * Clones this instance by returning a {@link Object#memberwiseClone} or this in case this instance is already {@link yfiles.view.Stroke#isFrozen frozen}.
       * @returns An instance of the same type as this instance.
       */
      clone():Object;
      /**
       * Clones the current value of this instance to a new unfrozen {@link yfiles.view.Stroke}.
       * @returns 
       */
      cloneCurrentValue():yfiles.view.Stroke;
      /**
       * Freezes this instance.
       * <p>
       * Freezing an object makes it immutable, which can result in performance enhancements.
       * </p>
       */
      freeze():void;
      /**
       * Determines if this instance is frozen.
       * @returns 
       */
      isFrozen():boolean;
      /**
       * Gets or sets the brush for the {@link yfiles.view.Stroke}.
       * @default 'Black'
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      /**
       * Gets or sets the dash cap.
       * @default yfiles.view.LineCap.FLAT
       * @type {yfiles.view.LineCap}
       */
      lineCap:yfiles.view.LineCap;
      /**
       * Gets or sets the line join property.
       * @default yfiles.view.LineJoin.MITER
       * @type {yfiles.view.LineJoin}
       */
      lineJoin:yfiles.view.LineJoin;
      /**
       * Gets or sets the dash style for the {@link yfiles.view.Stroke}.
       * <p>
       * The visual appearance of the 'dashes' and 'gaps' specified by this property depends on the value of the property {@link yfiles.view.Stroke#lineCap}, too.
       * </p>
       * <p>
       * The predefined dash styles are designed for dash caps that extend into the 'gaps' like {@link yfiles.view.LineCap#SQUARE} and
       * {@link yfiles.view.LineCap#ROUND}. With {@link yfiles.view.LineCap#FLAT}, the dashes might not look like expected, for example {@link yfiles.view.DashStyle#DOT} will not
       * be visible at all. Since {@link yfiles.view.LineCap#FLAT} is the default value, you should explicitly set another value in this
       * case.
       * </p>
       * @default 'Solid'
       * @type {yfiles.view.DashStyle}
       */
      dashStyle:yfiles.view.DashStyle;
      /**
       * Gets or sets the miter limit.
       * @default 10.0
       * @type {number}
       */
      miterLimit:number;
      /**
       * Gets or sets the thickness of the stroke.
       * @default 1.0
       * @type {number}
       */
      thickness:number;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ALICE_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ALICE_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ANTIQUE_WHITE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ANTIQUE_WHITE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#AQUA} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static AQUA:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#AQUAMARINE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static AQUAMARINE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#AZURE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static AZURE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BEIGE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BEIGE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BISQUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BISQUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BLACK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BLACK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BLANCHED_ALMOND} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BLANCHED_ALMOND:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BLUE_VIOLET} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BLUE_VIOLET:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BROWN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BROWN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#BURLY_WOOD} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static BURLY_WOOD:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CADET_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CADET_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CHARTREUSE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CHARTREUSE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CHOCOLATE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CHOCOLATE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CORAL} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CORAL:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CORNFLOWER_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CORNFLOWER_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CORNSILK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CORNSILK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CRIMSON} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CRIMSON:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#CYAN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static CYAN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_CYAN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_CYAN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_GOLDENROD} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_GOLDENROD:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_KHAKI} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_KHAKI:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_MAGENTA} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_MAGENTA:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_OLIVE_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_OLIVE_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_ORANGE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_ORANGE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_ORCHID} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_ORCHID:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_RED} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_RED:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_SALMON} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_SALMON:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_SEA_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_SEA_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_SLATE_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_SLATE_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_SLATE_GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_SLATE_GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_TURQUOISE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_TURQUOISE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DARK_VIOLET} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DARK_VIOLET:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DEEP_PINK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DEEP_PINK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DEEP_SKY_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DEEP_SKY_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DIM_GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DIM_GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#DODGER_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static DODGER_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#FIREBRICK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static FIREBRICK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#FLORAL_WHITE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static FLORAL_WHITE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#FOREST_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static FOREST_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#FUCHSIA} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static FUCHSIA:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GAINSBORO} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GAINSBORO:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GHOST_WHITE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GHOST_WHITE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GOLD} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GOLD:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GOLDENROD} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GOLDENROD:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#GREEN_YELLOW} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static GREEN_YELLOW:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#HONEYDEW} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static HONEYDEW:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#HOT_PINK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static HOT_PINK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#INDIAN_RED} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static INDIAN_RED:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#INDIGO} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static INDIGO:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#IVORY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static IVORY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#KHAKI} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static KHAKI:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LAVENDER} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LAVENDER:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LAVENDER_BLUSH} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LAVENDER_BLUSH:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LAWN_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LAWN_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LEMON_CHIFFON} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LEMON_CHIFFON:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_CORAL} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_CORAL:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_CYAN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_CYAN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_GOLDENROD_YELLOW} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_GOLDENROD_YELLOW:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_PINK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_PINK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_SALMON} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_SALMON:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_SEA_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_SEA_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_SKY_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_SKY_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_SLATE_GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_SLATE_GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_STEEL_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_STEEL_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIGHT_YELLOW} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIGHT_YELLOW:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIME} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIME:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LIME_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LIME_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#LINEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static LINEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MAGENTA} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MAGENTA:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MAROON} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MAROON:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_AQUAMARINE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_AQUAMARINE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_ORCHID} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_ORCHID:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_PURPLE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_PURPLE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_SEA_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_SEA_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_SLATE_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_SLATE_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_SPRING_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_SPRING_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_TURQUOISE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_TURQUOISE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MEDIUM_VIOLET_RED} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MEDIUM_VIOLET_RED:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MIDNIGHT_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MIDNIGHT_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MINT_CREAM} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MINT_CREAM:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MISTY_ROSE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MISTY_ROSE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#MOCCASIN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static MOCCASIN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#NAVAJO_WHITE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static NAVAJO_WHITE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#NAVY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static NAVY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#OLD_LACE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static OLD_LACE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#OLIVE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static OLIVE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#OLIVE_DRAB} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static OLIVE_DRAB:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ORANGE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ORANGE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ORANGE_RED} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ORANGE_RED:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ORCHID} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ORCHID:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PALE_GOLDENROD} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PALE_GOLDENROD:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PALE_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PALE_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PALE_TURQUOISE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PALE_TURQUOISE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PALE_VIOLET_RED} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PALE_VIOLET_RED:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PAPAYA_WHIP} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PAPAYA_WHIP:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PEACH_PUFF} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PEACH_PUFF:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PERU} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PERU:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PINK} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PINK:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PLUM} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PLUM:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#POWDER_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static POWDER_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#PURPLE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static PURPLE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#RED} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static RED:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ROSY_BROWN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ROSY_BROWN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#ROYAL_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static ROYAL_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SADDLE_BROWN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SADDLE_BROWN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SALMON} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SALMON:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SANDY_BROWN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SANDY_BROWN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SEA_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SEA_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SEA_SHELL} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SEA_SHELL:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SIENNA} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SIENNA:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SILVER} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SILVER:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SKY_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SKY_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SLATE_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SLATE_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SLATE_GRAY} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SLATE_GRAY:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SNOW} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SNOW:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#SPRING_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static SPRING_GREEN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#STEEL_BLUE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static STEEL_BLUE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#TAN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static TAN:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#TEAL} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static TEAL:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#THISTLE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static THISTLE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#TOMATO} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static TOMATO:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#TRANSPARENT} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static TRANSPARENT:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#TURQUOISE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static TURQUOISE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#VIOLET} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static VIOLET:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#WHEAT} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static WHEAT:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#WHITE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static WHITE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#WHITE_SMOKE} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static WHITE_SMOKE:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#YELLOW} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static YELLOW:yfiles.view.Stroke;
      /**
       * Gets a {@link yfiles.view.Stroke#isFrozen frozen} {@link yfiles.view.Stroke} that uses {@link yfiles.view.Fill#YELLOW_GREEN} as its {@link yfiles.view.Stroke#fill} and has a {@link yfiles.view.Stroke#thickness} of <code>1</code>.
       */
      static YELLOW_GREEN:yfiles.view.Stroke;
      static $class:yfiles.lang.Class;
    }
    export enum LineCap{
      /**
       * Indicates that the line cap of a {@link yfiles.view.Stroke} should be squared.
       */
      SQUARE,
      /**
       * Indicates that the line cap of a {@link yfiles.view.Stroke} should be flat.
       */
      FLAT,
      /**
       * Indicates that the line cap of a {@link yfiles.view.Stroke} should be a semicircle.
       */
      ROUND
    }
    export enum LineJoin{
      /**
       * Indicates that the line joins of a {@link yfiles.view.Stroke} should be beveled.
       */
      BEVEL,
      /**
       * Indicates that the line joins of a {@link yfiles.view.Stroke} should be rounded.
       */
      ROUND,
      /**
       * Indicates that the line joins of a {@link yfiles.view.Stroke} should be sharp.
       */
      MITER
    }
    /**
     * Fills an area with a radial gradient.
     * @class
     * @extends {yfiles.view.Fill}
     * @implements {yfiles.view.ISvgDefsCreator}
     */
    export interface RadialGradient extends yfiles.view.Fill,yfiles.view.ISvgDefsCreator{}
    export class RadialGradient {
      /**
       * Initializes a new instance of the {@link yfiles.view.RadialGradient} class.
       * @param {Object} [options=null] The parameters to pass.
       * @param options.gradientStops An array of {@link }s. Each item will be added to the {@link #gradientStops} collection.
       * @param {yfiles.geometry.Point} options.center The center of this radial gradient.
       * <p>
       * This option sets the {@link yfiles.view.RadialGradient#center} property on the created object.
       * </p>
       * @param {number} options.radiusX The radius of the largest circle of this radial gradient in horizontal direction.
       * <p>
       * This option sets the {@link yfiles.view.RadialGradient#radiusX} property on the created object.
       * </p>
       * @param {number} options.radiusY The radius of the largest circle of this radial gradient in vertical direction.
       * <p>
       * This option sets the {@link yfiles.view.RadialGradient#radiusY} property on the created object.
       * </p>
       * @param {yfiles.view.GradientSpreadMethod} options.spreadMethod The spread method of this gradient.
       * <p>
       * This option sets the {@link yfiles.view.RadialGradient#spreadMethod} property on the created object.
       * </p>
       * @param {yfiles.geometry.Point} options.gradientOrigin The origin of this radial gradient.
       * <p>
       * This option sets the {@link yfiles.view.RadialGradient#gradientOrigin} property on the created object.
       * </p>
       */
      constructor(options?:{center?:yfiles.geometry.Point,radiusX?:number,radiusY?:number,spreadMethod?:yfiles.view.GradientSpreadMethod,gradientOrigin?:yfiles.geometry.Point,gradientStops?:Array<any>});
      /**
       * 
       * @param context
       * @param item
       * @param id
       * @returns 
       */
      accept(context:yfiles.view.ICanvasContext,item:Node,id:string):boolean;
      /**
       * Creates a new SVG gradient that corresponds to this gradient brush.
       * @param context
       * @returns A new SVG gradient that corresponds to this gradient brush
       */
      createDefsElement(context:yfiles.view.ICanvasContext):SVGElement;
      /**
       * Creates a new SVG gradient that corresponds to this gradient brush.
       * @returns A new SVG gradient that corresponds to this gradient brush
       */
      toSvgGradient():SVGElement;
      /**
       * Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
       * @param context The context.
       * @param oldElement The old element.
       */
      updateDefsElement(context:yfiles.view.ICanvasContext,oldElement:SVGElement):void;
      /**
       * Updates the SVG gradient within the <code>defs</code> section of the SVG document if it has changed.
       * @param oldElement The old element.
       * @param context The context.
       */
      updateSvgGradient(oldElement:SVGElement,context:yfiles.view.ICanvasContext):void;
      /**
       * Gets or sets the center of this radial gradient.
       * @type {yfiles.geometry.Point}
       */
      center:yfiles.geometry.Point;
      /**
       * Gets or sets the origin of this radial gradient.
       * @type {yfiles.geometry.Point}
       */
      gradientOrigin:yfiles.geometry.Point;
      /**
       * Gets or sets the radius of the largest circle of this radial gradient in horizontal direction.
       * @type {number}
       */
      radiusX:number;
      /**
       * Gets or sets the radius of the largest circle of this radial gradient in vertical direction.
       * @type {number}
       */
      radiusY:number;
      /**
       * Gets or sets the spread method of this gradient.
       * @type {yfiles.view.GradientSpreadMethod}
       */
      spreadMethod:yfiles.view.GradientSpreadMethod;
      /**
       * Gets or sets the gradient stops of this brush.
       * @type {yfiles.collections.List.<yfiles.view.GradientStop>}
       */
      gradientStops:yfiles.collections.List<yfiles.view.GradientStop>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base class for {@link yfiles.view.Visual}s that use HTML5's Canvas for the visualization.
     * <p>
     * Note that instances of this class cannot be used anywhere where a {@link yfiles.view.Visual} is required. The visualization is
     * implemented by the {@link yfiles.view.HtmlCanvasVisual#paint} method. This means that this kind of visual does not work in a {@link yfiles.view.SvgVisualGroup}
     * and wrapping the visual will not work as there is no {@link SVGElement} that could be put into another SVG container. These kind
     * of visuals can be used at the top-level of a {@link yfiles.view.CanvasComponent}'s {@link yfiles.view.ICanvasObject}s, only.
     * </p>
     * @see yfiles.view.HtmlCanvasVisual#paint
     * @class
     * @extends {yfiles.view.Visual}
     */
    export interface HtmlCanvasVisual extends yfiles.view.Visual{}
    export class HtmlCanvasVisual {
      constructor();
      /**
       * Paints onto the context using HTML5 Canvas operations.
       * <p>
       * Implementations should not destroy the context's state, but should make sure to restore the state to the previously
       * active state. This is especially true for the transformation and clip.
       * </p>
       * @param renderContext The render context of the {@link }
       * @param ctx The HTML5 Canvas context to use for rendering.
       * @abstract
       */
      paint(renderContext:yfiles.view.IRenderContext,ctx:CanvasRenderingContext2D):void;
      static $class:yfiles.lang.Class;
    }
    export enum ShapeRenderingType{
      /**
       * Indicates that the rendering engine should use a balanced setup with a slight preference of geometric precision.
       */
      AUTO,
      /**
       * Indicates that the rendering speed should be optimized.
       */
      OPTIMIZE_SPEED,
      /**
       * Indicates that the rendering should be optimized for crisp edges.
       */
      CRISP_EDGES,
      /**
       * Indicates that the rendering should be optimized for geometric precision.
       */
      GEOMETRIC_PRECISION,
      INHERIT
    }
    /**
     * Fills an area with a specified solid color.
     * @class
     * @extends {yfiles.view.Fill}
     */
    export interface SolidColorFill extends yfiles.view.Fill{}
    export class SolidColorFill {
      /**
       * Initializes a new instance of the {@link yfiles.view.SolidColorFill} class with the specified rgba values.
       * <p>
       * All values must be between 0 and 255.
       * </p>
       * @param r The red value.
       * @param g The green value.
       * @param b The blue value.
       * @param [a=255] The alpha value, 255 by default.
       */
      constructor(r:number,g:number,b:number,a?:number);
      /**
       * Initializes a new instance of the {@link yfiles.view.SolidColorFill} class with the specified color.
       * @param [color=null] The solid color with which the area is filled.
       */
      constructor(color?:yfiles.view.Color);
      /**
       * Gets the solid color with which the area is filled.
       * @type {yfiles.view.Color}
       */
      color:yfiles.view.Color;
      static $class:yfiles.lang.Class;
    }
    /**
     * Wraps a {@link SVGElement} as {@link yfiles.view.Visual}.
     * @class
     * @extends {yfiles.view.Visual}
     */
    export interface SvgVisual extends yfiles.view.Visual{}
    export class SvgVisual {
      /**
       * Initializes a new instance of the {@link yfiles.view.SvgVisual} class which wraps the specified SVG element.
       * @param element The SVG element which this object wraps.
       */
      constructor(element:SVGElement);
      /**
       * Sets the transform to a scale transform using the given parameters.
       * <p>
       * This implementation uses caching and the baseVal property of the <code>transform</code> attribute if possible and thus performs
       * better than using the plain DOM API.
       * </p>
       * @param transformable The SVG element that should be translated.
       * @param scaleX The vertical scale.
       * @param scaleY The horizontal scale.
       * @static
       */
      static setScale(transformable:Element,scaleX:number,scaleY:number):void;
      /**
       * Sets the transform to a translate transform using the given parameters.
       * <p>
       * This implementation uses caching and the baseVal property of the <code>transform</code> attribute if possible and thus performs
       * better than using the plain DOM API.
       * </p>
       * @param transformable The SVG element that should be translated.
       * @param dx The offset on the x-axis.
       * @param dy The offset on the y-axis.
       * @static
       */
      static setTranslate(transformable:Element,dx:number,dy:number):void;
      /**
       * Gets or sets the SVG element that is wrapped by this object.
       * @type {SVGElement}
       */
      svgElement:SVGElement;
      static $class:yfiles.lang.Class;
    }
    export enum HorizontalTextAlignment{
      /**
       * Indicates that the text should be centered.
       */
      CENTER,
      /**
       * Indicates that the text should be aligned to the left.
       */
      LEFT,
      /**
       * Indicates that the text should be aligned to the right.
       */
      RIGHT
    }
    export enum TextDecoration{
      /**
       * No Text Decoration
       */
      NONE,
      /**
       * Underlined text
       */
      UNDERLINE,
      /**
       * Overlined Text
       */
      OVERLINE,
      /**
       * Text with line through
       */
      LINE_THROUGH,
      /**
       * Blinking Text
       */
      BLINK
    }
    export enum TextDecorations{
      /**
       * No text decoration
       */
      NONE,
      /**
       * Underlined text.
       */
      UNDERLINE,
      /**
       * Overlined text.
       */
      OVERLINE,
      /**
       * Crossed out text.
       */
      STRIKETHROUGH
    }
    export enum TextWrapping{
      /**
       * The text should not be trimmed.
       */
      NONE,
      /**
       * The text could be wrapped and clipped after any character.
       */
      CHARACTER,
      /**
       * The text could be wrapped and clipped after any character and an ellipsis is added to indicate clipped text.
       */
      CHARACTER_ELLIPSIS,
      /**
       * The text could be wrapped and clipped at word boundaries.
       */
      WORD,
      /**
       * The text could be wrapped and clipped at word boundaries and an ellipsis is added to indicate trimmed text.
       */
      WORD_ELLIPSIS
    }
    /**
     * Displays a string or any HTML element as tool tip.
     * @class
     */
    export interface ToolTip extends Object{}
    export class ToolTip {
      /**
       * Creates a new instance.
       * <p>
       * This implementation calls {@link yfiles.view.ToolTip#createToolTip} to create the HTML element to display as tool tip.
       * </p>
       */
      constructor();
      /**
       * Creates the {@link HTMLDivElement HTML element} in which the content is displayed.
       * <p>
       * This implementation creates a simple HTML <code>div</code> element with the CSS class <code>yfiles-tooltip</code>.
       * </p>
       * @returns A HTML div element.
       * @protected
       */
      createToolTip():HTMLDivElement;
      /**
       * Sets the content to the HTML element that is displayed as tool tip.
       * <p>
       * This implementation adds a <code>string</code> as <code>innerHTML</code> of the Div. A <code>HTMLElement</code> is added as child to the Div.
       * </p>
       * <p>
       * Subclasses which intend to use a more complex element as tool tip have to override this method to set the content
       * properly.
       * </p>
       * @param oldContent The content to remove.
       * @param newContent The content to add.
       * @protected
       */
      onContentChanged(oldContent:Object,newContent:Object):void;
      /**
       * Gets the div element that represents this {@link yfiles.view.ToolTip}.
       * @type {HTMLDivElement}
       */
      div:HTMLDivElement;
      /**
       * Specifies whether the tool tip is visible.
       * @type {boolean}
       */
      isOpen:boolean;
      /**
       * The top position of the tool tip.
       * @type {number}
       */
      verticalOffset:number;
      /**
       * The left position of the tool tip.
       * @type {number}
       */
      horizontalOffset:number;
      /**
       * Gets or sets the desired size of this control.
       * @type {yfiles.geometry.Size}
       */
      desiredSize:yfiles.geometry.Size;
      /**
       * Gets or sets the content of this tooltip.
       * <p>
       * The content can be either a string or any HTML element.
       * </p>
       * <p>
       * This setter invokes the method {@link yfiles.view.ToolTip#onContentChanged} if the content is changed.
       * </p>
       * @type {Object}
       */
      content:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * Describes a font, with size, family, style, weight and line spacing.
     * @class
     */
    export interface Font extends Object{}
    export class Font {
      /**
       * Initializes a new instance of the {@link yfiles.view.Font} class with size 10, "Arial" family, normal weight and style and line
       * spacing <code>0.5</code>, or the given values.
       * @param [fontFamily=null]
       * @param [fontSize=10]
       * @param [fontStyle=null]
       * @param [fontWeight=null]
       * @param [lineSpacing=0.5]
       * @param [textDecoration=null]
       * @see yfiles.view.Font#createCopy
       */
      constructor(fontFamily?:string,fontSize?:number,fontStyle?:yfiles.view.FontStyle,fontWeight?:yfiles.view.FontWeight,lineSpacing?:number,textDecoration?:yfiles.view.TextDecoration);
      /**
       * Initializes a new instance of the {@link yfiles.view.Font} class with size 10, "Arial" family, normal weight and style and line
       * spacing <code>0.5</code>, or the given values.
       * @param {Object} options The parameters to pass.
       * @param [options.fontFamily=null]
       * @param [options.fontSize=10]
       * @param [options.fontStyle=null]
       * @param [options.fontWeight=null]
       * @param [options.lineSpacing=0.5]
       * @param [options.textDecoration=null]
       * @see yfiles.view.Font#createCopy
       */
      constructor(options:{fontFamily?:string,fontSize?:number,fontStyle?:yfiles.view.FontStyle,fontWeight?:yfiles.view.FontWeight,lineSpacing?:number,textDecoration?:yfiles.view.TextDecoration});
      /**
       * Assigns the text properties of this instance to the given text element.
       * @param textElement The text element to assign set the {@link } on.
       */
      applyTo(textElement:SVGTextElement):void;
      /**
       * Creates a copy with the values of this instance or optionally the explicitly specified values.
       * @param [fontFamily=null] The font family to use for the copy.
       * @param [fontSize=-1] The font size to use for the copy.
       * @param [fontStyle=null] The font style to use for the copy.
       * @param [fontWeight=null] The font weight to use for the copy.
       * @param [lineSpacing=-1] The line spacing to use for the copy.
       * @param [textDecoration=null] The text decoration to use for the copy.
       * @returns A new {@link } instance.
       */
      createCopy(fontFamily?:string,fontSize?:number,fontStyle?:yfiles.view.FontStyle,fontWeight?:yfiles.view.FontWeight,lineSpacing?:number,textDecoration?:yfiles.view.TextDecoration):yfiles.view.Font;
      /**
       * Creates a copy with the values of this instance or optionally the explicitly specified values.
       * @param {Object} options The parameters to pass.
       * @param [options.fontFamily=null] The font family to use for the copy.
       * @param [options.fontSize=-1] The font size to use for the copy.
       * @param [options.fontStyle=null] The font style to use for the copy.
       * @param [options.fontWeight=null] The font weight to use for the copy.
       * @param [options.lineSpacing=-1] The line spacing to use for the copy.
       * @param [options.textDecoration=null] The text decoration to use for the copy.
       * @returns A new {@link } instance.
       */
      createCopy(options:{fontFamily?:string,fontSize?:number,fontStyle?:yfiles.view.FontStyle,fontWeight?:yfiles.view.FontWeight,lineSpacing?:number,textDecoration?:yfiles.view.TextDecoration}):yfiles.view.Font;
      /**
       * Gets the size of the font.
       * <p>
       * The font size is interpreted as px.
       * </p>
       * @default 10
       * @type {number}
       */
      fontSize:number;
      /**
       * Gets or sets the font family.
       * @default 'Arial'
       * @type {string}
       */
      fontFamily:string;
      /**
       * Gets or sets the font style.
       * @default yfiles.view.FontStyle.NORMAL
       * @type {yfiles.view.FontStyle}
       */
      fontStyle:yfiles.view.FontStyle;
      /**
       * Gets or sets the font weight.
       * @default yfiles.view.FontWeight.NORMAL
       * @type {yfiles.view.FontWeight}
       */
      fontWeight:yfiles.view.FontWeight;
      /**
       * Gets or sets the line spacing.
       * <p>
       * The value is interpreted as parts of the line height.
       * </p>
       * @default 0.5
       * @type {number}
       */
      lineSpacing:number;
      /**
       * Gets or sets the text decoration.
       * @default yfiles.view.TextDecoration.NONE
       * @type {yfiles.view.TextDecoration}
       */
      textDecoration:yfiles.view.TextDecoration;
      static $class:yfiles.lang.Class;
    }
    export enum FontStyle{
      NORMAL,
      ITALIC,
      OBLIQUE,
      INHERIT
    }
    export enum FontWeight{
      NORMAL,
      BOLD,
      BOLDER,
      LIGHTER,
      ITEM100,
      ITEM200,
      ITEM300,
      ITEM400,
      ITEM500,
      ITEM600,
      ITEM700,
      ITEM800,
      ITEM900,
      INHERIT
    }
    export enum VerticalTextAlignment{
      /**
       * Indicates that the object should be placed in the center of another object.
       */
      CENTER,
      /**
       * Indicates that the object should be placed at the top of another object.
       */
      TOP,
      /**
       * Indicates that the object should be placed at the bottom of another object.
       */
      BOTTOM
    }
    /**
     * Base class for all objects that support drawing in the context of a {@link yfiles.view.CanvasComponent}.
     * @see yfiles.view.SvgVisual
     * @see yfiles.view.HtmlCanvasVisual
     * @class
     */
    export interface Visual extends Object{}
    export class Visual {
      static $class:yfiles.lang.Class;
    }
    export enum MouseEventTypes{
      /**
       * Not a mouse event.
       */
      NONE,
      /**
       * The mouse has been moved.
       */
      MOVE,
      /**
       * The mouse has been dragged, which means it has been moved while at least one button has been pressed.
       */
      DRAG,
      /**
       * A button has been pressed.
       */
      DOWN,
      /**
       * A button has been released.
       * <p>
       * If there was no drag event since the last press event, this will result in a click.
       * </p>
       */
      UP,
      /**
       * The mouse has entered the control's bounds.
       */
      ENTER,
      /**
       * The mouse has exited the control's bounds.
       */
      LEAVE,
      /**
       * The mouse wheel has been turned.
       */
      WHEEL,
      /**
       * A mouse click has been recognized.
       * <p>
       * This is the case if a press has directly been followed by a release.
       * </p>
       */
      CLICK,
      /**
       * Mouse input capture has been lost.
       * <p>
       * This can happen during mouse drags, if any button is pressed and the focus is taken from the application or transferred
       * to another component by other means then moving the mouse.
       * </p>
       */
      LOST_CAPTURE
    }
    /**
     * Complex event arguments that is used by the {@link yfiles.view.CanvasComponent} to indicate mouse events in world coordinates.
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface MouseEventArgs extends yfiles.lang.EventArgs{}
    export class MouseEventArgs {
      /**
       * Constructs a new mouse event argument.
       * @param eventType The type of the event
       * @param location the position of the mouse in world coordinates at the time of the event
       * @param [changedButtons=yfiles.view.MouseButtons.NONE] the mouse buttons that have changed state if the type of event is {@link #DOWN} or
       * {@link #UP}
       * @param [changedModifiers=yfiles.view.ModifierKeys.NONE] the modifier Keys that have been changed
       * @param [buttons=yfiles.view.MouseButtons.NONE] the current state of the MouseButtons
       * @param [modifiers=yfiles.view.ModifierKeys.NONE] the current state of the modifier Keys
       * @param [clickCount=0] the number of clicks this event represents, 2 for a double click
       * @param [mouseWheelDelta=0] the signed number of mouse wheel turn units
       * @param [scrollAmount=0] The number of lines to scroll, if <code>scrollType</code> is {@link #LINES}.
       * @param [scrollType=yfiles.view.MouseWheelDeltaMode.LINES] The type of scrolling performed.
       * @param originalEvent
       */
      constructor(originalEvent:Event,eventType:yfiles.view.MouseEventTypes,location:yfiles.geometry.Point,changedButtons?:yfiles.view.MouseButtons,changedModifiers?:yfiles.view.ModifierKeys,buttons?:yfiles.view.MouseButtons,modifiers?:yfiles.view.ModifierKeys,clickCount?:number,mouseWheelDelta?:number,scrollAmount?:number,scrollType?:yfiles.view.MouseWheelDeltaMode);
      /**
       * Constructs a new mouse event argument.
       * @param {Object} options The parameters to pass.
       * @param options.eventType The type of the event
       * @param options.location the position of the mouse in world coordinates at the time of the event
       * @param [options.changedButtons=yfiles.view.MouseButtons.NONE] the mouse buttons that have changed state if the type of event is {@link #DOWN} or
       * {@link #UP}
       * @param [options.changedModifiers=yfiles.view.ModifierKeys.NONE] the modifier Keys that have been changed
       * @param [options.buttons=yfiles.view.MouseButtons.NONE] the current state of the MouseButtons
       * @param [options.modifiers=yfiles.view.ModifierKeys.NONE] the current state of the modifier Keys
       * @param [options.clickCount=0] the number of clicks this event represents, 2 for a double click
       * @param [options.mouseWheelDelta=0] the signed number of mouse wheel turn units
       * @param [options.scrollAmount=0] The number of lines to scroll, if <code>scrollType</code> is {@link #LINES}.
       * @param [options.scrollType=yfiles.view.MouseWheelDeltaMode.LINES] The type of scrolling performed.
       * @param options.originalEvent
       */
      constructor(options:{originalEvent:Event,eventType:yfiles.view.MouseEventTypes,location:yfiles.geometry.Point,changedButtons?:yfiles.view.MouseButtons,changedModifiers?:yfiles.view.ModifierKeys,buttons?:yfiles.view.MouseButtons,modifiers?:yfiles.view.ModifierKeys,clickCount?:number,mouseWheelDelta?:number,scrollAmount?:number,scrollType?:yfiles.view.MouseWheelDeltaMode});
      /**
       * Sets this event's {@link yfiles.view.MouseEventArgs#defaultPrevented} property to <code>true</code> signalling that the event has been properly handled and any default behaviors should
       * be prevented.
       * <p>
       * This will call the corresponding method on the {@link yfiles.view.MouseEventArgs#originalEvent original native event}.
       * </p>
       * @see yfiles.view.MouseEventArgs#defaultPrevented
       */
      preventDefault():void;
      /**
       * Gets the type of this event.
       * @type {yfiles.view.MouseEventTypes}
       */
      eventType:yfiles.view.MouseEventTypes;
      /**
       * Returns the coordinates in the world coordinate space associated with this event.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * Returns the number of clicks the current event represents if it is of type {@link yfiles.view.MouseEventTypes#CLICK}.
       * @type {number}
       */
      clickCount:number;
      /**
       * Gets the signed number of mouse wheel turn units.
       * <p>
       * Note that this can, for high-precision mouse wheels, include fractional turns.
       * </p>
       * @type {number}
       */
      wheelDelta:number;
      /**
       * Gets the distance to scroll at once when {@link yfiles.view.MouseEventArgs#deltaMode} is {@link yfiles.view.MouseWheelDeltaMode#LINES}.
       * <p>
       * This is a system setting that only applies to scrolling (not zooming) and usually translates to the number of small
       * scroll increments per scroll wheel notch.
       * </p>
       * <p>
       * This property has a value of <code>1</code> if {@link yfiles.view.MouseEventArgs#deltaMode} is {@link yfiles.view.MouseWheelDeltaMode#PAGE}, as it is not applicable in that case.
       * </p>
       * @type {number}
       */
      scrollAmount:number;
      /**
       * Gets the scrolling behavior.
       * <p>
       * This is a system setting that only applies to scrolling (not zooming) and can be either
       * {@link yfiles.view.MouseWheelDeltaMode#LINES}, in which case {@link yfiles.view.MouseEventArgs#scrollAmount} will contain the number of "lines" to scroll with each scroll
       * wheel notch, or {@link yfiles.view.MouseWheelDeltaMode#PAGE} in which case a single scroll wheel notch should scroll a "page" at
       * a time.
       * </p>
       * @type {yfiles.view.MouseWheelDeltaMode}
       */
      deltaMode:yfiles.view.MouseWheelDeltaMode;
      /**
       * Returns the modifier keys that have changed for this event.
       * @see yfiles.view.MouseEventArgs#modifiers
       * @type {yfiles.view.ModifierKeys}
       */
      changedModifiers:yfiles.view.ModifierKeys;
      /**
       * Returns the mouse buttons that have changed for this event.
       * @see yfiles.view.MouseEventArgs#buttons
       * @type {yfiles.view.MouseButtons}
       */
      changedButtons:yfiles.view.MouseButtons;
      /**
       * The state of the modifier keys at the time of the event creation.
       * @type {yfiles.view.ModifierKeys}
       */
      modifiers:yfiles.view.ModifierKeys;
      /**
       * The state of the mouse buttons at the time of the event creation.
       * @type {yfiles.view.MouseButtons}
       */
      buttons:yfiles.view.MouseButtons;
      /**
       * Gets the original DOM event associated with this event.
       * @type {Event}
       */
      originalEvent:Event;
      /**
       * Gets a value indicating whether the default behaviors for this event have been prevented by a previous call to
       * {@link yfiles.view.MouseEventArgs#preventDefault}.
       * @type {boolean}
       */
      defaultPrevented:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum MouseWheelDeltaMode{
      /**
       * Scrolling should scroll a set number of lines per mouse wheel notch.
       * @see yfiles.view.MouseEventArgs#scrollAmount
       */
      LINES,
      /**
       * Scrolling should scroll a single "page" per mouse wheel notch.
       */
      PAGE
    }
    export enum TouchEventTypes{
      /**
       * Not a touch event.
       */
      NONE,
      /**
       * A pointer has been moved.
       */
      MOVE,
      /**
       * A touch device went down.
       */
      DOWN,
      /**
       * A touch device went up.
       */
      UP,
      /**
       * A touch device has entered the control's bounds.
       */
      ENTER,
      /**
       * A touch device has exited the control's bounds.
       */
      LEAVE,
      /**
       * A touch tap has been recognized.
       * <p>
       * This is the case if a down has directly been followed by an up.
       * </p>
       */
      TAP,
      /**
       * A long press has been recognized.
       * <p>
       * This is the case if a down has been held a certain amount of time without a move.
       * </p>
       */
      LONG_PRESS,
      /**
       * Touch capture has been lost while the device was down. or sent to the background or similar actions.
       * <p>
       * This can be the case if the application or control is minimized
       * </p>
       */
      LOST_CAPTURE
    }
    /**
     * Complex event arguments that is used by the {@link yfiles.view.CanvasComponent} to indicate touch events in world coordinates.
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface TouchEventArgs extends yfiles.lang.EventArgs{}
    export class TouchEventArgs {
      /**
       * Constructs a new touch event argument.
       * @param eventType The type of the event
       * @param location The position of the touch device in world coordinates at the time of the event
       * @param device The device the event was triggered by
       * @param originalEvent
       */
      constructor(originalEvent:Event,eventType:yfiles.view.TouchEventTypes,location:yfiles.geometry.Point,device:yfiles.view.TouchDevice);
      /**
       * Constructs a new touch event argument.
       * @param eventType The type of the event
       * @param location The position of the touch device in world coordinates at the time of the event
       * @param device The device the event was triggered by
       * @param tapCount the number of taps this event represents, 2 for a double tap
       * @param originalEvent
       */
      constructor(originalEvent:Event,eventType:yfiles.view.TouchEventTypes,location:yfiles.geometry.Point,device:yfiles.view.TouchDevice,tapCount:number);
      /**
       * Sets this event's {@link yfiles.view.TouchEventArgs#defaultPrevented} property to <code>true</code> signalling that the event has been properly handled and any default behaviors should
       * be prevented.
       * <p>
       * This will call the corresponding method on the {@link yfiles.view.TouchEventArgs#originalEvent original native event}.
       * </p>
       * @see yfiles.view.TouchEventArgs#defaultPrevented
       */
      preventDefault():void;
      /**
       * Gets the type of this event.
       * @type {yfiles.view.TouchEventTypes}
       */
      eventType:yfiles.view.TouchEventTypes;
      /**
       * Returns the location represented by this event in world coordinates.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * Returns the touch device associated with this event.
       * @type {yfiles.view.TouchDevice}
       */
      device:yfiles.view.TouchDevice;
      /**
       * Returns the number of taps the current event represents if it is of type {@link yfiles.view.TouchEventTypes#TAP}.
       * @type {number}
       */
      tapCount:number;
      /**
       * Gets the original DOM event associated with this event.
       * @type {Event}
       */
      originalEvent:Event;
      /**
       * Gets a value indicating whether the default behaviors for this event have been prevented by a previous call to
       * {@link yfiles.view.TouchEventArgs#preventDefault}.
       * @type {boolean}
       */
      defaultPrevented:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The representation of a touch point on the screen.
     * @class
     */
    export interface TouchDevice extends Object{}
    export class TouchDevice {
      /**
       * Gets the device's index
       * @type {number}
       */
      deviceIndex:number;
      /**
       * Gets a value indicating whether this device is the primary one.
       * <p>
       * If {@link yfiles.view.TouchDevice#deviceIndex} is <code>0</code> this property has the value <code>true</code>, otherwise <code>false</code>.
       * </p>
       * @type {boolean}
       */
      isPrimaryDevice:boolean;
      /**
       * Gets the last device coordinate in the view coordinate system.
       * @type {yfiles.geometry.Point}
       */
      lastViewCoordinate:yfiles.geometry.Point;
      /**
       * Gets the time of the last {@link yfiles.view.TouchEventArgs event} that was raised by this device.
       * @type {Date}
       */
      lastEventTimeJS:Date;
      /**
       * Indicates if this device is currently down or not.
       * <p>
       * Default is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      isDown:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The event argument class that is used by {@link yfiles.view.DragSource#addQueryContinueDragListener QueryContinueDrag}
     * during drags to query the drag source whether the drag should be continued.
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @final
     */
    export interface QueryContinueDragEventArgs extends yfiles.lang.EventArgs{}
    export class QueryContinueDragEventArgs {
      /**
       * Gets or sets the action.
       * @type {yfiles.view.DragAction}
       */
      action:yfiles.view.DragAction;
      /**
       * Gets a value indicating whether the escape key had been pressed.
       * @type {boolean}
       */
      escapePressed:boolean;
      /**
       * Gets the current state of the keys.
       * @type {yfiles.view.DragDropKeyStates}
       */
      keyStates:yfiles.view.DragDropKeyStates;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.view.QueryContinueDragEventArgs} is handled.
       * @type {boolean}
       */
      handled:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum DragAction{
      /**
       * Whether the drag should be continued.
       */
      CONTINUE,
      /**
       * Whether the drag should be finalized by an immediate drop.
       */
      DROP,
      /**
       * Whether the drag operation should be canceled.
       */
      CANCEL
    }
    /**
     * Helper class for drag and drop support that is used for the source of drag and drop operations.
     * @class
     */
    export interface DragSource extends Object{}
    export class DragSource {
      /**
       * Initializes a new instance of the {@link yfiles.view.DragSource} class setting the {@link yfiles.view.DragSource#source} property to the given element.
       * @param source The source.
       */
      constructor(source:HTMLElement);
      /**
       * Cleans up by removing the association with the {@link yfiles.view.DragSource#source} element.
       */
      cleanUp():void;
      /**
       * Raises the {@link yfiles.view.DragSource#addQueryContinueDragListener QueryContinueDrag} event.
       * <p>
       * If the event is not {@link yfiles.view.QueryContinueDragEventArgs#handled handled}, a default logic will be applied: the drag operation is canceled if the left mouse button is
       * released or escape has been pressed, otherwise the drag will be {@link yfiles.view.DragAction#CONTINUE continued}.
       * </p>
       * @param evt The {@link } instance containing the event data.
       */
      onQueryContinueDrag(evt:yfiles.view.QueryContinueDragEventArgs):void;
      /**
       * Initiates a new drag operation using the provided drag data and effects.
       * <p>
       * If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
       * depending on the allowed drag-drop operations for these elements:
       * </p>
       * <ul>
       * <li><code>yfiles-cursor-dragdrop-move</code></li>
       * <li><code>yfiles-cursor-dragdrop-link</code></li>
       * <li><code>yfiles-cursor-dragdrop-all</code></li>
       * <li><code>yfiles-cursor-dragdrop-no-drop</code></li>
       * </ul>
       * @param dragData The data to use for the drag.
       * @param [dragDropEffects=yfiles.view.DragDropEffects.ALL] The allowed effects.
       * @param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
       */
      startDrag(dragData:yfiles.view.DragDropItem,dragDropEffects?:yfiles.view.DragDropEffects,useCssCursors?:boolean):void;
      /**
       * Initiates a new drag operation using the provided drag data and effects.
       * <p>
       * If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
       * depending on the allowed drag-drop operations for these elements:
       * </p>
       * <ul>
       * <li><code>yfiles-cursor-dragdrop-move</code></li>
       * <li><code>yfiles-cursor-dragdrop-link</code></li>
       * <li><code>yfiles-cursor-dragdrop-all</code></li>
       * <li><code>yfiles-cursor-dragdrop-no-drop</code></li>
       * </ul>
       * @param {Object} options The parameters to pass.
       * @param options.dragData The data to use for the drag.
       * @param [options.dragDropEffects=yfiles.view.DragDropEffects.ALL] The allowed effects.
       * @param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
       */
      startDrag(options:{dragData:yfiles.view.DragDropItem,dragDropEffects?:yfiles.view.DragDropEffects,useCssCursors?:boolean}):void;
      /**
       * Adds the given listener for the <code>QueryContinueDrag</code> event that occurs when the state of the {@link yfiles.view.DragDropKeyStates}
       * has changed to query the source whether the drag operation should be continued.
       * <p>
       * Handlers should adjust the {@link yfiles.view.QueryContinueDragEventArgs#handled} and {@link yfiles.view.QueryContinueDragEventArgs#action} properties accordingly.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.DragSource#removeQueryContinueDragListener
       */
      addQueryContinueDragListener(listener:(sender:Object,evt:yfiles.view.QueryContinueDragEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryContinueDrag</code> event that occurs when the state of the {@link yfiles.view.DragDropKeyStates}
       * has changed to query the source whether the drag operation should be continued.
       * <p>
       * Handlers should adjust the {@link yfiles.view.QueryContinueDragEventArgs#handled} and {@link yfiles.view.QueryContinueDragEventArgs#action} properties accordingly.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.DragSource#addQueryContinueDragListener
       */
      removeQueryContinueDragListener(listener:(sender:Object,evt:yfiles.view.QueryContinueDragEventArgs)=>void):void;
      /**
       * Gets or sets the source element from which drags can originate.
       * <p>
       * If the {@link yfiles.view.DragSource#allowDrag} property is enabled, this will automatically register mouse event handlers on the element that will recognize
       * default drag gestures (press followed by drag) to {@link yfiles.view.DragSource#startDrag initiate a drag and drop operation}.
       * </p>
       * @default null
       * @type {HTMLElement}
       */
      source:HTMLElement;
      /**
       * Gets or sets a value indicating whether default drag gestures should be automatically recognized on the {@link yfiles.view.DragSource#source}.
       * @default false
       * @type {boolean}
       */
      allowDrag:boolean;
      /**
       * Gets or sets the data that will be used for the {@link yfiles.view.DragSource#allowDrag automatic drag operation}.
       * <p>
       * If this data is not a {@link yfiles.view.DragDropItem}, the {@link yfiles.input.DropInputMode expected format} of the automatic drag operation will be <code>code</code>.
       * </p>
       * @default null
       * @type {Object}
       */
      item:Object;
      /**
       * Gets or sets the {@link yfiles.view.DragSource#effectAllowed} that will be used for the {@link yfiles.view.DragSource#allowDrag automatic drag operation}.
       * @default yfiles.view.DragDropEffects.ALL
       * @type {yfiles.view.DragDropEffects}
       */
      effectAllowed:yfiles.view.DragDropEffects;
      static $class:yfiles.lang.Class;
    }
    /**
     * Helper class that plays the role of the target during a drag and drop operation that is initiated using {@link yfiles.view.DragSource#startDrag}
     * in a {@link yfiles.view.DragSource}.
     * @class
     */
    export interface DropTarget extends Object{}
    export class DropTarget {
      /**
       * Initializes a new instance of the {@link yfiles.view.DropTarget} class.
       * <p>
       * In order to make the instance available to the drag and drop mechanisms, the {@link yfiles.view.DropTarget#setDropTarget} helper
       * method should be used. If the optional <code>element</code> is passed in, it will automatically be registered.
       * </p>
       * @param [element=null] The element to optionally {@link #setDropTarget associate} with the new instance.
       * @see yfiles.view.DropTarget#setDropTarget
       * @see yfiles.view.DropTarget#getDropTarget
       */
      constructor(element?:Element);
      /**
       * Gets the {@link yfiles.view.DropTarget} instance associated with the given element.
       * @param element The element to get the drop target instance of.
       * @returns The associated instance or <code>null</code>.
       * @see yfiles.view.DropTarget#setDropTarget
       * @static
       */
      static getDropTarget(element:Element):yfiles.view.DropTarget;
      /**
       * Sets the drop target for the given element.
       * @param element The element or <code>null</code> to register the drop target instance with.
       * @param dropTarget The instance to attach to the element, <code>null</code> to remove the association from the element.
       * @see yfiles.view.DropTarget#getDropTarget
       * @static
       */
      static setDropTarget(element:Element,dropTarget:yfiles.view.DropTarget):void;
      /**
       * Adds the given listener for the <code>DragEnter</code> event that occurs when a drag enters the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.DropTarget#removeDragEnterListener
       */
      addDragEnterListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragEnter</code> event that occurs when a drag enters the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.DropTarget#addDragEnterListener
       */
      removeDragEnterListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragLeave</code> event that occurs when a drag leaves the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.DropTarget#removeDragLeaveListener
       */
      addDragLeaveListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragLeave</code> event that occurs when a drag leaves the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.DropTarget#addDragLeaveListener
       */
      removeDragLeaveListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragOver</code> event that occurs when a drag is being performed over the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.DropTarget#removeDragOverListener
       */
      addDragOverListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragOver</code> event that occurs when a drag is being performed over the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.DropTarget#addDragOverListener
       */
      removeDragOverListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Drop</code> event that occurs when the drop operation is performed on the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.view.DropTarget#removeDropListener
       */
      addDropListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Drop</code> event that occurs when the drop operation is performed on the drop target.
       * <p>
       * The handler may adjust the {@link yfiles.view.DragEventArgs#dropEffect} and call {@link yfiles.view.DragEventArgs#handled}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.view.DropTarget#addDropListener
       */
      removeDropListener(listener:(sender:Object,evt:yfiles.view.DragEventArgs)=>void):void;
      /**
       * Gets or sets a value indicating whether dropping to this target is currently allowed.
       * @type {boolean}
       */
      allowDrop:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An animation for the {@link yfiles.graph.IStripe stripes} in an {@link yfiles.graph.ITable}.
     * @class
     * @implements {yfiles.view.IAnimation}
     */
    export interface TableAnimation extends Object,yfiles.view.IAnimation{}
    export class TableAnimation {
      /**
       * Initializes a new instance of the {@link yfiles.view.TableAnimation} class for the given table, the column widths, and row
       * heights.
       * <p>
       * Note that only the sizes of
       * <b>leaf stripes</b>, i.e. those without child stripes, are considered. The sizes of a stripe with children is implicitly determined by its
       * contents.
       * </p>
       * @param table The table to animate.
       * @param columnLayout The sizes of the <b>leaf</b> columns, in natural order.
       * @param rowLayout The sizes of the <b>leaf</b> rows, in natural order.
       */
      constructor(table:yfiles.graph.ITable,columnLayout:number[],rowLayout:number[]);
      /**
       * 
       * @param time
       */
      animate(time:number):void;
      /**
       * 
       */
      cleanup():void;
      /**
       * Create the animation for a single {@link yfiles.graph.IStripe}
       * @param stripe The stripe
       * @param targetSize The target size of the stripe
       * @returns An animation that morphs the stripe size
       * @protected
       */
      createStripeAnimation(stripe:yfiles.graph.IStripe,targetSize:number):yfiles.view.IAnimation;
      /**
       * 
       */
      initialize():void;
      /**
       * Gets or sets the preferred duration of the animation.
       * @type {yfiles.lang.TimeSpan}
       */
      preferredDuration:yfiles.lang.TimeSpan;
      static $class:yfiles.lang.Class;
    }
    /**
     * Manages the selection state of items in an {@link yfiles.graph.ITable} instance.
     * <p>
     * This interface provides access to the {@link yfiles.view.ISelectionModel.<T>} instances that manage the selection of rows and columns. The generic methods that
     * use {@link yfiles.graph.IModelItem} parameters delegate to the corresponding domain-specific selection models.
     * </p>
     * @see yfiles.view.ISelectionModel.<T>
     * @interface
     * @implements {yfiles.view.ISelectionModel.<yfiles.graph.IStripe>}
     */
    export interface IStripeSelection extends Object,yfiles.view.ISelectionModel<yfiles.graph.IStripe>{
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected rows.
       * <p>
       * This is the row part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IRow>}
       */
      selectedRows:yfiles.view.ISelectionModel<yfiles.graph.IRow>;
      /**
       * An {@link yfiles.view.ISelectionModel.<T>} of the selected columns.
       * <p>
       * This is the column part of the selection that this instance is a composite of.
       * </p>
       * @abstract
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IColumn>}
       */
      selectedColumns:yfiles.view.ISelectionModel<yfiles.graph.IColumn>;
      /**
       * Whether to allow mixed row/column selections
       * @abstract
       * @type {boolean}
       */
      allowMixedSelection:boolean;
      /**
       * Whether to allow the selection to span multiple table instances
       * <p>
       * The default is <code>false</code>
       * </p>
       * @abstract
       * @type {boolean}
       */
      allowCrossTableSelection:boolean;
    }
    var IStripeSelection:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Default implementation of the {@link yfiles.view.IStripeSelection} interface.
     * @class
     * @implements {yfiles.view.IStripeSelection}
     */
    export interface StripeSelection extends Object,yfiles.view.IStripeSelection{}
    export class StripeSelection {
      /**
       * Default constructor that creates empty selection models.
       */
      constructor();
      /**
       * 
       */
      clear():void;
      /**
       * 
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<yfiles.graph.IStripe>;
      /**
       * Return whether <code>stripe</code> is currently selected
       * @param stripe The stripe to test
       * @returns <code>true</code> iff <code>stripe</code> is currently selected
       */
      isSelected(stripe:yfiles.graph.IStripe):boolean;
      /**
       * Raises the {@link yfiles.view.StripeSelection#addItemSelectionChangedListener ItemSelectionChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onItemSelectionChanged(evt:yfiles.view.ItemSelectionChangedEventArgs<yfiles.graph.IStripe>):void;
      /**
       * 
       * @param item
       * @param selected
       */
      setSelected(item:yfiles.graph.IStripe,selected:boolean):void;
      /**
       * Whether to allow mixed row/column selections
       * <p>
       * The default is <code>false</code>
       * </p>
       * @type {boolean}
       */
      allowMixedSelection:boolean;
      /**
       * Whether to allow the selection to span multiple table instances
       * <p>
       * The default is <code>true</code>
       * </p>
       * @type {boolean}
       */
      allowCrossTableSelection:boolean;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IRow>}
       */
      selectedRows:yfiles.view.ISelectionModel<yfiles.graph.IRow>;
      /**
       * 
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IColumn>}
       */
      selectedColumns:yfiles.view.ISelectionModel<yfiles.graph.IColumn>;
      /**
       * 
       * @type {number}
       */
      size:number;
      /**
       * 
       * @param listener
       */
      addItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeItemSelectionChangedListener(listener:(sender:Object,evt:yfiles.view.ItemSelectionChangedEventArgs<yfiles.graph.IStripe>)=>void):void;
      static $class:yfiles.lang.Class;
    }
  }export namespace input{
    export enum GridSnapTypes{
      /**
       * There is no grid snapping at all.
       */
      NONE,
      /**
       * Elements shall be snapped to horizontal grid lines.
       */
      HORIZONTAL_LINES,
      /**
       * Elements shall be snapped to vertical grid lines.
       */
      VERTICAL_LINES,
      /**
       * Elements shall be snapped to horizontal and vertical grid lines.
       */
      LINES,
      /**
       * Elements shall be snapped to grid points.
       */
      GRID_POINTS,
      /**
       * Elements shall be snapped to horizontal and vertical grid lines as well as to grid points.
       */
      ALL
    }
    /**
     * An interface that can enumerate hits of a given type for a certain position in world coordinates.
     * Type parameter <code>T</code>: the type of hits returned by the enumerator
     * @interface
     * @template T
     */
    export interface IHitTester<T> extends Object{
      /**
       * Yields an enumerable that enumerates the hits for a given world coordinate.
       * @param context The context in which to perform the hit testing.
       * @param location the coordinates in the world coordinate system
       * @returns an enumerable that yields hits for the given coordinates
       * @abstract
       */
      enumerateHits(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):yfiles.collections.IEnumerable<T>;
    }
    var IHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a flagging interface for instances of {@link yfiles.input.IHitTester.<T>} that enumerate over a collection of {@link yfiles.graph.INode INodes}.
     * <p>
     * An implementation of this interface can be queried using the lookup of a {@link yfiles.view.GraphComponent}
     * </p>
     * @interface
     * @implements {yfiles.input.IHitTester.<yfiles.graph.INode>}
     */
    export interface INodeHitTester extends Object,yfiles.input.IHitTester<yfiles.graph.INode>{
    }
    var INodeHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a flagging interface for instances of {@link yfiles.input.IHitTester.<T>} that enumerate over a collection of {@link yfiles.graph.ILabel ILabels}.
     * <p>
     * An implementation of this interface can be queried using the lookup of a {@link yfiles.view.GraphComponent}
     * </p>
     * @interface
     * @implements {yfiles.input.IHitTester.<yfiles.graph.ILabel>}
     */
    export interface ILabelHitTester extends Object,yfiles.input.IHitTester<yfiles.graph.ILabel>{
    }
    var ILabelHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a flagging interface for instances of {@link yfiles.input.IHitTester.<T>} that enumerate over a collection of {@link yfiles.graph.ILabelOwner}.
     * <p>
     * An implementation of this interface can be queried using the lookup of a {@link yfiles.view.GraphComponent}
     * </p>
     * @interface
     * @implements {yfiles.input.IHitTester.<yfiles.graph.ILabelOwner>}
     */
    export interface ILabelOwnerHitTester extends Object,yfiles.input.IHitTester<yfiles.graph.ILabelOwner>{
    }
    var ILabelOwnerHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a flagging interface for instances of {@link yfiles.input.IHitTester.<T>} that enumerate over a collection of {@link yfiles.graph.IPort IPorts}.
     * <p>
     * An implementation of this interface can be queried using the lookup of a {@link yfiles.view.GraphComponent}
     * </p>
     * @interface
     * @implements {yfiles.input.IHitTester.<yfiles.graph.IPort>}
     */
    export interface IPortHitTester extends Object,yfiles.input.IHitTester<yfiles.graph.IPort>{
    }
    var IPortHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a flagging interface for instances of {@link yfiles.input.IHitTester.<T>} that enumerate over a collection of {@link yfiles.graph.IEdge IEdges}.
     * <p>
     * An implementation of this interface can be queried using the lookup of a {@link yfiles.view.GraphComponent}
     * </p>
     * @interface
     * @implements {yfiles.input.IHitTester.<yfiles.graph.IEdge>}
     */
    export interface IEdgeHitTester extends Object,yfiles.input.IHitTester<yfiles.graph.IEdge>{
    }
    var IEdgeHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a flagging interface for instances of {@link yfiles.input.IHitTester.<T>} that enumerate over a collection of {@link yfiles.graph.IBend IBends}.
     * <p>
     * An implementation of this interface can be queried using the lookup of a {@link yfiles.view.GraphComponent}
     * </p>
     * @interface
     * @implements {yfiles.input.IHitTester.<yfiles.graph.IBend>}
     */
    export interface IBendHitTester extends Object,yfiles.input.IHitTester<yfiles.graph.IBend>{
    }
    var IBendHitTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Abstract base class implementation of an {@link yfiles.input.IInputMode} that can be used concurrently with other {@link yfiles.input.IInputMode}
     * implementations if its {@link yfiles.input.InputModeBase#exclusive} property is <code>true</code>.
     * <p>
     * This class adds convenience methods for derived classes.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface InputModeBase extends Object,yfiles.input.IInputMode{}
    export class InputModeBase {
      /**
       * Initializes a new instance of the {@link yfiles.input.InputModeBase} class.
       * @param exclusive Iff this mode should be {@link #exclusive}.
       * @protected
       */
      constructor(exclusive:boolean);
      /**
       * Initializes a new instance of the {@link yfiles.input.InputModeBase} class that is {@link yfiles.input.InputModeBase#exclusive} by default.
       * @protected
       */
      constructor();
      /**
       * Cancels the editing of this mode by first disabling and afterwards reenabling the installed {@link yfiles.input.InputModeBase#controller}.
       * <p>
       * This implementation disables and reenables the {@link yfiles.input.InputModeBase#controller} and fires the {@link yfiles.input.InputModeBase#onCanceled} event afterwards.
       * Classes that need to perform clean-up should override the {@link yfiles.input.InputModeBase#onConcurrencyControllerDeactivated}
       * method and invoke <b>base.OnConcurrencyControllerDeactivated</b> as the last statement.
       * </p>
       * @see yfiles.input.InputModeBase#onCanceled
       * @see yfiles.input.InputModeBase#addCanceledListener
       */
      cancel():void;
      /**
       * Determines whether this instance can request the mutex.
       * <p>
       * This will return true, iff a {@link yfiles.input.InputModeBase#controller} has been injected and the helper allows requesting the mutex.
       * </p>
       * @returns Whether a call to {@link #requestMutex} would currently succeed.
       * @protected
       */
      canRequestMutex():boolean;
      /**
       * Creates an implementation of {@link yfiles.input.IInputModeContext} that is specific to this mode.
       * <p>
       * This is needed for events that are triggered by this InputMode to signalize where the event is coming from. A common
       * usage for this is for example in {@link yfiles.input.IHitTestable#isHit} calls.
       * </p>
       * <p>
       * The result can be given to {@link yfiles.input.IHitTestable#isHit} or {@link yfiles.view.CanvasComponent#hitElementsAt} to let
       * implementations of {@link yfiles.input.IHitTestable} get access to the current {@link yfiles.input.IInputModeContext}. This can be done
       * by looking up {@link yfiles.input.IInputModeContext} from the {@link yfiles.view.ICanvasContext}'s {@link yfiles.graph.ILookup#lookup} method or
       * simply downcasting.
       * </p>
       * @returns A readily configured {@link }.
       * @see yfiles.view.CanvasComponent#hitElementsAt
       * @see yfiles.input.IHitTestable#isHit
       * @see yfiles.input.IInputModeContext
       * @protected
       */
      createChildInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Helper method that yields a suitably configured {@link yfiles.input.InputModeEventArgs} for this input mode.
       * @param context An input mode context that is available in the {@link }.
       * <p>
       * Can be <code>null</code> in which case a new context for this instance is created automatically.
       * </p>
       * @returns An input mode event argument that is configured for this instance.
       * @protected
       */
      createInputModeEventArgs(context:yfiles.input.IInputModeContext):yfiles.input.InputModeEventArgs;
      /**
       * Determines whether this instance owns the input mutex.
       * <p>
       * If no {@link yfiles.input.InputModeBase#controller} had been associated with this instance upon the call to {@link yfiles.input.InputModeBase#requestMutex} this method will
       * return <code>false</code>.
       * </p>
       * @returns Whether this instance has the input mutex.
       * @protected
       */
      hasMutex():boolean;
      /**
       * Performs one-time initialization of this instance.
       * <p>
       * This method should not be invoked by subclasses. This will be done automatically upon first {@link yfiles.input.InputModeBase#install installation} of this mode.
       * </p>
       * <p>
       * This code will be executed only once per instance. The {@link yfiles.input.InputModeBase#inputModeContext} property will be <code>null</code> when this code is executed. This method
       * should not be used to install this mode into a specific canvas. Subclasses should always call <code>base.Initialize()</code> first.
       * </p>
       * @see yfiles.input.InputModeBase#install
       * @protected
       */
      initialize():void;
      /**
       * Installs this mode into a {@link yfiles.view.CanvasComponent} using the provided {@link yfiles.input.IInputModeContext}.
       * <p>
       * Subclasses should override this method and call<code>base.Install(context)</code>, first. One-time initialization should be performed
       * in the {@link yfiles.input.InputModeBase#initialize} method. This implementation will call the {@link yfiles.input.InputModeBase#initialize}
       * method the first time this mode gets installed. The initialization will be surrounded by calls to {@link yfiles.input.InputModeBase#onInitializing}
       * and {@link yfiles.input.InputModeBase#onInitialized} to trigger the corresponding events.
       * </p>
       * @param context the context to install this mode into
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.InputModeBase#inputModeContext
       * @see yfiles.view.CanvasComponent#inputMode
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Event dispatch method that will dispatch the {@link yfiles.input.InputModeBase#addCanceledListener Canceled}
       * event.
       * <p>
       * This method is called by subclasses when the editing process has been {@link yfiles.input.InputModeBase#cancel}ed.
       * </p>
       * @param evt The event arguments.
       * @see yfiles.input.InputModeBase#initialize
       * @protected
       */
      onCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Event dispatch method that will dispatch the {@link yfiles.input.InputModeBase#addInitializedListener Initialized}
       * event.
       * <p>
       * This method will be called by {@link yfiles.input.InputModeBase#install} for the first installation directly after {@link yfiles.input.InputModeBase#initialize}
       * has returned.
       * </p>
       * @param evt The event arguments.
       * @see yfiles.input.InputModeBase#initialize
       * @protected
       */
      onInitialized(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Event dispatch method that will dispatch the {@link yfiles.input.InputModeBase#addInitializingListener Initializing}
       * event.
       * <p>
       * This method will be called by {@link yfiles.input.InputModeBase#install} for the first installation directly before {@link yfiles.input.InputModeBase#initialize}
       * is invoked.
       * </p>
       * @param evt The event arguments.
       * @see yfiles.input.InputModeBase#initialize
       * @protected
       */
      onInitializing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called when this instance obtains the mutex.
       * <p>
       * If no {@link yfiles.input.InputModeBase#controller} had been associated with this instance upon the call to {@link yfiles.input.InputModeBase#requestMutex} this method will
       * not be called.
       * </p>
       * @protected
       */
      onMutexObtained():void;
      /**
       * Called when this instance released the mutex.
       * <p>
       * If no {@link yfiles.input.InputModeBase#controller} had been associated with this instance upon the call to {@link yfiles.input.InputModeBase#requestMutex} this method will
       * not be called.
       * </p>
       * @protected
       */
      onMutexReleased():void;
      /**
       * Event dispatch method that will dispatch the {@link yfiles.input.InputModeBase#addStoppedListener Stopped}
       * event.
       * <p>
       * This method will be called by {@link yfiles.input.InputModeBase#tryStop} or can be called by subclasses during a {@link yfiles.input.InputModeBase#tryStop}
       * operation.
       * </p>
       * @param evt The event arguments.
       * @see yfiles.input.InputModeBase#tryStop
       * @see yfiles.input.InputModeBase#addStoppedListener
       * @protected
       */
      onStopped(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Releases previously captured keyboard input.
       * @protected
       */
      releaseKeyboard():void;
      /**
       * Releases the mutex that is currently owned by this instance.
       * <p>
       * This will trigger the {@link yfiles.input.InputModeBase#onMutexReleased} method if this instance previously owned the mutex.
       * </p>
       * @protected
       */
      releaseMutex():void;
      /**
       * Releases previously captured mouse and touch input.
       * @protected
       */
      releasePointer():void;
      /**
       * Requests capturing of all keyboard input that happens in the {@link yfiles.view.CanvasComponent}.
       * <p>
       * This sets the CaptureAllKeyboardInput property to
       * <code>true</code>, which will prevent visuals inside the canvas from stealing key strokes.
       * </p>
       * @see yfiles.input.InputModeBase#releaseKeyboard
       * @protected
       */
      requestKeyboard():void;
      /**
       * Requests the mutex from the current {@link yfiles.input.InputModeBase#controller}.
       * <p>
       * This implementation will call the {@link yfiles.input.InputModeBase#onMutexObtained} method if the mutex has been successfully
       * obtained. If the mutex cannot be obtained this method will trigger an exception.
       * </p>
       * @throws {yfiles.lang.Exception} If the mutex could not be obtained.
       * @protected
       */
      requestMutex():void;
      /**
       * Requests capturing of all mouse and touch input that happens in the {@link yfiles.view.CanvasComponent}.
       * <p>
       * This sets the {@link yfiles.view.CanvasComponent#captureAllPointerInput} property to <code>true</code>, which will prevent visuals inside the canvas from stealing mouse input.
       * </p>
       * @see yfiles.input.InputModeBase#releasePointer
       * @protected
       */
      requestPointer():void;
      /**
       * Tries to stop the editing.
       * <p>
       * This implementation returns if this instance does not currently {@link yfiles.input.InputModeBase#hasMutex have the input mutex}.
       * </p>
       * <p>
       * This implementation calls {@link yfiles.input.InputModeBase#onStopped}. Subclasses that want to prevent a stop or need to do
       * special clean up should do this here if the base class implementation of {@link yfiles.input.InputModeBase#tryStop} returns
       * <code>true</code>. A typical implementation should follow this idiom if it wants to stop:
       * </p>
       * <p>
       * and if it doesn't want to stop simply return <code>false</code> immediately.
       * </p>
       * <p>
       * Calling this method raises the {@link yfiles.input.InputModeBase#addStoppedListener Stopped} event.
       * </p>
       * @returns <code>true</code> iff this instance does not {@link #hasMutex own the mutex}.
       * @see yfiles.input.InputModeBase#onStopped
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the canvas.
       * <p>
       * Subclasses should always call <code>base.Uninstall(context)</code> as the last statement.
       * </p>
       * @param context The context to remove this mode from. This is the same instance that has been passed to
       * {@link #install}.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Adds the given listener for the <code>Initializing</code> event that occurs before the call to {@link yfiles.input.InputModeBase#initialize}
       * <p>
       * Clients can use this event to perform configuration of the instance shortly before the initialization begins.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.InputModeBase#install
       * @see yfiles.input.InputModeBase#removeInitializingListener
       */
      addInitializingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Initializing</code> event that occurs before the call to
       * {@link yfiles.input.InputModeBase#initialize}
       * <p>
       * Clients can use this event to perform configuration of the instance shortly before the initialization begins.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.InputModeBase#install
       * @see yfiles.input.InputModeBase#addInitializingListener
       */
      removeInitializingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Initialized</code> event that occurs after the call to {@link yfiles.input.InputModeBase#initialize}
       * <p>
       * Clients can use this event to perform post configuration steps of the instance shortly after the initialization has been
       * performed.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.InputModeBase#install
       * @see yfiles.input.InputModeBase#removeInitializedListener
       */
      addInitializedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Initialized</code> event that occurs after the call to
       * {@link yfiles.input.InputModeBase#initialize}
       * <p>
       * Clients can use this event to perform post configuration steps of the instance shortly after the initialization has been
       * performed.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.InputModeBase#install
       * @see yfiles.input.InputModeBase#addInitializedListener
       */
      removeInitializedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Canceled</code> event that occurs after the call to {@link yfiles.input.InputModeBase#cancel}
       * <p>
       * Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been canceled.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.InputModeBase#onCanceled
       * @see yfiles.input.InputModeBase#cancel
       * @see yfiles.input.InputModeBase#tryStop
       * @see yfiles.input.InputModeBase#addStoppedListener
       * @see yfiles.input.InputModeBase#removeCanceledListener
       */
      addCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Canceled</code> event that occurs after the call to {@link yfiles.input.InputModeBase#cancel}
       * <p>
       * Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been canceled.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.InputModeBase#onCanceled
       * @see yfiles.input.InputModeBase#cancel
       * @see yfiles.input.InputModeBase#tryStop
       * @see yfiles.input.InputModeBase#addStoppedListener
       * @see yfiles.input.InputModeBase#addCanceledListener
       */
      removeCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Stopped</code> event that occurs after the call to {@link yfiles.input.InputModeBase#tryStop}
       * <p>
       * Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been stopped.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.InputModeBase#onStopped
       * @see yfiles.input.InputModeBase#cancel
       * @see yfiles.input.InputModeBase#tryStop
       * @see yfiles.input.InputModeBase#addCanceledListener
       * @see yfiles.input.InputModeBase#removeStoppedListener
       */
      addStoppedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Stopped</code> event that occurs after the call to {@link yfiles.input.InputModeBase#tryStop}
       * <p>
       * Clients can use this event to perform post cleanup steps of the instance shortly after the mode has been stopped.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.InputModeBase#onStopped
       * @see yfiles.input.InputModeBase#cancel
       * @see yfiles.input.InputModeBase#tryStop
       * @see yfiles.input.InputModeBase#addCanceledListener
       * @see yfiles.input.InputModeBase#addStoppedListener
       */
      removeStoppedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Returns whether this mode is currently installed in a {@link yfiles.view.CanvasComponent}.
       * @see yfiles.input.InputModeBase#install
       * @type {boolean}
       */
      installed:boolean;
      /**
       * Returns the context instance this mode is currently installed in or <code>null</code> if this instance is not installed.
       * <p>
       * Note that this instance should not be passed down to subordinate modes or instances. Instead a corresponding context
       * that has this instance set as the {@link yfiles.input.IInputModeContext#parentInputMode} should be used. One such instance can be obtained through helper method
       * {@link yfiles.input.InputModeBase#createChildInputModeContext}.
       * </p>
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.InputModeBase#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * Returns if this mode is active.
       * <p>
       * This mode is active if a {@link yfiles.input.InputModeBase#controller} is installed and is {@link yfiles.input.ConcurrencyController#active}.
       * </p>
       * @type {boolean}
       */
      active:boolean;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.InputModeBase#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Returns the installed {@link yfiles.input.InputModeBase#controller}.
       * @see yfiles.input.InputModeBase#requestMutex
       * @see yfiles.input.InputModeBase#releaseMutex
       * @see yfiles.input.InputModeBase#hasMutex
       * @see yfiles.input.InputModeBase#canRequestMutex
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments used by {@link yfiles.input.ClickInputMode#addClickedListener Clicked}.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface ClickEventArgs extends yfiles.input.InputModeEventArgs{}
    export class ClickEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.ClickEventArgs} class.
       * @param context The context in which the click occurred.
       * @param location The location.
       * @param modifiers State of the modifiers.
       * @param mouseButtons The mouse button(s) that have changed
       * @param clickCount
       */
      constructor(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,modifiers:yfiles.view.ModifierKeys,mouseButtons:yfiles.view.MouseButtons,clickCount:number);
      /**
       * Gets the location of the click.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * Gets the state of the modifiers at the time of the click.
       * @type {yfiles.view.ModifierKeys}
       */
      modifiers:yfiles.view.ModifierKeys;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.ClickEventArgs} have been handled.
       * <p>
       * Client code that is triggered by the event and does not want the event to become handled by other handles should set
       * this property to <code>true</code> to stop propagation of the event.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * Gets the MouseButtons that have changed for this event.
       * @type {yfiles.view.MouseButtons}
       */
      mouseButtons:yfiles.view.MouseButtons;
      /**
       * Gets the number of clicks associated with this event.
       * <p>
       * This can be used to distinguish between single and multiple clicks. Note that for compatibility reasons, a negative
       * value means that no click count has been provided at event creation time.
       * </p>
       * @type {number}
       */
      clickCount:number;
      static $class:yfiles.lang.Class;
    }
    export enum DoubleClickPolicy{
      /**
       * In case of a double click, only the double click event is raised, but no single click events.
       * <p>
       * Single clicks not belonging to a double click will be reported with a short delay to ensure that they do not become
       * double clicks.
       * </p>
       */
      DOUBLE_CLICK_ONLY,
      /**
       * In case of a double click, one single click event is raised, followed by a double click event.
       * <p>
       * The second click of the double click is not reported as a single click. Single clicks not belonging to a double click
       * will be reported instantly.
       * </p>
       */
      INITIAL_SINGLE_AND_DOUBLE_CLICK,
      /**
       * In case of a double click, two single click events are raised, followed by a double click event.
       */
      BOTH_SINGLE_CLICKS_AND_DOUBLE_CLICK
    }
    /**
     * An input mode that recognizes simple mouse clicks.
     * <p>
     * Clients register to {@link yfiles.input.ClickInputMode#addClickedListener Clicked} to get notified of mouse
     * clicks. The {@link yfiles.input.ClickInputMode#activeButtons} property can be used to switch between the detection of right and left mouse clicks.
     * <br />
     * This mode can be instructed to {@link yfiles.input.ClickInputMode#swallowFocusClick swallow clicks} if they lead happen shortly after the control gained focus. This is useful
     * to prevent accidental clicks from being processed if the user wanted to put the focus into the control, only.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.ClickInputMode#exclusive} by default.
     * </p>
     * @see yfiles.input.ClickInputMode#doubleClickPolicy
     * @see yfiles.input.ClickInputMode#swallowFocusClick
     * @see yfiles.input.ClickInputMode#activeButtons
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface ClickInputMode extends Object,yfiles.input.IInputMode{}
    export class ClickInputMode {
      /**
       * Creates a new instance of this mode that detects left mouse clicks.
       */
      constructor();
      /**
       * Cancels the editing of this mode.
       * <p>
       * This implementation {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
       * {@link yfiles.input.ClickInputMode#onCanceled}.
       * </p>
       * @see yfiles.input.ClickInputMode#onCanceled
       */
      cancel():void;
      /**
       * Creates the event recognizer that is used to recognize the clicks for this instance.
       * @returns 
       * @protected
       */
      createClickRecognizer():(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates the event recognizer that is used to recognize the press event for this instance.
       * @returns 
       * @protected
       */
      createPressRecognizer():(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates the event recognizer that is used to recognize the release event for this instance.
       * @returns 
       * @protected
       */
      createReleaseRecognizer():(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Determines whether the given event is a press event that occurred at an invalid location.
       * @param eventSource
       * @param evt
       * @returns 
       * @see yfiles.input.ClickInputMode#validClickHitTestable
       * @protected
       */
      isInvalidPress(eventSource:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Determines whether the given event is a valid click event.
       * @param src
       * @param evt
       * @returns 
       * @see yfiles.input.ClickInputMode#validClickHitTestable
       * @protected
       */
      isValidClick(src:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Determines whether the given event is a valid press event.
       * @param eventSource
       * @param evt
       * @returns 
       * @protected
       */
      isValidPress(eventSource:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Determines whether the given event is a valid release event.
       * @param src
       * @param evt
       * @returns 
       * @see yfiles.input.ClickInputMode#validClickHitTestable
       * @protected
       */
      isValidRelease(src:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Called after {@link yfiles.input.ClickInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called once this mode has recognized a click gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.ClickInputMode#addClickedListener Clicked} event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called once this mode has recognized a double-click gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.ClickInputMode#addDoubleClickedListener DoubleClicked} event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onDoubleClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called once this mode has recognized a left click gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.ClickInputMode#addLeftClickedListener LeftClicked} event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onLeftClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called once this mode has recognized a left double-click gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.ClickInputMode#addLeftDoubleClickedListener LeftDoubleClicked}
       * event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onLeftDoubleClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called once this mode has recognized a right click gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.ClickInputMode#addRightClickedListener RightClicked} event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onRightClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called once this mode has recognized a double-click gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.ClickInputMode#addRightDoubleClickedListener RightDoubleClicked}
       * event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onRightDoubleClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called after {@link yfiles.input.ClickInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Prevents a double-click event from being issued if the next click would do so and sends only a single click instead.
       * <p>
       * The intention is for "breaking" a double click if an input mode using {@link yfiles.input.ClickInputMode} performs actions that
       * would lead to surprising behavior if the next click would lead to a double-click. Examples of this are
       * {@link yfiles.input.GraphEditorInputMode}'s selection cycling where several clicks in short succession may occur, as well as
       * creating a node by clicking on the canvas and selecting it immediately afterwards with another click.
       * </p>
       * <p>
       * This method's effect is very short-lived. It really only prevents a double-click event for the very next click that this
       * input mode handles. The internal flag set by this method is re-set on every click received. This also means that if you
       * call this method and the next click is just a regular single-click the flag is cleared nonetheless.
       * </p>
       */
      preventNextDoubleClick():void;
      /**
       * Returns and resets the flag set by {@link yfiles.input.ClickInputMode#preventNextDoubleClick}.
       * <p>
       * To ensure that the flag really acts just once, this method resets the flag and returns its value prior to the reset.
       * </p>
       * @returns 
       * @protected
       */
      queryAndResetPreventNextDoubleClick():boolean;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.ClickInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.ClickInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Gets the installed {@link yfiles.input.ClickInputMode#controller}.
       * @see yfiles.input.IInputMode#install
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>Clicked</code> event that occurs once a click has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#removeClickedListener
       */
      addClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Clicked</code> event that occurs once a click has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#addClickedListener
       */
      removeClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LeftClicked</code> event that occurs once a left click has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#removeLeftClickedListener
       */
      addLeftClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LeftClicked</code> event that occurs once a left click has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#addLeftClickedListener
       */
      removeLeftClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>RightClicked</code> event that occurs once a right click has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#removeRightClickedListener
       */
      addRightClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>RightClicked</code> event that occurs once a right click has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#addRightClickedListener
       */
      removeRightClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DoubleClicked</code> event that occurs once a double-click has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#removeDoubleClickedListener
       */
      addDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DoubleClicked</code> event that occurs once a double-click has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#addDoubleClickedListener
       */
      removeDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LeftDoubleClicked</code> event that occurs once a left double-click has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#removeLeftDoubleClickedListener
       */
      addLeftDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LeftDoubleClicked</code> event that occurs once a left double-click has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#addLeftDoubleClickedListener
       */
      removeLeftDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>RightDoubleClicked</code> event that occurs once a right double-click has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#removeRightDoubleClickedListener
       */
      addRightDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>RightDoubleClicked</code> event that occurs once a right double-click has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.ClickEventArgs
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @see yfiles.input.ClickInputMode#addRightDoubleClickedListener
       */
      removeRightDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Gets or sets the click handling mode that determines the triggering behavior of {@link yfiles.input.ClickInputMode#addClickedListener Clicked}
       * and {@link yfiles.input.ClickInputMode#addDoubleClickedListener DoubleClicked}.
       * @see yfiles.input.ClickInputMode#doubleClickPolicy
       * @default yfiles.input.DoubleClickPolicy.BOTH_SINGLE_CLICKS_AND_DOUBLE_CLICK
       * @type {yfiles.input.DoubleClickPolicy}
       */
      doubleClickPolicy:yfiles.input.DoubleClickPolicy;
      /**
       * Gets or sets a hit test that determines where this mode should recognize clicks.
       * <p>
       * This implementation is tested during the {@link yfiles.input.ClickInputMode#isValidPress} to determine whether it is valid to
       * click here. The default implementation is {@link yfiles.input.IHitTestable#ALWAYS}.
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      validClickHitTestable:yfiles.input.IHitTestable;
      /**
       * Whether to {@link yfiles.input.ConcurrencyController#requestMutex request the mutex} on a click.
       * <p>
       * If set to <code>true</code> this will discard other concurrent input modes on a click. The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      requestMutexOnClick:boolean;
      /**
       * Gets or sets the cursor to use when hovering over a valid hit region.
       * <p>
       * The default is <code>null</code> and the current cursor is used instead.
       * </p>
       * @type {yfiles.view.Cursor}
       */
      validClickHitCursor:yfiles.view.Cursor;
      /**
       * Determines for which button(s) mouse clicks should be processed.
       * <p>
       * Default value is {@link yfiles.view.MouseButtons#LEFT}
       * </p>
       * @see yfiles.input.ClickInputMode#createClickRecognizer
       * @default yfiles.view.MouseButtons.LEFT
       * @type {yfiles.view.MouseButtons}
       */
      activeButtons:yfiles.view.MouseButtons;
      /**
       * Gets or sets a property that determines whether clicks should be swallowed if they happen within a short amount of time
       * after the {@link yfiles.view.CanvasComponent} got focus.
       * <p>
       * The default is <code>false</code>. If set to <code>true</code> clicks are discarded within 100 ms after the focus entered.
       * </p>
       * @default false
       * @type {boolean}
       */
      swallowFocusClick:boolean;
      /**
       * Gets the location of the last click.
       * @type {yfiles.geometry.Point}
       */
      clickLocation:yfiles.geometry.Point;
      /**
       * Returns the state of the modifier keys at the time of the end of the gesture.
       * @type {yfiles.view.ModifierKeys}
       */
      modifiers:yfiles.view.ModifierKeys;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for the {@link yfiles.input.SnapContext#addCollectSnapResultsListener CollectSnapResults} event.
     * <p>
     * Event handlers can use the {@link yfiles.input.CollectSnapResultsEventArgs#addSnapResult} method to add snapping results for the
     * current move operation.
     * </p>
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface CollectSnapResultsEventArgs extends yfiles.input.InputModeEventArgs{}
    export class CollectSnapResultsEventArgs {
      /**
       * Creates a new instance of the event arguments using the provided values for initialization.
       * @param context The context for which the event is queried.
       * @param originalLocation The original location of the mouse at the beginning of the gesture.
       * @param newLocation The new (current) location of the mouse.
       * @param snapDistance The zoom invariant snap distance.
       * @param snapResults The collection to store the snap results in.
       */
      constructor(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point,snapDistance:number,snapResults:yfiles.collections.ICollection<yfiles.input.SnapResult>);
      /**
       * Adds a {@link yfiles.input.SnapResult} to the collection of snap results.
       * <p>
       * <code>null</code> values will be discarded.
       * </p>
       * @param snapResult The result to add.
       */
      addSnapResult(snapResult:yfiles.input.SnapResult):void;
      /**
       * Gets the original location of the mouse at the beginning of the gesture.
       * @type {yfiles.geometry.Point}
       */
      originalLocation:yfiles.geometry.Point;
      /**
       * Gets the new (current) location of the mouse that is going to be manipulated by the snapping process.
       * @type {yfiles.geometry.Point}
       */
      newLocation:yfiles.geometry.Point;
      /**
       * Gets the vector by which the mouse has been moved since initialization of the snapping process.
       * @type {yfiles.geometry.Point}
       */
      delta:yfiles.geometry.Point;
      /**
       * The zoom invariant snap distance.
       * @type {number}
       */
      snapDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A controller class that manages a single {@link yfiles.input.IInputMode} implementation.
     * <p>
     * The controller will be passed to its associated {@link yfiles.input.IInputMode} in the input mode's {@link yfiles.input.IInputMode#install}
     * method. The associated mode can {@link yfiles.input.ConcurrencyController#requestMutex request} an input mutex to {@link yfiles.input.ConcurrencyController#active deactivate} other modes added to the same {@link yfiles.input.MultiplexingInputMode}
     * and using {@link yfiles.input.ConcurrencyController#exclusive} controllers. {@link yfiles.input.ConcurrencyController#releaseMutex Releasing the mutex} will reactivate the controller of other modes.
     * </p>
     * @see yfiles.input.IInputMode
     * @see yfiles.input.MultiplexingInputMode
     * @class
     * @final
     */
    export interface ConcurrencyController extends Object{}
    export class ConcurrencyController {
      /**
       * Creates an instance that can be used to {@link yfiles.input.IInputMode#install} the given <code>mode</code>.
       * <p>
       * The returned {@link yfiles.input.ConcurrencyController} is not synchronized with any other one so the <code>mode</code> will effectively run
       * in a non-{@link yfiles.input.ConcurrencyController#exclusive exclusive} mode.
       * </p>
       * @param mode The mode, the {@link } shall be created for.
       */
      constructor(mode:yfiles.input.IInputMode);
      /**
       * Determines if the mutex can be successfully requested currently.
       * <p>
       * This will return <code>true</code>, iff this instance is {@link yfiles.input.ConcurrencyController#active} and doesn't hold the mutex.
       * </p>
       * @returns Whether a call to {@link #requestMutex} would currently succeed.
       */
      canRequestMutex():boolean;
      /**
       * Determines whether this instance owns the input mutex.
       * <p>
       * The instance owns the input mutex if it has successfully {@link yfiles.input.ConcurrencyController#requestMutex requested} the mutex in the past and not yet {@link yfiles.input.ConcurrencyController#releaseMutex released} it.
       * </p>
       * @returns Whether this instance has the input mutex.
       */
      hasMutex():boolean;
      /**
       * Triggers a {@link yfiles.input.ConcurrencyController#addPriorityChangedListener PriorityChanged} event with
       * {@link yfiles.lang.EventArgs#EMPTY}.
       * <p>
       * This method should be called by the associated mode if its {@link yfiles.input.IInputMode#priority} has changed.
       * </p>
       */
      onPriorityChanged():void;
      /**
       * Releases the input mutex.
       * <p>
       * Modes should call this method when they have finished editing the contents exclusively. This will reactivate those modes
       * whose controller that had been {@link yfiles.input.ConcurrencyController#active deactivated} at the time the mutex has been requested.
       * </p>
       */
      releaseMutex():void;
      /**
       * Can be called to request an input mutex for the input mode.
       * <p>
       * Modes should call this method to request exclusive editing for a canvas control. {@link yfiles.input.ConcurrencyController#canRequestMutex}
       * should be called before requesting the mutex. If the mutex cannot be obtained, this method will trigger an exception.
       * </p>
       * @throws {yfiles.lang.Exception} If the mutex could not be obtained.
       */
      requestMutex():void;
      /**
       * Adds the given listener for the <code>MutexObtained</code> event that occurs when the mutex has been successfully obtained.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ConcurrencyController#removeMutexObtainedListener
       */
      addMutexObtainedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MutexObtained</code> event that occurs when the mutex has been successfully obtained.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ConcurrencyController#addMutexObtainedListener
       */
      removeMutexObtainedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>MutexReleased</code> event that occurs when the mutex has been released.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ConcurrencyController#removeMutexReleasedListener
       */
      addMutexReleasedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>MutexReleased</code> event that occurs when the mutex has been released.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ConcurrencyController#addMutexReleasedListener
       */
      removeMutexReleasedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets the current owner of the input mutex or <code>null</code> if no mode currently owns the mutex.
       * @type {yfiles.input.IInputMode}
       */
      mutexOwner:yfiles.input.IInputMode;
      /**
       * Gets or sets if this controller is enabled.
       * <p>
       * The default state is disabled. A disabled controller is always {@link yfiles.input.ConcurrencyController#active inactive}.
       * </p>
       * <p>
       * When the controller is {@link yfiles.input.IInputMode#install installed} and the input mode is ready for editing, it has to set this property to <code>true</code> if it
       * wants to use the input mutex.
       * </p>
       * @see yfiles.input.ConcurrencyController#active
       * @see yfiles.input.ConcurrencyController#addActiveChangedListener
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * Returns if this controller is active.
       * <p>
       * An {@link yfiles.input.IInputMode} should only handle events and gestures if this property on its associated {@link yfiles.input.ConcurrencyController}
       * is set to <code>true</code>.
       * </p>
       * <p>
       * If this controller is set as {@link yfiles.input.ConcurrencyController#exclusive non-exclusive} by its input mode it is active iff it is {@link yfiles.input.ConcurrencyController#enabled}. Otherwise it has to be {@link yfiles.input.ConcurrencyController#enabled} and no
       * other input mode may have the {@link yfiles.input.ConcurrencyController#requestMutex input mutex}.
       * </p>
       * @see yfiles.input.ConcurrencyController#addActiveChangedListener
       * @type {boolean}
       */
      active:boolean;
      /**
       * Adds the given listener for the <code>ActiveChanged</code> event that occurs the {@link yfiles.input.ConcurrencyController#active} property changes.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ConcurrencyController#removeActiveChangedListener
       */
      addActiveChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ActiveChanged</code> event that occurs the {@link yfiles.input.ConcurrencyController#active} property changes.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ConcurrencyController#addActiveChangedListener
       */
      removeActiveChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Gets a value indicating whether this controller will be the only one running when it has the mutex.
       * <p>
       * If this controller is marked as exclusive and has the mutex, all controllers of other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be {@link yfiles.input.ConcurrencyController#active deactivated}. Otherwise it will always run concurrently with all other controllers.
       * </p>
       * <p>
       * Changing this property will temporarily {@link yfiles.input.ConcurrencyController#enabled disable} the controller.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Returns the cursor the associated mode of this controller would like to have displayed in the
       * {@link yfiles.view.CanvasComponent}.
       * <p>
       * To indicate that the mode does not have any preferences the property should be set to <code>null</code>.
       * </p>
       * @type {yfiles.view.Cursor}
       */
      preferredCursor:yfiles.view.Cursor;
      /**
       * Adds the given listener for the <code>PreferredCursorChanged</code> event that occurs when the {@link yfiles.input.ConcurrencyController#preferredCursor} property changes.
       * <p>
       * {@link yfiles.input.MultiplexingInputMode} listens to property changes of the controller of its sub modes. properties. This
       * event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ConcurrencyController#removePreferredCursorChangedListener
       */
      addPreferredCursorChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PreferredCursorChanged</code> event that occurs when the {@link yfiles.input.ConcurrencyController#preferredCursor} property changes.
       * <p>
       * {@link yfiles.input.MultiplexingInputMode} listens to property changes of the controller of its sub modes. properties. This
       * event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ConcurrencyController#addPreferredCursorChangedListener
       */
      removePreferredCursorChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PriorityChanged</code> event that occurs when {@link yfiles.input.ConcurrencyController#onPriorityChanged}
       * is called by the associated mode.
       * <p>
       * {@link yfiles.input.MultiplexingInputMode} listens to priority changes of the controller of its sub modes.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ConcurrencyController#removePriorityChangedListener
       */
      addPriorityChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PriorityChanged</code> event that occurs when {@link yfiles.input.ConcurrencyController#onPriorityChanged}
       * is called by the associated mode.
       * <p>
       * {@link yfiles.input.MultiplexingInputMode} listens to priority changes of the controller of its sub modes.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ConcurrencyController#addPriorityChangedListener
       */
      removePriorityChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for the {@link yfiles.input.ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
     * event.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface PopulateMenuEventArgs extends yfiles.input.InputModeEventArgs{}
    export class PopulateMenuEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.PopulateMenuEventArgs} class.
       * @param context The context.
       * @param queryLocation The query location.
       */
      constructor(context:yfiles.input.IInputModeContext,queryLocation:yfiles.geometry.Point);
      /**
       * Gets the queried location in world coordinates.
       * @type {yfiles.geometry.Point}
       */
      queryLocation:yfiles.geometry.Point;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.PopulateMenuEventArgs} has been handled.
       * <p>
       * If this event is marked as handled, the input mode will use the property {@link yfiles.input.PopulateMenuEventArgs#showMenu} to decide whether to show the context menu.
       * </p>
       * <p>
       * This property is automatically set to <code>true</code> if a value is assigned to the property {@link yfiles.input.PopulateMenuEventArgs#showMenu}.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * Gets or sets a value indicating whether to actually display the context menu.
       * <p>
       * This will set the {@link yfiles.input.PopulateMenuEventArgs#handled} property to <code>true</code>.
       * </p>
       * <p>
       * The default value of this property is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      showMenu:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.input.IInputMode} interface that handles the display of a custom context menu when the
     * user right clicks on the {@link yfiles.view.CanvasComponent}.
     * <p>
     * This mode can be used with any context menu implementation since it does not impose any specific requirements.
     * Typically, it is used in the following way.
     * </p>
     * <p>
     * First, this mode must be informed that the user wants to show the context menu with the method
     * {@link yfiles.input.ContextMenuInputMode#shouldOpenMenu}, for example because a <code>contextmenu</code> event was fired.
     * </p>
     * <p>
     * Then, in response, this mode checks whether its current state allows opening a context menu, and fires the {@link yfiles.input.ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
     * and <code>GraphInputMode.populateItemContextMenu</code>.
     * </p>
     * <p>
     * If the context menu was closed because of user interaction with it, for example the user selected an menu entry, this
     * mode must be informed by calling the method {@link yfiles.input.ContextMenuInputMode#menuClosed}.
     * </p>
     * <p>
     * This mode detects automatically if the menu must be closed because of other user interaction, for example the user
     * clicked somewhere else, and fires the event {@link yfiles.input.ContextMenuInputMode#addCloseMenuListener CloseMenu}
     * to request closing of the context menu.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Context Menu, demo.yfiles.input.contextmenu</li>
     * </ul>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface ContextMenuInputMode extends Object,yfiles.input.IInputMode{}
    export class ContextMenuInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.ContextMenuInputMode} class.
       */
      constructor();
      /**
       * Cancels the display of the context menu and requests that the context menu is closed.
       */
      cancel():void;
      /**
       * Creates an {@link yfiles.input.IInputModeContext} for use with the {@link yfiles.input.ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
       * call in the upcoming query.
       * @returns An instance of {@link }.
       * @protected
       */
      createPopulateMenuContext():yfiles.input.IInputModeContext;
      /**
       * Installs this mode in the canvas.
       * @param context The context to install this mode into.
       * @param controller The {@link #controller} for this mode.
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * This method must be called by custom code to indicate that the menu has been closed.
       * <p>
       * Since this input mode cannot know when the user has closed a context menu, this callback should be used to allow the
       * instance to adjust the internal state.
       * </p>
       * @see yfiles.input.ContextMenuInputMode#addCloseMenuListener
       */
      menuClosed():void;
      /**
       * Called after {@link yfiles.input.ContextMenuInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called when the menu is closed and triggers the {@link yfiles.input.ContextMenuInputMode#addCloseMenuListener CloseMenu}
       * event.
       * @protected
       */
      onCloseMenu():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * This will populate the context menu for the given world coordinate.
       * <p>
       * This implementation will trigger the {@link yfiles.input.ContextMenuInputMode#addPopulateMenuListener PopulateMenu}
       * event and will use the {@link yfiles.input.PopulateMenuEventArgs#showMenu} value as the return value. If the event will not be handled, this method will return <code>true</code> if the
       * menu already contains elements.
       * </p>
       * @param location The location in the world coordinate system for which the context menu has been invoked.
       * @returns Whether to show the context menu.
       * @protected
       */
      onMenuOpening(location:yfiles.geometry.Point):boolean;
      /**
       * Raises the {@link yfiles.input.ContextMenuInputMode#addPopulateMenuListener PopulateMenu} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPopulateMenu(evt:yfiles.input.PopulateMenuEventArgs):void;
      /**
       * Called after {@link yfiles.input.ContextMenuInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * This method must be called by custom code to inform this input mode that a context menu is about to be opened by a user
       * gesture.
       * <p>
       * Since this input mode does not know when the user wants to open a context menu, this method must be called to possibly
       * cancel the request and, otherwise, update the internal state of this mode.
       * </p>
       * @param [queryLocation=null] The optional location for which the context menu content should be queried. This value will be passed to {@link #onPopulateMenu}
       * and will ultimately be available in {@link #queryLocation}. If not specified, the last known mouse location will be used.
       * @returns 
       * @see yfiles.input.ContextMenuInputMode#menuClosed
       * @see yfiles.input.ContextMenuInputMode#addCloseMenuListener
       */
      shouldOpenMenu(queryLocation?:yfiles.geometry.Point):boolean;
      /**
       * Stops the display of the context menu and requests that the context menu is closed.
       * @returns 
       */
      tryStop():boolean;
      /**
       * Removes the menu from the context and replaces it with the old instance.
       * @param context The context to uninstall this mode from.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.ContextMenuInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.ContextMenuInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Gets the installed {@link yfiles.input.ConcurrencyController}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>PopulateMenu</code> event that occurs when the context menu is about to be shown.
       * <p>
       * Handlers of the event can populate their context menu, set the {@link yfiles.input.PopulateMenuEventArgs#showMenu} and set or respect the {@link yfiles.input.PopulateMenuEventArgs#handled} property. Note that all handlers
       * will be called, even if one of them sets the {@link yfiles.input.PopulateMenuEventArgs#handled} property to
       * <code>true</code>. Every handler should query this property and decide carefully what to do.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ContextMenuInputMode#removePopulateMenuListener
       */
      addPopulateMenuListener(listener:(sender:Object,evt:yfiles.input.PopulateMenuEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PopulateMenu</code> event that occurs when the context menu is about to be shown.
       * <p>
       * Handlers of the event can populate their context menu, set the {@link yfiles.input.PopulateMenuEventArgs#showMenu} and set or respect the {@link yfiles.input.PopulateMenuEventArgs#handled} property. Note that all handlers
       * will be called, even if one of them sets the {@link yfiles.input.PopulateMenuEventArgs#handled} property to
       * <code>true</code>. Every handler should query this property and decide carefully what to do.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ContextMenuInputMode#addPopulateMenuListener
       */
      removePopulateMenuListener(listener:(sender:Object,evt:yfiles.input.PopulateMenuEventArgs)=>void):void;
      /**
       * Gets or sets an {@link yfiles.input.IHitTestable} that determines whether it is valid to open a context menu at the queried
       * position.
       * <p>
       * If the instance yields <code>false</code> {@link yfiles.input.ContextMenuInputMode#onMenuOpening} will not be queried and the context menu
       * will not be shown. By default there is a hit testable instance that always yields <code>true</code>.
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      validMenuLocationHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets or sets a value that determines whether clicks are swallowed if they happen within a short amount of time after a
       * context menu was closed.
       * <p>
       * This is especially useful to cancel the menu by clicking somewhere outside the menu without creating a node. The default
       * is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      swallowCloseClick:boolean;
      /**
       * Adds the given listener for the <code>CloseMenu</code> event that occurs when this instance requests closing an open context menu.
       * <p>
       * Custom context menus must be closed in response to this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ContextMenuInputMode#menuClosed
       * @see yfiles.input.ContextMenuInputMode#removeCloseMenuListener
       */
      addCloseMenuListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CloseMenu</code> event that occurs when this instance requests closing an open context
       * menu.
       * <p>
       * Custom context menus must be closed in response to this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ContextMenuInputMode#menuClosed
       * @see yfiles.input.ContextMenuInputMode#addCloseMenuListener
       */
      removeCloseMenuListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is an input mode handling drag and drop related events on a {@link yfiles.view.CanvasComponent}.
     * <p>
     * An instance of <code>DropInputMode</code> is initialized for an expected data format in the constructor. There are four events raised
     * by an instance of this class.
     * </p>
     * <ul>
     * <li>
     * {@link yfiles.input.DropInputMode#addDragEnteredListener DragEntered} is raised, if a drag operation enters the
     * {@link yfiles.view.CanvasComponent}.
     * </li>
     * <li>
     * {@link yfiles.input.DropInputMode#addDragOverListener DragOver} is raised, if a drag operation moves over the
     * {@link yfiles.view.CanvasComponent} after having entered it.
     * </li>
     * <li>
     * {@link yfiles.input.DropInputMode#addDragDroppedListener DragDropped} is raised, if a drag is dropped onto the
     * {@link yfiles.view.CanvasComponent}.
     * </li>
     * <li>
     * {@link yfiles.input.DropInputMode#addDragLeftListener DragLeft} is raised, if a drag is operation leaves the
     * {@link yfiles.view.CanvasComponent}.
     * </li>
     * </ul>
     * <p>
     * Clients can query the current {@link yfiles.input.DropInputMode#mousePosition} which is updated while a drag is in progress. Clients get the data from the drag
     * operation by calling {@link yfiles.input.DropInputMode#dropData} from an event handler registered for any of these events.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.DropInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface DropInputMode extends Object,yfiles.input.IInputMode{}
    export class DropInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.DropInputMode} class for the expected data format.
       * <p>
       * This instance fires events only if a drag operation carries data of the expected format.
       * </p>
       * @param expectedFormat This is a format that is used in {@link #getData}.
       * @see yfiles.input.DropInputMode#addDragEnteredListener
       * @see yfiles.input.DropInputMode#addDragDroppedListener
       */
      constructor(expectedFormat:string);
      /**
       * Callback that adjusts the effect accordingly.
       * @param evt
       * @returns 
       * @protected
       */
      adjustEffect(evt:yfiles.view.DragEventArgs):boolean;
      /**
       * Cleanly cancels this mode.
       */
      cancel():void;
      /**
       * Helper method that yields a suitably configured {@link yfiles.input.InputModeEventArgs} for this input mode.
       * @returns An input mode event argument that is configured for this instance.
       * @protected
       */
      createInputModeEventArgs():yfiles.input.InputModeEventArgs;
      /**
       * Installs this mode into the given canvas.
       * <p>
       * Clients should not call this method as this is handled by the framework.
       * </p>
       * @param context the context to install this mode into
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.view.CanvasComponent
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.DropInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called once a drag has been dropped on the canvas.
       * @param evt
       * @protected
       */
      onDragDropped(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called once a drag has entered the canvas.
       * @param evt
       * @protected
       */
      onDragEntered(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called whenever a drag is over the canvas.
       * @param evt
       * @protected
       */
      onDraggedOver(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called once a drag has left the canvas.
       * @param evt
       * @protected
       */
      onDragLeft(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called after {@link yfiles.input.DropInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * This method updates the {@link yfiles.input.DropInputMode#mousePosition} according to the coordinates passed in.
       * <p>
       * It is called prior to the {@link yfiles.input.DropInputMode#onDragEntered}, {@link yfiles.input.DropInputMode#onDraggedOver}, {@link yfiles.input.DropInputMode#onDragDropped}
       * and {@link yfiles.input.DropInputMode#onDragLeft} methods.
       * </p>
       * @param location The current location of the mouse in world coordinates.
       * @protected
       */
      setDragLocation(location:yfiles.geometry.Point):void;
      /**
       * Overridden to only return <code>true</code> if this instance does not currently {@link yfiles.input.ConcurrencyController#hasMutex have the input mutex}.
       * @returns <code>true</code> iff this instance does not {@link #hasMutex own the mutex}.
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the canvas.
       * <p>
       * Clients should not call this method as this is handled by the framework.
       * </p>
       * @param context the context
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.DropInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.DropInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.DropInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>DragEntered</code> event that occurs when a drag operation enters the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.DropInputMode#removeDragEnteredListener
       */
      addDragEnteredListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragEntered</code> event that occurs when a drag operation enters the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.DropInputMode#addDragEnteredListener
       */
      removeDragEnteredListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragOver</code> event that occurs when a drag operation drags over the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.DropInputMode#removeDragOverListener
       */
      addDragOverListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragOver</code> event that occurs when a drag operation drags over the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.DropInputMode#addDragOverListener
       */
      removeDragOverListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragLeft</code> event that occurs when a drag operation leaves the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.DropInputMode#removeDragLeftListener
       */
      addDragLeftListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragLeft</code> event that occurs when a drag operation leaves the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.DropInputMode#addDragLeftListener
       */
      removeDragLeftListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragDropped</code> event that occurs when a drag operation is dropped onto the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.DropInputMode#removeDragDroppedListener
       */
      addDragDroppedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragDropped</code> event that occurs when a drag operation is dropped onto the {@link yfiles.view.CanvasComponent}
       * into which this instance is installed.
       * <p>
       * No event is raised
       * </p>
       * <ul>
       * <li>if this instance is not {@link yfiles.input.DropInputMode#enabled},</li>
       * <li>if another concurrent input mode is active or</li>
       * <li>the data from the drag operation does not conform to the expected format.</li>
       * </ul>
       * <p>
       * To get additional data for the event, clients should use the sender which is set to the sending
       * {@link yfiles.input.DropInputMode}. The {@link yfiles.lang.EventArgs} provide no useful data.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.DropInputMode#addDragDroppedListener
       */
      removeDragDroppedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Gets or sets the {@link yfiles.view.DragDropEffects} that will be used for drags on the canvas.
       * @type {yfiles.view.DragDropEffects}
       */
      dragDropEffect:yfiles.view.DragDropEffects;
      /**
       * Gets or sets the {@link yfiles.input.IHitTestable} that determines whether the given location is a valid drop location.
       * @type {yfiles.input.IHitTestable}
       */
      validDropHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets the last processed drag event argument.
       * @type {yfiles.view.DragEventArgs}
       */
      lastDragEventArgs:yfiles.view.DragEventArgs;
      /**
       * Gets the current mouse position during drag operations.
       * <p>
       * The position is returned in world coordinates according to the {@link yfiles.view.CanvasComponent} into which this input mode is
       * installed.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      mousePosition:yfiles.geometry.IPoint;
      /**
       * Gets the mouse position after dropping an item.
       * <p>
       * The position is returned in world coordinates according to the {@link yfiles.view.CanvasComponent} into which this input mode is
       * installed.
       * </p>
       * @type {yfiles.geometry.Point}
       */
      dropLocation:yfiles.geometry.Point;
      /**
       * Gets the data carried by a drag operation.
       * <p>
       * This is only valid, if a drag entered the {@link yfiles.view.CanvasComponent} first.
       * </p>
       * @type {Object}
       */
      dropData:Object;
      /**
       * Gets or sets a value indicating whether to capture mouse input during drag operations.
       * <p>
       * The default is <code>true</code> and thus prevents mouse events from being processed by visuals in the canvas.
       * </p>
       * @default true
       * @type {boolean}
       */
      captureMouseInputDuringDrag:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} that can be added to a {@link yfiles.input.MultiplexingInputMode} to suppress other modes from doing
     * unwanted things if the control has just become focused.
     * <p>
     * This mode requests the input mutex once the canvas gains focus. It returns the mutex after the mode has regained focus
     * and the mouse has been clicked or 100 milliseconds have passed. This mode should be registered with the {@link yfiles.input.MultiplexingInputMode}
     * using the {@link yfiles.input.MultiplexingInputMode#add} method and a large priority number (e.g.
     * <code>1000</code>). So that other modes don't receive the input mutex once this one has released the mutex.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.ConcurrencyController#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface FocusGuardInputMode extends Object,yfiles.input.IInputMode{}
    export class FocusGuardInputMode {
      /**
       * Creates a new instance of this mode.
       */
      constructor();
      /**
       * 
       */
      cancel():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.FocusGuardInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called after {@link yfiles.input.FocusGuardInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.FocusGuardInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.FocusGuardInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets or sets the {@link yfiles.view.Cursor} that will be shown during active guards.
       * <p>
       * The default is {@link yfiles.view.Cursor#DEFAULT}
       * </p>
       * @type {yfiles.view.Cursor}
       */
      guardCursor:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} implementation that can handle a collection of {@link yfiles.input.IHandle}s.
     * <p>
     * This mode will render a visual representation of {@link yfiles.input.IHandle}s in the {@link yfiles.view.CanvasComponent} and deal with
     * mouse gestures that drag the visual representations accordingly.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.HandleInputMode#exclusive} by default.
     * </p>
     * @see yfiles.input.IHandle
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface HandleInputMode extends Object,yfiles.input.IInputMode{}
    export class HandleInputMode {
      /**
       * Creates a new mode with no handles initially.
       * @see yfiles.input.HandleInputMode#handles
       */
      constructor();
      /**
       * Adds a handle to the collection of handles managed by this instance.
       * @param handle The handle to add.
       */
      addHandle(handle:yfiles.input.IHandle):void;
      /**
       * Factory method that creates and {@link yfiles.input.HandleInputMode#addHandle adds} a handle to this mode by wrapping a given {@link yfiles.geometry.IMutablePoint} instance.
       * <p>
       * The <code>cursor</code> and <code>type</code> are used to initialize the {@link yfiles.input.IHandle} instance.
       * </p>
       * @param location The point to use as a handle.
       * @param [cursor=null] The cursor to use.
       * @param [type=yfiles.input.HandleTypes.DEFAULT] The type of the handle to create.
       * @returns The handle created.
       */
      addHandle(location:yfiles.geometry.IMutablePoint,cursor?:yfiles.view.Cursor,type?:yfiles.input.HandleTypes):yfiles.input.IHandle;
      /**
       * Factory method that creates and {@link yfiles.input.HandleInputMode#addHandle adds} a handle to this mode by wrapping a given {@link yfiles.geometry.IMutablePoint} instance.
       * <p>
       * The <code>cursor</code> and <code>type</code> are used to initialize the {@link yfiles.input.IHandle} instance.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.location The point to use as a handle.
       * @param [options.cursor=null] The cursor to use.
       * @param [options.type=yfiles.input.HandleTypes.DEFAULT] The type of the handle to create.
       * @returns The handle created.
       */
      addHandle(options:{location:yfiles.geometry.IMutablePoint,cursor?:yfiles.view.Cursor,type?:yfiles.input.HandleTypes}):yfiles.input.IHandle;
      /**
       * Callback method that is invoked when this mode gets "armed".
       * <p>
       * This happens when the mouse pointer hovers over a handle. This changes the mouse cursor to the <code>handle</code>'s {@link yfiles.input.IHandle#cursor}.
       * </p>
       * @param handle
       * @protected
       */
      arm(handle:yfiles.input.IHandle):void;
      /**
       * Method that can be used to initialize the dragging of a given handle by hand.
       * <p>
       * This will simulate the user having initialized a drag of the given handle.
       * </p>
       * @param handle The handle to be dragged.
       */
      beginDragging(handle:yfiles.input.IHandle):void;
      /**
       * 
       */
      cancel():void;
      /**
       * Creates an {@link yfiles.input.IInputModeContext} for use with the {@link yfiles.input.IHandle} interface for the upcoming drag
       * operation.
       * @returns An instance of {@link }.
       * @protected
       */
      createHandleInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Helper method that yields a suitably configured {@link yfiles.input.InputModeEventArgs} for this input mode.
       * @returns An input mode event argument that is configured for this instance.
       * @protected
       */
      createInputModeEventArgs():yfiles.input.InputModeEventArgs;
      /**
       * Callback that creates the {@link yfiles.view.Visual} for the given handle.
       * @param handle The handle to create a visual representation for.
       * @param context
       * @returns The {@link }.
       * @protected
       */
      createVisual(context:yfiles.view.IRenderContext,handle:yfiles.input.IHandle):yfiles.view.SvgVisual;
      /**
       * Callback method that is invoked when this mode gets "disarmed".
       * <p>
       * This happens when the mouse stops hovering over a handle. This resets the mouse cursor
       * </p>
       * @protected
       */
      disarm():void;
      /**
       * Finds the closest hit handle for the given world coordinate pair.
       * <p>
       * The default implementation compares the manhatten distance of the handles to find the closest handle.
       * </p>
       * @param location The coordinates in the world coordinate system.
       * @returns The closest handle that has been hit or <code>null</code>
       * @protected
       */
      getClosestHitHandle(location:yfiles.geometry.Point):yfiles.input.IHandle;
      /**
       * Finds the closest hit handle for the given world coordinate pair.
       * @param location The coordinates in the world coordinate system.
       * @returns The closest handle that has been hit or <code>null</code>
       * @protected
       */
      getClosestHitHandleTouch(location:yfiles.geometry.Point):yfiles.input.IHandle;
      /**
       * Determines whether a given handle has been visually hit by the mouse at the given world coordinates.
       * <p>
       * This implementation uses the manhatten distance as a metric to determine if the handle was hit.
       * </p>
       * @param handle The handle to check
       * @param location The view coordinates to check.
       * @param distance The distance of the handle to the location. In the default implementation, this is a tuple representing the x- and
       * y-distance of the handle to the location.
       * @returns Whether the handle has been hit.
       * @protected
       */
      handleIsHit(handle:yfiles.input.IHandle,location:yfiles.geometry.Point,distance:yfiles.geometry.Point):boolean;
      /**
       * Determines whether a given handle has been visually hit by the touch device at the given world coordinates.
       * @param handle The handle to check
       * @param location The world coordinates to check.
       * @param distance The distance of the handle to the touch location. In the default implementation, this is a tuple representing the x- and
       * y-distance of the handle to the touch location.
       * @returns Whether the handle has been hit.
       * @protected
       */
      handleIsHitTouch(handle:yfiles.input.IHandle,location:yfiles.geometry.Point,distance:yfiles.geometry.Point):boolean;
      /**
       * Installs the visual representation of the handles into the canvas into the {@link yfiles.view.CanvasComponent#inputModeGroup}.
       * @param context
       * @param controller
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Convenience method that invalidates the canvas this mode is currently installed in.
       * @protected
       */
      invalidate():void;
      /**
       * Event recognizer callback used to determine whether the mouse is currently hovering over a handle.
       * @param sender
       * @param evt
       * @returns Whether the mouse is hovering over handle.
       * @see yfiles.input.HandleInputMode#getClosestHitHandle
       * @protected
       */
      isHovering(sender:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Called after {@link yfiles.input.HandleInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Triggers the {@link yfiles.input.HandleInputMode#addDragCanceledListener DragCanceled} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.HandleInputMode#addDragCancelingListener DragCanceling} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceling(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered once the drag has been finalized.
       * <p>
       * This method triggers the {@link yfiles.input.HandleInputMode#addDragFinishedListener DragFinished} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinished(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered before the drag will be finalized.
       * <p>
       * This method triggers the {@link yfiles.input.HandleInputMode#addDragFinishingListener DragFinishing} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinishing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.HandleInputMode#addDraggedListener Dragged} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragged(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.HandleInputMode#addDraggingListener Dragging} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragging(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.HandleInputMode#addDragStartedListener DragStarted} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarted(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.HandleInputMode#addDragStartingListener DragStarting} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarting(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called after {@link yfiles.input.HandleInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Removes a handle from the collection of handles managed by this instance.
       * @param handle The handle to remove.
       */
      removeHandle(handle:yfiles.input.IHandle):void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Removes the {@link yfiles.view.ICanvasObject} that displays the handles from the canvas.
       * @param context
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.HandleInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.HandleInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * <p>
       * Disabling this mode will hide the handles and prevent the user from interacting with them.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.HandleInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#DEFAULT}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_DEFAULT_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#MOVE}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_MOVE_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#RESIZE}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_RESIZE_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#ROTATE}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_ROTATE_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#SHEAR}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_SHEAR_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#INVISIBLE}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_INVISIBLE_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#INVISIBLE} |
       * {@link yfiles.input.HandleTypes#VARIANT2}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_INVISIBLE_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#INVISIBLE} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_INVISIBLE_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#INVISIBLE} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_INVISIBLE_VARIANT4_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} H{@link yfiles.input.HandleTypes#DEFAULT} |
       * {@link yfiles.input.HandleTypes#VARIANT2}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_DEFAULT_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#MOVE} |
       * {@link yfiles.input.HandleTypes#VARIANT2}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_MOVE_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#RESIZE} |
       * {@link yfiles.input.HandleTypes#VARIANT2}
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_RESIZE_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#ROTATE} |
       * {@link yfiles.input.HandleTypes#VARIANT2}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_ROTATE_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#SHEAR} |
       * {@link yfiles.input.HandleTypes#VARIANT2}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_SHEAR_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#DEFAULT} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_DEFAULT_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#MOVE} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_MOVE_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#RESIZE} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_RESIZE_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#ROTATE} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_ROTATE_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#SHEAR} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_SHEAR_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#DEFAULT} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_DEFAULT_VARIANT4_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#MOVE} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_MOVE_VARIANT4_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#RESIZE} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_RESIZE_VARIANT4_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#ROTATE} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_ROTATE_VARIANT4_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#SHEAR} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_SHEAR_VARIANT4_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#WARP}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_WARP_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#WARP} |
       * {@link yfiles.input.HandleTypes#VARIANT2}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_WARP_VARIANT2_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#WARP} |
       * {@link yfiles.input.HandleTypes#VARIANT3}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_WARP_VARIANT3_KEY:string;
      /**
       * A {@link string} that can be used by instances of the {@link yfiles.input.HandleInputMode} to look up a {@link yfiles.view.IVisualTemplate} that will
       * be used for the rendering of {@link yfiles.input.IHandle}s of {@link yfiles.input.HandleTypes type} {@link yfiles.input.HandleTypes#WARP} |
       * {@link yfiles.input.HandleTypes#VARIANT4}.
       * <p>
       * The drawing returned should be centered around <code>(0,0)</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static HANDLE_DRAWING_WARP_VARIANT4_KEY:string;
      /**
       * Gets or sets event recognizer that recognizes when the user temporarily disables snapping.
       * @see yfiles.input.HandleInputMode#enableSnappingRecognizer
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      disableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets event recognizer that recognizes when the user reenables temporarily disabled snapping.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      enableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" state recognizer.
       * <p>
       * This recognizer instance will be used to determine when the user begins to move a handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer.
       * <p>
       * This recognizer instance determines when the user is moving a handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "cancel" recognizer.
       * <p>
       * This recognizer recognizes the cancel action during the move.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer.
       * <p>
       * This instance determines when the user has finished moving the handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" state recognizer for touch input.
       * <p>
       * This recognizer instance will be used to determine when the user begins to move a handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer for touch input.
       * <p>
       * This recognizer instance determines when the user is moving a handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer for touch input.
       * <p>
       * This instance determines when the user has finished moving the handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the collection of {@link yfiles.input.IHandle}s this mode manages.
       * @type {yfiles.collections.IObservableCollection.<yfiles.input.IHandle>}
       */
      handles:yfiles.collections.IObservableCollection<yfiles.input.IHandle>;
      /**
       * The {@link yfiles.input.HandleInputMode#snapContext} which manages snapping model items to certain coordinates (for instance, other items).
       * <p>
       * If set to <code>null</code> (the default) this input mode tries to obtain the {@link yfiles.input.HandleInputMode#snapContext} from the {@link yfiles.input.IInputModeContext}. To explicitly
       * disable snapping, a {@link yfiles.input.HandleInputMode#snapContext} implementation that does nothing has to be set to this instance.
       * </p>
       * @type {yfiles.input.SnapContext}
       */
      snapContext:yfiles.input.SnapContext;
      /**
       * Gets the handle that is currently being moved or <code>null</code> otherwise.
       * @type {yfiles.input.IHandle}
       */
      currentHandle:yfiles.input.IHandle;
      /**
       * Gets an immutable snapshot of the {@link yfiles.graph.IModelItem}s affected by the currently modified {@link yfiles.input.HandleInputMode#currentHandle handle} gesture.
       * <p>
       * When the gesture is {@link yfiles.input.HandleInputMode#addDragStartingListener starting} and the {@link yfiles.input.HandleInputMode#currentHandle} is {@link yfiles.input.IDragHandler#initializeDrag initialized}, the implementation can {@link yfiles.input.IModelItemCollector#add register} the affected item(s) through the {@link yfiles.input.IModelItemCollector}
       * instance that is bound to the {@link yfiles.input.HandleInputMode#createHandleInputModeContext context} available via its {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * <p>
       * Client code can register with the {@link yfiles.input.HandleInputMode#addDragStartedListener DragStarted}
       * event, as well as the {@link yfiles.input.HandleInputMode#addDragFinishedListener DragFinished} event to get
       * notified of the elements that may be or have been affected respectively by this input mode.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IModelItem>}
       */
      affectedItems:yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDragFinishingListener
       */
      addDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDragFinishingListener
       */
      removeDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDragFinishedListener
       */
      addDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDragFinishedListener
       */
      removeDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDragStartingListener
       */
      addDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDragStartingListener
       */
      removeDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDragStartedListener
       */
      addDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDragStartedListener
       */
      removeDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDraggingListener
       */
      addDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDraggingListener
       */
      removeDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDraggedListener
       */
      addDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDraggedListener
       */
      removeDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDragCanceledListener
       */
      addDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDragCanceledListener
       */
      removeDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to add.
       * @see yfiles.input.HandleInputMode#removeDragCancelingListener
       */
      addDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.HandleInputMode#addDragCancelingListener
       */
      removeDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for an object that can be used to drag something displayed in a {@link yfiles.view.CanvasComponent}
     * <p>
     * Items can be dragged with the mouse and an instance of this class will modify their position accordingly. A drag will
     * trigger the invocation of {@link yfiles.input.IDragHandler#initializeDrag}, zero or more {@link yfiles.input.IDragHandler#handleMove}
     * calls, and will be finalized by either {@link yfiles.input.IDragHandler#dragFinished} or {@link yfiles.input.IDragHandler#cancelDrag}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: PositionHandler, demo.yfiles.input.positionhandler</li>
     * </ul>
     * @see yfiles.input.IPositionHandler
     * @see yfiles.input.IHandle
     * @see yfiles.input.IInputModeContext
     * @interface
     */
    export interface IDragHandler extends Object{
      /**
       * Called by clients to indicate that the dragging has been canceled by the user.
       * <p>
       * This method may be called after the initial {@link yfiles.input.IDragHandler#initializeDrag} and zero or more invocations of
       * {@link yfiles.input.IDragHandler#handleMove}. Implementations should reset the position of the items they modify to their
       * initial state. Alternatively to this method the {@link yfiles.input.IDragHandler#dragFinished} method might be called.
       * </p>
       * @param context The context to retrieve information about the drag from.
       * @param originalLocation The value of the coordinate of the {@link #location} property at the time of {@link #initializeDrag}.
       * @abstract
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Called by clients to indicate that the repositioning has just been finished.
       * <p>
       * This method may be called after the initial {@link yfiles.input.IDragHandler#initializeDrag} and zero or more invocations of
       * {@link yfiles.input.IDragHandler#handleMove}. Alternatively to this method the {@link yfiles.input.IDragHandler#cancelDrag} method might
       * be called.
       * </p>
       * @param context The context to retrieve information about the drag from.
       * @param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #location} may or may not be modified to reflect the new value. This is the same value as delivered in the last
       * invocation of {@link #handleMove}
       * @param originalLocation The value of the {@link #location} property at the time of {@link #initializeDrag}.
       * @abstract
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Called by clients to indicate that the element has been dragged and its position should be updated.
       * <p>
       * This method may be called more than once after an initial {@link yfiles.input.IDragHandler#initializeDrag} and will the final
       * call will be followed by either one {@link yfiles.input.IDragHandler#dragFinished} or one {@link yfiles.input.IDragHandler#cancelDrag}
       * call.
       * </p>
       * @param context The context to retrieve information about the drag from.
       * @param originalLocation The value of the {@link #location} property at the time of {@link #initializeDrag}.
       * @param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #location} may or may not be modified to reflect the new value.
       * @abstract
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Called by clients to indicate that the element is going to be dragged.
       * <p>
       * This call will be followed by one or more calls to {@link yfiles.input.IDragHandler#handleMove}, and a final {@link yfiles.input.IDragHandler#dragFinished}
       * or {@link yfiles.input.IDragHandler#cancelDrag}.
       * </p>
       * @param context The context to retrieve information about the drag from.
       * @abstract
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Returns a view of the location of the item.
       * <p>
       * The point describes the current world coordinate of the element that can be modified by this handler.
       * </p>
       * @abstract
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
    }
    var IDragHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum SnapPolicy{
      /**
       * Don't perform snapping.
       */
      NO_SNAPPING,
      /**
       * Snap to the nearest grid coordinate.
       */
      TO_NEAREST,
      /**
       * Snap to the nearest grid coordinate whose value is not less than the current coordinate.
       */
      TO_GREATER,
      /**
       * Snap to the nearest grid coordinate whose value is not greater than the current coordinate.
       */
      TO_SMALLER
    }
    /**
     * Interface for classes that can snap coordinates of items of a specific type to a grid.
     * Type parameter <code>T</code>: The type of the items this class can snap
     * @see yfiles.input.GridConstraintProvider.<T>
     * @interface
     * @template T
     */
    export interface IGridConstraintProvider<T> extends Object{
      /**
       * Performs the actual snapping to grid coordinates.
       * @param context The context in which the snapping should occur.
       * @param item The item of which the coordinates should be snapped.
       * @param location The coordinates to snap.
       * @param xSnapPolicy The policy for the x coordinate.
       * @param ySnapPolicy The policy for the y coordinate.
       * @returns Whether the point has been snapped.
       * @abstract
       */
      snapToGrid(context:yfiles.input.IInputModeContext,item:T,location:yfiles.geometry.IMutablePoint,xSnapPolicy:yfiles.input.SnapPolicy,ySnapPolicy:yfiles.input.SnapPolicy):boolean;
      /**
       * Gets the canonic grid width in horizontal direction.
       * @abstract
       * @type {number}
       */
      horizontalGridWidth:number;
      /**
       * Gets the canonic grid width in vertical direction.
       * @abstract
       * @type {number}
       */
      verticalGridWidth:number;
      /**
       * Gets the canonic origin of the grid.
       * @abstract
       * @type {yfiles.geometry.Point}
       */
      gridOrigin:yfiles.geometry.Point;
    }
    var IGridConstraintProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A simple integer based implementation of {@link yfiles.input.IGridConstraintProvider.<T>}.
     * Type parameter <code>T</code>: The type of the items.
     * @class
     * @implements {yfiles.input.IGridConstraintProvider.<T>}
     * @template T
     */
    export interface GridConstraintProvider<T> extends Object,yfiles.input.IGridConstraintProvider<T>{}
    export class GridConstraintProvider<T> {
      /**
       * Creates a new instance using the given grid info.
       * @param gridInfo The grid info object.
       */
      constructor(gridInfo:yfiles.view.GridInfo);
      /**
       * Creates a new instance using the given grid specification.
       * @param gridWidth Width of the grid.
       * @param gridHeight Height of the grid.
       */
      constructor(gridWidth:number,gridHeight:number);
      /**
       * Creates a new instance using the given grid distances.
       * @param gridSize
       */
      constructor(gridSize:number);
      /**
       * 
       * @param context
       * @param item
       * @param location
       * @param xSnapPolicy
       * @param ySnapPolicy
       * @returns 
       */
      snapToGrid(context:yfiles.input.IInputModeContext,item:T,location:yfiles.geometry.IMutablePoint,xSnapPolicy:yfiles.input.SnapPolicy,ySnapPolicy:yfiles.input.SnapPolicy):boolean;
      /**
       * Gets or sets the grid info that describes the geometry of the grid.
       * @type {yfiles.view.GridInfo}
       */
      gridInfo:yfiles.view.GridInfo;
      /**
       * Gets the grid height.
       * <p>
       * This implementation delegates to the {@link yfiles.input.GridConstraintProvider.<T>#gridInfo}
       * </p>
       * @type {number}
       */
      horizontalGridWidth:number;
      /**
       * Gets the grid width.
       * <p>
       * This implementation delegates to the {@link yfiles.input.GridConstraintProvider.<T>#gridInfo}
       * </p>
       * @type {number}
       */
      verticalGridWidth:number;
      /**
       * Gets the grid origin.
       * <p>
       * This implementation delegates to the {@link yfiles.input.GridConstraintProvider.<T>#gridInfo}
       * </p>
       * @type {yfiles.geometry.Point}
       */
      gridOrigin:yfiles.geometry.Point;
      static $class:yfiles.lang.Class;
    }
    export enum HandleTypes{
      /**
       * A handle which serves a general purpose but should not be displayed.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      INVISIBLE,
      /**
       * A default handle which serves a general purpose.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      DEFAULT,
      /**
       * A handle which can be used to resize something.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      RESIZE,
      /**
       * A handle which can be used to rotate something.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      ROTATE,
      /**
       * A handle which can be used to move something.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      MOVE,
      /**
       * A handle which can be used to shear an object.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      SHEAR,
      /**
       * A handle which can be used to warp an object.
       * <p>
       * This is the basic type of a handle and can be queried using the {@link yfiles.input.HandleTypes#TYPE_MASK}.
       * </p>
       */
      WARP,
      /**
       * A bitmask that can be used to query the basic type of a handle type.
       * @see yfiles.input.HandleTypes#DEFAULT
       * @see yfiles.input.HandleTypes#MOVE
       * @see yfiles.input.HandleTypes#RESIZE
       * @see yfiles.input.HandleTypes#ROTATE
       * @see yfiles.input.HandleTypes#SHEAR
       * @see yfiles.input.HandleTypes#WARP
       */
      TYPE_MASK,
      /**
       * A default variant modifier that decorates a basic type.
       * @see yfiles.input.HandleTypes#VARIANT_MASK
       */
      VARIANT1,
      /**
       * The second variant modifier that decorates a basic type.
       * @see yfiles.input.HandleTypes#VARIANT_MASK
       */
      VARIANT2,
      /**
       * The third variant modifier that decorates a basic type.
       * @see yfiles.input.HandleTypes#VARIANT_MASK
       */
      VARIANT3,
      /**
       * The forth variant modifier that decorates a basic type.
       * @see yfiles.input.HandleTypes#VARIANT_MASK
       */
      VARIANT4,
      /**
       * A bitmask that can be used to query the variant of a basic handle type.
       * @see yfiles.input.HandleTypes#VARIANT1
       * @see yfiles.input.HandleTypes#VARIANT2
       * @see yfiles.input.HandleTypes#VARIANT3
       * @see yfiles.input.HandleTypes#VARIANT4
       */
      VARIANT_MASK,
      /**
       * A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
       * offset to the north.
       * @see yfiles.input.HandleTypes#OFFSET_MASK
       */
      OFFSET_NORTH,
      /**
       * A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
       * offset to the east.
       * @see yfiles.input.HandleTypes#OFFSET_MASK
       */
      OFFSET_EAST,
      /**
       * A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
       * offset to the south.
       * @see yfiles.input.HandleTypes#OFFSET_MASK
       */
      OFFSET_SOUTH,
      /**
       * A modifier that can be used to tell the rendering engine to render the visual representation of this handle with an
       * offset to the west.
       * @see yfiles.input.HandleTypes#OFFSET_MASK
       */
      OFFSET_WEST,
      /**
       * A bitmask that can be used to query the offsets of a basic handle type.
       * @see yfiles.input.HandleTypes#OFFSET_NORTH
       * @see yfiles.input.HandleTypes#OFFSET_EAST
       * @see yfiles.input.HandleTypes#OFFSET_SOUTH
       * @see yfiles.input.HandleTypes#OFFSET_WEST
       */
      OFFSET_MASK
    }
    export enum HandlePositions{
      /**
       * Indicates no position at all.
       */
      NONE,
      /**
       * Indicates the position at north west.
       */
      NORTH_WEST,
      /**
       * Indicates the position at north.
       */
      NORTH,
      /**
       * Indicates the position at north east.
       */
      NORTH_EAST,
      /**
       * Indicates the position at west.
       */
      WEST,
      /**
       * Indicates the position at the center.
       */
      CENTER,
      /**
       * Indicates the position at east.
       */
      EAST,
      /**
       * Indicates the position at south west.
       */
      SOUTH_WEST,
      /**
       * Indicates the position at south.
       */
      SOUTH,
      /**
       * Indicates the position at south east.
       */
      SOUTH_EAST,
      /**
       * A combined bitset of all possible handle positions at the four corners of a rectangle.
       */
      CORNERS,
      /**
       * A combined bitset of the two horizontal handles of a rectangle.
       */
      HORIZONTAL,
      /**
       * A combined bitset of the two vertical handles of a rectangle.
       */
      VERTICAL,
      /**
       * A combined bitset of all possible handle positions at the border of a rectangle.
       */
      BORDER,
      /**
       * A combined bitset of all possible handle positions.
       */
      ALL
    }
    /**
     * Interface for a handle that can be displayed in a {@link yfiles.view.CanvasComponent} as a UI element for the user to modify the
     * contents of the control.
     * <p>
     * Handles can be dragged with the mouse and will thereby modify their context. Typically client code will use the {@link yfiles.input.IDragHandler#location} to
     * render a visual handle that can be dragged using the mouse. A drag will trigger the invocation of
     * {@link yfiles.input.IDragHandler#initializeDrag}, zero or more {@link yfiles.input.IDragHandler#handleMove} calls, and will be finalized
     * by either {@link yfiles.input.IDragHandler#dragFinished} or {@link yfiles.input.IDragHandler#cancelDrag}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: ReshapeHandleProvider, demo.yfiles.input.reshapehandleprovider</li>
     * </ul>
     * @see yfiles.input.HandleInputMode
     * @see yfiles.input.IDragHandler
     * @interface
     * @implements {yfiles.input.IDragHandler}
     */
    export interface IHandle extends Object,yfiles.input.IDragHandler{
      /**
       * Returns the type of the handle that can be used by the rendering engine to render types differently.
       * @abstract
       * @type {yfiles.input.HandleTypes}
       */
      type:yfiles.input.HandleTypes;
      /**
       * Provides the cursor to display when the mouse hovers over or drags this handle.
       * @abstract
       * @type {yfiles.view.Cursor}
       */
      cursor:yfiles.view.Cursor;
    }
    var IHandle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for an object that provides a number of {@link yfiles.input.IHandle} implementations.
     * @interface
     */
    export interface IHandleProvider extends Object{
      /**
       * Returns a collection of zero or more {@link yfiles.input.IHandle} implementations that are associated with this instance.
       * @param context
       * @returns A collection of handles.
       * @abstract
       */
      getHandles(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IHandle>;
    }
    var IHandleProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A specialized version of the {@link yfiles.input.IHandleProvider} interface that can be used to query {@link yfiles.input.IHandle}
     * implementation for resizing an object.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: ReshapeHandleProvider, demo.yfiles.input.reshapehandleprovider</li>
     * </ul>
     * @interface
     */
    export interface IReshapeHandleProvider extends Object{
      /**
       * Returns a bitwise combination of all of the {@link yfiles.input.HandlePositions} this interface can provide an implementation
       * for.
       * <p>
       * Client code will use this method to query the available positions and may then call for each of the positions the {@link yfiles.input.IReshapeHandleProvider#getHandle}
       * method to retrieve an implementation.
       * </p>
       * @param context The context for which the handles are queried.
       * @returns A bitwise combination of all positions the {@link #getHandle} method can be queried for.
       * @abstract
       */
      getAvailableHandles(context:yfiles.input.IInputModeContext):yfiles.input.HandlePositions;
      /**
       * Returns an implementation of an {@link yfiles.input.IHandle} for the given position, that can be used to reshape an object.
       * <p>
       * This method may be called for each possible single position contained in the set as returned by
       * {@link yfiles.input.IReshapeHandleProvider#getAvailableHandles}.
       * </p>
       * @param context The context for which the handles are queried.
       * @param position The single position a handle implementation should be returned for.
       * @returns An implementation of {@link } for the given position.
       * @abstract
       */
      getHandle(context:yfiles.input.IInputModeContext,position:yfiles.input.HandlePositions):yfiles.input.IHandle;
    }
    var IReshapeHandleProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface implemented by input modes that can be registered with a {@link yfiles.view.CanvasComponent} instance.
     * <p>
     * Input modes capture user gestures (or any other kind of "input") and manipulate the contents of the scene graph or the
     * model behind the scene graph.
     * </p>
     * <p>
     * Implementations may request an input mutex from the {@link yfiles.input.ConcurrencyController} that will be given to them in
     * their {@link yfiles.input.IInputMode#install} method. This allows them to exclusively modify the contents of the view. If
     * another {@link yfiles.input.IInputMode} successfully acquires the mutex, the {@link yfiles.input.ConcurrencyController} will be
     * {@link yfiles.input.ConcurrencyController#active deactivated}.
     * </p>
     * @see yfiles.input.InputModeBase
     * @interface
     */
    export interface IInputMode extends Object{
      /**
       * Called by the client to unconditionally cancel all editing.
       * <p>
       * This will be called prior to the uninstalling of this instance.
       * </p>
       * <p>
       * In order to stop an active input mode manually, client code should use the following idiom:
       * </p>
       * @abstract
       */
      cancel():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * <br />
       * When this instance gets {@link yfiles.input.IInputMode#uninstall uninstalled} from the context the same context instance will be passed to it.
       * <br />
       * Implementations may hold a reference to the <code>context</code> instance and use it while they are being installed.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link } for this mode.
       * @see yfiles.input.IInputMode#uninstall
       * @abstract
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called by the client in order to stop a current editing progress.
       * <p>
       * This should stop the current edit, if one is in progress and possibly commit all of the changes. If stopping is not
       * possible, this method can return <code>false</code>
       * </p>
       * @returns <code>true</code> if and only if the editing has been stopped or there was no edit in progress
       * @see yfiles.input.IInputMode#cancel
       * @abstract
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       * @abstract
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Returns the context instance this mode is currently installed in or <code>null</code> if this instance is not installed.
       * <p>
       * Note that this instance should not be passed down to subordinate modes or instances. Instead a corresponding context
       * that has this instance set as the {@link yfiles.input.IInputModeContext#parentInputMode} should be used.
       * </p>
       * @abstract
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * The priority of this input mode.
       * <p>
       * The priority will influence the order in which the modes will be {@link yfiles.input.IInputMode#install installed} into the canvas control. The lower the
       * priority value, the earlier it will be installed. If two modes are installed using the same priority value, the first
       * one will be installed earlier.
       * </p>
       * @abstract
       * @type {number}
       */
      priority:number;
    }
    var IInputMode:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface for the context object that is passed to {@link yfiles.input.IInputMode} instances during {@link yfiles.input.IInputMode#install}
     * and {@link yfiles.input.IInputMode#uninstall} and {@link yfiles.input.IDragHandler} implementations like {@link yfiles.input.IPositionHandler}
     * and {@link yfiles.input.IHandle} to provide additional context for the implementations.
     * <p>
     * Implementations of this interface can carry additional information through their {@link yfiles.graph.ILookup#lookup} method.
     * </p>
     * @see yfiles.graph.ILookup
     * @see yfiles.input.IInputMode
     * @interface
     * @implements {yfiles.view.ICanvasContext}
     */
    export interface IInputModeContext extends Object,yfiles.view.ICanvasContext{
      /**
       * Tries to call {@link yfiles.graph.IGraph#invalidateDisplays} on the {@link yfiles.input.IInputModeContext#graph graph} obtains from the {@link yfiles.input.IInputModeContext}.
       * <p>
       * If the graph instance cannot be found, {@link yfiles.view.CanvasComponent#invalidate} will be called instead.
       * </p>
       */
      invalidateDisplays?():void;
      /**
       * Yields the {@link yfiles.input.IInputMode} which issued the context object.
       * <p>
       * This can be <code>null</code> if this context is used outside the context of an IInputMode.
       * </p>
       * @abstract
       * @type {yfiles.input.IInputMode}
       */
      parentInputMode:yfiles.input.IInputMode;
      /**
       * Tries to retrieve an {@link yfiles.graph.IGraph} instance from an {@link yfiles.input.IInputModeContext}.
       * <p>
       * This will use the {@link yfiles.graph.ILookup} and if this yields nothing, it will use the {@link yfiles.graph.ILookup} of the {@link yfiles.view.ICanvasContext#canvasComponent} or
       * finally test if it is a {@link yfiles.view.GraphComponent} to obtain the graph from.
       * </p>
       * @type {yfiles.graph.IGraph}
       */
      graph?:yfiles.graph.IGraph;
    }
    var IInputModeContext:{
      /**
       * Factory method that creates an {@link yfiles.input.IInputModeContext} that uses the provided parent mode, parent context, and
       * lookup decoration.
       * @param parent
       * @param parentContext
       * @param lookupCallback
       * @returns 
       * @static
       */
      createInputModeContext?(parent:yfiles.input.IInputMode,parentContext:yfiles.input.IInputModeContext,lookupCallback:(subject:Object,type:yfiles.lang.Class)=>Object):yfiles.input.IInputModeContext;
      /**
       * Factory method that creates an {@link yfiles.input.IInputModeContext} that uses the provided parent mode and lookup decoration.
       * @param parent
       * @param lookupCallback
       * @returns 
       * @static
       */
      createInputModeContext?(parent:yfiles.input.IInputMode,lookupCallback:(subject:Object,type:yfiles.lang.Class)=>Object):yfiles.input.IInputModeContext;
      /**
       * Factory method that creates an {@link yfiles.input.IInputModeContext} that uses the provided values.
       * @param canvas The {@link } to use, may not be <code>null</code>.
       * @param [parent=null] The mode to set as the parent. This may be <code>null</code> for the canvas' context.
       * @param [lookup=null] The lookup to use.
       * @returns 
       * @static
       */
      createInputModeContext?(canvas:yfiles.view.CanvasComponent,parent?:yfiles.input.IInputMode,lookup?:yfiles.graph.ILookup):yfiles.input.IInputModeContext;
      /**
       * Factory method that creates an {@link yfiles.input.IInputModeContext} that uses the provided values.
       * @param {Object} options The parameters to pass.
       * @param options.canvas The {@link } to use, may not be <code>null</code>.
       * @param [options.parent=null] The mode to set as the parent. This may be <code>null</code> for the canvas' context.
       * @param [options.lookup=null] The lookup to use.
       * @returns 
       * @static
       */
      createInputModeContext?(options:{canvas:yfiles.view.CanvasComponent,parent?:yfiles.input.IInputMode,lookup?:yfiles.graph.ILookup}):yfiles.input.IInputModeContext;
      /**
       * Factory method that creates an {@link yfiles.input.IInputModeContext} that uses the provided parent mode, parent context, and
       * lookup decoration.
       * @param parent
       * @param parentContext
       * @param lookup
       * @returns 
       * @static
       */
      createInputModeContext?(parent:yfiles.input.IInputMode,parentContext:yfiles.input.IInputModeContext,lookup:yfiles.graph.ILookup):yfiles.input.IInputModeContext;
      /**
       * Factory method that creates an {@link yfiles.input.IInputModeContext} that uses the provided parent mode and lookup decoration.
       * @param parent
       * @param [lookup=null]
       * @returns 
       * @static
       */
      createInputModeContext?(parent:yfiles.input.IInputMode,lookup?:yfiles.graph.ILookup):yfiles.input.IInputModeContext;
      /**
       * An empty context where both the inputmode and the canvas is <code>null</code>.
       * <p>
       * The lookup always yields null, also.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.IInputModeContext}
       */
      EMPTY?:yfiles.input.IInputModeContext;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface that allows input modes to collect {@link yfiles.graph.IModelItem}s for various purposes.
     * <p>
     * This is mainly used for {@link yfiles.input.HandleInputMode} and {@link yfiles.input.MoveInputMode} to allow access to the items that
     * were affected during the respective operation.
     * </p>
     * @see yfiles.input.HandleInputMode#affectedItems
     * @see yfiles.input.MoveInputMode#affectedItems
     * @interface
     */
    export interface IModelItemCollector extends Object{
      /**
       * Adds a model item to this collector instance.
       * @param item The model item to add.
       * @abstract
       */
      add(item:yfiles.graph.IModelItem):void;
    }
    var IModelItemCollector:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for an object that can handle the position of an item displayed in a {@link yfiles.view.CanvasComponent}
     * <p>
     * The point as indicated by {@link yfiles.input.IDragHandler#location} describes the current world coordinate position of the element. It is up to the
     * implementation how this position is interpreted. The values returned by that instance will be used for the
     * "originalLocation" parameter in the {@link yfiles.input.IDragHandler#handleMove}, {@link yfiles.input.IDragHandler#cancelDrag} and {@link yfiles.input.IDragHandler#dragFinished}
     * methods. Items can be dragged with the mouse and an instance of this class will modify their position accordingly.
     * Typically client code will use the {@link yfiles.input.IDragHandler#location}'s coordinates to determine the current position of the elements. A drag will trigger
     * the invocation of {@link yfiles.input.IDragHandler#initializeDrag}, zero or more {@link yfiles.input.IDragHandler#handleMove} calls, and
     * finally by either {@link yfiles.input.IDragHandler#dragFinished} or {@link yfiles.input.IDragHandler#cancelDrag}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: PositionHandler, demo.yfiles.input.positionhandler</li>
     * </ul>
     * @see yfiles.input.IDragHandler
     * @see yfiles.input.IHandle
     * @see yfiles.input.MoveInputMode
     * @interface
     * @implements {yfiles.input.IDragHandler}
     */
    export interface IPositionHandler extends Object,yfiles.input.IDragHandler{
    }
    var IPositionHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for an object that can be interactively reshaped in a {@link yfiles.view.CanvasComponent}
     * <p>
     * Items can be reshaped interactively and an instance of this class can be used to handle that process. A reshape
     * operation will be initialized by a call to {@link yfiles.input.IReshapeHandler#initializeReshape}, then zero or more {@link yfiles.input.IReshapeHandler#handleReshape}
     * calls follow, and the operation will be finalized by a call to either {@link yfiles.input.IReshapeHandler#reshapeFinished} or
     * {@link yfiles.input.IReshapeHandler#cancelReshape}. Note that implementations of this class will only be used during interactive
     * reshape operations.
     * </p>
     * @see yfiles.input.IDragHandler
     * @see yfiles.input.IPositionHandler
     * @see yfiles.input.IHandle
     * @see yfiles.input.IInputModeContext
     * @interface
     */
    export interface IReshapeHandler extends Object{
      /**
       * Called by clients to indicate that the reshaping has been canceled by the user.
       * <p>
       * This method may be called after the initial {@link yfiles.input.IReshapeHandler#initializeReshape} and zero or more invocations
       * of {@link yfiles.input.IReshapeHandler#handleReshape}. Implementations should reset the bounds of the items they modify to their
       * initial state. Alternatively to this method the {@link yfiles.input.IReshapeHandler#reshapeFinished} method might be called.
       * </p>
       * @param context The context to retrieve information about the reshaping from.
       * @param originalBounds The value of the coordinate of the {@link #bounds} property at the time of {@link #initializeReshape}.
       * @abstract
       */
      cancelReshape(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect):void;
      /**
       * Called by clients to indicate that the element has been dragged and its position should be updated.
       * <p>
       * This method may be called more than once after an initial {@link yfiles.input.IReshapeHandler#initializeReshape} and will the
       * final call will be followed by either one {@link yfiles.input.IReshapeHandler#reshapeFinished} or one {@link yfiles.input.IReshapeHandler#cancelReshape}
       * call.
       * </p>
       * @param context The context to retrieve information about the reshaping from.
       * @param originalBounds The value of the {@link #bounds} property at the time of {@link #initializeReshape}.
       * @param newBounds The coordinates of the bounds in the world coordinate system that the client wants the shape to be at. Depending on the
       * implementation the {@link #bounds} may or may not be modified to reflect the new value.
       * @abstract
       */
      handleReshape(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * Called by clients to indicate that the element is going to be reshaped.
       * <p>
       * This call will be followed by one or more calls to {@link yfiles.input.IReshapeHandler#handleReshape}, and a final {@link yfiles.input.IReshapeHandler#reshapeFinished}
       * or {@link yfiles.input.IReshapeHandler#cancelReshape}.
       * </p>
       * @param context The context to retrieve information about the drag from.
       * @abstract
       */
      initializeReshape(context:yfiles.input.IInputModeContext):void;
      /**
       * Called by clients to indicate that the reshaping has just been finished.
       * <p>
       * This method may be called after the initial {@link yfiles.input.IReshapeHandler#initializeReshape} and zero or more invocations
       * of {@link yfiles.input.IReshapeHandler#handleReshape}. Alternatively to this method the {@link yfiles.input.IReshapeHandler#cancelReshape}
       * method might be called.
       * </p>
       * @param context The context to retrieve information about the drag from.
       * @param newBounds The coordinates of the bounds in the world coordinate system that the client wants the shape to be at. Depending on the
       * implementation the {@link #bounds} may or may not be modified to reflect the new value. This is the same value as delivered in the last
       * invocation of {@link #handleReshape}
       * @param originalBounds The value of the coordinate of the {@link #bounds} property at the time of {@link #initializeReshape}.
       * @abstract
       */
      reshapeFinished(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * Returns a view of the bounds of the item.
       * <p>
       * The rectangle describes the current world coordinate of the element that can be modified by this handler.
       * </p>
       * @abstract
       * @type {yfiles.geometry.IRectangle}
       */
      bounds:yfiles.geometry.IRectangle;
    }
    var IReshapeHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Event argument base class that is used by events that are triggered by {@link yfiles.input.IInputMode} implementations
     * @class
     * @extends {yfiles.lang.EventArgs}
     */
    export interface InputModeEventArgs extends yfiles.lang.EventArgs{}
    export class InputModeEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.InputModeEventArgs} class.
       * @param context The context to initialize the {@link #context} property with.
       */
      constructor(context:yfiles.input.IInputModeContext);
      /**
       * Gets the context for the current event.
       * @type {yfiles.input.IInputModeContext}
       */
      context:yfiles.input.IInputModeContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} that recognizes simple key events and invokes a registered handler.
     * <p>
     * This mode is {@link yfiles.input.KeyboardInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface KeyboardInputMode extends Object,yfiles.input.IInputMode{}
    export class KeyboardInputMode {
      constructor();
      /**
       * Adds a command and associated execution handlers to this instance.
       * <p>
       * The command will not be allowed to execute if this mode is {@link yfiles.input.KeyboardInputMode#enabled disabled} or {@link yfiles.input.ConcurrencyController#active inactive}.
       * </p>
       * @param command The command to register handlers with.
       * @param [execute=null] The handler for the execution; a handler that will do nothing if omitted.
       * @param [canExecute=null] The handler that determines executability; a handler that is always enabled if omitted.
       * @returns A token for the newly created command binding that can used to later {@link #remove remove} this binding from this instance again.
       */
      addCommandBinding(command:yfiles.input.ICommand,execute?:(command:yfiles.input.ICommand,parameter:Object,source:Object)=>boolean,canExecute?:(command:yfiles.input.ICommand,parameter:Object,source:Object)=>boolean):yfiles.input.KeyboardInputModeBinding;
      /**
       * Adds a command and associated execution handlers to this instance.
       * <p>
       * The command will not be allowed to execute if this mode is {@link yfiles.input.KeyboardInputMode#enabled disabled} or {@link yfiles.input.ConcurrencyController#active inactive}.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.command The command to register handlers with.
       * @param [options.execute=null] The handler for the execution; a handler that will do nothing if omitted.
       * @param [options.canExecute=null] The handler that determines executability; a handler that is always enabled if omitted.
       * @returns A token for the newly created command binding that can used to later {@link #remove remove} this binding from this instance again.
       */
      addCommandBinding(options:{command:yfiles.input.ICommand,execute?:(command:yfiles.input.ICommand,parameter:Object,source:Object)=>boolean,canExecute?:(command:yfiles.input.ICommand,parameter:Object,source:Object)=>boolean}):yfiles.input.KeyboardInputModeBinding;
      /**
       * Adds an event handler for a specific key press gesture to this mode.
       * <p>
       * Using this registration method, the command will not be allowed to execute if this mode is {@link yfiles.input.KeyboardInputMode#enabled disabled} or {@link yfiles.input.ConcurrencyController#active inactive}.
       * </p>
       * @param key The key constant that should be recognized.
       * @param modifiers The modifiers that should be recognized when the key is pressed.
       * @param command The command to execute.
       * @param [commandParameter=null] The command parameter to use for the {@link #execute execution}; <code>null</code> if omitted.
       * @returns A token for the newly created command binding that can used to later {@link #remove remove} this binding from this instance again.
       */
      addKeyBinding(key:yfiles.view.Key,modifiers:yfiles.view.ModifierKeys,command:yfiles.input.ICommand,commandParameter?:Object):yfiles.input.KeyboardInputModeBinding;
      /**
       * Adds a given handler to this instance that will be triggered if the event recognizer recognizes a {@link yfiles.view.KeyEventArgs key event} that has
       * been triggered by the {@link yfiles.view.CanvasComponent}.
       * @param recognizer An event recognizer that will be fed with all key events.
       * @param command The command to invoke if the recognizer matches a key event.
       * @param [commandParameter=null] The optional parameter to provide to the command.
       * @returns The newly constructed binding on which {@link #remove} can be called to remove the
       * resulting binding from this instance.
       */
      addRecognizerBinding(recognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean,command:yfiles.input.ICommand,commandParameter?:Object):yfiles.input.KeyboardInputModeBinding;
      /**
       * 
       */
      cancel():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.KeyboardInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Enables the registered commands and can be overridden in subclasses to perform additional actions after the mode has
       * been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Disables the registered commands and can be overridden in subclasses to perform additional actions after the mode has
       * been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called after {@link yfiles.input.KeyboardInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Removes all previously registered commands from this instance.
       * <p>
       * This effectively disables the command for the instance managed by this mode.
       * </p>
       * @param command The command to remove from all registered bindings.
       * @see yfiles.input.KeyboardInputMode#addKeyBinding
       * @see yfiles.input.KeyboardInputMode#addRecognizerBinding
       */
      removeCommand(command:yfiles.input.ICommand):void;
      /**
       * Overridden to only return <code>true</code> if this instance does not currently {@link yfiles.input.ConcurrencyController#hasMutex have the input mutex}.
       * @returns <code>true</code> iff this instance does not {@link #hasMutex own the mutex}.
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.KeyboardInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.KeyboardInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.KeyboardInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      static $class:yfiles.lang.Class;
    }
    /**
     * A token returned by {@link yfiles.input.KeyboardInputMode}'s factory methods that can be used to {@link yfiles.input.KeyboardInputModeBinding#remove}
     * the binding again.
     * <p>
     * Instances of this class can only be obtained from AddCommandBinding, {@link yfiles.input.KeyboardInputMode#addKeyBinding}, and
     * {@link yfiles.input.KeyboardInputMode#addRecognizerBinding}. Removing the instances is done via the instance method {@link yfiles.input.KeyboardInputModeBinding#remove}
     * of this type.
     * </p>
     * @class
     * @final
     */
    export interface KeyboardInputModeBinding extends Object{}
    export class KeyboardInputModeBinding {
      /**
       * Removes the binding represented by this instance from the {@link yfiles.input.KeyboardInputMode} it has been created for.
       * <p>
       * Once this method has been called, the instance can be
       * </p>
       */
      remove():void;
      /**
       * Gets the {@link yfiles.input.ICommand} that is associated with this binding.
       * @type {yfiles.input.ICommand}
       */
      command:yfiles.input.ICommand;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} implementation that can handle marquee selections performed by the user with a mouse.
     * <p>
     * This implementation will request the mutex as soon as it recognizes a mouse drag.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.MarqueeSelectionInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface MarqueeSelectionInputMode extends Object,yfiles.input.IInputMode{}
    export class MarqueeSelectionInputMode {
      /**
       * Creates a new instance of the marquee selection mode.
       */
      constructor();
      /**
       * Cancels any pending marquee selection.
       */
      cancel():void;
      /**
       * Helper method that yields a suitably configured {@link yfiles.input.InputModeEventArgs} for this input mode.
       * @returns An input mode event argument that is configured for this instance.
       * @protected
       */
      createInputModeEventArgs():yfiles.input.MarqueeSelectionEventArgs;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.MarqueeSelectionInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDragCanceledListener DragCanceled}
       * event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceled(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDragCancelingListener DragCanceling}
       * event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceling(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Callback triggered once the drag has been finalized.
       * <p>
       * This method triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDragFinishedListener DragFinished}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinished(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Callback triggered before the drag will be finalized.
       * <p>
       * This method triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDragFinishingListener DragFinishing}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinishing(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDraggedListener Dragged}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragged(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDraggingListener Dragging}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragging(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDragStartedListener DragStarted}
       * event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarted(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MarqueeSelectionInputMode#addDragStartingListener DragStarting}
       * event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarting(evt:yfiles.input.MarqueeSelectionEventArgs):void;
      /**
       * Called after {@link yfiles.input.MarqueeSelectionInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.MarqueeSelectionInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.MarqueeSelectionInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.MarqueeSelectionInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets the current or last marquee selection rectangle.
       * @type {yfiles.geometry.Rect}
       */
      selectionRectangle:yfiles.geometry.Rect;
      /**
       * Gets or sets the template that is used for the visualization of the marquee rectangle.
       * @type {yfiles.view.IVisualTemplate}
       */
      template:yfiles.view.IVisualTemplate;
      /**
       * Gets or sets the "cancel" recognizer.
       * <p>
       * This recognizer recognizes the cancel action during the marquee selection.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * A {@link string} that can be used to store a {@link yfiles.view.IVisualTemplate} that can be used to create the visual that will be used to
       * draw the marquee box.
       * @const
       * @static
       * @type {string}
       */
      static MARQUEE_RECTANGLE_TEMPLATE_KEY:string;
      /**
       * Gets or sets the cursor to use during the dragging of the marquee.
       * <p>
       * The default is <code>Cursors.Cross</code>
       * </p>
       * @type {yfiles.view.Cursor}
       */
      marqueeCursor:yfiles.view.Cursor;
      /**
       * Gets or sets the "pressed" state recognizer.
       * <p>
       * This recognizer instance will be used to determine when the user begins to draw the marquee.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer.
       * <p>
       * This recognizer instance determines when the user is dragging the marquee.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer.
       * <p>
       * This instance determines when the user has finished creating the marquee.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" state recognizer for touch input.
       * <p>
       * This recognizer instance will be used to determine when the user begins to draw the marquee.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer for touch input.
       * <p>
       * This recognizer instance determines when the user is dragging the marquee.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer for touch input.
       * <p>
       * This instance determines when the user has finished creating the marquee.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDraggingListener
       */
      addDraggingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDraggingListener
       */
      removeDraggingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDraggedListener
       */
      addDraggedListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDraggedListener
       */
      removeDraggedListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDragCanceledListener
       */
      addDragCanceledListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDragCanceledListener
       */
      removeDragCanceledListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDragCancelingListener
       */
      addDragCancelingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDragCancelingListener
       */
      removeDragCancelingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDragFinishingListener
       */
      addDragFinishingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDragFinishingListener
       */
      removeDragFinishingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinished</code> event that occurs the drag has been finished.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDragFinishedListener
       */
      addDragFinishedListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinished</code> event that occurs the drag has been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDragFinishedListener
       */
      removeDragFinishedListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDragStartingListener
       */
      addDragStartingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDragStartingListener
       */
      removeDragStartingListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to add.
       * @see yfiles.input.MarqueeSelectionInputMode#removeDragStartedListener
       */
      addDragStartedListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to remove.
       * @see yfiles.input.MarqueeSelectionInputMode#addDragStartedListener
       */
      removeDragStartedListener(listener:(sender:Object,evt:yfiles.input.MarqueeSelectionEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments used by {@link yfiles.input.MarqueeSelectionInputMode}.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface MarqueeSelectionEventArgs extends yfiles.input.InputModeEventArgs{}
    export class MarqueeSelectionEventArgs {
      /**
       * Gets the current {@link yfiles.input.MarqueeSelectionInputMode#selectionRectangle selection rectangle} of the {@link yfiles.input.MarqueeSelectionInputMode}.
       * @type {yfiles.geometry.Rect}
       */
      rectangle:yfiles.geometry.Rect;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event argument that can be used to query the text to display when the mouse hovers over the given point in world
     * coordinate space.
     * @see yfiles.input.MouseHoverInputMode
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface ToolTipQueryEventArgs extends yfiles.input.InputModeEventArgs{}
    export class ToolTipQueryEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.ToolTipQueryEventArgs} class.
       * @param context The context of the query.
       * @param queryLocation The query location.
       */
      constructor(context:yfiles.input.IInputModeContext,queryLocation:yfiles.geometry.Point);
      /**
       * Gets the query location in world coordinates.
       * @type {yfiles.geometry.Point}
       */
      queryLocation:yfiles.geometry.Point;
      /**
       * Gets or sets the tool tip content to use.
       * <p>
       * The content can be either a string or any HTML element.
       * </p>
       * <p>
       * Setting this property will set the {@link yfiles.input.ToolTipQueryEventArgs#handled} property to <code>true</code>.
       * </p>
       * @type {Object}
       */
      toolTip:Object;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.ToolTipQueryEventArgs} has been handled.
       * <p>
       * This property is automatically set to <code>true</code> if {@link yfiles.input.ToolTipQueryEventArgs#toolTip} property has been assigned a value. Marking this event as handled tells
       * the issuer of the query whether the {@link yfiles.input.ToolTipQueryEventArgs#toolTip} property should be used or whether additional logic should be used to determine
       * whether to show the menu.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} that detects when the mouse hovers over the {@link yfiles.view.CanvasComponent}.
     * <p>
     * This implementation will show a {@link yfiles.input.MouseHoverInputMode#toolTip ToolTip}. In order to make use of this instance one has to register with the {@link yfiles.input.MouseHoverInputMode#addQueryToolTipListener QueryToolTip}
     * event. Setting a custom {@link yfiles.input.MouseHoverInputMode#validHoverLocationHitTestable} will restrict the area where a tool tip can be shown.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.MouseHoverInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface MouseHoverInputMode extends Object,yfiles.input.IInputMode{}
    export class MouseHoverInputMode {
      /**
       * Creates a new instance using the provided {@link yfiles.input.MouseHoverInputMode#toolTip} and query callback.
       * @param toolTip
       * @param textProvider
       */
      constructor(toolTip:yfiles.view.ToolTip,textProvider:(sender:Object,evt:yfiles.input.ToolTipQueryEventArgs)=>void);
      /**
       * Creates a new instance with default values.
       */
      constructor();
      /**
       * Adjust the calculated tooltip position.
       * <p>
       * This default implementation of this method moves the tooltip into the visible area if it extends beyond the document
       * bounds.
       * </p>
       * @param originalPosition The calculated tooltip position relative to the document root
       * @returns The adjusted tooltip location
       * @see yfiles.input.MouseHoverInputMode#getToolTipLocation
       * @protected
       */
      adjustTooltipPosition(originalPosition:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * 
       */
      cancel():void;
      /**
       * Creates an {@link yfiles.input.IInputModeContext} for use with the {@link yfiles.input.MouseHoverInputMode#addQueryToolTipListener QueryToolTip}
       * event for the upcoming text query operation.
       * @returns An instance of {@link }.
       * @protected
       */
      createHoverInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Factory method that creates the {@link yfiles.input.MouseHoverInputMode#toolTip}.
       * @returns A simple {@link #toolTip}.
       * @protected
       */
      createToolTip():yfiles.view.ToolTip;
      /**
       * Callback method that provides a text for the given location or <code>null</code>.
       * @param location The location to find a text for.
       * @returns A string or <code>null</code> to indicate that no tool tip should be shown.
       * @see yfiles.input.MouseHoverInputMode#addQueryToolTipListener
       * @see yfiles.input.MouseHoverInputMode#createHoverInputModeContext
       * @protected
       */
      getToolTipContent(location:yfiles.geometry.Point):Object;
      /**
       * Finds the position in canvas view coordinates to display the tool tip at for the given world coordinates.
       * <p>
       * This method adds the {@link yfiles.input.MouseHoverInputMode#toolTipLocationOffset} after converting the given location to view coordinates.
       * </p>
       * @param location The position in world coordinates.
       * @returns The position in view coordinates.
       * @see yfiles.input.MouseHoverInputMode#toolTipLocationOffset
       * @protected
       */
      getToolTipLocation(location:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Immediately hides the tool tip.
       */
      hide():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.MouseHoverInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called when the tooltip should be hidden.
       * @protected
       */
      onHide():void;
      /**
       * Raises the {@link yfiles.input.MouseHoverInputMode#addQueryToolTipListener QueryToolTip} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onQueryToolTip(evt:yfiles.input.ToolTipQueryEventArgs):void;
      /**
       * Triggered when the mouse hovers over the given coordinates.
       * <p>
       * This method will call {@link yfiles.input.MouseHoverInputMode#getToolTipContent} to query the content and will display the {@link yfiles.input.MouseHoverInputMode#toolTip} at
       * the location returned by {@link yfiles.input.MouseHoverInputMode#getToolTipLocation} if the content is not <code>null</code>.
       * </p>
       * @param location
       * @protected
       */
      onShow(location:yfiles.geometry.Point):void;
      /**
       * Called after {@link yfiles.input.MouseHoverInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Immediately shows the tool tip at the given location.
       * @param location The location in the world coordinate system.
       */
      show(location:yfiles.geometry.Point):void;
      /**
       * Overridden to only return <code>true</code> if this instance does not currently {@link yfiles.input.ConcurrencyController#hasMutex have the input mutex}.
       * @returns <code>true</code> iff this instance does not {@link #hasMutex own the mutex}.
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.MouseHoverInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.MouseHoverInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.MouseHoverInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets or sets an {@link yfiles.input.IHitTestable} that determines where the mouse may hover and a tool tip can be {@link yfiles.input.MouseHoverInputMode#onShow shown}.
       * <p>
       * The default value of this property is {@link yfiles.input.IHitTestable#ALWAYS}
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      validHoverLocationHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets or sets the {@link yfiles.input.MouseHoverInputMode#toolTip} instance to use during display.
       * <p>
       * This method will trigger the {@link yfiles.input.MouseHoverInputMode#createToolTip} method if no tool tip has been set.
       * </p>
       * @type {yfiles.view.ToolTip}
       */
      toolTip:yfiles.view.ToolTip;
      /**
       * Determines whether the tool tip is currently showing.
       * @type {boolean}
       */
      showing:boolean;
      /**
       * Gets or sets the tool tip location offset in view coordinates.
       * <p>
       * The value of this property is used by {@link yfiles.input.MouseHoverInputMode#getToolTipLocation} in order to move the tool tip
       * away from the mouse pointer. The default value is (0,0).
       * </p>
       * @see yfiles.input.MouseHoverInputMode#getToolTipLocation
       * @type {yfiles.geometry.Point}
       */
      toolTipLocationOffset:yfiles.geometry.Point;
      /**
       * Gets or sets the duration to show the tool tip.
       * <p>
       * The default value is 3 seconds.
       * </p>
       * @default '00:00:03.0'
       * @type {yfiles.lang.TimeSpan}
       */
      duration:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the delay time before showing the tool tip.
       * <p>
       * The default is
       * </p>
       * <pre>
       * 1
       * </pre>
       * 
       * <p>
       * second.
       * </p>
       * @default '00:00:01.0'
       * @type {yfiles.lang.TimeSpan}
       */
      delay:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the amount the mouse pointer has to move in order to hide the tooltip.
       * <p>
       * The default value is 10, 10.
       * </p>
       * @default '10, 10'
       * @type {yfiles.geometry.Size}
       */
      mouseHoverSize:yfiles.geometry.Size;
      /**
       * If <code>true</code> the tooltip is closed when the a click is performed over the tooltip.
       * @type {boolean}
       */
      closeOnClick:boolean;
      /**
       * Adds the given listener for the <code>QueryToolTip</code> event that occurs when this mode queries the tool tip for a certain query
       * location.
       * <p>
       * Handlers should set the {@link yfiles.input.ToolTipQueryEventArgs#toolTip} property or set/respect the {@link yfiles.input.ToolTipQueryEventArgs#handled} property accordingly.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.MouseHoverInputMode#removeQueryToolTipListener
       */
      addQueryToolTipListener(listener:(sender:Object,evt:yfiles.input.ToolTipQueryEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryToolTip</code> event that occurs when this mode queries the tool tip for a certain
       * query location.
       * <p>
       * Handlers should set the {@link yfiles.input.ToolTipQueryEventArgs#toolTip} property or set/respect the {@link yfiles.input.ToolTipQueryEventArgs#handled} property accordingly.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.MouseHoverInputMode#addQueryToolTipListener
       */
      removeQueryToolTipListener(listener:(sender:Object,evt:yfiles.input.ToolTipQueryEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Generic {@link yfiles.input.IInputMode} implementation that can be used to move something in the canvas using the mouse.
     * <p>
     * This implementation uses the {@link yfiles.input.MoveInputMode#hitTestable} to determine where the user can begin dragging the selection. It will then use the {@link yfiles.input.MoveInputMode#positionHandler} to
     * delegate the actual work of moving the elements to.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.MoveInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface MoveInputMode extends Object,yfiles.input.IInputMode{}
    export class MoveInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.MoveInputMode} class.
       */
      constructor();
      /**
       * Callback that is called when this mode is "armed".
       * <p>
       * This happens when the mouse {@link yfiles.input.MoveInputMode#hoverRecognizer hovers} over the item(s) to move. This implementation sets the mouse cursor to {@link yfiles.input.MoveInputMode#moveCursor}.
       * </p>
       * @protected
       */
      arm():void;
      /**
       * Cancels the editing of this mode.
       * <p>
       * This implementation {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
       * {@link yfiles.input.MoveInputMode#onCanceled}.
       * </p>
       * @see yfiles.input.MoveInputMode#onCanceled
       */
      cancel():void;
      /**
       * Creates an {@link yfiles.input.IInputModeContext} for use with the {@link yfiles.input.IPositionHandler} interface for the upcoming drag
       * operation.
       * @returns An instance of {@link }.
       * @protected
       */
      createPositionHandlerInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Callback that is called when this mode is "disarmed".
       * <p>
       * This happens when the mouse stops hovering over the item(s). This implementation resets the mouse cursor again.
       * </p>
       * @protected
       */
      disarm():void;
      /**
       * Simulates the start of a drag and puts the state machine into the dragging state.
       * <p>
       * Subsequent moves of input devices will behave as if the dragging has been successfully initiated at <code>location</code>.
       * </p>
       * @param location The location where the drag had been initialized.
       */
      doStartDrag(location:yfiles.geometry.Point):void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Determines whether the given location is a valid start location for the move gesture.
       * <p>
       * This implementation delegates to the {@link yfiles.input.MoveInputMode#hitTestable}. During this callback it is possible to reset or reconfigure the {@link yfiles.input.MoveInputMode#positionHandler}.
       * </p>
       * @param location The location in the world coordinate system.
       * @returns <code>true</code> if at the given location it is valid to start the move gesture; otherwise, <code>false</code>.
       * @protected
       */
      isValidStartLocation(location:yfiles.geometry.Point):boolean;
      /**
       * Called after {@link yfiles.input.MoveInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Triggers the {@link yfiles.input.MoveInputMode#addDragCanceledListener DragCanceled} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MoveInputMode#addDragCancelingListener DragCanceling} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceling(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered once the drag has been finalized.
       * <p>
       * This method triggers the {@link yfiles.input.MoveInputMode#addDragFinishedListener DragFinished} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinished(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered before the drag will be finalized.
       * <p>
       * This method triggers the {@link yfiles.input.MoveInputMode#addDragFinishingListener DragFinishing} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinishing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.MoveInputMode#addDraggedListener Dragged} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragged(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.MoveInputMode#addDraggingListener Dragging} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragging(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MoveInputMode#addDragStartedListener DragStarted} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarted(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MoveInputMode#addDragStartingListener DragStarting} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarting(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.MoveInputMode#addQueryPositionHandlerListener QueryPositionHandler} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onQueryPositionHandler(evt:yfiles.input.QueryPositionHandlerEventArgs):void;
      /**
       * Called after {@link yfiles.input.MoveInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.MoveInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.MoveInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.MoveInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets event recognizer that recognizes when the user temporarily disables snapping.
       * @see yfiles.input.MoveInputMode#enableSnappingRecognizer
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      disableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets event recognizer that recognizes when the user reenables temporarily disabled snapping.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      enableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" state recognizer.
       * <p>
       * This recognizer instance will be used to determine when the user begins to move the selection.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "hovering" state recognizer.
       * <p>
       * This recognizer instance will be used to determine whether the user is hovering over an item where pressing can
       * initialize a move operation. The default implementation uses the {@link yfiles.input.MoveInputMode#isValidStartLocation} method
       * to determine whether the mode can be initialized, here.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      hoverRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer.
       * <p>
       * This recognizer instance determines when the user is moving the selection
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "cancel" recognizer.
       * <p>
       * This recognizer recognizes a cancel action during the move.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer.
       * <p>
       * This instance determines when the user has finished the move.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" recognizer for touch events.
       * <p>
       * This recognizer instance determines when the user begins the move selection operation.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer for touch events.
       * <p>
       * This recognizer instance determines when the user is moving the selection
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer for touch events.
       * <p>
       * This instance determines when the user has finished the move using touch.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the {@link yfiles.input.IHitTestable} that will be used by this mode to determine where the user may start
       * dragging.
       * @type {yfiles.input.IHitTestable}
       */
      hitTestable:yfiles.input.IHitTestable;
      /**
       * Gets or sets the {@link yfiles.input.IPositionHandler} that will be used as fallback to handle that actual movement of the
       * elements during the drag.
       * <p>
       * Note that the {@link yfiles.input.IPositionHandler} retrieved by the {@link yfiles.input.MoveInputMode#addQueryPositionHandlerListener QueryPositionHandler}
       * event has priority before this property. Only if the event is not {@link yfiles.input.QueryPositionHandlerEventArgs#handled} this property is used.
       * </p>
       * @type {yfiles.input.IPositionHandler}
       */
      positionHandler:yfiles.input.IPositionHandler;
      /**
       * Gets or sets the cursor to use during the move.
       * @type {yfiles.view.Cursor}
       */
      moveCursor:yfiles.view.Cursor;
      /**
       * The {@link yfiles.input.MoveInputMode#snapContext} which manages snapping model items to certain coordinates (for instance, other items).
       * <p>
       * If set to <code>null</code> (the default) this input mode tries to obtain the {@link yfiles.input.MoveInputMode#snapContext} from the {@link yfiles.input.IInputModeContext}. To explicitly
       * disable snapping, a {@link yfiles.input.MoveInputMode#snapContext} implementation that does nothing has to be set to this instance.
       * </p>
       * @type {yfiles.input.SnapContext}
       */
      snapContext:yfiles.input.SnapContext;
      /**
       * Adds the given listener for the <code>QueryPositionHandler</code> event that occurs when a drag is recognized for the
       * {@link yfiles.input.MoveInputMode}.
       * <p>
       * If the event args are {@link yfiles.input.QueryPositionHandlerEventArgs#handled}, the {@link yfiles.input.IPositionHandler} that is carried with the event args is used to actually move
       * the items. Otherwise the pre-set {@link yfiles.input.MoveInputMode#positionHandler} property is used.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeQueryPositionHandlerListener
       */
      addQueryPositionHandlerListener(listener:(sender:Object,evt:yfiles.input.QueryPositionHandlerEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>QueryPositionHandler</code> event that occurs when a drag is recognized for the
       * {@link yfiles.input.MoveInputMode}.
       * <p>
       * If the event args are {@link yfiles.input.QueryPositionHandlerEventArgs#handled}, the {@link yfiles.input.IPositionHandler} that is carried with the event args is used to actually move
       * the items. Otherwise the pre-set {@link yfiles.input.MoveInputMode#positionHandler} property is used.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addQueryPositionHandlerListener
       */
      removeQueryPositionHandlerListener(listener:(sender:Object,evt:yfiles.input.QueryPositionHandlerEventArgs)=>void):void;
      /**
       * Gets a value indicating whether a drag operation is currently in progress.
       * @type {boolean}
       */
      isDragging:boolean;
      /**
       * Returns the initial position where the dragging was initiated.
       * @type {yfiles.geometry.Point}
       */
      initialLocation:yfiles.geometry.Point;
      /**
       * Gets an immutable snapshot of the {@link yfiles.graph.IModelItem}s affected by the currently {@link yfiles.input.MoveInputMode#isDragging active} gesture.
       * <p>
       * When the gesture is {@link yfiles.input.MoveInputMode#addDragStartingListener starting} and the {@link yfiles.input.MoveInputMode#positionHandler} is {@link yfiles.input.IDragHandler#initializeDrag initialized}, the implementation can {@link yfiles.input.IModelItemCollector#add register} the affected item(s) through the {@link yfiles.input.IModelItemCollector}
       * instance that is bound to the {@link yfiles.input.MoveInputMode#createPositionHandlerInputModeContext context} available via its {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * <p>
       * Client code can register with the {@link yfiles.input.MoveInputMode#addDragStartedListener DragStarted} event, as
       * well as the {@link yfiles.input.MoveInputMode#addDragFinishedListener DragFinished} event to get notified of the
       * elements that may be or have been affected respectively by this input mode.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IModelItem>}
       */
      affectedItems:yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDragFinishingListener
       */
      addDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDragFinishingListener
       */
      removeDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDragFinishedListener
       */
      addDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDragFinishedListener
       */
      removeDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDragStartingListener
       */
      addDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDragStartingListener
       */
      removeDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDragStartedListener
       */
      addDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDragStartedListener
       */
      removeDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDraggingListener
       */
      addDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDraggingListener
       */
      removeDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDraggedListener
       */
      addDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDraggedListener
       */
      removeDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDragCanceledListener
       */
      addDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDragCanceledListener
       */
      removeDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to add.
       * @see yfiles.input.MoveInputMode#removeDragCancelingListener
       */
      addDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveInputMode#addDragCancelingListener
       */
      removeDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event argument that is used to query the {@link yfiles.input.IPositionHandler} that is used to move the items while dragging via
     * {@link yfiles.input.MoveInputMode}.
     * @see yfiles.input.MoveInputMode
     * @see yfiles.input.MoveInputMode#addQueryPositionHandlerListener
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface QueryPositionHandlerEventArgs extends yfiles.input.InputModeEventArgs{}
    export class QueryPositionHandlerEventArgs {
      /**
       * Instantiates a new {@link yfiles.input.QueryPositionHandlerEventArgs}
       * @param context The {@link } of the operation.
       * @param queryLocation The initial location of the movement.
       */
      constructor(context:yfiles.input.IInputModeContext,queryLocation:yfiles.geometry.Point);
      /**
       * The {@link yfiles.input.IPositionHandler} to use for the movement of the items that are affected by the
       * {@link yfiles.input.MoveInputMode}.
       * @type {yfiles.input.IPositionHandler}
       */
      positionHandler:yfiles.input.IPositionHandler;
      /**
       * Gets the location of the movement (where the drag has started).
       * @type {yfiles.geometry.Point}
       */
      queryLocation:yfiles.geometry.Point;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.QueryPositionHandlerEventArgs} has been handled.
       * @type {boolean}
       */
      handled:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum InertiaPolicies{
      /**
       * Never move the viewport using inertia.
       */
      NEVER,
      /**
       * Move the viewport using inertia when the mouse has been used to drag the viewport.
       */
      MOUSE,
      /**
       * Move the viewport using inertia when touch has been used to drag the viewport.
       */
      TOUCH,
      /**
       * Always move the viewport using inertia no matter what input gesture has been used.
       */
      ALWAYS
    }
    /**
     * An {@link yfiles.input.IInputMode} implementation that can be used to grab and drag the viewport of the control it is installed
     * in.
     * <p>
     * This implementation will request the mutex as soon as it recognizes a drag with the mouse or touch. Zooming is done with
     * two fingers using the pinch zoom gesture. If the primary pointer is released during pinch, the secondary pointer is used
     * for viewpoint dragging. The pinch zoom then can be restarted by touching another pointer that is recognized as the
     * primary pointer.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.MoveViewportInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface MoveViewportInputMode extends Object,yfiles.input.IInputMode{}
    export class MoveViewportInputMode {
      /**
       * Creates a new instance of the move viewport mode
       */
      constructor();
      /**
       * Called to prepare moving the viewport when the mouse is pressed.
       * <p>
       * This will set the mouse cursor to {@link yfiles.input.MoveViewportInputMode#dragCursor}.
       * </p>
       * @protected
       */
      arm():void;
      /**
       * Cancels navigation.
       */
      cancel():void;
      /**
       * Called after moving the viewport is finished or canceled.
       * <p>
       * This will reset the mouse cursor.
       * </p>
       * @protected
       */
      disarm():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.MoveViewportInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Triggers the {@link yfiles.input.MoveViewportInputMode#addDragCanceledListener DragCanceled} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MoveViewportInputMode#addDragCancelingListener DragCanceling} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceling(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered once the drag has been finalized.
       * <p>
       * This method triggers the {@link yfiles.input.MoveViewportInputMode#addDragFinishedListener DragFinished}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinished(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered before the drag will be finalized.
       * <p>
       * This method triggers the {@link yfiles.input.MoveViewportInputMode#addDragFinishingListener DragFinishing}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinishing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.MoveViewportInputMode#addDraggedListener Dragged}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragged(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.MoveViewportInputMode#addDraggingListener Dragging}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragging(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MoveViewportInputMode#addDragStartedListener DragStarted} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarted(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.MoveViewportInputMode#addDragStartingListener DragStarting} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarting(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called after {@link yfiles.input.MoveViewportInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * 
       * @param context
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.MoveViewportInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.MoveViewportInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.MoveViewportInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets or sets the event recognizer that starts the 'move viewport' gesture.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      primaryDownRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that moves the viewport or zooms the canvas if a secondary device is down.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      primaryMoveRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that ends the 'move viewport' or 'pinch' gesture.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      primaryUpRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that starts the 'pinch' gesture
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      secondaryDownRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that zooms the canvas using the 'pinch' gesture or moves the viewport if no primary
       * device is down.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      secondaryMoveRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that ends the pinch zoom gesture or ends the move if no primary device is down.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      secondaryUpRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the factor that determines how much the recent movement data is taken into account for the calculation of
       * inertia speed and direction.
       * <p>
       * The default value is <code>0.02d</code>
       * </p>
       * @default 0.02
       * @type {number}
       */
      inertiaFactor:number;
      /**
       * Gets or sets the threshold that determines when inertia movement should stop.
       * <p>
       * The default value is <code>30</code>
       * </p>
       * @type {number}
       */
      inertiaThreshold:number;
      /**
       * The amount of damping that is applied to the inertia movement.
       * <p>
       * The default value is <code>0.15d</code>
       * </p>
       * @type {number}
       */
      inertiaDamping:number;
      /**
       * Gets or sets the mode that determines when to use inertia when dragging the viewport.
       * @default yfiles.input.InertiaPolicies.TOUCH
       * @type {yfiles.input.InertiaPolicies}
       */
      inertia:yfiles.input.InertiaPolicies;
      /**
       * Gets or sets a value indicating whether to allow the pinch zoom gesture.
       * @default true
       * @type {boolean}
       */
      allowPinchZoom:boolean;
      /**
       * The cursor to use during the dragging.
       * <p>
       * The default is {@link yfiles.view.Cursor#POINTER}.
       * </p>
       * @type {yfiles.view.Cursor}
       */
      dragCursor:yfiles.view.Cursor;
      /**
       * Gets or sets the "pressed" state recognizer.
       * <p>
       * This recognizer instance will be used to determine when the user begins the navigation gesture.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer.
       * <p>
       * This recognizer instance determines when the user is dragging the viewport.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer.
       * <p>
       * This instance determines when the user has finished dragging the viewport.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDraggingListener
       */
      addDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDraggingListener
       */
      removeDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDraggedListener
       */
      addDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDraggedListener
       */
      removeDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDragFinishingListener
       */
      addDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDragFinishingListener
       */
      removeDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDragFinishedListener
       */
      addDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDragFinishedListener
       */
      removeDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDragStartingListener
       */
      addDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDragStartingListener
       */
      removeDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDragStartedListener
       */
      addDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDragStartedListener
       */
      removeDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDragCanceledListener
       */
      addDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDragCanceledListener
       */
      removeDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to add.
       * @see yfiles.input.MoveViewportInputMode#removeDragCancelingListener
       */
      addDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.MoveViewportInputMode#addDragCancelingListener
       */
      removeDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * A composite {@link yfiles.input.IInputMode} implementation that additionally can deal with {@link yfiles.input.ConcurrencyController#exclusive} instances.
     * <p>
     * Instances of this class can {@link yfiles.input.MultiplexingInputMode#install} and {@link yfiles.input.MultiplexingInputMode#uninstall}
     * multiple {@link yfiles.input.IInputMode}s. Child input modes can be added using the {@link yfiles.input.MultiplexingInputMode#add}
     * method. By providing {@link yfiles.input.IInputMode#priority priorities} to the different modes, their {@link yfiles.input.IInputMode#install installation} order can be influenced. Input modes with
     * lower priorities are handled earlier.
     * </p>
     * <p>
     * Child input modes can run exclusively while they hold the mutex. Other exclusive input modes are temporarily {@link yfiles.input.ConcurrencyController#active deactivated}
     * until the mutex is released. This can be controlled by setting the {@link yfiles.input.ConcurrencyController#exclusive} property on the controller. Input modes that disable
     * the {@link yfiles.input.ConcurrencyController#exclusive} property will always run concurrently with all other input modes.
     * </p>
     * <p>
     * This class itself implements the {@link yfiles.input.IInputMode} interface so that hierarchies of instances of this class can
     * easily be built. This class will request the input mutex if one of its child modes requests the mutex. Also if the
     * instance itself gets {@link yfiles.input.MultiplexingInputMode#enabled disabled}, it will stop or cancel the current {@link yfiles.input.MultiplexingInputMode#mutexOwner owner of the mutex}.
     * </p>
     * @see yfiles.input.MultiplexingInputMode#mutexOwner
     * @see yfiles.input.MultiplexingInputMode#add
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface MultiplexingInputMode extends Object,yfiles.input.IInputMode{}
    export class MultiplexingInputMode {
      /**
       * Creates an instance with no initial modes.
       */
      constructor();
      /**
       * Adds the given <code>mode</code>.
       * <p>
       * The input modes will be ordered according to their {@link yfiles.input.IInputMode#priority}: Input modes with lower priority will be {@link yfiles.input.IInputMode#install installed} earlier.
       * </p>
       * <p>
       * Input modes will run exclusively if the {@link yfiles.input.ConcurrencyController#exclusive} property of their {@link yfiles.input.IInputMode#install installed controller} is set to <code>true</code>. Otherwise they can not
       * and will not be deactivated if another {@link yfiles.input.IInputMode} acquires the mutex.
       * </p>
       * @param mode The input mode to add to this mode.
       * @throws {Stubs.Exceptions.ArgumentError} If the same <code>mode</code> has is already added to this instance.
       */
      add(mode:yfiles.input.IInputMode):void;
      /**
       * Adjusts the cursor of the {@link yfiles.view.CanvasComponent} according to the current input mutex owner or the first mode in
       * the list whose {@link yfiles.input.ConcurrencyController} returns a non-null {@link yfiles.input.ConcurrencyController#preferredCursor}.
       * <p>
       * This method will set {@link yfiles.input.MultiplexingInputMode#defaultCursor} as the current cursor if no other {@link yfiles.input.ConcurrencyController#preferredCursor} has been specified.
       * </p>
       */
      adjustCursor():void;
      /**
       * Cancels all modes.
       */
      cancel():void;
      /**
       * Callback method that will be used by the {@link yfiles.input.MultiplexingInputMode#createChildInputModeContext child context's} {@link yfiles.graph.ILookup#lookup} method.
       * @param type The type argument passed to {@link #lookup}.
       * @returns The result of the lookup query, or <code>null</code>.
       * @see yfiles.input.MultiplexingInputMode#createChildInputModeContext
       * @protected
       */
      childInputModeContextLookup(type:yfiles.lang.Class):Object;
      /**
       * Yields an {@link yfiles.input.IInputModeContext} for the child modes of this mode.
       * <p>
       * This method is called during {@link yfiles.input.MultiplexingInputMode#install installation} to create a new context for the child modes and can be used by client code to
       * obtain a suitable context object. The {@link yfiles.input.MultiplexingInputMode#inputModeContext} property is already set, when this method is called. {@link yfiles.graph.ILookup#lookup}
       * calls for the created context will be resolved by this instance's {@link yfiles.input.MultiplexingInputMode#childInputModeContextLookup}
       * method.
       * </p>
       * @returns A new instance that delegates to the {@link #inputModeContext parent's context.}
       * @protected
       */
      createChildInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Helper method that yields a suitably configured {@link yfiles.input.InputModeEventArgs} for this input mode.
       * @param context An input mode context that is available in the {@link }.
       * <p>
       * Can be <code>null</code> in which case a new context for this instance is created automatically.
       * </p>
       * @returns An input mode event argument that is configured for this instance.
       * @protected
       */
      createInputModeEventArgs(context:yfiles.input.IInputModeContext):yfiles.input.InputModeEventArgs;
      /**
       * Returns a list of all modes managed by this instance in sorted order.
       * @returns A list of the modes.
       */
      getSortedModes():yfiles.collections.IList<yfiles.input.IInputMode>;
      /**
       * Performs one-time initialization of this instance.
       * <p>
       * This method should not be invoked by subclasses. This will be done automatically upon first {@link yfiles.input.MultiplexingInputMode#install installation} of this mode.
       * </p>
       * <p>
       * This code will be executed only once per instance. The {@link yfiles.input.MultiplexingInputMode#inputModeContext} property will be <code>null</code> when this code is executed. This method
       * should not be used to install this mode into a specific canvas. Subclasses should always call <code>base.Initialize()</code> first.
       * </p>
       * @see yfiles.input.MultiplexingInputMode#install
       * @protected
       */
      initialize():void;
      /**
       * 
       * @param context
       * @param controller
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Convenience method for subclass implementations that invalidates the canvas this mode is currently installed in.
       * @protected
       */
      invalidate():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * This implementation reenabled previously disabled concurrent child modes.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * This implementation sets the {@link yfiles.input.ConcurrencyController#preferredCursor} property to <code>null</code> and {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if the mutex is currently owned by this instance.
       * Also, all concurrent child modes will be disabled.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Removes the given mode from this compound mode.
       * @param mode The mode to remove.
       */
      remove(mode:yfiles.input.IInputMode):void;
      /**
       * Called when the {@link yfiles.input.IInputMode#priority} of an {@link yfiles.input.MultiplexingInputMode#add installed} sub mode has changed.
       * <p>
       * This implementation {@link yfiles.input.MultiplexingInputMode#uninstall uninstalls} all sub modes and then re-{@link yfiles.input.MultiplexingInputMode#install installs} them according to their new priorities.
       * </p>
       * @protected
       */
      subModePriorityChanged():void;
      /**
       * Tries to stop all modes.
       * @returns 
       */
      tryStop():boolean;
      /**
       * 
       * @param context
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.MultiplexingInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.MultiplexingInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.MultiplexingInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets the {@link yfiles.input.IInputMode} that currently owns the mutex.
       * @type {yfiles.input.IInputMode}
       */
      mutexOwner:yfiles.input.IInputMode;
      /**
       * Gets or sets the cursor to use whenever no child mode prefers a different cursor.
       * <p>
       * The default is <code>null</code>
       * </p>
       * @see yfiles.input.MultiplexingInputMode#adjustCursor
       * @default null
       * @type {yfiles.view.Cursor}
       */
      defaultCursor:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} that can be used for an overview {@link yfiles.view.CanvasComponent}.
     * <p>
     * This mode will add navigation capabilities to the control it is installed in.
     * </p>
     * @see yfiles.input.OverviewInputMode#canvasComponent
     * @see yfiles.input.OverviewInputMode#handleInputMode
     * @see yfiles.input.OverviewInputMode#moveInputMode
     * @see yfiles.input.OverviewInputMode#keyboardInputMode
     * @see yfiles.input.OverviewInputMode#clickInputMode
     * @see yfiles.input.OverviewInputMode#tapInputMode
     * @class
     * @extends {yfiles.input.MultiplexingInputMode}
     */
    export interface OverviewInputMode extends yfiles.input.MultiplexingInputMode{}
    export class OverviewInputMode {
      /**
       * Creates a new instance.
       */
      constructor();
      /**
       * Factory method for the {@link yfiles.input.OverviewInputMode#clickInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.OverviewInputMode#clickInputMode} property.
       * </p>
       * @returns a new instance of ClickInputMode
       * @protected
       */
      createClickInputMode():yfiles.input.ClickInputMode;
      /**
       * Factory method for the {@link yfiles.input.OverviewInputMode#handleInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.OverviewInputMode#handleInputMode} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createHandleInputMode():yfiles.input.HandleInputMode;
      /**
       * Factory method for the {@link yfiles.input.OverviewInputMode#keyboardInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.OverviewInputMode#keyboardInputMode} property.
       * </p>
       * @returns a new instance of {@link }.
       * @protected
       */
      createKeyboardInputMode():yfiles.input.KeyboardInputMode;
      /**
       * Factory method for the {@link yfiles.input.OverviewInputMode#moveInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.OverviewInputMode#moveInputMode} property.
       * </p>
       * @returns a new instance of MoveInputMode
       * @protected
       */
      createMoveInputMode():yfiles.input.MoveInputMode;
      /**
       * Factory method for the {@link yfiles.input.OverviewInputMode#tapInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.OverviewInputMode#tapInputMode} property.
       * </p>
       * @returns a new instance of TapInputMode
       * @protected
       */
      createTapInputMode():yfiles.input.TapInputMode;
      /**
       * Callback method that is used by {@link yfiles.input.OverviewInputMode#createKeyboardInputMode} to determine which of the
       * built-in {@link yfiles.input.ICommand}s to install.
       * <p>
       * This implementation unconditionally returns <code>true</code>, subclasses may override this method to adjust the behavior.
       * </p>
       * @param command The command to install.
       * @returns Whether to install this command.
       * @protected
       */
      shouldInstallCommand(command:yfiles.input.ICommand):boolean;
      /**
       * Updates the {@link yfiles.view.CanvasComponent#zoom zoom} and {@link yfiles.view.CanvasComponent#viewPoint viewpoint} of the overview control.
       * <p>
       * This method is called when the {@link yfiles.view.CanvasComponent#contentRect graph bounds} of the {@link yfiles.input.OverviewInputMode#canvasComponent } have changed. The {@link yfiles.input.OverviewInputMode#margins margins} should be respected.
       * </p>
       * @protected
       */
      updateVisibleArea():void;
      /**
       * Determines whether the the canvas this mode is installed in should automatically be invalidated if the client canvas
       * gets invalidated.
       * <p>
       * Automatic invalidation will be deferred to avoid too frequent updates of the overview.
       * </p>
       * <p>
       * To manually update the overview when this property is <code>false</code>, call the {@link yfiles.view.CanvasComponent#invalidate} method on
       * the {@link yfiles.view.GraphOverviewComponent}.
       * </p>
       * @type {boolean}
       */
      autoInvalidate:boolean;
      /**
       * A resource key that should yield a {@link yfiles.view.IVisualTemplate} that will be used by this mode render the viewport.
       * <p>
       * When explicitly setting a {@link yfiles.input.OverviewInputMode#template}, this key will not be used. This is only a fallback if {@link yfiles.input.OverviewInputMode#template} is <code>null</code>.
       * </p>
       * @const
       * @static
       * @type {string}
       */
      static VIEWPORT_TEMPLATE_KEY:string;
      /**
       * Gets the {@link yfiles.input.HandleInputMode}.
       * <p>
       * This mode is responsible for handling the single handle that allows to resize the viewport rectangle.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.OverviewInputMode#createHandleInputMode}
       * will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>10</code>.
       * </p>
       * @type {yfiles.input.HandleInputMode}
       */
      handleInputMode:yfiles.input.HandleInputMode;
      /**
       * Gets the {@link yfiles.input.MoveInputMode}.
       * <p>
       * This mode is responsible for moving the viewport rectangle.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.OverviewInputMode#createMoveInputMode}
       * will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>20</code>.
       * </p>
       * @type {yfiles.input.MoveInputMode}
       */
      moveInputMode:yfiles.input.MoveInputMode;
      /**
       * Gets the {@link yfiles.input.OverviewInputMode#clickInputMode}.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.OverviewInputMode#createClickInputMode}
       * will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>0</code>.
       * </p>
       * @type {yfiles.input.ClickInputMode}
       */
      clickInputMode:yfiles.input.ClickInputMode;
      /**
       * Gets the {@link yfiles.input.OverviewInputMode#tapInputMode}.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.OverviewInputMode#createTapInputMode}
       * will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>30</code>.
       * </p>
       * @type {yfiles.input.TapInputMode}
       */
      tapInputMode:yfiles.input.TapInputMode;
      /**
       * Gets the {@link yfiles.input.KeyboardInputMode}.
       * <p>
       * This mode handles all keyboard interaction gestures with the overview control.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.OverviewInputMode#createKeyboardInputMode}
       * will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>0</code>.
       * </p>
       * @type {yfiles.input.KeyboardInputMode}
       */
      keyboardInputMode:yfiles.input.KeyboardInputMode;
      /**
       * Gets the list of commands that are available in this instance.
       * <p>
       * Removing commands from this collection also removes the command bindings registered by this instance.
       * </p>
       * <p>
       * Add supported commands to make them available in this instance.
       * </p>
       * <p>
       * Supported commands are
       * </p>
       * <ul>
       * <li>{@link yfiles.input.ICommand#MOVE_LEFT}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_RIGHT}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_UP}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#ZOOM}</li>
       * <li>{@link yfiles.input.ICommand#INCREASE_ZOOM}</li>
       * <li>{@link yfiles.input.ICommand#DECREASE_ZOOM}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_UP}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_LEFT}</li>
       * <li>{@link yfiles.input.ICommand#SCROLL_PAGE_RIGHT}</li>
       * <li>{@link yfiles.input.ICommand#FIT_CONTENT}</li>
       * <li>{@link yfiles.input.ICommand#UPDATE_CONTENT_RECT}</li>
       * </ul>
       * @type {yfiles.collections.ICollection.<yfiles.input.ICommand>}
       */
      availableCommands:yfiles.collections.ICollection<yfiles.input.ICommand>;
      /**
       * Gets or sets the canvas this canvas should use to navigate.
       * @type {yfiles.view.CanvasComponent}
       */
      canvasComponent:yfiles.view.CanvasComponent;
      /**
       * Gets or sets the template that is used for the visualization of the marquee rectangle.
       * @type {yfiles.view.IVisualTemplate}
       */
      template:yfiles.view.IVisualTemplate;
      /**
       * Determines whether the mouse wheel can be used to perform zooming in the {@link yfiles.input.OverviewInputMode#canvasComponent}.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoMouseWheelZoom:boolean;
      /**
       * Gets and sets the insets in view coordinates that should be used by the {@link yfiles.input.OverviewInputMode#updateVisibleArea}
       * operation.
       * <p>
       * This influences the amount of visible whitespace in the view coordinate system around the graph after the {@link yfiles.view.CanvasComponent#contentRect content rect}
       * of the {@link yfiles.input.OverviewInputMode#canvasComponent} has changed. The default value is <code>(5,5,5,5)</code>.
       * </p>
       * @type {yfiles.geometry.Insets}
       */
      margins:yfiles.geometry.Insets;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of {@link yfiles.input.PopulateMenuEventArgs} that carries the {@link yfiles.input.PopulateItemContextMenuEventArgs.<TModelItem>#item} for which the context menu should be
     * populated.
     * Type parameter <code>TModelItem</code>: The type of the {@link yfiles.input.PopulateItemContextMenuEventArgs.<TModelItem>#item model item}.
     * @class
     * @extends {yfiles.input.PopulateMenuEventArgs}
     * @template TModelItem
     */
    export interface PopulateItemContextMenuEventArgs<TModelItem> extends yfiles.input.PopulateMenuEventArgs{}
    export class PopulateItemContextMenuEventArgs<TModelItem> {
      /**
       * Initializes a new instance of the {@link yfiles.input.PopulateItemContextMenuEventArgs.<TModelItem>} class.
       * @param context The context to populate the menu for.
       * @param queryLocation The query location.
       * @param item The item for which the context menu should be populated, may be <code>null</code>.
       */
      constructor(context:yfiles.input.IInputModeContext,queryLocation:yfiles.geometry.Point,item:TModelItem);
      /**
       * Gets the item for which the tool tip is queried.
       * @type {TModelItem}
       */
      item:TModelItem;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of {@link yfiles.input.ToolTipQueryEventArgs} that carries the {@link yfiles.input.QueryItemToolTipEventArgs.<TModelItem>#item} for which the tool tip is queried.
     * Type parameter <code>TModelItem</code>: The type of the {@link yfiles.input.QueryItemToolTipEventArgs.<TModelItem>#item model item}.
     * @class
     * @extends {yfiles.input.ToolTipQueryEventArgs}
     * @template TModelItem
     */
    export interface QueryItemToolTipEventArgs<TModelItem> extends yfiles.input.ToolTipQueryEventArgs{}
    export class QueryItemToolTipEventArgs<TModelItem> {
      /**
       * Initializes a new instance of the {@link yfiles.input.QueryItemToolTipEventArgs.<TModelItem>} class.
       * @param context The context to yield the tool tip for.
       * @param queryLocation The query location.
       * @param item The item for which the tool tip is queried..
       */
      constructor(context:yfiles.input.IInputModeContext,queryLocation:yfiles.geometry.Point,item:TModelItem);
      /**
       * Gets the item for which the tool tip is queried.
       * @type {TModelItem}
       */
      item:TModelItem;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple convenience implementation of an {@link yfiles.input.IReshapeHandleProvider} that returns handles that modify a
     * {@link yfiles.geometry.IMutableRectangle}.
     * @class
     * @extends {yfiles.input.ReshapeHandleProviderBase}
     */
    export interface RectangleReshapeHandleProvider extends yfiles.input.ReshapeHandleProviderBase{}
    export class RectangleReshapeHandleProvider {
      /**
       * Creates a default instance that provide handles for each of the positions given for the rectangle.
       * @param rectangle The rectangle to create handles for.
       * @param [handlePositions=yfiles.input.HandlePositions.BORDER] A bitwise combination of all handle positions this instance should provide a handle for.
       */
      constructor(rectangle:yfiles.geometry.IMutableRectangle,handlePositions?:yfiles.input.HandlePositions);
      /**
       * The rectangle to read the current state from.
       * @protected
       * @type {yfiles.geometry.IMutableRectangle}
       */
      rectangle:yfiles.geometry.IMutableRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * Abstract base class implementation of the {@link yfiles.input.IReshapeHandleProvider} interface that has properties to restrict
     * the size and area of a reshapeable.
     * @class
     * @implements {yfiles.input.IReshapeHandleProvider}
     */
    export interface ReshapeHandleProviderBase extends Object,yfiles.input.IReshapeHandleProvider{}
    export class ReshapeHandleProviderBase {
      constructor();
      /**
       * Returns the {@link yfiles.input.ReshapeHandleProviderBase#handlePositions} property.
       * @param context
       * @returns A bitwise combination of all handle positions this instance provides a handle for if queried in
       * {@link #getHandle}.
       */
      getAvailableHandles(context:yfiles.input.IInputModeContext):yfiles.input.HandlePositions;
      /**
       * Provides a {@link yfiles.input.IHandle} that uses the rectangle and reshapeable instance bound to this instance to perform the
       * actual reshaping.
       * <p>
       * This method may be called for each possible single position contained in the set as returned by
       * {@link yfiles.input.ReshapeHandleProviderBase#getAvailableHandles}.
       * </p>
       * @param context The context for which the handles are queried.
       * @param position The position to provide an instance for.
       * @returns A handle implementation.
       * @abstract
       */
      getHandle(context:yfiles.input.IInputModeContext,position:yfiles.input.HandlePositions):yfiles.input.IHandle;
      /**
       * Gets or sets the minimum size allowed for the reshapeable.
       * <p>
       * The default value is {@link yfiles.geometry.Size#ZERO}.
       * </p>
       * @see yfiles.input.RectangleHandle#minimumSize
       * @type {yfiles.geometry.ISize}
       */
      minimumSize:yfiles.geometry.ISize;
      /**
       * Gets or sets the minimum rectangular area that needs to be contained in the reshapeable.
       * <p>
       * The default value is {@link yfiles.geometry.Size#ZERO}.
       * </p>
       * @see yfiles.input.RectangleHandle#minimumEnclosedArea
       * @type {yfiles.geometry.IRectangle}
       */
      minimumEnclosedArea:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the maximum size allowed for the reshapeable.
       * <p>
       * The default value is {@link yfiles.geometry.Size#INFINITE}.
       * </p>
       * @see yfiles.input.RectangleHandle#maximumSize
       * @type {yfiles.geometry.ISize}
       */
      maximumSize:yfiles.geometry.ISize;
      /**
       * Gets or sets the bitwise combination of all handle positions this instance provides a handle for.
       * @see yfiles.input.ReshapeHandleProviderBase#getAvailableHandles
       * @see yfiles.input.RectangleReshapeHandleProvider#getHandle
       * @type {yfiles.input.HandlePositions}
       */
      handlePositions:yfiles.input.HandlePositions;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments that are used when modifying a {@link yfiles.view.ISelectionModel.<T>}
     * Type parameter <code>T</code>: The type of the items in the {@link yfiles.view.ISelectionModel.<T>}.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     * @template T
     */
    export interface SelectionEventArgs<T extends yfiles.graph.IModelItem> extends yfiles.input.InputModeEventArgs{}
    export class SelectionEventArgs<T extends yfiles.graph.IModelItem> {
      /**
       * Creates a new instance.
       * @param context The context the event with these event args is dispatched.
       * @param selection The selection that is modified.
       */
      constructor(context:yfiles.input.IInputModeContext,selection:yfiles.view.ISelectionModel<T>);
      /**
       * Gets the selection these event arguments refer to.
       * @type {yfiles.view.ISelectionModel.<T>}
       */
      selection:yfiles.view.ISelectionModel<T>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Manages interactive snapping of elements to other elements during drag operations like movements.
     * <p>
     * This class should be used by {@link yfiles.input.IInputMode} implementations and similar code as follows:
     * </p>
     * <ul>
     * <li>To initialize the operation {@link yfiles.input.SnapContext#initializeDrag} needs to be called.</li>
     * <li>
     * Method {@link yfiles.input.SnapContext#dragInitialized} needs to be called after the mode has performed all of its own
     * initialization.
     * </li>
     * <li>
     * Then a series of zero or more invocations {@link yfiles.input.SnapContext#handleMove} needs to be called for each incremental
     * movement of the user's input device.
     * </li>
     * <li>
     * After {@link yfiles.input.SnapContext#handleMove} has been called, the mode should perform the move and report back to the
     * instance by calling the {@link yfiles.input.SnapContext#dragged} method.
     * </li>
     * <li>If the operation is canceled by the user {@link yfiles.input.SnapContext#cancelDrag} should be called.</li>
     * <li>If the operation is finalized the {@link yfiles.input.SnapContext#dragFinished} method should be called.</li>
     * </ul>
     * @see yfiles.input.HandleInputMode#snapContext
     * @see yfiles.input.MoveInputMode#snapContext
     * @see yfiles.input.SnapContext#createSnapResultsModelManager
     * @class
     */
    export interface SnapContext extends Object{}
    export class SnapContext {
      /**
       * Initializes a new instance of the {@link yfiles.input.SnapContext} class.
       */
      constructor();
      /**
       * Called when a drag has been canceled.
       * <p>
       * Clears all temporary lists and sets {@link yfiles.input.SnapContext#isInitialized} back to false, as well as performs {@link yfiles.input.SnapContext#cleanUp} and finally
       * triggers the {@link yfiles.input.SnapContext#addCleanedUpListener CleanedUp} event.
       * </p>
       */
      cancelDrag():void;
      /**
       * Performs clean up procedures.
       * <p>
       * This is called in response to {@link yfiles.input.SnapContext#cancelDrag} and {@link yfiles.input.SnapContext#dragFinished} as well as
       * initially during {@link yfiles.input.SnapContext#initializeDrag}.
       * </p>
       * @protected
       */
      cleanUp():void;
      /**
       * Helper method used by {@link yfiles.input.SnapContext#createSnapResultsModelManager} to create the {@link yfiles.view.ICanvasObjectDescriptor}
       * for the {@link yfiles.input.SnapResult} type.
       * @returns An instance that can install {@link }s in a {@link }.
       */
      createSnapResultCanvasObjectDescriptor():yfiles.view.ICanvasObjectDescriptor;
      /**
       * Helper method that creates the a {@link yfiles.view.CollectionModelManager.<T>} that can be used to present the current {@link yfiles.input.SnapResult}s in the provided
       * {@link yfiles.view.CanvasComponent}.
       * @param canvas The control to initialize the {@link } with.
       * @returns The model manager that manages the visual representation of the {@link }s.
       */
      createSnapResultsModelManager(canvas:yfiles.view.CanvasComponent):yfiles.view.CollectionModelManager<yfiles.input.SnapResult>;
      /**
       * Called when a drag has been successfully finished.
       * <p>
       * Clears all temporary lists and sets {@link yfiles.input.SnapContext#isInitialized} back to false. Also the {@link yfiles.input.SnapContext#cleanUp} method is invoked and the
       * {@link yfiles.input.SnapContext#addCleanedUpListener CleanedUp} event is triggered.
       * </p>
       * @param newLocation The current mouse location.
       * @param snappingDisabled If set to <code>true</code> snapping is temporarily disabled. This will remove the snap line visualizations and return a {@link }
       * with the unchanged coordinates and the {@link #NOT_SNAPPED} {@link #snapType}.
       * @returns A {@link } that specifies the adjusted coordinates and how they have been adjusted. The unchanged
       * coordinates if snapping has been temporarily disabled by setting <code>snappingDisabled</code> to <code>true</code>.
       */
      dragFinished(newLocation:yfiles.geometry.Point,snappingDisabled:boolean):yfiles.input.SnapState;
      /**
       * This method needs to be called by the client {@link yfiles.input.IInputMode} that {@link yfiles.input.SnapContext#initializeDrag initialized} this instance after it has
       * performed the actual move operation with the adjusted coordinates after a call to {@link yfiles.input.SnapContext#handleMove}.
       * <p>
       * Most of the client code implementations will just call this method with the exact values returned by
       * {@link yfiles.input.SnapContext#handleMove}. This method does not need to be called if snapping has been temporarily disabled by
       * calling {@link yfiles.input.SnapContext#handleMove} with its <code>snappingDisabled</code> parameter set to <code>true</code>.
       * </p>
       * @param mouseLocation The actual mouse location.
       * @param finalSnapState The final snap state that contains the location that has been used by the client code for the move and how it has been
       * snapped.
       */
      dragged(mouseLocation:yfiles.geometry.Point,finalSnapState:yfiles.input.SnapState):void;
      /**
       * Collects a list of snap lines.
       * <p>
       * Snap lines represent possible horizontal or vertical lines to which a moved item can "snap".
       * </p>
       * <p>
       * This method is called at the start of {@link yfiles.input.MoveInputMode}'s or
       * {@link yfiles.input.HandleInputMode}'s gesture after all items to be moved or reshaped have been added.
       * </p>
       */
      dragInitialized():void;
      /**
       * Handles a move.
       * <p>
       * This method returns the adjusted (mouse) coordinates. It also manages the collection of {@link yfiles.input.SnapResult}s. If
       * snapping should be temporarily (for a mouse move) disabled, <code>snappingDisabled</code> has to be set to <code>true</code>.
       * </p>
       * @param newLocation The current mouse location.
       * @param snappingDisabled If set to <code>true</code> snapping is temporarily disabled. This will remove the snap line visualizations and return a {@link }
       * with the unchanged coordinates and the {@link #snapType} {@link #NOT_SNAPPED}.
       * @returns The adjusted coordinates and how they have been adjusted. The unchanged coordinates if snapping has been temporarily
       * disabled by setting <code>snappingDisabled</code> to <code>true</code>.
       */
      handleMove(newLocation:yfiles.geometry.Point,snappingDisabled:boolean):yfiles.input.SnapState;
      /**
       * Called by {@link yfiles.input.IInputMode} implementations when an interactive drag is started.
       * <p>
       * Updates {@link yfiles.input.SnapContext#isInitializing}, {@link yfiles.input.SnapContext#originalLocation}, and {@link yfiles.input.SnapContext#currentInputModeContext} and then triggers the {@link yfiles.input.SnapContext#addInitializingListener Initializing} event.
       * </p>
       * @param context The context in which the interactive drag is started.
       * @param originalLocation The original location of the mouse.
       * @throws {Stubs.Exceptions.InvalidOperationError} If this context is already {@link yfiles.input.SnapContext#isInitialized initialized}.
       */
      initializeDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Raises the {@link yfiles.input.SnapContext#cleanUp} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onCleanUp(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.SnapContext#addCollectSnapResultsListener CollectSnapResults} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onCollectSnapResults(evt:yfiles.input.CollectSnapResultsEventArgs):void;
      /**
       * Raises the {@link yfiles.input.SnapContext#addInitializedListener Initialized} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onInitialized(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.SnapContext#addInitializingListener Initializing} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onInitializing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Chooses the current {@link yfiles.input.SnapResult}s from a temporary list.
       * <p>
       * Chooses from a list of possible results the most important ones. Also maintains the collection of snap results.
       * </p>
       * @param snapResults A temporary list of suggested {@link }s.
       * @param originalLocation The point in world coordinates where the mouse move started.
       * @param newLocation The current mouse location.
       * @returns The adjusted coordinates and how they have been adjusted.
       * @protected
       */
      processSnapResults(snapResults:yfiles.collections.List<yfiles.input.SnapResult>,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):yfiles.input.SnapState;
      /**
       * Helper method that wraps the given <code>context</code> so that a {@link yfiles.graph.ILookup#lookup} query on the wrapped context for the {@link yfiles.input.SnapContext}
       * type yields this instance.
       * @param context The context to wrap and delegate all calls to.
       * @returns A modified instance that yields this instance if it is queried for the {@link } type.
       */
      wrapContext(context:yfiles.input.IInputModeContext):yfiles.input.IInputModeContext;
      /**
       * Adds the given listener for the <code>CollectSnapResults</code> event that occurs while {@link yfiles.input.IDragHandler#handleMove moving} items and handles.
       * <p>
       * The moving model items must register for this event, preferably in the {@link yfiles.input.IDragHandler#initializeDrag} method
       * of the {@link yfiles.input.IDragHandler}.
       * </p>
       * <p>
       * The event handler has to add {@link yfiles.input.SnapResult}s for the moving object to the list. The
       * {@link yfiles.input.IDragHandler}s of the moved items have to take care themselves whether to add a handler to the list
       * depending on the their settings.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.SnapContext#removeCollectSnapResultsListener
       */
      addCollectSnapResultsListener(listener:(sender:Object,evt:yfiles.input.CollectSnapResultsEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CollectSnapResults</code> event that occurs while {@link yfiles.input.IDragHandler#handleMove moving} items and handles.
       * <p>
       * The moving model items must register for this event, preferably in the {@link yfiles.input.IDragHandler#initializeDrag} method
       * of the {@link yfiles.input.IDragHandler}.
       * </p>
       * <p>
       * The event handler has to add {@link yfiles.input.SnapResult}s for the moving object to the list. The
       * {@link yfiles.input.IDragHandler}s of the moved items have to take care themselves whether to add a handler to the list
       * depending on the their settings.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.SnapContext#addCollectSnapResultsListener
       */
      removeCollectSnapResultsListener(listener:(sender:Object,evt:yfiles.input.CollectSnapResultsEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>CleanedUp</code> event that occurs when this instance has been cleaned up.
       * @param listener The listener to add.
       * @see yfiles.input.SnapContext#cleanUp
       * @see yfiles.input.SnapContext#removeCleanedUpListener
       */
      addCleanedUpListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CleanedUp</code> event that occurs when this instance has been cleaned up.
       * @param listener The listener to remove.
       * @see yfiles.input.SnapContext#cleanUp
       * @see yfiles.input.SnapContext#addCleanedUpListener
       */
      removeCleanedUpListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Initialized</code> event that occurs when this instance has been initialized.
       * @param listener The listener to add.
       * @see yfiles.input.SnapContext#initializeDrag
       * @see yfiles.input.SnapContext#removeInitializedListener
       */
      addInitializedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Initialized</code> event that occurs when this instance has been initialized.
       * @param listener The listener to remove.
       * @see yfiles.input.SnapContext#initializeDrag
       * @see yfiles.input.SnapContext#addInitializedListener
       */
      removeInitializedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Initializing</code> event that occurs when this instance is about to be initialized.
       * @param listener The listener to add.
       * @see yfiles.input.SnapContext#initializeDrag
       * @see yfiles.input.SnapContext#removeInitializingListener
       */
      addInitializingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Initializing</code> event that occurs when this instance is about to be initialized.
       * @param listener The listener to remove.
       * @see yfiles.input.SnapContext#initializeDrag
       * @see yfiles.input.SnapContext#addInitializingListener
       */
      removeInitializingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Gets or sets a value indicating whether to visualize the snap results.
       * @see yfiles.input.SnapContext#createSnapResultsModelManager
       * @type {boolean}
       */
      visualizeSnapResults:boolean;
      /**
       * Gets or sets a value indicating whether this instance is currently initializing.
       * @see yfiles.input.SnapContext#addInitializedListener
       * @see yfiles.input.SnapContext#addInitializingListener
       * @see yfiles.input.SnapContext#isInitialized
       * @type {boolean}
       */
      isInitializing:boolean;
      /**
       * Gets the current input mode context.
       * @protected
       * @type {yfiles.input.IInputModeContext}
       */
      currentInputModeContext:yfiles.input.IInputModeContext;
      /**
       * The collection of {@link yfiles.input.SnapResult}s.
       * <p>
       * This collection is automatically updated during moving the mouse.
       * </p>
       * @type {yfiles.collections.IObservableCollection.<yfiles.input.SnapResult>}
       */
      snapResults:yfiles.collections.IObservableCollection<yfiles.input.SnapResult>;
      /**
       * Whether the context is initialized.
       * <p>
       * The context is initialized between the calls to {@link yfiles.input.SnapContext#dragInitialized} and {@link yfiles.input.SnapContext#dragFinished}
       * or {@link yfiles.input.SnapContext#cancelDrag}.
       * </p>
       * @type {boolean}
       */
      isInitialized:boolean;
      /**
       * Sets the global snap distance, which is the maximum distance between the current mouse coordinates and the coordinates
       * to which the mouse will snap.
       * <p>
       * The distance is interpreted in view coordinates.
       * </p>
       * <p>
       * The default is 5.0d
       * </p>
       * @see yfiles.input.CollectSnapResultsEventArgs#snapDistance
       * @type {number}
       */
      snapDistance:number;
      /**
       * Gets the original location of the mouse at the time the gesture was initialized.
       * @see yfiles.input.SnapContext#initializeDrag
       * @type {yfiles.geometry.Point}
       */
      originalLocation:yfiles.geometry.Point;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.SnapContext} is enabled.
       * @type {boolean}
       */
      enabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is the abstract base class used by the implementers of {@link yfiles.input.SnapContext} to model the result of the mouse
     * being snapped to a certain location.
     * <p>
     * It carries a {@link yfiles.input.SnapResult#weight} and can be used to obtain a {@link yfiles.view.IVisualCreator} that will be included in the view if the result is
     * actually {@link yfiles.input.SnapResult#isSnapped snapped} after all other results with higher weight have been {@link yfiles.input.SnapResult#snap snapped}.
     * </p>
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface SnapResult extends Object,yfiles.lang.IComparable{}
    export class SnapResult {
      /**
       * Factory method that creates a {@link yfiles.input.SnapResult} to which a {@link yfiles.graph.ILabel} will snap, based on the given
       * <code>snapLine</code>.
       * @param layoutParameter The {@link } which represents the position to snap to.
       * @param weight The weight of this result. The higher the weight, the more important it is.
       * @param delta The value to add to the mouse coordinates to snap to this result.
       * @param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
       * {@link #weight} will be rendered.
       * @param snapLine The snap line this class would snap to or <code>null</code> if it doesn't snap to a <code>SnapLine</code>.
       * @param snappedCoordinates The coordinates at the moved item at which the drawn snap line should end.
       * @param movedObject The moved object for which this result is created.
       * @returns 
       * @static
       */
      static createLabelSnapResult(layoutParameter:yfiles.graph.ILabelModelParameter,weight:number,delta:yfiles.geometry.Point,tag:Object,snapLine:yfiles.input.SnapLine,snappedCoordinates:yfiles.geometry.Point,movedObject:Object):yfiles.input.SnapResult;
      /**
       * Factory method that creates a {@link yfiles.input.SnapResult} that snaps to a given location.
       * @param location The location to snap to.
       * @param weight The weight to assign to the result.
       * @param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
       * {@link #weight} will be rendered.
       * @param visualCreator The visual creator to use.
       * @param snapType The type of the snapping operation to apply.
       * @returns A result that snaps to the provided location using the provided mode.
       * @static
       */
      static createPointSnapResult(location:yfiles.geometry.Point,weight:number,tag:Object,visualCreator:yfiles.view.IVisualCreator,snapType:yfiles.input.SnapTypes):yfiles.input.SnapResult;
      /**
       * Factory method that creates a {@link yfiles.input.SnapResult} which snaps to a given size through a resize operation.
       * @param weight The weight of this result. The higher the weight, the more important it is.
       * @param delta The value to add to the mouse coordinates to snap to this result.
       * @param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
       * {@link #weight} will be rendered.
       * @param rectangles The rectangles that have the same width or height as the node will. The rectangle are used for visualization and should
       * include all objects for which a visualization should be displayed. Typically, these are all objects which have the same
       * size and the object which is currently reshaped.
       * @param size The target size to which the resized object will snap.
       * @param horizontal Indicates whether width or height should snap. <code>true</code> if the width should snap.
       * @returns 
       * @static
       */
      static createResizeSnapResult(weight:number,delta:number,tag:Object,rectangles:yfiles.collections.IEnumerable<yfiles.geometry.IRectangle>,size:number,horizontal:boolean):yfiles.input.SnapResult;
      /**
       * Factory method that creates a {@link yfiles.input.SnapResult} which represents a {@link yfiles.input.SnapLine} to which the current
       * moved item will potentially snap.
       * @param weight The weight of this result. The higher the weight, the more important it is.
       * @param delta The value to add to the mouse coordinates to snap to this result.
       * @param tag The tag associated with this result. If more than one result uses the same tag (not <code>null</code>), only the one with the highest
       * {@link #weight} will be rendered.
       * @param snapLine The snap line this class would snap to or <code>null</code> if it doesn't snap to a <code>SnapLine</code>.
       * @param snappedLocation The coordinates at the moved item at which the drawn snap line should end.
       * @param movedObject The moved object for which this result is created.
       * @returns 
       * @static
       */
      static createSnapLineSnapResult(weight:number,delta:yfiles.geometry.Point,tag:Object,snapLine:yfiles.input.SnapLine,snappedLocation:yfiles.geometry.Point,movedObject:Object):yfiles.input.SnapResult;
      /**
       * Returns a {@link yfiles.view.IVisualCreator} instance that can be used to create a {@link yfiles.view.Visual} for this result.
       * @returns A visual creator which creates a {@link } which depicts this snap result.
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       * @abstract
       */
      getVisualCreator():yfiles.view.IVisualCreator;
      /**
       * Checks whether this instance is still snapped given the final mouse location.
       * <p>
       * This implementation simply returns <code>false</code>.
       * </p>
       * @param unsnappedLocation The unsnapped location.
       * @param finalSnapState The final snap state that has been used by the client.
       * @returns Whether the location are still snapped for this instance.
       */
      isSnapped(unsnappedLocation:yfiles.geometry.Point,finalSnapState:yfiles.input.SnapState):boolean;
      /**
       * Core method that performs the actual snapping.
       * <p>
       * This implementation does nothing.
       * </p>
       * @param unsnappedLocation The location prior to the snapping.
       * @param currentSnapState The currently snapped location and the state of the snapping. This instance can be modified by subclasses.
       */
      snap(unsnappedLocation:yfiles.geometry.Point,currentSnapState:yfiles.input.SnapState):void;
      /**
       * Gets the weight of this result.
       * <p>
       * Higher weights represent more important results.
       * </p>
       * @type {number}
       */
      weight:number;
      /**
       * Gets the tag associated with this result.
       * <p>
       * If more than one result uses the same tag (not <code>null</code>), only the one with the highest {@link yfiles.input.SnapResult#weight} will be rendered.
       * </p>
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets a {@link yfiles.input.SnapResult} representing that there is no snapping of the x or y coordinate.
       * @static
       * @type {yfiles.input.SnapResult}
       */
      static NULL_RESULT:yfiles.input.SnapResult;
      static $class:yfiles.lang.Class;
    }
    export enum SnapTypes{
      /**
       * Indicates that the location has been snapped to its x value.
       * <p>
       * Constant for use in {@link yfiles.input.SnapResult#snap} and {@link yfiles.input.SnapResult#isSnapped}. The methods can take a bitwise
       * combination of this field and {@link yfiles.input.SnapTypes#SNAPPED_Y}, {@link yfiles.input.SnapTypes#SNAPPED_X_Y} and
       * {@link yfiles.input.SnapTypes#NOT_SNAPPED}.
       * </p>
       */
      SNAPPED_X,
      /**
       * Indicates that the location has been snapped to its y value.
       * <p>
       * Constant for use in {@link yfiles.input.SnapResult#snap} and {@link yfiles.input.SnapResult#isSnapped}. The methods can take a bitwise
       * combination of this field and {@link yfiles.input.SnapTypes#SNAPPED_X}, {@link yfiles.input.SnapTypes#SNAPPED_X_Y} and
       * {@link yfiles.input.SnapTypes#NOT_SNAPPED}.
       * </p>
       */
      SNAPPED_Y,
      /**
       * Indicates that the location has been snapped to its x and y values.
       * <p>
       * Constant for use in {@link yfiles.input.SnapResult#snap} and {@link yfiles.input.SnapResult#isSnapped}. This is a bitwise combination of
       * {@link yfiles.input.SnapTypes#SNAPPED_X} and {@link yfiles.input.SnapTypes#SNAPPED_Y}.
       * </p>
       */
      SNAPPED_X_Y,
      /**
       * Indicates that the location has not been snapped at all.
       * <p>
       * Constant for use in {@link yfiles.input.SnapResult#snap} and {@link yfiles.input.SnapResult#isSnapped}.
       * </p>
       */
      NOT_SNAPPED
    }
    /**
     * Describes the state of a snapping operation.
     * <p>
     * This class is used by {@link yfiles.input.SnapContext} to {@link yfiles.input.SnapContext#processSnapResults process} {@link yfiles.input.SnapResult#snap snap results}.
     * </p>
     * @class
     * @final
     */
    export interface SnapState extends Object{}
    export class SnapState {
      /**
       * Initializes a new instance of the {@link yfiles.input.SnapState} class.
       * @param location The initial location.
       * @param [snapType=yfiles.input.SnapTypes.NOT_SNAPPED] The current type of the snap.
       */
      constructor(location:yfiles.geometry.Point,snapType?:yfiles.input.SnapTypes);
      /**
       * Sets the {@link yfiles.input.SnapState#location} to the given point and adjusts the {@link yfiles.input.SnapState#snapType} accordingly.
       * @param newLocation The new location.
       */
      snapTo(newLocation:yfiles.geometry.Point):void;
      /**
       * Modifies the X property of the {@link yfiles.input.SnapState#location} and adjusts the {@link yfiles.input.SnapState#snapType} accordingly.
       * @param newX The new X.
       */
      snapX(newX:number):void;
      /**
       * Modifies the Y property of the {@link yfiles.input.SnapState#location} and adjusts the {@link yfiles.input.SnapState#snapType} accordingly.
       * @param newY The new Y.
       */
      snapY(newY:number):void;
      /**
       * Gets the current location.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * Gets or sets the type of the snap.
       * @type {yfiles.input.SnapTypes}
       */
      snapType:yfiles.input.SnapTypes;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments used by {@link yfiles.input.TapInputMode#addTappedListener Tapped}.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface TapEventArgs extends yfiles.input.InputModeEventArgs{}
    export class TapEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.TapEventArgs} class.
       * @param context The context in which the tap occurred.
       * @param location The location.
       */
      constructor(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point);
      /**
       * Initializes a new instance of the {@link yfiles.input.TapEventArgs} class.
       * @param location The location.
       */
      constructor(location:yfiles.geometry.Point);
      /**
       * Gets the location of the tap.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.TapEventArgs} have been handled.
       * <p>
       * Client code that is triggered by the event and does not want the event to become handled by other handles should set
       * this property to <code>true</code> to stop propagation of the event.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum TapHandlingPolicy{
      /**
       * Using this mode, in case of a double tap, only the double tap is reported, but neither of the two single taps.
       * <p>
       * Single taps will be reported with a short delay to ensure that they do not become double taps.
       * </p>
       */
      DOUBLE_TAP_ONLY,
      /**
       * In this mode, in case of a double tap, only one single tap is reported and one double tap.
       * <p>
       * The second tap for the double tap is not reported as a single tap.
       * </p>
       */
      INITIAL_SINGLE_AND_DOUBLE_TAP,
      /**
       * In this mode, every tap (both single tap and either of the taps of a multi-tap) is reported as a single tap and every
       * double tap is reported as a double tap.
       */
      BOTH_SINGLE_TAPS_AND_DOUBLE_TAP
    }
    /**
     * An input mode that recognizes simple touch taps.
     * <p>
     * Clients register to {@link yfiles.input.TapInputMode#addTappedListener Tapped} to get notified of touch taps.
     * This mode can be instructed to {@link yfiles.input.TapInputMode#swallowFocusTap swallow taps} if they lead happen shortly after the control gained focus. This is useful
     * to prevent accidental taps from being processed if the user wanted to put the focus into the control, only.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.TapInputMode#exclusive} by default.
     * </p>
     * @see yfiles.input.TapInputMode#tapHandlingPolicy
     * @see yfiles.input.TapInputMode#swallowFocusTap
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface TapInputMode extends Object,yfiles.input.IInputMode{}
    export class TapInputMode {
      /**
       * Creates a new instance of this mode that detects touch taps.
       */
      constructor();
      /**
       * Cancels the editing of this mode.
       * <p>
       * This implementation {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
       * {@link yfiles.input.TapInputMode#onCanceled}.
       * </p>
       * @see yfiles.input.TapInputMode#onCanceled
       */
      cancel():void;
      /**
       * Creates the event recognizer that is used to recognize the press event for this instance.
       * @returns 
       * @protected
       */
      createPressRecognizer():(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates the event recognizer that is used to recognize the release event for this instance.
       * @returns 
       * @protected
       */
      createReleaseRecognizer():(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates the event recognizer that is used to recognize the taps for this instance.
       * @returns 
       * @protected
       */
      createTapRecognizer():(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link } for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Determines whether the given event is a press event that occurred at an invalid location.
       * @param eventSource
       * @param evt
       * @returns 
       * @see yfiles.input.TapInputMode#validTapHitTestable
       * @protected
       */
      isInvalidPress(eventSource:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Determines whether the given event is a valid press event.
       * @param eventSource
       * @param evt
       * @returns 
       * @protected
       */
      isValidPress(eventSource:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Determines whether the given event is a valid release event.
       * @param eventSource
       * @param evt
       * @returns 
       * @protected
       */
      isValidRelease(eventSource:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Determines whether the given event is a valid tap event.
       * @param eventSource
       * @param evt
       * @returns 
       * @protected
       */
      isValidTap(eventSource:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Called after {@link yfiles.input.TapInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called once this mode has recognized a double-tap gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.TapInputMode#addDoubleTappedListener DoubleTapped} event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onDoubleTapped(evt:yfiles.input.TapEventArgs):void;
      /**
       * Called after {@link yfiles.input.TapInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Called once this mode has recognized a tap gesture.
       * <p>
       * This method will trigger the {@link yfiles.input.TapInputMode#addTappedListener Tapped} event.
       * </p>
       * @param evt The arguments.
       * @protected
       */
      onTapped(evt:yfiles.input.TapEventArgs):void;
      /**
       * Prevents a double-tap event from being issued if the next tap would do so and sends only a single tap instead.
       * <p>
       * The intention is for "breaking" a double tap if an input mode using {@link yfiles.input.TapInputMode} performs actions that
       * would lead to surprising behavior if the next tap would lead to a double-tap. Examples of this are
       * {@link yfiles.input.GraphEditorInputMode}'s selection cycling where several taps in short succession may occur, as well as
       * creating a node by tapping on the canvas and selecting it immediately afterwards with another tap.
       * </p>
       * <p>
       * This method's effect is very short-lived. It really only prevents a double-tap event for the very next tap that this
       * input mode handles. The internal flag set by this method is re-set on every tap received. This also means that if you
       * call this method and the next tap is just a regular single-tap the flag is cleared nonetheless.
       * </p>
       */
      preventNextDoubleTap():void;
      /**
       * Returns and resets the flag set by {@link yfiles.input.TapInputMode#preventNextDoubleTap}.
       * <p>
       * To ensure that the flag really acts just once, this method resets the flag and returns its value prior to the reset.
       * </p>
       * @returns 
       * @protected
       */
      queryAndResetPreventNextDoubleTap():boolean;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.TapInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.TapInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.ConcurrencyController}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>Tapped</code> event that occurs once a tap has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.TapEventArgs
       * @see yfiles.input.TapInputMode#tapHandlingPolicy
       * @see yfiles.input.TapInputMode#removeTappedListener
       */
      addTappedListener(listener:(sender:Object,evt:yfiles.input.TapEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Tapped</code> event that occurs once a tap has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.TapEventArgs
       * @see yfiles.input.TapInputMode#tapHandlingPolicy
       * @see yfiles.input.TapInputMode#addTappedListener
       */
      removeTappedListener(listener:(sender:Object,evt:yfiles.input.TapEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DoubleTapped</code> event that occurs once a double-tap has been detected.
       * @param listener The listener to add.
       * @see yfiles.input.TapEventArgs
       * @see yfiles.input.TapInputMode#tapHandlingPolicy
       * @see yfiles.input.TapInputMode#removeDoubleTappedListener
       */
      addDoubleTappedListener(listener:(sender:Object,evt:yfiles.input.TapEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DoubleTapped</code> event that occurs once a double-tap has been detected.
       * @param listener The listener to remove.
       * @see yfiles.input.TapEventArgs
       * @see yfiles.input.TapInputMode#tapHandlingPolicy
       * @see yfiles.input.TapInputMode#addDoubleTappedListener
       */
      removeDoubleTappedListener(listener:(sender:Object,evt:yfiles.input.TapEventArgs)=>void):void;
      /**
       * Gets or sets the tap handling policy that determines the triggering behavior of {@link yfiles.input.TapInputMode#addTappedListener Tapped}
       * and {@link yfiles.input.TapInputMode#addDoubleTappedListener DoubleTapped}.
       * @see yfiles.input.TapInputMode#tapHandlingPolicy
       * @default yfiles.input.TapHandlingPolicy.BOTH_SINGLE_TAPS_AND_DOUBLE_TAP
       * @type {yfiles.input.TapHandlingPolicy}
       */
      tapHandlingPolicy:yfiles.input.TapHandlingPolicy;
      /**
       * Gets or sets a hit test that determines where this mode should recognize taps.
       * <p>
       * This implementation is tested during the {@link yfiles.input.TapInputMode#isValidPress} to determine whether it is valid to tap
       * here. The default implementation is {@link yfiles.input.IHitTestable#ALWAYS}.
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      validTapHitTestable:yfiles.input.IHitTestable;
      /**
       * Whether to {@link yfiles.input.ConcurrencyController#requestMutex request the mutex} on a tap.
       * <p>
       * If set to <code>true</code> this will discard other concurrent input modes on a tap. The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      requestMutexOnTap:boolean;
      /**
       * Gets or sets a property that determines whether taps should be swallowed if they happen within a short amount of time
       * after the {@link yfiles.view.CanvasComponent} got focus.
       * <p>
       * The default is <code>false</code>. If set to <code>true</code> taps are discarded within 100 ms after the focus entered.
       * </p>
       * @default false
       * @type {boolean}
       */
      swallowFocusTap:boolean;
      /**
       * Gets the location of the last tap.
       * @type {yfiles.geometry.Point}
       */
      tapLocation:yfiles.geometry.Point;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple {@link yfiles.input.IInputMode} that displays a <code>textarea</code> in the {@link yfiles.view.CanvasComponent} to let the user edit a
     * text.
     * <p>
     * Hitting escape or enter will cancel or stop editing.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.TextEditorInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface TextEditorInputMode extends Object,yfiles.input.IInputMode{}
    export class TextEditorInputMode {
      /**
       * Creates a new instance using a default text box.
       */
      constructor();
      /**
       * Adjusts the position of the text box in the {@link yfiles.view.CanvasComponent}.
       * @protected
       */
      adjustPosition():void;
      /**
       * Adjusts the size of the text box in the {@link yfiles.input.TextEditorInputMode#editorContainer}.
       * @param maxSize The suggested maximum size to use.
       * @protected
       */
      adjustSize(maxSize:yfiles.geometry.Size):void;
      /**
       * Calculates the maximum size for this instance.
       * <p>
       * The result is passed to {@link yfiles.input.TextEditorInputMode#adjustSize}.
       * </p>
       * <p>
       * This implementation makes sure the textbox does not exceed the bounds of the canvas.
       * </p>
       * @returns The maximum size for the textbox.
       * @see yfiles.input.TextEditorInputMode#adjustSize
       * @protected
       */
      calculateMaxTextBoxSize():yfiles.geometry.Size;
      /**
       * Cancels editing of the text box.
       */
      cancel():void;
      /**
       * Resets the contents of the text box.
       */
      clear():void;
      /**
       * Starts interactive text editing.
       * @returns A Promise that will fulfill with the edited text or fulfill with <code>null</code> if editing is cancelled.
       */
      edit():Promise<string>;
      /**
       * Adjusts the {@link yfiles.view.CanvasComponent#viewport} of the {@link yfiles.view.CanvasComponent} such that the text box is in the visible area.
       * <p>
       * Called once when the editing starts.
       * </p>
       * @protected
       */
      ensureVisible():void;
      /**
       * Returns the bounds of the text area in world coordinates.
       * @returns 
       * @protected
       */
      getTextBoxBounds():yfiles.geometry.Rect;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Installs the text box in the canvas and puts focus into it.
       * @protected
       */
      installTextBox():void;
      /**
       * Called when editing starts.
       * <p>
       * Requests the input mutex, and installs the text box.
       * </p>
       * @protected
       */
      onBeginEditing():void;
      /**
       * Called after {@link yfiles.input.TextEditorInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called when editing gets canceled.
       * <p>
       * Removes the text box and releases the input mutex.
       * </p>
       * @protected
       */
      onCancelEditing():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Raises the {@link yfiles.input.TextEditorInputMode#addEditingCanceledListener EditingCanceled} event.
       * @param evt The event argument
       * @protected
       */
      onEditingCanceled(evt:yfiles.input.TextEventArgs):void;
      /**
       * Raises the {@link yfiles.input.TextEditorInputMode#addEditingStartedListener EditingStarted} event.
       * @param evt The event argument.
       * @protected
       */
      onEditingStarted(evt:yfiles.input.TextEventArgs):void;
      /**
       * Called when editing is stopped.
       * <p>
       * Triggers the {@link yfiles.input.TextEditorInputMode#addTextEditedListener TextEdited} event, removes the
       * box and releases the input mutex.
       * </p>
       * @protected
       */
      onStopEditing():void;
      /**
       * Called after {@link yfiles.input.TextEditorInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Raises the {@link yfiles.input.TextEditorInputMode#addTextEditedListener TextEdited} event.
       * @param evt The event arguments.
       * @protected
       */
      onTextEdited(evt:yfiles.input.TextEventArgs):void;
      /**
       * Stops editing and returns <code>true</code>.
       * @returns <code>base.Stop()</code>
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Removes the text box.
       * @protected
       */
      uninstallTextBox():void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.TextEditorInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.TextEditorInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.TextEditorInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>TextEdited</code> event that occurs once the text has been edited.
       * @param listener The listener to add.
       * @see yfiles.input.TextEditorInputMode#removeTextEditedListener
       */
      addTextEditedListener(listener:(sender:Object,evt:yfiles.input.TextEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>TextEdited</code> event that occurs once the text has been edited.
       * @param listener The listener to remove.
       * @see yfiles.input.TextEditorInputMode#addTextEditedListener
       */
      removeTextEditedListener(listener:(sender:Object,evt:yfiles.input.TextEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EditingCanceled</code> event that occurs if the editing has not been finished.
       * @param listener The listener to add.
       * @see yfiles.input.TextEditorInputMode#removeEditingCanceledListener
       */
      addEditingCanceledListener(listener:(sender:Object,evt:yfiles.input.TextEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EditingCanceled</code> event that occurs if the editing has not been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.TextEditorInputMode#addEditingCanceledListener
       */
      removeEditingCanceledListener(listener:(sender:Object,evt:yfiles.input.TextEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EditingStarted</code> event that occurs when text editing is started.
       * @param listener The listener to add.
       * @see yfiles.input.TextEditorInputMode#removeEditingStartedListener
       */
      addEditingStartedListener(listener:(sender:Object,evt:yfiles.input.TextEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EditingStarted</code> event that occurs when text editing is started.
       * @param listener The listener to remove.
       * @see yfiles.input.TextEditorInputMode#addEditingStartedListener
       */
      removeEditingStartedListener(listener:(sender:Object,evt:yfiles.input.TextEventArgs)=>void):void;
      /**
       * Gets or sets a property that determines whether the text edited should be committed if the focus gets lost or the
       * editing should be canceled.
       * <p>
       * The default behavior is to cancel the editing process.
       * </p>
       * @default false
       * @type {boolean}
       */
      autoCommitOnFocusLost:boolean;
      /**
       * Gets or sets a property that determines whether this mode should try to {@link yfiles.view.CanvasComponent#focus} the canvas
       * once the editor is closed via a keyboard gesture.
       * <p>
       * The default is <code>true</code>
       * </p>
       * @default true
       * @type {boolean}
       */
      autoFocusCanvasOnEditorClosed:boolean;
      /**
       * Determines whether this mode should automatically fire up the editor once it gets installed.
       * <p>
       * If this mode is permanently installed, set this flag to <code>false</code> and set the {@link yfiles.input.TextEditorInputMode#editing} flag to <code>true</code> in order to start editing.
       * </p>
       * @default false
       * @type {boolean}
       */
      autoStartEditing:boolean;
      /**
       * Gets or sets the event recognizer to detect the key gesture for new line breaks.
       * <p>
       * When the provided {@link yfiles.view.KeyEventArgs} are recognized, a new line break is inserted at the current caret position.
       * </p>
       * @see yfiles.input.TextEditorInputMode#cancelRecognizer
       * @see yfiles.input.TextEditorInputMode#stopRecognizer
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      lineBreakRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer to detect when editing the text shall be canceled.
       * <p>
       * When the provided {@link yfiles.view.KeyEventArgs} are recognized, {@link yfiles.input.TextEditorInputMode#cancel} is called.
       * </p>
       * @see yfiles.input.TextEditorInputMode#lineBreakRecognizer
       * @see yfiles.input.TextEditorInputMode#stopRecognizer
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer to detect when editing the text shall be stopped.
       * <p>
       * When the provided {@link yfiles.view.KeyEventArgs} are recognized, {@link yfiles.input.TextEditorInputMode#tryStop} is called.
       * </p>
       * @see yfiles.input.TextEditorInputMode#lineBreakRecognizer
       * @see yfiles.input.TextEditorInputMode#cancelRecognizer
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      stopRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the current editing state of the editor.
       * <p>
       * Setting a new value will trigger the {@link yfiles.input.TextEditorInputMode#onBeginEditing} and {@link yfiles.input.TextEditorInputMode#onCancelEditing}
       * methods respectively.
       * </p>
       * @type {boolean}
       */
      editing:boolean;
      /**
       * Gets or sets the location of the text box in world coordinates.
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets or sets the point that describes the "up"-vector that will be used to determine the orientation of the text box in
       * the world coordinate system.
       * <p>
       * The default is <code>0, -1</code>.
       * </p>
       * @see yfiles.input.TextEditorInputMode#rotateTextBox
       * @type {yfiles.geometry.IPoint}
       */
      upVector:yfiles.geometry.IPoint;
      /**
       * Gets or sets the anchor of the text box.
       * <p>
       * The anchor is the point inside the coordinate system of the text box that shall coincide with the {@link yfiles.input.TextEditorInputMode#location} in the world
       * coordinate system. The X and Y values are expressed as relative width/height ratios.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      anchor:yfiles.geometry.IPoint;
      /**
       * Gets the <code>div</code>-element that contains the text editor.
       * @type {HTMLDivElement}
       */
      editorContainer:HTMLDivElement;
      /**
       * Gets or sets the text that is displayed and edited.
       * <p>
       * If customizations change the way the text is edited by providing a custom editor, this property needs to be overridden
       * to return the current text contents.
       * </p>
       * @returns The textual contents of the editor.
       * @type {string}
       */
      editorText:string;
      /**
       * Gets or sets a value indicating whether to automatically flip the orientation of the text box if the {@link yfiles.input.TextEditorInputMode#upVector} actually points
       * downwards.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      autoFlipTextBox:boolean;
      /**
       * Gets or sets a value indiciating whether the text box should be rotated with the {@link yfiles.input.TextEditorInputMode#upVector}.
       * <p>
       * If the value is <code>false</code> (the default), the text box will always be horizontal.
       * </p>
       * @type {boolean}
       */
      rotateTextBox:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments used by {@link yfiles.input.TextEditorInputMode}.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface TextEventArgs extends yfiles.input.InputModeEventArgs{}
    export class TextEventArgs {
      /**
       * Creates a new instance.
       * @param context The context in which this text event happened.
       * @param text The text that was or will be edited.
       */
      constructor(context:yfiles.input.IInputModeContext,text:string);
      /**
       * Gets the text that was or will be edited.
       * @type {string}
       */
      text:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} implementation that can be used to block user interaction.
     * <p>
     * Setting the {@link yfiles.input.WaitInputMode#waiting} property will try to cancel ongoing edits of the {@link yfiles.input.ConcurrencyController#mutexOwner mutex owner} and set the preferred cursor to {@link yfiles.input.WaitInputMode#waitCursor}. This mode
     * will try to add itself to the {@link yfiles.view.CanvasComponent#lookup Lookup of the CanvasControl}, so that other clients can make use of its functionality.
     * </p>
     * <p>
     * This mode uses an {@link yfiles.input.ConcurrencyController#exclusive} controller by default.
     * </p>
     * @see yfiles.input.WaitInputMode#waiting
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface WaitInputMode extends Object,yfiles.input.IInputMode{}
    export class WaitInputMode {
      /**
       * Creates a new instance of this mode.
       */
      constructor();
      /**
       * 
       */
      cancel():void;
      /**
       * Returns if this mode can request the mutex when {@link yfiles.input.WaitInputMode#waiting} is set to <code>true</code>.
       * @returns if this mode can request the mutex when {@link #waiting} is set to <code>true</code>.
       */
      canStartWaiting():boolean;
      /**
       * Called when waiting is finished.
       * <p>
       * This will reset the {@link yfiles.input.ConcurrencyController#preferredCursor} and {@link yfiles.input.ConcurrencyController#releaseMutex the mutex is released.}
       * </p>
       * @protected
       */
      endWaiting():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.WaitInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called after {@link yfiles.input.WaitInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Raises the {@link yfiles.input.WaitInputMode#addWaitingEndedListener WaitingEnded} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onWaitingEnded(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.WaitInputMode#addWaitingStartedListener WaitingStarted} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onWaitingStarted(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * This will initiate the waiting process by trying to
       * {@link yfiles.input.ConcurrencyController#requestMutex}request the input mutex.
       * <p>
       * This will set the {@link yfiles.input.ConcurrencyController#preferredCursor} to {@link yfiles.input.WaitInputMode#waitCursor}.
       * </p>
       * @protected
       */
      startWaiting():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.WaitInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.WaitInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets or sets the Waiting property.
       * <p>
       * Setting this property to <code>true</code> will {@link yfiles.input.WaitInputMode#startWaiting start the waiting process.} Setting it to <code>false</code> will {@link yfiles.input.WaitInputMode#endWaiting end the waiting}.
       * </p>
       * @type {boolean}
       */
      waiting:boolean;
      /**
       * Adds the given listener for the <code>WaitingStarted</code> event that occurs when the waiting started.
       * @param listener The listener to add.
       * @see yfiles.input.WaitInputMode#removeWaitingStartedListener
       */
      addWaitingStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>WaitingStarted</code> event that occurs when the waiting started.
       * @param listener The listener to remove.
       * @see yfiles.input.WaitInputMode#addWaitingStartedListener
       */
      removeWaitingStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>WaitingEnded</code> event that occurs when the waiting ended.
       * @param listener The listener to add.
       * @see yfiles.input.WaitInputMode#removeWaitingEndedListener
       */
      addWaitingEndedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>WaitingEnded</code> event that occurs when the waiting ended.
       * @param listener The listener to remove.
       * @see yfiles.input.WaitInputMode#addWaitingEndedListener
       */
      removeWaitingEndedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Gets or sets the WaitCursor property.
       * @type {yfiles.view.Cursor}
       */
      waitCursor:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * A convenience implementation of the {@link yfiles.input.IPositionHandler} interface that can be used to constrain the position
     * of an item.
     * @see yfiles.input.ConstrainedDragHandler.<TWrapped>
     * @see yfiles.input.ConstrainedDragHandler.<TWrapped>#constrainNewLocation
     * @class
     * @extends {yfiles.input.ConstrainedDragHandler.<yfiles.input.IPositionHandler>}
     * @implements {yfiles.input.IPositionHandler}
     */
    export interface ConstrainedPositionHandler extends yfiles.input.ConstrainedDragHandler<yfiles.input.IPositionHandler>,yfiles.input.IPositionHandler{}
    export class ConstrainedPositionHandler {
      /**
       * Initializes a new instance of the {@link yfiles.input.ConstrainedPositionHandler} class that delegates to the <code>wrappedHandler</code>.
       * @param wrappedHandler The handler to wrap.
       * @protected
       */
      constructor(wrappedHandler:yfiles.input.IPositionHandler);
      static $class:yfiles.lang.Class;
    }
    /**
     * A convenience implementation of the {@link yfiles.input.IHandle} interface that can be used to constrain the handle of an item.
     * @see yfiles.input.ConstrainedDragHandler.<TWrapped>
     * @see yfiles.input.ConstrainedDragHandler.<TWrapped>#constrainNewLocation
     * @class
     * @extends {yfiles.input.ConstrainedDragHandler.<yfiles.input.IHandle>}
     * @implements {yfiles.input.IHandle}
     */
    export interface ConstrainedHandle extends yfiles.input.ConstrainedDragHandler<yfiles.input.IHandle>,yfiles.input.IHandle{}
    export class ConstrainedHandle {
      /**
       * Initializes a new instance of the {@link yfiles.input.ConstrainedHandle} class that delegates to the <code>wrappedHandle</code>.
       * @param wrappedHandle The handle to wrap.
       * @protected
       */
      constructor(wrappedHandle:yfiles.input.IHandle);
      /**
       * Delegates to {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#wrappedHandler}'s {@link yfiles.input.IHandle#type} property.
       * @type {yfiles.input.HandleTypes}
       */
      type:yfiles.input.HandleTypes;
      /**
       * Delegates to {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#wrappedHandler}'s {@link yfiles.input.IHandle#cursor} property.
       * @type {yfiles.view.Cursor}
       */
      cursor:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility implementation of the {@link yfiles.input.IReshapeHandler} interface that can be used to constrain the reshaping of an
     * existing instance conveniently.
     * @see yfiles.input.ConstrainedHandle
     * @see yfiles.input.ConstrainedPositionHandler
     * @see yfiles.input.ConstrainedReshapeHandler#constrainNewBounds
     * @class
     * @implements {yfiles.input.IReshapeHandler}
     */
    export interface ConstrainedReshapeHandler extends Object,yfiles.input.IReshapeHandler{}
    export class ConstrainedReshapeHandler {
      /**
       * Initializes a new instance of the {@link yfiles.input.ConstrainedReshapeHandler} class that delegates to the <code>wrappedHandler</code>.
       * @param wrappedHandler The handler to wrap.
       * @protected
       */
      constructor(wrappedHandler:yfiles.input.IReshapeHandler);
      /**
       * Cancels the move operation and calls {@link yfiles.input.ConstrainedReshapeHandler#onCanceled}
       * @param context The context to retrieve information about the drag from.
       * @param originalBounds The value of the coordinate of the {@link #bounds} property at the time of {@link #initializeReshape}.
       */
      cancelReshape(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect):void;
      /**
       * Callback that needs to implement the constraints for the new bounds.
       * @param context The context in which the reshape will be performed.
       * @param originalBounds The value of the {@link #bounds} property at the time of {@link #initializeReshape}.
       * @param newBounds The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #bounds} may or may not be modified to reflect the new value.
       * @returns The constrained value of <code>newBounds</code>.
       * @protected
       * @abstract
       */
      constrainNewBounds(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):yfiles.geometry.Rect;
      /**
       * Handles the reshape operation and subsequently calls {@link yfiles.input.ConstrainedReshapeHandler#onReshaped}
       * <p>
       * This method will use the {@link yfiles.input.ConstrainedReshapeHandler#constrainNewBounds} callback to let subclasses constrain
       * the reshaping.
       * </p>
       * @param context The context in which the reshape will be performed.
       * @param originalBounds The value of the {@link #bounds} property at the time of {@link #initializeReshape}.
       * @param newBounds The coordinates in the world coordinate system that the client wants to reshape the item to. Depending on the
       * implementation the {@link #bounds} may or may not be modified to reflect the new value.
       */
      handleReshape(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * Initializes the reshape operation and subsequently calls {@link yfiles.input.ConstrainedReshapeHandler#onInitialized}
       * @param context The context in which the drag will be performed.
       */
      initializeReshape(context:yfiles.input.IInputModeContext):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedReshapeHandler#cancelReshape} method has been called.
       * @param context The input mode context.
       * @param originalBounds The original bounds.
       * @protected
       */
      onCanceled(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedReshapeHandler#reshapeFinished} method has been called.
       * @param context The input mode context.
       * @param originalBounds The original bounds.
       * @param newBounds The new bounds.
       * @protected
       */
      onFinished(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedReshapeHandler#initializeReshape} method has been called.
       * @param context The input mode context.
       * @param originalBounds The original bounds.
       * @protected
       */
      onInitialized(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedReshapeHandler#handleReshape} operation has been performed.
       * @param context The input mode context.
       * @param originalBounds The original Bounds.
       * @param newBounds The new Bounds.
       * @protected
       */
      onReshaped(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * Handles the finish operation and invokes {@link yfiles.input.ConstrainedReshapeHandler#onFinished}
       * @param context The context to retrieve information about the drag from.
       * @param originalBounds The value of the {@link #bounds} property at the time of {@link #initializeReshape}.
       * @param newBounds The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #bounds} may or may not be modified to reflect the new value. This is the same value as delivered in the last
       * invocation of {@link #handleReshape}
       */
      reshapeFinished(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * Gets the wrapped handler to which calls will be ultimately delegated.
       * @protected
       * @type {yfiles.input.IReshapeHandler}
       */
      wrappedHandler:yfiles.input.IReshapeHandler;
      /**
       * Delegates to the {@link yfiles.input.ConstrainedReshapeHandler#wrappedHandler}'s {@link yfiles.input.IReshapeHandler#bounds} property.
       * @type {yfiles.geometry.IRectangle}
       */
      bounds:yfiles.geometry.IRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility implementation of the {@link yfiles.input.IDragHandler} interface that can be used to constrain the movement of an
     * existing instance.
     * Type parameter <code>TWrapped</code>: The exact type of the wrapped implementation.
     * @see yfiles.input.ConstrainedHandle
     * @see yfiles.input.ConstrainedPositionHandler
     * @see yfiles.input.ConstrainedDragHandler.<TWrapped>#constrainNewLocation
     * @class
     * @implements {yfiles.input.IDragHandler}
     * @template TWrapped
     */
    export interface ConstrainedDragHandler<TWrapped extends yfiles.input.IDragHandler> extends Object,yfiles.input.IDragHandler{}
    export class ConstrainedDragHandler<TWrapped extends yfiles.input.IDragHandler> {
      /**
       * Initializes a new instance of the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>} class that delegates to the <code>wrappedHandler</code>.
       * @param wrappedHandler The handler to wrap.
       * @protected
       */
      constructor(wrappedHandler:TWrapped);
      /**
       * Cancels the move operation and calls {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#onCanceled}
       * @param context The context to retrieve information about the drag from.
       * @param originalLocation The value of the coordinate of the {@link #location} property at the time of {@link #initializeDrag}.
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Callback that needs to implement the constraints for the new location.
       * @param context The context in which the drag will be performed.
       * @param originalLocation The value of the {@link #location} property at the time of {@link #initializeDrag}.
       * @param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #location} may or may not be modified to reflect the new value.
       * @returns The constrained value of <code>newLocation</code>.
       * @protected
       * @abstract
       */
      constrainNewLocation(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Handles the finish operation and invokes {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#onFinished}
       * @param context The context to retrieve information about the drag from.
       * @param originalLocation The value of the {@link #location} property at the time of {@link #initializeDrag}.
       * @param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #location} may or may not be modified to reflect the new value. This is the same value as delivered in the last
       * invocation of {@link #handleMove}
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Handles the move operation and subsequently calls {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#onMoved}
       * <p>
       * This method will use the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#constrainNewLocation} callback to let subclasses constrain the movement.
       * </p>
       * @param context The context in which the drag will be performed.
       * @param originalLocation The value of the {@link #location} property at the time of {@link #initializeDrag}.
       * @param newLocation The coordinates in the world coordinate system that the client wants the handle to be at. Depending on the
       * implementation the {@link #location} may or may not be modified to reflect the new value.
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Initializes the drag operation and subsequently calls {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#onInitialized}
       * @param context The context in which the drag will be performed.
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#cancelDrag} method has been called.
       * @param context The input mode context.
       * @param originalLocation The original location.
       * @protected
       */
      onCanceled(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#dragFinished} method has been called.
       * @param context The input mode context.
       * @param originalLocation The original location.
       * @param newLocation The new location.
       * @protected
       */
      onFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#initializeDrag} method has been called.
       * @param context The input mode context.
       * @param originalLocation The original location.
       * @protected
       */
      onInitialized(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Called when the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#handleMove} operation has been performed.
       * @param context The input mode context.
       * @param originalLocation The original location.
       * @param newLocation The new location.
       * @protected
       */
      onMoved(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Gets the wrapped handler to which calls will be ultimately delegated.
       * @protected
       * @type {TWrapped}
       */
      wrappedHandler:TWrapped;
      /**
       * Delegates to the {@link yfiles.input.ConstrainedDragHandler.<TWrapped>#wrappedHandler}'s {@link yfiles.input.IDragHandler#location} property.
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface that is implemented for elements that can be hit or picked in a coordinate system.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 05 Hit Test</li>
     * <li>Tutorial: 02 Custom Styles, step 17 Edge Hit Test</li>
     * </ul>
     * @see yfiles.view.CanvasComponent
     * @see yfiles.view.ICanvasObjectDescriptor
     * @see yfiles.view.ICanvasObject
     * @interface
     */
    export interface IHitTestable extends Object{
      /**
       * Determines if something has been hit at the given coordinates in the world coordinate system.
       * <p>
       * Implementations should inspect the {@link yfiles.view.ICanvasContext#hitTestRadius} and take the value into account when performing hit tests. This allows the user to
       * hit elements even if the zoom level is very small and allows for accurate hit tests at greater zoom levels.
       * </p>
       * @param context the context the hit test is performed in
       * @param location the coordinates in world coordinate system
       * @returns whether something has been hit
       * @abstract
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
    }
    var IHitTestable:{
      /**
       * Wrap a handler into an interface.
       * @param handler
       * @returns 
       * @static
       */
      create?(handler:(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point)=>boolean):yfiles.input.IHitTestable;
      /**
       * An {@link yfiles.input.IHitTestable} that always returns false.
       * @const
       * @static
       * @type {yfiles.input.IHitTestable}
       */
      NEVER?:yfiles.input.IHitTestable;
      /**
       * An {@link yfiles.input.IHitTestable} that always returns true.
       * @const
       * @static
       * @type {yfiles.input.IHitTestable}
       */
      ALWAYS?:yfiles.input.IHitTestable;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is used to test whether a rendered item is deemed to be inside a rectangular marquee selection.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 05 Hit Test</li>
     * </ul>
     * @interface
     */
    export interface IMarqueeTestable extends Object{
      /**
       * This callback returns <code>true</code> if the corresponding item is considered to intersect the given rectangular box.
       * <p>
       * This method may return <code>false</code> if the item cannot be selected using a selection marquee or optionally if the item is only
       * partially contained within the box.
       * </p>
       * <p>
       * Implementations should respect the {@link yfiles.view.ICanvasContext#hitTestRadius} if marquee selections should behave differently on different zoom levels.
       * </p>
       * @param context the current input mode context
       * @param rectangle the box describing the marquee's bounds
       * @returns <code>true</code> if the item is considered to be captured by the marquee
       * @abstract
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
    }
    var IMarqueeTestable:{
      /**
       * An implementation that always yields <code>true</code>.
       * @const
       * @static
       * @type {yfiles.input.IMarqueeTestable}
       */
      ALWAYS?:yfiles.input.IMarqueeTestable;
      /**
       * An implementation that always yields <code>false</code>.
       * @const
       * @static
       * @type {yfiles.input.IMarqueeTestable}
       */
      NEVER?:yfiles.input.IMarqueeTestable;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A more sophisticated {@link yfiles.input.IHandle} implementation that modifies a rectangle.
     * <p>
     * Implementations of this class work on a mutable rectangle instance for handling drags of the handle. A separate point
     * implementation is used for specifying the position of the handle. As a convenience this class also implements the {@link yfiles.geometry.IMutablePoint}
     * interface which modifies the handle's position.
     * </p>
     * @class
     * @implements {yfiles.geometry.IMutablePoint}
     * @implements {yfiles.input.IHandle}
     */
    export interface RectangleHandle extends Object,yfiles.geometry.IMutablePoint,yfiles.input.IHandle{}
    export class RectangleHandle {
      /**
       * Subclass constructor that creates a handle for the given position using the location instance as the {@link yfiles.input.IDragHandler#location} for the handle.
       * <p>
       * The instance modifies the instances using the reshapeable interface and reads the current state using the rect. The
       * handle will automatically trim changes of the rectangle to the specified minimum and maximum size.
       * </p>
       * @param position The position of the handle.
       * @param rectangle The rectangle to change.
       */
      constructor(position:yfiles.input.HandlePositions,rectangle:yfiles.geometry.IMutableRectangle);
      /**
       * Delegates the original values to {@link yfiles.input.RectangleHandle#set}
       * @param context
       * @param originalLocation
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * This implementation does nothing.
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Delegates to {@link yfiles.input.RectangleHandle#set}.
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * This implementation does nothing.
       * @param context
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Delegates the work to the reshapeable.
       * @param delta
       */
      moveBy(delta:yfiles.geometry.Point):void;
      /**
       * Delegates to {@link yfiles.input.RectangleHandle#moveBy}.
       * @param x
       * @param y
       */
      set(x:number,y:number):void;
      /**
       * The position of the handle.
       * @protected
       * @type {yfiles.input.HandlePositions}
       */
      position:yfiles.input.HandlePositions;
      /**
       * The rectangle instance to modify.
       * @protected
       * @type {yfiles.geometry.IMutableRectangle}
       */
      rectangle:yfiles.geometry.IMutableRectangle;
      /**
       * Gets or sets the minimum size allowed for the reshapeable.
       * <p>
       * The value is stored by reference. The default is {@link yfiles.geometry.Size#ZERO}.
       * </p>
       * @type {yfiles.geometry.ISize}
       */
      minimumSize:yfiles.geometry.ISize;
      /**
       * Gets or sets the minimum enclosed area that needs to be encompassed by this reshapeable.
       * <p>
       * The value is stored by reference. The default is {@link yfiles.geometry.Size#ZERO}, which does not restrict the area at all.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      minimumEnclosedArea:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the maximum size allowed for the reshapeable.
       * <p>
       * The value is stored by reference. The default is {@link yfiles.geometry.Size#INFINITE}.
       * </p>
       * @type {yfiles.geometry.ISize}
       */
      maximumSize:yfiles.geometry.ISize;
      /**
       * Returns a view of the center of the handle.
       * <p>
       * The point describes the current world coordinate of the element that can be modified by this handle.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets or sets the type of the handle that can be used by the rendering engine to render types differently.
       * @type {yfiles.input.HandleTypes}
       */
      type:yfiles.input.HandleTypes;
      /**
       * Gets or sets the cursor to display when the mouse hovers over or drags this handle.
       * @type {yfiles.view.Cursor}
       */
      cursor:yfiles.view.Cursor;
      /**
       * Gets or sets the x coordinate.
       * <p>
       * The setter delegates to {@link yfiles.input.RectangleHandle#moveBy}
       * </p>
       * @type {number}
       */
      x:number;
      /**
       * Gets or sets the y coordinate.
       * <p>
       * The setter delegates to {@link yfiles.input.RectangleHandle#moveBy}
       * </p>
       * @type {number}
       */
      y:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A more sophisticated {@link yfiles.input.IHandle} implementation that delegates to a {@link yfiles.input.IReshapeHandler}.
     * <p>
     * This class works on an {@link yfiles.input.IReshapeHandler}.
     * </p>
     * @class
     * @implements {yfiles.input.IHandle}
     */
    export interface ReshapeHandlerHandle extends Object,yfiles.input.IHandle{}
    export class ReshapeHandlerHandle {
      /**
       * Subclass constructor that creates a handle for the given position using the location instance as the {@link yfiles.input.IDragHandler#location} for the handle.
       * <p>
       * The instance modifies the instances using the reshapeable interface and reads the current state using the rect. The
       * handle will automatically trim changes of the rectangle to the specified minimum and maximum size.
       * </p>
       * @param position The position of the handle.
       * @param reshapeHandler The reshapeable to write the changes to.
       * @param [maximumSize=null] The maximum size allowed for the rectangle.
       * @param [minimumSize=null] The minimum size allowed for the rectangle.
       */
      constructor(position:yfiles.input.HandlePositions,reshapeHandler:yfiles.input.IReshapeHandler,minimumSize?:yfiles.geometry.ISize,maximumSize?:yfiles.geometry.ISize);
      /**
       * Subclass constructor that creates a handle for the given position using the location instance as the {@link yfiles.input.IDragHandler#location} for the handle.
       * <p>
       * The instance modifies the instances using the reshapeable interface and reads the current state using the rect. The
       * handle will automatically trim changes of the rectangle to the specified minimum and maximum size.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.position The position of the handle.
       * @param options.reshapeHandler The reshapeable to write the changes to.
       * @param [options.maximumSize=null] The maximum size allowed for the rectangle.
       * @param [options.minimumSize=null] The minimum size allowed for the rectangle.
       */
      constructor(options:{position:yfiles.input.HandlePositions,reshapeHandler:yfiles.input.IReshapeHandler,minimumSize?:yfiles.geometry.ISize,maximumSize?:yfiles.geometry.ISize});
      /**
       * Delegates the cancel request to the {@link yfiles.input.IReshapeHandler}.
       * @param context
       * @param originalLocation
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Delegates the finish request to the {@link yfiles.input.IReshapeHandler}.
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Delegates the move request to the {@link yfiles.input.IReshapeHandler#handleReshape} method.
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * This implementation does nothing.
       * @param context
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * The reshapeable instance to write the changes to.
       * @protected
       * @type {yfiles.input.IReshapeHandler}
       */
      reshapeHandler:yfiles.input.IReshapeHandler;
      /**
       * Gets or sets the minimum size allowed for the reshapeable.
       * <p>
       * The value is stored by reference. The default is {@link yfiles.geometry.Size#ZERO}.
       * </p>
       * @type {yfiles.geometry.ISize}
       */
      minimumSize:yfiles.geometry.ISize;
      /**
       * Gets or sets the minimum enclosed area that needs to be encompassed by this reshapeable.
       * <p>
       * The value is stored by reference. The default is {@link yfiles.geometry.Size#ZERO}, which does not restrict the area at all.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      minimumEnclosedArea:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the maximum size allowed for the reshapeable.
       * <p>
       * The value is stored by reference. The default is {@link yfiles.geometry.Size#INFINITE}.
       * </p>
       * @type {yfiles.geometry.ISize}
       */
      maximumSize:yfiles.geometry.ISize;
      /**
       * Returns a view of the center of the handle.
       * <p>
       * The point describes the current world coordinate of the element that can be modified by this handle.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets or sets the type of the handle that can be used by the rendering engine to render types differently.
       * @type {yfiles.input.HandleTypes}
       */
      type:yfiles.input.HandleTypes;
      /**
       * Gets or sets the cursor to display when the mouse hovers over or drags this handle.
       * @type {yfiles.view.Cursor}
       */
      cursor:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for the {@link yfiles.input.GraphSnapContext#addCollectSnapLinesListener CollectSnapLines} event.
     * <p>
     * Instances of this class are also passed to
     * {@link yfiles.input.ISnapLineProvider#addSnapLines}. Implementations of that interface can use the following methods to add snap
     * lines for specific purposes:
     * </p>
     * <ul>
     * <li><b>{@link yfiles.input.CollectGraphSnapLinesEventArgs#addFixedNodeSnapLine}</b> – Adds an orthogonal snap line for nodes.</li>
     * <li><b>{@link yfiles.input.CollectGraphSnapLinesEventArgs#addFixedSegmentSnapLine}</b> – Adds an orthogonal snap line for edge segments and bends.</li>
     * <li><b>{@link yfiles.input.CollectGraphSnapLinesEventArgs#addPortSnapLine}</b> – Adds an orthogonal snap line for ports.</li>
     * <li><b>{@link yfiles.input.CollectGraphSnapLinesEventArgs#addAdditionalSnapLine}</b> – Adds an additional orthogonal snap line to which all element will snap.</li>
     * </ul>
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface CollectGraphSnapLinesEventArgs extends yfiles.input.InputModeEventArgs{}
    export class CollectGraphSnapLinesEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.CollectGraphSnapLinesEventArgs} class.
       * @param context The context where the event is being used in.
       * @param fixedNodeSnapLines The fixed node snap lines collection to add to.
       * @param fixedSegmentSnapLines The fixed segment snap lines collection to add to.
       * @param fixedPortSnapLines The fixed port snap lines collection to add to.
       * @param additionalSnapLines The additional snap lines collection to add to.
       */
      constructor(context:yfiles.input.IInputModeContext,fixedNodeSnapLines:yfiles.collections.ICollection<yfiles.input.OrthogonalSnapLine>,fixedSegmentSnapLines:yfiles.collections.ICollection<yfiles.input.OrthogonalSnapLine>,fixedPortSnapLines:yfiles.collections.ICollection<yfiles.input.OrthogonalSnapLine>,additionalSnapLines:yfiles.collections.ICollection<yfiles.input.OrthogonalSnapLine>);
      /**
       * Adds a snap line all elements will snap to.
       * @param snapLine The snap line to add to the list of lines that all elements snap to by default.
       * @see yfiles.input.GraphSnapContext#additionalSnapLines
       */
      addAdditionalSnapLine(snapLine:yfiles.input.OrthogonalSnapLine):void;
      /**
       * Adds a snap line for nodes.
       * @param snapLine The snap line to add to the list of lines that nodes snap to by default.
       * @see yfiles.input.GraphSnapContext#fixedNodeSnapLines
       */
      addFixedNodeSnapLine(snapLine:yfiles.input.OrthogonalSnapLine):void;
      /**
       * Adds a snap line for edge segments and bends.
       * @param snapLine The snap line to add to the list of lines that segments snap to by default.
       * @see yfiles.input.GraphSnapContext#fixedSegmentSnapLines
       */
      addFixedSegmentSnapLine(snapLine:yfiles.input.OrthogonalSnapLine):void;
      /**
       * Adds a snap line for ports.
       * @param snapLine The snap line to add to the list of lines that ports snap to by default.
       * @see yfiles.input.GraphSnapContext#fixedPortSnapLines
       */
      addPortSnapLine(snapLine:yfiles.input.OrthogonalSnapLine):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Abstract base class for {@link yfiles.input.GraphViewerInputMode} and {@link yfiles.input.GraphEditorInputMode}.
     * <p>
     * This class is not supposed to and in fact cannot be extended from user code. It only exists to provide common
     * functionality to the two aforementioned input modes. If customizations are wanted or needed both {@link yfiles.input.GraphViewerInputMode}
     * and {@link yfiles.input.GraphEditorInputMode} can be extended.
     * </p>
     * @class
     * @extends {yfiles.input.MultiplexingInputMode}
     */
    export interface GraphInputMode extends yfiles.input.MultiplexingInputMode{}
    export class GraphInputMode {
      /**
       * Clears the selection on the current {@link yfiles.input.GraphInputMode#graphSelection}.
       * <p>
       * This implementation may be overridden for customizations. All it does is call {@link yfiles.view.ISelectionModel.<T>#clear}.
       * </p>
       */
      clearSelection():void;
      /**
       * Actually performs the click on the given item.
       * <p>
       * This will raise the {@link yfiles.input.GraphInputMode#addItemLeftClickedListener ItemLeftClicked}, {@link yfiles.input.GraphInputMode#addItemRightClickedListener ItemRightClicked}
       * (depending on the button), and {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} events. If
       * either event is unhandled by all its event handlers the item will be {@link yfiles.input.GraphInputMode#setSelected selected} and {@link yfiles.input.GraphInputMode#setCurrentItem focused}. Before selection
       * {@link yfiles.input.GraphInputMode#clearSelection the current selection will be cleared}, unless the {@link yfiles.input.GraphInputMode#multiSelectionRecognizer} is triggered.
       * </p>
       * @param item The item that has been clicked.
       * @param evt The original event arguments for the click. Setting its {@link #handled} property to <code>true</code> will indicate that the click was handled. By
       * default this happens when the clicked item is either selected or focused.
       * @protected
       */
      click(item:yfiles.graph.IModelItem,evt:yfiles.input.ClickEventArgs):void;
      /**
       * Clears the selection on click if {@link yfiles.view.CanvasComponent#lastMouseEvent the click} is not recognized by {@link yfiles.input.GraphInputMode#multiSelectionRecognizer}.
       * <p>
       * This method is only called if no item has been hit and at least one item is currently selected.
       * </p>
       * <p>
       * This will use the {@link yfiles.input.GraphInputMode#clearSelection} method to deselect all items.
       * </p>
       * @param context The context where the click appeared
       * @returns Whether the selection has been cleared by this method.
       * @protected
       */
      clickClearSelection(context:yfiles.input.IInputModeContext):boolean;
      /**
       * Copies the currently selected elements to the clipboard.
       */
      copy():void;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#clickInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#clickInputMode} property.
       * </p>
       * <p>
       * This implementation {@link yfiles.input.ClickInputMode#activeButtons activates} both {@link yfiles.view.MouseButtons#LEFT} and {@link yfiles.view.MouseButtons#RIGHT}
       * </p>
       * @returns a new instance of ClickInputMode
       * @protected
       */
      createClickInputMode():yfiles.input.ClickInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#contextMenuInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#contextMenuInputMode} property.
       * </p>
       * @returns a new instance of ContextMenuInputMode
       * @protected
       */
      createContextMenuInputMode():yfiles.input.ContextMenuInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#itemHoverInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#itemHoverInputMode} property.
       * </p>
       * @returns a new instance of {@link #itemHoverInputMode} with the {@link #hoverItems} property set to {@link #NONE}.
       * @protected
       */
      createItemHoverInputMode():yfiles.input.ItemHoverInputMode;
      /**
       * Factory method that creates the {@link yfiles.input.GraphInputMode#keyboardInputMode} instance.
       * @returns 
       * @protected
       */
      createKeyboardInputMode():yfiles.input.KeyboardInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#marqueeSelectionInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#marqueeSelectionInputMode} property.
       * </p>
       * @returns a new instance of MarqueeSelectionInputMode
       * @protected
       */
      createMarqueeSelectionInputMode():yfiles.input.MarqueeSelectionInputMode;
      /**
       * Factory method for the {@link yfiles.input.MouseHoverInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.MouseHoverInputMode} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createMouseHoverInputMode():yfiles.input.MouseHoverInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#moveViewportInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#moveViewportInputMode} property.
       * </p>
       * @returns a new instance of {@link #moveViewportInputMode}
       * @protected
       */
      createMoveViewportInputMode():yfiles.input.MoveViewportInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#navigationInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#navigationInputMode} property.
       * </p>
       * @returns a new instance of NavigationInputMode
       * @protected
       */
      createNavigationInputMode():yfiles.input.NavigationInputMode;
      /**
       * Helper method that yields a suitably configured {@link yfiles.input.SelectionEventArgs.<T>} using the {@link yfiles.input.GraphInputMode#graphSelection} for this input mode.
       * @param context An input mode context that is available in the {@link }.
       * <p>
       * Can be <code>null</code> in which case a new context for this instance is created automatically.
       * </p>
       * @returns A selection event argument that is configured for this instance.
       * @protected
       */
      createSelectionEventArgs(context:yfiles.input.IInputModeContext):yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#tapInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#tapInputMode} property.
       * </p>
       * @returns a new instance of TapInputMode
       * @protected
       */
      createTapInputMode():yfiles.input.TapInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphInputMode#waitInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphInputMode#waitInputMode} property.
       * </p>
       * @returns a new instance of {@link #waitInputMode}
       * @protected
       */
      createWaitInputMode():yfiles.input.WaitInputMode;
      /**
       * Actually performs a double-click on the given item.
       * <p>
       * This will raise the {@link yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} and {@link yfiles.input.GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}
       * (depending on the button), and {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
       * events.
       * </p>
       * @param item The item that has been double-clicked.
       * @param evt The original event arguments for the click. Setting its {@link #handled} property to <code>true</code> will indicate that the double-click was
       * handled and events on other items in the same location are no longer raised.
       * @protected
       */
      doubleClick(item:yfiles.graph.IModelItem,evt:yfiles.input.ClickEventArgs):void;
      /**
       * Actually performs a double-tap on the given item.
       * <p>
       * This will raise the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event.
       * </p>
       * @param item The item that has been double-tapped.
       * @param evt The original event arguments for the tap. Setting its {@link #handled} property to <code>true</code> will indicate that the double-tap was handled
       * and events on other items in the same location are no longer raised.
       * @protected
       */
      doubleTap(item:yfiles.graph.IModelItem,evt:yfiles.input.TapEventArgs):void;
      /**
       * Used as a callback to find the items hit underneath a certain point.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphEditorInputMode#hitTester} to determine the hit items.
       * </p>
       * @param context The context to use for to the {@link #isHit} callback.
       * @param location The location to test.
       * @param tests An array of {@link } values that encode for which model items the hit test should be performed for
       * prioritizing.
       * @param [filter=null] The predicate that can be used to filter the results. May be <code>null</code>.
       * @returns An enumerable over the items that have been found for the location.
       */
      findItems(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,tests:yfiles.graph.GraphItemTypes[],filter?:(obj:yfiles.graph.IModelItem)=>boolean):yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Used as a callback to find the items hit underneath a certain point.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphEditorInputMode#hitTester} to determine the hit items.
       * </p>
       * @param location The location to test.
       * @param tests An array of {@link } values that encode for which model items the hit test should be performed for
       * prioritizing.
       * @param [filter=null] The predicate that can be used to filter the results. May be <code>null</code>.
       * @returns An enumerable over the items that have been found for the location.
       */
      findItems(location:yfiles.geometry.Point,tests:yfiles.graph.GraphItemTypes[],filter?:(obj:yfiles.graph.IModelItem)=>boolean):yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Called by {@link yfiles.input.GraphInputMode#click} to query the <code>item</code> for an {@link yfiles.input.IClickListener} in its {@link yfiles.graph.ILookup#lookup}
       * and handle it appropriately.
       * <p>
       * This method will query the {@link yfiles.input.IClickListener} and check whether it was {@link yfiles.input.IClickListener#getHitTestable hit} by the click, and if so, will invoke
       * {@link yfiles.input.IClickListener#onClicked} and return <code>true</code>.
       * </p>
       * @param context The context for the click.
       * @param item The item that has been clicked.
       * @param location The click location.
       * @returns Whether the action has been invoked and handling should be stopped.
       * @see yfiles.input.IClickListener
       * @see yfiles.input.GraphInputMode#click
       * @protected
       */
      handleClickListener(context:yfiles.input.IInputModeContext,item:yfiles.graph.IModelItem,location:yfiles.geometry.Point):boolean;
      /**
       * Performs marquee-selection with the given rectangle.
       * <p>
       * It delegates directly to the {@link yfiles.input.GraphInputMode#onMarqueeSelect} overload internally.
       * </p>
       * @param marqueeRectangle The selection rectangle.
       */
      marqueeSelect(marqueeRectangle:yfiles.geometry.Rect):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addCanvasClickedListener CanvasClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ClickEventArgs#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the information about the click.
       * @protected
       */
      onCanvasClicked(evt:yfiles.input.ClickEventArgs):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#clickInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onClickInputModeChanged(oldMode:yfiles.input.ClickInputMode,newMode:yfiles.input.ClickInputMode):void;
      /**
       * Called when the {@link yfiles.input.ContextMenuInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onContextMenuInputModeChanged(oldMode:yfiles.input.ContextMenuInputMode,newMode:yfiles.input.ContextMenuInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the item that has been clicked.
       * @protected
       */
      onItemClicked(evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the item that has been double clicked.
       * @protected
       */
      onItemDoubleClicked(evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#itemHoverInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onItemHoverInputModeChanged(oldMode:yfiles.input.ItemHoverInputMode,newMode:yfiles.input.ItemHoverInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addItemLeftClickedListener ItemLeftClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the item that has been clicked.
       * @protected
       */
      onItemLeftClicked(evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the item that has been double clicked.
       * @protected
       */
      onItemLeftDoubleClicked(evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addItemRightClickedListener ItemRightClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the item that has been clicked.
       * @protected
       */
      onItemRightClicked(evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked} event.
       * <p>
       * This implementation will stop the event propagation as soon as an event handler has set the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code>.
       * </p>
       * @param evt The {@link } instance that contains the item that has been double clicked.
       * @protected
       */
      onItemRightDoubleClicked(evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#keyboardInputMode} changed.
       * @param oldMode The old {@link #keyboardInputMode}.
       * @param newMode The new {@link #keyboardInputMode}.
       * @protected
       */
      onKeyboardInputModeChanged(oldMode:yfiles.input.KeyboardInputMode,newMode:yfiles.input.KeyboardInputMode):void;
      /**
       * Performs marquee-selection with the given rectangle.
       * <p>
       * This method is called internally when either the {@link yfiles.input.GraphInputMode#marqueeSelect} method is called or the user
       * performs a marquee-selection gesture via {@link yfiles.input.GraphInputMode#marqueeSelectionInputMode}.
       * </p>
       * @param context The input mode context.
       * @param marqueeRectangle The selection rectangle.
       * @protected
       */
      onMarqueeSelect(context:yfiles.input.IInputModeContext,marqueeRectangle:yfiles.geometry.Rect):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#marqueeSelectionInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onMarqueeSelectionInputModeChanged(oldMode:yfiles.input.MarqueeSelectionInputMode,newMode:yfiles.input.MarqueeSelectionInputMode):void;
      /**
       * Selects the given elements in the editor inside the provided marquee-selection rectangle.
       * <p>
       * This implementation iterates over the items and queries them for an implementation of {@link yfiles.input.IMarqueeTestable}
       * using their {@link yfiles.graph.ILookup#lookup}. This instance will be used to determine whether the item should be selected.
       * </p>
       * <p>
       * This method is, in the default implementation, only called with individual item types in <code>items</code>.
       * </p>
       * @param context The input mode context.
       * @param marqueeRectangle The rectangle in world coordinates.
       * @param items The items to iterate over.
       * @param predicate The predicate that determines whether the provided item should be tested at all. May be <code>null</code>.
       * @see yfiles.input.GraphInputMode#setSelected
       * @see yfiles.input.GraphInputMode#onMarqueeSelect
       * @protected
       */
      onMarqueeSelectItems(context:yfiles.input.IInputModeContext,marqueeRectangle:yfiles.geometry.Rect,items:yfiles.collections.IEnumerable<yfiles.graph.IModelItem>,predicate:(obj:yfiles.graph.IModelItem)=>boolean):void;
      /**
       * Called when the {@link yfiles.input.MouseHoverInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onMouseHoverInputModeChanged(oldMode:yfiles.input.MouseHoverInputMode,newMode:yfiles.input.MouseHoverInputMode):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#moveViewportInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onMoveViewportInputModeChanged(oldMode:yfiles.input.MoveViewportInputMode,newMode:yfiles.input.MoveViewportInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addMultiSelectionFinishedListener MultiSelectionFinished} event.
       * @param evt
       * @protected
       */
      onMultiSelectionFinished(evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addMultiSelectionStartedListener MultiSelectionStarted} event.
       * @param evt
       * @protected
       */
      onMultiSelectionStarted(evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#navigationInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onNavigationInputModeChanged(oldMode:yfiles.input.NavigationInputMode,newMode:yfiles.input.NavigationInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addPopulateItemContextMenuListener PopulateItemContextMenu} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPopulateItemContextMenu(evt:yfiles.input.PopulateItemContextMenuEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphInputMode#addQueryItemToolTipListener QueryItemToolTip} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onQueryItemToolTip(evt:yfiles.input.QueryItemToolTipEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#tapInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onTapInputModeChanged(oldMode:yfiles.input.TapInputMode,newMode:yfiles.input.TapInputMode):void;
      /**
       * Called when the {@link yfiles.input.GraphInputMode#waitInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onWaitInputModeChanged(oldMode:yfiles.input.WaitInputMode,newMode:yfiles.input.WaitInputMode):void;
      /**
       * Clears the current selection and selects all nodes and bends in this graph.
       * <p>
       * By default, this method will be called in response to a Ctrl+A key press recognized by {@link yfiles.input.GraphInputMode#keyboardInputMode}.
       * </p>
       * @see yfiles.input.GraphInputMode#clearSelection
       * @see yfiles.input.GraphInputMode#setSelected
       * @see yfiles.input.ICommand#SELECT_ALL
       * @see yfiles.input.GraphInputMode#selectableItems
       * @see yfiles.input.GraphInputMode#ignoreVoidStyles
       */
      selectAll():void;
      /**
       * Callback that is used by this instance to set the current item on the {@link yfiles.input.NavigationInputMode#setCurrentItem NavigationInputMode}
       * @param item The item to set as the new "current" item.
       */
      setCurrentItem(item:yfiles.graph.IModelItem):void;
      /**
       * Uses the {@link yfiles.input.GraphInputMode#graphSelection} to select the given item.
       * @param item The item to set the selection state on.
       * @param selected The new selection state.
       */
      setSelected(item:yfiles.graph.IModelItem,selected:boolean):void;
      /**
       * Callback predicate method that is used to determine whether the given <code>item</code> can be {@link yfiles.input.GraphInputMode#click clicked}.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphInputMode#clickableItems} value to determine whether the item should be clicked. Note that this method will not be
       * called for items that don't match {@link yfiles.input.GraphInputMode#clickableItems} for performance reasons.
       * </p>
       * @param item The item to check.
       * @returns Whether the item should be {@link #click clicked} in response to a detected mouse click.
       * @protected
       */
      shouldClick(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback predicate method that is used to determine whether the given <code>item</code> should be selected when
       * {@link yfiles.input.GraphInputMode#click clicked with the primary mouse button}.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphInputMode#clickSelectableItems} value to determine whether the item should be selected. Note that this method will not be
       * called for items that don't match {@link yfiles.input.GraphInputMode#clickSelectableItems} and {@link yfiles.input.GraphInputMode#selectableItems} for performance reasons.
       * </p>
       * @param item The item to check.
       * @returns Whether the item should be selected in response to a detected mouse click with the primary button.
       * @protected
       */
      shouldClickSelect(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback predicate method that is used to determine whether the given <code>item</code> can be {@link yfiles.input.GraphInputMode#doubleClick double clicked}.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphInputMode#shouldClick} method to determine whether the item should be
       * double clicked. Note that this method will not be called for items that don't match {@link yfiles.input.GraphInputMode#clickableItems} for performance reasons.
       * </p>
       * @param item The item to check.
       * @returns Whether the item should be {@link #doubleClick double clicked} in response to a detected mouse click.
       * @protected
       */
      shouldDoubleClick(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback that determines whether the provided {@link yfiles.graph.IModelItem} should be {@link yfiles.input.GraphInputMode#setCurrentItem set to the current item}.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphInputMode#focusableItems} property to determine whether the item can be set to the current item. It also honors the
       * {@link yfiles.input.GraphInputMode#ignoreVoidStyles} property.
       * </p>
       * @param item The item to check.
       * @returns Whether to set the item as the current item.
       * @protected
       */
      shouldFocus(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback method that is used to determine which of the built-in {@link yfiles.input.ICommand}s to install.
       * <p>
       * This implementation unconditionally returns <code>true</code>, subclasses may override this method to adjust the behavior.
       * </p>
       * @param command The command to install.
       * @returns Whether to install this command.
       * @protected
       */
      shouldInstallCommand(command:yfiles.input.ICommand):boolean;
      /**
       * Callback method used by {@link yfiles.input.GraphInputMode#marqueeSelect} that takes {@link yfiles.input.GraphInputMode#marqueeSelectableItems} and {@link yfiles.input.GraphInputMode#shouldSelectItem}
       * into account.
       * @param item The item to check.
       * @returns Whether to possibly marquee select the item.
       * @protected
       */
      shouldMarqueeSelect(item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether for the given item a context menu should be queried.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphInputMode#contextMenuItems} property to determine whether a context menu should be queried for the given item.
       * </p>
       * @param item The item to check.
       * @returns Whether to raise a {@link #addPopulateItemContextMenuListener PopulateItemContextMenu} event for
       * the given item.
       * @protected
       */
      shouldPopulateContextMenu(item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether for the given item a tool tip should be queried.
       * <p>
       * This implementation uses the {@link yfiles.input.GraphInputMode#toolTipItems} property to determine whether tool tip should be queried for the given item.
       * </p>
       * @param item The item to check.
       * @returns Whether to raise a {@link #addQueryItemToolTipListener QueryItemToolTip} event for the
       * given item.
       * @protected
       */
      shouldQueryToolTip(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback predicate method that is used as a broad classification whether an item can generally be selected.
       * <p>
       * This implementation uses and honors the {@link yfiles.input.GraphInputMode#selectableItems}, {@link yfiles.input.GraphInputMode#ignoreVoidStyles} and {@link yfiles.input.GraphInputMode#selectablePredicate} properties. This method is by default also called by {@link yfiles.input.GraphInputMode#shouldClickSelect}
       * and {@link yfiles.input.GraphInputMode#shouldMarqueeSelect}, so turning selection off here also turns it off in other places.
       * </p>
       * @param item The item to check.
       * @returns <code>true</code> if the item is selectable in general, <code>false</code> if not.
       * @protected
       */
      shouldSelectItem(item:yfiles.graph.IModelItem):boolean;
      /**
       * Actually performs the tap on the given item.
       * <p>
       * This will raise the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event. If the event
       * is unhandled by all its event handlers the item will be {@link yfiles.input.GraphInputMode#setSelected selected} and {@link yfiles.input.GraphInputMode#setCurrentItem focused}. Before selection
       * {@link yfiles.input.GraphInputMode#clearSelection the current selection will be cleared}.
       * </p>
       * @param item The item that has been tapped.
       * @param evt The original event arguments for the tap. Setting its {@link #handled} property to <code>true</code> will indicate that the tap was handled. By
       * default this happens when the tapped item is either selected or focused.
       * @protected
       */
      tap(item:yfiles.graph.IModelItem,evt:yfiles.input.TapEventArgs):void;
      /**
       * Gets the graph instance from the {@link yfiles.input.MultiplexingInputMode#inputModeContext}.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Gets the graph selection from the {@link yfiles.input.MultiplexingInputMode#inputModeContext}.
       * @type {yfiles.view.IGraphSelection}
       */
      graphSelection:yfiles.view.IGraphSelection;
      /**
       * Gets or sets the order of the types of items that should be used to determine what item has been clicked or tapped.
       * <p>
       * Change this field to adjust which items will be {@link yfiles.input.GraphInputMode#setSelected selected} if there are multiple items at a given location. The default
       * order is
       * </p>
       * <ul>
       * <li>{@link yfiles.graph.GraphItemTypes#BEND}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#EDGE_LABEL}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#EDGE}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#NODE}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#NODE_LABEL}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#PORT}</li>
       * </ul>
       * @see yfiles.input.GraphInputMode#clickSelectableItems
       * @see yfiles.input.GraphInputMode#selectableItems
       * @see yfiles.input.GraphInputMode#findItems
       * @type {Array.<yfiles.graph.GraphItemTypes>}
       */
      clickHitTestOrder:yfiles.graph.GraphItemTypes[];
      /**
       * Gets or sets the order of the types of items that should be used to determine what item has been double-clicked or
       * double-tapped.
       * <p>
       * Change this field to adjust which items will be considered when {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener double-clicked} if there are multiple items at a given
       * location. The default order is
       * </p>
       * <ul>
       * <li>{@link yfiles.graph.GraphItemTypes#EDGE_LABEL}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#NODE_LABEL}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#BEND}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#EDGE}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#NODE}</li>
       * <li>{@link yfiles.graph.GraphItemTypes#PORT}</li>
       * </ul>
       * @see yfiles.input.GraphInputMode#clickSelectableItems
       * @see yfiles.input.GraphInputMode#selectableItems
       * @see yfiles.input.GraphInputMode#findItems
       * @type {Array.<yfiles.graph.GraphItemTypes>}
       */
      doubleClickHitTestOrder:yfiles.graph.GraphItemTypes[];
      /**
       * Gets or sets the event recognizer that will be queried to decide if a click should select the item that is currently
       * visible under the mouse cursor, without giving more important items higher priority.
       * <p>
       * If the recognizer determines that the user wanted to use the detail clicking behavior, the {@link yfiles.input.GraphEditorInputMode#hitTester} will be queried for the
       * given item and the first item that is returned will be considered a hit. Otherwise all hit items are examined and are
       * prioritized by item type. E.g. by default clicking on a node will select the node, even if there is a node label which
       * has been clicked at the same point. If detail selection is recognized, the label will be selected.
       * </p>
       * <p>
       * The default is {@link yfiles.input.KeyEventRecognizers#SHIFT_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      detailSelectionRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that will be queried to decide if a click should cycle through all the items that are
       * currently under the mouse cursor.
       * <p>
       * If the recognizer determines that the user wanted to use the cyclic click selection behavior, the {@link yfiles.input.GraphEditorInputMode#hitTester} will be queried for
       * the given item and previously reported items will be ignored. E.g. if multiple nodes are overlapping each other,
       * clicking on them will report the first item for the first click, the second one for the second click and so on. If all
       * elements have been reported previously, the first one is going to be reported again. If the order of the elements at the
       * clicked location changes, the cyclic selection is restarted anew.
       * </p>
       * <p>
       * The default is {@link yfiles.input.KeyEventRecognizers#ALT_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cyclicSelectionRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that will be queried to decide if a click is deemed a multi selection gesture.
       * <p>
       * The default is a combination of {@link yfiles.input.KeyEventRecognizers#CONTROL_PRESSED} or
       * {@link yfiles.input.KeyEventRecognizers#META_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      multiSelectionRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Determines which types of items should be reported through the {@link yfiles.input.GraphInputMode#addItemClickedListener item click events}.
       * <p>
       * This also affects taps.
       * </p>
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphInputMode#shouldClick
       * @type {yfiles.graph.GraphItemTypes}
       */
      clickableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets the items that can be given focus via the {@link yfiles.input.GraphInputMode#setCurrentItem} method.
       * <p>
       * The focusable items. The default is {@link yfiles.graph.GraphItemTypes#NODE}
       * </p>
       * @default yfiles.graph.GraphItemTypes.NODE
       * @type {yfiles.graph.GraphItemTypes}
       */
      focusableItems:yfiles.graph.GraphItemTypes;
      /**
       * Determines which types of items should be selectable at all.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphInputMode#setSelected
       * @type {yfiles.graph.GraphItemTypes}
       */
      selectableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets a predicate that is queried to determine whether a given {@link yfiles.graph.IModelItem} is selectable.
       * <p>
       * This allows to further restrict which items are selectable, compared to {@link yfiles.input.GraphInputMode#selectableItems}. If this property is set to <code>null</code> the effect is
       * the same as a predicate that always returns <code>true</code>.
       * </p>
       * @type {function(yfiles.graph.IModelItem): boolean}
       */
      selectablePredicate:(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Determines which types of items should be selectable by mouse clicks.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphInputMode#shouldClickSelect
       * @see yfiles.input.GraphInputMode#setSelected
       * @type {yfiles.graph.GraphItemTypes}
       */
      clickSelectableItems:yfiles.graph.GraphItemTypes;
      /**
       * Determines which types of items should be selected during {@link yfiles.input.GraphInputMode#marqueeSelect marquee selections}.
       * <p>
       * The default is a combination of {@link yfiles.graph.GraphItemTypes#NODE}, {@link yfiles.graph.GraphItemTypes#EDGE}, and
       * {@link yfiles.graph.GraphItemTypes#BEND}.
       * </p>
       * @see yfiles.input.GraphInputMode#shouldMarqueeSelect
       * @see yfiles.input.GraphInputMode#setSelected
       * @type {yfiles.graph.GraphItemTypes}
       */
      marqueeSelectableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets a value indicating whether void styles ({@link yfiles.styles.VoidNodeStyle}, {@link yfiles.styles.VoidEdgeStyle},
       * {@link yfiles.styles.VoidLabelStyle}, and {@link yfiles.styles.VoidPortStyle}) should be ignored when selecting or focusing items.
       * @type {boolean}
       */
      ignoreVoidStyles:boolean;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#contextMenuInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createContextMenuInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onContextMenuInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>60</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.ContextMenuInputMode}
       */
      contextMenuInputMode:yfiles.input.ContextMenuInputMode;
      /**
       * Gets or sets the types of items a context menu should be queried for.
       * <p>
       * The items for which a context menu should be {@link yfiles.input.GraphInputMode#addPopulateItemContextMenuListener queried}. The default value is
       * {@link yfiles.graph.GraphItemTypes#NODE}|{@link yfiles.graph.GraphItemTypes#EDGE}.
       * </p>
       * @default 1 | 2
       * @type {yfiles.graph.GraphItemTypes}
       */
      contextMenuItems:yfiles.graph.GraphItemTypes;
      /**
       * Adds the given listener for the <code>PopulateItemContextMenu</code> event that occurs when the {@link yfiles.input.ContextMenuInputMode context menu} over an item is about to
       * be opened to determine the contents of the Menu.
       * <p>
       * This method will only be called for items that match the {@link yfiles.input.GraphInputMode#contextMenuItems} type.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#removePopulateItemContextMenuListener
       */
      addPopulateItemContextMenuListener(listener:(sender:Object,evt:yfiles.input.PopulateItemContextMenuEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>PopulateItemContextMenu</code> event that occurs when the {@link yfiles.input.ContextMenuInputMode context menu} over an item is about
       * to be opened to determine the contents of the Menu.
       * <p>
       * This method will only be called for items that match the {@link yfiles.input.GraphInputMode#contextMenuItems} type.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#addPopulateItemContextMenuListener
       */
      removePopulateItemContextMenuListener(listener:(sender:Object,evt:yfiles.input.PopulateItemContextMenuEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#mouseHoverInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createMouseHoverInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onMouseHoverInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>100</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.MouseHoverInputMode}
       */
      mouseHoverInputMode:yfiles.input.MouseHoverInputMode;
      /**
       * Gets the types of the items that should be queried a tool tip for.
       * <p>
       * The items for which a tool tip text should be {@link yfiles.input.GraphInputMode#addQueryItemToolTipListener queried}. The default value is
       * {@link yfiles.graph.GraphItemTypes#NODE}|{@link yfiles.graph.GraphItemTypes#EDGE}|{@link yfiles.graph.GraphItemTypes#LABEL}|{@link yfiles.graph.GraphItemTypes#PORT}|.
       * </p>
       * @default 1 | 2 | 12 | 16
       * @type {yfiles.graph.GraphItemTypes}
       */
      toolTipItems:yfiles.graph.GraphItemTypes;
      /**
       * Adds the given listener for the <code>QueryItemToolTip</code> event that occurs when the mouse is {@link yfiles.input.MouseHoverInputMode hovering} over an item to determine
       * the tool tip to display.
       * <p>
       * This method will only be called for items that match the {@link yfiles.input.GraphInputMode#toolTipItems} type.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#removeQueryItemToolTipListener
       */
      addQueryItemToolTipListener(listener:(sender:Object,evt:yfiles.input.QueryItemToolTipEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>QueryItemToolTip</code> event that occurs when the mouse is {@link yfiles.input.MouseHoverInputMode hovering} over an item to
       * determine the tool tip to display.
       * <p>
       * This method will only be called for items that match the {@link yfiles.input.GraphInputMode#toolTipItems} type.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#addQueryItemToolTipListener
       */
      removeQueryItemToolTipListener(listener:(sender:Object,evt:yfiles.input.QueryItemToolTipEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Gets or sets the {@link yfiles.input.ClickInputMode} that is used by this instance to determine clicks.
       * <p>
       * Whenever the mode detects a {@link yfiles.input.ClickInputMode#addClickedListener click}, this mode will use the {@link yfiles.input.IHitTester.<T>} from the {@link yfiles.input.MultiplexingInputMode#inputModeContext} to determine the items being hit. If they match the
       * {@link yfiles.input.GraphInputMode#clickableItems} type, the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event will be triggered. If
       * they {@link yfiles.input.GraphInputMode#shouldFocus}, the item will be {@link yfiles.input.GraphInputMode#setCurrentItem set as the current item}.
       * </p>
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createClickInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onClickInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>10</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @see yfiles.input.GraphInputMode#onItemClicked
       * @type {yfiles.input.ClickInputMode}
       */
      clickInputMode:yfiles.input.ClickInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#tapInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createTapInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onTapInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>20</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.TapInputMode}
       */
      tapInputMode:yfiles.input.TapInputMode;
      /**
       * Adds the given listener for the <code>ItemClicked</code> event that occurs when an item has been clicked.
       * <p>
       * If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore.
       * </p>
       * <p>
       * This event will also be raised for {@link yfiles.input.GraphInputMode#tap taps}.
       * </p>
       * <p>
       * Button-specific events ({@link yfiles.input.GraphInputMode#addItemLeftClickedListener ItemLeftClicked} and
       * {@link yfiles.input.GraphInputMode#addItemRightClickedListener ItemRightClicked}) will be raised before this event.
       * If one of those is marked {@link yfiles.input.ItemClickedEventArgs.<T>#handled} this event will not be raised anymore.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightClickedListener
       * @see yfiles.input.GraphInputMode#removeItemClickedListener
       */
      addItemClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemClicked</code> event that occurs when an item has been clicked.
       * <p>
       * If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore.
       * </p>
       * <p>
       * This event will also be raised for {@link yfiles.input.GraphInputMode#tap taps}.
       * </p>
       * <p>
       * Button-specific events ({@link yfiles.input.GraphInputMode#addItemLeftClickedListener ItemLeftClicked} and
       * {@link yfiles.input.GraphInputMode#addItemRightClickedListener ItemRightClicked}) will be raised before this event.
       * If one of those is marked {@link yfiles.input.ItemClickedEventArgs.<T>#handled} this event will not be raised anymore.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightClickedListener
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       */
      removeItemClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemLeftClicked</code> event that occurs when an item has been left clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
       * case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       * @see yfiles.input.GraphInputMode#removeItemLeftClickedListener
       */
      addItemLeftClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemLeftClicked</code> event that occurs when an item has been left clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
       * case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftClickedListener
       */
      removeItemLeftClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemRightClicked</code> event that occurs when an item has been right clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
       * case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       * @see yfiles.input.GraphInputMode#removeItemRightClickedListener
       */
      addItemRightClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemRightClicked</code> event that occurs when an item has been right clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked} event will not be raised in that
       * case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightClickedListener
       */
      removeItemRightClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemDoubleClicked</code> event that occurs when an item has been double clicked.
       * <p>
       * If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore.
       * </p>
       * <p>
       * Depending on the value of the {@link yfiles.input.ClickInputMode#doubleClickPolicy} property of {@link yfiles.input.GraphInputMode#clickInputMode} this event may be preceded by no, one, or two {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked}
       * events (as well as the corresponding button-specific event).
       * </p>
       * <p>
       * Button-specific events ({@link yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} and
       * {@link yfiles.input.GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}) will be raised before this
       * event. If one of those is marked {@link yfiles.input.ItemClickedEventArgs.<T>#handled} this event will not be raised anymore.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       * @see yfiles.input.GraphInputMode#removeItemDoubleClickedListener
       */
      addItemDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemDoubleClicked</code> event that occurs when an item has been double clicked.
       * <p>
       * If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore.
       * </p>
       * <p>
       * Depending on the value of the {@link yfiles.input.ClickInputMode#doubleClickPolicy} property of {@link yfiles.input.GraphInputMode#clickInputMode} this event may be preceded by no, one, or two {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked}
       * events (as well as the corresponding button-specific event).
       * </p>
       * <p>
       * Button-specific events ({@link yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked} and
       * {@link yfiles.input.GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked}) will be raised before this
       * event. If one of those is marked {@link yfiles.input.ItemClickedEventArgs.<T>#handled} this event will not be raised anymore.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       */
      removeItemDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemLeftDoubleClicked</code> event that occurs when an item has been left double clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
       * in that case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       * @see yfiles.input.GraphInputMode#removeItemLeftDoubleClickedListener
       */
      addItemLeftDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemLeftDoubleClicked</code> event that occurs when an item has been left double clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
       * in that case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       */
      removeItemLeftDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>ItemRightDoubleClicked</code> event that occurs when an item has been right double clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
       * in that case.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       * @see yfiles.input.GraphInputMode#removeItemRightDoubleClickedListener
       */
      addItemRightDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemRightDoubleClicked</code> event that occurs when an item has been right double clicked.
       * <p>
       * This event will be raised before the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked}
       * event. If one of the event handlers sets the {@link yfiles.input.ItemClickedEventArgs.<T>#handled} property to <code>true</code> the event will not be propagated anymore. This also means
       * that the {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked} event will not be raised
       * in that case.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#clickableItems
       * @see yfiles.input.GraphInputMode#addItemDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener
       * @see yfiles.input.GraphInputMode#addItemRightDoubleClickedListener
       */
      removeItemRightDoubleClickedListener(listener:(sender:Object,evt:yfiles.input.ItemClickedEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>CanvasClicked</code> event that occurs when the empty canvas area has been clicked.
       * <p>
       * If one of the event handlers sets the {@link yfiles.input.ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#removeCanvasClickedListener
       */
      addCanvasClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CanvasClicked</code> event that occurs when the empty canvas area has been clicked.
       * <p>
       * If one of the event handlers sets the {@link yfiles.input.ClickEventArgs#handled} property to <code>true</code> the event will not be propagated anymore.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#addItemClickedListener
       * @see yfiles.input.GraphInputMode#addCanvasClickedListener
       */
      removeCanvasClickedListener(listener:(sender:Object,evt:yfiles.input.ClickEventArgs)=>void):void;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#marqueeSelectionInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createMarqueeSelectionInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onMarqueeSelectionInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>50</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.MarqueeSelectionInputMode}
       */
      marqueeSelectionInputMode:yfiles.input.MarqueeSelectionInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#navigationInputMode} property.
       * <p>
       * This mode is responsible for navigating and traversing the elements in the {@link yfiles.graph.IGraph}. If the field has not yet
       * been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createNavigationInputMode} will be
       * called. Upon change the {@link yfiles.input.GraphInputMode#onNavigationInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>55</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.NavigationInputMode}
       */
      navigationInputMode:yfiles.input.NavigationInputMode;
      /**
       * Adds the given listener for the <code>MultiSelectionStarted</code> event that occurs when a single or multi select operation has been
       * started.
       * <p>
       * The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#removeMultiSelectionStartedListener
       */
      addMultiSelectionStartedListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>MultiSelectionStarted</code> event that occurs when a single or multi select operation has
       * been started.
       * <p>
       * The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#addMultiSelectionStartedListener
       */
      removeMultiSelectionStartedListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>MultiSelectionFinished</code> event that occurs when a single or multi select operation has
       * been finished.
       * <p>
       * The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphInputMode#removeMultiSelectionFinishedListener
       */
      addMultiSelectionFinishedListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>MultiSelectionFinished</code> event that occurs when a single or multi select operation has
       * been finished.
       * <p>
       * The event is <b>not</b> triggered for the individual selection operations that constitute a multi selection operation.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphInputMode#addMultiSelectionFinishedListener
       */
      removeMultiSelectionFinishedListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Gets or sets the {@link yfiles.input.WaitInputMode} that is provided by this instance for those who need to make use of it.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createWaitInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onWaitInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>-1</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @see yfiles.input.WaitInputMode#waiting
       * @type {yfiles.input.WaitInputMode}
       */
      waitInputMode:yfiles.input.WaitInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#itemHoverInputMode} that is provided by this instance for those who need to make use of it.
       * <p>
       * Note that initially the {@link yfiles.input.ItemHoverInputMode#hoverItems} property is set to
       * {@link yfiles.graph.GraphItemTypes#NONE}, which effectively disables the functionality of the mode initially. In order to get
       * the mode to fire events, the property should be set to a corresponding value.
       * </p>
       * <p>
       * If the backing field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createItemHoverInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onItemHoverInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>55</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.ItemHoverInputMode}
       */
      itemHoverInputMode:yfiles.input.ItemHoverInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#moveViewportInputMode} associated with this instance.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphInputMode#createMoveViewportInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphInputMode#onMoveViewportInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>39</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.MoveViewportInputMode}
       */
      moveViewportInputMode:yfiles.input.MoveViewportInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphInputMode#keyboardInputMode}.
       * <p>
       * By default this input mode has a priority of <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.KeyboardInputMode}
       */
      keyboardInputMode:yfiles.input.KeyboardInputMode;
      /**
       * Gets a collection of commands that this input mode will handle.
       * <p>
       * This collection can be modified by removing commands to prevent the input mode from handling them. Previously removed
       * commands can also be re-added again to enable them again. However, the input mode will never handle commands that
       * weren't in the initial collection. Adding those will do nothing.
       * </p>
       * @type {yfiles.collections.ICollection.<yfiles.input.ICommand>}
       */
      availableCommands:yfiles.collections.ICollection<yfiles.input.ICommand>;
      /**
       * Gets or sets a property that determines whether clipboard operations with the usual shortcuts are enabled on the
       * canvas.
       * <p>
       * Default value is <code>true</code>
       * </p>
       * @default true
       * @type {boolean}
       */
      allowClipboardOperations:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} implementation for use in a {@link yfiles.view.GraphComponent} to assign to the {@link yfiles.view.CanvasComponent#inputMode} property.
     * <p>
     * This mode can be used in a viewer-centric application where there is no need to modify the {@link yfiles.graph.IGraph} but
     * browsing of the graph should be convenient. This mode allows for {@link yfiles.input.GraphInputMode#addItemClickedListener clicking on items using the mouse},
     * {@link yfiles.input.GraphInputMode#addQueryItemToolTipListener displaying tool tips for items}, {@link yfiles.input.GraphInputMode#addPopulateItemContextMenuListener showing a context menu}, {@link yfiles.input.NavigationInputMode allowing for easy navigation and traversal},
     * {@link yfiles.input.MoveViewportInputMode moving the view port}, and {@link yfiles.input.GraphInputMode#marqueeSelectableItems optionally doing marquee selection}. Also {@link yfiles.input.ItemHoverInputMode} can be used to get
     * notified of the elements that the mouse is hovering over.
     * </p>
     * <p>
     * Due to the viewer-centric nature of this input mode, collapsing and expanding groups is disabled per default. To enable
     * this feature, set {@link yfiles.input.NavigationInputMode#allowCollapseGroup} and {@link yfiles.input.NavigationInputMode#allowExpandGroup} to <code>true</code>.
     * </p>
     * <p>
     * This input mode manages a set of specialized input modes, each handling a specific part of the interaction with the
     * graph. The following list details those along with their default
     * {@link yfiles.input.IInputMode#priority priorities}. Lower priorities come earlier when handing events.
     * </p>
     * <ul>
     * <li><b>{@link yfiles.input.WaitInputMode} (−1)</b> – Disables this input mode when layouts or animations run.</li>
     * <li><b>{@link yfiles.input.KeyboardInputMode} (0)</b> – Handles commands and arbitrary keyboard shortcuts.</li>
     * <li><b>{@link yfiles.input.ClickInputMode} (10)</b> – Handles mouse clicks and double-clicks.</li>
     * <li><b>{@link yfiles.input.TapInputMode} (20)</b> – Handles taps and double-tap.</li>
     * <li><b>{@link yfiles.input.MarqueeSelectionInputMode} (30)</b> – Allows dragging a rectangle to select items within it.</li>
     * <li><b>{@link yfiles.input.MoveViewportInputMode} (39)</b> – Allows panning the viewport.</li>
     * <li><b>{@link yfiles.input.ItemHoverInputMode} (55)</b> – Provides events to indicate that the mouse pointer hovers over an item.</li>
     * <li><b>{@link yfiles.input.NavigationInputMode} (55)</b> – Provides general navigation facilities, such as navigating from one item to another with the arrow keys.</li>
     * <li><b>{@link yfiles.input.ContextMenuInputMode} (60)</b> – Handles querying the context menu contents based on a location as well as preparing the menu for display.</li>
     * <li><b>{@link yfiles.input.MouseHoverInputMode} (100)</b> – Handles tool tips.</li>
     * </ul>
     * @class
     * @extends {yfiles.input.GraphInputMode}
     */
    export interface GraphViewerInputMode extends yfiles.input.GraphInputMode{}
    export class GraphViewerInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.GraphViewerInputMode} class.
       */
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple interface that can be used to react to clicks on regions of {@link yfiles.styles.INodeStyle}s etc.
     * <p>
     * {@link yfiles.input.GraphEditorInputMode} will query the {@link yfiles.graph.IModelItem}'s Lookup method for this interface if it has
     * detected {@link yfiles.input.GraphInputMode#click}s. If the item yields an instance of this interface and the click was inside
     * the {@link yfiles.input.IClickListener#getHitTestable}, then {@link yfiles.input.IClickListener#onClicked} is invoked.
     * </p>
     * @see yfiles.input.GraphEditorInputMode
     * @see yfiles.styles.CollapsibleNodeStyleDecorator
     * @interface
     */
    export interface IClickListener extends Object{
      /**
       * Gets an {@link yfiles.input.IHitTestable} that can be used to check if {@link yfiles.input.IClickListener#onClicked} should be invoked.
       * @returns A hit testable that can determine whether {@link #onClicked} should be invoked by a given click.
       * @abstract
       */
      getHitTestable():yfiles.input.IHitTestable;
      /**
       * Called by the framework to indicate that the {@link yfiles.graph.IModelItem} has been clicked by the user at the specified
       * location.
       * <p>
       * This method is called by {@link yfiles.input.GraphInputMode#click} for the
       * {@link yfiles.graph.IModelItem}s that yielded an instance of this interface.
       * </p>
       * @param context The context to use for the click operation.
       * @param location The location of the click
       * @abstract
       */
      onClicked(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):void;
    }
    var IClickListener:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface for classes that can yield {@link yfiles.input.IHandle} implementations for the {@link yfiles.graph.IEdge#sourcePort source} and {@link yfiles.graph.IEdge#targetPort target} ends of an
     * {@link yfiles.graph.IEdge}.
     * <p>
     * This interface will be queried by the default implementation of the {@link yfiles.input.IHandleProvider} that is in the lookup
     * of an {@link yfiles.graph.IEdge}. It is recommended to use this interface instead of the more generic {@link yfiles.input.IHandleProvider}
     * interface, because other generic code can then use this interface to gain access to a specific {@link yfiles.input.IHandle} more
     * easily.
     * </p>
     * @interface
     */
    export interface IEdgePortHandleProvider extends Object{
      /**
       * Gets an {@link yfiles.input.IHandle} implementation for one end of the provided <code>edge</code>.
       * @param context The context in which the handle will be used.
       * @param edge The edge for which an handle is needed.
       * @param sourceHandle if set to <code>true</code> the handle for the source side/port should be returned.
       * @returns The handle to use for the provided side or <code>null</code>.
       * @abstract
       */
      getHandle(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,sourceHandle:boolean):yfiles.input.IHandle;
    }
    var IEdgePortHandleProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Helper interface to customize interactive label editing.
     * <p>
     * Implementations of this interface can be put into the {@link yfiles.graph.ILookup} of {@link yfiles.graph.ILabelOwner}s and
     * {@link yfiles.graph.ILabel}s so that it is possible to tweak the behavior on a case by case basis.
     * </p>
     * <p>
     * Conceptually, in most cases, implementations of this interface behaves like an additional event handler for {@link yfiles.input.GraphEditorInputMode#addLabelAddingListener LabelAdding}
     * and {@link yfiles.input.GraphEditorInputMode#addLabelEditingListener LabelEditing}. If no event handler
     * handled those events, an {@link yfiles.input.IEditLabelHelper} will be queried from the {@link yfiles.graph.ILookup} and the unhandled
     * event arguments passed to the respective methods.
     * </p>
     * @see yfiles.input.EditLabelHelper
     * @see yfiles.input.GraphEditorInputMode#createLabel
     * @see yfiles.input.GraphEditorInputMode#editLabel
     * @see yfiles.input.GraphEditorInputMode#addLabel
     * @see yfiles.input.GraphEditorInputMode#addLabelAddingListener
     * @see yfiles.input.GraphEditorInputMode#addLabelEditingListener
     * @interface
     */
    export interface IEditLabelHelper extends Object{
      /**
       * Helper method that will be called when trying to add a label to an {@link yfiles.graph.ILabelOwner}.
       * <p>
       * Implementations of this method can set the {@link yfiles.graph.ILabel#style}, {@link yfiles.graph.ILabel#layoutParameter}, {@link yfiles.graph.ILabel#preferredSize}, and {@link yfiles.graph.ITagOwner#tag} for newly-added labels by setting the appropriate properties on
       * <code>args</code>.
       * </p>
       * <p>
       * Setting the {@link yfiles.input.LabelEditingEventArgs#cancel} property on <code>args</code> to <code>true</code> will disallow label creation.
       * </p>
       * @param evt The original event arguments.
       * @see yfiles.input.IEditLabelHelper#onLabelEditing
       * @abstract
       */
      onLabelAdding(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Helper method that will be called when trying to edit a label.
       * <p>
       * This method can be called for editing labels for an {@link yfiles.graph.ILabelOwner}, in which case the {@link yfiles.input.LabelEditingEventArgs#label} property on <code>args</code> will
       * be <code>null</code>, but the {@link yfiles.input.LabelEditingEventArgs#owner} property will be non-<code>null</code>. It can also be called for editing a specific label, in which case the {@link yfiles.input.LabelEditingEventArgs#label}
       * property on <code>args</code> will be non-<code>null</code>, and the {@link yfiles.input.LabelEditingEventArgs#owner} property will be set to the label's owner.
       * </p>
       * <p>
       * Implementations of this method can set a specific label to be edited, by setting the {@link yfiles.input.LabelEditingEventArgs#label} property of the <code>args</code> to an
       * existing label instance. This label does not need to belong to the same owner that is queried for. Setting the {@link yfiles.input.LabelEditingEventArgs#label} property
       * to <code>null</code> will not edit an existing label, but instead add a new one on the {@link yfiles.graph.ILabelOwner} set in the {@link yfiles.input.LabelEditingEventArgs#owner}
       * property.
       * </p>
       * <p>
       * When a new label is added, the properties {@link yfiles.input.LabelEditingEventArgs#style}, {@link yfiles.input.LabelEditingEventArgs#layoutParameter}, {@link yfiles.input.LabelEditingEventArgs#preferredSize}, and {@link yfiles.input.LabelEditingEventArgs#tag} are used for the newly-created label, just as in
       * {@link yfiles.input.IEditLabelHelper#onLabelAdding}.
       * </p>
       * <p>
       * Setting the {@link yfiles.input.LabelEditingEventArgs#cancel} property on <code>args</code> to <code>true</code> will disallow label editing and creation.
       * </p>
       * @param evt The original event arguments.
       * @see yfiles.input.IEditLabelHelper#onLabelAdding
       * @abstract
       */
      onLabelEditing(evt:yfiles.input.LabelEditingEventArgs):void;
    }
    var IEditLabelHelper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides the snap lines and the snap results to the {@link yfiles.input.LabelSnapContext} during dragging of labels.
     * @see yfiles.input.LabelSnapContext
     * @see yfiles.graph.LabelDecorator#labelSnapContextHelperDecorator
     * @interface
     */
    export interface ILabelSnapContextHelper extends Object{
      /**
       * Called during {@link yfiles.input.IDragHandler#initializeDrag initialization} of a <code>label</code> dragging to add {@link yfiles.input.SnapLine snap lines} to which the label can potentially snap to the
       * <code>snapContext</code>.
       * @param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
       * the context explicitly.
       * @param inputModeContext The context of the input mode that handles the dragging.
       * @param label The label that is dragged.
       * @abstract
       */
      addSnapLines(snapContext:yfiles.input.LabelSnapContext,inputModeContext:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):void;
      /**
       * Called while the given <code>label</code> is {@link yfiles.input.IDragHandler#handleMove dragged} to add {@link yfiles.input.SnapResult snap results} for the {@link yfiles.input.SnapLine snap lines} provided by the <code>context</code>.
       * @param context The snap context which manages the snap lines and the settings.
       * @param evt The event argument to obtain the necessary information from and {@link #addSnapResult add results to}.
       * @param suggestedLayout The {@link #layout layout} of the label that would be used without snapping.
       * @param label The label that is dragged.
       * @abstract
       */
      collectSnapResults(context:yfiles.input.LabelSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLayout:yfiles.geometry.IOrientedRectangle,label:yfiles.graph.ILabel):void;
    }
    var ILabelSnapContextHelper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface used by implementations that provide various size constraints for {@link yfiles.graph.INode}s.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SizeConstraintProvider, demo.yfiles.input.sizeconstraintprovider</li>
     * </ul>
     * @see yfiles.input.NodeSizeConstraintProvider
     * @interface
     */
    export interface INodeSizeConstraintProvider extends Object{
      /**
       * Returns the maximum size allowed for the given <code>node</code>.
       * @param node The node to return the maximum size for.
       * @returns The maximum size or {@link #INFINITE} if there is no constraint on the size.
       * @abstract
       */
      getMaximumSize(node:yfiles.graph.INode):yfiles.geometry.Size;
      /**
       * Returns the minimum area that needs to be enclosed by the given <code>node</code>.
       * @param node The node to return the area for.
       * @returns The area to enclose or {@link #EMPTY} if there is no constraint on the size due to an enclosed area.
       * @abstract
       */
      getMinimumEnclosedArea(node:yfiles.graph.INode):yfiles.geometry.Rect;
      /**
       * Returns the minimum size allowed for the given <code>node</code>.
       * @param node The node to return the minimum size for.
       * @returns The minimum size or {@link #EMPTY} if there is no constraint on the size.
       * @abstract
       */
      getMinimumSize(node:yfiles.graph.INode):yfiles.geometry.Size;
    }
    var INodeSizeConstraintProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A simple default implementation of {@link yfiles.input.INodeSizeConstraintProvider}
     * @class
     * @implements {yfiles.input.INodeSizeConstraintProvider}
     */
    export interface NodeSizeConstraintProvider extends Object,yfiles.input.INodeSizeConstraintProvider{}
    export class NodeSizeConstraintProvider {
      /**
       * Creates a new instance using the provided size instances as the initial values.
       * <p>
       * Not specifing the minimumEnclosedArea will set it to {@link yfiles.geometry.Rect#EMPTY}.
       * </p>
       * @param minimumSize The {@link #minimumSize}.
       * @param maximumSize The {@link #maximumSize}.
       * @param [minimumEnclosedArea=null] The {@link #minimumEnclosedArea}.
       */
      constructor(minimumSize:yfiles.geometry.ISize,maximumSize:yfiles.geometry.ISize,minimumEnclosedArea?:yfiles.geometry.IRectangle);
      /**
       * 
       * @param node
       * @returns 
       */
      getMaximumSize(node:yfiles.graph.INode):yfiles.geometry.Size;
      /**
       * 
       * @param node
       * @returns 
       */
      getMinimumEnclosedArea(node:yfiles.graph.INode):yfiles.geometry.Rect;
      /**
       * 
       * @param node
       * @returns 
       */
      getMinimumSize(node:yfiles.graph.INode):yfiles.geometry.Size;
      /**
       * Gets or sets the instance to return by {@link yfiles.input.NodeSizeConstraintProvider#getMaximumSize}.
       * @type {yfiles.geometry.ISize}
       */
      maximumSize:yfiles.geometry.ISize;
      /**
       * Gets or sets the instance to return by {@link yfiles.input.NodeSizeConstraintProvider#getMinimumEnclosedArea}.
       * @type {yfiles.geometry.IRectangle}
       */
      minimumEnclosedArea:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the instance to return by {@link yfiles.input.NodeSizeConstraintProvider#getMinimumSize}.
       * @type {yfiles.geometry.ISize}
       */
      minimumSize:yfiles.geometry.ISize;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface used by {@link yfiles.input.OrthogonalEdgeEditingContext} and the like that can be decorated to the {@link yfiles.graph.ILookup}
     * of {@link yfiles.graph.IEdge}s.
     * <p>
     * This interface is for editing edges in such a way that their path stays orthogonal, i.e. all of the segments are
     * oriented either horizontally or vertically.
     * </p>
     * @see yfiles.input.OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper
     * @interface
     */
    export interface IOrthogonalEdgeHelper extends Object{
      /**
       * Callback method that is invoked after the provided edge has been edited orthogonally.
       * <p>
       * This callback can be used to remove unused bends and finalize the orthogonal edge editing gesture.
       * </p>
       * @param context The input mode context which edited the edge.
       * @param graph The graph to use for modifying the edge instance.
       * @param edge The edge to clean up the path.
       * @abstract
       */
      cleanUpEdge(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge):void;
      /**
       * Gets the declared orientation of the given segment at the provided edge.
       * <p>
       * The orientation cannot always be inferred from the current geometry: If a segment has a zero length it is unclear what
       * orientation it should have, also a segment could be accidentally orthogonally oriented, while in fact it is considered
       * {@link yfiles.input.SegmentOrientation#NON_ORTHOGONAL}.
       * </p>
       * @param context The input mode context in which the orientation is needed.
       * @param edge The edge to inspect.
       * @param segmentIndex The index of the segment.
       * @returns The declared orientation of the segment.
       * @abstract
       */
      getSegmentOrientation(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,segmentIndex:number):yfiles.input.SegmentOrientation;
      /**
       * Determines whether the provided edge should be edited orthogonally in the specified input mode context.
       * <p>
       * If this method returns <code>false</code>, the other methods will not be queried at all.
       * </p>
       * @param context The input mode context in which the edge is about to be edited.
       * @param edge The edge to inspect.
       * @returns <code>true</code> if the edge should be orthogonally edited in the specified input mode context; <code>false</code> otherwise.
       * @abstract
       */
      shouldEditOrthogonally(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge):boolean;
      /**
       * Determines whether this end of the provided edge can be moved in the input mode context.
       * <p>
       * This information is required to determine whether an edge's first or last segment needs to be split or the adjacent edge
       * end can be moved along with the other end of the segment.
       * </p>
       * @param context The input mode context in which the segment is edited.
       * @param edge The edge to inspect.
       * @param sourceEnd if set to <code>true</code> the source end of the edge is queried, otherwise the target end.
       * @returns <code>true</code> if the end of the edge can be moved for the specified input mode context; <code>false</code> otherwise, in which case the
       * corresponding segment needs to be split to keep the segment orientation orthogonal.
       * @abstract
       */
      shouldMoveEndImplicitly(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,sourceEnd:boolean):boolean;
    }
    var IOrthogonalEdgeHelper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used for implementations that recognize, approve and disapprove node reparenting gestures, as well as actually
     * performs the reparenting.
     * <p>
     * Implementations of this interface are queried from the {@link yfiles.input.IInputModeContext} by code that wants to reparent a
     * node. Specifically this is used by default to reparent nodes during the {@link yfiles.input.MoveInputMode dragging of nodes}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: ReparentHandler, demo.yfiles.input.reparenthandler</li>
     * </ul>
     * @interface
     */
    export interface IReparentNodeHandler extends Object{
      /**
       * Determines whether the current gesture that can be determined through the context is a reparent gesture.
       * @param context The context that provides information about the user input.
       * @param node The node that will possibly be reparented.
       * @returns Whether this is a reparenting gesture.
       * @abstract
       */
      isReparentGesture(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode):boolean;
      /**
       * Determines whether the provided node may be reparented to a <code>newParent</code>.
       * @param context The context that provides information about the user input.
       * @param node The node that will be reparented.
       * @param newParent The potential new parent or <code>null</code>. Can be a group or a non-group node.
       * @returns Whether <code>newParent</code> is a valid new parent for <code>node</code>.
       * @abstract
       */
      isValidParent(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode,newParent:yfiles.graph.INode):boolean;
      /**
       * Performs the actual reparenting after the reparent gesture has been finalized.
       * <p>
       * Implementations should {@link yfiles.graph.IGraph#setParent set the parent} of <code>node</code> to <code>newParent</code>.
       * </p>
       * @param context The context that provides information about the user input.
       * @param node The node that will be reparented.
       * @param newParent The potential new parent or <code>null</code>. Can be a group or a non-group node.
       * @abstract
       */
      reparent(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode,newParent:yfiles.graph.INode):void;
      /**
       * Determines whether the user may detach the given node from its current parent in order to reparent it.
       * @param context The context that provides information about the user input.
       * @param node The node that is about to be detached from its current parent.
       * @returns Whether the node may be detached and reparented.
       * @abstract
       */
      shouldReparent(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode):boolean;
    }
    var IReparentNodeHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Queried by the {@link yfiles.input.GraphSnapContext} to add {@link yfiles.input.OrthogonalSnapLine}s to its collections during the
     * {@link yfiles.input.SnapContext#isInitializing initialization phase}.
     * <p>
     * Implementations of this interface are queried by {@link yfiles.input.GraphSnapContext} for each item that is not modified during
     * the edit using the {@link yfiles.graph.ILookup#lookup} of the respective item.
     * </p>
     * @see yfiles.graph.NodeDecorator#snapLineProviderDecorator
     * @interface
     */
    export interface ISnapLineProvider extends Object{
      /**
       * Called by the {@link yfiles.input.GraphSnapContext} when a {@link yfiles.input.GraphSnapContext#initializeDrag drag} is about to start.
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param item The item to add snap lines for.
       * @abstract
       */
      addSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,item:yfiles.graph.IModelItem):void;
    }
    var ISnapLineProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides data for the {@link yfiles.input.GraphInputMode#addItemClickedListener ItemClicked},
     * {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener ItemDoubleClicked},
     * {@link yfiles.input.GraphInputMode#addItemLeftClickedListener ItemLeftClicked},
     * {@link yfiles.input.GraphInputMode#addItemLeftDoubleClickedListener ItemLeftDoubleClicked}, {@link yfiles.input.GraphInputMode#addItemRightClickedListener ItemRightClicked}
     * and {@link yfiles.input.GraphInputMode#addItemRightDoubleClickedListener ItemRightDoubleClicked} events.
     * Type parameter <code>T</code>: The type of the item this event carries.
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<T>}
     * @template T
     */
    export interface ItemClickedEventArgs<T> extends yfiles.collections.ItemEventArgs<T>{}
    export class ItemClickedEventArgs<T> {
      /**
       * Initializes a new instance of the {@link yfiles.input.ItemClickedEventArgs.<T>} class.
       * @param item The item which is the subject of the event.
       * @param location The location of the click.
       */
      constructor(item:T,location:yfiles.geometry.Point);
      /**
       * Gets or sets a value indicating whether this {@link yfiles.input.ItemClickedEventArgs.<T>} is handled.
       * <p>
       * Setting the event to handled indicates whether the event should be further propagated or whether there has not been any
       * code that actively handled the event to the event source. How this flag is actually being treated depends on the source
       * of the event.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * Gets the location of the click.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      static $class:yfiles.lang.Class;
    }
    /**
     * An input mode for use in a {@link yfiles.view.GraphComponent} that fires events when the mouse enters or leaves the
     * visualization of a {@link yfiles.graph.IModelItem graph item}.
     * <p>
     * This mode can be used to determine when the mouse is being moved from one item to the next.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.ItemHoverInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface ItemHoverInputMode extends Object,yfiles.input.IInputMode{}
    export class ItemHoverInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.ItemHoverInputMode} class.
       */
      constructor();
      /**
       * 
       */
      cancel():void;
      /**
       * Gets the items that have been hit at the given location.
       * @param location The location in world coordinates to query.
       * @returns An enumerable over all items that have been hit at the given location.
       * @protected
       */
      getHitItemsAt(location:yfiles.geometry.Point):yfiles.collections.IEnumerable<yfiles.graph.IModelItem>;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Determines whether the given item is a valid item to be considered for hovering.
       * <p>
       * This implementation checks whether the item is covered by the {@link yfiles.input.ItemHoverInputMode#hoverItems} set, only.
       * </p>
       * @param item The item to check.
       * @returns <code>true</code> if it is valid to report a hover over the specified item; <code>false</code> otherwise.
       * @see yfiles.input.ItemHoverInputMode#discardInvalidItems
       * @protected
       */
      isValidHoverItem(item:yfiles.graph.IModelItem):boolean;
      /**
       * Called after {@link yfiles.input.ItemHoverInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Raises the {@link yfiles.input.ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onHoveredItemChanged(evt:yfiles.input.HoveredItemChangedEventArgs):void;
      /**
       * Called after {@link yfiles.input.ItemHoverInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Overridden to only return <code>true</code> if this instance does not currently {@link yfiles.input.ConcurrencyController#hasMutex have the input mutex}.
       * @returns <code>true</code> iff this instance does not {@link #hasMutex own the mutex}.
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Forces a reevaluation of the item that the mouse is currently hovering over or at a specific location.
       * <p>
       * This method may be called by code that is aware of the fact that the visualization has changed, but the mouse may not
       * have been moved. By default this implementation will only re-query the items at the mouse location when the mouse has
       * moved. This method can be called to force a reevaluation in other cases.
       * </p>
       * @param [location=null] The query location.
       */
      updateHover(location?:yfiles.geometry.Point):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.ItemHoverInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.ItemHoverInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.ItemHoverInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>HoveredItemChanged</code> event that occurs when the item that is being hovered over with the
       * mouse changes.
       * <p>
       * This event is also fired when the mouse pointer leaves an item.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.ItemHoverInputMode#removeHoveredItemChangedListener
       */
      addHoveredItemChangedListener(listener:(sender:Object,evt:yfiles.input.HoveredItemChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>HoveredItemChanged</code> event that occurs when the item that is being hovered over with
       * the mouse changes.
       * <p>
       * This event is also fired when the mouse pointer leaves an item.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.ItemHoverInputMode#addHoveredItemChangedListener
       */
      removeHoveredItemChangedListener(listener:(sender:Object,evt:yfiles.input.HoveredItemChangedEventArgs)=>void):void;
      /**
       * Gets the current item the mouse is hovering over.
       * @type {yfiles.graph.IModelItem}
       */
      currentHoverItem:yfiles.graph.IModelItem;
      /**
       * Gets or sets which graph items are considered by this input mode.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @type {yfiles.graph.GraphItemTypes}
       */
      hoverItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets the cursor to use when the mouse is hovering over a {@link yfiles.input.ItemHoverInputMode#isValidHoverItem} valid
       * hover item.
       * @type {yfiles.view.Cursor}
       */
      hoverCursor:yfiles.view.Cursor;
      /**
       * Determines whether items that have been hit at the current location that are not {@link yfiles.input.ItemHoverInputMode#isValidHoverItem valid items} should be ignored or
       * reported as <code>null</code>.
       * @see yfiles.input.ItemHoverInputMode#isValidHoverItem
       * @type {boolean}
       */
      discardInvalidItems:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event argument class containing information about which {@link yfiles.graph.IModelItem} the mouse is currently hovering over.
     * <p>
     * This event is used by the {@link yfiles.input.ItemHoverInputMode#addHoveredItemChangedListener HoveredItemChanged}
     * event. The {@link yfiles.collections.ItemEventArgs.<T>#item} property yields the item that is currently being hovered over or <code>null</code> if the mouse just left an item without
     * hovering over an other one. The {@link yfiles.input.HoveredItemChangedEventArgs#oldItem} property provides the previously hovered item.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.IModelItem>}
     */
    export interface HoveredItemChangedEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>{}
    export class HoveredItemChangedEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.HoveredItemChangedEventArgs} class.
       * @param item The model item, possibly <code>null</code>.
       * @param oldItem The old item, possibly <code>null</code>.
       */
      constructor(item:yfiles.graph.IModelItem,oldItem:yfiles.graph.IModelItem);
      /**
       * The item that was previously hovered, possibly <code>null</code>.
       * @see yfiles.collections.ItemEventArgs.<T>#item
       * @type {yfiles.graph.IModelItem}
       */
      oldItem:yfiles.graph.IModelItem;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for the {@link yfiles.input.GraphEditorInputMode#addLabelAddingListener LabelAdding} and {@link yfiles.input.GraphEditorInputMode#addLabelEditingListener LabelEditing}
     * events, as well as the {@link yfiles.input.IEditLabelHelper} interface.
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     * @final
     */
    export interface LabelEditingEventArgs extends yfiles.input.InputModeEventArgs{}
    export class LabelEditingEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.LabelEditingEventArgs} class with the given context, owner, and label.
       * @param context The input mode context to use.
       * @param owner The {@link } whose label should be edited.
       * @param label The {@link } that should be edited.
       */
      constructor(context:yfiles.input.IInputModeContext,owner:yfiles.graph.ILabelOwner,label:yfiles.graph.ILabel);
      /**
       * Gets or sets the label to edit.
       * <p>
       * This can be set to an exiting label to edit a specific one, or set to <code>null</code> to add a new label instead (e.g. for
       * {@link yfiles.graph.ILabelOwner}s that have no label yet).
       * </p>
       * <p>
       * If this property is set to <code>null</code>, the properties {@link yfiles.input.LabelEditingEventArgs#style}, {@link yfiles.input.LabelEditingEventArgs#preferredSize}, {@link yfiles.input.LabelEditingEventArgs#layoutParameter}, and {@link yfiles.input.LabelEditingEventArgs#tag} are used for the newly-added label, just like with
       * {@link yfiles.input.LabelEditingEventArgs}.
       * </p>
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {yfiles.graph.ILabel}
       */
      label:yfiles.graph.ILabel;
      /**
       * Gets or sets a value indicating whether the action (adding or editing a label) should be canceled.
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {boolean}
       */
      cancel:boolean;
      /**
       * Gets or sets a value indicating whether this event has been handled.
       * <p>
       * A handled instance of {@link yfiles.input.LabelEditingEventArgs} will not be passed to further event handlers or an {@link yfiles.input.IEditLabelHelper}
       * instance.
       * </p>
       * <p>
       * Setting any of the other properties implicitly also sets this to <code>true</code>.
       * </p>
       * @type {boolean}
       */
      handled:boolean;
      /**
       * Gets or sets the owner of the label.
       * <p>
       * This property can be <code>null</code> if there is no owner known, e.g. if the event is raised for
       * non-{@link yfiles.graph.ILabelOwner}s. Otherwise it is initially set to the item for which the event is raised.
       * </p>
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {yfiles.graph.ILabelOwner}
       */
      owner:yfiles.graph.ILabelOwner;
      /**
       * Gets or sets the {@link yfiles.graph.ILabel#style} of the label to add.
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {yfiles.styles.ILabelStyle}
       */
      style:yfiles.styles.ILabelStyle;
      /**
       * Gets or sets the {@link yfiles.graph.ILabel#preferredSize} of the label to add.
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {yfiles.geometry.Size}
       */
      preferredSize:yfiles.geometry.Size;
      /**
       * Gets or sets the {@link yfiles.graph.ILabel#layoutParameter} of the label to add.
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {yfiles.graph.ILabelModelParameter}
       */
      layoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the {@link yfiles.graph.ITagOwner#tag} of the label to add.
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {Object}
       */
      tag:Object;
      /**
       * Configures the {@link yfiles.input.TextEditorInputMode} for editing the provided label instance.
       * <p>
       * Setting this property implicitly also sets {@link yfiles.input.LabelEditingEventArgs#handled} to <code>true</code>.
       * </p>
       * @type {function(yfiles.input.IInputModeContext, yfiles.input.TextEditorInputMode, yfiles.graph.ILabel): void}
       */
      textEditorInputModeConfigurator:(context:yfiles.input.IInputModeContext,mode:yfiles.input.TextEditorInputMode,label:yfiles.graph.ILabel)=>void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Manages interactive snapping of {@link yfiles.graph.ILabel}s to their owner during drag operations like movements.
     * <p>
     * This class provides a set of properties to customize the snapping behavior. {@link yfiles.input.IPositionHandler}
     * implementations and similar classes can use the following idiom to get an instance of this class:
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 01 Getting Started, step 15 Snapping</li>
     * </ul>
     * @see yfiles.input.MoveLabelInputMode
     * @see yfiles.input.GraphEditorInputMode#snapContext
     * @class
     * @extends {yfiles.input.SnapContext}
     */
    export interface LabelSnapContext extends yfiles.input.SnapContext{}
    export class LabelSnapContext {
      /**
       * Initializes a new instance of the {@link yfiles.input.LabelSnapContext} class.
       */
      constructor();
      /**
       * Adds the given <code>snapLine</code> to the {@link yfiles.input.LabelSnapContext#snapLines snap line} collection of this context.
       * @param snapLine The snap line to add.
       * @see yfiles.input.GraphSnapContext#additionalSnapLines
       */
      addSnapLine(snapLine:yfiles.input.SnapLine):void;
      /**
       * Raises the {@link yfiles.input.LabelSnapContext#addCollectSnapLinesListener CollectSnapLines} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onCollectSnapLines(evt:yfiles.input.CollectLabelSnapLineEventArgs):void;
      /**
       * Determines whether this instance should collect snap lines for the initial position of a label.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectInitialLocationSnapLines:boolean;
      /**
       * Determines whether this instance should collect six snap lines, two through the center and four through the border sides
       * of the label owner.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectNodeShapeSnapLines:boolean;
      /**
       * Determines whether this instance should collect four snap lines in parallel to the owner's borders at the initial label
       * distance.
       * <p>
       * Note that the label snaps to these snap lines with the side that is closer to the node's border. That way, snapping to
       * such a snap line preserves the perceived distance from the node border for both labels inside and outside the node
       * bounds.
       * </p>
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectOwnNodeDistanceSnapLines:boolean;
      /**
       * Determines whether this instance should collect snap lines in parallel to the border of the label owner at the distance
       * of other node labels of that owner.
       * <p>
       * Note that the label snaps to these snap lines with the side that is closer to the node's border. That way, snapping to
       * such a snap line preserves the perceived distance from the node border for both labels inside and outside the node
       * bounds.
       * </p>
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectSameOwnerNodeDistanceSnapLines:boolean;
      /**
       * Determines whether this instance should collect snap lines in parallel to the border of the label owner at the distance
       * of other node labels in the graph.
       * <p>
       * Note that the label snaps to these snap lines with the side that is closer to the node's border. That way, snapping to
       * such a snap line preserves the perceived distance from the node border for both labels inside and outside the node
       * bounds.
       * </p>
       * <p>
       * The default is <code>false</code>.
       * </p>
       * @default false
       * @type {boolean}
       */
      collectAllNodeDistanceSnapLines:boolean;
      /**
       * Determines whether this instance should collect snap lines on the edge path.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectEdgePathSnapLines:boolean;
      /**
       * Determines whether this instance should collect two snap lines in parallel to the edge path of the label owner at the
       * initial distance of the edge label.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectOwnEdgeDistanceSnapLines:boolean;
      /**
       * Determines whether this instance should collect snap lines in parallel to the edge path of the label owner at the
       * distances of all edge labels of that owner.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      collectSameOwnerEdgeDistanceSnapLines:boolean;
      /**
       * Determines whether this instance should collect snap lines in parallel to the edge path of the label owner at the
       * distance of other edge labels in the graph.
       * <p>
       * The default is <code>false</code>.
       * </p>
       * @default false
       * @type {boolean}
       */
      collectAllEdgeDistanceSnapLines:boolean;
      /**
       * Specifies whether this context will automatically snap the node labels to snap lines.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      snapNodeLabels:boolean;
      /**
       * Specifies whether this context will automatically snap the edge labels to snap lines.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      snapEdgeLabels:boolean;
      /**
       * Gets or sets the amount by which snap lines that are induced by existing edge segments are being extended.
       * <p>
       * The default is <code>40.0d</code>, this value will be used to prolongate the ends of the snap lines.
       * </p>
       * @default 40.0
       * @type {number}
       */
      snapLineExtension:number;
      /**
       * The collection of the {@link yfiles.input.SnapLine snap lines} that have been {@link yfiles.input.LabelSnapContext#addSnapLine added} to this context.
       * <p>
       * This collection is only available if {@link yfiles.input.SnapContext#isInitialized} is <code>true</code>.
       * </p>
       * @see yfiles.input.LabelSnapContext#addSnapLine
       * @type {yfiles.collections.IEnumerable.<yfiles.input.SnapLine>}
       */
      snapLines:yfiles.collections.IEnumerable<yfiles.input.SnapLine>;
      /**
       * Adds the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link yfiles.input.SnapContext#dragInitialized initialized} to
       * collect {@link yfiles.input.SnapLine}s.
       * <p>
       * Event handlers should add snap lines to the caller using the methods provided by
       * {@link yfiles.input.CollectLabelSnapLineEventArgs}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.LabelSnapContext#removeCollectSnapLinesListener
       */
      addCollectSnapLinesListener(listener:(sender:Object,evt:yfiles.input.CollectLabelSnapLineEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link yfiles.input.SnapContext#dragInitialized initialized} to
       * collect {@link yfiles.input.SnapLine}s.
       * <p>
       * Event handlers should add snap lines to the caller using the methods provided by
       * {@link yfiles.input.CollectLabelSnapLineEventArgs}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.LabelSnapContext#addCollectSnapLinesListener
       */
      removeCollectSnapLinesListener(listener:(sender:Object,evt:yfiles.input.CollectLabelSnapLineEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * The event arguments used by {@link yfiles.input.LabelSnapContext} to collect custom snap lines for each drag.
     * @see yfiles.input.LabelSnapContext#addCollectSnapLinesListener
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     */
    export interface CollectLabelSnapLineEventArgs extends yfiles.input.InputModeEventArgs{}
    export class CollectLabelSnapLineEventArgs {
      /**
       * Creates a new instance of this class.
       * @param context The context this event is being used in.
       * @param snapLines The snap lines collection to add to.
       */
      constructor(context:yfiles.input.IInputModeContext,snapLines:yfiles.collections.ICollection<yfiles.input.SnapLine>);
      /**
       * Adds the given snap line to the snap line collection of this class.
       * @param snapLine the snap line to add.
       */
      addSnapLine(snapLine:yfiles.input.OrthogonalSnapLine):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} that can {@link yfiles.input.NavigationInputMode#moveTo navigate} an {@link yfiles.graph.IGraph} displayed in a {@link yfiles.input.NavigationInputMode#graphComponent}.
     * <p>
     * This mode is {@link yfiles.input.NavigationInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface NavigationInputMode extends Object,yfiles.input.IInputMode{}
    export class NavigationInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.NavigationInputMode} class.
       */
      constructor();
      /**
       * Callback that adjusts the {@link yfiles.view.CanvasComponent#contentRect} to encompass all elements.
       * <p>
       * This implementation will try to delegate to {@link yfiles.input.GraphEditorInputMode#adjustContentRect} if it can find the {@link yfiles.input.GraphEditorInputMode}
       * in the {@link yfiles.input.NavigationInputMode#inputModeContext}, otherwise the {@link yfiles.input.ICommand#FIT_GRAPH_BOUNDS} is executed.
       * </p>
       * @protected
       */
      adjustContentRect():void;
      /**
       * Adjust the group node location according to the value of {@link yfiles.input.NavigationInputMode#autoGroupNodeAlignmentPolicy}.
       * @param collapse <code>true</code> iff the node has just been collapsed.
       * @param groupNode The node that has changed its state.
       * @protected
       */
      adjustGroupNodeLocation(collapse:boolean,groupNode:yfiles.graph.INode):void;
      /**
       * 
       */
      cancel():void;
      /**
       * Clears the selection in the {@link yfiles.input.NavigationInputMode#graphComponent}.
       */
      clearSelection():void;
      /**
       * {@link yfiles.graph.IFoldingView#collapse Collapses} the given group node to hide the contents of the group node from this {@link yfiles.graph.IFoldingView}.
       * @param groupNode The group node to collapse.
       * @see yfiles.graph.FoldingManager
       * @see yfiles.input.NavigationInputMode#allowCollapseGroup
       */
      collapseGroup(groupNode:yfiles.graph.INode):void;
      /**
       * Performs CollapseGroup for all {@link yfiles.view.IGraphSelection#selectedNodes}.
       */
      collapseSelection():void;
      /**
       * This method allows for entering a group node so that the currently displayed {@link yfiles.graph.IFoldingView}'s {@link yfiles.graph.IFoldingView#localRoot} will be reset
       * to the provided node.
       * <p>
       * It is possible to both use a local group node that is part of the currently displayed {@link yfiles.input.NavigationInputMode#graph} as the <code>node</code> argument, as well as
       * an item that belongs to the {@link yfiles.graph.FoldingManager#masterGraph} to allow for switching to group nodes which are currently not being displayed in this
       * view.
       * </p>
       * @param node The node that needs to be either part of the current graph, or part of the {@link #masterGraph}.
       * @see yfiles.input.NavigationInputMode#shouldEnterGroup
       * @see yfiles.input.NavigationInputMode#allowEnterGroup
       * @see yfiles.graph.FoldingManager
       */
      enterGroup(node:yfiles.graph.INode):void;
      /**
       * Performs {@link yfiles.input.NavigationInputMode#enterGroup} for the first valid {@link yfiles.view.IGraphSelection#selectedNodes}.
       */
      enterSelectedGroup():void;
      /**
       * Exits the current {@link yfiles.graph.IFoldingView#localRoot} of the currently displayed {@link yfiles.graph.IFoldingView view} and shows the contents of the parent container.
       * <p>
       * This method will also {@link yfiles.input.GraphInputMode#clearSelection clear the selection} and {@link yfiles.input.GraphInputMode#setSelected select} the exited group node.
       * </p>
       * @see yfiles.graph.FoldingManager
       * @see yfiles.input.NavigationInputMode#allowExitGroup
       * @see yfiles.input.NavigationInputMode#shouldExitGroup
       */
      exitGroup():void;
      /**
       * {@link yfiles.graph.IFoldingView#expand Expands} the given group node to show the contents of the collapsed group node in this {@link yfiles.graph.IFoldingView}.
       * @param groupNode The group node to expand
       * @see yfiles.graph.FoldingManager
       * @see yfiles.input.NavigationInputMode#allowExpandGroup
       */
      expandGroup(groupNode:yfiles.graph.INode):void;
      /**
       * Performs {@link yfiles.input.NavigationInputMode#expandGroup} for all {@link yfiles.view.IGraphSelection#selectedNodes}.
       */
      expandSelection():void;
      /**
       * Moves the focus into the given direction, extending the selection to the new element.
       * @param direction The direction to move the focus and extend the selection.
       */
      extendSelectionTo(direction:yfiles.input.MoveFocusDirection):void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Moves the focus into the given direction, not changing the current selection.
       * @param direction The direction to move the focus.
       */
      moveFocusTo(direction:yfiles.input.MoveFocusDirection):void;
      /**
       * Moves the focus into the given direction, setting the selection to the new element.
       * @param direction The direction to move the focus and selection.
       */
      moveTo(direction:yfiles.input.MoveFocusDirection):void;
      /**
       * Called after {@link yfiles.input.NavigationInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupCollapsedListener GroupCollapsed} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupCollapsed(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupCollapsingListener GroupCollapsing} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupCollapsing(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupEnteredListener GroupEntered} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupEntered(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupEnteringListener GroupEntering} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupEntering(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupExitedListener GroupExited} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupExited(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupExitingListener GroupExiting} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupExiting(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupExpandedListener GroupExpanded} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupExpanded(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.NavigationInputMode#addGroupExpandingListener GroupExpanding} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onGroupExpanding(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Callback that actually sets the current item property.
       * @param graphComponent The {@link #graphComponent} to set the current item of.
       * @param item The item to set.
       * @returns Whether the operation was actually performed.
       * @protected
       */
      onSetCurrentItem(graphComponent:yfiles.view.GraphComponent,item:yfiles.graph.IModelItem):boolean;
      /**
       * Called after {@link yfiles.input.NavigationInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Selects the {@link yfiles.view.GraphComponent#currentItem} current item.
       * <p>
       * This method delegates to {@link yfiles.input.NavigationInputMode#selectItem}.
       * </p>
       */
      selectCurrentItem():void;
      /**
       * Callback that selects the given item.
       * @param graphComponent The {@link #graphComponent}.
       * @param item The item to select.
       * @param extendSelection if set to <code>true</code> the current selection is extended otherwise it is cleared beforehand.
       * @see yfiles.input.NavigationInputMode#clearSelection
       * @protected
       */
      selectItem(graphComponent:yfiles.view.GraphComponent,item:yfiles.graph.IModelItem,extendSelection:boolean):void;
      /**
       * Sets the "current" item to the given one.
       * @param item The item to set as the current item.
       * @see yfiles.input.NavigationInputMode#onSetCurrentItem
       */
      setCurrentItem(item:yfiles.graph.IModelItem):void;
      /**
       * Sets the {@link yfiles.input.NavigationInputMode#graphComponent} to use for the various actions.
       * <p>
       * This will register or unregister the commands for the control. This method is called in response to {@link yfiles.input.NavigationInputMode#install}
       * and {@link yfiles.input.NavigationInputMode#uninstall}.
       * </p>
       * @param graphComponent The control to use or <code>null</code>.
       * @protected
       */
      setGraphComponent(graphComponent:yfiles.view.GraphComponent):void;
      /**
       * Predicate method that decides whether it is allowed to collapse the given group node.
       * <p>
       * This implementation returns the value of {@link yfiles.input.NavigationInputMode#allowCollapseGroup}.
       * </p>
       * @param groupNode
       * @returns Whether to collapse the given group or not.
       * @protected
       */
      shouldCollapseGroup(groupNode:yfiles.graph.INode):boolean;
      /**
       * Predicate method that decides whether it is allowed to {@link yfiles.input.NavigationInputMode#enterGroup enter} the specific group node.
       * <p>
       * This implementation yields the value of {@link yfiles.input.NavigationInputMode#allowEnterGroup}.
       * </p>
       * @param node The group node to enter.
       * @returns Whether to enter the group or not.
       * @protected
       */
      shouldEnterGroup(node:yfiles.graph.INode):boolean;
      /**
       * Predicate method that decides whether it is allowed to {@link yfiles.input.NavigationInputMode#exitGroup exit} the current group node.
       * <p>
       * This implementation yields the value of {@link yfiles.input.NavigationInputMode#allowExitGroup}.
       * </p>
       * @returns Whether to exit the current group or not.
       * @protected
       */
      shouldExitGroup():boolean;
      /**
       * Predicate method that decides whether it is allowed to {@link yfiles.input.NavigationInputMode#expandGroup expand} the given group node.
       * <p>
       * This implementation yields the value of {@link yfiles.input.NavigationInputMode#allowExpandGroup}.
       * </p>
       * @param node
       * @returns Whether to expand the given group or not.
       * @protected
       */
      shouldExpandGroup(node:yfiles.graph.INode):boolean;
      /**
       * Callback predicate method that determines whether a given model item should be navigated to.
       * <p>
       * This implementation uses the {@link yfiles.input.NavigationInputMode#navigableItems} property to determine whether the <code>item</code> can be navigated to.
       * </p>
       * @param item The model item.
       * @returns Whether the item should be considered for navigational commands.
       * @protected
       */
      shouldNavigateTo(item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether the input mode should try to request the mutex when a command is executed.
       * <p>
       * This resets the state of any other input mode, unless it currently possesses the mutex.
       * </p>
       * <p>
       * The value of this method is queried for {@link yfiles.input.ICommand#COLLAPSE_GROUP}, {@link yfiles.input.ICommand#EXPAND_GROUP},
       * {@link yfiles.input.ICommand#TOGGLE_EXPANSION_STATE}, {@link yfiles.input.ICommand#ENTER_GROUP} and {@link yfiles.input.ICommand#EXIT_GROUP}.
       * </p>
       * @returns whether the input mode should request the mutex.
       * @protected
       */
      shouldRequestMutex():boolean;
      /**
       * Callback predicate method that determines whether a given model item should be selected.
       * <p>
       * This implementation uses the {@link yfiles.input.NavigationInputMode#selectableItems} property to determine whether the <code>item</code> should be selected.
       * </p>
       * @param item The model item.
       * @returns Whether the item should be selected by the navigational commands.
       * @protected
       */
      shouldSelect(item:yfiles.graph.IModelItem):boolean;
      /**
       * Predicate method that decides whether it is allowed to {@link yfiles.input.NavigationInputMode#toggleExpansionState toggle} the collapsed state of the given group node.
       * <p>
       * This implementation uses {@link yfiles.input.NavigationInputMode#shouldExpandGroup} and {@link yfiles.input.NavigationInputMode#shouldCollapseGroup}
       * respectively.
       * </p>
       * @param node
       * @returns Whether to expand the given group or not.
       * @protected
       */
      shouldToggleExpansionState(node:yfiles.graph.INode):boolean;
      /**
       * Toggles the {@link yfiles.graph.IFoldingView#isExpanded expanded/collapsed} state for the given group node in the {@link yfiles.graph.IFoldingView}.
       * @param groupNode The group node to toggle the state for.
       * @see yfiles.graph.FoldingManager
       */
      toggleExpansionState(groupNode:yfiles.graph.INode):void;
      /**
       * Overridden to only return <code>true</code> if this instance does not currently {@link yfiles.input.ConcurrencyController#hasMutex have the input mutex}.
       * @returns <code>true</code> iff this instance does not {@link #hasMutex own the mutex}.
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.NavigationInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.NavigationInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.NavigationInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets or sets the item types that can be navigated to by this mode.
       * @see yfiles.input.NavigationInputMode#shouldNavigateTo
       * @type {yfiles.graph.GraphItemTypes}
       */
      navigableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets the types of the items that should be selectable by this instance.
       * <p>
       * The selectable items. The default value is {@link yfiles.graph.GraphItemTypes#NODE}.
       * </p>
       * @default yfiles.graph.GraphItemTypes.NODE
       * @type {yfiles.graph.GraphItemTypes}
       */
      selectableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets the the location that should be kept fixed if toggling a group node state.
       * <p>
       * Default value is {@link yfiles.input.NodeAlignmentPolicy#NONE}
       * </p>
       * @see yfiles.input.NavigationInputMode#expandGroup
       * @default yfiles.input.NodeAlignmentPolicy.NONE
       * @type {yfiles.input.NodeAlignmentPolicy}
       */
      autoGroupNodeAlignmentPolicy:yfiles.input.NodeAlignmentPolicy;
      /**
       * Gets or sets the {@link yfiles.input.NavigationInputMode#graphComponent} this mode acts on.
       * @type {yfiles.view.GraphComponent}
       */
      graphComponent:yfiles.view.GraphComponent;
      /**
       * Gets the list of commands that are available in this instance.
       * <p>
       * By default, all supported commands are available
       * </p>
       * <p>
       * Removing commands from this collection also removes the command bindings registered by this instance.
       * </p>
       * <p>
       * Add supported commands to make them available in this instance.
       * </p>
       * <p>
       * Supported commands are
       * </p>
       * <ul>
       * <li>{@link yfiles.input.ICommand#MOVE_LEFT}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_RIGHT}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_UP}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_TO_PAGE_UP}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_TO_PAGE_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_FOCUS_BACK}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_FOCUS_FORWARD}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_FOCUS_UP}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_FOCUS_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_FOCUS_PAGE_UP}</li>
       * <li>{@link yfiles.input.ICommand#MOVE_FOCUS_PAGE_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#EXTEND_SELECTION_LEFT}</li>
       * <li>{@link yfiles.input.ICommand#EXTEND_SELECTION_RIGHT}</li>
       * <li>{@link yfiles.input.ICommand#EXTEND_SELECTION_UP}</li>
       * <li>{@link yfiles.input.ICommand#EXTEND_SELECTION_DOWN}</li>
       * <li>{@link yfiles.input.ICommand#SELECT_TO_PAGE_UP}</li>
       * <li>{@link yfiles.input.ICommand#SELECT_TO_PAGE_DOWN}</li>
       * </ul>
       * @type {yfiles.collections.ICollection.<yfiles.input.ICommand>}
       */
      availableCommands:yfiles.collections.ICollection<yfiles.input.ICommand>;
      /**
       * Gets or sets a value indicating whether to use the {@link yfiles.view.GraphComponent#currentItem} as a fallback for the commands if no item is provided in the
       * parameter and the current selection is empty.
       * <p>
       * This applies to the following commands:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.ICommand#ENTER_GROUP}</li>
       * <li>{@link yfiles.input.ICommand#EXPAND_GROUP}</li>
       * <li>{@link yfiles.input.ICommand#COLLAPSE_GROUP}</li>
       * <li>{@link yfiles.input.ICommand#TOGGLE_EXPANSION_STATE}</li>
       * </ul>
       * @default false
       * @type {boolean}
       */
      useCurrentItemForCommands:boolean;
      /**
       * Gets or sets the {@link yfiles.view.IGraphSelection} this mode operates on.
       * @type {yfiles.view.IGraphSelection}
       */
      graphSelection:yfiles.view.IGraphSelection;
      /**
       * Gets or sets the graph this mode operates on.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Gets or sets a value that determines whether it is allowed to {@link yfiles.input.NavigationInputMode#enterGroup enter group nodes} via the
       * {@link yfiles.input.ICommand#ENTER_GROUP}.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.NavigationInputMode#shouldEnterGroup
       * @see yfiles.graph.FoldingManager
       * @type {boolean}
       */
      allowEnterGroup:boolean;
      /**
       * Gets or sets a value that determines whether it is allowed to collapse group nodes via the
       * {@link yfiles.input.ICommand#COLLAPSE_GROUP}.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.NavigationInputMode#collapseGroup
       * @see yfiles.input.NavigationInputMode#shouldCollapseGroup
       * @see yfiles.graph.FoldingManager
       * @type {boolean}
       */
      allowCollapseGroup:boolean;
      /**
       * Gets or sets a value that determines whether it is allowed to {@link yfiles.input.NavigationInputMode#expandGroup expand group nodes} via the
       * {@link yfiles.input.ICommand#EXPAND_GROUP}.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.NavigationInputMode#shouldExpandGroup
       * @see yfiles.graph.FoldingManager
       * @type {boolean}
       */
      allowExpandGroup:boolean;
      /**
       * Gets or sets a value that determines whether it is allowed to {@link yfiles.input.NavigationInputMode#exitGroup exit the current group node} via the
       * {@link yfiles.input.ICommand#EXIT_GROUP}.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.NavigationInputMode#shouldExitGroup
       * @see yfiles.graph.FoldingManager
       * @default true
       * @type {boolean}
       */
      allowExitGroup:boolean;
      /**
       * Gets or sets a value that determines whether {@link yfiles.view.CanvasComponent#fitContent} or the {@link yfiles.input.ICommand#FIT_CONTENT}
       * should be triggered after a group navigation action.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      fitContentAfterGroupActions:boolean;
      /**
       * Adds the given listener for the <code>GroupCollapsing</code> event that occurs before a group will be Collapsed.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupCollapsingListener
       */
      addGroupCollapsingListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupCollapsing</code> event that occurs before a group will be Collapsed.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupCollapsingListener
       */
      removeGroupCollapsingListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been Collapsed.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupCollapsedListener
       */
      addGroupCollapsedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been Collapsed.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupCollapsedListener
       */
      removeGroupCollapsedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupExpanding</code> event that occurs before a group will be {@link yfiles.input.NavigationInputMode#expandGroup Expanded}.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupExpandingListener
       */
      addGroupExpandingListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupExpanding</code> event that occurs before a group will be {@link yfiles.input.NavigationInputMode#expandGroup Expanded}.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupExpandingListener
       */
      removeGroupExpandingListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been {@link yfiles.input.NavigationInputMode#expandGroup Expanded}.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupExpandedListener
       */
      addGroupExpandedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been {@link yfiles.input.NavigationInputMode#expandGroup Expanded}.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupExpandedListener
       */
      removeGroupExpandedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupEntering</code> event that occurs before a group will be {@link yfiles.input.NavigationInputMode#enterGroup Entered}.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupEnteringListener
       */
      addGroupEnteringListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupEntering</code> event that occurs before a group will be {@link yfiles.input.NavigationInputMode#enterGroup Entered}.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupEnteringListener
       */
      removeGroupEnteringListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupEntered</code> event that occurs whenever a group has been {@link yfiles.input.NavigationInputMode#enterGroup Entered}.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupEnteredListener
       */
      addGroupEnteredListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupEntered</code> event that occurs whenever a group has been {@link yfiles.input.NavigationInputMode#enterGroup Entered}.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupEnteredListener
       */
      removeGroupEnteredListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupExiting</code> event that occurs before a group will be {@link yfiles.input.NavigationInputMode#exitGroup Exited}.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupExitingListener
       */
      addGroupExitingListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupExiting</code> event that occurs before a group will be {@link yfiles.input.NavigationInputMode#exitGroup Exited}.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupExitingListener
       */
      removeGroupExitingListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupExited</code> event that occurs whenever a group has been {@link yfiles.input.NavigationInputMode#exitGroup Exited}.
       * @param listener The listener to add.
       * @see yfiles.input.NavigationInputMode#removeGroupExitedListener
       */
      addGroupExitedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupExited</code> event that occurs whenever a group has been {@link yfiles.input.NavigationInputMode#exitGroup Exited}.
       * @param listener The listener to remove.
       * @see yfiles.input.NavigationInputMode#addGroupExitedListener
       */
      removeGroupExitedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      static $class:yfiles.lang.Class;
    }
    export enum MoveFocusDirection{
      /**
       * No direction.
       */
      NONE,
      /**
       * In the direction of the negative x Axis.
       */
      LEFT,
      /**
       * In the direction of the positive x Axis.
       */
      RIGHT,
      /**
       * In the direction of the negative y Axis.
       */
      UP,
      /**
       * In the direction of the positive y Axis.
       */
      DOWN,
      /**
       * In the direction of the {@link yfiles.graph.IGraph#setParent parent} in the grouped graph.
       */
      PARENT,
      /**
       * In the direction of the {@link yfiles.graph.IGraph#getChildren children} in the grouped graph.
       */
      CHILD
    }
    export enum NodeAlignmentPolicy{
      /**
       * Don't fix any specific point, only use the layout from the target state.
       */
      NONE,
      /**
       * Fix the center point.
       */
      CENTER,
      /**
       * Fix the upper left corner.
       */
      TOP_LEFT,
      /**
       * Fix the lower left corner.
       */
      BOTTOM_LEFT,
      /**
       * Fix the upper right corner.
       */
      TOP_RIGHT,
      /**
       * Fix the lower right corner.
       */
      BOTTOM_RIGHT,
      /**
       * Fix the uppermost center.
       */
      TOP_CENTER,
      /**
       * Fix the lowermost center.
       */
      BOTTOM_CENTER,
      /**
       * Fix the left center.
       */
      CENTER_LEFT,
      /**
       * Fix the right center.
       */
      CENTER_RIGHT
    }
    export enum OrthogonalEdgeEditingPolicy{
      /**
       * Enables orthogonal edge creation or editing for a specific sub-aspect automatically if an {@link yfiles.input.OrthogonalEdgeEditingContext#enabled} {@link yfiles.input.OrthogonalEdgeEditingContext}
       * instance is set in the {@link yfiles.input.GraphEditorInputMode#orthogonalEdgeEditingContext parent input mode}.
       */
      AUTO,
      /**
       * Enables orthogonal edge creation or editing for a specific sub-aspect of edge creation.
       */
      ALWAYS,
      /**
       * Disables orthogonal edge creation or editing for a specific sub-aspect of edge creation.
       */
      NEVER
    }
    export enum SegmentOrientation{
      /**
       * The segment has a horizontal orientation, which means that the end points should be moved vertically in synch so that
       * the orientation is not destroyed.
       */
      HORIZONTAL,
      /**
       * The segment has a vertical orientation, which means that the end points should be moved horizontally in synch so that
       * the orientation is not destroyed.
       */
      VERTICAL,
      /**
       * The segment does not need to be treated especially, both endpoints can be moved independently from each other.
       */
      NON_ORTHOGONAL
    }
    /**
     * Concrete {@link yfiles.input.SnapContext} implementation which handles snapping movement for {@link yfiles.graph.IGraph} items.
     * <p>
     * This class provides a set of properties that determine the way snapping operations are being performed in the context of
     * {@link yfiles.graph.IGraph}s. {@link yfiles.input.IPositionHandler} implementations and the like can use the following idiom to retrieve
     * instances of this class:
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 01 Getting Started, step 15 Snapping</li>
     * </ul>
     * @see yfiles.input.GraphEditorInputMode
     * @see yfiles.input.GraphEditorInputMode#snapContext
     * @class
     * @extends {yfiles.input.SnapContext}
     */
    export interface GraphSnapContext extends yfiles.input.SnapContext{}
    export class GraphSnapContext {
      /**
       * Creates a new instance with default settings.
       */
      constructor();
      /**
       * Adds {@link yfiles.input.MovementInfo} representing an {@link yfiles.graph.IBend} that will be considered for the next drag gesture.
       * <p>
       * The default {@link yfiles.input.IHandle} of an {@link yfiles.graph.IBend} calls this method in its {@link yfiles.input.IDragHandler#initializeDrag}
       * method so snap results are calculated when this bend is repositioned. Custom {@link yfiles.input.IHandle} implementations can
       * add infos created by {@link yfiles.input.GraphSnapContext#getMovementInfo}.
       * </p>
       * @param info The {@link } representing the bend to be moved. Can be created with
       * {@link #getMovementInfo}.
       */
      addBendToBeMoved(info:yfiles.input.MovementInfo):void;
      /**
       * Adds {@link yfiles.input.MovementInfo} representing the end of an {@link yfiles.graph.IEdge} that will be considered for the next drag
       * gesture.
       * <p>
       * The {@link yfiles.input.IHandle} returned by the default {@link yfiles.input.IEdgePortHandleProvider} of an {@link yfiles.graph.IEdge} calls
       * this method in its {@link yfiles.input.IDragHandler#initializeDrag} method so snap results are calculated when reparenting this
       * edge end. Custom {@link yfiles.input.IHandle} implementations can call {@link yfiles.input.GraphSnapContext#getMovementInfos} and add
       * the first or last {@link yfiles.input.MovementInfo} of the returned list.
       * </p>
       * @param info The {@link } representing the edge end to be reparented. Can be created with
       * {@link #getMovementInfos}.
       * @param sourceEnd <code>true</code> if the <code>info</code> represents the source end of the edge.
       */
      addEdgeEndToBeMoved(info:yfiles.input.MovementInfo,sourceEnd:boolean):void;
      /**
       * Adds an item which will be reshaped.
       * @param item The item to be reshaped.
       */
      addItemToBeReshaped(item:yfiles.graph.IModelItem):void;
      /**
       * Adds {@link yfiles.input.MovementInfo} representing an {@link yfiles.graph.INode} that will be considered for the next drag gesture.
       * <p>
       * The default {@link yfiles.input.IPositionHandler} of an {@link yfiles.graph.INode} calls this method in its {@link yfiles.input.IDragHandler#initializeDrag}
       * method so snap results are calculated when this node is repositioned. Custom {@link yfiles.input.IPositionHandler}
       * implementations can add infos created by {@link yfiles.input.GraphSnapContext#getMovementInfo}.
       * </p>
       * @param info The {@link } representing the node to be moved. Can be created with
       * {@link #getMovementInfo}.
       */
      addNodeToBeMoved(info:yfiles.input.MovementInfo):void;
      /**
       * Adds an orthogonally moved segment to this context for which this instance will automatically collect
       * {@link yfiles.input.SnapResult}s and provide a default visualization.
       * <p>
       * This method can be called during initialization to collect orthogonal segments that should be tested against {@link yfiles.input.GraphSnapContext#fixedSegmentSnapLines} and {@link yfiles.input.GraphSnapContext#additionalSnapLines}.
       * </p>
       * @param start The starting point of the manipulated segment.
       * @param end The end point of the manipulated segment.
       */
      addOrthogonalSegment(start:yfiles.input.MovementInfo,end:yfiles.input.MovementInfo):void;
      /**
       * Adds {@link yfiles.input.MovementInfo} representing an {@link yfiles.graph.IPort} that will be considered for the next drag gesture.
       * <p>
       * The default {@link yfiles.input.IHandle} of an {@link yfiles.graph.IPort} calls this method in its {@link yfiles.input.IDragHandler#initializeDrag}
       * method so snap results are calculated for this port. Custom {@link yfiles.input.IHandle} implementations can add infos created
       * by {@link yfiles.input.GraphSnapContext#getMovementInfo}.
       * </p>
       * @param info The {@link } representing the port to be moved. Can be created with
       * {@link #getMovementInfo}.
       */
      addPortToBeMoved(info:yfiles.input.MovementInfo):void;
      /**
       * Adds an entry for the same-height-snapping.
       * <p>
       * During {@link yfiles.input.SnapContext#isInitializing initialization} this method can be used to register entries that can later be retrieved using the {@link yfiles.input.GraphSnapContext#findSameHeightEntries}
       * method. The rectangle will be used to obtain the height value and will also be used for the visualization of the snap
       * result.
       * </p>
       * @param rectangle The rectangle to obtain the height value from. This rectangle will be used for the snap result visualization.
       * @see yfiles.input.GraphSnapContext#addSameWidthEntry
       */
      addSameHeightEntry(rectangle:yfiles.geometry.Rect):void;
      /**
       * Adds an entry for the same-width-snapping.
       * <p>
       * During {@link yfiles.input.SnapContext#isInitializing initialization} this method can be used to register entries that can later be retrieved using the {@link yfiles.input.GraphSnapContext#findSameWidthEntries}
       * method. The rectangle will be used to obtain the width value and will also be used for the visualization of the snap
       * result.
       * </p>
       * @param rectangle The rectangle to obtain the width value from. This rectangle will be used for the snap result visualization.
       * @see yfiles.input.GraphSnapContext#addSameHeightEntry
       */
      addSameWidthEntry(rectangle:yfiles.geometry.Rect):void;
      /**
       * Collects the {@link yfiles.input.OrthogonalSnapLine}s for the nodes which are not moving.
       * <p>
       * Delegates to the {@link yfiles.input.ISnapLineProvider} implementation in the lookup of the {@link yfiles.graph.INode}.
       * </p>
       * @param evt
       * @protected
       */
      collectFixedNodeSnapLines(evt:yfiles.input.CollectGraphSnapLinesEventArgs):void;
      /**
       * Collects the {@link yfiles.input.OrthogonalSnapLine}s for the ports which are not moving.
       * <p>
       * Delegates to the {@link yfiles.input.ISnapLineProvider} implementation in the lookup of the {@link yfiles.graph.IPort}.
       * </p>
       * @param evt
       * @protected
       */
      collectFixedPortSnapLines(evt:yfiles.input.CollectGraphSnapLinesEventArgs):void;
      /**
       * Collects snap lines for edge segments.
       * <p>
       * Delegates to the {@link yfiles.input.ISnapLineProvider} implementation in the lookup of the {@link yfiles.graph.IEdge}. This
       * implementation should then make us of {@link yfiles.input.GraphSnapContext#getMovementInfos} to determine what segments are
       * fixed.
       * </p>
       * @param evt
       * @protected
       */
      collectFixedSegmentSnapLines(evt:yfiles.input.CollectGraphSnapLinesEventArgs):void;
      /**
       * Collects all node widths and heights in the collections {@link yfiles.input.GraphSnapContext#findSameWidthEntries} and
       * {@link yfiles.input.GraphSnapContext#findSameHeightEntries}.
       * <p>
       * This method will only be called if {@link yfiles.input.GraphSnapContext#collectNodeSizes} is enabled and delegates to {@link yfiles.input.GraphSnapContext#addSameHeightEntry} and
       * {@link yfiles.input.GraphSnapContext#addSameWidthEntry}.
       * </p>
       * @protected
       */
      collectSameSizeEntries():void;
      /**
       * Finds pairs of nodes that are visually adjacent to create {@link yfiles.input.GraphSnapContext#collectNodePairCenterSnapLines center} and {@link yfiles.input.GraphSnapContext#collectNodePairSnapLines same distance} snap lines.
       * @protected
       */
      collectVisibleNodePairs():void;
      /**
       * Finds the entries ({@link yfiles.geometry.Rect}s) that have been registered using the {@link yfiles.input.GraphSnapContext#addSameHeightEntry}
       * previously that best match the given constraints.
       * @param size The size to find the closest set of identical sizes.
       * @param maxSize The maximum allowed size.
       * @param minSize The minimum allowed size.
       * @returns The set of rectangles that have been added previously using {@link #addSameHeightEntry}.
       */
      findSameHeightEntries(size:number,maxSize:number,minSize:number):yfiles.collections.IListEnumerable<yfiles.geometry.Rect>;
      /**
       * Finds the entries ({@link yfiles.geometry.Rect}s) that have been registered using the {@link yfiles.input.GraphSnapContext#addSameWidthEntry}
       * previously that best match the given constraints.
       * @param size The size to find the closest set of identical sizes.
       * @param maxSize The maximum allowed size.
       * @param minSize The minimum allowed size.
       * @returns The set of rectangles that have been added previously using {@link #addSameWidthEntry}.
       */
      findSameWidthEntries(size:number,maxSize:number,minSize:number):yfiles.collections.IListEnumerable<yfiles.geometry.Rect>;
      /**
       * Gets a subset of {@link yfiles.input.GraphSnapContext#additionalSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
       * @param orientation The orientation of the lines to return.
       * @param snapType The snap type to be included.
       * @param area The area to test for intersection.
       * @returns An enumerable of the snap lines that match the provided criteria.
       */
      getAdditionalSnapLines(orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,area:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * Gets a subset of {@link yfiles.input.GraphSnapContext#fixedNodeSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
       * @param orientation The orientation of the lines to return.
       * @param snapType The snap type to be included.
       * @param area The area to test for intersection.
       * @returns An enumerable of the snap lines that match the provided criteria.
       */
      getFixedNodeSnapLines(orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,area:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * Gets a subset of {@link yfiles.input.GraphSnapContext#fixedPortSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
       * @param orientation The orientation of the lines to return.
       * @param snapType The snap type to be included.
       * @param area The area to test for intersection.
       * @returns An enumerable of the snap lines that match the provided criteria.
       */
      getFixedPortSnapLines(orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,area:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * Gets a subset of {@link yfiles.input.GraphSnapContext#fixedSegmentSnapLines} that matches the given <code>orientation</code>, <code>snapType</code> and intersects with the provided <code>area</code>.
       * @param orientation The orientation of the lines to return.
       * @param snapType The snap type to be included.
       * @param area The area to test for intersection.
       * @returns An enumerable of the snap lines that match the provided criteria.
       */
      getFixedSegmentSnapLines(orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,area:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * Gets the movement info for the given port.
       * <p>
       * This is either the info passed to {@link yfiles.input.GraphSnapContext#addNodeToBeMoved} or another, dynamically created
       * instance, if no specific one had been registered.
       * </p>
       * @param port The port to obtain the info for.
       * @returns An instance of {@link } that describes the movement of the port.
       */
      getMovementInfo(port:yfiles.graph.IPort):yfiles.input.MovementInfo;
      /**
       * Gets the movement info for the given node.
       * <p>
       * This is either the info passed to {@link yfiles.input.GraphSnapContext#addNodeToBeMoved} or another, dynamically created
       * instance, if no specific one had been registered.
       * </p>
       * @param node The node to obtain the info for.
       * @returns An instance of {@link } that describes the movement of the node.
       */
      getMovementInfo(node:yfiles.graph.INode):yfiles.input.MovementInfo;
      /**
       * Gets the movement info for the given bend.
       * <p>
       * This is either the info passed to {@link yfiles.input.GraphSnapContext#addBendToBeMoved} or a dynamically created instance if no
       * specific one had been registered.
       * </p>
       * @param bend The bend to obtain the info for.
       * @returns An instance of {@link } that describes the movement of the bend.
       */
      getMovementInfo(bend:yfiles.graph.IBend):yfiles.input.MovementInfo;
      /**
       * Gets the {@link yfiles.input.MovementInfo movement information} for the provided edge.
       * <p>
       * The result describes the shape of the edge and the reshaping behavior of the geometry. The first element in the
       * enumerable is the source end of the edge, followed by bend like elements and the target side. Note that it cannot in
       * general be assumed {@link yfiles.input.MovementInfo#movedItem} of the first and last element is either a {@link yfiles.graph.IPort} or {@link yfiles.graph.IEdge}, also items in
       * between may not necessarily be bound to {@link yfiles.graph.IBend}s and therefore their {@link yfiles.input.MovementInfo#movedItem} can be of a different type than
       * {@link yfiles.graph.IBend}.
       * </p>
       * @param edge The edge to obtain the movement information of.
       * @returns An {@link } of {@link }s items that describe the geometry of the edge being modified.
       */
      getMovementInfos(edge:yfiles.graph.IEdge):yfiles.collections.IListEnumerable<yfiles.input.MovementInfo>;
      /**
       * Raises the {@link yfiles.input.GraphSnapContext#addCollectSnapLinesListener CollectSnapLines} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onCollectSnapLines(evt:yfiles.input.CollectGraphSnapLinesEventArgs):void;
      /**
       * Gets the nodes that stay fixed for the current edit.
       * @protected
       * @type {yfiles.collections.ICollection.<yfiles.graph.INode>}
       */
      fixedNodes:yfiles.collections.ICollection<yfiles.graph.INode>;
      /**
       * Adds the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link yfiles.input.SnapContext#dragInitialized initialized} to
       * collect {@link yfiles.input.OrthogonalSnapLine}s.
       * <p>
       * Event handlers should add snap lines to the caller using the methods provided by
       * {@link yfiles.input.CollectGraphSnapLinesEventArgs}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphSnapContext#removeCollectSnapLinesListener
       */
      addCollectSnapLinesListener(listener:(sender:Object,evt:yfiles.input.CollectGraphSnapLinesEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CollectSnapLines</code> event that occurs every time this instance has been {@link yfiles.input.SnapContext#dragInitialized initialized} to
       * collect {@link yfiles.input.OrthogonalSnapLine}s.
       * <p>
       * Event handlers should add snap lines to the caller using the methods provided by
       * {@link yfiles.input.CollectGraphSnapLinesEventArgs}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphSnapContext#addCollectSnapLinesListener
       */
      removeCollectSnapLinesListener(listener:(sender:Object,evt:yfiles.input.CollectGraphSnapLinesEventArgs)=>void):void;
      /**
       * Determines whether this instance should collect {@link yfiles.input.GraphSnapContext#fixedSegmentSnapLines fixed snap lines} for edge segments.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @see yfiles.input.ISnapLineProvider
       * @see yfiles.input.GraphSnapContext#collectFixedSegmentSnapLines
       * @default true
       * @type {boolean}
       */
      collectEdgeSnapLines:boolean;
      /**
       * Determines whether this instance should collect the sizes of fixed nodes.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphSnapContext#collectSameSizeEntries
       * @default true
       * @type {boolean}
       */
      collectNodeSizes:boolean;
      /**
       * Determines whether this instance should collect {@link yfiles.input.GraphSnapContext#fixedNodeSnapLines fixed snap lines} for nodes.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @see yfiles.input.ISnapLineProvider
       * @see yfiles.input.GraphSnapContext#collectFixedNodeSnapLines
       * @default true
       * @type {boolean}
       */
      collectNodeSnapLines:boolean;
      /**
       * Determines whether this instance should collect {@link yfiles.input.GraphSnapContext#fixedPortSnapLines fixed snap lines} for ports.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @see yfiles.input.ISnapLineProvider
       * @see yfiles.input.GraphSnapContext#collectFixedPortSnapLines
       * @default true
       * @type {boolean}
       */
      collectPortSnapLines:boolean;
      /**
       * Whether this context will automatically snap the movements of bends so that the two adjacent segments become
       * horizontally or vertically oriented.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      snapBendAdjacentSegments:boolean;
      /**
       * Whether this context will automatically snap the movements of nodes so that the first and last segments of edges become
       * orthogonal.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      snapPortAdjacentSegments:boolean;
      /**
       * Determines whether this context will automatically snap orthogonal edge segments to snap lines.
       * <p>
       * The default is true.
       * </p>
       * @default true
       * @type {boolean}
       */
      snapSegmentsToSnapLines:boolean;
      /**
       * Whether this context will automatically snap the bends to snap lines.
       * <p>
       * The default is false.
       * </p>
       * @default false
       * @type {boolean}
       */
      snapBendsToSnapLines:boolean;
      /**
       * Whether {@link yfiles.input.INodeSnapResultProvider} implementations should snap nodes to snap lines.
       * <p>
       * The default is true.
       * </p>
       * @default true
       * @type {boolean}
       */
      snapNodesToSnapLines:boolean;
      /**
       * The preferred distance between node borders.
       * <p>
       * This will influence the creation of the node snap lines. For each fixed node there will be snap lines on the borders of
       * the node itself and around the node at the specified distance.
       * </p>
       * <p>
       * The default is 0.0d which disables distant snap lines around the nodes.
       * </p>
       * @default 0.0
       * @type {number}
       */
      nodeToNodeDistance:number;
      /**
       * The preferred distance between node borders and edge segments.
       * <p>
       * This will influence the creation of the node snap lines. For each fixed node there will be snap lines on the borders of
       * the node itself and around the node at the specified distance to which orthogonal edge segments can snap.
       * </p>
       * <p>
       * The default is -1.0d which disables distant snap lines around the nodes for edge segments.
       * </p>
       * @default -1.0
       * @type {number}
       */
      nodeToEdgeDistance:number;
      /**
       * Gets the preferred distance between edges.
       * <p>
       * This will influence the creation of the segment snap lines. For each orthogonally oriented fixed segment there will be
       * snap lines on the segment itself and to both sides of the segment at the specified distance.
       * </p>
       * <p>
       * The default is 0.0d which disables parallel snap lines.
       * </p>
       * @default 0.0
       * @type {number}
       */
      edgeToEdgeDistance:number;
      /**
       * Determines whether this context will automatically snap the movement in such a way that the overall move gesture is
       * constrained to the horizontal or vertical axis.
       * <p>
       * The default is false.
       * </p>
       * @default false
       * @type {boolean}
       */
      snapOrthogonalMovement:boolean;
      /**
       * Determines whether this instance will try to align the centers of adjacent nodes so that their centers are equally far
       * away from each other.
       * <p>
       * Note that this will only make a difference for differently sized nodes.
       * </p>
       * <p>
       * By default this feature is turned off, so that the instance will try to distribute nodes evenly, so that there is the
       * same spacing between their borders, not between their centers.
       * </p>
       * @default false
       * @type {boolean}
       */
      collectNodePairCenterSnapLines:boolean;
      /**
       * Gets or sets a value indicating whether this instance will collect pairs of adjacent nodes and add snapping logic so
       * that moved nodes will snap so that the distances to the adjacent nodes are equal.
       * @default true
       * @type {boolean}
       */
      collectNodePairSnapLines:boolean;
      /**
       * Gets or sets a value indicating whether this instance will collect pairs of adjacent nodes and add snapping logic so
       * that moved edge segments will snap so that the distances to the adjacent nodes are equal.
       * @default true
       * @type {boolean}
       */
      collectNodePairSegmentSnapLines:boolean;
      /**
       * Gets or sets the type of the grid snapping that should be performed.
       * <p>
       * Determines which grid elements an object should snap to.
       * </p>
       * @see yfiles.input.GraphSnapContext#nodeGridConstraintProvider
       * @see yfiles.input.GraphSnapContext#bendGridConstraintProvider
       * @see yfiles.input.GraphSnapContext#portGridConstraintProvider
       * @default yfiles.input.GridSnapTypes.NONE
       * @type {yfiles.input.GridSnapTypes}
       */
      gridSnapType:yfiles.input.GridSnapTypes;
      /**
       * Gets or sets the GridSnapDistance.
       * <p>
       * The maximum distance between the current mouse coordinates and the coordinates to which the mouse will snap. The
       * distance is interpreted in view coordinates.
       * </p>
       * @default 10.0
       * @type {number}
       */
      gridSnapDistance:number;
      /**
       * Gets or sets the grid constraint provider for the {@link yfiles.graph.INode}s.
       * @type {yfiles.input.IGridConstraintProvider.<yfiles.graph.INode>}
       */
      nodeGridConstraintProvider:yfiles.input.IGridConstraintProvider<yfiles.graph.INode>;
      /**
       * Gets or sets the grid constraint provider for the {@link yfiles.graph.IBend}s.
       * @type {yfiles.input.IGridConstraintProvider.<yfiles.graph.IBend>}
       */
      bendGridConstraintProvider:yfiles.input.IGridConstraintProvider<yfiles.graph.IBend>;
      /**
       * Gets or sets the grid constraint provider for the {@link yfiles.graph.IPort}s.
       * @type {yfiles.input.IGridConstraintProvider.<yfiles.graph.IPort>}
       */
      portGridConstraintProvider:yfiles.input.IGridConstraintProvider<yfiles.graph.IPort>;
      /**
       * Gets or sets the amount by which snap lines that are induced by existing edge segments and node borders are being
       * extended.
       * <p>
       * The default is <code>40.0d</code>, this value will be used to extend the ends of the snap lines.
       * </p>
       * @default 40.0
       * @type {number}
       */
      snapLineExtension:number;
      /**
       * Gets or sets a value indicating whether to crop the snap lines at obstacles.
       * <p>
       * The default is <code>true</code>
       * </p>
       * @type {boolean}
       */
      cropSnapLines:boolean;
      /**
       * The collection of {@link yfiles.input.OrthogonalSnapLine}s for fixed nodes.
       * <p>
       * This collection is only available if {@link yfiles.input.SnapContext#isInitialized} is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphSnapContext#getFixedNodeSnapLines
       * @type {yfiles.collections.IEnumerable.<yfiles.input.OrthogonalSnapLine>}
       */
      fixedNodeSnapLines:yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * The collection of {@link yfiles.input.OrthogonalSnapLine}s for fixed segments.
       * <p>
       * This collection is only available if {@link yfiles.input.SnapContext#isInitialized} is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphSnapContext#getFixedSegmentSnapLines
       * @type {yfiles.collections.IEnumerable.<yfiles.input.OrthogonalSnapLine>}
       */
      fixedSegmentSnapLines:yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * The collection of {@link yfiles.input.OrthogonalSnapLine}s to which ports should be snapping.
       * <p>
       * This collection is only available if {@link yfiles.input.SnapContext#isInitialized} is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphSnapContext#getFixedPortSnapLines
       * @type {yfiles.collections.IEnumerable.<yfiles.input.OrthogonalSnapLine>}
       */
      fixedPortSnapLines:yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * The collection of additional {@link yfiles.input.OrthogonalSnapLine}s.
       * <p>
       * This collection is only available if {@link yfiles.input.SnapContext#isInitialized} is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphSnapContext#getAdditionalSnapLines
       * @type {yfiles.collections.IEnumerable.<yfiles.input.OrthogonalSnapLine>}
       */
      additionalSnapLines:yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      static $class:yfiles.lang.Class;
    }
    export enum MoveTypes{
      /**
       * The element does not move at all during the edit.
       */
      FIXED,
      /**
       * The element moves linearly with the mouse in horizontal direction.
       */
      LINEAR_MOVE_X,
      /**
       * The element moves linearly with the mouse in vertical direction.
       */
      LINEAR_MOVE_Y,
      /**
       * The element moves linearly with the mouse in both directions.
       */
      LINEAR_MOVE,
      /**
       * The element moves in horizontal direction but not linearly/predictably with the mouse.
       * <p>
       * A typical example is a port whose position is adjusted during a resize of its owning node.
       * </p>
       * <p>
       * Elements which do not move linearly with the mouse cannot snap to a defined position.
       * </p>
       */
      ARBITRARY_MOVE_X,
      /**
       * The element moves in vertical direction but not linearly/predictably with the mouse.
       * <p>
       * A typical example is a port whose position is adjusted during a resize of its owning node.
       * </p>
       * <p>
       * Elements which do not move linearly with the mouse cannot snap to a defined position.
       * </p>
       */
      ARBITRARY_MOVE_Y,
      /**
       * The element moves in both directions but not linearly/predictably with the mouse.
       * <p>
       * A typical example is a port whose position is adjusted during a resize of its owning node.
       * </p>
       * <p>
       * Elements which do not move linearly with the mouse cannot snap to a defined position.
       * </p>
       */
      ARBITRARY_MOVE,
      /**
       * The bitwise combination of {@link yfiles.input.MoveTypes#LINEAR_MOVE_X} and {@link yfiles.input.MoveTypes#ARBITRARY_MOVE_X}
       */
      MOVEMENT_X,
      /**
       * The bitwise combination of {@link yfiles.input.MoveTypes#LINEAR_MOVE_Y} and {@link yfiles.input.MoveTypes#ARBITRARY_MOVE_Y}
       */
      MOVEMENT_Y
    }
    /**
     * A helper class that is used in the context of {@link yfiles.input.GraphSnapContext} to describe the movement of entities during
     * the edit process.
     * @class
     */
    export interface MovementInfo extends Object{}
    export class MovementInfo {
      /**
       * Initializes a new instance of the {@link yfiles.input.MovementInfo} class.
       * @param originalLocation The original location.
       * @param moveType The type of the move.
       * @param movedItem The item that is being moved.
       * @protected
       */
      constructor(originalLocation:yfiles.geometry.Point,moveType:yfiles.input.MoveTypes,movedItem:yfiles.graph.IModelItem);
      /**
       * Gets the new x coordinate of the element if the mouse has been moved by <code>dx</code> during the gesture.
       * <p>
       * This takes the {@link yfiles.input.MovementInfo#moveType} and {@link yfiles.input.MovementInfo#originalLocation} into account.
       * </p>
       * @param dx The delta x coordinate.
       * @returns The new x coordinate.
       */
      getX(dx:number):number;
      /**
       * Gets the new y coordinate of the element if the mouse has been moved by <code>dy</code> during the gesture.
       * <p>
       * This takes the {@link yfiles.input.MovementInfo#moveType} and {@link yfiles.input.MovementInfo#originalLocation} into account.
       * </p>
       * @param dy The delta y coordinate.
       * @returns The new y coordinate.
       */
      getY(dy:number):number;
      /**
       * Determines whether this instances moves synchronized with the provided info.
       * @param info The info to check.
       * @returns Whether this info and the given info move horizontally synchronized.
       * @see yfiles.input.MovementInfo#horizontalAnchor
       */
      movesHorizontallySynchronizedWith(info:yfiles.input.MovementInfo):boolean;
      /**
       * Determines whether this instances moves synchronized with the provided info.
       * @param info The info to check.
       * @returns Whether this info and the given info move vertically synchronized.
       * @see yfiles.input.MovementInfo#verticalAnchor
       */
      movesVerticallySynchronizedWith(info:yfiles.input.MovementInfo):boolean;
      /**
       * Gets the item that is being moved.
       * @type {yfiles.graph.IModelItem}
       */
      movedItem:yfiles.graph.IModelItem;
      /**
       * Gets or sets the horizontal movement info to which this info is anchored horizontally.
       * <p>
       * This info can be declared to be implicitly anchored to another info. Method {@link yfiles.input.MovementInfo#movesHorizontallySynchronizedWith}
       * will try to match against the anchors to determine if two infos are moving synchronized.
       * </p>
       * @type {yfiles.input.MovementInfo}
       */
      horizontalAnchor:yfiles.input.MovementInfo;
      /**
       * Gets or sets the vertical movement info to which this info is anchored vertically.
       * <p>
       * This info can be declared to be implicitly anchored to another info. Method {@link yfiles.input.MovementInfo#movesVerticallySynchronizedWith}
       * will try to match against the anchors to determine if two infos are moving synchronized.
       * </p>
       * @type {yfiles.input.MovementInfo}
       */
      verticalAnchor:yfiles.input.MovementInfo;
      /**
       * Gets the original location of the element at the time the editing started.
       * @type {yfiles.geometry.Point}
       */
      originalLocation:yfiles.geometry.Point;
      /**
       * Gets the current location as a live point.
       * @abstract
       * @type {yfiles.geometry.IPoint}
       */
      currentLocation:yfiles.geometry.IPoint;
      /**
       * Indicates the type of the movement for this instance.
       * @type {yfiles.input.MoveTypes}
       */
      moveType:yfiles.input.MoveTypes;
      /**
       * Gets a value indicating whether the y coordinate of the item is fixed during the move operation.
       * @type {boolean}
       */
      isVerticallyFixed:boolean;
      /**
       * Gets a value indicating whether the x coordinate of the item is fixed during the move operation.
       * @type {boolean}
       */
      isHorizontallyFixed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A snap line is a line segment in the world coordinate system to which other items (lines or points) snap during
     * interactive movements.
     * <p>
     * {@link yfiles.input.GraphSnapContext} and {@link yfiles.input.LabelSnapContext} make use of this class to interactively snap moving
     * elements to snap lines.
     * </p>
     * @class
     */
    export interface SnapLine extends Object{}
    export class SnapLine {
      /**
       * Creates a new snap line using the provided attributes.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param from The first location that delimits this snap line.
       * @param to The second location that delimits this snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       */
      constructor(snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,from:yfiles.geometry.Point,to:yfiles.geometry.Point,tag:Object,weight:number);
      /**
       * Resource key which determines a snap line having a fixed visualization independent of the {@link yfiles.input.SnapResult}.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_FIXED_LINE_KEY:string;
      /**
       * Resource key which determines a snap line with three special locations, for example the start, end and center of an edge
       * segment.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_EDGE_SEGMENT_KEY:string;
      /**
       * Resource key which determines a grid snap line having a fixed visualization independent of the {@link yfiles.input.SnapResult}.
       * @const
       * @static
       * @type {string}
       */
      static GRID_LINE_FIXED_LINE_KEY:string;
      /**
       * Resource key which determines a decorated snap line between the {@link yfiles.input.SnapLine#coordinates} and the snapped coordinates.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_VARIABLE_LINE_KEY:string;
      /**
       * Resource key which determines a blank line between the {@link yfiles.input.SnapLine#coordinates} and the snapped coordinates.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_BLANK_VARIABLE_LINE_KEY:string;
      /**
       * Resource key which determines a decorated line between the {@link yfiles.input.SnapLine#coordinates} and the snapped coordinates that is extended on both sides.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_EXTENDED_VARIABLE_LINE_KEY:string;
      /**
       * Resource key which determines a snap line which lets an item snap in a defined distance to another graph item.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_FIXED_DISTANCE_KEY:string;
      /**
       * Resource key which determines a snap line which lets an item snap in the center between the bounds of two other items.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_CENTER_BETWEEN_BOUNDS_KEY:string;
      /**
       * Resource key which determines a snap line which lets an item snap in the center between the centers of two other items.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_CENTER_BETWEEN_CENTERS_KEY:string;
      /**
       * Resource key which determines a snap line which lets an item snap if it has an equal distance to two other items.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_EQUAL_DISTANCE_BETWEEN_BOUNDS_KEY:string;
      /**
       * Resource key which determines a snap line which lets an item snap if it's center has an equal distance to the centers'
       * of two other items.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_EQUAL_DISTANCE_BETWEEN_CENTERS_KEY:string;
      /**
       * Resource key which determines the snapping of a resized item if it's new width is equal to another item.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_EQUAL_WIDTH_KEY:string;
      /**
       * Resource key which determines the snapping of a resized item if it's new height is equal to another item.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_EQUAL_HEIGHT_KEY:string;
      /**
       * Resource key which determines the snapping of an item to a grid position.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_TO_GRID_KEY:string;
      /**
       * Resource key which determines the {@link yfiles.view.Stroke} used for the snap line visualization.
       * @const
       * @static
       * @type {string}
       */
      static SNAP_LINE_STROKE_KEY:string;
      /**
       * The type of the line that describes how other items will snap to this line.
       * <p>
       * his is one of {@link yfiles.input.SnapLineSnapTypes#TOP}, {@link yfiles.input.SnapLineSnapTypes#BOTTOM},
       * {@link yfiles.input.SnapLineSnapTypes#LEFT}, {@link yfiles.input.SnapLineSnapTypes#RIGHT}, or {@link yfiles.input.SnapLineSnapTypes#CENTER}.
       * </p>
       * @type {yfiles.input.SnapLineSnapTypes}
       */
      snapType:yfiles.input.SnapLineSnapTypes;
      /**
       * The coordinates of the center point of the snap line.
       * @type {yfiles.geometry.Point}
       */
      coordinates:yfiles.geometry.Point;
      /**
       * The tag that is associated with this snap line.
       * <p>
       * The tag is most commonly used for the {@link yfiles.input.SnapResult#tag snap result tag}.
       * </p>
       * @type {Object}
       */
      tag:Object;
      /**
       * The weight/importance of this snap line.
       * <p>
       * Greater values indicate greater importance.
       * </p>
       * @type {number}
       */
      weight:number;
      /**
       * The first location that delimits this snap line.
       * @type {yfiles.geometry.Point}
       */
      from:yfiles.geometry.Point;
      /**
       * The second location that delimits this snap line.
       * @type {yfiles.geometry.Point}
       */
      to:yfiles.geometry.Point;
      /**
       * The resource key which determines the visual appearance of the snap line.
       * @type {string}
       */
      resourceKey:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An orthogonal snap line, in other words, one that is parallel to the x or y axis.
     * @class
     * @extends {yfiles.input.SnapLine}
     */
    export interface OrthogonalSnapLine extends yfiles.input.SnapLine{}
    export class OrthogonalSnapLine {
      /**
       * Creates a new snap line using the provided attributes.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       */
      constructor(orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,tag:Object,weight:number);
      /**
       * Creates a new snap line using the provided attributes.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param from The smaller value of the coordinate that delimits this snap line.
       * @param to The greater value of the coordinate that delimits this snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       */
      constructor(orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,from:number,to:number,tag:Object,weight:number);
      /**
       * The orientation of the snap line.
       * <p>
       * This is one of {@link yfiles.input.SnapLineOrientation#HORIZONTAL} or {@link yfiles.input.SnapLineOrientation#VERTICAL}.
       * </p>
       * @type {yfiles.input.SnapLineOrientation}
       */
      orientation:yfiles.input.SnapLineOrientation;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized {@link yfiles.input.OrthogonalSnapLine} that stores a {@link yfiles.input.NodeBasedSnapLine#node} as additional information that can be used for the
     * visualization.
     * @class
     * @extends {yfiles.input.OrthogonalSnapLine}
     */
    export interface NodeBasedSnapLine extends yfiles.input.OrthogonalSnapLine{}
    export class NodeBasedSnapLine {
      /**
       * Initializes a new instance of the {@link yfiles.input.NodeBasedSnapLine} class.
       * @param node The node to assign to {@link #node}.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       * @param from The smaller value of the coordinate that delimits this snap line.
       * @param to The greater value of the coordinate that delimits this snap line.
       */
      constructor(node:yfiles.graph.INode,orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,from:number,to:number,tag:Object,weight:number);
      /**
       * Initializes a new instance of the {@link yfiles.input.NodeBasedSnapLine} class.
       * @param node The node to assign to {@link #node}.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       */
      constructor(node:yfiles.graph.INode,orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,tag:Object,weight:number);
      /**
       * Gets the node that is associated with this snap line.
       * <p>
       * Usually this is the instance that induced this snap line.
       * </p>
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized {@link yfiles.input.OrthogonalSnapLine} that stores two {@link yfiles.graph.INode nodes} as additional information that can be used for the
     * visualization.
     * @see yfiles.input.NodePairBasedSnapLine#firstNode
     * @see yfiles.input.NodePairBasedSnapLine#secondNode
     * @class
     * @extends {yfiles.input.OrthogonalSnapLine}
     */
    export interface NodePairBasedSnapLine extends yfiles.input.OrthogonalSnapLine{}
    export class NodePairBasedSnapLine {
      /**
       * Initializes a new instance of the {@link yfiles.input.NodePairBasedSnapLine} class.
       * @param firstNode The first node to assign to {@link #firstNode}.
       * @param secondNode The second node to assign to {@link #secondNode}.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       * @param from The smaller value of the coordinate that delimits this snap line.
       * @param to The greater value of the coordinate that delimits this snap line.
       */
      constructor(firstNode:yfiles.graph.INode,secondNode:yfiles.graph.INode,orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,from:number,to:number,tag:Object,weight:number);
      /**
       * Initializes a new instance of the {@link yfiles.input.NodePairBasedSnapLine} class.
       * @param firstNode The first node to assign to {@link #firstNode}.
       * @param secondNode The second node to assign to {@link #secondNode}.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       */
      constructor(firstNode:yfiles.graph.INode,secondNode:yfiles.graph.INode,orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,tag:Object,weight:number);
      /**
       * Gets the first node that is associated with this snap line.
       * <p>
       * Usually this is one of the instances that induced this snap line.
       * </p>
       * @type {yfiles.graph.INode}
       */
      firstNode:yfiles.graph.INode;
      /**
       * Gets the second node that is associated with this snap line.
       * <p>
       * Usually this is one of the instances that induced this snap line.
       * </p>
       * @type {yfiles.graph.INode}
       */
      secondNode:yfiles.graph.INode;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized {@link yfiles.input.OrthogonalSnapLine} that stores a {@link yfiles.input.PointBasedSnapLine#point} instance as additional information that can be used for the
     * visualization.
     * @class
     * @extends {yfiles.input.OrthogonalSnapLine}
     */
    export interface PointBasedSnapLine extends yfiles.input.OrthogonalSnapLine{}
    export class PointBasedSnapLine {
      /**
       * Initializes a new instance of the {@link yfiles.input.PointBasedSnapLine} class.
       * @param point The point to assign to {@link #point}.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       * @param from The smaller value of the coordinate that delimits this snap line.
       * @param to The greater value of the coordinate that delimits this snap line.
       */
      constructor(point:yfiles.geometry.IPoint,orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,from:number,to:number,tag:Object,weight:number);
      /**
       * Initializes a new instance of the {@link yfiles.input.PointBasedSnapLine} class.
       * @param point The point to assign to {@link #point}.
       * @param orientation The orientation of the snap line. This is one of {@link #HORIZONTAL} or
       * {@link #VERTICAL}.
       * @param snapType The type of the line that describes how other items will snap to this line. This is one of
       * {@link #TOP}, {@link #BOTTOM}, {@link #LEFT},
       * {@link #RIGHT}, or {@link #CENTER}.
       * @param resourceKey A resource key which determines the visual representation of this snap line.
       * @param coordinates The coordinates of the center point of the snap line.
       * @param tag A tag that is associated with this snap line - see {@link #tag} for a typical use of this value.
       * @param weight The weight (importance) of this snap line. If more than one snap line is snapped to, the one with the greater weight
       * will be used.
       */
      constructor(point:yfiles.geometry.IPoint,orientation:yfiles.input.SnapLineOrientation,snapType:yfiles.input.SnapLineSnapTypes,resourceKey:string,coordinates:yfiles.geometry.Point,tag:Object,weight:number);
      /**
       * Gets the point that is associated with this snap line.
       * <p>
       * Usually this is the instance that induced this snap line.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      point:yfiles.geometry.IPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized {@link yfiles.input.SnapLine} that is parallel to an edge segment and stores that edge.
     * <p>
     * In addition to the line itself, this class has three special nape locations the represent the source, center and end
     * location of the related edge segment. All snap lines of this type have the {@link yfiles.input.SnapLine#snapType} {@link yfiles.input.SnapLineSnapTypes#CENTER} and
     * the {@link yfiles.input.SnapLine#resourceKey} {@link yfiles.input.SnapLine#SNAP_LINE_EDGE_SEGMENT_KEY}.
     * </p>
     * @class
     * @extends {yfiles.input.SnapLine}
     */
    export interface EdgeSegmentSnapLine extends yfiles.input.SnapLine{}
    export class EdgeSegmentSnapLine {
      /**
       * Initializes a new instance of the {@link yfiles.input.PointBasedSnapLine} class.
       * @param edge The edge to assign to {@link #edge}.
       * @param from The first location that delimits this snap line.
       * @param to The second location that delimits this snap line.
       * @param segmentSource The location that represents the source of the related segment on this line.
       * @param segmentTarget The location that represents the target of the related segment on this line.
       * @param distance The distance of this snap line from its related edge segment.
       * @param segmentIndex The segment index of the related edge segment.
       * @param tag A tag that is associated with this snap line.
       * @param weight The weight (importance) of this snap line.
       */
      constructor(edge:yfiles.graph.IEdge,from:yfiles.geometry.Point,to:yfiles.geometry.Point,segmentSource:yfiles.geometry.Point,segmentTarget:yfiles.geometry.Point,distance:number,segmentIndex:number,tag:Object,weight:number);
      /**
       * Initializes a new instance of the {@link yfiles.input.PointBasedSnapLine} class.
       * @param edge The edge to assign to {@link #edge}.
       * @param segmentSource The location that represents the source of the related segment on this line. In addition, this location is used as {@link #from}
       * location.
       * @param segmentTarget The location that represents the target of the related segment on this line. In addition, this location is used as {@link #to}
       * location.
       * @param distance The distance of this snap line from its related edge segment.
       * @param segmentIndex The segment index of the related edge segment.
       */
      constructor(edge:yfiles.graph.IEdge,segmentSource:yfiles.geometry.Point,segmentTarget:yfiles.geometry.Point,distance:number,segmentIndex:number);
      /**
       * Gets the edge that is associated with this snap line.
       * @type {yfiles.graph.IEdge}
       */
      edge:yfiles.graph.IEdge;
      /**
       * Gets the direction vector of this snap line.
       * @type {yfiles.geometry.Point}
       */
      direction:yfiles.geometry.Point;
      /**
       * Gets the distance of this snap line from its related edge segment.
       * @type {number}
       */
      distance:number;
      /**
       * Gets the segment index of the related edge segment.
       * @type {number}
       */
      segmentIndex:number;
      /**
       * Gets the location that represents the source of the related segment on this line.
       * @type {yfiles.geometry.Point}
       */
      segmentSource:yfiles.geometry.Point;
      /**
       * Gets the location that represents the target of the related segment on this line.
       * @type {yfiles.geometry.Point}
       */
      segmentTarget:yfiles.geometry.Point;
      /**
       * Gets and sets the location of the source marker of this snap line.
       * <p>
       * If this is <code>null</code>, the location of the {@link yfiles.input.EdgeSegmentSnapLine#segmentSource} is marked instead.
       * </p>
       * @type {yfiles.geometry.Point}
       */
      sourceMarker:yfiles.geometry.Point;
      /**
       * Gets and sets the location of the target marker of this snap line.
       * <p>
       * If this is <code>null</code>, the location of the {@link yfiles.input.EdgeSegmentSnapLine#segmentTarget} is marked instead.
       * </p>
       * @type {yfiles.geometry.Point}
       */
      targetMarker:yfiles.geometry.Point;
      static $class:yfiles.lang.Class;
    }
    export enum SnapLineOrientation{
      /**
       * Constant that describes a horizontal {@link yfiles.input.SnapLineOrientation orientation} of a snap line.
       */
      HORIZONTAL,
      /**
       * Constant that describes a vertical {@link yfiles.input.SnapLineOrientation orientation} of a snap line.
       */
      VERTICAL
    }
    export enum SnapLineSnapTypes{
      /**
       * Describes snap lines to which nothing will snap.
       */
      NONE,
      /**
       * Describes snap lines to which the top sides of rectangular elements will snap.
       */
      TOP,
      /**
       * Describes snap lines to which the bottom sides of rectangular elements will snap.
       */
      BOTTOM,
      /**
       * Describes snap lines to which the left sides of rectangular elements will snap.
       */
      LEFT,
      /**
       * Describes snap lines to which the right sides of rectangular elements will snap.
       */
      RIGHT,
      /**
       * Describes snap lines to which the center of elements will snap.
       */
      CENTER,
      /**
       * Describes vertical snap lines
       */
      VERTICAL,
      /**
       * Describes horizontal snap lines
       */
      HORIZONTAL,
      /**
       * Describes snap lines to which all elements will snap at all sides
       */
      ALL
    }
    /**
     * A simple default implementation of an {@link yfiles.input.IPortCandidate}.
     * <p>
     * Subclasses need to override {@link yfiles.input.DefaultPortCandidate#createInstance} if they don't {@link yfiles.input.DefaultPortCandidate#setInstance set} a port instance.
     * </p>
     * @class
     * @implements {yfiles.input.IPortCandidate}
     */
    export interface DefaultPortCandidate extends Object,yfiles.input.IPortCandidate{}
    export class DefaultPortCandidate {
      /**
       * Creates a candidate for the given <code>owner</code> that uses the graph's defaults for the port's style and a default parameter for
       * the given <code>model</code>.
       * <p>
       * The {@link yfiles.input.DefaultPortCandidate#locationParameter} is set to the parameter that best matches the {@link yfiles.geometry.Point#ORIGIN origin} for the given <code>model</code>.
       * </p>
       * @param owner The owner.
       * @param model The model.
       */
      constructor(owner:yfiles.graph.IPortOwner,model:yfiles.graph.IPortLocationModel);
      /**
       * Creates a candidate for the given <code>owner</code> that uses the graph's defaults for the port's style and the given
       * <code>locationParameter</code>.
       * <p>
       * If no <code>locationParameter</code> is given, the used candidate has a fixed location at {@link yfiles.geometry.Point#ORIGIN} that does not
       * depend on the <code>owner</code>.
       * </p>
       * @param owner The owner.
       * @param [locationParameter=null] The location.
       * @param [validity=yfiles.input.PortCandidateValidity.VALID] The validity of the port candidate.
       */
      constructor(owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,validity?:yfiles.input.PortCandidateValidity);
      /**
       * Creates a candidate for the given <code>owner</code> that uses the graph's defaults for the port's style and the given
       * <code>locationParameter</code>.
       * <p>
       * If no <code>locationParameter</code> is given, the used candidate has a fixed location at {@link yfiles.geometry.Point#ORIGIN} that does not
       * depend on the <code>owner</code>.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.owner The owner.
       * @param [options.locationParameter=null] The location.
       * @param [options.validity=yfiles.input.PortCandidateValidity.VALID] The validity of the port candidate.
       */
      constructor(options:{owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,validity?:yfiles.input.PortCandidateValidity});
      /**
       * Creates a candidate using the given port as the template.
       * @param port The port to use.
       */
      constructor(port:yfiles.graph.IPort);
      /**
       * Actually creates the {@link yfiles.graph.IPort} instance.
       * <p>
       * This method is called in response to a call to {@link yfiles.input.DefaultPortCandidate#createPort} on this instance and the
       * instances created by {@link yfiles.input.DefaultPortCandidate#getPortCandidateAt}.
       * </p>
       * @param context The input mode context.
       * @param graph The graph to add the port to.
       * @param locationParameter The parameter to use.
       * @param style The style to use.
       * @param tag The tag to use.
       * @returns The newly created port.
       * @protected
       */
      createInstance(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,locationParameter:yfiles.graph.IPortLocationModelParameter,style:yfiles.styles.IPortStyle,tag:Object):yfiles.graph.IPort;
      /**
       * This implementation will throw a {@link yfiles.lang.Exception} if the validity is {@link yfiles.input.PortCandidateValidity#DYNAMIC}.
       * <p>
       * This implementation will delegate to the {@link yfiles.input.DefaultPortCandidate#createInstance} method.
       * </p>
       * @param context
       * @returns 
       */
      createPort(context:yfiles.input.IInputModeContext):yfiles.graph.IPort;
      /**
       * Returns a candidate that is derived from this instance that best fits the provided <code>location</code>.
       * <p>
       * The returned implementation will finally call {@link yfiles.input.DefaultPortCandidate#createInstance} using the newly
       * calculated {@link yfiles.input.DefaultPortCandidate#locationParameter}.
       * </p>
       * @param context The input mode context for which a concrete candidate is queried.
       * @param location The location for which a candidate should be returned.
       * @returns A {@link } implementation whose {@link #validity} is guaranteed to be non-{@link #DYNAMIC}.
       * @see yfiles.input.DefaultPortCandidate#validity
       */
      getPortCandidateAt(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Sets the instance that will be returned by {@link yfiles.input.DefaultPortCandidate#port} and {@link yfiles.input.DefaultPortCandidate#createPort}.
       * @param port A port that already exists in the graph or <code>null</code>.
       */
      setInstance(port:yfiles.graph.IPort):void;
      /**
       * Determines whether this candidate is a valid one depending on the context it has been obtained from.
       * @default yfiles.input.PortCandidateValidity.VALID
       * @type {yfiles.input.PortCandidateValidity}
       */
      validity:yfiles.input.PortCandidateValidity;
      /**
       * Gets or set the tag for this candidate.
       * <p>
       * The interpretation of this value is implementation dependent and may be used for visual feedback or interactive
       * behavior.
       * </p>
       * @default null
       * @type {Object}
       */
      candidateTag:Object;
      /**
       * Gets or sets the port tag to create the port with.
       * @default null
       * @type {Object}
       */
      portTag:Object;
      /**
       * Gets or sets the model parameter that will be used for the {@link yfiles.graph.IPort#locationParameter} if this candidate is chosen.
       * <p>
       * The parameter which may not be <code>null</code>.
       * </p>
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * The model that will be used by this instance to determine the new {@link yfiles.input.DefaultPortCandidate#locationParameter} if the {@link yfiles.input.DefaultPortCandidate#getPortCandidateAt}
       * is called and the {@link yfiles.input.DefaultPortCandidate#validity} is set to {@link yfiles.input.PortCandidateValidity#DYNAMIC}.
       * @default null
       * @type {yfiles.graph.IPortLocationModel}
       */
      model:yfiles.graph.IPortLocationModel;
      /**
       * Gets or sets the style to use for the newly created port.
       * @default null
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * Gets or sets the owner of the port to be created.
       * @type {yfiles.graph.IPortOwner}
       */
      owner:yfiles.graph.IPortOwner;
      /**
       * Returns the instance provided to the constructor or to the {@link yfiles.input.DefaultPortCandidate#setInstance setter}.
       * @see yfiles.input.DefaultPortCandidate#createPort
       * @type {yfiles.graph.IPort}
       */
      port:yfiles.graph.IPort;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.input.IPositionHandler} implementation used in the context of {@link yfiles.graph.IGraph}s where there is a
     * {@link yfiles.graph.IGraph#getChildren active hierarchy}.
     * <p>
     * This implementation delegates through to a simpler implementation that only deals with the geometric relocation of the
     * node. In addition to that it finds the {@link yfiles.input.IReparentNodeHandler} from the {@link yfiles.input.IInputModeContext} to
     * detect {@link yfiles.input.IReparentNodeHandler#isReparentGesture a reparenting gesture} by the user. If such a gesture is recognized, the node can be moved freely until a {@link yfiles.input.IReparentNodeHandler#isValidParent valid new parent node}
     * is found. This implementation then temporarily sets that node as a new visual parent and highlight the node. When the
     * gesture is finished, the {@link yfiles.input.IReparentNodeHandler#reparent} method is called to perform the actual reparenting.
     * </p>
     * <p>
     * In order to customize this default behavior, subclasses of this type can be {@link yfiles.graph.NodeDecorator#positionHandlerDecorator decorated} to the
     * {@link yfiles.graph.INode}s in the graph.
     * </p>
     * @class
     * @implements {yfiles.input.IPositionHandler}
     */
    export interface GroupingNodePositionHandler extends Object,yfiles.input.IPositionHandler{}
    export class GroupingNodePositionHandler {
      /**
       * Initializes a new instance of the {@link yfiles.input.GroupingNodePositionHandler} class.
       * <p>
       * This implementation uses the provided <code>wrappedHandler</code> to delegate the handling of the node's {@link yfiles.graph.INode#layout} changes.
       * </p>
       * @param node The node to reposition.
       * @param [wrappedHandler=null] The positionhandler to
       */
      constructor(node:yfiles.graph.INode,wrappedHandler?:yfiles.input.IPositionHandler);
      /**
       * 
       * @param inputModeContext
       * @param originalLocation
       */
      cancelDrag(inputModeContext:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * 
       * @param inputModeContext
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(inputModeContext:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * 
       * @param inputModeContext
       * @param originalLocation
       * @param newLocation
       */
      handleMove(inputModeContext:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * 
       * @param inputModeContext
       */
      initializeDrag(inputModeContext:yfiles.input.IInputModeContext):void;
      /**
       * Sets a temporary visual parent for the node during reparenting.
       * <p>
       * This is used to show the grouping hierarchy level a node would end in after finishing the reparenting gesture. Override
       * this method with an empty implementation to not change the z-order of the dragged node during the gesture.
       * </p>
       * @param inputModeContext The current {@link }.
       * @param node The node being reparented.
       * @param parent The temporary parent for the node.
       */
      setCurrentParent(inputModeContext:yfiles.input.IInputModeContext,node:yfiles.graph.INode,parent:yfiles.graph.INode):void;
      /**
       * Delegates to the wrapped {@link yfiles.input.IPositionHandler}'s {@link yfiles.input.IDragHandler#location location}.
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * Callback interface for instances that can create bends for a given edge and graph.
     * <p>
     * Implementations of this interface may be found in the {@link yfiles.graph.ILookup#lookup} of {@link yfiles.graph.IEdge} instances. This
     * interface is used by {@link yfiles.input.GraphEditorInputMode} to create new bends for an edge as soon as the user performs the
     * bend creation gesture.
     * </p>
     * @see yfiles.input.GraphEditorInputMode#onCreateBendInputModeBendCreated
     * @interface
     */
    export interface IBendCreator extends Object{
      /**
       * Creates a bend at the given graph for the given edge at the position supplied.
       * @param context The context for which the bend should be created.
       * @param graph The graph, the edge belongs to.
       * @param edge The edge.
       * @param location The preferred coordinates of the bend.
       * @returns The index of the bend in the edges' {@link #bends} or <code>-1</code> if no bend has been created.
       * @abstract
       */
      createBend(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,location:yfiles.geometry.Point):number;
    }
    var IBendCreator:{
      /**
       * A static instance that does not create any bends.
       * @const
       * @static
       * @type {yfiles.input.IBendCreator}
       */
      NO_BENDS?:yfiles.input.IBendCreator;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that can be used to find bends at specific locations in the canvas.
     * <p>
     * This interface is usually provided through the {@link yfiles.graph.ILookup#lookup} of {@link yfiles.graph.IEdge} implementations.
     * </p>
     * @interface
     */
    export interface IBendSelectionTester extends Object{
      /**
       * Returns the bends within the given rectangle.
       * @param context The input mode context to use for querying the position.
       * @param rectangle A rectangle in world coordinates.
       * @returns The bends inside of <code>rectangle</code>.
       * @abstract
       */
      getBendsInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.graph.IBend>;
      /**
       * Returns the bend at the given world coordinate position.
       * @param context The input mode context to use for querying the position.
       * @param location A location in world coordinates.
       * @returns The bend at the position or <code>null</code> if there is no bend.
       * @abstract
       */
      getHitBend(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):yfiles.graph.IBend;
    }
    var IBendSelectionTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface that checks for containment of an {@link yfiles.graph.IBend} in 2D coordinate space.
     * <p>
     * This interface is used to determine which bends of edges that connect child nodes are moved when the parent node is
     * moved. Therefore, special care has to be taken if you want to provide this interface in the <b>node lookup</b> for other purposes, as this
     * can also affect implicit bend movement.
     * </p>
     * @interface
     */
    export interface IContainsBendTester extends Object{
      /**
       * Determines whether the provided <code>bend</code> is deemed to lie within the shape this test is implemented for.
       * <p>
       * This method is used to determine which bends of edges that connect child nodes are moved when the parent node is moved.
       * If an implementation of {@link yfiles.input.IContainsBendTester} is present in the node's {@link yfiles.graph.ILookup#lookup}, this
       * method is called with the bend and the bend will be moved together with the parent node iff <code>true</code> is returned. If no
       * implemention is provided, only bends that lie inside the bounds of the parent node are moved.
       * </p>
       * @param context The input mode context to use.
       * @param bend The bend to test
       * @returns <code>true</code> iff the bend is contained within the shape.
       * @abstract
       */
      contains(context:yfiles.input.IInputModeContext,bend:yfiles.graph.IBend):boolean;
    }
    var IContainsBendTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is used by the {@link yfiles.input.GraphSnapContext} for {@link yfiles.graph.INode nodes} to collect a number of
     * {@link yfiles.input.SnapResult}s during the move/edit operation.
     * <p>
     * This interface is queried from the {@link yfiles.graph.ILookup} of the
     * {@link yfiles.graph.INode}s that are being moved during the edit.
     * </p>
     * @see yfiles.input.GraphSnapContext
     * @see yfiles.graph.NodeDecorator#nodeSnapResultProviderDecorator
     * @interface
     */
    export interface INodeSnapResultProvider extends Object{
      /**
       * Called when a node is {@link yfiles.input.IDragHandler#handleMove dragged} to add {@link yfiles.input.SnapResult}s for
       * {@link yfiles.input.OrthogonalSnapLine}s to which this node can potentially snap.
       * @param context The snap context which manages the snap lines and the settings.
       * @param evt The event argument to obtain the necessary information from and {@link #addSnapResult add results to}.
       * @param suggestedLayout The {@link #layout layout} of the node if the node would not snap.
       * @param node The node that is being moved.
       * @abstract
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLayout:yfiles.geometry.Rect,node:yfiles.graph.INode):void;
    }
    var INodeSnapResultProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for classes that help during snapping of parts of edges.
     * <p>
     * An implementation of this class is obtained from the lookup of the {@link yfiles.graph.IEdge}s which are being edited during an
     * operation where {@link yfiles.input.GraphSnapContext graph snapping} is involved.
     * </p>
     * @see yfiles.graph.EdgeDecorator#edgeSnapResultProviderDecorator
     * @interface
     */
    export interface IEdgeSnapResultProvider extends Object{
      /**
       * Called to during the snapping operation whenever {@link yfiles.input.SnapContext#addCollectSnapResultsListener snap results are collected}.
       * <p>
       * This method will be called for each collection of the results.
       * </p>
       * @param context The context that is using this instance.
       * @param evt The event argument to obtain the necessary information from and {@link #addSnapResult add results to}.
       * @param movementInfos The movement infos that describe the <code>edge</code>.
       * @param edge The edge for which the {@link }s should be {@link #addSnapResult collected}.
       * @abstract
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,movementInfos:yfiles.collections.IListEnumerable<yfiles.input.MovementInfo>,edge:yfiles.graph.IEdge):void;
    }
    var IEdgeSnapResultProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is used by the {@link yfiles.input.GraphSnapContext} for {@link yfiles.graph.IBend bends} to collect a number of
     * {@link yfiles.input.SnapResult}s during the move/edit operation.
     * <p>
     * This interface is queried from the {@link yfiles.graph.ILookup} of the
     * {@link yfiles.graph.IBend}s that are being moved during the edit.
     * </p>
     * @see yfiles.input.GraphSnapContext
     * @see yfiles.graph.BendDecorator#bendSnapResultProviderDecorator
     * @interface
     */
    export interface IBendSnapResultProvider extends Object{
      /**
       * Called when a node is {@link yfiles.input.IDragHandler#handleMove dragged} to add {@link yfiles.input.SnapResult}s for
       * {@link yfiles.input.OrthogonalSnapLine}s to which this bend can potentially snap.
       * @param context The snap context which manages the snap lines and the settings.
       * @param evt The event argument to obtain the necessary information from and {@link #addSnapResult add results to}.
       * @param suggestedLocation The {@link #location location} of the bend if the bend would not snap.
       * @param bend The bend that is being moved.
       * @abstract
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLocation:yfiles.geometry.Point,bend:yfiles.graph.IBend):void;
    }
    var IBendSnapResultProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is used by the {@link yfiles.input.GraphSnapContext} for {@link yfiles.graph.IPort port} to collect a number of
     * {@link yfiles.input.SnapResult}s during the move/edit operation.
     * <p>
     * This interface is queried from the {@link yfiles.graph.ILookup} of the
     * {@link yfiles.graph.IPort}s that are being moved during the edit.
     * </p>
     * @see yfiles.input.GraphSnapContext
     * @see yfiles.graph.PortDecorator#portSnapResultProviderDecorator
     * @interface
     */
    export interface IPortSnapResultProvider extends Object{
      /**
       * Called when a node is {@link yfiles.input.IDragHandler#handleMove dragged} to add {@link yfiles.input.SnapResult}s for
       * {@link yfiles.input.OrthogonalSnapLine}s to which this bend can potentially snap.
       * @param context The snap context which manages the snap lines and the settings.
       * @param evt The event argument to obtain the necessary information from and {@link #addSnapResult add results to}.
       * @param suggestedLocation The {@link #location location} of the bend if the bend would not snap.
       * @param port The port that is being moved
       * @abstract
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLocation:yfiles.geometry.Point,port:yfiles.graph.IPort):void;
    }
    var IPortSnapResultProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for an implementation which evaluates a list of given {@link yfiles.input.SnapLine}s for a node which is resized and
     * adds a set of {@link yfiles.input.SnapResult}s for sizes to which this node can potentially snap.
     * @interface
     */
    export interface INodeReshapeSnapResultProvider extends Object{
      /**
       * Called when a node's handle is {@link yfiles.input.IDragHandler#handleMove dragged} to add {@link yfiles.input.SnapResult}s for
       * {@link yfiles.input.OrthogonalSnapLine}s or sizes to which this node can potentially snap during resizing.
       * @param context The snap context which manages the snap lines and the settings.
       * @param evt The event argument to obtain the context from and add the results to.
       * @param node The node that is being reshaped.
       * @param reshapeContext Carries information about the reshape operation.
       * @abstract
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,reshapeContext:yfiles.input.ReshapeRectangleContext):void;
    }
    var INodeReshapeSnapResultProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Helper class that carries information about a reshape operation.
     * <p>
     * This class is mainly intended for use by {@link yfiles.input.INodeReshapeSnapResultProvider#collectSnapResults}.
     * </p>
     * @class
     * @final
     */
    export interface ReshapeRectangleContext extends Object{}
    export class ReshapeRectangleContext {
      /**
       * Initializes a new instance of the {@link yfiles.input.ReshapeRectangleContext} class.
       * @param initialBounds The initial bounds of the node.
       * @param minimumSize The minimum size of the node.
       * @param maximumSize The maximum size of the node.
       * @param minimumEnclosedArea The minimum area the node's layout needs to encompass.
       * @param reshapePosition The position of the reshape handle.
       * @param topLeftChangeFactor The factor by which the top left corner of the node layout changes when the handle is dragged.
       * @param bottomRightChangeFactor The factor by which the bottom right corner of the node layout changes when the handle is dragged.
       * @param sizeChangeFactor The factor by which the size of the node layout changes when the handle is dragged.
       */
      constructor(initialBounds:yfiles.geometry.Rect,minimumSize:yfiles.geometry.ISize,maximumSize:yfiles.geometry.ISize,minimumEnclosedArea:yfiles.geometry.IRectangle,reshapePosition:yfiles.input.HandlePositions,topLeftChangeFactor:yfiles.geometry.Point,bottomRightChangeFactor:yfiles.geometry.Point,sizeChangeFactor:yfiles.geometry.Size);
      /**
       * Gets the initial bounds of the node.
       * @type {yfiles.geometry.Rect}
       */
      initialBounds:yfiles.geometry.Rect;
      /**
       * Gets the minimum size of the node.
       * @type {yfiles.geometry.ISize}
       */
      minimumSize:yfiles.geometry.ISize;
      /**
       * Gets or sets the maximum size of the node.
       * @type {yfiles.geometry.ISize}
       */
      maximumSize:yfiles.geometry.ISize;
      /**
       * Gets minimum area the node's layout needs to encompass
       * @type {yfiles.geometry.IRectangle}
       */
      minimumEnclosedArea:yfiles.geometry.IRectangle;
      /**
       * Gets the position of the reshape handle.
       * @type {yfiles.input.HandlePositions}
       */
      reshapePosition:yfiles.input.HandlePositions;
      /**
       * Gets the factor by which the top left corner of the node layout changes when the handle is dragged.
       * @type {yfiles.geometry.Point}
       */
      topLeftChangeFactor:yfiles.geometry.Point;
      /**
       * Gets the factor by which the bottom right corner of the node layout changes when the handle is dragged.
       * @type {yfiles.geometry.Point}
       */
      bottomRightChangeFactor:yfiles.geometry.Point;
      /**
       * Gets the factor by which the size of the node layout changes when the handle is dragged.
       * @type {yfiles.geometry.Size}
       */
      sizeChangeFactor:yfiles.geometry.Size;
      static $class:yfiles.lang.Class;
    }
    /**
     * An interface for possible {@link yfiles.graph.IPort} candidates used by {@link yfiles.input.IPortCandidateProvider} and the like.
     * <p>
     * Note that if an instance's {@link yfiles.input.IPortCandidate#validity} property is {@link yfiles.input.PortCandidateValidity#DYNAMIC} this instance {@link yfiles.input.IPortCandidate#getPortCandidateAt}
     * method serves as a factory for the actual candidates.
     * </p>
     * @see yfiles.input.IEdgeReconnectionPortCandidateProvider
     * @see yfiles.input.IPortCandidateProvider
     * @see yfiles.input.DefaultPortCandidate
     * @interface
     */
    export interface IPortCandidate extends Object{
      /**
       * If the client decides to use this port candidate, this method will serve as a factory to create the instance.
       * <p>
       * If this candidate wraps an existing port, it may return that {@link yfiles.input.IPortCandidate#port instance}. If this instance's {@link yfiles.input.IPortCandidate#validity} is {@link yfiles.input.PortCandidateValidity#DYNAMIC}
       * this method may throw an {@link yfiles.lang.Exception}.
       * </p>
       * @param context The input mode context for which the port will be created. Implementations can depend on it providing an {@link }
       * instance in the lookup of the instance.
       * @returns The port instance to use if this candidate has been chosen. This method may not return <code>null</code>.
       * @throws {Stubs.Exceptions.NotSupportedError} If the {@link yfiles.input.IPortCandidate#validity} is {@link yfiles.input.PortCandidateValidity#DYNAMIC}.
       * @see yfiles.input.IInputModeContext#graph
       * @see yfiles.input.IPortCandidate#port
       * @see yfiles.input.IPortCandidate#getPortCandidateAt
       * @abstract
       */
      createPort(context:yfiles.input.IInputModeContext):yfiles.graph.IPort;
      /**
       * Returns a candidate that is derived from this instance that best fits the provided <code>location</code>.
       * <p>
       * This method needs to be called by clients if the {@link yfiles.input.IPortCandidate#validity} of this instance is {@link yfiles.input.PortCandidateValidity#DYNAMIC}. In
       * that case calling this method must return an instance whose validity is not {@link yfiles.input.PortCandidateValidity#DYNAMIC}.
       * </p>
       * @param context The input mode context for which a concrete candidate is queried.
       * @param location The location for which a candidate should be returned.
       * @returns A {@link } implementation whose {@link #validity} is guaranteed to be non-{@link #DYNAMIC}.
       * @see yfiles.input.IPortCandidate#validity
       * @abstract
       */
      getPortCandidateAt(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Possibly yields a user defined tag for this candidate.
       * <p>
       * The interpretation of this value is implementation dependent and may be used for visual feedback or interactive
       * behavior.
       * </p>
       * @abstract
       * @type {Object}
       */
      candidateTag:Object;
      /**
       * Determines whether this instance is a valid port candidate.
       * <p>
       * Implementations of {@link yfiles.input.IPortCandidateProvider} may return sets of port candidates where some of them are marked
       * as invalid e.g. in order to provide visual feedback. If the value is {@link yfiles.input.PortCandidateValidity#DYNAMIC}, method
       * {@link yfiles.input.IPortCandidate#getPortCandidateAt} needs to be used to determine a concrete candidate instance.
       * </p>
       * @see yfiles.input.IPortCandidate#getPortCandidateAt
       * @abstract
       * @type {yfiles.input.PortCandidateValidity}
       */
      validity:yfiles.input.PortCandidateValidity;
      /**
       * Returns the model parameter that will be used for the {@link yfiles.graph.IPort#locationParameter} if this candidate is chosen.
       * <p>
       * Note that the value of this property is undefined if this instance's {@link yfiles.input.IPortCandidate#validity} is {@link yfiles.input.PortCandidateValidity#DYNAMIC}. In
       * this case, the parameter needs to be obtained via the {@link yfiles.input.IPortCandidate#getPortCandidateAt} instance.
       * </p>
       * @abstract
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Returns the possible owner of the port to be created for this candidate.
       * @abstract
       * @type {yfiles.graph.IPortOwner}
       */
      owner:yfiles.graph.IPortOwner;
      /**
       * Gets the port instance that is already live in the graph if this candidate wraps an existing port or <code>null</code> otherwise.
       * @see yfiles.input.IPortCandidate#createPort
       * @abstract
       * @type {yfiles.graph.IPort}
       */
      port:yfiles.graph.IPort;
    }
    var IPortCandidate:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum PortCandidateValidity{
      /**
       * The candidate is valid and can be used to connect an edge to.
       */
      VALID,
      /**
       * The candidate should not be used to connect an edge to.
       */
      INVALID,
      /**
       * The candidate cannot be used to create a port, instead {@link yfiles.input.IPortCandidate#getPortCandidateAt} needs to be called
       * to obtain a {@link yfiles.input.IPortCandidate} that is either {@link yfiles.input.PortCandidateValidity#VALID} or
       * {@link yfiles.input.PortCandidateValidity#INVALID}.
       */
      DYNAMIC
    }
    /**
     * This interface is used to provide {@link yfiles.input.IPortCandidate}s for a specific {@link yfiles.graph.IPortOwner}.
     * <p>
     * It is used in the context of the items that will later become the owner ({@link yfiles.graph.IPortOwner}) of the ports obtained
     * from the candidates. They are mainly used during edge creation where there is no real edge yet (only the dummy edge) and
     * thus they need to use the opposite {@link yfiles.input.IPortCandidate} to understand the context.
     * </p>
     * <p>
     * Implementations of this interface may be queried using the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.IPortOwner}
     * implementations like {@link yfiles.graph.INode}.
     * </p>
     * <p>
     * Note that this interface is similar to the interface {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} in the sense
     * that they will both provide {@link yfiles.input.IPortCandidate} instances in a certain query context. They are used in different
     * query contexts, though. The two interfaces can come together in concrete implementations of
     * {@link yfiles.input.IEdgeReconnectionPortCandidateProvider}: an edge could potentially be connected to ports at any given {@link yfiles.graph.IPortOwner}
     * and thus the {@link yfiles.input.IPortCandidateProvider} interface can be queried from the respective {@link yfiles.graph.IPortOwner}
     * that the edge will be reconnected to.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: PortCandidateProvider, demo.yfiles.input.portcandidateprovider</li>
     * </ul>
     * @see yfiles.graph.NodeDecorator#portCandidateProviderDecorator
     * @interface
     */
    export interface IPortCandidateProvider extends Object{
      /**
       * Returns all port candidates that apply for the provided opposite port candidate.
       * <p>
       * The enumerable may both contain {@link yfiles.input.PortCandidateValidity valid, invalid, and dynamic} port candidates.
       * </p>
       * @param target The opposite port candidate.
       * @param context The context for which the candidates should be provided.
       * @returns An enumerable over all port candidates that are associated with the given opposite port.
       * @abstract
       */
      getSourcePortCandidates(context:yfiles.input.IInputModeContext,target:yfiles.input.IPortCandidate):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Returns all source port candidates that belong to the context of this provider.
       * <p>
       * The enumerable may both contain {@link yfiles.input.PortCandidateValidity valid, invalid, and dynamic} port candidates.
       * </p>
       * @param context The context for which the candidates should be provided.
       * @returns An enumerable over all source port candidates that are associated with the current context.
       * @abstract
       */
      getSourcePortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Returns all port candidates that apply for the provided opposite port candidate.
       * <p>
       * The enumerable may both contain {@link yfiles.input.PortCandidateValidity valid, invalid, and dynamic} port candidates.
       * </p>
       * @param context The context for which the candidates should be provided.
       * @param source The opposite port candidate.
       * @returns An enumerable over all port candidates that are associated with the given opposite port.
       * @abstract
       */
      getTargetPortCandidates(context:yfiles.input.IInputModeContext,source:yfiles.input.IPortCandidate):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Returns all target port candidates that belong to the context of this provider.
       * <p>
       * The enumerable may both contain {@link yfiles.input.PortCandidateValidity valid, invalid, and dynamic} port candidates.
       * </p>
       * @param context The context for which the candidates should be provided.
       * @returns An enumerable over all target port candidates that are associated with the current context.
       * @abstract
       */
      getTargetPortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
    }
    var IPortCandidateProvider:{
      /**
       * Creates a generic composite implementation for the {@link yfiles.input.IPortCandidateProvider} interface.
       * @param providers The providers to create the composite from.
       * @returns 
       * @static
       */
      combine?(...providers:yfiles.input.IPortCandidateProvider[]):yfiles.input.IPortCandidateProvider;
      /**
       * Creates a generic composite implementation for the {@link yfiles.input.IPortCandidateProvider} interface.
       * @param providers The providers to create the composite from.
       * @returns 
       * @static
       */
      combine?(providers:yfiles.collections.IEnumerable<yfiles.input.IPortCandidateProvider>):yfiles.input.IPortCandidateProvider;
      /**
       * Creates an implementation of an {@link yfiles.input.IPortCandidateProvider} that provides only the given port candidates.
       * @param candidates The port candidates the provider should provide.
       * @returns A new {@link }.
       * @static
       */
      fromCandidates?(...candidates:yfiles.input.IPortCandidate[]):yfiles.input.PortCandidateProviderBase;
      /**
       * Creates an implementation of an {@link yfiles.input.IPortCandidateProvider} that provides only the given port candidates.
       * @param candidates The port candidates the provider should provide.
       * @returns A new {@link }.
       * @static
       */
      fromCandidates?(candidates:yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>):yfiles.input.PortCandidateProviderBase;
      /**
       * Creates an implementation of the {@link yfiles.input.IPortCandidateProvider} interface that returns the ports that exist in the
       * given {@link yfiles.graph.IPortOwner#ports}'s {@link yfiles.graph.IPortOwner} collection.
       * @param owner The owner of the port collection.
       * @returns 
       * @static
       */
      fromExistingPorts?(owner:yfiles.graph.IPortOwner):yfiles.input.IPortCandidateProvider;
      /**
       * Creates a trivial implementation of an {@link yfiles.input.IPortCandidateProvider} that always returns exactly one candidate
       * that is centered at the node's {@link yfiles.graph.INode#layout}.
       * @param node The node to get the layout's center from.
       * @returns 
       * @static
       */
      fromNodeCenter?(node:yfiles.graph.INode):yfiles.input.PortCandidateProviderBase;
      /**
       * Creates a simple implementation of an {@link yfiles.input.IPortCandidateProvider} that always returns a candidate that uses the
       * default {@link yfiles.graph.IPortDefaults#locationParameter} for the corresponding {@link yfiles.graph.IPortDefaults}.
       * @param owner
       * @returns 
       * @static
       */
      fromPortDefaults?(owner:yfiles.graph.IPortOwner):yfiles.input.IPortCandidateProvider;
      /**
       * Creates an implementation of {@link yfiles.input.IPortCandidateProvider} that works for {@link yfiles.graph.IPortOwner} implementations
       * that have {@link yfiles.styles.IShapeGeometry} instances in their lookup.
       * @param owner The owner to receive the shape geometry from
       * @param minimumSegmentLength the minimum length a segment needs to have in order to be used to add port candidates.
       * @param ratios A number of double values that are interpreted as ratio values between 0.0 and 1.0. If none are given, the provider
       * inserts a port candidate at the center of each straight line segment.
       * @param addExistingPortsEnabled determines whether {@link #addExistingPorts existing ports should be added to the list of ports}.
       * @returns 
       * @static
       */
      fromShapeGeometry?(owner:yfiles.graph.IPortOwner,addExistingPortsEnabled:boolean,minimumSegmentLength:number,...ratios:number[]):yfiles.input.PortCandidateProviderBase;
      /**
       * Creates an implementation of {@link yfiles.input.IPortCandidateProvider} that works for {@link yfiles.graph.IPortOwner} implementations
       * that have {@link yfiles.styles.IShapeGeometry} instances in their lookup.
       * @param owner The owner to receive the shape geometry from
       * @param ratios A number of double values that are interpreted as ratio values between 0.0 and 1.0. If none are given, the provider
       * inserts a port candidate at the center of each straight line segment.
       * @returns 
       * @static
       */
      fromShapeGeometry?(owner:yfiles.graph.IPortOwner,...ratios:number[]):yfiles.input.PortCandidateProviderBase;
      /**
       * Creates a simple implementation of an {@link yfiles.input.IPortCandidateProvider} that returns unoccupied ports at a given
       * entity.
       * @param owner The owner to query the ports from.
       * @returns 
       * @static
       */
      fromUnoccupiedPorts?(owner:yfiles.graph.IPortOwner):yfiles.input.IPortCandidateProvider;
      /**
       * A generic implementation of the {@link yfiles.input.IPortCandidateProvider} interface that provides no candidates.
       * @const
       * @static
       * @type {yfiles.input.IPortCandidateProvider}
       */
      NO_CANDIDATES?:yfiles.input.IPortCandidateProvider;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface can be used to query {@link yfiles.input.IPortCandidate}s for a specific existing {@link yfiles.graph.IEdge}
     * <p>
     * As the name implies this is only used in the context of reconnecting existing edges. It is queried for existing edges to
     * find out about possible alternative port candidates that given edge could connect to. It is used during gestures where
     * an edge is reconnected to another port(candidate), i.e. when the source or target port handle of an edge is moved and
     * the source or target port properties of the edge get set to the new {@link yfiles.input.IPortCandidate}s.
     * </p>
     * <p>
     * Implementations of this interface can be queried using the {@link yfiles.graph.ILookup#lookup} method of an {@link yfiles.graph.IEdge}
     * implementation. This interface provides alternative candidates for the {@link yfiles.graph.IEdge#sourcePort} or {@link yfiles.graph.IEdge#targetPort} of an edge.
     * </p>
     * <p>
     * Note that this interface is similar to the interface {@link yfiles.input.IPortCandidateProvider} in the sense that they will
     * both provide {@link yfiles.input.IPortCandidate} instances in a certain query context. They are used in different query
     * contexts, though. The two interfaces can come together in concrete implementations of
     * {@link yfiles.input.IEdgeReconnectionPortCandidateProvider}: an edge could potentially be connected to ports at any given {@link yfiles.graph.IPortOwner}
     * and thus the {@link yfiles.input.IPortCandidateProvider} interface can be queried from the respective {@link yfiles.graph.IPortOwner}
     * that the edge will be reconnected to.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: EdgePortCandidateProvider, demo.yfiles.input.edgeportcandidateprovider</li>
     * </ul>
     * @interface
     */
    export interface IEdgeReconnectionPortCandidateProvider extends Object{
      /**
       * Returns all source port candidates that may be used for the edge.
       * <p>
       * The enumerable may both contain {@link yfiles.input.PortCandidateValidity valid, invalid, and dynamic} port candidates.
       * </p>
       * @param context The context for which the candidates should be provided.
       * @returns An enumerable over all source port candidates that are associated with the current context edge.
       * @abstract
       */
      getSourcePortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Returns all source port candidates that may be used for the edge.
       * <p>
       * The enumerable may both contain {@link yfiles.input.PortCandidateValidity valid, invalid, and dynamic} port candidates.
       * </p>
       * @param context The context for which the candidates should be provided.
       * @returns An enumerable over all target port candidates that are associated with the current context edge.
       * @abstract
       */
      getTargetPortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
    }
    var IEdgeReconnectionPortCandidateProvider:{
      /**
       * Creates an implementation of the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} interface that returns the
       * candidates provided by the source and target port's owner's {@link yfiles.input.IPortCandidateProvider} implementations or the
       * existing edge's port, if no such provider is registered.
       * <p>
       * This instance uses the lookup mechanism of the nodes to query the providers.
       * </p>
       * @param edge
       * @returns 
       * @static
       */
      fromSourceAndTarget?(edge:yfiles.graph.IEdge):yfiles.input.IEdgeReconnectionPortCandidateProvider;
      /**
       * Creates an implementation of the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} interface that returns a union of
       * all {@link yfiles.input.IPortCandidateProvider#getSourcePortCandidates source port candidates} or {@link yfiles.input.IPortCandidateProvider#getTargetPortCandidates target port candidates}, respectively, of nodes in the graph.
       * <p>
       * This implementation can be used to allow for {@link yfiles.input.PortRelocationHandle relocating an edge's port} to any available source or target port in the
       * graph.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.IEdgeReconnectionPortCandidateProvider}
       */
      ALL_NODE_CANDIDATES?:yfiles.input.IEdgeReconnectionPortCandidateProvider;
      /**
       * Creates an implementation of the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} interface that returns a union of
       * all {@link yfiles.input.IPortCandidateProvider#getSourcePortCandidates source port candidates} or {@link yfiles.input.IPortCandidateProvider#getTargetPortCandidates target port candidates}, respectively, of nodse and edges in the graph.
       * <p>
       * This implementation can be used to allow for {@link yfiles.input.PortRelocationHandle relocating an edge's port} to any available source or target port in the
       * graph.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.IEdgeReconnectionPortCandidateProvider}
       */
      ALL_NODE_AND_EDGE_CANDIDATES?:yfiles.input.IEdgeReconnectionPortCandidateProvider;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that can be used to find ports at specific locations in the canvas.
     * <p>
     * This interface is mostly provided through the {@link yfiles.graph.ILookup#lookup} of {@link yfiles.graph.IPortOwner} implementations
     * such as {@link yfiles.graph.INode}.
     * </p>
     * @interface
     */
    export interface IPortSelectionTester extends Object{
      /**
       * Returns the port at the given world coordinate position or <code>null</code> if there is no such port.
       * @param context The input mode context to use for querying the position.
       * @param location The coordinates of the position in the world coordinate system.
       * @returns The port at the position or <code>null</code>.
       * @abstract
       */
      getHitPort(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):yfiles.graph.IPort;
      /**
       * Returns the ports for the given marquee rectangle.
       * @param context The input mode context to use for querying the position.
       * @param rectangle The marquee selection box in the world coordinate system.
       * @returns An enumerable over the ports inside the marquee selection box.
       * @abstract
       */
      getPortsInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.graph.IPort>;
    }
    var IPortSelectionTester:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for implementations that yield insets for a given {@link yfiles.graph.INode}.
     * <p>
     * Implementations of this interface will be queried from an {@link yfiles.graph.INode}'s {@link yfiles.graph.ILookup#lookup} method. The
     * default implementation of a graph uses this instance by default to determine the insets of a group node.
     * </p>
     * @interface
     */
    export interface INodeInsetsProvider extends Object{
      /**
       * Returns the insets for the given <code>node</code>.
       * @param node The node to yield insets for.
       * @returns The insets for the node.
       * @abstract
       */
      getInsets(node:yfiles.graph.INode):yfiles.geometry.Insets;
    }
    var INodeInsetsProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Base interface for a command that can be executed.
     * <p>
     * Implementations of this interface are provided by the library. Use {@link yfiles.input.ICommand#createCommand} to conveniently
     * create your own implementation of this interface that can then be used with AddCommandBinding.
     * </p>
     * @see yfiles.input.ICommand#createCommand
     * @interface
     */
    export interface ICommand extends Object{
      /**
       * Determines whether this command be executed with respect to the given parameter and target.
       * @param [parameter=null] The parameter or <code>null</code>
       * @param [target=null] The target to execute the command on or <code>null</code>
       * @returns Whether {@link #execute} can be called with the given <code>parameter</code> and <code>target</code>.
       * @abstract
       */
      canExecute(parameter?:Object,target?:Object):boolean;
      /**
       * Determines whether this command be executed with respect to the given parameter and target.
       * @param {Object} options The parameters to pass.
       * @param [options.parameter=null] The parameter or <code>null</code>
       * @param [options.target=null] The target to execute the command on or <code>null</code>
       * @returns Whether {@link #execute} can be called with the given <code>parameter</code> and <code>target</code>.
       * @abstract
       */
      canExecute(options:{parameter?:Object,target?:Object}):boolean;
      /**
       * Executes this command with respect to the given parameter and target.
       * @param [parameter=null] The parameter or <code>null</code>
       * @param [target=null] The target to execute the command on or <code>null</code>
       * @abstract
       */
      execute(parameter?:Object,target?:Object):void;
      /**
       * Executes this command with respect to the given parameter and target.
       * @param {Object} options The parameters to pass.
       * @param [options.parameter=null] The parameter or <code>null</code>
       * @param [options.target=null] The target to execute the command on or <code>null</code>
       * @abstract
       */
      execute(options:{parameter?:Object,target?:Object}):void;
      /**
       * Adds the given listener for the <code>CanExecuteChanged</code> event that occurs when the result of a call to {@link yfiles.input.ICommand#canExecute}
       * may yield a different result.
       * @param listener The listener to add.
       * @see yfiles.input.ICommand#removeCanExecuteChangedListener
       * @abstract
       */
      addCanExecuteChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CanExecuteChanged</code> event that occurs when the result of a call to {@link yfiles.input.ICommand#canExecute}
       * may yield a different result.
       * @param listener The listener to remove.
       * @see yfiles.input.ICommand#addCanExecuteChangedListener
       * @abstract
       */
      removeCanExecuteChangedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
    }
    var ICommand:{
      /**
       * Creates a new {@link yfiles.input.ICommand} instance that does nothing.
       * <p>
       * The return value serves as a token and does not in itself contain any command handling logic. Use to associate command
       * handlers with the instance returned.
       * </p>
       * @param [name=null] The optional name for the command.
       * @returns A new {@link } instance.
       * @static
       */
      createCommand?(name?:string):yfiles.input.ICommand;
      /**
       * Suggests a requery of the {@link Root.UO#canExecute executability} of all routed commands that are known to the system to adjust the UI to properly
       * reflect the state.
       * <p>
       * This will enqueue a delayed query into the event queue.
       * </p>
       * @static
       */
      invalidateRequerySuggested?():void;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to select the item that is
       * specified by the <code>parameter</code>.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SELECT_ITEM?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to toggle the selection state of
       * the item that is specified by the <code>parameter</code>.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      TOGGLE_ITEM_SELECTION?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to deselect the item that is
       * specified by the <code>parameter</code>.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      DESELECT_ITEM?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.input.GraphEditorInputMode#editLabel edit the label} that is
       * specified by the <code>parameter</code>.
       * <p>
       * If no <code>parameter</code> is set, this will trigger the {@link yfiles.input.GraphEditorInputMode#onEditLabel} if {@link yfiles.input.GraphEditorInputMode}
       * is used.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EDIT_LABEL?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.input.GraphEditorInputMode#addLabel add another label} to the {@link yfiles.graph.ILabelOwner}
       * that is specified by the <code>parameter</code>.
       * <p>
       * If no <code>parameter</code> is set, this will trigger the {@link yfiles.input.GraphEditorInputMode#onAddLabel} if {@link yfiles.input.GraphEditorInputMode}
       * is used.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      ADD_LABEL?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to
       * {@link yfiles.input.GraphEditorInputMode#groupSelection group the currently selected items}.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      GROUP_SELECTION?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to
       * {@link yfiles.input.GraphEditorInputMode#adjustGroupNodeSize adjust the size of the selected group nodes}.
       * <p>
       * The default keyboard binding for this command is ctrl-shift-G.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      ADJUST_GROUP_NODE_SIZE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to
       * {@link yfiles.input.GraphEditorInputMode#ungroupSelection ungroup the currently selected items}.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      UNGROUP_SELECTION?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.view.ISelectionModel.<T>#clear clear the selection}.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      DESELECT_ALL?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.input.NavigationInputMode#expandGroup expand} group nodes.
       * <p>
       * If the parameter to this command is <code>null</code>, the currently selected group nodes are expanded. If the parameter is an {@link yfiles.graph.INode}
       * instance, that node will be expanded. If the parameter is an {@link yfiles.collections.IEnumerable.<T>} instance, then all group nodes in that enumeration are
       * expanded.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EXPAND_GROUP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to collapse group nodes.
       * <p>
       * If the parameter to this command is <code>null</code>, the currently selected group nodes are collapsed. If the parameter is an {@link yfiles.graph.INode}
       * instance, that group node will be collapsed. If the parameter is an {@link yfiles.collections.IEnumerable.<T>} instance, then all group nodes in that enumeration
       * are collapsed.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      COLLAPSE_GROUP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.input.NavigationInputMode#toggleExpansionState toggle} the collapsed/expanded
       * state of currently selected group nodes.
       * <p>
       * If the parameter to this command is <code>null</code>, the currently selected group nodes are toggled. If the parameter is an {@link yfiles.graph.INode}
       * instance, that group node will be toggled. If the parameter is an {@link yfiles.collections.IEnumerable.<T>} instance, then all group nodes in that enumeration
       * are toggled.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      TOGGLE_EXPANSION_STATE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.graph.IFoldingView#localRoot switch the local root} of the
       * current {@link yfiles.graph.IFoldingView} view to the given group node.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      ENTER_GROUP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.graph.IFoldingView#localRoot exit the current local root}
       * of the current {@link yfiles.graph.IFoldingView} view to the given parent view.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EXIT_GROUP?:yfiles.input.ICommand;
      /**
       * This is an {@link yfiles.input.ICommand} that {@link yfiles.input.CreateEdgeInputMode} instances can execute if they have been {@link yfiles.input.IInputMode#install installed}
       * in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * If this command gets executed, the <code>parameter</code> is inspected to see if it is set to either an {@link yfiles.input.IPortCandidate},
       * an {@link yfiles.graph.IPort}, or an {@link yfiles.graph.INode} that is an element in the currently edited graph instance. If so, method
       * {@link yfiles.input.CreateEdgeInputMode#doStartEdgeCreation} will be invoked and the user can finish the edge creation gesture.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      BEGIN_EDGE_CREATION?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that can be {@link yfiles.input.ICommand#execute executed} from within the {@link yfiles.view.GraphComponent} to {@link yfiles.graph.GraphClipboard#duplicate duplicate items} of the
       * current {@link yfiles.graph.IGraph}.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      DUPLICATE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that reverses the direction of edges.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} can execute this command if it is installed in a {@link yfiles.view.GraphComponent}. The <code>parameter</code>
       * specifies the edges to be reversed and can be an {@link yfiles.graph.IEdge}, an {@link yfiles.collections.IEnumerable.<T> IEnumerable&lt;IEdge>} or <code>null</code>. If the parameter is
       * <code>null</code> the currently selected edges will be reversed.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#reverseEdges
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      REVERSE_EDGE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to print.
       * <p>
       * This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
       * sense in the context of the application, e.g. printing the currently visible portion of the graph.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+P.
       * </p>
       * @see yfiles.input.ICommand#PRINT_PREVIEW
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      PRINT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to close a document.
       * <p>
       * This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
       * sense in the context of the application, e.g. closing the current window.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      CLOSE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to open help.
       * <p>
       * This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
       * sense in the context of the application, e.g. showing the online help.
       * </p>
       * <p>
       * The default shortcut for this command is F1.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      HELP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to show properties of an item.
       * <p>
       * This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
       * sense in the context of the application.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      PROPERTIES?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to create a new document.
       * <p>
       * This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
       * sense in the context of the application, e.g. creating a new graph.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+N.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      NEW?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to open a document.
       * <p>
       * {@link yfiles.view.GraphComponent} handles this command to open a dialog prompting to open a GraphML file.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+O.
       * </p>
       * @see yfiles.input.ICommand#SAVE
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      OPEN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to select all items.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} and {@link yfiles.input.GraphViewerInputMode} handle this command to select all items in the
       * graph.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+A.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SELECT_ALL?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to delete an item.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} handles this command to delete the currently selected items.
       * </p>
       * <p>
       * The default shortcut for this command is Del.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      DELETE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to save an item.
       * <p>
       * {@link yfiles.view.GraphComponent} handles this command to open a dialog prompting to save the graph as a GraphML file, if the
       * graph was never saved before, and simply saving to the previous file name if it was.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+S.
       * </p>
       * @see yfiles.input.ICommand#OPEN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SAVE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to cut an item and place it in the clipboard.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} handles this command in conjunction with {@link yfiles.graph.GraphClipboard}.
       * </p>
       * <p>
       * The default shortcuts for this command are Ctrl+X and Shift+Del.
       * </p>
       * @see yfiles.input.ICommand#COPY
       * @see yfiles.input.ICommand#PASTE
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      CUT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to copy to the clipboard.
       * <p>
       * {@link yfiles.input.GraphViewerInputMode} and {@link yfiles.input.GraphEditorInputMode} handle this command in conjunction with
       * {@link yfiles.graph.GraphClipboard}.
       * </p>
       * <p>
       * The default shortcuts for this command are Ctrl+C and Ctrl+Ins.
       * </p>
       * @see yfiles.input.ICommand#CUT
       * @see yfiles.input.ICommand#PASTE
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      COPY?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to paste from the clipboard.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} handles this command in conjunction with {@link yfiles.graph.GraphClipboard}. The command
       * parameter can be an {@link yfiles.geometry.IPoint} to specify a location for pasting.
       * </p>
       * <p>
       * The default shortcuts for this command are Ctrl+V and Shift+Ins.
       * </p>
       * @see yfiles.input.ICommand#CUT
       * @see yfiles.input.ICommand#COPY
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      PASTE?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to undo the previous operation.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} handles this command in conjunction with {@link yfiles.graph.UndoEngine}.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Z.
       * </p>
       * @see yfiles.input.ICommand#REDO
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      UNDO?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to redo a previously undone operation.
       * <p>
       * {@link yfiles.input.GraphEditorInputMode} handles this command in conjunction with {@link yfiles.graph.UndoEngine}.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Y.
       * </p>
       * @see yfiles.input.ICommand#UNDO
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      REDO?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that indicates the intention to print and show a preview of what to print beforehand.
       * <p>
       * This command is not necessarily handled by yFiles. Application writers can handle the command in a manner that makes
       * sense in the context of the application.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+F2.
       * </p>
       * @see yfiles.input.ICommand#PRINT
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      PRINT_PREVIEW?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to scroll up.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to move the viewport upwards. By default this is done by a full
       * viewport height. The parameter can be a factor that is applied to the viewport height to move the viewport by a
       * different amount.
       * </p>
       * <p>
       * The default shortcut for this command is Page Up.
       * </p>
       * @see yfiles.input.ICommand#SCROLL_PAGE_DOWN
       * @see yfiles.input.ICommand#SCROLL_PAGE_LEFT
       * @see yfiles.input.ICommand#SCROLL_PAGE_RIGHT
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SCROLL_PAGE_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to scroll down.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to move the viewport downwards. By default this is done by a full
       * viewport height. The parameter can be a factor that is applied to the viewport height to move the viewport by a
       * different amount.
       * </p>
       * <p>
       * The default shortcut for this command is Page Down.
       * </p>
       * @see yfiles.input.ICommand#SCROLL_PAGE_UP
       * @see yfiles.input.ICommand#SCROLL_PAGE_LEFT
       * @see yfiles.input.ICommand#SCROLL_PAGE_RIGHT
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SCROLL_PAGE_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to scroll left.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to move the viewport to the left. By default this is done by a full
       * viewport width. The parameter can be a factor that is applied to the viewport width to move the viewport by a different
       * amount.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @see yfiles.input.ICommand#SCROLL_PAGE_UP
       * @see yfiles.input.ICommand#SCROLL_PAGE_DOWN
       * @see yfiles.input.ICommand#SCROLL_PAGE_RIGHT
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SCROLL_PAGE_LEFT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to scroll right.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to move the viewport to the right. By default this is done by a full
       * viewport width. The parameter can be a factor that is applied to the viewport width to move the viewport by a different
       * amount.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @see yfiles.input.ICommand#SCROLL_PAGE_UP
       * @see yfiles.input.ICommand#SCROLL_PAGE_DOWN
       * @see yfiles.input.ICommand#SCROLL_PAGE_LEFT
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SCROLL_PAGE_RIGHT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the selection left.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to select the next item to the left.
       * </p>
       * <p>
       * The default shortcut for this command is Arrow Left.
       * </p>
       * @see yfiles.input.ICommand#MOVE_UP
       * @see yfiles.input.ICommand#MOVE_DOWN
       * @see yfiles.input.ICommand#MOVE_RIGHT
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_LEFT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the selection right.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to select the next item to the right.
       * </p>
       * <p>
       * The default shortcut for this command is Arrow Right.
       * </p>
       * @see yfiles.input.ICommand#MOVE_UP
       * @see yfiles.input.ICommand#MOVE_DOWN
       * @see yfiles.input.ICommand#MOVE_LEFT
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_RIGHT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the selection up.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to select the next item above.
       * </p>
       * <p>
       * The default shortcut for this command is Arrow Up.
       * </p>
       * @see yfiles.input.ICommand#MOVE_DOWN
       * @see yfiles.input.ICommand#MOVE_LEFT
       * @see yfiles.input.ICommand#MOVE_RIGHT
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the selection down.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to select the next item below.
       * </p>
       * <p>
       * The default shortcut for this command is Arrow Down.
       * </p>
       * @see yfiles.input.ICommand#MOVE_UP
       * @see yfiles.input.ICommand#MOVE_LEFT
       * @see yfiles.input.ICommand#MOVE_RIGHT
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the selection up a page.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to select the parent node, if possible.
       * </p>
       * <p>
       * The default shortcut for this command is Page Up.
       * </p>
       * @see yfiles.input.ICommand#MOVE_UP
       * @see yfiles.input.ICommand#MOVE_DOWN
       * @see yfiles.input.ICommand#MOVE_LEFT
       * @see yfiles.input.ICommand#MOVE_RIGHT
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_TO_PAGE_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the selection down a page.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to select the first child node, if possible.
       * </p>
       * <p>
       * The default shortcut for this command is Page Down.
       * </p>
       * @see yfiles.input.ICommand#MOVE_UP
       * @see yfiles.input.ICommand#MOVE_DOWN
       * @see yfiles.input.ICommand#MOVE_LEFT
       * @see yfiles.input.ICommand#MOVE_RIGHT
       * @see yfiles.input.ICommand#MOVE_TO_PAGE_UP
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_TO_PAGE_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the focus backwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to focus the next item to the left.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Arrow Left.
       * </p>
       * @see yfiles.input.ICommand#MOVE_FOCUS_FORWARD
       * @see yfiles.input.ICommand#MOVE_FOCUS_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_DOWN
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_FOCUS_BACK?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the focus forwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to focus the next item to the right.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Arrow Right.
       * </p>
       * @see yfiles.input.ICommand#MOVE_FOCUS_BACK
       * @see yfiles.input.ICommand#MOVE_FOCUS_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_DOWN
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_FOCUS_FORWARD?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the focus upwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to focus the next item above.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Arrow Up.
       * </p>
       * @see yfiles.input.ICommand#MOVE_FOCUS_BACK
       * @see yfiles.input.ICommand#MOVE_FOCUS_FORWARD
       * @see yfiles.input.ICommand#MOVE_FOCUS_DOWN
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_FOCUS_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the focus downwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to focus the next item below.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Arrow Down.
       * </p>
       * @see yfiles.input.ICommand#MOVE_FOCUS_BACK
       * @see yfiles.input.ICommand#MOVE_FOCUS_FORWARD
       * @see yfiles.input.ICommand#MOVE_FOCUS_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_FOCUS_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the focus up a page.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to focus the parent node, if possible.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Page Up.
       * </p>
       * @see yfiles.input.ICommand#MOVE_FOCUS_BACK
       * @see yfiles.input.ICommand#MOVE_FOCUS_FORWARD
       * @see yfiles.input.ICommand#MOVE_FOCUS_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_DOWN
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_FOCUS_PAGE_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to move the focus up a page.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to focus the first child node, if possible.
       * </p>
       * <p>
       * The default shortcut for this command is Ctrl+Page Down.
       * </p>
       * @see yfiles.input.ICommand#MOVE_FOCUS_BACK
       * @see yfiles.input.ICommand#MOVE_FOCUS_FORWARD
       * @see yfiles.input.ICommand#MOVE_FOCUS_UP
       * @see yfiles.input.ICommand#MOVE_FOCUS_DOWN
       * @see yfiles.input.ICommand#MOVE_FOCUS_PAGE_UP
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      MOVE_FOCUS_PAGE_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to extend the selection to the left.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to extend the selection with the next item to the left.
       * </p>
       * <p>
       * The default shortcut for this command is Shift+Arrow Left.
       * </p>
       * @see yfiles.input.ICommand#EXTEND_SELECTION_UP
       * @see yfiles.input.ICommand#EXTEND_SELECTION_DOWN
       * @see yfiles.input.ICommand#EXTEND_SELECTION_RIGHT
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_UP
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EXTEND_SELECTION_LEFT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to extend the selection to the right.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to extend the selection with the next item to the right.
       * </p>
       * <p>
       * The default shortcut for this command is Shift+Arrow Right.
       * </p>
       * @see yfiles.input.ICommand#EXTEND_SELECTION_UP
       * @see yfiles.input.ICommand#EXTEND_SELECTION_DOWN
       * @see yfiles.input.ICommand#EXTEND_SELECTION_LEFT
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_UP
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EXTEND_SELECTION_RIGHT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to extend the selection upwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to extend the selection with the next item above.
       * </p>
       * <p>
       * The default shortcut for this command is Shift+Arrow Up.
       * </p>
       * @see yfiles.input.ICommand#EXTEND_SELECTION_DOWN
       * @see yfiles.input.ICommand#EXTEND_SELECTION_LEFT
       * @see yfiles.input.ICommand#EXTEND_SELECTION_RIGHT
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_UP
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EXTEND_SELECTION_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to extend the selection downwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to extend the selection with the next item below.
       * </p>
       * <p>
       * The default shortcut for this command is Shift+Arrow Down.
       * </p>
       * @see yfiles.input.ICommand#EXTEND_SELECTION_UP
       * @see yfiles.input.ICommand#EXTEND_SELECTION_LEFT
       * @see yfiles.input.ICommand#EXTEND_SELECTION_RIGHT
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_UP
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      EXTEND_SELECTION_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to extend the selection a page upwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to extend the selection with the parent node, if possible.
       * </p>
       * <p>
       * The default shortcut for this command is Shift+Page Up.
       * </p>
       * @see yfiles.input.ICommand#EXTEND_SELECTION_UP
       * @see yfiles.input.ICommand#EXTEND_SELECTION_DOWN
       * @see yfiles.input.ICommand#EXTEND_SELECTION_LEFT
       * @see yfiles.input.ICommand#EXTEND_SELECTION_RIGHT
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_DOWN
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SELECT_TO_PAGE_UP?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to extend the selection a page downwards.
       * <p>
       * {@link yfiles.input.NavigationInputMode} handles this command to extend the selection with the first child node, if possible.
       * </p>
       * <p>
       * The default shortcut for this command is Shift+Page Down.
       * </p>
       * @see yfiles.input.ICommand#EXTEND_SELECTION_UP
       * @see yfiles.input.ICommand#EXTEND_SELECTION_DOWN
       * @see yfiles.input.ICommand#EXTEND_SELECTION_LEFT
       * @see yfiles.input.ICommand#EXTEND_SELECTION_RIGHT
       * @see yfiles.input.ICommand#SELECT_TO_PAGE_UP
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SELECT_TO_PAGE_DOWN?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to increase the zoom level.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to increase the zoom level by 20%. A different factor from the default
       * <code>1.2</code> may be given as the command parameter.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @see yfiles.input.ICommand#DECREASE_ZOOM
       * @see yfiles.input.ICommand#ZOOM
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      INCREASE_ZOOM?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to decrease the zoom level.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to decrease the zoom level by 20%. A different factor from the default
       * <code>1.2</code> may be given as the command parameter.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @see yfiles.input.ICommand#INCREASE_ZOOM
       * @see yfiles.input.ICommand#ZOOM
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      DECREASE_ZOOM?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} to set the zoom level.
       * <p>
       * {@link yfiles.view.CanvasComponent} handles this command to change the viewport. If the parameter is a {@link yfiles.geometry.Point},
       * the viewport will be centered as that point while not changing the zoom level. If the parameter is a
       * {@link yfiles.geometry.Rect}, the viewport will encompass that rectangle and change the zoom level accordingly. If the parameter
       * is an instance of {@link yfiles.graph.ILookup} which returns an instance of {@link yfiles.view.IBoundsProvider}, that {@link yfiles.view.IBoundsProvider}
       * will be used to obtain a rectangle, which will be used in the same manner as the {@link yfiles.geometry.Rect} mentioned before.
       * This can be used to zoom to a specific {@link yfiles.graph.IModelItem}. Finally, if the parameter is {@link yfiles.lang.Number convertible} to
       * {@link number}, it will be used as the new zoom level to set.
       * </p>
       * <p>
       * There is no default shortcut for this command.
       * </p>
       * @see yfiles.input.ICommand#INCREASE_ZOOM
       * @see yfiles.input.ICommand#DECREASE_ZOOM
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      ZOOM?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that invokes CurrentItem if executed.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      SET_CURRENT_ITEM?:yfiles.input.ICommand;
      /**
       * The {@link yfiles.input.ICommand} that performs the FitGraphBounds(InsetsD) action.
       * <p>
       * The parameter can be either an {@link yfiles.geometry.Insets} or convertible that will be used to create an appropriately sized
       * insets object from.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      FIT_GRAPH_BOUNDS?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that changes the {@link yfiles.view.CanvasComponent#zoom} and the {@link yfiles.view.CanvasComponent#viewPoint} in an animated fashion such that the {@link yfiles.view.GraphComponent#currentItem} becomes fully visible.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      ZOOM_TO_CURRENT_ITEM?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that invokes {@link yfiles.input.ICommand#FIT_CONTENT} if executed.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      FIT_CONTENT?:yfiles.input.ICommand;
      /**
       * A {@link yfiles.input.ICommand} that invokes {@link yfiles.view.CanvasComponent#updateContentRect} if executed.
       * @const
       * @static
       * @type {yfiles.input.ICommand}
       */
      UPDATE_CONTENT_RECT?:yfiles.input.ICommand;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Utility class that holds implementation singletons and utility methods for simple event recognizers.
     * @class
     * @static
     */
    export interface EventRecognizers extends Object{}
    export class EventRecognizers {
      /**
       * Creates an instance that is the logical 'and' combination of the provided recognizers.
       * @param recognizers The recognizers to perform the logical operation on.
       * @returns An instance that is the result of the logical 'and' operation of the provided instances.
       * @static
       */
      static createAndRecognizer(...recognizers:((eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean)[]):(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates an instance that is the logical 'or' combination of the provided recognizers.
       * @param recognizers The recognizers to perform the logical operation on.
       * @returns An instance that is the result of the logical 'or' operation of the provided instances.
       * @static
       */
      static createOrRecognizer(...recognizers:((eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean)[]):(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates an event recognizer instance that yields the boolean inverse of the given event recognizer.
       * @param recognizer The recognizer to invert.
       * @returns An implementation that performs the logical negation for the argument.
       * @static
       */
      static inverse(recognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean):(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * A callback that will always return true;
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static ALWAYS:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * A callback that will always return false;
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static NEVER:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Utility class with simple event recognizers that work on {@link yfiles.view.KeyEventArgs}.
     * @class
     * @static
     */
    export interface KeyEventRecognizers extends Object{}
    export class KeyEventRecognizers {
      /**
       * Creates an event recognizer for a given key and type using a given set of modifiers.
       * @param type The type of the event.
       * @param value The key that is subject of the event.
       * @param [modifiers=yfiles.view.ModifierKeys.NONE] The state of the modifiers that must be set. This is {@link #NONE} if not set.
       * @returns 
       * @static
       */
      static create(type:yfiles.view.KeyEventType,value:yfiles.view.Key,modifiers?:yfiles.view.ModifierKeys):(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Creates an event recognizer that matches key presses.
       * @param key The key to create the recognizer for.
       * @param [modifiers=yfiles.view.ModifierKeys.NONE] The modifier keys that should be pressed for the recognizer
       * @returns An event recognizer that uses the gesture.
       * @static
       */
      static createKeyPress(key:yfiles.view.Key,modifiers?:yfiles.view.ModifierKeys):(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when escape has been typed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static ESCAPE_TYPED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when any key has been pressed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static KEY_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when any key has been typed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static KEY_TYPED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when any key has been released.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static KEY_RELEASED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when escape has been pressed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static ESCAPE_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when delete has been pressed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static DELETE_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when escape has been released.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static ESCAPE_RELEASED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the shift modifier is active.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static SHIFT_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the alt modifier is active.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static ALT_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the ctrl modifier is active.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static CONTROL_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the Meta modifier is active.
       * <p>
       * Meta maps to the Windows key on Windows platforms and the Command key on Mac.
       * </p>
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static META_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the shift modifier is being pressed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static SHIFT_DOWN:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the shift modifier is being released.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static SHIFT_UP:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the ctrl modifier is being pressed.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static CTRL_DOWN:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that recognizes when the Ctrl modifier is being released.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static CTRL_UP:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that offers event recognizers that deal with {@link yfiles.view.MouseEventArgs}.
     * @class
     * @static
     */
    export interface MouseEventRecognizers extends Object{}
    export class MouseEventRecognizers {
      /**
       * An event recognizer that identifies mouse movements that are not drags.
       * @see yfiles.input.MouseEventRecognizers#DRAGGED
       * @see yfiles.input.MouseEventRecognizers#MOVED_OR_DRAGGED
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static MOVED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies mouse movements.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static MOVED_OR_DRAGGED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies <code>mousedown</code> events.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies <code>mouseup</code> events.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static RELEASED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies mouse input capture lost events.
       * <p>
       * This event will occur often, especially after all mouse buttons have been released. More relevant for
       * {@link yfiles.input.IInputMode}s is the {@link yfiles.input.MouseEventRecognizers#LOST_CAPTURE_DURING_DRAG} event recognizer.
       * </p>
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LOST_CAPTURE:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies mouse input capture lost events while at least one mouse button was pressed.
       * <p>
       * Since the mouse capture has been lost, the necessary {@link yfiles.view.MouseEventTypes#UP} events will never be triggered. This
       * recognizer is useful for identifying this situation.
       * </p>
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LOST_CAPTURE_DURING_DRAG:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies left mouse button clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LEFT_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies button clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies button multi-clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static MULTI_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies right mouse button clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static RIGHT_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies middle mouse button clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static MIDDLE_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies left mouse button double clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LEFT_DOUBLE_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies right mouse button double clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static RIGHT_DOUBLE_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies middle mouse button double clicks.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static MIDDLE_DOUBLE_CLICKED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies <code>mousedown</code> events of left mouse button.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LEFT_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies <code>mouseup</code> events of left mouse button.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LEFT_RELEASED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies <code>mousedown</code> events of right mouse button.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static RIGHT_PRESSED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies <code>mouseup</code> events of right mouse button.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static RIGHT_RELEASED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies mouse drag events.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static DRAGGED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies left mouse button drags.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LEFT_DRAGGED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies when the has left the control.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static EXITED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies when the mouse has entered the control.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static ENTERED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies right mouse button drags.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static RIGHT_DRAGGED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies middle mouse button drags.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static MIDDLE_DRAGGED:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that offers event recognizers that deal with {@link yfiles.view.TouchEventArgs}.
     * @class
     * @static
     */
    export interface TouchEventRecognizers extends Object{}
    export class TouchEventRecognizers {
      /**
       * An event recognizer that identifies touch movements of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_MOVED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies the loss of touch capture for the any device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LOST_CAPTURE:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies the loss of touch capture for the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LOST_CAPTURE_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies the loss of touch capture for the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static LOST_CAPTURE_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch movements of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_DOWN_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch movements of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_UP_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch taps of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_TAPPED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch double-taps of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_DOUBLE_TAPPED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch multi-taps of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_MULTI_TAPPED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies when the primary device has left the control.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_EXITED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies when the primary device has entered the control.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_ENTERED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies long presses of the primary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_LONG_PRESSED_PRIMARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch movements of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_MOVED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch movements of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_DOWN_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch movements of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_UP_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch taps of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_TAPPED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch double-taps of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_DOUBLE_TAPPED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies touch multi-taps of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_MULTI_TAPPED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies when the secondary device has left the control.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_EXITED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies when the secondary device has entered the control.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_ENTERED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * An event recognizer that identifies long presses of the secondary device.
       * @const
       * @static
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      static TOUCH_LONG_PRESSED_SECONDARY:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} used for recognizing the gesture of creating {@link yfiles.graph.IBend}s in an {@link yfiles.graph.IGraph}.
     * <p>
     * This mode is used by {@link yfiles.input.GraphEditorInputMode} but can be used stand-alone, too. Client code should register
     * with the {@link yfiles.input.CreateBendInputMode#addBendCreatedListener BendCreated} event to perform
     * additional actions after the bend has been created.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.CreateBendInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface CreateBendInputMode extends Object,yfiles.input.IInputMode{}
    export class CreateBendInputMode {
      /**
       * Creates a new instance that will use the {@link yfiles.graph.IGraph} from the {@link yfiles.input.CreateBendInputMode#inputModeContext} to create bends in.
       * @see yfiles.input.CreateBendInputMode#graph
       */
      constructor();
      /**
       * Called when a bend creation gesture can be started at the current mouse pointer location.
       * <p>
       * This implementation sets the {@link yfiles.input.ConcurrencyController#preferredCursor} to {@link yfiles.input.CreateBendInputMode#armedCursor}.
       * </p>
       * @protected
       */
      arm():void;
      /**
       * Cancels the editing of this mode.
       * <p>
       * This implementation {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
       * {@link yfiles.input.CreateBendInputMode#onCanceled}.
       * </p>
       * @see yfiles.input.CreateBendInputMode#onCanceled
       */
      cancel():void;
      /**
       * Creates the bend.
       * <p>
       * The bend is created using the hit edge's {@link yfiles.input.IBendCreator} implementation, as obtained from the edge's
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @param edge The hit edge.
       * @param location The coordinates of the hit.
       * @returns The created bend or <code>null</code>.
       * @see yfiles.input.CreateBendInputMode#createInputModeContext
       * @protected
       */
      createBend(edge:yfiles.graph.IEdge,location:yfiles.geometry.Point):yfiles.graph.IBend;
      /**
       * Creates an {@link yfiles.input.IInputModeContext} for use with the upcoming {@link yfiles.input.IBendCreator#createBend} call in
       * {@link yfiles.input.CreateBendInputMode#createBend}.
       * @returns An instance of {@link }.
       * @protected
       */
      createInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Called when a bend creation gesture can no longer be started at the current mouse pointer location.
       * <p>
       * This implementation sets the {@link yfiles.input.ConcurrencyController#preferredCursor} back to <code>null</code>.
       * </p>
       * @protected
       */
      disarm():void;
      /**
       * Updates the {@link yfiles.input.CreateBendInputMode#splitOrthogonalSegment} property and then delegates to {@link yfiles.input.CreateBendInputMode#createBend}.
       * <p>
       * This method triggers the {@link yfiles.input.CreateBendInputMode#addBendCreatedListener BendCreated} event
       * if a bend is created.
       * </p>
       * @param edge The edge to create a bend for.
       * @param dragLocation The location at which the user initiated the gesture.
       * @protected
       */
      dragSegment(edge:yfiles.graph.IEdge,dragLocation:yfiles.geometry.Point):void;
      /**
       * Finds the edge at the given coordinate.
       * @param location The coordinates.
       * @returns The edge that has been hit.
       * @protected
       */
      getEdge(location:yfiles.geometry.Point):yfiles.graph.IEdge;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Determines whether the current location is valid to begin a bend creation gesture.
       * @param source
       * @param evt
       * @returns 
       * @protected
       */
      isValidBegin(source:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Triggers the {@link yfiles.input.CreateBendInputMode#addBendCreatedListener BendCreated} event.
       * @param evt
       * @protected
       */
      onBendCreated(evt:yfiles.graph.BendEventArgs):void;
      /**
       * Called after {@link yfiles.input.CreateBendInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Triggers the {@link yfiles.input.CreateBendInputMode#addDragCanceledListener DragCanceled} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.CreateBendInputMode#addDraggedListener Dragged} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragged(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.CreateBendInputMode#addDraggingListener Dragging} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragging(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called after {@link yfiles.input.CreateBendInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.CreateBendInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.CreateBendInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.CreateBendInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Adds the given listener for the <code>BendCreated</code> event that occurs once a bend creation gesture has been recognized.
       * @param listener The listener to add.
       * @see yfiles.input.CreateBendInputMode#removeBendCreatedListener
       */
      addBendCreatedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>BendCreated</code> event that occurs once a bend creation gesture has been recognized.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateBendInputMode#addBendCreatedListener
       */
      removeBendCreatedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * Gets or sets the cursor to use when this mode is in "armed" state.
       * @type {yfiles.view.Cursor}
       */
      armedCursor:yfiles.view.Cursor;
      /**
       * Gets or sets the "pressed" state recognizer.
       * <p>
       * This recognizer instance will be used to determine when the user begins to move the selection.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer.
       * <p>
       * This recognizer instance determines when the user is moving the selection
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer.
       * <p>
       * This instance determines when the user has finished the move.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" state recognizer specifically for touch input.
       * <p>
       * This recognizer instance will be used to determine when the user begins to move the selection. The default value is
       * {@link yfiles.input.TouchEventRecognizers#TOUCH_LONG_PRESSED_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      prepareRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "moved" state recognizer specifically for touch input.
       * <p>
       * This recognizer instance will be used to determine when the user begins actually moves the selection. The default value
       * is {@link yfiles.input.TouchEventRecognizers#TOUCH_MOVED_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      movedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" state recognizer specifically for touch input.
       * <p>
       * This recognizer instance will be used to determine when the finalized the gesture. The default value is
       * {@link yfiles.input.TouchEventRecognizers#TOUCH_UP_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the {@link yfiles.input.IHitTestable} that determines where bends may be created.
       * @type {yfiles.input.IHitTestable}
       */
      beginHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets the location where the bend creation gesture was initialized.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * Gets the graph this mode is acting upon.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.CreateBendInputMode#removeDraggingListener
       */
      addDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateBendInputMode#addDraggingListener
       */
      removeDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.CreateBendInputMode#removeDraggedListener
       */
      addDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateBendInputMode#addDraggedListener
       */
      removeDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.CreateBendInputMode#removeDragCanceledListener
       */
      addDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateBendInputMode#addDragCanceledListener
       */
      removeDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Gets or sets the event recognizer that determines whether the bend creation should be treated as a split segment
       * operation.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      splitOrthogonalSegmentRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets a value indicating whether an orthogonal segment should be split by the last bend creation.
       * <p>
       * This property can be read by implementations of {@link yfiles.input.IBendCreator} if they retrieve this instance from their {@link yfiles.input.IInputModeContext#parentInputMode}. If
       * this property is
       * <code>true</code>, implementation are advised to treat the bend creation as a gesture that splits the segment into two.
       * </p>
       * @see yfiles.input.CreateBendInputMode#splitOrthogonalSegmentRecognizer
       * @type {boolean}
       */
      splitOrthogonalSegment:boolean;
      /**
       * Gets or sets a property that determines whether this mode should use an existing bend in {@link yfiles.input.CreateBendInputMode#createBend}
       * if there is one found at the given location.
       * <p>
       * The default is <code>true</code>
       * </p>
       * @default true
       * @type {boolean}
       */
      useExistingBend:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple default implementation of the {@link yfiles.input.IEditLabelHelper} interface.
     * <p>
     * This class provided a number of factory methods that can be overwritten to change the default behavior which is modeled
     * after the behavior used by {@link yfiles.input.GraphEditorInputMode}'s {@link yfiles.input.GraphEditorInputMode#createLabel} and
     * {@link yfiles.input.GraphEditorInputMode#editLabel}.
     * </p>
     * @class
     * @implements {yfiles.input.IEditLabelHelper}
     */
    export interface EditLabelHelper extends Object,yfiles.input.IEditLabelHelper{}
    export class EditLabelHelper {
      /**
       * Determines the label model parameter to use for the label, depending on the <code>owner</code>.
       * <p>
       * This method will obtain the default parameter instances from the {@link yfiles.input.EditLabelHelper#graph} considering the type of the current <code>owner</code>.
       * </p>
       * @param context The input mode context.
       * @param owner The label's owner.
       * @returns The layout parameter to use or <code>null</code>.
       * @protected
       */
      getLabelParameter(context:yfiles.input.IInputModeContext,owner:yfiles.graph.ILabelOwner):yfiles.graph.ILabelModelParameter;
      /**
       * Determines the style to use for the label, depending on the <code>owner</code>.
       * <p>
       * This method will obtain the default style instances from the {@link yfiles.input.EditLabelHelper#graph} considering the type of the current <code>owner</code>.
       * </p>
       * @param context The input mode context.
       * @param owner The label's owner.
       * @returns The style to use or <code>null</code>.
       * @protected
       */
      getLabelStyle(context:yfiles.input.IInputModeContext,owner:yfiles.graph.ILabelOwner):yfiles.styles.ILabelStyle;
      /**
       * Gets the preferred size for the label.
       * <p>
       * This implementation yields <code>null</code> to indicate that the preferred size should be calculated after the text has been
       * entered.
       * </p>
       * @param context The input mode context.
       * @param label The label to get the preferred size from.
       * @returns The size to use or <code>null</code>.
       * @protected
       */
      getPreferredSize(context:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):yfiles.geometry.Size;
      /**
       * Helper method that will be called for {@link yfiles.graph.ILabelOwner}s.
       * <p>
       * This method delegates to {@link yfiles.input.EditLabelHelper#getLabelParameter}, {@link yfiles.input.EditLabelHelper#getLabelStyle}, and
       * {@link yfiles.input.EditLabelHelper#getPreferredSize} to set the appropriate properties on <code>args</code> and sets the {@link yfiles.input.LabelEditingEventArgs#handled} property to
       * <code>true</code>. Label creation thus is always possible.
       * </p>
       * @param evt The original event arguments.
       */
      onLabelAdding(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Helper method that will be called for {@link yfiles.graph.ILabelOwner}s.
       * <p>
       * This method returns the first of the {@link yfiles.graph.ILabelOwner#labels} that belong to the {@link yfiles.input.LabelEditingEventArgs#owner}, if set. Otherwise, it delegates to
       * {@link yfiles.input.EditLabelHelper#getLabelParameter}, {@link yfiles.input.EditLabelHelper#getLabelStyle}, and {@link yfiles.input.EditLabelHelper#getPreferredSize}
       * to set the appropriate properties on <code>args</code>.
       * </p>
       * <p>
       * The arguments are always set as {@link yfiles.input.LabelEditingEventArgs#handled}, so label editing is always possible.
       * </p>
       * @param evt The original event arguments.
       */
      onLabelEditing(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Gets or sets the graph instance to retrieve the default values from.
       * <p>
       * If this instance is not set, the implementation will try to retrieve the graph from the {@link yfiles.input.IInputModeContext}'s
       * {@link yfiles.graph.ILookup}.
       * </p>
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} for interacticely creating edges between nodes and edges in an {@link yfiles.graph.IGraph} displayed
     * in a {@link yfiles.view.CanvasComponent} or {@link yfiles.view.GraphComponent}.
     * <p>
     * This class provides a {@link yfiles.input.CreateEdgeInputMode#edgeCreator factory callback} that the final edge creation will be delegated to. The styling of the newly
     * created edge can be customized via the {@link yfiles.input.CreateEdgeInputMode#edgeDefaults}. Defaults can be automatically adjusted in response to the various events that
     * will be triggered by this instance during the edge creation. Use the {@link yfiles.input.CreateEdgeInputMode#dummyEdgeGraph} property to modify the appearance of the dummy
     * edge. This class uses {@link yfiles.input.IPortCandidateProvider} instances found in the {@link yfiles.graph.ILookup} of the {@link yfiles.graph.INode nodes} in
     * the graph to display port candidates and to finally create the edges.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.CreateEdgeInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface CreateEdgeInputMode extends Object,yfiles.input.IInputMode{}
    export class CreateEdgeInputMode {
      /**
       * Creates a new instance that will use the {@link yfiles.graph.IGraph} from the {@link yfiles.input.CreateEdgeInputMode#inputModeContext} to create edges in.
       * <p>
       * Setting the {@link yfiles.input.CreateEdgeInputMode#graph} property can be used to optionally set a different graph to work on.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#graph
       */
      constructor();
      /**
       * Cancels the editing of this mode.
       * <p>
       * This implementation {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
       * {@link yfiles.input.CreateEdgeInputMode#onCanceled}.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#onCanceled
       */
      cancel():void;
      /**
       * Creates the dummy edge that will be displayed by the input mode during the creation.
       * @returns An {@link } implementation that can be used as a dummy.
       * @protected
       */
      createDummyEdge():yfiles.graph.IEdge;
      /**
       * Factory method that will create the dummy graph to hold the preview of the {@link yfiles.input.CreateEdgeInputMode#dummyEdge}.
       * <p>
       * Note that the instance returned by this instance must accept any port as valid source and target ports for it's {@link yfiles.graph.IGraph#setEdgePorts}
       * method. This will allow the preview edge to be connected to the actual {@link yfiles.input.CreateEdgeInputMode#graph}.
       * </p>
       * @returns A graph instance that will be rendered as a preview.
       * @protected
       */
      createDummyEdgeGraph():yfiles.graph.IGraph;
      /**
       * Callback used by {@link yfiles.input.CreateEdgeInputMode#createEdge} if both {@link yfiles.input.CreateEdgeInputMode#sourcePortCandidate} and {@link yfiles.input.CreateEdgeInputMode#targetPortCandidate} have been set.
       * <p>
       * This method will first {@link yfiles.input.IPortCandidate#createPort create the new ports} if necessary and then create the edge via the {@link yfiles.input.CreateEdgeInputMode#edgeCreator} callback to the ports. After
       * the ports are created the corresponding {@link yfiles.input.CreateEdgeInputMode#addPortAddedListener PortAdded}
       * events are triggered. If the edge creation is canceled by the creator callback, newly created ports will be removed
       * again.
       * </p>
       * @param graph The graph to create the edge for.
       * @param sourcePortCandidate The candidate to use for the source.
       * @param targetPortCandidate The candidate to use for the target.
       * @returns The edge created or <code>null</code>.
       * @protected
       */
      createEdge(graph:yfiles.graph.IGraph,sourcePortCandidate:yfiles.input.IPortCandidate,targetPortCandidate:yfiles.input.IPortCandidate):yfiles.graph.IEdge;
      /**
       * Callback at the end of the edge creation process.
       * <p>
       * This method will actually create the edge using the {@link yfiles.input.CreateEdgeInputMode#createEdge} method. This method will
       * ultimately call {@link yfiles.input.CreateEdgeInputMode#onEdgeCreated}.
       * </p>
       * @returns The edge created or <code>null</code>.
       * @protected
       */
      createEdge():yfiles.graph.IEdge;
      /**
       * Creates an {@link yfiles.input.IInputModeContext} for use with the port candidates queries for the upcoming edge creation
       * operation and the actual edge creation
       * @returns An instance of {@link } that is configured for this mode.
       * @protected
       */
      createEdgeCreationInputModeContext():yfiles.input.IInputModeContext;
      /**
       * Synthetically starts the interactive edge creation process using the provided {@link yfiles.input.IPortCandidate} as the source
       * port.
       * <p>
       * This instance needs to be {@link yfiles.input.IInputMode#install installed} and {@link yfiles.input.CreateEdgeInputMode#enabled} and it needs to be able to {@link yfiles.input.ConcurrencyController#canRequestMutex request the input mutex} in order to proceed. If any
       * of these conditions are not satisfied, this method will throw an {@link yfiles.lang.Exception}.
       * </p>
       * @param sourcePortCandidate The source port candidate to use for the edge creation.
       */
      doStartEdgeCreation(sourcePortCandidate:yfiles.input.IPortCandidate):void;
      /**
       * Finds the closest {@link yfiles.input.PortCandidateValidity#VALID} candidate given a number of
       * {@link yfiles.input.IPortCandidate}s and a location in world coordinates.
       * @param candidates The candidates to find the closest from.
       * @param location The location of the mouse in world coordinates.
       * @returns The candidate to use or <code>null</code> if no candidate satisfies the needs
       * @protected
       */
      getClosestCandidate(candidates:yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>,location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Finds the closest candidate given a number of {@link yfiles.input.IPortCandidate}s and a location in world coordinates.
       * @param candidates The candidates to find the closest from.
       * @param location The location of the pointer in world coordinates.
       * @returns The candidate to use or <code>null</code> if no candidate satisfies the needs
       * @protected
       */
      getClosestSourceCandidate(candidates:yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>,location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Finds the closest target candidate given a number of {@link yfiles.input.IPortCandidate}s and a location in world coordinates.
       * @param candidates The candidates to find the closest from.
       * @param location The location of the mouse in world coordinates.
       * @returns The candidate to use or <code>null</code> if no candidate satisfies the needs or {@link #enforceBendCreationRecognizer} yields <code>true</code>.
       * @protected
       */
      getClosestTargetCandidate(candidates:yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>,location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Gets the snap lines lines that are induced by the current dummy edge.
       * @param context The graph snap context.
       * @param provider The dummy edge segment snap line provider.
       * @returns The snap lines induced by the current dummy edge.
       * @protected
       */
      getDummyEdgeSnapLines(context:yfiles.input.GraphSnapContext,provider:yfiles.input.ISnapLineProvider):yfiles.collections.IEnumerable<yfiles.input.OrthogonalSnapLine>;
      /**
       * Calculates direction of first edge segment of the DummyNode.
       * <p>
       * An edge starting in an border will always be orthogonal to the border's outer line. An edge starting in a corner will
       * point the way the user dragged the mouse. An edge starting in the inner of the node will be directed the way dragged
       * unless another edge that is close to the new edge ({@link yfiles.input.CreateEdgeInputMode#preferredMinimalEdgeDistance}) already went that way.
       * </p>
       * @returns A {@link } defining the direction of the first edge segment.
       * @protected
       */
      getFirstSegmentDirection():yfiles.input.EdgeSegmentDirection;
      /**
       * Retrieves the port owner at a given position in world coordinates.
       * <p>
       * If there is a {@link yfiles.input.IHitTester.<T>} for type {@link yfiles.graph.INode} in the lookup of this mode's {@link yfiles.input.CreateEdgeInputMode#inputModeContext} then this instance will be used for the query.
       * 
       * <br />
       * Otherwise this implementation calls the {@link yfiles.styles.INodeStyleRenderer#getHitTestable} method of the nodes to find hit
       * nodes and uses an {@link yfiles.collections.IComparer.<T>} to determine the first hit node.
       * </p>
       * @param location The position in world coordinates.
       * @returns The node or <code>null</code> if no suitable node was found.
       * @protected
       */
      getPortOwner(location:yfiles.geometry.Point):yfiles.graph.IPortOwner;
      /**
       * Retrieves the port owners from the graph in the order of their importance.
       * <p>
       * This implementation only considers nodes in the graph unless {@link yfiles.input.CreateEdgeInputMode#allowEdgeToEdgeConnections} is set to
       * <code>true</code>, in which case edges are also added to the enumerable.
       * </p>
       * @returns An enumerable over all {@link }s in this graph
       * @protected
       */
      getPortOwners():yfiles.collections.IEnumerable<yfiles.graph.IPortOwner>;
      /**
       * Finds the source {@link yfiles.graph.IPortOwner port owner} at the specified location.
       * <p>
       * This always is the source {@link yfiles.graph.INode node} if {@link yfiles.input.CreateEdgeInputMode#allowEdgeToEdgeConnections} is set to <code>false</code>.
       * <br />
       * This default implementation simply delegates to {@link yfiles.input.CreateEdgeInputMode#getPortOwner}.
       * </p>
       * @param sourceLocation The location of the mouse at the beginning of the gesture.
       * @returns The port owner to use for the source node or <code>null</code>.
       * @protected
       */
      getSource(sourceLocation:yfiles.geometry.Point):yfiles.graph.IPortOwner;
      /**
       * Finds the best matching source port {@link yfiles.input.IPortCandidate} given a provider and the current location.
       * <p>
       * The <code>resolveCandidates</code> parameter determines whether dynamic port candidates should be {@link yfiles.input.CreateEdgeInputMode#resolveCandidates resolved} with respect to <code>location</code>
       * or simply be discarded.
       * </p>
       * @param provider The provider for the candidates.
       * @param location The location where the gesture was initiated.
       * @param resolveCandidates Determines whether {@link #DYNAMIC} ports should be resolved with respect to the <code>location</code> or
       * not.
       * @returns The candidate to use or <code>null</code>.
       * @see yfiles.input.CreateEdgeInputMode#resolveSourcePortCandidates
       * @see yfiles.input.CreateEdgeInputMode#portCandidateResolutionRecognizer
       * @protected
       */
      getSourcePortCandidate(provider:yfiles.input.IPortCandidateProvider,location:yfiles.geometry.Point,resolveCandidates:boolean):yfiles.input.IPortCandidate;
      /**
       * Finds the best matching source {@link yfiles.input.IPortCandidate port candidate} based on the given location.
       * <p>
       * This implementation will use the {@link yfiles.input.CreateEdgeInputMode#getSource port owner} to look up the {@link yfiles.input.IPortCandidateProvider port candidate provider} and finally delegate to
       * {@link yfiles.input.CreateEdgeInputMode#getSourcePortCandidate}.
       * </p>
       * @param location The location where the gesture was initiated.
       * @returns 
       * @see yfiles.input.CreateEdgeInputMode#getSource
       * @see yfiles.input.CreateEdgeInputMode#getSourcePortCandidateProvider
       * @see yfiles.input.CreateEdgeInputMode#getSourcePortCandidate
       * @protected
       */
      getSourcePortCandidate(location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Retrieves the port candidate provider instance given a source port owner.
       * <p>
       * This implementations uses the <code>sourceOwner</code>'s {@link yfiles.graph.ILookup#lookup} to retrieve an {@link yfiles.input.IPortCandidateProvider}
       * implementation.
       * </p>
       * @param sourceOwner The {@link } to find the candidates for.
       * @returns A provider instance or <code>null</code>.
       * @protected
       */
      getSourcePortCandidateProvider(sourceOwner:yfiles.graph.IPortOwner):yfiles.input.IPortCandidateProvider;
      /**
       * Finds the target {@link yfiles.graph.IPortOwner port owner} at the specified location.
       * <p>
       * This always is a target {@link yfiles.graph.INode node} if {@link yfiles.input.CreateEdgeInputMode#allowEdgeToEdgeConnections} is set to <code>false</code>.
       * <br />
       * This default implementation simply delegates to {@link yfiles.input.CreateEdgeInputMode#getPortOwner}.
       * </p>
       * @param targetLocation The location of the pointer at the beginning of the gesture.
       * @returns The port owner or <code>null</code>.
       * @protected
       */
      getTarget(targetLocation:yfiles.geometry.Point):yfiles.graph.IPortOwner;
      /**
       * Retrieves the port candidate provider instance given a source port candidate and a possible target item.
       * <p>
       * This implementations uses the <code>targetOwner</code>'s {@link yfiles.graph.ILookup#lookup} to retrieve an {@link yfiles.input.IPortCandidateProvider}
       * implementation.
       * </p>
       * @param sourcePortCandidate The current source port candidate.
       * @param targetOwner The {@link } to find the candidates for.
       * @param location The location of the mouse.
       * @returns A provider instance or <code>null</code>.
       * @protected
       */
      getTargetPortCandidateProvider(sourcePortCandidate:yfiles.input.IPortCandidate,targetOwner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point):yfiles.input.IPortCandidateProvider;
      /**
       * Retrieves the {@link yfiles.input.IPortCandidateProvider} instance that provides the possible candidates for the target of the
       * edge given the current source candidate.
       * <p>
       * This implementation queries all nodes in the graph for {@link yfiles.input.CreateEdgeInputMode#getTargetPortCandidateProvider their provider} and returns a composite unless {@link yfiles.input.CreateEdgeInputMode#useHitItemsCandidatesOnly} is set to <code>true</code> in
       * which case the {@link yfiles.input.CreateEdgeInputMode#getTargetPortCandidateProvider} is queried using the current {@link yfiles.input.CreateEdgeInputMode#getTarget target}.
       * </p>
       * @param sourcePortCandidate The source candidate that has been chosen for the edge creation.
       * @param location The location of the mouse.
       * @returns The provider to get the port candidates from.
       * @protected
       */
      getTargetPortCandidateProvider(sourcePortCandidate:yfiles.input.IPortCandidate,location:yfiles.geometry.Point):yfiles.input.IPortCandidateProvider;
      /**
       * Retrieves the target port candidates for a given location in world coordinates.
       * <p>
       * The <code>resolveCandidates</code> parameter determines whether dynamic port candidates should be {@link yfiles.input.CreateEdgeInputMode#resolveCandidates resolved} with respect to <code>location</code>
       * or simply be discarded. This implementation delegates to {@link yfiles.input.CreateEdgeInputMode#getTargetPortCandidateProvider}
       * or returns an empty enumerable if no provider has been found.
       * </p>
       * @param location The location of the mouse in world coordinates.
       * @param resolveCandidates Determines whether {@link #DYNAMIC} ports should be resolved with respect to the <code>location</code> or
       * not.
       * @returns A possibly empty enumeration over all target port candidates.
       * @see yfiles.input.CreateEdgeInputMode#sourcePortCandidate
       * @see yfiles.input.CreateEdgeInputMode#resolveTargetPortCandidates
       * @see yfiles.input.CreateEdgeInputMode#portCandidateResolutionRecognizer
       * @protected
       */
      getTargetPortCandidates(location:yfiles.geometry.Point,resolveCandidates:boolean):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called after {@link yfiles.input.CreateEdgeInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController} property of the installed {@link yfiles.input.ConcurrencyController} has
       * been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Called by {@link yfiles.input.CreateEdgeInputMode#createEdge} after the edge has been finalized.
       * <p>
       * This method will trigger the {@link yfiles.input.CreateEdgeInputMode#addEdgeCreatedListener EdgeCreated}
       * event.
       * </p>
       * @param evt The event argument holding the edge.
       * @protected
       */
      onEdgeCreated(evt:yfiles.graph.EdgeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.CreateEdgeInputMode#addEdgeCreationStartedListener EdgeCreationStarted} event when
       * the gesture for creating an edge has been initialized.
       * @param evt The {@link } instance containing the dummy edge that will be used during edge creation.
       * @protected
       */
      onEdgeCreationStarted(evt:yfiles.graph.EdgeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.CreateEdgeInputMode#addGestureCanceledListener GestureCanceled} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onGestureCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.CreateEdgeInputMode#addGestureCancelingListener GestureCanceling} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onGestureCanceling(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered once the drag has been finalized.
       * <p>
       * This method triggers the {@link yfiles.input.CreateEdgeInputMode#addGestureFinishedListener GestureFinished}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onGestureFinished(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered before the drag will be finalized.
       * <p>
       * This method triggers the {@link yfiles.input.CreateEdgeInputMode#addGestureFinishingListener GestureFinishing}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onGestureFinishing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.CreateEdgeInputMode#addGestureStartedListener GestureStarted} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onGestureStarted(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.CreateEdgeInputMode#addGestureStartingListener GestureStarting} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onGestureStarting(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.CreateEdgeInputMode#addMovedListener Moved} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onMoved(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.CreateEdgeInputMode#addMovingListener Moving} event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onMoving(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.CreateEdgeInputMode#addPortAddedListener PortAdded} event if the mode has
       * {@link yfiles.graph.IGraph#addPort added a port} for the source or target node to complete the edge creation.
       * @param evt The {@link } instance containing the port that has been added.
       * @see yfiles.input.CreateEdgeInputMode#addPortAddedListener
       * @protected
       */
      onPortAdded(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>):void;
      /**
       * Triggers the {@link yfiles.input.CreateEdgeInputMode#addSourcePortCandidateChangedListener SourcePortCandidateChanged}
       * event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onSourcePortCandidateChanged(evt:yfiles.collections.ItemEventArgs<yfiles.input.IPortCandidate>):void;
      /**
       * Called after {@link yfiles.input.CreateEdgeInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * Triggers the {@link yfiles.input.CreateEdgeInputMode#addTargetPortCandidateChangedListener TargetPortCandidateChanged}
       * event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onTargetPortCandidateChanged(evt:yfiles.collections.ItemEventArgs<yfiles.input.IPortCandidate>):void;
      /**
       * Returns an enumerable over the given candidates where no instance has {@link yfiles.input.PortCandidateValidity#DYNAMIC} {@link yfiles.input.IPortCandidate#validity}.
       * <p>
       * If the location has a value, method {@link yfiles.input.IPortCandidate#getPortCandidateAt} is used to resolve the dynamic
       * candidate to a real candidate, otherwise that candidate is discarded.
       * </p>
       * @param candidates The candidates to possibly resolve.
       * @param location The location to resolve dynamic candidates against or <code>null</code> if they should be discarded instead.
       * @returns An enumerable of non-{@link #DYNAMIC} port candidates.
       * @protected
       */
      resolveCandidates(candidates:yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>,location:yfiles.geometry.Point):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * Updates the {@link yfiles.input.CreateEdgeInputMode#dummyEdge} to reflect the current source port candidate.
       * <p>
       * This implementation creates a self loop of the dummy edge if the <code>sourcePortCandidate</code> is the same as the owner of the {@link yfiles.input.CreateEdgeInputMode#targetPortCandidate}.
       * </p>
       * @param sourcePortCandidate The new owner target port candidate.
       * @protected
       */
      updateDummyEdgeSourcePort(sourcePortCandidate:yfiles.input.IPortCandidate):void;
      /**
       * Updates the {@link yfiles.input.CreateEdgeInputMode#dummyEdge} to reflect the current target port candidate.
       * <p>
       * This implementation creates a self loop of the dummy edge if the <code>targetPortCandidate</code> is the same as the owner of the {@link yfiles.input.CreateEdgeInputMode#sourcePortCandidate}.
       * </p>
       * @param targetPortCandidate The new owner target port candidate.
       * @protected
       */
      updateDummyEdgeTargetPort(targetPortCandidate:yfiles.input.IPortCandidate):void;
      /**
       * Updates the drag point.
       * <p>
       * This method is called when the current target location for the creation gesture is changed.
       * </p>
       * @param location The new coordinates.
       * @protected
       */
      updateTargetLocation(location:yfiles.geometry.Point):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.CreateEdgeInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.CreateEdgeInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.CreateEdgeInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Sets the cursor that indicates a valid place to begin an edge creation.
       * @type {yfiles.view.Cursor}
       */
      validBeginCursor:yfiles.view.Cursor;
      /**
       * Sets the cursor that indicates a valid place to create a bend.
       * @type {yfiles.view.Cursor}
       */
      validBendCursor:yfiles.view.Cursor;
      /**
       * Sets the cursor that indicates a valid place to finish creation.
       * @type {yfiles.view.Cursor}
       */
      validEndCursor:yfiles.view.Cursor;
      /**
       * Gets or sets event recognizer that temporarily disables snapping.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      disableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets event recognizer that reenables temporarily disabled snapping.
       * @see yfiles.input.CreateEdgeInputMode#disableSnappingRecognizer
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      enableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that toggles the orientation of the first orthogonally created segment.
       * @see yfiles.input.CreateEdgeInputMode#orthogonalEdgeCreation
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      toggleSegmentOrientationRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the finishing gesture that is used to end the edge creation on the target node.
       * <p>
       * The default is a combination of {@link yfiles.input.MouseEventRecognizers#LEFT_PRESSED} and
       * {@link yfiles.input.MouseEventRecognizers#LEFT_RELEASED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      finishRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the starting gesture that is used to prepare the edge creation on the source node.
       * <p>
       * The default is {@link yfiles.input.MouseEventRecognizers#LEFT_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      prepareRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the initial dragging gesture that is used to initiate the creation after the
       * preparation.
       * <p>
       * The default is {@link yfiles.input.MouseEventRecognizers#DRAGGED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the movement of the end point of the edge.
       * <p>
       * The default is {@link yfiles.input.MouseEventRecognizers#MOVED_OR_DRAGGED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      endPointMovedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the creation of a bend.
       * <p>
       * The default is {@link yfiles.input.MouseEventRecognizers#LEFT_RELEASED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      createBendRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the property that determines cancel events.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the property that determines bend removal events.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      removeBendRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the event recognizer that determines whether dynamic port candidates should be resolved.
       * <p>
       * By default this is {@link yfiles.input.KeyEventRecognizers#SHIFT_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      portCandidateResolutionRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the finishing gesture that is used to end the edge creation on the target node by a
       * touch gesture.
       * <p>
       * The default is {@link yfiles.input.TouchEventRecognizers#TOUCH_UP_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      finishRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the starting gesture that is used to prepare the edge creation on the source node
       * via touch.
       * <p>
       * The default is {@link yfiles.input.TouchEventRecognizers#TOUCH_LONG_PRESSED_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      prepareRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the initial dragging gesture that is used to initiate the creation after the
       * preparation via touch.
       * <p>
       * The default is {@link yfiles.input.TouchEventRecognizers#TOUCH_MOVED_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * The event recognizer that recognizes the creation of a bend via touch.
       * <p>
       * The default is {@link yfiles.input.TouchEventRecognizers#TOUCH_UP_PRIMARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      createBendRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the property that determines cancel events via touch.
       * <p>
       * The default is {@link yfiles.input.TouchEventRecognizers#TOUCH_MULTI_TAPPED_SECONDARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the property that determines bend removal events via touch.
       * <p>
       * The default is {@link yfiles.input.TouchEventRecognizers#TOUCH_LONG_PRESSED_SECONDARY}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      removeBendRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Determines whether or not to display possible port candidates during the creation of the edge.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      showPortCandidates:boolean;
      /**
       * Determines whether or not edges are allowed to connect to other edges.
       * <p>
       * The default is <code>false</code>.
       * </p>
       * @default false
       * @type {boolean}
       */
      allowEdgeToEdgeConnections:boolean;
      /**
       * Gets or sets the distance in the view coordinate system that determines how far the mouse should be snapped to a port
       * candidate if {@link yfiles.input.CreateEdgeInputMode#snapToTargetCandidate} is enabled.
       * <p>
       * The default value is <code>20</code>.
       * </p>
       * @default 20.0
       * @type {number}
       */
      snapDistance:number;
      /**
       * The {@link yfiles.input.CreateEdgeInputMode#snapContext} which manages snapping model items to certain coordinates (for instance, other items).
       * <p>
       * If set to <code>null</code> (the default) this input mode tries to obtain the {@link yfiles.input.CreateEdgeInputMode#snapContext} from the {@link yfiles.input.IInputModeContext}. To explicitly
       * disable snapping, a {@link yfiles.input.CreateEdgeInputMode#snapContext} implementation that does nothing has to be set to this instance.
       * </p>
       * @type {yfiles.input.SnapContext}
       */
      snapContext:yfiles.input.SnapContext;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} that is used for visualizing the
       * {@link yfiles.input.IPortCandidate}s in the canvas during the edit.
       * <p>
       * This descriptor is queried for the {@link yfiles.view.IVisualCreator} to obtain the visualizations for the candidates that are
       * not currently closest. By default an instance of {@link yfiles.view.DefaultPortCandidateDescriptor} is used.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#closestCandidateDescriptor
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      candidateDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the {@link yfiles.view.ICanvasObjectDescriptor} that is used for visualizing the closest {@link yfiles.input.IPortCandidate}
       * in the canvas during the edit.
       * <p>
       * This descriptor is queried for the {@link yfiles.view.IVisualCreator} to obtain the visualization for the currently closest
       * candidate. By default an instance of {@link yfiles.view.DefaultPortCandidateDescriptor} is used.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#candidateDescriptor
       * @type {yfiles.view.ICanvasObjectDescriptor}
       */
      closestCandidateDescriptor:yfiles.view.ICanvasObjectDescriptor;
      /**
       * Gets or sets the distance in the view coordinate system that determines how far the mouse should be snapped to a
       * horizontal or vertical line.
       * <p>
       * The default value is <code>5</code>.
       * </p>
       * @default 5.0
       * @type {number}
       */
      orthogonalSnapDistance:number;
      /**
       * Gets or sets a property that determines whether the mouse should be forced to snap to the nearest valid port candidate
       * if the mouse hovers over a target node.
       * <p>
       * If the nearest port candidate is farther away than {@link yfiles.input.CreateEdgeInputMode#snapDistance} and the mouse hovers over a valid {@link yfiles.input.CreateEdgeInputMode#getTarget target}, this property will cause
       * the mouse to snap to the nearest valid candidate. The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      forceSnapToCandidate:boolean;
      /**
       * Gets or sets a property that determines whether this mode should use only the {@link yfiles.input.CreateEdgeInputMode#getTargetPortCandidates target port candidates} of the node the
       * mouse currently hovers over.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#showPortCandidates
       * @see yfiles.input.CreateEdgeInputMode#getTarget
       * @see yfiles.input.CreateEdgeInputMode#getTargetPortCandidates
       * @default true
       * @type {boolean}
       */
      useHitItemsCandidatesOnly:boolean;
      /**
       * Gets or sets the the policy that controls whether edges will be create orthogonally.
       * <p>
       * By default orthogonal edge creation is enabled if orthogonal edge editing is enabled on the {@link yfiles.input.GraphEditorInputMode parent input mode}.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#orthogonalEdgeEditingContext
       * @see yfiles.input.OrthogonalEdgeEditingContext#enabled
       * @default yfiles.input.OrthogonalEdgeEditingPolicy.AUTO
       * @type {yfiles.input.OrthogonalEdgeEditingPolicy}
       */
      orthogonalEdgeCreation:yfiles.input.OrthogonalEdgeEditingPolicy;
      /**
       * Defines the preferred minimal distance of orthogonal edges.
       * <p>
       * This information is needed when deciding which way to route the first segment of an orthogonal edge.
       * </p>
       * @default 10.0
       * @type {number}
       */
      preferredMinimalEdgeDistance:number;
      /**
       * Gets or sets a value indicating whether {@link yfiles.input.PortCandidateValidity#DYNAMIC} ports that are part of the {@link yfiles.input.CreateEdgeInputMode#getSourcePortCandidate source port candidates}
       * should be resolved for the current mouse location.
       * <p>
       * By default this feature is enabled and the {@link yfiles.input.CreateEdgeInputMode#portCandidateResolutionRecognizer} is configured so that the resolution mechanism is used if the user holds the
       * shift modifier during the gesture.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#portCandidateResolutionRecognizer
       * @see yfiles.input.CreateEdgeInputMode#resolveCandidates
       * @default true
       * @type {boolean}
       */
      resolveSourcePortCandidates:boolean;
      /**
       * Gets or sets a value indicating whether {@link yfiles.input.PortCandidateValidity#DYNAMIC} ports that are part of the {@link yfiles.input.CreateEdgeInputMode#getTargetPortCandidates target port candidates}
       * should be resolved for the current mouse location.
       * <p>
       * By default this feature is enabled and the {@link yfiles.input.CreateEdgeInputMode#portCandidateResolutionRecognizer} is configured so that the resolution mechanism is used if the user holds the
       * shift modifier during the gesture.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#portCandidateResolutionRecognizer
       * @see yfiles.input.CreateEdgeInputMode#resolveCandidates
       * @default true
       * @type {boolean}
       */
      resolveTargetPortCandidates:boolean;
      /**
       * Defines the width of a node's border which is taken into account when deciding which way to route the first segment of
       * an orthogonal edge.
       * <p>
       * An edge starting inside a given margin is always pointing away from it.
       * </p>
       * @default 0.1
       * @type {number}
       */
      nodeBorderWidthRatio:number;
      /**
       * Gets or sets the ports based edge creation callback.
       * <p>
       * The default creator will delegate to {@link yfiles.graph.IGraph#createEdge} and obtain the tag from the {@link yfiles.input.CreateEdgeInputMode#dummyEdge}. It will then copy over
       * all bends, labels, and ports from the {@link yfiles.input.CreateEdgeInputMode#dummyEdge}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} If the argument is <code>null</code>
       * @type {function(yfiles.input.IInputModeContext, yfiles.graph.IGraph, yfiles.input.IPortCandidate, yfiles.input.IPortCandidate, yfiles.graph.IEdge): yfiles.graph.IEdge}
       */
      edgeCreator:(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,sourcePortCandidate:yfiles.input.IPortCandidate,targetPortCandidate:yfiles.input.IPortCandidate,templateEdge:yfiles.graph.IEdge)=>yfiles.graph.IEdge;
      /**
       * Determines whether or not to allow the creation of self loops.
       * <p>
       * The default is <code>true</code>, which enables that an edge is connected to a {@link yfiles.graph.IEdge#sourcePort} and {@link yfiles.graph.IEdge#targetPort} that have the same {@link yfiles.graph.IPort#owner}.
       * </p>
       * @default true
       * @type {boolean}
       */
      allowSelfloops:boolean;
      /**
       * Determines whether an edge creation gesture will be canceled when the gesture is ended on an invalid target and no bend
       * may be created there.
       * <p>
       * If bend creation is allowed at the target location (i.e. {@link yfiles.input.CreateEdgeInputMode#validBendHitTestable} returns true), the gesture will always continue by creating a
       * bend. Otherwise, if this property is <code>true</code>, the gesture will be canceled automatically.
       * </p>
       * <p>
       * The default is <code>false</code>, meaning that the gesture must always be canceled explicitely.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#allowCreateBend
       * @see yfiles.input.CreateEdgeInputMode#validBendHitTestable
       * @type {boolean}
       */
      cancelGestureOnInvalidTarget:boolean;
      /**
       * Determines whether or not to allow the creation of bends.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      allowCreateBend:boolean;
      /**
       * Gets the dummy edge instance that will be used to render a preview of the edge to be created.
       * <p>
       * This instance will be used for rendering the edge during the gesture. If this property is unset initially, factory
       * method {@link yfiles.input.CreateEdgeInputMode#createDummyEdge} will be called.
       * </p>
       * @type {yfiles.graph.IEdge}
       */
      dummyEdge:yfiles.graph.IEdge;
      /**
       * Gets the dummy graph instance that will hold the items that will render a preview of the newly created edge.
       * <p>
       * The dummy edge graph can be used to change the appearance and other properties of the dummy edge, either when starting a
       * gesture or during it in response to certain events. Possible actions are:
       * </p>
       * <ul>
       * <li>Changing the style of the dummy edge.</li>
       * <li>Adding bends to the dummy edge.</li>
       * <li>Adding labels to the dummy edge.</li>
       * <li>Changing style and other properties of the added labels.</li>
       * </ul>
       * <p>
       * All changes made to the dummy edge during the gesture will be persisted to the created edge in the original graph. This
       * behavior can be changed by setting a custom {@link yfiles.input.CreateEdgeInputMode#edgeCreator}.
       * </p>
       * <p>
       * The dummy edge graph only allows operations on the dummy edge.
       * </p>
       * <p>
       * If the property is not set initially, method {@link yfiles.input.CreateEdgeInputMode#createDummyEdgeGraph} will be used as the
       * factory method.
       * </p>
       * @see yfiles.input.CreateEdgeInputMode#dummyEdge
       * @type {yfiles.graph.IGraph}
       */
      dummyEdgeGraph:yfiles.graph.IGraph;
      /**
       * Convenience property for customizing the appearance of newly created edges.
       * <p>
       * This property directly delegates to the {@link yfiles.graph.IGraph#edgeDefaults} of the {@link yfiles.input.CreateEdgeInputMode#dummyEdgeGraph}.
       * </p>
       * @type {yfiles.graph.IEdgeDefaults}
       */
      edgeDefaults:yfiles.graph.IEdgeDefaults;
      /**
       * Gets or sets the candidate for the {@link yfiles.graph.IEdge#sourcePort} of the edge to be created.
       * <p>
       * This property will be populated automatically when edge creation has started.
       * </p>
       * @type {yfiles.input.IPortCandidate}
       */
      sourcePortCandidate:yfiles.input.IPortCandidate;
      /**
       * Gets or sets the current candidate for the {@link yfiles.graph.IEdge#targetPort} of the edge to be created.
       * @type {yfiles.input.IPortCandidate}
       */
      targetPortCandidate:yfiles.input.IPortCandidate;
      /**
       * Adds the given listener for the <code>SourcePortCandidateChanged</code> event that occurs when the value of the {@link yfiles.input.CreateEdgeInputMode#sourcePortCandidate} property changes.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeSourcePortCandidateChangedListener
       */
      addSourcePortCandidateChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.input.IPortCandidate>)=>void):void;
      /**
       * Removes the given listener for the <code>SourcePortCandidateChanged</code> event that occurs when the value of the {@link yfiles.input.CreateEdgeInputMode#sourcePortCandidate} property
       * changes.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addSourcePortCandidateChangedListener
       */
      removeSourcePortCandidateChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.input.IPortCandidate>)=>void):void;
      /**
       * Adds the given listener for the <code>TargetPortCandidateChanged</code> event that occurs when the value of the {@link yfiles.input.CreateEdgeInputMode#targetPortCandidate} property changes.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeTargetPortCandidateChangedListener
       */
      addTargetPortCandidateChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.input.IPortCandidate>)=>void):void;
      /**
       * Removes the given listener for the <code>TargetPortCandidateChanged</code> event that occurs when the value of the {@link yfiles.input.CreateEdgeInputMode#targetPortCandidate} property
       * changes.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addTargetPortCandidateChangedListener
       */
      removeTargetPortCandidateChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.input.IPortCandidate>)=>void):void;
      /**
       * Gets or sets whether after calls to {@link yfiles.input.CreateEdgeInputMode#createEdge} the current undo edit should be canceled
       * if the method returns <code>null</code>.
       * <p>
       * This should be set to <code>false</code> if you want to perform some undoable actions in these methods and/or in the {@link yfiles.input.CreateEdgeInputMode#edgeCreator} callback, but
       * need to return <code>null</code> from any of these methods. Default value is <code>true</code>, meaning that returning <code>null</code> results in canceling
       * the composite undo entry.
       * </p>
       * @default true
       * @type {boolean}
       */
      cancelEditImplicitly:boolean;
      /**
       * Adds the given listener for the <code>EdgeCreated</code> event that occurs after an edge has been created by this mode.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#onEdgeCreated
       * @see yfiles.input.CreateEdgeInputMode#removeEdgeCreatedListener
       */
      addEdgeCreatedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeCreated</code> event that occurs after an edge has been created by this mode.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#onEdgeCreated
       * @see yfiles.input.CreateEdgeInputMode#addEdgeCreatedListener
       */
      removeEdgeCreatedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PortAdded</code> event that occurs when this instance {@link yfiles.graph.IGraph#addPort adds a port} to the source or target node
       * during completion of the edge creation gesture.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removePortAddedListener
       */
      addPortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * Removes the given listener for the <code>PortAdded</code> event that occurs when this instance {@link yfiles.graph.IGraph#addPort adds a port} to the source or target
       * node during completion of the edge creation gesture.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addPortAddedListener
       */
      removePortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * Determines whether or not the edge's end point should snap to the current target port candidate during the creation of
       * the edge.
       * <p>
       * This will make the edge's end point point to the position of the target port candidate. This will result in the preview
       * edge looking exactly like the edge after creation.
       * </p>
       * @type {boolean}
       */
      snapToTargetCandidate:boolean;
      /**
       * Gets or sets a {@link yfiles.input.IHitTestable} that determines, whether it is valid to start an edge creation gesture here.
       * <p>
       * The default implementation returns <code>true</code> if {@link yfiles.input.CreateEdgeInputMode#getSource} yields a non-<code>null</code> value.
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      beginHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets or sets a {@link yfiles.input.IHitTestable} that determines, whether it is valid to finish an edge creation gesture here.
       * <p>
       * If fact this hit testable is only queried at all if {@link yfiles.input.CreateEdgeInputMode#forceSnapToCandidate} is disabled and if there is currently no {@link yfiles.input.CreateEdgeInputMode#targetPortCandidate} available.
       * </p>
       * <p>
       * The default implementation returns <code>true</code> if there is a node at the given location.
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      endHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets or sets a {@link yfiles.input.IHitTestable} that determines, whether it is valid to create a bend here.
       * <p>
       * The default implementation returns always <code>true</code>.
       * </p>
       * @type {yfiles.input.IHitTestable}
       */
      validBendHitTestable:yfiles.input.IHitTestable;
      /**
       * Gets a copy of the starting point at which the edge creation gesture started.
       * @type {yfiles.geometry.Point}
       */
      startPoint:yfiles.geometry.Point;
      /**
       * Gets the dynamically updated dragging point for the edge creation.
       * @type {yfiles.geometry.IPoint}
       */
      dragPoint:yfiles.geometry.IPoint;
      /**
       * Determines whether currently an edge creation is in progress.
       * <p>
       * This property yields <code>true</code> after the edge creation has been started and before it has been {@link yfiles.input.CreateEdgeInputMode#cancel cancelled} or finalized.
       * </p>
       * @type {boolean}
       */
      isCreationInProgress:boolean;
      /**
       * Adds the given listener for the <code>GestureFinishing</code> event that occurs before the gesture will be finished.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeGestureFinishingListener
       */
      addGestureFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GestureFinishing</code> event that occurs before the gesture will be finished.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addGestureFinishingListener
       */
      removeGestureFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GestureFinished</code> event that occurs once the gesture has been finished.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeGestureFinishedListener
       */
      addGestureFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GestureFinished</code> event that occurs once the gesture has been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addGestureFinishedListener
       */
      removeGestureFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GestureStarting</code> event that occurs once the gesture is starting.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeGestureStartingListener
       */
      addGestureStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GestureStarting</code> event that occurs once the gesture is starting.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addGestureStartingListener
       */
      removeGestureStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GestureStarted</code> event that occurs once the gesture is initialized and has started.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeGestureStartedListener
       */
      addGestureStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GestureStarted</code> event that occurs once the gesture is initialized and has started.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addGestureStartedListener
       */
      removeGestureStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Moving</code> event that occurs at the start of every drag or move.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeMovingListener
       */
      addMovingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Moving</code> event that occurs at the start of every drag or move.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addMovingListener
       */
      removeMovingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Moved</code> event that occurs at the end of every drag or move.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeMovedListener
       */
      addMovedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Moved</code> event that occurs at the end of every drag or move.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addMovedListener
       */
      removeMovedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GestureCanceled</code> event that occurs when the gesture has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeGestureCanceledListener
       */
      addGestureCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GestureCanceled</code> event that occurs when the gesture has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addGestureCanceledListener
       */
      removeGestureCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>GestureCanceling</code> event that occurs before the gesture will be canceled.
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeGestureCancelingListener
       */
      addGestureCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>GestureCanceling</code> event that occurs before the gesture will be canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addGestureCancelingListener
       */
      removeGestureCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeCreationStarted</code> event that occurs when the edge creation has started.
       * <p>
       * The edge that is stored in the {@link yfiles.collections.ItemEventArgs.<T>#item} property is the {@link yfiles.input.CreateEdgeInputMode#dummyEdge} that is used during creation.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.CreateEdgeInputMode#removeEdgeCreationStartedListener
       */
      addEdgeCreationStartedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeCreationStarted</code> event that occurs when the edge creation has started.
       * <p>
       * The edge that is stored in the {@link yfiles.collections.ItemEventArgs.<T>#item} property is the {@link yfiles.input.CreateEdgeInputMode#dummyEdge} that is used during creation.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.CreateEdgeInputMode#addEdgeCreationStartedListener
       */
      removeEdgeCreationStartedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Gets or sets the event recognizer that will be queried to decide if bend creation is enforced.
       * <p>
       * The default is {@link yfiles.input.KeyEventRecognizers#CONTROL_PRESSED} or {@link yfiles.input.KeyEventRecognizers#META_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      enforceBendCreationRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the Graph this mode is acting upon.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    export enum EdgeSegmentDirection{
      /**
       * The edge segment is going into north direction
       */
      NORTH,
      /**
       * The edge segment is going into east direction
       */
      EAST,
      /**
       * The edge segment is going into south direction
       */
      SOUTH,
      /**
       * The edge segment is going into west direction
       */
      WEST
    }
    export enum AdjustContentRectPolicy{
      /**
       * The content rectangle is not adjusted automatically after interactive changes.
       */
      NEVER,
      /**
       * The new content rectangle is recalculated with the union of the previous content rectangle and the new
       * {@link yfiles.view.GraphComponent#fitGraphBounds bounds of the graph}.
       */
      UNION,
      /**
       * The new content rectangle is recalculated with the new {@link yfiles.view.GraphComponent#fitGraphBounds bounds of the graph}.
       */
      ALWAYS
    }
    /**
     * A complex {@link yfiles.input.IInputMode} that can be used in a {@link yfiles.input.GraphEditorInputMode#graphComponent} to edit an {@link yfiles.graph.IGraph} instance.
     * <p>
     * This class delegates most of the work to minor {@link yfiles.input.IInputMode} implementations. However it contains a number of
     * properties that influence the general behavior:
     * </p>
     * <ul>
     * <li><b>{@link yfiles.input.GraphInputMode#selectableItems}</b> – Determines the types of items that can be selected at all.</li>
     * <li><b>{@link yfiles.input.GraphInputMode#selectablePredicate}</b> – Allows to further constrain {@link yfiles.input.GraphInputMode#selectableItems} with a predicate.</li>
     * <li><b>{@link yfiles.input.GraphInputMode#marqueeSelectableItems}</b> – Determines the types of items that can be selected with {@link yfiles.input.MarqueeSelectionInputMode marquee selection}.</li>
     * <li><b>{@link yfiles.input.GraphInputMode#clickSelectableItems}</b> – Determines the types of items that can be selected by {@link yfiles.input.ClickInputMode clicking them}.</li>
     * <li>
     * <b>{@link yfiles.input.GraphInputMode#detailSelectionRecognizer}</b> – Determines when to use a detail click selection mode, which allows for selecting the item that is directly under the
     * mouse without taking into account the {@link yfiles.input.GraphInputMode#clickHitTestOrder}.
     * </li>
     * <li><b>{@link yfiles.input.GraphInputMode#clickHitTestOrder}</b> – Determines the priorities for hit tests during mouse clicks to determine which items should get {@link yfiles.input.GraphInputMode#addItemClickedListener click events}.</li>
     * <li><b>{@link yfiles.input.GraphInputMode#doubleClickHitTestOrder}</b> – Determines the priorities for hit tests when {@link yfiles.input.GraphInputMode#addItemDoubleClickedListener double-clicking items}.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#movableItems}</b> – Determines the type of the items that can be {@link yfiles.input.GraphEditorInputMode#moveInputMode moved} with the mouse.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#showHandleItems}</b> – Determines the type of the items whose {@link yfiles.input.IHandle}s should be shown and movable by the {@link yfiles.input.GraphEditorInputMode#handleInputMode}.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#deletableItems}</b> – Determines the type of the items that can be deleted by {@link yfiles.input.GraphEditorInputMode#deleteSelection}.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#deletablePredicate}</b> – Allows to further constrain {@link yfiles.input.GraphEditorInputMode#deletableItems} with a predicate.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#labelEditableItems}</b> – Determines the type of the items that can have their labels edited.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowGroupingOperations}</b> – Determines whether this input mode should allow grouping operations at all.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowGroupSelection}</b> – Determines whether this input mode should allow {@link yfiles.input.GraphEditorInputMode#groupSelection grouping the current selection}.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowUngroupSelection}</b> – Determines whether this input mode should allow {@link yfiles.input.GraphEditorInputMode#ungroupSelection ungrouping the current selection}.</li>
     * <li>
     * <b>{@link yfiles.input.GraphEditorInputMode#allowClearSelection}</b> – Determines whether this input mode should allow {@link yfiles.input.GraphInputMode#clearSelection clearing the selection}. This has implications both for the {@link yfiles.input.ICommand#DESELECT_ALL}
     * and for click/marquee selection which usually clear the selection unless the {@link yfiles.input.GraphInputMode#multiSelectionRecognizer} is pressed.
     * </li>
     * <li><b>{@link yfiles.input.GraphInputMode#allowClipboardOperations}</b> – Determines whether this input mode should enable {@link yfiles.graph.GraphClipboard graph clipboard operations.}</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowDuplicate}</b> – Determines whether this input mode should enable {@link yfiles.graph.GraphClipboard graph clipboard's} {@link yfiles.graph.GraphClipboard#duplicate} operation.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowEditLabel}</b> – Determines whether this input mode should allow editing labels.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowReparentNodes}</b> – Determines whether this input mode should allow {@link yfiles.input.IReparentNodeHandler reparenting nodes to other groups.}</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowCreateNode}</b> – Determines whether this input mode should allow {@link yfiles.input.GraphEditorInputMode#nodeCreator creating new nodes} via simple mouse clicks.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowCreateEdge}</b> – Determines whether {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode} should be {@link yfiles.input.CreateEdgeInputMode#enabled} to allow for creating new edges.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowEditLabelOnDoubleClick}</b> – Determines whether labels should be editable on otherwise unhandled double clicks.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowReverseEdge}</b> – Determines whether edges can be reversed.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#allowCreateBend}</b> – Determines whether the {@link yfiles.input.GraphEditorInputMode#createBendInputMode} should be {@link yfiles.input.CreateBendInputMode#enabled} to allow for creating new bends.</li>
     * <li><b>{@link yfiles.input.NavigationInputMode#allowEnterGroup}</b> – Determines whether the mode allows for {@link yfiles.input.NavigationInputMode#enterGroup entering} group nodes in {@link yfiles.graph.IFoldingView}s.</li>
     * <li><b>{@link yfiles.input.NavigationInputMode#allowExitGroup}</b> – Determines whether the mode allows for {@link yfiles.input.NavigationInputMode#exitGroup exiting} the current group node in {@link yfiles.graph.IFoldingView}s.</li>
     * <li><b>{@link yfiles.input.NavigationInputMode#allowCollapseGroup}</b> – Determines whether the mode allows for collapsing group nodes in {@link yfiles.graph.IFoldingView}s.</li>
     * <li><b>{@link yfiles.input.NavigationInputMode#allowExpandGroup}</b> – Determines whether the mode allows for {@link yfiles.input.NavigationInputMode#expandGroup expanding} group nodes in {@link yfiles.graph.IFoldingView}s.</li>
     * <li>
     * <b>{@link yfiles.input.ItemHoverInputMode#hoverItems}</b> – Determines which items should be reported as being hovered through the {@link yfiles.input.ItemHoverInputMode#addHoveredItemChangedListener corresponding event} in this instance's
     * {@link yfiles.input.ItemHoverInputMode}.
     * </li>
     * <li><b>{@link yfiles.input.GraphInputMode#shouldInstallCommand}</b> – Determines which of the built-in {@link yfiles.input.ICommand}s this input mode should provide and install.</li>
     * </ul>
     * <p>
     * By default, {@link yfiles.input.GraphEditorInputMode} supports the following commands. Most of them have default keybindings:
     * </p>
     * <ul>
     * <li><b>{@link yfiles.input.ICommand#UNDO} (Ctrl+Z)</b> – Undoes the last operation. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowUndoOperations}.</li>
     * <li><b>{@link yfiles.input.ICommand#REDO} (Ctrl+Y)</b> – Redoes the last operation that was undone. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowUndoOperations}.</li>
     * <li><b>{@link yfiles.input.ICommand#SELECT_ALL} (Ctrl+A)</b> – Selects all items. Can be configured with {@link yfiles.input.GraphInputMode#selectableItems} and {@link yfiles.input.GraphInputMode#selectablePredicate}.</li>
     * <li><b>{@link yfiles.input.ICommand#DESELECT_ALL} (Ctrl+Shift+A)</b> – Clears the current selection. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowClearSelection}.</li>
     * <li><b>{@link yfiles.input.ICommand#CUT} (Ctrl+X, Shift+Del)</b> – Cuts the current selection and places it in the clipboard. Can be disabled with {@link yfiles.input.GraphInputMode#allowClipboardOperations}.</li>
     * <li><b>{@link yfiles.input.ICommand#COPY} (Ctrl+C, Ctrl+Ins)</b> – Copies the current selection to the clipboard. Can be disabled with {@link yfiles.input.GraphInputMode#allowClipboardOperations}.</li>
     * <li><b>{@link yfiles.input.ICommand#PASTE} (Ctrl+V, Shift+Ins)</b> – Pastes the current clipboard contents into the graph. Can be disabled with {@link yfiles.input.GraphInputMode#allowClipboardOperations}.</li>
     * <li><b>{@link yfiles.input.ICommand#DUPLICATE} (Ctrl+D)</b> – Duplicates the current selection. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowDuplicate}.</li>
     * <li><b>{@link yfiles.input.ICommand#DELETE} (Del)</b> – Deletes the current selection. Can be configured with {@link yfiles.input.GraphEditorInputMode#deletablePredicate} and {@link yfiles.input.GraphEditorInputMode#deletableItems}.</li>
     * <li><b>{@link yfiles.input.ICommand#ADD_LABEL} (Shift+F2)</b> – Adds a new label to the selected item. Can be configured with {@link yfiles.input.GraphEditorInputMode#labelEditableItems} and disabled with {@link yfiles.input.GraphEditorInputMode#allowAddLabel}.</li>
     * <li><b>{@link yfiles.input.ICommand#EDIT_LABEL} (F2)</b> – Edits an existing label or adds a new one to the selected item. Can be configured with {@link yfiles.input.GraphEditorInputMode#labelEditableItems} and disabled with {@link yfiles.input.GraphEditorInputMode#allowEditLabel}.</li>
     * <li><b>{@link yfiles.input.ICommand#REVERSE_EDGE} (Ctrl+R)</b> – Reverses the direction of the selected edges. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowReverseEdge}.</li>
     * <li><b>{@link yfiles.input.ICommand#GROUP_SELECTION} (Ctrl+G)</b> – Creates a new group node and places the currently selected nodes into it. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} and {@link yfiles.input.GraphEditorInputMode#allowGroupSelection}.</li>
     * <li><b>{@link yfiles.input.ICommand#UNGROUP_SELECTION} (Ctrl+U)</b> – Creates a new group node and places the currently selected nodes into it. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} and {@link yfiles.input.GraphEditorInputMode#allowUngroupSelection}.</li>
     * <li><b>{@link yfiles.input.ICommand#ADJUST_GROUP_NODE_SIZE} (Ctrl+Shift+G)</b> – Adjusts the sizes of the selected group nodes to fit exactly around their contents. Can be disabled with {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} and {@link yfiles.input.GraphEditorInputMode#allowAdjustGroupNodeSize}.</li>
     * </ul>
     * <p>
     * All commands can also be selectively disabled by using the {@link yfiles.input.GraphInputMode#availableCommands} collection. Furthermore, there are a few commands provided
     * by {@link yfiles.input.GraphInputMode#navigationInputMode}:
     * </p>
     * <ul>
     * <li><b>{@link yfiles.input.ICommand#COLLAPSE_GROUP} (Ctrl+Minus)</b> – Collapses the currently selected group nodes. Can be disabled with {@link yfiles.input.NavigationInputMode#allowCollapseGroup} of {@link yfiles.input.GraphInputMode#navigationInputMode}.</li>
     * <li><b>{@link yfiles.input.ICommand#EXPAND_GROUP} (Ctrl+Plus)</b> – Expands the currently selected group nodes. Can be disabled with {@link yfiles.input.NavigationInputMode#allowExpandGroup} of {@link yfiles.input.GraphInputMode#navigationInputMode}.</li>
     * <li>
     * <b>{@link yfiles.input.ICommand#TOGGLE_EXPANSION_STATE} (Ctrl+Multiply)</b> – Collapses expanded group nodes and expands collapsed group nodes in the current selection. Can be disabled with {@link yfiles.input.NavigationInputMode#allowCollapseGroup} and {@link yfiles.input.NavigationInputMode#allowExpandGroup} of
     * {@link yfiles.input.GraphInputMode#navigationInputMode}.
     * </li>
     * <li><b>{@link yfiles.input.ICommand#ENTER_GROUP} (Ctrl+Enter)</b> – Replaces the currently displayed graph with the contents of the selected group node. Can be disabled with {@link yfiles.input.NavigationInputMode#allowEnterGroup} of {@link yfiles.input.GraphInputMode#navigationInputMode}.</li>
     * <li>
     * <b>{@link yfiles.input.ICommand#EXIT_GROUP} (Ctrl+Backspace)</b> – Replaces the currently displayed graph with the contents of the parent of the group node whose contents are currently
     * displayed. Can be disabled with {@link yfiles.input.NavigationInputMode#allowExitGroup} of {@link yfiles.input.GraphInputMode#navigationInputMode}.
     * </li>
     * </ul>
     * <p>
     * {@link yfiles.input.GraphEditorInputMode} can be configured to automatically {@link yfiles.input.GraphEditorInputMode#adjustContentRect adjust the content rectangle} of the {@link yfiles.input.GraphEditorInputMode#graphComponent} whenever the
     * graph is changed using gestures.
     * <br />
     * This class contains a number of public methods that can be used to programmatically perform the corresponding actions,
     * too.
     * <br />
     * Setting the {@link yfiles.input.GraphEditorInputMode#nodeCreator} to another instance or <code>null</code> changes the way nodes are created. Also the various minor {@link yfiles.input.IInputMode}
     * instances can be disabled, to change the behavior, too.
     * </p>
     * <p>
     * This input mode manages a set of specialized input modes, each handling a specific part of the interaction with the
     * graph. The following list details those along with their default
     * {@link yfiles.input.IInputMode#priority priorities}. Lower priorities come earlier when handling events.
     * </p>
     * <ul>
     * <li><b>{@link yfiles.input.WaitInputMode} (−1)</b> – Disables this input mode when layouts or animations run.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#handleInputMode} (0)</b> – Deals with displaying handles on or around items and interacting with them.</li>
     * <li><b>{@link yfiles.input.KeyboardInputMode} (0)</b> – Handles commands and arbitrary keyboard shortcuts.</li>
     * <li><b>{@link yfiles.input.ClickInputMode} (10)</b> – Handles mouse clicks and double-clicks.</li>
     * <li><b>{@link yfiles.input.TapInputMode} (20)</b> – Handles taps and double-tap.</li>
     * <li><b>{@link yfiles.input.MoveViewportInputMode} (39)</b> – Allows panning the viewport.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} (39)</b> – Allows moving labels to new positions.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#moveInputMode} (40)</b> – Allows moving items that are not labels.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#createBendInputMode} (42)</b> – Allows creating bends on edges.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#createEdgeInputMode} (45)</b> – Allows creating edges</li>
     * <li><b>{@link yfiles.input.MarqueeSelectionInputMode} (50)</b> – Allows dragging a rectangle to select items within it.</li>
     * <li><b>{@link yfiles.input.ItemHoverInputMode} (55)</b> – Provides events to indicate that the mouse pointer hovers over an item.</li>
     * <li>
     * <b>{@link yfiles.input.NavigationInputMode} (55)</b> – Provides general navigation facilities, such as navigating from one item to another with the arrow keys, collapsing,
     * expanding, entering or exiting groups, as well as providing commands for those tasks.
     * </li>
     * <li><b>{@link yfiles.input.ContextMenuInputMode} (60)</b> – Handles querying the context menu contents based on a location as well as preparing the menu for display.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#nodeDropInputMode} (70)</b> – Allows dropping nodes on the {@link yfiles.input.GraphEditorInputMode#graphComponent}, e.g. for dragging items from a palette.</li>
     * <li><b>{@link yfiles.input.MouseHoverInputMode} (100)</b> – Handles tool tips.</li>
     * <li><b>{@link yfiles.input.GraphEditorInputMode#textEditorInputMode} (100)</b> – Handles editing labels.</li>
     * </ul>
     * <p>
     * To enable single selection mode, disable {@link yfiles.input.MarqueeSelectionInputMode} and set {@link yfiles.input.GraphInputMode#multiSelectionRecognizer} to
     * {@link yfiles.input.EventRecognizers#NEVER}. Remove {@link yfiles.input.ICommand#SELECT_ALL} and {@link yfiles.input.ICommand#TOGGLE_ITEM_SELECTION}
     * from {@link yfiles.input.GraphInputMode#availableCommands} and from {@link yfiles.input.GraphInputMode#navigationInputMode}.{@link yfiles.input.NavigationInputMode#availableCommands}. An example can be found in <code>demo.yfiles.input.singleselection</code>.
     * </p>
     * <p>
     * An overview of this input mode is given in the section {@link @DGUIDE_PREFIX@/interaction-geim.html Editing Graphs}. Additionally, specific support for graph
     * hierarchies is discussed in {@link @DGUIDE_PREFIX@/interaction-support.html#interaction-grouping Supported User Interactions: Grouping}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleEditorForm, Demo.yFiles.Graph.SimpleEditor</li>
     * <li>Demo: GraphEditorForm, Demo.yFiles.GraphEditor</li>
     * <li>Demo: InputModeForm, Demo.yFiles.Graph.InputModes</li>
     * <li>Demo: DataSetGraphForm, Demo.yFiles.Graph.DataSetGraph</li>
     * <li>Demo: TreeLayoutForm, Demo.yFiles.Graph.Tree</li>
     * <li>Demo: SingleSelectionWindow, Demo.yFiles.Graph.Input.SingleSelection</li>
     * <li>Tutorial: 01 Getting Started, step 06 Basic Interaction</li>
     * </ul>
     * @see yfiles.input.GraphEditorInputMode#createEdgeInputMode
     * @see yfiles.input.GraphEditorInputMode#createBendInputMode
     * @see yfiles.input.GraphEditorInputMode#moveLabelInputMode
     * @see yfiles.input.GraphEditorInputMode#nodeDropInputMode
     * @class
     * @extends {yfiles.input.GraphInputMode}
     */
    export interface GraphEditorInputMode extends yfiles.input.GraphInputMode{}
    export class GraphEditorInputMode {
      /**
       * Creates a new instance that is not bound to an existing graph or selection, initially.
       * <p>
       * The {@link yfiles.graph.IGraph} and {@link yfiles.view.IGraphSelection} instances are obtained later from the {@link yfiles.input.IInputModeContext}
       * once this mode gets {@link yfiles.input.GraphEditorInputMode#install installed} in a suitable canvas control.
       * </p>
       */
      constructor();
      /**
       * Adds a new label to the given item interactively.
       * <p>
       * This will activate the {@link yfiles.input.GraphEditorInputMode#textEditorInputMode} to let the user interactively enter the label's text and unless the user cancels label creation
       * the returned {@link yfiles.graphml.Future.<T>} will contain the label.
       * </p>
       * <p>
       * The text that the user enters may be {@link yfiles.input.GraphEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
       * </p>
       * <p>
       * This method works nearly identical to {@link yfiles.input.GraphEditorInputMode#createLabel}. The only difference is when either
       * the {@link yfiles.input.GraphEditorInputMode#addLabelAddingListener LabelAdding} event is handled, or <code>owner</code>
       * has an
       * {@link yfiles.input.IEditLabelHelper}, and via those instead of adding a label an existing one should be edited. This method
       * will edit that label instead of adding a new one.
       * </p>
       * @param owner The item to add a label to.
       * @returns A Promise that fulfills with the edited label instance. It will fulfill with <code>null</code> if {@link #addEditingCanceledListener editing was canceled} or the text
       * did not {@link #addValidateLabelTextListener validate}.
       */
      addLabel(owner:yfiles.graph.ILabelOwner):Promise<yfiles.graph.ILabel>;
      /**
       * Updates the {@link yfiles.view.CanvasComponent#contentRect} using {@link yfiles.view.CanvasComponent#updateContentRect} and {@link yfiles.input.GraphEditorInputMode#contentRectMargins}
       * <p>
       * This method is called whenever the content changes and takes the {@link yfiles.input.GraphEditorInputMode#adjustContentRectPolicy} into account.
       * </p>
       */
      adjustContentRect():void;
      /**
       * {@link yfiles.graph.IGraph#adjustGroupNodeLayout Adjusts the size} of the group nodes in the enumerable.
       * <p>
       * This action is executed in response to the {@link yfiles.input.ICommand#ADJUST_GROUP_NODE_SIZE} if {@link yfiles.input.GraphEditorInputMode#allowAdjustGroupNodeSize} is set to <code>true</code>.
       * </p>
       * @param nodes The nodes to adjust their sizes.
       * @protected
       */
      adjustGroupNodeSize(nodes:yfiles.collections.IEnumerable<yfiles.graph.INode>):void;
      /**
       * {@link yfiles.graph.IGraph#adjustGroupNodeLayout Adjusts the size} of the selected group nodes.
       * <p>
       * This method delegates to {@link yfiles.input.GraphEditorInputMode#adjustGroupNodeSize} with the {@link yfiles.view.IGraphSelection#selectedNodes currently selected nodes}.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#adjustGroupNodeSize
       * @see yfiles.input.ICommand#ADJUST_GROUP_NODE_SIZE
       */
      adjustGroupNodeSizes():void;
      /**
       * Adjusts the node's {@link yfiles.graph.INode#layout} to adhere to possible {@link yfiles.input.INodeSizeConstraintProvider size constraints}.
       * <p>
       * This method will use the {@link yfiles.input.INodeSizeConstraintProvider} for {@link yfiles.graph.INode} that can be found in their {@link yfiles.graph.ILookup}
       * to make sure they are still valid. The actual work of this method will be delegated to
       * {@link yfiles.input.GraphEditorInputMode#setNodeLayout}.
       * </p>
       * @param node The node to possibly adjust the size of.
       */
      adjustToSizeConstraints(node:yfiles.graph.INode):void;
      /**
       * Creates a node on click.
       * <p>
       * This method is only called if no item has been and {@link yfiles.input.GraphInputMode#clickClearSelection} returned <code>false</code>.
       * </p>
       * @param context The input mode context.
       * @param location The location of the click.
       * @returns <code>true</code> if a node was successfully created, otherwise <code>false</code>.
       * @see yfiles.input.GraphEditorInputMode#shouldClickCreateNode
       * @protected
       */
      clickCreateNode(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Closes the label editor if it is currently open.
       * @param cancel Whether to cancel the editor and discard changes.
       */
      closeLabelEditor(cancel:boolean):void;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#createBendInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphEditorInputMode#createBendInputMode} property.
       * </p>
       * @returns a new instance of <code>CreateBendInputMode</code>
       * @protected
       */
      createCreateBendInputMode():yfiles.input.CreateBendInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode} property.
       * </p>
       * @returns a new instance of CreateEdgeInputMode
       * @protected
       */
      createCreateEdgeInputMode():yfiles.input.CreateEdgeInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#handleInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphEditorInputMode#handleInputMode} property.
       * </p>
       * @returns a new instance of HandleInputMode
       * @protected
       */
      createHandleInputMode():yfiles.input.HandleInputMode;
      /**
       * Interactively creates a new label for the provided label owner.
       * <p>
       * This method will invoke the text editor that will let the user edit the text of the label. If the user commits the label
       * text the label will be added to the label owner.
       * </p>
       * <p>
       * The text that the user enters may be {@link yfiles.input.GraphEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
       * </p>
       * <p>
       * This method works nearly identical to {@link yfiles.input.GraphEditorInputMode#addLabel}. The only difference is when either the
       * {@link yfiles.input.GraphEditorInputMode#addLabelAddingListener LabelAdding} event is handled, or <code>owner</code>
       * has an
       * {@link yfiles.input.IEditLabelHelper}, and via those instead of adding a label an existing one should be edited. This method
       * will do nothing instead of editing an existing label or adding a new one in that case.
       * </p>
       * @param owner The item to create a new label for.
       * @returns A Promise that fulfills with the edited label instance. It will fulfill with <code>null</code> if {@link #addEditingCanceledListener editing was canceled} or the text
       * did not {@link #addValidateLabelTextListener validate}.
       */
      createLabel(owner:yfiles.graph.ILabelOwner):Promise<yfiles.graph.ILabel>;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#moveInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.MoveInputMode} property.
       * </p>
       * @returns a new instance of MoveInputMode
       * @protected
       */
      createMoveInputMode():yfiles.input.MoveInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createMoveLabelInputMode():yfiles.input.MoveLabelInputMode;
      /**
       * Helper method that can be used to create a {@link yfiles.input.MoveInputMode} instance that can be used to move unselected model
       * items.
       * <p>
       * In order to use this mode, the created instance should be installed in a {@link yfiles.input.GraphEditorInputMode} like this but
       * using a smaller {@link yfiles.input.IInputMode#priority priority} value than the {@link yfiles.input.IInputMode#priority priority} of the mode that is {@link yfiles.input.GraphEditorInputMode#moveInputMode installed by default}.
       * </p>
       * @param recognizer Use this instance to make this mode work under certain conditions only. <code>null</code> for default behavior, one of the {@link }
       * constants, e.g. for alternate behavior.
       * @returns A {@link #moveInputMode} instance that uses the {@link } instances of the items that are hit at the position of the
       * cursor, regardless of their selection state.
       */
      createMoveUnselectedInputMode(recognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean):yfiles.input.MoveInputMode;
      /**
       * Creates a node given a certain click point.
       * <p>
       * This implementation delegates to the current node creation callback or simply returns <code>null</code> if there is no such callback
       * or {@link yfiles.input.GraphEditorInputMode#allowCreateNode} yields <code>false</code>. Finally, it tries to {@link yfiles.input.GraphInputMode#setCurrentItem set the node as the current item}.
       * </p>
       * @param location The point where the mouse had been clicked.
       * @returns The newly created node or <code>null</code>.
       * @see yfiles.input.GraphEditorInputMode#onNodeCreated
       * @see yfiles.input.GraphInputMode#setCurrentItem
       */
      createNode(location:yfiles.geometry.Point):yfiles.graph.INode;
      /**
       * Factory method that creates the {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode} lazily the first time the property is accessed.
       * @returns A plain new instance of the {@link } type, which is initially {@link #enabled disabled}.
       * @protected
       */
      createNodeDropInputMode():yfiles.input.NodeDropInputMode;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler} property.
       * </p>
       * @returns a new instance of {@link #reparentNodeHandler}
       * @protected
       */
      createReparentNodeHandler():yfiles.input.IReparentNodeHandler;
      /**
       * Factory method for the {@link yfiles.input.GraphEditorInputMode#textEditorInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.GraphEditorInputMode#textEditorInputMode} property.
       * </p>
       * @returns a new instance of TextEditorInputMode
       * @protected
       */
      createTextEditorInputMode():yfiles.input.TextEditorInputMode;
      /**
       * Cuts and inserts the currently selected elements to the clipboard.
       */
      cut():void;
      /**
       * This method deletes the currently selected elements.
       * <p>
       * If the {@link yfiles.view.GraphSelection} is non-empty this implementation triggers the {@link yfiles.input.GraphEditorInputMode#addDeletingSelectionListener DeletingSelection}
       * event, possibly a number of {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * events, and a final {@link yfiles.input.GraphEditorInputMode#deleteSelection} event. Note that {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * will not be called for items that are removed implicitly, e.g. if the event is triggered for a node, its labels, ports,
       * and adjacent edges will not be reported separately.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#addDeletingSelectionListener
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.input.GraphEditorInputMode#addDeletedSelectionListener
       */
      deleteSelection():void;
      /**
       * Initiates the {@link yfiles.input.GraphEditorInputMode#handleInputMode} to drag the given bend.
       * @param bend The bend to drag.
       * @see yfiles.input.GraphEditorInputMode#onCreateBendInputModeBendCreated
       */
      dragBend(bend:yfiles.graph.IBend):void;
      /**
       * Duplicates the selected elements.
       */
      duplicateSelection():void;
      /**
       * Starts editing the given label.
       * <p>
       * This implementation uses the {@link yfiles.input.TextEditorInputMode} to display an editor to edit the label. The text that the
       * user enters may be {@link yfiles.input.GraphEditorInputMode#addValidateLabelTextListener validated} before the label is actually edited.
       * </p>
       * <p>
       * The label to edit can be a
       * {@link yfiles.graph.SimpleLabel}, which will be edited as usual, except that the label text changes will be written directly
       * back into the instance.
       * </p>
       * <p>
       * This method will raise the {@link yfiles.input.GraphEditorInputMode#addLabelEditingListener LabelEditing}
       * event and query the {@link yfiles.input.IEditLabelHelper} for the label and its owner, but will ignore the result except for the
       * {@link yfiles.input.LabelEditingEventArgs#textEditorInputModeConfigurator} property on the {@link yfiles.input.LabelEditingEventArgs}. Thus editing a label via this method cannot be prevented by event
       * handlers or {@link yfiles.input.IEditLabelHelper}s.
       * </p>
       * @param label The label to edit.
       * @returns A Promise that fulfills with the edited label instance. It will also fulfill with the instance if editing is {@link #addEditingCanceledListener editing was canceled}
       * or the text did not {@link #addValidateLabelTextListener validate} and the edit was reverted automatically.
       * @see yfiles.input.GraphEditorInputMode#onLabelTextEdited
       */
      editLabel(label:yfiles.graph.ILabel):Promise<yfiles.graph.ILabel>;
      /**
       * Starts label editing by executing {@link yfiles.input.ICommand#EDIT_LABEL }.
       * <p>
       * The method should return whether the request was satisfied.
       * </p>
       * @param item The item whose label or the label itself that should be edited upon the double click gesture.
       * @returns <code>true</code> iff the request was handled.
       * @see yfiles.input.GraphInputMode#doubleClick
       * @see yfiles.input.GraphEditorInputMode#allowEditLabelOnDoubleClick
       * @protected
       */
      editLabelOnDoubleClick(item:yfiles.graph.IModelItem):boolean;
      /**
       * Creates a new group for all of the currently selected elements.
       * <p>
       * This method will also clear the selection and select the newly created group node.
       * </p>
       * <p>
       * The default shortcut for this is Ctrl+G.
       * </p>
       * @returns The newly created group node or <code>null</code>.
       * @see yfiles.input.ICommand#GROUP_SELECTION
       * @see yfiles.input.GraphInputMode#clearSelection
       * @see yfiles.input.GraphInputMode#setSelected
       */
      groupSelection():yfiles.graph.INode;
      /**
       * Callback that is invoked if the shift+F2 key is pressed and {@link yfiles.input.GraphEditorInputMode#allowAddLabel} is set to <code>true</code>.
       * <p>
       * This method determines the label owner to add to and delegates to {@link yfiles.input.GraphEditorInputMode#createLabel}.
       * </p>
       * @returns 
       * @protected
       */
      onAddLabel():boolean;
      /**
       * Callback that is called as soon as {@link yfiles.input.CreateBendInputMode} created a new bend.
       * <p>
       * This method selects the bend and starts the {@link yfiles.input.GraphEditorInputMode#handleInputMode} using the handle for the newly created bend. The bend is created using the
       * hit edge's {@link yfiles.input.IBendCreator} implementation, as obtained from the edge's {@link yfiles.graph.ILookup#lookup} and calls
       * {@link yfiles.input.GraphEditorInputMode#dragBend}. If the subsequent drag is canceled by the user, this call will remove the
       * bend again.
       * </p>
       * @param sender
       * @param evt
       * @protected
       */
      onCreateBendInputModeBendCreated(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#createBendInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onCreateBendInputModeChanged(oldMode:yfiles.input.CreateBendInputMode,newMode:yfiles.input.CreateBendInputMode):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onCreateEdgeInputModeChanged(oldMode:yfiles.input.CreateEdgeInputMode,newMode:yfiles.input.CreateEdgeInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem} event.
       * @param evt The instance containing the event data.
       * @protected
       */
      onDeletedItem(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addDeletedSelectionListener DeletedSelection} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onDeletedSelection(evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addDeletingSelectionListener DeletingSelection} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onDeletingSelection(evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Callback to be invoked after an edge's source and/or target ports have been changed as the result of an input gesture.
       * <p>
       * Dispatches the {@link yfiles.input.GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event.
       * </p>
       * @param evt The {@link } for the
       * {@link #addEdgePortsChangedListener EdgePortsChanged}.
       * @protected
       */
      onEdgePortsChanged(evt:yfiles.graph.EdgeEventArgs):void;
      /**
       * Callback that is invoked if the F2 key is pressed and {@link yfiles.input.GraphEditorInputMode#allowEditLabel} is set to <code>true</code>.
       * <p>
       * This method determines the label to edit and delegates to either {@link yfiles.input.GraphEditorInputMode#editLabel} or {@link yfiles.input.GraphEditorInputMode#createLabel}
       * if no label could be found.
       * </p>
       * @returns 
       * @protected
       */
      onEditLabel():boolean;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#graphComponent} property changes.
       * @param oldGraphComponent The old control.
       * @param newGraphComponent The new control.
       * @protected
       */
      onGraphComponentChanged(oldGraphComponent:yfiles.view.GraphComponent,newGraphComponent:yfiles.view.GraphComponent):void;
      /**
       * Called when the {@link yfiles.view.GraphSelection} property changes.
       * <p>
       * This will update the internal state of the modes.
       * </p>
       * @param oldSelection The old selection instance.
       * @param newSelection The new selection instance.
       * @protected
       */
      onGraphSelectionChanged(oldSelection:yfiles.view.IGraphSelection,newSelection:yfiles.view.IGraphSelection):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#handleInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onHandleInputModeChanged(oldMode:yfiles.input.HandleInputMode,newMode:yfiles.input.HandleInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addLabelAddedListener LabelAdded} event.
       * @param evt The {@link } instance that contains the {@link } that has been added.
       * @protected
       */
      onLabelAdded(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addLabelAddingListener LabelAdding} event.
       * <p>
       * Invoking the event handlers stops once one event marks the <code>args</code> as handled.
       * </p>
       * @param evt The event arguments.
       * @protected
       */
      onLabelAdding(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addLabelEditingListener LabelEditing} event.
       * <p>
       * Invoking the event handlers stops once one event marks the <code>args</code> as handled.
       * </p>
       * @param evt The event arguments.
       * @protected
       */
      onLabelEditing(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#labelSnapContext} property context changed.
       * @param oldContext The old snap context.
       * @param newContext The new snap context.
       * @protected
       */
      onLabelSnapContextChanged(oldContext:yfiles.input.SnapContext,newContext:yfiles.input.SnapContext):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addLabelTextChangedListener LabelTextChanged} event.
       * @param evt The {@link } instance that contains the {@link } that has changed the text.
       * @protected
       */
      onLabelTextChanged(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Called when the text of a label has been {@link yfiles.input.GraphEditorInputMode#editLabel edited}.
       * <p>
       * This method {@link yfiles.input.GraphEditorInputMode#addValidateLabelTextListener validates the label text} and if successful calls {@link yfiles.input.GraphEditorInputMode#setLabelText}.
       * </p>
       * @param label The label that was edited.
       * @param text The new text.
       * @protected
       */
      onLabelTextEdited(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addLabelTextEditingCanceledListener LabelTextEditingCanceled}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onLabelTextEditingCanceled(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addLabelTextEditingStartedListener LabelTextEditingStarted}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onLabelTextEditingStarted(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#moveInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onMoveInputModeChanged(oldMode:yfiles.input.MoveInputMode,newMode:yfiles.input.MoveInputMode):void;
      /**
       * Called when {@link yfiles.input.MoveInputMode#addDragCanceledListener DragCanceled} is triggered.
       * @param sender
       * @param evt
       * @protected
       */
      onMoveInputModeDragCanceled(sender:Object,evt:yfiles.lang.EventArgs):void;
      /**
       * Called when {@link yfiles.input.MoveInputMode#addDragFinishedListener DragFinished} is triggered.
       * @param sender
       * @param evt
       * @protected
       */
      onMoveInputModeDragFinished(sender:Object,evt:yfiles.lang.EventArgs):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onMoveLabelInputModeChanged(oldMode:yfiles.input.MoveLabelInputMode,newMode:yfiles.input.MoveLabelInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addNodeCreatedListener NodeCreated} event.
       * <p>
       * Note that if {@link yfiles.input.NodeDropInputMode#allowFolderNodeAsParent} is set to <code>true</code>, the reported node can actually be part of the {@link yfiles.graph.IFoldingView master graph}.
       * </p>
       * @param evt The {@link } instance containing the created node.
       * @protected
       */
      onNodeCreated(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onNodeDropInputModeChanged(oldMode:yfiles.input.NodeDropInputMode,newMode:yfiles.input.NodeDropInputMode):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode}'s {@link yfiles.input.ItemDropInputMode.<T>#addItemCreatedListener ItemCreated} event is triggered.
       * <p>
       * This implementation {@link yfiles.input.GraphEditorInputMode#adjustContentRect adjusts the content rect}, triggers the {@link yfiles.input.GraphEditorInputMode#addNodeCreatedListener NodeCreated}
       * event and tries to set the newly created node as the {@link yfiles.input.GraphInputMode#setCurrentItem current item}.
       * </p>
       * @param sender The sender.
       * @param evt The event argument instance containing the event data.
       * @protected
       */
      onNodeDropInputModeNodeCreated(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addNodeReparentedListener NodeReparented} event.
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.input.GraphEditorInputMode#reparentNodeHandler
       * @protected
       */
      onNodeReparented(evt:yfiles.graph.NodeEventArgs):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onReparentNodeHandlerChanged(oldMode:yfiles.input.IReparentNodeHandler,newMode:yfiles.input.IReparentNodeHandler):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#snapContext} property context changed.
       * @param oldContext The old snap context.
       * @param newContext The new snap context.
       * @protected
       */
      onSnapContextChanged(oldContext:yfiles.input.SnapContext,newContext:yfiles.input.SnapContext):void;
      /**
       * Called when the {@link yfiles.input.GraphEditorInputMode#textEditorInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onTextEditorInputModeChanged(oldMode:yfiles.input.TextEditorInputMode,newMode:yfiles.input.TextEditorInputMode):void;
      /**
       * Raises the {@link yfiles.input.GraphEditorInputMode#addValidateLabelTextListener ValidateLabelText} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onValidateLabelText(evt:yfiles.input.LabelTextValidatingEventArgs):void;
      /**
       * Pastes the current clipboard contents.
       * @see yfiles.input.GraphEditorInputMode#pasteAtLocation
       */
      paste():void;
      /**
       * Pastes the current clipboard contents at the given location.
       * <p>
       * The location is at the center of the bounds of the items that are pasted. This method reconfigures {@link yfiles.graph.GraphClipboard#pasteDelta} and calls
       * {@link yfiles.input.GraphEditorInputMode#paste}.
       * </p>
       * @param location The location at which the center of the clipboard's contents should be positioned.
       */
      pasteAtLocation(location:yfiles.geometry.Point):void;
      /**
       * Delegates to the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.UndoEngine}'s {@link yfiles.graph.UndoEngine#redo} method.
       */
      redo():void;
      /**
       * Trigger a reevaluation of all visible handles
       * <p>
       * This method is called when the {@link yfiles.input.GraphEditorInputMode#showHandleItems} property is changed.
       * </p>
       * @protected
       */
      requeryHandles():void;
      /**
       * Reverses the given edges.
       * <p>
       * The edge will only be reversed if {@link yfiles.input.GraphEditorInputMode#allowReverseEdge} is set to <code>true</code> and {@link yfiles.input.GraphEditorInputMode#shouldReverseEdge} returns <code>true</code>
       * for the edge. A {@link yfiles.input.GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event
       * will be dispatched after the edge has been reversed.
       * </p>
       * @param edge
       */
      reverseEdge(edge:yfiles.graph.IEdge):void;
      /**
       * Reverses the given edges.
       * <p>
       * The edges will only be reversed if {@link yfiles.input.GraphEditorInputMode#allowReverseEdge} is set to <code>true</code> and {@link yfiles.input.GraphEditorInputMode#shouldReverseEdge} returns <code>true</code>
       * for the edge.
       * </p>
       * <p>
       * For each edge a {@link yfiles.input.GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event
       * will be dispatched.
       * </p>
       * @param edges
       * @see yfiles.input.ICommand#REVERSE_EDGE
       */
      reverseEdges(edges:yfiles.collections.IEnumerable<yfiles.graph.IEdge>):void;
      /**
       * Reverses the selected edges.
       * <p>
       * This implementation invokes {@link yfiles.input.GraphEditorInputMode#reverseEdges} providing the {@link yfiles.view.GraphSelection selected edges}.
       * </p>
       * <p>
       * The edges will only be reversed if {@link yfiles.input.GraphEditorInputMode#shouldReverseEdge} returns <code>true</code> for each edge.
       * </p>
       * <p>
       * For each edge an {@link yfiles.input.GraphEditorInputMode#addEdgePortsChangedListener EdgePortsChanged} event
       * will be dispatched.
       * </p>
       */
      reverseSelectedEdges():void;
      /**
       * Selects the node and possible bends of selfloop edges.
       * <p>
       * This method delegates to {@link yfiles.input.GraphInputMode#setSelected} for the node and all adjacent self loop bends.
       * </p>
       * @param node The node to select.
       */
      selectNodeAndSelfloopBends(node:yfiles.graph.INode):void;
      /**
       * Sets the label's text after it has been {@link yfiles.input.GraphEditorInputMode#editLabel edited}.
       * <p>
       * Depending on {@link yfiles.input.GraphEditorInputMode#autoRemoveEmptyLabels} it will either set the label's text or remove it from the graph. Also this method will query the label's {@link yfiles.graph.ILabel#owner}
       * for an {@link yfiles.input.INodeSizeConstraintProvider} if the owner is an {@link yfiles.graph.INode} and will make sure that the size
       * constraints are still kept if the {@link yfiles.graph.ILabel#preferredSize} changes due to the edit by calling
       * {@link yfiles.input.GraphEditorInputMode#adjustToSizeConstraints}.
       * </p>
       * @param label The label to set the text or remove.
       * @param text The new text.
       */
      setLabelText(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Updates the bounds of the node.
       * <p>
       * This method differs from a simple call to {@link yfiles.graph.IGraph#setNodeLayout} in a number of ways:
       * </p>
       * <ul>
       * <li>An undo unit is enqueued.</li>
       * <li>
       * The node is reshaped interactively using the {@link yfiles.input.IReshapeHandler} implementation found in its lookup. This
       * enables correctly adjusting e.g. affected orthogonal edges and parent group nodes.
       * </li>
       * <li>{@link yfiles.input.GraphEditorInputMode#adjustContentRect} is called.</li>
       * </ul>
       * @param node The node to reshape.
       * @param layout The new node layout.
       */
      setNodeLayout(node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Predicate that determines whether a label may be added interactively to <code>item</code>
       * <p>
       * This is overridden if the item has a {@link yfiles.input.IEditLabelHelper} that returns <code>false</code> for its {@link yfiles.input.IEditLabelHelper#onLabelAdding}
       * method.
       * </p>
       * @param item The item to query
       * @returns <code>true</code> iff a label may be added
       * @protected
       */
      shouldAddLabel(item:yfiles.graph.IModelItem):boolean;
      /**
       * Specifies whether a node should be created by
       * {@link yfiles.input.ClickInputMode#addClickedListener Clicked}.
       * <p>
       * This implementation returns false when {@link yfiles.input.ContextMenuInputMode#swallowCloseClick} is <code>true</code> and the click has closed a context menu.
       * </p>
       * @param context The input mode context.
       * @param clickPoint The location of the click.
       * @returns Whether {@link #clickCreateNode} should be called to create a node.
       * @see yfiles.input.GraphEditorInputMode#clickCreateNode
       * @protected
       */
      shouldClickCreateNode(context:yfiles.input.IInputModeContext,clickPoint:yfiles.geometry.Point):boolean;
      /**
       * Callback method that determines whether the given item should be deleted during
       * {@link yfiles.input.GraphEditorInputMode#deleteSelection}.
       * <p>
       * This implementation uses and honors the {@link yfiles.input.GraphEditorInputMode#deletableItems} and {@link yfiles.input.GraphEditorInputMode#deletablePredicate} properties.
       * </p>
       * @param item The item.
       * @returns Whether to delete that item.
       * @protected
       */
      shouldDelete(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback method that determines whether the label or the labels of the provided item should be edited in response to {@link yfiles.input.GraphEditorInputMode#onEditLabel}
       * or {@link yfiles.input.GraphEditorInputMode#onAddLabel}.
       * @param item The item.
       * @returns Whether to edit the label or the labels for that item.
       * @protected
       */
      shouldEditLabel(item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether or not moving the item is allowed.
       * <p>
       * This predicate determines whether an item can be moved by <b>any</b> appropriate input mode, e.g by default it also evaluates to <code>true</code>
       * for {@link yfiles.graph.ILabel}s which are moved by {@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} instead of {@link yfiles.input.GraphEditorInputMode#moveInputMode}.
       * </p>
       * <p>
       * This implementation returns the result of the {@link yfiles.input.GraphEditorInputMode#movableItems} property for the given item.
       * </p>
       * @param item The item to check.
       * @returns Whether or not moving the item is allowed.
       * @see yfiles.input.GraphEditorInputMode#movableItems
       * @see yfiles.input.GraphEditorInputMode#moveInputMode
       * @see yfiles.input.GraphEditorInputMode#moveLabelInputMode
       * @protected
       */
      shouldMove(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback which determines whether the given edge should be reversed by {@link yfiles.input.GraphEditorInputMode#reverseEdge},
       * {@link yfiles.input.GraphEditorInputMode#reverseEdges}, or {@link yfiles.input.GraphEditorInputMode#reverseEdges}.
       * <p>
       * This implementation returns the value of {@link yfiles.input.GraphEditorInputMode#allowReverseEdge}.
       * </p>
       * @param edge The edge which should be reversed.
       * @returns <code>true</code> if the edge should be reversed.
       * @protected
       */
      shouldReverseEdge(edge:yfiles.graph.IEdge):boolean;
      /**
       * Can be overridden to determine which items should be selected after {@link yfiles.input.GraphEditorInputMode#paste} or
       * {@link yfiles.input.GraphEditorInputMode#duplicateSelection}.
       * <p>
       * This implementation returns <code>true</code> for the {@link yfiles.input.GraphEditorInputMode#pasteSelectableItems} items.
       * </p>
       * @param item The item to check.
       * @returns Whether to select the item after {@link #paste} or
       * {@link #duplicateSelection}.
       * @protected
       */
      shouldSelectAfterPaste(item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether to show the {@link yfiles.input.GraphEditorInputMode#handleInputMode handles} for the given item.
       * @param item The item to check
       * @returns Whether to show the item based on the setting of the corresponding {@link #showHandleItems}, property.
       * @protected
       */
      shouldShowHandles(item:yfiles.graph.IModelItem):boolean;
      /**
       * Snaps the node to the grid using the {@link yfiles.input.IGridConstraintProvider.<T>} for {@link yfiles.graph.INode}s queried from the given context.
       * <p>
       * This method is called by this instance whenever new nodes are created using the UI.
       * </p>
       * @param context
       * @param node
       * @see yfiles.input.MultiplexingInputMode#createChildInputModeContext
       */
      snapToGrid(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode):void;
      /**
       * Delegates to the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.UndoEngine}'s {@link yfiles.graph.UndoEngine#undo} method.
       */
      undo():void;
      /**
       * Makes each of the currently selected nodes a direct child of the root group.
       * <p>
       * The default shortcut for this is Ctrl+U.
       * </p>
       * @see yfiles.input.ICommand#UNGROUP_SELECTION
       */
      ungroupSelection():void;
      /**
       * Gets or sets the orthogonal edge editing context.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * <p><b>Related demos:</b></p>
       * <ul>
       * <li>Tutorial: 01 Getting Started, step 17 Orthogonal Edge Creation</li>
       * </ul>
       * @default null
       * @type {yfiles.input.OrthogonalEdgeEditingContext}
       */
      orthogonalEdgeEditingContext:yfiles.input.OrthogonalEdgeEditingContext;
      /**
       * Gets or sets the policy that controls whether removing bends from an orthogonal edge will keep the edge orthogonal.
       * <p>
       * By default this feature is enabled automatically if an {@link yfiles.input.GraphEditorInputMode#orthogonalEdgeEditingContext} is set.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#orthogonalEdgeEditingContext
       * @default yfiles.input.OrthogonalEdgeEditingPolicy.AUTO
       * @type {yfiles.input.OrthogonalEdgeEditingPolicy}
       */
      orthogonalBendRemoval:yfiles.input.OrthogonalEdgeEditingPolicy;
      /**
       * Gets or sets the {@link yfiles.input.IReparentNodeHandler} that is used for moving nodes out of group nodes or into other group
       * nodes.
       * <p>
       * This setting has no effect if reparenting nodes is generally disallowed by setting {@link yfiles.input.GraphEditorInputMode#allowReparentNodes} to <code>false</code>.
       * </p>
       * <p>
       * Child input modes will use this {@link yfiles.input.IReparentNodeHandler} as well, since it is added to
       * {@link yfiles.input.GraphEditorInputMode#childInputModeContextLookup}.
       * </p>
       * <p>
       * If not already initialized, the factory method {@link yfiles.input.GraphEditorInputMode#createReparentNodeHandler} will be
       * called on first access. Upon changes, the {@link yfiles.input.GraphEditorInputMode#onReparentNodeHandlerChanged} method will be
       * called.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowReparentNodes
       * @see yfiles.input.ReparentNodeHandler
       * @type {yfiles.input.IReparentNodeHandler}
       */
      reparentNodeHandler:yfiles.input.IReparentNodeHandler;
      /**
       * Gets or sets the hitTester property.
       * <p>
       * The enumerator will be queried for hit tests if the user clicks on the canvas.
       * </p>
       * @type {yfiles.input.IHitTester.<yfiles.graph.IModelItem>}
       */
      hitTester:yfiles.input.IHitTester<yfiles.graph.IModelItem>;
      /**
       * Determines which types of items should be selected after {@link yfiles.input.GraphEditorInputMode#paste} or
       * {@link yfiles.input.GraphEditorInputMode#duplicateSelection}.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#shouldSelectAfterPaste
       * @see yfiles.input.GraphEditorInputMode#smartPasteSelection
       * @see yfiles.input.GraphInputMode#setSelected
       * @default yfiles.graph.GraphItemTypes.ALL
       * @type {yfiles.graph.GraphItemTypes}
       */
      pasteSelectableItems:yfiles.graph.GraphItemTypes;
      /**
       * Determines whether nodes, edges, labels, and ports should only be selected after {@link yfiles.input.GraphEditorInputMode#paste}
       * or {@link yfiles.input.GraphEditorInputMode#duplicateSelection} if they were selected when they were initially copied into the
       * clipboard.
       * <p>
       * The default is <code>true</code>. Disabling this feature will select all {@link yfiles.input.GraphEditorInputMode#pasteSelectableItems} after a paste or duplicate operation.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#shouldSelectAfterPaste
       * @see yfiles.input.GraphEditorInputMode#pasteSelectableItems
       * @see yfiles.input.GraphInputMode#setSelected
       * @default true
       * @type {boolean}
       */
      smartPasteSelection:boolean;
      /**
       * Determines which types of items should be movable using the {@link yfiles.input.GraphEditorInputMode#moveInputMode} or {@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} respectively in case of labels.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#shouldMove
       * @type {yfiles.graph.GraphItemTypes}
       */
      movableItems:yfiles.graph.GraphItemTypes;
      /**
       * Determines which types of items may be deleted using the {@link yfiles.input.GraphEditorInputMode#deleteSelection} action.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#shouldDelete
       * @type {yfiles.graph.GraphItemTypes}
       */
      deletableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets a predicate that is queried to determine whether an {@link yfiles.graph.IModelItem} can be deleted or not.
       * <p>
       * This allows to further restrict which items are deletable, compared to {@link yfiles.input.GraphEditorInputMode#deletableItems}. If this property is set to <code>null</code> the effect is
       * the same as a predicate that always returns <code>true</code>.
       * </p>
       * @type {function(yfiles.graph.IModelItem): boolean}
       */
      deletablePredicate:(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Determines which types of items may have their {@link yfiles.input.GraphEditorInputMode#onEditLabel labels edited}.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#LABEL_OWNER}|{@link yfiles.graph.GraphItemTypes#LABEL}, which allows for editing the
       * labels of {@link yfiles.graph.ILabelOwner}s, and existing {@link yfiles.graph.ILabel}s.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#shouldEditLabel
       * @type {yfiles.graph.GraphItemTypes}
       */
      labelEditableItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#handleInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphEditorInputMode#createHandleInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphEditorInputMode#onHandleInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.HandleInputMode}
       */
      handleInputMode:yfiles.input.HandleInputMode;
      /**
       * Determines which types of items should have their {@link yfiles.input.IHandle}s shown.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#shouldShowHandles
       * @type {yfiles.graph.GraphItemTypes}
       */
      showHandleItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets a value indicating whether to use the {@link yfiles.view.GraphComponent#currentItem} as a fallback for the commands if no item is provided in the
       * parameter and the current selection is empty.
       * <p>
       * This applies to the following commands:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.ICommand#ADD_LABEL}</li>
       * <li>{@link yfiles.input.ICommand#EDIT_LABEL}</li>
       * <li>{@link yfiles.input.ICommand#ADJUST_GROUP_NODE_SIZE}</li>
       * </ul>
       * @default false
       * @type {boolean}
       */
      useCurrentItemForCommands:boolean;
      /**
       * The offset for paste operation if clipboard operations are enabled.
       * <p>
       * Default is (15, 15).
       * </p>
       * @default '15,15'
       * @type {yfiles.geometry.Point}
       */
      pasteDelta:yfiles.geometry.Point;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#EDIT_LABEL} command should be handled.
       * <p>
       * If this flag is set to <code>true</code> pressing F2 will start the label editor. By default this feature is enabled.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#editLabel
       * @see yfiles.input.GraphEditorInputMode#onEditLabel
       * @type {boolean}
       */
      allowEditLabel:boolean;
      /**
       * Gets or sets a value determining whether double-clicking should start label editing.
       * <p>
       * If enabled, double-clicking an item will automatically execute the {@link yfiles.input.ICommand#EDIT_LABEL} command.
       * </p>
       * <p>
       * This setting has no effect if label editing is generally disallowed by setting {@link yfiles.input.GraphEditorInputMode#allowEditLabel} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowEditLabel
       * @type {boolean}
       */
      allowEditLabelOnDoubleClick:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#ADD_LABEL} command should be handled.
       * <p>
       * If this flag is set to <code>true</code> pressing Shift+F2 will start the label editor. By default this feature is enabled.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#createLabel
       * @type {boolean}
       */
      allowAddLabel:boolean;
      /**
       * Adds the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.GraphEditorInputMode#createLabel}.
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#removeLabelAddedListener
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.GraphEditorInputMode#createLabel}.
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addLabelAddedListener
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is about
       * to start.
       * <p>
       * This allows to customize the label editing process.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#addValidateLabelTextListener
       * @see yfiles.input.GraphEditorInputMode#removeLabelTextEditingStartedListener
       */
      addLabelTextEditingStartedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is
       * about to start.
       * <p>
       * This allows to customize the label editing process.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addValidateLabelTextListener
       * @see yfiles.input.GraphEditorInputMode#addLabelTextEditingStartedListener
       */
      removeLabelTextEditingStartedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
       * canceled.
       * <p>
       * This allows to unconfigure temporary customizations for the actual label editing process.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#addValidateLabelTextListener
       * @see yfiles.input.GraphEditorInputMode#removeLabelTextEditingCanceledListener
       */
      addLabelTextEditingCanceledListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
       * canceled.
       * <p>
       * This allows to unconfigure temporary customizations for the actual label editing process.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addValidateLabelTextListener
       * @see yfiles.input.GraphEditorInputMode#addLabelTextEditingCanceledListener
       */
      removeLabelTextEditingCanceledListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>ValidateLabelText</code> event that occurs when a label that is about to be {@link yfiles.input.GraphEditorInputMode#addLabel added} or {@link yfiles.input.GraphEditorInputMode#editLabel edited}.
       * <p>
       * This event that can be used to validate the label text before inserting/updating the actual label.
       * </p>
       * <p>
       * Note that in the case of {@link yfiles.input.GraphEditorInputMode#addLabel} the {@link yfiles.input.LabelTextValidatingEventArgs#label} is not part of the current graph but only a
       * dummy instance.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#onValidateLabelText
       * @see yfiles.input.GraphEditorInputMode#removeValidateLabelTextListener
       */
      addValidateLabelTextListener(listener:(sender:Object,evt:yfiles.input.LabelTextValidatingEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ValidateLabelText</code> event that occurs when a label that is about to be {@link yfiles.input.GraphEditorInputMode#addLabel added} or
       * {@link yfiles.input.GraphEditorInputMode#editLabel edited}.
       * <p>
       * This event that can be used to validate the label text before inserting/updating the actual label.
       * </p>
       * <p>
       * Note that in the case of {@link yfiles.input.GraphEditorInputMode#addLabel} the {@link yfiles.input.LabelTextValidatingEventArgs#label} is not part of the current graph but only a
       * dummy instance.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#onValidateLabelText
       * @see yfiles.input.GraphEditorInputMode#addValidateLabelTextListener
       */
      removeValidateLabelTextListener(listener:(sender:Object,evt:yfiles.input.LabelTextValidatingEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.GraphEditorInputMode#editLabel}.
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#removeLabelTextChangedListener
       */
      addLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.GraphEditorInputMode#editLabel}.
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addLabelTextChangedListener
       */
      removeLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link yfiles.input.GraphEditorInputMode#deleteSelection}
       * method starts its work and will be followed by any number of {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * events and finalized by a {@link yfiles.input.GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
       * event.
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#deleteSelection
       * @see yfiles.input.GraphEditorInputMode#removeDeletingSelectionListener
       */
      addDeletingSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link yfiles.input.GraphEditorInputMode#deleteSelection}
       * method starts its work and will be followed by any number of {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * events and finalized by a {@link yfiles.input.GraphEditorInputMode#addDeletedSelectionListener DeletedSelection}
       * event.
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#deleteSelection
       * @see yfiles.input.GraphEditorInputMode#addDeletingSelectionListener
       */
      removeDeletingSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this mode.
       * <p>
       * Note that this event will not be triggered for items that are removed implicitly, e.g. if the event is triggered for a
       * node, its labels, ports, and adjacent edges will not necessarily be reported separately. In order to be notified of any
       * removal events, the {@link yfiles.graph.IGraph} events should be used instead.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#removeDeletedItemListener
       */
      addDeletedItemListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this
       * mode.
       * <p>
       * Note that this event will not be triggered for items that are removed implicitly, e.g. if the event is triggered for a
       * node, its labels, ports, and adjacent edges will not necessarily be reported separately. In order to be notified of any
       * removal events, the {@link yfiles.graph.IGraph} events should be used instead.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       */
      removeDeletedItemListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>DeletedSelection</code> event that occurs just before the {@link yfiles.input.GraphEditorInputMode#deleteSelection}
       * method has deleted the selection after all selected items have been removed.
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#deleteSelection
       * @see yfiles.input.GraphEditorInputMode#removeDeletedSelectionListener
       */
      addDeletedSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>DeletedSelection</code> event that occurs just before the {@link yfiles.input.GraphEditorInputMode#deleteSelection}
       * method has deleted the selection after all selected items have been removed.
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#deleteSelection
       * @see yfiles.input.GraphEditorInputMode#addDeletedSelectionListener
       */
      removeDeletedSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#moveInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphEditorInputMode#createMoveInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphEditorInputMode#onMoveInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>40</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @see yfiles.input.GraphEditorInputMode#shouldMove
       * @type {yfiles.input.MoveInputMode}
       */
      moveInputMode:yfiles.input.MoveInputMode;
      /**
       * Gets or sets a property that determines whether this instance should automatically adjust the {@link yfiles.view.CanvasComponent#contentRect} if the graph changes.
       * <p>
       * The {@link yfiles.input.GraphEditorInputMode#adjustContentRect} method is called whenever the content changes. The default is
       * {@link yfiles.input.AdjustContentRectPolicy#UNION}.
       * </p>
       * @default yfiles.input.AdjustContentRectPolicy.UNION
       * @type {yfiles.input.AdjustContentRectPolicy}
       */
      adjustContentRectPolicy:yfiles.input.AdjustContentRectPolicy;
      /**
       * Gets or sets the margins to use for {@link yfiles.view.CanvasComponent#updateContentRect} in {@link yfiles.input.GraphEditorInputMode#adjustContentRect}
       * calls.
       * @type {yfiles.geometry.Insets}
       */
      contentRectMargins:yfiles.geometry.Insets;
      /**
       * Gets the {@link yfiles.input.GraphEditorInputMode#graphComponent} instance this mode is working on or <code>null</code>.
       * <p>
       * This property is set when this mode is {@link yfiles.input.GraphEditorInputMode#install}ed in a {@link yfiles.input.GraphEditorInputMode#graphComponent}.
       * </p>
       * @type {yfiles.view.GraphComponent}
       */
      graphComponent:yfiles.view.GraphComponent;
      /**
       * The {@link yfiles.input.SnapContext} instance that handles interactive snapping of elements during drag operations like
       * movements.
       * <p>
       * Setting this property to a {@link yfiles.input.SnapContext} which is {@link yfiles.input.SnapContext#enabled disabled} will disable snapping, whereas setting it to <code>null</code>
       * will make the child modes use the context from their {@link yfiles.input.IInputModeContext}s, if available. Upon change the {@link yfiles.input.GraphEditorInputMode#onSnapContextChanged}
       * method is called, which will as a side effect configure the snap context to display the {@link yfiles.input.SnapResult}s in the
       * {@link yfiles.view.CanvasComponent}. Default is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.input.SnapContext}
       */
      snapContext:yfiles.input.SnapContext;
      /**
       * The {@link yfiles.input.LabelSnapContext} instance that handles interactive snapping of
       * {@link yfiles.graph.ILabel}s to their owner during drag operations like movements.
       * <p>
       * Setting this property to <code>null</code> will disable snapping, whereas setting it to <code>null</code> will make the child modes use the
       * context from their {@link yfiles.input.IInputModeContext}s, if available. Upon change the {@link yfiles.input.GraphEditorInputMode#onLabelSnapContextChanged}
       * method is called, which will as a side effect configure the snap context to display the {@link yfiles.input.SnapResult}s in the
       * {@link yfiles.view.CanvasComponent}. Default is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.input.SnapContext}
       */
      labelSnapContext:yfiles.input.SnapContext;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#moveLabelInputMode} property.
       * <p>
       * This mode is responsible for moving labels to another candidate position. If the field has not yet been initialized upon
       * first access, the factory method {@link yfiles.input.GraphEditorInputMode#createMoveLabelInputMode} will be called. Upon change
       * the {@link yfiles.input.GraphEditorInputMode#onMoveLabelInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>39</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @see yfiles.input.GraphEditorInputMode#shouldMove
       * @type {yfiles.input.MoveLabelInputMode}
       */
      moveLabelInputMode:yfiles.input.MoveLabelInputMode;
      /**
       * Gets or sets a value indicating whether grouping operations like {@link yfiles.input.GraphEditorInputMode#allowGroupSelection grouping selected nodes} or {@link yfiles.input.GraphEditorInputMode#allowReparentNodes moving nodes into group nodes}
       * should be enabled.
       * <p>
       * Setting this property to <code>false</code> effectively disables handling of all grouping {@link yfiles.input.ICommand commands}. If this property is set to <code>true</code>
       * the related properties {@link yfiles.input.GraphEditorInputMode#allowGroupSelection}, {@link yfiles.input.GraphEditorInputMode#allowUngroupSelection}, and {@link yfiles.input.GraphEditorInputMode#allowReparentNodes} allow for selectively enabling or disabling only certain grouping operations.
       * </p>
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowGroupSelection
       * @see yfiles.input.GraphEditorInputMode#allowUngroupSelection
       * @see yfiles.input.GraphEditorInputMode#allowAdjustGroupNodeSize
       * @see yfiles.input.GraphEditorInputMode#allowReparentNodes
       * @default false
       * @type {boolean}
       */
      allowGroupingOperations:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#GROUP_SELECTION} command should be handled.
       * <p>
       * This setting has no effect if grouping operations are generally disallowed by setting {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowGroupingOperations
       * @see yfiles.input.GraphEditorInputMode#groupSelection
       * @default true
       * @type {boolean}
       */
      allowGroupSelection:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#ADJUST_GROUP_NODE_SIZE} command should be handled.
       * <p>
       * This setting has no effect if grouping operations are generally disallowed by setting {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowGroupingOperations
       * @see yfiles.input.GraphEditorInputMode#adjustGroupNodeSizes
       * @see yfiles.input.GraphEditorInputMode#adjustGroupNodeSize
       * @default true
       * @type {boolean}
       */
      allowAdjustGroupNodeSize:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#UNGROUP_SELECTION} command should be handled.
       * <p>
       * This setting has no effect if grouping operations are generally disallowed by setting {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowGroupingOperations
       * @see yfiles.input.GraphEditorInputMode#ungroupSelection
       * @default true
       * @type {boolean}
       */
      allowUngroupSelection:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#DESELECT_ALL} command should be handled.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      allowClearSelection:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#PASTE} command should be handled.
       * <p>
       * This setting has no effect if clipboard operations are generally disallowed by setting {@link yfiles.input.GraphInputMode#allowClipboardOperations} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      allowPaste:boolean;
      /**
       * Gets or sets a value determining whether the {@link yfiles.input.ICommand#DUPLICATE} command should be handled.
       * <p>
       * This setting has no effect if clipboard operations are generally disallowed by setting {@link yfiles.input.GraphInputMode#allowClipboardOperations} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      allowDuplicate:boolean;
      /**
       * Gets or sets a value indicating whether undo operations, that is, {@link yfiles.input.GraphEditorInputMode#undo} and {@link yfiles.input.GraphEditorInputMode#redo}
       * should be enabled.
       * <p>
       * Note that even if this feature is enabled, an {@link yfiles.graph.UndoEngine undo engine} needs to be available in the {@link yfiles.input.InputModeBase#inputModeContext}in order to work.
       * </p>
       * <p>
       * Setting this property to <code>false</code> effectively disables handling of the commands {@link yfiles.input.ICommand#UNDO} and
       * {@link yfiles.input.ICommand#REDO}.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#undo
       * @see yfiles.input.GraphEditorInputMode#redo
       * @default true
       * @type {boolean}
       */
      allowUndoOperations:boolean;
      /**
       * Gets or sets a value indicating whether moving nodes out of group nodes or into other group nodes is allowed.
       * <p>
       * If set to <code>false</code> the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler} will effectively be disabled and not called anymore for reparenting operations.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#reparentNodeHandler
       * @default true
       * @type {boolean}
       */
      allowReparentNodes:boolean;
      /**
       * Gets or sets a value indicating whether moving nodes into {@link yfiles.graph.IGraph#isGroupNode non-group nodes} is allowed to convert them into group nodes.
       * <p>
       * If {@link yfiles.input.GraphEditorInputMode#allowReparentNodes reparenting nodes is allowed} in general and this property is enabled, users may reparent nodes not only to groups and
       * folders but also to non-group nodes. Still, the decision about the final acceptance of the reparenting and the actual
       * execution are delegated to the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler}.
       * </p>
       * <p>
       * This setting has no effect if grouping operations are generally disallowed by setting {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations} to <code>false</code>.
       * </p>
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#allowGroupingOperations
       * @default false
       * @type {boolean}
       */
      allowReparentToNonGroupNodes:boolean;
      /**
       * Adds the given listener for the <code>NodeReparented</code> event that occurs when a node has been reparented interactively.
       * <p>
       * This is triggered whenever the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler}'s {@link yfiles.input.IReparentNodeHandler#reparent} method has been called to reparent a node
       * interactively.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#reparentNodeHandler
       * @see yfiles.input.GraphEditorInputMode#removeNodeReparentedListener
       */
      addNodeReparentedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>NodeReparented</code> event that occurs when a node has been reparented interactively.
       * <p>
       * This is triggered whenever the {@link yfiles.input.GraphEditorInputMode#reparentNodeHandler}'s {@link yfiles.input.IReparentNodeHandler#reparent} method has been called to reparent a node
       * interactively.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#reparentNodeHandler
       * @see yfiles.input.GraphEditorInputMode#addNodeReparentedListener
       */
      removeNodeReparentedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeCreated</code> event that occurs when this mode has created a node in response to user
       * interaction.
       * <p>
       * This event is triggered if the node is created using a click gesture via {@link yfiles.input.GraphEditorInputMode#createNode},
       * or via a drag and drop operation that was completed by {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode}. Note that if {@link yfiles.input.NodeDropInputMode#allowFolderNodeAsParent} is set to <code>true</code>, the reported node can actually be
       * part of the {@link yfiles.graph.IFoldingView master graph}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#createNode
       * @see yfiles.input.GraphEditorInputMode#removeNodeCreatedListener
       */
      addNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>NodeCreated</code> event that occurs when this mode has created a node in response to user
       * interaction.
       * <p>
       * This event is triggered if the node is created using a click gesture via {@link yfiles.input.GraphEditorInputMode#createNode},
       * or via a drag and drop operation that was completed by {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode}. Note that if {@link yfiles.input.NodeDropInputMode#allowFolderNodeAsParent} is set to <code>true</code>, the reported node can actually be
       * part of the {@link yfiles.graph.IFoldingView master graph}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#createNode
       * @see yfiles.input.GraphEditorInputMode#addNodeCreatedListener
       */
      removeNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Gets or sets a value determining whether {@link yfiles.input.GraphEditorInputMode#createNode node creation} by clicking on an empty canvas location is enabled.
       * <p>
       * To disable node creation via drag and drop gestures, the {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode} must be disabled, too. However, it is disabled by default.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#nodeCreator
       * @see yfiles.input.GraphEditorInputMode#createNode
       * @type {boolean}
       */
      allowCreateNode:boolean;
      /**
       * Gets or sets a value determining whether {@link yfiles.input.GraphEditorInputMode#createBendInputMode bend creation} should be {@link yfiles.input.CreateBendInputMode#enabled enabled}.
       * <p>
       * This property delegates to {@link yfiles.input.GraphEditorInputMode#createBendInputMode}'s {@link yfiles.input.CreateBendInputMode#enabled} property.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      allowCreateBend:boolean;
      /**
       * Gets or sets a value determining whether {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode edge creation} should be {@link yfiles.input.CreateEdgeInputMode#enabled enabled}.
       * <p>
       * This property delegates to {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode}'s {@link yfiles.input.CreateEdgeInputMode#enabled} property.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      allowCreateEdge:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.input.ICommand#REVERSE_EDGE} command should be handled.
       * <p>
       * If set to <code>false</code> the methods {@link yfiles.input.GraphEditorInputMode#reverseEdge},
       * {@link yfiles.input.GraphEditorInputMode#reverseEdges}, and {@link yfiles.input.GraphEditorInputMode#reverseSelectedEdges} do nothing.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      allowReverseEdge:boolean;
      /**
       * Adds the given listener for the <code>EdgePortsChanged</code> event that occurs after an edge's source and/or target ports have been
       * changed as the result of an input gesture.
       * @param listener The listener to add.
       * @see yfiles.graph.EdgeEventArgs
       * @see yfiles.input.GraphEditorInputMode#reverseSelectedEdges
       * @see yfiles.input.GraphEditorInputMode#reverseEdges
       * @see yfiles.input.GraphEditorInputMode#reverseEdge
       * @see yfiles.input.GraphEditorInputMode#removeEdgePortsChangedListener
       */
      addEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgePortsChanged</code> event that occurs after an edge's source and/or target ports have
       * been changed as the result of an input gesture.
       * @param listener The listener to remove.
       * @see yfiles.graph.EdgeEventArgs
       * @see yfiles.input.GraphEditorInputMode#reverseSelectedEdges
       * @see yfiles.input.GraphEditorInputMode#reverseEdges
       * @see yfiles.input.GraphEditorInputMode#reverseEdge
       * @see yfiles.input.GraphEditorInputMode#addEdgePortsChangedListener
       */
      removeEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Gets or sets the callback that is responsible for creating a new node, for instance, in response to a mouse click.
       * <p>
       * A <code>null</code> value will disable node creation using mouse clicks, just as setting {@link yfiles.input.GraphEditorInputMode#allowCreateNode} to <code>false</code>.
       * </p>
       * @see yfiles.input.GraphEditorInputMode#createNode
       * @type {function(yfiles.input.IInputModeContext, yfiles.graph.IGraph, yfiles.geometry.Point, yfiles.graph.INode): yfiles.graph.INode}
       */
      nodeCreator:(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,location:yfiles.geometry.Point,parent:yfiles.graph.INode)=>yfiles.graph.INode;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#textEditorInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphEditorInputMode#createTextEditorInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphEditorInputMode#onTextEditorInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>100</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.TextEditorInputMode}
       */
      textEditorInputMode:yfiles.input.TextEditorInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#createEdgeInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphEditorInputMode#createCreateEdgeInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphEditorInputMode#onCreateEdgeInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>45</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.CreateEdgeInputMode}
       */
      createEdgeInputMode:yfiles.input.CreateEdgeInputMode;
      /**
       * Gets or sets a value indicating whether this mode should automatically {@link yfiles.graph.IGraph#remove remove labels} from the graph when a label text
       * has been edited and the label text is empty.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoRemoveEmptyLabels:boolean;
      /**
       * Gets or sets a property that determines whether clicking on a node should automatically select all bends of self loops
       * adjacent to that node.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.GraphInputMode#clickSelectableItems
       * @default true
       * @type {boolean}
       */
      autoSelectSelfloopBends:boolean;
      /**
       * Gets or sets a property that determines whether the label should be hidden while it is edited.
       * <p>
       * If enabled, any selection, focus, and highlight visualizations of the label are hidden along with the label
       * visualization itself.
       * </p>
       * <p>
       * Even if enabled, the label <b>text</b> is still visible in the label text editor.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      hideLabelDuringEditing:boolean;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#createBendInputMode} property.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.GraphEditorInputMode#createCreateBendInputMode}
       * will be called. Upon change the {@link yfiles.input.GraphEditorInputMode#onCreateBendInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>42</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.CreateBendInputMode}
       */
      createBendInputMode:yfiles.input.CreateBendInputMode;
      /**
       * Gets or sets the {@link yfiles.input.GraphEditorInputMode#nodeDropInputMode} property that is responsible for handling drag and drop operations of nodes onto the Canvas.
       * <p>
       * The mode by default is {@link yfiles.input.DropInputMode#enabled disabled} and needs to be enabled to work, first. If the field has not yet been initialized upon
       * first access, the factory method {@link yfiles.input.GraphEditorInputMode#createNodeDropInputMode} will be called. Upon change
       * the {@link yfiles.input.GraphEditorInputMode#onNodeDropInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>70</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.NodeDropInputMode}
       */
      nodeDropInputMode:yfiles.input.NodeDropInputMode;
      /**
       * Uses the {@link yfiles.input.WaitInputMode} to delegate wait requests to.
       * @type {boolean}
       */
      waiting:boolean;
      /**
       * Adds the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
       * <p>
       * Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
       * forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
       * {@link yfiles.input.LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#removeLabelAddingListener
       */
      addLabelAddingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
       * <p>
       * Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
       * forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
       * {@link yfiles.input.LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addLabelAddingListener
       */
      removeLabelAddingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
       * <p>
       * Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
       * entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
       * event handlers can also specify that instead of editing a label, a new one should be added.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.GraphEditorInputMode#removeLabelEditingListener
       */
      addLabelEditingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
       * <p>
       * Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
       * entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
       * event handlers can also specify that instead of editing a label, a new one should be added.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.GraphEditorInputMode#addLabelEditingListener
       */
      removeLabelEditingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.input.DropInputMode} providing a preview of the dragged item.
     * <p>
     * A dragged {@link yfiles.graph.IModelItem} is visualized during the drag operation. In addition, this input mode supports
     * snapping of the dragged {@link yfiles.graph.IModelItem} via the {@link yfiles.input.SnapContext} and highlighting the {@link yfiles.input.ItemDropInputMode.<T>#dropTarget}.
     * </p>
     * <p>
     * Highlighting is done via a {@link yfiles.view.HighlightIndicatorManager.<T>}.
     * </p>
     * Type parameter <code>T</code>.
     * @class
     * @extends {yfiles.input.DropInputMode}
     * @template T
     */
    export interface ItemDropInputMode<T extends yfiles.graph.IModelItem> extends yfiles.input.DropInputMode{}
    export class ItemDropInputMode<T extends yfiles.graph.IModelItem> {
      /**
       * Initializes a new instance of the {@link yfiles.input.ItemDropInputMode.<T>} class for the expected data format.
       * <p>
       * This instance fires events only if a drag operation carries data of the expected format.
       * </p>
       * @param expectedFormat This is a format that is used in {@link #getData}.
       * @protected
       */
      constructor(expectedFormat:string);
      /**
       * Cleanup method that calls {@link yfiles.input.ItemDropInputMode.<T>#cleanupSnapContext}, {@link yfiles.input.ItemDropInputMode.<T>#cleanupPreview} and {@link yfiles.input.ItemDropInputMode.<T>#cleanupDropTarget}.
       * <p>
       * This method should be called for cleanup actions after a drag/drop gesture has been finished or canceled.
       * </p>
       * @protected
       */
      cleanup():void;
      /**
       * Cleans up the drop target and its highlighting.
       * @protected
       */
      cleanupDropTarget():void;
      /**
       * Cleans up the item preview.
       * @protected
       */
      cleanupPreview():void;
      /**
       * Cleans up the snap context.
       * @protected
       */
      cleanupSnapContext():void;
      /**
       * Callback registered on the {@link yfiles.input.SnapContext} that collects {@link yfiles.input.SnapResult}s for the dragged element.
       * <p>
       * The default implementation doesn't collect any snap results.
       * </p>
       * @param source The {@link } this callback is registered at.
       * @param evt The arguments describing the movement {@link }s shall be collected for.
       * @protected
       */
      collectSnapResults(source:Object,evt:yfiles.input.CollectSnapResultsEventArgs):void;
      /**
       * Returns the drop target at the specified location.
       * <p>
       * The drop target is highlighted, if {@link yfiles.input.ItemDropInputMode.<T>#highlightDropTarget} is enabled. Per default, <code>null</code> is returned.
       * </p>
       * @param dragLocation The location to return the drop target for.
       * @returns The drop target at the specified location or <code>null</code> if no drop target can be found.
       * @protected
       */
      getDropTarget(dragLocation:yfiles.geometry.Point):yfiles.graph.IModelItem;
      /**
       * Initializes the drop target.
       * @protected
       */
      initializeDropTarget():void;
      /**
       * Initializes the item preview.
       * <p>
       * This method is called by {@link yfiles.input.ItemDropInputMode.<T>#onDragEntered}. If a preview {@link yfiles.input.ItemDropInputMode.<T>#showPreview shall be displayed}, the {@link yfiles.input.ItemDropInputMode.<T>#previewGraph preview graph} is initialized and {@link yfiles.input.ItemDropInputMode.<T>#populatePreviewGraph}
       * is called.
       * </p>
       * @protected
       */
      initializePreview():void;
      /**
       * Initializes the snapping context.
       * <p>
       * This method is called by {@link yfiles.input.ItemDropInputMode.<T>#onDragEntered}
       * </p>
       * @protected
       */
      initializeSnapContext():void;
      /**
       * Called whenever a new item is created
       * @param evt
       * @protected
       */
      onItemCreated(evt:yfiles.collections.ItemEventArgs<T>):void;
      /**
       * Subclasses shall fill the specified graph that is used to preview the dragged item.
       * @param previewGraph The preview graph to fill.
       * @protected
       */
      populatePreviewGraph(previewGraph:yfiles.graph.IGraph):void;
      /**
       * Calls {@link yfiles.input.ItemDropInputMode.<T>#getDropTarget} for {@link yfiles.input.ItemDropInputMode.<T>#snappedMousePosition} and sets the returned item as {@link yfiles.input.ItemDropInputMode.<T>#dropTarget}.
       * <p>
       * If {@link yfiles.input.ItemDropInputMode.<T>#highlightDropTarget} is enabled, the highlight is updated as well.
       * </p>
       * @param dragLocation The location to update the drop target for.
       * @protected
       */
      updateDropTarget(dragLocation:yfiles.geometry.Point):void;
      /**
       * Subclasses shall update the {@link yfiles.input.ItemDropInputMode.<T>#previewGraph preview graph} so the dragged item is displayed at the specified <code>dragLocation</code>.
       * @param previewGraph The preview graph to update.
       * @param dragLocation The current drag location.
       * @protected
       */
      updatePreview(previewGraph:yfiles.graph.IGraph,dragLocation:yfiles.geometry.Point):void;
      /**
       * Adds the given listener for the <code>ItemCreated</code> event that occurs when a new item gets created by this input mode.
       * @param listener The listener to add.
       * @see yfiles.input.ItemDropInputMode.<T>#removeItemCreatedListener
       */
      addItemCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Removes the given listener for the <code>ItemCreated</code> event that occurs when a new item gets created by this input mode.
       * @param listener The listener to remove.
       * @see yfiles.input.ItemDropInputMode.<T>#addItemCreatedListener
       */
      removeItemCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<T>)=>void):void;
      /**
       * Determines whether a preview of the dragged element is displayed during the drag
       * @type {boolean}
       */
      showPreview:boolean;
      /**
       * Switches snapping on and off.
       * @type {boolean}
       */
      snappingEnabled:boolean;
      /**
       * Gets or sets event recognizer that temporarily disables snapping.
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      disableSnappingRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Switches highlighting the drop target on and off.
       * @type {boolean}
       */
      highlightDropTarget:boolean;
      /**
       * The {@link yfiles.input.ItemDropInputMode.<T>#snapContext} which is used to snap the dragged element during the drag.
       * <p>
       * If set to <code>null</code> (the default) this input mode tries to obtain the {@link yfiles.input.ItemDropInputMode.<T>#snapContext} from the {@link yfiles.input.IInputModeContext}. To explicitly
       * disable snapping, a {@link yfiles.input.ItemDropInputMode.<T>#snapContext} implementation that does nothing has to be set to this instance.
       * </p>
       * @type {yfiles.input.SnapContext}
       */
      snapContext:yfiles.input.SnapContext;
      /**
       * Gets the current snapped mouse position during drag operations.
       * <p>
       * The position is returned in world coordinates according to the {@link yfiles.view.CanvasComponent} into which this input mode is
       * installed. If {@link yfiles.input.ItemDropInputMode.<T>#snappingEnabled} is false, this value corresponds with {@link yfiles.input.DropInputMode#mousePosition}.
       * </p>
       * @type {yfiles.geometry.Point}
       */
      snappedMousePosition:yfiles.geometry.Point;
      /**
       * Gets the drop target at {@link yfiles.input.ItemDropInputMode.<T>#snappedMousePosition}
       * @see yfiles.input.ItemDropInputMode.<T>#highlightDropTarget
       * @type {yfiles.graph.IModelItem}
       */
      dropTarget:yfiles.graph.IModelItem;
      /**
       * Gets the currently dragged {@link yfiles.graph.IModelItem} instance.
       * <p>
       * This implementation simply tries to cast the {@link yfiles.input.DropInputMode#dropData} to the type of the dragged items.
       * </p>
       * @protected
       * @abstract
       * @type {T}
       */
      draggedItem:T;
      /**
       * Gets or sets the callback for item creation.
       * <p>
       * Called by method {@link yfiles.input.ItemDropInputMode.<T>#onDragDropped}.
       * </p>
       * @type {function(yfiles.input.IInputModeContext, yfiles.graph.IGraph, T, yfiles.graph.IModelItem, yfiles.geometry.Point): T}
       */
      itemCreator:(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,draggedItem:T,dropTarget:yfiles.graph.IModelItem,dropLocation:yfiles.geometry.Point)=>T;
      /**
       * Returns the graph displayed as item preview.
       * @protected
       * @type {yfiles.graph.IGraph}
       */
      previewGraph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of {@link yfiles.input.IPositionHandler} that shows the various label position candidates and lets the user
     * move a label to one of those candidate positions.
     * @class
     * @implements {yfiles.input.IPositionHandler}
     */
    export interface LabelPositionHandler extends Object,yfiles.input.IPositionHandler{}
    export class LabelPositionHandler {
      /**
       * Initializes a new instance of the {@link yfiles.input.LabelPositionHandler} class.
       * @param label The label that shall be moved.
       */
      constructor(label:yfiles.graph.ILabel);
      /**
       * 
       * @param context
       * @param originalLocation
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Creates an {@link yfiles.view.ICanvasObjectDescriptor} to display the given candidate position.
       * @param context The context for which the descriptor is created.
       * @param highlight Whether to highlight the paintable. This will be <code>true</code> for the current position and <code>false</code> for the others.
       * @returns An implementation that will render the given candidate.
       * @protected
       */
      createCandidateDescriptor(context:yfiles.input.IInputModeContext,highlight:boolean):yfiles.view.ICanvasObjectDescriptor;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Calculates a distance value between a candidate rectangle and the mouse location.
       * @param rectangle The rectangle candidate.
       * @param location The mouse location.
       * @returns A value indicating the closeness of the mouse to the rectangle.
       * @protected
       */
      getDistance(rectangle:yfiles.geometry.IOrientedRectangle,location:yfiles.geometry.Point):number;
      /**
       * Callback method that gets the candidates for the given label.
       * @param label The label.
       * @returns A possibly empty enumerator over possible label parameter candidates.
       * @protected
       */
      getParameterCandidates(label:yfiles.graph.ILabel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * 
       * @param context
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Actually applies the given parameter at the end of the gesture.
       * @param label The label to set the parameter for.
       * @param layoutParameter The new parameter.
       * @protected
       */
      setLayoutParameter(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * 
       * @param location
       */
      setPosition(location:yfiles.geometry.Point):void;
      /**
       * Determines whether to use a {@link yfiles.graph.ILabelModelParameterFinder} to find the best candidate.
       * <p>
       * This implementation returns <code>true</code> if the {@link yfiles.input.LabelPositionHandler#useFinder} property is <code>true</code> and the control key is pressed.
       * </p>
       * @param context The context that is currently being used - may be <code>null</code> if the method is called without context.
       * @returns 
       * @protected
       */
      useParameterFinder(context:yfiles.input.IInputModeContext):boolean;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} that will be used to represent the rectangular selection.
       * @const
       * @static
       * @type {string}
       */
      static CANDIDATE_TEMPLATE_KEY:string;
      /**
       * A {@link string} that will be used to find the {@link yfiles.view.IVisualTemplate} that will be used to represent the rectangular selection.
       * @const
       * @static
       * @type {string}
       */
      static HIGHLIGHT_TEMPLATE_KEY:string;
      /**
       * Gets or sets the template to use for showing candidates.
       * @type {yfiles.view.IVisualTemplate}
       */
      candidateTemplate:yfiles.view.IVisualTemplate;
      /**
       * Gets or sets the template to use for showing highlighted candidates.
       * @type {yfiles.view.IVisualTemplate}
       */
      highlightTemplate:yfiles.view.IVisualTemplate;
      /**
       * Gets the graph instance from the {@link yfiles.input.IInputModeContext} that has been passed to the {@link yfiles.input.LabelPositionHandler#initializeDrag}
       * method.
       * @protected
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Gets the label instance.
       * @protected
       * @type {yfiles.graph.ILabel}
       */
      label:yfiles.graph.ILabel;
      /**
       * 
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets or sets a property that determines whether the handler may use a {@link yfiles.graph.ILabelModelParameterFinder} to assign
       * arbitrary positions.
       * <p>
       * The default value is <code>true</code>
       * </p>
       * @see yfiles.input.LabelPositionHandler#useParameterFinder
       * @type {boolean}
       */
      useFinder:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The default label snap context helper that provides the snap lines and the snap results to the {@link yfiles.input.LabelSnapContext}
     * during dragging of labels.
     * @see yfiles.input.LabelSnapContext
     * @see yfiles.graph.LabelDecorator#labelSnapContextHelperDecorator
     * @class
     * @implements {yfiles.input.ILabelSnapContextHelper}
     */
    export interface LabelSnapContextHelper extends Object,yfiles.input.ILabelSnapContextHelper{}
    export class LabelSnapContextHelper {
      /**
       * Adds snap lines to the <code>snapContext</code> that are parallel to the path segments of the owner edge at the given distance.
       * <p>
       * This method is not used by this class to create its snap lines. Instead, it can be called by custom sub-classes to
       * create this kind of snap lines.
       * </p>
       * @param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
       * the context explicitly.
       * @param inputModeContext The context of the input mode that handles the dragging.
       * @param label The edge label that is dragged.
       * @param distance The distance of the added snap lines from the edge path.
       * @protected
       */
      addEdgePathSnapLines(snapContext:yfiles.input.LabelSnapContext,inputModeContext:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel,distance:number):void;
      /**
       * Adds snap lines to the <code>snapContext</code> that are parallel to the path segments of the owner edge at particular distances.
       * <p>
       * The corresponding settings of the <code>snapContext</code> specify which snap lines are actually created.
       * </p>
       * @param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
       * the context explicitly.
       * @param inputModeContext The context of the input mode that handles the dragging.
       * @param label The edge label that is dragged.
       * @protected
       */
      addEdgePathSnapLines(snapContext:yfiles.input.LabelSnapContext,inputModeContext:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):void;
      /**
       * Adds two snap line for the initial center location of the label, one in the direction of the up vector, the other
       * orthogonal to that direction.
       * <p>
       * This method is called by {@link yfiles.input.LabelSnapContextHelper#addSnapLines} to add the snap lines for the initial position
       * label location.
       * </p>
       * @param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
       * the context explicitly.
       * @param inputModeContext The context of the input mode that handles the dragging.
       * @param label The label that is dragged.
       * @protected
       */
      addInitialLocationSnapLines(snapContext:yfiles.input.LabelSnapContext,inputModeContext:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):void;
      /**
       * Adds snap lines to the <code>snapContext</code> that are parallel to the owner node's border at particular distances.
       * <p>
       * The corresponding settings of the <code>snapContext</code> specify which snap lines are actually created.
       * </p>
       * @param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
       * the context explicitly.
       * @param inputModeContext The context of the input mode that handles the dragging.
       * @param label The node label that is dragged.
       * @protected
       */
      addNodeShapeSnapLines(snapContext:yfiles.input.LabelSnapContext,inputModeContext:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):void;
      /**
       * Called during {@link yfiles.input.IDragHandler#initializeDrag initialization} of a <code>label</code> dragging to add {@link yfiles.input.SnapLine snap lines} to which the label can potentially snap to the
       * <code>snapContext</code>.
       * @param snapContext The snap context which manages the snap lines and the settings. Note that implementations should not change the state of
       * the context explicitly.
       * @param inputModeContext The context of the input mode that handles the dragging.
       * @param label The label that is dragged.
       */
      addSnapLines(snapContext:yfiles.input.LabelSnapContext,inputModeContext:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):void;
      /**
       * 
       * @param context
       * @param evt
       * @param suggestedLayout
       * @param label
       */
      collectSnapResults(context:yfiles.input.LabelSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLayout:yfiles.geometry.IOrientedRectangle,label:yfiles.graph.ILabel):void;
      /**
       * Returns the static shared instance of this class.
       * <p>
       * Since this implementation does not carry any state, this instance can safely be used even in multi-threaded
       * environments.
       * </p>
       * @const
       * @static
       * @type {yfiles.input.LabelSnapContextHelper}
       */
      static INSTANCE:yfiles.input.LabelSnapContextHelper;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for the {@link yfiles.input.GraphEditorInputMode#addValidateLabelTextListener ValidateLabelText}
     * event.
     * <p>
     * Event handlers can set the {@link yfiles.input.LabelTextValidatingEventArgs#cancel} property to <code>true</code> to cancel label editing if validation failed, or set the {@link yfiles.input.LabelTextValidatingEventArgs#newText} property to a
     * changed, validated value.
     * </p>
     * @class
     * @extends {yfiles.input.InputModeEventArgs}
     * @final
     */
    export interface LabelTextValidatingEventArgs extends yfiles.input.InputModeEventArgs{}
    export class LabelTextValidatingEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.input.LabelTextValidatingEventArgs} class.
       * @param context The context.
       * @param label The label.
       * @param newText The initial new text as entered by the user.
       */
      constructor(context:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel,newText:string);
      /**
       * Gets the label that is being edited.
       * <p>
       * Note that the label might not belong to a graph if it is a dummy for a label that is about to be created.
       * </p>
       * @type {yfiles.graph.ILabel}
       */
      label:yfiles.graph.ILabel;
      /**
       * Gets or sets the new text to use for the label.
       * @see yfiles.input.LabelTextValidatingEventArgs#cancel
       * @type {string}
       */
      newText:string;
      /**
       * Gets or sets a value indicating whether the edit should be canceled.
       * @type {boolean}
       */
      cancel:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.IInputMode} that can be used to drag a {@link yfiles.graph.ILabel} in a {@link yfiles.view.GraphComponent}.
     * <p>
     * This implementation shows the possible candidate positions for a given label and allows the user to drag the label to
     * one of these positions.
     * </p>
     * @class
     * @extends {yfiles.input.MoveInputMode}
     */
    export interface MoveLabelInputMode extends yfiles.input.MoveInputMode{}
    export class MoveLabelInputMode {
      /**
       * Creates a new instance that will move labels in the graph retrieved from the {@link yfiles.input.MoveInputMode#inputModeContext}.
       * @see yfiles.input.MoveLabelInputMode#graph
       */
      constructor();
      /**
       * Creates the hit testable that will be used to determine whether this mode may be activated.
       * <p>
       * This implementation returns an instance that delegates to {@link yfiles.input.MoveLabelInputMode#isValidLabelHit}.
       * </p>
       * @returns A hit testable.
       * @protected
       */
      createLabelHitTestable():yfiles.input.IHitTestable;
      /**
       * Creates the {@link yfiles.input.IPositionHandler position handler} for the given label used in {@link yfiles.input.MoveLabelInputMode#isValidLabelHit}.
       * <p>
       * First, this implementation checks for a position handler in the label's
       * {@link yfiles.graph.ILookup lookup}. If this fails, it creates a new label position handler for the given label.
       * </p>
       * @param label The label to create a position handler for.
       * @returns A position handler for the given label.
       * @protected
       */
      createLabelPositionHandler(label:yfiles.graph.ILabel):yfiles.input.IPositionHandler;
      /**
       * Looks for a label that has been hit at the specified position.
       * <p>
       * This implementation checks the {@link yfiles.input.MoveLabelInputMode#graphSelection} to see if there is exactly one selected label. This label is returned if it is hit by
       * the mouse and {@link yfiles.input.MoveLabelInputMode#shouldMove} returns <code>true</code>. Otherwise, <code>null</code> is returned.
       * </p>
       * @param context The context for the hit test.
       * @param location The coordinates for the hit test.
       * @returns The label hit at the given location or <code>null</code>.
       * @see yfiles.input.MoveLabelInputMode#isValidLabelHit
       * @see yfiles.input.MoveLabelInputMode#shouldMove
       * @protected
       */
      getHitLabel(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):yfiles.graph.ILabel;
      /**
       * Determines whether or not a valid label has been hit at the specified position.
       * <p>
       * This implementation calls {@link yfiles.input.MoveLabelInputMode#getHitLabel} to find the label at the specified location. If a
       * label is hit, the {@link yfiles.input.IPositionHandler} is created by
       * {@link yfiles.input.MoveLabelInputMode#createLabelPositionHandler}.
       * </p>
       * @param context The context for the hit test.
       * @param location The coordinates for the hit test.
       * @returns <code>true</code> if a valid label is hit and the label moving should be started here. Otherwise, <code>false</code>.
       * @protected
       */
      isValidLabelHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Determines whether or not a label is allowed to be moved.
       * <p>
       * This implementation returns <code>false</code> if the parent input mode {@link yfiles.input.GraphEditorInputMode#shouldMove forbids} moving the label. Overriding implementations should
       * query this implementation as base if this feature is desired.
       * </p>
       * @param context The context.
       * @param label The label to be checked.
       * @returns <code>true</code> if the label is allowed to move.
       * @see yfiles.input.MoveLabelInputMode#getHitLabel
       * @protected
       */
      shouldMove(context:yfiles.input.IInputModeContext,label:yfiles.graph.ILabel):boolean;
      /**
       * Gets or sets a value indicating whether or not using the {@link yfiles.graph.ILabelModelParameterFinder} to assign new label
       * model parameters is allowed.
       * <p>
       * The default is <code>true</code>
       * </p>
       * <p>
       * This property has no effect if an {@link yfiles.input.IPositionHandler} is available in the label's {@link yfiles.graph.ILookup lookup}.
       * </p>
       * @see yfiles.input.MoveLabelInputMode#createLabelPositionHandler
       * @type {boolean}
       */
      useLabelModelParameterFinder:boolean;
      /**
       * Gets the label that is currently moved or <code>null</code> if there is no such label.
       * <p>
       * The value of this property is not <code>null</code> only if this mode is currently active and performs a move operation, that is
       * between the {@link yfiles.input.MoveInputMode#addDragStartingListener DragStarting} and the {@link yfiles.input.MoveInputMode#addDragFinishedListener DragFinished}
       * event handlers.
       * </p>
       * @type {yfiles.graph.ILabel}
       */
      movedLabel:yfiles.graph.ILabel;
      /**
       * Gets the {@link yfiles.view.IGraphSelection} this mode is acting upon.
       * @type {yfiles.view.IGraphSelection}
       */
      graphSelection:yfiles.view.IGraphSelection;
      /**
       * Gets the {@link yfiles.graph.IGraph} this mode is acting upon.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.ItemDropInputMode.<T>} specialized to drag'n'drop {@link yfiles.graph.INode}s.
     * <p>
     * A dragged {@link yfiles.graph.INode} is visualized during the drag operation. The input mode does also support snapping of the
     * dragged {@link yfiles.graph.INode} via the {@link yfiles.input.SnapContext}. When the {@link yfiles.input.ItemDropInputMode.<T>#draggedItem dragged node} is dropped, This mode can can drag
     * nodes onto {@link yfiles.graph.IGraph#isGroupNode group} and optional {@link yfiles.graph.IFoldingView#collapse folder nodes} and automatically add the node to the corresponding group node in the
     * hierarchy. In this case the {@link yfiles.input.NodeDropInputMode#isValidParentPredicate} will be queried if the dropped upon node is actually a valid parent.
     * </p>
     * @see yfiles.input.NodeDropInputMode#startDrag
     * @class
     * @extends {yfiles.input.ItemDropInputMode.<yfiles.graph.INode>}
     */
    export interface NodeDropInputMode extends yfiles.input.ItemDropInputMode<yfiles.graph.INode>{}
    export class NodeDropInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.NodeDropInputMode} class for the expected data format.
       * @param expectedFormat This is a format that is used in {@link #startDrag}.
       * @protected
       */
      constructor(expectedFormat:string);
      /**
       * Initializes a new instance of the {@link yfiles.input.NodeDropInputMode} class for {@link yfiles.graph.INode}s.
       * <p>
       * This instance fires events only if the format of the data of the drag operation is the full name of the {@link yfiles.graph.INode}
       * type. Such a drag operation can be started for example with {@link yfiles.input.NodeDropInputMode#startDrag}.
       * </p>
       */
      constructor();
      /**
       * Creates the node in the graph after it's been dropped.
       * <p>
       * This method is called by the {@link yfiles.input.ItemDropInputMode.<T>#itemCreator} that is set as default on this class.
       * </p>
       * @param context The context for which the node should be created.
       * @param graph The {@link  Graph} in which to create the item.
       * @param node The node that was dragged and should therefore be created.
       * @param dropTarget The {@link } on which the node is dropped.
       * @param layout The bounds of the new node.
       * @returns a newly created node.
       * @protected
       */
      createNode(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,node:yfiles.graph.INode,dropTarget:yfiles.graph.IModelItem,layout:yfiles.geometry.Rect):yfiles.graph.INode;
      /**
       * Calculates the layout of the new node.
       * <p>
       * In this implementation the mouse location is used as center of the node. Can be overridden in child class to implement a
       * different layout.
       * </p>
       * @param mouseLocation Current mouse position
       * @param size Size of the node
       * @returns a {@link } with the given size and the mouse location as center.
       * @protected
       */
      getNodeLayout(mouseLocation:yfiles.geometry.Point,size:yfiles.geometry.Size):yfiles.geometry.Rect;
      /**
       * Starts a drag and drop operation for a {@link yfiles.graph.INode}.
       * <p>
       * This convenience implementation delegates to {@link yfiles.view.DragSource#startDrag} passing the <code>node</code> to the {@link yfiles.view.DragDropItem}
       * using the corresponding type.
       * </p>
       * <p>
       * If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
       * depending on the allowed drag-drop operations for these elements:
       * </p>
       * <ul>
       * <li><code>yfiles-cursor-dragdrop-move</code></li>
       * <li><code>yfiles-cursor-dragdrop-link</code></li>
       * <li><code>yfiles-cursor-dragdrop-all</code></li>
       * <li><code>yfiles-cursor-dragdrop-no-drop</code></li>
       * </ul>
       * @param dragSource The source of the drag operation.
       * @param node The node to drag.
       * @param [dragDropEffects=yfiles.view.DragDropEffects.ALL] The allowed drag drop effects.
       * @param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
       * @see yfiles.view.DragSource#startDrag
       * @see yfiles.view.DragDropItem
       * @static
       */
      static startDrag(dragSource:HTMLElement,node:yfiles.graph.INode,dragDropEffects?:yfiles.view.DragDropEffects,useCssCursors?:boolean):void;
      /**
       * Starts a drag and drop operation for a {@link yfiles.graph.INode}.
       * <p>
       * This convenience implementation delegates to {@link yfiles.view.DragSource#startDrag} passing the <code>node</code> to the {@link yfiles.view.DragDropItem}
       * using the corresponding type.
       * </p>
       * <p>
       * If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
       * depending on the allowed drag-drop operations for these elements:
       * </p>
       * <ul>
       * <li><code>yfiles-cursor-dragdrop-move</code></li>
       * <li><code>yfiles-cursor-dragdrop-link</code></li>
       * <li><code>yfiles-cursor-dragdrop-all</code></li>
       * <li><code>yfiles-cursor-dragdrop-no-drop</code></li>
       * </ul>
       * @param {Object} options The parameters to pass.
       * @param options.dragSource The source of the drag operation.
       * @param options.node The node to drag.
       * @param [options.dragDropEffects=yfiles.view.DragDropEffects.ALL] The allowed drag drop effects.
       * @param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
       * @see yfiles.view.DragSource#startDrag
       * @see yfiles.view.DragDropItem
       * @static
       */
      static startDrag(options:{dragSource:HTMLElement,node:yfiles.graph.INode,dragDropEffects?:yfiles.view.DragDropEffects,useCssCursors?:boolean}):void;
      /**
       * Gets or sets the callback for testing whether a {@link yfiles.input.ItemDropInputMode.<T>#draggedItem dragged node} should be created as a {@link yfiles.graph.IGraph#isGroupNode group node}.
       * <p>
       * The default implementation returns <code>false</code> always. Can be set to a different value to implement custom group node
       * detection.
       * </p>
       * @type {function(yfiles.graph.INode): boolean}
       */
      isGroupNodePredicate:(obj:yfiles.graph.INode)=>boolean;
      /**
       * Gets or sets the callback for testing whether an existing node in the graph can be used as a valid parent node in the
       * context of grouping.
       * <p>
       * The default implementation returns <code>true</code> always. Can be set to a different value to implement custom group node
       * detection. Note that the node passed to the predicate may be <code>null</code> to indicate that the node will be placed at the root
       * of the hierarchy.
       * </p>
       * @see yfiles.input.NodeDropInputMode#allowNonGroupNodeAsParent
       * @see yfiles.input.NodeDropInputMode#allowFolderNodeAsParent
       * @type {function(yfiles.graph.INode): boolean}
       */
      isValidParentPredicate:(obj:yfiles.graph.INode)=>boolean;
      /**
       * Gets or sets a value indicating whether nodes can be dropped on {@link yfiles.graph.IFoldingView#collapse collapsed} folder nodes.
       * <p>
       * If this property is set to <code>true</code>, dropping nodes on collapsed folder nodes will create the node inside the folder node in
       * the master graph. In that case the {@link yfiles.input.ItemDropInputMode.<T>#addItemCreatedListener ItemCreated} event will yield the node in the {@link yfiles.graph.FoldingManager#masterGraph master graph}. The node will
       * not be {@link yfiles.graph.IGraph#contains contained} in the currently visible graph. By default this feature is disabled. In any case the {@link yfiles.input.NodeDropInputMode#isValidParentPredicate} will be queried
       * additionally if the node is a valid parent.
       * </p>
       * @type {boolean}
       */
      allowFolderNodeAsParent:boolean;
      /**
       * Gets or sets a value indicating whether nodes can be dropped onto nodes that are no {@link yfiles.graph.IGraph#isGroupNode group nodes}.
       * <p>
       * If this property is enabled, dropping a node on a non-group node will effectively convert the non-group node to a real
       * group node. By default this feature is disabled. In any case the {@link yfiles.input.NodeDropInputMode#isValidParentPredicate} will be queried additionally if the node is a valid
       * parent.
       * </p>
       * @type {boolean}
       */
      allowNonGroupNodeAsParent:boolean;
      /**
       * Gets the currently dragged {@link yfiles.graph.IModelItem} instance.
       * <p>
       * This implementation simply tries to cast the {@link yfiles.input.DropInputMode#dropData} to {@link yfiles.graph.INode}.
       * </p>
       * @protected
       * @type {yfiles.graph.INode}
       */
      draggedItem:yfiles.graph.INode;
      static $class:yfiles.lang.Class;
    }
    /**
     * A helper class that can be used to interactively reshape orthogonal edge paths.
     * <p>
     * This class will be queried from the {@link yfiles.input.IInputModeContext} of implementations of {@link yfiles.input.IPositionHandler},
     * {@link yfiles.input.IHandle}, and {@link yfiles.input.IReshapeHandler} that support orthogonal editing of edges.
     * </p>
     * <p>
     * In a first step, the edit (which is a movement of one or more {@link yfiles.graph.IPort}s, {@link yfiles.graph.IBend}s, {@link yfiles.graph.IEdge edge ends}, or
     * {@link yfiles.graph.IPortOwner}s) is being {@link yfiles.input.OrthogonalEdgeEditingContext#initializeDrag initialized} before anything has been changed. Then, client code that seeks to modify
     * the graph, uses the methods {@link yfiles.input.OrthogonalEdgeEditingContext#addExplicitlyMovedBend},
     * {@link yfiles.input.OrthogonalEdgeEditingContext#addMovedEdgeEnd}, {@link yfiles.input.OrthogonalEdgeEditingContext#addMovedPort}, and
     * {@link yfiles.input.OrthogonalEdgeEditingContext#addTransformedPortOwner}. In the next step, the {@link yfiles.input.OrthogonalEdgeEditingContext#dragInitialized drag is being initialized} and
     * all affected edges will be {@link yfiles.input.OrthogonalEdgeEditingContext#prepareEdgePaths prepared} so that the editing will not destroy the orthogonality. Interested editors now get
     * the chance to register and negotiate {@link yfiles.input.OrthogonalEdgeEditingContext#addImplicitlyMovedBend implicitly moved} bends. Then the actual editing is performed and the edit is either
     * {@link yfiles.input.OrthogonalEdgeEditingContext#cancelDrag canceled}, in which case the {@link yfiles.input.OrthogonalEdgeEditingContext#removeAddedBends added bends are removed} or the edit is {@link yfiles.input.OrthogonalEdgeEditingContext#dragFinished finished} and the affected edges are being
     * {@link yfiles.input.OrthogonalEdgeEditingContext#cleanUp cleaned up}.
     * </p>
     * @see yfiles.input.IOrthogonalEdgeHelper
     * @see yfiles.input.OrthogonalEdgeEditingContext#enabled
     * @see yfiles.input.OrthogonalEdgeEditingContext#movePorts
     * @class
     */
    export interface OrthogonalEdgeEditingContext extends Object{}
    export class OrthogonalEdgeEditingContext {
      /**
       * Initializes a new instance of the {@link yfiles.input.OrthogonalEdgeEditingContext} class that is initially {@link yfiles.input.OrthogonalEdgeEditingContext#enabled}.
       */
      constructor();
      /**
       * Registers the provided bend with this instance so that it will be considered explicitly moved for this edit.
       * <p>
       * {@link yfiles.input.IDragHandler} implementations that are being used to explicitly move a bend during the edit should use this
       * method to register their edit while they are being {@link yfiles.input.IDragHandler#initializeDrag initialized}.
       * </p>
       * @param movementInfo The movement info that describes the movement of the bend.
       */
      addExplicitlyMovedBend(movementInfo:yfiles.input.MovementInfo):void;
      /**
       * Registers the provided bend as being moved implicitly for this edit.
       * <p>
       * The instance will be returned by {@link yfiles.input.OrthogonalEdgeEditingContext#createImplicitlyMovedBendInfo} and the {@link yfiles.input.MovementInfo#moveType} will
       * be combined with the requested move type.
       * </p>
       * @param movementInfo The movement info for the bend.
       * @returns The info that has been registered with this instance.
       */
      addImplicitlyMovedBend(movementInfo:yfiles.input.MovementInfo):yfiles.input.MovementInfo;
      /**
       * Registers the provided end of the edge with this instance so that it will be considered explicitly moved for this edit.
       * <p>
       * {@link yfiles.input.IDragHandler} implementations that are being used to explicitly move the end of an edge during the edit
       * should use this method to register their edit while they are being {@link yfiles.input.IDragHandler#initializeDrag initialized}.
       * </p>
       * @param movementInfo The movement info that describes the movement of the end of the edge. The {@link #movedItem} must be an {@link }.
       * @param sourceEnd <code>true</code> if the <code>movementInfo</code> represents the source end of the edge.
       * @see yfiles.input.IEdgePortHandleProvider
       */
      addMovedEdgeEnd(movementInfo:yfiles.input.MovementInfo,sourceEnd:boolean):void;
      /**
       * Registers the provided port with this instance so that it will be considered explicitly moved for this edit.
       * <p>
       * {@link yfiles.input.IDragHandler} implementations that are being used to explicitly move a bend during the edit should use this
       * method to register their edit while they are being {@link yfiles.input.IDragHandler#initializeDrag initialized}.
       * </p>
       * @param movementInfo The movement info that describes the movement of the port.
       */
      addMovedPort(movementInfo:yfiles.input.MovementInfo):void;
      /**
       * Adds an {@link yfiles.graph.IPortOwner} that is being transformed somehow during the edit so that the attached {@link yfiles.graph.IPortOwner#ports ports} will be
       * moved in an irregular ({@link yfiles.input.MoveTypes#ARBITRARY_MOVE non-linear}) way.
       * <p>
       * Normally this will be nodes that are being resized or moved in a non-linear way, or edges, whose bends or ports are
       * being changed.
       * </p>
       * @param owner The item that will be transformed.
       */
      addTransformedPortOwner(owner:yfiles.graph.IPortOwner):void;
      /**
       * Called by client edit code when a drag has been canceled.
       * <p>
       * Resets all internal state lists and sets {@link yfiles.input.OrthogonalEdgeEditingContext#isInitialized} back to <code>false</code>. Also {@link yfiles.input.OrthogonalEdgeEditingContext#removeAddedBends temporarily added bends are removed} and {@link yfiles.input.OrthogonalEdgeEditingContext#addCleanedUpListener CleanedUp}
       * will be triggered.
       * </p>
       */
      cancelDrag():void;
      /**
       * Performs clean up procedures.
       * <p>
       * This is called in response to {@link yfiles.input.OrthogonalEdgeEditingContext#cancelDrag} and {@link yfiles.input.OrthogonalEdgeEditingContext#dragFinished}
       * as well as initially during {@link yfiles.input.OrthogonalEdgeEditingContext#initializeDrag}.
       * </p>
       * @protected
       */
      cleanUp():void;
      /**
       * Cleans up the edge's path after a successfully {@link yfiles.input.OrthogonalEdgeEditingContext#dragFinished finished drag}.
       * <p>
       * This implementations delegates to the {@link yfiles.input.IOrthogonalEdgeHelper}'s {@link yfiles.input.IOrthogonalEdgeHelper#cleanUpEdge}
       * method.
       * </p>
       * @param graph The graph in which the edge resides.
       * @param edge The modified edge.
       * @see yfiles.input.OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper
       * @protected
       */
      cleanUpEdgePath(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge):void;
      /**
       * Called after a successfully {@link yfiles.input.OrthogonalEdgeEditingContext#dragFinished finished drag} to clean up artifacts of modified edges.
       * @param graph The graph.
       * @see yfiles.input.OrthogonalEdgeEditingContext#cleanUpEdgePath
       * @protected
       */
      cleanupEdgePaths(graph:yfiles.graph.IGraph):void;
      /**
       * Creates or obtains a previously created {@link yfiles.input.MovementInfo} for the provided implicitly moved bend.
       * <p>
       * This method is queried by client {@link yfiles.input.IDragHandler} implementations that have been initialized for the current
       * edit that need to move adjacent bends implicitly. Implementations should pass as the parameters the movement information
       * that controls the vertical or horizontal movement of the bend implicitly.
       * </p>
       * @param bend The implicitly moved bend.
       * @param verticalAdjacentInfo The movement info that implicitly constrains the vertical movement of the bend or <code>null</code>.
       * @param horizontalAdjacentInfo The movement info that implicitly constrains the horizontal movement of the bend or <code>null</code>.
       * @returns The info to be used by clients that requested it.
       */
      createImplicitlyMovedBendInfo(bend:yfiles.graph.IBend,horizontalAdjacentInfo:yfiles.input.MovementInfo,verticalAdjacentInfo:yfiles.input.MovementInfo):yfiles.input.MovementInfo;
      /**
       * Called when a drag has been successfully finished.
       * <p>
       * This method will perform the necessary clean up and {@link yfiles.input.OrthogonalEdgeEditingContext#cleanupEdgePaths clean up edge paths}.
       * </p>
       * @see yfiles.input.OrthogonalEdgeEditingContext#addCleanedUpListener
       */
      dragFinished():void;
      /**
       * Needs to be called by client editing code after {@link yfiles.input.OrthogonalEdgeEditingContext#initializeDrag} has been called
       * and all {@link yfiles.input.IDragHandler}s have been initialized.
       * <p>
       * This method will update the {@link yfiles.input.OrthogonalEdgeEditingContext#isInitializing} and {@link yfiles.input.OrthogonalEdgeEditingContext#isInitialized} properties accordingly and will {@link yfiles.input.OrthogonalEdgeEditingContext#prepareEdgePaths prepare} the edge paths. Finally, the {@link yfiles.input.OrthogonalEdgeEditingContext#addInitializedListener Initialized}
       * event is raised so that registered handlers can perform their post-initialization process.
       * </p>
       */
      dragInitialized():void;
      /**
       * Gets the {@link yfiles.input.MovementInfo movement infos} that describe the orthogonal path of the edge.
       * @param edge The edge to obtain the infos for.
       * @returns An array that contains for the first entry the source end of the edge, then all bends and then as the final entry the
       * target end of the edge. For the bends, this can be be <code>null</code> values if nothing is known about the bend, yet.
       * @protected
       */
      getMovementInfos(edge:yfiles.graph.IEdge):yfiles.collections.IListEnumerable<yfiles.input.MovementInfo>;
      /**
       * Callback method that gets the {@link yfiles.input.IOrthogonalEdgeHelper} instance associated with the given edge.
       * <p>
       * This implementation obtains the helper instance from the {@link yfiles.graph.ILookup#lookup} of the edge.
       * </p>
       * @param edge The edge to obtain the helper instance for.
       * @returns The helper instance to use or <code>null</code>, in which case the default behavior will be used.
       * @see yfiles.input.OrthogonalEdgeHelper
       * @protected
       */
      getOrthogonalEdgeHelper(edge:yfiles.graph.IEdge):yfiles.input.IOrthogonalEdgeHelper;
      /**
       * Gets the declared segment orientation for the provided segment at the given edge.
       * <p>
       * This implementation uses the {@link yfiles.input.OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper GetOrthogonalEdgeHelper()}.{@link yfiles.input.IOrthogonalEdgeHelper#getSegmentOrientation GetSegmentOrientation} method to yield the orientation.
       * </p>
       * @param edge The edge to determine the orientation of the segment.
       * @param segmentIndex Index of the segment.
       * @returns The orientation of the segment.
       */
      getSegmentOrientation(edge:yfiles.graph.IEdge,segmentIndex:number):yfiles.input.SegmentOrientation;
      /**
       * Called by client code when a drag is started about to be started.
       * <p>
       * This code needs to be called before any of the {@link yfiles.input.IDragHandler} implementations are initialized so that they
       * can then register the items they are going to modify with this instance. After this method has been called, the handlers
       * that perform the actual edit need to be initialized and as soon as this has been done, {@link yfiles.input.OrthogonalEdgeEditingContext#dragInitialized}
       * should be called.
       * </p>
       * @param context The context in which the edit is going to be performed.
       * @throws {Stubs.Exceptions.InvalidOperationError} If this context is already {@link yfiles.input.OrthogonalEdgeEditingContext#addInitializedListener Initialized}
       * or currently
       * {@link yfiles.input.OrthogonalEdgeEditingContext#addInitializingListener Initializing}.
       * @see yfiles.input.OrthogonalEdgeEditingContext#dragInitialized
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Callback that can be used by
       * {@link yfiles.input.IDragHandler}s and the like to determines whether the given edge is orthogonally edited edge in the
       * specified input mode context.
       * <p>
       * This implementation will always yield <code>false</code> if it is {@link yfiles.input.OrthogonalEdgeEditingContext#enabled disabled}. Otherwise {@link yfiles.input.OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper}
       * will be used to delegate the query to {@link yfiles.input.IOrthogonalEdgeHelper#shouldEditOrthogonally}.
       * </p>
       * @param context The input mode context which is editing the edge.
       * @param edge The edge that will be edited.
       * @returns <code>true</code> if this edge should be orthogonally edited for the specified input mode context; <code>false</code> otherwise.
       * @see yfiles.input.OrthogonalEdgeEditingContext#enabled
       * @see yfiles.input.IOrthogonalEdgeHelper#shouldEditOrthogonally
       */
      isOrthogonallyEditedEdge(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge):boolean;
      /**
       * Locks the movement of the ports of the edges so that {@link yfiles.input.OrthogonalEdgeEditingContext#shouldMoveEndImplicitly}
       * will yield <code>false</code> for the provided edge during the current edit.
       * <p>
       * This state will be reset after the current edit.
       * </p>
       * @param edge The edge to lock the ports of.
       * @see yfiles.input.OrthogonalEdgeEditingContext#lockedPortEdges
       */
      lockPortMovement(edge:yfiles.graph.IEdge):void;
      /**
       * Raises the {@link yfiles.input.OrthogonalEdgeEditingContext#addCleanedUpListener CleanedUp}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onCleanedUp(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.OrthogonalEdgeEditingContext#addInitializedListener Initialized}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.input.OrthogonalEdgeEditingContext#addInitializedListener
       * @see yfiles.input.OrthogonalEdgeEditingContext#dragInitialized
       * @protected
       */
      onInitialized(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Raises the {@link yfiles.input.OrthogonalEdgeEditingContext#addInitializingListener Initializing}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @see yfiles.input.OrthogonalEdgeEditingContext#addInitializingListener
       * @see yfiles.input.OrthogonalEdgeEditingContext#initializeDrag
       * @protected
       */
      onInitializing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called during {@link yfiles.input.OrthogonalEdgeEditingContext#dragInitialized} to prepare the edge paths for orthogonal
       * editing.
       * @param graph The graph that contains the edges to be edited.
       * @see yfiles.input.OrthogonalEdgeEditingContext#prepareOrthogonalEdge
       * @protected
       */
      prepareEdgePaths(graph:yfiles.graph.IGraph):void;
      /**
       * Helper method that inspects and prepares an orthogonal edge for the upcoming edit process.
       * <p>
       * This method inspects an edge and possibly inserts new bends into an edge to assure that during editing the orthogonality
       * won't be lost.
       * </p>
       * @param graph The graph to use for modifying the bends.
       * @param edge The edge to inspect.
       * @param orientations The orientations of the segments - the array is of length (edge.Bends.Count + 1).
       * @param infos The {@link } instances for the source end, the bends, and the target end. This array contains {@link }
       * instances representing either edge ends or ports at the first and last position and {@link }s or <code>null</code>
       * entries for the bends. The length thus is (edge.Bends.Count + 2) and <code>null</code> entries indicate that there is no information
       * about the movement (in case the bend will only be moved implicitly later, if at all).
       * @protected
       */
      prepareOrthogonalEdge(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,orientations:yfiles.collections.IListEnumerable<yfiles.input.SegmentOrientation>,infos:yfiles.collections.IListEnumerable<yfiles.input.MovementInfo>):void;
      /**
       * Can be used by subclasses during {@link yfiles.input.OrthogonalEdgeEditingContext#prepareOrthogonalEdge} to register added bends
       * that can later be {@link yfiles.input.OrthogonalEdgeEditingContext#removeAddedBends removed} in case the operation is canceled.
       * <p>
       * This method will only record the addition of bends if this instance is {@link yfiles.input.OrthogonalEdgeEditingContext#isInitialized initialized} or {@link yfiles.input.OrthogonalEdgeEditingContext#addInitializingListener still initializing}.
       * </p>
       * @param bend The bend that has been added to guarantee orthogonality.
       * @protected
       */
      registerAddedBend(bend:yfiles.graph.IBend):void;
      /**
       * Removes previously {@link yfiles.input.OrthogonalEdgeEditingContext#registerAddedBend registered} added bends.
       * @param graph The graph to use for removing the bends.
       * @param addedBends The added bends.
       * @see yfiles.input.OrthogonalEdgeEditingContext#addedBends
       * @see yfiles.input.OrthogonalEdgeEditingContext#registerAddedBend
       * @protected
       */
      removeAddedBends(graph:yfiles.graph.IGraph,addedBends:yfiles.collections.IEnumerable<yfiles.graph.IBend>):void;
      /**
       * Callback that can be used by {@link yfiles.input.IDragHandler}s and the like to determines whether the specified end of the
       * provided <code>edge</code> should be moved implicitly.
       * <p>
       * If an item is next to the end of an edge and the last segment should be kept orthogonal, this may only be possible if
       * the end of the edge is moved, too. This can be achieved by moving the edge to another {@link yfiles.graph.IPort}, or by moving
       * the port instance itself. This depends on the implementation of the {@link yfiles.input.IEdgePortHandleProvider} for the given
       * edge. This implementation will delegate to the {@link yfiles.input.IOrthogonalEdgeHelper#shouldMoveEndImplicitly} method of the
       * {@link yfiles.input.IOrthogonalEdgeHelper} instance returned by {@link yfiles.input.OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper}
       * unless the side of the edge is connected to an {@link yfiles.graph.IPortOwner} instance that is marked as {@link yfiles.input.OrthogonalEdgeEditingContext#addTransformedPortOwner transformed} during the
       * edit.
       * </p>
       * @param edge The edge for which it should be determined whether the edge end can be moved.
       * @param sourceSide if set to <code>true</code> the source side of the end is queried, else the target side.
       * @returns <code>true</code> if the specified side of the edge can be moved; <code>false</code> otherwise, in which case the segment should be split to
       * maintain orthogonality.
       * @see yfiles.input.OrthogonalEdgeEditingContext#getOrthogonalEdgeHelper
       * @see yfiles.input.IOrthogonalEdgeHelper#shouldMoveEndImplicitly
       * @see yfiles.input.OrthogonalEdgeEditingContext#movePorts
       */
      shouldMoveEndImplicitly(edge:yfiles.graph.IEdge,sourceSide:boolean):boolean;
      /**
       * Adds the given listener for the <code>CleanedUp</code> event that occurs when the recent edit operation has been cleaned up.
       * @param listener The listener to add.
       * @see yfiles.input.OrthogonalEdgeEditingContext#cleanUp
       * @see yfiles.input.OrthogonalEdgeEditingContext#onCleanedUp
       * @see yfiles.input.OrthogonalEdgeEditingContext#removeCleanedUpListener
       */
      addCleanedUpListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>CleanedUp</code> event that occurs when the recent edit operation has been cleaned up.
       * @param listener The listener to remove.
       * @see yfiles.input.OrthogonalEdgeEditingContext#cleanUp
       * @see yfiles.input.OrthogonalEdgeEditingContext#onCleanedUp
       * @see yfiles.input.OrthogonalEdgeEditingContext#addCleanedUpListener
       */
      removeCleanedUpListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Initializing</code> event that occurs when the edit is about to be initialized.
       * @param listener The listener to add.
       * @see yfiles.input.OrthogonalEdgeEditingContext#initializeDrag
       * @see yfiles.input.OrthogonalEdgeEditingContext#onInitializing
       * @see yfiles.input.OrthogonalEdgeEditingContext#removeInitializingListener
       */
      addInitializingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Initializing</code> event that occurs when the edit is about to be initialized.
       * @param listener The listener to remove.
       * @see yfiles.input.OrthogonalEdgeEditingContext#initializeDrag
       * @see yfiles.input.OrthogonalEdgeEditingContext#onInitializing
       * @see yfiles.input.OrthogonalEdgeEditingContext#addInitializingListener
       */
      removeInitializingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Initialized</code> event that occurs when the edit has been initialized.
       * @param listener The listener to add.
       * @see yfiles.input.OrthogonalEdgeEditingContext#dragInitialized
       * @see yfiles.input.OrthogonalEdgeEditingContext#onInitialized
       * @see yfiles.input.OrthogonalEdgeEditingContext#removeInitializedListener
       */
      addInitializedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Initialized</code> event that occurs when the edit has been initialized.
       * @param listener The listener to remove.
       * @see yfiles.input.OrthogonalEdgeEditingContext#dragInitialized
       * @see yfiles.input.OrthogonalEdgeEditingContext#onInitialized
       * @see yfiles.input.OrthogonalEdgeEditingContext#addInitializedListener
       */
      removeInitializedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Gets or sets a value indicating whether this instance is currently initializing.
       * @type {boolean}
       */
      isInitializing:boolean;
      /**
       * Gets or sets a value indicating whether orthogonal edge editing is enabled at all.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.OrthogonalEdgeEditingContext#isOrthogonallyEditedEdge
       * @see yfiles.input.IOrthogonalEdgeHelper#shouldEditOrthogonally
       * @default true
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * Gets or sets a value indicating ports are allowed to be moved at all.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.input.OrthogonalEdgeEditingContext#shouldMoveEndImplicitly
       * @see yfiles.input.IOrthogonalEdgeHelper#shouldMoveEndImplicitly
       * @default true
       * @type {boolean}
       */
      movePorts:boolean;
      /**
       * Gets the current input mode context that is in effect for the current edit.
       * @protected
       * @type {yfiles.input.IInputModeContext}
       */
      currentInputModeContext:yfiles.input.IInputModeContext;
      /**
       * Whether the context is initialized.
       * <p>
       * The context is initialized between the calls to {@link yfiles.input.OrthogonalEdgeEditingContext#initializeDrag} and {@link yfiles.input.OrthogonalEdgeEditingContext#dragFinished}
       * or {@link yfiles.input.OrthogonalEdgeEditingContext#cancelDrag}.
       * </p>
       * @type {boolean}
       */
      isInitialized:boolean;
      /**
       * Gets the previously {@link yfiles.input.OrthogonalEdgeEditingContext#registerAddedBend registered added} bends.
       * @protected
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IBend>}
       */
      addedBends:yfiles.collections.IEnumerable<yfiles.graph.IBend>;
      /**
       * Provides access to the enumeration of {@link yfiles.graph.IPortOwner} instances that are being transformed during the edit.
       * <p>
       * For elements in the enumerable the attached {@link yfiles.graph.IPortOwner#ports} are considered to be moved in an irregular way during the edit.
       * </p>
       * @see yfiles.input.MoveTypes#ARBITRARY_MOVE
       * @protected
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IPortOwner>}
       */
      transformedPortOwners:yfiles.collections.IEnumerable<yfiles.graph.IPortOwner>;
      /**
       * Provides access to the enumeration of {@link yfiles.graph.IEdge} instances whose ports have been locked at source and target
       * end.
       * <p>
       * For elements in the enumerable the end points should not be moved and thus {@link yfiles.input.OrthogonalEdgeEditingContext#shouldMoveEndImplicitly}
       * returns false for these edges. Note that this collection is reset for each edit.
       * </p>
       * @see yfiles.input.MoveTypes#ARBITRARY_MOVE
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IEdge>}
       */
      lockedPortEdges:yfiles.collections.IEnumerable<yfiles.graph.IEdge>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default implementation of the {@link yfiles.input.IOrthogonalEdgeHelper}
     * <p>
     * This class inspects the edges in question and infers the
     * {@link yfiles.input.SegmentOrientation}s of the segments from the current geometry of the edge path.
     * </p>
     * @class
     * @implements {yfiles.input.IOrthogonalEdgeHelper}
     */
    export interface OrthogonalEdgeHelper extends Object,yfiles.input.IOrthogonalEdgeHelper{}
    export class OrthogonalEdgeHelper {
      /**
       * Returns whether the given bend can be removed.
       * <p>
       * This implementation always returns <code>true</code>.
       * </p>
       * @param context The input mode context.
       * @param bend The bend that is designated for removal.
       * @returns 
       * @protected
       */
      canRemoveBend(context:yfiles.input.IInputModeContext,bend:yfiles.graph.IBend):boolean;
      /**
       * Callback method that is invoked after the provided edge has been edited orthogonally.
       * <p>
       * The default implementation in this class removes duplicate and collinear bends, as well as segments of zero length.
       * </p>
       * @param context The input mode context which edited the edge.
       * @param graph The graph to use for modifying the edge instance.
       * @param edge The edge to clean up the path.
       */
      cleanUpEdge(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge):void;
      /**
       * Gets the orientation of the given segment by looking at the geometry of the segment.
       * <p>
       * If the segment has a zero length, the adjacent segments will be inspected to infer the orientation such that the
       * orientation toggles between {@link yfiles.input.SegmentOrientation#VERTICAL} and {@link yfiles.input.SegmentOrientation#HORIZONTAL}.
       * </p>
       * @param context The input mode context in which the orientation is needed.
       * @param edge The edge to inspect.
       * @param segmentIndex The index of the segment.
       * @returns The declared orientation of the segment.
       */
      getSegmentOrientation(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,segmentIndex:number):yfiles.input.SegmentOrientation;
      /**
       * Determines whether the provided edge should be edited orthogonally in the specified input mode context.
       * <p>
       * If this method returns <code>false</code>, the other methods will not be queried at all.
       * </p>
       * <p>
       * The default implementation in this class always returns <code>true</code>.
       * </p>
       * @param context The input mode context in which the edge is about to be edited.
       * @param edge The edge to inspect.
       * @returns <code>true</code> if the edge should be orthogonally edited in the specified input mode context; <code>false</code> otherwise.
       */
      shouldEditOrthogonally(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge):boolean;
      /**
       * Determines whether this end of the provided edge can be moved in the input mode context.
       * <p>
       * This information is required to determine whether an edge's first or last segment needs to be split or the adjacent edge
       * end can be moved along with the other end of the segment.
       * </p>
       * <p>
       * The default implementation in this class always returns <code>false</code>.
       * </p>
       * @param context The input mode context in which the segment is edited.
       * @param edge The edge to inspect.
       * @param sourceEnd if set to <code>true</code> the source end of the edge is queried, otherwise the target end.
       * @returns <code>true</code> if the end of the edge can be moved for the specified input mode context; <code>false</code> otherwise, in which case the
       * corresponding segment needs to be split to keep the segment orientation orthogonal.
       */
      shouldMoveEndImplicitly(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,sourceEnd:boolean):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.input.IHandle} that can be used to move an {@link yfiles.graph.IPort} interactively by assigning
     * new {@link yfiles.graph.IPortLocationModelParameter}s to the {@link yfiles.graph.IPort#locationParameter}
     * <p>
     * This class also provides snapping facilities and can be used in conjunction with both
     * {@link yfiles.input.OrthogonalEdgeEditingContext}, as well as {@link yfiles.input.GraphSnapContext}.
     * </p>
     * @class
     * @implements {yfiles.input.IHandle}
     */
    export interface PortLocationModelParameterHandle extends Object,yfiles.input.IHandle{}
    export class PortLocationModelParameterHandle {
      /**
       * Initializes a new instance of the {@link yfiles.input.PortLocationModelParameterHandle} class to change the {@link yfiles.graph.IPortLocationModelParameter}
       * of the given <code>port</code>
       * @param port The port to adjust the location parameter of.
       */
      constructor(port:yfiles.graph.IPort);
      /**
       * 
       * @param context
       * @param originalLocation
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Called during the drag to collect snap results.
       * <p>
       * This implementation will delegate to the {@link yfiles.input.IPortSnapResultProvider} that has been received from the port this
       * instance is bound to.
       * </p>
       * @param source The source of the event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      collectSnapResults(source:Object,evt:yfiles.input.CollectSnapResultsEventArgs):void;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Gets the graph to use for {@link yfiles.graph.IGraph#setPortLocationParameter setting the parameter} from the context.
       * @param context The input mode context to use.
       * @returns The graph or <code>null</code> if the graph could not be obtained.
       * @see yfiles.input.IInputModeContext#graph
       * @protected
       */
      getGraph(context:yfiles.input.IInputModeContext):yfiles.graph.IGraph;
      /**
       * Determines the type of the movement for the given port.
       * <p>
       * This is used for the {@link yfiles.input.MovementInfo} that is passed to the {@link yfiles.input.OrthogonalEdgeEditingContext#addMovedPort}
       * method.
       * </p>
       * @param port The port to determine the move type for.
       * @returns This implementation returns {@link #LINEAR_MOVE} for models of type {@link }
       * and {@link #ARBITRARY_MOVE} for all others.
       * @protected
       */
      getMoveType(port:yfiles.graph.IPort):yfiles.input.MoveTypes;
      /**
       * Callback method that obtains the new parameter for the given location and port.
       * @param port The port to obtain a parameter for.
       * @param model The model to use.
       * @param location The new location.
       * @returns The new parameter to use.
       * @protected
       */
      getNewParameter(port:yfiles.graph.IPort,model:yfiles.graph.IPortLocationModel,location:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * 
       * @param context
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Callback method that applies the parameter.
       * @param graph The graph to use for applying.
       * @param port The port to apply the parameter to.
       * @param locationParameter The new parameter to apply.
       * @protected
       */
      setParameter(graph:yfiles.graph.IGraph,port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * 
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets the port this instance acts on.
       * @protected
       * @type {yfiles.graph.IPort}
       */
      port:yfiles.graph.IPort;
      /**
       * Gets or sets the type of the handle that can be used by the rendering engine to render types differently.
       * @type {yfiles.input.HandleTypes}
       */
      type:yfiles.input.HandleTypes;
      /**
       * Gets or sets the cursor to display when the mouse hovers over or drags this handle.
       * @type {yfiles.view.Cursor}
       */
      cursor:yfiles.view.Cursor;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple default implementation of {@link yfiles.input.IReparentNodeHandler} that treats shift key presses as reparent gestures
     * and allows for reparenting all nodes.
     * @class
     * @implements {yfiles.input.IReparentNodeHandler}
     */
    export interface ReparentNodeHandler extends Object,yfiles.input.IReparentNodeHandler{}
    export class ReparentNodeHandler {
      constructor();
      /**
       * 
       * @param context
       * @param node
       * @returns 
       */
      isReparentGesture(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode):boolean;
      /**
       * 
       * @param context
       * @param node
       * @param newParent
       * @returns 
       */
      isValidParent(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode,newParent:yfiles.graph.INode):boolean;
      /**
       * 
       * @param context
       * @param node
       * @param newParent
       */
      reparent(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode,newParent:yfiles.graph.INode):void;
      /**
       * 
       * @param context
       * @param node
       * @returns 
       */
      shouldReparent(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode):boolean;
      /**
       * Gets or sets the event recognizer that will be queried to determine whether the gesture is a reparent gesture.
       * <p>
       * The default is {@link yfiles.input.KeyEventRecognizers#SHIFT_PRESSED}
       * </p>
       * @see yfiles.input.ReparentNodeHandler#isReparentGesture
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      reparentRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.input.ISnapLineProvider} interface that provides {@link yfiles.input.OrthogonalSnapLine}s for
     * {@link yfiles.graph.INode}s.
     * <p>
     * This implementation uses the {@link yfiles.input.GraphSnapContext} to create snap lines around the geometric border of the
     * node.
     * </p>
     * @class
     * @implements {yfiles.input.ISnapLineProvider}
     */
    export interface NodeSnapLineProvider extends Object,yfiles.input.ISnapLineProvider{}
    export class NodeSnapLineProvider {
      /**
       * Callback method that is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines} the collect snap lines for the centers
       * of the node.
       * <p>
       * This method is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines} and by default used the {@link yfiles.input.NodeSnapLineProvider#createCenterSnapLine}
       * to add them to the {@link yfiles.input.CollectGraphSnapLinesEventArgs#addFixedNodeSnapLine fixed node} and {@link yfiles.input.CollectGraphSnapLinesEventArgs#addFixedSegmentSnapLine fixed segment} collections
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param node The node to add snap lines for.
       * @param layout The layout of the node to use for adding the snap lines.
       * @protected
       */
      addCenterSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Callback method that adds the fixed snap lines around the border of the nodes' <code>layout</code>.
       * <p>
       * This method is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines}
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param node The node to add snap lines for.
       * @param layout The layout of the node to use for adding the snap lines.
       * @protected
       */
      addFixedNodeBorderSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Callback method that adds the fixed snap lines around the node for other nodes to snap to using the {@link yfiles.input.GraphSnapContext#nodeToNodeDistance}.
       * <p>
       * This method is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines} and uses the {@link yfiles.input.GraphSnapContext#nodeToNodeDistance} to determine the location of the
       * segments to add around the <code>layout</code>.
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param node The node to add snap lines for.
       * @param layout The layout of the node to use for adding the snap lines.
       * @protected
       */
      addNodeToNodeSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Callback method that adds the fixed snap lines around the node for edge segments to snap to.
       * <p>
       * This method is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines} and uses the {@link yfiles.input.GraphSnapContext#nodeToEdgeDistance} to determine the location of the
       * segments to add around the <code>layout</code>.
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param node The node to add snap lines for.
       * @param layout The layout of the node to use for adding the snap lines.
       * @protected
       */
      addNodeToSegmentSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Callback method that is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines} the collect snap lines to snap ports to
       * at the provided node.
       * <p>
       * This method is called by {@link yfiles.input.NodeSnapLineProvider#addSnapLines}
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param node The node to add snap lines for.
       * @param layout The layout of the node to use for adding the snap lines.
       * @protected
       */
      addPortSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Called by the {@link yfiles.input.GraphSnapContext} when a {@link yfiles.input.GraphSnapContext#initializeDrag drag} is about to start.
       * <p>
       * This means that the provided <code>item</code>, which can be cast to {@link yfiles.graph.INode} will not be moved during the edit and thus
       * can contribute to the set of fixed snap lines. This implementation delegates to
       * {@link yfiles.input.NodeSnapLineProvider#addFixedNodeBorderSnapLines}, {@link yfiles.input.NodeSnapLineProvider#addPortSnapLines},
       * {@link yfiles.input.NodeSnapLineProvider#addNodeToNodeSnapLines}, and {@link yfiles.input.NodeSnapLineProvider#addCenterSnapLines}.
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param evt The argument to use for adding snap lines.
       * @param item The item to add snap lines for.
       */
      addSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,item:yfiles.graph.IModelItem):void;
      /**
       * Convenience method that creates a snap line for the center of nodes.
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param node The node to add snap lines for.
       * @param orientation The snap line orientation.
       * @param location The location of the snap line.
       * @returns A snap line that other centers can snap to.
       * @protected
       */
      createCenterSnapLine(context:yfiles.input.GraphSnapContext,node:yfiles.graph.INode,orientation:yfiles.input.SnapLineOrientation,location:yfiles.geometry.Point):yfiles.input.OrthogonalSnapLine;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple implementation of the {@link yfiles.input.ISnapLineProvider} interface that can be used to add snap lines for
     * orthogonally routed {@link yfiles.graph.IEdge}s.
     * <p>
     * This implementation can be put into the {@link yfiles.graph.ILookup#lookup} of an {@link yfiles.graph.IEdge}.
     * </p>
     * @see yfiles.graph.EdgeDecorator#snapLineProviderDecorator
     * @class
     * @implements {yfiles.input.ISnapLineProvider}
     */
    export interface EdgeSnapLineProvider extends Object,yfiles.input.ISnapLineProvider{}
    export class EdgeSnapLineProvider {
      /**
       * Adds horizontal snap lines for a horizontally oriented fixed segment snap lines.
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param edge The edge to add snap lines for.
       * @param segmentStart The start of the segment.
       * @param evt The argument to use for adding snap lines.
       * @param segmentEnd The end of the segment.
       * @see yfiles.input.EdgeSnapLineProvider#addVerticalSegmentSnapLines
       * @protected
       */
      addHorizontalSegmentSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,edge:yfiles.graph.IEdge,segmentStart:yfiles.geometry.Point,segmentEnd:yfiles.geometry.Point):void;
      /**
       * Adds the snap lines for the given model item to the <code>args</code>.
       * <p>
       * This implementation queries the {@link yfiles.input.GraphSnapContext#getMovementInfos movement information} for the provided edge to determine which edge segments stay fixed.
       * For each fixed segment, it will call {@link yfiles.input.EdgeSnapLineProvider#addVerticalSegmentSnapLines} or {@link yfiles.input.EdgeSnapLineProvider#addHorizontalSegmentSnapLines}
       * respectively.
       * </p>
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param item The item to add snap lines for.
       * @param evt The argument to use for adding snap lines.
       */
      addSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,item:yfiles.graph.IModelItem):void;
      /**
       * Adds vertical snap lines for a vertically oriented fixed segment snap lines.
       * @param context The context which holds the settings for the snap lines. Note that implementations should not change the state of the
       * context explicitly.
       * @param edge The edge to add snap lines for.
       * @param segmentStart The start of the segment.
       * @param segmentEnd The end of the segment.
       * @param evt The argument to use for adding snap lines.
       * @see yfiles.input.EdgeSnapLineProvider#addHorizontalSegmentSnapLines
       * @protected
       */
      addVerticalSegmentSnapLines(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectGraphSnapLinesEventArgs,edge:yfiles.graph.IEdge,segmentStart:yfiles.geometry.Point,segmentEnd:yfiles.geometry.Point):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * The default implementation of the {@link yfiles.input.INodeSnapResultProvider} interface.
     * @class
     * @implements {yfiles.input.INodeSnapResultProvider}
     */
    export interface NodeSnapResultProvider extends Object,yfiles.input.INodeSnapResultProvider{}
    export class NodeSnapResultProvider {
      /**
       * Calculates a {@link yfiles.input.SnapResult} and {@link yfiles.input.CollectSnapResultsEventArgs#addSnapResult adds it} to the argument.
       * <p>
       * This method delegates to {@link yfiles.input.NodeSnapResultProvider#addGridSnapResultCore} using {@link yfiles.input.GraphSnapContext#gridSnapType} and {@link yfiles.input.SnapPolicy#TO_NEAREST}
       * as the additional arguments. This method snaps the <code>location</code> to the next grid point or grid lines that are determined by
       * {@link yfiles.input.GraphSnapContext#nodeGridConstraintProvider} or obtained from the {@link yfiles.input.InputModeEventArgs#context}.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The arguments to add the results to.
       * @param location A location in the layout of the suggested layout of the node that will be snapped to a grid point location.
       * @param node The node that is currently being processed.
       * @protected
       */
      addGridSnapResult(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,location:yfiles.geometry.Point,node:yfiles.graph.INode):void;
      /**
       * Calculates a {@link yfiles.input.SnapResult} and {@link yfiles.input.CollectSnapResultsEventArgs#addSnapResult adds it} to the argument.
       * <p>
       * This method snaps the <code>location</code> to the next grid point or grid lines that are determined by {@link yfiles.input.GraphSnapContext#nodeGridConstraintProvider} or obtained from the {@link yfiles.input.InputModeEventArgs#context}.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The arguments to add the results to.
       * @param location A location in the layout of the suggested layout of the node that will be snapped to a grid point location.
       * @param node The node that is currently being processed.
       * @param gridSnapType The type of snapping that should be performed.
       * @param xSnapPolicy How to snap the x coordinate.
       * @param ySnapPolicy How to snap the y coordinate.
       * @protected
       */
      addGridSnapResultCore(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,location:yfiles.geometry.Point,node:yfiles.graph.INode,gridSnapType:yfiles.input.GridSnapTypes,xSnapPolicy:yfiles.input.SnapPolicy,ySnapPolicy:yfiles.input.SnapPolicy):void;
      /**
       * Collects snap results that snap the node to a grid and {@link yfiles.input.CollectSnapResultsEventArgs#addSnapResult adds them} to the argument.
       * <p>
       * This implementation simply delegates to {@link yfiles.input.NodeSnapResultProvider#addGridSnapResult} using the center of the
       * <code>suggestedLayout</code>.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The arguments to add the results to.
       * @param suggestedLayout The layout of the node if it would move without snapping.
       * @param node The node that is currently being processed.
       * @protected
       */
      collectGridSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLayout:yfiles.geometry.Rect,node:yfiles.graph.INode):void;
      /**
       * Collects the results for the given layout for all snap lines in question.
       * <p>
       * For each snap line that could be snapped to, this method calls
       * {@link yfiles.input.NodeSnapResultProvider#collectSnapResultsForSnapLine}.
       * </p>
       * @param context The context for which the results are being queried.
       * @param evt The {@link } instance to which the results should be {@link #addSnapResult added}.
       * @param suggestedLayout The suggested layout of the node.
       * @param node The node that is being dragged.
       * @protected
       */
      collectSnapLineSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLayout:yfiles.geometry.Rect,node:yfiles.graph.INode):void;
      /**
       * 
       * @param context
       * @param evt
       * @param suggestedLayout
       * @param node
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,suggestedLayout:yfiles.geometry.Rect,node:yfiles.graph.INode):void;
      /**
       * Verifies whether the node will snap to the given snap line.
       * <p>
       * If the node will snap, a {@link yfiles.input.SnapResult} for a {@link yfiles.input.SnapLine} will be created and added to the event
       * argument.
       * </p>
       * @param evt The arguments to add the results to.
       * @param snapLine The snap line to test snapping for.
       * @param suggestedLayout The layout of the node if it would move without snapping.
       * @param node The node that is currently being processed.
       * @protected
       */
      collectSnapResultsForSnapLine(evt:yfiles.input.CollectSnapResultsEventArgs,snapLine:yfiles.input.OrthogonalSnapLine,suggestedLayout:yfiles.geometry.Rect,node:yfiles.graph.INode):void;
      /**
       * Yields the static shared instance of this class.
       * @const
       * @static
       * @type {yfiles.input.INodeSnapResultProvider}
       */
      static INSTANCE:yfiles.input.INodeSnapResultProvider;
      static $class:yfiles.lang.Class;
    }
    /**
     * The default implementation of the {@link yfiles.input.INodeReshapeSnapResultProvider} interface.
     * @class
     * @implements {yfiles.input.INodeReshapeSnapResultProvider}
     */
    export interface NodeReshapeSnapResultProvider extends Object,yfiles.input.INodeReshapeSnapResultProvider{}
    export class NodeReshapeSnapResultProvider {
      /**
       * Adds a snap result for a location that snaps to a grid line.
       * <p>
       * This method is called by {@link yfiles.input.NodeReshapeSnapResultProvider#collectGridSnapResults} for each grid line snap.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data.
       * @param node The node that is being reshaped.
       * @param delta The amount the mouse needs to be moved in order to get to the snapping location.
       * @param snappedLocation The location of the mouse that is associated with the grid point.
       * @param snapLineType The type of snap line.
       * @param snapLineLocation The point at which the snap line is anchored.
       * @param from Where the snap line starts.
       * @param to Where the snap line ends.
       * @param snapLinePoint A point of interest on the snap line that can be highlighted.
       * @protected
       */
      addGridLineSnapResult(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,snapLineType:yfiles.input.SnapLineSnapTypes,snapLineLocation:yfiles.geometry.IPoint,delta:number,snappedLocation:yfiles.geometry.Point,from:number,to:number,snapLinePoint:yfiles.geometry.Point):void;
      /**
       * Adds a snap result for a location that snaps to the grid.
       * <p>
       * This method is called by {@link yfiles.input.NodeReshapeSnapResultProvider#collectGridSnapResults} for each grid snap.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data.
       * @param node The node that is being reshaped.
       * @param provider The provider that is associated with the grid snapping.
       * @param gridSnappedPoint The point on the grid that should be highlighted.
       * @param snappedLocation The location of the mouse that is associated with the grid point.
       * @protected
       */
      addGridSnapResult(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,provider:yfiles.input.IGridConstraintProvider<yfiles.graph.INode>,gridSnappedPoint:yfiles.geometry.Point,snappedLocation:yfiles.geometry.Point):void;
      /**
       * Adds a snap result for "same size" to the collect event argument.
       * <p>
       * This method is called by {@link yfiles.input.NodeReshapeSnapResultProvider#collectSameSizeSnapResults} for each result found.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data. This will be used to {@link #addSnapResult add} the result
       * to.
       * @param node The node that is being reshaped.
       * @param orientation The orientation of the result - indicates whether width or height have been snapped.
       * @param rectangles The rectangles that have the same width or height as the node will - this does not yet includes the rectangle of the
       * node being reshaped.
       * @param finalSize The target size to which the node will snap.
       * @param snapDelta The snap delta that the mouse needs to be moved in order to snap.
       * @protected
       */
      addSameSizeSnapResult(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,orientation:yfiles.input.SnapLineOrientation,rectangles:yfiles.collections.IEnumerable<yfiles.geometry.Rect>,finalSize:number,snapDelta:number):void;
      /**
       * Adds a snap result for a location that snaps the node's bounds to a snap line.
       * <p>
       * This method is called by {@link yfiles.input.NodeReshapeSnapResultProvider#collectSnapLineResults} for each snap line snap.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data.
       * @param node The node that is being reshaped.
       * @param snapLine The snap line to which the node can be snapped
       * @param delta The amount the mouse needs to be moved in order to get to the snapping location.
       * @param snapPoint The point of interest that can be highlighted on the snap line.
       * @protected
       */
      addSnaplineSnapResult(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,snapLine:yfiles.input.OrthogonalSnapLine,delta:number,snapPoint:yfiles.geometry.Point):void;
      /**
       * Collects the snap results for the grid.
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data.
       * @param node The node that is being reshaped.
       * @param reshapeContext The reshape context that describes how the node's layout is reshaped.
       * @param suggestedLayout The suggested layout of the node.
       * @protected
       */
      collectGridSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,reshapeContext:yfiles.input.ReshapeRectangleContext,suggestedLayout:yfiles.geometry.Rect):void;
      /**
       * Collects the snap results that make the node the same size as other elements.
       * <p>
       * This method ultimately delegates to {@link yfiles.input.NodeReshapeSnapResultProvider#addSameSizeSnapResult} to add the results
       * to the event argument.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data.
       * @param node The node that is being reshaped.
       * @param reshapeContext The reshape context that contains information about the nature of the resize.
       * @param suggestedLayout The layout of the node as it would be if the mouse location would not be snapped.
       * @protected
       */
      collectSameSizeSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,reshapeContext:yfiles.input.ReshapeRectangleContext,suggestedLayout:yfiles.geometry.Rect):void;
      /**
       * Collects the snap results that makes the node's bounds snap to the {@link yfiles.input.GraphSnapContext#getFixedNodeSnapLines fixed node snap lines}.
       * <p>
       * This method ultimately delegates to {@link yfiles.input.NodeReshapeSnapResultProvider#addSnaplineSnapResult} to add the results
       * to the event argument. Method {@link yfiles.input.NodeReshapeSnapResultProvider#getSnapLines} is used to query the snap lines
       * that will be checked by this method.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param evt The {@link } instance containing the event data.
       * @param node The node that is being reshaped.
       * @param reshapeContext The reshape context that contains information about the nature of the resize.
       * @param suggestedLayout The layout of the node as it would be if the mouse location would not be snapped.
       * @protected
       */
      collectSnapLineResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,reshapeContext:yfiles.input.ReshapeRectangleContext,suggestedLayout:yfiles.geometry.Rect):void;
      /**
       * Called when a node is {@link yfiles.input.IDragHandler#handleMove dragged} to add {@link yfiles.input.SnapResult}s for
       * {@link yfiles.input.OrthogonalSnapLine}s or sizes to which this node can potentially snap during resizing.
       * <p>
       * This implementation delegates to the following methods in order:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.NodeReshapeSnapResultProvider#collectGridSnapResults}</li>
       * <li>{@link yfiles.input.NodeReshapeSnapResultProvider#collectSameSizeSnapResults}</li>
       * <li>{@link yfiles.input.NodeReshapeSnapResultProvider#collectSnapLineResults}</li>
       * </ul>
       * @param context The snap context which manages the snap lines and the settings.
       * @param evt The event argument to obtain the context from and add the results to.
       * @param node The node that is being reshaped.
       * @param reshapeContext Carries information about the reshape process.
       */
      collectSnapResults(context:yfiles.input.GraphSnapContext,evt:yfiles.input.CollectSnapResultsEventArgs,node:yfiles.graph.INode,reshapeContext:yfiles.input.ReshapeRectangleContext):void;
      /**
       * Helper method for method {@link yfiles.input.NodeReshapeSnapResultProvider#collectSnapLineResults} that gets all the {@link yfiles.input.SnapLine}
       * that should be checked for the currently moved node.
       * <p>
       * This method aggregates the {@link yfiles.input.GraphSnapContext#getFixedNodeSnapLines fixed nodes' snaplines} and the {@link yfiles.input.GraphSnapContext#getAdditionalSnapLines additional snap lines} that are relevant to the node.
       * </p>
       * @param context The context in which the snapping is performed.
       * @param node The node that is being reshaped.
       * @param reshapeContext The reshape context that contains information about the nature of the resize.
       * @param suggestedLayout The layout of the node as it would be if the mouse location would not be snapped.
       * @returns 
       * @protected
       */
      getSnapLines(context:yfiles.input.GraphSnapContext,node:yfiles.graph.INode,reshapeContext:yfiles.input.ReshapeRectangleContext,suggestedLayout:yfiles.geometry.Rect):yfiles.collections.IEnumerable<yfiles.input.SnapLine>;
      /**
       * A shared singleton instance of this type.
       * @const
       * @static
       * @type {yfiles.input.INodeReshapeSnapResultProvider}
       */
      static INSTANCE:yfiles.input.INodeReshapeSnapResultProvider;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple default implementations of the {@link yfiles.input.IBendCreator} interface used during the creation of bends in the UI.
     * <p>
     * An instance of this class can be found in the {@link yfiles.graph.ILookup#lookup} of an edge, e.g.
     * </p>
     * @class
     * @implements {yfiles.input.IBendCreator}
     */
    export interface DefaultBendCreator extends Object,yfiles.input.IBendCreator{}
    export class DefaultBendCreator {
      /**
       * 
       * @param context
       * @param graph
       * @param edge
       * @param location
       * @returns 
       */
      createBend(context:yfiles.input.IInputModeContext,graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,location:yfiles.geometry.Point):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract convenience implementation of the {@link yfiles.input.IPortCandidateProvider} interface.
     * @class
     * @implements {yfiles.input.IPortCandidateProvider}
     */
    export interface PortCandidateProviderBase extends Object,yfiles.input.IPortCandidateProvider{}
    export class PortCandidateProviderBase {
      /**
       * Convenience method for subclasses that adds all existing {@link yfiles.graph.IPortOwner#ports} of the <code>owner</code> to the provided list.
       * @param owner The port owner whose ports should be added.
       * @param list The list to add candidates for existing ports to.
       * @protected
       */
      addExistingPorts(owner:yfiles.graph.IPortOwner,list:yfiles.collections.IList<yfiles.input.IPortCandidate>):void;
      /**
       * Factory method that creates a simple candidate that will use the {@link yfiles.input.PortCandidateProviderBase#createInstance}
       * method of this instance to delegate {@link yfiles.input.IPortCandidate#createPort} queries to.
       * @param locationParameter The location of the candidate. This instance is assigned by reference to the candidate.
       * @param owner The owner to use for the candidate.
       * @param [valid=true] Whether the port should be {@link #VALID}.
       * @returns A candidate whose {@link #createPort} method delegates to this instance's
       * {@link #createInstance}.
       * @protected
       */
      createCandidate(owner:yfiles.graph.IPortOwner,locationParameter:yfiles.graph.IPortLocationModelParameter,valid?:boolean):yfiles.input.DefaultPortCandidate;
      /**
       * Callback method used by the ports created using the factory methods
       * {@link yfiles.input.PortCandidateProviderBase#createCandidate}.
       * <p>
       * This method can be overridden by subclasses to perform more sophisticated port creation logic.
       * </p>
       * @param context The context in which the instance is created.
       * @param candidate The candidate to create a port for.
       * @returns The new port.
       * @protected
       */
      createInstance(context:yfiles.input.IInputModeContext,candidate:yfiles.input.DefaultPortCandidate):yfiles.graph.IPort;
      /**
       * Callback method used by the ports created using the factory methods
       * {@link yfiles.input.PortCandidateProviderBase#createCandidate}.
       * <p>
       * This method can be overridden by subclasses to perform more sophisticated candidate creation logic.
       * </p>
       * @param context The context in which the instance is created.
       * @param candidate The candidate to create a port for.
       * @param suggestedLocation The location to get a candidate for.
       * @returns The new candidate whose {@link #validity} may not be {@link #DYNAMIC}.
       * @protected
       */
      getPortCandidateAt(context:yfiles.input.IInputModeContext,candidate:yfiles.input.DefaultPortCandidate,suggestedLocation:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Creates an enumeration of possibly port candidates.
       * <p>
       * This method is used as a callback by most of the getter methods in this class. Subclasses should override this method to
       * provide the same candidates for all use-cases.
       * </p>
       * @param context The context for which the candidates should be provided.
       * @returns An enumerable collection of port candidates.
       * @protected
       * @abstract
       */
      getPortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Convenience implementation that simply delegates to {@link yfiles.input.PortCandidateProviderBase#getPortCandidates}.
       * @param context
       * @param target
       * @returns 
       */
      getSourcePortCandidates(context:yfiles.input.IInputModeContext,target:yfiles.input.IPortCandidate):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Convenience implementation that simply delegates to {@link yfiles.input.PortCandidateProviderBase#getPortCandidates}.
       * @param context
       * @returns 
       */
      getSourcePortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Convenience implementation that simply delegates to {@link yfiles.input.PortCandidateProviderBase#getPortCandidates}.
       * @param context
       * @param source
       * @returns 
       */
      getTargetPortCandidates(context:yfiles.input.IInputModeContext,source:yfiles.input.IPortCandidate):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Convenience implementation that simply delegates to {@link yfiles.input.PortCandidateProviderBase#getPortCandidates}.
       * @param context
       * @returns 
       */
      getTargetPortCandidates(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * Gets or sets the style to use for the creation of the port in {@link yfiles.input.PortCandidateProviderBase#createInstance}.
       * <p>
       * A value of <code>null</code> will make the implementation use the corresponding current default port style.
       * </p>
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * The tag to assign to the port in {@link yfiles.input.PortCandidateProviderBase#createInstance}
       * @type {Object}
       */
      tag:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A sophisticated implementation of the {@link yfiles.input.IHandle} interface that lets the user change an edge's {@link yfiles.graph.IEdge#sourcePort source} and
     * {@link yfiles.graph.IEdge#targetPort target port}.
     * <p>
     * This handle will move one of the edge's end points visually to another {@link yfiles.graph.IPort} or {@link yfiles.input.IPortCandidate}
     * and upon finalization of the gesture will {@link yfiles.input.PortRelocationHandle#setPorts set the new ports} for the edge. This class heavily relies on the
     * implementation of the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} that is queried from the {@link yfiles.graph.IEdge}
     * this handle is acting on.
     * </p>
     * @see yfiles.input.PortRelocationHandle#getPortCandidates
     * @see yfiles.graph.EdgeDecorator#edgeReconnectionPortCandidateProviderDecorator
     * @class
     * @implements {yfiles.input.IHandle}
     */
    export interface PortRelocationHandle extends Object,yfiles.input.IHandle{}
    export class PortRelocationHandle {
      /**
       * Creates a new instance of the {@link yfiles.input.PortRelocationHandle} class.
       * @param graph The graph or <code>null</code>. If the graph instance is not specified, the handle will try to receive the graph from the {@link }
       * it is passed in during {@link #initializeDrag}.
       * @param edge The edge.
       * @param sourceEnd if set to <code>true</code> the source end will be subject to relocation, otherwise it will be the target port.
       */
      constructor(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,sourceEnd:boolean);
      /**
       * 
       * @param context
       * @param originalLocation
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Factory method that creates the {@link yfiles.view.ICanvasObjectDescriptor} that will be used to paint the "current"
       * {@link yfiles.input.IPortCandidate}.
       * @returns The descriptor to use for the rendering in the {@link }.
       * @protected
       */
      createCurrentPortCandidateDescriptor():yfiles.view.ICanvasObjectDescriptor;
      /**
       * Factory method that creates the dummy edge that will be shown during the drag operation.
       * @param edge The edge to be represented by the dummy edge.
       * @returns The dummy edge instance to use.
       * @see yfiles.input.PortRelocationHandle#showDummyEdge
       * @protected
       */
      createDummyEdge(edge:yfiles.graph.IEdge):yfiles.graph.SimpleEdge;
      /**
       * Factory method that creates a dummy paintable for the dummy edge during the drag operation.
       * @param dummy The dummy to create a paintable for.
       * @returns A paintable that shows the dummy.
       * @see yfiles.input.PortRelocationHandle#showDummyEdge
       * @protected
       */
      createDummyEdgeVisualCreator(dummy:yfiles.graph.SimpleEdge):yfiles.view.IVisualCreator;
      /**
       * Factory method that creates the candidate for the existing port.
       * @param port The port to create a candidate for.
       * @returns The candidate or <code>null</code>.
       * @see yfiles.input.PortRelocationHandle#getPortCandidates
       * @see yfiles.input.PortRelocationHandle#addExistingPort
       * @protected
       */
      createExistingPortCandidate(port:yfiles.graph.IPort):yfiles.input.IPortCandidate;
      /**
       * Factory method that creates the {@link yfiles.view.ICanvasObjectDescriptor} that will be used to paint the
       * {@link yfiles.input.IPortCandidate}s.
       * @returns The descriptor to use for the rendering in the {@link }.
       * @protected
       */
      createPortCandidateDescriptor():yfiles.view.ICanvasObjectDescriptor;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Finds the closest candidate from the given set of candidates.
       * @param context The context in for which the closest handle candidate is sought.
       * @param location The location for which to find a candidate.
       * @param candidates The list of possible candidates.
       * @returns The closes candidate or <code>null</code>.
       * @protected
       */
      getClosestCandidate(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,candidates:yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>):yfiles.input.IPortCandidate;
      /**
       * Gets the graph to use from the context.
       * @param context The input mode context.
       * @returns A graph instance or <code>null</code>.
       * @protected
       */
      getGraph(context:yfiles.input.IInputModeContext):yfiles.graph.IGraph;
      /**
       * Callback used during {@link yfiles.input.PortRelocationHandle#dragFinished} to actually get the new port from the chosen
       * candidate.
       * <p>
       * This implementation returns the result of {@link yfiles.input.IPortCandidate#createPort} or calls {@link yfiles.graph.IGraph#addPort} if
       * the result was <code>null</code>.
       * </p>
       * @param context
       * @param portCandidate The candidate to get a port instance from.
       * @param suggestedLocation The suggested location for the port.
       * @returns A non-<code>null</code> port instance.
       * @see yfiles.input.PortRelocationHandle#setPorts
       * @protected
       */
      getPort(context:yfiles.input.IInputModeContext,portCandidate:yfiles.input.IPortCandidate,suggestedLocation:yfiles.geometry.Point):yfiles.graph.IPort;
      /**
       * Gets the possible candidates for the given edge.
       * <p>
       * This implementation uses the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} from the edge's {@link yfiles.graph.ILookup#lookup}
       * to retrieve the candidate sets. If {@link yfiles.input.PortRelocationHandle#addExistingPort} is enabled, the existing port will be part of the candidates.
       * </p>
       * @param context The context that is used to retrieve the candidates for.
       * @param edge The edge.
       * @param sourcePort Whether to look for source port candidates.
       * @returns A non-<code>null</code> enumerable over the candidates.
       * @protected
       */
      getPortCandidates(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,sourcePort:boolean):yfiles.collections.IEnumerable<yfiles.input.IPortCandidate>;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Hides the original edge that during the drag operation.
       * @param edge The edge to hide.
       * @param context The context to get the canvas the edge should be hidden from.
       * @see yfiles.input.PortRelocationHandle#unhideOriginalEdge
       * @see yfiles.input.PortRelocationHandle#showDummyEdge
       * @protected
       */
      hideOriginalEdge(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge):void;
      /**
       * 
       * @param context
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Determines whether {@link yfiles.input.PortRelocationHandle#resolveCandidate port candidate resolution} is enabled for the current gesture.
       * <p>
       * This implementation checks whether the the parent input mode is an instance of {@link yfiles.input.HandleInputMode}, the {@link yfiles.input.PortRelocationHandle#portCandidateResolutionRecognizer}
       * recognizes its {@link yfiles.view.MouseEventArgs}, and the {@link yfiles.input.HandleInputMode#currentHandle}'s location is equal to this {@link yfiles.input.PortRelocationHandle#location}.
       * </p>
       * @param context The context to inspect.
       * @returns <code>true</code> if port candidates may be resolved; <code>false</code> otherwise.
       * @protected
       */
      isPortCandidateResolutionEnabled(context:yfiles.input.IInputModeContext):boolean;
      /**
       * Tries to {@link yfiles.input.IPortCandidate#getPortCandidateAt resolve} a dynamic port candidate for the given location.
       * @param context The context in which the candidate is resolved.
       * @param portCandidate The candidate.
       * @param location The current location.
       * @returns A candidate.
       * @protected
       */
      resolveCandidate(context:yfiles.input.IInputModeContext,portCandidate:yfiles.input.IPortCandidate,location:yfiles.geometry.Point):yfiles.input.IPortCandidate;
      /**
       * Updates the closest candidate for visual feedback.
       * @param portCandidate
       * @protected
       */
      setClosestCandidate(portCandidate:yfiles.input.IPortCandidate):void;
      /**
       * Callback that is triggered by {@link yfiles.input.PortRelocationHandle#dragFinished} to actually change the port.
       * @param context The context.
       * @param edge The edge to change ports.
       * @param setSourcePort Whether to set the source port. <code>false</code> for target ports.
       * @param portCandidate The candidate that has been chosen.
       * @param suggestedLocation The suggested location for the port.
       * @protected
       */
      setPort(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,setSourcePort:boolean,portCandidate:yfiles.input.IPortCandidate,suggestedLocation:yfiles.geometry.Point):void;
      /**
       * Finally sets the ports for the edge to the new values.
       * @param context The context in which the ports are set.
       * @param edge The edge to set the ports for.
       * @param sourcePort The (possibly) new source port.
       * @param targetPort The (possibly) new target port.
       * @protected
       */
      setPorts(context:yfiles.input.IInputModeContext,edge:yfiles.graph.IEdge,sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):void;
      /**
       * Updates the position of the handle.
       * @param location The new location of the handle.
       * @protected
       */
      setPosition(location:yfiles.geometry.Point):void;
      /**
       * Updates the dummy edge's visual appearance to reflect the new port candidate.
       * @param dummy The dummy edge.
       * @param source Whether to update the source or target port.
       * @param portCandidate The new candidate to indicate.
       * @protected
       */
      setToPortCandidate(dummy:yfiles.graph.SimpleEdge,source:boolean,portCandidate:yfiles.input.IPortCandidate):void;
      /**
       * Unhides the original edge that was hidden during the drag operation.
       * @param edge The edge to unhide.
       * @param canvas The canvas the edge was hidden from.
       * @see yfiles.input.PortRelocationHandle#hideOriginalEdge
       * @see yfiles.input.PortRelocationHandle#showDummyEdge
       * @protected
       */
      unhideOriginalEdge(edge:yfiles.graph.IEdge,canvas:yfiles.view.CanvasComponent):void;
      /**
       * Gets or sets a value indicating whether to show the candidates of hit {@link yfiles.graph.IPortOwner}s only.
       * @type {boolean}
       */
      showHitPortOwnerCandidatesOnly:boolean;
      /**
       * Gets or sets the maximum distance the pointer may be away of a candidate in order to be snapped to the candidate.
       * <p>
       * The distance is measured in view coordinates. The default value is <code>50.0</code>.
       * </p>
       * @type {number}
       */
      maximumSnapDistance:number;
      /**
       * Determines whether or not edges are allowed to connect to other edges.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      allowEdgeToEdgeConnections:boolean;
      /**
       * Determines whether or not cyclic port dependencies are allowed.
       * <p>
       * If {@link yfiles.input.PortRelocationHandle#allowEdgeToEdgeConnections edge to edge connections} are allowed by this instance, this property determines whether
       * {@link yfiles.input.PortRelocationHandle#getClosestCandidate}s are will yield candidates that would result in the edge being
       * modified would create a cyclic dependency. The default is <code>false</code>.
       * </p>
       * @default false
       * @type {boolean}
       */
      allowCyclicPortDependencies:boolean;
      /**
       * Gets or sets a property that determines whether the existing port should be available as a possible port candidate,
       * too.
       * <p>
       * The default is <code>true</code>. If this property is set to <code>false</code>, {@link yfiles.input.PortRelocationHandle#getPortCandidates} will return
       * only the ports provided by the {@link yfiles.input.IPortCandidateProvider} in the node's lookup.
       * </p>
       * @type {boolean}
       */
      addExistingPort:boolean;
      /**
       * 
       * @type {yfiles.input.HandleTypes}
       */
      type:yfiles.input.HandleTypes;
      /**
       * 
       * @type {yfiles.view.Cursor}
       */
      cursor:yfiles.view.Cursor;
      /**
       * Gets or sets a value indicating whether a {@link yfiles.input.PortCandidateValidity#DYNAMIC dynamic port candidate} should be resolved for the current mouse location.
       * <p>
       * By default this feature is enabled and the {@link yfiles.input.PortRelocationHandle#portCandidateResolutionRecognizer} is configured so that the resolution mechanism is used if the user {@link yfiles.input.KeyEventRecognizers#SHIFT_PRESSED holds the shift modifier}
       * during the gesture.
       * </p>
       * @see yfiles.input.PortRelocationHandle#portCandidateResolutionRecognizer
       * @see yfiles.input.PortRelocationHandle#resolveCandidate
       * @default true
       * @type {boolean}
       */
      resolvePortCandidates:boolean;
      /**
       * Gets or sets the event recognizer that determines whether a {@link yfiles.input.PortCandidateValidity#DYNAMIC dynamic port candidate} should be resolved.
       * <p>
       * Note that this property has no effect if {@link yfiles.input.PortRelocationHandle#resolvePortCandidates} is set to <code>false</code>.
       * </p>
       * <p>
       * By default this is set to {@link yfiles.input.KeyEventRecognizers#SHIFT_PRESSED}.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      portCandidateResolutionRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets a value indicating whether the source end of the edge is handled by this instance.
       * @protected
       * @type {boolean}
       */
      sourceEnd:boolean;
      /**
       * Gets the edge this instance acts upon.
       * @protected
       * @type {yfiles.graph.IEdge}
       */
      edge:yfiles.graph.IEdge;
      /**
       * Gets the dummy edge this instance uses during the edit.
       * @see yfiles.input.PortRelocationHandle#showDummyEdge
       * @protected
       * @type {yfiles.graph.SimpleEdge}
       */
      dummyEdge:yfiles.graph.SimpleEdge;
      /**
       * Gets or sets a value indicating whether to show a dummy edge during the drag operation.
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      showDummyEdge:boolean;
      /**
       * Gets or sets the context to operate on.
       * @protected
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns a view of the location of the item.
       * <p>
       * This implementation returns <code>this</code>.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple implementation of a {@link yfiles.input.IHandleProvider} that can be used for {@link yfiles.graph.IEdge}s.
     * <p>
     * It will return the {@link yfiles.input.IHandle}s for the {@link yfiles.graph.IEdge#sourcePort source} and {@link yfiles.graph.IEdge#targetPort target} ports using {@link yfiles.input.PortRelocationHandle}s.
     * </p>
     * <p>
     * Note that this implementation also explicitly implements the {@link yfiles.input.IEdgePortHandleProvider} and that it is
     * recommended to decorate the {@link yfiles.input.IEdgePortHandleProvider} interface with edges in order to influence the handles
     * of the source and target port of the edge instead of directly decorating and implementing the {@link yfiles.input.IHandleProvider}
     * interface. The default implementation of the {@link yfiles.input.IHandleProvider} interface in the lookup of the
     * {@link yfiles.graph.IEdge}s (see {@link yfiles.graph.DefaultGraph#DEFAULT_EDGE_LOOKUP}) will use the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} interface to collect the
     * source and target port handles.
     * </p>
     * @class
     * @implements {yfiles.input.IHandleProvider}
     * @implements {yfiles.input.IEdgePortHandleProvider}
     */
    export interface PortRelocationHandleProvider extends Object,yfiles.input.IHandleProvider,yfiles.input.IEdgePortHandleProvider{}
    export class PortRelocationHandleProvider {
      /**
       * Initializes a new instance of the {@link yfiles.input.PortRelocationHandleProvider} class using the given graph and edge.
       * @param graph The graph or null. If the graph is null, the handles will try to receive the graph from the {@link }
       * it is used in.
       * @param edge The edge.
       */
      constructor(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge);
      /**
       * Factory method that creates the handle for this provider.
       * @param graph The graph that contains the edge.
       * @param edge The edge.
       * @param sourcePort if set to <code>true</code> the handle for the source port is sought.
       * @returns A handle or <code>null</code>.
       * @protected
       */
      createPortRelocationHandle(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,sourcePort:boolean):yfiles.input.IHandle;
      /**
       * 
       * @param context
       * @returns 
       */
      getHandles(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IHandle>;
      /**
       * The graph this provider is working on.
       * @protected
       * @const
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * The edge this provider is working on.
       * @protected
       * @const
       * @type {yfiles.graph.IEdge}
       */
      edge:yfiles.graph.IEdge;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple implementation of an {@link yfiles.input.IHandleProvider} that returns all handles in the lookup of the ports owned by
     * the given owner.
     * <p>
     * Using this class to display port handles will result in exceptions if the underlying port instances are removed from the
     * graph while the handles are displayed. This can happen, for example, through <code>Undo</code> or if ports are removed
     * programmatically with
     * {@link yfiles.graph.IGraph#remove}. In that case, you need to query the handles again before interacting with the graph, for
     * example by temporarily unselecting and reselecting the port-owning nodes.
     * </p>
     * @class
     * @implements {yfiles.input.IHandleProvider}
     */
    export interface PortsHandleProvider extends Object,yfiles.input.IHandleProvider{}
    export class PortsHandleProvider {
      /**
       * Creates an instance using the given owner as the provider for the ports.
       * @param owner
       */
      constructor(owner:yfiles.graph.IPortOwner);
      /**
       * Retrieves the handle implementation from the port's lookup.
       * @param port
       * @returns 
       * @protected
       */
      getHandle(port:yfiles.graph.IPort):yfiles.input.IHandle;
      /**
       * Returns a collection of zero or more {@link yfiles.input.IHandle} implementations that are associated with the ports of this
       * context.
       * @param context
       * @returns A collection of handles.
       */
      getHandles(context:yfiles.input.IInputModeContext):yfiles.collections.IEnumerable<yfiles.input.IHandle>;
      /**
       * The owner of the ports.
       * @protected
       * @const
       * @type {yfiles.graph.IPortOwner}
       */
      portOwner:yfiles.graph.IPortOwner;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specialized input mode that is used to resize a stripe by dragging one of its borders.
     * <p>
     * By default, adjacent stripes are moved. To redistribute the space instead, use {@link yfiles.input.ResizeStripeInputMode#resizeNeighborsRecognizer}. Also, by default the content of the
     * stripes is either moved or constrains the minimum available sizes for the stripe resize operations. To completely ignore
     * all content nodes, use {@link yfiles.input.ResizeStripeInputMode#ignoreContentRecognizer}.
     * </p>
     * <p>
     * This mode is {@link yfiles.input.ResizeStripeInputMode#exclusive} by default.
     * </p>
     * @class
     * @implements {yfiles.input.IInputMode}
     */
    export interface ResizeStripeInputMode extends Object,yfiles.input.IInputMode{}
    export class ResizeStripeInputMode {
      /**
       * Creates a new instance of this input mode.
       */
      constructor();
      /**
       * Cancels the editing of this mode.
       * <p>
       * This implementation {@link yfiles.input.ConcurrencyController#releaseMutex releases} the mutex if it is currently owned by this instance and calls
       * {@link yfiles.input.ResizeStripeInputMode#onCanceled}.
       * </p>
       * @see yfiles.input.ResizeStripeInputMode#onCanceled
       */
      cancel():void;
      /**
       * Installs this mode into the given context that is provided by the canvas.
       * <p>
       * In general a mode can only be installed into a single canvas at all times.
       * </p>
       * <p>
       * This method is called to initialize this instance. Subclasses should override this method to register the corresponding
       * event handler delegates for the various input events they need to register with.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation, first.
       * </p>
       * @param context The context that this instance shall be installed into. The same instance will be passed to this instance during
       * {@link #uninstall}. A reference to the context may be kept and queried during the time the mode is
       * installed.
       * @param controller The {@link #controller} for this mode.
       * @see yfiles.input.IInputMode#uninstall
       */
      install(context:yfiles.input.IInputModeContext,controller:yfiles.input.ConcurrencyController):void;
      /**
       * Called to determine whether it is valid to begin the gesture
       * <p>
       * This implementation checks whether a stripe border has been hit at the latest {@link yfiles.view.MouseEventArgs}
       * </p>
       * @param source
       * @param evt
       * @returns true iff a stripe border has been hit at the latest {@link }
       * @protected
       */
      isValidBegin(source:Object,evt:yfiles.lang.EventArgs):boolean;
      /**
       * Called after {@link yfiles.input.ResizeStripeInputMode#cancel} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been canceled.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onCanceled():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>true</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been activated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerActivated():void;
      /**
       * Called after the {@link yfiles.input.ConcurrencyController#active} property of the installed {@link yfiles.input.ConcurrencyController} has been set to <code>false</code>.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been deactivated.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation.
       * </p>
       * @protected
       */
      onConcurrencyControllerDeactivated():void;
      /**
       * Triggers the {@link yfiles.input.ResizeStripeInputMode#addDragCanceledListener DragCanceled} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceled(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.ResizeStripeInputMode#addDragCancelingListener DragCanceling} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragCanceling(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered once the drag has been finalized.
       * <p>
       * This method triggers the {@link yfiles.input.ResizeStripeInputMode#addDragFinishedListener DragFinished}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinished(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered before the drag will be finalized.
       * <p>
       * This method triggers the {@link yfiles.input.ResizeStripeInputMode#addDragFinishingListener DragFinishing}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragFinishing(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the end of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.ResizeStripeInputMode#addDraggedListener Dragged}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragged(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Callback triggered at the start of each drag.
       * <p>
       * This method triggers the {@link yfiles.input.ResizeStripeInputMode#addDraggingListener Dragging}
       * event.
       * </p>
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragging(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.ResizeStripeInputMode#addDragStartedListener DragStarted} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarted(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Triggers the {@link yfiles.input.ResizeStripeInputMode#addDragStartingListener DragStarting} event.
       * @param evt The event argument that contains context information.
       * @protected
       */
      onDragStarting(evt:yfiles.input.InputModeEventArgs):void;
      /**
       * Called after {@link yfiles.input.ResizeStripeInputMode#tryStop} has been called.
       * <p>
       * Can be overridden in subclasses to perform additional actions after the mode has been stopped.
       * </p>
       * <p>
       * This implementation does nothing.
       * </p>
       * @protected
       */
      onStopped():void;
      /**
       * 
       * @returns 
       */
      tryStop():boolean;
      /**
       * Uninstalls this mode from the given context.
       * <p>
       * This code should clean up all changes made to the canvas in the {@link yfiles.input.IInputMode#install} method. After a mode has
       * been uninstalled it can be installed again into the same or another canvas.
       * </p>
       * <p>
       * Overriding implementations should call the base implementation after their own code.
       * </p>
       * @param context The context to deregister from. This is the same instance that had been passed to {@link #install} during
       * installation.
       */
      uninstall(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @type {number}
       */
      priority:number;
      /**
       * Gets or sets a value indicating whether this mode will be the only one running when it has the mutex.
       * <p>
       * The value of this property will be delegated to the {@link yfiles.input.ConcurrencyController#exclusive} property of the {@link yfiles.input.ResizeStripeInputMode#controller}.
       * </p>
       * <p>
       * If this mode is marked as exclusive and has the mutex, all other modes {@link yfiles.input.MultiplexingInputMode#add added} to the same {@link yfiles.input.MultiplexingInputMode}
       * will be deactivated. Otherwise it will always run concurrently with all other modes.
       * </p>
       * @type {boolean}
       */
      exclusive:boolean;
      /**
       * Gets or sets the enabled state of this input mode.
       * <p>
       * Clients can use this property to disable or reenable this instance. This will set the {@link yfiles.input.ConcurrencyController#enabled Enabled} property of the installed
       * {@link yfiles.input.ResizeStripeInputMode#controller} so a disabled instance should never try to acquire the input mutex.
       * </p>
       * @type {boolean}
       */
      enabled:boolean;
      /**
       * 
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Returns the installed {@link yfiles.input.ResizeStripeInputMode#controller}.
       * @protected
       * @type {yfiles.input.ConcurrencyController}
       */
      controller:yfiles.input.ConcurrencyController;
      /**
       * Gets or sets the "pressed" state recognizer for touch input.
       * <p>
       * This recognizer instance will be used to determine when the user begins to move a handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer for touch input.
       * <p>
       * This recognizer instance determines when the user is moving a handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer for touch input.
       * <p>
       * This instance determines when the user has finished moving the handle.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizerTouch:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "pressed" state recognizer.
       * <p>
       * This recognizer instance will be used to determine when the user begins to resize the stripe.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      pressedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "dragged" recognizer.
       * <p>
       * This recognizer instance determines when the user is dragging the mouse to resize the stripe.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      draggedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "cancel" recognizer.
       * <p>
       * This recognizer recognizes a cancel action during the resize operation.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      cancelRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the "released" recognizer.
       * <p>
       * This instance determines when the user has finished the resize operation.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      releasedRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the recognizer that changes the behavior to resize adjacent stripes instead of moving them.
       * <p>
       * If this recognizer recognizes its gesture, the behavior will be to resize adjacent stripes instead of moving them (which
       * is the default).
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      resizeNeighborsRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets the recognizer that changes the behavior to ignore the content of all stripes instead of moving them or
       * considering for minimum sizes.
       * <p>
       * If this recognizer recognizes its gesture, the behavior will be to completely ignore all stripe contents.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      ignoreContentRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Adds the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDragFinishingListener
       */
      addDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinishing</code> event that occurs before the drag will be finished.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDragFinishingListener
       */
      removeDragFinishingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDragFinishedListener
       */
      addDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragFinished</code> event that occurs once the drag has been finished.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDragFinishedListener
       */
      removeDragFinishedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDragStartingListener
       */
      addDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarting</code> event that occurs once the drag is starting.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDragStartingListener
       */
      removeDragStartingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDragStartedListener
       */
      addDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragStarted</code> event that occurs once the drag is initialized and has started.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDragStartedListener
       */
      removeDragStartedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDraggingListener
       */
      addDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragging</code> event that occurs at the start of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDraggingListener
       */
      removeDraggingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDraggedListener
       */
      addDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>Dragged</code> event that occurs at the end of every drag.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDraggedListener
       */
      removeDraggedListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDragCanceledListener
       */
      addDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceled</code> event that occurs when the drag has been canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDragCanceledListener
       */
      removeDragCanceledListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to add.
       * @see yfiles.input.ResizeStripeInputMode#removeDragCancelingListener
       */
      addDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DragCanceling</code> event that occurs before the drag will be canceled.
       * @param listener The listener to remove.
       * @see yfiles.input.ResizeStripeInputMode#addDragCancelingListener
       */
      removeDragCancelingListener(listener:(sender:Object,evt:yfiles.input.InputModeEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface used for implementations that recognize, approve and disapprove stripe reparenting gestures, as well as
     * actually performs the reparenting.
     * @see yfiles.input.ReparentStripePositionHandler
     * @interface
     */
    export interface IReparentStripeHandler extends Object{
      /**
       * Determines whether the provided node may be reparented to a <code>newParent</code>.
       * <p>
       * This method is called to determine whether a {@link yfiles.input.IReparentStripeHandler#reparent} operation should actually be
       * executed.
       * </p>
       * @param context The context that provides information about the user input.
       * @param stripe The stripe that will be reparented.
       * @param newParent The potential new parent.
       * @param index The index where the stripe would be {@link #reparent reparented}
       * @param reparentPosition Where to place the stripe as a result of the gesture that would be used for the {@link #reparent }
       * operation.
       * @returns Whether <code>newParent</code> is a valid new parent for <code>stripe</code> for the given index and gesture.
       * @abstract
       */
      isValidParent(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe,newParent:yfiles.graph.IStripe,index:number,reparentPosition:yfiles.input.StripeReparentPolicy):boolean;
      /**
       * Performs the actual reparenting after the reparent gesture has been finalized.
       * <p>
       * Implementations should use {@link yfiles.graph.ITable#setParent} or {@link yfiles.graph.ITable#setParent} to set the parent of <code>movedStripe</code>
       * to <code>newParent</code>.
       * </p>
       * @param context The context that provides information about the user input.
       * @param movedStripe The stripe that will be reparented.
       * @param newParent The potential new parent.
       * @param index The index where the stripe should be inserted.
       * @param reparentPosition Where to place the stripe as a result of the gesture that triggered the reparent operation.
       * @abstract
       */
      reparent(context:yfiles.input.IInputModeContext,movedStripe:yfiles.graph.IStripe,newParent:yfiles.graph.IStripe,index:number,reparentPosition:yfiles.input.StripeReparentPolicy):void;
      /**
       * Determines whether the user may detach the given stripe from its current parent in order to reparent it.
       * @param context The context that provides information about the user input.
       * @param stripe The stripe that is about to be detached from its current parent.
       * @returns Whether the stripe may be detached and reparented.
       * @abstract
       */
      shouldReparent(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe):boolean;
    }
    var IReparentStripeHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Default implementation of the {@link yfiles.input.IReparentStripeHandler} interface.
     * <p>
     * This implementation allows to specify a maximal nesting depth for reparent operations.
     * </p>
     * @class
     * @implements {yfiles.input.IReparentStripeHandler}
     */
    export interface ReparentStripeHandler extends Object,yfiles.input.IReparentStripeHandler{}
    export class ReparentStripeHandler {
      constructor();
      /**
       * Adjust the size of the source or the target stripe.
       * <p>
       * This implementation adjusts the size of <code>stripe</code> to <code>Math.Max(originalStripeSize, originalParentSize)</code> if <code>reparentPosition</code>
       * is an {@link yfiles.input.StripeReparentPolicy#ADD_CHILD} operation
       * </p>
       * @param context The context that provides information about the user input.
       * @param stripe The stripe that will be reparented.
       * @param newParent The potential new parent.
       * @param index The index where the stripe should be inserted.
       * @param reparentPosition Where to place the stripe as a result of the gesture that triggered the reparent operation.
       * @param originalStripeSize The original size of <code>stripe</code>
       * @param originalParentSize The original size of <code>newParent</code>
       * @protected
       */
      adjustSize(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe,newParent:yfiles.graph.IStripe,reparentPosition:yfiles.input.StripeReparentPolicy,index:number,originalStripeSize:number,originalParentSize:number):void;
      /**
       * Checks the constraints imposed by {@link yfiles.input.ReparentStripeHandler#maxRowLevel} or {@link yfiles.input.ReparentStripeHandler#maxColumnLevel} for a valid gesture.
       * @param context The context that provides information about the user input.
       * @param stripe The stripe that will be reparented.
       * @param newParent The potential new parent.
       * @param index The index where the stripe would be {@link #reparent reparented}
       * @param reparentPosition Where to place the stripe as a result of the gesture that would be used for the {@link #reparent }
       * operation.
       * @returns <code>true</code> iff the resulting nesting depth is smaller than {@link #maxRowLevel} or {@link #maxColumnLevel}, or if the nesting depth would not increase by the operation.
       */
      isValidParent(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe,newParent:yfiles.graph.IStripe,index:number,reparentPosition:yfiles.input.StripeReparentPolicy):boolean;
      /**
       * Performs the actual reparenting after the reparent gesture has been finalized.
       * <p>
       * This implementation adjusts the size of the moved and/or the target stripe by calling {@link yfiles.input.ReparentStripeHandler#adjustSize}
       * .
       * </p>
       * @param context The context that provides information about the user input.
       * @param stripe The stripe that will be reparented.
       * @param newParent The potential new parent.
       * @param index The index where the stripe should be inserted.
       * @param reparentPosition Where to place the stripe as a result of the gesture that triggered the reparent operation.
       */
      reparent(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe,newParent:yfiles.graph.IStripe,index:number,reparentPosition:yfiles.input.StripeReparentPolicy):void;
      /**
       * Determines whether the user may detach the given stripe from its current parent in order to reparent it.
       * @param context The context that provides information about the user input.
       * @param stripe The stripe that is about to be detached from its current parent.
       * @returns This implementation returns always <code>true</code>.
       */
      shouldReparent(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe):boolean;
      /**
       * The maximum nesting level the row hierarchy may acquire during a reparent gesture.
       * <p>
       * Note that it is always possible to decrease the nesting depth, even if the resulting depth would still be too high. This
       * allows to interactively reduce an invalid nesting depth in multiple steps. The default value is
       * <code>0x7FFFFFFF</code>, which effectively means an unlimited nesting depth.
       * </p>
       * @type {number}
       */
      maxRowLevel:number;
      /**
       * The maximum nesting level the column hierarchy may acquire during a reparent gesture.
       * <p>
       * Note that it is always possible to decrease the nesting depth, even if the resulting depth would still be too high. This
       * allows to interactively reduce an invalid nesting depth in multiple steps. The default value is
       * <code>0x7FFFFFFF</code>, which effectively means an unlimited nesting depth.
       * </p>
       * @type {number}
       */
      maxColumnLevel:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specialized input mode that is used to reparent a stripe inside an {@link yfiles.graph.ITable} structure.
     * <p>
     * This implementation delegates most work to an instance of {@link yfiles.input.ReparentStripePositionHandler}
     * </p>
     * @class
     * @extends {yfiles.input.MoveInputMode}
     */
    export interface ReparentStripeInputMode extends yfiles.input.MoveInputMode{}
    export class ReparentStripeInputMode {
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.input.ItemDropInputMode.<T>} specialized to drag 'n' drop {@link yfiles.graph.IStripe}s.
     * <p>
     * A dragged {@link yfiles.graph.IStripe} is optionally visualized during the drag operation.
     * </p>
     * @see yfiles.input.StripeDropInputMode#startDrag
     * @class
     * @extends {yfiles.input.ItemDropInputMode.<yfiles.graph.IStripe>}
     */
    export interface StripeDropInputMode extends yfiles.input.ItemDropInputMode<yfiles.graph.IStripe>{}
    export class StripeDropInputMode {
      /**
       * Initializes a new instance of the {@link yfiles.input.StripeDropInputMode} class for the expected data format.
       * <p>
       * You might want to adjust the {@link yfiles.input.StripeDropInputMode#draggedItem} getter, too.
       * </p>
       * @param expectedFormat This is a format that is used in {@link #startDrag}.
       * @protected
       */
      constructor(expectedFormat:string);
      /**
       * Initializes a new instance of the {@link yfiles.input.NodeDropInputMode} class for {@link yfiles.graph.INode}s.
       * <p>
       * This instance fires events only if the format of the data of the drag operation is the full name of the {@link yfiles.graph.IStripe}
       * type. Such a drag operation can be started for example with {@link yfiles.input.StripeDropInputMode#startDrag}.
       * </p>
       */
      constructor();
      /**
       * Create a preview version of the table that is temporarily used to show the dragged stripe.
       * @returns 
       * @protected
       */
      createPreviewTable():yfiles.graph.ITable;
      /**
       * Create a new stripe as the result of a drag and drop gesture.
       * <p>
       * This method is called by the {@link yfiles.input.ItemDropInputMode.<T>#itemCreator} that is set as default on this class.
       * </p>
       * @param newParent The parent for which to create the stripe.
       * @param context The context for which the stripe should be created.
       * @returns a newly created stripe.
       * @protected
       */
      createStripe(context:yfiles.input.IInputModeContext,newParent:yfiles.graph.IStripe):yfiles.graph.IStripe;
      /**
       * Calculates the layout of the new table.
       * <p>
       * In this implementation the mouse location is used as center of the table. Can be overridden in child class to implement
       * a different layout.
       * </p>
       * @param location Current mouse position
       * @param size Size of the table
       * @returns a {@link } with the given size and the mouse location as center.
       * @protected
       */
      getPreviewTableLayout(location:yfiles.geometry.Point,size:yfiles.geometry.Size):yfiles.geometry.Rect;
      /**
       * Called whenever a new stripe is created
       * @param evt
       * @protected
       */
      onStripeCreated(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>):void;
      /**
       * Starts a drag and drop operation for an {@link yfiles.graph.IStripe}.
       * <p>
       * This convenience implementation delegates to {@link yfiles.view.DragSource#startDrag} passing the <code>stripe</code> to the {@link yfiles.view.DragDropItem}
       * using the corresponding type.
       * </p>
       * <p>
       * If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
       * depending on the allowed drag-drop operations for these elements:
       * </p>
       * <ul>
       * <li><code>yfiles-cursor-dragdrop-move</code></li>
       * <li><code>yfiles-cursor-dragdrop-link</code></li>
       * <li><code>yfiles-cursor-dragdrop-all</code></li>
       * <li><code>yfiles-cursor-dragdrop-no-drop</code></li>
       * </ul>
       * @param dragSource The source of the drag operation.
       * @param stripe The stripe to drag.
       * @param [dragDropEffects=yfiles.view.DragDropEffects.ALL] The allowed drag drop effects.
       * @param [useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
       * @see yfiles.view.DragSource#startDrag
       * @see yfiles.view.DragDropItem
       * @static
       */
      static startDrag(dragSource:HTMLElement,stripe:yfiles.graph.IStripe,dragDropEffects?:yfiles.view.DragDropEffects,useCssCursors?:boolean):void;
      /**
       * Starts a drag and drop operation for an {@link yfiles.graph.IStripe}.
       * <p>
       * This convenience implementation delegates to {@link yfiles.view.DragSource#startDrag} passing the <code>stripe</code> to the {@link yfiles.view.DragDropItem}
       * using the corresponding type.
       * </p>
       * <p>
       * If <code>useCssCursors</code> is enabled, the following CSS classes are assigned to elements hovered during a drag operation,
       * depending on the allowed drag-drop operations for these elements:
       * </p>
       * <ul>
       * <li><code>yfiles-cursor-dragdrop-move</code></li>
       * <li><code>yfiles-cursor-dragdrop-link</code></li>
       * <li><code>yfiles-cursor-dragdrop-all</code></li>
       * <li><code>yfiles-cursor-dragdrop-no-drop</code></li>
       * </ul>
       * @param {Object} options The parameters to pass.
       * @param options.dragSource The source of the drag operation.
       * @param options.stripe The stripe to drag.
       * @param [options.dragDropEffects=yfiles.view.DragDropEffects.ALL] The allowed drag drop effects.
       * @param [options.useCssCursors=true] Whether to adjust the classes of elements being hovered over during the drag. The default is <code>true</code>.
       * @see yfiles.view.DragSource#startDrag
       * @see yfiles.view.DragDropItem
       * @static
       */
      static startDrag(options:{dragSource:HTMLElement,stripe:yfiles.graph.IStripe,dragDropEffects?:yfiles.view.DragDropEffects,useCssCursors?:boolean}):void;
      /**
       * Gets the currently dragged {@link yfiles.graph.IModelItem} instance.
       * <p>
       * This implementation simply tries to cast the {@link yfiles.input.DropInputMode#dropData} to {@link yfiles.graph.INode}.
       * </p>
       * @protected
       * @type {yfiles.graph.IStripe}
       */
      draggedItem:yfiles.graph.IStripe;
      /**
       * Adds the given listener for the <code>StripeCreated</code> event that occurs when a new stripe gets created by this input mode.
       * @param listener The listener to add.
       * @see yfiles.input.StripeDropInputMode#removeStripeCreatedListener
       */
      addStripeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Removes the given listener for the <code>StripeCreated</code> event that occurs when a new stripe gets created by this input mode.
       * @param listener The listener to remove.
       * @see yfiles.input.StripeDropInputMode#addStripeCreatedListener
       */
      removeStripeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Customized {@link yfiles.input.IPositionHandler} implementation that moves a stripe in the table hierarchy when the stripe is
     * dragged.
     * <p>
     * This class is used by default by both {@link yfiles.input.ReparentStripeInputMode} and {@link yfiles.input.StripeDropInputMode} to
     * perform the actual reparenting and insertion.
     * </p>
     * @class
     * @implements {yfiles.input.IPositionHandler}
     */
    export interface ReparentStripePositionHandler extends Object,yfiles.input.IPositionHandler{}
    export class ReparentStripePositionHandler {
      /**
       * Creates a new instance of this position handler which moves <code>movedStripe</code>
       * @param movedStripe
       */
      constructor(movedStripe:yfiles.graph.IStripe);
      /**
       * 
       * @param context
       * @param originalLocation
       */
      cancelDrag(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point):void;
      /**
       * Create a visualization for the source region of the reparent gesture.
       * <p>
       * The default implementation uses the return value of {@link yfiles.input.IStripeInputVisualizationHelper#getVisualCreator} with
       * type {@link yfiles.input.StripeVisualizationType#DRAG_SOURCE}.
       * </p>
       * @param context The current input mode context
       * @returns A canvas object that is used to visualize the target region.
       * @see yfiles.input.ReparentStripePositionHandler#updateSourceVisualization
       * @protected
       */
      createSourceGhostVisualization(context:yfiles.input.IInputModeContext):yfiles.view.ICanvasObject;
      /**
       * Create a visualization for the target region of the reparent gesture.
       * <p>
       * The default implementation uses the return value of {@link yfiles.input.IStripeInputVisualizationHelper#getVisualCreator} with
       * type {@link yfiles.input.StripeVisualizationType#DROP_TARGET}.
       * </p>
       * @param context The current input mode context
       * @param stripe The stripe for which the visualization should be created.
       * @returns A canvas object that is used to visualize the target region.
       * @see yfiles.input.ReparentStripePositionHandler#updateTargetVisualization
       * @protected
       */
      createTargetGhostVisualization(context:yfiles.input.IInputModeContext,stripe:yfiles.graph.IStripe):yfiles.view.ICanvasObject;
      /**
       * Determine the reparent gesture that would result from the given parameters.
       * <p>
       * The target region has already been determined by {@link yfiles.input.ReparentStripePositionHandler#getTargetSubregion}
       * </p>
       * @param context The current input mode context
       * @param location The current drag location
       * @param sourceStripe The stripe that is moved
       * @param targetSubregion The stripe subregion for the target.
       * @param targetBounds The target bounds
       * @returns A {@link } that specifies the operation to perform.
       * @protected
       */
      determineGesture(context:yfiles.input.IInputModeContext,location:yfiles.geometry.IPoint,sourceStripe:yfiles.graph.IStripe,targetSubregion:yfiles.input.StripeSubregion,targetBounds:yfiles.geometry.Rect):yfiles.input.StripeReparentPolicy;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      dragFinished(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * Finds the target stripe subregion at <code>location</code> and its associated owner node.
       * @param location The hit location.
       * @returns A {@link } that lies at <code>location</code>, or null if no such region could be found.
       * @protected
       */
      getTargetSubregion(location:yfiles.geometry.Point):yfiles.input.StripeSubregion;
      /**
       * 
       * @param context
       * @param originalLocation
       * @param newLocation
       */
      handleMove(context:yfiles.input.IInputModeContext,originalLocation:yfiles.geometry.Point,newLocation:yfiles.geometry.Point):void;
      /**
       * 
       * @param context
       */
      initializeDrag(context:yfiles.input.IInputModeContext):void;
      /**
       * Called whenever a change in the oldTable hierarchy occurs through this position handler
       * @param evt
       * @protected
       */
      onStripeChanged(evt:yfiles.graph.StripeEventArgs):void;
      /**
       * 
       * @param location
       */
      setPosition(location:yfiles.geometry.Point):void;
      /**
       * Modifies the source visualization itself
       * <p>
       * The default does not change <code>targetVisualization</code>.
       * </p>
       * @param context The current input mode context.
       * @param targetVisualization The canvas object for the target visualization
       * @param targetStripe The target for the reparent gesture
       * @param reparentPosition Where to place the stripe after reparenting.
       * @returns 
       * @protected
       */
      updateSourceVisualization(context:yfiles.input.IInputModeContext,targetVisualization:yfiles.view.ICanvasObject,targetStripe:yfiles.graph.IStripe,reparentPosition:yfiles.input.StripeReparentPolicy):yfiles.view.ICanvasObject;
      /**
       * Modifies the target visualization itself.
       * <p>
       * The default implementation only hides <code>targetVisualization</code> iff <code>reparentPosition</code> is
       * {@link yfiles.input.StripeReparentPolicy#INVALID}.
       * </p>
       * @param context The current input mode context.
       * @param targetVisualization The canvas object for the target visualization.
       * @param targetStripe The target for the reparent gesture.
       * @param reparentPosition The mode the describes the results of the reparent gesture.
       * @param targetBounds The current visualization bounds that have been computed with
       * {@link #updateTargetVisualizationBounds}
       * @returns 
       * @protected
       */
      updateTargetVisualization(context:yfiles.input.IInputModeContext,targetVisualization:yfiles.view.ICanvasObject,targetStripe:yfiles.graph.IStripe,reparentPosition:yfiles.input.StripeReparentPolicy,targetBounds:yfiles.geometry.Rect):yfiles.view.ICanvasObject;
      /**
       * Updates the visualization bounds for the target visualization depending on the provided values.
       * @param originalTargetBounds The target bounds prior to this method call.
       * @param reparentPosition Where to place the stripe after the reparenting operations according to the reparent gesture.
       * @param targetStripe The target stripe which has been determined by the input mode.
       * @returns Updated bounds for the target visualization.
       * @protected
       */
      updateTargetVisualizationBounds(originalTargetBounds:yfiles.geometry.Rect,reparentPosition:yfiles.input.StripeReparentPolicy,targetStripe:yfiles.graph.IStripe):yfiles.geometry.Rect;
      /**
       * 
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Allows to query the moved stripe.
       * @type {yfiles.graph.IStripe}
       */
      movedStripe:yfiles.graph.IStripe;
      /**
       * The current {@link yfiles.input.IInputModeContext}
       * @protected
       * @type {yfiles.input.IInputModeContext}
       */
      inputModeContext:yfiles.input.IInputModeContext;
      /**
       * Adds the given listener for the <code>StripeChanged</code> event that occurs when a stripe state is changed by this input mode.
       * @param listener The listener to add.
       * @see yfiles.input.ReparentStripePositionHandler#removeStripeChangedListener
       */
      addStripeChangedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>StripeChanged</code> event that occurs when a stripe state is changed by this input mode.
       * @param listener The listener to remove.
       * @see yfiles.input.ReparentStripePositionHandler#addStripeChangedListener
       */
      removeStripeChangedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Input mode that can be used to interactively edit a table.
     * <p>
     * This mode can either be used as a stand alone input mode, or as a child mode of
     * {@link yfiles.input.GraphEditorInputMode}. In that case, some child modes of this implementation are disabled:
     * </p>
     * <ul>
     * <li>{@link yfiles.input.TableEditorInputMode#clickInputMode}</li>
     * <li>{@link yfiles.input.TableEditorInputMode#textEditorInputMode}</li>
     * <li>{@link yfiles.input.TableEditorInputMode#keyboardInputMode}</li>
     * </ul>
     * <p>
     * In addition, the {@link yfiles.input.TableEditorInputMode#stripeSelection stripe selection} is optionally synchronized with the {@link yfiles.input.GraphInputMode#graphSelection}.
     * </p>
     * @class
     * @extends {yfiles.input.MultiplexingInputMode}
     */
    export interface TableEditorInputMode extends yfiles.input.MultiplexingInputMode{}
    export class TableEditorInputMode {
      /**
       * Default constructor.
       */
      constructor();
      /**
       * Adds a new label to the given stripe interactively.
       * <p>
       * This will activate the {@link yfiles.input.TableEditorInputMode#textEditorInputMode} to let the user interactively enter the label's text and unless the user cancels label creation
       * the returned {@link yfiles.graphml.Future.<T>} will contain the label.
       * </p>
       * <p>
       * The text that the user enters may be {@link yfiles.input.TableEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
       * </p>
       * <p>
       * This method is similar to {@link yfiles.input.TableEditorInputMode#createLabel}. It differs only when either the {@link yfiles.input.TableEditorInputMode#addLabelAddingListener LabelAdding}
       * event is handled or the <code>owner</code> has an {@link yfiles.input.IEditLabelHelper} requests editing an existing label instead of
       * creating a new one. In this case, this method will edit that label instead of adding a new one.
       * </p>
       * @param owner The item.
       * @returns A future of the label that will be notified of the newly created label or a <code>null</code> if the creation was canceled.
       */
      addLabel(owner:yfiles.graph.ILabelOwner):Promise<yfiles.graph.ILabel>;
      /**
       * Clears the current selection.
       * @see yfiles.input.TableEditorInputMode#selectableItems
       */
      clearSelection():void;
      /**
       * Factory method that creates the {@link yfiles.input.TableEditorInputMode#keyboardInputMode} instance.
       * @returns 
       * @protected
       */
      createClickInputMode():yfiles.input.ClickInputMode;
      /**
       * Factory method that creates the {@link yfiles.input.TableEditorInputMode#keyboardInputMode} instance.
       * @returns 
       * @protected
       */
      createKeyboardInputMode():yfiles.input.KeyboardInputMode;
      /**
       * Interactively creates a new label for the provided stripe.
       * <p>
       * This method will invoke the text editor that will let the user edit the text of the label. If the user commits the label
       * text the label will be added to the label owner.
       * </p>
       * <p>
       * The text that the user enters may be {@link yfiles.input.TableEditorInputMode#addValidateLabelTextListener validated} before the label is actually added.
       * </p>
       * <p>
       * This method is similar to {@link yfiles.input.TableEditorInputMode#addLabel}. It differs only when either the {@link yfiles.input.TableEditorInputMode#addLabelAddingListener LabelAdding}
       * event is handled or the <code>stripe</code> has an {@link yfiles.input.IEditLabelHelper} requests editing an existing label instead of
       * creating a new one. In both cases, this method will do nothing and neither adds a new label nor edits an existing one.
       * </p>
       * @param stripe The item to create a new label for.
       * @returns A future of the label that will be notified of the newly created label or a <code>null</code> if the creation was canceled.
       */
      createLabel(stripe:yfiles.graph.IStripe):Promise<yfiles.graph.ILabel>;
      /**
       * Factory method for the {@link yfiles.input.TableEditorInputMode#reparentStripeHandler} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.TableEditorInputMode#reparentStripeHandler} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createReparentStripeHandler():yfiles.input.IReparentStripeHandler;
      /**
       * Factory method that creates the {@link yfiles.input.TableEditorInputMode#reparentStripeInputMode} lazily the first time the property is accessed.
       * @returns A plain new instance of the {@link } type, which is initially {@link #enabled enabled}.
       * @protected
       */
      createReparentStripeInputMode():yfiles.input.ReparentStripeInputMode;
      /**
       * Factory method that creates the {@link yfiles.input.TableEditorInputMode#resizeStripeInputMode} lazily the first time the property is accessed.
       * @returns A plain new instance of the {@link } type, which is initially {@link #enabled enabled}.
       * @protected
       */
      createResizeStripeInputMode():yfiles.input.ResizeStripeInputMode;
      /**
       * Factory method that creates the {@link yfiles.input.TableEditorInputMode#stripeDropInputMode} lazily the first time the property is accessed.
       * @returns A plain new instance of the {@link } type, which is initially {@link #enabled disabled}.
       * @protected
       */
      createStripeDropInputMode():yfiles.input.StripeDropInputMode;
      /**
       * Create a new instance of {@link yfiles.view.IStripeSelection} the first time property {@link yfiles.input.TableEditorInputMode#stripeSelection} is accessed.
       * <p>
       * By default, the {@link yfiles.view.IStripeSelection#allowMixedSelection} property is <code>false</code>.
       * </p>
       * @returns A new {@link } instance.
       * @protected
       */
      createStripeSelection():yfiles.view.IStripeSelection;
      /**
       * Factory method for the {@link yfiles.input.TableEditorInputMode#textEditorInputMode} property.
       * <p>
       * This method will be called upon first access to the {@link yfiles.input.TableEditorInputMode#textEditorInputMode} property.
       * </p>
       * @returns a new instance of {@link }
       * @protected
       */
      createTextEditorInputMode():yfiles.input.TextEditorInputMode;
      /**
       * This method deletes the currently selected stripes.
       * <p>
       * If the {@link yfiles.input.TableEditorInputMode#stripeSelection} is non-empty this implementation triggers the {@link yfiles.input.TableEditorInputMode#addDeletingSelectionListener DeletingSelection}
       * event, possibly a number of {@link yfiles.input.TableEditorInputMode#addDeletedItemListener DeletedItem}
       * events, and a final {@link yfiles.input.TableEditorInputMode#deleteSelection} event.
       * </p>
       * @see yfiles.input.TableEditorInputMode#addDeletingSelectionListener
       * @see yfiles.input.TableEditorInputMode#addDeletedItemListener
       * @see yfiles.input.TableEditorInputMode#addDeletedSelectionListener
       */
      deleteSelection():void;
      /**
       * Deletes a single stripe
       * @param stripe The stripe to delete.
       */
      deleteStripe(stripe:yfiles.graph.IStripe):void;
      /**
       * Starts editing of the given label.
       * <p>
       * The label must be bound to an {@link yfiles.graph.IStripe} instance for this call to succeed.
       * </p>
       * <p>
       * This implementation uses the {@link yfiles.input.TableEditorInputMode#textEditorInputMode} to display an editor to edit the label. The text that the user enters may be {@link yfiles.input.TableEditorInputMode#addValidateLabelTextListener validated}
       * before the label is actually edited.
       * </p>
       * <p>
       * This method will raise the {@link yfiles.input.TableEditorInputMode#addLabelEditingListener LabelEditing}
       * event and query the {@link yfiles.input.IEditLabelHelper} for the label and its owner, but will ignore the result except for the
       * {@link yfiles.input.LabelEditingEventArgs#textEditorInputModeConfigurator} property on the {@link yfiles.input.LabelEditingEventArgs}. Thus editing a label via this method cannot be prevented by event
       * handlers or {@link yfiles.input.IEditLabelHelper}s.
       * </p>
       * @param label The label to edit.
       * @returns 
       * @see yfiles.input.TableEditorInputMode#onLabelTextEdited
       */
      editLabel(label:yfiles.graph.ILabel):Promise<yfiles.graph.ILabel>;
      /**
       * Used as a callback to find the items underneath a certain point.
       * @param location The location to test.
       * @param stripeTypes An enumeration value of {@link } to specify the stripe type.
       * @param subregionTypes An enumeration value of {@link } to further restrict the stripe region.
       * @param [filter=null] Additional predicate to further restrict the hit test results.
       * @returns The stripe subregions that have been found for the location or <code>null</code>.
       */
      findStripe(location:yfiles.geometry.Point,stripeTypes:yfiles.graph.StripeTypes,subregionTypes:yfiles.input.StripeSubregionTypes,filter?:(obj:yfiles.input.StripeSubregion)=>boolean):yfiles.input.StripeSubregion;
      /**
       * Used as a callback to find the items underneath a certain point.
       * @param location The location to test.
       * @param stripeTypes An enumeration value of {@link } to specify the stripe type.
       * @param subregionTypes An enumeration value of {@link } to further restrict the stripe region.
       * @param [filter=null] Additional predicate to further restrict the hit test results.
       * @returns The stripe subregions that have been found for the location.
       */
      findStripes(location:yfiles.geometry.Point,stripeTypes:yfiles.graph.StripeTypes,subregionTypes:yfiles.input.StripeSubregionTypes,filter?:(obj:yfiles.input.StripeSubregion)=>boolean):yfiles.collections.IEnumerable<yfiles.input.StripeSubregion>;
      /**
       * Convenience method that inserts a new stripe as child of <code>owner</code>
       * <p>
       * This method automatically resizes the table node and moves all content in neighboring stripes accordingly. The stripe is
       * created with an initial <code>size</code> and otherwise the default settings of the table where <code>owner</code> belongs to. If <code>size</code> is entered
       * as <code>0</code>, the owners default size is used.
       * </p>
       * @param owner The parent of the new stripe.
       * @param index The index where to insert the new stripe.
       * @param [size=null] The initial size of the stripe
       * @returns A new stripe at the given index.
       */
      insertChild(owner:yfiles.graph.IStripe,index:number,size?:number):yfiles.graph.IStripe;
      /**
       * Callback that is invoked if the shift+F2 key is pressed and {@link yfiles.input.TableEditorInputMode#allowAddLabel} is set to <code>true</code>.
       * <p>
       * This method determines the label owner to add to and delegates to {@link yfiles.input.TableEditorInputMode#createLabel}.
       * </p>
       * @returns 
       * @protected
       */
      onAddLabel():boolean;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#clickInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onClickInputModeChanged(oldMode:yfiles.input.ClickInputMode,newMode:yfiles.input.ClickInputMode):void;
      /**
       * Callback that gets triggered once {@link yfiles.input.TableEditorInputMode#clickInputMode} triggers the {@link yfiles.input.ClickInputMode#addClickedListener Clicked}
       * event.
       * @param sender
       * @param evt
       * @protected
       */
      onClickInputModeClicked(sender:Object,evt:yfiles.input.ClickEventArgs):void;
      /**
       * Callback that gets triggered once {@link yfiles.input.TableEditorInputMode#clickInputMode} triggers the {@link yfiles.input.ClickInputMode#addDoubleClickedListener DoubleClicked}
       * event.
       * @param sender
       * @param evt
       * @protected
       */
      onClickInputModeDoubleClicked(sender:Object,evt:yfiles.input.ClickEventArgs):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addDeletedItemListener DeletedItem} event.
       * @param evt The instance containing the event data.
       * @protected
       */
      onDeletedItem(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addDeletedSelectionListener DeletedSelection} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onDeletedSelection(evt:yfiles.input.SelectionEventArgs<yfiles.graph.IStripe>):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addDeletingSelectionListener DeletingSelection} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onDeletingSelection(evt:yfiles.input.SelectionEventArgs<yfiles.graph.IStripe>):void;
      /**
       * Callback that is invoked if the F2 key is pressed and {@link yfiles.input.TableEditorInputMode#allowEditLabel} is set to <code>true</code>.
       * <p>
       * This method determines the label to edit and delegates to either {@link yfiles.input.TableEditorInputMode#editLabel} or {@link yfiles.input.TableEditorInputMode#createLabel}
       * if no label could be found.
       * </p>
       * @returns 
       * @protected
       */
      onEditLabel():boolean;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#keyboardInputMode} changed.
       * @param oldMode The old {@link #keyboardInputMode}.
       * @param newMode The new {@link #keyboardInputMode}.
       * @protected
       */
      onKeyboardInputModeChanged(oldMode:yfiles.input.KeyboardInputMode,newMode:yfiles.input.KeyboardInputMode):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addLabelAddedListener LabelAdded} event.
       * @param evt The {@link } instance that contains the {@link } that has been added.
       * @protected
       */
      onLabelAdded(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addLabelAddingListener LabelAdding} event.
       * <p>
       * Invoking the event handlers stops once one event marks the <code>args</code> as handled.
       * </p>
       * @param evt The event arguments.
       * @protected
       */
      onLabelAdding(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addLabelEditingListener LabelEditing} event.
       * <p>
       * Invoking the event handlers stops once one event marks the <code>args</code> as handled.
       * </p>
       * @param evt The event arguments.
       * @protected
       */
      onLabelEditing(evt:yfiles.input.LabelEditingEventArgs):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addLabelTextChangedListener LabelTextChanged} event.
       * @param evt The {@link } instance that contains the {@link } that has changed the text.
       * @protected
       */
      onLabelTextChanged(evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>):void;
      /**
       * Called when the text of a label has been {@link yfiles.input.TableEditorInputMode#editLabel edited}.
       * <p>
       * This method {@link yfiles.input.TableEditorInputMode#addValidateLabelTextListener validates the label text} and if successful sets the label text.
       * </p>
       * @param label The label that was edited.
       * @param text The new text.
       * @protected
       */
      onLabelTextEdited(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addLabelTextEditingCanceledListener LabelTextEditingCanceled}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onLabelTextEditingCanceled(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addLabelTextEditingStartedListener LabelTextEditingStarted}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onLabelTextEditingStarted(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#reparentStripeHandler} property value changes and after initialization of the field.
       * @param oldHandler the old value, which may be <code>null</code> the first time
       * @param newHandler the new value
       * @protected
       */
      onReparentStripeHandlerChanged(oldHandler:yfiles.input.IReparentStripeHandler,newHandler:yfiles.input.IReparentStripeHandler):void;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#reparentStripeInputMode} property value changes and after initialization of the field.
       * @param oldHandler the old value, which may be <code>null</code> the first time
       * @param newHandler the new value
       * @protected
       */
      onReparentStripeInputModeChanged(oldHandler:yfiles.input.ReparentStripeInputMode,newHandler:yfiles.input.ReparentStripeInputMode):void;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#resizeStripeInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onResizeStripeInputModeChanged(oldMode:yfiles.input.ResizeStripeInputMode,newMode:yfiles.input.ResizeStripeInputMode):void;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#stripeDropInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onStripeDropInputModeChanged(oldMode:yfiles.input.StripeDropInputMode,newMode:yfiles.input.StripeDropInputMode):void;
      /**
       * Called when the {@link yfiles.input.TableEditorInputMode#textEditorInputMode} property value changes and after initialization of the field.
       * @param oldMode the old value, which may be <code>null</code> the first time
       * @param newMode the new value
       * @protected
       */
      onTextEditorInputModeChanged(oldMode:yfiles.input.TextEditorInputMode,newMode:yfiles.input.TextEditorInputMode):void;
      /**
       * Raises the {@link yfiles.input.TableEditorInputMode#addValidateLabelTextListener ValidateLabelText} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onValidateLabelText(evt:yfiles.input.LabelTextValidatingEventArgs):void;
      /**
       * Clears the current selection and selects all stripes in all table instances.
       * <p>
       * By default, this method will be called in response to a Ctrl-A-key press recognized by {@link yfiles.input.TableEditorInputMode#keyboardInputMode}.
       * </p>
       * @see yfiles.input.ICommand#SELECT_ALL
       * @see yfiles.input.TableEditorInputMode#selectableItems
       */
      selectAll():void;
      /**
       * Predicate that determines whether a label may be added interactively to <code>item</code>
       * <p>
       * This is overridden if the item has a {@link yfiles.input.IEditLabelHelper} that returns <code>false</code> for its {@link yfiles.input.IEditLabelHelper#onLabelAdding}
       * method.
       * </p>
       * @param item The item to query
       * @returns <code>true</code> iff a label may be added
       * @protected
       */
      shouldAddLabel(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback method that determines whether the given item should be deleted during
       * {@link yfiles.input.TableEditorInputMode#deleteSelection}.
       * <p>
       * This implementation deletes an item if it is of one of the {@link yfiles.input.TableEditorInputMode#deletableItems} type and if it is not the last stripe of a given type in the
       * table (e.g. the only row or the only column).
       * </p>
       * @param stripe The item.
       * @returns Whether to delete that item.
       * @protected
       */
      shouldDelete(stripe:yfiles.graph.IStripe):boolean;
      /**
       * Callback method that determines whether the label or the labels of the provided item should be edited in response to a
       * command or explicit method call to {@link yfiles.input.TableEditorInputMode#editLabel} or
       * {@link yfiles.input.TableEditorInputMode#createLabel}.
       * @param item The item.
       * @returns Whether to edit the label or the labels for that item.
       * @protected
       */
      shouldEditLabel(item:yfiles.graph.IModelItem):boolean;
      /**
       * Callback method that is used by {@link yfiles.input.TableEditorInputMode#onKeyboardInputModeChanged} to determine which of the
       * built-in {@link yfiles.input.ICommand}s to install.
       * <p>
       * This implementation unconditionally returns <code>true</code>, subclasses may override this method to adjust the behavior.
       * </p>
       * @param command The command to install.
       * @returns Whether to install this command.
       * @protected
       */
      shouldInstallCommand(command:yfiles.input.ICommand):boolean;
      /**
       * Gets or sets the {@link yfiles.input.ResizeStripeInputMode} that is used to change the size of a stripe interactively.
       * <p>
       * The mode by default is {@link yfiles.input.ResizeStripeInputMode#enabled enabled} with a priority of <code>0</code>. If the field has not yet been initialized upon first access, the
       * factory method {@link yfiles.input.TableEditorInputMode#createResizeStripeInputMode} will be called. Upon change the {@link yfiles.input.TableEditorInputMode#onResizeStripeInputModeChanged}
       * method will be called.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.ResizeStripeInputMode}
       */
      resizeStripeInputMode:yfiles.input.ResizeStripeInputMode;
      /**
       * Gets or sets the {@link yfiles.input.TableEditorInputMode#reparentStripeHandler} property.
       * <p>
       * This handler will be delegated to by an implementation of the {@link yfiles.input.IReparentStripeHandler} interface that this
       * class puts into the {@link yfiles.input.TableEditorInputMode#childInputModeContextLookup}. If the field has not yet been
       * initialized upon first access, the factory method {@link yfiles.input.TableEditorInputMode#createReparentStripeHandler} will be
       * called. Upon change the {@link yfiles.input.TableEditorInputMode#onReparentStripeHandlerChanged} method will be called.
       * </p>
       * @see yfiles.input.ReparentStripeHandler
       * @type {yfiles.input.IReparentStripeHandler}
       */
      reparentStripeHandler:yfiles.input.IReparentStripeHandler;
      /**
       * Gets or sets the {@link yfiles.input.ReparentStripeInputMode} that is used to change a stripes position in the table hierarchy.
       * <p>
       * The mode by default is {@link yfiles.input.MoveInputMode#enabled enabled} with a priority of <code>0</code>. If the field has not yet been initialized upon first access, the
       * factory method {@link yfiles.input.TableEditorInputMode#createReparentStripeInputMode} will be called. Upon change the {@link yfiles.input.TableEditorInputMode#onReparentStripeInputModeChanged}
       * method will be called.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.ReparentStripeInputMode}
       */
      reparentStripeInputMode:yfiles.input.ReparentStripeInputMode;
      /**
       * Gets or sets the {@link yfiles.input.TableEditorInputMode#stripeDropInputMode} property that is responsible for handling drag and drop operations of stripes onto the Canvas.
       * <p>
       * The mode by default is {@link yfiles.input.DropInputMode#enabled disabled} and needs to be enabled to work, first. If the field has not yet been initialized upon
       * first access, the factory method {@link yfiles.input.TableEditorInputMode#createStripeDropInputMode} will be called. Upon change
       * the {@link yfiles.input.TableEditorInputMode#onStripeDropInputModeChanged} method will be called.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>70</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.StripeDropInputMode}
       */
      stripeDropInputMode:yfiles.input.StripeDropInputMode;
      /**
       * Gets or sets the {@link yfiles.input.TableEditorInputMode#keyboardInputMode}.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.TableEditorInputMode#createKeyboardInputMode}
       * will be called. Upon change the {@link yfiles.input.TableEditorInputMode#onKeyboardInputModeChanged} method will be called. If
       * the parent {@link yfiles.input.TableEditorInputMode} instance is installed as child mode of a {@link yfiles.input.GraphEditorInputMode}
       * instance, this mode is <b>not</b> used, since all work is delegated to the parent's {@link yfiles.input.GraphInputMode#keyboardInputMode} instance. This implies that all modifications
       * made to the property value have no effect as long as the parent mode is child of a {@link yfiles.input.GraphEditorInputMode}
       * instance.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.KeyboardInputMode}
       */
      keyboardInputMode:yfiles.input.KeyboardInputMode;
      /**
       * Gets or sets the {@link yfiles.input.TableEditorInputMode#textEditorInputMode}.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.TableEditorInputMode#createTextEditorInputMode}
       * will be called. Upon change the {@link yfiles.input.TableEditorInputMode#onTextEditorInputModeChanged} method will be called. If
       * the parent {@link yfiles.input.TableEditorInputMode} instance is installed as child mode of a {@link yfiles.input.GraphEditorInputMode}
       * instance, this mode is <b>not</b> used, since all work is delegated to the parent's {@link yfiles.input.GraphEditorInputMode#textEditorInputMode} instance. This implies that all modifications
       * made to the property value have no effect as long as the parent mode is child of a {@link yfiles.input.GraphEditorInputMode}
       * instance.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>100</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.TextEditorInputMode}
       */
      textEditorInputMode:yfiles.input.TextEditorInputMode;
      /**
       * Gets or set a property that determines which types of items may have their {@link yfiles.input.TableEditorInputMode#editLabel labels edited}.
       * <p>
       * The default is {@link yfiles.graph.StripeTypes#ALL} which allows for editing the labels of all {@link yfiles.graph.ILabelOwner}s, and
       * existing {@link yfiles.graph.ILabel}s.
       * </p>
       * @see yfiles.input.TableEditorInputMode#shouldEditLabel
       * @type {yfiles.graph.StripeTypes}
       */
      labelEditableItems:yfiles.graph.StripeTypes;
      /**
       * Adds the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
       * <p>
       * Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
       * forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
       * {@link yfiles.input.LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#removeLabelAddingListener
       */
      addLabelAddingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelAdding</code> event that occurs when a label is about to be added.
       * <p>
       * Event handlers for this event can customize the behavior for adding a new label extensively. Adding a label can be
       * forbidden entirely, or the properties of the added label can be changed, by setting the appropriate properties on the
       * {@link yfiles.input.LabelEditingEventArgs}. It is even possible to forbid adding a label, but edit an existing one instead.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#addLabelAddingListener
       */
      removeLabelAddingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
       * <p>
       * Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
       * entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
       * event handlers can also specify that instead of editing a label, a new one should be added.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#removeLabelEditingListener
       */
      addLabelEditingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelEditing</code> event that occurs when a label is about to be edited.
       * <p>
       * Event handlers for this event can customize the behavior for editing an existing label. Editing can be forbidden
       * entirely, or redirected to a different label, even on a different owner. In case editing existing labels is unwanted,
       * event handlers can also specify that instead of editing a label, a new one should be added.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#addLabelEditingListener
       */
      removeLabelEditingListener(listener:(sender:Object,evt:yfiles.input.LabelEditingEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.TableEditorInputMode#createLabel}.
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#removeLabelAddedListener
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelAdded</code> event that occurs when this mode has triggered the addition of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.TableEditorInputMode#createLabel}.
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#addLabelAddedListener
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.TableEditorInputMode#editLabel}.
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#removeLabelTextChangedListener
       */
      addLabelTextChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextChanged</code> event that occurs when this mode has triggered the edit of an
       * {@link yfiles.graph.ILabel}, for instance, in response to {@link yfiles.input.TableEditorInputMode#editLabel}.
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#addLabelTextChangedListener
       */
      removeLabelTextChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is about
       * to start.
       * <p>
       * This allows to customize the actual label editing process further.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#onValidateLabelText
       * @see yfiles.input.TableEditorInputMode#removeLabelTextEditingStartedListener
       */
      addLabelTextEditingStartedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextEditingStarted</code> event that occurs when the actual label editing process is
       * about to start.
       * <p>
       * This allows to customize the actual label editing process further.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#onValidateLabelText
       * @see yfiles.input.TableEditorInputMode#addLabelTextEditingStartedListener
       */
      removeLabelTextEditingStartedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
       * canceled.
       * <p>
       * This allows to unconfigure temporary customizations for the actual label editing process.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#onValidateLabelText
       * @see yfiles.input.TableEditorInputMode#removeLabelTextEditingCanceledListener
       */
      addLabelTextEditingCanceledListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextEditingCanceled</code> event that occurs when the actual label editing process is
       * canceled.
       * <p>
       * This allows to unconfigure temporary customizations for the actual label editing process.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#onValidateLabelText
       * @see yfiles.input.TableEditorInputMode#addLabelTextEditingCanceledListener
       */
      removeLabelTextEditingCanceledListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>ValidateLabelText</code> event that occurs after label editing to validate the label text for a
       * label that is about to be added or edited.
       * <p>
       * Note that the {@link yfiles.input.LabelTextValidatingEventArgs#label} may only a dummy instance, i.e. not bound to a live stripe instance.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#onValidateLabelText
       * @see yfiles.input.TableEditorInputMode#removeValidateLabelTextListener
       */
      addValidateLabelTextListener(listener:(sender:Object,evt:yfiles.input.LabelTextValidatingEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ValidateLabelText</code> event that occurs after label editing to validate the label text
       * for a label that is about to be added or edited.
       * <p>
       * Note that the {@link yfiles.input.LabelTextValidatingEventArgs#label} may only a dummy instance, i.e. not bound to a live stripe instance.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#onValidateLabelText
       * @see yfiles.input.TableEditorInputMode#addValidateLabelTextListener
       */
      removeValidateLabelTextListener(listener:(sender:Object,evt:yfiles.input.LabelTextValidatingEventArgs)=>void):void;
      /**
       * Gets or sets a value indicating whether this mode should automatically {@link yfiles.graph.ITable#remove remove labels} from the table when a label text
       * has been edited and the label text is empty.
       * <p>
       * If the {@link yfiles.input.TableEditorInputMode} instance is installed as child mode of a {@link yfiles.input.GraphEditorInputMode}
       * instance, auto removal is only enabled allowed if both the value of this property and the value of {@link yfiles.input.GraphEditorInputMode#autoRemoveEmptyLabels} are <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoRemoveEmptyLabels:boolean;
      /**
       * Gets or sets the {@link yfiles.input.TableEditorInputMode#clickInputMode}.
       * <p>
       * If the field has not yet been initialized upon first access, the factory method {@link yfiles.input.TableEditorInputMode#createClickInputMode}
       * will be called. Upon change the {@link yfiles.input.TableEditorInputMode#onClickInputModeChanged} method will be called. If the
       * parent {@link yfiles.input.TableEditorInputMode} instance is installed as child mode of a {@link yfiles.input.GraphEditorInputMode}
       * instance, this mode is <b>not</b> used, since all work is delegated to the parent's {@link yfiles.input.GraphInputMode#clickInputMode} instance. This implies that all modifications
       * made to the property value have no effect as long as the parent mode is child of a {@link yfiles.input.GraphEditorInputMode}
       * instance.
       * </p>
       * <p>
       * By default this input mode has a priority of <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.NotSupportedError} If an attempt is made to change the instance while this mode is {@link yfiles.input.IInputMode#install installed}. To exchange a mode, first
       * {@link yfiles.input.IInputMode#uninstall}, then {@link yfiles.input.IInputMode#install reinstall} to ensure that all data is initialized correctly.
       * @type {yfiles.input.ClickInputMode}
       */
      clickInputMode:yfiles.input.ClickInputMode;
      /**
       * Adds the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link yfiles.input.TableEditorInputMode#deleteSelection}
       * method starts its work and will be followed by any number of {@link yfiles.input.TableEditorInputMode#addDeletedItemListener DeletedItem}
       * events and finalized by a {@link yfiles.input.TableEditorInputMode#addDeletedSelectionListener DeletedSelection}
       * event.
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#deleteSelection
       * @see yfiles.input.TableEditorInputMode#removeDeletingSelectionListener
       */
      addDeletingSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Removes the given listener for the <code>DeletingSelection</code> event that occurs just before the {@link yfiles.input.TableEditorInputMode#deleteSelection}
       * method starts its work and will be followed by any number of {@link yfiles.input.TableEditorInputMode#addDeletedItemListener DeletedItem}
       * events and finalized by a {@link yfiles.input.TableEditorInputMode#addDeletedSelectionListener DeletedSelection}
       * event.
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#deleteSelection
       * @see yfiles.input.TableEditorInputMode#addDeletingSelectionListener
       */
      removeDeletingSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Adds the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this mode.
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#removeDeletedItemListener
       */
      addDeletedItemListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Removes the given listener for the <code>DeletedItem</code> event that occurs when an item has been deleted interactively by this
       * mode.
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#addDeletedItemListener
       */
      removeDeletedItemListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IModelItem>)=>void):void;
      /**
       * Adds the given listener for the <code>DeletedSelection</code> event that occurs just before the {@link yfiles.input.TableEditorInputMode#deleteSelection}
       * method has deleted the selection after all selected items have been removed.
       * @param listener The listener to add.
       * @see yfiles.input.TableEditorInputMode#deleteSelection
       * @see yfiles.input.TableEditorInputMode#removeDeletedSelectionListener
       */
      addDeletedSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Removes the given listener for the <code>DeletedSelection</code> event that occurs just before the {@link yfiles.input.TableEditorInputMode#deleteSelection}
       * method has deleted the selection after all selected items have been removed.
       * @param listener The listener to remove.
       * @see yfiles.input.TableEditorInputMode#deleteSelection
       * @see yfiles.input.TableEditorInputMode#addDeletedSelectionListener
       */
      removeDeletedSelectionListener(listener:(sender:Object,evt:yfiles.input.SelectionEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Gets or set a property that determines which types of items may be deleted using the {@link yfiles.input.TableEditorInputMode#deleteSelection}
       * action.
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @see yfiles.input.TableEditorInputMode#shouldDelete
       * @type {yfiles.graph.StripeTypes}
       */
      deletableItems:yfiles.graph.StripeTypes;
      /**
       * Gets or set a property that determines which types of items should be selectable at all.
       * <p>
       * The default is {@link yfiles.graph.StripeTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphInputMode#setSelected
       * @type {yfiles.graph.StripeTypes}
       */
      selectableItems:yfiles.graph.StripeTypes;
      /**
       * Gets or set a property that determines which types of items should be selectable through {@link yfiles.input.TableEditorInputMode#onClickInputModeClicked mouse clicks.}.
       * <p>
       * The default is {@link yfiles.graph.StripeTypes#ALL}.
       * </p>
       * @see yfiles.input.GraphInputMode#setSelected
       * @type {yfiles.graph.StripeTypes}
       */
      clickSelectableItems:yfiles.graph.StripeTypes;
      /**
       * Gets or sets whether the label editor will be automatically invoked if the F2 key is pressed.
       * <p>
       * If the {@link yfiles.input.TableEditorInputMode} instance is installed as child mode of a {@link yfiles.input.GraphEditorInputMode}
       * instance, editing a label is only allowed if both the value of this property and the value of {@link yfiles.input.GraphEditorInputMode#allowEditLabel} are
       * <code>true</code>. If label editing is allowed, pressing F2 will start the label editor. By default this feature is enabled.
       * </p>
       * @see yfiles.input.TableEditorInputMode#editLabel
       * @default true
       * @type {boolean}
       */
      allowEditLabel:boolean;
      /**
       * Gets or sets whether the label editor will be automatically invoked if the Shift+F2 key is pressed.
       * <p>
       * If the {@link yfiles.input.TableEditorInputMode} instance is installed as child mode of a {@link yfiles.input.GraphEditorInputMode}
       * instance, adding a label is only allowed if both the value of this property and the value of {@link yfiles.input.GraphEditorInputMode#allowAddLabel} are <code>true</code>. If label adding
       * is allowed, <code>true</code> pressing Shift+F2 will start the label editor. By default this feature is enabled.
       * </p>
       * @see yfiles.input.TableEditorInputMode#createLabel
       * @default true
       * @type {boolean}
       */
      allowAddLabel:boolean;
      /**
       * Gets or sets the event recognizer that will be queried to decide if a click is deemed a multi selection gesture.
       * <p>
       * The default is a combination of {@link yfiles.input.KeyEventRecognizers#CONTROL_PRESSED} or
       * {@link yfiles.input.KeyEventRecognizers#META_PRESSED}. If the {@link yfiles.input.TableEditorInputMode} instance is installed as child
       * mode of a {@link yfiles.input.GraphEditorInputMode} instance, the value of {@link yfiles.input.GraphInputMode#multiSelectionRecognizer} is used instead.
       * </p>
       * @type {function(Object, yfiles.lang.EventArgs): boolean}
       */
      multiSelectionRecognizer:(eventSource:Object,evt:yfiles.lang.EventArgs)=>boolean;
      /**
       * Gets or sets clicks on which {@link yfiles.input.StripeSubregionTypes}(s) should toggle a stripe selection state.
       * <p>
       * This is independent to {@link yfiles.input.TableEditorInputMode#clickSelectableItems}. By default, only clicks on a stripe {@link yfiles.input.StripeSubregionTypes#HEADER} are handled.
       * </p>
       * @default yfiles.input.StripeSubregionTypes.HEADER
       * @type {yfiles.input.StripeSubregionTypes}
       */
      clickSelectableRegions:yfiles.input.StripeSubregionTypes;
      /**
       * Gets or sets a value indicating whether {@link yfiles.input.TableEditorInputMode#stripeSelection} should be synchronized with the {@link yfiles.input.GraphInputMode#graphSelection} if this instance is installed as a child of a
       * {@link yfiles.input.GraphEditorInputMode} instance.
       * <p>
       * This feature needs to be disabled temporarily if the {@link yfiles.view.GraphComponent#selection} is changed.
       * </p>
       * <p>
       * By default, this feature is enabled.
       * </p>
       * @default true
       * @type {boolean}
       */
      synchronizeWithGraphSelection:boolean;
      /**
       * Whether to allow mixed selections of graph elements and table elements
       * <p>
       * Default is <code>true</code>. Changing this property only has an effetc if {@link yfiles.input.TableEditorInputMode#synchronizeWithGraphSelection} is enabled.
       * </p>
       * @type {boolean}
       */
      allowMixedSelection:boolean;
      /**
       * Returns the {@link yfiles.view.IStripeSelection selected stripes}
       * @type {yfiles.view.IStripeSelection}
       */
      stripeSelection:yfiles.view.IStripeSelection;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.input.IReshapeHandler} implementation that constrains node resizes so that minimum size constraints of an
     * associated {@link yfiles.graph.ITable} are satisfied and which resizes the outer
     * {@link yfiles.graph.IStripe}s of such a table when the node is resized.
     * <p>
     * The constraints are applied in addition to any constraints imposed by the {@link yfiles.input.IReshapeHandler} which is wrapped
     * by this implementation.
     * </p>
     * @class
     * @implements {yfiles.input.IReshapeHandler}
     */
    export interface TableReshapeHandler extends Object,yfiles.input.IReshapeHandler{}
    export class TableReshapeHandler {
      /**
       * Creates a new instance of this Handler which wraps an existing instance of <code>coreHandler</code> and uses <code>table</code> for additional
       * constraints.
       * <p>
       * In addition, <code>table</code>'s outer stripes are resized if necessary.
       * </p>
       * <p>
       * <code>table</code> should be associated to <code>tableNode</code>
       * </p>
       * @param coreHandler The handler that is wrapped by this instance.
       * @param tableNode The node for which this handler is used
       * @param table The table that is used for additional constraints.
       */
      constructor(coreHandler:yfiles.input.IReshapeHandler,tableNode:yfiles.graph.INode,table:yfiles.graph.ITable);
      /**
       * 
       * @param context
       * @param originalBounds
       */
      cancelReshape(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect):void;
      /**
       * 
       * @param context
       * @param originalBounds
       * @param newBounds
       */
      handleReshape(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * 
       * @param context
       */
      initializeReshape(context:yfiles.input.IInputModeContext):void;
      /**
       * 
       * @param context
       * @param originalBounds
       * @param newBounds
       */
      reshapeFinished(context:yfiles.input.IInputModeContext,originalBounds:yfiles.geometry.Rect,newBounds:yfiles.geometry.Rect):void;
      /**
       * The core reshape handler wrapped by this instance.
       * @type {yfiles.input.IReshapeHandler}
       */
      coreHandler:yfiles.input.IReshapeHandler;
      /**
       * The node this handler is used for.
       * @type {yfiles.graph.INode}
       */
      tableNode:yfiles.graph.INode;
      /**
       * The table used for additional constraints.
       * @type {yfiles.graph.ITable}
       */
      table:yfiles.graph.ITable;
      /**
       * 
       * @type {yfiles.geometry.IRectangle}
       */
      bounds:yfiles.geometry.IRectangle;
      static $class:yfiles.lang.Class;
    }
    export enum StripeReparentPolicy{
      /**
       * Add the moved stripe as a new child of the target stripe
       */
      ADD_CHILD,
      /**
       * Insert the moved stripe before the target stripe
       */
      INSERT_BEFORE,
      /**
       * Insert the moved stripe after the target stripe
       */
      INSERT_AFTER,
      /**
       * The current gesture would result in an invalid operation (such as trying to make a stripe to its own child).
       */
      INVALID
    }
    /**
     * Helper interface that allows to bundle {@link yfiles.input.IHitTestable} implementations for the main regions of a {@link yfiles.graph.IStripe}
     * instance.
     * <p>
     * The default {@link yfiles.graph.IRow} and {@link yfiles.graph.IColumn} implementations provide default implementations of this
     * interface.
     * </p>
     * @interface
     */
    export interface IStripeHitTestHelper extends Object{
      /**
       * Return a {@link yfiles.input.IHitTestable} that should report hits on the far border region of the stripe.
       * <p>
       * The far border is the region that is used for the resize gesture in {@link yfiles.input.ResizeStripeInputMode} to expand the
       * stripe to the bottom or the right. This is typically the bottom border for an {@link yfiles.graph.IRow} instance and the right
       * border for an {@link yfiles.graph.IColumn}.
       * </p>
       * @returns a {@link } that should report hits on the far border region of the stripe.
       * @see yfiles.input.StripeSubregion
       * @see yfiles.input.StripeSubregionTypes#FAR_BORDER
       * @abstract
       */
      getFarBorderHitTestable():yfiles.input.IHitTestable;
      /**
       * Return a {@link yfiles.input.IHitTestable} that should report hits on the leading header region of the stripe.
       * <p>
       * The leading header is the region that is used by default for click selection and as start location for a stripe drag.
       * Typically, this is the area occupied by the left insets for an {@link yfiles.graph.IRow} instance and by the top insets for an {@link yfiles.graph.IColumn}
       * instance.
       * </p>
       * @returns a {@link } that should report hits on the header region of the stripe.
       * @see yfiles.input.StripeSubregion
       * @see yfiles.input.StripeSubregionTypes#HEADER
       * @abstract
       */
      getLeadingHeaderHitTestable():yfiles.input.IHitTestable;
      /**
       * Return a {@link yfiles.input.IHitTestable} that should report hits on the near border region of the stripe.
       * <p>
       * The near border is the region that is used for the resize gesture in {@link yfiles.input.ResizeStripeInputMode} to expand the
       * stripe to the top or the left. This is typically the upper border for an {@link yfiles.graph.IRow} instance and the left border
       * for an {@link yfiles.graph.IColumn}.
       * </p>
       * @returns a {@link } that should report hits on the near border region of the stripe.
       * @see yfiles.input.StripeSubregion
       * @see yfiles.input.StripeSubregionTypes#NEAR_BORDER
       * @abstract
       */
      getNearBorderHitTestable():yfiles.input.IHitTestable;
      /**
       * Return a {@link yfiles.input.IHitTestable} that should report hits on the whole stripe.
       * @returns a {@link } that should report hits on the whole stripe.
       * @see yfiles.input.StripeSubregion
       * @see yfiles.input.StripeSubregionTypes#STRIPE
       * @abstract
       */
      getStripeHitTestable():yfiles.input.IHitTestable;
      /**
       * Return a {@link yfiles.input.IHitTestable} that should report hits on the trailing header region of the stripe.
       * <p>
       * The trailing header is the region that is used by default for click selection and as start location for a stripe drag.
       * Typically, this is the area occupied by the right insets for an {@link yfiles.graph.IRow} instance and by the bottom insets for
       * an {@link yfiles.graph.IColumn} instance.
       * </p>
       * @returns a {@link } that should report hits on the header region of the stripe.
       * @see yfiles.input.StripeSubregion
       * @see yfiles.input.StripeSubregionTypes#HEADER
       * @abstract
       */
      getTrailingHeaderHitTestable():yfiles.input.IHitTestable;
    }
    var IStripeHitTestHelper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum StripeSubregionTypes{
      /**
       * No active hotspot region.
       */
      NONE,
      /**
       * The hot spot region includes the stripe itself.
       * @see yfiles.input.IStripeHitTestHelper#getStripeHitTestable
       */
      STRIPE,
      /**
       * The hot spot region includes the leading stripe header.
       * <p>
       * The leading header is the region that is used by default for click selection and as start location for a stripe drag.
       * Typically, this is the area occupied by the left insets for an {@link yfiles.graph.IRow} instance and by the top insets for an {@link yfiles.graph.IColumn}
       * instance.
       * </p>
       * @see yfiles.input.IStripeHitTestHelper#getLeadingHeaderHitTestable
       */
      LEADING_HEADER,
      /**
       * The hot spot region includes the trailing stripe header.
       * <p>
       * The trailing header is the region that is used by default for click selection and as start location for a stripe drag.
       * Typically, this is the area occupied by the right insets for an {@link yfiles.graph.IRow} instance and by the bottom insets for
       * an {@link yfiles.graph.IColumn} instance.
       * </p>
       * @see yfiles.input.IStripeHitTestHelper#getTrailingHeaderHitTestable
       */
      TRAILING_HEADER,
      /**
       * The hot spot region includes both stripe headers.
       * <p>
       * This is a bitwise combination of {@link yfiles.input.StripeSubregionTypes#LEADING_HEADER} and
       * {@link yfiles.input.StripeSubregionTypes#TRAILING_HEADER}
       * </p>
       * @see yfiles.input.IStripeHitTestHelper#getTrailingHeaderHitTestable
       */
      HEADER,
      /**
       * The hot spot region includes the near-border region of the stripe.
       * <p>
       * The near border is the region that is used for the resize gesture in {@link yfiles.input.ResizeStripeInputMode} to expand the
       * stripe to the top or the left. This is typically the upper border for an {@link yfiles.graph.IRow} instance and the left border
       * for an {@link yfiles.graph.IColumn}.
       * </p>
       * @see yfiles.input.IStripeHitTestHelper#getNearBorderHitTestable
       */
      NEAR_BORDER,
      /**
       * The hot spot region includes the stripe far-border region.
       * <p>
       * The far border is the region that is used for the resize gesture in {@link yfiles.input.ResizeStripeInputMode} to expand the
       * stripe to the bottom or the right. This is typically the bottom border for an {@link yfiles.graph.IRow} instance and the right
       * border for an {@link yfiles.graph.IColumn}.
       * </p>
       * @see yfiles.input.IStripeHitTestHelper#getFarBorderHitTestable
       */
      FAR_BORDER,
      /**
       * Flag for any stripe region.
       */
      ALL
    }
    /**
     * Helper class that further describes the result of a hit test on a stripe or table.
     * <p>
     * Instances of this class are returned by the default {@link yfiles.input.StripeHitTester} implementation.
     * </p>
     * @class
     * @final
     */
    export interface StripeSubregion extends Object{}
    export class StripeSubregion {
      /**
       * Create a new instance that describes the specific subregion.
       * @param subregionType The specific subregion
       * @param stripe The stripe
       */
      constructor(subregionType:yfiles.input.StripeSubregionTypes,stripe:yfiles.graph.IStripe);
      /**
       * Gets the specific subregion type that this instance describes.
       * @type {yfiles.input.StripeSubregionTypes}
       */
      subregionType:yfiles.input.StripeSubregionTypes;
      /**
       * Gets the stripe that this instance describes.
       * @type {yfiles.graph.IStripe}
       */
      stripe:yfiles.graph.IStripe;
      /**
       * Gets or sets the node the region is associated to.
       * @type {yfiles.graph.INode}
       */
      targetTableNode:yfiles.graph.INode;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default implementation of {@link yfiles.input.IStripeInputVisualizationHelper} that uses
     * {@link yfiles.view.IVisualTemplate}s for the actual visualization.
     * <p>
     * If you want to change the global behavior, you can just provide a different {@link yfiles.view.IVisualTemplate} for the various
     * predefined template keys in the application resources. If you want to override the visualization for a specific stripe
     * only, assign a different {@link yfiles.view.IVisualTemplate} to the instance that is already present in the stripe's lookup.
     * </p>
     * @class
     * @implements {yfiles.input.IStripeInputVisualizationHelper}
     */
    export interface DefaultStripeInputVisualizationHelper extends Object,yfiles.input.IStripeInputVisualizationHelper{}
    export class DefaultStripeInputVisualizationHelper {
      /**
       * Default constructor
       * @param stripe The stripe for which the visualizations are intended
       */
      constructor(stripe:yfiles.graph.IStripe);
      /**
       * Provide a {@link yfiles.view.IVisualTemplate} that is used to visualize <code>type</code>.
       * @param context The input mode context. Allows you to access the {@link } to retrieve the resources.
       * @param type The visualization type
       * @returns A {@link } that is used to visualize <code>type</code>.
       * @protected
       */
      findTemplate(context:yfiles.input.IInputModeContext,type:yfiles.input.StripeVisualizationType):yfiles.view.IVisualTemplate;
      /**
       * Return an {@link yfiles.view.IVisualCreator} that is used to visualize the input operation specified by <code>type</code>.
       * <p>
       * This implementation uses {@link yfiles.view.IVisualTemplate}s for the actual visualization.
       * </p>
       * @param context The input mode context which provides further information.
       * @param tableNode The node where the stripe's owner is currently bound to.
       * @param type The actual visualization type
       * @returns A {@link } instance that renders a representation for the current operation.
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#resizeStripeTemplate
       */
      getVisualCreator(context:yfiles.input.IInputModeContext,tableNode:yfiles.graph.INode,type:yfiles.input.StripeVisualizationType):yfiles.view.IVisualCreator;
      /**
       * A {@link string} that can be used to store a {@link yfiles.view.IVisualTemplate} that can be used to create the visual that will be used to
       * draw the highlight for the stripe that is resized.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#RESIZE}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @const
       * @static
       * @type {string}
       */
      static RESIZE_STRIPE_TEMPLATE_KEY:string;
      /**
       * {@link yfiles.view.IVisualTemplate} that is used to visualize the current stripe resize operation.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#RESIZE}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @type {yfiles.view.IVisualTemplate}
       */
      resizeStripeTemplate:yfiles.view.IVisualTemplate;
      /**
       * A {@link string} that can be used to store a {@link yfiles.view.IVisualTemplate} that can be used to create the visual that will be used to
       * draw the highlight for the stripe that is selected.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#SELECTION}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @const
       * @static
       * @type {string}
       */
      static SELECTED_STRIPE_TEMPLATE_KEY:string;
      /**
       * {@link yfiles.view.IVisualTemplate} that is used to visualize the current stripe resize operation.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#SELECTION}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @type {yfiles.view.IVisualTemplate}
       */
      selectedStripeTemplate:yfiles.view.IVisualTemplate;
      /**
       * A {@link string} that can be used to store a {@link yfiles.view.IVisualTemplate} that can be used to create the visual that will be used to
       * draw the highlight for the current source of a stripe reparent operation.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#DRAG_SOURCE}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @const
       * @static
       * @type {string}
       */
      static DRAG_SOURCE_STRIPE_TEMPLATE_KEY:string;
      /**
       * {@link yfiles.view.IVisualTemplate} that is used to visualize the current source of a stripe reparent operation.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#DRAG_SOURCE}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @type {yfiles.view.IVisualTemplate}
       */
      dragSourceStripeTemplate:yfiles.view.IVisualTemplate;
      /**
       * A {@link string} that can be used to store a {@link yfiles.view.IVisualTemplate} that can be used to create the visual that will be used to
       * draw the highlight for the current target of a stripe reparent operation.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#DROP_TARGET}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @const
       * @static
       * @type {string}
       */
      static DROP_TARGET_STRIPE_TEMPLATE_KEY:string;
      /**
       * {@link yfiles.view.IVisualTemplate} that is used to visualize the current target of a stripe reparent operation.
       * <p>
       * This template is used for {@link yfiles.input.StripeVisualizationType#DROP_TARGET}
       * </p>
       * @see yfiles.input.DefaultStripeInputVisualizationHelper#getVisualCreator
       * @type {yfiles.view.IVisualTemplate}
       */
      dropTargetStripeTemplate:yfiles.view.IVisualTemplate;
      static $class:yfiles.lang.Class;
    }
    export enum StripeVisualizationType{
      /**
       * Visualization for a selected stripe.
       */
      SELECTION,
      /**
       * Visualization for the source stripe of a reparent operation.
       * @see yfiles.input.ReparentStripeInputMode
       */
      DRAG_SOURCE,
      /**
       * Visualization for the target stripe of a reparent or drag and drop operation.
       * @see yfiles.input.ReparentStripeInputMode
       * @see yfiles.input.StripeDropInputMode
       */
      DROP_TARGET,
      /**
       * Visualization for resize operations that are triggered for example by {@link yfiles.input.ResizeStripeInputMode}
       */
      RESIZE
    }
    /**
     * This interface allows to change the visualization for different input gestures.
     * <p>
     * A stripe should provide an implementation of this interface in its lookup which is queried by the various table related
     * input modes.
     * </p>
     * @interface
     */
    export interface IStripeInputVisualizationHelper extends Object{
      /**
       * Return an {@link yfiles.view.IVisualCreator} that is used to visualize the input operation specified by <code>type</code>.
       * @param context The input mode context which provides further information.
       * @param tableNode The node where the stripe's owner is currently bound to.
       * @param type The actual visualization type
       * @returns A {@link } instance that renders a representation for the current operation.
       * @abstract
       */
      getVisualCreator(context:yfiles.input.IInputModeContext,tableNode:yfiles.graph.INode,type:yfiles.input.StripeVisualizationType):yfiles.view.IVisualCreator;
    }
    var IStripeInputVisualizationHelper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Support class that allows to iterate over hits on table elements or subregions of table elements.
     * @class
     */
    export interface StripeHitTester extends Object{}
    export class StripeHitTester {
      /**
       * Create a new instance for a specific table instance that use the order specified by <code>tableRenderingOrder</code> to determine the
       * order of the hit test results.
       * @param table The table to use
       * @param tableRenderingOrder The hit test order. This determines the order in which stripes are returned by
       * {@link #enumerateHits}
       */
      constructor(table:yfiles.graph.ITable,tableRenderingOrder:yfiles.styles.TableRenderingOrder);
      /**
       * Return a collection of hits at <code>location</code>.
       * <p>
       * This implementation returns the hits in the following order:
       * </p>
       * <ol>
       * <li>First all {@link yfiles.graph.IRow}s or {@link yfiles.graph.IColumn}s at the given location, in the order specified by the {@link yfiles.input.StripeHitTester#tableRenderingOrder} property.</li>
       * <li>For each stripe type, the stripes are returned in bottom up order, i.e. children are returned before their parents.</li>
       * <li>
       * For each stripe, the subregions are returned in the order {@link yfiles.input.StripeSubregionTypes#NEAR_BORDER},
       * {@link yfiles.input.StripeSubregionTypes#FAR_BORDER}, {@link yfiles.input.StripeSubregionTypes#LEADING_HEADER},
       * {@link yfiles.input.StripeSubregionTypes#TRAILING_HEADER},{@link yfiles.input.StripeSubregionTypes#STRIPE}, e.g. a header hit is always
       * returned before a hit on the whole stripe.
       * </li>
       * </ol>
       * @param context The input mode context to use.
       * @param location The location in absolute coordinates.
       * @param node The node where the table instance is currently bound to.
       * @returns A collection of {@link }s at <code>location</code>
       */
      enumerateHits(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,node:yfiles.graph.INode):yfiles.collections.IEnumerable<yfiles.input.StripeSubregion>;
      /**
       * Return the hit test order that is currently used by this instance
       * @type {yfiles.styles.TableRenderingOrder}
       */
      tableRenderingOrder:yfiles.styles.TableRenderingOrder;
      static $class:yfiles.lang.Class;
    }
  }export namespace graph{
    /**
     * Base interface for items that can be used in a {@link yfiles.view.CanvasComponent}.
     * <p>
     * Most prominently this is the interface all graph elements used in {@link yfiles.graph.IGraph} implement.
     * </p>
     * <p>
     * It adds no functionality over the {@link yfiles.graph.ILookup} and {@link yfiles.graph.ITagOwner} interfaces it inherits.
     * </p>
     * <p>
     * To add custom visuals to a {@link yfiles.view.CanvasComponent} it is not necessary to implement this interface. Instead, any
     * object can be {@link yfiles.view.ICanvasObjectGroup#addChild added directly to the canvas}, provided there is a suitable {@link yfiles.view.ICanvasObjectDescriptor}
     * implementation for rendering it.
     * </p>
     * @interface
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.graph.ITagOwner}
     */
    export interface IModelItem extends Object,yfiles.graph.ILookup,yfiles.graph.ITagOwner{
    }
    var IModelItem:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An implementation of the {@link yfiles.graph.IPortLocationModel} that can create {@link yfiles.graph.IPortLocationModelParameter}s that
     * describe the location of ports that are attached to the bends of {@link yfiles.graph.IEdge}s.
     * @class
     * @implements {yfiles.graph.IPortLocationModel}
     * @final
     */
    export interface BendAnchoredPortLocationModel extends Object,yfiles.graph.IPortLocationModel{}
    export class BendAnchoredPortLocationModel {
      /**
       * Creates a parameter that indicates the location of the bend with index <code>bendIndex</code> starting from the source side.
       * @param bendIndex The index of the bend counting from the source side.
       * @returns A parameter that anchors the port at a bend.
       */
      createFromSource(bendIndex:number):yfiles.graph.IPortLocationModelParameter;
      /**
       * Creates a parameter that indicates the location of the bend with index <code>bendIndex</code> starting from the target side.
       * @param bendIndex The index of the bend counting from the target side.
       * @returns A parameter that anchors the port at a bend.
       */
      createFromTarget(bendIndex:number):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param owner
       * @param location
       * @returns 
       */
      createParameter(owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getContext(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getLocation(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.geometry.Point;
      /**
       * This implementation has nothing in its lookup and will always yield <code>null</code>
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A static immutable global instance of this class.
       * @const
       * @static
       * @type {yfiles.graph.BendAnchoredPortLocationModel}
       */
      static INSTANCE:yfiles.graph.BendAnchoredPortLocationModel;
      /**
       * A convenience instance of a parameter that is bound to the first {@link yfiles.graph.IBend} of the edge.
       * @see yfiles.graph.BendAnchoredPortLocationModel#createFromSource
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static FIRST_BEND:yfiles.graph.IPortLocationModelParameter;
      /**
       * A convenience instance of a parameter that is bound to the last {@link yfiles.graph.IBend} of the edge.
       * @see yfiles.graph.BendAnchoredPortLocationModel#createFromTarget
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static LAST_BEND:yfiles.graph.IPortLocationModelParameter;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of {@link yfiles.graph.ILabelModel} that can be used to create custom label models out of existing {@link yfiles.graph.ILabelModel}
     * instances.
     * <p>
     * This class holds a number of models and can provide all of their parameters in
     * {@link yfiles.graph.CompositeLabelModel#getParameters}. A custom model can easily be created that way:
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     */
    export interface CompositeLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider,yfiles.graph.ILabelModelParameterFinder{}
    export class CompositeLabelModel {
      /**
       * Creates a new instance of this label model with an empty {@link yfiles.graph.CompositeLabelModel#labelModels} collection.
       */
      constructor();
      /**
       * Creates a default parameter that can be used for this model.
       * <p>
       * If {@link yfiles.graph.CompositeLabelModel#labelModels} is not empty, the default parameter for the first model in that list is returned. Otherwise, the default parameter
       * for {@link yfiles.graph.FreeLabelModel#createDefaultParameter} is returned.
       * </p>
       * @returns a parameter for this model instance
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Tries to find a parameter that best matches the given layout for the provided label instance.
       * <p>
       * This implementation tests all parameters from all models in {@link yfiles.graph.CompositeLabelModel#labelModels}.
       * </p>
       * @param label The label to find a parameter for.
       * @param model The model instance to use. This should be the instance this instance has been obtained from.
       * @param layout The anticipated layout for the label.
       * @returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
       */
      findBestParameter(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel,layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * Returns an enumerator over a set of possible {@link yfiles.graph.ILabelModelParameter} instances that can be used for the given
       * label and model.
       * <p>
       * This implementation returns all parameters from all models in {@link yfiles.graph.CompositeLabelModel#labelModels}
       * </p>
       * @param label The label instance to use.
       * @param model The model to provide parameters for.
       * @returns A possibly empty enumerator over a set of label model parameters.
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets a list of {@link yfiles.graph.ILabelModel} implementations that are wrapped by this instance.
       * @type {yfiles.collections.IList.<yfiles.graph.ILabelModel>}
       */
      labelModels:yfiles.collections.IList<yfiles.graph.ILabelModel>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable implementation of the {@link yfiles.graph.IBend} interface that can be used stand-alone.
     * <p>
     * Note that this class is not the one used as the default implementation of a {@link yfiles.graph.DefaultGraph}'s
     * {@link yfiles.graph.IBend}s and casting them to {@link yfiles.graph.SimpleBend} will fail. Instances of this class can be used without
     * an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
     * rendering of a {@link yfiles.graph.SimpleEdge} with bends into a graphics context: Simply create two {@link yfiles.graph.SimpleNode}s, a
     * {@link yfiles.graph.SimpleEdge} to connect the two nodes and add instances of this class to the edge's {@link yfiles.graph.SimpleEdge#bends} model. Then render the
     * edge.
     * </p>
     * @class
     * @implements {yfiles.graph.IBend}
     */
    export interface SimpleBend extends Object,yfiles.graph.IBend{}
    export class SimpleBend {
      /**
       * Initializes a new instance of the {@link yfiles.graph.SimpleBend} class using the given owner and location.
       * @param owner The owner of the bend.
       * @param location The bend's location.
       */
      constructor(owner:yfiles.graph.IEdge,location:yfiles.geometry.IPoint);
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the edge this bend instance belongs to.
       * <p>
       * This implies that <code>Owner.Bends</code> contains this instance.
       * </p>
       * @type {yfiles.graph.IEdge}
       */
      owner:yfiles.graph.IEdge;
      /**
       * Gets or sets the location of this bend.
       * @see yfiles.graph.SimpleBend#x
       * @see yfiles.graph.SimpleBend#y
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets the current x coordinate of this bend.
       * <p>
       * This delegates to the {@link yfiles.graph.SimpleBend#location} instance.
       * </p>
       * @type {number}
       */
      x:number;
      /**
       * Gets the current y coordinate of this bend.
       * <p>
       * This delegates to the {@link yfiles.graph.SimpleBend#location} instance.
       * </p>
       * @type {number}
       */
      y:number;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets or sets the {@link yfiles.graph.ILookup} implementation used for calls to {@link yfiles.graph.SimpleBend#lookup} on this instance.
       * @type {yfiles.graph.ILookup}
       */
      lookupImplementation:yfiles.graph.ILookup;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable implementation of the {@link yfiles.graph.IEdge} interface that can be used stand-alone.
     * <p>
     * Note that this class is not the one used as the default implementation of a {@link yfiles.graph.DefaultGraph}'s
     * {@link yfiles.graph.IEdge}s and casting them to {@link yfiles.graph.SimpleEdge} will fail. Instances of this class can be used without
     * an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
     * rendering of an edge into a graphics context: Simply create a
     * {@link yfiles.graph.SimpleEdge}, associate a style with it and use the style's renderer instance to obtain a rendering for the
     * edge.
     * </p>
     * @class
     * @implements {yfiles.graph.IEdge}
     */
    export interface SimpleEdge extends Object,yfiles.graph.IEdge{}
    export class SimpleEdge {
      /**
       * Creates an edge with a {@link yfiles.graph.DefaultGraph#DEFAULT_EDGE_LOOKUP default lookup}, and the given source and target ports.
       * <p>
       * While technically the source and target port can be <code>null</code> here, this will cause all kinds of issues, e.g. with {@link yfiles.styles.IEdgeStyle styles} or
       * {@link yfiles.graph.ILabelModel label models} that require the ports to be non-<code>null</code> to work properly.
       * </p>
       * @param sourcePort The source port for the edge.
       * @param targetPort The target port for the edge.
       */
      constructor(sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort);
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the style that is responsible for the visual representation of this edge in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * Note that the style instance associated with an edge instance may be shared between multiple edge instances and that the
       * modification of this style will result in a change of the appearance of all edges that are associated with the same
       * style instance.
       * </p>
       * @type {yfiles.styles.IEdgeStyle}
       */
      style:yfiles.styles.IEdgeStyle;
      /**
       * Gets or sets the collection of {@link yfiles.graph.IBend bends} that describe the geometry of this edge.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IBend>}
       */
      bends:yfiles.collections.IListEnumerable<yfiles.graph.IBend>;
      /**
       * Gets or sets the source port instance this edge is connected to.
       * <p>
       * Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
       * the edge should be treated as such.
       * </p>
       * @type {yfiles.graph.IPort}
       */
      sourcePort:yfiles.graph.IPort;
      /**
       * Gets or sets the target port instance this edge is connected to.
       * <p>
       * Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
       * the edge should be treated as such.
       * </p>
       * @type {yfiles.graph.IPort}
       */
      targetPort:yfiles.graph.IPort;
      /**
       * Gets or sets the collection of {@link yfiles.graph.IPort ports} that are owned by this instance.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
      /**
       * Gets or sets the collection of {@link yfiles.graph.ILabel labels} that are owned by this instance.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets or sets the {@link yfiles.graph.ILookup} implementation used for calls to {@link yfiles.graph.SimpleEdge#lookup} on this instance.
       * @type {yfiles.graph.ILookup}
       */
      lookupImplementation:yfiles.graph.ILookup;
      static $class:yfiles.lang.Class;
    }
    /**
     * Central implementation of the {@link yfiles.graph.IGraph} interface.
     * <p>
     * This implementation deals with the proper firing of events and handling of default values.
     * <br />
     * This implementation has inherent support for undoability. If the {@link yfiles.graph.DefaultGraph#undoEngineEnabled} property is set to <code>true</code>, changes to the structure of
     * the graph will automatically be recorded using the {@link yfiles.graph.UndoEngine} instance that can be queried from this
     * instance's {@link yfiles.graph.DefaultGraph#lookup} method. Clients that wish to enqueue custom undo events or that need to make
     * non-structural changes to the graph can use the {@link yfiles.graph.UndoEngine} to do so. The current instance of {@link yfiles.graph.UndoEngine}
     * can be queried using the {@link yfiles.graph.ILookup#lookup} method of this instance. Also there are several protected methods <code>Create...UndoUnit</code>
     * that will be used by this instance to create the actual undo units for structural changes in the graph's structure. If
     * clients need to customize these units they can create a subclass of {@link yfiles.graph.DefaultGraph} and override these
     * methods.
     * <br />
     * Finally this implementation supports grouped graphs.
     * </p>
     * <p>
     * <code>DefaultGraph</code> is {@link yfiles.graph.IGraph}'s default implementation as described in the section {@link @DGUIDE_PREFIX@/graph_model-graph.html The Graph Model - IGraph}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Simple Editor Demo, Demo.yFiles.Graph.SimpleEditor</li>
     * </ul>
     * @see yfiles.graph.DefaultGraph#createUndoEngine
     * @see yfiles.graph.DefaultGraph#lookup
     * @see yfiles.graph.DefaultGraph
     * @see yfiles.graph.FoldingManager
     * @class
     * @implements {yfiles.graph.IGraph}
     */
    export interface DefaultGraph extends Object,yfiles.graph.IGraph{}
    export class DefaultGraph {
      /**
       * Creates a new instance of a graph that provides an {@link yfiles.graph.ILookupDecorator} for all of its entities.
       * @param {Object} [options=null] The parameters to pass.
       * @param options.nodes An array of nodes to create in this graph. Each item will be passed to the {@link #createNode} method.
       * @param {yfiles.graph.IEdgeDefaults} [options.edgeDefaults=null] Sets the edgeDefaults property.
       * @param {yfiles.graph.INodeDefaults} [options.groupNodeDefaults=null] Sets the groupNodeDefaults property.
       * @param {yfiles.graph.IMapperRegistry} options.mapperRegistry The mapper registry that is associated with this graph instance.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultGraph#mapperRegistry} property on the created object.
       * </p>
       * @param {yfiles.graph.INodeDefaults} [options.nodeDefaults=null] Sets the nodeDefaults property.
       * @param {boolean} options.undoEngineEnabled Whether or not the {@link yfiles.graph.UndoEngine} used for this instance should be enabled.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultGraph#undoEngineEnabled} property on the created object.
       * </p>
       */
      constructor(options?:{edgeDefaults?:yfiles.graph.IEdgeDefaults,groupNodeDefaults?:yfiles.graph.INodeDefaults,mapperRegistry?:yfiles.graph.IMapperRegistry,nodeDefaults?:yfiles.graph.INodeDefaults,undoEngineEnabled?:boolean,nodes?:Array<any>});
      /**
       * 
       * @param edge
       * @param location
       * @param [index=-1]
       * @returns 
       */
      addBend(edge:yfiles.graph.IEdge,location:yfiles.geometry.Point,index?:number):yfiles.graph.IBend;
      /**
       * 
       * @param owner
       * @param text
       * @param [layoutParameter=null]
       * @param [style=null]
       * @param [preferredSize=null]
       * @param [tag=null]
       * @returns 
       */
      addLabel(owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object):yfiles.graph.ILabel;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.owner
       * @param options.text
       * @param [options.layoutParameter=null]
       * @param [options.style=null]
       * @param [options.preferredSize=null]
       * @param [options.tag=null]
       * @returns 
       */
      addLabel(options:{owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object}):yfiles.graph.ILabel;
      /**
       * Adds the chain element to the lookup for this instance.
       * @param lookup The lookup to decorate the current instance with.
       */
      addLookup(lookup:yfiles.graph.IContextLookupChainLink):void;
      /**
       * Add a port to the given port owner using the coordinates as the new initial position of the port anchor.
       * @param owner the owner to add the port instance to.
       * @param [locationParameter=null] the parameter to use for the port to determine its location.
       * @param [style=null] the style to assign initially to the port, e.g. {@link #INSTANCE}.
       * @param [tag=null] The tag to associate with the port, may be <code>null</code>.
       * @returns the newly created port
       * @throws {Stubs.Exceptions.NotSupportedError} If this instance cannot add a port to <code>owner</code>.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>owner</code> is not in this graph.
       * @see yfiles.graph.DefaultGraph#addPortAddedListener
       */
      addPort(owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,style?:yfiles.styles.IPortStyle,tag?:Object):yfiles.graph.IPort;
      /**
       * Add a port to the given port owner using the coordinates as the new initial position of the port anchor.
       * @param {Object} options The parameters to pass.
       * @param options.owner the owner to add the port instance to.
       * @param [options.locationParameter=null] the parameter to use for the port to determine its location.
       * @param [options.style=null] the style to assign initially to the port, e.g. {@link #INSTANCE}.
       * @param [options.tag=null] The tag to associate with the port, may be <code>null</code>.
       * @returns the newly created port
       * @throws {Stubs.Exceptions.NotSupportedError} If this instance cannot add a port to <code>owner</code>.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>owner</code> is not in this graph.
       * @see yfiles.graph.DefaultGraph#addPortAddedListener
       */
      addPort(options:{owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,style?:yfiles.styles.IPortStyle,tag?:Object}):yfiles.graph.IPort;
      /**
       * Determines whether this graph contains the specified item.
       * @param item The item.
       * @returns <code>true</code> if this graph contains the specified item; otherwise, <code>false</code>.
       */
      contains(item:yfiles.graph.IModelItem):boolean;
      /**
       * 
       * @param sourcePort
       * @param targetPort
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createEdge(sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort,style?:yfiles.styles.IEdgeStyle,tag?:Object):yfiles.graph.IEdge;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.sourcePort
       * @param options.targetPort
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createEdge(options:{sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort,style?:yfiles.styles.IEdgeStyle,tag?:Object}):yfiles.graph.IEdge;
      /**
       * Convenience method that creates and returns an edge that connects to the given node instances using the given style
       * instance.
       * <p>
       * The nodes must be part of this graph at the time of the invocation, and the implementation will choose the {@link yfiles.graph.IPort}
       * instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
       * trigger the corresponding events.
       * </p>
       * @param source The source node the created edge will connect to. This implementation queries the {@link #getNewSourcePort}
       * method to determine which port to use.
       * @param target The target node the created edge will connect to. This implementation queries the {@link #getNewSourcePort}
       * method to determine which port to use.
       * @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [tag=null] the initial {@link #tag} to assign.
       * @returns the newly created edge instance
       * @throws {Stubs.Exceptions.ArgumentError} If <code>source</code> or <code>target</code> is not in this graph.
       * @see yfiles.graph.DefaultGraph#addEdgeCreatedListener
       * @see yfiles.graph.DefaultGraph#getNewSourcePort
       * @see yfiles.graph.DefaultGraph#getNewTargetPort
       * @see yfiles.graph.DefaultGraph#createEdge
       */
      createEdge(source:yfiles.graph.INode,target:yfiles.graph.INode,style?:yfiles.styles.IEdgeStyle,tag?:Object):yfiles.graph.IEdge;
      /**
       * Convenience method that creates and returns an edge that connects to the given node instances using the given style
       * instance.
       * <p>
       * The nodes must be part of this graph at the time of the invocation, and the implementation will choose the {@link yfiles.graph.IPort}
       * instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
       * trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.source The source node the created edge will connect to. This implementation queries the {@link #getNewSourcePort}
       * method to determine which port to use.
       * @param options.target The target node the created edge will connect to. This implementation queries the {@link #getNewSourcePort}
       * method to determine which port to use.
       * @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [options.tag=null] the initial {@link #tag} to assign.
       * @returns the newly created edge instance
       * @throws {Stubs.Exceptions.ArgumentError} If <code>source</code> or <code>target</code> is not in this graph.
       * @see yfiles.graph.DefaultGraph#addEdgeCreatedListener
       * @see yfiles.graph.DefaultGraph#getNewSourcePort
       * @see yfiles.graph.DefaultGraph#getNewTargetPort
       * @see yfiles.graph.DefaultGraph#createEdge
       */
      createEdge(options:{source:yfiles.graph.INode,target:yfiles.graph.INode,style?:yfiles.styles.IEdgeStyle,tag?:Object}):yfiles.graph.IEdge;
      /**
       * 
       * @param [parent=null]
       * @param [layout=null]
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createGroupNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null]
       * @param [options.layout=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createGroupNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * 
       * @param layout
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createNode(layout:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.layout
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createNode(options:{layout:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * 
       * @param [parent=null]
       * @param [layout=null]
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null]
       * @param [options.layout=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * Creates an {@link yfiles.graph.UndoEngine} instance that automatically triggers {@link yfiles.graph.DefaultGraph#invalidateDisplays}
       * upon each invocation of {@link yfiles.graph.UndoEngine#undo} and {@link yfiles.graph.UndoEngine#redo}
       * @returns An {@link } instance.
       * @protected
       */
      createUndoEngine():yfiles.graph.UndoEngine;
      /**
       * Creates the bend addition {@link yfiles.graph.IUndoUnit undo unit} for the given bend.
       * @param bend The bend that has been added.
       * @param index The index at which the bend has been added.
       * @returns The unit that can be used to undo and redo the creation.
       * @protected
       */
      createUndoUnitForBendAddition(bend:yfiles.graph.IBend,index:number):yfiles.graph.IUndoUnit;
      /**
       * Creates the bend removal {@link yfiles.graph.IUndoUnit undo unit} for the given bend.
       * @param bend The bend that will be removed.
       * @returns The unit that can be used to undo and redo the removal.
       * @protected
       */
      createUndoUnitForBendRemoval(bend:yfiles.graph.IBend):yfiles.graph.IUndoUnit;
      /**
       * Creates the edge creation {@link yfiles.graph.IUndoUnit undo unit} for the given edge.
       * @param edge The edge that has been created.
       * @returns The unit that can be used to undo and redo the creation.
       * @protected
       */
      createUndoUnitForEdgeCreation(edge:yfiles.graph.IEdge):yfiles.graph.IUndoUnit;
      /**
       * Creates the edge reconnection {@link yfiles.graph.IUndoUnit undo unit} for the given edge.
       * @param edge The edge that has been reconnected to other ports.
       * @param oldSource The old source port the edge connected to before the change.
       * @param oldTarget The old target port the edge connected to before the change.
       * @returns The unit that can be used to undo and redo the port change operation.
       * @protected
       */
      createUndoUnitForEdgeReconnection(edge:yfiles.graph.IEdge,oldSource:yfiles.graph.IPort,oldTarget:yfiles.graph.IPort):yfiles.graph.IUndoUnit;
      /**
       * Creates the edge removal {@link yfiles.graph.IUndoUnit undo unit} for the given edge.
       * @param edge The edge that will be removed.
       * @returns The unit that can be used to undo and redo the removal.
       * @protected
       */
      createUndoUnitForEdgeRemoval(edge:yfiles.graph.IEdge):yfiles.graph.IUndoUnit;
      /**
       * Creates the {@link yfiles.graph.IUndoUnit undo unit} for changing whether the given node is a group node.
       * @param node The node whose group node status has been changed.
       * @param isGroupNode The new group node status.
       * @returns The unit that can be used to undo and redo the group node status change.
       * @protected
       */
      createUndoUnitForIsGroupNodeChange(node:yfiles.graph.INode,isGroupNode:boolean):yfiles.graph.IUndoUnit;
      /**
       * Creates the label addition {@link yfiles.graph.IUndoUnit undo unit} for the given label.
       * @param label The label that has been added.
       * @returns The unit that can be used to undo and redo the creation.
       * @protected
       */
      createUndoUnitForLabelAddition(label:yfiles.graph.ILabel):yfiles.graph.IUndoUnit;
      /**
       * Creates the label removal {@link yfiles.graph.IUndoUnit undo unit} for the given label.
       * @param label The label that will be removed.
       * @returns The unit that can be used to undo and redo the removal.
       * @protected
       */
      createUndoUnitForLabelRemoval(label:yfiles.graph.ILabel):yfiles.graph.IUndoUnit;
      /**
       * Creates the node creation {@link yfiles.graph.IUndoUnit undo unit} for the given node.
       * @param node The node that has been created.
       * @param parent
       * @param isGroupNode
       * @returns The unit that can be used to undo and redo the creation.
       * @protected
       */
      createUndoUnitForNodeCreation(node:yfiles.graph.INode,parent:yfiles.graph.INode,isGroupNode:boolean):yfiles.graph.IUndoUnit;
      /**
       * Creates the node removal {@link yfiles.graph.IUndoUnit undo unit} for the given node.
       * @param node The node that will be removed.
       * @returns The unit that can be used to undo and redo the removal.
       * @protected
       */
      createUndoUnitForNodeRemoval(node:yfiles.graph.INode):yfiles.graph.IUndoUnit;
      /**
       * Creates the parent change {@link yfiles.graph.IUndoUnit undo unit} for the given node.
       * @param node The node whose parent has been changed.
       * @param oldParent
       * @param newParent The node which is the new parent.
       * @returns The unit that can be used to undo and redo the parent change.
       * @protected
       */
      createUndoUnitForParentChange(node:yfiles.graph.INode,oldParent:yfiles.graph.INode,newParent:yfiles.graph.INode):yfiles.graph.IUndoUnit;
      /**
       * Creates the port addition {@link yfiles.graph.IUndoUnit undo unit} for the given port.
       * @param port The port that has been added.
       * @returns The unit that can be used to undo and redo the creation.
       * @protected
       */
      createUndoUnitForPortAddition(port:yfiles.graph.IPort):yfiles.graph.IUndoUnit;
      /**
       * Creates the port removal {@link yfiles.graph.IUndoUnit undo unit} for the given port.
       * @param port The port that will be removed.
       * @returns The unit that can be used to undo and redo the removal.
       * @protected
       */
      createUndoUnitForPortRemoval(port:yfiles.graph.IPort):yfiles.graph.IUndoUnit;
      /**
       * 
       * @param owner
       * @param [type=yfiles.graph.AdjacencyTypes.ALL]
       * @returns 
       */
      edgesAt(owner:yfiles.graph.IPortOwner,type?:yfiles.graph.AdjacencyTypes):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @param port
       * @param [type=yfiles.graph.AdjacencyTypes.ALL]
       * @returns 
       */
      edgesAt(port:yfiles.graph.IPort,type?:yfiles.graph.AdjacencyTypes):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @param node
       * @returns 
       */
      getChildren(node:yfiles.graph.INode):yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * Returns the lookup implementation that is used for {@link yfiles.graph.DefaultGraph#lookup} calls.
       * @returns The replacing lookup instance or <code>null</code> if the internal lookup mechanism is used.
       * @see yfiles.graph.DefaultGraph#setLookupImplementation
       */
      getLookup():yfiles.graph.ILookup;
      /**
       * Determines a {@link yfiles.graph.IPort} instance to use for the creation of a new edge that starts at the given
       * {@link yfiles.graph.IPortOwner}.
       * <p>
       * This implementation will {@link yfiles.graph.DefaultGraph#addPort add a new center anchored port} to the node.
       * </p>
       * @param source The source node to find a port for.
       * @returns The port to use for a newly created edge.
       * @see yfiles.graph.DefaultGraph#createEdge
       * @protected
       */
      getNewSourcePort(source:yfiles.graph.IPortOwner):yfiles.graph.IPort;
      /**
       * Determines a {@link yfiles.graph.IPort} instance to use for the creation of a new edge that ends at the given
       * {@link yfiles.graph.IPortOwner}.
       * <p>
       * This implementation will {@link yfiles.graph.DefaultGraph#addPort add a new center anchored port} to the node.
       * </p>
       * @param targetOwner The target node to find a port for.
       * @returns The port to use for a newly created edge.
       * @see yfiles.graph.DefaultGraph#createEdge
       * @protected
       */
      getNewTargetPort(targetOwner:yfiles.graph.IPortOwner):yfiles.graph.IPort;
      /**
       * 
       * @param node
       * @returns 
       */
      getParent(node:yfiles.graph.INode):yfiles.graph.INode;
      /**
       * The last element in the lookup chain is implemented by this method.
       * @param type
       * @returns 
       * @protected
       */
      innerLookup(type:yfiles.lang.Class):Object;
      /**
       * Triggers the {@link yfiles.graph.DefaultGraph#addDisplaysInvalidatedListener DisplaysInvalidated} event.
       * <p>
       * Clients can call this method to force an invalidation of the displays that show this graph instance. This implementation
       * delegates to {@link yfiles.graph.DefaultGraph#onInvalidateDisplays}.
       * </p>
       */
      invalidateDisplays():void;
      /**
       * 
       * @param node
       * @returns 
       */
      isGroupNode(node:yfiles.graph.INode):boolean;
      /**
       * Returns an instance that provides another aspect of this instance of the given type or <code>null</code>.
       * <p>
       * Typically, this method will be called in order to obtain a different view or aspect of the current instance. This is
       * quite similar to casting or using a super type or interface of this instance, but is not limited to inheritance or
       * compile time constraints. This method is not required to return non-<code>null</code> implementations for the types, nor does it have
       * to return the same instance any time. Also it depends on the type and context whether the instance returned stays up to
       * date or needs to be re-obtained for subsequent use. This implementation uses the {@link yfiles.graph.DefaultGraph#setLookupImplementation set lookup implementation} if it has
       * been set or an internal {@link yfiles.graph.LookupChain lookup chain}, that can be customized using the {@link yfiles.graph.DefaultGraph#addLookup} and {@link yfiles.graph.DefaultGraph#removeLookup}
       * methods.
       * </p>
       * @param type the type for which an instance shall be returned
       * @returns An instance that is assignable to type or <code>null</code>
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Callback that is invoked before a bend is added to this graph's structure.
       * @param edge The edge the bend will be added to.
       * @param bend The bend that will be added to the edge.
       * @param index The index where the bend will be added to the edge.
       * @protected
       */
      onAddingBend(edge:yfiles.graph.IEdge,bend:yfiles.graph.IBend,index:number):void;
      /**
       * Callback method that will be called just before an edge label is added to an edge.
       * @param edge
       * @param label
       * @protected
       */
      onAddingEdgeLabel(edge:yfiles.graph.IEdge,label:yfiles.graph.ILabel):void;
      /**
       * Callback method that will be called just before a node label is added to a node.
       * @param node
       * @param label
       * @protected
       */
      onAddingNodeLabel(node:yfiles.graph.INode,label:yfiles.graph.ILabel):void;
      /**
       * Called when a port is going to be added to a node.
       * @param owner The node.
       * @param port The port to be added.
       * @protected
       */
      onAddingPort(owner:yfiles.graph.IPortOwner,port:yfiles.graph.IPort):void;
      /**
       * Callback that after a bend has been added to this graph's structure.
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param bend The bend that has been added to the edge.
       * @protected
       */
      onBendAdded(bend:yfiles.graph.IBend):void;
      /**
       * Callback that is invoked after a bend location has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addBendLocationChangedListener BendLocationChanged} event.
       * </p>
       * @param bend The bend whose location has changed.
       * @param oldLocation The bend location before the change
       * @protected
       */
      onBendLocationChanged(bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point):void;
      /**
       * Callback method that is invoked just after a bend has been removed from its edge.
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param owner The old edge.
       * @param bend The bend.
       * @param index The former index of the bend in the {@link #bends} list.
       * @protected
       */
      onBendRemoved(owner:yfiles.graph.IEdge,bend:yfiles.graph.IBend,index:number):void;
      /**
       * Callback that is invoked after a bend tag has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addBendTagChangedListener BendTagChanged} event.
       * </p>
       * @param bend The bend whose tag has changed.
       * @param oldTag The bend tag before the change
       * @protected
       */
      onBendTagChanged(bend:yfiles.graph.IBend,oldTag:Object):void;
      /**
       * Callback that is invoked before the ports of an edge are being changed.
       * @param edge An edge whose the ports are to be changed.
       * @param newSource The new source port
       * @param newTarget The new target port
       * @protected
       */
      onChangingEdgePorts(edge:yfiles.graph.IEdge,newSource:yfiles.graph.IPort,newTarget:yfiles.graph.IPort):void;
      /**
       * Callback that is invoked before an edge style is being changed.
       * @param edge An edge whose the style is going to be changed.
       * @param newStyle The new edge style.
       * @protected
       */
      onChangingEdgeStyle(edge:yfiles.graph.IEdge,newStyle:yfiles.styles.IEdgeStyle):void;
      /**
       * Callback that is invoked before the graph's tag is being changed.
       * @param newTag The new port tag
       * @protected
       */
      onChangingGraphTag(newTag:Object):void;
      /**
       * Callback that is invoked before the {@link yfiles.graph.ILabel#layoutParameter} of a label is being changed.
       * @param label A label whose {@link #layoutParameter} is to be changed.
       * @param newLayoutParameter The new label model parameter
       * @protected
       */
      onChangingLabelLayoutParameter(label:yfiles.graph.ILabel,newLayoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * Callback that is invoked before the preferred size of a label is being changed.
       * @param label A label whose preferred size is going to be changed.
       * @param newSize The new preferred size
       * @protected
       */
      onChangingLabelPreferredSize(label:yfiles.graph.ILabel,newSize:yfiles.geometry.Size):void;
      /**
       * Callback that is invoked before a label style is being changed.
       * @param label A label whose the style is going to be changed.
       * @param newStyle The new label style.
       * @protected
       */
      onChangingLabelStyle(label:yfiles.graph.ILabel,newStyle:yfiles.styles.ILabelStyle):void;
      /**
       * Callback that is invoked before the text of a label is being changed.
       * @param label A label whose text is going to be changed.
       * @param newText The new label text
       * @protected
       */
      onChangingLabelText(label:yfiles.graph.ILabel,newText:string):void;
      /**
       * Callback that is invoked before a node style is being changed.
       * @param node A node whose the style is going to be changed.
       * @param newStyle The new node style.
       * @protected
       */
      onChangingNodeStyle(node:yfiles.graph.INode,newStyle:yfiles.styles.INodeStyle):void;
      /**
       * Callback that is invoked before the {@link yfiles.graph.IPort#locationParameter} of a port is being changed.
       * @param port A port whose {@link #locationParameter} is to be changed.
       * @param newLocationParameter The new location model parameter
       * @protected
       */
      onChangingPortLocationModelParameter(port:yfiles.graph.IPort,newLocationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * Callback that is invoked before a port style is being changed.
       * @param port A port whose the style is going to be changed.
       * @param newStyle The new port style.
       * @protected
       */
      onChangingPortStyle(port:yfiles.graph.IPort,newStyle:yfiles.styles.IPortStyle):void;
      /**
       * Callback that is invoked before the edge is added to this graph's structure.
       * @param edge An edge that is not yet contained in this graph.
       * @param sourcePort The source port this edge will connect to.
       * @param targetPort The target port this edge will connect to.
       * @protected
       */
      onCreatingEdge(edge:yfiles.graph.IEdge,sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):void;
      /**
       * Callback that is invoked before the node is added to this graph's structure.
       * @param node A node that is not yet contained in this graph.
       * @protected
       */
      onCreatingNode(node:yfiles.graph.INode):void;
      /**
       * Callback that triggers the {@link yfiles.graph.DefaultGraph#addEdgeCreatedListener EdgeCreated} event.
       * @param edge The edge that has been created.
       * @protected
       */
      onEdgeCreated(edge:yfiles.graph.IEdge):void;
      /**
       * Called after a label has been added to an edge.
       * <p>
       * This method triggers the {@link yfiles.graph.DefaultGraph#addLabelAddedListener LabelAdded} event.
       * </p>
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param label The label that has just been added.
       * @protected
       */
      onEdgeLabelAdded(label:yfiles.graph.ILabel):void;
      /**
       * Called after a label has been removed from its edge.
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param label The label that has just been removed.
       * @param owner The previous owner of the label.
       * @protected
       */
      onEdgeLabelRemoved(owner:yfiles.graph.ILabelOwner,label:yfiles.graph.ILabel):void;
      /**
       * Callback that is invoked after the ports of an edge has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addEdgePortsChangedListener EdgePortsChanged} event.
       * </p>
       * @param edge The edge whose ports have changed.
       * @param oldSource The source port that the edge had been connected to before the change.
       * @param oldTarget The target port that the edge had been connected to before the change.
       * @protected
       */
      onEdgePortsChanged(edge:yfiles.graph.IEdge,oldSource:yfiles.graph.IPort,oldTarget:yfiles.graph.IPort):void;
      /**
       * Callback that triggers the {@link yfiles.graph.DefaultGraph#addEdgeRemovedListener EdgeRemoved} event.
       * @param edge The edge that got removed
       * @param oldSource The source port that the edge had been connected to.
       * @param oldTarget The target port that the edge had been connected to.
       * @protected
       */
      onEdgeRemoved(edge:yfiles.graph.IEdge,oldSource:yfiles.graph.IPort,oldTarget:yfiles.graph.IPort):void;
      /**
       * Callback that is invoked after the style of an edge has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addEdgeStyleChangedListener EdgeStyleChanged} event.
       * </p>
       * @param edge The edge whose style that has changed.
       * @param oldStyle The edge style before the change
       * @protected
       */
      onEdgeStyleChanged(edge:yfiles.graph.IEdge,oldStyle:yfiles.styles.IEdgeStyle):void;
      /**
       * Callback that is invoked after the tag of an edge has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addEdgeTagChangedListener EdgeTagChanged} event.
       * </p>
       * @param edge The edge whose tag has changed.
       * @param oldTag The edge tag before the change.
       * @protected
       */
      onEdgeTagChanged(edge:yfiles.graph.IEdge,oldTag:Object):void;
      /**
       * Callback that triggers the {@link yfiles.graph.DefaultGraph#addGraphTagChangedListener GraphTagChanged} event.
       * @param oldTag The graph tag before the change.
       * @protected
       */
      onGraphTagChanged(oldTag:Object):void;
      /**
       * Triggers the {@link yfiles.graph.DefaultGraph#addDisplaysInvalidatedListener DisplaysInvalidated} event.
       * @param evt The event arguments.
       * @protected
       */
      onInvalidateDisplays(evt:yfiles.lang.EventArgs):void;
      /**
       * Callback that is invoked after a label model parameter has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addLabelLayoutParameterChangedListener LabelLayoutParameterChanged} event.
       * </p>
       * @param label The label whose model parameter has changed.
       * @param oldLayoutParamater The label model parameter before the change
       * @protected
       */
      onLabelLayoutParameterChanged(label:yfiles.graph.ILabel,oldLayoutParamater:yfiles.graph.ILabelModelParameter):void;
      /**
       * Callback that is invoked after the preferred size of a label has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addLabelPreferredSizeChangedListener LabelPreferredSizeChanged} event.
       * </p>
       * @param label The label whose preferred size has changed.
       * @param oldSize The preferred size of the label before the change
       */
      onLabelPreferredSizeChanged(label:yfiles.graph.ILabel,oldSize:yfiles.geometry.Size):void;
      /**
       * Callback that is invoked after a label style has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addLabelStyleChangedListener LabelStyleChanged} event.
       * </p>
       * @param label The label whose style has changed.
       * @param oldStyle The label style before the change.
       * @protected
       */
      onLabelStyleChanged(label:yfiles.graph.ILabel,oldStyle:yfiles.styles.ILabelStyle):void;
      /**
       * Callback that is invoked after a label tag has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addLabelTagChangedListener LabelTagChanged} event.
       * </p>
       * @param label The label whose tag has changed.
       * @param oldTag The label tag before the change
       */
      onLabelTagChanged(label:yfiles.graph.ILabel,oldTag:Object):void;
      /**
       * Callback that is invoked after a label text has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addLabelTextChangedListener LabelTextChanged} event.
       * </p>
       * @param label The label whose text has changed.
       * @param oldText The label text before the change.
       */
      onLabelTextChanged(label:yfiles.graph.ILabel,oldText:string):void;
      /**
       * Callback that triggers the {@link yfiles.graph.DefaultGraph#addNodeCreatedListener NodeCreated} event.
       * @param node The node that has been created.
       * @protected
       */
      onNodeCreated(node:yfiles.graph.INode):void;
      /**
       * Called after a label has been added to a node.
       * <p>
       * This method triggers the {@link yfiles.graph.DefaultGraph#addLabelAddedListener LabelAdded} event.
       * </p>
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param label The label that has just been added.
       * @protected
       */
      onNodeLabelAdded(label:yfiles.graph.ILabel):void;
      /**
       * Called after a label has been removed from its node.
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param label The label that has just been removed.
       * @param owner The old owner of the label.
       * @protected
       */
      onNodeLabelRemoved(owner:yfiles.graph.ILabelOwner,label:yfiles.graph.ILabel):void;
      /**
       * Callback that is invoked after a node layout has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addNodeLayoutChangedListener NodeLayoutChanged} event.
       * </p>
       * @param node The node whose layout has changed.
       * @param oldLayout The node layout before the change
       * @protected
       */
      onNodeLayoutChanged(node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect):void;
      /**
       * Callback that triggers the {@link yfiles.graph.DefaultGraph#addNodeRemovedListener NodeRemoved} event.
       * @param node The node that got removed
       * @param oldParent
       * @param oldIsGroupNode
       * @protected
       */
      onNodeRemoved(node:yfiles.graph.INode,oldParent:yfiles.graph.INode,oldIsGroupNode:boolean):void;
      /**
       * Callback that is invoked after a node style has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addNodeStyleChangedListener NodeStyleChanged} event.
       * </p>
       * @param node The node whose style has changed.
       * @param oldStyle The node style before the change
       * @protected
       */
      onNodeStyleChanged(node:yfiles.graph.INode,oldStyle:yfiles.styles.INodeStyle):void;
      /**
       * Callback that is invoked after a node tag has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addNodeTagChangedListener NodeTagChanged} event.
       * </p>
       * @param node The node whose tag has changed.
       * @param oldTag The node tag before the change
       * @protected
       */
      onNodeTagChanged(node:yfiles.graph.INode,oldTag:Object):void;
      /**
       * Called when a port has been added to a node.
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param port The port that has just been added to its owner.
       * @protected
       */
      onPortAdded(port:yfiles.graph.IPort):void;
      /**
       * Callback that is invoked after a port location model parameter has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addPortLocationParameterChangedListener PortLocationParameterChanged}
       * event.
       * </p>
       * @param port The port whose location model parameter has changed.
       * @param oldLocationParameter The port location model parameter before the change
       */
      onPortLocationParameterChanged(port:yfiles.graph.IPort,oldLocationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * Callback method that is called just after a port has been removed from its owner.
       * <p>
       * When overriding this method it is mandatory to call the base implementation.
       * </p>
       * @param port The port that has been removed.
       * @param oldOwner The previous owner of the port.
       * @protected
       */
      onPortRemoved(oldOwner:yfiles.graph.IPortOwner,port:yfiles.graph.IPort):void;
      /**
       * Callback that is invoked after a port style has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addPortStyleChangedListener PortStyleChanged} event.
       * </p>
       * @param port The port whose style has changed.
       * @param oldStyle The port style before the change
       * @protected
       */
      onPortStyleChanged(port:yfiles.graph.IPort,oldStyle:yfiles.styles.IPortStyle):void;
      /**
       * Callback that is invoked after a port tag has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.DefaultGraph#addPortTagChangedListener PortTagChanged} event.
       * </p>
       * @param port The port whose tag has changed.
       * @param oldTag The port tag before the change
       * @protected
       */
      onPortTagChanged(port:yfiles.graph.IPort,oldTag:Object):void;
      /**
       * Callback method that is invoked just before a bend is removed.
       * @param bend The bend that will be removed.
       * @protected
       */
      onRemovingBend(bend:yfiles.graph.IBend):void;
      /**
       * Callback that will be invoked before the node will be removed.
       * @param edge That edge that is going to be removed.
       * @protected
       */
      onRemovingEdge(edge:yfiles.graph.IEdge):void;
      /**
       * Callback method that will be called just before an edge label is removed from its edge.
       * @param label
       * @protected
       */
      onRemovingEdgeLabel(label:yfiles.graph.ILabel):void;
      /**
       * Callback that will be invoked before the node will be removed.
       * @param node That node that is going to be removed.
       * @protected
       */
      onRemovingNode(node:yfiles.graph.INode):void;
      /**
       * Callback method that will be called just before a node label is removed from its node.
       * @param label
       * @protected
       */
      onRemovingNodeLabel(label:yfiles.graph.ILabel):void;
      /**
       * Callback method that is called just before a port will be removed.
       * @param port The port that is about to be removed.
       * @protected
       */
      onRemovingPort(port:yfiles.graph.IPort):void;
      /**
       * 
       * @param item
       */
      remove(item:yfiles.graph.IModelItem):void;
      /**
       * Removes a previously added lookup chain element from the lookup of <code>this</code>.
       * @param lookup The element to remove.
       */
      removeLookup(lookup:yfiles.graph.IContextLookupChainLink):void;
      /**
       * Modifies the location of the given bend.
       * <p>
       * This implementation will also trigger an {@link yfiles.graph.DefaultGraph#invalidateDisplays} call.
       * </p>
       * @param bend the bend whose location is to be modified
       * @param location the new coordinates of the bend
       * @throws {Stubs.Exceptions.ArgumentError} If <code>bend</code> is not in this graph.
       * @see yfiles.graph.DefaultGraph#addBend
       */
      setBendLocation(bend:yfiles.graph.IBend,location:yfiles.geometry.Point):void;
      /**
       * Sets the ports of the given edge to the new values.
       * <p>
       * This will trigger an {@link yfiles.graph.DefaultGraph#addEdgePortsChangedListener EdgePortsChanged} event if source or
       * target ports differ from the current ones. Both ports and the edge must belong to the current graph instance. If {@link yfiles.graph.IPortDefaults#autoCleanup} is
       * enabled, this method will remove unoccupied ports after they have been changed.
       * </p>
       * @param edge The edge to change the ports.
       * @param sourcePort The new source port instance.
       * @param targetPort The new target port instance.
       */
      setEdgePorts(edge:yfiles.graph.IEdge,sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):void;
      /**
       * 
       * @param node
       * @param isGroupNode
       */
      setIsGroupNode(node:yfiles.graph.INode,isGroupNode:boolean):void;
      /**
       * 
       * @param label
       * @param layoutParameter
       */
      setLabelLayoutParameter(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * 
       * @param label
       * @param size
       */
      setLabelPreferredSize(label:yfiles.graph.ILabel,size:yfiles.geometry.Size):void;
      /**
       * Sets the label text of the given label.
       * <p>
       * This implementation will {@link yfiles.graph.IGraph#adjustLabelPreferredSize adjust the preferred size} for the label if {@link yfiles.graph.ILabelDefaults#autoAdjustPreferredSize} is enabled.
       * </p>
       * @param label the label to modify
       * @param text the new text of the label
       * @throws {Stubs.Exceptions.ArgumentError} If <code>label</code> is not in this graph.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>text</code> is <code>null</code>.
       * @see yfiles.graph.ILabel#text
       */
      setLabelText(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Sets the lookup implementation that will be used for {@link yfiles.graph.DefaultGraph#lookup} calls.
       * <p>
       * A value of <code>null</code> will revert to the default behavior.
       * </p>
       * @param lookup The lookup instance to delegate lookup calls to.
       * @see yfiles.graph.DefaultGraph#addLookup
       * @see yfiles.graph.DefaultGraph#getLookup
       * @protected
       */
      setLookupImplementation(lookup:yfiles.graph.ILookup):void;
      /**
       * Sets the layout of the given node to the new values.
       * <p>
       * This will trigger a call to {@link yfiles.graph.DefaultGraph#invalidateDisplays}.
       * </p>
       * @param node a live node that belongs to this graph
       * @param layout the new absolute layout in world coordinates of the node
       * @see yfiles.graph.INode#layout
       */
      setNodeLayout(node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * 
       * @param node
       * @param parent
       */
      setParent(node:yfiles.graph.INode,parent:yfiles.graph.INode):void;
      /**
       * 
       * @param port
       * @param locationParameter
       */
      setPortLocationParameter(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * 
       * @param port
       * @param style
       */
      setStyle(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):void;
      /**
       * 
       * @param edge
       * @param style
       */
      setStyle(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):void;
      /**
       * Assigns the given style instance by reference to the label.
       * <p>
       * Style instances can be shared.
       * </p>
       * <p>
       * This implementation will {@link yfiles.graph.IGraph#adjustLabelPreferredSize adjust the preferred size} for the label if {@link yfiles.graph.ILabelDefaults#autoAdjustPreferredSize} is enabled.
       * </p>
       * @param label The label that will be assigned the new style
       * @param style The style instance that will be assigned to the label.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>label</code> is not in this graph.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>style</code> is <code>null</code>.
       * @see yfiles.graph.ILabel#style
       * @see yfiles.graph.DefaultGraph#onChangingLabelStyle
       * @see yfiles.graph.DefaultGraph#addLabelStyleChangedListener
       */
      setStyle(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):void;
      /**
       * Assigns the given style instance by reference to the node.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param node The node that will be assigned the new style
       * @param style The style instance that will be assigned to the node.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>node</code> is not in this graph.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>style</code> is <code>null</code>.
       * @see yfiles.graph.INode#style
       * @see yfiles.graph.DefaultGraph#onChangingNodeStyle
       * @see yfiles.graph.DefaultGraph#addNodeStyleChangedListener
       */
      setStyle(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):void;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.INode}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the node lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_NODE_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.IEdge}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the edge lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_EDGE_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.ILabel}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the label lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_LABEL_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.IPort}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the port lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_PORT_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.IBend}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the bend lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_BEND_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets the mapper registry that is associated with this graph instance.
       * <p>
       * The registry can be used to store data mappings for the items in this instance. This field is populated upon first
       * access. Also this property provides write access.
       * </p>
       * @see yfiles.graph.IMapperRegistry
       * @see yfiles.graph.IModelItem
       * @see yfiles.graph.ITagOwner#tag
       * @type {yfiles.graph.IMapperRegistry}
       */
      mapperRegistry:yfiles.graph.IMapperRegistry;
      /**
       * Gets or sets whether or not the {@link yfiles.graph.UndoEngine} used for this instance should be enabled.
       * <p>
       * The default is <code>false</code>. In order to enable undoability for this instance, set this property to <code>true</code> and {@link yfiles.graph.DefaultGraph#lookup}
       * the {@link yfiles.graph.UndoEngine} type. Disabling this property clears and removes the current {@link yfiles.graph.UndoEngine} (and
       * all enqueued IUndoUnits).
       * </p>
       * @default false
       * @type {boolean}
       */
      undoEngineEnabled:boolean;
      /**
       * 
       * @type {yfiles.graph.IEdgeDefaults}
       */
      edgeDefaults:yfiles.graph.IEdgeDefaults;
      /**
       * 
       * @type {yfiles.graph.INodeDefaults}
       */
      nodeDefaults:yfiles.graph.INodeDefaults;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.INode>}
       */
      nodes:yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IEdge>}
       */
      edges:yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @param listener
       */
      addGraphTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeGraphTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeRemovedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeRemovedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeLayoutChangedListener(listener:(source:Object,node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeLayoutChangedListener(listener:(source:Object,node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeRemovedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeRemovedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendRemovedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendRemovedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendLocationChangedListener(listener:(source:Object,bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendLocationChangedListener(listener:(source:Object,bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelPreferredSizeChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelPreferredSizeChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelLayoutParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelLayoutParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortRemovedListener(listener:(sender:Object,evt:yfiles.graph.PortEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortRemovedListener(listener:(sender:Object,evt:yfiles.graph.PortEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortLocationParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortLocationParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addDisplaysInvalidatedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeDisplaysInvalidatedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * 
       * @type {yfiles.graph.INodeDefaults}
       */
      groupNodeDefaults:yfiles.graph.INodeDefaults;
      /**
       * 
       * @param listener
       */
      addParentChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeParentChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addIsGroupNodeChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeIsGroupNodeChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable implementation of the {@link yfiles.graph.ILabel} interface that can be used stand-alone.
     * <p>
     * Note that this class is not the one used as the default implementation of a {@link yfiles.graph.DefaultGraph}'s
     * {@link yfiles.graph.ILabel}s and casting them to {@link yfiles.graph.SimpleLabel} will fail. Instances of this class can be used without
     * an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
     * rendering of a label into a graphics context: Simply create a
     * {@link yfiles.graph.SimpleLabel}, associate a style with it and use the style's renderer instance to obtain a rendering for the
     * label.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabel}
     */
    export interface SimpleLabel extends Object,yfiles.graph.ILabel{}
    export class SimpleLabel {
      /**
       * Creates an instance of the {@link yfiles.graph.SimpleLabel} class with default {@link yfiles.graph.DefaultGraph#DEFAULT_LABEL_LOOKUP lookup}, an instance of {@link yfiles.styles.VoidLabelStyle}
       * and the given {@link yfiles.graph.ILabelModelParameter parameter}, text, and owner.
       * @param layoutParameter The parameter that describes the label's layout.
       * @param text The label's text.
       * @param owner The label's owner.
       */
      constructor(owner:yfiles.graph.ILabelOwner,text:string,layoutParameter:yfiles.graph.ILabelModelParameter);
      /**
       * Gets the {@link yfiles.styles.ILabelStyleRenderer#getPreferredSize} from the labels style and sets the {@link yfiles.graph.SimpleLabel#preferredSize} to that value.
       */
      adoptPreferredSizeFromStyle():void;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the label model's parameter that determines the positioning of this label.
       * <p>
       * Note that the label model parameter instance associated with a label instance may be shared between multiple label
       * instances and that the modification of this instance or its {@link yfiles.graph.ILabelModelParameter#model} will result in a change of the positioning of all labels
       * that are associated with the same parameter or model instance.
       * </p>
       * @type {yfiles.graph.ILabelModelParameter}
       */
      layoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the text string associated with this label.
       * <p>
       * It is up to the visualization engine to interpret this property for the visualization of the label. Normally, it will
       * render the text into the {@link yfiles.graph.ILabel#layout layout} of this instance.
       * </p>
       * @type {string}
       */
      text:string;
      /**
       * Gets or sets the style that is responsible for the visual representation of this node in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * Note that the style instance associated with a label instance may be shared between multiple label instances and that
       * the modification of this style will result in a change of the appearance of all labels that are associated with the same
       * style instance.
       * </p>
       * @type {yfiles.styles.ILabelStyle}
       */
      style:yfiles.styles.ILabelStyle;
      /**
       * Gets or sets the owner of this label.
       * <p>
       * Typically this will yield an instance of {@link yfiles.graph.INode} or {@link yfiles.graph.IEdge}.
       * </p>
       * @type {yfiles.graph.ILabelOwner}
       */
      owner:yfiles.graph.ILabelOwner;
      /**
       * Gets or sets the preferred size of the label with respect to its current contents and the implementation of the
       * visualization.
       * <p>
       * Often times the {@link yfiles.graph.ILabel#layout layout's} size will be the same as the preferred size, but it's up to the implementation of the {@link yfiles.graph.ILabelModel}
       * to interpret it differently.
       * </p>
       * @type {yfiles.geometry.Size}
       */
      preferredSize:yfiles.geometry.Size;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets or sets the {@link yfiles.graph.ILookup} implementation used for calls to {@link yfiles.graph.SimpleLabel#lookup} on this
       * instance.
       * @type {yfiles.graph.ILookup}
       */
      lookupImplementation:yfiles.graph.ILookup;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable implementation of the {@link yfiles.graph.INode} interface that can be used stand-alone.
     * <p>
     * Note that this class is not the one used as the default implementation of a {@link yfiles.graph.DefaultGraph}'s
     * {@link yfiles.graph.INode}s and casting them to {@link yfiles.graph.SimpleNode} will fail.
     * </p>
     * <p>
     * Instances of this class can be used without an accompanying graph instance. All of the properties are mutable. An
     * example of a possible use case would be the rendering of a node into a graphics context: Simply create a
     * {@link yfiles.graph.SimpleNode}, associate a style with it and use the style's renderer instance to obtain a rendering for the
     * node.
     * </p>
     * @class
     * @implements {yfiles.graph.INode}
     */
    export interface SimpleNode extends Object,yfiles.graph.INode{}
    export class SimpleNode {
      /**
       * Creates a default node with default {@link yfiles.graph.DefaultGraph#DEFAULT_NODE_LOOKUP lookup}, an instance of {@link yfiles.styles.VoidNodeStyle}, empty {@link yfiles.graph.SimpleNode#layout} and no labels or ports.
       * <p>
       * The instances assigned to {@link yfiles.graph.SimpleNode#layout}, {@link yfiles.graph.SimpleNode#ports} and {@link yfiles.graph.SimpleNode#labels} are immutable by default. To modify the values of those properties, first assign an
       * instance of a mutable class.
       * </p>
       */
      constructor();
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the layout of the node.
       * <p>
       * The layout of a node is a rectangle in the world coordinate system that describes the bounding box of the representation
       * of a node.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the style that is responsible for the visual representation of this node in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * Note that the style instance associated with a node instance may be shared between multiple node instances and that the
       * modification of this style will result in a change of the appearance of all nodes that are associated with the same
       * style instance.
       * </p>
       * @type {yfiles.styles.INodeStyle}
       */
      style:yfiles.styles.INodeStyle;
      /**
       * Gets or sets the collection of {@link yfiles.graph.IPort ports} that are owned by this instance.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
      /**
       * Gets or sets the collection of {@link yfiles.graph.ILabel labels} that are owned by this instance.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets or sets the {@link yfiles.graph.ILookup} implementation used for calls to {@link yfiles.graph.SimpleNode#lookup} on this instance.
       * @type {yfiles.graph.ILookup}
       */
      lookupImplementation:yfiles.graph.ILookup;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable implementation of the {@link yfiles.graph.IPort} interface that can be used stand-alone.
     * <p>
     * Note that this class is not the one used as the default implementation of a {@link yfiles.graph.DefaultGraph}'s
     * {@link yfiles.graph.IPort}s and casting them to {@link yfiles.graph.SimplePort} will fail. Instances of this class can be used without
     * an accompanying graph instance. All of the properties are mutable. An example of a possible use case would be the
     * rendering of a port into a graphics context: Simply create a
     * {@link yfiles.graph.SimplePort}, associate a style with it and use the style's renderer instance to obtain a rendering for the
     * port.
     * </p>
     * @class
     * @implements {yfiles.graph.IPort}
     */
    export interface SimplePort extends Object,yfiles.graph.IPort{}
    export class SimplePort {
      /**
       * Creates a port with a {@link yfiles.graph.DefaultGraph#DEFAULT_PORT_LOOKUP default lookup} using the given location and owner.
       * <p>
       * While technically the owner can be <code>null</code> here, this can cause issues, e.g. with {@link yfiles.graph.IPortLocationModel port location models} that position the
       * port relative to its owner.
       * </p>
       * @param owner The port's owner.
       * @param locationParameter The port's location model parameter.
       */
      constructor(owner:yfiles.graph.IPortOwner,locationParameter:yfiles.graph.IPortLocationModelParameter);
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Sets the location of this port.
       * @param location
       */
      setLocation(location:yfiles.geometry.Point):void;
      /**
       * Gets or sets the style that is responsible for the visual representation of this port in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * Note that the style instance associated with a port instance may be shared between multiple port instances and that the
       * modification of this style will result in a change of the appearance of all ports that are associated with the same
       * style instance.
       * </p>
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * Gets or sets the {@link yfiles.graph.IPortLocationModelParameter} that is used to determine the location of this port.
       * <p>
       * The port location can be calculated using the {@link yfiles.graph.IPortLocationModelParameter#model}'s {@link yfiles.graph.IPortLocationModel#getLocation} method or the convenience
       * methods {@link yfiles.graph.IPort#location} and {@link yfiles.graph.IPort#dynamicLocation}.
       * </p>
       * <p>
       * Note that parameters may be shared across port instances.
       * </p>
       * @see yfiles.graph.IGraph#setPortLocationParameter
       * @see yfiles.graph.IPort#location
       * @see yfiles.graph.IPort#dynamicLocation
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Gets or sets the owner of this port.
       * @type {yfiles.graph.IPortOwner}
       */
      owner:yfiles.graph.IPortOwner;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets or sets the {@link yfiles.graph.ILookup} implementation used for calls to {@link yfiles.graph.SimplePort#lookup} on this instance.
       * @type {yfiles.graph.ILookup}
       */
      lookupImplementation:yfiles.graph.ILookup;
      static $class:yfiles.lang.Class;
    }
    /**
     * A label model that wraps an existing label model and decorates it with a {@link yfiles.graph.ILabelCandidateDescriptor descriptor}.
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelCandidateDescriptorProvider}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface DescriptorWrapperLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelCandidateDescriptorProvider,yfiles.graph.ILabelModelParameterProvider{}
    export class DescriptorWrapperLabelModel {
      /**
       * Initializes a new instance of the {@link yfiles.graph.DescriptorWrapperLabelModel} class.
       * @param innerModel The wrapped label model.
       */
      constructor(innerModel:yfiles.graph.ILabelModel);
      /**
       * Initializes a new instance of the {@link yfiles.graph.DescriptorWrapperLabelModel} class.
       * @param {Object} options The parameters to pass.
       * @param options.innerModel The wrapped label model.
       * @param {yfiles.graph.ILabelCandidateDescriptor} options.descriptor The descriptor.
       * <p>
       * This option sets the {@link yfiles.graph.DescriptorWrapperLabelModel#descriptor} property on the created object.
       * </p>
       */
      constructor(options:{innerModel:yfiles.graph.ILabelModel,descriptor?:yfiles.graph.ILabelCandidateDescriptor});
      /**
       * Creates a new instance that uses {@link yfiles.graph.FreeLabelModel#INSTANCE} as inner model.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.graph.ILabelCandidateDescriptor} options.descriptor The descriptor.
       * <p>
       * This option sets the {@link yfiles.graph.DescriptorWrapperLabelModel#descriptor} property on the created object.
       * </p>
       * @param {yfiles.graph.ILabelModel} options.innerModel The inner label model
       * <p>
       * This option sets the {@link yfiles.graph.DescriptorWrapperLabelModel#innerModel} property on the created object.
       * </p>
       */
      constructor(options?:{descriptor?:yfiles.graph.ILabelCandidateDescriptor,innerModel?:yfiles.graph.ILabelModel});
      /**
       * Creates a default parameter that can be used for this model.
       * @returns The default parameter from {@link #innerModel}
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getDescriptor(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILabelCandidateDescriptor;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the descriptor.
       * @type {yfiles.graph.ILabelCandidateDescriptor}
       */
      descriptor:yfiles.graph.ILabelCandidateDescriptor;
      /**
       * Gets or sets the inner label model
       * @type {yfiles.graph.ILabelModel}
       */
      innerModel:yfiles.graph.ILabelModel;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data holder that can be used to describe a {@link yfiles.graph.IFoldingView#isInFoldingState folding} edge that connects to nodes inside a collapsed group or to other
     * folder nodes in a {@link yfiles.graph.IFoldingView}.
     * <p>
     * Instances of this type are passed to the various methods in {@link yfiles.graph.FoldingManager} to identify different possible
     * states of an edge inside {@link yfiles.graph.IFoldingView}s. A folding edge is identified through the master edge, which is an
     * edge in the {@link yfiles.graph.FoldingManager#masterGraph} and a source and target node. Also whether those source and target nodes are currently {@link yfiles.graph.IFoldingView#isInFoldingState folder} nodes needs
     * to be taken into account. The source and target nodes need to be either the true {@link yfiles.graph.IPort#owner owners} of the adjacent edge's {@link yfiles.graph.IEdge#sourcePort} and {@link yfiles.graph.IEdge#targetPort}
     * respectively, or {@link yfiles.graph.IGraph#getParent ancestor} group nodes in the {@link yfiles.graph.FoldingManager#masterGraph}. In the latter case these folding edges are used to describe the state of
     * the folding edges that are present in the views of the corresponding group nodes are in {@link yfiles.graph.IFoldingView#collapse collapsed} state, so that the
     * true source and target nodes are not currently visible in the view. As such the {@link yfiles.graph.FoldingEdgeStateId#currentMasterSource} and {@link yfiles.graph.FoldingEdgeStateId#currentMasterTarget} nodes represent the {@link yfiles.graph.IFoldingView#getMasterItem masters} of the
     * nearest ancestors in the {@link yfiles.graph.FoldingManager#masterGraph} that are visible in the {@link yfiles.graph.IFoldingView folding view} graph.
     * </p>
     * @see yfiles.graph.FoldingManager#hasFoldingEdgeState
     * @see yfiles.graph.FoldingManager#getFoldingEdgeState
     * @see yfiles.graph.IFoldingView#isInFoldingState
     * @class
     */
    export interface FoldingEdgeStateId extends Object{}
    export class FoldingEdgeStateId {
      /**
       * Initializes an id for a folding edge in a {@link yfiles.graph.IFoldingView view} that represent a master edge and connects a specific pair of nodes.
       * <p>
       * All of the arguments need to be entities of the {@link yfiles.graph.FoldingManager#masterGraph master graph}.
       * </p>
       * @param masterEdge An edge in the {@link #masterGraph} that is represented by the folding edge.
       * @param currentMasterSource The source node of the edge or one of its ancestors to which the folding edge connects in the view, if the actual parent
       * is currently {@link #collapse collapsed in an ancestor node}.
       * @param sourceCollapsed <code>true</code> if the <code>currentMasterSource</code> is the true source of the <code>masterEdge</code>, but is in its {@link #collapse collapsed/dummy} state.
       * @param targetCollapsed <code>true</code> if the <code>currentMasterTarget</code> is the true target of the <code>masterEdge</code>, but is in its {@link #collapse collapsed/dummy} state.
       * @param currentMasterTarget The target node of the edge or one of its ancestors to which the folding edge connects in the view, if the actual parent
       * is currently {@link #collapse collapsed in an ancestor node}.
       */
      constructor(masterEdge:yfiles.graph.IEdge,currentMasterSource:yfiles.graph.INode,sourceCollapsed:boolean,currentMasterTarget:yfiles.graph.INode,targetCollapsed:boolean);
      /**
       * Initializes an id for a folding edge that is currently part of the given view.
       * @param foldingView The view the folding edge is part of.
       * @param foldingEdge The instance that is part of the {@link #graph folding view graph}.
       */
      constructor(foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge);
      /**
       * The edge instance in the {@link yfiles.graph.FoldingManager#masterGraph} that is represented by the folding edge in a view.
       * @type {yfiles.graph.IEdge}
       */
      masterEdge:yfiles.graph.IEdge;
      /**
       * The source node of the edge or an ancestor (both in the {@link yfiles.graph.FoldingManager#masterGraph}).
       * <p>
       * This may be an ancestor of the true source node if this ancestor is {@link yfiles.graph.IFoldingView#collapse collapsed}. More specific it is the source node of
       * the {@link yfiles.graph.FoldingEdgeStateId#masterEdge} if the source node is represented in the view and otherwise the first ancestor of the source node which is
       * represented in the view.
       * </p>
       * @type {yfiles.graph.INode}
       */
      currentMasterSource:yfiles.graph.INode;
      /**
       * The target node of the edge or an ancestor (both in the {@link yfiles.graph.FoldingManager#masterGraph}).
       * <p>
       * This may be an ancestor of the true target node if this ancestor is {@link yfiles.graph.IFoldingView#collapse collapsed}. More precisely it is the target node of
       * the {@link yfiles.graph.FoldingEdgeStateId#masterEdge} if the target node is represented in the view and otherwise the first ancestor of the target node which is
       * represented in the view.
       * </p>
       * @type {yfiles.graph.INode}
       */
      currentMasterTarget:yfiles.graph.INode;
      /**
       * Specifies whether the {@link yfiles.graph.FoldingEdgeStateId#currentMasterSource} node {@link yfiles.graph.IFoldingView#isInFoldingState is in folder state}.
       * <p>
       * Since an edge will be converted to a folding edge if the adjacent node itself will be {@link yfiles.graph.IFoldingView#collapse collapsed} to a folder node, this
       * property determines which state the source node is in for this folding edge.
       * </p>
       * @type {boolean}
       */
      sourceIsCollapsed:boolean;
      /**
       * Specifies whether the {@link yfiles.graph.FoldingEdgeStateId#currentMasterTarget} node {@link yfiles.graph.IFoldingView#isInFoldingState is in folder state}.
       * <p>
       * Since an edge will be converted to a folding edge if the adjacent node itself will be {@link yfiles.graph.IFoldingView#collapse collapsed} to a folder node, this
       * property determines which state the source node is in for this folding edge.
       * </p>
       * @type {boolean}
       */
      targetIsCollapsed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An edge label model that allows placement of labels along the edge path at a set of continuous positions at specified
     * {@link yfiles.graph.EdgePathLabelModel#sideOfEdge sides of the edge path}.
     * <p>
     * The set of positions can be influenced by specifying the {@link yfiles.graph.EdgePathLabelModel#sideOfEdge} value that controls on which side of the edge labels positions
     * should be considered. Furthermore, it's possible to specify distance values that control the distance between label and
     * edge and between label and nodes.
     * </p>
     * <p>
     * If a label model parameter is created {@link yfiles.graph.EdgeSides#ON_EDGE}, the label's center is usually placed on the edge.
     * It can be shifted by specifying a {@link yfiles.graph.EdgePathLabelModel#offset} value but the {@link yfiles.graph.EdgePathLabelModel#distance} value is ignored. For all other values of {@link yfiles.graph.EdgeSides} the
     * total distance between the label bounds and the edge path is the sum of {@link yfiles.graph.EdgePathLabelModel#offset} and {@link yfiles.graph.EdgePathLabelModel#distance}.
     * </p>
     * <p>
     * The label placements use a {@link yfiles.graph.EdgePathLabelModel#createRatioParameter ratio} value as fraction of the full edge path length. This is the main difference to the {@link yfiles.graph.EdgePathLabelModel}
     * which uses a specified segment index from the {@link yfiles.graph.EdgeSegmentLabelModel#createParameterFromSource source} or {@link yfiles.graph.EdgeSegmentLabelModel#createParameterFromTarget target} side and a ratio on this segment.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     */
    export interface EdgePathLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider,yfiles.graph.ILabelModelParameterFinder{}
    export class EdgePathLabelModel {
      /**
       * Initializes a new instance of the {@link yfiles.graph.EdgePathLabelModel} class.
       * @param distance the distance to the edge.
       * @param offset the offset to the default placement.
       * @param angle the angle of the label's rotation in radians.
       * @param autoRotationEnabled if set to <code>true</code> auto rotation is enabled.
       * @param sideOfEdge the side of the edge labels.
       */
      constructor(distance:number,offset:number,angle:number,autoRotationEnabled:boolean,sideOfEdge:yfiles.graph.EdgeSides);
      /**
       * Returns a new instance of {@link yfiles.graph.EdgePathLabelModel} class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.angle The rotation angle of all labels with this model in radians.
       * <p>
       * This option sets the {@link yfiles.graph.EdgePathLabelModel#angle} property on the created object.
       * </p>
       * @param {boolean} options.autoRotationEnabled Whether edge labels are automatically rotated according to the angle of the corresponding reference edge segment.
       * <p>
       * This option sets the {@link yfiles.graph.EdgePathLabelModel#autoRotationEnabled} property on the created object.
       * </p>
       * @param {number} options.distance The distance between the label box and the edge path.
       * <p>
       * This option sets the {@link yfiles.graph.EdgePathLabelModel#distance} property on the created object.
       * </p>
       * @param {number} options.offset Pr sets the offset of the label box and the edge path relative to the default placement.
       * <p>
       * This option sets the {@link yfiles.graph.EdgePathLabelModel#offset} property on the created object.
       * </p>
       * @param {yfiles.graph.EdgeSides} options.sideOfEdge The side placement specifiers for edge labels.
       * <p>
       * This option sets the {@link yfiles.graph.EdgePathLabelModel#sideOfEdge} property on the created object.
       * </p>
       */
      constructor(options?:{angle?:number,autoRotationEnabled?:boolean,distance?:number,offset?:number,sideOfEdge?:yfiles.graph.EdgeSides});
      /**
       * A model parameter that encodes the default position of this model's allowed edge label positions.
       * <p>
       * Returns a model parameter that encodes the default position of this model's allowed edge label positions.
       * </p>
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter at the provided edge path ratio.
       * @param edgePathRatio The ratio at which to place the label at the edge path. A ratio of 0.0 will place the label at the source side of the
       * edge path, a ratio of 1.0 at the target side.
       * @param sideOfEdge The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed.
       * @returns A label parameter at the provided edge path ratio for this model instance.
       */
      createRatioParameter(edgePathRatio:number,sideOfEdge:yfiles.graph.EdgeSides):yfiles.graph.ILabelModelParameter;
      /**
       * Returns an empty context.
       * @param label The label to use in the context.
       * @param layoutParameter The parameter to use for the label in the context.
       * @returns An empty context.
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the side placement specifiers for edge labels.
       * <p>
       * The label model parameter {@link yfiles.graph.ILabelModelParameterFinder finder} and {@link yfiles.graph.ILabelModelParameterProvider provider} returned by {@link yfiles.graph.EdgePathLabelModel#lookup} consider the specified
       * sides.
       * </p>
       * @see yfiles.graph.EdgeSides
       * @default 'OnEdge, LeftOfEdge, RightOfEdge'
       * @type {yfiles.graph.EdgeSides}
       */
      sideOfEdge:yfiles.graph.EdgeSides;
      /**
       * Gets or sets the distance between the label box and the edge path.
       * <p>
       * The interpretation of the values depends on the {@link yfiles.graph.EdgePathLabelModel#sideOfEdge} of the individual {@link yfiles.graph.ILabelModelParameter}.
       * </p>
       * <p>
       * Note that for {@link yfiles.graph.EdgeSides#ON_EDGE} this property is ignored. For all other values of {@link yfiles.graph.EdgeSides}
       * the {@link yfiles.graph.EdgePathLabelModel#offset} property is added to this distance.
       * </p>
       * @see yfiles.graph.EdgePathLabelModel#sideOfEdge
       * @see yfiles.graph.EdgeSides
       * @see yfiles.graph.EdgePathLabelModel#offset
       * @default 0.0
       * @type {number}
       */
      distance:number;
      /**
       * Gets pr sets the offset of the label box and the edge path relative to the default placement.
       * <p>
       * For {@link yfiles.graph.EdgeSides#ON_EDGE} this is the offset between the center of the label box and the edge path. For all
       * other {@link yfiles.graph.EdgeSides} values this offset is added to the {@link yfiles.graph.EdgePathLabelModel#distance} property which results in the total distance between
       * the label box and the edge path.
       * </p>
       * @see yfiles.graph.EdgePathLabelModel#sideOfEdge
       * @see yfiles.graph.EdgeSides
       * @default 0.0
       * @type {number}
       */
      offset:number;
      /**
       * Gets or sets whether edge labels are automatically rotated according to the angle of the corresponding reference edge
       * segment.
       * <p>
       * By default, this feature is enabled.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoRotationEnabled:boolean;
      /**
       * Gets or sets the rotation angle of all labels with this model in radians.
       * @default 0.0
       * @type {number}
       */
      angle:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An edge label model that allows placement of labels relative to an edge segment at a set of continuous positions at
     * specified {@link yfiles.graph.EdgeSegmentLabelModel#sideOfEdge sides of the edge path}.
     * <p>
     * The set of positions can be influenced by specifying the {@link yfiles.graph.EdgeSegmentLabelModel#sideOfEdge} value that controls on which side of the edge labels positions
     * should be considered. Furthermore, it's possible to specify distance values that control the distance between label and
     * edge and between label and nodes.
     * </p>
     * <p>
     * If a label model parameter is created {@link yfiles.graph.EdgeSides#ON_EDGE}, the label's center is usually placed on the edge.
     * It can be shifted by specifying a {@link yfiles.graph.EdgeSegmentLabelModel#offset} value but the {@link yfiles.graph.EdgeSegmentLabelModel#distance} value is ignored. For all other values of {@link yfiles.graph.EdgeSides} the
     * total distance between the label bounds and the edge path is the sum of {@link yfiles.graph.EdgeSegmentLabelModel#offset} and {@link yfiles.graph.EdgeSegmentLabelModel#distance}.
     * </p>
     * <p>
     * The label placements use a specified segment index from the {@link yfiles.graph.EdgeSegmentLabelModel#createParameterFromSource source} or {@link yfiles.graph.EdgeSegmentLabelModel#createParameterFromTarget target} side and a ratio on this segment. This is
     * the main difference to the {@link yfiles.graph.EdgePathLabelModel} which only uses a ratio value of the full edge path.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     */
    export interface EdgeSegmentLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider,yfiles.graph.ILabelModelParameterFinder{}
    export class EdgeSegmentLabelModel {
      /**
       * Initializes a new instance of the {@link yfiles.graph.EdgeSegmentLabelModel} class.
       * @param distance the distance to the edge.
       * @param offset the offset to the default placement.
       * @param angle the angle of the label's rotation in radians.
       * @param autoRotationEnabled if set to <code>true</code> auto rotation is enabled.
       * @param sideOfEdge the side of the edge labels.
       */
      constructor(distance:number,offset:number,angle:number,autoRotationEnabled:boolean,sideOfEdge:yfiles.graph.EdgeSides);
      /**
       * Returns a new instance of {@link yfiles.graph.EdgeSegmentLabelModel}.
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.angle The rotation angle of all labels with this model in radians.
       * <p>
       * This option sets the {@link yfiles.graph.EdgeSegmentLabelModel#angle} property on the created object.
       * </p>
       * @param {boolean} options.autoRotationEnabled Whether edge labels are automatically rotated according to the angle of the corresponding reference edge segment.
       * <p>
       * This option sets the {@link yfiles.graph.EdgeSegmentLabelModel#autoRotationEnabled} property on the created object.
       * </p>
       * @param {number} options.distance The distance between the label box and the edge path.
       * <p>
       * This option sets the {@link yfiles.graph.EdgeSegmentLabelModel#distance} property on the created object.
       * </p>
       * @param {number} options.offset The offset of the label box and the edge path relative to the default placement.
       * <p>
       * This option sets the {@link yfiles.graph.EdgeSegmentLabelModel#offset} property on the created object.
       * </p>
       * @param {yfiles.graph.EdgeSides} options.sideOfEdge The side placement specifiers for edge labels.
       * <p>
       * This option sets the {@link yfiles.graph.EdgeSegmentLabelModel#sideOfEdge} property on the created object.
       * </p>
       */
      constructor(options?:{angle?:number,autoRotationEnabled?:boolean,distance?:number,offset?:number,sideOfEdge?:yfiles.graph.EdgeSides});
      /**
       * A model parameter that encodes the default position of this model's allowed edge label positions.
       * <p>
       * Returns a model parameter that encodes the default position of this model's allowed edge label positions.
       * </p>
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter where the label is attached to the edge segment that contains the edge's midpoint.
       * @param segmentRatio The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the source side of the
       * segment, a ratio of 1.0 at the target side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
       * coordinates.
       * @param sideOfEdge The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed.
       * @returns A label parameter that describes the provided parameters for this model instance.
       */
      createParameterFromCenter(segmentRatio:number,sideOfEdge:yfiles.graph.EdgeSides):yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter that measures the provided segment index from the source side of the edge path.
       * @param segmentIndex The zero-based index of the segment beginning from the source side.
       * @param segmentRatio The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the source side of the
       * segment, a ratio of 1.0 at the target side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
       * coordinates.
       * @param sideOfEdge The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed.
       * @returns A label parameter that describes the provided parameters for this model instance.
       */
      createParameterFromSource(segmentIndex:number,segmentRatio:number,sideOfEdge:yfiles.graph.EdgeSides):yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter that measures the provided segment index from the target side of the edge path.
       * @param segmentIndex The zero-based index of the segment beginning from the target side.
       * @param segmentRatio The ratio at which to place the label at the segment. A ratio of 0.0 will place the label at the target side of the
       * segment, a ratio of 1.0 at the source side. Ratios &lt; 0.0 or > 1.0 will be interpreted as absolute values in world
       * coordinates.
       * @param sideOfEdge The side of the edge the label is placed on. Note that only single enum values but no combined ones are allowed.
       * @returns A label parameter that describes the provided parameters for this model instance.
       */
      createParameterFromTarget(segmentIndex:number,segmentRatio:number,sideOfEdge:yfiles.graph.EdgeSides):yfiles.graph.ILabelModelParameter;
      /**
       * Returns an empty context.
       * @param label The label to use in the context.
       * @param layoutParameter The parameter to use for the label in the context.
       * @returns An empty context.
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets the side placement specifiers for edge labels.
       * <p>
       * The label model parameter {@link yfiles.graph.ILabelModelParameterFinder finder} and {@link yfiles.graph.ILabelModelParameterProvider provider} returned by {@link yfiles.graph.EdgeSegmentLabelModel#lookup} consider the
       * specified sides.
       * </p>
       * @see yfiles.graph.EdgeSides
       * @default 'OnEdge, LeftOfEdge, RightOfEdge'
       * @type {yfiles.graph.EdgeSides}
       */
      sideOfEdge:yfiles.graph.EdgeSides;
      /**
       * Gets or sets the distance between the label box and the edge path.
       * <p>
       * The interpretation of the values depends on the {@link yfiles.graph.EdgeSegmentLabelModel#sideOfEdge} of the individual {@link yfiles.graph.ILabelModelParameter}.
       * </p>
       * <p>
       * Note that for {@link yfiles.graph.EdgeSides#ON_EDGE} this property is ignored. For all other values of {@link yfiles.graph.EdgeSides}
       * the {@link yfiles.graph.EdgeSegmentLabelModel#offset} property is added to this distance.
       * </p>
       * @see yfiles.graph.EdgeSegmentLabelModel#sideOfEdge
       * @see yfiles.graph.EdgeSides
       * @see yfiles.graph.EdgeSegmentLabelModel#offset
       * @default 0.0
       * @type {number}
       */
      distance:number;
      /**
       * Gets or sets the offset of the label box and the edge path relative to the default placement.
       * <p>
       * For {@link yfiles.graph.EdgeSides#ON_EDGE} this is the offset between the center of the label box and the edge path. For all
       * other {@link yfiles.graph.EdgeSides} values this offset is added to the {@link yfiles.graph.EdgeSegmentLabelModel#distance} property which results in the total distance between
       * the label box and the edge path.
       * </p>
       * @see yfiles.graph.EdgeSegmentLabelModel#sideOfEdge
       * @see yfiles.graph.EdgeSides
       * @default 0.0
       * @type {number}
       */
      offset:number;
      /**
       * Gets or sets whether edge labels are automatically rotated according to the angle of the corresponding reference edge
       * segment.
       * <p>
       * By default, this feature is enabled.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoRotationEnabled:boolean;
      /**
       * Gets or sets the rotation angle of all labels with this model in radians.
       * @default 0.0
       * @type {number}
       */
      angle:number;
      static $class:yfiles.lang.Class;
    }
    export enum EdgeSides{
      /**
       * The label should be placed on the path of the edge.
       */
      ON_EDGE,
      /**
       * The label should be placed on the left hand side of the edge path if looking from the source node into the direction of
       * the target node.
       */
      LEFT_OF_EDGE,
      /**
       * The label should be placed on the right hand side of the edge path if looking from the source node into the direction of
       * the target node.
       */
      RIGHT_OF_EDGE,
      /**
       * The label should be placed above the edge (in geometric sense).
       */
      ABOVE_EDGE,
      /**
       * The label should be placed below the edge (in geometric sense).
       */
      BELOW_EDGE
    }
    export enum PlaceAlongEdge{
      /**
       * The segment index is counted from the edge source.
       */
      AT_SOURCE,
      /**
       * The segment index is counted from the edge target.
       */
      AT_TARGET,
      /**
       * The label is at the center of the edge and the segment index is ignored.
       */
      AT_CENTER
    }
    export enum ExteriorLabelModelPosition{
      /**
       * Encodes a position outside the node at the top
       */
      NORTH,
      /**
       * Encodes a position outside the node at the right side
       */
      EAST,
      /**
       * Encodes a position outside the node at the bottom
       */
      SOUTH,
      /**
       * Encodes a position outside the node at the left side
       */
      WEST,
      /**
       * Encodes a position outside the node at the upper right corner
       */
      NORTH_EAST,
      /**
       * Encodes a position outside the node at the lower right corner
       */
      SOUTH_EAST,
      /**
       * Encodes a position outside the node at the upper left corner
       */
      NORTH_WEST,
      /**
       * Encodes a position outside the node at the lower left corner
       */
      SOUTH_WEST
    }
    /**
     * A label model for node labels that lie outside of the node's {@link yfiles.graph.INode#layout} bounds.
     * <p>
     * This label model supports eight positions outside of the layout's bounds.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface ExteriorLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class ExteriorLabelModel {
      /**
       * Creates a new instance of this model
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} options.insets The insets to use by this model.
       * <p>
       * This option sets the {@link yfiles.graph.ExteriorLabelModel#insets} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a new parameter that is associated with this instance for the provided position.
       * @param position The position constant that describes the position of the label.
       * @returns A new parameter instance.
       */
      createParameter(position:yfiles.graph.ExteriorLabelModelPosition):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH_EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH_WEST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static WEST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH_EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.ExteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH_WEST:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the insets to use by this model.
       * <p>
       * The insets are in fact "outsets", i.e. they are treated as margins. Positive values will increase the distance of the
       * labels to the node.
       * </p>
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.graph.IGraph} that contains a live subgraph of another {@link yfiles.graph.IGraph}.
     * <p>
     * {@link yfiles.graph.FilteredGraphWrapper} is a decorator for another {@link yfiles.graph.IGraph} instance and can be used to retain only
     * a subset of nodes and edges of the original graph. The properties {@link yfiles.graph.FilteredGraphWrapper#nodePredicate} and {@link yfiles.graph.FilteredGraphWrapper#edgePredicate} control which nodes and edges are retained in
     * the filtered graph. Edges adjacent to filtered nodes are also filtered automatically (as their source or target would
     * not be in the graph anyway).
     * </p>
     * <p>
     * Note that {@link yfiles.graph.FilteredGraphWrapper} has to listen to events raised by the wrapped graph to keep itself updated,
     * so {@link yfiles.graph.FilteredGraphWrapper#dispose} should be called if this instance is not used any more. Otherwise the
     * wrapped graph may be kept alive for longer than necessary.
     * </p>
     * <p>
     * {@link yfiles.graph.IPortOwner#ports} at {@link yfiles.graph.IEdge}s (i.e. edge-to-edge connections) are not fully supported.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: OrgChart, Demo.yFiles.Graph.OrgChart</li>
     * <li>Demo: Collapsible Tree, Demo.yFiles.Graph.Collapse</li>
     * </ul>
     * @class
     * @extends {yfiles.graph.GraphWrapperBase}
     * @final
     */
    export interface FilteredGraphWrapper extends yfiles.graph.GraphWrapperBase{}
    export class FilteredGraphWrapper {
      /**
       * Creates a new graph instance that wraps the original graph and uses the predicates to determine which nodes and edges
       * should be contained in the graph.
       * <p>
       * Note that this instance will register listeners with the <code>graph</code>, so {@link yfiles.graph.FilteredGraphWrapper#dispose} should be
       * called if this instance is not used any more.
       * </p>
       * @param graph The graph to wrap
       * @param nodePredicate The predicate that determines which nodes to keep.
       * @param edgePredicate The predicate that determines which edges to keep.
       */
      constructor(graph:yfiles.graph.IGraph,nodePredicate:(obj:yfiles.graph.INode)=>boolean,edgePredicate:(obj:yfiles.graph.IEdge)=>boolean);
      /**
       * Should be called by the client if the {@link yfiles.graph.FilteredGraphWrapper#edgePredicate} changed for the given edge in the graph.
       * @param edge
       */
      edgePredicateChanged(edge:yfiles.graph.IEdge):void;
      /**
       * Should be called by the client if the {@link yfiles.graph.FilteredGraphWrapper#edgePredicate} changed for the edge in the graph.
       */
      edgePredicateChanged():void;
      /**
       * Should be called by clients if the {@link yfiles.graph.FilteredGraphWrapper#nodePredicate} changed for the given node.
       * @param node The node.
       */
      nodePredicateChanged(node:yfiles.graph.INode):void;
      /**
       * Should be called by the client if the {@link yfiles.graph.FilteredGraphWrapper#nodePredicate} changed for the nodes in the graph.
       */
      nodePredicateChanged():void;
      /**
       * Gets the predicate function for the nodes.
       * @type {function(yfiles.graph.INode): boolean}
       */
      nodePredicate:(obj:yfiles.graph.INode)=>boolean;
      /**
       * Gets the predicate function for the edges.
       * @type {function(yfiles.graph.IEdge): boolean}
       */
      edgePredicate:(obj:yfiles.graph.IEdge)=>boolean;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.INode>}
       */
      nodes:yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IEdge>}
       */
      edges:yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
      static $class:yfiles.lang.Class;
    }
    /**
     * An edge label model that can be used to describe any position and orientation of a label.
     * <p>
     * The position of the label is stored relative to the positions of the ports of the edge the label belongs to.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     * @implements {yfiles.graph.ILabelModel}
     */
    export interface FreeEdgeLabelModel extends Object,yfiles.graph.ILabelModelParameterFinder,yfiles.graph.ILabelModel{}
    export class FreeEdgeLabelModel {
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter for the given location.
       * @param ratio The ratio at which the label will be anchored between the location of the source port and the location of the target
       * port.
       * @param distance The distance orthogonally to the connection between the source and target point.
       * @param angle The angle by which the label should be rotated.
       * @returns A parameter that describes the position.
       * @see yfiles.graph.FreeEdgeLabelModel#edgeRelativeAngle
       */
      createEdgeAnchored(ratio:number,distance:number,angle:number):yfiles.graph.ILabelModelParameter;
      /**
       * Tries to find a parameter that best matches the given layout for the provided label instance.
       * <p>
       * This implementation will match the provided layout perfectly.
       * </p>
       * @param label The label to find a parameter for.
       * @param model The model instance to use. This should be this instance.
       * @param layout The anticipated layout for the label.
       * @returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
       */
      findBestParameter(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel,layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets a property that indicates whether the angle used by the {@link yfiles.graph.FreeEdgeLabelModel#createEdgeAnchored parameter} should be interpreted relative to the
       * edge's orientation or as an absolute angle in the coordinate system.
       * <p>
       * By default the angle is interpreted to be relative to the world coordinates axes.
       * </p>
       * @default false
       * @type {boolean}
       */
      edgeRelativeAngle:boolean;
      /**
       * A shared unmodifiable public instance that can be used to obtain parameters.
       * @const
       * @static
       * @type {yfiles.graph.FreeEdgeLabelModel}
       */
      static INSTANCE:yfiles.graph.FreeEdgeLabelModel;
      static $class:yfiles.lang.Class;
    }
    /**
     * A label model that can be used to describe any position and orientation of a label.
     * <p>
     * The position of the label is stored absolutely and is not influenced by the layout or position of the owner of the
     * label. In fact the owner is not even queried during a call to {@link yfiles.graph.FreeLabelModel#getGeometry}.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     * @implements {yfiles.graph.ILabelModel}
     */
    export interface FreeLabelModel extends Object,yfiles.graph.ILabelModelParameterFinder,yfiles.graph.ILabelModel{}
    export class FreeLabelModel {
      /**
       * Creates a new label model parameter that places the label at the given absolute position using the provided angle.
       * @param anchorLocation The location for the parameter.
       * @param angle The angle of rotation.
       * @returns A new parameter that is associated with this instance.
       */
      createAbsolute(anchorLocation:yfiles.geometry.Point,angle:number):yfiles.graph.ILabelModelParameter;
      /**
       * Creates a new label at the given dynamic location using the provided angle.
       * @param location The location to place the parameter at. That instance is stored by reference.
       * @param angle The angle of rotation.
       * @returns A new parameter that is associated with this instance.
       */
      createAnchored(location:yfiles.geometry.IPoint,angle:number):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a new label model parameter that dynamically reads the label layout from the provided instance.
       * @param layout The oriented rectangle to dynamically retrieve the values from.
       * @returns A new parameter that is associated with this instance.
       */
      createDynamic(layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A shared public instance that can be used to obtain parameters.
       * @const
       * @static
       * @type {yfiles.graph.FreeLabelModel}
       */
      static INSTANCE:yfiles.graph.FreeLabelModel;
      static $class:yfiles.lang.Class;
    }
    /**
     * This node label model can be used to specify any position and orientation of a label.
     * <p>
     * The position of the label is stored relative to the layout of the node the label belongs to.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     */
    export interface FreeNodeLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterFinder{}
    export class FreeNodeLabelModel {
      /**
       * Creates a parameter instance that anchors the label to the node at the canonical position with respect to the given
       * reference point.
       * <p>
       * This method {@link yfiles.graph.FreeNodeLabelModel#createParameter creates a parameter} with the canonical settings for the ratios and offsets in the following way. Each
       * coordinate axis is considered independently. If the reference point coordinate is within the node layout interval, its
       * relative location specifies the layout ratio for this coordinate. Otherwise, the absolute distance of the reference
       * point and the interval specifies the layout offset and the layout ratio is either either <code>0.0</code> or
       * <code>1.0</code>. The parameters for the label ratio and label offset are calculated analogously. If no reference point is provided,
       * the reference point is determined automatically in the following way: For each coordinate, if the node layout and label
       * layout do not overlap, the reference is the label corner that is closest to the node. If they overlap and if in addition
       * the start or end coordinate of the label matches the start, center or end coordinate of the node, that coordinate is the
       * reference. Otherwise, the center of the label specifies the reference.
       * </p>
       * @param nodeLayout The node layout.
       * @param labelLayout The label layout.
       * @param [referenceLocation=null] The reference location.
       * @returns The canonical parameter for the specified node and label layout.
       */
      createCanonicalParameter(nodeLayout:yfiles.geometry.Rect,labelLayout:yfiles.geometry.IOrientedRectangle,referenceLocation?:yfiles.geometry.Point):yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter that is anchored at the node center and has no rotation.
       * @returns A parameter that is anchored at the node center and has no rotation.
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter instance that anchors the label to the node at a given position.
       * <p>
       * The parameter will use the ratio and offset values to determine a point on both the label and the node. It will place
       * the label in such a way that both points coincide.
       * </p>
       * @param layoutRatio The ratio that describes the point on the node's layout relative to its size.
       * @param layoutOffset The offset to apply to the point on the node after the ratio has been determined.
       * @param labelRatio The ratio that describes the point on the label's layout relative to its size.
       * @param labelOffset The offset to apply to the point on the label after the ratio has been determined.
       * @param angle The angle by which the label should be rotated around the point described using the ratio and offset, measured in
       * radians.
       * @returns A parameter that exactly fits the described location.
       */
      createParameter(layoutRatio:yfiles.geometry.Point,layoutOffset:yfiles.geometry.Point,labelRatio:yfiles.geometry.Point,labelOffset:yfiles.geometry.Point,angle:number):yfiles.graph.ILabelModelParameter;
      /**
       * Tries to find a parameter that best matches the given layout for the provided label instance.
       * <p>
       * This implementation will match the provided layout perfectly.
       * </p>
       * @param label The label to find a parameter for.
       * @param model The model instance to use. This should be this instance.
       * @param layout The anticipated layout for the label.
       * @returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
       */
      findBestParameter(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel,layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
      /**
       * Returns an empty context.
       * @param label The label to use in the context.
       * @param layoutParameter The parameter to use for the label in the context.
       * @returns An empty context.
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * Returns this instance if an instance of {@link yfiles.graph.ILabelModelParameterFinder} is requested and <code>null</code> otherwise.
       * @param type The type for which an instance is requested.
       * @returns An instance that is assignable to the given type or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A shared public instance that can be used to obtain parameters.
       * @const
       * @static
       * @type {yfiles.graph.FreeNodeLabelModel}
       */
      static INSTANCE:yfiles.graph.FreeNodeLabelModel;
      static $class:yfiles.lang.Class;
    }
    /**
     * A straightforward implementation of an {@link yfiles.graph.IPortLocationModel} for {@link yfiles.graph.IPort}s at {@link yfiles.graph.INode}s
     * that determine the location dynamically using the {@link yfiles.graph.INode#layout}.
     * <p>
     * Internally the location is stored as the ratios by which the {@link yfiles.geometry.ISize#width} and {@link yfiles.geometry.ISize#height} of the {@link yfiles.graph.INode#layout} need to be scaled and additional offsets to
     * obtain the offset to the top-left corner of the node layout.
     * </p>
     * @class
     * @implements {yfiles.graph.IPortLocationModel}
     * @final
     */
    export interface FreeNodePortLocationModel extends Object,yfiles.graph.IPortLocationModel{}
    export class FreeNodePortLocationModel {
      /**
       * 
       * @param owner
       * @param location
       * @returns 
       */
      createParameter(owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * Creates a parameter that uses the given ratios and offsets.
       * <p>
       * The <code>ratios</code> will be scaled by the node layout's size and added to the node's top left corner and the <code>offset</code> to determine
       * the port location.
       * </p>
       * @param ratios The ratios of the node layout's size.
       * @param [offset=null] The offset that is added after the <code>ratios</code> are applied.
       * @returns A new parameter that matches the specification.
       */
      createParameterForRatios(ratios:yfiles.geometry.Point,offset?:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getContext(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getLocation(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.geometry.Point;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A read only immutable instance of the {@link yfiles.graph.FreeNodePortLocationModel}.
       * @const
       * @static
       * @type {yfiles.graph.FreeNodePortLocationModel}
       */
      static INSTANCE:yfiles.graph.FreeNodePortLocationModel;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the center of the node layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0.5,0.5)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_CENTER_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the center of the left border of the
       * node layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0,0.5)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_LEFT_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the center of the right border of the
       * node layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(1,0.5)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_RIGHT_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the center of the top border of the
       * node layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0.5,0)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_TOP_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the center of the bottom border of the
       * node layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0.5,1)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_BOTTOM_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the top left corner of the node
       * layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0,0)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_TOP_LEFT_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the top right corner of the node
       * layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(1,0)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_TOP_RIGHT_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the bottom right corner of the node
       * layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(1,1)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_BOTTOM_RIGHT_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      /**
       * A read only immutable singleton instance of a parameter that locates the port at the bottom left corner of the node
       * layout.
       * <p>
       * This is the same as {@link yfiles.graph.FreeNodePortLocationModel#createParameterForRatios creating a parameter} using <code>(0,1)</code> as ratios and <code>(0,0)</code> as offsets.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      static NODE_BOTTOM_LEFT_ANCHORED:yfiles.graph.IPortLocationModelParameter;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of {@link yfiles.graph.ILabelModel} that can be used to create custom label models out of existing {@link yfiles.graph.ILabelModelParameter}
     * instances.
     * <p>
     * This class holds a number of parameters and serves as a container for these parameters. Custom models can easily be
     * created that way:
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     * @implements {yfiles.graph.ILabelCandidateDescriptorProvider}
     * @implements {yfiles.graphml.IMarkupExtensionConverter}
     */
    export interface GenericLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider,yfiles.graph.ILabelCandidateDescriptorProvider,yfiles.graphml.IMarkupExtensionConverter{}
    export class GenericLabelModel {
      /**
       * Create a new generic label model using the provided parameter as the {@link yfiles.graph.GenericLabelModel#createDefaultParameter default parameter.}
       * <p>
       * The parameter is stored by reference.
       * </p>
       * @param defaultLayoutParameter The parameter to use as the default.
       * @param [descriptor=null] The descriptor to associate with the parameter.
       * @throws {Stubs.Exceptions.ArgumentError} <code>defaultParameter</code> is null.
       * @see yfiles.graph.GenericLabelModel#getDescriptor
       */
      constructor(defaultLayoutParameter:yfiles.graph.ILabelModelParameter,descriptor?:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Adds another parameter to this models collection of parameters.
       * <p>
       * Note that the parameter will be wrapped by this class and the resulting parameter which will use this model instance as
       * its {@link yfiles.graph.ILabelModelParameter#model} will be returned. The parameter is stored by reference and subsequent modification of the parameter or its
       * associated model will be reflected by this instance. Also the methods allows for associating a {@link yfiles.graph.ILabelCandidateDescriptor}
       * with each parameter that can later be retrieved via the {@link yfiles.graph.GenericLabelModel#getDescriptor} method.
       * </p>
       * @param layoutParameter The parameter to add to this instance.
       * @param [descriptor=null] The descriptor to associate with the parameter.
       * @returns A new parameter that wraps the provided parameter and uses this instance as its model.
       */
      addParameter(layoutParameter:yfiles.graph.ILabelModelParameter,descriptor?:yfiles.graph.ILabelCandidateDescriptor):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * Yields the {@link yfiles.graph.ILabelCandidateDescriptor} that has been associated with the instance during
       * {@link yfiles.graph.GenericLabelModel#addParameter}.
       * @param label The label instance to use.
       * @param layoutParameter The parameter to get the descriptor for.
       * @returns The descriptor instance that had been added to this instance during {@link #addParameter}.
       */
      getDescriptor(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILabelCandidateDescriptor;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of {@link yfiles.graph.IPortLocationModel} that can be used to create custom port location models out of
     * existing {@link yfiles.graph.IPortLocationModelParameter} instances.
     * <p>
     * This class holds a number of parameters and serves as a container for these parameters.
     * </p>
     * <p>
     * Custom models can easily be created in the following way:
     * </p>
     * <p>
     * Note that in order to work, this model needs to have at least one {@link yfiles.graph.GenericPortLocationModel#addParameter parameter added}.
     * </p>
     * @see yfiles.graph.GenericPortLocationModel#addParameter
     * @class
     * @implements {yfiles.graph.IPortLocationModel}
     * @implements {yfiles.collections.IEnumerable.<yfiles.graph.IPortLocationModelParameter>}
     * @implements {yfiles.graphml.IMarkupExtensionConverter}
     * @final
     */
    export interface GenericPortLocationModel extends Object,yfiles.graph.IPortLocationModel,yfiles.collections.IEnumerable<yfiles.graph.IPortLocationModelParameter>,yfiles.graphml.IMarkupExtensionConverter{}
    export class GenericPortLocationModel {
      /**
       * Initializes a new instance of the {@link yfiles.graph.GenericPortLocationModel} class.
       */
      constructor();
      /**
       * This method simply delegates to {@link yfiles.graph.GenericPortLocationModel#addParameter}.
       * <p>
       * It is there to allow for convenient initialization in code.
       * </p>
       * <p>
       * This method is used by the compiler if the collection initializer syntax is used.
       * </p>
       * @param locationParameter The parameter to add to this instance.
       * @see yfiles.graph.GenericPortLocationModel#addParameter
       */
      add(locationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * Adds the given parameter to the {@link yfiles.graph.GenericPortLocationModel#locationParameters} of this instance.
       * <p>
       * Note that this will create a new parameter instance that is then actually bound to this instance and returned. The
       * instance is stored by reference so if this is a mutable instance this instance will be modified, implicitly, too.
       * </p>
       * @param locationParameter The parameter to add to this instance.
       * @returns The parameter to use with this model that wraps the given parameter.
       * @see yfiles.graph.GenericPortLocationModel#add
       */
      addParameter(locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.graph.IPortLocationModelParameter;
      /**
       * Factory method that creates a parameter for the given port that tries to match the provided location in absolute world
       * coordinates.
       * <p>
       * This implementation iterates over all {@link yfiles.graph.GenericPortLocationModel#locationParameters} to return the one that matches the given location the best.
       * </p>
       * @param owner The port owner that will own the port for which the parameter shall be created.
       * @param location The location in the world coordinate system that should be matched as best as possible.
       * @returns A new instance that can be used to describe the location of an {@link } at the given <code>owner</code>.
       */
      createParameter(owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getContext(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getLocation(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.geometry.Point;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets all parameters that are associated with this instance.
       * <p>
       * This enumerates the instances that wraps the parameters that have been added to this instance via
       * {@link yfiles.graph.GenericPortLocationModel#addParameter}
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IPortLocationModelParameter>}
       */
      locationParameters:yfiles.collections.IEnumerable<yfiles.graph.IPortLocationModelParameter>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This interface can be used to customize the clipboard behavior for individual {@link yfiles.graph.IModelItem}s.
     * <p>
     * Implementations of this interface can {@link yfiles.graph.ILookupDecorator#addLookup decorate} the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.IModelItem}s to
     * change their behavior during {@link yfiles.graph.GraphClipboard clipboard} actions.
     * </p>
     * <p>
     * The easiest way to decorate certain model items with a custom {@link yfiles.graph.IClipboardHelper} implementation is via the
     * various decorators available on {@link yfiles.graph.GraphDecorator}:
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Clipboard, Demo.yFiles.Graph.Clipboard</li>
     * </ul>
     * @interface
     */
    export interface IClipboardHelper extends Object{
      /**
       * This method is called during the copy operation of the given item to retrieve additional state that can be later used
       * during a {@link yfiles.graph.IClipboardHelper#paste} operation.
       * <p>
       * If the source graph is a managed folding view, the <code>item</code> is always a master item.
       * </p>
       * @param context The context in which this interface is used, can be <code>null</code>
       * @param item The item to copy to the clipboard.
       * @returns A custom user object that will later be provided to {@link #paste}.
       * @abstract
       */
      copy(context:yfiles.graph.IGraphClipboardContext,item:yfiles.graph.IModelItem):Object;
      /**
       * This method is called during the cut operation of the given item to retrieve additional state that can be later used
       * during a {@link yfiles.graph.IClipboardHelper#paste} operation.
       * <p>
       * If the source graph is a managed folding view, the <code>item</code> is always a master item.
       * </p>
       * @param context The context in which this interface is used, can be <code>null</code>
       * @param item The item to cut to the clipboard.
       * @returns A custom user object that will later be provided to {@link #paste}.
       * @abstract
       */
      cut(context:yfiles.graph.IGraphClipboardContext,item:yfiles.graph.IModelItem):Object;
      /**
       * This method is called after the item has been pasted from the clipboard.
       * <p>
       * The <code>item</code> is always a master item since the clipboard graph is not a managed folding view.
       * </p>
       * @param context The context in which this interface is used, can be <code>null</code>
       * @param item The item that has been created during the paste operation.
       * @param userData The data that had been queried during {@link #cut} or {@link #copy}
       * respectively.
       * @abstract
       */
      paste(context:yfiles.graph.IGraphClipboardContext,item:yfiles.graph.IModelItem,userData:Object):void;
      /**
       * Determines whether the given item can or should be copied to the clipboard.
       * <p>
       * If the source graph is a managed folding view, the <code>item</code> is always a view item. Furthermore, if collapsed nodes are to be
       * copied, this method will be queried with the collapsed group node as <code>item</code> for each "hidden" node that is currently not
       * visible.
       * </p>
       * @param context The context in which this interface is used, can be <code>null</code>
       * @param item The item in question.
       * @returns Whether this item should be copied.
       * @abstract
       */
      shouldCopy(context:yfiles.graph.IGraphClipboardContext,item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether the given item can or should be cut to the clipboard.
       * <p>
       * If the source graph is a managed folding view, the <code>item</code> is always a view item. Furthermore, if collapsed nodes are to be
       * cut, this method will be queried with the collapsed group node as <code>item</code> for each "hidden" node that is currently not
       * visible.
       * </p>
       * @param context The context in which this interface is used, can be <code>null</code>
       * @param item The item in question.
       * @returns Whether this item should be cut.
       * @abstract
       */
      shouldCut(context:yfiles.graph.IGraphClipboardContext,item:yfiles.graph.IModelItem):boolean;
      /**
       * Determines whether the given item can or should be pasted from the clipboard to the target graph.
       * <p>
       * The <code>item</code> is always a master item since the clipboard graph is not a managed folding view.
       * </p>
       * @param context The context in which this interface is used, can be <code>null</code>
       * @param item The item in the clipboard graph.
       * @param userData The state memento that had been created during {@link #cut} or {@link #copy}.
       * @returns Whether this item should be pasted.
       * @abstract
       */
      shouldPaste(context:yfiles.graph.IGraphClipboardContext,item:yfiles.graph.IModelItem,userData:Object):boolean;
    }
    var IClipboardHelper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface for the context that is passed by the {@link yfiles.graph.GraphClipboard} implementation to the {@link yfiles.graph.IClipboardHelper}
     * methods and {@link yfiles.graph.IClipboardIdProvider}.
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface IGraphClipboardContext extends Object,yfiles.graph.ILookup{
      /**
       * Gets the clipboard that created this context and is used during the current operation.
       * @abstract
       * @type {yfiles.graph.GraphClipboard}
       */
      clipboard:yfiles.graph.GraphClipboard;
      /**
       * Gets the source graph for the current operation.
       * @abstract
       * @type {yfiles.graph.IGraph}
       */
      sourceGraph:yfiles.graph.IGraph;
      /**
       * Gets the target graph for the current operation.
       * @abstract
       * @type {yfiles.graph.IGraph}
       */
      targetGraph:yfiles.graph.IGraph;
    }
    var IGraphClipboardContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Used by the {@link yfiles.graph.GraphClipboard} to assign IDs for elements in the clipboard.
     * <p>
     * IDs are created when items are copied (or cut) into the clipboard and queried when the items are pasted from the
     * clipboard.
     * <br />
     * Providing a custom ID provider allows for identifying conceptually identical items across different graph instances.
     * <br />
     * The default implementation uses the {@link yfiles.graph.IModelItem} object reference as the ID.
     * </p>
     * @interface
     */
    export interface IClipboardIdProvider extends Object{
      /**
       * Gets the ID for the given <code>originalItem</code>.
       * @param context The clipboard context
       * @param originalItem The original item which is copied. The item is part of the original graph from which the item is copied.
       * @returns An ID for the given item. The object should be suitable for use in a {@link }.
       * @abstract
       */
      getId(context:yfiles.graph.IGraphClipboardContext,originalItem:yfiles.graph.IModelItem):Object;
      /**
       * Gets the item which is represented by the given <code>id</code>.
       * @param context The clipboard context.
       * @param id The ID to get the item for.
       * @returns The item which is represented by the given ID or <code>null</code> if the item cannot be located or is invalid. The item needs to be
       * part of the target graph for paste operations.
       * @abstract
       */
      getItem(context:yfiles.graph.IGraphClipboardContext,id:Object):yfiles.graph.IModelItem;
    }
    var IClipboardIdProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum ParentNodeDetectionModes{
      /**
       * Nodes should always be pasted into the root group.
       */
      ROOT,
      /**
       * If exactly one group node is selected the nodes should be pasted into that group.
       * <p>
       * As long as {@link yfiles.graph.ParentNodeDetectionModes#ALLOW_SELF} is not set, too, a node will not be pasted into its original
       * node. Note that it can be pasted into a copy of its original, though.
       * </p>
       * <p>
       * If this setting is combined with {@link yfiles.graph.ParentNodeDetectionModes#ROOT} or {@link yfiles.graph.ParentNodeDetectionModes#PREVIOUS_PARENT}
       * it will override that setting if a group node is selected.
       * </p>
       */
      SELECTION,
      /**
       * A node should be pasted into its original parent.
       * <p>
       * If the parent has been removed from the graph in between the copy and paste operation the node will be placed into the
       * parent's parent.
       * </p>
       * <p>
       * If the previous parent has been moved too far away the node will be placed into the root unless {@link yfiles.graph.ParentNodeDetectionModes#FORCE}
       * is set, too.
       * </p>
       * <p>
       * If this setting is combined with {@link yfiles.graph.ParentNodeDetectionModes#SELECTION} or {@link yfiles.graph.ParentNodeDetectionModes#AT_LOCATION}
       * it will be overridden by the other setting.
       * </p>
       */
      PREVIOUS_PARENT,
      /**
       * A node should be pasted into the group node at the location where the node is pasted to or the root if there is no group
       * node.
       * <p>
       * As long as {@link yfiles.graph.ParentNodeDetectionModes#ALLOW_SELF} is not set, too, the node will not be pasted into its
       * original node. Note that it can be pasted into a copy of its original, though.
       * </p>
       */
      AT_LOCATION,
      /**
       * When used in combination with {@link yfiles.graph.ParentNodeDetectionModes#SELECTION} or {@link yfiles.graph.ParentNodeDetectionModes#AT_LOCATION}
       * this setting will allow pasting nodes into their original.
       */
      ALLOW_SELF,
      /**
       * If combined with {@link yfiles.graph.ParentNodeDetectionModes#PREVIOUS_PARENT} forces the node to be pasted into its original
       * parent, even if the parent has moved far away and pasting the node would increase the parents size considerably.
       */
      FORCE,
      /**
       * A bitmask that can be used to query the actual mode ({@link yfiles.graph.ParentNodeDetectionModes#ROOT},
       * {@link yfiles.graph.ParentNodeDetectionModes#SELECTION}, {@link yfiles.graph.ParentNodeDetectionModes#AT_LOCATION}, or
       * {@link yfiles.graph.ParentNodeDetectionModes#PREVIOUS_PARENT}).
       */
      MODE_MASK,
      /**
       * A bitmask that can be used to query modifications of the actual mode such as {@link yfiles.graph.ParentNodeDetectionModes#ALLOW_SELF}
       * or {@link yfiles.graph.ParentNodeDetectionModes#FORCE}.
       */
      MODIFIER_MASK
    }
    /**
     * Clipboard implementation for {@link yfiles.graph.IGraph} instances.
     * <p>
     * This implementation copies a given graph to an internally held
     * {@link yfiles.graph.GraphClipboard#clipboardGraph clipboard graph}. The clipboard graph can then be pasted to a target graph instance.
     * </p>
     * <p>
     * The clipboard can be configured to copy and paste {@link yfiles.graph.GraphClipboard#independentCopyItems items that depend on other items} without copying the items it depends
     * on, e.g. labels, ports, edges, and bends.
     * </p>
     * <p>
     * Also depending on the selection in the target graph, items can be pasted at multiple locations at once, e.g. a single
     * label can be copied to the clipboard independently and pasted onto several selected target items.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Simple Editor, Demo.yFiles.Graph.SimpleEditor</li>
     * </ul>
     * @see yfiles.graph.IClipboardHelper
     * @see yfiles.graph.GraphClipboard#cut
     * @see yfiles.graph.GraphClipboard#onCopy
     * @see yfiles.graph.GraphClipboard#paste
     * @see yfiles.graph.GraphClipboard#clipboardGraph
     * @class
     */
    export interface GraphClipboard extends Object{}
    export class GraphClipboard {
      /**
       * Creates a new initially empty clipboard.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Point} options.pasteDelta A vector that will be used to move nodes and bends by after they have been pasted to the target graph.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#pasteDelta} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphCopier} options.toClipboardCopier The {@link yfiles.graph.GraphCopier} instance that will be used to copy the graph to the clipboard.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#toClipboardCopier} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphCopier} options.fromClipboardCopier The {@link yfiles.graph.GraphCopier} instance that will be used to copy the graph from the clipboard to the target graph.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#fromClipboardCopier} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphCopier} options.duplicateCopier The {@link yfiles.graph.GraphCopier} instance that will be used to duplicate elements inside the graph.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#duplicateCopier} property on the created object.
       * </p>
       * @param {yfiles.graph.IClipboardIdProvider} options.idProvider The {@link yfiles.graph.IClipboardIdProvider} used by this instance to provide IDs for copied items.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#idProvider} property on the created object.
       * </p>
       * @param {yfiles.graph.IGraph} options.clipboardGraph The graph that contains the clipboard's contents.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#clipboardGraph} property on the created object.
       * </p>
       * @param {yfiles.graph.ParentNodeDetectionModes} options.parentNodeDetection Whether or not nodes should be automatically put into a group node and if so how this group will be determined.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#parentNodeDetection} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphItemTypes} options.dependentCopyItems The type of items that depend on other elements but should automatically be copied implicitly to the clipboard if all elements that it depends on are explicitly selected for copying.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#dependentCopyItems} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphItemTypes} options.independentCopyItems The type of items that can be copied and pasted independently of their owning element.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#independentCopyItems} property on the created object.
       * </p>
       * @param {yfiles.graph.GraphItemTypes} options.copyItems The type of items that can be copied and pasted at all using the clipboard.
       * <p>
       * This option sets the {@link yfiles.graph.GraphClipboard#copyItems} property on the created object.
       * </p>
       */
      constructor(options?:{pasteDelta?:yfiles.geometry.Point,toClipboardCopier?:yfiles.graph.GraphCopier,fromClipboardCopier?:yfiles.graph.GraphCopier,duplicateCopier?:yfiles.graph.GraphCopier,idProvider?:yfiles.graph.IClipboardIdProvider,clipboardGraph?:yfiles.graph.IGraph,parentNodeDetection?:yfiles.graph.ParentNodeDetectionModes,dependentCopyItems?:yfiles.graph.GraphItemTypes,independentCopyItems?:yfiles.graph.GraphItemTypes,copyItems?:yfiles.graph.GraphItemTypes});
      /**
       * Clears the clipboard's contents.
       */
      clear():void;
      /**
       * Copies the elements indicated by the filter to the clipboard graph.
       * <p>
       * For each element copied to the clipboard graph the {@link yfiles.graph.GraphClipboard#onElementCopied} delegate will be called.
       * The actual work of this method is delegated to the {@link yfiles.graph.GraphClipboard#toClipboardCopier}'s {@link yfiles.graph.GraphCopier#copy} method.
       * </p>
       * @param sourceGraph The graph to copy the contents from.
       * @param filter The filter to apply to the source graph's elements.
       */
      copy(sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean):void;
      /**
       * Creates the graph used for the clipboard.
       * @returns A new readily configured {@link } instance.
       * @protected
       */
      createClipboardGraph():yfiles.graph.IGraph;
      /**
       * Convenience method that creates a predicate that is based upon the outcome of the {@link yfiles.graph.IClipboardHelper#shouldCopy}
       * method that can be found in the {@link yfiles.graph.ILookup#lookup} of the provided {@link yfiles.graph.IModelItem}.
       * @returns A predicate that uses {@link #shouldCopy}.
       * @see yfiles.graph.GraphClipboard#onCopy
       */
      createClipBoardHelperCopyFilter():(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Convenience method that creates a predicate that is based upon the outcome of the {@link yfiles.graph.IClipboardHelper#shouldCut}
       * method that can be found in the {@link yfiles.graph.ILookup#lookup} of the provided {@link yfiles.graph.IModelItem}.
       * @returns A predicate that uses {@link #shouldCut}.
       * @see yfiles.graph.GraphClipboard#cut
       */
      createClipBoardHelperCutFilter():(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Convenience method that creates a predicate that is based upon the outcome of the {@link yfiles.graph.IClipboardHelper#shouldPaste}
       * method that can be found in the {@link yfiles.graph.ILookup#lookup} of the provided {@link yfiles.graph.IModelItem}.
       * @returns A predicate that uses {@link #shouldPaste}.
       * @see yfiles.graph.GraphClipboard#cut
       */
      createClipBoardHelperPasteFilter():(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Factory method that creates the context for the upcoming calls to the {@link yfiles.graph.IClipboardHelper} methods.
       * @param sourceGraph The source graph of the operation.
       * @param targetGraph The target graph of the operation.
       * @returns A context that has the properties configured according to the parameters.
       * @protected
       */
      createContext(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph):yfiles.graph.IGraphClipboardContext;
      /**
       * Creates the default {@link yfiles.graph.IClipboardIdProvider} used by this instance to provide IDs for copied items.
       * <p>
       * The default implementation simply uses a reference to the original item as ID.
       * </p>
       * <p>
       * Custom ID can be used to map between elements in different graphs.
       * </p>
       * @returns 
       * @protected
       */
      createDefaultClipboardIdProvider():yfiles.graph.IClipboardIdProvider;
      /**
       * Creates a predicate for use in the {@link yfiles.graph.GraphClipboard#onCopy} method.
       * <p>
       * This filter uses two predicates to decide whether a graph item can be copied. The base set of items specified by this
       * filter is induced by the {@link yfiles.graph.GraphClipboard#createSelectionFilter}. Optionally, this set can be restricted
       * further by an {@link yfiles.graph.IClipboardHelper#shouldCopy} predicate. If such a predicate returns <code>false</code> for the item itself
       * or any item this item depends on, then the item will not be copied. Items without such a predicate are handled solely
       * according to their selection state.
       * </p>
       * @param selection The selection model to base the filter on.
       * @param graph The graph to use for determining adjacency.
       * @returns A predicate that filters subgraph induced by the selection of nodes and restricted by an optional {@link #shouldCopy}
       * predicate.
       */
      createDefaultCopyFilter(selection:yfiles.view.IGraphSelection,graph:yfiles.graph.IGraph):(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Creates a predicate for use in the {@link yfiles.graph.GraphClipboard#cut} method.
       * <p>
       * This filter uses two predicates to decide whether a graph item can be cut. The base set of items specified by this
       * filter is induced by the {@link yfiles.graph.GraphClipboard#createSelectionFilter}. Optionally, this set can be restricted
       * further by an {@link yfiles.graph.IClipboardHelper#shouldCut} predicate. If such a predicate returns <code>false</code> for the item itself
       * or any item this item depends on, then the item will not be cut. Items without such a predicate are handled solely
       * according to their selection state.
       * </p>
       * @param selection The selection model to base the filter on.
       * @param graph The graph to use for determining adjacency.
       * @returns A predicate that filters subgraph induced by the selection of nodes and restricted by an optional {@link #shouldCut}
       * predicate.
       */
      createDefaultCutFilter(selection:yfiles.view.IGraphSelection,graph:yfiles.graph.IGraph):(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Creates a predicate for use in the {@link yfiles.graph.GraphClipboard#duplicate} method.
       * <p>
       * By default this method uses the same filter implementation as {@link yfiles.graph.GraphClipboard#createDefaultCopyFilter}: This
       * filter uses two predicates to decide whether a graph item will be duplicated. The base set of items specified by this
       * filter is induced by the {@link yfiles.graph.GraphClipboard#createSelectionFilter}. Optionally, this set can be restricted
       * further by an {@link yfiles.graph.IClipboardHelper#shouldCopy} predicate. If such a predicate returns <code>false</code> for the item itself
       * or any item this item depends on, then the item will not be copied. Items without such a predicate are handled solely
       * according to their selection state.
       * </p>
       * @param selection The selection model to base the filter on.
       * @param graph The graph to use for determining adjacency.
       * @returns A predicate that filters subgraph induced by the selection of nodes and restricted by an optional {@link #shouldCopy}
       * predicate.
       */
      createDefaultDuplicateFilter(selection:yfiles.view.IGraphSelection,graph:yfiles.graph.IGraph):(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Factory method that creates the {@link yfiles.graph.GraphClipboard#duplicateCopier}.
       * @returns A new {@link } instance.
       * @protected
       */
      createDuplicateCopier():yfiles.graph.GraphCopier;
      /**
       * Factory method that creates the {@link yfiles.graph.GraphClipboard#fromClipboardCopier}.
       * @returns A new {@link } instance.
       * @protected
       */
      createFromClipboardCopier():yfiles.graph.GraphCopier;
      /**
       * Creates a predicate for use in the {@link yfiles.graph.GraphClipboard#cut} and {@link yfiles.graph.GraphClipboard#onCopy} methods.
       * <p>
       * This predicate is based on the element induced by the set of selected nodes, e.g. each edge that has both of its
       * adjacent port owners selected will be included as well as each label and port whose owner is included. Also ports where
       * no adjacent edge is included (but edges exist), will not be included, unless the {@link yfiles.graph.IPortDefaults#autoCleanup} is set to <code>false</code>.
       * </p>
       * @param selection The selection model to base the filter on.
       * @param graph The graph to use for determining adjacency.
       * @returns A predicate that filters subgraph induced by the selection of nodes.
       */
      createSelectionFilter(selection:yfiles.view.IGraphSelection,graph:yfiles.graph.IGraph):(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Factory method that creates the {@link yfiles.graph.GraphClipboard#toClipboardCopier}.
       * @returns A new {@link } instance.
       * @protected
       */
      createToClipboardCopier():yfiles.graph.GraphCopier;
      /**
       * Creates an induced predicate from <code>coreFilter</code> for use in the {@link yfiles.graph.GraphClipboard#cut} and {@link yfiles.graph.GraphClipboard#onCopy}
       * methods that takes dependent items into account.
       * <p>
       * This method can create a predicate for only a core set of items which is then extended or restricted depending on the
       * predicate values of graph item dependencies.
       * </p>
       * <p>
       * The <code>restrictive</code> parameter determines whether the result restricts or extends the set of elements handled by <code>coreFilter</code>:
       * </p>
       * <ul>
       * <li>
       * If <code>restrictive</code> is <code>false</code>, an item is allowed if either <code>coreFilter</code> returns <code>true</code> for itself or if all items it depends on
       * (e.g. label/port owner, source/target port etc.) are allowed by <code>coreFilter</code>.
       * </li>
       * <li>
       * If <code>restrictive</code> is <code>true</code>, an item is forbidden if either <code>coreFilter</code> returns <code>false</code> for itself or if any item it depends on
       * (e.g. label/port owner, source/target port etc.) is forbidden by <code>coreFilter</code>.
       * </li>
       * </ul>
       * @param coreFilter The core predicate for the base set of elements.
       * @param graph The graph to use for determining adjacency.
       * @param restrictive Whether to create a restriction or extension for the <code>coreFilter</code> predicate
       * @returns 
       * @static
       */
      static createWrappedFilter(coreFilter:(obj:yfiles.graph.IModelItem)=>boolean,graph:yfiles.graph.IGraph,restrictive:boolean):(obj:yfiles.graph.IModelItem)=>boolean;
      /**
       * Cuts the items indicated by <code>filter</code> to the clipboard and removes them from the graph.
       * <p>
       * This method delegates the core work to {@link yfiles.graph.GraphClipboard#onCopy} which will use the {@link yfiles.graph.GraphClipboard#toClipboardCopier} to copy the cut elements
       * to the clipboard.
       * </p>
       * @param sourceGraph The graph to cut the items from.
       * @param filter The filter to use.
       * @see yfiles.graph.GraphClipboard#createSelectionFilter
       */
      cut(sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean):void;
      /**
       * Duplicates the elements indicated by the filter.
       * <p>
       * The {@link yfiles.graph.GraphClipboard#clipboardGraph} is not altered by this operation.
       * </p>
       * @param context The current {@link }.
       * @param sourceGraph The graph whose elements should be duplicated.
       * @param filter A predicate which returns <code>true</code> for elements which should be duplicated.
       * @param elementDuplicated An element copied callback which gets invoked for each element which has been duplicated. The two parameters passed into
       * the callback are guaranteed to be part of the <code>sourceGraph</code>.
       */
      duplicate(context:yfiles.input.IInputModeContext,sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean,elementDuplicated:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void):void;
      /**
       * Retrieves the {@link yfiles.graph.IClipboardHelper} instance associated with the given item.
       * <p>
       * This implementation uses the item's {@link yfiles.graph.ILookup#lookup} method to retrieve the helper instance.
       * </p>
       * @param item The item to find the clipboard helper for.
       * @returns An instance or <code>null</code>.
       * @protected
       */
      getClipboardHelper(item:yfiles.graph.IModelItem):yfiles.graph.IClipboardHelper;
      /**
       * Gets a folding state of an edge for a given {@link yfiles.graph.FoldingEdgeStateId}.
       * <p>
       * This method might be overridden to map folding states for edges whose source or target node's hierarchy might have
       * changed during a paste operation.
       * </p>
       * @param edge The edge to get the folding state for.
       * @param id The original {@link } to get the folding state for. Passing <code>null</code> will return the folding state
       * which was visible when the edge was copied into the clipboard.
       * @returns The folding state which is mapped to the given <code>id</code>. May be <code>null</code>.
       * @protected
       */
      getFoldingEdgeState(edge:yfiles.graph.IEdge,id:yfiles.graph.FoldingEdgeStateId):yfiles.graph.IEdge;
      /**
       * Gets a {@link yfiles.graph.ILabelModelParameter} for the given <code>label</code> to be placed at the given <code>newOwner</code>.
       * <p>
       * Implementations might delegate to {@link yfiles.graph.GraphCopier#copyLabelLayoutParameter} to copy the existing parameter or
       * use the {@link yfiles.graph.IGraph#nodeDefaults} or {@link yfiles.graph.IGraph#edgeDefaults} to create a new one. Note that the type of the owner might have changed during the copy operation.
       * </p>
       * @param copier The {@link } whose {@link #copyLabelLayoutParameter} method can be used to copy the
       * existing parameter.
       * @param graph The {@link } whose {@link #nodeDefaults} or {@link #edgeDefaults} can be used to create a new parameter.
       * @param label The label to be copied.
       * @param newOwner The new owner for the label.
       * @returns An {@link } for the copied label.
       * @protected
       */
      getLabelLayoutParameter(copier:yfiles.graph.GraphCopier,graph:yfiles.graph.IGraph,label:yfiles.graph.ILabel,newOwner:yfiles.graph.ILabelOwner):yfiles.graph.ILabelModelParameter;
      /**
       * Retrieves the memento associated with an item in the clipboard graph that has been stored with the item during {@link yfiles.graph.GraphClipboard#onElementCopied}
       * and {@link yfiles.graph.GraphClipboard#onElementCut}.
       * @param clipboardItem The item in the clipboard graph.
       * @returns The object returned by the {@link } methods or <code>null</code>.
       */
      getMemento(clipboardItem:yfiles.graph.IModelItem):Object;
      /**
       * Gets a {@link yfiles.graph.IPortLocationModelParameter} for the given <code>originalPort</code> to be placed at the given <code>newOwner</code>.
       * <p>
       * Implementations might delegate to {@link yfiles.graph.GraphCopier#copyPortLocationParameter} to copy the existing parameter or
       * use the {@link yfiles.graph.IGraph#nodeDefaults} or {@link yfiles.graph.IGraph#edgeDefaults} to create a new one. Note that the type of the owner might have changed during the copy operation.
       * </p>
       * @param copier The {@link } whose {@link #copyPortLocationParameter} method can be used to copy the
       * existing parameter.
       * @param graph The {@link } whose {@link #nodeDefaults} or {@link #edgeDefaults} can be used to create a new parameter.
       * @param originalPort The port to be copied.
       * @param newOwner The new owner for the port.
       * @returns An {@link } for the copied port.
       * @protected
       */
      getPortLocationParameter(copier:yfiles.graph.GraphCopier,graph:yfiles.graph.IGraph,originalPort:yfiles.graph.IPort,newOwner:yfiles.graph.IPortOwner):yfiles.graph.IPortLocationModelParameter;
      /**
       * Provides an {@link yfiles.graph.IEdge} where the given <code>originalBend</code> can be added to.
       * <p>
       * Called for {@link yfiles.graph.IBend}s whose edge is not copied to provide a new owner. Might return <code>null</code> if no target for the
       * given <code>originalBend</code> is found. In this case the bend will not be copied.
       * </p>
       * @param originalBend The bend to be copied.
       * @returns An {@link } for the bend to be added to or <code>null</code> if the bend should not be copied.
       * @protected
       */
      getTargetEdge(originalBend:yfiles.graph.IBend):yfiles.graph.IEdge;
      /**
       * Provides an {@link yfiles.graph.ILabelOwner} where the given <code>originalLabel</code> can be added to.
       * <p>
       * Called for {@link yfiles.graph.ILabel}s whose owners are not copied to provide a new owner. Might return <code>null</code> if no target for
       * the given <code>originalLabel</code> is found. In this case the label will not be copied.
       * </p>
       * @param originalLabel The label to be copied.
       * @returns An {@link } for the label to be added to or <code>null</code> if the label should not be copied.
       * @protected
       */
      getTargetLabeledItem(originalLabel:yfiles.graph.ILabel):yfiles.graph.ILabelOwner;
      /**
       * Provides a parent group node for the given <code>originalNode</code>.
       * <p>
       * Called for {@link yfiles.graph.INode}s to place them inside a group node. Might return <code>null</code> if the given node should be placed
       * at the root or if the graph is not grouped.
       * </p>
       * @param originalNode The node to be copied.
       * @returns The new parent group of the node or <code>null</code> if the node should be placed at the root of the hierarchy or if the graph is
       * not grouped.
       * @see yfiles.graph.GraphClipboard#parentNodeDetection
       * @protected
       */
      getTargetNode(originalNode:yfiles.graph.INode):yfiles.graph.INode;
      /**
       * Provides an {@link yfiles.graph.IPort} where the given <code>originalEdge</code> can be added to.
       * <p>
       * Called for {@link yfiles.graph.IEdge}s whose source or target ports are not copied to provide a new port. Might return <code>null</code> if
       * no target for the given <code>originalEdge</code> is found. In this case the edge will not be copied.
       * </p>
       * @param originalEdge The edge to be copied.
       * @param atSource Whether to look for a source port.
       * @returns An {@link } for the edge to be added to or <code>null</code> if the edge should not be copied.
       * @protected
       */
      getTargetPort(originalEdge:yfiles.graph.IEdge,atSource:boolean):yfiles.graph.IPort;
      /**
       * Provides an {@link yfiles.graph.IPortOwner} where the given <code>originalPort</code> can be added to.
       * <p>
       * Called for {@link yfiles.graph.IPort}s whose owners are not copied to provide a new owner. Might return <code>null</code> if no target for
       * the given <code>originalPort</code> is found. In this case the port will not be copied.
       * </p>
       * @param originalPort The port to be copied.
       * @returns An {@link } for the port to be added to or <code>null</code> if the port should not be copied.
       * @protected
       */
      getTargetPortOwner(originalPort:yfiles.graph.IPort):yfiles.graph.IPortOwner;
      /**
       * Whether the given <code>item</code> is a "dummy" item which should not be pasted.
       * <p>
       * A dummy item is used to keep {@link yfiles.graph.IModelItem}s which cannot exist without an owner in the {@link yfiles.graph.GraphClipboard#clipboardGraph} (e.g.
       * {@link yfiles.graph.ILabel}s).
       * </p>
       * @param item The item to test.
       * @returns <code>true</code> if the item is a "dummy" item.
       */
      isDummy(item:yfiles.graph.IModelItem):boolean;
      /**
       * Whether the given <code>item</code> exists in a folding state of its <code>owner</code>.
       * @param owner The owner of the given item.
       * @param item An {@link }, {@link }, or {@link }.
       * @returns <code>true</code> if the given item exists in a folding state.
       * @protected
       * @static
       */
      static isFoldingStateElement(owner:yfiles.graph.IModelItem,item:yfiles.graph.IModelItem):boolean;
      /**
       * Calls {@link yfiles.graph.GraphCopier#copy} on the current {@link yfiles.graph.GraphClipboard#toClipboardCopier} instance or {@link yfiles.graph.GraphClipboard#fromClipboardCopier} instance, depending on whether the <code>targetGraph</code> is the {@link yfiles.graph.GraphClipboard#clipboardGraph}
       * instance.
       * @param sourceGraph The source graph to pass to {@link #copy}
       * @param filter The predicate to pass on.
       * @param targetGraph The target graph to pass on.
       * @param targetRootNode The node to use as the root node in the grouped <code>targetGraph</code>.
       * @param elementCopiedCallback The delegate to pass on. For each element that is copied during the operation to the clipboard, this callback will be
       * passed the original and the copy of the item. In case the <code>sourceGraph</code> is a {@link #views folding view graph} both elements are passed
       * as {@link #masterGraph master} elements or {@link #getFolderNodeState folding state elements} if a master element does not exist.
       * @protected
       */
      onCopy(sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean,targetGraph:yfiles.graph.IGraph,targetRootNode:yfiles.graph.INode,elementCopiedCallback:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void):void;
      /**
       * A callback that will be invoked for each item that has been copied into the clipboard graph in response to a {@link yfiles.graph.GraphClipboard#onCopy}
       * operation.
       * <p>
       * This implementation will {@link yfiles.graph.ILookup#lookup} a {@link yfiles.graph.IClipboardHelper} for each item in the original graph
       * and remember the memento retrieved by the {@link yfiles.graph.IClipboardHelper#copy} method for later use during the {@link yfiles.graph.GraphClipboard#onElementPasted}
       * operation.
       * </p>
       * @param original The original element from the source graph instance. If the source graph is a managed folding view, this is always a
       * master item.
       * @param copy The copy from the clipboard graph.
       * @protected
       */
      onElementCopied(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem):void;
      /**
       * A callback function that will be invoked for each item that has been copied into the clipboard graph in response to a {@link yfiles.graph.GraphClipboard#cut}
       * operation.
       * <p>
       * This implementation will {@link yfiles.graph.ILookup#lookup} a {@link yfiles.graph.IClipboardHelper} for each item in the original graph
       * and remember the memento retrieved by the {@link yfiles.graph.IClipboardHelper#cut} method for later use during the {@link yfiles.graph.GraphClipboard#onElementPasted}
       * operation.
       * </p>
       * @param original The original element from the source graph instance. If the source graph is a managed folding view, this is always a
       * master item.
       * @param copy The copy from the clipboard graph.
       * @protected
       */
      onElementCut(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem):void;
      /**
       * A callback function that will be invoked for each item that has been pasted into the target graph in response to a
       * Paste(IGraph) operation.
       * <p>
       * This method will use a previously stored {@link yfiles.graph.IClipboardHelper} to let it {@link yfiles.graph.IClipboardHelper#paste finish} its work.
       * </p>
       * @param original The original element from the clipboard graph instance. If the target graph is a managed folding view, this is always a
       * master item.
       * @param copy The copy from the target graph.
       * @protected
       */
      onElementPasted(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem):void;
      /**
       * Calls {@link yfiles.graph.GraphCopier#copy} on the current {@link yfiles.graph.GraphClipboard#fromClipboardCopier} instance, depending on whether the <code>targetGraph</code> is the {@link yfiles.graph.GraphClipboard#clipboardGraph} instance.
       * @param sourceGraph The source graph to pass to {@link #copy}
       * @param filter The predicate to pass on.
       * @param targetGraph The target graph to pass on.
       * @param targetRootNode The node to use as the root node in the grouped <code>targetGraph</code>.
       * @param elementCopiedCallback The delegate to pass on. For each element that is copied during the operation to the clipboard, this callback will be
       * passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link #views folding view graph} the original element is
       * passed as {@link #masterGraph master} elements or {@link #getFolderNodeState folding state elements} if a master element does not exist. The copied element is always
       * part of the <code>targetGraph</code>.
       * @param targetFilter This predicate is used to determine which {@link }s and {@link }s can be used as targets for paste
       * operations, e.g. if a labels or ports are copied without their owners. If <code>null</code> all model items can be used as targets.
       * @protected
       */
      onPaste(sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean,targetGraph:yfiles.graph.IGraph,targetRootNode:yfiles.graph.INode,elementCopiedCallback:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void,targetFilter:(obj:yfiles.graph.IModelItem)=>boolean):void;
      /**
       * Pastes the {@link yfiles.graph.GraphClipboard#clipboardGraph}'s contents into the <code>targetGraph</code>, selecting all pasted elements.
       * <p>
       * The actual work of this method is delegated to the {@link yfiles.graph.GraphClipboard#fromClipboardCopier}'s {@link yfiles.graph.GraphCopier#copy} method.
       * </p>
       * @param targetGraph The graph to paste into.
       * @param selection The selection model to set the selection on.
       */
      paste(targetGraph:yfiles.graph.IGraph,selection:yfiles.view.IGraphSelection):void;
      /**
       * Pastes the contents from the {@link yfiles.graph.GraphClipboard#clipboardGraph} to the <code>targetGraph</code> after applying a <code>filter</code>.
       * <p>
       * For each element pasted into the graph the <code>elementPasted</code> delegate will be invoked and the {@link yfiles.graph.GraphClipboard#onElementPasted}
       * method will be called. The actual work of this method is delegated to the {@link yfiles.graph.GraphClipboard#fromClipboardCopier}'s {@link yfiles.graph.GraphCopier#copy} method.
       * <br />
       * Providing a valid <code>targetFilter</code> allows for pasting {@link yfiles.graph.ILabel}s, {@link yfiles.graph.IPort}s, and
       * {@link yfiles.graph.IEdge}s without valid owners.
       * </p>
       * @param context A valid input mode context. Passing a valid input mode context is required for keeping orthogonal edges at group nodes
       * whose size might be adjusted after the paste operation.
       * @param targetGraph The graph to paste the contents in.
       * @param filter The filter to apply to the clipboard graph.
       * @param elementPasted The callback to use for each element that has been pasted. For each element that is copied during the operation to the
       * clipboard, this callback will be passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link #views folding view graph}
       * the original element is passed as {@link #masterGraph master} elements or {@link #getFolderNodeState folding state elements} if a master element does not exist. The
       * copied element is always part of the <code>targetGraph</code>.
       * @param targetFilter A filter which returns <code>true</code> for model items which can be used as target for clipboard items which need an owner (e.g.
       * {@link }s which were copied without their owner). If <code>null</code> all model items can be used as targets.
       */
      paste(context:yfiles.input.IInputModeContext,targetGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean,elementPasted:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void,targetFilter:(obj:yfiles.graph.IModelItem)=>boolean):void;
      /**
       * Pastes the contents from the {@link yfiles.graph.GraphClipboard#clipboardGraph} to the <code>targetGraph</code> after applying a <code>filter</code>.
       * <p>
       * For each element pasted into the graph the <code>elementPasted</code> delegate will be invoked and the {@link yfiles.graph.GraphClipboard#onElementPasted}
       * method will be called. The actual work of this method is delegated to the {@link yfiles.graph.GraphClipboard#fromClipboardCopier}'s {@link yfiles.graph.GraphCopier#copy} method.
       * </p>
       * @param targetGraph The graph to paste the contents in.
       * @param [filter=null] The filter to apply to the clipboard graph.
       * @param [elementPasted=null] The callback to use for each element that has been pasted. For each element that is copied during the operation to the
       * clipboard, this callback will be passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link #views folding view graph}
       * the original element is passed as {@link #masterGraph master} elements or {@link #getFolderNodeState folding state elements} if a master element does not exist. The
       * copied element is always part of the <code>targetGraph</code>.
       */
      paste(targetGraph:yfiles.graph.IGraph,filter?:(obj:yfiles.graph.IModelItem)=>boolean,elementPasted?:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void):void;
      /**
       * Pastes the contents from the {@link yfiles.graph.GraphClipboard#clipboardGraph} to the <code>targetGraph</code> after applying a <code>filter</code>.
       * <p>
       * For each element pasted into the graph the <code>elementPasted</code> delegate will be invoked and the {@link yfiles.graph.GraphClipboard#onElementPasted}
       * method will be called. The actual work of this method is delegated to the {@link yfiles.graph.GraphClipboard#fromClipboardCopier}'s {@link yfiles.graph.GraphCopier#copy} method.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.targetGraph The graph to paste the contents in.
       * @param [options.filter=null] The filter to apply to the clipboard graph.
       * @param [options.elementPasted=null] The callback to use for each element that has been pasted. For each element that is copied during the operation to the
       * clipboard, this callback will be passed the original and the copy of the item. In case the <code>targetGraph</code> is a {@link #views folding view graph}
       * the original element is passed as {@link #masterGraph master} elements or {@link #getFolderNodeState folding state elements} if a master element does not exist. The
       * copied element is always part of the <code>targetGraph</code>.
       */
      paste(options:{targetGraph:yfiles.graph.IGraph,filter?:(obj:yfiles.graph.IModelItem)=>boolean,elementPasted?:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void}):void;
      /**
       * Removes elements from the graph based on a predicate.
       * @param sourceGraph The graph to remove elements from.
       * @param predicate The predicate that decides which elements to remove.
       * @see yfiles.graph.GraphClipboard#cut
       * @protected
       */
      removeElements(sourceGraph:yfiles.graph.IGraph,predicate:(obj:yfiles.graph.IModelItem)=>boolean):void;
      /**
       * Gets or sets a vector that will be used to move nodes and bends by after they have been pasted to the target graph.
       * @type {yfiles.geometry.Point}
       */
      pasteDelta:yfiles.geometry.Point;
      /**
       * Determines whether the clipboard graph is empty.
       * @type {boolean}
       */
      empty:boolean;
      /**
       * Gets or sets the {@link yfiles.graph.GraphCopier} instance that will be used to copy the graph to the clipboard.
       * <p>
       * This instance is lazily created using {@link yfiles.graph.GraphClipboard#createToClipboardCopier}.
       * </p>
       * @type {yfiles.graph.GraphCopier}
       */
      toClipboardCopier:yfiles.graph.GraphCopier;
      /**
       * Gets or sets the {@link yfiles.graph.GraphCopier} instance that will be used to copy the graph from the clipboard to the target
       * graph.
       * <p>
       * This instance is lazily created using {@link yfiles.graph.GraphClipboard#createFromClipboardCopier}.
       * </p>
       * @type {yfiles.graph.GraphCopier}
       */
      fromClipboardCopier:yfiles.graph.GraphCopier;
      /**
       * Gets or sets the {@link yfiles.graph.GraphCopier} instance that will be used to duplicate elements inside the graph.
       * <p>
       * This instance is lazily created using {@link yfiles.graph.GraphClipboard#createDuplicateCopier}.
       * </p>
       * @type {yfiles.graph.GraphCopier}
       */
      duplicateCopier:yfiles.graph.GraphCopier;
      /**
       * Gets or sets the {@link yfiles.graph.IClipboardIdProvider} used by this instance to provide IDs for copied items.
       * <p>
       * The default implementation simply uses a reference to the original item as
       * ID.
       * <br />
       * Custom ID can be used to map between elements in different graphs.
       * </p>
       * @type {yfiles.graph.IClipboardIdProvider}
       */
      idProvider:yfiles.graph.IClipboardIdProvider;
      /**
       * Gets or sets the graph that contains the clipboard's contents.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>value</code> is <code>null</code>.
       * @type {yfiles.graph.IGraph}
       */
      clipboardGraph:yfiles.graph.IGraph;
      /**
       * Gets or sets whether or not nodes should be automatically put into a group node and if so how this group will be
       * determined.
       * <p>
       * This property is used by {@link yfiles.graph.GraphClipboard#getTargetNode} to decide what node to yield as the target group node
       * for the newly pasted node.
       * </p>
       * <p>
       * The default is {@link yfiles.graph.ParentNodeDetectionModes#SELECTION}|{@link yfiles.graph.ParentNodeDetectionModes#PREVIOUS_PARENT}.
       * </p>
       * @type {yfiles.graph.ParentNodeDetectionModes}
       */
      parentNodeDetection:yfiles.graph.ParentNodeDetectionModes;
      /**
       * Gets or sets the type of items that depend on other elements but should automatically be copied implicitly to the
       * clipboard if all elements that it depends on are explicitly selected for copying.
       * <p>
       * {@link yfiles.graph.ILabel Labels} depend on their {@link yfiles.graph.ILabelOwner owner}, like {@link yfiles.graph.IPort ports} depend on their {@link yfiles.graph.IPortOwner owner}. Also {@link yfiles.graph.IBend bends} of course depend on their {@link yfiles.graph.IEdge owning edge}.
       * </p>
       * <p>
       * If the corresponding element type is set in this property, items that depend on other items are implicitly included in
       * the copy operation, if all of the items that they depend on are included recursively, even if they are not included
       * explicitly themselves.
       * </p>
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL} minus
       * {@link yfiles.graph.GraphItemTypes#NODE}, which means that selecting a number of nodes will automatically select all elements
       * that directly and indirectly depend on the existence of this set of nodes will be copied to the clipboard, too.
       * </p>
       * <p>
       * Technically nodes do not depend on anything. Setting this property to a value that includes {@link yfiles.graph.GraphItemTypes#NODE}
       * will include all child nodes for every selected group node.
       * </p>
       * @default 63 & ~1
       * @type {yfiles.graph.GraphItemTypes}
       */
      dependentCopyItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets the type of items that can be copied and pasted independently of their owning element.
       * <p>
       * {@link yfiles.graph.ILabel Labels} depend on their {@link yfiles.graph.ILabelOwner owner}, like {@link yfiles.graph.IPort ports} depend on their {@link yfiles.graph.IPortOwner owner}. Also {@link yfiles.graph.IBend bends} of course depend on their {@link yfiles.graph.IEdge owning edge}.
       * </p>
       * <p>
       * If the corresponding element type is set in this property, items can be copied alone to the clipboard and be pasted onto
       * different owning elements.
       * </p>
       * <p>
       * This enables copying of a label to one or more owners, without having to copy the owner of the label. Also ports can be
       * copied to other {@link yfiles.graph.IPortOwner}s and bends to other edges.
       * </p>
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL} which means that all elements can be copied to the clipboard even if the
       * items that they depend on are not copied to the clipboard.
       * </p>
       * <p>
       * Technically owning elements are still part of the clipboard as
       * {@link yfiles.graph.GraphClipboard#isDummy dummy elements}, but these elements will not be pasted to the target during paste or duplicate operations.
       * </p>
       * @default yfiles.graph.GraphItemTypes.ALL
       * @type {yfiles.graph.GraphItemTypes}
       */
      independentCopyItems:yfiles.graph.GraphItemTypes;
      /**
       * Gets or sets the type of items that can be copied and pasted at all using the clipboard.
       * <p>
       * If the corresponding element type is set in this property, items can be effectively copied.
       * </p>
       * <p>
       * If the corresponding element type is not set in this property, items will not be pasted, even if they have been
       * explicitly selected. They may copied to the clipboard and become part of the {@link yfiles.graph.GraphClipboard#clipboardGraph}, nevertheless though, if they are required
       * as {@link yfiles.graph.GraphClipboard#isDummy dummy elements}. But these elements will not be pasted to the target during paste or duplicate operations.
       * </p>
       * <p>
       * The default is {@link yfiles.graph.GraphItemTypes#ALL}.
       * </p>
       * @default yfiles.graph.GraphItemTypes.ALL
       * @type {yfiles.graph.GraphItemTypes}
       */
      copyItems:yfiles.graph.GraphItemTypes;
      /**
       * Whether to paste edges whose source or target node was selected when copying to the clipboard while the other node was
       * not.
       * <p>
       * The default is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      ignoreEdgesWithOneNode:boolean;
      /**
       * Adds the given listener for the <code>ElementsCut</code> event that occurs when the {@link yfiles.graph.GraphClipboard#cut} operation
       * finished successfully.
       * @param listener The listener to add.
       * @see yfiles.graph.GraphClipboard#removeElementsCutListener
       */
      addElementsCutListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ElementsCut</code> event that occurs when the {@link yfiles.graph.GraphClipboard#cut} operation
       * finished successfully.
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphClipboard#addElementsCutListener
       */
      removeElementsCutListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>ElementsCopied</code> event that occurs when the {@link yfiles.graph.GraphClipboard#copy} operation
       * finished successfully.
       * @param listener The listener to add.
       * @see yfiles.graph.GraphClipboard#removeElementsCopiedListener
       */
      addElementsCopiedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ElementsCopied</code> event that occurs when the {@link yfiles.graph.GraphClipboard#copy} operation
       * finished successfully.
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphClipboard#addElementsCopiedListener
       */
      removeElementsCopiedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>ElementsPasted</code> event that occurs when the {@link yfiles.graph.GraphClipboard#paste} operation
       * finished successfully.
       * @param listener The listener to add.
       * @see yfiles.graph.GraphClipboard#removeElementsPastedListener
       */
      addElementsPastedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ElementsPasted</code> event that occurs when the {@link yfiles.graph.GraphClipboard#paste} operation
       * finished successfully.
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphClipboard#addElementsPastedListener
       */
      removeElementsPastedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    export enum CloneTypes{
      /**
       * Indicates the {@link yfiles.styles.INodeStyle} constant.
       */
      NODE_STYLE,
      /**
       * Indicates the {@link yfiles.styles.IEdgeStyle} constant.
       */
      EDGE_STYLE,
      /**
       * Indicates the {@link yfiles.styles.IPortStyle} constant.
       */
      PORT_STYLE,
      /**
       * Indicates the {@link yfiles.styles.ILabelStyle} constant.
       */
      LABEL_STYLE,
      /**
       * Indicates the {@link yfiles.graph.ILabelModelParameter} constant.
       */
      LABEL_MODEL_PARAMETER,
      /**
       * Indicates the {@link yfiles.graph.IPortLocationModelParameter} constant.
       */
      PORT_LOCATION_MODEL_PARAMETER,
      /**
       * Indicates the {@link yfiles.graph.ITagOwner#tag} constant.
       */
      TAGS,
      /**
       * A combination of all flags in this enumeration.
       */
      ALL
    }
    /**
     * Utility class that copies parts of an {@link yfiles.graph.IGraph} to another instance.
     * <p>
     * Instances of this class can be used in {@link yfiles.graph.GraphClipboard}, e.g.
     * </p>
     * @class
     */
    export interface GraphCopier extends Object{}
    export class GraphCopier {
      constructor();
      /**
       * Callback method that adds a bend to the targetEdge in the targetGraph as a copy of the source bend.
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param targetEdge The target edge to add the bend to.
       * @param sourceBend The source bend to copy the values from.
       * @param offset The offset to apply to the copied bend.
       * @param index The index to add the bend at.
       * @returns The newly created bend.
       * @protected
       */
      addBend(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,targetEdge:yfiles.graph.IEdge,sourceBend:yfiles.graph.IBend,offset:yfiles.geometry.Point,index:number):yfiles.graph.IBend;
      /**
       * Callback method that adds a label to the target item in the targetGraph as a copy of the source label.
       * <p>
       * This implementation uses {@link yfiles.graph.GraphCopier#copyLabelLayoutParameter}, {@link yfiles.graph.GraphCopier#copyLabelStyle}, and
       * {@link yfiles.graph.GraphCopier#copyLabelTag} as callbacks.
       * </p>
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param targetOwner The item owner in the target graph to add the label to.
       * @param sourceLabel The source label in the source graph.
       * @returns The newly created label.
       * @protected
       */
      addLabel(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,targetOwner:yfiles.graph.ILabelOwner,sourceLabel:yfiles.graph.ILabel):yfiles.graph.ILabel;
      /**
       * Callback method that adds a port to the targetPortOwner in the targetGraph as a copy of the source port.
       * <p>
       * This implementation does not need to copy the adjacent {@link yfiles.graph.IEdge}s of the <code>sourcePort</code>. This implementation uses
       * {@link yfiles.graph.GraphCopier#copyPortLocationParameter}, {@link yfiles.graph.GraphCopier#copyPortStyle}, and {@link yfiles.graph.GraphCopier#copyPortTag}
       * as callbacks.
       * </p>
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param targetOwner The port owner in the target graph to add the port to.
       * @param sourcePort The source port in the source graph.
       * @returns The newly created port.
       */
      addPort(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,targetOwner:yfiles.graph.IPortOwner,sourcePort:yfiles.graph.IPort):yfiles.graph.IPort;
      /**
       * This method places the given copy into an internal cache.
       * <p>
       * The cache can be queried using {@link yfiles.graph.GraphCopier#getCopy}. The cache is held until {@link yfiles.graph.GraphCopier#clearCopyCache}
       * is called.
       * </p>
       * Type parameter <code>T</code>: The type of the item to cache.
       * @param original The original item.
       * @param copy The copy of the item.
       * @see yfiles.graph.GraphCopier#getOrCreateCopy
       * @protected
       * @template T
       */
      cacheCopy<T>(original:T,copy:T):void;
      /**
       * Clears the cache for the {@link yfiles.graph.GraphCopier#cacheCopy cached copies} of the instance copied using this class.
       * <p>
       * This method is called before and after a copy operation.
       * </p>
       * @protected
       */
      clearCopyCache():void;
      /**
       * Copies a subgraph induced by a predicate from the <code>sourceGraph</code> to the <code>targetGraph</code>.
       * <p>
       * For each element that has been copied, the <code>elementCopiedCallback</code> will be invoked.
       * </p>
       * @param sourceGraph The graph to copy the elements from.
       * @param filter The filter that decides which elements to copy.
       * @param targetGraph The graph to copy the elements to.
       * @param targetRootNode The group node that should act as the root node for the pasted elements. May be <code>null</code> if there is no such node.
       * @param offset The offset to apply to the copied items. Only nodes and bends will be offset explicitly, while other items will move
       * implicitly.
       * @param elementCopiedCallback A delegate to invoke for each element being copied.
       * @throws {Stubs.Exceptions.ArgumentError} if <code>targetRootNode</code> is not in <code>targetGraph</code>
       */
      copy(sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean,targetGraph:yfiles.graph.IGraph,targetRootNode:yfiles.graph.INode,offset:yfiles.geometry.Point,elementCopiedCallback:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void):void;
      /**
       * Copies a subgraph described by a predicate from the <code>sourceGraph</code> to the <code>targetGraph</code>.
       * <p>
       * For each element that has been copied, the <code>elementCopiedCallback</code> will be invoked. This method actually delegates to {@link yfiles.graph.GraphCopier#copy}
       * and provides the root as the "targetGraph" parameter if there is a hierarchy associated with <code>targetGraph</code>.
       * </p>
       * @param sourceGraph The graph to copy the elements from.
       * @param filter The filter that decides which elements to copy.
       * @param targetGraph The graph to copy the elements to.
       * @param offset The offset to apply to the copied items. Only nodes and bends will be offset explicitly, while other items will move
       * implicitly.
       * @param elementCopiedCallback A delegate to invoke for each element being copied.
       */
      copy(sourceGraph:yfiles.graph.IGraph,filter:(obj:yfiles.graph.IModelItem)=>boolean,targetGraph:yfiles.graph.IGraph,offset:yfiles.geometry.Point,elementCopiedCallback:(original:yfiles.graph.IModelItem,copy:yfiles.graph.IModelItem)=>void):void;
      /**
       * Copies <code>sourceGraph</code> to <code>targetGraph</code>.
       * <p>
       * This method actually delegates to {@link yfiles.graph.GraphCopier#copy} and provides the root as the "targetGraph" parameter if
       * there is a hierarchy associated with <code>targetGraph</code>.
       * </p>
       * @param sourceGraph The graph to copy.
       * @param targetGraph The graph to copy to.
       */
      copy(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph):void;
      /**
       * Callback method that adds an edge to the targetGraph as a copy of the source edge.
       * <p>
       * This implementation does not need to copy the {@link yfiles.graph.IBend}s of the <code>sourceEdge</code>, nor should it copy
       * {@link yfiles.graph.IPort}s or {@link yfiles.graph.ILabel}s. This implementation uses {@link yfiles.graph.GraphCopier#copyEdgeStyle} and {@link yfiles.graph.GraphCopier#copyEdgeTag}
       * as callbacks.
       * </p>
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param sourceEdge The edge in the source graph.
       * @param targetSourcePort The new source port in the target graph.
       * @param targetTargetPort The new target port in the target graph.
       * @returns The newly created edge.
       * @protected
       */
      copyEdge(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,sourceEdge:yfiles.graph.IEdge,targetSourcePort:yfiles.graph.IPort,targetTargetPort:yfiles.graph.IPort):yfiles.graph.IEdge;
      /**
       * Copies an {@link yfiles.styles.IEdgeStyle} instance.
       * <p>
       * This implementation uses the {@link yfiles.lang.ICloneable#clone} method to create a copy.
       * </p>
       * @param graph The graph that contains the edge.
       * @param edge The edge to copy the style from.
       * @returns The style to apply for the copy.
       * @see yfiles.graph.GraphCopier#copy
       */
      copyEdgeStyle(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge):yfiles.styles.IEdgeStyle;
      /**
       * Callback that copies the {@link yfiles.graph.ITagOwner#tag} of an {@link yfiles.graph.IEdge}.
       * <p>
       * This implementation simply delegates to {@link yfiles.graph.GraphCopier#copyTag}
       * </p>
       * @param edge The old edge.
       * @param tag The tag to copy.
       * @returns The copy of the tag.
       * @protected
       */
      copyEdgeTag(edge:yfiles.graph.IEdge,tag:Object):Object;
      /**
       * Callback method that creates a copy of the <code>sourceGroupNode</code> in the <code>targetGraph</code>.
       * <p>
       * This implementation does not need to copy {@link yfiles.graph.IPort}s or {@link yfiles.graph.ILabel}s. This implementation uses {@link yfiles.graph.GraphCopier#copyNodeStyle}
       * and {@link yfiles.graph.GraphCopier#copyNodeTag} as callbacks.
       * </p>
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param sourceGroupNode The source node to create a copy of in the <code>targetGraph</code>.
       * @param targetParent The parent of the group node in the target graph.
       * @param offset The offset to apply to the copied node.
       * @returns The newly created node.
       * @protected
       */
      copyGroupNode(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,sourceGroupNode:yfiles.graph.INode,targetParent:yfiles.graph.INode,offset:yfiles.geometry.Point):yfiles.graph.INode;
      /**
       * Copies a {@link yfiles.graph.ILabelModelParameter} for a given label.
       * <p>
       * This implementation uses the {@link yfiles.lang.ICloneable#clone} method to create a copy.
       * </p>
       * @param graph The graph that contains the label.
       * @param label The label to copy the label model parameter from.
       * @returns The copy of the parameter to apply for the copy.
       * @see yfiles.graph.GraphCopier#copy
       */
      copyLabelLayoutParameter(graph:yfiles.graph.IGraph,label:yfiles.graph.ILabel):yfiles.graph.ILabelModelParameter;
      /**
       * Copies an {@link yfiles.styles.ILabelStyle} instance.
       * <p>
       * This implementation uses the {@link yfiles.lang.ICloneable#clone} method to create a copy.
       * </p>
       * @param graph The graph that contains the label.
       * @param label The label to copy the style from.
       * @returns The style to apply for the copy.
       * @see yfiles.graph.GraphCopier#copy
       */
      copyLabelStyle(graph:yfiles.graph.IGraph,label:yfiles.graph.ILabel):yfiles.styles.ILabelStyle;
      /**
       * Callback that copies the {@link yfiles.graph.ITagOwner#tag} of a {@link yfiles.graph.ILabel}.
       * <p>
       * This implementation simply delegates to {@link yfiles.graph.GraphCopier#copyTag}
       * </p>
       * @param label The old label.
       * @param tag The tag to copy.
       * @returns The copy of the tag.
       */
      copyLabelTag(label:yfiles.graph.ILabel,tag:Object):Object;
      /**
       * Callback method that creates a copy of the <code>node</code> in the <code>targetGraph</code>.
       * <p>
       * This implementation does not need to copy {@link yfiles.graph.IPort}s or {@link yfiles.graph.ILabel}s. This implementation uses {@link yfiles.graph.GraphCopier#copyNodeStyle}
       * and {@link yfiles.graph.GraphCopier#copyNodeTag} as callbacks.
       * </p>
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param targetParent The parent of the group node in the target graph.
       * @param node The source node to create a copy of in the <code>targetGraph</code>.
       * @param offset The offset to apply to the copied node.
       * @returns The newly created node.
       * @protected
       */
      copyNode(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,node:yfiles.graph.INode,targetParent:yfiles.graph.INode,offset:yfiles.geometry.Point):yfiles.graph.INode;
      /**
       * Callback method that creates a copy of the <code>node</code> in the <code>targetGraph</code>.
       * <p>
       * This implementation does not need to copy {@link yfiles.graph.IPort}s or {@link yfiles.graph.ILabel}s. This implementation uses {@link yfiles.graph.GraphCopier#copyNodeStyle}
       * and {@link yfiles.graph.GraphCopier#copyNodeTag} as callbacks.
       * </p>
       * @param sourceGraph The source graph.
       * @param targetGraph The target graph.
       * @param node The source node to create a copy of in the <code>targetGraph</code>.
       * @param offset The offset to apply to the copied node.
       * @returns The newly created node.
       * @protected
       */
      copyNode(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph,node:yfiles.graph.INode,offset:yfiles.geometry.Point):yfiles.graph.INode;
      /**
       * Copies an {@link yfiles.styles.INodeStyle} instance.
       * <p>
       * This implementation uses the {@link yfiles.lang.ICloneable#clone} method to create a copy.
       * </p>
       * @param graph The graph that contains the node.
       * @param node The node to copy the style from.
       * @returns The style to apply for the copy.
       * @see yfiles.graph.GraphCopier#copy
       */
      copyNodeStyle(graph:yfiles.graph.IGraph,node:yfiles.graph.INode):yfiles.styles.INodeStyle;
      /**
       * Callback that copies the {@link yfiles.graph.ITagOwner#tag} of a {@link yfiles.graph.INode}.
       * <p>
       * This implementation simply delegates to {@link yfiles.graph.GraphCopier#copyTag}
       * </p>
       * @param node The old node.
       * @param tag The tag to copy.
       * @returns The copy of the tag.
       * @protected
       */
      copyNodeTag(node:yfiles.graph.INode,tag:Object):Object;
      /**
       * Copies a {@link yfiles.graph.IPortLocationModelParameter} for a given port.
       * <p>
       * This implementation uses the {@link yfiles.lang.ICloneable#clone} method to create a copy.
       * </p>
       * @param graph The graph that contains the port.
       * @param port The port to copy the model parameter from.
       * @returns The copy of the parameter to apply for the copy.
       * @see yfiles.graph.GraphCopier#copy
       */
      copyPortLocationParameter(graph:yfiles.graph.IGraph,port:yfiles.graph.IPort):yfiles.graph.IPortLocationModelParameter;
      /**
       * Copies an {@link yfiles.styles.IPortStyle} instance.
       * <p>
       * This implementation uses the {@link yfiles.lang.ICloneable#clone} method to create a copy.
       * </p>
       * @param graph The graph that contains the port.
       * @param port The port to copy the style from.
       * @returns The style to apply for the copy.
       * @see yfiles.graph.GraphCopier#copy
       */
      copyPortStyle(graph:yfiles.graph.IGraph,port:yfiles.graph.IPort):yfiles.styles.IPortStyle;
      /**
       * Callback that copies the {@link yfiles.graph.ITagOwner#tag} of a {@link yfiles.graph.IPort}.
       * <p>
       * This implementation simply delegates to {@link yfiles.graph.GraphCopier#copyTag}
       * </p>
       * @param port The old port.
       * @param tag The tag to copy.
       * @returns The copy of the tag.
       * @protected
       */
      copyPortTag(port:yfiles.graph.IPort,tag:Object):Object;
      /**
       * Callback that will copy a {@link yfiles.graph.ITagOwner#tag} for the target graph.
       * <p>
       * This implementation will try to {@link yfiles.lang.ICloneable#clone} the <code>tag</code> if the {@link yfiles.graph.GraphCopier#clone} property has the {@link yfiles.graph.CloneTypes#TAGS}
       * flag set. If this is not the case or the tag is not {@link yfiles.lang.ICloneable}, the instance will be returned.
       * </p>
       * @param owner The old owner of the tag.
       * @param tag The tag to copy.
       * @returns A copy of the tag.
       */
      copyTag(owner:yfiles.graph.IModelItem,tag:Object):Object;
      /**
       * Tries to obtain a previously {@link yfiles.graph.GraphCopier#cacheCopy cached} copy for the given original.
       * <p>
       * Values will be stored in the cache until the next call to {@link yfiles.graph.GraphCopier#clearCopyCache}. In order to indicate
       * that no copy is available, this implementation yields {@link yfiles.graph.GraphCopier#NO_COPY}. In contrast, <code>null</code> is a valid
       * return value that is the copied value of the original.
       * </p>
       * Type parameter <code>T</code>: The type of the item to copy.
       * @param itemType The type of the item to copy.
       * @param original The original item.
       * @returns The copy of <code>original</code> or {@link #NO_COPY} if no copy has been found.
       * @see yfiles.graph.GraphCopier#cacheCopy
       * @see yfiles.graph.GraphCopier#getOrCreateCopy
       * @protected
       * @template T
       */
      getCopy<T>(itemType:yfiles.lang.Class,original:T):T;
      /**
       * Convenience method that either yields a previously {@link yfiles.graph.GraphCopier#cacheCopy cached} copy for the given original or uses the <code>copyDelegate</code> to create
       * the copy of the original.
       * <p>
       * If there is a cache miss, the delegate will be used to create the copy and the copy will be placed into the {@link yfiles.graph.GraphCopier#cacheCopy cache}.
       * </p>
       * Type parameter <code>T</code>: The type of the item to copy.
       * @param tType The type of the item to copy.
       * @param original The original item.
       * @param copyDelegate The copy delegate to create the copy.
       * @returns A copy of the original, either cached, or newly created and then cached.
       * @template T
       */
      getOrCreateCopy<T>(tType:yfiles.lang.Class,original:T,copyDelegate:(arg:T)=>T):T;
      /**
       * Called after a bend has been copied.
       * <p>
       * Raises the {@link yfiles.graph.GraphCopier#addBendCopiedListener BendCopied} event.
       * </p>
       * @param original The original of the copied bend.
       * @param copy The copied bend.
       */
      onBendCopied(original:yfiles.graph.IBend,copy:yfiles.graph.IBend):void;
      /**
       * Called after an edge has been copied.
       * <p>
       * Raises the {@link yfiles.graph.GraphCopier#addEdgeCopiedListener EdgeCopied} event.
       * </p>
       * @param original The original of the copied edge.
       * @param copy The copied edge.
       * @protected
       */
      onEdgeCopied(original:yfiles.graph.IEdge,copy:yfiles.graph.IEdge):void;
      /**
       * Called after the entire graph has been copied.
       * <p>
       * Raises the {@link yfiles.graph.GraphCopier#addGraphCopiedListener GraphCopied} event.
       * </p>
       * @param sourceGraph The source graph from which the elements have been copied.
       * @param targetGraph The graph to which the elements have been copied.
       * @protected
       */
      onGraphCopied(sourceGraph:yfiles.graph.IGraph,targetGraph:yfiles.graph.IGraph):void;
      /**
       * Called after a label has been copied.
       * <p>
       * Raises the {@link yfiles.graph.GraphCopier#addLabelCopiedListener LabelCopied} event.
       * </p>
       * @param original The original of the copied label.
       * @param copy The copied label.
       */
      onLabelCopied(original:yfiles.graph.ILabel,copy:yfiles.graph.ILabel):void;
      /**
       * Called after a node has been copied.
       * <p>
       * Raises the {@link yfiles.graph.GraphCopier#addNodeCopiedListener NodeCopied} event.
       * </p>
       * @param original The original of the copied node.
       * @param copy The copied node.
       * @protected
       */
      onNodeCopied(original:yfiles.graph.INode,copy:yfiles.graph.INode):void;
      /**
       * Emits the {@link yfiles.graph.GraphCopier#addObjectCopiedListener ObjectCopied} event.
       * @param evt The argument to use.
       * @protected
       */
      onObjectCopied(evt:yfiles.graph.ItemCopiedEventArgs<Object>):void;
      /**
       * Called after a port has been copied.
       * <p>
       * Raises the {@link yfiles.graph.GraphCopier#addPortCopiedListener PortCopied} event.
       * </p>
       * @param original The original of the copied port.
       * @param copy The copied port.
       */
      onPortCopied(original:yfiles.graph.IPort,copy:yfiles.graph.IPort):void;
      /**
       * Gets or sets the types of items that should be {@link yfiles.lang.ICloneable#clone cloned} for the copy operation.
       * @type {yfiles.graph.CloneTypes}
       */
      clone:yfiles.graph.CloneTypes;
      /**
       * Gets or sets the types of objects where the referential identity should be kept across clipboard operations.
       * <p>
       * For each of the types that are set via this property, this implementation will use a caching copy mechanism as follows:
       * If there has already been an instance copied via this clipboard ({@link yfiles.graph.GraphCopier#getCopy}), that instance will
       * instead be returned, otherwise, this implementation will clone the original instance (depending on the setting of {@link yfiles.graph.GraphCopier#clone}) and
       * {@link yfiles.graph.GraphCopier#cacheCopy cache the copy} until after the operation the {@link yfiles.graph.GraphCopier#clearCopyCache cache is cleared}. This feature is implemented directly by the various {@link yfiles.graph.GraphCopier#copyNodeStyle CopyXXX}
       * methods.
       * </p>
       * @see yfiles.graph.GraphCopier#getOrCreateCopy
       * @type {yfiles.graph.CloneTypes}
       */
      referentialIdentityTypes:yfiles.graph.CloneTypes;
      /**
       * The sentinel return value for use in {@link yfiles.graph.GraphCopier#getCopy}.
       * @returns This value is used as a place-holder to indicate that no copy has been stored previously.
       * @protected
       * @const
       * @static
       * @type {Object}
       */
      static NO_COPY:Object;
      /**
       * Gets or sets a property that determines whether grouping information should be copied to target graphs.
       * <p>
       * With this property set to <code>false</code> the {@link yfiles.graph.GraphCopier} can be used to flatten a grouped graph, i.e. remove the
       * grouping information.
       * </p>
       * <p>
       * The default is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      copyGrouping:boolean;
      /**
       * Gets or sets a property that determines whether {@link yfiles.graph.FoldingManager additional folding states}.
       * <p>
       * Should be copied to target graphs.
       * </p>
       * <p>
       * If the source and target graphs have an instance of {@link yfiles.graph.FoldingManager} in their lookup, the additional {@link yfiles.graph.FoldingManager#getFolderNodeState folding state}
       * of the nodes and edges will be copied. The default is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      copyFoldingStates:boolean;
      /**
       * Adds the given listener for the <code>NodeCopied</code> event that occurs after a node has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removeNodeCopiedListener
       */
      addNodeCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>NodeCopied</code> event that occurs after a node has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addNodeCopiedListener
       */
      removeNodeCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>BendCopied</code> event that occurs after a bend has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * For {@link yfiles.graph.FoldingManager#getFoldingEdgeState folding edge} bends, their view state representation is passed. These elements are neither part of the {@link yfiles.graph.FoldingManager#masterGraph} nor the
       * {@link yfiles.graph.FoldingManager#views view graph}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removeBendCopiedListener
       */
      addBendCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * Removes the given listener for the <code>BendCopied</code> event that occurs after a bend has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * For {@link yfiles.graph.FoldingManager#getFoldingEdgeState folding edge} bends, their view state representation is passed. These elements are neither part of the {@link yfiles.graph.FoldingManager#masterGraph} nor the
       * {@link yfiles.graph.FoldingManager#views view graph}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addBendCopiedListener
       */
      removeBendCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * Adds the given listener for the <code>ObjectCopied</code> event that occurs after a style has been copied.
       * <p>
       * This event is triggered for all minor objects the graph items are made up of, e.g. the {@link yfiles.graph.GraphCopier#copyNodeStyle styles}, {@link yfiles.graph.GraphCopier#copyTag tags}, {@link yfiles.graph.GraphCopier#copyLabelLayoutParameter labels}, and {@link yfiles.graph.GraphCopier#copyPortLocationParameter port}
       * parameters.
       * </p>
       * <p>
       * This event will <b>not</b> be triggered for the graph items themselves and will not be triggered if the object reference is
       * reused, i.e. it will only be triggered if the old and new references actually differ.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removeObjectCopiedListener
       */
      addObjectCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<Object>)=>void):void;
      /**
       * Removes the given listener for the <code>ObjectCopied</code> event that occurs after a style has been copied.
       * <p>
       * This event is triggered for all minor objects the graph items are made up of, e.g. the {@link yfiles.graph.GraphCopier#copyNodeStyle styles}, {@link yfiles.graph.GraphCopier#copyTag tags}, {@link yfiles.graph.GraphCopier#copyLabelLayoutParameter labels}, and {@link yfiles.graph.GraphCopier#copyPortLocationParameter port}
       * parameters.
       * </p>
       * <p>
       * This event will <b>not</b> be triggered for the graph items themselves and will not be triggered if the object reference is
       * reused, i.e. it will only be triggered if the old and new references actually differ.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addObjectCopiedListener
       */
      removeObjectCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<Object>)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeCopied</code> event that occurs after an edge has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removeEdgeCopiedListener
       */
      addEdgeCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeCopied</code> event that occurs after an edge has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addEdgeCopiedListener
       */
      removeEdgeCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * Adds the given listener for the <code>PortCopied</code> event that occurs after a port has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removePortCopiedListener
       */
      addPortCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * Removes the given listener for the <code>PortCopied</code> event that occurs after a port has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addPortCopiedListener
       */
      removePortCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelCopied</code> event that occurs after a label has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * For {@link yfiles.graph.FoldingManager#getFoldingEdgeState folding edge} or dummy node labels, their view state representation is passed. These elements are neither part of the
       * {@link yfiles.graph.FoldingManager#masterGraph} nor the {@link yfiles.graph.FoldingManager#views view graph}.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removeLabelCopiedListener
       */
      addLabelCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelCopied</code> event that occurs after a label has been copied.
       * <p>
       * This event is dispatched after the element copied callback has been executed.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied} item is a master item, i.e. it belongs to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * </p>
       * <p>
       * For {@link yfiles.graph.FoldingManager#getFoldingEdgeState folding edge} or dummy node labels, their view state representation is passed. These elements are neither part of the
       * {@link yfiles.graph.FoldingManager#masterGraph} nor the {@link yfiles.graph.FoldingManager#views view graph}.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addLabelCopiedListener
       */
      removeLabelCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>GraphCopied</code> event that occurs after the entire graph has been copied.
       * <p>
       * This event is dispatched at the end of the {@link yfiles.graph.GraphCopier#copy} method.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original source graph} item is the {@link yfiles.graph.FoldingManager#masterGraph master graph} of the folding view.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy target graph} is the {@link yfiles.graph.FoldingManager#masterGraph master graph} of the folding view.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.GraphCopier#removeGraphCopiedListener
       */
      addGraphCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IGraph>)=>void):void;
      /**
       * Removes the given listener for the <code>GraphCopied</code> event that occurs after the entire graph has been copied.
       * <p>
       * This event is dispatched at the end of the {@link yfiles.graph.GraphCopier#copy} method.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#toClipboardCopier} (Cut/Copy) and the source graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original source graph} item is the {@link yfiles.graph.FoldingManager#masterGraph master graph} of the folding view.
       * </p>
       * <p>
       * If this GraphCopier instance is used as {@link yfiles.graph.GraphClipboard}'s {@link yfiles.graph.GraphClipboard#fromClipboardCopier} (Paste) and the target graph is a folding view
       * graph the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy target graph} is the {@link yfiles.graph.FoldingManager#masterGraph master graph} of the folding view.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.GraphCopier#addGraphCopiedListener
       */
      removeGraphCopiedListener(listener:(sender:Object,evt:yfiles.graph.ItemCopiedEventArgs<yfiles.graph.IGraph>)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event arguments for the events in {@link yfiles.graph.GraphCopier}.
     * <p>
     * This event is used for the {@link yfiles.graph.GraphCopier#addNodeCopiedListener NodeCopied},
     * {@link yfiles.graph.GraphCopier#addEdgeCopiedListener EdgeCopied},
     * {@link yfiles.graph.GraphCopier#addPortCopiedListener PortCopied},
     * {@link yfiles.graph.GraphCopier#addGraphCopiedListener GraphCopied},
     * {@link yfiles.graph.GraphCopier#addLabelCopiedListener LabelCopied}, and {@link yfiles.graph.GraphCopier#addObjectCopiedListener ObjectCopied}
     * events. This event holds references to the {@link yfiles.graph.ItemCopiedEventArgs.<T>#original original} of the copied item and the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied item} itself.
     * </p>
     * Type parameter <code>T</code>: The logical base type of the copied item.
     * @class
     * @extends {yfiles.lang.EventArgs}
     * @template T
     */
    export interface ItemCopiedEventArgs<T> extends yfiles.lang.EventArgs{}
    export class ItemCopiedEventArgs<T> {
      /**
       * Creates a new argument for the given item and its original.
       * @param original The {@link #original original} of the copied item.
       * @param copy The {@link #copy copied item}.
       */
      constructor(original:T,copy:T);
      /**
       * The original of the {@link yfiles.graph.ItemCopiedEventArgs.<T>#copy copied item}.
       * @type {T}
       */
      original:T;
      /**
       * The copied item.
       * @type {T}
       */
      copy:T;
      static $class:yfiles.lang.Class;
    }
    /**
     * Helper class that can be used to aid in decorating {@link yfiles.graph.IModelItem}s from an {@link yfiles.graph.IGraph} instance.
     * <p>
     * This class servers as a factory for lookup decorators for the various items that make up an {@link yfiles.graph.IGraph}. This is
     * a convenience class that obtains the {@link yfiles.graph.ILookupDecorator} from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}
     * and passes that to the various specialized decorator convenience classes like e.g. {@link yfiles.graph.NodeDecorator}.
     * </p>
     * @see yfiles.graph.GraphDecorator#nodeDecorator
     * @see yfiles.graph.NodeDecorator
     * @see yfiles.graph.GraphDecorator#edgeDecorator
     * @see yfiles.graph.EdgeDecorator
     * @see yfiles.graph.GraphDecorator#labelDecorator
     * @see yfiles.graph.LabelDecorator
     * @see yfiles.graph.GraphDecorator#portDecorator
     * @see yfiles.graph.PortDecorator
     * @see yfiles.graph.GraphDecorator#bendDecorator
     * @see yfiles.graph.BendDecorator
     * @class
     * @final
     */
    export interface GraphDecorator extends Object{}
    export class GraphDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.graph.GraphDecorator} class.
       * @param graph The graph.
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Returns a {@link yfiles.graph.GraphDecorator#nodeDecorator} instance that can be used to decorate the {@link yfiles.graph.INode} instances in the graph that this decorator has
       * been created for.
       * <p>
       * This will obtain the {@link yfiles.graph.ILookupDecorator} instance from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}
       * method and use it to initialize the {@link yfiles.graph.NodeDecorator}
       * </p>
       * @type {yfiles.graph.NodeDecorator}
       */
      nodeDecorator:yfiles.graph.NodeDecorator;
      /**
       * Returns an {@link yfiles.graph.GraphDecorator#edgeDecorator} instance that can be used to decorate the {@link yfiles.graph.IEdge} instances in the graph that this decorator has
       * been created for.
       * <p>
       * This will obtain the {@link yfiles.graph.ILookupDecorator} instance from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}
       * method and use it to initialize the {@link yfiles.graph.EdgeDecorator}
       * </p>
       * @type {yfiles.graph.EdgeDecorator}
       */
      edgeDecorator:yfiles.graph.EdgeDecorator;
      /**
       * Returns a {@link yfiles.graph.GraphDecorator#labelDecorator} instance that can be used to decorate the {@link yfiles.graph.ILabel} instances in the graph that this decorator has
       * been created for.
       * <p>
       * This will obtain the {@link yfiles.graph.ILookupDecorator} instance from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}
       * method and use it to initialize the {@link yfiles.graph.LabelDecorator}
       * </p>
       * @type {yfiles.graph.LabelDecorator}
       */
      labelDecorator:yfiles.graph.LabelDecorator;
      /**
       * Returns a {@link yfiles.graph.GraphDecorator#portDecorator} instance that can be used to decorate the {@link yfiles.graph.IPort} instances in the graph that this decorator has
       * been created for.
       * <p>
       * This will obtain the {@link yfiles.graph.ILookupDecorator} instance from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}
       * method and use it to initialize the {@link yfiles.graph.PortDecorator}
       * </p>
       * @type {yfiles.graph.PortDecorator}
       */
      portDecorator:yfiles.graph.PortDecorator;
      /**
       * Returns a {@link yfiles.graph.GraphDecorator#bendDecorator} instance that can be used to decorate the {@link yfiles.graph.IBend} instances in the graph that this decorator has
       * been created for.
       * <p>
       * This will obtain the {@link yfiles.graph.ILookupDecorator} instance from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}
       * method and use it to initialize the {@link yfiles.graph.BendDecorator}
       * </p>
       * @type {yfiles.graph.BendDecorator}
       */
      bendDecorator:yfiles.graph.BendDecorator;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that helps in {@link yfiles.graph.ILookupDecorator#addLookup decorating} the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.ILabel} instances that
     * are contained in an {@link yfiles.graph.IGraph} which are managed by an {@link yfiles.graph.ILookupDecorator}.
     * <p>
     * This decorator can be used to conveniently {@link yfiles.graph.ILookupDecorator decorate the lookup} of {@link yfiles.graph.ILabel}s in an {@link yfiles.graph.IGraph}. It
     * serves as a factory for predefined {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} instances that are specifically useful for the {@link yfiles.graph.ILabel} type. Note that
     * the list of possible decorations that are available using this class is not exhaustive in any way. This is merely a way
     * to provide access to those interfaces that are more commonly being decorated by the developer.
     * </p>
     * @see yfiles.graph.GraphDecorator
     * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>
     * @class
     * @final
     */
    export interface LabelDecorator extends Object{}
    export class LabelDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.graph.LabelDecorator} class.
       * @param decorator The decorator to use, which can be <code>null</code>.
       */
      constructor(decorator:yfiles.graph.ILookupDecorator);
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.ILabel}s that can be used to decorate <code>TInterface</code> types of the labels.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean):yfiles.graph.LookupDecorator<yfiles.graph.ILabel,TInterface>;
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.ILabel}s that can be used to decorate <code>TInterface</code> types of the labels.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param {Object} options The parameters to pass.
       * @param options.type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(options:{type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean}):yfiles.graph.LookupDecorator<yfiles.graph.ILabel,TInterface>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPositionHandler} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPositionHandler} implementations can be used to restrict or enhance the way the user moves elements
       * interactively in the editor. This interface is mainly used by the {@link yfiles.input.MoveInputMode}. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.input.IPositionHandler>}
       */
      positionHandlerDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.input.IPositionHandler>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IClipboardHelper} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IClipboardHelper} implementations can be used to customize the way the clipboard operations are
       * performed by {@link yfiles.graph.GraphClipboard}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.graph.IClipboardHelper>}
       */
      clipboardHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.graph.IClipboardHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IMarqueeTestable} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
       * performed on the items in the editor. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode}. This is a
       * convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.input.IMarqueeTestable>}
       */
      marqueeTestableDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.input.IMarqueeTestable>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IMementoSupport} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IMementoSupport} implementations can be used to make the {@link yfiles.graph.UndoEngine} aware of changes to
       * data that is associated with the model items. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.graph.IMementoSupport>}
       */
      mementoSupportDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.graph.IMementoSupport>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.ISelectionIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.ISelectionIndicatorInstaller} implementations can be used change the way the selection of an item is
       * represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.SelectionIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.view.ISelectionIndicatorInstaller>}
       */
      selectionDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.view.ISelectionIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IHighlightIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an item
       * is represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.HighlightIndicatorManager.<T>} in the
       * {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.view.IHighlightIndicatorInstaller>}
       */
      highlightDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.view.IHighlightIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IFocusIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IFocusIndicatorInstaller} implementations can be used change the way the focus of an item is
       * represented in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.view.FocusIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.view.IFocusIndicatorInstaller>}
       */
      focusIndicatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.view.IFocusIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandleProvider} implementations provide interactive draggable handles for the user to change the
       * geometry or other aspects of items in the {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in the {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.input.IHandleProvider>}
       */
      handleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.input.IHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IObstacleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.view.IObstacleProvider} implementations provide information about obstacles that will be considered by {@link yfiles.view.GraphObstacleProvider}
       * which itself serves as a provider for geometric obstacles to {@link yfiles.view.BridgeManager} that manages the rendering of
       * bridges in edge paths. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is
       * provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @see yfiles.view.GraphObstacleProvider
       * @see yfiles.view.BridgeManager
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.view.IObstacleProvider>}
       */
      obstacleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.view.IObstacleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.ILabelModelParameterFinder} type for the
       * {@link yfiles.graph.ILabel}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.ILabelModelParameterFinder} implementations can be used change the way {@link yfiles.graph.ILabel}s can be
       * dragged interactively by the user. It helps in determining the right {@link yfiles.graph.ILabelModelParameter} for a given
       * location that has been chosen by the user. This interface is mainly used by the {@link yfiles.input.LabelPositionHandler} in the
       * {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameterFinder>}
       */
      labelModelParameterFinderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameterFinder>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IEditLabelHelper} type for the
       * {@link yfiles.graph.ILabel}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IEditLabelHelper} implementations can be used to change or customize interactive label editing. This
       * interface is mainly used by the {@link yfiles.input.GraphEditorInputMode} which uses the callbacks from the implementation to
       * determine whether the label can be edited, as well as to customize the {@link yfiles.input.TextEditorInputMode} appearance for
       * the upcoming edit. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is
       * provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.input.IEditLabelHelper>}
       */
      editLabelHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.input.IEditLabelHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.ILabelModelParameterProvider} type for the
       * {@link yfiles.graph.ILabel}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.ILabelModelParameterProvider} implementations can be used to change the set of possible
       * {@link yfiles.graph.ILabelModelParameter}s that the user can assign to a label interactively by dragging it in a
       * {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.LabelPositionHandler} and {@link yfiles.input.MoveLabelInputMode}
       * in the {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameterProvider>}
       */
      labelModelParameterProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameterProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.ILabelSnapContextHelper} for the
       * {@link yfiles.graph.ILabel}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.ILabelSnapContextHelper} implementations can be used to provide custom {@link yfiles.input.SnapLine snap lines} and {@link yfiles.input.SnapResult snap results}
       * based on the {@link yfiles.graph.ILabel}s this instance has been created for. This interface is mainly used by the label's {@link yfiles.input.IPositionHandler}
       * to collect snap results during a drag gesture for the label. This is a convenient alternative to the direct usage of the
       * {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.ILabel,yfiles.input.ILabelSnapContextHelper>}
       */
      labelSnapContextHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.ILabel,yfiles.input.ILabelSnapContextHelper>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that helps in {@link yfiles.graph.ILookupDecorator#addLookup decorating} the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.IEdge} instances that are
     * contained in an {@link yfiles.graph.IGraph} which are managed by an {@link yfiles.graph.ILookupDecorator}.
     * <p>
     * This decorator can be used to conveniently {@link yfiles.graph.ILookupDecorator decorate the lookup} of {@link yfiles.graph.IEdge}s in an {@link yfiles.graph.IGraph}. It
     * serves as a factory for predefined {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} instances that are specifically useful for the {@link yfiles.graph.IEdge} type. Note that
     * the list of possible decorations that are available using this class is not exhaustive in any way. This is merely a way
     * to provide access to those interfaces that are more commonly being decorated by the developer.
     * </p>
     * @see yfiles.graph.GraphDecorator
     * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>
     * @class
     * @final
     */
    export interface EdgeDecorator extends Object{}
    export class EdgeDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.graph.EdgeDecorator} class.
       * @param decorator The decorator to use, which can be <code>null</code>.
       */
      constructor(decorator:yfiles.graph.ILookupDecorator);
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.IEdge}s that can be used to decorate <code>TInterface</code> types of the edges.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean):yfiles.graph.LookupDecorator<yfiles.graph.IEdge,TInterface>;
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.IEdge}s that can be used to decorate <code>TInterface</code> types of the edges.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param {Object} options The parameters to pass.
       * @param options.type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(options:{type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean}):yfiles.graph.LookupDecorator<yfiles.graph.IEdge,TInterface>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} type for the
       * {@link yfiles.graph.IEdge}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} implementations can be used to change the set of possible
       * {@link yfiles.input.IPortCandidate}s that are available for an existing edge in the {@link yfiles.graph.IGraph} if the user
       * interactively tries to {@link yfiles.graph.IGraph#setEdgePorts change the ports} of an {@link yfiles.graph.IEdge} in a {@link yfiles.view.GraphComponent}. This interface is
       * mainly used by the {@link yfiles.input.PortRelocationHandle}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IEdgeReconnectionPortCandidateProvider>}
       */
      edgeReconnectionPortCandidateProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IEdgeReconnectionPortCandidateProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPortCandidateProvider} for the
       * {@link yfiles.graph.IEdge}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPortCandidateProvider} implementations can be used to present the user with a set of
       * {@link yfiles.input.IPortCandidate}s that newly created edges can use as their {@link yfiles.graph.IEdge#sourcePort source} and {@link yfiles.graph.IEdge#targetPort target} {@link yfiles.graph.IPort}s. This
       * interface is mainly used by the {@link yfiles.input.CreateEdgeInputMode} mode of {@link yfiles.input.GraphEditorInputMode} if {@link yfiles.input.CreateEdgeInputMode#allowEdgeToEdgeConnections} is
       * enabled. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by
       * the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IPortCandidateProvider>}
       */
      portCandidateProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IPortCandidateProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IEdgePortHandleProvider} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.input.IEdgePortHandleProvider} implementations can be used provide interactive draggable handles for the
       * source and target end points of an edge. This interface is mainly used by the {@link yfiles.input.HandleInputMode} in the {@link yfiles.input.GraphEditorInputMode}
       * and by the implementations that work with {@link yfiles.input.OrthogonalEdgeEditingContext}. Implementations may decide, e.g.
       * whether to yield {@link yfiles.input.IHandle} implementations that either move the edge to another port(-{@link yfiles.input.IPortCandidate candidate}) using {@link yfiles.graph.IGraph#setEdgePorts}
       * or whether to move the port the edge is currently connected to itself, instead. This is a convenient alternative to the
       * direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @see yfiles.input.PortRelocationHandle
       * @see yfiles.input.IHandle
       * @see yfiles.input.IHandleProvider
       * @see yfiles.input.OrthogonalEdgeEditingContext
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IEdgePortHandleProvider>}
       */
      edgePortHandleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IEdgePortHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IOrthogonalEdgeHelper} type for the
       * {@link yfiles.graph.IEdge}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IOrthogonalEdgeHelper} implementations can be used to change the orthogonal edge editing behavior in
       * the context of {@link yfiles.input.OrthogonalEdgeEditingContext}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IOrthogonalEdgeHelper>}
       */
      orthogonalEdgeHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IOrthogonalEdgeHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IEdgeSnapResultProvider} type for the
       * {@link yfiles.graph.IEdge}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IEdgeSnapResultProvider} implementations can be used to change the snapping behavior of edges while
       * they are being moved in the context of a {@link yfiles.input.GraphSnapContext}. This is a convenient alternative to the direct
       * usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IEdgeSnapResultProvider>}
       */
      edgeSnapResultProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IEdgeSnapResultProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IEditLabelHelper} type for the
       * {@link yfiles.graph.IEdge}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IEditLabelHelper} implementations can be used to change or customize interactive label editing and
       * label creation. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode} which uses the callbacks from the
       * implementation to determine the label to add or edit, as well as to customize the {@link yfiles.input.TextEditorInputMode}
       * appearance for the upcoming edit. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IEditLabelHelper>}
       */
      editLabelHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IEditLabelHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IBendCreator} type for the
       * {@link yfiles.graph.IEdge}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IBendCreator} implementations can be used to influence the interactive creation of new
       * {@link yfiles.graph.IBend}s. This interface is mainly used by the {@link yfiles.input.CreateBendInputMode} in
       * {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IBendCreator>}
       */
      bendCreatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IBendCreator>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IBendSelectionTester} type for the
       * {@link yfiles.graph.IEdge}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IBendSelectionTester} implementations can be used to influence hit test behavior for bends on edges.
       * This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode} and {@link yfiles.input.CreateBendInputMode} to determine
       * whether an existing bend has been hit. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IBendSelectionTester>}
       */
      bendSelectionTesterDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IBendSelectionTester>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.ISnapLineProvider} for the
       * {@link yfiles.graph.IEdge}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.ISnapLineProvider} implementations can be used to provide custom {@link yfiles.input.OrthogonalSnapLine}s
       * based on the {@link yfiles.graph.IEdge}s this instance has been created for. This could e.g. be a horizontal {@link yfiles.input.OrthogonalSnapLine}
       * vertically centered between two horizontal segments of one of these edges. This interface is mainly used by the {@link yfiles.input.GraphSnapContext}
       * to collect all available {@link yfiles.input.OrthogonalSnapLine}s of orthogonal edge segments. This is a convenient alternative
       * to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.ISnapLineProvider>}
       */
      snapLineProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.ISnapLineProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPositionHandler} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPositionHandler} implementations can be used to restrict or enhance the way the user moves elements
       * interactively in the editor. This interface is mainly used by the {@link yfiles.input.MoveInputMode}. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IPositionHandler>}
       */
      positionHandlerDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IPositionHandler>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IClipboardHelper} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IClipboardHelper} implementations can be used to customize the way the clipboard operations are
       * performed by {@link yfiles.graph.GraphClipboard}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.graph.IClipboardHelper>}
       */
      clipboardHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.graph.IClipboardHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IMarqueeTestable} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
       * performed on the items in the editor. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode}. This is a
       * convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IMarqueeTestable>}
       */
      marqueeTestableDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IMarqueeTestable>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IMementoSupport} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IMementoSupport} implementations can be used to make the {@link yfiles.graph.UndoEngine} aware of changes to
       * data that is associated with the model items. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.graph.IMementoSupport>}
       */
      mementoSupportDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.graph.IMementoSupport>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.ISelectionIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.ISelectionIndicatorInstaller} implementations can be used change the way the selection of an item is
       * represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.SelectionIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.view.ISelectionIndicatorInstaller>}
       */
      selectionDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.view.ISelectionIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IHighlightIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an item
       * is represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.HighlightIndicatorManager.<T>} in the
       * {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.view.IHighlightIndicatorInstaller>}
       */
      highlightDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.view.IHighlightIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IFocusIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IFocusIndicatorInstaller} implementations can be used change the way the focus of an item is
       * represented in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.view.FocusIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.view.IFocusIndicatorInstaller>}
       */
      focusIndicatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.view.IFocusIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandleProvider} implementations provide interactive draggable handles for the user to change the
       * geometry or other aspects of items in the {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in the {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.input.IHandleProvider>}
       */
      handleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.input.IHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IObstacleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.view.IObstacleProvider} implementations provide information about obstacles that will be considered by {@link yfiles.view.GraphObstacleProvider}
       * which itself serves as a provider for geometric obstacles to {@link yfiles.view.BridgeManager} that manages the rendering of
       * bridges in edge paths. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is
       * provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @see yfiles.view.GraphObstacleProvider
       * @see yfiles.view.BridgeManager
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IEdge,yfiles.view.IObstacleProvider>}
       */
      obstacleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IEdge,yfiles.view.IObstacleProvider>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that helps in {@link yfiles.graph.ILookupDecorator#addLookup decorating} the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.IBend} instances that are
     * contained in an {@link yfiles.graph.IGraph} which are managed by an {@link yfiles.graph.ILookupDecorator}.
     * <p>
     * This decorator can be used to conveniently {@link yfiles.graph.ILookupDecorator decorate the lookup} of {@link yfiles.graph.IBend}s in an {@link yfiles.graph.IGraph}. It
     * serves as a factory for predefined {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} instances that are specifically useful for the {@link yfiles.graph.IBend} type. Note that
     * the list of possible decorations that are available using this class is not exhaustive in any way. This is merely a way
     * to provide access to those interfaces that are more commonly being decorated by the developer.
     * </p>
     * @see yfiles.graph.GraphDecorator
     * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>
     * @class
     * @final
     */
    export interface BendDecorator extends Object{}
    export class BendDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.graph.BendDecorator} class.
       * @param decorator The decorator to use, which can be <code>null</code>.
       */
      constructor(decorator:yfiles.graph.ILookupDecorator);
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.IBend}s that can be used to decorate <code>TInterface</code> types of the bends.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean):yfiles.graph.LookupDecorator<yfiles.graph.IBend,TInterface>;
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.IBend}s that can be used to decorate <code>TInterface</code> types of the bends.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param {Object} options The parameters to pass.
       * @param options.type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(options:{type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean}):yfiles.graph.LookupDecorator<yfiles.graph.IBend,TInterface>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandle} type for the
       * {@link yfiles.graph.IBend}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandle} implementations can be used to change the way the user can drag a bend interactively to
       * change its position in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.input.IHandle>}
       */
      handleDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.input.IHandle>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IBendSnapResultProvider} for the
       * {@link yfiles.graph.IBend}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IBendSnapResultProvider} implementations can be used to provide custom {@link yfiles.input.SnapResult}s based
       * on the {@link yfiles.graph.IBend}s this instance has been created for. This could e.g. be a {@link yfiles.input.SnapResult} describing
       * that the bend wants to snap vertically to the top border of a node. This interface is mainly used by the default {@link yfiles.input.IHandle}
       * for {@link yfiles.graph.IBend}s to collect {@link yfiles.input.SnapResult}s during a drag gesture for the bend. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.input.IBendSnapResultProvider>}
       */
      bendSnapResultProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.input.IBendSnapResultProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPositionHandler} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPositionHandler} implementations can be used to restrict or enhance the way the user moves elements
       * interactively in the editor. This interface is mainly used by the {@link yfiles.input.MoveInputMode}. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.input.IPositionHandler>}
       */
      positionHandlerDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.input.IPositionHandler>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IClipboardHelper} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IClipboardHelper} implementations can be used to customize the way the clipboard operations are
       * performed by {@link yfiles.graph.GraphClipboard}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.graph.IClipboardHelper>}
       */
      clipboardHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.graph.IClipboardHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IMarqueeTestable} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
       * performed on the items in the editor. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode}. This is a
       * convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.input.IMarqueeTestable>}
       */
      marqueeTestableDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.input.IMarqueeTestable>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IMementoSupport} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IMementoSupport} implementations can be used to make the {@link yfiles.graph.UndoEngine} aware of changes to
       * data that is associated with the model items. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.graph.IMementoSupport>}
       */
      mementoSupportDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.graph.IMementoSupport>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.ISelectionIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.ISelectionIndicatorInstaller} implementations can be used change the way the selection of an item is
       * represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.SelectionIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.view.ISelectionIndicatorInstaller>}
       */
      selectionDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.view.ISelectionIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IHighlightIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an item
       * is represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.HighlightIndicatorManager.<T>} in the
       * {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.view.IHighlightIndicatorInstaller>}
       */
      highlightDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.view.IHighlightIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IFocusIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IFocusIndicatorInstaller} implementations can be used change the way the focus of an item is
       * represented in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.view.FocusIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.view.IFocusIndicatorInstaller>}
       */
      focusIndicatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.view.IFocusIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandleProvider} implementations provide interactive draggable handles for the user to change the
       * geometry or other aspects of items in the {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in the {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.input.IHandleProvider>}
       */
      handleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.input.IHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IObstacleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.view.IObstacleProvider} implementations provide information about obstacles that will be considered by {@link yfiles.view.GraphObstacleProvider}
       * which itself serves as a provider for geometric obstacles to {@link yfiles.view.BridgeManager} that manages the rendering of
       * bridges in edge paths. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is
       * provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @see yfiles.view.GraphObstacleProvider
       * @see yfiles.view.BridgeManager
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IBend,yfiles.view.IObstacleProvider>}
       */
      obstacleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IBend,yfiles.view.IObstacleProvider>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that helps in {@link yfiles.graph.ILookupDecorator#addLookup decorating} the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.IPort} instances that are
     * contained in an {@link yfiles.graph.IGraph} which are managed by an {@link yfiles.graph.ILookupDecorator}.
     * <p>
     * This decorator can be used to conveniently {@link yfiles.graph.ILookupDecorator decorate the lookup} of {@link yfiles.graph.IPort}s in an {@link yfiles.graph.IGraph}. It
     * serves as a factory for predefined {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} instances that are specifically useful for the {@link yfiles.graph.IPort} type. Note that
     * the list of possible decorations that are available using this class is not exhaustive in any way. This is merely a way
     * to provide access to those interfaces that are more commonly being decorated by the developer.
     * </p>
     * @see yfiles.graph.GraphDecorator
     * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>
     * @class
     * @final
     */
    export interface PortDecorator extends Object{}
    export class PortDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.graph.PortDecorator} class.
       * @param decorator The decorator to use, which can be <code>null</code>.
       */
      constructor(decorator:yfiles.graph.ILookupDecorator);
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.IPort}s that can be used to decorate <code>TInterface</code> types of the ports.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean):yfiles.graph.LookupDecorator<yfiles.graph.IPort,TInterface>;
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.IPort}s that can be used to decorate <code>TInterface</code> types of the ports.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param {Object} options The parameters to pass.
       * @param options.type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(options:{type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean}):yfiles.graph.LookupDecorator<yfiles.graph.IPort,TInterface>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandle} type for the
       * {@link yfiles.graph.IPort}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandle} implementations can be used to change the way the user can drag a port interactively to
       * change its position in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in {@link yfiles.input.GraphEditorInputMode} which obtains the {@link yfiles.input.IHandle} from the {@link yfiles.graph.IPort} by querying the
       * {@link yfiles.input.IHandleProvider} from it, which in turn will query the {@link yfiles.input.IHandle} from itself. Alternatively,
       * selected nodes can put the {@link yfiles.input.IHandle} of their ports into their {@link yfiles.input.IHandleProvider} implementation.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.input.IHandle>}
       */
      handleDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.input.IHandle>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.styles.IEdgePathCropper} type for the
       * {@link yfiles.graph.IPort}s of the graph this instance has been created for.
       * <p>
       * Custom {@link yfiles.styles.IEdgePathCropper} implementations can be used to change the way the path of {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>} implementations
       * calculate the {@link yfiles.geometry.GeneralPath} of the visual representation of an {@link yfiles.graph.IEdge} in a
       * {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>} to find the intersection of an edge with the bounds of
       * the adjacent node to properly crop the edge path. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.styles.IEdgePathCropper>}
       */
      edgePathCropperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.styles.IEdgePathCropper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.ISnapLineProvider} for the
       * {@link yfiles.graph.IPort}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.ISnapLineProvider} implementations can be used to provide custom {@link yfiles.input.OrthogonalSnapLine}s
       * based on the {@link yfiles.graph.IPort}s this instance has been created for. This could e.g. be a {@link yfiles.input.OrthogonalSnapLine}
       * with a horizontal or vertical offset to the port's location. This interface is mainly used by the {@link yfiles.input.GraphSnapContext}
       * to collect all available {@link yfiles.input.OrthogonalSnapLine}s. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.input.ISnapLineProvider>}
       */
      snapLineProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.input.ISnapLineProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPortSnapResultProvider} for the
       * {@link yfiles.graph.IPort}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPortSnapResultProvider} implementations can be used to provide custom {@link yfiles.input.SnapResult}s based
       * on the {@link yfiles.graph.IPort}s this instance has been created for. This could e.g. be a {@link yfiles.input.SnapResult} describing
       * that the port wants to snap to a grid point. This interface is mainly used by the {@link yfiles.input.PortLocationModelParameterHandle}
       * to collect {@link yfiles.input.SnapResult}s during a drag gesture for the port. This is a convenient alternative to the direct
       * usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.input.IPortSnapResultProvider>}
       */
      portSnapResultProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.input.IPortSnapResultProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPositionHandler} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPositionHandler} implementations can be used to restrict or enhance the way the user moves elements
       * interactively in the editor. This interface is mainly used by the {@link yfiles.input.MoveInputMode}. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.input.IPositionHandler>}
       */
      positionHandlerDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.input.IPositionHandler>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IClipboardHelper} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IClipboardHelper} implementations can be used to customize the way the clipboard operations are
       * performed by {@link yfiles.graph.GraphClipboard}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.graph.IClipboardHelper>}
       */
      clipboardHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.graph.IClipboardHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IMarqueeTestable} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
       * performed on the items in the editor. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode}. This is a
       * convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.input.IMarqueeTestable>}
       */
      marqueeTestableDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.input.IMarqueeTestable>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IMementoSupport} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IMementoSupport} implementations can be used to make the {@link yfiles.graph.UndoEngine} aware of changes to
       * data that is associated with the model items. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.graph.IMementoSupport>}
       */
      mementoSupportDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.graph.IMementoSupport>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.ISelectionIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.ISelectionIndicatorInstaller} implementations can be used change the way the selection of an item is
       * represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.SelectionIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.view.ISelectionIndicatorInstaller>}
       */
      selectionDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.view.ISelectionIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IHighlightIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an item
       * is represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.HighlightIndicatorManager.<T>} in the
       * {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.view.IHighlightIndicatorInstaller>}
       */
      highlightDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.view.IHighlightIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IFocusIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IFocusIndicatorInstaller} implementations can be used change the way the focus of an item is
       * represented in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.view.FocusIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.view.IFocusIndicatorInstaller>}
       */
      focusIndicatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.view.IFocusIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandleProvider} implementations provide interactive draggable handles for the user to change the
       * geometry or other aspects of items in the {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in the {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.input.IHandleProvider>}
       */
      handleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.input.IHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IObstacleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.view.IObstacleProvider} implementations provide information about obstacles that will be considered by {@link yfiles.view.GraphObstacleProvider}
       * which itself serves as a provider for geometric obstacles to {@link yfiles.view.BridgeManager} that manages the rendering of
       * bridges in edge paths. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is
       * provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @see yfiles.view.GraphObstacleProvider
       * @see yfiles.view.BridgeManager
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.IPort,yfiles.view.IObstacleProvider>}
       */
      obstacleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.IPort,yfiles.view.IObstacleProvider>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that helps in {@link yfiles.graph.ILookupDecorator#addLookup decorating} the {@link yfiles.graph.ILookup#lookup} method of {@link yfiles.graph.INode} instances that are
     * contained in an {@link yfiles.graph.IGraph} which are managed by an {@link yfiles.graph.ILookupDecorator}.
     * <p>
     * This decorator can be used to conveniently {@link yfiles.graph.ILookupDecorator decorate the lookup} of {@link yfiles.graph.INode}s in an {@link yfiles.graph.IGraph}. It
     * serves as a factory for predefined {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} instances that are specifically useful for the {@link yfiles.graph.INode} type. Note that
     * the list of possible decorations that are available using this class is not exhaustive in any way. This is merely a way
     * to provide access to those interfaces that are more commonly being decorated by the developer.
     * </p>
     * @see yfiles.graph.GraphDecorator
     * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>
     * @class
     * @final
     */
    export interface NodeDecorator extends Object{}
    export class NodeDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.graph.NodeDecorator} class.
       * @param decorator The decorator to use, which can be <code>null</code>.
       */
      constructor(decorator:yfiles.graph.ILookupDecorator);
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.INode}s that can be used to decorate <code>TInterface</code> types of the nodes.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean):yfiles.graph.LookupDecorator<yfiles.graph.INode,TInterface>;
      /**
       * Creates a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} for {@link yfiles.graph.INode}s that can be used to decorate <code>TInterface</code> types of the nodes.
       * Type parameter <code>TInterface</code>: The type of the interface that should be decorated of the instances' {@link yfiles.graph.ILookup#lookup}.
       * @param {Object} options The parameters to pass.
       * @param options.type The type of the interface that should be decorated of the instances' {@link #lookup}.
       * @param [options.decorateNulls=true] if set to <code>true</code> <code>null</code> values will be decorated, otherwise not.
       * @param [options.nullIsFallback=false] if set to <code>true</code> <code>null</code> values will be treated as fallback values, otherwise they will be used as the final result.
       * @returns A new {@link } for the specified interface.
       * @template TInterface
       */
      getDecoratorFor<TInterface>(options:{type:yfiles.lang.Class,decorateNulls?:boolean,nullIsFallback?:boolean}):yfiles.graph.LookupDecorator<yfiles.graph.INode,TInterface>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IReshapeHandleProvider} type for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IReshapeHandleProvider} implementations can be used to change the number and type of
       * {@link yfiles.input.IHandle}s that can be used by the user to reshape the {@link yfiles.graph.INode}s in interactively in the editor.
       * This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode} which passes the {@link yfiles.input.IHandle}s to the
       * {@link yfiles.input.HandleInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.IReshapeHandleProvider>}
       */
      reshapeHandleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.IReshapeHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IEditLabelHelper} type for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IEditLabelHelper} implementations can be used to change or customize interactive label editing and
       * label creation. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode} which uses the callbacks from the
       * implementation to determine the label to add or edit, as well as to customize the {@link yfiles.input.TextEditorInputMode}
       * appearance for the upcoming edit. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.IEditLabelHelper>}
       */
      editLabelHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.IEditLabelHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.styles.IShapeGeometry} type for the type
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.styles.IShapeGeometry} implementations can be used to change the way different code parts interpret the {@link yfiles.graph.INode#layout} of
       * an {@link yfiles.graph.INode}. This interface is mainly used by the {@link yfiles.styles.DefaultEdgePathCropper}, and a custom {@link yfiles.input.IPortCandidateProvider}
       * that can be obtained from {@link yfiles.input.IPortCandidateProvider#fromShapeGeometry}. Note that decorating the {@link yfiles.styles.IShapeGeometry}
       * for an {@link yfiles.graph.INode} will not change the result of a call to {@link yfiles.styles.INodeStyleRenderer#getShapeGeometry} of
       * the {@link yfiles.graph.INode#style}. So standard hit tests will not be influenced by such a change. This is a convenient alternative to the direct
       * usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.styles.IShapeGeometry>}
       */
      shapeGeometryDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.styles.IShapeGeometry>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.INodeSizeConstraintProvider} type for the type
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.INodeSizeConstraintProvider} implementations can be used to add constraints to the size of a node.
       * These constraints are used by the default {@link yfiles.input.IReshapeHandleProvider} implementation for nodes to enforce size
       * constraints for interactive resize gestures. Also {@link yfiles.input.GraphEditorInputMode} uses this interface to constrain the
       * size of a node whenever its labels are being edited interactively. This is a convenient alternative to the direct usage
       * of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.INodeSizeConstraintProvider>}
       */
      sizeConstraintProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.INodeSizeConstraintProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IGroupBoundsCalculator} type for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IGroupBoundsCalculator} implementations can be used to change the way the bounds of {@link yfiles.graph.IGraph#createGroupNode group nodes} are
       * being calculated if the geometry of child nodes is changed by the user. This is a convenient alternative to the direct
       * usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.graph.IGroupBoundsCalculator>}
       */
      groupBoundsCalculatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.graph.IGroupBoundsCalculator>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.INodeInsetsProvider} (tied to the {@link yfiles.graph.INode} type parameter) for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.INodeInsetsProvider} implementations can be used to change the way the bounds of {@link yfiles.graph.IGraph#createGroupNode group nodes} are
       * being calculated if the geometry of child nodes is changed by the user. This interface will be used if there is no
       * custom {@link yfiles.graph.IGroupBoundsCalculator} associated with the nodes (see {@link yfiles.graph.NodeDecorator#groupBoundsCalculatorDecorator}). This is a convenient alternative to the
       * direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}. Note that many {@link yfiles.styles.INodeStyle} implementations provide an <code>Insets</code> property whose
       * value will be reported by a {@link yfiles.input.INodeInsetsProvider} implementation that is provided by the corresponding
       * {@link yfiles.styles.INodeStyleRenderer}. Using this property instead of decorating the lookup of the {@link yfiles.graph.INode} is thus
       * often simpler.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.INodeInsetsProvider>}
       */
      insetsProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.INodeInsetsProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPortCandidateProvider} for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPortCandidateProvider} implementations can be used to present the user with a set of
       * {@link yfiles.input.IPortCandidate}s that newly created edges can use as their {@link yfiles.graph.IEdge#sourcePort source} and {@link yfiles.graph.IEdge#targetPort target} {@link yfiles.graph.IPort}s. This
       * interface is mainly used by the {@link yfiles.input.CreateEdgeInputMode} mode of {@link yfiles.input.GraphEditorInputMode}. Also the
       * default implementation of {@link yfiles.input.IEdgeReconnectionPortCandidateProvider} will use this interface to determine the
       * candidates that are available if the user interactively moves the ports using {@link yfiles.input.PortRelocationHandle} (see {@link yfiles.graph.PortDecorator#handleDecorator}).
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.IPortCandidateProvider>}
       */
      portCandidateProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.IPortCandidateProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.ISnapLineProvider} for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.ISnapLineProvider} implementations can be used to provide custom {@link yfiles.input.OrthogonalSnapLine}s
       * based on the {@link yfiles.graph.INode}s this instance has been created for. This could e.g. be a horizontal {@link yfiles.input.OrthogonalSnapLine}
       * at the vertical center of the node to which other nodes should snap with their top border. This interface is mainly used
       * by the {@link yfiles.input.GraphSnapContext} to collect all available {@link yfiles.input.OrthogonalSnapLine}s. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.ISnapLineProvider>}
       */
      snapLineProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.ISnapLineProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.INodeSnapResultProvider} for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.INodeSnapResultProvider} implementations can be used to provide custom {@link yfiles.input.SnapResult}s based
       * on the {@link yfiles.graph.INode}s this instance has been created for. This could e.g. be a {@link yfiles.input.SnapResult} describing
       * that the top left corner of the node wants to snap to a grid point. This interface is mainly used by the default {@link yfiles.input.IPositionHandler}
       * for {@link yfiles.graph.INode}s to collect {@link yfiles.input.SnapResult}s during a drag gesture for the node. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.INodeSnapResultProvider>}
       */
      nodeSnapResultProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.INodeSnapResultProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.ISnapLineProvider} for the
       * {@link yfiles.graph.INode}s this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.INodeReshapeSnapResultProvider} implementations can be used to provide custom
       * {@link yfiles.input.SnapResult}s based on the {@link yfiles.graph.INode}s this instance has been created for. This could e.g. be a {@link yfiles.input.SnapResult}
       * describing that the top left corner of the node wants to snap to a grid point. This interface is mainly used by the
       * default {@link yfiles.input.IReshapeHandler} for {@link yfiles.graph.INode}s to collect {@link yfiles.input.SnapResult}s during a resize gesture
       * for the node. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided
       * by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.INodeReshapeSnapResultProvider>}
       */
      nodeReshapeSnapResultProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.INodeReshapeSnapResultProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IPositionHandler} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IPositionHandler} implementations can be used to restrict or enhance the way the user moves elements
       * interactively in the editor. This interface is mainly used by the {@link yfiles.input.MoveInputMode}. This is a convenient
       * alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.IPositionHandler>}
       */
      positionHandlerDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.IPositionHandler>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IClipboardHelper} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IClipboardHelper} implementations can be used to customize the way the clipboard operations are
       * performed by {@link yfiles.graph.GraphClipboard}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.graph.IClipboardHelper>}
       */
      clipboardHelperDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.graph.IClipboardHelper>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IMarqueeTestable} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IMarqueeTestable} implementations can be used to change the way tests for marquee inclusions are
       * performed on the items in the editor. This interface is mainly used by the {@link yfiles.input.GraphEditorInputMode}. This is a
       * convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.IMarqueeTestable>}
       */
      marqueeTestableDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.IMarqueeTestable>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.graph.IMementoSupport} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.graph.IMementoSupport} implementations can be used to make the {@link yfiles.graph.UndoEngine} aware of changes to
       * data that is associated with the model items. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.graph.IMementoSupport>}
       */
      mementoSupportDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.graph.IMementoSupport>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.ISelectionIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.ISelectionIndicatorInstaller} implementations can be used change the way the selection of an item is
       * represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.SelectionIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.view.ISelectionIndicatorInstaller>}
       */
      selectionDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.view.ISelectionIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IHighlightIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IHighlightIndicatorInstaller} implementations can be used change the way the highlighting of an item
       * is represented in a {@link yfiles.view.CanvasComponent}. This interface is mainly used by the {@link yfiles.view.HighlightIndicatorManager.<T>} in the
       * {@link yfiles.view.GraphComponent}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.view.IHighlightIndicatorInstaller>}
       */
      highlightDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.view.IHighlightIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IFocusIndicatorInstaller} type for the type of items this instance has been
       * created for.
       * <p>
       * Custom {@link yfiles.view.IFocusIndicatorInstaller} implementations can be used change the way the focus of an item is
       * represented in a {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.view.FocusIndicatorManager.<T>} in the {@link yfiles.view.GraphComponent}.
       * This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is provided by the
       * {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.view.IFocusIndicatorInstaller>}
       */
      focusIndicatorDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.view.IFocusIndicatorInstaller>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.input.IHandleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.input.IHandleProvider} implementations provide interactive draggable handles for the user to change the
       * geometry or other aspects of items in the {@link yfiles.view.GraphComponent}. This interface is mainly used by the {@link yfiles.input.HandleInputMode}
       * in the {@link yfiles.input.GraphEditorInputMode}. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator}
       * that is provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.input.IHandleProvider>}
       */
      handleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.input.IHandleProvider>;
      /**
       * Gets a {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} that decorates the {@link yfiles.view.IObstacleProvider} type for the type of items this instance has been created for.
       * <p>
       * Custom {@link yfiles.view.IObstacleProvider} implementations provide information about obstacles that will be considered by {@link yfiles.view.GraphObstacleProvider}
       * which itself serves as a provider for geometric obstacles to {@link yfiles.view.BridgeManager} that manages the rendering of
       * bridges in edge paths. This is a convenient alternative to the direct usage of the {@link yfiles.graph.ILookupDecorator} that is
       * provided by the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup#lookup}.
       * </p>
       * @see yfiles.view.GraphObstacleProvider
       * @see yfiles.view.BridgeManager
       * @type {yfiles.graph.LookupDecorator.<yfiles.graph.INode,yfiles.view.IObstacleProvider>}
       */
      obstacleProviderDecorator:yfiles.graph.LookupDecorator<yfiles.graph.INode,yfiles.view.IObstacleProvider>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A convenience class that simplifies using an {@link yfiles.graph.ILookupDecorator}.
     * <p>
     * This class provides convenience methods that help in performing common tasks related to decorating the {@link yfiles.graph.ILookup#lookup}
     * method for a certain <code>TDecoratedType</code>.
     * </p>
     * Type parameter <code>TDecoratedType</code>: The type of the instances whose {@link yfiles.graph.ILookup#lookup} behavior should be modified.
     * Type parameter <code>TInterface</code>: The interface or type that clients will use during the {@link yfiles.graph.ILookup#lookup} call. Note that this is normally an
     * interface and not the implementation of the interface since client code will use the interface for the query.
     * @see yfiles.graph.ILookup
     * @see yfiles.graph.GraphDecorator
     * @class
     * @template TDecoratedType,TInterface
     * @final
     */
    export interface LookupDecorator<TDecoratedType,TInterface> extends Object{}
    export class LookupDecorator<TDecoratedType,TInterface> {
      /**
       * Initializes a new instance of the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>} class.
       * @param decoratedType The type of the instances whose {@link #lookup} behavior should be modified.
       * @param interfaceType The interface or type that clients will use during the {@link #lookup} call. Note that this is normally an
       * interface and not the implementation of the interface since client code will use the interface for the query.
       * @param decorator The decorator instance that will be used to {@link #addLookup extend the Lookup mechanism}.
       * @param decorateNulls The initial value of the {@link #decorateNulls} property.
       * @param nullIsFallback The initial value of the {@link #nullIsFallback} property.
       */
      constructor(decoratedType:yfiles.lang.Class,interfaceType:yfiles.lang.Class,decorator:yfiles.graph.ILookupDecorator,decorateNulls:boolean,nullIsFallback:boolean);
      /**
       * Simple helper method that tries to add the provided chain link to the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#decorator}.
       * @param lookup The link to add.
       * @returns The <code>lookup</code> that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if the decoration was unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      addChainLink(lookup:yfiles.graph.IContextLookupChainLink):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for all <code>TDecoratedType</code> items matching a <code>predicate</code> to return <code>null</code> if <code>TInterface</code> is queried during the
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to temporarily or permanently disable a certain behavior for all items. To do this only for items
       * matching a predicate, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#hideImplementation} instead.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param [predicate=null] The predicate that determines for which items the implementation shall be hidden.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      hideImplementation(predicate?:(obj:TDecoratedType)=>boolean):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for items matching a <code>predicate</code> with a <code>factory</code> that creates a <code>TInterface</code> instance if <code>TInterface</code> is queried during the
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of all items matching a certain predicate where the returned instance either
       * depends on the item or uses it in some way.
       * </p>
       * <p>
       * To adjust the lookup of all items in this manner, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setFactory} instead. To adjust the lookup of a single item, use
       * {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * Note that depending on the value of the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#nullIsFallback} property a <code>null</code> return value from the <code>factory</code> will be either returned directly,
       * or interpreted as a hint to return a fallback value which is obtained by querying the remainder of the lookup chain.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param predicate The predicate that determines for which items the factory shall be called.
       * @param factory The factory that is queried to create the result.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#nullIsFallback
       */
      setFactory(predicate:(obj:TDecoratedType)=>boolean,factory:(arg:TDecoratedType)=>TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for all <code>TDecoratedType</code> items, so that <code>factory</code> is used to create a <code>TInterface</code> instance if <code>TInterface</code> is queried during the
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of all items where the returned instance either depends on the item or uses it in
       * some way.
       * </p>
       * <p>
       * To only adjust the lookup of items matching a predicate in this manner, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setFactory} instead. To adjust the lookup of
       * a single item, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * Note that depending on the value of the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#nullIsFallback} property a <code>null</code> return value from the <code>factory</code> will be either returned directly,
       * or interpreted as a hint to return a fallback value which is obtained by querying the remainder of the lookup chain.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param factory The factory that is queried to create the result.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       * @see yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#nullIsFallback
       */
      setFactory(factory:(arg:TDecoratedType)=>TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for all items matching a <code>predicate</code> with a fixed <code>TInterface</code> <code>implementation</code> if <code>TInterface</code> is queried on an item's
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of all items matching a certain predicate in the same way.
       * </p>
       * <p>
       * To adjust the lookup of only a single item, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation} instead. To adjust the lookup of all items, use
       * {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param predicate A predicate that determines which items are affected.
       * @param implementation The implementation to return if the <code>predicate</code> matches and <code>TInterface</code> is queried.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      setImplementation(predicate:(obj:TDecoratedType)=>boolean,implementation:TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for a single item with a fixed <code>TInterface</code> <code>implementation</code> if <code>TInterface</code> is queried on the <code>item</code>'s
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of a single item only.
       * </p>
       * <p>
       * To adjust the lookup of all items matching a predicate, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation} instead. To adjust the lookup of all
       * items, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param item The item for which the implementation shall be returned.
       * @param implementation The implementation to return if <code>item</code> is queried for <code>TInterface</code>.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      setImplementation(item:TDecoratedType,implementation:TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain}, so that <code>implementation</code> is always returned if <code>TInterface</code> is queried on an item's
       * {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of all items in the same way.
       * </p>
       * <p>
       * To adjust the lookup of only a single item, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation} instead. To adjust the lookup of all items matching a
       * certain predicate, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param implementation The implementation to return unconditionally.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      setImplementation(implementation:TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for all items matching a <code>predicate</code> with a <code>factory</code> callback that wraps the underlying
       * implementation if <code>TInterface</code> is queried during the {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of all items matching a certain predicate where the factory implementation may
       * delegate to the wrapped implementation in certain cases.
       * </p>
       * <p>
       * To adjust the lookup for all items in this manner, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementationWrapper} instead. To adjust the lookup of a
       * single item, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * Note that depending on the value of the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#decorateNulls} property, the factory will not not be called to decorate <code>null</code> results.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param predicate The predicate that determines for which items the wrapping shall be performed.
       * @param factory The factory that will be passed the underlying implementation and that is queried to create the result.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      setImplementationWrapper(predicate:(obj:TDecoratedType)=>boolean,factory:(arg1:TDecoratedType,arg2:TInterface)=>TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Amends the {@link yfiles.graph.IContextLookupChainLink lookup chain} for all items with a <code>factory</code> callback that wraps the underlying implementation if <code>TInterface</code> is queried
       * during the {@link yfiles.graph.ILookup#lookup}.
       * <p>
       * This can be used to modify the lookup of all items matching where the factory implementation may delegate to the wrapped
       * implementation in certain cases.
       * </p>
       * <p>
       * To adjust the lookup in this manner only for items matching a predicate, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementationWrapper} instead. To
       * adjust the lookup of a single item, use {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation}.
       * </p>
       * <p>
       * Note that depending on the value of the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#decorateNulls} property, the factory will not not be called to decorate <code>null</code> results.
       * </p>
       * <p>
       * The effects of this method call can be undone by passing the return value to
       * {@link yfiles.graph.ILookupDecorator#removeLookup}.
       * </p>
       * @param factory The factory that will be passed the underlying implementation and that is queried to create the result.
       * @returns An {@link } that has been {@link #addLookup added} to the {@link #decorator} as a result of this call, or <code>null</code> if decoration was
       * unsuccessful.
       * @see yfiles.graph.ILookupDecorator#addLookup
       */
      setImplementationWrapper(factory:(arg1:TDecoratedType,arg2:TInterface)=>TInterface):yfiles.graph.IContextLookupChainLink;
      /**
       * Gets the {@link yfiles.graph.ILookupDecorator} instance that is used.
       * <p>
       * This can be <code>null</code>. In this case all methods of this class do nothing.
       * </p>
       * @type {yfiles.graph.ILookupDecorator}
       */
      decorator:yfiles.graph.ILookupDecorator;
      /**
       * Gets or sets a value indicating whether wrapping implementations will wrap <code>null</code> values from the rest of the lookup chain
       * or those values will simply be returned instead.
       * <p>
       * If this is <code>true</code>, then the {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementationWrapper SetImplementationWrapper} methods will not wrap <code>null</code> values from the rest of the lookup chain
       * and the result of the call to {@link yfiles.graph.ILookup#lookup} will simply be <code>null</code>. If this is <code>false</code>, the wrapper will get <code>null</code>
       * as the implementation to wrap and should handle this accordingly.
       * </p>
       * @type {boolean}
       */
      decorateNulls:boolean;
      /**
       * Gets or sets a value indicating whether a <code>null</code> result value of the methods {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation} and {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setFactory} should be
       * interpreted as taking the fallback value, or actually return <code>null</code>.
       * <p>
       * If this is <code>true</code>, then <code>null</code> values returned from the implementation or factory set with {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setImplementation} or {@link yfiles.graph.LookupDecorator.<TDecoratedType,TInterface>#setFactory}
       * are not returned, but rather interpreted as a hint to query the rest of the lookup chain. If this is <code>false</code>, then <code>null</code>
       * values are returned as is.
       * </p>
       * @type {boolean}
       */
      nullIsFallback:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides less frequently used methods for analyzing and managing grouped graphs.
     * <p>
     * An instance of this class which is configured for a given {@link yfiles.graph.IGraph} instance can be obtained via {@link yfiles.graph.IGraph#groupingSupport}.
     * </p>
     * @class
     */
    export interface GroupingSupport extends Object{}
    export class GroupingSupport {
      /**
       * Creates a new instance which operates on the given <code>graph</code>.
       * @param graph The graph to use for operations on this instance.
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Calculates the minimum area to enclose by the given group node with respect to its {@link yfiles.graph.IGroupBoundsCalculator}.
       * @param groupNode The group node to calculate the minimum enclosed area of.
       * @returns The area to enclose for the given group node. This is {@link #EMPTY} if the node is not a group node, or if
       * it hasn't any children.
       */
      calculateMinimumEnclosedArea(groupNode:yfiles.graph.INode):yfiles.geometry.Rect;
      /**
       * Enlarges all group nodes in the graph in an interactive scenario, so that the {@link yfiles.graph.GroupingSupport#calculateMinimumEnclosedArea minimum enclosed area} is respected.
       * <p>
       * This method should be used in an interactive editing scenario. It records an {@link yfiles.graph.ICompoundEdit} to support
       * undoability and uses the {@link yfiles.input.IReshapeHandler} for the resizing of the nodes. That way, e.g. orthogonally edited
       * edges will be properly reshaped.
       * </p>
       * @param context The context to use for the {@link }s.
       * @see yfiles.graph.GroupingSupport#enlargeGroupNode
       */
      enlargeAllGroupNodes(context:yfiles.input.IInputModeContext):void;
      /**
       * Enlarges all group nodes in the graph so that the {@link yfiles.graph.GroupingSupport#calculateMinimumEnclosedArea minimum enclosed area} is respected.
       * @see yfiles.graph.GroupingSupport#enlargeGroupNode
       */
      enlargeAllGroupNodes():void;
      /**
       * Enlarges the group nodes in an interactive scenario, using {@link yfiles.input.IReshapeHandler} implementations of the group
       * nodes to perform the actual resizing.
       * <p>
       * This method should be used in an interactive editing scenario. It records an {@link yfiles.graph.ICompoundEdit} to support
       * undoability and uses the {@link yfiles.input.IReshapeHandler} for the resizing of the nodes. That way, e.g. orthogonally edited
       * edges will be properly reshaped.
       * </p>
       * @param context The context to use for the {@link }s.
       * @param node The node to enlarge appropriately.
       * @param ancestors if set to <code>true</code> ancestors the ancestor group nodes will be adjusted, too, if necessary.
       * @see yfiles.input.IReshapeHandler
       */
      enlargeGroupNode(context:yfiles.input.IInputModeContext,node:yfiles.graph.INode,ancestors:boolean):void;
      /**
       * Enlarges the given group node to ensure that the {@link yfiles.graph.GroupingSupport#calculateMinimumEnclosedArea minimum enclosed} area is inside of its bounds.
       * <p>
       * This method can be used to resize a group node and all of its parent group nodes to fully contain its children, e.g.
       * after a child node has changed its bounds. Note that this method will only enlarge the bounds of the group nodes, it
       * will never reduce the size of a group node.
       * </p>
       * @param groupNode The group node to resize.
       * @param ancestors if set to <code>true</code> all ancestor group nodes will be resized, too, if necessary
       * @see yfiles.graph.GroupingSupport#enlargeGroupNode
       */
      enlargeGroupNode(groupNode:yfiles.graph.INode,ancestors:boolean):void;
      /**
       * Returns all descendants of the provided node that are part of the grouped graph.
       * <p>
       * The enumeration will be top down, i.e. all of a node's ancestors will be enumerated before the respective node.
       * </p>
       * @param node The root node to get the descendants from.
       * @returns An enumeration of the children of the node at the time of this invocation in reverse DFS prefix order.
       */
      getDescendants(node:yfiles.graph.INode):yfiles.collections.IEnumerable<yfiles.graph.INode>;
      /**
       * Returns all descendants of the provided node that are part of the grouped graph.
       * <p>
       * The enumeration will be bottom up, i.e. all of a node's children will be enumerated before the respective node.
       * </p>
       * @param node The root node to get the descendants from.
       * @returns An enumeration of the children of the node at the time of this invocation in DFS postfix order.
       */
      getDescendantsBottomUp(node:yfiles.graph.INode):yfiles.collections.IEnumerable<yfiles.graph.INode>;
      /**
       * Determines the nearest common ancestor of the provided nodes in the graph.
       * @param nodes The nodes to find the nearest common ancestor of.
       * @returns 
       */
      getNearestCommonAncestor(...nodes:yfiles.graph.INode[]):yfiles.graph.INode;
      /**
       * Determines the nearest common ancestor of the provided nodes in the graph.
       * @param nodes The nodes to find the nearest common ancestor of.
       * @returns 
       */
      getNearestCommonAncestor(nodes:yfiles.collections.IEnumerable<yfiles.graph.INode>):yfiles.graph.INode;
      /**
       * Gets the path to root traversing all {@link yfiles.graph.IGraph#getParent parents} of the given item.
       * @param node The node to start.
       * @returns A list that includes the node but not the root, unless they are identical and all parents in between them.
       */
      getPathToRoot(node:yfiles.graph.INode):yfiles.collections.IList<yfiles.graph.INode>;
      /**
       * Determines whether there are group nodes in the graph.
       * <p>
       * This methods considers all {@link yfiles.graph.IGraph#isGroupNode group} nodes, that is, nodes that could have children.
       * </p>
       * @returns <code>true</code> if there are {@link #isGroupNode group}.
       */
      hasGroupNodes():boolean;
      /**
       * Determines whether <code>ancestor</code> is an ancestor of <code>node</code> in the grouped graph.
       * @param node The node to check.
       * @param ancestor The ancestor to check.
       * @returns Whether <code>ancestor</code> is an ancestor of <code>node</code>.
       */
      isDescendant(node:yfiles.graph.INode,ancestor:yfiles.graph.INode):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract {@link yfiles.graph.IGraph} implementation that delegates all of its work to an existing implementation.
     * <p>
     * Subclasses can override all of the methods to customize the behavior of this instance. Note that this instance will
     * register listeners with the wrapped graph instance, so {@link yfiles.graph.GraphWrapperBase#dispose} should be called if this
     * instance is not used any more.
     * </p>
     * @class
     * @implements {yfiles.graph.IGraph}
     */
    export interface GraphWrapperBase extends Object,yfiles.graph.IGraph{}
    export class GraphWrapperBase {
      /**
       * Creates a new instance that wraps a given {@link yfiles.graph.IGraph}.
       * <p>
       * Note that this instance will {@link yfiles.graph.GraphWrapperBase#addEventHandlers register listeners} with the <code>graph</code>, so {@link yfiles.graph.GraphWrapperBase#dispose} should be
       * called if this instance is not used any more.
       * </p>
       * @param graph The graph to wrap.
       * @protected
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * 
       * @param edge
       * @param location
       * @param [index=-1]
       * @returns 
       */
      addBend(edge:yfiles.graph.IEdge,location:yfiles.geometry.Point,index?:number):yfiles.graph.IBend;
      /**
       * Adds event handlers that propagate graph events from <code>graph</code> to listeners on this instance.
       * @param graph The graph for which event propagators should be created
       * @protected
       */
      addEventHandlers(graph:yfiles.graph.IGraph):void;
      /**
       * 
       * @param owner
       * @param text
       * @param [layoutParameter=null]
       * @param [style=null]
       * @param [preferredSize=null]
       * @param [tag=null]
       * @returns 
       */
      addLabel(owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object):yfiles.graph.ILabel;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.owner
       * @param options.text
       * @param [options.layoutParameter=null]
       * @param [options.style=null]
       * @param [options.preferredSize=null]
       * @param [options.tag=null]
       * @returns 
       */
      addLabel(options:{owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object}):yfiles.graph.ILabel;
      /**
       * 
       * @param owner
       * @param [locationParameter=null]
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      addPort(owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,style?:yfiles.styles.IPortStyle,tag?:Object):yfiles.graph.IPort;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.owner
       * @param [options.locationParameter=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      addPort(options:{owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,style?:yfiles.styles.IPortStyle,tag?:Object}):yfiles.graph.IPort;
      /**
       * 
       * @param item
       * @returns 
       */
      contains(item:yfiles.graph.IModelItem):boolean;
      /**
       * 
       * @param source
       * @param target
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createEdge(source:yfiles.graph.INode,target:yfiles.graph.INode,style?:yfiles.styles.IEdgeStyle,tag?:Object):yfiles.graph.IEdge;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.source
       * @param options.target
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createEdge(options:{source:yfiles.graph.INode,target:yfiles.graph.INode,style?:yfiles.styles.IEdgeStyle,tag?:Object}):yfiles.graph.IEdge;
      /**
       * 
       * @param sourcePort
       * @param targetPort
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createEdge(sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort,style?:yfiles.styles.IEdgeStyle,tag?:Object):yfiles.graph.IEdge;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.sourcePort
       * @param options.targetPort
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createEdge(options:{sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort,style?:yfiles.styles.IEdgeStyle,tag?:Object}):yfiles.graph.IEdge;
      /**
       * 
       * @param [parent=null]
       * @param [layout=null]
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createGroupNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null]
       * @param [options.layout=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createGroupNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * 
       * @param layout
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createNode(layout:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.layout
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createNode(options:{layout:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * 
       * @param [parent=null]
       * @param [layout=null]
       * @param [style=null]
       * @param [tag=null]
       * @returns 
       */
      createNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null]
       * @param [options.layout=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @returns 
       */
      createNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * Clears the internal state of this instance and released memory and bound listeners.
       * <p>
       * After this instance has been disposed of, it will not work as expected anymore. This implementation {@link yfiles.graph.GraphWrapperBase#removeEventHandlers removes all event handlers}
       * from the {@link yfiles.graph.GraphWrapperBase#wrappedGraph} and releases the reference.
       * </p>
       */
      dispose():void;
      /**
       * 
       * @param owner
       * @param [type=yfiles.graph.AdjacencyTypes.ALL]
       * @returns 
       */
      edgesAt(owner:yfiles.graph.IPortOwner,type?:yfiles.graph.AdjacencyTypes):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @param port
       * @param [type=yfiles.graph.AdjacencyTypes.ALL]
       * @returns 
       */
      edgesAt(port:yfiles.graph.IPort,type?:yfiles.graph.AdjacencyTypes):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @param node
       * @returns 
       */
      getChildren(node:yfiles.graph.INode):yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * Gets the lookup instance to use in {@link yfiles.graph.GraphWrapperBase#lookup}
       * @returns 
       */
      getLookup():yfiles.graph.ILookup;
      /**
       * 
       * @param node
       * @returns 
       */
      getParent(node:yfiles.graph.INode):yfiles.graph.INode;
      /**
       * 
       */
      invalidateDisplays():void;
      /**
       * 
       * @param node
       * @returns 
       */
      isGroupNode(node:yfiles.graph.INode):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addBendAddedListener BendAdded} event
       * @param evt The parameters for the event
       * @protected
       */
      onBendAdded(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addBendLocationChangedListener BendLocationChanged} event
       * @param bend The bend whose location changed
       * @param oldLocation The bend location before the change
       * @protected
       */
      onBendLocationChanged(bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addBendRemovedListener BendRemoved} event
       * @param evt The parameters for the event
       * @protected
       */
      onBendRemoved(evt:yfiles.graph.BendEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addBendTagChangedListener BendTagChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onBendTagChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addEdgeCreatedListener EdgeCreated} event
       * @param evt The parameters for the event
       * @protected
       */
      onEdgeCreated(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addEdgePortsChangedListener EdgePortsChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onEdgePortsChanged(evt:yfiles.graph.EdgeEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addEdgeRemovedListener EdgeRemoved} event
       * @param evt The parameters for the event
       * @protected
       */
      onEdgeRemoved(evt:yfiles.graph.EdgeEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addEdgeStyleChangedListener EdgeStyleChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onEdgeStyleChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addEdgeTagChangedListener EdgeTagChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onEdgeTagChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>):void;
      /**
       * Called when the {@link yfiles.graph.GraphWrapperBase#wrappedGraph} property value changes and after initialization of the field.
       * <p>
       * Delegates to {@link yfiles.graph.GraphWrapperBase#addEventHandlers} for <code>newGraph</code> and to {@link yfiles.graph.GraphWrapperBase#removeEventHandlers}
       * for <code>oldGraph</code>.
       * </p>
       * @param oldGraph the old value, which may be <code>null</code> the first time
       * @param newGraph the new value
       * @protected
       */
      onGraphChanged(oldGraph:yfiles.graph.IGraph,newGraph:yfiles.graph.IGraph):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addGraphTagChangedListener GraphTagChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onGraphTagChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addDisplaysInvalidatedListener DisplaysInvalidated} event
       * @param evt The parameters for the event
       * @protected
       */
      onInvalidateDisplays(evt:yfiles.lang.EventArgs):void;
      /**
       * Dispatches the {@link yfiles.graph.GraphWrapperBase#addIsGroupNodeChangedListener IsGroupNodeChanged} event.
       * @param evt The event args to dispatch.
       * @protected
       */
      onIsGroupNodeChanged(evt:yfiles.graph.NodeEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelAddedListener LabelAdded} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelAdded(evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelLayoutParameterChangedListener LabelLayoutParameterChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelLayoutParameterChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelPreferredSizeChangedListener LabelPreferredSizeChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelPreferredSizeChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelRemovedListener LabelRemoved} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelRemoved(evt:yfiles.graph.LabelEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelStyleChangedListener LabelStyleChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelStyleChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelTagChangedListener LabelTagChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelTagChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addLabelTextChangedListener LabelTextChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onLabelTextChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addNodeCreatedListener NodeCreated} event
       * @param evt The parameters for the event
       * @protected
       */
      onNodeCreated(evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addNodeLayoutChangedListener NodeLayoutChanged} event
       * @param node The node whose layout changed
       * @param oldLayout The node layout before the change
       * @protected
       */
      onNodeLayoutChanged(node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addNodeRemovedListener NodeRemoved} event
       * @param evt The parameters for the event
       * @protected
       */
      onNodeRemoved(evt:yfiles.graph.NodeEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addNodeStyleChangedListener NodeStyleChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onNodeStyleChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addNodeTagChangedListener NodeTagChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onNodeTagChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>):void;
      /**
       * Dispatches the {@link yfiles.graph.GraphWrapperBase#addParentChangedListener ParentChanged} event.
       * @param evt The event args to dispatch.
       * @protected
       */
      onParentChanged(evt:yfiles.graph.NodeEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addPortAddedListener PortAdded} event
       * @param evt The parameters for the event
       * @protected
       */
      onPortAdded(evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addPortLocationParameterChangedListener PortLocationParameterChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onPortLocationParameterChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addPortRemovedListener PortRemoved} event
       * @param evt The parameters for the event
       * @protected
       */
      onPortRemoved(evt:yfiles.graph.PortEventArgs):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addPortStyleChangedListener PortStyleChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onPortStyleChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>):void;
      /**
       * Raises the {@link yfiles.graph.GraphWrapperBase#addPortTagChangedListener PortTagChanged} event
       * @param evt The parameters for the event
       * @protected
       */
      onPortTagChanged(evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>):void;
      /**
       * 
       * @param item
       */
      remove(item:yfiles.graph.IModelItem):void;
      /**
       * Removes event handlers for <code>graph</code> that have been added with {@link yfiles.graph.GraphWrapperBase#addEventHandlers}.
       * @param graph The graph for which event propagators should be removed
       * @protected
       */
      removeEventHandlers(graph:yfiles.graph.IGraph):void;
      /**
       * 
       * @param bend
       * @param location
       */
      setBendLocation(bend:yfiles.graph.IBend,location:yfiles.geometry.Point):void;
      /**
       * 
       * @param edge
       * @param sourcePort
       * @param targetPort
       */
      setEdgePorts(edge:yfiles.graph.IEdge,sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):void;
      /**
       * 
       * @param node
       * @param isGroupNode
       */
      setIsGroupNode(node:yfiles.graph.INode,isGroupNode:boolean):void;
      /**
       * 
       * @param label
       * @param layoutParameter
       */
      setLabelLayoutParameter(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * 
       * @param label
       * @param preferredSize
       */
      setLabelPreferredSize(label:yfiles.graph.ILabel,preferredSize:yfiles.geometry.Size):void;
      /**
       * 
       * @param label
       * @param text
       */
      setLabelText(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Sets the lookup instance to use in {@link yfiles.graph.GraphWrapperBase#lookup}
       * @param lookup
       */
      setLookup(lookup:yfiles.graph.ILookup):void;
      /**
       * 
       * @param node
       * @param layout
       */
      setNodeLayout(node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * 
       * @param node
       * @param parent
       */
      setParent(node:yfiles.graph.INode,parent:yfiles.graph.INode):void;
      /**
       * 
       * @param port
       * @param locationParameter
       */
      setPortLocationParameter(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * 
       * @param port
       * @param style
       */
      setStyle(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):void;
      /**
       * 
       * @param edge
       * @param style
       */
      setStyle(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):void;
      /**
       * 
       * @param label
       * @param style
       */
      setStyle(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):void;
      /**
       * 
       * @param node
       * @param style
       */
      setStyle(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):void;
      /**
       * Gets the graph that has been wrapped by this instance.
       * @type {yfiles.graph.IGraph}
       */
      wrappedGraph:yfiles.graph.IGraph;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.INode>}
       */
      nodes:yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IEdge>}
       */
      edges:yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
      /**
       * 
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
      /**
       * 
       * @type {yfiles.graph.IEdgeDefaults}
       */
      edgeDefaults:yfiles.graph.IEdgeDefaults;
      /**
       * 
       * @type {yfiles.graph.INodeDefaults}
       */
      nodeDefaults:yfiles.graph.INodeDefaults;
      /**
       * 
       * @param listener
       */
      addGraphTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeGraphTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeRemovedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeRemovedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeLayoutChangedListener(listener:(source:Object,node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeLayoutChangedListener(listener:(source:Object,node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect)=>void):void;
      /**
       * 
       * @param listener
       */
      addNodeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeNodeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeRemovedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeRemovedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addEdgeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeEdgeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendRemovedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendRemovedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendLocationChangedListener(listener:(source:Object,bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendLocationChangedListener(listener:(source:Object,bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point)=>void):void;
      /**
       * 
       * @param listener
       */
      addBendTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeBendTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelPreferredSizeChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelPreferredSizeChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelLayoutParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelLayoutParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortRemovedListener(listener:(sender:Object,evt:yfiles.graph.PortEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortRemovedListener(listener:(sender:Object,evt:yfiles.graph.PortEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortLocationParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortLocationParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>)=>void):void;
      /**
       * 
       * @param listener
       */
      addPortTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      removePortTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>)=>void):void;
      /**
       * 
       * @param listener
       */
      addDisplaysInvalidatedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeDisplaysInvalidatedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * 
       * @type {yfiles.graph.IMapperRegistry}
       */
      mapperRegistry:yfiles.graph.IMapperRegistry;
      /**
       * 
       * @type {yfiles.graph.INodeDefaults}
       */
      groupNodeDefaults:yfiles.graph.INodeDefaults;
      /**
       * 
       * @param listener
       */
      addParentChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeParentChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addIsGroupNodeChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeIsGroupNodeChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * 
       * @type {Object}
       */
      tag:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * The interface used in an {@link yfiles.graph.IGraph} implementation to control the layout of {@link yfiles.graph.IEdge edges}.
     * <p>
     * This interface provides read-only access to the properties of a bend. In order to modify the state of an instance use
     * the various methods provided by the {@link yfiles.graph.IGraph} this instance belongs to. Bends are stored in {@link yfiles.collections.IListEnumerable.<T>}s, that can be
     * obtained from the {@link yfiles.graph.IEdge} implementation that owns this bend. Like all items in an <code>IGraph</code>, this item supports
     * the {@link yfiles.graph.ILookup#lookup} method that can be used to query additional aspects of the item.
     * </p>
     * <p>
     * The graph model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/graph.html The Graph Model}.
     * </p>
     * <p>
     * Using the look-up mechanism is explained in the section {@link @DGUIDE_PREFIX@/customizing_concepts_lookup.html Service Locator Pattern: Lookup}.
     * </p>
     * @interface
     * @implements {yfiles.graph.IModelItem}
     */
    export interface IBend extends Object,yfiles.graph.IModelItem{
      /**
       * Returns the edge this bend instance belongs to.
       * <p>
       * This implies that <code>Owner.Bends</code> contains this instance.
       * </p>
       * @abstract
       * @type {yfiles.graph.IEdge}
       */
      owner:yfiles.graph.IEdge;
      /**
       * Gets a view of the location of the bend in the world coordinate system.
       * <p>
       * The {@link yfiles.geometry.IPoint#x} and {@link yfiles.geometry.IPoint#y} properties describe the location of the bend in the world coordinate system. This property is a live view of
       * the location and will be updated in the future if the location changes. To keep the current state of the location,
       * callers need to create a copy of it
       * </p>
       * @see yfiles.geometry.IPoint#toPoint
       * @abstract
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Gets the index of the bend at its {@link yfiles.graph.IBend#owner}.
       * @type {number}
       */
      index?:number;
    }
    var IBend:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface used to model edges in an {@link yfiles.graph.IGraph} implementation.
     * <p>
     * This interface provides read-only access to the properties of an edge. In order to modify the state of an instance use
     * the various methods provided by the {@link yfiles.graph.IGraph} this instance belongs to. An edge connects to two ports and may
     * consist of a number of bends. Also it can have zero or more labels. This interface combines the functionality of {@link yfiles.graph.IPortOwner}
     * to get access to the ports that edges can optionally have, {@link yfiles.graph.ILabelOwner} to get access to the labels, and,
     * like all items in an <code>IGraph</code>, edges support the {@link yfiles.graph.ILookup#lookup} method inherited from the {@link yfiles.graph.IModelItem}
     * interface that can be used to query additional aspects of each instance.
     * </p>
     * <p>
     * The graph model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/graph.html The Graph Model}.
     * </p>
     * <p>
     * Using the look-up mechanism is explained in the section {@link @DGUIDE_PREFIX@/customizing_concepts_lookup.html Service Locator Pattern: Lookup}.
     * </p>
     * @interface
     * @implements {yfiles.graph.IPortOwner}
     * @implements {yfiles.graph.ILabelOwner}
     * @implements {yfiles.graph.IModelItem}
     */
    export interface IEdge extends Object,yfiles.graph.IPortOwner,yfiles.graph.ILabelOwner,yfiles.graph.IModelItem{
      /**
       * Gets the opposite port of an {@link yfiles.graph.IEdge}.
       * @param port The port that the {@link } is connected to.
       * @returns The opposite port.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>port</code> is neither the source or target of the edge.
       */
      opposite?(port:yfiles.graph.IPort):yfiles.graph.IPort;
      /**
       * Gets the opposite port owner of an {@link yfiles.graph.IEdge}.
       * @param owner The owner of the port that the {@link } is connected to.
       * @returns The owner of the opposite port.
       * @throws {Stubs.Exceptions.ArgumentError} If <code>owner</code> is neither the source or target of the edge.
       */
      opposite?(owner:yfiles.graph.IPortOwner):yfiles.graph.IPortOwner;
      /**
       * Provides access to a collection of {@link yfiles.graph.IBend bends} that describe the geometry of this edge.
       * <p>
       * This gives access to a read-only live view of the bends, i.e. the collection can change over time, as well as the bends
       * contained in it. If a snapshot of the current state is needed, one needs to copy the collection. In order to modify the
       * bend collection of an edge, use the {@link yfiles.graph.IGraph#addBend various methods} in {@link yfiles.graph.IGraph}.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IBend>}
       */
      bends:yfiles.collections.IListEnumerable<yfiles.graph.IBend>;
      /**
       * Gets the source port instance this edge is connected to.
       * <p>
       * Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
       * the edge should be treated as such.
       * </p>
       * @abstract
       * @type {yfiles.graph.IPort}
       */
      sourcePort:yfiles.graph.IPort;
      /**
       * Gets the target port instance this edge is connected to.
       * <p>
       * Although the notion of source and target port is used for directed edges, it is still up to the client to decide whether
       * the edge should be treated as such.
       * </p>
       * @abstract
       * @type {yfiles.graph.IPort}
       */
      targetPort:yfiles.graph.IPort;
      /**
       * Returns the style that is responsible for the visual representation of this edge in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * In order to set the style on an instance, use the {@link yfiles.graph.IGraph#setStyle} method. Note that the style instance
       * associated with an edge instance may be shared between multiple edge instances and that the modification of this style
       * will result in a change of the appearance of all edges that are associated with the same style instance.
       * </p>
       * @abstract
       * @type {yfiles.styles.IEdgeStyle}
       */
      style:yfiles.styles.IEdgeStyle;
      /**
       * Gets the source node for the given edge or <code>null</code>.
       * @type {yfiles.graph.INode}
       */
      sourceNode?:yfiles.graph.INode;
      /**
       * Gets the target node for the given edge or <code>null</code>.
       * @type {yfiles.graph.INode}
       */
      targetNode?:yfiles.graph.INode;
      /**
       * Determines whether an {@link yfiles.graph.IEdge edge} is a self-loop, that is, it starts and ends at the same port owner.
       * @type {boolean}
       */
      isSelfloop?:boolean;
    }
    var IEdge:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.graph.IGraph} to declare and obtain the defaults for edges and their labels and ports.
     * <p>
     * Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
     * after the change are affected.
     * </p>
     * @see yfiles.graph.IGraph#edgeDefaults
     * @interface
     */
    export interface IEdgeDefaults extends Object{
      /**
       * Factory method that returns a style instance for use with newly created edges.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.IEdgeDefaults#style} property, if {@link yfiles.graph.IEdgeDefaults#shareStyleInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @returns The style to use, which for most implementations is either a {@link #clone clone} of or the {@link #style} property, if {@link #shareStyleInstance} is enabled.
       * @abstract
       */
      getStyleInstance():yfiles.styles.IEdgeStyle;
      /**
       * Gets or sets the defaults for ports at edges.
       * <p>
       * Note that these are not the {@link yfiles.graph.IEdge#sourcePort}s or {@link yfiles.graph.IEdge#targetPort}s, because conceptually they belong to the nodes (and their
       * {@link yfiles.graph.INodeDefaults}), but rather for ports at edges to which other edges can connect.
       * </p>
       * @abstract
       * @type {yfiles.graph.IPortDefaults}
       */
      ports:yfiles.graph.IPortDefaults;
      /**
       * Gets or sets the defaults for labels at edges.
       * @abstract
       * @type {yfiles.graph.ILabelDefaults}
       */
      labels:yfiles.graph.ILabelDefaults;
      /**
       * Gets or sets the style to use for edges.
       * <p>
       * Depending on the setting of {@link yfiles.graph.IEdgeDefaults#shareStyleInstance}, the {@link yfiles.graph.IEdgeDefaults#getStyleInstance} method should return a {@link yfiles.lang.ICloneable#clone clone} of this
       * instance or the very same instance.
       * </p>
       * @see yfiles.graph.IEdgeDefaults#shareStyleInstance
       * @abstract
       * @type {yfiles.styles.IEdgeStyle}
       */
      style:yfiles.styles.IEdgeStyle;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.IEdgeDefaults#style} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.IEdgeDefaults#getStyleInstance}.
       * @see yfiles.graph.IEdgeDefaults#getStyleInstance
       * @see yfiles.graph.IEdgeDefaults#style
       * @abstract
       * @type {boolean}
       */
      shareStyleInstance:boolean;
    }
    var IEdgeDefaults:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is used for manipulating a {@link yfiles.graph.FoldingManager#createFoldingView managed view} of an {@link yfiles.graph.IGraph} that supports folding operations like {@link yfiles.graph.IFoldingView#expand expanding}
     * and {@link yfiles.graph.IFoldingView#collapse collapsing} of nodes.
     * <p>
     * The {@link yfiles.graph.FoldingManager} creates views of a {@link yfiles.graph.FoldingManager#masterGraph} which is created using copies of the original "master" entities. The
     * views are being synchronized via the {@link yfiles.graph.FoldingManager#masterGraph}, but may contain additional {@link yfiles.graph.FoldingManager#getFolderNodeState view state}. Typically nodes and edges are being copied
     * from the {@link yfiles.graph.FoldingManager#masterGraph} and they share many of their properties by reference (e.g. {@link yfiles.graph.INode#style}). However if group nodes are being represented by
     * {@link yfiles.graph.IFoldingView#collapse collapsed} group nodes in this view, they maintain a separate set of {@link yfiles.graph.ILabel labels}, {@link yfiles.graph.IPort ports}, {@link yfiles.graph.IBend bends}, and {@link yfiles.graph.INode#layout geometry}. This view-local
     * state is shared between all views that have been {@link yfiles.graph.FoldingManager#createFoldingView created} by the same Manager and can be obtained and modified via the
     * {@link yfiles.graph.FoldingManager}'s <code>GetFoldingEdgeState</code> methods for {@link yfiles.graph.FoldingManager#getFoldingEdgeState folding edges} and {@link yfiles.graph.FoldingManager#getFolderNodeState folder nodes} in the absence of a
     * corresponding {@link yfiles.graph.IFoldingView} view instance. Developers typically don't need to implement this interface
     * themselves, rather an implementation is provided by the framework and can be obtained from the
     * {@link yfiles.graph.FoldingManager}. Use {@link yfiles.graph.FoldingManager#createFoldingView} to obtain an {@link yfiles.graph.IGraph}
     * implementation that contains an implementation of this class in its {@link yfiles.graph.ILookup#lookup} method:
     * </p>
     * <p>
     * This interface extends the {@link yfiles.lang.IPropertyObservable} interface. This can be used to easily monitor the {@link yfiles.graph.IFoldingView#invalid validity} of
     * this view. Also changes to the {@link yfiles.graph.IFoldingView#localRoot} property will be published via this event.
     * </p>
     * <p>
     * A brief description of the interface's API and some usage aspects is given in the section {@link @DGUIDE_PREFIX@/folding.html#folding-working_with_folding Working with Folding}. Class
     * <code>FoldingManager</code>'s central role for folding support in general is discussed in the section {@link @DGUIDE_PREFIX@/folding-technical_background.html Folding Revisited}.
     * </p>
     * @see yfiles.graph.FoldingManager
     * @interface
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.lang.IPropertyObservable}
     */
    export interface IFoldingView extends Object,yfiles.graph.ILookup,yfiles.lang.IPropertyObservable{
      /**
       * This method will collapse the {@link yfiles.graph.IGraph#getChildren children} of a group node into the group node.
       * <p>
       * If a {@link yfiles.graph.IGraph#isGroupNode group} node that belongs to this view's hierarchy is in {@link yfiles.graph.IFoldingView#expand expanded} state, calling this method will remove the {@link yfiles.graph.IGraph#getChildren children}
       * from this {@link yfiles.graph.IFoldingView#graph view}. This method will do nothing if the node is already collapsed. Note that collapsing a group node will
       * make it a {@link yfiles.graph.IGraph#isGroupNode non-group} node in the hierarchy. The collapsed group node has its own separate set of attributes, which can be
       * obtained using the {@link yfiles.graph.FoldingManager#getFolderNodeState} method. Therefore it may appear that the node changes
       * its location during a collapse operation. This behavior ensures that collapsing a node does not modify the state of the
       * model (only the state in the view is changed), but depending on the application this may confuse the user because he
       * might loose his mental map of the diagram. In order to customize the behavior custom code can register with the {@link yfiles.graph.IFoldingView#addGroupCollapsedListener GroupCollapsed}
       * event to perform the necessary modifications, like adjusting the view port or moving the node to the desired location.
       * </p>
       * <p>
       * In addition to that this method can also be used for {@link yfiles.graph.FoldingManager#masterGraph master} nodes which are currently not represented in this view. In
       * this case these nodes will be displayed as collapsed the next time they will be included in this view.
       * </p>
       * @param groupNode A group node that is part of the {@link #graph} or the {@link #masterGraph master graph}.
       * @see yfiles.graph.IFoldingView#isExpanded
       * @abstract
       */
      collapse(groupNode:yfiles.graph.INode):void;
      /**
       * Directly creates a collapsed node on this instance with the given parameters.
       * <p>
       * This will implicitly create a new empty group node on the {@link yfiles.graph.FoldingManager#masterGraph} with the defaults for such a group node. The <code>parent</code> must be
       * part of the {@link yfiles.graph.IFoldingView#graph Folded Graph View} or <code>null</code> to create a folder at the root of the current hierarchy. It will be converted into
       * a group node if necessary. The returned {@link yfiles.graph.INode} is also part of the {@link yfiles.graph.IFoldingView#graph Folded Graph View}.
       * </p>
       * @param [parent=null] The parent node in the {@link #graph}, or <code>null</code>
       * @param [layout=null] The layout for the collapsed node.
       * @param [style=null] The style for the collapsed node
       * @param [tag=null] The tag of the node.
       * @returns A collapsed (folder) node that lives in {@link #graph}
       * @abstract
       */
      createFolderNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * Directly creates a collapsed node on this instance with the given parameters.
       * <p>
       * This will implicitly create a new empty group node on the {@link yfiles.graph.FoldingManager#masterGraph} with the defaults for such a group node. The <code>parent</code> must be
       * part of the {@link yfiles.graph.IFoldingView#graph Folded Graph View} or <code>null</code> to create a folder at the root of the current hierarchy. It will be converted into
       * a group node if necessary. The returned {@link yfiles.graph.INode} is also part of the {@link yfiles.graph.IFoldingView#graph Folded Graph View}.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null] The parent node in the {@link #graph}, or <code>null</code>
       * @param [options.layout=null] The layout for the collapsed node.
       * @param [options.style=null] The style for the collapsed node
       * @param [options.tag=null] The tag of the node.
       * @returns A collapsed (folder) node that lives in {@link #graph}
       * @abstract
       */
      createFolderNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object}):yfiles.graph.INode;
      /**
       * Disposes of this view explicitly.
       * <p>
       * Normally it is not necessary to do so, because the view will be garbage collected automatically. However if the view
       * still displays contents, this might have unwanted negative side effects (group node sizes being calculated, etc.) A
       * disposed of instance is not synchronized with the {@link yfiles.graph.FoldingManager} anymore and should not be used anymore.
       * </p>
       * @abstract
       */
      dispose():void;
      /**
       * This method will expand a previously {@link yfiles.graph.IFoldingView#collapse collapsed} group node to make its contents appear in this view.
       * <p>
       * If a node that belongs to this view is in {@link yfiles.graph.IFoldingView#collapse collapsed} state, calling this method will make the {@link yfiles.graph.IGraph#getChildren children} appear in this
       * {@link yfiles.graph.IFoldingView#graph view}. This method will do nothing if the node is already expanded. Note that expanding a collapsed group node will make
       * the node a {@link yfiles.graph.IGraph#isGroupNode group node}, whereas a collapsed group node is no group node in this hierarchy and thus cannot have visible
       * children. The expanded group node has its own separate set of attributes, which may be different from the collapsed
       * state that can be obtained using the {@link yfiles.graph.FoldingManager#getFolderNodeState} method. The expanded state is always
       * the same as the state of the {@link yfiles.graph.IFoldingView#getMasterItem master node}. Because of these differences it may appear that the node changes its location
       * during an expand operation. This behavior ensures that expanding a node does not modify the state of the model by moving
       * the node or its descendants (only the state in the view is changed), but depending on the application this may confuse
       * the user because he might loose his mental map of the diagram. In order to customize the behavior custom code can
       * register with the {@link yfiles.graph.IFoldingView#addGroupExpandedListener GroupExpanded} event to perform the
       * necessary modifications, like adjusting the view port or moving the node and its descendants to the desired location.
       * </p>
       * <p>
       * In addition to that this method can also be used for {@link yfiles.graph.FoldingManager#masterGraph master} nodes which are currently not represented in this view. In
       * this case these nodes will be displayed as expanded the next time they will be included in this view.
       * </p>
       * @param groupNode A group node that is part of the {@link #graph} or the {@link #masterGraph master graph}.
       * @see yfiles.graph.IFoldingView#isExpanded
       * @see yfiles.graph.IFoldingView#collapse
       * @abstract
       */
      expand(groupNode:yfiles.graph.INode):void;
      /**
       * Helper method that corresponds to the {@link yfiles.graph.IFoldingView#getMasterItem} method, but is used for folding edges,
       * only.
       * <p>
       * Since folding edges can represent more than one master edge, this method can be used to query all of the master edges
       * that the provided folding edge represents. If the folding edge corresponds to a single master edge, the result will
       * enumerate that edge, only.
       * </p>
       * @param foldingEdge The folding edge for which the master edges should be returned.
       * @returns An enumerable that can enumerate all of the edges in the {@link #masterGraph} that are being represented by the folding edge in this {@link #graph view}.
       * @see yfiles.graph.IFoldingEdgeFactory#addToExistingFoldingEdge
       * @see yfiles.graph.IFoldingView#getViewItem
       * @abstract
       */
      getMasterEdges(foldingEdge:yfiles.graph.IEdge):yfiles.collections.IEnumerable<yfiles.graph.IEdge>;
      /**
       * Helper method that can be used to retrieve the original "master" items in the {@link yfiles.graph.FoldingManager#masterGraph} that is managed by the Manager associated
       * with this view.
       * <p>
       * Since the {@link yfiles.graph.IFoldingView#graph} view that is associated with this instance maintains a copy of the entities in the Manager's {@link yfiles.graph.FoldingManager#masterGraph}, there is a
       * mapping between elements that belong to this view's {@link yfiles.graph.IFoldingView#graph} and the elements in the master graph. This method can be called for
       * the elements contained in this {@link yfiles.graph.IGraph} and will return the corresponding item in the master graph that the
       * element corresponds to, if any. Note that for some items in the {@link yfiles.graph.IFoldingView#graph}, there is no master item in the {@link yfiles.graph.FoldingManager#masterGraph}, e.g. the {@link yfiles.graph.IPort}
       * instances to which {@link yfiles.graph.FoldingEdgeStateId folding edges} connect have no corresponding ports in the master graph. The same holds true for the {@link yfiles.graph.IBend bends}
       * and {@link yfiles.graph.ILabel labels} of folding edges and folder nodes. For folding edges, this method will yield the main representing edge in
       * the master, if the folding edge represents more than one master edge. For nodes, the master node will be returned, no
       * matter whether the node is currently {@link yfiles.graph.IFoldingView#collapse collapsed} or not.
       * </p>
       * Type parameter <code>T</code>: The type of the argument, which is typically {@link yfiles.graph.INode}, {@link yfiles.graph.IEdge}, {@link yfiles.graph.ILabel},
       * {@link yfiles.graph.IBend}, or {@link yfiles.graph.IPort}.
       * @param item The item that is part of this {@link #graph} for which the original "master" item in the {@link #masterGraph} will be returned.
       * @returns An item in the {@link #masterGraph} that corresponds to the <code>item</code> or <code>null</code>, if there is no such item.
       * @see yfiles.graph.IFoldingView#getViewItem
       * @template T
       * @abstract
       */
      getMasterItem<T extends yfiles.graph.IModelItem>(item:T):T;
      /**
       * Helper method that can be used to retrieve the items in this {@link yfiles.graph.IFoldingView#graph}-view for all items that are part of the {@link yfiles.graph.FoldingManager#masterGraph} that is
       * associated with the Manager of this instance.
       * <p>
       * Since the {@link yfiles.graph.IFoldingView#graph} view that is associated with this instance maintains a copy of the entities in the Manager's {@link yfiles.graph.FoldingManager#masterGraph}, there is a
       * mapping between elements that belong to master graph and the elements in the {@link yfiles.graph.IFoldingView#graph} of this view. This method can be called
       * for the elements contained in this {@link yfiles.graph.FoldingManager#masterGraph master graph's} items and will return the corresponding item in this view's {@link yfiles.graph.IFoldingView#graph} if the
       * element is represented by an item in this view. Note that for some items in the {@link yfiles.graph.FoldingManager#masterGraph}, there may be no item in the {@link yfiles.graph.IFoldingView#graph} instance,
       * e.g. those elements which are part of a {@link yfiles.graph.IFoldingView#collapse collapsed} subtree in the {@link yfiles.graph.FoldingManager#masterGraph} hierarchy or those items that do not belong to the
       * subtree that is induced by the {@link yfiles.graph.IFoldingView#localRoot} of this view. Also, for nodes and edges that are being represented by dummies in this
       * view, labels, ports, and bends are not being represented directly by corresponding entities in this graph. If more than
       * one edge is represented by a folding edge in this view, this method will yield the same folding edge instance for each
       * of them.
       * </p>
       * Type parameter <code>T</code>: The type of the argument, which is typically {@link yfiles.graph.INode}, {@link yfiles.graph.IEdge}, {@link yfiles.graph.ILabel},
       * {@link yfiles.graph.IBend}, or {@link yfiles.graph.IPort}.
       * @param item An item that is part of the {@link #masterGraph} that is associated with the Manager of this instance.
       * @returns An item in the {@link #graph local graph view} that corresponds to the <code>item</code> or
       * <code>null</code>, if the item is not currently being represented in this view.
       * @see yfiles.graph.IFoldingView#getMasterItem
       * @see yfiles.graph.IFoldingView#getMasterEdges
       * @template T
       * @abstract
       */
      getViewItem<T extends yfiles.graph.IModelItem>(item:T):T;
      /**
       * Determines whether the provided node is currently expanded or will be displayed in
       * {@link yfiles.graph.IFoldingView#expand}ed state the next time it will be included in this view.
       * <p>
       * Expanded group nodes will have all of their {@link yfiles.graph.IGraph#getChildren children} visible in this {@link yfiles.graph.IFoldingView#graph view}.
       * </p>
       * <p>
       * The <code>groupNode</code> can be either a node of the {@link yfiles.graph.IFoldingView#graph view} or the {@link yfiles.graph.FoldingManager#masterGraph master graph}. If the node is from the master graph and is
       * currently not represented in this view, this method will return the
       * {@link yfiles.graph.IFoldingView#expand}ed state it will have the next time it will be included in this view.
       * </p>
       * @param groupNode The group node for which the {@link #expand expanded} state will be returned.
       * @returns <code>true</code>, if the group node can be {@link #collapse collapsed}.
       * @abstract
       */
      isExpanded(groupNode:yfiles.graph.INode):boolean;
      /**
       * Determines whether the specified item is a folding state in this view.
       * <p>
       * This method can be used to determine if an {@link yfiles.graph.INode}, {@link yfiles.graph.IEdge}, {@link yfiles.graph.IPort},
       * {@link yfiles.graph.ILabel}, or {@link yfiles.graph.IBend} has a {@link yfiles.graph.FoldingManager#getFolderNodeState view local state} and is not a direct copy of the {@link yfiles.graph.IFoldingView#getMasterItem master item}, if any.
       * Note that {@link yfiles.graph.IFoldingView#collapse collapsed} group nodes are in folding state, even though they do have a {@link yfiles.graph.IFoldingView#getMasterItem master} item in the master graph and
       * folding edges always have at least one
       * {@link yfiles.graph.IFoldingView#getMasterEdges master edge}, while all other folding items don't have a corresponding master item in the master graph.
       * </p>
       * @param item One of the items that are {@link #contains contained} in the {@link #graph}.
       * @returns Whether the item is in folding state and is not an exact copy of an item in the {@link #masterGraph}.
       * @abstract
       */
      isInFoldingState(item:yfiles.graph.IModelItem):boolean;
      /**
       * Adds the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been
       * {@link yfiles.graph.IFoldingView#collapse}d.
       * @param listener The listener to add.
       * @see yfiles.graph.IFoldingView#removeGroupCollapsedListener
       * @abstract
       */
      addGroupCollapsedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupCollapsed</code> event that occurs whenever a group has been
       * {@link yfiles.graph.IFoldingView#collapse}d.
       * @param listener The listener to remove.
       * @see yfiles.graph.IFoldingView#addGroupCollapsedListener
       * @abstract
       */
      removeGroupCollapsedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been
       * {@link yfiles.graph.IFoldingView#expand}ed.
       * @param listener The listener to add.
       * @see yfiles.graph.IFoldingView#removeGroupExpandedListener
       * @abstract
       */
      addGroupExpandedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>GroupExpanded</code> event that occurs whenever a group has been
       * {@link yfiles.graph.IFoldingView#expand}ed.
       * @param listener The listener to remove.
       * @see yfiles.graph.IFoldingView#addGroupExpandedListener
       * @abstract
       */
      removeGroupExpandedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Provides a reference to the {@link yfiles.graph.IGraph} implementation that provides the actual view data.
       * <p>
       * The instance has an implementation of this interface in its {@link yfiles.graph.ILookup} that has been obtained from the
       * {@link yfiles.graph.FoldingManager}'s {@link yfiles.graph.FoldingManager#createFoldingView} set of methods. The instance holds the
       * elements in the view only, which in general is a subset of the {@link yfiles.graph.FoldingManager#masterGraph}. Also note that the elements are not the same references
       * as their {@link yfiles.graph.IFoldingView#getMasterItem masters}, they are copies that may share the same attributes.
       * </p>
       * @abstract
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Determines whether this instance should automatically switch the {@link yfiles.graph.IFoldingView#localRoot} to an ancestor of the current root, if this view
       * becomes {@link yfiles.graph.IFoldingView#invalid}.
       * <p>
       * By default this property is set to <code>true</code> which assures that this view will never become
       * {@link yfiles.graph.IFoldingView#invalid}. It is the responsibility of the client code to check for the validity of this view before trying to modify it if this
       * feature is disabled.
       * </p>
       * @abstract
       * @type {boolean}
       */
      autoSwitchToAncestor:boolean;
      /**
       * Determines if this view is currently valid and can be edited.
       * <p>
       * A view is invalid if the {@link yfiles.graph.IFoldingView#localRoot} is not currently part of the {@link yfiles.graph.FoldingManager#masterGraph} hierarchy. This can happen if the root node is removed from the
       * {@link yfiles.graph.FoldingManager#masterGraph}. Note that the view can become valid again if the removal is undone.
       * </p>
       * @see yfiles.graph.IFoldingView#autoSwitchToAncestor
       * @abstract
       * @type {boolean}
       */
      invalid:boolean;
      /**
       * Determines whether changes to the {@link yfiles.graph.IFoldingView#localRoot} property and calls to {@link yfiles.graph.IFoldingView#collapse} and {@link yfiles.graph.IFoldingView#expand}
       * should be recorded in undo history.
       * <p>
       * By default this property is set to <code>false</code>, however depending on the use case it may make sense to enqueue
       * {@link yfiles.graph.IUndoUnit}s for this type of operations.
       * </p>
       * @see yfiles.graph.IFoldingView#autoSwitchToAncestor
       * @abstract
       * @type {boolean}
       */
      enqueueNavigationalUndoUnits:boolean;
      /**
       * Gets or sets an entity of the {@link yfiles.graph.FoldingManager#masterGraph} hierarchy to serve as the virtual root for the view's local hierarchy.
       * <p>
       * This property can be set to any {@link yfiles.graph.IGraph#isGroupNode group} node in the
       * {@link yfiles.graph.FoldingManager#masterGraph}'s hierarchy whose contents will then be represented in this view. Note that the root of the view's local hierarchy is
       * not set to the same instance.
       * </p>
       * @abstract
       * @type {yfiles.graph.INode}
       */
      localRoot:yfiles.graph.INode;
      /**
       * Provides access to the manager that created this view and that contains a reference to the {@link yfiles.graph.FoldingManager#masterGraph}.
       * @type {yfiles.graph.FoldingManager}
       */
      manager?:yfiles.graph.FoldingManager;
    }
    var IFoldingView:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum AdjacencyTypes{
      /**
       * Neither {@link yfiles.graph.AdjacencyTypes#INCOMING} nor {@link yfiles.graph.AdjacencyTypes#OUTGOING}
       */
      NONE,
      /**
       * The constant for incoming edges.
       * <p>
       * An edge that is incoming at its owner has the owner as its target.
       * </p>
       */
      INCOMING,
      /**
       * The constant for outgoing edges.
       * <p>
       * An edge that is outgoing from its owner has the owner as its source.
       * </p>
       */
      OUTGOING,
      /**
       * Both {@link yfiles.graph.AdjacencyTypes#INCOMING} and {@link yfiles.graph.AdjacencyTypes#OUTGOING}
       */
      ALL
    }
    export enum GraphItemTypes{
      /**
       * Constant for nodes.
       * @see yfiles.graph.INode
       */
      NODE,
      /**
       * Constant for edges.
       * @see yfiles.graph.IEdge
       */
      EDGE,
      /**
       * Constant for node labels.
       * @see yfiles.graph.ILabel
       */
      NODE_LABEL,
      /**
       * Constant for edge labels.
       * @see yfiles.graph.ILabel
       */
      EDGE_LABEL,
      /**
       * Constant for ports.
       * @see yfiles.graph.IPort
       */
      PORT,
      /**
       * Constant for all kinds of labels.
       * <p>
       * This is a bitwise combination of {@link yfiles.graph.GraphItemTypes#NODE_LABEL} and {@link yfiles.graph.GraphItemTypes#EDGE_LABEL}.
       * </p>
       * @see yfiles.graph.ILabel
       */
      LABEL,
      /**
       * Constant for all kinds of {@link yfiles.graph.ILabelOwner}.
       * <p>
       * This is a bitwise combination of {@link yfiles.graph.GraphItemTypes#NODE} and {@link yfiles.graph.GraphItemTypes#EDGE}.
       * </p>
       * @see yfiles.graph.ILabelOwner
       */
      LABEL_OWNER,
      /**
       * Constant for edge bends.
       * @see yfiles.graph.IBend
       */
      BEND,
      /**
       * Constant for all item types.
       * <p>
       * This is a bitwise combination of all other types declared in this enum.
       * </p>
       */
      ALL,
      /**
       * Constant for no item type.
       * <p>
       * This is a constant where no type bit has been set, i.e. it is <code>0</code>.
       * </p>
       */
      NONE
    }
    /**
     * Central interface that models a graph which can be displayed in a {@link yfiles.view.CanvasComponent canvas} or {@link yfiles.view.GraphComponent}.
     * <p>
     * This interface can be used to query structural information, it also offers methods that change the structure of the
     * graph and its attributes. A number of events will trigger appropriate events if the structure of the graph changes.
     * </p>
     * <p>
     * The graph is made up of collections of {@link yfiles.graph.INode nodes} and {@link yfiles.graph.IEdge edges}. Each node and edge can be associated with a number of {@link yfiles.graph.ILabel labels} and
     * possibly {@link yfiles.graph.IPort ports} to which edges connect. An edge connects to two ports and may consist of zero or more {@link yfiles.graph.IBend bends}. The graph is
     * treated as a directed graph, i.e. edges have a {@link yfiles.graph.IEdge#sourcePort source port} and a
     * {@link yfiles.graph.IEdge#targetPort target port}. It is up to the algorithms and the visualization to possibly treat them as undirected.
     * </p>
     * <p>
     * This interface also provides support for hierarchically organized, i.e. grouped graphs. This means that nodes together
     * with their connecting edges can be {@link yfiles.graph.IGraph#setParent put into} other nodes. The containing node is referred to as a "group node." To create
     * and edit group nodes interactively use the {@link yfiles.input.GraphEditorInputMode} as input mode and enable the
     * {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations grouping operations}.
     * </p>
     * <p>
     * The {@link yfiles.graph.IGraph#edgesAt} and {@link yfiles.graph.IGraph#edgesAt} methods can be used to query adjacency information from
     * the graph's structure.
     * </p>
     * <p>
     * Each element in the graph can be associated with a style that is used for the visualization of the element. Styles can
     * be shared between multiple instances.
     * </p>
     * <p>
     * To associate data with the elements in the graph, code can make use of the {@link yfiles.graph.ITagOwner#tag} property that all {@link yfiles.graph.IModelItem}s
     * provide. In order to associate more than one data element with the graph items, compound data objects can be used.
     * Alternatively the {@link yfiles.graph.IMapperRegistry} that can be obtained from the {@link yfiles.graph.IGraph#mapperRegistry} property can be used to associate
     * arbitrary data sets with the items in this graph.
     * </p>
     * <p>
     * This interface provides a number of events that can be used to get notified for changes in the graph structure. These
     * events are raised whenever the corresponding state change occurs, e.g. also when loading the graph from a file. If you
     * are only interested in changes that are triggered interactively, you should subscribe to the corresponding events on the
     * {@link yfiles.input.IInputMode} implementations that implement the user interaction. Especially, you may not modify the graph
     * structure in handlers for these event, e.g. try to prevent or undo the change that has raised the event.
     * </p>
     * <p>
     * The graph model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/graph.html The Graph Model}.
     * </p>
     * <p>
     * More information on visual styles can be found in the section {@link @DGUIDE_PREFIX@/view_visualization-graph-elements.html Visualization of Graph Elements}.
     * </p>
     * @see yfiles.graph.DefaultGraph
     * @see yfiles.graph.INode
     * @see yfiles.graph.IEdge
     * @see yfiles.graph.IPort
     * @see yfiles.graph.ILabel
     * @see yfiles.graph.IBend
     * @see yfiles.graph.IMapperRegistry
     * @interface
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.graph.ITagOwner}
     */
    export interface IGraph extends Object,yfiles.graph.ILookup,yfiles.graph.ITagOwner{
      /**
       * Adds a bend at the given index to the given edge using the coordinates provided.
       * <p>
       * The added instance will be returned.
       * </p>
       * @param edge The edge to which the bend will be added.
       * @param location the coordinates to use for the newly created bend
       * @param [index=-1] The index for the newly added bend; a negative value (which is the default) indicates that the bend should be appended
       * to the end of the list of bends.
       * @returns a newly created live bend
       * @throws {Stubs.Exceptions.ArgumentError} <code>edge</code> is not in this graph.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.graph.IGraph#setBendLocation
       * @abstract
       */
      addBend(edge:yfiles.graph.IEdge,location:yfiles.geometry.Point,index?:number):yfiles.graph.IBend;
      /**
       * Adds bends with the given locations to the end of the bend list of the given edge.
       * @param edge The edge to add the bends to.
       * @param locations The locations of the bends.
       * @throws {Stubs.Exceptions.ArgumentError} <code>edge</code> is not in this graph.
       */
      addBends?(edge:yfiles.graph.IEdge,locations:yfiles.collections.IEnumerable<yfiles.geometry.Point>):void;
      /**
       * Add a label to the given node or edge using the text as the initial label text and label model parameter, style and
       * tag.
       * @param owner The node or edge to add the label to.
       * @param text The initial text of the label
       * @param [layoutParameter=null] The label model parameter instance to use.
       * @param [style=null] The style to use for the label
       * @param [preferredSize=null] The initial values to use for the {@link #preferredSize}.
       * @param [tag=null] The initial {@link #tag} to assign.
       * @returns the newly created label
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       * @see yfiles.graph.IGraph#addLabelAddedListener
       * @see yfiles.graph.IGraph#setLabelText
       * @see yfiles.graph.IGraph#setLabelLayoutParameter
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.styles.VoidLabelStyle#INSTANCE
       * @abstract
       */
      addLabel(owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object):yfiles.graph.ILabel;
      /**
       * Add a label to the given node or edge using the text as the initial label text and label model parameter, style and
       * tag.
       * @param {Object} options The parameters to pass.
       * @param options.owner The node or edge to add the label to.
       * @param options.text The initial text of the label
       * @param [options.layoutParameter=null] The label model parameter instance to use.
       * @param [options.style=null] The style to use for the label
       * @param [options.preferredSize=null] The initial values to use for the {@link #preferredSize}.
       * @param [options.tag=null] The initial {@link #tag} to assign.
       * @returns the newly created label
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       * @see yfiles.graph.IGraph#addLabelAddedListener
       * @see yfiles.graph.IGraph#setLabelText
       * @see yfiles.graph.IGraph#setLabelLayoutParameter
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.styles.VoidLabelStyle#INSTANCE
       * @abstract
       */
      addLabel(options:{owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object}):yfiles.graph.ILabel;
      /**
       * Add a port to the given port owner using the location model parameter, style and tag.
       * <p>
       * The <code>locationParameter</code> determines the location of the port.
       * </p>
       * <p>
       * Depending on the implementation this method may throw an {@link yfiles.lang.Exception} if the type of the <code>owner</code> instance does
       * not support adding of ports.
       * </p>
       * @param owner the owner to add the port instance to.
       * @param [locationParameter=null] the parameter to use for the port to determine its location.
       * @param [style=null] the style to initially assign to the {@link #style} property, e.g. {@link #INSTANCE}.
       * @param [tag=null] the initial {@link #tag} to assign.
       * @returns the newly created port
       * @throws {Stubs.Exceptions.NotSupportedError} If this instance cannot add a port to <code>owner</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       * @see yfiles.graph.IGraph#addPortAddedListener
       * @see yfiles.styles.VoidPortStyle#INSTANCE
       * @abstract
       */
      addPort(owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,style?:yfiles.styles.IPortStyle,tag?:Object):yfiles.graph.IPort;
      /**
       * Add a port to the given port owner using the location model parameter, style and tag.
       * <p>
       * The <code>locationParameter</code> determines the location of the port.
       * </p>
       * <p>
       * Depending on the implementation this method may throw an {@link yfiles.lang.Exception} if the type of the <code>owner</code> instance does
       * not support adding of ports.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.owner the owner to add the port instance to.
       * @param [options.locationParameter=null] the parameter to use for the port to determine its location.
       * @param [options.style=null] the style to initially assign to the {@link #style} property, e.g. {@link #INSTANCE}.
       * @param [options.tag=null] the initial {@link #tag} to assign.
       * @returns the newly created port
       * @throws {Stubs.Exceptions.NotSupportedError} If this instance cannot add a port to <code>owner</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       * @see yfiles.graph.IGraph#addPortAddedListener
       * @see yfiles.styles.VoidPortStyle#INSTANCE
       * @abstract
       */
      addPort(options:{owner:yfiles.graph.IPortOwner,locationParameter?:yfiles.graph.IPortLocationModelParameter,style?:yfiles.styles.IPortStyle,tag?:Object}):yfiles.graph.IPort;
      /**
       * Add a port to the given port owner using the absolute coordinates as the new initial position of the port anchor.
       * @param owner the owner to add the port instance to.
       * @param location the location to use for the port to determine its location. This is passed to the {@link #createDefaultPortLocationParameter}
       * method to determine the initial {@link } to use.
       * @param [style=null] the style to initially assign to the {@link #style} property, e.g. {@link #INSTANCE}.
       * @param [tag=null] the initial {@link #tag} to assign.
       * @returns the newly created port
       * @throws {Stubs.Exceptions.NotSupportedError} If this instance cannot add a port to <code>owner</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      addPortAt?(owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point,style?:yfiles.styles.IPortStyle,tag?:Object):yfiles.graph.IPort;
      /**
       * Add a port to the given port owner using the absolute coordinates as the new initial position of the port anchor.
       * @param {Object} options The parameters to pass.
       * @param options.owner the owner to add the port instance to.
       * @param options.location the location to use for the port to determine its location. This is passed to the {@link #createDefaultPortLocationParameter}
       * method to determine the initial {@link } to use.
       * @param [options.style=null] the style to initially assign to the {@link #style} property, e.g. {@link #INSTANCE}.
       * @param [options.tag=null] the initial {@link #tag} to assign.
       * @returns the newly created port
       * @throws {Stubs.Exceptions.NotSupportedError} If this instance cannot add a port to <code>owner</code>.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      addPortAt?(options:{owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point,style?:yfiles.styles.IPortStyle,tag?:Object}):yfiles.graph.IPort;
      /**
       * Adds a new port to the graph at the node using a location that is relative to the center of the node.
       * <p>
       * The port style for the newly-created port is taken from the graph's defaults. The location parameter is determined by
       * delegating to {@link yfiles.graph.IGraph#addPortAt}.
       * </p>
       * @param node The owner of the port.
       * @param relativeLocation The offset of the port relative to the center of the {@link #layout}.
       * @returns The newly added port instance.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       */
      addRelativePort?(node:yfiles.graph.INode,relativeLocation:yfiles.geometry.Point):yfiles.graph.IPort;
      /**
       * Convenience method that uses the {@link yfiles.graph.UndoEngine} from the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup} to
       * {@link yfiles.graph.UndoEngine#addUnit add a unit}.
       * @param undoName The name of the undo operation.
       * @param redoName The name of the redo operation.
       * @param undo The undo action.
       * @param redo The redo action.
       */
      addUndoUnit?(undoName:string,redoName:string,undo:()=>void,redo:()=>void):void;
      /**
       * Method to adjust the size of a group node.
       * <p>
       * This will resize the group node bounds such that the node requires the least amount of space. If the node does not have
       * any {@link yfiles.graph.IGraph#getChildren children}, its bounds will be left unchanged. This will also respect any {@link yfiles.input.INodeSizeConstraintProvider}s
       * for {@link yfiles.graph.INode}s, that are available in the lookup of the <code>groupNode</code>.
       * </p>
       * @param groupNode The group node to adjust the size of.
       * @throws {Stubs.Exceptions.ArgumentError} <code>groupNode</code> is not in this graph.
       * @see yfiles.graph.IGroupBoundsCalculator
       * @see yfiles.input.INodeSizeConstraintProvider
       * @see yfiles.graph.GroupingSupport#calculateMinimumEnclosedArea
       * @see yfiles.graph.GroupingSupport#enlargeGroupNode
       * @see yfiles.graph.GroupingSupport#enlargeAllGroupNodes
       */
      adjustGroupNodeLayout?(groupNode:yfiles.graph.INode):void;
      /**
       * Adjusts the {@link yfiles.graph.ILabel#preferredSize} property of a label to fit the suggested size of its {@link yfiles.styles.ILabelStyleRenderer}.
       * <p>
       * This implementation uses the {@link yfiles.graph.ILabel#style style's} renderer for the label to determine the preferred rendering size. This is useful
       * after the label's content or style have been changed.
       * </p>
       * @param label The label to adjust the size for.
       * @see yfiles.graph.ILabelDefaults#autoAdjustPreferredSize
       */
      adjustLabelPreferredSize?(label:yfiles.graph.ILabel):void;
      /**
       * Convenience method that runs a layout on a graph using {@link yfiles.layout.LayoutGraphAdapter#applyLayout}.
       * @param layout The layout.
       * @param [layoutData=null] The layout data.
       * @see yfiles.view.GraphComponent#morphLayout
       */
      applyLayout?(layout:yfiles.layout.ILayoutAlgorithm,layoutData?:yfiles.layout.LayoutData):void;
      /**
       * Starts an {@link yfiles.graph.ICompoundEdit} that uses the {@link yfiles.graph.IMementoSupport memento design pattern} to record changes to the items in the given <code>items</code>
       * collection.
       * <p>
       * This method uses the {@link yfiles.graph.IMementoSupport} returned by the <code>provider</code> to record the state of an item at the
       * beginning of the edit and when {@link yfiles.graph.ICompoundEdit#commit} is called to create an {@link yfiles.graph.IUndoUnit} that can
       * revert the item to the recorded state and back. If no <code>provider</code> is given, this method uses the {@link yfiles.graph.IMementoSupport}
       * returned by the {@link yfiles.graph.ILookup#lookup} implementation of the items to record the state of an item at the beginning
       * of the edit and when {@link yfiles.graph.ICompoundEdit#commit} is called to create an {@link yfiles.graph.IUndoUnit} that can revert the
       * item to the recorded state and back.
       * </p>
       * <p>
       * Calling this method will immediately enqueue an {@link yfiles.graph.IUndoUnit} into the undo queue. Subsequent additions to the
       * queue will be added after the created instance, even if they are added to the queue before the {@link yfiles.graph.ICompoundEdit#commit}
       * method has been called.
       * </p>
       * <p>
       * The following is an example implementation of an item that is being managed using {@link yfiles.graph.IMementoSupport}:
       * </p>
       * <p>
       * A collection of items from this type can then be watched using the following code snippet:
       * </p>
       * <p>
       * Alternatively, when using a specific provider, consider the following examples. The following is an example
       * implementation of an item that is being managed using {@link yfiles.graph.IMementoSupport}:
       * </p>
       * <p>
       * A collection of items from this type can then be watched using the following code snippet, using the <code>provider</code> to return
       * an appropriate {@link yfiles.graph.IMementoSupport} implementation:
       * </p>
       * <p>
       * Implementing the {@link yfiles.graph.IMementoSupport} interface is quite unrestrained, the type of the state returned by {@link yfiles.graph.IMementoSupport#getState}
       * method can by anything as long as the {@link yfiles.graph.IMementoSupport#applyState} and {@link yfiles.graph.IMementoSupport#stateEquals}
       * methods can deal with it:
       * </p>
       * <p>
       * In summary, use this concept when you want to track the state of items during certain operations for undo/redo. This is
       * efficient if it's easier to handle an item's state than the changes to the item themselves. If you want to focus on the
       * changes or on certain events, you should use custom {@link yfiles.graph.IUndoUnit}
       * </p>
       * Type parameter <code>T</code>: The type of the items that will be modified subsequently.
       * @param undoName The {@link #undoName} of the {@link } that will be placed into the undo queue after {@link #commit} has
       * been called.
       * @param redoName The {@link #redoName} of the {@link } that will be placed into the undo queue after {@link #commit} has
       * been called.
       * @param items The items that will be changed after this call and before the call to {@link #commit}.
       * @param [provider=null] The provider for the {@link } of the <code>items</code>. if the provider returns <code>null</code> for a given item, changes
       * to this item are not being recorded.
       * @returns An implementation of the {@link } interface whose {@link #commit} or {@link #cancel}
       * methods need to be called after the items have been modified.
       * @see yfiles.graph.IMementoSupport
       * @template T
       */
      beginEdit?<T>(undoName:string,redoName:string,items:yfiles.collections.IEnumerable<T>,provider?:(item:T)=>yfiles.graph.IMementoSupport):yfiles.graph.ICompoundEdit;
      /**
       * Starts an {@link yfiles.graph.ICompoundEdit} that records graph changes and {@link yfiles.graph.IGraph#addUndoUnit custom undo units} in a single compound unit.
       * <p>
       * This method can be used to bracket several undo units. All edits added to the queue after this call and before a call to
       * {@link yfiles.graph.ICompoundEdit#cancel} or {@link yfiles.graph.ICompoundEdit#commit} will be placed into the queue as a single block.
       * </p>
       * <p>
       * Client code needs to make sure that either the {@link yfiles.graph.ICompoundEdit#cancel} or {@link yfiles.graph.ICompoundEdit#commit}
       * method is called on the returned instance.
       * </p>
       * @param undoName The undo name for the compound edit.
       * @param redoName The redo name for the compound edit.
       * @returns The handle to stop the recording by calling {@link #cancel} or {@link #commit} on
       * it.
       */
      beginEdit?(undoName:string,redoName:string):yfiles.graph.ICompoundEdit;
      /**
       * Calculates the preferred size of a label with the given properties.
       * @param owner The item that will own the label.
       * @param text The text.
       * @param [layoutParameter=null] The label model parameter.
       * @param [style=null] The label style.
       * @param [tag=null] The tag for the label.
       * @returns The size as calculated by the {@link }.
       */
      calculateLabelPreferredSize?(owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,tag?:Object):yfiles.geometry.Size;
      /**
       * Calculates the preferred size of a label with the given properties.
       * @param {Object} options The parameters to pass.
       * @param options.owner The item that will own the label.
       * @param options.text The text.
       * @param [options.layoutParameter=null] The label model parameter.
       * @param [options.style=null] The label style.
       * @param [options.tag=null] The tag for the label.
       * @returns The size as calculated by the {@link }.
       */
      calculateLabelPreferredSize?(options:{owner:yfiles.graph.ILabelOwner,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,tag?:Object}):yfiles.geometry.Size;
      /**
       * Clears the graph, removing all items in proper order.
       * @see yfiles.graph.IGraph#remove
       */
      clear?():void;
      /**
       * Removes all bends from the given edge.
       * <p>
       * The edge must be part of this graph at the time of the invocation. This will trigger the corresponding events.
       * </p>
       * @param edge the edge whose bends will be removed
       * @throws {Stubs.Exceptions.ArgumentError} <code>edge</code> is not in this graph.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.graph.IGraph#addBendRemovedListener
       */
      clearBends?(edge:yfiles.graph.IEdge):void;
      /**
       * Determines whether this graph contains the specified item.
       * @param item The item.
       * @returns <code>true</code> if this graph contains the specified item; otherwise, <code>false</code>.
       * @abstract
       */
      contains(item:yfiles.graph.IModelItem):boolean;
      /**
       * Creates the label layout parameter for a given {@link yfiles.graph.ILabelOwner}.
       * <p>
       * This implementation uses the {@link yfiles.graph.IGraph#getLabelDefaults label defaults} for the <code>graph</code> to {@link yfiles.graph.ILabelDefaults#getLayoutParameterInstance obtain the parameter instance}.
       * </p>
       * @param owner The item that is the {@link #owner} of the label in question.
       * @returns The default label layout parameter to use for newly created labels at the item.
       * @see yfiles.graph.ILabelDefaults#getLayoutParameterInstance
       * @see yfiles.graph.IGraph#getLabelDefaults
       */
      createDefaultLabelLayoutParameter?(owner:yfiles.graph.ILabelOwner):yfiles.graph.ILabelModelParameter;
      /**
       * Creates a location model parameter for a newly created {@link yfiles.graph.IPort} at the <code>owner</code> that matches the <code>location</code>.
       * <p>
       * If <code>location</code> is <code>null</code>, this method uses the {@link yfiles.graph.IGraph#getPortDefaults port defaults} for the <code>owner</code> to obtain the {@link yfiles.graph.IPortDefaults#getLocationParameterInstance location parameter}.
       * </p>
       * @param owner The owner of the port.
       * @param [location=null] The location in the world coordinate system where the port should be added.
       * @returns Either a location model parameter that matches the location, or the default parameter to use for the {@link }
       * as returned by {@link #getLocationParameterInstance}.
       * @see yfiles.graph.IPortLocationModel#createParameter
       * @see yfiles.graph.IGraph#getPortDefaults
       */
      createDefaultPortLocationParameter?(owner:yfiles.graph.IPortOwner,location?:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * Convenience method that creates and returns an edge that connects to the given node instances using the given style
       * instance.
       * <p>
       * The nodes must be part of this graph at the time of the invocation, and the implementation will choose the {@link yfiles.graph.IPort}
       * instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
       * trigger the corresponding events.
       * </p>
       * @param source The source node the created edge will connect to. It is up to the implementation to decide which port to use at the
       * given node. The implementation may create a new port of the edge.
       * @param target The target node the created edge will connect to. It is up to the implementation to decide which port to use at the
       * given node. The implementation may create a new port of the edge.
       * @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [tag=null] The initial value of the {@link #tag} that will be assigned to the new edge.
       * @returns the newly created edge instance
       * @see yfiles.graph.IGraph#addEdgeCreatedListener
       * @see yfiles.styles.VoidEdgeStyle#INSTANCE
       * @abstract
       */
      createEdge(source:yfiles.graph.INode,target:yfiles.graph.INode,style?:yfiles.styles.IEdgeStyle,tag?:Object):yfiles.graph.IEdge;
      /**
       * Convenience method that creates and returns an edge that connects to the given node instances using the given style
       * instance.
       * <p>
       * The nodes must be part of this graph at the time of the invocation, and the implementation will choose the {@link yfiles.graph.IPort}
       * instances to which the edge will be connected. The edge will be a part of this graph after the method returns. This will
       * trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.source The source node the created edge will connect to. It is up to the implementation to decide which port to use at the
       * given node. The implementation may create a new port of the edge.
       * @param options.target The target node the created edge will connect to. It is up to the implementation to decide which port to use at the
       * given node. The implementation may create a new port of the edge.
       * @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [options.tag=null] The initial value of the {@link #tag} that will be assigned to the new edge.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @param options.bends An array of bends to add to the newly created edge. Each item will be passed to the {@link #addBend} method.
       * @returns the newly created edge instance
       * @see yfiles.graph.IGraph#addEdgeCreatedListener
       * @see yfiles.styles.VoidEdgeStyle#INSTANCE
       * @abstract
       */
      createEdge(options:{source:yfiles.graph.INode,target:yfiles.graph.INode,style?:yfiles.styles.IEdgeStyle,tag?:Object,labels?:Array<any>,ports?:Array<any>,bends?:Array<any>}):yfiles.graph.IEdge;
      /**
       * Creates and returns an edge that connects to the given port instances.
       * <p>
       * The ports must be part of this graph at the time of the invocation. The edge will be a part of this graph after the
       * method returns. This will trigger the corresponding events.
       * </p>
       * @param sourcePort The source port the created edge will connect to.
       * @param targetPort The target port the created edge will connect to.
       * @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [tag=null] The initial value of the {@link #tag} that will be assigned to the new edge.
       * @returns the newly created edge instance
       * @see yfiles.graph.IGraph#addEdgeCreatedListener
       * @see yfiles.styles.VoidEdgeStyle#INSTANCE
       * @abstract
       */
      createEdge(sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort,style?:yfiles.styles.IEdgeStyle,tag?:Object):yfiles.graph.IEdge;
      /**
       * Creates and returns an edge that connects to the given port instances.
       * <p>
       * The ports must be part of this graph at the time of the invocation. The edge will be a part of this graph after the
       * method returns. This will trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.sourcePort The source port the created edge will connect to.
       * @param options.targetPort The target port the created edge will connect to.
       * @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [options.tag=null] The initial value of the {@link #tag} that will be assigned to the new edge.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @param options.bends An array of bends to add to the newly created edge. Each item will be passed to the {@link #addBend} method.
       * @returns the newly created edge instance
       * @see yfiles.graph.IGraph#addEdgeCreatedListener
       * @see yfiles.styles.VoidEdgeStyle#INSTANCE
       * @abstract
       */
      createEdge(options:{sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort,style?:yfiles.styles.IEdgeStyle,tag?:Object,labels?:Array<any>,ports?:Array<any>,bends?:Array<any>}):yfiles.graph.IEdge;
      /**
       * Creates a new group node using the provided style and layout as a child of <code>parent</code>.
       * <p>
       * The group node will be a direct descendant of <code>parent</code>.
       * </p>
       * <p>
       * To create group nodes interactively use the {@link yfiles.input.GraphEditorInputMode} as input mode and enable the
       * {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations grouping operations}.
       * </p>
       * @param [parent=null] The node to use as the parent in the grouping hierarchy or <code>null</code> if the new node should become a top-level node.
       * @param [layout=null] The initial layout to use for the new node.
       * @param [style=null] The style to use for the new node.
       * @param [tag=null] The {@link #tag} to assign to the {@link }.
       * @returns The newly created group node.
       * @abstract
       */
      createGroupNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * Creates a new group node using the provided style and layout as a child of <code>parent</code>.
       * <p>
       * The group node will be a direct descendant of <code>parent</code>.
       * </p>
       * <p>
       * To create group nodes interactively use the {@link yfiles.input.GraphEditorInputMode} as input mode and enable the
       * {@link yfiles.input.GraphEditorInputMode#allowGroupingOperations grouping operations}.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null] The node to use as the parent in the grouping hierarchy or <code>null</code> if the new node should become a top-level node.
       * @param [options.layout=null] The initial layout to use for the new node.
       * @param [options.style=null] The style to use for the new node.
       * @param [options.tag=null] The {@link #tag} to assign to the {@link }.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @param options.children An array of nodes to create as child nodes of this group node. Each item will be passed to the {@link #createNode}
       * method.
       * @returns The newly created group node.
       * @abstract
       */
      createGroupNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object,labels?:Array<any>,ports?:Array<any>,children?:Array<any>}):yfiles.graph.INode;
      /**
       * Creates and returns a node using the specified values for the initial geometry, style, and {@link yfiles.graph.ITagOwner#tag}.
       * <p>
       * The node will be a part of this graph after the method returns. This will trigger the corresponding events.
       * </p>
       * @param layout The layout to use initially. The values will be copied to the node's {@link #layout Layout} field
       * @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [tag=null] The initial value of the {@link #tag} that will be assigned to the new node.
       * @returns A newly created node instance
       * @see yfiles.graph.IGraph#addNodeCreatedListener
       * @see yfiles.styles.VoidNodeStyle#INSTANCE
       * @abstract
       */
      createNode(layout:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * Creates and returns a node using the specified values for the initial geometry, style, and {@link yfiles.graph.ITagOwner#tag}.
       * <p>
       * The node will be a part of this graph after the method returns. This will trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.layout The layout to use initially. The values will be copied to the node's {@link #layout Layout} field
       * @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [options.tag=null] The initial value of the {@link #tag} that will be assigned to the new node.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @returns A newly created node instance
       * @see yfiles.graph.IGraph#addNodeCreatedListener
       * @see yfiles.styles.VoidNodeStyle#INSTANCE
       * @abstract
       */
      createNode(options:{layout:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object,labels?:Array<any>,ports?:Array<any>}):yfiles.graph.INode;
      /**
       * Creates a new ordinary node as a direct descendant of <code>parent</code> using the given layout and style.
       * @param [parent=null] The node to use as the parent in the grouping hierarchy or <code>null</code> if the new node should become a top-level node.
       * @param [layout=null] The layout to use initially. The values will be copied to the node's {@link #layout Layout} field.
       * @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [tag=null] The {@link #tag} to assign to the {@link }.
       * @returns The newly created node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>parent</code> is not in this graph.
       * @see yfiles.graph.IGraph#setParent
       * @see yfiles.graph.IGraph#createNode
       * @see yfiles.graph.IGraph#createGroupNode
       * @abstract
       */
      createNode(parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * Creates a new ordinary node as a direct descendant of <code>parent</code> using the given layout and style.
       * @param {Object} options The parameters to pass.
       * @param [options.parent=null] The node to use as the parent in the grouping hierarchy or <code>null</code> if the new node should become a top-level node.
       * @param [options.layout=null] The layout to use initially. The values will be copied to the node's {@link #layout Layout} field.
       * @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [options.tag=null] The {@link #tag} to assign to the {@link }.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @returns The newly created node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>parent</code> is not in this graph.
       * @see yfiles.graph.IGraph#setParent
       * @see yfiles.graph.IGraph#createNode
       * @see yfiles.graph.IGraph#createGroupNode
       * @abstract
       */
      createNode(options:{parent?:yfiles.graph.INode,layout?:yfiles.geometry.Rect,style?:yfiles.styles.INodeStyle,tag?:Object,labels?:Array<any>,ports?:Array<any>}):yfiles.graph.INode;
      /**
       * Creates and returns a node using the specified initial center location and style, as well as the tag.
       * <p>
       * The node will be a part of this graph after the method returns. This will trigger the corresponding events.
       * </p>
       * @param location the initial coordinates of the center of the node's {@link #layout Layout} property
       * @param [style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [tag=null] The initial value of the {@link #tag} that will be assigned to the new node.
       * @returns A newly created node instance
       * @see yfiles.graph.IGraph#addNodeCreatedListener
       */
      createNodeAt?(location:yfiles.geometry.Point,style?:yfiles.styles.INodeStyle,tag?:Object):yfiles.graph.INode;
      /**
       * Creates and returns a node using the specified initial center location and style, as well as the tag.
       * <p>
       * The node will be a part of this graph after the method returns. This will trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.location the initial coordinates of the center of the node's {@link #layout Layout} property
       * @param [options.style=null] The style instance that will be assigned to the newly created instance. This is done by reference.
       * @param [options.tag=null] The initial value of the {@link #tag} that will be assigned to the new node.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @returns A newly created node instance
       * @see yfiles.graph.IGraph#addNodeCreatedListener
       */
      createNodeAt?(options:{location:yfiles.geometry.Point,style?:yfiles.styles.INodeStyle,tag?:Object,labels?:Array<any>,ports?:Array<any>}):yfiles.graph.INode;
      /**
       * Calculates the number of edges at the given {@link yfiles.graph.IPort} for this graph.
       * <p>
       * Note that an edge that is both incoming and outgoing will be counted twice.
       * </p>
       * @param port The port owner to count the degree of.
       * @returns The number of edges that are incident to the port.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       */
      degree?(port:yfiles.graph.IPort):number;
      /**
       * Calculates the number of edges at the given {@link yfiles.graph.IPortOwner} for this graph.
       * <p>
       * Note that an edge that is both incoming and outgoing will be counted twice.
       * </p>
       * @param owner The port owner to count the degree of.
       * @returns The number of edges that are incident to the port owner.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      degree?(owner:yfiles.graph.IPortOwner):number;
      /**
       * Returns an {@link yfiles.collections.IListEnumerable.<T>} for all edges that have the given port owner as their {@link yfiles.graph.IEdge#targetPort target port's} or {@link yfiles.graph.IEdge#sourcePort source port's} {@link yfiles.graph.IPort#owner} depending on the
       * {@link yfiles.graph.AdjacencyTypes}
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the adjacent
       * edges via a <code>foreach</code> loop.
       * </p>
       * @param [type=yfiles.graph.AdjacencyTypes.ALL] The type of adjacency to consider.
       * @param owner the port owner to check
       * @returns An enumeration of all adjacent edges of the given type.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       * @see yfiles.graph.IGraph#inEdgesAt
       * @see yfiles.graph.IGraph#outEdgesAt
       * @see yfiles.graph.AdjacencyTypes
       * @abstract
       */
      edgesAt(owner:yfiles.graph.IPortOwner,type?:yfiles.graph.AdjacencyTypes):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Returns an {@link yfiles.collections.IEnumerable.<T>} for all edges that are adjacent to the given port as a {@link yfiles.graph.IEdge#sourcePort} or {@link yfiles.graph.IEdge#targetPort}.
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the adjacent
       * edges via a <code>foreach</code> loop.
       * </p>
       * @param port the port to check
       * @param [type=yfiles.graph.AdjacencyTypes.ALL] The type of adjacency to consider.
       * @returns An enumeration of all adjacent edges of the given type.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       * @see yfiles.graph.IGraph#inEdgesAt
       * @see yfiles.graph.IGraph#outEdgesAt
       * @see yfiles.graph.AdjacencyTypes
       * @abstract
       */
      edgesAt(port:yfiles.graph.IPort,type?:yfiles.graph.AdjacencyTypes):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Returns an enumerable over the children of the provided node.
       * @param node The node for which to return the children or <code>null</code> if the top-level nodes should be returned.
       * @returns All nodes that have <code>node</code> as their {@link #getParent parent}.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       * @abstract
       */
      getChildren(node:yfiles.graph.INode):yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * Finds an edge that connects <code>sourcePort</code> and <code>targetPort</code> in the given graph.
       * @param sourcePort The {@link #sourcePort} of the edge to find.
       * @param targetPort The {@link #targetPort} of the edge to find.
       * @returns An edge that satisfies the constraints or <code>null</code>, if none was found.
       * @throws {Stubs.Exceptions.ArgumentError} <code>sourcePort</code> or <code>targetPort</code> are not in this graph.
       */
      getEdge?(sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):yfiles.graph.IEdge;
      /**
       * Finds an edge that connects <code>from</code> and <code>to</code> in the given graph.
       * @param from The {@link #sourcePort} owner of the edge to find.
       * @param to The {@link #targetPort} owner of the edge to find.
       * @returns An edge that satisfies the constraints or <code>null</code>, if none was found.
       * @throws {Stubs.Exceptions.ArgumentError} <code>from</code> or <code>to</code> are not in this graph.
       */
      getEdge?(from:yfiles.graph.IPortOwner,to:yfiles.graph.IPortOwner):yfiles.graph.IEdge;
      /**
       * Gets the {@link yfiles.graph.ILabelDefaults label defaults} for a given {@link yfiles.graph.ILabelOwner} in the context of the <code>graph</code>.
       * @param owner The item that the label defaults are returned for. If this is a group node, the {@link #groupNodeDefaults}'s {@link #labels label defaults} will be returned,
       * otherwise the {@link #nodeDefaults} or {@link #edgeDefaults} {@link #labels labels} will be returned.
       * @returns Appropriate {@link } for the provided <code>owner</code>.
       */
      getLabelDefaults?(owner:yfiles.graph.ILabelOwner):yfiles.graph.ILabelDefaults;
      /**
       * Returns the parent node of the node or <code>null</code> if <code>node</code> is a top-level node.
       * @param node The node to retrieve the parent node for.
       * @returns The parent node in this hierarchy or <code>null</code> if <code>node</code> is a top-level node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       * @abstract
       */
      getParent(node:yfiles.graph.INode):yfiles.graph.INode;
      /**
       * Gets the {@link yfiles.graph.IPortDefaults port defaults} for a given {@link yfiles.graph.IPortOwner} in the context of the <code>graph</code>.
       * @param owner The item that the label defaults are returned for. If this is a group node, the {@link #groupNodeDefaults}'s {@link #ports port defaults} will be returned,
       * otherwise the {@link #nodeDefaults} or {@link #edgeDefaults} {@link #ports ports} will be returned.
       * @returns Appropriate {@link } for the provided <code>owner</code>.
       */
      getPortDefaults?(owner:yfiles.graph.IPortOwner):yfiles.graph.IPortDefaults;
      /**
       * Groups the nodes in <code>children</code> into the provided group node.
       * <p>
       * The <code>parent</code> needs to be a group node at the time of the invocation. This operation is basically the same as calling {@link yfiles.graph.IGraph#setParent}
       * for each node in <code>children</code> whose parent is not part of the set.
       * </p>
       * @param parent The node to use as the parent in the grouping hierarchy.
       * @param children The children to group into the group node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>parent</code> or one of <code>children</code> is not in this graph.
       * @see yfiles.graph.IGraph#groupNodes
       * @see yfiles.graph.IGraph#setParent
       * @see yfiles.graph.IGraph#createGroupNode
       */
      groupNodes?(parent:yfiles.graph.INode,children:yfiles.collections.IEnumerable<yfiles.graph.INode>):void;
      /**
       * Groups the nodes in <code>children</code> into a newly created group node.
       * <p>
       * The group node will be created at the common ancestor level of all nodes in <code>children</code>.
       * </p>
       * @param children The children to group into the new group node.
       * @returns The newly created group node.
       * @throws {Stubs.Exceptions.ArgumentError} One of <code>children</code> is not in this graph.
       * @see yfiles.graph.GroupingSupport#getNearestCommonAncestor
       * @see yfiles.graph.IGraph#groupNodes
       */
      groupNodes?(children:yfiles.collections.IEnumerable<yfiles.graph.INode>):yfiles.graph.INode;
      /**
       * Groups the nodes in <code>children</code> into a newly created group node.
       * <p>
       * The group node will be created at the common ancestor level of all nodes in <code>children</code>.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.children The children to group into the new group node.
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.ports An array of ports to add to the newly created node. Each item will be passed to the {@link #addPort} method.
       * @returns The newly created group node.
       * @throws {Stubs.Exceptions.ArgumentError} One of <code>children</code> is not in this graph.
       * @see yfiles.graph.GroupingSupport#getNearestCommonAncestor
       * @see yfiles.graph.IGraph#groupNodes
       */
      groupNodes?(options:{children:yfiles.collections.IEnumerable<yfiles.graph.INode>,labels?:Array<any>,ports?:Array<any>}):yfiles.graph.INode;
      /**
       * Calculates the number of incoming edges at the given {@link yfiles.graph.IPort} for this graph.
       * @param port The port to count the incoming edges of.
       * @returns The number of edges that have the port as their target port.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       */
      inDegree?(port:yfiles.graph.IPort):number;
      /**
       * Calculates the number of incoming edges at the given {@link yfiles.graph.IPortOwner} for this graph.
       * @param owner The port owner to count the incoming edges of.
       * @returns The number of edges that have the port owner as their target port's owner.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      inDegree?(owner:yfiles.graph.IPortOwner):number;
      /**
       * Convenience method that yields the incoming edges at the given port.
       * <p>
       * This method delegates to {@link yfiles.graph.IGraph#edgesAt} using {@link yfiles.graph.AdjacencyTypes#INCOMING}.
       * </p>
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the incoming
       * edges via a <code>foreach</code> loop.
       * </p>
       * @param port The port of the edges.
       * @returns An enumerable for the edges.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       */
      inEdgesAt?(port:yfiles.graph.IPort):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Convenience method that yields the incoming edges at the given owner.
       * <p>
       * This method delegates to {@link yfiles.graph.IGraph#edgesAt} using {@link yfiles.graph.AdjacencyTypes#INCOMING}.
       * </p>
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the incoming
       * edges via a <code>foreach</code> loop.
       * </p>
       * @param owner The owner of the edges.
       * @returns An enumerable for the edges.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      inEdgesAt?(owner:yfiles.graph.IPortOwner):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Causes the {@link yfiles.graph.IGraph#addDisplaysInvalidatedListener DisplaysInvalidated} event to be triggered.
       * <p>
       * This method may be called by client code to invalidate all views of the graph that have registered with the {@link yfiles.graph.IGraph#addDisplaysInvalidatedListener DisplaysInvalidated}
       * event. Views that need to be informed if non-structural changes have been made to the graph should register with the
       * corresponding event.
       * </p>
       * @abstract
       */
      invalidateDisplays():void;
      /**
       * Returns whether the given node is considered a group node.
       * <p>
       * Group nodes may have {@link yfiles.graph.IGraph#getChildren children} but do not necessarily have to.
       * </p>
       * @param node The node to check.
       * @returns Whether the node is considered a group node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       * @see yfiles.graph.IGraph#getChildren
       * @see yfiles.graph.IGraph#setIsGroupNode
       * @abstract
       */
      isGroupNode(node:yfiles.graph.INode):boolean;
      /**
       * Enumerates the neighbors of a given {@link yfiles.graph.INode}.
       * <p>
       * Neighbors are calculated by going through all {@link yfiles.graph.IPortOwner#ports} and inspecting the {@link yfiles.graph.IGraph#edgesAt edges at these ports}, collecting the
       * {@link yfiles.graph.IEdge#opposite}s.
       * </p>
       * @param node The node.
       * @returns An enumerable over all neighbors.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       */
      neighbors?(node:yfiles.graph.INode):yfiles.collections.IEnumerable<yfiles.graph.INode>;
      /**
       * Calculates the number of outgoing edges at the given {@link yfiles.graph.IPort} for this graph.
       * @param port The port to count the outgoing edges of.
       * @returns The number of edges that have the port as their source port.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       */
      outDegree?(port:yfiles.graph.IPort):number;
      /**
       * Calculates the number of outgoing edges at the given {@link yfiles.graph.IPortOwner} for this graph.
       * @param owner The port owner to count the outgoing edges of.
       * @returns The number of edges that have the port owner as their source port's owner.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      outDegree?(owner:yfiles.graph.IPortOwner):number;
      /**
       * Convenience method that yields the outgoing edges at the given port.
       * <p>
       * This method delegates to {@link yfiles.graph.IGraph#edgesAt} using {@link yfiles.graph.AdjacencyTypes#OUTGOING}.
       * </p>
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the outgoing
       * edges via a <code>foreach</code> loop.
       * </p>
       * @param port The owner of the edges.
       * @returns An enumerable for the edges.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       */
      outEdgesAt?(port:yfiles.graph.IPort):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Convenience method that yields the outgoing edges at the given owner.
       * <p>
       * This method delegates to {@link yfiles.graph.IGraph#edgesAt} using {@link yfiles.graph.AdjacencyTypes#OUTGOING}.
       * </p>
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the outgoing
       * edges via a <code>foreach</code> loop.
       * </p>
       * @param owner The owner of the edges.
       * @returns An enumerable for the edges.
       * @throws {Stubs.Exceptions.ArgumentError} <code>owner</code> is not in this graph.
       */
      outEdgesAt?(owner:yfiles.graph.IPortOwner):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Enumerates the predecessors of a given {@link yfiles.graph.INode}.
       * <p>
       * Predecessors are calculated by going through all {@link yfiles.graph.IPortOwner#ports} and inspecting the {@link yfiles.graph.IGraph#inEdgesAt incoming edges at these ports}, collecting the {@link yfiles.graph.IEdge#sourcePort} {@link yfiles.graph.IPort#owner}s.
       * </p>
       * @param node The node.
       * @returns An enumerable over all predecessors.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       */
      predecessors?(node:yfiles.graph.INode):yfiles.collections.IEnumerable<yfiles.graph.INode>;
      /**
       * Removes the given {@link yfiles.graph.IModelItem item} from this graph.
       * <p>
       * The item must be a part of this graph.
       * </p>
       * <p>
       * If the item is a {@link yfiles.graph.INode node} the {@link yfiles.graph.IGraph#addNodeRemovedListener NodeRemoved} event will be triggered. The
       * implementation will remove all adjacent edges and their corresponding ports in proper order before the node will be
       * removed. Also this will trigger the removal of all labels owned by this instance.
       * </p>
       * <p>
       * If the item is an {@link yfiles.graph.IEdge edge} the {@link yfiles.graph.IGraph#addEdgeRemovedListener EdgeRemoved} event will be triggered. The
       * implementation may decide to remove the corresponding ports from the node if no other edge connects to them after the
       * given edge has been removed. Also this will trigger the removal of all labels and bends owned by this instance.
       * </p>
       * <p>
       * If the item is a {@link yfiles.graph.IBend bend} the {@link yfiles.graph.IGraph#addBendRemovedListener BendRemoved} event will be triggered.
       * </p>
       * <p>
       * If the item is a {@link yfiles.graph.IPort port} the {@link yfiles.graph.IGraph#addPortRemovedListener PortRemoved} event will be triggered. This
       * will also remove all edges that are currently connected to the port.
       * </p>
       * <p>
       * If the item is a {@link yfiles.graph.ILabel label} the {@link yfiles.graph.IGraph#addLabelRemovedListener LabelRemoved} event will be triggered.
       * </p>
       * @param item the live item to be removed from this graph instance
       * @throws {Stubs.Exceptions.ArgumentError} <code>item</code> is not in this graph.
       * @abstract
       */
      remove(item:yfiles.graph.IModelItem):void;
      /**
       * Reverses an edge by {@link yfiles.graph.IGraph#setEdgePorts setting source and target port} to {@link yfiles.graph.IEdge#targetPort} and {@link yfiles.graph.IEdge#sourcePort}.
       * <p>
       * This also reverses the bends by clearing them and reinserting them in reverse order if there is more than one bend.
       * </p>
       * @param edge The edge to reverse.
       * @throws {Stubs.Exceptions.ArgumentError} <code>edge</code> is not in this graph.
       */
      reverse?(edge:yfiles.graph.IEdge):void;
      /**
       * Modifies the location of the given bend.
       * @param bend the bend whose location is to be modified
       * @param location the new location of the bend
       * @throws {Stubs.Exceptions.ArgumentError} <code>bend</code> is not in this graph.
       * @see yfiles.graph.IGraph#addBend
       * @abstract
       */
      setBendLocation(bend:yfiles.graph.IBend,location:yfiles.geometry.Point):void;
      /**
       * Sets the ports of the given edge to the new values.
       * <p>
       * This will trigger an {@link yfiles.graph.IGraph#addEdgePortsChangedListener EdgePortsChanged} event if source or target ports
       * differ from the current ones. Both ports and the edge must belong to the current graph instance.
       * </p>
       * @param edge The edge to change the ports.
       * @param sourcePort The new source port instance.
       * @param targetPort The new target port instance.
       * @throws {Stubs.Exceptions.ArgumentError} Either <code>edge</code>, <code>sourcePort</code>, or <code>targetPort</code> are not in this graph.
       * @abstract
       */
      setEdgePorts(edge:yfiles.graph.IEdge,sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):void;
      /**
       * Determines whether the given node should be considered a group node.
       * <p>
       * Group nodes may have {@link yfiles.graph.IGraph#getChildren children} but do not necessarily need to have one. Attempting to set a node to the
       * non-group-node-status while it has children at the same time will result in an {@link yfiles.lang.Exception}.
       * </p>
       * @param node The node to set the group node status for.
       * @param isGroupNode Whether to make the node a group node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       * @throws {Stubs.Exceptions.InvalidOperationError} <code>node</code> is <code>null</code> or currently has children and <code>isGroupNode</code> is <code>false</code>.
       * @see yfiles.graph.IGraph#getChildren
       * @see yfiles.graph.IGraph#isGroupNode
       * @abstract
       */
      setIsGroupNode(node:yfiles.graph.INode,isGroupNode:boolean):void;
      /**
       * Sets the label model parameter for the given label.
       * @param label The label.
       * @param layoutParameter The new parameter.
       * @throws {Stubs.Exceptions.ArgumentError} <code>label</code> is not in this graph, or <code>layoutParameter</code> cannot be used for <code>label</code>.
       * @abstract
       */
      setLabelLayoutParameter(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * Sets the preferred size of the label.
       * @param label The label.
       * @param preferredSize The new preferred size.
       * @throws {Stubs.Exceptions.ArgumentError} <code>label</code> is not in this graph.
       * @see yfiles.graph.ILabel#preferredSize
       * @abstract
       */
      setLabelPreferredSize(label:yfiles.graph.ILabel,preferredSize:yfiles.geometry.Size):void;
      /**
       * Sets the label text of the given label.
       * @param label the label to modify
       * @param text the new text of the label
       * @throws {Stubs.Exceptions.ArgumentError} <code>label</code> is not in this graph.
       * @see yfiles.graph.ILabel#text
       * @abstract
       */
      setLabelText(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Sets the center of a node to the given world coordinates.
       * <p>
       * This implementation delegates to {@link yfiles.graph.IGraph#setNodeLayout}
       * </p>
       * @param node The node to recenter.
       * @param center The new center coordinates of the node in the world coordinate system.
       */
      setNodeCenter?(node:yfiles.graph.INode,center:yfiles.geometry.Point):void;
      /**
       * Sets the layout of the given node to the new value.
       * @param node a live node that belongs to this graph
       * @param layout The new layout of the node to assign to its {@link #layout}.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       * @see yfiles.graph.INode#layout
       * @abstract
       */
      setNodeLayout(node:yfiles.graph.INode,layout:yfiles.geometry.Rect):void;
      /**
       * Sets the parent node for a given node.
       * <p>
       * Use <code>null</code> as <code>parent</code> to make <code>node</code> a top-level node for this graph.
       * </p>
       * @param node The node to assign a new parent.
       * @param parent The parent group node to assign to <code>node</code> or <code>null</code> to make <code>node</code> a top-level node.
       * @throws {Stubs.Exceptions.ArgumentError} Either <code>node</code> or <code>parent</code> are not in this graph.
       * @see yfiles.graph.IGraph#getParent
       * @abstract
       */
      setParent(node:yfiles.graph.INode,parent:yfiles.graph.INode):void;
      /**
       * Convenience method that tries to set the absolute coordinates of the given port to the given values.
       * <p>
       * For full control over the placement of the ports, the {@link yfiles.graph.IGraph#setPortLocationParameter} method should be used
       * instead. This implementation will use the port's {@link yfiles.graph.IPort#locationParameter}'s {@link yfiles.graph.IPortLocationModelParameter#model} to obtain a new {@link yfiles.graph.IPortLocationModelParameter} via the {@link yfiles.graph.IPortLocationModel#createParameter}
       * method. This might result in the port using a different location, because the model might not support parameters that
       * result in the given location. This will also trigger an {@link yfiles.graph.IGraph#invalidateDisplays} call.
       * </p>
       * @param port The port to modify
       * @param location the new absolute coordinates of the port
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       * @see yfiles.graph.IGraph#addPort
       */
      setPortLocation?(port:yfiles.graph.IPort,location:yfiles.geometry.Point):void;
      /**
       * Sets a new {@link yfiles.graph.IPortLocationModelParameter} for the given port.
       * @param port The port to modify
       * @param locationParameter the new parameter that determines the coordinates of the port
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph, or <code>locationParameter</code> cannot be used for <code>port</code>.
       * @abstract
       */
      setPortLocationParameter(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):void;
      /**
       * Tries to set the location of the port relative to its {@link yfiles.graph.IPort#owner owner} if the owner is a {@link yfiles.graph.INode node}.
       * <p>
       * If the port is not owned by a node that is part of this graph, this method will throw an {@link yfiles.lang.Exception}. This
       * method will delegate to {@link yfiles.graph.IGraph#setPortLocation}.
       * </p>
       * @param port the port
       * @param relativeLocation the new coordinate offsets relative to the center of the node's {@link #layout}'s center.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph or has no owner.
       * @see yfiles.graph.INode#layout
       */
      setRelativePortLocation?(port:yfiles.graph.IPort,relativeLocation:yfiles.geometry.Point):void;
      /**
       * Assigns the given style instance by reference to the port.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param port The port that will be assigned the new style
       * @param style The style instance that will be assigned to the port.
       * @throws {Stubs.Exceptions.ArgumentError} <code>port</code> is not in this graph.
       * @see yfiles.graph.IPort#style
       * @see yfiles.graph.IGraph#addPortStyleChangedListener
       * @see yfiles.styles.VoidPortStyle#INSTANCE
       * @abstract
       */
      setStyle(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):void;
      /**
       * Assigns the given style instance by reference to the edge.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param edge The edge that will be assigned the new style
       * @param style The style instance that will be assigned to the edge.
       * @throws {Stubs.Exceptions.ArgumentError} <code>edge</code> is not in this graph.
       * @see yfiles.graph.IEdge#style
       * @see yfiles.graph.IGraph#addEdgeStyleChangedListener
       * @see yfiles.styles.VoidEdgeStyle#INSTANCE
       * @abstract
       */
      setStyle(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):void;
      /**
       * Assigns the given style instance by reference to the label.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param label The label that will be assigned the new style
       * @param style The style instance that will be assigned to the label.
       * @throws {Stubs.Exceptions.ArgumentError} <code>label</code> is not in this graph.
       * @see yfiles.graph.ILabel#style
       * @see yfiles.graph.IGraph#addLabelStyleChangedListener
       * @see yfiles.styles.VoidLabelStyle#INSTANCE
       * @abstract
       */
      setStyle(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):void;
      /**
       * Assigns the given style instance by reference to the node.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param node The node that will be assigned the new style
       * @param style The style instance that will be assigned to the node.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       * @see yfiles.graph.INode#style
       * @see yfiles.graph.IGraph#addNodeStyleChangedListener
       * @see yfiles.styles.VoidNodeStyle#INSTANCE
       * @abstract
       */
      setStyle(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):void;
      /**
       * Enumerates the successors of a given {@link yfiles.graph.INode}.
       * <p>
       * Successors are calculated by going through all {@link yfiles.graph.IPortOwner#ports} and inspecting the {@link yfiles.graph.IGraph#outEdgesAt outgoing edges at these ports}, collecting the {@link yfiles.graph.IEdge#targetPort} {@link yfiles.graph.IPort#owner}s.
       * </p>
       * @param node The node.
       * @returns An enumerable over all successors.
       * @throws {Stubs.Exceptions.ArgumentError} <code>node</code> is not in this graph.
       */
      successors?(node:yfiles.graph.INode):yfiles.collections.IEnumerable<yfiles.graph.INode>;
      /**
       * A view of the nodes contained in this graph.
       * <p>
       * This is a live view of the nodes that always represents the current state of the graph. The same reference will be
       * returned for each invocation.
       * </p>
       * <p>
       * Note that even though nodes can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the nodes in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the nodes via a <code>foreach</code>
       * loop.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.INode>}
       */
      nodes:yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * A view of the edges contained in this graph.
       * <p>
       * This is a live view of the edges that always represents the current state of the graph. The same reference will be
       * returned for each invocation.
       * </p>
       * <p>
       * Note that even though edges can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the edges in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the edges via a <code>foreach</code>
       * loop.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IEdge>}
       */
      edges:yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * A view of the node and edge labels contained in this graph.
       * <p>
       * This is a live view of the labels that always represents the current state of the graph. The same reference will be
       * returned for each invocation.
       * </p>
       * <p>
       * Note that even though labels can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the labels in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the labels via a
       * <code>foreach</code> loop.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
      /**
       * A view of the ports contained in this graph.
       * <p>
       * This is a live view of the ports that always represents the current state of the graph. The same reference will be
       * returned for each invocation.
       * </p>
       * <p>
       * Note that even though ports can be accessed via index, the underlying graph structure in the default {@link yfiles.graph.IGraph}
       * implementation is a linked list and indexed access can be slow. In those cases it is recommended to store the ports in
       * your own list, if possible. This is not necessary for the first or last element or when iterating over the ports via a <code>foreach</code>
       * loop.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
      /**
       * Gets or sets the defaults for normal nodes.
       * <p>
       * The settings that are obtained from the instance influence newly created elements only. Setting different defaults
       * afterwards does not influence existing elements.
       * </p>
       * @abstract
       * @type {yfiles.graph.INodeDefaults}
       */
      nodeDefaults:yfiles.graph.INodeDefaults;
      /**
       * Gets or sets the defaults for normal edges.
       * <p>
       * The settings that are obtained from the instance influence newly created elements only. Setting different defaults
       * afterwards does not influence existing elements.
       * </p>
       * @abstract
       * @type {yfiles.graph.IEdgeDefaults}
       */
      edgeDefaults:yfiles.graph.IEdgeDefaults;
      /**
       * Gets the mapper registry that is associated with this graph instance.
       * <p>
       * The registry can be used to store data mappings for the items in this instance.
       * </p>
       * @see yfiles.graph.IMapperRegistry
       * @see yfiles.graph.IModelItem
       * @see yfiles.graph.ITagOwner#tag
       * @abstract
       * @type {yfiles.graph.IMapperRegistry}
       */
      mapperRegistry:yfiles.graph.IMapperRegistry;
      /**
       * Adds the given listener for the <code>GraphTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of the graph has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#removeGraphTagChangedListener
       * @abstract
       */
      addGraphTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>)=>void):void;
      /**
       * Removes the given listener for the <code>GraphTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of the graph has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#addGraphTagChangedListener
       * @abstract
       */
      removeGraphTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IGraph,Object>)=>void):void;
      /**
       * Adds the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addNodeCreatedListener NodeCreated}
       * event if you are interested only in node creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#createNode
       * @see yfiles.graph.IGraph#createNode
       * @see yfiles.graph.IGraph#createGroupNode
       * @see yfiles.input.GraphEditorInputMode#addNodeCreatedListener
       * @see yfiles.graph.IGraph#removeNodeCreatedListener
       * @abstract
       */
      addNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Removes the given listener for the <code>NodeCreated</code> event that occurs when a node has been created.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addNodeCreatedListener NodeCreated}
       * event if you are interested only in node creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#createNode
       * @see yfiles.graph.IGraph#createNode
       * @see yfiles.graph.IGraph#createGroupNode
       * @see yfiles.input.GraphEditorInputMode#addNodeCreatedListener
       * @see yfiles.graph.IGraph#addNodeCreatedListener
       * @abstract
       */
      removeNodeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.INode>)=>void):void;
      /**
       * Adds the given listener for the <code>DisplaysInvalidated</code> event that occurs when the graph has changed visually and the
       * display should be updated to reflect the changes.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#removeDisplaysInvalidatedListener
       * @abstract
       */
      addDisplaysInvalidatedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>DisplaysInvalidated</code> event that occurs when the graph has changed visually and the
       * display should be updated to reflect the changes.
       * <p>
       * This event is invoked with {@link yfiles.lang.EventArgs#EMPTY} per default.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#addDisplaysInvalidatedListener
       * @abstract
       */
      removeDisplaysInvalidatedListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in node removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#removeNodeRemovedListener
       * @abstract
       */
      addNodeRemovedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>NodeRemoved</code> event that occurs when a node has been removed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in node removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the node removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#addNodeRemovedListener
       * @abstract
       */
      removeNodeRemovedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>NodeStyleChanged</code> event that occurs when a node style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#removeNodeStyleChangedListener
       * @abstract
       */
      addNodeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>)=>void):void;
      /**
       * Removes the given listener for the <code>NodeStyleChanged</code> event that occurs when a node style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#addNodeStyleChangedListener
       * @abstract
       */
      removeNodeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,yfiles.styles.INodeStyle>)=>void):void;
      /**
       * Adds the given listener for the <code>NodeLayoutChanged</code> event that occurs when a node layout has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setNodeLayout
       * @see yfiles.graph.IGraph#removeNodeLayoutChangedListener
       * @abstract
       */
      addNodeLayoutChangedListener(listener:(source:Object,node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect)=>void):void;
      /**
       * Removes the given listener for the <code>NodeLayoutChanged</code> event that occurs when a node layout has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setNodeLayout
       * @see yfiles.graph.IGraph#addNodeLayoutChangedListener
       * @abstract
       */
      removeNodeLayoutChangedListener(listener:(source:Object,node:yfiles.graph.INode,oldLayout:yfiles.geometry.Rect)=>void):void;
      /**
       * Adds the given listener for the <code>NodeTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a node has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#removeNodeTagChangedListener
       * @abstract
       */
      addNodeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>)=>void):void;
      /**
       * Removes the given listener for the <code>NodeTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a node has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#addNodeTagChangedListener
       * @abstract
       */
      removeNodeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.INode,Object>)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.CreateEdgeInputMode#addEdgeCreatedListener EdgeCreated}
       * event if you are interested only in edge creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#createEdge
       * @see yfiles.input.CreateEdgeInputMode#addEdgeCreatedListener
       * @see yfiles.graph.IGraph#removeEdgeCreatedListener
       * @abstract
       */
      addEdgeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeCreated</code> event that occurs when an edge has been created.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.CreateEdgeInputMode#addEdgeCreatedListener EdgeCreated}
       * event if you are interested only in edge creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#createEdge
       * @see yfiles.input.CreateEdgeInputMode#addEdgeCreatedListener
       * @see yfiles.graph.IGraph#addEdgeCreatedListener
       * @abstract
       */
      removeEdgeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
       * <p>
       * This event will be triggered, too, prior to a node removal.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in edge removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#removeEdgeRemovedListener
       * @abstract
       */
      addEdgeRemovedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeRemoved</code> event that occurs when an edge has been removed.
       * <p>
       * This event will be triggered, too, prior to a node removal.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in edge removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the edge removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#addEdgeRemovedListener
       * @abstract
       */
      removeEdgeRemovedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeStyleChanged</code> event that occurs when an edge style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#removeEdgeStyleChangedListener
       * @abstract
       */
      addEdgeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeStyleChanged</code> event that occurs when an edge style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#addEdgeStyleChangedListener
       * @abstract
       */
      removeEdgeStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,yfiles.styles.IEdgeStyle>)=>void):void;
      /**
       * Adds the given listener for the <code>EdgePortsChanged</code> event that occurs when an edge had its {@link yfiles.graph.IEdge#sourcePort} or {@link yfiles.graph.IEdge#targetPort} {@link yfiles.graph.IGraph#setEdgePorts changed}.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setEdgePorts
       * @see yfiles.graph.IGraph#removeEdgePortsChangedListener
       * @abstract
       */
      addEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>EdgePortsChanged</code> event that occurs when an edge had its {@link yfiles.graph.IEdge#sourcePort} or {@link yfiles.graph.IEdge#targetPort} {@link yfiles.graph.IGraph#setEdgePorts changed}.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setEdgePorts
       * @see yfiles.graph.IGraph#addEdgePortsChangedListener
       * @abstract
       */
      removeEdgePortsChangedListener(listener:(sender:Object,evt:yfiles.graph.EdgeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>EdgeTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of an edge has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#removeEdgeTagChangedListener
       * @abstract
       */
      addEdgeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>)=>void):void;
      /**
       * Removes the given listener for the <code>EdgeTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of an edge has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#addEdgeTagChangedListener
       * @abstract
       */
      removeEdgeTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IEdge,Object>)=>void):void;
      /**
       * Adds the given listener for the <code>BendAdded</code> event that occurs when a bend has been added to an edge in this graph.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.CreateBendInputMode#addBendCreatedListener BendCreated}
       * event if you are interested only in bend creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#addBend
       * @see yfiles.input.CreateBendInputMode#addBendCreatedListener
       * @see yfiles.graph.IGraph#removeBendAddedListener
       * @abstract
       */
      addBendAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * Removes the given listener for the <code>BendAdded</code> event that occurs when a bend has been added to an edge in this graph.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.CreateBendInputMode#addBendCreatedListener BendCreated}
       * event if you are interested only in bend creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#addBend
       * @see yfiles.input.CreateBendInputMode#addBendCreatedListener
       * @see yfiles.graph.IGraph#addBendAddedListener
       * @abstract
       */
      removeBendAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IBend>)=>void):void;
      /**
       * Adds the given listener for the <code>BendRemoved</code> event that occurs when a bend has been removed from an edge in this graph.
       * <p>
       * This event will be triggered, too, if an edge has been removed from the graph, for each of the bends that belonged to
       * the edge.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in bend removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#removeBendRemovedListener
       * @abstract
       */
      addBendRemovedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>BendRemoved</code> event that occurs when a bend has been removed from an edge in this
       * graph.
       * <p>
       * This event will be triggered, too, if an edge has been removed from the graph, for each of the bends that belonged to
       * the edge.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in bend removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the bend removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#addBendRemovedListener
       * @abstract
       */
      removeBendRemovedListener(listener:(sender:Object,evt:yfiles.graph.BendEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>BendLocationChanged</code> event that occurs when the location of a bend has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setBendLocation
       * @see yfiles.graph.IGraph#removeBendLocationChangedListener
       * @abstract
       */
      addBendLocationChangedListener(listener:(source:Object,bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point)=>void):void;
      /**
       * Removes the given listener for the <code>BendLocationChanged</code> event that occurs when the location of a bend has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setBendLocation
       * @see yfiles.graph.IGraph#addBendLocationChangedListener
       * @abstract
       */
      removeBendLocationChangedListener(listener:(source:Object,bend:yfiles.graph.IBend,oldLocation:yfiles.geometry.Point)=>void):void;
      /**
       * Adds the given listener for the <code>BendTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a bend has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#removeBendTagChangedListener
       * @abstract
       */
      addBendTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>)=>void):void;
      /**
       * Removes the given listener for the <code>BendTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a bend has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#addBendTagChangedListener
       * @abstract
       */
      removeBendTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IBend,Object>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addLabelAddedListener LabelAdded}
       * event if you are interested only in label creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#addLabel
       * @see yfiles.input.GraphEditorInputMode#addLabelAddedListener
       * @see yfiles.graph.IGraph#removeLabelAddedListener
       * @abstract
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addLabelAddedListener LabelAdded}
       * event if you are interested only in label creation events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#addLabel
       * @see yfiles.input.GraphEditorInputMode#addLabelAddedListener
       * @see yfiles.graph.IGraph#addLabelAddedListener
       * @abstract
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph instance.
       * <p>
       * This event will also be triggered, prior to the removal of the {@link yfiles.graph.ILabel#owner owner} of the label.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in label removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.input.GraphEditorInputMode#addLabelTextChangedListener
       * @see yfiles.graph.IGraph#removeLabelRemovedListener
       * @abstract
       */
      addLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph
       * instance.
       * <p>
       * This event will also be triggered, prior to the removal of the {@link yfiles.graph.ILabel#owner owner} of the label.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in label removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the label removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.input.GraphEditorInputMode#addLabelTextChangedListener
       * @see yfiles.graph.IGraph#addLabelRemovedListener
       * @abstract
       */
      removeLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelStyleChanged</code> event that occurs when a label style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#removeLabelStyleChangedListener
       * @abstract
       */
      addLabelStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelStyleChanged</code> event that occurs when a label style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#addLabelStyleChangedListener
       * @abstract
       */
      removeLabelStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.styles.ILabelStyle>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelPreferredSizeChanged</code> event that occurs when the preferred size of a label has been
       * changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setLabelPreferredSize
       * @see yfiles.graph.IGraph#removeLabelPreferredSizeChangedListener
       * @abstract
       */
      addLabelPreferredSizeChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelPreferredSizeChanged</code> event that occurs when the preferred size of a label has
       * been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setLabelPreferredSize
       * @see yfiles.graph.IGraph#addLabelPreferredSizeChangedListener
       * @abstract
       */
      removeLabelPreferredSizeChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.geometry.Size>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTextChanged</code> event that occurs when the text of a label has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setLabelText
       * @see yfiles.graph.IGraph#removeLabelTextChangedListener
       * @abstract
       */
      addLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTextChanged</code> event that occurs when the text of a label has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setLabelText
       * @see yfiles.graph.IGraph#addLabelTextChangedListener
       * @abstract
       */
      removeLabelTextChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,string>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelLayoutParameterChanged</code> event that occurs when the model parameter of a label has
       * been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setLabelLayoutParameter
       * @see yfiles.graph.IGraph#removeLabelLayoutParameterChangedListener
       * @abstract
       */
      addLabelLayoutParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelLayoutParameterChanged</code> event that occurs when the model parameter of a label has
       * been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setLabelLayoutParameter
       * @see yfiles.graph.IGraph#addLabelLayoutParameterChangedListener
       * @abstract
       */
      removeLabelLayoutParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,yfiles.graph.ILabelModelParameter>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a label has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#removeLabelTagChangedListener
       * @abstract
       */
      addLabelTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a label has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#addLabelTagChangedListener
       * @abstract
       */
      removeLabelTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.ILabel,Object>)=>void):void;
      /**
       * Adds the given listener for the <code>PortAdded</code> event that occurs when a port has been added to this graph instance.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#addPort
       * @see yfiles.graph.IGraph#createEdge
       * @see yfiles.graph.IGraph#removePortAddedListener
       * @abstract
       */
      addPortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * Removes the given listener for the <code>PortAdded</code> event that occurs when a port has been added to this graph instance.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port creation
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#addPort
       * @see yfiles.graph.IGraph#createEdge
       * @see yfiles.graph.IGraph#addPortAddedListener
       * @abstract
       */
      removePortAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IPort>)=>void):void;
      /**
       * Adds the given listener for the <code>PortRemoved</code> event that occurs when a port has been removed from its {@link yfiles.graph.IPort#owner owner}.
       * <p>
       * This event will also be triggered prior to the removal of the corresponding owner of the port.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in port removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#removePortRemovedListener
       * @abstract
       */
      addPortRemovedListener(listener:(sender:Object,evt:yfiles.graph.PortEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PortRemoved</code> event that occurs when a port has been removed from its {@link yfiles.graph.IPort#owner owner}.
       * <p>
       * This event will also be triggered prior to the removal of the corresponding owner of the port.
       * </p>
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure. Please use the {@link yfiles.input.GraphEditorInputMode#addDeletedItemListener DeletedItem}
       * event if you are interested only in port removal events that result from user interaction.
       * <b>Note:</b>You may not modify the graph in the event handler for this event. Especially you may not prevent/undo the port removal
       * that has triggered this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#remove
       * @see yfiles.input.GraphEditorInputMode#addDeletedItemListener
       * @see yfiles.graph.IGraph#addPortRemovedListener
       * @abstract
       */
      removePortRemovedListener(listener:(sender:Object,evt:yfiles.graph.PortEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>PortStyleChanged</code> event that occurs when a port style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#removePortStyleChangedListener
       * @abstract
       */
      addPortStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>)=>void):void;
      /**
       * Removes the given listener for the <code>PortStyleChanged</code> event that occurs when a port style has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setStyle
       * @see yfiles.graph.IGraph#addPortStyleChangedListener
       * @abstract
       */
      removePortStyleChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.styles.IPortStyle>)=>void):void;
      /**
       * Adds the given listener for the <code>PortLocationParameterChanged</code> event that occurs when the location model parameter of a
       * port has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#setPortLocationParameter
       * @see yfiles.graph.IGraph#removePortLocationParameterChangedListener
       * @abstract
       */
      addPortLocationParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>)=>void):void;
      /**
       * Removes the given listener for the <code>PortLocationParameterChanged</code> event that occurs when the location model parameter of a
       * port has been changed.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#setPortLocationParameter
       * @see yfiles.graph.IGraph#addPortLocationParameterChangedListener
       * @abstract
       */
      removePortLocationParameterChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,yfiles.graph.IPortLocationModelParameter>)=>void):void;
      /**
       * Adds the given listener for the <code>PortTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a port has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#removePortTagChangedListener
       * @abstract
       */
      addPortTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>)=>void):void;
      /**
       * Removes the given listener for the <code>PortTagChanged</code> event that occurs when the {@link yfiles.graph.ITagOwner#tag tag} of a port has been replaced.
       * <p>
       * This event is intended to provide notification of low level changes in the graph structure.
       * <b>Note:</b>You may not modify the graph in the event handler for this event.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITagOwner#tag
       * @see yfiles.graph.IGraph#addPortTagChangedListener
       * @abstract
       */
      removePortTagChangedListener(listener:(sender:Object,evt:yfiles.graph.ItemChangedEventArgs<yfiles.graph.IPort,Object>)=>void):void;
      /**
       * Gets or sets the defaults for group nodes.
       * @abstract
       * @type {yfiles.graph.INodeDefaults}
       */
      groupNodeDefaults:yfiles.graph.INodeDefaults;
      /**
       * Adds the given listener for the <code>ParentChanged</code>
       * event.
       * <br />
       * Event that will be triggered if an node has been reparented in the model.
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#removeParentChangedListener
       * @abstract
       */
      addParentChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>ParentChanged</code>
       * event.
       * <br />
       * Event that will be triggered if an node has been reparented in the model.
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#addParentChangedListener
       * @abstract
       */
      removeParentChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>IsGroupNodeChanged</code>
       * event.
       * <br />
       * Event that will be triggered if the group node status of a node has changed.
       * @param listener The listener to add.
       * @see yfiles.graph.IGraph#removeIsGroupNodeChangedListener
       * @abstract
       */
      addIsGroupNodeChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>IsGroupNodeChanged</code>
       * event.
       * <br />
       * Event that will be triggered if the group node status of a node has changed.
       * @param listener The listener to remove.
       * @see yfiles.graph.IGraph#addIsGroupNodeChangedListener
       * @abstract
       */
      removeIsGroupNodeChangedListener(listener:(sender:Object,evt:yfiles.graph.NodeEventArgs)=>void):void;
      /**
       * Gets a {@link yfiles.graph.GraphDecorator} instance for use with the given graph.
       * @type {yfiles.graph.GraphDecorator}
       */
      decorator?:yfiles.graph.GraphDecorator;
      /**
       * Creates a {@link yfiles.graph.GroupingSupport} instance for the graph.
       * <p>
       * {@link yfiles.graph.GroupingSupport} provides less frequently used methods for grouped graphs, such as methods for analyzing and
       * walking the grouping hierarchy.
       * </p>
       * @type {yfiles.graph.GroupingSupport}
       */
      groupingSupport?:yfiles.graph.GroupingSupport;
      /**
       * Gets the {@link yfiles.graph.IFoldingView folding view} instance associated with this {@link yfiles.graph.IGraph} or <code>null</code> if none is associated with it.
       * <p>
       * This convenience method uses the {@link yfiles.graph.ILookup#lookup} of the {@link yfiles.graph.IGraph} to obtain the
       * {@link yfiles.graph.IFoldingView}.
       * </p>
       * @see yfiles.graph.FoldingManager
       * @see yfiles.graph.FoldingManager#createFoldingView
       * @type {yfiles.graph.IFoldingView}
       */
      foldingView?:yfiles.graph.IFoldingView;
      /**
       * Gets the {@link yfiles.graph.UndoEngine} instance associated with this {@link yfiles.graph.IGraph} or <code>null</code> if none is associated with
       * it.
       * <p>
       * This convenience method uses the {@link yfiles.graph.ILookup#lookup} of the {@link yfiles.graph.IGraph} to obtain the
       * {@link yfiles.graph.UndoEngine}.
       * </p>
       * @type {yfiles.graph.UndoEngine}
       */
      undoEngine?:yfiles.graph.UndoEngine;
      /**
       * Gets or sets whether or not the {@link yfiles.graph.UndoEngine} used for this instance should be enabled.
       * @see yfiles.graph.IGraph#undoEngine
       * @type {boolean}
       */
      undoEngineEnabled?:boolean;
      /**
       * Gets a live view of all bends contained in this {@link yfiles.graph.IGraph}.
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IBend>}
       */
      bends?:yfiles.collections.IEnumerable<yfiles.graph.IBend>;
      /**
       * Gets a live view of all edge labels contained in this {@link yfiles.graph.IGraph}.
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.ILabel>}
       */
      edgeLabels?:yfiles.collections.IEnumerable<yfiles.graph.ILabel>;
      /**
       * Gets a live view of all node labels contained in this {@link yfiles.graph.IGraph}.
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.ILabel>}
       */
      nodeLabels?:yfiles.collections.IEnumerable<yfiles.graph.ILabel>;
    }
    var IGraph:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A specialized subclass of the {@link yfiles.collections.ItemEventArgs.<T>} class that is bound to the {@link yfiles.graph.INode} type and carries additional information
     * about the parent node and the group node status.
     * <p>
     * Since for some events ({@link yfiles.graph.IGraph#addNodeRemovedListener NodeRemoved},
     * {@link yfiles.graph.IGraph#addIsGroupNodeChangedListener IsGroupNodeChanged},
     * {@link yfiles.graph.IGraph#addParentChangedListener ParentChanged}) it is unclear which {@link yfiles.graph.NodeEventArgs#parent} or {@link yfiles.graph.NodeEventArgs#isGroupNode} the node had before the event
     * this event can be used to carry exactly that information.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.INode>}
     * @final
     */
    export interface NodeEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.INode>{}
    export class NodeEventArgs {
      /**
       * Creates a new instance.
       * @param item The node that has changed.
       * @param oldParent The parent the node had before this event.
       * @param oldIsGroupNode Whether the node was a group node before this event.
       */
      constructor(item:yfiles.graph.INode,oldParent:yfiles.graph.INode,oldIsGroupNode:boolean);
      /**
       * The parent of the {@link yfiles.collections.ItemEventArgs.<T>#item node} before this event.
       * @type {yfiles.graph.INode}
       */
      parent:yfiles.graph.INode;
      /**
       * Whether the {@link yfiles.collections.ItemEventArgs.<T>#item node} was a group node before this event.
       * @type {boolean}
       */
      isGroupNode:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of the {@link yfiles.collections.ItemEventArgs.<T>} class that is bound to the {@link yfiles.graph.IEdge} type and carries additional source and
     * target port information.
     * <p>
     * Since for some events ({@link yfiles.graph.IGraph#addEdgePortsChangedListener EdgePortsChanged} and
     * {@link yfiles.graph.IGraph#addEdgeRemovedListener EdgeRemoved}) it is unclear what {@link yfiles.graph.IEdge#sourcePort} and {@link yfiles.graph.IEdge#targetPort} the edge had been connected to
     * before the event, this event can be used to carry exactly that information.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.IEdge>}
     * @final
     */
    export interface EdgeEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.IEdge>{}
    export class EdgeEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graph.EdgeEventArgs} class.
       * @param edge The edge to assign to the {@link #item} property.
       * @param [sourcePort=null] The source port the edge was connected to before the event happened.
       * @param [targetPort=null] The target port the edge was connected to before the event happened.
       * @param [sourcePortOwner=null] The owner of the source port the edge was connected to before the event happened.
       * @param [targetPortOwner=null] The owner of the target port the edge was connected to before the event happened.
       */
      constructor(edge:yfiles.graph.IEdge,sourcePort?:yfiles.graph.IPort,targetPort?:yfiles.graph.IPort,sourcePortOwner?:yfiles.graph.IPortOwner,targetPortOwner?:yfiles.graph.IPortOwner);
      /**
       * Initializes a new instance of the {@link yfiles.graph.EdgeEventArgs} class.
       * @param {Object} options The parameters to pass.
       * @param options.edge The edge to assign to the {@link #item} property.
       * @param [options.sourcePort=null] The source port the edge was connected to before the event happened.
       * @param [options.targetPort=null] The target port the edge was connected to before the event happened.
       * @param [options.sourcePortOwner=null] The owner of the source port the edge was connected to before the event happened.
       * @param [options.targetPortOwner=null] The owner of the target port the edge was connected to before the event happened.
       */
      constructor(options:{edge:yfiles.graph.IEdge,sourcePort?:yfiles.graph.IPort,targetPort?:yfiles.graph.IPort,sourcePortOwner?:yfiles.graph.IPortOwner,targetPortOwner?:yfiles.graph.IPortOwner});
      /**
       * Gets the source port the edge was connected to before the event happened.
       * @type {yfiles.graph.IPort}
       */
      sourcePort:yfiles.graph.IPort;
      /**
       * Gets the target port the edge was connected to before the event happened.
       * @type {yfiles.graph.IPort}
       */
      targetPort:yfiles.graph.IPort;
      /**
       * Gets the owner of the source port the edge was connected to before the event happened.
       * @type {yfiles.graph.IPortOwner}
       */
      sourcePortOwner:yfiles.graph.IPortOwner;
      /**
       * Gets the owner of the target port the edge was connected to before the event happened.
       * @type {yfiles.graph.IPortOwner}
       */
      targetPortOwner:yfiles.graph.IPortOwner;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of the {@link yfiles.collections.ItemEventArgs.<T>} class that is bound to the {@link yfiles.graph.IPort} type and carries additional {@link yfiles.graph.IPortOwner}
     * information.
     * <p>
     * Since for some events ({@link yfiles.graph.IGraph#addPortRemovedListener PortRemoved}) it is unclear what {@link yfiles.graph.IPort#owner} the port had
     * been attached to before the event, this event can be used to carry exactly that information.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.IPort>}
     * @final
     */
    export interface PortEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.IPort>{}
    export class PortEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graph.PortEventArgs} class.
       * @param port The port to assign to the {@link #item} property.
       * @param owner The owner of the port the port was attached to.
       */
      constructor(port:yfiles.graph.IPort,owner:yfiles.graph.IPortOwner);
      /**
       * Gets the owner the port the was connected to before the event happened.
       * @type {yfiles.graph.IPortOwner}
       */
      owner:yfiles.graph.IPortOwner;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of the {@link yfiles.collections.ItemEventArgs.<T>} class that is bound to the {@link yfiles.graph.ILabel} type and carries additional {@link yfiles.graph.ILabelOwner owner}
     * information.
     * <p>
     * Since for some events ({@link yfiles.graph.IGraph#addLabelRemovedListener LabelRemoved}) it is unclear what {@link yfiles.graph.ILabel#owner} the label
     * was owned by before the event, this event can be used to carry exactly that information.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.ILabel>}
     * @final
     */
    export interface LabelEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>{}
    export class LabelEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graph.LabelEventArgs} class.
       * @param label The label to assign to the {@link #item} property.
       * @param owner The previous owner of the label.
       */
      constructor(label:yfiles.graph.ILabel,owner:yfiles.graph.ILabelOwner);
      /**
       * Gets the owner of the label that owned the label before the event happened.
       * @type {yfiles.graph.ILabelOwner}
       */
      owner:yfiles.graph.ILabelOwner;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of the {@link yfiles.collections.ItemEventArgs.<T>} class that is bound to the {@link yfiles.graph.IBend} type and carries additional {@link yfiles.graph.IEdge owner} and index
     * information.
     * <p>
     * Since for some events ({@link yfiles.graph.IGraph#addBendRemovedListener BendRemoved}) it is unclear what {@link yfiles.graph.IBend#owner} the bend was
     * owned by before the event, this event can be used to carry exactly that information.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.IBend>}
     * @final
     */
    export interface BendEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.IBend>{}
    export class BendEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graph.BendEventArgs} class.
       * @param bend The bend to assign to the {@link #item} property.
       * @param owner The previous owner of the bend.
       * @param index The previous index of the bend in the {@link #bends} list.
       */
      constructor(bend:yfiles.graph.IBend,owner:yfiles.graph.IEdge,index:number);
      /**
       * Gets the owner of the bend that owned the bend before the event happened.
       * @type {yfiles.graph.IEdge}
       */
      owner:yfiles.graph.IEdge;
      /**
       * Gets the former index of the bend in the {@link yfiles.graph.IEdge#bends} list.
       * @type {number}
       */
      index:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Event argument class used by {@link yfiles.graph.IGraph} if a model item's property has been changed.
     * Type parameter <code>TItem</code>: The type of the item contained in the event.
     * Type parameter <code>TValue</code>: The type of the item's property that has been changed.
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<TItem>}
     * @template TItem,TValue
     */
    export interface ItemChangedEventArgs<TItem,TValue> extends yfiles.collections.ItemEventArgs<TItem>{}
    export class ItemChangedEventArgs<TItem,TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.graph.ItemChangedEventArgs.<TItem,TValue>} class.
       * @param item The item whose property has been changed.
       * @param oldValue The value of the property before the change.
       */
      constructor(item:TItem,oldValue:TValue);
      /**
       * Gets the value of the property before the change.
       * @type {TValue}
       */
      oldValue:TValue;
      static $class:yfiles.lang.Class;
    }
    /**
     * The interface used in an {@link yfiles.graph.IGraph} implementation for labels.
     * <p>
     * This interface provides read-only access to the properties of a label. In order to modify the state of an instance use
     * the various methods provided by the {@link yfiles.graph.IGraph} this instance belongs to. Labels are stored in
     * {@link yfiles.collections.IListEnumerable.<T> IListEnumerable&lt;ILabel>}s, that can be obtained from {@link yfiles.graph.ILabelOwner} implementations' {@link yfiles.graph.ILabelOwner#labels} property. Typical owners
     * of a label are {@link yfiles.graph.INode} and {@link yfiles.graph.IEdge} instances. Like all items in an <code>IGraph</code>, this item supports the
     * {@link yfiles.graph.ILookup#lookup} method that can be used to query additional aspects of the item.
     * </p>
     * <p>
     * The graph model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/graph.html The Graph Model}.
     * </p>
     * <p>
     * Using the look-up mechanism is explained in the section {@link @DGUIDE_PREFIX@/customizing_concepts_lookup.html Service Locator Pattern: Lookup}.
     * </p>
     * @interface
     * @implements {yfiles.graph.IModelItem}
     */
    export interface ILabel extends Object,yfiles.graph.IModelItem{
      /**
       * Returns the style that is responsible for the visual representation of this node in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * In order to set the style on an instance, use the {@link yfiles.graph.IGraph#setStyle} method. Note that the style instance
       * associated with a label instance may be shared between multiple label instances and that the modification of this style
       * will result in a change of the appearance of all labels that are associated with the same style instance.
       * </p>
       * @abstract
       * @type {yfiles.styles.ILabelStyle}
       */
      style:yfiles.styles.ILabelStyle;
      /**
       * Gets the preferred size of the label with respect to its current contents and the implementation of the visualization.
       * <p>
       * Often times the {@link yfiles.graph.ILabel#layout layout's} size will be the same as the preferred size, but it's up to the implementation of the {@link yfiles.graph.ILabelModel}
       * to interpret it differently. In order to set a different preferred size for this label instance use {@link yfiles.graph.IGraph}'s
       * {@link yfiles.graph.IGraph#setLabelPreferredSize} method.
       * </p>
       * @abstract
       * @type {yfiles.geometry.Size}
       */
      preferredSize:yfiles.geometry.Size;
      /**
       * Gets the owner of this label.
       * <p>
       * Typically this will yield an instance of {@link yfiles.graph.INode} or {@link yfiles.graph.IEdge}.
       * </p>
       * @abstract
       * @type {yfiles.graph.ILabelOwner}
       */
      owner:yfiles.graph.ILabelOwner;
      /**
       * Gets the text string associated with this label.
       * <p>
       * It is up to the visualization engine to interpret this property for the visualization of the label. Normally, it will
       * render the text into the {@link yfiles.graph.ILabel#layout layout} of this instance.
       * </p>
       * @abstract
       * @type {string}
       */
      text:string;
      /**
       * Gets the label model's parameter that determines the positioning of this label.
       * <p>
       * In order to set a different parameter for this label, use {@link yfiles.graph.IGraph}'s {@link yfiles.graph.IGraph#setLabelLayoutParameter}
       * method. Note that the label model parameter instance associated with a label instance may be shared between multiple
       * label instances and that the modification of this instance or its {@link yfiles.graph.ILabelModelParameter#model} will result in a change of the positioning of all
       * labels that are associated with the same parameter or model instance.
       * </p>
       * @abstract
       * @type {yfiles.graph.ILabelModelParameter}
       */
      layoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * Gets the index of the label at its {@link yfiles.graph.ILabel#owner}.
       * @type {number}
       */
      index?:number;
      /**
       * Gets a snapshot of the current layout of the label.
       * <p>
       * Labels have a {@link yfiles.geometry.ISize size} and are anchored at a given location in world coordinate space. The anchor is the point around which
       * the oriented rectangle is rotated in the world coordinate system. If the oriented rectangle has default orientation,
       * i.e. its up vector points north (<code>&lt;0,-1></code>), it coincides with its lower left corner. The positioning of the label is
       * determined using a pair of {@link yfiles.graph.ILabelModel} and {@link yfiles.graph.ILabelModelParameter}. In order to modify the layout
       * of a label instance, set another {@link yfiles.graph.ILabel#layoutParameter} for this label or modify its associated instance.
       * </p>
       * @type {yfiles.geometry.IOrientedRectangle}
       */
      layout?:yfiles.geometry.IOrientedRectangle;
    }
    var ILabel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.graph.IGraph} and the like to declare and obtain the defaults for labels.
     * <p>
     * Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
     * after the change are affected.
     * </p>
     * @see yfiles.graph.INodeDefaults#labels
     * @see yfiles.graph.IEdgeDefaults#labels
     * @see yfiles.graph.IGraph#nodeDefaults
     * @see yfiles.graph.IGraph#edgeDefaults
     * @see yfiles.graph.IGraph#groupNodeDefaults
     * @interface
     */
    export interface ILabelDefaults extends Object{
      /**
       * Factory method that returns a label model parameter instance for use with newly created labels.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.ILabelDefaults#layoutParameter} property, if {@link yfiles.graph.ILabelDefaults#shareLayoutParameterInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @param owner The owner of the label that will be created.
       * @returns The parameter to use, which for most implementations is either a {@link #clone clone} of or the {@link #layoutParameter} property, if {@link #shareLayoutParameterInstance} is enabled.
       * @abstract
       */
      getLayoutParameterInstance(owner:yfiles.graph.ILabelOwner):yfiles.graph.ILabelModelParameter;
      /**
       * Factory method that returns a style instance for use with newly created labels.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.ILabelDefaults#style} property, if {@link yfiles.graph.ILabelDefaults#shareStyleInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @param owner The owner of the label that will be created.
       * @returns The style to use, which for most implementations is either a {@link #clone clone} of or the {@link #style} property, if {@link #shareStyleInstance} is enabled.
       * @abstract
       */
      getStyleInstance(owner:yfiles.graph.ILabelOwner):yfiles.styles.ILabelStyle;
      /**
       * Gets or sets the style to use for labels.
       * <p>
       * Depending on the setting of {@link yfiles.graph.ILabelDefaults#shareStyleInstance}, the {@link yfiles.graph.ILabelDefaults#getStyleInstance} method should return a {@link yfiles.lang.ICloneable#clone clone} of this
       * instance or the very same instance.
       * </p>
       * @see yfiles.graph.ILabelDefaults#shareStyleInstance
       * @abstract
       * @type {yfiles.styles.ILabelStyle}
       */
      style:yfiles.styles.ILabelStyle;
      /**
       * Gets or sets a property that determines whether to automatically adjust the preferred size of a label.
       * <p>
       * On a call to {@link yfiles.graph.IGraph#setLabelText} or
       * {@link yfiles.graph.IGraph#setStyle}, the preferred size of the label will automatically be adjusted to the preferred size that
       * is suggested by the label's style renderer, if this property is set to true.
       * </p>
       * @see yfiles.styles.ILabelStyleRenderer#getPreferredSize
       * @abstract
       * @type {boolean}
       */
      autoAdjustPreferredSize:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.ILabelDefaults#layoutParameter} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.ILabelDefaults#getLayoutParameterInstance}.
       * @see yfiles.graph.ILabelDefaults#getLayoutParameterInstance
       * @see yfiles.graph.ILabelDefaults#layoutParameter
       * @abstract
       * @type {boolean}
       */
      shareLayoutParameterInstance:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.ILabelDefaults#style} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.ILabelDefaults#getStyleInstance}.
       * @see yfiles.graph.ILabelDefaults#getStyleInstance
       * @see yfiles.graph.ILabelDefaults#style
       * @abstract
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      /**
       * Gets or sets the label model parameter to use for labels.
       * <p>
       * Depending on the setting of {@link yfiles.graph.ILabelDefaults#shareLayoutParameterInstance}, the {@link yfiles.graph.ILabelDefaults#getLayoutParameterInstance} method should return a {@link yfiles.lang.ICloneable#clone clone} of
       * this instance or the very same instance.
       * </p>
       * @see yfiles.graph.ILabelDefaults#shareLayoutParameterInstance
       * @see yfiles.graph.ILabelDefaults#getLayoutParameterInstance
       * @abstract
       * @type {yfiles.graph.ILabelModelParameter}
       */
      layoutParameter:yfiles.graph.ILabelModelParameter;
    }
    var ILabelDefaults:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Defines the geometry of {@link yfiles.graph.ILabel} instances with respect to specific {@link yfiles.graph.ILabelModelParameter}s.
     * <p>
     * In addition to defining the geometry of a label with respect to a specific
     * {@link yfiles.graph.ILabelModelParameter}, a label model is typically the factory of the label model parameters that are
     * compatible with it, too.
     * </p>
     * <p>
     * Typically, the label model defines a, possibly infinite, compilation of valid geometries for labels, and a specific
     * label model parameter specifies an exact geometry for a label.
     * </p>
     * <p>
     * By design, the same instance of a label model can be shared by multiple label model parameters. Similarly, the same
     * instance of a label model parameter can be shared by multiple labels.
     * </p>
     * <p>
     * This interface supports the {@link yfiles.graph.ILookup#lookup} method that can be used to query additional aspects of the
     * implementation.
     * </p>
     * @see yfiles.graph.ILabelModelParameter
     * @see yfiles.graph.ILabel
     * @see yfiles.graph.IGraph
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface ILabelModel extends Object,yfiles.graph.ILookup{
      /**
       * Creates a default parameter that can be used for this model.
       * @returns a parameter for this model instance
       * @abstract
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Provides a {@link yfiles.graph.ILookup lookup context} for the given combination of label and parameter.
       * @param label The label to use in the context.
       * @param layoutParameter The parameter to use for the label in the context.
       * @returns An implementation of the {@link } interface that can be used to query additional aspects of the
       * label/parameter combination.
       * @see yfiles.graph.ILookup#EMPTY
       * @abstract
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * Calculates the {@link yfiles.geometry.IOrientedRectangle geometry} of the given label using the given model parameter.
       * @param label The label to calculate the geometry for.
       * @param layoutParameter A parameter that is compatible with this model. Typically, this is a parameter that has been created by this model, and
       * its property {@link #model} returns this instance.
       * @returns An {@link } that describes the geometry of the label. This is typically designed as a
       * flyweight, therefore clients should not cache the instance but store the values if they need a snapshot for later use.
       * @abstract
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
    }
    var ILabelModel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that provides more information about a potential label geometry (a candidate) to an automatic labeling
     * algorithm.
     * <p>
     * Typically, instances of this interface are created by an implementation of {@link yfiles.graph.ILabelCandidateDescriptorProvider}
     * for a pair of an {@link yfiles.graph.ILabel} and a {@link yfiles.graph.ILabelModelParameter} that describes a potential placement of
     * that label (a 'candidate').
     * </p>
     * @see yfiles.graph.ILabelCandidateDescriptorProvider
     * @interface
     */
    export interface ILabelCandidateDescriptor extends Object{
      /**
       * Gets a value indicating whether this candidate is deemed an external candidate.
       * <p>
       * This is mainly for {@link yfiles.graph.INode} labels that are a visual part of the node's internals. It could be used for edge
       * labels, too, if the edge occupies significant amounts of visual space.
       * </p>
       * @abstract
       * @type {boolean}
       */
      externalCandidate:boolean;
      /**
       * Gets a penalty value that is used by labeling algorithms to judge the severity if this candidate overlaps with an edge
       * path.
       * <p>
       * This value must be between 0 and 1 (inclusive). A value of 0 means that an overlap is considered not important, while a
       * value of 1 means that an overlap is considered severe.
       * </p>
       * @abstract
       * @type {number}
       */
      edgeOverlapPenalty:number;
      /**
       * Gets a penalty value that is used by labeling algorithms to judge the severity if this label candidate overlaps with a
       * node's bounds.
       * <p>
       * This value must be between 0 and 1 (inclusive). A value of 0 means that an overlap is considered not important, while a
       * value of 1 means that an overlap is considered severe.
       * </p>
       * @abstract
       * @type {number}
       */
      nodeOverlapPenalty:number;
      /**
       * Gets a profit value that is used by labeling algorithms to judge how good this label candidate is.
       * <p>
       * This value must be between 0 and 1 (inclusive). Higher profit means better candidates. Hence, there is a higher
       * probability that the candidate is chosen by a labeling algorithm.
       * </p>
       * @abstract
       * @type {number}
       */
      profit:number;
    }
    var ILabelCandidateDescriptor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface that can be used to retrieve {@link yfiles.graph.ILabelCandidateDescriptor} instances for a given pair of a {@link yfiles.graph.ILabel}
     * and a {@link yfiles.graph.ILabelModelParameter candidate parameter}.
     * <p>
     * This interface can be used by automatic labeling algorithms to help decide which labels candidate positions should be
     * used.
     * </p>
     * @see yfiles.graph.ILabelCandidateDescriptor
     * @interface
     */
    export interface ILabelCandidateDescriptorProvider extends Object{
      /**
       * Gets the descriptor for a given pair of a label and a {@link yfiles.graph.ILabelModelParameter}.
       * @param label The label to possibly assign the <code>layoutParameter</code>.
       * @param layoutParameter The parameter to provide a descriptor for if applied to the given <code>label</code>.
       * @returns A descriptor or <code>null</code>.
       * @abstract
       */
      getDescriptor(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILabelCandidateDescriptor;
    }
    var ILabelCandidateDescriptorProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An implementation of the {@link yfiles.graph.ILabelCandidateDescriptorProvider} interface that always yields a constant
     * {@link yfiles.graph.ILabelCandidateDescriptor}.
     * @class
     * @implements {yfiles.graph.ILabelCandidateDescriptorProvider}
     * @final
     */
    export interface ConstantLabelCandidateDescriptorProvider extends Object,yfiles.graph.ILabelCandidateDescriptorProvider{}
    export class ConstantLabelCandidateDescriptorProvider {
      /**
       * Initializes a new instance of the {@link yfiles.graph.ConstantLabelCandidateDescriptorProvider} class that always yields <code>descriptor</code>
       * in {@link yfiles.graph.ConstantLabelCandidateDescriptorProvider#getDescriptor}.
       * @param descriptor The descriptor to yield in {@link #getDescriptor}.
       */
      constructor(descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Always yields the parameter that has been passed to the constructor.
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getDescriptor(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILabelCandidateDescriptor;
      /**
       * A sharable implementation of the {@link yfiles.graph.ILabelCandidateDescriptorProvider} interface that always yields
       * {@link yfiles.graph.LabelCandidateDescriptor#EXTERNAL_DESCRIPTOR}.
       * @const
       * @static
       * @type {yfiles.graph.ILabelCandidateDescriptorProvider}
       */
      static EXTERNAL_DESCRIPTOR_PROVIDER:yfiles.graph.ILabelCandidateDescriptorProvider;
      /**
       * A sharable implementation of the {@link yfiles.graph.ILabelCandidateDescriptorProvider} interface that always yields
       * {@link yfiles.graph.LabelCandidateDescriptor#INTERNAL_DESCRIPTOR}.
       * @const
       * @static
       * @type {yfiles.graph.ILabelCandidateDescriptorProvider}
       */
      static INTERNAL_DESCRIPTOR_PROVIDER:yfiles.graph.ILabelCandidateDescriptorProvider;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple mutable default implementation of the {@link yfiles.graph.ILabelCandidateDescriptor} interface.
     * <p>
     * All properties are read/write in this implementation.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelCandidateDescriptor}
     */
    export interface LabelCandidateDescriptor extends Object,yfiles.graph.ILabelCandidateDescriptor{}
    export class LabelCandidateDescriptor {
      /**
       * A sharable immutable implementation of the {@link yfiles.graph.ILabelCandidateDescriptor} interface that has penalties of <code>1.0d</code>
       * and a profit of <code>1.0d</code> and yields <code>true</code> for the {@link yfiles.graph.ILabelCandidateDescriptor#externalCandidate} property.
       * @const
       * @static
       * @type {yfiles.graph.ILabelCandidateDescriptor}
       */
      static EXTERNAL_DESCRIPTOR:yfiles.graph.ILabelCandidateDescriptor;
      /**
       * A sharable immutable implementation of the {@link yfiles.graph.ILabelCandidateDescriptor} interface that has penalties of <code>1.0d</code>
       * and a profit of <code>1.0d</code> and yields <code>false</code> for the {@link yfiles.graph.ILabelCandidateDescriptor#externalCandidate} property.
       * @const
       * @static
       * @type {yfiles.graph.ILabelCandidateDescriptor}
       */
      static INTERNAL_DESCRIPTOR:yfiles.graph.ILabelCandidateDescriptor;
      /**
       * 
       * @type {boolean}
       */
      externalCandidate:boolean;
      /**
       * 
       * @default 1.0
       * @type {number}
       */
      edgeOverlapPenalty:number;
      /**
       * 
       * @default 1.0
       * @type {number}
       */
      nodeOverlapPenalty:number;
      /**
       * 
       * @default 1.0
       * @type {number}
       */
      profit:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Defines a state that can be used to {@link yfiles.graph.ILabelModel#getGeometry determine the geometry of a label} with a {@link yfiles.graph.ILabelModel}.
     * <p>
     * Typically, implementations of this interface are created by factory methods of the corresponding {@link yfiles.graph.ILabelModelParameter#model}.
     * </p>
     * <p>
     * This interface extends the {@link yfiles.lang.ICloneable} interface. This allows clients to obtain a persistent copy of the
     * current state of this parameter. Immutable parameter implementations may return themselves.
     * </p>
     * @see yfiles.graph.ILabelModel
     * @see yfiles.graph.ILabel
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface ILabelModelParameter extends Object,yfiles.lang.ICloneable{
      /**
       * Determines whether this parameter can be used for a given label instance.
       * <p>
       * A parameter must not necessarily support all types of labels. This method returns whether it supports the given <code>label</code>.
       * For example, a parameter might only support labels that are owned by an {@link yfiles.graph.INode}.
       * </p>
       * @param label The label to test.
       * @returns <code>true</code> if this instance can be used for the given label, <code>false</code> otherwise.
       * @abstract
       */
      supports(label:yfiles.graph.ILabel):boolean;
      /**
       * Gets the model that can be used to determine the geometry of a label given this parameter and a label instance.
       * @abstract
       * @type {yfiles.graph.ILabelModel}
       */
      model:yfiles.graph.ILabelModel;
    }
    var ILabelModelParameter:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Utility interface used by {@link yfiles.graph.ILabelModel} implementations to find the best {@link yfiles.graph.ILabelModelParameter} to
     * approximate a given layout.
     * <p>
     * Implementations of this class may be retrieved from {@link yfiles.graph.ILabelModel} implementations using their {@link yfiles.graph.ILookup#lookup}
     * method. Note that implementations are not required to return an instance if queried for this interface.
     * </p>
     * @see yfiles.graph.DefaultLabelModelParameterFinder
     * @interface
     */
    export interface ILabelModelParameterFinder extends Object{
      /**
       * Tries to find a parameter that best matches the given layout for the provided label instance.
       * <p>
       * This method may not necessarily find a parameter that matches the provided layout exactly. Implementations may choose to
       * simply return the model's {@link yfiles.graph.ILabelModel#createDefaultParameter default parameter} but may never return <code>null</code>.
       * </p>
       * @param label The label to find a parameter for.
       * @param model The model instance to use. This should be the instance this instance has been obtained from.
       * @param layout The anticipated layout for the label.
       * @returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
       * @abstract
       */
      findBestParameter(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel,layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
    }
    var ILabelModelParameterFinder:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A simple default implementation of the {@link yfiles.graph.ILabelModelParameterFinder} interface that uses the
     * {@link yfiles.graph.ILabelModel}'s own {@link yfiles.graph.ILabelModelParameterProvider} instance to find the closest match.
     * <p>
     * This implementation uses a metric to find the closest of the provided parameter candidates to the layout provided in
     * {@link yfiles.graph.DefaultLabelModelParameterFinder#selectBestParameter}.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     */
    export interface DefaultLabelModelParameterFinder extends Object,yfiles.graph.ILabelModelParameterFinder{}
    export class DefaultLabelModelParameterFinder {
      /**
       * Calculates a metric for the given label/parameter pair and the target label layout.
       * <p>
       * This implementation gets the geometry from the parameter's model's {@link yfiles.graph.ILabelModel#getGeometry} method and
       * returns the result of a call to {@link yfiles.graph.DefaultLabelModelParameterFinder#distance}.
       * </p>
       * @param label The label to apply the parameter to.
       * @param layoutParameter The parameter candidate.
       * @param layout The targeted layout rectangle.
       * @returns A metric that indicates the distance between the candidate's layout and the target layout.
       * @protected
       */
      calculateMetric(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter,layout:yfiles.geometry.IOrientedRectangle):number;
      /**
       * Returns the distance between two {@link yfiles.geometry.IOrientedRectangle} instances.
       * <p>
       * The distance is calculated using the Euclidean distance between the two centers and the angular deviation between the
       * two up vectors.
       * </p>
       * @param rectangle1 The first geometry.
       * @param rectangle2 The geometry to compare the first one with.
       * @returns A value indicating the distance between the two instances.
       * @protected
       */
      distance(rectangle1:yfiles.geometry.IOrientedRectangle,rectangle2:yfiles.geometry.IOrientedRectangle):number;
      /**
       * Tries to find a parameter that best matches the given layout for the provided label instance.
       * <p>
       * This method may not necessarily find a parameter that matches the provided layout exactly. It will first query the {@link yfiles.graph.DefaultLabelModelParameterFinder#getCandidateParameters}
       * parameter for an enumeration of candidates and will then find the best matching candidate using a metric calculated by
       * {@link yfiles.graph.DefaultLabelModelParameterFinder#calculateMetric}. If no parameter can be found this method returns the {@link yfiles.graph.ILabelModel#createDefaultParameter default parameter}
       * for the model.
       * </p>
       * @param label The label to find a parameter for.
       * @param model The model instance to use. This should be the instance this instance has been obtained from.
       * @param layout The anticipated layout for the label.
       * @returns A non-<code>null</code> parameter that can be used for the label to approximate the provided layout.
       */
      findBestParameter(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel,layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
      /**
       * Retrieves an enumerator over all candidates for a given label and model.
       * @param label The label.
       * @param model The model of the label.
       * @returns An enumerator over the candidates. The enumerator may be empty.
       * @protected
       */
      getCandidateParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * Tries to find a parameter that best matches the given layout for the provided label instance among the set of given
       * parameters.
       * <p>
       * Null may be returned if an empty enumerable is passed.
       * </p>
       * @param label The label to find a parameter for.
       * @param layout The anticipated layout for the label.
       * @param layoutParameters A set of parameters to choose from.
       * @returns A parameter that can be used for the label to approximate the provided layout.
       */
      selectBestParameter(label:yfiles.graph.ILabel,layout:yfiles.geometry.IOrientedRectangle,layoutParameters:yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>):yfiles.graph.ILabelModelParameter;
      /**
       * A singleton instance of this class.
       * @const
       * @static
       * @type {yfiles.graph.DefaultLabelModelParameterFinder}
       */
      static INSTANCE:yfiles.graph.DefaultLabelModelParameterFinder;
      static $class:yfiles.lang.Class;
    }
    /**
     * Helper interface used by {@link yfiles.graph.ILabelModel} implementations to provide possible candidate
     * {@link yfiles.graph.ILabelModelParameter}s for a given label and model.
     * <p>
     * Implementations of this interface can be retrieved from the {@link yfiles.graph.ILabelModel}'s {@link yfiles.graph.ILookup#lookup}
     * method. Note that not all models necessarily need to or can provide implementations of this interface.
     * </p>
     * @interface
     */
    export interface ILabelModelParameterProvider extends Object{
      /**
       * Returns an enumerator over a set of possible {@link yfiles.graph.ILabelModelParameter} instances that can be used for the given
       * label and model.
       * @param label The label instance to use.
       * @param model The model to provide parameters for.
       * @returns A possibly empty enumerator over a set of label model parameters.
       * @abstract
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
    }
    var ILabelModelParameterProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Common interface for items in an {@link yfiles.graph.IGraph} implementation that may own {@link yfiles.graph.ILabel labels}.
     * <p>
     * Like all items in an <code>IGraph</code>, this item supports the {@link yfiles.graph.ILookup#lookup} method that can be used to query
     * additional aspects of the item.
     * </p>
     * @see yfiles.graph.INode
     * @see yfiles.graph.IEdge
     * @interface
     * @implements {yfiles.graph.IModelItem}
     */
    export interface ILabelOwner extends Object,yfiles.graph.IModelItem{
      /**
       * Provides access to a collection of {@link yfiles.graph.ILabel labels} that are owned by this instance.
       * <p>
       * This gives access to a read-only live view of the labels, i.e. the collection can change over time, as well as the
       * labels contained in it. If a snapshot of the current state is needed, one needs to copy the collection and its
       * contents.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.ILabel>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.ILabel>;
    }
    var ILabelOwner:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A registry that can be used to store {@link yfiles.collections.IMapper.<K,V>} implementations by tag.
     * <p>
     * Implementations of this interface can be found in the {@link yfiles.graph.ILookup#lookup} of the default {@link yfiles.graph.IGraph}
     * implementations in the framework.
     * </p>
     * <p>
     * Using <code>IMapperRegistry</code> is described in the section {@link @DGUIDE_PREFIX@/customizing_graph-binding_data.html Associating Data with Graph Elements}.
     * </p>
     * @see yfiles.graph.MapperRegistry
     * @interface
     */
    export interface IMapperRegistry extends Object{
      /**
       * Registers a mapper for the given tag.
       * <p>
       * If there already was a mapper for <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>K</code>: The key type for the mapper.
       * Type parameter <code>V</code>: The value type for the mapper.
       * @param valueType The value type for the mapper.
       * @param keyType The key type for the mapper.
       * @param tag The tag to use.
       * @param mapper The mapper to register.
       * @template K,V
       * @abstract
       */
      addMapper<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,tag:Object,mapper:yfiles.collections.IMapper<K,V>):void;
      /**
       * Creates a simple read-only {@link yfiles.collections.IMapper.<K,V>} implementation with key type {@link yfiles.graph.ILabel} and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using the
       * provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param constant The value to yield during {@link } reads.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template V
       */
      createConstantMapper?<V>(tag:yfiles.algorithms.ILabelLayoutDpKey<V>,constant:V):yfiles.collections.IMapper<yfiles.graph.ILabel,V>;
      /**
       * Creates a simple read-only {@link yfiles.collections.IMapper.<K,V>} implementation with key type {@link yfiles.graph.IEdge} and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using the
       * provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param constant The value to yield during {@link } reads.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template V
       */
      createConstantMapper?<V>(tag:yfiles.algorithms.EdgeDpKey<V>,constant:V):yfiles.collections.IMapper<yfiles.graph.IEdge,V>;
      /**
       * Creates a simple read-only {@link yfiles.collections.IMapper.<K,V>} implementation with key type {@link yfiles.graph.INode} and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using the
       * provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param constant The value to yield during {@link } reads.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template V
       */
      createConstantMapper?<V>(tag:yfiles.algorithms.NodeDpKey<V>,constant:V):yfiles.collections.IMapper<yfiles.graph.INode,V>;
      /**
       * Creates a simple read-only {@link yfiles.collections.IMapper.<K,V>} implementation and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using the provided <code>tag</code>.
       * <p>
       * If there already was a mapper for <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>K</code>: The type of the keys of the mapper.
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param constant The value to yield during {@link } reads.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromConstant
       * @template K,V
       */
      createConstantMapper?<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,tag:Object,constant:V):yfiles.collections.IMapper<K,V>;
      /**
       * Creates a delegate-based read-only {@link yfiles.collections.IMapper.<K,V>} implementation with key type {@link yfiles.graph.ILabel} and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using
       * the provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param getter The getter delegate to which {@link } reads will be delegated.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template V
       */
      createDelegateMapper?<V>(tag:yfiles.algorithms.ILabelLayoutDpKey<V>,getter:(key:yfiles.graph.ILabel)=>V):yfiles.collections.IMapper<yfiles.graph.ILabel,V>;
      /**
       * Creates a delegate-based read-only {@link yfiles.collections.IMapper.<K,V>} implementation with key type {@link yfiles.graph.IEdge} and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using
       * the provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param getter The getter delegate to which {@link } reads will be delegated.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template V
       */
      createDelegateMapper?<V>(tag:yfiles.algorithms.EdgeDpKey<V>,getter:(key:yfiles.graph.IEdge)=>V):yfiles.collections.IMapper<yfiles.graph.IEdge,V>;
      /**
       * Creates a delegate-based read-only {@link yfiles.collections.IMapper.<K,V>} implementation with key type {@link yfiles.graph.INode} and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using
       * the provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param getter The getter delegate to which {@link } reads will be delegated.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template V
       */
      createDelegateMapper?<V>(tag:yfiles.algorithms.NodeDpKey<V>,getter:(key:yfiles.graph.INode)=>V):yfiles.collections.IMapper<yfiles.graph.INode,V>;
      /**
       * Creates a delegate-based read-only {@link yfiles.collections.IMapper.<K,V>} implementation and {@link yfiles.graph.IMapperRegistry#addMapper adds} it to the <code>registry</code> using the provided <code>tag</code>.
       * <p>
       * If there already was a mapper for <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>K</code>: The type of the keys of the mapper.
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @param getter The getter delegate to which {@link } reads will be delegated.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.IMapper.<K,V>#fromDelegate
       * @template K,V
       */
      createDelegateMapper?<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,tag:Object,getter:(key:K)=>V):yfiles.collections.IMapper<K,V>;
      /**
       * Creates and {@link yfiles.graph.IMapperRegistry#addMapper adds} a new {@link yfiles.collections.Mapper.<K,V>} with key type {@link yfiles.graph.IEdge} to the <code>registry</code> using the provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.Mapper.<K,V>
       * @template V
       */
      createMapper?<V>(tag:yfiles.algorithms.EdgeDpKey<V>):yfiles.collections.Mapper<yfiles.graph.IEdge,V>;
      /**
       * Creates and {@link yfiles.graph.IMapperRegistry#addMapper adds} a new {@link yfiles.collections.Mapper.<K,V>} with key type {@link yfiles.graph.ILabel} to the <code>registry</code> using the provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.Mapper.<K,V>
       * @template V
       */
      createMapper?<V>(tag:yfiles.algorithms.ILabelLayoutDpKey<V>):yfiles.collections.Mapper<yfiles.graph.ILabel,V>;
      /**
       * Creates and {@link yfiles.graph.IMapperRegistry#addMapper adds} a new {@link yfiles.collections.Mapper.<K,V>} with key type {@link yfiles.graph.INode} to the <code>registry</code> using the provided <code>tag</code>.
       * <p>
       * The value type of the mapper is inferred by the provided <code>tag</code>. If there already was a mapper for
       * <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.Mapper.<K,V>
       * @template V
       */
      createMapper?<V>(tag:yfiles.algorithms.NodeDpKey<V>):yfiles.collections.Mapper<yfiles.graph.INode,V>;
      /**
       * Creates and {@link yfiles.graph.IMapperRegistry#addMapper adds} a new {@link yfiles.collections.Mapper.<K,V>} to the <code>registry</code> using the provided <code>tag</code>.
       * <p>
       * If there already was a mapper for <code>tag</code>, it will be replaced.
       * </p>
       * Type parameter <code>K</code>: The type of the keys of the mapper.
       * Type parameter <code>V</code>: The type of the values of the mapper.
       * @param tag The tag.
       * @returns The newly created mapper instance.
       * @see yfiles.collections.Mapper.<K,V>
       * @template K,V
       */
      createMapper?<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,tag:Object):yfiles.collections.Mapper<K,V>;
      /**
       * Returns a mapper that has previously been registered with this instance for the given tag.
       * Type parameter <code>K</code>: The key type for the mapper.
       * Type parameter <code>V</code>: The value type for the mapper.
       * @param tag The tag to use to look up the mapper.
       * @returns The previously registered instance or <code>null</code>.
       * @template K,V
       * @abstract
       */
      getMapper<K,V>(tag:Object):yfiles.collections.IMapper<K,V>;
      /**
       * Gets the mapper {@link yfiles.graph.MapperMetadata meta data} for the mapper that is registered using the <code>tag</code>.
       * @param tag The tag.
       * @returns The meta data or <code>null</code> if there is no such mapper registered for the given tag.
       * @abstract
       */
      getMapperMetadata(tag:Object):yfiles.graph.MapperMetadata;
      /**
       * Removes a previously registered mapper for the given tag.
       * @param tag The tag that has been used for registering the mapper.
       * @abstract
       */
      removeMapper(tag:Object):void;
      /**
       * Sets the mapper meta data for the mapper that has been {@link yfiles.graph.IMapperRegistry#addMapper registered} using the provided <code>tag</code>.
       * @param tag The tag.
       * @param metadata The meta data to store with the mapper.
       * @throws {Stubs.Exceptions.ArgumentError} If the {@link yfiles.graph.MapperMetadata#keyType} or {@link yfiles.graph.MapperMetadata#valueType} mismatch the mapper instance in the registry.
       * @abstract
       */
      setMapperMetadata(tag:Object,metadata:yfiles.graph.MapperMetadata):void;
      /**
       * Gets an enumerable over all tags that have been used to register mapper implementations with this interface.
       * @abstract
       * @type {yfiles.collections.IEnumerable.<Object>}
       */
      registeredTags:yfiles.collections.IEnumerable<Object>;
    }
    var IMapperRegistry:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface for node entities in an {@link yfiles.graph.IGraph}.
     * <p>
     * This interface provides read-only access to the properties of a node. In order to modify the state of an instance use
     * the various methods provided by the {@link yfiles.graph.IGraph} this instance belongs to. Nodes and {@link yfiles.graph.IEdge edges} are the main entities
     * that make up an {@link yfiles.graph.IGraph}. Nodes have zero or more {@link yfiles.graph.IPortOwner#ports} to which zero or more edges can connect. {@link yfiles.graph.IGraph}
     * provides the {@link yfiles.graph.IGraph#edgesAt} method for querying the edges that connect to nodes. Also it is possible to
     * query the {@link yfiles.graph.IGraph#edgesAt adjacent edges for each port} that is owned by nodes. This interface combines the functionality of {@link yfiles.graph.IPortOwner}
     * to get access to the ports, {@link yfiles.graph.ILabelOwner} to get access to the labels, and, like all items in an <code>IGraph</code>,
     * nodes support the {@link yfiles.graph.ILookup#lookup} method inherited from the {@link yfiles.graph.IModelItem} interface can be used to
     * query additional aspects of each instance.
     * </p>
     * <p>
     * The graph model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/graph.html The Graph Model}.
     * </p>
     * <p>
     * Using the look-up mechanism is explained in the section {@link @DGUIDE_PREFIX@/customizing_concepts_lookup.html Service Locator Pattern: Lookup}.
     * </p>
     * @see yfiles.graph.IGraph
     * @see yfiles.graph.IEdge
     * @interface
     * @implements {yfiles.graph.IPortOwner}
     * @implements {yfiles.graph.ILabelOwner}
     */
    export interface INode extends Object,yfiles.graph.IPortOwner,yfiles.graph.ILabelOwner{
      /**
       * Returns a live view of the layout of the node.
       * <p>
       * The layout of a node is a rectangle in the world coordinate system that describes the bounding box of the representation
       * of a node. Since this method will yield a live view, it is up to the client to copy the values of the instance if a
       * snapshot of the state is needed. In order to modify the layout of a node, use the {@link yfiles.graph.IGraph#setNodeLayout various methods} in
       * {@link yfiles.graph.IGraph}.
       * </p>
       * @abstract
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Returns the style that is responsible for the visual representation of this node in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * In order to set the style on an instance, use the {@link yfiles.graph.IGraph#setStyle} method. Note that the style instance
       * associated with a node instance may be shared between multiple node instances and that the modification of this style
       * will result in a change of the appearance of all nodes that are associated with the same style instance.
       * </p>
       * @abstract
       * @type {yfiles.styles.INodeStyle}
       */
      style:yfiles.styles.INodeStyle;
    }
    var INode:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.graph.IGraph} and the like to declare and obtain the defaults for nodes and their labels or
     * ports.
     * <p>
     * Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
     * after the change are affected.
     * </p>
     * @see yfiles.graph.IGraph#nodeDefaults
     * @see yfiles.graph.IGraph#groupNodeDefaults
     * @interface
     */
    export interface INodeDefaults extends Object{
      /**
       * Factory method that returns a style instance for use with newly created nodes.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.INodeDefaults#style} property, if {@link yfiles.graph.INodeDefaults#shareStyleInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @returns The style to use, which for most implementations is either a {@link #clone clone} of or the {@link #style} property, if {@link #shareStyleInstance} is enabled.
       * @abstract
       */
      getStyleInstance():yfiles.styles.INodeStyle;
      /**
       * Gets or sets the style to use for nodes.
       * <p>
       * Depending on the setting of {@link yfiles.graph.INodeDefaults#shareStyleInstance}, the {@link yfiles.graph.INodeDefaults#getStyleInstance} method should return a {@link yfiles.lang.ICloneable#clone clone} of this
       * instance or the very same instance.
       * </p>
       * @see yfiles.graph.INodeDefaults#shareStyleInstance
       * @abstract
       * @type {yfiles.styles.INodeStyle}
       */
      style:yfiles.styles.INodeStyle;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.INodeDefaults#style} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.INodeDefaults#getStyleInstance}.
       * @see yfiles.graph.INodeDefaults#getStyleInstance
       * @see yfiles.graph.INodeDefaults#style
       * @abstract
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      /**
       * Gets or sets the defaults for ports at nodes.
       * @abstract
       * @type {yfiles.graph.IPortDefaults}
       */
      ports:yfiles.graph.IPortDefaults;
      /**
       * Gets or sets the defaults for labels at nodes.
       * @abstract
       * @type {yfiles.graph.ILabelDefaults}
       */
      labels:yfiles.graph.ILabelDefaults;
      /**
       * Gets or sets the default node size.
       * <p>
       * The values of this size will be used by the {@link yfiles.graph.IGraph#createNodeAt} and {@link yfiles.graph.IGraph#createNodeAt}
       * methods.
       * </p>
       * @abstract
       * @type {yfiles.geometry.Size}
       */
      size:yfiles.geometry.Size;
    }
    var INodeDefaults:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface used in an {@link yfiles.graph.IGraph} implementation for {@link yfiles.graph.IEdge}s to connect to.
     * <p>
     * This interface provides read-only access to the properties of a port. In order to modify the state of an instance use
     * the various methods provided by the {@link yfiles.graph.IGraph} this instance belongs to. Ports are owned by {@link yfiles.graph.IPortOwner IPortOwners},
     * normally an {@link yfiles.graph.INode}, but this can also be an {@link yfiles.graph.IEdge} in special graph implementations. To obtain
     * the {@link yfiles.graph.IEdge} instances that are connected to a certain port instance, applications need to use the {@link yfiles.graph.IGraph#edgesAt}
     * method provided by {@link yfiles.graph.IGraph}. Zero or more edges may be connected to a port, depending on the implementation
     * of the graph. Like all items in an <code>IGraph</code>, this item supports the {@link yfiles.graph.ILookup#lookup} method that can be used to
     * query additional aspects of the item.
     * </p>
     * <p>
     * The graph model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/graph.html The Graph Model}.
     * </p>
     * <p>
     * Using the look-up mechanism is explained in the section {@link @DGUIDE_PREFIX@/customizing_concepts_lookup.html Service Locator Pattern: Lookup}.
     * </p>
     * @interface
     * @implements {yfiles.graph.IModelItem}
     */
    export interface IPort extends Object,yfiles.graph.IModelItem{
      /**
       * Returns the owner of this port.
       * <p>
       * In traditional {@link yfiles.graph.IGraph} implementations, this will be an {@link yfiles.graph.INode} and can safely be cast to one. In
       * order to get to the {@link yfiles.graph.IEdge}s that connect to this instance, use {@link yfiles.graph.IGraph}'s {@link yfiles.graph.IGraph#edgesAt}
       * method.
       * </p>
       * @abstract
       * @type {yfiles.graph.IPortOwner}
       */
      owner:yfiles.graph.IPortOwner;
      /**
       * Returns the style that is responsible for the visual representation of this port in a {@link yfiles.view.CanvasComponent}.
       * <p>
       * In order to set the style on an instance, use the {@link yfiles.graph.IGraph#setStyle} method. Note that the style instance
       * associated with a port instance may be shared between multiple port instances and that the modification of this style
       * will result in a change of the appearance of all ports that are associated with the same style instance.
       * </p>
       * @abstract
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * Gets the {@link yfiles.graph.IPortLocationModelParameter} that is used to determine the location of this port.
       * <p>
       * The port location can be calculated using the {@link yfiles.graph.IPortLocationModelParameter#model}'s {@link yfiles.graph.IPortLocationModel#getLocation} method or the convenience
       * methods {@link yfiles.graph.IPort#location} and {@link yfiles.graph.IPort#dynamicLocation}.
       * </p>
       * <p>
       * Note that parameters may be shared across port instances.
       * </p>
       * @see yfiles.graph.IGraph#setPortLocationParameter
       * @see yfiles.graph.IPort#location
       * @see yfiles.graph.IPort#dynamicLocation
       * @abstract
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Gets a snapshot of the current location of the port.
       * <p>
       * Unlike {@link yfiles.graph.IPort#dynamicLocation} this does not return a dynamic point that always refers to the current location.
       * </p>
       * @type {yfiles.geometry.Point}
       */
      location?:yfiles.geometry.Point;
      /**
       * Gets a live view of the location of the port in world coordinates.
       * <p>
       * The location is the anchor for the edges, that connect to this port, however it is up to the visualization logic where
       * exactly the visual part of an edge will end. As this will yield a live view, it is up to the client to copy the values
       * if a snapshot of the state is needed. In order to modify the location of a port, use the {@link yfiles.graph.IGraph#setPortLocationParameter}
       * in {@link yfiles.graph.IGraph}.
       * </p>
       * @see yfiles.graph.IPort#locationParameter
       * @type {yfiles.geometry.IPoint}
       */
      dynamicLocation?:yfiles.geometry.IPoint;
    }
    var IPort:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The common interface for items in an {@link yfiles.graph.IGraph} that may own {@link yfiles.graph.IPort ports}.
     * <p>
     * Typically this interface is actively implemented by {@link yfiles.graph.INode}'s in an {@link yfiles.graph.IGraph}. To get to the edge
     * instances that connect to the ports, the {@link yfiles.graph.IGraph#edgesAt} method can be used. Alternatively for each {@link yfiles.graph.IPort}
     * in {@link yfiles.graph.IPortOwner#ports}, one can use the {@link yfiles.graph.IGraph#edgesAt} method provided by {@link yfiles.graph.IGraph}. Like all items in an <code>IGraph</code>,
     * this item supports the {@link yfiles.graph.ILookup#lookup} method that can be used to query additional aspects of the item.
     * </p>
     * @see yfiles.graph.INode
     * @see yfiles.graph.IEdge
     * @interface
     * @implements {yfiles.graph.IModelItem}
     */
    export interface IPortOwner extends Object,yfiles.graph.IModelItem{
      /**
       * Provides access to a collection of {@link yfiles.graph.IPort ports} that are owned by this instance.
       * <p>
       * This gives access to a read-only live view of the ports, i.e. the collection can change over time, as well as the ports
       * contained in it. If a snapshot of the current state is needed, one needs to copy the collection.
       * </p>
       * @abstract
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.IPort>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.IPort>;
    }
    var IPortOwner:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.graph.IGraph} to declare and obtain the defaults for ports at nodes and edges.
     * <p>
     * Note that changing these defaults does <i>not</i> change properties of already created model items. Rather, only items created
     * after the change are affected.
     * </p>
     * @see yfiles.graph.IPort
     * @see yfiles.graph.INodeDefaults#ports
     * @see yfiles.graph.IEdgeDefaults#ports
     * @see yfiles.graph.IGraph#nodeDefaults
     * @see yfiles.graph.IGraph#edgeDefaults
     * @interface
     */
    export interface IPortDefaults extends Object{
      /**
       * Factory method that returns a location model parameter instance for use with newly created ports.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.IPortDefaults#locationParameter} property, if {@link yfiles.graph.IPortDefaults#shareLocationParameterInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @param owner The owner of the port that will be created.
       * @returns The parameter to use, which for most implementations is either a {@link #clone clone} of or the {@link #locationParameter} property, if {@link #shareLocationParameterInstance} is enabled.
       * @abstract
       */
      getLocationParameterInstance(owner:yfiles.graph.IPortOwner):yfiles.graph.IPortLocationModelParameter;
      /**
       * Factory method that returns a style instance for use with newly created ports.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.IPortDefaults#style} property, if {@link yfiles.graph.IPortDefaults#shareStyleInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @param owner The owner of the port that will be created.
       * @returns The style to use, which for most implementations is either a {@link #clone clone} of or the {@link #style} property, if {@link #shareStyleInstance} is enabled.
       * @abstract
       */
      getStyleInstance(owner:yfiles.graph.IPortOwner):yfiles.styles.IPortStyle;
      /**
       * Gets or sets the style to use for ports.
       * <p>
       * Depending on the setting of {@link yfiles.graph.IPortDefaults#shareStyleInstance}, the {@link yfiles.graph.IPortDefaults#getStyleInstance} method should return a {@link yfiles.lang.ICloneable#clone clone} of this
       * instance or the very same instance.
       * </p>
       * @see yfiles.graph.IPortDefaults#shareStyleInstance
       * @abstract
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * Determines whether unused ports should automatically be removed from their owners as soon as no further edge is
       * connected to them.
       * @see yfiles.graph.IGraph#remove
       * @abstract
       * @type {boolean}
       */
      autoCleanup:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.IPortDefaults#style} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.IPortDefaults#getStyleInstance}.
       * @see yfiles.graph.IPortDefaults#getStyleInstance
       * @see yfiles.graph.IPortDefaults#style
       * @abstract
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      /**
       * Gets or sets the location model parameter to use for ports.
       * <p>
       * Depending on the setting of {@link yfiles.graph.IPortDefaults#shareLocationParameterInstance}, the {@link yfiles.graph.IPortDefaults#getLocationParameterInstance} method should return a {@link yfiles.lang.ICloneable#clone clone}
       * of this instance or the very same instance.
       * </p>
       * @see yfiles.graph.IPortDefaults#shareLocationParameterInstance
       * @see yfiles.graph.IPortDefaults#getLocationParameterInstance
       * @abstract
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.IPortDefaults#locationParameter} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.IPortDefaults#getLocationParameterInstance}.
       * @see yfiles.graph.IPortDefaults#getLocationParameterInstance
       * @see yfiles.graph.IPortDefaults#locationParameter
       * @abstract
       * @type {boolean}
       */
      shareLocationParameterInstance:boolean;
    }
    var IPortDefaults:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Defines the geometry of {@link yfiles.graph.IPort} instances with respect to specific
     * {@link yfiles.graph.IPortLocationModelParameter}s.
     * <p>
     * In addition to defining the location of a port with respect to a specific
     * {@link yfiles.graph.IPortLocationModelParameter}, a port location model is typically the factory of the model parameters that
     * are compatible with it, too.
     * </p>
     * <p>
     * Typically, the port location model defines a compilation of valid locations for ports, and a specific model parameter
     * specifies an exact geometry for a port.
     * </p>
     * <p>
     * By design, the same instance of a port location model can be shared by multiple port location model parameters.
     * Similarly, the same instance of a port location model parameter can be shared by multiple ports.
     * </p>
     * <p>
     * This interface supports the {@link yfiles.graph.ILookup#lookup} method that can be used to query additional aspects of the
     * implementation.
     * </p>
     * @see yfiles.graph.IPortLocationModelParameter
     * @see yfiles.graph.IPort
     * @see yfiles.graph.IGraph
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface IPortLocationModel extends Object,yfiles.graph.ILookup{
      /**
       * Factory method that creates a parameter for the given port that tries to match the provided location in absolute world
       * coordinates.
       * @param owner The port owner that will own the port for which the parameter shall be created.
       * @param location The location in the world coordinate system that should be matched as best as possible.
       * @returns A new instance that can be used to describe the location of an {@link } at the given <code>owner</code>.
       * @abstract
       */
      createParameter(owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * Provides a {@link yfiles.graph.ILookup lookup context} for the given combination of port and parameter.
       * @param port The port to use in the context.
       * @param locationParameter The parameter to use for the port in the context.
       * @returns An implementation of the {@link } interface that can be used to query additional aspects of the
       * port/parameter combination.
       * @abstract
       */
      getContext(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.graph.ILookup;
      /**
       * Determines the location of the port for the given parameter.
       * @param port The port to determine the location for.
       * @param locationParameter The parameter to use. The parameter can be expected to be created by this instance having the {@link #model} property set to this
       * instance..
       * @returns The calculated location of the port.
       * @abstract
       */
      getLocation(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.geometry.Point;
    }
    var IPortLocationModel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Defines a state that can be used to {@link yfiles.graph.IPortLocationModel#getLocation determine the location of a port} with a {@link yfiles.graph.IPortLocationModel}.
     * <p>
     * Typically, implementations of this interface are created by factory methods of the corresponding {@link yfiles.graph.IPortLocationModelParameter#model}.
     * </p>
     * <p>
     * This interface extends the {@link yfiles.lang.ICloneable} interface. This allows clients to obtain a persistent copy of the
     * current state of this parameter. Immutable parameter implementations may return themselves.
     * </p>
     * @see yfiles.graph.IPortLocationModel
     * @see yfiles.graph.IPort
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface IPortLocationModelParameter extends Object,yfiles.lang.ICloneable{
      /**
       * Determines whether this parameter can be used for ports at the given <code>owner</code>.
       * <p>
       * A parameter must not necessarily support all types of ports. This method returns whether it supports ports at the given
       * <code>owner</code>.
       * </p>
       * @param owner The port owner to test.
       * @returns <code>true</code> if this instance can be used for ports of the given port owner, <code>false</code> otherwise.
       * @abstract
       */
      supports(owner:yfiles.graph.IPortOwner):boolean;
      /**
       * Gets the model that created this parameter.
       * @abstract
       * @type {yfiles.graph.IPortLocationModel}
       */
      model:yfiles.graph.IPortLocationModel;
    }
    var IPortLocationModelParameter:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum InteriorStretchLabelModelPosition{
      /**
       * Encodes a position at the top side of the node interior
       */
      NORTH,
      /**
       * Encodes a position at the right side of the node interior
       */
      EAST,
      /**
       * Encodes a position at the bottom side of the node interior
       */
      SOUTH,
      /**
       * Encodes a position at the left side of the node interior
       */
      WEST,
      /**
       * Encodes a position at the center of the node interior
       */
      CENTER
    }
    /**
     * A label model for node labels that lie inside of the node's {@link yfiles.graph.INode#layout} bounds and have been stretched to fit the node's size.
     * <p>
     * This label model supports 5 positions inside of the layout's bounds.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface InteriorStretchLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class InteriorStretchLabelModel {
      /**
       * Creates a new instance of this model with empty insets.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} options.insets The insets to use within the node's {@link yfiles.graph.INode#layout}.
       * <p>
       * This option sets the {@link yfiles.graph.InteriorStretchLabelModel#insets} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates the parameter for the given position.
       * @param position The position.
       * @returns 
       */
      createParameter(position:yfiles.graph.InteriorStretchLabelModelPosition):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * Returns the minimum size this model would require for the node owner of the label if the the given label was used with
       * the provided parameter.
       * @param label The label to use.
       * @param layoutParameter The parameter to use.
       * @returns The minimum size requirements for this label.
       */
      getMinimumNodeSize(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.Size;
      /**
       * Returns the insets this model would require for the node owner of the label if the the given label was used with the
       * provided parameter.
       * @param label The label to use.
       * @param layoutParameter The parameter to use.
       * @returns The insets rectangle.
       */
      getNodeInsets(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.Insets;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instance's {@link yfiles.graph.InteriorStretchLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instance's {@link yfiles.graph.InteriorStretchLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instance's {@link yfiles.graph.InteriorStretchLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instance's {@link yfiles.graph.InteriorStretchLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static WEST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instance's {@link yfiles.graph.InteriorStretchLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static CENTER:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the insets to use within the node's {@link yfiles.graph.INode#layout}.
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      static $class:yfiles.lang.Class;
    }
    export enum InteriorLabelModelPosition{
      /**
       * Encodes a position at the top side of the node interior
       */
      NORTH,
      /**
       * Encodes a position at the right side of the node interior
       */
      EAST,
      /**
       * Encodes a position at the bottom side of the node interior
       */
      SOUTH,
      /**
       * Encodes a position at the left side of the node interior
       */
      WEST,
      /**
       * Encodes a position at the upper right corner of the node interior
       */
      NORTH_EAST,
      /**
       * Encodes a position at the lower right corner of the node interior
       */
      SOUTH_EAST,
      /**
       * Encodes a position at the upper left corner of the node interior
       */
      NORTH_WEST,
      /**
       * Encodes a position at the lower left corner of the node interior
       */
      SOUTH_WEST,
      /**
       * Encodes a position at the center of the node interior
       */
      CENTER
    }
    /**
     * A label model for node labels that lie inside of the node's {@link yfiles.graph.INode#layout} bounds.
     * <p>
     * This label model supports nine positions inside of the layout's bounds.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface InteriorLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class InteriorLabelModel {
      /**
       * Creates a new instance of this model with empty insets.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} options.insets The insets to use within the node's {@link yfiles.graph.INode#layout}.
       * <p>
       * This option sets the {@link yfiles.graph.InteriorLabelModel#insets} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates the parameter for the given position.
       * @param position The position.
       * @returns 
       */
      createParameter(position:yfiles.graph.InteriorLabelModelPosition):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * Returns the minimum size this model would require for the node owner of the label if the the given label was used with
       * the provided parameter.
       * @param label The label to use.
       * @param layoutParameter The parameter to use.
       * @returns The minimum size requirements for this label.
       */
      getMinimumNodeSize(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.Size;
      /**
       * Returns the insets this model would require for the node owner of the label if the the given label was used with the
       * provided parameter.
       * @param label The label to use.
       * @param layoutParameter The parameter to use.
       * @returns The insets rectangle.
       */
      getNodeInsets(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.Insets;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH_EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH_WEST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static WEST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH_EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH_WEST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.InteriorLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static CENTER:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the insets to use within the node's {@link yfiles.graph.INode#layout}.
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      static $class:yfiles.lang.Class;
    }
    /**
     * The meta data that is associated with {@link yfiles.collections.IMapper.<K,V>}s in a {@link yfiles.graph.IMapperRegistry}.
     * @see yfiles.graph.IMapperRegistry#getMapperMetadata
     * @see yfiles.graph.IMapperRegistry#setMapperMetadata
     * @class
     */
    export interface MapperMetadata extends Object{}
    export class MapperMetadata {
      /**
       * Initializes a new instance of the {@link yfiles.graph.MapperMetadata} class.
       * @param keyType Type of the keys.
       * @param valueType Type of the values.
       */
      constructor(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class);
      /**
       * Initializes a new instance of the {@link yfiles.graph.MapperMetadata} from the given instance class.
       * @param metadata The metadata to copy key and value type from.
       * @protected
       */
      constructor(metadata:yfiles.graph.MapperMetadata);
      /**
       * Creates an instance for the specified mapper.
       * Type parameter <code>TKey</code>: The type of the keys.
       * Type parameter <code>TValue</code>: The type of the values.
       * @param valueType The type of the values.
       * @param keyType The type of the keys.
       * @param mapper The mapper instance to create the metadata for.
       * @returns An instance that uses the {@link #keyType} and {@link #valueType} of the mapper.
       * @template TKey,TValue
       * @static
       */
      static create<TKey,TValue>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,mapper:yfiles.collections.IMapper<TKey,TValue>):yfiles.graph.MapperMetadata;
      /**
       * 
       * @type {yfiles.lang.Class}
       */
      keyType:yfiles.lang.Class;
      /**
       * 
       * @type {yfiles.lang.Class}
       */
      valueType:yfiles.lang.Class;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple {@link yfiles.collections.Map.<TKey,TValue>}-based implementation of the {@link yfiles.graph.IMapperRegistry} interface.
     * @class
     * @implements {yfiles.graph.IMapperRegistry}
     * @final
     */
    export interface MapperRegistry extends Object,yfiles.graph.IMapperRegistry{}
    export class MapperRegistry {
      constructor();
      /**
       * 
       * Type parameter <code>K</code>.
       * Type parameter <code>V</code>.
       * @param valueType
       * @param keyType
       * @param tag
       * @param mapper
       * @template K,V
       */
      addMapper<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,tag:Object,mapper:yfiles.collections.IMapper<K,V>):void;
      /**
       * 
       * Type parameter <code>K</code>.
       * Type parameter <code>V</code>.
       * @param tag
       * @returns 
       * @template K,V
       */
      getMapper<K,V>(tag:Object):yfiles.collections.IMapper<K,V>;
      /**
       * 
       * @param tag
       * @returns 
       */
      getMapperMetadata(tag:Object):yfiles.graph.MapperMetadata;
      /**
       * 
       * @param tag
       */
      removeMapper(tag:Object):void;
      /**
       * 
       * @param tag
       * @param metadata
       */
      setMapperMetadata(tag:Object,metadata:yfiles.graph.MapperMetadata):void;
      /**
       * 
       * @type {yfiles.collections.IEnumerable.<Object>}
       */
      registeredTags:yfiles.collections.IEnumerable<Object>;
      static $class:yfiles.lang.Class;
    }
    export enum NinePositionsEdgeLabelModelPosition{
      /**
       * Encodes a label position at the source above the edge
       */
      SOURCE_ABOVE,
      /**
       * Encodes a label position at the center above the edge
       */
      CENTER_ABOVE,
      /**
       * Encodes a label position at the target above the edge
       */
      TARGET_ABOVE,
      /**
       * Encodes a label position at the source of the edge that lies on the edge
       */
      SOURCE_CENTERED,
      /**
       * Encodes a label position at the center of the edge that lies on the edge
       */
      CENTER_CENTERED,
      /**
       * Encodes a label position at the source of the edge that lies on the edge
       */
      TARGET_CENTERED,
      /**
       * Encodes a label position at the source below the edge
       */
      SOURCE_BELOW,
      /**
       * Encodes a label position at the center below the edge
       */
      CENTER_BELOW,
      /**
       * Encodes a label position at the target below the edge
       */
      TARGET_BELOW
    }
    /**
     * An edge label model that knows nine different label positions.
     * <p>
     * The possible positions are near the source port, at the center of the edge, or near the target port, each above, below,
     * or centered on the edge. The distance to the edge and the absolute angular orientation of the label can be specified.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface NinePositionsEdgeLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class NinePositionsEdgeLabelModel {
      /**
       * Initializes a new instance of the {@link yfiles.graph.NinePositionsEdgeLabelModel} class.
       * @param distance The distance.
       * @param angle The angle.
       */
      constructor(distance:number,angle:number);
      /**
       * Initializes a new instance of the {@link yfiles.graph.NinePositionsEdgeLabelModel} class with a {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} of 10 and an {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} of 0.
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.angle The angle for the rotation of the labels.
       * <p>
       * This option sets the {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} property on the created object.
       * </p>
       * @param {number} options.distance The distance of the labels above or below the edge.
       * <p>
       * This option sets the {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} property on the created object.
       * </p>
       */
      constructor(options?:{angle?:number,distance?:number});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Create a model parameter for the specified <code>position</code>
       * @param position The position for this parameter
       * @returns a model parameter for the specified <code>position</code>
       */
      createParameter(position:yfiles.graph.NinePositionsEdgeLabelModelPosition):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOURCE_ABOVE:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static CENTER_ABOVE:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static TARGET_ABOVE:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOURCE_CENTERED:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static CENTER_CENTERED:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static TARGET_CENTERED:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOURCE_BELOW:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static CENTER_BELOW:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.NinePositionsEdgeLabelModel#angle} or {@link yfiles.graph.NinePositionsEdgeLabelModel#distance} properties will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static TARGET_BELOW:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the angle for the rotation of the labels.
       * <p>
       * A value of <code>0</code> makes the label appear horizontally aligned.
       * </p>
       * @default 0.0
       * @type {number}
       */
      angle:number;
      /**
       * Gets or sets the distance of the labels above or below the edge.
       * @default 10.0
       * @type {number}
       */
      distance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A node label model that knows only two possible positions.
     * <p>
     * The label will be placed centered on top or below the node's {@link yfiles.graph.INode#layout} using a specifiable {@link yfiles.graph.SandwichLabelModel#yOffset}.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface SandwichLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class SandwichLabelModel {
      /**
       * Initializes a new instance of the {@link yfiles.graph.SandwichLabelModel} class with zero offset.
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.yOffset The distance between the node's {@link yfiles.graph.INode#layout} and the label's {@link yfiles.graph.ILabel#layout layout}.
       * <p>
       * This option sets the {@link yfiles.graph.SandwichLabelModel#yOffset} property on the created object.
       * </p>
       */
      constructor(options?:{yOffset?:number});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter for the north side of the node.
       * @returns 
       */
      createNorthParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter for the south side of the node.
       * @returns 
       */
      createSouthParameter():yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.SandwichLabelModel#yOffset} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.SandwichLabelModel#yOffset} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the distance between the node's {@link yfiles.graph.INode#layout} and the label's {@link yfiles.graph.ILabel#layout layout}.
       * @default 0.0
       * @type {number}
       */
      yOffset:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A port location model that places the port on a certain edge segment at a specified ratio.
     * @class
     * @implements {yfiles.graph.IPortLocationModel}
     * @final
     */
    export interface SegmentRatioPortLocationModel extends Object,yfiles.graph.IPortLocationModel{}
    export class SegmentRatioPortLocationModel {
      /**
       * Creates the a new parameter for the specified segment index and ratio.
       * @param ratio The ratio for the segment, with <code>0</code> being at the start of the segment and <code>1</code> at the end.
       * @param segmentIndex The index of the segment.
       * @returns A parameter for the given ratio and segment.
       */
      createFromSource(ratio:number,segmentIndex:number):yfiles.graph.IPortLocationModelParameter;
      /**
       * Creates the a new parameter for the specified segment index counted from the target end of the edge and the provided
       * ratio.
       * @param ratio The ratio for the segment, with <code>0</code> being at the start of the segment and <code>1</code> at the end.
       * @param segmentIndex The index of the segment counted from the target end.
       * @returns A parameter for the given ratio and segment.
       */
      createFromTarget(ratio:number,segmentIndex:number):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param owner
       * @param location
       * @returns 
       */
      createParameter(owner:yfiles.graph.IPortOwner,location:yfiles.geometry.Point):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getContext(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.graph.ILookup;
      /**
       * 
       * @param port
       * @param locationParameter
       * @returns 
       */
      getLocation(port:yfiles.graph.IPort,locationParameter:yfiles.graph.IPortLocationModelParameter):yfiles.geometry.Point;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A static immutable global instance of this class.
       * @const
       * @static
       * @type {yfiles.graph.SegmentRatioPortLocationModel}
       */
      static INSTANCE:yfiles.graph.SegmentRatioPortLocationModel;
      static $class:yfiles.lang.Class;
    }
    /**
     * An edge label model that allows the placement at any position.
     * <p>
     * Similar to
     * {@link yfiles.graph.EdgeSegmentLabelModel}, a position is specified by an edge segment and a ratio on that segment, amongst
     * others. During changes of the edge path, the label keeps its relative location with respect to the bends of the
     * reference segment in a similar way.
     * </p>
     * <p>
     * In contrast to {@link yfiles.graph.EdgeSegmentLabelModel}, the distance from the edge path is not a property of the model but of
     * the particular parameter. In addition, this class does not implement {@link yfiles.graph.ILabelModelParameterProvider} and
     * therefore, labels with this model can be moved freely and are not restricted to a fixed set of candidates at a given
     * distance from the edge.
     * </p>
     * <p>
     * If {@link yfiles.graph.SmartEdgeLabelModel#autoRotation} is enabled, labels are automatically rotated according to the angle of the corresponding reference edge segment.
     * </p>
     * <p>
     * During movements, labels with this model snap to noteable positions if the {@link yfiles.input.MoveLabelInputMode} provides a {@link yfiles.input.LabelSnapContext}
     * that is enabled and has a suitable configuration.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterFinder}
     */
    export interface SmartEdgeLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterFinder{}
    export class SmartEdgeLabelModel {
      /**
       * Initializes a new instance of this class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.angle The rotation angle of all labels with this model.
       * <p>
       * This option sets the {@link yfiles.graph.SmartEdgeLabelModel#angle} property on the created object.
       * </p>
       * @param {boolean} options.autoRotation Whether or not edge labels are automatically rotated according to the angle of the corresponding reference edge segment.
       * <p>
       * This option sets the {@link yfiles.graph.SmartEdgeLabelModel#autoRotation} property on the created object.
       * </p>
       */
      constructor(options?:{angle?:number,autoRotation?:boolean});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter that measures the provided segment index from the source side of the edge path.
       * @param segmentIndex The zero-based index of the segment beginning from the source side.
       * @param distance The distance between the label's box and the edge's path.
       * @param segmentRatio The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the source side of the
       * segment, a ratio of <code>1.0</code> at the target side. Ratios lesser than <code>0.0</code> or greater than <code>1.0</code> will be interpreted as absolute
       * values in world coordinates.
       * @returns A label parameter that describes the provided parameters for this model instance.
       */
      createParameterFromSource(segmentIndex:number,distance:number,segmentRatio:number):yfiles.graph.ILabelModelParameter;
      /**
       * Creates a parameter that measures the provided segment index from the target side of the edge path.
       * @param segmentIndex The zero-based index of the segment beginning from the target side.
       * @param distance The distance between the label's box and the edge's path.
       * @param segmentRatio The ratio at which to place the label at the segment. A ratio of <code>0.0</code> will place the label at the target side of the
       * segment, a ratio of <code>1.0</code> at the source side. Ratios lesser than <code>0.0</code> or greater than <code>1.0</code> will be interpreted as absolute
       * values in world coordinates.
       * @returns A label parameter that describes the provided parameters for this model instance.
       */
      createParameterFromTarget(segmentIndex:number,distance:number,segmentRatio:number):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param model
       * @param layout
       * @returns 
       */
      findBestParameter(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel,layout:yfiles.geometry.IOrientedRectangle):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * Gets the distance from the edge path described by the specified parameter.
       * @param layoutParameter The parameter to get the edge path distance for.
       * @returns The distance from the edge path
       */
      getDistance(layoutParameter:yfiles.graph.ILabelModelParameter):number;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Gets or sets whether or not edge labels are automatically rotated according to the angle of the corresponding reference
       * edge segment.
       * <p>
       * By default, this feature is enabled.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoRotation:boolean;
      /**
       * Gets or sets the rotation angle of all labels with this model.
       * @default 0.0
       * @type {number}
       */
      angle:number;
      static $class:yfiles.lang.Class;
    }
    export enum StripeTypes{
      /**
       * Constant for rows.
       * @see yfiles.graph.StripeTypes#ROW
       */
      ROW,
      /**
       * Constant for columns.
       * @see yfiles.graph.StripeTypes#COLUMN
       */
      COLUMN,
      /**
       * Constant for all item types.
       * <p>
       * This is a bitwise combination of all other types declared in this enum.
       * </p>
       */
      ALL,
      /**
       * Constant for no item type.
       * <p>
       * This is a constant where no type bit has been set, i.e. it is <code>0</code>.
       * </p>
       */
      NONE
    }
    /**
     * A canonic implementation of the {@link yfiles.graph.IPortDefaults} interface.
     * @class
     * @implements {yfiles.graph.IPortDefaults}
     */
    export interface PortDefaults extends Object,yfiles.graph.IPortDefaults{}
    export class PortDefaults {
      /**
       * Initializes a new instance of the {@link yfiles.graph.PortDefaults} class using a trivial location model parameter.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} [options.autoCleanup=null] Sets the autoCleanup property.
       * @param {yfiles.graph.IPortLocationModelParameter} [options.locationParameter=null] Sets the locationParameter property.
       * @param {boolean} [options.shareLocationParameterInstance=null] Sets the shareLocationParameterInstance property.
       * @param {yfiles.styles.IPortStyle} [options.style=null] Sets the style property.
       * @param {boolean} [options.shareStyleInstance=null] Sets the shareStyleInstance property.
       */
      constructor(options?:{autoCleanup?:boolean,locationParameter?:yfiles.graph.IPortLocationModelParameter,shareLocationParameterInstance?:boolean,style?:yfiles.styles.IPortStyle,shareStyleInstance?:boolean});
      /**
       * Factory method that returns a style instance for use with newly created ports.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.PortDefaults#style} property, if {@link yfiles.graph.PortDefaults#shareStyleInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @returns The style to use, which for most implementations is either a {@link #clone clone} of or the {@link #style} property, if {@link #shareStyleInstance} is enabled.
       */
      GetDefaultStyleInstance():yfiles.styles.IPortStyle;
      /**
       * 
       * @param owner
       * @returns 
       */
      getLocationParameterInstance(owner:yfiles.graph.IPortOwner):yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @param owner
       * @returns 
       */
      getStyleInstance(owner:yfiles.graph.IPortOwner):yfiles.styles.IPortStyle;
      /**
       * 
       * @default true
       * @type {boolean}
       */
      autoCleanup:boolean;
      /**
       * 
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * 
       * @default true
       * @type {boolean}
       */
      shareLocationParameterInstance:boolean;
      /**
       * 
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * 
       * @default true
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A canonic implementation of the {@link yfiles.graph.INodeDefaults} interface.
     * @class
     * @implements {yfiles.graph.INodeDefaults}
     */
    export interface NodeDefaults extends Object,yfiles.graph.INodeDefaults{}
    export class NodeDefaults {
      /**
       * Initializes a new instance of the {@link yfiles.graph.NodeDefaults} class using default properties.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Size} [options.size=null] Sets the size property.
       * @param {yfiles.graph.ILabelDefaults} [options.labels=null] Sets the labels property.
       * @param {yfiles.graph.IPortDefaults} [options.ports=null] Sets the ports property.
       * @param {yfiles.styles.INodeStyle} [options.style=null] Sets the style property.
       * @param {boolean} [options.shareStyleInstance=null] Sets the shareStyleInstance property.
       */
      constructor(options?:{size?:yfiles.geometry.Size,labels?:yfiles.graph.ILabelDefaults,ports?:yfiles.graph.IPortDefaults,style?:yfiles.styles.INodeStyle,shareStyleInstance?:boolean});
      /**
       * 
       * @returns 
       */
      getStyleInstance():yfiles.styles.INodeStyle;
      /**
       * 
       * @type {yfiles.geometry.Size}
       */
      size:yfiles.geometry.Size;
      /**
       * 
       * @type {yfiles.graph.ILabelDefaults}
       */
      labels:yfiles.graph.ILabelDefaults;
      /**
       * 
       * @type {yfiles.graph.IPortDefaults}
       */
      ports:yfiles.graph.IPortDefaults;
      /**
       * 
       * @type {yfiles.styles.INodeStyle}
       */
      style:yfiles.styles.INodeStyle;
      /**
       * 
       * @default true
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A canonic implementation of the {@link yfiles.graph.IEdgeDefaults} interface
     * @class
     * @implements {yfiles.graph.IEdgeDefaults}
     */
    export interface EdgeDefaults extends Object,yfiles.graph.IEdgeDefaults{}
    export class EdgeDefaults {
      /**
       * Initializes a new instance of the {@link yfiles.graph.EdgeDefaults} class with default edge style and label and port defaults.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.graph.ILabelDefaults} [options.labels=null] Sets the labels property.
       * @param {yfiles.graph.IPortDefaults} [options.ports=null] Sets the ports property.
       * @param {yfiles.styles.IEdgeStyle} [options.style=null] Sets the style property.
       * @param {boolean} [options.shareStyleInstance=null] Sets the shareStyleInstance property.
       */
      constructor(options?:{labels?:yfiles.graph.ILabelDefaults,ports?:yfiles.graph.IPortDefaults,style?:yfiles.styles.IEdgeStyle,shareStyleInstance?:boolean});
      /**
       * 
       * @returns 
       */
      getStyleInstance():yfiles.styles.IEdgeStyle;
      /**
       * 
       * @type {yfiles.graph.ILabelDefaults}
       */
      labels:yfiles.graph.ILabelDefaults;
      /**
       * 
       * @type {yfiles.graph.IPortDefaults}
       */
      ports:yfiles.graph.IPortDefaults;
      /**
       * 
       * @type {yfiles.styles.IEdgeStyle}
       */
      style:yfiles.styles.IEdgeStyle;
      /**
       * 
       * @default true
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A canonic implementation of the {@link yfiles.graph.ILabelDefaults} interface
     * @class
     * @implements {yfiles.graph.ILabelDefaults}
     */
    export interface LabelDefaults extends Object,yfiles.graph.ILabelDefaults{}
    export class LabelDefaults {
      /**
       * Initializes a new instance of the {@link yfiles.graph.LabelDefaults} class using a trivial parameter.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} [options.autoAdjustPreferredSize=null] Sets the autoAdjustPreferredSize property.
       * @param {boolean} [options.shareLayoutParameterInstance=null] Sets the shareLayoutParameterInstance property.
       * @param {yfiles.graph.ILabelModelParameter} [options.layoutParameter=null] Sets the layoutParameter property.
       * @param {boolean} [options.shareStyleInstance=null] Sets the shareStyleInstance property.
       * @param {yfiles.styles.ILabelStyle} [options.style=null] Sets the style property.
       */
      constructor(options?:{autoAdjustPreferredSize?:boolean,shareLayoutParameterInstance?:boolean,layoutParameter?:yfiles.graph.ILabelModelParameter,shareStyleInstance?:boolean,style?:yfiles.styles.ILabelStyle});
      /**
       * 
       * @param owner
       * @returns 
       */
      getLayoutParameterInstance(owner:yfiles.graph.ILabelOwner):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param owner
       * @returns 
       */
      getStyleInstance(owner:yfiles.graph.ILabelOwner):yfiles.styles.ILabelStyle;
      /**
       * 
       * @type {boolean}
       */
      autoAdjustPreferredSize:boolean;
      /**
       * 
       * @type {boolean}
       */
      shareLayoutParameterInstance:boolean;
      /**
       * 
       * @type {yfiles.graph.ILabelModelParameter}
       */
      layoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @type {yfiles.styles.ILabelStyle}
       */
      style:yfiles.styles.ILabelStyle;
      /**
       * 
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for implementations that can calculate the bounds of a group node in a grouped graph.
     * <p>
     * Implementations of this interface will be queried from {@link yfiles.graph.INode}'s {@link yfiles.graph.ILookup#lookup} method.
     * </p>
     * @interface
     */
    export interface IGroupBoundsCalculator extends Object{
      /**
       * Calculates the bounds of the {@link yfiles.graph.INode#layout} for the given <code>groupNode</code> that is a group node in the <code>graph</code>.
       * @param graph The graph which holds the <code>groupNode</code>.
       * @param groupNode The node to calculate the bounds for.
       * @returns The minimum bounds to use for the given group node.
       * @abstract
       */
      calculateBounds(graph:yfiles.graph.IGraph,groupNode:yfiles.graph.INode):yfiles.geometry.Rect;
    }
    var IGroupBoundsCalculator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A group of {@link yfiles.graph.IUndoUnit}s that are undone/redone as a single unit.
     * <p>
     * Clients can use this class to build an {@link yfiles.graph.IUndoUnit} that groups together multiple {@link yfiles.graph.IUndoUnit}s. The
     * {@link yfiles.graph.IUndoUnit}s added to this unit are undone/redone in the order in which they were added. This implementation
     * will try to {@link yfiles.graph.IUndoUnit#tryMergeUnit merge} or {@link yfiles.graph.IUndoUnit#tryReplaceUnit replace} units that are {@link yfiles.graph.CompositeUndoUnit#add added}.
     * </p>
     * @class
     * @implements {yfiles.graph.IUndoUnit}
     */
    export interface CompositeUndoUnit extends Object,yfiles.graph.IUndoUnit{}
    export class CompositeUndoUnit {
      /**
       * Creates a new instance using the provided names.
       * @param [undoName=null]
       * @param [redoName=null]
       */
      constructor(undoName?:string,redoName?:string);
      /**
       * Creates a new instance using the provided names.
       * @param {Object} options The parameters to pass.
       * @param [options.undoName=null]
       * @param [options.redoName=null]
       */
      constructor(options:{undoName?:string,redoName?:string});
      /**
       * Adds a new {@link yfiles.graph.IUndoUnit} to the queue.
       * @param unit The unit of work to add.
       */
      add(unit:yfiles.graph.IUndoUnit):void;
      /**
       * Determines whether a call to {@link yfiles.graph.CompositeUndoUnit#redo} can be made.
       * @returns 
       */
      canRedo():boolean;
      /**
       * Determines whether a call to {@link yfiles.graph.CompositeUndoUnit#undo} can be made.
       * @returns 
       */
      canUndo():boolean;
      /**
       * Clears the internal collection of units.
       */
      clear():void;
      /**
       * Disposes this unit and all of its contents.
       * <p>
       * This unit cannot be used anymore after this has been invoked.
       * </p>
       */
      dispose():void;
      /**
       * Redoes the next {@link yfiles.graph.IUndoUnit}.
       * @throws {Stubs.Exceptions.NotSupportedError} If an undo operation is already in progress.
       * @throws {yfiles.lang.Exception} If {@link yfiles.graph.CompositeUndoUnit#canRedo} would yield <code>false</code>.
       */
      redo():void;
      /**
       * Tries to remove a given unit from this compound unit.
       * <p>
       * This method will throw an exception if this unit has been undone or an undo or redo is in progress or this unit has
       * already been disposed. This method will not automatically {@link yfiles.graph.IUndoUnit#dispose Dispose} removed units.
       * </p>
       * @param unit The unit to remove.
       * @returns <code>true</code> iff the unit has been removed from this instance.
       */
      removeUnit(unit:yfiles.graph.IUndoUnit):boolean;
      /**
       * Undoes the next {@link yfiles.graph.IUndoUnit}.
       * @throws {Stubs.Exceptions.NotSupportedError} If an undo operation is already in progress.
       * @throws {yfiles.lang.Exception} If {@link yfiles.graph.CompositeUndoUnit#canUndo} would yield <code>false</code>.
       */
      undo():void;
      /**
       * Yields the number of units that are contained in this compound unit.
       * @type {number}
       */
      count:number;
      /**
       * Returns the current redo name.
       * @type {string}
       */
      redoName:string;
      /**
       * Returns the current redo name.
       * @type {string}
       */
      undoName:string;
      /**
       * Determines whether this instance has been {@link yfiles.graph.CompositeUndoUnit#dispose disposed.}
       * <p>
       * An instance that has been disposed cannot be {@link yfiles.graph.CompositeUndoUnit#undo undone} or {@link yfiles.graph.CompositeUndoUnit#redo redone}.
       * </p>
       * @type {boolean}
       */
      disposed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This interface is a means to bracket several
     * {@link yfiles.graph.IUndoUnit}s, that is a collection of units that are treated as one (for example they are always undone or
     * redone as a single unit).
     * <p>
     * {@link yfiles.graph.ICompoundEdit}s have two states: <i>active</i> and <i>settled</i>. Whenever an {@link yfiles.graph.ICompoundEdit} is retrieved from the {@link yfiles.graph.UndoEngine}
     * or the graph instance, it automatically is <i>active</i>. While an edit is in the <i>active</i> state, it gathers {@link yfiles.graph.IUndoUnit}s until
     * the {@link yfiles.graph.ICompoundEdit#commit} or {@link yfiles.graph.ICompoundEdit#cancel} methods are called. Calling the {@link yfiles.graph.ICompoundEdit#commit}
     * method commits all gathered {@link yfiles.graph.IUndoUnit}s to the {@link yfiles.graph.UndoEngine}, while canceling will {@link yfiles.graph.IUndoUnit#dispose dispose} them.
     * Calling either method will put the edit in the <i>settled</i> state. In this state, the edit does not gather units anymore and is not
     * reusable.
     * </p>
     * <p>
     * {@link yfiles.graph.ICompoundEdit}s can also be nested, in this case the changes of the inner edit are not committed directly to
     * the {@link yfiles.graph.UndoEngine} but to the outer edit. Only when the outer edit is committed, the units are committed to the
     * {@link yfiles.graph.UndoEngine}.
     * </p>
     * @see yfiles.graph.UndoEngine
     * @see yfiles.graph.IUndoUnit
     * @interface
     */
    export interface ICompoundEdit extends Object{
      /**
       * This method cancels the compound editing process.
       * <p>
       * The recorded {@link yfiles.graph.IUndoUnit}s are {@link yfiles.graph.IUndoUnit#dispose disposed} and the edit is not usable anymore.
       * </p>
       * @abstract
       */
      cancel():void;
      /**
       * This method commits the compound editing process.
       * <p>
       * The {@link yfiles.graph.IUndoUnit}s are enqueued into the corresponding {@link yfiles.graph.UndoEngine} instance and the edit is not
       * usable anymore.
       * </p>
       * @abstract
       */
      commit():void;
    }
    var ICompoundEdit:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The basic lookup interface that allows for retrieving implementations for different aspects of the current instance.
     * <p>
     * The lookup idiom allows for dynamically providing, adding, changing, and removing implementation aspects of an instance
     * that would not be possible to achieve using ordinary object oriented methods.
     * </p>
     * @interface
     */
    export interface ILookup extends Object{
      /**
       * Returns an instance that implements the given type or <code>null</code>.
       * <p>
       * Typically, this method will be called in order to obtain a different view or aspect of the current instance. This is
       * quite similar to casting or using a super type or interface of this instance, but is not limited to inheritance or
       * compile time constraints. An instance implementing this method is not required to return non-<code>null</code> implementations for
       * the types, nor does it have to return the same instance any time. Also it depends on the type and context whether the
       * instance returned stays up to date or needs to be reobtained for subsequent use.
       * </p>
       * @param type the type for which an instance shall be returned
       * @returns an instance that is assignable to type or <code>null</code>
       * @abstract
       */
      lookup(type:yfiles.lang.Class):Object;
    }
    var ILookup:{
      /**
       * Creates a lookup implementation that wraps an existing <code>ILookup</code> instance using an additional lookup implementations.
       * <p>
       * For each lookup call that the <code>additionalLookup</code> would yield <code>null</code>, the <code>basicLookup</code> will be queried instead.
       * </p>
       * @param basicLookup the fallback lookup implementation
       * @param additionalLookup the lookup that has precedence over the fallback
       * @returns an implementation that decorates the first with the latter
       * @static
       */
      createCascadingLookup?(basicLookup:yfiles.graph.ILookup,additionalLookup:yfiles.graph.ILookup):yfiles.graph.ILookup;
      /**
       * Creates a dynamic lookup implementation that for each type that the given subject can be assigned to yields that
       * subject.
       * @param subject the subject to yield if it is assignable to the type in the lookup query
       * @returns an instance that will either yield <code>subject</code> or <code>null</code>
       * @static
       */
      createDynamicLookup?(subject:Object):yfiles.graph.ILookup;
      /**
       * Creates a simple mutable {@link yfiles.graph.ILookup} implementation that is backed by an {@link yfiles.collections.IMap.<TKey,TValue>}.
       * <p>
       * The {@link yfiles.graph.ILookup#lookup lookup} method uses the specified type as key when delegating to the <code>backingDictionary</code>.
       * </p>
       * @param backingDictionary The backing {@link } that is used in the {@link #lookup lookup} calls.
       * @returns A simple mutable {@link } implementation that is backed by a dictionary.
       * @static
       */
      createMapLookup?(backingDictionary:yfiles.collections.Map<yfiles.lang.Class,Object>):yfiles.graph.ILookup;
      /**
       * Creates a simple lookup implementation that yields <code>subject</code> if <code>type</code> is queried.
       * @param subject the subject to yield
       * @param type the type that should yield the subject
       * @returns an ILookup implementation
       * @static
       */
      createSingleLookup?(subject:Object,type:yfiles.lang.Class):yfiles.graph.ILookup;
      /**
       * An <code>ILookup</code> instance that always returns <code>null</code>.
       * @const
       * @static
       * @type {yfiles.graph.ILookup}
       */
      EMPTY?:yfiles.graph.ILookup;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface for instances that can provide a {@link yfiles.graph.ILookup} capabilities for a given object.
     * <p>
     * This is useful in the case where lookup functionality is not intrinsic to an item or where an existing object cannot
     * implement the interface or when an item needs to be decorated externally with new lookup functionality.
     * </p>
     * @see yfiles.graph.ILookup
     * @interface
     */
    export interface IContextLookup extends Object{
      /**
       * Tries to create or retrieve an implementation of the given type for a given item.
       * @param item the item to lookup a type for
       * @param type the type to look up
       * @returns an implementation or <code>null</code>
       * @see yfiles.graph.ILookup#lookup
       * @abstract
       */
      contextLookup(item:Object,type:yfiles.lang.Class):Object;
    }
    var IContextLookup:{
      /**
       * An <code>ILookupContext</code> instance that always returns <code>null</code>.
       * @const
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      EMPTY_CONTEXT_LOOKUP?:yfiles.graph.IContextLookup;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An interface for instances that can be used to chain lookup operations.
     * <p>
     * Implementations will perform the lookup in the {@link yfiles.graph.IContextLookup#contextLookup} method and delegate to the next
     * implementation in the chain if they cannot satisfy the request.
     * </p>
     * @interface
     * @implements {yfiles.graph.IContextLookup}
     */
    export interface IContextLookupChainLink extends Object,yfiles.graph.IContextLookup{
      /**
       * This method is called by the framework to register the fallback lookup implementation that should be used during a call
       * to {@link yfiles.graph.IContextLookup#contextLookup}.
       * <p>
       * If the implementation cannot satisfy the query, it will use the provided context as a fallback. Note that
       * implementations can also use the results returned by the next lookup and decorate it appropriately.
       * </p>
       * @param next The context to use as a fallback.
       * @abstract
       */
      setNext(next:yfiles.graph.IContextLookup):void;
    }
    var IContextLookupChainLink:{
      /**
       * Factory method for a lookup chain element that simply adds an entry to a lookup chain.
       * @param type The type to add to the lookup.
       * @param instance The instance to return if the <code>type</code> is being looked up.
       * @returns An instance that can be used in a lookup chain.
       * @static
       */
      addingLookupChainLink?(type:yfiles.lang.Class,instance:Object):yfiles.graph.IContextLookupChainLink;
      /**
       * Factory method for a lookup chain element that adds an {@link yfiles.graph.ILookup} layer to a lookup chain.
       * <p>
       * If the lookup returns <code>null</code>, the call will be propagated to the chain.
       * </p>
       * @param lookup The lookup to use.
       * @returns An instance that can be used in a lookup chain.
       * @static
       */
      addingLookupChainLink?(lookup:yfiles.graph.ILookup):yfiles.graph.IContextLookupChainLink;
      /**
       * Factory method for a lookup chain element that adds an dynamic instance to a lookup chain.
       * <p>
       * The instance returned will check whether <code>instance</code> is of the queried type and will then return the instance.
       * </p>
       * @param instance The instance to query the type from dynamically.
       * @returns An instance that can be used in a lookup chain.
       * @static
       */
      addingLookupChainLink?(instance:Object):yfiles.graph.IContextLookupChainLink;
      /**
       * Creates a lookup chain link that is backed by the given callback.
       * <p>
       * If the callback returns <code>null</code> the request is passed to the chain.
       * </p>
       * @param callback The callback to invoke.
       * @returns The implementation of the link that uses the callback.
       * @static
       */
      createContextLookupChainLink?(callback:(subject:Object,type:yfiles.lang.Class)=>Object):yfiles.graph.IContextLookupChainLink;
      /**
       * Factory method for a lookup chain element that uses a function of type <code>function(</code><code>TContext</code><code>)</code> to yield a <code>TResult</code> for a specific <code>TContext</code>.
       * <p>
       * If the resulting link is {@link yfiles.graph.IContextLookup#contextLookup queried} for <code>TResult</code> and the context is of type <code>TContext</code>, the <code>factory</code> method will be queried and the result
       * will be returned. Otherwise the {@link yfiles.graph.IContextLookupChainLink#setNext next} chain link will be delegated the request to.
       * </p>
       * Type parameter <code>TContext</code>.
       * Type parameter <code>TResult</code>.
       * @param factory The factory delegate to use that yields the result for a given context.
       * @returns An instance that can be used in a lookup chain.
       * @template TContext,TResult
       * @static
       */
      factoryLookupChainLink?<TContext,TResult>(contextType:yfiles.lang.Class,resultType:yfiles.lang.Class,factory:(arg:TContext)=>TResult):yfiles.graph.IContextLookupChainLink;
      /**
       * Factory method for a lookup chain element that removes the entry from the lookup chain for a given type.
       * @param type The type to hide from the lookup.
       * @returns An instance that can be used in a lookup chain.
       * @static
       */
      hidingLookupChainLink?(type:yfiles.lang.Class):yfiles.graph.IContextLookupChainLink;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that can be used to decorate the lookup result for a set of types.
     * @see yfiles.graph.ILookup
     * @see yfiles.graph.IContextLookupChainLink
     * @interface
     */
    export interface ILookupDecorator extends Object{
      /**
       * Adds a lookup implementation for a given type using a function of type <code>function(TContext, TResult)</code>.
       * <p>
       * This method is closely related to the {@link yfiles.graph.ILookupDecorator#add} method, however it passes the result of the
       * remaining chain to the delegate for convenient wrapping of the result. Note that if the implementation does not depend
       * on the previous result in order to wrap it, it is better to use that variant, in order to avoid the possibly costly
       * evaluation of the remainder of the lookup chain.
       * </p>
       * Type parameter <code>TContext</code>: The type of the context subject.
       * Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
       * @param wrapperFactory The factory delegate that will be used to satisfy queries of type <code>TResult</code>
       * @param [decorateNull=true] Whether to actually decorate <code>null</code> results of remainder of the chain. If this is set to <code>false</code>, the <code>wrapperFactory</code> will
       * never be called with <code>null</code> as the second argument but the result of the query for this chain link will be the <code>null</code>
       * value.
       * @returns The {@link } instance as returned by the {@link #addLookup} call or
       * <code>null</code>, if either <code>decorator</code> was <code>null</code>, or it {@link #canDecorate could not decorate} the <code>TContext</code> type.
       * @template TContext,TResult
       */
      add?<TContext,TResult>(contextType:yfiles.lang.Class,resultType:yfiles.lang.Class,wrapperFactory:(arg1:TContext,arg2:TResult)=>TResult,decorateNull?:boolean):yfiles.graph.IContextLookupChainLink;
      /**
       * Adds a lookup implementation for a given type using a func.
       * Type parameter <code>TContext</code>: The type of the context subject.
       * Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
       * @param factory The factory delegate that will be used to satisfy queries of type <code>TResult</code>.
       * @param [nullIsFallback=true] Whether to treat <code>null</code>-results of the <code>factory</code> as hints to use the remainder of the chain link or to actually yield the
       * value as the final result.
       * @returns The {@link } instance as returned by the {@link #addLookup} call or
       * <code>null</code>, if either <code>decorator</code> was <code>null</code>, or it {@link #canDecorate could not decorate} the <code>TContext</code> type.
       * @template TContext,TResult
       */
      add?<TContext,TResult>(contextType:yfiles.lang.Class,resultType:yfiles.lang.Class,factory:(arg:TContext)=>TResult,nullIsFallback?:boolean):yfiles.graph.IContextLookupChainLink;
      /**
       * Add a lookup implementation for a given type that always yields a constant result.
       * Type parameter <code>TContext</code>: The type of the context subject.
       * Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
       * @param result The constant value to yield if the chain is queried for an implementation of type <code>TResult</code>.
       * @returns The {@link } instance as returned by the {@link #addLookup} call or
       * <code>null</code>, if either <code>decorator</code> was <code>null</code>, or it {@link #canDecorate could not decorate} the <code>TContext</code> type.
       * @template TContext,TResult
       */
      addConstant?<TContext,TResult>(contextType:yfiles.lang.Class,resultType:yfiles.lang.Class,result:TResult):yfiles.graph.IContextLookupChainLink;
      /**
       * Adds a lookup chain element to the chain of lookups for a given type.
       * <p>
       * The lookup chain is context specific. This method will not generally decorate the lookup of all instances of type <code>t</code>, but
       * only those that stand in context of this instance. This method will only succeed if {@link yfiles.graph.ILookupDecorator#canDecorate}
       * yields <code>true</code> for <code>t</code>.
       * </p>
       * @param t The type to decorate the lookup.
       * @param lookup The chain element to add to the lookup of the given type.
       * @see yfiles.graph.ILookupDecorator#removeLookup
       * @see yfiles.graph.ILookupDecorator#add
       * @see yfiles.graph.ILookupDecorator#add
       * @see yfiles.graph.ILookupDecorator#addConstant
       * @abstract
       */
      addLookup(t:yfiles.lang.Class,lookup:yfiles.graph.IContextLookupChainLink):void;
      /**
       * Determines whether this instance can be used to decorate the lookup for a certain type.
       * @param t The type to decorate the lookup for.
       * @returns 
       * @abstract
       */
      canDecorate(t:yfiles.lang.Class):boolean;
      /**
       * Removes a previously registered lookup chain element from the chain of lookups for a given type.
       * @param t The type to remove the decorator from.
       * @param lookup The chain element to remove from the lookup of the given type.
       * @see yfiles.graph.ILookupDecorator#addLookup
       * @abstract
       */
      removeLookup(t:yfiles.lang.Class,lookup:yfiles.graph.IContextLookupChainLink):void;
    }
    var ILookupDecorator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface implemented by items that can be tagged with arbitrary objects.
     * <p>
     * The object held by implementations of this interface is usually a user defined object. Implementations don't depend on
     * this tag. They just serve as a storage for the tag that will be used by other parts of the implementation.
     * </p>
     * @interface
     */
    export interface ITagOwner extends Object{
      /**
       * Gets or sets the tag associated with this instance.
       * <p>
       * The implementation itself does not normally depend on the tag associated with it. It serves as storage for the object
       * only.
       * </p>
       * @abstract
       * @type {Object}
       */
      tag:Object;
    }
    var ITagOwner:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An implementation of a chain of {@link yfiles.graph.IContextLookupChainLink}s.
     * @class
     * @implements {yfiles.graph.IContextLookup}
     * @implements {yfiles.lang.IPropertyObservable}
     */
    export interface LookupChain extends Object,yfiles.graph.IContextLookup,yfiles.lang.IPropertyObservable{}
    export class LookupChain {
      constructor();
      /**
       * A convenience method that adds a {@link yfiles.graph.IContextLookupChainLink} to the given chain that is based on a function of
       * type <code>function(TContext, TResult)</code>.
       * <p>
       * Depending on <code>decorateNull</code>, <code>null</code> results of the underlying chain will be passed to the
       * <code>wrapperFactory</code>, or simply yielded as the final result.
       * </p>
       * Type parameter <code>TContext</code>: The type of the context.
       * Type parameter <code>TResult</code>: The type of the result.
       * @param resultType The type of the result.
       * @param contextType The type of the context.
       * @param wrapperFactory The wrapper factory that will be called for queries to <code>TResult</code> to wrap the result of the query to the remainder of the chain.
       * @param [decorateNull=false] if set to <code>true</code> <code>null</code> return values of the remainder of the chain will be passed to the wrapper factory so that it can
       * decorate the null, otherwise null will be yielded as the final result.
       * @returns The link that has been registered with the chain.
       * @template TContext,TResult
       */
      add<TContext,TResult>(contextType:yfiles.lang.Class,resultType:yfiles.lang.Class,wrapperFactory:(arg1:TContext,arg2:TResult)=>TResult,decorateNull?:boolean):yfiles.graph.IContextLookupChainLink;
      /**
       * A convenience method that adds a {@link yfiles.graph.IContextLookupChainLink} to the given chain that is based on a function of
       * type <code>function(</code><code>TContext</code><code>)</code>.
       * <p>
       * Depending on <code>nullIsFallback</code>, <code>null</code> return values of the <code>factory</code> will be interpreted as the final result or indicate to
       * the implementation that the chain should be queried for the result.
       * </p>
       * Type parameter <code>TContext</code>: The type of the context.
       * Type parameter <code>TResult</code>: The type of the result.
       * @param resultType The type of the result.
       * @param contextType The type of the context.
       * @param factory The factory that will be called for queries to <code>TResult</code>.
       * @param [nullIsFallback=true] If set to <code>true</code>, <code>null</code> return values of the factory method will be interpreted as the final result, otherwise the request
       * will propagate down the chain.
       * @returns The link that has been registered with the chain.
       * @template TContext,TResult
       */
      add<TContext,TResult>(contextType:yfiles.lang.Class,resultType:yfiles.lang.Class,factory:(arg:TContext)=>TResult,nullIsFallback?:boolean):yfiles.graph.IContextLookupChainLink;
      /**
       * Adds the given link to the end of the chain.
       * @param lookup The link to add.
       * @see yfiles.graph.LookupChain#remove
       */
      add(lookup:yfiles.graph.IContextLookupChainLink):void;
      /**
       * Convenience method that can be used instead of the {@link yfiles.graph.LookupChain#add} method to easily add a lookup
       * implementation for a given type that always yields a constant result.
       * Type parameter <code>TResult</code>: The type of the query, which is a base type or the type of the result.
       * @param result The constant value to yield if the chain is queried for an implementation of type <code>TResult</code>.
       * @returns The {@link } instance that has been added to the chain.
       * @template TResult
       */
      addConstant<TResult>(resultType:yfiles.lang.Class,result:TResult):yfiles.graph.IContextLookupChainLink;
      /**
       * Performs the actual lookup operation.
       * <p>
       * This call will traverse the chain to satisfy the query.
       * </p>
       * @param item The item to use as the context.
       * @param type The type of the query.
       * @returns The result of the query.
       */
      contextLookup(item:Object,type:yfiles.lang.Class):Object;
      /**
       * Can be called by client code to trigger the {@link yfiles.graph.LookupChain#addPropertyChangedListener PropertyChanged}
       * event manually.
       */
      notifyChange():void;
      /**
       * Notifies client code that the lookup of this instance might have changed.
       * @param evt The arguments
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Removes the link that has previously been added to this instance from this chain.
       * @param link The link to remove.
       */
      remove(link:yfiles.graph.IContextLookupChainLink):void;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when the lookup of this instance has changed for some
       * or all of the types or items.
       * @param listener The listener to add.
       * @see yfiles.graph.LookupChain#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when the lookup of this instance has changed for
       * some or all of the types or items.
       * @param listener The listener to remove.
       * @see yfiles.graph.LookupChain#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * The main class to provide undo and redo functionality.
     * <p>
     * The {@link yfiles.graph.UndoEngine} is in charge of {@link yfiles.graph.UndoEngine#beginCompoundEdit recording}, {@link yfiles.graph.UndoEngine#addUnit managing} and {@link yfiles.graph.UndoEngine#undo executing} single units of work that can be undone
     * and redone, represented by the interface {@link yfiles.graph.IUndoUnit}.
     * </p>
     * <p>
     * These units are <b>managed</b> in a queue-like structure and can be added manually to the queue by calling the {@link yfiles.graph.UndoEngine#addUnit}
     * method.
     * </p>
     * <p>
     * <b>Executing</b> the units is done by client code or commands by calling {@link yfiles.graph.UndoEngine#undo} or
     * {@link yfiles.graph.UndoEngine#redo}. The engine automatically manages the queue so that calling these two methdos will
     * consistently move the units in the queue.
     * </p>
     * <p>
     * The {@link yfiles.graph.UndoEngine#beginCompoundEdit} method allows to <b>record</b> units by bracketing serveral changes in an
     * {@link yfiles.graph.ICompoundEdit}. Implementations of the {@link yfiles.graph.ICompoundEdit} interface record all subsequent changes
     * until it is {@link yfiles.graph.ICompoundEdit#commit committed} and therefore all recorded units are added to the engine. For more information and examples see
     * {@link yfiles.graph.ICompoundEdit}.
     * </p>
     * <p>
     * Depending on the properties {@link yfiles.graph.UndoEngine#mergeUnits} and {@link yfiles.graph.UndoEngine#autoMergeTime}, the engine tries to collapse added {@link yfiles.graph.IUndoUnit}s if possible. This enhances
     * the performance and reduces the required memory for undo changes. For example, when a node is moved interactively,
     * instead of saving every position that the node had when being dragged the engine will only hold one {@link yfiles.graph.IUndoUnit}
     * at the end to undo the entire movement.
     * </p>
     * <p>
     * <br /> <b>Customizing Undo/Redo</b>
     * <ul>
     *   <li>
     *     In general, to be able to undo or redo certain work client code can <b>implement {@link yfiles.graph.IUndoUnit} or subclass 
     *     {@link yfiles.graph.UndoUnitBase UndoUnitBase}</b>. The client application needs to encapsulate 
     *     changes in this implementation and provide the logic to completely undo and redo these changes. 
     *     Instances of your custom {@link yfiles.graph.IUndoUnit} implementations need to be added to the {@link yfiles.graph.UndoEngine} 
     *     manually via {@link yfiles.graph.UndoEngine#addUnit}.
     *   </li>
     *   <li>
     *     In certain cases it is difficult or not efficient to track and save the changes between certain states,
     *     and better to save the states themselves and to restore these. For example when there are potentially
     *     many intermediate steps between two states of interest and it would be complicated
     *     to produce {@link yfiles.graph.IUndoUnit}s for each intermediate step and merge them in the end.
     *     In such cases client code should <b>implement {@link yfiles.graph.IMementoSupport}</b> instead.
     *     See the documentation of {@link yfiles.graph.IMementoSupport} for examples and guidelines on how to implement
     *     this memento design pattern concept.
     *   </li>
     *   <li>
     *     <b>It is generally not necessary to subclass {@link yfiles.graph.UndoEngine}</b> unless you want to customize the internal
     *     handling of the units, which is both complicated and prone to errors. In most use cases it is sufficient 
     *     to provide custom {@link yfiles.graph.IUndoUnit}s or {@link yfiles.graph.IMementoSupport}.
     *   </li>
     * </ul>
     * </p>
     * @see yfiles.graph.IUndoUnit
     * @see yfiles.graph.UndoUnitBase
     * @see yfiles.graph.IMementoSupport
     * @see yfiles.graph.ICompoundEdit
     * @class
     * @implements {yfiles.lang.IPropertyObservable}
     */
    export interface UndoEngine extends Object,yfiles.lang.IPropertyObservable{}
    export class UndoEngine {
      /**
       * Initializes a new instance of the {@link yfiles.graph.UndoEngine} class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.mergeUnits A value that indicates whether or not this instance should try to merge newly added units.
       * <p>
       * This option sets the {@link yfiles.graph.UndoEngine#mergeUnits} property on the created object.
       * </p>
       * @param {yfiles.lang.TimeSpan} options.autoMergeTime The duration during which the engine will try to merge newly added units.
       * <p>
       * This option sets the {@link yfiles.graph.UndoEngine#autoMergeTime} property on the created object.
       * </p>
       * @param {number} options.size The maximum size of the undo queue this instance is managing.
       * <p>
       * This option sets the {@link yfiles.graph.UndoEngine#size} property on the created object.
       * </p>
       */
      constructor(options?:{mergeUnits?:boolean,autoMergeTime?:yfiles.lang.TimeSpan,size?:number});
      /**
       * Adds a new {@link yfiles.graph.IUndoUnit} to the queue.
       * <p>
       * This implementation will automatically group multiple units into a single unit if the time since the last add is less
       * than {@link yfiles.graph.UndoEngine#autoMergeTime}.
       * </p>
       * @param unit The unit of work to add.
       */
      addUnit(unit:yfiles.graph.IUndoUnit):void;
      /**
       * Begins a compound edit that will use the provided name.
       * <p>
       * This will create a new edit that can independently be {@link yfiles.graph.ICompoundEdit#cancel canceled} or {@link yfiles.graph.ICompoundEdit#commit committed}. Note that only if the outer-most instances
       * is committed, the corresponding {@link yfiles.graph.IUndoUnit} units will be enqueued into this instance.
       * </p>
       * @param undoName The undo name to use for the edit.
       * @param redoName The redo name to use for the edit.
       * @returns A compound edit implementation that needs to be {@link #cancel canceled} or {@link #commit committed} later.
       */
      beginCompoundEdit(undoName:string,redoName:string):yfiles.graph.ICompoundEdit;
      /**
       * Determines whether a call to {@link yfiles.graph.UndoEngine#redo} can be made.
       * @returns 
       */
      canRedo():boolean;
      /**
       * Determines whether a call to {@link yfiles.graph.UndoEngine#undo} can be made.
       * @returns 
       */
      canUndo():boolean;
      /**
       * Clears the internal queue and {@link yfiles.graph.IUndoUnit#dispose disposes} all units in it.
       */
      clear():void;
      /**
       * Returns a token that can be used to store and compare the state of the undo queue.
       * <p>
       * E.g. an application can retrieve the token once the user has saved his document. Comparing the token returned by this
       * instance with another one retrieved at a later point in time enables the application to determine whether the document
       * is in the same state.
       * </p>
       * @returns An object that can be checked against other tokens via the {@link #equals} method.
       */
      getToken():Object;
      /**
       * This will trigger the corresponding {@link yfiles.graph.UndoEngine#addPropertyChangedListener PropertyChanged} event.
       * @param name The name of the property that changed.
       * @protected
       */
      onPropertyChanged(name:string):void;
      /**
       * Redoes the next {@link yfiles.graph.IUndoUnit}.
       * @throws {Stubs.Exceptions.NotSupportedError} If an undo operation is already in progress.
       * @throws {yfiles.lang.Exception} If {@link yfiles.graph.UndoEngine#canRedo} would yield <code>false</code>.
       */
      redo():void;
      /**
       * Undoes the next {@link yfiles.graph.IUndoUnit}.
       * @throws {Stubs.Exceptions.NotSupportedError} If an undo operation is already in progress.
       * @throws {yfiles.lang.Exception} If {@link yfiles.graph.UndoEngine#canUndo} would yield <code>false</code>.
       */
      undo():void;
      /**
       * Gets or sets a value that indicates whether or not this instance should try to merge newly added units.
       * <p>
       * If <code>true</code> this instance will try to {@link yfiles.graph.IUndoUnit#tryMergeUnit merge} or {@link yfiles.graph.IUndoUnit#tryReplaceUnit replace} units in the queue. Note that when the {@link yfiles.graph.UndoEngine#autoMergeTime} property is set to {@link yfiles.lang.TimeSpan#ZERO}
       * and this property is set to <code>true</code>, the engine will always try to merge incoming units. Otherwise the {@link yfiles.graph.UndoEngine#autoMergeTime} dictates whether
       * the engine will try to merge a unit or not.
       * </p>
       * @see yfiles.graph.UndoEngine#autoMergeTime
       * @type {boolean}
       */
      mergeUnits:boolean;
      /**
       * Gets or sets the duration during which the engine will try to merge newly added units.
       * <p>
       * If this is set to {@link yfiles.lang.TimeSpan#ZERO}, the engine will try to merge incoming units depending on the {@link yfiles.graph.UndoEngine#mergeUnits} property.
       * Otherwise, if the time span between the last added unit and the new unit exceeds the set value, the engine does not try
       * to merge the units regardless what the {@link yfiles.graph.UndoEngine#mergeUnits} property is set to. Likewise, if the time span is smaller than the set value,
       * the engine will always try to merge units.
       * </p>
       * @see yfiles.graph.UndoEngine#mergeUnits
       * @type {yfiles.lang.TimeSpan}
       */
      autoMergeTime:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the maximum size of the undo queue this instance is managing.
       * <p>
       * A size of <code>0</code> effectively disables this implementation.
       * </p>
       * @type {number}
       */
      size:number;
      /**
       * Returns the {@link yfiles.graph.IUndoUnit#undoName} of the next {@link yfiles.graph.UndoEngine#undo} operation.
       * @type {string}
       */
      undoName:string;
      /**
       * Returns the {@link yfiles.graph.IUndoUnit#redoName} of the next {@link yfiles.graph.UndoEngine#redo} operation.
       * @type {string}
       */
      redoName:string;
      /**
       * Indicates whether this instance is currently performing an undo operation.
       * @type {boolean}
       */
      performingUndo:boolean;
      /**
       * Indicates whether this instance is currently performing a redo operation.
       * @type {boolean}
       */
      performingRedo:boolean;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when {@link yfiles.graph.UndoEngine#canUndo},
       * {@link yfiles.graph.UndoEngine#canRedo}, {@link yfiles.graph.UndoEngine#undoName}, or {@link yfiles.graph.UndoEngine#redoName} changed its value.
       * @param listener The listener to add.
       * @see yfiles.graph.UndoEngine#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when {@link yfiles.graph.UndoEngine#canUndo},
       * {@link yfiles.graph.UndoEngine#canRedo}, {@link yfiles.graph.UndoEngine#undoName}, or {@link yfiles.graph.UndoEngine#redoName} changed its value.
       * @param listener The listener to remove.
       * @see yfiles.graph.UndoEngine#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>UnitUndone</code> event that occurs when the engine has successfully executed the {@link yfiles.graph.IUndoUnit#undo}
       * operation of an {@link yfiles.graph.IUndoUnit}.
       * @param listener The listener to add.
       * @see yfiles.graph.UndoEngine#removeUnitUndoneListener
       */
      addUnitUndoneListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>UnitUndone</code> event that occurs when the engine has successfully executed the {@link yfiles.graph.IUndoUnit#undo}
       * operation of an {@link yfiles.graph.IUndoUnit}.
       * @param listener The listener to remove.
       * @see yfiles.graph.UndoEngine#addUnitUndoneListener
       */
      removeUnitUndoneListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>UnitRedone</code> event that occurs when the engine has successfully executed the {@link yfiles.graph.IUndoUnit#redo}
       * operation of an {@link yfiles.graph.IUndoUnit}.
       * @param listener The listener to add.
       * @see yfiles.graph.UndoEngine#removeUnitRedoneListener
       */
      addUnitRedoneListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>UnitRedone</code> event that occurs when the engine has successfully executed the {@link yfiles.graph.IUndoUnit#redo}
       * operation of an {@link yfiles.graph.IUndoUnit}.
       * @param listener The listener to remove.
       * @see yfiles.graph.UndoEngine#addUnitRedoneListener
       */
      removeUnitRedoneListener(listener:(sender:Object,evt:yfiles.lang.EventArgs)=>void):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a unit of work that can be undone and redone.
     * <p>
     * Clients should use this interface when certain actions, changes or events should be monitored and have undoability
     * provided for them. If you simply want to track the state of certain items between two states of interest, you should
     * consider using {@link yfiles.graph.IMementoSupport mementos} instead.
     * </p>
     * <p>
     * The central methods of this interface are {@link yfiles.graph.IUndoUnit#undo} and {@link yfiles.graph.IUndoUnit#redo} which contain the
     * logic to undo or redo a unit of work. Keep in mind that undo/redo are sensible operations and should maintain a
     * consistent state before and after each operation since they may be executed potentially often one after another.
     * </p>
     * <p>
     * The requirement for an {@link yfiles.graph.IUndoUnit} is that when a program is in a certain state and a call to {@link yfiles.graph.IUndoUnit#undo}
     * is followed by a call to {@link yfiles.graph.IUndoUnit#redo} then the program is in the exact same state as before (the same
     * holds true for the other way around).
     * </p>
     * <p>
     * A default implementation of this interface is the abstract class {@link yfiles.graph.UndoUnitBase} which forces only the
     * implementation of the undo/redo logic and defaults the other methods. Clients should primarily extend this class instead
     * of implementing the whole interface.
     * </p>
     * <p>
     * IUndoUnits are managed by the {@link yfiles.graph.UndoEngine}. Custom units can always be added to the engine using the method
     * {@link yfiles.graph.UndoEngine#addUnit}. Also consider to use one of the convenience implementations: {@link yfiles.graph.DelegateUndoUnit.<T>} takes delegates for the
     * undo/redo operations and {@link yfiles.graph.CompositeUndoUnit} creates a bracketing unit comprising several other units.
     * </p>
     * <p>
     * Also note that in order to keep a consistent state, methods of objects of this type should not be called by clientcode
     * directly but use the {@link yfiles.graph.UndoEngine} instead.
     * </p>
     * @interface
     */
    export interface IUndoUnit extends Object{
      /**
       * Called by the {@link yfiles.graph.UndoEngine} and client code to let the unit dispose of references aren't needed anymore.
       * <p>
       * When this method is called, the other methods of this interface will not be called anymore by the undo engine. The undo
       * engine calls this method on instances that it will not use anymore to enable the instances to release instances and
       * memory actively.
       * </p>
       * @abstract
       */
      dispose():void;
      /**
       * Redoes the work that is represented by this unit.
       * <p>
       * Undo/redo are sensible operations and should maintain a consistent state before and after each operation since they may
       * be executed potentially often one after another.
       * </p>
       * @see yfiles.graph.IUndoUnit#redoName
       * @abstract
       */
      redo():void;
      /**
       * Tries to merge the given unit with this one.
       * <p>
       * This method is called when <code>this</code> unit is the head of an {@link yfiles.graph.UndoEngine}'s or {@link yfiles.graph.CompositeUndoUnit}'s
       * queue and a new unit is added. It is meant to try to incorporate the change of the given <code>unit</code> into <code>this</code> and if
       * successful return <code>true</code>. This should be the case when the end state of <code>this</code> unit is equal to the start state of the given
       * <code>unit</code>.
       * </p>
       * <p>
       * For example, if <code>this</code> unit is the head of the queue and represents the work from state <code>A</code> to state <code>B</code> and the given unit
       * the work from state <code>B</code> to state <code>C</code>, then this method should try to make <code>this</code> unit shift from state <code>A</code> to state <code>C</code>. If this
       * is successful, the method is expected to return <code>true</code>. The other unit is then {@link yfiles.graph.IUndoUnit#dispose disposed} by the {@link yfiles.graph.UndoEngine}
       * afterwards.
       * </p>
       * <p>
       * Clients don't necessarily have to implement this method if the unit doesn't happen very often. In fact, the default
       * implementation of {@link yfiles.graph.UndoUnitBase#tryMergeUnit} simply returns <code>false</code>. Implementing this method faithfully will
       * cause multiple units of work to inseparably appear as one and undoing/redoing it will undo/redo the work of all
       * collapsed units. Depending on the situation this may be reasonable, for example when there are potentially a lot of
       * changes where not every intermediate step is required to be recorded. In this case implementing this method faithfully
       * will greatly improve the performance and reduce the required amount of memory of the undo process. If you want to group
       * together multiple units as a single block but still want to be able to separate each step, consider to use a {@link yfiles.graph.CompositeUndoUnit}
       * instead.
       * </p>
       * @param unit The unit to incorporate that happened after this unit.
       * @returns Whether the state change of <code>unit</code> has been incorporated into this unit and <code>unit</code> can be disposed of.
       * @abstract
       */
      tryMergeUnit(unit:yfiles.graph.IUndoUnit):boolean;
      /**
       * Tries to replace the given unit with this one.
       * <p>
       * This method is called if a newly added unit couldn't be {@link yfiles.graph.IUndoUnit#tryMergeUnit merged} with the head unit of the queue (i.e. returned false).
       * Instead of trying to merge a newly added unit, this method tries to replace the given unit (which is the head of the
       * queue) with <code>this</code> by incorporating the change of the given <code>unit</code> into <code>this</code> and if successful return <code>true</code>. This should be
       * the case when the start state of <code>this</code> unit is equal to the end state of the given <code>unit</code>.
       * </p>
       * <p>
       * For example, if the given <code>unit</code> represents the work from state <code>A</code> to state <code>B</code> and <code>this</code> unit the work from state <code>B</code> to state
       * <code>C</code>, then this method should try to make <code>this</code> unit shift from state <code>A</code> to state <code>C</code>. If this is successful, the method is
       * expected to return <code>true</code>. The given <code>unit</code> is then {@link yfiles.graph.IUndoUnit#dispose disposed} by the {@link yfiles.graph.UndoEngine} afterwards and replaced with <code>this</code>
       * unit in the queue.
       * </p>
       * <p>
       * Clients don't necessarily have to implement this method if the unit doesn't happen very often. In fact, the default
       * implementation of {@link yfiles.graph.UndoUnitBase#tryReplaceUnit} simply returns <code>false</code>. Implementing this method faithfully
       * will cause multiple units of work to inseparably appear as one and undoing/redoing it will undo/redo the work of all
       * collapsed units. Depending on the situation this may be reasonable, for example when there are potentially a lot of
       * changes where not every intermediate step is required to be recorded. In this case implementing this method faithfully
       * will greatly improve the performance and reduce the required amount of memory of the undo process. If you want to group
       * together multiple units as a single block but still want to be able to separate each step, consider to use a {@link yfiles.graph.CompositeUndoUnit}
       * instead.
       * </p>
       * @param unit The unit to incorporate that happened before this unit.
       * @returns Whether the state change of <code>this</code> unit has been incorporated into the given <code>unit</code> and <code>this</code> can be disposed of.
       * @abstract
       */
      tryReplaceUnit(unit:yfiles.graph.IUndoUnit):boolean;
      /**
       * Undoes the work that is represented by this unit.
       * <p>
       * Undo/redo are sensible operations and should maintain a consistent state before and after each operation since they may
       * be executed potentially often one after another.
       * </p>
       * @see yfiles.graph.IUndoUnit#undoName
       * @abstract
       */
      undo():void;
      /**
       * Returns the name of the undo unit.
       * <p>
       * Depending on the implementation and context this might be a human readable representation of the undo action or a
       * symbolic name that needs localization.
       * </p>
       * @abstract
       * @type {string}
       */
      undoName:string;
      /**
       * Returns the name of the redo unit.
       * <p>
       * Depending on the implementation and context this might be a human readable representation of the redo action or a
       * symbolic name that needs localization.
       * </p>
       * @abstract
       * @type {string}
       */
      redoName:string;
    }
    var IUndoUnit:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides access to the memento design pattern to provide undoability for arbitrary models.
     * <p>
     * By implementing this interface as well as {@link yfiles.graph.ILookup}, clients can add undoability for changes to their model
     * classes. The yFiles undo mechanism uses the return value of {@link yfiles.graph.IMementoSupport#getState} to retrieve a state of
     * an item at the beginning of the compound editing process. When the process ends, another state will be retrieved of the
     * same item and compared to the original state. If they differ, an {@link yfiles.graph.IUndoUnit} is created that uses the {@link yfiles.graph.IMementoSupport#applyState}
     * method to apply either state to the item in case of {@link yfiles.graph.IUndoUnit#undo undo} or {@link yfiles.graph.IUndoUnit#redo redo}. This represents an abstraction to the undo mechanism
     * where it is only needed to define "states" of items and hides the more complicated mechanism of creating and inserting
     * {@link yfiles.graph.IUndoUnit}s.
     * </p>
     * <p>
     * The following is an example implementation of an item that is being managed using {@link yfiles.graph.IMementoSupport}:
     * </p>
     * <p>
     * A collection of items from this type can then be watched using the following code snippet:
     * </p>
     * <p>
     * Implementing the {@link yfiles.graph.IMementoSupport} interface is quite unrestrained, the type of the state returned by {@link yfiles.graph.IMementoSupport#getState}
     * method can by anything as long as the {@link yfiles.graph.IMementoSupport#applyState} and {@link yfiles.graph.IMementoSupport#stateEquals}
     * methods can deal with it:
     * </p>
     * <p>
     * In summary, use this concept when you want to track the state of items during certain operations for undo/redo. This is
     * efficient if it's easier to handle an item's state than the changes to the item themselves. If you want to focus on the
     * changes or on certain events, you should use custom {@link yfiles.graph.IUndoUnit} implementations instead.
     * </p>
     * @see yfiles.graph.ILookup
     * @see yfiles.graph.ICompoundEdit
     * @see yfiles.graph.UndoEngine
     * @interface
     */
    export interface IMementoSupport extends Object{
      /**
       * Reapplies a previously queried state object to a given subject.
       * <p>
       * The state object has been created using the {@link yfiles.graph.IMementoSupport#getState} method.
       * </p>
       * @param subject The subject to modify the state.
       * @param state The state object as returned by {@link #getState}
       * @abstract
       */
      applyState(subject:Object,state:Object):void;
      /**
       * Retrieves an object representing the state at the moment this method is called.
       * <p>
       * The returned state may be reapplied later to the subject in the {@link yfiles.graph.IMementoSupport#applyState} method.
       * </p>
       * @param subject The subject to read the state from
       * @returns An object that describes the current state of <code>subject</code>.
       * @abstract
       */
      getState(subject:Object):Object;
      /**
       * Determines whether two state objects are equal.
       * <p>
       * The undo mechanism in yFiles calls this method to determine if the subject has changed and whether an {@link yfiles.graph.IUndoUnit}
       * should be created for these changes. If this method returns <code>false</code>, an {@link yfiles.graph.IUndoUnit} is created for the two
       * states that can reapply either state (for either undo or redo). If this method returns <code>true</code> the state is considered not
       * to have changed and no undo unit will be created. Conservative implementations my simply return <code>false</code>.
       * </p>
       * @param state1 The first state as obtained from {@link #getState}
       * @param state2 The second state as obtained from {@link #getState}
       * @returns <code>true</code> if the states are equal; <code>false</code> otherwise.
       * @abstract
       */
      stateEquals(state1:Object,state2:Object):boolean;
    }
    var IMementoSupport:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A convenience implementation of the {@link yfiles.graph.IUndoUnit} interface that uses simple delegates for the actual undo and
     * redo operations.
     * Type parameter <code>T</code>: The type of the single parameter that is passed to the delegate.
     * @class
     * @extends {yfiles.graph.UndoUnitBase}
     * @template T
     * @final
     */
    export interface DelegateUndoUnit<T> extends yfiles.graph.UndoUnitBase{}
    export class DelegateUndoUnit<T> {
      /**
       * Creates a new instance using the provided name and a undo and redo handler as well as the parameters to pass to the
       * delegates.
       * @param undoName The name of the undo.
       * @param undo The undo handler delegate.
       * @param redo The redo handler delegate.
       * @param undoParam The undo parameter to pass to the delegate.
       * @param redoParam The redo parameter to pass to the delegate.
       */
      constructor(undoName:string,undo:(param:T)=>void,redo:(param:T)=>void,undoParam:T,redoParam:T);
      /**
       * Creates a new instance using the provided name and a undo/redo handler as well as the parameters to pass to the
       * delegates.
       * @param undoName The name of the undo.
       * @param undoRedo The undo and redo handler delegate.
       * @param undoParam The undo parameter to pass to the delegate.
       * @param redoParam The redo parameter to pass to the delegate.
       */
      constructor(undoName:string,undoRedo:(param:T)=>void,undoParam:T,redoParam:T);
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract convenience implementation of {@link yfiles.graph.IUndoUnit}.
     * <p>
     * This class defaults most methods required by the {@link yfiles.graph.IUndoUnit} interface. Subclasses only have to implement the
     * undo/redo logic. Most notably, {@link yfiles.graph.IUndoUnit#tryMergeUnit} and {@link yfiles.graph.IUndoUnit#tryReplaceUnit} are
     * implemented to return
     * <code>false</code>. Subclasses are of course allowed to override more methods if clients want to enable additional functionality.
     * </p>
     * <p>
     * Use this class for simple custom implementations of {@link yfiles.graph.IUndoUnit}.
     * </p>
     * @see yfiles.graph.IUndoUnit
     * @see yfiles.graph.UndoEngine
     * @class
     * @implements {yfiles.graph.IUndoUnit}
     */
    export interface UndoUnitBase extends Object,yfiles.graph.IUndoUnit{}
    export class UndoUnitBase {
      /**
       * Creates a new instance using the given name as undo and redo name.
       * @param undoName The undo name.
       * @param [redoName=null] The redo name.
       * @protected
       */
      constructor(undoName:string,redoName?:string);
      /**
       * 
       */
      dispose():void;
      /**
       * 
       * @abstract
       */
      redo():void;
      /**
       * 
       * @param unit
       * @returns 
       */
      tryMergeUnit(unit:yfiles.graph.IUndoUnit):boolean;
      /**
       * 
       * @param unit
       * @returns 
       */
      tryReplaceUnit(unit:yfiles.graph.IUndoUnit):boolean;
      /**
       * 
       * @abstract
       */
      undo():void;
      /**
       * 
       * @type {string}
       */
      undoName:string;
      /**
       * 
       * @type {string}
       */
      redoName:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract basic implementation of the {@link yfiles.graph.IFoldingEdgeConverter} interface that may be derived from to create
     * a customized {@link yfiles.graph.FoldingManager#foldingEdgeConverter}.
     * <p>
     * This class provides default mechanisms and callbacks that can be used to set another style for folding edges, for the
     * ports of folding edges, synchronizing the first label of a folding edge with the master edge, etc. Subclasses need to
     * implement the {@link yfiles.graph.FoldingEdgeConverterBase#addFoldingEdge} method, only.
     * </p>
     * @see yfiles.graph.DefaultFoldingEdgeConverter
     * @see yfiles.graph.FoldingManager
     * @class
     * @implements {yfiles.graph.IFoldingEdgeConverter}
     */
    export interface FoldingEdgeConverterBase extends Object,yfiles.graph.IFoldingEdgeConverter{}
    export class FoldingEdgeConverterBase {
      /**
       * 
       * @param factory
       * @param foldingView
       * @param masterEdge
       * @param source
       * @param sourceIsCollapsed
       * @param target
       * @param targetIsCollapsed
       * @returns 
       * @abstract
       */
      addFoldingEdge(factory:yfiles.graph.IFoldingEdgeFactory,foldingView:yfiles.graph.IFoldingView,masterEdge:yfiles.graph.IEdge,source:yfiles.graph.INode,sourceIsCollapsed:boolean,target:yfiles.graph.INode,targetIsCollapsed:boolean):yfiles.graph.IEdge;
      /**
       * Callback factory method that creates the {@link yfiles.styles.IEdgeStyle} for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgeStyle}.
       * @param foldingView The folding view instance.
       * @param foldingEdge The folding edge in the view whose {@link #style} should be initialized.
       * @param masterEdges The edges that are being represented by the folding edge.
       * @returns The {@link #foldingEdgeStyle} or a {@link #clone} of it depending on the {@link #cloneEdgeStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createEdgeStyle(foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):yfiles.styles.IEdgeStyle;
      /**
       * Callback factory method that creates the {@link yfiles.graph.ILabelModelParameter} for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgeLabels}.
       * @param foldingView The folding view instance.
       * @param localLabel The local label in the view whose {@link #layoutParameter} should be initialized. May be <code>null</code> if the label layout parameter is requested for the
       * label creation.
       * @param masterLabel The label that is being represented by the folding label.
       * @returns The {@link #labelLayoutParameter} or a {@link #clone} of it depending on the {@link #cloneLabelLayoutParameter} property. This method may return <code>null</code> to indicate that
       * the default label layout parameter should not be changed.
       * @protected
       */
      createLabelLayoutParameter(foldingView:yfiles.graph.IFoldingView,localLabel:yfiles.graph.ILabel,masterLabel:yfiles.graph.ILabel):yfiles.graph.ILabelModelParameter;
      /**
       * Callback factory method that creates the {@link yfiles.styles.ILabelStyle} for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgeLabels}.
       * @param foldingView The folding view instance.
       * @param localLabel The local label in the view whose {@link #style} should be initialized. May be <code>null</code> if the label style is requested for the label
       * creation.
       * @param masterLabel The label that is being represented by the folding label.
       * @returns The {@link #labelStyle} or a {@link #clone} of it depending on the {@link #cloneLabelStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createLabelStyle(foldingView:yfiles.graph.IFoldingView,localLabel:yfiles.graph.ILabel,masterLabel:yfiles.graph.ILabel):yfiles.styles.ILabelStyle;
      /**
       * Callback factory method that creates the {@link yfiles.graph.IPortLocationModelParameter} of the source port for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
       * @param foldingView The folding view instance.
       * @param localEdge The local edge in the view whose source port's {@link #locationParameter} should be initialized.
       * @param masterEdges The master edges that the folding edge represents.
       * @returns The {@link #sourcePortLocationParameter} or a {@link #clone} of it depending on the {@link #cloneSourcePortLocationParameter} property. This method may return <code>null</code> to indicate that
       * the default port location parameter should not be changed.
       * @protected
       */
      createSourcePortLocationParameter(foldingView:yfiles.graph.IFoldingView,localEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):yfiles.graph.IPortLocationModelParameter;
      /**
       * Callback factory method that creates the {@link yfiles.styles.IPortStyle} of the source port for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
       * @param foldingView The folding view instance.
       * @param localEdge The local edge in the view whose source port's {@link #style} should be initialized.
       * @param masterEdges The master edges that the folding edge represents.
       * @returns The {@link #sourcePortStyle} or a {@link #clone} of it depending on the {@link #cloneSourcePortStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createSourcePortStyle(foldingView:yfiles.graph.IFoldingView,localEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):yfiles.styles.IPortStyle;
      /**
       * Callback factory method that creates the {@link yfiles.graph.IPortLocationModelParameter} of the target port for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
       * @param foldingView The folding view instance.
       * @param localEdge The local edge in the view whose target port's {@link #locationParameter} should be initialized.
       * @param masterEdges The master edges that the folding edge represents.
       * @returns The {@link #targetPortLocationParameter} or a {@link #clone} of it depending on the {@link #cloneTargetPortLocationParameter} property. This method may return <code>null</code> to indicate that
       * the default port location parameter should not be changed.
       * @protected
       */
      createTargetPortLocationParameter(foldingView:yfiles.graph.IFoldingView,localEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):yfiles.graph.IPortLocationModelParameter;
      /**
       * Callback factory method that creates the {@link yfiles.styles.IPortStyle} of the target port for use in
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgePorts}.
       * @param foldingView The folding view instance.
       * @param localEdge The local edge in the view whose target port's {@link #style} should be initialized.
       * @param masterEdges The master edges that the folding edge represents.
       * @returns The {@link #targetPortStyle} or a {@link #clone} of it depending on the {@link #cloneTargetPortStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createTargetPortStyle(foldingView:yfiles.graph.IFoldingView,localEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):yfiles.styles.IPortStyle;
      /**
       * Callback method that initializes the bends of the folding edge.
       * <p>
       * This will call the {@link yfiles.graph.FoldingEdgeState#clearBends} callback if {@link yfiles.graph.FoldingEdgeConverterBase#resetBends} is enabled.
       * </p>
       * @param state The viewstate to change.
       * @param foldingView The folding view instance that has triggered the {@link #initializeFoldingEdgeState} call.
       * @param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
       * @param masterEdges The master edges that the folding edge represents.
       * @protected
       */
      initializeFoldingEdgeBends(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Callback method that initializes the initial labels of the folding edge.
       * <p>
       * This will first {@link yfiles.graph.FoldingLabelOwnerState#clearLabels clear all the labels} and then try to copy the first label of the single master edge if {@link yfiles.graph.FoldingEdgeConverterBase#copyFirstLabel} is enabled. It
       * will call {@link yfiles.graph.FoldingEdgeConverterBase#createLabelStyle} and {@link yfiles.graph.FoldingEdgeConverterBase#createLabelLayoutParameter}
       * and use the returned style and layout parameter if they are non-<code>null</code> when {@link yfiles.graph.FoldingEdgeState#addLabel adding the label} on the folding edge.
       * </p>
       * @param state The viewstate to change.
       * @param foldingView The folding view instance that has triggered the {@link #initializeFoldingEdgeState} call.
       * @param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
       * @param masterEdges The master edges that the folding edge represents.
       * @see yfiles.graph.FoldingEdgeConverterBase#labelStyle
       * @see yfiles.graph.FoldingEdgeConverterBase#labelLayoutParameter
       * @protected
       */
      initializeFoldingEdgeLabels(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Callback method that initializes the ports of the folding edge.
       * <p>
       * If the source port of the <code>foldingEdge</code> {@link yfiles.graph.IFoldingView#isInFoldingState is in folding state}, this implementation calls {@link yfiles.graph.FoldingEdgeConverterBase#createSourcePortStyle}
       * and {@link yfiles.graph.FoldingEdgeConverterBase#createSourcePortLocationParameter} and {@link yfiles.graph.FoldingPortState#style assigns the style} and {@link yfiles.graph.FoldingPortState#locationParameter location parameter}
       * to the folding source port if non-<code>null</code> values have been returned. The same holds true for the target port where the {@link yfiles.graph.FoldingEdgeConverterBase#createTargetPortStyle}
       * and {@link yfiles.graph.FoldingEdgeConverterBase#createTargetPortLocationParameter} callbacks are used.
       * </p>
       * @param state The viewState to change.
       * @param foldingView The folding view instance that has triggered the {@link #initializeFoldingEdgeState} call.
       * @param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
       * @param masterEdges The master edges that the folding edge represents.
       * @see yfiles.graph.FoldingEdgeConverterBase#sourcePortStyle
       * @see yfiles.graph.FoldingEdgeConverterBase#targetPortStyle
       * @see yfiles.graph.FoldingEdgeConverterBase#sourcePortLocationParameter
       * @see yfiles.graph.FoldingEdgeConverterBase#targetPortLocationParameter
       * @see yfiles.graph.FoldingEdgeConverterBase#createSourcePortStyle
       * @see yfiles.graph.FoldingEdgeConverterBase#createTargetPortStyle
       * @see yfiles.graph.FoldingEdgeConverterBase#createSourcePortLocationParameter
       * @see yfiles.graph.FoldingEdgeConverterBase#createTargetPortLocationParameter
       * @protected
       */
      initializeFoldingEdgePorts(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Implements the {@link yfiles.graph.IFoldingEdgeConverter#initializeFoldingEdgeState} method and initializes the folding edge
       * appearance.
       * <p>
       * This method calls {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgeStyle},
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgePorts},
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgeBends},
       * {@link yfiles.graph.FoldingEdgeConverterBase#initializeFoldingEdgeLabels}.
       * </p>
       * @param state
       * @param foldingView
       * @param foldingEdge
       * @param masterEdges
       */
      initializeFoldingEdgeState(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Callback method that initializes the {@link yfiles.graph.FoldingEdgeState#style style property} of the folding edge.
       * <p>
       * This implementation calls {@link yfiles.graph.FoldingEdgeConverterBase#createEdgeStyle} and {@link yfiles.graph.FoldingEdgeState#style assigns} the style to the folding
       * edge if a non-<code>null</code> value has been returned.
       * </p>
       * @param state The viewstate to change.
       * @param foldingView The folding view instance that has triggered the {@link #initializeFoldingEdgeState} call.
       * @param foldingEdge The folding edge in the view that may be changed using the <code>state</code>.
       * @param masterEdges The master edges that the folding edge represents.
       * @protected
       */
      initializeFoldingEdgeStyle(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Called by {@link yfiles.graph.FoldingEdgeConverterBase#updateFoldingEdgeState} to synchronize the first label if {@link yfiles.graph.FoldingEdgeConverterBase#copyFirstLabel} is enabled.
       * <p>
       * This will adjust the label's text, preferred size and tag properties or remove the label if there is no master label or
       * there is more than one master edge.
       * </p>
       * @param state The viewstate of the edge to change.
       * @param foldingView The folding view.
       * @param foldingEdge The folding edge.
       * @param masterEdges The master edges.
       * @protected
       */
      synchronizeLabels(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Implements the {@link yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState} method and changes the folding edge
       * appearance.
       * <p>
       * This method calls {@link yfiles.graph.FoldingEdgeConverterBase#synchronizeLabels}, only.
       * </p>
       * @param state
       * @param foldingView
       * @param foldingEdge
       * @param masterEdges
       */
      updateFoldingEdgeState(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#foldingEdgeStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneEdgeStyle:boolean;
      /**
       * Gets or sets the {@link yfiles.styles.IPortStyle} to use for the folding source port.
       * <p>
       * A value of <code>null</code> will leave the style property unmodified.
       * </p>
       * @type {yfiles.styles.IPortStyle}
       */
      sourcePortStyle:yfiles.styles.IPortStyle;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#sourcePortStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone clone} or not.
       * @type {boolean}
       */
      cloneSourcePortStyle:boolean;
      /**
       * Gets or sets the {@link yfiles.styles.IPortStyle} to use for the folding target port.
       * <p>
       * A value of <code>null</code> will leave the style property unmodified.
       * </p>
       * @type {yfiles.styles.IPortStyle}
       */
      targetPortStyle:yfiles.styles.IPortStyle;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#targetPortStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneTargetPortStyle:boolean;
      /**
       * Gets or sets the {@link yfiles.graph.IPortLocationModelParameter} to use for the source port of the folding edge.
       * <p>
       * Since the appearance of the folding edge may totally differ from the appearance of the master edge, the location of the
       * source port may also be different. This property can be used to assign a different specific location parameter to the
       * source port if the edge {@link yfiles.graph.IFoldingView#isInFoldingState is in folding state}. A value of <code>null</code> will leave the location parameter property unmodified. The
       * default is <code>null</code>
       * </p>
       * @see yfiles.graph.FoldingEdgeConverterBase#createSourcePortLocationParameter
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      sourcePortLocationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#sourcePortLocationParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneSourcePortLocationParameter:boolean;
      /**
       * Gets or sets the {@link yfiles.graph.IPortLocationModelParameter} to use for the target port of the folding edge.
       * <p>
       * Since the appearance of the folding edge may totally differ from the appearance of the master edge, the location of the
       * target port may also be different. This property can be used to assign a different specific location parameter to the
       * target port if the edge {@link yfiles.graph.IFoldingView#isInFoldingState is in folding state}. A value of <code>null</code> will leave the location parameter property unmodified. The
       * default is <code>null</code>
       * </p>
       * @see yfiles.graph.FoldingEdgeConverterBase#createTargetPortLocationParameter
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      targetPortLocationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#targetPortLocationParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneTargetPortLocationParameter:boolean;
      /**
       * Gets or sets the {@link yfiles.styles.ILabelStyle} to use for the first label of the folding edge.
       * <p>
       * This will only affect the folding edge if the {@link yfiles.graph.FoldingEdgeConverterBase#copyFirstLabel} property is set to <code>true</code>.
       * </p>
       * <p>
       * A value of <code>null</code> will leave the style property unmodified.
       * </p>
       * @type {yfiles.styles.ILabelStyle}
       */
      labelStyle:yfiles.styles.ILabelStyle;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#labelStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneLabelStyle:boolean;
      /**
       * Gets or sets the {@link yfiles.graph.ILabelModelParameter} to use for the first label of the folding edge.
       * <p>
       * This will only affect the folding edge if the {@link yfiles.graph.FoldingEdgeConverterBase#copyFirstLabel} property is set to <code>true</code>.
       * </p>
       * <p>
       * A value of <code>null</code> will leave the label model parameter property unmodified.
       * </p>
       * @type {yfiles.graph.ILabelModelParameter}
       */
      labelLayoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.FoldingEdgeConverterBase#labelLayoutParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneLabelLayoutParameter:boolean;
      /**
       * Gets or sets the {@link yfiles.styles.IEdgeStyle} to use for the folding edge.
       * <p>
       * Depending on the {@link yfiles.graph.FoldingEdgeConverterBase#cloneEdgeStyle} property this implementation will assign a {@link yfiles.lang.ICloneable#clone} to the property or this
       * instance.
       * </p>
       * <p>
       * A value of <code>null</code> (the default) will leave the style property unmodified.
       * </p>
       * @type {yfiles.styles.IEdgeStyle}
       */
      foldingEdgeStyle:yfiles.styles.IEdgeStyle;
      /**
       * Gets or sets a value indicating whether to reset and clear the bends of a folding edge initially.
       * @type {boolean}
       */
      resetBends:boolean;
      /**
       * Gets or sets a value indicating whether the first label of the {@link yfiles.graph.IFoldingView#getMasterItem master edge} should be recreated for the folding edge.
       * <p>
       * This setting can be used to initially create a copy of the first label of the master edge (if any) and subsequently
       * synchronize the {@link yfiles.graph.ILabel#text} property with the master's edge label text.
       * </p>
       * @see yfiles.graph.FoldingEdgeConverterBase#labelStyle
       * @see yfiles.graph.FoldingEdgeConverterBase#labelLayoutParameter
       * @type {boolean}
       */
      copyFirstLabel:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The default implementation of the {@link yfiles.graph.IFoldingEdgeConverter} that is used by the {@link yfiles.graph.FoldingManager}
     * initially and can be customized to suit the application's needs.
     * @see yfiles.graph.FoldingEdgeConverterBase
     * @see yfiles.graph.DefaultFoldingEdgeConverter#reuseMasterPorts
     * @class
     * @extends {yfiles.graph.FoldingEdgeConverterBase}
     */
    export interface DefaultFoldingEdgeConverter extends yfiles.graph.FoldingEdgeConverterBase{}
    export class DefaultFoldingEdgeConverter {
      constructor();
      /**
       * Gets or sets a property that determines whether this implementation should reuse the ports of the {@link yfiles.graph.FoldingManager#masterGraph} at non-{@link yfiles.graph.IFoldingView#isInFoldingState folder} nodes
       * for the folding edges.
       * <p>
       * The {@link yfiles.graph.IFoldingView view} can be configured to use separate port instances for folding edges. This makes it possible to assign different
       * styles and locations to source and target ports of folding edges, which may be desirable if the style and geometry of
       * folding edges differs a lot from the style of the master edges. Folding edges may of course still connect to non-folder
       * nodes at one of their ports. Setting this property to <code>true</code> will make the folding edge connect to the {@link yfiles.graph.IFoldingView#getViewItem view} port instance.
       * This property can be set to <code>true</code>, iff the graph model used is port-centric, i.e. each edge connects to specific ports
       * and even folding edges should still connect to those specific view ports. The default value is <code>false</code>
       * </p>
       * @see yfiles.graph.DefaultFoldingEdgeConverter#reuseFolderNodePorts
       * @type {boolean}
       */
      reuseMasterPorts:boolean;
      /**
       * Gets or sets a property that determines whether this implementation should reuse the ports of {@link yfiles.graph.IFoldingView#isInFoldingState folder} nodes for the
       * folding edges.
       * <p>
       * The {@link yfiles.graph.IFoldingView view} can be configured to use separate port instances for folding edges. This makes it possible to assign different
       * styles and locations to source and target ports of folding edges, which may be desirable if the style and geometry of
       * folding edges differs a lot from the style of the master edges. Folder nodes ({@link yfiles.graph.IFoldingView#collapse collapsed} group nodes), will have all of
       * their master's ports {@link yfiles.graph.IFoldingView#getViewItem represented} in the view. This property can be used to make folding edges that connect to the master
       * ports connect to these folding port instances instead of creating extra folding ports for the edges. Note that ports can
       * only be automatically reused by folding edges if they still connect to the same nodes. Otherwise the view will have to
       * create new folding ports to let the edges connect to the representing adjacent nodes. This property can be set to <code>true</code>,
       * iff the graph model used is port-centric, i.e. each edge connects to specific ports and even folding edges should still
       * connect to those specific port instances, if possible. The default value is <code>false</code>.
       * </p>
       * @see yfiles.graph.DefaultFoldingEdgeConverter#reuseMasterPorts
       * @type {boolean}
       */
      reuseFolderNodePorts:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Simple default implementation of the {@link yfiles.graph.IFolderNodeConverter} interface that is used by the
     * {@link yfiles.graph.FoldingManager}'s {@link yfiles.graph.FoldingManager#folderNodeConverter} property.
     * <p>
     * This class has a couple of properties that can be adjusted to customize the behavior. Also, it contains a number of
     * callback methods for conveniently overriding the implementation in custom sub classes.
     * </p>
     * @see yfiles.graph.FoldingManager
     * @see yfiles.graph.IFolderNodeConverter
     * @see yfiles.graph.DefaultFoldingEdgeConverter
     * @class
     * @implements {yfiles.graph.IFolderNodeConverter}
     */
    export interface DefaultFolderNodeConverter extends Object,yfiles.graph.IFolderNodeConverter{}
    export class DefaultFolderNodeConverter {
      /**
       * Initializes a new instance of this class using the values provided by the <code>folderNodeDefaults</code> parameter.
       * @param folderNodeDefaults
       */
      constructor(folderNodeDefaults:yfiles.graph.INodeDefaults);
      /**
       * Initializes a new instance of this class using the values provided by the <code>folderNodeDefaults</code> parameter.
       * @param {Object} options The parameters to pass.
       * @param options.folderNodeDefaults
       * @param {boolean} options.cloneLabelLayoutParameter A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#cloneLabelLayoutParameter} property on the created object.
       * </p>
       * @param {boolean} options.cloneLabelStyle A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#labelStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#cloneLabelStyle} property on the created object.
       * </p>
       * @param {boolean} options.cloneNodeStyle A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#cloneNodeStyle} property on the created object.
       * </p>
       * @param {boolean} options.clonePortStyle A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#portStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#clonePortStyle} property on the created object.
       * </p>
       * @param {boolean} options.copyFirstLabel A value indicating whether the first label of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} should be recreated for the collapsed group node instance.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.folderNodeSize The initial size of the {@link yfiles.graph.INode#layout} of the collapsed group node that will be assigned during the {@link yfiles.graph.IFolderNodeConverter#initializeFolderNodeState creation} of the appearance of the collapsed group node.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeSize} property on the created object.
       * </p>
       * @param {yfiles.styles.INodeStyle} options.folderNodeStyle The {@link yfiles.styles.INodeStyle} to use for the {@link yfiles.graph.IFoldingView#collapse collapsed} group nodes.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeStyle} property on the created object.
       * </p>
       * @param {yfiles.graph.ILabelModelParameter} options.labelLayoutParameter The {@link yfiles.graph.ILabelModelParameter} to use for the first label of the collapsed group node.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter} property on the created object.
       * </p>
       * @param {yfiles.styles.ILabelStyle} options.labelStyle The {@link yfiles.styles.ILabelStyle} to use for the first label of the collapsed group nodes.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#labelStyle} property on the created object.
       * </p>
       * @param {yfiles.graph.IPortLocationModelParameter} options.portLocationParameter The {@link yfiles.graph.IPortLocationModelParameter} to use for the ports at the collapsed group node that {@link yfiles.graph.IFoldingView#getViewItem represent} the ports of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} in the collapsed state.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#portLocationParameter} property on the created object.
       * </p>
       * @param {yfiles.styles.IPortStyle} options.portStyle The {@link yfiles.styles.IPortStyle} to use for the ports at the collapsed group node that {@link yfiles.graph.IFoldingView#getViewItem represent} the ports of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} in the collapsed state.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#portStyle} property on the created object.
       * </p>
       */
      constructor(options:{folderNodeDefaults:yfiles.graph.INodeDefaults,cloneLabelLayoutParameter?:boolean,cloneLabelStyle?:boolean,cloneNodeStyle?:boolean,clonePortStyle?:boolean,copyFirstLabel?:boolean,folderNodeSize?:yfiles.geometry.Size,folderNodeStyle?:yfiles.styles.INodeStyle,labelLayoutParameter?:yfiles.graph.ILabelModelParameter,labelStyle?:yfiles.styles.ILabelStyle,portLocationParameter?:yfiles.graph.IPortLocationModelParameter,portStyle?:yfiles.styles.IPortStyle});
      /**
       * Initializes a new instance of this class with default values.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.cloneLabelLayoutParameter A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#cloneLabelLayoutParameter} property on the created object.
       * </p>
       * @param {boolean} options.cloneLabelStyle A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#labelStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#cloneLabelStyle} property on the created object.
       * </p>
       * @param {boolean} options.cloneNodeStyle A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#cloneNodeStyle} property on the created object.
       * </p>
       * @param {boolean} options.clonePortStyle A value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#portStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#clonePortStyle} property on the created object.
       * </p>
       * @param {boolean} options.copyFirstLabel A value indicating whether the first label of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} should be recreated for the collapsed group node instance.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.folderNodeSize The initial size of the {@link yfiles.graph.INode#layout} of the collapsed group node that will be assigned during the {@link yfiles.graph.IFolderNodeConverter#initializeFolderNodeState creation} of the appearance of the collapsed group node.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeSize} property on the created object.
       * </p>
       * @param {yfiles.styles.INodeStyle} options.folderNodeStyle The {@link yfiles.styles.INodeStyle} to use for the {@link yfiles.graph.IFoldingView#collapse collapsed} group nodes.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeStyle} property on the created object.
       * </p>
       * @param {yfiles.graph.ILabelModelParameter} options.labelLayoutParameter The {@link yfiles.graph.ILabelModelParameter} to use for the first label of the collapsed group node.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter} property on the created object.
       * </p>
       * @param {yfiles.styles.ILabelStyle} options.labelStyle The {@link yfiles.styles.ILabelStyle} to use for the first label of the collapsed group nodes.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#labelStyle} property on the created object.
       * </p>
       * @param {yfiles.graph.IPortLocationModelParameter} options.portLocationParameter The {@link yfiles.graph.IPortLocationModelParameter} to use for the ports at the collapsed group node that {@link yfiles.graph.IFoldingView#getViewItem represent} the ports of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} in the collapsed state.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#portLocationParameter} property on the created object.
       * </p>
       * @param {yfiles.styles.IPortStyle} options.portStyle The {@link yfiles.styles.IPortStyle} to use for the ports at the collapsed group node that {@link yfiles.graph.IFoldingView#getViewItem represent} the ports of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} in the collapsed state.
       * <p>
       * This option sets the {@link yfiles.graph.DefaultFolderNodeConverter#portStyle} property on the created object.
       * </p>
       */
      constructor(options?:{cloneLabelLayoutParameter?:boolean,cloneLabelStyle?:boolean,cloneNodeStyle?:boolean,clonePortStyle?:boolean,copyFirstLabel?:boolean,folderNodeSize?:yfiles.geometry.Size,folderNodeStyle?:yfiles.styles.INodeStyle,labelLayoutParameter?:yfiles.graph.ILabelModelParameter,labelStyle?:yfiles.styles.ILabelStyle,portLocationParameter?:yfiles.graph.IPortLocationModelParameter,portStyle?:yfiles.styles.IPortStyle});
      /**
       * Callback factory method that creates the {@link yfiles.graph.ILabelModelParameter} for use in
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeLabels}.
       * @param foldingView The folding view instance.
       * @param localLabel The local label in the view whose {@link #layoutParameter} should be initialized. May be <code>null</code> if the label layout parameter is requested for the
       * label creation.
       * @param masterLabel The label that is being represented by the folding label.
       * @returns The {@link #labelLayoutParameter} or a {@link #clone} of it depending on the {@link #cloneLabelLayoutParameter} property. This method may return <code>null</code> to indicate that
       * the default label layout parameter should not be changed.
       * @protected
       */
      createLabelLayoutParameter(foldingView:yfiles.graph.IFoldingView,localLabel:yfiles.graph.ILabel,masterLabel:yfiles.graph.ILabel):yfiles.graph.ILabelModelParameter;
      /**
       * Callback factory method that creates the {@link yfiles.styles.ILabelStyle} for use in
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeLabels}.
       * @param foldingView The folding view instance.
       * @param localLabel The local label in the view whose {@link #style} should be initialized. May be <code>null</code> if the label style is requested for the label
       * creation.
       * @param masterLabel The label that is being represented by the folding label.
       * @returns The {@link #labelStyle} or a {@link #clone} of it depending on the {@link #cloneLabelStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createLabelStyle(foldingView:yfiles.graph.IFoldingView,localLabel:yfiles.graph.ILabel,masterLabel:yfiles.graph.ILabel):yfiles.styles.ILabelStyle;
      /**
       * Callback factory method that creates the {@link yfiles.styles.INodeStyle} for use in
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeStyle}.
       * @param foldingView The folding view instance.
       * @param viewNode The local folder node in the view whose {@link #style} should be initialized.
       * @param masterNode The node that is being represented by the folder node.
       * @returns The {@link #folderNodeStyle} or a {@link #clone} of it depending on the {@link #cloneNodeStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createNodeStyle(foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):yfiles.styles.INodeStyle;
      /**
       * Callback factory method that creates the {@link yfiles.graph.IPortLocationModelParameter} for use in
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodePorts}.
       * @param foldingView The folding view instance.
       * @param localPort The local port in the view whose {@link #locationParameter} should be initialized.
       * @param masterPort The port that is being represented by the folding port.
       * @returns The {@link #portLocationParameter} or a {@link #clone} of it depending on the {@link #clonePortLocationParameter} property. This method may return <code>null</code> to indicate that
       * the default port location parameter should not be changed.
       * @protected
       */
      createPortLocationParameter(foldingView:yfiles.graph.IFoldingView,localPort:yfiles.graph.IPort,masterPort:yfiles.graph.IPort):yfiles.graph.IPortLocationModelParameter;
      /**
       * Callback factory method that creates the {@link yfiles.styles.IPortStyle} for use in
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodePorts}.
       * @param foldingView The folding view instance.
       * @param localPort The local port in the view whose {@link #style} should be initialized.
       * @param masterPort The port that is being represented by the folding port.
       * @returns The {@link #portStyle} or a {@link #clone} of it depending on the {@link #clonePortStyle} property. This method may return <code>null</code> to indicate that
       * the default style should not be changed.
       * @protected
       */
      createPortStyle(foldingView:yfiles.graph.IFoldingView,localPort:yfiles.graph.IPort,masterPort:yfiles.graph.IPort):yfiles.styles.IPortStyle;
      /**
       * Callback method that initializes the initial labels of the collapsed group node.
       * <p>
       * This will try to copy the first label of the master node if {@link yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel} is enabled. It will call {@link yfiles.graph.DefaultFolderNodeConverter#createLabelStyle}
       * and {@link yfiles.graph.DefaultFolderNodeConverter#createLabelLayoutParameter} and use the returned style and layout parameter
       * if they are non-<code>null</code> when {@link yfiles.graph.FolderNodeState#addLabel adding the label} on the folder node.
       * </p>
       * @param state The node view state whose labels should be synchronized.
       * @param foldingView The folding view instance that has triggered the {@link #initializeFolderNodeState}
       * call.
       * @param viewNode The collapsed group node in the view that may be changed using the {@link }.
       * @param masterNode The master node that the folder node represents.
       * @see yfiles.graph.DefaultFolderNodeConverter#labelStyle
       * @see yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter
       * @see yfiles.graph.FoldingManager#getFolderNodeState
       * @protected
       */
      initializeFolderNodeLabels(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Callback method that initializes the {@link yfiles.graph.INode#layout} of the collapsed group node.
       * <p>
       * This implementation will {@link yfiles.graph.FolderNodeState#layout set the bounds} of the nodes in such a way that the size will be that of the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeSize} property (if
       * non-<code>null</code>) but not smaller than the {@link yfiles.input.INodeSizeConstraintProvider#getMinimumSize minimum} size that is reported by a potential {@link yfiles.input.INodeSizeConstraintProvider}
       * for {@link yfiles.graph.INode}s that has been found in the {@link yfiles.graph.ILookup} of the <code>viewNode</code>.
       * </p>
       * @param state The node view state whose layout should be synchronized.
       * @param foldingView The folding view instance.
       * @param viewNode The local collapsed group node to change.
       * @param masterNode The master node that is represented by the local group node.
       * @protected
       */
      initializeFolderNodeLayout(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Called by {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeState} to initialize the appearance of the {@link yfiles.graph.IFoldingView#getViewItem representatives}
       * of the master ports at the collapsed group node.
       * <p>
       * This method will call {@link yfiles.graph.DefaultFolderNodeConverter#createPortStyle} and {@link yfiles.graph.DefaultFolderNodeConverter#createPortLocationParameter}
       * for each port at the group node and assign the returned style and location parameter if they are non-<code>null</code>.
       * </p>
       * @param state The node view state whose ports should be synchronized.
       * @param foldingView The folding view instance.
       * @param viewNode The local group node.
       * @param masterNode The master group node.
       * @protected
       */
      initializeFolderNodePorts(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Implements the {@link yfiles.graph.IFolderNodeConverter#initializeFolderNodeState} method and initializes the appearance of the
       * collapsed group node.
       * <p>
       * This method calls {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeStyle},
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeLabels},
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeLayout}, and
       * {@link yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodePorts}.
       * </p>
       * @param state
       * @param foldingView
       * @param viewNode
       * @param masterNode
       */
      initializeFolderNodeState(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Callback method that initializes the {@link yfiles.graph.FolderNodeState#style style property} of the collapsed group node.
       * <p>
       * This implementation calls {@link yfiles.graph.DefaultFolderNodeConverter#createNodeStyle} and {@link yfiles.graph.FolderNodeState#style assigns} the style to the group
       * node if a non-<code>null</code> value has been returned.
       * </p>
       * @param state The node view state whose style should be synchronized.
       * @param foldingView The folding view instance that has triggered the {@link #initializeFolderNodeState}
       * call.
       * @param viewNode The local node in the view that may be changed using the {@link }.
       * @param masterNode The master node that the local folder node represents.
       * @see yfiles.graph.FoldingManager#getFolderNodeState
       * @protected
       */
      initializeFolderNodeStyle(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Called by {@link yfiles.graph.DefaultFolderNodeConverter#updateFolderNodeState} to synchronize the first label if {@link yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel} is enabled.
       * <p>
       * This will adjust the label text property or remove the label if there is no master label.
       * </p>
       * @param state The node view state whose labels should be synchronized.
       * @param foldingView The folding view.
       * @param viewNode The local node instance.
       * @param masterNode The master node.
       * @protected
       */
      synchronizeLabels(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Implements the {@link yfiles.graph.IFolderNodeConverter#updateFolderNodeState} method and changes the folder node appearance.
       * <p>
       * This method calls {@link yfiles.graph.DefaultFolderNodeConverter#synchronizeLabels}, only.
       * </p>
       * @param state
       * @param foldingView
       * @param viewNode
       * @param masterNode
       */
      updateFolderNodeState(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Gets or sets the {@link yfiles.styles.INodeStyle} to use for the {@link yfiles.graph.IFoldingView#collapse collapsed} group nodes.
       * <p>
       * Depending on the {@link yfiles.graph.DefaultFolderNodeConverter#cloneNodeStyle} property this implementation will assign a {@link yfiles.lang.ICloneable#clone} to the property or this
       * instance. The default is <code>null</code>
       * </p>
       * <p>
       * A value of <code>null</code> will leave the style property unmodified.
       * </p>
       * @type {yfiles.styles.INodeStyle}
       */
      folderNodeStyle:yfiles.styles.INodeStyle;
      /**
       * Gets or sets a value indicating whether the first label of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} should be recreated for the collapsed
       * group node instance.
       * <p>
       * This setting can be used to initially create a copy of the first label of the master group node (if any) and
       * subsequently synchronize the {@link yfiles.graph.ILabel#text} property with the master's node label text. Set it to <code>true</code> if the first label should be
       * copied; <code>false</code> otherwise. The default is <code>false</code>.
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter#labelStyle
       * @see yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter
       * @type {boolean}
       */
      copyFirstLabel:boolean;
      /**
       * Gets or sets the {@link yfiles.styles.ILabelStyle} to use for the first label of the collapsed group nodes.
       * <p>
       * This will only affect the collapsed group node if the {@link yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel} property is set to <code>true</code>. The default is <code>null</code>
       * </p>
       * <p>
       * A value of <code>null</code> will leave the style property unmodified.
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel
       * @type {yfiles.styles.ILabelStyle}
       */
      labelStyle:yfiles.styles.ILabelStyle;
      /**
       * Gets or sets the {@link yfiles.graph.ILabelModelParameter} to use for the first label of the collapsed group node.
       * <p>
       * This will only affect the folder node if the {@link yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel} property is set to <code>true</code>. The default is <code>null</code>
       * </p>
       * <p>
       * A value of <code>null</code> will leave the label model parameter property unmodified.
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter#copyFirstLabel
       * @see yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeLabels
       * @type {yfiles.graph.ILabelModelParameter}
       */
      labelLayoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the {@link yfiles.styles.IPortStyle} to use for the ports at the collapsed group node that {@link yfiles.graph.IFoldingView#getViewItem represent} the ports of
       * the {@link yfiles.graph.IFoldingView#getMasterItem master group node} in the collapsed state.
       * <p>
       * Since the appearance of the collapsed group node may totally differ from the appearance of the expanded group node, the
       * appearance of the ports may also be different. This property can be used to assign a different specific style to the
       * ports if the node is in collapsed state. A value of <code>null</code> will leave the style property unmodified. The default is <code>null</code>
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter#createPortStyle
       * @type {yfiles.styles.IPortStyle}
       */
      portStyle:yfiles.styles.IPortStyle;
      /**
       * Sets the initial size of the {@link yfiles.graph.INode#layout} of the collapsed group node that will be assigned during the {@link yfiles.graph.IFolderNodeConverter#initializeFolderNodeState creation} of the appearance of
       * the collapsed group node.
       * <p>
       * Since the {@link yfiles.graph.INode#layout} of an {@link yfiles.graph.IFoldingView#expand expanded} group node normally encompasses the child nodes, it will have quite a large size. Since the
       * contents of a collapsed group node are not visible, there is no need for the collapsed group node to encompass the area
       * of its children. Thus the size can be different and most of the time should be smaller. Setting this property to non-<code>null</code>
       * values will make the initial layout of the collapsed node become the given size assigned. The default is <code>null</code>
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter#initializeFolderNodeLayout
       * @type {yfiles.geometry.Size}
       */
      folderNodeSize:yfiles.geometry.Size;
      /**
       * Gets or sets the {@link yfiles.graph.IPortLocationModelParameter} to use for the ports at the collapsed group node that {@link yfiles.graph.IFoldingView#getViewItem represent}
       * the ports of the {@link yfiles.graph.IFoldingView#getMasterItem master group node} in the collapsed state.
       * <p>
       * Since the appearance of the collapsed group node may totally differ from the appearance of the expanded group node, the
       * appearance of the ports may also be different. This property can be used to assign a different specific location
       * parameter to the ports if the node is in collapsed state. A value of <code>null</code> will leave the location parameter property
       * unmodified. The default is <code>null</code>
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter#createPortLocationParameter
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      portLocationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#folderNodeStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneNodeStyle:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#portStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      clonePortStyle:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#portLocationParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone clone} or not.
       * @type {boolean}
       */
      clonePortLocationParameter:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#labelStyle} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneLabelStyle:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.DefaultFolderNodeConverter#labelLayoutParameter} instance should be assigned as a {@link yfiles.lang.ICloneable#clone}clone or not.
       * @type {boolean}
       */
      cloneLabelLayoutParameter:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple implementation of the {@link yfiles.graph.IFoldingEdgeConverter} interface for use in the {@link yfiles.graph.FoldingManager#foldingEdgeConverter} property, that will prevent
     * folding edges from appearing in the {@link yfiles.graph.IFoldingView view}.
     * @class
     * @implements {yfiles.graph.IFoldingEdgeConverter}
     */
    export interface ExcludingFoldingEdgeConverter extends Object,yfiles.graph.IFoldingEdgeConverter{}
    export class ExcludingFoldingEdgeConverter {
      /**
       * Always calls {@link yfiles.graph.IFoldingEdgeFactory#excludeFoldingEdge}
       * @param factory
       * @param foldingView
       * @param masterEdge
       * @param source
       * @param sourceIsCollapsed
       * @param target
       * @param targetIsCollapsed
       * @returns 
       */
      addFoldingEdge(factory:yfiles.graph.IFoldingEdgeFactory,foldingView:yfiles.graph.IFoldingView,masterEdge:yfiles.graph.IEdge,source:yfiles.graph.INode,sourceIsCollapsed:boolean,target:yfiles.graph.INode,targetIsCollapsed:boolean):yfiles.graph.IEdge;
      /**
       * Does nothing and should not normally be called by the view since all folding edges are excluded from it.
       * @param state
       * @param foldingView
       * @param foldingEdge
       * @param masterEdges
       */
      initializeFoldingEdgeState(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Does nothing and should not normally be called by the view since all folding edges are excluded from it.
       * @param state
       * @param foldingView
       * @param foldingEdge
       * @param masterEdges
       */
      updateFoldingEdgeState(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple implementation of the {@link yfiles.graph.IFoldingEdgeConverter} interface that will make the {@link yfiles.graph.IFoldingView view} contain at most one
     * folding edge between each pair of nodes by letting a single folding edge represent all master edges.
     * <p>
     * This implementation by default, will have the {@link yfiles.graph.FoldingEdgeConverterBase#sourcePortLocationParameter} and the {@link yfiles.graph.FoldingEdgeConverterBase#targetPortLocationParameter} properties set to {@link yfiles.graph.FreeNodePortLocationModel#NODE_CENTER_ANCHORED}
     * and the {@link yfiles.graph.FoldingEdgeConverterBase#resetBends} property set to <code>true</code>.
     * </p>
     * @class
     * @extends {yfiles.graph.FoldingEdgeConverterBase}
     */
    export interface MergingFoldingEdgeConverter extends yfiles.graph.FoldingEdgeConverterBase{}
    export class MergingFoldingEdgeConverter {
      /**
       * Initializes a new instance of the {@link yfiles.graph.MergingFoldingEdgeConverter} class.
       */
      constructor();
      /**
       * Adds the first separate edge to the source and target node pair using the {@link yfiles.graph.IFoldingEdgeFactory#addAsSeparateEdge}
       * method.
       * @param factory
       * @param foldingView
       * @param masterEdge
       * @param viewSourceNode
       * @param sourceIsCollapsed
       * @param targetSourceNode
       * @param targetIsCollapsed
       * @returns 
       * @protected
       */
      addFirstSeparateEdge(factory:yfiles.graph.IFoldingEdgeFactory,foldingView:yfiles.graph.IFoldingView,masterEdge:yfiles.graph.IEdge,viewSourceNode:yfiles.graph.INode,sourceIsCollapsed:boolean,targetSourceNode:yfiles.graph.INode,targetIsCollapsed:boolean):yfiles.graph.IEdge;
      /**
       * Gets or sets a value indicating whether edge direction should be ignored for the determination of the
       * {@link yfiles.graph.IFoldingEdgeFactory#getExistingFoldingEdges existing folding edges}.
       * @type {boolean}
       */
      ignoreEdgeDirection:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class adds folding support to {@link yfiles.graph.IGraph} implementations.
     * <p>
     * This implementation manages the varying states nodes and edges can have in different views and during different {@link yfiles.graph.IFoldingView#isExpanded collapsed/expanded}
     * states of the nodes. Create an instance of this class for a given {@link yfiles.graph.IGraph} and use the {@link yfiles.graph.FoldingManager#createFoldingView}
     * methods to create an {@link yfiles.graph.IGraph} implementation that provides a view of a subset of the graph, where {@link yfiles.graph.IGraph#isGroupNode group nodes}
     * can be {@link yfiles.graph.IFoldingView#collapse collapsed} and {@link yfiles.graph.IFoldingView#expand expanded}.
     * </p>
     * <p>
     * This class can be used to {@link yfiles.graph.FoldingManager#createFoldingView create folding views} of the {@link yfiles.graph.FoldingManager#masterGraph}, where group nodes can appear as {@link yfiles.graph.IFoldingView#collapse collapsed} group nodes with their
     * contents hidden. Edges that connect to elements inside collapsed group nodes can be {@link yfiles.graph.FoldingManager#foldingEdgeConverter configured} to be represented by {@link yfiles.graph.FoldingEdgeStateId folding edges}
     * in the views. The process of converting one or more true edges from the {@link yfiles.graph.FoldingManager#masterGraph} to zero or more folding edges can be
     * customized, as well as the process of {@link yfiles.graph.FoldingManager#folderNodeConverter converting} an expanded group node to a collapsed group node.
     * </p>
     * <p>
     * Class <code>FoldingManager</code>'s central role for folding support is discussed in the section {@link @DGUIDE_PREFIX@/folding-technical_background.html Folding Revisited}. A brief
     * description of the class's API, set-up, etc. is given in the section {@link @DGUIDE_PREFIX@/folding.html#folding-working_with_folding Working with Folding}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Simple Editor Demo, Demo.yFiles.Graph.SimpleEditor</li>
     * </ul>
     * @see yfiles.graph.IFoldingView
     * @see yfiles.graph.FoldingManager#createFoldingView
     * @class
     */
    export interface FoldingManager extends Object{}
    export class FoldingManager {
      /**
       * Creates a manager instance for the given {@link yfiles.graph.FoldingManager#masterGraph} to create views on top of it that support folding operations.
       * @param masterGraph The graph to use as the {@link #masterGraph}.
       * @throws {Stubs.Exceptions.ArgumentError} <code>masterGraph</code> is null.
       */
      constructor(masterGraph:yfiles.graph.IGraph);
      /**
       * Creates a manager instance for the given {@link yfiles.graph.FoldingManager#masterGraph} to create views on top of it that support folding operations.
       * @param {Object} options The parameters to pass.
       * @param options.masterGraph The graph to use as the {@link #masterGraph}.
       * @param {yfiles.graph.IFoldingEdgeConverter} options.foldingEdgeConverter The {@link yfiles.graph.IFoldingEdgeConverter} implementation that is used to create/convert and modify the {@link yfiles.graph.FoldingEdgeStateId folding edges} inside the view instances.
       * <p>
       * This option sets the {@link yfiles.graph.FoldingManager#foldingEdgeConverter} property on the created object.
       * </p>
       * @param {yfiles.graph.IFolderNodeConverter} options.folderNodeConverter The {@link yfiles.graph.IFolderNodeConverter} implementation that is used to create/convert and modify the folder nodes inside the view instances.
       * <p>
       * This option sets the {@link yfiles.graph.FoldingManager#folderNodeConverter} property on the created object.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} <code>masterGraph</code> is null.
       */
      constructor(options:{masterGraph:yfiles.graph.IGraph,foldingEdgeConverter?:yfiles.graph.IFoldingEdgeConverter,folderNodeConverter?:yfiles.graph.IFolderNodeConverter});
      /**
       * Creates a new manager with a default backing graph implementation.
       * <p>
       * Use {@link yfiles.graph.FoldingManager#createFoldingView} to create a view that supports the {@link yfiles.graph.IFoldingView} interface
       * for displaying and working with a collapsible graph structure.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.graph.IFoldingEdgeConverter} options.foldingEdgeConverter The {@link yfiles.graph.IFoldingEdgeConverter} implementation that is used to create/convert and modify the {@link yfiles.graph.FoldingEdgeStateId folding edges} inside the view instances.
       * <p>
       * This option sets the {@link yfiles.graph.FoldingManager#foldingEdgeConverter} property on the created object.
       * </p>
       * @param {yfiles.graph.IFolderNodeConverter} options.folderNodeConverter The {@link yfiles.graph.IFolderNodeConverter} implementation that is used to create/convert and modify the folder nodes inside the view instances.
       * <p>
       * This option sets the {@link yfiles.graph.FoldingManager#folderNodeConverter} property on the created object.
       * </p>
       * @see yfiles.graph.FoldingManager#createFoldingView
       * @see yfiles.graph.FoldingManager#masterGraph
       */
      constructor(options?:{foldingEdgeConverter?:yfiles.graph.IFoldingEdgeConverter,folderNodeConverter?:yfiles.graph.IFolderNodeConverter});
      /**
       * Creates a separate view instance of the {@link yfiles.graph.FoldingManager#masterGraph} which is a synchronized copy of a subset of the items from the master graph.
       * <p>
       * This method will create a new instance of an {@link yfiles.graph.IGraph} that is constructed as a synchronized copy of the {@link yfiles.graph.FoldingManager#masterGraph}.
       * That instance can be obtained from the {@link yfiles.graph.IFoldingView#graph} property of the returned {@link yfiles.graph.IFoldingView} implementation. The graph
       * instance will provide an {@link yfiles.graph.IFoldingView} instance in its {@link yfiles.graph.ILookup#lookup} mechanism:
       * </p>
       * <p>
       * This instance can be used to gain access to this manager instance and the folding operations.
       * </p>
       * <p>
       * Note that the entities that are contained in the returned graph instance are not the same instances as the original
       * "master" entities in the {@link yfiles.graph.FoldingManager#masterGraph}. However they share the same properties (style, {@link yfiles.graph.ITagOwner#tag tag}, {@link yfiles.graph.ILabelModelParameter parameters}, and {@link yfiles.graph.INode#layout geometry}). For collapsed
       * group nodes and view edges, there is a separate set of properties available for each possible state combination. The {@link yfiles.graph.FoldingManager#getFoldingEdgeState}
       * method for edges and the {@link yfiles.graph.FoldingManager#getFolderNodeState} method for nodes provide access to that state
       * while it is not being displayed in a view.
       * </p>
       * @param [root=null] A group node or the root of the {@link #masterGraph} whose contents will be displayed in the view.
       * @param [isExpanded=null] A predicate that can be provided to the view that will be used as a callback to determine the initial {@link #isExpanded expansion state} of
       * group nodes in the view.
       * @returns A new graph instance that represents a synchronized view on the {@link #masterGraph}.
       * @see yfiles.graph.FoldingManager#prepareFoldingView
       */
      createFoldingView(root?:yfiles.graph.INode,isExpanded?:(obj:yfiles.graph.INode)=>boolean):yfiles.graph.IFoldingView;
      /**
       * Creates a separate view instance of the {@link yfiles.graph.FoldingManager#masterGraph} which is a synchronized copy of a subset of the items from the master graph.
       * <p>
       * This method will create a new instance of an {@link yfiles.graph.IGraph} that is constructed as a synchronized copy of the {@link yfiles.graph.FoldingManager#masterGraph}.
       * That instance can be obtained from the {@link yfiles.graph.IFoldingView#graph} property of the returned {@link yfiles.graph.IFoldingView} implementation. The graph
       * instance will provide an {@link yfiles.graph.IFoldingView} instance in its {@link yfiles.graph.ILookup#lookup} mechanism:
       * </p>
       * <p>
       * This instance can be used to gain access to this manager instance and the folding operations.
       * </p>
       * <p>
       * Note that the entities that are contained in the returned graph instance are not the same instances as the original
       * "master" entities in the {@link yfiles.graph.FoldingManager#masterGraph}. However they share the same properties (style, {@link yfiles.graph.ITagOwner#tag tag}, {@link yfiles.graph.ILabelModelParameter parameters}, and {@link yfiles.graph.INode#layout geometry}). For collapsed
       * group nodes and view edges, there is a separate set of properties available for each possible state combination. The {@link yfiles.graph.FoldingManager#getFoldingEdgeState}
       * method for edges and the {@link yfiles.graph.FoldingManager#getFolderNodeState} method for nodes provide access to that state
       * while it is not being displayed in a view.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.root=null] A group node or the root of the {@link #masterGraph} whose contents will be displayed in the view.
       * @param [options.isExpanded=null] A predicate that can be provided to the view that will be used as a callback to determine the initial {@link #isExpanded expansion state} of
       * group nodes in the view.
       * @returns A new graph instance that represents a synchronized view on the {@link #masterGraph}.
       * @see yfiles.graph.FoldingManager#prepareFoldingView
       */
      createFoldingView(options:{root?:yfiles.graph.INode,isExpanded?:(obj:yfiles.graph.INode)=>boolean}):yfiles.graph.IFoldingView;
      /**
       * Disposes of this instance.
       * <p>
       * This will disconnect the {@link yfiles.graph.FoldingManager#masterGraph} from the previously created {@link yfiles.graph.FoldingManager#createFoldingView managed views}. Also each view will be {@link yfiles.graph.IFoldingView#dispose disposed}.
       * </p>
       */
      dispose():void;
      /**
       * Provides access to the folder node state of a node.
       * <p>
       * This method can be used to query the collapsed state of a node, even if the node is currently not present in any managed
       * view or the node is in fact expanded in all views.
       * </p>
       * @param masterNode The node in the {@link #masterGraph} for which the state should be returned.
       * @returns A state holder implementation of a node, which is not part of any graph.
       * @throws {Stubs.Exceptions.ArgumentError} The master node does not belong to the managed graph.
       */
      getFolderNodeState(masterNode:yfiles.graph.INode):yfiles.graph.FolderNodeState;
      /**
       * Return a data container implementation that is not part of any actual {@link yfiles.graph.IGraph} that describes the state of
       * the edge for the given {@link yfiles.graph.FoldingEdgeStateId}.
       * <p>
       * This method can be used to query the state of an edge that is either not currently visible in any {@link yfiles.graph.FoldingManager#views view} or connects to
       * different source and target nodes because of the {@link yfiles.graph.IFoldingView#collapse collapsed} states of the nodes and their parents in the view.
       * </p>
       * @param id The ID of the folding edge for which the folding state should be returned.
       * @returns A {@link } that can be used to query and change the properties of the edge.
       * @see yfiles.graph.FoldingManager#getFolderNodeState
       * @see yfiles.graph.FoldingEdgeStateId
       */
      getFoldingEdgeState(id:yfiles.graph.FoldingEdgeStateId):yfiles.graph.FoldingEdgeState;
      /**
       * Determines whether for the given <code>masterNode</code> this manager has stored associated folder node state.
       * <p>
       * The manager does not necessarily have folder node state for folder nodes if that specific state has never been requested
       * by a {@link yfiles.graph.IFoldingView view} because it has never been visualized. This method indicates whether there is state information available for
       * the given master node.
       * </p>
       * @param masterNode The master node for which the state is requested.
       * @returns Whether {@link #getFolderNodeState} would return existing state information.
       * @see yfiles.graph.FoldingManager#getFolderNodeState
       */
      hasFolderNodeState(masterNode:yfiles.graph.INode):boolean;
      /**
       * Determines whether for the given {@link yfiles.graph.FoldingEdgeStateId} this manager has stored associated view local state.
       * <p>
       * The manager does not necessarily have a {@link yfiles.graph.FoldingEdgeState} for folding elements if that specific state has
       * never been requested by a {@link yfiles.graph.IFoldingView} because it has never been visualized. This method indicates whether
       * there is state information available for the given id.
       * </p>
       * @param id The id for which state is requested.
       * @returns Whether {@link #getFoldingEdgeState} would return existing state information.
       * @see yfiles.graph.FoldingManager#getFoldingEdgeState
       */
      hasFoldingEdgeState(id:yfiles.graph.FoldingEdgeStateId):boolean;
      /**
       * Callback that can be overridden by subclasses to prepare the {@link yfiles.graph.IFoldingView} instance right before it will be
       * initialized with a copy of the elements in the initial graph.
       * <p>
       * Subclasses could override this method to register additional {@link yfiles.collections.IMapper.<K,V>} instances with the graph instance or add
       * {@link yfiles.graph.IContextLookupChainLink}s via the {@link yfiles.graph.ILookupDecorator} of the {@link yfiles.graph.IFoldingView#graph} to the instance if they are
       * essential for the initial creation of the elements in the view instance. Overriding methods should always consider to
       * call the base class implementation, first.
       * </p>
       * @param foldingView The graph view that has just been created but is still empty.
       * @protected
       */
      prepareFoldingView(foldingView:yfiles.graph.IFoldingView):void;
      /**
       * Triggers a call to {@link yfiles.graph.IFolderNodeConverter#updateFolderNodeState} for all view nodes that {@link yfiles.graph.IFoldingView#getViewItem represent} the given <code>masterNode</code>
       * in its collapsed state.
       * <p>
       * In all {@link yfiles.graph.FoldingManager#views} that are currrently showing the node in collapsed state, the {@link yfiles.graph.FoldingManager#getFolderNodeState view-local state} will be updated via a call to the
       * {@link yfiles.graph.IFolderNodeConverter}'s {@link yfiles.graph.IFolderNodeConverter#updateFolderNodeState} method.
       * </p>
       * <p>
       * Note that all {@link yfiles.graph.FoldingManager#views} share the same {@link yfiles.graph.FolderNodeState} for the <code>masterNode</code>.
       * </p>
       * <p>
       * This method is useful in situations where a change in the data on the master graph did not automatically trigger a call
       * to the converter. Using this method, the state of the collapsed nodes in the views can be synchronized according to the
       * logic in the {@link yfiles.graph.IFolderNodeConverter} implementation.
       * </p>
       * @param masterNode The master node that belongs to the {@link #masterGraph}.
       * @throws {Stubs.Exceptions.ArgumentError} If the provided master node does not belong to the master graph.
       * @see yfiles.graph.FoldingManager#updateFoldingEdgeStates
       */
      updateFolderNodeStates(masterNode:yfiles.graph.INode):void;
      /**
       * Triggers a call to {@link yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState} for all folder edges that {@link yfiles.graph.IFoldingView#getViewItem represent} the
       * given <code>masterEdge</code>.
       * <p>
       * In all {@link yfiles.graph.FoldingManager#views} that are currrently showing the view edge for the provided master edge, the {@link yfiles.graph.FoldingManager#getFoldingEdgeState view-local state} will be updated via
       * a call to the {@link yfiles.graph.IFoldingEdgeConverter}'s {@link yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState} method.
       * </p>
       * <p>
       * This method is useful in situations where a change in the data on the master graph did not automatically trigger a call
       * to the converter. Using this method, the state of the representing edges in the views can be synchronized according to
       * the logic in the {@link yfiles.graph.IFoldingEdgeConverter} implementation.
       * </p>
       * @param masterEdge The master edge that belongs to the {@link #masterGraph}.
       * @throws {Stubs.Exceptions.ArgumentError} If the provided master edge does not belong to the master graph.
       * @see yfiles.graph.FoldingManager#updateFolderNodeStates
       */
      updateFoldingEdgeStates(masterEdge:yfiles.graph.IEdge):void;
      /**
       * Gets or sets the {@link yfiles.graph.IFoldingEdgeConverter} implementation that is used to create/convert and modify the {@link yfiles.graph.FoldingEdgeStateId folding edges}
       * inside the view instances.
       * <p>
       * The converter instance will be used to {@link yfiles.graph.IFoldingEdgeConverter#initializeFoldingEdgeState create the initial} and {@link yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState change} the appearance of the folding edges as soon as they
       * appear in a view.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} <code>value</code> is null.
       * @see yfiles.graph.DefaultFoldingEdgeConverter
       * @type {yfiles.graph.IFoldingEdgeConverter}
       */
      foldingEdgeConverter:yfiles.graph.IFoldingEdgeConverter;
      /**
       * Gets or sets the {@link yfiles.graph.IFolderNodeConverter} implementation that is used to create/convert and modify the folder
       * nodes inside the view instances.
       * <p>
       * The converter instance will be used to {@link yfiles.graph.IFolderNodeConverter#initializeFolderNodeState create the initial} and {@link yfiles.graph.IFolderNodeConverter#updateFolderNodeState change} the appearance of the folder nodes as soon as they
       * appear in a view.
       * </p>
       * @see yfiles.graph.DefaultFolderNodeConverter
       * @type {yfiles.graph.IFolderNodeConverter}
       */
      folderNodeConverter:yfiles.graph.IFolderNodeConverter;
      /**
       * Yields the master {@link yfiles.graph.IGraph} instance that holds the complete model in expanded group node state.
       * <p>
       * This graph contains all non-folding edges and expanded group nodes, no folding edges and no collapsed group nodes.
       * </p>
       * @type {yfiles.graph.IGraph}
       */
      masterGraph:yfiles.graph.IGraph;
      /**
       * Yields all currently known {@link yfiles.graph.FoldingManager#createFoldingView view instances} that are managed by this instance.
       * @see yfiles.graph.IFoldingView#graph
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IFoldingView>}
       */
      views:yfiles.collections.IEnumerable<yfiles.graph.IFoldingView>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data container representing the state of a bend of a {@link yfiles.graph.FoldingEdgeState}.
     * <p>
     * When the properties of the state are changed, these changes are reflected immediately in the {@link yfiles.graph.FoldingManager#views}.
     * </p>
     * @class
     * @final
     */
    export interface FoldingBendState extends Object{}
    export class FoldingBendState {
      /**
       * Returns an {@link yfiles.graph.IBend} instance which represents this bend state.
       * <p>
       * The returned bend is not present in any graph but may be passed to {@link yfiles.graph.IFoldingView#getViewItem} to get the bend
       * state's view bend in the {@link yfiles.graph.IFoldingView#graph folding view graph}.
       * </p>
       * @returns An {@link } instance which represents this bend state.
       */
      asBend():yfiles.graph.IBend;
      /**
       * The location of the bend.
       * @type {yfiles.geometry.Point}
       */
      location:yfiles.geometry.Point;
      /**
       * The tag of the bend.
       * @type {Object}
       */
      tag:Object;
      /**
       * Gets the bend's index in the {@link yfiles.graph.FoldingEdgeState#bends} of its owner.
       * @type {number}
       */
      index:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data container representing the state of an edge whose source and/or target node are collapsed.
     * <p>
     * When the properties of the state are changed, these changes are reflected immediately in the {@link yfiles.graph.FoldingManager#views}.
     * </p>
     * @see yfiles.graph.FoldingEdgeStateId
     * @see yfiles.graph.FoldingManager#getFoldingEdgeState
     * @class
     * @extends {yfiles.graph.FoldingLabelOwnerState}
     * @final
     */
    export interface FoldingEdgeState extends yfiles.graph.FoldingLabelOwnerState{}
    export class FoldingEdgeState {
      /**
       * Adds a new {@link yfiles.graph.FoldingBendState} to this edge state at the given <code>index</code> and <code>location</code>.
       * @param location The location of the new bend state.
       * @param [index=-1] The index the bend state shall be inserted in the
       * {@link #bends}; a negative value (which is the default) indicates that the bend should be appended to the end of the list of bends.
       * @returns The newly added bend state.
       */
      addBend(location:yfiles.geometry.Point,index?:number):yfiles.graph.FoldingBendState;
      /**
       * Returns an {@link yfiles.graph.IEdge} instance which represents this edge state.
       * <p>
       * The returned edge is not present in any graph but may be passed to {@link yfiles.graph.IFoldingView#getViewItem} to get the edge
       * state's view edge in the {@link yfiles.graph.IFoldingView#graph folding view graph}.
       * </p>
       * @returns An {@link } instance which represents this edge state.
       */
      asEdge():yfiles.graph.IEdge;
      /**
       * Removes all {@link yfiles.graph.FoldingEdgeState#bends}.
       */
      clearBends():void;
      /**
       * 
       * @param label
       */
      remove(label:yfiles.graph.FoldingLabelState):void;
      /**
       * Removes the <code>bend</code> from this edge state.
       * @param bend The bend to remove.
       * @throws {Stubs.Exceptions.ArgumentError} Thrown if the <code>bend</code> doesn't belong to this edge state.
       */
      remove(bend:yfiles.graph.FoldingBendState):void;
      /**
       * The tag of the edge.
       * @type {Object}
       */
      tag:Object;
      /**
       * The {@link yfiles.graph.FoldingBendState}s of this state.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.FoldingBendState>}
       */
      bends:yfiles.collections.IListEnumerable<yfiles.graph.FoldingBendState>;
      /**
       * The {@link yfiles.graph.FoldingPortState} which represents the source port of this edge state.
       * @type {yfiles.graph.FoldingPortState}
       */
      sourcePort:yfiles.graph.FoldingPortState;
      /**
       * The {@link yfiles.graph.FoldingPortState} which represents the target port of this edge state.
       * @type {yfiles.graph.FoldingPortState}
       */
      targetPort:yfiles.graph.FoldingPortState;
      /**
       * The style of this edge.
       * @type {yfiles.styles.IEdgeStyle}
       */
      style:yfiles.styles.IEdgeStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data container representing the state of a label at a {@link yfiles.graph.FolderNodeState} or {@link yfiles.graph.FoldingEdgeState}.
     * <p>
     * When the properties of the state are changed, these changes are reflected immediately in the {@link yfiles.graph.FoldingManager#views}.
     * </p>
     * @see yfiles.graph.FoldingLabelOwnerState#labels
     * @see yfiles.graph.FoldingLabelOwnerState#addLabel
     * @class
     * @final
     */
    export interface FoldingLabelState extends Object{}
    export class FoldingLabelState {
      /**
       * Returns an {@link yfiles.graph.ILabel} instance which represents this label state.
       * <p>
       * The returned label is not present in any graph but may be passed to {@link yfiles.graph.IFoldingView#getViewItem} to get the
       * label state's view label in the {@link yfiles.graph.IFoldingView#graph folding view graph}.
       * </p>
       * @returns An {@link } instance which represents this label state.
       */
      asLabel():yfiles.graph.ILabel;
      /**
       * The style of this label.
       * @type {yfiles.styles.ILabelStyle}
       */
      style:yfiles.styles.ILabelStyle;
      /**
       * The preferred size of the label.
       * @type {yfiles.geometry.Size}
       */
      preferredSize:yfiles.geometry.Size;
      /**
       * The label's text.
       * @type {string}
       */
      text:string;
      /**
       * The {@link yfiles.graph.ILabelModelParameter} used to describe the label's layout.
       * @type {yfiles.graph.ILabelModelParameter}
       */
      layoutParameter:yfiles.graph.ILabelModelParameter;
      /**
       * The tag of the label.
       * @type {Object}
       */
      tag:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class for data containers representing the state of model items which can own labels.
     * <p>
     * Developers should not extend this class. Only instances of its subclasses {@link yfiles.graph.FolderNodeState} and {@link yfiles.graph.FoldingEdgeState}
     * are supported.
     * </p>
     * <p>
     * When the properties of the state are changed, these changes are reflected immediately in the {@link yfiles.graph.FoldingManager#views}.
     * </p>
     * @see yfiles.graph.FolderNodeState
     * @see yfiles.graph.FoldingEdgeState
     * @see yfiles.graph.FoldingLabelState
     * @class
     */
    export interface FoldingLabelOwnerState extends Object{}
    export class FoldingLabelOwnerState {
      constructor();
      /**
       * Adds a {@link yfiles.graph.FoldingLabelState} to this state.
       * @param text The text of the new label.
       * @param layoutParameter The layout parameter of the new label.
       * @param style The style of the new label.
       * @param preferredSize The preferred size of the label.
       * @param tag The tag of the label.
       * @returns The newly added {@link }.
       * @abstract
       */
      addLabel(text:string,layoutParameter:yfiles.graph.ILabelModelParameter,style:yfiles.styles.ILabelStyle,preferredSize:yfiles.geometry.Size,tag:Object):yfiles.graph.FoldingLabelState;
      /**
       * Removes all {@link yfiles.graph.FoldingLabelOwnerState#labels}.
       */
      clearLabels():void;
      /**
       * The {@link yfiles.graph.FoldingLabelState}s owned by this state.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.FoldingLabelState>}
       */
      labels:yfiles.collections.IListEnumerable<yfiles.graph.FoldingLabelState>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data container representing the state of a port at a {@link yfiles.graph.FolderNodeState} or {@link yfiles.graph.FoldingEdgeState}.
     * <p>
     * When the properties of the state are changed, these changes are reflected immediately in the {@link yfiles.graph.FoldingManager#views}.
     * </p>
     * @see yfiles.graph.FoldingEdgeState#sourcePort
     * @see yfiles.graph.FoldingEdgeState#targetPort
     * @see yfiles.graph.FolderNodeState#ports
     * @see yfiles.graph.FolderNodeState#getFoldingPortState
     * @class
     */
    export interface FoldingPortState extends Object{}
    export class FoldingPortState {
      /**
       * Returns an {@link yfiles.graph.IPort} instance which represents this port state.
       * <p>
       * The returned port is not present in any graph but may be passed to {@link yfiles.graph.IFoldingView#getViewItem} to get the port
       * state's view port in the {@link yfiles.graph.IFoldingView#graph folding view graph}.
       * </p>
       * @returns An {@link } instance which represents this port state.
       * @abstract
       */
      asPort():yfiles.graph.IPort;
      /**
       * The style of this port.
       * @type {yfiles.styles.IPortStyle}
       */
      style:yfiles.styles.IPortStyle;
      /**
       * The {@link yfiles.graph.IPortLocationModelParameter} describing the position of this port.
       * @type {yfiles.graph.IPortLocationModelParameter}
       */
      locationParameter:yfiles.graph.IPortLocationModelParameter;
      /**
       * The tag of the port.
       * @abstract
       * @type {Object}
       */
      tag:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A data container representing the collapsed state of a node.
     * <p>
     * When the properties of the state are changed, these changes are reflected immediately in the {@link yfiles.graph.FoldingManager#views}.
     * </p>
     * @see yfiles.graph.FoldingManager#getFolderNodeState
     * @class
     * @extends {yfiles.graph.FoldingLabelOwnerState}
     * @final
     */
    export interface FolderNodeState extends yfiles.graph.FoldingLabelOwnerState{}
    export class FolderNodeState {
      /**
       * Returns an {@link yfiles.graph.INode} instance which represents this folder node state.
       * <p>
       * The returned node is not present in any graph but may be passed to {@link yfiles.graph.IFoldingView#getViewItem} to get the node
       * state's folder node in the {@link yfiles.graph.IFoldingView#graph folding view graph}.
       * </p>
       * @returns An {@link } instance which represents this folder node state.
       */
      asNode():yfiles.graph.INode;
      /**
       * Provides access to the folding state of a master port.
       * <p>
       * This method can be used to query the folding state of a port at the folder which is represented by this state.
       * </p>
       * @param masterPort The master port for which the state should be returned.
       * @returns A state holder implementation of the <code>masterPort</code>, which reflects the port in the node's collapsed state.
       * @see yfiles.graph.FoldingPortState#style
       * @see yfiles.graph.FoldingPortState#locationParameter
       */
      getFoldingPortState(masterPort:yfiles.graph.IPort):yfiles.graph.FoldingPortState;
      /**
       * 
       * @param label
       */
      remove(label:yfiles.graph.FoldingLabelState):void;
      /**
       * The layout of the node.
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * The style of this node.
       * @type {yfiles.styles.INodeStyle}
       */
      style:yfiles.styles.INodeStyle;
      /**
       * The tag of the node.
       * @type {Object}
       */
      tag:Object;
      /**
       * The {@link yfiles.graph.FoldingPortState}s owned by this state.
       * @type {yfiles.collections.IListEnumerable.<yfiles.graph.FoldingPortState>}
       */
      ports:yfiles.collections.IListEnumerable<yfiles.graph.FoldingPortState>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface used by {@link yfiles.graph.FoldingManager} and {@link yfiles.graph.IFoldingView} which is responsible for managing edges
     * between folder nodes.
     * <p>
     * Since {@link yfiles.graph.FoldingManager#createFoldingView} creates views that can potentially contain edges that connect to
     * nodes which have a different set of attributes, these edges might need to have a separate set of attributes, too.
     * Implementations of this interface are being called by the
     * {@link yfiles.graph.IFoldingView}s to configure these kind of folding edges.
     * </p>
     * @see yfiles.graph.DefaultFoldingEdgeConverter
     * @see yfiles.graph.FoldingManager#foldingEdgeConverter
     * @see yfiles.graph.FoldingEdgeStateId
     * @interface
     */
    export interface IFoldingEdgeConverter extends Object{
      /**
       * This method gets called by the {@link yfiles.graph.IFoldingView} implementation to determine whether a given edge in the {@link yfiles.graph.FoldingManager#masterGraph} should
       * be represented by a folding edge in the given view.
       * <p>
       * The implementation will be passed in a <code>factory</code> object that implements the {@link yfiles.graph.IFoldingEdgeFactory} interface,
       * which needs to be used by the implementation to communicate the result of the query. The implementation needs to either
       * call {@link yfiles.graph.IFoldingEdgeFactory#addAsSeparateEdge}, {@link yfiles.graph.IFoldingEdgeFactory#addToExistingFoldingEdge}, or {@link yfiles.graph.IFoldingEdgeFactory#excludeFoldingEdge}
       * and return the values that these implementation yields to the caller. The implementation can optionally query the {@link yfiles.graph.IFoldingEdgeFactory#getExistingFoldingEdges existing folding edges}
       * between the source and target node and decide whether to add the edge {@link yfiles.graph.IFoldingEdgeFactory#addToExistingFoldingEdge to the existing folding edge}.
       * </p>
       * @param factory The {@link } implementation that needs to be called in order to communicate the results of this
       * query.
       * @param foldingView The instance into which the folding edge is going to be inserted.
       * @param masterEdge The edge from the {@link #masterGraph} that needs to be represented by a folding edge. Note that you may not return this instance.
       * @param source The source node that belongs to the {@link } graph that will act as the local view node of the actual
       * source node.
       * @param sourceIsCollapsed Determines whether the <code>source</code> is currently a folder node.
       * @param target The target node that belongs to the {@link } graph that will act as the local view node of the actual
       * target node.
       * @param targetIsCollapsed Determines whether the <code>target</code> is currently a folder node.
       * @returns The edge as returned by {@link #addAsSeparateEdge}, or
       * {@link #addToExistingFoldingEdge}, or <code>null</code> if the edge will be {@link #excludeFoldingEdge excluded} from the view.
       * @abstract
       */
      addFoldingEdge(factory:yfiles.graph.IFoldingEdgeFactory,foldingView:yfiles.graph.IFoldingView,masterEdge:yfiles.graph.IEdge,source:yfiles.graph.INode,sourceIsCollapsed:boolean,target:yfiles.graph.INode,targetIsCollapsed:boolean):yfiles.graph.IEdge;
      /**
       * Callback that is triggered by the {@link yfiles.graph.IFoldingView view} to initially create the appearance of a folding edge.
       * <p>
       * Implementations may not use the {@link yfiles.graph.IFoldingView}'s {@link yfiles.graph.IFoldingView#graph} instance to modify the <code>foldingEdge</code>, because this would
       * create and enqueue undo events. Rather the properties provided by the <code>state</code> implementation must be set.
       * </p>
       * @param state The {@link } of the edge to change.
       * @param foldingView The graph to which the folding edge belongs.
       * @param foldingEdge The folding edge for which the appearance shall be determined.
       * @param masterEdges The list of {@link #getMasterEdges master edges}, that the folding edge initially represents.
       * @see yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState
       * @abstract
       */
      initializeFoldingEdgeState(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
      /**
       * Callback that is triggered by the {@link yfiles.graph.IFoldingView view} to adjust the appearance of a folding edge
       * <p>
       * For instance, it can be used to reflect a state change in the {@link yfiles.graph.IFoldingView#getMasterEdges master edges} that make up that folding edge.
       * </p>
       * <p>
       * Implementations may not use the {@link yfiles.graph.IFoldingView}'s {@link yfiles.graph.IFoldingView#graph} instance to modify the <code>foldingEdge</code>, because this would
       * create and enqueue undo events. Rather the properties provided by the <code>state</code> implementation must be set.
       * </p>
       * @param state The {@link } of the edge to change.
       * @param foldingView The graph to which the folding edge belongs.
       * @param foldingEdge The folding edge for which the appearance might need an update.
       * @param masterEdges The list of {@link #getMasterEdges master edges}, that the folding edge currently represents.
       * @see yfiles.graph.IFoldingEdgeConverter#initializeFoldingEdgeState
       * @abstract
       */
      updateFoldingEdgeState(state:yfiles.graph.FoldingEdgeState,foldingView:yfiles.graph.IFoldingView,foldingEdge:yfiles.graph.IEdge,masterEdges:yfiles.collections.IList<yfiles.graph.IEdge>):void;
    }
    var IFoldingEdgeConverter:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.graph.FoldingManager} and {@link yfiles.graph.IFoldingView} which is responsible for managing the
     * appearance of folder nodes.
     * <p>
     * Since {@link yfiles.graph.FoldingManager#createFoldingView} creates views that can potentially contain nodes which have a
     * different set of attributes, these attributes need to be determined somehow. Implementations of this interface are being
     * called by the {@link yfiles.graph.IFoldingView view}s to configure the appearance of these {@link yfiles.graph.IFoldingView#isInFoldingState folder nodes}.
     * </p>
     * @see yfiles.graph.DefaultFolderNodeConverter
     * @see yfiles.graph.FoldingManager#folderNodeConverter
     * @see yfiles.graph.IFoldingView#isInFoldingState
     * @interface
     */
    export interface IFolderNodeConverter extends Object{
      /**
       * Callback that is used by {@link yfiles.graph.IFoldingView view} implementations to initially create the appearance of a {@link yfiles.graph.IFoldingView#isInFoldingState folder} node.
       * <p>
       * This method is called by the view to allow for initially creating the appearance of a local folder node in the view when
       * it is included in the view for the first time. This can be used, e.g. to initialize the labels or style properties with
       * the corresponding properties of the <code>masterNode</code>.
       * </p>
       * <p>
       * By default the <code>state</code> is an exact copy of the <code>masterNode</code>, including layout, style, labels, and ports.
       * </p>
       * <p>
       * Note that changing the appearance has to be done on <code>state</code>.
       * </p>
       * @param state The folder node state to initialize.
       * @param foldingView The graph instance for which the folder node has been created.
       * @param viewNode The folder node instance in the view
       * @param masterNode The node that represents the {@link #getMasterItem} of the viewstate to change.
       * @see yfiles.graph.IFolderNodeConverter#updateFolderNodeState
       * @abstract
       */
      initializeFolderNodeState(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
      /**
       * Callback that is used by {@link yfiles.graph.IFoldingView view} implementations to change the appearance of a {@link yfiles.graph.IFoldingView#isInFoldingState folder} node.
       * <p>
       * This method is called by the view to allow for adjusting the appearance of a local folder node in the view when the {@link yfiles.graph.IFoldingView#getMasterItem master}
       * node for it has changed properties. This can be used, e.g. to synchronize the label or style properties with the
       * corresponding properties of the <code>masterNode</code>.
       * </p>
       * <p>
       * Note that changing the appearance has to be done on <code>state</code>.
       * </p>
       * @param state The folder node state to update.
       * @param foldingView The graph instance for which the folder node can be changed.
       * @param viewNode The folder node instance in the view
       * @param masterNode The node that represents the {@link #getMasterItem} of the viewstate to change.
       * @see yfiles.graph.IFolderNodeConverter#initializeFolderNodeState
       * @abstract
       */
      updateFolderNodeState(state:yfiles.graph.FolderNodeState,foldingView:yfiles.graph.IFoldingView,viewNode:yfiles.graph.INode,masterNode:yfiles.graph.INode):void;
    }
    var IFolderNodeConverter:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * The interface of the factory that is passed to implementations of the {@link yfiles.graph.IFoldingEdgeConverter}'s {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge}
     * method.
     * <p>
     * Implementors of the {@link yfiles.graph.IFoldingEdgeConverter} need to use this interface to tell the calling {@link yfiles.graph.IFoldingView}
     * implementation how to add the folding edge to the given view. They need to call exactly one of the following methods for
     * each factory:
     * </p>
     * <ul>
     * <li>
     * <b>{@link yfiles.graph.IFoldingEdgeFactory#addAsSeparateEdge}</b> – Call this to add a separate folding edge for the given master edge to the view. Pass the return value of that method to
     * the caller.
     * </li>
     * <li>
     * <b>{@link yfiles.graph.IFoldingEdgeFactory#addToExistingFoldingEdge}</b> – Call this to reuse a folding edge that is already part of the view as a representative for the master edge. Pass the
     * return value of that method to the caller.
     * </li>
     * <li><b>{@link yfiles.graph.IFoldingEdgeFactory#excludeFoldingEdge}</b> – Call this in order not to add a folding edge for the master edge to the view at all. Pass <code>null</code> to the caller.</li>
     * </ul>
     * @see yfiles.graph.IFoldingEdgeConverter#addFoldingEdge
     * @see yfiles.graph.IFoldingEdgeConverter
     * @see yfiles.graph.FoldingManager#foldingEdgeConverter
     * @interface
     */
    export interface IFoldingEdgeFactory extends Object{
      /**
       * Tells the {@link yfiles.graph.IFoldingView} implementation to add the edge provided to the {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge}
       * method as a separate folding edge to the view at the given ports.
       * <p>
       * This tells the view that there is currently no {@link yfiles.graph.IFoldingEdgeFactory#getExistingFoldingEdges existing folding edge} in the view to which the master edge should be
       * added to. Instead the view should add a separate folding edge that represents the master edge. Use the result of this
       * method call as the return value for the {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge} call. Note that the returned
       * folding edge should not be modified by this code, instead the {@link yfiles.graph.IFoldingEdgeConverter#initializeFoldingEdgeState}
       * and {@link yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState} callbacks need to be used to change the appearance of the
       * folding edge. Existing ports at the provided source and target nodes may be specified to further influence the folding
       * edge creation.
       * </p>
       * @param sourcePort The source port at the source node in the view to use or <code>null</code> if an extra {@link #isInFoldingState folding state} port should be added for the
       * edge to connect to at the source.
       * @param targetPort The target port at the target node in the view to use or <code>null</code> if an extra {@link #isInFoldingState folding state} port should be added for the
       * edge to connect to at the target.
       * @returns The folding edge that has been created in the view. Don't change the properties of the edge directly.
       * @see yfiles.graph.IFoldingEdgeConverter
       * @see yfiles.graph.IFoldingView#getViewItem
       * @abstract
       */
      addAsSeparateEdge(sourcePort:yfiles.graph.IPort,targetPort:yfiles.graph.IPort):yfiles.graph.IEdge;
      /**
       * Tells the {@link yfiles.graph.IFoldingView} implementation to logically add the edge provided to the {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge}
       * method to a folding edge that already exists in the view.
       * <p>
       * This tells the view that there is already an {@link yfiles.graph.IFoldingEdgeFactory#getExistingFoldingEdges existing folding edge} in the view to which the master edge should be added
       * logically added to. The given <code>foldingEdge</code> will have the master edge added to its collection of {@link yfiles.graph.IFoldingView#getMasterEdges master edges}. Note that
       * the returned folding edge should not be modified by this code, instead the {@link yfiles.graph.IFoldingEdgeConverter#updateFoldingEdgeState}
       * callback needs to be used to change the appearance of the folding edge. Implementations can use the {@link yfiles.graph.IFoldingEdgeFactory#getExistingFoldingEdges}
       * method to retrieve the current set of existing folding edges that interconnect the same source and target nodes. The
       * range of valid parameters is constrained to this collection.
       * </p>
       * @param foldingEdge One of the {@link #getExistingFoldingEdges existing folding edges} from the view which has been chosen to represent the master edge in this view.
       * @returns The folding edge that represents the edge in the view, which is the same as the <code>foldingEdge</code> parameter. Don't change the
       * properties of the edge directly.
       * @see yfiles.graph.IFoldingEdgeConverter
       * @see yfiles.graph.IFoldingEdgeFactory#getExistingFoldingEdges
       * @abstract
       */
      addToExistingFoldingEdge(foldingEdge:yfiles.graph.IEdge):yfiles.graph.IEdge;
      /**
       * Tells the {@link yfiles.graph.IFoldingView} implementation not to add a folding edge for the edge provided to the {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge}
       * method in this view.
       * <p>
       * This tells the view that there should not be a representation of the master edge in this view. If this method is called,
       * <code>null</code> should be returned as the result of a {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge} call, which is the same as
       * the return value of this method. So it is possible to just pass the return value on to the caller.
       * </p>
       * @returns This method will always return <code>null</code>. This is for symmetry reasons to be easily exchangeable with the other methods.
       * @see yfiles.graph.IFoldingEdgeConverter
       * @abstract
       */
      excludeFoldingEdge():yfiles.graph.IEdge;
      /**
       * Helper methods that can be used by the implementation of the {@link yfiles.graph.IFoldingEdgeConverter#addFoldingEdge} method to
       * determine the folding edges that are currently part of the view and interconnect the same nodes as the folding edge
       * would, if it was included in the view.
       * <p>
       * This method will yield all {@link yfiles.graph.IFoldingView#isInFoldingState folding edges} that interconnect the same source and target node as that the folding edge
       * would, if it was included in this view. One of the items in the returned collection can be passed to the {@link yfiles.graph.IFoldingEdgeFactory#addToExistingFoldingEdge}
       * method.
       * </p>
       * @param ignoreDirection Whether the direction of the edges should be ignored so that reverse edges are included in the enumeration, too.
       * @returns An collection of currently existing folding edges in the view that interconnects the same source and target node.
       * @abstract
       */
      getExistingFoldingEdges(ignoreDirection:boolean):yfiles.collections.IEnumerable<yfiles.graph.IEdge>;
    }
    var IFoldingEdgeFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface defines a child element of a table or a table element container
     * @interface
     * @implements {yfiles.graph.ILabelOwner}
     */
    export interface IStripe extends Object,yfiles.graph.ILabelOwner{
      /**
       * Returns the layout of <code>stripe</code> in absolute coordinates.
       * <p>
       * Since {@link yfiles.graph.IStripe#layout} is relative to the owning node's left corner, you need to provide an <code>node</code> node to calculate the absolute layout.
       * </p>
       * @param node The node relative to which the stripe layout is calculated. This should be the same node where {@link #table} for <code>stripe</code> is associated
       * to.
       * @returns 
       */
      getAbsoluteBounds?(node:yfiles.graph.INode):yfiles.geometry.Rect;
      /**
       * Gets the insets for this stripe.
       * <p>
       * These insets are applied in addition to any implicit insets provided by the child stripes. To find out the actual insets
       * that are currently in effect, use {@link yfiles.graph.IStripe#actualInsets}.
       * </p>
       * @abstract
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * Returns a live view of the relative layout of the stripe.
       * <p>
       * The layout of a stripe is a rectangle in the coordinate system that is relative to the upper left corner of the parent.
       * Since this method will yield a live view, it is up to the client to copy the values of the instance if a snapshot of the
       * state is needed.
       * </p>
       * @abstract
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Gets the style for this stripe.
       * @abstract
       * @type {yfiles.styles.IStripeStyle}
       */
      style:yfiles.styles.IStripeStyle;
      /**
       * The current changeable dimension of the stripe.
       * <p>
       * This is always the stripe size which is not implicitly determined by other stripes, i.e. the height for {@link yfiles.graph.IRow}
       * instances and the width for {@link yfiles.graph.IColumn} instances.
       * </p>
       * @abstract
       * @type {number}
       */
      size:number;
      /**
       * The minimal permissible value for {@link yfiles.graph.IStripe#size}.
       * <p>
       * This value determines how much the stripe may be shrunk by interactive manipulation.
       * </p>
       * @abstract
       * @type {number}
       */
      minimumSize:number;
      /**
       * The {@link yfiles.graph.ITable} instance where this stripe is installed.
       * @abstract
       * @type {yfiles.graph.ITable}
       */
      table:yfiles.graph.ITable;
      /**
       * Checks whether <code>stripe</code> is the root item of a stripe hierarchy.
       * @see yfiles.graph.IStripe#root
       * @type {boolean}
       */
      isRoot?:boolean;
      /**
       * Gets the index of <code>stripe</code> in its {@link yfiles.graph.IStripe#parentStripe} collection of {@link yfiles.graph.IStripe#childStripes}.
       * <p>
       * The <code>stripe</code> is the <b>GetIndex</b>th child of its parent.
       * </p>
       * @see yfiles.graph.IColumn#childColumns
       * @see yfiles.graph.IRow#childRows
       * @type {number}
       */
      index?:number;
      /**
       * Returns the actual minimum size a stripe can acquire.
       * <p>
       * This is the maximum of {@link yfiles.graph.IStripe#minimumSize} and the horizontal or vertical {@link yfiles.graph.IStripe#insets}.
       * </p>
       * @type {number}
       */
      actualMinSize?:number;
      /**
       * Gets the direct children of <code>stripe</code> in a table hierarchy.
       * <p>
       * For {@link yfiles.graph.IColumn} instances, this returns {@link yfiles.graph.IColumn#childColumns}, for {@link yfiles.graph.IRow} instances, this returns {@link yfiles.graph.IRow#childRows}
       * </p>
       * @see yfiles.graph.IColumn#childColumns
       * @see yfiles.graph.IRow#childRows
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IStripe>}
       */
      childStripes?:yfiles.collections.IEnumerable<yfiles.graph.IStripe>;
      /**
       * Gets the parent of <code>stripe</code> in a table hierarchy.
       * <p>
       * For {@link yfiles.graph.IColumn} instances, this returns {@link yfiles.graph.IColumn#parentColumn}, for {@link yfiles.graph.IRow} instances, this returns {@link yfiles.graph.IRow#parentRow}
       * </p>
       * @see yfiles.graph.IColumn#parentColumn
       * @see yfiles.graph.IRow#parentRow
       * @type {yfiles.graph.IStripe}
       */
      parentStripe?:yfiles.graph.IStripe;
      /**
       * Gets the root of the stripe hierarchy of <code>stripe</code> in a table hierarchy.
       * <p>
       * If the stripe is part of an {@link yfiles.graph.ITable} instance, this returns {@link yfiles.graph.ITable#rootColumn} for {@link yfiles.graph.IColumn} instances and {@link yfiles.graph.IRow#parentRow} for {@link yfiles.graph.ITable#rootRow}
       * instances. Otherwise <code>null</code> is returned.
       * </p>
       * @see yfiles.graph.ITable#rootColumn
       * @see yfiles.graph.ITable#rootRow
       * @see yfiles.graph.IStripe#isRoot
       * @type {yfiles.graph.IStripe}
       */
      root?:yfiles.graph.IStripe;
      /**
       * Returns a collection of all descendants of <code>stripe</code> that are leaves and therefore don't have any child stripes.
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IStripe>}
       */
      leaves?:yfiles.collections.IEnumerable<yfiles.graph.IStripe>;
      /**
       * Returns a collection of all descendants of <code>stripe</code>.
       * <p>
       * The descendants are returned in top to bottom order.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IStripe>}
       */
      descendants?:yfiles.collections.IEnumerable<yfiles.graph.IStripe>;
      /**
       * Gets the insets that are currently in effect for this stripe.
       * <p>
       * This value is determined by the accumulated insets of all ancestor, sibling and descendant insets. These insets may be
       * larger than {@link yfiles.graph.IStripe#insets}.
       * </p>
       * @type {yfiles.geometry.Insets}
       */
      actualInsets?:yfiles.geometry.Insets;
      /**
       * The current actual dimension of the stripe.
       * <p>
       * This is always the stripe size which is actually used for the stripe. For leaf stripes, this is the same as
       * {@link yfiles.graph.IStripe#size}, otherwise, it is the accumulated size of all descendants with taking nested insets into account.
       * </p>
       * @type {number}
       */
      actualSize?:number;
    }
    var IStripe:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Central interface which provides support for tabular data presentation.
     * <p>
     * This interface supports a presentation of data where the nodes of a diagram are organized in a tabular way, i.e., where
     * each node is associated to a specific row and column in the grid-like structure of a table. The {@link yfiles.graph.ITable}
     * together with its associated interfaces {@link yfiles.graph.IColumn} and {@link yfiles.graph.IRow} represents the model for the tabular
     * presentation (or table, for short). It also provides means for altering the model, in particular creation ({@link yfiles.graph.ITable#createChildColumn}
     * and {@link yfiles.graph.ITable#createChildRow}) and {@link yfiles.graph.ITable#remove removal} of columns and rows but also changing their size or appearance.
     * </p>
     * <p>
     * The table structure is backed by a group node. As a consequence, the nodes which should be presented in a tabular
     * fashion need to be set up as children of this group node. Using a group node to hold the content nodes has several
     * advantages: For example, when the group node is moved, the child nodes move accordingly, thus maintaining the visual
     * clue that they are contained in the table group node.
     * </p>
     * <p>
     * Note that the table structure and its content nodes are only loosely coupled: the node's center coordinates determine
     * the row and column that it belongs to.
     * </p>
     * <p>
     * Associating an {@link yfiles.graph.ITable} instance with its backing group node can most conveniently be done by setting a {@link yfiles.styles.TableNodeStyle}
     * instance as style of the associated group node. This style manages the visualization of the individual rows and columns.
     * It also associates its {@link yfiles.styles.TableNodeStyle#table} with the group node by decorating its {@link yfiles.graph.ILookup#lookup} to return the table instance.
     * </p>
     * <p>
     * User interaction is supported by the {@link yfiles.input.TableEditorInputMode} which provides support for intractively resizing
     * rows and columns, re-parenting them, or editing their labels.
     * </p>
     * <p>
     * The table model with all relevant types and their relationships is presented in detail in the section {@link @DGUIDE_PREFIX@/tables.html Tables}.
     * </p>
     * @see yfiles.graph.IStripe
     * @see yfiles.graph.IColumn
     * @see yfiles.graph.IRow
     * @see yfiles.graph.Table
     * @see yfiles.input.TableEditorInputMode
     * @interface
     * @implements {yfiles.graph.ILookup}
     */
    export interface ITable extends Object,yfiles.graph.ILookup{
      /**
       * Add a label to the given owner using the text as the initial label text and label model parameter and style.
       * @param owner the owner to add the label to.
       * @param text the initial text of the label
       * @param [layoutParameter=null] The label model parameter instance to use.
       * @param [style=null] The style to use for the label
       * @param [preferredSize=null] The initial values to use for the {@link #preferredSize}.
       * @param [tag=null] the initial {@link #tag} to assign.
       * @returns the newly created label
       * @see yfiles.graph.ITable#setLabelText
       * @see yfiles.graph.ITable#setLabelLayoutParameter
       * @see yfiles.graph.ITable#setStyle
       * @abstract
       */
      addLabel(owner:yfiles.graph.IStripe,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object):yfiles.graph.ILabel;
      /**
       * Add a label to the given owner using the text as the initial label text and label model parameter and style.
       * @param {Object} options The parameters to pass.
       * @param options.owner the owner to add the label to.
       * @param options.text the initial text of the label
       * @param [options.layoutParameter=null] The label model parameter instance to use.
       * @param [options.style=null] The style to use for the label
       * @param [options.preferredSize=null] The initial values to use for the {@link #preferredSize}.
       * @param [options.tag=null] the initial {@link #tag} to assign.
       * @returns the newly created label
       * @see yfiles.graph.ITable#setLabelText
       * @see yfiles.graph.ITable#setLabelLayoutParameter
       * @see yfiles.graph.ITable#setStyle
       * @abstract
       */
      addLabel(options:{owner:yfiles.graph.IStripe,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object}):yfiles.graph.ILabel;
      /**
       * Adjusts the {@link yfiles.graph.ILabel#preferredSize} property of a label to fit the suggested size of its {@link yfiles.styles.ILabelStyleRenderer}.
       * <p>
       * This implementation uses the {@link yfiles.graph.ILabel#style style's} renderer for the label to determine the preferred rendering size. This is useful
       * after the label's content or style have been changed.
       * </p>
       * @param label The label to adjust the size for.
       * @see yfiles.graph.ILabelDefaults#autoAdjustPreferredSize
       */
      adjustLabelPreferredSize?(label:yfiles.graph.ILabel):void;
      /**
       * Calculates the preferred size of a label with the given properties.
       * @param stripe The item that will own the label.
       * @param style The label style.
       * @param layoutParameter The label model parameter.
       * @param text The text.
       * @param tag The tag for the label.
       * @returns The size as calculated by the {@link }.
       */
      calculateLabelPreferredSize?(stripe:yfiles.graph.IStripe,style:yfiles.styles.ILabelStyle,layoutParameter:yfiles.graph.ILabelModelParameter,text:string,tag:Object):yfiles.geometry.Size;
      /**
       * Convenience method that clears all stripes from a table.
       * <p>
       * All existing rows and columns are cleared from <code>table</code>
       * </p>
       */
      clear?():void;
      /**
       * Create a new column as the <code>index</code>th child of <code>owner</code> with the given parameters.
       * @param owner The owner of the new column.
       * @param [index=null] The position in the child list.
       * @param [width=null] The actual width of the column.
       * @param [minWidth=null] The minimal width of the column.
       * @param [insets=null] The insets of the column.
       * @param [style=null] The style of the column.
       * @param [tag=null] The tag of the column
       * @returns A new column instance
       * @abstract
       */
      createChildColumn(owner:yfiles.graph.IColumn,width?:number,minWidth?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number):yfiles.graph.IColumn;
      /**
       * Create a new column as the <code>index</code>th child of <code>owner</code> with the given parameters.
       * @param {Object} options The parameters to pass.
       * @param options.owner The owner of the new column.
       * @param [options.index=null] The position in the child list.
       * @param [options.width=null] The actual width of the column.
       * @param [options.minWidth=null] The minimal width of the column.
       * @param [options.insets=null] The insets of the column.
       * @param [options.style=null] The style of the column.
       * @param [options.tag=null] The tag of the column
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.childColumns An array of rows to create as children of this row. Each item will be passed to the {@link #createChildColumn}
       * method.
       * @returns A new column instance
       * @abstract
       */
      createChildColumn(options:{owner:yfiles.graph.IColumn,width?:number,minWidth?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number,labels?:Array<any>,childColumns?:Array<any>}):yfiles.graph.IColumn;
      /**
       * Create a new row as the <code>index</code>th child of <code>owner</code> with the given parameters.
       * @param owner The owner of the new row.
       * @param [index=null] The position in the child list.
       * @param [height=null] The actual height of the row.
       * @param [minHeight=null] The minimal height of the row.
       * @param [insets=null] The insets of the row.
       * @param [style=null] The style of the row.
       * @param [tag=null] The tag of the row
       * @returns A new row instance
       * @abstract
       */
      createChildRow(owner:yfiles.graph.IRow,height?:number,minHeight?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number):yfiles.graph.IRow;
      /**
       * Create a new row as the <code>index</code>th child of <code>owner</code> with the given parameters.
       * @param {Object} options The parameters to pass.
       * @param options.owner The owner of the new row.
       * @param [options.index=null] The position in the child list.
       * @param [options.height=null] The actual height of the row.
       * @param [options.minHeight=null] The minimal height of the row.
       * @param [options.insets=null] The insets of the row.
       * @param [options.style=null] The style of the row.
       * @param [options.tag=null] The tag of the row
       * @param options.labels An array of labels to add to the newly created node. Each item will be passed to the {@link #addLabel}
       * method.
       * @param options.childRows An array of rows to create as children of this row. Each item will be passed to the {@link #createChildRow}
       * method.
       * @returns A new row instance
       * @abstract
       */
      createChildRow(options:{owner:yfiles.graph.IRow,height?:number,minHeight?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number,labels?:Array<any>,childRows?:Array<any>}):yfiles.graph.IRow;
      /**
       * Creates and returns a column as last child of <code>table</code>.
       * <p>
       * The column will be a part of this table after the method returns. This will trigger the corresponding events.
       * </p>
       * @param [width=null] The actual width of the column.
       * @param [minWidth=null] The minimal width of the column.
       * @param [insets=null] The insets of the column.
       * @param [style=null] The style of the column.
       * @param [tag=null] The tag of the column
       * @param [index=null] The position in the child list.
       * @returns A newly created column instance
       * @see yfiles.graph.ITable#addStripeCreatedListener
       */
      createColumn?(width?:number,minWidth?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number):yfiles.graph.IColumn;
      /**
       * Creates and returns a column as last child of <code>table</code>.
       * <p>
       * The column will be a part of this table after the method returns. This will trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.width=null] The actual width of the column.
       * @param [options.minWidth=null] The minimal width of the column.
       * @param [options.insets=null] The insets of the column.
       * @param [options.style=null] The style of the column.
       * @param [options.tag=null] The tag of the column
       * @param [options.index=null] The position in the child list.
       * @returns A newly created column instance
       * @see yfiles.graph.ITable#addStripeCreatedListener
       */
      createColumn?(options:{width?:number,minWidth?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number}):yfiles.graph.IColumn;
      /**
       * Convenience method that creates a <code>columns</code> x <code>rows</code>.
       * <p>
       * All existing rows and columns are cleared from <code>table</code>
       * </p>
       * @param columns The number of columns to test.
       * @param rows The number of rows to test.
       */
      createGrid?(columns:number,rows:number):void;
      /**
       * Creates the label model parameter for a given {@link yfiles.graph.ILabelOwner}.
       * <p>
       * This implementation uses the label defaults for the <code>table</code> to {@link yfiles.graph.ILabelDefaults#getLayoutParameterInstance obtain the parameter instance}.
       * </p>
       * @param stripe The item that is the {@link #owner} of the label in question.
       * @returns The default label model parameter to use for newly created labels at the item.
       * @see yfiles.graph.ILabelDefaults#getLayoutParameterInstance
       */
      createLabelLayoutParameter?(stripe:yfiles.graph.IStripe):yfiles.graph.ILabelModelParameter;
      /**
       * Convenience method that delegates to the {@link yfiles.graph.ILabelDefaults#getStyleInstance} method for the given {@link yfiles.graph.IStripe stripe}.
       * @param stripe The item the newly created label will belong to.
       * @returns The style instance to use for newly created stripes.
       */
      createLabelStyle?(stripe:yfiles.graph.IStripe):yfiles.styles.ILabelStyle;
      /**
       * Creates and returns a row as last child of <code>table</code>.
       * <p>
       * The row will be a part of this table after the method returns. This will trigger the corresponding events.
       * </p>
       * @param [height=null] The actual height of the row.
       * @param [minHeight=null] The minimal height of the row.
       * @param [insets=null] The insets of the row.
       * @param [style=null] The style of the row.
       * @param [tag=null] The tag of the row
       * @param [index=null] The position in the child list.
       * @returns A newly created row instance
       * @see yfiles.graph.ITable#addStripeCreatedListener
       */
      createRow?(height?:number,minHeight?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number):yfiles.graph.IRow;
      /**
       * Creates and returns a row as last child of <code>table</code>.
       * <p>
       * The row will be a part of this table after the method returns. This will trigger the corresponding events.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param [options.height=null] The actual height of the row.
       * @param [options.minHeight=null] The minimal height of the row.
       * @param [options.insets=null] The insets of the row.
       * @param [options.style=null] The style of the row.
       * @param [options.tag=null] The tag of the row
       * @param [options.index=null] The position in the child list.
       * @returns A newly created row instance
       * @see yfiles.graph.ITable#addStripeCreatedListener
       */
      createRow?(options:{height?:number,minHeight?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number}):yfiles.graph.IRow;
      /**
       * Convenience method to find a column underneath a certain point.
       * <p>
       * The column are tested in bottom up order.
       * </p>
       * @param node The node this table is currently bound to
       * @param location The location to test.
       * @param filter Additional predicate to further restrict the results.
       * @returns The topmost column that has been found for the location or <code>null</code>.
       */
      findColumn?(node:yfiles.graph.INode,location:yfiles.geometry.Point,filter:(obj:yfiles.graph.IStripe)=>boolean):yfiles.graph.IColumn;
      /**
       * Convenience method to find a row underneath a certain point.
       * <p>
       * The rows are tested in bottom up order.
       * </p>
       * @param node The node this table is currently bound to
       * @param location The location to test.
       * @param filter Additional predicate to further restrict the results.
       * @returns The topmost row that has been found for the location or <code>null</code>.
       */
      findRow?(node:yfiles.graph.INode,location:yfiles.geometry.Point,filter:(obj:yfiles.graph.IStripe)=>boolean):yfiles.graph.IRow;
      /**
       * Convenience method to find all stripes underneath a certain point.
       * <p>
       * The stripes are always returned in bottom up, rows first order. Note that this method does not take any {@link yfiles.view.ICanvasContext#hitTestRadius} into account.
       * </p>
       * @param node The node this table is currently bound to
       * @param location The location to test.
       * @param stripeTypes An enumeration value of {@link } to specify the stripe type.
       * @param filter Additional predicate to further restrict the hit test results.
       * @returns The stripes that have been found for the location or <code>null</code>.
       */
      findStripes?(node:yfiles.graph.INode,location:yfiles.geometry.Point,stripeTypes:yfiles.graph.StripeTypes,filter:(obj:yfiles.graph.IStripe)=>boolean):yfiles.collections.IEnumerable<yfiles.graph.IStripe>;
      /**
       * Return the accumulated insets for the table.
       * @returns 
       */
      getAccumulatedInsets?():yfiles.geometry.Insets;
      /**
       * Calculates the relative bounds of a table cell that is determined by <code>column</code> and <code>row</code>
       * <p>
       * Both <code>column</code> and <code>row</code> must live in the provided <code>table</code> instance. Otherwise, an {@link yfiles.lang.Exception} is thrown.
       * </p>
       * @param column The column that determines the cell
       * @param row The row that determines the cell
       * @returns The relative bounds of a the cell determined by <code>column</code> and <code>row</code>
       * @throws {Stubs.Exceptions.ArgumentError} If <code>column</code> and <code>row</code> are part of a different table instance.
       */
      getCellBounds?(column:yfiles.graph.IColumn,row:yfiles.graph.IRow):yfiles.geometry.Rect;
      /**
       * Removes <code>stripe</code> from its parent container.
       * <p>
       * This method reparents all children of <code>stripe</code> to the parent of the stripe.
       * </p>
       * @param stripe The stripe to remove
       * @abstract
       */
      remove(stripe:yfiles.graph.IStripe):void;
      /**
       * Removes the given label from its owner.
       * <p>
       * This will trigger the corresponding event.
       * </p>
       * @param label the label to remove
       * @abstract
       */
      remove(label:yfiles.graph.ILabel):void;
      /**
       * {@link yfiles.graph.ITable#remove Removes} <code>stripe</code> and all of its descendants from <code>table</code>.
       * <p>
       * This method does not resize any stripes, use {@link yfiles.graph.ITable#removeRecursivelyWithResize} instead.
       * </p>
       * @param stripe The stripe to remove
       */
      removeRecursively?(stripe:yfiles.graph.IStripe):void;
      /**
       * {@link yfiles.graph.ITable#removeWithResize Removes} the given <code>stripe</code> and all of its descendants from <code>table</code> and resizes all affected stripes so that the table size
       * does not change if possible.
       * @param stripe The stripe to remove
       */
      removeRecursivelyWithResize?(stripe:yfiles.graph.IStripe):void;
      /**
       * {@link yfiles.graph.ITable#remove Removes} <code>stripe</code> from <code>table</code> and resizes all affected stripes so that the table size does not change if possible.
       * @param stripe The stripe to remove
       */
      removeWithResize?(stripe:yfiles.graph.IStripe):void;
      /**
       * Sets the label model parameter for the given label.
       * @param label The label.
       * @param layoutParameter The new parameter.
       * @throws {Stubs.Exceptions.ArgumentError} If the parameter cannot be used for this label.
       * @abstract
       */
      setLabelLayoutParameter(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * Sets the preferred size of the label.
       * @param label The label.
       * @param preferredSize The new preferred size.
       * @see yfiles.graph.ILabel#preferredSize
       * @abstract
       */
      setLabelPreferredSize(label:yfiles.graph.ILabel,preferredSize:yfiles.geometry.Size):void;
      /**
       * Sets the label text of the given label.
       * @param label the label to modify
       * @param text the new text of the label
       * @see yfiles.graph.ILabel#text
       * @abstract
       */
      setLabelText(label:yfiles.graph.ILabel,text:string):void;
      /**
       * Sets the minimum size of <code>stripe</code>.
       * <p>
       * This method sets the {@link yfiles.graph.IStripe#minimumSize} of <code>stripe</code>.
       * </p>
       * @param stripe The stripe to change
       * @param minimumSize The new minimum size of the stripe
       * @abstract
       */
      setMinimumSize(stripe:yfiles.graph.IStripe,minimumSize:number):void;
      /**
       * Sets <code>owner</code> as new parent of <code>column</code>, inserting <code>column</code> at the given <code>index</code>.
       * <p>
       * The column will be inserted at position <code>index</code> in the child list of the new parent.
       * </p>
       * @param owner
       * @param column
       * @param [index=null]
       * @abstract
       */
      setParent(owner:yfiles.graph.IColumn,column:yfiles.graph.IColumn,index?:number):void;
      /**
       * Sets <code>owner</code> as new parent of <code>row</code>, inserting <code>row</code> at the given <code>index</code>.
       * <p>
       * The row will be inserted at position <code>index</code> in the child list of the new parent.
       * </p>
       * @param owner
       * @param row
       * @param [index=null]
       * @abstract
       */
      setParent(owner:yfiles.graph.IRow,row:yfiles.graph.IRow,index?:number):void;
      /**
       * Sets the actual size of <code>stripe</code>.
       * <p>
       * This method sets the {@link yfiles.graph.IStripe#size} of <code>stripe</code>.
       * </p>
       * @param stripe The stripe to change
       * @param size The new size of the stripe
       * @abstract
       */
      setSize(stripe:yfiles.graph.IStripe,size:number):void;
      /**
       * Sets the insets of <code>stripe</code>.
       * @param stripe The stripe to change
       * @param insets The new insets of the stripe
       * @abstract
       */
      setStripeInsets(stripe:yfiles.graph.IStripe,insets:yfiles.geometry.Insets):void;
      /**
       * Sets the style of <code>stripe</code>.
       * @param stripe The stripe to change
       * @param style The new style of the stripe
       * @abstract
       */
      setStyle(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):void;
      /**
       * Assigns the given style instance by reference to the label.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param label The label that will be assigned the new style
       * @param style The style instance that will be assigned to the label.
       * @see yfiles.graph.ILabel#style
       * @abstract
       */
      setStyle(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):void;
      /**
       * Gets or sets the insets for this table.
       * <p>
       * These insets are applied in addition to any implicit insets provided by the child stripes.
       * </p>
       * @abstract
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * Gets or sets the relative location of the upper left corner of the table.
       * <p>
       * The actual interpretation of this value depends on the context. If the table is bound to a node, this location is
       * usually interpreted relative to the upper left corner of the node layout.
       * </p>
       * @see yfiles.graph.ITable#layout
       * @abstract
       * @default '0,0'
       * @type {yfiles.geometry.Point}
       */
      relativeLocation:yfiles.geometry.Point;
      /**
       * Gets the relative layout of the table.
       * <p>
       * The upper left corner of the layout always coincides with {@link yfiles.graph.ITable#relativeLocation}.
       * </p>
       * @see yfiles.graph.ITable#relativeLocation
       * @abstract
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the defaults for rows.
       * <p>
       * The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
       * does not influence existing elements.
       * </p>
       * @abstract
       * @type {yfiles.graph.IStripeDefaults}
       */
      rowDefaults:yfiles.graph.IStripeDefaults;
      /**
       * Gets or sets the defaults for Columns.
       * <p>
       * The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
       * does not influence existing elements.
       * </p>
       * @abstract
       * @type {yfiles.graph.IStripeDefaults}
       */
      columnDefaults:yfiles.graph.IStripeDefaults;
      /**
       * Gets the virtual root of the row hierarchy
       * <p>
       * This row should not be queried except for the {@link yfiles.graph.IRow#parentRow} and the associated {@link yfiles.graph.IStripe#table}
       * </p>
       * @see yfiles.graph.ITable#setParent
       * @see yfiles.graph.ITable#createChildRow
       * @abstract
       * @type {yfiles.graph.IRow}
       */
      rootRow:yfiles.graph.IRow;
      /**
       * Gets the virtual root of the column hierarchy
       * <p>
       * This column should not be queried except for the {@link yfiles.graph.IColumn#parentColumn} and the associated {@link yfiles.graph.IStripe#table}
       * </p>
       * @see yfiles.graph.ITable#setParent
       * @see yfiles.graph.ITable#createChildColumn
       * @abstract
       * @type {yfiles.graph.IColumn}
       */
      rootColumn:yfiles.graph.IColumn;
      /**
       * Adds the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
       * @param listener The listener to add.
       * @see yfiles.graph.ITable#addLabel
       * @see yfiles.graph.ITable#removeLabelAddedListener
       * @abstract
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelAdded</code> event that occurs when a label has been added to this graph instance.
       * @param listener The listener to remove.
       * @see yfiles.graph.ITable#addLabel
       * @see yfiles.graph.ITable#addLabelAddedListener
       * @abstract
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph instance.
       * <p>
       * This event will also be triggered, prior to the removal of the {@link yfiles.graph.ILabel#owner owner} of the label.
       * </p>
       * <p>
       * Implementations may choose to use the {@link yfiles.graph.LabelEventArgs} to carry additional label owner information. The {@link yfiles.graph.LabelEventArgs#owner}
       * property will be set to the the owner of the label that owned it before the removal.
       * </p>
       * @param listener The listener to add.
       * @see yfiles.graph.ITable#remove
       * @see yfiles.graph.ITable#removeLabelRemovedListener
       * @abstract
       */
      addLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>LabelRemoved</code> event that occurs when a label has been removed from this graph
       * instance.
       * <p>
       * This event will also be triggered, prior to the removal of the {@link yfiles.graph.ILabel#owner owner} of the label.
       * </p>
       * <p>
       * Implementations may choose to use the {@link yfiles.graph.LabelEventArgs} to carry additional label owner information. The {@link yfiles.graph.LabelEventArgs#owner}
       * property will be set to the the owner of the label that owned it before the removal.
       * </p>
       * @param listener The listener to remove.
       * @see yfiles.graph.ITable#remove
       * @see yfiles.graph.ITable#addLabelRemovedListener
       * @abstract
       */
      removeLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>LabelChanged</code> event that occurs when a label has been changed, for instance, if its style
       * has been replaced.
       * @param listener The listener to add.
       * @see yfiles.graph.ITable#setStyle
       * @see yfiles.graph.ITable#removeLabelChangedListener
       * @abstract
       */
      addLabelChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Removes the given listener for the <code>LabelChanged</code> event that occurs when a label has been changed, for instance, if its
       * style has been replaced.
       * @param listener The listener to remove.
       * @see yfiles.graph.ITable#setStyle
       * @see yfiles.graph.ITable#addLabelChangedListener
       * @abstract
       */
      removeLabelChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Adds the given listener for the <code>StripeCreated</code> event that occurs when a stripe has been added.
       * @param listener The listener to add.
       * @see yfiles.graph.ITable#createChildRow
       * @see yfiles.graph.ITable#createChildColumn
       * @see yfiles.graph.ITable#removeStripeCreatedListener
       * @abstract
       */
      addStripeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Removes the given listener for the <code>StripeCreated</code> event that occurs when a stripe has been added.
       * @param listener The listener to remove.
       * @see yfiles.graph.ITable#createChildRow
       * @see yfiles.graph.ITable#createChildColumn
       * @see yfiles.graph.ITable#addStripeCreatedListener
       * @abstract
       */
      removeStripeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * Adds the given listener for the <code>StripeRemoved</code> event that occurs when a stripe has been removed from its parent.
       * @param listener The listener to add.
       * @see yfiles.graph.ITable#remove
       * @see yfiles.graph.ITable#removeStripeRemovedListener
       * @abstract
       */
      addStripeRemovedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>StripeRemoved</code> event that occurs when a stripe has been removed from its parent.
       * @param listener The listener to remove.
       * @see yfiles.graph.ITable#remove
       * @see yfiles.graph.ITable#addStripeRemovedListener
       * @abstract
       */
      removeStripeRemovedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * Adds the given listener for the <code>StripeChanged</code> event that occurs when a stripe has been changed, for instance, if its
       * style has been replaced.
       * @param listener The listener to add.
       * @see yfiles.graph.ITable#setStyle
       * @see yfiles.graph.ITable#setStripeInsets
       * @see yfiles.graph.ITable#setSize
       * @see yfiles.graph.ITable#setMinimumSize
       * @see yfiles.graph.ITable#removeStripeChangedListener
       * @abstract
       */
      addStripeChangedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>StripeChanged</code> event that occurs when a stripe has been changed, for instance, if its
       * style has been replaced.
       * @param listener The listener to remove.
       * @see yfiles.graph.ITable#setStyle
       * @see yfiles.graph.ITable#setStripeInsets
       * @see yfiles.graph.ITable#setSize
       * @see yfiles.graph.ITable#setMinimumSize
       * @see yfiles.graph.ITable#addStripeChangedListener
       * @abstract
       */
      removeStripeChangedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
    }
    var ITable:{
      /**
       * Convenience method to find a stripe underneath a certain point.
       * @param location The location to test.
       * @param stripeTypes An enumeration value of {@link } to specify the stripe type.
       * @param subregionTypes An enumeration value of {@link } to further restrict the stripe region.
       * @param canvas The canvas that provides necessary context for hit determination, such as {@link #hitTestRadius}.
       * @param filter Additional predicate to further restrict the hit test results.
       * @returns The stripe subregions that have been found for the location or <code>null</code>.
       * @static
       */
      findStripe?(location:yfiles.geometry.Point,stripeTypes:yfiles.graph.StripeTypes,subregionTypes:yfiles.input.StripeSubregionTypes,canvas:yfiles.view.CanvasComponent,filter:(obj:yfiles.input.StripeSubregion)=>boolean):yfiles.input.StripeSubregion;
      /**
       * Used as a callback to find the items underneath a certain point.
       * @param location The location to test.
       * @param stripeTypes An enumeration value of {@link } to specify the stripe type.
       * @param subregionTypes An enumeration value of {@link } to further restrict the stripe region.
       * @param canvas The canvas that provides necessary context for hit determination, such as {@link #hitTestRadius}.
       * @param filter Additional predicate to further restrict the hit test results.
       * @returns The stripe subregions that have been found for the location.
       * @static
       */
      findStripes?(location:yfiles.geometry.Point,stripeTypes:yfiles.graph.StripeTypes,subregionTypes:yfiles.input.StripeSubregionTypes,canvas:yfiles.view.CanvasComponent,filter:(obj:yfiles.input.StripeSubregion)=>boolean):yfiles.collections.IEnumerable<yfiles.input.StripeSubregion>;
      /**
       * Calculates the absolute bounds of a table cell that is determined by <code>column</code> and <code>row</code>
       * <p>
       * Both <code>column</code> and <code>row</code> must live in the same {@link yfiles.graph.ITable} instance that is also present in the <code>tableNode</code>s
       * {@link yfiles.graph.ILookup}. Otherwise, an {@link yfiles.lang.Exception} is thrown.
       * </p>
       * @param tableNode The node where a table is bound to via the node's lookup
       * @param column The column that determines the cell
       * @param row The row that determines the cell
       * @returns The absolute bounds of a the cell determined by <code>column</code> and <code>row</code>
       * @throws {Stubs.Exceptions.ArgumentError} If no table is present in the {@link yfiles.graph.ILookup} of <code>tableNode</code>, or if <code>column</code> and <code>row</code> are part of a different table
       * instance.
       * @static
       */
      getCellBounds?(tableNode:yfiles.graph.INode,column:yfiles.graph.IColumn,row:yfiles.graph.IRow):yfiles.geometry.Rect;
      /**
       * Places <code>node</code> in the center of the table cell determined by <code>column</code> and <code>row</code>.
       * <p>
       * Both <code>column</code> and <code>row</code> must live in the same {@link yfiles.graph.ITable} instance that is also present in the <code>tableNode</code>s
       * {@link yfiles.graph.ILookup}. Otherwise, an {@link yfiles.lang.Exception} is thrown. The cell bounds are determined by
       * {@link yfiles.graph.ITable#getCellBounds}.
       * </p>
       * <p>
       * If <code>reparent</code> is <code>true</code>, the node is made a child node of <code>tableNode</code> if it is no descendant of the table node hierarchy yet.
       * Otherwise, the grouping hierarchy is not changed.
       * </p>
       * <p>
       * If <code>alwaysMove</code> is
       * <code>true</code>, the node is moved even if its center is already contained in the cell, otherwise only when the node layout center
       * is currently outside of the cell bounds.
       * </p>
       * <p>
       * This method does not change the size of the target stripes if they would be too small to contain the whole node layout.
       * Callers must increase the stripe size before or after calling this method, if necessary.
       * </p>
       * @param graph The graph instance where the move should be performed
       * @param node The node that should be moved into the given cell.
       * @param tableNode The node where a table is bound to via the node's lookup
       * @param column The column that determines the cell
       * @param row The row that determines the cell
       * @param [reparent=true] Whether <code>node</code> should also be made part of <code>tableNode</code>'s grouping hierarchy if it is not yet contained in it.
       * @param [alwaysMove=false] Whether <code>node</code> should be moved even if it is already placed in the cell.
       * @throws {Stubs.Exceptions.ArgumentError} If no table is present in the {@link yfiles.graph.ILookup} of <code>tableNode</code>, or if <code>column</code> and <code>row</code> are part of a different table
       * instance.
       * @static
       */
      placeNodeInCell?(graph:yfiles.graph.IGraph,node:yfiles.graph.INode,tableNode:yfiles.graph.INode,column:yfiles.graph.IColumn,row:yfiles.graph.IRow,reparent?:boolean,alwaysMove?:boolean):void;
      /**
       * Places <code>node</code> in the center of the table cell determined by <code>column</code> and <code>row</code>.
       * <p>
       * Both <code>column</code> and <code>row</code> must live in the same {@link yfiles.graph.ITable} instance that is also present in the <code>tableNode</code>s
       * {@link yfiles.graph.ILookup}. Otherwise, an {@link yfiles.lang.Exception} is thrown. The cell bounds are determined by
       * {@link yfiles.graph.ITable#getCellBounds}.
       * </p>
       * <p>
       * If <code>reparent</code> is <code>true</code>, the node is made a child node of <code>tableNode</code> if it is no descendant of the table node hierarchy yet.
       * Otherwise, the grouping hierarchy is not changed.
       * </p>
       * <p>
       * If <code>alwaysMove</code> is
       * <code>true</code>, the node is moved even if its center is already contained in the cell, otherwise only when the node layout center
       * is currently outside of the cell bounds.
       * </p>
       * <p>
       * This method does not change the size of the target stripes if they would be too small to contain the whole node layout.
       * Callers must increase the stripe size before or after calling this method, if necessary.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.graph The graph instance where the move should be performed
       * @param options.node The node that should be moved into the given cell.
       * @param options.tableNode The node where a table is bound to via the node's lookup
       * @param options.column The column that determines the cell
       * @param options.row The row that determines the cell
       * @param [options.reparent=true] Whether <code>node</code> should also be made part of <code>tableNode</code>'s grouping hierarchy if it is not yet contained in it.
       * @param [options.alwaysMove=false] Whether <code>node</code> should be moved even if it is already placed in the cell.
       * @throws {Stubs.Exceptions.ArgumentError} If no table is present in the {@link yfiles.graph.ILookup} of <code>tableNode</code>, or if <code>column</code> and <code>row</code> are part of a different table
       * instance.
       * @static
       */
      placeNodeInCell?(options:{graph:yfiles.graph.IGraph,node:yfiles.graph.INode,tableNode:yfiles.graph.INode,column:yfiles.graph.IColumn,row:yfiles.graph.IRow,reparent?:boolean,alwaysMove?:boolean}):void;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that describes a row in a {@link yfiles.graph.ITable} instance.
     * <p>
     * Default implementations of this interface are created by the factory methods in the {@link yfiles.graph.ITable} interface and
     * its implementations, therefore, it usually shouldn't be necessary to implement this interface yourself.
     * </p>
     * @see yfiles.graph.ITable#createChildRow
     * @interface
     * @implements {yfiles.graph.IStripe}
     */
    export interface IRow extends Object,yfiles.graph.IStripe{
      /**
       * Returns the parent of this row.
       * <p>
       * In a table, each row has a parent, except for the {@link yfiles.graph.ITable#rootRow} hierarchy root.
       * </p>
       * @abstract
       * @type {yfiles.graph.IRow}
       */
      parentRow:yfiles.graph.IRow;
      /**
       * Returns the child rows.
       * <p>
       * This collection may be empty. Each entry in this collection has this instance as {@link yfiles.graph.IRow#parentRow}.
       * </p>
       * @abstract
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IRow>}
       */
      childRows:yfiles.collections.IEnumerable<yfiles.graph.IRow>;
    }
    var IRow:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that describes a column in a {@link yfiles.graph.ITable} instance.
     * <p>
     * Default implementations of this interface are created by the factory methods in the {@link yfiles.graph.ITable} interface and
     * its implementations, therefore, it usually shouldn't be necessary to implement this interface yourself.
     * </p>
     * @see yfiles.graph.ITable#createChildColumn
     * @interface
     * @implements {yfiles.graph.IStripe}
     */
    export interface IColumn extends Object,yfiles.graph.IStripe{
      /**
       * Returns the parent of this column.
       * <p>
       * In a table, each column has a parent, except for the {@link yfiles.graph.ITable#rootColumn} hierarchy root.
       * </p>
       * @abstract
       * @type {yfiles.graph.IColumn}
       */
      parentColumn:yfiles.graph.IColumn;
      /**
       * Returns the child columns.
       * <p>
       * This collection may be empty. Each entry in this collection has this instance as {@link yfiles.graph.IColumn#parentColumn}.
       * </p>
       * @abstract
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IColumn>}
       */
      childColumns:yfiles.collections.IEnumerable<yfiles.graph.IColumn>;
    }
    var IColumn:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum StretchStripeLabelModelPosition{
      /**
       * Encodes a position at the top side of the node interior
       */
      NORTH,
      /**
       * Encodes a position at the right side of the node interior
       */
      EAST,
      /**
       * Encodes a position at the bottom side of the node interior
       */
      SOUTH,
      /**
       * Encodes a position at the left side of the node interior
       */
      WEST
    }
    /**
     * A label model for labels that is especially tailored to be used as a model for the four default positions where the
     * label of a stripe may lie.
     * <p>
     * This label model supports four positions inside of the stripe bounds. The east and west positions are rotated
     * automatically. The {@link yfiles.graph.ILabel#owner} is expected to be an {@link yfiles.graph.IStripe} instance, i.e. the parameter supports only {@link yfiles.graph.IStripe}
     * instances.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface StretchStripeLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class StretchStripeLabelModel {
      /**
       * Creates a new instance of this model with empty insets.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} options.insets The insets to use within the stripe's {@link yfiles.graph.IStripe#layout}.
       * <p>
       * This option sets the {@link yfiles.graph.StretchStripeLabelModel#insets} property on the created object.
       * </p>
       * @param {boolean} options.useActualInsets Whether or not to use the {@link yfiles.graph.IStripe#actualInsets actual insets} or the normal {@link yfiles.graph.IStripe#insets} of a stripe for centering the label inside the stripe header area.
       * <p>
       * This option sets the {@link yfiles.graph.StretchStripeLabelModel#useActualInsets} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets,useActualInsets?:boolean});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates the parameter for the given position.
       * @param position The position.
       * @returns 
       */
      createParameter(position:yfiles.graph.StretchStripeLabelModelPosition):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * Calculates the geometry in form of an {@link yfiles.geometry.IOrientedRectangle} for a given label using the given model
       * parameter.
       * <p>
       * The geometry is calculated in table coordinates, i.e. relative to {@link yfiles.graph.ITable#relativeLocation}.
       * </p>
       * @param label the label to calculate the geometry for
       * @param layoutParameter A parameter that has been created by this model. This is typically the parameter that yielded this instance through its
       * {@link #model} property.
       * @returns An instance that describes the geometry. This is typically an instance designed as a flyweight, so clients should not
       * cache the instance but store the values if they need a snapshot for later use
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StretchStripeLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StretchStripeLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StretchStripeLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StretchStripeLabelModel#insets} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static WEST:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the insets to use within the stripe's {@link yfiles.graph.IStripe#layout}.
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * Gets or sets whether or not to use the {@link yfiles.graph.IStripe#actualInsets actual insets} or the normal {@link yfiles.graph.IStripe#insets} of a stripe for centering the label inside the
       * stripe header area.
       * <p>
       * If <code>true</code> the header insets are used. Default is <code>false</code>
       * </p>
       * @default false
       * @type {boolean}
       */
      useActualInsets:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A specialized subclass of the {@link yfiles.collections.ItemEventArgs.<T>} class that is bound to the {@link yfiles.graph.IStripe} type and carries additional {@link yfiles.graph.IStripe owner}
     * information.
     * <p>
     * Since for some events ({@link yfiles.graph.ITable#addStripeRemovedListener StripeRemoved}) it is unclear what {@link yfiles.graph.IStripe#parentStripe parent} the
     * stripe was owned by before the event, this event can be used to carry exactly that information.
     * </p>
     * @class
     * @extends {yfiles.collections.ItemEventArgs.<yfiles.graph.IStripe>}
     * @final
     */
    export interface StripeEventArgs extends yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>{}
    export class StripeEventArgs {
      /**
       * Initializes a new instance of the {@link yfiles.graph.StripeEventArgs} class.
       * @param stripe The stripe to assign to the {@link #item} property.
       * @param parentStripe The previous parent of the stripe.
       * @param table The table the stripe belonged to previously.
       */
      constructor(stripe:yfiles.graph.IStripe,parentStripe:yfiles.graph.IStripe,table:yfiles.graph.ITable);
      /**
       * Gets the parent of the stripe that owned the stripe before the event happened.
       * @type {yfiles.graph.IStripe}
       */
      parentStripe:yfiles.graph.IStripe;
      /**
       * Gets the table that owned the stripe before the event happened.
       * @type {yfiles.graph.ITable}
       */
      table:yfiles.graph.ITable;
      static $class:yfiles.lang.Class;
    }
    export enum StripeLabelModelPosition{
      /**
       * Encodes a position at the top side of the node interior
       */
      NORTH,
      /**
       * Encodes a position at the right side of the node interior
       */
      EAST,
      /**
       * Encodes a position at the bottom side of the node interior
       */
      SOUTH,
      /**
       * Encodes a position at the left side of the node interior
       */
      WEST
    }
    /**
     * A label model for labels that is especially tailored to be used as a model for the four default positions where the
     * label of a stripe may lie.
     * <p>
     * This label model supports four positions inside of the stripe bounds. The east and west positions are rotated
     * automatically. The {@link yfiles.graph.ILabel#owner} is expected to be an {@link yfiles.graph.IStripe} instance, i.e. the parameter supports only {@link yfiles.graph.IStripe}
     * instances.
     * </p>
     * @class
     * @implements {yfiles.graph.ILabelModel}
     * @implements {yfiles.graph.ILabelModelParameterProvider}
     */
    export interface StripeLabelModel extends Object,yfiles.graph.ILabelModel,yfiles.graph.ILabelModelParameterProvider{}
    export class StripeLabelModel {
      /**
       * Creates a new instance of this model with empty insets.
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.ratio The ratio how far the label should be positioned from the border in the header area.
       * <p>
       * This option sets the {@link yfiles.graph.StripeLabelModel#ratio} property on the created object.
       * </p>
       * @param {boolean} options.useActualInsets A value indicating whether to use the {@link yfiles.graph.IStripe#actualInsets actual insets} or the normal {@link yfiles.graph.IStripe#insets} of a stripe for centering the label inside the stripe header area.
       * <p>
       * This option sets the {@link yfiles.graph.StripeLabelModel#useActualInsets} property on the created object.
       * </p>
       */
      constructor(options?:{ratio?:number,useActualInsets?:boolean});
      /**
       * 
       * @returns 
       */
      createDefaultParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Creates the parameter for the given position.
       * @param position The position.
       * @returns 
       */
      createParameter(position:yfiles.graph.StripeLabelModelPosition):yfiles.graph.ILabelModelParameter;
      /**
       * 
       * @param label
       * @param layoutParameter
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILookup;
      /**
       * Calculates the geometry in form of an {@link yfiles.geometry.IOrientedRectangle} for a given label using the given model
       * parameter.
       * <p>
       * The geometry is calculated in table coordinates, i.e. relative to {@link yfiles.graph.ITable#relativeLocation}.
       * </p>
       * @param label the label to calculate the geometry for
       * @param layoutParameter A parameter that has been created by this model. This is typically the parameter that yielded this instance through its
       * {@link #model} property.
       * @returns An instance that describes the geometry. This is typically an instance designed as a flyweight, so clients should not
       * cache the instance but store the values if they need a snapshot for later use
       */
      getGeometry(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.geometry.IOrientedRectangle;
      /**
       * 
       * @param label
       * @param model
       * @returns 
       */
      getParameters(label:yfiles.graph.ILabel,model:yfiles.graph.ILabelModel):yfiles.collections.IEnumerable<yfiles.graph.ILabelModelParameter>;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StripeLabelModel#ratio} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static NORTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StripeLabelModel#ratio} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static SOUTH:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StripeLabelModel#ratio} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static EAST:yfiles.graph.ILabelModelParameter;
      /**
       * A convenience parameter instance that can be shared between label instances.
       * <p>
       * Trying to change that instances' {@link yfiles.graph.StripeLabelModel#ratio} will raise an {@link yfiles.lang.Exception}.
       * </p>
       * @const
       * @static
       * @type {yfiles.graph.ILabelModelParameter}
       */
      static WEST:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets a value indicating whether to use the {@link yfiles.graph.IStripe#actualInsets actual insets} or the normal {@link yfiles.graph.IStripe#insets} of a stripe for centering the label
       * inside the stripe header area.
       * <p>
       * If <code>true</code> the header insets are used. Default is <code>false</code>
       * </p>
       * @default false
       * @type {boolean}
       */
      useActualInsets:boolean;
      /**
       * Gets or sets the ratio how far the label should be positioned from the border in the header area.
       * <p>
       * 0 means the label is positioned on the outer border, 1 means it is positioned at the inner border of the header or inset
       * area. Default value is 0.5
       * </p>
       * @default 0.5
       * @type {number}
       */
      ratio:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default implementation of the {@link yfiles.graph.ITable} interface.
     * @class
     * @implements {yfiles.graph.ITable}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface Table extends Object,yfiles.graph.ITable,yfiles.lang.ICloneable{}
    export class Table {
      /**
       * Default constructor that creates an empty table
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} [options.insets=null] Sets the insets property.
       * @param {yfiles.geometry.Point} [options.relativeLocation=null] Sets the relativeLocation property.
       * @param {yfiles.graph.IStripeDefaults} options.rowDefaults The defaults for rows.
       * <p>
       * This option sets the {@link yfiles.graph.Table#rowDefaults} property on the created object.
       * </p>
       * @param {yfiles.graph.IStripeDefaults} options.columnDefaults The defaults for Columns.
       * <p>
       * This option sets the {@link yfiles.graph.Table#columnDefaults} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets,relativeLocation?:yfiles.geometry.Point,rowDefaults?:yfiles.graph.IStripeDefaults,columnDefaults?:yfiles.graph.IStripeDefaults});
      /**
       * 
       * @param owner
       * @param text
       * @param [layoutParameter=null]
       * @param [style=null]
       * @param [preferredSize=null]
       * @param [tag=null]
       * @returns 
       */
      addLabel(owner:yfiles.graph.IStripe,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object):yfiles.graph.ILabel;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.owner
       * @param options.text
       * @param [options.layoutParameter=null]
       * @param [options.style=null]
       * @param [options.preferredSize=null]
       * @param [options.tag=null]
       * @returns 
       */
      addLabel(options:{owner:yfiles.graph.IStripe,text:string,layoutParameter?:yfiles.graph.ILabelModelParameter,style?:yfiles.styles.ILabelStyle,preferredSize?:yfiles.geometry.Size,tag?:Object}):yfiles.graph.ILabel;
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Called during {@link yfiles.graph.Table#clone} to create a copy of <code>column</code>
       * <p>
       * If <code>column</code> implements the {@link yfiles.lang.ICloneable} interface, the <code>column</code> instance is
       * {@link yfiles.lang.ICloneable#clone}d, otherwise, the original instance is returned unchanged.
       * </p>
       * @param column The column that should be copied during a clone operation.
       * @returns A clone of <code>column</code> if possible, otherwise the original reference to <code>column</code>
       * @protected
       */
      copyColumn(column:yfiles.graph.IColumn):yfiles.graph.IColumn;
      /**
       * Called during {@link yfiles.graph.Table#clone} to create a copy of <code>originalDefaults</code>
       * <p>
       * This implementation returns an unchanged reference to <code>originalDefaults</code>. If you need to create a deep clone of
       * <code>originalDefaults</code>, you'll have to perform the cloning yourself.
       * </p>
       * @param originalDefaults The default settings that should be copied during a clone operation.
       * @returns The original reference to <code>originalDefaults</code>
       * @protected
       */
      copyDefaults(originalDefaults:yfiles.graph.IStripeDefaults):yfiles.graph.IStripeDefaults;
      /**
       * Called during {@link yfiles.graph.Table#clone} to create a copy of <code>row</code>
       * <p>
       * If <code>row</code> implements the {@link yfiles.lang.ICloneable} interface, the <code>row</code> instance is
       * {@link yfiles.lang.ICloneable#clone}d, otherwise, the original instance is returned unchanged.
       * </p>
       * @param row The row that should be copied during a clone operation.
       * @returns A clone of <code>row</code> if possible, otherwise the original reference to <code>row</code>
       * @protected
       */
      copyRow(row:yfiles.graph.IRow):yfiles.graph.IRow;
      /**
       * 
       * @param owner
       * @param [width=null]
       * @param [minWidth=null]
       * @param [insets=null]
       * @param [style=null]
       * @param [tag=null]
       * @param [index=null]
       * @returns 
       */
      createChildColumn(owner:yfiles.graph.IColumn,width?:number,minWidth?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number):yfiles.graph.IColumn;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.owner
       * @param [options.width=null]
       * @param [options.minWidth=null]
       * @param [options.insets=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @param [options.index=null]
       * @returns 
       */
      createChildColumn(options:{owner:yfiles.graph.IColumn,width?:number,minWidth?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number}):yfiles.graph.IColumn;
      /**
       * 
       * @param owner
       * @param [height=null]
       * @param [minHeight=null]
       * @param [insets=null]
       * @param [style=null]
       * @param [tag=null]
       * @param [index=null]
       * @returns 
       */
      createChildRow(owner:yfiles.graph.IRow,height?:number,minHeight?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number):yfiles.graph.IRow;
      /**
       * 
       * @param {Object} options The parameters to pass.
       * @param options.owner
       * @param [options.height=null]
       * @param [options.minHeight=null]
       * @param [options.insets=null]
       * @param [options.style=null]
       * @param [options.tag=null]
       * @param [options.index=null]
       * @returns 
       */
      createChildRow(options:{owner:yfiles.graph.IRow,height?:number,minHeight?:number,insets?:yfiles.geometry.Insets,style?:yfiles.styles.IStripeStyle,tag?:Object,index?:number}):yfiles.graph.IRow;
      /**
       * Callback method that creates the Column defaults.
       * @returns A new instance of the {@link } class which is used for newly created columns.
       * @protected
       */
      createColumnDefaults():yfiles.graph.IStripeDefaults;
      /**
       * Callback method that creates the row defaults.
       * @returns A new instance of the {@link } class that is used for newly created rows.
       * @protected
       */
      createRowDefaults():yfiles.graph.IStripeDefaults;
      /**
       * Installs the undo support for use with all tables that are bound to nodes in <code>graph</code>.
       * <p>
       * This method queries the <code>graph</code> for its undo support dynamically whenever undo support is queried in the table. In
       * contrast to {@link yfiles.graph.Table#installStaticUndoSupport} the undo support installed with this method will thus always be
       * the one which is the currently available support for the <code>graph</code>.
       * </p>
       * @param graph The graph where the nodes exist or will be created/changed and where the undo support should come from.
       * @see yfiles.graph.Table#uninstallDynamicUndoSupport
       * @see yfiles.graph.Table#installStaticUndoSupport
       * @static
       */
      static installDynamicUndoSupport(graph:yfiles.graph.IGraph):void;
      /**
       * Installs the undo support for use with all tables that are bound to nodes in <code>graph</code>.
       * <p>
       * This methods installs the undo support which manages the <code>graph</code> at the time this method is called. In contrast to {@link yfiles.graph.Table#installDynamicUndoSupport}
       * the installed undo support will not change when the <code>graph</code>'s undo support changes.
       * </p>
       * <p>
       * This method adds undo support both for existing nodes in <code>graph</code> as well as for newly created or changed nodes.
       * </p>
       * @param graph The graph where the nodes exist or will be created/changed.
       * @see yfiles.graph.Table#uninstallStaticUndoSupport
       * @see yfiles.graph.Table#installDynamicUndoSupport
       * @static
       */
      static installStaticUndoSupport(graph:yfiles.graph.IGraph):void;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Called after a label has been added to a stripe.
       * <p>
       * This method raises the {@link yfiles.graph.Table#addLabelAddedListener LabelAdded} event.
       * </p>
       * @param label The label that has just been added.
       * @protected
       */
      onLabelAdded(label:yfiles.graph.ILabel):void;
      /**
       * Callback that is invoked after a label has changed.
       * <p>
       * This method raises the {@link yfiles.graph.Table#addLabelChangedListener LabelChanged} event.
       * </p>
       * @param label The label that has changed.
       * @protected
       */
      onLabelChanged(label:yfiles.graph.ILabel):void;
      /**
       * Called after a label has been removed from its stripe.
       * <p>
       * This method raises the {@link yfiles.graph.Table#addLabelRemovedListener LabelRemoved} event.
       * </p>
       * @param label The label that has just been removed.
       * @param owner The old owner of the label.
       * @protected
       */
      onLabelRemoved(owner:yfiles.graph.ILabelOwner,label:yfiles.graph.ILabel):void;
      /**
       * Callback that is invoked after a stripe has changed.
       * <p>
       * This will trigger the {@link yfiles.graph.Table#addStripeChangedListener StripeChanged} event.
       * </p>
       * @param owner The original owner
       * @param stripe The stripe that has changed.
       * @param oldTable The original table owner, in case the stripe has been moved between different table instances
       * @protected
       */
      onStripeChanged(owner:yfiles.graph.IStripe,stripe:yfiles.graph.IStripe,oldTable:yfiles.graph.ITable):void;
      /**
       * Called after a stripe has been created and added to a table.
       * <p>
       * This method raises the {@link yfiles.graph.Table#addStripeCreatedListener StripeCreated} event.
       * </p>
       * @param stripe The stripe that has just been created.
       * @protected
       */
      onStripeCreated(stripe:yfiles.graph.IStripe):void;
      /**
       * Called after a stripe has been removed from its owner.
       * <p>
       * This method raises the {@link yfiles.graph.Table#addStripeRemovedListener StripeRemoved} event.
       * </p>
       * @param stripe The stripe that has just been removed.
       * @param owner The old owner of the stripe.
       * @protected
       */
      onStripeRemoved(owner:yfiles.graph.IStripe,stripe:yfiles.graph.IStripe):void;
      /**
       * Removes the given label from its owner.
       * <p>
       * This will trigger the corresponding event.
       * </p>
       * @param label the label to remove
       * @throws {Stubs.Exceptions.ArgumentError} If the label is not associated with this table instance.
       */
      remove(label:yfiles.graph.ILabel):void;
      /**
       * 
       * @param stripe
       */
      remove(stripe:yfiles.graph.IStripe):void;
      /**
       * Sets the label model parameter for the given label.
       * @param label The label.
       * @param layoutParameter The new parameter.
       * @throws {Stubs.Exceptions.ArgumentError} If the parameter cannot be used for this label or if the label is not associated with this table instance.
       */
      setLabelLayoutParameter(label:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):void;
      /**
       * Sets the preferred size of the label.
       * @param label The label.
       * @param preferredSize The new preferred size.
       * @throws {Stubs.Exceptions.ArgumentError} If the label is not associated with this table instance.
       * @see yfiles.graph.ILabel#preferredSize
       */
      setLabelPreferredSize(label:yfiles.graph.ILabel,preferredSize:yfiles.geometry.Size):void;
      /**
       * Sets the label text of the given label.
       * @param label the label to modify
       * @param text the new text of the label
       * @throws {Stubs.Exceptions.ArgumentError} If the label is not associated with this table instance.
       * @see yfiles.graph.ILabel#text
       */
      setLabelText(label:yfiles.graph.ILabel,text:string):void;
      /**
       * 
       * @param stripe
       * @param minimumSize
       */
      setMinimumSize(stripe:yfiles.graph.IStripe,minimumSize:number):void;
      /**
       * 
       * @param owner
       * @param column
       * @param [index=null]
       */
      setParent(owner:yfiles.graph.IColumn,column:yfiles.graph.IColumn,index?:number):void;
      /**
       * 
       * @param owner
       * @param row
       * @param [index=null]
       */
      setParent(owner:yfiles.graph.IRow,row:yfiles.graph.IRow,index?:number):void;
      /**
       * 
       * @param stripe
       * @param size
       */
      setSize(stripe:yfiles.graph.IStripe,size:number):void;
      /**
       * 
       * @param stripe
       * @param insets
       */
      setStripeInsets(stripe:yfiles.graph.IStripe,insets:yfiles.geometry.Insets):void;
      /**
       * Assigns the given style instance by reference to the label.
       * <p>
       * Style instances can be shared.
       * </p>
       * @param label The label that will be assigned the new style
       * @param style The style instance that will be assigned to the label.
       * @see yfiles.graph.ILabel#style
       */
      setStyle(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):void;
      /**
       * 
       * @param stripe
       * @param style
       */
      setStyle(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):void;
      /**
       * Uninstalls the undo support that has previously been installed with {@link yfiles.graph.Table#installDynamicUndoSupport}
       * @param graph The graph where the tables are installed for which the undo support should be cleared.
       * @static
       */
      static uninstallDynamicUndoSupport(graph:yfiles.graph.IGraph):void;
      /**
       * Uninstalls the undo support that has previously been installed with {@link yfiles.graph.Table#installStaticUndoSupport}
       * @param graph The graph where the tables are installed for which the undo support should be cleared.
       * @static
       */
      static uninstallStaticUndoSupport(graph:yfiles.graph.IGraph):void;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.IStripe}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the stripe lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_STRIPE_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.IColumn}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the column lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_COLUMN_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * Gets an {@link yfiles.graph.IContextLookup} that provides the default implementations returned by an
       * {@link yfiles.graph.IRow}'s lookup.
       * <p>
       * The provided default implementations can for example be used as fallback when the row lookup is wrapped.
       * </p>
       * @static
       * @type {yfiles.graph.IContextLookup}
       */
      static DEFAULT_ROW_LOOKUP:yfiles.graph.IContextLookup;
      /**
       * The toplevel {@link yfiles.graph.IRow}s in this table.
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IRow>}
       */
      rows:yfiles.collections.IEnumerable<yfiles.graph.IRow>;
      /**
       * The toplevel {@link yfiles.graph.IColumn}s in this table.
       * @type {yfiles.collections.IEnumerable.<yfiles.graph.IColumn>}
       */
      columns:yfiles.collections.IEnumerable<yfiles.graph.IColumn>;
      /**
       * 
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * 
       * @default '0,0'
       * @type {yfiles.geometry.Point}
       */
      relativeLocation:yfiles.geometry.Point;
      /**
       * Gets the relative layout of the table.
       * <p>
       * The upper left corner of the layout always coincides with {@link yfiles.graph.Table#relativeLocation}.
       * </p>
       * @see yfiles.graph.Table#relativeLocation
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * 
       * @param listener
       */
      addStripeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeStripeCreatedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.IStripe>)=>void):void;
      /**
       * 
       * @param listener
       */
      addStripeRemovedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeStripeRemovedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addStripeChangedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeStripeChangedListener(listener:(sender:Object,evt:yfiles.graph.StripeEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelAddedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelRemovedListener(listener:(sender:Object,evt:yfiles.graph.LabelEventArgs)=>void):void;
      /**
       * 
       * @param listener
       */
      addLabelChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * 
       * @param listener
       */
      removeLabelChangedListener(listener:(sender:Object,evt:yfiles.collections.ItemEventArgs<yfiles.graph.ILabel>)=>void):void;
      /**
       * Gets or sets the defaults for rows.
       * <p>
       * The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
       * does not influence existing elements.
       * </p>
       * <p>
       * If no value is set explicitly a default will be created using {@link yfiles.graph.Table#createRowDefaults} upon first read
       * access.
       * </p>
       * @type {yfiles.graph.IStripeDefaults}
       */
      rowDefaults:yfiles.graph.IStripeDefaults;
      /**
       * Gets or sets the defaults for Columns.
       * <p>
       * The settings that are obtained from the instance influence newly created elements only. Setting different defaults later
       * does not influence existing elements.
       * </p>
       * <p>
       * If no value is set explicitly a default will be created using {@link yfiles.graph.Table#createColumnDefaults} upon first read
       * access.
       * </p>
       * @type {yfiles.graph.IStripeDefaults}
       */
      columnDefaults:yfiles.graph.IStripeDefaults;
      /**
       * 
       * @type {yfiles.graph.IRow}
       */
      rootRow:yfiles.graph.IRow;
      /**
       * 
       * @type {yfiles.graph.IColumn}
       */
      rootColumn:yfiles.graph.IColumn;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface used by {@link yfiles.graph.ITable} and the like to declare and obtain the defaults for stripes and their labels.
     * @see yfiles.graph.ITable#rowDefaults
     * @see yfiles.graph.ITable#columnDefaults
     * @interface
     */
    export interface IStripeDefaults extends Object{
      /**
       * Factory method that returns a style instance for use with newly created stripes.
       * <p>
       * Most implementations will yield either, a {@link yfiles.lang.ICloneable#clone clone} of or the {@link yfiles.graph.IStripeDefaults#style} property, if {@link yfiles.graph.IStripeDefaults#shareStyleInstance} is enabled, but they might use more complicated
       * logic, too.
       * </p>
       * @returns The style to use, which for most implementations is either a {@link #clone clone} of or the {@link #style} property, if {@link #shareStyleInstance} is enabled.
       * @abstract
       */
      getStyleInstance():yfiles.styles.IStripeStyle;
      /**
       * Gets or sets the default insets for a stripe.
       * @abstract
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * Gets or sets the default stripe size.
       * @see yfiles.graph.IStripe#size
       * @abstract
       * @type {number}
       */
      size:number;
      /**
       * Gets or sets the default minimum stripe size.
       * @see yfiles.graph.IStripe#minimumSize
       * @abstract
       * @type {number}
       */
      minimumSize:number;
      /**
       * Gets or sets the defaults for labels at stripes.
       * @abstract
       * @type {yfiles.graph.ILabelDefaults}
       */
      labels:yfiles.graph.ILabelDefaults;
      /**
       * Gets or sets the style to use for stripes.
       * <p>
       * Depending on the setting of {@link yfiles.graph.IStripeDefaults#shareStyleInstance}, the {@link yfiles.graph.IStripeDefaults#getStyleInstance} method should return a {@link yfiles.lang.ICloneable#clone clone} of this
       * instance or the very same instance.
       * </p>
       * @see yfiles.graph.IStripeDefaults#shareStyleInstance
       * @abstract
       * @type {yfiles.styles.IStripeStyle}
       */
      style:yfiles.styles.IStripeStyle;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.graph.IStripeDefaults#style} instance should be shared referentially or {@link yfiles.lang.ICloneable#clone cloned} upon a call to
       * {@link yfiles.graph.IStripeDefaults#getStyleInstance}.
       * @see yfiles.graph.IStripeDefaults#getStyleInstance
       * @see yfiles.graph.IStripeDefaults#style
       * @abstract
       * @type {boolean}
       */
      shareStyleInstance:boolean;
    }
    var IStripeDefaults:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A canonic implementation of the {@link yfiles.graph.IStripeDefaults} interface.
     * @class
     * @implements {yfiles.graph.IStripeDefaults}
     */
    export interface StripeDefaults extends Object,yfiles.graph.IStripeDefaults{}
    export class StripeDefaults {
      /**
       * Default constructor
       * <p>
       * This constructor sets the style to {@link yfiles.styles.VoidNodeStyle#INSTANCE} and the insets to {@link yfiles.geometry.Insets#EMPTY}
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {number} options.size The default stripe size.
       * <p>
       * This option sets the {@link yfiles.graph.StripeDefaults#size} property on the created object.
       * </p>
       * @param {number} options.minimumSize The default minimum stripe size.
       * <p>
       * This option sets the {@link yfiles.graph.StripeDefaults#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.graph.ILabelDefaults} [options.labels=null] Sets the labels property.
       * @param {yfiles.styles.IStripeStyle} [options.style=null] Sets the style property.
       * @param {boolean} [options.shareStyleInstance=null] Sets the shareStyleInstance property.
       */
      constructor(options?:{size?:number,minimumSize?:number,labels?:yfiles.graph.ILabelDefaults,style?:yfiles.styles.IStripeStyle,shareStyleInstance?:boolean});
      /**
       * 
       * @returns 
       */
      getStyleInstance():yfiles.styles.IStripeStyle;
      /**
       * Gets or sets the default insets for a stripe.
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * Gets or sets the default stripe size.
       * @see yfiles.graph.IStripe#size
       * @default 100
       * @type {number}
       */
      size:number;
      /**
       * Gets or sets the default minimum stripe size.
       * @see yfiles.graph.IStripe#minimumSize
       * @default 10
       * @type {number}
       */
      minimumSize:number;
      /**
       * 
       * @type {yfiles.graph.ILabelDefaults}
       */
      labels:yfiles.graph.ILabelDefaults;
      /**
       * 
       * @type {yfiles.styles.IStripeStyle}
       */
      style:yfiles.styles.IStripeStyle;
      /**
       * 
       * @default true
       * @type {boolean}
       */
      shareStyleInstance:boolean;
      static $class:yfiles.lang.Class;
    }
  }export namespace geometry{
    export enum PathType{
      /**
       * The type that is used to move the {@link yfiles.view.Stroke} to the next position.
       * <p>
       * This type uses one coordinate pair.
       * </p>
       */
      MOVE_TO,
      /**
       * The type that is used to add a line from the current position to the next position.
       * <p>
       * This type uses one coordinate pair.
       * </p>
       */
      LINE_TO,
      /**
       * The type that is used to add a quadratic Bézier curve from the current position to the next position using one control
       * point.
       * <p>
       * This type uses two coordinate pairs.
       * </p>
       */
      QUAD_TO,
      /**
       * The type that is used to add a cubic Bézier curve from the current position to the next position using two intermediate
       * control points.
       * <p>
       * This type uses three coordinate pairs.
       * </p>
       */
      CUBIC_TO,
      /**
       * The type that is used to add a line from the current position to the last {@link yfiles.geometry.PathType#MOVE_TO} position.
       * <p>
       * This type uses no coordinate pair.
       * </p>
       */
      CLOSE
    }
    /**
     * A class that is used by {@link yfiles.geometry.GeneralPath} to iterate over the elements the path is made up of.
     * @see yfiles.geometry.GeneralPath#createCursor
     * @class
     */
    export interface GeneralPathCursor extends Object{}
    export class GeneralPathCursor {
      /**
       * Places the coordinates for the current type into the given array.
       * <p>
       * The array needs to have a length of at least 6, since {@link yfiles.geometry.PathType#CUBIC_TO} needs six coordinates. If the
       * path has been flattened a length of two suffices.
       * </p>
       * @param coordinates
       * @returns The current type of the path element.
       * @see yfiles.geometry.GeneralPathCursor#getCurrentEndPoint
       */
      getCurrent(coordinates:number[]):yfiles.geometry.PathType;
      /**
       * Places the current end coordinates of the last path element into the array.
       * @param coordinates An array with length at least 2.
       * @returns The type of the current element.
       */
      getCurrentEndPoint(coordinates:number[]):yfiles.geometry.PathType;
      /**
       * Advances the cursor to the next position if possible.
       * @returns Whether the cursor has been moved successfully. <code>false</code> if the cursor has been moved beyond the end of the path.
       */
      moveNext():boolean;
      /**
       * Moves the cursor to the previous position in the path.
       * @returns Whether the move was successful.
       */
      movePrevious():boolean;
      /**
       * Reset the cursor to point to the beginning of the path.
       */
      reset():void;
      /**
       * Resets the cursor to point to the last element in the path.
       * @returns Whether the cursor has been moved successfully.
       */
      toLast():boolean;
      /**
       * Returns the current position of the cursor.
       * @type {number}
       */
      index:number;
      /**
       * Returns the type of the path at the position of the cursor.
       * @type {yfiles.geometry.PathType}
       */
      pathType:yfiles.geometry.PathType;
      /**
       * Returns the current x coordinate of the last path element.
       * @type {number}
       */
      currentEndPointX:number;
      /**
       * Gets the current end point of the last path element.
       * @type {yfiles.geometry.Point}
       */
      currentEndPoint:yfiles.geometry.Point;
      /**
       * Returns the current y coordinate of the last path element.
       * @type {number}
       */
      currentEndPointY:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Models a path that consists of lines and Bézier curves.
     * <p>
     * This class uses double precision coordinates to model a virtual pen that can be moved across the world coordinate system
     * and which can be used to draw lines and Bézier curves. This class provides many geometric utility methods, such as
     * calculating the bounds, hit tests, containment tests, curve flattening, path transformation, path modification, and path
     * iteration.
     * </p>
     * @class
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface GeneralPath extends Object,yfiles.lang.ICloneable{}
    export class GeneralPath {
      /**
       * Creates a new instance with given initial capacity.
       * @param [capacity=16] The number of elements for which storage will be allocated initially.
       */
      constructor(capacity?:number);
      /**
       * Appends parts of the contents of the given path to this path.
       * <p>
       * If <code>connect</code> is <code>true</code>, an initial {@link yfiles.geometry.PathType#MOVE_TO} in <code>other</code> will be transformed into a
       * {@link yfiles.geometry.PathType#LINE_TO}.
       * </p>
       * @param other The path to append to this path.
       * @param connect Whether to connect the path to the end of this path.
       * @param beginIndex The first element to be copied from <code>other</code>, inclusively.
       * @param endIndex The last element to be copied from <code>other</code>, exclusively.
       */
      append(other:yfiles.geometry.GeneralPath,beginIndex:number,endIndex:number,connect:boolean):void;
      /**
       * Appends the contents of the given path to this path.
       * <p>
       * If <code>connect</code> is <code>true</code>, an initial {@link yfiles.geometry.PathType#MOVE_TO} in <code>other</code> will be transformed into a
       * {@link yfiles.geometry.PathType#LINE_TO}.
       * </p>
       * @param other The path to append to this path.
       * @param connect Whether to connect the path to the end of this path.
       */
      append(other:yfiles.geometry.GeneralPath,connect:boolean):void;
      /**
       * Appends an ellipse using the given bounding box to this instance.
       * @param bounds The bounds of the ellipse.
       * @param connect Whether to initially {@link #lineTo} the ellipses lowest point.
       */
      appendEllipse(bounds:yfiles.geometry.IRectangle,connect:boolean):void;
      /**
       * Appends an {@link yfiles.geometry.IOrientedRectangle} to this path instance.
       * @param rectangle The {@link } to get the coordinates from.
       * @param connect Whether to use a {@link #lineTo} to the anchor corner.
       */
      appendOrientedRectangle(rectangle:yfiles.geometry.IOrientedRectangle,connect:boolean):void;
      /**
       * Appends a rectangle to this path instance.
       * @param rectangle The rectangle to get the coordinates from.
       * @param connect Whether to use a {@link #lineTo} to the upper left corner.
       */
      appendRectangle(rectangle:yfiles.geometry.IRectangle,connect:boolean):void;
      /**
       * Checks whether the point lies inside the area that is described by this path instance.
       * <p>
       * Note that this will only work as expected for closed paths. The epsilon is used for internally flattening Bézier
       * curves.
       * </p>
       * @param point The coordinates of the point to test.
       * @param [eps=0.5] The value to use for flattening Bézier curves during the test. <code>0.5</code> is the default value.
       * @returns Whether the point lies within the area described by this path.
       */
      areaContains(point:yfiles.geometry.IPoint,eps?:number):boolean;
      /**
       * Resets this path to be an empty path.
       */
      clear():void;
      /**
       * Creates a clone of this instance, copying the path information to the new instance.
       * @returns An exact clone of this instance.
       */
      clone():Object;
      /**
       * Appends a {@link yfiles.geometry.PathType#CLOSE} operation to the path elements, creating a line to the last {@link yfiles.geometry.PathType#MOVE_TO}
       * position.
       * <p>
       * This draws a line from the current end point to the last {@link yfiles.geometry.GeneralPath#moveTo} position, closing the
       * current sub path.
       * </p>
       */
      close():void;
      /**
       * Compresses the internal data structures so that the amount of memory used by this instance is minimal with respect to
       * the information stored in it.
       */
      compress():void;
      /**
       * Creates a cursor for iterating over the elements of this path.
       * <p>
       * The cursor is fail-fast, i.e. if the path's structure is modified after the construction of the cursor any cursor
       * operation will fail.
       * </p>
       * @returns A cursor to iterate over this path.
       */
      createCursor():yfiles.geometry.GeneralPathCursor;
      /**
       * Creates a new path based on this one where corners are smoothed with a quadratic Bézier arc starting at <code>smoothingLength</code>
       * away from the corner.
       * @param smoothingLength Distance from the corner where the arc begins.
       * @returns A new path, with its corners smoothed.
       */
      createSmoothedPath(smoothingLength:number):yfiles.geometry.GeneralPath;
      /**
       * Creates an SVG <code>path</code> element using the provided parameters.
       * @param [transform=null] The matrix to apply to the path data.
       * @returns A {@link  path} that displays this instance using the provided attributes.
       */
      createSvgPath(transform?:yfiles.geometry.Matrix):SVGPathElement;
      /**
       * Creates the data for a SVG <code>path</code> element that corresponds to this instance with the provided transform.
       * @param [transform=null] The transform to apply to the created data.
       * @returns The path data for an SVG <code>path</code> element that corresponds to this instance with the provided transform.
       */
      createSvgPathData(transform?:yfiles.geometry.Matrix):string;
      /**
       * Creates a new {@link yfiles.geometry.GeneralPath} that is a copy of this path with all geometry transformed using the supplied
       * matrix.
       * @param transform The matrix to multiply the geometry with.
       * @returns A new path.
       */
      createTransformedPath(transform:yfiles.geometry.Matrix):yfiles.geometry.GeneralPath;
      /**
       * Crops the path after the position indicated by the cursor.
       * <p>
       * This will remove all path operations that follow the position indicated by the cursor.
       * </p>
       * @param cursor The cursor that depicts a position in this path.
       */
      cropAfter(cursor:yfiles.geometry.GeneralPathCursor):void;
      /**
       * Crops the path before the position indicated by the cursor.
       * <p>
       * This will remove all path operations that precede the position indicated by the cursor.
       * </p>
       * @param cursor The cursor that depicts a position in this path.
       */
      cropBefore(cursor:yfiles.geometry.GeneralPathCursor):void;
      /**
       * Appends a {@link yfiles.geometry.PathType#CUBIC_TO cubic Bézier curve} to the path elements.
       * <p>
       * This draws a cubic Bézier curve from the current end point to the given position using the two given control points.
       * </p>
       * @param c1x The x coordinate of the first Bézier control point.
       * @param c1y The y coordinate of the first Bézier control point.
       * @param c2x The x coordinate of the second Bézier control point.
       * @param c2y The y coordinate of the second Bézier control point.
       * @param x The x coordinate of the curve's end point.
       * @param y The y coordinate of the curve's end point.
       */
      cubicTo(c1x:number,c1y:number,c2x:number,c2y:number,x:number,y:number):void;
      /**
       * Appends a {@link yfiles.geometry.PathType#CUBIC_TO cubic Bézier curve} to the path elements.
       * <p>
       * This draws a cubic Bézier curve from the current end point to the given position using the two given control points.
       * </p>
       * @param c1 The coordinates of the first Bézier control point.
       * @param c2 The coordinates of the second Bézier control point.
       * @param end The coordinates of the curve's end point.
       */
      cubicTo(c1:yfiles.geometry.IPoint,c2:yfiles.geometry.IPoint,end:yfiles.geometry.IPoint):void;
      /**
       * Finds an intersection point between a line and this {@link yfiles.geometry.GeneralPath}.
       * <p>
       * The value returned will be the smallest positive value smaller than 1 such that the point depicted by <code>start + result * (end - start)</code>
       * is an intersection point between the line and the path.
       * </p>
       * @param start The coordinates of the first point of the line.
       * @param end The coordinates of the second point of the line.
       * @param [eps=0.5] The value to use for interpolating Bézier curves.
       * @returns The factor to calculate the intersection point or <code>double.PositiveInfinity</code>.
       */
      findLineIntersection(start:yfiles.geometry.Point,end:yfiles.geometry.Point,eps?:number):number;
      /**
       * Finds the first intersection of a ray with this {@link yfiles.geometry.GeneralPath} using a given epsilon for flattening Bézier
       * curves contained in the path.
       * <p>
       * The ray is treated like an infinite line starting at the given anchor point and continuing in the direction of rayX,
       * rayY. The ray vector does not need to be normalized. The value returned will be the smallest positive value such that
       * the point depicted by <code>(anchorX + result * rayX, anchorY + result * rayY)</code> is an intersection point between the ray and
       * the path.
       * </p>
       * @param anchor The coordinates of the anchor point of the ray.
       * @param direction The coordinates of the direction vector of the ray.
       * @param [eps=0.5] The epsilon value that is used for the accuracy with which the implementation should test Bézier curves within the path
       * for intersections.
       * @returns The factor to calculate the intersection point or <code>double.PositiveInfinity</code>.
       */
      findRayIntersection(anchor:yfiles.geometry.Point,direction:yfiles.geometry.Point,eps?:number):number;
      /**
       * Creates a new path from this instance flattening all Bézier curves using the given epsilon.
       * <p>
       * This will create a path that consists of {@link yfiles.geometry.PathType#MOVE_TO}, {@link yfiles.geometry.PathType#LINE_TO}, and {@link yfiles.geometry.PathType#CLOSE}
       * operations only.
       * </p>
       * @param eps The maximum value the flattened path may diverge from the original path for the Bézier curve line interpolations.
       * @returns A new path that does not contain Bézier curves.
       */
      flatten(eps:number):yfiles.geometry.GeneralPath;
      /**
       * Gets the bounds of a flattened version the path.
       * @param eps The epsilon to use for the flattening operation.
       * @returns The bounds of the flattened path.
       */
      getBounds(eps:number):yfiles.geometry.Rect;
      /**
       * Calculates the bounds of this path and returns it.
       * <p>
       * This will not yield tight bounds if the path contains Bézier curves. The bounds are calculated using the union of all
       * points and control points contained in this path.
       * </p>
       * @returns A rectangle that describes the geometric bounds of this path.
       * @see yfiles.geometry.GeneralPath#getBounds
       */
      getBounds():yfiles.geometry.Rect;
      /**
       * Returns the geometric length of this path.
       * <p>
       * Note that Bézier curves are still treated as line segments.
       * </p>
       * @returns The length of this path.
       */
      getLength():number;
      /**
       * Gets a point on this path instance at the given ratio.
       * <p>
       * Note that this implementation still treats Bézier curves as linear segments.
       * </p>
       * @param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of this path.
       * @returns The coordinates of the point.
       */
      getPoint(ratio:number):yfiles.geometry.Point;
      /**
       * Gets the number of segments in this path, that is the number of {@link yfiles.geometry.PathType#LINE_TO} and {@link yfiles.geometry.PathType#CLOSE}
       * operations.
       * @returns The number of segments.
       */
      getSegmentCount():number;
      /**
       * Gets the tangent on this path instance at the segment and segment ratio.
       * <p>
       * Note that this implementation still treats Bézier curves as linear segments. Also note that the tangent vector may not
       * be normalized.
       * </p>
       * @param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of the segment at segmentIndex
       * for this path.
       * @param segmentIndex The segment index to determine a point at.
       * @returns The tangent, if any exists. The tangent vector may not be normalized.
       */
      getTangent(segmentIndex:number,ratio:number):yfiles.geometry.Tangent;
      /**
       * Gets the tangent on this path instance at the given ratio.
       * <p>
       * Note that this implementation still treats Bézier curves as linear segments.
       * </p>
       * @param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of this path.
       * @returns The tangent, if any exists. The tangent vector may not be normalized.
       */
      getTangent(ratio:number):yfiles.geometry.Tangent;
      /**
       * Determines whether <code>this</code> has the same values like <code>path</code>.
       * @param path The object to compare <code>this</code> to. Must be of the same type.
       * @returns <code>true</code> if <code>this</code> has the same values like the other object.
       */
      hasSameValue(path:yfiles.geometry.GeneralPath):boolean;
      /**
       * Checks whether this path intersects the given rectangle.
       * <p>
       * This will yield false if the rectangle is fully contained within the area of the path.
       * </p>
       * @param rectangle The rectangle to test.
       * @param eps The epsilon to use for fuzzy testing.
       * @returns Whether the path described by this instance intersects the rectangle.
       */
      intersects(rectangle:yfiles.geometry.Rect,eps:number):boolean;
      /**
       * Appends a {@link yfiles.geometry.PathType#LINE_TO} operation to the path elements.
       * <p>
       * This draws a line from the current end point to the given position.
       * </p>
       * @param x The x coordinate of the line's end point.
       * @param y The y coordinate of the line's end point.
       */
      lineTo(x:number,y:number):void;
      /**
       * Appends a {@link yfiles.geometry.PathType#LINE_TO} operation to the path elements.
       * <p>
       * This draws a line from the current end point to the given position.
       * </p>
       * @param point The coordinates of the line's end point.
       */
      lineTo(point:yfiles.geometry.IPoint):void;
      /**
       * Performs a quick test to decide whether this path might intersect the clipping rectangle, provided the path would be
       * rendered using stroke of the given width.
       * <p>
       * This method will return false if the clip is entirely inside the area of this path.
       * </p>
       * @param clip The clip to check for intersection.
       * @param width The width of the stroke.
       * @returns <code>false</code> if it is guaranteed that this instance would not intersect the given clip.
       */
      mayIntersectClip(clip:yfiles.geometry.Rect,width:number):boolean;
      /**
       * Appends a {@link yfiles.geometry.PathType#MOVE_TO} operation to the path elements.
       * <p>
       * This moves the pen to a new position without drawing a line.
       * </p>
       * @param x The next x coordinate.
       * @param y The next y coordinate.
       */
      moveTo(x:number,y:number):void;
      /**
       * Appends a {@link yfiles.geometry.PathType#MOVE_TO} operation to the path elements.
       * <p>
       * This moves the pen to a new position without drawing a line.
       * </p>
       * @param point The next coordinate.
       */
      moveTo(point:yfiles.geometry.IPoint):void;
      /**
       * Tests whether the line of the path is hit at the given point using an epsilon for fuzzy hit testing.
       * @param point The p coordinates of the point to test.
       * @param [eps=0.5] The allowed distance from the point to the path that is considered a hit.<code>0.5</code> is the default value.
       * @returns Whether the point hits the path.
       */
      pathContains(point:yfiles.geometry.IPoint,eps?:number):boolean;
      /**
       * Adds the contents of the given path to the beginning of this path.
       * <p>
       * If <code>connect</code> is <code>true</code>, the initial {@link yfiles.geometry.PathType#MOVE_TO} in <code>this</code> will be transformed into a
       * {@link yfiles.geometry.PathType#LINE_TO}.
       * </p>
       * @param other The path to append to this path.
       * @param connect Whether to connect the path to the end of this path.
       */
      prepend(other:yfiles.geometry.GeneralPath,connect:boolean):void;
      /**
       * Appends a {@link yfiles.geometry.PathType#QUAD_TO quadratic Bézier curve} to the path elements.
       * <p>
       * This draws a quadratic Bézier curve from the current end point to the given position using the given control point.
       * </p>
       * @param cx The x coordinate of the Bézier control point.
       * @param cy The y coordinate of the Bézier control point.
       * @param x The x coordinate of the curve's end point.
       * @param y The y coordinate of the curve's end point.
       */
      quadTo(cx:number,cy:number,x:number,y:number):void;
      /**
       * Appends a {@link yfiles.geometry.PathType#QUAD_TO quadratic Bézier curve} to the path elements.
       * <p>
       * This draws a quadratic Bézier curve from the current end point to the given position using the given control point.
       * </p>
       * @param center The coordinates of the Bézier control point.
       * @param point The coordinates of the curve's end point.
       */
      quadTo(center:yfiles.geometry.IPoint,point:yfiles.geometry.IPoint):void;
      /**
       * Transforms this {@link yfiles.geometry.GeneralPath} in place using the given transform.
       * @param transform The matrix to multiply the geometry with.
       */
      transform(transform:yfiles.geometry.Matrix):void;
      /**
       * Gets the last x coordinate of the last path element.
       * @type {number}
       */
      lastX:number;
      /**
       * Gets the last coordinates of the last path element.
       * @type {yfiles.geometry.Point}
       */
      lastCoordinate:yfiles.geometry.Point;
      /**
       * Gets the last y coordinate of the last path element.
       * @type {number}
       */
      lastY:number;
      /**
       * Gets a value indicating whether this path contains elements other than an initial {@link yfiles.geometry.PathType#MOVE_TO}.
       * @type {boolean}
       */
      isEmpty:boolean;
      /**
       * Gets a value indicating whether the path contains visible parts.
       * @type {boolean}
       */
      isVisible:boolean;
      /**
       * Gets the number of {@link yfiles.geometry.PathType} operations stored in this instance.
       * @type {number}
       */
      size:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An immutable {@link yfiles.geometry.ISize} implementation to describe a size in two-dimensional space.
     * @class
     * @implements {yfiles.geometry.ISize}
     * @final
     */
    export interface Size extends yfiles.lang.Struct,yfiles.geometry.ISize{}
    export class Size {
      /**
       * Creates a new size using the provided width and height.
       * @param width The new width.
       * @param height The new height.
       */
      constructor(width:number,height:number);
      /**
       * 
       * @returns 
       */
      clone():yfiles.geometry.Size;
      /**
       * Performs an implicit conversion from {@link yfiles.geometry.MutableSize} to {@link yfiles.geometry.Size}.
       * @param size The size.
       * @returns The result of the conversion.
       * @static
       */
      static convertFrom(size:yfiles.geometry.MutableSize):yfiles.geometry.Size;
      /**
       * Performs an explicit conversion from {@link yfiles.geometry.Size} to {@link yfiles.geometry.MutableSize}.
       * @param size The size.
       * @returns The result of the conversion.
       * @static
       */
      static convertToMutableSize(size:yfiles.geometry.Size):yfiles.geometry.MutableSize;
      /**
       * Returns a size whose {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height} is the {@link Math#max} of the respecting properties of the two parameters.
       * @param size1 The first size.
       * @param size2 The second size.
       * @returns A size whose {@link #width} and {@link #height} is the {@link #max} of the respecting properties of the two parameters.
       * @static
       */
      static max(size1:yfiles.geometry.Size,size2:yfiles.geometry.Size):yfiles.geometry.Size;
      /**
       * Returns a size whose {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height} is the {@link Math#min} of the respecting properties of the two parameters.
       * @param size1 The first size.
       * @param size2 The second size.
       * @returns A size whose {@link #width} and {@link #height} is the {@link #min} of the respecting properties of the two parameters.
       * @static
       */
      static min(size1:yfiles.geometry.Size,size2:yfiles.geometry.Size):yfiles.geometry.Size;
      /**
       * Multiplies the {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height} by the given factor and returns the result.
       * @param factor The factor to multiply the width and height by.
       * @returns A new size that has the width and height multiplied by the factor.
       */
      multiply(factor:number):yfiles.geometry.Size;
      /**
       * Converts this instance to an {@link yfiles.geometry.MutableSize} instance
       * @returns The {@link } instance, whose attributes have been initialized with the values of this instance.
       */
      toMutableSize():yfiles.geometry.MutableSize;
      /**
       * Creates a {@link yfiles.algorithms.YDimension} from a given {@link yfiles.geometry.Size}.
       * @returns The {@link }.
       */
      toYDimension():yfiles.algorithms.YDimension;
      /**
       * The "empty" size that has {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height} set to <code>-1.0d</code>.
       * @const
       * @static
       * @type {yfiles.geometry.Size}
       */
      static EMPTY:yfiles.geometry.Size;
      /**
       * The "zero" size that has {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height} set to <code>0.0d</code>.
       * <p>
       * This size is not treated as {@link yfiles.geometry.Size#isEmpty}.
       * </p>
       * @const
       * @static
       * @type {yfiles.geometry.Size}
       */
      static ZERO:yfiles.geometry.Size;
      /**
       * The "infinite" size that has {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height} set to {@link number#POSITIVE_INFINITY}.
       * @const
       * @static
       * @type {yfiles.geometry.Size}
       */
      static INFINITE:yfiles.geometry.Size;
      /**
       * Gets the width.
       * <p>
       * Negative values for the width indicate an {@link yfiles.geometry.Size#isEmpty empty} size.
       * </p>
       * @type {number}
       */
      width:number;
      /**
       * Gets the height.
       * <p>
       * Negative values for the height indicate an {@link yfiles.geometry.Size#isEmpty empty} size.
       * </p>
       * @type {number}
       */
      height:number;
      /**
       * Gets the area of this instance which is the product of {@link yfiles.geometry.Size#width} and {@link yfiles.geometry.Size#height}.
       * @type {number}
       */
      area:number;
      /**
       * Gets a value indicating whether this instance is deemed empty.
       * @type {boolean}
       */
      isEmpty:boolean;
      /**
       * Gets a value indicating whether this instance is finite.
       * <p>
       * This means that neither {@link yfiles.geometry.Size#width} nor {@link yfiles.geometry.Size#height} are infinite.
       * </p>
       * @type {boolean}
       */
      isFinite:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An immutable {@link yfiles.geometry.IPoint} that models a point in 2-d Cartesian coordinate space with double coordinates.
     * <p>
     * This class implements the {@link yfiles.geometry.IPoint} interface so that it can be used in methods that require that
     * interface.
     * </p>
     * @class
     * @implements {yfiles.geometry.IPoint}
     * @final
     */
    export interface Point extends yfiles.lang.Struct,yfiles.geometry.IPoint{}
    export class Point {
      /**
       * Initializes a new instance using the given values for the {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y} properties.
       * @param x The x coordinate.
       * @param y The y coordinate.
       */
      constructor(x:number,y:number);
      /**
       * Adds the given vector to this instance.
       * <p>
       * This operator is applied componentwise to {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y}.
       * </p>
       * @param otherPoint The second point.
       * @returns The result of the vector addition.
       */
      add(otherPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * 
       * @returns 
       */
      clone():yfiles.geometry.Point;
      /**
       * Performs an explicit conversion from {@link yfiles.geometry.MutablePoint} to {@link yfiles.geometry.Point}.
       * @param p The point to convert.
       * @returns The result of the conversion.
       * @static
       */
      static convertFrom(p:yfiles.geometry.MutablePoint):yfiles.geometry.Point;
      /**
       * Performs an implicit conversion from {@link yfiles.geometry.Point} to {@link yfiles.geometry.MutablePoint}.
       * @param point The point to convert.
       * @returns 
       * @static
       */
      static convertToMutablePoint(point:yfiles.geometry.Point):yfiles.geometry.MutablePoint;
      /**
       * Determines the distance between this point and a line segment.
       * @param start The coordinates of the first point of the line.
       * @param end The coordinates of the second point of the line.
       * @returns The distance between this point and the closest point on the line segment.
       */
      distanceToSegment(start:yfiles.geometry.Point,end:yfiles.geometry.Point):number;
      /**
       * Determines whether the two given points have the same coordinates with respect to a certain given <code>eps</code>.
       * @param other The other point to check for equality against this point.
       * @param eps The epsilon value.
       * @returns Whether both coordinates are equal with respect for the given epsilon.
       */
      equalsEps(other:yfiles.geometry.Point,eps:number):boolean;
      /**
       * Create a constrained copy of this instance that lies within the given non-empty <code>rectangle</code>.
       * <p>
       * If the given rectangle is {@link yfiles.geometry.Rect#EMPTY}, this implementation will silently return.
       * </p>
       * @param rectangle The rectangle to constrain this instance by.
       * @returns A constrained copy of this instance.
       */
      getConstrained(rectangle:yfiles.geometry.Rect):yfiles.geometry.Point;
      /**
       * Calculates the projection of this point onto a line.
       * @param anchor An anchor for the infinite line.
       * @param direction A direction vector for the infinite line.
       * @returns The point on the line that is closest to this point.
       */
      getProjectionOnLine(anchor:yfiles.geometry.Point,direction:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Calculates the projection of this point onto a ray.
       * <p>
       * If the perpendicular projection onto the line is outside of the ray ("behind" the <code>rayStart</code>) the <code>rayStart</code> is returned
       * instead.
       * </p>
       * @param rayStart The start of the segment.
       * @param direction The direction of the ray.
       * @returns The point on the ray that is closest to this point.
       */
      getProjectionOnRay(rayStart:yfiles.geometry.Point,direction:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Calculates the projection of this point onto a segment.
       * <p>
       * If the perpendicular projection onto the line is outside of the segment the nearest segment endpoint is returned.
       * </p>
       * @param start The start of the segment.
       * @param end The end of the segment.
       * @returns The point on the segment that is closest to this point.
       */
      getProjectionOnSegment(start:yfiles.geometry.Point,end:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Determines if the point lies close to this point given an epsilon.
       * @param other The coordinates of the other point.
       * @param hitTestRadius The hit test epsilon.
       * @returns Whether the distance between the two points is smaller than <code>hitTestRadius</code>
       */
      hits(other:yfiles.geometry.Point,hitTestRadius:number):boolean;
      /**
       * Determines whether this point hits the line segment with respect to a given <code>radius</code>.
       * @param start The starting point of the line segment to test.
       * @param end The ending point of the line segment to test.
       * @param radius The hit test radius.
       * @returns Whether this point hits the given line segment within the radius.
       */
      hitsLineSegment(start:yfiles.geometry.Point,end:yfiles.geometry.Point,radius:number):boolean;
      /**
       * Determines whether a polygonal line is hit by this point given an epsilon.
       * @param points The list of points that is treated as a polygon
       * @param radius A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns Whether the point hits the polygon.
       */
      hitsPolyline(points:yfiles.collections.IEnumerable<yfiles.geometry.IPoint>,radius:number):boolean;
      /**
       * Applies the given matrix to this instance by calling {@link yfiles.geometry.Matrix#transform} and returns the result.
       * @param matrix The matrix to use for the transformation.
       * @returns The result of the transformation.
       */
      multiply(matrix:yfiles.geometry.Matrix):yfiles.geometry.Point;
      /**
       * Multiplies the given factor with this instance using scalar multiplication and returns the result.
       * <p>
       * This factor is applied componentwise to {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y}.
       * </p>
       * @param factor The factor to scale the components by.
       * @returns The result of the scalar multiplication.
       */
      multiply(factor:number):yfiles.geometry.Point;
      /**
       * Calculates the scalar product of this and the given vector.
       * @param other The other vector.
       * @returns The scalar product (<code>X*other.X + Y*other.Y</code>)
       */
      scalarProduct(other:yfiles.geometry.Point):number;
      /**
       * Calculates the scalar product of the two given points.
       * @param point1 The first point.
       * @param point2 The second point.
       * @returns The scalar product of the two points.
       * @static
       */
      static scalarProduct(point1:yfiles.geometry.Point,point2:yfiles.geometry.Point):number;
      /**
       * Subtracts the given vector from this instance.
       * <p>
       * This operator is applied componentwise to {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y}.
       * </p>
       * @param otherPoint The second point.
       * @returns The result of the vector subtraction.
       */
      subtract(otherPoint:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Creates an {@link yfiles.geometry.IMutablePoint} that has the same coordinates as this instance.
       * @returns A {@link } with the same coordinates.
       */
      toMutablePoint():yfiles.geometry.IMutablePoint;
      /**
       * Creates a {@link yfiles.algorithms.Point2D} from a given {@link yfiles.geometry.Point}.
       * @returns The {@link }.
       */
      toPoint2D():yfiles.algorithms.Point2D;
      /**
       * Creates a {@link yfiles.algorithms.YPoint} from a given {@link yfiles.geometry.Point}.
       * @returns The {@link }.
       */
      toYPoint():yfiles.algorithms.YPoint;
      /**
       * Creates a {@link yfiles.algorithms.YVector} from a given {@link yfiles.geometry.Point}.
       * @returns The {@link }.
       */
      toYVector():yfiles.algorithms.YVector;
      /**
       * An instance that has {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y} set to <code>0.0d</code>.
       * @const
       * @static
       * @type {yfiles.geometry.Point}
       */
      static ORIGIN:yfiles.geometry.Point;
      /**
       * Gets the X coordinate of the point.
       * @type {number}
       */
      x:number;
      /**
       * Gets the Y coordinate of the point.
       * @type {number}
       */
      y:number;
      /**
       * Yields the length of the vector that has {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y} as its components.
       * @type {number}
       */
      vectorLength:number;
      /**
       * Yields the squared length of the vector that has {@link yfiles.geometry.Point#x} and {@link yfiles.geometry.Point#y} as its components.
       * @type {number}
       */
      squaredVectorLength:number;
      /**
       * Gets the normalized version of this vector.
       * @type {yfiles.geometry.Point}
       */
      normalized:yfiles.geometry.Point;
      /**
       * Indicates whether this instance is a vertically oriented vector.
       * @type {boolean}
       */
      isVerticalVector:boolean;
      /**
       * Indicates whether this instance is a horizontally oriented vector.
       * @type {boolean}
       */
      isHorizontalVector:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A class that models the thickness of insets as double precision floating point values.
     * @class
     * @final
     */
    export interface Insets extends yfiles.lang.Struct{}
    export class Insets {
      /**
       * Initializes a new instance.
       * @param left The left inset.
       * @param top The top inset.
       * @param right The right inset.
       * @param bottom The bottom inset.
       */
      constructor(left:number,top:number,right:number,bottom:number);
      /**
       * Initializes a new instance using the provided inset for all four sides.
       * @param inset The inset to use for all sides.
       */
      constructor(inset:number);
      /**
       * 
       * @returns 
       */
      clone():yfiles.geometry.Insets;
      /**
       * Calculates the union of this instance and the given inset which is done by performing {@link Math#max} on all four inset
       * values.
       * @param insets The insets to max with these insets.
       * @returns 
       */
      createUnion(insets:yfiles.geometry.Insets):yfiles.geometry.Insets;
      /**
       * Creates an enlarged instance by adding the insets of the specified insets to this instance and returning the result.
       * @param insets The insets to add to this instance.
       * @returns 
       */
      getEnlarged(insets:yfiles.geometry.Insets):yfiles.geometry.Insets;
      /**
       * Multiplies each side of the insets by the given factor and returns the result.
       * @param factor The factor to scales up by.
       * @returns New insets scaled up by the factor.
       */
      multiply(factor:number):yfiles.geometry.Insets;
      /**
       * Creates a {@link yfiles.algorithms.Insets} from a given {@link yfiles.geometry.Insets}.
       * @returns The {@link }.
       */
      toInsets():yfiles.algorithms.Insets;
      /**
       * The "empty" insets that has all properties set to <code>0.0d</code>.
       * @const
       * @static
       * @type {yfiles.geometry.Insets}
       */
      static EMPTY:yfiles.geometry.Insets;
      /**
       * Gets the top inset.
       * @type {number}
       */
      top:number;
      /**
       * Gets the left inset.
       * @type {number}
       */
      left:number;
      /**
       * Gets the bottom inset.
       * @type {number}
       */
      bottom:number;
      /**
       * Gets the right inset.
       * @type {number}
       */
      right:number;
      /**
       * Gets the vertical insets, which is the sum of {@link yfiles.geometry.Insets#top} and {@link yfiles.geometry.Insets#bottom}.
       * @type {number}
       */
      verticalInsets:number;
      /**
       * Gets the horizontal insets, which is the sum of {@link yfiles.geometry.Insets#left} and {@link yfiles.geometry.Insets#right}.
       * @type {number}
       */
      horizontalInsets:number;
      /**
       * Gets a value indicating whether this instance is empty, that is all insets are <code>0.0d</code>.
       * @type {boolean}
       */
      isEmpty:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An immutable class that models a rectangle in 2-d Cartesian coordinate space with double coordinates.
     * <p>
     * This class differs from other possible implementation in the way negative {@link yfiles.geometry.Rect#width} and {@link yfiles.geometry.Rect#height} properties are handled and interpreted.
     * Width and/or height values may be negative in which case the rectangle is considered {@link yfiles.geometry.Rect#isEmpty empty} and non-existent. Empty
     * rectangles are not considered in {@link yfiles.geometry.Rect#add union-like} operations, whereas rectangles with {@link yfiles.geometry.Rect#width} and {@link yfiles.geometry.Rect#height} of <code>0.0d</code> are. This class implements
     * the {@link yfiles.geometry.IRectangle} interface so that it can be used in methods that require that interface.
     * </p>
     * @class
     * @implements {yfiles.geometry.IRectangle}
     * @final
     */
    export interface Rect extends yfiles.lang.Struct,yfiles.geometry.IRectangle{}
    export class Rect {
      /**
       * Initializes a new instance.
       * @param x The {@link #x} coordinate of the top left corner.
       * @param y The {@link #y} coordinate of the top left corner.
       * @param width The {@link #width} of the rectangle.
       * @param height The {@link #height} of the rectangle.
       */
      constructor(x:number,y:number,width:number,height:number);
      /**
       * Initializes a new instance using the {@link yfiles.geometry.Rect#topLeft} corner and the {@link yfiles.geometry.Rect#size}.
       * @param location The top left corner.
       * @param size The size to use.
       */
      constructor(location:yfiles.geometry.Point,size:yfiles.geometry.Size);
      /**
       * Initializes a new instance using two point to define the bounds
       * <p>
       * This will always result in non-{@link yfiles.geometry.Rect#isEmpty} rectangles as the coordinates of the points are sorted so that the smaller gets assigned
       * to {@link yfiles.geometry.Rect#x} and {@link yfiles.geometry.Rect#y} respectively and the greater ones define the {@link yfiles.geometry.Rect#width} and {@link yfiles.geometry.Rect#height}.
       * </p>
       * @param p1 The first point to determine the bounds.
       * @param p2 The second point to determine the bounds.
       */
      constructor(p1:yfiles.geometry.Point,p2:yfiles.geometry.Point);
      /**
       * Initializes a new instance of the {@link yfiles.geometry.Rect} class with the initial values obtained from the given rectangle.
       * @param rectangle The rectangle to retrieve the initial values from.
       */
      constructor(rectangle:yfiles.geometry.IRectangle);
      /**
       * Returns the union of this rectangle and the given point.
       * @param point The point to include in the rectangle's bounds.
       * @returns The enlarged rectangle.
       */
      add(point:yfiles.geometry.Point):yfiles.geometry.Rect;
      /**
       * Returns the union of the given rectangles.
       * <p>
       * {@link yfiles.geometry.Rect#isEmpty empty} rectangles will not be considered.
       * </p>
       * @param firstRectangle The first rectangle to use for the union.
       * @param secondRectangle The second rectangle to use for the union.
       * @returns A rectangle that encompasses the area of the two given rectangles.
       * @static
       */
      static add(firstRectangle:yfiles.geometry.Rect,secondRectangle:yfiles.geometry.Rect):yfiles.geometry.Rect;
      /**
       * 
       * @returns 
       */
      clone():yfiles.geometry.Rect;
      /**
       * Determines whether this rectangle contains the specified point with respect to a given epsilon.
       * @param point The point to test.
       * @param eps The positive epsilon distance that the point may lie outside the rectangle and still be considered contained.
       * @returns <code>true</code> if this rectangle contains the specified point; <code>false</code> otherwise. An {@link #isEmpty empty} instance never contains any point. A
       * point is considered to be contained in the rectangle when the coordinates are not smaller than {@link #minX} and {@link #minY} minus <code>eps</code> nor
       * greater than {@link #maxX} and {@link #maxY} plus <code>eps</code>.
       */
      containsWithEps(point:yfiles.geometry.Point,eps:number):boolean;
      /**
       * Performs an explicit conversion from {@link yfiles.geometry.MutableRectangle} to {@link yfiles.geometry.Rect}.
       * @param rectangle The rectangle to convert.
       * @returns The result of the conversion.
       * @static
       */
      static convertFrom(rectangle:yfiles.geometry.MutableRectangle):yfiles.geometry.Rect;
      /**
       * Performs an explicit conversion from {@link yfiles.geometry.Rect} to {@link yfiles.geometry.MutableRectangle}.
       * @param rectangle The rectangle to convert.
       * @returns The result of the conversion.
       * @static
       */
      static convertToMutableRectangle(rectangle:yfiles.geometry.Rect):yfiles.geometry.MutableRectangle;
      /**
       * Finds the intersection between a rectangle and a line.
       * @param inner The coordinates of a point lying inside the rectangle.
       * @param outer The coordinates of a point lying outside the rectangle.
       * @returns The intersection point if the inner point lies inside the rectangle, the outer point lies outside the rectangle and thus
       * an intersection point has been found, or <code>null</code> otherwise.
       */
      findLineIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Creates a new instance given the center of the rectangle and its size.
       * @param center The center to use.
       * @param size The size to assign.
       * @returns An instance whose center is set to <code>center</code> and size is <code>size</code>
       * @static
       */
      static fromCenter(center:yfiles.geometry.Point,size:yfiles.geometry.Size):yfiles.geometry.Rect;
      /**
       * Creates an enlarged instance of this rectangle using the same insets for all sides.
       * <p>
       * If this instance {@link yfiles.geometry.Rect#isEmpty}, the result will be the same.
       * </p>
       * @param size The inset to add to each of the sides to grow the new instance.
       * @returns 
       * @see yfiles.geometry.Rect#getEnlarged
       */
      getEnlarged(size:number):yfiles.geometry.Rect;
      /**
       * Creates an enlarged instance that is the same as this one but enlarged by the specified insets.
       * <p>
       * If this instance {@link yfiles.geometry.Rect#isEmpty}, the same will be returned.
       * </p>
       * @param insets The insets to use to add to the instance.
       * @returns 
       */
      getEnlarged(insets:yfiles.geometry.Insets):yfiles.geometry.Rect;
      /**
       * Transforms this instance using the specified matrix and returns the result.
       * @param transform The transform matrix to apply to this instance.
       * @returns 
       */
      getTransformed(transform:yfiles.geometry.Matrix):yfiles.geometry.Rect;
      /**
       * Returns a translated instance of this which has modified {@link yfiles.geometry.Rect#x} and {@link yfiles.geometry.Rect#y} by the given amount.
       * @param delta The delta to add to {@link #x} and {@link #y}.
       * @returns 
       */
      getTranslated(delta:yfiles.geometry.Point):yfiles.geometry.Rect;
      /**
       * Determines whether the bounds of this instance intersect with the bounds of the specified rectangle.
       * @param rectangle The rectangle to check.
       * @returns Whether both instance are non-empty and have an intersection with positive {@link #area}.
       */
      intersects(rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Determines whether this rectangle intersects an {@link yfiles.geometry.IOrientedRectangle}, given an epsilon.
       * @param rectangle The {@link } to test.
       * @param [eps=0] A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns Whether they have a non-empty intersection.
       */
      intersects(rectangle:yfiles.geometry.IOrientedRectangle,eps?:number):boolean;
      /**
       * Determines whether this rectangle intersects a line.
       * @param start The first end point of the line.
       * @param end The second end point of the line.
       * @returns Whether the line intersects the rectangle.
       */
      intersectsLine(start:yfiles.geometry.Point,end:yfiles.geometry.Point):boolean;
      /**
       * Determines whether a rectangle intersects a polygonal line.
       * @param points The list of points that is interpreted as a number of line segments.
       * @returns <code>true</code> if the rectangle intersects at least one segment of the line.
       */
      intersectsPolyline(points:yfiles.collections.IEnumerable<yfiles.geometry.IPoint>):boolean;
      /**
       * Creates an {@link yfiles.geometry.IMutableRectangle} using the values from this instance.
       * @returns An instance that has been initialized from the values of this instance.
       */
      toMutableRectangle():yfiles.geometry.IMutableRectangle;
      /**
       * Creates a {@link yfiles.algorithms.Rectangle2D Rectangle2D} from a given {@link yfiles.geometry.Rect}.
       * @returns The {@link  Rectangle2D}.
       */
      toRectangle2D():yfiles.algorithms.Rectangle2D;
      /**
       * Creates a {@link yfiles.algorithms.YRectangle} from a given {@link yfiles.geometry.Rect}.
       * @returns The {@link }.
       */
      toYRectangle():yfiles.algorithms.YRectangle;
      /**
       * An {@link yfiles.geometry.Rect#isEmpty empty} rectangle.
       * <p>
       * {@link yfiles.geometry.Rect#width} and {@link yfiles.geometry.Rect#height} are set to <code>-1.0d</code>, {@link yfiles.geometry.Rect#x} and {@link yfiles.geometry.Rect#y} are set to <code>0.0d</code>.
       * </p>
       * @const
       * @static
       * @type {yfiles.geometry.Rect}
       */
      static EMPTY:yfiles.geometry.Rect;
      /**
       * An infinite rectangle.
       * <p>
       * {@link yfiles.geometry.Rect#width} and {@link yfiles.geometry.Rect#height} are set to {@link number#POSITIVE_INFINITY}, {@link yfiles.geometry.Rect#x} and {@link yfiles.geometry.Rect#y} are set to {@link number#NEGATIVE_INFINITY}.
       * </p>
       * @const
       * @static
       * @type {yfiles.geometry.Rect}
       */
      static INFINITE:yfiles.geometry.Rect;
      /**
       * Gets the X coordinate of the upper left corner of the rectangle.
       * @type {number}
       */
      x:number;
      /**
       * Gets the Y coordinate of the upper left corner of the rectangle.
       * @type {number}
       */
      y:number;
      /**
       * Gets the width of the rectangle.
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of the rectangle.
       * @type {number}
       */
      height:number;
      /**
       * Gets the center X coordinate of the rectangle.
       * @type {number}
       */
      centerX:number;
      /**
       * Gets the center Y coordinate of the rectangle.
       * @type {number}
       */
      centerY:number;
      /**
       * Gets the center of the rectangle using {@link yfiles.geometry.Rect#centerX} and {@link yfiles.geometry.Rect#centerY}.
       * @type {yfiles.geometry.Point}
       */
      center:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the top left corner of the rectangle.
       * @type {yfiles.geometry.Point}
       */
      topLeft:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the top right corner of the rectangle.
       * @type {yfiles.geometry.Point}
       */
      topRight:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the bottom left corner of the rectangle.
       * @type {yfiles.geometry.Point}
       */
      bottomLeft:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the bottom right corner of the rectangle.
       * @type {yfiles.geometry.Point}
       */
      bottomRight:yfiles.geometry.Point;
      /**
       * Gets the size of this instance.
       * @type {yfiles.geometry.Size}
       */
      size:yfiles.geometry.Size;
      /**
       * Gets a value indicating whether this instance is considered empty.
       * <p>
       * Yields <code>true</code> if this at least one of {@link yfiles.geometry.Rect#width} or {@link yfiles.geometry.Rect#height} is negative; <code>false</code> otherwise. Note that a width and height of both <code>0.0d</code> is not
       * considered empty for this implementation.
       * </p>
       * @type {boolean}
       */
      isEmpty:boolean;
      /**
       * Gets a value indicating whether this instance is finite.
       * @type {boolean}
       */
      isFinite:boolean;
      /**
       * Gets the coordinates of the right border for this instance.
       * @type {number}
       */
      maxX:number;
      /**
       * Gets the coordinates of the lower border for this instance.
       * @type {number}
       */
      maxY:number;
      /**
       * Gets the X coordinates of the left side of this instance.
       * @type {number}
       */
      minX:number;
      /**
       * Gets the Y coordinates of the top side of this instance.
       * @type {number}
       */
      minY:number;
      /**
       * Gets the area of this instance which is the product of {@link yfiles.geometry.Rect#width} and {@link yfiles.geometry.Rect#height}.
       * @type {number}
       */
      area:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An immutable class that represents a tangent in a 2-d Cartesian coordinate space with double values.
     * <p>
     * The tangent consists of its point of tangency ({@link yfiles.geometry.Tangent#point}) and a direction ({@link yfiles.geometry.Tangent#vector}).
     * </p>
     * @class
     * @final
     */
    export interface Tangent extends yfiles.lang.Struct{}
    export class Tangent {
      /**
       * Initializes a new instance of the {@link yfiles.geometry.Tangent} class using the given values for the {@link yfiles.geometry.Tangent#point} and {@link yfiles.geometry.Tangent#vector} properties.
       * @param point The point of tangency.
       * @param vector The directional vector.
       */
      constructor(point:yfiles.geometry.Point,vector:yfiles.geometry.Point);
      /**
       * Gets the point of tangency.
       * @type {yfiles.geometry.Point}
       */
      point:yfiles.geometry.Point;
      /**
       * Gets the directional vector.
       * @type {yfiles.geometry.Point}
       */
      vector:yfiles.geometry.Point;
      static $class:yfiles.lang.Class;
    }
    /**
     * Utility class that performs all kind of geometric operations.
     * <p>
     * Also it serves as a factory for various implementations of geometric primitives.
     * </p>
     * @class
     * @static
     */
    export interface GeomUtilities extends Object{}
    export class GeomUtilities {
      /**
       * Checks whether an ellipse contains the given point.
       * @param bounds The coordinates of the bounds of the ellipse's enclosing rectangle.
       * @param point The coordinates of the point to test.
       * @param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns Whether the point lies within the ellipse
       * @static
       */
      static ellipseContains(bounds:yfiles.geometry.Rect,point:yfiles.geometry.Point,eps:number):boolean;
      /**
       * Returns the ellipse/line intersection point for the given point pair.
       * <p>
       * This will always return the intersection point that lies in the direction from inner to outer.
       * </p>
       * @param bounds The coordinates of the bounds of the ellipse's enclosing rectangle.
       * @param inner The coordinates of a point lying inside the ellipse.
       * @param outer The coordinates of a point lying outside the ellipse.
       * @returns The intersection point iff the inner point lies inside the ellipse and an intersection point has been found, otherwise
       * <code>null</code>.
       * @static
       */
      static findEllipseLineIntersection(bounds:yfiles.geometry.Rect,inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Finds the intersection between a line segment and an infinite ray.
       * <p>
       * The ray is described using an anchor point and a ray direction. The direction vector does not need to be normalized. In
       * order to obtain the intersection point do the following:
       * </p>
       * @param start The coordinates of the first end point of the line segment.
       * @param end The coordinates of the second end point of the line segment.
       * @param anchor The coordinates of the starting point of the ray.
       * @param rayDirection The direction vector of the ray.
       * @returns The distance factor or {@link #POSITIVE_INFINITY} if the ray does not intersect the line.
       * @static
       */
      static findRayIntersection(start:yfiles.geometry.Point,end:yfiles.geometry.Point,anchor:yfiles.geometry.Point,rayDirection:yfiles.geometry.Point):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Interface for mutable oriented rectangles in 2D coordinate space with double precision coordinates.
     * @see yfiles.geometry.ISize
     * @see yfiles.geometry.IOrientedRectangle
     * @see yfiles.geometry.OrientedRectangle
     * @interface
     * @implements {yfiles.geometry.IOrientedRectangle}
     * @implements {yfiles.geometry.IMutableSize}
     */
    export interface IMutableOrientedRectangle extends Object,yfiles.geometry.IOrientedRectangle,yfiles.geometry.IMutableSize{
      /**
       * Sets the anchor vector of the oriented rectangle to the given value.
       * @param location The coordinates of the new anchor location.
       */
      setAnchor?(location:yfiles.geometry.Point):void;
      /**
       * Sets the center of the oriented rectangle to the given value.
       * @param center The coordinates of the new center.
       */
      setCenter?(center:yfiles.geometry.Point):void;
      /**
       * Sets the orientation of this oriented rectangle by modifying the up vector components.
       * <p>
       * It is up to the caller to assure that the values describe a vector of length 1.
       * </p>
       * @param upx The x component of the normalized up vector.
       * @param upy The y component of the normalized up vector.
       * @see yfiles.geometry.IOrientedRectangle#upX
       * @see yfiles.geometry.IOrientedRectangle#upY
       * @abstract
       */
      setUpVector(upx:number,upy:number):void;
      /**
       * Gets or sets the x coordinate for the anchor of the rectangle.
       * <p>
       * Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
       * depending on context the value that can be read might not necessarily be the exact same value that has just been
       * written.
       * </p>
       * @abstract
       * @type {number}
       */
      anchorX:number;
      /**
       * Gets or sets the y coordinate for the anchor of the rectangle.
       * <p>
       * Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
       * depending on context the value that can be read might not necessarily be the exact same value that has just been
       * written.
       * </p>
       * @abstract
       * @type {number}
       */
      anchorY:number;
    }
    var IMutableOrientedRectangle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for a mutable point in 2D coordinate space with double precision coordinates.
     * <p>
     * This interface provides read and write access to the coordinates. It adds write access to the read only interface
     * {@link yfiles.geometry.IPoint}.
     * </p>
     * @see yfiles.geometry.IPoint
     * @see yfiles.geometry.MutablePoint
     * @interface
     * @implements {yfiles.geometry.IPoint}
     */
    export interface IMutablePoint extends Object,yfiles.geometry.IPoint{
      /**
       * Sets the coordinates of the point to the given values.
       * @param x The new x coordinate
       * @param y The new y coordinate
       */
      relocate?(x:number,y:number):void;
      /**
       * Sets the coordinates of the point to the given values.
       * @param location The new location.
       */
      relocate?(location:yfiles.geometry.IPoint):void;
      /**
       * Gets or sets the x coordinate for this point.
       * <p>
       * Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
       * depending on context the value that can be read might not necessarily be the exact same value that has just been
       * written.
       * </p>
       * @see yfiles.geometry.IPoint#x
       * @abstract
       * @type {number}
       */
      x:number;
      /**
       * Gets or sets the y coordinate for this point.
       * <p>
       * Implementations may adjust the coordinates internally, e.g. to automatically snap the coordinates to grid points. So
       * depending on context the value that can be read might not necessarily be the exact same value that has just been
       * written.
       * </p>
       * @see yfiles.geometry.IPoint#y
       * @abstract
       * @type {number}
       */
      y:number;
    }
    var IMutablePoint:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for mutable rectangles aligned to the axes in 2D coordinate space with double precision coordinates.
     * <p>
     * This interface provides read and write access to the features of the rectangle. This interface does not declare any
     * additional methods. It combines the {@link yfiles.geometry.IMutablePoint} interface that describes the upper left corner of the
     * rectangle with the {@link yfiles.geometry.IMutableSize} interface that describes the width and height of the rectangle.
     * </p>
     * @see yfiles.geometry.ISize
     * @see yfiles.geometry.IPoint
     * @see yfiles.geometry.MutableRectangle
     * @interface
     * @implements {yfiles.geometry.IMutableSize}
     * @implements {yfiles.geometry.IMutablePoint}
     * @implements {yfiles.geometry.IRectangle}
     */
    export interface IMutableRectangle extends Object,yfiles.geometry.IMutableSize,yfiles.geometry.IMutablePoint,yfiles.geometry.IRectangle{
      /**
       * Adds a point to a rectangle, possibly enlarging the rectangle.
       * <p>
       * If the rectangle is initially empty, i.e. its width or height is negative, the bounds of the rectangle will be set to
       * (p.x, p.y, 0, 0)
       * </p>
       * @param point The coordinate to include in the bounds.
       */
      add?(point:yfiles.geometry.IPoint):void;
      /**
       * Adds a rectangle to another one.
       * <p>
       * The result is placed into the first rectangle, which is returned. If either of the two rectangles is empty, i.e. it's
       * width or height is negative, the result will be the other rectangle.
       * </p>
       * @param rectangle2 The rectangle to be added.
       */
      add?(rectangle2:yfiles.geometry.IRectangle):void;
      /**
       * Applies a new position and size to a given mutable rectangle.
       * @param x The new x coordinate of the upper left corner of the rectangle.
       * @param y The new y coordinate of the upper left corner of the rectangle.
       * @param width The new width of the rectangle.
       * @param height The new height of the rectangle.
       */
      reshape?(x:number,y:number,width:number,height:number):void;
      /**
       * Applies a new position and size to a given mutable rectangle.
       * @param position The new location.
       * @param size The new size.
       */
      reshape?(position:yfiles.geometry.IPoint,size:yfiles.geometry.ISize):void;
      /**
       * Applies a new position and size to a given mutable rectangle.
       * @param newRectangle The bounds to set to the <code>rectangle</code>.
       */
      reshape?(newRectangle:yfiles.geometry.IRectangle):void;
      /**
       * Sets the center of the rectangle to the provided value.
       * @param center The new center coordinates.
       */
      setCenter?(center:yfiles.geometry.Point):void;
      /**
       * Creates the union of two rectangles, placing the result in the this parameter.
       * <p>
       * Either of the two parameters rectangles may be the same as the first parameter. The result is placed into the this
       * parameter. This method treats rectangles with negative width or height as empty.
       * </p>
       * @param rectangle1 The first rectangle to create the union of.
       * @param rectangle2 The second rectangle to create the union of.
       */
      setToUnion?(rectangle1:yfiles.geometry.IRectangle,rectangle2:yfiles.geometry.IRectangle):void;
    }
    var IMutableRectangle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for a mutable size in 2D coordinate space with double precision width and height.
     * <p>
     * This interface provides read and write access to the size. It extends the read only interface {@link yfiles.geometry.ISize}.
     * </p>
     * @see yfiles.geometry.ISize
     * @see yfiles.geometry.MutableSize
     * @interface
     * @implements {yfiles.geometry.ISize}
     */
    export interface IMutableSize extends Object,yfiles.geometry.ISize{
      /**
       * Sets the coordinates of the size to the given values.
       * @param newWidth The new x coordinate
       * @param newHeight The new y coordinate
       */
      resize?(newWidth:number,newHeight:number):void;
      /**
       * Sets the coordinates of the size to the given values.
       * @param newSize The new location.
       */
      resize?(newSize:yfiles.geometry.ISize):void;
      /**
       * Gets or sets the width of this instance.
       * <p>
       * Implementations may adjust the values internally, e.g. to automatically snap the values to grid sizes. So depending on
       * context the value that can be read might not necessarily be the exact same value that has just been written.
       * </p>
       * @see yfiles.geometry.ISize#width
       * @abstract
       * @type {number}
       */
      width:number;
      /**
       * Gets or sets the height of this instance.
       * <p>
       * Implementations may adjust the values internally, e.g. to automatically snap the values to grid sizes. So depending on
       * context the value that can be read might not necessarily be the exact same value that has just been written.
       * </p>
       * @see yfiles.geometry.ISize#height
       * @abstract
       * @type {number}
       */
      height:number;
    }
    var IMutableSize:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for oriented rectangles in 2D coordinate space with double precision coordinates.
     * <p>
     * The rectangle described by this interface has its lower left corner at the anchor as described by the {@link yfiles.geometry.IOrientedRectangle#anchorX} and {@link yfiles.geometry.IOrientedRectangle#anchorY} properties
     * and is oriented in space so that its upper left corner lies in direction of the ({@link yfiles.geometry.IOrientedRectangle#upX},{@link yfiles.geometry.IOrientedRectangle#upY}) vector. The size of the instance
     * does not describe the bounds of the instance but the lengths of the sides of the rectangle. An {@link yfiles.geometry.IRectangle}
     * would thus be an oriented rectangle, whose anchor is the lower left corner of the rectangle, whose size is the same as
     * that of the rectangle and whose up vector is <code>(0,-1)</code>. This interface provides read access to the features of the oriented
     * rectangle only, however this does not mean that an instance that implements <code>IOrientedRectangle</code> will always return the
     * same values for its properties. Often times the instance provides a dynamic read access to the current state of a
     * rectangle. It depends on the context whether it is allowed to or even necessary to copy the state of the properties or
     * whether the reference to the instance should always be used to query the values.
     * </p>
     * @see yfiles.geometry.ISize
     * @see yfiles.geometry.IRectangle
     * @see yfiles.geometry.OrientedRectangle
     * @see yfiles.geometry.MutableRectangle
     * @interface
     * @implements {yfiles.geometry.ISize}
     */
    export interface IOrientedRectangle extends Object,yfiles.geometry.ISize{
      /**
       * Determines whether the given oriented rectangle contains the provided point, using an epsilon value.
       * @param point The coordinates of the point to test.
       * @param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns <code>true</code> iff the point lies inside the rectangle.
       */
      containsWithEps?(point:yfiles.geometry.Point,eps:number):boolean;
      /**
       * Creates a transformation matrix that can be used to transform points that are in the local coordinate system of the
       * oriented rectangle if the top-left corner is the origin.
       * @returns A matrix that can be used to transform from oriented rectangle coordinates to world coordinates.
       */
      createTransform?():yfiles.geometry.Matrix;
      /**
       * Determines whether the oriented rectangle contains the provided point, using an epsilon value.
       * @param location The coordinates of the point to test.
       * @param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns <code>true</code> iff the point lies inside the rectangle.
       */
      hits?(location:yfiles.geometry.Point,eps:number):boolean;
      /**
       * Creates a {@link yfiles.algorithms.YOrientedRectangle} from a given {@link yfiles.geometry.IOrientedRectangle}.
       * @returns The {@link }.
       */
      toOrientedRectangle?():yfiles.algorithms.YOrientedRectangle;
      /**
       * Returns the x coordinate of the anchor of the oriented rectangle.
       * <p>
       * The anchor is the lower left corner of the oriented rectangle if the up vector is <code>(0,-1)</code>.
       * </p>
       * @abstract
       * @type {number}
       */
      anchorX:number;
      /**
       * Returns the y coordinate of the anchor of the oriented rectangle.
       * <p>
       * The anchor is the lower left corner of the oriented rectangle if the up vector is <code>(0,-1)</code>.
       * </p>
       * @abstract
       * @type {number}
       */
      anchorY:number;
      /**
       * Returns the x value of the up vector.
       * <p>
       * The up vector points from the lower left corner to the upper left corner and is always normalized, i.e. it's length is
       * always 1.
       * </p>
       * @abstract
       * @type {number}
       */
      upX:number;
      /**
       * Returns the y value of the up vector.
       * <p>
       * The up vector points from the lower left corner to the upper left corner and is always normalized, i.e. it's length is
       * always 1.
       * </p>
       * @abstract
       * @type {number}
       */
      upY:number;
      /**
       * Gets the up vector of the oriented rectangle as a {@link yfiles.geometry.Point} struct.
       * @type {yfiles.geometry.Point}
       */
      upVector?:yfiles.geometry.Point;
      /**
       * Gets the anchor location of the oriented rectangle as a {@link yfiles.geometry.Point} struct.
       * @type {yfiles.geometry.Point}
       */
      anchorLocation?:yfiles.geometry.Point;
      /**
       * Gets the current center of the oriented rectangle as a {@link yfiles.geometry.Point} struct.
       * @type {yfiles.geometry.Point}
       */
      orientedRectangleCenter?:yfiles.geometry.Point;
      /**
       * Determines the bounds of an oriented rectangle.
       * <p>
       * Bounding rectangles are parallel to the coordinate axes.
       * </p>
       * @type {yfiles.geometry.Rect}
       */
      bounds?:yfiles.geometry.Rect;
    }
    var IOrientedRectangle:{
      /**
       * An immutable empty {@link yfiles.geometry.IOrientedRectangle} with width and height of <code>-1</code>.
       * @const
       * @static
       * @type {yfiles.geometry.IOrientedRectangle}
       */
      EMPTY?:yfiles.geometry.IOrientedRectangle;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for a point in 2D coordinate space with double precision coordinates.
     * <p>
     * This interface provides read access to the coordinates only, however this does not mean that an instance that implements
     * <code>IPoint</code> will always return the same values for the coordinate properties. Often times the instance provides a dynamic
     * read access to the current state of a point. It depends on the context whether it is allowed to or even necessary to
     * copy the state of the properties or whether the reference to the instance should always be used to query the values.
     * </p>
     * @see yfiles.geometry.IMutablePoint
     * @see yfiles.geometry.MutablePoint
     * @interface
     */
    export interface IPoint extends Object{
      /**
       * Calculates the Euclidean distance between two points.
       * @param point2 The second point.
       * @returns The distance between the two points.
       */
      distanceTo?(point2:yfiles.geometry.IPoint):number;
      /**
       * Copies the current values of the coordinates of the point to a {@link yfiles.geometry.Point} struct.
       * <p>
       * This method is useful to obtain a copy of the state and for making use of the various utility methods that are provided
       * by {@link yfiles.geometry.Point}.
       * </p>
       * @returns The current values of the coordinates of the point.
       * @see yfiles.geometry.Point#toMutablePoint
       */
      toPoint?():yfiles.geometry.Point;
      /**
       * Gets the current x coordinate of this point.
       * <p>
       * Depending on context the values returned may change over time.
       * </p>
       * @abstract
       * @type {number}
       */
      x:number;
      /**
       * Gets the current y coordinate of this point.
       * <p>
       * Depending on context the values returned may change over time.
       * </p>
       * @abstract
       * @type {number}
       */
      y:number;
    }
    var IPoint:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for rectangles aligned to the axes in 2D coordinate space with double precision coordinates.
     * <p>
     * This interface provides read access to the features of the rectangle only, however this does not mean that an instance
     * that implements <code>IRectangle</code> will always return the same values for its properties. Often times the instance provides a
     * dynamic read access to the current state of a rectangle. It depends on the context whether it is allowed to or even
     * necessary to copy the state of the properties or whether the reference to the instance should always be used to query
     * the values. This interface does not declare any additional methods. It combines the {@link yfiles.geometry.IPoint} interface
     * that describes the upper left corner of the rectangle with the {@link yfiles.geometry.ISize} interface that describes the width
     * and height of the rectangle.
     * </p>
     * @see yfiles.geometry.IMutableRectangle
     * @see yfiles.geometry.IMutablePoint
     * @see yfiles.geometry.ISize
     * @see yfiles.geometry.IPoint
     * @see yfiles.geometry.MutableRectangle
     * @interface
     * @implements {yfiles.geometry.ISize}
     * @implements {yfiles.geometry.IPoint}
     */
    export interface IRectangle extends Object,yfiles.geometry.ISize,yfiles.geometry.IPoint{
      /**
       * Determines whether the given rectangle contains the provided point.
       * @param point The point to test.
       * @returns <code>true</code> iff the point lies inside the rectangle.
       */
      contains?(point:yfiles.geometry.Point):boolean;
      /**
       * Determines whether the given rectangle contains the provided point.
       * @param point The point to test.
       * @returns <code>true</code> iff the point lies inside the rectangle.
       */
      contains?(point:yfiles.geometry.IPoint):boolean;
      /**
       * Copies the current values of the rectangle to {@link yfiles.geometry.Rect} struct.
       * <p>
       * This method can be used to obtain a copy of the current state of the rectangle and for using the utility methods that
       * are available for the {@link yfiles.geometry.Rect} type.
       * </p>
       * @returns A {@link } that holds the values of the <code>rectangle</code> at the time of the invocation.
       * @see yfiles.geometry.IMutableRectangle#reshape
       * @see yfiles.geometry.Rect#toMutableRectangle
       */
      toRect?():yfiles.geometry.Rect;
      /**
       * Gets the coordinates of the top left corner of the rectangle as a {@link yfiles.geometry.Point}.
       * @type {yfiles.geometry.Point}
       */
      topLeft?:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the bottom left corner of the rectangle as a {@link yfiles.geometry.Point}.
       * @type {yfiles.geometry.Point}
       */
      bottomLeft?:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the center of the rectangle as a {@link yfiles.geometry.Point}.
       * @type {yfiles.geometry.Point}
       */
      center?:yfiles.geometry.Point;
      /**
       * Gets the maximum X coordinate of the rectangle.
       * <p>
       * This is the x coordinate of the right side of the rectangle, or the left side if the rectangle is {@link yfiles.geometry.IRectangle#isEmpty}.
       * </p>
       * @type {number}
       */
      maxX?:number;
      /**
       * Determines whether the specified rectangle is empty.
       * <p>
       * {@link yfiles.geometry.IRectangle} instances are considered empty if their {@link yfiles.geometry.ISize#width} or {@link yfiles.geometry.ISize#height} is less than <code>0.0d</code>.
       * </p>
       * @type {boolean}
       */
      isEmpty?:boolean;
      /**
       * Gets the maximum Y coordinate of the rectangle.
       * <p>
       * This is the y coordinate of the bottom side of the rectangle, or the top side if the rectangle is {@link yfiles.geometry.IRectangle#isEmpty}.
       * </p>
       * @type {number}
       */
      maxY?:number;
      /**
       * Gets the coordinates of the top right corner of the rectangle as a {@link yfiles.geometry.Point}.
       * @type {yfiles.geometry.Point}
       */
      topRight?:yfiles.geometry.Point;
      /**
       * Gets the coordinates of the bottom right corner of the rectangle as a {@link yfiles.geometry.Point}.
       * @type {yfiles.geometry.Point}
       */
      bottomRight?:yfiles.geometry.Point;
    }
    var IRectangle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface for an object that has a size in 2D coordinate space with double precision floating point size values.
     * <p>
     * This interface provides read access to the size only, however this does not mean that an instance that implements <code>ISize</code>
     * will always return the same values for the size properties. Often times the instance provides a dynamic read access to
     * the current state of an instance. It depends on the context whether it is allowed to or even necessary to copy the state
     * of the properties or whether the reference to the instance should always be used to query the values.
     * </p>
     * @see yfiles.geometry.IMutableSize
     * @see yfiles.geometry.MutableSize
     * @interface
     */
    export interface ISize extends Object{
      /**
       * Converts the {@link yfiles.geometry.ISize} to a {@link yfiles.geometry.Size} struct.
       * @returns A {@link } struct that has been initialized with the current values of <code>size</code>.
       */
      toSize?():yfiles.geometry.Size;
      /**
       * Gets the current width of this instance.
       * <p>
       * Depending on context the values returned may change over time.
       * </p>
       * @abstract
       * @type {number}
       */
      width:number;
      /**
       * Gets the current height of this instance.
       * <p>
       * Depending on context the values returned may change over time.
       * </p>
       * @abstract
       * @type {number}
       */
      height:number;
    }
    var ISize:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum MatrixOrder{
      /**
       * Constant that defines a prepend operation for matrix multiplications.
       * <p>
       * Prepending a matrix <b>T</b> to another matrix <b>M</b> results in the operation <b>M</b>′&nbsp;=&nbsp;<b>M</b>&nbsp;<b>T</b>. Essentially, this means that <b>T</b> is applied before
       * <b>M</b> when applying <b>M</b>′ to a vector.
       * </p>
       */
      PREPEND,
      /**
       * Constant that defines an append operation for matrix multiplications.
       * <p>
       * Appending a matrix <b>T</b> to another matrix <b>M</b> results in the operation <b>M</b>′&nbsp;=&nbsp;<b>T</b>&nbsp;<b>M</b>. Essentially, this means that <b>T</b> is applied after <b>M</b>
       * when applying <b>M</b>′ to a vector.
       * </p>
       */
      APPEND
    }
    /**
     * An implementation of a matrix that represents an affine transformation and works with {@link yfiles.geometry.Point}.
     * <p>
     * The matrix is interpreted row-major. The rows are defined as follows: [ m11 m12 dx ], [ m21 m22 dy ], ([ 0 0 1 ]
     * implicitly).
     * </p>
     * <p>
     * When transforming a vector <b>v</b> using this matrix, the multiplication is done in this order: <b>v</b>'&nbsp;=&nbsp;<b>M</b>&nbsp;<b>v</b>.
     * </p>
     * <p>
     * {@link yfiles.geometry.MatrixOrder#PREPEND Prepending} a matrix <b>T</b> to this instance results in the operation <b>M</b>′&nbsp;=&nbsp;<b>M</b>&nbsp;<b>T</b>. In concept, this means that <b>T</b> is applied before <b>M</b>
     * when applying <b>M</b>' to a vector. {@link yfiles.geometry.MatrixOrder#APPEND Appending} <b>T</b> to <b>M</b> results in <b>M</b>′&nbsp;=&nbsp;<b>T</b>&nbsp;<b>M</b>.
     * </p>
     * @class
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface Matrix extends Object,yfiles.lang.ICloneable{}
    export class Matrix {
      /**
       * Create a matrix using the provided matrix entries.
       * @param m11
       * @param m12
       * @param m21
       * @param m22
       * @param dx
       * @param dy
       */
      constructor(m11:number,m12:number,m21:number,m22:number,dx:number,dy:number);
      /**
       * Create an identity matrix.
       */
      constructor();
      /**
       * Applies this matrix to the given SVG element.
       * @param element The element to transform.
       */
      applyTo(element:SVGElement):void;
      /**
       * Applies this matrix to the given HTML canvas rendering context.
       * @param ctx The context to transform.
       */
      applyTo(ctx:CanvasRenderingContext2D):void;
      /**
       * Creates a matrix rotation instance around the origin.
       * @param theta The rotation angle in radians.
       * @returns A new matrix.
       * @static
       */
      static createRotateInstance(theta:number):yfiles.geometry.Matrix;
      /**
       * Determines whether <code>this</code> has the same values like <code>other</code>.
       * @param other The object to compare <code>this</code> to. Must be of the same type.
       * @returns <code>true</code> if <code>this</code> has the same values like the other object.
       */
      hasSameValue(other:yfiles.geometry.Matrix):boolean;
      /**
       * Inverts this instance.
       */
      invert():void;
      /**
       * Multiplies this matrix instance by the given instance using the given order.
       * @param matrix The matrix to multiply with this one.
       * @param [order=yfiles.geometry.MatrixOrder.PREPEND] The order of the multiplication. {@link #PREPEND} if not specified.
       */
      multiply(matrix:yfiles.geometry.Matrix,order?:yfiles.geometry.MatrixOrder):void;
      /**
       * Resets this instance to the identity.
       */
      reset():void;
      /**
       * Prepends or appends a rotation operation to this matrix around the specified rotation center.
       * @param theta The rotation angle in radians
       * @param center The coordinate of the center of the rotation.
       * @param [order=yfiles.geometry.MatrixOrder.PREPEND] Whether to append or prepend the rotation matrix.
       */
      rotate(theta:number,center:yfiles.geometry.Point,order?:yfiles.geometry.MatrixOrder):void;
      /**
       * Prepends or appends a rotation operation to this matrix around the origin.
       * @param theta The rotation angle in radians
       * @param [order=yfiles.geometry.MatrixOrder.PREPEND] Whether to append or prepend the rotation matrix.
       */
      rotate(theta:number,order?:yfiles.geometry.MatrixOrder):void;
      /**
       * Appends or prepends a scale operation to this instance.
       * @param x
       * @param y
       * @param [order=yfiles.geometry.MatrixOrder.PREPEND]
       */
      scale(x:number,y:number,order?:yfiles.geometry.MatrixOrder):void;
      /**
       * Sets all elements of this instance.
       * @param m0
       * @param m1
       * @param m2
       * @param m3
       * @param dx
       * @param dy
       */
      set(m0:number,m1:number,m2:number,m3:number,dx:number,dy:number):void;
      /**
       * Sets the values of the given matrix to this instance.
       * @param matrix
       */
      set(matrix:yfiles.geometry.Matrix):void;
      /**
       * Converts the {@link yfiles.geometry.Matrix} object to an SVG transform string.
       * <p>
       * The returned string can be used in a transform attribute.
       * </p>
       * @returns The transform string
       */
      toSvgTransform():string;
      /**
       * Transforms the given coordinate.
       * @param point The coordinate to transform.
       * @returns The transformed coordinates.
       */
      transform(point:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Transforms the given point in place.
       * @param point The point to transform and return.
       * @returns <code>point</code>
       */
      transform(point:yfiles.geometry.IMutablePoint):yfiles.geometry.IMutablePoint;
      /**
       * Appends or Prepends a translation to this instance.
       * @param delta
       * @param [order=yfiles.geometry.MatrixOrder.PREPEND]
       */
      translate(delta:yfiles.geometry.Point,order?:yfiles.geometry.MatrixOrder):void;
      /**
       * Returns a new double[] of the elements describing the matrix.
       * <p>
       * The order is m11, m12, m21, m22, dx, dy.
       * </p>
       * @type {Array.<number>}
       */
      elements:number[];
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple default implementation of a mutable point in 2D coordinate space with double precision coordinates.
     * <p>
     * This implementation stores the values of the coordinates in double precision floating point members. As a convenience it
     * implements the {@link yfiles.lang.ICloneable} interface.
     * </p>
     * @see yfiles.geometry.IPoint
     * @class
     * @implements {yfiles.geometry.IMutablePoint}
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface MutablePoint extends Object,yfiles.geometry.IMutablePoint,yfiles.lang.ICloneable{}
    export class MutablePoint {
      /**
       * Creates an instance using the given coordinate pair.
       * @param x The x coordinate
       * @param y The y coordinate
       */
      constructor(x:number,y:number);
      /**
       * Creates an instance using the given coordinate pair.
       * @param point The coordinate.
       */
      constructor(point:yfiles.geometry.IPoint);
      /**
       * Creates the point with initially <code>0.0d</code> values for {@link yfiles.geometry.MutablePoint#x} and {@link yfiles.geometry.MutablePoint#y}.
       */
      constructor();
      /**
       * Moves this instance by adding the provided offsets to the coordinates of this point.
       * @param delta The offset to add to this point's x coordinate
       */
      moveBy(delta:yfiles.geometry.Point):void;
      /**
       * Normalizes this point as if it was a vector.
       * <p>
       * After this the "length" of this point is <code>1.0d</code> if it is treated as a vector.
       * </p>
       * @see yfiles.geometry.Point#normalized
       */
      normalize():void;
      /**
       * Gets or sets the x coordinate.
       * @type {number}
       */
      x:number;
      /**
       * Gets or sets the y coordinate.
       * @type {number}
       */
      y:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple default implementation of a mutable rectangle in 2D coordinate space with double precision values stored in an
     * instance of {@link yfiles.geometry.IMutablePoint} and {@link yfiles.geometry.IMutableSize}.
     * <p>
     * This implementation delegates the storage to implementations of {@link yfiles.geometry.IMutablePoint} for the upper left corner
     * of the rectangle and {@link yfiles.geometry.IMutableSize} for the size of the rectangle. members. As a convenience it implements
     * the {@link yfiles.geometry.IOrientedRectangle}, and the {@link yfiles.lang.ICloneable} interfaces.
     * </p>
     * @see yfiles.geometry.IRectangle
     * @see yfiles.geometry.IMutableRectangle
     * @class
     * @implements {yfiles.geometry.IMutableRectangle}
     * @implements {yfiles.geometry.IOrientedRectangle}
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface MutableRectangle extends Object,yfiles.geometry.IMutableRectangle,yfiles.geometry.IOrientedRectangle,yfiles.lang.ICloneable{}
    export class MutableRectangle {
      /**
       * Creates a new instance using the provided values to initialize the position and size.
       * <p>
       * An instance of {@link yfiles.geometry.MutablePoint} will be used to store the position and an instance of {@link yfiles.geometry.MutableRectangle#size} will be used to store
       * the size.
       * </p>
       * @param x The x coordinate of the upper left corner of the rectangle.
       * @param y The y coordinate of the upper left corner of the rectangle.
       * @param width The width of the rectangle.
       * @param height The height of the rectangle.
       */
      constructor(x:number,y:number,width:number,height:number);
      /**
       * Creates a new instance using the provided values to initialize the position and size.
       * <p>
       * An instance of {@link yfiles.geometry.MutablePoint} will be used to store the position and an instance of {@link yfiles.geometry.MutableRectangle#size} will be used to store
       * the size.
       * </p>
       * @param location The coordinates of the upper left corner of the rectangle.
       * @param size The size of the rectangle.
       */
      constructor(location:yfiles.geometry.Point,size:yfiles.geometry.Size);
      /**
       * Creates a new instance using the provided implementation for the storage of the position and size of the rectangle.
       * <p>
       * The instances provided will be referenced by this instance. This instance will be used as a live view over the provided
       * instances. Changes to them will indirectly change the state of this instance and changes applied through this instance
       * will be written to the referenced implementations.
       * </p>
       * @param location The provider for the dynamic position of this instance.
       * @param size The provider for the dynamic size of this instance.
       */
      constructor(location:yfiles.geometry.IMutablePoint,size:yfiles.geometry.IMutableSize);
      /**
       * Creates a new instance initialized to the values of the provided argument.
       * @param rectangle
       */
      constructor(rectangle:yfiles.geometry.IRectangle);
      /**
       * Creates a new empty instance located at the origin with width and height set to <code>-1</code>.
       */
      constructor();
      /**
       * Moves this rectangle by applying the offset to the {@link yfiles.geometry.MutableRectangle#location}
       * @param delta The offset to move the rectangle's position by.
       */
      moveBy(delta:yfiles.geometry.Point):void;
      /**
       * Returns whether this instance has negative width or height.
       * @type {boolean}
       */
      isEmpty:boolean;
      /**
       * Returns the instance that stores the position of this rectangle.
       * <p>
       * This will return a live view of the position of this rectangle. However setting a position instance will only copy the
       * values of the point provided to the current position.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      location:yfiles.geometry.IPoint;
      /**
       * Returns the instance that stores the size of this rectangle.
       * <p>
       * This will return a live view of the size of this rectangle. However setting a size instance will only copy the values of
       * the size provided to the current size.
       * </p>
       * @type {yfiles.geometry.ISize}
       */
      size:yfiles.geometry.ISize;
      /**
       * Gets or sets the width of this instance from the {@link yfiles.geometry.MutableRectangle#size} instance.
       * <p>
       * Note that depending on the implementation used for holding the size the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      width:number;
      /**
       * Gets or sets the height of this instance from the {@link yfiles.geometry.MutableRectangle#size} instance.
       * <p>
       * Note that that depending on the implementation used for holding the size the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      height:number;
      /**
       * Gets or sets the x coordinate of the upper left corner of this instance from the {@link yfiles.geometry.MutableRectangle#location} instance.
       * <p>
       * Note that depending on the implementation used for holding the position the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      x:number;
      /**
       * Gets or sets the y coordinate of the upper left corner of this instance from the {@link yfiles.geometry.MutableRectangle#location} instance.
       * <p>
       * Note that depending on the implementation used for holding the position the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      y:number;
      /**
       * Gets or sets the x coordinate of the lower right corner of this instance using the {@link yfiles.geometry.MutableRectangle#location} and {@link yfiles.geometry.MutableRectangle#size} instances.
       * <p>
       * Setting this value will update the width accordingly. Note that depending on the implementation used for holding the {@link yfiles.geometry.MutableRectangle#size}
       * the results may vary. However the default implementation will do just that: read and write the values without modifying
       * them in any way.
       * </p>
       * @type {number}
       */
      x2:number;
      /**
       * Gets or sets the y coordinate of the lower right corner of this instance using the {@link yfiles.geometry.MutableRectangle#location} and {@link yfiles.geometry.MutableRectangle#size} instances.
       * <p>
       * Setting this value will update the height accordingly. Note that depending on the implementation used for holding the {@link yfiles.geometry.MutableRectangle#size}
       * the results may vary. However the default implementation will do just that: read and write the values without modifying
       * them in any way.
       * </p>
       * @type {number}
       */
      y2:number;
      /**
       * Implements the {@link yfiles.geometry.IOrientedRectangle} and will return the x coordinate of the {@link yfiles.geometry.MutableRectangle#location}.
       * <p>
       * This implementation behaves like an {@link yfiles.geometry.IOrientedRectangle} that is anchored at the lower left corner of this
       * rectangle and whose up vector is (0, -1).
       * </p>
       * @type {number}
       */
      anchorX:number;
      /**
       * Implements the {@link yfiles.geometry.IOrientedRectangle} and will return the y coordinate of the {@link yfiles.geometry.MutableRectangle#location} plus the {@link yfiles.geometry.ISize#height}.
       * <p>
       * This implementation behaves like an {@link yfiles.geometry.IOrientedRectangle} that is anchored at the lower left corner of this
       * rectangle and whose up vector is (0, -1).
       * </p>
       * @type {number}
       */
      anchorY:number;
      /**
       * Always returns <code>0</code>
       * <p>
       * This implementation behaves like an {@link yfiles.geometry.IOrientedRectangle} that is anchored at the lower left corner of this
       * rectangle and whose up vector is (0, -1).
       * </p>
       * @type {number}
       */
      upX:number;
      /**
       * Always returns <code>-1</code>
       * <p>
       * This implementation behaves like an {@link yfiles.geometry.IOrientedRectangle} that is anchored at the lower left corner of this
       * rectangle and whose up vector is (0, -1).
       * </p>
       * @type {number}
       */
      upY:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple default implementation of a mutable size in 2D coordinate space with double precision values.
     * <p>
     * This implementation stores the values of the width and height in double precision floating point members.
     * </p>
     * @see yfiles.geometry.ISize
     * @class
     * @implements {yfiles.geometry.IMutableSize}
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface MutableSize extends Object,yfiles.geometry.IMutableSize,yfiles.lang.ICloneable{}
    export class MutableSize {
      /**
       * Creates an instance using the given width and height.
       * @param width The width.
       * @param height The height
       */
      constructor(width:number,height:number);
      /**
       * Creates an instance using the values provided by the size instance.
       * <p>
       * This will not create a dynamic instance. The values will be copied from <code>size</code> immediately and no reference is held to
       * that instance thereafter.
       * </p>
       * @param size A size to retrieve the initial values from.
       */
      constructor(size:yfiles.geometry.ISize);
      /**
       * Creates the initially empty size, that is both {@link yfiles.geometry.MutableSize#width} and {@link yfiles.geometry.MutableSize#height} are <code>0.0d</code>.
       */
      constructor();
      /**
       * Gets or sets the width.
       * @type {number}
       */
      width:number;
      /**
       * Gets or sets the height.
       * @type {number}
       */
      height:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple default implementation of a mutable oriented rectangle in 2D coordinate space with double precision values
     * stored in an instance of {@link yfiles.geometry.IMutablePoint} and {@link yfiles.geometry.IMutableSize} as well as two doubles for the
     * up vector components.
     * <p>
     * This implementation delegates the storage to implementations of {@link yfiles.geometry.IMutablePoint} for the anchor of the
     * oriented rectangle. and {@link yfiles.geometry.IMutableSize} for the size of the oriented rectangle. members. As a convenience
     * it implements the {@link yfiles.geometry.IMutablePoint} and the {@link yfiles.lang.ICloneable} interfaces.
     * </p>
     * @see yfiles.geometry.IOrientedRectangle
     * @see yfiles.geometry.IMutableOrientedRectangle
     * @class
     * @implements {yfiles.geometry.IMutablePoint}
     * @implements {yfiles.geometry.IMutableOrientedRectangle}
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface OrientedRectangle extends Object,yfiles.geometry.IMutablePoint,yfiles.geometry.IMutableOrientedRectangle,yfiles.lang.ICloneable{}
    export class OrientedRectangle {
      /**
       * Creates a new instance using the provided values to initialize the anchor and size.
       * <p>
       * The up vector will be initialized to
       * <code>(0, -1)</code>. The instances provided will be referenced by this instance. This instance will be used as a live view over the
       * provided instances. Changes to them will indirectly change the state of this instance and changes applied through this
       * instance will be written to the referenced implementations.
       * </p>
       * @param anchor The provider for the dynamic anchor of this instance.
       * @param size The provider for the dynamic size of this instance.
       */
      constructor(anchor:yfiles.geometry.IMutablePoint,size:yfiles.geometry.IMutableSize);
      /**
       * Creates a new instance using the provided values to initialize.
       * <p>
       * This will basically create a copy of the given rectangle
       * </p>
       * @param rectangle The rectangle to initialize the anchor and size and up vector from.
       */
      constructor(rectangle:yfiles.geometry.IOrientedRectangle);
      /**
       * Creates a new instance using the provided rectangle to initialize the anchor and size.
       * <p>
       * The anchor will be set to the lower left corner of the provided rectangle. Thus this instance will have the exact same
       * bounds as the rectangle initially. The up vector will be initialized to <code>(0, -1)</code>.
       * </p>
       * @param rectangle The rectangle to initialize the anchor and size from.
       */
      constructor(rectangle:yfiles.geometry.IRectangle);
      /**
       * Creates a new instance using the provided values to initialize the anchor and size.
       * <p>
       * It is up to the caller to ensure that the up vector is normalized.
       * </p>
       * @param anchorX The x coordinate of the anchor of the oriented rectangle.
       * @param anchorY The y coordinate of the anchor of the oriented rectangle.
       * @param width The width of the rectangle.
       * @param height The height of the rectangle.
       * @param [upX=0] The x component of the up vector.
       * @param [upY=-1] The y component of the up vector.
       */
      constructor(anchorX:number,anchorY:number,width:number,height:number,upX?:number,upY?:number);
      /**
       * Creates a new instance using the provided values to initialize the anchor and size.
       * <p>
       * It is up to the caller to ensure that the up vector is normalized.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.anchorX The x coordinate of the anchor of the oriented rectangle.
       * @param options.anchorY The y coordinate of the anchor of the oriented rectangle.
       * @param options.width The width of the rectangle.
       * @param options.height The height of the rectangle.
       * @param [options.upX=0] The x component of the up vector.
       * @param [options.upY=-1] The y component of the up vector.
       */
      constructor(options:{anchorX:number,anchorY:number,width:number,height:number,upX?:number,upY?:number});
      /**
       * Initializes a new instance of the {@link yfiles.geometry.OrientedRectangle} class located at <code>0.0d,0.0d</code> with empty width and
       * height (<code>-1.0d</code>).
       */
      constructor();
      /**
       * A casting operator that creates a new instance of {@link yfiles.geometry.OrientedRectangle} using the current state of the
       * rectangle that is cast.
       * <p>
       * The anchor will be set to the lower left corner of the provided rectangle. Thus this instance will have the exact same
       * bounds as the rectangle initially. The up vector will be initialized to
       * <code>(0, -1)</code>. Unlike true casting this will create a new instance that will be initialized to the current state of the object
       * being cast, but which will not stay in sync with the state of the object.
       * </p>
       * @param rect the rectangle to get the current state from
       * @returns A new instance of {@link } that holds no reference to the provided rectangle.
       * @static
       */
      static convertFrom(rect:yfiles.geometry.MutableRectangle):yfiles.geometry.OrientedRectangle;
      /**
       * Creates an immutable {@link yfiles.geometry.IOrientedRectangle} with the given values.
       * @param anchorX
       * @param anchorY
       * @param width
       * @param height
       * @param upX
       * @param upY
       * @returns 
       * @static
       */
      static createImmutable(anchorX:number,anchorY:number,width:number,height:number,upX:number,upY:number):yfiles.geometry.IOrientedRectangle;
      /**
       * Moves this rectangle by applying the offset to the {@link yfiles.geometry.OrientedRectangle#anchor}
       * @param delta The offset to move the rectangle's anchor by.
       */
      moveBy(delta:yfiles.geometry.Point):void;
      /**
       * Applies the values provided by the given instance to the values of this instance.
       * @param rectangle The instance to retrieve the values from.
       */
      reshape(rectangle:yfiles.geometry.IOrientedRectangle):void;
      /**
       * Sets the components of the up vector to the new values.
       * <p>
       * It is up to the caller to assure that the values describe a vector of length 1.
       * </p>
       * @param upx The x component of the normalized up vector.
       * @param upy The y component of the normalized up vector.
       * @see yfiles.geometry.OrientedRectangle#angle
       */
      setUpVector(upx:number,upy:number):void;
      /**
       * Returns whether this instance has negative width or height.
       * @type {boolean}
       */
      isEmpty:boolean;
      /**
       * Returns the instance that stores the anchor of this oriented rectangle.
       * <p>
       * This will return a live view of the anchor of this rectangle. However setting an anchor instance will only copy the
       * values of the point provided to the current anchor.
       * </p>
       * @type {yfiles.geometry.IPoint}
       */
      anchor:yfiles.geometry.IPoint;
      /**
       * Returns the instance that stores the size of this rectangle.
       * <p>
       * This will return a live view of the size of this rectangle. However setting a size instance will only copy the values of
       * the size provided to the current size.
       * </p>
       * @type {yfiles.geometry.ISize}
       */
      size:yfiles.geometry.ISize;
      /**
       * Gets or sets the width of this instance from the {@link yfiles.geometry.OrientedRectangle#size} instance.
       * <p>
       * Note that depending on the implementation used for holding the size the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      width:number;
      /**
       * Gets or sets the height of this instance from the {@link yfiles.geometry.OrientedRectangle#size} instance.
       * <p>
       * Note that that depending on the implementation used for holding the size the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      height:number;
      /**
       * Gets or sets the x coordinate of the anchor of this instance from the {@link yfiles.geometry.OrientedRectangle#anchor} instance.
       * <p>
       * Note that depending on the implementation used for holding the anchor the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      x:number;
      /**
       * Gets or sets the y coordinate of the anchor of this instance from the {@link yfiles.geometry.OrientedRectangle#anchor} instance.
       * <p>
       * Note that depending on the implementation used for holding the anchor the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      y:number;
      /**
       * Gets or sets the x coordinate of the anchor of this instance from the {@link yfiles.geometry.OrientedRectangle#anchor} instance.
       * <p>
       * Note that depending on the implementation used for holding the anchor the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      anchorX:number;
      /**
       * Gets or sets the y coordinate of the anchor of this instance from the {@link yfiles.geometry.OrientedRectangle#anchor} instance.
       * <p>
       * Note that depending on the implementation used for holding the anchor the results may vary. However the default
       * implementation will do just that: read and write the values without modifying them in any way.
       * </p>
       * @type {number}
       */
      anchorY:number;
      /**
       * Gets or sets the x component of the up vector to the new value.
       * <p>
       * It is up to the caller to assure that the values describe a vector of length 1.
       * </p>
       * @see yfiles.geometry.OrientedRectangle#angle
       * @see yfiles.geometry.OrientedRectangle#setUpVector
       * @see yfiles.geometry.OrientedRectangle#upY
       * @type {number}
       */
      upX:number;
      /**
       * Gets or sets the y component of the up vector to the new value.
       * <p>
       * It is up to the caller to assure that the values describe a vector of length 1.
       * </p>
       * @see yfiles.geometry.OrientedRectangle#angle
       * @see yfiles.geometry.OrientedRectangle#setUpVector
       * @see yfiles.geometry.OrientedRectangle#upX
       * @type {number}
       */
      upY:number;
      /**
       * Gets or sets the orientation of this rectangle.
       * <p>
       * The angle is specified in radians. An angle of <code>0</code> means the up vector points up in direction <code>(0,-1)</code>. The angle value is
       * not stored and will be recalculated from {@link yfiles.geometry.OrientedRectangle#upX} and {@link yfiles.geometry.OrientedRectangle#upY} every time this is invoked, so this is a costly method. Setting the angle
       * changes the {@link yfiles.geometry.OrientedRectangle#upX} and {@link yfiles.geometry.OrientedRectangle#upY} values accordingly.
       * </p>
       * @type {number}
       */
      angle:number;
      static $class:yfiles.lang.Class;
    }
  }export namespace styles{
    /**
     * A default implementation of the {@link yfiles.styles.IArrow} interface that can render {@link yfiles.styles.ArrowType}s.
     * @class
     * @implements {yfiles.styles.IArrow}
     * @implements {yfiles.lang.ICloneable}
     * @final
     */
    export interface Arrow extends Object,yfiles.styles.IArrow,yfiles.lang.ICloneable{}
    export class Arrow {
      /**
       * Creates a new arrow with the given color to use for {@link yfiles.styles.Arrow#stroke} and brush.
       * <p>
       * The {@link yfiles.styles.Arrow#stroke} is created with thickness 1.
       * </p>
       * @param color The color to use for {@link #stroke} and brush.
       */
      constructor(color:yfiles.view.Color);
      /**
       * Creates a new arrow with the given color to use for {@link yfiles.styles.Arrow#stroke} and brush.
       * <p>
       * The {@link yfiles.styles.Arrow#stroke} is created with thickness 1.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.color The color to use for {@link #stroke} and brush.
       * @param {number} options.cropLength The cropping length associated with this instance.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#cropLength} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.Arrow#stroke} of the outline of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#stroke} property on the created object.
       * </p>
       * @param {number} options.scale The scale factor of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#scale} property on the created object.
       * </p>
       * @param {yfiles.styles.ArrowType} options.type The type of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#type} property on the created object.
       * </p>
       */
      constructor(options:{color:yfiles.view.Color,cropLength?:number,stroke?:yfiles.view.Stroke,scale?:number,type?:yfiles.styles.ArrowType});
      /**
       * Creates a new arrow with default values for {@link yfiles.styles.Arrow#stroke} and brush.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Fill} options.fill The {@link yfiles.styles.Arrow#fill} of the fill of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#fill} property on the created object.
       * </p>
       * @param {number} options.cropLength The cropping length associated with this instance.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#cropLength} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.Arrow#stroke} of the outline of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#stroke} property on the created object.
       * </p>
       * @param {number} options.scale The scale factor of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#scale} property on the created object.
       * </p>
       * @param {yfiles.styles.ArrowType} options.type The type of this arrow.
       * <p>
       * This option sets the {@link yfiles.styles.Arrow#type} property on the created object.
       * </p>
       */
      constructor(options?:{fill?:yfiles.view.Fill,cropLength?:number,stroke?:yfiles.view.Stroke,scale?:number,type?:yfiles.styles.ArrowType});
      /**
       * Returns an immutable {@link yfiles.styles.IArrow} instance that uses the current properties.
       * @returns An immutable instance
       */
      asFrozen():yfiles.styles.IArrow;
      /**
       * Gets or sets the type of this arrow.
       * @default yfiles.styles.ArrowType.DEFAULT
       * @type {yfiles.styles.ArrowType}
       */
      type:yfiles.styles.ArrowType;
      /**
       * Gets or sets the scale factor of this arrow.
       * <p>
       * The arrow bounds are scaled with this factor.
       * </p>
       * @default 1.0
       * @type {number}
       */
      scale:number;
      /**
       * Gets or sets the {@link yfiles.styles.Arrow#stroke} of the outline of this arrow.
       * @default null
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the {@link yfiles.styles.Arrow#fill} of the fill of this arrow.
       * @default null
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      /**
       * Gets the cropping length associated with this instance.
       * <p>
       * This value is used by {@link yfiles.styles.IEdgeStyle} implementations to let the edge appear to end shortly before its actual
       * target.
       * </p>
       * @default 0.0
       * @type {number}
       */
      cropLength:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * The default implementation of the {@link yfiles.styles.IEdgePathCropper}.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 18 Edge Cropping</li>
     * </ul>
     * @class
     * @implements {yfiles.styles.IEdgePathCropper}
     */
    export interface DefaultEdgePathCropper extends Object,yfiles.styles.IEdgePathCropper{}
    export class DefaultEdgePathCropper {
      /**
       * Calculates the total length the edge path is cropped.
       * <p>
       * This method can be overridden to change the length that is cropped from an edge. The default implementation returns the
       * sum of {@link yfiles.styles.DefaultEdgePathCropper#extraCropLength}, {@link yfiles.styles.IArrow#length arrow length} and {@link yfiles.styles.IArrow#cropLength arrow crop length}
       * </p>
       * @param arrow The arrow at this edge end.
       * @param atSource <code>true</code> if the crop length should be calculated at the edge source. <code>false</code> otherwise.
       * @returns The total length the edge path is cropped.
       * @see yfiles.styles.DefaultEdgePathCropper#cropEdgePath
       * @see yfiles.styles.DefaultEdgePathCropper#cropEdgePathAtArrow
       * @protected
       */
      calculateTotalCropLength(arrow:yfiles.styles.IArrow,atSource:boolean):number;
      /**
       * Crops the provided <code>path</code> at one end of an edge.
       * <p>
       * If {@link yfiles.styles.DefaultEdgePathCropper#cropAtPort} is <code>true</code> this implementation delegates to {@link yfiles.styles.DefaultEdgePathCropper#cropEdgePathAtPortGeometry}. Otherwise
       * the edge path is cropped at the {@link yfiles.styles.DefaultEdgePathCropper#getNodeGeometry node geometry}. If this would result in an empty path, the cropping is delegated to {@link yfiles.styles.DefaultEdgePathCropper#handleEmptyPath}
       * instead.
       * </p>
       * @param edge The edge whose path is to be cropped.
       * @param atSource Whether to crop the source or target side of the path.
       * @param arrow The arrow that is used at the end of the edge.
       * @param path The path to crop.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       */
      cropEdgePath(edge:yfiles.graph.IEdge,atSource:boolean,arrow:yfiles.styles.IArrow,path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
      /**
       * Crops an edge's path at the source or target side with respect to the given arrow.
       * <p>
       * The <code>path</code> is cropped by the length {@link yfiles.styles.DefaultEdgePathCropper#calculateTotalCropLength} returns for <code>arrow</code> and
       * <code>atSource</code>.
       * </p>
       * @param atSource if set to <code>true</code> the source side is cropped.
       * @param arrow The arrow to consider for the cropping.
       * @param path The edge's path to crop.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       */
      cropEdgePathAtArrow(atSource:boolean,arrow:yfiles.styles.IArrow,path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
      /**
       * Crops an edge's path at the source or target side at the {@link yfiles.styles.DefaultEdgePathCropper#getPortGeometry port geometry} with respect to the given arrow.
       * @param edge The edge whose path is to be cropped.
       * @param atSource Whether to crop the source or target side of the path.
       * @param arrow The arrow that is used at the end of the edge.
       * @param path The path to crop.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       * @protected
       */
      cropEdgePathAtPortGeometry(edge:yfiles.graph.IEdge,atSource:boolean,arrow:yfiles.styles.IArrow,path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
      /**
       * Finds the intersection between a node and the edge.
       * @param node
       * @param nodeShapeGeometry
       * @param edge
       * @param inner
       * @param outer
       * @returns 
       * @protected
       */
      getIntersection(node:yfiles.graph.INode,nodeShapeGeometry:yfiles.styles.IShapeGeometry,edge:yfiles.graph.IEdge,inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Returns the {@link yfiles.styles.IShapeGeometry geometry} that is used to crop the edge at the node outline.
       * <p>
       * The default implementation queries the {@link yfiles.styles.INodeStyleRenderer#getShapeGeometry node style renderer} for an implementation of {@link yfiles.styles.IShapeGeometry}. This
       * method can be overridden to return an arbitrary {@link yfiles.styles.IShapeGeometry geometry} for a node.
       * </p>
       * @param node The node the edge should be cropped at.
       * @returns The {@link  geometry} of the node.
       * @protected
       */
      getNodeGeometry(node:yfiles.graph.INode):yfiles.styles.IShapeGeometry;
      /**
       * Returns the {@link yfiles.styles.IShapeGeometry geometry} that is used to crop the edge at the port outline if {@link yfiles.styles.DefaultEdgePathCropper#cropAtPort} is set to <code>true</code>.
       * <p>
       * The default implementation returns <code>null</code>. This method can be overridden to return an arbitrary {@link yfiles.styles.IShapeGeometry geometry} for a port.
       * </p>
       * @param port The port the edge should be cropped at.
       * @returns The {@link  geometry} of the port.
       * @see yfiles.styles.DefaultEdgePathCropper#cropAtPort
       * @protected
       */
      getPortGeometry(port:yfiles.graph.IPort):yfiles.styles.IShapeGeometry;
      /**
       * Handles the edge path cropping if {@link yfiles.styles.DefaultEdgePathCropper#cropEdgePath} would result in a {@link yfiles.geometry.GeneralPath#clear cleared} path.
       * <p>
       * This method is called by {@link yfiles.styles.DefaultEdgePathCropper#cropEdgePath} if {@link yfiles.styles.DefaultEdgePathCropper#cropAtPort} is <code>false</code> and cropping the edge path at
       * the {@link yfiles.styles.DefaultEdgePathCropper#getNodeGeometry node geometry} would result in an empty path.
       * </p>
       * <p>
       * The default implementation is to {@link yfiles.geometry.GeneralPath#clear} the edge path.
       * </p>
       * @param edge The edge whose path is to be cropped.
       * @param atSource Whether to crop the source or target side of the path.
       * @param arrow The arrow that is used at the end of the edge.
       * @param path The path to crop.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       * @protected
       */
      handleEmptyPath(edge:yfiles.graph.IEdge,atSource:boolean,arrow:yfiles.styles.IArrow,path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
      /**
       * Checks whether a given point is inside a node's shape geometry with respect to the edge that is being calculated.
       * @param location
       * @param node
       * @param nodeShapeGeometry
       * @param edge
       * @returns 
       * @protected
       */
      isInside(location:yfiles.geometry.Point,node:yfiles.graph.INode,nodeShapeGeometry:yfiles.styles.IShapeGeometry,edge:yfiles.graph.IEdge):boolean;
      /**
       * A singleton instance.
       * <p>
       * Note that this is an immutable instance that cannot be cast to the {@link yfiles.styles.DefaultEdgePathCropper} type.
       * </p>
       * @const
       * @static
       * @type {yfiles.styles.IEdgePathCropper}
       */
      static INSTANCE:yfiles.styles.IEdgePathCropper;
      /**
       * Gets or sets an extra length the edge is cropped.
       * <p>
       * The default value is 0.
       * </p>
       * @type {number}
       */
      extraCropLength:number;
      /**
       * Gets the value that determines if the edge path is cropped at the port or at the node bounds.
       * <p>
       * The default is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      cropAtPort:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The interface for the visual representation of an arrow that is normally used to decorate the visual representation of
     * an {@link yfiles.graph.IEdge}.
     * <p>
     * Instances of this class are used by {@link yfiles.styles.IEdgeStyle} implementations.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 20 Custom Arrow</li>
     * <li>Tutorial: 02 Custom Styles, step 21 Arrow Thickness</li>
     * </ul>
     * @interface
     */
    export interface IArrow extends Object{
      /**
       * Gets an {@link yfiles.view.IBoundsProvider} implementation that can yield this arrow's bounds if painted at the given location
       * using the given direction for the given edge.
       * @param edge the edge this arrow belongs to
       * @param atSource whether this will be the source arrow
       * @param anchor the anchor point for the tip of the arrow
       * @param directionVector the direction the arrow is pointing in
       * @returns an implementation of the {@link } interface that can subsequently be used to query the bounds.
       * Clients will always call this method before using the implementation and may not cache the instance returned. This
       * allows for applying the flyweight design pattern to implementations.
       * @abstract
       */
      getBoundsProvider(edge:yfiles.graph.IEdge,atSource:boolean,anchor:yfiles.geometry.Point,directionVector:yfiles.geometry.Point):yfiles.view.IBoundsProvider;
      /**
       * Gets an {@link yfiles.view.IVisualCreator} implementation that will paint this arrow at the given location using the given
       * direction for the given edge.
       * @param edge the edge this arrow belongs to
       * @param atSource whether this will be the source arrow
       * @param anchor the anchor point for the tip of the arrow
       * @param direction the direction the arrow is pointing in
       * @returns an implementation of the {@link } interface that can subsequently be used to perform the actual
       * painting. Clients will always call this method before using the implementation and may not cache the instance returned.
       * This allows for applying the flyweight design pattern to implementations.
       * @abstract
       */
      getVisualCreator(edge:yfiles.graph.IEdge,atSource:boolean,anchor:yfiles.geometry.Point,direction:yfiles.geometry.Point):yfiles.view.IVisualCreator;
      /**
       * Returns the length of the arrow (the distance from the arrow's tip to the position where the visual representation of
       * the edge's path should begin).
       * @abstract
       * @type {number}
       */
      length:number;
      /**
       * Gets the cropping length associated with this instance.
       * <p>
       * This value is used by {@link yfiles.styles.IEdgeStyle} implementations to let the edge appear to end shortly before its actual
       * target.
       * </p>
       * @abstract
       * @type {number}
       */
      cropLength:number;
    }
    var IArrow:{
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints no arrow at all.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      NONE?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a simple black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      SIMPLE?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a default shaped black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      DEFAULT?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a short black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      SHORT?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a simple diamond-shaped black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      DIAMOND?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a simple cross-shaped black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      CROSS?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a simple circle-shaped black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      CIRCLE?:yfiles.styles.IArrow;
      /**
       * A singleton {@link yfiles.styles.IArrow} instance that paints a simple triangle-shaped black arrow.
       * @const
       * @static
       * @type {yfiles.styles.IArrow}
       */
      TRIANGLE?:yfiles.styles.IArrow;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A void implementation of an edge style that does nothing and behaves like an invisible style.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidEdgeStyle#INSTANCE} of this class
     * instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.graph.IEdge#style} property of an
     * {@link yfiles.graph.IEdge}.
     * </p>
     * @class
     * @implements {yfiles.styles.IEdgeStyle}
     * @final
     */
    export interface VoidEdgeStyle extends Object,yfiles.styles.IEdgeStyle{}
    export class VoidEdgeStyle {
      /**
       * Returns <code>this</code>.
       * @returns <code>this</code>
       */
      clone():Object;
      /**
       * The {@link yfiles.styles.VoidEdgeStyle} singleton.
       * @const
       * @static
       * @type {yfiles.styles.VoidEdgeStyle}
       */
      static INSTANCE:yfiles.styles.VoidEdgeStyle;
      /**
       * Yields the {@link yfiles.styles.VoidEdgeStyleRenderer#INSTANCE VoidEdgeStyleRenderer instance}.
       * @see yfiles.styles.IEdgeStyle#renderer
       * @type {yfiles.styles.IEdgeStyleRenderer}
       */
      renderer:yfiles.styles.IEdgeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of an edge style renderer that does nothing.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidEdgeStyleRenderer#INSTANCE} of this
     * class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.styles.IEdgeStyle#renderer} property of an
     * {@link yfiles.styles.IEdgeStyle}.
     * </p>
     * @class
     * @implements {yfiles.styles.IEdgeStyleRenderer}
     * @final
     */
    export interface VoidEdgeStyleRenderer extends Object,yfiles.styles.IEdgeStyleRenderer{}
    export class VoidEdgeStyleRenderer {
      /**
       * Yields the {@link yfiles.view.IBoundsProvider#EMPTY} that will return empty bounds.
       * @param edge
       * @param style
       * @returns 
       */
      getBoundsProvider(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IBoundsProvider;
      /**
       * Yields the {@link yfiles.graph.ILookup#EMPTY} that will not yield anything.
       * @param edge
       * @param style
       * @returns 
       */
      getContext(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.graph.ILookup;
      /**
       * Yields the {@link yfiles.input.IHitTestable#NEVER} that will always report misses.
       * @param edge
       * @param style
       * @returns 
       */
      getHitTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.input.IHitTestable;
      /**
       * Yields the {@link yfiles.input.IMarqueeTestable#NEVER} that will always report misses.
       * @param edge
       * @param style
       * @returns 
       */
      getMarqueeTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.input.IMarqueeTestable;
      /**
       * This method always returns the {@link yfiles.styles.VoidPathGeometry#INSTANCE VoidPathGeometry instance}.
       * @param edge The edge to provide an instance for
       * @param style The style to use for the rendering
       * @returns The {@link #INSTANCE VoidPathGeometry instance}.
       * @see yfiles.styles.IEdgeStyleRenderer#getPathGeometry
       */
      getPathGeometry(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.styles.IPathGeometry;
      /**
       * Yields the {@link yfiles.view.IVisibilityTestable#NEVER} that will always claim invisibility.
       * @param edge
       * @param style
       * @returns 
       */
      getVisibilityTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Yields the {@link yfiles.view.VoidVisualCreator#INSTANCE} that will do nothing.
       * @param edge
       * @param style
       * @returns 
       */
      getVisualCreator(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IVisualCreator;
      /**
       * The {@link yfiles.styles.VoidEdgeStyleRenderer} singleton.
       * @const
       * @static
       * @type {yfiles.styles.IEdgeStyleRenderer}
       */
      static INSTANCE:yfiles.styles.IEdgeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of the {@link yfiles.styles.IPathGeometry} interface that does nothing and behaves like an empty path.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidPathGeometry#INSTANCE} of this class
     * instead of <code>null</code> were <code>null</code> is not allowed. The {@link yfiles.styles.VoidEdgeStyleRenderer} does so for example.
     * </p>
     * @class
     * @implements {yfiles.styles.IPathGeometry}
     * @final
     */
    export interface VoidPathGeometry extends Object,yfiles.styles.IPathGeometry{}
    export class VoidPathGeometry {
      /**
       * The {@link yfiles.styles.VoidPathGeometry} singleton.
       * @const
       * @static
       * @type {yfiles.styles.IPathGeometry}
       */
      static INSTANCE:yfiles.styles.IPathGeometry;
      static $class:yfiles.lang.Class;
    }
    /**
     * Style implementation for {@link yfiles.graph.IEdge} instances in an {@link yfiles.graph.IGraph}.
     * <p>
     * Implementations are responsible for rendering {@link yfiles.graph.IEdge} instances in a {@link yfiles.view.CanvasComponent}. It is up to
     * the implementation to interpret the visual appearance of an edge. The framework uses the associated, possibly shared {@link yfiles.styles.IEdgeStyle#renderer} to
     * perform the actual rendering of this style for a given edge. This interface extends the {@link yfiles.lang.ICloneable}
     * interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
     * implementations may return themselves.
     * </p>
     * <p>
     * Working with styles (and their style renderers) is explained in detail in the section {@link @DGUIDE_PREFIX@/view_visualization-graph-elements.html Visualization of Graph Elements}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 16 Custom Edge Style</li>
     * </ul>
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface IEdgeStyle extends Object,yfiles.lang.ICloneable{
      /**
       * Gets the renderer implementation that can be queried for implementations that provide details about the visual
       * appearance and visual behavior for a given edge and this style instance.
       * <p>
       * The idiom for retrieving, e.g. an {@link yfiles.view.IVisualCreator} implementation for a given style is:
       * </p>
       * @abstract
       * @type {yfiles.styles.IEdgeStyleRenderer}
       */
      renderer:yfiles.styles.IEdgeStyleRenderer;
    }
    var IEdgeStyle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.styles.IEdgeStyle} implementations that can be used to query the actual
     * {@link yfiles.view.IVisualCreator}, {@link yfiles.view.IVisibilityTestable}, {@link yfiles.view.IBoundsProvider}, {@link yfiles.input.IHitTestable}, and
     * {@link yfiles.input.IMarqueeTestable} implementations for an edge and an associated style.
     * <p>
     * Implementations of this interface should be designed to be sharable between multiple style instances and should
     * therefore not contain style specific state.
     * </p>
     * @see yfiles.styles.IEdgeStyle
     * @see yfiles.graph.IEdge
     * @see yfiles.styles.INodeStyleRenderer
     * @see yfiles.styles.ILabelStyleRenderer
     * @see yfiles.styles.IPortStyleRenderer
     * @interface
     */
    export interface IEdgeStyleRenderer extends Object{
      /**
       * Gets an implementation of the {@link yfiles.view.IBoundsProvider} interface that can handle the provided edge and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param edge The edge to provide an instance for
       * @param style The style to use for the calculating the painting bounds
       * @returns An implementation that may be used to subsequently query the edge's painting bounds. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getBoundsProvider(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IBoundsProvider;
      /**
       * Gets a temporary context instance that can be used to query additional information for the edge's style.
       * <p>
       * Implementations may return {@link yfiles.graph.ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
       * </p>
       * @param edge The edge to provide a context instance for.
       * @param style The style to use for the context.
       * @returns An non-<code>null</code> lookup implementation.
       * @see yfiles.graph.ILookup#EMPTY
       * @see yfiles.graph.ILookup
       * @abstract
       */
      getContext(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.graph.ILookup;
      /**
       * Gets an implementation of the {@link yfiles.input.IHitTestable} interface that can handle the provided edge and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param edge The edge to provide an instance for
       * @param style The style to use for the querying hit tests
       * @returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
       * always call this method immediately before using the value returned. This enables the use of the flyweight design
       * pattern for implementations. This method may return <code>null</code> to indicate that the edge cannot be hit tested.
       * @abstract
       */
      getHitTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.input.IHitTestable;
      /**
       * Gets an implementation of the {@link yfiles.input.IMarqueeTestable} interface that can handle the provided edge and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param edge The edge to provide an instance for
       * @param style The style to use for the querying marquee intersection test.
       * @returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getMarqueeTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Gets an implementation of the {@link yfiles.styles.IPathGeometry} interface that can handle the provided edge and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param edge The edge to provide an instance for
       * @param style The style to use for the rendering
       * @returns An implementation that may be used to subsequently query geometry information from. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getPathGeometry(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.styles.IPathGeometry;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisibilityTestable} interface that can handle the provided edge and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param edge The edge to provide an instance for
       * @param style The style to use for the testing the visibility
       * @returns An implementation that may be used to subsequently query the edge's visibility. Clients should not cache this instance
       * and must always call this method immediately before using the value returned. This enables the use of the flyweight
       * design pattern for implementations
       * @abstract
       */
      getVisibilityTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisualCreator} interface that can handle the provided edge and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation, but never <code>null</code>.
       * </p>
       * @param edge The edge to provide an instance for
       * @param style The style to use for the creation of the visual
       * @returns An implementation that may be used to subsequently create or update the visual for the edge. Clients should not cache
       * this instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link #INSTANCE void} implementation
       * instead.
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       * @abstract
       */
      getVisualCreator(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IVisualCreator;
    }
    var IEdgeStyleRenderer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    export enum ArrowType{
      /**
       * Default arrow
       */
      DEFAULT,
      /**
       * Simple arrow
       */
      SIMPLE,
      /**
       * Short arrow
       */
      SHORT,
      /**
       * Diamond shaped arrow
       */
      DIAMOND,
      /**
       * No arrow
       */
      NONE,
      /**
       * Circular shaped arrow
       */
      CIRCLE,
      /**
       * Cross shaped arrow
       */
      CROSS,
      /**
       * Triangular shaped arrow
       */
      TRIANGLE
    }
    /**
     * Interface used by the framework mainly for calculating the visible path of an edge.
     * <p>
     * This interface is queried through the {@link yfiles.graph.ILookup#lookup} of {@link yfiles.graph.IPort} instances to calculate the
     * visible portion of an edge path.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 18 Edge Cropping</li>
     * </ul>
     * @interface
     */
    export interface IEdgePathCropper extends Object{
      /**
       * Crops the provided <code>path</code> at one end of an edge.
       * @param edge The edge whose path is to be cropped.
       * @param atSource Whether to crop the source or target side of the path.
       * @param arrow The arrow that is used at the end of the edge.
       * @param path The path to crop.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       * @abstract
       */
      cropEdgePath(edge:yfiles.graph.IEdge,atSource:boolean,arrow:yfiles.styles.IArrow,path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
    }
    var IEdgePathCropper:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A void implementation of a label style that does nothing and behaves like an invisible style.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidLabelStyle#INSTANCE} of this class
     * instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.graph.ILabel#style} property of an
     * {@link yfiles.graph.ILabel}.
     * </p>
     * @class
     * @implements {yfiles.styles.ILabelStyle}
     * @final
     */
    export interface VoidLabelStyle extends Object,yfiles.styles.ILabelStyle{}
    export class VoidLabelStyle {
      /**
       * Returns <code>this</code>.
       * @returns <code>this</code>
       */
      clone():Object;
      /**
       * The singleton instance of this style.
       * @const
       * @static
       * @type {yfiles.styles.VoidLabelStyle}
       */
      static INSTANCE:yfiles.styles.VoidLabelStyle;
      /**
       * Yields the {@link yfiles.styles.VoidLabelStyleRenderer#INSTANCE VoidLabelStyleRenderer instance}.
       * @see yfiles.styles.ILabelStyle#renderer
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of a label style renderer that does nothing.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidLabelStyleRenderer#INSTANCE} of this
     * class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.styles.ILabelStyle#renderer} property of an
     * {@link yfiles.styles.ILabelStyle}.
     * </p>
     * @class
     * @implements {yfiles.styles.ILabelStyleRenderer}
     * @final
     */
    export interface VoidLabelStyleRenderer extends Object,yfiles.styles.ILabelStyleRenderer{}
    export class VoidLabelStyleRenderer {
      /**
       * Yields the {@link yfiles.view.IBoundsProvider#EMPTY} that will return empty bounds.
       * @param label
       * @param style
       * @returns 
       */
      getBoundsProvider(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IBoundsProvider;
      /**
       * Yields the {@link yfiles.graph.ILookup#EMPTY} that will not yield anything.
       * @param label
       * @param style
       * @returns 
       */
      getContext(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.graph.ILookup;
      /**
       * Yields the {@link yfiles.input.IHitTestable#NEVER} that will always report misses.
       * @param label
       * @param style
       * @returns 
       */
      getHitTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IHitTestable;
      /**
       * Yields the {@link yfiles.input.IMarqueeTestable#NEVER} that will always report misses.
       * @param label
       * @param style
       * @returns 
       */
      getMarqueeTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IMarqueeTestable;
      /**
       * Yields {@link yfiles.geometry.Size#EMPTY}.
       * @param label
       * @param style
       * @returns 
       */
      getPreferredSize(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.geometry.Size;
      /**
       * Yields the {@link yfiles.view.IVisibilityTestable#NEVER} that will always claim invisibility.
       * @param label
       * @param style
       * @returns 
       */
      getVisibilityTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisibilityTestable;
      /**
       * Yields the {@link yfiles.view.VoidVisualCreator#INSTANCE} that will do nothing.
       * @param label
       * @param style
       * @returns 
       */
      getVisualCreator(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisualCreator;
      /**
       * The singleton instance of this renderer.
       * @const
       * @static
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      static INSTANCE:yfiles.styles.ILabelStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Style implementation for {@link yfiles.graph.ILabel} instances in an {@link yfiles.graph.IGraph}.
     * <p>
     * Implementations are responsible for rendering {@link yfiles.graph.ILabel} instances in a {@link yfiles.view.CanvasComponent}. It is up
     * to the implementation to interpret the visual appearance of a label. The framework uses the associated, possibly shared
     * {@link yfiles.styles.ILabelStyle#renderer} to perform the actual rendering of this style for a given label. This interface extends the {@link yfiles.lang.ICloneable}
     * interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
     * implementations may return themselves.
     * </p>
     * <p>
     * Working with styles (and their style renderers) is explained in detail in the section {@link @DGUIDE_PREFIX@/view_visualization-graph-elements.html Visualization of Graph Elements}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 10 Custom Label Style</li>
     * </ul>
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface ILabelStyle extends Object,yfiles.lang.ICloneable{
      /**
       * Gets the renderer implementation that can be queried for implementations that provide details about the visual
       * appearance and visual behavior for a given label and this style instance.
       * <p>
       * The idiom for retrieving, e.g. an {@link yfiles.view.IVisualCreator} implementation for a given style is:
       * </p>
       * @abstract
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
    }
    var ILabelStyle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.styles.ILabelStyle} implementations to query the actual {@link yfiles.view.IVisualCreator},
     * {@link yfiles.view.IVisibilityTestable}, {@link yfiles.view.IBoundsProvider}, {@link yfiles.input.IHitTestable}, and {@link yfiles.input.IMarqueeTestable}
     * implementations for a label and an associated style.
     * <p>
     * Implementations of this interface should be designed to be sharable between multiple style instances and should
     * therefore not contain style specific state.
     * </p>
     * @see yfiles.styles.ILabelStyle
     * @see yfiles.graph.ILabel
     * @see yfiles.styles.INodeStyleRenderer
     * @see yfiles.styles.IEdgeStyleRenderer
     * @see yfiles.styles.IPortStyleRenderer
     * @interface
     */
    export interface ILabelStyleRenderer extends Object{
      /**
       * Gets an implementation of the {@link yfiles.view.IBoundsProvider} interface that can handle the provided label and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param label The label to provide an instance for
       * @param style The style to use for the calculating the painting bounds
       * @returns An implementation that may be used to subsequently query the label's painting bounds. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getBoundsProvider(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IBoundsProvider;
      /**
       * Gets a temporary context instance that can be used to query additional information for the label's style.
       * <p>
       * Implementations may return {@link yfiles.graph.ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
       * </p>
       * @param label The label to provide a context instance for.
       * @param style The style to use for the context.
       * @returns An non-<code>null</code> lookup implementation.
       * @see yfiles.graph.ILookup#EMPTY
       * @see yfiles.graph.ILookup
       * @abstract
       */
      getContext(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.graph.ILookup;
      /**
       * Gets an implementation of the {@link yfiles.input.IHitTestable} interface that can handle the provided label and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param label The label to provide an instance for
       * @param style The style to use for the querying hit tests
       * @returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
       * always call this method immediately before using the value returned. This enables the use of the flyweight design
       * pattern for implementations. This method may return <code>null</code> to indicate that the label cannot be hit tested.
       * @abstract
       */
      getHitTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IHitTestable;
      /**
       * Gets an implementation of the {@link yfiles.input.IMarqueeTestable} interface that can handle the provided label and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param label The label to provide an instance for
       * @param style The style to use for the querying marquee intersection test.
       * @returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getMarqueeTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IMarqueeTestable;
      /**
       * Calculates the {@link yfiles.graph.ILabel#preferredSize preferred size} of a given label using the associated style.
       * @param label The label to determine the preferred size for
       * @param style The style instance that uses this instance as its {@link #renderer}
       * @returns A size that can be used as the {@link #preferredSize} if this renderer renders the label using the associated style.
       * @abstract
       */
      getPreferredSize(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.geometry.Size;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisibilityTestable} interface that can handle the provided label and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param label The label to provide an instance for
       * @param style The style to use for the testing the visibility
       * @returns An implementation that may be used to subsequently query the label's visibility. Clients should not cache this instance
       * and must always call this method immediately before using the value returned. This enables the use of the flyweight
       * design pattern for implementations
       * @abstract
       */
      getVisibilityTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisibilityTestable;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisualCreator} interface that can handle the provided label and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation, but never <code>null</code>.
       * </p>
       * @param label The label to provide an instance for
       * @param style The style to use for the creation of the visual
       * @returns An implementation that may be used to subsequently create or update the visual for the label. Clients should not cache
       * this instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link #INSTANCE void} implementation
       * instead.
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       * @abstract
       */
      getVisualCreator(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisualCreator;
    }
    var ILabelStyleRenderer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A void implementation of the {@link yfiles.styles.IShapeGeometry} interface that does nothing and behaves like an empty
     * geometry.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidShapeGeometry#INSTANCE} of this
     * class instead of <code>null</code> were <code>null</code> is not allowed. The {@link yfiles.styles.VoidNodeStyleRenderer} does so for example.
     * </p>
     * @class
     * @implements {yfiles.styles.IShapeGeometry}
     * @final
     */
    export interface VoidShapeGeometry extends Object,yfiles.styles.IShapeGeometry{}
    export class VoidShapeGeometry {
      /**
       * This implementation always returns <code>null</code> (a {@link T} with no value).
       * @param inner The first point of the line that is inside the shape.
       * @param outer The second point of the line that is outside the shape.
       * @returns A {@link } with no value.
       * @see yfiles.styles.IShapeGeometry#getIntersection
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * This implementation returns an empty path.
       * <p>
       * This means that the outline never intersects any line or contains any point.
       * </p>
       * @returns The empty path.
       * @see yfiles.styles.IShapeGeometry#getOutline
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * This implementation always returns <code>false</code>.
       * @param location The point to test.
       * @returns <code>false</code>
       * @see yfiles.styles.IShapeGeometry#isInside
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * The {@link yfiles.styles.VoidShapeGeometry} singleton.
       * @const
       * @static
       * @type {yfiles.styles.IShapeGeometry}
       */
      static INSTANCE:yfiles.styles.IShapeGeometry;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of a node style that does nothing and behaves like an invisible style.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidNodeStyle#INSTANCE} of this class
     * instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.graph.INode#style} property of an
     * {@link yfiles.graph.INode}.
     * </p>
     * @class
     * @implements {yfiles.styles.INodeStyle}
     * @final
     */
    export interface VoidNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class VoidNodeStyle {
      /**
       * Returns <code>this</code>.
       * @returns <code>this</code>
       */
      clone():Object;
      /**
       * The {@link yfiles.styles.VoidNodeStyle} singleton.
       * @const
       * @static
       * @type {yfiles.styles.VoidNodeStyle}
       */
      static INSTANCE:yfiles.styles.VoidNodeStyle;
      /**
       * Yields the {@link yfiles.styles.VoidNodeStyleRenderer#INSTANCE VoidNodeStyleRenderer instance}.
       * @see yfiles.styles.INodeStyle#renderer
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of a node style renderer that does nothing.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidNodeStyleRenderer#INSTANCE} of this
     * class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.styles.INodeStyle#renderer} property of an
     * {@link yfiles.styles.INodeStyle}.
     * </p>
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @final
     */
    export interface VoidNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer{}
    export class VoidNodeStyleRenderer {
      /**
       * Yields the {@link yfiles.view.IBoundsProvider#EMPTY} that will return empty bounds.
       * @param node
       * @param style
       * @returns 
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Yields the {@link yfiles.graph.ILookup#EMPTY} that will not yield anything.
       * @param node
       * @param style
       * @returns 
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Yields the {@link yfiles.input.IHitTestable#NEVER} that will always report misses.
       * @param node
       * @param style
       * @returns 
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * Yields the {@link yfiles.input.IMarqueeTestable#NEVER} that will always report misses.
       * @param node
       * @param style
       * @returns 
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * This implementation always returns the {@link yfiles.styles.VoidShapeGeometry#INSTANCE VoidShapeGeometry instance}.
       * @param node The node to provide an instance for
       * @param style The style to use for the rendering
       * @returns The {@link #INSTANCE VoidShapeGeometry instance}.
       * @see yfiles.styles.INodeStyleRenderer#getShapeGeometry
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Yields the {@link yfiles.view.IVisibilityTestable#NEVER} that will always claim invisibility.
       * @param node
       * @param style
       * @returns 
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Yields the {@link yfiles.view.VoidVisualCreator#INSTANCE} that will do nothing.
       * @param node
       * @param style
       * @returns 
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * The {@link yfiles.styles.VoidNodeStyleRenderer} singleton.
       * @const
       * @static
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      static INSTANCE:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Style implementation for {@link yfiles.graph.INode} instances in an {@link yfiles.graph.IGraph}.
     * <p>
     * Implementations are responsible for rendering {@link yfiles.graph.INode} instances in a {@link yfiles.view.CanvasComponent}. It is up to
     * the implementation to interpret the visual appearance of a node. The framework uses the associated, possibly shared {@link yfiles.styles.INodeStyle#renderer} to
     * perform the actual rendering of this style for a given node. This interface extends the {@link yfiles.lang.ICloneable}
     * interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
     * implementations may return themselves.
     * </p>
     * <p>
     * Working with styles (and their style renderers) is explained in detail in the section {@link @DGUIDE_PREFIX@/view_visualization-graph-elements.html Visualization of Graph Elements}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 01 Custom Node Style</li>
     * </ul>
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface INodeStyle extends Object,yfiles.lang.ICloneable{
      /**
       * Gets the renderer implementation that can be queried for implementations that provide details about the visual
       * appearance and visual behavior for a given node and this style instance.
       * <p>
       * The idiom for retrieving, e.g. an {@link yfiles.view.IVisualCreator} implementation for a given style is:
       * </p>
       * @abstract
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
    }
    var INodeStyle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.styles.INodeStyle} implementations that can be used to query the actual
     * {@link yfiles.view.IVisualCreator}, {@link yfiles.view.IBoundsProvider}, {@link yfiles.input.IHitTestable}, and {@link yfiles.input.IMarqueeTestable}
     * implementations for a node and an associated style.
     * <p>
     * Implementations of this interface should be designed to be sharable between multiple style instances and should
     * therefore not contain style specific state.
     * </p>
     * @see yfiles.styles.INodeStyle
     * @see yfiles.graph.INode
     * @see yfiles.styles.IEdgeStyleRenderer
     * @see yfiles.styles.ILabelStyleRenderer
     * @see yfiles.styles.IPortStyleRenderer
     * @interface
     */
    export interface INodeStyleRenderer extends Object{
      /**
       * Gets an implementation of the {@link yfiles.view.IBoundsProvider} interface that can handle the provided node and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param node The node to provide an instance for
       * @param style The style to use for the calculating the painting bounds
       * @returns An implementation that may be used to subsequently query the node's painting bounds. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Gets a temporary context instance that can be used to query additional information for the node's style.
       * <p>
       * Implementations may return {@link yfiles.graph.ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
       * </p>
       * @param node The node to provide a context instance for.
       * @param style The style to use for the context.
       * @returns An non-<code>null</code> lookup implementation.
       * @see yfiles.graph.ILookup#EMPTY
       * @see yfiles.graph.ILookup
       * @abstract
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Gets an implementation of the {@link yfiles.input.IHitTestable} interface that can handle the provided node and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param node The node to provide an instance for
       * @param style The style to use for the querying hit tests
       * @returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
       * always call this method immediately before using the value returned. This enables the use of the flyweight design
       * pattern for implementations. This method may return <code>null</code> to indicate that the node cannot be hit tested.
       * @abstract
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * Gets an implementation of the {@link yfiles.input.IMarqueeTestable} interface that can handle the provided node and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param node The node to provide an instance for
       * @param style The style to use for the querying marquee intersection test.
       * @returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Gets an implementation of the {@link yfiles.styles.IShapeGeometry} interface that can handle the provided node and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param node The node to provide an instance for
       * @param style The style to use for the rendering
       * @returns An implementation that may be used to subsequently query geometry information from. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisibilityTestable} interface that can handle the provided node and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param node The node to provide an instance for
       * @param style The style to use for the testing the visibility
       * @returns An implementation that may be used to subsequently query the node's visibility. Clients should not cache this instance
       * and must always call this method immediately before using the value returned. This enables the use of the flyweight
       * design pattern for implementations
       * @abstract
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisualCreator} interface that can handle the provided node and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation, but never <code>null</code>.
       * </p>
       * @param node The node to provide an instance for
       * @param style The style to use for the creation of the visual
       * @returns An implementation that may be used to subsequently create or update the visual for the node. Clients should not cache
       * this instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link #INSTANCE void} implementation
       * instead.
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       * @abstract
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
    }
    var INodeStyleRenderer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that is used mainly by {@link yfiles.styles.IEdgeStyleRenderer} implementations to reveal geometric details about the
     * visualization of an edge's path.
     * @interface
     */
    export interface IPathGeometry extends Object{
      /**
       * Returns a representation of the visible path of the edge in form of a {@link yfiles.geometry.GeneralPath}
       * @returns An instance that describes the visible path or <code>null</code> if this is not applicable for the current geometry.
       * @abstract
       */
      getPath():yfiles.geometry.GeneralPath;
      /**
       * Returns the number of "segments" this edge's path consists of.
       * @returns the number of segments or <code>-1</code> if there is no such thing as a segment for this edge.
       * @abstract
       */
      getSegmentCount():number;
      /**
       * Calculates the tangent on the edge's path at the given ratio point for the given segment.
       * @param segmentIndex the segment to use for the calculation
       * @param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the segment
       * @returns The tangential line, if any exists.
       * @see yfiles.styles.IPathGeometry#getTangent
       * @see yfiles.styles.IPathGeometry#getSegmentCount
       * @abstract
       */
      getTangent(segmentIndex:number,ratio:number):yfiles.geometry.Tangent;
      /**
       * Calculates the tangent on the edge's path at the given ratio point.
       * @param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the visible edge path
       * @returns The tangential line, if any exists.
       * @abstract
       */
      getTangent(ratio:number):yfiles.geometry.Tangent;
    }
    var IPathGeometry:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A void implementation of a port style renderer that does nothing.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidPortStyleRenderer#INSTANCE} of this
     * class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.styles.IPortStyle#renderer} property of an
     * {@link yfiles.styles.IPortStyle}.
     * </p>
     * @class
     * @implements {yfiles.styles.IPortStyleRenderer}
     * @final
     */
    export interface VoidPortStyleRenderer extends Object,yfiles.styles.IPortStyleRenderer{}
    export class VoidPortStyleRenderer {
      /**
       * Yields the {@link yfiles.view.IBoundsProvider#EMPTY} that will return empty bounds.
       * @param port
       * @param style
       * @returns 
       */
      getBoundsProvider(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IBoundsProvider;
      /**
       * Yields the {@link yfiles.graph.ILookup#EMPTY} that will not yield anything.
       * @param port
       * @param style
       * @returns 
       */
      getContext(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.graph.ILookup;
      /**
       * Yields the {@link yfiles.input.IHitTestable#NEVER} that will always report misses.
       * @param port
       * @param style
       * @returns 
       */
      getHitTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.input.IHitTestable;
      /**
       * Yields the {@link yfiles.input.IMarqueeTestable#NEVER} that will always report misses.
       * @param port
       * @param style
       * @returns 
       */
      getMarqueeTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.input.IMarqueeTestable;
      /**
       * Yields the {@link yfiles.view.IVisibilityTestable#NEVER} that will always claim invisibility.
       * @param port
       * @param style
       * @returns 
       */
      getVisibilityTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IVisibilityTestable;
      /**
       * Yields the {@link yfiles.view.VoidVisualCreator#INSTANCE} that will do nothing.
       * @param port
       * @param style
       * @returns 
       */
      getVisualCreator(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IVisualCreator;
      /**
       * The {@link yfiles.styles.VoidPortStyleRenderer} singleton.
       * @const
       * @static
       * @type {yfiles.styles.IPortStyleRenderer}
       */
      static INSTANCE:yfiles.styles.IPortStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of a port style that does nothing and behaves like an invisible style.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidPortStyle#INSTANCE} of this class
     * instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.graph.IPort#style} property of an
     * {@link yfiles.graph.IPort}.
     * </p>
     * @class
     * @implements {yfiles.styles.IPortStyle}
     * @final
     */
    export interface VoidPortStyle extends Object,yfiles.styles.IPortStyle{}
    export class VoidPortStyle {
      /**
       * Returns <code>this</code>.
       * @returns <code>this</code>
       */
      clone():Object;
      /**
       * The {@link yfiles.styles.VoidPortStyle} singleton.
       * @const
       * @static
       * @type {yfiles.styles.VoidPortStyle}
       */
      static INSTANCE:yfiles.styles.VoidPortStyle;
      /**
       * Yields the {@link yfiles.styles.VoidPortStyleRenderer#INSTANCE VoidPortStyleRenderer instance}.
       * @see yfiles.styles.IPortStyle#renderer
       * @type {yfiles.styles.IPortStyleRenderer}
       */
      renderer:yfiles.styles.IPortStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Style implementation for {@link yfiles.graph.IPort} instances in an {@link yfiles.graph.IGraph}.
     * <p>
     * Implementations are responsible for rendering {@link yfiles.graph.IPort} instances in a {@link yfiles.view.CanvasComponent}. It is up to
     * the implementation to interpret the visual appearance of a port. The framework uses the associated, possibly shared {@link yfiles.styles.IPortStyle#renderer} to
     * perform the actual rendering of this style for a given port. This interface extends the {@link yfiles.lang.ICloneable}
     * interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
     * implementations may return themselves.
     * </p>
     * <p>
     * Working with styles (and their style renderers) is explained in detail in the section {@link @DGUIDE_PREFIX@/view_visualization-graph-elements.html Visualization of Graph Elements}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 22 Custom Ports</li>
     * </ul>
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface IPortStyle extends Object,yfiles.lang.ICloneable{
      /**
       * Gets the renderer implementation that can be queried for implementations that provide details about the visual
       * appearance and visual behavior for a given port and this style instance.
       * <p>
       * The idiom for retrieving, e.g. an {@link yfiles.view.IVisualCreator} implementation for a given style is:
       * </p>
       * @abstract
       * @type {yfiles.styles.IPortStyleRenderer}
       */
      renderer:yfiles.styles.IPortStyleRenderer;
    }
    var IPortStyle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.styles.IPortStyle} implementations that can be used to query the actual
     * {@link yfiles.view.IVisualCreator}, {@link yfiles.view.IBoundsProvider}, {@link yfiles.input.IHitTestable}, {@link yfiles.view.IVisibilityTestable}, and
     * {@link yfiles.input.IMarqueeTestable} implementations for a port and an associated style.
     * <p>
     * Implementations of this interface should be designed to be sharable between multiple style instances and should
     * therefore not contain style specific state.
     * </p>
     * @see yfiles.styles.IPortStyle
     * @see yfiles.graph.IPort
     * @see yfiles.styles.INodeStyleRenderer
     * @see yfiles.styles.IEdgeStyleRenderer
     * @see yfiles.styles.ILabelStyleRenderer
     * @interface
     */
    export interface IPortStyleRenderer extends Object{
      /**
       * Gets an implementation of the {@link yfiles.view.IBoundsProvider} interface that can handle the provided port and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param port The port to provide an instance for
       * @param style The style to use for the calculating the painting bounds
       * @returns An implementation that may be used to subsequently query the port's painting bounds. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getBoundsProvider(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IBoundsProvider;
      /**
       * Gets a temporary context instance that can be used to query additional information for the port's style.
       * <p>
       * Implementations may return {@link yfiles.graph.ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
       * </p>
       * @param port The port to provide a context instance for.
       * @param style The style to use for the context.
       * @returns An non-<code>null</code> lookup implementation.
       * @see yfiles.graph.ILookup#EMPTY
       * @see yfiles.graph.ILookup
       * @abstract
       */
      getContext(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.graph.ILookup;
      /**
       * Gets an implementation of the {@link yfiles.input.IHitTestable} interface that can handle the provided port and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param port The port to provide an instance for
       * @param style The style to use for the querying hit tests
       * @returns An implementation that may be used to subsequently perform hit tests. Clients should not cache this instance and must
       * always call this method immediately before using the value returned. This enables the use of the flyweight design
       * pattern for implementations. This method may return <code>null</code> to indicate that the port cannot be hit tested.
       * @abstract
       */
      getHitTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.input.IHitTestable;
      /**
       * Gets an implementation of the {@link yfiles.input.IMarqueeTestable} interface that can handle the provided port and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param port The port to provide an instance for
       * @param style The style to use for the querying marquee intersection test.
       * @returns An implementation that may be used to subsequently query the marquee intersections. Clients should not cache this
       * instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations
       * @abstract
       */
      getMarqueeTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.input.IMarqueeTestable;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisibilityTestable} interface that can handle the provided port and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation.
       * </p>
       * @param port The port to provide an instance for
       * @param style The style to use for the testing the visibility
       * @returns An implementation that may be used to subsequently query the port's visibility. Clients should not cache this instance
       * and must always call this method immediately before using the value returned. This enables the use of the flyweight
       * design pattern for implementations
       * @abstract
       */
      getVisibilityTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IVisibilityTestable;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisualCreator} interface that can handle the provided port and its associated
       * style.
       * <p>
       * This method may return a flyweight implementation, but never <code>null</code>.
       * </p>
       * @param port The port to provide an instance for
       * @param style The style to use for the creation of the visual
       * @returns An implementation that may be used to subsequently create or update the visual for the port. Clients should not cache
       * this instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link #INSTANCE void} implementation
       * instead.
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       * @abstract
       */
      getVisualCreator(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IVisualCreator;
    }
    var IPortStyleRenderer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that describes the geometry of a shape which has an interior and an exterior.
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Tutorial: 02 Custom Styles, step 04 IsInside</li>
     * <li>Tutorial: 02 Custom Styles, step 06 GetBounds</li>
     * </ul>
     * @interface
     */
    export interface IShapeGeometry extends Object{
      /**
       * Returns the intersection for the given line with this shape's geometry.
       * @param inner The first point of the line that is inside the shape.
       * @param outer The second point of the line that is outside the shape.
       * @returns The coordinates of the intersection point, if an intersection was found.
       * @abstract
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Returns the outline of the shape or <code>null</code>.
       * @returns The outline or <code>null</code> if no outline can be provided.
       * @abstract
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Checks whether the given coordinate is deemed to lie within the shape's geometric bounds.
       * @param location The point to test.
       * @returns True if the point lies within the shape.
       * @abstract
       */
      isInside(location:yfiles.geometry.Point):boolean;
    }
    var IShapeGeometry:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An {@link yfiles.view.IVisualCreator} for use in a {@link yfiles.view.CanvasComponent} that renders a {@link yfiles.graph.IGraph} instance in a
     * sloppy overview style.
     * @class
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface GraphOverviewSvgVisualCreator extends Object,yfiles.view.IVisualCreator{}
    export class GraphOverviewSvgVisualCreator {
      /**
       * Creates a new instance for the given graph.
       * @param graph
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Callback that returns and/or configures a edge style for the given edge to render.
       * @param edge The edge to render.
       * @returns The style or <code>null</code>
       * @protected
       */
      getEdgeStyle(edge:yfiles.graph.IEdge):yfiles.styles.IEdgeStyle;
      /**
       * Obtains the {@link yfiles.view.IVisualCreator} for the given edge.
       * @param context The context.
       * @param edge The edge.
       * @returns The visual creator that is obtained from the {@link #getEdgeStyle}'s {@link #getVisualCreator}
       * method.
       * @protected
       */
      getEdgeVisualCreator(context:yfiles.view.IRenderContext,edge:yfiles.graph.IEdge):yfiles.view.IVisualCreator;
      /**
       * Callback that returns and/or configures a group node style for the given node to render.
       * @param node The group node to render.
       * @returns The style or <code>null</code>
       * @protected
       */
      getGroupNodeStyle(node:yfiles.graph.INode):yfiles.styles.INodeStyle;
      /**
       * Obtains the {@link yfiles.view.IVisualCreator} for the given group node.
       * @param context The context.
       * @param node The group node.
       * @returns The visual creator that is obtained from the {@link #getGroupNodeStyle}'s {@link #getVisualCreator}
       * method.
       * @protected
       */
      getGroupNodeVisualCreator(context:yfiles.view.IRenderContext,node:yfiles.graph.INode):yfiles.view.IVisualCreator;
      /**
       * Callback that returns and/or configures a node style for the given node to render.
       * @param node The node to render.
       * @returns The style or <code>null</code>
       * @protected
       */
      getNodeStyle(node:yfiles.graph.INode):yfiles.styles.INodeStyle;
      /**
       * Obtains the {@link yfiles.view.IVisualCreator} for the given node.
       * @param context The context.
       * @param node The node.
       * @returns The visual creator that is obtained from the {@link #getNodeStyle}'s {@link #getVisualCreator}
       * method.
       * @protected
       */
      getNodeVisualCreator(context:yfiles.view.IRenderContext,node:yfiles.graph.INode):yfiles.view.IVisualCreator;
      /**
       * Gets or sets the overview node style.
       * @type {yfiles.styles.INodeStyle}
       */
      nodeStyle:yfiles.styles.INodeStyle;
      /**
       * Gets or sets the overview group node style.
       * @type {yfiles.styles.INodeStyle}
       */
      groupNodeStyle:yfiles.styles.INodeStyle;
      /**
       * Gets or sets the overview edge style.
       * @type {yfiles.styles.IEdgeStyle}
       */
      edgeStyle:yfiles.styles.IEdgeStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class that makes it possible to easily implement a custom {@link yfiles.styles.IEdgeStyle}.
     * <p>
     * The only method that needs to be implemented by subclasses is {@link yfiles.styles.EdgeStyleBase#createVisual}, however to
     * improve rendering performance it is highly recommended to implement at least
     * {@link yfiles.styles.EdgeStyleBase#updateVisual}, too.
     * </p>
     * <p>
     * This implementation differs from the straightforward {@link yfiles.styles.IEdgeStyle} implementation in that there is no visible
     * separation between the style and its
     * {@link yfiles.styles.IEdgeStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
     * the style instance.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 16 Custom Edge Style</li>
     * <li>Tutorial: 02 Custom Styles, step 17 Edge Hit Test</li>
     * <li>Tutorial: 02 Custom Styles, step 18 Edge Cropping</li>
     * </ul>
     * @class
     * @implements {yfiles.styles.IEdgeStyle}
     */
    export interface EdgeStyleBase extends Object,yfiles.styles.IEdgeStyle{}
    export class EdgeStyleBase {
      /**
       * Initializes a new instance of the {@link yfiles.styles.EdgeStyleBase} class.
       * @protected
       */
      constructor();
      /**
       * Convenience method that adds the arrows to a given container.
       * @param context The context for the rendering.
       * @param group The container to which the arrows should be added.
       * @param edge The edge that is being rendered.
       * @param edgePath The edge path.
       * @param sourceArrow The source arrow.
       * @param targetArrow The target arrow.
       * @protected
       */
      addArrows(context:yfiles.view.IRenderContext,group:SVGGElement,edge:yfiles.graph.IEdge,edgePath:yfiles.geometry.GeneralPath,sourceArrow:yfiles.styles.IArrow,targetArrow:yfiles.styles.IArrow):void;
      /**
       * Creates a new object that is a copy of the current instance.
       * <p>
       * Immutable subclasses should consider returning <code>this</code>.
       * </p>
       * @returns A new object that is a copy of this instance using {@link #memberwiseClone}.
       */
      clone():Object;
      /**
       * Callback that creates the visual.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#createVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.EdgeStyleBase#renderer}.
       * </p>
       * @param context The render context.
       * @param edge The edge to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.EdgeStyleBase#updateVisual
       * @protected
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext,edge:yfiles.graph.IEdge):yfiles.view.Visual;
      /**
       * Convenience method that crops the edge's path at the nodes.
       * <p>
       * This implementation uses the {@link yfiles.styles.IEdgePathCropper} instances found in the {@link yfiles.graph.ILookup#lookup} of the
       * source and target port of the edge to perform the actual cropping.
       * </p>
       * @param edge The edge that is being rendered.
       * @param path The path that should be cropped.
       * @param sourceArrow The source arrow instance.
       * @param targetArrow The target arrow instance.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       * @protected
       */
      cropPath(edge:yfiles.graph.IEdge,sourceArrow:yfiles.styles.IArrow,targetArrow:yfiles.styles.IArrow,path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
      /**
       * Gets the bounds of the visual for the edge in the given context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IBoundsProvider#getBounds} call to the instance that has been queried
       * from the {@link yfiles.styles.EdgeStyleBase#renderer}. This implementation simply yields a {@link yfiles.geometry.Rect rectangle} containing. the locations of the {@link yfiles.graph.IEdge#sourcePort source port} and the {@link yfiles.graph.IEdge#targetPort target port}
       * of the edge and the locations of all its {@link yfiles.graph.IEdge#bends bends}.
       * </p>
       * @param context The canvas context.
       * @param edge The edge to which this style instance is assigned.
       * @returns The visual bounds of the visual representation.
       * @protected
       */
      getBounds(context:yfiles.view.ICanvasContext,edge:yfiles.graph.IEdge):yfiles.geometry.Rect;
      /**
       * Gets the path of the edge.
       * @param edge The edge.
       * @returns The path.
       * @protected
       */
      getPath(edge:yfiles.graph.IEdge):yfiles.geometry.GeneralPath;
      /**
       * Gets the number of segments of the edge.
       * @param edge The edge.
       * @returns The segment count.
       * @protected
       */
      getSegmentCount(edge:yfiles.graph.IEdge):number;
      /**
       * Convenience method that calculates the source arrow anchor and direction for a a given arrow and path.
       * @param path The path of the edge.
       * @param arrow The arrow.
       * @returns The anchor and directional vector of the arrow, if any exist.
       * @protected
       */
      getSourceArrowAnchor(path:yfiles.geometry.GeneralPath,arrow:yfiles.styles.IArrow):yfiles.geometry.Tangent;
      /**
       * Gets the tangent to the edge at the specified ratio of a segment of the edge and the corresponding touch point.
       * @param edge The edge.
       * @param segmentIndex Index of the segment of the edge.
       * @param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of the segment of the edge.
       * @returns The tangent, if any exists.
       * @protected
       */
      getTangent(edge:yfiles.graph.IEdge,segmentIndex:number,ratio:number):yfiles.geometry.Tangent;
      /**
       * Gets the tangent to the edge at the specified ratio and the corresponding touch point.
       * @param edge The edge.
       * @param ratio A value between 0 and 1 inclusively that indicates a ratio from the beginning to the end of the path of the edge.
       * @returns The tangent, if any exists.
       * @protected
       */
      getTangent(edge:yfiles.graph.IEdge,ratio:number):yfiles.geometry.Tangent;
      /**
       * Convenience method that calculates the target arrow anchor and direction for a a given arrow and path.
       * @param path The path of the edge.
       * @param arrow The arrow.
       * @returns The anchor and directional vector of the arrow, if any exist.
       * @protected
       */
      getTargetArrowAnchor(path:yfiles.geometry.GeneralPath,arrow:yfiles.styles.IArrow):yfiles.geometry.Tangent;
      /**
       * Determines whether the visual representation of the edge has been hit at the given location.
       * <p>
       * This method is called in response to a {@link yfiles.input.IHitTestable#isHit} call to the instance that has been queried from
       * the {@link yfiles.styles.EdgeStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation returns the result of the {@link yfiles.geometry.Point#hitsPolyline} method of class {@link yfiles.geometry.Point}
       * with the polygonal line defined by the source port, the target port and the bends of the edge and the {@link yfiles.view.ICanvasContext#hitTestRadius} of the
       * {@link yfiles.view.ICanvasContext canvas context}.
       * </p>
       * @param context The input mode context.
       * @param location The point to test.
       * @param edge The edge to which this style instance is assigned.
       * @returns <code>true</code> if the specified edge representation is hit; <code>false</code> otherwise.
       * @see yfiles.geometry.Point#hitsPolyline
       * @protected
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,edge:yfiles.graph.IEdge):boolean;
      /**
       * Determines whether the visualization for the specified edge is included in the marquee selection.
       * <p>
       * This method is called in response to a {@link yfiles.input.IMarqueeTestable#isInBox} call to the instance that has been queried
       * from the {@link yfiles.styles.EdgeStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation returns the result of the {@link yfiles.geometry.Rect#intersectsPolyline} method of class {@link yfiles.geometry.Rect}
       * with the polygonal line defined by the source port, the target port and the bends of the edge.
       * </p>
       * @param context The input mode context.
       * @param rectangle The marquee selection box.
       * @param edge The edge to which this style instance is assigned.
       * @returns <code>true</code> if the specified edge is visible is selected by the marquee rectangle; <code>false</code> otherwise.
       * @protected
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect,edge:yfiles.graph.IEdge):boolean;
      /**
       * Determines whether the visualization for the specified edge is visible in the context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisibilityTestable#isVisible} call to the instance that has been
       * queried from the {@link yfiles.styles.EdgeStyleBase#renderer}. This implementation simply tests whether the {@link yfiles.styles.EdgeStyleBase#getBounds bounds} intersect the clip.
       * </p>
       * @param context The canvas context.
       * @param rectangle The clipping rectangle.
       * @param edge The edge to which this style instance is assigned.
       * @returns <code>true</code> if the specified edge is visible in the clipping rectangle; <code>false</code> otherwise.
       * @protected
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect,edge:yfiles.graph.IEdge):boolean;
      /**
       * Performs the {@link yfiles.graph.ILookup#lookup} operation for the {@link yfiles.styles.IEdgeStyleRenderer#getContext} that has been
       * queried from the {@link yfiles.styles.EdgeStyleBase#renderer}.
       * <p>
       * This implementation yields <code>null</code> for everything but:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.IHitTestable}</li>
       * <li>{@link yfiles.view.IVisualCreator}</li>
       * <li>{@link yfiles.view.IBoundsProvider}</li>
       * <li>{@link yfiles.view.IVisibilityTestable}</li>
       * <li>{@link yfiles.input.IMarqueeTestable}</li>
       * <li>{@link yfiles.graph.ILookup}</li>
       * <li>{@link yfiles.styles.IPathGeometry}</li>
       * </ul>
       * <p>
       * For these interfaces an implementation will be returned that delegates to the methods in this instance.
       * </p>
       * @param edge The edge to use for the context lookup.
       * @param type The type to query.
       * @returns An implementation of the <code>type</code> or <code>null</code>.
       * @protected
       */
      lookup(edge:yfiles.graph.IEdge,type:yfiles.lang.Class):Object;
      /**
       * Convenience method that updates the arrows in a given container.
       * @param context The context for the rendering.
       * @param group The container to which the arrows should be added.
       * @param edge The edge that is being rendered.
       * @param edgePath The edge path.
       * @param sourceArrow The source arrow.
       * @param targetArrow The target arrow.
       * @protected
       */
      updateArrows(context:yfiles.view.IRenderContext,group:SVGGElement,edge:yfiles.graph.IEdge,edgePath:yfiles.geometry.GeneralPath,sourceArrow:yfiles.styles.IArrow,targetArrow:yfiles.styles.IArrow):void;
      /**
       * Callback that updates the visual previously created by {@link yfiles.styles.EdgeStyleBase#createVisual}.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#updateVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.EdgeStyleBase#renderer}. This implementation simply delegates to {@link yfiles.styles.EdgeStyleBase#createVisual} so subclasses should
       * override to improve rendering performance.
       * </p>
       * @param context The render context.
       * @param oldVisual The visual that has been created in the call to {@link #createVisual}.
       * @param edge The edge to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.EdgeStyleBase#createVisual
       * @protected
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual,edge:yfiles.graph.IEdge):yfiles.view.Visual;
      /**
       * Gets the renderer implementation for this instance.
       * <p>
       * The private implementation will delegate all API calls back to this instance.
       * </p>
       * @type {yfiles.styles.IEdgeStyleRenderer}
       */
      renderer:yfiles.styles.IEdgeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Decorates an arbitrary node styles with an additional collapse/expand handle.
     * <p>
     * This style uses the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer} to adorn an existing {@link yfiles.styles.INodeStyle}
     * with a <code>Button</code> that can then be used to trigger the {@link yfiles.input.ICommand#TOGGLE_EXPANSION_STATE}.
     * </p>
     * <p>
     * By default, this handle is rendered as a plus ("+") sign when in collapsed state, and a minus ("-") sign when in
     * expanded state.
     * </p>
     * <p>
     * The collapse button can be styled using the CSS class selectors 'yfiles-collapsebutton', 'yfiles-collapsebutton-checked'
     * and 'yfiles-collapsebutton-unchecked'.
     * </p>
     * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface CollapsibleNodeStyleDecorator extends Object,yfiles.styles.INodeStyle{}
    export class CollapsibleNodeStyleDecorator {
      /**
       * Initializes a new instance of the {@link yfiles.styles.CollapsibleNodeStyleDecorator} class using the provided style for the {@link yfiles.styles.CollapsibleNodeStyleDecorator#wrapped}
       * property and the provided {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer} as renderer.
       * @param wrapped The decorated style.
       * @param [renderer=null] An instance of {@link } to render this style.
       */
      constructor(wrapped:yfiles.styles.INodeStyle,renderer?:yfiles.styles.CollapsibleNodeStyleDecoratorRenderer);
      /**
       * Initializes a new instance of the {@link yfiles.styles.CollapsibleNodeStyleDecorator} class using the provided style for the {@link yfiles.styles.CollapsibleNodeStyleDecorator#wrapped}
       * property and the provided {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer} as renderer.
       * @param {Object} options The parameters to pass.
       * @param options.wrapped The decorated style.
       * @param [options.renderer=null] An instance of {@link } to render this style.
       * @param {yfiles.graph.ILabelModelParameter} options.buttonPlacement An {@link yfiles.graph.ILabelModelParameter} that determines the placement of the button for toggling the expanded state.
       * <p>
       * This option sets the {@link yfiles.styles.CollapsibleNodeStyleDecorator#buttonPlacement} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.insets The insets to use for the {@link yfiles.input.INodeInsetsProvider} that will be reported by the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer}'s {@link yfiles.graph.ILookup}.
       * <p>
       * This option sets the {@link yfiles.styles.CollapsibleNodeStyleDecorator#insets} property on the created object.
       * </p>
       */
      constructor(options:{wrapped:yfiles.styles.INodeStyle,renderer?:yfiles.styles.CollapsibleNodeStyleDecoratorRenderer,buttonPlacement?:yfiles.graph.ILabelModelParameter,insets?:yfiles.geometry.Insets});
      /**
       * Initializes a new instance of the {@link yfiles.styles.CollapsibleNodeStyleDecorator} class.
       * <p>
       * The {@link yfiles.styles.CollapsibleNodeStyleDecorator#wrapped} property will be initialized with a {@link yfiles.styles.ShapeNodeStyle}.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.graph.ILabelModelParameter} options.buttonPlacement An {@link yfiles.graph.ILabelModelParameter} that determines the placement of the button for toggling the expanded state.
       * <p>
       * This option sets the {@link yfiles.styles.CollapsibleNodeStyleDecorator#buttonPlacement} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.insets The insets to use for the {@link yfiles.input.INodeInsetsProvider} that will be reported by the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer}'s {@link yfiles.graph.ILookup}.
       * <p>
       * This option sets the {@link yfiles.styles.CollapsibleNodeStyleDecorator#insets} property on the created object.
       * </p>
       */
      constructor(options?:{buttonPlacement?:yfiles.graph.ILabelModelParameter,insets?:yfiles.geometry.Insets});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets an {@link yfiles.graph.ILabelModelParameter} that determines the placement of the button for toggling the expanded
       * state.
       * <p>
       * The {@link yfiles.graph.ILabelModelParameter} should {@link yfiles.graph.ILabelModelParameter#supports support} labels bound to an {@link yfiles.graph.INode} because the button will be
       * positioned as if it was an {@link yfiles.graph.ILabel} owned by the {@link yfiles.graph.INode} that uses this style.
       * </p>
       * @default 'yWorks.yFiles.UI.LabelModels.InteriorLabelModel.NorthWest'
       * @type {yfiles.graph.ILabelModelParameter}
       */
      buttonPlacement:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the insets to use for the {@link yfiles.input.INodeInsetsProvider} that will be reported by the
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer}'s {@link yfiles.graph.ILookup}.
       * <p>
       * This effectively influences the way the bounds of a group node are being calculated. The default {@link yfiles.graph.IGroupBoundsCalculator}
       * implementation will use these insets to determine the minimum size of a group node given a set of contained nodes.
       * </p>
       * @default '5, 16, 5, 5'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      /**
       * Gets or sets the wrapped node style that will be used to perform the actual rendering of the node.
       * @type {yfiles.styles.INodeStyle}
       */
      wrapped:yfiles.styles.INodeStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.styles.INodeStyleRenderer} implementation that can be used in conjunction with {@link yfiles.styles.CollapsibleNodeStyleDecorator}
     * instances.
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface CollapsibleNodeStyleDecoratorRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class CollapsibleNodeStyleDecoratorRenderer {
      /**
       * Initializes a new instance of the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer} class.
       */
      constructor();
      /**
       * Adds the toggle expansion state command to the given button visual.
       * <p>
       * This method adds event listeners for click and tap events to the given button visual that execute
       * {@link yfiles.input.ICommand#TOGGLE_EXPANSION_STATE}. It is called by
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#createButton}.
       * </p>
       * @param button The button visual to add the event listeners to.
       * @param currentNode The group node whose state is to be toggled.
       * @param context The context.
       * @static
       */
      static addToggleExpansionStateCommand(button:yfiles.view.SvgVisual,currentNode:yfiles.graph.INode,context:yfiles.view.IRenderContext):void;
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#style} and {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#node} properies have been populated by the
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getVisualCreator},
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getBoundsProvider},
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getHitTestable}, or {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * Creates the button visual.
       * <p>
       * This method is called from {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#createVisual} to create the button. Custom
       * implementations can override this method to use a custom button visualization.
       * </p>
       * <p>
       * This implementation adds the {@link yfiles.input.ICommand#TOGGLE_EXPANSION_STATE} command to the button. The created button is
       * automatically unchecked if the group node is {@link yfiles.graph.IFoldingView#isExpanded expanded} and vice versa.
       * </p>
       * @param context The context.
       * @param expanded Specifies whether the button visual is created for the expanded or the collapsed state.
       * @param size The button size.
       * @returns 
       * @protected
       */
      createButton(context:yfiles.view.IRenderContext,expanded:boolean,size:yfiles.geometry.Size):yfiles.view.SvgVisual;
      /**
       * Callback that provides a customized {@link yfiles.view.ISelectionIndicatorInstaller}.
       * @returns This implementation yields a {@link } that uses
       * {@link #SELECTION_TEMPLATE_KEY}.
       * @protected
       */
      createSelectionInstaller():yfiles.view.ISelectionIndicatorInstaller;
      /**
       * Creates the visual for the given context.
       * <p>
       * This method uses {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#createButton} to create the togglebutton instance.
       * It will then use the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getButtonLocationParameter} to determine the
       * placement of the button and use the {@link yfiles.styles.CollapsibleNodeStyleDecorator#wrapped wrapped style} to create the visual for the actual node.
       * </p>
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle} and takes the button into account.
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and
       * returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Yields the {@link yfiles.styles.CollapsibleNodeStyleDecorator#buttonPlacement} for the current {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#style}
       * @returns 
       * @protected
       */
      getButtonLocationParameter():yfiles.graph.ILabelModelParameter;
      /**
       * Returns the size of new buttons.
       * @returns The size of new buttons
       * @protected
       */
      getButtonSize():yfiles.geometry.Size;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and
       * returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure}, instead subclasses should ensure that in the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#lookup}
       * method call they should call {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} only if needed, i.e. if they
       * decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and
       * returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and
       * returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Configures the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#style} and {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#node} properties, calls {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#isInside
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and
       * returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call
       * {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure}. If the subclass implementation depends on this instance
       * to be configured, it needs to call <code>Configure</code> in {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#configure} and
       * returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#createVisual
       * @see yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * Yields the {@link yfiles.styles.CollapsibleNodeStyleDecorator#wrapped} property for the current {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#style}
       * @returns 
       * @protected
       */
      getWrappedStyle():yfiles.styles.INodeStyle;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle} and checks the button.
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * Delegates to the {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#getWrappedStyle}.
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Updates the button visual.
       * <p>
       * This method is called from {@link yfiles.styles.CollapsibleNodeStyleDecoratorRenderer#updateVisual} to update an existing button
       * or to create a new one. Custom implementations can override this method to use a custom button visualization.
       * </p>
       * @param context The context.
       * @param expanded Specifies whether the button visual is updated for the expanded or the collapsed state.
       * @param size The button size.
       * @param oldButton The old button visual.
       * @returns 
       * @protected
       */
      updateButton(context:yfiles.view.IRenderContext,expanded:boolean,size:yfiles.geometry.Size,oldButton:yfiles.view.Visual):yfiles.view.SvgVisual;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.CollapsibleNodeStyleDecorator}
       */
      style:yfiles.styles.CollapsibleNodeStyleDecorator;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class that makes it possible to easily implement a custom {@link yfiles.styles.ILabelStyle}.
     * <p>
     * Only {@link yfiles.styles.LabelStyleBase#createVisual} and {@link yfiles.styles.LabelStyleBase#getPreferredSize} need to be implemented
     * by subclasses, however to improve rendering performance it is highly recommended to implement at least
     * {@link yfiles.styles.LabelStyleBase#updateVisual}, too.
     * </p>
     * <p>
     * This implementation differs from the straightforward {@link yfiles.styles.ILabelStyle} implementation in that there is no
     * visible separation between the style and its
     * {@link yfiles.styles.ILabelStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
     * the style instance.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 10 Custom Label Style</li>
     * <li>Tutorial: 02 Custom Styles, step 11 Label Preferred Size</li>
     * <li>Tutorial: 02 Custom Styles, step 12 High Performance Rendering of Label</li>
     * </ul>
     * @class
     * @implements {yfiles.styles.ILabelStyle}
     */
    export interface LabelStyleBase extends Object,yfiles.styles.ILabelStyle{}
    export class LabelStyleBase {
      /**
       * Initializes a new instance of the {@link yfiles.styles.LabelStyleBase} class.
       * @protected
       */
      constructor();
      /**
       * Creates a new object that is a copy of the current instance.
       * <p>
       * Immutable subclasses should consider returning <code>this</code>.
       * </p>
       * @returns A new object that is a copy of this instance using {@link #memberwiseClone}.
       */
      clone():Object;
      /**
       * Creates a transform matrix that can be applied to a {@link yfiles.view.Visual} to arrange it according to the given <code>layout</code> and <code>autoFlip</code>
       * rule.
       * @param layout The layout used to create the transform matrix.
       * @param autoFlip Whether to automatically flip the mtrix by 180 degrees, if the result would be pointing downwards.
       * @returns The transform matrix.
       * @static
       */
      static createLayoutTransform(layout:yfiles.geometry.IOrientedRectangle,autoFlip:boolean):yfiles.geometry.Matrix;
      /**
       * Callback that creates the visual.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#createVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.LabelStyleBase#renderer}.
       * </p>
       * @param context The render context.
       * @param label The label to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.LabelStyleBase#updateVisual
       * @protected
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext,label:yfiles.graph.ILabel):yfiles.view.Visual;
      /**
       * Gets the bounds of the visual for the label in the given context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IBoundsProvider#getBounds} call to the instance that has been queried
       * from the {@link yfiles.styles.LabelStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation simply yields the {@link yfiles.geometry.IOrientedRectangle#bounds bounds} of the {@link yfiles.graph.ILabel#layout layout} of the given label.
       * </p>
       * @param context The canvas context.
       * @param label The label to which this style instance is assigned.
       * @returns The visual bounds of the visual representation.
       * @protected
       */
      getBounds(context:yfiles.view.ICanvasContext,label:yfiles.graph.ILabel):yfiles.geometry.Rect;
      /**
       * Callback that returns the preferred {@link yfiles.geometry.Size size} of the label.
       * @param label The label to which this style instance is assigned.
       * @returns The preferred size.
       * @protected
       * @abstract
       */
      getPreferredSize(label:yfiles.graph.ILabel):yfiles.geometry.Size;
      /**
       * Determines whether the visual representation of the label has been hit at the given location.
       * <p>
       * This method is called in response to a {@link yfiles.input.IHitTestable#isHit} call to the instance that has been queried from
       * the {@link yfiles.styles.LabelStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation returns the result of the {@link yfiles.geometry.IOrientedRectangle#hits} method invoked for the {@link yfiles.graph.ILabel#layout layout} of
       * the label, the given {@link yfiles.geometry.Point point} and the {@link yfiles.view.ICanvasContext#hitTestRadius hit test radius} of the {@link yfiles.view.ICanvasContext canvas context}.
       * </p>
       * @param context The input mode context.
       * @param location The point to test.
       * @param label The label to which this style instance is assigned.
       * @returns <code>true</code> if the specified label representation is hit; <code>false</code> otherwise.
       * @protected
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,label:yfiles.graph.ILabel):boolean;
      /**
       * Determines whether the visualization for the specified label is included in the marquee selection.
       * <p>
       * This method is called in response to a {@link yfiles.input.IMarqueeTestable#isInBox} call to the instance that has been queried
       * from the {@link yfiles.styles.LabelStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation returns the result of the {@link yfiles.geometry.Rect#intersects} method invoked on <code>rectangle</code> for the {@link yfiles.graph.ILabel#layout layout}
       * of the label and the <code>context</code>.
       * </p>
       * @param context The input mode context.
       * @param rectangle The marquee selection box.
       * @param label The label to which this style instance is assigned.
       * @returns <code>true</code> if the specified label is visible is selected by the marquee rectangle; <code>false</code> otherwise.
       * @protected
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect,label:yfiles.graph.ILabel):boolean;
      /**
       * Determines whether the visualization for the specified label is visible in the context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisibilityTestable#isVisible} call to the instance that has been
       * queried from the {@link yfiles.styles.LabelStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation simply tests whether the {@link yfiles.styles.LabelStyleBase#getBounds bounds} intersect the clip.
       * </p>
       * @param context The canvas context.
       * @param rectangle The clipping rectangle.
       * @param label The label to which this style instance is assigned.
       * @returns <code>true</code> if the specified label is visible in the clipping rectangle; <code>false</code> otherwise.
       * @protected
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect,label:yfiles.graph.ILabel):boolean;
      /**
       * Performs the {@link yfiles.graph.ILookup#lookup} operation for the {@link yfiles.styles.ILabelStyleRenderer#getContext} that has been
       * queried from the {@link yfiles.styles.LabelStyleBase#renderer}.
       * <p>
       * This implementation yields <code>null</code> for everything but:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.IHitTestable}</li>
       * <li>{@link yfiles.view.IVisualCreator}</li>
       * <li>{@link yfiles.view.IBoundsProvider}</li>
       * <li>{@link yfiles.view.IVisibilityTestable}</li>
       * <li>{@link yfiles.input.IMarqueeTestable}</li>
       * <li>{@link yfiles.graph.ILookup}</li>
       * </ul>
       * <p>
       * For these interfaces an implementation will be returned that delegates to the methods in this instance.
       * </p>
       * @param label The label to use for the context lookup.
       * @param type The type to query.
       * @returns An implementation of the <code>type</code> or <code>null</code>.
       * @protected
       */
      lookup(label:yfiles.graph.ILabel,type:yfiles.lang.Class):Object;
      /**
       * Callback that updates the visual previously created by {@link yfiles.styles.LabelStyleBase#createVisual}.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#updateVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.LabelStyleBase#renderer}. This implementation simply delegates to {@link yfiles.styles.LabelStyleBase#createVisual} so subclasses
       * should override to improve rendering performance.
       * </p>
       * @param context The render context.
       * @param oldVisual The visual that has been created in the call to {@link #createVisual}.
       * @param label The label to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.LabelStyleBase#createVisual
       * @protected
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual,label:yfiles.graph.ILabel):yfiles.view.Visual;
      /**
       * Gets the renderer implementation for this instance.
       * <p>
       * The private implementation will delegate all API calls back to this instance.
       * </p>
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class that makes it possible to easily implement a custom {@link yfiles.styles.INodeStyle}.
     * <p>
     * The only method that needs to be implemented by subclasses is {@link yfiles.styles.NodeStyleBase#createVisual}, however to
     * improve rendering performance it is highly recommended to implement at least
     * {@link yfiles.styles.NodeStyleBase#updateVisual}, too.
     * </p>
     * <p>
     * This implementation differs from the straightforward {@link yfiles.styles.INodeStyle} implementation in that there is no visible
     * separation between the style and its
     * {@link yfiles.styles.INodeStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
     * the style instance.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 01 Custom Node Style</li>
     * <li>Tutorial: 02 Custom Styles, step 03 UpdateVisual and RenderDataCache</li>
     * <li>Tutorial: 02 Custom Styles, step 05 Hit Test</li>
     * </ul>
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface NodeStyleBase extends Object,yfiles.styles.INodeStyle{}
    export class NodeStyleBase {
      /**
       * Initializes a new instance of the {@link yfiles.styles.NodeStyleBase} class.
       * @protected
       */
      constructor();
      /**
       * Creates a new object that is a copy of the current instance.
       * <p>
       * Immutable subclasses should consider returning <code>this</code>.
       * </p>
       * @returns A new object that is a copy of this instance using {@link #memberwiseClone}.
       */
      clone():Object;
      /**
       * Callback that creates the visual.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#createVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.NodeStyleBase#renderer}.
       * </p>
       * @param context The render context.
       * @param node The node to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.NodeStyleBase#updateVisual
       * @protected
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext,node:yfiles.graph.INode):yfiles.view.Visual;
      /**
       * Gets the bounds of the visual for the node in the given context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IBoundsProvider#getBounds} call to the instance that has been queried
       * from the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation simply yields the {@link yfiles.graph.INode#layout}.
       * </p>
       * @param context The canvas context.
       * @param node The node to which this style instance is assigned.
       * @returns The visual bounds of the visual representation.
       * @protected
       */
      getBounds(context:yfiles.view.ICanvasContext,node:yfiles.graph.INode):yfiles.geometry.Rect;
      /**
       * Gets the intersection of a line with the visual representation of the node.
       * <p>
       * This method is called in response to a {@link yfiles.styles.IShapeGeometry#getIntersection} call to the instance that has been
       * queried from the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation simply uses the {@link yfiles.styles.NodeStyleBase#getOutline outline} to determine the intersection or the {@link yfiles.graph.INode#layout} if the outline is
       * <code>null</code>. If it is feasible to determine the intersection point for the current shape, this method should be implemented in
       * addition to {@link yfiles.styles.NodeStyleBase#getOutline} to improve performance.
       * </p>
       * @param node The node to which this style instance is assigned.
       * @param inner The coordinates of a point lying {@link #isInside inside} the shape.
       * @param outer The coordinates of a point lying outside the shape.
       * @returns The intersection point if one has been found or <code>null</code>, otherwise.
       * @see yfiles.styles.NodeStyleBase#isInside
       * @protected
       */
      getIntersection(node:yfiles.graph.INode,inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Gets the outline of the visual style.
       * <p>
       * This implementation yields <code>null</code> to indicate that the {@link yfiles.graph.INode#layout} depicts the outline. Implementing this method influences the
       * behavior of {@link yfiles.styles.NodeStyleBase#isInside} and {@link yfiles.styles.NodeStyleBase#getIntersection} since the default
       * implementations delegate to it.
       * </p>
       * @param node The node to which this style instance is assigned.
       * @returns The outline of the visual representation or <code>null</code>.
       * @protected
       */
      getOutline(node:yfiles.graph.INode):yfiles.geometry.GeneralPath;
      /**
       * Determines whether the visual representation of the node has been hit at the given location.
       * <p>
       * This method is called in response to a {@link yfiles.input.IHitTestable#isHit} call to the instance that has been queried from
       * the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation uses the {@link yfiles.styles.NodeStyleBase#getOutline outline} to determine whether the node has been hit.
       * </p>
       * @param context The canvas context.
       * @param location The point to test.
       * @param node The node to which this style instance is assigned.
       * @returns <code>true</code> if the specified node representation is hit; <code>false</code> otherwise.
       * @protected
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,node:yfiles.graph.INode):boolean;
      /**
       * Determines whether the visualization for the specified node is included in the marquee selection.
       * <p>
       * This method is called in response to a {@link yfiles.input.IMarqueeTestable#isInBox} call to the instance that has been queried
       * from the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation simply tests whether the {@link yfiles.styles.NodeStyleBase#getBounds bounds} intersect the marquee box.
       * </p>
       * @param context The input mode context.
       * @param rectangle The marquee selection box.
       * @param node The node to which this style instance is assigned.
       * @returns <code>true</code> if the specified node is visible is selected by the marquee rectangle; <code>false</code> otherwise.
       * @protected
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect,node:yfiles.graph.INode):boolean;
      /**
       * Determines whether the provided point is geometrically inside the visual bounds of the node.
       * <p>
       * This method is called in response to a {@link yfiles.styles.IShapeGeometry#isInside} call to the instance that has been queried
       * from the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation simply uses the {@link yfiles.styles.NodeStyleBase#getOutline outline} to determine whether the point is contained or the {@link yfiles.graph.INode#layout} if the
       * outline is <code>null</code>. If it is feasible to determine whether a given point lies inside the shape, this method should be
       * implemented in addition to {@link yfiles.styles.NodeStyleBase#getOutline} to improve performance.
       * </p>
       * @param node The node to which this style instance is assigned.
       * @param location The point to test.
       * @returns Whether the point is considered to lie inside the shape.
       * @protected
       */
      isInside(node:yfiles.graph.INode,location:yfiles.geometry.Point):boolean;
      /**
       * Determines whether the visualization for the specified node is visible in the context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisibilityTestable#isVisible} call to the instance that has been
       * queried from the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation simply tests whether the {@link yfiles.styles.NodeStyleBase#getBounds bounds} intersect the clip.
       * </p>
       * @param context The canvas context.
       * @param rectangle The clipping rectangle.
       * @param node The node to which this style instance is assigned.
       * @returns <code>true</code> if the specified node is visible in the clipping rectangle; <code>false</code> otherwise.
       * @protected
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect,node:yfiles.graph.INode):boolean;
      /**
       * Performs the {@link yfiles.graph.ILookup#lookup} operation for the {@link yfiles.styles.INodeStyleRenderer#getContext} that has been
       * queried from the {@link yfiles.styles.NodeStyleBase#renderer}.
       * <p>
       * This implementation yields <code>null</code> for everything but:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.IHitTestable}</li>
       * <li>{@link yfiles.view.IVisualCreator}</li>
       * <li>{@link yfiles.view.IBoundsProvider}</li>
       * <li>{@link yfiles.view.IVisibilityTestable}</li>
       * <li>{@link yfiles.input.IMarqueeTestable}</li>
       * <li>{@link yfiles.graph.ILookup}</li>
       * <li>{@link yfiles.styles.IShapeGeometry}</li>
       * </ul>
       * <p>
       * For these interfaces an implementation will be returned that delegates to the methods in this instance.
       * </p>
       * @param node The node to use for the context lookup.
       * @param type The type to query.
       * @returns An implementation of the <code>type</code> or <code>null</code>.
       * @protected
       */
      lookup(node:yfiles.graph.INode,type:yfiles.lang.Class):Object;
      /**
       * Callback that updates the visual previously created by {@link yfiles.styles.NodeStyleBase#createVisual}.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#updateVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.NodeStyleBase#renderer}. This implementation simply delegates to {@link yfiles.styles.NodeStyleBase#createVisual} so subclasses should
       * override to improve rendering performance.
       * </p>
       * @param context The render context.
       * @param oldVisual The visual that has been created in the call to {@link #createVisual}.
       * @param node The node to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.NodeStyleBase#createVisual
       * @protected
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual,node:yfiles.graph.INode):yfiles.view.Visual;
      /**
       * Gets the renderer implementation for this instance.
       * <p>
       * The private implementation will delegate all API calls back to this instance.
       * </p>
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A label style decorator that uses a node style instance to render the background and a label style instance to render
     * the foreground of a label.
     * @class
     * @implements {yfiles.styles.ILabelStyle}
     */
    export interface NodeStyleLabelStyleAdapter extends Object,yfiles.styles.ILabelStyle{}
    export class NodeStyleLabelStyleAdapter {
      /**
       * Creates a label style that uses the provided node style to render the background and the label style to render the
       * foreground of this style.
       * <p>
       * Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
       * instance.
       * </p>
       * @param nodeStyle The style to use for rendering the background of the label.
       * @param labelStyle The style to use for rendering the foreground of the label.
       */
      constructor(nodeStyle:yfiles.styles.INodeStyle,labelStyle:yfiles.styles.ILabelStyle);
      /**
       * Creates a label style that uses the provided node style to render the background and the label style to render the
       * foreground of this style.
       * <p>
       * Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
       * instance.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.nodeStyle The style to use for rendering the background of the label.
       * @param options.labelStyle The style to use for rendering the foreground of the label.
       * @param {boolean} options.autoFlip A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleLabelStyleAdapter#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.labelStyleInsets The insets to apply for the {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyle} as margins.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyleInsets} property on the created object.
       * </p>
       */
      constructor(options:{nodeStyle:yfiles.styles.INodeStyle,labelStyle:yfiles.styles.ILabelStyle,autoFlip?:boolean,labelStyleInsets?:yfiles.geometry.Insets});
      /**
       * Initializes a new instance of the {@link yfiles.styles.NodeStyleLabelStyleAdapter} class.
       * <p>
       * This constructor uses a {@link yfiles.styles.ShapeNodeStyle} and a {@link yfiles.styles.DefaultLabelStyle} for the {@link yfiles.styles.NodeStyleLabelStyleAdapter#nodeStyle} and {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyle} properties.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.autoFlip A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleLabelStyleAdapter#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.styles.ILabelStyle} options.labelStyle The {@link yfiles.styles.ILabelStyle} that is used for rendering the foreground of the label.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyle} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.labelStyleInsets The insets to apply for the {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyle} as margins.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyleInsets} property on the created object.
       * </p>
       * @param {yfiles.styles.INodeStyle} options.nodeStyle The {@link yfiles.styles.INodeStyle} that is used for rendering the background of the label.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleLabelStyleAdapter#nodeStyle} property on the created object.
       * </p>
       */
      constructor(options?:{autoFlip?:boolean,labelStyle?:yfiles.styles.ILabelStyle,labelStyleInsets?:yfiles.geometry.Insets,nodeStyle?:yfiles.styles.INodeStyle});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * 
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
      /**
       * Gets the {@link yfiles.styles.INodeStyle} that is used for rendering the background of the label.
       * @type {yfiles.styles.INodeStyle}
       */
      nodeStyle:yfiles.styles.INodeStyle;
      /**
       * Gets the {@link yfiles.styles.ILabelStyle} that is used for rendering the foreground of the label.
       * @type {yfiles.styles.ILabelStyle}
       */
      labelStyle:yfiles.styles.ILabelStyle;
      /**
       * Gets a value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented
       * downwards, otherwise.
       * @default true
       * @type {boolean}
       */
      autoFlip:boolean;
      /**
       * Gets or sets the insets to apply for the {@link yfiles.styles.NodeStyleLabelStyleAdapter#labelStyle} as margins.
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      labelStyleInsets:yfiles.geometry.Insets;
      static $class:yfiles.lang.Class;
    }
    /**
     * A port style decorator that uses a node style instance to render the port.
     * @class
     * @implements {yfiles.styles.IPortStyle}
     */
    export interface NodeStylePortStyleAdapter extends Object,yfiles.styles.IPortStyle{}
    export class NodeStylePortStyleAdapter {
      /**
       * Creates a port style that uses the provided node style to render the port.
       * <p>
       * Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
       * instance.
       * </p>
       * @param nodeStyle The style to use for rendering the port.
       */
      constructor(nodeStyle:yfiles.styles.INodeStyle);
      /**
       * Creates a port style that uses the provided node style to render the port.
       * <p>
       * Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
       * instance.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.nodeStyle The style to use for rendering the port.
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.MutableSize} of the port that will be displayed using {@link yfiles.styles.NodeStylePortStyleAdapter#nodeStyle}.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStylePortStyleAdapter#renderSize} property on the created object.
       * </p>
       */
      constructor(options:{nodeStyle:yfiles.styles.INodeStyle,renderSize?:yfiles.geometry.Size});
      /**
       * Initializes a new instance of the {@link yfiles.styles.NodeStylePortStyleAdapter} class.
       * <p>
       * This constructor uses an ellipse {@link yfiles.styles.ShapeNodeStyle} with a black {@link yfiles.view.Fill} for the {@link yfiles.styles.NodeStylePortStyleAdapter#nodeStyle} property.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.INodeStyle} options.nodeStyle The {@link yfiles.styles.INodeStyle} that is used for rendering the port.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStylePortStyleAdapter#nodeStyle} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.MutableSize} of the port that will be displayed using {@link yfiles.styles.NodeStylePortStyleAdapter#nodeStyle}.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStylePortStyleAdapter#renderSize} property on the created object.
       * </p>
       */
      constructor(options?:{nodeStyle?:yfiles.styles.INodeStyle,renderSize?:yfiles.geometry.Size});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * 
       * @type {yfiles.styles.IPortStyleRenderer}
       */
      renderer:yfiles.styles.IPortStyleRenderer;
      /**
       * Gets the {@link yfiles.styles.INodeStyle} that is used for rendering the port.
       * @type {yfiles.styles.INodeStyle}
       */
      nodeStyle:yfiles.styles.INodeStyle;
      /**
       * Gets or sets the {@link yfiles.geometry.MutableSize} of the port that will be displayed using {@link yfiles.styles.NodeStylePortStyleAdapter#nodeStyle}.
       * <p>
       * The default value is (5,5).
       * </p>
       * @default '5,5'
       * @type {yfiles.geometry.Size}
       */
      renderSize:yfiles.geometry.Size;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class that makes it possible to easily implement a custom {@link yfiles.styles.IPortStyle}.
     * <p>
     * Only {@link yfiles.styles.PortStyleBase#createVisual} and {@link yfiles.styles.PortStyleBase#getBounds} need to be implemented by
     * subclasses, however to improve rendering performance it is highly recommended to implement at least
     * {@link yfiles.styles.PortStyleBase#updateVisual}, too.
     * </p>
     * <p>
     * This implementation differs from the straightforward {@link yfiles.styles.IPortStyle} implementation in that there is no visible
     * separation between the style and its
     * {@link yfiles.styles.IPortStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
     * the style instance.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: SimpleCustomStyle, demo.yfiles.style.simplecustomstyle</li>
     * <li>Tutorial: 02 Custom Styles, step 22 Custom Ports</li>
     * </ul>
     * @class
     * @implements {yfiles.styles.IPortStyle}
     */
    export interface PortStyleBase extends Object,yfiles.styles.IPortStyle{}
    export class PortStyleBase {
      /**
       * Initializes a new instance of the {@link yfiles.styles.PortStyleBase} class.
       * @protected
       */
      constructor();
      /**
       * Creates a new object that is a copy of the current instance.
       * <p>
       * Immutable subclasses should consider returning <code>this</code>.
       * </p>
       * @returns A new object that is a copy of this instance using {@link #memberwiseClone}.
       */
      clone():Object;
      /**
       * Callback that creates the visual.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#createVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.PortStyleBase#renderer}.
       * </p>
       * @param context The render context.
       * @param port The port to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.PortStyleBase#updateVisual
       * @protected
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext,port:yfiles.graph.IPort):yfiles.view.Visual;
      /**
       * Callback that returns the bounds of the visual for the port in the given context.
       * @param context The canvas context.
       * @param port The port to which this style instance is assigned.
       * @returns The visual bounds of the visual representation.
       * @protected
       * @abstract
       */
      getBounds(context:yfiles.view.ICanvasContext,port:yfiles.graph.IPort):yfiles.geometry.Rect;
      /**
       * Determines whether the visual representation of the port has been hit at the given location.
       * <p>
       * This method is called in response to a {@link yfiles.input.IHitTestable#isHit} call to the instance that has been queried from
       * the {@link yfiles.styles.PortStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation uses the {@link yfiles.styles.PortStyleBase#getBounds bounds} to determine whether the port has been hit.
       * </p>
       * @param context The input mode context.
       * @param location The point to test.
       * @param port The port to which this style instance is assigned.
       * @returns <code>true</code> if the specified port representation is hit; <code>false</code> otherwise.
       * @protected
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point,port:yfiles.graph.IPort):boolean;
      /**
       * Determines whether the visualization for the specified port is included in the marquee selection.
       * <p>
       * This method is called in response to a {@link yfiles.input.IMarqueeTestable#isInBox} call to the instance that has been queried
       * from the {@link yfiles.styles.PortStyleBase#renderer}.
       * </p>
       * <p>
       * This implementation simply tests whether the {@link yfiles.styles.PortStyleBase#getBounds bounds} intersect the marquee box.
       * </p>
       * @param context The input mode context.
       * @param rectangle The marquee selection box.
       * @param port The port to which this style instance is assigned.
       * @returns <code>true</code> if the specified port is visible is selected by the marquee rectangle; <code>false</code> otherwise.
       * @protected
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect,port:yfiles.graph.IPort):boolean;
      /**
       * Determines whether the visualization for the specified port is visible in the context.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisibilityTestable#isVisible} call to the instance that has been
       * queried from the {@link yfiles.styles.PortStyleBase#renderer}. This implementation simply tests whether the {@link yfiles.styles.PortStyleBase#getBounds bounds} intersect the clip.
       * </p>
       * @param context The canvas context.
       * @param rectangle The clipping rectangle.
       * @param port The port to which this style instance is assigned.
       * @returns <code>true</code> if the specified port is visible in the clipping rectangle; <code>false</code> otherwise.
       * @protected
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect,port:yfiles.graph.IPort):boolean;
      /**
       * Performs the {@link yfiles.graph.ILookup#lookup} operation for the {@link yfiles.styles.IPortStyleRenderer#getContext} that has been
       * queried from the {@link yfiles.styles.PortStyleBase#renderer}.
       * <p>
       * This implementation yields <code>null</code> for everything but:
       * </p>
       * <ul>
       * <li>{@link yfiles.input.IHitTestable}</li>
       * <li>{@link yfiles.view.IVisualCreator}</li>
       * <li>{@link yfiles.view.IBoundsProvider}</li>
       * <li>{@link yfiles.view.IVisibilityTestable}</li>
       * <li>{@link yfiles.input.IMarqueeTestable}</li>
       * <li>{@link yfiles.graph.ILookup}</li>
       * </ul>
       * <p>
       * For these interfaces an implementation will be returned that delegates to the methods in this instance.
       * </p>
       * @param port The port to use for the context lookup.
       * @param type The type to query.
       * @returns An implementation of the <code>type</code> or <code>null</code>.
       * @protected
       */
      lookup(port:yfiles.graph.IPort,type:yfiles.lang.Class):Object;
      /**
       * Callback that updates the visual previously created by {@link yfiles.styles.PortStyleBase#createVisual}.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#updateVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.PortStyleBase#renderer}. This implementation simply delegates to {@link yfiles.styles.PortStyleBase#createVisual} so subclasses should
       * override to improve rendering performance.
       * </p>
       * @param context The render context.
       * @param oldVisual The visual that has been created in the call to {@link #createVisual}.
       * @param port The port to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.PortStyleBase#createVisual
       * @protected
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual,port:yfiles.graph.IPort):yfiles.view.Visual;
      /**
       * Gets the renderer implementation for this instance.
       * <p>
       * The private implementation will delegate all API calls back to this instance.
       * </p>
       * @type {yfiles.styles.IPortStyleRenderer}
       */
      renderer:yfiles.styles.IPortStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A utility class that provides {@link yfiles.styles.TextRenderSupport#measureText text size measurement} and {@link yfiles.styles.TextRenderSupport#addText text placement} functionality, for instance, for label rendering
     * and measurement.
     * @class
     */
    export interface TextRenderSupport extends Object{}
    export class TextRenderSupport {
      /**
       * Add the text content to the provided SVG <code>text</code> element.
       * @param targetElement An SVG text element to add the provided text to (either as <code>textContent</code>, or by appending <code>tspan</code> elements).
       * @param text The text content to add to the provided text element (may contain newline characters).
       * @param font The {@link } that defines the font properties to apply to the added text.
       * @param maximumSize The bounds that shouldn't be exceeded when placing the text; <code>null</code> or not provided means unbound.
       * @param [wrapping=yfiles.view.TextWrapping.NONE] The {@link } policy to apply when the text exceeds the provided <code>maximumSize</code>; {@link #WORD}
       * per default.
       * @param [measurePolicy=yfiles.styles.TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text; {@link #AUTOMATIC} per default.
       * @returns The text that was actually placed in the <code>targetElement</code>.
       * @static
       */
      static addText(targetElement:SVGTextElement,text:string,font:yfiles.view.Font,maximumSize:yfiles.geometry.Size,wrapping?:yfiles.view.TextWrapping,measurePolicy?:yfiles.styles.TextMeasurePolicy):string;
      /**
       * Add the text content to the provided SVG <code>text</code> element.
       * @param {Object} options The parameters to pass.
       * @param options.targetElement An SVG text element to add the provided text to (either as <code>textContent</code>, or by appending <code>tspan</code> elements).
       * @param options.text The text content to add to the provided text element (may contain newline characters).
       * @param options.font The {@link } that defines the font properties to apply to the added text.
       * @param options.maximumSize The bounds that shouldn't be exceeded when placing the text; <code>null</code> or not provided means unbound.
       * @param [options.wrapping=yfiles.view.TextWrapping.NONE] The {@link } policy to apply when the text exceeds the provided <code>maximumSize</code>; {@link #WORD}
       * per default.
       * @param [options.measurePolicy=yfiles.styles.TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text; {@link #AUTOMATIC} per default.
       * @returns The text that was actually placed in the <code>targetElement</code>.
       * @static
       */
      static addText(options:{targetElement:SVGTextElement,text:string,font:yfiles.view.Font,maximumSize:yfiles.geometry.Size,wrapping?:yfiles.view.TextWrapping,measurePolicy?:yfiles.styles.TextMeasurePolicy}):string;
      /**
       * Calculate the width and height required to render the provided text using the provided {@link yfiles.view.Font}.
       * @param text The text that should be measured.
       * @param font The {@link } to apply to the text before measuring.
       * @param [measurePolicy=yfiles.styles.TextMeasurePolicy.AUTOMATIC] The measure policy to use for measuring the size of the text; {@link #AUTOMATIC} per default.
       * @returns The measured text size
       * @static
       */
      static measureText(text:string,font:yfiles.view.Font,measurePolicy?:yfiles.styles.TextMeasurePolicy):yfiles.geometry.Size;
      static $class:yfiles.lang.Class;
    }
    export enum TextMeasurePolicy{
      /**
       * The measure policy will be determined heuristically.
       */
      AUTOMATIC,
      /**
       * Use <a href="http://www.w3.org/TR/SVG11/types.html#__svg__SVGLocatable__getBBox" target="_blank">getBBox()</a> to measure the size of text elements.
       */
      SVG,
      /**
       * Use <a href="http://www.whatwg.org/specs/web-apps/current-work/multipage/the-canvas-element.html#dom-context-2d-measuretext" target="_blank">CanvasRenderingContext2D.measureText()</a> to measure the width of text elements.
       */
      CANVAS
    }
    /**
     * A basic label style which renders the label's {@link yfiles.graph.ILabel#text} and optionally a {@link yfiles.styles.DefaultLabelStyle#backgroundFill background} and {@link yfiles.styles.DefaultLabelStyle#backgroundStroke border}.
     * <p>
     * This style uses a {@link yfiles.styles.DefaultLabelStyleRenderer} to create its visualization.
     * </p>
     * @class
     * @implements {yfiles.styles.ILabelStyle}
     */
    export interface DefaultLabelStyle extends Object,yfiles.styles.ILabelStyle{}
    export class DefaultLabelStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderer:yfiles.styles.ILabelStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer The custom renderer for this style instance.
       * @param {boolean} options.autoFlip A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.view.Fill} options.backgroundFill The brush to use for the background box of the label.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#backgroundFill} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.backgroundStroke The pen to use for the background box of the label.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#backgroundStroke} property on the created object.
       * </p>
       * @param {boolean} options.clipText A value that determines whether text should be clipped.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#clipText} property on the created object.
       * </p>
       * @param {yfiles.view.HorizontalTextAlignment} options.horizontalTextAlignment The text alignment.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#horizontalTextAlignment} property on the created object.
       * </p>
       * @param {yfiles.view.Fill} options.textFill The brush to use for the label's text.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#textFill} property on the created object.
       * </p>
       * @param {number} options.textSize The size of the text.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#textSize} property on the created object.
       * </p>
       * @param {yfiles.view.TextWrapping} options.wrapping The text trimming.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#wrapping} property on the created object.
       * </p>
       * @param {yfiles.view.Font} options.font The typeface to use for the label.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#font} property on the created object.
       * </p>
       * @param {yfiles.view.VerticalTextAlignment} options.verticalTextAlignment The vertical text alignment to use if the label is assigned more space than needed.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#verticalTextAlignment} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.ILabelStyleRenderer,autoFlip?:boolean,backgroundFill?:yfiles.view.Fill,backgroundStroke?:yfiles.view.Stroke,clipText?:boolean,horizontalTextAlignment?:yfiles.view.HorizontalTextAlignment,textFill?:yfiles.view.Fill,textSize?:number,wrapping?:yfiles.view.TextWrapping,font?:yfiles.view.Font,verticalTextAlignment?:yfiles.view.VerticalTextAlignment});
      /**
       * Creates a new instance using {@link yfiles.styles.DefaultLabelStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.autoFlip A value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented downwards, otherwise.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.view.Fill} options.backgroundFill The brush to use for the background box of the label.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#backgroundFill} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.backgroundStroke The pen to use for the background box of the label.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#backgroundStroke} property on the created object.
       * </p>
       * @param {boolean} options.clipText A value that determines whether text should be clipped.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#clipText} property on the created object.
       * </p>
       * @param {yfiles.view.HorizontalTextAlignment} options.horizontalTextAlignment The text alignment.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#horizontalTextAlignment} property on the created object.
       * </p>
       * @param {yfiles.view.Fill} options.textFill The brush to use for the label's text.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#textFill} property on the created object.
       * </p>
       * @param {number} options.textSize The size of the text.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#textSize} property on the created object.
       * </p>
       * @param {yfiles.view.TextWrapping} options.wrapping The text trimming.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#wrapping} property on the created object.
       * </p>
       * @param {yfiles.view.Font} options.font The typeface to use for the label.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#font} property on the created object.
       * </p>
       * @param {yfiles.view.VerticalTextAlignment} options.verticalTextAlignment The vertical text alignment to use if the label is assigned more space than needed.
       * <p>
       * This option sets the {@link yfiles.styles.DefaultLabelStyle#verticalTextAlignment} property on the created object.
       * </p>
       */
      constructor(options?:{autoFlip?:boolean,backgroundFill?:yfiles.view.Fill,backgroundStroke?:yfiles.view.Stroke,clipText?:boolean,horizontalTextAlignment?:yfiles.view.HorizontalTextAlignment,textFill?:yfiles.view.Fill,textSize?:number,wrapping?:yfiles.view.TextWrapping,font?:yfiles.view.Font,verticalTextAlignment?:yfiles.view.VerticalTextAlignment});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the brush to use for the background box of the label.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.view.Fill}
       */
      backgroundFill:yfiles.view.Fill;
      /**
       * Gets or sets the vertical text alignment to use if the label is assigned more space than needed.
       * <p>
       * The default value is {@link yfiles.view.VerticalTextAlignment#TOP}.
       * </p>
       * @default yfiles.view.VerticalTextAlignment.TOP
       * @type {yfiles.view.VerticalTextAlignment}
       */
      verticalTextAlignment:yfiles.view.VerticalTextAlignment;
      /**
       * Gets or sets the text alignment.
       * <p>
       * The default value is {@link yfiles.view.HorizontalTextAlignment#LEFT}.
       * </p>
       * @default yfiles.view.HorizontalTextAlignment.LEFT
       * @type {yfiles.view.HorizontalTextAlignment}
       */
      horizontalTextAlignment:yfiles.view.HorizontalTextAlignment;
      /**
       * Gets or sets the text trimming.
       * <p>
       * The default value is {@link yfiles.view.TextWrapping#NONE}.
       * </p>
       * @default yfiles.view.TextWrapping.NONE
       * @type {yfiles.view.TextWrapping}
       */
      wrapping:yfiles.view.TextWrapping;
      /**
       * Gets or sets a value that determines whether text should be clipped.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      clipText:boolean;
      /**
       * Gets or sets the pen to use for the background box of the label.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.view.Stroke}
       */
      backgroundStroke:yfiles.view.Stroke;
      /**
       * 
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
      /**
       * Gets or sets the typeface to use for the label.
       * <p>
       * The default value is an instance of {@link yfiles.styles.DefaultLabelStyle#font} for <code>Arial</code>".
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} value cannot be null!
       * @type {yfiles.view.Font}
       */
      font:yfiles.view.Font;
      /**
       * Gets or sets a value indicating whether the label should be flipped 180 degrees automatically, if it would be oriented
       * downwards, otherwise.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoFlip:boolean;
      /**
       * Gets or sets the brush to use for the label's text.
       * <p>
       * The default value is {@link yfiles.view.Fill#BLACK}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} value cannot be null!
       * @default 'Black'
       * @type {yfiles.view.Fill}
       */
      textFill:yfiles.view.Fill;
      /**
       * Gets or sets the size of the text.
       * <p>
       * This property delegates to the size of the {@link yfiles.styles.DefaultLabelStyle#font} property.
       * </p>
       * <p>
       * The default value is <code>12.0d</code>.
       * </p>
       * @default 12
       * @type {number}
       */
      textSize:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A label style that draws an icon in addition to the text.
     * <p>
     * This style draws the icon at the location specified by {@link yfiles.styles.IconLabelStyle#iconPlacement} and delegates the drawing of the label's text to the {@link yfiles.styles.IconLabelStyle#wrapped} style.
     * </p>
     * @class
     * @implements {yfiles.styles.ILabelStyle}
     */
    export interface IconLabelStyle extends Object,yfiles.styles.ILabelStyle{}
    export class IconLabelStyle {
      /**
       * Creates a new instance using the provided icon and renderer.
       * @param icon The icon to draw.
       * @param [renderer=null] The custom renderer for this style instance.
       */
      constructor(icon:string,renderer?:yfiles.styles.ILabelStyleRenderer);
      /**
       * Creates a new instance using the provided icon and renderer.
       * @param {Object} options The parameters to pass.
       * @param options.icon The icon to draw.
       * @param [options.renderer=null] The custom renderer for this style instance.
       * @param {boolean} options.autoFlip A value indicating whether to automatically flip the rendering should the {@link yfiles.graph.ILabel#layout layout}'s {@link yfiles.geometry.IOrientedRectangle#upY up vector} point downwards.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.graph.ILabelModelParameter} options.iconPlacement The parameter that specifies the icon placement.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#iconPlacement} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.iconSize The size of the icon to paint for the label.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#iconSize} property on the created object.
       * </p>
       * @param {yfiles.styles.ILabelStyle} options.wrapped The style that renders the text of the label.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#wrapped} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.wrappedInsets The insets that should be applied for the {@link yfiles.graph.ILabel#layout layout} of the {@link yfiles.styles.IconLabelStyle#wrapped wrapped style} rendering.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#wrappedInsets} property on the created object.
       * </p>
       */
      constructor(options:{icon:string,renderer?:yfiles.styles.ILabelStyleRenderer,autoFlip?:boolean,iconPlacement?:yfiles.graph.ILabelModelParameter,iconSize?:yfiles.geometry.Size,wrapped?:yfiles.styles.ILabelStyle,wrappedInsets?:yfiles.geometry.Insets});
      /**
       * Creates a new instance using no icon and {@link yfiles.styles.IconLabelStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.autoFlip A value indicating whether to automatically flip the rendering should the {@link yfiles.graph.ILabel#layout layout}'s {@link yfiles.geometry.IOrientedRectangle#upY up vector} point downwards.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#autoFlip} property on the created object.
       * </p>
       * @param {string} options.icon The icon to paint for the label.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#icon} property on the created object.
       * </p>
       * @param {yfiles.graph.ILabelModelParameter} options.iconPlacement The parameter that specifies the icon placement.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#iconPlacement} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.iconSize The size of the icon to paint for the label.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#iconSize} property on the created object.
       * </p>
       * @param {yfiles.styles.ILabelStyle} options.wrapped The style that renders the text of the label.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#wrapped} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.wrappedInsets The insets that should be applied for the {@link yfiles.graph.ILabel#layout layout} of the {@link yfiles.styles.IconLabelStyle#wrapped wrapped style} rendering.
       * <p>
       * This option sets the {@link yfiles.styles.IconLabelStyle#wrappedInsets} property on the created object.
       * </p>
       */
      constructor(options?:{autoFlip?:boolean,icon?:string,iconPlacement?:yfiles.graph.ILabelModelParameter,iconSize?:yfiles.geometry.Size,wrapped?:yfiles.styles.ILabelStyle,wrappedInsets?:yfiles.geometry.Insets});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the icon to paint for the label.
       * @type {string}
       */
      icon:string;
      /**
       * Gets or sets the size of the icon to paint for the label.
       * <p>
       * The default value is {@link yfiles.geometry.Size#EMPTY}.
       * </p>
       * @type {yfiles.geometry.Size}
       */
      iconSize:yfiles.geometry.Size;
      /**
       * Gets or sets the parameter that specifies the icon placement.
       * <p>
       * All valid node label model parameters can be used. The label's {@link yfiles.graph.ILabel#layout layout} will be interpreted as the a node's layout and the
       * icon will be placed relative to that layout as if it was a node's label.
       * </p>
       * <p>
       * The default value is {@link yfiles.graph.ExteriorLabelModel#WEST}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} <code>value</code> is null.
       * @type {yfiles.graph.ILabelModelParameter}
       */
      iconPlacement:yfiles.graph.ILabelModelParameter;
      /**
       * Gets or sets the style that renders the text of the label.
       * <p>
       * This style will be rendered before the icon and is responsible for rendering the {@link yfiles.graph.ILabel#text}, since {@link yfiles.styles.IconLabelStyleRenderer}
       * will only render the {@link yfiles.styles.IconLabelStyle#icon}.
       * </p>
       * <p>
       * The default value is {@link yfiles.styles.DefaultLabelStyle}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} <code>value</code> is null.
       * @type {yfiles.styles.ILabelStyle}
       */
      wrapped:yfiles.styles.ILabelStyle;
      /**
       * Gets or sets the insets that should be applied for the {@link yfiles.graph.ILabel#layout layout} of the {@link yfiles.styles.IconLabelStyle#wrapped wrapped style} rendering.
       * <p>
       * The {@link yfiles.styles.IconLabelStyleRenderer#getPreferredSize} will take the preferred size of the {@link yfiles.styles.IconLabelStyle#wrapped}'s {@link yfiles.styles.ILabelStyleRenderer}
       * and add these insets to it. During the {@link yfiles.view.IVisualCreator rendering} these insets will be used to offset the rendering of the wrapped
       * style.
       * </p>
       * <p>
       * The default insets are empty.
       * </p>
       * @default '0'
       * @type {yfiles.geometry.Insets}
       */
      wrappedInsets:yfiles.geometry.Insets;
      /**
       * 
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
      /**
       * Gets a value indicating whether to automatically flip the rendering should the {@link yfiles.graph.ILabel#layout layout}'s {@link yfiles.geometry.IOrientedRectangle#upY up vector} point downwards.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoFlip:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A simple {@link yfiles.styles.ILabelStyleRenderer} implementation that can handle {@link yfiles.styles.DefaultLabelStyle} instances.
     * <p>
     * This implementation will just draw the label's {@link yfiles.graph.ILabel#text} using the properties provided by {@link yfiles.styles.DefaultLabelStyle} into
     * the label's {@link yfiles.graph.ILabel#layout layout} area.
     * </p>
     * @class
     * @implements {yfiles.styles.ILabelStyleRenderer}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface DefaultLabelStyleRenderer extends Object,yfiles.styles.ILabelStyleRenderer,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class DefaultLabelStyleRenderer {
      /**
       * Creates a new default instance.
       */
      constructor();
      /**
       * Add the text content to the provided SVG text element.
       * @param textElement An SVG text element to add the provided text to.
       * @param font The {@link } that defines the font properties to apply to the added text.
       * @param text The text content to add to the provided text element (may contain newline characters).
       * @param maxSize The bounds that shouldn't be exceeded when placing the text; <code>null</code> or not provided means unbound.
       * @param wrapping The {@link } policy to apply when the text exceeds the provided <code>maxSize</code>.
       * @returns 
       * @protected
       */
      addTextElements(textElement:SVGTextElement,font:yfiles.view.Font,text:string,maxSize:yfiles.geometry.Size,wrapping:yfiles.view.TextWrapping):string;
      /**
       * Prepares this instance for subsequent calls after the style and label have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.DefaultLabelStyleRenderer#style} and {@link yfiles.styles.DefaultLabelStyleRenderer#label} properties have been populated by the
       * {@link yfiles.styles.DefaultLabelStyleRenderer#getVisualCreator}, {@link yfiles.styles.DefaultLabelStyleRenderer#getBoundsProvider},
       * {@link yfiles.styles.DefaultLabelStyleRenderer#getHitTestable}, or {@link yfiles.styles.DefaultLabelStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * Stores the {@link yfiles.graph.ILabel#layout layout} into {@link yfiles.styles.DefaultLabelStyleRenderer#layout}. Subclasses which to override this method should call the base implementation.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to retrieve the bounds provider for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.DefaultLabelStyleRenderer#getBounds
       */
      getBoundsProvider(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link yfiles.styles.DefaultLabelStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.DefaultLabelStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.DefaultLabelStyleRenderer#lookup} method call they should call
       * {@link yfiles.styles.DefaultLabelStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance that
       * depends on a correctly configured <code>this</code>.
       * </p>
       * @param label The label to query the context for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.DefaultLabelStyleRenderer#lookup
       */
      getContext(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to query hit test with. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.DefaultLabelStyleRenderer#isHit
       */
      getHitTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IHitTestable;
      /**
       * Determines how the text should be aligned within the assigned label bounds.
       * @returns This value as obtained from the style.
       * @protected
       */
      getHorizontalTextAlignment():yfiles.view.HorizontalTextAlignment;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to query marquee intersection tests. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.DefaultLabelStyleRenderer#isInBox
       */
      getMarqueeTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @param label
       * @param style
       * @returns 
       */
      getPreferredSize(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.geometry.Size;
      /**
       * Calculates the preferred size given the current state of the renderer.
       * @returns The size as suggested by this renderer.
       * @protected
       */
      getPreferredSizeCore():yfiles.geometry.Size;
      /**
       * Determines how the text should be aligned vertically within the assigned label bounds.
       * @returns This value as obtained from the style.
       * @protected
       */
      getVerticalTextAlignment():yfiles.view.VerticalTextAlignment;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link yfiles.styles.DefaultLabelStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.DefaultLabelStyleRenderer#configure}. If
       * the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.DefaultLabelStyleRenderer#isVisible}.
       * </p>
       * @param label The label to query visibility tests. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.DefaultLabelStyleRenderer#isVisible
       */
      getVisibilityTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.DefaultLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to retrieve the {@link } for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.DefaultLabelStyleRenderer#createVisual
       * @see yfiles.styles.DefaultLabelStyleRenderer#updateVisual
       */
      getVisualCreator(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisualCreator;
      /**
       * Determines how the text should be trimmed to fit the assigned label bounds.
       * @returns This value as obtained from the style.
       * @protected
       */
      getWrapping():yfiles.view.TextWrapping;
      /**
       * Delegates to {@link yfiles.styles.DefaultLabelStyle#autoFlip}
       * @returns Whether to flip the drawing if it is upside down.
       * @protected
       */
      isAutoFlip():boolean;
      /**
       * Determines whether text should be forced to be clipped inside the rectangle.
       * @returns This value as obtained from the style.
       * @protected
       */
      isClippingText():boolean;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Uses the {@link yfiles.graph.ILabel#layout} to determine whether the clip intersects.
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implements the {@link yfiles.graph.ILookup} interface.
       * <p>
       * This method will be used by default if {@link yfiles.styles.DefaultLabelStyleRenderer#getContext} has been queried for a lookup
       * implementation. Note that it cannot be assumed that {@link yfiles.styles.DefaultLabelStyleRenderer#configure} has already been
       * invoked. However, normally {@link yfiles.styles.DefaultLabelStyleRenderer#label} and {@link yfiles.styles.DefaultLabelStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
       * implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
       * This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link yfiles.styles.DefaultLabelStyleRenderer#configure}
       * on success and return <code>this</code>.
       * </p>
       * @param type The type to query for.
       * @returns An implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Calculate the width and height required to render the provided text using the provided {@link yfiles.view.Font}.
       * @param text The text that should be measured.
       * @param font The {@link } to apply to the text before measuring.
       * @returns The text size.
       * @protected
       */
      measureText(text:string,font:yfiles.view.Font):yfiles.geometry.Size;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Retrieves the current {@link yfiles.graph.ILabel#layout layout}.
       * @protected
       * @type {yfiles.geometry.IOrientedRectangle}
       */
      layout:yfiles.geometry.IOrientedRectangle;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.DefaultLabelStyle}
       */
      style:yfiles.styles.DefaultLabelStyle;
      /**
       * Gets or sets the currently configured label.
       * @protected
       * @type {yfiles.graph.ILabel}
       */
      label:yfiles.graph.ILabel;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.ILabelStyleRenderer} that can handle {@link yfiles.styles.IconLabelStyle} instances.
     * <p>
     * This implementation renders the label's inner style and an icon.
     * </p>
     * @class
     * @implements {yfiles.styles.ILabelStyleRenderer}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface IconLabelStyleRenderer extends Object,yfiles.styles.ILabelStyleRenderer,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class IconLabelStyleRenderer {
      /**
       * Creates a new instance of this renderer.
       */
      constructor();
      /**
       * 
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.IconLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to retrieve the bounds provider for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.IconLabelStyleRenderer#getBounds
       */
      getBoundsProvider(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link yfiles.styles.IconLabelStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.IconLabelStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.IconLabelStyleRenderer#lookup} method call they should call {@link yfiles.styles.IconLabelStyleRenderer#configure}
       * only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param label The label to query the context for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.IconLabelStyleRenderer#lookup
       */
      getContext(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.IconLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to query hit test with. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.IconLabelStyleRenderer#isHit
       */
      getHitTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IHitTestable;
      /**
       * Retrieves the {@link yfiles.styles.IconLabelStyle#icon} from the {@link yfiles.styles.IconLabelStyleRenderer#style}.
       * @returns The icon.
       * @protected
       */
      getIcon():string;
      /**
       * Retrieves the {@link yfiles.styles.IconLabelStyle#iconPlacement} from the {@link yfiles.styles.IconLabelStyleRenderer#style}.
       * @returns The placement parameter.
       * @protected
       */
      getIconPlacement():yfiles.graph.ILabelModelParameter;
      /**
       * Gets the size of the icon from the {@link yfiles.styles.IconLabelStyle#iconSize} property.
       * @returns The size to use for the icon.
       * @protected
       */
      getIconSize():yfiles.geometry.Size;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.IconLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to query marquee intersection tests. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.IconLabelStyleRenderer#isInBox
       */
      getMarqueeTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @param label
       * @param style
       * @returns 
       */
      getPreferredSize(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.geometry.Size;
      /**
       * Calculates the preferred size given the current state of the renderer.
       * @returns The size as suggested by this renderer.
       * @protected
       */
      getPreferredSizeCore():yfiles.geometry.Size;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, does <b>not</b> call {@link yfiles.styles.IconLabelStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.IconLabelStyleRenderer#configure}. If the
       * subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.IconLabelStyleRenderer#isVisible}.
       * </p>
       * @param label The label to query visibility tests. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.IconLabelStyleRenderer#isVisible
       */
      getVisibilityTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>label</code> parameters, calls {@link yfiles.styles.IconLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to retrieve the {@link } for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.IconLabelStyleRenderer#createVisual
       * @see yfiles.styles.IconLabelStyleRenderer#updateVisual
       */
      getVisualCreator(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisualCreator;
      /**
       * Retrieves the {@link yfiles.styles.IconLabelStyle#wrapped} from the {@link yfiles.styles.IconLabelStyleRenderer#style}.
       * @returns The style.
       * @protected
       */
      getWrappedStyle():yfiles.styles.ILabelStyle;
      /**
       * Retrieves the {@link yfiles.styles.IconLabelStyle#wrappedInsets} from the {@link yfiles.styles.IconLabelStyleRenderer#style}.
       * @returns The insets.
       * @protected
       */
      getWrappedStyleInsets():yfiles.geometry.Insets;
      /**
       * Retrieves the {@link yfiles.styles.IconLabelStyle#autoFlip} property from the {@link yfiles.styles.IconLabelStyleRenderer#style}.
       * @returns The autoflip property value.
       * @protected
       */
      isAutoFlip():boolean;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implements the {@link yfiles.graph.ILookup} interface.
       * <p>
       * This method will be used by default if {@link yfiles.styles.IconLabelStyleRenderer#getContext} has been queried for a lookup
       * implementation. Note that it cannot be assumed that {@link yfiles.styles.IconLabelStyleRenderer#configure} has already been
       * invoked. However, normally {@link yfiles.styles.IconLabelStyleRenderer#label} and {@link yfiles.styles.IconLabelStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
       * implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
       * This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link yfiles.styles.IconLabelStyleRenderer#configure}
       * on success and return <code>this</code>.
       * </p>
       * @param type The type to query for.
       * @returns An implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Returns whether the icon bounds should be considered for hit testing.
       * @returns true
       * @protected
       */
      shouldHitTestIcon():boolean;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.IconLabelStyle}
       */
      style:yfiles.styles.IconLabelStyle;
      /**
       * Gets or sets the currently configured label.
       * @protected
       * @type {yfiles.graph.ILabel}
       */
      label:yfiles.graph.ILabel;
      static $class:yfiles.lang.Class;
    }
    /**
     * A node style that can display node shapes with a boundary that is defined by a {@link yfiles.geometry.GeneralPath} instance.
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface GeneralPathNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class GeneralPathNodeStyle {
      /**
       * Creates a new instance using the specified path and a custom renderer instance.
       * @param path The path that defines the boundary of this style.
       * @param [renderer=null] Custom renderer instance for this style
       */
      constructor(path:yfiles.geometry.GeneralPath,renderer?:yfiles.styles.GeneralPathNodeStyleRenderer);
      /**
       * Creates a new instance using the specified path and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.path The path that defines the boundary of this style.
       * @param [options.renderer=null] Custom renderer instance for this style
       * @param {yfiles.view.Fill} options.fill The {@link yfiles.view.Fill} that is used to draw the shape.
       * <p>
       * This option sets the {@link yfiles.styles.GeneralPathNodeStyle#fill} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.view.Stroke} that is used to draw the shape.
       * <p>
       * This option sets the {@link yfiles.styles.GeneralPathNodeStyle#stroke} property on the created object.
       * </p>
       */
      constructor(options:{path:yfiles.geometry.GeneralPath,renderer?:yfiles.styles.GeneralPathNodeStyleRenderer,fill?:yfiles.view.Fill,stroke?:yfiles.view.Stroke});
      /**
       * Creates a new instance using a simple rectangular path and {@link yfiles.styles.GeneralPathNodeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Fill} options.fill The {@link yfiles.view.Fill} that is used to draw the shape.
       * <p>
       * This option sets the {@link yfiles.styles.GeneralPathNodeStyle#fill} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.path The path that defines the boundary of the shape.
       * <p>
       * This option sets the {@link yfiles.styles.GeneralPathNodeStyle#path} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.view.Stroke} that is used to draw the shape.
       * <p>
       * This option sets the {@link yfiles.styles.GeneralPathNodeStyle#stroke} property on the created object.
       * </p>
       */
      constructor(options?:{fill?:yfiles.view.Fill,path?:yfiles.geometry.GeneralPath,stroke?:yfiles.view.Stroke});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Get or sets the path that defines the boundary of the shape.
       * <p>
       * The path must be defined in such a way that &lt;0,0> will be at the top left of a node's {@link yfiles.graph.INode#layout} and &lt;1,1> will be at the lower
       * right. The path will be scaled accordingly during the painting.
       * </p>
       * <p>
       * The default is a simple rectangular path.
       * </p>
       * @type {yfiles.geometry.GeneralPath}
       */
      path:yfiles.geometry.GeneralPath;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      /**
       * Gets or sets the {@link yfiles.view.Fill} that is used to draw the shape.
       * <p>
       * The default value is {@link yfiles.view.Fill#WHITE}.
       * </p>
       * @default 'System.Drawing.Brushes.White'
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      /**
       * Gets or sets the {@link yfiles.view.Stroke} that is used to draw the shape.
       * <p>
       * The default value is {@link yfiles.view.Stroke#BLACK}.
       * </p>
       * @default 'System.Drawing.Pens.Black'
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.INodeStyleRenderer} implementation that can handle {@link yfiles.styles.GeneralPathNodeStyle} instances.
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface GeneralPathNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class GeneralPathNodeStyleRenderer {
      /**
       * Initializes a new instance of the {@link yfiles.styles.GeneralPathNodeStyleRenderer} class.
       */
      constructor();
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.GeneralPathNodeStyleRenderer#style} and {@link yfiles.styles.GeneralPathNodeStyleRenderer#node} properties have been populated by the
       * {@link yfiles.styles.GeneralPathNodeStyleRenderer#getVisualCreator},
       * {@link yfiles.styles.GeneralPathNodeStyleRenderer#getBoundsProvider}, {@link yfiles.styles.GeneralPathNodeStyleRenderer#getHitTestable},
       * or {@link yfiles.styles.GeneralPathNodeStyleRenderer#getMarqueeTestable} methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.GeneralPathNodeStyleRenderer#lookup} method call they should
       * call {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
       * instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * 
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Configures the {@link yfiles.styles.GeneralPathNodeStyleRenderer#style} and {@link yfiles.styles.GeneralPathNodeStyleRenderer#node} properties, calls {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#isInside
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure}. If
       * the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.GeneralPathNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#createVisual
       * @see yfiles.styles.GeneralPathNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * Determines visibility by checking for an intersection with the {@link yfiles.styles.GeneralPathNodeStyleRenderer#layout}.
       * @param context The context.
       * @param rectangle The current clip
       * @returns Whether the {@link #layout} is visible in the clip.
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implements the {@link yfiles.graph.ILookup} interface.
       * <p>
       * This method will be used by default if {@link yfiles.styles.GeneralPathNodeStyleRenderer#getContext} has been queried for a
       * lookup implementation. Note that it cannot be assumed that {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure} has
       * already been invoked. However, normally {@link yfiles.styles.GeneralPathNodeStyleRenderer#node} and {@link yfiles.styles.GeneralPathNodeStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>.
       * Subclass implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful
       * lookup. This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link yfiles.styles.GeneralPathNodeStyleRenderer#configure}
       * on success and return <code>this</code>.
       * </p>
       * @param type The type to query for.
       * @returns An implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.GeneralPathNodeStyle}
       */
      style:yfiles.styles.GeneralPathNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class for {@link yfiles.styles.IEdgeStyleRenderer} implementations that are based on the calculation of a
     * {@link yfiles.geometry.GeneralPath}.
     * Type parameter <code>TStyle</code>: The exact type of style to use for this instance.
     * @class
     * @implements {yfiles.styles.IEdgeStyleRenderer}
     * @implements {yfiles.styles.IPathGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     * @implements {yfiles.view.IObstacleProvider}
     * @template TStyle
     */
    export interface PathBasedEdgeStyleRenderer<TStyle extends yfiles.styles.IEdgeStyle> extends Object,yfiles.styles.IEdgeStyleRenderer,yfiles.styles.IPathGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator,yfiles.view.IObstacleProvider{}
    export class PathBasedEdgeStyleRenderer<TStyle extends yfiles.styles.IEdgeStyle> {
      /**
       * 
       * @param edgeStyleType The exact type of style to use for this instance.
       */
      constructor(edgeStyleType:yfiles.lang.Class);
      /**
       * Prepares this instance for subsequent calls after the style and edge have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#style} and {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#edge} properties have been populated by the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getVisualCreator}, {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getBoundsProvider}, {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getHitTestable},
       * or {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getMarqueeTestable} methods.
       * </p>
       * <p>
       * Subclasses which override this method have to call the base implementation, first.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * This method should create the basic uncropped path given the control points.
       * @returns A path that has to be cropped, yet.
       * @protected
       * @abstract
       */
      createPath():yfiles.geometry.GeneralPath;
      /**
       * Static utility method that creates a rectangular self loop path.
       * @param sx The x coordinate of the source port.
       * @param sy The y coordinate of the source port.
       * @param oppositeX The x coordinate of the single control point.
       * @param oppositeY The y coordinate of the single control point.
       * @param tx The x coordinate of the target port.
       * @param ty The y coordinate of the target port.
       * @returns A path that describes a rectangular self loop.
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#createSelfLoopPath
       * @protected
       * @static
       */
      static createRectangleSelfLoop(sx:number,sy:number,oppositeX:number,oppositeY:number,tx:number,ty:number):yfiles.geometry.GeneralPath;
      /**
       * Static utility method that creates a round self loop path.
       * @param sx The x coordinate of the source port.
       * @param sy The y coordinate of the source port.
       * @param oppositeX The x coordinate of the single control point.
       * @param oppositeY The y coordinate of the single control point.
       * @param tx The x coordinate of the target port.
       * @param ty The y coordinate of the target port.
       * @returns A path that describes a round self loop.
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#createSelfLoopPath
       * @protected
       * @static
       */
      static createRoundSelfLoop(sx:number,sy:number,oppositeX:number,oppositeY:number,tx:number,ty:number):yfiles.geometry.GeneralPath;
      /**
       * Special implementation that will create a self loop path.
       * @param roundSelfLoop
       * @returns 
       * @protected
       */
      createSelfLoopPath(roundSelfLoop:boolean):yfiles.geometry.GeneralPath;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Crops the edge's path at the nodes.
       * <p>
       * This implementation uses the {@link yfiles.styles.IEdgePathCropper} instances found in the {@link yfiles.graph.ILookup#lookup} of the
       * source and target port of the edge to perform the actual cropping.
       * </p>
       * @param path The path that should be cropped.
       * @returns The cropped path. This is can be either the same instance of the given path or a newly created instance.
       * @protected
       */
      cropPath(path:yfiles.geometry.GeneralPath):yfiles.geometry.GeneralPath;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>edge</code> parameters, calls {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} and returns <code>this</code>.
       * @param edge The edge to retrieve the bounds provider for. The value will be stored in the {@link #edge} property.
       * @param style The style to associate with the edge. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getBounds
       */
      getBoundsProvider(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IBoundsProvider;
      /**
       * Yields the {@link yfiles.view.IBridgeCreator} to use for {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#addBridges adding bridges} to the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#createPath created path}.
       * <p>
       * The return value of this method (which may be <code>null</code>) will be passed to the {@link yfiles.view.BridgeManager}'s {@link yfiles.view.BridgeManager#addBridges}
       * method.
       * </p>
       * @returns This implementation returns <code>null</code>.
       * @protected
       */
      getBridgeCreator():yfiles.view.IBridgeCreator;
      /**
       * Configures the <code>style</code> and <code>edge</code> parameters, does <b>not</b> call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure}, instead subclasses should ensure that in
       * the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#lookup} method call they should call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance
       * that depends on a correctly configured <code>this</code>.
       * </p>
       * @param edge The edge to query the context for. The value will be stored in the {@link #edge} property.
       * @param style The style to associate with the edge. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#lookup
       */
      getContext(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>edge</code> parameters and returns <code>this</code>.
       * <p>
       * This implementation does not call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure}. If subclasses depend on a configured instance in an override of {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isHit},
       * they need to override this method, too and call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} if the base call returns a non-<code>null</code> value.
       * </p>
       * @param edge
       * @param style
       * @returns 
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isHit
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure
       * @see yfiles.styles.IEdgeStyleRenderer#getHitTestable
       */
      getHitTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.input.IHitTestable;
      /**
       * Configures the <code>style</code> and <code>edge</code> parameters, calls {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} and returns <code>this</code>.
       * @param edge The edge to query marquee intersection tests. The value will be stored in the {@link #edge} property.
       * @param style The style to associate with the edge. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isInBox
       */
      getMarqueeTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Returns {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#createPath} unless {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isVisible} yields <code>false</code> for the provided context and {@link yfiles.view.IRenderContext#clip}.
       * @param context The context to yield the obstacles for.
       * @returns Either <code>null</code> or the result of {@link #createPath}.
       * @see yfiles.view.IObstacleProvider
       */
      getObstacles(context:yfiles.view.IRenderContext):yfiles.geometry.GeneralPath;
      /**
       * 
       * @returns 
       */
      getPath():yfiles.geometry.GeneralPath;
      /**
       * Gets an implementation of the {@link yfiles.styles.IPathGeometry} interface that can handle the provided edge and its associated
       * style.
       * @param edge The edge to provide an instance for
       * @param style The style to use for the painting
       * @returns <code>this</code>
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getTangent
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getTangent
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getSegmentCount
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPath
       */
      getPathGeometry(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.styles.IPathGeometry;
      /**
       * Returns the number of "segments" this edge's path consists of.
       * <p>
       * This method is part of the {@link yfiles.styles.IPathGeometry} interface, that is implemented by this class. This interface will
       * be returned by {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPathGeometry}.
       * </p>
       * @returns the number of segments or <code>-1</code> if there is no such thing as a segment for this edge.
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPathGeometry
       * @abstract
       */
      getSegmentCount():number;
      /**
       * Gets the value that determines the radius of the smoothing arcs that can be added to the path at the bends.
       * <p>
       * This implementation returns <code>0.0d</code>, which effectively turns off path smoothing.
       * </p>
       * @returns The radius of the arcs to use at the bends of the path.
       * @protected
       */
      getSmoothingLength():number;
      /**
       * Gets the source arrow from the style.
       * @returns The arrow to use.
       * @protected
       * @abstract
       */
      getSourceArrow():yfiles.styles.IArrow;
      /**
       * Calculate and update the anchor and the source arrow's direction vector.
       * @param arrow The arrow to calculate the anchor for.
       * @returns The anchor and directional vector of the arrow, if any exist.
       * @protected
       */
      getSourceArrowAnchor(arrow:yfiles.styles.IArrow):yfiles.geometry.Tangent;
      /**
       * Gets the {@link yfiles.view.Stroke} to use to render the path.
       * @returns The {@link } or <code>null</code>.
       * @protected
       * @abstract
       */
      getStroke():yfiles.view.Stroke;
      /**
       * Calculates the tangent on the edge's path at the given ratio point for the given segment.
       * <p>
       * This method is part of the {@link yfiles.styles.IPathGeometry} interface, that is implemented by this class. This interface will
       * be returned by {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPathGeometry}.
       * </p>
       * @param segmentIndex the segment to use for the calculation
       * @param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the segment
       * @returns The tangential line, if any exists.
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getTangent
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getSegmentCount
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPathGeometry
       * @abstract
       */
      getTangent(segmentIndex:number,ratio:number):yfiles.geometry.Tangent;
      /**
       * Calculates the tangent on the edge's path at the given ratio point.
       * <p>
       * This method is part of the {@link yfiles.styles.IPathGeometry} interface, that is implemented by this class. This interface will
       * be returned by {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPathGeometry}.
       * </p>
       * @param ratio a value in [0,1] where 0 is the source's end and 1 is at the target's end of the visible edge path
       * @returns The tangential line, if any exists.
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getPathGeometry
       * @abstract
       */
      getTangent(ratio:number):yfiles.geometry.Tangent;
      /**
       * Gets the target arrow from the style.
       * @returns The arrow to use.
       * @protected
       * @abstract
       */
      getTargetArrow():yfiles.styles.IArrow;
      /**
       * Calculate and update the anchor and the target arrow's direction vector.
       * @param arrow The arrow to calculate the anchor for.
       * @returns The anchor and directional vector of the arrow, if any exist.
       * @protected
       */
      getTargetArrowAnchor(arrow:yfiles.styles.IArrow):yfiles.geometry.Tangent;
      /**
       * Configures the <code>style</code> and <code>edge</code> parameters, does <b>not</b> call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure}. If the subclass implementation depends on
       * this instance to be configured, it needs to call <code>Configure</code> in {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isVisible}.
       * </p>
       * @param edge The edge to query visibility tests. The value will be stored in the {@link #edge} property.
       * @param style The style to associate with the edge. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isVisible
       */
      getVisibilityTestable(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>edge</code> parameters, calls {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} and returns <code>this</code>.
       * @param edge The edge to retrieve the {@link } for. The value will be stored in the {@link #edge} property.
       * @param style The style to associate with the edge. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#createVisual
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#updateVisual
       */
      getVisualCreator(edge:yfiles.graph.IEdge,style:yfiles.styles.IEdgeStyle):yfiles.view.IVisualCreator;
      /**
       * This method is overridden for performance reasons.
       * <p>
       * This implementation uses {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#isVisible} as an early exit indicator. If <code>IsVisible</code> returns <code>false</code> for a certain rectangle, the
       * instance will not be configured but a non-hit will be assumed. Otherwise this instance will be configured and the
       * standard path based hit test routine
       * <br />
       * Subclasses should not depend on the fact that {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} has already been called. This may not be the case. If your
       * subclass depends on Configure being called, override {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getHitTestable} and call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} after the base class call.
       * </p>
       * @param context
       * @param location
       * @returns 
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getHitTestable
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Calls {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure} and test the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#edge} using the {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#style} for a visible intersection with the clip bounds.
       * <p>
       * This simple implementation uses {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getBounds} to determine the visibility. Subclasses may choose to override this behavior
       * for improved performance but need to <b>make sure to call {@link yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#configure}</b> if they depend on a configured instance.
       * </p>
       * @param context the current canvas context
       * @param rectangle the clip bounds
       * @returns <code>true</code> if the edge may be painted within the clip bounds.
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getVisibilityTestable
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * The path instance that this instance is working with.
       * @protected
       * @type {yfiles.geometry.GeneralPath}
       */
      path:yfiles.geometry.GeneralPath;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {TStyle}
       */
      style:TStyle;
      /**
       * Gets or sets the currently configured edge.
       * @protected
       * @type {yfiles.graph.IEdge}
       */
      edge:yfiles.graph.IEdge;
      /**
       * Determines whether the rendered path should use the {@link yfiles.view.BridgeManager} to {@link yfiles.view.BridgeManager#addBridges add bridges to it.}
       * @see yfiles.styles.PathBasedEdgeStyleRenderer.<TStyle>#getBridgeCreator
       * @protected
       * @type {boolean}
       */
      addBridges:boolean;
      /**
       * Gets or sets the distance between the node's layout and its self loop control points.
       * @type {number}
       */
      selfLoopDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.IEdgeStyleRenderer} that will render {@link yfiles.styles.PolylineEdgeStyle} instances.
     * @class
     * @extends {yfiles.styles.PathBasedEdgeStyleRenderer.<yfiles.styles.PolylineEdgeStyle>}
     */
    export interface PolylineEdgeStyleRenderer extends yfiles.styles.PathBasedEdgeStyleRenderer<yfiles.styles.PolylineEdgeStyle>{}
    export class PolylineEdgeStyleRenderer {
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * A polygonal {@link yfiles.styles.IEdgeStyle} where straight line segments are used to connect the bends.
     * @class
     * @implements {yfiles.styles.IEdgeStyle}
     */
    export interface PolylineEdgeStyle extends Object,yfiles.styles.IEdgeStyle{}
    export class PolylineEdgeStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderer:yfiles.styles.PolylineEdgeStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer The custom renderer for this style instance.
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.PolylineEdgeStyle#stroke} for the line.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#stroke} property on the created object.
       * </p>
       * @param {number} options.smoothingLength The smoothing length used for creating smooth bends.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#smoothingLength} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.sourceArrow The visual arrow at the source end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#sourceArrow} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.targetArrow The visual arrow at the target end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#targetArrow} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.PolylineEdgeStyleRenderer,stroke?:yfiles.view.Stroke,smoothingLength?:number,sourceArrow?:yfiles.styles.IArrow,targetArrow?:yfiles.styles.IArrow});
      /**
       * Creates a new instance using {@link yfiles.styles.PolylineEdgeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.PolylineEdgeStyle#stroke} for the line.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#stroke} property on the created object.
       * </p>
       * @param {number} options.smoothingLength The smoothing length used for creating smooth bends.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#smoothingLength} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.sourceArrow The visual arrow at the source end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#sourceArrow} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.targetArrow The visual arrow at the target end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.PolylineEdgeStyle#targetArrow} property on the created object.
       * </p>
       */
      constructor(options?:{stroke?:yfiles.view.Stroke,smoothingLength?:number,sourceArrow?:yfiles.styles.IArrow,targetArrow?:yfiles.styles.IArrow});
      /**
       * Performs a {@link Object#memberwiseClone}
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the {@link yfiles.styles.PolylineEdgeStyle#stroke} for the line.
       * <p>
       * The default value is {@link yfiles.view.Stroke#BLACK}.
       * </p>
       * @default 'System.Drawing.Pens.Black'
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the smoothing length used for creating smooth bends.
       * <p>
       * A value of <code>0.0d</code> will disable smoothing.
       * </p>
       * <p>
       * The default value is <code>0</code>.
       * </p>
       * @default 0.0
       * @type {number}
       */
      smoothingLength:number;
      /**
       * Gets or sets the visual arrow at the target end of edges that use this style.
       * <p>
       * Arrow instances may be shared between multiple style instances.
       * </p>
       * <p>
       * This property may not be <code>null</code>. To have no target arrow set, use {@link yfiles.styles.IArrow#NONE} instead.
       * </p>
       * <p>
       * The default value is {@link yfiles.styles.IArrow#NONE}.
       * </p>
       * @default 'None'
       * @type {yfiles.styles.IArrow}
       */
      targetArrow:yfiles.styles.IArrow;
      /**
       * Gets or sets the visual arrow at the source end of edges that use this style.
       * <p>
       * Arrow instances may be shared between multiple style instances.
       * </p>
       * <p>
       * This property may not be <code>null</code>. To have no source arrow set, use {@link yfiles.styles.IArrow#NONE} instead.
       * </p>
       * <p>
       * The default value is {@link yfiles.styles.IArrow#NONE}.
       * </p>
       * @default 'None'
       * @type {yfiles.styles.IArrow}
       */
      sourceArrow:yfiles.styles.IArrow;
      /**
       * Gets the renderer implementation that has been supplied to this instance in the constructor.
       * @type {yfiles.styles.IEdgeStyleRenderer}
       */
      renderer:yfiles.styles.IEdgeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    export enum ShapeNodeShape{
      /**
       * A rectangular shape
       */
      RECTANGLE,
      /**
       * A rectangular shape with rounded edges
       */
      ROUND_RECTANGLE,
      /**
       * An elliptical shape
       */
      ELLIPSE,
      /**
       * A triangular shape that points to the top
       */
      TRIANGLE,
      /**
       * A triangular shape that points to the bottom
       */
      TRIANGLE2,
      /**
       * A rectangle that is sheared in the horizontal direction to the right
       */
      SHEARED_RECTANGLE,
      /**
       * A rectangle that is sheared in the horizontal direction to the left
       */
      SHEARED_RECTANGLE2,
      /**
       * A trapezoid shape that is smaller at the bottom
       */
      TRAPEZ,
      /**
       * A trapezoid shape that is smaller at the top
       */
      TRAPEZ2,
      /**
       * A 5-star shape
       */
      STAR5,
      /**
       * A 6-star shape
       */
      STAR6,
      /**
       * An 8-star shape
       */
      STAR8,
      /**
       * An arrow like shape that points to the right
       */
      FAT_ARROW,
      /**
       * An arrow like shape that points to the left
       */
      FAT_ARROW2,
      /**
       * A symmetric parallelogram shape that has sloped edges
       */
      DIAMOND,
      /**
       * A regular eight-sided shape,
       */
      OCTAGON,
      /**
       * A regular six-sided shape,
       */
      HEXAGON
    }
    /**
     * Default renderer implementation that can be used for {@link yfiles.styles.ShapeNodeStyle} instances.
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface ShapeNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class ShapeNodeStyleRenderer {
      constructor();
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.ShapeNodeStyleRenderer#style} and {@link yfiles.styles.ShapeNodeStyleRenderer#node} properies have been populated by the {@link yfiles.styles.ShapeNodeStyleRenderer#getVisualCreator},
       * {@link yfiles.styles.ShapeNodeStyleRenderer#getBoundsProvider}, {@link yfiles.styles.ShapeNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.ShapeNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Returns the layout bounds as an upper bound for the painting bound.
       * <p>
       * This implementation simply sets the {@link yfiles.styles.ShapeNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
       * </p>
       * @param context the context to calculate the bounds for
       * @returns the bounds or <code>null</code> to indicate an unbound area
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.ShapeNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.ShapeNodeStyleRenderer#lookup} method call they should call {@link yfiles.styles.ShapeNodeStyleRenderer#configure}
       * only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Get the brush that is currently used by this renderer.
       * <p>
       * This implementation retrieves the brush from the owning style
       * </p>
       * @returns The brush that is currently used by this renderer.
       * @protected
       */
      getFill():yfiles.view.Fill;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * 
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Get the shape that is currently used by this renderer.
       * <p>
       * This implementation retrieves the shape from the owning style
       * </p>
       * @returns The shape that is currently used by this renderer.
       * @protected
       */
      getShape():yfiles.styles.ShapeNodeShape;
      /**
       * Configures the {@link yfiles.styles.ShapeNodeStyleRenderer#style} and {@link yfiles.styles.ShapeNodeStyleRenderer#node} properties, calls {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#isInside
       * @see yfiles.styles.ShapeNodeStyleRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Get the pen that is currently used by this renderer.
       * <p>
       * This implementation retrieves the {@link yfiles.view.Stroke} from the owning style
       * </p>
       * @returns The {@link } that is currently used by this renderer.
       * @protected
       */
      getStroke():yfiles.view.Stroke;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.ShapeNodeStyleRenderer#configure}. If the
       * subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.ShapeNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShapeNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShapeNodeStyleRenderer#createVisual
       * @see yfiles.styles.ShapeNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Tests the {@link yfiles.styles.ShapeNodeStyleRenderer#node node} using the current {@link yfiles.styles.ShapeNodeStyleRenderer#style} for an intersection with the given box.
       * <p>
       * This implementation uses the {@link yfiles.styles.ShapeNodeStyleRenderer#layout} to determine whether the node is in the box.
       * </p>
       * @param context the current input mode context
       * @param rectangle the box describing the marquee's bounds
       * @returns <code>true</code> if the node is considered to be captured by the marquee
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * Determines visibility by checking for an intersection with the {@link yfiles.styles.ShapeNodeStyleRenderer#layout}.
       * @param context The context.
       * @param rectangle The current clip
       * @returns Whether the {@link #layout} is visible in the clip.
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implements the {@link yfiles.graph.ILookup} interface.
       * <p>
       * This method will be used by default if {@link yfiles.styles.ShapeNodeStyleRenderer#getContext} has been queried for a lookup
       * implementation. Note that it cannot be assumed that {@link yfiles.styles.ShapeNodeStyleRenderer#configure} has already been
       * invoked. However, normally {@link yfiles.styles.ShapeNodeStyleRenderer#node} and {@link yfiles.styles.ShapeNodeStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
       * implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
       * This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link yfiles.styles.ShapeNodeStyleRenderer#configure}
       * on success and return <code>this</code>.
       * </p>
       * @param type The type to query for.
       * @returns An implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.ShapeNodeStyle}
       */
      style:yfiles.styles.ShapeNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Gets or sets the radius that is used to draw rounded edges.
       * @type {number}
       */
      roundRectArcRadius:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Node style implementation that can show predefined shapes of type {@link yfiles.styles.ShapeNodeShape}.
     * <p>
     * This style can be used together with {@link yfiles.styles.ShapeNodeStyleRenderer} instances.
     * </p>
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface ShapeNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class ShapeNodeStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer Custom renderer instance for this style.
       */
      constructor(renderer:yfiles.styles.ShapeNodeStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer Custom renderer instance for this style.
       * @param {yfiles.view.Fill} options.fill The {@link yfiles.styles.ShapeNodeStyle#fill} for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShapeNodeStyle#fill} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.ShapeNodeStyle#stroke} for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShapeNodeStyle#stroke} property on the created object.
       * </p>
       * @param {yfiles.styles.ShapeNodeShape} options.shape The shape for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShapeNodeStyle#shape} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.ShapeNodeStyleRenderer,fill?:yfiles.view.Fill,stroke?:yfiles.view.Stroke,shape?:yfiles.styles.ShapeNodeShape});
      /**
       * Creates a new instance using {@link yfiles.styles.ShapeNodeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Fill} options.fill The {@link yfiles.styles.ShapeNodeStyle#fill} for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShapeNodeStyle#fill} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.ShapeNodeStyle#stroke} for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShapeNodeStyle#stroke} property on the created object.
       * </p>
       * @param {yfiles.styles.ShapeNodeShape} options.shape The shape for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShapeNodeStyle#shape} property on the created object.
       * </p>
       */
      constructor(options?:{fill?:yfiles.view.Fill,stroke?:yfiles.view.Stroke,shape?:yfiles.styles.ShapeNodeShape});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the {@link yfiles.styles.ShapeNodeStyle#stroke} for this style.
       * <p>
       * The default value is {@link yfiles.view.Stroke#BLACK}.
       * </p>
       * @default 'System.Drawing.Pens.Black'
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the {@link yfiles.styles.ShapeNodeStyle#fill} for this style.
       * <p>
       * The default value is {@link yfiles.view.Fill#WHITE}.
       * </p>
       * @default 'System.Drawing.Brushes.White'
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      /**
       * Gets or sets the shape for this style.
       * <p>
       * The default value is {@link yfiles.styles.ShapeNodeShape#RECTANGLE}.
       * </p>
       * @default yfiles.styles.ShapeNodeShape.RECTANGLE
       * @type {yfiles.styles.ShapeNodeShape}
       */
      shape:yfiles.styles.ShapeNodeShape;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default renderer implementation for {@link yfiles.styles.ArcEdgeStyle}
     * @class
     * @extends {yfiles.styles.PathBasedEdgeStyleRenderer.<yfiles.styles.ArcEdgeStyle>}
     */
    export interface ArcEdgeStyleRenderer extends yfiles.styles.PathBasedEdgeStyleRenderer<yfiles.styles.ArcEdgeStyle>{}
    export class ArcEdgeStyleRenderer {
      constructor();
      /**
       * Retrieves the {@link yfiles.styles.ArcEdgeStyle#height} of the style.
       * @returns the height.
       */
      getHeight():number;
      /**
       * Gets a value indicating whether this {@link yfiles.styles.ArcEdgeStyleRenderer#getHeight} should be interpreted as an absolute
       * or relative value.
       * @returns <code>true</code> if the height value should be interpreted as a fixed value, otherwise, <code>false</code>.
       * @see yfiles.styles.ArcEdgeStyleRenderer#getHeight
       */
      isFixedHeight():boolean;
      /**
       * Overridden to return <code>false</code> since the flattening interferes with its own obstacles.
       * @protected
       * @type {boolean}
       */
      addBridges:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides an arc representation for an edge.
     * <p>
     * The rendering does not take into account bends of an edge, instead the arc's height is determined as a fixed ratio or
     * fixed height.
     * </p>
     * <p>
     * This style uses an {@link yfiles.styles.ArcEdgeStyleRenderer} for its visualization.
     * </p>
     * @class
     * @implements {yfiles.styles.IEdgeStyle}
     */
    export interface ArcEdgeStyle extends Object,yfiles.styles.IEdgeStyle{}
    export class ArcEdgeStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderer:yfiles.styles.ArcEdgeStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer The custom renderer for this style instance.
       * @param {boolean} options.fixedHeight A value indicating whether this {@link yfiles.styles.ArcEdgeStyle} interprets the {@link yfiles.styles.ArcEdgeStyle#height} value as an absolute or relative value.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#fixedHeight} property on the created object.
       * </p>
       * @param {number} options.height The "height" of the arc.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#height} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.ArcEdgeStyle#stroke} that is used to draw the arc
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#stroke} property on the created object.
       * </p>
       * @param {boolean} options.provideHeightHandle A value indicating whether the {@link yfiles.styles.ArcEdgeStyleRenderer} should provide an {@link yfiles.input.IHandle} if queried for the {@link yfiles.input.IHandleProvider} implementation that allows for adjusting the {@link yfiles.styles.ArcEdgeStyle#height} of this instance.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#provideHeightHandle} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.sourceArrow The visual arrow at the source end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#sourceArrow} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.targetArrow The visual arrow at the target end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#targetArrow} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.ArcEdgeStyleRenderer,fixedHeight?:boolean,height?:number,stroke?:yfiles.view.Stroke,provideHeightHandle?:boolean,sourceArrow?:yfiles.styles.IArrow,targetArrow?:yfiles.styles.IArrow});
      /**
       * Creates a new instance using {@link yfiles.styles.ArcEdgeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.fixedHeight A value indicating whether this {@link yfiles.styles.ArcEdgeStyle} interprets the {@link yfiles.styles.ArcEdgeStyle#height} value as an absolute or relative value.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#fixedHeight} property on the created object.
       * </p>
       * @param {number} options.height The "height" of the arc.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#height} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.ArcEdgeStyle#stroke} that is used to draw the arc
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#stroke} property on the created object.
       * </p>
       * @param {boolean} options.provideHeightHandle A value indicating whether the {@link yfiles.styles.ArcEdgeStyleRenderer} should provide an {@link yfiles.input.IHandle} if queried for the {@link yfiles.input.IHandleProvider} implementation that allows for adjusting the {@link yfiles.styles.ArcEdgeStyle#height} of this instance.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#provideHeightHandle} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.sourceArrow The visual arrow at the source end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#sourceArrow} property on the created object.
       * </p>
       * @param {yfiles.styles.IArrow} options.targetArrow The visual arrow at the target end of edges that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.ArcEdgeStyle#targetArrow} property on the created object.
       * </p>
       */
      constructor(options?:{fixedHeight?:boolean,height?:number,stroke?:yfiles.view.Stroke,provideHeightHandle?:boolean,sourceArrow?:yfiles.styles.IArrow,targetArrow?:yfiles.styles.IArrow});
      /**
       * Performs a {@link Object#memberwiseClone}
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the {@link yfiles.styles.ArcEdgeStyle#stroke} that is used to draw the arc
       * <p>
       * The default value is {@link yfiles.view.Stroke#BLACK}.
       * </p>
       * @default 'System.Drawing.Pens.Black'
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the "height" of the arc.
       * <p>
       * Depending on the setting of {@link yfiles.styles.ArcEdgeStyle#fixedHeight}, this value is interpreted differently: If {@link yfiles.styles.ArcEdgeStyle#fixedHeight} is disabled, the height of the arc will depend
       * on it's width. The width will be multiplied by this value to obtain the height. If the {@link yfiles.styles.ArcEdgeStyle#fixedHeight} feature is enabled, this value
       * will be interpreted as the absolute height.
       * </p>
       * <p>
       * The default value is <code>0</code>.
       * </p>
       * @default 0
       * @type {number}
       */
      height:number;
      /**
       * Gets or sets a value indicating whether this {@link yfiles.styles.ArcEdgeStyle} interprets the {@link yfiles.styles.ArcEdgeStyle#height} value as an absolute or relative
       * value.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.styles.ArcEdgeStyle#height
       * @default true
       * @type {boolean}
       */
      fixedHeight:boolean;
      /**
       * Gets or sets a value indicating whether the {@link yfiles.styles.ArcEdgeStyleRenderer} should provide an {@link yfiles.input.IHandle} if
       * queried for the {@link yfiles.input.IHandleProvider} implementation that allows for adjusting the {@link yfiles.styles.ArcEdgeStyle#height} of this instance.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      provideHeightHandle:boolean;
      /**
       * Gets or sets the visual arrow at the target end of edges that use this style.
       * <p>
       * Arrow instances may be shared between multiple style instances.
       * </p>
       * <p>
       * The default value is {@link yfiles.styles.IArrow#NONE}.
       * </p>
       * @default 'None'
       * @type {yfiles.styles.IArrow}
       */
      targetArrow:yfiles.styles.IArrow;
      /**
       * Gets or sets the visual arrow at the source end of edges that use this style.
       * <p>
       * Arrow instances may be shared between multiple style instances.
       * </p>
       * <p>
       * The default value is {@link yfiles.styles.IArrow#NONE}.
       * </p>
       * @default 'None'
       * @type {yfiles.styles.IArrow}
       */
      sourceArrow:yfiles.styles.IArrow;
      /**
       * Gets the renderer implementation that has been supplied to this instance in the constructor.
       * @type {yfiles.styles.IEdgeStyleRenderer}
       */
      renderer:yfiles.styles.IEdgeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.INodeStyle} {@link yfiles.styles.INodeStyleRenderer renderer} implementation that draws a rounded rectangle with a bevel border in a 'shiny plate'
     * fashion.
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface BevelNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class BevelNodeStyleRenderer {
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.BevelNodeStyleRenderer#style} and {@link yfiles.styles.BevelNodeStyleRenderer#node} properties have been populated by the {@link yfiles.styles.BevelNodeStyleRenderer#getVisualCreator},
       * {@link yfiles.styles.BevelNodeStyleRenderer#getBoundsProvider}, {@link yfiles.styles.BevelNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.BevelNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Returns the layout bounds as an upper bound for the painting bound.
       * <p>
       * This implementation simply sets the {@link yfiles.styles.BevelNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
       * </p>
       * @param context the context to calculate the bounds for
       * @returns the bounds or <code>null</code> to indicate an unbound area
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.BevelNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.BevelNodeStyleRenderer#lookup} method call they should call {@link yfiles.styles.BevelNodeStyleRenderer#configure}
       * only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * 
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Configures the {@link yfiles.styles.BevelNodeStyleRenderer#style} and {@link yfiles.styles.BevelNodeStyleRenderer#node} properties, calls {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#isInside
       * @see yfiles.styles.BevelNodeStyleRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.BevelNodeStyleRenderer#configure}. If the
       * subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.BevelNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.BevelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.BevelNodeStyleRenderer#createVisual
       * @see yfiles.styles.BevelNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Tests the {@link yfiles.styles.BevelNodeStyleRenderer#node node} using the current {@link yfiles.styles.BevelNodeStyleRenderer#style} for an intersection with the given box.
       * <p>
       * This implementation uses the {@link yfiles.styles.BevelNodeStyleRenderer#layout} to determine whether the node is in the box.
       * </p>
       * @param context the current input mode context
       * @param rectangle the box describing the marquee's bounds
       * @returns <code>true</code> if the node is considered to be captured by the marquee
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * Determines visibility by checking for an intersection with the {@link yfiles.styles.BevelNodeStyleRenderer#layout}.
       * @param context The context.
       * @param rectangle The current clip
       * @returns Whether the {@link #layout} is visible in the clip.
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.BevelNodeStyle}
       */
      style:yfiles.styles.BevelNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Retrieves the color from the style.
       * @protected
       * @type {yfiles.view.Color}
       */
      color:yfiles.view.Color;
      /**
       * Retrieves the inset from the style.
       * @protected
       * @type {number}
       */
      inset:number;
      /**
       * Retrieves the radius to use from the style.
       * @protected
       * @type {number}
       */
      radius:number;
      /**
       * Gets or sets a property that determines whether this instance should provide an {@link yfiles.input.IHandle} for editing the
       * radius of the shape.
       * <p>
       * The default is <code>false</code>. This implementation will add an appropriate {@link yfiles.input.IHandleProvider} to the {@link yfiles.styles.BevelNodeStyleRenderer#lookup}
       * of this instance if this feature is enabled.
       * </p>
       * @default false
       * @type {boolean}
       */
      provideRadiusHandle:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.styles.INodeStyle} that can be used to create rectangular nodes with rounded corners, a
     * bevel border, and a shining background.
     * <p>
     * Optionally, this style renders a drop shadow by itself.
     * </p>
     * <p>
     * This style uses the {@link yfiles.styles.BevelNodeStyleRenderer} to visualize a node.
     * </p>
     * @see yfiles.styles.BevelNodeStyleRenderer
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface BevelNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class BevelNodeStyle {
      /**
       * Creates a new instance using the provided renderer to share.
       * @param renderer
       */
      constructor(renderer:yfiles.styles.BevelNodeStyleRenderer);
      /**
       * Creates a new instance using the provided renderer to share.
       * @param {Object} options The parameters to pass.
       * @param options.renderer
       * @param {yfiles.view.Color} options.color The base color to use.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#color} property on the created object.
       * </p>
       * @param {boolean} options.drawShadow A value indicating whether to draw a drop shadow.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#drawShadow} property on the created object.
       * </p>
       * @param {number} options.inset The insets to use for the bevel.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#inset} property on the created object.
       * </p>
       * @param {number} options.radius The radius of the corner of the rounded rectangle.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#radius} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.BevelNodeStyleRenderer,color?:yfiles.view.Color,drawShadow?:boolean,inset?:number,radius?:number});
      /**
       * Creates a new instance using {@link yfiles.styles.BevelNodeStyleRenderer} as renderer to share.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Color} options.color The base color to use.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#color} property on the created object.
       * </p>
       * @param {boolean} options.drawShadow A value indicating whether to draw a drop shadow.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#drawShadow} property on the created object.
       * </p>
       * @param {number} options.inset The insets to use for the bevel.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#inset} property on the created object.
       * </p>
       * @param {number} options.radius The radius of the corner of the rounded rectangle.
       * <p>
       * This option sets the {@link yfiles.styles.BevelNodeStyle#radius} property on the created object.
       * </p>
       */
      constructor(options?:{color?:yfiles.view.Color,drawShadow?:boolean,inset?:number,radius?:number});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the insets to use for the bevel.
       * <p>
       * The default value is <code>3</code>.
       * </p>
       * @type {number}
       */
      inset:number;
      /**
       * Gets or sets the radius of the corner of the rounded rectangle.
       * <p>
       * The default value is <code>10</code>.
       * </p>
       * @type {number}
       */
      radius:number;
      /**
       * Gets or sets the base color to use.
       * <p>
       * The default value is {@link yfiles.view.Color#BLACK}.
       * </p>
       * @default 'Black'
       * @type {yfiles.view.Color}
       */
      color:yfiles.view.Color;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      /**
       * Gets or sets a value indicating whether to draw a drop shadow.
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @default false
       * @type {boolean}
       */
      drawShadow:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A style renderer implementation that can be used together with {@link yfiles.styles.ImageNodeStyle} instances.
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface ImageNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class ImageNodeStyleRenderer {
      constructor();
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.ImageNodeStyleRenderer#style} and {@link yfiles.styles.ImageNodeStyleRenderer#node} properties have been populated by the {@link yfiles.styles.ImageNodeStyleRenderer#getVisualCreator},
       * {@link yfiles.styles.ImageNodeStyleRenderer#getBoundsProvider}, {@link yfiles.styles.ImageNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.ImageNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Returns the layout bounds as an upper bound for the painting bound.
       * <p>
       * This implementation simply sets the {@link yfiles.styles.ImageNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
       * </p>
       * @param context the context to calculate the bounds for
       * @returns the bounds or <code>null</code> to indicate an unbound area
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.ImageNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.ImageNodeStyleRenderer#lookup} method call they should call {@link yfiles.styles.ImageNodeStyleRenderer#configure}
       * only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Get the style's fallback image.
       * @returns 
       * @protected
       */
      getFallbackImage():string;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * Get the image that is currently used to render the style.
       * @returns 
       * @protected
       */
      getImage():string;
      /**
       * Tries to use the {@link yfiles.styles.ImageNodeStyle#normalizedOutline} to perform the intersection calculation, otherwise reverts to default (rectangular) behavior.
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Tries to use the {@link yfiles.styles.ImageNodeStyle#normalizedOutline} to create the outline shape path, otherwise reverts to default (rectangular) behavior.
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Gets the value that determines whether the aspect ratio of the image should be preserved.
       * @returns false
       * @protected
       */
      getPreserveAspectRatio():boolean;
      /**
       * Configures the {@link yfiles.styles.ImageNodeStyleRenderer#style} and {@link yfiles.styles.ImageNodeStyleRenderer#node} properties, calls {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#isInside
       * @see yfiles.styles.ImageNodeStyleRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.ImageNodeStyleRenderer#configure}. If the
       * subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.ImageNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ImageNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ImageNodeStyleRenderer#createVisual
       * @see yfiles.styles.ImageNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * Tries to use {@link yfiles.styles.ImageNodeStyle#normalizedOutline} to perform the hit test analysis, otherwise reverts to default (rectangular) behavior.
       * @param context the context the hit test is performed in
       * @param location the coordinates in world coordinate system
       * @returns whether something has been hit
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Tries to use the {@link yfiles.styles.ImageNodeStyle#normalizedOutline} to perform the marquee intersection analysis, otherwise reverts to default (rectangular) behavior.
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Tries to use the {@link yfiles.styles.ImageNodeStyle#normalizedOutline} to perform the contains test, otherwise reverts to default (rectangular) behavior.
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * Determines visibility by checking for an intersection with the {@link yfiles.styles.ImageNodeStyleRenderer#layout}.
       * @param context The context.
       * @param rectangle The current clip
       * @returns Whether the {@link #layout} is visible in the clip.
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implements the {@link yfiles.graph.ILookup} interface.
       * <p>
       * This method will be used by default if {@link yfiles.styles.ImageNodeStyleRenderer#getContext} has been queried for a lookup
       * implementation. Note that it cannot be assumed that {@link yfiles.styles.ImageNodeStyleRenderer#configure} has already been
       * invoked. However, normally {@link yfiles.styles.ImageNodeStyleRenderer#node} and {@link yfiles.styles.ImageNodeStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
       * implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
       * This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link yfiles.styles.ImageNodeStyleRenderer#configure}
       * on success and return <code>this</code>.
       * </p>
       * @param type The type to query for.
       * @returns An implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.ImageNodeStyle}
       */
      style:yfiles.styles.ImageNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * A {@link string} that will be used to find the fallback image url that will be used to render the node, if the style does not
       * explicitly specify a {@link yfiles.styles.ImageNodeStyle#fallbackImage fallback image}.
       * @const
       * @static
       * @type {string}
       */
      static TEMPLATE_KEY:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * A node style that can use an image for the visual representation of a node.
     * <p>
     * This style can be used together with {@link yfiles.styles.ImageNodeStyleRenderer} instances.
     * </p>
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface ImageNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class ImageNodeStyle {
      /**
       * Creates a new instance that uses the specified image and a custom renderer instance.
       * @param image The image.
       * @param [fallbackImage=null] The fallback image.
       * @param [renderer=null] The renderer for this instance
       */
      constructor(image:string,fallbackImage?:string,renderer?:yfiles.styles.ImageNodeStyleRenderer);
      /**
       * Creates a new instance that uses the specified image and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.image The image.
       * @param [options.fallbackImage=null] The fallback image.
       * @param [options.renderer=null] The renderer for this instance
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of the image that will be displayed.
       * <p>
       * This option sets the {@link yfiles.styles.ImageNodeStyle#normalizedOutline} property on the created object.
       * </p>
       */
      constructor(options:{image:string,fallbackImage?:string,renderer?:yfiles.styles.ImageNodeStyleRenderer,normalizedOutline?:yfiles.geometry.GeneralPath});
      /**
       * Creates a new instance with no image and {@link yfiles.styles.ImageNodeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {string} [options.fallbackImage=null] Sets the fallbackImage property.
       * @param {string} options.image The image that is used for the style instance.
       * <p>
       * This option sets the {@link yfiles.styles.ImageNodeStyle#image} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of the image that will be displayed.
       * <p>
       * This option sets the {@link yfiles.styles.ImageNodeStyle#normalizedOutline} property on the created object.
       * </p>
       */
      constructor(options?:{fallbackImage?:string,image?:string,normalizedOutline?:yfiles.geometry.GeneralPath});
      /**
       * This implementation performs a shallow copy with respect to the {@link yfiles.styles.ImageNodeStyle#image} property.
       * <p>
       * If clients need to have a deep copy of this instance, they need to manually clone and reassign the {@link yfiles.styles.ImageNodeStyle#image} to the clone.
       * </p>
       * @returns A shallow copy of this instance.
       */
      clone():Object;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      /**
       * Gets or sets the image that is used for the style instance.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @type {string}
       */
      image:string;
      /**
       * 
       * @default null
       * @type {string}
       */
      fallbackImage:string;
      /**
       * Gets or sets the normalized outline of the image that will be displayed.
       * <p>
       * This shape will be used to satisfy requests to {@link yfiles.input.IMarqueeTestable#isInBox} and {@link yfiles.input.IHitTestable#isHit}
       * and may be left <code>null</code> to indicate default (rectangular) behavior.
       * </p>
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.geometry.GeneralPath}
       */
      normalizedOutline:yfiles.geometry.GeneralPath;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of {@link yfiles.styles.INodeStyle} that draws a simple floating panel with a slight gradient, a thin border
     * and optionally a drop shadow.
     * <p>
     * This style can be used to create the visual representation for group nodes, for example. By default, it a drop shadow by
     * itself.
     * </p>
     * <p>
     * This style uses the {@link yfiles.styles.PanelNodeStyleRenderer} to visualize a node.
     * </p>
     * @see yfiles.styles.PanelNodeStyleRenderer
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface PanelNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class PanelNodeStyle {
      /**
       * Creates a new instance using the provided renderer to share.
       * @param renderer
       */
      constructor(renderer:yfiles.styles.PanelNodeStyleRenderer);
      /**
       * Creates a new instance using the provided renderer to share.
       * @param {Object} options The parameters to pass.
       * @param options.renderer
       * @param {yfiles.view.Color} options.color The base color to use.
       * <p>
       * This option sets the {@link yfiles.styles.PanelNodeStyle#color} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.insets The instance to use that provides the insets for this style.
       * <p>
       * This option sets the {@link yfiles.styles.PanelNodeStyle#insets} property on the created object.
       * </p>
       * @param {yfiles.view.Color} options.labelInsetsColor The base color to use for drawing the label insets background.
       * <p>
       * This option sets the {@link yfiles.styles.PanelNodeStyle#labelInsetsColor} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.PanelNodeStyleRenderer,color?:yfiles.view.Color,insets?:yfiles.geometry.Insets,labelInsetsColor?:yfiles.view.Color});
      /**
       * Creates a new instance using {@link yfiles.styles.PanelNodeStyleRenderer} as renderer to share.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Color} options.color The base color to use.
       * <p>
       * This option sets the {@link yfiles.styles.PanelNodeStyle#color} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.insets The instance to use that provides the insets for this style.
       * <p>
       * This option sets the {@link yfiles.styles.PanelNodeStyle#insets} property on the created object.
       * </p>
       * @param {yfiles.view.Color} options.labelInsetsColor The base color to use for drawing the label insets background.
       * <p>
       * This option sets the {@link yfiles.styles.PanelNodeStyle#labelInsetsColor} property on the created object.
       * </p>
       */
      constructor(options?:{color?:yfiles.view.Color,insets?:yfiles.geometry.Insets,labelInsetsColor?:yfiles.view.Color});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the base color to use.
       * <p>
       * The default value is {@link yfiles.view.Color#BLACK}.
       * </p>
       * @default 'Black'
       * @type {yfiles.view.Color}
       */
      color:yfiles.view.Color;
      /**
       * Gets or sets the base color to use for drawing the label insets background.
       * <p>
       * Setting this to <code>null</code> effectively disables label insets background coloring.
       * </p>
       * <p>
       * The default value is {@link yfiles.view.Color#LIGHT_GRAY}.
       * </p>
       * @default 'LightGray'
       * @type {yfiles.view.Color}
       */
      labelInsetsColor:yfiles.view.Color;
      /**
       * Gets or sets the instance to use that provides the insets for this style.
       * <p>
       * The {@link yfiles.styles.PanelNodeStyleRenderer} will use these insets and return them via an {@link yfiles.input.INodeInsetsProvider}
       * if such an instance is queried through the {@link yfiles.styles.INodeStyleRenderer#getContext context lookup}.
       * </p>
       * <p>
       * The default insets are set to (5,5,5,5).
       * </p>
       * @see yfiles.input.INodeInsetsProvider
       * @default '5, 5, 5, 5'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.INodeStyle} {@link yfiles.styles.INodeStyleRenderer renderer} implementation that draws a simple floating panel with a slight gradient, a thin
     * border and a simple drop shadow.
     * @see yfiles.styles.PanelNodeStyle
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface PanelNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class PanelNodeStyleRenderer {
      /**
       * The style that it currently assigned to this renderer instance.
       */
      constructor();
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.PanelNodeStyleRenderer#style} and {@link yfiles.styles.PanelNodeStyleRenderer#node} properties have been populated by the {@link yfiles.styles.PanelNodeStyleRenderer#getVisualCreator},
       * {@link yfiles.styles.PanelNodeStyleRenderer#getBoundsProvider}, {@link yfiles.styles.PanelNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.PanelNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Determines whether to draw the drop shadow.
       * @returns <code>true</code>, always.
       * @protected
       */
      drawShadow():boolean;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.PanelNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.PanelNodeStyleRenderer#lookup} method call they should call {@link yfiles.styles.PanelNodeStyleRenderer#configure}
       * only if needed, i.e. if they decide to return <code>this</code> or an instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * Finds an intersection for the current {@link yfiles.styles.PanelNodeStyleRenderer#node node} using the {@link yfiles.styles.PanelNodeStyleRenderer#style}.
       * <p>
       * This implementation returns the intersection for the {@link yfiles.styles.PanelNodeStyleRenderer#layout}.
       * </p>
       * @param inner
       * @param outer
       * @returns True if an intersection was actually found
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Returns the outline of the shape using the node's {@link yfiles.styles.PanelNodeStyleRenderer#layout}.
       * @returns The outline of the bounds of the node.
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Configures the {@link yfiles.styles.PanelNodeStyleRenderer#style} and {@link yfiles.styles.PanelNodeStyleRenderer#node} properties, calls {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#isInside
       * @see yfiles.styles.PanelNodeStyleRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.PanelNodeStyleRenderer#configure}. If the
       * subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.PanelNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.PanelNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.PanelNodeStyleRenderer#createVisual
       * @see yfiles.styles.PanelNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * Hit tests the {@link yfiles.styles.PanelNodeStyleRenderer#node node} using the current {@link yfiles.styles.PanelNodeStyleRenderer#style}.
       * <p>
       * This implementation uses the {@link yfiles.styles.PanelNodeStyleRenderer#layout} to determine whether the node has been hit. The check is delegated to {@link yfiles.geometry.Rect#containsWithEps}
       * using {@link yfiles.view.ICanvasContext#hitTestRadius} as the last argument.
       * </p>
       * @param context the context the hit test is performed in
       * @param location the coordinates in world coordinate system
       * @returns whether something has been hit
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Tests the {@link yfiles.styles.PanelNodeStyleRenderer#node node} using the current {@link yfiles.styles.PanelNodeStyleRenderer#style} for an intersection with the given box.
       * <p>
       * This implementation uses the {@link yfiles.styles.PanelNodeStyleRenderer#layout} to determine whether the node is in the box.
       * </p>
       * @param context the current input mode context
       * @param rectangle the box describing the marquee's bounds
       * @returns <code>true</code> if the node is considered to be captured by the marquee
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Checks whether the given coordinate lies within the shape's geometric bounds.
       * <p>
       * This implementation checks for containment in the {@link yfiles.styles.PanelNodeStyleRenderer#layout}.
       * </p>
       * @param location
       * @returns True if the point lies within the geometry of the rendering.
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.PanelNodeStyle}
       */
      style:yfiles.styles.PanelNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Retrieves the color from the style.
       * @protected
       * @type {yfiles.view.Color}
       */
      color:yfiles.view.Color;
      static $class:yfiles.lang.Class;
    }
    /**
     * Decorates other node styles with a smooth drop shadow.
     * <p>
     * This implementation uses an SVG filter effect. This may cause performance issues for larger nodes or high zoom factors.
     * In this case it might be more appropriate to implement a drop shadow implementation using canvas drawing and displaying
     * the canvas content using an <code>image</code> element with <code>canvas.toDataURL</code>.
     * </p>
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface ShadowNodeStyleDecorator extends Object,yfiles.styles.INodeStyle{}
    export class ShadowNodeStyleDecorator {
      /**
       * Creates a new instance wrapping the provided style.
       * <p>
       * The provided instance is used by reference, so subsequent changes to the style will affect the appearance of this
       * style.
       * </p>
       * @param wrapped The style to wrap.
       */
      constructor(wrapped:yfiles.styles.INodeStyle);
      /**
       * Creates a new instance wrapping a simple {@link yfiles.styles.ShapeNodeStyle}.
       * <p>
       * The provided instance is used by reference, so subsequent changes to the style will affect the appearance of this
       * style.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.INodeStyle} options.wrapped The wrapped node style instance by reference.
       * <p>
       * This option sets the {@link yfiles.styles.ShadowNodeStyleDecorator#wrapped} property on the created object.
       * </p>
       * @see yfiles.styles.ShadowNodeStyleDecorator#wrapped
       */
      constructor(options?:{wrapped?:yfiles.styles.INodeStyle});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the wrapped node style instance by reference.
       * @type {yfiles.styles.INodeStyle}
       */
      wrapped:yfiles.styles.INodeStyle;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Implementation of the {@link yfiles.styles.INodeStyle} which creates rectangular nodes with rounded corners and a highlighted
     * ('shiny plate') interior.
     * <p>
     * By default, this style renders a drop shadow by itself.
     * </p>
     * <p>
     * This style uses the {@link yfiles.styles.ShinyPlateNodeStyleRenderer} to visualize a node.
     * </p>
     * @see yfiles.styles.ShinyPlateNodeStyleRenderer
     * @class
     * @implements {yfiles.styles.INodeStyle}
     */
    export interface ShinyPlateNodeStyle extends Object,yfiles.styles.INodeStyle{}
    export class ShinyPlateNodeStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer
       */
      constructor(renderer:yfiles.styles.ShinyPlateNodeStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer
       * @param {yfiles.view.Fill} options.fill The background brush for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#fill} property on the created object.
       * </p>
       * @param {boolean} options.drawShadow A value indicating whether to draw a drop shadow.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#drawShadow} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.insets The instance to use that provides the insets for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#insets} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.ShinyPlateNodeStyle#stroke} to use for the outline.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#stroke} property on the created object.
       * </p>
       * @param {number} options.radius The corner radius of the rounded rectangle
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#radius} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.ShinyPlateNodeStyleRenderer,fill?:yfiles.view.Fill,drawShadow?:boolean,insets?:yfiles.geometry.Insets,stroke?:yfiles.view.Stroke,radius?:number});
      /**
       * Creates a new instance using {@link yfiles.styles.ShinyPlateNodeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.view.Fill} options.fill The background brush for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#fill} property on the created object.
       * </p>
       * @param {boolean} options.drawShadow A value indicating whether to draw a drop shadow.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#drawShadow} property on the created object.
       * </p>
       * @param {yfiles.geometry.Insets} options.insets The instance to use that provides the insets for this style.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#insets} property on the created object.
       * </p>
       * @param {yfiles.view.Stroke} options.stroke The {@link yfiles.styles.ShinyPlateNodeStyle#stroke} to use for the outline.
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#stroke} property on the created object.
       * </p>
       * @param {number} options.radius The corner radius of the rounded rectangle
       * <p>
       * This option sets the {@link yfiles.styles.ShinyPlateNodeStyle#radius} property on the created object.
       * </p>
       */
      constructor(options?:{fill?:yfiles.view.Fill,drawShadow?:boolean,insets?:yfiles.geometry.Insets,stroke?:yfiles.view.Stroke,radius?:number});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Gets or sets the background brush for this style.
       * <p>
       * The default value is {@link yfiles.view.Fill#BLACK}.
       * </p>
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      /**
       * Gets or sets the {@link yfiles.styles.ShinyPlateNodeStyle#stroke} to use for the outline.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Gets or sets the corner radius of the rounded rectangle
       * <p>
       * The default value is <code>5</code>.
       * </p>
       * @default 5
       * @type {number}
       */
      radius:number;
      /**
       * Gets or sets a value indicating whether to draw a drop shadow.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      drawShadow:boolean;
      /**
       * Gets or sets the instance to use that provides the insets for this style.
       * <p>
       * The {@link yfiles.styles.ShinyPlateNodeStyleRenderer} will use these insets and return them via an {@link yfiles.input.INodeInsetsProvider}
       * if such an instance is queried through the {@link yfiles.styles.INodeStyleRenderer#getContext context lookup}.
       * </p>
       * <p>
       * The default insets are set to (5,5,5,5).
       * </p>
       * @see yfiles.input.INodeInsetsProvider
       * @default '5,5,5,5'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.INodeStyle} {@link yfiles.styles.INodeStyleRenderer renderer} implementation that draws a simple floating shinyPlate with a slight gradient, a
     * thin border and a simple drop shadow.
     * @see yfiles.styles.ShinyPlateNodeStyle
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface ShinyPlateNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class ShinyPlateNodeStyleRenderer {
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.ShinyPlateNodeStyleRenderer#style} and {@link yfiles.styles.ShinyPlateNodeStyleRenderer#node} properties have been populated by the
       * {@link yfiles.styles.ShinyPlateNodeStyleRenderer#getVisualCreator}, {@link yfiles.styles.ShinyPlateNodeStyleRenderer#getBoundsProvider},
       * {@link yfiles.styles.ShinyPlateNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.ShinyPlateNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Determines whether to draw the drop shadow.
       * @returns {@link #drawShadow}.
       * @protected
       */
      drawShadow():boolean;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the bounds provider for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#getBounds
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.ShinyPlateNodeStyleRenderer#lookup} method call they should
       * call {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
       * instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query hit test with. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * 
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to query marquee intersection tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Configures the {@link yfiles.styles.ShinyPlateNodeStyleRenderer#style} and {@link yfiles.styles.ShinyPlateNodeStyleRenderer#node} properties, calls {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the shape geometry for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#isInside
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#getIntersection
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure}. If
       * the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.ShinyPlateNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.ShinyPlateNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#createVisual
       * @see yfiles.styles.ShinyPlateNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Tests the {@link yfiles.styles.ShinyPlateNodeStyleRenderer#node node} using the current {@link yfiles.styles.ShinyPlateNodeStyleRenderer#style} for an intersection with the given box.
       * <p>
       * This implementation uses the {@link yfiles.styles.ShinyPlateNodeStyleRenderer#layout} to determine whether the node is in the box.
       * </p>
       * @param context the current input mode context
       * @param rectangle the box describing the marquee's bounds
       * @returns <code>true</code> if the node is considered to be captured by the marquee
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.ShinyPlateNodeStyle}
       */
      style:yfiles.styles.ShinyPlateNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      /**
       * Retrieves the brush from the style.
       * @protected
       * @type {yfiles.view.Fill}
       */
      fill:yfiles.view.Fill;
      /**
       * Retrieves the {@link yfiles.styles.ShinyPlateNodeStyleRenderer#stroke} from the style.
       * @protected
       * @type {yfiles.view.Stroke}
       */
      stroke:yfiles.view.Stroke;
      /**
       * Retrieves the radius from the style.
       * @protected
       * @type {number}
       */
      radius:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base class for stripe styles that use a SVG snippet as template for the visualization of {@link yfiles.graph.IStripe}s.
     * @see yfiles.styles.TemplateStripeStyle
     * @see yfiles.styles.StringTemplateStripeStyle
     * @class
     * @implements {yfiles.styles.IStripeStyle}
     * @implements {yfiles.lang.IPropertyObservable}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface TemplateStripeStyleBase extends Object,yfiles.styles.IStripeStyle,yfiles.lang.IPropertyObservable,yfiles.lang.ICloneable{}
    export class TemplateStripeStyleBase {
      /**
       * Constructs a new instance using the provided <code>renderer</code>.
       * @param renderer
       * @protected
       */
      constructor(renderer:yfiles.styles.TemplateStripeStyleRenderer);
      /**
       * Constructs a new instance of this style with default values.
       * @protected
       */
      constructor();
      /**
       * Performs a shallow clone of this instance, except for the value of {@link yfiles.styles.TemplateStripeStyleBase#styleTag} which is cloned, too, if it can be cloned.
       * @returns A clone of this instance.
       */
      clone():Object;
      /**
       * Factory method that is called by the {@link yfiles.styles.TemplateStripeStyleRenderer} to create the visual that will be used
       * for the display of the stripe.
       * @param context The context for which the visual should be created.
       * @param stripe The stripe that will be rendered.
       * @returns The readily styled visual.
       */
      createTemplate(context:yfiles.view.IRenderContext,stripe:yfiles.graph.IStripe):yfiles.view.SvgVisual;
      /**
       * Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
       * <p>
       * This method can be called to perform the template loading at a certain point in time, for example during application
       * startup. By default, a template is loaded lazily when the first visual using this template is built.
       * </p>
       * @static
       */
      static loadAllTemplates():void;
      /**
       * Raises the {@link yfiles.styles.TemplateStripeStyleBase#addPropertyChangedListener PropertyChanged}
       * event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to add.
       * @see yfiles.styles.TemplateStripeStyleBase#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to remove.
       * @see yfiles.styles.TemplateStripeStyleBase#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the tag that is associated with this style instance.
       * <p>
       * The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
       * templates and controls. The framework does not make any assumptions about the value of this property. It is there for
       * convenient use in the process of data binding the templates. This property supports {@link yfiles.lang.IPropertyObservable property change notification} and can
       * thus be data bound easily.
       * </p>
       * @default null
       * @type {Object}
       */
      styleTag:Object;
      /**
       * Gets or sets an implementation of {@link yfiles.graph.IContextLookup} that can be used to satisfy queries that are made to the
       * implementation which is returned by calls to {@link yfiles.styles.IStripeStyleRenderer#getContext} that are made on the style
       * renderer that is associated with this style instance.
       * <p>
       * This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
       * instance. The default value is {@link yfiles.graph.IContextLookup#EMPTY_CONTEXT_LOOKUP}
       * </p>
       * @default 'Lookups.EmptyContextLookup'
       * @type {yfiles.graph.IContextLookup}
       */
      contextLookup:yfiles.graph.IContextLookup;
      /**
       * Convenience getter that yields the {@link yfiles.styles.TemplateStripeStyleRenderer} that is used by this instance.
       * @see yfiles.styles.IStripeStyle#renderer
       * @type {yfiles.styles.IStripeStyleRenderer}
       */
      renderer:yfiles.styles.IStripeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.IStripeStyle} implementation that can render complex {@link yfiles.view.Visual}s for the visualization of
     * {@link yfiles.graph.IStripe}s using a {@link yfiles.styles.TemplateStripeStyleRenderer}.
     * <p>
     * The {@link yfiles.styles.TemplateStripeStyle#styleResourceKey} is used to determine the template that will be applied to the {@link yfiles.view.Visual}.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: Control Styles Demo, Demo.yFiles.Graph.ControlStyles</li>
     * <li>Demo: OrgChart Demo, Demo.yFiles.Graph.OrgChart</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplateStripeStyleBase}
     */
    export interface TemplateStripeStyle extends yfiles.styles.TemplateStripeStyleBase{}
    export class TemplateStripeStyle {
      /**
       * Creates a new instance using the given resource key and a custom renderer instance.
       * @param styleResourceKey The key of the style resource that will be used for the component.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(styleResourceKey:string,renderer:yfiles.styles.TemplateStripeStyleRenderer);
      /**
       * Creates a new instance using the given resource key and {@link yfiles.styles.TemplateStripeStyleRenderer} as renderer.
       * @param styleResourceKey The key of the style resource that will be used for the component.
       */
      constructor(styleResourceKey:string);
      /**
       * Creates a new instance with no style resource key and {@link yfiles.styles.TemplateStripeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {string} options.styleResourceKey The {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateStripeStyle#styleResourceKey} property on the created object.
       * </p>
       */
      constructor(options?:{styleResourceKey?:string});
      /**
       * Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
       * @default null
       * @type {string}
       */
      styleResourceKey:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.IStripeStyle} that uses an SVG snippet as template for the visualization of {@link yfiles.graph.IStripe}s.
     * <p>
     * In contrast to {@link yfiles.styles.TemplateStripeStyle}, the SVG template used by this style must be encoded as string and set
     * to the {@link yfiles.styles.StringTemplateStripeStyle#svgContent} property.
     * </p>
     * <p>
     * It is vital that the {@link yfiles.styles.StringTemplateStripeStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
     * in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
     * namespace before they are added to the DOM.
     * </p>
     * @class
     * @extends {yfiles.styles.TemplateStripeStyleBase}
     */
    export interface StringTemplateStripeStyle extends yfiles.styles.TemplateStripeStyleBase{}
    export class StringTemplateStripeStyle {
      /**
       * Creates a new instance using the given SVG template and a custom renderer instance.
       * @param svgContent The SVG snippet that will be used as a template for the component.
       * @param renderer The custom renderer for this instance.
       */
      constructor(svgContent:string,renderer:yfiles.styles.TemplateStripeStyleRenderer);
      /**
       * Creates a new instance using the given SVG template and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.svgContent The SVG snippet that will be used as a template for the component.
       * @param options.renderer The custom renderer for this instance.
       * @param {Object} options.styleTag The tag that is associated with this style instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateStripeStyleBase#styleTag} property on the created object.
       * </p>
       */
      constructor(options:{svgContent:string,renderer:yfiles.styles.TemplateStripeStyleRenderer,styleTag?:Object});
      /**
       * Creates a new instance using the given SVG template and {@link yfiles.styles.TemplateStripeStyleRenderer} as renderer.
       * @param svgContent The SVG snippet that will be used as a template for the component.
       */
      constructor(svgContent:string);
      /**
       * Creates a new instance with no SVG template and {@link yfiles.styles.TemplateStripeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {string} options.svgContent The SVG content that is the template for the stripe visualization.
       * <p>
       * This option sets the {@link yfiles.styles.StringTemplateStripeStyle#svgContent} property on the created object.
       * </p>
       */
      constructor(options?:{svgContent?:string});
      /**
       * Gets or sets the SVG content that is the template for the stripe visualization.
       * @default null
       * @type {string}
       */
      svgContent:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.styles.IStripeStyleRenderer} interface that can render {@link yfiles.styles.TemplateStripeStyle}
     * instances.
     * @see yfiles.styles.TemplateStripeStyle
     * @class
     * @implements {yfiles.styles.IStripeStyleRenderer}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface TemplateStripeStyleRenderer extends Object,yfiles.styles.IStripeStyleRenderer,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class TemplateStripeStyleRenderer {
      /**
       * Prepares this instance for subsequent calls after the style and stripe have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.TemplateStripeStyleRenderer#style} and {@link yfiles.styles.TemplateStripeStyleRenderer#stripe} properties have been populated by the {@link yfiles.styles.TemplateStripeStyleRenderer#getVisualCreator}
       * or {@link yfiles.styles.TemplateStripeStyleRenderer#getContext} methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * Creates the visual by delegating to CreateControl and then initializing the control with the context.
       * @param context The context for the creation.
       * @returns The control that is used for rendering the stripe.
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Configures the <code>style</code> and <code>stripe</code> parameters, does <b>not</b> call {@link yfiles.styles.TemplateStripeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.TemplateStripeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.TemplateStripeStyleRenderer#lookup} method call they should
       * call {@link yfiles.styles.TemplateStripeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
       * instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param stripe The stripe to query the context for. The value will be stored in the {@link #stripe} property.
       * @param style The style to associate with the stripe. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateStripeStyleRenderer#lookup
       */
      getContext(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):yfiles.graph.ILookup;
      /**
       * Gets the context by delegating to {@link yfiles.styles.TemplateStripeStyleBase#contextLookup}.
       * @param style
       * @returns 
       * @protected
       */
      getContextLookup(style:yfiles.styles.TemplateStripeStyleBase):yfiles.graph.IContextLookup;
      /**
       * Configures the <code>style</code> and <code>stripe</code> parameters, calls {@link yfiles.styles.TemplateStripeStyleRenderer#configure} and returns <code>this</code>.
       * @param stripe The stripe to retrieve the {@link } for. The value will be stored in the {@link #stripe} property.
       * @param style The style to associate with the stripe. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateStripeStyleRenderer#createVisual
       * @see yfiles.styles.TemplateStripeStyleRenderer#updateVisual
       */
      getVisualCreator(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):yfiles.view.IVisualCreator;
      /**
       * Implementation of the {@link yfiles.graph.ILookup#lookup} method.
       * <p>
       * Delegates to the lookup returned by GetContextLookup(StripeControlStripeStyle), first. Last, it returns this class if it
       * implements the queried <code>type</code>.
       * </p>
       * @param type The type to query an instance for.
       * @returns The implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Updates the component correspondingly.
       * @param context The context for the creation.
       * @param oldVisual The currently rendered visual.
       * @returns The control that is used for rendering the stripe.
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.TemplateStripeStyleBase}
       */
      style:yfiles.styles.TemplateStripeStyleBase;
      /**
       * Gets or sets the currently configured stripe.
       * @protected
       * @type {yfiles.graph.IStripe}
       */
      stripe:yfiles.graph.IStripe;
      /**
       * Returns the {@link yfiles.graph.IStripe#layout} of the stripe this renderer is currently configured for.
       * <p>
       * Returns <code>Stripe.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.INodeStyle} for {@link yfiles.graph.ITable tables}.
     * <p>
     * This implementations stores the table instance and therefore, its instances cannot be shared among different table
     * instances.
     * </p>
     * @class
     * @implements {yfiles.styles.INodeStyle}
     * @implements {yfiles.lang.IPropertyObservable}
     */
    export interface TableNodeStyle extends Object,yfiles.styles.INodeStyle,yfiles.lang.IPropertyObservable{}
    export class TableNodeStyle {
      /**
       * Constructor with a custom renderer implementation that uses <code>table</code> as backing table model.
       * @param table The table instance that defines the tabular structure.
       * @param [renderer=null] Custom renderer instance for this style.
       */
      constructor(table:yfiles.graph.ITable,renderer?:yfiles.styles.TableNodeStyleRenderer);
      /**
       * Constructor with a custom renderer implementation that uses <code>table</code> as backing table model.
       * @param {Object} options The parameters to pass.
       * @param options.table The table instance that defines the tabular structure.
       * @param [options.renderer=null] Custom renderer instance for this style.
       * @param {yfiles.styles.INodeStyle} options.backgroundStyle The style that is used to draw the background of the table
       * <p>
       * This option sets the {@link yfiles.styles.TableNodeStyle#backgroundStyle} property on the created object.
       * </p>
       * @param {yfiles.styles.TableRenderingOrder} options.tableRenderingOrder The order in which rows and columns are rendered.
       * <p>
       * This option sets the {@link yfiles.styles.TableNodeStyle#tableRenderingOrder} property on the created object.
       * </p>
       */
      constructor(options:{table:yfiles.graph.ITable,renderer?:yfiles.styles.TableNodeStyleRenderer,backgroundStyle?:yfiles.styles.INodeStyle,tableRenderingOrder?:yfiles.styles.TableRenderingOrder});
      /**
       * Default constructor that creates an empty table.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.INodeStyle} options.backgroundStyle The style that is used to draw the background of the table
       * <p>
       * This option sets the {@link yfiles.styles.TableNodeStyle#backgroundStyle} property on the created object.
       * </p>
       * @param {yfiles.graph.ITable} options.table The {@link yfiles.graph.ITable} instance that defines the tabular structure.
       * <p>
       * This option sets the {@link yfiles.styles.TableNodeStyle#table} property on the created object.
       * </p>
       * @param {yfiles.styles.TableRenderingOrder} options.tableRenderingOrder The order in which rows and columns are rendered.
       * <p>
       * This option sets the {@link yfiles.styles.TableNodeStyle#tableRenderingOrder} property on the created object.
       * </p>
       */
      constructor(options?:{backgroundStyle?:yfiles.styles.INodeStyle,table?:yfiles.graph.ITable,tableRenderingOrder?:yfiles.styles.TableRenderingOrder});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Create a copy of the background style when this style instance is cloned
       * @param style The original background node style
       * @returns This implementation always returns the original <code>style</code> instance.
       * @protected
       */
      copyBackgroundStyle(style:yfiles.styles.INodeStyle):yfiles.styles.INodeStyle;
      /**
       * Create a copy of the table when this style instance is cloned
       * @param table The original table instance
       * @returns This implementation always tries to {@link #clone} clone the original <code>table</code> instance, if possible,
       * otherwise it returns <code>table</code> unchanged.
       * @protected
       */
      copyTable(table:yfiles.graph.ITable):yfiles.graph.ITable;
      /**
       * Raises the {@link yfiles.styles.TableNodeStyle#addPropertyChangedListener PropertyChanged} event.
       * <p>
       * Called when the value of the {@link yfiles.styles.TableNodeStyle#table} property changes.
       * </p>
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Gets or sets the order in which rows and columns are rendered.
       * @default yfiles.styles.TableRenderingOrder.COLUMNS_FIRST
       * @type {yfiles.styles.TableRenderingOrder}
       */
      tableRenderingOrder:yfiles.styles.TableRenderingOrder;
      /**
       * Gets or sets the style that is used to draw the background of the table
       * @default 'TableNodeStyle.defaultBackgroundStyle'
       * @type {yfiles.styles.INodeStyle}
       */
      backgroundStyle:yfiles.styles.INodeStyle;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when the value of the {@link yfiles.styles.TableNodeStyle#table} property changes.
       * @param listener The listener to add.
       * @see yfiles.styles.TableNodeStyle#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when the value of the {@link yfiles.styles.TableNodeStyle#table} property changes.
       * @param listener The listener to remove.
       * @see yfiles.styles.TableNodeStyle#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the {@link yfiles.graph.ITable} instance that defines the tabular structure.
       * @type {yfiles.graph.ITable}
       */
      table:yfiles.graph.ITable;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * Default renderer implementation that can be used for {@link yfiles.styles.TableNodeStyle} instances.
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface TableNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class TableNodeStyleRenderer {
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.TableNodeStyleRenderer#style} and {@link yfiles.styles.TableNodeStyleRenderer#node} properties have been populated by the {@link yfiles.styles.TableNodeStyleRenderer#getVisualCreator},
       * {@link yfiles.styles.TableNodeStyleRenderer#getBoundsProvider}, {@link yfiles.styles.TableNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.TableNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * 
       * @param context
       * @returns 
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Get the {@link yfiles.styles.INodeStyle background style} that is currently used by this renderer.
       * <p>
       * This implementation retrieves the {@link yfiles.styles.TableNodeStyle#backgroundStyle} from the owning style
       * </p>
       * @returns The {@link  background style} that is currently used by this renderer.
       * @protected
       */
      getBackgroundStyle():yfiles.styles.INodeStyle;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * 
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * 
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Gets the table object that should be used for rendering.
       * <p>
       * This implementation delegates to the Table provided by the {@link yfiles.styles.TableNodeStyle#table} property.
       * </p>
       * @returns The table that should be used for the rendering process.
       * @protected
       */
      getTable():yfiles.graph.ITable;
      /**
       * Get the {@link yfiles.styles.TableRenderingOrder} that is currently used by this renderer.
       * <p>
       * This implementation retrieves the {@link yfiles.styles.TableRenderingOrder} from the owning style
       * </p>
       * @returns The {@link } that is currently used by this renderer.
       * @protected
       */
      getTableRenderingOrder():yfiles.styles.TableRenderingOrder;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * 
       * @param node
       * @param style
       * @returns 
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * 
       * @param type
       * @returns 
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * 
       * @param context
       * @param oldVisual
       * @returns 
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.TableNodeStyle}
       */
      style:yfiles.styles.TableNodeStyle;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      static $class:yfiles.lang.Class;
    }
    /**
     * The context of a <code>TemplateBinding</code>. This interface exposes properties of a graph element onto which a <code>TemplateBinding</code> can
     * be bound.
     * @interface
     * @implements {yfiles.lang.IPropertyObservable}
     */
    export interface ITemplateStyleBindingContext extends Object,yfiles.lang.IPropertyObservable{
      /**
       * Gets the width of the visual.
       * @abstract
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of the visual.
       * @abstract
       * @type {number}
       */
      height:number;
      /**
       * Gets or sets the {@link yfiles.view.CanvasComponent} instance this instance is used in.
       * @abstract
       * @type {yfiles.view.CanvasComponent}
       */
      canvasComponent:yfiles.view.CanvasComponent;
      /**
       * Gets or sets a value indicating whether the item is currently selected.
       * <p>
       * This state is bound to the {@link yfiles.view.IGraphSelection} which is used in the {@link yfiles.styles.ITemplateStyleBindingContext#canvasComponent} this visual is being used in. Setting this
       * value will write through to the {@link yfiles.view.IGraphSelection}.
       * </p>
       * @abstract
       * @type {boolean}
       */
      itemSelected:boolean;
      /**
       * Gets or sets a value indicating whether the item is the currently focused item in the {@link yfiles.view.GraphComponent}.
       * <p>
       * This state is bound to the {@link yfiles.view.GraphComponent#currentItem} which is used in the {@link yfiles.styles.ITemplateStyleBindingContext#canvasComponent} this visual is being used in. Setting this value will write through to
       * the {@link yfiles.view.GraphComponent#currentItem}.
       * </p>
       * @abstract
       * @type {boolean}
       */
      itemFocused:boolean;
      /**
       * Gets or sets a value indicating whether the item is currently highlighted.
       * <p>
       * This state is bound to the {@link yfiles.view.HighlightIndicatorManager.<T>} which is used in the {@link yfiles.styles.ITemplateStyleBindingContext#canvasComponent} this visual is being used in. Setting this value will write through to
       * the {@link yfiles.view.HighlightIndicatorManager.<T>}'s {@link yfiles.view.HighlightIndicatorManager.<T>#selectionModel}
       * </p>
       * @abstract
       * @type {boolean}
       */
      itemHighlighted:boolean;
      /**
       * Gets or sets the item that is rendered by this instance.
       * @abstract
       * @type {yfiles.graph.IModelItem}
       */
      item:yfiles.graph.IModelItem;
      /**
       * Gets or sets the style tag that is associated with the style that created this visual.
       * <p>
       * This property can be used by implementers as a convenient property to pass data to the visual.
       * </p>
       * @abstract
       * @type {Object}
       */
      styleTag:Object;
    }
    var ITemplateStyleBindingContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Enhances the context of a <code>TemplateBinding</code> ({@link yfiles.styles.ITemplateStyleBindingContext}) for labels. This interface
     * exposes additional properties of a label onto which a <code>TemplateBinding</code> can be bound.
     * @interface
     * @implements {yfiles.lang.IPropertyObservable}
     */
    export interface ILabelTemplateStyleBindingContext extends Object,yfiles.lang.IPropertyObservable{
      /**
       * Gets or sets the label text property that is populated using the {@link yfiles.graph.ILabel}'s {@link yfiles.graph.ILabel#text} property.
       * @abstract
       * @type {string}
       */
      labelText:string;
      /**
       * Gets a value indicating whether this instance has been flipped so that it points upwards.
       * @abstract
       * @type {boolean}
       */
      isFlipped:boolean;
      /**
       * Gets a value indicating whether this instance is currently rendered upside down.
       * @abstract
       * @type {boolean}
       */
      isUpsideDown:boolean;
    }
    var ILabelTemplateStyleBindingContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Base class for label styles that use a SVG snippet as template for the visualization of {@link yfiles.graph.ILabel}s.
     * @see yfiles.styles.TemplateLabelStyle
     * @see yfiles.styles.StringTemplateLabelStyle
     * @class
     * @implements {yfiles.styles.ILabelStyle}
     * @implements {yfiles.lang.IPropertyObservable}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface TemplateLabelStyleBase extends Object,yfiles.styles.ILabelStyle,yfiles.lang.IPropertyObservable,yfiles.lang.ICloneable{}
    export class TemplateLabelStyleBase {
      /**
       * Constructs a new instance using the provided <code>renderer</code>.
       * @param renderer
       */
      constructor(renderer:yfiles.styles.TemplateLabelStyleRenderer);
      /**
       * Constructs a new instance of this style with default values.
       * @protected
       */
      constructor();
      /**
       * Performs a shallow clone of this instance, except for the value of {@link yfiles.styles.TemplateLabelStyleBase#styleTag} which is cloned, too, if it can be cloned.
       * @returns A clone of this instance.
       */
      clone():Object;
      /**
       * Factory method that is called by the {@link yfiles.styles.TemplateLabelStyleRenderer} to create the visual that will be used for
       * the display of the label.
       * @param context The context for which the visual should be created.
       * @param label The label that will be rendered.
       * @returns The readily styled visual.
       */
      createTemplate(context:yfiles.view.IRenderContext,label:yfiles.graph.ILabel):yfiles.view.SvgVisual;
      /**
       * Convenience method that determines the preferred size of the label if this style was applied.
       * @param context The context for which the size should be calculated.
       * @param label The label to determine the preferred size of.
       * @returns The preferred size.
       */
      getPreferredSize(context:yfiles.view.IRenderContext,label:yfiles.graph.ILabel):yfiles.geometry.Size;
      /**
       * Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
       * <p>
       * This method can be called to perform the template loading at a certain point in time, for example during application
       * startup. By default, a template is loaded lazily when the first visual using this template is built.
       * </p>
       * @static
       */
      static loadAllTemplates():void;
      /**
       * Raises the {@link yfiles.styles.TemplateLabelStyleBase#addPropertyChangedListener PropertyChanged} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to add.
       * @see yfiles.styles.TemplateLabelStyleBase#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to remove.
       * @see yfiles.styles.TemplateLabelStyleBase#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the tag that is associated with this style instance.
       * <p>
       * The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
       * templates and controls. The framework does not make any assumptions about the value of this property. It is there for
       * convenient use in the process of data binding the templates. This property supports {@link yfiles.lang.IPropertyObservable property change notification} and can
       * thus be data bound easily.
       * </p>
       * @default null
       * @type {Object}
       */
      styleTag:Object;
      /**
       * Gets or sets an implementation of {@link yfiles.graph.IContextLookup} that can be used to satisfy queries that are made to the
       * implementation which is returned by calls to {@link yfiles.styles.ILabelStyleRenderer#getContext} that are made on the style
       * renderer that is associated with this style instance.
       * <p>
       * This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
       * instance. The default value is {@link yfiles.graph.IContextLookup#EMPTY_CONTEXT_LOOKUP}
       * </p>
       * @default 'Lookups.EmptyContextLookup'
       * @type {yfiles.graph.IContextLookup}
       */
      contextLookup:yfiles.graph.IContextLookup;
      /**
       * Gets or sets the normalized outline of a label that will be displayed using the created visual.
       * <p>
       * The shape will automatically arranged and transformed to have the bounds of the label. This shape will be used to
       * satisfy requests to {@link yfiles.input.IMarqueeTestable#isInBox} and {@link yfiles.input.IHitTestable#isHit} and may be left <code>null</code> to
       * indicate default (rectangular) behavior.
       * </p>
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.geometry.GeneralPath}
       */
      normalizedOutline:yfiles.geometry.GeneralPath;
      /**
       * Gets or sets a value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if
       * otherwise it would be rendered upside-down.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @default true
       * @type {boolean}
       */
      autoFlip:boolean;
      /**
       * Gets or sets the preferred size of the labels that use this style.
       * @type {yfiles.geometry.Size}
       */
      preferredSize:yfiles.geometry.Size;
      /**
       * 
       * @type {yfiles.styles.ILabelStyleRenderer}
       */
      renderer:yfiles.styles.ILabelStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.ILabelStyle} implementation that uses an SVG snippet as template for the visualization of
     * {@link yfiles.graph.ILabel}s.
     * <p>
     * The {@link yfiles.styles.TemplateLabelStyle#styleResourceKey} is used to determine the template that will be applied to the component. The template must be specified in the DOM
     * in a script tag with <code>type='text/yfiles-template'</code>.
     * </p>
     * <p>
     * The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
     * value to the value of the property with the given name in the node's tag.
     * </p>
     * <p>
     * TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
     * well-defined properties of the label that are provided by the style. The available template binding properties are:
     * </p>
     * <ul>
     * <li>width: The width of the graph element</li>
     * <li>height: The height of the graph element</li>
     * <li>bounds: The bounding rectangle of the graph element</li>
     * <li>canvas: The {@link yfiles.view.GraphComponent} the element is rendered in</li>
     * <li>itemSelected: Whether the element is selected</li>
     * <li>itemFocused: Whether the element has the focus</li>
     * <li>itemHighlighted: Whether the element is highlighted</li>
     * <li>styleTag: The tag associated with the element's style instance</li>
     * <li>labelText: The label text property that is populated using the {@link yfiles.graph.ILabel}'s {@link yfiles.graph.ILabel#text} property</li>
     * <li>isFlipped: Whether the element has been flipped</li>
     * <li>isUpsideDown: Whether the element is currently rendered upside down</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplateLabelStyleBase}
     */
    export interface TemplateLabelStyle extends yfiles.styles.TemplateLabelStyleBase{}
    export class TemplateLabelStyle {
      /**
       * Creates a new instance using the given render template id and a custom renderer instance.
       * @param renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderTemplateId:string,renderer:yfiles.styles.TemplateLabelStyleRenderer);
      /**
       * Creates a new instance using the given render template id and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param options.renderer The custom renderer for this style instance.
       * @param {boolean} options.autoFlip A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a label that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.preferredSize The preferred size of the labels that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#preferredSize} property on the created object.
       * </p>
       */
      constructor(options:{renderTemplateId:string,renderer:yfiles.styles.TemplateLabelStyleRenderer,autoFlip?:boolean,normalizedOutline?:yfiles.geometry.GeneralPath,preferredSize?:yfiles.geometry.Size});
      /**
       * Creates a new instance using the given render template id.
       * @param renderTemplateId The id of the render template that will be used for the component.
       */
      constructor(renderTemplateId:string);
      /**
       * Creates a new instance using the given render template id.
       * @param {Object} options The parameters to pass.
       * @param options.renderTemplateId The id of the render template that will be used for the component.
       * @param {boolean} options.autoFlip A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a label that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.preferredSize The preferred size of the labels that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#preferredSize} property on the created object.
       * </p>
       */
      constructor(options:{renderTemplateId:string,autoFlip?:boolean,normalizedOutline?:yfiles.geometry.GeneralPath,preferredSize?:yfiles.geometry.Size});
      /**
       * Creates a new instance with no render template id and {@link yfiles.styles.TemplateLabelStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.autoFlip A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a label that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.preferredSize The preferred size of the labels that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#preferredSize} property on the created object.
       * </p>
       * @param {string} options.styleResourceKey The {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyle#styleResourceKey} property on the created object.
       * </p>
       */
      constructor(options?:{autoFlip?:boolean,normalizedOutline?:yfiles.geometry.GeneralPath,preferredSize?:yfiles.geometry.Size,styleResourceKey?:string});
      /**
       * Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {string}
       */
      styleResourceKey:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.ILabelStyle} that uses an SVG snippet as template for the visualization of {@link yfiles.graph.ILabel}s.
     * <p>
     * In contrast to {@link yfiles.styles.TemplateLabelStyle}, the SVG template used by this style must be encoded as string and set
     * to the {@link yfiles.styles.StringTemplateLabelStyle#svgContent} property.
     * </p>
     * <p>
     * It is vital that the {@link yfiles.styles.StringTemplateLabelStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
     * in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
     * namespace before they are added to the DOM.
     * </p>
     * <p>
     * The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
     * value to the value of the property with the given name in the node's tag.
     * </p>
     * <p>
     * TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
     * well-defined properties of the label that are provided by the style. The available template binding properties are:
     * </p>
     * <ul>
     * <li>width: The width of the graph element</li>
     * <li>height: The height of the graph element</li>
     * <li>bounds: The bounding rectangle of the graph element</li>
     * <li>canvas: The {@link yfiles.view.GraphComponent} the element is rendered in</li>
     * <li>itemSelected: Whether the element is selected</li>
     * <li>itemFocused: Whether the element has the focus</li>
     * <li>itemHighlighted: Whether the element is highlighted</li>
     * <li>styleTag: The tag associated with the element's style instance</li>
     * <li>labelText: The label text property that is populated using the {@link yfiles.graph.ILabel}'s {@link yfiles.graph.ILabel#text} property</li>
     * <li>isFlipped: Whether this instance has been flipped so that it points upwards</li>
     * <li>isUpsideDown: Whether this instance is currently rendered upside down</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplateLabelStyleBase}
     */
    export interface StringTemplateLabelStyle extends yfiles.styles.TemplateLabelStyleBase{}
    export class StringTemplateLabelStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderer:yfiles.styles.TemplateLabelStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer The custom renderer for this style instance.
       * @param {boolean} options.autoFlip A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a label that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.preferredSize The preferred size of the labels that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#preferredSize} property on the created object.
       * </p>
       * @param {string} options.svgContent The SVG content that is the template for the label visualization.
       * <p>
       * This option sets the {@link yfiles.styles.StringTemplateLabelStyle#svgContent} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.TemplateLabelStyleRenderer,autoFlip?:boolean,normalizedOutline?:yfiles.geometry.GeneralPath,preferredSize?:yfiles.geometry.Size,svgContent?:string});
      /**
       * Initializes a new instance of this class and sets the given string as its {@link yfiles.styles.StringTemplateLabelStyle#svgContent}.
       * @param svgContent The SVG snippet to use as template.
       */
      constructor(svgContent:string);
      /**
       * Initializes a new instance of this class and sets the given string as its {@link yfiles.styles.StringTemplateLabelStyle#svgContent}.
       * @param {Object} options The parameters to pass.
       * @param options.svgContent The SVG snippet to use as template.
       * @param {boolean} options.autoFlip A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a label that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.preferredSize The preferred size of the labels that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#preferredSize} property on the created object.
       * </p>
       */
      constructor(options:{svgContent:string,autoFlip?:boolean,normalizedOutline?:yfiles.geometry.GeneralPath,preferredSize?:yfiles.geometry.Size});
      /**
       * Initializes a new instance of this class.
       * @param {Object} [options=null] The parameters to pass.
       * @param {boolean} options.autoFlip A value indicating whether the rendering should automatically be flipped (rotated by 180 degrees) if otherwise it would be rendered upside-down.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a label that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.preferredSize The preferred size of the labels that use this style.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateLabelStyleBase#preferredSize} property on the created object.
       * </p>
       * @param {string} options.svgContent The SVG content that is the template for the label visualization.
       * <p>
       * This option sets the {@link yfiles.styles.StringTemplateLabelStyle#svgContent} property on the created object.
       * </p>
       */
      constructor(options?:{autoFlip?:boolean,normalizedOutline?:yfiles.geometry.GeneralPath,preferredSize?:yfiles.geometry.Size,svgContent?:string});
      /**
       * Gets or sets the SVG content that is the template for the label visualization.
       * @default null
       * @type {string}
       */
      svgContent:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.styles.ILabelStyleRenderer} interface that can render {@link yfiles.styles.TemplateLabelStyleBase}
     * instances.
     * @see yfiles.styles.TemplateLabelStyle
     * @see yfiles.styles.StringTemplateLabelStyle
     * @class
     * @implements {yfiles.styles.ILabelStyleRenderer}
     * @implements {yfiles.view.IVisualCreator}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.graph.ILookup}
     */
    export interface TemplateLabelStyleRenderer extends Object,yfiles.styles.ILabelStyleRenderer,yfiles.view.IVisualCreator,yfiles.view.IBoundsProvider,yfiles.input.IHitTestable,yfiles.input.IMarqueeTestable,yfiles.view.IVisibilityTestable,yfiles.graph.ILookup{}
    export class TemplateLabelStyleRenderer {
      /**
       * Stores the {@link yfiles.graph.ILabel#layout layout}.
       * <p>
       * Subclasses should override this method, call the super implementation and configure their painting entities.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * Creates the visual by delegating to {@link yfiles.styles.TemplateLabelStyleBase#createTemplate} and then initializing the
       * control with the context.
       * @param context The context for the creation.
       * @returns The visual that is used for rendering the label.
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the {@link yfiles.styles.TemplateLabelStyleRenderer#style} and {@link yfiles.styles.TemplateLabelStyleRenderer#label} properties, calls {@link yfiles.styles.TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to retrieve the bounds provider for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateLabelStyleRenderer#getBounds
       */
      getBoundsProvider(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the {@link yfiles.styles.TemplateLabelStyleRenderer#style} and {@link yfiles.styles.TemplateLabelStyleRenderer#label} properties, does <b>not</b> call {@link yfiles.styles.TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.TemplateLabelStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.TemplateLabelStyleRenderer#lookup} method call they should
       * call {@link yfiles.styles.TemplateLabelStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an
       * instance that depends on a correctly configured <code>this</code>.
       * </p>
       * @param label The label to query the context for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateLabelStyleRenderer#lookup
       */
      getContext(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.graph.ILookup;
      /**
       * Configures the {@link yfiles.styles.TemplateLabelStyleRenderer#style} and {@link yfiles.styles.TemplateLabelStyleRenderer#label} properties, calls {@link yfiles.styles.TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to query hit test with. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateLabelStyleRenderer#isHit
       */
      getHitTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IHitTestable;
      /**
       * Configures the {@link yfiles.styles.TemplateLabelStyleRenderer#style} and {@link yfiles.styles.TemplateLabelStyleRenderer#label} properties, calls {@link yfiles.styles.TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to query marquee intersection tests. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateLabelStyleRenderer#isInBox
       */
      getMarqueeTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.input.IMarqueeTestable;
      /**
       * Gets the normalized outline by delegating to {@link yfiles.styles.TemplateLabelStyleBase#normalizedOutline}.
       * @param style
       * @returns 
       * @protected
       */
      getNormalizedOutline(style:yfiles.styles.TemplateLabelStyleBase):yfiles.geometry.GeneralPath;
      /**
       * Calculates the preferred size for the given label and style.
       * @param label The label instance.
       * @param style The style instance to apply.
       * @returns The preferred size for the given label and style.
       * @see yfiles.styles.TemplateLabelStyleRenderer#getPreferredSizeCore
       */
      getPreferredSize(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.geometry.Size;
      /**
       * Gets the size of the preferred for the current configuration using the provided context.
       * @param context The context.
       * @returns 
       * @protected
       */
      getPreferredSizeCore(context:yfiles.view.IRenderContext):yfiles.geometry.Size;
      /**
       * Calculates the preferred size for the given label and style.
       * @param context
       * @param label The label instance.
       * @param style The style instance to apply.
       * @returns The preferred size for the given label and style.
       * @see yfiles.styles.TemplateLabelStyleRenderer#getPreferredSizeCore
       */
      getPreferredSizeWithContext(context:yfiles.view.IRenderContext,label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.geometry.Size;
      /**
       * Configures the {@link yfiles.styles.TemplateLabelStyleRenderer#style} and {@link yfiles.styles.TemplateLabelStyleRenderer#label} properties, does <b>not</b> call {@link yfiles.styles.TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.TemplateLabelStyleRenderer#configure}. If
       * the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.TemplateLabelStyleRenderer#isVisible}.
       * </p>
       * @param label The label to query visibility tests. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateLabelStyleRenderer#isVisible
       */
      getVisibilityTestable(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the {@link yfiles.styles.TemplateLabelStyleRenderer#style} and {@link yfiles.styles.TemplateLabelStyleRenderer#label} properties, calls {@link yfiles.styles.TemplateLabelStyleRenderer#configure} and returns <code>this</code>.
       * @param label The label to retrieve the {@link } for. The value will be stored in the {@link #label} property.
       * @param style The style to associate with the label. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateLabelStyleRenderer#createVisual
       * @see yfiles.styles.TemplateLabelStyleRenderer#updateVisual
       */
      getVisualCreator(label:yfiles.graph.ILabel,style:yfiles.styles.ILabelStyle):yfiles.view.IVisualCreator;
      /**
       * Delegates to the {@link yfiles.styles.TemplateLabelStyleBase}'s {@link yfiles.styles.TemplateLabelStyleBase#autoFlip} property.
       * @param style
       * @returns 
       * @protected
       */
      isAutoFlip(style:yfiles.styles.TemplateLabelStyleBase):boolean;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Uses the {@link yfiles.graph.ILabel#layout layout} to determine whether the clip intersects.
       * @param context
       * @param rectangle
       * @returns 
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Delegates to the {@link yfiles.styles.TemplateLabelStyleBase#contextLookup} of the {@link yfiles.styles.ILabelStyle}.
       * @param type The type to query an instance for.
       * @returns The implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Delegates to the {@link yfiles.styles.TemplateLabelStyleBase}'s {@link yfiles.styles.TemplateLabelStyleBase#contextLookup}.
       * @param style The style to look up the context for.
       * @param label The current label.
       * @param type The type to look up.
       * @returns The result from the context or <code>null</code>.
       * @protected
       */
      lookupContext(style:yfiles.styles.TemplateLabelStyleBase,label:yfiles.graph.ILabel,type:yfiles.lang.Class):Object;
      /**
       * Updates the visual correspondingly.
       * @param context The context for the creation.
       * @param oldVisual The currently rendered visual.
       * @returns The visual that is used for rendering the label.
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Holds the layout of the current label.
       * <p>
       * This value is set during {@link yfiles.styles.TemplateLabelStyleRenderer#configure} to the label's {@link yfiles.graph.ILabel#layout layout}
       * </p>
       * @protected
       * @type {yfiles.geometry.IOrientedRectangle}
       */
      layout:yfiles.geometry.IOrientedRectangle;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.TemplateLabelStyleBase}
       */
      style:yfiles.styles.TemplateLabelStyleBase;
      /**
       * Gets or sets the currently configured label.
       * @protected
       * @type {yfiles.graph.ILabel}
       */
      label:yfiles.graph.ILabel;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base class for node styles that use a SVG snippet as template for the visualization of {@link yfiles.graph.INode}s.
     * @see yfiles.styles.TemplateNodeStyle
     * @see yfiles.styles.StringTemplateNodeStyle
     * @class
     * @implements {yfiles.styles.INodeStyle}
     * @implements {yfiles.lang.IPropertyObservable}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface TemplateNodeStyleBase extends Object,yfiles.styles.INodeStyle,yfiles.lang.IPropertyObservable,yfiles.lang.ICloneable{}
    export class TemplateNodeStyleBase {
      /**
       * Constructs a new instance using the provided <code>renderer</code>.
       * @param renderer
       * @protected
       */
      constructor(renderer:yfiles.styles.TemplateNodeStyleRenderer);
      /**
       * Constructs a new instance of this style with default values.
       * @protected
       */
      constructor();
      /**
       * Performs a shallow clone of this instance, except for the value of {@link yfiles.styles.TemplateNodeStyleBase#styleTag} which is cloned, too, if it can be cloned.
       * @returns A clone of this instance.
       */
      clone():Object;
      /**
       * Factory method that is called by the {@link yfiles.styles.TemplateNodeStyleRenderer} to create the visual that will be used for
       * the display of the node.
       * @param context The context for which the visual should be created.
       * @param node The node that will be rendered.
       * @returns The readily styled visual.
       */
      createTemplate(context:yfiles.view.IRenderContext,node:yfiles.graph.INode):yfiles.view.SvgVisual;
      /**
       * Convenience method that determines the preferred size of the node if this style was applied.
       * @param context The context for which the size should be calculated. Can be obtained through
       * {@link #createRenderContext}.
       * @param node The node to determine the preferred size of. Can be a live node or an {@link }.
       * @returns The preferred size.
       * @see yfiles.view.CanvasComponent#createRenderContext
       */
      getPreferredSize(context:yfiles.view.IRenderContext,node:yfiles.graph.INode):yfiles.geometry.Size;
      /**
       * Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
       * <p>
       * This method can be called to perform the template loading at a certain point in time, for example during application
       * startup. By default, a template is loaded lazily when the first visual using this template is built.
       * </p>
       * @static
       */
      static loadAllTemplates():void;
      /**
       * Raises the {@link yfiles.styles.TemplateNodeStyleBase#addPropertyChangedListener PropertyChanged} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to add.
       * @see yfiles.styles.TemplateNodeStyleBase#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to remove.
       * @see yfiles.styles.TemplateNodeStyleBase#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the tag that is associated with this style instance.
       * <p>
       * The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
       * templates and controls. The framework does not make any assumptions about the value of this property. It is there for
       * convenient use in the process of data binding the templates. This property supports {@link yfiles.lang.IPropertyObservable property change notification} and can
       * thus be data bound easily.
       * </p>
       * @default null
       * @type {Object}
       */
      styleTag:Object;
      /**
       * Gets or sets an implementation of {@link yfiles.graph.IContextLookup} that can be used to satisfy queries that are made to the
       * implementation which is returned by calls to {@link yfiles.styles.INodeStyleRenderer#getContext} that are made on the style
       * renderer that is associated with this style instance.
       * <p>
       * This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
       * instance. E.g. it is possible to provide customized implementations of {@link yfiles.view.ISelectionIndicatorInstaller},
       * {@link yfiles.view.IHighlightIndicatorInstaller}, {@link yfiles.input.IHandleProvider}, {@link yfiles.input.INodeSizeConstraintProvider}, and
       * {@link yfiles.input.INodeInsetsProvider}. Simple style implementations can use {@link yfiles.graph.IContextLookup#EMPTY_CONTEXT_LOOKUP}
       * but may not use <code>null</code> for this property. The default value is {@link yfiles.graph.IContextLookup#EMPTY_CONTEXT_LOOKUP}
       * </p>
       * @default 'Lookups.EmptyContextLookup'
       * @type {yfiles.graph.IContextLookup}
       */
      contextLookup:yfiles.graph.IContextLookup;
      /**
       * Gets or sets the insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this
       * instance.
       * <p>
       * This is especially useful if this style is used for {@link yfiles.graph.IGraph#createGroupNode group nodes}, as they require insets for the calculation of the
       * bounds. The insets can be used to make sure that child nodes do not cover visual elements of the style inside the
       * insets. The default value is an {@link yfiles.geometry.Insets} insets with <code>5</code> for all insets.
       * </p>
       * @see yfiles.input.INodeInsetsProvider
       * @see yfiles.styles.INodeStyleRenderer#getContext
       * @default '5'
       * @type {yfiles.geometry.Insets}
       */
      insets:yfiles.geometry.Insets;
      /**
       * Gets or sets the minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should
       * yield.
       * <p>
       * The default value is {@link yfiles.geometry.Size#EMPTY}.
       * </p>
       * @see yfiles.input.INodeSizeConstraintProvider
       * @default 'Empty'
       * @type {yfiles.geometry.Size}
       */
      minimumSize:yfiles.geometry.Size;
      /**
       * Gets or sets the normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This shape will be used to satisfy requests to {@link yfiles.input.IMarqueeTestable#isInBox}, {@link yfiles.input.IHitTestable#isHit},
       * as well as for {@link yfiles.styles.IShapeGeometry} methods (which affect edge cropping at the node outline).
       * </p>
       * <p>
       * This property may be left at or set to <code>null</code> to indicate default (rectangular) behavior.
       * </p>
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.geometry.GeneralPath}
       */
      normalizedOutline:yfiles.geometry.GeneralPath;
      /**
       * 
       * @type {yfiles.styles.INodeStyleRenderer}
       */
      renderer:yfiles.styles.INodeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.INodeStyle} implementation that uses an SVG snippet as template for the visualization of
     * {@link yfiles.graph.INode}s.
     * <p>
     * The {@link yfiles.styles.TemplateNodeStyle#styleResourceKey} is used to determine the template that will be applied to the {@link yfiles.view.Visual}. The template must be specified in
     * the DOM in a script tag with <code>type='text/yfiles-template'</code>.
     * </p>
     * <p>
     * The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
     * value to the value of the property with the given name in the node's tag.
     * </p>
     * <p>
     * TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
     * well-defined properties of the node that are provided by the style. The available template binding properties are:
     * </p>
     * <ul>
     * <li>width: The width of the graph element</li>
     * <li>height: The height of the graph element</li>
     * <li>bounds: The bounding rectangle of the graph element</li>
     * <li>canvas: The {@link yfiles.view.GraphComponent} the element is rendered in</li>
     * <li>itemSelected: Whether the element is selected</li>
     * <li>itemFocused: Whether the element has the focus</li>
     * <li>itemHighlighted: Whether the element is highlighted</li>
     * <li>styleTag: The tag associated with the element's style instance</li>
     * </ul>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: OrgChart Demo, Demo.yFiles.Graph.OrgChart</li>
     * <li>Demo: Collapse Demo, Demo.yFiles.Graph.Collapse</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplateNodeStyleBase}
     */
    export interface TemplateNodeStyle extends yfiles.styles.TemplateNodeStyleBase{}
    export class TemplateNodeStyle {
      /**
       * Creates a new instance using the given render template id and a custom renderer instance.
       * @param renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param renderer The custom renderer for this instance.
       */
      constructor(renderTemplateId:string,renderer:yfiles.styles.TemplateNodeStyleRenderer);
      /**
       * Creates a new instance using the given render template id and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param options.renderer The custom renderer for this instance.
       * @param {yfiles.geometry.Insets} options.insets The insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#insets} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.minimumSize The minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should yield.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline} property on the created object.
       * </p>
       */
      constructor(options:{renderTemplateId:string,renderer:yfiles.styles.TemplateNodeStyleRenderer,insets?:yfiles.geometry.Insets,minimumSize?:yfiles.geometry.Size,normalizedOutline?:yfiles.geometry.GeneralPath});
      /**
       * Creates a new instance using the given render template id and {@link yfiles.styles.TemplateNodeStyleRenderer} as renderer.
       * @param renderTemplateId The id of the SVG snippet that will be used for the component.
       */
      constructor(renderTemplateId:string);
      /**
       * Creates a new instance using the given render template id and {@link yfiles.styles.TemplateNodeStyleRenderer} as renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param {yfiles.geometry.Insets} options.insets The insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#insets} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.minimumSize The minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should yield.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline} property on the created object.
       * </p>
       */
      constructor(options:{renderTemplateId:string,insets?:yfiles.geometry.Insets,minimumSize?:yfiles.geometry.Size,normalizedOutline?:yfiles.geometry.GeneralPath});
      /**
       * Creates a new instance with no render template id key and {@link yfiles.styles.TemplateNodeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} options.insets The insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#insets} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.minimumSize The minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should yield.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {string} options.styleResourceKey The {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyle#styleResourceKey} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets,minimumSize?:yfiles.geometry.Size,normalizedOutline?:yfiles.geometry.GeneralPath,styleResourceKey?:string});
      /**
       * Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {string}
       */
      styleResourceKey:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.INodeStyle} that uses an SVG snippet as template for the visualization of {@link yfiles.graph.INode}s.
     * <p>
     * In contrast to {@link yfiles.styles.TemplateNodeStyle}, the SVG template used by this style must be encoded as string and set to
     * the {@link yfiles.styles.StringTemplateNodeStyle#svgContent} property.
     * </p>
     * <p>
     * It is vital that the {@link yfiles.styles.StringTemplateNodeStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
     * in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
     * namespace before they are added to the DOM.
     * </p>
     * <p>
     * The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
     * value to the value of the property with the given name in the node's tag.
     * </p>
     * <p>
     * TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
     * well-defined properties of the node that are provided by the style. The available template binding properties are:
     * </p>
     * <ul>
     * <li>width: The width of the graph element</li>
     * <li>height: The height of the graph element</li>
     * <li>bounds: The bounding rectangle of the graph element</li>
     * <li>canvas: The {@link yfiles.view.GraphComponent} the element is rendered in</li>
     * <li>itemSelected: Whether the element is selected</li>
     * <li>itemFocused: Whether the element has the focus</li>
     * <li>itemHighlighted: Whether the element is highlighted</li>
     * <li>styleTag: The tag associated with the element's style instance</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplateNodeStyleBase}
     */
    export interface StringTemplateNodeStyle extends yfiles.styles.TemplateNodeStyleBase{}
    export class StringTemplateNodeStyle {
      /**
       * Creates a new instance using the given SVG template and a custom renderer instance.
       * @param svgContent The SVG snippet that will be used as a template for the component.
       * @param renderer The custom renderer for this instance.
       */
      constructor(svgContent:string,renderer:yfiles.styles.TemplateNodeStyleRenderer);
      /**
       * Creates a new instance using the given SVG template and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.svgContent The SVG snippet that will be used as a template for the component.
       * @param options.renderer The custom renderer for this instance.
       * @param {yfiles.geometry.Insets} options.insets The insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#insets} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.minimumSize The minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should yield.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline} property on the created object.
       * </p>
       */
      constructor(options:{svgContent:string,renderer:yfiles.styles.TemplateNodeStyleRenderer,insets?:yfiles.geometry.Insets,minimumSize?:yfiles.geometry.Size,normalizedOutline?:yfiles.geometry.GeneralPath});
      /**
       * Creates a new instance using the given SVG template and {@link yfiles.styles.TemplateNodeStyleRenderer} as renderer.
       * @param svgContent The SVG snippet that will be used as a template for the component.
       */
      constructor(svgContent:string);
      /**
       * Creates a new instance using the given SVG template and {@link yfiles.styles.TemplateNodeStyleRenderer} as renderer.
       * @param {Object} options The parameters to pass.
       * @param options.svgContent The SVG snippet that will be used as a template for the component.
       * @param {yfiles.geometry.Insets} options.insets The insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#insets} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.minimumSize The minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should yield.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline} property on the created object.
       * </p>
       */
      constructor(options:{svgContent:string,insets?:yfiles.geometry.Insets,minimumSize?:yfiles.geometry.Size,normalizedOutline?:yfiles.geometry.GeneralPath});
      /**
       * Creates a new instance with no SVG template and {@link yfiles.styles.TemplateNodeStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.Insets} options.insets The insets the {@link yfiles.input.INodeInsetsProvider} implementation for {@link yfiles.graph.INode} should yield for this instance.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#insets} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.minimumSize The minimum size an {@link yfiles.input.INodeSizeConstraintProvider} implementation for {@link yfiles.graph.INode} should yield.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#minimumSize} property on the created object.
       * </p>
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a node that will be displayed using the created visual.
       * <p>
       * This option sets the {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {string} options.svgContent The SVG content that is the template for the node visualization.
       * <p>
       * This option sets the {@link yfiles.styles.StringTemplateNodeStyle#svgContent} property on the created object.
       * </p>
       */
      constructor(options?:{insets?:yfiles.geometry.Insets,minimumSize?:yfiles.geometry.Size,normalizedOutline?:yfiles.geometry.GeneralPath,svgContent?:string});
      /**
       * Gets or sets the SVG content that is the template for the node visualization.
       * @default null
       * @type {string}
       */
      svgContent:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.styles.INodeStyleRenderer} interface that can render {@link yfiles.styles.TemplateNodeStyleBase}
     * instances.
     * @see yfiles.styles.TemplateNodeStyle
     * @see yfiles.styles.StringTemplateNodeStyle
     * @class
     * @implements {yfiles.styles.INodeStyleRenderer}
     * @implements {yfiles.styles.IShapeGeometry}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface TemplateNodeStyleRenderer extends Object,yfiles.styles.INodeStyleRenderer,yfiles.styles.IShapeGeometry,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class TemplateNodeStyleRenderer {
      /**
       * Prepares this instance for subsequent calls after the style and node have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.TemplateNodeStyleRenderer#style} and {@link yfiles.styles.TemplateNodeStyleRenderer#node} properties have been populated by the
       * {@link yfiles.styles.TemplateNodeStyleRenderer#getVisualCreator}, {@link yfiles.styles.TemplateNodeStyleRenderer#getBoundsProvider},
       * {@link yfiles.styles.TemplateNodeStyleRenderer#getHitTestable}, or {@link yfiles.styles.TemplateNodeStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * <p>
       * This is an empty implementation. Subclasses might have to override this method.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * Creates the visual by delegating to {@link yfiles.styles.TemplateNodeStyleBase#createTemplate} and then initializing the control
       * with the context.
       * @param context The context for the creation.
       * @returns The visual that is used for rendering the node.
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * Returns the layout bounds as an upper bound for the painting bound.
       * <p>
       * This implementation simply sets the {@link yfiles.styles.TemplateNodeStyleRenderer#layout}'s bounds to the scratch variable and returns.
       * </p>
       * @param context the context to calculate the bounds for
       * @returns the bounds or <code>null</code> to indicate an unbound area
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Provides a {@link yfiles.view.IBoundsProvider bounds calculation} implementation for the given style and node.
       * <p>
       * If {@link yfiles.styles.TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link yfiles.view.IBoundsProvider} interface than this instance will be returned,
       * otherwise the default (rectangular) behavior will be used.
       * </p>
       * @param node The node to query the provider for.
       * @param style The style whose provider is queried.
       * @returns An implementation that can calculate visual bounds.
       */
      getBoundsProvider(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.TemplateNodeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.TemplateNodeStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.TemplateNodeStyleRenderer#lookup} method call they should call
       * {@link yfiles.styles.TemplateNodeStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance that
       * depends on a correctly configured <code>this</code>.
       * </p>
       * @param node The node to query the context for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateNodeStyleRenderer#lookup
       */
      getContext(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.graph.ILookup;
      /**
       * Gets the context by delegating to {@link yfiles.styles.TemplateNodeStyleBase#contextLookup}.
       * @param style
       * @returns 
       * @protected
       */
      getContextLookup(style:yfiles.styles.TemplateNodeStyleBase):yfiles.graph.IContextLookup;
      /**
       * Provides a {@link yfiles.input.IHitTestable hit testing} implementation for the given style and node.
       * <p>
       * If {@link yfiles.styles.TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link yfiles.input.IHitTestable} interface than this instance will be returned, otherwise
       * the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} will be used to perform the hit testing.
       * </p>
       * @param node The node to query the tester for.
       * @param style The style whose tester is queried.
       * @returns An implementation that can perform the hit testing.
       * @see yfiles.styles.TemplateNodeStyleRenderer#isHit
       */
      getHitTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IHitTestable;
      /**
       * Gets the insets by delegating to {@link yfiles.styles.TemplateNodeStyleBase#insets}.
       * @param style
       * @returns 
       * @protected
       */
      getInsets(style:yfiles.styles.TemplateNodeStyleBase):yfiles.geometry.Insets;
      /**
       * Tries to use the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the intersection calculation, otherwise reverts to default (rectangular)
       * behavior.
       * @param inner
       * @param outer
       * @returns 
       */
      getIntersection(inner:yfiles.geometry.Point,outer:yfiles.geometry.Point):yfiles.geometry.Point;
      /**
       * Provides a {@link yfiles.input.IMarqueeTestable marquee testing} implementation for the given style and node.
       * <p>
       * If {@link yfiles.styles.TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link yfiles.input.IMarqueeTestable} interface than this instance will be returned,
       * otherwise the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} will be used to determine the marquee intersection test with the visual representation of
       * the node.
       * </p>
       * @param node The node to query the tester for.
       * @param style The style whose tester is queried.
       * @returns An implementation that can test for marquee intersections.
       * @see yfiles.styles.TemplateNodeStyleRenderer#isInBox
       */
      getMarqueeTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.input.IMarqueeTestable;
      /**
       * Gets the minimum size by delegating to {@link yfiles.styles.TemplateNodeStyleBase#minimumSize}.
       * @param style
       * @returns 
       * @protected
       */
      getMinimumSize(style:yfiles.styles.TemplateNodeStyleBase):yfiles.geometry.Size;
      /**
       * Gets the normalized outline by delegating to {@link yfiles.styles.TemplateNodeStyleBase#normalizedOutline}.
       * @param style
       * @returns 
       * @protected
       */
      getNormalizedOutline(style:yfiles.styles.TemplateNodeStyleBase):yfiles.geometry.GeneralPath;
      /**
       * Tries to use the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to create the outline shape path, otherwise reverts to default (rectangular) behavior.
       * @returns 
       */
      getOutline():yfiles.geometry.GeneralPath;
      /**
       * Calculates the preferred size for the given node and style in the provided context.
       * @param context The context for which the preferred size should be calculated.
       * @param node The node instance.
       * @param style The style instance to apply.
       * @returns The preferred size for the given label and style.
       * @see yfiles.styles.TemplateNodeStyleRenderer#getPreferredSizeCore
       */
      getPreferredSize(context:yfiles.view.IRenderContext,node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.geometry.Size;
      /**
       * Calculates the preferred size for the given node and style.
       * @param node The node instance.
       * @param style The style instance to apply.
       * @returns The preferred size for the given label and style.
       * @see yfiles.styles.TemplateNodeStyleRenderer#getPreferredSizeCore
       */
      getPreferredSize(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.geometry.Size;
      /**
       * Calculates the preferred size given the current state of the renderer for the given context.
       * @param context The render context for which the preferred size should be calculated.
       * @returns The size as suggested by this renderer.
       * @protected
       */
      getPreferredSizeCore(context:yfiles.view.IRenderContext):yfiles.geometry.Size;
      /**
       * Provides the geometry for the visual representation.
       * <p>
       * If {@link yfiles.styles.TemplateNodeStyleBase#contextLookup} provides an implementation of the {@link yfiles.styles.IShapeGeometry} interface than this instance will be returned,
       * otherwise the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} will be used to determine the outline of the visual representation of a node.
       * </p>
       * @param node The node to query the geometry for.
       * @param style The style for which the geometry is queried.
       * @returns An implementation that describes the outline geometry of the shape.
       * @see yfiles.styles.TemplateNodeStyleRenderer#isInside
       * @see yfiles.styles.TemplateNodeStyleRenderer#getOutline
       */
      getShapeGeometry(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.styles.IShapeGeometry;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, does <b>not</b> call {@link yfiles.styles.TemplateNodeStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.TemplateNodeStyleRenderer#configure}. If
       * the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.TemplateNodeStyleRenderer#isVisible}.
       * </p>
       * @param node The node to query visibility tests. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateNodeStyleRenderer#isVisible
       */
      getVisibilityTestable(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>node</code> parameters, calls {@link yfiles.styles.TemplateNodeStyleRenderer#configure} and returns <code>this</code>.
       * @param node The node to retrieve the {@link } for. The value will be stored in the {@link #node} property.
       * @param style The style to associate with the node. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplateNodeStyleRenderer#createVisual
       * @see yfiles.styles.TemplateNodeStyleRenderer#updateVisual
       */
      getVisualCreator(node:yfiles.graph.INode,style:yfiles.styles.INodeStyle):yfiles.view.IVisualCreator;
      /**
       * Tries to use the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the hit test analysis, otherwise reverts to default (rectangular) behavior.
       * @param context the context the hit test is performed in
       * @param location the hit point in world coordinates
       * @returns whether something has been hit
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * Tries to use the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the marquee intersection analysis, otherwise reverts to default (rectangular)
       * behavior.
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Tries to use the {@link yfiles.styles.TemplateNodeStyleRenderer#getNormalizedOutline outline shape} to perform the contains test, otherwise reverts to default (rectangular) behavior.
       * @param location
       * @returns 
       */
      isInside(location:yfiles.geometry.Point):boolean;
      /**
       * Determines visibility by checking for an intersection with the {@link yfiles.styles.TemplateNodeStyleRenderer#layout}.
       * @param context The context.
       * @param rectangle The current clip
       * @returns Whether the {@link #layout} is visible in the clip.
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implementation of the {@link yfiles.graph.ILookup#lookup} method.
       * <p>
       * Delegates to the lookup returned by {@link yfiles.styles.TemplateNodeStyleRenderer#getContextLookup}, first. Returns special {@link yfiles.input.INodeInsetsProvider}
       * or {@link yfiles.input.INodeSizeConstraintProvider} implementations if asked for. Last, it returns this class if it implements
       * the queried <code>type</code>.
       * </p>
       * @param type The type to query an instance for.
       * @returns The implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Updates the component correspondingly.
       * @param context The context for the creation.
       * @param oldVisual The currently rendered visual.
       * @returns The visual that is used for rendering the node.
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.TemplateNodeStyleBase}
       */
      style:yfiles.styles.TemplateNodeStyleBase;
      /**
       * Gets or sets the currently configured node.
       * @protected
       * @type {yfiles.graph.INode}
       */
      node:yfiles.graph.INode;
      /**
       * Returns the {@link yfiles.graph.INode#layout} of the node this renderer is currently configured for.
       * <p>
       * Returns <code>Node.Layout</code>.
       * </p>
       * @type {yfiles.geometry.IRectangle}
       */
      layout:yfiles.geometry.IRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base class for port styles that use a SVG snippet as template for the visualization of {@link yfiles.graph.IPort}s.
     * @see yfiles.styles.TemplatePortStyle
     * @see yfiles.styles.TemplatePortStyle
     * @class
     * @implements {yfiles.styles.IPortStyle}
     * @implements {yfiles.lang.IPropertyObservable}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface TemplatePortStyleBase extends Object,yfiles.styles.IPortStyle,yfiles.lang.IPropertyObservable,yfiles.lang.ICloneable{}
    export class TemplatePortStyleBase {
      /**
       * Constructs a new instance using the provided <code>renderer</code>.
       * @param renderer
       * @protected
       */
      constructor(renderer:yfiles.styles.TemplatePortStyleRenderer);
      /**
       * Constructs a new instance of this style with default values.
       * @protected
       */
      constructor();
      /**
       * Performs a shallow clone of this instance, except for the value of {@link yfiles.styles.TemplatePortStyleBase#styleTag} which is cloned, too, if it can be cloned.
       * @returns A clone of this instance.
       */
      clone():Object;
      /**
       * Factory method that is called by the {@link yfiles.styles.TemplatePortStyleRenderer} to create the visual that will be used for
       * the display of the port.
       * @param context The context for which the visual should be created.
       * @param port The port that will be rendered.
       * @returns The readily styled visual.
       */
      createTemplate(context:yfiles.view.IRenderContext,port:yfiles.graph.IPort):yfiles.view.SvgVisual;
      /**
       * Convenience method that determines the preferred {@link yfiles.styles.TemplatePortStyleBase#renderSize} of the port if this style was applied.
       * @param context The context for which the size should be calculated.
       * @param port The port to determine the preferred size of.
       * @returns The preferred size.
       */
      getPreferredSize(context:yfiles.view.IRenderContext,port:yfiles.graph.IPort):yfiles.geometry.Size;
      /**
       * Loads all template script elements that have not been parsed, yet, and creates the respective template cache objects.
       * <p>
       * This method can be called to perform the template loading at a certain point in time, for example during application
       * startup. By default, a template is loaded lazily when the first visual using this template is built.
       * </p>
       * @static
       */
      static loadAllTemplates():void;
      /**
       * Raises the {@link yfiles.styles.TemplatePortStyleBase#addPropertyChangedListener PropertyChanged} event.
       * @param evt The {@link } instance containing the event data.
       * @protected
       */
      onPropertyChanged(evt:yfiles.lang.PropertyChangedEventArgs):void;
      /**
       * Adds the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to add.
       * @see yfiles.styles.TemplatePortStyleBase#removePropertyChangedListener
       */
      addPropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Removes the given listener for the <code>PropertyChanged</code> event that occurs when a property value changes.
       * @param listener The listener to remove.
       * @see yfiles.styles.TemplatePortStyleBase#addPropertyChangedListener
       */
      removePropertyChangedListener(listener:(sender:Object,args:yfiles.lang.PropertyChangedEventArgs)=>void):void;
      /**
       * Gets or sets the tag that is associated with this style instance.
       * <p>
       * The various style renderer implementations can use this tag to make it accessible in the context of the instantiated
       * templates and controls. The framework does not make any assumptions about the value of this property. It is there for
       * convenient use in the process of data binding the templates. This property supports {@link yfiles.lang.IPropertyObservable property change notification} and can
       * thus be data bound easily.
       * </p>
       * @default null
       * @type {Object}
       */
      styleTag:Object;
      /**
       * Gets or sets an implementation of {@link yfiles.graph.IContextLookup} that can be used to satisfy queries that are made to the
       * implementation which is returned by calls to {@link yfiles.styles.IPortStyleRenderer#getContext} that are made on the style
       * renderer that is associated with this style instance.
       * <p>
       * This can be used to conveniently customize the lookup behavior of the style renderers that are associated with this
       * instance. E.g. it is possible to provide customized implementations of {@link yfiles.view.ISelectionIndicatorInstaller},
       * {@link yfiles.view.IHighlightIndicatorInstaller}, {@link yfiles.input.IHandleProvider}, {@link yfiles.input.INodeSizeConstraintProvider}, and
       * {@link yfiles.input.INodeInsetsProvider}. Simple style implementations can use {@link yfiles.graph.IContextLookup#EMPTY_CONTEXT_LOOKUP}
       * but may not use <code>null</code> for this property. The default value is {@link yfiles.graph.IContextLookup#EMPTY_CONTEXT_LOOKUP}
       * </p>
       * @default 'Lookups.EmptyContextLookup'
       * @type {yfiles.graph.IContextLookup}
       */
      contextLookup:yfiles.graph.IContextLookup;
      /**
       * Gets or sets the {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This size will be used to arrange the visual correctly. The default value is (5,5).
       * </p>
       * @default '5,5'
       * @type {yfiles.geometry.Size}
       */
      renderSize:yfiles.geometry.Size;
      /**
       * Gets or sets the normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * The shape will automatically be arranged and transformed to have the bounds of the port. This shape will be used to
       * satisfy requests to {@link yfiles.input.IMarqueeTestable#isInBox} and {@link yfiles.input.IHitTestable#isHit} and may be left <code>null</code> to
       * indicate default (rectangular) behavior.
       * </p>
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {yfiles.geometry.GeneralPath}
       */
      normalizedOutline:yfiles.geometry.GeneralPath;
      /**
       * 
       * @type {yfiles.styles.IPortStyleRenderer}
       */
      renderer:yfiles.styles.IPortStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.styles.IPortStyle} that uses an SVG snippet as template for the visualization of {@link yfiles.graph.IPort}s.
     * <p>
     * The {@link yfiles.styles.TemplatePortStyle#styleResourceKey} is used to determine the template that will be applied to the component. The template must be specified in the DOM
     * in a script tag with <code>type='text/yfiles-template'</code>.
     * </p>
     * <p>
     * Note that the {@link yfiles.styles.TemplatePortStyleBase#renderSize} should be specified to match the size of the visual's template.
     * </p>
     * <p>
     * The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
     * value to the value of the property with the given name in the node's tag.
     * </p>
     * <p>
     * TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
     * well-defined properties of the port that are provided by the style. The available template binding properties are:
     * </p>
     * <ul>
     * <li>width: The width of the graph element</li>
     * <li>height: The height of the graph element</li>
     * <li>bounds: The bounding rectangle of the graph element</li>
     * <li>canvas: The {@link yfiles.view.GraphComponent} the element is rendered in</li>
     * <li>itemSelected: Whether the element is selected</li>
     * <li>itemFocused: Whether the element has the focus</li>
     * <li>itemHighlighted: Whether the element is highlighted</li>
     * <li>styleTag: The tag associated with the element's style instance</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplatePortStyleBase}
     */
    export interface TemplatePortStyle extends yfiles.styles.TemplatePortStyleBase{}
    export class TemplatePortStyle {
      /**
       * Creates a new instance using the given render template id and a custom renderer instance.
       * @param renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderTemplateId:string,renderer:yfiles.styles.TemplatePortStyleRenderer);
      /**
       * Creates a new instance using the given render template id and a custom renderer instance.
       * @param {Object} options The parameters to pass.
       * @param options.renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param options.renderer The custom renderer for this style instance.
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#renderSize} property on the created object.
       * </p>
       */
      constructor(options:{renderTemplateId:string,renderer:yfiles.styles.TemplatePortStyleRenderer,normalizedOutline?:yfiles.geometry.GeneralPath,renderSize?:yfiles.geometry.Size});
      /**
       * Creates a new instance using the given resource key and a custom renderer instance.{@link yfiles.styles.TemplatePortStyleRenderer}
       * as renderer.
       * @param renderTemplateId The id of the SVG snippet that will be used for the component.
       */
      constructor(renderTemplateId:string);
      /**
       * Creates a new instance using the given resource key and a custom renderer instance.{@link yfiles.styles.TemplatePortStyleRenderer}
       * as renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderTemplateId The id of the SVG snippet that will be used for the component.
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#renderSize} property on the created object.
       * </p>
       */
      constructor(options:{renderTemplateId:string,normalizedOutline?:yfiles.geometry.GeneralPath,renderSize?:yfiles.geometry.Size});
      /**
       * Creates a new instance with no render template id and {@link yfiles.styles.TemplatePortStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#renderSize} property on the created object.
       * </p>
       * @param {string} options.styleResourceKey The {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyle#styleResourceKey} property on the created object.
       * </p>
       */
      constructor(options?:{normalizedOutline?:yfiles.geometry.GeneralPath,renderSize?:yfiles.geometry.Size,styleResourceKey?:string});
      /**
       * Gets or sets the {@link string} that will be used to look up the template that is applied to the component.
       * <p>
       * The default value is <code>null</code>.
       * </p>
       * @default null
       * @type {string}
       */
      styleResourceKey:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of the {@link yfiles.styles.IPortStyleRenderer} interface that can render {@link yfiles.styles.TemplatePortStyleBase}
     * instances.
     * @see yfiles.styles.TemplatePortStyle
     * @see yfiles.styles.StringTemplatePortStyle
     * @class
     * @implements {yfiles.styles.IPortStyleRenderer}
     * @implements {yfiles.view.IBoundsProvider}
     * @implements {yfiles.view.IVisibilityTestable}
     * @implements {yfiles.input.IMarqueeTestable}
     * @implements {yfiles.input.IHitTestable}
     * @implements {yfiles.graph.ILookup}
     * @implements {yfiles.view.IVisualCreator}
     */
    export interface TemplatePortStyleRenderer extends Object,yfiles.styles.IPortStyleRenderer,yfiles.view.IBoundsProvider,yfiles.view.IVisibilityTestable,yfiles.input.IMarqueeTestable,yfiles.input.IHitTestable,yfiles.graph.ILookup,yfiles.view.IVisualCreator{}
    export class TemplatePortStyleRenderer {
      constructor();
      /**
       * Prepares this instance for subsequent calls after the style and port have been initialized.
       * <p>
       * Upon invocation the {@link yfiles.styles.TemplatePortStyleRenderer#style} and {@link yfiles.styles.TemplatePortStyleRenderer#port} properties have been populated by the
       * {@link yfiles.styles.TemplatePortStyleRenderer#getVisualCreator}, {@link yfiles.styles.TemplatePortStyleRenderer#getBoundsProvider},
       * {@link yfiles.styles.TemplatePortStyleRenderer#getHitTestable}, or {@link yfiles.styles.TemplatePortStyleRenderer#getMarqueeTestable}
       * methods.
       * </p>
       * @protected
       */
      configure():void;
      /**
       * Creates the visual by delegating to {@link yfiles.styles.TemplatePortStyleBase#createTemplate} and then initializing the visual
       * with the context.
       * @param context The context for the creation.
       * @returns The visual that is used for rendering the port.
       */
      createVisual(context:yfiles.view.IRenderContext):yfiles.view.Visual;
      /**
       * 
       * @param context
       * @returns 
       */
      getBounds(context:yfiles.view.ICanvasContext):yfiles.geometry.Rect;
      /**
       * Configures the <code>style</code> and <code>port</code> parameters, calls {@link yfiles.styles.TemplatePortStyleRenderer#configure} and returns <code>this</code>.
       * @param port The port to retrieve the bounds provider for. The value will be stored in the {@link #port} property.
       * @param style The style to associate with the port. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplatePortStyleRenderer#getBounds
       */
      getBoundsProvider(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IBoundsProvider;
      /**
       * Configures the <code>style</code> and <code>port</code> parameters, does <b>not</b> call {@link yfiles.styles.TemplatePortStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * As this method may be called often it will not automatically call {@link yfiles.styles.TemplatePortStyleRenderer#configure},
       * instead subclasses should ensure that in the {@link yfiles.styles.TemplatePortStyleRenderer#lookup} method call they should call
       * {@link yfiles.styles.TemplatePortStyleRenderer#configure} only if needed, i.e. if they decide to return <code>this</code> or an instance that
       * depends on a correctly configured <code>this</code>.
       * </p>
       * @param port The port to query the context for. The value will be stored in the {@link #port} property.
       * @param style The style to associate with the port. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplatePortStyleRenderer#lookup
       */
      getContext(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.graph.ILookup;
      /**
       * Configures the <code>style</code> and <code>port</code> parameters, calls {@link yfiles.styles.TemplatePortStyleRenderer#configure} and returns <code>this</code>.
       * @param port The port to query hit test with. The value will be stored in the {@link #port} property.
       * @param style The style to associate with the port. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplatePortStyleRenderer#isHit
       */
      getHitTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.input.IHitTestable;
      /**
       * Configures the <code>style</code> and <code>port</code> parameters, calls {@link yfiles.styles.TemplatePortStyleRenderer#configure} and returns <code>this</code>.
       * @param port The port to query marquee intersection tests. The value will be stored in the {@link #port} property.
       * @param style The style to associate with the port. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplatePortStyleRenderer#isInBox
       */
      getMarqueeTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.input.IMarqueeTestable;
      /**
       * Gets the normalized outline by delegating to {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline}.
       * @param style
       * @returns 
       * @protected
       */
      getNormalizedOutline(style:yfiles.styles.TemplatePortStyleBase):yfiles.geometry.GeneralPath;
      /**
       * Calculates the preferred size for the given port and style.
       * @param context The render context which can be <code>null</code>.
       * @param port The port instance.
       * @param style The style instance to apply.
       * @returns The preferred size for the given port and style.
       * @see yfiles.styles.TemplatePortStyleRenderer#getPreferredSizeCore
       */
      getPreferredSize(context:yfiles.view.IRenderContext,port:yfiles.graph.IPort,style:yfiles.styles.TemplatePortStyleBase):yfiles.geometry.Size;
      /**
       * Gets the preferred size for the current configuration.
       * @param context
       * @returns The preferred size.
       * @protected
       */
      getPreferredSizeCore(context:yfiles.view.IRenderContext):yfiles.geometry.Size;
      /**
       * Gets the size of the rendering from the style.
       * @returns The {@link #renderSize}.
       * @protected
       */
      getRenderSize():yfiles.geometry.Size;
      /**
       * Configures the <code>style</code> and <code>port</code> parameters, does <b>not</b> call {@link yfiles.styles.TemplatePortStyleRenderer#configure} and returns
       * <code>this</code>.
       * <p>
       * Unlike most of the other methods this implementation does <b>not</b> call {@link yfiles.styles.TemplatePortStyleRenderer#configure}. If
       * the subclass implementation depends on this instance to be configured, it needs to call <code>Configure</code> in
       * {@link yfiles.styles.TemplatePortStyleRenderer#isVisible}.
       * </p>
       * @param port The port to query visibility tests. The value will be stored in the {@link #port} property.
       * @param style The style to associate with the port. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplatePortStyleRenderer#isVisible
       */
      getVisibilityTestable(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IVisibilityTestable;
      /**
       * Configures the <code>style</code> and <code>port</code> parameters, calls {@link yfiles.styles.TemplatePortStyleRenderer#configure} and returns <code>this</code>.
       * @param port The port to retrieve the {@link } for. The value will be stored in the {@link #port} property.
       * @param style The style to associate with the port. The value will be stored in the {@link #style} property.
       * @returns <code>this</code>
       * @see yfiles.styles.TemplatePortStyleRenderer#createVisual
       * @see yfiles.styles.TemplatePortStyleRenderer#updateVisual
       */
      getVisualCreator(port:yfiles.graph.IPort,style:yfiles.styles.IPortStyle):yfiles.view.IVisualCreator;
      /**
       * 
       * @param context
       * @param location
       * @returns 
       */
      isHit(context:yfiles.input.IInputModeContext,location:yfiles.geometry.Point):boolean;
      /**
       * 
       * @param context
       * @param rectangle
       * @returns 
       */
      isInBox(context:yfiles.input.IInputModeContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Calls {@link yfiles.styles.TemplatePortStyleRenderer#configure} and test the {@link yfiles.styles.TemplatePortStyleRenderer#port} using the {@link yfiles.styles.TemplatePortStyleRenderer#style} for a visible intersection with the
       * clip bounds.
       * <p>
       * This simple implementation uses {@link yfiles.styles.TemplatePortStyleRenderer#getBounds} to determine the visibility.
       * Subclasses may choose to override this behavior for improved performance but need to <b>make sure to call {@link yfiles.styles.TemplatePortStyleRenderer#configure}</b> if they depend on a configured
       * instance.
       * </p>
       * @param context the current canvas context
       * @param rectangle the clip bounds
       * @returns <code>true</code> if the port may be painted within the clip bounds.
       * @see yfiles.styles.TemplatePortStyleRenderer#getVisibilityTestable
       */
      isVisible(context:yfiles.view.ICanvasContext,rectangle:yfiles.geometry.Rect):boolean;
      /**
       * Implements the {@link yfiles.graph.ILookup} interface.
       * <p>
       * This method will be used by default if {@link yfiles.styles.TemplatePortStyleRenderer#getContext} has been queried for a lookup
       * implementation. Note that it cannot be assumed that {@link yfiles.styles.TemplatePortStyleRenderer#configure} has already been
       * invoked. However, normally {@link yfiles.styles.TemplatePortStyleRenderer#port} and {@link yfiles.styles.TemplatePortStyleRenderer#style} will be correctly configured if invoked directly after <code>GetContext</code>. Subclass
       * implementations should make sure to configure this instance before they return <code>this</code> as a result of a successful lookup.
       * This implementation will check if <code>type.IsInstanceOfType(this)</code> and will call {@link yfiles.styles.TemplatePortStyleRenderer#configure}
       * on success and return <code>this</code>.
       * </p>
       * @param type The type to query for.
       * @returns An implementation or <code>null</code>.
       */
      lookup(type:yfiles.lang.Class):Object;
      /**
       * Delegates to the {@link yfiles.styles.TemplatePortStyleBase}'s {@link yfiles.styles.TemplatePortStyleBase#contextLookup}.
       * @param style
       * @param port
       * @param type
       * @returns 
       * @protected
       */
      lookupContext(style:yfiles.styles.TemplatePortStyleBase,port:yfiles.graph.IPort,type:yfiles.lang.Class):Object;
      /**
       * Updates the component correspondingly.
       * @param context The context for the creation.
       * @param oldVisual The currently rendered visual.
       * @returns The visual that is used for rendering the port.
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual):yfiles.view.Visual;
      /**
       * Gets the center of the port.
       * @protected
       * @type {yfiles.geometry.Point}
       */
      center:yfiles.geometry.Point;
      /**
       * Gets or sets the currently configured style.
       * @protected
       * @type {yfiles.styles.TemplatePortStyleBase}
       */
      style:yfiles.styles.TemplatePortStyleBase;
      /**
       * Gets or sets the currently configured port.
       * @protected
       * @type {yfiles.graph.IPort}
       */
      port:yfiles.graph.IPort;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.styles.TemplatePortStyleBase} that uses an SVG snippet as template for the visualization of
     * {@link yfiles.graph.IPort}s.
     * <p>
     * In contrast to {@link yfiles.styles.TemplatePortStyle}, the SVG template used by this style must be encoded as string and set to
     * the {@link yfiles.styles.StringTemplatePortStyle#svgContent} property. Note that the RenderSize should be specified to match the size of the visual's template.
     * </p>
     * <p>
     * It is vital that the {@link yfiles.styles.StringTemplatePortStyle#svgContent SVG snippet} does not declare the SVG namespace itself. Its content is parsed in a dummy namespace
     * in order to avoid immediate validation of the result. The resulting SVG elements are automatically created in the SVG
     * namespace before they are added to the DOM.
     * </p>
     * <p>
     * The SVG template snippet can contain bindings in the form <code>attribute="{Binding propertyName}"</code> that bind the attribute
     * value to the value of the property with the given name in the node's tag.
     * </p>
     * <p>
     * TemplateBindings in the form <code>attribute="{TemplateBinding propertyName}"</code> bind the attribute value to a set of
     * well-defined properties of the port that are provided by the style. The available template binding properties are:
     * </p>
     * <ul>
     * <li>width: The width of the graph element</li>
     * <li>height: The height of the graph element</li>
     * <li>bounds: The bounding rectangle of the graph element</li>
     * <li>canvas: The {@link yfiles.view.GraphComponent} the element is rendered in</li>
     * <li>itemSelected: Whether the element is selected</li>
     * <li>itemFocused: Whether the element has the focus</li>
     * <li>itemHighlighted: Whether the element is highlighted</li>
     * <li>styleTag: The tag associated with the element's style instance</li>
     * </ul>
     * @class
     * @extends {yfiles.styles.TemplatePortStyleBase}
     */
    export interface StringTemplatePortStyle extends yfiles.styles.TemplatePortStyleBase{}
    export class StringTemplatePortStyle {
      /**
       * Creates a new instance using the provided renderer.
       * @param renderer The custom renderer for this style instance.
       */
      constructor(renderer:yfiles.styles.TemplatePortStyleRenderer);
      /**
       * Creates a new instance using the provided renderer.
       * @param {Object} options The parameters to pass.
       * @param options.renderer The custom renderer for this style instance.
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#renderSize} property on the created object.
       * </p>
       */
      constructor(options:{renderer:yfiles.styles.TemplatePortStyleRenderer,normalizedOutline?:yfiles.geometry.GeneralPath,renderSize?:yfiles.geometry.Size});
      /**
       * Initializes a new instance of this class and sets the given string as its {@link yfiles.styles.StringTemplatePortStyle#svgContent}.
       * @param svgContent The SVG snippet to use as template.
       */
      constructor(svgContent:string);
      /**
       * Initializes a new instance of this class and sets the given string as its {@link yfiles.styles.StringTemplatePortStyle#svgContent}.
       * @param {Object} options The parameters to pass.
       * @param options.svgContent The SVG snippet to use as template.
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#renderSize} property on the created object.
       * </p>
       */
      constructor(options:{svgContent:string,normalizedOutline?:yfiles.geometry.GeneralPath,renderSize?:yfiles.geometry.Size});
      /**
       * Creates a new instance using {@link yfiles.styles.TemplatePortStyleRenderer} as renderer.
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.geometry.GeneralPath} options.normalizedOutline The normalized outline of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#normalizedOutline} property on the created object.
       * </p>
       * @param {yfiles.geometry.Size} options.renderSize The {@link yfiles.geometry.Size size} of the of a port that will be displayed using the {@link yfiles.styles.TemplatePortStyleBase#createTemplate created visual}.
       * <p>
       * This option sets the {@link yfiles.styles.TemplatePortStyleBase#renderSize} property on the created object.
       * </p>
       * @param {string} options.svgContent The SVG content that is the template for the port visualization.
       * <p>
       * This option sets the {@link yfiles.styles.StringTemplatePortStyle#svgContent} property on the created object.
       * </p>
       */
      constructor(options?:{normalizedOutline?:yfiles.geometry.GeneralPath,renderSize?:yfiles.geometry.Size,svgContent?:string});
      /**
       * Gets or sets the SVG content that is the template for the port visualization.
       * @default null
       * @type {string}
       */
      svgContent:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * Style implementation for {@link yfiles.graph.IStripe} instances in an {@link yfiles.graph.ITable}.
     * <p>
     * It is up to the implementation to interpret the visual appearance of a stripe. The framework uses the associated,
     * possibly shared {@link yfiles.styles.IStripeStyle#renderer} to perform the actual rendering of this style for a given stripe. This interface extends the {@link yfiles.lang.ICloneable}
     * interface. This allows clients to obtain a persistent copy of the current state of this style. Immutable style
     * implementations may return themselves.
     * </p>
     * <p><b>Related demos:</b></p>
     * <ul>
     * <li>Demo: TableNodeStyle, Demo.yFiles.Graph.TableNodeStyle</li>
     * </ul>
     * @interface
     * @implements {yfiles.lang.ICloneable}
     */
    export interface IStripeStyle extends Object,yfiles.lang.ICloneable{
      /**
       * Gets the renderer implementation that can be queried for implementations that provide details about the visual
       * appearance and visual behavior for a given stripe and this style instance.
       * <p>
       * The idiom for retrieving, e.g. an {@link yfiles.view.IVisualCreator} implementation for a given stripe is:
       * </p>
       * @abstract
       * @type {yfiles.styles.IStripeStyleRenderer}
       */
      renderer:yfiles.styles.IStripeStyleRenderer;
    }
    var IStripeStyle:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface used by {@link yfiles.styles.IStripeStyle} implementations that can be used to query the actual {@link yfiles.view.IVisualCreator}
     * implementation for a stripe and an associated style.
     * <p>
     * Implementations of this interface should be designed to be sharable between multiple style instances and should
     * therefore not contain style specific state.
     * </p>
     * @see yfiles.styles.IStripeStyle
     * @see yfiles.graph.IStripe
     * @interface
     */
    export interface IStripeStyleRenderer extends Object{
      /**
       * Gets a temporary context instance that can be used to query additional information for the stripe's style.
       * <p>
       * Implementations may return {@link yfiles.graph.ILookup#EMPTY} if they don't support this, but may not return <code>null</code>.
       * </p>
       * @param stripe The stripe to provide a context instance for.
       * @param style The style to use for the context.
       * @returns An non-<code>null</code> lookup implementation.
       * @see yfiles.graph.ILookup#EMPTY
       * @see yfiles.graph.ILookup
       * @abstract
       */
      getContext(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):yfiles.graph.ILookup;
      /**
       * Gets an implementation of the {@link yfiles.view.IVisualCreator} interface that can handle the provided stripe and its
       * associated style.
       * <p>
       * This method may return a flyweight implementation, but never <code>null</code>.
       * </p>
       * @param stripe The nstripeode to provide an instance for
       * @param style The style to use for the creation of the visual
       * @returns An implementation that may be used to subsequently create or update the visual for the stripe. Clients should not cache
       * this instance and must always call this method immediately before using the value returned. This enables the use of the
       * flyweight design pattern for implementations. This method may not return <code>null</code> but should yield a {@link #INSTANCE void} implementation
       * instead.
       * @see yfiles.view.VoidVisualCreator#INSTANCE
       * @abstract
       */
      getVisualCreator(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):yfiles.view.IVisualCreator;
    }
    var IStripeStyleRenderer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A void implementation of a stripe style that does nothing and behaves like an invisible style.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidStripeStyle#INSTANCE} of this class
     * instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.graph.IStripe#style} property of an
     * {@link yfiles.graph.IStripe}.
     * </p>
     * @class
     * @implements {yfiles.styles.IStripeStyle}
     * @final
     */
    export interface VoidStripeStyle extends Object,yfiles.styles.IStripeStyle{}
    export class VoidStripeStyle {
      /**
       * Returns <code>this</code>.
       * @returns <code>this</code>
       */
      clone():Object;
      /**
       * The {@link yfiles.styles.VoidStripeStyle} singleton.
       * @const
       * @static
       * @type {yfiles.styles.VoidStripeStyle}
       */
      static INSTANCE:yfiles.styles.VoidStripeStyle;
      /**
       * Yields the {@link yfiles.styles.VoidStripeStyleRenderer#INSTANCE VoidStripeStyleRenderer instance}.
       * @see yfiles.styles.IStripeStyle#renderer
       * @type {yfiles.styles.IStripeStyleRenderer}
       */
      renderer:yfiles.styles.IStripeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A void implementation of a stripe style renderer that does nothing.
     * <p>
     * This class implements the singleton pattern. Use the shared static {@link yfiles.styles.VoidStripeStyleRenderer#INSTANCE} of
     * this class instead of <code>null</code> where <code>null</code> is not allowed. For example you cannot assign <code>null</code> to the {@link yfiles.styles.IStripeStyle#renderer} property of an
     * {@link yfiles.styles.IStripeStyle}.
     * </p>
     * @class
     * @implements {yfiles.styles.IStripeStyleRenderer}
     * @final
     */
    export interface VoidStripeStyleRenderer extends Object,yfiles.styles.IStripeStyleRenderer{}
    export class VoidStripeStyleRenderer {
      /**
       * Yields the {@link yfiles.graph.ILookup#EMPTY} that will not yield anything.
       * @param stripe
       * @param style
       * @returns 
       */
      getContext(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):yfiles.graph.ILookup;
      /**
       * Yields the {@link yfiles.view.VoidVisualCreator#INSTANCE} that will do nothing.
       * @param stripe
       * @param style
       * @returns 
       */
      getVisualCreator(stripe:yfiles.graph.IStripe,style:yfiles.styles.IStripeStyle):yfiles.view.IVisualCreator;
      /**
       * The {@link yfiles.styles.VoidStripeStyleRenderer} singleton.
       * @const
       * @static
       * @type {yfiles.styles.IStripeStyleRenderer}
       */
      static INSTANCE:yfiles.styles.IStripeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
    /**
     * A stripe style decorator that uses a node style instance to render the stripe.
     * @class
     * @implements {yfiles.styles.IStripeStyle}
     */
    export interface NodeStyleStripeStyleAdapter extends Object,yfiles.styles.IStripeStyle{}
    export class NodeStyleStripeStyleAdapter {
      /**
       * Creates a stripe style that uses the provided node style to render the stripe.
       * <p>
       * Note that the styles will be stored by reference, thus modifying the style will directly affect the rendering of this
       * instance.
       * </p>
       * @param nodeStyle The node style to use for rendering the stripe.
       */
      constructor(nodeStyle:yfiles.styles.INodeStyle);
      /**
       * Initializes a new instance of the {@link yfiles.styles.NodeStyleStripeStyleAdapter} class.
       * <p>
       * This constructor uses an rectangular {@link yfiles.styles.ShapeNodeStyle} with a transparent brush and a black pen for the {@link yfiles.styles.NodeStyleStripeStyleAdapter#nodeStyle}
       * property.
       * </p>
       * @param {Object} [options=null] The parameters to pass.
       * @param {yfiles.styles.INodeStyle} options.nodeStyle The {@link yfiles.styles.INodeStyle} that is used for rendering the stripe.
       * <p>
       * This option sets the {@link yfiles.styles.NodeStyleStripeStyleAdapter#nodeStyle} property on the created object.
       * </p>
       */
      constructor(options?:{nodeStyle?:yfiles.styles.INodeStyle});
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * 
       * @type {yfiles.styles.IStripeStyleRenderer}
       */
      renderer:yfiles.styles.IStripeStyleRenderer;
      /**
       * Gets the {@link yfiles.styles.INodeStyle} that is used for rendering the stripe.
       * @type {yfiles.styles.INodeStyle}
       */
      nodeStyle:yfiles.styles.INodeStyle;
      static $class:yfiles.lang.Class;
    }
    export enum TableRenderingOrder{
      /**
       * Draw columns, then rows
       */
      COLUMNS_FIRST,
      /**
       * Draw rows, then columns
       */
      ROWS_FIRST
    }
    /**
     * An abstract base class that makes it possible to easily implement a custom {@link yfiles.styles.IStripeStyle}.
     * <p>
     * The only method that needs to be implemented by subclasses is {@link yfiles.styles.StripeStyleBase#createVisual}, however to
     * improve rendering performance it is highly recommended to implement at least
     * {@link yfiles.styles.StripeStyleBase#updateVisual}, too.
     * </p>
     * <p>
     * This implementation differs from the straightforward {@link yfiles.styles.IStripeStyle} implementation in that there is no
     * visible separation between the style and its
     * {@link yfiles.styles.IStripeStyleRenderer}. Instead the renderer used by the base class is fixed and delegates all calls back to
     * the style instance.
     * </p>
     * @class
     * @implements {yfiles.styles.IStripeStyle}
     */
    export interface StripeStyleBase extends Object,yfiles.styles.IStripeStyle{}
    export class StripeStyleBase {
      /**
       * Initializes a new instance of the {@link yfiles.styles.StripeStyleBase} class.
       * @protected
       */
      constructor();
      /**
       * Creates a new object that is a copy of the current instance.
       * <p>
       * Immutable subclasses should consider returning <code>this</code>.
       * </p>
       * @returns A new object that is a copy of this instance using {@link #memberwiseClone}.
       */
      clone():Object;
      /**
       * Callback that creates the visual.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#createVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.StripeStyleBase#renderer}.
       * </p>
       * @param context
       * @param stripe The stripe to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.StripeStyleBase#updateVisual
       * @protected
       * @abstract
       */
      createVisual(context:yfiles.view.IRenderContext,stripe:yfiles.graph.IStripe):yfiles.view.Visual;
      /**
       * Performs the {@link yfiles.graph.ILookup#lookup} operation for the {@link yfiles.styles.IStripeStyleRenderer#getContext} that has been
       * queried from the {@link yfiles.styles.StripeStyleBase#renderer}.
       * <p>
       * This implementation yields <code>null</code> for everything but:
       * </p>
       * <ul>
       * <li>{@link yfiles.view.IVisualCreator}</li>
       * <li>{@link yfiles.graph.ILookup}</li>
       * </ul>
       * <p>
       * For these interfaces an implementation will be returned that delegates to the methods in this instance.
       * </p>
       * @param stripe The stripe to use for the context lookup.
       * @param type The type to query.
       * @returns An implementation of the <code>type</code> or <code>null</code>.
       * @protected
       */
      lookup(stripe:yfiles.graph.IStripe,type:yfiles.lang.Class):Object;
      /**
       * Callback that updates the visual previously created by {@link yfiles.styles.StripeStyleBase#createVisual}.
       * <p>
       * This method is called in response to a {@link yfiles.view.IVisualCreator#updateVisual} call to the instance that has been
       * queried from the {@link yfiles.styles.StripeStyleBase#renderer}. This implementation simply delegates to {@link yfiles.styles.StripeStyleBase#createVisual} so subclasses
       * should override to improve rendering performance.
       * </p>
       * @param context
       * @param oldVisual The visual that has been created in the call to {@link #createVisual}.
       * @param stripe The stripe to which this style instance is assigned.
       * @returns The visual as required by the {@link #createVisual} interface.
       * @see yfiles.styles.StripeStyleBase#createVisual
       * @protected
       */
      updateVisual(context:yfiles.view.IRenderContext,oldVisual:yfiles.view.Visual,stripe:yfiles.graph.IStripe):yfiles.view.Visual;
      /**
       * Gets the renderer implementation for this instance.
       * <p>
       * The private implementation will delegate all API calls back to this instance.
       * </p>
       * @type {yfiles.styles.IStripeStyleRenderer}
       */
      renderer:yfiles.styles.IStripeStyleRenderer;
      static $class:yfiles.lang.Class;
    }
  }export namespace binding{
    /**
     * This class is an adapter that populates a {@link yfiles.binding.GraphBuilderBase#graph} from custom business data.
     * <p>
     * This class can be used when the business data specifies a collection of nodes and the edges are implicitly defined as
     * collections of predecessors and successors of each node object.
     * </p>
     * <p>
     * The nodes in the graph are determined using the {@link yfiles.binding.GraphBuilderBase#nodesSource}, {@link yfiles.binding.AdjacentNodesGraphBuilder#predecessorsBinding} and {@link yfiles.binding.AdjacentNodesGraphBuilder#successorsBinding} properties. The predecessors and successors bindings define the
     * direct neighbors of a node. Edges are automatically added between a node and its predecessors and successors. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is
     * set, the {@link yfiles.binding.AdjacentNodesGraphBuilder#predecessorsBinding} and {@link yfiles.binding.AdjacentNodesGraphBuilder#successorsBinding} should yield these IDs instead of the node business objects.
     * </p>
     * <p>
     * The method {@link yfiles.binding.GraphBuilderBase#updateGraph} can be used to update the {@link yfiles.graph.IGraph} when the underlying
     * collection of the business objects changed. In this case, the nodes specified by <code>NodesSource</code> are always present. All
     * other nodes, that are the ones defined only in the predecessor or successor binding, persist only if they are reachable
     * by a sequence of neighborhood bindings that originates at a node specified by <code>NodesSource</code>. Graph components that are no
     * longer reachable by a sequence of neighborhood bindings from a node specified by <code>NodesSource</code> are completely removed from
     * the graph.
     * </p>
     * <p>
     * For assigning nodes to groups the {@link yfiles.binding.GraphBuilderBase#groupBinding} has to be defined. Group nodes can be nested using the {@link yfiles.binding.GraphBuilderBase#parentGroupBinding} property.
     * </p>
     * <p>
     * The visual appearance of the graph constructed from the business objects can be configured using the {@link yfiles.graph.IGraph#nodeDefaults}, the {@link yfiles.graph.IGraph#groupNodeDefaults}, and the {@link yfiles.graph.IGraph#edgeDefaults} of
     * the {@link yfiles.graph.IGraph}.
     * </p>
     * @class
     * @extends {yfiles.binding.GraphBuilderBase}
     */
    export interface AdjacentNodesGraphBuilder extends yfiles.binding.GraphBuilderBase{}
    export class AdjacentNodesGraphBuilder {
      /**
       * Initializes a new instance of this class using the given <code>graph</code>.
       * <p>
       * The <code>graph</code> will be {@link yfiles.graph.IGraph#clear cleared} and re-built from the data in {@link yfiles.binding.GraphBuilderBase#nodesSource} when {@link yfiles.binding.GraphBuilderBase#buildGraph} is called.
       * </p>
       * @param graph
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Creates a new edge connecting the given nodes.
       * <p>
       * This class calls this method to create all new edges, and customers may override it to customize edge creation.
       * </p>
       * @param graph The graph.
       * @param source The source node of the edge.
       * @param target The target node of the edge.
       * @returns The created edge.
       * @protected
       */
      createEdge(graph:yfiles.graph.IGraph,source:yfiles.graph.INode,target:yfiles.graph.INode):yfiles.graph.IEdge;
      /**
       * Updates an existing edge connecting the given nodes when {@link yfiles.binding.GraphBuilderBase#updateGraph} is called and the
       * edge should remain in the graph.
       * @param graph The graph.
       * @param edge The edge to update.
       * @param source The source node of the edge.
       * @param target The target node of the edge.
       * @protected
       */
      updateEdge(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,source:yfiles.graph.INode,target:yfiles.graph.INode):void;
      /**
       * Gets or sets the successors binding.
       * <p>
       * This binding maps a business object that represents a node to another business object <code>o</code> which specifies the successor
       * nodes of this node. Similar to {@link yfiles.binding.GraphBuilderBase#nodesSource}, the object <code>o</code> can either be an array, an {@link yfiles.collections.IEnumerable.<T>}, or a plain JavaScript object and should
       * contain business objects that represent nodes. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is set, this collection should contain the IDs of the objects that
       * specifies the nodes instead of the objects themselves.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @see yfiles.binding.AdjacentNodesGraphBuilder#predecessorsBinding
       * @type {Object}
       */
      successorsBinding:Object;
      /**
       * Gets or sets the predecessors binding.
       * <p>
       * This binding maps a business object that represents a node to another business object <code>o</code> which specifies the predecessor
       * nodes of this node. Similar to {@link yfiles.binding.GraphBuilderBase#nodesSource}, the object <code>o</code> can either be an array, an {@link yfiles.collections.IEnumerable.<T>}, or a plain JavaScript object and should
       * contain business objects that represent nodes. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is set, this collection should contain the IDs of the objects that
       * specifies the nodes instead of the objects themselves.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @see yfiles.binding.AdjacentNodesGraphBuilder#successorsBinding
       * @type {Object}
       */
      predecessorsBinding:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is an adapter that populates a {@link yfiles.binding.GraphBuilderBase#graph} from custom business data.
     * <p>
     * This class can be used when the business data specifies a collection of nodes, a collection of edges, and optionally, a
     * collection of groups. The business data can be set using the {@link yfiles.binding.GraphBuilderBase#nodesSource}, {@link yfiles.binding.GraphBuilderBase#groupsSource} and {@link yfiles.binding.GraphBuilder#edgesSource} properties.
     * </p>
     * <p>
     * To determine the source and target of an edge, the {@link yfiles.binding.GraphBuilder#sourceNodeBinding} and the {@link yfiles.binding.GraphBuilder#targetNodeBinding} need to be set for business objects that are the source for
     * edges. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is set, the {@link yfiles.binding.GraphBuilder#sourceNodeBinding} and {@link yfiles.binding.GraphBuilder#targetNodeBinding} should yield these IDs instead of the node business objects.
     * </p>
     * <p>
     * For assigning nodes to groups the {@link yfiles.binding.GraphBuilderBase#groupBinding} has to be defined. Group nodes can be nested using the {@link yfiles.binding.GraphBuilderBase#parentGroupBinding} property. Labels for the
     * edges can be defined by setting a suitable {@link yfiles.binding.GraphBuilder#edgeLabelBinding}.
     * </p>
     * <p>
     * The visual appearance of the graph constructed from the business objects can be configured using the {@link yfiles.graph.IGraph#nodeDefaults}, the {@link yfiles.graph.IGraph#groupNodeDefaults}, and the {@link yfiles.graph.IGraph#edgeDefaults} of
     * the {@link yfiles.graph.IGraph}.
     * </p>
     * @class
     * @extends {yfiles.binding.GraphBuilderBase}
     */
    export interface GraphBuilder extends yfiles.binding.GraphBuilderBase{}
    export class GraphBuilder {
      /**
       * Initializes a new instance of this class using the given <code>graph</code>.
       * <p>
       * The <code>graph</code> will be {@link yfiles.graph.IGraph#clear cleared} and re-built from the data in {@link yfiles.binding.GraphBuilderBase#nodesSource} and {@link yfiles.binding.GraphBuilder#edgesSource} when {@link yfiles.binding.GraphBuilderBase#buildGraph} is
       * called.
       * </p>
       * @param graph
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Creates a new edge connecting the given nodes and assigns the <code>businessObject</code>parameter to the edge's {@link yfiles.graph.ITagOwner#tag Tag} property.
       * <p>
       * This class calls this method to create all new edges, and customers may override it to customize edge creation.
       * </p>
       * @param graph The graph.
       * @param source The source node of the edge.
       * @param target The target node of the edge.
       * @param labelData The optional label data of the edge if an {@link #edgeLabelBinding} is specified.
       * @param businessObject The business data associated with the edge.
       * @returns The created edge.
       * @protected
       */
      createEdge(graph:yfiles.graph.IGraph,source:yfiles.graph.INode,target:yfiles.graph.INode,labelData:Object,businessObject:Object):yfiles.graph.IEdge;
      /**
       * Retrieves the associated edge in the {@link yfiles.binding.GraphBuilderBase#graph} for a business object from the {@link yfiles.binding.GraphBuilder#edgesSource}.
       * @param businessObject A business object from the {@link #edgesSource} to get the edge for.
       * @returns The edge associated with the business object or <code>null</code> for unknown objects.
       * @see yfiles.binding.GraphBuilderBase#getBusinessObject
       * @see yfiles.binding.GraphBuilderBase#getNode
       * @see yfiles.binding.GraphBuilderBase#getGroup
       */
      getEdge(businessObject:Object):yfiles.graph.IEdge;
      /**
       * Updates an existing edge when {@link yfiles.binding.GraphBuilderBase#updateGraph} is called and the edge should remain in the
       * graph.
       * @param graph The graph.
       * @param edge The edge to update.
       * @param labelData The optional label data of the edge if an {@link #edgeLabelBinding} is specified.
       * @param businessObject The business data associated with the edge.
       * @protected
       */
      updateEdge(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,labelData:Object,businessObject:Object):void;
      /**
       * Gets or sets a value indicating whether or not the source and target bindings must yield business objects that are
       * contained in {@link yfiles.binding.GraphBuilderBase#nodesSource}.
       * <p>
       * If this property is set to <code>false</code>, an edge is created if and only if its source and target bindings yield a business
       * object that is contained in {@link yfiles.binding.GraphBuilderBase#nodesSource}. Otherwise, if this property is <code>true</code>, an edge is created for each item of {@link yfiles.binding.GraphBuilder#edgesSource} and nodes are
       * created for the source and target as needed.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @see yfiles.binding.GraphBuilder#edgesSource
       * @type {boolean}
       */
      lazyNodeDefinition:boolean;
      /**
       * Get or set the business objects to be represented as edges of the {@link yfiles.binding.GraphBuilderBase#graph}.
       * <p>
       * This can either be an array of objects, a plain JavaScript object or an {@link yfiles.collections.IEnumerable.<Object>} containing the objects to be displayed as
       * edges.
       * </p>
       * <p>
       * In case the value of this property is a plain JavaScript object, the object's properties are enumerated with <code>Object.getOwnPropertyNames</code>
       * and used as the source for the graph edges.
       * </p>
       * @type {Object}
       */
      edgesSource:Object;
      /**
       * Gets or sets the mandatory source node binding.
       * <p>
       * This binding maps a business object <code>e</code> that represents an edge to another business object <code>n</code> which represents a node. This
       * node will become the source node of the edge. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is set, the object <code>n</code> should be the ID of the object that specifies
       * the node instead of the object itself.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @see yfiles.binding.GraphBuilder#targetNodeBinding
       * @see yfiles.binding.GraphBuilder#edgesSource
       * @type {Object}
       */
      sourceNodeBinding:Object;
      /**
       * Gets or sets the mandatory target node binding.
       * <p>
       * This binding maps a business object <code>e</code> that represents an edge to another business object <code>n</code> which represents a node. This
       * node will become the target node of the edge. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is set, the object <code>n</code> should be the ID of the object that specifies
       * the node instead of the object itself.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @see yfiles.binding.GraphBuilder#sourceNodeBinding
       * @see yfiles.binding.GraphBuilder#edgesSource
       * @type {Object}
       */
      targetNodeBinding:Object;
      /**
       * Gets or sets the edge label binding.
       * <p>
       * This binding maps a business object that represents an edge to an object that specifies the label data of that edge. By
       * default, the label data object must be convertible into a string. This can be customized by overriding {@link yfiles.binding.GraphBuilder#createEdge}
       * and {@link yfiles.binding.GraphBuilder#updateEdge}.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilder#edgesSource
       * @type {Object}
       */
      edgeLabelBinding:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base class for adapters that populate a {@link yfiles.binding.GraphBuilderBase#graph} from custom business data.
     * <p>
     * Customers should not create own implementations based on this class but use one of the ready-made adapters {@link yfiles.binding.GraphBuilder}
     * or {@link yfiles.binding.AdjacentNodesGraphBuilder}.
     * </p>
     * @class
     */
    export interface GraphBuilderBase extends Object{}
    export class GraphBuilderBase {
      /**
       * Initializes a new instance of this class using the given <code>graph</code>.
       * <p>
       * The <code>graph</code> will be {@link yfiles.graph.IGraph#clear cleared} and re-built from the data in {@link yfiles.binding.GraphBuilderBase#nodesSource} when {@link yfiles.binding.GraphBuilderBase#buildGraph} is called.
       * </p>
       * @param graph
       * @protected
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Populates the graph instance with items generated from the bound business data.
       * <p>
       * First, this method clears the graph, and then it adds new groups, nodes, and edges as specified by the business data and
       * bindings.
       * </p>
       * @returns The created graph.
       * @see yfiles.binding.GraphBuilderBase#updateGraph
       */
      buildGraph():yfiles.graph.IGraph;
      /**
       * Creates a new group node and assigns the <code>businessObject</code>parameter to the group node's {@link yfiles.graph.ITagOwner#tag Tag} property.
       * @param graph The graph.
       * @param businessObject The business data associated with the group node.
       * @returns The created node.
       * @protected
       */
      createGroupNode(graph:yfiles.graph.IGraph,businessObject:Object):yfiles.graph.INode;
      /**
       * Creates a new node with the specified parent and assigns the <code>businessObject</code>parameter to the node's {@link yfiles.graph.ITagOwner#tag Tag} property.
       * <p>
       * This class calls this method to create all new nodes, and customers may override it to customize node creation.
       * </p>
       * @param graph The graph.
       * @param parent The node's parent node.
       * @param location The location of the node.
       * @param labelData The optional label data of the node if an {@link #nodeLabelBinding} is specified.
       * @param businessObject The business data associated with the node.
       * @returns The created node.
       * @protected
       */
      createNode(graph:yfiles.graph.IGraph,parent:yfiles.graph.INode,location:yfiles.geometry.Point,labelData:Object,businessObject:Object):yfiles.graph.INode;
      /**
       * Retrieves the associated business object for a given {@link yfiles.graph.IModelItem graph item} from the {@link yfiles.binding.GraphBuilderBase#graph}.
       * @param item The item of the graph to get the business object for.
       * @returns The business object associated with the graph item.
       * @see yfiles.binding.GraphBuilderBase#getNode
       * @see yfiles.binding.GraphBuilderBase#getGroup
       */
      getBusinessObject(item:yfiles.graph.IModelItem):Object;
      /**
       * Retrieves the associated group node in the {@link yfiles.binding.GraphBuilderBase#graph} for a business object from the {@link yfiles.binding.GraphBuilderBase#groupsSource}.
       * @param businessObject A business object from the {@link #groupsSource} to get the group for.
       * @returns The group associated with the business object or <code>null</code> for unknown objects.
       * @see yfiles.binding.GraphBuilderBase#getBusinessObject
       * @see yfiles.binding.GraphBuilderBase#getNode
       */
      getGroup(businessObject:Object):yfiles.graph.INode;
      /**
       * Retrieves the associated node in the {@link yfiles.binding.GraphBuilderBase#graph} for a business object from the {@link yfiles.binding.GraphBuilderBase#nodesSource}.
       * @param businessObject A business object from the {@link #nodesSource} to get the node for.
       * @returns The node associated with the business object or <code>null</code> for unknown objects.
       * @see yfiles.binding.GraphBuilderBase#getBusinessObject
       * @see yfiles.binding.GraphBuilderBase#getGroup
       */
      getNode(businessObject:Object):yfiles.graph.INode;
      /**
       * Updates the graph instance to fit the bound business data.
       * <p>
       * This method can be used to update the graph to fit changed business data. In contrast to
       * {@link yfiles.binding.GraphBuilderBase#buildGraph}, it does not initially clear the graph. Instead, it keeps graph elements that
       * for business objects are still present in the data, creates new elements for new data, and removes obsolete elements.
       * </p>
       */
      updateGraph():void;
      /**
       * Updates an existing group node when {@link yfiles.binding.GraphBuilderBase#updateGraph} is called and the group should remain in
       * the graph.
       * <p>
       * This class calls this method to create all new groups, and customers may override it to customize group creation.
       * </p>
       * @param graph The graph.
       * @param groupNode The group node to update.
       * @param businessObject The business data associated with the group node.
       * @protected
       */
      updateGroupNode(graph:yfiles.graph.IGraph,groupNode:yfiles.graph.INode,businessObject:Object):void;
      /**
       * Updates an existing node when {@link yfiles.binding.GraphBuilderBase#updateGraph} is called and the node should remain in the
       * graph.
       * @param graph The graph.
       * @param node The node to update.
       * @param parent The node's parent node.
       * @param location The location of the node.
       * @param labelData The optional label data of the node if an {@link #nodeLabelBinding} is specified.
       * @param businessObject The business data associated with the node.
       * @protected
       */
      updateNode(graph:yfiles.graph.IGraph,node:yfiles.graph.INode,parent:yfiles.graph.INode,location:yfiles.geometry.Point,labelData:Object,businessObject:Object):void;
      /**
       * Returns the {@link yfiles.graph.IGraph graph} used by this class.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Gets or sets the business objects to be represented as nodes of the {@link yfiles.binding.GraphBuilderBase#graph}.
       * <p>
       * This can either be an array of objects, a plain JavaScript object or an {@link yfiles.collections.IEnumerable.<Object>} containing the objects to be displayed as
       * nodes.
       * </p>
       * <p>
       * In case the value of this property is a plain JavaScript object, the object's properties are enumerated with <code>Object.getOwnPropertyNames</code>
       * and used as the source for the graph nodes.
       * </p>
       * @type {Object}
       */
      nodesSource:Object;
      /**
       * Gets or sets the business objects to be represented as group nodes of the {@link yfiles.binding.GraphBuilderBase#graph}.
       * <p>
       * This can either be an array of objects, a plain JavaScript object or an {@link yfiles.collections.IEnumerable.<Object>} containing the objects to be displayed as group
       * nodes.
       * </p>
       * <p>
       * In case the value of this property is a plain JavaScript object, the object's properties are enumerated with <code>Object.getOwnPropertyNames</code>
       * and used as the source for the graph nodes.
       * </p>
       * @type {Object}
       */
      groupsSource:Object;
      /**
       * Gets or sets the node ID binding.
       * <p>
       * This binding maps a business object that represents a node to its ID. If this binding is set, node IDs can be used in a
       * business object that represents an edge to identify the source node and target node of the corresponding edge. The
       * binding should not be changed once the graph is built.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @type {Object}
       */
      nodeIdBinding:Object;
      /**
       * Gets or sets the node label binding.
       * <p>
       * This binding maps a business object that represents a node to an object that specifies the label data of that node.
       * </p>
       * <p>
       * By default, the label data object must be convertible into a string. This can be customized by overriding {@link yfiles.binding.GraphBuilderBase#createNode}
       * and {@link yfiles.binding.GraphBuilderBase#updateNode}.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @type {Object}
       */
      nodeLabelBinding:Object;
      /**
       * Gets or sets the group node ID binding.
       * <p>
       * This binding maps a business object that represents a group to its ID. If this binding is set, group IDs can be used in
       * a business object that represents a node to identify the parent group of the corresponding node. The binding should not
       * be changed once the graph is built.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. Functions will be called with both <code>this</code> and the first and only argument as the value to convert.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#groupsSource
       * @type {Object}
       */
      groupIdBinding:Object;
      /**
       * Gets or sets the group binding.
       * <p>
       * This binding maps a business object <code>o1</code> that represents a group to another object <code>o2</code> that specifies the parent group of
       * <code>o1</code>. If <code>o2</code> is contained in {@link yfiles.binding.GraphBuilderBase#groupsSource}, then the node for <code>o1</code> becomes a child node of the group for <code>o2</code>.
       * </p>
       * <p>
       * If a {@link yfiles.binding.GraphBuilderBase#groupIdBinding} is set, to object <code>o2</code> should be the ID of the object that specifies the group instead of the object itself.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @type {Object}
       */
      groupBinding:Object;
      /**
       * Gets or sets the parent group binding.
       * <p>
       * This binding maps a business object <code>o1</code> that represents a group to another object <code>o2</code> that specifies the parent group of
       * <code>o1</code>. If <code>o2</code> is contained in {@link yfiles.binding.GraphBuilderBase#groupsSource}, then the group for <code>o1</code> becomes a child node of the group for <code>o2</code>.
       * </p>
       * <p>
       * If a {@link yfiles.binding.GraphBuilderBase#groupIdBinding} is set, to object <code>o2</code> should be the ID of the object that specifies the group instead of the object itself.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @type {Object}
       */
      parentGroupBinding:Object;
      /**
       * Gets or sets the binding for determining a node's position on the x-axis.
       * <p>
       * This binding maps a business object that represents a node to a number that specifies the x-coordinate of that node.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @type {Object}
       */
      locationXBinding:Object;
      /**
       * Gets or sets the binding for determining a node's position on the y-axis.
       * <p>
       * This binding maps a business object that represents a node to a number that specifies the y-coordinate of that node.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @type {Object}
       */
      locationYBinding:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is an adapter which populates a {@link yfiles.binding.TreeBuilder#graph} from hierarchical business data.
     * <p>
     * The nodes in the tree are determined using the {@link yfiles.binding.TreeBuilder#nodesSource} and {@link yfiles.binding.TreeBuilder#childBinding} properties. Optionally, a collection of group nodes can be
     * specified with {@link yfiles.binding.TreeBuilder#groupsSource}.
     * </p>
     * <p>
     * Graph items will be visualized using the styles defined in the {@link yfiles.graph.IGraph#nodeDefaults}, {@link yfiles.graph.IGraph#edgeDefaults} and {@link yfiles.graph.IGraph#groupNodeDefaults} property of the {@link yfiles.binding.TreeBuilder#graph}.
     * </p>
     * <p>
     * The created {@link yfiles.binding.TreeBuilder#graph} is a directed forest, a set of directed trees. Each directed tree has exactly one root node with no
     * incoming edges. For every other node v in a directed tree there is exactly one directed path from the root node of the
     * directed tree to node v.
     * </p>
     * <p>
     * To assign nodes to groups the {@link yfiles.binding.GraphBuilderBase#groupBinding} has to be defined. Group nodes can be nested using the {@link yfiles.binding.GraphBuilderBase#parentGroupBinding} property. Labels for the edges
     * can be defined by setting a suitable {@link yfiles.binding.TreeBuilder#edgeLabelBinding}, which is resolved on the target (child) node.
     * </p>
     * <p>
     * This class can be used to {@link yfiles.binding.TreeBuilder#updateGraph update} the {@link yfiles.binding.TreeBuilder#graph} when the underlying collection of the business objects ({@link yfiles.binding.TreeBuilder#nodesSource} and {@link yfiles.binding.TreeBuilder#childBinding}) changes.
     * </p>
     * @see yfiles.binding.TreeBuilder#nodesSource
     * @see yfiles.binding.TreeBuilder#childBinding
     * @class
     */
    export interface TreeBuilder extends Object{}
    export class TreeBuilder {
      /**
       * Creates a new empty instance using the given <code>graph</code>.
       * <p>
       * The <code>graph</code> will be {@link yfiles.graph.IGraph#clear cleared} and re-built from the data in {@link yfiles.binding.TreeBuilder#nodesSource} when {@link yfiles.binding.TreeBuilder#buildGraph} is called. The {@link yfiles.binding.TreeBuilder#childBinding}
       * needs to be set in order to actually create a tree.
       * </p>
       * @param graph
       */
      constructor(graph:yfiles.graph.IGraph);
      /**
       * Populates the graph instance with items generated from the bound business data.
       * <p>
       * First, this method clears the graph, and then it adds new groups, nodes, and edges as specified by the business data and
       * bindings.
       * </p>
       * @returns The created graph.
       * @see yfiles.binding.TreeBuilder#updateGraph
       */
      buildGraph():yfiles.graph.IGraph;
      /**
       * Creates an edge between the source and target node.
       * @param graph The graph.
       * @param source The source node of the edge.
       * @param target The target node of the edge.
       * @param labelData The optional label data of the edge if an {@link #edgeLabelBinding} is specified.
       * @returns The created edge.
       * @protected
       */
      createEdge(graph:yfiles.graph.IGraph,source:yfiles.graph.INode,target:yfiles.graph.INode,labelData:Object):yfiles.graph.IEdge;
      /**
       * Creates a new group node and assigns the <code>businessObject</code>parameter to the group node's {@link yfiles.graph.ITagOwner#tag Tag} property.
       * @param graph The graph.
       * @param businessObject The business data associated with the group node.
       * @returns The created node.
       * @protected
       */
      createGroupNode(graph:yfiles.graph.IGraph,businessObject:Object):yfiles.graph.INode;
      /**
       * Creates a node with the specified parent and assigns the <code>businessObject</code> parameter to the node's {@link yfiles.graph.ITagOwner#tag Tag} property.
       * @param graph The graph.
       * @param parent The node's parent node.
       * @param location The location of the node.
       * @param labelData The optional label data of the node if an {@link #nodeLabelBinding} is specified.
       * @param businessObject The business data associated with the node.
       * @returns The created node.
       * @protected
       */
      createNode(graph:yfiles.graph.IGraph,parent:yfiles.graph.INode,location:yfiles.geometry.Point,labelData:Object,businessObject:Object):yfiles.graph.INode;
      /**
       * Retrieves the associated business object for a given {@link yfiles.graph.INode node} from the {@link yfiles.binding.TreeBuilder#graph}.
       * @param item The node of the graph to get the business object for.
       * @returns The business object associated with the graph element.
       * @see yfiles.binding.TreeBuilder#getNode
       */
      getBusinessObject(item:yfiles.graph.IModelItem):Object;
      /**
       * Retrieves the associated node in the {@link yfiles.binding.TreeBuilder#graph} for a business object from the {@link yfiles.binding.TreeBuilder#nodesSource}.
       * @param businessObject A business object from the {@link #nodesSource} to get the node for.
       * @returns The node associated with the business object or <code>null</code> for unknown objects.
       * @see yfiles.binding.TreeBuilder#getBusinessObject
       */
      getNode(businessObject:Object):yfiles.graph.INode;
      /**
       * Updates an edge with the given data.
       * @param graph The graph.
       * @param edge The edge to update.
       * @param source The edge's source node.
       * @param target The edge's target node.
       * @param labelData The business data associated with the edge.
       * @protected
       */
      updateEdge(graph:yfiles.graph.IGraph,edge:yfiles.graph.IEdge,source:yfiles.graph.INode,target:yfiles.graph.INode,labelData:Object):void;
      /**
       * Updates the graph instance to fit the bound business data.
       * <p>
       * This method can be used to update the graph to fit changed business data. In contrast to
       * {@link yfiles.binding.TreeBuilder#buildGraph}, it does not initially clear the graph. Instead, it keeps graph elements that for
       * business objects are still present in the data, creates new elements for new data, and removes obsolete elements.
       * </p>
       */
      updateGraph():void;
      /**
       * Updates an existing group node when {@link yfiles.binding.TreeBuilder#updateGraph} is called and the group should remain in the
       * graph.
       * <p>
       * This class calls this method to create all new groups, and customers may override it to customize group creation.
       * </p>
       * @param graph The graph.
       * @param groupNode The group node to update.
       * @param businessObject The business data associated with the group node.
       * @protected
       */
      updateGroupNode(graph:yfiles.graph.IGraph,groupNode:yfiles.graph.INode,businessObject:Object):void;
      /**
       * Updates a node with the given data.
       * @param graph The graph.
       * @param node The node to update.
       * @param parent The node's parent node.
       * @param location The location of the node.
       * @param labelData The optional label data of the edge if an {@link #nodeLabelBinding} is specified.
       * @param businessObject The business data associated with the node.
       * @protected
       */
      updateNode(graph:yfiles.graph.IGraph,node:yfiles.graph.INode,parent:yfiles.graph.INode,location:yfiles.geometry.Point,labelData:Object,businessObject:Object):void;
      /**
       * Returns the {@link yfiles.graph.IGraph graph} used by this instance.
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      /**
       * Gets or sets the business objects to be represented as nodes.
       * <p>
       * This can either be an array of objects, a plain JavaScript object, or an {@link yfiles.collections.IEnumerable.<Object>} containing the objects to be displayed as
       * nodes
       * </p>
       * <p>
       * Note that it is not necessary to include all nodes in this property, if they can be reached via the
       * {@link yfiles.binding.TreeBuilder#childBinding}. In this case it suffices to include all root nodes.
       * </p>
       * <p>
       * If the nodes source contains more than just the root nodes, some node sources are visited more than once during the
       * construction of the
       * {@link yfiles.binding.TreeBuilder#graph}. In order to avoid duplicate nodes, the tree source keeps track of node sources for which nodes have already been
       * constructed.
       * </p>
       * <p>
       * In case the value of this property is a plain JavaScript object, the object's properties are enumerated with <code>Object.getOwnPropertyNames</code>
       * and used as the source for the tree nodes.
       * </p>
       * @type {Object}
       */
      nodesSource:Object;
      /**
       * Gets or sets the business objects to be represented as group nodes of the {@link yfiles.binding.TreeBuilder#graph}.
       * <p>
       * This can either be an array of objects, a plain JavaScript object or an {@link yfiles.collections.IEnumerable.<Object>} containing the objects to be displayed as group
       * nodes.
       * </p>
       * <p>
       * In case the value of this property is a plain JavaScript object, the object's properties are enumerated with <code>Object.getOwnPropertyNames</code>
       * and used as the source for the graph nodes.
       * </p>
       * @type {Object}
       */
      groupsSource:Object;
      /**
       * Gets or sets the binding for the children of a node.
       * <p>
       * This binding maps a business object that represents a node to another business object <code>o</code> which specifies the children of
       * this node. Similar to {@link yfiles.binding.GraphBuilderBase#nodesSource}, the object <code>o</code> can either be an array, an {@link yfiles.collections.IEnumerable.<Object>}, or a plain JavaScript object and should contain
       * business objects that represent nodes. If a {@link yfiles.binding.GraphBuilderBase#nodeIdBinding} is set, these collection should contain the IDs of the objects that
       * specifies the nodes instead of the objects themselves.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.GraphBuilderBase#nodesSource
       * @type {Object}
       */
      childBinding:Object;
      /**
       * Gets or sets the node ID binding.
       * <p>
       * This binding maps a business object that represents a node to its ID. If this binding is set, node IDs can be used in a
       * business object that represents an edge to identify the source node and target node of the corresponding edge. The
       * binding should not be changed once the graph is built.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.TreeBuilder#nodesSource
       * @type {Object}
       */
      idBinding:Object;
      /**
       * Gets or sets the edge label binding.
       * <p>
       * This binding maps a business object that represents a node to an object that specifies the label data of the incoming
       * edge of that node. By default, the label data object must be convertible into a string. This can be customized by
       * overriding {@link yfiles.binding.TreeBuilder#createEdge} and {@link yfiles.binding.TreeBuilder#updateEdge}.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @type {Object}
       */
      edgeLabelBinding:Object;
      /**
       * Gets or sets the node label binding.
       * <p>
       * This binding maps a business object that represents a node to an object that specifies the label data of that node.
       * </p>
       * <p>
       * By default, the label data object must be convertible into a string. This can be customized by overriding {@link yfiles.binding.TreeBuilder#createNode}
       * and {@link yfiles.binding.TreeBuilder#updateNode}.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.TreeBuilder#nodesSource
       * @type {Object}
       */
      nodeLabelBinding:Object;
      /**
       * Gets or sets the binding for determining a node's position on the x-axis.
       * <p>
       * This binding maps a business object that represents a node to a number that specifies the x-coordinate of that node.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.TreeBuilder#nodesSource
       * @type {Object}
       */
      locationXBinding:Object;
      /**
       * Gets or sets the binding for determining a node's position on the y-axis.
       * <p>
       * This binding maps a business object that represents a node to a number that specifies the y-coordinate of that node.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.TreeBuilder#nodesSource
       * @type {Object}
       */
      locationYBinding:Object;
      /**
       * Gets or sets the group node ID binding.
       * <p>
       * This binding maps a business object that represents a group to its ID. If this binding is set, group IDs can be used in
       * a business object that represents a node to identify the parent group of the corresponding node. The binding should not
       * be changed once the graph is built.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.TreeBuilder#groupsSource
       * @type {Object}
       */
      groupIdBinding:Object;
      /**
       * Gets or sets the group binding.
       * <p>
       * This binding maps a business object <code>o1</code> that represents a group to another object <code>o2</code> that specifies the parent group of
       * <code>o1</code>. If <code>o2</code> is contained in {@link yfiles.binding.TreeBuilder#groupsSource}, then the node for <code>o1</code> becomes a child node of the group for <code>o2</code>.
       * </p>
       * <p>
       * If a {@link yfiles.binding.TreeBuilder#groupIdBinding} is set, to object <code>o2</code> should be the ID of the object that specifies the group instead of the object itself.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @see yfiles.binding.TreeBuilder#nodesSource
       * @type {Object}
       */
      groupBinding:Object;
      /**
       * Gets or sets the parent group binding.
       * <p>
       * This binding maps a business object <code>o1</code> that represents a group to another object <code>o2</code> that specifies the parent group of
       * <code>o1</code>. If <code>o2</code> is contained in {@link yfiles.binding.TreeBuilder#groupsSource}, then the group for <code>o1</code> becomes a child node of the group for <code>o2</code>.
       * </p>
       * <p>
       * If a {@link yfiles.binding.TreeBuilder#groupIdBinding} is set, to object <code>o2</code> should be the ID of the object that specifies the group instead of the object itself.
       * </p>
       * <p>
       * The binding can either be a plain JavaScript function, a String, <code>null</code>, or an array which contains the same types
       * recursively. A function is called with the business object to convert as first and only parameter, and the function's <code>this</code>
       * is set to the business object, too.
       * </p>
       * @type {Object}
       */
      parentGroupBinding:Object;
      static $class:yfiles.lang.Class;
    }
  }export namespace algorithms{
    /**
     * This class provides a means for early termination of graph algorithms.
     * <p>
     * Instances of this class may be attached to and retrieved from a graph and may receive requests for stopping and
     * canceling an algorithm that is currently executed on the given graph.
     * </p>
     * <p>
     * <b>Client Code Usage</b>
     * </p>
     * <p>
     * An instance of this class can be attached to a graph using method {@link yfiles.algorithms.AbortHandler#createForGraph}. The
     * handler's {@link yfiles.algorithms.AbortHandler#stopDuration} and {@link yfiles.algorithms.AbortHandler#cancelDuration} methods can be used to automatically stop or cancel an algorithm after a specified period of time has
     * elapsed. Algorithms can check for these requests and handle them appropriately.
     * </p>
     * <ul>
     * <li>
     * <b>Stop</b> – The algorithm should terminate gracefully, delivering a consistent result. Although the termination will be early, it
     * will usually not be immediate.
     * </li>
     * <li><b>Cancel</b> – The algorithm should terminate immediately and all work done so far will be discarded.</li>
     * </ul>
     * <p>
     * If a graph with an attached handler is processed by multiple algorithms (or multiple times by the same algorithm), the
     * attached handler has to be {@link yfiles.algorithms.AbortHandler#reset} between algorithm runs. Otherwise, previous requests for
     * early termination may lead to an undesired early termination of the next algorithm run.
     * </p>
     * <p>
     * <b>Usage in Algorithms</b>
     * </p>
     * <p>
     * Algorithms have to retrieve an instance of this class from the graph that is processed using method
     * {@link yfiles.algorithms.AbortHandler#getFromGraph}. Then, the algorithm needs to query the retrieved instance of this class for
     * stop or cancel requests using method {@link yfiles.algorithms.AbortHandler#check}.
     * </p>
     * <p>
     * Alternatively, convenience method {@link yfiles.algorithms.AbortHandler#check} for one-time checks is available. For performance
     * critical code that checks repeatedly, it is recommended to follow the first approach, though.
     * </p>
     * <p>
     * When handling a stop request, algorithms should ensure that the resulting graph is still in a consistent state.
     * </p>
     * @class
     */
    export interface AbortHandler extends Object{}
    export class AbortHandler {
      /**
       * Creates a new {@link yfiles.algorithms.AbortHandler} instance.
       * @see yfiles.algorithms.AbortHandler#stopDuration
       * @see yfiles.algorithms.AbortHandler#cancelDuration
       * @see yfiles.algorithms.AbortHandler#reset
       */
      constructor();
      /**
       * Determines whether or not an algorithm should terminate immediately.
       * <p>
       * This method returns <code>true</code> if the algorithm should terminate gracefully and ensures that the processed graph remains in a
       * consistent state.
       * </p>
       * @returns <code>true</code>, if the algorithm should terminate immediately, <code>false</code> otherwise
       * @throws {yfiles.algorithms.AlgorithmAbortedError} if the algorithm should terminate immediately
       * @see yfiles.algorithms.AbortHandler#stop
       */
      check():boolean;
      /**
       * Determines whether or not an algorithm should terminate immediately.
       * <p>
       * This method returns <code>true</code> if the algorithm should terminate gracefully and ensures that the processed graph remains in a
       * consistent state.
       * </p>
       * <p>
       * This convenience method is meant for one-time checks only. For performance critical code that needs to check repeatedly,
       * it is recommended to retrieve the given graph's attached handler once and only call the handler's {@link yfiles.algorithms.AbortHandler#check}
       * method repeatedly.
       * </p>
       * @param graph
       * @returns <code>true</code>, if the algorithm should stop immediately while still providing some valid result, <code>false</code> otherwise
       * @throws {yfiles.algorithms.AlgorithmAbortedError} if the algorithm should terminate immediately
       * @throws {Stubs.Exceptions.ArgumentError} if the given graph is <code>null</code>
       * @see yfiles.algorithms.AbortHandler#check
       * @see yfiles.algorithms.AbortHandler#stop
       * @static
       */
      static check(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Attaches the {@link yfiles.algorithms.AbortHandler} instance of the given source graph to the target graph as well.
       * @param source the graph whose handler is attached to the target graph
       * @param target the graph to which the handler of the source graph is attached
       * @throws {Stubs.Exceptions.ArgumentError} if the given source is <code>null</code>
       * @static
       */
      static copyHandler(source:yfiles.algorithms.Graph,target:yfiles.algorithms.Graph):void;
      /**
       * Creates an {@link yfiles.algorithms.AbortHandler handler} instance and attaches it to the given graph.
       * <p>
       * If the given graph has already an attached handler instance, this is the instance that will be returned and this method
       * will not create a new {@link yfiles.algorithms.AbortHandler}.
       * </p>
       * <p>
       * This method should be called by client code prior to starting a graph algorithm that may be terminated early.
       * </p>
       * @param graph the graph to which the handler will be attached
       * @returns the {@link } instance for the given graph
       * @throws {Stubs.Exceptions.ArgumentError} if the given graph is <code>null</code>.
       * @see yfiles.algorithms.AbortHandler#hasHandler
       * @static
       */
      static createForGraph(graph:yfiles.algorithms.Graph):yfiles.algorithms.AbortHandler;
      /**
       * Returns an {@link yfiles.algorithms.AbortHandler} instance for the given graph.
       * <p>
       * If {@link yfiles.algorithms.AbortHandler#createForGraph} has been used for attaching a new handler to the given graph, this is
       * the instance that will be returned. Otherwise, a non-functional instance is returned whose methods do nothing. Use {@link yfiles.algorithms.AbortHandler#hasHandler}
       * to check whether or not a handler has been already attached to the given graph.
       * </p>
       * @param graph the given graph
       * @returns an {@link } for the given graph or a non-functional instance if no handler has been previously
       * created
       * @throws {Stubs.Exceptions.ArgumentError} if the given graph is <code>null</code>
       * @see yfiles.algorithms.AbortHandler#createForGraph
       * @see yfiles.algorithms.AbortHandler#hasHandler
       * @static
       */
      static getFromGraph(graph:yfiles.algorithms.Graph):yfiles.algorithms.AbortHandler;
      /**
       * Determines whether or not an {@link yfiles.algorithms.AbortHandler} instance is attached to the given graph.
       * @param graph the given graph
       * @returns <code>true</code> if a handler is attached to the given graph, <code>false</code> otherwise
       * @throws {Stubs.Exceptions.ArgumentError} if the given graph is <code>null</code>
       * @static
       */
      static hasHandler(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Removes any attached {@link yfiles.algorithms.AbortHandler} instance from the given graph.
       * @param graph the given graph
       * @throws {Stubs.Exceptions.ArgumentError} if the given graph is <code>null</code>.
       * @static
       */
      static removeFromGraph(graph:yfiles.algorithms.Graph):void;
      /**
       * Resets the state of the {@link yfiles.algorithms.AbortHandler handler}.
       * <p>
       * Resetting the {@link yfiles.algorithms.AbortHandler handler} discards any previous stop or cancel requests. Moreover, the handler's internal timestamp that is
       * used for determining whether or not an algorithm should be stopped or cancelled automatically is reset as well.
       * </p>
       * <p>
       * This method should be called whenever a graph with an attached handler is processed an additional time to prevent
       * previous requests for early termination to result in an undesired early termination of the next algorithm run.
       * </p>
       * @see yfiles.algorithms.AbortHandler#cancelDuration
       * @see yfiles.algorithms.AbortHandler#stopDuration
       */
      reset():void;
      /**
       * Schedules a stop request.
       * <p>
       * Algorithms that detect stop requests should terminate gracefully and ensure that the processed graph remains in a
       * consistent state.
       * </p>
       * <p>
       * If a Cancel request has already been scheduled for this handler, the stop request is ignored. If a Cancel request is
       * scheduled later on, the stop request is overridden.
       * </p>
       * @see yfiles.algorithms.AbortHandler#check
       * @see yfiles.algorithms.AbortHandler#cancelDuration
       */
      stop():void;
      /**
       * Determines the remaining time (in milliseconds) until an algorithm that {@link yfiles.algorithms.AbortHandler#check checks} this handler is cancelled automatically.
       * @returns the remaining time until the algorithm is cancelled automatically.
       * @see yfiles.algorithms.AbortHandler#cancelDuration
       * @see yfiles.algorithms.AbortHandler#reset
       * @see yfiles.algorithms.AbortHandler#cancelDuration
       */
      timeToCancel():yfiles.lang.TimeSpan;
      /**
       * Determines the remaining time until an algorithm that {@link yfiles.algorithms.AbortHandler#check checks} this handler is stopped automatically.
       * <p>
       * If the {@link yfiles.algorithms.AbortHandler#stopDuration stop duration} is less than or equal to zero, <code>-1</code> will be returned which means that the algorithm may run
       * unrestricted.
       * </p>
       * @returns the remaining time until the algorithm is stopped automatically or <code>-1</code> if the algorithm may run unrestricted
       * @see yfiles.algorithms.AbortHandler#stopDuration
       * @see yfiles.algorithms.AbortHandler#reset
       * @see yfiles.algorithms.AbortHandler#stop
       */
      timeToStop():yfiles.lang.TimeSpan;
      /**
       * Data provider key for attaching an AbortHandler instance to a graph
       * <p>
       * Only instances of {@link yfiles.algorithms.AbortHandler} should be assigned to this {@link yfiles.algorithms.IDataProvider}, otherwise a
       * {@link yfiles.lang.Exception} will occur. Layout algorithms will use the attached handler to check for requests to cancel or
       * stop the layout process.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.GraphDpKey.<yfiles.algorithms.AbortHandler>}
       */
      static ABORT_HANDLER_DP_KEY:yfiles.algorithms.GraphDpKey<yfiles.algorithms.AbortHandler>;
      /**
       * Returns whether or not a stop request was scheduled explicitly with the {@link yfiles.algorithms.AbortHandler#stop} method.
       * @type {boolean}
       */
      stopRequested:boolean;
      /**
       * Gets or sets the duration an algorithm may run before being stopped automatically.
       * <p>
       * An algorithm is terminated gracefully, if the time in between {@link yfiles.algorithms.AbortHandler creating} or {@link yfiles.algorithms.AbortHandler#reset resetting} this handler and calling {@link yfiles.algorithms.AbortHandler#check}
       * exceeds the stop duration.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the duration is negative
       * @see yfiles.algorithms.AbortHandler#timeToStop
       * @see yfiles.algorithms.AbortHandler#reset
       * @see yfiles.algorithms.AbortHandler#stop
       * @type {yfiles.lang.TimeSpan}
       */
      stopDuration:yfiles.lang.TimeSpan;
      /**
       * Gets or sets the duration an algorithm may run before being cancelled automatically.
       * <p>
       * An algorithm is terminated immediately, if the time in between {@link yfiles.algorithms.AbortHandler creating} or {@link yfiles.algorithms.AbortHandler#reset resetting} this handler and calling {@link yfiles.algorithms.AbortHandler#check}
       * exceeds the cancel duration.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the duration is negative
       * @see yfiles.algorithms.AbortHandler#reset
       * @see yfiles.algorithms.AbortHandler#cancelDuration
       * @type {yfiles.lang.TimeSpan}
       */
      cancelDuration:yfiles.lang.TimeSpan;
      /**
       * Gets whether or not methods {@link yfiles.algorithms.AbortHandler#check} or {@link yfiles.algorithms.AbortHandler#check} were called
       * after a stop or cancel event.
       * <p>
       * More precisely, it returns <code>true</code> if one of the check methods either threw an {@link yfiles.lang.Exception} or returned <code>true</code> to
       * indicate that the calling algorithm should terminate gracefully. Otherwise, this method returns <code>false</code>.
       * </p>
       * @see yfiles.algorithms.AbortHandler#check
       * @see yfiles.algorithms.AbortHandler#check
       * @type {boolean}
       */
      checkFailed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides services that center around breadth first search (BFS).
     * <p>
     * Breadth first search starts at a given set of nodes and explores the neighboring nodes first, before visiting the next
     * level neighbors.
     * </p>
     * <p>
     * A breadth first search run can be either directed or undirected. All methods require a list of nodes that are considered
     * as the core nodes from which the breadth first search starts. These nodes can be identified either with a {@link yfiles.algorithms.NodeList}
     * or with a {@link yfiles.algorithms.IDataProvider} that returns <code>true</code> for core nodes and <code>false</code> for all other nodes. The output is
     * given as an array of {@link yfiles.algorithms.NodeList}s each of which contains the nodes of a particular layer.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0},"n":[[-35,-35,1,[[-25.34,-10.65,10.67,18.7,"0"]]],[-35,85,[[-25.34,109.35,10.67,18.7,"1"]]],[85,-35,[[94.66,-10.65,10.67,18.7,"1"]]],[85,85,[[94.66,109.35,10.67,18.7,"2"]]],[205,-35,[[214.66,-10.65,10.67,18.7,"3"]]],[205,85,[[214.66,109.35,10.67,18.7,"4"]]]],"e":[[4,3],[2,0],[0,1],[1,3],[2,3],[4,5]],"vp":[-35.0,-35.0,270.0,150.0]}} Example for a BFS run. The marked node is the core node from which BFS starts while node labels indicate the resulting
     * layers.
     * </p>
     * @class
     * @static
     */
    export interface Bfs extends Object{}
    export class Bfs {
      /**
       * Returns the layers of nodes constructed by a directed/undirected breadth first search where the maximum number of layers
       * is restricted.
       * <p>
       * The first of these layers contains all nodes within the given {@link yfiles.algorithms.NodeList}. These nodes are the core nodes
       * from which a breadth first search to the other nodes starts. The algorithm fills the provided {@link yfiles.algorithms.INodeMap}
       * with an integer indicating the layer of each node.
       * </p>
       * <p>
       * In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
       * </p>
       * @param graph the given graph
       * @param coreNodes the list of core nodes
       * @param direction one of the predefined direction specifiers
       * @param layerIDMap the {@link } that will be filled during the BFS execution and holds the zero-based index of the BFS
       * layer to which each node belongs or <code>-1</code> if the node is not reachable
       * @param [maxLayers=0] the number of layers that will be returned or <code>0</code> if all layers are required
       * @returns an array of {@link }s each of which contains the nodes of a particular layer
       * @throws {Stubs.Exceptions.ArgumentError} if the given direction is not supported
       * @static
       */
      static getLayers(graph:yfiles.algorithms.Graph,coreNodes:yfiles.algorithms.NodeList,direction:yfiles.algorithms.BfsDirection,layerIDMap:yfiles.algorithms.INodeMap,maxLayers?:number):yfiles.algorithms.NodeList[];
      /**
       * Returns the layers of nodes constructed by a directed/undirected breadth first search where the maximum number of layers
       * is restricted.
       * <p>
       * The first of these layers contains all nodes within the given {@link yfiles.algorithms.NodeList}. These nodes are the core nodes
       * from which either a directed or undirected breadth first search to the other nodes starts. The algorithm fills the
       * provided {@link yfiles.algorithms.INodeMap} with an integer indicating the layer of each node.
       * </p>
       * <p>
       * In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
       * </p>
       * @param graph the given graph
       * @param coreNodes the list of core nodes from which the BFS starts
       * @param directed <code>true</code> if the graph should be considered directed, <code>false</code> otherwise
       * @param layerIDMap the {@link } that will be filled during the BFS execution and holds the zero-based index of the BFS
       * layer to which each node belongs or <code>-1</code> if the node is not reachable
       * @param [maxLayers=0] the number of layers that will be returned or <code>0</code> if all layers are required
       * @returns an array of {@link }s each of which contains the nodes of a particular layer
       * @static
       */
      static getLayers(graph:yfiles.algorithms.Graph,coreNodes:yfiles.algorithms.NodeList,directed:boolean,layerIDMap:yfiles.algorithms.INodeMap,maxLayers?:number):yfiles.algorithms.NodeList[];
      /**
       * Returns the layers of nodes constructed by a breadth first search.
       * <p>
       * The first of these layers contains all nodes within the given {@link yfiles.algorithms.NodeList}. These nodes are the core nodes
       * from which an undirected breadth first search to the other nodes starts. The algorithm fills the provided {@link yfiles.algorithms.INodeMap}
       * with an integer indicating the layer of each node.
       * </p>
       * <p>
       * In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
       * </p>
       * @param graph the given graph
       * @param coreNodes the list of core nodes from which the BFS starts
       * @param layerIDMap the {@link } that will be filled during the BFS execution and holds the zero-based index of the BFS
       * layer to which each node belongs or <code>-1</code> if the node is not reachable
       * @returns an array of {@link }s each of which contains the nodes of a particular layer
       * @static
       */
      static getLayers(graph:yfiles.algorithms.Graph,coreNodes:yfiles.algorithms.NodeList,layerIDMap:yfiles.algorithms.INodeMap):yfiles.algorithms.NodeList[];
      /**
       * Returns the layers of nodes constructed by a breadth first search.
       * <p>
       * The first of these layers contains all nodes for which the given {@link yfiles.algorithms.IDataProvider} returns <code>true</code>. These
       * nodes are the core nodes from which an undirected breadth first search to the other nodes starts. The algorithm fills
       * the provided {@link yfiles.algorithms.INodeMap} with an integer indicating the layer of each node.
       * </p>
       * <p>
       * In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
       * </p>
       * @param graph the given graph
       * @param isCoreNode the {@link } that contains the nodes from which the BFS starts; core nodes are marked with a <code>true</code>
       * value
       * @param layerIDMap the {@link } that will be filled during the BFS execution and holds the zero-based index of the BFS
       * layer to which each node belongs or <code>-1</code> if the node is not reachable
       * @returns an array of {@link }s each of which contains the nodes of a particular layer
       * @static
       */
      static getLayers(graph:yfiles.algorithms.Graph,isCoreNode:yfiles.algorithms.IDataProvider,layerIDMap:yfiles.algorithms.INodeMap):yfiles.algorithms.NodeList[];
      /**
       * Returns the layers of nodes calculated by a breadth first search.
       * <p>
       * The first of these layers contains all nodes within the given
       * {@link yfiles.algorithms.NodeList}. These nodes are the core nodes from which an undirected breadth first search to the other
       * nodes starts.
       * </p>
       * <p>
       * In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
       * </p>
       * @param graph the given graph
       * @param coreNodes the list of core nodes from which the BFS starts
       * @returns an array of {@link }s each of which contains the nodes of a particular layer
       * @static
       */
      static getLayers(graph:yfiles.algorithms.Graph,coreNodes:yfiles.algorithms.NodeList):yfiles.algorithms.NodeList[];
      /**
       * Returns the layers of nodes constructed by a breadth first search.
       * <p>
       * The first of these layers contains all nodes for which the given {@link yfiles.algorithms.IDataProvider} returns
       * <code>true</code>. These nodes are the core nodes from which an undirected breadth first search to the other nodes starts.
       * </p>
       * <p>
       * In the <code>i</code>-th layer are previously unassigned nodes that are connected to nodes in the <code>(i-1)</code>-th layer.
       * </p>
       * @param graph the given graph
       * @param isCoreNode the {@link } that contains the nodes from which the BFS starts; core nodes are marked with a <code>true</code>
       * value
       * @returns an array of {@link }s each of which contains the nodes of a particular layer
       * @static
       */
      static getLayers(graph:yfiles.algorithms.Graph,isCoreNode:yfiles.algorithms.IDataProvider):yfiles.algorithms.NodeList[];
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides methods to determine whether a graph is bipartite and to obtain the corresponding partitions.
     * <p>
     * A <b>bipartite</b> graph is a graph whose nodes can be partitioned into two sets such that each edges connects two nodes of different
     * sets. In other words, there are no edges connecting nodes that belong to the same partition.
     * </p>
     * <p>
     * The two sets/partitions are represented by two {@link Object} constants i.e., {@link yfiles.algorithms.Bipartitions#RED} or
     * {@link yfiles.algorithms.Bipartitions#BLUE}.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[40,40],"d":0, "b":0},"styles":{"6":{"c":1, "b":0}},"n":[[365,205,6],[365,305,6],[365,405,6],[505,205],[505,305],[505,405]],"e":[[0,4],[0,3],[0,5],[1,4],[2,3],[2,5],[1,5]],"vp":[365.0,200.0,170.0,250.0]}} Example of a bipartite graph. Circular and rectangular nodes represent the two partitions.
     * </p>
     * @class
     * @static
     */
    export interface Bipartitions extends Object{}
    export class Bipartitions {
      /**
       * Calculates a bipartition of the given graph, if one exists.
       * <p>
       * If the graph is bipartite, then for all nodes of the given graph either {@link yfiles.algorithms.Bipartitions#RED} or {@link yfiles.algorithms.Bipartitions#BLUE}
       * objects will be set in the given {@link yfiles.algorithms.INodeMap}, depending on the partition to which each node belongs.
       * </p>
       * @param graph the given graph
       * @param markMap the {@link } that will be filled during the BFS execution and returns the partition (either {@link #RED}
       * or {@link #BLUE}) to which each node belongs
       * @returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
       * @static
       */
      static getBipartition(graph:yfiles.algorithms.Graph,markMap:yfiles.algorithms.INodeMap):boolean;
      /**
       * Determines whether or not the given graph is bipartite.
       * @param graph the given graph
       * @returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
       * @static
       */
      static isBipartite(graph:yfiles.algorithms.Graph):boolean;
      /**
       * A constant for marking a node that belongs to the red partition.
       * @const
       * @static
       * @type {Object}
       */
      static RED:Object;
      /**
       * A constant for marking a node that belongs to the blue partition.
       * @const
       * @static
       * @type {Object}
       */
      static BLUE:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides methods to determine various centrality indices of nodes or edges of a graph.
     * <p>
     * Centrality indices serve to quantify an intuitive feeling that in most networks some nodes or edges are "more central"
     * than others. The provided methods assign a double value to each node or edge of a graph that represents its centrality.
     * The higher an assigned value, the more central the element is considered by the algorithm.
     * </p>
     * <p>
     * Also, this class provides convenience methods that normalize the returned centrality values such that they lie within
     * the interval <code>[0,1]</code>.
     * </p>
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li><b>Betweenness centrality</b> is a measure for how often a node/edge lies on a shortest path between each pair of nodes in the graph.</li>
     * <li><b>Closeness centrality</b> is the reciprocal of the sum of shortest path distances of a node to all other nodes in the graph.</li>
     * <li><b>Graph centrality</b> is the reciprocal of the maximum of all shortest path distances from a node to all other nodes in the graph.</li>
     * <li>
     * <b>Degree centrality</b> is the number of the incoming, outgoing or overall edges incident to a node (measures incoming, outgoing and overall
     * degree).
     * </li>
     * <li><b>Weight centrality</b> measures the weight associated with incoming, outgoing, or all edges of a node.</li>
     * </ul>
     * @class
     * @static
     */
    export interface Centrality extends Object{}
    export class Centrality {
      /**
       * Computes the closeness centrality for the nodes of a graph.
       * <p>
       * <i>Closeness centrality</i> is defined as the reciprocal of the sum of shortest path distances of a node to all other nodes in the graph.
       * Therefore, a node with high closeness centrality has short distances to all other nodes of a graph. If the sum of the
       * shortest path distances is <code>0</code>, the closeness of a node is set to {@link number#POSITIVE_INFINITY}.
       * </p>
       * @param graph the input graph
       * @param closeness the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param edgeCosts the {@link } that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost
       * @static
       */
      static closenessCentrality(graph:yfiles.algorithms.Graph,closeness:yfiles.algorithms.INodeMap,directed:boolean,edgeCosts:yfiles.algorithms.IDataProvider):void;
      /**
       * Computes the degree centrality for the nodes of a given graph.
       * <p>
       * <i>Degree centrality</i> is the number of the incoming, outgoing or overall edges incident to a node (measures incoming, outgoing and overall
       * degree).
       * </p>
       * @param graph the input graph
       * @param centrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * node
       * @param considerInEdges <code>true</code> if the incoming edges should be considered, <code>false</code> otherwise
       * @param considerOutEdges <code>true</code> if the outgoing edges should be considered, <code>false</code> otherwise
       * @static
       */
      static degreeCentrality(graph:yfiles.algorithms.Graph,centrality:yfiles.algorithms.INodeMap,considerInEdges:boolean,considerOutEdges:boolean):void;
      /**
       * Computes betweenness centrality for each edge of a given graph.
       * <p>
       * <i>Betweenness centrality</i> is a measure for how often an edge lies on a shortest path between each pair of nodes in the graph. Removing a central
       * edge will cause many shortest paths to change.
       * </p>
       * @param graph the input graph
       * @param centrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * edge
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param edgeCosts the {@link } that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost;
       * for invalid input values the algorithm uses cost <code>1.0</code>
       * @static
       */
      static edgeBetweenness(graph:yfiles.algorithms.Graph,centrality:yfiles.algorithms.IEdgeMap,directed:boolean,edgeCosts:yfiles.algorithms.IDataProvider):void;
      /**
       * Computes the graph centrality for the nodes of a graph.
       * <p>
       * <i>Graph centrality</i> is defined as the reciprocal of the maximum of all shortest path distances from a node to all other nodes in the graph.
       * Nodes with high graph centrality have short distances to all other nodes in the graph.
       * </p>
       * @param graph the input graph
       * @param centrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param edgeCosts the {@link } that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost
       * @static
       */
      static graphCentrality(graph:yfiles.algorithms.Graph,centrality:yfiles.algorithms.INodeMap,directed:boolean,edgeCosts:yfiles.algorithms.IDataProvider):void;
      /**
       * Computes betweenness centrality for each node of a given graph.
       * <p>
       * <i>Betweenness centrality</i> is a measure for how often a node lies on a shortest path between each pair of nodes in the graph. Removing a central
       * edge will cause many shortest paths to change.
       * </p>
       * @param graph the input graph
       * @param centrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param edgeCosts the {@link } that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost;
       * for invalid input values the algorithm uses cost <code>1.0</code>
       * @static
       */
      static nodeBetweenness(graph:yfiles.algorithms.Graph,centrality:yfiles.algorithms.INodeMap,directed:boolean,edgeCosts:yfiles.algorithms.IDataProvider):void;
      /**
       * Computes betweenness centrality for each node and edge of a given graph.
       * <p>
       * <i>Betweenness centrality</i> is a measure for how often a node/edge lies on a shortest path between each pair of nodes in the graph. Removing a
       * central node/edge will cause many shortest paths to change.
       * </p>
       * @param graph the input graph
       * @param nodeCentrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * node
       * @param edgeCentrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * edge
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param edgeCosts the {@link } that returns a positive double value (cost) or <code>null</code> if the edges are of equal cost;
       * for invalid input values the algorithm uses cost <code>1.0</code>
       * @static
       */
      static nodeEdgeBetweenness(graph:yfiles.algorithms.Graph,nodeCentrality:yfiles.algorithms.INodeMap,edgeCentrality:yfiles.algorithms.IEdgeMap,directed:boolean,edgeCosts:yfiles.algorithms.IDataProvider):void;
      /**
       * Normalizes the <code>double</code> values of a given {@link yfiles.algorithms.IEdgeMap} by dividing each of them by the maximum of all values
       * (maximum norm).
       * @param graph the input graph
       * @param map the {@link } that will be filled during the execution and returns a double value from <code>[0,1]</code> interval
       * @static
       */
      static normalizeEdgeMap(graph:yfiles.algorithms.Graph,map:yfiles.algorithms.IEdgeMap):void;
      /**
       * Normalizes the <code>double</code> values of a given {@link yfiles.algorithms.INodeMap} by dividing each of them by the maximum of all values
       * (maximum norm).
       * @param graph the input graph
       * @param map the {@link } that will be filled during the execution and returns a double value from <code>[0,1]</code> interval
       * @static
       */
      static normalizeNodeMap(graph:yfiles.algorithms.Graph,map:yfiles.algorithms.INodeMap):void;
      /**
       * Computes the weight centrality for the nodes of a graph.
       * <p>
       * <i>Weight centrality</i> measures the weight associated with incoming, outgoing, or all edges of a node.
       * </p>
       * @param graph the input graph
       * @param centrality the {@link } that will be filled during the execution and returns a double value (centrality) for each
       * node
       * @param considerInEdges <code>true</code> if the incoming edges should be considered, <code>false</code> otherwise
       * @param considerOutEdges <code>true</code> if the outgoing edges should be considered, <code>false</code> otherwise
       * @param edgeWeights the {@link } that returns a positive double value (weight) or <code>null</code> if the edges are considered to
       * have uniform weight of <code>1.0</code>
       * @static
       */
      static weightCentrality(graph:yfiles.algorithms.Graph,centrality:yfiles.algorithms.INodeMap,considerInEdges:boolean,considerOutEdges:boolean,edgeWeights:yfiles.algorithms.IDataProvider):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is responsible for finding cycles within a graph that have certain properties.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>
     * An edge path with vertices <code>v0, v1, v2, ... , vk</code> (with k > 0 and (vi-1,vi), 0 &lt; i &lt;= k is edge of the given graph) is
     * called <b>simple</b> if no vertex appears more than once.
     * </li>
     * <li>An edge path forms a <b>cycle</b> if <code>v0 = vk</code>.</li>
     * <li>A cycle is called <b>simple</b> if no vertex appears more than once.</li>
     * <li>A graph that contains no cycles is called <b>acyclic</b>.</li>
     * </ul>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1, "c":1},"n":[[45,5],[105,65],[-15,65],[45,125],[185,65],[-95,65]],"e":[[0,1],[0,2],[1,3],[2,3],[1,4],[2,5]],"vp":[-95.0,5.0,310.0,150.0]}} Example of an acyclic directed graph (edge directions are considered) {@graph {"ann":{"s":[30,30],"d":1, "c":1},"n":[[305,185],[385,125],[345,285],[505,125],[545,225],[465,305]],"e":[[0,1],[1,3],[4,5,1],[5,2,1],[2,0,1],[0,4,1]],"vp":[305.0,125.0,270.0,210.0]}} Example of a graph containing a cycle. Marked
     * edges form a directed cycle.
     * </p>
     * @class
     * @static
     */
    export interface Cycles extends Object{}
    export class Cycles {
      /**
       * Returns an {@link yfiles.algorithms.EdgeList} that contains all the edges that are part of at least one directed or undirected
       * simple cycle.
       * @param graph the input graph
       * @param directed <code>true</code> if the graph should be considered directed, <code>false</code> otherwise
       * @returns an {@link } that contains all the edges that are part of at least one directed or undirected simple
       * cycle
       * @static
       */
      static findAllCycleEdges(graph:yfiles.algorithms.Graph,directed:boolean):yfiles.algorithms.EdgeList;
      /**
       * Returns an {@link yfiles.algorithms.EdgeList} that contains the edges of a cycle found in the given graph.
       * <p>
       * The edges are returned in the order they appear in the detected cycle.
       * </p>
       * <p>
       * If the returned cycle is empty, no cycle has been found in the given graph.
       * </p>
       * @param graph the given graph
       * @param directed <code>true</code> if the graph should be considered directed, <code>false</code> otherwise
       * @returns an {@link } containing the edges of a cycle or an empty {@link } if the graph is
       * acyclic
       * @static
       */
      static findCycle(graph:yfiles.algorithms.Graph,directed:boolean):yfiles.algorithms.EdgeList;
      /**
       * Marks the edges of a given graph whose removal or reversal would make the graph acyclic while trying to minimize the
       * cost associated with the marked edges.
       * <p>
       * This minimization is performed heuristically, since it is a well known hard problem to come up with an optimal
       * solution.
       * </p>
       * <p>
       * The costs are assigned using a {@link yfiles.algorithms.IDataProvider} that holds a non-negative double value for each edge.
       * </p>
       * @param graph the given graph
       * @param cycleEdges the {@link } that will be filled during the execution and returns whether an edge is a detected cycle
       * edge
       * @param [costDP=null] the {@link } that holds the non-negative {@link } reversal cost for each edge
       * @static
       */
      static findCycleEdges(graph:yfiles.algorithms.Graph,cycleEdges:yfiles.algorithms.IEdgeMap,costDP?:yfiles.algorithms.IDataProvider):void;
      /**
       * Marks the edges of a given graph whose removal or reversal would make the graph acyclic based on a depth first search.
       * <p>
       * The number of marked cycle edges is expected to be slightly greater than when using
       * {@link yfiles.algorithms.Cycles#findCycleEdges}. The advantage of this method is that the result set is more stable when edges
       * are added or removed over time.
       * </p>
       * @param graph the given graph
       * @param cycleEdges the {@link } that will be filled during the execution and returns a boolean value indicating whether
       * or not an edge is a detected cycle edge
       * @static
       */
      static findCycleEdgesDFS(graph:yfiles.algorithms.Graph,cycleEdges:yfiles.algorithms.IEdgeMap):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a dart of a face of a {@link yfiles.algorithms.PlanarEmbedding}.
     * <p>
     * For each edge there exist two darts, one that represents the edge in its original direction (i.e., from source to
     * target) and one that represents its reverse. Each dart is associated with a {@link yfiles.algorithms.Dart#face face}.
     * </p>
     * @class
     * @final
     */
    export interface Dart extends Object{}
    export class Dart {
      /**
       * Gets the edge associated with this dart.
       * @type {yfiles.algorithms.Edge}
       */
      associatedEdge:yfiles.algorithms.Edge;
      /**
       * Gets whether this dart represents the original edge or its reverse.
       * @type {boolean}
       */
      reversed:boolean;
      /**
       * Gets the opposite dart.
       * <p>
       * The opposite dart is associated with the same edge as this dart but with a different {@link yfiles.algorithms.Dart#reversed direction}.
       * </p>
       * @type {yfiles.algorithms.Dart}
       */
      oppositeDart:yfiles.algorithms.Dart;
      /**
       * Gets the face list associated with this dart.
       * @type {yfiles.collections.IList.<yfiles.algorithms.Dart>}
       */
      face:yfiles.collections.IList<yfiles.algorithms.Dart>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Framework class for implementing depth first search (DFS) based algorithms.
     * <p>
     * A depth first search starts from a specified node and traverses the neighbors within a branch as deeply as possible
     * (i.e., without visiting any vertex twice) before backtracking.
     * </p>
     * <p>
     * A depth first search traversal induces a DFS forest that may consist of more than one DFS tree. Based on these trees,
     * the edges are divided into four categories (see example):
     * </p>
     * <ul>
     * <li><b>Tree Edges:</b> The edges of the DFS forest.</li>
     * <li><b>Back edges:</b> Non-tree edges that connect nodes with their ancestors in the DFS tree. Self-loops are also considered as back edges.</li>
     * <li><b>Forward edges:</b> Edges that connect nodes of the DFS tree to one of their descendants.</li>
     * <li><b>Cross edges:</b> All other edges.</li>
     * </ul>
     * <p>
     * During a DFS run each node of the graph can be in one of the following states:
     * </p>
     * <ul>
     * <li><b>Undiscovered:</b> A node that has not been visited yet.</li>
     * <li>
     * <b>Discovered but not finished:</b> A node that has been already visited, but has not been completed yet, i.e. it is still part of an active path of the
     * DFS tree.
     * </li>
     * <li>
     * <b>Finished:</b> A node that has been completed, i.e. it has been visited before and is not part of an active path of the DFS tree
     * anymore.
     * </li>
     * </ul>
     * <p>
     * Graph algorithms, which are based on a depth first search, can extend this class and override appropriate callback
     * methods provided by this class.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1, "c":1},"styles" : {"6" : {"s" : "#000000", "d" : 1,  "l" : [3] }, "7" : {"s" : "#000000", "d" : 1,  "l" : [1,4] }, "8" : {"s" : "#000000", "d" : 1,  "l" : [7] }},"n":[[145,-15,[[154.66,9.35,10.67,18.7,"1"]]],[65,65,[[74.66,89.35,10.67,18.7,"2"]]],[185,65,[[194.66,89.35,10.67,18.7,"8"]]],[45,225,[[54.66,249.35,10.67,18.7,"5"]]],[125,145,[[134.66,169.35,10.67,18.7,"7"]]],[65,145,[[74.66,169.35,10.67,18.7,"6"]]],[-35,225,[[-25.34,249.35,10.67,18.7,"4"]]],[5,145,[[14.66,169.35,10.67,18.7,"3"]]],[185,145,[[194.66,169.35,10.67,18.7,"9"]]]],"e":[[0,1],[1,7],[0,2],[7,3],[1,4],[1,5],[7,6],[2,1,6,[],[[118.33,72.85,35.34,18.7,"Cross"]]],[6,1,8,[-60,100,-60,100],[[-95.2,149.04,30.68,18.7,2.4492935982947064E-16,-1.0,"Back"]]],[0,3,7,[360,140],[[270.38,202.85,48.01,18.7,2.4492935982947064E-16,-1.0,"Forward"]]],[2,8]],"vp":[-96.0,-15.0,456.0,270.0]}} Example of a DFS traversal. Node labels indicate the order in which nodes are visited. Solid edges are the edges of the
     * DFS tree while dashed edges represent back, forward and cross edges.
     * </p>
     * @class
     */
    export interface Dfs extends Object{}
    export class Dfs {
      /**
       * Creates a new {@link yfiles.algorithms.Dfs} instance with default settings.
       */
      constructor();
      /**
       * Cancels the depth first search.
       * <p>
       * It may be overridden to support custom implementations.
       * </p>
       */
      cancel():void;
      /**
       * Callback method that will be invoked whenever DFS continues its search at a new root node.
       * <p>
       * By default, this method does nothing. It may be overridden to support custom implementations.
       * </p>
       * @param v the new root node
       * @protected
       */
      lookFurther(v:yfiles.algorithms.Node):void;
      /**
       * Callback method that will be invoked after the DFS has returned from the given node.
       * <p>
       * By default, this method does nothing. It may be overridden to support custom implementations.
       * </p>
       * @param edge the given edge
       * @param node the node that has been reached via the given edge
       * @protected
       */
      postTraverse(edge:yfiles.algorithms.Edge,node:yfiles.algorithms.Node):void;
      /**
       * Callback method that will be invoked whenever a node visit has been completed.
       * <p>
       * By default, this method does nothing. It may be overridden to support custom implementations.
       * </p>
       * @param node the given node
       * @param dfsNumber the DFS number of the given node
       * @param compNumber the completion number of the given node
       * @protected
       */
      postVisit(node:yfiles.algorithms.Node,dfsNumber:number,compNumber:number):void;
      /**
       * Callback method that will be invoked if the given edge will be considered the first (and only) time during the DFS.
       * <p>
       * By default, this method does nothing. It may be overridden to support custom implementations.
       * </p>
       * @param edge the given edge
       * @param node the node to be visited next only if <code>treeEdge == true</code>
       * @param treeEdge <code>true</code> if the <code>node</code> will be visited, <code>false</code> otherwise
       * @protected
       */
      preTraverse(edge:yfiles.algorithms.Edge,node:yfiles.algorithms.Node,treeEdge:boolean):void;
      /**
       * Callback method that will be invoked whenever a formerly unvisited node gets visited for the first time.
       * <p>
       * By default, this method does nothing. It may be overridden to support custom implementations.
       * </p>
       * @param node the given node
       * @param dfsNumber the DFS number of the given node
       * @protected
       */
      preVisit(node:yfiles.algorithms.Node,dfsNumber:number):void;
      /**
       * Starts a depth first search from a given {@link yfiles.algorithms.Node} of the input graph.
       * @param graph the input graph
       * @param start the given start node
       */
      start(graph:yfiles.algorithms.Graph,start:yfiles.algorithms.Node):void;
      /**
       * Starts a depth first search on the given graph.
       * <p>
       * The first node of the graph will be visited first.
       * </p>
       * @param graph the input graph
       */
      start(graph:yfiles.algorithms.Graph):void;
      /**
       * A {@link yfiles.algorithms.INodeMap} that holds for each {@link yfiles.algorithms.Node} an {@link Object} indicating the current state of the given
       * node as it is visited by this algorithm.
       * <p>
       * Each node will be assigned one of the {@link yfiles.algorithms.Dfs#WHITE WHITE}, {@link yfiles.algorithms.Dfs#GRAY GRAY} or {@link yfiles.algorithms.Dfs#BLACK BLACK} objects that describe the state of the nodes.
       * </p>
       * @protected
       * @type {yfiles.algorithms.INodeMap}
       */
      stateMap:yfiles.algorithms.INodeMap;
      /**
       * A constant indicating a node has not been visited yet.
       * @protected
       * @const
       * @static
       * @type {Object}
       */
      static WHITE:Object;
      /**
       * A constant indicating a node has already been visited, but has not been completed yet.
       * <p>
       * Such a node is still part of an active path of the DFS tree.
       * </p>
       * @protected
       * @const
       * @static
       * @type {Object}
       */
      static GRAY:Object;
      /**
       * A constant indicating a node has been completed.
       * <p>
       * Such a node has been visited before and is no longer part of an active path of the DFS tree.
       * </p>
       * @protected
       * @const
       * @static
       * @type {Object}
       */
      static BLACK:Object;
      /**
       * Sets whether or not to interpret the edges of the graph as directed.
       * @type {boolean}
       */
      directedMode:boolean;
      /**
       * Sets whether or not to continue the depth first search after all nodes reachable from the first node have been visited.
       * @type {boolean}
       */
      lookFurtherMode:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is the base class of all look-up keys that are used to register {@link yfiles.algorithms.IDataProvider} with a
     * {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @template TValue
     */
    export interface DpKeyBase<TValue> extends Object{}
    export class DpKeyBase<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.DpKeyBase.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      /**
       * Returns whether this key is equal to the specified <code>other</code>.
       * @param other The other key to compare with this instance.
       * @returns <code>true</code> if the specified key is equal to this instance; otherwise, <code>false</code>.
       * @protected
       */
      equalsCore(other:yfiles.algorithms.DpKeyBase<TValue>):boolean;
      /**
       * Gets the type that declares this key.
       * @type {yfiles.lang.Class}
       */
      declaringType:yfiles.lang.Class;
      /**
       * Gets the name of this key.
       * @type {string}
       */
      name:string;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for {@link yfiles.algorithms.Edge}s with a
     * {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface EdgeDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class EdgeDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.EdgeDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for the {@link yfiles.algorithms.Graph} itself
     * with a graph.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface GraphDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class GraphDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.GraphDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for {@link yfiles.algorithms.GraphObject}s
     * with a {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface GraphObjectDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class GraphObjectDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.GraphObjectDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for IEdgeLabelLayouts with a
     * {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface IEdgeLabelLayoutDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class IEdgeLabelLayoutDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.IEdgeLabelLayoutDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for INodeLabelLayouts with a
     * {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface INodeLabelLayoutDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class INodeLabelLayoutDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.INodeLabelLayoutDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for ILabelLayouts with a
     * {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface ILabelLayoutDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class ILabelLayoutDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.ILabelLayoutDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used as look-up key for registering {@link yfiles.algorithms.IDataProvider} for {@link yfiles.algorithms.Node}s with a
     * {@link yfiles.algorithms.Graph}.
     * Type parameter <code>TValue</code>: The type of the values that are returned by the {@link yfiles.algorithms.IDataProvider} registered with this class.
     * @see yfiles.algorithms.Graph#addDataProvider
     * @see yfiles.algorithms.Graph#getDataProvider
     * @see yfiles.algorithms.Graph#removeDataProvider
     * @class
     * @extends {yfiles.algorithms.DpKeyBase.<TValue>}
     * @template TValue
     */
    export interface NodeDpKey<TValue> extends yfiles.algorithms.DpKeyBase<TValue>{}
    export class NodeDpKey<TValue> {
      /**
       * Initializes a new instance of the {@link yfiles.algorithms.NodeDpKey.<TValue>} class.
       * @param valueType The type of the values that are returned by the {@link } registered with this class.
       * @param declaringType Type that declares this key.
       * @param name The name of this key.
       */
      constructor(valueType:yfiles.lang.Class,declaringType:yfiles.lang.Class,name:string);
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents an edge, i.e., a directed connection between two nodes (represented by instances of class
     * {@link yfiles.algorithms.Node}) in the directed graph data type {@link yfiles.algorithms.Graph}.
     * <p>
     * <br /> The <i>directed</i> stems from the fact that an edge has a distinct source node and a distinct target node. Using pair notation, an
     * edge would be written as (<i>&lt;source node></i>, <i>&lt;target node></i>).
     * </p>
     * <p>
     * Most notably, an edge provides access to its source node ({@link yfiles.algorithms.Edge#source}) and its target node ({@link yfiles.algorithms.Edge#target}). Note that an edge can have the same
     * node as its source and target. Such an edge is then called "self-loop" and method {@link yfiles.algorithms.Edge#selfLoop} yields <code>true</code>.
     * </p>
     * <p>
     * <b>Important:</b> Class Graph is the single authority for any structural changes to the graph data type. Specifically, this means that
     * there is no way to create or delete a node or an edge without using an actual Graph instance.
     * </p>
     * @class
     * @extends {yfiles.algorithms.GraphObject}
     */
    export interface Edge extends yfiles.algorithms.GraphObject{}
    export class Edge {
      /**
       * Creates a new edge that belongs to the given graph.
       * <p>
       * The new edge <code>e</code> has source node <code>v</code> and target node <code>w</code>.
       * <br />
       * 
       * <br />
       * Edge <code>e</code> is inserted in such a way that an iteration over the edges at node <code>v</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>e1</code>, if <code>d1 == AFTER</code></li>
       * <li>before <code>e1</code>, if <code>d1 == BEFORE</code>,</li>
       * </ul>
       * <p>
       * and an iteration over the edges at <code>w</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>e2</code>, if <code>d2 == AFTER</code></li>
       * <li>before <code>e2</code>, if <code>d2 == BEFORE</code>.</li>
       * </ul>
       * @param v The source node of the edge.
       * @param e1 An edge with source node <code>v</code>.
       * @param w The target node of the edge.
       * @param e2 An edge with target node <code>w</code>.
       * @param d1 One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       * @param d2 One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       * @param g
       */
      constructor(g:yfiles.algorithms.Graph,v:yfiles.algorithms.Node,e1:yfiles.algorithms.Edge,w:yfiles.algorithms.Node,e2:yfiles.algorithms.Edge,d1:yfiles.algorithms.GraphElementInsertion,d2:yfiles.algorithms.GraphElementInsertion);
      /**
       * Creates a copy of this edge that will be inserted into the given graph connecting the given source and target nodes.
       * @param g The graph the created edge will belong to.
       * @param v The source node of the created edge.
       * @param w The target node of the created edge.
       * @returns The newly created Edge object.
       */
      createCopy(g:yfiles.algorithms.Graph,v:yfiles.algorithms.Node,w:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Callback method that is invoked from a graph just before this edge will be reinserted into that graph.
       */
      onReinsert():void;
      /**
       * Returns the node at the opposite edge end with respect to the given node.
       * <p>
       * Note that self-loops have the same node at both edge ends.
       * </p>
       * @param v
       * @returns 
       */
      opposite(v:yfiles.algorithms.Node):yfiles.algorithms.Node;
      /**
       * Gets the graph this edge belongs to.
       * <p>
       * If the edge does not belong to a graph, because it was removed or hidden from it, this method returns <code>null</code>.
       * </p>
       * @type {yfiles.algorithms.Graph}
       */
      graph:yfiles.algorithms.Graph;
      /**
       * Gets the index of this edge within its graph G.
       * <p>
       * Edge indices represent the ordering of standard edge iteration on G. The value of an index is <code> >= 0</code> and
       * <code> &lt; G.edgeCount()</code>.
       * </p>
       * <p>
       * Note that indices are subject to change whenever the sequence of edges in a graph is modified by either removing,
       * hiding, reinserting, or unhiding an edge, or by explicitly changing its position in the sequence.
       * </p>
       * @see yfiles.algorithms.Graph#removeEdge
       * @see yfiles.algorithms.Graph#hide
       * @see yfiles.algorithms.Graph#reInsertEdge
       * @see yfiles.algorithms.Graph#unhide
       * @see yfiles.algorithms.Graph#moveToFirst
       * @see yfiles.algorithms.Graph#moveToLast
       * @type {number}
       */
      index:number;
      /**
       * Gets the source node connected to this edge.
       * @see yfiles.algorithms.Edge#target
       * @type {yfiles.algorithms.Node}
       */
      source:yfiles.algorithms.Node;
      /**
       * Gets the target node connected to this edge.
       * @see yfiles.algorithms.Edge#source
       * @type {yfiles.algorithms.Node}
       */
      target:yfiles.algorithms.Node;
      /**
       * Gets <code>true</code> if and only if this edge is a self-loop.
       * <p>
       * An edge is called a self-loop, if it is adjacent to only one node, i.e., source node and target node are the same.
       * </p>
       * @type {boolean}
       */
      selfLoop:boolean;
      /**
       * Gets the successor of this edge in the list of outgoing edges at its source node.
       * <p>
       * If this edge is the last outgoing edge at its source node, then <code>null</code> is returned.
       * </p>
       * @see yfiles.algorithms.Edge#prevOutEdge
       * @see yfiles.algorithms.Edge#nextInEdge
       * @type {yfiles.algorithms.Edge}
       */
      nextOutEdge:yfiles.algorithms.Edge;
      /**
       * Gets the successor of this edge in the list of incoming edges at its target node.
       * <p>
       * If this edge is the last incoming edge at its target node, then <code>null</code> is returned.
       * </p>
       * @see yfiles.algorithms.Edge#prevInEdge
       * @see yfiles.algorithms.Edge#nextOutEdge
       * @type {yfiles.algorithms.Edge}
       */
      nextInEdge:yfiles.algorithms.Edge;
      /**
       * Gets the predecessor of this edge in the list of outgoing edges at its source node.
       * <p>
       * If this edge is the first outgoing edge at its source node, then <code>null</code> is returned.
       * </p>
       * @see yfiles.algorithms.Edge#nextOutEdge
       * @see yfiles.algorithms.Edge#prevInEdge
       * @type {yfiles.algorithms.Edge}
       */
      prevOutEdge:yfiles.algorithms.Edge;
      /**
       * Gets the predecessor of this edge in the list of incoming edges at its target node.
       * <p>
       * If this edge is the first incoming edge at its target node, then <code>null</code> is returned.
       * </p>
       * @see yfiles.algorithms.Edge#nextInEdge
       * @see yfiles.algorithms.Edge#prevOutEdge
       * @type {yfiles.algorithms.Edge}
       */
      prevInEdge:yfiles.algorithms.Edge;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specialized list implementation for instances of type {@link yfiles.algorithms.Edge}.
     * @class
     * @extends {yfiles.algorithms.YList}
     * @implements {yfiles.collections.IEnumerable.<yfiles.algorithms.Edge>}
     */
    export interface EdgeList extends yfiles.algorithms.YList{}
    export class EdgeList {
      /**
       * Creates a list that is initialized with an EdgeList.
       * @param edgeList
       */
      constructor(edgeList:yfiles.algorithms.EdgeList);
      /**
       * Creates a list that is initialized with a single edge provided.
       * @param e
       */
      constructor(e:yfiles.algorithms.Edge);
      /**
       * Creates a list that is initialized with those edges from the given EdgeCursor object for which the given data provider
       * returns <code>true</code> upon calling its {@link yfiles.algorithms.IDataProvider#getBoolean getBool} method.
       * @param ec An edge cursor providing edges that should be added to this list.
       * @param predicate A data provider that acts as a inclusion predicate for each edge accessible by the given edge cursor.
       */
      constructor(ec:yfiles.algorithms.IEdgeCursor,predicate:yfiles.algorithms.IDataProvider);
      /**
       * Creates a list that is initialized with the edges provided by the given array of edges.
       * @param a
       */
      constructor(a:yfiles.algorithms.Edge[]);
      /**
       * Creates a list that is initialized with the edges provided by the given EdgeCursor object.
       * @param c
       */
      constructor(c:yfiles.algorithms.IEdgeCursor);
      /**
       * Creates an empty edge list.
       */
      constructor();
      /**
       * Returns an edge cursor for this edge list.
       * @returns An edge cursor granting access to the edges within this list.
       */
      edges():yfiles.algorithms.IEdgeCursor;
      /**
       * Returns the first edge in this list, or <code>null</code> when the list is empty.
       * @returns The first edge in the list.
       */
      firstEdge():yfiles.algorithms.Edge;
      /**
       * Returns an enumerator for this collection.
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<yfiles.algorithms.Edge>;
      /**
       * Returns the last edge in this list, or <code>null</code> when the list is empty.
       * @returns The last edge in the list.
       */
      lastEdge():yfiles.algorithms.Edge;
      /**
       * Removes the first edge from this list and returns it.
       * @returns The first edge from the list.
       */
      popEdge():yfiles.algorithms.Edge;
      /**
       * Returns an edge array containing all elements of this list in the canonical order.
       * @returns 
       */
      toEdgeArray():yfiles.algorithms.Edge[];
      static $class:yfiles.lang.Class;
    }
    export enum BfsDirection{
      /**
       * An edge direction specifier for incoming edges.
       * @see yfiles.algorithms.Bfs#getLayers
       */
      PREDECESSOR,
      /**
       * An edge direction specifier for outgoing edges.
       * @see yfiles.algorithms.Bfs#getLayers
       */
      SUCCESSOR,
      /**
       * An edge direction specifier for both incoming and outgoing edges.
       * @see yfiles.algorithms.Bfs#getLayers
       */
      BOTH
    }
    export enum GraphElementInsertion{
      /**
       * Object insertion specifier. An object gets inserted before another one.
       */
      BEFORE,
      /**
       * Object insertion specifier. An object gets inserted after another one.
       */
      AFTER
    }
    export enum GraphEventType{
      /**
       * Type constant that identifies an event that gets fired immediately after a node has been created. The data of the event
       * is the newly created node.
       */
      NODE_CREATION,
      /**
       * Type constant that identifies an event that gets fired immediately after an edge has been created. The data of the event
       * is the newly created edge.
       */
      EDGE_CREATION,
      /**
       * Type constant that identifies an event that gets fired immediately before a node will be removed from the graph. The
       * data of the event is the node to be removed.
       */
      PRE_NODE_REMOVAL,
      /**
       * Type constant that identifies an event that gets fired immediately after a node has been removed from the graph. The
       * data of the event is the removed node.
       */
      POST_NODE_REMOVAL,
      /**
       * Type constant that identifies an event that gets fired immediately before an edge will be removed from the graph. The
       * data of the event is the edge to be removed.
       */
      PRE_EDGE_REMOVAL,
      /**
       * Type constant that identifies an event that gets fired immediately after an edge has been removed from the graph. The
       * data of the event is the removed edge.
       */
      POST_EDGE_REMOVAL,
      /**
       * Type constant that identifies an event that gets fired immediately after a node has been reinserted into the graph. The
       * data of the event is the reinserted node.
       */
      NODE_REINSERTION,
      /**
       * Type constant that identifies an event that gets fired immediately after an edge has been reinserted into the graph. The
       * data of the event is the reinserted edge.
       */
      EDGE_REINSERTION,
      /**
       * Type constant that identifies an event that gets fired immediately before the end points of an edge will be changed. The
       * data of the event is the edge to be redefined.
       */
      PRE_EDGE_CHANGE,
      /**
       * Type constant that identifies an event that gets fired immediately after the end points of an edge have been changed.
       * The data of the event is the redefined edge.
       */
      POST_EDGE_CHANGE,
      /**
       * Type constant that identifies an event that gets fired after a subgraph of a graph G has been moved to the emitting
       * graph. The data of the event is a {@link yfiles.algorithms.NodeList} containing the nodes that induce the moved subgraph.
       * <p>
       * This event gets fired just after the {@link yfiles.algorithms.GraphEventType#SUBGRAPH_REMOVAL} event got fired on the subgraph's
       * original graph G. Note that at the time the event gets fired, the nodes from the node list are already part of the
       * emitting graph.
       * </p>
       */
      SUBGRAPH_INSERTION,
      /**
       * Type constant that identifies an event that gets fired after a subgraph of the emitting graph has been moved to a graph
       * G. The data of the event is a {@link yfiles.algorithms.NodeList} containing the nodes that induce the moved subgraph.
       * <p>
       * This event gets fired just before the {@link yfiles.algorithms.GraphEventType#SUBGRAPH_INSERTION} event will be fired on the
       * subgraph's new graph G. Note that at the time the event gets fired, the nodes from the node list are already part of
       * graph G.
       * </p>
       */
      SUBGRAPH_REMOVAL,
      /**
       * Type constant that signals the start of a some logically coherent event sequence. If specified, the data of this event
       * is its ID.
       */
      PRE_EVENT,
      /**
       * Type constant that signals the end of a some logically coherent event sequence. If specified, the data of this event is
       * its ID.
       */
      POST_EVENT
    }
    export enum DistanceMetric{
      /**
       * A specifier for euclidean distance metric.
       * @see yfiles.algorithms.Groups#kMeansClustering
       */
      EUCLIDEAN,
      /**
       * A specifier for euclidean squared distance metric.
       * @see yfiles.algorithms.Groups#kMeansClustering
       */
      EUCLIDEAN_SQUARED,
      /**
       * A specifier for Manhattan distance metric.
       * @see yfiles.algorithms.Groups#kMeansClustering
       */
      MANHATTAN,
      /**
       * A specifier for Chebychev distance metric.
       * @see yfiles.algorithms.Groups#kMeansClustering
       */
      CHEBYCHEV
    }
    export enum Linkage{
      /**
       * A specifier for single-linkage clustering.
       * <p>
       * At the beginning, each element belongs to its own cluster. At each step of the clustering algorithm the two most
       * "similar" clusters are merged together until all nodes belong to the same cluster.
       * </p>
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       */
      SINGLE,
      /**
       * A specifier for complete-linkage clustering.
       * <p>
       * At the beginning, each element belongs to its own cluster. At each step of the clustering algorithm the two most
       * "dissimilar" clusters are merged and the distance equals to the distance of those two nodes that are farthest away from
       * each other. The algorithm continues until all nodes belong to the same cluster.
       * </p>
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       */
      COMPLETE,
      /**
       * A specifier for average-linkage clustering.
       * <p>
       * At the beginning, each element belongs to its own cluster. At each step of the clustering algorithm the two most
       * "similar" clusters are combined and the distance between them is defined as the average all distances between the nodes
       * that belong to the two clusters that are combined. The algorithm continues until all nodes belong to the same cluster.
       * </p>
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       * @see yfiles.algorithms.Groups#hierarchicalClustering
       */
      AVERAGE
    }
    /**
     * This class represents a line in the 2D-dimensional affine space.
     * <p>
     * The line is defined by the equation ax + by + c = 0
     * </p>
     * @class
     */
    export interface AffineLine extends Object{}
    export class AffineLine {
      /**
       * Creates an affine line which is defined by two points.
       * @param p1
       * @param p2
       */
      constructor(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint);
      /**
       * Creates an affine line which is defined by a point and a vector.
       * @param p1
       * @param v
       */
      constructor(p1:yfiles.algorithms.YPoint,v:yfiles.algorithms.YVector);
      /**
       * Returns the crossing of two lines.
       * <p>
       * If the lines are parallel, <code>null</code> is returned.
       * </p>
       * @param l1
       * @param l2
       * @returns 
       * @static
       */
      static getCrossing(l1:yfiles.algorithms.AffineLine,l2:yfiles.algorithms.AffineLine):yfiles.algorithms.YPoint;
      /**
       * Projects an point on the line in direction of the X-axis.
       * @param p
       * @returns 
       */
      getXProjection(p:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Projects an point on the line in direction of the Y-axis.
       * @param p
       * @returns 
       */
      getYProjection(p:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Gets a from ax+by+c = 0
       * @type {number}
       */
      a:number;
      /**
       * Gets b from ax+by+c = 0
       * @type {number}
       */
      b:number;
      /**
       * Gets c from ax+by+c = 0
       * @type {number}
       */
      c:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * The handle of a segment of a borderline.
     * @class
     */
    export interface BorderLineSegment extends Object{}
    export class BorderLineSegment {
      /**
       * Returns the segment's value at the given position.
       * <p>
       * Note: In case the position lies outside the segments range, the calculated value might be invalid. As the segment is not
       * aware of any offsets the position also must not include any offsets.
       * </p>
       * @param position the position the value is retrieved for.
       * @returns the segment's value at the given position.
       */
      getValueAt(position:number):number;
      /**
       * Returns the next segment or <code>null</code> if there is no such segment.
       * @returns 
       */
      next():yfiles.algorithms.BorderLineSegment;
      /**
       * Returns the previous segment or <code>null</code> if there is no such segment.
       * @returns 
       */
      prev():yfiles.algorithms.BorderLineSegment;
      /**
       * Gets the end of this segment.
       * @type {number}
       */
      end:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class can be used to easily model an orthogonal border line or sky-line.
     * <p>
     * It provides methods for measuring the distance between different BorderLine instances, merging multiple instances,
     * modifying and efficiently moving them around.
     * </p>
     * @class
     */
    export interface BorderLine extends Object{}
    export class BorderLine {
      /**
       * Creates a new BorderLine from a single segment.
       * @param min the beginning of this borderline
       * @param max the ending of this borderline
       * @param valueAtMin the value of the segment at the beginning of this borderline
       * @param valueAtMax the value of the segment at the ending of this borderline
       */
      constructor(min:number,max:number,valueAtMin:number,valueAtMax:number);
      /**
       * Creates a new BorderLine from a single segment.
       * @param min the beginning of this borderline
       * @param max the ending of this borderline
       * @param value the value of the segment
       */
      constructor(min:number,max:number,value:number);
      /**
       * Creates a new BorderLine with the given value from -Double.MAX_VALUE to Double.MAX_VALUE.
       * @param value the value of the segment
       */
      constructor(value:number);
      /**
       * Adds the given offset to the segments' positions.
       * <p>
       * This method has complexity O(1).
       * </p>
       * @param delta the delta to add to the positions
       */
      addOffset(delta:number):void;
      /**
       * Adds the given offset to the current values of the whole borderline.
       * <p>
       * This method has complexity O(1).
       * </p>
       * @param delta the delta to add to the values
       */
      addValueOffset(delta:number):void;
      /**
       * Convenience method that copies the actual data from the given argument to this instance.
       * @param other the argument to retrieve the values from
       */
      adoptValues(other:yfiles.algorithms.BorderLine):void;
      /**
       * Creates a copy of this borderline.
       * <p>
       * Optionally negates the values or offsets.
       * </p>
       * @param negateValues whether the values are negated
       * @param negateOffsets whether the offsets are negated
       * @returns the copy of the borderline
       */
      createCopy(negateValues:boolean,negateOffsets:boolean):yfiles.algorithms.BorderLine;
      /**
       * Merges this borderline with the given borderline using the "maximum" policy.
       * <p>
       * That means the resulting borderline will have greater value of both borderline on each position. If you imagine each
       * borderline as a the upper border of a plane, the resulting borderline will be the upper border of the merged planes.
       * </p>
       * @param other the other borderline
       * @returns a new borderline that is the result of the merge
       */
      createMax(other:yfiles.algorithms.BorderLine):yfiles.algorithms.BorderLine;
      /**
       * Merges this borderline with the given borderline using the "minimum" policy.
       * <p>
       * That means the resulting borderline will have smaller value of both borderline on each position. If you imagine each
       * borderline as a the lower border of a plane, the resulting borderline will be the lower border of the merged planes.
       * </p>
       * @param other the other borderline
       * @returns a new borderline that is the result of the merge
       */
      createMin(other:yfiles.algorithms.BorderLine):yfiles.algorithms.BorderLine;
      /**
       * Returns the first segment or <code>null</code> if there is no such segment.
       * @returns 
       */
      firstSegment():yfiles.algorithms.BorderLineSegment;
      /**
       * Calculates the minimal distance between this borderline and the other one.
       * <p>
       * The other one is treated as if the values were all greater.
       * </p>
       * @param greater
       * @returns 
       */
      getDistanceTo(greater:yfiles.algorithms.BorderLine):number;
      /**
       * Returns the maximum position of the given segment.
       * @param s the segment
       * @returns 
       */
      getMax(s:yfiles.algorithms.BorderLineSegment):number;
      /**
       * Calculates the maximum value in the interval from->to.
       * @param from
       * @param to
       * @returns 
       */
      getMaxValue(from:number,to:number):number;
      /**
       * Returns the minimum position of the given segment.
       * @param s the segment
       * @returns 
       */
      getMin(s:yfiles.algorithms.BorderLineSegment):number;
      /**
       * Calculates the minimum value in the interval from->to.
       * @param from
       * @param to
       * @returns 
       */
      getMinValue(from:number,to:number):number;
      /**
       * Returns the segment at the given position.
       * @param pos the position
       * @returns 
       */
      getSegmentAt(pos:number):yfiles.algorithms.BorderLineSegment;
      /**
       * Returns the slope of the given segment.
       * @param s the segment
       * @returns 
       */
      getSlope(s:yfiles.algorithms.BorderLineSegment):number;
      /**
       * Returns the value of the minimum of the given segment.
       * <p>
       * If the segment's slope is 0, it's the value of the whole segment. In case the slope differs from 0, it's the value of
       * the start of the slope.
       * </p>
       * @param s the segment
       * @returns 
       */
      getValue(s:yfiles.algorithms.BorderLineSegment):number;
      /**
       * Returns the value that is set on this borderline at the specified position.
       * <p>
       * The position must lie within the range of the segment.
       * </p>
       * @param segment The segment whose value shall be returned.
       * @param pos the position where the value will be retrieved.
       * @returns the value
       * @throws {Stubs.Exceptions.ArgumentError} if <code>pos</code> is outside the segment's range.
       */
      getValueAt(segment:yfiles.algorithms.BorderLineSegment,pos:number):number;
      /**
       * Returns the value that is set on this borderline at the specified position.
       * <p>
       * The position must lie within the range of the segment that is stored in <code>cell</code>.
       * </p>
       * @param cell The list cell containing the segment whose value shall be returned.
       * @param pos the position
       * @returns the value
       * @throws {Stubs.Exceptions.ArgumentError} if <code>pos</code> is outside the segment's range that is stored in <code>cell</code>.
       */
      getValueAt(cell:yfiles.algorithms.ListCell,pos:number):number;
      /**
       * Returns the value that is set on this borderline at the specified position.
       * @param pos the position
       * @returns the value
       * @throws {Stubs.Exceptions.IndexOutOfRangeError} if the position is outside of the borderline.
       */
      getValueAt(pos:number):number;
      /**
       * Grows this BorderLine horizontally, so that the {@link yfiles.algorithms.BorderLine#getValueAt values} of the BorderLine stay the same however their {@link yfiles.algorithms.BorderLine#getMin start} and {@link yfiles.algorithms.BorderLine#getMax end}
       * points are moved in the direction of <code>toMin</code> and <code>toMax</code>.
       * <p>
       * This is useful for scenarios where a BorderLine is needed that consists of an enlarged border.
       * </p>
       * <p>
       * Note that this method normalizes the segments, i.e., it transforms each segment with slope != 0 to a segment with slope
       * == 0.
       * </p>
       * @param toMin the delta by which the border should be extended towards -Infinity
       * @param toMax the delta by which the border should be extended towards +Infinity
       * @param positive whether the BorderLine should be interpreted to point in positive direction. This influences the direction into which a
       * segment's border is extended.
       */
      grow(toMin:number,toMax:number,positive:boolean):void;
      /**
       * Returns the last segment or <code>null</code> if there is no such segment.
       * @returns 
       */
      lastSegment():yfiles.algorithms.BorderLineSegment;
      /**
       * Merges this borderline with the given borderline using the "maximum" policy.
       * @param other the other borderline
       */
      mergeWithMax(other:yfiles.algorithms.BorderLine):void;
      /**
       * Merges this borderline with the given borderline using the "minimum" policy.
       * @param other the other borderline
       */
      mergeWithMin(other:yfiles.algorithms.BorderLine):void;
      /**
       * Returns the next segment or <code>null</code> if there is no such segment.
       * @param s
       * @returns 
       */
      next(s:yfiles.algorithms.BorderLineSegment):yfiles.algorithms.BorderLineSegment;
      /**
       * Returns the previous segment or <code>null</code> if there is no such segment.
       * @param s
       * @returns 
       */
      prev(s:yfiles.algorithms.BorderLineSegment):yfiles.algorithms.BorderLineSegment;
      /**
       * Assures that all values in the given interval are greater or equal than the given value.
       * @param min the lower end of the interval
       * @param max the upper end of the interval
       * @param value the smallest possible value for the interval
       */
      setMaxValue(min:number,max:number,value:number):void;
      /**
       * Assures that all values in the given interval are less or equal than the given value.
       * @param min the lower end of the interval
       * @param max the upper end of the interval
       * @param value the greatest possible value for the interval
       */
      setMinValue(min:number,max:number,value:number):void;
      /**
       * Sets a specific interval to a slope starting at a given value.
       * @param min the left side of the interval.
       * @param max the right side of the interval.
       * @param value the value at <code>min</code> where the slope starts.
       * @param slope the slope of the segment in the given interval.
       * @throws {Stubs.Exceptions.ArgumentError} if <code>min</code> is greater than <code>max</code>.
       */
      setSloped(min:number,max:number,value:number,slope:number):void;
      /**
       * Sets a specific interval described by <code>min</code> and <code>max</code> to a given value.
       * @param min the left side of the interval.
       * @param max the right side of the interval.
       * @param value the value for the whole interval.
       */
      setValue(min:number,max:number,value:number):void;
      /**
       * Gets the smallest position of this borderline
       * @type {number}
       */
      min:number;
      /**
       * Gets the greatest position of this borderline
       * @type {number}
       */
      max:number;
      /**
       * Gets the minimum value that is set on this borderline
       * @type {number}
       */
      minValue:number;
      /**
       * Gets the maximum value that is set on this borderline
       * @type {number}
       */
      maxValue:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides useful geometric primitives and advanced geometric algorithms.
     * @class
     * @static
     */
    export interface Geom extends Object{}
    export class Geom {
      /**
       * Calculates the convex hull for a set of points.
       * @param points a list of {@link } objects
       * @returns a list of {@link } objects that constitute the convex hull of the given points. The list contains points
       * in counter clockwise order around the hull. The first point is the one with the smallest <code>x</code> coordinate. If two such
       * points exist then of these points the one with the smallest <code>y</code> coordinate is chosen as the first one.
       * @static
       */
      static calcConvexHull(points:yfiles.algorithms.YList):yfiles.algorithms.YList;
      /**
       * Calculates the intersection point of two <b>affine</b> lines.
       * <p>
       * Each line is given by two points.
       * </p>
       * @param p1 one point on the first line.
       * @param p2 another point on the first line.
       * @param p3 one point on the second line.
       * @param p4 another point on the second line.
       * @returns the intersection point of the specified lines or <code>null</code> if there is no intersection.
       * @static
       */
      static calcIntersection(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint,p3:yfiles.algorithms.YPoint,p4:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Calculates the intersection point of two affine lines.
       * <p>
       * Each line is given by a point and a direction vector.
       * </p>
       * @param p1 origin point of the first line.
       * @param d1 direction vector of the first line.
       * @param p2 origin point of the second line.
       * @param d2 direction vector of the second line.
       * @returns the intersection point of the specified lines or <code>null</code> if there is no intersection.
       * @static
       */
      static calcIntersection(p1:yfiles.algorithms.YPoint,d1:yfiles.algorithms.YVector,p2:yfiles.algorithms.YPoint,d2:yfiles.algorithms.YVector):yfiles.algorithms.YPoint;
      /**
       * Calculates the intersection point of two <b>affine</b> lines.
       * <p>
       * Each line is given by the coordinates of two points.
       * </p>
       * @param x1 x-coordinate of one point on the first line.
       * @param y1 y-coordinate of one point on the first line.
       * @param x2 x-coordinate of another point on the first line.
       * @param y2 y-coordinate of another point on the first line.
       * @param x3 x-coordinate of one point on the second line.
       * @param y3 y-coordinate of one point on the second line.
       * @param x4 x-coordinate of another point on the second line.
       * @param y4 y-coordinate of another point on the second line.
       * @returns the intersection point of the specified lines or <code>null</code> if there is no intersection.
       * @static
       */
      static calcIntersection(x1:number,y1:number,x2:number,y2:number,x3:number,y3:number,x4:number,y4:number):yfiles.algorithms.YPoint;
      /**
       * Intersects the pair of specified source <code>Rectangle2D</code> objects and puts the result into the specified destination <code>Rectangle2D</code>
       * object.
       * <p>
       * If one or both of the source rectangles have negative width or height, the resulting rectangle will be located at (0,0)
       * with a width and height of -1. One of the source rectangles can also be the destination to avoid creating a third
       * Rectangle2D object, but in this case the original points of this source rectangle will be overwritten by this method.
       * </p>
       * @param r1 the first of a pair of <code>Rectangle2D</code> objects to be intersected with each other
       * @param r2 the second of a pair of <code>Rectangle2D</code> objects to be intersected with each other
       * @param dest the <code>Rectangle2D</code> that holds the results of the intersection of <code>r1</code> and <code>r2</code>
       * @returns 
       * @static
       */
      static calcIntersection(r1:yfiles.algorithms.Rectangle2D,r2:yfiles.algorithms.Rectangle2D,dest:yfiles.algorithms.Rectangle2D):yfiles.algorithms.Rectangle2D;
      /**
       * Unions the pair of source <code>Rectangle2D</code> objects and puts the result into the specified destination <code>Rectangle2D</code> object.
       * <p>
       * If one of the source rectangles has negative width or height, it is excluded from the union. If both source rectangles
       * have negative width or height, the destination rectangle will become a copy of <code>r1</code>. One of the source rectangles can also
       * be the destination to avoid creating a third Rectangle2D object, but in this case the original points of this source
       * rectangle will be overwritten by this method. If the destination is <code>null</code>, a new <code>Rectangle2D</code> is created.
       * </p>
       * @param r1 the first of a pair of <code>Rectangle2D</code> objects to be combined with each other
       * @param r2 the second of a pair of <code>Rectangle2D</code> objects to be combined with each other
       * @param dest the <code>Rectangle2D</code> that holds the results of the union of <code>r1</code> and <code>r2</code>
       * @returns 
       * @static
       */
      static calcUnion(r1:yfiles.algorithms.Rectangle2D,r2:yfiles.algorithms.Rectangle2D,dest:yfiles.algorithms.Rectangle2D):yfiles.algorithms.Rectangle2D;
      /**
       * Returns <code>true</code> iff the given points are collinear, i.e.
       * <p>
       * all three points lie on a common line.
       * </p>
       * <p>
       * Same as {@link yfiles.algorithms.Geom#orientation orientation(p,q,r) == 0}
       * </p>
       * @param p
       * @param q
       * @param r
       * @returns 
       * @static
       */
      static collinear(p:yfiles.algorithms.YPoint,q:yfiles.algorithms.YPoint,r:yfiles.algorithms.YPoint):boolean;
      /**
       * Determines the distance of the point <code>p</code> to the line segment <code>[l1, l2]</code>.
       * @param pointX the x coordinate of p
       * @param pointY the y coordinate of p
       * @param lineX1 the x coordinate of l1
       * @param lineY1 the y coordinate of l1
       * @param lineX2 the x coordinate of l2
       * @param lineY2 the y coordinate of l2
       * @returns 
       * @static
       */
      static distanceToLineSegment(pointX:number,pointY:number,lineX1:number,lineY1:number,lineX2:number,lineY2:number):number;
      /**
       * Same as {@link yfiles.algorithms.Geom#orientation orientation(p,q,r) > 0}
       * @param p
       * @param q
       * @param r
       * @returns 
       * @static
       */
      static leftTurn(p:yfiles.algorithms.YPoint,q:yfiles.algorithms.YPoint,r:yfiles.algorithms.YPoint):boolean;
      /**
       * Returns whether the two lines defined by the given coordinates intersect or not.
       * @param x1
       * @param y1
       * @param x2
       * @param y2
       * @param x3
       * @param y3
       * @param x4
       * @param y4
       * @returns 
       * @static
       */
      static linesIntersect(x1:number,y1:number,x2:number,y2:number,x3:number,y3:number,x4:number,y4:number):boolean;
      /**
       * Same as {@link yfiles.algorithms.Geom#orientation} with double values as arguments.
       * @param px
       * @param py
       * @param qx
       * @param qy
       * @param rx
       * @param ry
       * @returns 
       * @static
       */
      static orientation(px:number,py:number,qx:number,qy:number,rx:number,ry:number):number;
      /**
       * Returns the orientation of point <code>r</code> relative to the directed line from point <code>p</code> to point <code>q</code>.
       * <p>
       * The given tuple of points is said to have positive orientation if <code>p</code> and <code>q</code> are distinct and <code>r</code> lies to the left of the
       * oriented line passing through <code>p</code> and <code>q</code> and oriented from <code>p</code> to <code>q</code>.
       * </p>
       * <p>
       * The tuple is said to have negative orientation if <code>p</code> and <code>q</code> are distinct and <code>r</code> lies to the right of the line, and the
       * tuple is said to have orientation zero if the three points are collinear.
       * </p>
       * @param p
       * @param q
       * @param r
       * @returns <b>+1</b> in the case of positive orientation, <b>-1</b> in the case of negative orientation and <b>0</b> in the case of zero orientation.
       * @static
       */
      static orientation(p:yfiles.algorithms.YPoint,q:yfiles.algorithms.YPoint,r:yfiles.algorithms.YPoint):number;
      /**
       * Determines the projection of the point <code>p</code> onto the line segment <code>[l1, l2]</code>.
       * <p>
       * The resulting point is
       * </p>
       * <ul>
       * <li>the orthogonal projection of <code>p</code> onto the line through <code>l1</code> and <code>l2</code>, iff the projection lies on the line segment <code>[l1, l2]</code></li>
       * <li>the end point of the line segment <code>[l1, l2]</code> that is closest to <code>p</code>, otherwise</li>
       * </ul>
       * @param pointX the x coordinate of p
       * @param pointY the y coordinate of p
       * @param lineX1 the x coordinate of l1
       * @param lineY1 the y coordinate of l1
       * @param lineX2 the x coordinate of l2
       * @param lineY2 the y coordinate of l2
       * @returns 
       * @static
       */
      static projection(pointX:number,pointY:number,lineX1:number,lineY1:number,lineX2:number,lineY2:number):yfiles.algorithms.YPoint;
      /**
       * Same as {@link yfiles.algorithms.Geom#orientation orientation(p,q,r) &lt; 0}
       * @param p
       * @param q
       * @param r
       * @returns 
       * @static
       */
      static rightTurn(p:yfiles.algorithms.YPoint,q:yfiles.algorithms.YPoint,r:yfiles.algorithms.YPoint):boolean;
      /**
       * Returns +1 if point <code>d</code> lies left of the directed circle through points <code>a</code>, <code>b</code>, and <code>c</code>, 0 if <code>a,b,c</code> and <code>d</code> are cocircular, and
       * -1 otherwise.
       * @param a
       * @param b
       * @param c
       * @param d
       * @returns 
       * @static
       */
      static sideOfCircle(a:yfiles.algorithms.YPoint,b:yfiles.algorithms.YPoint,c:yfiles.algorithms.YPoint,d:yfiles.algorithms.YPoint):number;
      /**
       * Converts the given degree value from radian to angular
       * @param angrad
       * @returns 
       * @static
       */
      static toDegrees(angrad:number):number;
      /**
       * Converts the given degree value from angular to radian.
       * @param angdeg
       * @returns 
       * @static
       */
      static toRadians(angdeg:number):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is an interface for a sequence of instances of LineSegment.
     * @interface
     * @implements {yfiles.algorithms.ICursor}
     */
    export interface ILineSegmentCursor extends Object,yfiles.algorithms.ICursor{
      /**
       * Gets the instance of LineSegment the cursor is currently pointing on.
       * @abstract
       * @type {yfiles.algorithms.LineSegment}
       */
      lineSegment:yfiles.algorithms.LineSegment;
    }
    var ILineSegmentCursor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface describes a 2-dimensional object which has a finite bounding box.
     * @interface
     */
    export interface IPlaneObject extends Object{
      /**
       * Gets the smallest Rectangle which contains the object.
       * @abstract
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
    }
    var IPlaneObject:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is an interface for a sequence of instances of YPoint.
     * @interface
     * @implements {yfiles.algorithms.ICursor}
     */
    export interface IPointCursor extends Object,yfiles.algorithms.ICursor{
      /**
       * Gets the instance of YPoint the cursor is currently pointing on.
       * @abstract
       * @type {yfiles.algorithms.YPoint}
       */
      point:yfiles.algorithms.YPoint;
    }
    var IPointCursor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Double-precision immutable insets representation.
     * @class
     * @implements {yfiles.lang.ICloneable}
     */
    export interface Insets extends Object,yfiles.lang.ICloneable{}
    export class Insets {
      /**
       * Creates a new instance with the given values
       * @param top The new top inset value
       * @param left The new left inset value
       * @param bottom The new bottom inset value
       * @param right The new right inset value
       */
      constructor(top:number,left:number,bottom:number,right:number);
      /**
       * 
       * @returns 
       */
      clone():Object;
      /**
       * Creates a {@link yfiles.geometry.Insets} from a given {@link yfiles.algorithms.Insets}.
       * @returns The {@link }.
       */
      toInsets():yfiles.geometry.Insets;
      /**
       * The inset from the top.
       * @const
       * @type {number}
       */
      top:number;
      /**
       * The inset from the left.
       * @const
       * @type {number}
       */
      left:number;
      /**
       * The inset from the bottom.
       * @const
       * @type {number}
       */
      bottom:number;
      /**
       * The inset from the right.
       * @const
       * @type {number}
       */
      right:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An instance of this interface handles intersections found by the <code>IntersectionAlgorithm</code>,
     * @interface
     */
    export interface IIntersectionHandler extends Object{
      /**
       * This method is called at every intersection.
       * @param a
       * @param b
       * @abstract
       */
      checkIntersection(a:Object,b:Object):void;
    }
    var IIntersectionHandler:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class calculates the intersection of rectangles in the plane with the help of a sweep-line algorithm.
     * <p>
     * <br /> The complexity is <code>O(n log n + s)</code> where <code>n</code> is the number of rectangles and <code>s</code> the number of intersections.
     * </p>
     * @class
     * @static
     */
    export interface IntersectionAlgorithm extends Object{}
    export class IntersectionAlgorithm {
      /**
       * Calculates the intersections of rectangles in the plane.
       * <p>
       * Every found intersection is reported to an
       * <code>IntersectionHandler</code>. Rectangles with negative size are completely ignored by this implementation (i.e. never generate
       * intersections)
       * </p>
       * @param objects a list of <code>PlaneObject</code> objects.
       * @param iHandler intersections are reported to this class.
       * @static
       */
      static intersect(objects:yfiles.algorithms.YList,iHandler:yfiles.algorithms.IIntersectionHandler):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a line segment in the plane.
     * <p>
     * A line segment is defined by its two end points.
     * </p>
     * @class
     * @implements {yfiles.algorithms.IPlaneObject}
     */
    export interface LineSegment extends Object,yfiles.algorithms.IPlaneObject{}
    export class LineSegment {
      /**
       * Returns a new LineSegment.
       * @param p1 the first end point of the line segment.
       * @param p2 the second end point of the line segment.
       */
      constructor(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint);
      /**
       * Checks whether a line segment intersects a paraxial box.
       * <p>
       * Implemented using the Cohen-Sutherland algorithm.
       * </p>
       * @param boxX1 x-coordinate of the upper left corner of the box
       * @param boxY1 y-coordinate of the upper left corner of the box
       * @param boxX2 x-coordinate of the lower right corner of the box
       * @param boxY2 y-coordinate of the lower right corner of the box
       * @param sX1 x-coordinate of the first end point of the line segment
       * @param sY1 y-coordinate of the first end point of the line segment
       * @param sX2 x-coordinate of the second end point of the line segment
       * @param sY2 y-coordinate of the second end point of the line segment
       * @returns <code>true</code> if the line segment intersects the box, <code>false</code> otherwise.
       * @static
       */
      static boxIntersectsSegment(boxX1:number,boxY1:number,boxX2:number,boxY2:number,sX1:number,sY1:number,sX2:number,sY2:number):boolean;
      /**
       * Checks whether a line segment intersects a box.
       * <p>
       * Implemented using the Cohen-Sutherland algorithm.
       * </p>
       * @param box A rectangle
       * @param x1 x-coordinate of the first end point of the line segment
       * @param y1 y-coordinate of the first end point of the line segment
       * @param x2 x-coordinate of the second end point of the line segment
       * @param y2 y-coordinate of the second end point of the line segment
       * @returns <code>true</code> if the line segment intersects the box, <code>false</code> otherwise.
       * @static
       */
      static boxIntersectsSegment(box:yfiles.algorithms.YRectangle,x1:number,y1:number,x2:number,y2:number):boolean;
      /**
       * Checks whether a line segment intersects a box.
       * @param box A rectangle.
       * @param s first end point of the line segment.
       * @param t second end point of the line segment.
       * @returns <code>true</code> if the line segments intersects the box, <code>false</code> otherwise.
       * @static
       */
      static boxIntersectsSegment(box:yfiles.algorithms.YRectangle,s:yfiles.algorithms.YPoint,t:yfiles.algorithms.YPoint):boolean;
      /**
       * Checks whether a given point lies on this line segment.
       * @param point an arbitrary point.
       * @returns <code>true</code> if the line segments intersects the box, <code>false</code> otherwise.
       */
      contains(point:yfiles.algorithms.YPoint):boolean;
      /**
       * Returns intersection point between the two line segments, if there is one or <code>null</code> if the two line segments do not
       * intersect.
       * @param s1 first line segment
       * @param s2 second line segment
       * @returns 
       * @static
       */
      static getIntersection(s1:yfiles.algorithms.LineSegment,s2:yfiles.algorithms.LineSegment):yfiles.algorithms.YPoint;
      /**
       * Checks whether the line segment intersects a point.
       * @param p a point
       * @returns <code>true</code> if the line segments intersects the given point, <code>false</code> otherwise.
       */
      intersects(p:yfiles.algorithms.YPoint):boolean;
      /**
       * Checks whether the line segment intersects a box.
       * @param box A rectangle.
       * @returns <code>true</code> if the line segments intersects the box, <code>false</code> otherwise.
       */
      intersects(box:yfiles.algorithms.YRectangle):boolean;
      /**
       * Determines if the given points define a horizontal line segment.
       * <p>
       * This methods allows for y-coordinate differences of up to 1e-8.
       * </p>
       * @param p1
       * @param p2
       * @returns <code>true</code> if the given points define a horizontal line segment; <code>false</code> otherwise.
       * @static
       */
      static isHorizontalSegment(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint):boolean;
      /**
       * Returns if the projection on the X axis of the line segment covers a certain point on the X Axis.
       * @param x
       * @returns 
       */
      isInXIntervall(x:number):boolean;
      /**
       * Returns if the projection on the Y axis of the line segment covers a certain point on the Y Axis.
       * @param y
       * @returns 
       */
      isInYIntervall(y:number):boolean;
      /**
       * Determines if the given points define a vertical line segment.
       * <p>
       * This methods allows for x-coordinate differences of up to 1e-8.
       * </p>
       * @param p1
       * @param p2
       * @returns <code>true</code> if the given points define a vertical line segment; <code>false</code> otherwise.
       * @static
       */
      static isVerticalSegment(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint):boolean;
      /**
       * Returns the length of the line segment, this is the value of the Euclidean norm.
       * @returns an value > 0.
       */
      length():number;
      /**
       * Returns the affine line defined by the end points of the line segment.
       * @returns 
       */
      toAffineLine():yfiles.algorithms.AffineLine;
      /**
       * Returns the vector pointing from the first end point to the second end point of the line segment.
       * @returns 
       */
      toYVector():yfiles.algorithms.YVector;
      /**
       * Gets if this segment is considered vertical, i.e.
       * <p>
       * the x values of the end point differ less then 0.00000001
       * </p>
       * @type {boolean}
       */
      isVertical:boolean;
      /**
       * Gets if the interval is horizontal.
       * @type {boolean}
       */
      isHorizontal:boolean;
      /**
       * Gets the first end point of the line segment.
       * @type {yfiles.algorithms.YPoint}
       */
      firstEndPoint:yfiles.algorithms.YPoint;
      /**
       * Gets the second end point of the line segment.
       * @type {yfiles.algorithms.YPoint}
       */
      secondEndPoint:yfiles.algorithms.YPoint;
      /**
       * Gets the y value of the line on x coordinate 0.
       * @type {number}
       */
      xOffset:number;
      /**
       * Gets the slope of the line segment.
       * @type {number}
       */
      slope:number;
      /**
       * Gets the smallest Rectangle which contains the object.
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
      /**
       * Gets the distance from start to end point in x-coordinates.
       * @type {number}
       */
      deltaX:number;
      /**
       * Gets the distance from start to end point in y-coordinates.
       * @type {number}
       */
      deltaY:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a two-dimensional point located at <code>(x, y)</code>.
     * @class
     * @implements {yfiles.lang.ICloneable}
     */
    export interface Point2D extends Object,yfiles.lang.ICloneable{}
    export class Point2D {
      /**
       * Creates a new instance at the given location.
       * @param x The x coordinate of the new point
       * @param y The x coordinate of the new point
       */
      constructor(x:number,y:number);
      /**
       * Creates a new instance that is located at <code>(0, 0)</code>
       */
      constructor();
      /**
       * Creates a new instance of this class that has the same values as this objects.
       * @returns a new instance of this class that has the same values as this objects.
       */
      clone():Object;
      /**
       * Calculates the distance between this points and another one, given as its coordinates.
       * @param px The x coordinate of the second point.
       * @param py The y coordinate of the second point.
       * @returns the distance between this points and another one, given as its coordinates.
       */
      distance(px:number,py:number):number;
      /**
       * Calculates the distance between this points and another one.
       * @param p The second point.
       * @returns the distance between this points and another one.
       */
      distance(p:yfiles.algorithms.Point2D):number;
      /**
       * Calculates the distance between two points, given as their coordinates.
       * @param x1 The x coordinate of the first point.
       * @param y1 The y coordinate of the first point.
       * @param x2 The x coordinate of the second point.
       * @param y2 The y coordinate of the second point.
       * @returns the distance between two points, given as their coordinates.
       * @static
       */
      static distance(x1:number,y1:number,x2:number,y2:number):number;
      /**
       * Calculates the squared distance between this points and another one, given as its coordinates.
       * @param px The x coordinate of the second point.
       * @param py The y coordinate of the second point.
       * @returns the squared distance between this points and another one, given as its coordinates.
       */
      distanceSq(px:number,py:number):number;
      /**
       * Calculates the squared distance between this points and another one.
       * @param p The second point.
       * @returns the squared distance between this points and another one.
       */
      distanceSq(p:yfiles.algorithms.Point2D):number;
      /**
       * Calculates the squared distance between two points, given as their coordinates.
       * @param x1 The x coordinate of the first point.
       * @param y1 The y coordinate of the first point.
       * @param x2 The x coordinate of the second point.
       * @param y2 The y coordinate of the second point.
       * @returns the squared distance between two points, given as their coordinates.
       * @static
       */
      static distanceSq(x1:number,y1:number,x2:number,y2:number):number;
      /**
       * Sets the location of this point to the given coordinates.
       * @param x the new x coordinate
       * @param y the new y coordinate
       */
      setLocation(x:number,y:number):void;
      /**
       * Creates a {@link yfiles.geometry.Point} from a given {@link yfiles.algorithms.Point2D}.
       * @returns The {@link }.
       */
      toPoint():yfiles.geometry.Point;
      /**
       * Sets the location of this point to the same values as for the given point.
       * @type {yfiles.algorithms.Point2D}
       */
      location:yfiles.algorithms.Point2D;
      /**
       * Gets the x coordinate of this point.
       * @type {number}
       */
      x:number;
      /**
       * Gets the y coordinate of this point.
       * @type {number}
       */
      y:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a two-dimensional rectangle of size <code>(width x height)</code>, located at the point <code>(x, y)</code>.
     * @class
     * @implements {yfiles.lang.ICloneable}
     */
    export interface Rectangle2D extends Object,yfiles.lang.ICloneable{}
    export class Rectangle2D {
      /**
       * Creates a new rectangle of the specified size at the specified location.
       * @param x the x-coordinate of the upper left corner of the created rectangle.
       * @param y the y-coordinate of the upper left corner of the created rectangle.
       * @param width the width of the created rectangle.
       * @param height the height of the created rectangle.
       */
      constructor(x:number,y:number,width:number,height:number);
      /**
       * Creates a new rectangle of size <code>(0 x 0)</code> at the location <code>(0, 0)</code>.
       */
      constructor();
      /**
       * Adds a point, specified by its coordinates, to this rectangle.
       * <p>
       * The rectangle will be grown if necessary. Note that for points that would lie on the right or bottom border of the
       * rectangle, {@link yfiles.algorithms.Rectangle2D#contains} will still return <code>false</code> for the added point.
       * </p>
       * @param px the x coordinate of the point to add
       * @param py the y coordinate of the point to add
       */
      add(px:number,py:number):void;
      /**
       * Adds a rectangle to this rectangle.
       * <p>
       * The rectangle will be grown to the union of both rectangles
       * </p>
       * @param r the rectangle to add
       */
      add(r:yfiles.algorithms.Rectangle2D):void;
      /**
       * Creates a new instance of this class that has the same values as this objects.
       * @returns a new instance of this class that has the same values as this objects.
       */
      clone():Object;
      /**
       * Checks whether this rectangle completely contains the second specified rectangle.
       * @param x the x coordinate of the second rectangle.
       * @param y the y coordinate of the second rectangle.
       * @param width the width coordinate of the second rectangle.
       * @param height the height coordinate of the second rectangle.
       * @returns <code>true</code> if this rectangle contains the second one
       */
      contains(x:number,y:number,width:number,height:number):boolean;
      /**
       * Checks if the point specified by the given coordinates is contained in this rectangle.
       * @param x the x-coordinate of the point.
       * @param y the y-coordinate of the point.
       * @returns <code>true</code> if the specified point is contained in this rectangle; <code>false</code> otherwise.
       */
      contains(x:number,y:number):boolean;
      /**
       * Checks whether this rectangle completely contains the second specified rectangle.
       * @param rect the rectangle to check for containment
       * @returns <code>true</code> if this rectangle contains the second one
       */
      contains(rect:yfiles.algorithms.Rectangle2D):boolean;
      /**
       * Calculates the intersection of this rectangle with the given rectangle and returns the result as new rectangle.
       * @param r a rectangle to intersect with this rectangle.
       * @returns a new rectangle that represents the calculated intersection.
       */
      createIntersection(r:yfiles.algorithms.Rectangle2D):yfiles.algorithms.Rectangle2D;
      /**
       * Calculates the union of this rectangle with the given rectangle and returns the result as new rectangle.
       * @param r a rectangle to union with this rectangle.
       * @returns a new rectangle that represents the calculated union.
       */
      createUnion(r:yfiles.algorithms.Rectangle2D):yfiles.algorithms.Rectangle2D;
      /**
       * Calculates the intersection of the first and second specified rectangle and sets the resulting location and size to the
       * third rectangle.
       * @param src1 the first rectangle to intersect.
       * @param src2 the second rectangle to intersect.
       * @param dst the rectangle to which the result is set.
       * @static
       */
      static intersect(src1:yfiles.algorithms.Rectangle2D,src2:yfiles.algorithms.Rectangle2D,dst:yfiles.algorithms.Rectangle2D):void;
      /**
       * Checks whether this rectangle and the second specified rectangle intersect.
       * @param x the x coordinate of the second rectangle.
       * @param y the y coordinate of the second rectangle.
       * @param width the width coordinate of the second rectangle.
       * @param height the height coordinate of the second rectangle.
       * @returns <code>true</code> if both rectangles intersect
       */
      intersects(x:number,y:number,width:number,height:number):boolean;
      /**
       * Checks whether this rectangle and the second specified rectangle intersect.
       * @param rect the rectangle to check for intersection
       * @returns <code>true</code> if both rectangles intersect
       */
      intersects(rect:yfiles.algorithms.Rectangle2D):boolean;
      /**
       * Checks if the line segment specified by the given coordinates intersects this rectangle.
       * @param x1 the x-coordinate of the first end point of the line.
       * @param y1 the y-coordinate of the first end point of the line.
       * @param x2 the x-coordinate of the other end point of the line.
       * @param y2 the y-coordinate of the other end point of the line.
       * @returns <code>true</code> if the specified line intersects this rectangle; <code>false</code> otherwise.
       */
      intersectsLine(x1:number,y1:number,x2:number,y2:number):boolean;
      /**
       * Sets the outer bounds of this rectangle based on the specified location and size.
       * @param x the new x-coordinate of the upper left corner.
       * @param y the new y-coordinate of the upper left corner.
       * @param width the new width.
       * @param height the new height.
       */
      setFrame(x:number,y:number,width:number,height:number):void;
      /**
       * Sets the outer bounds of this rectangle based on the specified new center and corner.
       * @param centerX the x-coordinate of the new center of this rectangle.
       * @param centerY the y-coordinate of the new center of this rectangle.
       * @param cornerX the x-coordinate of any new corner of this rectangle.
       * @param cornerY the y-coordinate of any new corner of this rectangle.
       */
      setFrameFromCenter(centerX:number,centerY:number,cornerX:number,cornerY:number):void;
      /**
       * Sets the outer bounds of this rectangle based on the specified new center and corner.
       * @param center the new center of this rectangle.
       * @param corner any new corner of this rectangle.
       */
      setFrameFromCenter(center:yfiles.algorithms.Point2D,corner:yfiles.algorithms.Point2D):void;
      /**
       * Sets the outer bounds of this rectangle based on the end points of one of its diagonals.
       * @param x1 the x-coordinate of the first end point of a diagonal.
       * @param y1 the y-coordinate of the first end point of a diagonal.
       * @param x2 the x-coordinate of the other end point of a diagonal.
       * @param y2 the y-coordinate of the other end point of a diagonal.
       */
      setFrameFromDiagonal(x1:number,y1:number,x2:number,y2:number):void;
      /**
       * Sets the outer bounds of this rectangle based on the end points of one of its diagonals.
       * @param p1 the first end point of a diagonal.
       * @param p2 the other end point of a diagonal.
       */
      setFrameFromDiagonal(p1:yfiles.algorithms.Point2D,p2:yfiles.algorithms.Point2D):void;
      /**
       * Sets the location and size of this rectangle to the specified values.
       * @param x the new x-coordinate of the upper left corner.
       * @param y the new y-coordinate of the upper left corner.
       * @param width the new width.
       * @param height the new height.
       */
      setRect(x:number,y:number,width:number,height:number):void;
      /**
       * Creates a {@link yfiles.geometry.Rect} from a given {@link yfiles.algorithms.Rectangle2D}.
       * @returns The {@link }.
       */
      toRect():yfiles.geometry.Rect;
      /**
       * Calculates the union of the first and second specified rectangle and sets the resulting location and size to the third
       * rectangle.
       * @param src1 the first rectangle to union.
       * @param src2 the second rectangle to union.
       * @param dst the rectangle to which the result is set.
       * @static
       */
      static union(src1:yfiles.algorithms.Rectangle2D,src2:yfiles.algorithms.Rectangle2D,dst:yfiles.algorithms.Rectangle2D):void;
      /**
       * Gets or sets the outer bounds of this rectangle in double coordinates.
       * @type {yfiles.algorithms.Rectangle2D}
       */
      frame:yfiles.algorithms.Rectangle2D;
      /**
       * Sets the location and size of this rectangle to be similar to the specified rectangle.
       * @type {yfiles.algorithms.Rectangle2D}
       */
      rect:yfiles.algorithms.Rectangle2D;
      /**
       * Gets the x coordinate of the upper left corner.
       * @type {number}
       */
      x:number;
      /**
       * Gets the y coordinate of the upper left corner.
       * @type {number}
       */
      y:number;
      /**
       * Gets the width of this rectangle.
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of this rectangle.
       * @type {number}
       */
      height:number;
      /**
       * Gets whether this instance is empty, i.e.
       * <p>
       * covers no area.
       * </p>
       * @type {boolean}
       */
      empty:boolean;
      /**
       * Gets the smallest x coordinate of this rectangle.
       * @type {number}
       */
      minX:number;
      /**
       * Gets the smallest y coordinate of this rectangle.
       * @type {number}
       */
      minY:number;
      /**
       * Gets the largest x coordinate of this rectangle.
       * @type {number}
       */
      maxX:number;
      /**
       * Gets the largest y coordinate of this rectangle.
       * @type {number}
       */
      maxY:number;
      /**
       * Gets the x coordinate of the center point of this rectangle.
       * @type {number}
       */
      centerX:number;
      /**
       * Gets the y coordinate of the center point of this rectangle.
       * @type {number}
       */
      centerY:number;
      /**
       * Gets the bounds of this instance in double precision.
       * @type {yfiles.algorithms.Rectangle2D}
       */
      bounds2D:yfiles.algorithms.Rectangle2D;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides algorithms for the triangulation of point sets in the plane.
     * @class
     * @static
     */
    export interface Triangulator extends Object{}
    export class Triangulator {
      /**
       * Computes a Delauney triangulation of the given points.
       * <p>
       * A Delauney triangulation is a triangulation such that none of the given points is inside the circumcircle of any of the
       * calculated triangles.
       * </p>
       * <p>
       * The calculated triangulation is represented by an embedded graph, i.e. to each edge there exists a reverse edge and the
       * outedges around each node are in embedded order. The returned edge and the (optional) reverseEdgeMap can be used to
       * construct all faces of the plane graph and to determine its outer face.
       * </p>
       * @param result a graph whose nodes represent the points that need to be triangulated.
       * @param pointData must provide the location (YPoint) for each node in the given graph.
       * @param revMap a node map that will contain for each edge its reverse edge. If this argument is <code>null</code> then no reverse edge information
       * will be available.
       * @returns an edge on the outer face of the result graph.
       * @static
       */
      static calcDelauneyTriangulation(result:yfiles.algorithms.Graph,pointData:yfiles.algorithms.IDataProvider,revMap:yfiles.algorithms.IEdgeMap):yfiles.algorithms.Edge;
      /**
       * Computes a triangulation of the given points.
       * <p>
       * The calculated triangulation is represented by an embedded graph, i.e. to each edge there exists a reverse edge and the
       * outedges around each node are in embedded order. The returned edge and the (optional) reverseEdgeMap can be used to
       * construct all faces of the plane graph and to determine its outer face.
       * </p>
       * @param points the point set to be triangulated. The points must be provided as a YList of YPoints.
       * @param result the resulting triangulation
       * @param resultMap the node map that forms the link between a point and a node.
       * @param reverseEdgeMap a node map that will contain for each edge its reverse edge. If this argument is <code>null</code> then no reverse edge information
       * will be available.
       * @returns an edge on the outer face of the result graph.
       * @static
       */
      static triangulatePoints(points:yfiles.algorithms.YList,result:yfiles.algorithms.Graph,resultMap:yfiles.algorithms.INodeMap,reverseEdgeMap:yfiles.algorithms.IEdgeMap):yfiles.algorithms.Edge;
      /**
       * Computes a triangulation of the given points.
       * <p>
       * The calculated triangulation is represented by an embedded graph, i.e. to each edge there exists a reverse edge and the
       * outedges around each node are in embedded order. The returned edge and the (optional) reverseEdgeMap can be used to
       * construct all faces of the plane graph and to determine its outer face.
       * </p>
       * @param result a graph whose nodes represent the points that need to be triangulated.
       * @param pointData must provide the location (YPoint) for each node in the given graph.
       * @param reverseEdgeMap a node map that will contain for each edge its reverse edge. If this argument is <code>null</code> then no reverse edge information
       * will be available.
       * @returns an edge on the outer face of the result graph.
       * @static
       */
      static triangulatePoints(result:yfiles.algorithms.Graph,pointData:yfiles.algorithms.IDataProvider,reverseEdgeMap:yfiles.algorithms.IEdgeMap):yfiles.algorithms.Edge;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents the size of an object.
     * <p>
     * An instance of this class implements the immutable design pattern.
     * </p>
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface YDimension extends Object,yfiles.lang.IComparable{}
    export class YDimension {
      /**
       * Creates a new YDimension2D object for given size.
       * @param width
       * @param height
       */
      constructor(width:number,height:number);
      /**
       * 
       * @param o
       * @returns 
       */
      compareTo(o:Object):number;
      /**
       * Creates a {@link yfiles.geometry.Size} from a given {@link yfiles.algorithms.YDimension}.
       * @returns The {@link }.
       */
      toSize():yfiles.geometry.Size;
      /**
       * Gets the width of the dimension object.
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of the dimension object.
       * @type {number}
       */
      height:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An oriented rectangle in 2D coordinate space with double precision coordinates.
     * <p>
     * The rectangle's <code>height</code> extends from its <em>{@link yfiles.algorithms.YOrientedRectangle#anchor anchor point}</em> in the direction of its <em>up
     * vector</em> ({@link yfiles.algorithms.YOrientedRectangle#upX ux}, {@link yfiles.algorithms.YOrientedRectangle#upY uy}). Its <code>width</code> extends from its {@link yfiles.algorithms.YOrientedRectangle#anchor anchor point} in
     * direction <code>(-uy, ux)</code> (i.e. perpendicular to the <em>up vector</em>). This means that an oriented rectangle with anchor point <code>(0, 0)</code> width
     * <code>100</code>, height <code>10</code>, and up vector <code>(0, -1)</code> is a paraxial rectangle with upper left corner <code>(0, -10)</code> and lower right corner
     * <code>(100, 0)</code>.
     * </p>
     * @class
     * @implements {yfiles.algorithms.IPlaneObject}
     */
    export interface YOrientedRectangle extends Object,yfiles.algorithms.IPlaneObject{}
    export class YOrientedRectangle {
      /**
       * Creates a new instance using the provided rectangle's values to initialize anchor, size, and up vector.
       * @param rect the provided rectangle.
       */
      constructor(rect:yfiles.algorithms.YOrientedRectangle);
      /**
       * Creates a new instance using the provided rectangle's values to initialize anchor and size.
       * <p>
       * The oriented rectangle's up vector will be <code>(0, -1)</code>.
       * </p>
       * @param rect the provided rectangle.
       */
      constructor(rect:yfiles.algorithms.YRectangle);
      /**
       * Creates a new instance using the provided values to initialize anchor, size, and up vector.
       * @param anchorX The x coordinate of the anchor of the oriented rectangle.
       * @param anchorY The y coordinate of the anchor of the oriented rectangle.
       * @param width The width of the rectangle.
       * @param height The height of the rectangle.
       * @param upX The x component of the up vector.
       * @param upY The y component of the up vector.
       */
      constructor(anchorX:number,anchorY:number,width:number,height:number,upX:number,upY:number);
      /**
       * Creates a new instance using the provided values to initialize anchor and size.
       * <p>
       * The oriented rectangle's up vector will be <code>(0, -1)</code>.
       * </p>
       * @param anchorX The x coordinate of the anchor of the oriented rectangle.
       * @param anchorY The y coordinate of the anchor of the oriented rectangle.
       * @param width The width of the rectangle.
       * @param height The height of the rectangle.
       */
      constructor(anchorX:number,anchorY:number,width:number,height:number);
      /**
       * Creates a new instance using the provided values to initialize anchor, size, and up vector.
       * @param position The provider for the dynamic anchor of this instance.
       * @param size The provider for the dynamic size of this instance.
       * @param upVector The up vector.
       */
      constructor(position:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,upVector:yfiles.algorithms.YVector);
      /**
       * Creates a new instance using the provided values to initialize the anchor and size.
       * <p>
       * The oriented rectangle's up vector will be <code>(0, -1)</code>.
       * </p>
       * @param anchor The provider for the dynamic anchor of this instance.
       * @param size The provider for the dynamic size of this instance.
       */
      constructor(anchor:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension);
      /**
       * Copies the actual values from the given <code>OrientedRectangle</code> to this instance.
       * @param other the <code>OrientedRectangle</code> to retrieve the values from
       */
      adoptValues(other:yfiles.algorithms.YOrientedRectangle):void;
      /**
       * Determines the four corner points of an oriented rectangle.
       * @param rect The rectangle to determine the bounds.
       * @returns the array of corner points.
       * @static
       */
      static calcPoints(rect:yfiles.algorithms.YOrientedRectangle):yfiles.algorithms.YPoint[];
      /**
       * Determines the coordinates of the four corners of an oriented rectangle.
       * @param rect The rectangle to determine the bounds.
       * @returns the array of coordinates of the corner points in which each even index has the x-coordinate and each odd index the
       * y-coordinate
       * @static
       */
      static calcPointsInDouble(rect:yfiles.algorithms.YOrientedRectangle):number[];
      /**
       * Determines whether or not the specified point lies inside this oriented rectangle.
       * @param x the x-coordinate of the point to check.
       * @param y the y-coordinate of the point to check.
       * @param [closed=false] if <code>true</code>, all points on the border of the rectangle are considered to be <em>contained</em>.
       * @returns <code>true</code> iff the specified point lies inside; <code>false</code> otherwise.
       */
      contains(x:number,y:number,closed?:boolean):boolean;
      /**
       * Determines whether the given rectangle r1 contains rectangle r2, using an epsilon value.
       * @param r1 The first rectangle.
       * @param r2 The second rectangle.
       * @param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns true iff the r1 contains r2.
       * @static
       */
      static contains(r1:yfiles.algorithms.YOrientedRectangle,r2:yfiles.algorithms.YOrientedRectangle,eps:number):boolean;
      /**
       * Determines whether the given oriented rectangle contains the provided point, using an epsilon value.
       * @param rect The rectangle.
       * @param p The point to test.
       * @param eps fuzziness range. A positive value allows for fuzzy hit testing. If a point lies outside the given rectangle, but its
       * distance is less than or equal to that value, it will be considered a hit.
       * @returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
       * @static
       */
      static contains(rect:yfiles.algorithms.YOrientedRectangle,p:yfiles.algorithms.YPoint,eps:number):boolean;
      /**
       * Determines whether the given oriented rectangle contains the provided point, using an epsilon value.
       * @param rect The rectangle.
       * @param x x-coordinate of the point to test.
       * @param y y-coordinate of the point to test.
       * @param eps fuzziness range. A positive value allows for fuzzy hit testing. If a point lies outside the given rectangle, but its
       * distance is less than or equal to that value, it will be considered a hit.
       * @returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
       * @static
       */
      static contains(rect:yfiles.algorithms.YOrientedRectangle,x:number,y:number,eps:number):boolean;
      /**
       * Creates a new <code>OrientedRectangle</code> instance whose anchor point is moved by the specified distance values, but has the same
       * width, height, and up vector as this rectangle.
       * @param dx the distance to move the anchor point in x-direction. A positive value means "move" to the right, a negative value means
       * "move" to the left.
       * @param dy the distance to move the anchor point in y-direction. A positive value means "move" downwards, a negative value means
       * "move" upwards.
       * @returns a new <code>OrientedRectangle</code> instance whose anchor point is moved by the specified distance values.
       */
      getMovedInstance(dx:number,dy:number):yfiles.algorithms.YOrientedRectangle;
      /**
       * Creates a new <code>OrientedRectangle</code> instance that has the specified width and height, but has the same anchor point and up
       * vector as this rectangle.
       * @param width the width of the new rectangle.
       * @param height the height of the new rectangle.
       * @returns a new <code>OrientedRectangle</code> instance that has the specified width and height.
       */
      getResizedInstance(width:number,height:number):yfiles.algorithms.YOrientedRectangle;
      /**
       * Determines an intersection point of the specified oriented rectangle and the specified line segment.
       * <p>
       * Note: there might be more than one intersection point. However this method only returns one intersection point or <code>null</code>
       * if there is no intersection.
       * </p>
       * @param rect
       * @param line
       * @param eps
       * @returns an intersection point of the specified oriented rectangle and the specified line segment or <code>null</code> if the rectangle and
       * the segment do not intersect.
       * @static
       */
      static intersectionPoint(rect:yfiles.algorithms.YOrientedRectangle,line:yfiles.algorithms.LineSegment,eps:number):yfiles.algorithms.YPoint;
      /**
       * Determines whether or not the specified oriented rectangle and the specified line segment intersect.
       * @param rect
       * @param line
       * @param eps
       * @returns <code>true</code> if the rectangle and the segment intersect and <code>false</code> otherwise.
       * @static
       */
      static intersects(rect:yfiles.algorithms.YOrientedRectangle,line:yfiles.algorithms.LineSegment,eps:number):boolean;
      /**
       * Determines whether a rectangle intersects an oriented rectangle, given an epsilon.
       * @param orientedRectangle The oriented rectangle to test.
       * @param rectangle The rectangle to test.
       * @param eps A positive value allows for fuzzy hit testing. If the point lies outside the given object but it's distance is less than
       * or equal to that value, it will be considered a hit.
       * @returns Whether they have a non-empty intersection.
       * @static
       */
      static intersects(orientedRectangle:yfiles.algorithms.YOrientedRectangle,rectangle:yfiles.algorithms.YRectangle,eps:number):boolean;
      /**
       * Moves this rectangle by applying the offset to the anchor.
       * @param dx The x offset to move the rectangle's position by.
       * @param dy The y offset to move the rectangle's position by.
       */
      moveBy(dx:number,dy:number):void;
      /**
       * Sets the anchor of this rectangle.
       * @param x the new x-coordinate of the anchor point.
       * @param y the new y-coordinate of the anchor point.
       */
      setAnchor(x:number,y:number):void;
      /**
       * Sets the anchor of the OrientedRectangle so that the center of the rectangle coincides with the given coordinate pair.
       * @param cx The x coordinate of the center.
       * @param cy The y coordinate of the center.
       */
      setCenter(cx:number,cy:number):void;
      /**
       * Sets the size of this rectangle.
       * @param width the new width.
       * @param height the new height.
       */
      setSize(width:number,height:number):void;
      /**
       * Sets the components of the up vector to the new values.
       * @param upX The x component of the normalized up vector.
       * @param upY The y component of the normalized up vector.
       */
      setUpVector(upX:number,upY:number):void;
      /**
       * Creates an immutable {@link yfiles.geometry.IOrientedRectangle} from a given {@link yfiles.algorithms.YOrientedRectangle}.
       * @returns The {@link }.
       */
      toImmutableOrientedRectangle():yfiles.geometry.IOrientedRectangle;
      /**
       * Gets whether this instance has negative width or height.
       * @type {boolean}
       */
      empty:boolean;
      /**
       * Gets or sets the anchor of this oriented rectangle.
       * @type {yfiles.algorithms.YPoint}
       */
      anchor:yfiles.algorithms.YPoint;
      /**
       * Gets the x-coordinate of this rectangle's anchor point.
       * @type {number}
       */
      anchorX:number;
      /**
       * Gets the y-coordinate of this rectangle's anchor point.
       * @type {number}
       */
      anchorY:number;
      /**
       * Gets or sets the size of this rectangle.
       * @type {yfiles.algorithms.YDimension}
       */
      size:yfiles.algorithms.YDimension;
      /**
       * Gets the width of this rectangle.
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of this rectangle.
       * @type {number}
       */
      height:number;
      /**
       * Gets the x-component of this rectangle's up vector.
       * @type {number}
       */
      upX:number;
      /**
       * Gets the y-component of this rectangle's up vector.
       * @type {number}
       */
      upY:number;
      /**
       * Gets or sets the angle (measured in radians) of this rectangle.
       * <p>
       * The angle of an oriented rectangle is the angle between the vector <code>(0, -1)</code> and the rectangle's up vector in counter
       * clockwise order. An angle of 0 means the up vector points up in direction <code>(0, -1)</code>.
       * </p>
       * @type {number}
       */
      angle:number;
      /**
       * Gets or sets the current center of the oriented rectangle.
       * @type {yfiles.algorithms.YPoint}
       */
      center:yfiles.algorithms.YPoint;
      /**
       * Calculates the paraxial bounding box of this oriented rectangle.
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a point in the plane with double coordinates.
     * <p>
     * This class implements the immutable design pattern.
     * </p>
     * @class
     * @implements {yfiles.lang.IComparable}
     * @final
     */
    export interface YPoint extends Object,yfiles.lang.IComparable{}
    export class YPoint {
      /**
       * Creates a new YPoint object for a given position.
       * @param x the x coordinate of the point.
       * @param y the y coordinate of the point.
       */
      constructor(x:number,y:number);
      /**
       * Creates a new YPoint at location (0,0)
       */
      constructor();
      /**
       * Adds two points and returns the result.
       * @param p1 an arbitrary instance of YPoint.
       * @param p2 an arbitrary instance of YPoint.
       * @returns 
       * @static
       */
      static add(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Comparable implementation.
       * <p>
       * YPoints are ordered by ascending x-coordinates. If the x-coordinates of two points equal, then these points are ordered
       * by ascending y-coordinates.
       * </p>
       * @param o
       * @returns 
       */
      compareTo(o:Object):number;
      /**
       * Returns the euclidean distance between two points.
       * @param x1 x-coordinate of first point
       * @param y1 y-coordinate of first point
       * @param x2 x-coordinate of second point
       * @param y2 y-coordinate of second point
       * @returns the euclidean distance between first and second point
       * @static
       */
      static distance(x1:number,y1:number,x2:number,y2:number):number;
      /**
       * Returns the euclidean distance between two points.
       * @param p1 an arbitrary point
       * @param p2 an arbitrary point
       * @returns the Euclidean distance between p1 and p2.
       * @static
       */
      static distance(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint):number;
      /**
       * Returns the euclidean distance between this point and a given point.
       * @param x the x coordinate of an arbitrary point
       * @param y the y coordinate of an arbitrary point
       * @returns the Euclidean distance between this point and the point (x,y).
       */
      distanceTo(x:number,y:number):number;
      /**
       * Returns the euclidean distance between this point and a given point.
       * @param p an arbitrary point
       * @returns the Euclidean distance between this point and p.
       */
      distanceTo(p:yfiles.algorithms.YPoint):number;
      /**
       * Returns a point that geometrically lies in in the middle of the line formed by the given points.
       * @param p1 an arbitrary instance of YPoint.
       * @param p2 an arbitrary instance of YPoint.
       * @returns 
       * @static
       */
      static midPoint(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Returns the point, got by moving this point to another position.
       * @param x the value which is added on the x-coordinate of the point.
       * @param y the value which is added on the y-coordinate of the point.
       * @returns a new instance of YPoint which is the result of the moving operation.
       */
      moveBy(x:number,y:number):yfiles.algorithms.YPoint;
      /**
       * Subtracts two points (p1 - p2) and returns the result.
       * @param p1 an arbitrary instance of YPoint.
       * @param p2 an arbitrary instance of YPoint.
       * @returns 
       * @static
       */
      static subtract(p1:yfiles.algorithms.YPoint,p2:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Returns a copy of the given point with exchanged x- and y-coordinates.
       * @param p an arbitrary instance of YPoint.
       * @returns 
       * @static
       */
      static swap(p:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Creates a {@link yfiles.geometry.Point} from a given {@link yfiles.algorithms.YPoint}.
       * @returns The {@link }.
       */
      toPoint():yfiles.geometry.Point;
      /**
       * A YPoint constant with coordinates (0,0).
       * @const
       * @static
       * @type {yfiles.algorithms.YPoint}
       */
      static ORIGIN:yfiles.algorithms.YPoint;
      /**
       * Gets the x-coordinate of the point object.
       * @type {number}
       */
      x:number;
      /**
       * Gets the y-coordinate of the point object.
       * @type {number}
       */
      y:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents an ordered list of points in the plane.
     * @class
     * @final
     */
    export interface YPointPath extends Object{}
    export class YPointPath {
      /**
       * Creates a new path from an array of points.
       * @param path
       */
      constructor(path:yfiles.algorithms.YPoint[]);
      /**
       * Creates a new path from a list of points.
       * @param l a list of {@link } instances.
       */
      constructor(l:yfiles.collections.IList<Object>);
      /**
       * Creates a new empty path.
       */
      constructor();
      /**
       * Calculate the (geometric) length of the path.
       * <p>
       * The length of the path is the sum of lengths of all line segments making up the path.
       * </p>
       * @returns the (geometric) length of the path
       */
      calculateLength():number;
      /**
       * Create a point path with reverse ordering of the points.
       * @returns 
       */
      createReverse():yfiles.algorithms.YPointPath;
      /**
       * Get the points in the path.
       * @returns 
       */
      cursor():yfiles.algorithms.ICursor;
      /**
       * Returns a line segment in the path.
       * @param i
       * @returns 
       */
      getLineSegment(i:number):yfiles.algorithms.LineSegment;
      /**
       * Get the number of points in the path.
       * @returns 
       */
      length():number;
      /**
       * Get the points in the path.
       * @returns 
       */
      lineSegments():yfiles.algorithms.ILineSegmentCursor;
      /**
       * Get the points in the path.
       * @returns 
       */
      points():yfiles.algorithms.IPointCursor;
      /**
       * Get the points in the list as array.
       * @returns 
       */
      toArray():yfiles.algorithms.YPoint[];
      /**
       * Get the points in the path as list.
       * @returns a list of {@link } instances.
       */
      toList():yfiles.collections.IList<Object>;
      /**
       * Defines a path with no points.
       * @const
       * @static
       * @type {yfiles.algorithms.YPointPath}
       */
      static EMPTY_PATH:yfiles.algorithms.YPointPath;
      /**
       * Gets the first point in the path.
       * @type {yfiles.algorithms.YPoint}
       */
      first:yfiles.algorithms.YPoint;
      /**
       * Gets the last point in the path.
       * @type {yfiles.algorithms.YPoint}
       */
      last:yfiles.algorithms.YPoint;
      /**
       * Gets the number of line segments in the path.
       * @type {number}
       */
      lineSegmentCount:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class defines a rectangle and provides utility methods for it.
     * @class
     * @extends {yfiles.algorithms.YDimension}
     * @implements {yfiles.algorithms.IPlaneObject}
     */
    export interface YRectangle extends yfiles.algorithms.YDimension,yfiles.algorithms.IPlaneObject{}
    export class YRectangle {
      /**
       * Creates a new rectangle with given upper left corner and size.
       * @param x x-coordinate of upper left corner of the rectangle.
       * @param y y-coordinate of upper left corner of the rectangle.
       * @param width width of the rectangle.
       * @param height height of the rectangle.
       */
      constructor(x:number,y:number,width:number,height:number);
      /**
       * Creates a new rectangle with given upper left corner and size.
       * @param pos upper left corner of the rectangle.
       * @param size size of the rectangle.
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension);
      /**
       * Creates a new rectangle with upper left corner (0,0) and size (0,0).
       */
      constructor();
      /**
       * Checks whether or not this <code>YRectangle</code> contains the given rectangle.
       * @param p
       * @returns 
       */
      contains(p:yfiles.algorithms.YRectangle):boolean;
      /**
       * Checks whether or not this <code>YRectangle</code> contains the given point.
       * @param p
       * @returns 
       */
      contains(p:yfiles.algorithms.YPoint):boolean;
      /**
       * Checks whether or not this <code>YRectangle</code> contains the rectangle defined by the given frame.
       * @param x
       * @param y
       * @param width
       * @param height
       * @returns 
       */
      contains(x:number,y:number,width:number,height:number):boolean;
      /**
       * Checks whether or not this <code>YRectangle</code> contains the given point.
       * @param x the x-coordinate of the point to check.
       * @param y the x-coordinate of the point to check.
       * @returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
       */
      contains(x:number,y:number):boolean;
      /**
       * Determines whether the specified rectangle contains the specified point.
       * @param rx the x-coordinate of the upper left corner of the rectangle.
       * @param ry the y-coordinate of the upper left corner of the rectangle.
       * @param rw the width of the rectangle.
       * @param rh the height of the rectangle.
       * @param x the x-coordinate of the point to check.
       * @param y the x-coordinate of the point to check.
       * @param [closed=false] if <code>true</code>, all points on the border of the rectangle are considered to be <em>contained</em>.
       * @returns <code>true</code> if the point lies inside the rectangle; <code>false</code> otherwise.
       * @static
       */
      static contains(rx:number,ry:number,rw:number,rh:number,x:number,y:number,closed?:boolean):boolean;
      /**
       * Returns the Euclidean distance to the passed rectangle.
       * <p>
       * If they overlap, the distance is 0.
       * </p>
       * @param other the second rectangle.
       * @returns the distance to the given rectangle.
       */
      getEuclideanDistance(other:yfiles.algorithms.YRectangle):number;
      /**
       * Returns the Manhattan distance to the passed rectangle.
       * <p>
       * If they overlap the distance is 0.
       * </p>
       * @param other the second rectangle.
       * @returns the distance to the given rectangle.
       */
      getManhattanDistance(other:yfiles.algorithms.YRectangle):number;
      /**
       * Returns whether or not the given rectangles intersect.
       * @param r1
       * @param r2
       * @returns 
       * @static
       */
      static intersects(r1:yfiles.algorithms.YRectangle,r2:yfiles.algorithms.YRectangle):boolean;
      /**
       * Creates a {@link yfiles.geometry.Rect} from a given {@link yfiles.algorithms.YRectangle}.
       * @returns The {@link }.
       */
      toRect():yfiles.geometry.Rect;
      /**
       * Gets x-coordinate of upper left corner.
       * @type {number}
       */
      x:number;
      /**
       * Gets y-coordinate of upper left corner.
       * @type {number}
       */
      y:number;
      /**
       * Gets coordinates of upper left corner.
       * @type {yfiles.algorithms.YPoint}
       */
      location:yfiles.algorithms.YPoint;
      /**
       * Gets this object.
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a vector in the 2-dimensional real vector space.
     * <p>
     * This vector is an ordered 2 tuple and is defined by two doubles.
     * </p>
     * @class
     */
    export interface YVector extends Object{}
    export class YVector {
      /**
       * Creates a new vector, whose direction is given by two points.
       * <p>
       * The vector is defined by <code>p1</code> - <code>p2</code>.
       * </p>
       * @param p1 The first point.
       * @param [p2=null] The second point. If no point is spcified, <code>(0,0)</code> is used instead.
       */
      constructor(p1:yfiles.algorithms.YPoint,p2?:yfiles.algorithms.YPoint);
      /**
       * Creates a new vector which is a copy of another vector.
       * @param v the vector, whose values are copied.
       */
      constructor(v:yfiles.algorithms.YVector);
      /**
       * Creates a new vector, whose direction is given by two points.
       * <p>
       * The vector is defined by <code>(x1 - x2, y1 - y2)</code>.
       * </p>
       * @param x1 the X-coordinate of the first point.
       * @param y1 the Y-coordinate of the first point.
       * @param x2 the X-coordinate of the second point.
       * @param y2 the Y-coordinate of the second point.
       */
      constructor(x1:number,y1:number,x2:number,y2:number);
      /**
       * Creates a new vector with given direction.
       * @param dx the first coordinate
       * @param dy the second coordinate
       */
      constructor(dx:number,dy:number);
      /**
       * Adds a vector to this vector.
       * @param v the vector to add.
       */
      add(v:yfiles.algorithms.YVector):void;
      /**
       * Adds the vector to a point and returns the resulting point.
       * @param p a point.
       * @param v the vector to add to the point.
       * @returns p+v
       * @static
       */
      static add(p:yfiles.algorithms.YPoint,v:yfiles.algorithms.YVector):yfiles.algorithms.YPoint;
      /**
       * Adds two vectors and returns the result.
       * @param v first vector to sum.
       * @param w second vector to sum.
       * @returns v+w
       * @static
       */
      static add(v:yfiles.algorithms.YVector,w:yfiles.algorithms.YVector):yfiles.algorithms.YVector;
      /**
       * Returns the result of the addition of two angles between <code>0</code> and <code>2*Pi</code>.
       * <p>
       * The result is calculated modulo <code>2*Pi</code>.
       * </p>
       * @param a1 a value in <code>[0,2Pi)</code>.
       * @param a2 a value in <code>[0,2Pi)</code>.
       * @returns The sum of a1 and a2 modulo <code>2*Pi</code>.
       * @static
       */
      static addAngle(a1:number,a2:number):number;
      /**
       * Returns the angle (measured in radians) between two vectors in clockwise order (with regards to screen coordinates) from
       * v1 to v2.
       * <p>
       * Screen coordinates mean positive x-direction is from left to right and positive y-direction is from top to bottom.
       * </p>
       * @param v1
       * @param v2
       * @returns 
       * @static
       */
      static angle(v1:yfiles.algorithms.YVector,v2:yfiles.algorithms.YVector):number;
      /**
       * Returns this vector with unit length.
       * @param v
       * @returns 
       * @static
       */
      static getNormal(v:yfiles.algorithms.YVector):yfiles.algorithms.YVector;
      /**
       * Returns the length of the vector, this is the value of the euclidean norm.
       * @returns a value > 0.
       */
      length():number;
      /**
       * Assigns unit length to the vector.
       * <br />
       */
      norm():void;
      /**
       * Returns the vector which is orthogonal to the given one and has unit length.
       * @param v a vector.
       * @returns a vector which is orthogonal to v with unit length.
       * @static
       */
      static orthoNormal(v:yfiles.algorithms.YVector):yfiles.algorithms.YVector;
      /**
       * Returns true if vector v1 is on the right side of v2.
       * @param v1
       * @param v2
       * @returns 
       * @static
       */
      static rightOf(v1:yfiles.algorithms.YVector,v2:yfiles.algorithms.YVector):boolean;
      /**
       * Returns a new <code>YVector</code> instance that is obtained by rotating this vector by the given angle (measured in radians) in
       * clockwise direction (with regards to screen coordinates).
       * <p>
       * Screen coordinates mean positive x-direction is from left to right and positive y-direction is from top to bottom.
       * </p>
       * @param angle the angle of rotation in radians.
       * @returns the rotated vector.
       */
      rotate(angle:number):yfiles.algorithms.YVector;
      /**
       * Returns the value of the scalar product of two vectors.
       * @param v1 the first vector.
       * @param v2 the second vector.
       * @returns <code>v1.x * v2.x + v1.y * v2.y</code>
       * @static
       */
      static scalarProduct(v1:yfiles.algorithms.YVector,v2:yfiles.algorithms.YVector):number;
      /**
       * Scales the vector by an factor.
       * @param factor the scale factor, with which the length is multiplied.
       */
      scale(factor:number):void;
      /**
       * Creates a {@link yfiles.geometry.Point} from a given {@link yfiles.algorithms.YVector}.
       * @returns The {@link }.
       */
      toPoint():yfiles.geometry.Point;
      /**
       * Gets the first coordinate of the vector.
       * @type {number}
       */
      x:number;
      /**
       * Gets the second coordinate of the vector.
       * @type {number}
       */
      y:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class implements a directed graph structure.
     * <p>
     * Basically, a directed graph consists of a set of objects called "nodes" (represented by instances of class
     * {@link yfiles.algorithms.Node}) and a set of node pairs which are called "edges" (represented by instances of class
     * {@link yfiles.algorithms.Edge}).
     * <br />
     * The <i>directed</i> stems from the fact that all edges in the graph have direction, i.e., they have a distinct source node and a
     * distinct target node. Using the aforementioned pair notation, an edge would be written as (<i>&lt;source node></i>, <i>&lt;target node></i>).
     * </p>
     * <p>
     * Class Graph presents a proper data type that provides support for all essential operations like element creation,
     * removal, access, and iteration.
     * <br />
     * <b>Important:</b> Class Graph is the single authority for any structural changes to the graph data type. Specifically, this means that
     * there is no way to create or delete a node or an edge without using an actual Graph instance.
     * </p>
     * <p>
     * Furthermore, this class is also responsible for providing access to its elements. This is done by means of bidirectional
     * cursors that present a read-only view on the node set (interface {@link yfiles.algorithms.INodeCursor}) and edge set (interface
     * {@link yfiles.algorithms.IEdgeCursor}).
     * </p>
     * <p>
     * Class Graph fires notification events that signal structural changes, like, e.g., creation, removal, reinsertion, or
     * modification of graph elements.
     * </p>
     * <p>
     * This class provides direct support for the notion of data accessors. It allows to register so-called data providers
     * (implementations of interface {@link yfiles.algorithms.IDataProvider}) that hold arbitrary data which is associated to its nodes
     * and/or edges.
     * <br />
     * Also, it serves as a factory to create so-called maps ({@link yfiles.algorithms.INodeMap},
     * {@link yfiles.algorithms.IEdgeMap}) that can be utilized to bind arbitrary data to nodes and edges.
     * </p>
     * @class
     */
    export interface Graph extends Object{}
    export class Graph {
      /**
       * Instantiates a new Graph object as a partial copy of the given graph.
       * <p>
       * Only the subgraph induced by the given cursor will be copied to the new Graph instance. If no cursor is specified, the
       * complete graph is copied. Values bound to the argument graph via node and edge keys are available in the new Graph
       * instance with the keys registered with <code>graph</code>. Only references to these values are copied.
       * </p>
       * <p>
       * The new Graph instance also inherits all graph listeners registered with the given graph.
       * </p>
       * @param graph The graph to be (partially) copied.
       * @param [subNodes=null] A cursor to iterate over the nodes that actually induce the subgraph to be copied.
       */
      constructor(graph:yfiles.algorithms.Graph,subNodes?:yfiles.algorithms.ICursor);
      /**
       * Instantiates an empty Graph object.
       */
      constructor();
      /**
       * Registers the given data provider using the given look-up key.
       * <p>
       * If there is already a data provider registered with that key, then it will be overwritten with the new one.
       * </p>
       * @param providerKey
       * @param data
       */
      addDataProvider(providerKey:Object,data:yfiles.algorithms.IDataProvider):void;
      /**
       * Redefines an edge's end points and fires corresponding notification events to inform registered listeners.
       * <p>
       * Edge <code>e</code> has source node <code>v := sourceReference.source()</code> or <code>v := newSource</code>, if <code>sourceReference == null</code> and target node <code>w := targetReference.target()</code>
       * or <code>w := newTarget</code>, if <code>targetReference == null</code>.
       * <br />
       * 
       * <br />
       * Edge <code>e</code> is inserted in such a way that an iteration over the edges at <code>v</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>sourceReference</code>, if <code>sourceD == AFTER</code></li>
       * <li>before <code>sourceReference</code>, if <code>sourceD == BEFORE</code>,</li>
       * </ul>
       * <p>
       * and an iteration over the edges at <code>w</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>targetReference</code>, if <code>targetD == AFTER</code></li>
       * <li>before <code>targetReference</code>, if <code>targetD == BEFORE</code>.</li>
       * </ul>
       * @param e The edge to be changed.
       * @param newSource The new source node.
       * @param sourceReference Reference edge for insertion at the new source node.
       * @param sourceD One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       * @param newTarget The new target node.
       * @param targetReference Reference edge for insertion at the new target node.
       * @param targetD One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       */
      changeEdge(e:yfiles.algorithms.Edge,newSource:yfiles.algorithms.Node,sourceReference:yfiles.algorithms.Edge,sourceD:yfiles.algorithms.GraphElementInsertion,newTarget:yfiles.algorithms.Node,targetReference:yfiles.algorithms.Edge,targetD:yfiles.algorithms.GraphElementInsertion):void;
      /**
       * Redefines an edge's end points and fires corresponding notification events to inform registered listeners.
       * <p>
       * Edge <code>e</code> has source node <code>v := e1.source()</code> and target node <code>w := e2.target()</code>.
       * <br />
       * 
       * <br />
       * Edge <code>e</code> is inserted in such a way that an iteration over the edges at <code>v</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>e1</code>, if <code>d1 == AFTER</code></li>
       * <li>before <code>e1</code>, if <code>d1 == BEFORE</code>,</li>
       * </ul>
       * <p>
       * and an iteration over the edges at <code>w</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>e2</code>, if <code>d2 == AFTER</code></li>
       * <li>before <code>e2</code>, if <code>d2 == BEFORE</code>.</li>
       * </ul>
       * @param e The edge to be changed.
       * @param e1 Reference edge for insertion at a new source node.
       * @param e2 Reference edge for insertion at a new target node.
       * @param d1 One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       * @param d2 One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       */
      changeEdge(e:yfiles.algorithms.Edge,e1:yfiles.algorithms.Edge,e2:yfiles.algorithms.Edge,d1:yfiles.algorithms.GraphElementInsertion,d2:yfiles.algorithms.GraphElementInsertion):void;
      /**
       * Redefines an edge's end points and fires corresponding notification events to inform registered listeners.
       * <p>
       * The edge is appended to the lists of incoming and outgoing edges at the given source node and target node,
       * respectively.
       * </p>
       * @param e The edge to be changed.
       * @param newSource The new source node of the given edge.
       * @param newTarget The new target node of the given edge.
       */
      changeEdge(e:yfiles.algorithms.Edge,newSource:yfiles.algorithms.Node,newTarget:yfiles.algorithms.Node):void;
      /**
       * Removes all nodes and edges from this graph and fires corresponding notification events to inform registered listeners.
       */
      clear():void;
      /**
       * Whether or not this graph contains the given edge.
       * @param e
       * @returns 
       */
      contains(e:yfiles.algorithms.Edge):boolean;
      /**
       * Whether or not this graph contains the given node.
       * @param v
       * @returns 
       */
      contains(v:yfiles.algorithms.Node):boolean;
      /**
       * Returns whether or not this graph contains an edge that connects the given nodes.
       * @param source The source node.
       * @param target The target node.
       * @returns 
       * @see yfiles.algorithms.Node#getEdgeTo
       * @see yfiles.algorithms.Node#getEdgeFrom
       * @see yfiles.algorithms.Node#getEdge
       */
      containsEdge(source:yfiles.algorithms.Node,target:yfiles.algorithms.Node):boolean;
      /**
       * Creates a copy of this graph.
       * <p>
       * Invokes {@link yfiles.algorithms.Graph#Graph}.
       * </p>
       * @returns The newly created Graph object.
       */
      createCopy():yfiles.algorithms.Graph;
      /**
       * Creates a new edge in this graph to be ordered before or after a given edge and fires a corresponding notification event
       * to inform registered listeners.
       * <p>
       * The new edge <code>e</code> has source node <code>v</code> and target node <code>w</code>, i.e., would be written as edge <code>e = (v, w)</code>.
       * <br />
       * 
       * <br />
       * Edge <code>e</code> is inserted in such a way that an iteration over the edges at node <code>v</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>e1</code>, if <code>d1 == AFTER</code></li>
       * <li>before <code>e1</code>, if <code>d1 == BEFORE</code>,</li>
       * </ul>
       * <p>
       * and an iteration over the edges at <code>w</code> returns <code>e</code>
       * </p>
       * <ul>
       * <li>after <code>e2</code>, if <code>d2 == AFTER</code></li>
       * <li>before <code>e2</code>, if <code>d2 == BEFORE</code>.</li>
       * </ul>
       * @param v The source node of the edge.
       * @param e1 An edge with source node <code>v</code>.
       * @param w The target node of the edge.
       * @param e2 An edge with target node <code>w</code>.
       * @param d1 One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       * @param d2 One of the object insertion specifiers {@link #BEFORE} or
       * {@link #AFTER}.
       * @returns The newly created Edge object.
       */
      createEdge(v:yfiles.algorithms.Node,e1:yfiles.algorithms.Edge,w:yfiles.algorithms.Node,e2:yfiles.algorithms.Edge,d1:yfiles.algorithms.GraphElementInsertion,d2:yfiles.algorithms.GraphElementInsertion):yfiles.algorithms.Edge;
      /**
       * Creates a new edge in this graph and fires a corresponding notification event to inform registered listeners.
       * <p>
       * The new edge has source node <code>v</code> and target node <code>w</code>, i.e., would be written as edge <code>e = (v, w)</code>.
       * </p>
       * <p>
       * The edge is appended to the lists of incoming and outgoing edges at the source node and target node, respectively.
       * </p>
       * @param v The source node of the edge.
       * @param w The target node of the edge.
       * @returns The newly created Edge object.
       */
      createEdge(v:yfiles.algorithms.Node,w:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Returns a newly created edge map that is valid for the edges in this graph.
       * <p>
       * The implementation returned by this method can be used for any edge that is part of this Graph instance at any point of
       * time, i.e., it is safe to modify the graph structure (add and remove nodes and edges) freely.
       * </p>
       * <p>
       * The implementation returned uses <code>O(m)</code> memory at all times and provides true <code>O(1)</code> read and write access for each edge.
       * </p>
       * <p>
       * In order to release the resources held by this map, {@link yfiles.algorithms.Graph#disposeEdgeMap} has to be called.
       * </p>
       * @returns 
       */
      createEdgeMap():yfiles.algorithms.IEdgeMap;
      /**
       * Creates an empty base object of the same type as this graph.
       * <p>
       * Subclasses should override this method.
       * </p>
       * @returns 
       */
      createGraph():yfiles.algorithms.Graph;
      /**
       * Creates a new node in this graph and fires a corresponding notification event to inform registered listeners.
       * @returns The newly created Node object.
       */
      createNode():yfiles.algorithms.Node;
      /**
       * Returns a newly created node map that is valid for the nodes in this graph.
       * <p>
       * The implementation returned by this method can be used for any node that is part of this Graph instance at any point of
       * time, i.e., it is safe to modify the graph structure (add and remove nodes and edges) freely.
       * </p>
       * <p>
       * The implementation returned uses <code>O(n)</code> memory at all times and provides true <code>O(1)</code> read and write access for each node.
       * </p>
       * <p>
       * In order to release the resources held by this map, {@link yfiles.algorithms.Graph#disposeNodeMap} has to be called.
       * </p>
       * @returns 
       */
      createNodeMap():yfiles.algorithms.INodeMap;
      /**
       * Informs the graph that the given edge map is no longer needed.
       * <p>
       * This method is used for EdgeMap implementations that have been obtained using the {@link yfiles.algorithms.Graph#createEdgeMap}
       * factory method.
       * </p>
       * <p>
       * Calling this method will destroy the edge map and associated resources can be freed. It is strongly recommended to
       * dispose of all edge maps that are not needed anymore using this method.
       * </p>
       * @param map
       */
      disposeEdgeMap(map:yfiles.algorithms.IEdgeMap):void;
      /**
       * Informs the graph that the given node map is no longer needed.
       * <p>
       * This method is used for NodeMap implementations that have been obtained using the {@link yfiles.algorithms.Graph#createNodeMap}
       * factory method.
       * </p>
       * <p>
       * Calling this method will destroy the node map and associated resources can be freed. It is strongly recommended to
       * dispose of all node maps that are not needed anymore using this method.
       * </p>
       * @param map
       */
      disposeNodeMap(map:yfiles.algorithms.INodeMap):void;
      /**
       * Low-level iteration support for adjacent edges.
       * @param v
       * @returns 
       * @protected
       * @static
       */
      static firstOutEdge(v:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Returns the data provider that is registered with the graph using the given look-up key.
       * <p>
       * The look-up domain of a returned data provider normally consists of either the nodes of the graph, or its edges, or
       * both.
       * </p>
       * @param providerKey
       * @returns 
       */
      getDataProvider(providerKey:Object):yfiles.algorithms.IDataProvider;
      /**
       * Returns an array containing all edges of this graph.
       * @returns 
       */
      getEdgeArray():yfiles.algorithms.Edge[];
      /**
       * Provides access to the edges of the graph.
       * @returns An EdgeCursor to iterate over the edges in the graph.
       */
      getEdgeCursor():yfiles.algorithms.IEdgeCursor;
      /**
       * Returns an array containing all nodes of this graph.
       * @returns 
       */
      getNodeArray():yfiles.algorithms.Node[];
      /**
       * Provides access to the nodes of the graph.
       * @returns A NodeCursor to iterate over the nodes in the graph.
       */
      getNodeCursor():yfiles.algorithms.INodeCursor;
      /**
       * Hides the given node from this graph.
       * <p>
       * Hiding a node means to (temporarily) remove the node from the graph.
       * </p>
       * <p>
       * The only difference to a proper node removal as performed by {@link yfiles.algorithms.Graph#removeNode} is that no event will be
       * emitted that signals the structural change (i.e. the node's removal).
       * </p>
       * <p>
       * Generally, hiding should <i>only</i> be used in the sense of temporarily removing an object that will be reinserted shortly after.
       * </p>
       * <p>
       * To reinsert a hidden node use {@link yfiles.algorithms.Graph#unhide}.
       * </p>
       * @param v
       * @see yfiles.algorithms.Graph#hide
       * @see yfiles.algorithms.Graph#unhide
       */
      hide(v:yfiles.algorithms.Node):void;
      /**
       * Hides the given edge from this graph.
       * <p>
       * Hiding an edge means to (temporarily) remove the edge from the graph.
       * </p>
       * <p>
       * The only difference to a proper edge removal as performed by {@link yfiles.algorithms.Graph#removeEdge} is that no event will be
       * emitted that signals the structural change (i.e. the edge's removal).
       * </p>
       * <p>
       * Generally, hiding should <i>only</i> be used in the sense of temporarily removing an object that will be reinserted shortly after.
       * </p>
       * <p>
       * To reinsert a hidden edge use {@link yfiles.algorithms.Graph#unhide}.
       * </p>
       * @param e
       * @see yfiles.algorithms.Graph#hide
       * @see yfiles.algorithms.Graph#unhide
       */
      hide(e:yfiles.algorithms.Edge):void;
      /**
       * Moves the given edge to the first position within the sequence of edges in this graph.
       * @param e
       */
      moveToFirst(e:yfiles.algorithms.Edge):void;
      /**
       * Moves the given node to the first position within the sequence of nodes in this graph.
       * @param v
       */
      moveToFirst(v:yfiles.algorithms.Node):void;
      /**
       * Moves the given edge to the last position within the sequence of edges in this graph.
       * @param e
       */
      moveToLast(e:yfiles.algorithms.Edge):void;
      /**
       * Moves the given node to the last position within the sequence of nodes in this graph.
       * @param v
       */
      moveToLast(v:yfiles.algorithms.Node):void;
      /**
       * For internal debugging purposes only.
       */
      printNodeSlotSize():void;
      /**
       * Reinserts a formerly removed edge into this graph and fires a corresponding notification event to inform registered
       * listeners.
       * <p>
       * The reinserted edge is appended to the sequence of edges in this graph, i.e., normally, its new position does not match
       * the position before its removal. The same holds for the edge's positions in the list of incoming and outgoing edges at
       * its source node and target node, respectively.
       * </p>
       * <p>
       * Note that reinserting an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed)
       * causes an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
       * </p>
       * @param e The edge to be reinserted.
       * @see yfiles.algorithms.Graph#removeEdge
       */
      reInsertEdge(e:yfiles.algorithms.Edge):void;
      /**
       * Reinserts a formerly removed node into this graph and fires a corresponding notification event to inform registered
       * listeners.
       * <p>
       * The reinserted node is appended to the sequence of nodes in this graph, i.e., normally, its new position does not match
       * the position before its removal.
       * </p>
       * @param v The node to be reinserted.
       * @see yfiles.algorithms.Graph#removeNode
       */
      reInsertNode(v:yfiles.algorithms.Node):void;
      /**
       * Removes the data provider that is registered using the given look-up key.
       * @param providerKey
       */
      removeDataProvider(providerKey:Object):void;
      /**
       * Removes the given edge from this graph and fires a corresponding notification event to inform registered listeners.
       * <p>
       * The edge will be deselected before it gets removed.
       * </p>
       * @param e The edge to be removed.
       */
      removeEdge(e:yfiles.algorithms.Edge):void;
      /**
       * Removes the given node from this graph.
       * <p>
       * All edges connecting to the given node are removed as well (preceding the actual node removal). Corresponding
       * notification events are fired to inform registered listeners.
       * </p>
       * <p>
       * The node will be deselected before it gets removed.
       * </p>
       * @param v The node to be removed from this graph.
       */
      removeNode(v:yfiles.algorithms.Node):void;
      /**
       * Reverses the given edge and fires corresponding notification events to inform registered listeners.
       * <p>
       * This operation exchanges source and target node of the edge.
       * </p>
       * @param e
       */
      reverseEdge(e:yfiles.algorithms.Edge):void;
      /**
       * Sorts incoming and outgoing edges at each node of the graph.
       * <p>
       * If a given comparator is <code>null</code>, then the corresponding edges (i.e., incoming/outgoing) will not be sorted. This sorts the
       * order of the edges as returned by {@link yfiles.algorithms.Node#getOutEdgeCursor} and {@link yfiles.algorithms.Node#getInEdgeCursor}
       * respectively.
       * </p>
       * @param inComparer The comparator used for the incoming edges at each node.
       * @param outComparer The comparator used for the outgoing edges at each node.
       */
      sortEdges(inComparer:yfiles.collections.IComparer<Object>,outComparer:yfiles.collections.IComparer<Object>):void;
      /**
       * Sorts the internally held list of edges.
       * <p>
       * If the given comparator is <code>null</code>, then the edges will not be sorted. This list determines the order of the edges as
       * returned by {@link yfiles.algorithms.Graph#getEdgeCursor}.
       * </p>
       * @param comparer The comparator used for the edges.
       */
      sortEdges(comparer:yfiles.collections.IComparer<Object>):void;
      /**
       * Sorts the internally held list of nodes.
       * <p>
       * If the given comparator is <code>null</code>, then the nodes will not be sorted. This list determines the order of the nodes as
       * returned by {@link yfiles.algorithms.Graph#getNodeCursor}.
       * </p>
       * @param comparer The comparator used for the nodes.
       */
      sortNodes(comparer:yfiles.collections.IComparer<Object>):void;
      /**
       * Unhides the given node in this graph.
       * <p>
       * Unhiding a node means to reinsert a node that was formerly hidden from this graph by a call to
       * {@link yfiles.algorithms.Graph#hide}.
       * </p>
       * <p>
       * The only difference to a proper node reinsertion as performed by {@link yfiles.algorithms.Graph#reInsertNode} is that no event
       * will be emitted that signals the structural change (i.e. the node's reinsertion).
       * </p>
       * @param v
       */
      unhide(v:yfiles.algorithms.Node):void;
      /**
       * Unhides the given edge in this graph.
       * <p>
       * Unhiding an edge means to reinsert an edge that was formerly hidden from this graph by a call to
       * {@link yfiles.algorithms.Graph#hide}.
       * </p>
       * <p>
       * The only difference to a proper edge reinsertion as performed by {@link yfiles.algorithms.Graph#reInsertEdge} is that no event
       * will be emitted that signals the structural change (i.e. the edge's reinsertion).
       * </p>
       * <p>
       * Note that unhiding an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed) causes
       * an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
       * </p>
       * @param e
       * @see yfiles.algorithms.Graph#hide
       * @see yfiles.algorithms.Graph#unhide
       */
      unhide(e:yfiles.algorithms.Edge):void;
      /**
       * Gets the number of nodes in this graph.
       * <p>
       * Same as {@link yfiles.algorithms.Graph#nodeCount}.
       * </p>
       * @type {number}
       */
      n:number;
      /**
       * Gets the number of nodes in this graph.
       * @type {number}
       */
      nodeCount:number;
      /**
       * Gets the number of edges in this graph.
       * <p>
       * Same as {@link yfiles.algorithms.Graph#edgeCount}.
       * </p>
       * @type {number}
       */
      e:number;
      /**
       * Gets the number of edges in this graph.
       * @type {number}
       */
      edgeCount:number;
      /**
       * Gets <code>true</code> if this graph contains no nodes.
       * @type {boolean}
       */
      empty:boolean;
      /**
       * Gets the first node in this graph.
       * @type {yfiles.algorithms.Node}
       */
      firstNode:yfiles.algorithms.Node;
      /**
       * Gets the first edge in this graph.
       * @type {yfiles.algorithms.Edge}
       */
      firstEdge:yfiles.algorithms.Edge;
      /**
       * Gets the last node in this graph.
       * @type {yfiles.algorithms.Node}
       */
      lastNode:yfiles.algorithms.Node;
      /**
       * Gets the last edge in this graph.
       * @type {yfiles.algorithms.Edge}
       */
      lastEdge:yfiles.algorithms.Edge;
      /**
       * Gets all node maps that have been created by this graph but have not yet been disposed.
       * @see yfiles.algorithms.Graph#createNodeMap
       * @see yfiles.algorithms.Graph#disposeNodeMap
       * @type {Array.<yfiles.algorithms.INodeMap>}
       */
      registeredNodeMaps:yfiles.algorithms.INodeMap[];
      /**
       * Gets all edge maps that have been created by this graph but have not yet been disposed.
       * @see yfiles.algorithms.Graph#createEdgeMap
       * @see yfiles.algorithms.Graph#disposeEdgeMap
       * @type {Array.<yfiles.algorithms.IEdgeMap>}
       */
      registeredEdgeMaps:yfiles.algorithms.IEdgeMap[];
      /**
       * Gets an array of all data provider look-up keys that are registered with this graph.
       * @type {Array.<Object>}
       */
      dataProviderKeys:Object[];
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for
       * {@link yfiles.algorithms.Node}s that can be used to iterate over the nodes that are contained in this instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the graph. Note that changes to the graph structure
       * during the traversal should be carried out with great care.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Node>}
       */
      nodes:yfiles.collections.IEnumerable<yfiles.algorithms.Node>;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for
       * {@link yfiles.algorithms.Edge}s that can be used to iterate over the edges that are contained in this instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the graph. Note that changes to the graph structure
       * during the traversal should be carried out with great care.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Edge>}
       */
      edges:yfiles.collections.IEnumerable<yfiles.algorithms.Edge>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides methods that check structural properties of a given graph.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li><b>Cycle:</b> An edge path with vertices <code>v0, v1, v2, ... , vk</code> forms a cycle if <code>v0 = vk</code> and consists of at least one edge.</li>
     * <li><b>Acyclic graph:</b> A graph that contains no directed cycle.</li>
     * <li><b>Cyclic graph:</b> A graph that contains a directed cycle.</li>
     * <li><b>Connected graph:</b> A graph in which there exists an undirected path of edges between every pair of nodes.</li>
     * <li><b>Strongly connected graph:</b> A graph in which there exists a directed path between each pair of nodes.</li>
     * <li><b>Biconnected graph:</b> A graph that has no cut vertex or articulation point (i.e., a node whose removal disconnects the graph).</li>
     * <li><b>Bipartite graph:</b> A graph whose nodes can be partitioned into two sets such that each edge connects two nodes of different sets.</li>
     * <li>
     * <b>Tree graph:</b> An acyclic graph, in which any pair of vertices is connected through a path. If one vertex of a tree is distinguished
     * from the other vertices, then the tree is called <b>rooted tree</b>.
     * </li>
     * <li><b>N-ary tree graph:</b> A directed rooted tree where each node has a maximum of <code>n</code> children.</li>
     * <li><b>Forest graph:</b> A graph whose connected components are trees.</li>
     * <li><b>Simple graph:</b> A graph that contains no self-loops and parallel edges.</li>
     * <li><b>Planar graph:</b> A graph that can be drawn on the plane without edge crossings.</li>
     * </ul>
     * @class
     * @static
     */
    export interface GraphChecker extends Object{}
    export class GraphChecker {
      /**
       * Checks whether or not the given directed graph is acyclic.
       * <p>
       * A graph is called <i>acyclic</i> if it contains no directed cycle.
       * </p>
       * @param graph the given graph
       * @returns <code>true</code> if the graph is acyclic, <code>false</code>, otherwise
       * @static
       */
      static isAcyclic(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given undirected graph is biconnected.
       * <p>
       * A graph is called <i>biconnected</i> if it has no cut vertex or articulation point, i.e., no node whose removal disconnects the graph.
       * </p>
       * @param graph the given undirected graph
       * @returns <code>true</code> if the graph is biconnected, <code>false</code> otherwise
       * @static
       */
      static isBiconnected(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given undirected graph is bipartite.
       * <p>
       * A graph is called <i>bipartite</i> if its nodes can be partitioned into two sets such that each edge connects two nodes of different
       * sets.
       * </p>
       * @param graph the given undirected graph
       * @returns <code>true</code> if the graph is bipartite, <code>false</code> otherwise
       * @static
       */
      static isBipartite(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph is connected.
       * <p>
       * A graph is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
       * </p>
       * @param graph the given graph
       * @returns <code>true</code> if the graph is connected, <code>false</code> otherwise
       * @static
       */
      static isConnected(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given directed graph is cyclic.
       * <p>
       * A graph is called <i>cyclic</i> if it contains a directed cycle.
       * </p>
       * @param graph the given graph
       * @returns <code>true</code> if the graph is cyclic, <code>false</code>, otherwise
       * @static
       */
      static isCyclic(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether the given graph is a forest.
       * <p>
       * A graph is a <i>forest</i> if its connected components are trees.
       * </p>
       * @param graph the given graph
       * @returns <code>true</code> if the graph is a forest, <code>false</code> otherwise
       * @static
       */
      static isForest(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given undirected graph contains no multiple edges.
       * <p>
       * More precisely, the method returns <code>true</code> if the graph contains no two distinct edges <code>e1, e2</code> that connect the same pairs
       * of nodes in either direction.
       * </p>
       * @param graph the given undirected graph
       * @returns <code>true</code> if the graph contains no multiple edges, <code>false</code> otherwise
       * @static
       */
      static isMultipleEdgeFree(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph is a directed rooted tree where each node has a maximum of <code>n</code> children.
       * @param graph the given graph
       * @param n
       * @returns <code>true</code> if the graph is a directed rooted tree where each node has at most <code>n</code> children, <code>false</code> otherwise
       * @static
       */
      static isNaryTree(graph:yfiles.algorithms.Graph,n:number):boolean;
      /**
       * Checks whether or not the given directed graph is a directed rooted tree.
       * @param graph the given graph
       * @returns <code>true</code> if the graph is a directed rooted tree, <code>false</code> otherwise
       * @static
       */
      static isRootedTree(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph contains no self-loops.
       * @param graph the given graph
       * @returns <code>true</code> if the graph contains no self-loops, <code>false</code> otherwise
       * @static
       */
      static isSelfLoopFree(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given directed graph is simple.
       * <p>
       * A graph is called <i>simple</i> if it contains no two distinct edges <code>e1, e2</code> where
       * <code>e1.source() == e2.source() &amp;&amp; e1.target() == e2.target()</code>.
       * </p>
       * @param graph the given directed graph
       * @returns <code>true</code> if the graph is simple, <code>false</code> otherwise
       * @static
       */
      static isSimple(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given directed graph is strongly connected.
       * <p>
       * A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
       * </p>
       * @param graph the given directed graph
       * @returns <code>true</code> if the graph is strongly connected, <code>false</code>, otherwise
       * @static
       */
      static isStronglyConnected(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph is an undirected tree.
       * @param graph the given graph
       * @returns <code>true</code> if the graph is an undirected tree, <code>false</code> otherwise
       * @static
       */
      static isTree(graph:yfiles.algorithms.Graph):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides algorithms for determining certain connectivity components within a graph.
     * <p>
     * It also provides convenience methods for working with these components.
     * </p>
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li><b>Connected graph:</b> A graph is called connected if there exists an undirected path of edges between every pair of nodes.</li>
     * <li><b>Strongly connected graph:</b> A graph is called strongly connected if there exists a directed path between each pair of nodes.</li>
     * <li>
     * <b>Biconnected graph:</b> A graph is called biconnected if it has no cut vertex or articulation point (i.e., a node whose removal disconnects the
     * remaining graph).
     * </li>
     * </ul>
     * @class
     * @static
     */
    export interface GraphConnectivity extends Object{}
    export class GraphConnectivity {
      /**
       * Calculates the biconnected components and the articulation points of a given undirected graph and returns the number of
       * biconnected components.
       * <p>
       * Articulation points are returned in the form of a {@link yfiles.algorithms.INodeMap} that returns for each node a boolean value
       * indicating whether or not it is an articulation point.
       * </p>
       * @param graph the input graph
       * @param compNum the {@link } that will be filled during the execution and returns the zero-based index of the
       * biconnected component to which each edge belongs or <code>-1</code> for self-loops
       * @param [aPoint=null] the {@link } that will be filled during the execution and returns a boolean value indicating whether
       * or not a given node is an articulation point
       * @returns the number of biconnected components
       * @see yfiles.algorithms.GraphConnectivity#biconnectedComponents
       * @see yfiles.algorithms.GraphConnectivity#biconnectedComponents
       * @static
       */
      static biconnectedComponents(graph:yfiles.algorithms.Graph,compNum:yfiles.algorithms.IEdgeMap,aPoint?:yfiles.algorithms.INodeMap):number;
      /**
       * Calculates the biconnected components of a given undirected graph.
       * <p>
       * The result is returned as an array of
       * {@link yfiles.algorithms.EdgeList}s each containing all edges that belong to the same biconnected component.
       * </p>
       * @param graph the input graph
       * @returns an array of {@link }s each containing all edges that belong to the same biconnected component
       * @see yfiles.algorithms.GraphConnectivity#biconnectedComponents
       * @static
       */
      static biconnectedComponents(graph:yfiles.algorithms.Graph):yfiles.algorithms.EdgeList[];
      /**
       * Calculates the connected components of a given graph and returns their number.
       * <p>
       * A graph <code>G</code> is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
       * </p>
       * <p>
       * The connected components of a graph are the maximal connected subgraphs of which the graph consists.
       * </p>
       * @param compNum the {@link } that will be filled during the execution and returns the zero-based index of the
       * connected component to which each node belongs
       * @param graph
       * @returns the number of connected components of the given graph
       * @static
       */
      static connectedComponents(graph:yfiles.algorithms.Graph,compNum:yfiles.algorithms.INodeMap):number;
      /**
       * Calculates the connected components of a given graph.
       * <p>
       * A graph <code>G</code> is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
       * </p>
       * <p>
       * The connected components of a graph are the maximal connected subgraphs of which the graph consists.
       * </p>
       * @param graph the input graph
       * @returns an array of {@link }s each of which contains the nodes that belong to the same connected component
       * @static
       */
      static connectedComponents(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList[];
      /**
       * Determines the direct or indirect neighbors of a given set of nodes.
       * <ul>
       * <li>A <i>direct</i> neighbor of a node is directly connected by an edge to that node.</li>
       * <li>An <i>indirect</i> neighbor of a node is directly connected to another direct or indirect neighbor of a node.</li>
       * </ul>
       * <p>
       * The order of the returned nodes is determined by a breadth first search. No start node will be part of the resulting
       * set.
       * </p>
       * <p>
       * To obtain the result, an integer value should be given as input that limits the distance between a start node and a
       * returned node. For all returned nodes there must be a path to a start node that has a length equal to or smaller than
       * this distance.
       * </p>
       * <p>
       * Setting the maximum distance to <code>1</code> will only yield the direct neighbors of all start nodes. On the other hand, setting
       * the maximum distance to <code>graph.N()</code> or larger will yield all neighbors of all start nodes.
       * </p>
       * @param graph
       * @param startNodes
       * @param maxDistance
       * @returns a {@link } that contains all direct and indirect neighbors of a node
       * @static
       */
      static getNeighbors(graph:yfiles.algorithms.Graph,startNodes:yfiles.algorithms.NodeList,maxDistance:number):yfiles.algorithms.NodeList;
      /**
       * Determines the direct or indirect predecessors of a given list of nodes.
       * <ul>
       * <li>A <i>direct</i> predecessor of a node is the source node of an incoming edge connected to a node.</li>
       * <li>An <i>indirect</i> predecessor of a node is a direct predecessor to another predecessor of a node.</li>
       * </ul>
       * <p>
       * The order of the returned nodes is determined by a breadth first search. No start node will be part of the resulting
       * set.
       * </p>
       * <p>
       * To obtain the result, an integer value should be given as input that limits the distance between a start node and a
       * returned node. For all returned nodes there must be a path to a start node that has a length equal to or smaller than
       * this distance.
       * </p>
       * <p>
       * Setting the maximum distance to <code>1</code> will only yield the direct predecessors of all start nodes. On the other hand, setting
       * the maximum distance to <code>graph.N()</code> or larger will yield all predecessors of all start nodes.
       * </p>
       * @param graph the given graph
       * @param startNodes a {@link } containing the nodes from which the search starts
       * @param maxDistance an integer value that limits the distance between a start node and a returned node
       * @returns a {@link } that contains all direct and indirect predecessors of a node
       * @static
       */
      static getPredecessors(graph:yfiles.algorithms.Graph,startNodes:yfiles.algorithms.NodeList,maxDistance:number):yfiles.algorithms.NodeList;
      /**
       * Determines the direct or indirect successors of a given list of nodes.
       * <ul>
       * <li>A <i>direct</i> successor of a node is the target node of an outgoing edge connected to a node.</li>
       * <li>An <i>indirect</i> successor of a node is a direct successor to another successor of a node.</li>
       * </ul>
       * <p>
       * The order of the returned nodes is determined by a breadth first search. No start node will be part of the resulting
       * set.
       * </p>
       * <p>
       * To obtain the result, an integer value should be given as input that limits the distance between a start node and a
       * returned node. For all returned nodes there must be a path to a start node that has a length equal to or smaller than
       * this distance.
       * </p>
       * <p>
       * Setting the maximum distance to <code>1</code> will only yield the direct successors of all start nodes. On the other hand, setting
       * the maximum distance to <code>graph.N()</code> or larger will yield all successors of all start nodes.
       * </p>
       * @param graph the given graph
       * @param startNodes a {@link } containing the nodes from which the search starts
       * @param maxDistance an integer value that limits the distance between a start node and a returned node
       * @returns a {@link } that contains all direct and indirect successors of a node
       * @static
       */
      static getSuccessors(graph:yfiles.algorithms.Graph,startNodes:yfiles.algorithms.NodeList,maxDistance:number):yfiles.algorithms.NodeList;
      /**
       * Checks whether or not the given undirected graph is biconnected.
       * <p>
       * A graph is called <i>biconnected</i> if it has no cut vertex or articulation point, i.e., no node whose removal disconnects the graph.
       * </p>
       * @param graph the given undirected graph
       * @returns <code>true</code> if the graph is biconnected, <code>false</code> otherwise
       * @static
       */
      static isBiconnected(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph is connected.
       * <p>
       * A graph is called <i>connected</i> if there exists an undirected path of edges between every pair of nodes.
       * </p>
       * @param graph the given graph
       * @returns <code>true</code> if the graph is connected, <code>false</code> otherwise
       * @static
       */
      static isConnected(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given directed graph is strongly connected.
       * <p>
       * A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
       * </p>
       * @param graph the given directed graph
       * @returns <code>true</code> if the graph is strongly connected, <code>false</code>, otherwise
       * @static
       */
      static isStronglyConnected(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Makes the given graph biconnected by inserting a minimum number of edges in the graph.
       * <p>
       * The given graph is considered to be undirected.
       * </p>
       * @param graph the input graph
       * @returns an {@link } containing the edges added to the graph
       * @static
       */
      static makeBiconnected(graph:yfiles.algorithms.Graph):yfiles.algorithms.EdgeList;
      /**
       * Makes a graph connected by adding additional edges to the graph.
       * <p>
       * The number of edges that will be added equals the number of separate components of the original graph minus <code>1</code>.
       * </p>
       * @param graph the input graph
       * @returns an {@link } containing the edges added to the graph
       * @static
       */
      static makeConnected(graph:yfiles.algorithms.Graph):yfiles.algorithms.EdgeList;
      /**
       * Determines the set of nodes that are reachable from a given node when a set of edges that cannot be traversed is
       * specified.
       * <p>
       * The result is based on a depth first search.
       * </p>
       * @param graph the given graph
       * @param start the node from which the search starts
       * @param directed <code>true</code> if the edges should be traversed from source to target, <code>false</code> if edges can be traversed in both directions
       * @param [forbidden=null] an array that holds for each {@link } a {@link } value indicating whether or not an edge can be traversed; an edge
       * <code>e</code> is marked as forbidden if <code>forbidden[e.index()] == true</code>
       * @param reached an array that will be filled during the execution and returns for each {@link } a {@link } value based on whether
       * the node can be reached during the DFS; if a node <code>v</code> is reachable, then <code>reached[v.index()] = true</code>
       * @static
       */
      static reachable(graph:yfiles.algorithms.Graph,start:yfiles.algorithms.Node,directed:boolean,reached:boolean[],forbidden?:boolean[]):void;
      /**
       * Calculates the strongly connected components of a given graph and returns their number.
       * <p>
       * A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
       * </p>
       * <p>
       * The strongly connected components of a graph are the strongly connected subgraphs of which it consists.
       * </p>
       * @param graph the input graph
       * @param compNum the {@link } that will be filled during the execution and returns the zero-based index of the
       * connected component to which each node belongs
       * @returns the number of strongly connected components of the given graph
       * @see yfiles.algorithms.GraphConnectivity#stronglyConnectedComponents
       * @static
       */
      static stronglyConnectedComponents(graph:yfiles.algorithms.Graph,compNum:yfiles.algorithms.INodeMap):number;
      /**
       * Calculates the strongly connected components of a given graph.
       * <p>
       * A graph is called <i>strongly connected</i> if there exists a directed path between each pair of nodes.
       * </p>
       * <p>
       * The strongly connected components of a graph are the strongly connected subgraphs of which it consists.
       * </p>
       * @param graph the input graph
       * @returns an array of
       * {@link }s each of which contains the nodes that belong to the same strongly connected component
       * @see yfiles.algorithms.GraphConnectivity#stronglyConnectedComponents
       * @static
       */
      static stronglyConnectedComponents(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList[];
      /**
       * Transforms the return values of {@link yfiles.algorithms.GraphConnectivity#biconnectedComponents} to an array of
       * {@link yfiles.algorithms.EdgeList}s, like it is returned by {@link yfiles.algorithms.GraphConnectivity#biconnectedComponents}.
       * @param graph the input graph
       * @param compNum the {@link } that will be filled during the execution and returns the zero-based index of the
       * connected component to which each edge belongs
       * @param maxCompNum the maximum number of biconnected components
       * @returns an array of {@link }s each containing all edges that belong to the same biconnected component
       * @static
       */
      static toEdgeListArray(graph:yfiles.algorithms.Graph,compNum:yfiles.algorithms.IEdgeMap,maxCompNum:number):yfiles.algorithms.EdgeList[];
      /**
       * Transforms the return values of method {@link yfiles.algorithms.GraphConnectivity#connectedComponents} to an array of
       * {@link yfiles.algorithms.NodeList}s, like it is returned by {@link yfiles.algorithms.GraphConnectivity#connectedComponents}.
       * @param graph the input graph
       * @param compNum the {@link } that will be filled during the execution and returns the zero-based index of the
       * connected component to which each node belongs
       * @param maxCompNum the maximum number of connected components
       * @returns an array of {@link }s each of which contains the nodes that belong to the same connected component
       * @static
       */
      static toNodeListArray(graph:yfiles.algorithms.Graph,compNum:yfiles.algorithms.INodeMap,maxCompNum:number):yfiles.algorithms.NodeList[];
      static $class:yfiles.lang.Class;
    }
    /**
     * Common base type for both {@link yfiles.algorithms.Node} and {@link yfiles.algorithms.Edge}.
     * <p>
     * This type does not add public functionality to its base type.
     * </p>
     * @class
     */
    export interface GraphObject extends Object{}
    export class GraphObject {
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides the result of hierarchical clustering algorithms by means of a binary tree structure.
     * <p>
     * The dendrogram is created using the agglomerative strategy (i.e., a bottom-up approach) based on the following steps:
     * </p>
     * <ul>
     * <li>
     * At the beginning (i.e., at level
     * <code>0</code>), each node belongs to its own cluster. At this point, each node of the original graph (the one that will be
     * clustered) is mapped to one of the dendrogram nodes that represent a leaf node (i.e., a node with no children).
     * </li>
     * <li>
     * At each step, pairs of clusters are merged while moving up to the hierarchy. In this case, a new dendrogram node is
     * created and the corresponding dissimilarity value between the two merged clusters can be obtained using
     * {@link yfiles.algorithms.Dendrogram#getDissimilarityValue}. The dissimilarity values are monotone increasing starting from zero
     * which corresponds to the leaf nodes of the dendrogram.
     * </li>
     * <li>
     * The algorithm continues until all nodes belong to the same cluster. At this point, the root of the dendrogram is reached
     * and can be obtained using method {@link yfiles.algorithms.Dendrogram#root}.
     * </li>
     * </ul>
     * <p>
     * The dendrogram is a binary tree that is directed from the root to the leaves. This means that one can iterate the
     * dendrogram starting with the {@link yfiles.algorithms.Dendrogram#root root node}, moving on to its {@link yfiles.algorithms.Dendrogram#getChildren children}, then moving on to their children and so on.
     * </p>
     * <p>
     * From each node of the dendrogram, it is possible to retrieve the set of nodes of the original graph that belong to this
     * dendrogram node using {@link yfiles.algorithms.Dendrogram#getClusterNodes}. This method returns an array of
     * {@link yfiles.algorithms.NodeList}s that contain the nodes of all clusters that are merged when this dendrogram node was
     * created. For example, say that clusters <code>C1 = {v1, v2}</code> and <code>C2 = {v3, v4}</code> are merged, where <code>{v1, v2, v3, v4}</code> are nodes of
     * the original graph. Say that dendrogram node <code>u</code> is created by this merging. Then, the cluster nodes of <code>u</code> will be returned
     * as two {@link yfiles.algorithms.NodeList}s where the first contains nodes <code>v1, v2</code> and the second <code>v3, v4</code>.
     * </p>
     * @see yfiles.algorithms.Groups#hierarchicalClustering
     * @see yfiles.algorithms.Groups#hierarchicalClustering
     * @see yfiles.algorithms.Groups#hierarchicalClustering
     * @class
     */
    export interface Dendrogram extends Object{}
    export class Dendrogram {
      /**
       * Returns a {@link yfiles.algorithms.NodeList} that contains the child nodes of the given parent node.
       * <p>
       * If the given node is a {@link yfiles.algorithms.Dendrogram#isLeaf leaf} node, the returned list will be empty.
       * </p>
       * @param parent the parent node for which to retrieve the child nodes
       * @returns the list of child nodes of the given parent node
       */
      getChildren(parent:yfiles.algorithms.Node):yfiles.algorithms.NodeList;
      /**
       * Returns an array of
       * {@link yfiles.algorithms.NodeList}s that contain the nodes of the original graph that are associated with the given dendrogram
       * node.
       * <p>
       * Each such {@link yfiles.algorithms.NodeList} represents nodes that belong to the same cluster. For example, say that clusters <code>C1 = {v1, v2}</code>
       * and <code>C2 = {v3, v4}</code> are merged, where <code>{v1, v2, v3, v4}</code> are nodes of the original graph. Say that dendrogram node <code>u</code> is
       * created by this merging. Then, the cluster nodes of <code>u</code> will be returned as two {@link yfiles.algorithms.NodeList}s where the
       * first contains nodes <code>v1, v2</code> and the second <code>v3, v4</code>.
       * </p>
       * @param node the given dendrogram node
       * @returns an array of
       * {@link }s that contain the nodes of the original graph that are associated with the given dendrogram
       * node
       */
      getClusterNodes(node:yfiles.algorithms.Node):yfiles.algorithms.NodeList[];
      /**
       * Returns the dissimilarity value associated with the given node of the dendrogram.
       * @param node the given dendrogram node
       * @returns the dissimilarity value associated with the given dendrogram node
       */
      getDissimilarityValue(node:yfiles.algorithms.Node):number;
      /**
       * Returns the level of the given node of the dendrogram.
       * @param node the given dendrogram node
       * @returns the level of the given dendrogram node
       */
      getLevel(node:yfiles.algorithms.Node):number;
      /**
       * Returns the node of the dendrogram that belongs to the given level index.
       * <p>
       * If the given level index is greater than the {@link yfiles.algorithms.Dendrogram#levelCount number of levels}, then this method will return <code>null</code>.
       * </p>
       * @param level the given level index
       * @returns the node of the dendrogram that belongs to the given level index or <code>null</code> if there exists no level with the given index.
       * @throws {Stubs.Exceptions.ArgumentError} if the given level is negative or greater than the {@link yfiles.algorithms.Dendrogram#nodeCount node count} minus one.
       */
      getNodeAtLevel(level:number):yfiles.algorithms.Node;
      /**
       * Returns the node of original graph that is mapped with the given leaf node of the dendrogram.
       * @param node the given leaf node
       * @returns the node of original graph that is mapped with the given leaf node or <code>null</code> if the given node is an inner dendrogram
       * node
       */
      getOriginalNode(node:yfiles.algorithms.Node):yfiles.algorithms.Node;
      /**
       * Returns whether or not the given dendrogram node is a leaf node.
       * <p>
       * A leaf node has no further children and represents a node of the original graph. The original node can be retrieved via
       * {@link yfiles.algorithms.Dendrogram#getOriginalNode}.
       * </p>
       * @param node a dendrogram node
       * @returns <code>true</code> if the given node is a leaf node, <code>false</code> otherwise
       */
      isLeaf(node:yfiles.algorithms.Node):boolean;
      /**
       * Gets the root node of the dendrogram.
       * <p>
       * A dendrogram traversal can be implemented by starting from this node, then moving on to its children using method
       * {@link yfiles.algorithms.Dendrogram#getChildren}, and so on.
       * </p>
       * @type {yfiles.algorithms.Node}
       */
      root:yfiles.algorithms.Node;
      /**
       * Gets the number of nodes in this dendrogram.
       * @type {number}
       */
      nodeCount:number;
      /**
       * Gets the number of levels this dendrogram has.
       * @type {number}
       */
      levelCount:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An interface that determines the distance between two nodes of a graph.
     * <p>
     * It is used by the hierarchical clustering algorithm in order to determine the distance between any two nodes of a
     * graph.
     * </p>
     * @see yfiles.algorithms.Groups#hierarchicalClustering
     * @see yfiles.algorithms.Groups#hierarchicalClustering
     * @see yfiles.algorithms.Groups#hierarchicalClustering
     * @interface
     */
    export interface INodeDistanceProvider extends Object{
      /**
       * Returns the distance between two given nodes of a graph.
       * <p>
       * Implementations of this method have to guarantee that the returned value is greater or equal to zero. Furthermore, the
       * distances between two nodes have to be symmetric, i.e., the distance between node <code>n1</code> and <code>n2</code> is equal to the distance
       * between node <code>n2</code> and <code>n1</code>.
       * </p>
       * @param n1 the first node
       * @param n2 the second node
       * @returns the distance between the two given nodes
       * @abstract
       */
      getDistance(n1:yfiles.algorithms.Node,n2:yfiles.algorithms.Node):number;
    }
    var INodeDistanceProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class provides methods for automatically partitioning nodes of a graph into groups.
     * <p>
     * Partitions can be defined using edge betweenness centrality, biconnectivity, k-means clustering or hierarchical
     * clustering.
     * </p>
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li><b>Betweenness centrality</b> is a measure for how often a node lies on a shortest path between each pair of nodes in the graph.</li>
     * <li><b>Biconnected graph</b> is a graph that has no cut vertex or articulation point (i.e., a node whose removal disconnects the graph).</li>
     * <li><b>K-means clustering algorithm</b> partitions the nodes of a graph into <code>k</code>-clusters based on their positions on the plane and a given distance metric.</li>
     * <li><b>Hierarchical clustering</b> creates a hierarchy of clusters in a bottom-to-top approach based on some distance metric and linkage.</li>
     * </ul>
     * @class
     * @static
     */
    export interface Groups extends Object{}
    export class Groups {
      /**
       * This method partitions the graph by analyzing its <i>biconnected components</i>.
       * <p>
       * Nodes will be grouped such that the nodes within each group are biconnected. Nodes that belong to multiple biconnected
       * components will be assigned to exactly one of these components.
       * </p>
       * @param graph the input graph
       * @param groupIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @returns the resulting number of different groups
       * @static
       */
      static biconnectedComponentGrouping(graph:yfiles.algorithms.Graph,groupIDs:yfiles.algorithms.INodeMap):number;
      /**
       * Partitions the graph into groups using <i>edge betweenness clustering</i> proposed by Girvan and Newman.
       * <p>
       * In each iteration the edge with the highest betweenness centrality is removed from the graph. The method stops, if there
       * are no more edges to remove or if the requested maximum number of groups is found. The clustering with the best quality
       * reached during the process is returned.
       * </p>
       * <p>
       * The algorithm includes several heuristic speed-up techniques available through the quality/time ratio. For the highest
       * quality setting, it is used almost unmodified. The fast betweenness approximation of Brandes and Pich (<i>Centrality Estimation in Large Networks</i>) is employed for
       * values around <code>0.5</code>. Typically, this results in a tiny decrease in quality but a large speed-up and is the recommended
       * setting. To achieve the lowest running time, a local betweenness calculation is used (Gregory: <i>Local Betweenness for Finding Communities in Networks</i>).
       * </p>
       * <p>
       * The method requires the maximum number of groups that will be returned. The smaller this value is, the faster the
       * overall computation time. The upper bound on the number of groups is
       * <code>graph.N()</code>. Also, the number of returned groups is never smaller than the number of connected components of the graph.
       * </p>
       * @param graph the input graph
       * @param clusterIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @param qualityTimeRatio a value between <code>0.0</code> (low quality, fast) and <code>1.0</code> (high quality, slow); the recommended value is <code>0.5</code>
       * @param minGroupCount the minimum number of groups that will be returned
       * @param maxGroupCount the maximum number of groups that will be returned
       * @param refine <code>true</code> if the algorithm refines the current grouping, <code>false</code> if the algorithm discards the current grouping
       * @returns the resulting number of different groups
       * @throws {Stubs.Exceptions.ArgumentError} if <code>minGroupCount > maxGroupCount</code> or <code>minGroupCount > graph.N()</code> or <code>maxGroupCount &lt;= 0</code>
       * @static
       */
      static edgeBetweennessClustering(graph:yfiles.algorithms.Graph,clusterIDs:yfiles.algorithms.INodeMap,qualityTimeRatio:number,minGroupCount:number,maxGroupCount:number,refine:boolean):number;
      /**
       * Partitions the graph into groups using <i>edge betweenness centrality</i>.
       * <p>
       * In each iteration the edge with the highest betweenness centrality is removed from the graph. The method stops, if there
       * are no more edges to remove. The clustering with the best quality reached during the process will be returned.
       * </p>
       * <p>
       * The method requires the maximum number of groups that will be returned. The smaller this value is, the faster the
       * overall computation time. The upper bound on the number of groups is
       * <code>graph.N()</code>. Also, the number of returned groups is never smaller than the number of connected components of the graph.
       * </p>
       * @param graph the input graph
       * @param clusterIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param minGroupCount the minimum number of groups that will be returned
       * @param maxGroupCount the maximum number of groups that will be returned
       * @param edgeCosts the {@link } that holds a positive {@link } cost or <code>null</code> if the edges of the graph are considered to be
       * of equal cost
       * @returns the resulting number of different groups
       * @throws {Stubs.Exceptions.ArgumentError} if <code>minGroupCount > maxGroupCount</code> or <code>minGroupCount > graph.N()</code> or <code>maxGroupCount &lt;= 0</code>
       * @static
       */
      static edgeBetweennessClustering(graph:yfiles.algorithms.Graph,clusterIDs:yfiles.algorithms.INodeMap,directed:boolean,minGroupCount:number,maxGroupCount:number,edgeCosts:yfiles.algorithms.IDataProvider):number;
      /**
       * Partitions the graph into clusters based on hierarchical clustering, while the dendrogram is cut based on a given
       * maximum number of clusters.
       * <p>
       * The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which at the
       * beginning each node belongs to its own cluster. At each step pairs of clusters are merged while moving up to the
       * hierarchy. The dissimilarity between clusters is determined based on the given linkage and the given node distances. The
       * algorithm continues until all nodes belong to the same cluster.
       * </p>
       * <p>
       * The result will be given based on the given maximum number of clusters value that is used for cutting the hierarchical
       * tree at a point such that the number of remaining clusters equals to this value.
       * </p>
       * <p>
       * The maximum number of clusters needs to be greater than zero and less than the number of the nodes of the graph.
       * </p>
       * @param graph the input graph
       * @param maxCluster the maximum number of clusters that determines where to cut the hierarchic tree into clusters
       * @param clusterIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @param distances a given {@link } object that determines the distance between any two graph nodes
       * @param linkage one of the predefined linkage values
       * @returns the resulting number of clusters
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown linkage is given or if the maximum number of clusters is less than or equal to zero or greater than the
       * number of nodes of the graph
       * @static
       */
      static hierarchicalClustering(graph:yfiles.algorithms.Graph,maxCluster:number,clusterIDs:yfiles.algorithms.INodeMap,distances:yfiles.algorithms.INodeDistanceProvider,linkage:yfiles.algorithms.Linkage):number;
      /**
       * Partitions the graph into clusters based on hierarchical clustering, while the dendrogram is cut based on a given
       * cut-off value.
       * <p>
       * The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which at the
       * beginning each node belongs to its own cluster. At each step pairs of clusters are merged while moving up to the
       * hierarchy. The dissimilarity between clusters is determined based on the given linkage and the given node distances. The
       * algorithm continues until all nodes belong to the same cluster.
       * </p>
       * <p>
       * The result will be given based on the given cut-off value that is used for cutting the hierarchical tree at a point such
       * that the dissimilarity values of the nodes that remain at the dendrogram are less than this value.
       * </p>
       * @param graph the input graph
       * @param clusterIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @param distances a given {@link } object that determines the distance between any two nodes
       * @param linkage one of the predefined linkage values
       * @param cutOff the cut-off value that determines where to cut the hierarchic tree into clusters
       * @returns the resulting number of clusters
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown linkage is used
       * @static
       */
      static hierarchicalClustering(graph:yfiles.algorithms.Graph,clusterIDs:yfiles.algorithms.INodeMap,distances:yfiles.algorithms.INodeDistanceProvider,linkage:yfiles.algorithms.Linkage,cutOff:number):number;
      /**
       * Partitions the graph into clusters based on hierarchical clustering.
       * <p>
       * The clustering is performed using the agglomerative strategy i.e., a bottom-up approach according to which at the
       * beginning each node belongs to its own cluster. At each step pairs of clusters are merged while moving up to the
       * hierarchy. The dissimilarity between clusters is determined based on the given linkage and the given node distances
       * metric. The algorithm continues until all nodes belong to the same cluster.
       * </p>
       * <p>
       * The result is returned as a {@link yfiles.algorithms.Dendrogram} object which represents the result of the clustering algorithm
       * as a binary tree structure. It can easily be traversed by starting from the {@link yfiles.algorithms.Dendrogram#root root node} and moving on to nodes of the next
       * level via method {@link yfiles.algorithms.Dendrogram#getChildren}.
       * </p>
       * @param graph the input graph
       * @param distances a given {@link } object that determines the distance between any two nodes
       * @param linkage one of the predefined linkage values
       * @returns a {@link } which represents the result of the clustering as a binary tree
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown linkage is given
       * @static
       */
      static hierarchicalClustering(graph:yfiles.algorithms.Graph,distances:yfiles.algorithms.INodeDistanceProvider,linkage:yfiles.algorithms.Linkage):yfiles.algorithms.Dendrogram;
      /**
       * Partitions the graph into clusters using k-means clustering algorithm.
       * <p>
       * The nodes of the graph will be partitioned in <code>k</code> clusters based on their positions such that their distance from the
       * cluster's mean (centroid) is minimized.
       * </p>
       * <p>
       * The distance can be defined using diverse metrics as euclidean distance, euclidean-squared distance, manhattan distance
       * or chebychev distance.
       * </p>
       * @param graph the input graph
       * @param clusterIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @param nodePositions the {@link } that holds a {@link  point} representing the current position of each node in the graph
       * @param distanceMetric one of the predefined distance metrics
       * @param k the number of clusters
       * @param [iterations=100] the maximum number of iterations performed by the algorithm for convergence
       * @param [centroids=null] the initial centroids
       * @returns the number of resulting (non-empty) clusters
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance metric is not supported
       * @static
       */
      static kMeansClustering(graph:yfiles.algorithms.Graph,clusterIDs:yfiles.algorithms.INodeMap,nodePositions:yfiles.algorithms.IDataProvider,distanceMetric:yfiles.algorithms.DistanceMetric,k:number,iterations?:number,centroids?:yfiles.algorithms.YPoint[]):number;
      /**
       * Partitions the graph into clusters using k-means clustering algorithm.
       * <p>
       * The nodes of the graph will be partitioned in <code>k</code> clusters based on their positions such that their distance from the
       * cluster's mean (centroid) is minimized.
       * </p>
       * <p>
       * The distance can be defined using diverse metrics as euclidean distance, euclidean-squared distance, manhattan distance
       * or chebychev distance.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.graph the input graph
       * @param options.clusterIDs the {@link } that will be filled during the execution and returns an integer value (cluster ID) for
       * each node
       * @param options.nodePositions the {@link } that holds a {@link  point} representing the current position of each node in the graph
       * @param options.distanceMetric one of the predefined distance metrics
       * @param options.k the number of clusters
       * @param [options.iterations=100] the maximum number of iterations performed by the algorithm for convergence
       * @param [options.centroids=null] the initial centroids
       * @returns the number of resulting (non-empty) clusters
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance metric is not supported
       * @static
       */
      static kMeansClustering(options:{graph:yfiles.algorithms.Graph,clusterIDs:yfiles.algorithms.INodeMap,nodePositions:yfiles.algorithms.IDataProvider,distanceMetric:yfiles.algorithms.DistanceMetric,k:number,iterations?:number,centroids?:yfiles.algorithms.YPoint[]}):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A general interface for iterating over a collection of objects.
     * <p>
     * It can be regarded as a read-only view of such a collection.
     * </p>
     * <p>
     * A YCursor acts like a movable pointer on the elements of a collection. The pointer can be moved forward and backward and
     * the element currently pointed on can be accessed. The removal of elements can only be performed on the provider of the
     * cursor, not on the cursor itself. (That's why the cursor presents a read-only view.)
     * </p>
     * <p>
     * Implementations of this interface do not need to support operations marked "optional."
     * </p>
     * @interface
     */
    export interface ICursor extends Object{
      /**
       * Moves this cursor one position forward.
       * @abstract
       */
      next():void;
      /**
       * Moves this cursor one position backward (optional).
       * @abstract
       */
      prev():void;
      /**
       * Moves this cursor to the first valid cursor position (optional).
       * @abstract
       */
      toFirst():void;
      /**
       * Moves this cursor to the last valid cursor position (optional).
       * @abstract
       */
      toLast():void;
      /**
       * Gets <code>true</code> if the current cursor position is valid.
       * @abstract
       * @type {boolean}
       */
      ok:boolean;
      /**
       * Gets the object currently pointed on.
       * @abstract
       * @type {Object}
       */
      current:Object;
      /**
       * Gets the number of elements that can be accessed with this cursor.
       * @abstract
       * @type {number}
       */
      size:number;
    }
    var ICursor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A general interface for setting data.
     * <p>
     * A data acceptor associates data with data holders. It constitutes a write-only view on particular data.
     * </p>
     * @interface
     */
    export interface IDataAcceptor extends Object{
      /**
       * Sets an object value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @param value
       * @abstract
       */
      set(dataHolder:Object,value:Object):void;
      /**
       * Sets a boolean value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @param value
       * @abstract
       */
      setBoolean(dataHolder:Object,value:boolean):void;
      /**
       * Sets an integer value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @param value
       * @abstract
       */
      setInt(dataHolder:Object,value:number):void;
      /**
       * Sets a double value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @param value
       * @abstract
       */
      setNumber(dataHolder:Object,value:number):void;
    }
    var IDataAcceptor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that combines the {@link yfiles.algorithms.IDataProvider} and {@link yfiles.algorithms.IDataAcceptor} interfaces.
     * <p>
     * This interface does not declare any additional methods.
     * </p>
     * @interface
     * @implements {yfiles.algorithms.IDataProvider}
     * @implements {yfiles.algorithms.IDataAcceptor}
     */
    export interface IDataMap extends Object,yfiles.algorithms.IDataProvider,yfiles.algorithms.IDataAcceptor{
    }
    var IDataMap:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A general interface for data provision.
     * <p>
     * A data provider grants access to data associated with one or more data holders. It constitutes a read-only view on
     * particular data.
     * </p>
     * @interface
     */
    export interface IDataProvider extends Object{
      /**
       * Returns an object value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @returns 
       * @abstract
       */
      get(dataHolder:Object):Object;
      /**
       * Returns a boolean value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @returns 
       * @abstract
       */
      getBoolean(dataHolder:Object):boolean;
      /**
       * Returns an integer value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @returns 
       * @abstract
       */
      getInt(dataHolder:Object):number;
      /**
       * Returns a double value associated with the given data holder.
       * <p>
       * This method may throw an UnsupportedOperationException.
       * </p>
       * @param dataHolder
       * @returns 
       * @abstract
       */
      getNumber(dataHolder:Object):number;
    }
    var IDataProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A cursor interface for iterating over edges.
     * @interface
     * @implements {yfiles.algorithms.ICursor}
     */
    export interface IEdgeCursor extends Object,yfiles.algorithms.ICursor{
      /**
       * Moves the cursor to the cyclic next element of the underlying sequence.
       * <p>
       * This is the next element if available, else it is the first element.
       * </p>
       * @abstract
       */
      cyclicNext():void;
      /**
       * Moves the cursor to the cyclic previous element of the underlying sequence.
       * <p>
       * This is the previous element if available, else it is the last element.
       * </p>
       * @abstract
       */
      cyclicPrev():void;
      /**
       * Gets the edge at the current location of the cursor.
       * <p>
       * This method is the typed variant of {@link yfiles.algorithms.ICursor#current}.
       * </p>
       * @abstract
       * @type {yfiles.algorithms.Edge}
       */
      edge:yfiles.algorithms.Edge;
    }
    var IEdgeCursor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides access to data associated with an edge.
     * <p>
     * An edge map can be considered as a map that allows only edges as keys. Edge keys of an edge map must belong to the same
     * graph.
     * </p>
     * <p>
     * There are data access methods defined for the most common typed values as well.
     * </p>
     * <p>
     * The edge values are initialized with Java(TM) default values (null, 0, 0.0, false) upon initialization.
     * </p>
     * @interface
     * @implements {yfiles.algorithms.IDataProvider}
     * @implements {yfiles.algorithms.IDataAcceptor}
     * @implements {yfiles.algorithms.IDataMap}
     */
    export interface IEdgeMap extends Object,yfiles.algorithms.IDataProvider,yfiles.algorithms.IDataAcceptor,yfiles.algorithms.IDataMap{
    }
    var IEdgeMap:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A cursor interface for iterating over nodes.
     * @interface
     * @implements {yfiles.algorithms.ICursor}
     */
    export interface INodeCursor extends Object,yfiles.algorithms.ICursor{
      /**
       * Moves the cursor to the cyclic next element of the underlying sequence.
       * <p>
       * This is the next element if available, else it is the first element.
       * </p>
       * @abstract
       */
      cyclicNext():void;
      /**
       * Moves the cursor to the cyclic previous element of the underlying sequence.
       * <p>
       * This is the previous element if available, else it is the last element.
       * </p>
       * @abstract
       */
      cyclicPrev():void;
      /**
       * Gets the node at the current location of the cursor.
       * <p>
       * This method is the typed variant of {@link yfiles.algorithms.ICursor#current}.
       * </p>
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      node:yfiles.algorithms.Node;
    }
    var INodeCursor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Provides access to data associated with a node.
     * <p>
     * A node map can be considered as a map that allows only nodes as keys. Node keys of a node map must belong to the same
     * graph.
     * </p>
     * <p>
     * There are data access methods defined for the most common typed values as well.
     * </p>
     * <p>
     * The node values are initialized with Java(TM) default values (null, 0, 0.0, false) upon initialization.
     * </p>
     * @interface
     * @implements {yfiles.algorithms.IDataProvider}
     * @implements {yfiles.algorithms.IDataAcceptor}
     * @implements {yfiles.algorithms.IDataMap}
     */
    export interface INodeMap extends Object,yfiles.algorithms.IDataProvider,yfiles.algorithms.IDataAcceptor,yfiles.algorithms.IDataMap{
    }
    var INodeMap:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This is a generic interface for classes that provide an ordering for the nodes of a graph.
     * @interface
     */
    export interface INodeSequencer extends Object{
      /**
       * Returns a cursor that grants access to <b>all</b> nodes of the given graph in some order.
       * @param graph the input graph
       * @returns a {@link } that grants access to all nodes of a graph
       * @abstract
       */
      nodes(graph:yfiles.algorithms.Graph):yfiles.algorithms.INodeCursor;
    }
    var INodeSequencer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class provides methods for calculating independent sets.
     * <p>
     * An <b>independent set</b> is a set of nodes in a graph, in which no two nodes are adjacent.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0, "c":0},"styles":{"6":{"s":[35,35],"c":1, "f":"#FF0000", "b":0}},"n":[[425,305,6],[645,305],[425,525],[645,525,6],[485,365],[585,365,6],[485,465,6],[585,465]],"e":[[0,4],[0,1],[1,3],[3,2],[2,0],[4,6],[6,2],[6,7],[7,5],[5,4],[5,1],[7,3]],"vp":[425.0,305.0,250.0,250.0]}} Circular nodes represent one of the independent sets of the given graph
     * </p>
     * @class
     * @static
     */
    export interface IndependentSets extends Object{}
    export class IndependentSets {
      /**
       * Calculates an independent set for a given graph.
       * <p>
       * A greedy heuristic is applied which tries to find a large independent set.
       * </p>
       * @param conflictGraph the input graph
       * @returns a {@link } containing an independent set of nodes
       * @see yfiles.algorithms.IndependentSets#getIndependentSet
       * @static
       */
      static getIndependentSet(conflictGraph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList;
      /**
       * Partitions the set of nodes of the given graph into independent sets.
       * <p>
       * The method iteratively calls method {@link yfiles.algorithms.IndependentSets#getIndependentSet}.
       * </p>
       * @param conflictGraph the input graph
       * @returns an array of {@link }s each of which contains an independent set of nodes
       * @see yfiles.algorithms.IndependentSets#getIndependentSet
       * @static
       */
      static getIndependentSets(conflictGraph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList[];
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a so-called "cell" or "link" of the doubly linked list implementation {@link yfiles.algorithms.YList}.
     * <p>
     * It may be used to perform fast access and remove operations on that type of list.
     * </p>
     * @class
     * @final
     */
    export interface ListCell extends Object{}
    export class ListCell {
      /**
       * Returns the predecessor cell of this cell.
       * <p>
       * If there is no predecessor, then <code>null</code> is returned.
       * </p>
       * @returns 
       */
      pred():yfiles.algorithms.ListCell;
      /**
       * Returns the successor cell of this cell.
       * <p>
       * If there is no successor, then <code>null</code> is returned.
       * </p>
       * @returns 
       */
      succ():yfiles.algorithms.ListCell;
      /**
       * Gets or sets the element stored in this cell.
       * @type {Object}
       */
      info:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides sophisticated algorithms for solving classical network flow problems.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>
     * <b><b>Maximum flow problem:</b></b> – Given a directed graph in which each edge has a capacity and given a source node <code>s</code> and a sink node <code>t</code>, find a flow of
     * maximum value from <code>s</code> to <code>t</code>.
     * </li>
     * <li>
     * <b><b>Minimum cut problem:</b></b> – Given a directed graph in which each edge has a capacity and given a source node <code>s</code> and a sink node <code>t</code>, find an <code>s-t</code> cut of
     * minimum capacity (i.e., a set of edges of minimum capacity whose removal would disconnect <code>t</code> from <code>s</code>).
     * </li>
     * <li>
     * <b><b>Minimum cost flow problem:</b></b> – Given a directed graph in which each edge has a cost and a capacity and each node has a supply or demand, find a flow of
     * minimum total cost that satisfies the edge capacities and the node balances.
     * </li>
     * <li>
     * <b><b>Minimum cost maximum flow problem:</b></b> – Given a directed graph in which each edge has a cost and a capacity and given a source node <code>s</code> and a sink node <code>t</code>, find a
     * flow of maximum value from <code>s</code> to <code>t</code> that has the minimum total cost.
     * </li>
     * </ul>
     * @class
     * @static
     */
    export interface NetworkFlows extends Object{}
    export class NetworkFlows {
      /**
       * Solves a maximum flow problem using the preflow-push method.
       * <p>
       * The implementation is based on
       * </p>
       * <ul>
       * <li>Mehlhorn, Naeher: <i>LEDA: a platform for combinatorial and geometric computing</i>, Cambridge University Press, 2000, pp. 443-488.</li>
       * </ul>
       * <p>
       * The worst case running time is <code>O(mdeg * n^2 * m^(1/2))</code>, where <code>n</code> is the number of nodes in the network, <code>m</code> the number of
       * edges and <code>mdeg</code> the maximal degree of any node.
       * </p>
       * <p>
       * Edges may have infinite capacity, which is denoted by the value <code>0x7FFFFFFF</code>.
       * </p>
       * @param graph the given network
       * @param source the source node of the network
       * @param sink the sink node of the network
       * @param eCapDP the {@link } that returns the integer capacity of each edge or <code>null</code> if no bound is specified
       * @param flowEM the {@link } that will be filled during the execution with an integer flow for each edge
       * @returns the maximum flow value
       * @see yfiles.algorithms.NetworkFlows#calcMaxFlowMinCut
       * @static
       */
      static calcMaxFlow(graph:yfiles.algorithms.Graph,source:yfiles.algorithms.Node,sink:yfiles.algorithms.Node,eCapDP:yfiles.algorithms.IDataProvider,flowEM:yfiles.algorithms.IEdgeMap):number;
      /**
       * Solves a maximum flow problem using the preflow-push method but additionally marks all nodes that belong to the minimum
       * cut set that is associated with the source of the network.
       * @param graph the given network
       * @param source the source node of the network
       * @param sink the sink node of the network
       * @param eCapDP the {@link } that returns the integer capacity of each edge or <code>null</code> if no bound is specified
       * @param flowEM the {@link } that will be filled during the execution with an integer flow for each edge
       * @param sourceCutNM the {@link } that will be filled during the execution and returns a boolean value indicating whether
       * or not a node belongs to the cut set associated with the source of the network
       * @returns the maximum flow value
       * @see yfiles.algorithms.NetworkFlows#calcMaxFlow
       * @static
       */
      static calcMaxFlowMinCut(graph:yfiles.algorithms.Graph,source:yfiles.algorithms.Node,sink:yfiles.algorithms.Node,eCapDP:yfiles.algorithms.IDataProvider,flowEM:yfiles.algorithms.IEdgeMap,sourceCutNM:yfiles.algorithms.INodeMap):number;
      /**
       * Solves a minimum cost maximum flow problem.
       * @param graph the given network
       * @param s the source node of the network
       * @param t the sink node of the network
       * @param uCapDP the {@link } that returns the integer capacity of each edge or <code>null</code> if no bound is specified
       * @param cost0DP the {@link } that returns a double value (cost) for each edge
       * @param flowEM the {@link } that will be filled during the execution with an integer flow for each edge
       * @param dualsNM the {@link } that will be filled during the execution with an integer value (dual value) for each node
       * or <code>null</code> if no such values occur; dual values are also referred as potentials
       * @returns the total cost of the flow
       * @see yfiles.algorithms.NetworkFlows#minCostFlow
       * @see yfiles.algorithms.NetworkFlows#minCostFlow
       * @static
       */
      static minCostFlow(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,uCapDP:yfiles.algorithms.IDataProvider,cost0DP:yfiles.algorithms.IDataProvider,flowEM:yfiles.algorithms.IEdgeMap,dualsNM:yfiles.algorithms.INodeMap):number;
      /**
       * Solves a minimum cost flow problem with a capacity scaling algorithm.
       * <p>
       * This algorithm is a variant of the successive shortest path algorithm (see Ahuja,Magnanti,Orlin: <i>Network flows</i>, Prentice Hall, 1993,
       * pp.320-324). It has the pseudo-polynomial running time <code>O(m*log U*(m+n log n))</code> where <code>n</code> is the number of nodes in the
       * network, <code>m</code> the number of edges and <code>U</code> the maximal edge capacity.
       * </p>
       * <p>
       * Edges may have infinite capacity, which is denoted by the value <code>0x7FFFFFFF</code>.
       * </p>
       * <p>
       * There are no restriction for the costs. In particular, they can also be negative.
       * </p>
       * @param graph the given network
       * @param lCapDP the {@link } that returns the integer lower bound for the capacity of each edge or <code>null</code> if no
       * bound is specified
       * @param uCapDP the {@link } that returns the integer upper bound for the capacity of each edge or <code>null</code> if no
       * bound is specified
       * @param cost0DP the {@link } that returns a double value (cost) of each edge
       * @param supplyDP the {@link } that returns the supply/demand of each node; supply is denoted by a positive value,
       * demand by a negative value
       * @param flowEM the {@link } that will be filled during the execution with an integer flow for each edge
       * @param dualsNM the {@link } that will be filled during the execution with an integer value (dual value) for each node
       * or <code>null</code> if no such values occur; dual values are also referred as potentials
       * @returns the total cost of the flow
       * @see yfiles.algorithms.NetworkFlows#minCostFlow
       * @see yfiles.algorithms.NetworkFlows#minCostFlow
       * @static
       */
      static minCostFlow(graph:yfiles.algorithms.Graph,lCapDP:yfiles.algorithms.IDataProvider,uCapDP:yfiles.algorithms.IDataProvider,cost0DP:yfiles.algorithms.IDataProvider,supplyDP:yfiles.algorithms.IDataProvider,flowEM:yfiles.algorithms.IEdgeMap,dualsNM:yfiles.algorithms.INodeMap):number;
      /**
       * Uses method {@link yfiles.algorithms.NetworkFlows#minCostFlow} to solve a minimum cost flow problem.
       * @param graph the given network
       * @param uCapDP the {@link } that returns the integer capacity of each edge or <code>null</code> if no bound is specified
       * @param cost0DP the {@link } that returns a double value (cost) for each edge
       * @param supplyDP the {@link } that returns the supply/demand of each node; supply is denoted by a positive value,
       * demand by a negative value
       * @param flowEM the {@link } that will be filled during the execution with an integer flow for each edge
       * @param dualsNM the {@link } that will be filled during the execution with an integer value (dual value) for each node
       * or <code>null</code> if no such values occur; dual values are also referred as potentials
       * @returns the total cost of the flow
       * @see yfiles.algorithms.NetworkFlows#minCostFlow
       * @see yfiles.algorithms.NetworkFlows#minCostFlow
       * @static
       */
      static minCostFlow(graph:yfiles.algorithms.Graph,uCapDP:yfiles.algorithms.IDataProvider,cost0DP:yfiles.algorithms.IDataProvider,supplyDP:yfiles.algorithms.IDataProvider,flowEM:yfiles.algorithms.IEdgeMap,dualsNM:yfiles.algorithms.INodeMap):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a so-called node in the directed graph data type {@link yfiles.algorithms.Graph}.
     * <p>
     * Most notably, a node provides access to its adjacent edges (represented by instances of class
     * {@link yfiles.algorithms.Edge}). These can be distinguished into the sets of incoming and outgoing edges.
     * </p>
     * <p>
     * Iteration over all three sets of edges is provided by means of bidirectional cursors that present a read-only view of
     * the respective set ({@link yfiles.algorithms.Node#getEdgeCursor}, {@link yfiles.algorithms.Node#getInEdgeCursor},
     * {@link yfiles.algorithms.Node#getOutEdgeCursor}). Also supported is iteration over all nodes at opposite ends of either incoming
     * edges or outgoing edges ({@link yfiles.algorithms.Node#getPredecessorCursor}, {@link yfiles.algorithms.Node#getSuccessorCursor}).
     * </p>
     * <p>
     * The number of overall edges at a node is called its degree ({@link yfiles.algorithms.Node#degree}), which is the sum of incoming and outgoing edges ({@link yfiles.algorithms.Node#inDegree}, {@link yfiles.algorithms.Node#outDegree}).
     * </p>
     * <p>
     * <b>Important:</b> Class Graph is the single authority for any structural changes to the graph data type. Specifically, this means that
     * there is no way to create or delete a node or an edge without using an actual Graph instance.
     * </p>
     * @class
     * @extends {yfiles.algorithms.GraphObject}
     */
    export interface Node extends yfiles.algorithms.GraphObject{}
    export class Node {
      /**
       * Instantiates a new Node object that will be part of the given graph.
       * @param g The graph that the created node will belong to.
       */
      constructor(g:yfiles.algorithms.Graph);
      /**
       * Creates a copy of this node that will be inserted into the given graph.
       * @param g The graph that the created node will belong to.
       * @returns The newly created Node object.
       */
      createCopy(g:yfiles.algorithms.Graph):yfiles.algorithms.Node;
      /**
       * Returns an edge that connects this node with the given node, if such an edge exists.
       * <p>
       * Otherwise <code>null</code> is returned.
       * </p>
       * <p>
       * Note that the first matching edge is returned, and that outgoing edges are tested prior to incoming edges.
       * </p>
       * @param opposite
       * @returns 
       * @see yfiles.algorithms.Node#getEdgeFrom
       * @see yfiles.algorithms.Node#getEdgeTo
       */
      getEdge(opposite:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Returns an edge cursor for all incoming and outgoing edges at this node.
       * @returns 
       * @see yfiles.algorithms.Node#getInEdgeCursor
       * @see yfiles.algorithms.Node#getOutEdgeCursor
       */
      getEdgeCursor():yfiles.algorithms.IEdgeCursor;
      /**
       * Returns an incoming edge that connects the given node with this node, if such an edge exists.
       * <p>
       * Otherwise <code>null</code> is returned.
       * </p>
       * @param source
       * @returns 
       * @see yfiles.algorithms.Node#getEdge
       * @see yfiles.algorithms.Node#getEdgeTo
       */
      getEdgeFrom(source:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Returns an outgoing edge that connects this node with the given node, if such an edge exists.
       * <p>
       * Otherwise <code>null</code> is returned.
       * </p>
       * @param target
       * @returns 
       * @see yfiles.algorithms.Node#getEdge
       * @see yfiles.algorithms.Node#getEdgeFrom
       */
      getEdgeTo(target:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Returns an edge cursor for incoming edges at this node.
       * <p>
       * If an edge is specified, the cursor starts at the given edge, and the cyclic sequence order is the same as returned by
       * {@link yfiles.algorithms.Node#getInEdgeCursor}.
       * </p>
       * @param [startEdge=null] The first edge being accessed by the returned cursor.
       * @returns 
       * @see yfiles.algorithms.Node#getOutEdgeCursor
       */
      getInEdgeCursor(startEdge?:yfiles.algorithms.Edge):yfiles.algorithms.IEdgeCursor;
      /**
       * Returns a node cursor for all neighbor nodes of this node.
       * <p>
       * Neighbor nodes are those at the opposite ends of both incoming and outgoing edges.
       * </p>
       * @returns 
       * @see yfiles.algorithms.Node#getPredecessorCursor
       * @see yfiles.algorithms.Node#getSuccessorCursor
       */
      getNeighborCursor():yfiles.algorithms.INodeCursor;
      /**
       * Returns an edge cursor for outgoing edges at this node.
       * <p>
       * If an edge is specified, the cursor starts at the given edge, and the cyclic sequence order is the same as returned by
       * {@link yfiles.algorithms.Node#getOutEdgeCursor}.
       * </p>
       * @param [startEdge=null] The first edge being accessed by the returned cursor.
       * @returns 
       * @see yfiles.algorithms.Node#getInEdgeCursor
       */
      getOutEdgeCursor(startEdge?:yfiles.algorithms.Edge):yfiles.algorithms.IEdgeCursor;
      /**
       * Returns a node cursor for all predecessor nodes of this node.
       * <p>
       * Predecessor nodes are those at the opposite ends of incoming edges.
       * </p>
       * @returns 
       * @see yfiles.algorithms.Node#getSuccessorCursor
       */
      getPredecessorCursor():yfiles.algorithms.INodeCursor;
      /**
       * Returns a node cursor for all successor nodes of this node.
       * <p>
       * Successor nodes are those at the opposite ends of outgoing edges.
       * </p>
       * @returns 
       * @see yfiles.algorithms.Node#getPredecessorCursor
       */
      getSuccessorCursor():yfiles.algorithms.INodeCursor;
      /**
       * Sorts incoming edges at this node according to the given comparator.
       * @param c
       * @see yfiles.algorithms.Node#sortOutEdges
       */
      sortInEdges(c:yfiles.collections.IComparer<Object>):void;
      /**
       * Sorts outgoing edges at this node according to the given comparator.
       * @param c
       * @see yfiles.algorithms.Node#sortInEdges
       */
      sortOutEdges(c:yfiles.collections.IComparer<Object>):void;
      /**
       * Gets the overall number of incoming and outgoing edges at this node.
       * <p>
       * Note that self-loops are counted twice.
       * </p>
       * @see yfiles.algorithms.Edge
       * @see yfiles.algorithms.Node#inDegree
       * @see yfiles.algorithms.Node#outDegree
       * @type {number}
       */
      degree:number;
      /**
       * Gets the number of incoming edges at this node.
       * @see yfiles.algorithms.Node#degree
       * @see yfiles.algorithms.Node#outDegree
       * @type {number}
       */
      inDegree:number;
      /**
       * Gets the number of outgoing edges at this node.
       * @see yfiles.algorithms.Node#degree
       * @see yfiles.algorithms.Node#inDegree
       * @type {number}
       */
      outDegree:number;
      /**
       * Gets the index of this node within its graph G.
       * <p>
       * Node indices represent the ordering of standard node iteration on G. The value of an index is <code> >= 0</code> and
       * <code> &lt; G.nodeCount()</code>.
       * </p>
       * <p>
       * Note that indices are subject to change whenever the sequence of nodes in a graph is modified by either removing,
       * hiding, reinserting, or unhiding a node, or by explicitly changing its position in the sequence.
       * </p>
       * @see yfiles.algorithms.Graph#removeNode
       * @see yfiles.algorithms.Graph#hide
       * @see yfiles.algorithms.Graph#reInsertNode
       * @see yfiles.algorithms.Graph#unhide
       * @see yfiles.algorithms.Graph#moveToFirst
       * @see yfiles.algorithms.Graph#moveToLast
       * @type {number}
       */
      index:number;
      /**
       * Gets the graph this node belongs to.
       * <p>
       * If the node does not belong to a graph, because it was removed or hidden from it, this method returns <code>null</code>.
       * </p>
       * @type {yfiles.algorithms.Graph}
       */
      graph:yfiles.algorithms.Graph;
      /**
       * Gets the first outgoing edge at this node, or <code>null</code> if it does not exist.
       * @see yfiles.algorithms.Node#firstInEdge
       * @see yfiles.algorithms.Node#lastOutEdge
       * @type {yfiles.algorithms.Edge}
       */
      firstOutEdge:yfiles.algorithms.Edge;
      /**
       * Gets the first incoming edge at this node, or <code>null</code> if it does not exist.
       * @see yfiles.algorithms.Node#firstOutEdge
       * @see yfiles.algorithms.Node#lastInEdge
       * @type {yfiles.algorithms.Edge}
       */
      firstInEdge:yfiles.algorithms.Edge;
      /**
       * Gets the last outgoing edge at this node, or <code>null</code> if it does not exist.
       * @see yfiles.algorithms.Node#firstOutEdge
       * @see yfiles.algorithms.Node#lastInEdge
       * @type {yfiles.algorithms.Edge}
       */
      lastOutEdge:yfiles.algorithms.Edge;
      /**
       * Gets the last incoming edge at this node, or <code>null</code> if it does not exist.
       * @see yfiles.algorithms.Node#firstInEdge
       * @see yfiles.algorithms.Node#lastOutEdge
       * @type {yfiles.algorithms.Edge}
       */
      lastInEdge:yfiles.algorithms.Edge;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for {@link yfiles.algorithms.Edge}s that can be used to iterate over outgoing edges at this instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
       * graph structure during the traversal should be carried out with great care. Note that self-loop edges are reported,
       * too.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Edge>}
       */
      outEdges:yfiles.collections.IEnumerable<yfiles.algorithms.Edge>;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for {@link yfiles.algorithms.Edge}s that can be used to iterate over ingoing edges at this instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
       * graph structure during the traversal should be carried out with great care. Note that self-loop edges are reported,
       * too.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Edge>}
       */
      inEdges:yfiles.collections.IEnumerable<yfiles.algorithms.Edge>;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for {@link yfiles.algorithms.Edge}s that can be used to iterate over the adjacent edges at this instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
       * graph structure during the traversal should be carried out with great care. Note that self-loop edges are reported twice
       * (as in edge and as out edge).
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Edge>}
       */
      edges:yfiles.collections.IEnumerable<yfiles.algorithms.Edge>;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for
       * {@link yfiles.algorithms.Node}s that can be used to iterate over the opposite sides of adjacent outgoing edges at this
       * instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
       * graph structure during the traversal should be carried out with great care. Note that for self-loop edges this node
       * itself will be reported as a successor.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Node>}
       */
      successors:yfiles.collections.IEnumerable<yfiles.algorithms.Node>;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for
       * {@link yfiles.algorithms.Node}s that can be used to iterate over the opposite sides of adjacent incoming edges at this
       * instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
       * graph structure during the traversal should be carried out with great care. Note that for self-loop edges this node
       * itself will be reported as a predecessor.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Node>}
       */
      predecessors:yfiles.collections.IEnumerable<yfiles.algorithms.Node>;
      /**
       * Yields a dynamic {@link yfiles.collections.IEnumerable.<T>} for
       * {@link yfiles.algorithms.Node}s that can be used to iterate over the opposite sides of adjacent adjacent edges at this
       * instance.
       * <p>
       * This is a live enumerable and will thus reflect the current state of the node's adjacency. Note that changes to the
       * graph structure during the traversal should be carried out with great care. Note that for self-loop edges this node
       * itself will be reported as a neighbor, twice.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.algorithms.Node>}
       */
      neighbors:yfiles.collections.IEnumerable<yfiles.algorithms.Node>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specialized list implementation for instances of type {@link yfiles.algorithms.Node}.
     * @class
     * @extends {yfiles.algorithms.YList}
     * @implements {yfiles.collections.IEnumerable.<yfiles.algorithms.Node>}
     */
    export interface NodeList extends yfiles.algorithms.YList{}
    export class NodeList {
      /**
       * Creates a list that is initialized with the entries of the given list.
       * @param list the values are added to the new list
       */
      constructor(list:yfiles.algorithms.NodeList);
      /**
       * Creates a list that is initialized with a single node provided.
       * @param v
       */
      constructor(v:yfiles.algorithms.Node);
      /**
       * Creates a list that is initialized with the nodes provided by the given array of nodes.
       * @param a
       */
      constructor(a:yfiles.algorithms.Node[]);
      /**
       * Creates a list that is initialized with those nodes from the given NodeCursor object for which the given data provider
       * returns <code>true</code> upon calling its {@link yfiles.algorithms.IDataProvider#getBoolean getBool} method.
       * @param nc A node cursor providing nodes that should be added to this list.
       * @param predicate A data provider that acts as a inclusion predicate for each node accessible by the given node cursor.
       */
      constructor(nc:yfiles.algorithms.INodeCursor,predicate:yfiles.algorithms.IDataProvider);
      /**
       * Creates a list that is initialized with the nodes provided by the given NodeCursor object.
       * @param c
       */
      constructor(c:yfiles.algorithms.INodeCursor);
      /**
       * Creates an empty node list.
       */
      constructor();
      /**
       * Returns the first node in this list, or <code>null</code> when the list is empty.
       * @returns The first node in the list.
       */
      firstNode():yfiles.algorithms.Node;
      /**
       * Returns an enumerator for this collection.
       * @returns 
       */
      getEnumerator():yfiles.collections.IEnumerator<yfiles.algorithms.Node>;
      /**
       * Returns the last node in this list, or <code>null</code> when the list is empty.
       * @returns The last node in the list.
       */
      lastNode():yfiles.algorithms.Node;
      /**
       * Returns a node cursor for this node list.
       * @returns A node cursor granting access to the nodes within this list.
       */
      nodes():yfiles.algorithms.INodeCursor;
      /**
       * Removes the first node from this list and returns it.
       * @returns The first node from the list.
       */
      popNode():yfiles.algorithms.Node;
      /**
       * Returns a node array containing all elements of this list in the canonical order.
       * @returns 
       */
      toNodeArray():yfiles.algorithms.Node[];
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides algorithms that order the nodes of a graph using specific criteria.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>
     * <b>A topological ordering</b> of the nodes of a directed graph is a linear ordering of the nodes such that for each directed edge <code>(u,v)</code>, node <code>u</code> lies
     * before <code>v</code> in the ordering.
     * </li>
     * <li>A <b>DFS completion ordering</b> of the nodes of a graph is a node ordering identical to the order of node completion events in a {@link yfiles.algorithms.Dfs depth first search}.</li>
     * <li>An <b>st-ordering</b> <code>(v_1,v_2,....,v_n)</code> of a biconnected graph is a node ordering which guarantees that:</li>
     * </ul>
     * @class
     * @static
     */
    export interface NodeOrders extends Object{}
    export class NodeOrders {
      /**
       * Calculates an ordering of the nodes identical to the order of node completion events in a depth first search.
       * <p>
       * This ordering is a reversed topological ordering in case the input graph is acyclic.
       * </p>
       * @param graph the input graph
       * @param order an array of {@link }s that returns for each {@link } <code>v</code>, its zero-based index within the calculated ordering, i.e.,
       * <code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
       * @see yfiles.algorithms.NodeOrders#topological
       * @see yfiles.algorithms.NodeOrders#dfsCompletion
       * @static
       */
      static dfsCompletion(graph:yfiles.algorithms.Graph,order:number[]):void;
      /**
       * Calculates an ordering of the nodes identical to the order of node completion events in a depth first search.
       * <p>
       * Like {@link yfiles.algorithms.NodeOrders#dfsCompletion} but the result is returned as a {@link yfiles.algorithms.NodeList}.
       * </p>
       * <p>
       * This ordering is a reversed topological ordering in case the input graph is acyclic.
       * </p>
       * @param graph the input graph
       * @returns a {@link } containing the nodes of the graph in the order identical to the order of node completion
       * events in a depth first search
       * @see yfiles.algorithms.NodeOrders#topological
       * @see yfiles.algorithms.NodeOrders#dfsCompletion
       * @static
       */
      static dfsCompletion(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList;
      /**
       * Assigns an <code>st</code>-ordering to the nodes of a biconnected graph given the edge between source node <code>s</code> and sink node <code>t</code>.
       * <p>
       * An <b><code>st</code>-ordering</b> <code>(v_1,v_2,....,v_n)</code> of a biconnected graph is a node ordering which guarantees that:
       * </p>
       * <ul>
       * <li>Source node <code>s</code> and sink node <code>t</code> are connected by an edge.</li>
       * <li>For each node <code>v_i</code> in the ordering other than <code>s</code> or <code>t</code>, there are neighbors <code>v_j</code> and <code>v_k</code> with <code>j &lt; i</code> and <code>k > i</code>.</li>
       * </ul>
       * @param graph the input graph
       * @param stOrder an array of {@link }s that will be filled during the execution and returns for each {@link } <code>v</code>, its zero-based
       * index within the calculated ordering, i.e., <code>stOrder[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
       * @param [stEdge=null] an {@link } that connects source node <code>s</code> and sink node <code>t</code>
       * @see yfiles.algorithms.NodeOrders#st
       * @see yfiles.algorithms.NodeOrders#st
       * @static
       */
      static st(graph:yfiles.algorithms.Graph,stOrder:number[],stEdge?:yfiles.algorithms.Edge):void;
      /**
       * Assigns an <code>st</code>-ordering to the nodes of a biconnected graph.
       * <p>
       * Like {@link yfiles.algorithms.NodeOrders#st} but the result is returned as a {@link yfiles.algorithms.NodeList}.
       * </p>
       * <p>
       * An <b><code>st</code>-ordering</b> <code>(v_1,v_2,....,v_n)</code> of a biconnected graph is a node ordering which guarantees that:
       * </p>
       * <ul>
       * <li>Source node <code>s</code> and sink node <code>t</code> are connected by an edge.</li>
       * <li>For each node <code>v_i</code> in the ordering other than <code>s</code> or <code>t</code>, there are neighbors <code>v_j</code> and <code>v_k</code> with <code>j &lt; i</code> and <code>k > i</code>.</li>
       * </ul>
       * @param graph the input graph
       * @returns a {@link } containing the nodes of the graph in the order defined by the <code>st</code>-ordering
       * @see yfiles.algorithms.NodeOrders#st
       * @static
       */
      static st(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList;
      /**
       * Converts an array-based result returned by a method of this class to a {@link yfiles.algorithms.NodeList} that contains all
       * nodes in the order provided by the given array.
       * @param graph the input graph
       * @param order an array of {@link }s that will be filled during the execution and returns for each {@link } <code>v</code>, its zero-based
       * index within the calculated ordering, i.e., <code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
       * @returns a {@link } containing the nodes of the graph in the order provided by the given array
       * @static
       */
      static toNodeList(graph:yfiles.algorithms.Graph,order:number[]):yfiles.algorithms.NodeList;
      /**
       * Copies an array-based result returned by a method of this class to a {@link yfiles.algorithms.INodeMap} that will provide values
       * of basic type <code>int</code>.
       * @param graph the input graph
       * @param order an array of {@link }s that returns for each {@link } <code>v</code>, its zero-based index within the calculated ordering, i.e.,
       * <code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
       * @param result the {@link } that will be filled during the execution with the zero-based index of each node within
       * the calculated ordering
       * @static
       */
      static toNodeMap(graph:yfiles.algorithms.Graph,order:number[],result:yfiles.algorithms.INodeMap):void;
      /**
       * Copies a {@link yfiles.algorithms.NodeList}-based result returned by a method of this class to a {@link yfiles.algorithms.INodeMap} that
       * will provide values of basic type <code>int</code>.
       * @param order a {@link } containing the nodes of the graph in the appropriate order
       * @param result the {@link } that will be filled during the execution with the zero-based index of each node within
       * the calculated ordering
       * @static
       */
      static toNodeMap(order:yfiles.algorithms.NodeList,result:yfiles.algorithms.INodeMap):void;
      /**
       * Assigns a topological ordering to the nodes of a directed acyclic graph.
       * <p>
       * A topological ordering of the nodes of a directed graph is a linear ordering of the nodes such that for each directed
       * edge <code>(u,v)</code>, node <code>u</code> lies before <code>v</code> in the ordering.
       * </p>
       * @param graph the input graph
       * @param order an array of {@link }s that will be filled during the execution and returns for each {@link } <code>v</code>, its zero-based
       * index within the calculated ordering, i.e., <code>order[v.index()] == 5</code> means that <code>v</code> is the <code>6</code>-th node within the ordering
       * @returns <code>true</code> if the graph is acyclic, <code>false</code> otherwise
       * @see yfiles.algorithms.NodeOrders#topological
       * @static
       */
      static topological(graph:yfiles.algorithms.Graph,order:number[]):boolean;
      /**
       * Returns a topological ordering of the nodes of a directed acyclic graph.
       * @param graph the input graph
       * @returns a {@link } containing the nodes of the graph in the order they appear in the topological ordering
       * @throws {Stubs.Exceptions.ArgumentError} if the graph is cyclic
       * @see yfiles.algorithms.NodeOrders#topological
       * @static
       */
      static topological(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides methods for finding paths within a graph that have certain properties.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>
     * A <b>path</b> of length <code>k</code> from node <code>u</code> to node <code>v</code> in a graph is a sequence <code>v0, v1, v2, ... , vk</code> of nodes such that <code>u = v0, v = vk</code> and
     * <code>(vi-1, vi)</code> in <code>E</code>, for each <code>i = 1, 2, .. , k</code>.
     * </li>
     * <li>A path is called <b>simple</b> if no node appears twice.</li>
     * <li>A <b>chain</b> is a path of maximum length in which each internal node has degree <code>2</code>.</li>
     * <li>
     * The <b>longest path problem</b> is the problem of finding a simple path of maximum length in a given graph. The longest path problem is
     * <code>NP</code>-hard for undirected graphs, but can be solved in linear time for directed acyclic graphs.
     * </li>
     * </ul>
     * @class
     * @static
     */
    export interface Paths extends Object{}
    export class Paths {
      /**
       * Constructs a {@link yfiles.algorithms.NodeList path of nodes} from a given {@link yfiles.algorithms.EdgeList path of edges}.
       * <p>
       * The returned node path has length <code>path.size()+1</code>, if the given path is not empty. Otherwise, the returned path will be
       * empty. The <code>i</code>-th node in the returned path will be either source or target node of the <code>i</code>-th edge in the given path.
       * </p>
       * @param path the given {@link  path of edges}
       * @returns a {@link  path of nodes} from the given {@link  path of edges}
       * @static
       */
      static constructNodePath(path:yfiles.algorithms.EdgeList):yfiles.algorithms.NodeList;
      /**
       * Returns all chains present in the given graph.
       * <p>
       * A chain is a path of maximum length in which each internal node has degree <code>2</code>.
       * </p>
       * <p>
       * The internal nodes on directed chains all have in-degree <code>1</code> and out-degree <code>1</code>.
       * </p>
       * @param graph the input graph
       * @param directed <code>true</code> if the chain should be considered as directed, <code>false</code> otherwise
       * @returns an array of {@link }s each of which contains the edges (at least two) that make up a chain
       * @see yfiles.algorithms.Paths#constructNodePath
       * @static
       */
      static findAllChains(graph:yfiles.algorithms.Graph,directed:boolean):yfiles.algorithms.EdgeList[];
      /**
       * A variant of {@link yfiles.algorithms.Paths#findAllPaths} which returns all simple directed or undirected paths between two
       * given nodes and, additionally, allows to specify a filter for the paths to be returned.
       * @param graph the input graph
       * @param startNode the given start node
       * @param endNode the given end node
       * @param directed <code>true</code> if the path should be considered as directed, <code>false</code> otherwise
       * @param filter a predicate that accepts or rejects a found {@link } and adds it to the result
       * @returns an array of {@link }s each of which represents a path between the start and end node.
       * @static
       */
      static findAllPaths(graph:yfiles.algorithms.Graph,startNode:yfiles.algorithms.Node,endNode:yfiles.algorithms.Node,directed:boolean,filter:(obj:yfiles.algorithms.EdgeList)=>boolean):yfiles.algorithms.EdgeList[];
      /**
       * Returns all simple directed or undirected paths that connect a <code>start</code> node with an <code>end</code> node.
       * @param graph the input graph
       * @param startNode the given start node
       * @param endNode the given end node
       * @param directed <code>true</code> if the path should be considered as directed, <code>false</code> otherwise
       * @returns an array of {@link }s each of which represents a path between the start and end node
       * @static
       */
      static findAllPaths(graph:yfiles.algorithms.Graph,startNode:yfiles.algorithms.Node,endNode:yfiles.algorithms.Node,directed:boolean):yfiles.algorithms.EdgeList[];
      /**
       * Finds all edges that belong to a directed path from a <code>start</code> node to an <code>end</code> node.
       * @param graph the input graph
       * @param startNode the given start node
       * @param endNode the given end node
       * @param pathEdges the {@link } that will be filled during the execution with a boolean value indicating whether or not
       * an edge belongs to a path connecting the two given nodes
       * @static
       */
      static findAllPaths(graph:yfiles.algorithms.Graph,startNode:yfiles.algorithms.Node,endNode:yfiles.algorithms.Node,pathEdges:yfiles.algorithms.IEdgeMap):void;
      /**
       * A variant of
       * {@link yfiles.algorithms.Paths#findAllPaths}, which returns all simple directed or undirected paths between two given nodes as a
       * special cursor that calculates the next path in the sequence, only when needed.
       * <p>
       * The returned cursor only supports the operation {@link yfiles.algorithms.ICursor#ok}, {@link yfiles.algorithms.ICursor#current}, {@link yfiles.algorithms.ICursor#size} and {@link yfiles.algorithms.ICursor#next}.
       * </p>
       * @param graph the input graph
       * @param startNode the given start node
       * @param endNode the given end node
       * @param directed <code>true</code> if the path should be considered as directed, <code>false</code> otherwise
       * @returns a {@link } that calculates the next path in the sequence
       * @static
       */
      static findAllPathsCursor(graph:yfiles.algorithms.Graph,startNode:yfiles.algorithms.Node,endNode:yfiles.algorithms.Node,directed:boolean):yfiles.algorithms.ICursor;
      /**
       * Returns the longest directed path in a given acyclic weighted graph.
       * <p>
       * All edges of the graph have an integral length associated with them. The longest path is defined as one of all directed
       * paths within the graph for which the edge lengths of all contained edges sum up to a maximum.
       * </p>
       * @param graph a directed acyclic graph
       * @param [edgeLength=null] the {@link } that returns the non-negative integer length of each edge
       * @returns an {@link } containing the edges of the longest directed path
       * @static
       */
      static findLongestPath(graph:yfiles.algorithms.Graph,edgeLength?:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList;
      /**
       * Calculates the longest path from a given node to all other node in a given directed acyclic graph.
       * @param graph a directed acyclic graph
       * @param startNode the node for which the distances are calculated
       * @param dist the {@link } that returns the distance (i.e. weight) of type double for each edge
       * @param maxDist the {@link } that will be filled during the execution and holds the maximum distance between the given
       * node and all other nodes
       * @param predicate the {@link } that returns a boolean value indicating whether or not an edge should be considered
       * during the path search
       * @static
       */
      static findLongestPaths(graph:yfiles.algorithms.Graph,startNode:yfiles.algorithms.Node,dist:yfiles.algorithms.IEdgeMap,maxDist:yfiles.algorithms.INodeMap,predicate:yfiles.algorithms.IEdgeMap):void;
      /**
       * Returns an {@link yfiles.algorithms.EdgeList} containing the edges of an undirected simple path within the given graph.
       * <p>
       * The edges are returned in the order that they appear in the found path.
       * </p>
       * <p>
       * A heuristic is used for finding a path that is long. It is not guaranteed, though, that the returned path is actually
       * the longest path within the given graph, since that is a well known hard problem.
       * </p>
       * @param graph the given graph
       * @returns an {@link } containing the edges of an undirected simple path
       * @static
       */
      static findLongPath(graph:yfiles.algorithms.Graph):yfiles.algorithms.EdgeList;
      /**
       * Returns whether or not a directed path from a start node to another node in an acyclic graph exists.
       * @param graph an acyclic graph which contains the two nodes
       * @param topSort a {@link  list} of nodes sorted in topological order
       * @param predicate the {@link } that returns a boolean value indicating whether or not an edge should be considered
       * during the path search
       * @param startNode
       * @param endNode
       * @returns <code>true</code> if a directed path from a start node to another node exists, <code>false</code> otherwise
       * @static
       */
      static findPath(graph:yfiles.algorithms.Graph,topSort:yfiles.algorithms.NodeList,startNode:yfiles.algorithms.Node,endNode:yfiles.algorithms.Node,predicate:yfiles.algorithms.IEdgeMap):boolean;
      /**
       * Returns an {@link yfiles.algorithms.EdgeList} containing the edges of a path from the given start node to the given end node, if
       * such a path exists.
       * <p>
       * The edges are returned in the order that they appear in the found path. If the returned path is empty, no path between
       * the given nodes was found.
       * </p>
       * @param graph the input graph
       * @param startNode the first node of the path
       * @param endNode the last node of the path
       * @param directed <code>true</code> if the path should be directed, <code>false</code> otherwise
       * @returns an {@link } containing the path edges between the start node and the end node
       * @static
       */
      static findPath(graph:yfiles.algorithms.Graph,startNode:yfiles.algorithms.Node,endNode:yfiles.algorithms.Node,directed:boolean):yfiles.algorithms.EdgeList;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents an embedding of a planar graph.
     * <p>
     * A planar embedding is represented by a {@link yfiles.algorithms.PlanarEmbedding#faces list of faces}. Each face is represented by a list of {@link yfiles.algorithms.Dart darts} which describe the
     * border of the face.
     * </p>
     * @class
     * @final
     */
    export interface PlanarEmbedding extends Object{}
    export class PlanarEmbedding {
      /**
       * Creates a new embedding for the specified planar graph.
       * @param graph a planar graph
       * @throws {Stubs.Exceptions.ArgumentError} if the specified graph is not planar
       */
      constructor(graph:yfiles.algorithms.Graph);
      /**
       * Returns the cyclic next dart of the given dart.
       * @param dart a dart
       * @returns the cyclic next dart of the given dart
       */
      getCyclicNext(dart:yfiles.algorithms.Dart):yfiles.algorithms.Dart;
      /**
       * Returns the cyclic previous dart of the given dart.
       * @param dart a dart
       * @returns the cyclic previous dart of the given dart
       */
      getCyclicPrevious(dart:yfiles.algorithms.Dart):yfiles.algorithms.Dart;
      /**
       * Returns the two darts associated with the given edge.
       * @param edge an edge
       * @returns the two darts associated with the given edge
       */
      getDarts(edge:yfiles.algorithms.Edge):yfiles.algorithms.Dart[];
      /**
       * Returns a {@link yfiles.collections.IList.<T>} containing the (cyclic ordered) outgoing darts of the specified node.
       * @param node a node
       * @returns a {@link } containing the (cyclic ordered) outgoing darts
       */
      getOutgoingDarts(node:yfiles.algorithms.Node):yfiles.collections.IList<yfiles.algorithms.Dart>;
      /**
       * Return whether or not the given graph is planar.
       * @param graph the input graph
       * @returns <code>true</code> if the given graph is planar, <code>false</code> otherwise
       * @static
       */
      static isPlanar(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Gets a {@link yfiles.collections.IList.<T>} of face lists.
       * <p>
       * More precisely, each entry of this list is another list that contains all {@link yfiles.algorithms.Dart darts} of a face.
       * </p>
       * @type {yfiles.collections.IList.<yfiles.collections.IList.<yfiles.algorithms.Dart>>}
       */
      faces:yfiles.collections.IList<yfiles.collections.IList<yfiles.algorithms.Dart>>;
      /**
       * Gets the {@link yfiles.collections.IList.<T>} that represents the outer face.
       * @see yfiles.algorithms.PlanarEmbedding#faces
       * @type {yfiles.collections.IList.<yfiles.algorithms.Dart>}
       */
      outerFace:yfiles.collections.IList<yfiles.algorithms.Dart>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides algorithms for solving the rank assignment problem.
     * <p>
     * <b>Definitions</b> Let <code>G=(V,E)</code> be a directed acyclic graph. Let <code>length(e)</code> denote the minimum length and <code>weight(e)</code> the weight of an edge
     * <code>e</code>.
     * </p>
     * <p>
     * The <b>rank assignment problem</b> is the problem of finding integer values <code>rank(v)</code> for all <code>v</code> in <code>V</code>, such that:
     * </p>
     * <ul>
     * <li><code>rank(v) - rank(w) >= length(v,w)</code>, for all <code>(v,w)</code> in <code>E</code> and,</li>
     * <li>the sum <code>∑(weight(v,w) * (rank(v) - rank(w)))</code> over all <code>(v,w)</code> in <code>E</code> is minimized.</li>
     * </ul>
     * @class
     * @static
     */
    export interface RankAssignments extends Object{}
    export class RankAssignments {
      /**
       * Like {@link yfiles.algorithms.RankAssignments#simple}, but arrays are used instead of {@link yfiles.algorithms.INodeMap}s and
       * {@link yfiles.algorithms.IEdgeMap}s.
       * <p>
       * Minimum edge length and weights should be non-negative.
       * </p>
       * @param graph the input graph in which all the edges have directions, such that <code>rank[source] &lt; rank[target]</code> and
       * <code>rank[target] - rank[source] >= minlength[edge]</code>
       * @param rank an array that will be filled with the ranking <code>r</code> of each node <code>v</code> such that <code>rank[v.index] == r</code>
       * @param minLength an array holding a non-negative value <code>len</code> of each edge <code>e</code> such that <code>minLength[e.index] == len</code>
       * @param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
       * @returns the number of layers
       * @see yfiles.algorithms.RankAssignments#simple
       * @static
       */
      static simple(graph:yfiles.algorithms.Graph,rank:number[],minLength:number[],maximalDuration?:number):number;
      /**
       * This method quickly calculates a tight tree given a maximum time duration for the algorithm.
       * <p>
       * The algorithm is using a highly optimized version of Gansner's algorithm:
       * </p>
       * <ul>
       * <li>E.R. Gansner et al., <i>A Technique for Drawing Directed Graphs</i>, IEEE Transactions on Software Engineering, Vol.19, No.3, March 1993.</li>
       * </ul>
       * <p>
       * Minimum edge length and weights should be non-negative.
       * </p>
       * @param graph the input graph in which all the edges have directions, such that <code>rank[source] &lt; rank[target]</code> and
       * <code>rank[target] - rank[source] >= minlength[edge]</code>
       * @param rank the {@link } that will be filled during the execution and returns the integer ranking of each node
       * @param minLength the {@link } that returns an integer value (minimum/tight length) of each edge
       * @param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
       * @returns 
       * @see yfiles.algorithms.RankAssignments#simple
       * @static
       */
      static simple(graph:yfiles.algorithms.Graph,rank:yfiles.algorithms.INodeMap,minLength:yfiles.algorithms.IEdgeMap,maximalDuration?:number):number;
      /**
       * Similar to {@link yfiles.algorithms.RankAssignments#simplex} but, additionally, it is possible to provide a valid initial tree
       * solution for the problem.
       * <p>
       * Minimum edge length and weights should be non-negative.
       * </p>
       * @param graph the given graph
       * @param layer the {@link } that will be filled during the execution and returns the zero-based ranking index for
       * each node
       * @param w the {@link } that returns an integer value (weight) of each edge
       * @param minLength the {@link } that returns an integer value (minimum length) of each edge
       * @param tree the {@link } that returns a boolean value indicating whether or not an edge is a tree edge
       * @param _root the given root node of the tree solution
       * @param validRanking <code>true</code> if the argument <code>layer</code> contains a valid ranking, <code>false</code> otherwise
       * @param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
       * @returns the number of layers
       * @see yfiles.algorithms.RankAssignments#simplex
       * @see yfiles.algorithms.RankAssignments#simplex
       * @static
       */
      static simplex(graph:yfiles.algorithms.Graph,layer:yfiles.algorithms.INodeMap,w:yfiles.algorithms.IDataProvider,minLength:yfiles.algorithms.IDataProvider,tree:yfiles.algorithms.IEdgeMap,_root:yfiles.algorithms.Node,validRanking:boolean,maximalDuration?:number):number;
      /**
       * Solves the rank assignment problem using the simplex method given a maximum time duration for the algorithm.
       * <p>
       * This method assigns a minimum rank to the nodes in a acyclic graph.
       * </p>
       * <p>
       * Although its time complexity has not been proven polynomial, in practice it takes few iterations and runs quickly.
       * </p>
       * <p>
       * The algorithm is based on:
       * </p>
       * <ul>
       * <li>E.R. Gansner et al., <i>A Technique for Drawing Directed Graphs</i>, IEEE Transactions on Software Engineering, Vol.19, No.3, March 1993.</li>
       * </ul>
       * <p>
       * Minimum edge length and weights should be non-negative.
       * </p>
       * @param graph the given graph
       * @param layer the {@link } that will be filled during the execution and returns the zero-based ranking index for
       * each node
       * @param w the {@link } that returns an integer value (weight) of each edge
       * @param minLength the {@link } that returns an integer value (minimum length) of each edge
       * @param [maximalDuration=0x7FFFFFFFFFFFFFFF] a preferred time limit for the algorithm (in milliseconds)
       * @returns the number of layers
       * @see yfiles.algorithms.RankAssignments#simplex
       * @see yfiles.algorithms.RankAssignments#simplex
       * @static
       */
      static simplex(graph:yfiles.algorithms.Graph,layer:yfiles.algorithms.INodeMap,w:yfiles.algorithms.IDataProvider,minLength:yfiles.algorithms.IDataProvider,maximalDuration?:number):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides diverse algorithms and helper methods for solving the shortest path problem on weighted graphs.
     * <p>
     * <b>Definitions</b> Given a weighted directed/undirected graph:
     * </p>
     * <ul>
     * <li>The <b>shortest path problem</b> is the problem of finding a shortest path between <b>a source node <code>s</code>
     * and a target node <code>t</code></b> such that the sum of the edge costs is minimized.</li>
     * <li>The <b>k-shortest path problem</b> is the problem of finding <b><code>k</code> shortest paths</b> between <b>a source
     * node <code>s</code> and a target node <code>t</code></b> such that the sum of the edge costs is minimized.</li>
     * <li>The <b>single-source shortest path problem</b> is the problem of finding shortest paths from <b>a source node
     * <code>s</code> to all</b> other nodes such that the sum of the edge costs is minimized.</li>
     * <li>The <b>single-source single-sink shortest path problem</b> is the problem of finding shortest paths from <b>a source node <code>s</code> to a target node <code>t</code></b> such that the sum of the edge costs is minimized.</li>
     * <li>The <b>all-pairs shortest path problem</b> is the problem of finding shortest paths between <b>every pair of
     * nodes</b> such that the sum of the edge costs is minimized.</li>
     * </ul>
     * @class
     * @static
     */
    export interface ShortestPaths extends Object{}
    export class ShortestPaths {
      /**
       * Solves the single-source shortest path problem for <b>acyclic</b> directed graphs.
       * <p>
       * Each edge is associated with an arbitrary double value that represents the cost of that edge.
       * </p>
       * <p>
       * This method yields the shortest distance from a given node <code>s</code> to all other nodes.
       * </p>
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
       * nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
       * <code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
       * @param [pred=null] an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @returns <code>true</code> if the input graph is acyclic, <code>false</code> otherwise
       * @see yfiles.algorithms.ShortestPaths#acyclic
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @static
       */
      static acyclic(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,cost:number[],dist:number[],pred?:yfiles.algorithms.Edge[]):boolean;
      /**
       * Solves the single-source shortest path problem for <b>acyclic</b> directed graphs.
       * <p>
       * Each edge is associated with an arbitrary double value that represents the cost of that edge.
       * </p>
       * <p>
       * This method yields the shortest distance from a given node <code>s</code> to all other nodes.
       * </p>
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param cost the {@link } that returns the double value (cost) for traversing each edge
       * @param dist the {@link } that will be filled during the execution and returns a double value (shortest distance)
       * from node <code>s</code> to all other nodes or {@link #POSITIVE_INFINITY} if no such paths exist
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @returns <code>true</code> if the input graph is acyclic, <code>false</code> otherwise
       * @see yfiles.algorithms.ShortestPaths#acyclic
       * @static
       */
      static acyclic(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,cost:yfiles.algorithms.IDataProvider,dist:yfiles.algorithms.INodeMap,pred:yfiles.algorithms.INodeMap):boolean;
      /**
       * This method solves the all-pairs shortest path problem for graphs with arbitrary edge costs.
       * <p>
       * If the given graph contains a negative-cost cycle, then <code>false</code> is returned and the values returned in <code>dist</code> are left
       * unspecified.
       * </p>
       * @param graph the input graph
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @param dist an array of values that will be filled during the execution and returns the shortest path distances from all pairs of
       * nodes <code>s</code> and <code>t</code> in the graph. The distance from <code>s</code> to <code>t</code> is <code>dist[s.index()][t.index()]</code>. If there is no path from <code>s</code> to <code>t</code>,
       * then <code>dist[s.index()][t.index()] == Double.POSITIVE_INFINITY</code>.
       * @returns <code>true</code> if the given graph does not contain a negative-cost cycle, <code>false</code> otherwise
       * @static
       */
      static allPairs(graph:yfiles.algorithms.Graph,directed:boolean,cost:number[],dist:number[][]):boolean;
      /**
       * Solves the single-source shortest path problem for arbitrary graphs.
       * <p>
       * Each edge is associated with an arbitrary double value that represents the cost of this edge.
       * </p>
       * <p>
       * In case the given weighted graph contains no <b>negative-cost cycles</b>, this method will yield the shortest distance from a given node <code>s</code> to all
       * other nodes. If, on the other hand, the given graph contains negative-cost cycles, this method will yield no reasonable
       * result which will be indicated by the return value <code>false</code>.
       * </p>
       * @param graph the input graph
       * @param s the source node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost the {@link } that returns the double value (cost) for traversing each edge
       * @param dist the {@link } that will be filled during the execution and returns a double value (shortest distance)
       * from node <code>s</code> to all other nodes or {@link #POSITIVE_INFINITY} if no such paths exist
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @returns 
       * @see yfiles.algorithms.ShortestPaths#bellmanFord
       * @static
       */
      static bellmanFord(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,cost:yfiles.algorithms.IDataProvider,dist:yfiles.algorithms.INodeMap,pred:yfiles.algorithms.INodeMap):boolean;
      /**
       * Solves the single-source shortest path problem for arbitrary graphs.
       * <p>
       * Each edge is associated with an arbitrary double value that represents the cost of this edge.
       * </p>
       * <p>
       * In case the given weighted graph contains no <b>negative cost cycles</b>, this method will yield the shortest distance from a given node <code>s</code> to all
       * other nodes. If, on the other hand, the given graph contains negative-cost cycles, this method will yield no reasonable
       * result which will be indicated by the return value <code>false</code>.
       * </p>
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
       * nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
       * <code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
       * @param [pred=null] an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @returns 
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @see yfiles.algorithms.ShortestPaths#bellmanFord
       * @static
       */
      static bellmanFord(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,cost:number[],dist:number[],pred?:yfiles.algorithms.Edge[]):boolean;
      /**
       * Convenience method that constructs an explicit {@link yfiles.algorithms.EdgeList path of edges} from the result returned by one of the shortest paths
       * methods defined in this class.
       * <p>
       * If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
       * </p>
       * @param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
       * @param t the target node of the path
       * @param pred an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @returns a {@link  list} containing the edges on the shortest path from <code>s</code> to <code>t</code> in the correct order
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @static
       */
      static constructEdgePath(s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,pred:yfiles.algorithms.Edge[]):yfiles.algorithms.EdgeList;
      /**
       * Like {@link yfiles.algorithms.ShortestPaths#constructEdgePath} but the path edges are given by a
       * {@link yfiles.algorithms.IDataProvider}.
       * <p>
       * If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
       * </p>
       * @param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
       * @param t the target node of the path
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @returns a {@link  list} containing the edges on the shortest path from <code>s</code> to <code>t</code> in the correct order
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @static
       */
      static constructEdgePath(s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,pred:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList;
      /**
       * Convenience method that constructs an explicit {@link yfiles.algorithms.NodeList path of nodes} from the result returned by one of the shortest paths
       * methods defined in this class.
       * <p>
       * If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
       * </p>
       * @param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
       * @param t the target node of the path
       * @param pred an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @returns a {@link  list} containing the nodes on the shortest path from <code>s</code> to <code>t</code> in the correct order
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @static
       */
      static constructNodePath(s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,pred:yfiles.algorithms.Edge[]):yfiles.algorithms.NodeList;
      /**
       * Like {@link yfiles.algorithms.ShortestPaths#constructNodePath} but the path edges are given by a
       * {@link yfiles.algorithms.IDataProvider}.
       * <p>
       * If there is no path from node <code>s</code> to <code>t</code>, then an empty list is returned.
       * </p>
       * @param s the start node of the shortest path which must be the same start node that was specified when <code>pred</code> was calculated
       * @param t the target node of the path
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @returns a {@link  list} containing the nodes on the shortest path from <code>s</code> to <code>t</code> in the correct order
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @static
       */
      static constructNodePath(s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,pred:yfiles.algorithms.IDataProvider):yfiles.algorithms.NodeList;
      /**
       * Solves the single-source shortest path problem for arbitrary graphs.
       * <p>
       * Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
       * </p>
       * <p>
       * This method yields the shortest distance from a given node <code>s</code> to all other nodes.
       * </p>
       * <p>
       * The costs should be non-negative.
       * </p>
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
       * nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
       * <code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
       * @param [pred=null] an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @see yfiles.algorithms.ShortestPaths#dijkstra
       * @static
       */
      static dijkstra(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,cost:number[],dist:number[],pred?:yfiles.algorithms.Edge[]):void;
      /**
       * Solves the single-source shortest path problem for arbitrary graphs.
       * <p>
       * Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
       * </p>
       * <p>
       * This method yields the shortest distance from a given node <code>s</code> to all other nodes.
       * </p>
       * <p>
       * The costs should be non-negative.
       * </p>
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost the {@link } that returns the double value (cost) for traversing each edge
       * @param dist the {@link } that will be filled during the execution and returns a double value (shortest distance)
       * from node <code>s</code> to all other nodes or {@link #POSITIVE_INFINITY} if no such paths exist
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @see yfiles.algorithms.ShortestPaths#dijkstra
       * @static
       */
      static dijkstra(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,cost:yfiles.algorithms.IDataProvider,dist:yfiles.algorithms.INodeMap,pred:yfiles.algorithms.INodeMap):void;
      /**
       * Finds all nodes and edges that belong to a shortest path from a <code>start</code> node to a set of target nodes in the graph not
       * farther away than a given distance.
       * <p>
       * This method assumes that each edge of the input graph has a <b>cost of 1.0</b>.
       * </p>
       * @param graph the input graph
       * @param start the start node
       * @param targetMap the {@link } that returns a boolean value indicating whether or not a node belongs to the set of
       * target nodes
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param maxLength the maximum edge length of the shortest paths
       * @param pathEdges a {@link  list} that will be filled during the execution and returns the edges on the shortest path from <code>s</code> to <code>t</code> in the correct
       * order
       * @param pathNodes a {@link  list} that will be filled during the execution and returns the nodes on the shortest path from <code>s</code> to <code>t</code> in the correct
       * order
       * @static
       */
      static findShortestUniformPaths(graph:yfiles.algorithms.Graph,start:yfiles.algorithms.Node,targetMap:yfiles.algorithms.IDataProvider,directed:boolean,maxLength:number,pathEdges:yfiles.algorithms.EdgeList,pathNodes:yfiles.algorithms.NodeList):void;
      /**
       * Marks all edges that belong to a shortest path from <code>start</code> node to <code>target</code> node.
       * <p>
       * This method assumes that each edge of the input graph has a <b>cost of 1.0</b>.
       * </p>
       * @param graph the input graph
       * @param start the start node
       * @param end the target node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param pathMap the {@link } that will be filled during the execution and returns a boolean value indicating whether
       * or not the edge belongs to a shortest path connecting the two nodes
       * @static
       */
      static findShortestUniformPaths(graph:yfiles.algorithms.Graph,start:yfiles.algorithms.Node,end:yfiles.algorithms.Node,directed:boolean,pathMap:yfiles.algorithms.IEdgeMap):void;
      /**
       * This method finds the <code>k</code> shortest paths connecting a pair of nodes in a directed graph with non-negative edge costs.
       * <p>
       * The result will be returned as a list of {@link yfiles.algorithms.EdgeList} objects.
       * </p>
       * <p>
       * The cost should be non-negative.
       * </p>
       * @param graph the input graph
       * @param costDP the {@link } that returns a double value (cost) for traversing each edge
       * @param start the given start node
       * @param end the given target node
       * @param k a non-negative integer value
       * @returns a {@link  list} of {@link } objects each of which represents a path from <code>start</code> node to <code>target</code> node. The <code>i</code>-th
       * path in the list contains the <code>i</code>-th shortest path between the <code>start</code> and <code>target</code> node.
       * @static
       */
      static kShortestPaths(graph:yfiles.algorithms.Graph,costDP:yfiles.algorithms.IDataProvider,start:yfiles.algorithms.Node,end:yfiles.algorithms.Node,k:number):yfiles.algorithms.YList;
      /**
       * A variant of {@link yfiles.algorithms.ShortestPaths#kShortestPaths} that returns the result as a special cursor that calculates
       * the next path in the sequence only when needed.
       * <p>
       * The returned cursor only supports the operation {@link yfiles.algorithms.ICursor#ok}, {@link yfiles.algorithms.ICursor#current} and {@link yfiles.algorithms.ICursor#next}.
       * </p>
       * <p>
       * The cost should be non-negative.
       * </p>
       * @param graph the input graph
       * @param costDP the {@link } that returns a double value (cost) for traversing each edge
       * @param start the given start node
       * @param end the given target node
       * @param k a non-negative integer value
       * @returns a {@link  cursor} that calculates the next path in the sequence only when needed
       * @static
       */
      static kShortestPathsCursor(graph:yfiles.algorithms.Graph,costDP:yfiles.algorithms.IDataProvider,start:yfiles.algorithms.Node,end:yfiles.algorithms.Node,k:number):yfiles.algorithms.ICursor;
      /**
       * Returns two edge-disjoint paths in a non-negatively weighted directed graph, such that both paths connect nodes <code>s</code> and <code>t</code>
       * and have minimum total length.
       * @param graph the input graph
       * @param source the source node of the shortest pair
       * @param target the target node of the shortest pair
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param costDP the {@link } that returns a double value (cost) for traversing each edge
       * @returns a two-dimensional array of {@link }s holding the resulting edge-disjoint paths or <code>null</code> if no such
       * edge-disjoint paths exist
       * @static
       */
      static shortestPair(graph:yfiles.algorithms.Graph,source:yfiles.algorithms.Node,target:yfiles.algorithms.Node,directed:boolean,costDP:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList[];
      /**
       * This method solves the single-source shortest path problem for arbitrary graphs.
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
       * nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
       * <code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
       * @param [pred=null] an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @returns <code>true</code> if the weighted graph does not contain a negative-cost cycle, <code>false</code> otherwise
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @see yfiles.algorithms.ShortestPaths#singleSource
       * @static
       */
      static singleSource(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,cost:number[],dist:number[],pred?:yfiles.algorithms.Edge[]):boolean;
      /**
       * This method solves the single-source shortest path problem for arbitrary graphs.
       * @param graph the input graph
       * @param s the source node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost the {@link } that returns the double value (cost) for traversing each edge
       * @param dist the {@link } that will be filled during the execution and returns a double value (shortest distance)
       * from node <code>s</code> to all other nodes or {@link #POSITIVE_INFINITY} if no such paths exist
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @returns <code>true</code> if the weighted graph does not contain a negative-cost cycle, <code>false</code> otherwise
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @see yfiles.algorithms.ShortestPaths#singleSource
       * @static
       */
      static singleSource(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,cost:yfiles.algorithms.IDataProvider,dist:yfiles.algorithms.INodeMap,pred:yfiles.algorithms.INodeMap):boolean;
      /**
       * This method solves the single-source single-sink shortest path problem for arbitrary graphs.
       * <p>
       * Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
       * </p>
       * <p>
       * This method returns the shortest distance from node <code>s</code> to node
       * <code>t</code>. It also returns information to construct the actual path between these two nodes.
       * </p>
       * <p>
       * The costs should be non-negative.
       * </p>
       * @param graph the input graph
       * @param s the source node
       * @param t the sink node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @param pred an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @returns the distance between <code>s</code>and <code>t</code> if a path between these two nodes exists or <code>Double.POSITIVE_INFINITY</code> otherwise
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @static
       */
      static singleSourceSingleSink(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,directed:boolean,cost:number[],pred:yfiles.algorithms.Edge[]):number;
      /**
       * Like {@link yfiles.algorithms.ShortestPaths#singleSourceSingleSink} but uses {@link yfiles.algorithms.INodeMap}s and
       * {@link yfiles.algorithms.IDataProvider}s instead of arrays.
       * <p>
       * Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
       * </p>
       * <p>
       * The costs should be non-negative.
       * </p>
       * @param graph the input graph
       * @param s the source node
       * @param t the sink node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost the {@link } that returns the double value (cost) for traversing each edge
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @returns a shortest {@link  path of edges} between source and sink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @static
       */
      static singleSourceSingleSink(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,directed:boolean,cost:yfiles.algorithms.IDataProvider,pred:yfiles.algorithms.INodeMap):number;
      /**
       * Similar to {@link yfiles.algorithms.ShortestPaths#singleSourceSingleSink} but instead of returning the shortest distance between
       * the source and sink the actual shortest edge path between these nodes will be returned.
       * <p>
       * Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
       * </p>
       * <p>
       * If the returned path is empty, then there is no path between the nodes.
       * </p>
       * <p>
       * The costs should be non-negative.
       * </p>
       * @param graph the input graph
       * @param s the source node
       * @param t the sink node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost an array of double values that returns the costs for traversing each edge; edge <code>e</code> has cost <code>cost[e.index()]</code>
       * @returns a shortest {@link  path of edges} between source and sink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @static
       */
      static singleSourceSingleSink(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,directed:boolean,cost:number[]):yfiles.algorithms.EdgeList;
      /**
       * Similar to {@link yfiles.algorithms.ShortestPaths#singleSourceSingleSink} but instead of returning the shortest distance between
       * the source and sink the actual shortest edge path between these nodes will be returned.
       * <p>
       * Each edge is associated with a <b>non-negative</b> double value that represents the cost of the edge.
       * </p>
       * <p>
       * If the returned path is empty, then there is no path between the nodes.
       * </p>
       * <p>
       * The costs should be non-negative.
       * </p>
       * @param graph the input graph
       * @param s the source node
       * @param t the sink node
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param cost the {@link } that returns the double value (cost) for traversing each edge
       * @returns a shortest {@link  path of edges} between source and sink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @see yfiles.algorithms.ShortestPaths#singleSourceSingleSink
       * @static
       */
      static singleSourceSingleSink(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,t:yfiles.algorithms.Node,directed:boolean,cost:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList;
      /**
       * Solves the single-source shortest path problem for arbitrary graphs in which each edge has a <b>uniform cost of
       * <code>1.0</code></b>.
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param dist an array of values that will be filled during the execution and returns the shortest distance from node <code>s</code> to all other
       * nodes. The distance from <code>s</code> to <code>v</code> is <code>dist[v.index()]</code>. If there is no path from <code>s</code> to <code>v</code>, then
       * <code>dist[v.index()] == Double.POSITIVE_INFINITY</code>.
       * @param [pred=null] an array of {@link }s that will be filled during the execution and returns for each node <code>t</code> the shortest
       * path edge <code>pred[t.index()]</code> which is the last edge on the shortest path from <code>s</code> to <code>t</code>. If <code>t == s</code> or if there is no shortest
       * path from <code>s</code> to <code>t</code>, then <code>pred[t.index()] == null</code>.
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @static
       */
      static uniform(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,dist:number[],pred?:yfiles.algorithms.Edge[]):void;
      /**
       * Like {@link yfiles.algorithms.ShortestPaths#uniform} but uses {@link yfiles.algorithms.INodeMap}s instead of arrays.
       * @param graph the input graph
       * @param s the node from which the shortest path search starts
       * @param directed <code>true</code> if the graph should be considered as directed, <code>false</code> otherwise
       * @param dist the {@link } that will be filled during the execution and returns a double value (shortest distance)
       * from node <code>s</code> to all other nodes or {@link #POSITIVE_INFINITY} if no such paths exist
       * @param pred the {@link } that will be filled during the execution and returns for each node <code>t</code> the last edge on the
       * shortest path from <code>s</code> to <code>t</code> or <code>null</code> if <code>t == s</code> or no shortest path from <code>s</code> to <code>t</code> exists
       * @see yfiles.algorithms.ShortestPaths#uniform
       * @see yfiles.algorithms.ShortestPaths#constructNodePath
       * @see yfiles.algorithms.ShortestPaths#constructEdgePath
       * @static
       */
      static uniform(graph:yfiles.algorithms.Graph,s:yfiles.algorithms.Node,directed:boolean,dist:yfiles.algorithms.INodeMap,pred:yfiles.algorithms.INodeMap):void;
      /**
       * Convenience method that returns an array containing uniform edge costs of <code>1.0</code> for each edge of the given graph.
       * @param graph the input graph
       * @returns an array that contains uniform edge costs of value <code>1.0</code> (i.e., for each edge <code>e: cost[e.index()] == 1.0</code>)
       * @static
       */
      static uniformCost(graph:yfiles.algorithms.Graph):number[];
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides methods for efficiently sorting graph elements in graph structures.
     * @class
     * @static
     */
    export interface Sorting extends Object{}
    export class Sorting {
      /**
       * Sorts the nodes of a given graph by degree in ascending order.
       * @param graph the given graph
       * @returns an array of {@link }s sorted by degree in ascending order
       * @static
       */
      static sortNodesByDegree(graph:yfiles.algorithms.Graph):yfiles.algorithms.Node[];
      /**
       * Sorts the nodes of a given graph by an integer key associated with each node through a {@link yfiles.algorithms.IDataProvider}.
       * <p>
       * The nodes are sorted in ascending order.
       * </p>
       * @param graph the given graph
       * @param keys the {@link } that returns the keys based on which the nodes will be sorted
       * @returns an array of {@link }s sorted by an integer key in ascending order
       * @static
       */
      static sortNodesByIntKey(graph:yfiles.algorithms.Graph,keys:yfiles.algorithms.IDataProvider):yfiles.algorithms.Node[];
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides (minimum) spanning tree algorithms for graphs.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>A <b>spanning tree</b> of an undirected connected graph is a subset of its edges that induce a tree that connects all nodes of the graph.</li>
     * <li>
     * A <b>minimum spanning tree</b> of a weighted connected graph is a spanning tree whose edges have minimum overall cost among all spanning trees of
     * that graph.
     * </li>
     * </ul>
     * @class
     * @static
     */
    export interface SpanningTrees extends Object{}
    export class SpanningTrees {
      /**
       * Returns the overall cost of a previously calculated minimum spanning tree.
       * @param treeEdges the given {@link  list} of edges that form a minimum spanning tree
       * @param edgeCost the {@link } that returns a double value (cost) for each tree edge
       * @returns the overall cost of the tree edges
       * @static
       */
      static cost(treeEdges:yfiles.algorithms.EdgeList,edgeCost:yfiles.algorithms.IDataProvider):number;
      /**
       * Calculates a minimum spanning tree for the given graph.
       * <p>
       * The implementation is based on an algorithm originally published in:
       * </p>
       * <ul>
       * <li>J.B. Kruskal. On the shortest spanning subtree of a graph and the traveling salesman problem. <i>Proceedings of the American Mathematical Society</i>, pages 48-50, 1956.</li>
       * </ul>
       * @param graph the input graph
       * @param cost the {@link } that returns a double value (cost) for each edge
       * @returns a {@link  list} containing the edges that form the minimum spanning tree
       * @static
       */
      static kruskal(graph:yfiles.algorithms.Graph,cost:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList;
      /**
       * Calculates a minimum spanning tree for the given graph.
       * <p>
       * Currently, the result is obtained by calling {@link yfiles.algorithms.SpanningTrees#prim}.
       * </p>
       * @param graph the input graph
       * @param cost the {@link } that returns a double value (cost) for each edge
       * @returns a {@link  list} containing the edges that form the minimum spanning tree
       * @static
       */
      static minimum(graph:yfiles.algorithms.Graph,cost:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList;
      /**
       * Calculates a minimum spanning tree for the given graph.
       * <p>
       * The implementation is based on an algorithm originally published in:
       * </p>
       * <ul>
       * <li>R.C. Prim. Shortest connection networks and some generalizations. <i>Bell System Technical Journal</i>, 36:1389-1401, 1957.</li>
       * </ul>
       * @param graph the input graph
       * @param cost the {@link } that returns a double value (cost) for each edge
       * @returns a {@link  list} containing the edges that form the minimum spanning tree
       * @static
       */
      static prim(graph:yfiles.algorithms.Graph,cost:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList;
      /**
       * Calculates a spanning tree for the given graph in which each edge has a uniform cost of <code>1.0</code>.
       * @param graph the input graph
       * @returns a {@link  list} containing the edges that form the minimum spanning tree
       * @static
       */
      static uniform(graph:yfiles.algorithms.Graph):yfiles.algorithms.EdgeList;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides algorithms to compute reachability information for directed, acyclic graphs.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>
     * <b>Reflexive, transitive closure:</b> Let <code>G = (V,E)</code> be a directed acyclic graph. The <b>reflexive, transitive closure</b> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists a path
     * from <code>v</code> to <code>w</code> in <code>G</code>.
     * </li>
     * <li>
     * <b>Transitive reduction:</b> Let <code>G = (V,E)</code> be a directed acyclic graph. The <b>transitive reduction</b> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists no path
     * from <code>v</code> to <code>w</code> in <code>G</code> of length <code>2</code> or more.
     * </li>
     * </ul>
     * @class
     * @static
     */
    export interface Transitivity extends Object{}
    export class Transitivity {
      /**
       * Calculates the transitive closure for a directed acyclic graph.
       * <p>
       * Given a <code>G = (V,E)</code> be a directed acyclic graph. The <i>reflexive, transitive closure</i> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists a
       * path from <code>v</code> to <code>w</code> in <code>G</code>.
       * </p>
       * @param graph the input graph to which this method will add transitive edges, if necessary
       * @param [addedEdges=null] a {@link  list} that will be filled during the execution and contains the edges that have been added to the graph by this method
       * @static
       */
      static transitiveClosure(graph:yfiles.algorithms.Graph,addedEdges?:yfiles.algorithms.EdgeList):void;
      /**
       * Calculates the transitive reduction for a directed acyclic graph.
       * <p>
       * The transitive edges in the graph will be removed by this method.
       * </p>
       * <p>
       * Given <code>G = (V,E)</code> be a directed acyclic graph. The <i>transitive reduction</i> of <code>G</code> is a graph which contains edge <code>(v,w)</code> only if there exists no path
       * from <code>v</code> to <code>w</code> in <code>G</code> of length <code>2</code> or more.
       * </p>
       * @param graph the input graph
       * @param [transitiveEdges=null] a {@link  list} that will be filled during the execution and contains all transitive edges of the given graph; removal of these
       * edges will yield the transitive reduction of the graph
       * @static
       */
      static transitiveReduction(graph:yfiles.algorithms.Graph,transitiveEdges?:yfiles.algorithms.EdgeList):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides diverse algorithms and services for tree-structured graphs or subgraphs.
     * <p>
     * <b>Definitions</b>
     * </p>
     * <ul>
     * <li>
     * <b>Tree:</b> An acyclic graph, in which any pair of vertices (nodes) is connected through a path. If one vertex of a tree is
     * distinguished from the other vertices, then this vertex is called the <b>root</b> and the tree is called a <b>rooted tree</b>.
     * </li>
     * <li><b>Directed rooted tree:</b> A rooted tree where edges are directed from the root to the leaves.</li>
     * <li><b>Depth:</b> The depth of a vertex in a rooted tree is the number of edges of the unique path between this vertex and the root.</li>
     * <li>
     * <b>Parent:</b> In a rooted tree a vertex <code>v</code> is called parent of a vertex <code>w</code> if <code>v</code> and <code>w</code> are adjacent (i.e. connected by an edge) and the
     * unique path between <code>w</code> and the root contains <code>v</code>. Note that each vertex except the root has exactly one parent.
     * </li>
     * <li><b>Children:</b> In a rooted tree a vertex <code>w</code> is called child of a vertex <code>v</code> if <code>v</code> is the parent of <code>w</code>. A vertex may have several children.</li>
     * <li><b>N-ary tree:</b> A directed rooted tree where each node has a maximum of <code>n</code> children.</li>
     * <li><b>Forest:</b> A graph whose connected components are trees.</li>
     * <li>
     * <b>Leaf:</b> A leaf <code>v</code> is a node with out-degree (i.e., the number of edges having <code>v</code> as a target) zero if the input is a directed
     * rooted tree, and a node with degree (i.e., the number of edges incident to <code>v</code>) one, otherwise.
     * </li>
     * <li><b>Subtree:</b> A subtree of a tree <code>T</code> is a subgraph of <code>T</code> which is also a tree.</li>
     * <li>
     * <b>Nearest or Lowest or Least common ancestor:</b> The nearest common ancestor of two nodes <code>u</code> and <code>v</code> in a tree graph is the shared ancestor of <code>u</code> and <code>v</code> that is located
     * farthest from the root.
     * </li>
     * <li><b>Eccentricity:</b> The eccentricity of a tree node is the maximum distance to any other node.</li>
     * <li><b>Center node:</b> The center of a tree is the set of nodes that have minimal eccentricity.</li>
     * </ul>
     * @class
     * @static
     */
    export interface Trees extends Object{}
    export class Trees {
      static collectSubtree(root:yfiles.algorithms.Node,nodes:yfiles.algorithms.NodeList):void;
      /**
       * Converts the given tree to a directed rooted tree with the given node as root element by reversing some edges.
       * <p>
       * A list of all reversed edges will be returned by this method.
       * </p>
       * @param tree the given tree
       * @param [root=null] the given root element
       * @returns an {@link } containing the reversed edges
       * @static
       */
      static directTree(tree:yfiles.algorithms.Graph,root?:yfiles.algorithms.Node):yfiles.algorithms.EdgeList;
      /**
       * Returns the center node of an undirected tree.
       * <p>
       * The center node has the property of inducing a minimum depth tree when being used as the root of that tree.
       * </p>
       * @param tree the given undirected tree
       * @returns the center node of the given undirected tree
       * @static
       */
      static getCenterRoot(tree:yfiles.algorithms.Graph):yfiles.algorithms.Node;
      /**
       * Returns all leaf nodes of the given tree.
       * <p>
       * A leaf node is a node with <code>outdegree == 0</code> if the input is a directed rooted tree, and a node with
       * <code>degree == 1</code>, otherwise.
       * </p>
       * @param tree the given tree
       * @param directedRootedTree <code>true</code> if the algorithm should consider the tree as directed, <code>false</code> otherwise
       * @returns a {@link } that contains all leaf nodes of the given tree
       * @static
       */
      static getLeafNodes(tree:yfiles.algorithms.Graph,directedRootedTree:boolean):yfiles.algorithms.NodeList;
      /**
       * Returns the nearest common ancestor of a subset of nodes within a directed rooted tree.
       * <p>
       * It is not part of the given subset.
       * </p>
       * @param tree the given directed rooted tree
       * @param root the root of the tree
       * @param rootedDownward <code>true</code> if the tree is directed from the root to the leaves, <code>false</code> otherwise
       * @param nodes the subset of nodes
       * @returns the {@link  nearest common ancestor} of the given subset of nodes
       * @static
       */
      static getNearestCommonAncestor(tree:yfiles.algorithms.Graph,root:yfiles.algorithms.Node,rootedDownward:boolean,nodes:yfiles.algorithms.NodeList):yfiles.algorithms.Node;
      /**
       * Returns a possible root for the given (undirected) tree.
       * <p>
       * More precisely:
       * </p>
       * <ul>
       * <li>If the input is a directed rooted tree or reversed directed rooted tree, it returns the corresponding root node.</li>
       * <li>
       * If the input is a tree, the method returns a maximum weight center node as defined in
       * {@link yfiles.algorithms.Trees#getWeightedCenterNode}.
       * </li>
       * <li>If the input is not a tree, a node with <code>indegree == 0</code> (or <code>outdegree == 0</code>) is returned.</li>
       * </ul>
       * @param tree the given tree
       * @returns a possible root for the given tree
       * @static
       */
      static getRoot(tree:yfiles.algorithms.Graph):yfiles.algorithms.Node;
      /**
       * Returns the depths of each subtree of a rooted directed tree.
       * @param tree a rooted directed tree graph
       * @param subtreeDepthMap the {@link } that will be filled during the execution with the depth of the subtree rooted at each
       * node
       * @static
       */
      static getSubTreeDepths(tree:yfiles.algorithms.Graph,subtreeDepthMap:yfiles.algorithms.INodeMap):void;
      /**
       * Returns the size (number of nodes) of each subtree of a rooted directed tree.
       * @param tree a rooted directed tree graph
       * @param subtreeSizeMap the {@link } that will be filled during the execution with the size of the subtree rooted at each
       * node
       * @static
       */
      static getSubTreeSizes(tree:yfiles.algorithms.Graph,subtreeSizeMap:yfiles.algorithms.INodeMap):void;
      /**
       * Returns an array of {@link yfiles.algorithms.EdgeList} objects each containing edges that belong to a maximal directed subtree
       * of the given graph.
       * <p>
       * This method can also be applied to the result obtained by
       * {@link yfiles.algorithms.Trees#getUndirectedTreeNodes}. In this case, the subtrees are considered to be undirected.
       * </p>
       * @param graph the given graph
       * @param [treeNodes=null] an array of {@link }s previously calculated by {@link #getTreeNodes}
       * @returns an array of {@link } objects each containing edges that belong to a maximal subtree
       * @static
       */
      static getTreeEdges(graph:yfiles.algorithms.Graph,treeNodes?:yfiles.algorithms.NodeList[]):yfiles.algorithms.EdgeList[];
      /**
       * Returns an array of {@link yfiles.algorithms.NodeList} objects each containing nodes that belong to a maximal directed subtree
       * of the given graph.
       * <p>
       * For each list of tree nodes, the first node element is the root of a tree. On each such root, all outgoing edges connect
       * to nodes in the subtree and each in-degree of the root is at least two.
       * </p>
       * @param graph the given graph
       * @returns an array of {@link } objects each containing nodes that belong to a maximal directed subtree
       * @static
       */
      static getTreeNodes(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList[];
      /**
       * Returns an array of {@link yfiles.algorithms.NodeList} objects each containing nodes that belong to a maximal undirected subtree
       * of the given graph.
       * <p>
       * For each list of tree nodes, the first node is the only node of the subtree that may be incident to non-tree edges.
       * </p>
       * @param graph the given graph
       * @returns an array of {@link } objects each containing nodes that belong to a maximal undirected subtree
       * @static
       */
      static getUndirectedTreeNodes(graph:yfiles.algorithms.Graph):yfiles.algorithms.NodeList[];
      /**
       * Finds a node used by the greatest number of all (undirected) paths interconnecting all nodes with each other.
       * <p>
       * The number of paths per node are stored in the given {@link yfiles.algorithms.INodeMap}.
       * </p>
       * @param tree the given tree
       * @param [intWeight=null] the {@link } that holds the number of paths per node
       * @returns a {@link } used by the greatest number of all undirected paths
       * @static
       */
      static getWeightedCenterNode(tree:yfiles.algorithms.Graph,intWeight?:yfiles.algorithms.INodeMap):yfiles.algorithms.Node;
      /**
       * Checks whether or not the given graph is a forest.
       * <p>
       * If
       * <code>directedRootedTree == true</code>, each component has to be a directed rooted tree. Otherwise, each component has to be an
       * undirected tree.
       * </p>
       * @param graph the given graph
       * @param directedRootedTree <code>true</code> if the algorithm should check for directed rooted trees, <code>false</code> otherwise
       * @returns <code>true</code> if the given graph is a forest, <code>false</code> otherwise
       * @static
       */
      static isForest(graph:yfiles.algorithms.Graph,directedRootedTree:boolean):boolean;
      /**
       * Checks whether or not the given graph is a forest, that is, a graph whose connected components are directed rooted
       * trees.
       * @param graph the given graph
       * @returns <code>true</code> if the given graph is a forest, <code>false</code> otherwise
       * @static
       */
      static isForest(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph is a directed rooted tree in which each node has a maximum of <code>n</code> children.
       * @param graph the given graph
       * @param n the allowed maximum of children
       * @returns <code>true</code> if the given graph is a n-ary tree, <code>false</code> otherwise
       * @static
       */
      static isNaryTree(graph:yfiles.algorithms.Graph,n:number):boolean;
      /**
       * Checks whether or not the given graph is a directed rooted tree.
       * @param graph the given graph
       * @returns <code>true</code> if the given graph is a directed rooted tree, <code>false</code> otherwise
       * @static
       */
      static isRootedTree(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Checks whether or not the given graph is an undirected tree.
       * @param graph the given graph
       * @returns <code>true</code> if the given graph is an undirected tree, <code>false</code> otherwise
       * @static
       */
      static isTree(graph:yfiles.algorithms.Graph):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides access to some Comparator instances that are commonly used in yFiles.
     * @class
     * @static
     */
    export interface Comparers extends Object{}
    export class Comparers {
      /**
       * Compares the specified floating point numbers.
       * <p>
       * Returns a negative integer, zero, or a positive integer as the first argument is less than, equal to, or greater than
       * the second.
       * </p>
       * <p>
       * <b>Warning:</b> This method does <b>not</b> handle <code>NaN</code>! If you need <code>NaN</code>-safe comparison, use {@link yfiles.lang.Object_Interface#compare} instead.
       * </p>
       * @param d1 the first number to compare.
       * @param d2 the second number to compare.
       * @returns a negative integer, zero, or a positive integer as the first argument is less than, equal to, or greater than the
       * second.
       * @see yfiles.collections.IComparer.<T>#compare
       * @static
       */
      static compare(d1:number,d2:number):number;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares to Objects of type Comparable.
       * @returns 
       * @static
       */
      static createComparableComparer():yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of arbitrary type.
       * <p>
       * Two objects are compared by comparing the {@link yfiles.lang.IComparable} instances the given data provider returns for each of
       * these objects.
       * </p>
       * @param dp a data provider that must return a {@link } for each object that is being compared by this comparator.
       * @returns a {@link } that compares arbitrary objects.
       * @static
       */
      static createComparableDataComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of arbitrary type.
       * <p>
       * Two objects are compared by comparing the int value the given data provider returns for each of these objects.
       * </p>
       * @param dp a data provider that must return an int value for each object that is being compared by this comparator.
       * @returns a {@link } that compares arbitrary objects.
       * @static
       */
      static createIntDataComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of type {@link yfiles.algorithms.Edge}.
       * <p>
       * Two edges are compared by comparing their source nodes. Each source node <code>e.source()</code> in turn is compared by the int value
       * provided by the given data provider: <code>dp.getInt(e.source())</code>.
       * </p>
       * @param dp a data provider that must return an int value for the source node of each edge being compared.
       * @returns a {@link } that compares edges.
       * @static
       */
      static createIntDataSourceComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of type {@link yfiles.algorithms.Edge}.
       * <p>
       * Two edges are compared by comparing their target nodes. Each target node <code>e.target()</code> in turn is compared by the int value
       * provided by the given data provider: <code>dp.getInt(e.target())</code>.
       * </p>
       * @param dp a data provider that must return an int value for the target node of each edge being compared.
       * @returns a {@link } that compares edges.
       * @static
       */
      static createIntDataTargetComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of arbitrary type.
       * <p>
       * Two objects are compared by comparing the double value the given data provider returns for each of these objects.
       * </p>
       * @param dp a data provider that must return a double value for each object that is being compared by this comparator.
       * @returns a {@link } that compares arbitrary objects.
       * @static
       */
      static createNumberDataComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of type {@link yfiles.algorithms.Edge}.
       * <p>
       * Two edges are compared by comparing their source nodes. Each source node <code>e.source()</code> in turn is compared by the double
       * value provided by the given data provider: <code>dp.getDouble(e.source())</code>.
       * </p>
       * @param dp a data provider that must return a double value for the source node of each edge being compared.
       * @returns a {@link } that compares edges.
       * @static
       */
      static createNumberDataSourceComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      /**
       * Returns a {@link yfiles.collections.IComparer.<T>} that compares objects of type {@link yfiles.algorithms.Edge}.
       * <p>
       * Two edges are compared by comparing their target nodes. Each target node <code>e.target()</code> in turn is compared by the double
       * value provided by the given data provider: <code>dp.getDouble(e.target())</code>.
       * </p>
       * @param dp a data provider that must return a double value for the target node of each edge being compared.
       * @returns a {@link } that compares edges.
       * @static
       */
      static createNumberDataTargetComparer(dp:yfiles.algorithms.IDataProvider):yfiles.collections.IComparer<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides utility methods for working with {@link yfiles.algorithms.ICursor cursors}.
     * @class
     * @static
     */
    export interface Cursors extends Object{}
    export class Cursors {
      /**
       * Creates a new cursor that provides a logical view on the concatenation of the two given cursors.
       * @param c1 - first concatenation argument
       * @param c2 - second concatenation argument
       * @returns 
       * @static
       */
      static concatenate(c1:yfiles.algorithms.ICursor,c2:yfiles.algorithms.ICursor):yfiles.algorithms.ICursor;
      /**
       * Creates a cursor view of the given collection.
       * <p>
       * Creates an ICursor view of the given collection.
       * </p>
       * <p>
       * Note that the returned cursor does not support the {@link yfiles.algorithms.ICursor#prev prev} and {@link yfiles.algorithms.ICursor#toLast toLast} operations.
       * </p>
       * Type parameter <code>T</code>.
       * @param c The collection.
       * @returns The cursor view of the given collection.
       * @template T
       * @static
       */
      static createCursor<T>(c:yfiles.collections.ICollection<T>):yfiles.algorithms.ICursor;
      /**
       * Creates a cursor view of the given collection.
       * <p>
       * Note that the returned cursor does not support the operations {@link yfiles.algorithms.ICursor#prev prev}, {@link yfiles.algorithms.ICursor#toLast toLast}, and
       * {@link yfiles.algorithms.IEdgeCursor#cyclicPrev}.
       * </p>
       * Type parameter <code>T</code>.
       * @param c The collection.
       * @returns an {@link } view of the given collection.
       * @template T
       * @static
       */
      static createEdgeCursor<T>(c:yfiles.collections.ICollection<T>):yfiles.algorithms.IEdgeCursor;
      /**
       * Creates a cursor view of the given collection.
       * <p>
       * Note that the returned cursor does not support the operations {@link yfiles.algorithms.ICursor#prev prev}, {@link yfiles.algorithms.ICursor#toLast toLast}, and
       * {@link yfiles.algorithms.INodeCursor#cyclicPrev}.
       * </p>
       * Type parameter <code>T</code>.
       * @param c The collection.
       * @returns an {@link } view of the given collection.
       * @template T
       * @static
       */
      static createNodeCursor<T>(c:yfiles.collections.ICollection<T>):yfiles.algorithms.INodeCursor;
      /**
       * Creates or fills an array with the values provided by the cursor
       * @param cursor the cursor
       * @param dest the array to fill with the values or <code>null</code> if the method should create the array itself
       * @returns dest or a newly created array filled with the values from cursor
       * @static
       */
      static toArray(cursor:yfiles.algorithms.ICursor,dest:Object[]):Object[];
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract adapter class for providing data.
     * <p>
     * The data provision methods in this class throw a {@link yfiles.lang.Exception} and {@link yfiles.algorithms.DataProviderAdapter#defined}
     * always returns <code>false</code>.
     * </p>
     * <p>
     * This class exists as a convenience for creating data provider objects.
     * </p>
     * <p>
     * Extend this class to provide either typed or untyped data for a certain lookup domain.
     * </p>
     * @class
     * @implements {yfiles.algorithms.IDataProvider}
     */
    export interface DataProviderAdapter extends Object,yfiles.algorithms.IDataProvider{}
    export class DataProviderAdapter {
      /**
       * Returns <code>false</code> for all data holders.
       * <p>
       * Subclasses should override this method to make clear for which data holders there is a value accessible via this data
       * provider.
       * </p>
       * @param dataHolder
       * @returns <code>false</code>.
       */
      defined(dataHolder:Object):boolean;
      /**
       * Subclasses may override this method to provide access to object values.
       * @param dataHolder
       * @returns 
       * @throws {Stubs.Exceptions.NotSupportedError} unless overwritten.
       */
      get(dataHolder:Object):Object;
      /**
       * Subclasses may override this method to provide access to boolean values.
       * @param dataHolder
       * @returns 
       * @throws {Stubs.Exceptions.NotSupportedError} unless overwritten.
       */
      getBoolean(dataHolder:Object):boolean;
      /**
       * Subclasses may override this method to provide access to integer values.
       * @param dataHolder
       * @returns 
       * @throws {Stubs.Exceptions.NotSupportedError} unless overwritten.
       */
      getInt(dataHolder:Object):number;
      /**
       * Subclasses may override this method to provide access to double values.
       * @param dataHolder
       * @returns 
       * @throws {Stubs.Exceptions.NotSupportedError} unless overwritten.
       */
      getNumber(dataHolder:Object):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides convenience and transformation services for DataProviders.
     * @class
     * @static
     */
    export interface DataProviders extends Object{}
    export class DataProviders {
      /**
       * Returns a DataProvider that returns the given value for each key.
       * @param data constant Object data returned by the created data provider.
       * @returns a data provider view of a single value.
       * @static
       */
      static createConstantDataProvider(data:Object):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of an Object array defined for edges.
       * <p>
       * The Object value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>get(edge)</code>.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createEdgeDataProvider(data:Object[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of a double, int, boolean and Object array defined for edges.
       * <p>
       * The double value <code>doubleData[edge.index()]</code> will be returned by the data provider upon the method call <code>getDouble(edge)</code>.
       * </p>
       * <p>
       * The int value <code>intData[edge.index()]</code> will be returned by the data provider upon the method call <code>getInt(edge)</code>.
       * </p>
       * <p>
       * The boolean value <code>boolData[edge.index()]</code> will be returned by the data provider upon the method call <code>getBool(edge)</code>.
       * </p>
       * <p>
       * The Object value <code>objectData[edge.index()]</code> will be returned by the data provider upon the method call <code>get(edge)</code>.
       * </p>
       * @param doubleData double data for each edge of a static graph
       * @param intData int data for each edge of a static graph
       * @param boolData boolean data for each edge of a static graph
       * @param objectData Object data for each edge of a static graph
       * @returns a data provider view of the given arrays
       * @static
       */
      static createEdgeDataProviderForArrays(doubleData:number[],intData:number[],boolData:boolean[],objectData:Object[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of a boolean array defined for edges.
       * <p>
       * The boolean value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>getBool(edge)</code>.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createEdgeDataProviderForBoolean(data:boolean[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of an int array defined for edges.
       * <p>
       * The int value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>getInt(edge)</code>.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createEdgeDataProviderForInt(data:number[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of a double array defined for edges.
       * <p>
       * The double value <code>data[edge.index()]</code> will be returned by the data provider upon the method call <code>getDouble(edge)</code>.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createEdgeDataProviderForNumber(data:number[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider that returns the negated boolean values provided by another data provider.
       * @param data
       * @returns 
       * @static
       */
      static createNegatedDataProvider(data:yfiles.algorithms.IDataProvider):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of an Object array defined for nodes.
       * <p>
       * The Object value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>get(node)</code>.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createNodeDataProvider(data:Object[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of a boolean array defined for nodes.
       * <p>
       * The boolean value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>getBool(node)</code>.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createNodeDataProviderForBoolean(data:boolean[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of an int array defined for nodes.
       * <p>
       * The int value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>getInt(node)</code>.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createNodeDataProviderForInt(data:number[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of a double array defined for nodes.
       * <p>
       * The double value <code>data[node.index()]</code> will be returned by the data provider upon the method call <code>getDouble(node)</code>.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a data provider view of the given array
       * @static
       */
      static createNodeDataProviderForNumber(data:number[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider view of a double, int, boolean and Object array defined for nodes.
       * <p>
       * The double value <code>doubleData[node.index()]</code> will be returned by the data provider upon the method call <code>getDouble(node)</code>.
       * </p>
       * <p>
       * The int value <code>intData[node.index()]</code> will be returned by the data provider upon the method call <code>getInt(node)</code>.
       * </p>
       * <p>
       * The boolean value <code>boolData[node.index()]</code> will be returned by the data provider upon the method call <code>getBool(node)</code>.
       * </p>
       * <p>
       * The Object value <code>objectData[node.index()]</code> will be returned by the data provider upon the method call <code>get(node)</code>.
       * </p>
       * @param doubleData double data for each node of a static graph
       * @param intData int data for each node of a static graph
       * @param boolData boolean data for each node of a static graph
       * @param objectData Object data for each node of a static graph
       * @returns a data provider view of the given arrays
       * @static
       */
      static createNodeDataProviderWithArrays(doubleData:number[],intData:number[],boolData:boolean[],objectData:Object[]):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider for edges that return the data provider values bound to their source nodes.
       * @param nodeData
       * @returns 
       * @static
       */
      static createSourceDataProvider(nodeData:yfiles.algorithms.IDataProvider):yfiles.algorithms.IDataProvider;
      /**
       * Returns a DataProvider for edges that return the data provider values bound to their target nodes.
       * @param nodeData
       * @returns 
       * @static
       */
      static createTargetDataProvider(nodeData:yfiles.algorithms.IDataProvider):yfiles.algorithms.IDataProvider;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides functionality to hide and unhide partitions of nodes and their adjacent edges of a graph temporarily for
     * algorithmic operations.
     * <p>
     * This class can be used to temporarily hide away certain elements of a graph and to unhide that parts at a later time
     * again. Instances of this class keep track of graph elements that were hidden from a graph in order to make them visible
     * again at a later time.
     * </p>
     * <p>
     * Note that this class should not be used to hide elements from a Graph2D for pure hiding purposes. Since this class will
     * by default prevent the graph instance from firing events, other code might cease to work correctly. Use this class for
     * short term removal of nodes and edges, only.
     * </p>
     * @class
     */
    export interface GraphPartitionManager extends Object{}
    export class GraphPartitionManager {
      /**
       * Instantiates a new GraphPartitionManager for the given graph.
       * <p>
       * All non-static hiding and unhiding methods will refer to the given graph.
       * </p>
       * @param graph
       * @param partitionId
       */
      constructor(graph:yfiles.algorithms.Graph,partitionId:yfiles.algorithms.IDataProvider);
      /**
       * Assures that only nodes are visible in the graph that are associated with the given partitionId.
       * @param partitionId the partitionId for the nodes that will be made visible
       */
      displayPartition(partitionId:Object):void;
      /**
       * Hides the given list of nodes from the graph.
       * <p>
       * The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param nl
       */
      hide(nl:yfiles.algorithms.NodeList):void;
      /**
       * Hides the given list of edges from the graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param el
       */
      hide(el:yfiles.algorithms.EdgeList):void;
      /**
       * Hides the given edge from the graph.
       * <p>
       * The hidden edge will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param e
       */
      hide(e:yfiles.algorithms.Edge):void;
      /**
       * Hides the given node and all it's adjacent edges from the graph.
       * <p>
       * The hidden elements will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param v
       */
      hide(v:yfiles.algorithms.Node):void;
      /**
       * Hides all nodes and edges from this graph.
       * <p>
       * The hidden elements will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideAll():void;
      /**
       * Hides the given edges from the graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param ec
       */
      hideEdgeCursor(ec:yfiles.algorithms.IEdgeCursor):void;
      /**
       * Hides all edges from this graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideEdges():void;
      /**
       * Hides the given elements from the graph.
       * <p>
       * The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param cursor
       */
      hideItemCursor(cursor:yfiles.algorithms.ICursor):void;
      /**
       * Hides multiple edges from the graph.
       * <p>
       * If there are multiple edges connecting two nodes then all but one (representative) of these edges will be hidden. The
       * hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideMultipleEdges():void;
      /**
       * Hides the given nodes from the graph.
       * <p>
       * The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param nc
       */
      hideNodeCursor(nc:yfiles.algorithms.INodeCursor):void;
      /**
       * Hides nodes and adjacent edges that have the given partitionId associated.
       * @param partitionId the id
       */
      hidePartition(partitionId:Object):void;
      /**
       * Hides all self-loop edges from this graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideSelfLoops():void;
      /**
       * Initializes internal data structures using the new DataProvider.
       * <p>
       * This method must also be called whenever the content of the given DataProvider changes.
       * </p>
       * @param partitionId the data provider that holds the partitionIds for all elements.
       */
      initPartitions(partitionId:yfiles.algorithms.IDataProvider):void;
      /**
       * Hides all self-loops and multiple edges from the graph.
       * <p>
       * The overall effect of this method is that the minimum number of edges are hidden from the graph such that it contains no
       * self-loops and no multiple edges anymore. The hidden edges will be stored so that they can be unhidden again at a later
       * time.
       * </p>
       */
      simplifyGraph():void;
      /**
       * This method will be called whenever the partition manager is requested to unhide the given node from the graph.
       * @param v
       * @protected
       */
      unhide(v:yfiles.algorithms.Node):void;
      /**
       * This method will be called whenever the partition manager is requested to unhide the given edge from the graph.
       * @param e
       * @protected
       */
      unhide(e:yfiles.algorithms.Edge):void;
      /**
       * Unhides all formerly hidden elements in the graph.
       */
      unhideAll():void;
      /**
       * Unhides all formerly hidden edges in the graph.
       */
      unhideEdges():void;
      /**
       * Unhides all formerly hidden nodes in the graph.
       * <p>
       * Note that this method does not unhide hidden edges.
       * </p>
       */
      unhideNodes():void;
      /**
       * Unhides nodes that have the given partitionId associated.
       * @param partitionId the id
       */
      unhidePartition(partitionId:Object):void;
      /**
       * Gets or sets whether or not this partition manager should fire graph events.
       * <p>
       * By default the partition manager does not fire graph events.
       * </p>
       * @type {boolean}
       */
      fireGraphEventsEnabled:boolean;
      /**
       * Gets the <code>Graph</code> for which this partition manager was created.
       * @type {yfiles.algorithms.Graph}
       */
      graph:yfiles.algorithms.Graph;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides functionality to hide and unhide nodes and edges of a graph temporarily for algorithmic operations.
     * <p>
     * This class can be used to temporarily hide away certain elements of a graph and to unhide that parts at a later time
     * again. Instances of this class keep track of graph elements that were hidden from a graph in order to make them visible
     * again at a later time.
     * </p>
     * <p>
     * Note that this class should not be used to hide elements from a Graph2D for pure hiding purposes. Since this class will
     * by default prevent the graph instance from firing events, other code might cease to work correctly. Use this class for
     * short term removal of nodes and edges, only.
     * </p>
     * @class
     */
    export interface LayoutGraphHider extends Object{}
    export class LayoutGraphHider {
      /**
       * Instantiates a new GraphHider for the given graph.
       * <p>
       * All non-static hiding and unhiding methods will refer to the given graph.
       * </p>
       * @param g
       */
      constructor(g:yfiles.algorithms.Graph);
      /**
       * The edges that are currently hidden
       * @returns a cursor of the currently hidden edges
       */
      getHiddenEdgeCursor():yfiles.algorithms.IEdgeCursor;
      /**
       * The nodes that are currently hidden
       * @returns a cursor of the currently hidden nodes
       */
      getHiddenNodeCursor():yfiles.algorithms.INodeCursor;
      /**
       * Hides the given list of nodes from the graph.
       * <p>
       * The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param nl
       */
      hide(nl:yfiles.algorithms.NodeList):void;
      /**
       * Hides the given list of edges from the graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param el
       */
      hide(el:yfiles.algorithms.EdgeList):void;
      /**
       * Hides the given edge from the graph.
       * <p>
       * The hidden edge will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param e
       */
      hide(e:yfiles.algorithms.Edge):void;
      /**
       * Hides the given node and all it's adjacent edges from the graph.
       * <p>
       * The hidden elements will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param v
       */
      hide(v:yfiles.algorithms.Node):void;
      /**
       * Hides all nodes and edges from this graph.
       * <p>
       * The hidden elements will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideAll():void;
      /**
       * Hides the given edges from the graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param ec
       */
      hideEdgeCursor(ec:yfiles.algorithms.IEdgeCursor):void;
      /**
       * Hides all edges from this graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideEdges():void;
      /**
       * Hides the given elements from the graph.
       * <p>
       * The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param cursor
       */
      hideItemCursor(cursor:yfiles.algorithms.ICursor):void;
      /**
       * Hides multiple edges from the graph.
       * <p>
       * If there are multiple edges connecting two nodes then all but one (representative) of these edges will be hidden. The
       * hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideMultipleEdges():void;
      /**
       * Hides the given nodes from the graph.
       * <p>
       * The hidden nodes and adjacent edges will be stored so that they can be unhidden again at a later time.
       * </p>
       * @param nc
       */
      hideNodeCursor(nc:yfiles.algorithms.INodeCursor):void;
      /**
       * Hides all self-loop edges from this graph.
       * <p>
       * The hidden edges will be stored so that they can be unhidden again at a later time.
       * </p>
       */
      hideSelfLoops():void;
      /**
       * Hides the subgraph induced by the given edges from the given graph.
       * <p>
       * The induced subgraph defined by the given edges consists of the given edges and all nodes that are solely connected to
       * the rest of the graph by the given edges.
       * </p>
       * @param graph
       * @param ec
       * @static
       */
      static hideSubgraph(graph:yfiles.algorithms.Graph,ec:yfiles.algorithms.IEdgeCursor):void;
      /**
       * Hides all self-loops and multiple edges from the graph.
       * <p>
       * The overall effect of this method is that the minimum number of edges are hidden from the graph such that it contains no
       * self-loops and no multiple edges anymore. The hidden edges will be stored so that they can be unhidden again at a later
       * time.
       * </p>
       */
      simplifyGraph():void;
      /**
       * This method will be called whenever the hider is requested to unhide the given node from the graph.
       * @param v
       * @protected
       */
      unhide(v:yfiles.algorithms.Node):void;
      /**
       * This method will be called whenever the hider is requested to unhide the given edge from the graph.
       * @param e
       * @protected
       */
      unhide(e:yfiles.algorithms.Edge):void;
      /**
       * Unhides all formerly hidden elements in the graph.
       */
      unhideAll():void;
      /**
       * Unhides the given edge.
       * <p>
       * Also updates {@link yfiles.algorithms.LayoutGraphHider#hiddenEdges}.
       * </p>
       * <p>
       * Note that unhiding an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed) causes
       * an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
       * </p>
       * @param e the edge that will be unhidden
       */
      unhideEdge(e:yfiles.algorithms.Edge):void;
      /**
       * Unhides the given edges.
       * <p>
       * Also updates {@link yfiles.algorithms.LayoutGraphHider#hiddenEdges}.
       * </p>
       * <p>
       * Note that unhiding an edge whose source/target is not in the graph (e.g., because it's currently hidden/removed) causes
       * an exception. Hence, in such cases, you first have to unhide/reinsert the corresponding endpoints.
       * </p>
       * @param edges the edges that will be unhidden
       */
      unhideEdges(edges:yfiles.algorithms.EdgeList):void;
      /**
       * Unhides all formerly hidden edges in the graph.
       */
      unhideEdges():void;
      /**
       * Unhides the given node and if requested its adjacent edges.
       * <p>
       * Also updates {@link yfiles.algorithms.LayoutGraphHider#hiddenNodes} and {@link yfiles.algorithms.LayoutGraphHider#hiddenEdges}.
       * </p>
       * @param v the node that will be unhidden
       * @param unhideAdjacentEdges whether of not to unhide previously hidden edges connected at <code>v</code> whose other end point is not hidden, i.e. it is part of
       * the graph.
       */
      unhideNode(v:yfiles.algorithms.Node,unhideAdjacentEdges:boolean):void;
      /**
       * Unhides the given nodes and if requested its adjacent edges.
       * <p>
       * Also updates {@link yfiles.algorithms.LayoutGraphHider#hiddenNodes} and {@link yfiles.algorithms.LayoutGraphHider#hiddenEdges}.
       * </p>
       * @param nodes the nodes that will be unhidden
       * @param unhideAdjacentEdges whether of not to unhide previously hidden edges connected at the given nodes whose other end point is not hidden, i.e.
       * it is part of the graph.
       */
      unhideNodes(nodes:yfiles.algorithms.NodeList,unhideAdjacentEdges:boolean):void;
      /**
       * Unhides all formerly hidden nodes in the graph.
       * <p>
       * Note that this method does not unhide hidden edges.
       * </p>
       */
      unhideNodes():void;
      /**
       * Unhides the subgraph induced by the given edges in the given graph.
       * <p>
       * The induced subgraph defined by the given edges consists of the given edges and all nodes that are either source or
       * target of at least one of the given edges.
       * </p>
       * <p>
       * Parts of the subgraph that are already contained in the given graph will not be unhidden and pose no problem to this
       * method.
       * </p>
       * @param graph
       * @param ec
       * @static
       */
      static unhideSubgraph(graph:yfiles.algorithms.Graph,ec:yfiles.algorithms.IEdgeCursor):void;
      /**
       * holds the list of the hidden edges in stack order
       * @protected
       * @type {yfiles.algorithms.EdgeList}
       */
      hiddenEdges:yfiles.algorithms.EdgeList;
      /**
       * holds the list of the hidden nodes in stack order
       * @protected
       * @type {yfiles.algorithms.NodeList}
       */
      hiddenNodes:yfiles.algorithms.NodeList;
      /**
       * Gets or sets whether or not this hider should fire graph events.
       * <p>
       * By default the hider does not fire graph events.
       * </p>
       * @type {boolean}
       */
      fireGraphEvents:boolean;
      /**
       * Gets the <code>Graph</code> for which this <code>GraphHider</code> was created.
       * @type {yfiles.algorithms.Graph}
       */
      graph:yfiles.algorithms.Graph;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides convenience and transformation services for Node- and EdgeMaps and other layout-related collection
     * types.
     * @class
     * @static
     */
    export interface Maps extends Object{}
    export class Maps {
      /**
       * Create a DataMap view of the given map.
       * <p>
       * Accessing basic value types is solved by storing the corresponding wrapper types Double, Integer and Boolean within the
       * given Map.
       * </p>
       * @param map
       * @returns 
       * @static
       */
      static createDataMap(map:yfiles.collections.IMap<Object,Object>):yfiles.algorithms.IDataMap;
      /**
       * Create an EdgeMap view of the given map.
       * <p>
       * Accessing basic value types is solved by storing the corresponding wrapper types Double, Integer and Boolean within the
       * given Map.
       * </p>
       * @param map
       * @returns 
       * @static
       */
      static createEdgeMap(map:yfiles.collections.IMap<Object,Object>):yfiles.algorithms.IEdgeMap;
      /**
       * Creates a DataMap that is based on hashing.
       * <p>
       * The preconditions specified in java.util.HashMap apply for the keys and values of this map.
       * </p>
       * @returns 
       * @static
       */
      static createHashedDataMap():yfiles.algorithms.IDataMap;
      /**
       * Creates an EdgeMap that is based on hashing.
       * <p>
       * The preconditions specified in java.util.HashMap apply for the keys and values of this map.
       * </p>
       * @returns 
       * @static
       */
      static createHashedEdgeMap():yfiles.algorithms.IEdgeMap;
      /**
       * Creates a NodeMap that is based on hashing.
       * <p>
       * The preconditions specified in java.util.HashMap apply for the keys and values of this map.
       * </p>
       * @returns 
       * @static
       */
      static createHashedNodeMap():yfiles.algorithms.INodeMap;
      /**
       * Factory method that creates a {@link yfiles.collections.IMap.<TKey,TValue>} that is a special hash map implementation mainly used by the layout algorithms.
       * Type parameter <code>K</code>.
       * Type parameter <code>V</code>.
       * @param [entries=null] The items to optionally populate the set with initially.
       * @returns A new instance.
       * @template K,V
       * @static
       */
      static createHashMap<K,V>(entries?:yfiles.collections.ICollection<yfiles.collections.MapEntry<K,V>>):yfiles.collections.IMap<K,V>;
      /**
       * Factory method that creates an {@link yfiles.collections.ICollection.<T>} implementation with set semantics that is a special hash map based implementation mainly
       * used by the layout algorithms.
       * Type parameter <code>T</code>.
       * @param [items=null] The items to optionally populate the set with initially.
       * @returns A new instance.
       * @template T
       * @static
       */
      static createHashSet<T>(items?:yfiles.collections.IEnumerable<T>):yfiles.collections.ICollection<T>;
      /**
       * Creates a resettable high performance map for values of type <code>double</code>.
       * <p>
       * A high performance map is a facade to another map which will allow to reset all bound values to a default value in
       * constant time.
       * </p>
       * @param backingProvider
       * @param backingAcceptor
       * @param defaultValue
       * @returns 
       * @static
       */
      static createHighPerformanceDoubleMap(backingProvider:yfiles.algorithms.IDataProvider,backingAcceptor:yfiles.algorithms.IDataAcceptor,defaultValue:number):yfiles.algorithms.INodeMap;
      /**
       * Creates a resettable high performance map for values of type <code>double</code>.
       * <p>
       * A high performance map is a facade to another map which will allow to reset all bound values to a default value in
       * constant time.
       * </p>
       * @param backingMap
       * @param defaultValue
       * @returns 
       * @static
       */
      static createHighPerformanceDoubleMap(backingMap:yfiles.algorithms.IDataMap,defaultValue:number):yfiles.algorithms.INodeMap;
      /**
       * Creates a resettable high performance map for values of type <code>int</code>.
       * <p>
       * A high performance map is a facade to another map which will allow to reset all bound values to a default value in
       * constant time.
       * </p>
       * @param backingProvider
       * @param backingAcceptor
       * @param defaultValue
       * @returns 
       * @static
       */
      static createHighPerformanceIntMap(backingProvider:yfiles.algorithms.IDataProvider,backingAcceptor:yfiles.algorithms.IDataAcceptor,defaultValue:number):yfiles.algorithms.INodeMap;
      /**
       * Creates a resettable high performance map for values of type <code>int</code>.
       * <p>
       * A high performance map is a facade to another map which will allow to reset all bound values to a default value in
       * constant time.
       * </p>
       * @param backingMap
       * @param defaultValue
       * @returns 
       * @static
       */
      static createHighPerformanceIntMap(backingMap:yfiles.algorithms.IDataMap,defaultValue:number):yfiles.algorithms.INodeMap;
      /**
       * Creates a resettable high performance map for values of type <code>Object</code>.
       * <p>
       * A high performance map is a facade to another map which will allow to reset all bound values to a default value in
       * constant time.
       * </p>
       * @param backingProvider
       * @param backingAcceptor
       * @param defaultValue
       * @returns 
       * @static
       */
      static createHighPerformanceMap(backingProvider:yfiles.algorithms.IDataProvider,backingAcceptor:yfiles.algorithms.IDataAcceptor,defaultValue:Object):yfiles.algorithms.INodeMap;
      /**
       * Creates a resettable high performance map for values of type <code>Object</code>.
       * <p>
       * A high performance map is a facade to another map which will allow to reset all bound values to a default value in
       * constant time.
       * </p>
       * @param backingMap
       * @param defaultValue
       * @returns 
       * @static
       */
      static createHighPerformanceMap(backingMap:yfiles.algorithms.IDataMap,defaultValue:Object):yfiles.algorithms.INodeMap;
      /**
       * Returns a EdgeMap view of an Object array defined for edges.
       * <p>
       * The Object value <code>data[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>get(edge)</code> and
       * <code>set(edge,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed edges must not change during the use of this EdgeMap.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a EdgeMap view of the given array
       * @static
       */
      static createIndexEdgeMap(data:Object[]):yfiles.algorithms.IEdgeMap;
      /**
       * Returns a EdgeMap view of a boolean array defined for edges.
       * <p>
       * The boolean value <code>data[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>getBool(edge)</code> and
       * <code>setBool(edge,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed edges must not change during the use of this EdgeMap.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a EdgeMap view of the given array
       * @static
       */
      static createIndexEdgeMapForBoolean(data:boolean[]):yfiles.algorithms.IEdgeMap;
      /**
       * Returns a EdgeMap view of an int array defined for edges.
       * <p>
       * The int value <code>data[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>getInt(edge)</code> and
       * <code>setInt(edge,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed edges must not change during the use of this EdgeMap.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a EdgeMap view of the given array
       * @static
       */
      static createIndexEdgeMapForInt(data:number[]):yfiles.algorithms.IEdgeMap;
      /**
       * Returns a EdgeMap view of a double array defined for edges.
       * <p>
       * The double value <code>data[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>getDouble(edge)</code> and
       * <code>setDouble(edge,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed edges must not change during the use of this EdgeMap.
       * </p>
       * @param data array data for each edge of a static graph
       * @returns a EdgeMap view of the given array
       * @static
       */
      static createIndexEdgeMapForNumber(data:number[]):yfiles.algorithms.IEdgeMap;
      /**
       * Returns a EdgeMap view of a double, int, boolean and Object array defined for edges.
       * <p>
       * The double value <code>doubleData[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>getDouble(edge)</code> and
       * <code>setDouble(edge,value)</code>.
       * </p>
       * <p>
       * The int value <code>intData[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>getInt(edge)</code> and
       * <code>setInt(edge,value)</code>.
       * </p>
       * <p>
       * The boolean value <code>boolData[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>getBool(edge)</code> and
       * <code>setBool(edge,value)</code>.
       * </p>
       * <p>
       * The Object value <code>objectData[edge.index()]</code> will be accessed by the EdgeMap upon the method calls <code>get(edge)</code> and
       * <code>set(edge,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed edges must not change during the use of this EdgeMap.
       * </p>
       * @param doubleData double data for each edge of a static graph
       * @param intData int data for each edge of a static graph
       * @param boolData boolean data for each edge of a static graph
       * @param objectData Object data for each edge of a static graph
       * @returns a EdgeMap view of the given arrays
       * @static
       */
      static createIndexEdgeMapFromArrays(doubleData:number[],intData:number[],boolData:boolean[],objectData:Object[]):yfiles.algorithms.IEdgeMap;
      /**
       * Returns a NodeMap view of an Object array defined for nodes.
       * <p>
       * The Object value <code>data[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>get(node)</code> and <code>set(node,value)</code>.
       * <b>Warning!</b> The indices of the accessed nodes must not change during the use of this NodeMap.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a NodeMap view of the given array
       * @static
       */
      static createIndexNodeMap(data:Object[]):yfiles.algorithms.INodeMap;
      /**
       * Returns a NodeMap view of a boolean array defined for nodes.
       * <p>
       * The boolean value <code>data[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>getBool(node)</code> and
       * <code>setBool(node,value)</code>.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a NodeMap view of the given array
       * @static
       */
      static createIndexNodeMapForBoolean(data:boolean[]):yfiles.algorithms.INodeMap;
      /**
       * Returns a NodeMap view of an int array defined for nodes.
       * <p>
       * The int value <code>data[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>getInt(node)</code> and
       * <code>setInt(node,value)</code>. <b>Warning!</b> The indices of the accessed nodes must not change during the use of this NodeMap.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a NodeMap view of the given array
       * @static
       */
      static createIndexNodeMapForInt(data:number[]):yfiles.algorithms.INodeMap;
      /**
       * Returns a NodeMap view of a double array defined for nodes.
       * <p>
       * The double value <code>data[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>getDouble(node)</code> and
       * <code>setDouble(node,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed nodes must not change during the use of this NodeMap.
       * </p>
       * @param data array data for each node of a static graph
       * @returns a NodeMap view of the given array
       * @static
       */
      static createIndexNodeMapForNumber(data:number[]):yfiles.algorithms.INodeMap;
      /**
       * Returns a NodeMap view of a double, int, boolean and Object array defined for nodes.
       * <p>
       * The double value <code>doubleData[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>getDouble(node)</code> and
       * <code>setDouble(node,value)</code>.
       * </p>
       * <p>
       * The int value <code>intData[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>getInt(node)</code> and
       * <code>setInt(node,value)</code>.
       * </p>
       * <p>
       * The boolean value <code>boolData[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>getBool(node)</code> and
       * <code>setBool(node,value)</code>.
       * </p>
       * <p>
       * The Object value <code>objectData[node.index()]</code> will be accessed by the NodeMap upon the method calls <code>get(node)</code> and
       * <code>set(node,value)</code>.
       * </p>
       * <p>
       * <b>Warning!</b> The indices of the accessed nodes must not change during the use of this NodeMap.
       * </p>
       * @param doubleData double data for each node of a static graph
       * @param intData int data for each node of a static graph
       * @param boolData boolean data for each node of a static graph
       * @param objectData Object data for each node of a static graph
       * @returns a NodeMap view of the given arrays
       * @static
       */
      static createIndexNodeMapFromArrays(doubleData:number[],intData:number[],boolData:boolean[],objectData:Object[]):yfiles.algorithms.INodeMap;
      /**
       * Create a NodeMap view of the given map.
       * <p>
       * Accessing basic value types is solved by storing the corresponding wrapper types Double, Integer and Boolean within the
       * given Map.
       * </p>
       * @param map
       * @returns 
       * @static
       */
      static createNodeMap(map:yfiles.collections.IMap<Object,Object>):yfiles.algorithms.INodeMap;
      static $class:yfiles.lang.Class;
    }
    /**
     * An implementation of a doubly linked list that provides direct access to the cells that store the elements.
     * <p>
     * The cells are represented by class {@link yfiles.algorithms.ListCell}.
     * </p>
     * <p>
     * This class supports fast access and removal operations, specifically, it is possible to remove an element in constant
     * time (i.e. O(1)) given a reference to its list cell.
     * </p>
     * <p>
     * Class YList supports iteration over the elements either by using the list cells directly (methods {@link yfiles.algorithms.YList#firstCell}/{@link yfiles.algorithms.YList#lastCell} together with
     * {@link yfiles.algorithms.YList#succCell}/{@link yfiles.algorithms.YList#predCell}, respectively) or by means of a cursor
     * ({@link yfiles.algorithms.YList#cursor}).
     * </p>
     * <p>
     * Furthermore, YList offers its own {@link yfiles.algorithms.YList#sort} method. Note that this class also provides all relevant
     * methods to use the list like a stack data type.
     * </p>
     * <p>
     * This implementation permits <code>null</code> as values. It implements the {@link yfiles.collections.IList.<T>} interface but does not support the {@link yfiles.algorithms.YList#subList}
     * method. The implementation of this method will throw an {@link yfiles.lang.Exception} if invoked. The {@link yfiles.algorithms.YList#cursor}
     * returned by instances of this class is not fail fast.
     * </p>
     * @class
     * @implements {yfiles.collections.IList.<Object>}
     */
    export interface YList extends Object,yfiles.collections.IList<Object>{}
    export class YList {
      /**
       * Creates a list that is initialized with the elements provided by the given array of objects.
       * @param a
       */
      constructor(a:Object[]);
      /**
       * Creates a list that is initialized with those elements from the given YCursor object for which the given data provider
       * returns <code>true</code> upon calling its {@link yfiles.algorithms.IDataProvider#getBoolean getBool} method.
       * @param c A cursor providing objects that should be added to this list.
       * @param predicate A data provider that acts as a inclusion predicate for each object accessible by the given cursor.
       */
      constructor(c:yfiles.algorithms.ICursor,predicate:yfiles.algorithms.IDataProvider);
      /**
       * Creates a list that is initialized with the elements provided by the given Collection object.
       * @param c
       */
      constructor(c:yfiles.collections.IEnumerable<Object>);
      /**
       * Creates a list that is initialized with the elements provided by the given YCursor object.
       * @param c
       */
      constructor(c:yfiles.algorithms.ICursor);
      /**
       * Creates an empty doubly linked list.
       */
      constructor();
      /**
       * Appends all elements provided by the given cursor to this list.
       * <p>
       * The cursor will be moved from its given position to the end.
       * </p>
       * <p>
       * Be aware that a statement like <code>aList.append(aList.cursor())</code> results in an infinite recursion.
       * </p>
       * @param c
       */
      addAll(c:yfiles.algorithms.ICursor):void;
      /**
       * Appends all elements provided by the given collection to this list.
       * @param collection
       * @returns Whether there have been elements appended.
       */
      addAll(collection:yfiles.collections.ICollection<Object>):boolean;
      /**
       * 
       * @param index
       * @param c
       * @returns 
       */
      addAll(index:number,c:yfiles.collections.ICollection<Object>):boolean;
      /**
       * Inserts the given object at the head of this list.
       * @param o
       * @returns The newly created ListCell object that stores the given object.
       */
      addFirst(o:Object):yfiles.algorithms.ListCell;
      /**
       * Adds a formerly removed ListCell object at the head of this list.
       * <p>
       * <b>Attention:</b> If the ListCell object is still part of any list, then that list will be corrupted afterwards.
       * </p>
       * @param cell A list cell which is not part of any list.
       */
      addFirstCell(cell:yfiles.algorithms.ListCell):void;
      /**
       * Inserts the given object at the tail of this list.
       * @param o
       * @returns The newly created ListCell object that stores the given object.
       */
      addLast(o:Object):yfiles.algorithms.ListCell;
      /**
       * Adds a formerly removed ListCell object at the tail of this list.
       * <p>
       * <b>Attention:</b> If the ListCell object is still part of any list, then that list will be corrupted afterwards.
       * </p>
       * @param cell A list cell which is not part of any list.
       */
      addLastCell(cell:yfiles.algorithms.ListCell):void;
      /**
       * Removes all elements from this list.
       */
      clear():void;
      /**
       * Whether or not this list contains all the elements in the given collection.
       * <p>
       * Equality of elements is defined by the {@link Object#equals} method.
       * </p>
       * @param collection
       * @returns 
       */
      containsAll(collection:yfiles.collections.ICollection<Object>):boolean;
      /**
       * Copies the elements of this collection to an array, starting at the specified array index.
       * @param array The one-dimensional array that is the destination of the elements copied from this collection.
       * @param arrayIndex The zero-based index in <code>array</code> at which copying begins.
       */
      copyTo(array:Object[],arrayIndex:number):void;
      /**
       * Returns a cursor for this list.
       * <p>
       * All cursor operations are supported. This cursor implementation is not fail-fast and continues to work if this list is
       * modified during the traversal as long as the current ListCell the cursor points at is this in this list or has been
       * removed from this list but has not been added to another instance since then.
       * </p>
       * @returns 
       */
      cursor():yfiles.algorithms.ICursor;
      /**
       * Returns the cyclic predecessor cell of the given list cell.
       * <p>
       * The last cell is returned as the cyclic predecessor of the first list cell.
       * </p>
       * @param c
       * @returns 
       */
      cyclicPred(c:yfiles.algorithms.ListCell):yfiles.algorithms.ListCell;
      /**
       * Returns the cyclic successor cell of the given list cell.
       * <p>
       * The first cell is returned as the cyclic successor of the last list cell.
       * </p>
       * @param c
       * @returns 
       */
      cyclicSucc(c:yfiles.algorithms.ListCell):yfiles.algorithms.ListCell;
      /**
       * Returns the i-th element of this list.
       * @param i
       * @returns 
       */
      elementAt(i:number):Object;
      /**
       * Returns the {@link yfiles.algorithms.ListCell} where object <code>o</code> is stored.
       * <p>
       * This operation returns <code>null</code>, if no such cell exists. Equality of elements is defined by the {@link Object#equals} method. The first
       * element in the list that matches that criteria is returned.
       * </p>
       * @param o
       * @returns the ListCell that contains the element or <code>null</code> if no such ListCell was found
       */
      findCell(o:Object):yfiles.algorithms.ListCell;
      /**
       * Gets the element at the specified index.
       * @param index The zero-based index of the element to get or set.
       * @returns The element at the specified index.
       * @see yfiles.algorithms.YList#set
       */
      get(index:number):Object;
      /**
       * Gets the cell at the given index.
       * @param index the zero-based index of the cell in this list.
       * @returns The cell.
       * @throws {Stubs.Exceptions.IndexOutOfRangeError} if the index is negative or greater or equal than the {@link yfiles.algorithms.YList#size}
       */
      getCell(index:number):yfiles.algorithms.ListCell;
      /**
       * Returns an enumerator that iterates through this collection.
       * @returns An {@link } that can be used to iterate through this collection.
       */
      getEnumerator():yfiles.collections.IEnumerator<Object>;
      /**
       * Returns the element stored in the given list cell.
       * @param c
       * @returns 
       */
      getInfo(c:yfiles.algorithms.ListCell):Object;
      /**
       * Whether or not this list contains the given element.
       * <p>
       * Equality of elements is defined by the {@link Object#equals} method.
       * </p>
       * @param o
       * @returns 
       */
      includes(o:Object):boolean;
      /**
       * Returns the zero-based index of the given element in this list.
       * <p>
       * If the given element is not in the list, -1 is returned.
       * </p>
       * @param obj
       * @returns 
       */
      indexOf(obj:Object):number;
      /**
       * 
       * @param index
       * @param element
       */
      insert(index:number,element:Object):void;
      /**
       * Inserts the given object into this list with respect to a given reference list cell.
       * <p>
       * The (newly created) list cell that stores the object is inserted right after the reference list cell <code>refCell</code>.
       * </p>
       * <p>
       * If <code>refCell == null</code>, the given object is inserted at the head of the list.
       * </p>
       * @param o The object to be inserted.
       * @param refCell The list cell used to reference the position.
       * @returns The newly created ListCell object that stores object <code>o</code>.
       */
      insertAfter(o:Object,refCell:yfiles.algorithms.ListCell):yfiles.algorithms.ListCell;
      /**
       * Inserts the given object into this list with respect to a given reference list cell.
       * <p>
       * The (newly created) list cell that stores the object is inserted right before the reference list cell <code>refCell</code>.
       * </p>
       * <p>
       * If <code>refCell == null</code>, the given object is appended to the list.
       * </p>
       * @param o The object to be inserted.
       * @param refCell The list cell used to reference the position.
       * @returns The newly created ListCell object that stores object <code>o</code>.
       */
      insertBefore(o:Object,refCell:yfiles.algorithms.ListCell):yfiles.algorithms.ListCell;
      /**
       * Inserts a formerly removed ListCell object into this list with respect to a given reference list cell.
       * <p>
       * The ListCell object is inserted right after the reference list cell <code>refCell</code>.
       * </p>
       * <p>
       * <b>Attention:</b> If the ListCell object is still part of any list, then that list will be corrupted afterwards.
       * </p>
       * @param cellToInsert A list cell which is not part of any list.
       * @param refCell The list cell used to reference the position.
       */
      insertCellAfter(cellToInsert:yfiles.algorithms.ListCell,refCell:yfiles.algorithms.ListCell):void;
      /**
       * Inserts a formerly removed ListCell object into this list with respect to a given reference list cell.
       * <p>
       * The ListCell object is inserted right before the reference list cell <code>refCell</code>.
       * </p>
       * <p>
       * <b>Attention:</b> If the ListCell object is still part of any list, then that list will be corrupted afterwards.
       * </p>
       * @param cellToInsert A list cell which is not part of any list.
       * @param refCell The list cell used to reference the position.
       */
      insertCellBefore(cellToInsert:yfiles.algorithms.ListCell,refCell:yfiles.algorithms.ListCell):void;
      /**
       * Checks whether this list contains elements.
       * @returns 
       */
      isEmpty():boolean;
      /**
       * 
       * @param o
       * @returns 
       */
      lastIndexOf(o:Object):number;
      /**
       * Equivalent to {@link yfiles.algorithms.YList#firstObject}.
       * @returns 
       */
      peek():Object;
      /**
       * Removes the first element from this list and returns it.
       * @returns 
       */
      pop():Object;
      /**
       * Removes the last element from this list and returns it.
       * @returns 
       */
      popLast():Object;
      /**
       * Returns the predecessor cell of the given list cell.
       * @param c
       * @returns 
       */
      predCell(c:yfiles.algorithms.ListCell):yfiles.algorithms.ListCell;
      /**
       * Equivalent to {@link yfiles.algorithms.YList#addFirst}.
       * @param o
       * @returns 
       */
      push(o:Object):yfiles.algorithms.ListCell;
      /**
       * Removes the given object from this list.
       * <p>
       * Only the first element for which equality to <code>o</code> holds gets removed.
       * </p>
       * @param o
       * @returns 
       */
      remove(o:Object):boolean;
      /**
       * Removes the given collection of objects from this list.
       * @param collection
       * @returns Whether there have been elements removed.
       */
      removeAll(collection:yfiles.collections.ICollection<Object>):boolean;
      /**
       * 
       * @param index
       */
      removeAt(index:number):void;
      /**
       * Removes the element pointed to by the given YCursor object.
       * @param c
       * @returns The removed element.
       */
      removeAtCursor(c:yfiles.algorithms.ICursor):Object;
      /**
       * Removes the given list cell, and hence the element stored in it, from this list.
       * @param c
       * @returns The element that is stored in the removed cell.
       */
      removeCell(c:yfiles.algorithms.ListCell):Object;
      /**
       * Retains only those elements in this list which are contained in the given collection.
       * @param collection
       * @returns Whether there have been elements removed.
       */
      retainAll(collection:yfiles.collections.ICollection<Object>):boolean;
      /**
       * Reverses the sequence of elements in this list.
       */
      reverse():void;
      /**
       * Sets the element at the specified index.
       * @param index The zero-based index of the element to get or set.
       * @param value The element at the specified index.
       * @see yfiles.algorithms.YList#get
       */
      set(index:number,value:Object):void;
      /**
       * Updates the element stored in the given list cell with the given object.
       * @param c
       * @param value
       */
      setInfo(c:yfiles.algorithms.ListCell,value:Object):void;
      /**
       * Sorts the elements in this list according to the given comparator.
       * <p>
       * NOTE: The elements will be assigned to different list cells by this method.
       * </p>
       * @param comparer
       */
      sort(comparer:yfiles.collections.IComparer<Object>):void;
      /**
       * Sorts the elements in this list into ascending order, according to their natural ordering.
       * <p>
       * All elements must implement the {@link yfiles.lang.IComparable} interface. Furthermore, all elements in this list must be
       * mutually comparable (that is, <code>e1.compareTo(e2)</code> must not throw a ClassCastException for any elements <code>e1</code> and <code>e2</code> in this
       * list).
       * </p>
       * <p>
       * NOTE: The elements will be assigned to different list cells by this method.
       * </p>
       */
      sort():void;
      /**
       * Transfers the contents of the given list to the end of this list.
       * <p>
       * The given list will be empty after this operation.
       * </p>
       * <p>
       * Note that this operation transfers the list cells of the given list to this list. No new list cells are created by this
       * operation.
       * </p>
       * @param list
       */
      splice(list:yfiles.algorithms.YList):void;
      /**
       * 
       * @param fromIndex
       * @param toIndex
       * @returns 
       */
      subList(fromIndex:number,toIndex:number):yfiles.collections.IList<Object>;
      /**
       * Returns the successor cell of the given list cell.
       * @param c
       * @returns 
       */
      succCell(c:yfiles.algorithms.ListCell):yfiles.algorithms.ListCell;
      /**
       * Returns an array representation of this list.
       * @returns 
       */
      toArray():Object[];
      /**
       * Gets the number of elements in this list.
       * @type {number}
       */
      size:number;
      /**
       * Gets the first element of this list.
       * @type {Object}
       */
      firstObject:Object;
      /**
       * Gets the last element of this list.
       * @type {Object}
       */
      lastObject:Object;
      /**
       * Gets the first cell of this list.
       * @type {yfiles.algorithms.ListCell}
       */
      firstCell:yfiles.algorithms.ListCell;
      /**
       * Gets the last cell of this list.
       * @type {yfiles.algorithms.ListCell}
       */
      lastCell:yfiles.algorithms.ListCell;
      static $class:yfiles.lang.Class;
    }
  }export namespace layout{
    /**
     * {@link yfiles.layout.BendConverter} replaces edge bends with dummy nodes for the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}.
     * <p>
     * <b>Concept</b> Replacing the bends works in three steps:
     * </p>
     * <ol>
     * <li>Replacing the bends and the edge segments between them with dummy nodes and edges</li>
     * <li>Invoking the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm} on the altered graph</li>
     * <li>Recreating the bends and removing all dummy elements</li>
     * </ol>
     * <p>
     * When the dummy elements have been inserted, edge information about {@link yfiles.layout.BendConverter#adoptPortConstraints port constraints}, {@link yfiles.layout.BendConverter#adoptEdgeGroups edge groups} and {@link yfiles.layout.BendConverter#adoptAffectedEdges selection state} can
     * be adopted from the original edges. That way, the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm} can access the mentioned information.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface BendConverter extends Object,yfiles.layout.ILayoutStage{}
    export class BendConverter {
      /**
       * Creates a new {@link yfiles.layout.BendConverter} instance with the given size as the width and height for the dummy nodes.
       * @param size the size for the dummy nodes
       * @throws {Stubs.Exceptions.ArgumentError} if the given size is negative
       */
      constructor(size:number);
      /**
       * Creates a new instance of {@link yfiles.layout.BendConverter} with the given {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}.
       * @param core the core layout algorithm
       */
      constructor(core:yfiles.layout.ILayoutAlgorithm);
      /**
       * Creates a new {@link yfiles.layout.BendConverter} instance with default settings.
       */
      constructor();
      /**
       * Adds information to newly created dummy nodes.
       * <p>
       * This is a callback method that will be invoked after new elements for the given edge have been added to the given graph
       * by {@link yfiles.layout.BendConverter#prepare}. At this point, the original edge is still in the graph.
       * </p>
       * <p>
       * The current implementation does nothing.
       * </p>
       * @param graph the input graph
       * @param edge the edge whose bends were replaced by the given dummy nodes
       * @param dummyNodes the dummy nodes representing the bends of the edge
       * @protected
       */
      addedPathForEdge(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge,dummyNodes:yfiles.algorithms.NodeList):void;
      /**
       * Replaces all bends in the current {@link yfiles.layout.BendConverter#DEFAULT_AFFECTED_EDGES_DP_KEY scope} with dummy nodes before invoking the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}.
       * <p>
       * After the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm} has finished, the edges between the dummy nodes are reconnected and the dummy nodes are
       * replaced by bends again.
       * </p>
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Prepares the graph before calling the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}.
       * <p>
       * The bends of the edges belonging to the current scope are replaced by dummy nodes and the segments between those dummy
       * nodes are replaced by dummy edges.
       * </p>
       * <p>
       * Edge-specific information like {@link yfiles.layout.PortConstraint}s, edge groups or selection state can be applied to the dummy
       * edges to keep this information for the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.BendConverter#applyLayout} before invoking the
       * {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}. It may be overridden to extend the edge-specific information that should be adopted by the dummy
       * edges.
       * </p>
       * @param graph the input graph
       * @see yfiles.layout.BendConverter#restore
       * @see yfiles.layout.BendConverter#adoptAffectedEdges
       * @see yfiles.layout.BendConverter#adoptPortConstraints
       * @see yfiles.layout.BendConverter#adoptEdgeGroups
       * @protected
       */
      prepare(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Restores the structure of the graph after the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm} has finished.
       * <p>
       * This method is called by {@link yfiles.layout.BendConverter#applyLayout} after invoking the {@link yfiles.layout.BendConverter#coreLayout core layout algorithm}. It may be
       * overridden to clean up all additional information added in {@link yfiles.layout.BendConverter#prepare}.
       * </p>
       * @param graph the graph arranged by the {@link #coreLayout core layout algorithm}
       * @see yfiles.layout.BendConverter#prepare
       * @see yfiles.layout.BendConverter#adoptAffectedEdges
       * @see yfiles.layout.BendConverter#adoptPortConstraints
       * @see yfiles.layout.BendConverter#adoptEdgeGroups
       * @protected
       */
      restore(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Data provider key for specifying the set of edges for which this stage will replace bends
       * <p>
       * Only the bends of the marked edges will be replaced with dummy nodes.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DEFAULT_AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the core layout algorithm that is wrapped by this {@link yfiles.layout.ILayoutStage}.
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets whether or not edge grouping information of the original edge should be adopted by its replacement edges.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {boolean}
       */
      adoptEdgeGroups:boolean;
      /**
       * Gets or sets whether or not the associated
       * {@link yfiles.layout.PortConstraint}s of the original edge should be adopted by its replacement edges.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {boolean}
       */
      adoptPortConstraints:boolean;
      /**
       * Gets or sets whether or not the selection state of the original edge should be adopted by its replacement edges.
       * <p>
       * The selection state is looked up in a {@link yfiles.algorithms.IDataProvider} registered with the key returned by {@link yfiles.layout.BendConverter#affectedEdgesDpKey}.
       * </p>
       * @see yfiles.layout.BendConverter#affectedEdgesDpKey
       * @type {boolean}
       */
      adoptAffectedEdges:boolean;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} which identifies edges as <em>selected</em>.
       * <p>
       * If {@link yfiles.layout.BendConverter#adoptAffectedEdges the selection is conserved}, the selection state described by the {@link yfiles.algorithms.IDataProvider} registered with this
       * key is applied to the according dummy edges.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @see yfiles.layout.BendConverter#adoptAffectedEdges
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.BufferedLayout} is a hull algorithm that invokes its {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} on a copy of the input graph.
     * <p>
     * After the layout process has finished, the calculated layout will be written back to the original graph.
     * </p>
     * <p>
     * This class also provides the possibility to perform arbitrary layout algorithms merely on a graph interface plus
     * associated graph layout. This comes in handy if an application has its own graph and only provides a graph interface
     * adapter to the <em>yFiles</em> graph layout machinery.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface BufferedLayout extends yfiles.layout.LayoutStageBase{}
    export class BufferedLayout {
      /**
       * Creates a new {@link yfiles.layout.BufferedLayout} instance using the given {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] the core layout routine
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.ComponentLayout} arranges the connected components of a graph.
     * <p>
     * <b>Layout Style</b> The components can be arranged using different {@link yfiles.layout.ComponentLayout#style styles}. All styles except {@link yfiles.layout.ComponentArrangementStyles#NONE}
     * place the components without overlaps. {@graph {"ann":{"s":[30,30],"d":1},"n":[[552,150],[477,75],[75,150],[150,375],[150,225],[150,300],[351,150],[150,75],[0,150],[552,75],[477,0],[426,75],[351,225],[552,225],[150,150],[351,0],[276,75],[351,75],[75,300],[426,150],[225,150],[75,75],[225,300],[150,0],[477,351],[477,276],[553,351],[553,276],[150,450],[327,402],[276,402],[378,402],[429,402]],"e":[[0,1,-15,0,0,15,[492,165]],[3,5,0,-15,0,15],[5,4,0,-15,0,15],[8,2,15,0,-15,0],[0,9,0,-15,0,15],[9,1,-15,0,15,0],[1,10,0,-15,0,15],[12,6,0,-15,0,15],[0,13,0,15,0,-15],[4,14,0,-15,0,15],[14,2,-15,0,15,0],[14,7,0,-15,0,15],[16,15,0,-15,-15,0,[291,15]],[15,17,0,15,0,-15],[17,11,15,0,-15,0],[17,6,0,15,0,-15],[17,16,-15,0,15,0],[18,5,15,0,-15,0],[6,19,15,0,-15,0],[19,11,0,-15,0,15],[20,14,-15,0,15,0],[2,21,0,-15,0,15],[22,5,-15,0,15,0],[22,20,0,-15,0,15],[23,7,0,15,0,-15],[24,25,0,-15,0,15],[24,26],[26,27,0,-15,0,15],[25,27],[3,28,0,15,0,-15],[30,29],[31,32]],"vp":[0.0,0.0,583.0,480.0]}}
     * </p>
     * <p>
     * Layout with orthogonal components using {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS_COMPACT}
     * </p>
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * {@link yfiles.layout.ComponentLayout} is a {@link yfiles.layout.ILayoutStage} that can wrap another {@link yfiles.layout.ILayoutAlgorithm layout algorithm}. That way, it
     * allows handling disconnected graphs for the wrapped {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * </p>
     * <p>
     * The following steps outline the concept of {@link yfiles.layout.ComponentLayout}:
     * </p>
     * <ol>
     * <li>Determine the connected components of the graph</li>
     * <li>Hide all graph components</li>
     * <li>Apply the following steps to each component separately</li>
     * <li>Unhide all graph components</li>
     * <li>{@link yfiles.layout.ComponentLayout#componentArrangement Optionally arrange} the components</li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * To arrange the subgraphs of the components, {@link yfiles.layout.ComponentLayout} uses the specified {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. If
     * there is no {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} specified, it will keep the locations in the subgraph and arrange the components as
     * they are.
     * </p>
     * <p>
     * Hierarchically grouped graphs are handled in a special way. The contents of a group node will always belong to the same
     * component as the group node itself. To change that behavior {@link yfiles.layout.ComponentLayout#considerGrouping grouping} can be disabled.
     * </p>
     * <p>
     * By default, the components consist of the connected nodes in a graph. To choose custom subgraphs to form components,
     * register a {@link yfiles.algorithms.IDataProvider} with {@link yfiles.layout.ComponentLayout#COMPONENT_ID_DP_KEY} and assign component
     * IDs to the nodes.
     * </p>
     * @see yfiles.layout.MultiStageLayout
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface ComponentLayout extends yfiles.layout.LayoutStageBase{}
    export class ComponentLayout {
      /**
       * Creates a new {@link yfiles.layout.ComponentLayout} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algortihm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Produces a component graph layout.
       * <p>
       * This method is called by {@link yfiles.layout.ComponentLayout#applyLayout} in case {@link yfiles.layout.ComponentLayout#componentArrangement component arrangement} is enabled. It moves
       * the graph's components such that their bounding boxes do not overlap. Subclasses may want to override this method to
       * introduce custom component arrangement styles.
       * </p>
       * @param graph the input graph
       * @param nodes the nodes of the components; the i-th list contains the nodes of the i-th component
       * @param edges the edges of the components; the i-th list contains the edges of the i-th component
       * @param bbox the bounds of the components; the i-th rectangle describes the bounding box of the i-th component
       * @param boxes the extended bounds of the components; the i-th rectangle describes the bounding box of the i-th component extended by
       * the spacing between components. The method arranges these boxes in such a way that they do not overlap. Then, the i-th
       * graph component must be placed inside the i-th box
       * @see yfiles.layout.ComponentLayout#arrangeFields
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleRows
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleMultiRows
       * @protected
       */
      arrangeComponents(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.NodeList[],edges:yfiles.algorithms.EdgeList[],bbox:yfiles.algorithms.YRectangle[],boxes:yfiles.algorithms.Rectangle2D[]):void;
      /**
       * Arranges the bounding boxes of the components.
       * <p>
       * This method is called by {@link yfiles.layout.ComponentLayout#arrangeComponents} if the {@link yfiles.layout.ComponentLayout#style style} is set to
       * {@link yfiles.layout.ComponentArrangementStyles#PACKED_RECTANGLE},
       * {@link yfiles.layout.ComponentArrangementStyles#PACKED_COMPACT_RECTANGLE}, {@link yfiles.layout.ComponentArrangementStyles#PACKED_CIRCLE}
       * or
       * {@link yfiles.layout.ComponentArrangementStyles#PACKED_COMPACT_CIRCLE}. It may be overridden to adjust the component arrangement
       * strategy of the mentioned styles.
       * </p>
       * @param graph the input graph
       * @param nodes the nodes of the components; the i-th list contains the nodes of the i-th component
       * @param edges the edges of the components; the i-th list contains the edges of the i-th component
       * @param bbox the bounds of the components; the i-th rectangle describes the bounding box of the i-th component
       * @param boxes the extended bounds of the components; the i-th rectangle describes the bounding box of the i-th component
       * @param circular <code>true</code> if the arrangement should be circular, <code>false</code> if it should be rectangular
       * @param fill <code>true</code> if it is allowed to place components in empty spaces <em>inside other
       * components</em>, <code>false</code> otherwise
       * @param fromSketch <code>true</code> if the initial coordinates should be considered, <code>false</code> otherwise
       * @protected
       */
      arrangeFields(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.NodeList[],edges:yfiles.algorithms.EdgeList[],bbox:yfiles.algorithms.YRectangle[],boxes:yfiles.algorithms.Rectangle2D[],circular:boolean,fill:boolean,fromSketch:boolean):void;
      /**
       * Calculates the bounding box of a graph component including {@link yfiles.layout.NodeHalo}s.
       * <p>
       * This method will be invoked for each component of the graph. Depending on the state of property
       * {@link yfiles.layout.ComponentLayout#considerLabels}, the returned bounding box will also include node and edge labels. It may be overridden to extend the bounds to reserve
       * space for other elements.
       * </p>
       * @param graph the subgraph containing the nodes and edges of a component
       * @returns the bounding box of the component
       * @see yfiles.layout.ComponentLayout#considerLabels
       * @protected
       */
      calculateBounds(graph:yfiles.layout.LayoutGraph):yfiles.algorithms.Rectangle2D;
      /**
       * Determines which nodes belong to the same graph component.
       * <p>
       * This implementation uses the graph connectivity to sort the nodes into different components. Nodes that are not
       * connected by a path will be in separate components.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.ComponentLayout#applyLayout}. It may be overridden to choose another approach to
       * find components that will be passed to the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. However, most of the layout algorithms cannot handle disconnected graphs. Also, edges between
       * custom components will be ignored.
       * </p>
       * @param graph the input graph
       * @param compNumber a map that will be filled with the zero-based index of the component to which the node belongs
       * @returns the number of connected components of this graph
       * @see yfiles.algorithms.GraphConnectivity#connectedComponents
       * @protected
       */
      findGraphComponents(graph:yfiles.layout.LayoutGraph,compNumber:yfiles.algorithms.INodeMap):number;
      /**
       * Moves the subgraph containing the given nodes and edges to the specified origin.
       * <p>
       * This method is called by {@link yfiles.layout.ComponentLayout#arrangeComponents} and {@link yfiles.layout.ComponentLayout#arrangeFields}
       * to move the components to overlap-free positions.
       * </p>
       * @param graph the input graph
       * @param nodes the nodes in the moving subgraph
       * @param edges the edges in the moving subgraph
       * @param origin the new origin of the graph
       * @param rectangle the current bounds of the subgraph
       * @protected
       */
      setOrigin(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.NodeList,edges:yfiles.algorithms.EdgeList,origin:yfiles.algorithms.YPoint,rectangle:yfiles.algorithms.YRectangle):void;
      /**
       * Data provider key for specifying which nodes should be arranged
       * <p>
       * If no {@link yfiles.algorithms.IDataProvider} is registered with this key, all components will be laid out by the core layout
       * algorithm.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static AFFECTED_COMPONENTS_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for specifying custom graph components
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.lang.IComparable>}
       */
      static COMPONENT_ID_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.lang.IComparable>;
      /**
       * Gets or sets the preferred size of the layout.
       * <p>
       * The layout size also defines the desired aspect ratio (width/height).
       * </p>
       * <p>
       * The width and height need to be greater than zero.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified width or height is negative or zero.
       * @type {yfiles.algorithms.YDimension}
       */
      preferredSize:yfiles.algorithms.YDimension;
      /**
       * Gets or sets whether or not to take node and edge labels into account when calculating the bounding box of the graph
       * components.
       * @see yfiles.layout.ComponentLayout#calculateBounds
       * @type {boolean}
       */
      considerLabels:boolean;
      /**
       * Gets or sets whether or not the separately arranged components of the input graph should be arranged relative to each
       * other.
       * <p>
       * If enabled, the components are arranged using a specific {@link yfiles.layout.ComponentLayout#style style} without producing overlaps between components (except for
       * {@link yfiles.layout.ComponentArrangementStyles#NONE}). Otherwise, the layout algorithm will keep the components at their
       * location. Then, the components may overlap.
       * </p>
       * @type {boolean}
       */
      componentArrangement:boolean;
      /**
       * Gets or sets the current grid spacing.
       * <p>
       * Components will be moved by multiples of this value, thus keeping their offset to the grid. That way, components or
       * parts of them that were placed on a grid before, will stay on their original grid. The grid spacing also influences the
       * distance between the components.
       * </p>
       * <p>
       * The spacing needs to be a non-negative value. If the grid spacing is set to <code>0</code>, the grid won't be considered at all.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given spacing is negative
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets or sets the distance between the bounding boxes of the components.
       * <p>
       * The spacing needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the spacing is negative
       * @type {number}
       */
      componentSpacing:number;
      /**
       * Gets or sets whether or not grouping information bound to the graph should be considered when determining the graph
       * components.
       * @type {boolean}
       */
      considerGrouping:boolean;
      /**
       * Gets or sets how the components are arranged.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified style is unknown
       * @type {yfiles.layout.ComponentArrangementStyles}
       */
      style:yfiles.layout.ComponentArrangementStyles;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.CompositeLayoutStage} allows for combining multiple {@link yfiles.layout.ILayoutStage}s.
     * <p>
     * The {@link yfiles.layout.ILayoutStage}s are organized in a pipeline where additional {@link yfiles.layout.ILayoutStage}s can be {@link yfiles.layout.CompositeLayoutStage#prependStage prepended}
     * or {@link yfiles.layout.CompositeLayoutStage#appendStage appended}.
     * </p>
     * <p>
     * Each {@link yfiles.layout.ILayoutStage} in the pipeline executes its pre-processing code before calling its successor
     * {@link yfiles.layout.ILayoutStage}. Then, after all subsequent
     * {@link yfiles.layout.ILayoutStage}s have finished, it will execute its post-processing code.
     * </p>
     * <p>
     * Note that the pipeline only works when all {@link yfiles.layout.ILayoutStage}s call their core layout algorithm which is the
     * next {@link yfiles.layout.ILayoutStage} in the pipeline.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface CompositeLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class CompositeLayoutStage {
      /**
       * Creates a new {@link yfiles.layout.CompositeLayoutStage} instance with two {@link yfiles.layout.ILayoutStage}s in the pipeline.
       * <p>
       * The inner {@link yfiles.layout.ILayoutStage} will be wrapped by the outer {@link yfiles.layout.ILayoutStage}.
       * </p>
       * @param outerStage the wrapping {@link }
       * @param innerStage the wrapped {@link }
       */
      constructor(outerStage:yfiles.layout.ILayoutStage,innerStage:yfiles.layout.ILayoutStage);
      /**
       * Creates a new {@link yfiles.layout.CompositeLayoutStage} instance with an empty pipeline.
       */
      constructor();
      /**
       * Appends a {@link yfiles.layout.ILayoutStage} to the previously added {@link yfiles.layout.ILayoutStage}s.
       * <p>
       * The added {@link yfiles.layout.ILayoutStage} will be invoked just before the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} will be invoked. Its
       * pre-processing code will be executed after the already added {@link yfiles.layout.ILayoutStage}s' pre-processing code, while its
       * post-processing code will be executed before the post-processing code of all the other {@link yfiles.layout.ILayoutStage}s.
       * </p>
       * @param stage the stage to append
       */
      appendStage(stage:yfiles.layout.ILayoutStage):void;
      /**
       * Prepends a {@link yfiles.layout.ILayoutStage} to the previously added {@link yfiles.layout.ILayoutStage}s.
       * <p>
       * The added {@link yfiles.layout.ILayoutStage} will be invoked before any other {@link yfiles.layout.ILayoutStage}. Its pre-processing
       * code will be executed before the already added {@link yfiles.layout.ILayoutStage}s' pre-processing code, while its
       * post-processing code will be executed after the post-processing code of all the other {@link yfiles.layout.ILayoutStage}s.
       * </p>
       * @param stage the stage to prepend
       */
      prependStage(stage:yfiles.layout.ILayoutStage):void;
      /**
       * Gets the complete pipeline of {@link yfiles.layout.ILayoutStage}s that were added to this {@link yfiles.layout.CompositeLayoutStage}.
       * @type {yfiles.collections.IList.<Object>}
       */
      layoutStages:yfiles.collections.IList<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.CopiedLayoutGraph} is a {@link yfiles.layout.LayoutGraph} that serves as a <b>copy of another graph</b> with layout information.
     * <p>
     * A {@link yfiles.layout.CopiedLayoutGraph} comes to use if a {@link yfiles.layout.ILayoutAlgorithm layout algorithm} should not run directly on the original graph.
     * Using a copy, the calculated layout will not be applied to the original graph before the layout algorithm finished
     * successfully. This can <em>prevent inconsistencies</em> in case something goes wrong during layout calculations.
     * </p>
     * <p>
     * <b>Concept</b> {@link yfiles.layout.CopiedLayoutGraph} contains corresponding nodes and edges for each node and edge in the original graph and
     * copies the labels of the original nodes and edges. It also delegates all
     * {@link yfiles.algorithms.IDataProvider}s registered with the original graph.
     * </p>
     * <p>
     * To preserve the relations between the original elements and the copied elements in this graph, {@link yfiles.layout.CopiedLayoutGraph}
     * updates mappings between those elements.
     * </p>
     * <p>
     * If there are structural changes to the original graph, they can be transferred to this copied graph using
     * {@link yfiles.layout.CopiedLayoutGraph#syncStructure}. Then the copied graph will be altered, adding and removing elements,
     * until it corresponds to the original again.
     * </p>
     * <p>
     * Method {@link yfiles.layout.CopiedLayoutGraph#commitLayoutToOriginalGraph} will transfer updated layout information to the
     * original graph. The original graph will stay unchanged until this method is called.
     * </p>
     * @see yfiles.layout.BufferedLayout
     * @class
     * @extends {yfiles.layout.LayoutGraph}
     */
    export interface CopiedLayoutGraph extends yfiles.layout.LayoutGraph{}
    export class CopiedLayoutGraph {
      /**
       * Creates a new {@link yfiles.layout.CopiedLayoutGraph} instance which is a copy of the given graph.
       * @param graph the original graph
       */
      constructor(graph:yfiles.layout.LayoutGraph);
      /**
       * Writes the current layout information of all elements of this copied graph to the original graph.
       */
      commitLayoutToOriginalGraph():void;
      /**
       * Creates an edge in the copied graph that corresponds to the given original edge.
       * <p>
       * The edge will connect to the copies of the source and target of the original edge. All labels of the original edge will
       * be also copied and assigned to this edge.
       * </p>
       * <p>
       * The mappings are updated. Hence, it is possible to look up the newly created edge by using {@link yfiles.layout.CopiedLayoutGraph#getCopiedEdge}
       * or the original edge by using {@link yfiles.layout.CopiedLayoutGraph#getOriginalEdge}.
       * </p>
       * <p>
       * This method can be used for keeping the {@link yfiles.layout.CopiedLayoutGraph} in sync with the original graph.
       * </p>
       * @param originalEdge the original edge
       * @returns the copy of the original edge
       */
      createCopiedEdge(originalEdge:Object):yfiles.algorithms.Edge;
      /**
       * Creates a node in the copied graph that corresponds to the given original node.
       * <p>
       * All labels of the original node will be copied and assigned to this node.
       * </p>
       * <p>
       * The mappings are updated. Hence, it is possible to look up the newly created node by using {@link yfiles.layout.CopiedLayoutGraph#getCopiedNode}
       * or the original node by using {@link yfiles.layout.CopiedLayoutGraph#getOriginalNode}.
       * </p>
       * <p>
       * This method can be used for keeping the {@link yfiles.layout.CopiedLayoutGraph} in sync with the original graph.
       * </p>
       * @param originalNode the original node
       * @returns the copy of the original node
       */
      createCopiedNode(originalNode:Object):yfiles.algorithms.Node;
      /**
       * Returns the copied edge that corresponds to the given original edge.
       * @param originalEdge an edge of the original graph whose copy resides in this graph
       * @returns an edge in this graph that is the copy of the given original edge
       */
      getCopiedEdge(originalEdge:Object):yfiles.algorithms.Edge;
      /**
       * Returns the copied node that corresponds to the given original node.
       * @param originalNode a node of the original graph whose copy resides in this graph
       * @returns a node in this graph that is the copy of the given original node
       */
      getCopiedNode(originalNode:Object):yfiles.algorithms.Node;
      /**
       * Returns the original edge that corresponds to the given copied edge.
       * @param copiedEdge the edge in this graph that is a copy of the returned edge
       * @returns the edge in the original graph whose copy is the given edge
       */
      getOriginalEdge(copiedEdge:yfiles.algorithms.Edge):Object;
      /**
       * Returns the original node that corresponds to the given copied node.
       * @param copiedNode the node in this graph that is a copy of the returned node
       * @returns the node in the original graph whose copy is the given node
       */
      getOriginalNode(copiedNode:yfiles.algorithms.Node):Object;
      /**
       * Synchronizes the structure of the {@link yfiles.layout.CopiedLayoutGraph} with the actual structure of the underlying original
       * graph.
       */
      syncStructure():void;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.CurveFittingLayoutStage} fits a piecewise <em>cubic bezier curve</em> to given arbitrary edge paths.
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * The {@link yfiles.layout.LayoutGraph#getPath path} of an edge, i.e., its start and end point as well as the intermediate control points, are interpreted as the
     * set of input points. A cubic bezier curve is then fitted to these points.
     * </p>
     * <p>
     * After the fitting, the points of the processed edges can be interpreted as cubic bezier control points. Each four
     * consecutive points form a bezier curve, where the first and last of the four points represent the start and end of the
     * curve and the second and third point are the control points defining how the curve looks. The second and third point do
     * not necessarily lie on the actual curve.
     * </p>
     * <p>
     * There are two exceptions to the stated rules:
     * </p>
     * <ul>
     * <li>If an input edge only consists of a source and a target point, it will be ignored by this stage.</li>
     * <li>
     * Self-loop edges with only one intermediate control point are also ignored. Note that self-loops with more control points
     * are, however, handled by this stage.
     * </li>
     * </ul>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The {@link yfiles.layout.CurveFittingLayoutStage#maximumError maximum allowed error} for the fitting process can be selected to control how high the precision should be. A high
     * allowed error means that the approximation process has a lot of freedom. Curves are only loosely bound to the original
     * edge path. If this is not desired, e.g., because no new intersection with other graph elements should be introduced,
     * then a rather low error value should be specified. The lower the error value, the larger the number of bend points of an
     * approximated edge will be.
     * </p>
     * <p>
     * It is possible to define the set of edges that should be processed by this layout stage. Therefore, only paths of a
     * sub-set of edges can be changed. Use a {@link yfiles.algorithms.IDataProvider} registered with the specified {@link yfiles.layout.CurveFittingLayoutStage#affectedEdgesDpKey selection key} to
     * mark edges which should be part of the considered sub-set.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface CurveFittingLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class CurveFittingLayoutStage {
      /**
       * Creates a new {@link yfiles.layout.CurveFittingLayoutStage} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The core layout algorithm.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Data provider key for selecting edges for curve fitting
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DEFAULT_AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the maximum allowed error for the fitting process.
       * <p>
       * The error is defined as the minimum distance between an original input control point (i.e. bend) of and edge and the
       * fitted bezier curve.
       * </p>
       * <p>
       * The maximum error must be greater than or equal to zero.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified maximum error is negative
       * @type {number}
       */
      maximumError:number;
      /**
       * Gets or sets the ratio that defines the distance between the intermediate control points and the end points of a cubic
       * segment which actually represents a straight line, i.e., in case the four points are collinear.
       * <ul>
       * <li><code>0.33</code> - the second and third control point are equally distributed between te first and fourth point.</li>
       * <li><code>0.0</code> - the first and second control point as well as the third and fourth point are equal.</li>
       * <li><code>0.5</code> - the second and third control point are equal and exactly the mid-point between the first and fourth point.</li>
       * </ul>
       * <p>
       * The control point ratio is defined to be within the interval <code>[0.0, 0.5]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified ratio is outside the interval <code>[0.0, 0.5]</code>
       * @type {number}
       */
      straightControlPointRatio:number;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key to look up the selection state of the edges that defines whether or
       * not an edge's path should be approximated by a bezier curve.
       * <p>
       * If there is a {@link yfiles.algorithms.IDataProvider} registered with the input graph with this key, then only marked edges are
       * processed by this layout stage. Otherwise, all edges are processed.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.algorithms.IDataProvider} key is <code>null</code>
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.DefaultLayoutGraph} is a default implementation of {@link yfiles.layout.LayoutGraph} which holds the complete
     * layout information about the graph and its elements.
     * @class
     * @extends {yfiles.layout.LayoutGraph}
     */
    export interface DefaultLayoutGraph extends yfiles.layout.LayoutGraph{}
    export class DefaultLayoutGraph {
      /**
       * Creates a new {@link yfiles.layout.DefaultLayoutGraph} instance which is a copy of the given subgraph.
       * @param graph the original graph
       * @param nodeSubset the nodes that induce a subgraph of the original graph
       */
      constructor(graph:yfiles.layout.LayoutGraph,nodeSubset:yfiles.algorithms.ICursor);
      /**
       * Creates a new {@link yfiles.layout.DefaultLayoutGraph} instance which is a copy of the given graph.
       * @param argGraph the original graph
       */
      constructor(argGraph:yfiles.layout.LayoutGraph);
      /**
       * Creates a new {@link yfiles.layout.DefaultLayoutGraph} instance with default settings.
       */
      constructor();
      /**
       * Creates a new implementation of {@link yfiles.layout.IEdgeLayout}.
       * <p>
       * This method is called by {@link yfiles.layout.DefaultLayoutGraph#getLayout} if there is no layout information assigned to an
       * edge, yet. It may be overridden to create custom {@link yfiles.layout.IEdgeLayout}s.
       * </p>
       * @returns the layout of an edge
       * @protected
       */
      createEdgeLayout():yfiles.layout.IEdgeLayout;
      /**
       * Creates a new implementation of {@link yfiles.layout.INodeLayout}.
       * <p>
       * This method is called by {@link yfiles.layout.DefaultLayoutGraph#getLayout} if there is no layout information assigned to a
       * node, yet. It may be overridden to create custom {@link yfiles.layout.INodeLayout}s.
       * </p>
       * @returns the layout of a node
       * @protected
       */
      createNodeLayout():yfiles.layout.INodeLayout;
      /**
       * Specifies the layout information for all associated labels of the given edge.
       * <p>
       * The layout information consists of the location, size and orientation of the label.
       * </p>
       * @param edge the edge
       * @param layout the layout information for the labels of the edge
       */
      setLabelLayout(edge:yfiles.algorithms.Edge,layout:yfiles.layout.IEdgeLabelLayout[]):void;
      /**
       * Specifies the layout information for all associated labels of the given node.
       * <p>
       * The layout information consists of the location, size and orientation of the label.
       * </p>
       * @param node the node
       * @param layout the layout information for the labels of the node
       */
      setLabelLayout(node:yfiles.algorithms.Node,layout:yfiles.layout.INodeLabelLayout[]):void;
      /**
       * Specifies the layout information for all associated labels of the given node in case there is only one label.
       * <p>
       * The layout information consists of the location, size and orientation of the label.
       * </p>
       * @param node the node
       * @param layout the layout information for the label of the node
       */
      setLabelLayout(node:yfiles.algorithms.Node,layout:yfiles.layout.INodeLabelLayout):void;
      /**
       * Specifies the layout information for the given edge.
       * <p>
       * The layout information consists of the locations of the control and end points of the edge.
       * </p>
       * @param edge the edge
       * @param layout the layout information for the edge
       */
      setLayout(edge:yfiles.algorithms.Edge,layout:yfiles.layout.IEdgeLayout):void;
      /**
       * Specifies the layout information for the given node.
       * <p>
       * The layout information consists of the location and size of the node.
       * </p>
       * @param node the node
       * @param layout the layout information for the node
       */
      setLayout(node:yfiles.algorithms.Node,layout:yfiles.layout.INodeLayout):void;
      /**
       * A {@link yfiles.algorithms.INodeMap} associating {@link yfiles.layout.INodeLabelLayout}s with the nodes of this graph.
       * @see yfiles.layout.DefaultLayoutGraph#setLabelLayout
       * @type {yfiles.algorithms.INodeMap}
       */
      nodeLabelMap:yfiles.algorithms.INodeMap;
      /**
       * An {@link yfiles.algorithms.IEdgeMap} associating {@link yfiles.layout.IEdgeLabelLayout}s with the edges of this graph.
       * @see yfiles.layout.DefaultLayoutGraph#setLabelLayout
       * @type {yfiles.algorithms.IEdgeMap}
       */
      edgeLabelMap:yfiles.algorithms.IEdgeMap;
      /**
       * A {@link yfiles.collections.IMap.<T1,T2>} associating the owner of a node label with the node label itself.
       * @type {yfiles.collections.IMap.<Object,Object>}
       */
      nodeLabelFeatureMap:yfiles.collections.IMap<Object,Object>;
      /**
       * A {@link yfiles.collections.IMap.<T1,T2>} associating the owner of an edge label with the edge label itself.
       * @type {yfiles.collections.IMap.<Object,Object>}
       */
      edgeLabelFeatureMap:yfiles.collections.IMap<Object,Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.Direction} provides four main directions and some methods to work with them.
     * @class
     * @final
     */
    export interface Direction extends Object{}
    export class Direction {
      /**
       * Returns the direction encoded as an integer value.
       * @returns <code>0</code> for direction {@link #UP}, <code>1</code> for {@link #RIGHT}, <code>2</code> for {@link #DOWN}, and
       * <code>3</code> for {@link #LEFT}
       */
      getDirection():number;
      /**
       * Decodes the up direction.
       * @const
       * @static
       * @type {yfiles.layout.Direction}
       */
      static UP:yfiles.layout.Direction;
      /**
       * Decodes the right direction.
       * @const
       * @static
       * @type {yfiles.layout.Direction}
       */
      static RIGHT:yfiles.layout.Direction;
      /**
       * Decodes the down direction.
       * @const
       * @static
       * @type {yfiles.layout.Direction}
       */
      static DOWN:yfiles.layout.Direction;
      /**
       * Decodes the left direction.
       * @const
       * @static
       * @type {yfiles.layout.Direction}
       */
      static LEFT:yfiles.layout.Direction;
      /**
       * Gets the direction that follows this direction in clockwise order.
       * <p>
       * The new direction is turned <code>90</code> degrees in clockwise direction compared to this instance.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the current direction value is invalid
       * @type {yfiles.layout.Direction}
       */
      turnCW:yfiles.layout.Direction;
      /**
       * Gets the direction that follows this direction in counterclockwise order.
       * <p>
       * The new direction is turned <code>90</code> degrees in counterclockwise direction compared to this instance.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the current direction value is invalid
       * @type {yfiles.layout.Direction}
       */
      turnCCW:yfiles.layout.Direction;
      /**
       * Gets the mirror direction.
       * <p>
       * The new direction is turned <code>180</code> degrees compared to this instance.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the current direction value is invalid
       * @type {yfiles.layout.Direction}
       */
      mirror:yfiles.layout.Direction;
      /**
       * Gets whether or not this direction is horizontal ({@link yfiles.layout.Direction#RIGHT} or {@link yfiles.layout.Direction#LEFT}).
       * @type {boolean}
       */
      horizontal:boolean;
      /**
       * Gets whether or not this direction is vertical ({@link yfiles.layout.Direction#UP} or {@link yfiles.layout.Direction#DOWN}).
       * @type {boolean}
       */
      vertical:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.DiscreteEdgeLabelLayoutModel} describes the placement of rotated labels at some predefined positions along
     * the edge.
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1,"lblB":1},"n":[[0,0],[200,0]],"e":[[0,1,[],[[35,-13.59,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[158.19,-13.59,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[96.59,-13.59,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[35,33.22,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[158.19,33.22,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[96.59,33.22,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[35,9.82,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[96.59,9.82,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"],[158.19,9.82,33.36,18.7,0.7071067811865475,-0.7071067811865476,"Label"]]]],"vp":[0.0,-27.0,230.0,84.0]}}
     * </p>
     * <p>
     * The predefined positions in this model
     * </p>
     * <p>
     * It's possible to specify a distance value that controls the distance between label and edge.
     * </p>
     * <p>
     * Furthermore, there's the possibility to mask out arbitrary edge label candidates. This can either be done by specifying
     * predefined candidate masks or combining several label positions with a logical <code>or</code>-operation to a user-defined mask.
     * </p>
     * @class
     * @implements {yfiles.layout.IEdgeLabelLayoutModel}
     */
    export interface DiscreteEdgeLabelLayoutModel extends Object,yfiles.layout.IEdgeLabelLayoutModel{}
    export class DiscreteEdgeLabelLayoutModel {
      /**
       * Creates a new instance of {@link yfiles.layout.DiscreteEdgeLabelLayoutModel} using the given candidate mask.
       * <p>
       * The candidates mask can either describe multiple or single valid positions.
       * </p>
       * @param candidateMask the position mask that defines the allowed positions for an edge label
       */
      constructor(candidateMask:yfiles.layout.DiscreteEdgeLabelPositions);
      /**
       * Creates a new instance of {@link yfiles.layout.DiscreteEdgeLabelLayoutModel} with default settings.
       */
      constructor();
      /**
       * 
       * @param labelBounds
       * @param edgeLayout
       * @param sourceNode
       * @param targetNode
       * @returns 
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,edgeLayout:yfiles.layout.IEdgeLayout,sourceNode:yfiles.layout.INodeLayout,targetNode:yfiles.layout.INodeLayout):Object;
      /**
       * Returns a model parameter that encodes the specified position.
       * <p>
       * This model parameter can be passed to {@link yfiles.layout.DiscreteEdgeLabelLayoutModel#getLabelPlacement} to determine the
       * label's position.
       * </p>
       * @param position one of the valid positions
       * @returns a model parameter that encodes the specified position
       * @throws {Stubs.Exceptions.ArgumentError} if the specified position is unknown
       * @static
       */
      static createPositionParameter(position:number):Object;
      /**
       * 
       * @param label
       * @param edgeLayout
       * @param sourceNode
       * @param targetNode
       * @returns 
       */
      getLabelCandidates(label:yfiles.layout.IEdgeLabelLayout,edgeLayout:yfiles.layout.IEdgeLayout,sourceNode:yfiles.layout.INodeLayout,targetNode:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * 
       * @param labelSize
       * @param edgeLayout
       * @param sourceNode
       * @param targetNode
       * @param param
       * @returns 
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,edgeLayout:yfiles.layout.IEdgeLayout,sourceNode:yfiles.layout.INodeLayout,targetNode:yfiles.layout.INodeLayout,param:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Returns the oriented box of the label for the given label position.
       * <p>
       * This method is called by {@link yfiles.layout.DiscreteEdgeLabelLayoutModel#getLabelPlacement} and {@link yfiles.layout.DiscreteEdgeLabelLayoutModel#getLabelCandidates}
       * to retrieve a valid position.
       * </p>
       * @param labelSize the size of the label that should be placed
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @param position the label position that is valid in this model
       * @returns the oriented box of the label
       * @protected
       */
      getLabelPlacementForPosition(labelSize:yfiles.algorithms.YDimension,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout,position:yfiles.layout.DiscreteEdgeLabelPositions):yfiles.algorithms.YOrientedRectangle;
      /**
       * Returns the position specifier that is encoded by the given model parameter.
       * @param parameter the model parameter
       * @returns the position specifier
       * @throws {Stubs.Exceptions.ArgumentError} if the specified model parameter is not valid for this model
       * @static
       */
      static getPosition(parameter:Object):number;
      /**
       * Checks whether or not the given model parameter encodes a valid edge label position for this model.
       * <p>
       * If the model parameter describes a position that is accepted by the candidate mask, this is a valid parameter.
       * </p>
       * @param parameter the model parameter
       * @returns <code>true</code> if the label position described by the given model parameter is allowed, <code>false</code> otherwise
       */
      isParameterValid(parameter:Object):boolean;
      /**
       * Gets the candidate mask which specifies the valid positions for edge labels.
       * @type {yfiles.layout.DiscreteEdgeLabelPositions}
       */
      candidateMask:yfiles.layout.DiscreteEdgeLabelPositions;
      /**
       * Gets or sets whether or not the label position mask should be interpreted relative to the edge segment.
       * <p>
       * If this option is disabled, the position mask is interpreted in a geometric sense.
       * </p>
       * @type {boolean}
       */
      positionRelativeToSegment:boolean;
      /**
       * Gets or sets whether or not edge labels are automatically rotated according to the angle of the corresponding reference
       * edge segment.
       * @type {boolean}
       */
      autoRotationEnabled:boolean;
      /**
       * Gets or sets whether or not edge labels get flipped if they would be upside down in their current position.
       * @type {boolean}
       */
      autoFlipping:boolean;
      /**
       * Gets or sets the angle (measured in radians) of the label model.
       * <p>
       * The angle is applied in clockwise direction.
       * </p>
       * @type {number}
       */
      angle:number;
      /**
       * Gets or sets the distance between the label's bounding box and the edge path.
       * <p>
       * The distance must be a non-negative value.
       * </p>
       * @type {number}
       */
      distance:number;
      /**
       * Gets a model parameter that encodes the default position of this model's allowed edge label positions.
       * <p>
       * Default positions are (in descending order):
       * </p>
       * <ul>
       * <li>{@link yfiles.layout.DiscreteEdgeLabelPositions#CENTER}</li>
       * <li>{@link yfiles.layout.DiscreteEdgeLabelPositions#SOURCE_CENTER}</li>
       * <li>{@link yfiles.layout.DiscreteEdgeLabelPositions#TAIL}</li>
       * <li>{@link yfiles.layout.DiscreteEdgeLabelPositions#SOURCE_TAIL}</li>
       * </ul>
       * <p>
       * Descending order means that whenever two or more of the above default positions are part of the allowed positions, then
       * the model parameter encodes the one that is listed first.
       * </p>
       * @type {Object}
       */
      defaultParameter:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.DiscreteNodeLabelLayoutModel} allows placing labels at eight positions around a node and at nine positions
     * inside the node.
     * <p>
     * {@graph {"ann":{"s":[120,80],"d":1,"lblB":1},"n":[[-11.9,13.17,[[31.42,9.17,33.36,18.7,"Label"],[112.1,111.88,33.36,18.7,"Label"],[112.1,62.53,33.36,18.7,"Label"],[31.42,115.88,33.36,18.7,"Label"],[-49.27,111.88,33.36,18.7,"Label"],[-49.27,62.53,33.36,18.7,"Label"],[112.1,13.17,33.36,18.7,"Label"],[-49.27,13.17,33.36,18.7,"Label"],[31.42,62.53,33.36,18.7,"Label"],[-7.9,62.53,33.36,18.7,"Label"],[-7.9,89.17,33.36,18.7,"Label"],[31.42,89.17,33.36,18.7,"Label"],[70.73,89.17,33.36,18.7,"Label"],[-7.9,35.88,33.36,18.7,"Label"],[70.73,62.53,33.36,18.7,"Label"],[70.73,35.88,33.36,18.7,"Label"],[31.42,35.88,33.36,18.7,"Label"]]]],"e":[],"vp":[-50.0,-10.0,196.0,126.0]}}
     * </p>
     * <p>
     * The predefined positions in this model
     * </p>
     * <p>
     * It's possible to specify an insets value that controls the distance between label and node.
     * </p>
     * <p>
     * Furthermore, there's the possibility to mask out arbitrary node label candidates. This can either be done by specifying
     * predefined candidate masks or a combination of valid positions of this model using a logical <code>or</code>-operation.
     * </p>
     * @class
     * @implements {yfiles.layout.INodeLabelLayoutModel}
     */
    export interface DiscreteNodeLabelLayoutModel extends Object,yfiles.layout.INodeLabelLayoutModel{}
    export class DiscreteNodeLabelLayoutModel {
      /**
       * Creates a new instance of {@link yfiles.layout.DiscreteNodeLabelLayoutModel} using the given candidate mask and insets.
       * <p>
       * The specified mask can also describe a single position.
       * </p>
       * @param candidateMask the position mask that defines the allowed positions for a node label
       * @param [inset=4.0] the inset value that defines the internal label distance to the node's border
       */
      constructor(candidateMask:yfiles.layout.DiscreteNodeLabelPositions,inset?:number);
      /**
       * Creates a new instance of {@link yfiles.layout.DiscreteNodeLabelLayoutModel} with default settings.
       */
      constructor();
      /**
       * 
       * @param labelBounds
       * @param nodeLayout
       * @returns 
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,nodeLayout:yfiles.layout.INodeLayout):Object;
      /**
       * 
       * @param labelLayout
       * @param nodeLayout
       * @returns 
       */
      getLabelCandidates(labelLayout:yfiles.layout.INodeLabelLayout,nodeLayout:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * 
       * @param labelSize
       * @param nodeLayout
       * @param parameter
       * @returns 
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,nodeLayout:yfiles.layout.INodeLayout,parameter:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Returns the oriented box of the label for the given label position.
       * <p>
       * This method is called by {@link yfiles.layout.DiscreteNodeLabelLayoutModel#getLabelPlacement} and {@link yfiles.layout.DiscreteNodeLabelLayoutModel#getLabelCandidates}
       * to retrieve a valid position.
       * </p>
       * @param labelSize the size of the label that should be placed
       * @param nodeLayout the layout of the node to which the label belongs
       * @param position the label position that is valid in this model
       * @returns the oriented box of the label
       * @protected
       */
      getLabelPlacementForPosition(labelSize:yfiles.algorithms.YDimension,nodeLayout:yfiles.layout.INodeLayout,position:yfiles.layout.DiscreteNodeLabelPositions):yfiles.algorithms.YOrientedRectangle;
      /**
       * Checks whether or not the given model parameter encodes a valid node label position for this model.
       * <p>
       * If the model parameter describes a position that is accepted by the candidate mask, this is a valid parameter.
       * </p>
       * @param parameter the model parameter
       * @returns <code>true</code> if the label position described by the given model parameter is allowed, <code>false</code> otherwise
       */
      isParameterValid(parameter:Object):boolean;
      /**
       * Gets the candidate mask which specifies the valid positions for node labels.
       * @type {yfiles.layout.DiscreteNodeLabelPositions}
       */
      candidateMask:yfiles.layout.DiscreteNodeLabelPositions;
      /**
       * Gets or sets the distance between the label's bounding box and the node.
       * <p>
       * The distance must be a non-negative value.
       * </p>
       * @type {number}
       */
      distance:number;
      /**
       * Gets a model parameter that encodes the default position of this model's allowed node label positions.
       * <p>
       * Default positions are (in descending order):
       * </p>
       * <ul>
       * <li>{@link yfiles.layout.DiscreteNodeLabelPositions#CENTER}</li>
       * <li>{@link yfiles.layout.DiscreteNodeLabelPositions#NORTH_EAST}</li>
       * <li>{@link yfiles.layout.DiscreteNodeLabelPositions#NORTH}</li>
       * <li>{@link yfiles.layout.DiscreteNodeLabelPositions#EAST}</li>
       * </ul>
       * <p>
       * Descending order means that whenever two or more of the above default positions are part of the allowed positions, then
       * the model parameter encodes the one that is listed first.
       * </p>
       * @type {Object}
       */
      defaultParameter:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.layout.EdgeBundleDescriptor} defines the {@link yfiles.layout.EdgeBundling bundling} settings of an edge.
     * <p>
     * The defined settings will be considered by {@link yfiles.layout.ILayoutAlgorithm layout algorithms} which support edge bundling.
     * </p>
     * <p>
     * Each {@link yfiles.layout.EdgeBundleDescriptor} can either be specified for a single edge or shared by several edges. It is
     * assigned to an edge by registering a {@link yfiles.algorithms.IDataProvider} with key {@link yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}
     * to the input graph.
     * </p>
     * @see yfiles.layout.EdgeBundling
     * @see yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY
     * @class
     */
    export interface EdgeBundleDescriptor extends Object{}
    export class EdgeBundleDescriptor {
      /**
       * Creates a new {@link yfiles.layout.EdgeBundleDescriptor} representing a copy of the given other descriptor instance.
       * @param descriptor the descriptor to copy
       */
      constructor(descriptor:yfiles.layout.EdgeBundleDescriptor);
      /**
       * Creates a new {@link yfiles.layout.EdgeBundleDescriptor} with default settings.
       */
      constructor();
      /**
       * Gets or sets whether or not the direction of the edge associated to this descriptor should be considered.
       * <p>
       * During the bundling procedure, the edges are bundled based on their direction such that the incoming edges of a node are
       * bundled separately from its outgoing edges. If the direction of edges adjacent to a node is not considered, then they
       * are not bundled separately.
       * </p>
       * <p>
       * If the direction of some edges is taken into consideration while the direction of some others (adjacent to the same
       * node) is ignored, the undirected edges are also bundled separately. This is done to distinguish them from the incoming
       * and outgoing edges of the particular node.
       * </p>
       * @type {boolean}
       */
      considerDirection:boolean;
      /**
       * Gets or sets whether or not an edge associated to this descriptor should be bundled.
       * @type {boolean}
       */
      bundled:boolean;
      /**
       * Gets or sets whether or not a bezier curve should be fit to an edge associated to this descriptor.
       * <p>
       * If disabled, the edges resemble B-splines where points of the path are actually curve points. Therefore, this
       * representation requires a high number of bends to model curves.
       * </p>
       * <p>
       * If this feature is enabled, a bezier curve is fitted to the start and end point as well as the intermediate control
       * points of the edge. After the fitting, the points of the edge can be interpreted as cubic bezier control points. Each
       * four consecutive points form a bezier curve, where the first and last of the four points represent the start and end of
       * the curve and the second and third point are the control points defining how the curve looks. The second and third point
       * do not necessarily lie on the actual curve.
       * </p>
       * @type {boolean}
       */
      bezierFitting:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.layout.EdgeBundling} instance defines if and how the edges of a graph should be bundled by a
     * {@link yfiles.layout.ILayoutAlgorithm layout algorithm}, given that the algorithm supports edge bundling.
     * <p>
     * Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
     * source and target point, the edges are again clearly split. Edge bundling is useful to increase the readability of graph
     * drawings with a high number of edges that connect a comparably small number of nodes. Without bundling, such drawings
     * can often contain visual clutter and feature bad readability.
     * </p>
     * <p>
     * The bundling {@link yfiles.layout.EdgeBundling#bundlingStrength strength} and {@link yfiles.layout.EdgeBundling#bundlingQuality quality} are global settings for the bundling process. On the other hand, each edge can get
     * specific settings by assigning a {@link yfiles.layout.EdgeBundleDescriptor} to it. A {@link yfiles.algorithms.IDataProvider} can be
     * registered with the input graph with key {@link yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY} to assign descriptors
     * to edges. This allows, for example, to define which edges should actually be bundled.
     * </p>
     * @see yfiles.layout.EdgeBundleDescriptor
     * @see yfiles.circular.CircularLayout#edgeBundling
     * @class
     */
    export interface EdgeBundling extends Object{}
    export class EdgeBundling {
      /**
       * Creates a new {@link yfiles.layout.EdgeBundling} instance with the given {@link yfiles.layout.EdgeBundling#bundlingStrength bundling strength} and {@link yfiles.layout.EdgeBundling#bundlingQuality bundling quality}.
       * @param bundlingStrength the bundling strength from the interval <code>[0,1]</code>
       * @param bundlingQuality the bundling quality from the interval <code>[0,1]</code>
       * @throws {Stubs.Exceptions.ArgumentError} if the given bundling strength or quality is not within <code>[0,1]</code>
       */
      constructor(bundlingStrength:number,bundlingQuality:number);
      /**
       * Creates a new {@link yfiles.layout.EdgeBundling} instance with default settings.
       */
      constructor();
      /**
       * Data provider key for specifying individual bundling settings for edges
       * <p>
       * If no {@link yfiles.layout.EdgeBundleDescriptor} is mapped to an edge, the {@link yfiles.layout.EdgeBundling#defaultBundleDescriptor default descriptor} is used by the layout algorithm to
       * obtain the bundling setup of that edge.
       * </p>
       * @see yfiles.layout.EdgeBundling#defaultBundleDescriptor
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.layout.EdgeBundleDescriptor>}
       */
      static EDGE_BUNDLE_DESCRIPTOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.layout.EdgeBundleDescriptor>;
      /**
       * Gets or sets the desired quality of the calculated edge bundling.
       * <p>
       * Higher quality indicates that the bundling procedure uses more sophisticated methods to compute the actual bundles and
       * the routing of the edges. This may lead to a significantly higher running time, especially for large graphs. On the
       * other hand, a low quality indicates that run-time is more important than highly optimized bundling results, leading to,
       * for example, more crossings between different bundles.
       * </p>
       * <p>
       * The quality is defined to lie within <code>[0,1]</code>, where higher values stand for higher quality.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given quality value is not within <code>[0,1]</code>
       * @type {number}
       */
      bundlingQuality:number;
      /**
       * Gets or sets the strength of the edge bundling.
       * <p>
       * The strength controls how tightly the edges are bundled and influences the shape of the curves of bundled edges. Low
       * values lead to only slightly bundled edges; results will mostly show individual node-to-node connectivity information.
       * High values show the connectivity on a higher level, strongly bundling edges together and generating highly curved edge
       * paths.
       * </p>
       * <p>
       * The bundling strength is defined as a value from the interval <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given strength is less than <code>0</code> or greater than <code>1</code>
       * @type {number}
       */
      bundlingStrength:number;
      /**
       * Gets or sets the default {@link yfiles.layout.EdgeBundleDescriptor} used for all edges which do not have a specific descriptor
       * assigned via a {@link yfiles.algorithms.IDataProvider} registered with key
       * {@link yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
       * @throws {Stubs.Exceptions.ArgumentError} if the given descriptor is <code>null</code>
       * @see yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.EdgeBundleDescriptor}
       */
      defaultBundleDescriptor:yfiles.layout.EdgeBundleDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.layout.EdgeLabelCandidate} describes one valid placement for an edge label considering the label model.
     * @class
     * @extends {yfiles.layout.LabelCandidate}
     */
    export interface EdgeLabelCandidate extends yfiles.layout.LabelCandidate{}
    export class EdgeLabelCandidate {
      /**
       * Creates a new instance of {@link yfiles.layout.EdgeLabelCandidate} described by location, size, model parameter and internal
       * flag.
       * @param pos the location of the upper left corner of the candidate
       * @param size the size of the candidate
       * @param param the parameters of the label model associated with this candidate
       * @param owner the label associated with the candidate
       * @param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.IEdgeLabelLayout,internal?:boolean);
      /**
       * Creates a new instance of {@link yfiles.layout.EdgeLabelCandidate} described by its box, model parameter and internal flag.
       * @param labelBox the box that specifies the candidate's size and position
       * @param param the parameters of the label model associated with this candidate
       * @param owner the label associated with the candidate
       * @param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
       */
      constructor(labelBox:yfiles.algorithms.YOrientedRectangle,param:Object,owner:yfiles.layout.IEdgeLabelLayout,internal?:boolean);
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.EdgeLabelOrientationSupport} provides orientation and mirroring support for {@link yfiles.layout.ILayoutAlgorithm layout algorithms} with
     * integrated edge labeling.
     * <p>
     * To calculate the position and orientation of edge labels whose placements are defined with the
     * {@link yfiles.layout.PreferredPlacementDescriptor}, the {@link yfiles.layout.OrientationLayout} would need the direction of the
     * associated edge. Since this direction is not available to the {@link yfiles.layout.OrientationLayout}, because it is a wrapper,
     * the calculation must be done in the core layout algorithm itself with the help of the
     * {@link yfiles.layout.EdgeLabelOrientationSupport}.
     * </p>
     * <p>
     * Important: Since layout algorithms with integrated edge labeling cannot deal with ambiguous preferred placement
     * descriptors (combinations of the same setting, e.g., source or target for the {@link yfiles.layout.PreferredPlacementDescriptor#placeAlongEdge}), {@link yfiles.layout.EdgeLabelOrientationSupport#preProcessLabel}
     * removes combinations and replaces the original descriptor with a <em>non-ambiguous</em> one during the layout calculation.
     * </p>
     * @class
     */
    export interface EdgeLabelOrientationSupport extends Object{}
    export class EdgeLabelOrientationSupport {
      /**
       * Creates a new {@link yfiles.layout.EdgeLabelOrientationSupport} instance.
       * @see yfiles.layout.OrientationLayout#considerEdgeLabels
       */
      constructor();
      /**
       * Creates and returns a non-ambiguous {@link yfiles.layout.PreferredPlacementDescriptor} of the given potentially ambiguous one.
       * <p>
       * The created descriptor will be one without any combinations of settings, e.g., source or target for the {@link yfiles.layout.PreferredPlacementDescriptor#placeAlongEdge}.
       * </p>
       * @param ambiguous an ambiguous {@link }
       * @returns a non-ambiguous version of the given ambiguous {@link }
       * @protected
       */
      createNonAmbiguous(ambiguous:yfiles.layout.PreferredPlacementDescriptor):yfiles.layout.PreferredPlacementDescriptor;
      /**
       * Calculates the up vector of an edge label considering the preferred placement of the label as well as the direction of
       * the edge segment the label is attached to.
       * @param descriptor the data describing the preferred label orientation
       * @param segmentDirection the direction the edge segment is pointing to
       * @returns the up vector for the edge label that considers the preferred placement of the label as well as the direction of the
       * edge segment the label is attached to
       * @static
       */
      static getEdgeLabelUpVector(descriptor:yfiles.layout.PreferredPlacementDescriptor,segmentDirection:yfiles.layout.Direction):yfiles.algorithms.YVector;
      /**
       * Calculates the up vector for a label whose right vector is rotated clockwise by the given angle.
       * <p>
       * The up vector starts in the lower-left corner of the label and points up. If the label is paraxial it is <code>(0, -1)</code>. The
       * right vector starts at the same point and points right. In a paraxial label it is <code>(1, 0)</code>.
       * </p>
       * @param rightVectorAngle the angle in radians the right vector is rotated by
       * @returns the up vector for a label whose right vector is rotated clockwise by the given angle
       * @static
       */
      static getLabelUpVector(rightVectorAngle:number):yfiles.algorithms.YVector;
      /**
       * Transforms a direction of the layout to the oriented layout direction.
       * @param direction direction in the layout
       * @param orientation the orientation of the layout
       * @param mirrorMask the mask to determine whether or not the orientation is mirrored
       * @returns the direction in the oriented layout
       * @static
       */
      static getOrientedDirection(direction:yfiles.layout.Direction,orientation:yfiles.layout.LayoutOrientation,mirrorMask:yfiles.layout.MirrorModes):yfiles.layout.Direction;
      /**
       * Returns whether or not the orientation is mirrored.
       * @param orientation the orientation of the layout
       * @param mirrorMask the mask to determine whether or not the orientation is mirrored
       * @returns <code>true</code> if the orientation is mirrored, <code>false</code> otherwise
       * @static
       */
      static isOrientationMirrored(orientation:yfiles.layout.LayoutOrientation,mirrorMask:yfiles.layout.MirrorModes):boolean;
      /**
       * Marks the given port label for a non-port placement (fallback).
       * <p>
       * The given edge label's {@link yfiles.layout.PreferredPlacementDescriptor} will internally be changed such that its {@link yfiles.layout.PreferredPlacementDescriptor#placeAlongEdge placement along edge}
       * is either {@link yfiles.layout.LabelPlacements#AT_SOURCE} or {@link yfiles.layout.LabelPlacements#AT_TARGET}.
       * </p>
       * <p>
       * Algorithms that are executed after this call and before {@link yfiles.layout.EdgeLabelOrientationSupport#resetAmbiguousLabelDescriptors}
       * will handle port labels like normal source or target labels.
       * </p>
       * @param portLabel an edge label that had a {@link #AT_SOURCE_PORT} or {@link #AT_TARGET_PORT}
       * placement preference
       * @throws {Stubs.Exceptions.InvalidOperationError} if {@link yfiles.layout.EdgeLabelOrientationSupport#replaceAmbiguousLabelDescriptors} was never called for the graph containing
       * the given label.
       * @see yfiles.layout.LabelPlacements#AT_SOURCE_PORT
       * @see yfiles.layout.LabelPlacements#AT_TARGET_PORT
       */
      markPortLabelForFallbackPlacement(portLabel:yfiles.layout.LabelLayoutData):void;
      /**
       * Restores the original preferred placement and updates the label rotation according to the layout orientation.
       * <p>
       * The original {@link yfiles.layout.LabelLayoutData#preferredPlacementDescriptor} that has been replaced in {@link yfiles.layout.EdgeLabelOrientationSupport#preProcessLabel} gets restored. Then it
       * moves and rotates the {@link yfiles.layout.LabelLayoutData#bounds label box} in respect of the orientation.
       * </p>
       * <p>
       * This method is called by the core layout algorithm with the integrated edge labeling for each edge label after the
       * location of the label is determined.
       * </p>
       * @param graph the graph to get the orientation information from
       * @param label the information about the edge label whose location is determined
       */
      postProcessLabel(graph:yfiles.algorithms.Graph,label:yfiles.layout.LabelLayoutData):void;
      /**
       * Prepares the label for the core layout algorithm.
       * <p>
       * First, the {@link yfiles.layout.LabelLayoutData#preferredPlacementDescriptor preferred placement} of the edge label is replaced by a non-ambiguous {@link yfiles.layout.PreferredPlacementDescriptor}
       * that has only relative references. This descriptor is orientation independent and can be used in the core layout
       * algorithm. Second, the method rotates the {@link yfiles.layout.LabelLayoutData#bounds label box} in respect of its {@link yfiles.layout.PreferredPlacementDescriptor} and the
       * direction of the segment to which it belongs. After that call, the core layout algorithm can use the geometry ({@link yfiles.layout.LabelLayoutData#width} and
       * {@link yfiles.layout.LabelLayoutData#height}) of the label to calculate its location.
       * </p>
       * <p>
       * This method is called by the core layout algorithm with the integrated edge labeling for each edge label before the
       * layout algorithm uses the geometry of labels. It should be called as soon as the direction of the segment the label
       * belongs to is determined.
       * </p>
       * @param graph the graph to get the orientation information from
       * @param label the information about the edge label whose segment direction is determined
       * @param segmentDirection the direction of the segment the edge label belongs to
       */
      preProcessLabel(graph:yfiles.algorithms.Graph,label:yfiles.layout.LabelLayoutData,segmentDirection:yfiles.layout.Direction):void;
      /**
       * Replaces the
       * {@link yfiles.layout.PreferredPlacementDescriptor}s of all edge labels in the given graph with non-ambiguous descriptors.
       * <p>
       * The non-ambiguous descriptors are retrieved from the original descriptors. They describe only one of the possible
       * placements.
       * </p>
       * <p>
       * This method should be called by a layout algorithm at the beginning of the layout calculation to assure that the label's
       * placement is handled consistently.
       * </p>
       * @param graph the input graph
       */
      replaceAmbiguousLabelDescriptors(graph:yfiles.algorithms.Graph):void;
      /**
       * Restores the
       * {@link yfiles.layout.PreferredPlacementDescriptor}s of all edge labels in the given graph with their original descriptors.
       * <p>
       * This method should be called by a layout algorithm at the end of the layout calculation to assure that the original
       * state of the graph is not corrupted.
       * </p>
       * @param graph the input graph
       */
      resetAmbiguousLabelDescriptors(graph:yfiles.algorithms.Graph):void;
      /**
       * Calculates and sets the up vector of an edge label considering the preferred placement of the label as well as the
       * direction of the edge segment the label is attached to.
       * @param label the data that describes the label orientation and is used for setting the new up vector
       * @param segmentDirection the direction the edge segment is pointing to
       * @see yfiles.layout.EdgeLabelOrientationSupport#getEdgeLabelUpVector
       * @static
       */
      static updateLabelOrientation(label:yfiles.layout.LabelLayoutData,segmentDirection:yfiles.layout.Direction):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.FixNodeLayoutStage} fixes the initial position of either a <em>single</em> node or the alignment of the bounding box of
     * several nodes (but not the positions of the individual nodes).
     * <p>
     * The absolute coordinates of the {@link yfiles.layout.FixNodeLayoutStage#calculateFixPoint fix point} are the same before and after invoking the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. However, the
     * relative coordinates of the nodes are altered by the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface FixNodeLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class FixNodeLayoutStage {
      /**
       * Creates a new {@link yfiles.layout.FixNodeLayoutStage} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Calculates the bounding box of the specified nodes.
       * <p>
       * The calculated bounds may contain {@link yfiles.layout.FixNodeLayoutStage#includingEdges edges} and/or {@link yfiles.layout.FixNodeLayoutStage#includingLabels labels}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.FixNodeLayoutStage#calculateFixPoint} and may be overridden to customize the size
       * of the bounds.
       * </p>
       * @param graph the graph associated with the specified nodes
       * @param fixed the list of nodes for which to determine the bounding box
       * @returns the bounding box of the specified nodes
       * @see yfiles.layout.FixNodeLayoutStage#includingEdges
       * @see yfiles.layout.FixNodeLayoutStage#includingLabels
       * @see yfiles.layout.FixNodeLayoutStage#calculateFixPoint
       * @protected
       */
      calculateBounds(graph:yfiles.layout.LayoutGraph,fixed:yfiles.algorithms.NodeList):yfiles.algorithms.Rectangle2D;
      /**
       * Calculates the fix point for the specified nodes in the specified graph.
       * <p>
       * The default implementation calls {@link yfiles.layout.FixNodeLayoutStage#calculateBounds} and
       * {@link yfiles.layout.FixNodeLayoutStage#calculateFixPoint}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.FixNodeLayoutStage#applyLayout} to determine the point that will have the same
       * coordinates before and after invoking the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to introduce a custom policy for retrieving the fix point.
       * </p>
       * @param graph the graph associated with the specified nodes
       * @param fixed the list of nodes for which a fix point has to be determined
       * @returns the fix point for the specified nodes in the specified graph
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       * @see yfiles.layout.FixNodeLayoutStage#calculateBounds
       * @see yfiles.layout.FixNodeLayoutStage#calculateFixPoint
       * @protected
       */
      calculateFixPoint(graph:yfiles.layout.LayoutGraph,fixed:yfiles.algorithms.NodeList):yfiles.algorithms.YPoint;
      /**
       * Calculates the fix point of the specified rectangle according to the policy returned by {@link yfiles.layout.FixNodeLayoutStage#fixPointPolicy}.
       * <p>
       * If the specified rectangle is <code>null</code> or the rectangle has negative <code>width</code> or negative <code>height</code> then <code>null</code> will be returned.
       * </p>
       * <p>
       * This method is called by
       * {@link yfiles.layout.FixNodeLayoutStage#calculateFixPoint}. It may be overridden to introduce a custom policy for retrieving the
       * fix point.
       * </p>
       * @param bounds the rectangle for which to determine the fix point
       * @returns the fix point of the specified rectangle or <code>null</code> if nothing should be fixed
       * @throws {Stubs.Exceptions.ArgumentError} if the current fix point policy is invalid
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       * @protected
       */
      calculateFixPoint(bounds:yfiles.algorithms.Rectangle2D):yfiles.algorithms.YPoint;
      /**
       * Data provider key for determining the nodes that should be considered fixed
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static FIXED_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets or sets the fix point calculation policy used in {@link yfiles.layout.FixNodeLayoutStage#calculateFixPoint} to determine
       * which point in the corresponding rectangle should be considered fixed.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified policy is unknown
       * @type {yfiles.layout.FixPointPolicy}
       */
      fixPointPolicy:yfiles.layout.FixPointPolicy;
      /**
       * Gets or sets whether or not subgraph edges are taken into account when calculating the bounding box of the fixed nodes.
       * @see yfiles.layout.FixNodeLayoutStage#calculateBounds
       * @type {boolean}
       */
      includingEdges:boolean;
      /**
       * Gets or sets whether or not label geometries are taken into account when calculating the bounding box of the fixed
       * nodes.
       * @see yfiles.layout.FixNodeLayoutStage#calculateBounds
       * @type {boolean}
       */
      includingLabels:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.FixPortLocationStage} ensures that layout algorithms that cannot handle port constraints keep the ports of
     * edges with strong {@link yfiles.layout.PortConstraint}s.
     * <p>
     * Without this stage, the port locations get lost.
     * </p>
     * <p>
     * For each edge with strong {@link yfiles.layout.PortConstraint}s, this stage simply stores the original port and restores it
     * after applying the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. Therefore, it either replaces the source/target port by the original port or
     * adds the original port to the edge path calculated by the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} (see method {@link yfiles.layout.FixPortLocationStage#keepCalculatedPorts}).
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface FixPortLocationStage extends yfiles.layout.LayoutStageBase{}
    export class FixPortLocationStage {
      /**
       * Creates a new {@link yfiles.layout.FixPortLocationStage} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Gets or sets whether or not the final edge routes should contain the ports calculated by the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * <p>
       * If this option is disabled, this stage simply replaces the current source/target ports (calculated by the core layout
       * algorithm) by the original ports (for strong ports only). Otherwise, the calculated ports are still contained in the
       * edge paths.
       * </p>
       * @type {boolean}
       */
      keepCalculatedPorts:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This model parameter encodes a label position within {@link yfiles.layout.FreeEdgeLabelLayoutModel}.
     * <p>
     * It represents every possible location in relation to the source point of the edge. The radius describes the distance of
     * the label center from the source point. There is an angle <code>theta</code> which defines the location on the circle with the given
     * radius. It is the angle between the first segment of the underlying edge and the imaginary line to the center of the
     * label. Finally, there is a second angle that describes the rotation of the label box.
     * </p>
     * @see yfiles.layout.FreeEdgeLabelLayoutModelParameter#setPoint
     * @see yfiles.layout.FreeEdgeLabelLayoutModelParameter#angle
     * @class
     */
    export interface FreeEdgeLabelLayoutModelParameter extends Object{}
    export class FreeEdgeLabelLayoutModelParameter {
      /**
       * Creates a new instance of {@link yfiles.layout.FreeEdgeLabelLayoutModelParameter} with the given radius, angle and rotation
       * angle.
       * @param radius the distance of the label from the source node of the underlying edge
       * @param theta the angle in radians between the first segment of the underlying edge and the imaginary line to the center of the label
       * @param [angle=0] the rotation angle of the label
       */
      constructor(radius:number,theta:number,angle?:number);
      /**
       * Creates a new instance of {@link yfiles.layout.FreeEdgeLabelLayoutModelParameter} with radius <code>50</code> and angle <code>0</code>.
       */
      constructor();
      /**
       * Specifies a point that represents the location of the label's center in relation to the source point of the edge.
       * <p>
       * The x-coordinate corresponds to the radius which describes the distance of the label to the source point. The
       * y-coordinate corresponds to the angle between the first segment of the underlying edge and the imaginary line to the
       * center of the label.
       * </p>
       * @param radius the distance to the source point
       * @param theta the location on the radius
       */
      setPoint(radius:number,theta:number):void;
      /**
       * Gets or sets the rotation angle of the label that is placed using this model parameter.
       * @type {number}
       */
      angle:number;
      /**
       * Gets a point that represents the location of the label's center in relation to the source point of the edge.
       * <p>
       * The x-coordinate corresponds to the radius which describes the distance of the label to the source point. The
       * y-coordinate corresponds to the angle between the first segment of the underlying edge and the imaginary line to the
       * center of the label.
       * </p>
       * @type {yfiles.algorithms.YPoint}
       */
      point:yfiles.algorithms.YPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.FreeEdgeLabelLayoutModel} allows placing edge labels at any location.
     * @class
     * @implements {yfiles.layout.IEdgeLabelLayoutModel}
     */
    export interface FreeEdgeLabelLayoutModel extends Object,yfiles.layout.IEdgeLabelLayoutModel{}
    export class FreeEdgeLabelLayoutModel {
      /**
       * Creates a new instance of {@link yfiles.layout.FreeEdgeLabelLayoutModel}.
       */
      constructor();
      /**
       * Creates a model parameter that represents the given edge label position within this model.
       * <p>
       * The created model parameter represents the current location of the given label.
       * </p>
       * <p>
       * This parameter can be passed to {@link yfiles.layout.FreeEdgeLabelLayoutModel#getLabelPlacement} to retrieve the current label
       * box.
       * </p>
       * @param labelBounds the oriented box of the label the parameter represents
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @returns the model parameter representing the given label location
       * @throws {Stubs.Exceptions.ArgumentError} if one or more of the given bounds' values are invalid, i.e., <code>NaN</code>
       * @see yfiles.layout.FreeEdgeLabelLayoutModel#getLabelPlacement
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout):Object;
      /**
       * Returns the {@link yfiles.layout.EdgeLabelCandidate} that describes the current label position.
       * @param labelLayout the label for which candidates should be generated
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @returns a list of {@link } instances
       */
      getLabelCandidates(labelLayout:yfiles.layout.IEdgeLabelLayout,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * 
       * @param labelSize
       * @param edgeLayout
       * @param sourceNode
       * @param targetNode
       * @param param
       * @returns 
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,edgeLayout:yfiles.layout.IEdgeLayout,sourceNode:yfiles.layout.INodeLayout,targetNode:yfiles.layout.INodeLayout,param:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets the model parameter that encodes the default position of {@link yfiles.layout.FreeEdgeLabelLayoutModel}.
       * <p>
       * The distance of the label to the edge's source point will be <code>20</code>. The angle between the line to the center of the label
       * and the first segment is <code>20</code> radians. The label is not rotated.
       * </p>
       * <p>
       * This parameter can be passed to {@link yfiles.layout.FreeEdgeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
       * label box.
       * </p>
       * @type {Object}
       */
      defaultParameter:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * The model parameter that encodes a label position within {@link yfiles.layout.FreeNodeLabelLayoutModel}.
     * <p>
     * It represents every possible location in relation to the node. The point describes the offset of the label's center to
     * the upper-left corner of the node.
     * </p>
     * @see yfiles.layout.FreeNodeLabelLayoutModelParameter#setPoint
     * @class
     */
    export interface FreeNodeLabelLayoutModelParameter extends Object{}
    export class FreeNodeLabelLayoutModelParameter {
      /**
       * Creates a new instance of {@link yfiles.layout.FreeNodeLabelLayoutModelParameter} with the given offset.
       * @param dx the horizontal offset
       * @param dy the vertical offset
       */
      constructor(dx:number,dy:number);
      /**
       * Creates a new instance of {@link yfiles.layout.FreeNodeLabelLayoutModelParameter}.
       * <p>
       * It has no offset, i.e., the upper-left corners of both node label and node are at the same location.
       * </p>
       */
      constructor();
      /**
       * Specifies the offset of the label's center to the upper-left corner of the node.
       * @param dx the horizontal offset
       * @param dy the vertical offset
       */
      setPoint(dx:number,dy:number):void;
      /**
       * Gets the offset of the label's center to the upper-left corner of the node.
       * @type {yfiles.algorithms.YPoint}
       */
      point:yfiles.algorithms.YPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.FreeNodeLabelLayoutModel} allows placing node labels at any desired position.
     * <p>
     * The location of the label does not need to satisfy any constraints. The {@link yfiles.layout.FreeNodeLabelLayoutModel#createModelParameter model parameter} describes a fixed offset between
     * the upper-left corner of the node and the center of the label.
     * </p>
     * @class
     * @implements {yfiles.layout.INodeLabelLayoutModel}
     */
    export interface FreeNodeLabelLayoutModel extends Object,yfiles.layout.INodeLabelLayoutModel{}
    export class FreeNodeLabelLayoutModel {
      /**
       * Returns a new instance of {@link yfiles.layout.FreeNodeLabelLayoutModel}.
       */
      constructor();
      /**
       * Creates a model parameter that represents the given node label position within this model.
       * <p>
       * The created model parameter represents the current location of the given label.
       * </p>
       * <p>
       * This parameter can be passed to {@link yfiles.layout.FreeNodeLabelLayoutModel#getLabelPlacement} to retrieve the current label
       * box.
       * </p>
       * @param labelBounds the bounds of the label the parameter represents
       * @param nodeLayout the layout of the node to which the label belongs
       * @returns the model parameter
       * @throws {Stubs.Exceptions.ArgumentError} if one or more of the given bounds' values are invalid, i.e., <code>NaN</code>
       * @see yfiles.layout.FreeNodeLabelLayoutModel#getLabelPlacement
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,nodeLayout:yfiles.layout.INodeLayout):Object;
      /**
       * Returns the {@link yfiles.layout.NodeLabelCandidate} that describes the current label position.
       * @param labelLayout the label for which candidates should be generated
       * @param nodeLayout the layout of the node to which the label belongs
       * @returns a list of {@link } instances
       */
      getLabelCandidates(labelLayout:yfiles.layout.INodeLabelLayout,nodeLayout:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * 
       * @param labelSize
       * @param nodeLayout
       * @param param
       * @returns 
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,nodeLayout:yfiles.layout.INodeLayout,param:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets a model parameter that encodes the default position of {@link yfiles.layout.INodeLabelLayoutModel}.
       * <p>
       * The node label's upper-left corner is placed <code>20</code> pixels to the left and <code>20</code> pixels above the node.
       * </p>
       * <p>
       * This parameter can be passed to {@link yfiles.layout.FreeNodeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
       * label box.
       * </p>
       * @type {Object}
       */
      defaultParameter:Object;
      static $class:yfiles.lang.Class;
    }
    export enum ComponentArrangementStyles{
      /**
       * Style specifier describing no special component arrangement at all. Components will be centered at the same position
       * they resided at before the layout started. If combined with the style modifier
       * {@link yfiles.layout.ComponentArrangementStyles#MODIFIER_NO_OVERLAP}, components might get moved so that they don't overlap
       * after the layout.
       * @see yfiles.layout.ComponentLayout#style
       */
      NONE,
      /**
       * Style specifier describing a component arrangement strategy that places components in multiple rows so that the overall
       * aspect ratio of the whole graph gets as close to the aspect ratio of the preferred layout size as possible.
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      ROWS,
      /**
       * Style specifier describing a component arrangement strategy that places all components next to each other in a single
       * row. If combined with the style modifier
       * {@link yfiles.layout.ComponentArrangementStyles#MODIFIER_AS_IS}, components will be placed in the same order as they were placed
       * before the layout.
       * @see yfiles.layout.ComponentLayout#style
       */
      SINGLE_ROW,
      /**
       * Style specifier describing a component arrangement strategy that places components above each other in a single column.
       * If combined with the style modifier
       * {@link yfiles.layout.ComponentArrangementStyles#MODIFIER_AS_IS}, components will be placed in the same order as they were placed
       * before the layout.
       * @see yfiles.layout.ComponentLayout#style
       */
      SINGLE_COLUMN,
      /**
       * Style specifier describing a component arrangement strategy that places components in a rectangular cloud around the
       * biggest component. Entities of different components will not overlap, however the bounding boxes of the components may
       * overlap.
       * <p>
       * The aspect ratio of the rectangle is defined by the {@link yfiles.layout.ComponentLayout#preferredSize preferred size} of the graph.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      PACKED_RECTANGLE,
      /**
       * Style specifier describing a component arrangement strategy that places components in a rectangular cloud around the
       * biggest component. Entities of different components will not overlap, however the bounding boxes of the components may
       * overlap.
       * <p>
       * In contrast to
       * {@link yfiles.layout.ComponentArrangementStyles#PACKED_RECTANGLE}, components might even be placed in empty spaces inside other
       * components.
       * </p>
       * <p>
       * The aspect ratio of the rectangle is defined by the {@link yfiles.layout.ComponentLayout#preferredSize preferred size} of the graph.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      PACKED_COMPACT_RECTANGLE,
      /**
       * Style specifier describing a component arrangement strategy that places components in a circular cloud around the
       * biggest component. Entities of different components will not overlap, however the bounding boxes of the components may
       * overlap.
       * @see yfiles.layout.ComponentLayout#style
       */
      PACKED_CIRCLE,
      /**
       * Style specifier describing a component arrangement strategy that places components in a circular compact cloud around
       * the biggest component. Entities of different components will not overlap, however the bounding boxes of the components
       * may overlap.
       * <p>
       * In contrast to
       * {@link yfiles.layout.ComponentArrangementStyles#PACKED_CIRCLE}, components might even be placed in empty spaces inside other
       * components.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       */
      PACKED_COMPACT_CIRCLE,
      /**
       * Style specifier describing a component arrangement strategy that places components in multiple rows.
       * <p>
       * The strategy tries to arrange the components such that the aspect ratio of the final component placement gets as close
       * as possible to the aspect ratio of the preferred layout size, configurable using {@link yfiles.layout.ComponentLayout#preferredSize}.
       * </p>
       * <p>
       * In contrast to
       * {@link yfiles.layout.ComponentArrangementStyles#ROWS}, each row may also be divided into further sub-rows, which is especially
       * useful if there are few very large components and many significantly smaller ones.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      MULTI_ROWS,
      /**
       * Style specifier describing a component arrangement strategy that places components in multiple rows.
       * <p>
       * The strategy tries to arrange the components such that the aspect ratio of the final component placement gets close to
       * the aspect ratio of the preferred layout size, configurable using {@link yfiles.layout.ComponentLayout#preferredSize}. But in contrast to
       * {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS}, this strategy does not always try to come as close to the preferred
       * view ratio, if a ratio close to the preferred ratio only induces a lot of unused view space. This unused space will be
       * minimized.
       * </p>
       * <p>
       * In contrast to
       * {@link yfiles.layout.ComponentArrangementStyles#ROWS}, each row may also be divided into further sub-rows, which is especially
       * useful if there are few very large components and many significantly smaller ones.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      MULTI_ROWS_COMPACT,
      /**
       * Style specifier describing a component arrangement strategy that places the components in {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS}
       * with a restricted height. The resulting height does not exceed the preferred height ({@link yfiles.layout.ComponentLayout#preferredSize}).
       * <p>
       * In contrast to
       * {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS_HEIGHT_CONSTRAINT_COMPACT}, the result will be an arrangement which
       * uses as much height as possible, even if that means that all components will be arranged in one single column.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      MULTI_ROWS_HEIGHT_CONSTRAINT,
      /**
       * Style specifier describing a component arrangement strategy that places the components in {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS}
       * with a restricted height and as compact as possible. The resulting height does not exceed the preferred height ({@link yfiles.layout.ComponentLayout#preferredSize}).
       * <p>
       * In comparison with
       * {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS_HEIGHT_CONSTRAINT}, the result will be compacted, which means that an
       * arrangement with the minimum width such that the height constraint is still fulfilled will be realized.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      MULTI_ROWS_HEIGHT_CONSTRAINT_COMPACT,
      /**
       * Style specifier describing a component arrangement strategy that places the components in {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS}
       * with a restricted width. The resulting width does not exceed the preferred width ({@link yfiles.layout.ComponentLayout#preferredSize}).
       * <p>
       * In contrast to {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS_WIDTH_CONSTRAINT_COMPACT} the result will be an
       * arrangement which uses as much space in width as possible, even if that means that all components will be arranged in a
       * single row.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      MULTI_ROWS_WIDTH_CONSTRAINT,
      /**
       * Style specifier describing a component arrangement strategy that places the components in {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS}
       * with a restricted width and as compact as possible. The resulting width does not exceed the preferred width ({@link yfiles.layout.ComponentLayout#preferredSize}).
       * <p>
       * In comparison with {@link yfiles.layout.ComponentArrangementStyles#MULTI_ROWS_HEIGHT_CONSTRAINT} the result will be compacted,
       * in order to minimize unused view space induced by components that are large in height.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       * @see yfiles.layout.ComponentLayout#preferredSize
       */
      MULTI_ROWS_WIDTH_CONSTRAINT_COMPACT,
      /**
       * A constant for masking actual style constants from style modifiers.
       * @see yfiles.layout.ComponentLayout#style
       */
      MASK,
      /**
       * A modifier that can be added to normal style specifiers to indicate that the arrangement strategy should take the
       * initial positions of the components into account.
       * <p>
       * Combining this modifier is supported with the following styles: {@link yfiles.layout.ComponentArrangementStyles#SINGLE_ROW},
       * {@link yfiles.layout.ComponentArrangementStyles#SINGLE_COLUMN}, {@link yfiles.layout.ComponentArrangementStyles#PACKED_RECTANGLE},
       * {@link yfiles.layout.ComponentArrangementStyles#PACKED_COMPACT_RECTANGLE}, {@link yfiles.layout.ComponentArrangementStyles#PACKED_CIRCLE}
       * and {@link yfiles.layout.ComponentArrangementStyles#PACKED_COMPACT_CIRCLE}.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       */
      MODIFIER_AS_IS,
      /**
       * A modifier that can be added to normal style specifiers to indicate that the arrangement strategy should produce a
       * non-overlapping component layout.
       * <p>
       * Combining this modifier is not supported with {@link yfiles.layout.ComponentArrangementStyles#NONE}.
       * </p>
       * @see yfiles.layout.ComponentLayout#style
       */
      MODIFIER_NO_OVERLAP
    }
    export enum DiscreteEdgeLabelPositions{
      /**
       * Position specifier that describes a label placement near the source, left/above the edge's path.
       */
      SOURCE_HEAD,
      /**
       * Position specifier that describes a label placement near the middle of the edge, left/above the edge's path.
       */
      HEAD,
      /**
       * Position specifier that describes a label placement near the target, left/above the edge's path.
       */
      TARGET_HEAD,
      /**
       * Position specifier that describes a label placement near the source, right/below the edge's path.
       */
      SOURCE_TAIL,
      /**
       * Position specifier that describes a label placement near the middle of the edge, right/below the edge's path.
       */
      TAIL,
      /**
       * Position specifier that describes a label placement near the target, right/below the edge's path.
       */
      TARGET_TAIL,
      /**
       * Position specifier that describes a label placement near the source, directly on the edge's path.
       */
      SOURCE_CENTER,
      /**
       * Position specifier that describes a label placement near the middle of the edge, directly on the edge's path.
       */
      CENTER,
      /**
       * Position specifier that describes a label placement near the target, directly on the edge's path.
       */
      TARGET_CENTER,
      /**
       * Position mask that constrains the allowed positions to the two locations near the middle of the edge beside the edge's
       * path.
       * @see yfiles.layout.DiscreteEdgeLabelPositions#HEAD
       * @see yfiles.layout.DiscreteEdgeLabelPositions#TAIL
       */
      TWO_POS,
      /**
       * Position mask that constrains the allowed positions to the {@link yfiles.layout.DiscreteEdgeLabelPositions#CENTER} of the edge's
       * path.
       * @see yfiles.layout.DiscreteEdgeLabelPositions#CENTER
       */
      CENTERED,
      /**
       * Position mask that constrains the allowed positions to a set of six positions beside the edge path.
       * <p>
       * Two of the positions are near the source and two others are near the target. The remaining positions are in the middle
       * of the edge.
       * </p>
       * @see yfiles.layout.DiscreteEdgeLabelPositions#SOURCE_HEAD
       * @see yfiles.layout.DiscreteEdgeLabelPositions#SOURCE_TAIL
       * @see yfiles.layout.DiscreteEdgeLabelPositions#HEAD
       * @see yfiles.layout.DiscreteEdgeLabelPositions#TAIL
       * @see yfiles.layout.DiscreteEdgeLabelPositions#TARGET_HEAD
       * @see yfiles.layout.DiscreteEdgeLabelPositions#TARGET_TAIL
       */
      SIX_POS,
      /**
       * Position mask that constrains the allowed positions to a set of three positions directly on the edge's path.
       * @see yfiles.layout.DiscreteEdgeLabelPositions#SOURCE_CENTER
       * @see yfiles.layout.DiscreteEdgeLabelPositions#CENTER
       * @see yfiles.layout.DiscreteEdgeLabelPositions#TARGET_CENTER
       */
      THREE_CENTER
    }
    export enum DiscreteNodeLabelPositions{
      /**
       * Position specifier that describes a label placement north of the node.
       */
      NORTH,
      /**
       * Position specifier that describes a label placement north-west of the node.
       */
      NORTH_WEST,
      /**
       * Position specifier that describes a label placement north-east of the node.
       */
      NORTH_EAST,
      /**
       * Position specifier that describes a label placement east of the node.
       */
      EAST,
      /**
       * Position specifier that describes a label placement west of the node.
       */
      WEST,
      /**
       * Position specifier that describes a label placement south of the node.
       */
      SOUTH,
      /**
       * Position specifier that describes a label placement south-west of the node.
       */
      SOUTH_WEST,
      /**
       * Position specifier that describes a label placement south-east of the node.
       */
      SOUTH_EAST,
      /**
       * Position specifier that describes a label placement at the center of the node.
       */
      CENTER,
      /**
       * Position specifier that describes a label placement at the bottom inside the node.
       */
      BOTTOM,
      /**
       * Position specifier that describes a label placement at the top inside the node.
       */
      TOP,
      /**
       * Position specifier that describes a label placement at the left inside the node.
       */
      LEFT,
      /**
       * Position specifier that describes a label placement at the right inside the node.
       */
      RIGHT,
      /**
       * Position specifier that describes a label placement at the top-left inside the node.
       */
      TOP_LEFT,
      /**
       * Position specifier that describes a label placement at the top-right inside the node.
       */
      TOP_RIGHT,
      /**
       * Position specifier that describes a label placement at the bottom-left inside the node.
       */
      BOTTOM_LEFT,
      /**
       * Position specifier that describes a label placement at the bottom-right inside the node.
       */
      BOTTOM_RIGHT,
      /**
       * Position mask that constrains the positions to {@link yfiles.layout.DiscreteNodeLabelPositions#NORTH above} and {@link yfiles.layout.DiscreteNodeLabelPositions#SOUTH below} the node.
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH
       */
      SANDWICH_MASK,
      /**
       * Position mask that constrains the positions to the nine node-internal positions.
       * @see yfiles.layout.DiscreteNodeLabelPositions#CENTER
       * @see yfiles.layout.DiscreteNodeLabelPositions#TOP
       * @see yfiles.layout.DiscreteNodeLabelPositions#BOTTOM
       * @see yfiles.layout.DiscreteNodeLabelPositions#LEFT
       * @see yfiles.layout.DiscreteNodeLabelPositions#RIGHT
       * @see yfiles.layout.DiscreteNodeLabelPositions#TOP_LEFT
       * @see yfiles.layout.DiscreteNodeLabelPositions#TOP_RIGHT
       * @see yfiles.layout.DiscreteNodeLabelPositions#BOTTOM_LEFT
       * @see yfiles.layout.DiscreteNodeLabelPositions#BOTTOM_RIGHT
       */
      INTERNAL_MASK,
      /**
       * Position mask that constrains the positions to the node's four sides.
       * @see yfiles.layout.DiscreteNodeLabelPositions#EAST
       * @see yfiles.layout.DiscreteNodeLabelPositions#WEST
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH
       */
      SIDES_MASK,
      /**
       * Position mask that constrains the positions to the corners outside the node.
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH_EAST
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH_WEST
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH_EAST
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH_WEST
       */
      CORNER_MASK,
      /**
       * Position mask that constraints the positions to the eight node-external positions.
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH
       * @see yfiles.layout.DiscreteNodeLabelPositions#EAST
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH
       * @see yfiles.layout.DiscreteNodeLabelPositions#WEST
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH_EAST
       * @see yfiles.layout.DiscreteNodeLabelPositions#NORTH_WEST
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH_EAST
       * @see yfiles.layout.DiscreteNodeLabelPositions#SOUTH_WEST
       */
      EIGHT_POS_MASK
    }
    export enum FixPointPolicy{
      /**
       * Fix point calculation policy specifying that the <em>center</em> of a rectangle containing all fixed nodes is considered the fix
       * point.
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       */
      CENTER,
      /**
       * Fix point calculation policy specifying that the <em>upper left corner</em> of a rectangle containing all fixed nodes is considered the fix
       * point.
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       */
      UPPER_LEFT,
      /**
       * Fix point calculation policy specifying that the <em>upper right corner</em> of a rectangle containing all fixed nodes is considered the fix
       * point.
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       */
      UPPER_RIGHT,
      /**
       * Fix point calculation policy specifying that the <em>lower right corner</em> of a rectangle containing all fixed nodes is considered the fix
       * point.
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       */
      LOWER_RIGHT,
      /**
       * Fix point calculation policy specifying that the <em>lower left corner</em> of a rectangle containing all fixed nodes is considered the fix
       * point.
       * @see yfiles.layout.FixNodeLayoutStage#fixPointPolicy
       */
      LOWER_LEFT
    }
    export enum LabelPlacements{
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed near the source node of an
       * edge.
       */
      AT_SOURCE,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed near the target node of an
       * edge.
       */
      AT_TARGET,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed near the center of the
       * edge path.
       */
      AT_CENTER,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed directly at the source
       * port of the edge.
       */
      AT_SOURCE_PORT,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed directly at the target
       * port of the edge.
       */
      AT_TARGET_PORT,
      /**
       * Placement specifier mask that masks all placement specifiers that describe the placement along the edge.
       */
      ALONG_EDGE_MASK,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed on the path of the edge.
       */
      ON_EDGE,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed on the left hand side of
       * the edge path.
       * <p>
       * The side is interpreted relative to the direction of the edge, if no different {@link yfiles.layout.PreferredPlacementDescriptor#sideReference side reference} is defined.
       * </p>
       */
      LEFT_OF_EDGE,
      /**
       * Preferred placement specifier for edge labels which expresses that the label should be placed on the right hand side of
       * the edge path.
       * <p>
       * The side is interpreted relative to the direction of the edge, if no different {@link yfiles.layout.PreferredPlacementDescriptor#sideReference side reference} is defined.
       * </p>
       */
      RIGHT_OF_EDGE,
      /**
       * Placement specifier mask that masks all placement specifiers that describe the side of the edge.
       */
      ON_SIDE_OF_EDGE_MASK,
      /**
       * Preferred placement specifier for edge labels which expresses that the label can be placed anywhere along the edge or on
       * any side of the edge.
       */
      ANYWHERE
    }
    export enum LayoutOrientation{
      /**
       * Orientation specifier which defines that the main layout orientation is from top to bottom. This is the default for all
       * layout algorithms.
       */
      TOP_TO_BOTTOM,
      /**
       * Orientation specifier which defines that the main layout orientation is from left to right. The layout will be rotated
       * by <code>90</code> degrees counterclockwise.
       */
      LEFT_TO_RIGHT,
      /**
       * Orientation specifier which defines that the main layout orientation is from right to left. The layout will be rotated
       * by <code>90</code> degrees clockwise.
       */
      RIGHT_TO_LEFT,
      /**
       * Orientation specifier which defines that the main layout orientation is from bottom to top. The layout will be flipped
       * along the x-axis.
       */
      BOTTOM_TO_TOP
    }
    export enum RowAlignment{
      /**
       * Row alignment specifier which defines that each row will be aligned with its left border.
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleRows
       */
      LEADING,
      /**
       * Row alignment specifier which defines that each row will be aligned with its right border.
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleRows
       */
      TRAILING,
      /**
       * Row alignment specifier which defines that each row will be aligned with its left border and its right border.
       * <p>
       * Rectangles in smaller rows will be distributed over the row, so that the row aligns with both sides.
       * </p>
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleRows
       */
      JUSTIFIED,
      /**
       * Row alignment specifier which defines that each row will be aligned with its center.
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleRows
       */
      CENTER
    }
    export enum MultiRowConstraint{
      /**
       * Constraint specifier for a multi-row rectangle arrangement that is unconstrained. Both, height and width, are not
       * limited in size. The rectangles will be arranged in a way that the view ratio defined by the preferred height and width
       * will be achieved.
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleMultiRows
       */
      NONE,
      /**
       * Constraint specifier for a multi-row rectangle arrangement that constrains the height of the layout. Limits the height
       * of the arrangement to the given preferred height value. If some rectangles are higher than allowed, then they are placed
       * nevertheless, but the constraint will still be considered for the rest and not weakened to the height of the higher
       * rectangles. Having one rectangle being higher than the constraint does always result in arrangements consisting only of
       * a single top-level row.
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleMultiRows
       */
      HEIGHT,
      /**
       * Constraint specifier for a multi-row rectangle arrangement that constrains the width of the layout. Limits the width of
       * the arrangement to the given preferred width value. If a rectangle is wider than allowed, then it will nevertheless be
       * placed (in a row containing the rectangle alone), but the constraint will be still considered for other rows and not
       * weakened.
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleMultiRows
       */
      WIDTH
    }
    export enum EdgeFlow{
      /**
       * Edge flow specifier which defines that the flow direction of most edges is up.
       * @see yfiles.layout.LayoutGraphUtilities#determineEdgeFlowDirection
       */
      UP,
      /**
       * Edge flow specifier which defines that the flow direction of most edges is down.
       * @see yfiles.layout.LayoutGraphUtilities#determineEdgeFlowDirection
       */
      DOWN,
      /**
       * Edge flow specifier which defines that the flow direction of most edges is left.
       * @see yfiles.layout.LayoutGraphUtilities#determineEdgeFlowDirection
       */
      LEFT,
      /**
       * Edge flow specifier which defines that the flow direction of most edges is right.
       * @see yfiles.layout.LayoutGraphUtilities#determineEdgeFlowDirection
       */
      RIGHT,
      /**
       * Edge flow specifier that defines that there is no main flow direction.
       * @see yfiles.layout.LayoutGraphUtilities#determineEdgeFlowDirection
       */
      NONE
    }
    export enum NodeAlignment{
      /**
       * Node alignment policy that results in top-aligned nodes for vertical alignment and left-aligned nodes for horizontal
       * alignments.
       * @see yfiles.layout.LayoutGraphUtilities#alignNodeLayouts
       */
      LEADING,
      /**
       * Node alignment policy that results in center-aligned nodes.
       * @see yfiles.layout.LayoutGraphUtilities#alignNodeLayouts
       */
      CENTERED,
      /**
       * Node alignment policy that results in bottom-aligned nodes for vertical alignment and right-aligned nodes for horizontal
       * alignments.
       * @see yfiles.layout.LayoutGraphUtilities#alignNodeLayouts
       */
      TRAILING,
      /**
       * Node alignment policy that results in vertically or horizontally distributed nodes. In this context, <em>distributed</em> means that the
       * vertical (or horizontal) extensions of nodes do not overlap, independent of their horizontal (or vertical) position.
       * @see yfiles.layout.LayoutGraphUtilities#alignNodeLayouts
       */
      DISTRIBUTED
    }
    export enum MirrorModes{
      /**
       * Mirroring specifier that can be used for preventing any mirroring of layout orientations.
       * @see yfiles.layout.OrientationLayout#mirrorMode
       * @see yfiles.layout.OrientationLayout#orientation
       */
      NONE,
      /**
       * Mirroring specifier that defines that layout orientation {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM} will be
       * mirrored.
       * <p>
       * This specifier can be used for creating a mirror mask that defines which layout orientations shall be mirrored at their
       * corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
       * </p>
       * @see yfiles.layout.OrientationLayout#mirrorMode
       * @see yfiles.layout.OrientationLayout#orientation
       */
      TOP_TO_BOTTOM,
      /**
       * Mirroring specifier that defines that layout orientation {@link yfiles.layout.LayoutOrientation#RIGHT_TO_LEFT} will be
       * mirrored.
       * <p>
       * This specifier can be used for creating a mirror mask that defines which layout orientations shall be mirrored at their
       * corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
       * </p>
       * @see yfiles.layout.OrientationLayout#mirrorMode
       * @see yfiles.layout.OrientationLayout#orientation
       */
      RIGHT_TO_LEFT,
      /**
       * Mirroring specifier that defines that layout orientation {@link yfiles.layout.LayoutOrientation#BOTTOM_TO_TOP} will be
       * mirrored.
       * <p>
       * This specifier can be used for creating a mirror mask that defines which layout orientations shall be mirrored at their
       * corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
       * </p>
       * @see yfiles.layout.OrientationLayout#mirrorMode
       * @see yfiles.layout.OrientationLayout#orientation
       */
      BOTTOM_TO_TOP,
      /**
       * Mirroring specifier that defines that layout orientation {@link yfiles.layout.LayoutOrientation#LEFT_TO_RIGHT} will be
       * mirrored.
       * <p>
       * This specifier can be used to create a mirror mask that defines which layout orientations shall be mirrored at their
       * corresponding axis. It can be combined with the other mirroring specifiers to mirror several orientations.
       * </p>
       * @see yfiles.layout.OrientationLayout#mirrorMode
       * @see yfiles.layout.OrientationLayout#orientation
       */
      LEFT_TO_RIGHT
    }
    export enum PortDirections{
      /**
       * A port direction specifier indicating a port on the north (i.e., top) side of a node.
       */
      NORTH,
      /**
       * A port direction specifier indicating a port on the east (i.e., right) side of a node.
       */
      EAST,
      /**
       * A port direction specifier indicating a port on the west (i.e., left) side of a node.
       */
      WEST,
      /**
       * A port direction specifier indicating a port on the south (i.e., bottom) side of a node.
       */
      SOUTH,
      /**
       * A port direction specifier indicating a port lying in the direction of the main flow of the edge or of the overall
       * layout.
       * <p>
       * The exact direction is domain-specific.
       * </p>
       */
      WITH_THE_FLOW,
      /**
       * A port direction specifier indicating a port lying in the opposite direction of the main flow of the edge or of the
       * overall layout.
       * <p>
       * The exact direction is domain-specific.
       * </p>
       */
      AGAINST_THE_FLOW,
      /**
       * A port direction specifier indicating a port lying left of the direction of the main flow of the edge or of the overall
       * layout.
       * <p>
       * The exact direction is domain-specific.
       * </p>
       */
      LEFT_IN_FLOW,
      /**
       * A port direction specifier indicating a port lying right of the direction of the main flow of the edge or of the overall
       * layout.
       * <p>
       * The exact direction is domain-specific.
       * </p>
       */
      RIGHT_IN_FLOW,
      /**
       * A combination of all possible direction specifiers indicating that edges can connect to any side of a node.
       */
      ANY
    }
    export enum PortSide{
      /**
       * A weak constraint indicating that edges can connect to any side of a node.
       * @see yfiles.layout.PortConstraint#create
       */
      ANY,
      /**
       * A weak constraint indicating that edges can connect to the north side (i.e. top side) of a node.
       * @see yfiles.layout.PortConstraint#create
       */
      NORTH,
      /**
       * A weak constraint indicating that edges can connect to the south side (i.e. bottom side) of a node.
       * @see yfiles.layout.PortConstraint#create
       */
      SOUTH,
      /**
       * A weak constraint indicating that edges can connect to the east side (i.e. right side) of a node.
       * @see yfiles.layout.PortConstraint#create
       */
      EAST,
      /**
       * A weak constraint indicating that edges can connect to the west side (i.e. left side) of a node.
       * @see yfiles.layout.PortConstraint#create
       */
      WEST
    }
    export enum LabelSideReferences{
      /**
       * Side placement specifier for edge labels which expresses that the preferred side of the edge is interpreted relative to
       * the edge flow.
       * @see yfiles.layout.PreferredPlacementDescriptor#sideReference
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      RELATIVE_TO_EDGE_FLOW,
      /**
       * Side placement specifier for edge labels which defines that the preferred side for the label is interpreted
       * independently of the edge direction placing {@link yfiles.layout.LabelPlacements#LEFT_OF_EDGE left labels} above horizontal segments.
       * <p>
       * For vertical segments the label is placed {@link yfiles.layout.LabelPlacements#LEFT_OF_EDGE left} or {@link yfiles.layout.LabelPlacements#RIGHT_OF_EDGE right} of the edge. In case the segment is horizontal, the label is
       * placed above the segment if it should be left of the edge, otherwise below the segment.
       * </p>
       * @see yfiles.layout.PreferredPlacementDescriptor#sideReference
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      ABSOLUTE_WITH_LEFT_IN_NORTH,
      /**
       * Side placement specifier for edge labels which defines that the preferred side for the label is interpreted
       * independently of the edge direction placing {@link yfiles.layout.LabelPlacements#RIGHT_OF_EDGE left labels} above horizontal segments.
       * <p>
       * For vertical segments the label is placed {@link yfiles.layout.LabelPlacements#LEFT_OF_EDGE left} or {@link yfiles.layout.LabelPlacements#RIGHT_OF_EDGE right} of the edge. In case the segment is horizontal, the label is
       * placed above the segment if it should be right of the edge, otherwise below the segment.
       * </p>
       * @see yfiles.layout.PreferredPlacementDescriptor#sideReference
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      ABSOLUTE_WITH_RIGHT_IN_NORTH
    }
    export enum LabelAngleReferences{
      /**
       * Angle interpretation specifier for edge labels which expresses that the rotation angle should be interpreted absolute
       * and not relative to the edge slope.
       * @see yfiles.layout.PreferredPlacementDescriptor#angleReference
       * @see yfiles.layout.PreferredPlacementDescriptor#angle
       */
      ABSOLUTE,
      /**
       * Angle interpretation specifier for edge labels which expresses that the rotation angle should be interpreted relative to
       * the edge slope.
       * @see yfiles.layout.PreferredPlacementDescriptor#angleReference
       * @see yfiles.layout.PreferredPlacementDescriptor#angle
       */
      RELATIVE_TO_EDGE_FLOW
    }
    export enum LabelAngleOnRightSideRotations{
      /**
       * Angle interpretation specifier which expresses that the rotation angle should be interpreted <em>clockwise</em>.
       * <p>
       * Therefore, edge labels whose descriptor {@link yfiles.layout.PreferredPlacementDescriptor#isRightOfEdge} will be co-rotating with the angle of labels that are placed left of or
       * centered on the edge.
       * </p>
       * @see yfiles.layout.PreferredPlacementDescriptor#angleRotationOnRightSide
       * @see yfiles.layout.PreferredPlacementDescriptor#angle
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      CLOCKWISE,
      /**
       * Angle interpretation specifier which expresses that the rotation angle should be interpreted <em>counter-clockwise</em>.
       * <p>
       * Therefore, edge labels whose descriptor {@link yfiles.layout.PreferredPlacementDescriptor#isRightOfEdge} will be counter-rotating with the angle of labels that are placed left of or
       * centered on the edge.
       * </p>
       * @see yfiles.layout.PreferredPlacementDescriptor#angleRotationOnRightSide
       * @see yfiles.layout.PreferredPlacementDescriptor#angle
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      COUNTER_CLOCKWISE
    }
    export enum LabelAngleOnRightSideOffsets{
      /**
       * Angle offset specifier for edge labels which expresses that no additional offset should be added to the angle.
       * @see yfiles.layout.PreferredPlacementDescriptor#angleOffsetOnRightSide
       * @see yfiles.layout.PreferredPlacementDescriptor#angle
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      NONE,
      /**
       * Angle offset specifier for edge labels which expresses that an additional offset of <code>180</code> degrees should be added to the
       * angle. That way, point-symmetric label placements left and right of the edge can be attained.
       * @see yfiles.layout.PreferredPlacementDescriptor#angleOffsetOnRightSide
       * @see yfiles.layout.PreferredPlacementDescriptor#angle
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       */
      SEMI
    }
    export enum SliderMode{
      /**
       * Slider mode specifier which describes continuous label positions directly on the edge path.
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#mode
       */
      CENTER,
      /**
       * Slider mode specifier which describes continuous label positions along the sides of the edge path.
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#mode
       */
      SIDE,
      /**
       * Slider mode specifier which describes continuous label positions along one side of the edge path.
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#mode
       */
      SINGLE_SIDE
    }
    export enum SelfLoopStyle{
      /**
       * Self-loop style specifier which defines that the routes of self-loops are orthogonal.
       * <p>
       * The routes consist of an alternating sequence of horizontal and vertical line segments.
       * </p>
       * @see yfiles.layout.SelfLoopRouter#layoutStyle
       */
      ORTHOGONAL,
      /**
       * Self-loop style specifier which defines that the routes of self-loops are rounded.
       * <p>
       * The {@link yfiles.layout.SelfLoopRouter#lineDistance distance between incident self-loops} and {@link yfiles.layout.SelfLoopRouter#cornerApproximationPointsCount rounding of the corners} can be customized for rounded self-loops.
       * </p>
       * @see yfiles.layout.SelfLoopRouter#layoutStyle
       * @see yfiles.layout.SelfLoopRouter#lineDistance
       * @see yfiles.layout.SelfLoopRouter#cornerApproximationPointsCount
       */
      ROUNDED
    }
    export enum SwimlanesMode{
      /**
       * Swimlane ordering mode specifier defining that node grouping is ignored during the swimlane ordering optimization.
       * @see yfiles.layout.Swimlanes#arrangeSwimlanes
       */
      IGNORE_GROUPS,
      /**
       * Swimlane ordering mode specifier defining that only node grouping is considered during the swimlane ordering
       * optimization.
       * @see yfiles.layout.Swimlanes#arrangeSwimlanes
       */
      ONLY_GROUPS,
      /**
       * Swimlane ordering mode specifier defining that both node grouping and edge length are considered during the swimlane
       * ordering optimization.
       * @see yfiles.layout.Swimlanes#arrangeSwimlanes
       */
      MIXED
    }
    /**
     * {@link yfiles.layout.GraphLayoutLineWrapper} can line-wrap or column-wrap a graph layout.
     * <p>
     * This {@link yfiles.layout.ILayoutStage} supports both wrapping a layout at a given width (height) and wrapping a layout
     * automatically at some width (height) such that the resulting overall aspect ratio is close to a desired {@link yfiles.layout.GraphLayoutLineWrapper#targetRatio target ratio}.
     * </p>
     * <p>
     * The {@link yfiles.layout.GraphLayoutLineWrapper} can be used on an existing graph layout or on the result of the
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface GraphLayoutLineWrapper extends yfiles.layout.LayoutStageBase{}
    export class GraphLayoutLineWrapper {
      /**
       * Creates a new {@link yfiles.layout.GraphLayoutLineWrapper} instance with default settings.
       */
      constructor();
      /**
       * Gets or sets the space that should be kept between edges.
       * <p>
       * The spacing must be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given spacing is a negative value
       * @type {number}
       */
      edgeSpacing:number;
      /**
       * Gets or sets whether this {@link yfiles.layout.ILayoutStage} wraps lines/rows or columns.
       * <p>
       * If this mode is enabled, the layout is wrapped vertically and new columns are appended to the right of the first column.
       * Otherwise, the layout is wrapped horizontally and new rows are appended below the first row.
       * </p>
       * <p>
       * Also, the {@link yfiles.layout.GraphLayoutLineWrapper#fixedWidth} property will be interpreted as a fixed height if this mode is enabled.
       * </p>
       * @see yfiles.layout.GraphLayoutLineWrapper#fixedWidth
       * @type {boolean}
       */
      columnMode:boolean;
      /**
       * Gets or sets the space between adjacent lines of the wrapped graph layout.
       * <p>
       * If the {@link yfiles.layout.GraphLayoutLineWrapper#columnMode column mode} is enabled, this spacing defines the distance between adjacent columns.
       * </p>
       * <p>
       * The spacing should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the spacing is negative
       * @type {number}
       */
      spacing:number;
      /**
       * Gets or sets whether or not every other line is mirrored.
       * <p>
       * When breaking a line and placing the remaining line below, this remainder can be rotated by <code>180</code> degrees. Then, the edges
       * connecting both lines
       * <em>stay short</em>. In case the remainder is not rotated, the edges are routed between the two lines to connect to the front of the
       * non-mirrored line.
       * </p>
       * @type {boolean}
       */
      mirror:boolean;
      /**
       * Gets or sets whether the algorithm should use the {@link yfiles.layout.GraphLayoutLineWrapper#fixedWidth fixed width} or {@link yfiles.layout.GraphLayoutLineWrapper#targetRatio target aspect ratio} to determine line breaks.
       * @see yfiles.layout.GraphLayoutLineWrapper#fixedWidth
       * @see yfiles.layout.GraphLayoutLineWrapper#targetRatio
       * @type {boolean}
       */
      fixedWidthLineBreaks:boolean;
      /**
       * Gets or sets the desired target aspect ratio for the layout.
       * <p>
       * The aspect ratio needs to be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the target ratio is <code>0</code> or less
       * @see yfiles.layout.GraphLayoutLineWrapper#fixedWidthLineBreaks
       * @type {number}
       */
      targetRatio:number;
      /**
       * Gets or sets the desired width of the lines.
       * <p>
       * The {@link yfiles.layout.GraphLayoutLineWrapper} breaks the lines at the specified width.
       * </p>
       * <p>
       * In {@link yfiles.layout.GraphLayoutLineWrapper#columnMode column mode}, the fixed width is interpreted as fixed height for a column.
       * </p>
       * @see yfiles.layout.GraphLayoutLineWrapper#fixedWidthLineBreaks
       * @see yfiles.layout.GraphLayoutLineWrapper#columnMode
       * @type {number}
       */
      fixedWidth:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a column of a {@link yfiles.layout.PartitionGrid partition grid} structure.
     * <p>
     * The results of the calculation of the geometry of the column will be placed into the instances of this class after the
     * layout.
     * </p>
     * @see yfiles.layout.PartitionGrid
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface ColumnDescriptor extends Object,yfiles.lang.IComparable{}
    export class ColumnDescriptor {
      /**
       * Compares this {@link yfiles.layout.ColumnDescriptor} instance with a given other {@link yfiles.layout.ColumnDescriptor} instance based
       * on the indices of the two instances.
       * @param o the {@link } to compare to
       * @returns <code>-1</code>, <code>0</code> or <code>1</code> if this {@link } is less than, equal to, or greater than the given other
       * {@link }
       */
      compareTo(o:Object):number;
      /**
       * Gets the index of the column within the {@link yfiles.layout.PartitionGrid partition grid}.
       * @type {number}
       */
      index:number;
      /**
       * Gets or sets the minimum width of the column.
       * <p>
       * The minimum width is defined to be at least <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum width is negative
       * @type {number}
       */
      minimumWidth:number;
      /**
       * Gets or sets the left column inset in which no element will lie in the resulting layout.
       * <p>
       * The column inset must be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the left inset is negative
       * @type {number}
       */
      leftInset:number;
      /**
       * Gets or sets the right column inset in which no element will lie in the resulting layout.
       * <p>
       * The column inset must be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the right inset is negative
       * @type {number}
       */
      rightInset:number;
      /**
       * Gets or sets the computed width of the column after the layout has been calculated.
       * <p>
       * The computed width needs to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the computed width is negative
       * @type {number}
       */
      computedWidth:number;
      /**
       * Gets or sets the original width of the column.
       * <p>
       * The original width should be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the original width is negative
       * @type {number}
       */
      originalWidth:number;
      /**
       * Gets or sets the original position (smallest x-coordinate) of the column.
       * @type {number}
       */
      originalPosition:number;
      /**
       * Gets or sets the computed position (smallest x-coordinate) of the column after the layout has been calculated.
       * @type {number}
       */
      computedPosition:number;
      /**
       * Gets or sets the tightness factor of this column.
       * <p>
       * This value must lie within the interval <code>[0,1]</code>.
       * </p>
       * <p>
       * The larger the value, the more the column will be forced to obtain its minimum width:
       * </p>
       * <ul>
       * <li>a value of <code>0.0d</code> will disable the compression of the column.</li>
       * <li>a value of <code>1.0d</code> will try to force the column to obtain its specified {@link yfiles.layout.ColumnDescriptor#minimumWidth minimum width}.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if the tightness value is outside the interval <code>[0,1]</code>
       * @type {number}
       */
      tightness:number;
      /**
       * Gets or sets whether the index of the column is fixed or it should be chosen automatically in order to minimize edge
       * lengths.
       * <p>
       * For all columns where this property is enabled, the relative ordering given by the indices is preserved. The remaining
       * columns may be sorted again so that the overall edge lengths are minimized.
       * </p>
       * @type {boolean}
       */
      indexFixed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a single partition cell, i.e., a pair consisting of a row and a column.
     * @class
     * @implements {yfiles.lang.IComparable}
     * @final
     */
    export interface PartitionCellIdEntry extends Object,yfiles.lang.IComparable{}
    export class PartitionCellIdEntry {
      /**
       * Compares this {@link yfiles.layout.PartitionCellIdEntry} instance with a given other {@link yfiles.layout.PartitionCellIdEntry}
       * instance.
       * <p>
       * The comparison is based on the indices of the {@link yfiles.layout.PartitionCellIdEntry#row rows} and {@link yfiles.layout.PartitionCellIdEntry#column columns} of the two pairs. A {@link yfiles.layout.PartitionCellIdEntry} is
       * considered to be smaller than another pair if its {@link yfiles.layout.RowDescriptor#index row index} is smaller than the row index of the other pair. If the row
       * indices are equal, the {@link yfiles.layout.PartitionCellIdEntry} with the smaller {@link yfiles.layout.ColumnDescriptor#index column index} is said to be smaller. Hence,
       * two {@link yfiles.layout.PartitionCellIdEntry} instances are equal, if the row and column indices are equal.
       * </p>
       * @param o the {@link } to be compared
       * @returns <code>-1</code>, <code>0</code> or <code>1</code> if this {@link } is smaller than, equal to, or greater than the given other {@link }
       * instance
       */
      compareTo(o:Object):number;
      /**
       * Gets the {@link yfiles.layout.RowDescriptor} associated with this {@link yfiles.layout.PartitionCellId partition cell}.
       * @type {yfiles.layout.RowDescriptor}
       */
      row:yfiles.layout.RowDescriptor;
      /**
       * Gets the {@link yfiles.layout.ColumnDescriptor} associated with this {@link yfiles.layout.PartitionCellId partition cell}.
       * @type {yfiles.layout.ColumnDescriptor}
       */
      column:yfiles.layout.ColumnDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.PartitionCellId} represents an identifier for partition cells of a {@link yfiles.layout.PartitionGrid partition grid} structure.
     * @see yfiles.layout.PartitionGrid
     * @class
     */
    export interface PartitionCellId extends Object{}
    export class PartitionCellId {
      /**
       * Gets a {@link yfiles.collections.ICollection.<T> collection} of elements of type {@link yfiles.layout.PartitionCellIdEntry}.
       * <p>
       * Each pair consists of a {@link yfiles.layout.ColumnDescriptor column} and a {@link yfiles.layout.RowDescriptor row} defining a single {@link yfiles.layout.PartitionCellId partition cell}.
       * </p>
       * @type {yfiles.collections.IEnumerable.<yfiles.layout.PartitionCellIdEntry>}
       */
      cells:yfiles.collections.IEnumerable<yfiles.layout.PartitionCellIdEntry>;
      /**
       * Gets whether or not this {@link yfiles.layout.PartitionCellId} represents a multi-cell, i.e., a cell spanning multiple
       * rows/columns.
       * <p>
       * This means that it was created using {@link yfiles.layout.PartitionGrid#createCellSpanId},
       * {@link yfiles.layout.PartitionGrid#createCellSpanId}, {@link yfiles.layout.PartitionGrid#createCellSpanId}, {@link yfiles.layout.PartitionGrid#createColumnSpanId}
       * or {@link yfiles.layout.PartitionGrid#createRowSpanId}.
       * </p>
       * @type {boolean}
       */
      isSpanning:boolean;
      /**
       * Gets the first {@link yfiles.layout.RowDescriptor row} associated with this {@link yfiles.layout.PartitionCellId partition cell} identifier.
       * @type {yfiles.layout.RowDescriptor}
       */
      row:yfiles.layout.RowDescriptor;
      /**
       * Gets the first {@link yfiles.layout.ColumnDescriptor column} associated with this {@link yfiles.layout.PartitionCellId partition cell} identifier.
       * @type {yfiles.layout.ColumnDescriptor}
       */
      column:yfiles.layout.ColumnDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a partition grid that partitions the drawing area into rectangular partition cells.
     * <p>
     * Such a structure allows, for example, to model vertical/horizontal swimlanes which are often used in flow diagrams to
     * separate logical units.
     * </p>
     * <p>
     * The grid consists of rows and columns that are created using the corresponding constructors of this class or methods {@link yfiles.layout.PartitionGrid#addRow}
     * and {@link yfiles.layout.PartitionGrid#addColumn}. The {@link yfiles.layout.PartitionGrid} object has to be registered with the graph
     * using the {@link yfiles.algorithms.IDataProvider} key {@link yfiles.layout.PartitionGrid#PARTITION_GRID_DP_KEY}.
     * </p>
     * <p>
     * Each node can be assigned to a {@link yfiles.layout.PartitionCellId} which represents a set of partition cells. The mapping has
     * to be registered with the graph using {@link yfiles.algorithms.IDataProvider} key
     * {@link yfiles.layout.PartitionGrid#PARTITION_CELL_ID_DP_KEY}. Simple partition cell identifiers which represent a single
     * partition cell, i.e., a row/column pair, can be created using method {@link yfiles.layout.PartitionGrid#createCellId} or
     * {@link yfiles.layout.PartitionGrid#createCellId}.
     * </p>
     * <p>
     * Nodes can also be mapped to a partition cell identifier that represents a multi-cell, i.e., a cell spanning multiple
     * columns and rows. Please note, however, that multi-cells are not allowed to overlap each other. This means that the user
     * has to ensure that each partition cell (row/column pair) is associated with at most one partition cell identifier. The
     * multi-cell identifier can be created using methods {@link yfiles.layout.PartitionGrid#createCellSpanId},
     * {@link yfiles.layout.PartitionGrid#createCellSpanId}, {@link yfiles.layout.PartitionGrid#createCellSpanId},
     * {@link yfiles.layout.PartitionGrid#createColumnSpanId}, and {@link yfiles.layout.PartitionGrid#createRowSpanId}.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[60,30],"d":1, "lblB" : 1, "lblFont" : "16px sans-serif"},"n":[[265,290,230,142,2,[[265,313.6,230,23.6,5,"Column 0"]]],[311.5,386.5,[[336.16,410.85,10.67,18.7]]],[411.5,346.5,[[436.16,370.85,10.67,18.7]]],[505,289,299,144,2,[[505,312.6,299,23.6,5,"Column 1"]]],[631.5,366.5,[[656.16,390.85,10.67,18.7]]],[731.5,326.5,[[756.16,350.85,10.67,18.7]]],[531.5,326.5,[[556.16,350.85,10.67,18.7]]],[819,290,250,140,2,[[819,313.6,250,23.6,5,"Column 2"]]],[831.5,326.5,[[856.16,350.85,10.67,18.7]]],[265,449,230,123,2],[411.5,466.5,[[436.16,490.85,10.67,18.7]]],[411.5,526.5,[[436.16,550.85,10.67,18.7]]],[505,449,299,123,2],[819,449,250,123,2],[931.5,466.5,[[956.16,490.85,10.67,18.7]]],[505,589,299,104,2],[819,589,250,104,2],[531.5,606.5,[[552.83,630.85,17.35,18.7]]],[265,589,230,104,2],[411.5,606.5,[[432.83,630.85,17.35,18.7]]],[831.5,606.5,[[852.83,630.85,17.35,18.7]]],[105,348.2,150,23.6,2,[[105,371.8,150,23.6,5,"Row 0"]]],[105,488.2,150,23.6,2,[[105,511.8,150,23.6,5,"Row 1"]]],[105,628.2,150,23.6,2,[[105,651.8,150,23.6,5,"Row 2"]]]],"e":[[1,2,30,-10,-30,-0,[381.5,391.5,381.5,361.5]],[1,4,30,-0,-30,10,[581.5,401.5,581.5,391.5]],[2,4,30,7.5,-30,-0,[481.5,369,481.5,381.5]],[4,5,30,-0,-30,7.5,[701.5,381.5,701.5,349]],[2,6,30,-7.5,-30,-0,[481.5,354,481.5,341.5]],[6,4,30,7.5,-30,-10,[601.5,349,601.5,371.5]],[6,5,30,-7.5,-30,-7.5,[621.5,334,621.5,341.5,681.5,341.5,681.5,334]],[5,8,30,-0,-30,-0],[8,14,30,-0,-30,-7.5,[901.5,341.5,901.5,474]],[10,14,30,-0,-30,7.5,[881.5,481.5,881.5,489]],[1,17,30,10,-30,10,[381.5,411.5,381.5,661.5,481.5,661.5,481.5,631.5]],[11,17,30,-0,-30,-10,[481.5,541.5,481.5,611.5]],[19,17,30,-0,-30,-0],[17,20,30,-0,-30,-0]],"vp":[105.0,289.0,964.0,404.0]}} Sample output of a layout algorithm with a partition grid
     * </p>
     * @class
     */
    export interface PartitionGrid extends Object{}
    export class PartitionGrid {
      /**
       * Creates an instance of {@link yfiles.layout.PartitionGrid} with the given number of rows and columns as well as with given
       * insets and minimum sizes.
       * @param rowCount the number of rows
       * @param columnCount the number of columns
       * @param rowInsets the top and bottom insets of the rows
       * @param columnInsets the left and right insets of the columns
       * @param minRowSize the minimum row height
       * @param minColumnSize the minimum column width
       */
      constructor(rowCount:number,columnCount:number,rowInsets:number,columnInsets:number,minRowSize:number,minColumnSize:number);
      /**
       * Creates an instance of {@link yfiles.layout.PartitionGrid} with the given number of rows and columns.
       * @param rowCount the number of rows
       * @param columnCount the number of columns
       */
      constructor(rowCount:number,columnCount:number);
      /**
       * Creates a new empty instance of {@link yfiles.layout.PartitionGrid}.
       */
      constructor();
      /**
       * Adds a new {@link yfiles.layout.ColumnDescriptor column} to the {@link yfiles.layout.PartitionGrid} instance.
       * <p>
       * The index of the column (as returned by {@link yfiles.layout.ColumnDescriptor#index}) is equal to the size of the list returned by {@link yfiles.layout.PartitionGrid#columns}.
       * </p>
       * @returns a new {@link } instance
       */
      addColumn():yfiles.layout.ColumnDescriptor;
      /**
       * Adds a new {@link yfiles.layout.RowDescriptor row} to the {@link yfiles.layout.PartitionGrid} instance.
       * <p>
       * The index of the row (as returned by {@link yfiles.layout.RowDescriptor#index}) is equal to the size of the list returned by {@link yfiles.layout.PartitionGrid#rows}.
       * </p>
       * @returns a new {@link } instance
       */
      addRow():yfiles.layout.RowDescriptor;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents the cell defined by the given column and row index.
       * @param rowIndex the row index of the row defining the cell
       * @param columnIndex the column index of the column defining the cell
       * @returns a {@link } instance
       * @throws {Stubs.Exceptions.ArgumentError} if the given column/row index is not valid
       */
      createCellId(rowIndex:number,columnIndex:number):yfiles.layout.PartitionCellId;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents the cell defined by the given {@link yfiles.layout.ColumnDescriptor column} and {@link yfiles.layout.RowDescriptor row}.
       * @param rd the row defining the cell
       * @param cd the column defining the cell
       * @returns a {@link } instance
       * @throws {Stubs.Exceptions.ArgumentError} if the given column/row is <code>null</code>
       */
      createCellId(rd:yfiles.layout.RowDescriptor,cd:yfiles.layout.ColumnDescriptor):yfiles.layout.PartitionCellId;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents a multi-cell spanning multiple columns and rows.
       * <p>
       * The partition cell identifier represents all cells defined by a row between <code>fromRow</code> and <code>toRow</code> (both including) and a
       * column between <code>fromRow</code> and <code>toCol</code> (both including).
       * </p>
       * <p>
       * A group node mapped to such a partition cell identifier represents the multi-cell, i.e., its boundary corresponds to the
       * smallest rectangle containing all partition cells specified by the partition cell identifier. The boundary does not
       * depend on the partition cells associated with the group's descendants. Hence, each non-group descendant has to be
       * manually assigned to one of the partition cells. Furthermore, edges incident to such a group node are not considered
       * during the layout.
       * </p>
       * @param fromRow the topmost row defining the cell
       * @param fromCol the leftmost column defining the cell
       * @param toRow the bottommost row defining the cell
       * @param toCol the rightmost column defining the cell
       * @returns a {@link } instance
       */
      createCellSpanId(fromRow:yfiles.layout.RowDescriptor,fromCol:yfiles.layout.ColumnDescriptor,toRow:yfiles.layout.RowDescriptor,toCol:yfiles.layout.ColumnDescriptor):yfiles.layout.PartitionCellId;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents a multi-cell spanning multiple columns and rows.
       * <p>
       * The partition cell identifier represents all cells defined by a row with index between <code>fromRowIndex</code> and <code>toRowIndex</code> (both
       * including) and a column with index between <code>fromColIndex</code> and <code>toColIndex</code> (both including).
       * </p>
       * <p>
       * A group node mapped to such a partition cell identifier represents the multi-cell, i.e., its boundary corresponds to the
       * smallest rectangle containing all partition cells specified by the partition cell identifier. The boundary does not
       * depend on the partition cells associated with the group's descendants. Hence, each non-group descendant has to be
       * manually assigned to one of the partition cells. Furthermore, edges incident to such a group node are not considered
       * during the layout.
       * </p>
       * @param fromRowIndex the row index of the topmost row defining the cell
       * @param fromColIndex the column index of the leftmost column defining the cell
       * @param toRowIndex the row index of the bottommost row defining the cell
       * @param toColIndex the column index of the rightmost column defining the cell
       * @returns a {@link } instance
       */
      createCellSpanId(fromRowIndex:number,fromColIndex:number,toRowIndex:number,toColIndex:number):yfiles.layout.PartitionCellId;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents a multi-cell spanning multiple columns and rows.
       * <p>
       * The partition cell identifier represents all cells spanned by the columns of <code>columnList</code> and rows of <code>rowList</code>.
       * </p>
       * <p>
       * A group node mapped to such a partition cell identifier represents the multi-cell, i.e., its boundary corresponds to the
       * smallest rectangle containing all partition cells specified by the partition cell identifier. The boundary does not
       * depend on the partition cells associated with the group's descendants. Hence, each non-group descendant has to be
       * manually assigned to one of the partition cells. Furthermore, edges incident to such a group node are not considered
       * during the layout.
       * </p>
       * @param rowList the {@link  rows} defining the cell
       * @param columnList the {@link  columns} defining the cell
       * @returns a {@link } instance
       */
      createCellSpanId(rowList:yfiles.collections.ICollection<Object>,columnList:yfiles.collections.ICollection<Object>):yfiles.layout.PartitionCellId;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents a cell spanning a whole column.
       * <p>
       * A group node mapped to such a partition cell identifier represents the column, i.e., its enclosing boundary corresponds
       * to that of the column. The boundary does not depend on the partition cells associated with the group's descendants.
       * Hence, each non-group descendant has to be manually assigned to one of the partition cells. Furthermore, edges incident
       * to such a group node are not considered during the layout.
       * </p>
       * @param columnIndex the column index of the column defining the cell
       * @returns a {@link } instance
       */
      createColumnSpanId(columnIndex:number):yfiles.layout.PartitionCellId;
      /**
       * Creates a {@link yfiles.layout.PartitionCellId partition cell} identifier that represents a cell spanning a whole row.
       * <p>
       * A group node mapped to such a partition cell identifier represents the row, i.e., its enclosing boundary corresponds to
       * that of the row. The boundary does not depend on the partition cells associated with the group's descendants. Hence,
       * each non-group descendant has to be manually assigned to one of the partition cells. Furthermore, edges incident to such
       * a group node are not considered during the layout.
       * </p>
       * @param rowIndex the row index of the row defining the cell
       * @returns a {@link } instance
       */
      createRowSpanId(rowIndex:number):yfiles.layout.PartitionCellId;
      /**
       * Finalizes the orientation changes and is called at the end of the {@link yfiles.layout.OrientationLayout} stage.
       * <p>
       * Points are transferred to their final positions through {@link yfiles.layout.OrientationLayout#completeTransform}.
       * </p>
       * @param orientationLayouter an {@link } instance
       * @see yfiles.layout.OrientationLayout
       */
      finalizeOrientationChange(orientationLayouter:yfiles.layout.OrientationLayout):void;
      /**
       * Returns the {@link yfiles.layout.ColumnDescriptor column} with the given index.
       * @param index the index of the column to be returned
       * @returns the {@link  column} with the given index or <code>null</code> if no such column exists
       */
      getColumn(index:number):yfiles.layout.ColumnDescriptor;
      /**
       * Returns the {@link yfiles.layout.PartitionGrid} instance associated with the given graph.
       * @param graph the given graph
       * @returns the {@link } of the graph or <code>null</code> if the graph has no associated partition grid
       * @static
       */
      static getPartitionGrid(graph:yfiles.algorithms.Graph):yfiles.layout.PartitionGrid;
      /**
       * Returns the {@link yfiles.layout.RowDescriptor row} with the given index.
       * @param index the index of the row to be returned
       * @returns the {@link  row} with the given index or <code>null</code> if no such row exists
       */
      getRow(index:number):yfiles.layout.RowDescriptor;
      static hasAtLeastTwoNonEmptyRows(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Prepares for the orientation changes and is called at the beginning of the {@link yfiles.layout.OrientationLayout} stage.
       * <p>
       * Original points are transferred to their temporary positions through {@link yfiles.layout.OrientationLayout#prepareTransform}.
       * </p>
       * @param orientationLayouter an {@link } instance
       * @see yfiles.layout.OrientationLayout
       */
      prepareOrientationChange(orientationLayouter:yfiles.layout.OrientationLayout):void;
      /**
       * Data provider key for storing the partition grid structure
       * <p>
       * If the {@link yfiles.algorithms.IDataProvider} contains a partition grid structure, this structure will be considered during
       * layout (provided that the layout algorithm supports such constraints).
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.GraphDpKey.<yfiles.layout.PartitionGrid>}
       */
      static PARTITION_GRID_DP_KEY:yfiles.algorithms.GraphDpKey<yfiles.layout.PartitionGrid>;
      /**
       * Data provider key for storing a set of partition cells for each node
       * <p>
       * Each node is placed inside the columns/rows defined by the corresponding {@link yfiles.layout.PartitionCellId} identifier.
       * Instances can be shared among multiple nodes, but don't have to be shared.
       * </p>
       * @see yfiles.layout.PartitionCellId
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.layout.PartitionCellId>}
       */
      static PARTITION_CELL_ID_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.layout.PartitionCellId>;
      /**
       * Gets or sets whether or not the order of the columns should be chosen automatically to minimize edge lengths.
       * <p>
       * For all columns where {@link yfiles.layout.ColumnDescriptor#indexFixed} is set to
       * <code>true</code>, the relative ordering given by the indices is preserved. The remaining columns may be sorted again so that the
       * overall edge lengths are minimized.
       * </p>
       * @type {boolean}
       */
      optimizeColumnOrder:boolean;
      /**
       * Gets or sets whether or not the order of the rows should be chosen automatically to minimize edge lengths.
       * <p>
       * For all rows where {@link yfiles.layout.RowDescriptor#indexFixed} is set to
       * <code>true</code>, the relative ordering given by the indices is preserved. The remaining rows may be sorted again so that the
       * overall edge lengths are minimized.
       * </p>
       * @type {boolean}
       */
      optimizeRowOrder:boolean;
      /**
       * Gets the list of rows of the {@link yfiles.layout.PartitionGrid partition grid}.
       * @type {yfiles.algorithms.YList}
       */
      rows:yfiles.algorithms.YList;
      /**
       * Gets the columns of the {@link yfiles.layout.PartitionGrid partition grid}.
       * @type {yfiles.algorithms.YList}
       */
      columns:yfiles.algorithms.YList;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a row of a {@link yfiles.layout.PartitionGrid partition grid} structure.
     * <p>
     * The results of the calculation of the geometry of the row will be placed into the instances of this class after the
     * layout.
     * </p>
     * @see yfiles.layout.PartitionGrid
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface RowDescriptor extends Object,yfiles.lang.IComparable{}
    export class RowDescriptor {
      /**
       * Compares this {@link yfiles.layout.RowDescriptor} instance with a given other {@link yfiles.layout.RowDescriptor} instance based on the
       * indices of the two instances.
       * @param o the {@link } to compare to
       * @returns <code>-1</code>, <code>0</code> or <code>1</code> if this {@link } is less than, equal to, or greater than the given other
       * {@link }
       */
      compareTo(o:Object):number;
      /**
       * Gets the index of the row within the {@link yfiles.layout.PartitionGrid partition grid}.
       * @type {number}
       */
      index:number;
      /**
       * Gets or sets the minimum height of the row.
       * <p>
       * The minimum height is defined to be at least <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum height is negative
       * @type {number}
       */
      minimumHeight:number;
      /**
       * Gets or sets the top inset in which no element will lie in the resulting layout.
       * <p>
       * The inset value needs to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given top inset is negative
       * @type {number}
       */
      topInset:number;
      /**
       * Gets or sets the bottom inset in which no element will lie in the resulting layout.
       * <p>
       * The inset value needs to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given bottom inset is negative
       * @type {number}
       */
      bottomInset:number;
      /**
       * Gets or sets the computed height of the row after the layout has been calculated.
       * <p>
       * The computed height needs to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the computed height is negative
       * @type {number}
       */
      computedHeight:number;
      /**
       * Gets or sets the original position (smallest y-coordinate) of the row.
       * @type {number}
       */
      originalPosition:number;
      /**
       * Gets or sets the original height of the row.
       * <p>
       * The original height needs to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the original height is negative
       * @type {number}
       */
      originalHeight:number;
      /**
       * Gets or sets the computed position (smallest y-coordinate) of the row after the layout has been calculated.
       * @type {number}
       */
      computedPosition:number;
      /**
       * Gets or sets the tightness factor of this row.
       * <p>
       * This value must lie within the interval <code>[0,1]</code>.
       * </p>
       * <p>
       * The larger the value, the more the row will be forced to obtain its minimum height:
       * </p>
       * <ul>
       * <li>a value of <code>0.0d</code> will disable the compression of the row.</li>
       * <li>a value of <code>1.0d</code> will try to force the row to obtain its specified {@link yfiles.layout.RowDescriptor#minimumHeight minimum height}.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if the tightness value is outside the interval <code>[0,1]</code>
       * @type {number}
       */
      tightness:number;
      /**
       * Gets or sets whether the index of the row is fixed or it should be chosen automatically in order to minimize edge
       * lengths.
       * <p>
       * For all rows where this property is enabled, the relative ordering given by the indices is preserved. The remaining rows
       * may be sorted again so that the overall edge lengths are minimized.
       * </p>
       * @type {boolean}
       */
      indexFixed:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum InterEdgeRoutingStyle{
      /**
       * An inter-edge routing style based on which the edge layout of inter-edges will be adopted from the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @see yfiles.layout.FixGroupLayoutStage#interEdgeRoutingStyle
       */
      ADOPT,
      /**
       * An inter-edge routing style based on which inter-edges are routed in an orthogonal style.
       * @see yfiles.layout.FixGroupLayoutStage#interEdgeRoutingStyle
       * @see yfiles.layout.FixGroupLayoutStage#interEdgeRouter
       */
      ORTHOGONAL,
      /**
       * An inter-edge routing style based on which inter-edges are routed as straight lines.
       * @see yfiles.layout.FixGroupLayoutStage#interEdgeRoutingStyle
       * @see yfiles.layout.FixGroupLayoutStage#interEdgeRouter
       */
      STRAIGHTLINE
    }
    /**
     * This interface defines {@link yfiles.algorithms.IDataProvider} keys for those layout algorithms that can handle hierarchically
     * grouped graphs.
     * <p>
     * There are two groups of {@link yfiles.algorithms.IDataProvider} keys. {@link yfiles.algorithms.IDataProvider}s registered with
     * {@link yfiles.layout.GroupingKeys#NODE_ID_DP_KEY}, {@link yfiles.layout.GroupingKeys#GROUP_DP_KEY} and {@link yfiles.layout.GroupingKeys#PARENT_NODE_ID_DP_KEY}
     * define the relationship of nodes within the hierarchy. A parent node is defined for each node in the graph. This parent
     * is the containing group node. For top-level nodes, this parent is <code>null</code>.
     * </p>
     * <p>
     * {@link yfiles.layout.GroupingKeys#GROUP_NODE_INSETS_DP_KEY} and {@link yfiles.layout.GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY} are used for
     * registering
     * {@link yfiles.algorithms.IDataProvider}s that add layout information to group nodes such as the insets and the minimum size
     * respectively.
     * </p>
     * @see yfiles.layout.GroupingSupport
     * @class
     * @static
     */
    export interface GroupingKeys extends Object{}
    export class GroupingKeys {
      /**
       * Data provider key for assigning a unique ID for each node
       * <p>
       * The IDs of the nodes need to differ based on {@link Object#hashCode} and {@link Object#equals}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static NODE_ID_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Data provider key for describing the hierarchy of nodes in the graph
       * @see yfiles.layout.GroupingKeys#NODE_ID_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static PARENT_NODE_ID_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Data provider key for distinguishing normal nodes from group nodes
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static GROUP_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for assigning insets for group nodes
       * <p>
       * Insets reserve space around the contents of a group node. Labels and other visual decorations may be placed inside this
       * space.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.algorithms.Insets>}
       */
      static GROUP_NODE_INSETS_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.algorithms.Insets>;
      /**
       * Data provider key for assigning minimum sizes for group nodes
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.algorithms.YDimension>}
       */
      static MINIMUM_NODE_SIZE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.algorithms.YDimension>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides convenient and efficient access to the hierarchic graph structure.
     * <p>
     * The grouping information is provided through {@link yfiles.algorithms.IDataProvider}s registered with the graph with keys
     * {@link yfiles.layout.GroupingKeys#NODE_ID_DP_KEY}, {@link yfiles.layout.GroupingKeys#PARENT_NODE_ID_DP_KEY} and
     * {@link yfiles.layout.GroupingKeys#GROUP_DP_KEY}. It describes the hierarchy as a tree where a group node is the parent of the
     * nodes that it contains.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[444,-37.38,62,82.38,2,[[444,-15,62,22.38,"9"]]],[460,0,[[469.66,24.35,10.67,18.7,"5"]]],[536,0,[[545.66,24.35,10.67,18.7,"6"]]],[384,0,[[393.66,24.35,10.67,18.7,"4"]]],[596,-10,50,50,2,[[596,12.38,50,22.38,"7"]]],[140,-74.75,214,134.75,2,[[140,-52.38,214,22.38,"10"]]],[156,0,[[165.66,24.35,10.67,18.7,"1"]]],[216,-37.38,122,82.38,2,[[216,-15,122,22.38,"8"]]],[232,0,[[241.66,24.35,10.67,18.7,"2"]]],[292,0,[[301.66,24.35,10.67,18.7,"3"]]]],"e":[],"vp":[140.0,-75.0,506.0,135.0]}} This is an example graph that contains group nodes. {@graph {"ann":{"s":[30,30],"d":1},"n":[[34.5,70,42,52.38,2,[[34.5,92.38,42,22.38,"10"]]],[200,162.38,42,52.38,2,[[200,184.75,42,22.38,"9"]]],[150,277.13,[[159.66,301.48,10.67,18.7,"4"]]],[179.25,0,[[160.9,22.7,66.7,18.7,"Virtual Root"]]],[0,277.13,[[9.66,301.48,10.67,18.7,"1"]]],[206,277.13,[[215.66,301.48,10.67,18.7,"5"]]],[69,162.38,42,52.38,2,[[69,184.75,42,22.38,"8"]]],[50,277.13,[[59.66,301.48,10.67,18.7,"2"]]],[100,277.13,[[109.66,301.48,10.67,18.7,"3"]]],[312,254.75,42,52.38,2,[[312,277.13,42,22.38,"7"]]],[262,277.13,[[271.66,301.48,10.67,18.7,"6"]]]],"e":[[3,0,[194.25,50,55.5,50]],[3,2,[194.25,50,165,50]],[3,1,[194.25,50,221,50]],[0,4,[55.5,142.38,15,142.38]],[1,5],[0,6,[55.5,142.38,90,142.38]],[6,7,[90,234.75,65,234.75]],[6,8,[90,234.75,115,234.75]],[3,9,[194.25,50,333,50]],[3,10,[194.25,50,277,50]]],"vp":[0.0,0.0,354.0,308.0]}} The hierarchy tree of the grouped graph reveals the relationship
     * among the nodes. The corresponding nodes have the same labels as in the grouped graph. Group nodes are considered as the
     * parents of their content which can consist of group nodes and/or normal nodes. Top-level nodes are children of a
     * {@link yfiles.layout.GroupingSupport#root virtual root}. Leaf nodes can either be normal nodes or empty groups.
     * </p>
     * @class
     */
    export interface GroupingSupport extends Object{}
    export class GroupingSupport {
      /**
       * Creates a new {@link yfiles.layout.GroupingSupport} instance that represents the hierarchy of the graph.
       * <p>
       * The hierarchy information is cached during the constructor call. Later modifications to the graph are not considered and
       * may lead to errors when using this {@link yfiles.layout.GroupingSupport} instance.
       * </p>
       * @param graph the graph instance from which the hierarchy information is obtained
       */
      constructor(graph:yfiles.algorithms.Graph);
      /**
       * Creates a {@link yfiles.algorithms.INodeMap} to store hierarchy information for each node.
       * <p>
       * This method is called in {@link yfiles.layout.GroupingSupport#init} and may be overridden to customize how the {@link yfiles.algorithms.INodeMap}
       * is created.
       * </p>
       * @param graph the current graph
       * @returns the new instance of {@link }
       * @see yfiles.layout.GroupingSupport#init
       * @see yfiles.layout.GroupingSupport#disposeInfoMap
       * @protected
       */
      createInfoMap(graph:yfiles.algorithms.Graph):yfiles.algorithms.INodeMap;
      /**
       * Disposes of all internal data structures held by this {@link yfiles.layout.GroupingSupport instance}.
       * <p>
       * {@link yfiles.layout.GroupingSupport} should be disposed of when it is not needed anymore, or if the state of the graph changes
       * (e.g. new nodes are added).
       * </p>
       */
      dispose():void;
      /**
       * Disposes of the {@link yfiles.algorithms.INodeMap} created to store hierarchy information for each node.
       * <p>
       * This method is called in {@link yfiles.layout.GroupingSupport#dispose} and may be overridden to revert changes from
       * {@link yfiles.layout.GroupingSupport#createInfoMap}.
       * </p>
       * @param graph the current graph
       * @param infoMap the {@link } which contains the hierarchy information for each node
       * @see yfiles.layout.GroupingSupport#dispose
       * @see yfiles.layout.GroupingSupport#createInfoMap
       * @protected
       */
      disposeInfoMap(graph:yfiles.algorithms.Graph,infoMap:yfiles.algorithms.INodeMap):void;
      /**
       * Returns the direct children of the given group node.
       * <p>
       * If the {@link yfiles.layout.GroupingSupport#root virtual root of the hierarchy tree} is passed, all top-level nodes are returned.
       * </p>
       * @param parent the group node
       * @returns a set of child nodes of the given parent node
       * @see yfiles.layout.GroupingSupport#root
       * @see yfiles.layout.GroupingSupport#getParent
       * @see yfiles.layout.GroupingSupport#getDescendants
       */
      getChildren(parent:yfiles.algorithms.Node):yfiles.algorithms.NodeList;
      /**
       * Returns all descendants of the given group node.
       * <p>
       * All nodes in the subtree rooted at this group node are collected recursively.
       * </p>
       * <p>
       * If the {@link yfiles.layout.GroupingSupport#root virtual root of the hierarchy tree} is passed, all nodes in the graph are returned.
       * </p>
       * @param parent the group node
       * @returns a set of nodes that belong to a subtree of the hierarchy tree rooted at the given parent
       * @see yfiles.layout.GroupingSupport#root
       * @see yfiles.layout.GroupingSupport#getParent
       * @see yfiles.layout.GroupingSupport#getChildren
       */
      getDescendants(parent:yfiles.algorithms.Node):yfiles.algorithms.NodeList;
      /**
       * Returns all edges crossing the bounds of the given group node such that their targets are inside the group node while
       * their sources lie outside the group node.
       * @param group the group node whose bounds are crossed
       * @returns a set of edges that cross the bounds of the group node starting outside the group node
       * @see yfiles.layout.GroupingSupport#getEdgesGoingOut
       * @see yfiles.layout.GroupingSupport#isNormalEdge
       */
      getEdgesGoingIn(group:yfiles.algorithms.Node):yfiles.algorithms.EdgeList;
      /**
       * Returns all edges crossing the bounds of the given group node such that their sources are inside the group node while
       * their targets lie outside the group node.
       * @param group the group node whose bounds are crossed
       * @returns a set of edges that cross the bounds of the group node starting inside the group node
       * @see yfiles.layout.GroupingSupport#getEdgesGoingIn
       * @see yfiles.layout.GroupingSupport#isNormalEdge
       */
      getEdgesGoingOut(group:yfiles.algorithms.Node):yfiles.algorithms.EdgeList;
      /**
       * Returns the nearest common ancestor of the given nodes in the hierarchy structure.
       * <p>
       * The nearest common ancestor is the local root of the smallest subtree in the hierarchy tree that contains both nodes.
       * This group is the first node that contains these two nodes. If no such node exists, the {@link yfiles.layout.GroupingSupport#root virtual root} is returned.
       * </p>
       * <p>
       * Common ancestors in special cases:
       * </p>
       * <ul>
       * <li><code>node1 == node2</code>: the parent of <code>node1</code>/<code>node2</code></li>
       * <li><code>node1 is parent of node2</code>: the parent of <code>node1</code></li>
       * <li><code>node2 is parent of node1</code>: the parent of <code>node2</code></li>
       * </ul>
       * @param node1 one of the two nodes whose common ancestor is determined
       * @param node2 one of the two nodes whose common ancestor is determined
       * @returns the nearest common ancestor node or {@link #root}
       */
      getNearestCommonAncestor(node1:yfiles.algorithms.Node,node2:yfiles.algorithms.Node):yfiles.algorithms.Node;
      /**
       * Returns the parent for the given node.
       * <p>
       * The parent is the containing group node.
       * </p>
       * @param node the node for which the parent is determined
       * @returns the parent of the given node or <code>null</code> for a top-level node
       * @throws {Stubs.Exceptions.ArgumentError} if the given node is <code>null</code>
       * @see yfiles.layout.GroupingSupport#getChildren
       * @see yfiles.layout.GroupingSupport#getDescendants
       * @see yfiles.layout.GroupingSupport#getNearestCommonAncestor
       */
      getParent(node:yfiles.algorithms.Node):yfiles.algorithms.Node;
      /**
       * Returns an immediate child of the group node that represents the given node.
       * <p>
       * If this node is already a child of the group node, the group node will be its representative. In case the node is
       * located further down in the hierarchy, the child of the group node, which is also an ancestor of the node, is used.
       * There is no representative if the node is not a descendant of the group.
       * </p>
       * @param node the node for which a representative is searched
       * @param group the group node to which the representative should belong
       * @returns the representative or <code>null</code> if the node is not a descendant of the group
       * @throws {Stubs.Exceptions.ArgumentError} if the given node is <code>null</code>
       */
      getRepresentative(node:yfiles.algorithms.Node,group:yfiles.algorithms.Node):yfiles.algorithms.Node;
      /**
       * Determines whether or not the given node is a group node with children.
       * <p>
       * Normal nodes never have children.
       * </p>
       * @param node the potential parent
       * @returns <code>true</code> if the given node has children, <code>false</code> otherwise
       * @see yfiles.layout.GroupingSupport#getChildren
       * @see yfiles.layout.GroupingSupport#isGroupNode
       */
      hasChildren(node:yfiles.algorithms.Node):boolean;
      /**
       * Initializes internal data structures.
       * <p>
       * This method is called from {@link yfiles.layout.GroupingSupport} and retrieves the state of the hierarchic graph at the moment
       * of invocation. It may be overridden to append additional information used for group handling.
       * </p>
       * @see yfiles.layout.GroupingSupport#dispose
       * @protected
       */
      init():void;
      /**
       * Returns whether or not the given graph is flat.
       * <p>
       * A graph is considered to be <i>flat</i> if there exist no group nodes or there exist group nodes but none of them has children.
       * </p>
       * @param graph the input graph
       * @returns <code>true</code> if there are no groups or only empty groups in the graph, <code>false</code> otherwise
       * @see yfiles.layout.GroupingSupport#isGrouped
       * @static
       */
      static isFlat(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Returns whether or not the given graph is grouped.
       * <p>
       * A graph is considered to be grouped if there are {@link yfiles.algorithms.IDataProvider}s registered with the graph with keys
       * {@link yfiles.layout.GroupingKeys#GROUP_DP_KEY}, {@link yfiles.layout.GroupingKeys#NODE_ID_DP_KEY} and
       * {@link yfiles.layout.GroupingKeys#PARENT_NODE_ID_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @returns <code>true</code> if the graph contains hierarchic information, <code>false</code> otherwise
       * @see yfiles.layout.GroupingSupport#isFlat
       * @static
       */
      static isGrouped(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Determines whether or not the given node is a group node.
       * <p>
       * While group nodes may have children, normal nodes never have children.
       * </p>
       * @param node the potential group node
       * @returns <code>true</code> if the given node is a group node, <code>false</code> otherwise
       * @see yfiles.layout.GroupingSupport#hasChildren
       */
      isGroupNode(node:yfiles.algorithms.Node):boolean;
      /**
       * Determines whether or not the given edge is a normal edge as opposed to an edge that crosses any group node bounds.
       * <p>
       * Source and target of a normal edge are part of the same hierarchy level and share the same parent node.
       * </p>
       * @param edge the edge to check
       * @returns <code>true</code> if the given edge is normal, <code>false</code> otherwise
       * @see yfiles.layout.GroupingSupport#getEdgesGoingIn
       * @see yfiles.layout.GroupingSupport#getEdgesGoingOut
       */
      isNormalEdge(edge:yfiles.algorithms.Edge):boolean;
      /**
       * Gets the {@link yfiles.algorithms.Graph} instance for which this {@link yfiles.layout.GroupingSupport} object provides hierarchy
       * information.
       * @type {yfiles.algorithms.Graph}
       */
      graph:yfiles.algorithms.Graph;
      /**
       * Gets a {@link yfiles.algorithms.Node} instance associated with the virtual root of the hierarchy tree.
       * <p>
       * This instance can be given to the {@link yfiles.layout.GroupingSupport#getChildren} to obtain the nodes of the top level of the
       * hierarchy.
       * </p>
       * @see yfiles.layout.GroupingSupport#getChildren
       * @type {yfiles.algorithms.Node}
       */
      root:yfiles.algorithms.Node;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.ILayoutStage layout stage} that hides the group nodes of hierarchically grouped graphs.
     * <p>
     * This stage removes all group nodes and adjacent edges from the graph before passing it to the
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. After the core layout algorithm has arranged the graph, the hidden elements will be reinserted.
     * </p>
     * <p>
     * This stage is used by layout algorithms that cannot handle grouped graphs (e.g. {@link yfiles.circular.CircularLayout}).
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface HideGroupsStage extends yfiles.layout.LayoutStageBase{}
    export class HideGroupsStage {
      /**
       * Creates an instance of {@link yfiles.layout.HideGroupsStage} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Hides the group nodes and their incident edges of the input graph.
       * <p>
       * Furthermore, this method removes all grouping-related {@link yfiles.algorithms.IDataProvider}s from the input graph.
       * </p>
       * @param graph the input graph
       * @see yfiles.layout.GroupingKeys#NODE_ID_DP_KEY
       * @see yfiles.layout.GroupingKeys#PARENT_NODE_ID_DP_KEY
       * @see yfiles.layout.GroupingKeys#GROUP_DP_KEY
       * @see yfiles.layout.HideGroupsStage#unhideGroupNodes
       */
      hideGroupNodes(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Un-hides the group nodes and their incident edges of the input graph that were previously hidden using method
       * {@link yfiles.layout.HideGroupsStage#hideGroupNodes}.
       * <p>
       * Furthermore, this method restores all previously removed grouping related
       * {@link yfiles.algorithms.IDataProvider}s on the input graph.
       * </p>
       * @param graph the given graph
       * @see yfiles.layout.GroupingKeys#NODE_ID_DP_KEY
       * @see yfiles.layout.GroupingKeys#PARENT_NODE_ID_DP_KEY
       * @see yfiles.layout.GroupingKeys#GROUP_DP_KEY
       * @see yfiles.layout.HideGroupsStage#hideGroupNodes
       */
      unhideGroupNodes(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Gets or sets whether or not empty group nodes, i.e., group nodes without children, will be hidden by this stage.
       * @type {boolean}
       */
      hidingEmptyGroupNodes:boolean;
      /**
       * Gets or sets the {@link yfiles.layout.IGroupBoundsCalculator} instance for calculating the sizes of group nodes.
       * <p>
       * For every group which is in its open state, the {@link yfiles.layout.IGroupBoundsCalculator} will be asked to calculate the
       * bounds for the group's child nodes. The resulting size will be used during the ongoing layout calculation.
       * </p>
       * @type {yfiles.layout.IGroupBoundsCalculator}
       */
      groupBoundsCalculator:yfiles.layout.IGroupBoundsCalculator;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.IGroupBoundsCalculator} is responsible for calculating the bounds of a group node during a layout
     * process.
     * @interface
     */
    export interface IGroupBoundsCalculator extends Object{
      /**
       * Calculates the bounds of the given group node that contains the given list of child nodes.
       * <p>
       * This method is called during the layout process.
       * </p>
       * @param graph the input graph
       * @param groupNode the group node whose bounds will be calculated
       * @param children a {@link } containing the nodes that reside inside the group node
       * @returns a {@link } instance that describes the bounds of the group node
       * @abstract
       */
      calculateBounds(graph:yfiles.layout.LayoutGraph,groupNode:yfiles.algorithms.Node,children:yfiles.algorithms.NodeList):yfiles.algorithms.Rectangle2D;
    }
    var IGroupBoundsCalculator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This {@link yfiles.layout.IGroupBoundsCalculator} calculates the bounds of group nodes and allows to specify custom insets for
     * each group node.
     * <p>
     * {@link yfiles.algorithms.Insets Insets} are provided by a {@link yfiles.algorithms.IDataProvider} that must have been registered with the graph using the
     * specified {@link yfiles.layout.InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key} prior to the invocation of the layout algorithm.
     * </p>
     * @class
     * @implements {yfiles.layout.IGroupBoundsCalculator}
     */
    export interface InsetsGroupBoundsCalculator extends Object,yfiles.layout.IGroupBoundsCalculator{}
    export class InsetsGroupBoundsCalculator {
      /**
       * Constructs an instance of {@link yfiles.layout.InsetsGroupBoundsCalculator} using the given {@link yfiles.algorithms.IDataProvider} key
       * as {@link yfiles.layout.InsetsGroupBoundsCalculator#groupNodeInsetsDpKey group node insets key} and the given insets as {@link yfiles.layout.InsetsGroupBoundsCalculator#defaultInsets default insets}.
       * @param [groupNodeInsetsDPKey=null] the {@link } key to use as {@link #groupNodeInsetsDpKey insets key}
       * @param [defaultInsets=null] the insets to use as {@link #defaultInsets default insets}
       */
      constructor(groupNodeInsetsDPKey?:Object,defaultInsets?:yfiles.algorithms.Insets);
      /**
       * Constructs an instance of {@link yfiles.layout.InsetsGroupBoundsCalculator} using the given {@link yfiles.algorithms.IDataProvider} key
       * as {@link yfiles.layout.InsetsGroupBoundsCalculator#groupNodeInsetsDpKey group node insets key} and the given insets as {@link yfiles.layout.InsetsGroupBoundsCalculator#defaultInsets default insets}.
       * @param {Object} options The parameters to pass.
       * @param [options.groupNodeInsetsDPKey=null] the {@link } key to use as {@link #groupNodeInsetsDpKey insets key}
       * @param [options.defaultInsets=null] the insets to use as {@link #defaultInsets default insets}
       */
      constructor(options:{groupNodeInsetsDPKey?:Object,defaultInsets?:yfiles.algorithms.Insets});
      /**
       * Calculates the bounds of the given group node and enlarges the bounds by the insets associated with the group node.
       * <p>
       * The {@link yfiles.algorithms.Insets insets} associated with the group node will be retrieved from a {@link yfiles.algorithms.IDataProvider} registered with the
       * graph with the current {@link yfiles.layout.InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}.
       * </p>
       * @param graph the input graph
       * @param groupNode the group node whose bounds will be calculated
       * @param children a {@link } containing the nodes that reside inside the group node
       * @returns a {@link } instance that describes the bounds of the group node
       */
      calculateBounds(graph:yfiles.layout.LayoutGraph,groupNode:yfiles.algorithms.Node,children:yfiles.algorithms.NodeList):yfiles.algorithms.Rectangle2D;
      /**
       * Enlarges the given graph bounds by the insets associated with the given group node.
       * <p>
       * The {@link yfiles.algorithms.Insets insets} associated with the group node will be retrieved from a {@link yfiles.algorithms.IDataProvider} registered with the
       * graph with the current {@link yfiles.layout.InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}.
       * </p>
       * <p>
       * This method is called while {@link yfiles.layout.InsetsGroupBoundsCalculator#calculateBounds calculating the group bounds} to realize the support for custom insets. It may be overridden
       * to change the way specific insets influence the group node bounds.
       * </p>
       * @param graph the input graph
       * @param groupNode the group node whose bounds should be enlarged
       * @param children the list of nodes that the given group node contains
       * @param bounds a {@link } instance representing the bounds of the given group node
       * @returns a {@link } instance representing the enlarged bounds of the given group node
       */
      createEnlargedBounds(graph:yfiles.layout.LayoutGraph,groupNode:yfiles.algorithms.Node,children:yfiles.algorithms.NodeList,bounds:yfiles.algorithms.Rectangle2D):yfiles.algorithms.Rectangle2D;
      /**
       * Gets or sets whether or not the labels of nodes contained in a group are considered while calculating the bounds of the
       * group node.
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not the labels of edges contained in a group are considered while calculating the bounds of the
       * group node.
       * @type {boolean}
       */
      considerEdgeLabels:boolean;
      /**
       * Gets or sets whether or not the calculation of the bounds of a group node considers the {@link yfiles.layout.NodeHalo NodeHalos} associated with the
       * group's child nodes.
       * @type {boolean}
       */
      considerNodeHalos:boolean;
      /**
       * Gets or sets the default insets that will be used if the {@link yfiles.algorithms.IDataProvider} registered with the {@link yfiles.layout.InsetsGroupBoundsCalculator#groupNodeInsetsDpKey insets key}
       * does not contain a mapping for a specific node.
       * @type {yfiles.algorithms.Insets}
       */
      defaultInsets:yfiles.algorithms.Insets;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that is used to access the insets for the group
       * nodes.
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} registered with this key must provide a mapping from each group node to a {@link yfiles.algorithms.Insets}
       * (or {@link yfiles.algorithms.Insets}) instance.
       * </p>
       * @type {Object}
       */
      groupNodeInsetsDpKey:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A special {@link yfiles.layout.ComponentLayout} implementation that allows to lay out isolated subgraphs within a group as
     * separate components.
     * <p>
     * {@link yfiles.orthogonal.OrthogonalLayout OrthogonalLayout} uses this layout stage to arrange components.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0},"n":[[-5,16,210,336.38,2],[10,110,1],[10,35,1],[85,35,1],[160,35,1],[160,110,1],[10,285],[85,285],[85,210]],"e":[[1,2,0,-15,0,15],[2,3,15,0,-15,0],[3,4,15,0,-15,0],[4,5,0,15,0,-15],[5,1,-15,0,15,0],[6,8,0,-15,-15,0,[25,225]],[8,7,0,15,0,-15],[7,6,-15,0,15,0]],"vp":[-5.0,16.0,210.0,337.0]}} Example of a hierarchically grouped graph with two components that are laid out by {@link yfiles.orthogonal.OrthogonalLayout OrthogonalLayout} as separate
     * components.
     * </p>
     * @class
     * @extends {yfiles.layout.ComponentLayout}
     */
    export interface IsolatedGroupComponentLayout extends yfiles.layout.ComponentLayout{}
    export class IsolatedGroupComponentLayout {
      /**
       * Creates an instance of {@link yfiles.layout.IsolatedGroupComponentLayout} with the given layout algorithm as
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] the core layout algorithm that will be invoked for each component of the graph
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Determines the nodes that belong to the same graph component.
       * <p>
       * Unlike
       * {@link yfiles.layout.ComponentLayout}, this implementation additionally identifies isolated subgraphs within a group as separate
       * components.
       * </p>
       * @param graph the input graph
       * @param compNumber the {@link } that will be filled by the layout algorithm, such that it contains the zero-based index
       * of the component number to which each node belongs
       * @returns the number of separate components
       * @protected
       */
      findIsolatedGraphComponents(graph:yfiles.layout.LayoutGraph,compNumber:yfiles.algorithms.INodeMap):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This {@link yfiles.layout.IGroupBoundsCalculator} allows custom insets for group nodes and guarantees that their provided
     * minimum sizes will be maintained.
     * <p>
     * As a subclass of {@link yfiles.layout.InsetsGroupBoundsCalculator}, this class supports custom insets for each group node.
     * Furthermore, it guarantees that a desired minimum size is maintained. The minimum sizes are provided by a {@link yfiles.algorithms.IDataProvider}
     * registered with the graph using the specified {@link yfiles.layout.MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum node size key}.
     * </p>
     * @class
     * @extends {yfiles.layout.InsetsGroupBoundsCalculator}
     */
    export interface MinimumSizeGroupBoundsCalculator extends yfiles.layout.InsetsGroupBoundsCalculator{}
    export class MinimumSizeGroupBoundsCalculator {
      /**
       * Creates a new instance of {@link yfiles.layout.MinimumSizeGroupBoundsCalculator} with the given key as {@link yfiles.layout.MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum node size key}.
       * @param minSizeDataProviderKey the {@link } key to use as {@link #minimumNodeSizeDpKey minimum size key}
       */
      constructor(minSizeDataProviderKey:Object);
      /**
       * Creates a new instance of {@link yfiles.layout.MinimumSizeGroupBoundsCalculator} with default settings.
       */
      constructor();
      /**
       * Gets or sets how the nodes are resized horizontally if the calculated width is smaller than the preferred minimum width
       * of a group node.
       * <p>
       * The alignment is defined to be a value between <code>0</code> and <code>1</code>, such that:
       * </p>
       * <ul>
       * <li>A value of <code>0</code> means that the nodes grow to the right while the left border remains unchanged.</li>
       * <li>A value of <code>1</code> means that the nodes grow to the left while the right border remains unchanged.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if the given value does not lie within <code>[0,1]</code>
       * @type {number}
       */
      xAlignment:number;
      /**
       * Gets or sets how the nodes are resized vertically if the calculated height is smaller than the preferred minimum height
       * of a group node.
       * <p>
       * The alignment is defined to be a value between <code>0</code> and <code>1</code>, such that:
       * </p>
       * <ul>
       * <li>A value of <code>0</code> means that the nodes grow downwards while the top border remains unchanged.</li>
       * <li>A value of <code>1</code> means that the nodes grow upwards while the bottom border remains unchanged.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if the given value does not lie within <code>[0,1]</code>
       * @type {number}
       */
      yAlignment:number;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that is used to access the minimum sizes for the
       * group nodes.
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} registered with this key must provide a mapping from each group node to a {@link yfiles.algorithms.YDimension}
       * object defining the minimum width and height of the group node.
       * </p>
       * @see yfiles.layout.MinimumSizeGroupBoundsCalculator#defaultMinimumNodeSize
       * @type {Object}
       */
      minimumNodeSizeDpKey:Object;
      /**
       * Gets or sets the default minimum size (width and height) for group nodes.
       * <p>
       * The default minimum size is used if the {@link yfiles.algorithms.IDataProvider} registered with the {@link yfiles.layout.MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey minimum size key} does not
       * contain a mapping for a group node.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given {@link yfiles.algorithms.YDimension} is <code>null</code>
       * @see yfiles.layout.MinimumSizeGroupBoundsCalculator#minimumNodeSizeDpKey
       * @type {yfiles.algorithms.YDimension}
       */
      defaultMinimumNodeSize:yfiles.algorithms.YDimension;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm recursively traverses a hierarchically organized graph in a bottom-up fashion and applies a
     * specified layout algorithm to the contents (direct children) of each group node.
     * <p>
     * <b>Layout Style</b> The way a graph is arranged depends on the {@link yfiles.layout.ILayoutAlgorithm layout algorithms} which are applied to the different group nodes. {@link yfiles.layout.RecursiveGroupLayout}
     * is able to produce different layout styles for the content of each group node.
     * </p>
     * <p>
     * This layout algorithm can be either applied if a {@link yfiles.layout.ILayoutAlgorithm layout algorithm} cannot handle grouped graphs by itself or if the
     * content of (some) group nodes should be arranged differently.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"styles":{"6":{"c":1}},"n":[[-17.5,755,335,340,2],[166.25,770],[247.5,840],[85,840],[172.5,910],[272.5,910],[72.5,910],[272.5,980],[222.5,910],[97.5,980],[47.5,980],[-2.5,910],[147.5,980],[122.5,1050],[172.5,1050],[197.5,980],[-2.5,980],[-2.5,1050],[929.97,593,673.03,664,2],[1028.43,230.79,493.13,238.42,2],[1451.74,249.79,54.83,54.83],[1043.43,336.11,54.83,54.83],[1376.74,346.07],[1143.26,334.82],[1223.95,267.93,37.09,37.09],[1223.95,335.02,37.09,37.09],[1223.95,402.12,37.09,37.09],[1306.05,338.57],[351.41,46.3,547.17,607.39,2],[392.91,738,464.18,374,2],[600.69,849.79,44.43,44.43],[803.73,927.82,38.37,38.37],[600.69,1024.79,44.43,44.43],[707.91,932],[707.91,1032],[507.91,757],[503.73,852.82,38.37,38.37],[407.91,857],[1329,1038,6],[1269,795,6],[1472,612,6],[1411,1025,6],[1427,744,6],[1355,779,6],[1157,888,6],[1108,838,6],[1230,723,6],[1351,946,6],[1185,810,6],[1387,847,6],[1185,956,6],[1221,1212,6],[1311,715,6],[962,988,6],[1558,700,6],[1053,639,6],[1264,993,6],[1509,1121,6],[1143,742,6],[1453,871,6],[1248,1077,6],[1430,943,6],[1093,947,6],[1173,1025,6],[1249.93,858.93,83.14,83.14,6],[406.42,266.45,6],[535.29,224.58,6],[407.4,568.81,54.23,54.23,6],[366.41,92.7,54.23,54.23,6],[464.05,443.83,6],[450.74,65.3,54.23,54.23,6],[464.05,224.58,6],[592.93,401.96,6],[714.87,571.77,41.35,41.35,6],[842.24,530.39,41.35,41.35,6],[793.59,463.42,41.35,41.35,6],[793.59,597.35,41.35,41.35,6],[614.94,334.2,6],[535.29,443.83,6],[714.87,489,41.35,41.35,6],[406.42,401.96,6],[592.93,266.45,6],[384.4,334.2,6],[595.06,86,54.23,54.23,6],[668.28,135.99,54.23,54.23,6],[720.4,207.72,54.23,54.23,6],[745.32,292.81,54.23,54.23,6]],"e":[[1,2],[1,3],[3,4],[2,5],[3,6],[5,7],[2,8],[6,9],[6,10],[3,11],[4,12],[12,13],[12,14],[4,15],[11,16],[16,17],[0,19,0,-170,0,-119.21,[150,0,1275,0]],[21,22,27.41,13.71,-15,7.5,[1108.26,377.23,1158.26,454.21,1336.05,454.21,1361.74,368.57]],[22,20,15,-0,-27.41,18.28,[1416.74,361.07,1436.74,295.48]],[21,23,27.41,-13.71,-15,-0],[23,20,15,-11.25,-27.41,-18.28,[1183.26,338.57,1208.95,252.93,1391.74,252.93,1436.74,258.93]],[23,24,15,-3.75,-18.55,-0,[1183.26,346.07,1208.95,286.48]],[24,20,18.55,-9.27,-27.41,-0],[23,25,15,3.75,-18.55,-0],[23,26,15,11.25,-18.55,-0,[1183.26,361.07,1208.95,420.66]],[25,27,18.55,-0,-15,-0],[24,27,18.55,9.27,-15,-10,[1271.05,295.75,1291.05,343.57]],[26,27,18.55,-0,-15,10,[1271.05,420.66,1291.05,363.57]],[27,22,15,-0,-15,-7.5],[29,28,0,-187,0,303.7],[31,30,0,-19.18,22.21,0,[822.91,872]],[31,32,0,19.18,0,22.21,[822.91,1097,622.91,1097]],[2,32],[32,30,0,-22.21,0,22.21],[31,33,-19.18,0,15,0],[33,32,-15,0,12.5,-22.21,[635.41,947]],[33,34,0,15,0,-15],[34,32,-15,0,22.21,0],[34,31,15,0,-12.5,19.18,[810.41,1047]],[35,30,15,0,0,-22.21,[622.91,772]],[35,36,0,15,0,-19.18],[32,36,-22.21,-12.5,0,19.18,[522.91,1034.5]],[36,30,19.18,0,-22.21,0],[36,37,-19.18,0,15,0],[32,37,-22.21,0,0,15,[422.91,1047]],[37,35,0,-15,-15,0,[422.91,772]],[42,40],[42,54],[41,57],[58,55],[60,51],[62,53],[64,38],[64,39],[64,41,[1330.6,964.06,1337.99,974.52,1346.6,984.13,1356.64,992.4]],[64,42,[1301.28,892.93,1380.34,828.69,1390.42,819.61,1400.14,810]],[64,43],[64,44],[64,45],[64,46,[1257.8,823.49,1254.58,812.63,1252.57,801.6]],[64,47],[64,48],[64,49],[64,50],[64,52,[1313.2,816.82,1315.29,805.72]],[64,56],[64,58,[1227.75,807.43,1218.48,798.12,1198.26,783.57]],[64,59,[1401.91,896.21,1413.4,895.03]],[64,60,[1249.83,992.24,1246.96,1005.02,1247.3,1018.2]],[64,61],[64,62],[64,63],[40,19],[69,67],[65,71],[66,71],[71,70],[71,69],[71,68],[72,21],[73,74],[75,74],[73,76],[74,76],[77,72],[77,71],[71,78],[78,72],[78,69],[72,79],[79,75],[79,73],[69,80],[72,80],[66,81],[81,77],[65,82],[82,81],[82,80],[82,77],[83,81],[84,81],[85,81],[86,81],[51,33]],"vp":[-18.0,0.0,1621.0,1257.0]}}<b>Concept</b>
     * </p>
     * <p>
     * {@link yfiles.layout.RecursiveGroupLayout} uses a hierarchy tree representation of the grouped graph in which the content nodes
     * are the children of their containing group node. That way, it can traverse the tree recursively while arranging only the
     * direct children of each group node. The layout algorithm starts by arranging the leaves in the hierarchy tree, then
     * works its way up to the root computing the layout for each group node in the tree.
     * </p>
     * <p>
     * All nodes other than the direct children are temporarily hidden. The layout algorithm performs two steps for each group
     * node.
     * </p>
     * <ol>
     * <li>
     * It arranges the direct children using either the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} or a special layout algorithm retrieved from a {@link yfiles.algorithms.IDataProvider}
     * registered with
     * {@link yfiles.layout.RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY}. The content of group nodes among the children is already
     * arranged at this time and will be ignored. These group nodes are handled like normal nodes with a size that encloses the
     * content.
     * </li>
     * <li>
     * Then {@link yfiles.layout.RecursiveGroupLayout} computes the final size of the group node using an implementation of
     * {@link yfiles.layout.IGroupBoundsCalculator}. Customized {@link yfiles.layout.IGroupBoundsCalculator}s can be specified using
     * {@link yfiles.layout.RecursiveGroupLayout#groupBoundsCalculator}. Aside from the resulting layout, this size is used in the following iteration.
     * </li>
     * </ol>
     * <p>
     * After a layout is applied to all group nodes, the layout algorithm computes routes for the edges whose source node is
     * located at a different hierarchy level than its target node. The edge routing algorithm for these so-called inter-edges
     * can be {@link yfiles.layout.RecursiveGroupLayout#interEdgeRouter customized}.
     * </p>
     * <p>
     * Note that {@link yfiles.layout.RecursiveGroupLayout} can run without a
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. In this case no layout is calculated, instead the group node bounds are merely adjusted to fit
     * their respective contents.
     * </p>
     * <p>
     * <b>Features</b> There are two alternatives for applying different layout styles to the contents of group nodes:
     * </p>
     * <ol>
     * <li>
     * Mapping each group node to a corresponding {@link yfiles.layout.ILayoutAlgorithm layout algorithm} by registering a {@link yfiles.algorithms.IDataProvider} with key
     * {@link yfiles.layout.RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY}. The content of the hierarchy root is arranged with the
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * </li>
     * <li>Using {@link yfiles.layout.LayoutMultiplexer} as {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.</li>
     * </ol>
     * <p>
     * Since {@link yfiles.layout.RecursiveGroupLayout} delegates the actual arrangement of the graph to other layout algorithms, it
     * will support the same features as the currently used layout algorithm.
     * </p>
     * <p>
     * The improvement of the routing of inter-edges is based on the {@link yfiles.layout.RecursiveGroupLayout#autoAssignPortCandidates insertion} of {@link yfiles.layout.PortCandidate}s or the {@link yfiles.layout.RecursiveGroupLayout#replacePortConstraints conversion}
     * of {@link yfiles.layout.PortConstraint}s into {@link yfiles.layout.PortCandidate}s. Hence, they only work well if the applied layout
     * algorithm supports {@link yfiles.layout.PortCandidate}s.
     * </p>
     * <p>
     * This algorithm also provides a {@link yfiles.layout.RecursiveGroupLayout#fromSketchMode From Sketch mode} that should be activated if the applied layout algorithm runs in From
     * Sketch mode, too. Otherwise, the initial coordinates may not be considered correctly.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface RecursiveGroupLayout extends yfiles.layout.LayoutStageBase{}
    export class RecursiveGroupLayout {
      /**
       * Creates a new instance of {@link yfiles.layout.RecursiveGroupLayout} with default settings using the given {@link yfiles.layout.ILayoutAlgorithm layout algorithm} and
       * {@link yfiles.layout.IGroupBoundsCalculator} implementation.
       * @param core the layout algorithm that is applied in each step of the recursion
       * @param gbc the {@link } for calculating group sizes
       */
      constructor(core:yfiles.layout.ILayoutAlgorithm,gbc:yfiles.layout.IGroupBoundsCalculator);
      /**
       * Creates a new instance of {@link yfiles.layout.RecursiveGroupLayout} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The layout algorithm that is applied in each recursion step.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Reroutes the given inter-edges using the current {@link yfiles.layout.RecursiveGroupLayout#interEdgeRouter edge routing algorithm}.
       * <p>
       * This method is called after calculating the overall layout when the positions of all nodes and normal edges are fixed.
       * </p>
       * <p>
       * If no {@link yfiles.layout.RecursiveGroupLayout#interEdgeRouter inter-edge router} is specified, this method {@link yfiles.layout.LayoutGraphUtilities#resetPath resets the path} of all inter-edges that don't connect to the proper
       * location within the group. This may happen for inter-edges without
       * {@link yfiles.layout.PortCandidate}s or if the applied layout algorithm doesn't support such constraints.
       * </p>
       * @param graph the input graph
       * @param interEdges the edges which traverse the boundary of a group node
       * @see yfiles.layout.RecursiveGroupLayout#autoAssignPortCandidates
       * @see yfiles.layout.RecursiveGroupLayout#replacePortConstraints
       * @see yfiles.layout.RecursiveGroupLayout#interEdgeRouter
       * @protected
       */
      routeInterEdges(graph:yfiles.layout.LayoutGraph,interEdges:yfiles.algorithms.EdgeList):void;
      /**
       * Data provider key for arranging the content of each group node with an individual layout algorithm
       * <p>
       * The specified layouter instance is applied to the content of the group node. To arrange the top level elements the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}
       * is used.
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#NULL_LAYOUT
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.layout.ILayoutAlgorithm>}
       */
      static GROUP_NODE_LAYOUT_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.layout.ILayoutAlgorithm>;
      /**
       * Data provider key for assigning source split ids to edges connecting to group nodes
       * <p>
       * The edges will be aligned with edges that connect to the same group node and have the same split id at their source
       * (preferably) or target.
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#TARGET_SPLIT_ID_DP_KEY
       * @see yfiles.hierarchic.HierarchicLayout
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#directGroupContentEdgeRouting
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static SOURCE_SPLIT_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data provider key for assigning target split ids to edges connecting to group nodes
       * <p>
       * The edges will be aligned with edges that connect to the same group node and have the same split id at their source or
       * target (preferably).
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#SOURCE_SPLIT_ID_DP_KEY
       * @see yfiles.hierarchic.HierarchicLayout
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#directGroupContentEdgeRouting
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static TARGET_SPLIT_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * A constant that represents a {@link yfiles.layout.ILayoutAlgorithm} implementation that does nothing.
       * <p>
       * This implementation can be assigned to group nodes to keep their content unchanged. The layout algorithm will still
       * calculate the sizes of the group nodes.
       * </p>
       * @const
       * @static
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      static NULL_LAYOUT:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets whether or not to consider the initial coordinates of the graph elements.
       * <p>
       * When using the initial coordinates, {@link yfiles.layout.RecursiveGroupLayout} sets the coordinates of the nodes to their
       * initial position before the corresponding {@link yfiles.layout.ILayoutAlgorithm layout algorithm} is called.
       * </p>
       * @type {boolean}
       */
      fromSketchMode:boolean;
      /**
       * Gets or sets whether or not temporary {@link yfiles.layout.PortCandidate}s are inserted to improve the routing of inter-edges.
       * <p>
       * If enabled, {@link yfiles.layout.RecursiveGroupLayout} will insert {@link yfiles.layout.PortCandidate}s for all inter-edges that cross a
       * group node border. Those {@link yfiles.layout.PortCandidate}s are located at the relative position of the real source/target
       * node. Inter-edges that connect to such {@link yfiles.layout.PortCandidate}s will be routed when the layout of the containing
       * group node is calculated and will not be {@link yfiles.layout.RecursiveGroupLayout#routeInterEdges rerouted} later. This may produce more suitable edge routes but cannot prevent
       * edges from crossing nodes.
       * </p>
       * <p>
       * Without temporary or user specified {@link yfiles.layout.PortCandidate}s, inter-edges will always end at the border/center of
       * the corresponding group node. Thus, they are rerouted afterwards using an {@link yfiles.layout.RecursiveGroupLayout#interEdgeRouter edge routing algorithm}.
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#routeInterEdges
       * @see yfiles.layout.RecursiveGroupLayout#interEdgeRouter
       * @see yfiles.layout.PortCandidate
       * @type {boolean}
       */
      autoAssignPortCandidates:boolean;
      /**
       * Gets or sets whether or not {@link yfiles.layout.PortConstraint}s of inter-edges are temporarily replaced by
       * {@link yfiles.layout.PortCandidate}s.
       * <p>
       * If disabled, inter-edges will always end at the border/center of the corresponding group node, even if those edges have
       * port constraints. Thus, they are {@link yfiles.layout.RecursiveGroupLayout#routeInterEdges rerouted} later without considering the constraint. Enabling this settings may produce
       * more suitable edge routes but cannot prevent edges from crossing nodes.
       * </p>
       * <p>
       * Port candidates are automatically redirected to their original location. Hence, enabling this option may produce more
       * suitable edge routes if the layout algorithm applied to the content of a group node can handle port candidates.
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#routeInterEdges
       * @see yfiles.layout.PortCandidate
       * @type {boolean}
       */
      replacePortConstraints:boolean;
      /**
       * Gets or sets whether empty group nodes are handled like group nodes with content or like normal nodes.
       * <p>
       * If they are handled like other group nodes, {@link yfiles.layout.RecursiveGroupLayout} will resize them according to their
       * (non-existing) content. This results in small empty group nodes. Handled like normal nodes, empty group nodes will keep
       * their initial size.
       * </p>
       * @type {boolean}
       */
      considerEmptyGroups:boolean;
      /**
       * Gets or sets the current edge routing algorithm for handling inter-edges.
       * <p>
       * During layout, edges that connect from outside a group node to the content inside (inter-edges) are temporarily
       * connected to the group node itself. Hence, these edges have to be routed after restoring the original graph structure
       * using this edge routing algorithm.
       * </p>
       * <p>
       * It is required that a suitable {@link yfiles.layout.RecursiveGroupLayout#interEdgesDpKey selection key} is specified. The same selection key must be used for setting the sphere of
       * action for the edge router.
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#interEdgesDpKey
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      interEdgeRouter:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the key for marking the inter-edges to be routed.
       * <p>
       * The key should be used by the specified {@link yfiles.layout.RecursiveGroupLayout#interEdgeRouter inter-edge routing algorithm} to obtain the edges to be routed. This layouter
       * automatically marks these edges and registers the {@link yfiles.algorithms.IDataProvider} using the specified key.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @see yfiles.layout.RecursiveGroupLayout#interEdgeRouter
       * @type {Object}
       */
      interEdgesDpKey:Object;
      /**
       * Gets or sets a {@link yfiles.layout.IGroupBoundsCalculator} which computes the sizes of all group nodes.
       * <p>
       * This {@link yfiles.layout.IGroupBoundsCalculator} is used each time after calculating the layout for a content graph.
       * </p>
       * @type {yfiles.layout.IGroupBoundsCalculator}
       */
      groupBoundsCalculator:yfiles.layout.IGroupBoundsCalculator;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.ILayoutStage} that temporarily removes all nodes and edges with <code>NaN</code>-coordinates.
     * <p>
     * This <code>layout stage</code> may be necessary when a {@link yfiles.layout.ILayoutAlgorithm} takes the initial coordinates into account. In
     * this case initial coordinates containing <code>NaN</code>-values, can cause exceptions or strange results.
     * </p>
     * <p>
     * <code>Layouter</code>s that consider initial coordinates:
     * </p>
     * <ul>
     * <li>{@link yfiles.hierarchic.HierarchicLayout} in incremental mode</li>
     * <li>{@link yfiles.orthogonal.OrthogonalLayout} with activated from sketch drawing</li>
     * <li>{@link yfiles.organic.OrganicLayout} for a subset of nodes</li>
     * <li>{@link yfiles.tree.TreeLayout} when using a from sketch comparator.</li>
     * <li>{@link yfiles.tree.BalloonLayout} in incremental mode</li>
     * <li>{@link yfiles.partial.PartialLayout}</li>
     * <li>
     * all edge routing algorithms: {@link yfiles.router.EdgeRouter}, {@link yfiles.router.ChannelEdgeRouter}, {@link yfiles.router.BusRouter},
     * {@link yfiles.router.OrganicEdgeRouter}
     * </li>
     * </ul>
     * <p>
     * Since this stage only hides the invalid graph elements and doesn't apply any new coordinates, it is not defined where
     * those nodes and edges will be placed within the calculated layout.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface HandleNaNCoordinatesStage extends yfiles.layout.LayoutStageBase{}
    export class HandleNaNCoordinatesStage {
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.IEdgeLabelLayout} defines the properties of one edge label from the perspective of a labeling algorithm.
     * <p>
     * Note: To avoid inconsistencies, changing the {@link yfiles.layout.ILabelLayout#modelParameter model parameter} should always be combined with updating the corresponding
     * {@link yfiles.layout.ILabelLayout#orientedBox oriented box}.
     * </p>
     * @interface
     * @implements {yfiles.layout.ILabelLayout}
     */
    export interface IEdgeLabelLayout extends Object,yfiles.layout.ILabelLayout{
      /**
       * Gets the label model that describes the valid locations for this label.
       * @abstract
       * @type {yfiles.layout.IEdgeLabelLayoutModel}
       */
      labelModel:yfiles.layout.IEdgeLabelLayoutModel;
      /**
       * Gets the preferred placement for this edge label, which can be considered by {@link yfiles.layout.ILayoutAlgorithm layout algorithms}.
       * @abstract
       * @type {yfiles.layout.PreferredPlacementDescriptor}
       */
      preferredPlacementDescriptor:yfiles.layout.PreferredPlacementDescriptor;
    }
    var IEdgeLabelLayout:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface defines the properties of the model associated with an {@link yfiles.layout.IEdgeLabelLayout}.
     * <p>
     * An {@link yfiles.layout.IEdgeLabelLayoutModel} provides a set of possible <em>candidates</em> for the placement of an edge label. It also defines a
     * <em>parameter</em> that describes the current location of the label.
     * </p>
     * <p>
     * It is important that all state information is encapsulated in the {@link yfiles.layout.IEdgeLabelLayoutModel#createModelParameter model parameter}. {@link yfiles.layout.IEdgeLabelLayoutModel}
     * instances may be shared between multiple edge labels but produce different parameters for labels with different
     * locations.
     * </p>
     * @interface
     */
    export interface IEdgeLabelLayoutModel extends Object{
      /**
       * Creates a model parameter that represents the given edge label position within this model.
       * <p>
       * The created model parameter is the closest parameter representation of the given label location that can be achieved
       * within this model.
       * </p>
       * <p>
       * This parameter can be passed to {@link yfiles.layout.IEdgeLabelLayoutModel#getLabelPlacement} to retrieve the current label
       * box.
       * </p>
       * <p>
       * A model parameter can be an arbitrary {@link Object}. However it must contain all information to allow restoring of the encoded
       * location with this {@link yfiles.layout.IEdgeLabelLayoutModel}.
       * </p>
       * @param labelBounds the oriented box of the label, encoding the label location for which the parameter should be created
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @returns the model parameter representing the given label location
       * @see yfiles.layout.IEdgeLabelLayoutModel#getLabelPlacement
       * @abstract
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout):Object;
      /**
       * Returns all {@link yfiles.layout.EdgeLabelCandidate}s that describe valid label positions within this model.
       * <p>
       * In case the implementing {@link yfiles.layout.IEdgeLabelLayoutModel} allows every possible location, this method always returns
       * the current location as a {@link yfiles.layout.LabelCandidate}.
       * </p>
       * @param labelLayout the label for which candidates should be generated
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @returns a list of {@link } instances
       * @abstract
       */
      getLabelCandidates(labelLayout:yfiles.layout.IEdgeLabelLayout,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * Returns the oriented box of the label for the position encoded by the given model parameter.
       * @param labelSize the width and height of the label
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @param parameter the model parameter that describes the abstract position of the label within this model
       * @returns the oriented bounds of the label
       * @abstract
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout,parameter:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets a model parameter that encodes the default position of this model's allowed edge label positions.
       * <p>
       * This parameter can be passed to {@link yfiles.layout.IEdgeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
       * label box.
       * </p>
       * @abstract
       * @type {Object}
       */
      defaultParameter:Object;
    }
    var IEdgeLabelLayoutModel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An {@link yfiles.layout.IEdgeLayout} encapsulates the layout information for an edge.
     * <p>
     * The layout information consists of the two end points of the edge layout and a sequence of control points that determine
     * the visual path of the edge.
     * </p>
     * @interface
     */
    export interface IEdgeLayout extends Object{
      /**
       * Adds a control point to the end of the control point sequence.
       * @param x the absolute x-coordinate of the new control point
       * @param y the absolute y-coordinate of the new control point
       * @abstract
       */
      addPoint(x:number,y:number):void;
      /**
       * Removes all control points from this edge layout.
       * <p>
       * The {@link yfiles.layout.IEdgeLayout} still keeps the {@link yfiles.layout.IEdgeLayout#sourcePoint source point} and the {@link yfiles.layout.IEdgeLayout#targetPoint target point}.
       * </p>
       * @abstract
       */
      clearPoints():void;
      /**
       * Returns the control point at the given position in the sequence of control points.
       * <p>
       * The first control point in the sequence has index <code>0</code> and the last control point has index {@link yfiles.layout.IEdgeLayout#pointCount}
       * <code>-1</code>.
       * </p>
       * @param index the position of the control point in the control point sequence
       * @returns the absolute coordinates of the control point at the given index
       * @see yfiles.layout.IEdgeLayout#setPoint
       * @abstract
       */
      getPoint(index:number):yfiles.algorithms.YPoint;
      /**
       * Returns the number of control points of the edge.
       * <p>
       * The source and target points are not included in the point count.
       * </p>
       * @returns the number of control points
       * @abstract
       */
      pointCount():number;
      /**
       * Sets the absolute coordinates of the control point at the given position in the sequence of control points.
       * <p>
       * The first control point in the sequence has index <code>0</code> and the last control point has index {@link yfiles.layout.IEdgeLayout#pointCount}
       * <code>-1</code>.
       * </p>
       * @param index the position of the changing control point in the sequence
       * @param x the new absolute x-coordinate of the control point at the given index
       * @param y the new absolute y-coordinate of the control point at the given index
       * @abstract
       */
      setPoint(index:number,x:number,y:number):void;
      /**
       * Gets or sets the relative coordinates of the first end point of this layout.
       * <p>
       * These coordinates are relative to the center location of the edge's source node. To obtain the absolute coordinates of
       * the end point, the relative coordinates have to be added to the center coordinates of the source
       * {@link yfiles.layout.INodeLayout}.
       * </p>
       * @see yfiles.layout.LayoutGraph#getCenterX
       * @see yfiles.layout.LayoutGraph#getCenterY
       * @see yfiles.layout.INodeLayout#x
       * @see yfiles.layout.INodeLayout#y
       * @abstract
       * @type {yfiles.algorithms.YPoint}
       */
      sourcePoint:yfiles.algorithms.YPoint;
      /**
       * Gets or sets the relative coordinates of the second end point of this layout.
       * <p>
       * These coordinates are relative to the center location of the edge's target node. To obtain the absolute coordinates of
       * the end point, the relative coordinates have to be added to the center coordinates of the target
       * {@link yfiles.layout.INodeLayout}.
       * </p>
       * @see yfiles.layout.LayoutGraph#getCenterX
       * @see yfiles.layout.LayoutGraph#getCenterY
       * @see yfiles.layout.INodeLayout#x
       * @see yfiles.layout.INodeLayout#y
       * @abstract
       * @type {yfiles.algorithms.YPoint}
       */
      targetPoint:yfiles.algorithms.YPoint;
    }
    var IEdgeLayout:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An {@link yfiles.layout.IIntersectionCalculator} is responsible for calculating the intersection point between an edge and the
     * visual representation of its source or target node.
     * <p>
     * This interface is especially useful in conjunction with {@link yfiles.layout.PortCalculator} when adjusting the final port
     * locations after the layout is calculated.
     * </p>
     * @see yfiles.layout.PortCalculator
     * @interface
     */
    export interface IIntersectionCalculator extends Object{
      /**
       * Calculates an intersection point between the edge and the given {@link yfiles.layout.INodeLayout}.
       * <p>
       * Implementing classes use a point on the connecting edge segment (specified with an offset to the node's center) along
       * with a direction pointing to the node and calculate the intersection between this line and the node.
       * </p>
       * @param nodeLayout the layout information of the node
       * @param xOffset the x-coordinate of a point on the last line segment relative to the node's center
       * @param yOffset the y-coordinate of a point on the last line segment relative to the node's center
       * @param dx the x-component of the normalized direction vector indicating the direction of the segment pointing towards the node
       * @param dy the y-component of the normalized direction vector indicating the direction of the segment pointing towards the node
       * @returns the intersection coordinates relative to the node's center or <code>null</code> if no such intersection was found
       * @abstract
       */
      calculateIntersectionPoint(nodeLayout:yfiles.layout.INodeLayout,xOffset:number,yOffset:number,dx:number,dy:number):yfiles.algorithms.YPoint;
    }
    var IIntersectionCalculator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Helper class that declares several keys for {@link yfiles.algorithms.IDataProvider} that provide
     * {@link yfiles.layout.IIntersectionCalculator}.
     * @class
     * @static
     */
    export interface IntersectionCalculatorKeys extends Object{}
    export class IntersectionCalculatorKeys {
      /**
       * Data provider key for providing an individual intersection calculation for each edge source in the graph
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.layout.IIntersectionCalculator>}
       */
      static SOURCE_INTERSECTION_CALCULATOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.layout.IIntersectionCalculator>;
      /**
       * Data provider key for providing an individual intersection calculation for each edge target in the graph
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.layout.IIntersectionCalculator>}
       */
      static TARGET_INTERSECTION_CALCULATOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.layout.IIntersectionCalculator>;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.ILabelLayout} defines the properties of a label from the perspective of a labeling algorithm.
     * <p>
     * Both node and edge labels need to implement this interface. It describes the size and position of a label.
     * </p>
     * <p>
     * Note: To avoid inconsistencies, changing the {@link yfiles.layout.ILabelLayout#modelParameter model parameter} should always be combined with updating the corresponding
     * {@link yfiles.layout.ILabelLayout#orientedBox oriented box}.
     * </p>
     * @interface
     */
    export interface ILabelLayout extends Object{
      /**
       * Gets the bounding box of the label.
       * <p>
       * The bounding box is the smallest, paraxial rectangle that contains the whole label.
       * </p>
       * @abstract
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
      /**
       * Gets the box of the label.
       * <p>
       * This box matches the area that is covered by the label. The {@link yfiles.algorithms.YOrientedRectangle#setUpVector up vector} of the rectangle encodes the orientation of the
       * label.
       * </p>
       * @abstract
       * @type {yfiles.algorithms.YOrientedRectangle}
       */
      orientedBox:yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets or sets the label model parameter that describes the position of this label.
       * <p>
       * The model parameter must fit the according label model.
       * </p>
       * @abstract
       * @type {Object}
       */
      modelParameter:Object;
    }
    var ILabelLayout:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * {@link yfiles.layout.ILabelLayoutFactory} provides methods to create, add and remove node/edge labels for
     * {@link yfiles.layout.LayoutGraph}s.
     * <p>
     * It can be used, e.g., to temporarily add labels during layout calculation.
     * </p>
     * <p>
     * Note that original labels which are removed during a layout calculation have to be re-added when the layout calculation
     * is done.
     * </p>
     * <p>
     * An instance of this factory can be obtained with method {@link yfiles.layout.LayoutGraphUtilities#getLabelFactory}. This method
     * uses the {@link yfiles.layout.LayoutGraph#createLabelFactory} method to create the factory.
     * </p>
     * <p>
     * Note that each factory instance is bound to a specific graph instance, i.e., you can only create, add and remove labels
     * for the graph which was given to method {@link yfiles.layout.LayoutGraphUtilities#getLabelFactory}.
     * </p>
     * @interface
     */
    export interface ILabelLayoutFactory extends Object{
      /**
       * Adds the given {@link yfiles.layout.IEdgeLabelLayout} to the given edge.
       * <p>
       * The given {@link yfiles.layout.ILabelLayout} has to be created with method {@link yfiles.layout.ILabelLayoutFactory#createLabelLayout}
       * or {@link yfiles.layout.ILabelLayoutFactory#createLabelLayout}.
       * </p>
       * @param edge the edge
       * @param labelLayout the edge label layout that should be added to the given edge
       * @throws {Stubs.Exceptions.ArgumentError} if the given label layout has already been added to the graph
       * @see yfiles.layout.ILabelLayoutFactory#createLabelLayout
       * @see yfiles.layout.ILabelLayoutFactory#createLabelLayout
       * @abstract
       */
      addLabelLayout(edge:yfiles.algorithms.Edge,labelLayout:yfiles.layout.IEdgeLabelLayout):void;
      /**
       * Adds the given node label layout to the given node.
       * <p>
       * The given label layout has to be created with method {@link yfiles.layout.ILabelLayoutFactory#createLabelLayout} or
       * {@link yfiles.layout.ILabelLayoutFactory#createLabelLayout}.
       * </p>
       * @param node the node
       * @param labelLayout the node label layout that should be added to the given node
       * @throws {Stubs.Exceptions.ArgumentError} if the given label layout has already been added to the graph
       * @see yfiles.layout.ILabelLayoutFactory#createLabelLayout
       * @see yfiles.layout.ILabelLayoutFactory#createLabelLayout
       * @abstract
       */
      addLabelLayout(node:yfiles.algorithms.Node,labelLayout:yfiles.layout.INodeLabelLayout):void;
      /**
       * Creates a new {@link yfiles.layout.ILabelLayout} for the given edge.
       * @param edge the edge
       * @param orientedBox the box specifying the size and location of the label
       * @param model specifies the model of the created edge label
       * @param descriptor specifies the preferred placement descriptor of the created label. If this value is
       * <code>null</code>, the default descriptor is used
       * @returns a new edge label layout
       * @see yfiles.layout.ILabelLayoutFactory#addLabelLayout
       * @abstract
       */
      createLabelLayout(edge:yfiles.algorithms.Edge,orientedBox:yfiles.algorithms.YOrientedRectangle,model:yfiles.layout.IEdgeLabelLayoutModel,descriptor:yfiles.layout.PreferredPlacementDescriptor):yfiles.layout.IEdgeLabelLayout;
      /**
       * Creates a new {@link yfiles.layout.ILabelLayout} for the given node.
       * @param node the node
       * @param orientedBox the box specifying the size and location of the label
       * @param model specifies the model of the created node label
       * @returns a new node label layout
       * @see yfiles.layout.ILabelLayoutFactory#addLabelLayout
       * @abstract
       */
      createLabelLayout(node:yfiles.algorithms.Node,orientedBox:yfiles.algorithms.YOrientedRectangle,model:yfiles.layout.INodeLabelLayoutModel):yfiles.layout.INodeLabelLayout;
      /**
       * Creates a new {@link yfiles.layout.ILabelLayout} for the given edge.
       * <p>
       * This method should create a {@link yfiles.layout.ILabelLayout} with an {@link yfiles.layout.IEdgeLabelLayoutModel} that supports the
       * desired oriented label box exactly as given.
       * </p>
       * @param edge the edge
       * @param orientedBox the box specifying the size and location of the label
       * @returns a new edge label layout
       * @see yfiles.layout.ILabelLayoutFactory#addLabelLayout
       * @abstract
       */
      createLabelLayout(edge:yfiles.algorithms.Edge,orientedBox:yfiles.algorithms.YOrientedRectangle):yfiles.layout.IEdgeLabelLayout;
      /**
       * Creates a new {@link yfiles.layout.ILabelLayout} for the given node.
       * <p>
       * This method should create a {@link yfiles.layout.ILabelLayout} with a {@link yfiles.layout.INodeLabelLayoutModel} that supports the
       * desired oriented label box exactly as given.
       * </p>
       * @param node the node
       * @param orientedBox the box specifying the size and location of the label
       * @returns a new node label layout
       * @see yfiles.layout.ILabelLayoutFactory#addLabelLayout
       * @abstract
       */
      createLabelLayout(node:yfiles.algorithms.Node,orientedBox:yfiles.algorithms.YOrientedRectangle):yfiles.layout.INodeLabelLayout;
      /**
       * Removes the given {@link yfiles.layout.IEdgeLabelLayout} from the given edge.
       * @param edge the edge
       * @param labelLayout the label layout that should be removed from the given edge
       * @see yfiles.layout.ILabelLayoutFactory#addLabelLayout
       * @abstract
       */
      removeLabelLayout(edge:yfiles.algorithms.Edge,labelLayout:yfiles.layout.IEdgeLabelLayout):void;
      /**
       * Removes the given node label layout from the given node.
       * @param node the node
       * @param labelLayout the label layout that should be removed from the given node
       * @see yfiles.layout.ILabelLayoutFactory#addLabelLayout
       * @abstract
       */
      removeLabelLayout(node:yfiles.algorithms.Node,labelLayout:yfiles.layout.INodeLabelLayout):void;
      /**
       * Gets the graph associated with this label factory instance.
       * @abstract
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
    }
    var ILabelLayoutFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * {@link yfiles.layout.ILayoutAlgorithm} describes the general interface for algorithms that perform a layout process on a
     * {@link yfiles.layout.LayoutGraph}.
     * <p>
     * The main method for invoking the layout calculation is
     * {@link yfiles.layout.ILayoutAlgorithm#applyLayout}. Implementing classes will arrange the graph there.
     * </p>
     * <p>
     * This class also provides keys to register {@link yfiles.algorithms.IDataProvider}s with the graph that contain information about
     * the nodes and edges in the graph. {@link yfiles.algorithms.IDataProvider}s registered with {@link yfiles.layout.LayoutKeys#NODE_ID_DP_KEY}
     * and {@link yfiles.layout.LayoutKeys#EDGE_ID_DP_KEY} contain unique identifiers for both nodes and edges. These identifiers allow
     * for ensuring consistent results even if the order of nodes and edges was changed between two layout runs.
     * <br />
     * The selection state of nodes and edges is stored in {@link yfiles.algorithms.IDataProvider}s registered with the keys {@link yfiles.layout.LayoutKeys#AFFECTED_NODES_DP_KEY}
     * and
     * {@link yfiles.layout.LayoutKeys#AFFECTED_EDGES_DP_KEY}. Based on this information, the layout algorithm may reduce its
     * calculations to the selected subset of nodes or edges.
     * </p>
     * @interface
     */
    export interface ILayoutAlgorithm extends Object{
      /**
       * Main layout routine that assigns new layout information to the given graph.
       * <p>
       * The call to this routine will only succeed if the layout algorithm can handle the input graph.
       * </p>
       * @param graph the input graph
       * @abstract
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
    }
    var ILayoutAlgorithm:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Helper class that provides unique identifiers for some general purpose {@link yfiles.algorithms.IDataProvider data providers}.
     * @class
     * @static
     */
    export interface LayoutKeys extends Object{}
    export class LayoutKeys {
      /**
       * Data provider key for looking up a unique identifier for each node in a graph
       * <p>
       * Layout algorithms may use this information to provide consistent layouts for multiple runs.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static NODE_ID_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Data provider key for looking up a unique identifier for each edge in a graph
       * <p>
       * Layout algorithms may use this information to provide consistent layouts for multiple runs.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static EDGE_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data provider key for looking up the selected state of the nodes in the graph
       * <p>
       * A layout algorithm can retrieve a {@link yfiles.algorithms.IDataProvider} registered with this key to apply a special handling
       * to the selected nodes. For example, the layout algorithm may restrict its scope, i.e., it only arranges the selected
       * nodes.
       * </p>
       * <p>
       * However, it is often more convenient to use a specific {@link yfiles.algorithms.IDataProvider} key for this purpose, e.g., if
       * you want to combine two layout algorithms of the same kind which have to operate on different subsets of the graph.
       * Thus, these algorithms may provide a method like <code>setSubgraphNodesDpKey(Object)</code> that customizes the {@link yfiles.algorithms.IDataProvider}
       * key allowing to specify different sets of selected nodes for nested layout algorithms.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static AFFECTED_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for looking up the selected state of the edges in the graph
       * <p>
       * A layout algorithm can retrieve a {@link yfiles.algorithms.IDataProvider} registered with this key to apply a special handling
       * to the selected edges. For example, the layout algorithm may restrict its scope, i.e., it only routes the selected
       * edges.
       * </p>
       * <p>
       * However, it is often more convenient to use a specific {@link yfiles.algorithms.IDataProvider} key for this purpose, e.g., if
       * you want to combine two layout algorithms of the same kind which have to operate on different subsets of the graph.
       * Thus, these algorithms may provide a method like <code>setSubgraphEdgesDpKey(Object)</code> that customizes the {@link yfiles.algorithms.IDataProvider}
       * key allowing to specify different sets of selected edges for nested layout algorithms.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.ILayoutStage} is a layout algorithm that represents a stage of a larger layout process.
     * <p>
     * It delegates the core layout process to another {@link yfiles.layout.ILayoutAlgorithm layout algorithm}. Implementing classes may perform pre-processing steps
     * before the {@link yfiles.layout.ILayoutStage#coreLayout core layout algorithm} gets called and post-processing steps afterwards.
     * </p>
     * @see yfiles.layout.MultiStageLayout
     * @interface
     * @implements {yfiles.layout.ILayoutAlgorithm}
     */
    export interface ILayoutStage extends Object,yfiles.layout.ILayoutAlgorithm{
      /**
       * Gets or sets the core layout algorithm.
       * <p>
       * This algorithm is wrapped by this stage. It is invoked in {@link yfiles.layout.ILayoutAlgorithm#applyLayout}. The {@link yfiles.layout.ILayoutStage}
       * may add pre- and post-processing steps before and after calling the core layout algorithm.
       * </p>
       * @abstract
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
    }
    var ILayoutStage:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * {@link yfiles.layout.INodeLabelLayout} defines the properties of one node label from the perspective of a labeling algorithm.
     * <p>
     * Note: To avoid inconsistencies, changing the {@link yfiles.layout.ILabelLayout#modelParameter model parameter} should always be combined with updating the corresponding
     * {@link yfiles.layout.ILabelLayout#orientedBox oriented box}.
     * </p>
     * @interface
     * @implements {yfiles.layout.ILabelLayout}
     */
    export interface INodeLabelLayout extends Object,yfiles.layout.ILabelLayout{
      /**
       * Gets the label model that describes the valid locations for this label.
       * @abstract
       * @type {yfiles.layout.INodeLabelLayoutModel}
       */
      labelModel:yfiles.layout.INodeLabelLayoutModel;
    }
    var INodeLabelLayout:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * {@link yfiles.layout.INodeLabelLayoutModel} defines the properties of the model associated with a
     * {@link yfiles.layout.INodeLabelLayout}.
     * <p>
     * A {@link yfiles.layout.INodeLabelLayoutModel} provides a set of possible <em>candidates</em> for the placement of a node label. It also defines a <em>parameter</em>
     * that describes the current location of the label.
     * </p>
     * <p>
     * It is important that all state information is encapsulated in the {@link yfiles.layout.INodeLabelLayoutModel#createModelParameter model parameter}. {@link yfiles.layout.INodeLabelLayoutModel}
     * instances may be shared between multiple node labels but produce different parameters for labels with different
     * locations.
     * </p>
     * @interface
     */
    export interface INodeLabelLayoutModel extends Object{
      /**
       * Creates a model parameter that represents the given node label position within this model.
       * <p>
       * The created model parameter is the closest parameter representation of the given label location that can be achieved
       * within this model.
       * </p>
       * <p>
       * This parameter can be passed to {@link yfiles.layout.INodeLabelLayoutModel#getLabelPlacement} to retrieve the current label
       * box.
       * </p>
       * <p>
       * A model parameter can be an arbitrary {@link Object}. However it must contain all information to allow restoring of the encoded
       * location with this {@link yfiles.layout.INodeLabelLayoutModel}.
       * </p>
       * @param labelBounds the box of the label, encoding the label location for which the parameter should be created
       * @param nodeLayout the layout of the node to which the label belongs
       * @returns the model parameter representing the given label location
       * @see yfiles.layout.INodeLabelLayoutModel#getLabelPlacement
       * @abstract
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,nodeLayout:yfiles.layout.INodeLayout):Object;
      /**
       * Returns all {@link yfiles.layout.NodeLabelCandidate}s that describe valid label positions within this model.
       * <p>
       * In case the implementing {@link yfiles.layout.INodeLabelLayoutModel} allows every possible location, this method always returns
       * the current location as a {@link yfiles.layout.LabelCandidate}.
       * </p>
       * @param labelLayout the label for which candidates should be generated
       * @param nodeLayout the layout of the node to which the label belongs
       * @returns a list of {@link } instances
       * @abstract
       */
      getLabelCandidates(labelLayout:yfiles.layout.INodeLabelLayout,nodeLayout:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * Returns the bounds of the label for the position encoded by the given model parameter.
       * @param labelSize the width and height of the label
       * @param nodeLayout the layout of the node to which the label belongs
       * @param param the model parameter that describes the abstract position of the label within this model
       * @returns the oriented box of the label
       * @abstract
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,nodeLayout:yfiles.layout.INodeLayout,param:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets a model parameter that encodes the default position of this model's allowed node label positions.
       * <p>
       * This parameter can be passed to {@link yfiles.layout.INodeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
       * label box.
       * </p>
       * @abstract
       * @type {Object}
       */
      defaultParameter:Object;
    }
    var INodeLabelLayoutModel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.layout.INodeLayout} encapsulates the layout information for a node.
     * <p>
     * The layout information consists of the size and position of the node.
     * </p>
     * @interface
     */
    export interface INodeLayout extends Object{
      /**
       * Sets the coordinates of the upper-left corner of the node.
       * @param x the new x-coordinate of the node
       * @param y the new y-coordinate of the node
       * @abstract
       */
      setLocation(x:number,y:number):void;
      /**
       * Sets the size of the node.
       * @param width the new width of the node
       * @param height the new height of the node
       * @abstract
       */
      setSize(width:number,height:number):void;
      /**
       * Gets the x-coordinate of the upper-left corner of the node.
       * <p>
       * The location of the node is described by the coordinates of the upper-left corner of the node.
       * </p>
       * @abstract
       * @type {number}
       */
      x:number;
      /**
       * Gets the y-coordinate of the upper-left corner of the node.
       * <p>
       * The location of the node is described by the coordinates of the upper-left corner of the node.
       * </p>
       * @abstract
       * @type {number}
       */
      y:number;
      /**
       * Gets the width of the node.
       * <p>
       * The width of the node is the horizontal distance the node spans.
       * </p>
       * @abstract
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of the node.
       * <p>
       * The height of the node is the vertical distance the node spans.
       * </p>
       * @abstract
       * @type {number}
       */
      height:number;
    }
    var INodeLayout:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * An {@link yfiles.layout.IPartitionInterEdgeRouter} is responsible for routing inter-edges.
     * <p>
     * An inter-edge is an edge that connects nodes that belong to different partitions.
     * </p>
     * @interface
     */
    export interface IPartitionInterEdgeRouter extends Object{
      /**
       * Routes inter-edges of the partitioned graph.
       * @param graph the input graph
       * @param partitionIDDP the partition ID for each node in the graph
       * @param interEdgeDP the map that marks inter-edges
       * @abstract
       */
      routeInterEdges(graph:yfiles.layout.LayoutGraph,partitionIDDP:yfiles.algorithms.IDataProvider,interEdgeDP:yfiles.algorithms.IDataProvider):void;
      /**
       * Gets or sets whether or not to route inter-edges only.
       * <p>
       * Inter-edges are edges that connect nodes from different partitions. If this feature is disabled, all edges are routed by
       * this routing algorithm.
       * </p>
       * @abstract
       * @type {boolean}
       */
      routeInterEdgesOnly:boolean;
    }
    var IPartitionInterEdgeRouter:{
      /**
       * Routes inter-edges using {@link yfiles.router.ChannelEdgeRouter}.
       * @param [router=null] The router instance to use for the edge routing. If not specified, a internal instance is used.
       * @returns 
       * @static
       */
      createChannelInterEdgeRouter?(router?:yfiles.router.ChannelEdgeRouter):yfiles.layout.IPartitionInterEdgeRouter;
      /**
       * Routes inter-edges using {@link yfiles.router.EdgeRouter}.
       * @param [router=null] The router instance to use for the edge routing. If not specified, a internal instance is used.
       * @returns 
       * @static
       */
      createPolylineInterEdgeRouter?(router?:yfiles.router.EdgeRouter):yfiles.layout.IPartitionInterEdgeRouter;
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.layout.IProfitModel} ranks
     * {@link yfiles.layout.LabelCandidate}s used for prioritizing certain label locations over others.
     * <p>
     * Generic labeling algorithm {@link yfiles.labeling.GenericLabeling} will use the ranking to decide which
     * {@link yfiles.layout.LabelCandidate}s to eliminate.
     * </p>
     * <p>
     * {@link yfiles.layout.LabelCandidate}s with a high profit will be preferred over
     * {@link yfiles.layout.LabelCandidate}s with a low profit.
     * </p>
     * @interface
     */
    export interface IProfitModel extends Object{
      /**
       * Returns the profit for placing a label using the given {@link yfiles.layout.LabelCandidate}.
       * <p>
       * Higher profit means better candidates. Hence, there is a higher probability that the candidate is chosen by a labeling
       * algorithm.
       * </p>
       * <p>
       * Profits need to have a value between <code>0</code> and <code>1</code>.
       * </p>
       * @param candidate the candidate
       * @returns the profit of the candidate
       * @abstract
       */
      getProfit(candidate:yfiles.layout.LabelCandidate):number;
    }
    var IProfitModel:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.layout.LabelCandidate} describes one valid placement for a label considering the label model.
     * <p>
     * Generic labeling algorithms place labels according to their label model. To get valid locations for the label, the label
     * model generates various
     * {@link yfiles.layout.LabelCandidate}s. One of these candidates will be chosen to provide the actual placement of the label.
     * </p>
     * @see yfiles.layout.INodeLabelLayoutModel
     * @see yfiles.layout.IEdgeLabelLayoutModel
     * @see yfiles.labeling.GenericLabeling
     * @class
     * @implements {yfiles.algorithms.IPlaneObject}
     */
    export interface LabelCandidate extends Object,yfiles.algorithms.IPlaneObject{}
    export class LabelCandidate {
      /**
       * Creates a new instance of {@link yfiles.layout.LabelCandidate} described by location, size, model parameter and internal flag.
       * @param pos the location of the upper left corner of the candidate
       * @param size the size of the candidate
       * @param param the parameters of the label model associated with this candidate
       * @param owner the label associated with the candidate
       * @param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.ILabelLayout,internal?:boolean);
      /**
       * Creates a new instance of {@link yfiles.layout.LabelCandidate} described by its box, model parameter and internal flag.
       * @param labelBox the box that specifies the candidate's size and position
       * @param param the parameters of the label model associated with this candidate
       * @param owner the label associated with the candidate
       * @param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
       */
      constructor(labelBox:yfiles.algorithms.YOrientedRectangle,param:Object,owner:yfiles.layout.ILabelLayout,internal?:boolean);
      /**
       * Replaces the model parameter and box of the owner with the model parameter and box of this candidate.
       * <p>
       * When this candidate is chosen to provide the placement of the label, this method transfers the model parameter and the
       * candidate box to the {@link yfiles.layout.ILabelLayout}.
       * </p>
       * @see yfiles.layout.LabelCandidate#owner
       * @see yfiles.layout.LabelCandidate#modelParameter
       * @see yfiles.layout.LabelCandidate#box
       */
      propagate():void;
      /**
       * Gets or sets the custom profit for this candidate.
       * <p>
       * The profit describes the priority of this candidate with respect to other candidates. Labeling algorithms will use the
       * profit to decide which candidate to choose.
       * </p>
       * <p>
       * The value is restricted to the interval <code>[0,1]</code>, where <code>1</code> specifies the highest possible profit.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value lies outside of <code>[0,1]</code>
       * @type {number}
       */
      customProfit:number;
      /**
       * Gets the model parameter that was used by the underlying model to generate this label candidate.
       * @type {Object}
       */
      modelParameter:Object;
      /**
       * Gets the coordinates of the upper-left corner of the candidate's bounds.
       * <p>
       * The bounds are the smallest paraxial box that contains the candidate.
       * </p>
       * @see yfiles.layout.LabelCandidate#boundingBox
       * @type {yfiles.algorithms.YPoint}
       */
      location:yfiles.algorithms.YPoint;
      /**
       * Gets the width and height of the candidate's bounds.
       * <p>
       * The bounds are the smallest paraxial box that contains the candidate.
       * </p>
       * @see yfiles.layout.LabelCandidate#boundingBox
       * @type {yfiles.algorithms.YDimension}
       */
      size:yfiles.algorithms.YDimension;
      /**
       * Gets the minimum x-coordinate of the candidate's bounds.
       * <p>
       * The bounds are the smallest paraxial box that contains the candidate.
       * </p>
       * @see yfiles.layout.LabelCandidate#location
       * @see yfiles.layout.LabelCandidate#boundingBox
       * @type {number}
       */
      x:number;
      /**
       * Gets the minimum y-coordinate of the candidate's bounds.
       * <p>
       * The bounds are the smallest paraxial box that contains the candidate.
       * </p>
       * @see yfiles.layout.LabelCandidate#location
       * @see yfiles.layout.LabelCandidate#boundingBox
       * @type {number}
       */
      y:number;
      /**
       * Gets the width of the candidate's bounds.
       * <p>
       * The bounds are the smallest paraxial box that contains the candidate.
       * </p>
       * @see yfiles.layout.LabelCandidate#size
       * @see yfiles.layout.LabelCandidate#boundingBox
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of the candidate's bounding box.
       * <p>
       * The bounding box is the smallest paraxial box that contains the candidate.
       * </p>
       * @see yfiles.layout.LabelCandidate#size
       * @see yfiles.layout.LabelCandidate#boundingBox
       * @type {number}
       */
      height:number;
      /**
       * Gets the bounding box of this candidate.
       * <p>
       * The bounding box describes the smallest paraxial box that contains the candidate.
       * </p>
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
      /**
       * Gets the oriented box of this candidate.
       * <p>
       * If the label is rotated, this box will be oriented accordingly.
       * </p>
       * @type {yfiles.algorithms.YOrientedRectangle}
       */
      box:yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets the {@link yfiles.layout.ILabelLayout} for which this candidate was created.
       * @type {yfiles.layout.ILabelLayout}
       */
      owner:yfiles.layout.ILabelLayout;
      /**
       * Gets whether or not this candidate describes an internal label.
       * <p>
       * Internal labels reside inside the boundaries of a node.
       * </p>
       * @type {boolean}
       */
      internal:boolean;
      /**
       * Gets or sets a penalty for a candidate position that overlaps the box of a node.
       * <p>
       * The penalty should be a factor between <code>0</code> and <code>1</code>, where <code>0</code> means that an overlap is considered unimportant and <code>1</code> means that
       * an overlap is considered severe.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified penalty is outside <code>[0,1]</code>
       * @type {number}
       */
      nodeOverlapPenalty:number;
      /**
       * Gets or sets a penalty for a candidate position that overlaps the path of an edge.
       * <p>
       * The penalty should be a factor between <code>0</code> and <code>1</code>, where <code>0</code> means that an overlap is considered unimportant and <code>1</code> means that
       * an overlap is considered severe.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified penalty is outside <code>[0,1]</code>
       * @type {number}
       */
      edgeOverlapPenalty:number;
      /**
       * Gets the sum of node overlap penalty and edge overlap penalty.
       * @see yfiles.layout.LabelCandidate#edgeOverlapPenalty
       * @see yfiles.layout.LabelCandidate#nodeOverlapPenalty
       * @type {number}
       */
      overlapPenalty:number;
      /**
       * Gets whether or not this candidate's box had already been chosen by the labeling algorithm.
       * <p>
       * If this candidate is propagated, its model parameter and box were transferred to the {@link yfiles.layout.LabelCandidate#owner label}.
       * </p>
       * @see yfiles.layout.LabelCandidate#propagate
       * @type {boolean}
       */
      propagated:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.LabelLayoutBase} is the default implementation of the {@link yfiles.layout.ILabelLayout} interface.
     * @class
     * @implements {yfiles.layout.ILabelLayout}
     */
    export interface LabelLayoutBase extends Object,yfiles.layout.ILabelLayout{}
    export class LabelLayoutBase {
      /**
       * Creates a new {@link yfiles.layout.LabelLayoutBase} instance.
       */
      constructor();
      /**
       * Sets the box of the label.
       * <p>
       * This box matches the area that is covered by the label. The {@link yfiles.algorithms.YOrientedRectangle#setUpVector up vector} of the rectangle encodes the orientation of the
       * label.
       * </p>
       * @type {yfiles.algorithms.YOrientedRectangle}
       */
      orientedBox:yfiles.algorithms.YOrientedRectangle;
      /**
       * Sets the bounding box of the label.
       * <p>
       * The bounding box is the smallest, paraxial rectangle that contains the whole label.
       * </p>
       * @type {yfiles.algorithms.YRectangle}
       */
      boundingBox:yfiles.algorithms.YRectangle;
      /**
       * 
       * @type {Object}
       */
      modelParameter:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.LabelLayoutData} encapsulates layout information for a label.
     * <p>
     * The information is used by clients to inform label-aware {@link yfiles.layout.ILayoutAlgorithm layout algorithms} about labeling constraints like label size
     * and preferred placement.
     * </p>
     * <p>
     * Label-aware algorithms like {@link yfiles.hierarchic.HierarchicLayout} return the calculated label positions by assigning the
     * corresponding box to the provided {@link yfiles.layout.LabelLayoutData} instances.
     * </p>
     * <p>
     * {@link yfiles.algorithms.IDataProvider DataProviders} are used for associating an array of {@link yfiles.layout.LabelLayoutData} instances with either the nodes or the
     * edges of a {@link yfiles.layout.LayoutGraph}. The {@link yfiles.algorithms.IDataProvider DataProviders} must be registered with the input graph using one of the keys
     * defined in {@link yfiles.layout.LabelLayoutKeys}.
     * </p>
     * @class
     */
    export interface LabelLayoutData extends Object{}
    export class LabelLayoutData {
      /**
       * Creates a new instance of {@link yfiles.layout.LabelLayoutData} for a label with the given width and height.
       * @param width the width of the label
       * @param height the height of the label
       */
      constructor(width:number,height:number);
      /**
       * Creates a new instance of {@link yfiles.layout.LabelLayoutData} for a label with the given oriented box and preferred placement
       * descriptor.
       * @param bounds the box of the label
       * @param [preferredPlacement=null] a preferred placement descriptor
       * @throws {Stubs.Exceptions.ArgumentError} if the specified descriptor is <code>null</code>
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LabelLayoutData#preferredPlacementDescriptor
       */
      constructor(bounds:yfiles.algorithms.YOrientedRectangle,preferredPlacement?:yfiles.layout.PreferredPlacementDescriptor);
      /**
       * Specifies the coordinates of the upper-left corner of the bounding box of the label.
       * @param x the new x-coordinate of the upper-left corner
       * @param y the new y-coordinate of the upper-left corner
       * @see yfiles.layout.LabelLayoutData#x
       * @see yfiles.layout.LabelLayoutData#y
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       */
      setLocation(x:number,y:number):void;
      /**
       * Specifies the width and height of the {@link yfiles.layout.LabelLayoutData#bounds oriented box} of this label.
       * @param width the width of the oriented box
       * @param height the height of the oriented box
       * @see yfiles.layout.LabelLayoutData#bounds
       */
      setSize(width:number,height:number):void;
      /**
       * Gets or sets the {@link yfiles.algorithms.YOrientedRectangle oriented box} of the label.
       * @type {yfiles.algorithms.YOrientedRectangle}
       */
      bounds:yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets the width of the label's bounding box.
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of the label's bounding box.
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       * @type {number}
       */
      height:number;
      /**
       * Gets the x-coordinate of the upper-left corner of the label's bounding box.
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       * @type {number}
       */
      x:number;
      /**
       * Gets the y-coordinate of the upper-left corner of the label's bounding box.
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       * @type {number}
       */
      y:number;
      /**
       * Gets or sets the preferred placement of this label.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified descriptor is <code>null</code>
       * @type {yfiles.layout.PreferredPlacementDescriptor}
       */
      preferredPlacementDescriptor:yfiles.layout.PreferredPlacementDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.LabelLayoutKeys} are used for associating
     * {@link yfiles.algorithms.IDataProvider}s that contain label-specific information for edge and node labels.
     * <p>
     * Label-aware layout algorithms like {@link yfiles.hierarchic.HierarchicLayout} may look for such {@link yfiles.algorithms.IDataProvider}s
     * that contain {@link yfiles.layout.LabelLayoutData} for each edge.
     * </p>
     * <p>
     * Instead of setting up {@link yfiles.layout.LabelLayoutData} and {@link yfiles.algorithms.IDataProvider}s one can also use {@link yfiles.layout.LabelLayoutTranslator}
     * to automatically translate information provided by the standard label layout classes {@link yfiles.layout.IEdgeLabelLayout} and
     * {@link yfiles.layout.INodeLabelLayout} to layout data accessible through the keys defined below.
     * </p>
     * @class
     * @static
     */
    export interface LabelLayoutKeys extends Object{}
    export class LabelLayoutKeys {
      /**
       * Data provider key for accessing label information of each edge in the input graph
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.layout.LabelLayoutData>}
       */
      static EDGE_LABEL_LAYOUT_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.layout.LabelLayoutData>;
      /**
       * Data provider key for accessing label information of each node in the input graph
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.layout.LabelLayoutData>}
       */
      static NODE_LABEL_LAYOUT_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.layout.LabelLayoutData>;
      /**
       * Data provider key for marking labels that should be ignored by the layout algorithm
       * <p>
       * A label is ignored if there is a {@link yfiles.algorithms.IDataProvider} registered with this key which returns <code>true</code> for the
       * given {@link yfiles.layout.ILabelLayout}.
       * </p>
       * <p>
       * The {@link yfiles.layout.LabelLayoutTranslator} only translates labels that should not be ignored.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.ILabelLayoutDpKey.<boolean>}
       */
      static IGNORED_LABELS_DP_KEY:yfiles.algorithms.ILabelLayoutDpKey<boolean>;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.LabelLayoutTranslator} automatically translates label layout information provided by the standard label
     * layout classes {@link yfiles.layout.IEdgeLabelLayout} and {@link yfiles.layout.INodeLabelLayout} to layout data of type {@link yfiles.layout.LabelLayoutData}
     * that is accessible by the keys defined in class {@link yfiles.layout.LabelLayoutKeys}.
     * <p>
     * One can use this {@link yfiles.layout.ILayoutStage} as label layout algorithm of a label-aware layout algorithm like
     * {@link yfiles.hierarchic.HierarchicLayout}. This can be done by assigning the stage by an appropriate call to {@link yfiles.layout.MultiStageLayout#labeling setLabelLayouter}.
     * </p>
     * <p>
     * When writing the {@link yfiles.layout.LabelLayoutData}, the {@link yfiles.layout.LabelLayoutData#bounds boxes} are set relative. In case of a node label, the box's {@link yfiles.algorithms.YOrientedRectangle#anchor anchor} is
     * relative to the center of the owning node. In case of an edge label the box's anchor is relative to the center of the
     * source node of the owning edge.
     * </p>
     * <p>
     * If {@link yfiles.layout.LabelLayoutTranslator#writeBackNodeLabels writing back node labels} and/or {@link yfiles.layout.LabelLayoutTranslator#writeBackEdgeLabels writing back edge labels} is enabled, the {@link yfiles.layout.LabelLayoutData} is transferred
     * back to the {@link yfiles.layout.INodeLabelLayout}s and/or {@link yfiles.layout.IEdgeLabelLayout}s.
     * </p>
     * <p>
     * By default, the {@link yfiles.layout.LabelLayoutData#bounds boxes} are now interpreted absolute but by activating {@link yfiles.layout.LabelLayoutTranslator#writeBackRelativeNodeLabelLocation} and/or {@link yfiles.layout.LabelLayoutTranslator#writeBackRelativeEdgeLabelLocation} this can be changed to interpret them
     * relative as explained above.
     * </p>
     * <p>
     * Note that care must be taken on the choice of the label model that is used by the classes {@link yfiles.layout.IEdgeLabelLayout}
     * and {@link yfiles.layout.INodeLabelLayout}. The calculated label positions must be consistent with the label positions allowed
     * by the label model. The best label layout results are achieved by choosing {@link yfiles.layout.FreeEdgeLabelLayoutModel} for
     * edge layouts and {@link yfiles.layout.FreeNodeLabelLayoutModel} for node layouts.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface LabelLayoutTranslator extends Object,yfiles.layout.ILayoutStage{}
    export class LabelLayoutTranslator {
      /**
       * Creates a new {@link yfiles.layout.LabelLayoutTranslator} instance with default settings.
       */
      constructor();
      /**
       * Translates traditional {@link yfiles.layout.ILabelLayout} information to {@link yfiles.algorithms.IDataProvider}-based
       * {@link yfiles.layout.LabelLayoutData}.
       * <p>
       * Afterwards, the calculated layout data will be written back to the original label layout.
       * </p>
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Gets or sets the core layout algorithm that is wrapped by this {@link yfiles.layout.ILayoutStage}.
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets whether or not this stage should automatically flip edge label content depending on the label's rotation
       * angle.
       * <p>
       * More precisely, if this option is enabled and the up vector of the {@link yfiles.algorithms.YOrientedRectangle} associated with
       * an {@link yfiles.layout.IEdgeLabelLayout} points downwards ({@link yfiles.algorithms.YOrientedRectangle#upY upY} > 0), this {@link yfiles.layout.ILayoutStage} automatically flips the
       * label. It rotates the label by <code>180</code> degrees without changing the label's center.
       * </p>
       * @see yfiles.layout.LayoutGraphUtilities#autoFlipBox
       * @see yfiles.layout.ILabelLayout#orientedBox
       * @type {boolean}
       */
      autoFlippingEnabled:boolean;
      /**
       * Gets or sets whether or not to reset the orientation of node labels.
       * <p>
       * If this option is enabled, the up vector of the corresponding oriented box is set to <code>(0,-1)</code> before adding it to the
       * {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * @type {boolean}
       */
      resettingNodeLabelOrientation:boolean;
      /**
       * Gets or sets whether or not to reset the orientation of edge labels.
       * <p>
       * If this option is enabled, the up vector of the corresponding oriented box is set to <code>(0,-1)</code> before adding it to the
       * {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * @type {boolean}
       */
      resettingEdgeLabelOrientation:boolean;
      /**
       * Gets or sets whether or not edge label information is written back to the model after core layout.
       * @type {boolean}
       */
      writeBackEdgeLabels:boolean;
      /**
       * Gets or sets whether or not edge label {@link yfiles.layout.LabelLayoutData#bounds boxes} are interpreted relative to the edge when writing them back to the model.
       * @type {boolean}
       */
      writeBackRelativeEdgeLabelLocation:boolean;
      /**
       * Gets or sets whether or not node label information is written back to the model after the core layout.
       * @type {boolean}
       */
      writeBackNodeLabels:boolean;
      /**
       * Gets or sets whether or not node label {@link yfiles.layout.LabelLayoutData#bounds bounds} are interpreted relative to the node when writing them back to the model.
       * @type {boolean}
       */
      writeBackRelativeNodeLabelLocation:boolean;
      /**
       * Gets or sets whether or not node label information is translated.
       * @type {boolean}
       */
      translateNodeLabels:boolean;
      /**
       * Gets or sets whether or not edge label information is translated.
       * @type {boolean}
       */
      translateEdgeLabels:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.LayoutGraph} is a {@link yfiles.algorithms.Graph} with attached layout information that basically represents a
     * drawing of a graph.
     * <p>
     * It provides access to the layout information for nodes, edges and labels. This layout information consists of
     * coordinates for the elements.
     * </p>
     * <p>
     * {@link yfiles.layout.ILayoutAlgorithm Layout algorithms} work on {@link yfiles.layout.LayoutGraph}s and arrange their nodes and edges.
     * </p>
     * @class
     * @extends {yfiles.algorithms.Graph}
     */
    export interface LayoutGraph extends yfiles.algorithms.Graph{}
    export class LayoutGraph {
      /**
       * Creates a new {@link yfiles.layout.LayoutGraph} which is a copy of the given subgraph.
       * @param graph the original graph
       * @param nodeSubset the nodes that induce the subgraph
       */
      constructor(graph:yfiles.layout.LayoutGraph,nodeSubset:yfiles.algorithms.ICursor);
      /**
       * Creates a new {@link yfiles.layout.LayoutGraph} which is a copy of the given subgraph.
       * @param subgraph the original subgraph
       */
      constructor(subgraph:yfiles.layout.LayoutGraph);
      /**
       * Creates a new, empty {@link yfiles.layout.LayoutGraph} with default settings.
       */
      constructor();
      /**
       * Creates a new {@link yfiles.layout.ILabelLayoutFactory} for this {@link yfiles.layout.LayoutGraph} that can be used for copying layout
       * information of labels.
       * @returns the new {@link }
       * @protected
       * @abstract
       */
      createLabelFactory():yfiles.layout.ILabelLayoutFactory;
      /**
       * Returns the rectangle describing the bounding box of the given node.
       * @param node the node
       * @returns the bounds of the node
       * @see yfiles.layout.LayoutGraph#getLocation
       * @see yfiles.layout.LayoutGraph#getSize
       */
      getBoundingBox(node:yfiles.algorithms.Node):yfiles.algorithms.YRectangle;
      /**
       * Returns the center coordinates of the given node.
       * @param node the node
       * @returns the location of the node's center
       * @see yfiles.layout.INodeLayout
       * @see yfiles.layout.LayoutGraph#setCenter
       */
      getCenter(node:yfiles.algorithms.Node):yfiles.algorithms.YPoint;
      /**
       * Returns the x-coordinate of the given node's center.
       * @param node the node
       * @returns the x-value of the node's center coordinate
       * @see yfiles.layout.INodeLayout
       */
      getCenterX(node:yfiles.algorithms.Node):number;
      /**
       * Returns the y-coordinate of the given node's center.
       * @param node the node
       * @returns the y-value of the node's center coordinate
       * @see yfiles.layout.INodeLayout
       */
      getCenterY(node:yfiles.algorithms.Node):number;
      /**
       * Returns the height of the given node.
       * @param node the node
       * @returns the height of the node
       * @see yfiles.layout.INodeLayout#height
       */
      getHeight(node:yfiles.algorithms.Node):number;
      /**
       * Returns the layout information for all labels associated with the given edge.
       * <p>
       * The layout information consists of the location, size and orientation of the label.
       * </p>
       * @param edge the edge
       * @returns the layout information for the edge's labels
       * @abstract
       */
      getLabelLayout(edge:yfiles.algorithms.Edge):yfiles.layout.IEdgeLabelLayout[];
      /**
       * Returns the layout information for all labels associated with the given node.
       * <p>
       * The layout information consists of the location, size and orientation of the label.
       * </p>
       * @param node the node
       * @returns the layout information for the node's labels
       * @abstract
       */
      getLabelLayout(node:yfiles.algorithms.Node):yfiles.layout.INodeLabelLayout[];
      /**
       * Returns the layout information for the given edge.
       * <p>
       * The layout information consists of the locations of the control points and end points of the edge.
       * </p>
       * @param edge the edge
       * @returns the layout information for the edge
       * @abstract
       */
      getLayout(edge:yfiles.algorithms.Edge):yfiles.layout.IEdgeLayout;
      /**
       * Returns the layout information for the given node.
       * <p>
       * The layout information consists of the location and size of the node.
       * </p>
       * @param node the node
       * @returns the layout information for the node
       * @abstract
       */
      getLayout(node:yfiles.algorithms.Node):yfiles.layout.INodeLayout;
      /**
       * Returns the coordinates of the given node's upper-left corner.
       * @param node the node
       * @returns the location of the node's upper-left corner
       * @see yfiles.layout.INodeLayout#x
       * @see yfiles.layout.INodeLayout#y
       */
      getLocation(node:yfiles.algorithms.Node):yfiles.algorithms.YPoint;
      /**
       * Returns the edge to which the given {@link yfiles.layout.IEdgeLabelLayout} belongs.
       * @param labelLayout the label's layout information
       * @returns the edge that owns the label
       * @abstract
       */
      getOwnerEdge(labelLayout:yfiles.layout.IEdgeLabelLayout):yfiles.algorithms.Edge;
      /**
       * Returns the node to which the given {@link yfiles.layout.INodeLabelLayout} belongs.
       * @param labelLayout the label's layout information
       * @returns the node that owns the label
       * @abstract
       */
      getOwnerNode(labelLayout:yfiles.layout.INodeLabelLayout):yfiles.algorithms.Node;
      /**
       * Returns the path of an edge, including the ports.
       * <p>
       * The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
       * of the target port.
       * </p>
       * @param edge the edge
       * @returns the path of the given edge
       */
      getPath(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPointPath;
      /**
       * Returns the path of an edge, including the ports.
       * <p>
       * The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
       * of the target port.
       * </p>
       * @param edge the edge
       * @returns a list of {@link }s representing the path of the given edge
       */
      getPathList(edge:yfiles.algorithms.Edge):yfiles.algorithms.YList;
      /**
       * Returns the control points of an edge.
       * @param edge the edge
       * @returns a list of {@link }s representing the sequence of control points of the edge
       * @see yfiles.layout.IEdgeLayout
       */
      getPointList(edge:yfiles.algorithms.Edge):yfiles.algorithms.YList;
      /**
       * Returns the control points of an edge.
       * @param edge the edge
       * @returns a {@link } representing the sequence of control points of the edge
       * @see yfiles.layout.IEdgeLayout
       */
      getPoints(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPointPath;
      /**
       * Returns the size of the given node.
       * @param node the node
       * @returns the size of the node
       * @see yfiles.layout.INodeLayout#width
       * @see yfiles.layout.INodeLayout#height
       */
      getSize(node:yfiles.algorithms.Node):yfiles.algorithms.YDimension;
      /**
       * Returns the absolute coordinates of the source point of the given edge.
       * @param edge the edge
       * @returns the absolute source point coordinates of the edge
       */
      getSourcePointAbs(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPoint;
      /**
       * Returns the relative coordinates of the source point of the given edge.
       * <p>
       * The coordinates are interpreted relative to the center of the source node.
       * </p>
       * @param edge the edge
       * @returns the relative source point coordinates of the edge
       * @see yfiles.layout.IEdgeLayout#sourcePoint
       */
      getSourcePointRel(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPoint;
      /**
       * Returns the absolute coordinates of the target point of the given edge.
       * @param edge the edge
       * @returns the absolute target point coordinates of the edge
       */
      getTargetPointAbs(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPoint;
      /**
       * Returns the relative coordinates of the target point of the given edge.
       * <p>
       * The coordinates are interpreted relative to the center of the target node.
       * </p>
       * @param edge the edge
       * @returns the relative target point coordinates of the edge
       * @see yfiles.layout.IEdgeLayout#targetPoint
       */
      getTargetPointRel(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPoint;
      /**
       * Returns the width of the given node.
       * @param node the node
       * @returns the width of the node
       * @see yfiles.layout.INodeLayout#width
       */
      getWidth(node:yfiles.algorithms.Node):number;
      /**
       * Returns the x-coordinate of the given node's upper-left corner.
       * @param node the node
       * @returns the x-value of the node's upper-left corner
       * @see yfiles.layout.INodeLayout#x
       */
      getX(node:yfiles.algorithms.Node):number;
      /**
       * Returns the y-coordinate of the given node's upper-left corner.
       * @param node the node
       * @returns the y-value of the node's upper-left corner
       * @see yfiles.layout.INodeLayout#y
       */
      getY(node:yfiles.algorithms.Node):number;
      /**
       * Moves the upper-left corner of a node by a given vector.
       * @param node the node
       * @param dx the x-component of the vector
       * @param dy the y-component of the vector
       */
      moveBy(node:yfiles.algorithms.Node,dx:number,dy:number):void;
      /**
       * Specifies the center coordinates of the given node.
       * @param node the node
       * @param x the new x-coordinate of the node's center
       * @param y the new y-coordinate of the node's center
       * @see yfiles.layout.INodeLayout
       */
      setCenter(node:yfiles.algorithms.Node,x:number,y:number):void;
      /**
       * Specifies the center coordinates of the given node.
       * @param node the node
       * @param position the new location of the node's center
       * @see yfiles.layout.INodeLayout
       * @see yfiles.layout.LayoutGraph#setCenter
       */
      setCenter(node:yfiles.algorithms.Node,position:yfiles.algorithms.YPoint):void;
      /**
       * Specifies the two end points of the given edge in absolute coordinates.
       * @param edge the edge
       * @param source the port on the source side of the edge
       * @param target the port on the target side of the edge
       * @see yfiles.layout.LayoutGraph#setSourcePointAbs
       * @see yfiles.layout.LayoutGraph#setTargetPointAbs
       */
      setEndPointsAbs(edge:yfiles.algorithms.Edge,source:yfiles.algorithms.YPoint,target:yfiles.algorithms.YPoint):void;
      /**
       * Specifies the coordinates of the upper-left corner of the given node.
       * @param node the node
       * @param x the new x-coordinate of the upper-left corner
       * @param y the new y-coordinate of the upper-left corner
       * @see yfiles.layout.INodeLayout#setLocation
       */
      setLocation(node:yfiles.algorithms.Node,x:number,y:number):void;
      /**
       * Specifies the coordinates of the upper-left corner of the given node.
       * @param node the node
       * @param position the new location of the upper-left corner
       * @see yfiles.layout.INodeLayout#setLocation
       * @see yfiles.layout.LayoutGraph#setLocation
       */
      setLocation(node:yfiles.algorithms.Node,position:yfiles.algorithms.YPoint):void;
      /**
       * Sets the path of the given edge.
       * <p>
       * The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
       * of the target port.
       * </p>
       * <p>
       * Both source and target port are expected to be given in absolute coordinates.
       * </p>
       * @param edge an edge in the graph
       * @param path the list of {@link }s representing the new path sequence of the edge
       */
      setPath(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList):void;
      /**
       * Sets the path of the given edge.
       * <p>
       * The path of an edge is the position of the source port, followed by the positions of the bends, followed by the position
       * of the target port.
       * </p>
       * <p>
       * Both source and target port are expected to be given in absolute coordinates.
       * </p>
       * @param edge an edge in the graph
       * @param path the new path sequence of the edge
       */
      setPath(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YPointPath):void;
      /**
       * Sets the control points of the given edge.
       * <p>
       * The source and target points of the edge will not be modified.
       * </p>
       * @param edge an edge in the graph
       * @param points the list of {@link }s representing the new sequence of control points
       */
      setPoints(edge:yfiles.algorithms.Edge,points:yfiles.algorithms.YList):void;
      /**
       * Sets the control points of the given edge.
       * <p>
       * The source and target points of the edge will not be modified.
       * </p>
       * @param edge an edge in the graph
       * @param points the new sequence of control points
       */
      setPoints(edge:yfiles.algorithms.Edge,points:yfiles.algorithms.YPointPath):void;
      /**
       * Specifies the width and height of the given node.
       * @param node the node
       * @param width the new width of the node
       * @param height the new height of the node
       * @see yfiles.layout.INodeLayout#setSize
       */
      setSize(node:yfiles.algorithms.Node,width:number,height:number):void;
      /**
       * Specifies the size of the given node.
       * @param node the node
       * @param size the new size of the node
       * @see yfiles.layout.LayoutGraph#setSize
       * @see yfiles.layout.INodeLayout#setSize
       */
      setSize(node:yfiles.algorithms.Node,size:yfiles.algorithms.YDimension):void;
      /**
       * Specifies the absolute coordinates of the source point of the given edge.
       * @param edge the edge
       * @param point the new absolute source point coordinates of the edge
       */
      setSourcePointAbs(edge:yfiles.algorithms.Edge,point:yfiles.algorithms.YPoint):void;
      /**
       * Specifies the relative coordinates of the source point of the given edge.
       * <p>
       * The coordinates are interpreted relative to the center of the source node.
       * </p>
       * @param edge the edge
       * @param point the new relative source point coordinates of the edge
       * @see yfiles.layout.IEdgeLayout#sourcePoint
       */
      setSourcePointRel(edge:yfiles.algorithms.Edge,point:yfiles.algorithms.YPoint):void;
      /**
       * Specifies the absolute coordinates of the target point of the given edge.
       * @param edge the edge
       * @param point the new absolute target point coordinates of the edge
       */
      setTargetPointAbs(edge:yfiles.algorithms.Edge,point:yfiles.algorithms.YPoint):void;
      /**
       * Specifies the relative coordinates of the target point of the given edge.
       * <p>
       * The coordinates are interpreted relative to the center of the target node.
       * </p>
       * @param edge the edge
       * @param point the new relative target point coordinates of the edge
       * @see yfiles.layout.IEdgeLayout#targetPoint
       */
      setTargetPointRel(edge:yfiles.algorithms.Edge,point:yfiles.algorithms.YPoint):void;
      /**
       * Gets a list of all edges in the graph.
       * @type {yfiles.algorithms.EdgeList}
       */
      edgeList:yfiles.algorithms.EdgeList;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.layout.LayoutGraphUtilities} provides several helper and utility functions for {@link yfiles.layout.LayoutGraph} and
     * the {@link yfiles.layout.ILayoutAlgorithm layout algorithms}.
     * @see yfiles.layout.LayoutGraph
     * @class
     * @static
     */
    export interface LayoutGraphUtilities extends Object{}
    export class LayoutGraphUtilities {
      /**
       * Aligns the specified nodes either vertically or horizontally according to the specified node alignment mode.
       * <p>
       * For vertical arrangements, nodes are either {@link yfiles.layout.NodeAlignment#LEADING top-aligned}, {@link yfiles.layout.NodeAlignment#CENTERED center-aligned}, {@link yfiles.layout.NodeAlignment#TRAILING bottom-aligned}, or {@link yfiles.layout.NodeAlignment#DISTRIBUTED vertically distributed}.
       * </p>
       * <p>
       * For horizontal arrangements, nodes are either {@link yfiles.layout.NodeAlignment#LEADING left-aligned}, {@link yfiles.layout.NodeAlignment#CENTERED center-aligned}, {@link yfiles.layout.NodeAlignment#TRAILING right-aligned}, or {@link yfiles.layout.NodeAlignment#DISTRIBUTED horizontally distributed}.
       * </p>
       * <p>
       * In this context, <em>distributed</em> means that the specified nodes are placed in such a way that the projections of their bounds onto the
       * vertical or horizontal coordinate axis do not overlap. For example, after vertically distributing nodes, <code>(n1.y + n1.height) &lt;= n2.y</code>
       * for two consecutively placed nodes <code>n1</code> and <code>n2</code>.
       * </p>
       * @param graph the graph that contains the specified nodes
       * @param nodes the subset of the specified graph's nodes that is arranged
       * @param referenceNode if <code>null</code>, the common coordinate for aligning the specified nodes is calculated as the the vertical minimum (top-aligned),
       * the vertical center, the vertical maximum (bottom-aligned), the horizontal minimum (left-aligned), the horizontal
       * center, or the horizontal maximum (right-aligned) of the bounds of the specified nodes; otherwise the appropriate
       * coordinate of the given reference node's bounds is used as common coordinate for aligning the specified nodes. The
       * reference node is ignored when {@link #DISTRIBUTED distributing} nodes.
       * @param vertical <code>true</code> if nodes should be aligned (or distributed) according to their y-coordinates; otherwise nodes are aligned (or
       * distributed) according to their x-coordinates.
       * @param mode one of the valid alignment modes
       * @static
       */
      static alignNodeLayouts(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.NodeList,referenceNode:yfiles.algorithms.Node,vertical:boolean,mode:yfiles.layout.NodeAlignment):void;
      /**
       * Reassigns the bounds of the rectangles in a grid to fit in a large rectangle with an aspect ratio close to the specified
       * aspect ratio.
       * <p>
       * The aspect ratio must be greater than <code>0</code>.
       * </p>
       * @param rectangles the rectangles whose coordinates will be modified
       * @param finalRect the exact dimension will be placed in this rectangle
       * @param aspectRatio the preferred aspect ratio of the grid
       * @returns the dimension of the grid
       * @throws {Stubs.Exceptions.ArgumentError} if the given aspect ratio is less than or equal to zero
       * @static
       */
      static arrangeRectangleGrid(rectangles:yfiles.algorithms.Rectangle2D[],finalRect:yfiles.algorithms.Rectangle2D,aspectRatio:number):yfiles.algorithms.YDimension;
      /**
       * Reassigns the bounds of the rectangles in multiple rows to fit the preferred width and height.
       * <p>
       * The rectangles will be placed within this bound, arranged in rows, where each row may be split into more sub-rows. This
       * often leads to more compact results than provided by the procedure used in
       * {@link yfiles.layout.LayoutGraphUtilities#arrangeRectangleRows}. A row will be subdivided into more rows, if two successive
       * rectangles to be placed in the row could be placed stacked within this row without making the row larger in height.
       * </p>
       * <p>
       * The preferred width and height should be greater than <code>0</code>, otherwise the nodes are not arranged.
       * </p>
       * @param rectangles the rectangles to arrange (coordinates will be assigned)
       * @param finalRect the exact resulting dimension will be stored in this rectangle
       * @param preferredWidth the preferred width of the result rectangle
       * @param preferredHeight the preferred height of the result rectangle
       * @param compaction <code>true</code> if the algorithm tries to always minimize the resulting dimension, whilst respecting the given constraints, <code>false</code>
       * otherwise
       * @param constraintMode the width or height constraints to consider
       * @param [alignment=yfiles.layout.RowAlignment.LEADING] the alignment inside a row
       * @returns the number of rows used
       * @see yfiles.layout.LayoutGraphUtilities#arrangeRectangleMultiRows
       * @static
       */
      static arrangeRectangleMultiRows(rectangles:yfiles.algorithms.Rectangle2D[],finalRect:yfiles.algorithms.Rectangle2D,preferredWidth:number,preferredHeight:number,compaction:boolean,constraintMode:yfiles.layout.MultiRowConstraint,alignment?:yfiles.layout.RowAlignment):number;
      /**
       * Reassigns the bounds of the rectangles to fit in a large rectangle with an aspect ratio close to the specified aspect
       * ratio.
       * <p>
       * This is achieved by arranging the rectangles in rows.
       * </p>
       * <p>
       * The aspect ratio must be greater than <code>0</code>.
       * </p>
       * @param rectangles the rectangles whose coordinates will be modified
       * @param finalRect the exact dimension of the resulting rectangle
       * @param aspectRatio the preferred aspect ratio of the resulting rectangle
       * @param [alignment=yfiles.layout.RowAlignment.LEADING] one of the valid row alignments
       * @returns the number of used rows
       * @throws {Stubs.Exceptions.ArgumentError} if the aspect ratio is <code>0</code> or less
       * @static
       */
      static arrangeRectangleRows(rectangles:yfiles.algorithms.Rectangle2D[],finalRect:yfiles.algorithms.Rectangle2D,aspectRatio:number,alignment?:yfiles.layout.RowAlignment):number;
      /**
       * Applies the reversed layout of the first edge to the second edge.
       * @param graph the graph to which both edges belong
       * @param edge1 the edge which adopts the points
       * @param edge2 the template
       * @static
       */
      static assignReverseLayout(graph:yfiles.layout.LayoutGraph,edge1:yfiles.algorithms.Edge,edge2:yfiles.algorithms.Edge):void;
      /**
       * Flips the orientation of a given rectangle, if its up vector points downward <code>(</code>{@link yfiles.algorithms.YOrientedRectangle#upY upY}<code> > 0)</code>.
       * <p>
       * That means, it rotates the rectangle by <code>180</code> degrees without changing the rectangle's center.
       * </p>
       * @param rect the given rectangle
       * @returns <code>true</code> if the given rectangle was flipped, <code>false</code> otherwise
       * @static
       */
      static autoFlipBox(rect:yfiles.algorithms.YOrientedRectangle):boolean;
      /**
       * Clips the path of the given edge on the bounding box of the source and target points.
       * <p>
       * On a clipped edge, the source and target port will be located at the border of the corresponding node.
       * </p>
       * @param graph the graph to which the edge belongs
       * @param edge the edge to clip
       * @static
       */
      static clipEdgeOnBounds(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):void;
      /**
       * Returns the path of an edge clipped on the bounding box of the source and target node.
       * <p>
       * On the clipped edge, the source and target port will be located at the border of the corresponding node.
       * </p>
       * @param graph the graph to which the edge belongs
       * @param edge the clipped edge
       * @returns the clipped path of the edge
       * @static
       */
      static clipEdgePathOnBounds(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):yfiles.algorithms.YPointPath;
      /**
       * Returns the path of an edge clipped on insets of the bounding box of the source and target node.
       * <p>
       * On the clipped edge, the source and target port will be located inside the node with an inset to the border of the
       * corresponding node.
       * </p>
       * @param edge the clipped edge
       * @param source the source of the edge
       * @param target the target of the edge
       * @param [inset=0.0] the inset of the node
       * @returns the clipped path of the edge
       * @static
       */
      static clipEdgePathOnBounds(edge:yfiles.layout.IEdgeLayout,source:yfiles.layout.INodeLayout,target:yfiles.layout.INodeLayout,inset?:number):yfiles.algorithms.YPointPath;
      /**
       * Clips the path of all edges in the graph on the bounding box of their source and target points.
       * <p>
       * On a clipped edge, the source and target port will be located at the border of the corresponding node.
       * </p>
       * @param graph the graph
       * @see yfiles.layout.LayoutGraphUtilities#clipEdgeOnBounds
       * @static
       */
      static clipEdgesOnBounds(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Returns the path of a given edge path clipped on the given bounding boxes of the source and target node.
       * @param path the path that will be clipped
       * @param sourceBox the box of the source node that the given path should be clipped at
       * @param targetBox the box of the target node that the given path should be clipped at
       * @returns the path clipped on the given nodes boxes
       * @static
       */
      static clipPathOnBounds(path:yfiles.algorithms.YPointPath,sourceBox:yfiles.algorithms.YRectangle,targetBox:yfiles.algorithms.YRectangle):yfiles.algorithms.YPointPath;
      /**
       * Determines the main direction of the edge flow by analyzing the current layout of the graph.
       * <p>
       * The method only considers edges that are marked in the given {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * @param graph the underlying graph
       * @param considerEdges the edges to consider
       * @returns one of the predefined flow directions
       * @static
       */
      static determineEdgeFlowDirection(graph:yfiles.layout.LayoutGraph,considerEdges:yfiles.algorithms.IDataProvider):yfiles.layout.EdgeFlow;
      /**
       * Returns a {@link string} representation of the path of the given edge.
       * @param graph the graph to which the edge belongs
       * @param edge the edge
       * @returns the {@link } that contains all control points of the edge
       * @static
       */
      static edgeLayoutString(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):string;
      /**
       * Returns the bounding box of the given edge.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the control points as well as the
       * absolute coordinates of the source and target point of the edge.
       * </p>
       * @param graph the graph that contains the edge
       * @param edge the edge
       * @returns the bounding box of the edge
       * @see yfiles.layout.LayoutGraph#getSourcePointAbs
       * @see yfiles.layout.LayoutGraph#getTargetPointAbs
       * @see yfiles.layout.LayoutGraph#getLayout
       * @static
       */
      static getBoundingBox(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the bounding box of the given node.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the given node. It is identical to the
       * node's bounds.
       * </p>
       * @param graph the graph that contains the node
       * @param node the node
       * @returns the bounding box of the node
       * @see yfiles.layout.LayoutGraph#getLayout
       * @static
       */
      static getBoundingBox(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the bounding box of the nodes and edges accessible through the given cursors, optionally including node labels,
       * edge labels or {@link yfiles.layout.NodeHalo}s.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes and the control points
       * as well as the source and target points of all edges. If node or edge labels should be included, it also contains their
       * boxes. {@link yfiles.layout.NodeHalo}s can also extend the bounding box if desired.
       * </p>
       * @param graph the graph that contains the given nodes and edges
       * @param nodes the nodes to include in the bounding box
       * @param edges the edges to include in the bounding box
       * @param includeNodeLabels <code>true</code> if labels of nodes should be included, <code>false</code> otherwise
       * @param includeEdgeLabels <code>true</code> if labels of edges should be included, <code>false</code> otherwise
       * @param [includeHalos=false] <code>true</code> if {@link }s should be included, <code>false</code> otherwise
       * @returns the bounding box of the given nodes and edges
       * @static
       */
      static getBoundingBox(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.INodeCursor,edges:yfiles.algorithms.IEdgeCursor,includeNodeLabels:boolean,includeEdgeLabels:boolean,includeHalos?:boolean):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the bounding box of the nodes and edges accessible through the given cursors, optionally including node labels
       * and edge labels.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes and the control points
       * as well as the source and target points of all edges. If labels should be included, it also contains their boxes.
       * </p>
       * @param graph the graph that contains the given nodes and edges
       * @param nodes the nodes to include in the bounding box
       * @param edges the edges to include in the bounding box
       * @param includeLabels <code>true</code> if labels of nodes and edges should be included, <code>false</code> otherwise
       * @returns the bounding box of the given nodes and edges
       * @see yfiles.layout.LayoutGraphUtilities#getBoundingBox
       * @static
       */
      static getBoundingBox(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.INodeCursor,edges:yfiles.algorithms.IEdgeCursor,includeLabels:boolean):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the bounding box of the nodes and edges accessible through the given cursors.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes and the control points
       * as well as the source and target points of all edges.
       * </p>
       * @param graph the graph that contains the nodes and edges
       * @param nodes the nodes to include in the bounding box
       * @param edges the edges to include in the bounding box
       * @returns the bounding box containing the given nodes and edges
       * @see yfiles.layout.LayoutGraphUtilities#getBoundingBoxOfEdges
       * @see yfiles.layout.LayoutGraphUtilities#getBoundingBoxOfNodes
       * @static
       */
      static getBoundingBox(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.INodeCursor,edges:yfiles.algorithms.IEdgeCursor):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the bounding box of the edges accessible through the given cursor.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the control points as well as the
       * absolute coordinates of the source and target points of all given edges.
       * </p>
       * @param graph the graph that contains the edges
       * @param edges the edges to include in the bounding box
       * @returns the bounding box containing the given edges
       * @see yfiles.layout.LayoutGraph#getSourcePointAbs
       * @see yfiles.layout.LayoutGraph#getTargetPointAbs
       * @see yfiles.layout.LayoutGraph#getLayout
       * @static
       */
      static getBoundingBoxOfEdges(graph:yfiles.layout.LayoutGraph,edges:yfiles.algorithms.IEdgeCursor):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the bounding box of the nodes accessible through the given cursor.
       * <p>
       * In this case, the bounding box is the smallest paraxial rectangle which contains the given nodes.
       * </p>
       * @param graph the graph that contains the nodes
       * @param nodes the nodes to include in the bounding box
       * @returns the bounding box containing the given nodes
       * @see yfiles.layout.LayoutGraph#getLayout
       * @static
       */
      static getBoundingBoxOfNodes(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.INodeCursor):yfiles.algorithms.Rectangle2D;
      /**
       * Returns the {@link yfiles.layout.ILabelLayoutFactory} for the given graph.
       * @param graph the graph
       * @returns the label factory of the graph
       * @static
       */
      static getLabelFactory(graph:yfiles.layout.LayoutGraph):yfiles.layout.ILabelLayoutFactory;
      /**
       * Returns the upper-left corner of the paraxial bounding box of the given edge label as encoded by the specified model
       * parameter.
       * <p>
       * The parameter describes the abstract position of the label within the specified model. The parameter must have been
       * generated by that model.
       * </p>
       * @param model the {@link } used for placing the label
       * @param labelSize the size of the label that should be placed
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the edge's source node
       * @param targetLayout the layout of the edge's target node
       * @param param the model parameter
       * @returns the upper-left corner of the label's bounding box
       * @see yfiles.layout.IEdgeLabelLayoutModel#getLabelPlacement
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       * @static
       */
      static getLabelPlacement(model:yfiles.layout.IEdgeLabelLayoutModel,labelSize:yfiles.algorithms.YDimension,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout,param:Object):yfiles.algorithms.YPoint;
      /**
       * Returns the upper-left corner of the paraxial bounding box of the given node label as encoded by the specified model
       * parameter.
       * <p>
       * The parameter describes the abstract position of the label within the specified model. The parameter must have been
       * generated by that model.
       * </p>
       * @param model the {@link } used for placing the label
       * @param labelSize the size of the label
       * @param nodeLayout the layout of the node to which the label belongs
       * @param param the model parameter
       * @returns the upper-left corner of the label's bounding box
       * @see yfiles.layout.INodeLabelLayoutModel#getLabelPlacement
       * @see yfiles.algorithms.YOrientedRectangle#boundingBox
       * @static
       */
      static getLabelPlacement(model:yfiles.layout.INodeLabelLayoutModel,labelSize:yfiles.algorithms.YDimension,nodeLayout:yfiles.layout.INodeLayout,param:Object):yfiles.algorithms.YPoint;
      /**
       * Returns the distance between the centers of the two given nodes.
       * @param graph the graph to which the nodes belong
       * @param node1 the first node
       * @param node2 the second node
       * @returns the distance between the nodes' centers
       * @static
       */
      static getNodeDistance(graph:yfiles.layout.LayoutGraph,node1:yfiles.algorithms.Node,node2:yfiles.algorithms.Node):number;
      /**
       * Returns a {@link string} representation of all node positions in the graph.
       * @param graph the graph
       * @returns the {@link } containing all node locations
       * @see yfiles.layout.LayoutGraph#getLocation
       * @static
       */
      static getNodePositions(graph:yfiles.layout.LayoutGraph):string;
      /**
       * Returns the length of the path of the given edge.
       * <p>
       * The path starts at the source point, traverses through all control points and ends at the target point.
       * </p>
       * @param graph the graph to which the edge belongs
       * @param edge the edge
       * @returns the length of the edge's path
       * @static
       */
      static getPathLength(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):number;
      /**
       * Checks whether or not the path of an edge is outside the bounding box of the source and target node, considering the
       * given halo.
       * <p>
       * The edge is outside the nodes if its control points as well as the source and target point are not inside the nodes'
       * bounding boxes.
       * </p>
       * <p>
       * The nodes can be enlarged by a halo to check if the edge keeps some distance to the nodes.
       * </p>
       * @param edge the edge
       * @param source the source of the edge
       * @param target the target of the edge
       * @param halo the halo for the nodes
       * @returns <code>true</code> if the edge is outside the nodes, <code>false</code> otherwise
       * @static
       */
      static isEdgeOutsideNodes(edge:yfiles.layout.IEdgeLayout,source:yfiles.layout.INodeLayout,target:yfiles.layout.INodeLayout,halo:number):boolean;
      /**
       * Checks whether or not the path of an edge is outside the bounding box of the source and target node.
       * <p>
       * The edge is outside the nodes if its control points as well as the source and target point are not inside the nodes'
       * bounding boxes.
       * </p>
       * @param graph the graph to which the edge belongs
       * @param edge the edge
       * @returns <code>true</code> if the edge is outside the nodes, <code>false</code> otherwise
       * @static
       */
      static isEdgeOutsideNodes(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):boolean;
      /**
       * Determines whether or not most of the edges of the graph are routed octilinear.
       * <p>
       * The method only considers edges marked in the given {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * @param graph the underlying graph
       * @param considerEdges the edges to consider
       * @returns <code>true</code> if most of the edges in the graph are routed octilinear, <code>false</code> otherwise
       * @static
       */
      static isUsingOctilinearEdgeRoutes(graph:yfiles.layout.LayoutGraph,considerEdges:yfiles.algorithms.IDataProvider):boolean;
      /**
       * Determines whether or not most of the edges of the graph are routed orthogonally.
       * <p>
       * The method only considers edges marked in the given {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * @param graph the underlying graph
       * @param considerEdges the edges to consider
       * @returns <code>true</code> if most of the edges in the graph are routed orthogonally, <code>false</code> otherwise
       * @static
       */
      static isUsingOrthogonalEdgeRoutes(graph:yfiles.layout.LayoutGraph,considerEdges:yfiles.algorithms.IDataProvider):boolean;
      /**
       * Moves all control points of the given edge by the vector <code>(dx,dy)</code>.
       * @param graph the graph
       * @param edge the edge
       * @param dx the horizontal distance to move
       * @param dy the vertical distance to move
       * @static
       */
      static moveEdge(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge,dx:number,dy:number):void;
      /**
       * Moves the control points of all edges accessible through the given {@link yfiles.algorithms.IEdgeCursor} by the vector <code>(dx,dy)</code>.
       * @param graph the graph
       * @param edges the edges to be moved
       * @param dx the horizontal distance to move
       * @param dy the vertical distance to move
       * @static
       */
      static moveEdges(graph:yfiles.layout.LayoutGraph,edges:yfiles.algorithms.IEdgeCursor,dx:number,dy:number):void;
      /**
       * Moves the given node by the vector <code>(dx,dy)</code>.
       * @param graph the graph
       * @param node the node
       * @param dx the horizontal distance to move
       * @param dy the vertical distance to move
       * @static
       */
      static moveNode(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node,dx:number,dy:number):void;
      /**
       * Moves all nodes accessible through the given {@link yfiles.algorithms.INodeCursor} by the vector <code>(dx,dy)</code>.
       * @param graph the graph
       * @param nodes the nodes to be moved
       * @param dx the horizontal distance to move
       * @param dy the vertical distance to move
       * @static
       */
      static moveNodes(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.INodeCursor,dx:number,dy:number):void;
      /**
       * Moves the subgraph induced by the nodes accessible through the given {@link yfiles.algorithms.INodeCursor} by the vector
       * <code>(dx,dy)</code>.
       * @param graph the graph that contains the subgraph
       * @param nodes the nodes that induce the subgraph
       * @param dx the horizontal distance to move
       * @param dy the vertical distance to move
       * @static
       */
      static moveSubgraph(graph:yfiles.layout.LayoutGraph,nodes:yfiles.algorithms.INodeCursor,dx:number,dy:number):void;
      /**
       * Moves the subgraph induced by the edges accessible through the given {@link yfiles.algorithms.IEdgeCursor} by the vector
       * <code>(dx,dy)</code>.
       * @param graph the graph that contains the subgraph
       * @param edges the edges that induce the subgraph
       * @param dx the horizontal distance to move
       * @param dy the vertical distance to move
       * @static
       */
      static moveSubgraphOfEdges(graph:yfiles.layout.LayoutGraph,edges:yfiles.algorithms.IEdgeCursor,dx:number,dy:number):void;
      /**
       * Checks whether or not the path of the given edge intersects the interior of a given rectangular area.
       * @param graph the graph to which the edge belongs
       * @param edge the edge
       * @param rect the rectangular area to check for intersection
       * @returns <code>true</code> if the edge path intersects with the given rectangle, <code>false</code> otherwise
       * @static
       */
      static pathIntersectsRect(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge,rect:yfiles.algorithms.Rectangle2D):boolean;
      /**
       * Removes successive bends that have the same coordinates from the given edge.
       * @param graph the graph
       * @param edge the edge
       * @static
       */
      static removeDuplicateBends(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):void;
      /**
       * Removes successive bends that have the same coordinates from all edges in the graph.
       * @param graph the graph
       * @static
       */
      static removeDuplicateBends(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Assigns a trivial path which has no control points to the specified edge.
       * @param graph the graph
       * @param edge the edge
       * @param [resetPorts=true] <code>true</code> if the ports of the edge should be set to the center of the corresponding nodes, <code>false</code> otherwise
       * @static
       */
      static resetPath(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge,resetPorts?:boolean):void;
      /**
       * Assigns a trivial path which has no control points to all edges in the specified graph.
       * @param graph the graph
       * @param [resetPorts=true] <code>true</code> if the ports of the edges should be set to the center of the corresponding nodes, <code>false</code> otherwise
       * @static
       */
      static resetPaths(graph:yfiles.layout.LayoutGraph,resetPorts?:boolean):void;
      /**
       * Sets the ports of all edges in the graph to the center of the nodes.
       * @param graph the graph
       * @see yfiles.layout.LayoutGraph#setSourcePointRel
       * @see yfiles.layout.LayoutGraph#setTargetPointRel
       * @static
       */
      static resetPorts(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Reverses the layout of an edge.
       * <p>
       * The order of the control points is reversed and source and target point are swapped.
       * </p>
       * @param el the layout information of an edge
       * @static
       */
      static reverseEdgeLayout(el:yfiles.layout.IEdgeLayout):void;
      /**
       * Rounds the locations of the control points and ports of the given edge to integer values.
       * @param graph the graph to which the edge belongs
       * @param edge the edge whose values are rounded
       * @static
       */
      static roundEdgeLayout(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):void;
      /**
       * Rounds coordinates and sizes in the layout of an entire graph to integer values.
       * @param graph the graph
       * @see yfiles.layout.LayoutGraphUtilities#roundNodeLayout
       * @see yfiles.layout.LayoutGraphUtilities#roundEdgeLayout
       * @static
       */
      static roundLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Rounds the location and size of the given node to integer values.
       * @param graph the graph to which the node belongs
       * @param node the node whose values are rounded
       * @static
       */
      static roundNodeLayout(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):void;
      /**
       * Routes a list of edges which are incident to the same nodes in parallel, optionally joining the end points.
       * <p>
       * All edges get a path that is parallel to the path of the first (leading) edge.
       * </p>
       * <p>
       * The position of the leading edge may be adjusted if there is an odd number of parallel edges. It will be moved from the
       * center to achieve a more symmetric port assignment.
       * </p>
       * @param graph the graph
       * @param leadingEdge the leading edge
       * @param edges the list of edges that will be aligned with the leading edge
       * @param lineDistance the distance between two edges
       * @param [adjustLeadingEdge=false] <code>true</code> if the leading edge should be adjusted if the size of list is odd, <code>false</code> otherwise
       * @param [joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
       * @param [absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
       * @param [relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
       * @static
       */
      static routeEdgesParallel(graph:yfiles.layout.LayoutGraph,leadingEdge:yfiles.algorithms.Edge,edges:yfiles.algorithms.EdgeList,lineDistance:number,adjustLeadingEdge?:boolean,joinEnds?:boolean,absJoinDistance?:number,relJoinDistance?:number):void;
      /**
       * Routes a list of edges which are incident to the same nodes in parallel, optionally joining the end points.
       * <p>
       * All edges get a path that is parallel to the path of the first (leading) edge.
       * </p>
       * <p>
       * The position of the leading edge may be adjusted if there is an odd number of parallel edges. It will be moved from the
       * center to achieve a more symmetric port assignment.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.graph the graph
       * @param options.leadingEdge the leading edge
       * @param options.edges the list of edges that will be aligned with the leading edge
       * @param options.lineDistance the distance between two edges
       * @param [options.adjustLeadingEdge=false] <code>true</code> if the leading edge should be adjusted if the size of list is odd, <code>false</code> otherwise
       * @param [options.joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
       * @param [options.absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
       * @param [options.relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
       * @static
       */
      static routeEdgesParallel(options:{graph:yfiles.layout.LayoutGraph,leadingEdge:yfiles.algorithms.Edge,edges:yfiles.algorithms.EdgeList,lineDistance:number,adjustLeadingEdge?:boolean,joinEnds?:boolean,absJoinDistance?:number,relJoinDistance?:number}):void;
      /**
       * Routes two edges which are incident to the same nodes in parallel, optionally joining the end points.
       * <p>
       * The second edge gets a path that is parallel to the path of the first (leading) edge.
       * </p>
       * @param graph the graph
       * @param leadingEdge the leading edge
       * @param edge the edge to be adjusted
       * @param lineDistance the distance between the two edges
       * @param [joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
       * @param [absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
       * @param [relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
       * @static
       */
      static routeEdgesParallel(graph:yfiles.layout.LayoutGraph,leadingEdge:yfiles.algorithms.Edge,edge:yfiles.algorithms.Edge,lineDistance:number,joinEnds?:boolean,absJoinDistance?:number,relJoinDistance?:number):void;
      /**
       * Routes two edges which are incident to the same nodes in parallel, optionally joining the end points.
       * <p>
       * The second edge gets a path that is parallel to the path of the first (leading) edge.
       * </p>
       * @param {Object} options The parameters to pass.
       * @param options.graph the graph
       * @param options.leadingEdge the leading edge
       * @param options.edge the edge to be adjusted
       * @param options.lineDistance the distance between the two edges
       * @param [options.joinEnds=false] <code>true</code> if the edges should share their end points (the ones of the leading edge), <code>false</code> otherwise
       * @param [options.absJoinDistance=0.0] the absolute distance between the end points and the beginning of the parallel segment routing if the ends are joined
       * @param [options.relJoinDistance=0.0] the relative distance, measured relative to the length of the first/last segments if the ends are joined
       * @static
       */
      static routeEdgesParallel(options:{graph:yfiles.layout.LayoutGraph,leadingEdge:yfiles.algorithms.Edge,edge:yfiles.algorithms.Edge,lineDistance:number,joinEnds?:boolean,absJoinDistance?:number,relJoinDistance?:number}):void;
      /**
       * Routes a self-loop.
       * <p>
       * For a self-loop edge, source and target are identical. The path of the self-loop will start at the top of the node, go
       * around the upper-left corner and end at the left side of the node.
       * </p>
       * @param graph the graph to which the edge belongs
       * @param edge the self-loop
       * @static
       */
      static routeSelfLoop(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.LayoutMultiplexer} delegates its layout calls to a core layout algorithm that is dynamically chosen at
     * runtime by inspecting the nodes of the input graph.
     * <p>
     * If a {@link yfiles.algorithms.IDataProvider} is registered with the given graph using the look-up key
     * {@link yfiles.layout.LayoutMultiplexer#LAYOUT_DP_KEY}, it is used for retrieving the {@link yfiles.layout.ILayoutAlgorithm} objects that
     * are stored for the graph's nodes. If no such {@link yfiles.algorithms.IDataProvider} is registered, this layout stage's {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}
     * (if any) is used for the layout calculation of all nodes.
     * </p>
     * <p>
     * Note that the first non-<code>null</code> {@link yfiles.layout.ILayoutAlgorithm} instance retrieved from the {@link yfiles.algorithms.IDataProvider}
     * is used for all nodes.
     * </p>
     * <p>
     * The main purpose of this class is to enable the use of individual layout algorithms for different components (when used
     * as the core layout algorithm of {@link yfiles.layout.ComponentLayout}) or for the contents of group nodes (when used as the core
     * layout algorithm of {@link yfiles.layout.RecursiveGroupLayout}).
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface LayoutMultiplexer extends yfiles.layout.LayoutStageBase{}
    export class LayoutMultiplexer {
      /**
       * Creates a new instance of {@link yfiles.layout.LayoutMultiplexer}.
       */
      constructor();
      /**
       * Returns the core layout algorithm associated with the nodes contained in the given graph.
       * <p>
       * This implementation looks up the layout algorithm in the {@link yfiles.algorithms.IDataProvider} registered with the key
       * {@link yfiles.layout.LayoutMultiplexer#LAYOUT_DP_KEY}. The first non-<code>null</code> {@link yfiles.layout.ILayoutAlgorithm layout algorithm} instance found for a node of the
       * given graph will be returned by this method. If no algorithm is registered at all, it will fall back to the
       * {@link yfiles.layout.LayoutStageBase#coreLayout default core layout algorithm}.
       * </p>
       * <p>
       * This method is called in {@link yfiles.layout.LayoutMultiplexer#applyLayout} and may be overridden to implement a different way
       * of selecting the {@link yfiles.layout.ILayoutAlgorithm layout algorithms}.
       * </p>
       * @param graph the input graph
       * @returns the according layout algorithm
       * @protected
       */
      getCoreLayout(graph:yfiles.layout.LayoutGraph):yfiles.layout.ILayoutAlgorithm;
      /**
       * Data provider key for assigning layout algorithms to the nodes
       * <p>
       * At runtime, the layout algorithms are determined and applied to the current graph. The first non-<code>null</code> {@link yfiles.layout.ILayoutAlgorithm}
       * instance is used for the whole graph.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.layout.ILayoutAlgorithm>}
       */
      static LAYOUT_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.layout.ILayoutAlgorithm>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Abstract base class implementing {@link yfiles.layout.ILayoutStage} that handles the management of the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * <p>
     * Subclasses of this stage can use the provided methods, such that the management of the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} does not
     * need to be considered anymore.
     * </p>
     * @see yfiles.layout.MultiStageLayout
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface LayoutStageBase extends Object,yfiles.layout.ILayoutStage{}
    export class LayoutStageBase {
      /**
       * Creates a new {@link yfiles.layout.ILayoutStage} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * 
       * @param graph
       * @abstract
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Invokes the layout process of the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * <p>
       * Overriding classes may call this method during {@link yfiles.layout.ILayoutAlgorithm#applyLayout} to delegate arranging the
       * graph to the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. They can add pre- and post-processing code before and after this call.
       * </p>
       * @param graph the input graph
       * @protected
       */
      applyLayoutCore(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Gets or sets the core layout algorithm that is wrapped by this stage.
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.MinimumNodeSizeStage} enforces a given minimum width/height of the nodes of a graph during the layout
     * process.
     * <p>
     * It temporarily enlarges nodes whose width/height is below the specified minimum values.
     * </p>
     * <p>
     * This {@link yfiles.layout.ILayoutStage} is especially useful to prevent that the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} has to handle zero-sized
     * nodes or nodes with negative sizes.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface MinimumNodeSizeStage extends yfiles.layout.LayoutStageBase{}
    export class MinimumNodeSizeStage {
      /**
       * Creates a new {@link yfiles.layout.MinimumNodeSizeStage} instance using the given {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} along with a specific
       * minimum width and height for the nodes.
       * @param coreLayouter the core layout routine
       * @param minWidth the minimum width of nodes that should be enforced
       * @param minHeight the minimum height of nodes that should be enforced
       */
      constructor(coreLayouter:yfiles.layout.ILayoutAlgorithm,minWidth:number,minHeight:number);
      /**
       * Creates a new {@link yfiles.layout.MinimumNodeSizeStage} instance using the given {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * <p>
       * The nodes will be enlarged to be at least <code>1</code> wide and <code>1</code> high.
       * </p>
       * @param coreLayouter the core routine
       */
      constructor(coreLayouter:yfiles.layout.ILayoutAlgorithm);
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class for layout algorithms that provides services to simplify and decompose the input graph before it
     * is passed to the {@link yfiles.layout.MultiStageLayout#applyLayoutCore core layout algorithm} itself.
     * <p>
     * Subclasses of this layout algorithm have to provide implementations for the abstract method
     * {@link yfiles.layout.MultiStageLayout#applyLayoutCore}. It invokes the actual layout routine of this layout algorithm.
     * </p>
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * This class provides a configurable pipeline that contains {@link yfiles.layout.ILayoutStage}s that add preprocessing steps
     * and/or postprocessing steps to the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm}. {@link yfiles.layout.MultiStageLayout} provides some predefined
     * {@link yfiles.layout.ILayoutStage}s, that simplify the input graph for the layout algorithm and complement the result. It is
     * also possible to add custom {@link yfiles.layout.ILayoutStage}s which are executed {@link yfiles.layout.MultiStageLayout#prependStage before} or {@link yfiles.layout.MultiStageLayout#appendStage after the predefined ones}.
     * </p>
     * <p>
     * Each {@link yfiles.layout.ILayoutStage} in the pipeline wraps its successor. When the layout pipeline is processed, the
     * preprocessing code of the preceding {@link yfiles.layout.ILayoutStage} is executed before the code in the current {@link yfiles.layout.ILayoutStage}
     * while the postprocessing code is executed afterwards.
     * </p>
     * <p>
     * The pipeline of predefined {@link yfiles.layout.ILayoutStage}s of {@link yfiles.layout.MultiStageLayout} (when all are activated) causes
     * the following sequence of steps:
     * </p>
     * <ol>
     * <li>Hides the group nodes.</li>
     * <li>Hides all except for the induced subgraph.</li>
     * <li>Decomposes the graph into its connected components. Performs steps 4-12 on each component.</li>
     * <li>Prepares for label placement.</li>
     * <li>Prepares the layout for an orientation change.</li>
     * <li>Hides self-loops.</li>
     * <li>Hides parallel edges.</li>
     * <li>Invokes the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} on the simplified graph.</li>
     * <li>Un-hides and routes parallel edges.</li>
     * <li>Un-hides and routes self-loops.</li>
     * <li>Completes orientation change of the layout.</li>
     * <li>Assigns label positions.</li>
     * <li>Arranges the connected components of the graph.</li>
     * <li>Un-hides non-induced subgraph.</li>
     * <li>Un-hides the group nodes.</li>
     * </ol>
     * <p>
     * In order to take effect, a stage that is not already activated by default needs to be activated using the corresponding
     * method, e.g., {@link yfiles.layout.MultiStageLayout#hideGroupsStageEnabled}. Vice versa, a stage can also be deactivated, if necessary.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutAlgorithm}
     */
    export interface MultiStageLayout extends Object,yfiles.layout.ILayoutAlgorithm{}
    export class MultiStageLayout {
      /**
       * Creates a new {@link yfiles.layout.MultiStageLayout} instance with default settings.
       */
      constructor();
      /**
       * Appends the given {@link yfiles.layout.ILayoutStage} to the layout pipeline.
       * <p>
       * Stages that are added with this method will be invoked at the very end of the layout pipeline. This means after all
       * prepended {@link yfiles.layout.ILayoutStage}s, all predefined stages, and all previously appended {@link yfiles.layout.ILayoutStage}s,
       * but just before the invocation of the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm}.
       * </p>
       * @param stage the {@link } instance to be added
       * @see yfiles.layout.MultiStageLayout#removeStage
       * @see yfiles.layout.MultiStageLayout#prependStage
       */
      appendStage(stage:yfiles.layout.ILayoutStage):void;
      /**
       * Calculates a layout for the given graph and applies it directly to the graph.
       * <p>
       * This method executes all activated {@link yfiles.layout.ILayoutStage}s in the layout pipeline as well as the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm}.
       * </p>
       * @param layoutGraph the input graph
       * @see yfiles.layout.MultiStageLayout#appendStage
       * @see yfiles.layout.MultiStageLayout#prependStage
       * @see yfiles.layout.MultiStageLayout#applyLayoutCore
       */
      applyLayout(layoutGraph:yfiles.layout.LayoutGraph):void;
      /**
       * Invokes the core layout algorithm.
       * <p>
       * This method should be implemented by subclasses in order to perform the layout routine of the layout algorithm.
       * </p>
       * @param graph the input graph
       * @abstract
       */
      applyLayoutCore(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Checks the sizes of the nodes to be non-zero.
       * @param g The graph to check.
       * @protected
       */
      checkNodeSize(g:yfiles.layout.LayoutGraph):void;
      /**
       * Deactivates all predefined {@link yfiles.layout.ILayoutStage}s so that upon {@link yfiles.layout.MultiStageLayout#applyLayout} only the
       * {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} will be executed.
       */
      disableAllStages():void;
      /**
       * Prepends the given {@link yfiles.layout.ILayoutStage} to the layout pipeline.
       * <p>
       * Stages that are added with this method will be invoked at the very beginning of the layout pipeline. This means before
       * all previously prepended {@link yfiles.layout.ILayoutStage}s, all predefined stages, and all appended
       * {@link yfiles.layout.ILayoutStage}s.
       * </p>
       * @param stage the {@link } instance to be added
       * @see yfiles.layout.MultiStageLayout#removeStage
       * @see yfiles.layout.MultiStageLayout#appendStage
       */
      prependStage(stage:yfiles.layout.ILayoutStage):void;
      /**
       * Removes the given {@link yfiles.layout.ILayoutStage} from the layout pipeline.
       * <p>
       * This method can only remove {@link yfiles.layout.ILayoutStage}s that have been previously added using {@link yfiles.layout.MultiStageLayout#appendStage}
       * or {@link yfiles.layout.MultiStageLayout#prependStage}. Predefined {@link yfiles.layout.ILayoutStage}s can be deactivated separately.
       * </p>
       * @param stage a {@link } to be removed from the layout pipeline
       * @see yfiles.layout.MultiStageLayout#prependStage
       * @see yfiles.layout.MultiStageLayout#appendStage
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#subgraphLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#labelingEnabled
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       */
      removeStage(stage:yfiles.layout.ILayoutStage):void;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that places the labels of the input graph.
       * @see yfiles.layout.MultiStageLayout#labelingEnabled
       * @see yfiles.labeling.GenericLabeling
       * @type {yfiles.layout.ILayoutStage}
       */
      labeling:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that routes self-loops.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.SelfLoopRouter
       * @type {yfiles.layout.ILayoutStage}
       */
      selfLoopRouter:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that routes parallel edges.
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.router.ParallelEdgeRouter
       * @type {yfiles.layout.ILayoutStage}
       */
      parallelEdgeRouter:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that arranges the connected components of an input graph.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.ComponentLayout
       * @type {yfiles.layout.ILayoutStage}
       */
      componentLayout:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that constrains the layout process to a subgraph of the input graph.
       * @see yfiles.layout.MultiStageLayout#subgraphLayoutEnabled
       * @see yfiles.layout.SubgraphLayout
       * @type {yfiles.layout.ILayoutStage}
       */
      subgraphLayout:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that hides the group nodes of the input graph.
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @see yfiles.layout.HideGroupsStage
       * @type {yfiles.layout.ILayoutStage}
       */
      hideGroupsStage:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that modifies the orientation of a computed layout.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {yfiles.layout.ILayoutStage}
       */
      orientationLayout:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} may be deactivated if the orientation is irrelevant for the layout or if the core layout
       * algorithm already handles the layout orientation by itself.
       * </p>
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      /**
       * Gets or sets the main orientation of the layout.
       * <p>
       * This is a convenience method that configures the {@link yfiles.layout.MultiStageLayout#orientationLayout orientation layout stage}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified orientation does not match a default layout orientation
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @type {yfiles.layout.LayoutOrientation}
       */
      layoutOrientation:yfiles.layout.LayoutOrientation;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} should be deactivated if the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} handles self-loops by itself.
       * </p>
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} used for placing the labels of the input graph is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} may be activated to apply a generic labeling algorithm to the input graph. It will then
       * try to find the best locations for the labels in the layout calculated by the
       * {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm}. When the stage is deactivated, the labels are ignored or placed by an integrated labeling of the
       * layout algorithm.
       * </p>
       * @see yfiles.layout.MultiStageLayout#labeling
       * @see yfiles.labeling.GenericLabeling
       * @type {boolean}
       */
      labelingEnabled:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} used for hiding group nodes is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} should be deactivated if the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} handles group nodes itself.
       * </p>
       * @see yfiles.layout.MultiStageLayout#hideGroupsStage
       * @see yfiles.layout.HideGroupsStage
       * @type {boolean}
       */
      hideGroupsStageEnabled:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} should be deactivated if the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} is able to handle several connected
       * components.
       * </p>
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing parallel edges is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} should be deactivated if the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} handles parallel edges itself.
       * </p>
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouter
       * @see yfiles.router.ParallelEdgeRouter
       * @type {boolean}
       */
      parallelEdgeRouterEnabled:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ILayoutStage} used for constraining the layout process to a subgraph of the
       * input graph is activated.
       * <p>
       * This {@link yfiles.layout.ILayoutStage} may be activated if the {@link yfiles.layout.MultiStageLayout#applyLayoutCore layout algorithm} should be applied only to a specific part of
       * the graph. The remaining graph will stay unchanged.
       * </p>
       * @see yfiles.layout.MultiStageLayout#subgraphLayout
       * @see yfiles.layout.SubgraphLayout
       * @type {boolean}
       */
      subgraphLayoutEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.NodeHalo} specifies a rectangular area around a specific node.
     * <p>
     * A {@link yfiles.layout.ILayoutAlgorithm layout algorithm} that supports
     * {@link yfiles.layout.NodeHalo}s, keeps this area clear of graph elements, except the node labels of this specific node and the
     * adjacent segments of its edges. All minimum distances to this node, which are used in the layout calculation, e.g.,
     * minimum first segment length, will be extended by the halo size.
     * </p>
     * <p>
     * This class defines a key used for registering a {@link yfiles.algorithms.IDataProvider} that provides {@link yfiles.layout.NodeHalo}s
     * for all nodes of the input graph. {@link yfiles.layout.NodeHalo}-aware layout algorithms like {@link yfiles.hierarchic.HierarchicLayout}
     * look for {@link yfiles.algorithms.IDataProvider}s registered with this key to retrieve {@link yfiles.layout.NodeHalo} information.
     * </p>
     * <p>
     * The following layouters support {@link yfiles.layout.NodeHalo}s with different restrictions:
     * </p>
     * <ul>
     * <li>{@link yfiles.hierarchic.HierarchicLayout}</li>
     * <li>{@link yfiles.orthogonal.OrthogonalLayout}</li>
     * <li>{@link yfiles.orthogonal.CompactOrthogonalLayout}</li>
     * <li>{@link yfiles.tree.TreeLayout}</li>
     * <li>{@link yfiles.tree.ClassicTreeLayout}</li>
     * <li>{@link yfiles.organic.OrganicLayout}</li>
     * <li>{@link yfiles.labeling.GenericLabeling}</li>
     * <li>{@link yfiles.router.EdgeRouter}</li>
     * <li>{@link yfiles.circular.CircularLayout}</li>
     * <li>{@link yfiles.tree.BalloonLayout}</li>
     * <li>{@link yfiles.layout.SelfLoopRouter}</li>
     * <li>{@link yfiles.radial.RadialLayout}</li>
     * <li>{@link yfiles.layout.RecursiveGroupLayout}</li>
     * <li>{@link yfiles.layout.ComponentLayout}</li>
     * </ul>
     * @class
     * @final
     */
    export interface NodeHalo extends Object{}
    export class NodeHalo {
      /**
       * Creates a {@link yfiles.layout.NodeHalo} with the specified values.
       * <p>
       * The values must not be negative, {@link number#NaN} or {@link number#POSITIVE_INFINITY}.
       * </p>
       * @param top the {@link } size at the top side of the node
       * @param left the {@link } size at the left side of the node
       * @param bottom the {@link } size at the bottom side of the node
       * @param right the {@link } size at the right side of the node
       * @returns the newly created {@link }
       * @throws {Stubs.Exceptions.ArgumentError} if one of the values is negative, {@link number#NaN} or {@link number#POSITIVE_INFINITY}
       * @static
       */
      static create(top:number,left:number,bottom:number,right:number):yfiles.layout.NodeHalo;
      /**
       * Creates a {@link yfiles.layout.NodeHalo} object with the specified value on each side of the node.
       * <p>
       * The value must not be negative, {@link number#NaN} or {@link number#POSITIVE_INFINITY}.
       * </p>
       * @param value the halo size at each side of the node
       * @returns the newly created {@link }
       * @throws {Stubs.Exceptions.ArgumentError} if the value is negative, {@link number#NaN} or {@link number#POSITIVE_INFINITY}
       * @static
       */
      static create(value:number):yfiles.layout.NodeHalo;
      /**
       * Returns the {@link yfiles.layout.NodeHalo} associated with the given node.
       * <p>
       * If no such {@link yfiles.layout.NodeHalo} is available, this method returns a {@link yfiles.layout.NodeHalo#ZERO_HALO}.
       * </p>
       * @param graph the graph containing the given node
       * @param node the node whose {@link } will be determined
       * @returns the {@link } of the given node if available, otherwise a {@link #ZERO_HALO}
       * @static
       */
      static getHalo(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):yfiles.layout.NodeHalo;
      /**
       * Returns a {@link yfiles.algorithms.YRectangle} instance with the bounds of the given node including its {@link yfiles.layout.NodeHalo}.
       * <p>
       * This {@link yfiles.algorithms.YRectangle} is located at the position of the node.
       * </p>
       * <p>
       * If the node does not have a {@link yfiles.layout.NodeHalo} assigned, this box has the same size as the node.
       * </p>
       * @param graph the graph containing the given node
       * @param node the node whose {@link } will be determined
       * @returns a {@link } instance with the bounds of the given node including its {@link }
       * @static
       */
      static getHaloBox(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):yfiles.algorithms.YRectangle;
      /**
       * Checks whether or not the given graph contains {@link yfiles.layout.NodeHalo} information.
       * <p>
       * This method looks for a {@link yfiles.algorithms.IDataProvider} that has been registered with the given graph using
       * {@link yfiles.layout.NodeHalo#NODE_HALO_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @returns <code>true</code> if a {@link } has been registered with the according key, <code>false</code> otherwise
       * @static
       */
      static hasHalos(graph:yfiles.layout.LayoutGraph):boolean;
      /**
       * Data provider key for assigning halos for the nodes
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.layout.NodeHalo>}
       */
      static NODE_HALO_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.layout.NodeHalo>;
      /**
       * A constant holding a {@link yfiles.layout.NodeHalo} with zero size on each side.
       * @const
       * @static
       * @type {yfiles.layout.NodeHalo}
       */
      static ZERO_HALO:yfiles.layout.NodeHalo;
      /**
       * The {@link yfiles.layout.NodeHalo} size at the top side of the node.
       * @const
       * @type {number}
       */
      top:number;
      /**
       * The {@link yfiles.layout.NodeHalo} size at the left side of the node.
       * @const
       * @type {number}
       */
      left:number;
      /**
       * The {@link yfiles.layout.NodeHalo} size at the bottom side of the node.
       * @const
       * @type {number}
       */
      bottom:number;
      /**
       * The {@link yfiles.layout.NodeHalo} size at the right side of the node.
       * @const
       * @type {number}
       */
      right:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.NodeLabelCandidate} describes one valid placement for a node label considering the label model.
     * @class
     * @extends {yfiles.layout.LabelCandidate}
     */
    export interface NodeLabelCandidate extends yfiles.layout.LabelCandidate{}
    export class NodeLabelCandidate {
      /**
       * Creates a new instance of {@link yfiles.layout.NodeLabelCandidate} described by location, size, model parameter and internal
       * flag.
       * @param pos the location of the upper left corner of the candidate
       * @param size the size of the candidate
       * @param param the parameters of the label model associated with this candidate
       * @param owner the label associated with the candidate
       * @param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.INodeLabelLayout,internal?:boolean);
      /**
       * Creates a new instance of {@link yfiles.layout.NodeLabelCandidate} described by its box, model parameter and internal flag.
       * @param labelBox the box that specifies the candidate's size and position
       * @param param the parameters of the label model associated with this candidate
       * @param owner the label associated with the candidate
       * @param [internal=false] <code>true</code> if the candidate is allowed to overlap with its owner, <code>false</code> otherwise
       */
      constructor(labelBox:yfiles.algorithms.YOrientedRectangle,param:Object,owner:yfiles.layout.INodeLabelLayout,internal?:boolean);
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.NormalizeGraphElementOrderStage} normalizes the order of the elements within a graph to ensure the same
     * order for multiple layout invocations.
     * <p>
     * Among other factors, the results produced by layout algorithms usually depend on the order of the nodes and edges within
     * a graph. Unfortunately, useful operations such as hiding or unhiding elements from a graph or simply invoking layout
     * algorithms on a graph will have the potential side effect of changing that order.
     * </p>
     * <p>
     * With this {@link yfiles.layout.ILayoutStage} it is possible to establish a predefined order of nodes and edges within a graph to
     * avoid non-deterministic layout behavior. Defining the order of nodes and edges is done by associating each node or edge
     * in the graph with a {@link yfiles.lang.IComparable} value using a {@link yfiles.algorithms.IDataProvider} registered with the key {@link yfiles.layout.NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY}
     * or {@link yfiles.layout.NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY}, respectively.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface NormalizeGraphElementOrderStage extends yfiles.layout.LayoutStageBase{}
    export class NormalizeGraphElementOrderStage {
      /**
       * Creates a new {@link yfiles.layout.NormalizeGraphElementOrderStage} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Assigns comparable values for each node and edge.
       * <p>
       * The compared values are induced from the current ordering of the nodes and edges within the given graph.
       * </p>
       * <p>
       * This is a convenient method to fill the given {@link yfiles.algorithms.IDataMap}s which can then be registered with the graph
       * with keys {@link yfiles.layout.NormalizeGraphElementOrderStage#NODE_COMPARABLE_DP_KEY} and {@link yfiles.layout.NormalizeGraphElementOrderStage#EDGE_COMPARABLE_DP_KEY}
       * and used for several layout calculations.
       * </p>
       * @param graph the input graph
       * @param comparableNodeMap the map that will be filled with comparable values for the nodes
       * @param comparableEdgeMap the map that will be filled with comparable values for the edges
       * @static
       */
      static fillComparableMapFromGraph(graph:yfiles.algorithms.Graph,comparableNodeMap:yfiles.algorithms.IDataMap,comparableEdgeMap:yfiles.algorithms.IDataMap):void;
      /**
       * Data provider key for defining a (global) order for nodes
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.lang.IComparable>}
       */
      static NODE_COMPARABLE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.lang.IComparable>;
      /**
       * Data provider key for defining a (global) order for edges
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.lang.IComparable>}
       */
      static EDGE_COMPARABLE_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.lang.IComparable>;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.OrientationLayout} is a {@link yfiles.layout.ILayoutStage} that changes the orientation of the layout.
     * <p>
     * <b>Layout Style</b> There are four orientations that can be used for a layout. These orientations can exist with or without mirroring at
     * the x-axis or y-axis.
     * </p>
     * <p>
     * Changing the layout orientation is useful for layout styles that show a main direction.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[327.5,0],[136.25,100.31],[327.5,100.31],[507.5,100.31],[507.5,180.94],[260,180.94],[327.5,180.94],[447.5,180.94],[440,262.19],[136.25,180.94],[76.25,180.94],[387.5,342.81],[380,262.19],[500,262.19],[320,262.19],[320,342.81],[560,262.19],[196.25,180.94],[20,262.19],[260,262.19],[80,262.19],[387.5,417.81],[140,262.19],[447.5,342.81],[80,342.81],[620,180.94],[200,262.19],[387.5,180.94],[200,342.81],[140,342.81],[20,342.81],[447.5,417.81],[200,417.81],[320,417.81],[620,262.19],[507.5,342.81],[507.5,417.81],[620,342.81],[567.5,417.81],[687.5,342.81],[687.5,417.81],[627.5,417.81]],"e":[[0,1,-10,15,0,-15,[332.5,40,151.25,85.31]],[0,2,0,15,0,-15],[0,3,10,15,0,-15,[352.5,40,522.5,85.31]],[3,4,0,15,0,-15],[2,5,-10,15,0,-15,[332.5,140.31,275,165.94]],[2,6,0,15,0,-15],[3,7,-10,15,0,-15,[512.5,140.31,462.5,165.94]],[7,8,0,15,0,-15,[462.5,220.94,455,247.19]],[1,9,0,15,0,-15],[1,10,-10,15,0,-15,[141.25,140.31,91.25,165.94]],[8,11,-7.5,15,0,-15,[447.5,302.19,402.5,327.81]],[6,12,7.5,15,0,-15,[350,220.94,395,247.19]],[4,13,-7.5,15,0,-15],[6,14,-7.5,15,0,-15],[14,15,0,15,0,-15],[4,16,7.5,15,0,-15,[530,220.94,575,247.19]],[1,17,10,15,0,-15,[161.25,140.31,211.25,165.94]],[9,18,-11.25,15,0,-15,[140,220.94,35,247.19]],[5,19,0,15,0,-15],[9,20,-3.75,15,0,-15,[147.5,220.94,95,247.19]],[11,21,0,15,0,-15],[9,22,3.75,15,0,-15],[8,23,7.5,15,0,-15],[20,24,0,15,0,-15],[3,25,10,15,0,-15,[532.5,140.31,635,165.94]],[9,26,11.25,15,0,-15,[162.5,220.94,215,247.19]],[2,27,10,15,0,-15,[352.5,140.31,402.5,165.94]],[26,28,0,15,0,-15],[22,29,0,15,0,-15],[18,30,0,15,0,-15],[23,31,0,15,0,-15],[28,32,0,15,0,-15],[15,33,0,15,0,-15],[25,34,0,15,0,-15],[34,35,-10,15,0,-15,[625,302.19,522.5,327.81]],[35,36,0,15,0,-15],[34,37,0,15,0,-15],[37,38,-7.5,15,0,-15,[627.5,382.81,582.5,402.81]],[34,39,10,15,0,-15,[645,302.19,702.5,327.81]],[39,40,0,15,0,-15],[37,41,7.5,15,0,-15]],"vp":[20.0,0.0,698.0,448.0]}}
     * </p>
     * <p>
     * {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM} layout
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[327.5,-30],[136.25,-130.31],[327.5,-130.31],[507.5,-130.31],[507.5,-210.94],[260,-210.94],[327.5,-210.94],[447.5,-210.94],[440,-292.19],[136.25,-210.94],[76.25,-210.94],[387.5,-372.81],[380,-292.19],[500,-292.19],[320,-292.19],[320,-372.81],[560,-292.19],[196.25,-210.94],[20,-292.19],[260,-292.19],[80,-292.19],[387.5,-447.81],[140,-292.19],[447.5,-372.81],[80,-372.81],[620,-210.94],[200,-292.19],[387.5,-210.94],[200,-372.81],[140,-372.81],[20,-372.81],[447.5,-447.81],[200,-447.81],[320,-447.81],[620,-292.19],[507.5,-372.81],[507.5,-447.81],[620,-372.81],[567.5,-447.81],[687.5,-372.81],[687.5,-447.81],[627.5,-447.81]],"e":[[0,1,-10,-15,0,15,[332.5,-40,151.25,-85.31]],[0,2,0,-15,0,15],[0,3,10,-15,0,15,[352.5,-40,522.5,-85.31]],[3,4,0,-15,0,15],[2,5,-10,-15,0,15,[332.5,-140.31,275,-165.94]],[2,6,0,-15,0,15],[3,7,-10,-15,0,15,[512.5,-140.31,462.5,-165.94]],[7,8,0,-15,0,15,[462.5,-220.94,455,-247.19]],[1,9,0,-15,0,15],[1,10,-10,-15,0,15,[141.25,-140.31,91.25,-165.94]],[8,11,-7.5,-15,0,15,[447.5,-302.19,402.5,-327.81]],[6,12,7.5,-15,0,15,[350,-220.94,395,-247.19]],[4,13,-7.5,-15,0,15],[6,14,-7.5,-15,0,15],[14,15,0,-15,0,15],[4,16,7.5,-15,0,15,[530,-220.94,575,-247.19]],[1,17,10,-15,0,15,[161.25,-140.31,211.25,-165.94]],[9,18,-11.25,-15,0,15,[140,-220.94,35,-247.19]],[5,19,0,-15,0,15],[9,20,-3.75,-15,0,15,[147.5,-220.94,95,-247.19]],[11,21,0,-15,0,15],[9,22,3.75,-15,0,15],[8,23,7.5,-15,0,15],[20,24,0,-15,0,15],[3,25,10,-15,0,15,[532.5,-140.31,635,-165.94]],[9,26,11.25,-15,0,15,[162.5,-220.94,215,-247.19]],[2,27,10,-15,0,15,[352.5,-140.31,402.5,-165.94]],[26,28,0,-15,0,15],[22,29,0,-15,0,15],[18,30,0,-15,0,15],[23,31,0,-15,0,15],[28,32,0,-15,0,15],[15,33,0,-15,0,15],[25,34,0,-15,0,15],[34,35,-10,-15,0,15,[625,-302.19,522.5,-327.81]],[35,36,0,-15,0,15],[34,37,0,-15,0,15],[37,38,-7.5,-15,0,15,[627.5,-382.81,582.5,-402.81]],[34,39,10,-15,0,15,[645,-302.19,702.5,-327.81]],[39,40,0,-15,0,15],[37,41,7.5,-15,0,15]],"vp":[20.0,-448.0,698.0,448.0]}}
     * </p>
     * <p>
     * {@link yfiles.layout.LayoutOrientation#BOTTOM_TO_TOP} layout with {@link yfiles.layout.MirrorModes#BOTTOM_TO_TOP mirroring}
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,-357.5],[100.31,-166.25],[100.31,-357.5],[100.31,-537.5],[180.94,-537.5],[180.94,-290],[180.94,-357.5],[180.94,-477.5],[262.19,-470],[180.94,-166.25],[180.94,-106.25],[342.81,-417.5],[262.19,-410],[262.19,-530],[262.19,-350],[342.81,-350],[262.19,-590],[180.94,-226.25],[262.19,-50],[262.19,-290],[262.19,-110],[417.81,-417.5],[262.19,-170],[342.81,-477.5],[342.81,-110],[180.94,-650],[262.19,-230],[180.94,-417.5],[342.81,-230],[342.81,-170],[342.81,-50],[417.81,-477.5],[417.81,-230],[417.81,-350],[262.19,-650],[342.81,-537.5],[417.81,-537.5],[342.81,-650],[417.81,-597.5],[342.81,-717.5],[417.81,-717.5],[417.81,-657.5]],"e":[[0,1,15,10,-15,-0,[40,-332.5,85.31,-151.25]],[0,2,15,-0,-15,-0],[0,3,15,-10,-15,-0,[40,-352.5,85.31,-522.5]],[3,4,15,-0,-15,-0],[2,5,15,10,-15,-0,[140.31,-332.5,165.94,-275]],[2,6,15,-0,-15,-0],[3,7,15,10,-15,-0,[140.31,-512.5,165.94,-462.5]],[7,8,15,-0,-15,-0,[220.94,-462.5,247.19,-455]],[1,9,15,-0,-15,-0],[1,10,15,10,-15,-0,[140.31,-141.25,165.94,-91.25]],[8,11,15,7.5,-15,-0,[302.19,-447.5,327.81,-402.5]],[6,12,15,-7.5,-15,-0,[220.94,-350,247.19,-395]],[4,13,15,7.5,-15,-0],[6,14,15,7.5,-15,-0],[14,15,15,-0,-15,-0],[4,16,15,-7.5,-15,-0,[220.94,-530,247.19,-575]],[1,17,15,-10,-15,-0,[140.31,-161.25,165.94,-211.25]],[9,18,15,11.25,-15,-0,[220.94,-140,247.19,-35]],[5,19,15,-0,-15,-0],[9,20,15,3.75,-15,-0,[220.94,-147.5,247.19,-95]],[11,21,15,-0,-15,-0],[9,22,15,-3.75,-15,-0],[8,23,15,-7.5,-15,-0],[20,24,15,-0,-15,-0],[3,25,15,-10,-15,-0,[140.31,-532.5,165.94,-635]],[9,26,15,-11.25,-15,-0,[220.94,-162.5,247.19,-215]],[2,27,15,-10,-15,-0,[140.31,-352.5,165.94,-402.5]],[26,28,15,-0,-15,-0],[22,29,15,-0,-15,-0],[18,30,15,-0,-15,-0],[23,31,15,-0,-15,-0],[28,32,15,-0,-15,-0],[15,33,15,-0,-15,-0],[25,34,15,-0,-15,-0],[34,35,15,10,-15,-0,[302.19,-625,327.81,-522.5]],[35,36,15,-0,-15,-0],[34,37,15,-0,-15,-0],[37,38,15,7.5,-15,-0,[382.81,-627.5,402.81,-582.5]],[34,39,15,-10,-15,-0,[302.19,-645,327.81,-702.5]],[39,40,15,-0,-15,-0],[37,41,15,-7.5,-15,-0]],"vp":[0.0,-718.0,448.0,698.0]}}
     * </p>
     * <p>
     * {@link yfiles.layout.LayoutOrientation#LEFT_TO_RIGHT} layout
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[-30,327.5],[-130.31,136.25],[-130.31,327.5],[-130.31,507.5],[-210.94,507.5],[-210.94,260],[-210.94,327.5],[-210.94,447.5],[-292.19,440],[-210.94,136.25],[-210.94,76.25],[-372.81,387.5],[-292.19,380],[-292.19,500],[-292.19,320],[-372.81,320],[-292.19,560],[-210.94,196.25],[-292.19,20],[-292.19,260],[-292.19,80],[-447.81,387.5],[-292.19,140],[-372.81,447.5],[-372.81,80],[-210.94,620],[-292.19,200],[-210.94,387.5],[-372.81,200],[-372.81,140],[-372.81,20],[-447.81,447.5],[-447.81,200],[-447.81,320],[-292.19,620],[-372.81,507.5],[-447.81,507.5],[-372.81,620],[-447.81,567.5],[-372.81,687.5],[-447.81,687.5],[-447.81,627.5]],"e":[[0,1,-15,-10,15,0,[-40,332.5,-85.31,151.25]],[0,2,-15,0,15,0],[0,3,-15,10,15,0,[-40,352.5,-85.31,522.5]],[3,4,-15,0,15,0],[2,5,-15,-10,15,0,[-140.31,332.5,-165.94,275]],[2,6,-15,0,15,0],[3,7,-15,-10,15,0,[-140.31,512.5,-165.94,462.5]],[7,8,-15,0,15,0,[-220.94,462.5,-247.19,455]],[1,9,-15,0,15,0],[1,10,-15,-10,15,0,[-140.31,141.25,-165.94,91.25]],[8,11,-15,-7.5,15,0,[-302.19,447.5,-327.81,402.5]],[6,12,-15,7.5,15,0,[-220.94,350,-247.19,395]],[4,13,-15,-7.5,15,0],[6,14,-15,-7.5,15,0],[14,15,-15,0,15,0],[4,16,-15,7.5,15,0,[-220.94,530,-247.19,575]],[1,17,-15,10,15,0,[-140.31,161.25,-165.94,211.25]],[9,18,-15,-11.25,15,0,[-220.94,140,-247.19,35]],[5,19,-15,0,15,0],[9,20,-15,-3.75,15,0,[-220.94,147.5,-247.19,95]],[11,21,-15,0,15,0],[9,22,-15,3.75,15,0],[8,23,-15,7.5,15,0],[20,24,-15,0,15,0],[3,25,-15,10,15,0,[-140.31,532.5,-165.94,635]],[9,26,-15,11.25,15,0,[-220.94,162.5,-247.19,215]],[2,27,-15,10,15,0,[-140.31,352.5,-165.94,402.5]],[26,28,-15,0,15,0],[22,29,-15,0,15,0],[18,30,-15,0,15,0],[23,31,-15,0,15,0],[28,32,-15,0,15,0],[15,33,-15,0,15,0],[25,34,-15,0,15,0],[34,35,-15,-10,15,0,[-302.19,625,-327.81,522.5]],[35,36,-15,0,15,0],[34,37,-15,0,15,0],[37,38,-15,-7.5,15,0,[-382.81,627.5,-402.81,582.5]],[34,39,-15,10,15,0,[-302.19,645,-327.81,702.5]],[39,40,-15,0,15,0],[37,41,-15,7.5,15,0]],"vp":[-448.0,20.0,448.0,698.0]}}
     * </p>
     * <p>
     * {@link yfiles.layout.LayoutOrientation#RIGHT_TO_LEFT} layout
     * </p>
     * <p>
     * <b>Concept</b> {@link yfiles.layout.OrientationLayout} performs three basic steps.
     * </p>
     * <ol>
     * <li>
     * Preparation: The sizes and locations of nodes and edge bends as well as labels and {@link yfiles.layout.PortConstraint}s are
     * transformed to the {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM} orientation.
     * </li>
     * <li>
     * Delegation: The {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} is invoked and calculates the layout for orientation
     * {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}.
     * </li>
     * <li>Completion: The result of the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} is rotated and mirrored to fit the desired layout orientation.</li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * {@link yfiles.layout.OrientationLayout} can either be used as a {@link yfiles.layout.ILayoutStage} wrapping a {@link yfiles.layout.ILayoutAlgorithm layout algorithm}. Then it
     * will change the orientation of the result of the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * <br />
     * If no {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} is specified, {@link yfiles.layout.OrientationLayout} can work alone. It will just change the
     * orientation of the layout without arranging nodes and edges.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface OrientationLayout extends yfiles.layout.LayoutStageBase{}
    export class OrientationLayout {
      /**
       * Creates a new {@link yfiles.layout.OrientationLayout} instance using the given orientation.
       * @param orientation the orientation specifier
       * @throws {Stubs.Exceptions.ArgumentError} if the specified orientation is unknown
       * @see yfiles.layout.OrientationLayout#orientation
       */
      constructor(orientation:yfiles.layout.LayoutOrientation);
      /**
       * Creates a new {@link yfiles.layout.OrientationLayout} instance using the given {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param coreLayouter the core layout routine
       */
      constructor(coreLayouter:yfiles.layout.ILayoutAlgorithm);
      /**
       * Creates a new {@link yfiles.layout.OrientationLayout} instance with default settings.
       */
      constructor();
      /**
       * Applies the desired orientation to the graph.
       * <p>
       * It performs the inverse coordinate transformation that has been applied to the layout by method
       * {@link yfiles.layout.OrientationLayout#prepareOrientationChange}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.OrientationLayout#applyLayout} after invoking the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to extend the orientation change to custom layout features.
       * </p>
       * @param graph the input graph
       * @see yfiles.layout.OrientationLayout#prepareOrientationChange
       */
      completeOrientationChange(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Transforms the given point during completion.
       * <p>
       * The point is transformed from {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM} orientation to the desired orientation.
       * </p>
       * @param point the calculated point
       * @returns the transformed point
       * @see yfiles.layout.OrientationLayout#completeOrientationChange
       */
      completeTransform(point:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Creates {@link yfiles.algorithms.Insets} that are a geometric transformation of the given {@link yfiles.algorithms.Insets}.
       * <p>
       * The transform will correspond to the currently set orientation.
       * </p>
       * @param insets the original insets
       * @returns the oriented insets
       * @see yfiles.layout.OrientationLayout#orientation
       */
      createOrientedInsets(insets:yfiles.algorithms.Insets):yfiles.algorithms.Insets;
      /**
       * Creates a {@link yfiles.layout.NodeHalo} that is a geometric transformation of the given {@link yfiles.layout.NodeHalo}.
       * <p>
       * The transform will correspond to the currently set orientation.
       * </p>
       * @param halo the original {@link }
       * @returns the oriented {@link }
       * @see yfiles.layout.OrientationLayout#orientation
       */
      createOrientedNodeHalo(halo:yfiles.layout.NodeHalo):yfiles.layout.NodeHalo;
      /**
       * Creates a {@link yfiles.algorithms.YDimension} that is a geometric transformation of the given size.
       * <p>
       * The transform will correspond to the currently set orientation.
       * </p>
       * @param nodeSize the original node size
       * @returns the oriented node size
       * @see yfiles.layout.OrientationLayout#orientation
       */
      createOrientedNodeSize(nodeSize:yfiles.algorithms.YDimension):yfiles.algorithms.YDimension;
      /**
       * Returns whether or not this {@link yfiles.layout.OrientationLayout} instance will mirror the graph for a given layout
       * orientation.
       * <p>
       * Horizontal orientations get mirrored at the x-axis while vertical orientations are mirrored at the y-axis.
       * </p>
       * @param orientation the layout orientation for which to check the mirror state
       * @returns <code>true</code> if the layout algorithm will mirror the graph at the corresponding axis for the given layout orientation, <code>false</code>
       * otherwise
       * @see yfiles.layout.OrientationLayout#orientation
       * @see yfiles.layout.OrientationLayout#mirrorMode
       */
      isOrientationMirrored(orientation:yfiles.layout.LayoutOrientation):boolean;
      /**
       * Prepares the layout for the desired orientation.
       * <p>
       * It will rotate and mirror the coordinates and bounds of the graph layout such that the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} can perform
       * its calculations assuming the canonical {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM} orientation.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.OrientationLayout#applyLayout} before invoking the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. It might be overridden to extend the preparations to custom layout features.
       * </p>
       * @param graph the input graph
       * @see yfiles.layout.OrientationLayout#completeOrientationChange
       */
      prepareOrientationChange(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Transforms the given point during preparation.
       * <p>
       * The point is transformed so it matches the {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM} orientation.
       * </p>
       * @param point the original point
       * @returns the transformed point
       * @see yfiles.layout.OrientationLayout#prepareOrientationChange
       */
      prepareTransform(point:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * Transforms the given point during the preparation or completion step.
       * <p>
       * This method is called by {@link yfiles.layout.OrientationLayout#prepareOrientationChange} and {@link yfiles.layout.OrientationLayout#completeOrientationChange}
       * to get the according point location.
       * </p>
       * @param point the original/calculated point
       * @param prepare <code>true</code> if this method is called during preparation, <code>false</code> if it is called during completion
       * @returns the transformed point
       * @see yfiles.layout.OrientationLayout#prepareOrientationChange
       * @see yfiles.layout.OrientationLayout#completeOrientationChange
       * @see yfiles.layout.OrientationLayout#prepareTransform
       * @see yfiles.layout.OrientationLayout#completeTransform
       * @protected
       */
      transform(point:yfiles.algorithms.YPoint,prepare:boolean):yfiles.algorithms.YPoint;
      /**
       * Gets or sets the main direction of the layout.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified orientation is unknown
       * @type {yfiles.layout.LayoutOrientation}
       */
      orientation:yfiles.layout.LayoutOrientation;
      /**
       * Gets or sets the mirror mask that defines which orientations this {@link yfiles.layout.OrientationLayout} shall mirror.
       * <p>
       * {@link yfiles.layout.OrientationLayout#orientation Setting a layout orientation} can be seen as rotating the graph by <code>90</code>, <code>180</code> or <code>270</code> degrees. Afterwards the graph can be
       * mirrored at the x-axis (for horizontal layout orientations) or y-axis (for vertical layout orientations).
       * </p>
       * <p>
       * Which directions are mirrored can be defined by the given mask. It can combine several mirrored directions with a
       * logical <code>or</code>-operation.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown mask is specified
       * @type {yfiles.layout.MirrorModes}
       */
      mirrorMode:yfiles.layout.MirrorModes;
      /**
       * Gets whether or not the current orientation is horizontal.
       * <p>
       * The orientation is horizontal if it is {@link yfiles.layout.LayoutOrientation#LEFT_TO_RIGHT} or
       * {@link yfiles.layout.LayoutOrientation#RIGHT_TO_LEFT}.
       * </p>
       * @see yfiles.layout.OrientationLayout#orientation
       * @type {boolean}
       */
      horizontalOrientation:boolean;
      /**
       * Gets or sets whether or not the position of the edge labels should be changed during orientation change.
       * @type {boolean}
       */
      considerEdgeLabels:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.PortCalculator} adjusts the final port assignments after a layout has been calculated.
     * <p>
     * This can be useful if the port assignment calculated by the layout algorithm is insufficient. This stage uses {@link yfiles.algorithms.IDataProvider}
     * instances registered with the graph using the keys defined in the {@link yfiles.layout.IIntersectionCalculator} interface to
     * calculate the new port offsets.
     * </p>
     * <p>
     * For each edge in the graph, {@link yfiles.layout.IIntersectionCalculator}s are retrieved from these
     * {@link yfiles.algorithms.IDataProvider}s. If such an {@link yfiles.layout.IIntersectionCalculator} is assigned, it will provide the
     * intersection point of the edge and the node. Then this point is added to the {@link yfiles.layout.IEdgeLayout}.
     * </p>
     * <p>
     * For this layout stage to work properly, use {@link yfiles.layout.CompositeLayoutStage#prependStage} or {@link yfiles.layout.MultiStageLayout#prependStage}
     * to add this layout stage and register appropriate {@link yfiles.algorithms.IDataProvider}s with the graph using the keys defined
     * in {@link yfiles.layout.IIntersectionCalculator}.
     * </p>
     * <p>
     * Note that this class will not change the coordinates of a port if it is associated with a strong port constraint.
     * </p>
     * @see yfiles.layout.IIntersectionCalculator
     * @see yfiles.layout.IntersectionCalculatorKeys#SOURCE_INTERSECTION_CALCULATOR_DP_KEY
     * @see yfiles.layout.IntersectionCalculatorKeys#TARGET_INTERSECTION_CALCULATOR_DP_KEY
     * @see yfiles.layout.PortConstraint
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface PortCalculator extends yfiles.layout.LayoutStageBase,yfiles.layout.ILayoutStage{}
    export class PortCalculator {
      /**
       * Creates a new {@link yfiles.layout.PortCalculator} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       */
      constructor();
      /**
       * Performs the actual port calculation on the specified graph instance.
       * @param graph the input graph
       * @protected
       */
      calculatePorts(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Determines whether two points are equal.
       * <p>
       * This implementation uses {@link yfiles.layout.PortCalculator#Eps} to add some tolerance to the comparison. It is called by {@link yfiles.layout.PortCalculator#calculatePorts}
       * and may be overridden to change the accuracy/tolerance of the comparison.
       * </p>
       * @param x1 the x-coordinate of the first point
       * @param y1 the y-coordinate of the first point
       * @param x2 the x-coordinate of the second point
       * @param y2 the y-coordinate of the second point
       * @returns <code>true</code> if both points are considered equal, <code>false</code> otherwise
       * @see yfiles.layout.PortCalculator#calculatePorts
       * @see yfiles.layout.PortCalculator#Eps
       * @protected
       */
      equalsEps(x1:number,y1:number,x2:number,y2:number):boolean;
      /**
       * EPSILON used by {@link yfiles.layout.PortCalculator#equalsEps}.
       * <p>
       * By default this is <code>0.2d</code>.
       * </p>
       * @protected
       * @type {number}
       */
      Eps:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents a candidate port on which edges can be connected to their source and/or target node.
     * <p>
     * The notion of {@link yfiles.layout.PortCandidate}s extends the one of {@link yfiles.layout.PortConstraint}s. In contrast to
     * {@link yfiles.layout.PortConstraint}s, they can be associated with both edges and nodes. Moreover, more than one {@link yfiles.layout.PortCandidate}
     * can be specified for an edge or a node.
     * </p>
     * <p>
     * <b>Edge</b> {@link yfiles.layout.PortCandidate}s can be introduced to define specific sides or exact locations on which a given edge can be
     * connected to its source or target node.
     * <br />
     * Information about edge {@link yfiles.layout.PortCandidate}s is held by {@link yfiles.algorithms.IDataProvider}s that are registered with
     * the graph using keys {@link yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY} (to specify
     * {@link yfiles.layout.PortCandidate}s for an edge on the source node) and {@link yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY}
     * (to specify {@link yfiles.layout.PortCandidate}s for an edge on the target node). The {@link yfiles.algorithms.IDataProvider}s hold a {@link yfiles.collections.ICollection.<T> collection}
     * of {@link yfiles.layout.PortCandidate}s for each edge.
     * </p>
     * <p>
     * <b>Node</b> {@link yfiles.layout.PortCandidate}s determine available ports on the nodes to which edges can be connected. This means that an
     * edge of the graph can be connected to any of these node ports (unless edge {@link yfiles.layout.PortCandidate}s are specified
     * too).
     * <br />
     * To register and define node {@link yfiles.layout.PortCandidate}s refer to {@link yfiles.layout.PortCandidateSet}.
     * </p>
     * <p>
     * A {@link yfiles.layout.PortCandidate} is described by the following attributes:
     * </p>
     * <ul>
     * <li>
     * Offsets: The X-offset (Y-offset) determines the horizontal (vertical) distance of the {@link yfiles.layout.PortCandidate}
     * relative to the center of the node.
     * </li>
     * <li>
     * Fixed or free: Fixed
     * {@link yfiles.layout.PortCandidate}s are defined using exact coordinates, whereas coordinates of free candidates are selected
     * arbitrarily.
     * </li>
     * <li>
     * Direction: Corresponds to the incoming or outgoing directions of the edges when connecting to a target or source node,
     * respectively.
     * </li>
     * <li>
     * Penalty cost: Represents the cost of usage of a
     * {@link yfiles.layout.PortCandidate}. During the processing of an edge by a layout or routing algorithm, candidates of lower cost
     * will be considered first.
     * </li>
     * </ul>
     * <p>
     * {@graph {"ann":{"s":[60,30],"d":1},"styles":{"6":{"f":"#ff0000","b":1, "c":1}, "7":{"f":"#00ffff","b":1}, "8":{"f":"#008000","b":1}},"n":[[646.5,475.5],[725.5,528.5],[831.5,470],[669.5,605.5],[722.5,530.5,6,6,8],[724,549.5,3,8,7],[783,531,5,5,6],[783,551,5,5,6],[667.5,502,6,6,8],[696.5,604.5,8,3,7],[848,498,5,5,6],[696.5,634,8,3,7],[848,468.12,5,5,6],[667.5,472.5,6,6,8],[737.5,525,6,6,8]],"e":[[0,1,-6,15,-30,-10,[670.5,533.5]],[2,1,-11,15,30,-10,[850.5,533.5]],[3,1,1,-15,-30,10,[700.5,553.5]]],"vp":[646.0,468.0,246.0,169.0]}} Example of using edge port candidates. Edge port candidates are colored uniquely based on the edge with which they are
     * associated.
     * </p>
     * <p>
     * Not all layout and routing algorithms are able to handle {@link yfiles.layout.PortCandidate}s. Major layout algorithms that obey
     * them are {@link yfiles.hierarchic.HierarchicLayout}, {@link yfiles.router.EdgeRouter}, {@link yfiles.router.ChannelEdgeRouter} and
     * {@link yfiles.router.BusRouter}.
     * </p>
     * @class
     */
    export interface PortCandidate extends Object{}
    export class PortCandidate {
      /**
       * Factory method that creates a fixed {@link yfiles.layout.PortCandidate} instance with the given cost of usage, direction and
       * offset values.
       * @param directionMask any combination of the default direction specifiers
       * @param xOffset the X-offset relative to the center of the node
       * @param yOffset the Y-offset relative to the center of the node
       * @param [cost=0.0] the penalty cost for using this {@link } instance
       * @returns a new {@link } instance
       * @static
       */
      static createCandidate(xOffset:number,yOffset:number,directionMask:yfiles.layout.PortDirections,cost?:number):yfiles.layout.PortCandidate;
      /**
       * Factory method that creates a free {@link yfiles.layout.PortCandidate} instance with the given direction and cost of usage.
       * @param directionMask any combination of the default direction specifiers
       * @param cost the penalty cost for using this {@link }
       * @returns a new {@link } instance
       * @static
       */
      static createCandidate(directionMask:yfiles.layout.PortDirections,cost:number):yfiles.layout.PortCandidate;
      /**
       * Factory method that returns a free {@link yfiles.layout.PortCandidate} instance with zero cost of usage and the direction
       * specified by the {@link yfiles.layout.PortConstraint#side side} of the given {@link yfiles.layout.PortConstraint}.
       * @param fromPortConstraint the {@link } whose {@link #side side} should be used as direction
       * @returns a new {@link } instance
       * @static
       */
      static createCandidate(fromPortConstraint:yfiles.layout.PortConstraint):yfiles.layout.PortCandidate;
      /**
       * Factory method that creates a free {@link yfiles.layout.PortCandidate} instance with the given direction and zero cost of
       * usage.
       * @param directionMask any combination of the default direction specifiers
       * @returns a new {@link } instance
       * @static
       */
      static createCandidate(directionMask:yfiles.layout.PortDirections):yfiles.layout.PortCandidate;
      /**
       * Returns the direction of this {@link yfiles.layout.PortCandidate} instance with respect to the given {@link yfiles.layout.OrientationLayout#orientation layout orientation}.
       * <p>
       * If the direction mask is not required to be calculated with respect to the actual {@link yfiles.layout.OrientationLayout#orientation layout orientation}, {@link yfiles.layout.PortCandidate#direction} should be used
       * instead.
       * </p>
       * @param layoutOrientation
       * @returns the direction of this {@link } with respect to the given layout orientation
       * @see yfiles.layout.PortCandidate#direction
       */
      getDirectionForLayoutOrientation(layoutOrientation:yfiles.layout.LayoutOrientation):yfiles.layout.PortDirections;
      /**
       * Returns the X-offset of the {@link yfiles.layout.PortCandidate} with respect to the center of the corresponding node for the
       * given {@link yfiles.layout.OrientationLayout#orientation layout orientation} and {@link yfiles.layout.OrientationLayout#mirrorMode mirror mask}.
       * <p>
       * The X-offset encodes the horizontal distance of the port relative to the center of a node. If the {@link yfiles.layout.PortCandidate}
       * is not {@link yfiles.layout.PortCandidate#fixed fixed} (i.e., free), the X-offset is equal to zero.
       * </p>
       * @param layoutOrientation the layout orientation
       * @param mirrorMask the mirror mask
       * @returns the X-offset of the port with respect to the given orientation and mirror mask
       */
      getXOffsetForLayoutOrientation(layoutOrientation:yfiles.layout.LayoutOrientation,mirrorMask:yfiles.layout.MirrorModes):number;
      /**
       * Returns the X-offset of the {@link yfiles.layout.PortCandidate} with respect to the center of the corresponding node for the
       * given {@link yfiles.layout.OrientationLayout#orientation layout orientation}.
       * <p>
       * The X-offset encodes the horizontal distance of the port relative to the center of a node. If the {@link yfiles.layout.PortCandidate}
       * is not {@link yfiles.layout.PortCandidate#fixed fixed} (i.e., free), the X-offset is equal to zero.
       * </p>
       * @param layoutOrientation the layout orientation
       * @returns the X-offset of the {@link } with respect to the given layout orientation
       */
      getXOffsetForLayoutOrientation(layoutOrientation:yfiles.layout.LayoutOrientation):number;
      /**
       * Returns the Y-offset of the {@link yfiles.layout.PortCandidate} with respect to the center of the corresponding node for the
       * given {@link yfiles.layout.OrientationLayout#orientation layout orientation} and {@link yfiles.layout.OrientationLayout#mirrorMode mirror mask}.
       * <p>
       * The Y-offset encodes the vertical distance of the port relative to the center of a node. If the {@link yfiles.layout.PortCandidate}
       * is not {@link yfiles.layout.PortCandidate#fixed fixed} (i.e., free), the Y-offset is equal to zero.
       * </p>
       * @param layoutOrientation the layout orientation
       * @param mirrorMask the mirror mask
       * @returns the Y-offset of the {@link } with respect to the given orientation and mirror mask
       */
      getYOffsetForLayoutOrientation(layoutOrientation:yfiles.layout.LayoutOrientation,mirrorMask:yfiles.layout.MirrorModes):number;
      /**
       * Returns the Y-offset of the {@link yfiles.layout.PortCandidate} with respect to the center of the corresponding node for the
       * given {@link yfiles.layout.OrientationLayout#orientation layout orientation}.
       * <p>
       * The Y-offset encodes the vertical distance of the port relative to the center of a node. If the {@link yfiles.layout.PortCandidate}
       * is not {@link yfiles.layout.PortCandidate#fixed fixed} (i.e., free), the Y-offset is equal to zero.
       * </p>
       * @param layoutOrientation the layout orientation
       * @returns the Y-offset of the {@link } with respect to the given layout orientation
       */
      getYOffsetForLayoutOrientation(layoutOrientation:yfiles.layout.LayoutOrientation):number;
      /**
       * Returns whether or not this {@link yfiles.layout.PortCandidate} lies in the given direction.
       * @param directionMask one of the default direction specifiers
       * @returns <code>true</code> if this {@link } lies in the given direction, <code>false</code> otherwise
       */
      isInDirection(directionMask:yfiles.layout.PortDirections):boolean;
      /**
       * A utility method that converts the current {@link yfiles.layout.PortCandidate} instance to a {@link yfiles.layout.PortConstraint}.
       * <p>
       * The {@link yfiles.layout.PortConstraint#side side} of the created {@link yfiles.layout.PortConstraint} matches the {@link yfiles.layout.PortCandidate#direction direction} of this {@link yfiles.layout.PortCandidate}.
       * </p>
       * <p>
       * If this {@link yfiles.layout.PortCandidate} is {@link yfiles.layout.PortCandidate#fixed fixed}, the resulting {@link yfiles.layout.PortConstraint} will be {@link yfiles.layout.PortConstraint#strong strong}.
       * </p>
       * @returns a {@link } that matches this {@link }
       */
      toPortConstraint():yfiles.layout.PortConstraint;
      /**
       * A utility method that converts the current {@link yfiles.layout.PortCandidate} instance to a {@link yfiles.layout.PortConstraint} with
       * respect to the given {@link yfiles.layout.OrientationLayout#orientation layout orientation}.
       * <p>
       * The {@link yfiles.layout.PortConstraint#side side} of the created {@link yfiles.layout.PortConstraint} is the {@link yfiles.layout.PortCandidate#direction direction} of this {@link yfiles.layout.PortCandidate} interpreted
       * for the given orientation.
       * </p>
       * <p>
       * If this {@link yfiles.layout.PortCandidate} is {@link yfiles.layout.PortCandidate#fixed fixed}, the resulting {@link yfiles.layout.PortConstraint} will be {@link yfiles.layout.PortConstraint#strong strong}.
       * </p>
       * @param layoutOrientation
       * @returns a {@link } that matches this {@link } with respect to the given orientation
       */
      toPortConstraintForLayoutOrientation(layoutOrientation:yfiles.layout.LayoutOrientation):yfiles.layout.PortConstraint;
      /**
       * Data provider key for specifying source port candidates for each edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      static SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Data provider key for specifying target port candidates for each edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      static TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets the direction of this {@link yfiles.layout.PortCandidate}.
       * <p>
       * If the direction is required to be calculated with respect to the actual {@link yfiles.layout.OrientationLayout#orientation layout orientation}, {@link yfiles.layout.PortCandidate#getDirectionForLayoutOrientation}
       * should be used instead.
       * </p>
       * @see yfiles.layout.PortCandidate#getDirectionForLayoutOrientation
       * @type {yfiles.layout.PortDirections}
       */
      direction:yfiles.layout.PortDirections;
      /**
       * Gets whether or not this {@link yfiles.layout.PortCandidate} is fixed.
       * <p>
       * Fixed
       * {@link yfiles.layout.PortCandidate}s are defined using exact coordinates, whereas coordinates of free candidates are selected
       * arbitrarily.
       * </p>
       * @type {boolean}
       */
      fixed:boolean;
      /**
       * Gets the X-offset of the {@link yfiles.layout.PortCandidate} with respect to the center of the corresponding node.
       * <p>
       * The X-offset encodes the horizontal distance of the port relative to the center of a node. If the {@link yfiles.layout.PortCandidate}
       * is not {@link yfiles.layout.PortCandidate#fixed fixed} (i.e., free), the X-offset is zero.
       * </p>
       * @type {number}
       */
      xOffset:number;
      /**
       * Gets the Y-offset of the {@link yfiles.layout.PortCandidate} with respect to the center of the corresponding node.
       * <p>
       * The Y-offset encodes the vertical distance of the port relative to the center of a node. If the {@link yfiles.layout.PortCandidate}
       * is not {@link yfiles.layout.PortCandidate#fixed fixed} (i.e., free), the Y-offset is zero.
       * </p>
       * @type {number}
       */
      yOffset:number;
      /**
       * Gets the penalty cost for using this {@link yfiles.layout.PortCandidate} instance.
       * @type {number}
       */
      cost:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.IPortCandidateMatcher} retrieves the best matching {@link yfiles.layout.IPortCandidateSetEntry entries} from a {@link yfiles.layout.PortCandidateSet}.
     * <p>
     * How well a candidate is matching can be decided based on its cost, its currently available capacity, its offsets or
     * whether or not its direction is consistent with the desired one.
     * </p>
     * <p>
     * To use custom implementations of this interface, method {@link yfiles.layout.PortCandidateSet#createMatcher} should be
     * overridden.
     * </p>
     * @interface
     */
    export interface IPortCandidateMatcher extends Object{
      /**
       * Returns the next best matching {@link yfiles.layout.IPortCandidateSetEntry} for the given parameters.
       * @param edge the edge for which to find a candidate
       * @param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
       * @param xOffset the preferred X-offset of the port relative to the center of the node
       * @param yOffset the preferred Y-offset of the port relative to the center of the node
       * @param directionMask any combination of default directions defined in {@link }
       * @returns a matching {@link } instance
       * @abstract
       */
      findMatchingCandidate(edge:yfiles.algorithms.Edge,source:boolean,xOffset:number,yOffset:number,directionMask:yfiles.layout.PortDirections):yfiles.layout.IPortCandidateSetEntry;
      /**
       * Returns the next best matching {@link yfiles.layout.IPortCandidateSetEntry} for the given parameters.
       * @param edge the edge for which to find a candidate
       * @param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
       * @param directionMask any combination of default directions defined in {@link }
       * @returns a matching {@link } instance
       * @abstract
       */
      findMatchingCandidate(edge:yfiles.algorithms.Edge,source:boolean,directionMask:yfiles.layout.PortDirections):yfiles.layout.IPortCandidateSetEntry;
      /**
       * Returns the next best matching {@link yfiles.layout.IPortCandidateSetEntry}.
       * @returns a matching {@link } instance
       * @abstract
       */
      findMatchingCandidate():yfiles.layout.IPortCandidateSetEntry;
    }
    var IPortCandidateMatcher:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface represents an entry of a {@link yfiles.layout.PortCandidateSet} and provides access to the actual {@link yfiles.layout.PortCandidate}
     * instance and the maximum number of edges allowed to connect to it.
     * @interface
     */
    export interface IPortCandidateSetEntry extends Object{
      /**
       * Gets the actual {@link yfiles.layout.PortCandidate} associated with this {@link yfiles.layout.IPortCandidateSetEntry}.
       * @abstract
       * @type {yfiles.layout.PortCandidate}
       */
      portCandidate:yfiles.layout.PortCandidate;
      /**
       * Gets the maximum number of edges allowed to connect to the {@link yfiles.layout.PortCandidate} associated with this
       * {@link yfiles.layout.IPortCandidateSetEntry}.
       * <p>
       * <code>0x7FFFFFFF</code> indicates that the number of connections is unbounded.
       * </p>
       * @abstract
       * @type {number}
       */
      connections:number;
    }
    var IPortCandidateSetEntry:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class associates {@link yfiles.layout.PortCandidate}s with the nodes of a graph.
     * <p>
     * {@link yfiles.layout.PortCandidate}s defined on nodes specify sides or even exact locations at which edges can be connected.
     * This means that an edge can be connected to any of the available node ports (unless edge {@link yfiles.layout.PortCandidate}s
     * are specified too). Each node {@link yfiles.layout.PortCandidate} has a capacity, i.e., a maximum number of edges allowed to
     * connect to it.
     * </p>
     * <p>
     * Information about {@link yfiles.layout.PortCandidate}s on nodes is held by a {@link yfiles.algorithms.IDataProvider} that stores a {@link yfiles.layout.PortCandidateSet}
     * for each node and is registered with the graph using key {@link yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY}.
     * </p>
     * <p>
     * The {@link yfiles.layout.PortCandidate}s are retrieved from the {@link yfiles.layout.PortCandidateSet} through a {@link yfiles.layout.IPortCandidateSetEntry}
     * that holds the information about the actual {@link yfiles.layout.PortCandidate}. This object is also used for retrieving the
     * best candidate from a {@link yfiles.layout.PortCandidateSet} based on the cost (lower costs are considered first) and the
     * available capacity of each candidate. Custom searching for the best {@link yfiles.layout.PortCandidate} can be achieved by
     * implementing {@link yfiles.layout.IPortCandidateMatcher} interface.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[5,5],"d":1},"styles":{"6":{"f":"#008000","b":1, "c":1}, "7":{"f":"#00ffff","b":1}, "8":{"f":"#ff0000","b":1}},"n":[[-54,-100,60,30],[25,-47,60,30],[131,-105.5,60,30],[-31,30,60,30],[111,30.5,60,30],[63.81,-49.5,8],[40.81,-49.5,8],[23.5,-46,3,8,7],[23.5,-26,3,8,7],[83.5,-46,3,8,7],[83.5,-26,3,8,7,6],[40.81,-19.5,8],[63.81,-19.5,8],[-32.5,-73,6],[-2.5,28,6],[132.5,28,6],[147.5,-77.5,6]],"e":[[0,1,-6,15,-30,-10,[-30,-42]],[2,1,-11,15,30,-10,[150,-42]],[3,1,1,-15,-30,10,[0,-22]],[4,1,-6,-15,30,10,[135,-22]]],"vp":[-54.0,-106.0,245.0,167.0]}} Example of node port candidates. Circular ports represent node port candidates on source nodes of each edge. Square and
     * rectangular ports are possible port candidates on the target node. Rectangular ports have zero cost of usage and unit
     * capacity whereas square rectangles have unit cost and capacity.
     * </p>
     * <p>
     * Not all layout and routing algorithms are able to handle {@link yfiles.layout.PortCandidate}s. Major layout algorithms that obey
     * them are {@link yfiles.hierarchic.HierarchicLayout}, {@link yfiles.router.EdgeRouter}, {@link yfiles.router.ChannelEdgeRouter} and
     * {@link yfiles.router.BusRouter}.
     * </p>
     * @class
     */
    export interface PortCandidateSet extends Object{}
    export class PortCandidateSet {
      /**
       * Creates a new empty {@link yfiles.layout.PortCandidateSet} instance.
       * <p>
       * Addition of {@link yfiles.layout.PortCandidate}s to this {@link yfiles.layout.PortCandidateSet set} can be performed using {@link yfiles.layout.PortCandidateSet#add}.
       * </p>
       */
      constructor();
      /**
       * Adds a {@link yfiles.layout.PortCandidate} with the given capacity to this {@link yfiles.layout.PortCandidateSet}.
       * <p>
       * The capacity value defines the maximum number of edges allowed to connect to the candidate. If no limitation for this
       * number exists, <code>0x7FFFFFFF</code> can be used.
       * </p>
       * <p>
       * The capacity must be greater than zero.
       * </p>
       * @param p the {@link } to add
       * @param [connections=1] the capacity of the given port
       * @throws {Stubs.Exceptions.ArgumentError} if the given capacity is less than or equal to zero
       */
      add(p:yfiles.layout.PortCandidate,connections?:number):void;
      /**
       * Creates a {@link yfiles.layout.IPortCandidateMatcher} that retrieves {@link yfiles.layout.IPortCandidateSetEntry entries} from this {@link yfiles.layout.PortCandidateSet}.
       * <p>
       * Each next best matching candidate is determined based on its cost, its current available capacity and based on whether
       * or not its direction is consistent with the desired one.
       * </p>
       * <p>
       * This method may be overridden to support custom implementations of {@link yfiles.layout.IPortCandidateMatcher}.
       * </p>
       * @returns a new {@link } instance
       */
      createMatcher():yfiles.layout.IPortCandidateMatcher;
      /**
       * Calculates the penalty cost for the usage of the given {@link yfiles.layout.IPortCandidateSetEntry entry} and the given parameters.
       * <p>
       * The cost is equal to the {@link yfiles.layout.PortCandidate#cost cost of usage} of the {@link yfiles.layout.IPortCandidateSetEntry#portCandidate associated port candidate}.
       * </p>
       * <p>
       * This method may be overridden to customize the calculation of the cost of usage of an {@link yfiles.layout.IPortCandidateSetEntry entry}.
       * </p>
       * @param entry the entry for which the cost is computed
       * @param edge the edge that will connect to the {@link } associated with the entry
       * @param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
       * @param xOffset the preferred X-offset of the port relative to the center of the node
       * @param yOffset the preferred Y-offset of the port relative to the center of the node
       * @returns the penalty cost for the usage of the given {@link }
       */
      getCost(entry:yfiles.layout.IPortCandidateSetEntry,edge:yfiles.algorithms.Edge,source:boolean,xOffset:number,yOffset:number):number;
      /**
       * Calculates the priority of the given {@link yfiles.layout.IPortCandidateSetEntry entry}.
       * <p>
       * Higher priority means that the entry will be considered earlier by a {@link yfiles.layout.IPortCandidateMatcher}.
       * </p>
       * <p>
       * This method may be overridden to customize the calculation of the priority of {@link yfiles.layout.IPortCandidateSetEntry entries}.
       * </p>
       * @param entry the entry for which the cost is computed
       * @param edge the edge that will connect to the {@link } associated with the entry
       * @param source <code>true</code> if the given edge connects to its source node, <code>false</code> otherwise
       * @returns the priority of the given {@link }
       */
      getPriority(entry:yfiles.layout.IPortCandidateSetEntry,edge:yfiles.algorithms.Edge,source:boolean):number;
      /**
       * Removes the given {@link yfiles.layout.IPortCandidateSetEntry entry} from the list of candidate entries.
       * @param entry the entry to remove
       */
      remove(entry:yfiles.layout.IPortCandidateSetEntry):void;
      /**
       * Data provider key for specifying a port candidate set for each node
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.layout.PortCandidateSet>}
       */
      static NODE_PORT_CANDIDATE_SET_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.layout.PortCandidateSet>;
      /**
       * Gets the total number of connections allowed by the {@link yfiles.layout.PortCandidate}s described by this {@link yfiles.layout.PortCandidateSet}
       * or <code>0x7FFFFFFF</code> if the number is unbounded.
       * @type {number}
       */
      connectionCount:number;
      /**
       * Gets an {@link yfiles.collections.IEnumerator.<T>} over the list of candidate {@link yfiles.layout.IPortCandidateSetEntry entries} of this {@link yfiles.layout.PortCandidateSet}.
       * @type {yfiles.collections.IEnumerator.<Object>}
       */
      entries:yfiles.collections.IEnumerator<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.PortConstraint} represents a constraint for the source or target port of an edge path.
     * <p>
     * It determines the location at which an edge path can be connected to its source and/or target node. There exist two
     * different types of {@link yfiles.layout.PortConstraint}s:
     * </p>
     * <ul>
     * <li><b>Weak</b> {@link yfiles.layout.PortConstraint}s determine the side of the node to which an edge can be connected.</li>
     * <li>
     * <b>Strong</b>
     * {@link yfiles.layout.PortConstraint}s determine the exact coordinates of the point to which an edge path can be connected. These
     * coordinates are determined at the time the edge is processed by a layout or routing algorithm and are always interpreted
     * relative to a node's center.
     * </li>
     * </ul>
     * <p>
     * {@graph {"ann":{"s":[3,8],"d":1},"styles":{"6":{"f":"#FF0000"},"7":{"f":"#000000"}},"n":[[-570,-40,90,30],[-395,-140,90,30],[-563.02,-184.16,90,30],[-398.2,-140,9,30,6],[-477,-184.16,9,30,6],[-484.4,-40,9,30,6],[-474,-173.16,7],[-395.2,-135.01,7],[-395.2,-122.49,7],[-481.4,-29,7]],"e":[[0,1,45,0,-45,6,[-463.02,-25,-463.02,-119]],[2,1,45,0,-45,-6,[-424.34,-169.16,-424.34,-131]]],"vp":[-570.0,-185.0,265.0,175.0]}} Sample output of <em>weak</em> port constraints. For both edges, source port constraints are associated with side {@link yfiles.layout.PortSide#EAST}
     * while target port constraints are associated with side {@link yfiles.layout.PortSide#WEST}. Red rectangles indicate that edges
     * can be connected to any point of these sides. Black rectangles represent the selected ports. {@graph {"ann":{"s":[90,30],"d":1},"styles":{"6":{"f":"#FF0000"},"7":{"f":"#000000"}},"n":[[-145,135.67],[30,35.67],[-138.02,-8.49],[-49.82,1.51,4,10,7],[28,45.67,4,10,7],[-57.36,145.67,4,10,7]],"e":[[0,1,[-35,150.67,-35,50.67]],[2,1,[-0.22,6.51,-0.22,50.67]]],"vp":[-145.0,-9.0,265.0,175.0]}} Sample output of <em>strong</em> port
     * constraints. For both edges, source port constraints are associated with side {@link yfiles.layout.PortSide#EAST} while target
     * port constraints are associated with side
     * {@link yfiles.layout.PortSide#WEST}. Black rectangles indicate the exact points on the corresponding sides of the nodes to which
     * edges can be connected.
     * </p>
     * <p>
     * Information about {@link yfiles.layout.PortConstraint}s is held by {@link yfiles.algorithms.IDataProvider}s that are registered with the
     * graph using keys {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY} (to specify
     * {@link yfiles.layout.PortConstraint}s for an edge on its source node) and {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
     * (to specify {@link yfiles.layout.PortConstraint}s for an edge on its target node).
     * </p>
     * <p>
     * Not all layout and routing algorithms are able to handle {@link yfiles.layout.PortConstraint}s. Major layout algorithms that
     * obey them are {@link yfiles.hierarchic.HierarchicLayout}, {@link yfiles.tree.TreeLayout}, {@link yfiles.router.EdgeRouter}, {@link yfiles.router.ChannelEdgeRouter}
     * and {@link yfiles.router.BusRouter}.
     * </p>
     * @see yfiles.layout.PortConstraintKeys
     * @class
     */
    export interface PortConstraint extends Object{}
    export class PortConstraint {
      /**
       * Creates a new {@link yfiles.layout.PortConstraint} that constrains the edge to connect to the given side of a node and that can
       * be either strong or weak.
       * <p>
       * If the {@link yfiles.layout.PortConstraint} is strong, the layout or routing algorithm should use the port coordinates of this
       * constraint.
       * </p>
       * @param side one of the default side specifiers
       * @param [strong=false] <code>true</code> if this {@link } is strong, <code>false</code> otherwise
       * @returns a new {@link } instance
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown side is given
       * @static
       */
      static create(side:yfiles.layout.PortSide,strong?:boolean):yfiles.layout.PortConstraint;
      /**
       * Returns the source {@link yfiles.layout.PortConstraint} associated with the given edge.
       * <p>
       * To retrieve the {@link yfiles.algorithms.IDataProvider} that holds information about source {@link yfiles.layout.PortConstraint}s, key {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}
       * is used.
       * </p>
       * @param graph the input graph
       * @param e the edge
       * @returns the source {@link } of the given edge or <code>null</code> if there is none
       * @static
       */
      static getSPC(graph:yfiles.layout.LayoutGraph,e:yfiles.algorithms.Edge):yfiles.layout.PortConstraint;
      /**
       * Returns the target {@link yfiles.layout.PortConstraint} associated with the given edge.
       * <p>
       * To retrieve the {@link yfiles.algorithms.IDataProvider} that holds information about target {@link yfiles.layout.PortConstraint}s, key {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
       * is used.
       * </p>
       * @param graph the input graph
       * @param e the edge
       * @returns the target {@link } of the given edge or <code>null</code> if there is none
       * @static
       */
      static getTPC(graph:yfiles.layout.LayoutGraph,e:yfiles.algorithms.Edge):yfiles.layout.PortConstraint;
      /**
       * Checks whether or not there exist source or target {@link yfiles.layout.PortConstraint}s associated with the given edge.
       * <p>
       * To retrieve the {@link yfiles.algorithms.IDataProvider}s that hold information about source or target
       * {@link yfiles.layout.PortConstraint}s, keys {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY} and {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
       * are used.
       * </p>
       * @param graph the input graph
       * @param e the edge
       * @returns <code>true</code> if the edge has source or target {@link }s, <code>false</code> otherwise
       * @static
       */
      static hasPC(graph:yfiles.layout.LayoutGraph,e:yfiles.algorithms.Edge):boolean;
      /**
       * Checks whether or not there exist source {@link yfiles.layout.PortConstraint}s associated with the given edge.
       * <p>
       * To retrieve the {@link yfiles.algorithms.IDataProvider} that holds information about source {@link yfiles.layout.PortConstraint}s, key {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}
       * is used.
       * </p>
       * @param graph the input graph
       * @param e the edge
       * @returns <code>true</code> if the edge has source {@link }s, <code>false</code> otherwise
       * @static
       */
      static hasSPC(graph:yfiles.layout.LayoutGraph,e:yfiles.algorithms.Edge):boolean;
      /**
       * Checks whether or not there exist target {@link yfiles.layout.PortConstraint}s associated with the given edge.
       * <p>
       * To retrieve the {@link yfiles.algorithms.IDataProvider} that holds information about target {@link yfiles.layout.PortConstraint}s, key {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}
       * is used.
       * </p>
       * @param graph the input graph
       * @param e the edge
       * @returns <code>true</code> if the edge has target {@link }s, <code>false</code> otherwise
       * @static
       */
      static hasTPC(graph:yfiles.layout.LayoutGraph,e:yfiles.algorithms.Edge):boolean;
      /**
       * Gets whether or not this {@link yfiles.layout.PortConstraint} instance represents a strong constraint.
       * <p>
       * A strong {@link yfiles.layout.PortConstraint} determines the exact coordinates where the edge path's end should be located.
       * </p>
       * @type {boolean}
       */
      strong:boolean;
      /**
       * Gets the side of the node to which an edge should connect.
       * @type {yfiles.layout.PortSide}
       */
      side:yfiles.layout.PortSide;
      /**
       * Gets whether or not this {@link yfiles.layout.PortConstraint} instance is associated with side {@link yfiles.layout.PortSide#NORTH}.
       * @type {boolean}
       */
      atNorth:boolean;
      /**
       * Gets whether or not this {@link yfiles.layout.PortConstraint} instance is associated with side {@link yfiles.layout.PortSide#SOUTH}.
       * @type {boolean}
       */
      atSouth:boolean;
      /**
       * Gets whether or not this {@link yfiles.layout.PortConstraint} instance is associated with side {@link yfiles.layout.PortSide#EAST}.
       * @type {boolean}
       */
      atEast:boolean;
      /**
       * Gets whether or not this {@link yfiles.layout.PortConstraint} instance is associated with side {@link yfiles.layout.PortSide#WEST}.
       * @type {boolean}
       */
      atWest:boolean;
      /**
       * Gets whether or not this {@link yfiles.layout.PortConstraint} instance is associated with side {@link yfiles.layout.PortSide#ANY}.
       * @type {boolean}
       */
      atAnySide:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.layout.PortConstraintConfigurator} provides the possibility to create
     * {@link yfiles.layout.PortConstraint}s according to the initial connecting side of an edge.
     * @class
     */
    export interface PortConstraintConfigurator extends Object{}
    export class PortConstraintConfigurator {
      /**
       * Creates a new {@link yfiles.layout.PortConstraintConfigurator} instance.
       */
      constructor();
      /**
       * Creates a {@link yfiles.layout.PortConstraint} for the given edge.
       * <p>
       * The side of the constraint is set up according to the current drawing of the edge within the given graph.
       * </p>
       * @param graph the input graph
       * @param edge the edge
       * @param source <code>true</code> if a source {@link } for the given edge will be created, <code>false</code> for a target
       * {@link }
       * @param strong <code>true</code> if a strong {@link } is created, <code>false</code> for a weak {@link }
       * @returns the {@link } for the given edge whose side is retrieved from the initial drawing
       */
      createPortConstraintFromSketch(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge,source:boolean,strong:boolean):yfiles.layout.PortConstraint;
      /**
       * Creates {@link yfiles.layout.PortConstraint}s for some edges of the given graph.
       * @param graph the input graph
       * @param ec the {@link } that provides access to the edges for which to create {@link }s
       * @param spcMap the {@link } that will be used for storing the source {@link }s of the edges
       * @param strongSP <code>true</code> if strong source {@link }s should be assigned, <code>false</code> otherwise
       * @param tpcMap the {@link } that will be used for storing the target {@link }s of the edges
       * @param strongTP <code>true</code> if strong target {@link }s should be assigned, <code>false</code> otherwise
       */
      createPortConstraintsFromSketch(graph:yfiles.layout.LayoutGraph,ec:yfiles.algorithms.IEdgeCursor,spcMap:yfiles.algorithms.IEdgeMap,strongSP:boolean,tpcMap:yfiles.algorithms.IEdgeMap,strongTP:boolean):void;
      /**
       * Creates weak {@link yfiles.layout.PortConstraint}s for all edges of the given graph.
       * @param graph the input graph
       * @param spcMap the {@link } that will be used for storing the source {@link }s of the edges
       * @param tpcMap the {@link } that will be used for storing the target {@link }s of the edges
       */
      createPortConstraintsFromSketch(graph:yfiles.layout.LayoutGraph,spcMap:yfiles.algorithms.IEdgeMap,tpcMap:yfiles.algorithms.IEdgeMap):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.PortConstraintKeys} are used for associating {@link yfiles.algorithms.IDataProvider DataProviders} that contain constraints for the edges of the
     * input graph.
     * <p>
     * These constraints can either be {@link yfiles.layout.PortConstraint}s or edge groups.
     * {@link yfiles.layout.PortConstraint}s force the edges to start/end on specified sides of the node. They may also restrict ports
     * of edges to certain coordinates. Edge groups will bundle edges at a common source or target. They will start at the same
     * coordinates and may share parts of their edge routes.
     * </p>
     * <p>
     * Layout algorithms that support {@link yfiles.layout.PortConstraint}s or edge groups will access the
     * {@link yfiles.algorithms.IDataProvider}s using the provided keys.
     * </p>
     * @class
     * @static
     */
    export interface PortConstraintKeys extends Object{}
    export class PortConstraintKeys {
      /**
       * Data provider key for providing source constraints for edges in the input graph
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.layout.PortConstraint>}
       */
      static SOURCE_PORT_CONSTRAINT_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.layout.PortConstraint>;
      /**
       * Data provider key for providing target constraints for edges in the input graph
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.layout.PortConstraint>}
       */
      static TARGET_PORT_CONSTRAINT_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.layout.PortConstraint>;
      /**
       * Data provider key for assigning source group information for edges of the input graph
       * <p>
       * Edges sharing a source group identifier will share a common bus near the source or at a common source node if possible.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static SOURCE_GROUP_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data provider key for assigning target group information for edges of the input graph
       * <p>
       * Edges sharing a target group identifier will share a common bus near the target or at a common target node if possible.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static TARGET_GROUP_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data provider key for assigning port group ids to the edges' source
       * <p>
       * All edges with the same port id at a node will share the same port location. However, they will be routed
       * independently.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static SOURCE_PORT_GROUP_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data provider key for assigning port group ids to the edges' target
       * <p>
       * All edges with the same port id at a node will share the same port location. However, they will be routed
       * independently.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static TARGET_PORT_GROUP_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.PreferredPlacementDescriptor} provides placement information for automatic edge labeling.
     * <p>
     * It provides several options that influence the label placement in generic labeling algorithms and layout algorithms that
     * support integrated label placement.
     * </p>
     * <p>
     * The preferred placement can describe multiple locations. For some options, like the {@link yfiles.layout.PreferredPlacementDescriptor#placeAlongEdge placement along the edge}, it is
     * possible to have a combination of the valid values. These values can be connected with a logical <code>or</code>-operation.
     * </p>
     * <p>
     * The following options describe the preferred placement:
     * </p>
     * <ul>
     * <li>the placement along the edge</li>
     * <li>the side of the edge</li>
     * <li>the reference of the side</li>
     * <li>the rotation angle of the label</li>
     * <li>the reference for the rotation angle</li>
     * <li>the rotation behavior for label on the right side of the edge</li>
     * <li>the angle offset for label on the right side of the edge</li>
     * <li>the distance between the label and its edge segment</li>
     * </ul>
     * <p>
     * The {@link yfiles.layout.PreferredPlacementDescriptor} will be immutable after {@link yfiles.layout.PreferredPlacementDescriptor#freeze}
     * is called.
     * </p>
     * <p>
     * A {@link yfiles.layout.PreferredPlacementDescriptor} must be set for each label and will be retrieved by the layout algorithms
     * calling {@link yfiles.layout.IEdgeLabelLayout#preferredPlacementDescriptor}.
     * </p>
     * @class
     */
    export interface PreferredPlacementDescriptor extends Object{}
    export class PreferredPlacementDescriptor {
      /**
       * Creates a new {@link yfiles.layout.PreferredPlacementDescriptor} instance that copies the given descriptor.
       * @param descriptor the {@link } that is copied
       */
      constructor(descriptor:yfiles.layout.PreferredPlacementDescriptor);
      /**
       * Creates a new {@link yfiles.layout.PreferredPlacementDescriptor} instance with default settings.
       */
      constructor();
      /**
       * Clones this instance by returning a {@link Object#memberwiseClone} or this in case this instance is already {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen frozen}.
       * @returns An instance of the same type as this instance.
       */
      clone():Object;
      /**
       * Makes this {@link yfiles.layout.PreferredPlacementDescriptor} instance immutable.
       * <p>
       * If this instance is immutable, all <code>setter</code> methods will throw an {@link yfiles.lang.Exception} when invoked.
       * </p>
       * @see yfiles.layout.PreferredPlacementDescriptor#isFrozen
       */
      freeze():void;
      /**
       * Returns whether or not this is an immutable instance of {@link yfiles.layout.PreferredPlacementDescriptor}.
       * <p>
       * If this instance is immutable, all <code>setter</code> methods will throw an {@link yfiles.lang.Exception} when invoked.
       * </p>
       * @returns <code>true</code> if this instance is immutable, <code>false</code> otherwise
       * @see yfiles.layout.PreferredPlacementDescriptor#freeze
       */
      isFrozen():boolean;
      /**
       * Returns an immutable descriptor instance for the specified placement.
       * <p>
       * Descriptor instances returned by this method will use default values for all properties but {@link yfiles.layout.PreferredPlacementDescriptor#placeAlongEdge placement along edge} and
       * {@link yfiles.layout.PreferredPlacementDescriptor#sideOfEdge side of edge}.
       * </p>
       * @param placement a combination of valid placements along the edge and valid sides of the edge
       * @returns an immutable descriptor instance for the specified placement
       * @see yfiles.layout.PreferredPlacementDescriptor#isFrozen
       * @see yfiles.layout.PreferredPlacementDescriptor#placeAlongEdge
       * @see yfiles.layout.PreferredPlacementDescriptor#sideOfEdge
       * @static
       */
      static newSharedInstance(placement:yfiles.layout.LabelPlacements):yfiles.layout.PreferredPlacementDescriptor;
      /**
       * Gets or sets the preferred placement along the edge.
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @type {yfiles.layout.LabelPlacements}
       */
      placeAlongEdge:yfiles.layout.LabelPlacements;
      /**
       * Gets or sets the preferred side of the edge.
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @type {yfiles.layout.LabelPlacements}
       */
      sideOfEdge:yfiles.layout.LabelPlacements;
      /**
       * Gets or sets how to interpret the preferred side as given by {@link yfiles.layout.PreferredPlacementDescriptor#sideOfEdge}.
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @throws {Stubs.Exceptions.ArgumentError} if the specified side reference is invalid
       * @type {yfiles.layout.LabelSideReferences}
       */
      sideReference:yfiles.layout.LabelSideReferences;
      /**
       * Gets or sets the angle (in radians) for the label rotation.
       * <p>
       * How this angle is applied depends on the {@link yfiles.layout.PreferredPlacementDescriptor#angleReference angle reference}, the {@link yfiles.layout.PreferredPlacementDescriptor#angleRotationOnRightSide rotation} and the {@link yfiles.layout.PreferredPlacementDescriptor#angleOffsetOnRightSide offset} for labels on the right side of
       * the edge.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @see yfiles.layout.PreferredPlacementDescriptor#angleReference
       * @see yfiles.layout.PreferredPlacementDescriptor#angleRotationOnRightSide
       * @see yfiles.layout.PreferredPlacementDescriptor#angleOffsetOnRightSide
       * @type {number}
       */
      angle:number;
      /**
       * Gets or sets the reference of the angle given by {@link yfiles.layout.PreferredPlacementDescriptor#angle}.
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is not a valid angle reference
       * @type {yfiles.layout.LabelAngleReferences}
       */
      angleReference:yfiles.layout.LabelAngleReferences;
      /**
       * Gets or sets how the angle is applied to labels on the right side in respect of the labels of the left side of the
       * edge.
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is not a valid angle rotation
       * @type {yfiles.layout.LabelAngleOnRightSideRotations}
       */
      angleRotationOnRightSide:yfiles.layout.LabelAngleOnRightSideRotations;
      /**
       * Gets or sets the angle offset for labels that are placed on the right side of the edge.
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is not valid angle offset
       * @type {yfiles.layout.LabelAngleOnRightSideOffsets}
       */
      angleOffsetOnRightSide:yfiles.layout.LabelAngleOnRightSideOffsets;
      /**
       * Gets or sets the preferred distance between a label and the corresponding edge segment.
       * <p>
       * If the given distance is <code>&lt; 0</code>, the distance is not fixed, i.e., it is chosen by the automatic labeling algorithm.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if this instance is {@link yfiles.layout.PreferredPlacementDescriptor#isFrozen immutable}
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is infinite or <code>NaN</code>
       * @type {number}
       */
      distanceToEdge:number;
      /**
       * Gets whether or not one of the preferred placement locations for this label is at source.
       * @see yfiles.layout.LabelPlacements#AT_SOURCE
       * @type {boolean}
       */
      isAtSource:boolean;
      /**
       * Gets whether or not one of the preferred placement locations for this label is at center.
       * @see yfiles.layout.LabelPlacements#AT_CENTER
       * @type {boolean}
       */
      isAtCenter:boolean;
      /**
       * Gets whether or not one of the preferred placements for this label is at target.
       * @see yfiles.layout.LabelPlacements#AT_TARGET
       * @type {boolean}
       */
      isAtTarget:boolean;
      /**
       * Gets whether or not one of the preferred placements for this label is at the source port.
       * @see yfiles.layout.LabelPlacements#AT_SOURCE_PORT
       * @type {boolean}
       */
      isAtSourcePort:boolean;
      /**
       * Gets whether or not one of the preferred placements for this label is at the target port.
       * @see yfiles.layout.LabelPlacements#AT_TARGET_PORT
       * @type {boolean}
       */
      isAtTargetPort:boolean;
      /**
       * Gets whether or not one of the preferred sides for this label is left of the edge.
       * @see yfiles.layout.LabelPlacements#LEFT_OF_EDGE
       * @type {boolean}
       */
      isLeftOfEdge:boolean;
      /**
       * Gets whether or not one of the preferred sides for this label is on the edge.
       * @see yfiles.layout.LabelPlacements#ON_EDGE
       * @type {boolean}
       */
      isOnEdge:boolean;
      /**
       * Gets whether or not one of the preferred sides for this label is right of the edge.
       * @see yfiles.layout.LabelPlacements#RIGHT_OF_EDGE
       * @type {boolean}
       */
      isRightOfEdge:boolean;
      /**
       * Gets whether or not one of the angle interpretations is absolute.
       * @see yfiles.layout.LabelAngleReferences#ABSOLUTE
       * @type {boolean}
       */
      isAngleAbsolute:boolean;
      /**
       * Gets whether or not one of the angle interpretation is relative to the edge slope.
       * @see yfiles.layout.LabelAngleReferences#RELATIVE_TO_EDGE_FLOW
       * @type {boolean}
       */
      isAngleRelativeToEdgeFlow:boolean;
      /**
       * Gets whether or not one of the angle interpretations is to rotate the angle of labels that are right of the edge
       * co-rotating with the labels left of or centered on the edge, i.e., the angle of all labels is interpreted clockwise.
       * @see yfiles.layout.PreferredPlacementDescriptor#isRightOfEdge
       * @see yfiles.layout.LabelAngleOnRightSideRotations#CLOCKWISE
       * @type {boolean}
       */
      isAngleOnRightSideCoRotating:boolean;
      /**
       * Gets whether or not one of the angle interpretations is to rotate the angle of labels that are right of the edge
       * counter-rotating with the labels left of or centered on the edge, i.e., the angle of labels left or centered on edge is
       * interpreted clockwise while the angle of labels right of edge is interpreted counter-clockwise.
       * @see yfiles.layout.PreferredPlacementDescriptor#isRightOfEdge
       * @see yfiles.layout.LabelAngleOnRightSideRotations#COUNTER_CLOCKWISE
       * @type {boolean}
       */
      isAngleOnRightSideCounterRotating:boolean;
      /**
       * Gets whether or not one of the angle interpretations is to add no angle offset to labels that are right of the edge.
       * @see yfiles.layout.PreferredPlacementDescriptor#isRightOfEdge
       * @see yfiles.layout.LabelAngleOnRightSideOffsets#NONE
       * @type {boolean}
       */
      isAngleOffsetOnRightSideZero:boolean;
      /**
       * Gets whether or not one of the angle interpretations is to add an angle offset of <code>180</code> degrees to labels that are right
       * of the edge.
       * @see yfiles.layout.PreferredPlacementDescriptor#isRightOfEdge
       * @see yfiles.layout.LabelAngleOnRightSideOffsets#SEMI
       * @type {boolean}
       */
      isAngleOffsetOnRightSide180:boolean;
      /**
       * Gets whether or not the preferred side specified by {@link yfiles.layout.PreferredPlacementDescriptor#sideOfEdge} is interpreted relative to the edge flow.
       * @see yfiles.layout.LabelSideReferences#RELATIVE_TO_EDGE_FLOW
       * @type {boolean}
       */
      isSideRelativeToEdgeFlow:boolean;
      /**
       * Gets whether or not the preferred side specified by {@link yfiles.layout.PreferredPlacementDescriptor#sideOfEdge} is interpreted absolute, and on horizontal segments, labels with
       * preferred side {@link yfiles.layout.LabelPlacements#LEFT_OF_EDGE} are placed above the segment.
       * @see yfiles.layout.LabelSideReferences#ABSOLUTE_WITH_LEFT_IN_NORTH
       * @type {boolean}
       */
      isSideAbsoluteWithLeftInNorth:boolean;
      /**
       * Gets whether or not the preferred side specified by {@link yfiles.layout.PreferredPlacementDescriptor#sideOfEdge} is interpreted absolute, and on horizontal segments, labels with
       * preferred side {@link yfiles.layout.LabelPlacements#RIGHT_OF_EDGE} are placed above the segment.
       * @see yfiles.layout.LabelSideReferences#ABSOLUTE_WITH_RIGHT_IN_NORTH
       * @type {boolean}
       */
      isSideAbsoluteWithRightInNorth:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.layout.RemoveCollinearBendsStage} removes all collinear bends found in a graph.
     * <p>
     * The algorithm traverses each edge path from source to target and removes for each triple of consecutive collinear bends
     * the middle one.
     * </p>
     * <p>
     * A collinear bend forms a straight line with its preceding bend and its subsequent bend. The angle between the connecting
     * edge segments is either <code>0</code> or <code>180</code> degrees.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface RemoveCollinearBendsStage extends yfiles.layout.LayoutStageBase{}
    export class RemoveCollinearBendsStage {
      /**
       * Creates a new {@link yfiles.layout.RemoveCollinearBendsStage} instance with default settings.
       */
      constructor();
      /**
       * Gets or sets the scale that defines how accurate the algorithm is when it comes to comparing the coordinates of two
       * bends.
       * <p>
       * This value determines how the values of the coordinates are rounded. If rounded coordinates lead to a collinear bend,
       * this bend will be removed.
       * </p>
       * <p>
       * The scaling factor needs to be positive.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified factor is less than or equal to <code>0</code>
       * @type {number}
       */
      scale:number;
      /**
       * Gets or sets whether the algorithm removes only collinear bends which lie in the middle of their neighbors or all
       * collinear bends.
       * <p>
       * In the first case, the angle at the middle bend must be <code>180</code> degrees, while in the second case, the angle can be either <code>180</code>
       * degrees or <code>0</code> degrees.
       * </p>
       * @type {boolean}
       */
      removeStraightOnly:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.ReverseEdgesStage} selectively reverses edges in a graph, while keeping the layout and label positions of
     * the reversed edges as close to the pre-reversal layout and positions as possible.
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface ReverseEdgesStage extends yfiles.layout.LayoutStageBase{}
    export class ReverseEdgesStage {
      /**
       * Creates a new {@link yfiles.layout.ReverseEdgesStage} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       * @see yfiles.layout.LayoutStageBase#coreLayout
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Marks edges in a tree-structured graph component that need to be reversed to make the specified node the root of the
       * tree component.
       * <p>
       * Only edges in the graph component to which the specified node belongs will be marked.
       * </p>
       * @param graph the graph to which the specified node belongs
       * @param root the node to be considered the root of its tree component
       * @param reversedEdges a {@link } to mark edges for reversal
       * @throws {Stubs.Exceptions.ArgumentError} if the specified root node is <code>null</code> or does not belong to the specified graph
       * @see yfiles.layout.ReverseEdgesStage#AFFECTED_EDGES_DP_KEY
       * @static
       */
      static findReversedTreeEdges(graph:yfiles.layout.LayoutGraph,root:yfiles.algorithms.Node,reversedEdges:yfiles.algorithms.IDataAcceptor):void;
      /**
       * Reverses the specified edge in the specified graph.
       * <p>
       * The layout of the specified edge is reversed as specified in
       * {@link yfiles.layout.LayoutGraphUtilities#reverseEdgeLayout}. Moreover, the positions of all labels associated with the
       * specified edge are adjusted to match the pre-reversal position as closely as possible. (Depending on a label's model it
       * might not always be possible to achieve an exact match.)
       * </p>
       * @param graph the graph to which the specified edge belongs
       * @param edge the edge to be reversed
       * @see yfiles.layout.LayoutGraphUtilities#reverseEdgeLayout
       */
      reverseEdge(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):void;
      /**
       * Reverses <em>selected</em> edges in the specified graph.
       * <p>
       * <em>Selected</em> edges are marked in a {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.layout.ReverseEdgesStage#AFFECTED_EDGES_DP_KEY}.
       * </p>
       * <p>
       * Implementation detail: This method calls {@link yfiles.layout.ReverseEdgesStage#reverseEdge} in order to reverse a <em>selected</em> edge.
       * </p>
       * @param graph the graph in which to reverse edges.
       * @see yfiles.layout.ReverseEdgesStage#AFFECTED_EDGES_DP_KEY
       * @see yfiles.layout.ReverseEdgesStage#reverseEdge
       */
      reverseEdges(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Data provider key for selecting edges for reversal
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.SelfLoopRouter} routes the self-loops (reflexive edges) of a graph.
     * <p>
     * <b>Layout Style</b> The nodes and normal edges are arranged by the
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. This layout algorithm only handles the paths of self-loops.
     * </p>
     * <p>
     * Self-loops are routed either {@link yfiles.layout.SelfLoopStyle#ORTHOGONAL orthogonal} or with {@link yfiles.layout.SelfLoopStyle#ROUNDED rounded} corners. The layout algorithm places the self-loops in the least
     * crowded quadrant around a node.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[100,100],[100,0],[0,100],[0,0]],"e":[[0,0,7.5,15,15,7.5,[122.5,145,145,145,145,122.5]],[0,1,5,-15,5,15],[1,1,10,-15,15,-10,[125,-15,145,-15,145,5]],[1,1,5,-15,15,-5,[120,-20,150,-20,150,10]],[1,0,-5,15,-5,-15],[0,2,-15,5,15,5],[2,2,-10,15,-15,10,[5,145,-15,145,-15,125]],[2,2,-5,15,-15,5,[10,150,-20,150,-20,120]],[2,0,15,-5,-15,-5],[1,3,-15,5,15,5],[2,3,5,-15,5,15],[3,3,-7.5,-15,-15,-7.5,[7.5,-15,-15,-15,-15,7.5]],[3,1,15,-5,-15,-5],[3,2,-5,15,-5,-15]],"vp":[-20.0,-20.0,170.0,170.0]}}
     * </p>
     * <p>
     * Small graph with self-loops
     * </p>
     * <p>
     * <b>Concept</b> Self-loops are handled in four steps:
     * </p>
     * <ol>
     * <li>Remove all self-loops of the input graph</li>
     * <li>Invoke the core layout algorithm on the now self-loops free graph</li>
     * <li>Reinsert all formerly removed self-loops</li>
     * <li>Route the self-loops of the input graph</li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * {@link yfiles.layout.SelfLoopRouter} can either be used as a {@link yfiles.layout.ILayoutStage} wrapping a {@link yfiles.layout.ILayoutAlgorithm layout algorithm} which cannot
     * handle self-loops. Then it will hide the self-loops from this {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} and take over the routing of them.
     * <br />
     * If no {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} is specified, {@link yfiles.layout.SelfLoopRouter} can work alone. It will route only the self-loops
     * and keep the remaining graph unchanged.
     * </p>
     * <p>
     * When all self-loops already have a suitable layout, this layout algorithm can {@link yfiles.layout.SelfLoopRouter#keepAllSelfLoopRoutes keep the self-loop paths}. It will just
     * hide the self-loops before invoking the core layout algorithm, such that they are not altered by it.
     * <br />
     * To only exclude some self-loops from routing, a {@link yfiles.algorithms.IDataProvider} registered with {@link yfiles.layout.SelfLoopRouter#KEEP_SELF_LOOP_LAYOUT_DP_KEY}
     * can be used for marking those self-loops.
     * </p>
     * @see yfiles.layout.MultiStageLayout
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface SelfLoopRouter extends yfiles.layout.LayoutStageBase{}
    export class SelfLoopRouter {
      /**
       * Creates a new {@link yfiles.layout.SelfLoopRouter} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Calculates the routes of the self-loops in the given graph.
       * <p>
       * This method will only change the routes of self-loops that are not marked to be {@link yfiles.layout.SelfLoopRouter#KEEP_SELF_LOOP_LAYOUT_DP_KEY kept}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.layout.SelfLoopRouter#applyLayout} after the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} has arranged the graph
       * and all self-loops have been reinserted into the graph. It may be overridden to introduce custom self-loop styles.
       * </p>
       * @param graph the input graph
       * @protected
       */
      layoutSelfLoops(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Data provider key for determining which self-loops should keep their current path
       * <p>
       * If no {@link yfiles.algorithms.IDataProvider} is registered with this key, all self-loops will be routed.
       * </p>
       * @see yfiles.layout.SelfLoopRouter#keepAllSelfLoopRoutes
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static KEEP_SELF_LOOP_LAYOUT_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the number of points that are used for rounding the corners of the self-loops.
       * <p>
       * For rounded corners, the number of points should be at least
       * <code>2</code>. All smaller values will produce one bend and, therefore, an orthogonal path.
       * </p>
       * <p>
       * The number of points for a corner must be at least <code>1</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified point count is less than <code>1</code>
       * @see yfiles.layout.SelfLoopRouter#layoutStyle
       * @see yfiles.layout.SelfLoopStyle#ROUNDED
       * @type {number}
       */
      cornerApproximationPointsCount:number;
      /**
       * Gets or sets the distance between two self-loops incident to the same node.
       * <p>
       * The distance needs to be a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @see yfiles.layout.SelfLoopStyle#ROUNDED
       * @type {number}
       */
      lineDistance:number;
      /**
       * Gets or sets whether or not the algorithm chooses the side of the self-loop depending on the incident edges.
       * <p>
       * If this option is enabled, self-loops are placed on one of the four corners of the corresponding node. For the choice of
       * the corner, the algorithm considers all incident edges. Otherwise, self-loops are always placed at the upper left corner
       * of the corresponding node.
       * </p>
       * @type {boolean}
       */
      smartSelfLoopPlacement:boolean;
      /**
       * Gets or sets the layout style of the self-loops.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified style is unknown
       * @type {yfiles.layout.SelfLoopStyle}
       */
      layoutStyle:yfiles.layout.SelfLoopStyle;
      /**
       * Gets or sets whether or not this algorithm should keep all self-loop routes.
       * <p>
       * When enabled, no self-loop will be routed, instead the existing self-loop layout will be moved with its node.
       * </p>
       * @type {boolean}
       */
      keepAllSelfLoopRoutes:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.SequentialLayout} allows for chaining multiple {@link yfiles.layout.ILayoutAlgorithm layout algorithms}.
     * <p>
     * The {@link yfiles.layout.ILayoutAlgorithm layout algorithms} are invoked one after the other in the order in which they were
     * {@link yfiles.layout.SequentialLayout#appendLayout added to the chain}. They all run independently of each other, i.e., they only influence each other by the changed input
     * graph. Hence, the sequence determines the result.
     * </p>
     * <p>
     * One possible application would be to bundle several {@link yfiles.layout.ILayoutAlgorithm layout algorithms} and pass a single {@link yfiles.layout.ILayoutAlgorithm}
     * instance to a {@link yfiles.layout.BufferedLayout}.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutAlgorithm}
     */
    export interface SequentialLayout extends Object,yfiles.layout.ILayoutAlgorithm{}
    export class SequentialLayout {
      /**
       * Creates a new {@link yfiles.layout.SequentialLayout} instance with an empty chain of {@link yfiles.layout.ILayoutAlgorithm layout algorithms}.
       */
      constructor();
      /**
       * Appends the given {@link yfiles.layout.ILayoutAlgorithm layout algorithm} to the end of the chain.
       * @param layouter the {@link  layout algorithm} to append
       */
      appendLayout(layouter:yfiles.layout.ILayoutAlgorithm):void;
      /**
       * Appends the specified list of {@link yfiles.layout.ILayoutAlgorithm layout algorithms} to the end of the chain.
       * @param layouters the list of {@link  layout algorithms} to append
       */
      appendLayouts(layouters:yfiles.collections.ICollection<yfiles.layout.ILayoutAlgorithm>):void;
      /**
       * Executes all {@link yfiles.layout.ILayoutAlgorithm layout algorithms} in the chain, one after the other.
       * <p>
       * The order of the layout algorithms is defined by the order in which they were appended.
       * </p>
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Clears the current chain of {@link yfiles.layout.ILayoutAlgorithm layout algorithms} and therefore removes all layout algorithms.
       */
      removeAll():void;
      /**
       * Gets or sets the chain of {@link yfiles.layout.ILayoutAlgorithm layout algorithms}.
       * @see yfiles.layout.SequentialLayout#appendLayout
       * @see yfiles.layout.SequentialLayout#appendLayouts
       * @type {yfiles.collections.IList.<yfiles.layout.ILayoutAlgorithm>}
       */
      layouts:yfiles.collections.IList<yfiles.layout.ILayoutAlgorithm>;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.SimpleProfitModel} implements the ranking for edge and node labels.
     * <p>
     * In this implementation node label positions have no ranking, the profit for all candidates is <code>1</code>.
     * </p>
     * <p>
     * The ranking of edge label positions depends on the locations described in the {@link yfiles.layout.PreferredPlacementDescriptor}
     * associated with an edge.
     * </p>
     * @class
     * @implements {yfiles.layout.IProfitModel}
     */
    export interface SimpleProfitModel extends Object,yfiles.layout.IProfitModel{}
    export class SimpleProfitModel {
      /**
       * Creates a new instance of {@link yfiles.layout.SimpleProfitModel}.
       */
      constructor();
      /**
       * 
       * @param candidate
       * @returns 
       */
      getProfit(candidate:yfiles.layout.LabelCandidate):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.SliderEdgeLabelLayoutModel} allows placement of labels at a set of continuous rotated positions along both
     * sides of an edge or directly on the edge path.
     * <p>
     * It is possible to specify distance values that control the distance between label and edge and between label and nodes.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1,"lblB":1},"n":[[14,10],[14,310],[184,10],[184,310],[354,10],[354,310]],"e":[[0,1,[],[[10.59,58.22,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,281.41,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,253.51,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,225.61,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,197.72,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,169.82,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,141.92,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,114.02,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[10.59,86.12,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"]]],[2,3,[],[[157.19,58.22,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,281.41,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,253.51,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,281.41,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,253.51,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,225.61,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,197.72,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,169.82,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,114.02,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,86.12,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,141.92,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[204,58.22,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,225.61,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,197.72,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,169.82,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,114.02,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,141.92,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[157.19,86.12,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"]]],[4,5,[],[[327.19,58.22,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,281.41,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,253.51,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,225.61,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,197.72,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,141.92,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,169.82,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,114.02,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"],[327.19,86.12,33.36,18.7,0.7071067811865477,-0.7071067811865474,"Label"]]]],"vp":[10.0,10.0,374.0,330.0]}}
     * </p>
     * <p>
     * {@link yfiles.layout.SliderMode#CENTER} mode on the left, {@link yfiles.layout.SliderMode#SIDE} mode in the middle and {@link yfiles.layout.SliderMode#SINGLE_SIDE}
     * on the right
     * </p>
     * @class
     * @implements {yfiles.layout.IEdgeLabelLayoutModel}
     */
    export interface SliderEdgeLabelLayoutModel extends Object,yfiles.layout.IEdgeLabelLayoutModel{}
    export class SliderEdgeLabelLayoutModel {
      /**
       * Creates a new instance of {@link yfiles.layout.SliderEdgeLabelLayoutModel} with the given mode.
       * @param mode the slider mode
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#mode
       */
      constructor(mode:yfiles.layout.SliderMode);
      /**
       * 
       * @param labelBounds
       * @param edgeLayout
       * @param sourceNode
       * @param targetNode
       * @returns 
       */
      createModelParameter(labelBounds:yfiles.algorithms.YOrientedRectangle,edgeLayout:yfiles.layout.IEdgeLayout,sourceNode:yfiles.layout.INodeLayout,targetNode:yfiles.layout.INodeLayout):Object;
      /**
       * Returns all {@link yfiles.layout.EdgeLabelCandidate}s that describe valid label positions within this model.
       * <p>
       * The number of candidates and their respective locations are computed depending on the geometries of both label and
       * edge.
       * </p>
       * @param labelLayout the label for which candidates should be generated
       * @param edgeLayout the layout of the edge to which the label belongs
       * @param sourceLayout the layout of the source node of the label-owning edge
       * @param targetLayout the layout of the target node of the label-owning edge
       * @returns a list of {@link } instances
       */
      getLabelCandidates(labelLayout:yfiles.layout.IEdgeLabelLayout,edgeLayout:yfiles.layout.IEdgeLayout,sourceLayout:yfiles.layout.INodeLayout,targetLayout:yfiles.layout.INodeLayout):yfiles.algorithms.YList;
      /**
       * 
       * @param labelSize
       * @param edgeLayout
       * @param sourceNode
       * @param targetNode
       * @param para
       * @returns 
       */
      getLabelPlacement(labelSize:yfiles.algorithms.YDimension,edgeLayout:yfiles.layout.IEdgeLayout,sourceNode:yfiles.layout.INodeLayout,targetNode:yfiles.layout.INodeLayout,para:Object):yfiles.algorithms.YOrientedRectangle;
      /**
       * Gets the model's slider mode which determines whether the label slides {@link yfiles.layout.SliderMode#CENTER on the edge}, {@link yfiles.layout.SliderMode#SIDE along both sides of the edge} or
       * {@link yfiles.layout.SliderMode#SINGLE_SIDE along one side of the edge}.
       * @type {yfiles.layout.SliderMode}
       */
      mode:yfiles.layout.SliderMode;
      /**
       * Gets or sets whether or not the distance to the edge is interpreted relative to the edge's path.
       * <p>
       * If enabled, the label is placed to the left of the edge segment (relative to the segment direction) if the given
       * distance is less than <code>0</code> and to the right of the edge segment if the given distance is greater than <code>0</code>.
       * </p>
       * <p>
       * If disabled, the label is placed below the edge segment (in geometric sense) if the distance is less than <code>0</code> and above
       * the edge segment if the distance is greater than <code>0</code>.
       * </p>
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#distance
       * @type {boolean}
       */
      distanceRelativeToEdge:boolean;
      /**
       * Gets or sets the distance between the label's box and the edge's path.
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#distanceRelativeToEdge
       * @type {number}
       */
      distance:number;
      /**
       * Gets or sets whether or not edge labels are automatically rotated according to the angle of the corresponding reference
       * edge segment.
       * @type {boolean}
       */
      autoRotationEnabled:boolean;
      /**
       * Gets or sets whether or not edge labels get flipped if they would be upside down in their current position.
       * @type {boolean}
       */
      autoFlipping:boolean;
      /**
       * Gets or sets the angle (measured in radians) of the label model.
       * <p>
       * The angle is applied in clockwise direction.
       * </p>
       * @type {number}
       */
      angle:number;
      /**
       * Gets a model parameter that encodes the default position of this model's allowed edge label positions.
       * <p>
       * This parameter can be passed to {@link yfiles.layout.SliderEdgeLabelLayoutModel#getLabelPlacement} to retrieve the corresponding
       * label box.
       * </p>
       * <p>
       * The default positions for the {@link yfiles.layout.SliderEdgeLabelLayoutModel#mode slider modes} are relative to the first edge segment, either at the beginning of the
       * segment ({@link yfiles.layout.SliderMode#SIDE} or
       * {@link yfiles.layout.SliderMode#SINGLE_SIDE}) or at its middle ({@link yfiles.layout.SliderMode#CENTER}).
       * </p>
       * @see yfiles.layout.SliderEdgeLabelLayoutModel#mode
       * @type {Object}
       */
      defaultParameter:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.SubgraphLayout} is a {@link yfiles.layout.ILayoutStage} that filters out all graph elements that are not part of
     * a specific subgraph.
     * <p>
     * The nodes that induce this subgraph are marked using a {@link yfiles.algorithms.IDataProvider} registered with a {@link yfiles.layout.SubgraphLayout#affectedNodesDpKey specific key}.
     * They will be arranged by the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface SubgraphLayout extends yfiles.layout.LayoutStageBase{}
    export class SubgraphLayout {
      /**
       * Creates a new {@link yfiles.layout.SubgraphLayout} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} which marks the nodes that induce the subgraph to be
       * arranged.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @type {Object}
       */
      affectedNodesDpKey:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.SwimlaneRepresentative SwimlaneRepresentative} represents the position and arrangement mode for a swimlane.
     * @class
     */
    export interface SwimlaneRepresentative extends Object{}
    export class SwimlaneRepresentative {
      /**
       * Creates a new {@link yfiles.layout.SwimlaneRepresentative} instance.
       * @param swimlanePos denotes the current position of the represented swimlane
       * @param allowRearrangement <code>true</code> if the represented swimlane may be {@link #allowRearrangement rearranged}, <code>false</code> otherwise
       */
      constructor(swimlanePos:number,allowRearrangement:boolean);
      /**
       * Gets or sets the current position of the represented swimlane.
       * <p>
       * The swimlane positions are described by a zero-based index and must not be negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the position is smaller than <code>0</code>
       * @type {number}
       */
      swimlanePos:number;
      /**
       * Gets or sets whether or not the represented swimlane can be rearranged.
       * <p>
       * For all swimlanes with disabled rearrangement, the relative ordering given by {@link yfiles.layout.SwimlaneRepresentative#swimlanePos} is preserved. The other swimlanes may be
       * rearranged.
       * </p>
       * @type {boolean}
       */
      allowRearrangement:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.layout.Swimlanes} provides helper classes and methods to handle swimlanes.
     * <p>
     * It is used by {@link yfiles.hierarchic.HierarchicLayout} to arrange the nodes in lanes.
     * </p>
     * @class
     * @static
     */
    export interface Swimlanes extends Object{}
    export class Swimlanes {
      /**
       * Calculates an ordering of the swimlanes considering the specified ordering mode.
       * <p>
       * Depending on the swimlane mode, the ordering will try to reduce:
       * </p>
       * <ol>
       * <li>the number of swimlanes traversed by edges</li>
       * <li>the number of swimlanes spanned by {@link yfiles.layout.GroupingSupport#isGroupNode group nodes}</li>
       * </ol>
       * @param graph the input graph
       * @param node2Swimlane the assignment of nodes to swimlanes
       * @param iterations the number of iterations performed by the heuristic.
       * @param mode one of the valid swimlane ordering mode specifier
       * @see yfiles.layout.SwimlaneRepresentative
       * @see yfiles.layout.GroupingSupport
       * @static
       */
      static arrangeSwimlanes(graph:yfiles.algorithms.Graph,node2Swimlane:yfiles.algorithms.IDataProvider,iterations:number,mode:yfiles.layout.SwimlanesMode):void;
      /**
       * Calculates an ordering of the swimlanes.
       * <p>
       * The ordering will try to reduce:
       * </p>
       * <ol>
       * <li>the number of swimlanes traversed by edges</li>
       * <li>the number of swimlanes spanned by {@link yfiles.layout.GroupingSupport#isGroupNode group nodes}</li>
       * </ol>
       * @param graph the input graph
       * @param node2Swimlane the assignment of nodes to swimlanes
       * @see yfiles.layout.SwimlaneRepresentative
       * @see yfiles.layout.GroupingSupport
       * @static
       */
      static arrangeSwimlanes(graph:yfiles.algorithms.Graph,node2Swimlane:yfiles.algorithms.IDataProvider):void;
      static $class:yfiles.lang.Class;
    }
    export enum OperationType{
      /**
       * Operation specifier defining that the graph will be mirrored at the x-axis.
       * @see yfiles.layout.GraphTransformer#operation
       */
      MIRROR_X_AXIS,
      /**
       * Operation specifier defining that the graph will be mirrored at the y-axis.
       * @see yfiles.layout.GraphTransformer#operation
       */
      MIRROR_Y_AXIS,
      /**
       * Operation specifier defining that the graph will be rotated.
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.GraphTransformer#rotationAngle
       * @see yfiles.layout.GraphTransformer#bestFitRotationEnabled
       */
      ROTATE,
      /**
       * Operation specifier defining that the graph will be scaled.
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.GraphTransformer#scaleFactor
       * @see yfiles.layout.GraphTransformer#setScaleFactors
       * @see yfiles.layout.GraphTransformer#scaleNodeSize
       */
      SCALE,
      /**
       * Operation specifier defining that the graph will be translated.
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.GraphTransformer#translateX
       * @see yfiles.layout.GraphTransformer#translateY
       */
      TRANSLATE
    }
    /**
     * This layout algorithm applies geometric transformations to (sub-)graphs.
     * <p>
     * <b>Layout Style</b> There are several ways to transform the graph that include mirroring, rotating, scaling and translating. {@graph {"ann":{"s":[30,30],"d":1},"n":[[-15,235],[-15,360],[-15,110],[235,485],[110,485],[360,485],[-15,485],[485,235],[485,360],[235,-15],[110,-15],[360,-15],[485,110],[-15,-15],[485,485],[485,-15],[235,235,1],[146.61,146.61,1],[146.61,323.39,1],[58.22,235,1],[323.39,323.39,1],[235,411.78,1],[323.39,146.61,1],[235,58.22,1],[411.78,235,1]],"e":[[1,0],[0,2],[4,3],[3,5],[1,6],[6,4],[7,8],[9,10],[11,9],[12,7],[2,13],[13,10],[8,14],[14,5],[11,15],[12,15],[16,17],[16,18],[19,17],[19,18],[20,16],[18,21],[21,20],[22,16],[23,22],[23,17],[20,24],[22,24],[17,0],[19,1],[23,2],[18,3],[4,19],[5,21],[20,7],[21,8],[9,22],[10,23],[24,11],[24,12]],"vp":[-15.0,-15.0,530.0,530.0]}}
     * </p>
     * <p>
     * The highlighted subgraph is rotated by <code>45</code> degrees
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The layout can be {@link yfiles.layout.OperationType#SCALE scaled} with independent factors for the horizontal and vertical scaling. Optionally, the sizes of
     * nodes can be scaled, too.
     * </p>
     * <p>
     * The layout can be mirrored either on the {@link yfiles.layout.OperationType#MIRROR_X_AXIS x-axis} or the {@link yfiles.layout.OperationType#MIRROR_Y_AXIS y-axis}.
     * </p>
     * <p>
     * A {@link yfiles.layout.OperationType#ROTATE rotation} can be specified. Instead of {@link yfiles.layout.GraphTransformer#rotationAngle defining} a rotation angle the angle can be {@link yfiles.layout.GraphTransformer#bestFitRotationEnabled determined automatically} to fit
     * specific bounds.
     * </p>
     * <p>
     * The coordinates in the layout can be {@link yfiles.layout.OperationType#TRANSLATE translated} by a specific vector.
     * </p>
     * <p>
     * To only apply previously mentioned transformations to a subgraph, the {@link yfiles.layout.SubgraphLayout} needs to be {@link yfiles.layout.MultiStageLayout#subgraphLayoutEnabled enabled}.
     * Then the nodes in the subgraph must be marked in a {@link yfiles.algorithms.IDataProvider} registered with the according {@link yfiles.layout.SubgraphLayout#affectedNodesDpKey key}.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface GraphTransformer extends yfiles.layout.MultiStageLayout,yfiles.layout.ILayoutStage{}
    export class GraphTransformer {
      /**
       * Creates a new instance of {@link yfiles.layout.GraphTransformer} with default settings.
       */
      constructor();
      /**
       * Rotates the given graph so its resulting bounding box fits best to the given bounds.
       * @param graph the graph that is rotated
       * @param width the width of the preferred layout bounds
       * @param height the height of the preferred layout bounds
       * @returns the rotation angle
       * @see yfiles.layout.GraphTransformer#bestFitRotationEnabled
       * @static
       */
      static applyBestFitRotationAngle(graph:yfiles.layout.LayoutGraph,width:number,height:number):number;
      /**
       * Returns the rotation angle for the graph that will result in a layout that fits best to the given bounds.
       * @param graph the graph that is rotated
       * @param width the width of the preferred layout bounds
       * @param height the height of the preferred layout bounds
       * @returns the rotation angle
       * @see yfiles.layout.GraphTransformer#applyBestFitRotationAngle
       * @static
       */
      static findBestFitRotationAngle(graph:yfiles.layout.LayoutGraph,width:number,height:number):number;
      /**
       * Specifies the preferred size of the layout.
       * <p>
       * If performing a {@link yfiles.layout.OperationType#ROTATE rotation} and this rotation should be
       * {@link yfiles.layout.GraphTransformer#bestFitRotationEnabled automatically fitted}, then the given preferred size defines the aspect ratio that will be used for fitting the layout
       * rotation.
       * </p>
       * <p>
       * The preferred width and height need to be non-negative.
       * </p>
       * @param width the preferred width of the layout
       * @param height the preferred height of the layout
       * @throws {Stubs.Exceptions.ArgumentError} if the specified width or height is negative
       * @see yfiles.layout.GraphTransformer#preferredWidth
       * @see yfiles.layout.GraphTransformer#preferredHeight
       * @see yfiles.layout.GraphTransformer#bestFitRotationEnabled
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#ROTATE
       */
      preferredLayoutSize(width:number,height:number):void;
      /**
       * Fits the given graph into the specified rectangular bounds.
       * <p>
       * If the bounding box fits in the given bounds, nothing is done. Otherwise, the graph is scaled and optionally moved to
       * fit.
       * </p>
       * @param graph the graph to modify
       * @param x the upper left x-coordinate of the bounds
       * @param y the upper left y-coordinate of the bounds
       * @param w the width of the bounds
       * @param h the height of the bounds
       * @static
       */
      static setMaximalBounds(graph:yfiles.layout.LayoutGraph,x:number,y:number,w:number,h:number):void;
      /**
       * Specifies the horizontal and vertical scaling factor used for the {@link yfiles.layout.OperationType#SCALE scale operation}.
       * <p>
       * Only positive scaling factors are allowed.
       * </p>
       * @param xFactor the horizontal scaling factor
       * @param yFactor the vertical scaling factor
       * @throws {Stubs.Exceptions.ArgumentError} if one the specified scaling factors is negative or <code>0</code>
       * @see yfiles.layout.GraphTransformer#scaleFactorX
       * @see yfiles.layout.GraphTransformer#scaleFactorY
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#SCALE
       */
      setScaleFactors(xFactor:number,yFactor:number):void;
      /**
       * Translates the layout coordinates by the given vector.
       * @param graph the graph that is translated
       * @param dx the horizontal distance the given graph is moved
       * @param dy the vertical distance the given graph is moved
       * @see yfiles.layout.OperationType#TRANSLATE
       * @see yfiles.layout.GraphTransformer#operation
       * @static
       */
      static translate(graph:yfiles.layout.LayoutGraph,dx:number,dy:number):void;
      /**
       * 
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the transformation operation.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified operation is unknown
       * @type {yfiles.layout.OperationType}
       */
      operation:yfiles.layout.OperationType;
      /**
       * Gets or sets the angle of rotation.
       * <p>
       * The angle must be given in degrees.
       * </p>
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#ROTATE
       * @type {number}
       */
      rotationAngle:number;
      /**
       * Gets or sets whether or not to automatically choose the best fitting rotation angle.
       * <p>
       * The angle will be chosen such that the bounding box after the rotation fits best to the {@link yfiles.layout.GraphTransformer#preferredLayoutSize preferred aspect ratio}.
       * </p>
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#ROTATE
       * @type {boolean}
       */
      bestFitRotationEnabled:boolean;
      /**
       * Gets or sets the preferred height of the layout.
       * <p>
       * This value will be used for {@link yfiles.layout.GraphTransformer#bestFitRotationEnabled fitting the rotation} to preferred bounds.
       * </p>
       * <p>
       * The preferred height needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified height is negative
       * @see yfiles.layout.GraphTransformer#preferredWidth
       * @see yfiles.layout.GraphTransformer#bestFitRotationEnabled
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#ROTATE
       * @type {number}
       */
      preferredHeight:number;
      /**
       * Gets or sets the preferred width of the layout.
       * <p>
       * This value will be used for {@link yfiles.layout.GraphTransformer#bestFitRotationEnabled fitting the rotation} to preferred bounds.
       * </p>
       * <p>
       * The preferred width needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified width is negative
       * @see yfiles.layout.GraphTransformer#preferredHeight
       * @see yfiles.layout.GraphTransformer#bestFitRotationEnabled
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#ROTATE
       * @type {number}
       */
      preferredWidth:number;
      /**
       * Gets or sets whether or not to also scale node sizes when performing the {@link yfiles.layout.OperationType#SCALE scale operation}.
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#SCALE
       * @type {boolean}
       */
      scaleNodeSize:boolean;
      /**
       * Sets the uniform scaling factor used for the {@link yfiles.layout.OperationType#SCALE scale operation}.
       * <p>
       * This factor applies vertical and horizontal.
       * </p>
       * <p>
       * Only positive scaling factors are allowed.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified scaling factor is negative or <code>0</code>
       * @see yfiles.layout.GraphTransformer#scaleFactorX
       * @see yfiles.layout.GraphTransformer#scaleFactorY
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#SCALE
       * @type {number}
       */
      scaleFactor:number;
      /**
       * Gets or sets the vertical scaling factor used for the {@link yfiles.layout.OperationType#SCALE scale operation}.
       * <p>
       * Only positive scaling factors are allowed.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified scaling factor is negative or <code>0</code>
       * @see yfiles.layout.GraphTransformer#scaleFactorX
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#SCALE
       * @type {number}
       */
      scaleFactorY:number;
      /**
       * Gets or sets the horizontal scaling factor used for the {@link yfiles.layout.OperationType#SCALE scale operation}.
       * <p>
       * Only positive scaling factors are allowed.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified scaling factor is negative or <code>0</code>
       * @see yfiles.layout.GraphTransformer#scaleFactorY
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#SCALE
       * @type {number}
       */
      scaleFactorX:number;
      /**
       * Gets or sets the horizontal translation distance.
       * <p>
       * A positive value means the graph is moved to the right while, a negative value means the graph is moved to the left.
       * </p>
       * @see yfiles.layout.GraphTransformer#translateY
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#TRANSLATE
       * @type {number}
       */
      translateX:number;
      /**
       * Gets or sets the vertical translation distance.
       * <p>
       * A positive value means the graph is moved downwards, while a negative value means the graph is moved upwards.
       * </p>
       * @see yfiles.layout.GraphTransformer#translateX
       * @see yfiles.layout.GraphTransformer#operation
       * @see yfiles.layout.OperationType#TRANSLATE
       * @type {number}
       */
      translateY:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.ILayoutStage layout stage} that can handle fixed nodes in hierarchically grouped graphs.
     * <p>
     * This stage guarantees that a {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} will <b>maintain
     * the relative positions</b> of nodes within the same group.
     * </p>
     * <p>
     * Also, this stage is responsible for routing edges that connect nodes belonging to hierarchically unrelated groups. More
     * precisely, the stage temporarily replaces fixed group nodes (see
     * {@link yfiles.layout.FixGroupLayoutStage#FIXED_GROUP_NODES_DP_KEY}) with normal nodes of the same size while the content of the
     * groups is removed from the graph during layout.
     * </p>
     * <p>
     * The nodes are used as placeholders and are responsible for keeping the relations to the remaining graph elements. Edges
     * that originate from the outside of the group node and terminate in its interior (inter-edges) are redirected such that
     * they connect to the node that replaces the group.
     * </p>
     * <p>
     * After the layout process, the group nodes are restored and the stage inserts the content at the new group position
     * (derived from the position of the associated node). The content itself (including the edges between content nodes)
     * remains unchanged and is only relocated. Furthermore, the inter-edges are connected to their original source/target and
     * the stage reroutes them using the specified inter-edge routing algorithm (see {@link yfiles.layout.FixGroupLayoutStage#interEdgeRouter}).
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"styles":{"6":{"c":1}},"n":[[759.46,-86.88,290.54,283.26,2],[982.99,109.5,6],[796.47,109.5,6],[854.11,-67.88,6],[925.35,-67.88,6],[1005,41.75,6],[925.35,151.37,6],[796.47,-26.01,6],[774.46,41.75,6],[982.99,-26.01,6],[854.11,151.37,6],[123.31,-231.69,216.69,224.72,2],[212.57,-132.33,6],[263.97,-67.88,6],[263.97,-196.77,6],[138.31,-168.09,6],[194.23,-51.97,6],[295,-132.33,6],[138.31,-96.56,6],[194.23,-212.69,6],[265,40.5,430,339,2],[465,234.5,45,30],[380,59.5,45,30],[465,284.5,45,30],[380,109.5,45,30],[465,84.5,45,30],[465,134.5,45,30],[550,209.5,45,30],[380,309.5,45,30],[550,259.5,45,30],[380,259.5,45,30],[465,334.5,45,30],[550,134.5,45,30],[635,134.5,45,30],[465,184.5,45,30],[280,184.5,60,30]],"e":[[9,4],[3,7],[8,7],[4,3],[2,8],[10,2],[6,10],[1,6],[5,1],[2,3],[2,9],[2,1],[5,9],[12,18],[12,13],[12,14],[12,15],[12,19],[12,17],[12,16],[30,21],[21,29],[21,27],[35,22],[30,23],[35,24],[24,26],[24,25],[26,32],[35,28],[35,30],[30,31],[30,34],[32,33],[33,2],[33,8],[33,7],[35,17]],"vp":[123.0,-232.0,927.0,612.0]}} Initial graph. Nodes inside groups have to remain at their relative positions. {@graph {"ann":{"s":[30,30],"d":1},"styles":{"6":{"c":1}},"n":[[754.73,68.37,290.54,283.26,2],[978.26,264.75,6],[791.74,264.75,6],[849.38,87.37,6],[920.62,87.37,6],[1000.27,197,6],[920.62,306.63,6],[791.74,129.25,6],[769.73,197,6],[978.26,129.25,6],[849.38,306.63,6],[-18.35,97.64,216.69,224.72,2],[70.92,197,6],[122.31,261.44,6],[122.31,132.56,6],[-3.35,161.24,6],[52.58,277.36,6],[153.35,197,6],[-3.35,232.76,6],[52.58,116.64,6],[265,40.5,430,339,2],[465,234.5,45,30],[380,59.5,45,30],[465,284.5,45,30],[380,109.5,45,30],[465,84.5,45,30],[465,134.5,45,30],[550,209.5,45,30],[380,309.5,45,30],[550,259.5,45,30],[380,259.5,45,30],[465,334.5,45,30],[550,134.5,45,30],[635,134.5,45,30],[465,184.5,45,30],[280,184.5,60,30]],"e":[[9,4],[3,7],[8,7],[4,3],[2,8],[10,2],[6,10],[1,6],[5,1],[2,3],[2,9],[2,1],[5,9],[12,18],[12,13],[12,14],[12,15],[12,19],[12,17],[12,16],[30,21],[21,29],[21,27],[35,22],[30,23],[35,24],[24,26],[24,25],[26,32],[35,28],[35,30],[30,31],[30,34],[32,33],[33,8,22.5,6,-2.49,-14.78,[782.24,155.5]],[33,7,22.5,-6,-15,-0.75],[33,2,0,15,-15,0,[657.5,279.75]],[35,17,-30,6.25,13.66,-6.25]],"vp":[-19.0,40.0,1065.0,340.0]}} Example after using {@link yfiles.layout.FixGroupLayoutStage}
     * with {@link yfiles.orthogonal.OrthogonalLayout}. Nodes inside groups did not change relative positions.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface FixGroupLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class FixGroupLayoutStage {
      /**
       * Creates a new instance of {@link yfiles.layout.FixGroupLayoutStage} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The core layout algorithm.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Data provider key for selecting group nodes whose contents should keep their relative positions
       * <p>
       * A group node is considered <i>selected</i> in this context, if the corresponding {@link yfiles.algorithms.IDataProvider} returns <code>true</code> for the
       * group node or any of its ancestor (group) nodes.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static FIXED_GROUP_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets or sets the inter-edge routing style used by this layout stage.
       * @throws {Stubs.Exceptions.ArgumentError} if the given routing style is unknown
       * @type {yfiles.layout.InterEdgeRoutingStyle}
       */
      interEdgeRoutingStyle:yfiles.layout.InterEdgeRoutingStyle;
      /**
       * Gets or sets the edge routing algorithm that is applied to route the inter-edges.
       * <p>
       * If no routing algorithm is set, a new instance of {@link yfiles.layout.CompositeLayoutStage} associated with a {@link yfiles.layout.HideGroupsStage}
       * and an {@link yfiles.router.EdgeRouter} will be used.
       * </p>
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      interEdgeRouter:yfiles.layout.ILayoutAlgorithm;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.IPartitionFinder} is responsible for partitioning the input graph.
     * @interface
     */
    export interface IPartitionFinder extends Object{
      /**
       * Returns a partition ID for each node of an input graph.
       * <p>
       * Nodes that share a common partition ID are considered to belong to the same partition.
       * </p>
       * @param graph the input graph
       * @param partitionIDMap the map that is filled with the resulting partition IDs
       * @abstract
       */
      findPartitions(graph:yfiles.layout.LayoutGraph,partitionIDMap:yfiles.algorithms.INodeMap):void;
    }
    var IPartitionFinder:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.layout.IPartitionPlacer} is responsible for arranging the graph partitions.
     * <p>
     * Arranging a graph partition usually involves applying a coordinate translation to all elements that belong to the
     * partition in order to generate a non-overlapping node arrangement.
     * </p>
     * @interface
     */
    export interface IPartitionPlacer extends Object{
      /**
       * Arranges the graph while taking partition IDs and inter-edge information into account.
       * <p>
       * Edges which connect nodes that belong to two different partitions are called inter-edges
       * </p>
       * @param graph the input graph
       * @param partitionIDDP the partition ID for each node in the graph
       * @param interEdgeDP the map which marks inter-edges
       * @abstract
       */
      placePartitions(graph:yfiles.layout.LayoutGraph,partitionIDDP:yfiles.algorithms.IDataProvider,interEdgeDP:yfiles.algorithms.IDataProvider):void;
    }
    var IPartitionPlacer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * {@link yfiles.layout.PartitionLayout} provides a framework for layouts that are based on a divide and conquer approach.
     * <p>
     * <b>Concept</b> The approach can be subdivided into four stages:
     * </p>
     * <ol>
     * <li>The input graph is partitioned into smaller units. Implementations of this phase can be set using method {@link yfiles.layout.PartitionLayout#partitionFinder}.</li>
     * <li>Optionally, the layout algorithm arranges each partition. Implementations of this phase can be set using method {@link yfiles.layout.LayoutStageBase#coreLayout}.</li>
     * <li>
     * The graph partitions are arranged next to each other keeping their layout. Implementations of this phase can be set
     * using method {@link yfiles.layout.PartitionLayout#partitionPlacer}.
     * </li>
     * <li>
     * Edges that connect nodes in different graph partitions will be routed. Implementations of this phase can be set using
     * method {@link yfiles.layout.PartitionLayout#interEdgeRouter}
     * </li>
     * </ol>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface PartitionLayout extends yfiles.layout.LayoutStageBase{}
    export class PartitionLayout {
      /**
       * Creates a new {@link yfiles.layout.PartitionLayout} instance with default settings.
       */
      constructor();
      /**
       * Creates a {@link yfiles.layout.IPartitionPlacer} that uses {@link yfiles.layout.ComponentLayout} to place the partitions.
       * @param [componentLayout=null] An optional {@link } to use.
       * @returns 
       * @static
       */
      static createComponentPartitionPlacer(componentLayout?:yfiles.layout.ComponentLayout):yfiles.layout.IPartitionPlacer;
      /**
       * Creates a {@link yfiles.layout.IPartitionFinder} that uses {@link yfiles.algorithms.Groups#edgeBetweennessClustering} as strategy for
       * finding graph partitions.
       * @param [minimumPartitionCount=0] The minimum number of partitions the result should have.
       * @param [maximumPartitionCount=0x7FFFFFFF] The maximum number of partitions the result should have.
       * @returns 
       * @static
       */
      static createEdgeBetweennessPartitionFinder(minimumPartitionCount?:number,maximumPartitionCount?:number):yfiles.layout.IPartitionFinder;
      /**
       * Creates a {@link yfiles.layout.IPartitionFinder} that uses {@link yfiles.algorithms.Groups#edgeBetweennessClustering} as strategy for
       * finding graph partitions.
       * @param {Object} options The parameters to pass.
       * @param [options.minimumPartitionCount=0] The minimum number of partitions the result should have.
       * @param [options.maximumPartitionCount=0x7FFFFFFF] The maximum number of partitions the result should have.
       * @returns 
       * @static
       */
      static createEdgeBetweennessPartitionFinder(options:{minimumPartitionCount?:number,maximumPartitionCount?:number}):yfiles.layout.IPartitionFinder;
      /**
       * Gets or sets the {@link yfiles.layout.IPartitionFinder} instance that is responsible for partitioning the input graph.
       * @type {yfiles.layout.IPartitionFinder}
       */
      partitionFinder:yfiles.layout.IPartitionFinder;
      /**
       * Gets or sets the {@link yfiles.layout.IPartitionPlacer} instance that is responsible for arranging the graph partitions.
       * @type {yfiles.layout.IPartitionPlacer}
       */
      partitionPlacer:yfiles.layout.IPartitionPlacer;
      /**
       * Gets or sets the IInterEdgeRouter instance that is responsible for inter-edge routing.
       * @type {yfiles.layout.IPartitionInterEdgeRouter}
       */
      interEdgeRouter:yfiles.layout.IPartitionInterEdgeRouter;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.layout.PortPlacementStage} assigns edges to the ports specified by {@link yfiles.layout.PortConstraint}s or
     * {@link yfiles.layout.PortCandidate}s after calling the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * <p>
     * It can be used for adjusting ports for layout algorithms that cannot handle {@link yfiles.layout.PortConstraint}s.
     * </p>
     * <p>
     * If {@link yfiles.layout.PortCandidate}s are assigned for edges and nodes, this stage tries to match them. When there is no
     * match, the {@link yfiles.layout.PortCandidate} with the lowest costs specified for the edge is chosen. While
     * {@link yfiles.layout.PortConstraint}s are also matched to
     * {@link yfiles.layout.PortCandidate}s at nodes and edges, it is not recommended to mix them because they belong to different
     * concepts.
     * </p>
     * <p>
     * Furthermore, option {@link yfiles.layout.PortPlacementStage#pathCorrection} allows to specify whether the edge path should be corrected after correcting a port.
     * </p>
     * <p>
     * It is recommended that the {@link yfiles.layout.PortPlacementStage} is prepended to the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} in order to be
     * able to process all the edges and especially, self-loops and parallel edges. Note that edges that might be hidden by
     * other {@link yfiles.layout.ILayoutStage}s cannot be handled by {@link yfiles.layout.PortPlacementStage}.
     * </p>
     * @see yfiles.layout.PortConstraint
     * @see yfiles.layout.PortCandidate
     * @see yfiles.layout.PortCandidateSet
     * @see yfiles.layout.PortPlacementStage#pathCorrection
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface PortPlacementStage extends yfiles.layout.LayoutStageBase{}
    export class PortPlacementStage {
      /**
       * Creates a new {@link yfiles.layout.PortPlacementStage} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Gets or sets whether or not this {@link yfiles.layout.ILayoutStage} corrects the edge paths after moving the port to the
       * location specified by a {@link yfiles.layout.PortConstraint} or {@link yfiles.layout.PortCandidate}s.
       * <p>
       * If path correction is enabled, the edge will be rerouted within a specific area around its source/target node. Outside
       * this area, it will keep the route that was calculated by the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. If this feature is disabled, only the ports are moved which may be more suitable for initially
       * straight edges.
       * </p>
       * @type {boolean}
       */
      pathCorrection:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.layout.ComponentLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface ComponentLayoutData extends yfiles.layout.LayoutData{}
    export class ComponentLayoutData {
      constructor();
      /**
       * Gets or sets a collection of nodes that determine the components that shall be laid out.
       * <p>
       * Note that components will be laid out if and only if at least one of the nodes is in this collection.
       * </p>
       * @see yfiles.layout.ComponentLayout#AFFECTED_COMPONENTS_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      affectedComponents:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the mapping from nodes to their component comparable that is used to sort the components.
       * @see yfiles.layout.ComponentLayout#COMPONENT_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.lang.IComparable>}
       */
      componentIds:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.lang.IComparable>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
    /**
     * Combines several {@link yfiles.layout.LayoutData} instances.
     * <p>
     * This is especially useful if several layouts are run at once, for example a general layout and a subsequent edge
     * routing.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface CompositeLayoutData extends yfiles.layout.LayoutData{}
    export class CompositeLayoutData {
      constructor();
      /**
       * Gets or sets a collection of composed layout data objects.
       * @type {yfiles.collections.ICollection.<yfiles.layout.LayoutData>}
       */
      items:yfiles.collections.ICollection<yfiles.layout.LayoutData>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides different ways to define a mapping from items to values that takes a context object into consideration, too.
     * <p>
     * Only one of the provided ways to define a mapping can be used at the same time, therefore only one of the properties {@link yfiles.layout.ContextItemMapping.<TItem,TValue,TContext>#contextDelegate}, {@link yfiles.layout.ItemMapping.<TItem,TValue>#delegate},
     * {@link yfiles.layout.ItemMapping.<TItem,TValue>#mapper}, and {@link yfiles.layout.ItemMapping.<TItem,TValue>#constant} should be set on each instance.
     * </p>
     * <p>
     * This class supports {@link yfiles.layout.LayoutData} implementations and does not usually have to be used explicitly.
     * </p>
     * Type parameter <code>TItem</code>: The type of the items that are mapped.
     * Type parameter <code>TValue</code>: The type of the values the items are mapped to.
     * Type parameter <code>TContext</code>: The type of the context passed to the {@link yfiles.layout.ContextItemMapping.<TItem,TValue,TContext>#contextDelegate}
     * @class
     * @extends {yfiles.layout.ItemMapping.<TItem,TValue>}
     * @template TItem,TValue,TContext
     */
    export interface ContextItemMapping<TItem,TValue,TContext> extends yfiles.layout.ItemMapping<TItem,TValue>{}
    export class ContextItemMapping<TItem,TValue,TContext> {
      constructor();
      /**
       * Provides a mapper mapping the items to values.
       * @param adapter The graph adapter the mapper shall be provided for.
       * @param layout The layout using the mapper.
       * @param context The context object passed to the {@link #contextDelegate}.
       * @returns A mapper mapping the items to values.
       */
      provideMapperForContext(adapter:yfiles.layout.LayoutGraphAdapter,layout:yfiles.layout.ILayoutAlgorithm,context:TContext):yfiles.collections.IMapper<TItem,TValue>;
      /**
       * Gets or sets a delegate from the items and a context object to the values.
       * @type {function(TItem, TContext): TValue}
       */
      contextDelegate:(arg1:TItem,arg2:TContext)=>TValue;
      static $class:yfiles.lang.Class;
    }
    /**
     * Wraps an {@link yfiles.collections.IMapper.<K,V>} into an {@link yfiles.algorithms.IDataProvider}.
     * <p>
     * This class is used by {@link yfiles.layout.LayoutExecutor}, {@link yfiles.layout.LayoutGraphAdapter}, and {@link yfiles.layout.YGraphAdapter} to
     * convert an {@link yfiles.graph.IGraph} into a corresponding
     * {@link yfiles.algorithms.Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
     * class usually doesn't need to be used explicitly.
     * </p>
     * Type parameter <code>TKey</code>: The type of the keys.
     * Type parameter <code>TValue</code>: The type of the values.
     * @class
     * @implements {yfiles.algorithms.IDataProvider}
     * @template TKey,TValue
     */
    export interface DataProviderAdapter<TKey,TValue> extends Object,yfiles.algorithms.IDataProvider{}
    export class DataProviderAdapter<TKey,TValue> {
      /**
       * Creates a new instance that adapts the provided mapper instance.
       * @param keyType The type of the keys.
       * @param mapper The mapper to adapt.
       */
      constructor(keyType:yfiles.lang.Class,mapper:yfiles.collections.IMapper<TKey,TValue>);
      /**
       * 
       * @param dataHolder
       * @returns 
       */
      get(dataHolder:Object):Object;
      /**
       * 
       * @param dataHolder
       * @returns 
       */
      getBoolean(dataHolder:Object):boolean;
      /**
       * 
       * @param dataHolder
       * @returns 
       */
      getInt(dataHolder:Object):number;
      /**
       * 
       * @param dataHolder
       * @returns 
       */
      getNumber(dataHolder:Object):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An adapter implementation that wraps an {@link yfiles.collections.IMapper.<K,V>} to an {@link yfiles.algorithms.IDataMap}.
     * Type parameter <code>K</code>: The type of the key for the mapper.
     * Type parameter <code>V</code>: The type of the value for the mapper.
     * @see yfiles.layout.DataProviderAdapter.<K,V>
     * @class
     * @extends {yfiles.layout.DataProviderAdapter.<K,V>}
     * @implements {yfiles.algorithms.IDataMap}
     * @template K,V
     */
    export interface DataMapAdapter<K,V> extends yfiles.layout.DataProviderAdapter<K,V>,yfiles.algorithms.IDataMap{}
    export class DataMapAdapter<K,V> {
      /**
       * Initializes a new instance of the {@link yfiles.layout.DataMapAdapter.<K,V>} class.
       * @param keyType The type of the key for the mapper.
       * @param valueType The type of the value for the mapper.
       * @param mapper The mapper to adapt.
       */
      constructor(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,mapper:yfiles.collections.IMapper<K,V>);
      /**
       * 
       * @param dataHolder
       * @param value
       */
      set(dataHolder:Object,value:Object):void;
      /**
       * 
       * @param dataHolder
       * @param value
       */
      setBoolean(dataHolder:Object,value:boolean):void;
      /**
       * 
       * @param dataHolder
       * @param value
       */
      setInt(dataHolder:Object,value:number):void;
      /**
       * 
       * @param dataHolder
       * @param value
       */
      setNumber(dataHolder:Object,value:number):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * An item collection that allows to specify the data provider key the collection is registered with.
     * <p>
     * This class supports {@link yfiles.layout.LayoutData} implementations and does not usually have to be used explicitly.
     * </p>
     * Type parameter <code>T</code>: The type of the items in the collection.
     * @class
     * @extends {yfiles.layout.ItemCollection.<T>}
     * @template T
     */
    export interface DpKeyItemCollection<T> extends yfiles.layout.ItemCollection<T>{}
    export class DpKeyItemCollection<T> {
      constructor();
      /**
       * Gets or sets the data provider key that is used for the item collection.
       * <p>
       * If no key is set, the default data provider key is used.
       * </p>
       * @type {yfiles.algorithms.DpKeyBase.<boolean>}
       */
      dpKey:yfiles.algorithms.DpKeyBase<boolean>;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.layout.EdgeLabelCandidate} that provides a {@link yfiles.graph.ILabelCandidateDescriptor candidate descriptor}.
     * <p>
     * This class is used by {@link yfiles.layout.LayoutExecutor} and {@link yfiles.layout.LayoutGraphAdapter} to convert an {@link yfiles.graph.IGraph}
     * into a corresponding
     * {@link yfiles.algorithms.Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
     * class usually doesn't need to be used explicitly.
     * </p>
     * @class
     * @extends {yfiles.layout.EdgeLabelCandidate}
     */
    export interface ExtendedEdgeLabelCandidate extends yfiles.layout.EdgeLabelCandidate{}
    export class ExtendedEdgeLabelCandidate {
      /**
       * Returns a new instance of ExtendedEdgeLabelCandidate.
       * <p>
       * Returns a new instance of ExtendedEdgeLabelCandidate.
       * </p>
       * @param pos the location of the upper left corner of the candidate.
       * @param size the size of the candidate.
       * @param param the parameters of the labeling model associated with this candidate.
       * @param owner the label associated to the candidate.
       * @param internal flag whether the candidate is allowed to overlap the layout of the edge.
       * @param descriptor the label candidate descriptor.
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.IEdgeLabelLayout,internal:boolean,descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Returns a new instance of ExtendedEdgeLabelCandidate.
       * <p>
       * Returns a new instance of ExtendedEdgeLabelCandidate.
       * </p>
       * @param orientedBox the label size and orientation
       * @param param the parameters of the labeling model associated with this candidate.
       * @param owner the label associated to the candidate.
       * @param internal flag whether the candidate is allowed to overlap the layout of the edge.
       * @param descriptor the label candidate descriptor.
       */
      constructor(orientedBox:yfiles.algorithms.YOrientedRectangle,param:Object,owner:yfiles.layout.IEdgeLabelLayout,internal:boolean,descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Returns a new instance of ExtendedEdgeLabelCandidate.
       * <p>
       * Returns a new instance of ExtendedEdgeLabelCandidate.
       * </p>
       * @param pos the location of the upper left corner of the candidate.
       * @param size the size of the candidate.
       * @param param the parameters of the labeling model associated with this candidate.
       * @param owner the label associated to the candidate.
       * @param descriptor the label candidate descriptor.
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.IEdgeLabelLayout,descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Gets the label candidate descriptor.
       * @type {yfiles.graph.ILabelCandidateDescriptor}
       */
      descriptor:yfiles.graph.ILabelCandidateDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.layout.IProfitModel} that returns the profit of a label candidate for instances of {@link yfiles.layout.ExtendedNodeLabelCandidate}
     * and {@link yfiles.layout.ExtendedEdgeLabelCandidate}.
     * <p>
     * This class is used by {@link yfiles.layout.LayoutExecutor} and {@link yfiles.layout.LayoutGraphAdapter} to convert an {@link yfiles.graph.IGraph}
     * into a corresponding
     * {@link yfiles.algorithms.Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
     * class usually doesn't need to be used explicitly.
     * </p>
     * @class
     * @implements {yfiles.layout.IProfitModel}
     */
    export interface ExtendedLabelCandidateProfitModel extends Object,yfiles.layout.IProfitModel{}
    export class ExtendedLabelCandidateProfitModel {
      /**
       * Returns the profit for placing a label-candidate.
       * <p>
       * This implementation returns the profit for instances of {@link yfiles.layout.ExtendedNodeLabelCandidate} or 0.0 for other
       * {@link yfiles.layout.LabelCandidate}s.
       * </p>
       * @param candidate a label candidate
       * @returns a value between 0 and 1.
       */
      getProfit(candidate:yfiles.layout.LabelCandidate):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.layout.NodeLabelCandidate} that provides a {@link yfiles.graph.ILabelCandidateDescriptor candidate descriptor}.
     * <p>
     * This class is used by {@link yfiles.layout.LayoutExecutor} and {@link yfiles.layout.LayoutGraphAdapter} to convert an {@link yfiles.graph.IGraph}
     * into a corresponding
     * {@link yfiles.algorithms.Graph}. Unless that conversion needs to be customized, or aforementioned classes cannot be used, this
     * class usually doesn't need to be used explicitly.
     * </p>
     * @class
     * @extends {yfiles.layout.NodeLabelCandidate}
     */
    export interface ExtendedNodeLabelCandidate extends yfiles.layout.NodeLabelCandidate{}
    export class ExtendedNodeLabelCandidate {
      /**
       * Returns a new instance of NodeLabelCandidate.
       * <p>
       * Returns a new instance of NodeLabelCandidate.
       * </p>
       * @param pos the location of the upper left corner of the candidate.
       * @param size the size of the candidate.
       * @param param the parameters of the labeling model associated with this candidate.
       * @param owner the label associated to the candidate.
       * @param internal flag whether the candidate is inside the node, or outside.
       * @param descriptor the label candidate descriptor.
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.INodeLabelLayout,internal:boolean,descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Returns a new instance of NodeLabelCandidate.
       * <p>
       * Returns a new instance of NodeLabelCandidate.
       * </p>
       * @param orientedBox the label size and orientation
       * @param param the parameters of the labeling model associated with this candidate.
       * @param owner the label associated to the candidate.
       * @param internal flag whether the candidate is inside the node, or outside.
       * @param descriptor the label candidate descriptor.
       */
      constructor(orientedBox:yfiles.algorithms.YOrientedRectangle,param:Object,owner:yfiles.layout.INodeLabelLayout,internal:boolean,descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Returns a new instance of ExtendedNodeLabelCandidate.
       * <p>
       * Returns a new instance of ExtendedNodeLabelCandidate.
       * </p>
       * @param pos the location of the upper left corner of the candidate.
       * @param size the size of the candidate.
       * @param param the parameters of the labeling model associated with this candidate.
       * @param owner the label associated to the candidate.
       * @param descriptor the label candidate descriptor.
       */
      constructor(pos:yfiles.algorithms.YPoint,size:yfiles.algorithms.YDimension,param:Object,owner:yfiles.layout.INodeLabelLayout,descriptor:yfiles.graph.ILabelCandidateDescriptor);
      /**
       * Gets the label candidate descriptor.
       * @type {yfiles.graph.ILabelCandidateDescriptor}
       */
      descriptor:yfiles.graph.ILabelCandidateDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.layout.FixGroupLayoutStage}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface FixGroupLayoutData extends yfiles.layout.LayoutData{}
    export class FixGroupLayoutData {
      constructor();
      /**
       * Gets or sets the collection of group nodes whose contents should keep their relative positions.
       * @see yfiles.layout.FixGroupLayoutStage#FIXED_GROUP_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      fixedGroupNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.layout.FixNodeLayoutStage}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface FixNodeLayoutData extends yfiles.layout.LayoutData{}
    export class FixNodeLayoutData {
      constructor();
      /**
       * Gets or sets the collection of nodes which should be considered fixed.
       * @see yfiles.layout.FixNodeLayoutStage#FIXED_NODE_DP_KEY
       * @type {yfiles.layout.SingleItemCollection.<yfiles.graph.INode>}
       */
      fixedNode:yfiles.layout.SingleItemCollection<yfiles.graph.INode>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides different ways to define the items in a collection for use with {@link yfiles.layout.LayoutData}.
     * <p>
     * Only one of the provided ways to define a collection can be used at the same time, therefore only one of the properties
     * {@link yfiles.layout.ItemCollection.<TItem>#items}, {@link yfiles.layout.ItemCollection.<TItem>#mapper}, and {@link yfiles.layout.ItemCollection.<TItem>#delegate} should be set on each instance.
     * </p>
     * <p>
     * This class supports {@link yfiles.layout.LayoutData} implementations and does not usually have to be used explicitly.
     * </p>
     * Type parameter <code>TItem</code>: The type of the items in the collection.
     * @class
     * @template TItem
     */
    export interface ItemCollection<TItem> extends Object{}
    export class ItemCollection<TItem> {
      /**
       * Provides a mapper describing which elements are contained in this {@link yfiles.layout.ItemCollection.<T>}.
       * @param adapter The graph adapter the mapper shall be provided for.
       * @param layout The layout using the item collection.
       * @returns 
       */
      provideMapper(adapter:yfiles.layout.LayoutGraphAdapter,layout:yfiles.layout.ILayoutAlgorithm):yfiles.collections.IMapper<TItem,boolean>;
      /**
       * Gets or sets a mapping from the items to a boolean value.
       * <p>
       * The mapped boolean value specifies whether an item is contained in the collection.
       * </p>
       * @type {yfiles.collections.IMapper.<TItem,boolean>}
       */
      mapper:yfiles.collections.IMapper<TItem,boolean>;
      /**
       * Gets or sets a delegate from the items to a boolean value.
       * <p>
       * The returned boolean value describes whether an item is contained in the collection.
       * </p>
       * @type {function(TItem): boolean}
       */
      delegate:(obj:TItem)=>boolean;
      /**
       * Gets or sets a collection of items.
       * <p>
       * This collection contains exactly the items in the given collection.
       * </p>
       * @type {yfiles.collections.ICollection.<TItem>}
       */
      items:yfiles.collections.ICollection<TItem>;
      /**
       * Sets an enumerable of items.
       * <p>
       * This collection contains exactly the items in the given enumerable.
       * </p>
       * @type {yfiles.collections.IEnumerable.<TItem>}
       */
      source:yfiles.collections.IEnumerable<TItem>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides different ways to define a mapping from items to values for use with {@link yfiles.layout.LayoutData}.
     * <p>
     * Only one of the provided ways to define a mapping can be used at the same time, therefore only one of the properties {@link yfiles.layout.ItemMapping.<TItem,TValue>#mapper}, {@link yfiles.layout.ItemMapping.<TItem,TValue>#delegate}
     * and {@link yfiles.layout.ItemMapping.<TItem,TValue>#constant} should be set on each instance.
     * </p>
     * <p>
     * This class supports {@link yfiles.layout.LayoutData} implementations and does not usually have to be instantiated by client code
     * explicitly. Instances of this class are properties of concrete {@link yfiles.layout.LayoutData} implementations.
     * </p>
     * Type parameter <code>TItem</code>: The type of the items that are mapped.
     * Type parameter <code>TValue</code>: The type of the values the items are mapped to.
     * @class
     * @template TItem,TValue
     */
    export interface ItemMapping<TItem,TValue> extends Object{}
    export class ItemMapping<TItem,TValue> {
      /**
       * Provides a mapper mapping the items to values.
       * <p>
       * This implementation returns the {@link yfiles.layout.ItemMapping.<TItem,TValue>#mapper} if one has been set, otherwise it will create a mapper from the {@link yfiles.layout.ItemMapping.<TItem,TValue>#delegate} if one has been
       * specified or uses a constant mapper if a {@link yfiles.layout.ItemMapping.<TItem,TValue>#constant} has been specified. If none have been specified, <code>null</code> is returned.
       * </p>
       * @param adapter The graph adapter the mapper shall be provided for.
       * @param layout The layout using the mapper.
       * @returns A mapper that maps items to values or <code>null</code>.
       */
      provideMapper(adapter:yfiles.layout.LayoutGraphAdapter,layout:yfiles.layout.ILayoutAlgorithm):yfiles.collections.IMapper<TItem,TValue>;
      /**
       * Gets or sets a delegate that implements the mapping.
       * <p>
       * This value will be ignored if either {@link yfiles.layout.ItemMapping.<TItem,TValue>#mapper} as also been set. It overrides values set to the {@link yfiles.layout.ItemMapping.<TItem,TValue>#constant} property.
       * </p>
       * @type {function(TItem): TValue}
       */
      delegate:(key:TItem)=>TValue;
      /**
       * Gets or sets an {@link yfiles.collections.IMapper.<K,V>} that realizes the mapping.
       * <p>
       * Getting this property will automatically create an instance. Values set will override the effect of any values set to
       * the {@link yfiles.layout.ItemMapping.<TItem,TValue>#constant} or {@link yfiles.layout.ItemMapping.<TItem,TValue>#delegate} properties.
       * </p>
       * @type {yfiles.collections.IMapper.<TItem,TValue>}
       */
      mapper:yfiles.collections.IMapper<TItem,TValue>;
      /**
       * Gets or sets a constant value all items are mapped to.
       * <p>
       * This value will be ignored if either {@link yfiles.layout.ItemMapping.<TItem,TValue>#mapper} or {@link yfiles.layout.ItemMapping.<TItem,TValue>#delegate} have also been set.
       * </p>
       * @type {TValue}
       */
      constant:TValue;
      static $class:yfiles.lang.Class;
    }
    /**
     * Base for convenience classes that specify custom data for a layout calculation done with the
     * {@link yfiles.layout.LayoutExecutor}.
     * <p>
     * Implementations of this class provide item-specific configuration options for a specific layout. Single properties that
     * affect the complete layout are available on the specific layout class itself.
     * </p>
     * <p>
     * The {@link yfiles.layout.LayoutExecutor} {@link yfiles.layout.LayoutData#apply applies} these configurations when {@link yfiles.layout.LayoutExecutor#start started}. So does
     * {@link yfiles.layout.LayoutGraphAdapter#applyLayout}.
     * </p>
     * @see yfiles.layout.LayoutExecutor#layoutData
     * @see yfiles.layout.LayoutGraphAdapter#applyLayout
     * @class
     */
    export interface LayoutData extends Object{}
    export class LayoutData {
      /**
       * Applies this layout data to the specified <code>layoutGraphAdapter</code>.
       * <p>
       * This method should be called before a layout is executed. It will configure the adapter and graph for to the layout.
       * </p>
       * @param layoutGraphAdapter The adapter to add the layout data to.
       * @param layout The layout that shall use this layout data.
       * @param layoutGraph The layout graph used for the layout calculation.
       * @abstract
       */
      apply(layoutGraphAdapter:yfiles.layout.LayoutGraphAdapter,layout:yfiles.layout.ILayoutAlgorithm,layoutGraph:yfiles.layout.CopiedLayoutGraph):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * Executes a {@link yfiles.layout.ILayoutAlgorithm} and optionally animates the transition to the calculated layout.
     * <p>
     * This class is the preferred way to execute a layout. Its animation can be customized in various ways.
     * </p>
     * <p>
     * If no fine-grain control of the animation is required, the convenience methods {@link yfiles.graph.IGraph#applyLayout} and {@link yfiles.view.GraphComponent#morphLayout}
     * can be used instead.
     * </p>
     * @see yfiles.layout.LayoutGraphAdapter
     * @class
     */
    export interface LayoutExecutor extends Object{}
    export class LayoutExecutor {
      /**
       * Initializes a new instance of the {@link yfiles.layout.LayoutExecutor} class.
       * @param graphComponent The control which will be animated and provides the {@link } instance.
       * @param graph The graph to layout.
       * @param layout The {@link } to use.
       */
      constructor(graphComponent:yfiles.view.GraphComponent,graph:yfiles.graph.IGraph,layout:yfiles.layout.ILayoutAlgorithm);
      /**
       * Initializes a new instance of the {@link yfiles.layout.LayoutExecutor} class.
       * <p>
       * Use the {@link yfiles.layout.LayoutExecutor#start} method to actually execute the task. This will disable animation and content
       * rect updates.
       * </p>
       * @param graphComponent The control which will be animated and provides the {@link } instance.
       * @param layout The {@link } to use.
       */
      constructor(graphComponent:yfiles.view.GraphComponent,layout:yfiles.layout.ILayoutAlgorithm);
      /**
       * Creates an instance of {@link yfiles.algorithms.AbortHandler}.
       * <p>
       * This method is called upon first access to the {@link yfiles.layout.LayoutExecutor#abortHandler} property.
       * </p>
       * @returns A new {@link } instance to use during layout calculation.
       * @protected
       */
      createAbortHandler():yfiles.algorithms.AbortHandler;
      /**
       * Factory method that creates the {@link yfiles.view.IAnimation} that will be used by this instance after the layout has been
       * calculated.
       * @returns The animation to use after the layout.
       * @see yfiles.layout.LayoutExecutor#duration
       * @see yfiles.layout.LayoutExecutor#animateViewport
       * @see yfiles.layout.LayoutExecutor#createMorphAnimation
       * @see yfiles.layout.LayoutExecutor#createViewportAnimation
       * @protected
       */
      createAnimation():yfiles.view.IAnimation;
      /**
       * Factory method that creates the animation for the {@link yfiles.graph.IGraph}.
       * @returns The animation instance.
       * @see yfiles.layout.LayoutExecutor#createAnimation
       * @protected
       */
      createMorphAnimation():yfiles.view.IAnimation;
      /**
       * Creates an animation that morphs the layout of all {@link yfiles.graph.ITable}s in the graph.
       * @returns 
       * @see yfiles.view.TableAnimation
       * @see yfiles.layout.LayoutExecutor#configureTableLayout
       * @protected
       */
      createTableAnimations():yfiles.view.IAnimation;
      /**
       * Create a new instance of {@link yfiles.layout.TableLayoutConfigurator} that is used if {@link yfiles.layout.LayoutExecutor#configureTableLayout} is enabled.
       * <p>
       * This method is called upon first access to the {@link yfiles.layout.LayoutExecutor#tableLayoutConfigurator} property.
       * </p>
       * @returns A new instance of the {@link } class.
       * @protected
       */
      createTableLayoutConfigurator():yfiles.layout.TableLayoutConfigurator;
      /**
       * Factory method that creates the animation for the {@link yfiles.view.CanvasComponent#viewport}.
       * @param targetBounds The target bounds of the animation.
       * @returns The animation instance.
       * @see yfiles.layout.LayoutExecutor#createAnimation
       * @protected
       */
      createViewportAnimation(targetBounds:yfiles.geometry.Rect):yfiles.view.IAnimation;
      /**
       * Callback method that performs the actual layout.
       * @protected
       */
      execute():void;
      /**
       * Calculate the target bounds to be used for the {@link yfiles.view.CanvasComponent#contentRect} as well as the {@link yfiles.view.ViewportAnimation} after the layout has
       * finished.
       * <p>
       * By default, the value is computed with {@link yfiles.layout.LayoutGraphUtilities#getBoundingBox} with labels and {@link yfiles.layout.LayoutExecutor#targetBoundsInsets} taken into
       * account.
       * </p>
       * @returns The desired content rectangle and the bounds for a viewport animation.
       * @protected
       */
      getTargetBounds():yfiles.geometry.Rect;
      /**
       * Set up {@link yfiles.layout.LayoutExecutor#tableLayoutConfigurator} for a layout.
       * <p>
       * This implementation configures {@link yfiles.layout.TableLayoutConfigurator#horizontalLayout} according to the {@link yfiles.layout.MultiStageLayout#layoutOrientation} and calls {@link yfiles.layout.TableLayoutConfigurator#prepare}
       * </p>
       * @protected
       */
      prepareTableLayout():void;
      /**
       * Writes the table layout information provided through {@link yfiles.layout.LayoutExecutor#tableLayoutConfigurator} back to all tables.
       * <p>
       * This method is only called when the layout is not animated.
       * </p>
       * @see yfiles.layout.LayoutExecutor#prepareTableLayout
       * @protected
       */
      restoreTableLayout():void;
      /**
       * Actually starts the layout calculation and the optional animation asynchronously using a Promise.
       * <p>
       * This method will ultimately call the {@link yfiles.layout.LayoutExecutor#execute} method. If the {@link yfiles.layout.LayoutExecutor#duration} is zero no animation will be
       * performed.
       * </p>
       * <p>
       * If this instance is already running, this method returns immedetaly without doing anything and returns the previous
       * Promise.
       * </p>
       * @returns A Promise that will be fulfilled once the layout and optional animation is done.
       * @see yfiles.layout.LayoutExecutor#execute
       */
      start():Promise<Object>;
      /**
       * Stops a currently running layout calculation or animation.
       * <p>
       * If a layout calculation is still running, it will be requested to stopped via {@link yfiles.algorithms.AbortHandler#stop} and
       * the animation will not run. If the layout calculation was already completed, the animation will be aborted immediately
       * and the layout result will be shown immediately.
       * </p>
       * <p>
       * To just skip the animation but let the calculation finish normally, the {@link yfiles.layout.LayoutExecutor#duration} can be set to zero at any time before the
       * animation was started.
       * </p>
       * @returns 
       */
      stop():Promise<Object>;
      /**
       * Gets the {@link yfiles.algorithms.AbortHandler} that is used during the layout calculation, unless another {@link yfiles.algorithms.AbortHandler}
       * has been configured via {@link yfiles.layout.LayoutExecutor#layoutData} or by registering an {@link yfiles.collections.IMapper.<K,V>} in the graph's {@link yfiles.graph.IGraph#mapperRegistry}.
       * @see yfiles.layout.LayoutExecutor#createAbortHandler
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets the {@link yfiles.layout.LayoutExecutor#tableLayoutConfigurator} that is used if {@link yfiles.layout.LayoutExecutor#configureTableLayout} is enabled.
       * @see yfiles.layout.LayoutExecutor#createTableLayoutConfigurator
       * @type {yfiles.layout.TableLayoutConfigurator}
       */
      tableLayoutConfigurator:yfiles.layout.TableLayoutConfigurator;
      /**
       * Gets or sets a value indicating whether to automatically perform calls to {@link yfiles.layout.TableLayoutConfigurator#prepare}
       * and {@link yfiles.layout.TableLayoutConfigurator#restore} in order to layout table nodes.
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      configureTableLayout:boolean;
      /**
       * Gets or sets the duration of the animation.
       * <p>
       * The default value is {@link yfiles.lang.TimeSpan#ZERO}.
       * </p>
       * @type {yfiles.lang.TimeSpan}
       */
      duration:yfiles.lang.TimeSpan;
      /**
       * Gets or sets a value indicating whether edge groups are automatically created for edges that are connected to the same
       * port.
       * <p>
       * If this property is enabled and no edge groups are explicitly created (no {@link yfiles.collections.IMapper.<K,V>} are registered for the keys {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
       * and {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), for each port with multiple outgoing resp. incoming edges
       * these edges will be assigned to the same edge group. The necessary
       * {@link yfiles.algorithms.IDataProvider}s will be created automatically.
       * </p>
       * <p>
       * Without an {@link yfiles.layout.ILayoutAlgorithm} that supports edge groups, this property ahs no visual effect.
       * </p>
       * <p>
       * The default value is <code>true</code>.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @see yfiles.layout.LayoutGraphAdapter#automaticEdgeGrouping
       * @see yfiles.layout.LayoutExecutor#fixPorts
       * @type {boolean}
       */
      automaticEdgeGrouping:boolean;
      /**
       * Controls whether strong port constraints are automatically created.
       * <p>
       * If this property is enabled and no edge groups are explicitly created (no {@link yfiles.collections.IMapper.<K,V>} are registered for the keys {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
       * and {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), for all edges strong source and target point constraints
       * with {@link yfiles.layout.PortSide#ANY} are created. The necessary
       * {@link yfiles.algorithms.IDataProvider}s will be created automatically.
       * </p>
       * <p>
       * This ensures that view {@link yfiles.graph.IPort}s are not moved during the layout, if the {@link yfiles.layout.ILayoutAlgorithm}
       * supports port constraints.
       * </p>
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @see yfiles.layout.LayoutGraphAdapter#fixPorts
       * @see yfiles.layout.LayoutExecutor#automaticEdgeGrouping
       * @type {boolean}
       */
      fixPorts:boolean;
      /**
       * Gets or sets a value indicating whether to animate the viewport.
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      animateViewport:boolean;
      /**
       * Gets or sets a value indicating whether to respect the {@link yfiles.view.CanvasComponent#viewportLimiter} of the {@link yfiles.view.GraphComponent} of this instance.
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      considerViewportLimiter:boolean;
      /**
       * Gets or sets a value indicating whether to use eased animation.
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      easedAnimation:boolean;
      /**
       * Gets or sets the insets (in world coordinates) that will be added to the content rectangle when calculating the target
       * viewport.
       * <p>
       * The default value is {@link yfiles.geometry.Insets#EMPTY}.
       * </p>
       * @type {yfiles.geometry.Insets}
       */
      targetBoundsInsets:yfiles.geometry.Insets;
      /**
       * Gets or sets a value indicating whether the content rectangle property of the {@link yfiles.view.CanvasComponent} should be
       * updated upon completion.
       * <p>
       * The default value is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      updateContentRect:boolean;
      /**
       * Gets a value indicating whether this instance is currently running.
       * @type {boolean}
       */
      running:boolean;
      /**
       * Gets or sets the layout data that is applied when {@link yfiles.layout.LayoutExecutor#start starting} the executor.
       * @type {yfiles.layout.LayoutData}
       */
      layoutData:yfiles.layout.LayoutData;
      /**
       * Gets or sets the {@link yfiles.view.ISelectionModel.<T>} to use for the automatically registered {@link yfiles.algorithms.IDataProvider} instances for {@link yfiles.layout.LayoutKeys#AFFECTED_NODES_DP_KEY}
       * and {@link yfiles.layout.LayoutKeys#AFFECTED_EDGES_DP_KEY}.
       * <p>
       * If this value is <code>null</code> and {@link yfiles.layout.LayoutExecutor#graph} is the same instance as {@link yfiles.layout.LayoutExecutor#graphComponent}'s {@link yfiles.graph.IGraph} instance, the selection model from {@link yfiles.layout.LayoutExecutor#graphComponent} is used
       * instead.
       * </p>
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IModelItem>}
       */
      selectionModel:yfiles.view.ISelectionModel<yfiles.graph.IModelItem>;
      /**
       * Gets the layout graph that is used by this instance to calculate the layout.
       * @protected
       * @type {yfiles.layout.CopiedLayoutGraph}
       */
      layoutGraph:yfiles.layout.CopiedLayoutGraph;
      /**
       * Gets the control this instance has been created for.
       * @protected
       * @type {yfiles.view.GraphComponent}
       */
      graphComponent:yfiles.view.GraphComponent;
      /**
       * Gets the {@link yfiles.layout.ILayoutAlgorithm} this instance is using.
       * @protected
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      layout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets the graph this instance is working on.
       * @protected
       * @type {yfiles.graph.IGraph}
       */
      graph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * Adapter class that can be used to obtain an {@link yfiles.layout.LayoutGraph} from an {@link yfiles.graph.IGraph}.
     * <p>
     * Typically, this class is not used directly but through the convenience class {@link yfiles.layout.LayoutExecutor} or the
     * convenience methods {@link yfiles.graph.IGraph#applyLayout} and MorphLayout
     * </p>
     * <p>
     * This class will use the provided {@link yfiles.layout.LayoutGraphAdapter#selectionModel} to register corresponding {@link yfiles.algorithms.IDataProvider}s for the data provider keys
     * {@link yfiles.layout.LayoutKeys#AFFECTED_NODES_DP_KEY} and {@link yfiles.layout.LayoutKeys#AFFECTED_EDGES_DP_KEY} to the layout graph.
     * Moreover, it will automatically convert {@link yfiles.collections.IMapper.<K,V>} instances registered with the {@link yfiles.graph.IMapperRegistry} of the {@link yfiles.layout.LayoutGraphAdapter#adaptedGraph} into
     * corresponding {@link yfiles.algorithms.IDataProvider} instances of the layout graph.
     * </p>
     * <p>
     * In addition, this class supports {@link yfiles.graph.IGraph}s with edge to edge connections, i.e., edges that are adjacent to
     * other edges. Since such edges are not allowed in an {@link yfiles.layout.LayoutGraph}, the property {@link yfiles.layout.LayoutGraphAdapter#hideEdgesAtEdges} specifies whether these
     * edges shouldn't be converted into the layout graph at all or converted into a corresponding structure with dummy nodes.
     * </p>
     * <p>
     * The chapter {@link @DGUIDE_LAYOUT_PREFIX@/layout-applying_a_layout.html Applying an Automatic Layout} describes how automatic layout and graph analysis algorithms from the yFiles
     * Layout and yFiles Basic library components can be accessed from @COMPONENT_NAME@.
     * </p>
     * @see yfiles.layout.CopiedLayoutGraph
     * @see yfiles.layout.ILayoutAlgorithm
     * @class
     */
    export interface LayoutGraphAdapter extends Object{}
    export class LayoutGraphAdapter {
      /**
       * Creates the adapter using a given {@link yfiles.graph.IGraph} and {@link yfiles.view.ISelectionModel.<T>}.
       * @param graph The graph to build an adapter for.
       * @param [selectionModel=null] The optional selection model to use by the {@link #AFFECTED_NODES_DP_KEY} and {@link #AFFECTED_EDGES_DP_KEY}
       * {@link } instances that are automatically bound to this adapter.
       * @throws {Stubs.Exceptions.ArgumentError} <code>graph</code> is null.
       */
      constructor(graph:yfiles.graph.IGraph,selectionModel?:yfiles.view.ISelectionModel<yfiles.graph.IModelItem>);
      /**
       * Creates the adapter with no initial {@link yfiles.graph.IGraph} and no {@link yfiles.view.ISelectionModel.<T>}.
       * <p>
       * Before this instance can be used for automatic layouts, you mus assign a value to the {@link yfiles.layout.LayoutGraphAdapter#adaptedGraph} property.
       * </p>
       */
      constructor();
      /**
       * Registers a data provider with the graph using the given look-up key and wrapping the given mapper.
       * <p>
       * If a mapper is already registered under the given key, it is overwritten.
       * </p>
       * Type parameter <code>K</code>: The type of the key to use.
       * Type parameter <code>V</code>: The type of the value to use.
       * @param valueType The type of the value to use.
       * @param keyType The type of the key to use.
       * @param dataKey The look-up key to register the data provider with.
       * @param mapper The mapper wrapped as data provider.
       * @returns 
       * @template K,V
       */
      addDataProvider<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,dataKey:Object,mapper:yfiles.collections.IMapper<K,V>):yfiles.algorithms.IDataProvider;
      /**
       * Convenience method that applies a {@link yfiles.layout.ILayoutAlgorithm layout algorithm} to an {@link yfiles.graph.IGraph}, optionally applying some
       * {@link yfiles.layout.LayoutData}.
       * @param layout The layout algorithm to apply.
       * @param [layoutData=null] The layout data.
       */
      applyLayout(layout:yfiles.layout.ILayoutAlgorithm,layoutData?:yfiles.layout.LayoutData):void;
      /**
       * Static convenience method that applies a {@link yfiles.layout.ILayoutAlgorithm layout algorithm} to an {@link yfiles.graph.IGraph}.
       * @param layout The layout algorithm to apply.
       * @param graph The graph to compute the layout on.
       * @param [layoutData=null] The layout data.
       * @static
       */
      static applyLayout(graph:yfiles.graph.IGraph,layout:yfiles.layout.ILayoutAlgorithm,layoutData?:yfiles.layout.LayoutData):void;
      /**
       * Creates a {@link yfiles.layout.CopiedLayoutGraph} for this instance.
       * @returns An instance whose {@link #commitLayoutToOriginalGraph} will update this instance.
       */
      createCopiedLayoutGraph():yfiles.layout.CopiedLayoutGraph;
      /**
       * Creates the {@link yfiles.layout.EdgeLabelCandidate} for a given label and {@link yfiles.graph.ILabelModelParameter} combination.
       * @param originalLabel The original label.
       * @param dummyLabel The dummy label that describes the current laid out label.
       * @param layoutParameter The parameter to create a candidate for.
       * @param labelLayout The {@link } of the label to pass to the candidate.
       * @returns A candidate to yield for the labeling algorithms or <code>null</code>.
       */
      createEdgeLabelCandidate(originalLabel:yfiles.graph.ILabel,dummyLabel:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter,labelLayout:yfiles.layout.IEdgeLabelLayout):yfiles.layout.EdgeLabelCandidate;
      /**
       * Factory method that creates an {@link yfiles.layout.IEdgeLayout} implementation that wraps the given {@link yfiles.graph.IEdge}
       * @param edge The edge to wrap.
       * @returns A wrapper instance.
       * @protected
       */
      createEdgeLayout(edge:yfiles.graph.IEdge):yfiles.layout.IEdgeLayout;
      /**
       * Callback factory method that creates the {@link yfiles.layout.INodeLayout} for the given {@link yfiles.graph.IPort} that is used as a
       * dummy to represent the port at the {@link yfiles.graph.IEdge} that owns <code>port</code>.
       * @param port The port to create the layout for.
       * @returns A suitable implementation of {@link } that uses the {@link #location port's location} as the center of the node.
       * @see yfiles.layout.LayoutGraphAdapter#edgePortNodeSize
       * @see yfiles.layout.LayoutGraphAdapter#hideEdgesAtEdges
       * @protected
       */
      createEdgePortLayout(port:yfiles.graph.IPort):yfiles.layout.INodeLayout;
      /**
       * Creates a {@link yfiles.layout.NodeLabelCandidate} for a given label and {@link yfiles.graph.ILabelModelParameter} combination.
       * @param originalLabel The original label.
       * @param dummyLabel The dummy label that describes the current laid out label.
       * @param layoutParameter The parameter to create a candidate for.
       * @param ownerLayout The {@link } of the label to pass to the candidate.
       * @returns A candidate to yield for the labeling algorithms or <code>null</code>.
       */
      createNodeLabelCandidate(originalLabel:yfiles.graph.ILabel,dummyLabel:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter,ownerLayout:yfiles.layout.INodeLabelLayout):yfiles.layout.NodeLabelCandidate;
      /**
       * Factory method that creates a {@link yfiles.layout.INodeLayout} implementation that wraps the given {@link yfiles.graph.INode}
       * @param node The node to wrap.
       * @returns A wrapper instance.
       * @protected
       */
      createNodeLayout(node:yfiles.graph.INode):yfiles.layout.INodeLayout;
      /**
       * 
       * @returns 
       */
      edgeObjects():yfiles.collections.IEnumerable<Object>;
      /**
       * 
       * @param dataKey
       * @returns 
       */
      getDataProvider(dataKey:Object):yfiles.algorithms.IDataProvider;
      /**
       * 
       * @param edge
       * @returns 
       */
      getEdgeLabelLayout(edge:Object):yfiles.layout.IEdgeLabelLayout[];
      /**
       * 
       * @param edge
       * @returns 
       */
      getEdgeLayout(edge:Object):yfiles.layout.IEdgeLayout;
      /**
       * Callback that retrieves the label candidate descriptor for the given label and parameter.
       * @param originalLabel The original label to which a parameter will be applied.
       * @param dummyLabel The dummy label to which the parameter will be applied.
       * @param layoutParameter The parameter to apply.
       * @returns A descriptor or <code>null</code>.
       * @see yfiles.layout.LayoutGraphAdapter#createNodeLabelCandidate
       * @see yfiles.layout.LayoutGraphAdapter#createEdgeLabelCandidate
       * @see yfiles.layout.LayoutGraphAdapter#getLabelCandidateDescriptorProvider
       * @protected
       */
      getLabelCandidateDescriptor(originalLabel:yfiles.graph.ILabel,dummyLabel:yfiles.graph.ILabel,layoutParameter:yfiles.graph.ILabelModelParameter):yfiles.graph.ILabelCandidateDescriptor;
      /**
       * Callback method that gets the label candidate descriptor provider for the given {@link yfiles.graph.IGraph}.
       * <p>
       * This implementation uses the {@link yfiles.graph.IGraph}'s {@link yfiles.graph.ILookup} to find an implementation of the
       * {@link yfiles.graph.ILabelCandidateDescriptorProvider}.
       * </p>
       * @param graph The graph.
       * @returns The result of a {@link #lookup} query on <code>graph</code> for the {@link } type.
       * @protected
       */
      getLabelCandidateDescriptorProvider(graph:yfiles.graph.IGraph):yfiles.graph.ILabelCandidateDescriptorProvider;
      /**
       * 
       * @param node
       * @returns 
       */
      getNodeLabelLayout(node:Object):yfiles.layout.INodeLabelLayout[];
      /**
       * 
       * @param o
       * @returns 
       */
      getNodeLayout(o:Object):yfiles.layout.INodeLayout;
      /**
       * 
       * @param edgeObject
       * @returns 
       */
      getSource(edgeObject:Object):Object;
      /**
       * 
       * @param edgeObject
       * @returns 
       */
      getTarget(edgeObject:Object):Object;
      /**
       * 
       * @returns 
       */
      nodeObjects():yfiles.collections.IEnumerable<Object>;
      /**
       * Callback method that assigns a new location that matches <code>newLocation</code> to <code>port</code>.
       * <p>
       * This method is called only for ports which are owned by edges. These are represented by dummy nodes during the layout,
       * and <code>newLocation</code> is the position of the dummy node after the layout.
       * </p>
       * @param port The edge port that should be placed.
       * @param newLocation The new location of the port as calculated by the layout algorithm.
       * @protected
       */
      setEdgePortLocation(port:yfiles.graph.IPort,newLocation:yfiles.geometry.Point):void;
      /**
       * Callback method that assigns a new relative location to <code>port</code>.
       * <p>
       * The default implementation calls {@link yfiles.graph.IGraph#setRelativePortLocation} with
       * <code>newRelativeLocation</code>. Depending on the port location model, the actual resulting port location may be different.
       * </p>
       * @param port The port that should be placed.
       * @param newRelativeLocation The new coordinate offsets relative to the center of the node's {@link #layout}'s center.
       * @protected
       */
      setRelativePortLocation(port:yfiles.graph.IPort,newRelativeLocation:yfiles.geometry.Point):void;
      /**
       * An {@link yfiles.graph.IMapperRegistry} tag for an {@link yfiles.collections.IMapper.<K,V>} that provides for the edge {@link yfiles.graph.ILabel}s in the {@link yfiles.layout.LayoutGraphAdapter#adaptedGraph} a {@link yfiles.layout.PreferredPlacementDescriptor}
       * instance.
       * <p>
       * If there is a {@link yfiles.collections.IMapper.<K,V>} in the {@link yfiles.graph.IGraph#mapperRegistry} it will be queried for each {@link yfiles.graph.IEdge edge} {@link yfiles.graph.ILabel label} to set the {@link yfiles.layout.IEdgeLabelLayout#preferredPlacementDescriptor} property of the adapted implementation.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.IEdgeLabelLayoutDpKey.<yfiles.layout.PreferredPlacementDescriptor>}
       */
      static EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY:yfiles.algorithms.IEdgeLabelLayoutDpKey<yfiles.layout.PreferredPlacementDescriptor>;
      /**
       * A data provider key that can be used to look up the original {@link yfiles.graph.INode} of a {@link yfiles.algorithms.Node}.
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.graph.INode>}
       */
      static ORIGINAL_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.graph.INode>;
      /**
       * A data provider key that can be used to look up the original {@link yfiles.graph.IEdge} of an {@link yfiles.algorithms.Edge}.
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.graph.IEdge>}
       */
      static ORIGINAL_EDGE_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.graph.IEdge>;
      /**
       * A data provider key that can be used to look up the original {@link yfiles.graph.ILabel} of an {@link yfiles.layout.INodeLabelLayout}.
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.INodeLabelLayoutDpKey.<yfiles.graph.ILabel>}
       */
      static ORIGINAL_NODE_LABEL_DP_KEY:yfiles.algorithms.INodeLabelLayoutDpKey<yfiles.graph.ILabel>;
      /**
       * A data provider key that can be used to look up the original {@link yfiles.graph.ILabel} of an {@link yfiles.layout.IEdgeLabelLayout}.
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.IEdgeLabelLayoutDpKey.<yfiles.graph.ILabel>}
       */
      static ORIGINAL_EDGE_LABEL_DP_KEY:yfiles.algorithms.IEdgeLabelLayoutDpKey<yfiles.graph.ILabel>;
      /**
       * A data provider key that can be used to look up the {@link yfiles.graph.IPort source port} of the original {@link yfiles.graph.IEdge} of an
       * {@link yfiles.algorithms.Edge}.
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.graph.IPort>}
       */
      static ORIGINAL_SOURCE_PORT_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.graph.IPort>;
      /**
       * A data provider key that can be used to look up the {@link yfiles.graph.IPort target port} of the original {@link yfiles.graph.IEdge} of an
       * {@link yfiles.algorithms.Edge}.
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.graph.IPort>}
       */
      static ORIGINAL_TARGET_PORT_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.graph.IPort>;
      /**
       * A data provider key that can be used to look up the original <b>unconnected</b>{@link yfiles.graph.IPort ports} of the original {@link yfiles.graph.IPortOwner} of an {@link yfiles.algorithms.Node}
       * or, in the case of edge-to-edge connections, an {@link yfiles.algorithms.Edge}.
       * <p>
       * You can access <b>all</b> (connected and unconnected) ports through the {@link yfiles.graph.IPortOwner#ports} property of the original {@link yfiles.graph.IGraph} elements,
       * which can be retrieved through the {@link yfiles.layout.LayoutGraphAdapter#ORIGINAL_NODE_DP_KEY} and {@link yfiles.layout.LayoutGraphAdapter#ORIGINAL_EDGE_DP_KEY}
       * data providers. The information about whether a port is connected or not is not readily accessible from there, though.
       * You can use this data provider to specifically find these port, e.g. to create additional
       * {@link yfiles.layout.PortCandidate}s or just reserve space for them.
       * </p>
       * <p>
       * This data provider is created automatically for a {@link yfiles.layout.CopiedLayoutGraph} that is created as a copy of an {@link yfiles.graph.IGraph}
       * with the help of a {@link yfiles.layout.LayoutGraphAdapter}. Especially, it is available if the layout is run with a {@link yfiles.layout.LayoutExecutor}
       * or the convenience methods MorphLayout and {@link yfiles.graph.IGraph#applyLayout}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.GraphObjectDpKey.<yfiles.collections.IEnumerable.<yfiles.graph.IPort>>}
       */
      static UNCONNECTED_PORTS_DP_KEY:yfiles.algorithms.GraphObjectDpKey<yfiles.collections.IEnumerable<yfiles.graph.IPort>>;
      /**
       * This key is used to provide information about dummy nodes that have been inserted into the graph to model edges at
       * edges.
       * <p>
       * This key can be used to query an {@link yfiles.algorithms.IDataProvider} that yields <code>true</code> for {@link yfiles.algorithms.IDataProvider#getBoolean}
       * for nodes that are actually representatives of {@link yfiles.graph.IPort}s at {@link yfiles.graph.IEdge}s. Also the {@link yfiles.algorithms.IDataProvider#get}
       * method will yield the corresponding original {@link yfiles.graph.IEdge}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.graph.IEdge>}
       */
      static PORT_DUMMY_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.graph.IEdge>;
      /**
       * Sets or yields the original {@link yfiles.graph.IGraph} instance that is adapted by this instance.
       * <p>
       * Before performing a layout, this property must be set <b>or</b> the adapter must have been created with
       * {@link yfiles.layout.LayoutGraphAdapter#LayoutGraphAdapter}. Note that this property cannot be assigned again if its value is
       * already non <code>null</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the value of this property is already non <code>null</code>
       * @type {yfiles.graph.IGraph}
       */
      adaptedGraph:yfiles.graph.IGraph;
      /**
       * Gets or sets the {@link yfiles.view.ISelectionModel.<T>} to use for the automatically registered {@link yfiles.algorithms.IDataProvider} instances for the keys {@link yfiles.layout.LayoutKeys#AFFECTED_NODES_DP_KEY}
       * and {@link yfiles.layout.LayoutKeys#AFFECTED_EDGES_DP_KEY}.
       * @type {yfiles.view.ISelectionModel.<yfiles.graph.IModelItem>}
       */
      selectionModel:yfiles.view.ISelectionModel<yfiles.graph.IModelItem>;
      /**
       * Gets or sets the size of the nodes that are inserted for the ports that are created for {@link yfiles.graph.IEdge}s that are
       * connected at other {@link yfiles.graph.IEdge}s.
       * <p>
       * If this instance is configured to create dummy nodes for ports at edges, this will be the initial size of the dummy
       * nodes with the center of the node always being the location of the port. The default is <code>3x3</code>.
       * </p>
       * @see yfiles.layout.LayoutGraphAdapter#hideEdgesAtEdges
       * @type {yfiles.geometry.Size}
       */
      edgePortNodeSize:yfiles.geometry.Size;
      /**
       * Determines whether edges at other edges will be hidden from the layout graph or included.
       * <p>
       * If this property is set to <code>true</code>, edges at other edges are simply ignored and not part of the layout graph. The default
       * is <code>false</code> in which case dummy nodes are inserted into the layout graph for each source and target port of an {@link yfiles.graph.IEdge}
       * that is {@link yfiles.graph.IPort#owner owned} by an {@link yfiles.graph.IEdge}.
       * </p>
       * @see yfiles.layout.LayoutGraphAdapter#edgePortNodeSize
       * @type {boolean}
       */
      hideEdgesAtEdges:boolean;
      /**
       * Controls whether edge groups are automatically created for edges that are connected to the same port.
       * <p>
       * If this property is enabled and no edge groups are explicitly created ({@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
       * and {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}), for all ports with multiple outgoing resp. incoming edges
       * these edges will be assigned to the same edge group. The necessary
       * {@link yfiles.algorithms.IDataProvider}s will be created automatically.
       * </p>
       * <p>
       * This features requires edge group support from the {@link yfiles.layout.ILayoutAlgorithm}.
       * </p>
       * <p>
       * Default value is <code>true</code>
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @see yfiles.layout.LayoutExecutor#automaticEdgeGrouping
       * @see yfiles.layout.LayoutGraphAdapter#fixPorts
       * @type {boolean}
       */
      automaticEdgeGrouping:boolean;
      /**
       * Controls whether strong port constraints are automatically created.
       * <p>
       * If this property is enabled and no port constraints are explicitly created ({@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}
       * and {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}), for all edges strong source and target point
       * constraints with {@link yfiles.layout.PortSide#ANY} are created. The necessary
       * {@link yfiles.algorithms.IDataProvider}s will be created automatically.
       * </p>
       * <p>
       * This ensures that view {@link yfiles.graph.IPort}s are not moved during the layout, if the {@link yfiles.layout.ILayoutAlgorithm}
       * supports port constraints.
       * </p>
       * <p>
       * Default value is <code>false</code>
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @see yfiles.layout.LayoutExecutor#fixPorts
       * @see yfiles.layout.LayoutGraphAdapter#automaticEdgeGrouping
       * @type {boolean}
       */
      fixPorts:boolean;
      /**
       * 
       * @type {Array.<Object>}
       */
      dataProviderKeys:Object[];
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies a {@link yfiles.layout.PartitionGrid} for a layout.
     * <p>
     * The {@link yfiles.layout.PartitionCellId} mapping has to be specified by either setting the {@link yfiles.layout.PartitionGridData#cellIds} property, or either or both of the
     * {@link yfiles.layout.PartitionGridData#rowIndices} and {@link yfiles.layout.PartitionGridData#columnIndices} properties. When {@link yfiles.layout.PartitionGridData#cellIds} is set, the {@link yfiles.layout.PartitionGridData#grid} property must be set as well. Otherwise the size of the grid may be induced by the
     * maximum row and column indices returned by {@link yfiles.layout.PartitionGridData#rowIndices} and {@link yfiles.layout.PartitionGridData#columnIndices}.
     * </p>
     * <p>
     * Note that cell spanning is only supported using the {@link yfiles.layout.PartitionGridData#cellIds}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface PartitionGridData extends yfiles.layout.LayoutData{}
    export class PartitionGridData {
      constructor();
      /**
       * Gets or sets the partition grid.
       * @see yfiles.layout.PartitionGrid#PARTITION_GRID_DP_KEY
       * @type {yfiles.layout.PartitionGrid}
       */
      grid:yfiles.layout.PartitionGrid;
      /**
       * Gets or sets the mapping from nodes to {@link yfiles.layout.PartitionCellId} using the partition grid as context.
       * @type {yfiles.layout.ContextItemMapping.<yfiles.graph.INode,yfiles.layout.PartitionCellId,yfiles.layout.PartitionGrid>}
       */
      cellIds:yfiles.layout.ContextItemMapping<yfiles.graph.INode,yfiles.layout.PartitionCellId,yfiles.layout.PartitionGrid>;
      /**
       * Gets or sets the mapping from nodes to the index of their rows.
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      rowIndices:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      /**
       * Gets or sets the mapping from nodes to the index of their columns.
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      columnIndices:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.layout.PartitionLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface PartitionLayoutData extends yfiles.layout.LayoutData{}
    export class PartitionLayoutData {
      constructor();
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.layout.RecursiveGroupLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface RecursiveGroupLayoutData extends yfiles.layout.LayoutData{}
    export class RecursiveGroupLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from group nodes to the {@link yfiles.layout.ILayoutAlgorithm} used for their child nodes.
       * @see yfiles.layout.RecursiveGroupLayout#GROUP_NODE_LAYOUT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.ILayoutAlgorithm>}
       */
      groupNodeLayouts:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.ILayoutAlgorithm>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets a mapping from edges connecting to group nodes to source split ids.
       * <p>
       * Edges will be aligned with edges that connect to the same group node and have the same split id at their source
       * (preferably) or target.
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#SOURCE_SPLIT_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceSplitIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges connecting to group nodes to target split ids.
       * <p>
       * Edges will be aligned with edges that connect to the same group node and have the same split id at their source or
       * target (preferably).
       * </p>
       * @see yfiles.layout.RecursiveGroupLayout#TARGET_SPLIT_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetSplitIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from nodes to their {@link yfiles.layout.PortCandidateSet}.
       * @see yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.PortCandidateSet>}
       */
      nodePortCandidateSets:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.PortCandidateSet>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.layout.ReverseEdgesStage}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface ReverseEdgesStageData extends yfiles.layout.LayoutData{}
    export class ReverseEdgesStageData {
      constructor();
      /**
       * Gets or sets the collection of affected edges.
       * @see yfiles.layout.ReverseEdgesStage#AFFECTED_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides different ways to define the items in a collection for use with
     * {@link yfiles.layout.LayoutData}, especially if the collection contains only one item.
     * <p>
     * Only one of the provided ways to define a collection can be used at the same time, therefore only one of the properties
     * {@link yfiles.layout.SingleItemCollection.<V>#item}, {@link yfiles.layout.ItemCollection.<TItem>#items}, {@link yfiles.layout.ItemCollection.<TItem>#mapper}, and {@link yfiles.layout.ItemCollection.<TItem>#delegate} should be set on each instance.
     * </p>
     * <p>
     * This class supports {@link yfiles.layout.LayoutData} implementations and does not usually have to be used explicitly.
     * </p>
     * Type parameter <code>V</code>: The type of the items in the collection.
     * @class
     * @extends {yfiles.layout.ItemCollection.<V>}
     * @template V
     */
    export interface SingleItemCollection<V> extends yfiles.layout.ItemCollection<V>{}
    export class SingleItemCollection<V> {
      constructor();
      /**
       * Gets or sets the single item this item collection contains.
       * @type {V}
       */
      item:V;
      static $class:yfiles.lang.Class;
    }
    /**
     * Creates a {@link yfiles.layout.PartitionGrid} that matches a given table configuration of an {@link yfiles.graph.IGraph} and registers {@link yfiles.collections.IMapper.<K,V>}
     * for that grid at the graph.
     * <p>
     * Typically, this class is used as part of the convenience class {@link yfiles.layout.LayoutExecutor} to {@link yfiles.layout.TableLayoutConfigurator#prepare}
     * a partition grid before the layout is calculated and {@link yfiles.layout.TableLayoutConfigurator#restore} the graph
     * afterwards.
     * </p>
     * @class
     */
    export interface TableLayoutConfigurator extends Object{}
    export class TableLayoutConfigurator {
      /**
       * Destroy all information that results from a previous {@link yfiles.layout.TableLayoutConfigurator#prepare} and a subsequent
       * layout
       * @param graph
       */
      cleanUp(graph:yfiles.graph.IGraph):void;
      /**
       * Return the sizes of all <b>leaf</b> rows.
       * <p>
       * This method does not modify <code>originalTable</code>. You also need to call {@link yfiles.layout.TableLayoutConfigurator#cleanUp}
       * explicitly if you don't need the {@link yfiles.layout.TableLayoutConfigurator} anymore.
       * </p>
       * @param originalTable The table
       * @param nodeLayout The final layout of the node that owns the table
       * @returns The sizes of all <b>leaf</b> rows in <code>originalTable</code>, in their natural order.
       */
      getColumnLayout(originalTable:yfiles.graph.ITable,nodeLayout:yfiles.geometry.Rect):number[];
      /**
       * Return the sizes of all <b>leaf</b> rows.
       * <p>
       * This method does not modify <code>originalTable</code>. You also need to call {@link yfiles.layout.TableLayoutConfigurator#cleanUp}
       * explicitly if you don't need the {@link yfiles.layout.TableLayoutConfigurator} anymore.
       * </p>
       * @param originalTable The table
       * @param nodeLayout The final layout of the node that owns the table
       * @returns The sizes of all <b>leaf</b> rows in <code>originalTable</code>, in their natural order.
       */
      getRowLayout(originalTable:yfiles.graph.ITable,nodeLayout:yfiles.geometry.Rect):number[];
      /**
       * Setup partition grid information from a graph that contains table structures
       * @param graph
       */
      prepare(graph:yfiles.graph.IGraph):void;
      /**
       * Write back all information from the partition grid
       * @param graph
       */
      restore(graph:yfiles.graph.IGraph):void;
      /**
       * Gets or sets the minimum distance between two different table nodes
       * @type {number}
       */
      minimumTableDistance:number;
      /**
       * Specifies whether or not to enable size compaction for table nodes.
       * <p>
       * If size compaction is enabled, the size of a column or row (or a table node for that matter) may be reduced due to
       * layout calculation, but never below the column's or row's minimum size. If size compaction is disabled, columns and rows
       * (and therefore table nodes) may only grow due to layout calculation.
       * </p>
       * <p>
       * The default value for this property is <code>true</code>.
       * </p>
       * @type {boolean}
       */
      compaction:boolean;
      /**
       * Gets or sets whether the configurator will prepare its associated graph for a vertical (that is top-to-bottom or
       * bottom-to-top) layout calculation or a horizontal (that is left-to-right or right-to-left) one.
       * <p>
       * The default value for this property is <code>false</code>.
       * </p>
       * @type {boolean}
       */
      horizontalLayout:boolean;
      /**
       * Specifies whether or not from sketch mode should be enabled.
       * <p>
       * If enabled, the positions of the table group nodes are considered, i.e., for horizontal layout configuration these nodes
       * are sorted according to their y-coordinate and for vertical layout configuration according to their x-coordinate.
       * </p>
       * @type {boolean}
       */
      fromSketch:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Converts an {@link yfiles.graph.IGraph} instance into a {@link yfiles.algorithms.Graph} instance.
     * <p>
     * This class creates a structural copy of a given {@link yfiles.graph.IGraph} and provides convenience methods to convert items
     * from one graph instance to their corresponding item in the other graph instance.
     * </p>
     * <p>
     * The main use case of this class is executing one of the analysis algorithms on an {@link yfiles.graph.IGraph}, for example
     * {@link yfiles.algorithms.ShortestPaths}, {@link yfiles.algorithms.Cycles}, and {@link yfiles.algorithms.Centrality}. For executing a layout
     * algorithm, use the convenience class {@link yfiles.layout.LayoutExecutor}, the convenience methods {@link yfiles.graph.IGraph#applyLayout}
     * and MorphLayout, or {@link yfiles.layout.LayoutGraphAdapter}.
     * </p>
     * @see yfiles.layout.LayoutGraphAdapter
     * @see yfiles.layout.LayoutExecutor
     * @see yfiles.algorithms.Bipartitions
     * @see yfiles.algorithms.Centrality
     * @see yfiles.algorithms.Cycles
     * @see yfiles.algorithms.GraphChecker
     * @see yfiles.algorithms.GraphConnectivity
     * @see yfiles.algorithms.Groups
     * @see yfiles.algorithms.IndependentSets
     * @see yfiles.algorithms.NetworkFlows
     * @see yfiles.algorithms.NodeOrders
     * @see yfiles.algorithms.Paths
     * @see yfiles.algorithms.RankAssignments
     * @see yfiles.algorithms.ShortestPaths
     * @see yfiles.algorithms.SpanningTrees
     * @see yfiles.algorithms.Transitivity
     * @see yfiles.algorithms.Trees
     * @class
     */
    export interface YGraphAdapter extends Object{}
    export class YGraphAdapter {
      /**
       * Initializes a new instance of the {@link yfiles.layout.YGraphAdapter} class.
       * <p>
       * This will create a copy of the <code>originalGraph</code> that has the same structure and makes it available via the {@link yfiles.layout.YGraphAdapter#yGraph} property.
       * </p>
       * @param originalGraph The original graph whose structure will be copied.
       */
      constructor(originalGraph:yfiles.graph.IGraph);
      /**
       * Creates an {@link yfiles.algorithms.IDataMap} that delegates to the provided {@link yfiles.collections.IMapper.<K,V>} when queried for items in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.IDataMap}s as input and that input can be obtained from
       * {@link yfiles.collections.IMapper.<K,V>} instances that provide and accept values for the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes} and {@link yfiles.layout.YGraphAdapter#getOriginalEdge edges}.
       * </p>
       * Type parameter <code>K</code>: The type of the keys in the mapper.
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param valueType The type of the values in the mapper.
       * @param keyType The type of the keys in the mapper.
       * @param mapper The mapper to direct requests to.
       * @returns An {@link } that will delegate the functionality to the <code>mapper</code>.
       * @see yfiles.layout.YGraphAdapter#createDataProvider
       * @template K,V
       */
      createDataMap<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,mapper:yfiles.collections.IMapper<K,V>):yfiles.algorithms.IDataMap;
      /**
       * Creates an {@link yfiles.algorithms.IDataProvider} that delegates to the provided delegate when queried for items in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.IDataProvider}s as input and that input can be through a
       * dynamic callback that provides the values for the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes} and {@link yfiles.layout.YGraphAdapter#getOriginalEdge edges}.
       * </p>
       * Type parameter <code>K</code>: The type of the keys in the mapper.
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param valueType The type of the values in the mapper.
       * @param keyType The type of the keys in the mapper.
       * @param getter The mapper delegate to direct requests to.
       * @returns A DataProvider that will delegate requests to the <code>getter</code>.
       * @template K,V
       */
      createDataProvider<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,getter:(key:K)=>V):yfiles.algorithms.IDataProvider;
      /**
       * Creates an {@link yfiles.algorithms.IDataProvider} that delegates to the provided {@link yfiles.collections.IMapper.<K,V>} when queried for items in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.IDataProvider}s as input and that input can be obtained
       * from {@link yfiles.collections.IMapper.<K,V>} instances that provide the values for the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes} and {@link yfiles.layout.YGraphAdapter#getOriginalEdge edges}.
       * </p>
       * Type parameter <code>K</code>: The type of the keys in the mapper.
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param valueType The type of the values in the mapper.
       * @param keyType The type of the keys in the mapper.
       * @param mapper The mapper to direct requests to.
       * @returns A DataProvider that will delegate requests to the <code>mapper</code>.
       * @template K,V
       */
      createDataProvider<K,V>(keyType:yfiles.lang.Class,valueType:yfiles.lang.Class,mapper:yfiles.collections.IMapper<K,V>):yfiles.algorithms.IDataProvider;
      /**
       * Creates an {@link yfiles.collections.IEnumerable.<T> enumerable of edges} that maps the edges from the {@link yfiles.algorithms.EdgeList} to their {@link yfiles.layout.YGraphAdapter#getOriginalEdge original} ones.
       * <p>
       * This method is useful where algorithms yield {@link yfiles.algorithms.EdgeList} instances as results and the result needs to be
       * interpreted in terms of the {@link yfiles.layout.YGraphAdapter#originalGraph}.
       * </p>
       * @param edges The edge list.
       * @returns An enumerable that contains the {@link #getOriginalEdge original} edges for the ones in the list.
       */
      createEdgeEnumerable(edges:yfiles.algorithms.EdgeList):yfiles.collections.IListEnumerable<yfiles.graph.IEdge>;
      /**
       * Creates an {@link yfiles.algorithms.EdgeList} that maps the edges from the {@link yfiles.collections.IEnumerable.<T>} to their {@link yfiles.layout.YGraphAdapter#getCopiedEdge copied} ones.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.EdgeList} instances as input and those edges are
       * available as {@link yfiles.collections.IEnumerable.<T>}s in the {@link yfiles.layout.YGraphAdapter#originalGraph}.
       * </p>
       * @param edges The edges from the {@link #originalGraph}.
       * @returns An {@link } that contains the {@link #getCopiedEdge copied} edges for the ones in the enumerable.
       */
      createEdgeList(edges:yfiles.collections.IEnumerable<yfiles.graph.IEdge>):yfiles.algorithms.EdgeList;
      /**
       * Creates an {@link yfiles.algorithms.IEdgeMap} that delegates to the provided delegate when queried for {@link yfiles.algorithms.Edge}s
       * in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.IEdgeMap}s as input and this can be mapped to a
       * corresponding dynamic callback that provides the values for the {@link yfiles.layout.YGraphAdapter#getOriginalEdge original edges}.
       * </p>
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param getter The mapper delegate to direct read requests to.
       * @returns An {@link } that will delegate reads to the <code>getter</code>.
       * @template V
       */
      createEdgeMap<V>(getter:(key:yfiles.graph.IEdge)=>V):yfiles.algorithms.IEdgeMap;
      /**
       * Creates an {@link yfiles.algorithms.IEdgeMap} that delegates to the provided {@link yfiles.collections.IMapper.<K,V>} when queried for {@link yfiles.algorithms.Edge}s in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.IEdgeMap}s as input and this can be mapped to
       * corresponding {@link yfiles.collections.IMapper.<K,V>} instances that provide and accept the values for the {@link yfiles.layout.YGraphAdapter#getOriginalEdge original edges}.
       * </p>
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param mapper The mapper to direct requests to.
       * @returns An {@link } that will delegate reads and writes to the <code>mapper</code>.
       * @template V
       */
      createEdgeMap<V>(mapper:yfiles.collections.IMapper<yfiles.graph.IEdge,V>):yfiles.algorithms.IEdgeMap;
      /**
       * Creates an {@link yfiles.collections.IMapper.<K,V>} that delegates to the provided {@link yfiles.algorithms.INodeMap} when queried for items in the
       * {@link yfiles.graph.IGraph}.
       * <p>
       * This method is useful where algorithms yield {@link yfiles.algorithms.INodeMap}s and their values should be provided using {@link yfiles.collections.IMapper.<K,V>}
       * instances that use the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes}.
       * </p>
       * Type parameter <code>T</code>: The type of the values in the mapper.
       * @param nodeMap The map to direct requests to.
       * @returns A mapper that will delegate requests to the <code>nodeMap</code>.
       * @template T
       */
      createEdgeMapper<T>(nodeMap:yfiles.algorithms.INodeMap):yfiles.collections.IMapper<yfiles.graph.INode,T>;
      /**
       * Creates an {@link yfiles.collections.IMapper.<K,V>} that delegates to the provided {@link yfiles.algorithms.IDataProvider} when queried for items in the
       * {@link yfiles.graph.IGraph}.
       * <p>
       * This method is useful where algorithms yield {@link yfiles.algorithms.IDataProvider}s and their values should be provided using
       * {@link yfiles.collections.IMapper.<K,V>} instances that use the {@link yfiles.layout.YGraphAdapter#getOriginalEdge original edges} and the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes}.
       * </p>
       * Type parameter <code>T</code>: The type of the values in the mapper.
       * @param dataProvider The provider to direct requests to.
       * @returns A mapper that will delegate requests to the <code>dataProvider</code>.
       * @template T
       */
      createMapper<T>(dataProvider:yfiles.algorithms.IDataProvider):yfiles.collections.IMapper<yfiles.graph.IModelItem,T>;
      /**
       * Creates an {@link yfiles.collections.IEnumerable.<T> enumerable of nodes} that maps the nodes from the {@link yfiles.algorithms.NodeList} to their {@link yfiles.layout.YGraphAdapter#getOriginalNode original} ones.
       * <p>
       * This method is useful where algorithms yield {@link yfiles.algorithms.NodeList} instances as results and the result needs to be
       * interpreted in terms of the {@link yfiles.layout.YGraphAdapter#originalGraph}.
       * </p>
       * @param nodes The node list.
       * @returns An enumerable that contains the {@link #getOriginalNode original} nodes for the ones in the list.
       */
      createNodeEnumerable(nodes:yfiles.algorithms.NodeList):yfiles.collections.IListEnumerable<yfiles.graph.INode>;
      /**
       * Creates a {@link yfiles.algorithms.NodeList} that maps the nodes from the {@link yfiles.collections.IEnumerable.<T>} to their {@link yfiles.layout.YGraphAdapter#getCopiedNode copied} ones.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.NodeList} instances as input and those nodes are
       * available as {@link yfiles.collections.IEnumerable.<T>}s in the {@link yfiles.layout.YGraphAdapter#originalGraph}.
       * </p>
       * @param nodes The nodes from the {@link #originalGraph}.
       * @returns A {@link } that contains the {@link #getCopiedNode copied} nodes for the ones in the enumerable.
       */
      createNodeList(nodes:yfiles.collections.IEnumerable<yfiles.graph.INode>):yfiles.algorithms.NodeList;
      /**
       * Creates an {@link yfiles.algorithms.INodeMap} that delegates to the provided delegate when queried for {@link yfiles.algorithms.Node}s
       * in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.INodeMap}s as input and this can be mapped to a
       * corresponding dynamic callback that provides the values for the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes}.
       * </p>
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param getter The mapper delegate to direct read requests to.
       * @returns An {@link } that will delegate reads to the <code>getter</code>.
       * @template V
       */
      createNodeMap<V>(getter:(key:yfiles.graph.INode)=>V):yfiles.algorithms.INodeMap;
      /**
       * Creates an {@link yfiles.algorithms.INodeMap} that delegates to the provided {@link yfiles.collections.IMapper.<K,V>} when queried for {@link yfiles.algorithms.Node}s in the {@link yfiles.layout.YGraphAdapter#yGraph}.
       * <p>
       * This method is useful where algorithms require {@link yfiles.algorithms.INodeMap}s as input and this can be mapped to
       * corresponding {@link yfiles.collections.IMapper.<K,V>} instances that provide and accept the values for the {@link yfiles.layout.YGraphAdapter#getOriginalNode original nodes}.
       * </p>
       * Type parameter <code>V</code>: The type of the values in the mapper.
       * @param mapper The mapper to direct requests to.
       * @returns An {@link } that will delegate reads and writes to the <code>mapper</code>.
       * @template V
       */
      createNodeMap<V>(mapper:yfiles.collections.IMapper<yfiles.graph.INode,V>):yfiles.algorithms.INodeMap;
      /**
       * Creates an {@link yfiles.collections.IMapper.<K,V>} that delegates to the provided {@link yfiles.algorithms.IEdgeMap} when queried for items in the
       * {@link yfiles.graph.IGraph}.
       * <p>
       * This method is useful where algorithms yield {@link yfiles.algorithms.IDataProvider}s and their values should be provided using
       * {@link yfiles.collections.IMapper.<K,V>} instances that use the {@link yfiles.layout.YGraphAdapter#getOriginalEdge original edges}.
       * </p>
       * Type parameter <code>T</code>: The type of the values in the mapper.
       * @param edgeMap The map to direct requests to.
       * @returns A mapper that will delegate requests to the <code>edgeMap</code>.
       * @template T
       */
      createNodeMapper<T>(edgeMap:yfiles.algorithms.IEdgeMap):yfiles.collections.IMapper<yfiles.graph.IEdge,T>;
      /**
       * Yields the edge that has been created as a copy in {@link yfiles.layout.YGraphAdapter#yGraph} to represent the given edge.
       * @param edge The edge for which the representation is returned.
       * @returns The edge that represents <code>edge</code> in {@link #yGraph}.
       */
      getCopiedEdge(edge:yfiles.graph.IEdge):yfiles.algorithms.Edge;
      /**
       * Yields the node that has been created as a copy in {@link yfiles.layout.YGraphAdapter#yGraph} to represent the given node.
       * @param node The node for which the representation is returned.
       * @returns The node that represents <code>node</code> in {@link #yGraph}.
       */
      getCopiedNode(node:yfiles.graph.INode):yfiles.algorithms.Node;
      /**
       * Yields the original edge from the {@link yfiles.layout.YGraphAdapter#originalGraph original IGraph} that the given edge has been created for.
       * @param edge The edge for which the original instance will be returned.
       * @returns The original instance.
       */
      getOriginalEdge(edge:yfiles.algorithms.Edge):yfiles.graph.IEdge;
      /**
       * Yields the original node from the {@link yfiles.layout.YGraphAdapter#originalGraph original IGraph} that the given node has been created for.
       * @param node The node for which the original instance will be returned.
       * @returns The original instance.
       */
      getOriginalNode(node:yfiles.algorithms.Node):yfiles.graph.INode;
      /**
       * Gets the graph instance that is created during the constructor call.
       * @type {yfiles.algorithms.Graph}
       */
      yGraph:yfiles.algorithms.Graph;
      /**
       * Gets the original graph instance.
       * @type {yfiles.graph.IGraph}
       */
      originalGraph:yfiles.graph.IGraph;
      static $class:yfiles.lang.Class;
    }
  }export namespace hierarchic{
    /**
     * This interface describes a layering constraint.
     * <p>
     * Such constraints can be given as additional input to the {@link yfiles.hierarchic.HierarchicLayout}.
     * </p>
     * @see yfiles.hierarchic.ILayerConstraintFactory
     * @interface
     */
    export interface ILayerConstraint extends Object{
      /**
       * Gets or sets the priority of a constraint.
       * <p>
       * This value is used for resolving conflicting constraints. A constraint with a higher priority is more likely to be
       * considered.
       * </p>
       * <p>
       * The priority has to be greater than <code>0</code>.
       * </p>
       * @abstract
       * @type {number}
       */
      priority:number;
    }
    var ILayerConstraint:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides methods for creating layering constraints.
     * @interface
     */
    export interface ILayerConstraintFactory extends Object{
      /**
       * Adds a constraint that forces the {@link yfiles.algorithms.Node} with ID <code>aboveId</code> to lie at least <code>minDistance</code> layers above the {@link yfiles.algorithms.Node}
       * with ID <code>referenceId</code> with a given weight penalty for larger layer differences.
       * <p>
       * The minimum distance includes potentially empty layers that are removed by the {@link yfiles.hierarchic.ILayerer} instance. In
       * that case, the actual layer difference may be smaller than the minimum distance.
       * </p>
       * @param referenceId the ID of the reference node
       * @param aboveId the ID of the node that should lie above
       * @param minDistance the minimum layer distance between the node and its reference node
       * @param [weight=0] the weight penalty for larger layer differences
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeAboveConstraint(referenceId:Object,aboveId:Object,minDistance:number,weight?:number):yfiles.hierarchic.ILayerConstraint;
      /**
       * Adds a constraint that forces the {@link yfiles.algorithms.Node} with ID <code>aboveId</code> to lie above the {@link yfiles.algorithms.Node} with ID
       * <code>referenceId</code>.
       * @param referenceId the ID of the reference node
       * @param aboveId the ID of the node that should lie above
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeAboveConstraint(referenceId:Object,aboveId:Object):yfiles.hierarchic.ILayerConstraint;
      /**
       * Adds a constraint that places a {@link yfiles.algorithms.Node} on the bottom layer.
       * @param nodeId the ID of the node that should lie on the bottom layer
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeAtBottomConstraint(nodeId:Object):yfiles.hierarchic.ILayerConstraint;
      /**
       * Adds a constraint that places a {@link yfiles.algorithms.Node} on the topmost layer.
       * @param nodeId the ID of the node that should lie on the top layer
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeAtTopConstraint(nodeId:Object):yfiles.hierarchic.ILayerConstraint;
      /**
       * Adds a constraint that forces the {@link yfiles.algorithms.Node} with ID <code>belowId</code> to lie at least <code>minDistance</code> layers below the {@link yfiles.algorithms.Node}
       * with ID <code>referenceId</code> with a given weight penalty for larger layer differences.
       * <p>
       * The minimum distance includes potentially empty layers that are removed by the {@link yfiles.hierarchic.ILayerer} instance. In
       * that case, the actual layer difference may be smaller than the minimum distance.
       * </p>
       * @param referenceId the ID of the reference node
       * @param belowId the ID of the node that should lie below
       * @param minDistance the minimum layer distance between the node and its reference node
       * @param [weight=0] the weight penalty for larger layer differences
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeBelowConstraint(referenceId:Object,belowId:Object,minDistance:number,weight?:number):yfiles.hierarchic.ILayerConstraint;
      /**
       * Adds a constraint that forces a {@link yfiles.algorithms.Node} with ID <code>belowId</code> to lie below the {@link yfiles.algorithms.Node} with ID
       * <code>referenceId</code>.
       * @param referenceId the ID of the reference node
       * @param belowId the ID of the node that should lie below
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeBelowConstraint(referenceId:Object,belowId:Object):yfiles.hierarchic.ILayerConstraint;
      /**
       * Adds a constraint that forces the {@link yfiles.algorithms.Node} with ID <code>sameLayerId</code> to lie in the same layer as the {@link yfiles.algorithms.Node}
       * with ID <code>referenceId</code>.
       * @param referenceId the ID of the reference node
       * @param sameLayerId the ID of the node that should lie in the same layer
       * @returns a {@link } object that represents the constraint
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      addPlaceNodeInSameLayerConstraint(referenceId:Object,sameLayerId:Object):yfiles.hierarchic.ILayerConstraint;
      /**
       * Disposes this factory.
       * <p>
       * This method should be called when this factory is not needed anymore, i.e. after the layout has been calculated. Calling
       * this method also clears all constraints.
       * </p>
       * @abstract
       */
      dispose():void;
      /**
       * Clears all constraints for a given {@link yfiles.algorithms.Node}.
       * @param nodeId the ID of the node for which all constraints should be cleared
       * @see yfiles.hierarchic.ILayerConstraint
       * @abstract
       */
      removeConstraints(nodeId:Object):void;
      /**
       * Gets a token that allows to bind a constraint factory to a graph instance after creation.
       * <p>
       * This method should only be used if the constraint factory is not bound to a graph instance initially. It allows to bind
       * this factory to a graph instance after creation. See the factory methods that create instances of this interface for a
       * description.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#createLayerConstraintFactory
       * @abstract
       * @type {Object}
       */
      memento:Object;
    }
    var ILayerConstraintFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface specifies the general contract for factory classes that can be used to associate sequence constraints
     * with a graph.
     * <p>
     * Sequence constraints affect the sequence of nodes within a {@link yfiles.hierarchic.ILayer layer}.
     * </p>
     * <p>
     * A {@link yfiles.hierarchic.ISequenceConstraintFactory} has to be {@link yfiles.hierarchic.ISequenceConstraintFactory#dispose disposed of} after use. Disposing of the factory will also
     * remove all constraints previously specified for the factory's associated graph.
     * </p>
     * @interface
     */
    export interface ISequenceConstraintFactory extends Object{
      /**
       * Adds a constraint that forces the {@link yfiles.algorithms.Node} or {@link yfiles.algorithms.Edge} with id <code>after</code> to lie after the {@link yfiles.algorithms.Node}
       * or {@link yfiles.algorithms.Edge} with id <code>reference</code>.
       * @param referenceId the ID of the reference node or edge
       * @param afterId the ID of the node or edge that should be placed after the reference element
       * @abstract
       */
      addPlaceAfterConstraint(referenceId:Object,afterId:Object):void;
      /**
       * Adds a constraint that places a {@link yfiles.algorithms.Node} or {@link yfiles.algorithms.Edge} at the start of the sequence.
       * @param id the ID of the node or edge that should be placed at the start
       * @abstract
       */
      addPlaceAtHeadConstraint(id:Object):void;
      /**
       * Adds a constraint that places a {@link yfiles.algorithms.Node} or {@link yfiles.algorithms.Edge} at the end of the sequence.
       * @param id the ID of the node or edge that should be placed at the end
       * @abstract
       */
      addPlaceAtTailConstraint(id:Object):void;
      /**
       * Adds a constraint that forces the {@link yfiles.algorithms.Node} or {@link yfiles.algorithms.Edge} with id <code>before</code> to lie before the {@link yfiles.algorithms.Node}
       * or {@link yfiles.algorithms.Edge} with id <code>reference</code>.
       * @param referenceId the ID of the reference node or edge
       * @param beforeId the ID of the node or edge that should be placed before the reference element
       * @abstract
       */
      addPlaceBeforeConstraint(referenceId:Object,beforeId:Object):void;
      /**
       * Disposes of the {@link yfiles.hierarchic.ISequenceConstraintFactory}.
       * <p>
       * This method should be called when the factory is not needed anymore, i.e. after the layout has been calculated. Calling
       * this method also clears all constraints.
       * </p>
       * @abstract
       */
      dispose():void;
      /**
       * Gets a token that allows to bind a constraint factory to a graph instance after creation.
       * <p>
       * This method should only be used if the constraint factory is not bound to a graph instance initially. It allows to bind
       * this factory to a graph instance after creation.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createSequenceConstraintFactory
       * @abstract
       * @type {Object}
       */
      memento:Object;
    }
    var ISequenceConstraintFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * {@link yfiles.hierarchic.AsIsLayerer} assigns nodes to layers by analyzing already existing node coordinates.
     * <p>
     * Nodes whose bounding boxes overlap in the main layout direction are assigned to the same layer. To influence to which
     * amount the boxes need to overlap, the nodes can temporarily be
     * {@link yfiles.hierarchic.AsIsLayerer#nodeScalingFactor scaled}. Alternatively, a minimum and maximum size or a halo can be specified for the nodes.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[1.64,133.62,[[11.31,157.97,10.67,18.7,"1"]]],[51.6,4.78,[[61.27,29.13,10.67,18.7,"2"]]],[190,14.75,[[199.66,39.1,10.67,18.7,"3"]]],[226.76,148.06,[[236.42,172.41,10.67,18.7,"4"]]],[108.72,221.62,[[118.38,245.97,10.67,18.7,"5"]]]],"e":[[0,1],[1,2],[3,2],[0,4],[4,3]],"vp":[1.0,4.0,256.0,248.0]}}
     * </p>
     * <p>
     * Initial graph
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[20,50,[[29.66,74.35,10.67,18.7,"1"]]],[20,0,[[29.66,24.35,10.67,18.7,"2"]]],[100,0,[[109.66,24.35,10.67,18.7,"3"]]],[100,50,[[109.66,74.35,10.67,18.7,"4"]]],[60,110,[[69.66,134.35,10.67,18.7,"5"]]]],"e":[[0,1,0,-15,0,15],[1,2,15,0,-15,0],[3,2,0,-15,0,15],[0,4,0,15,-7.5,-15,[35,95,67.5,95]],[4,3,7.5,-15,0,15,[82.5,95,115,95]]],"vp":[20.0,0.0,110.0,140.0]}}
     * </p>
     * <p>
     * Hierarchic layout when the initial drawing is used as sketch
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[50,0,[[59.66,24.35,10.67,18.7,"1"]]],[20,60,[[29.66,84.35,10.67,18.7,"2"]]],[27.5,170,[[37.16,194.35,10.67,18.7,"3"]]],[80,110,[[89.66,134.35,10.67,18.7,"4"]]],[80,60,[[89.66,84.35,10.67,18.7,"5"]]]],"e":[[0,1,-7.5,15,0,-15,[57.5,45,35,45]],[1,2,0,15,-7.5,-15],[3,2,0,15,7.5,-15,[95,155,50,155]],[0,4,7.5,15,0,-15,[72.5,45,95,45]],[4,3,0,15,0,-15]],"vp":[20.0,0.0,90.0,200.0]}}
     * </p>
     * <p>
     * Hierarchic layout when the initial drawing is not used as sketch
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface AsIsLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class AsIsLayerer {
      /**
       * Creates an instance of {@link yfiles.hierarchic.AsIsLayerer} with default settings.
       */
      constructor();
      /**
       * Assigns all nodes of the graph to layers by analyzing already existing node coordinates and adds them to the {@link yfiles.hierarchic.ILayers}
       * instance.
       * <p>
       * Nodes whose bounding boxes overlap in the main layout direction are assigned to the same layer. To influence to which
       * amount the boxes need to overlap, the nodes can temporarily be {@link yfiles.hierarchic.AsIsLayerer#nodeScalingFactor scaled}.
       * </p>
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for querying information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Callback used for calculating the lower (max) value of a given node.
       * @param graph the graph
       * @param node the node
       * @returns the lower (max) of a given node
       * @protected
       */
      getMax(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):number;
      /**
       * Callback used for calculating the upper (min) value of a given node.
       * @param graph the graph
       * @param node the node
       * @returns the upper (min) value of a given node
       * @protected
       */
      getMin(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):number;
      /**
       * Gets or sets the scaling factor used for scaling the size of the nodes.
       * <p>
       * Nodes are being scaled from their center. This can be used for more fuzzy layering.
       * </p>
       * <p>
       * The scaling factor can take positive values. Factors smaller than <code>1</code> only consider large overlaps of the nodes' bounding
       * boxes in the main layout direction. Factors larger than <code>1</code> also combine nodes with some distance between them.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is less than <code>0</code>
       * @see yfiles.hierarchic.AsIsLayerer#minimumNodeSize
       * @see yfiles.hierarchic.AsIsLayerer#maximumNodeSize
       * @see yfiles.hierarchic.AsIsLayerer#nodeHalo
       * @type {number}
       */
      nodeScalingFactor:number;
      /**
       * Gets or sets the maximum size of a node used for calculating layer overlaps.
       * <p>
       * This can be used for more fuzzy layering.
       * </p>
       * <p>
       * The maximum size can take positive values.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is less than <code>0</code>
       * @see yfiles.hierarchic.AsIsLayerer#nodeScalingFactor
       * @see yfiles.hierarchic.AsIsLayerer#nodeHalo
       * @type {number}
       */
      maximumNodeSize:number;
      /**
       * Gets or sets the minimum size of a node used for calculating layer overlaps.
       * <p>
       * This can be used for more fuzzy layering.
       * </p>
       * <p>
       * The minimum size can take positive values.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is less than <code>0</code>
       * @see yfiles.hierarchic.AsIsLayerer#nodeScalingFactor
       * @see yfiles.hierarchic.AsIsLayerer#nodeHalo
       * @type {number}
       */
      minimumNodeSize:number;
      /**
       * Gets or sets the size of the halo around a node, that means the insets that are used for calculating layer overlaps.
       * <p>
       * If the value of the halo is positive, it specifies a border around a node. In case it is negative, it is considered as
       * an inset that defines a smaller area inside the node.
       * </p>
       * <p>
       * This can be used for more fuzzy layering.
       * </p>
       * @see yfiles.hierarchic.AsIsLayerer#nodeScalingFactor
       * @see yfiles.hierarchic.AsIsLayerer#minimumNodeSize
       * @see yfiles.hierarchic.AsIsLayerer#maximumNodeSize
       * @type {number}
       */
      nodeHalo:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class calculates the sequence of the nodes within each layer using the nodes' coordinates.
     * <p>
     * For {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}
     * {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}, the order corresponds to that obtain when sorting the nodes according to
     * their x-coordinate.
     * </p>
     * <p>
     * An instance of this class can be set to {@link yfiles.hierarchic.HierarchicLayout} using method {@link yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer} and {@link yfiles.hierarchic.HierarchicLayout#fromScratchSequencer} respectively.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[111.8,0,[[121.47,24.35,10.67,18.7,"1"]]],[0,36.33,[[9.66,60.68,10.67,18.7,"2"]]],[180.9,95.11,[[190.56,119.46,10.67,18.7,"3"]]],[0,153.88,[[9.66,178.23,10.67,18.7,"4"]]],[111.8,190.21,[[121.47,214.56,10.67,18.7,"5"]]]],"e":[[0,1],[0,3],[1,3],[0,2],[2,4],[4,3]],"vp":[0.0,0.0,211.0,221.0]}} Initial graph that will be used as sketch {@graph {"ann":{"s":[30,30],"d":1},"n":[[50,0,[[59.66,24.35,10.67,18.7,"1"]]],[20,60,[[29.66,84.35,10.67,18.7,"2"]]],[80,100,[[89.66,124.35,10.67,18.7,"3"]]],[42.5,145,[[52.16,169.35,10.67,18.7,"4"]]],[72.5,205,[[82.16,229.35,10.67,18.7,"5"]]]],"e":[[0,1,-10,15,0,-15,[55,45,35,45]],[0,3,0,15,7.5,-15],[1,3,0,15,-7.5,-15,[35,130,50,130]],[0,2,10,15,0,-15,[75,45,95,45]],[2,4,0,15,7.5,-15],[4,3,-7.5,-15,0,15,[80,190,57.5,190]]],"vp":[20.0,0.0,90.0,235.0]}} Hierarchic layout when the initial graph is used as sketch
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ISequencer}
     */
    export interface AsIsSequencer extends Object,yfiles.hierarchic.ISequencer{}
    export class AsIsSequencer {
      /**
       * Creates a new {@link yfiles.hierarchic.AsIsSequencer} instance.
       */
      constructor();
      /**
       * Calculates the sequence of the nodes within each layer using the nodes' coordinates.
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayoutCore} and finally writes back the calculated sequence using
       * method {@link yfiles.hierarchic.ILayer#setNodeOrder}.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance that defines the layering structure
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @param itemFactory the {@link } used for creating and destroying helper structures
       */
      sequenceNodeLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a wrapper {@link yfiles.hierarchic.ILayerer} implementation that delegates the actual layering to a delegate {@link yfiles.hierarchic.ILayerer}
     * object.
     * <p>
     * If the graph consists of multiple components, they will be identified and each of the components will be layered
     * separately using the delegate {@link yfiles.hierarchic.ILayerer} instance. After that they will all be merged such that the
     * desired aspect ratio is fulfilled best.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[60,30],"d":1},"styles":{"6":{"f":"#ccccff"}},"n":[[110,160],[20,160],[20,220],[110,220],[20,280],[252.5,160,6],[305,220,6],[380,280,6],[200,220,6],[290,280,6],[290,330,6],[380,330,6],[200,280,6],[27.5,0,180,80],[87.5,100]],"e":[[14,0,15,15,0,-15,[132.5,145,140,145]],[14,1,-15,15,0,-15,[102.5,145,50,145]],[1,2,0,15,0,-15],[0,3,0,15,0,-15],[2,4,0,15,0,-15],[5,6,15,15,0,-15,[297.5,205,335,205]],[6,7,15,15,0,-15,[350,265,410,265]],[5,8,-15,15,0,-15,[267.5,205,230,205]],[6,9,-15,15,0,-15],[9,10,0,15,0,-15],[7,11,0,15,0,-15],[8,12,0,15,0,-15],[13,14,0,40,0,-15]],"vp":[20.0,0.0,420.0,360.0]}} Sample output of {@link yfiles.hierarchic.AspectRatioComponentLayerer} with default settings on a graph with two connected
     * components.
     * </p>
     * @see yfiles.hierarchic.AspectRatioComponentLayerer#desiredAspectRatio
     * @see yfiles.hierarchic.MultiComponentLayerer
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface AspectRatioComponentLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class AspectRatioComponentLayerer {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.AspectRatioComponentLayerer} using the given delegate {@link yfiles.hierarchic.ILayerer}
       * instance.
       * @param singleComponentLayerer
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.ILayerer} is <code>null</code>
       */
      constructor(singleComponentLayerer:yfiles.hierarchic.ILayerer);
      /**
       * Assigns all nodes of the graph to layers and registers them to the {@link yfiles.hierarchic.ILayers} instance based on the given
       * aspect ratio.
       * <p>
       * If the graph consists of multiple components, they will be identified and each of the components will be layered
       * separately using the delegate {@link yfiles.hierarchic.ILayerer} instance. After that, they will all be merged such that the
       * desired aspect ratio is best fulfilled.
       * </p>
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for query information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Gets or sets the delegate {@link yfiles.hierarchic.ILayerer} instance.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.ILayerer} is <code>null</code>
       * @type {yfiles.hierarchic.ILayerer}
       */
      singleComponentLayerer:yfiles.hierarchic.ILayerer;
      /**
       * Gets or sets whether or not the size of the nodes should be considered.
       * <p>
       * If this option is disabled, all nodes are considered to be of equal size. Hence, the given aspect ratio specifies the
       * ratio between the number of nodes within a layer and the overall number of layers.
       * </p>
       * @type {boolean}
       */
      considerNodeSize:boolean;
      /**
       * Gets or sets the desired aspect ratio.
       * <p>
       * If the graph consists of multiple components, they will be identified and each of the components will be layered
       * separately using the delegate {@link yfiles.hierarchic.ILayerer} instance. After that, they will all be merged such that the
       * desired aspect ratio is best fulfilled.
       * </p>
       * <p>
       * The given aspect ratio should be greater or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the aspect ratio is negative
       * @type {number}
       */
      desiredAspectRatio:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.hierarchic.BFSLayerer} uses a breadth first search for assigning layers to the nodes.
     * <p>
     * The nodes of the first layer can be freely selected (see
     * {@link yfiles.hierarchic.BFSLayerer#CORE_NODES_DP_KEY}). The nodes that belong to a subsequent layer are determined as follows:
     * Add all yet unassigned nodes to the new layer that are connected to nodes already assigned. As a consequence, all
     * connected nodes will be at most one layer apart. Also, edges between nodes that belong to the same layer are possible.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[46.17,0,[[55.83,24.35,10.67,18.7,"1"]]],[100,70,[[109.66,94.35,10.67,18.7,"2"]]],[0,70,[[9.66,94.35,10.67,18.7,"3"]]],[211.77,0,[[221.43,24.35,10.67,18.7,"6"]]],[211.77,70,[[221.43,94.35,10.67,18.7,"7"]]],[100,0,[[109.66,24.35,10.67,18.7,"8"]]],[50,70,[[56.33,94.35,17.35,18.7,"12"]]],[3.17,140,[[12.83,164.35,10.67,18.7,"8"]]],[113.17,140,[[122.83,164.35,10.67,18.7,"9"]]]],"e":[[5,1,0,15,7.5,-15,[115,50,122.5,50]],[0,1,10,15,-7.5,-15,[71.17,50,107.5,50]],[0,2,-10,15,0,-15,[51.17,50,15,50]],[4,1,-15,0,15,0],[3,4,0,15,0,-15],[0,6,-0,15,0,-15,[61.17,50,65,50]],[6,7,-7.5,15,7.5,-15,[57.5,120,25.67,120]],[6,8,7.5,15,0,-15,[72.5,120,128.17,120]],[8,7,-15,0,15,0],[2,7,0,15,-7.5,-15,[15,120,10.67,120]]],"vp":[0.0,0.0,242.0,170.0]}}
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface BFSLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class BFSLayerer {
      /**
       * Creates an instance of {@link yfiles.hierarchic.BFSLayerer}.
       */
      constructor();
      /**
       * Assigns all nodes of the graph to layers and adds them to the {@link yfiles.hierarchic.ILayers} instance.
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for query information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Data provider key for identifying the core nodes of the BFS layering
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} registered with this key should provide boolean values for the nodes of the graph.
       * The boolean value determines whether or not a node is to be placed in the first layer. If there are no nodes marked as
       * core nodes, then nodes with in-degree <code>0</code> are considered to be the core nodes.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static CORE_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a {@link yfiles.hierarchic.ILayerer} implementation that observes relative and absolute layering constraints
     * defined by the {@link yfiles.hierarchic.ILayerConstraintFactory layer constraint factory}.
     * <p>
     * Furthermore, it can handle the edge directedness as specified by {@link yfiles.algorithms.IDataProvider} key
     * {@link yfiles.hierarchic.HierarchicLayout#EDGE_DIRECTEDNESS_DP_KEY}.
     * </p>
     * <p>
     * For a given graph instance, the layer constraint factory can be created using method
     * {@link yfiles.hierarchic.HierarchicLayout#createLayerConstraintFactory}. The {@link yfiles.hierarchic.ILayerer} can be used for both
     * common layering and layering of incremental nodes.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[60,30],"d":1},"styles":{"6":{"f":"#ff0000"},"7":{"f":"#0099ff"}, "8":{"f":"#00cc66"}},"n":[[120,0,6,[[144.66,24.35,10.67,18.7,"1"]]],[10,50,7,[[34.66,74.35,10.67,18.7,"2"]]],[120,50,7,[[144.66,74.35,10.67,18.7,"3"]]],[280,125,8,[[304.66,149.35,10.67,18.7,"7"]]],[10,125,8,[[34.66,149.35,10.67,18.7,"4"]]],[100,125,8,[[124.66,149.35,10.67,18.7,"5"]]],[190,125,8,[[214.66,149.35,10.67,18.7,"6"]]]],"e":[[2,0,0,-15,0,15],[2,1,-30,0,30,0],[3,2,0,-15,22.5,15,[310,95,172.5,95]],[2,4,-22.5,15,0,-15,[127.5,90,40,90]],[2,5,-7.5,15,0,-15,[142.5,100,130,100]],[2,6,7.5,15,0,-15,[157.5,105,220,105]]],"vp":[10.0,0.0,330.0,155.0]}} Sample output of applying {@link yfiles.hierarchic.ConstraintIncrementalLayerer}. The red colored node should reside in the
     * topmost layer, blue nodes in the medium layer while green nodes in the bottommost layer. {@graph {"ann":{"s":[60,30],"d":1},"styles":{"6":{"f":"#0099ff"},"7":{"f":"#e3e3ff", "s":"#e3e3ff"}, "8":{"f":"#00cc66"}},"n":[[330,136.5,380,67,7],[330,206.5,380,127,7],[330,337,380,80,7],[490,156.5,[[514.66,180.85,10.67,18.7,"1"]]],[350,226.5,6,[[374.66,250.85,10.67,18.7,"2"]]],[440,226.5,6,[[464.66,250.85,10.67,18.7,"3"]]],[450,286.5,6,[[474.66,310.85,10.67,18.7,"4"]]],[415,346.5,8,[[439.66,370.85,10.67,18.7,"5"]]],[505,346.5,8,[[529.66,370.85,10.67,18.7,"6"]]],[595,346.5,8,[[619.66,370.85,10.67,18.7,"7"]]]],"e":[[3,4,-25,15,0,-15,[495,196.5,380,196.5]],[3,5,-15,15,0,-15,[505,206.5,470,206.5]],[3,6,-5,15,15,-15,[515,266.5,495,266.5]],[5,6,0,15,-15,-15,[470,266.5,465,266.5]],[3,7,5,15,0,-15,[525,326.5,445,326.5]],[3,8,15,15,0,-15],[3,9,25,15,0,-15,[545,196.5,625,196.5]]],"vp":[330.0,136.0,380.0,281.0]}} Sample output of applying {@link yfiles.hierarchic.ConstraintIncrementalLayerer}
     * on a graph with a
     * {@link yfiles.layout.PartitionGrid}. Same colors represent nodes that belong to the same partition grid cell and should be
     * layered with respect to the partition grid cell to which they belong.
     * </p>
     * @see yfiles.hierarchic.ILayerConstraintFactory
     * @see yfiles.hierarchic.HierarchicLayout
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface ConstraintIncrementalLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class ConstraintIncrementalLayerer {
      /**
       * Creates a new {@link yfiles.hierarchic.ConstraintIncrementalLayerer} with the given {@link yfiles.hierarchic.ILayerer} instance.
       * @param coreLayerer the given {@link } instance
       */
      constructor(coreLayerer:yfiles.hierarchic.ILayerer);
      /**
       * Assigns all nodes of the graph to layers and adds them to the {@link yfiles.hierarchic.ILayers} instance based on relative and
       * absolute layering constraints defined by the given layering constraints.
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for query information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Checks if the current setting of constraints is consistent (i.e.
       * <p>
       * the induced constraint graph has no cycles).
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the constraint graph is inconsistent
       * @protected
       */
      checkConstraints():void;
      /**
       * Data provider key for assigning additional edge weights
       * <p>
       * The {@link yfiles.hierarchic.ILayerer} tries to maintain edges with higher weights short.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static ADDITIONAL_EDGE_WEIGHT_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Gets or sets whether or not same-layer edges can be created by this {@link yfiles.hierarchic.ILayerer} instance.
       * <p>
       * This only concerns edges between nodes that have no hard constraints that will force a same-layer edge (i.e. a
       * same-layer constraint).
       * </p>
       * @type {boolean}
       */
      allowSameLayerEdges:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a default {@link yfiles.hierarchic.IDrawingDistanceCalculator} implementation that is used by {@link yfiles.hierarchic.HierarchicLayoutCore}
     * to configure the {@link yfiles.hierarchic.INodePlacer}.
     * @class
     * @implements {yfiles.hierarchic.IDrawingDistanceCalculator}
     */
    export interface DefaultDrawingDistanceCalculator extends Object,yfiles.hierarchic.IDrawingDistanceCalculator{}
    export class DefaultDrawingDistanceCalculator {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator} with default settings.
       */
      constructor();
      /**
       * 
       * @param graph
       * @param layers
       * @param ldp
       */
      dispose(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Determines the minimum distance between two {@link yfiles.algorithms.Node}s of the same layer.
       * @param graph the input graph
       * @param layer the {@link } object that will be used during subsequent calls
       * @param ldp the {@link } used for querying information
       * @param left the left of the two {@link }s whose minimum distance is to be determined or <code>null</code> if only the left border
       * of the right {@link } is of interest
       * @param right the right of the two {@link }s whose minimum distance is to be determined or <code>null</code> if only the right border
       * of the left {@link } is of interest
       * @returns the minimum distances between {@link }s of same layer
       */
      getMinDistance(graph:yfiles.layout.LayoutGraph,layer:yfiles.hierarchic.ILayer,ldp:yfiles.hierarchic.ILayoutDataProvider,left:yfiles.algorithms.Node,right:yfiles.algorithms.Node):number;
      /**
       * 
       * @param graph
       * @param layers
       * @param ldp
       */
      initialize(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Gets or sets whether or not an adaptive minimum edge distance is used.
       * <p>
       * If this option is enabled the minimum distance between two adjacent edges is the minimum of the current minimum edge
       * distance and the distance of the edges' source/target points.
       * </p>
       * @type {boolean}
       */
      adaptiveMinimumEdgeDistanceEnabled:boolean;
      /**
       * Gets or sets whether or not the compaction option is enabled.
       * @type {boolean}
       */
      compaction:boolean;
      /**
       * Gets or sets the minimum distance between two {@link yfiles.algorithms.Node}s.
       * <p>
       * Minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      nodeToNodeDistance:number;
      /**
       * Gets or sets the minimum distance between a {@link yfiles.algorithms.Node} and an (non-adjacent) {@link yfiles.algorithms.Edge}.
       * <p>
       * Minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      nodeToEdgeDistance:number;
      /**
       * Gets or sets the minimum distance between two edges.
       * <p>
       * Minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      edgeToEdgeDistance:number;
      /**
       * Gets or sets whether or not to use an optimized minimum distance calculation for swimlane layouts.
       * <p>
       * If set to <code>true</code> this instance will report <code>0.0d</code> as the minimum distance between two nodes if they belong to different
       * swimlanes. This avoids unwanted feedback between different swimlanes during node placement.
       * </p>
       * @type {boolean}
       */
      optimizeSwimlaneDistances:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a {@link yfiles.hierarchic.ISequencer} implementation that performs the second phase of the Sugiyama algorithm.
     * <p>
     * It minimizes the crossings in the diagram by using either the barycentric or median heuristic.
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ISequencer}
     */
    export interface DefaultLayerSequencer extends Object,yfiles.hierarchic.ISequencer{}
    export class DefaultLayerSequencer {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.DefaultLayerSequencer}.
       */
      constructor();
      /**
       * 
       * @param graph
       * @param glayers
       * @param ldp
       * @param itemFactory
       */
      sequenceNodeLayers(graph:yfiles.layout.LayoutGraph,glayers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Gets or sets whether or not to apply an additional crossing minimization heuristic.
       * <p>
       * Activating this heuristic can reduce the overall number of edge crossings. On the other hand, it may increase the
       * running time.
       * </p>
       * @type {boolean}
       */
      transposition:boolean;
      /**
       * Gets or sets whether or not an additional crossing minimization heuristic should be used in the presence of grouped
       * graphs.
       * <p>
       * Activating this heuristic can reduce the overall number of edge crossings in grouped graphs. On the other hand, it may
       * increase running time.
       * </p>
       * @type {boolean}
       */
      groupTransposition:boolean;
      /**
       * Gets or sets the weight heuristic that should be used.
       * @throws {Stubs.Exceptions.ArgumentError} if the constant is unknown
       * @type {yfiles.hierarchic.WeightHeuristic}
       */
      weightHeuristic:yfiles.hierarchic.WeightHeuristic;
      /**
       * Gets or sets the time limit (in milliseconds) set for this sequencer per execution.
       * <p>
       * Values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative.
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the number of randomized rounds that this algorithm performs, if there was no optimal solution.
       * <p>
       * Values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if a negative value is given
       * @type {number}
       */
      randomizationRounds:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A default implementation of a {@link yfiles.hierarchic.ILayeredComponentsMerger} that provides simple default behaviors.
     * <p>
     * Instances of this class are used internally by {@link yfiles.hierarchic.HierarchicLayoutCore} and {@link yfiles.hierarchic.MultiComponentLayerer}
     * when e.g. the graph is grouped or it consists of more than one connected component.
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayeredComponentsMerger}
     */
    export interface DefaultLayeredComponentsMerger extends Object,yfiles.hierarchic.ILayeredComponentsMerger{}
    export class DefaultLayeredComponentsMerger {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.DefaultLayeredComponentsMerger} using the given merging policy.
       * @param policy one of the predefined merging constants
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown policy is given
       */
      constructor(policy:yfiles.hierarchic.MergingPolicy);
      /**
       * Creates a new instance of {@link yfiles.hierarchic.DefaultLayeredComponentsMerger} with default settings.
       */
      constructor();
      /**
       * 
       * @param graph
       * @param ldp
       * @param srcLayers
       * @param targetLayers
       */
      merge(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,srcLayers:yfiles.hierarchic.ILayers,targetLayers:yfiles.hierarchic.ILayers):void;
      /**
       * Gets or sets the merging policy constant as defined in {@link yfiles.hierarchic.DefaultLayeredComponentsMerger}.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown policy is given
       * @type {yfiles.hierarchic.MergingPolicy}
       */
      policy:yfiles.hierarchic.MergingPolicy;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a default {@link yfiles.hierarchic.IPortAllocator} implementation.
     * <p>
     * It is used by {@link yfiles.hierarchic.HierarchicLayoutCore} after the sequencing phase.
     * </p>
     * @class
     * @implements {yfiles.hierarchic.IPortAllocator}
     */
    export interface DefaultPortAllocator extends Object,yfiles.hierarchic.IPortAllocator{}
    export class DefaultPortAllocator {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.DefaultPortAllocator} with default settings.
       * @see yfiles.hierarchic.DefaultPortAllocator#defaultPortBorderGapRatio
       */
      constructor();
      /**
       * Assigns source and target port coordinates to each edge of the graph.
       * <p>
       * This method assigns:
       * </p>
       * <ul>
       * <li>{@link yfiles.layout.PortSide#ANY} ports to appropriate sides.</li>
       * <li>{@link yfiles.layout.PortConstraint#strong Weak} ports to appropriate positions between strong ones.</li>
       * </ul>
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayoutCore} after the sequencing phase.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance containing the layering structure
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @param itemFactory the {@link } used for creating and destroying helper structures
       */
      assignPorts(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Callback method used for determining the port border gap for each node and side.
       * <p>
       * It may be overridden for a custom calculation of port border gap on a given node.
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @param node the given node
       * @param sideIndex the zero-based (top is 0) clock-wise index of the side of the node (for top-to-bottom layouts)
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges/ports that connect to this side
       * @returns the absolute gap on both sides of the ports
       * @see yfiles.hierarchic.DefaultPortAllocator#getPortDistanceDelta
       * @protected
       */
      getPortBorderGap(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,node:yfiles.algorithms.Node,sideIndex:number,sideLength:number,edgeCount:number):number;
      /**
       * Callback method used for determining the port border gap ratio.
       * <p>
       * See {@link yfiles.hierarchic.DefaultPortAllocator#defaultPortBorderGapRatio} for an explanation.
       * </p>
       * <p>
       * It may be overridden for a custom calculation of port border gap ratio on a given node.
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @param node the given node
       * @param sideIndex the zero-based (top is 0) clock-wise index of the side of the node (for top-to-bottom layouts)
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges/ports that connect to this side
       * @returns the border gap ratio
       * @protected
       */
      getPortBorderGapRatio(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,node:yfiles.algorithms.Node,sideIndex:number,sideLength:number,edgeCount:number):number;
      /**
       * Callback method used for determining the distance between two adjacent ports.
       * <p>
       * It may be overridden for a custom calculation of the distance between two adjacent ports..
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @param node the given node
       * @param sideIndex the zero-based (top is 0) clock-wise index of the side of the node (for top-to-bottom layouts)
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges/ports that connect to this side
       * @param portBorderGap the previously calculated port border gap
       * @returns the absolute distance between two adjacent ports
       * @protected
       */
      getPortDistanceDelta(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,node:yfiles.algorithms.Node,sideIndex:number,sideLength:number,edgeCount:number,portBorderGap:number):number;
      /**
       * Gets or sets the border gap ratio for the default port.
       * <p>
       * This ratio determines the gap between a corner of the node and the first assigned port.
       * </p>
       * <p>
       * The ratio should be greater than <code>0</code>.
       * </p>
       * <p>
       * A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
       * </p>
       * <p>
       * A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
       * the node and the first port is half as wide as the distance between two adjacent ports.
       * </p>
       * <p>
       * A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given ratio is negative
       * @type {number}
       */
      defaultPortBorderGapRatio:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used by {@link yfiles.hierarchic.HierarchicLayoutCore} during the various phases to provide routing details for
     * the edges of the graph.
     * <p>
     * An {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance can be specified individually for single edges using a {@link yfiles.algorithms.IDataProvider}
     * that returns an {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance for each edge of the graph, or <code>null</code> if no {@link yfiles.hierarchic.EdgeLayoutDescriptor}
     * is bound to an edge. The {@link yfiles.algorithms.IDataProvider} is registered with the graph using key
     * {@link yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
     * </p>
     * <p>
     * This class is designed as a class to allow future additions of new getter methods.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
     * @see yfiles.hierarchic.NodeLayoutDescriptor
     * @class
     */
    export interface EdgeLayoutDescriptor extends Object{}
    export class EdgeLayoutDescriptor {
      /**
       * Creates a new instance of an {@link yfiles.hierarchic.EdgeLayoutDescriptor} with the default values.
       */
      constructor();
      /**
       * Creates a copy of this {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance.
       * @returns a copy of this {@link } instance
       */
      createCopy():yfiles.hierarchic.EdgeLayoutDescriptor;
      /**
       * Gets or sets the minimum length of the first segment of the edge path (at the source node).
       * <p>
       * The minimum length should be greater than <code>0</code>.
       * </p>
       * <p>
       * If the value is <code>0.0d</code> the first segment does not need to be orthogonal.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length of the first segment is negative
       * @type {number}
       */
      minimumFirstSegmentLength:number;
      /**
       * Gets or sets the minimum length of the last segment of the edge path (at the target node).
       * <p>
       * The minimum length should be greater than <code>0</code>.
       * </p>
       * <p>
       * If the value is <code>0.0d</code> the last segment does not need to be orthogonal.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length of the last segment is negative
       * @type {number}
       */
      minimumLastSegmentLength:number;
      /**
       * Gets or sets the minimum length of an edge.
       * <p>
       * If this is used for polyline routing, it describes the minimum vertical length of the mid segment. If it is used for
       * orthogonal routing, it describes the minimum length of the whole edge.
       * </p>
       * <p>
       * The minimum length should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length is negative
       * @type {number}
       */
      minimumLength:number;
      /**
       * Gets or sets the preferred minimum distance to obstacles.
       * <p>
       * The minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length is negative
       * @type {number}
       */
      minimumDistance:number;
      /**
       * Gets or sets the minimum slope that the mid segments of this edge should have, if the edge is routed polyline in
       * vertical layouts.
       * <p>
       * The minimum slope should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum slope is negative
       * @type {number}
       */
      minimumSlope:number;
      /**
       * Gets or sets whether or not source port optimization is enabled.
       * <p>
       * This means that the algorithm should try to find a better source {@link yfiles.layout.PortConstraint port constraint} if the current constraint is <code>null</code> or
       * {@link yfiles.layout.PortSide#ANY}.
       * </p>
       * @type {boolean}
       */
      sourcePortOptimization:boolean;
      /**
       * Gets or sets whether or not target port optimization is enabled.
       * <p>
       * This means that the algorithm should try to find a better target {@link yfiles.layout.PortConstraint port constraint} if the current constraint is <code>null</code> or
       * {@link yfiles.layout.PortSide#ANY}.
       * </p>
       * @type {boolean}
       */
      targetPortOptimization:boolean;
      /**
       * Gets or sets the routing style for this edge.
       * @see yfiles.hierarchic.RoutingStyle
       * @type {yfiles.hierarchic.RoutingStyle}
       */
      routingStyle:yfiles.hierarchic.RoutingStyle;
      /**
       * Gets or sets the minimum length of octilinear segments for this edge.
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length of the first segment is negative
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#routingStyle
       * @type {number}
       */
      minimumOctilinearSegmentLength:number;
      /**
       * Gets or sets how edges in grouped graphs are routed between the content of different groups.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown style is specified
       * @type {yfiles.hierarchic.RecursiveEdgeStyle}
       */
      recursiveEdgeStyle:yfiles.hierarchic.RecursiveEdgeStyle;
      /**
       * Gets or sets whether or not this edge is routed directly to a group node's border if it connects the group node with one
       * of its descendants.
       * <p>
       * When enabled, the edge will connect to the inner side of the group node's border instead of leaving the group node and
       * connect from outside.
       * </p>
       * @type {boolean}
       */
      directGroupContentEdgeRouting:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum WeightHeuristic{
      /**
       * A weight assignment specifier based on a barycenter heuristic.
       * <p>
       * The position of a node within a layer will be determined by the barycenter of its successor (downward pass) and
       * predecessor (upward pass) nodes.
       * </p>
       * @see yfiles.hierarchic.DefaultLayerSequencer#weightHeuristic
       */
      BARYCENTER,
      /**
       * A weight assignment specifier based on a median heuristic.
       * <p>
       * The position of a node within a layer will be determined by the median position of its successor (downward pass) and
       * predecessor (upward pass) nodes.
       * </p>
       * @see yfiles.hierarchic.DefaultLayerSequencer#weightHeuristic
       */
      MEDIAN
    }
    export enum MergingPolicy{
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} below the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_BELOW,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} above the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_ABOVE,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} to the right of the layers of the target layering.
       * <p>
       * Both {@link yfiles.hierarchic.ILayers layering}s will become top-aligned before the merge.
       * </p>
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_RIGHT_TOP_ALIGNED,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} to the right of the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * <p>
       * Both {@link yfiles.hierarchic.ILayers layering}s will become bottom-aligned before the merge.
       * </p>
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_RIGHT_BOTTOM_ALIGNED,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} to the right of the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * <p>
       * Both {@link yfiles.hierarchic.ILayers layering}s will become center-aligned before the merge.
       * </p>
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_RIGHT_CENTER_ALIGNED,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} to the left of the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * <p>
       * Both {@link yfiles.hierarchic.ILayers layering}s will become top-aligned before the merge.
       * </p>
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_LEFT_TOP_ALIGNED,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} to the left of the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * <p>
       * Both {@link yfiles.hierarchic.ILayers layering}s will become bottom-aligned before the merge.
       * </p>
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_LEFT_BOTTOM_ALIGNED,
      /**
       * A constant that adds the new layers of the source {@link yfiles.hierarchic.ILayers layering} to the left of the layers of the target {@link yfiles.hierarchic.ILayers layering}.
       * <p>
       * Both {@link yfiles.hierarchic.ILayers layering}s will become center-aligned before the merge.
       * </p>
       * @see yfiles.hierarchic.DefaultLayeredComponentsMerger#policy
       */
      ADD_LEFT_CENTER_ALIGNED
    }
    export enum EdgeDataType{
      /**
       * A constant describing a normal edge.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      NORMAL,
      /**
       * A constant describing an edge that connects to a source group node.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      SOURCE_GROUP_NODE_CONNECTOR,
      /**
       * A constant describing an edge that connects to a target group node.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      TARGET_GROUP_NODE_CONNECTOR,
      /**
       * A constant describing an edge that connects two group nodes.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      GROUP_NODE_INTERCONNECTOR,
      /**
       * A constant describing a same-layer edge that connects directly two neighboring nodes.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      DIRECT_SAME_LAYER_EDGE,
      /**
       * A constant describing a same-layer edge that does not connect directly two neighboring nodes.
       * <p>
       * This means that two nodes of the same layer are not connected directly with a straight-line edge, but with an edge
       * consisting of polyline segments.
       * </p>
       * @see yfiles.hierarchic.IEdgeData#type
       */
      NON_DIRECT_SAME_LAYER_EDGE,
      /**
       * A constant describing an edge that connects group border nodes in adjacent layers.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      BORDER_EDGE,
      /**
       * A constant describing an edge that has been added temporarily to replace an edge connected to a group node.
       * @see yfiles.hierarchic.IEdgeData#type
       */
      REDIRECTED_GROUP_EDGE
    }
    export enum RecursiveEdgeStyle{
      /**
       * Routing specifier that routes edges between different group contents non-recursively.
       * {@link yfiles.algorithms.Edge}s may leave and enter group nodes at the left and right side in order to connect as directly as
       * possible.
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#recursiveEdgeStyle
       */
      OFF,
      /**
       * Routing specifier that routes edges between different group contents recursively and directed.
       * {@link yfiles.algorithms.Edge}s can only leave the groups at the bottom side and enter them at the top side.
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#recursiveEdgeStyle
       */
      DIRECTED,
      /**
       * Routing specifier that routes edges between different group contents recursively and undirected.
       * {@link yfiles.algorithms.Edge}s can leave the groups at the bottom or top side and enter them at the bottom or top side. This
       * style is more compact than the directed edge style.
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#recursiveEdgeStyle
       */
      UNDIRECTED
    }
    export enum ComponentArrangementPolicy{
      /**
       * A component arrangement policy that achieves a <code>1:1</code> aspect ratio for the graph.
       * <p>
       * After layering the single components with the core layerer, the components will be merged such that an 1:1 aspect ratio
       * is fulfilled best.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#componentArrangementPolicy
       */
      COMPACT,
      /**
       * A component arrangement policy that aligns the components with their topmost layer.
       * <p>
       * After layering the single components with the core layerer, the components are aligned with their topmost layer.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#componentArrangementPolicy
       */
      TOPMOST
    }
    export enum LayoutMode{
      /**
       * A layout mode that allows to keep some nodes fixed and insert the remaining nodes incrementally.
       * <p>
       * Fixed elements will be handled by the {@link yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer fixed elements sequencer} and {@link yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer fixed elements layerer}. Incremental elements will be
       * added during the layering or the sequencing phase. They are handled by {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayerer from scratch layerer} and {@link yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer fixed elements layerer}
       * respectively.
       * </p>
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createLayerIncrementallyHint
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @see yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer
       * @see yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer
       */
      INCREMENTAL,
      /**
       * A layout mode that will recompute the complete layout from scratch.
       * <p>
       * All elements will be handled by the same {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayerer layerer} and {@link yfiles.hierarchic.HierarchicLayout#fromScratchSequencer sequencer}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchSequencer
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayerer
       */
      FROM_SCRATCH
    }
    export enum LayeringStrategy{
      /**
       * A layering strategy which places each node in the highest possible layer without passing its predecessor in the
       * topological order of nodes.
       * <p>
       * All nodes with indegree zero will be assigned to the topmost layer of the layout. The number of separate layers will be
       * as small as possible.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      HIERARCHICAL_TOPMOST,
      /**
       * A layering strategy which places each node in the optimal layer to minimize the layer distances.
       * <p>
       * The layer distance of an edge is the absolute difference between the layer numbers of its source and target node. The
       * layer assignment will minimize the overall sum of the layer distances of all edges in the layout.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      HIERARCHICAL_OPTIMAL,
      /**
       * A layering strategy which applies a heuristic to approximate the {@link yfiles.hierarchic.LayeringStrategy#HIERARCHICAL_OPTIMAL optimal layering}.
       * <p>
       * The quality of the result may not be as good as with {@link yfiles.hierarchic.LayeringStrategy#HIERARCHICAL_OPTIMAL optimal layering} but it is usually achieved faster.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      HIERARCHICAL_TIGHT_TREE,
      /**
       * A layering strategy which applies a fast heuristic that improves the layering done by {@link yfiles.hierarchic.LayeringStrategy#HIERARCHICAL_TOPMOST}
       * by shifting some nodes down.
       * <p>
       * The quality of the result is usually worse than the one produced by
       * {@link yfiles.hierarchic.LayeringStrategy#HIERARCHICAL_TIGHT_TREE}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      HIERARCHICAL_DOWNSHIFT,
      /**
       * A layering strategy based on a breadth first search (BFS).
       * <p>
       * All edges will span at most one layer in the resulting drawing. Edges between nodes that belong to the same layer are
       * possible.
       * </p>
       * <p>
       * The nodes that will be placed in the first layer can be provided by a data provider bound to the input graph using the
       * key
       * {@link yfiles.hierarchic.BFSLayerer#CORE_NODES_DP_KEY}. If this data provider is not given, then nodes that have no incoming
       * edges are placed in the first layer.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      BFS,
      /**
       * A layering strategy which derives the layers from the initial coordinates of the nodes.
       * <p>
       * It tries to find a layering that is similar to the one in the input graph. The layout algorithm may place nodes in the
       * same layer, even though they are connected by an edge.
       * </p>
       * <p>
       * In a vertically oriented layout, nodes that overlap vertically are placed in the same layer. The same holds for a
       * horizontally oriented layout in the case where the nodes overlap horizontally.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      FROM_SKETCH,
      /**
       * A layering strategy which uses a layer assignment specified by the user.
       * <p>
       * The nodes' layer assignment must be provided by a data provider bound to the input graph using the key
       * {@link yfiles.hierarchic.GivenLayersLayerer#LAYER_ID_DP_KEY}.
       * </p>
       * <p>
       * This layering strategy allows edges whose source and target share a layer.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      USER_DEFINED,
      /**
       * A dummy layering strategy which describes a strategy that is not part of the default strategies.
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy
       */
      UNKNOWN
    }
    export enum GroupAlignmentPolicy{
      /**
       * A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the top with
       * respect to their inner layers.
       * @see yfiles.hierarchic.HierarchicLayout#groupAlignmentPolicy
       * @see yfiles.hierarchic.HierarchicLayout#recursiveGroupLayering
       */
      TOP,
      /**
       * A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the center with
       * respect to their inner layers.
       * @see yfiles.hierarchic.HierarchicLayout#groupAlignmentPolicy
       * @see yfiles.hierarchic.HierarchicLayout#recursiveGroupLayering
       */
      CENTER,
      /**
       * A group layering alignment strategy which aligns groups and normal nodes that occupy the same layer to the bottom with
       * respect to their inner layers.
       * @see yfiles.hierarchic.HierarchicLayout#groupAlignmentPolicy
       * @see yfiles.hierarchic.HierarchicLayout#recursiveGroupLayering
       */
      BOTTOM
    }
    export enum LayerType{
      /**
       * A constant describing a normal layer consisting mainly of normal nodes.
       */
      NORMAL,
      /**
       * A constant describing a layer consisting mainly of label nodes or dummy nodes.
       */
      LABEL,
      /**
       * A constant describing a layer consisting of source group nodes and dummy nodes.
       */
      SOURCE_GROUP_NODES,
      /**
       * A constant describing a layer consisting of target group nodes and dummy nodes.
       */
      TARGET_GROUP_NODES,
      /**
       * Describes a layer above a group to store the connector proxies where edges connect to groups.
       */
      UPPER_GROUP_CONNECTOR_NODES,
      /**
       * Describes a layer below a group to store the connector proxies where edges connect to groups.
       */
      LOWER_GROUP_CONNECTOR_NODES
    }
    export enum NodeDataType{
      /**
       * A constant describing a normal {@link yfiles.algorithms.Node}.
       * <p>
       * A normal node may have any degree, any size and any number of connected same-layer edges.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      NORMAL,
      /**
       * A constant describing a bend {@link yfiles.algorithms.Node}.
       * <p>
       * A bend node has degree <code>2</code>, small size and no same-layer edges.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      BEND,
      /**
       * A constant describing a side proxy {@link yfiles.algorithms.Node}.
       * <p>
       * A side proxy node, in case port constraints are used for the drawing phase, has degree
       * <code>1</code>, small size and no same-layer edges. The associated node is the one of which this node is the proxy.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      SIDE_PROXY,
      /**
       * A constant describing a side proxy {@link yfiles.algorithms.Node} for drawing same-layer edges.
       * <p>
       * A side proxy node for drawing same-layer edges has degree
       * <code>0</code>, small size and one same-layer edge. The associated node is the one of which this node is the proxy. The associated
       * edge is the same-layer edge of which this node is the proxy.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      SAME_LAYER_SIDE_PROXY,
      /**
       * A constant describing a label {@link yfiles.algorithms.Node}.
       * <p>
       * A label node has degree
       * <code>2</code>, size of the labels and no same-layer edges. The borders of this node describe the layout of the labels.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      LABEL,
      /**
       * A constant describing a group {@link yfiles.algorithms.Node} for nested graph layouts.
       * <p>
       * This group node carries additional {@link yfiles.hierarchic.ILayers} information.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       * @see yfiles.hierarchic.INodeData#groupLayers
       */
      GROUP,
      /**
       * A constant describing a source group {@link yfiles.algorithms.Node} for bus edges.
       * <p>
       * This node serves as a dummy node in a {@link yfiles.hierarchic.LayerType#SOURCE_GROUP_NODES} layer.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       * @see yfiles.hierarchic.INodeData#groupId
       */
      SOURCE_GROUP_NODE,
      /**
       * A constant describing a target group {@link yfiles.algorithms.Node} for bus edges.
       * <p>
       * This node serves as a dummy node in a {@link yfiles.hierarchic.LayerType#TARGET_GROUP_NODES} layer.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       * @see yfiles.hierarchic.INodeData#groupId
       */
      TARGET_GROUP_NODE,
      /**
       * A constant describing a back-loop proxy that belongs to the source of an edge.
       * <p>
       * A back-loop proxy that belongs to the source of an edge has degree <code>2</code>, small size and exactly one same-layer edge.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      SOURCE_BACK_LOOP_PROXY,
      /**
       * A constant describing a back-loop proxy that belongs to the target of an edge.
       * <p>
       * A back-loop proxy that belongs to the target of an edge has degree <code>2</code>, small size and exactly one same-layer edge.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      TARGET_BACK_LOOP_PROXY,
      /**
       * A constant describing a same-layer edge crossing dummy {@link yfiles.algorithms.Node}.
       * <p>
       * A same-layer edge crossing dummy node has degree
       * <code>2</code>, small size and exactly two same-layer edges. It is used for marking the crossing of the layer, if source and target
       * port are at opposite layer sides.
       * </p>
       * @see yfiles.hierarchic.INodeData#type
       */
      SAME_LAYER_CENTER_NODE,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used for the drawing phase to keep the distance between two nodes.
       * @see yfiles.hierarchic.INodeData#type
       */
      DISTANCE_NODE,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used during the drawing phase for marking the beginning of a node group
       * interval.
       * @see yfiles.hierarchic.INodeData#type
       */
      GROUP_BEGIN,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used during the drawing phase for marking the ending of a node group
       * interval.
       * @see yfiles.hierarchic.INodeData#type
       */
      GROUP_END,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used during the sequencing and drawing phase for marking a dummy node
       * inserted to guarantee that a group is non-empty on the layer on which it is defined.
       * @see yfiles.hierarchic.INodeData#type
       */
      GROUP_LAYER_DUMMY,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used during the sequencing and drawing phase for marking a dummy node
       * inserted as a legal endpoint of an edge connecting to a group node.
       * @see yfiles.hierarchic.INodeData#type
       */
      PROXY_FOR_EDGE_AT_GROUP,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used during the sequencing and drawing phase for marking a dummy node
       * inserted as a legal endpoint of an edge connecting a descendant node to a group node. It has degree <code>0</code> or <code>1</code> depending on
       * whether or not the opposite node of the edge is assigned to the same layer. This kind of proxy nodes is only used if
       * there are edges that shall connect to the group node without backloops.
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#directGroupContentEdgeRouting
       */
      PROXY_FOR_CONTENT_EDGE_AT_GROUP,
      /**
       * A constant describing a {@link yfiles.algorithms.Node} used during sequencing and drawing phase for routing an {@link yfiles.algorithms.Edge}
       * recursively. An {@link yfiles.algorithms.Edge} that connects to the content of a group node has to leave the group node at the
       * bottom side and enter it at the top side.
       * @see yfiles.hierarchic.INodeData#type
       */
      RECURSIVE_EDGE_DUMMY
    }
    export enum PortAssignmentMode{
      /**
       * A port assignment strategy that distributes ports evenly along the border of the node.
       * <p>
       * This is the default port assignment strategy of {@link yfiles.hierarchic.HierarchicLayout}.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#portAssignment
       */
      DEFAULT,
      /**
       * A port assignment strategy that places all edges on grid lines.
       * <p>
       * If there are not enough grid lines for each port, ports may overlap. In case there is no grid line available at the side
       * of a node, the ports are placed centered at that side.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#portAssignment
       */
      ON_GRID,
      /**
       * A port assignment strategy that places all edges on grid lines or sub-grid lines.
       * <p>
       * If there are not enough grid lines for each port, the grid gets subdivided with sub-grid lines until each edge has space
       * for its port.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#portAssignment
       */
      ON_SUBGRID
    }
    export enum NodeLabelMode{
      /**
       * A constant that prevents node labels from being considered during the layout.
       * <p>
       * It is used by {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode} and {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode}.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode
       */
      NEVER,
      /**
       * A constant that makes the algorithm consider node labels for self-loops, node placement and routing.
       * <p>
       * It is used by {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode} and {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode}.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode
       */
      CONSIDER_FOR_SELF_LOOPS,
      /**
       * A constant that makes the algorithm consider node labels for node placement and routing, but not for self-loops.
       * <p>
       * It is used by {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode} and {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode}.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode
       */
      CONSIDER_FOR_DRAWING,
      /**
       * A constant that makes the algorithm consider node labels during routing only.
       * <p>
       * It is used by {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode} and {@link yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode}.
       * </p>
       * @see yfiles.hierarchic.NodeLayoutDescriptor#nodeLabelMode
       */
      CONSIDER_FOR_ROUTING
    }
    export enum EdgeRoutingStyle{
      /**
       * A routing style constant specifying that the edges should be orthogonal. Orthogonal edges only consist of vertical and
       * horizontal segments.
       */
      ORTHOGONAL,
      /**
       * A routing style constant specifying that the edges should be octilinear. Octilinear edges consist of vertical and
       * horizontal segments as well as segments with slope <code>-1</code> and <code>1</code>.
       */
      OCTILINEAR,
      /**
       * A routing style constant specifying that the edges should be polyline.
       */
      POLYLINE
    }
    export enum GroupCompactionPolicy{
      /**
       * A strategy specifying no horizontal group compaction.
       * <p>
       * This means that group node contents will occupy nearly the same horizontal positions as when not grouped at all.
       * </p>
       * @see yfiles.hierarchic.SimplexNodePlacer#groupCompactionStrategy
       */
      NONE,
      /**
       * A strategy specifying maximal horizontal group compaction.
       * <p>
       * This means that the node placer will try to minimize the horizontally occupied space for a group node.
       * </p>
       * @see yfiles.hierarchic.SimplexNodePlacer#groupCompactionStrategy
       */
      MAXIMAL
    }
    export enum RankingPolicy{
      /**
       * A ranking policy that specifies that nodes are not re-ranked after the initial topological layering step.
       */
      NO_RERANKING,
      /**
       * A ranking policy that specifies that nodes are re-ranked by an inexpensive downshift rule.
       */
      DOWN_SHIFT,
      /**
       * A ranking policy that specifies that nodes are re-ranked by finding a spanning tree that contains only tight (length
       * <code>1</code>) edges.
       */
      TIGHT_TREE
    }
    /**
     * This layering algorithm implementation assigns nodes to layers given a mapping of nodes to layer IDs.
     * <p>
     * The layer IDs are given by a {@link yfiles.algorithms.IDataProvider} that returns the integer layer ID of each node of the
     * graph. Nodes with the same ID are in the same layer while the layers are sorted according to their IDs such that the
     * smallest ID represents the top layer.
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface GivenLayersLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class GivenLayersLayerer {
      /**
       * Creates an instance of {@link yfiles.hierarchic.GivenLayersLayerer}.
       */
      constructor();
      /**
       * Assigns all nodes of the graph to layers and adds them to the {@link yfiles.hierarchic.ILayers} instance.
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for querying information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Convenience method that removes empty layers and ensures that the smallest layer has value <code>0</code>.
       * @param graph the graph
       * @param layerId the {@link } that returns an integer value (layer) for each node; the first layer has the number
       * <code>0</code>
       * @param normalizedLayerId the {@link } that will be filled by the layout algorithm and holds an integer value (layer ID
       * after normalization) for each node
       * @returns the number of layers after the removal of empty layers
       */
      normalize(graph:yfiles.algorithms.Graph,layerId:yfiles.algorithms.IDataProvider,normalizedLayerId:yfiles.algorithms.IDataAcceptor):number;
      /**
       * Data provider key for storing the layer IDs
       * <p>
       * This {@link yfiles.algorithms.IDataProvider} stores for each node the layer ID to which it belongs. Nodes with the same ID are
       * in the same layer while the layers are sorted according to their IDs such that the smallest ID represents the top
       * layer.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static LAYER_ID_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a {@link yfiles.hierarchic.ISequencer} implementation that determines the sequence of nodes of the same {@link yfiles.hierarchic.ILayer layer}
     * based on a given {@link yfiles.collections.IComparer.<T> comparator} constraint.
     * @class
     * @implements {yfiles.hierarchic.ISequencer}
     */
    export interface GivenSequenceSequencer extends Object,yfiles.hierarchic.ISequencer{}
    export class GivenSequenceSequencer {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.GivenSequenceSequencer} using a given {@link yfiles.collections.IComparer.<T>} for the sequencing.
       * @param [c=null]
       */
      constructor(c?:yfiles.collections.IComparer<Object>);
      /**
       * Calculates the sequence of the nodes within a {@link yfiles.hierarchic.ILayers} instance based on the given {@link yfiles.collections.IComparer.<T>} instance.
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayoutCore} during the sequencing phase and finally writes back the
       * calculated sequence using the {@link yfiles.hierarchic.ILayer#setNodeOrder} method.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance containing the elements of the layers
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @param itemFactory the {@link } used for creating and destroying helper structures
       */
      sequenceNodeLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Gets or sets the {@link yfiles.collections.IComparer.<T>} used by this {@link yfiles.hierarchic.GivenSequenceSequencer} to determine the sequence of the nodes.
       * @type {yfiles.collections.IComparer.<Object>}
       */
      sequenceComparer:yfiles.collections.IComparer<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Visitor interface that can be used to traverse the descendants of a group node
     * @interface
     */
    export interface IVisitor extends Object{
      /**
       * Visits the descendants of a group node.
       * @param node A descandant of the group node.
       * @param parentNode A group node.
       * @abstract
       */
      visit(node:yfiles.algorithms.Node,parentNode:yfiles.algorithms.Node):void;
    }
    var IVisitor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class is responsible for managing hierarchically grouped graphs.
     * <p>
     * The grouping information is provided through {@link yfiles.algorithms.IDataProvider}s registered with the graph with keys
     * {@link yfiles.layout.GroupingKeys#NODE_ID_DP_KEY}, {@link yfiles.layout.GroupingKeys#PARENT_NODE_ID_DP_KEY} and
     * {@link yfiles.layout.GroupingKeys#GROUP_DP_KEY}.
     * </p>
     * @see yfiles.layout.GroupingSupport
     * @see yfiles.layout.GroupingKeys
     * @class
     */
    export interface GroupingSupport extends Object{}
    export class GroupingSupport {
      /**
       * Assigns edge group nodes to groups based on a given {@link yfiles.hierarchic.ILayers} instance.
       * @param layers the given {@link } instance containing the elements in the layering
       * @see yfiles.hierarchic.NodeDataType#SOURCE_GROUP_NODE
       * @see yfiles.hierarchic.NodeDataType#TARGET_GROUP_NODE
       */
      assignEdgeGroupNodesToGroups(layers:yfiles.hierarchic.ILayers):void;
      /**
       * Assigns label nodes to groups.
       * @param layers the given {@link } instance containing the elements in the layering
       * @param itemFactory the {@link } used temporarily for modifying the graph instance
       * @see yfiles.hierarchic.NodeDataType#LABEL
       * @see yfiles.hierarchic.NodeDataType#BEND
       * @see yfiles.hierarchic.NodeDataType#GROUP_BEGIN
       * @see yfiles.hierarchic.NodeDataType#GROUP_END
       */
      assignLabelNodesToGroups(layers:yfiles.hierarchic.ILayers,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Returns the direct children of the given group node.
       * @param groupNode the given group node
       * @returns a {@link } of all direct children or <code>null</code> if {@link } {@link #active is not active}
       */
      getChildren(groupNode:yfiles.algorithms.Node):yfiles.algorithms.NodeList;
      /**
       * Returns the descendant {@link yfiles.algorithms.Node}s of the given group node.
       * <p>
       * The descendants are all direct or indirect children of a group node.
       * </p>
       * @param groupNode the given group node
       * @returns a {@link } of all descendants or <code>null</code> if {@link } {@link #active is not active}
       */
      getDescendants(groupNode:yfiles.algorithms.Node):yfiles.algorithms.NodeList;
      /**
       * Returns the parent group node of the given {@link yfiles.algorithms.Node} or <code>null</code> if the given node is a top-level node.
       * @param node the given {@link }
       * @returns the parent group node or <code>null</code> if the given node is a top-level node or if {@link }
       * {@link #active is not active}
       */
      getParentNode(node:yfiles.algorithms.Node):yfiles.algorithms.Node;
      /**
       * Hides the group nodes and incident edges from the graph.
       */
      hideGroupNodes():void;
      /**
       * Returns whether or not the given node is a descendant {@link yfiles.algorithms.Node} of the given group node.
       * <p>
       * The descendants are all direct or indirect children of a group node.
       * </p>
       * @param node the given node
       * @param groupNode the given group node
       * @returns <code>true</code> if the given node is a descendant of the given group node, <code>false</code> otherwise
       */
      isDescendant(node:yfiles.algorithms.Node,groupNode:yfiles.algorithms.Node):boolean;
      /**
       * Returns whether or not the given {@link yfiles.algorithms.Node} is a non-empty group node.
       * @param node the given {@link }
       * @returns <code>true</code> if the given node is a non-empty group node, <code>false</code> otherwise
       */
      isGroupNode(node:yfiles.algorithms.Node):boolean;
      /**
       * Removes the current assignment of source/target edge groups from a given {@link yfiles.hierarchic.ILayers} instance.
       * @param layers the given {@link } instance containing the elements in the layering
       * @see yfiles.hierarchic.NodeDataType#SOURCE_GROUP_NODE
       * @see yfiles.hierarchic.NodeDataType#TARGET_GROUP_NODE
       */
      removeEdgeGroupAssignment(layers:yfiles.hierarchic.ILayers):void;
      /**
       * Un-hides all previously hidden group nodes and incident edges.
       */
      unhideGroupNodes():void;
      /**
       * Visits all descendant {@link yfiles.algorithms.Node}s of the given group node.
       * <p>
       * The descendants are all direct or indirect children of a group node.
       * </p>
       * @param groupNode the given group node
       * @param visitor a {@link } implementation
       */
      visitDescendants(groupNode:yfiles.algorithms.Node,visitor:yfiles.hierarchic.IVisitor):void;
      /**
       * Gets or sets the minimum distance between group nodes.
       * <p>
       * The minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      minimumGroupDistance:number;
      /**
       * Gets whether or not {@link yfiles.hierarchic.GroupingSupport} is active.
       * @type {boolean}
       */
      active:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm arranges graphs in a hierarchic fashion.
     * <p>
     * <b>Layout Style</b> The nodes are distributed into layers so that most of the edges point to the main layout direction. The order of the
     * nodes within the layers ensures that the number of edge crossings is as small as possible. There are different edge
     * routing styles available. Edges can be orthogonal, polyline or octilinear.
     * </p>
     * <p>
     * Hierarchical diagrams are commonly used for the visualization of hierarchical data, since they facilitate the
     * identification of dependencies and relationships among the nodes of the graph. Possible application domains are the
     * following: workflow visualization, call graph visualization, entity-relationship diagrams, biochemical pathways and
     * network management.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[37.5,37.5],"d":1},"n":[[137.81,67.5],[137.81,135],[718.44,67.5],[147.19,202.5],[305.62,135],[296.25,202.5],[771.88,450],[354.38,285],[290,367.5],[366.88,367.5],[400.62,67.5],[709.06,135],[709.06,202.5],[366.88,450],[79.69,202.5],[492.5,285],[501.88,367.5],[627.5,285],[357.5,532.5],[363.75,202.5],[286.88,285],[425,532.5],[400.62,135],[569.38,450],[569.38,367.5],[155,450],[434.38,367.5],[636.88,367.5],[84.38,285],[151.88,285],[222.5,532.5],[290,532.5],[155,532.5],[87.5,532.5],[20,532.5],[219.38,285],[69.38,367.5],[713.75,285],[704.38,367.5],[771.88,367.5],[848.75,285],[906.88,367.5],[839.38,367.5],[400.62,0]],"e":[[0,1,0,18.75,0,-18.75],[1,3,9.38,18.75,0,-18.75],[0,4,12.5,18.75,0,-18.75,[169.06,120,324.38,120]],[4,5,-9.38,18.75,0,-18.75],[2,6,9.38,18.75,12.5,-18.75,[746.56,120,959.38,120,959.38,420,803.12,420]],[5,7,9.38,18.75,0,-18.75,[324.38,255,373.12,255]],[7,8,0,18.75,0,-18.75,[373.12,352.5,308.75,352.5]],[7,9,12.5,18.75,0,-18.75],[2,11,-9.38,18.75,0,-18.75],[11,12,0,18.75,0,-18.75],[9,13,0,18.75,0,-18.75],[1,14,-9.38,18.75,0,-18.75,[147.19,187.5,98.44,187.5]],[12,15,-14.06,18.75,0,-18.75,[713.75,255,511.25,255]],[15,16,9.38,18.75,0,-18.75],[12,17,-4.69,18.75,0,-18.75,[723.13,270,646.25,270]],[13,18,-9.38,18.75,0,-18.75],[4,19,9.38,18.75,0,-18.75,[333.75,187.5,382.5,187.5]],[5,20,-9.38,18.75,0,-18.75],[13,21,9.38,18.75,0,-18.75,[395,502.5,443.75,502.5]],[10,22,0,18.75,0,-18.75],[22,23,0,18.75,-12.5,-18.75,[419.38,420,575.62,420]],[17,24,-9.38,18.75,0,-18.75,[636.88,337.5,588.12,337.5]],[24,23,0,18.75,0,-18.75],[0,25,-12.5,18.75,-9.38,-18.75,[144.06,120,54.38,120,54.38,420,164.38,420]],[8,25,-9.38,18.75,9.38,-18.75,[299.38,420,183.12,420]],[15,26,-9.38,18.75,0,-18.75,[501.88,337.5,453.12,337.5]],[17,27,9.38,18.75,0,-18.75],[27,6,0,18.75,-12.5,-18.75,[655.62,420,778.12,420]],[3,28,-4.69,18.75,0,-18.75,[161.25,270,103.12,270]],[3,29,4.69,18.75,0,-18.75],[25,30,7.5,18.75,0,-18.75,[181.25,517.5,241.25,517.5]],[25,31,15,18.75,-9.38,-18.75,[188.75,502.5,299.38,502.5]],[8,31,9.38,18.75,9.38,-18.75],[25,32,0,18.75,0,-18.75],[25,33,-7.5,18.75,0,-18.75,[166.25,517.5,106.25,517.5]],[25,34,-15,18.75,0,-18.75,[158.75,502.5,38.75,502.5]],[3,35,14.06,18.75,0,-18.75,[180,255,238.12,255]],[3,36,-14.06,18.75,-9.38,-18.75,[151.88,255,69.38,255,69.38,337.5,78.75,337.5]],[7,36,-12.5,18.75,9.38,-18.75,[360.62,337.5,97.5,337.5]],[12,37,4.69,18.75,0,-18.75],[37,38,-9.38,18.75,0,-18.75],[38,23,0,18.75,12.5,-18.75,[723.12,435,600.62,435]],[37,39,9.38,18.75,0,-18.75,[741.88,337.5,790.62,337.5]],[39,6,0,18.75,0,-18.75],[12,40,14.06,18.75,0,-18.75,[741.88,255,867.5,255]],[40,41,9.38,18.75,0,-18.75,[876.88,337.5,925.62,337.5]],[40,42,-9.38,18.75,0,-18.75],[43,0,-12.5,18.75,0,-18.75,[406.88,52.5,156.56,52.5]],[43,2,12.5,18.75,0,-18.75,[431.88,52.5,737.19,52.5]],[43,10,0,18.75,0,-18.75]],"vp":[20.0,0.0,940.0,570.0]}} Hierarchic Layout obtained with default settings<b>Concept</b>The layout algorithm runs in three main phases:
     * </p>
     * <ul>
     * <li>
     * <b>Layering</b> – The nodes are distributed into layers by means of {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayerer} or
     * {@link yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer}, respectively. If the layout orientation is top-to-bottom, the nodes in each layer are arranged horizontally while the
     * layers are ordered vertically top-to-bottom.
     * </li>
     * <li>
     * <b>Sequencing</b> – The order of the nodes in each layer is determined such that the number of edge crossings is as small as possible. To
     * specify the sequencing algorithm, use {@link yfiles.hierarchic.HierarchicLayout#fromScratchSequencer} or {@link yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer}.
     * </li>
     * <li><b>Drawing</b> – The layout algorithm assigns the final coordinates to all nodes and routes the edges.</li>
     * </ul>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * This layout algorithm is able to create hierarchic layouts from scratch or add new elements to the existing sketch
     * drawing incrementally. In order to add elements incrementally to the current sketch or let the algorithm optimize
     * certain elements in the current sketch, set the layout mode to {@link yfiles.hierarchic.LayoutMode#INCREMENTAL}. Then register a
     * {@link yfiles.algorithms.IDataProvider} (e.g. use {@link yfiles.algorithms.Maps#createHashedDataMap}) with the graph using the {@link yfiles.hierarchic.HierarchicLayout#INCREMENTAL_HINTS_DP_KEY}
     * DataProvider key and associate the hints obtained from the {@link yfiles.hierarchic.IIncrementalHintsFactory} with the elements
     * to be added incrementally.
     * </p>
     * <p>
     * {@link yfiles.hierarchic.NodeLayoutDescriptor} and {@link yfiles.hierarchic.EdgeLayoutDescriptor} instances can be used for specifying
     * individual information (e.g. distances or routing styles) for each node and edge in the graph. The descriptors are bound
     * to the graph using {@link yfiles.algorithms.IDataProvider}s registered with {@link yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}
     * or {@link yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If there is no descriptor assigned to some
     * nodes/edges, a default descriptor will be used. To set default descriptors use {@link yfiles.hierarchic.HierarchicLayout#nodeLayoutDescriptor} and {@link yfiles.hierarchic.HierarchicLayout#edgeLayoutDescriptor}.
     * </p>
     * <p>
     * {@link yfiles.hierarchic.HierarchicLayout} supports two approaches to connect edges on a specific side or even an exact location
     * to a node. {@link yfiles.layout.PortConstraint}s define a single constraint for the ports of an edge. To realize more complex
     * port restrictions, several {@link yfiles.layout.PortCandidate}s or {@link yfiles.layout.PortCandidateSet}s can be assigned to edges or
     * nodes. If an edge with registered {@link yfiles.layout.PortCandidate}s connects to nodes with {@link yfiles.layout.PortCandidateSet}s,
     * the layouter will try to match both collections to find an appropriate port. In case there is no matching port
     * candidate, a {@link yfiles.layout.PortCandidate} specified for the edge is preferred. Since their simultaneous existence at the
     * same node may be ambiguous, it is not recommended to use a combination of {@link yfiles.layout.PortConstraint}s and
     * {@link yfiles.layout.PortCandidate}s in the same layout.
     * </p>
     * <p>
     * The edge grouping feature of this layout algorithm is restricted to normal, hierarchic edges. Edges with {@link yfiles.hierarchic.EdgeLayoutDescriptor#recursiveEdgeStyle recursive edge style}
     * {@link yfiles.hierarchic.RecursiveEdgeStyle#DIRECTED} or {@link yfiles.hierarchic.RecursiveEdgeStyle#UNDIRECTED} will not be grouped.
     * They are also not grouped when enabling {@link yfiles.hierarchic.HierarchicLayout#automaticEdgeGrouping automatic edge grouping}.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface HierarchicLayout extends yfiles.layout.MultiStageLayout{}
    export class HierarchicLayout {
      /**
       * Creates a new {@link yfiles.hierarchic.HierarchicLayout} instance with the default settings.
       */
      constructor();
      /**
       * Configures the core layout algorithm with the settings of this {@link yfiles.hierarchic.HierarchicLayout} instance.
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayout#applyLayoutCore} before the actual layout is calculated. It
       * may be overridden in order to manually reconfigure the core layout algorithm.
       * </p>
       * <p>
       * This implementation will temporarily set a {@link yfiles.hierarchic.PortCandidateOptimizer} if a {@link yfiles.algorithms.IDataProvider}
       * is registered with {@link yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY} and no {@link yfiles.hierarchic.HierarchicLayoutCore#portConstraintOptimizer} is assigned.
       * </p>
       * @param graph the input graph
       * @param coreLayouter the given core layout algorithm instance
       * @protected
       */
      configureCoreLayout(graph:yfiles.layout.LayoutGraph,coreLayouter:yfiles.hierarchic.HierarchicLayoutCore):void;
      /**
       * Returns a new {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance that will be used during the various phases of the layout
       * algorithm to determine the drawing details of the edges of the graph.
       * <p>
       * This method may be overridden to create a new {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance with different
       * configuration settings.
       * </p>
       * @returns a new {@link } instance
       * @protected
       */
      createEdgeLayoutDescriptor():yfiles.hierarchic.EdgeLayoutDescriptor;
      /**
       * Returns a new {@link yfiles.hierarchic.HierarchicLayoutCore} instance.
       * <p>
       * This method may be overridden to create a new {@link yfiles.hierarchic.HierarchicLayoutCore} object with different configuration
       * settings.
       * </p>
       * <p>
       * This factory method provides the initial {@link yfiles.hierarchic.HierarchicLayoutCore} instance.
       * </p>
       * @returns a new {@link } instance
       * @protected
       */
      createHierarchicLayoutCore():yfiles.hierarchic.HierarchicLayoutCore;
      /**
       * Returns a {@link yfiles.hierarchic.IIncrementalHintsFactory} instance that must be used to obtain hints to be associated with
       * graph elements that should be laid out incrementally.
       * <p>
       * Use this factory and a {@link yfiles.algorithms.IDataProvider} that is registered to the graph using the {@link yfiles.hierarchic.HierarchicLayout#INCREMENTAL_HINTS_DP_KEY}
       * key to associate appropriate hints with the graph elements that should be laid out incrementally by the algorithm.
       * </p>
       * @returns a new {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#createIncrementalHintsFactory
       * @see yfiles.hierarchic.HierarchicLayout#INCREMENTAL_HINTS_DP_KEY
       * @see yfiles.algorithms.Graph#addDataProvider
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @see yfiles.hierarchic.LayoutMode#INCREMENTAL
       */
      createIncrementalHintsFactory():yfiles.hierarchic.IIncrementalHintsFactory;
      /**
       * Creates and registers an incremental layer constraint factory for an {@link yfiles.graph.IGraph}.
       * @param graph The graph for which the factory is created
       * @returns the sequence constraint factory
       */
      createLayerConstraintFactory(graph:yfiles.graph.IGraph):yfiles.hierarchic.ILayerConstraintFactory;
      /**
       * Returns a {@link yfiles.hierarchic.ILayerConstraintFactory} instance that can be used for specifying layer constraints for the
       * given graph.
       * <p>
       * The instance is usually bound to {@link yfiles.algorithms.Graph} instance
       * <code>graph</code>, i.e., if the input graph for the layerer changes, a new instance must be retrieved. This instance can be used for
       * creating constraints for this graph instance.
       * </p>
       * <p>
       * You can create an instance without binding it to a graph instance initially by passing a <code>null</code> parameter. In that case,
       * you <b>must</b> bind the returned instance to the graph, see {@link yfiles.hierarchic.HierarchicLayout#LAYER_CONSTRAINTS_MEMENTO_DP_KEY}
       * and {@link yfiles.hierarchic.ILayerConstraintFactory#memento}.
       * </p>
       * @param graph the input graph
       * @returns a new {@link } instance
       */
      createLayerConstraintFactory(graph:yfiles.algorithms.Graph):yfiles.hierarchic.ILayerConstraintFactory;
      /**
       * Returns a new {@link yfiles.hierarchic.NodeLayoutDescriptor} instance that will be used during the various phases of the layout
       * algorithm to determine the drawing details of the nodes of the graph.
       * <p>
       * This method may be overridden to create a new {@link yfiles.hierarchic.NodeLayoutDescriptor} instance with different
       * configuration settings.
       * </p>
       * @returns a new {@link } instance
       * @protected
       */
      createNodeLayoutDescriptor():yfiles.hierarchic.NodeLayoutDescriptor;
      /**
       * Creates and registers a sequencer constraint factory for an {@link yfiles.graph.IGraph}.
       * @param graph The graph.
       * @returns the sequence constraint factory
       */
      createSequenceConstraintFactory(graph:yfiles.graph.IGraph):yfiles.hierarchic.ISequenceConstraintFactory;
      /**
       * Returns a {@link yfiles.hierarchic.ISequenceConstraintFactory} instance that can be used for specifying sequence constraints for
       * the given graph.
       * <p>
       * For these sequence constraints to have any effect, the {@link yfiles.hierarchic.ISequencer} that determines the in-layer node
       * order (<em>sequence</em>) has to support constraints. Both, {@link yfiles.hierarchic.DefaultLayerSequencer DefaultLayerSequencer} and the incremental sequencer used internally support
       * sequence constraints.
       * </p>
       * @param graph the input graph
       * @returns a {@link } instance
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchSequencer
       * @see yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer
       */
      createSequenceConstraintFactory(graph:yfiles.layout.LayoutGraph):yfiles.hierarchic.ISequenceConstraintFactory;
      /**
       * Disposes of the core layout algorithm.
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayout#applyLayoutCore} after the actual layout is calculated. It
       * may be overridden in order to revert a custom configuration made in
       * {@link yfiles.hierarchic.HierarchicLayout#configureCoreLayout}.
       * </p>
       * <p>
       * This implementation will remove the {@link yfiles.hierarchic.PortCandidateOptimizer} that was created in case a {@link yfiles.algorithms.IDataProvider}
       * is registered with {@link yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY} and no {@link yfiles.hierarchic.HierarchicLayoutCore#portConstraintOptimizer} was initially assigned.
       * </p>
       * @param graph the input graph
       * @param coreLayouter the given core layout algorithm instance
       * @protected
       */
      disposeCoreLayout(graph:yfiles.layout.LayoutGraph,coreLayouter:yfiles.hierarchic.HierarchicLayoutCore):void;
      /**
       * Returns the mirror mask of the {@link yfiles.layout.MultiStageLayout#orientationLayout orientation layouter}.
       * <p>
       * It may be overridden in order to configure a different mirror mask.
       * </p>
       * @returns the mirror mask
       * @protected
       */
      getMirrorMode():yfiles.layout.MirrorModes;
      /**
       * Data provider key for specifying the directedness of edges
       * <p>
       * Generally, the hierarchic layout algorithm assigns nodes to layers such that most of the edges point in the
       * {@link yfiles.layout.MultiStageLayout#layoutOrientation main layout direction}. The directedness of an edge specifies whether it should comply with this strategy. More
       * precisely, a value of <code>1</code> means that the edge should fully comply, a value of <code>-1</code> that it should comply inversely (the edge
       * should point against the main layout direction), and a value of <code>0</code> means that the direction doesn't matter at all and the
       * endpoints of the edges may be placed at the same layer. If there are conflicting preferences, edges with higher absolute
       * values are more likely to point in the desired direction.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static EDGE_DIRECTEDNESS_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for defining the priority of critical edges
       * <p>
       * Critical edges highlight different edge paths that are relevant for a user. The layouter tries to vertically align each
       * node pair that is connected by a critical edge. Conflicts between different critical edges are always resolved in favor
       * of the higher priority.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static CRITICAL_EDGE_PRIORITY_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for specifying the thickness of the edges
       * <p>
       * The specified non-negative thickness is considered when calculating minimum distances so that there are no overlaps
       * between edges and other graph elements. By default, each edge has thickness <code>0</code>.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static EDGE_THICKNESS_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for specifying incremental hints
       * <p>
       * Incremental hints are created using an {@link yfiles.hierarchic.HierarchicLayout#createIncrementalHintsFactory incremental hints factory}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
       * @see yfiles.hierarchic.IIncrementalHintsFactory
       * @const
       * @static
       * @type {yfiles.algorithms.GraphObjectDpKey.<Object>}
       */
      static INCREMENTAL_HINTS_DP_KEY:yfiles.algorithms.GraphObjectDpKey<Object>;
      /**
       * Data acceptor key for publishing the layer IDs for all nodes in the graph
       * @see yfiles.hierarchic.HierarchicLayoutCore#LAYER_INDEX_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static LAYER_INDEX_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data acceptor key for publishing the index inside their layer for all nodes in the graph
       * @see yfiles.hierarchic.HierarchicLayoutCore#SEQUENCE_INDEX_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static SEQUENCE_INDEX_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data provider key for defining swimlanes for the nodes
       * <p>
       * The layout algorithm will arrange nodes in swimlanes according to the registered descriptors.
       * </p>
       * <p>
       * Layout information about the swimlanes is finally written back to the descriptor instances. Instances can be shared
       * among multiple nodes in the same lane.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.hierarchic.SwimlaneDescriptor>}
       */
      static SWIMLANE_DESCRIPTOR_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.hierarchic.SwimlaneDescriptor>;
      /**
       * Data provider key for associating an alternative bounds with the collapsed/expanded group
       * <p>
       * When running in {@link yfiles.hierarchic.LayoutMode#INCREMENTAL incremental}
       * {@link yfiles.hierarchic.HierarchicLayout#layoutMode layout mode}, the alternative bounds of the collapsed/expanded group will be used during the layering and sequencing
       * phase of the algorithm.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#ALTERNATIVE_EDGE_PATH_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.algorithms.YRectangle>}
       */
      static ALTERNATIVE_GROUP_BOUNDS_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.algorithms.YRectangle>;
      /**
       * Data provider key for associating alternative paths for edges connecting to groups, group content or folder nodes
       * <p>
       * When running in {@link yfiles.hierarchic.LayoutMode#INCREMENTAL incremental}
       * {@link yfiles.hierarchic.HierarchicLayout#layoutMode layout mode}, the alternative edge paths are considered during the routing of fixed (i.e., non-incremental) edges.
       * </p>
       * <p>
       * The alternative paths should be used in conjunction with {@link yfiles.hierarchic.HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY alternative group bounds} to achieve more stable layout results
       * when collapsing/expanding a group node as follows:
       * </p>
       * <ol>
       * <li>
       * Collapsing: edges adjacent to the group itself and edges where one of the endpoints (source/target) lies inside the
       * group should get the path <em>before</em> collapsing the group as alternative path. If both endpoints are either inside or outside the
       * group, no alternative path is required.
       * </li>
       * <li>
       * Expanding: edges adjacent to the expanded folder node (which is now a group) should get the path <em>before</em> expanding as
       * alternative path.
       * </li>
       * </ol>
       * @see yfiles.hierarchic.HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.algorithms.YPointPath>}
       */
      static ALTERNATIVE_EDGE_PATH_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.algorithms.YPointPath>;
      /**
       * Data provider key for marking folder nodes
       * <p>
       * When using recursive edge styles in incremental mode, edges will also start at the bottom and end at the top of marked
       * folder nodes. This will keep the edge routes more stable since the connection sides won't change.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static FOLDER_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for storing the constraint graph
       * <p>
       * A <code>v1 before v2</code> constraint is represented as an edge between the representatives of <code>v1</code> and <code>v2</code> in the constraint graph.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.GraphDpKey.<Object>}
       */
      static SEQUENCE_CONSTRAINTS_MEMENTO_DP_KEY:yfiles.algorithms.GraphDpKey<Object>;
      /**
       * Data provider key for storing the constraint graph
       * @const
       * @static
       * @type {yfiles.algorithms.GraphDpKey.<Object>}
       */
      static LAYER_CONSTRAINTS_MEMENTO_DP_KEY:yfiles.algorithms.GraphDpKey<Object>;
      /**
       * Gets or sets the group layer alignment strategy used for recursive group layering.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown group alignment policy is given
       * @see yfiles.hierarchic.HierarchicLayout#recursiveGroupLayering
       * @type {yfiles.hierarchic.GroupAlignmentPolicy}
       */
      groupAlignmentPolicy:yfiles.hierarchic.GroupAlignmentPolicy;
      /**
       * Gets or sets whether or not layer compaction for recursive group layering is active.
       * <p>
       * The number of node layers is decreased if possible without reversing edge directions. The resulting layering tries to
       * keep the layer span of a group node minimum while minimizing the overall vertical space.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#recursiveGroupLayering
       * @type {boolean}
       */
      compactGroups:boolean;
      /**
       * Gets or sets the policy that specifies how to arrange connected components.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified policy does not match a default component arrangement policy
       * @type {yfiles.hierarchic.ComponentArrangementPolicy}
       */
      componentArrangementPolicy:yfiles.hierarchic.ComponentArrangementPolicy;
      /**
       * Gets or sets the time limit (in milliseconds) set for the layout algorithm.
       * <p>
       * Values have to be greater or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets whether or not groups are respected during the layering stage.
       * <p>
       * If this option is enabled, groups are layered recursively, i.e.
       * </p>
       * <ul>
       * <li>nodes in the same group always occupy adjacent layers</li>
       * <li>layer intervals spanned by different group nodes are either disjoint or are nested</li>
       * </ul>
       * <p>
       * If it is disabled, group information is ignored for the layering step.
       * </p>
       * <p>
       * If the graph is flat, this setting is ignored.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#compactGroups
       * @type {boolean}
       */
      recursiveGroupLayering:boolean;
      /**
       * Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
       * <p>
       * When the spacing is negative or zero, no grid is defined, otherwise nodes and edges are placed on multiples of the grid
       * spacing.
       * </p>
       * @see yfiles.hierarchic.IIncrementalHintsFactory
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets or sets whether or not reversed edges should be routed as back-loops.
       * <p>
       * When this option is enabled, back-loops attach to the same node sides as the other edges, i.e., for layout orientation
       * top-to-bottom, they leave their source at the bottom and enter their target at the top.
       * </p>
       * @type {boolean}
       */
      backLoopRouting:boolean;
      /**
       * Gets or sets whether or not self-loops should be routed in a similar manner as back-loops.
       * <p>
       * Self-loops will start at the bottom of their attached nodes and end at the top of them.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#backLoopRouting
       * @type {boolean}
       */
      backLoopRoutingForSelfLoops:boolean;
      /**
       * Gets or sets whether or not edges are grouped automatically.
       * <p>
       * The automatic edge grouping tries to group a high number of edges without changing the semantic of the graph, i.e., it
       * groups edges either at a common source node or a common target node. Edge groupings often allow more compact layouts
       * since grouped edges are routed in a bus-style manner.
       * </p>
       * @type {boolean}
       */
      automaticEdgeGrouping:boolean;
      /**
       * Gets or sets whether or not edges should be routed orthogonally.
       * <p>
       * When orthogonal routing is enabled, all resulting edge paths will be composed of vertical and horizontal segments,
       * only.
       * </p>
       * @type {boolean}
       */
      orthogonalRouting:boolean;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for labels and places them.
       * <p>
       * To define the desired placement for each label add a {@link yfiles.layout.PreferredPlacementDescriptor} on
       * {@link yfiles.layout.IEdgeLabelLayout}.
       * </p>
       * <p>
       * This method is a convenience method that assures that the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm} is of type {@link yfiles.layout.LabelLayoutTranslator}
       * and {@link yfiles.layout.LabelLayoutTranslator#translateEdgeLabels} is set to <code>true</code>.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if integrated labeling should be enabled but the current {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm} is not of type
       * {@link yfiles.layout.LabelLayoutTranslator}
       * @throws {Stubs.Exceptions.InvalidOperationError} if no properly configured {@link yfiles.layout.LabelLayoutTranslator} is registered even though integrated labeling was enabled
       * earlier (can happen when manually specifying the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm}).
       * @type {boolean}
       */
      integratedEdgeLabeling:boolean;
      /**
       * Gets or sets whether or not the layout algorithm considers node labels when calculating node positions to avoid
       * overlaps.
       * <p>
       * This method is a convenience method that assures that the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm} is of type {@link yfiles.layout.LabelLayoutTranslator}
       * and {@link yfiles.layout.LabelLayoutTranslator#translateNodeLabels} is set to <code>true</code>.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if this property should be enabled but the current {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm} is not of type
       * {@link yfiles.layout.LabelLayoutTranslator}
       * @throws {Stubs.Exceptions.InvalidOperationError} if no properly configured {@link yfiles.layout.LabelLayoutTranslator} is registered even though this property was enabled
       * earlier (can happen when manually specifying the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm}).
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets the minimum distance between two adjacent layers.
       * <p>
       * The specified distance should have a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the distance is negative
       * @type {number}
       */
      minimumLayerDistance:number;
      /**
       * Gets or sets whether or not to stop the layout algorithm after the layering step.
       * <p>
       * By then, each node will be assigned to a layer. Since the sequencing and drawing phases are skipped, the order of the
       * nodes within a layer matches the initial order of the nodes, edges won't be routed and the nodes won't get coordinates
       * assigned.
       * </p>
       * <p>
       * The calculated layering information can be retrieved from the {@link yfiles.algorithms.IDataProvider} registered with the key
       * {@link yfiles.hierarchic.HierarchicLayout#LAYER_INDEX_DP_KEY}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#LAYER_INDEX_DP_KEY
       * @type {boolean}
       */
      stopAfterLayering:boolean;
      /**
       * Gets or sets whether or not to stop the layout algorithm after the sequencing step.
       * <p>
       * By then, each node will be assigned to a layer and will have a place in the sequence of nodes in this layer. Since the
       * drawing phase is skipped, edges won't be routed and the nodes won't get coordinates assigned.
       * </p>
       * <p>
       * The calculated layering and sequencing information can be retrieved from the {@link yfiles.algorithms.IDataProvider}s registered
       * with the keys {@link yfiles.hierarchic.HierarchicLayout#LAYER_INDEX_DP_KEY} and
       * {@link yfiles.hierarchic.HierarchicLayout#SEQUENCE_INDEX_DP_KEY}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#SEQUENCE_INDEX_DP_KEY
       * @see yfiles.hierarchic.HierarchicLayout#LAYER_INDEX_DP_KEY
       * @type {boolean}
       */
      stopAfterSequencing:boolean;
      /**
       * Gets or sets the minimum distance between two adjacent nodes in one layer.
       * <p>
       * All values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the distance is negative
       * @throws {Stubs.Exceptions.InvalidOperationError} if the current {@link yfiles.hierarchic.IDrawingDistanceCalculator} is not an instance of
       * {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator}
       * @type {number}
       */
      nodeToNodeDistance:number;
      /**
       * Gets or sets the minimum distance between an edge and an adjacent node in one layer.
       * <p>
       * All values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the distance is negative
       * @throws {Stubs.Exceptions.InvalidOperationError} if the current {@link yfiles.hierarchic.IDrawingDistanceCalculator} is not an instance of
       * {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator}
       * @type {number}
       */
      nodeToEdgeDistance:number;
      /**
       * Gets or sets the minimum distance between two adjacent edges in one layer.
       * <p>
       * All values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the distance is negative
       * @throws {Stubs.Exceptions.InvalidOperationError} if the current {@link yfiles.hierarchic.IDrawingDistanceCalculator} is not an instance of
       * {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator}
       * @type {number}
       */
      edgeToEdgeDistance:number;
      /**
       * Gets or sets whether or not to separate layers.
       * <p>
       * In case layers are not separated, nodes of a layer may extend into adjacent layers. Otherwise, nodes of different layers
       * are strictly separated, i.e., layers are placed below each other.
       * </p>
       * @type {boolean}
       */
      separateLayers:boolean;
      /**
       * Gets the {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator} that is registered with the layout algorithm by default.
       * @throws {Stubs.Exceptions.InvalidOperationError} if the current instance returned by {@link yfiles.hierarchic.HierarchicLayoutCore#drawingDistanceCalculator} is not an instance of {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator}
       * @protected
       * @type {yfiles.hierarchic.DefaultDrawingDistanceCalculator}
       */
      defaultDrawingDistanceCalculator:yfiles.hierarchic.DefaultDrawingDistanceCalculator;
      /**
       * Gets the current layout algorithm instance.
       * <p>
       * By default, this method will return a {@link yfiles.hierarchic.HierarchicLayoutCore} instance created with
       * {@link yfiles.hierarchic.HierarchicLayout#createHierarchicLayoutCore}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#createHierarchicLayoutCore
       * @type {yfiles.hierarchic.HierarchicLayoutCore}
       */
      hierarchicLayoutCore:yfiles.hierarchic.HierarchicLayoutCore;
      /**
       * Gets or sets a predefined layering strategy for the {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayerer from scratch} layerer.
       * <p>
       * The layouter assigns the nodes to separate layers. The nodes within each layer will be placed on the same horizontal
       * layer. The layers will be arranged vertically starting with the small-numbered layers.
       * </p>
       * <p>
       * An important layering strategy for the hierarchic layout style is called Hierarchical Layering. A hierarchical layering
       * tries to assign nodes to layers in a way such that as much edges of the graph as possible will point to the main layout
       * direction, i.e., the start nodes of the edges will be in a layer with a smaller number than the corresponding end nodes.
       * Also, a hierarchical layering will never put two connected nodes in the same layer.
       * </p>
       * <p>
       * This method wraps the internal implementations into a {@link yfiles.hierarchic.MultiComponentLayerer} instance so that it is
       * possible to specify the behavior of the algorithm if the {@link yfiles.layout.MultiStageLayout#componentLayoutEnabled component layouter} is disabled.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown strategy is given
       * @type {yfiles.hierarchic.LayeringStrategy}
       */
      fromScratchLayeringStrategy:yfiles.hierarchic.LayeringStrategy;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ILayerer} instance that obtains the layering for the nodes if the layout algorithm
       * runs in {@link yfiles.hierarchic.LayoutMode#FROM_SCRATCH From Scratch mode}.
       * <p>
       * If the graph consists of multiple components, then the {@link yfiles.hierarchic.ILayerer} instance should be wrapped in
       * {@link yfiles.hierarchic.MultiComponentLayerer}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the {@link yfiles.hierarchic.ILayerer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @type {yfiles.hierarchic.ILayerer}
       */
      fromScratchLayerer:yfiles.hierarchic.ILayerer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ISequencer} instance that calculates the node sequence if the layout algorithm runs in
       * {@link yfiles.hierarchic.LayoutMode#FROM_SCRATCH From Scratch mode}.
       * @throws {Stubs.Exceptions.ArgumentError} if the {@link yfiles.hierarchic.ISequencer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayerer
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @type {yfiles.hierarchic.ISequencer}
       */
      fromScratchSequencer:yfiles.hierarchic.ISequencer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.INodePlacer} instance that will calculate the final node placement of the layout.
       * @throws {Stubs.Exceptions.ArgumentError} if the {@link yfiles.hierarchic.INodePlacer} is <code>null</code>
       * @type {yfiles.hierarchic.INodePlacer}
       */
      nodePlacer:yfiles.hierarchic.INodePlacer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ILayerer} instance that obtains the layering for fixed nodes during the incremental
       * layout run.
       * @throws {Stubs.Exceptions.ArgumentError} if the {@link yfiles.hierarchic.ILayerer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @type {yfiles.hierarchic.ILayerer}
       */
      fixedElementsLayerer:yfiles.hierarchic.ILayerer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ISequencer} instance that calculates the sequence of the fixed nodes during the
       * incremental layout run.
       * @throws {Stubs.Exceptions.ArgumentError} if the given {@link yfiles.hierarchic.ISequencer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayout#layoutMode
       * @type {yfiles.hierarchic.ISequencer}
       */
      fixedElementsSequencer:yfiles.hierarchic.ISequencer;
      /**
       * Gets or sets the layout mode this layouter should use for upcoming layouts.
       * <p>
       * Depending on the mode the layout algorithm will use different {@link yfiles.hierarchic.ILayerer} and {@link yfiles.hierarchic.ISequencer}
       * implementations.
       * <br />
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given layout mode is unknown
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayerer
       * @see yfiles.hierarchic.HierarchicLayout#fromScratchSequencer
       * @see yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer
       * @see yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer
       * @type {yfiles.hierarchic.LayoutMode}
       */
      layoutMode:yfiles.hierarchic.LayoutMode;
      /**
       * Gets or sets the {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance used for all those edges that do not have a specific
       * layout descriptor assigned.
       * <p>
       * By default, this method will return a {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance created with
       * {@link yfiles.hierarchic.HierarchicLayout#createEdgeLayoutDescriptor}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the {@link yfiles.hierarchic.EdgeLayoutDescriptor} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.hierarchic.EdgeLayoutDescriptor}
       */
      edgeLayoutDescriptor:yfiles.hierarchic.EdgeLayoutDescriptor;
      /**
       * Gets or sets the {@link yfiles.hierarchic.NodeLayoutDescriptor} instance used for all those nodes that do not have a specific
       * layout descriptor assigned.
       * <p>
       * By default, this method will return a {@link yfiles.hierarchic.NodeLayoutDescriptor} instance created with
       * {@link yfiles.hierarchic.HierarchicLayout#createNodeLayoutDescriptor}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given {@link yfiles.hierarchic.NodeLayoutDescriptor} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
       * @see yfiles.hierarchic.HierarchicLayout#createNodeLayoutDescriptor
       * @type {yfiles.hierarchic.NodeLayoutDescriptor}
       */
      nodeLayoutDescriptor:yfiles.hierarchic.NodeLayoutDescriptor;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class arranges graphs in a hierarchic/layered fashion and supports complete re-layout as well as incremental graph
     * layout.
     * <p>
     * Note that class {@link yfiles.hierarchic.HierarchicLayout} offers a facade to this class which is easier to use. Hence, we
     * strongly recommend to not directly use class {@link yfiles.hierarchic.HierarchicLayoutCore}.
     * </p>
     * <p>
     * <b>Layout Style</b> The nodes are distributed into layers such that most of the edges point to the main layout direction. The order of the
     * nodes within the layers ensures that the number of edge crossings is as small as possible. There are different edge
     * routing styles available. Edges can be orthogonal, polyline or octilinear.
     * </p>
     * <p>
     * Hierarchical diagrams are commonly used for the visualization of hierarchical data, since they facilitate the
     * identification of dependencies and relationships among the nodes of the graph. Possible application domains are the
     * following: workflow visualization, call graph visualization, entity-relationship diagrams, biochemical pathways and
     * network management.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[37.5,37.5],"d":1},"n":[[137.81,67.5],[137.81,135],[718.44,67.5],[147.19,202.5],[305.62,135],[296.25,202.5],[771.88,450],[354.38,285],[290,367.5],[366.88,367.5],[400.62,67.5],[709.06,135],[709.06,202.5],[366.88,450],[79.69,202.5],[492.5,285],[501.88,367.5],[627.5,285],[357.5,532.5],[363.75,202.5],[286.88,285],[425,532.5],[400.62,135],[569.38,450],[569.38,367.5],[155,450],[434.38,367.5],[636.88,367.5],[84.38,285],[151.88,285],[222.5,532.5],[290,532.5],[155,532.5],[87.5,532.5],[20,532.5],[219.38,285],[69.38,367.5],[713.75,285],[704.38,367.5],[771.88,367.5],[848.75,285],[906.88,367.5],[839.38,367.5],[400.62,0]],"e":[[0,1,0,18.75,0,-18.75],[1,3,9.38,18.75,0,-18.75],[0,4,12.5,18.75,0,-18.75,[169.06,120,324.38,120]],[4,5,-9.38,18.75,0,-18.75],[2,6,9.38,18.75,12.5,-18.75,[746.56,120,959.38,120,959.38,420,803.12,420]],[5,7,9.38,18.75,0,-18.75,[324.38,255,373.12,255]],[7,8,0,18.75,0,-18.75,[373.12,352.5,308.75,352.5]],[7,9,12.5,18.75,0,-18.75],[2,11,-9.38,18.75,0,-18.75],[11,12,0,18.75,0,-18.75],[9,13,0,18.75,0,-18.75],[1,14,-9.38,18.75,0,-18.75,[147.19,187.5,98.44,187.5]],[12,15,-14.06,18.75,0,-18.75,[713.75,255,511.25,255]],[15,16,9.38,18.75,0,-18.75],[12,17,-4.69,18.75,0,-18.75,[723.13,270,646.25,270]],[13,18,-9.38,18.75,0,-18.75],[4,19,9.38,18.75,0,-18.75,[333.75,187.5,382.5,187.5]],[5,20,-9.38,18.75,0,-18.75],[13,21,9.38,18.75,0,-18.75,[395,502.5,443.75,502.5]],[10,22,0,18.75,0,-18.75],[22,23,0,18.75,-12.5,-18.75,[419.38,420,575.62,420]],[17,24,-9.38,18.75,0,-18.75,[636.88,337.5,588.12,337.5]],[24,23,0,18.75,0,-18.75],[0,25,-12.5,18.75,-9.38,-18.75,[144.06,120,54.38,120,54.38,420,164.38,420]],[8,25,-9.38,18.75,9.38,-18.75,[299.38,420,183.12,420]],[15,26,-9.38,18.75,0,-18.75,[501.88,337.5,453.12,337.5]],[17,27,9.38,18.75,0,-18.75],[27,6,0,18.75,-12.5,-18.75,[655.62,420,778.12,420]],[3,28,-4.69,18.75,0,-18.75,[161.25,270,103.12,270]],[3,29,4.69,18.75,0,-18.75],[25,30,7.5,18.75,0,-18.75,[181.25,517.5,241.25,517.5]],[25,31,15,18.75,-9.38,-18.75,[188.75,502.5,299.38,502.5]],[8,31,9.38,18.75,9.38,-18.75],[25,32,0,18.75,0,-18.75],[25,33,-7.5,18.75,0,-18.75,[166.25,517.5,106.25,517.5]],[25,34,-15,18.75,0,-18.75,[158.75,502.5,38.75,502.5]],[3,35,14.06,18.75,0,-18.75,[180,255,238.12,255]],[3,36,-14.06,18.75,-9.38,-18.75,[151.88,255,69.38,255,69.38,337.5,78.75,337.5]],[7,36,-12.5,18.75,9.38,-18.75,[360.62,337.5,97.5,337.5]],[12,37,4.69,18.75,0,-18.75],[37,38,-9.38,18.75,0,-18.75],[38,23,0,18.75,12.5,-18.75,[723.12,435,600.62,435]],[37,39,9.38,18.75,0,-18.75,[741.88,337.5,790.62,337.5]],[39,6,0,18.75,0,-18.75],[12,40,14.06,18.75,0,-18.75,[741.88,255,867.5,255]],[40,41,9.38,18.75,0,-18.75,[876.88,337.5,925.62,337.5]],[40,42,-9.38,18.75,0,-18.75],[43,0,-12.5,18.75,0,-18.75,[406.88,52.5,156.56,52.5]],[43,2,12.5,18.75,0,-18.75,[431.88,52.5,737.19,52.5]],[43,10,0,18.75,0,-18.75]],"vp":[20.0,0.0,940.0,570.0]}} Hierarchic Layout obtained with default settings<b>Concept</b>The layout algorithm runs in three main phases:
     * </p>
     * <ul>
     * <li>
     * <b>Layering</b> – The nodes are distributed into layers by means of
     * {@link yfiles.hierarchic.HierarchicLayoutCore#layerer}. If the layout orientation is top-to-bottom, the nodes in each layer are arranged horizontally while the layers are
     * ordered vertically top-to-bottom.
     * </li>
     * <li>
     * <b>Sequencing</b> – The order of the nodes in each layer is determined such that the number of edge crossings is as small as possible. To
     * specify the sequencing algorithm, use {@link yfiles.hierarchic.HierarchicLayoutCore#sequencer}.
     * </li>
     * <li><b>Drawing</b> – The layout algorithm assigns the final coordinates to all nodes and routes the edges.</li>
     * </ul>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * In order to customize this layout algorithm, modify the {@link yfiles.hierarchic.HierarchicLayoutCore#layerer Layerer}, {@link yfiles.hierarchic.HierarchicLayoutCore#sequencer Sequencer}, {@link yfiles.hierarchic.HierarchicLayoutCore#portAllocator PortAllocator}, {@link yfiles.hierarchic.HierarchicLayoutCore#drawingDistanceCalculator DrawingDistanceCalculator},
     * and {@link yfiles.hierarchic.HierarchicLayoutCore#nodePlacer NodePlacer} instances.
     * </p>
     * <p>
     * This layout algorithm is able to incrementally add new elements to an existing drawing of a graph. In order to add
     * elements incrementally, register a {@link yfiles.algorithms.IDataProvider} with the graph using the {@link yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}
     * DataProvider key and associate the hints obtained from the {@link yfiles.hierarchic.IIncrementalHintsFactory} with the elements
     * to be added incrementally.
     * </p>
     * <p>
     * {@link yfiles.hierarchic.NodeLayoutDescriptor} and {@link yfiles.hierarchic.EdgeLayoutDescriptor} instances can be used for specifying
     * individual information (e.g. distances or routing styles) for each node and edge in the graph. The descriptors are bound
     * to the graph using {@link yfiles.algorithms.IDataProvider}s registered with {@link yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}
     * or {@link yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
     * </p>
     * <p>
     * {@link yfiles.hierarchic.HierarchicLayoutCore} supports two approaches to connect edges on a specific side or even an exact
     * location to a node. {@link yfiles.layout.PortConstraint}s define a single constraint for the ports of an edge. To realize more
     * complex port restrictions, several {@link yfiles.layout.PortCandidate}s or {@link yfiles.layout.PortCandidateSet}s can be assigned to
     * edges or nodes. If an edge with registered {@link yfiles.layout.PortCandidate}s connects to nodes with
     * {@link yfiles.layout.PortCandidateSet}s, the layouter will try to match both collections to find an appropriate port. In case
     * there is no matching port candidate, a {@link yfiles.layout.PortCandidate} specified for the edge is preferred. Since their
     * simultaneous existence at the same node may be ambiguous, it is not recommended to use a combination of
     * {@link yfiles.layout.PortConstraint}s and {@link yfiles.layout.PortCandidate}s in the same layout.
     * </p>
     * <p>
     * This algorithm specifies a {@link yfiles.layout.LabelLayoutTranslator} instance as the current {@link yfiles.layout.MultiStageLayout#labeling} and configures it such that the
     * node labels are passed to it by setting {@link yfiles.layout.LabelLayoutTranslator#translateNodeLabels} to <code>true</code>.
     * </p>
     * <p>
     * In order to enable integrated edge labeling of this algorithm, make sure {@link yfiles.layout.LabelLayoutTranslator} is
     * registered and edge labels are being translated and written back after the layout.
     * </p>
     * <p>
     * Moreover, this algorithm supports layer and sequence constraints. These constraints can be specified using a {@link yfiles.hierarchic.ILayerConstraintFactory}
     * and {@link yfiles.hierarchic.ISequenceConstraintFactory}, respectively.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayout
     * @see yfiles.hierarchic.ILayerer
     * @see yfiles.hierarchic.ISequencer
     * @see yfiles.hierarchic.IPortAllocator
     * @see yfiles.hierarchic.IDrawingDistanceCalculator
     * @see yfiles.hierarchic.INodePlacer
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface HierarchicLayoutCore extends yfiles.layout.MultiStageLayout{}
    export class HierarchicLayoutCore {
      /**
       * Creates a new {@link yfiles.hierarchic.HierarchicLayoutCore} instance with the default settings.
       * @see yfiles.hierarchic.HierarchicLayoutCore#createLayerer
       * @see yfiles.hierarchic.HierarchicLayoutCore#createIncrementalLayerer
       * @see yfiles.hierarchic.HierarchicLayoutCore#createSequencer
       * @see yfiles.hierarchic.HierarchicLayoutCore#createDrawingDistanceCalculator
       * @see yfiles.hierarchic.HierarchicLayoutCore#createNodePlacer
       */
      constructor();
      /**
       * Factory method that is called lazily upon first usage by {@link yfiles.hierarchic.HierarchicLayoutCore#drawingDistanceCalculator} if the given {@link yfiles.hierarchic.IDrawingDistanceCalculator}
       * instance is <code>null</code>.
       * <p>
       * The default implementation returns a {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator} instance. It may be overridden
       * to return a custom {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator} instance.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#drawingDistanceCalculator
       * @protected
       */
      createDrawingDistanceCalculator():yfiles.hierarchic.IDrawingDistanceCalculator;
      /**
       * Factory method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and reates an appropriate {@link yfiles.hierarchic.IEdgeReverser}
       * implementation.
       * <p>
       * By default, a suitable private implementation of {@link yfiles.hierarchic.IEdgeReverser} is returned. It may be overridden to
       * return a custom {@link yfiles.hierarchic.IEdgeReverser} implementation.
       * </p>
       * @returns an implementation
       * @protected
       */
      createEdgeReverser():yfiles.hierarchic.IEdgeReverser;
      /**
       * Factory method that is called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and creates a {@link yfiles.layout.GroupingSupport}
       * instance in case of a grouped graph.
       * @param graph the input graph
       * @returns a {@link } instance or <code>null</code> if the graph is not grouped
       * @protected
       */
      createGrouping(graph:yfiles.layout.LayoutGraph):yfiles.layout.GroupingSupport;
      /**
       * Creates an {@link yfiles.hierarchic.IIncrementalHintsFactory} implementation used for obtaining hint objects associated with
       * nodes and edges of the graph prior to the invocation of the layout algorithm.
       * <p>
       * Incremental hints can be obtained from a {@link yfiles.algorithms.IDataProvider} that is registered with the graph using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}.
       * </p>
       * @returns an {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
       */
      createIncrementalHintsFactory():yfiles.hierarchic.IIncrementalHintsFactory;
      /**
       * Factory method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and creates an appropriate {@link yfiles.hierarchic.ILayerer}
       * implementation.
       * <p>
       * By default, a {@link yfiles.hierarchic.ConstraintIncrementalLayerer} instance with a {@link yfiles.hierarchic.TopologicalLayerer}
       * instance as argument is returned. This method may be overridden to return a custom {@link yfiles.hierarchic.ILayerer}
       * implementation.
       * </p>
       * @returns a {@link } implementation
       * @protected
       */
      createIncrementalLayerer():yfiles.hierarchic.ILayerer;
      /**
       * Callback method that is called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore}, creates the {@link yfiles.hierarchic.INodeData}
       * and {@link yfiles.hierarchic.IEdgeData} instances and binds them to the elements using the given {@link yfiles.hierarchic.IItemFactory itemFactory}.
       * @param g the input graph
       * @param itemFactory the given {@link } instance
       * @protected
       */
      createItemData(g:yfiles.layout.LayoutGraph,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Creates a {@link yfiles.hierarchic.ILayerConstraintFactory layer constraint factory} that allows to create hints that affect the assignment of the nodes to layers.
       * <p>
       * By default, a suitable private implementation of {@link yfiles.hierarchic.ILayerConstraintFactory} is returned.
       * </p>
       * @param graph the given graph
       * @returns a {@link } instance
       */
      createLayerConstraintFactory(graph:yfiles.algorithms.Graph):yfiles.hierarchic.ILayerConstraintFactory;
      /**
       * Factory method that is called lazily upon first usage by {@link yfiles.hierarchic.HierarchicLayoutCore#layerer} if no {@link yfiles.hierarchic.ILayerer} instance is given as input.
       * <p>
       * The default implementation returns a <code>new MultiComponentLayerer(new WeightedLayerer())</code> instance. It may be overridden to
       * return a custom {@link yfiles.hierarchic.ILayerer} instance.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#layerer
       * @see yfiles.hierarchic.MultiComponentLayerer
       * @see yfiles.hierarchic.WeightedLayerer
       * @protected
       */
      createLayerer():yfiles.hierarchic.ILayerer;
      /**
       * Factory method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and creates an appropriate {@link yfiles.hierarchic.ILayers}
       * implementation using the given {@link yfiles.hierarchic.ILayoutDataProvider} instance.
       * <p>
       * By default, a suitable private implementation of {@link yfiles.hierarchic.ILayers} is returned. It may be overridden to return
       * custom implementations of the {@link yfiles.hierarchic.ILayers} interface.
       * </p>
       * @param ldp the given {@link } instance
       * @returns a {@link } implementation
       * @protected
       */
      createLayers(ldp:yfiles.hierarchic.ILayoutDataProvider):yfiles.hierarchic.ILayers;
      /**
       * Factory method that is called lazily upon first usage by {@link yfiles.hierarchic.HierarchicLayoutCore#nodePlacer} if the given {@link yfiles.hierarchic.INodePlacer} instance is <code>null</code>.
       * <p>
       * The default implementation returns a {@link yfiles.hierarchic.SimplexNodePlacer} instance. It may be overridden to return a
       * custom {@link yfiles.hierarchic.INodePlacer} instance.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#nodePlacer
       * @see yfiles.hierarchic.SimplexNodePlacer
       * @protected
       */
      createNodePlacer():yfiles.hierarchic.INodePlacer;
      /**
       * Factory method that is called lazily upon first usage by {@link yfiles.hierarchic.HierarchicLayoutCore#portAllocator} if the given {@link yfiles.hierarchic.IPortAllocator} instance is
       * <code>null</code>.
       * <p>
       * The default implementation returns a {@link yfiles.hierarchic.DefaultPortAllocator} instance. It may be overridden to return a
       * custom {@link yfiles.hierarchic.IPortAllocator} instance.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#portAllocator
       * @protected
       */
      createPortAllocator():yfiles.hierarchic.IPortAllocator;
      /**
       * Factory method that is called lazily upon first usage by {@link yfiles.hierarchic.HierarchicLayoutCore#portConstraintOptimizer} if the given {@link yfiles.hierarchic.IPortConstraintOptimizer}
       * instance is <code>null</code>.
       * <p>
       * The default implementation returns <code>null</code>. It may be overridden to return a custom {@link yfiles.hierarchic.IPortConstraintOptimizer}
       * instance.
       * </p>
       * @returns <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#portConstraintOptimizer
       * @protected
       */
      createPortConstraintOptimizer():yfiles.hierarchic.IPortConstraintOptimizer;
      /**
       * Creates sequence constraints that affect the sequence of the nodes within each layer.
       * <p>
       * By default, a suitable private implementation of {@link yfiles.hierarchic.ISequenceConstraintFactory} is returned.
       * </p>
       * @param graph the given graph
       * @returns a {@link } instance
       */
      createSequenceConstraintFactory(graph:yfiles.algorithms.Graph):yfiles.hierarchic.ISequenceConstraintFactory;
      /**
       * Factory method that is called lazily upon first usage by {@link yfiles.hierarchic.HierarchicLayoutCore#sequencer} if the given {@link yfiles.hierarchic.ISequencer} instance is <code>null</code>.
       * <p>
       * The default implementation returns a {@link yfiles.hierarchic.DefaultLayerSequencer} instance. It may be overridden to return a
       * custom {@link yfiles.hierarchic.ISequencer} instance.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.hierarchic.HierarchicLayoutCore#sequencer
       * @see yfiles.hierarchic.DefaultLayerSequencer
       * @protected
       */
      createSequencer():yfiles.hierarchic.ISequencer;
      /**
       * Factory method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and creates a {@link yfiles.hierarchic.ISequencer}
       * implementation that can sequence subgraphs incrementally.
       * <p>
       * By default, a suitable private implementation of {@link yfiles.hierarchic.ISequencer} is returned. It may be overridden to
       * return custom implementations of the {@link yfiles.hierarchic.ISequencer} interface.
       * </p>
       * @returns a {@link } implementation
       * @protected
       */
      createSubgraphLayerSequencer():yfiles.hierarchic.ISequencer;
      /**
       * Provides access to implementation specific properties of the algorithms used.
       * <p>
       * Used for internal purposes.
       * </p>
       * @param key the key to a property
       * @returns the associated value or <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#setAlgorithmProperty
       */
      getAlgorithmProperty(key:Object):Object;
      /**
       * Callback method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and returns a {@link yfiles.algorithms.IDataProvider}
       * that holds the {@link yfiles.hierarchic.EdgeLayoutDescriptor} information.
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} holding the information about the node layout is registered with the graph using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @returns a {@link } instance or <code>null</code>
       * @see yfiles.hierarchic.EdgeLayoutDescriptor
       * @see yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @protected
       */
      getEdgeLayoutDescriptors(graph:yfiles.layout.LayoutGraph):yfiles.algorithms.IDataProvider;
      /**
       * Callback method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and returns a {@link yfiles.algorithms.IDataProvider}
       * that holds the incremental hints information.
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} holding the information about incremental hints is registered with the graph using
       * key {@link yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @returns a {@link } instance or <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
       * @protected
       */
      getIncrementalHints(graph:yfiles.layout.LayoutGraph):yfiles.algorithms.IDataProvider;
      /**
       * Callback method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and returns a {@link yfiles.algorithms.IDataProvider}
       * that holds the {@link yfiles.hierarchic.NodeLayoutDescriptor} information.
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} holding the information about the node layout is registered with the graph using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @returns a {@link } instance or <code>null</code>
       * @see yfiles.hierarchic.NodeLayoutDescriptor
       * @see yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
       * @protected
       */
      getNodeLayoutDescriptors(graph:yfiles.layout.LayoutGraph):yfiles.algorithms.IDataProvider;
      /**
       * Callback method called during {@link yfiles.hierarchic.HierarchicLayoutCore#applyLayoutCore} and returns a {@link yfiles.algorithms.IDataProvider}
       * that holds the {@link yfiles.hierarchic.SwimlaneDescriptor} information.
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} holding the information about swimlanes is registered with the graph using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @returns a {@link } instance or <code>null</code>
       * @see yfiles.hierarchic.SwimlaneDescriptor
       * @see yfiles.hierarchic.HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY
       * @protected
       */
      getSwimLaneDescriptors(graph:yfiles.layout.LayoutGraph):yfiles.algorithms.IDataProvider;
      /**
       * Callback method that publishes the layering information.
       * <p>
       * The layering information is stored in a {@link yfiles.algorithms.IDataProvider} registered with the graph using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#LAYER_INDEX_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param layers the {@link } implementation that provides the layering information
       * @see yfiles.hierarchic.HierarchicLayoutCore#LAYER_INDEX_DP_KEY
       * @protected
       */
      publishLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers):void;
      /**
       * Callback method that publishes the sequencing information.
       * <p>
       * The sequencing information is stored in a {@link yfiles.algorithms.IDataProvider} registered with the graph using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#SEQUENCE_INDEX_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param layers the {@link } implementation that provides the layering information
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @see yfiles.hierarchic.HierarchicLayoutCore#LAYER_INDEX_DP_KEY
       * @protected
       */
      publishSequences(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Removes the bends which are obviously not necessary.
       * <p>
       * By default, this method removes collinear bends from the graph. A bend point is considered to be collinear if it lies on
       * a line with its preceding and succeeding bend point. It may be overridden for a custom implementation of bend removal.
       * </p>
       * @param graph the input graph
       * @param bendReductionThreshold this method removes a bend, if its horizontal/vertical distance to the closest collinear bend is above this value
       * @protected
       */
      reduceBendCount(graph:yfiles.layout.LayoutGraph,bendReductionThreshold:number):void;
      /**
       * Removes the bends which are obviously not necessary.
       * <p>
       * By default, this method removes collinear bends from the graph. A bend point is considered to be collinear if it lies on
       * a line with its preceding and succeeding bend point. It may be overridden for a custom implementation of bend removal.
       * </p>
       * @param graph the input graph
       * @protected
       */
      reduceBendCount(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Provides access to implementation specific properties of the algorithms used.
       * <p>
       * Used for internal purposes.
       * </p>
       * @param key the key to a property
       * @param value the value to associate with the key
       * @see yfiles.hierarchic.HierarchicLayoutCore#setAlgorithmProperty
       */
      setAlgorithmProperty(key:Object,value:Object):void;
      /**
       * Data provider key for specifying incremental hints
       * <p>
       * Incremental hints are created using an {@link yfiles.hierarchic.HierarchicLayoutCore#createIncrementalHintsFactory incremental hints factory}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY
       * @see yfiles.hierarchic.IIncrementalHintsFactory
       * @const
       * @static
       * @type {yfiles.algorithms.GraphObjectDpKey.<Object>}
       */
      static INCREMENTAL_HINTS_DP_KEY:yfiles.algorithms.GraphObjectDpKey<Object>;
      /**
       * Data provider key for providing layout information for each edge
       * @see yfiles.hierarchic.IEdgeData#edgeLayoutDescriptor
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.hierarchic.EdgeLayoutDescriptor>}
       */
      static EDGE_LAYOUT_DESCRIPTOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.hierarchic.EdgeLayoutDescriptor>;
      /**
       * Data provider key for providing layout information for each node
       * @see yfiles.hierarchic.INodeData#nodeLayoutDescriptor
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.hierarchic.NodeLayoutDescriptor>}
       */
      static NODE_LAYOUT_DESCRIPTOR_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.hierarchic.NodeLayoutDescriptor>;
      /**
       * Data provider key for providing information about swimlanes
       * <p>
       * If this key is present during the layout, the layout algorithm will arrange nodes in swimlanes. The information about
       * the swimlanes is finally written back to the descriptor instances.
       * </p>
       * <p>
       * Instances can be shared among multiple nodes in the same lane, but don't have to be shared.
       * </p>
       * @see yfiles.hierarchic.SwimlaneDescriptor
       * @see yfiles.hierarchic.INodeData#swimLaneDescriptor
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.hierarchic.SwimlaneDescriptor>}
       */
      static SWIMLANE_DESCRIPTOR_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.hierarchic.SwimlaneDescriptor>;
      /**
       * Data acceptor key for publishing the layer IDs for all nodes in the graph
       * @see yfiles.hierarchic.HierarchicLayout#LAYER_INDEX_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static LAYER_INDEX_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data acceptor key for publishing the index inside their layer for all nodes in the graph
       * @see yfiles.hierarchic.HierarchicLayout#SEQUENCE_INDEX_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static SEQUENCE_INDEX_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data provider key for internal use only
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static INCREMENTAL_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Factory method called during the initialization of this {@link yfiles.hierarchic.HierarchicLayoutCore} instance that creates a
       * {@link yfiles.layout.MultiStageLayout#labeling label layouter}.
       * <p>
       * In order to use the integrated node label awareness feature one has to use a {@link yfiles.layout.LabelLayoutTranslator}
       * instance and set {@link yfiles.layout.LabelLayoutTranslator#translateNodeLabels} to <code>true</code>. Likewise, in order to make use of the integrated edge labeling {@link yfiles.layout.LabelLayoutTranslator#translateEdgeLabels} must be set to <code>true</code> and also
       * {@link yfiles.layout.LabelLayoutTranslator#writeBackEdgeLabels} must be set to <code>true</code> (which is the default).
       * </p>
       * @protected
       * @type {yfiles.layout.ILayoutStage}
       */
      createLabelLayout:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets whether or not the {@link yfiles.layout.ComponentLayout} should be enabled.
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Gets or sets the time limit (in milliseconds) set for the layout algorithm.
       * <p>
       * Values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ILayerer} instance responsible for generating the layer assignment (first phase of the
       * layout algorithm).
       * <p>
       * To set the default layerer, see {@link yfiles.hierarchic.HierarchicLayoutCore#createLayerer}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.ILayerer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createLayerer
       * @type {yfiles.hierarchic.ILayerer}
       */
      layerer:yfiles.hierarchic.ILayerer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ISequencer} instance responsible for calculating the sequence of nodes within each
       * layer (second phase of the layout algorithm).
       * <p>
       * To set the default sequencer, see {@link yfiles.hierarchic.HierarchicLayoutCore#createSequencer}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.ISequencer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createSequencer
       * @type {yfiles.hierarchic.ISequencer}
       */
      sequencer:yfiles.hierarchic.ISequencer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.INodePlacer} instance responsible for the assignment of the x-(sequence) and
       * preliminary y-coordinates of the nodes.
       * <p>
       * To set the default node placer, see {@link yfiles.hierarchic.HierarchicLayoutCore#createNodePlacer}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.INodePlacer} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createNodePlacer
       * @type {yfiles.hierarchic.INodePlacer}
       */
      nodePlacer:yfiles.hierarchic.INodePlacer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.IPortAllocator} instance responsible for assigning port coordinates to the adjacent
       * edges of each node.
       * <p>
       * To set the default port allocator, see {@link yfiles.hierarchic.HierarchicLayoutCore#createPortAllocator}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.IPortAllocator} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createPortAllocator
       * @type {yfiles.hierarchic.IPortAllocator}
       */
      portAllocator:yfiles.hierarchic.IPortAllocator;
      /**
       * Gets or sets the {@link yfiles.hierarchic.IPortConstraintOptimizer} instance responsible for optimizing the assignment of port
       * constraints to the adjacent edges of each node.
       * <p>
       * To set the default port allocator, see {@link yfiles.hierarchic.HierarchicLayoutCore#createPortConstraintOptimizer}.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createPortConstraintOptimizer
       * @type {yfiles.hierarchic.IPortConstraintOptimizer}
       */
      portConstraintOptimizer:yfiles.hierarchic.IPortConstraintOptimizer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.IDrawingDistanceCalculator} instance responsible for determining the minimum distances
       * between elements that belong to the same layer.
       * <p>
       * To set the default port allocator, see {@link yfiles.hierarchic.HierarchicLayoutCore#createDrawingDistanceCalculator}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.IDrawingDistanceCalculator} is <code>null</code>
       * @see yfiles.hierarchic.HierarchicLayoutCore#createDrawingDistanceCalculator
       * @type {yfiles.hierarchic.IDrawingDistanceCalculator}
       */
      drawingDistanceCalculator:yfiles.hierarchic.IDrawingDistanceCalculator;
      /**
       * Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
       * <p>
       * When the spacing is negative or zero, no grid is defined, otherwise nodes and edges are placed on multiples of the grid
       * spacing.
       * </p>
       * @see yfiles.hierarchic.IIncrementalHintsFactory
       * @type {number}
       */
      gridSpacing:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This interface determines the minimum distances between elements that belong to the same layer.
     * <p>
     * {@link yfiles.hierarchic.IDrawingDistanceCalculator} is implemented by a helper class used by {@link yfiles.hierarchic.HierarchicLayoutCore}
     * during the phase of node placement (i.e. when the final coordinates for all nodes are calculated). {@link yfiles.hierarchic.INodePlacer}
     * uses instances of this interface to determine the minimum distances between elements that belong to the same layer.
     * </p>
     * <p>
     * Information about graph elements (i.e. nodes or edges) is given using a {@link yfiles.hierarchic.ILayoutDataProvider} while
     * information about layers is given through {@link yfiles.hierarchic.ILayer} (describes a single layer of the hierarchic layout)
     * or {@link yfiles.hierarchic.ILayers} (manages multiple {@link yfiles.hierarchic.ILayer layers} of the hierarchic layout).
     * </p>
     * <p>
     * Implementations not needed any more should be {@link yfiles.hierarchic.IDrawingDistanceCalculator#dispose disposed of}.
     * </p>
     * <p>
     * The default implementation used by the {@link yfiles.hierarchic.HierarchicLayoutCore} is
     * {@link yfiles.hierarchic.DefaultDrawingDistanceCalculator}.
     * </p>
     * @interface
     */
    export interface IDrawingDistanceCalculator extends Object{
      /**
       * Disposes of internal data structures.
       * <p>
       * Implementations should release internally held data structures when they are not needed any more.
       * </p>
       * @param graph the input graph that contains all elements that are used during the node placement
       * @param layers the {@link } object that will be used during subsequent calls
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @see yfiles.hierarchic.IDrawingDistanceCalculator#initialize
       * @abstract
       */
      dispose(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Main interface method called by {@link yfiles.hierarchic.INodePlacer} instances to determine the minimum distance between
       * {@link yfiles.algorithms.Node}s of same layer.
       * @param graph the input graph
       * @param layer the {@link } object that will be used during subsequent calls
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param left the left of the two {@link }s whose minimum distance is to be determined or <code>null</code> if only the left border
       * of the right {@link } is of interest
       * @param right the right of the two {@link }s whose minimum distance is to be determined or <code>null</code> if only the right border
       * of the left {@link } is of interest
       * @returns the minimum distance between two {@link }s of the same layer
       * @abstract
       */
      getMinDistance(graph:yfiles.layout.LayoutGraph,layer:yfiles.hierarchic.ILayer,ldp:yfiles.hierarchic.ILayoutDataProvider,left:yfiles.algorithms.Node,right:yfiles.algorithms.Node):number;
      /**
       * Initializes internal data structures.
       * <p>
       * Implementations may cache lengthy calculations in the initialization phase.
       * </p>
       * @param graph the input graph that contains all elements that are used during the node placement
       * @param layers the {@link } object that will be used during subsequent calls
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @see yfiles.hierarchic.IDrawingDistanceCalculator#dispose
       * @abstract
       */
      initialize(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
    }
    var IDrawingDistanceCalculator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides layout information about the edges of the graph during the layout.
     * <p>
     * {@link yfiles.hierarchic.IEdgeData} is a multi-purpose edge descriptor for each edge, holding information about the {@link yfiles.hierarchic.IEdgeData#type type} of an
     * edge, associated {@link yfiles.layout.PortConstraint port constraints}, {@link yfiles.layout.PortCandidate port candidates}, edge groups as well as the current edge state.
     * </p>
     * <p>
     * It is used in all phases of the {@link yfiles.hierarchic.HierarchicLayoutCore} to store edge information.
     * </p>
     * @interface
     */
    export interface IEdgeData extends Object{
      /**
       * Gets the type of the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData} instance.
       * @abstract
       * @type {yfiles.hierarchic.EdgeDataType}
       */
      type:yfiles.hierarchic.EdgeDataType;
      /**
       * Gets the {@link yfiles.algorithms.Node} associated with this {@link yfiles.hierarchic.IEdgeData} instance.
       * <p>
       * If no {@link yfiles.algorithms.Node} is associated with this {@link yfiles.hierarchic.IEdgeData} instance, <code>null</code> should be returned.
       * </p>
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      associatedNode:yfiles.algorithms.Node;
      /**
       * Gets the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData} instance.
       * <p>
       * If no {@link yfiles.algorithms.Edge} is associated with this {@link yfiles.hierarchic.IEdgeData} instance, <code>null</code> should be returned.
       * </p>
       * @abstract
       * @type {yfiles.algorithms.Edge}
       */
      associatedEdge:yfiles.algorithms.Edge;
      /**
       * Gets the {@link yfiles.layout.PortConstraint port constraints} for the source port of the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance.
       * <p>
       * {@link yfiles.layout.PortConstraint}s are obtained by a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}.
       * </p>
       * @abstract
       * @type {yfiles.layout.PortConstraint}
       */
      sourcePortConstraint:yfiles.layout.PortConstraint;
      /**
       * Gets the ID of the edge port group at the source node to which the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance belongs (if any).
       * <p>
       * Information about edge port grouping at the source node is obtained by a {@link yfiles.algorithms.IDataProvider} registered with
       * the graph with key {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY}.
       * </p>
       * @abstract
       * @type {Object}
       */
      sourcePortGroup:Object;
      /**
       * Gets the {@link yfiles.collections.ICollection.<T> collection} of {@link yfiles.layout.PortCandidate}s for the source port of the {@link yfiles.algorithms.Edge} associated with
       * this {@link yfiles.hierarchic.IEdgeData} instance.
       * <p>
       * {@link yfiles.layout.PortCandidate Source port candidates} are obtained by a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY}.
       * </p>
       * @abstract
       * @type {yfiles.collections.ICollection.<Object>}
       */
      sourcePortCandidates:yfiles.collections.ICollection<Object>;
      /**
       * Gets the {@link yfiles.layout.PortConstraint port constraints} for the target port of the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance.
       * <p>
       * {@link yfiles.layout.PortConstraint}s are obtained by a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY}.
       * </p>
       * @abstract
       * @type {yfiles.layout.PortConstraint}
       */
      targetPortConstraint:yfiles.layout.PortConstraint;
      /**
       * Gets the ID of the edge port group at the target node to which the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance belongs (if any).
       * <p>
       * Information about edge port grouping at the target node is obtained by a {@link yfiles.algorithms.IDataProvider} registered with
       * the graph with key {@link yfiles.layout.PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY}.
       * </p>
       * @abstract
       * @type {Object}
       */
      targetPortGroup:Object;
      /**
       * Gets the {@link yfiles.collections.ICollection.<T> collection} of {@link yfiles.layout.PortCandidate}s for the target port of the {@link yfiles.algorithms.Edge} associated with
       * this {@link yfiles.hierarchic.IEdgeData} instance.
       * <p>
       * {@link yfiles.layout.PortCandidate Target port candidates} are obtained by a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY}.
       * </p>
       * @abstract
       * @type {yfiles.collections.ICollection.<Object>}
       */
      targetPortCandidates:yfiles.collections.ICollection<Object>;
      /**
       * Gets the ID of the edge group at the source node to which the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance belongs (if any).
       * <p>
       * Information about edge grouping at the source node is obtained by a {@link yfiles.algorithms.IDataProvider} registered with the
       * graph with key {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}.
       * </p>
       * @abstract
       * @type {Object}
       */
      sourceGroup:Object;
      /**
       * Gets the ID of the edge group at the target node to which the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance belongs (if any).
       * <p>
       * Information about edge grouping at the target node is obtained by a {@link yfiles.algorithms.IDataProvider} registered with the
       * graph with key {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}.
       * </p>
       * @abstract
       * @type {Object}
       */
      targetGroup:Object;
      /**
       * Gets the group that is represented by the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance.
       * @abstract
       * @type {Object}
       */
      group:Object;
      /**
       * Gets whether or not the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData} instance has been
       * reversed.
       * @abstract
       * @type {boolean}
       */
      reversed:boolean;
      /**
       * Gets whether or not the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData} instance is an upper
       * same-layer edge.
       * <p>
       * An upper same-layer edge is an edge connecting two nodes of the same layer with polyline segments (not directly, with a
       * straight-line) and is routed above the layer.
       * </p>
       * @abstract
       * @type {boolean}
       */
      upperSameLayerEdge:boolean;
      /**
       * Gets the {@link yfiles.hierarchic.EdgeLayoutDescriptor} instance bound to the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData}
       * instance.
       * <p>
       * If no {@link yfiles.hierarchic.EdgeLayoutDescriptor} is bound to the edge, <code>null</code> should be returned.
       * </p>
       * @abstract
       * @type {yfiles.hierarchic.EdgeLayoutDescriptor}
       */
      edgeLayoutDescriptor:yfiles.hierarchic.EdgeLayoutDescriptor;
      /**
       * Gets or sets whether or not the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData} instance should
       * be an upper same-layer edge (if it is a same-layer edge).
       * <p>
       * An upper same-layer edge is an edge connecting two nodes of the same layer with polyline segments (not directly, with a
       * straight-line) and is routed above the layer.
       * </p>
       * <p>
       * If enabled the edge will be routed above the layer, otherwise below it.
       * </p>
       * @abstract
       * @type {boolean}
       */
      fallbackUpperSameLayerEdge:boolean;
      /**
       * Gets the thickness of the {@link yfiles.algorithms.Edge} associated with this {@link yfiles.hierarchic.IEdgeData} instance.
       * <p>
       * The thickness is considered when calculating minimum distances so that there are no overlaps between edges and other
       * graph elements.
       * </p>
       * <p>
       * The thickness must have a non-negative value.
       * </p>
       * @abstract
       * @type {number}
       */
      thickness:number;
    }
    var IEdgeData:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface handles edges that should be reversed during the hierarchic layout process.
     * <p>
     * {@link yfiles.hierarchic.IEdgeReverser} contains methods for reversing all (or a given edge) that point to the wrong direction,
     * i.e. the layer indices of the source nodes are greater than the layer indices of the target nodes. It also provides
     * methods to restore the edges that have been reversed at the end of the layout algorithm.
     * </p>
     * <p>
     * Implementations of this interface perform operations after the layering phase of the hierarchic layout algorithm.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore
     * @interface
     */
    export interface IEdgeReverser extends Object{
      /**
       * Reverses all edges which point to the wrong direction.
       * <p>
       * It is called after the layering phase of the {@link yfiles.hierarchic.HierarchicLayoutCore hierarchic layout algorithm}.
       * </p>
       * <p>
       * An edge is reversed if the layer index of its source node is greater than the layer index of its target node.
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } containing layering information about the nodes and edges of the graph
       * @see yfiles.hierarchic.ILayoutDataProvider#getNodeData
       * @see yfiles.hierarchic.INodeData#layer
       * @abstract
       */
      normalizeEdges(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Restores the original direction of edges marked as reversed.
       * <p>
       * It is called at the end of the {@link yfiles.hierarchic.HierarchicLayoutCore hierarchic layout algorithm}.
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } containing information about the nodes and edges of the graph
       * @abstract
       */
      restoreEdgeDirections(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Reverses a given edge which points to the wrong direction.
       * <p>
       * It may be called by the incremental parts of the {@link yfiles.hierarchic.HierarchicLayoutCore hierarchic layout algorithm}.
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } containing layering information about the nodes and edges of the graph
       * @param edge the edge to be reversed
       * @see yfiles.hierarchic.ILayoutDataProvider#getNodeData
       * @see yfiles.hierarchic.INodeData#layer
       * @abstract
       */
      reverse(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,edge:yfiles.algorithms.Edge):void;
    }
    var IEdgeReverser:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides factory methods for creating incremental hints for the hierarchic layout.
     * <p>
     * Incremental hints can be used for specifying that a subset of nodes (called <em>incremental</em> nodes) should be added to an existing graph
     * layout. A layer hint specifies that both, a node's layer and its position within the layer should be freely determined
     * by the algorithm. A sequence hint specifies that the algorithm should consider the current layer of a node while its
     * position within the layer should be freely determined by the algorithm. A node without any hint is called a <em>fixed</em> node.
     * </p>
     * <p>
     * Sequence hints can also be specified for edges. These <em>incremental</em> edges are inserted in a suitable position within the layers
     * crossed by the edges. For <em>fixed</em> edges the given input route is considered to calculate the position with the layers.
     * </p>
     * <p>
     * Information about incremental hints is provided by a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
     * {@link yfiles.hierarchic.HierarchicLayout#INCREMENTAL_HINTS_DP_KEY}.
     * </p>
     * <p>
     * An instance of a class implementing this interface can be obtained through method
     * {@link yfiles.hierarchic.HierarchicLayout#createIncrementalHintsFactory}.
     * </p>
     * @interface
     */
    export interface IIncrementalHintsFactory extends Object{
      /**
       * Creates a hint {@link Object} for a group {@link yfiles.algorithms.Node} that should be inserted incrementally during the layering phase.
       * <p>
       * The group will be placed on a suitable position. The descendants of the group may be associated with hints created by
       * methods {@link yfiles.hierarchic.IIncrementalHintsFactory#createLayerIncrementallyHint}, {@link yfiles.hierarchic.IIncrementalHintsFactory#createSequenceIncrementallyHint}
       * or this method (if the descendant is an inner group).
       * </p>
       * <p>
       * All hints of the descendants of a group are interpreted relative to the group node. Descendants without hints maintain
       * their relative order within the group node (but not with elements outside the group).
       * </p>
       * <p>
       * The positions of groups without incremental hints depend on the position of their descendants (i.e., the group is not
       * interpreted as fixed - it is simply ignored).
       * </p>
       * @param forItemId the group node to be inserted incrementally
       * @returns an {@link } that can be interpreted as a hint by {@link }
       * @abstract
       */
      createIncrementalGroupHint(forItemId:Object):Object;
      /**
       * Creates a hint {@link Object} for a {@link yfiles.algorithms.Node} that should be inserted incrementally during the layering phase.
       * <p>
       * This will place the node in a suitable layer, possibly creating new layers.
       * </p>
       * <p>
       * Neighbors of this node may be also marked such that they are laid out incrementally. This makes it possible to
       * incrementally add whole subgraphs to the current layout.
       * </p>
       * @param forNodeId the node to be layered and sequenced incrementally
       * @returns an {@link } that can be interpreted as a hint by {@link }
       * @abstract
       */
      createLayerIncrementallyHint(forNodeId:Object):Object;
      /**
       * Creates a hint {@link Object} for a {@link yfiles.algorithms.Node} or {@link yfiles.algorithms.Edge} that should be inserted incrementally during the
       * sequencing phase.
       * <p>
       * All nodes that are not incrementally inserted during the layering phase (see
       * {@link yfiles.hierarchic.IIncrementalHintsFactory#createLayerIncrementallyHint}) stay in their current layer. More precisely,
       * the default {@link yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer} implementation is an instance of {@link yfiles.hierarchic.AsIsLayerer} that determines the layer of the "fixed"
       * nodes by analyzing the current node coordinates. Note that for the other nodes, the layout algorithm may insert new
       * layers in between the existing layers.
       * </p>
       * <p>
       * For edges associated with this hint (or which are incident to an incremental node), the algorithm automatically
       * calculates a suitable sequencing (position within the layer). For edges between two non-incremental nodes that do not
       * have a hint, the sequencing phase considers the current route.
       * </p>
       * @param forItemId the node or edge to be sequenced/inserted incrementally
       * @returns an {@link } that can be interpreted as a hint by {@link }
       * @abstract
       */
      createSequenceIncrementallyHint(forItemId:Object):Object;
      /**
       * Creates a hint {@link Object} for a {@link yfiles.algorithms.Node} that should be inserted incrementally into the graph on its exact current
       * position.
       * <p>
       * As for nodes without hints, the algorithm uses the {@link yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer} instance to determine the layer of such nodes and the {@link yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer} instance to
       * determine the position within the layer. By default, both implementations use the current coordinates of the nodes to
       * derive the layer/position. However, for nodes marked with this hint, the algorithm should also preserve the exact
       * coordinates while for nodes without hints the coordinates may be changed (these nodes only keep their relative
       * positions).
       * </p>
       * @param forNodeId the node to be placed on its exact current position
       * @returns an {@link } that can be interpreted as a hint by {@link }
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createUseExactSequenceCoordinatesHint
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createUseExactLayerCoordinatesHint
       * @see yfiles.hierarchic.SimplexNodePlacer#exactPlacement
       * @abstract
       */
      createUseExactCoordinatesHint(forNodeId:Object):Object;
      /**
       * Creates a hint {@link Object} for a {@link yfiles.algorithms.Node} that should be inserted incrementally into the graph on its exact current
       * layer position.
       * <p>
       * The algorithm uses the {@link yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer} instance to determine the layer of such nodes and the {@link yfiles.hierarchic.HierarchicLayout#fromScratchSequencer} instance to determine the sequencing.
       * </p>
       * @param forNodeId the node to be placed on its exact current layer position
       * @returns an {@link } that can be interpreted as a hint by {@link }
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createUseExactCoordinatesHint
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createUseExactSequenceCoordinatesHint
       * @see yfiles.hierarchic.SimplexNodePlacer#exactPlacement
       * @abstract
       */
      createUseExactLayerCoordinatesHint(forNodeId:Object):Object;
      /**
       * Creates a hint {@link Object} for a {@link yfiles.algorithms.Node} that should be inserted incrementally into the graph on its exact current
       * sequence position.
       * <p>
       * The algorithm uses the {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayerer} instance to determine the layer of such nodes and the {@link yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer} instance to determine the sequencing.
       * </p>
       * @param forNodeId the node to be placed on its exact current sequence position
       * @returns an {@link } that can be interpreted as a hint by {@link }
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createUseExactCoordinatesHint
       * @see yfiles.hierarchic.IIncrementalHintsFactory#createUseExactLayerCoordinatesHint
       * @see yfiles.hierarchic.SimplexNodePlacer#exactPlacement
       * @abstract
       */
      createUseExactSequenceCoordinatesHint(forNodeId:Object):Object;
    }
    var IIncrementalHintsFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface is a factory for creating and destroying helper structures used in the graph during layout.
     * <p>
     * An implementation of this interface is provided by the {@link yfiles.hierarchic.HierarchicLayoutCore} during the layout for the
     * different sub-modules.
     * </p>
     * @interface
     */
    export interface IItemFactory extends Object{
      /**
       * Converts a given {@link yfiles.algorithms.Node} to a label node.
       * @param dummyNode the given {@link } to be converted
       * @abstract
       */
      convertToLabelNode(dummyNode:yfiles.algorithms.Node):void;
      /**
       * Creates a bend {@link yfiles.algorithms.Node} for the given {@link yfiles.algorithms.Edge} in the given {@link yfiles.hierarchic.ILayer} and
       * assigns it to the given {@link yfiles.hierarchic.SwimlaneDescriptor} instance.
       * @param layer the given {@link } instance
       * @param edge the given {@link }
       * @param [laneDescriptor=null] a given {@link } instance
       * @returns the new bend {@link }
       * @abstract
       */
      createBendNode(layer:yfiles.hierarchic.ILayer,edge:yfiles.algorithms.Edge,laneDescriptor?:yfiles.hierarchic.SwimlaneDescriptor):yfiles.algorithms.Node;
      /**
       * Creates a {@link yfiles.hierarchic.INodeData} object for a bend {@link yfiles.algorithms.Node} and associates it with the
       * {@link yfiles.algorithms.Node}.
       * <p>
       * The bend is assigned to the given {@link yfiles.hierarchic.SwimlaneDescriptor} instance.
       * </p>
       * @param node the given {@link }
       * @param edge the given {@link }
       * @param [laneDescriptor=null] the given {@link } instance
       * @returns a {@link } object for the given bend {@link }
       * @abstract
       */
      createBendNodeData(node:yfiles.algorithms.Node,edge:yfiles.algorithms.Edge,laneDescriptor?:yfiles.hierarchic.SwimlaneDescriptor):yfiles.hierarchic.INodeData;
      /**
       * Creates a dummy {@link yfiles.algorithms.Node} that mimics a connector to a group {@link yfiles.algorithms.Node}.
       * @param groupNode the given group {@link }
       * @param groupId the given group ID
       * @param layer the given {@link } instance
       * @param edge the given {@link }
       * @returns the new {@link }
       * @abstract
       */
      createConnectorProxyForGroup(groupNode:yfiles.algorithms.Node,groupId:Object,layer:yfiles.hierarchic.ILayer,edge:yfiles.algorithms.Edge):yfiles.algorithms.Node;
      /**
       * Creates a dummy node that mimics a connector to a group node from a descendant.
       * @param groupNode the group node to which the edge is connected
       * @param groupId the id of the group node
       * @param layer the layer to which the dummy node will be assigned
       * @param e the edge that connects to the dummy node
       * @returns the dummy node
       * @abstract
       */
      createContentConnectorProxyForGroup(groupNode:yfiles.algorithms.Node,groupId:Object,layer:yfiles.hierarchic.ILayer,e:yfiles.algorithms.Edge):yfiles.algorithms.Node;
      /**
       * Creates a spacer {@link yfiles.algorithms.Node} for the drawing phase using the given bounds in the given
       * {@link yfiles.hierarchic.ILayer}.
       * @param layer the given {@link } instance
       * @param size the given bounds
       * @param edges an array of {@link }s
       * @returns the new spacer {@link }
       * @abstract
       */
      createDistanceNode(layer:yfiles.hierarchic.ILayer,size:yfiles.algorithms.Rectangle2D,edges:yfiles.algorithms.Edge[]):yfiles.algorithms.Node;
      /**
       * Creates a dummy {@link yfiles.algorithms.Edge} using the given data.
       * @param newSource the source node of the {@link }
       * @param newTarget the target node of the {@link }
       * @param oldEdge the old {@link } from which to retrieve the data
       * @param sourceEnd <code>true</code> if the source node of the edge is referred, <code>false</code> otherwise
       * @param targetEnd <code>true</code> if the target node of the edge is referred, <code>false</code> otherwise
       * @returns a new dummy {@link }
       * @abstract
       */
      createDummyEdge(newSource:yfiles.algorithms.Node,newTarget:yfiles.algorithms.Node,oldEdge:yfiles.algorithms.Edge,sourceEnd:boolean,targetEnd:boolean):yfiles.algorithms.Edge;
      /**
       * Creates an edge group {@link yfiles.algorithms.Node} in the given {@link yfiles.hierarchic.ILayer}.
       * @param layer the given {@link } instance
       * @param groupId the ID of the edge group
       * @returns the new group {@link }
       * @abstract
       */
      createEdgeGroupNode(layer:yfiles.hierarchic.ILayer,groupId:Object):yfiles.algorithms.Node;
      /**
       * Creates a {@link yfiles.hierarchic.INodeData} object for an edge group {@link yfiles.algorithms.Node} and associates it with the
       * {@link yfiles.algorithms.Node}.
       * @param node the given {@link }
       * @param groupId the given group ID
       * @param source <code>true</code> if the edge group is referred to the source node, <code>false</code> otherwise
       * @returns a {@link } object for the given edge group {@link }
       * @abstract
       */
      createEdgeGroupNodeData(node:yfiles.algorithms.Node,groupId:Object,source:boolean):yfiles.hierarchic.INodeData;
      /**
       * Creates an {@link yfiles.algorithms.Edge} that connects two group border {@link yfiles.algorithms.Node}s.
       * @param source the source {@link }
       * @param target the target {@link }
       * @returns the new {@link } connecting the two group {@link }s
       * @abstract
       */
      createGroupBorderEdge(source:yfiles.algorithms.Node,target:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Creates a group boundary {@link yfiles.algorithms.Node} for a group node in the given {@link yfiles.hierarchic.ILayer}.
       * @param groupNode the given group {@link }
       * @param layer the given {@link }
       * @param type one of {@link #GROUP_BEGIN} or {@link #GROUP_END}
       * @returns the new group boundary {@link }
       * @abstract
       */
      createGroupBorderNode(groupNode:yfiles.algorithms.Node,layer:yfiles.hierarchic.ILayer,type:yfiles.hierarchic.NodeDataType):yfiles.algorithms.Node;
      /**
       * Create a dummy {@link yfiles.algorithms.Node} for a group {@link yfiles.hierarchic.ILayer layer}.
       * @returns a new dummy {@link }
       * @abstract
       */
      createGroupLayerDummyNode():yfiles.algorithms.Node;
      /**
       * Creates a group node connector {@link yfiles.algorithms.Edge} between two group {@link yfiles.algorithms.Node}s.
       * @param groupNode1 the first group node
       * @param groupNode2 the first group node
       * @param representative the given representative {@link }
       * @returns a new connector {@link } between two group {@link }s
       * @abstract
       */
      createGroupNodeConnectorEdge(groupNode1:yfiles.algorithms.Node,groupNode2:yfiles.algorithms.Node,representative:yfiles.algorithms.Edge):yfiles.algorithms.Edge;
      /**
       * Creates an {@link yfiles.hierarchic.IEdgeData} object for a normal {@link yfiles.algorithms.Edge} and associates it with the edge.
       * @param edge the given {@link }
       * @returns an {@link } object for the given {@link }
       * @abstract
       */
      createNormalEdgeData(edge:yfiles.algorithms.Edge):yfiles.hierarchic.IEdgeData;
      /**
       * Creates a {@link yfiles.hierarchic.INodeData} object for a normal {@link yfiles.algorithms.Node} and associates it with the
       * {@link yfiles.algorithms.Node}.
       * @param node the given {@link }
       * @returns a {@link } object for the given {@link }
       * @abstract
       */
      createNormalNodeData(node:yfiles.algorithms.Node):yfiles.hierarchic.INodeData;
      /**
       * Creates a proxy {@link yfiles.algorithms.Node} for an {@link yfiles.algorithms.Edge} during the drawing phase, changing the {@link yfiles.algorithms.Edge}
       * to end/start at the proxy and assigns it to a {@link yfiles.hierarchic.SwimlaneDescriptor} instance.
       * @param edge the given {@link }
       * @param source <code>true</code> if the node is the source node of the edge, <code>false</code> otherwise
       * @param [laneDescriptor=null] a given {@link } instance
       * @returns the new proxy {@link }
       * @abstract
       */
      createProxyNode(edge:yfiles.algorithms.Edge,source:boolean,laneDescriptor?:yfiles.hierarchic.SwimlaneDescriptor):yfiles.algorithms.Node;
      /**
       * Creates a {@link yfiles.hierarchic.INodeData} object for a proxy {@link yfiles.algorithms.Node} and associates it with the proxy node.
       * @param node the given {@link }
       * @param proxy the given proxy node
       * @param edge the given {@link }
       * @returns a {@link } object for the given proxy node
       * @abstract
       */
      createProxyNodeData(node:yfiles.algorithms.Node,proxy:yfiles.algorithms.Node,edge:yfiles.algorithms.Edge):yfiles.hierarchic.INodeData;
      /**
       * Creates a dummy edge that belongs to a recursive {@link yfiles.algorithms.Edge}.
       * @param newSource the source node of the {@link }
       * @param newTarget the target node of the {@link }
       * @param oldEdge the old {@link } from which to retrieve the data
       * @param sourceEnd <code>true</code> if the source node of the edge is referred, <code>false</code> otherwise
       * @param targetEnd <code>true</code> if the target node of the edge is referred, <code>false</code> otherwise
       * @returns a new dummy {@link }
       * @abstract
       */
      createRecursiveDummyEdge(newSource:yfiles.algorithms.Node,newTarget:yfiles.algorithms.Node,oldEdge:yfiles.algorithms.Edge,sourceEnd:boolean,targetEnd:boolean):yfiles.algorithms.Edge;
      /**
       * Creates a dummy {@link yfiles.algorithms.Node} that represents a bend node of a recursive {@link yfiles.algorithms.Edge}.
       * @param layer the given {@link } to which the dummy {@link } is assigned
       * @param edge the given {@link } to which the dummy {@link } belongs
       * @param laneDescriptor the given {@link } instance
       * @returns the new {@link }
       * @abstract
       */
      createRecursiveEdgeDummy(layer:yfiles.hierarchic.ILayer,edge:yfiles.algorithms.Edge,laneDescriptor:yfiles.hierarchic.SwimlaneDescriptor):yfiles.algorithms.Node;
      /**
       * Creates a {@link yfiles.hierarchic.INodeData} for a dummy {@link yfiles.algorithms.Node} of a recursive {@link yfiles.algorithms.Edge}.
       * @param node the given {@link }
       * @param edge the given {@link } to which the dummy {@link } belongs
       * @param laneDescriptor the given {@link } instance
       * @returns the new {@link } instance
       * @abstract
       */
      createRecursiveEdgeNodeData(node:yfiles.algorithms.Node,edge:yfiles.algorithms.Edge,laneDescriptor:yfiles.hierarchic.SwimlaneDescriptor):yfiles.hierarchic.INodeData;
      /**
       * Creates an {@link yfiles.hierarchic.IEdgeData} object for a same-layer recursive {@link yfiles.algorithms.Edge} and associates it with
       * the edge.
       * @param edge the given {@link }
       * @param originalEdgeData the {@link } object of the original edge
       * @param isUpper <code>true</code> is the recursive edge is an upper same layer edge, <code>false</code> otherwise
       * @returns an {@link } object for the recursive same-layer {@link }
       * @abstract
       */
      createRecursiveSameLayerEdgeData(edge:yfiles.algorithms.Edge,originalEdgeData:yfiles.hierarchic.IEdgeData,isUpper:boolean):yfiles.hierarchic.IEdgeData;
      /**
       * Creates a redirected {@link yfiles.algorithms.Edge} to replace an {@link yfiles.algorithms.Edge} connected to a group
       * {@link yfiles.algorithms.Node}.
       * @param newSource the source node of the {@link }
       * @param newTarget the target node of the {@link }
       * @param groupEdge the given group {@link }
       * @returns a new redirected {@link }
       * @abstract
       */
      createRedirectedGroupEdge(newSource:yfiles.algorithms.Node,newTarget:yfiles.algorithms.Node,groupEdge:yfiles.algorithms.Edge):yfiles.algorithms.Edge;
      /**
       * Creates a reversed dummy {@link yfiles.algorithms.Edge} using the given data.
       * @param newSource the source node of the {@link }
       * @param newTarget the target node of the {@link }
       * @param oldEdge the old {@link } from which to retrieve the data
       * @param sourceEnd <code>true</code> if the source node of the edge is referred, <code>false</code> otherwise
       * @param targetEnd <code>true</code> if the target node of the edge is referred, <code>false</code> otherwise
       * @returns a new reversed dummy {@link }
       * @abstract
       */
      createReverseDummyEdge(newSource:yfiles.algorithms.Node,newTarget:yfiles.algorithms.Node,oldEdge:yfiles.algorithms.Edge,sourceEnd:boolean,targetEnd:boolean):yfiles.algorithms.Edge;
      /**
       * Creates an {@link yfiles.hierarchic.IEdgeData} object for a same-layer {@link yfiles.algorithms.Edge} and associates it with the edge.
       * @param edge the given {@link }
       * @returns an {@link } object for the same-layer {@link }
       * @abstract
       */
      createSameLayerEdgeData(edge:yfiles.algorithms.Edge):yfiles.hierarchic.IEdgeData;
      /**
       * Creates a same-layer {@link yfiles.algorithms.Edge} proxy connecting a newly created proxy {@link yfiles.algorithms.Node} and a given
       * proxy {@link yfiles.algorithms.Node} in the given {@link yfiles.hierarchic.ILayer}.
       * @param layer the given {@link }
       * @param edge the given {@link }
       * @param toProxy the given proxy {@link }
       * @returns a new redefined same-layer {@link }
       * @abstract
       */
      createSameLayerProxy(layer:yfiles.hierarchic.ILayer,edge:yfiles.algorithms.Edge,toProxy:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Creates a proxy {@link yfiles.algorithms.Node} for a same-layer {@link yfiles.algorithms.Edge} during the drawing phase that ends at the
       * side of a node.
       * @param inLayer the given {@link } instance
       * @param forNode the given {@link }
       * @param edge the given {@link }
       * @param [sld=null] a given {@link } instance
       * @returns the new proxy {@link }
       * @abstract
       */
      createSameLayerSideProxy(inLayer:yfiles.hierarchic.ILayer,forNode:yfiles.algorithms.Node,edge:yfiles.algorithms.Edge,sld?:yfiles.hierarchic.SwimlaneDescriptor):yfiles.algorithms.Node;
      /**
       * Creates a same-layer switch proxy {@link yfiles.algorithms.Node} (switching between two same-layer
       * {@link yfiles.algorithms.Edge}s on two different sides of the layer).
       * @param layer the given {@link }
       * @param edge the given {@link }
       * @returns the new same-layer switch proxy {@link }
       * @abstract
       */
      createSameLayerSwitchProxy(layer:yfiles.hierarchic.ILayer,edge:yfiles.algorithms.Edge):yfiles.algorithms.Node;
      /**
       * Creates an {@link yfiles.hierarchic.IEdgeData} object for a self-loop {@link yfiles.algorithms.Edge} and associates it with the edge.
       * @param edge the given {@link }
       * @returns an {@link } object for a self-loop {@link }
       * @abstract
       */
      createSelfLoopEdgeData(edge:yfiles.algorithms.Edge):yfiles.hierarchic.IEdgeData;
      /**
       * Destroys a spacer {@link yfiles.algorithms.Node} for the drawing phase created using
       * {@link yfiles.hierarchic.IItemFactory#createDistanceNode}.
       * @param distanceNode the {@link } to be destroyed
       * @abstract
       */
      destroyDistanceNode(distanceNode:yfiles.algorithms.Node):void;
      /**
       * Removes the given layer for proxies at groups to handle direct group content connector edges between a node in the
       * first/last layer of the group node and the groups border.
       * @param layer the layer to destroy
       * @param useInEdges whether the incoming edges should be used as the resulting edges
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#directGroupContentEdgeRouting
       * @abstract
       */
      destroyGroupConnectorLayer(layer:yfiles.hierarchic.ILayer,useInEdges:boolean):void;
      /**
       * Destroys a previously created label layer
       * @param layer the layer to destroy
       * @param useInEdges whether the incoming edges should be used as the resulting edges
       * @abstract
       */
      destroyLabelNodeLayer(layer:yfiles.hierarchic.ILayer,useInEdges:boolean):void;
      /**
       * Destroys a proxy {@link yfiles.algorithms.Node} created using {@link yfiles.hierarchic.IItemFactory#createProxyNode} for the drawing
       * phase.
       * @param proxyNode the proxy {@link } to be destroyed
       * @returns 
       * @abstract
       */
      destroyProxyNode(proxyNode:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Destroys a same-layer {@link yfiles.algorithms.Edge} proxy created using {@link yfiles.hierarchic.IItemFactory#createSameLayerProxy}.
       * @param edge the {@link } whose proxy will be destroyed
       * @returns the redefined {@link } after the removal of the proxy {@link }
       * @abstract
       */
      destroySameLayerProxy(edge:yfiles.algorithms.Edge):yfiles.algorithms.Edge;
      /**
       * Destroys a proxy {@link yfiles.algorithms.Node} created using {@link yfiles.hierarchic.IItemFactory#createProxyNode} for the drawing
       * phase.
       * @param proxyNode the proxy {@link } to be destroyed
       * @abstract
       */
      destroySameLayerSideProxy(proxyNode:yfiles.algorithms.Node):void;
      /**
       * Destroys a same-layer switch proxy {@link yfiles.algorithms.Node} created using
       * {@link yfiles.hierarchic.IItemFactory#createSameLayerSwitchProxy}.
       * @param node the {@link } to be destroyed
       * @returns the redefined {@link } after the removal of the same-layer switch proxy {@link }
       * @abstract
       */
      destroySameLayerSwitchProxy(node:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Inserts a same-layer {@link yfiles.algorithms.Edge} or an {@link yfiles.algorithms.Edge} that may span multiple layers into the data
       * structure and returns the resulting list of
       * {@link yfiles.algorithms.Edge}s that has been created if this edge spans multiple layers.
       * @param edge the given {@link }
       * @returns the resulting list of {@link }s that has been created if this edge spans multiple layers
       * @abstract
       */
      insertEdge(edge:yfiles.algorithms.Edge):yfiles.algorithms.EdgeList;
      /**
       * Inserts a layer for proxies at groups to handle direct group content connector edges between a node in the first/last
       * layer of the group node and the groups border.
       * @param top <code>true</code> if this layer is placed above a group nodes, <code>false</code> if placed below
       * @param index the index where the layer is inserted
       * @returns the new layer
       * @see yfiles.hierarchic.EdgeLayoutDescriptor#directGroupContentEdgeRouting
       * @abstract
       */
      insertGroupConnectorLayer(top:boolean,index:number):yfiles.hierarchic.ILayer;
      /**
       * Inserts a {@link yfiles.hierarchic.ILayer layer} for group {@link yfiles.algorithms.Node}s.
       * @param source <code>true</code> if the group refers to the source node of an edge, <code>false</code> otherwise
       * @param index the index at which the specified {@link  layer} is to be inserted
       * @returns a {@link  layer} instance
       * @abstract
       */
      insertGroupNodeLayer(source:boolean,index:number):yfiles.hierarchic.ILayer;
      /**
       * Inserts a {@link yfiles.hierarchic.ILayer} for labels.
       * @param source <code>true</code> if the source node of the edge, <code>false</code> otherwise
       * @param index the index at which the specified {@link } is to be inserted
       * @returns a {@link  layer} instance
       * @abstract
       */
      insertLabelNodeLayer(source:boolean,index:number):yfiles.hierarchic.ILayer;
      /**
       * Registers an {@link yfiles.algorithms.Edge} as a same-layer {@link yfiles.algorithms.Edge} appropriately into all data structures.
       * @param edge the given {@link } to be registered
       * @abstract
       */
      registerSameLayerEdge(edge:yfiles.algorithms.Edge):void;
      /**
       * Converts a same-layer {@link yfiles.algorithms.Edge} to an {@link yfiles.algorithms.Edge} of type
       * {@link yfiles.hierarchic.EdgeDataType#DIRECT_SAME_LAYER_EDGE}.
       * @param sameLayerEdge the given same-layer {@link }
       * @abstract
       */
      setDirectlyConnectSameLayerEdge(sameLayerEdge:yfiles.algorithms.Edge):void;
      /**
       * Specifies an edge group constraint for an {@link yfiles.algorithms.Edge} and assigns it to an {@link yfiles.hierarchic.IEdgeData}
       * object.
       * @param edge the given {@link }
       * @param sgId the ID of the edge group at the source node
       * @param tgId the ID of the edge group at the target node
       * @returns an {@link } object
       * @abstract
       */
      setTemporaryEdgeGroups(edge:yfiles.algorithms.Edge,sgId:Object,tgId:Object):yfiles.hierarchic.IEdgeData;
      /**
       * Specifies the thickness for an {@link yfiles.algorithms.Edge} and defines it in an {@link yfiles.hierarchic.IEdgeData} object.
       * @param edge the given {@link }
       * @param thickness the new edge thickness
       * @returns an {@link } object with an adjusted {@link }
       * @abstract
       */
      setTemporaryEdgeThickness(edge:yfiles.algorithms.Edge,thickness:number):yfiles.hierarchic.IEdgeData;
      /**
       * Specifies a port constraint for an {@link yfiles.algorithms.Edge} at the given side and assigns it to an {@link yfiles.hierarchic.IEdgeData}
       * object.
       * @param edge the given {@link }
       * @param source <code>true</code> of the port constraint is defined on the source node of the {@link }, <code>false</code> otherwise
       * @param pc the given {@link } instance
       * @returns an {@link } object
       * @abstract
       */
      setTemporaryPortConstraint(edge:yfiles.algorithms.Edge,source:boolean,pc:yfiles.layout.PortConstraint):yfiles.hierarchic.IEdgeData;
      /**
       * Reverts a label node to the normal {@link yfiles.algorithms.Node}.
       * @param labelNode the given label node to be converted
       * @abstract
       */
      unconvertToLabelNode(labelNode:yfiles.algorithms.Node):void;
    }
    var IItemFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface describes a single layer in a hierarchic layout.
     * <p>
     * A layer consists of nodes along with the associated same-layer edges. Each layer has a {@link yfiles.hierarchic.ILayer#type type} and an index representing
     * its order in the {@link yfiles.hierarchic.ILayers list of layers}.
     * </p>
     * @see yfiles.hierarchic.ILayers
     * @see yfiles.hierarchic.HierarchicLayoutCore
     * @see yfiles.hierarchic.ILayerer
     * @interface
     */
    export interface ILayer extends Object{
      /**
       * Adds a newly created {@link yfiles.algorithms.Node} to this {@link yfiles.hierarchic.ILayer} instance.
       * @param node the {@link } to be added
       * @abstract
       */
      add(node:yfiles.algorithms.Node):void;
      /**
       * Adds a same-layer {@link yfiles.algorithms.Edge} to this {@link yfiles.hierarchic.ILayer} instance.
       * @param edge the same-layer {@link } to be added
       * @abstract
       */
      addSameLayerEdge(edge:yfiles.algorithms.Edge):void;
      /**
       * Removes a {@link yfiles.algorithms.Node} from this {@link yfiles.hierarchic.ILayer} instance.
       * @param node the {@link } to be removed
       * @abstract
       */
      remove(node:yfiles.algorithms.Node):void;
      /**
       * Removes the current layer from the {@link yfiles.hierarchic.ILayers list of layers}.
       * @abstract
       */
      remove():void;
      /**
       * Adjusts the order of the {@link yfiles.algorithms.Node}s of this {@link yfiles.hierarchic.ILayer} instance according to the given
       * order.
       * @param list the given list to be used
       * @abstract
       */
      setNodeOrder(list:yfiles.algorithms.YList):void;
      /**
       * Gets a list of all same-layer {@link yfiles.algorithms.Edge}s associated with this {@link yfiles.hierarchic.ILayer} instance.
       * @abstract
       * @type {yfiles.algorithms.YList}
       */
      sameLayerEdges:yfiles.algorithms.YList;
      /**
       * Gets a list of the {@link yfiles.algorithms.Node}s associated with this {@link yfiles.hierarchic.ILayer} instance.
       * @abstract
       * @type {yfiles.algorithms.NodeList}
       */
      list:yfiles.algorithms.NodeList;
      /**
       * Gets the type of this {@link yfiles.hierarchic.ILayer} instance.
       * @abstract
       * @type {yfiles.hierarchic.LayerType}
       */
      type:yfiles.hierarchic.LayerType;
      /**
       * Gets the index of this {@link yfiles.hierarchic.ILayer} instance in the {@link yfiles.hierarchic.ILayers list of all layers}.
       * @abstract
       * @type {number}
       */
      index:number;
      /**
       * Gets or sets the {@link yfiles.layout.RowDescriptor} associated with this {@link yfiles.hierarchic.ILayer} instance.
       * @abstract
       * @type {yfiles.layout.RowDescriptor}
       */
      row:yfiles.layout.RowDescriptor;
    }
    var ILayer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface merges two {@link yfiles.hierarchic.ILayers} instances whose nodes reside in the same graph.
     * <p>
     * Implementations of this interface may provide different strategies for merging.
     * </p>
     * <p>
     * {@link yfiles.hierarchic.DefaultLayeredComponentsMerger Default implementations} of this interface are used internally by {@link yfiles.hierarchic.HierarchicLayoutCore} and {@link yfiles.hierarchic.MultiComponentLayerer}
     * when e.g. the graph is grouped or it consists of more than one connected component.
     * </p>
     * @interface
     */
    export interface ILayeredComponentsMerger extends Object{
      /**
       * Merges two {@link yfiles.hierarchic.ILayers} instances.
       * <p>
       * All nodes that belong to the source and target {@link yfiles.hierarchic.ILayers} are part of the graph at the moment of
       * invocation.
       * </p>
       * <p>
       * The state of the source {@link yfiles.hierarchic.ILayers} instance is discarded after this call and does not need to be updated
       * to reflect the changes. The target {@link yfiles.hierarchic.ILayers} instance must be updated accordingly.
       * </p>
       * @param graph the graph containing the nodes of source and target {@link } instances
       * @param ldp the {@link } used for querying the layer indices
       * @param srcLayers the source {@link } instance that will be merged into the target {@link } instance
       * @param targetLayers the {@link } instance that will be modified to contain the resulting layering
       * @abstract
       */
      merge(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,srcLayers:yfiles.hierarchic.ILayers,targetLayers:yfiles.hierarchic.ILayers):void;
    }
    var ILayeredComponentsMerger:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface is used by classes capable of generating a layer assignment.
     * <p>
     * Layer assignment takes place during the first part of the hierarchic layout algorithms. During this phase, each node of
     * the graph is assigned to a {@link yfiles.hierarchic.ILayer layer}.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore#layerer
     * @see yfiles.hierarchic.HierarchicLayout#fixedElementsLayerer
     * @see yfiles.hierarchic.HierarchicLayout#fromScratchLayerer
     * @interface
     */
    export interface ILayerer extends Object{
      /**
       * Assigns all nodes of the graph to layers and adds them to the {@link yfiles.hierarchic.ILayers} instance.
       * <p>
       * In order to create new layers, factory method {@link yfiles.hierarchic.ILayers#insert} has to be used.
       * </p>
       * <p>
       * Information about the nodes is provided by a {@link yfiles.hierarchic.ILayoutDataProvider}. However, positional information (see
       * {@link yfiles.hierarchic.INodeData#position} and {@link yfiles.hierarchic.INodeData#layer}) is not available during this phase.
       * </p>
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for querying information about the nodes and edges
       * @see yfiles.hierarchic.ILayers#insert
       * @see yfiles.hierarchic.ILayer#add
       * @abstract
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
    }
    var ILayerer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface manages multiple {@link yfiles.hierarchic.ILayer} instances.
     * <p>
     * {@link yfiles.hierarchic.HierarchicLayoutCore} creates instances of this class and passes them to the instances that handle the
     * various sub-tasks of the layout process during automatic layout.
     * </p>
     * <p>
     * Method {@link yfiles.hierarchic.ILayers#createInstance} can be used as a factory method to obtain temporary {@link yfiles.hierarchic.ILayers}
     * instances.
     * </p>
     * @see yfiles.hierarchic.ILayer
     * @interface
     */
    export interface ILayers extends Object{
      /**
       * Creates a new and empty {@link yfiles.hierarchic.ILayers} instance that can be used on the same graph instance for temporary
       * results.
       * @returns a new and empty {@link } instance
       * @abstract
       */
      createInstance():yfiles.hierarchic.ILayers;
      /**
       * Returns the {@link yfiles.hierarchic.ILayer layer} at the specified position in this {@link yfiles.hierarchic.ILayers list of layers}.
       * @param i the zero-based index of the {@link  layer} to return
       * @returns the {@link  layer} at the specified position
       * @abstract
       */
      getLayer(i:number):yfiles.hierarchic.ILayer;
      /**
       * Creates, inserts and returns a {@link yfiles.hierarchic.ILayer layer} of a given type associated with this {@link yfiles.hierarchic.ILayers list of layers}.
       * @param type one of the predefined layer types as defined in {@link } interface
       * @param position the index at which the specified {@link  layer} is to be inserted
       * @returns a new {@link } instance
       * @abstract
       */
      insert(type:yfiles.hierarchic.LayerType,position:number):yfiles.hierarchic.ILayer;
      /**
       * Removes the {@link yfiles.hierarchic.ILayer layer} at the specified position from this {@link yfiles.hierarchic.ILayers list of layers}.
       * @param index the zero-based index of the {@link  layer} to remove
       * @abstract
       */
      remove(index:number):void;
      /**
       * Returns the number of {@link yfiles.hierarchic.ILayer layers} associated with this {@link yfiles.hierarchic.ILayers} instance.
       * @returns the number of layers of this {@link } instance
       * @abstract
       */
      size():number;
    }
    var ILayers:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface retrieves {@link yfiles.hierarchic.INodeData} and {@link yfiles.hierarchic.IEdgeData} instances for elements in the
     * current layout graph.
     * <p>
     * Instances of this interface are provided by {@link yfiles.hierarchic.HierarchicLayoutCore} for instances implementing sub-tasks
     * during the layout.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore
     * @interface
     */
    export interface ILayoutDataProvider extends Object{
      /**
       * Returns the {@link yfiles.hierarchic.IEdgeData} instance associated with the given {@link yfiles.algorithms.Edge}.
       * @param edge the given edge
       * @returns the {@link } instance associated with the edge
       * @abstract
       */
      getEdgeData(edge:yfiles.algorithms.Edge):yfiles.hierarchic.IEdgeData;
      /**
       * Returns the {@link yfiles.hierarchic.INodeData} instance associated with the given {@link yfiles.algorithms.Node}.
       * @param node the given node
       * @returns the {@link } instance associated with the node
       * @abstract
       */
      getNodeData(node:yfiles.algorithms.Node):yfiles.hierarchic.INodeData;
    }
    var ILayoutDataProvider:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides layout information about the nodes of the graph during the layout.
     * <p>
     * {@link yfiles.hierarchic.INodeData} is a multi-purpose node descriptor for each node of the graph during the hierarchic layout
     * used internally by the algorithm implementations.
     * </p>
     * <p>
     * Instances of this interface can be obtained during the layout from instances of the {@link yfiles.hierarchic.ILayoutDataProvider}
     * interface.
     * </p>
     * <p>
     * {@link yfiles.hierarchic.INodeData} typically carries the {@link yfiles.hierarchic.INodeData#type type} of the node and depending on the type an optional {@link yfiles.hierarchic.INodeData#associatedEdge associated edge}
     * and
     * {@link yfiles.hierarchic.INodeData#associatedNode associated node}. Optionally, it may carry a geometric description of the borders of the node and descriptors for various
     * aspects of the layout.
     * </p>
     * @interface
     */
    export interface INodeData extends Object{
      /**
       * Creates, stores and returns a {@link yfiles.algorithms.BorderLine} for the given side or returns the current {@link yfiles.algorithms.BorderLine}
       * instance if it already exists.
       * <p>
       * Not every {@link yfiles.hierarchic.INodeData#type type} of node supports this operation. {@link yfiles.hierarchic.NodeDataType#NORMAL} supports
       * {@link yfiles.algorithms.BorderLine}s.
       * </p>
       * <p>
       * Side value must be inside <code>[0,3]</code> interval, where <code>0</code> corresponds to the top side, <code>1</code> to the right side, <code>2</code> to the bottom side
       * and <code>3</code> to the left side.
       * </p>
       * @param side an integer value from <code>[0,3]</code> interval
       * @param nodeLayout the {@link } of the current node
       * @returns a {@link } instance
       * @throws {Stubs.Exceptions.NotSupportedError} if this {@link yfiles.hierarchic.INodeData#type type} of node does not support node borders
       * @see yfiles.hierarchic.INodeData#getNormalizedBorderLine
       * @abstract
       */
      createBorderLine(side:number,nodeLayout:yfiles.layout.INodeLayout):yfiles.algorithms.BorderLine;
      /**
       * Returns a {@link yfiles.algorithms.BorderLine} for the given side which is treated as if the position of the node is at (0,0).
       * <p>
       * Clients may modify this instance but should always reset it to the original position, since this is a shared instance.
       * </p>
       * <p>
       * It may return <code>null</code> to indicate that the {@link yfiles.algorithms.BorderLine} can be determined through the {@link yfiles.layout.INodeLayout}
       * instance.
       * </p>
       * <p>
       * Side value must be inside <code>[0,3]</code> interval, where <code>0</code> corresponds to the top side, <code>1</code> to the right side, <code>2</code> to the bottom side
       * and <code>3</code> to the left side.
       * </p>
       * @param side an integer value from <code>[0,3]</code> interval
       * @returns a {@link } instance or <code>null</code>
       * @see yfiles.hierarchic.INodeData#createBorderLine
       * @abstract
       */
      getNormalizedBorderLine(side:number):yfiles.algorithms.BorderLine;
      /**
       * Returns the number of same-layer edges associated with the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData}
       * instance.
       * @returns the number of same-layer edges associated with the node
       * @abstract
       */
      sameLayerEdgeCount():number;
      /**
       * Gets the group node this node belongs to.
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      groupNode:yfiles.algorithms.Node;
      /**
       * Gets the first same-layer edge {@link yfiles.algorithms.ListCell} of all same-layer edges or <code>null</code> if there exists no same-layer
       * edge.
       * @abstract
       * @type {yfiles.algorithms.ListCell}
       */
      firstSameLayerEdgeCell:yfiles.algorithms.ListCell;
      /**
       * Gets the type constant for the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance.
       * @abstract
       * @type {yfiles.hierarchic.NodeDataType}
       */
      type:yfiles.hierarchic.NodeDataType;
      /**
       * Gets the group ID of the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance if it is a bus-group node.
       * @see yfiles.hierarchic.NodeDataType#SOURCE_GROUP_NODE
       * @see yfiles.hierarchic.NodeDataType#TARGET_GROUP_NODE
       * @abstract
       * @type {Object}
       */
      groupId:Object;
      /**
       * Gets the associated {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance (in case of e.g.
       * <p>
       * back-loop proxy and side proxy).
       * </p>
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      associatedNode:yfiles.algorithms.Node;
      /**
       * Gets the associated {@link yfiles.algorithms.Edge} of the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance
       * (in case of e.g.
       * <p>
       * bend or same-layer center node).
       * </p>
       * @abstract
       * @type {yfiles.algorithms.Edge}
       */
      associatedEdge:yfiles.algorithms.Edge;
      /**
       * Gets the index of the {@link yfiles.hierarchic.ILayer} instance to which the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData}
       * instance belongs.
       * @abstract
       * @type {number}
       */
      layer:number;
      /**
       * Gets the current position of the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance within its layer.
       * @abstract
       * @type {number}
       */
      position:number;
      /**
       * Gets the parent group node of the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance (if any).
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      parentGroupNode:yfiles.algorithms.Node;
      /**
       * Gets the inner layers of the group {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance.
       * <p>
       * If this node is of type {@link yfiles.hierarchic.NodeDataType#GROUP}, this method returns the {@link yfiles.hierarchic.ILayers} object
       * that describes the layering in the subgraph.
       * </p>
       * @abstract
       * @type {yfiles.hierarchic.ILayers}
       */
      groupLayers:yfiles.hierarchic.ILayers;
      /**
       * Gets the hint of the incremental mode of the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData} instance if this
       * node is of type {@link yfiles.hierarchic.NodeDataType#NORMAL}.
       * <p>
       * The corresponding hint has been set by a {@link yfiles.algorithms.IDataProvider} using key
       * {@link yfiles.hierarchic.HierarchicLayoutCore#INCREMENTAL_HINTS_DP_KEY}.
       * </p>
       * @abstract
       * @type {Object}
       */
      incrementalHint:Object;
      /**
       * Gets the {@link yfiles.hierarchic.NodeLayoutDescriptor} instance initially bound to the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData}
       * instance or <code>null</code> if no {@link yfiles.hierarchic.NodeLayoutDescriptor} has been registered.
       * @abstract
       * @type {yfiles.hierarchic.NodeLayoutDescriptor}
       */
      nodeLayoutDescriptor:yfiles.hierarchic.NodeLayoutDescriptor;
      /**
       * Gets the {@link yfiles.hierarchic.SwimlaneDescriptor} instance initially bound to the {@link yfiles.algorithms.Node} of this {@link yfiles.hierarchic.INodeData}
       * instance or <code>null</code> if no {@link yfiles.hierarchic.SwimlaneDescriptor} has been registered.
       * @abstract
       * @type {yfiles.hierarchic.SwimlaneDescriptor}
       */
      swimLaneDescriptor:yfiles.hierarchic.SwimlaneDescriptor;
    }
    var INodeData:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface is responsible for the assignment of the x-(sequence) and preliminary y-coordinates of the nodes in a
     * hierarchic layout.
     * <p>
     * Implementations of this interface are used during the drawing phase of
     * {@link yfiles.hierarchic.HierarchicLayoutCore}. They determine preliminary y-coordinates (which may be adjusted during the final
     * edge routing) and the resulting x-coordinates of all elements that are part of the graph during the node placement
     * phase.
     * </p>
     * <p>
     * The minimum distance between each pair of nodes of the graph is determined by a {@link yfiles.hierarchic.IDrawingDistanceCalculator}
     * instance.
     * </p>
     * <p>
     * The default implementation used by the {@link yfiles.hierarchic.HierarchicLayoutCore} is {@link yfiles.hierarchic.SimplexNodePlacer}.
     * Custom implementations of {@link yfiles.hierarchic.INodePlacer} interface can be used for hierarchic layout algorithms using
     * methods {@link yfiles.hierarchic.HierarchicLayoutCore#nodePlacer} or {@link yfiles.hierarchic.HierarchicLayout#nodePlacer}.
     * </p>
     * @see yfiles.hierarchic.SimplexNodePlacer
     * @see yfiles.hierarchic.IDrawingDistanceCalculator
     * @see yfiles.hierarchic.HierarchicLayoutCore
     * @see yfiles.hierarchic.HierarchicLayoutCore#nodePlacer
     * @see yfiles.hierarchic.HierarchicLayout#nodePlacer
     * @interface
     */
    export interface INodePlacer extends Object{
      /**
       * Assigns preliminary y-coordinates for each layer of a hierarchic layout.
       * <p>
       * The distance between two layers will be adjusted later by the edge routing algorithm. This method is responsible for
       * assigning the relative positions of the nodes within each layer.
       * </p>
       * @param graph the input graph
       * @param layoutDataProvider the {@link } containing information about the elements
       * @param layers the {@link } instance that will be calculated by this method
       * @abstract
       */
      assignLayerCoordinates(graph:yfiles.layout.LayoutGraph,layoutDataProvider:yfiles.hierarchic.ILayoutDataProvider,layers:yfiles.hierarchic.ILayers):void;
      /**
       * Determines the resulting x-coordinates of a hierarchic layout.
       * <p>
       * The minimum distance between each pair of nodes of the graph is determined by a {@link yfiles.hierarchic.IDrawingDistanceCalculator}
       * instance.
       * </p>
       * @param graph the input graph
       * @param layoutDataProvider the {@link } containing information about the elements
       * @param layers the {@link } instance that will be calculated by this method
       * @param drawingDistanceCalculator the given {@link } instance
       * @abstract
       */
      assignSequenceCoordinates(graph:yfiles.layout.LayoutGraph,layoutDataProvider:yfiles.hierarchic.ILayoutDataProvider,layers:yfiles.hierarchic.ILayers,drawingDistanceCalculator:yfiles.hierarchic.IDrawingDistanceCalculator):void;
    }
    var INodePlacer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface is responsible for assigning port coordinates to the adjacent edges of each node of the graph.
     * <p>
     * {@link yfiles.hierarchic.IPortAllocator} is used by {@link yfiles.hierarchic.HierarchicLayoutCore} after the sequencing phase.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore
     * @interface
     */
    export interface IPortAllocator extends Object{
      /**
       * Assigns to each edge of the graph a source port and target port coordinate pair.
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayoutCore} after the sequencing phase.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance containing the elements in the layering
       * @param ldp the {@link } containing layering information about the nodes and edges of the graph
       * @param itemFactory the {@link } used temporarily for modifying the graph instance
       * @abstract
       */
      assignPorts(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
    }
    var IPortAllocator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface serves as a callback for {@link yfiles.hierarchic.HierarchicLayoutCore} after the layering and sequencing
     * phases.
     * <p>
     * Implementations of this interface may modify the {@link yfiles.layout.PortConstraint port constraint} information via
     * {@link yfiles.hierarchic.IItemFactory#setTemporaryPortConstraint}.
     * </p>
     * @interface
     */
    export interface IPortConstraintOptimizer extends Object{
      /**
       * Assigns new temporary port constraints after the layering information has been determined.
       * @param graph the input graph
       * @param layers the given {@link } instance
       * @param ldp the {@link } containing information about the elements
       * @param itemFactory the {@link } to set the temporary port constraints with
       * @see yfiles.hierarchic.IItemFactory#setTemporaryPortConstraint
       * @abstract
       */
      optimizeAfterLayering(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Assigns new temporary port constraints after the sequence of the nodes has been determined.
       * @param graph the input graph
       * @param layers the given {@link } instance
       * @param ldp the {@link } containing information about the elements
       * @param itemFactory the {@link } to set the temporary port constraints with
       * @see yfiles.hierarchic.IItemFactory#setTemporaryPortConstraint
       * @abstract
       */
      optimizeAfterSequencing(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
    }
    var IPortConstraintOptimizer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface is used by {@link yfiles.hierarchic.HierarchicLayoutCore} to calculate the order of the nodes within each {@link yfiles.hierarchic.ILayers layer}.
     * <p>
     * Implementations are used during the second phase of the hierarchic layout process and can be set using method {@link yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer} or {@link yfiles.hierarchic.HierarchicLayout#fromScratchSequencer}).
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore#sequencer
     * @see yfiles.hierarchic.HierarchicLayout#fixedElementsSequencer
     * @see yfiles.hierarchic.HierarchicLayout#fromScratchSequencer
     * @interface
     */
    export interface ISequencer extends Object{
      /**
       * Calculates the sequence of the nodes within each {@link yfiles.hierarchic.ILayers layer}.
       * <p>
       * This method is called by {@link yfiles.hierarchic.HierarchicLayoutCore} during the second phase and finally writes back the
       * calculated sequence using the {@link yfiles.hierarchic.ILayer#setNodeOrder} method.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance containing the elements in the layering
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link } used temporarily for modifying the graph instance
       * @abstract
       */
      sequenceNodeLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
    }
    var ISequencer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class is a wrapper {@link yfiles.hierarchic.ILayerer} implementation that delegates the actual layering to a delegate
     * {@link yfiles.hierarchic.ILayerer}.
     * <p>
     * If the graph consists of multiple components, they will be identified and sorted and each of the components will be
     * layered separately using the delegate {@link yfiles.hierarchic.ILayerer} instance. After that they will all be merged using the
     * specified {@link yfiles.hierarchic.ILayeredComponentsMerger} instance.
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface MultiComponentLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class MultiComponentLayerer {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.MultiComponentLayerer} using the given {@link yfiles.hierarchic.ILayeredComponentsMerger}
       * and {@link yfiles.collections.IComparer.<T>} instances.
       * @param singleComponentLayerer
       * @param merger
       * @param componentComparer
       * @throws {Stubs.Exceptions.ArgumentError} if the given layerer or merger is <code>null</code>
       */
      constructor(singleComponentLayerer:yfiles.hierarchic.ILayerer,merger:yfiles.hierarchic.ILayeredComponentsMerger,componentComparer:yfiles.collections.IComparer<Object>);
      /**
       * Creates a new instance of {@link yfiles.hierarchic.MultiComponentLayerer} using the given delegate {@link yfiles.hierarchic.ILayerer}.
       * <p>
       * A new instance of {@link yfiles.hierarchic.DefaultLayeredComponentsMerger} will be used as merger while no component {@link yfiles.collections.IComparer.<T>} will be
       * used.
       * </p>
       * @param singleComponentLayerer
       * @throws {Stubs.Exceptions.ArgumentError} if the given layerer is <code>null</code>
       */
      constructor(singleComponentLayerer:yfiles.hierarchic.ILayerer);
      /**
       * 
       * @param graph
       * @param layers
       * @param ldp
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Sorts an array which consists of
       * {@link yfiles.algorithms.NodeList}s each of which contains nodes that belong to the same component.
       * <p>
       * It is called during {@link yfiles.hierarchic.MultiComponentLayerer#assignLayers}.
       * </p>
       * <p>
       * By default this method uses the {@link yfiles.hierarchic.MultiComponentLayerer#componentComparer ComponentComparator} to sort the list and has no effect if the instance is <code>null</code>.
       * </p>
       * @param nodeLists the array of {@link }s each of which containing nodes that belong to the same component
       * @param componentIndexProvider the {@link } that returns an object (the component index) for each node (this value can be either
       * integer or object)
       * @protected
       */
      sort(nodeLists:yfiles.algorithms.NodeList[],componentIndexProvider:yfiles.algorithms.IDataProvider):void;
      /**
       * Gets or sets the delegate {@link yfiles.hierarchic.ILayerer} instance.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.ILayerer} is <code>null</code>
       * @type {yfiles.hierarchic.ILayerer}
       */
      singleComponentLayerer:yfiles.hierarchic.ILayerer;
      /**
       * Gets or sets the {@link yfiles.hierarchic.ILayeredComponentsMerger merger} instance.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.hierarchic.ILayeredComponentsMerger} is <code>null</code>
       * @type {yfiles.hierarchic.ILayeredComponentsMerger}
       */
      merger:yfiles.hierarchic.ILayeredComponentsMerger;
      /**
       * Gets or sets the component {@link yfiles.collections.IComparer.<T>} instance.
       * @type {yfiles.collections.IComparer.<Object>}
       */
      componentComparer:yfiles.collections.IComparer<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used by {@link yfiles.hierarchic.HierarchicLayoutCore} during the various phases to provide the drawing details of
     * the nodes of the graph.
     * <p>
     * A {@link yfiles.hierarchic.NodeLayoutDescriptor} instance can be specified individually for single nodes using a {@link yfiles.algorithms.IDataProvider}
     * that returns a {@link yfiles.hierarchic.NodeLayoutDescriptor} instance for each node of the graph, or <code>null</code> if no {@link yfiles.hierarchic.NodeLayoutDescriptor}
     * is bound to an edge. The {@link yfiles.algorithms.IDataProvider} is registered with the graph with key
     * {@link yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY}.
     * </p>
     * <p>
     * This class is designed as a class to allow future additions of new getter methods.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
     * @see yfiles.hierarchic.EdgeLayoutDescriptor
     * @class
     */
    export interface NodeLayoutDescriptor extends Object{}
    export class NodeLayoutDescriptor {
      /**
       * Creates a new instance of a {@link yfiles.hierarchic.NodeLayoutDescriptor} with the default values.
       */
      constructor();
      /**
       * Returns the port border gap ratio for the port distribution at a given side of the node.
       * <p>
       * This ratio determines the gap between a corner of the node and the first assigned port.
       * </p>
       * <p>
       * The ratio should be greater than <code>0</code>.
       * </p>
       * <p>
       * A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
       * </p>
       * <p>
       * A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
       * the node and the first port is half as wide as the distance between two adjacent ports.
       * </p>
       * <p>
       * A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
       * </p>
       * @param side the zero-based clockwise side index for top-to-bottom layouts (i.e., top is 0)
       * @returns the border gap ratio of a given side
       * @see yfiles.hierarchic.NodeLayoutDescriptor#portBorderGapRatios
       * @see yfiles.hierarchic.NodeLayoutDescriptor#setPortBorderGapRatio
       */
      getPortBorderGapRatio(side:number):number;
      /**
       * Specifies the port border gap ratio for the port distribution at a given side of the node.
       * <p>
       * This ratio determines the gap between a corner of the node and the first assigned port.
       * </p>
       * <p>
       * The ratio should be greater than <code>0</code>.
       * </p>
       * <p>
       * A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
       * </p>
       * <p>
       * A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
       * the node and the first port is half as wide as the distance between two adjacent ports.
       * </p>
       * <p>
       * A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
       * </p>
       * @param side the zero-based clockwise side index for top-to-bottom layouts (i.e., top is 0)
       * @param ratio the given ratio
       * @throws {Stubs.Exceptions.ArgumentError} if the given ratio is negative
       * @see yfiles.hierarchic.NodeLayoutDescriptor#portBorderGapRatios
       */
      setPortBorderGapRatio(side:number,ratio:number):void;
      /**
       * Sets the port border gap ratio for the port distribution on all sides of the node.
       * <p>
       * This ratio determines the gap between a corner of the node and the first assigned port.
       * </p>
       * <p>
       * The ratio should be greater than <code>0</code>.
       * </p>
       * <p>
       * A value of <code>0.0d</code> results in ports being placed directly on the corner of the node (if there is more than one port).
       * </p>
       * <p>
       * A value of <code>0.5d</code> results in ports being distributed along the side of the node so that the distance between the corner of
       * the node and the first port is half as wide as the distance between two adjacent ports.
       * </p>
       * <p>
       * A value of <code>Double.POSITIVE_INFINITY</code> results in all ports being centered at the side in one point.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given ratio is negative
       * @see yfiles.hierarchic.NodeLayoutDescriptor#setPortBorderGapRatio
       * @type {number}
       */
      portBorderGapRatios:number;
      /**
       * Gets or sets the alignment of the {@link yfiles.algorithms.Node node} of this {@link yfiles.hierarchic.NodeLayoutDescriptor} instance within its layer.
       * <ul>
       * <li>A value of <code>0.0d</code> corresponds to top-alignment.</li>
       * <li>A value of <code>0.5d</code> corresponds to center-alignment.</li>
       * <li>A value of <code>1.0d</code> corresponds to bottom-alignment.</li>
       * </ul>
       * <p>
       * This value should be within <code>[0,1]</code> interval.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the alignment is not within [0.0d .. 1.0d]
       * @type {number}
       */
      layerAlignment:number;
      /**
       * Gets or sets the mode that determines how node labels are handled by the algorithm during the layout.
       * @throws {Stubs.Exceptions.ArgumentError} if the constant is unknown.
       * @see yfiles.hierarchic.HierarchicLayout#considerNodeLabels
       * @type {yfiles.hierarchic.NodeLabelMode}
       */
      nodeLabelMode:yfiles.hierarchic.NodeLabelMode;
      /**
       * Gets or sets the minimum height of the layer to which this node is assigned.
       * <p>
       * Values should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum height is negative
       * @type {number}
       */
      minimumLayerHeight:number;
      /**
       * Gets or sets the preferred minimum distance to obstacles.
       * <p>
       * The minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length is negative
       * @type {number}
       */
      minimumDistance:number;
      /**
       * Gets or sets a reference point relative to the center of the node which will be placed on a grid coordinate.
       * @type {yfiles.algorithms.YPoint}
       */
      gridReference:yfiles.algorithms.YPoint;
      /**
       * Gets or sets the assignment strategy which distributes the ports along the node borders.
       * @type {yfiles.hierarchic.PortAssignmentMode}
       */
      portAssignment:yfiles.hierarchic.PortAssignmentMode;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class implements a {@link yfiles.layout.ILayoutStage layout stage} that handles a given {@link yfiles.layout.PartitionGrid partition grid} structure.
     * <p>
     * Information about the {@link yfiles.layout.PartitionGrid partition grid} structure are held by a {@link yfiles.algorithms.IDataProvider} registered with the graph
     * with key {@link yfiles.layout.PartitionGrid#PARTITION_GRID_DP_KEY}.
     * </p>
     * @see yfiles.layout.PartitionGrid
     * @see yfiles.hierarchic.HierarchicLayout
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface PartitionGridLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class PartitionGridLayoutStage {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.PartitionGridLayoutStage}.
       */
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is an implementation of the {@link yfiles.hierarchic.IPortConstraintOptimizer} interface that tries to assign the
     * edges of the graph to {@link yfiles.layout.PortConstraint ports} considering the specified {@link yfiles.layout.PortCandidate}s.
     * <p>
     * Instances of this class can be set using property {@link yfiles.hierarchic.HierarchicLayoutCore#portConstraintOptimizer}.
     * </p>
     * <p>
     * This implementation will query for {@link yfiles.algorithms.IDataProvider}s registered with the graph with keys
     * {@link yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY}, {@link yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY}
     * and {@link yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY}. It will <b>try</b> to assign each edge one of the matching
     * {@link yfiles.layout.PortCandidate}s without introducing too many crossings and without violating the {@link yfiles.layout.PortCandidate#cost cost} or {@link yfiles.layout.PortCandidateSet#connectionCount capacity}
     * constraints.
     * </p>
     * @see yfiles.layout.PortCandidate#cost
     * @see yfiles.layout.PortCandidateSet#entries
     * @see yfiles.layout.PortCandidateSet#connectionCount
     * @class
     * @extends {yfiles.hierarchic.PortConstraintOptimizerBase}
     */
    export interface PortCandidateOptimizer extends yfiles.hierarchic.PortConstraintOptimizerBase{}
    export class PortCandidateOptimizer {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.PortCandidateOptimizer} with the default settings.
       */
      constructor();
      /**
       * Returns the data provider that provides the port candidate sets for nodes.
       * @param graph The graph that stores the data provider
       * @returns 
       * @protected
       */
      getPortCandidateSetDataProvider(graph:yfiles.layout.LayoutGraph):yfiles.algorithms.IDataProvider;
      /**
       * Gets or sets whether or not this implementation uses a deterministic algorithm to assign the
       * {@link yfiles.layout.PortCandidate}s.
       * @type {boolean}
       */
      deterministic:boolean;
      /**
       * Gets or sets the penalty cost associated with each back-loop.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the penalty cost is negative
       * @type {number}
       */
      backLoopPenalty:number;
      /**
       * Gets or sets the penalty cost associated with a crossing that would occur if a given combination of
       * {@link yfiles.layout.PortCandidate}s would be chosen.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the crossing penalty is negative
       * @type {number}
       */
      crossingPenalty:number;
      /**
       * Gets or sets the penalty cost associated with each edge being assigned to a port which has already reached its
       * {@link yfiles.layout.IPortCandidateSetEntry#connections capacity}.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the penalty cost is negative
       * @see yfiles.layout.IPortCandidateSetEntry#connections
       * @type {number}
       */
      overUsagePenalty:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This static class provides information about the same-layer structures created by class
     * {@link yfiles.hierarchic.PortConstraintOptimizerBase}.
     * @class
     */
    export interface PortConstraintOptimizerSameLayerData extends Object{}
    export class PortConstraintOptimizerSameLayerData {
      /**
       * Creates a new {@link yfiles.hierarchic.PortConstraintOptimizerSameLayerData} instance.
       */
      constructor();
      /**
       * Adds a given dummy {@link yfiles.algorithms.Node} to this {@link yfiles.hierarchic.PortConstraintOptimizerSameLayerData} instance and
       * associates it with the given original (same-layer) {@link yfiles.algorithms.Edge}.
       * @param sameLayerDummy a dummy node
       * @param originalEdge the original (same-layer) edge
       */
      addDummyNode(sameLayerDummy:yfiles.algorithms.Node,originalEdge:yfiles.algorithms.Edge):void;
      /**
       * Returns the original (same-layer) edge associated with the given dummy {@link yfiles.algorithms.Node}.
       * @param sameLayerDummy a given dummy node
       * @returns the original (same-layer) edge associated with the given dummy node
       */
      getOriginalEdge(sameLayerDummy:yfiles.algorithms.Node):yfiles.algorithms.Edge;
      /**
       * Gets all dummy nodes of this {@link yfiles.hierarchic.PortConstraintOptimizerSameLayerData} instance.
       * @type {yfiles.algorithms.NodeList}
       */
      dummyNodes:yfiles.algorithms.NodeList;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is an abstract implementation of the {@link yfiles.hierarchic.IPortConstraintOptimizer} interface that minimizes the
     * effort required to modify the port assignment after the sequencing phase of hierarchic layout algorithm.
     * <p>
     * The {@link yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencing callback method invoked after sequencing} performs the following three steps:
     * </p>
     * <ol>
     * <li>
     * Restores <b>temporarily</b> all same-layer edges which are not currently present in the layout graph at this time of the layout
     * algorithm.
     * </li>
     * <li>Invokes {@link yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencingForSingleNode the hook} in which the custom port assignment should be done.</li>
     * <li>Restores the original state of the layout graph by removing the temporary same-layer edges added in Step <code>1</code>.</li>
     * </ol>
     * @class
     * @implements {yfiles.hierarchic.IPortConstraintOptimizer}
     */
    export interface PortConstraintOptimizerBase extends Object,yfiles.hierarchic.IPortConstraintOptimizer{}
    export class PortConstraintOptimizerBase {
      /**
       * Creates a new {@link yfiles.hierarchic.PortConstraintOptimizerBase} instance with default settings.
       * @protected
       */
      constructor();
      /**
       * Inserts a same-layer edge structure for each same-layer edge of the original graph.
       * @param graph the input graph
       * @param layers the given {@link } instance holding the layering information
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link  factory} that sets the temporary port constraints
       * @returns 
       * @see yfiles.hierarchic.IItemFactory#setTemporaryPortConstraint
       * @protected
       */
      insertSameLayerStructures(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):yfiles.hierarchic.PortConstraintOptimizerSameLayerData;
      /**
       * Assigns new temporary port constraints after the nodes have been assigned to layers.
       * <p>
       * More precisely, it is called after the layering information has been determined. In this phase, it is possible to create
       * back-loops by assigning incoming edges to the south (i.e. bottom) side or outgoing edges to the north (i.e. top) side,
       * respectively.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance holding the layering information
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link  factory} that sets the temporary port constraints
       * @see yfiles.hierarchic.IItemFactory#setTemporaryPortConstraint
       * @abstract
       */
      optimizeAfterLayering(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Assigns new temporary port constraints after the order of the nodes in each layer has been determined.
       * <p>
       * More precisely, it is called after the sequence of the nodes has been determined. It {@link yfiles.hierarchic.PortConstraintOptimizerBase#insertSameLayerStructures inserts the same-layer structures},
       * invokes {@link yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencingForSingleNode the hook} in which the custom port assignment should be done, and finally {@link yfiles.hierarchic.PortConstraintOptimizerBase#removeSameLayerStructures restores the original state} of the
       * layout graph by removing the temporary edges.
       * </p>
       * @param graph the input graph
       * @param layers the given {@link } instance holding the layering information
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link  factory} that sets the temporary port constraints
       * @see yfiles.hierarchic.PortConstraintOptimizerBase#insertSameLayerStructures
       * @see yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencing
       * @see yfiles.hierarchic.PortConstraintOptimizerBase#removeSameLayerStructures
       */
      optimizeAfterSequencing(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Assigns new temporary port constraints after the order of the nodes in each layer has been determined.
       * <p>
       * More precisely, it is called after the sequence of the nodes has been determined. It invokes {@link yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencingForSingleNode}
       * for every node of the original layout graph and omits the nodes of the same-layer structures.
       * </p>
       * <p>
       * Incoming and outgoing edges are sorted using {@link yfiles.collections.IComparer.<T>} instances which define the preferred ordering of the incoming and outgoing
       * edges from left to right.
       * </p>
       * @param inEdgeOrder a given {@link } instance for incoming edges
       * @param outEdgeOrder a given {@link } instance for outgoing edges
       * @param graph the input graph
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link  factory} that sets the temporary port constraints
       * @param layers
       * @see yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencing
       * @protected
       */
      optimizeAfterSequencingForAllNodes(inEdgeOrder:yfiles.collections.IComparer<Object>,outEdgeOrder:yfiles.collections.IComparer<Object>,graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Assigns new temporary port constraints to a given node of the graph after the order of the nodes in each layer has been
       * determined.
       * <p>
       * More precisely, it is called after the sequence of the nodes has been determined.
       * </p>
       * <p>
       * Incoming and outgoing edges are sorted using {@link yfiles.collections.IComparer.<T>} instances which define the preferred ordering of the incoming and outgoing
       * edges from left to right.
       * </p>
       * @param node the original node to set temporary port constraints
       * @param inEdgeOrder a given {@link } instance for incoming edges
       * @param outEdgeOrder a given {@link } instance for outgoing edges
       * @param graph the input graph
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link  factory} that sets the temporary port constraints
       * @see yfiles.hierarchic.PortConstraintOptimizerBase#optimizeAfterSequencing
       * @protected
       * @abstract
       */
      optimizeAfterSequencingForSingleNode(node:yfiles.algorithms.Node,inEdgeOrder:yfiles.collections.IComparer<Object>,outEdgeOrder:yfiles.collections.IComparer<Object>,graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Removes the same-layer edge structure created using
       * {@link yfiles.hierarchic.PortConstraintOptimizerBase#insertSameLayerStructures}.
       * @param sameLayerData a given {@link } instance holding the information about the same-layer
       * structures.
       * @param graph the input graph
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param itemFactory the {@link  factory} that sets the temporary port constraints
       * @see yfiles.hierarchic.IItemFactory#setTemporaryPortConstraint
       * @protected
       */
      removeSameLayerStructures(sameLayerData:yfiles.hierarchic.PortConstraintOptimizerSameLayerData,graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,itemFactory:yfiles.hierarchic.IItemFactory):void;
      /**
       * Gets or sets the mirror mask that defines which orientations should be mirrored.
       * <p>
       * Setting a layout orientation can be considered as rotating the graph by <code>90</code>, <code>180</code> or <code>270</code> degrees. Afterwards, the graph
       * can be mirrored at the x-axis (for horizontal layout orientations) or y-axis (for vertical layout orientations). Which
       * directions are mirrored can be defined by the given mask.
       * </p>
       * @see yfiles.layout.OrientationLayout
       * @see yfiles.layout.LayoutOrientation
       * @type {yfiles.layout.MirrorModes}
       */
      mirrorMode:yfiles.layout.MirrorModes;
      /**
       * Gets or sets the layout orientation.
       * <p>
       * This setting is necessary to correctly interpret the values provided by the {@link yfiles.layout.PortCandidate}s since the {@link yfiles.layout.OrientationLayout}
       * cannot automatically adjust these values.
       * </p>
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @type {yfiles.layout.LayoutOrientation}
       */
      layoutOrientation:yfiles.layout.LayoutOrientation;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used by {@link yfiles.hierarchic.EdgeLayoutDescriptor} to specify the routing style for different types of edges.
     * @see yfiles.hierarchic.EdgeLayoutDescriptor#routingStyle
     * @see yfiles.hierarchic.IEdgeData#type
     * @class
     */
    export interface RoutingStyle extends Object{}
    export class RoutingStyle {
      /**
       * Creates a new {@link yfiles.hierarchic.RoutingStyle} instance with the given routing style for each edge.
       * @param routingStyle one of the predefined routing styles
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown routing style is given
       */
      constructor(routingStyle:yfiles.hierarchic.EdgeRoutingStyle);
      /**
       * Gets or sets the routing style for back-loop edges.
       * <p>
       * This style is used for routing U-turns of common edges (i.e., edges that are neither self-loops nor same-layer edges). A
       * U-turn is a non-monotonic part of the edge route that is required for reversed edges if option {@link yfiles.hierarchic.HierarchicLayout#backLoopRouting} is enabled or in some
       * other scenarios with port constraints/candidates.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown routing style is given
       * @type {yfiles.hierarchic.EdgeRoutingStyle}
       */
      backLoopRoutingStyle:yfiles.hierarchic.EdgeRoutingStyle;
      /**
       * Gets or sets the routing style for grouped edges at the common segments.
       * <p>
       * More precisely, grouped edges are routed in a bus-style fashion (i.e., the paths of the edges will share a common edge
       * segment) and this option allows to specify the routing style at the bus.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown routing style is given
       * @type {yfiles.hierarchic.EdgeRoutingStyle}
       */
      edgeGroupRoutingStyle:yfiles.hierarchic.EdgeRoutingStyle;
      /**
       * Gets or sets the default routing style for edges that have no individual routing style.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown routing style is given
       * @type {yfiles.hierarchic.EdgeRoutingStyle}
       */
      defaultEdgeRoutingStyle:yfiles.hierarchic.EdgeRoutingStyle;
      /**
       * Gets or sets the routing style for same-layer edges.
       * <p>
       * A same-layer edge is an edge whose source/target are assigned to the same layer.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown routing style is given
       * @type {yfiles.hierarchic.EdgeRoutingStyle}
       */
      sameLayerEdgeRoutingStyle:yfiles.hierarchic.EdgeRoutingStyle;
      /**
       * Gets or sets the routing style for self-loops.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown routing style is given
       * @type {yfiles.hierarchic.EdgeRoutingStyle}
       */
      selfLoopRoutingStyle:yfiles.hierarchic.EdgeRoutingStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class calculates bend points for self-loops routed in orthogonal or octilinear fashion.
     * <p>
     * It is used during the layout phase of {@link yfiles.hierarchic.HierarchicLayoutCore}, but can be used as a stand-alone tool.
     * </p>
     * @class
     */
    export interface SelfLoopCalculator extends Object{}
    export class SelfLoopCalculator {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.SelfLoopCalculator} using the given parameters.
       * @param minFirstSegmentLength the minimum length of the first and last segment of an orthogonally routed self-loop
       * @param minEdgeDistance the minimum distance between a pair of self-loops
       */
      constructor(minFirstSegmentLength:number,minEdgeDistance:number);
      /**
       * Convenience method that calculates a list of self-loops that belong to a given node.
       * <p>
       * The current port positions will be used for determining the start and end ports. The {@link yfiles.algorithms.IDataProvider}
       * instances hold information about the source/target ports of the edges and can be used for specifying the direction of
       * first and last segments.
       * </p>
       * @param graph the input graph
       * @param node the node whose self-loops will be routed
       * @param selfLoops a list of {@link }s that will be routed
       * @param spc the {@link } that returns the {@link } object associated with the source
       * ports of the edges
       * @param tpc the {@link } that returns the {@link } object associated with the target
       * ports of the edges
       */
      calculateSelfLoops(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node,selfLoops:yfiles.algorithms.YList,spc:yfiles.algorithms.IDataProvider,tpc:yfiles.algorithms.IDataProvider):void;
      /**
       * Calculates all self-loops at the given node of the given input graph.
       * <p>
       * The current port positions will be used for determining the start and end ports. The {@link yfiles.algorithms.IDataProvider}
       * instances hold information about the source/target ports of the edges and can be used for specifying the direction of
       * first and last segments.
       * </p>
       * @param graph the input graph
       * @param node the node whose self-loops will be routed
       * @param spc the {@link } that returns the {@link } object associated with the source
       * ports of the edges
       * @param tpc the {@link } that returns the {@link } object associated with the target
       * ports of the edges
       */
      calculateSelfLoops(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node,spc:yfiles.algorithms.IDataProvider,tpc:yfiles.algorithms.IDataProvider):void;
      /**
       * Returns the minimum length between two edges associated with the given edge.
       * <p>
       * The value is fetched from the corresponding {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.hierarchic.SelfLoopCalculator#MINIMUM_EDGE_DISTANCE_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param edge the given edge
       * @returns the minimum length between two edges
       * @protected
       */
      getMinimumDistance(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):number;
      /**
       * Returns the minimum length of the first segment of the given edge.
       * <p>
       * The value is fetched from the corresponding {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.hierarchic.SelfLoopCalculator#MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param edge the given edge
       * @returns the minimum length of the first segment of an edge
       * @protected
       */
      getMinimumFirstSegmentLength(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):number;
      /**
       * Returns the minimum length of the last segment of the given edge.
       * <p>
       * The value is fetched from the corresponding {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.hierarchic.SelfLoopCalculator#MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param edge the given edge
       * @returns the minimum length of the last segment of the given edge
       * @protected
       */
      getMinimumLastSegmentLength(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):number;
      /**
       * Returns the minimum distance between two nodes associated with the given node.
       * <p>
       * The value is fetched from the corresponding {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.hierarchic.SelfLoopCalculator#MINIMUM_NODE_DISTANCE_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param node the given node
       * @returns the minimum distance between two nodes
       * @protected
       */
      getMinimumNodeDistance(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):number;
      /**
       * Returns the minimum octilinear segment length of the given edge.
       * <p>
       * The value is fetched from the corresponding {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.hierarchic.SelfLoopCalculator#MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param edge the given edge
       * @returns the minimum octilinear segment length of the given edge
       * @protected
       */
      getMinimumOctilinearSegmentLength(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):number;
      /**
       * Returns whether or not the given edge is octilinear.
       * <p>
       * The value is fetched from the corresponding {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.hierarchic.SelfLoopCalculator#OCTILINEAR_EDGES_DP_KEY}.
       * </p>
       * @param graph the input graph
       * @param edge the given edge
       * @returns <code>true</code> if the edge is octilinear, <code>false</code> otherwise
       * @protected
       */
      isOctilinearEdge(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):boolean;
      /**
       * Data provider key for defining for each node a minimum distance to the self-loop edges
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static MINIMUM_NODE_DISTANCE_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data provider key for specifying whether or not an edge is octilinear
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static OCTILINEAR_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Data provider key for specifying the minimum length of the octilinear segments of an octilinear edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for specifying the minimum length of the first segment of each edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for specifying the minimum length of the last segment of each edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for specifying the minimum distance to the next edge or node side
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static MINIMUM_EDGE_DISTANCE_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a {@link yfiles.hierarchic.INodePlacer} implementation based on {@link yfiles.algorithms.RankAssignments rank assignment}.
     * <p>
     * The drawer works with integer values and rounds double values to integers using sophisticated quantizing.
     * </p>
     * @class
     * @implements {yfiles.hierarchic.INodePlacer}
     */
    export interface SimplexNodePlacer extends Object,yfiles.hierarchic.INodePlacer{}
    export class SimplexNodePlacer {
      constructor();
      /**
       * 
       * @param graph
       * @param layoutDataProvider
       * @param layers
       */
      assignLayerCoordinates(graph:yfiles.layout.LayoutGraph,layoutDataProvider:yfiles.hierarchic.ILayoutDataProvider,layers:yfiles.hierarchic.ILayers):void;
      /**
       * Assigns the nodes of a given layer to the corresponding upper/lower sublayer if {@link yfiles.hierarchic.SimplexNodePlacer#nodeCompaction node compaction} is enabled.
       * <p>
       * Information about sublayers is given by a {@link yfiles.algorithms.INodeMap} whose GetBool method has to return <code>true</code> for each
       * node that should be placed in the lower sublayer (values are set by this method).
       * </p>
       * @param layerNodes a {@link  list} of all @link y.layout.hierarchic.incremental.NodeData#TYPE_NORMAL normal nodes} of a layer
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param graph the given graph
       * @param lowerSublayer the {@link } holding sublayer information
       * @see yfiles.hierarchic.SimplexNodePlacer#nodeCompaction
       * @protected
       */
      assignNodesToSublayer(layerNodes:yfiles.algorithms.NodeList,ldp:yfiles.hierarchic.ILayoutDataProvider,graph:yfiles.layout.LayoutGraph,lowerSublayer:yfiles.algorithms.INodeMap):void;
      /**
       * 
       * @param graph
       * @param layoutDataProvider
       * @param layers
       * @param drawingDistanceCalculator
       */
      assignSequenceCoordinates(graph:yfiles.layout.LayoutGraph,layoutDataProvider:yfiles.hierarchic.ILayoutDataProvider,layers:yfiles.hierarchic.ILayers,drawingDistanceCalculator:yfiles.hierarchic.IDrawingDistanceCalculator):void;
      /**
       * Returns the alignment of the node with a specified layer.
       * <p>
       * This callback method used by {@link yfiles.hierarchic.SimplexNodePlacer#assignLayerCoordinates}.
       * </p>
       * @param graph the input graph
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param node the given node
       * @param layerIndex the index of the layer
       * @param minLayerHeight the minimum height of the layer
       * @returns 
       * @protected
       */
      getLayerAlignment(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,node:yfiles.algorithms.Node,layerIndex:number,minLayerHeight:number):number;
      /**
       * Specifies the minimum allowed distance between two nodes of the same given layer.
       * <p>
       * This callback method is used by {@link yfiles.hierarchic.SimplexNodePlacer#assignSequenceCoordinates}.
       * </p>
       * @param graph the graph that contains the nodes
       * @param layer the layer that contains the nodes
       * @param predNode the node to the left of the other one, may be <code>null</code> to indicate a border line
       * @param succ the node to the right of the other one, may be <code>null</code> to indicate a border line
       * @returns the minimum distance allowed between those two nodes ( >=0 )
       * @see yfiles.hierarchic.IDrawingDistanceCalculator
       * @protected
       */
      getMinDistance(graph:yfiles.layout.LayoutGraph,layer:yfiles.hierarchic.ILayer,predNode:yfiles.algorithms.Node,succ:yfiles.algorithms.Node):number;
      /**
       * Returns the minimum height of a given {@link yfiles.hierarchic.ILayer layer}.
       * <p>
       * This callback method is used by {@link yfiles.hierarchic.SimplexNodePlacer#assignLayerCoordinates}.
       * </p>
       * @param graph the given graph
       * @param ldp the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param layer the given {@link } object
       * @returns the minimum height
       * @protected
       */
      getMinimumLayerHeight(graph:yfiles.layout.LayoutGraph,ldp:yfiles.hierarchic.ILayoutDataProvider,layer:yfiles.hierarchic.ILayer):number;
      /**
       * Returns whether or not the given node should be treated as a node with fixed (given) coordinates.
       * <p>
       * This callback method is used by both {@link yfiles.hierarchic.SimplexNodePlacer#assignLayerCoordinates} and
       * {@link yfiles.hierarchic.SimplexNodePlacer#assignSequenceCoordinates}.
       * </p>
       * <p>
       * If <code>inLayer</code> argument is <code>true</code>, this method is called from within the {@link yfiles.hierarchic.SimplexNodePlacer#assignLayerCoordinates}
       * method.
       * </p>
       * @param graph the given graph
       * @param provider the {@link } implementation which provides access to the {@link } and {@link }
       * instances
       * @param node the given node
       * @param inLayer <code>true</code> if the layer coordinate of the node is queried, <code>false</code> if the sequence coordinate is queried
       * @returns <code>true</code> if the node should be treated as fixed, <code>false</code> otherwise
       * @protected
       */
      isFixedNode(graph:yfiles.layout.LayoutGraph,provider:yfiles.hierarchic.ILayoutDataProvider,node:yfiles.algorithms.Node,inLayer:boolean):boolean;
      /**
       * Gets or sets whether or not nodes should be placed in a more compact style with respect to layout width.
       * <p>
       * If enabled, the algorithm may place the nodes of a layer in a stacked style (horizontally interleaving), i.e., it splits
       * the layer into an upper and lower sublayer and places adjacent nodes into different sublayers if this reduces the width
       * of the layer.
       * </p>
       * @see yfiles.hierarchic.SimplexNodePlacer#minimumSublayerDistance
       * @see yfiles.hierarchic.SimplexNodePlacer#assignNodesToSublayer
       * @type {boolean}
       */
      nodeCompaction:boolean;
      /**
       * Gets or sets the minimum distance between the upper and lower sublayer.
       * <p>
       * A too small distance value may prevent adjacent nodes from being placed in a stacked style (horizontally interleaving)
       * because the vertical distance between these nodes must be greater than or equal to the value specified by {@link yfiles.hierarchic.NodeLayoutDescriptor#minimumDistance}.
       * </p>
       * <p>
       * Values should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if minimum sublayer distance is negative
       * @see yfiles.hierarchic.SimplexNodePlacer#nodeCompaction
       * @type {number}
       */
      minimumSublayerDistance:number;
      /**
       * Gets or sets whether or not the {@link yfiles.hierarchic.SimplexNodePlacer} places labels in a more compact style.
       * <p>
       * If this option is enabled the algorithm tries to place adjacent label elements in a stacked style, i.e., horizontally
       * interleaving.
       * </p>
       * @type {boolean}
       */
      labelCompaction:boolean;
      /**
       * Gets or sets whether or not an optimization step should be applied that tries to further reduce the number of bends.
       * @see yfiles.hierarchic.SimplexNodePlacer#maximumDuration
       * @type {boolean}
       */
      bendReduction:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.hierarchic.SimplexNodePlacer} tries to create a maximally compact horizontal
       * layout at the cost of more bends.
       * <p>
       * For best results, {@link yfiles.hierarchic.SimplexNodePlacer#breakLongSegments} and {@link yfiles.hierarchic.SimplexNodePlacer#labelCompaction} should also be enabled.
       * </p>
       * @see yfiles.hierarchic.SimplexNodePlacer#labelCompaction
       * @see yfiles.hierarchic.SimplexNodePlacer#breakLongSegments
       * @type {boolean}
       */
      horizontalCompaction:boolean;
      /**
       * Gets or sets whether or not a postprocessing step should be applied to reduce the number of bends.
       * <p>
       * Furthermore, enabling this option may violate some minimum distances specified by the user and the edge distribution is
       * no longer uniform.
       * </p>
       * @type {boolean}
       */
      straightenEdges:boolean;
      /**
       * Gets or sets the strategy used for controlling the horizontal compactness of group nodes.
       * @type {yfiles.hierarchic.GroupCompactionPolicy}
       */
      groupCompactionStrategy:yfiles.hierarchic.GroupCompactionPolicy;
      /**
       * Gets or sets whether or not the {@link yfiles.hierarchic.SimplexNodePlacer} enforces the placement of nodes at their exact
       * current positions, even if this violates minimum distance constraints.
       * @type {boolean}
       */
      exactPlacement:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.hierarchic.SimplexNodePlacer} should try to use the coordinates given from the
       * current sketch for the determination of the layer coordinates.
       * @type {boolean}
       */
      fromSketchLayerAssignment:boolean;
      /**
       * Gets or sets the time limit (in milliseconds) set for the layout algorithm.
       * <p>
       * Values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the relative weight of edges crossing a swimlane relative to edges that stay in their lane.
       * <p>
       * Higher values lead to more compact drawings but may cause additional bends.
       * </p>
       * <p>
       * Values should lie within <code>[0,1]</code> interval.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the relative weight does not lie within [0,1]
       * @type {number}
       */
      swimLaneCrossingWeight:number;
      /**
       * Gets or sets whether or not a barycenter drawing mode should be used.
       * <p>
       * If this mode is enabled the resulting drawing is likely to have more bends, but may be more symmetric.
       * </p>
       * @type {boolean}
       */
      barycenterMode:boolean;
      /**
       * Gets or sets whether or not the {@link yfiles.hierarchic.SimplexNodePlacer} should break long edge segments in favor of a more
       * compact layout.
       * @type {boolean}
       */
      breakLongSegments:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used by {@link yfiles.hierarchic.HierarchicLayoutCore} during the various phases and models a swimlane structure.
     * <p>
     * For layout orientation
     * {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}, swimlanes partition the drawing area into vertical stripes. Each node is
     * placed within a user-specified lane. Swimlanes are, for example, often used in flow diagrams to separate logical units.
     * </p>
     * <p>
     * Each node should be associated with a SwimLaneDescriptor instance; nodes in the same lane may share one instance. The
     * results of the calculation of the swimlanes' geometry will be stored by the instances after the layout.
     * </p>
     * <p>
     * A {@link yfiles.hierarchic.SwimlaneDescriptor} instance can be bound to the layout algorithm using a {@link yfiles.algorithms.IDataProvider}
     * registered with the graph using key {@link yfiles.hierarchic.HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY} that returns a {@link yfiles.hierarchic.SwimlaneDescriptor}
     * instance for each node of the graph.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayoutCore#SWIMLANE_DESCRIPTOR_DP_KEY
     * @class
     * @implements {yfiles.lang.IComparable}
     */
    export interface SwimlaneDescriptor extends Object,yfiles.lang.IComparable{}
    export class SwimlaneDescriptor {
      /**
       * Creates a new {@link yfiles.hierarchic.SwimlaneDescriptor} instance with a given client {@link Object} and a given {@link yfiles.collections.IComparer.<T>}.
       * <p>
       * The given client {@link Object} should be an instance of {@link yfiles.lang.IComparable} if the given {@link yfiles.collections.IComparer.<T>} is <code>null</code>.
       * </p>
       * @param clientObject a client {@link } used for sorting the lanes
       * @param comparer a {@link } used for comparing the client objects
       * @throws {Stubs.Exceptions.ArgumentError} if the specified client {@link Object} is <code>null</code> or the specified {@link yfiles.collections.IComparer.<T>} is <code>null</code> but the specified client {@link Object} is not an instance of
       * {@link yfiles.lang.IComparable}.
       */
      constructor(clientObject:Object,comparer:yfiles.collections.IComparer<Object>);
      /**
       * Creates a new {@link yfiles.hierarchic.SwimlaneDescriptor} instance with a given {@link yfiles.lang.IComparable} object.
       * @param clientObject a {@link } used for determining the order of the lanes
       * @throws {Stubs.Exceptions.ArgumentError} if the specified client object is <code>null</code>.
       */
      constructor(clientObject:yfiles.lang.IComparable);
      /**
       * Implements the {@link yfiles.lang.IComparable} interface using the {@link yfiles.hierarchic.SwimlaneDescriptor#comparer} and {@link yfiles.hierarchic.SwimlaneDescriptor#clientObject} fields.
       * @param o
       * @returns 
       */
      compareTo(o:Object):number;
      /**
       * Gets or sets whether or not an index of a swimlane is fixed (based on the result of the comparison with the other
       * swimlanes) or whether the index should be chosen automatically in order to minimize edge lengths.
       * <p>
       * For all swimlanes where this property is set to
       * <code>true</code>, the relative ordering given by the client objects is preserved. The remaining swimlanes may be sorted again so
       * that the overall edge lengths are minimized.
       * </p>
       * @type {boolean}
       */
      indexFixed:boolean;
      /**
       * Gets or sets the client {@link Object object}.
       * <p>
       * The given client {@link Object} should be an instance of {@link yfiles.lang.IComparable}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if there is currently no {@link yfiles.collections.IComparer.<T> comparator} and the specified object is not a {@link yfiles.lang.IComparable} instance
       * @see yfiles.hierarchic.SwimlaneDescriptor#comparer
       * @see yfiles.hierarchic.SwimlaneDescriptor#comparer
       * @type {Object}
       */
      clientObject:Object;
      /**
       * Gets or sets the tightness factor of the lane.
       * <p>
       * This value should lie within <code>[0,1]</code> interval.
       * </p>
       * <p>
       * The greater the value, the more will the lane be forced to obtain its minimum possible width.
       * </p>
       * <ul>
       * <li>A value of <code>0.0d</code> will disable compression of the lane.</li>
       * <li>A value of <code>1.0d</code> will try to force the lane to obtain its {@link yfiles.hierarchic.SwimlaneDescriptor#minimumLaneWidth minimum width}.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if the tightness value is outside <code>[0,1]</code> interval
       * @type {number}
       */
      laneTightness:number;
      /**
       * Gets or sets the {@link yfiles.collections.IComparer.<T>} instance used for sorting the lanes.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.collections.IComparer.<T>} is <code>null</code>.
       * @type {yfiles.collections.IComparer.<Object>}
       */
      comparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets the minimum width of the swimlane.
       * <p>
       * The minimum width should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value negative
       * @type {number}
       */
      minimumLaneWidth:number;
      /**
       * Gets or sets the left insets of the swimlane in which no element will lie in the resulting layout.
       * <p>
       * The inset should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is less than <code>0</code>.
       * @type {number}
       */
      leftLaneInset:number;
      /**
       * Gets or sets the right insets of the swimlane in which no element will lie in the resulting layout.
       * <p>
       * The inset should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is less than <code>0</code>.
       * @type {number}
       */
      rightLaneInset:number;
      /**
       * Gets the computed position (smaller coordinate) of the swimlane after the layout has been calculated.
       * @type {number}
       */
      computedLanePosition:number;
      /**
       * Gets the computed width of the swimlane after the layout has been calculated.
       * @type {number}
       */
      computedLaneWidth:number;
      /**
       * Gets the computed zero-based index of the swimlane after the layout has been calculated.
       * @type {number}
       */
      computedLaneIndex:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This {@link yfiles.layout.ILayoutStage layout stage} can be used for automatically assigning {@link yfiles.hierarchic.SwimlaneDescriptor} instances to nodes by
     * treating top-level group nodes as swimlanes.
     * <p>
     * This stage will modify the hierarchy by temporarily removing the top-level group nodes and assigning corresponding
     * descriptors to the remaining nodes. After running the core layout, the top-level group nodes will be arranged to
     * function as swimlanes. This class allows using the current coordinates of the group nodes to
     * {@link yfiles.hierarchic.TopLevelGroupToSwimlaneStage#orderSwimlanesFromSketch arrange the swimlanes from sketch}. Also, the {@link yfiles.hierarchic.TopLevelGroupToSwimlaneStage#spacing spacing} between swimlanes may be specified.
     * </p>
     * <p>
     * This stage can be {@link yfiles.layout.MultiStageLayout#appendStage appended} to the {@link yfiles.hierarchic.HierarchicLayout}.
     * </p>
     * @see yfiles.hierarchic.HierarchicLayout
     * @see yfiles.hierarchic.SwimlaneDescriptor
     * @see yfiles.layout.GroupingKeys
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface TopLevelGroupToSwimlaneStage extends yfiles.layout.LayoutStageBase{}
    export class TopLevelGroupToSwimlaneStage {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.TopLevelGroupToSwimlaneStage} with the default settings.
       */
      constructor();
      /**
       * Gets or sets the spacing between the swimlanes.
       * <p>
       * The spacing should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the spacing is negative
       * @type {number}
       */
      spacing:number;
      /**
       * Gets or sets whether or not the order of the swimlanes should be determined based on the current sketch.
       * @type {boolean}
       */
      orderSwimlanesFromSketch:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class implements the layering phase of the {@link yfiles.hierarchic.HierarchicLayout} (i.e., assignment of the nodes to
     * layers).
     * <p>
     * It uses topological sorting to assign layers to the nodes.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[63.79,0,[[73.45,24.35,10.67,18.7,"1"]]],[121,150,[[130.66,174.35,10.67,18.7,"2"]]],[71,150,[[80.66,174.35,10.67,18.7,"3"]]],[162.99,0,[[172.65,24.35,10.67,18.7,"6"]]],[162.99,70,[[172.65,94.35,10.67,18.7,"7"]]],[112.99,70,[[122.65,94.35,10.67,18.7,"8"]]],[30.99,70,[[37.31,94.35,17.35,18.7,"12"]]],[35.5,220,[[45.16,244.35,10.67,18.7,"8"]]],[0,150,[[9.66,174.35,10.67,18.7,"9"]]]],"e":[[5,1,0,15,0,-15,[127.99,120,136,120]],[0,1,10,15,-10,-15,[88.79,50,92.49,50,92.49,130,126,130]],[0,2,0,15,0,-15,[78.79,50,81.49,50,81.49,120,86,120]],[4,1,0,15,10,-15,[177.99,120,146,120]],[3,4,0,15,0,-15],[0,6,-10,15,0,-15,[68.79,50,45.99,50]],[6,7,7.5,15,0,-15,[53.49,120,50.5,120]],[6,8,-7.5,15,0,-15,[38.49,120,15,120]],[8,7,0,15,-10,-15,[15,200,40.5,200]],[2,7,0,15,10,-15,[86,200,60.5,200]]],"vp":[0.0,0.0,193.0,250.0]}}
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface TopologicalLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class TopologicalLayerer {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.TopologicalLayerer} with default settings.
       */
      constructor();
      /**
       * Assigns all nodes of the graph to layers and adds them to the {@link yfiles.hierarchic.ILayers} instance.
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for querying information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Gets or sets the (re-)ranking policy used by this {@link yfiles.hierarchic.TopologicalLayerer} instance.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown ranking policy is given
       * @type {yfiles.hierarchic.RankingPolicy}
       */
      rankingPolicy:yfiles.hierarchic.RankingPolicy;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a {@link yfiles.hierarchic.IDrawingDistanceCalculator} implementation that returns the minimum distances for each
     * kind of node pair based on their type as returned by {@link yfiles.hierarchic.INodeData#type}.
     * @class
     * @implements {yfiles.hierarchic.IDrawingDistanceCalculator}
     */
    export interface TypeBasedDrawingDistanceCalculator extends Object,yfiles.hierarchic.IDrawingDistanceCalculator{}
    export class TypeBasedDrawingDistanceCalculator {
      /**
       * Creates a new instance of {@link yfiles.hierarchic.TypeBasedDrawingDistanceCalculator} with default settings.
       */
      constructor();
      /**
       * 
       * @param graph
       * @param layers
       * @param ldp
       */
      dispose(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * 
       * @param graph
       * @param layer
       * @param ldp
       * @param left
       * @param right
       * @returns 
       */
      getMinDistance(graph:yfiles.layout.LayoutGraph,layer:yfiles.hierarchic.ILayer,ldp:yfiles.hierarchic.ILayoutDataProvider,left:yfiles.algorithms.Node,right:yfiles.algorithms.Node):number;
      /**
       * 
       * @param graph
       * @param layers
       * @param ldp
       */
      initialize(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Gets or sets whether or not the compaction option is enabled.
       * @see yfiles.hierarchic.SimplexNodePlacer#nodeCompaction
       * @see yfiles.hierarchic.SimplexNodePlacer#labelCompaction
       * @type {boolean}
       */
      compaction:boolean;
      /**
       * Gets or sets the minimum distance between two {@link yfiles.algorithms.Node}s.
       * <p>
       * Minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      nodeToNodeDistance:number;
      /**
       * Gets or sets the minimum distance between a {@link yfiles.algorithms.Node} and an (non-adjacent) {@link yfiles.algorithms.Edge}.
       * <p>
       * Minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      nodeToEdgeDistance:number;
      /**
       * Gets or sets the minimum distance between two edges.
       * <p>
       * Minimum distance should be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum distance is negative
       * @type {number}
       */
      edgeToEdgeDistance:number;
      /**
       * Gets or sets the minimum length of the first segment of the edge path (at the source node).
       * <p>
       * The minimum length should be greater than <code>0</code>.
       * </p>
       * <p>
       * If the value is <code>0.0d</code> the first segment does not need to be orthogonal.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length of the first segment is negative
       * @type {number}
       */
      minimumFirstSegmentLength:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class implements the layering phase of the {@link yfiles.hierarchic.HierarchicLayout} (i.e., the assignment of the nodes to
     * layers).
     * <p>
     * It first makes the directed graph acyclic. Then, it assigns the layers such that edge weights are respected. A weight
     * defines a minimum distance between the two end points of an edge.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[100,70,[[109.66,94.35,10.67,18.7,"1"]]],[45.59,140,[[55.25,164.35,10.67,18.7,"2"]]],[95.59,140,[[105.25,164.35,10.67,18.7,"3"]]],[0,0,[[9.66,24.35,10.67,18.7,"6"]]],[0,70,[[9.66,94.35,10.67,18.7,"7"]]],[50,70,[[59.66,94.35,10.67,18.7,"8"]]],[145.59,140,[[151.91,164.35,17.35,18.7,"12"]]],[125.59,280,[[135.25,304.35,10.67,18.7,"8"]]],[125.59,210,[[135.25,234.35,10.67,18.7,"9"]]]],"e":[[5,1,0,15,0,-15,[65,120,60.59,120]],[0,1,-10,15,10,-15,[105,120,70.59,120]],[0,2,0,15,0,-15,[115,120,110.59,120]],[4,1,0,15,-10,-15,[15,120,50.59,120]],[3,4,0,15,0,-15],[0,6,10,15,0,-15,[125,120,160.59,120]],[6,7,7.5,15,10,-15,[168.09,190,176.09,190,176.09,260,150.59,260]],[6,8,-7.5,15,0,-15,[153.09,190,140.59,190]],[8,7,0,15,0,-15],[2,7,0,15,-10,-15,[110.59,190,105.09,190,105.09,260,130.59,260]]],"vp":[0.0,0.0,177.0,310.0]}}
     * </p>
     * @class
     * @implements {yfiles.hierarchic.ILayerer}
     */
    export interface WeightedLayerer extends Object,yfiles.hierarchic.ILayerer{}
    export class WeightedLayerer {
      /**
       * Creates an instance of {@link yfiles.hierarchic.WeightedLayerer} with the given key that provides access to the {@link yfiles.algorithms.IDataProvider}
       * that holds the edge weights and unlimited maximum duration for the {@link yfiles.hierarchic.WeightedLayerer}.
       * @param [key=null] the key to access the {@link } that holds the edge weights
       */
      constructor(key?:Object);
      /**
       * Assigns all nodes of the graph to layers and adds them to the {@link yfiles.hierarchic.ILayers} instance.
       * @param graph the input graph
       * @param layers the {@link } instance that will be filled with the results of the calculation
       * @param ldp the {@link } used for querying information about the nodes and edges
       */
      assignLayers(graph:yfiles.layout.LayoutGraph,layers:yfiles.hierarchic.ILayers,ldp:yfiles.hierarchic.ILayoutDataProvider):void;
      /**
       * Assigns all nodes of the graph to layers.
       * @param graph the graph for which the layers are determined
       * @param layerID the {@link } that will be filled by the layout algorithm and returns a zero-based layer index for each
       * node
       * @returns the number of layers
       */
      assignLayersFast(graph:yfiles.algorithms.Graph,layerID:yfiles.algorithms.INodeMap):number;
      /**
       * Assigns all nodes of the graph to layers.
       * @param graph the graph for which the layers are determined
       * @param layerID the {@link } that will be filled by the layout algorithm and returns the zero-based index of the layer
       * to which each node belongs
       * @returns the number of layers
       */
      assignLayersToMap(graph:yfiles.algorithms.Graph,layerID:yfiles.algorithms.INodeMap):number;
      /**
       * Assigns all nodes of the graph to layers.
       * @param graph the graph for which the layers are determined
       * @param layerID the {@link } that will be filled by the layout algorithm and returns a zero-based layer index for each
       * node
       * @param reversedEdges an {@link } containing the edges which had been reversed
       * @param weight the {@link } that returns an integer value (weight) for each edge
       * @returns the number of layers
       */
      assignNodeLayerWithDataProvider(graph:yfiles.layout.LayoutGraph,layerID:yfiles.algorithms.INodeMap,reversedEdges:yfiles.algorithms.EdgeList,weight:yfiles.algorithms.IDataProvider):number;
      /**
       * Returns the edge weight of the given edge using the weight based cycle removal heuristic.
       * <p>
       * By default, all edges are considered to have weight <code>1.0</code>.
       * </p>
       * <p>
       * It may be overridden if non-uniform weights are desired.
       * </p>
       * @param edge given edge
       * @returns the edge weight of the specified edge
       * @see yfiles.hierarchic.WeightedLayerer#weightedCycleRemoval
       * @see yfiles.hierarchic.WeightedLayerer#weightedCycleRemoval
       * @protected
       */
      getWeight(edge:yfiles.algorithms.Edge):number;
      /**
       * Removes cycles from the graph using a depth first search.
       * @param graph the graph
       * @param reversedEdges an {@link } containing the edges which will be reversed
       */
      makeDFSAcyclic(graph:yfiles.algorithms.Graph,reversedEdges:yfiles.algorithms.EdgeList):void;
      /**
       * This is a {@link yfiles.algorithms.IDataProvider} that holds for each {@link yfiles.algorithms.Edge} an integer value representing its
       * weight.
       * @protected
       * @type {yfiles.algorithms.IDataProvider}
       */
      weight:yfiles.algorithms.IDataProvider;
      /**
       * The key to access the {@link yfiles.algorithms.IDataProvider} that holds the edge weights.
       * <p>
       * More precisely, the corresponding {@link yfiles.algorithms.IDataProvider} maps each edge of the input graph to an integer value
       * representing the edge's weight.
       * </p>
       * @protected
       * @type {Object}
       */
      key:Object;
      /**
       * Gets or sets whether or not the edges that need to be reversed are determined using an edge weight based heuristic or
       * using a depth first search based approach.
       * <p>
       * The weight based approach may result in fewer reversed edges (especially for cases with many multi-edges) whereas the
       * DFS based approach is likely to be faster.
       * </p>
       * @type {boolean}
       */
      weightedCycleRemoval:boolean;
      /**
       * Gets or sets the time limit in milliseconds set for the layout algorithm.
       * <p>
       * Values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.hierarchic.HierarchicLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface HierarchicLayoutData extends yfiles.layout.LayoutData{}
    export class HierarchicLayoutData {
      constructor();
      /**
       * Gets or sets a mapping from edges to alternative paths for edges connecting to groups, group content or folder nodes.
       * @see yfiles.hierarchic.HierarchicLayout#ALTERNATIVE_EDGE_PATH_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.algorithms.YPointPath>}
       */
      alternativeEdgePath:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.algorithms.YPointPath>;
      /**
       * Gets or sets a mapping from group/folder nodes to alternative bounds for these nodes.
       * @see yfiles.hierarchic.HierarchicLayout#ALTERNATIVE_GROUP_BOUNDS_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.algorithms.YRectangle>}
       */
      alternativeGroupBounds:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.algorithms.YRectangle>;
      /**
       * Gets or sets a mapping from edges to their priority to be a 'critical' edge.
       * <p>
       * The layout tries to vertically align each node pair that is connected by a critical edge (integer value > 0).
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#CRITICAL_EDGE_PRIORITY_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      criticalEdgePriorities:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the collection of folder nodes used for recursive edge styles in incremental mode.
       * @see yfiles.hierarchic.HierarchicLayout#FOLDER_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      folderNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the mapping of edges to their {@link yfiles.hierarchic.EdgeLayoutDescriptor}.
       * <p>
       * If an edge is mapped to <code>null</code>, the {@link yfiles.hierarchic.HierarchicLayout#edgeLayoutDescriptor default descriptor} is used.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.hierarchic.EdgeLayoutDescriptor>}
       */
      edgeLayoutDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.hierarchic.EdgeLayoutDescriptor>;
      /**
       * Gets or sets the mapping of nodes to their {@link yfiles.hierarchic.NodeLayoutDescriptor}
       * <p>
       * If a node is mapped to <code>null</code>, the {@link yfiles.hierarchic.HierarchicLayout#nodeLayoutDescriptor default descriptor} is used.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayoutCore#NODE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.hierarchic.NodeLayoutDescriptor>}
       */
      nodeLayoutDescriptors:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.hierarchic.NodeLayoutDescriptor>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from nodes to their {@link yfiles.layout.PortCandidateSet}.
       * @see yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.PortCandidateSet>}
       */
      nodePortCandidateSets:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.PortCandidateSet>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their source port group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourcePortGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target port group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetPortGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the factory to specify layer constraints.
       * @see yfiles.hierarchic.HierarchicLayout#createLayerConstraintFactory
       * @see yfiles.hierarchic.ILayerConstraintFactory
       * @type {yfiles.hierarchic.ILayerConstraintFactory}
       */
      layerConstraintFactory:yfiles.hierarchic.ILayerConstraintFactory;
      /**
       * Gets or sets the factory to specify sequence constraints.
       * @see yfiles.hierarchic.HierarchicLayout#createSequenceConstraintFactory
       * @see yfiles.hierarchic.ISequenceConstraintFactory
       * @type {yfiles.hierarchic.ISequenceConstraintFactory}
       */
      sequenceConstraintFactory:yfiles.hierarchic.ISequenceConstraintFactory;
      /**
       * Gets or sets a mapping from nodes and edges to their incrementally hints.
       * @see yfiles.hierarchic.HierarchicLayout#INCREMENTAL_HINTS_DP_KEY
       * @type {yfiles.hierarchic.IncrementalHintItemMapping}
       */
      incrementalHints:yfiles.hierarchic.IncrementalHintItemMapping;
      /**
       * Gets or sets a mapper from nodes to the index of their layer.
       * <p>
       * If this property is set, the mapper is filled by the layout with the layer information.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#LAYER_INDEX_DP_KEY
       * @type {yfiles.collections.IMapper.<yfiles.graph.INode,number>}
       */
      layerIndices:yfiles.collections.IMapper<yfiles.graph.INode,number>;
      /**
       * Gets or sets a mapper from nodes to the sequence index in their layer.
       * <p>
       * If this property is set, the mapper is filled by the layout with the sequence information.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#SEQUENCE_INDEX_DP_KEY
       * @type {yfiles.collections.IMapper.<yfiles.graph.INode,number>}
       */
      sequenceIndices:yfiles.collections.IMapper<yfiles.graph.INode,number>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the partition grid layout data.
       * @type {yfiles.layout.PartitionGridData}
       */
      partitionGridData:yfiles.layout.PartitionGridData;
      /**
       * Gets or sets the layout data for the {@link yfiles.hierarchic.SelfLoopCalculator}.
       * @type {yfiles.hierarchic.SelfLoopCalculatorData}
       */
      selfLoopCalculatorData:yfiles.hierarchic.SelfLoopCalculatorData;
      /**
       * Gets or sets the mapping from edges to an additional weight used by the {@link yfiles.hierarchic.ConstraintIncrementalLayerer}.
       * <p>
       * The Layerer tries to keep edges with higher weights short.
       * </p>
       * @see yfiles.hierarchic.ConstraintIncrementalLayerer#ADDITIONAL_EDGE_WEIGHT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      constraintIncrementalLayererAdditionalEdgeWeights:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the collection of core nodes used by the {@link yfiles.hierarchic.BFSLayerer}.
       * <p>
       * The {@link yfiles.hierarchic.BFSLayerer} is used when {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy} is set to {@link yfiles.hierarchic.LayeringStrategy#BFS}.
       * </p>
       * @see yfiles.hierarchic.BFSLayerer#CORE_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      bfsLayererCoreNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or set the mapping from nodes to their layer index when using the {@link yfiles.hierarchic.GivenLayersLayerer}.
       * <p>
       * The {@link yfiles.hierarchic.GivenLayersLayerer} is used when {@link yfiles.hierarchic.HierarchicLayout#fromScratchLayeringStrategy} is set to {@link yfiles.hierarchic.LayeringStrategy#USER_DEFINED}.
       * </p>
       * @see yfiles.hierarchic.GivenLayersLayerer#LAYER_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      givenLayersLayererIds:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      /**
       * Gets or sets the mapping from edges to their directedness.
       * <p>
       * Generally, the hierarchic layout algorithm assigns nodes to layers such that most of the edges point in the
       * {@link yfiles.layout.MultiStageLayout#layoutOrientation main layout direction}. The directedness of an edge specifies whether it should comply with this strategy. More
       * precisely, a value of <code>1</code> means that the edge should fully comply, a value of <code>-1</code> that it should comply inversely (the edge
       * should point against the main layout direction), and a value of <code>0</code> means that the direction doesn't matter at all and the
       * endpoints of the edges may be placed at the same layer. If there are conflicting preferences, edges with higher absolute
       * values are more likely to point in the desired direction.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#EDGE_DIRECTEDNESS_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      edgeDirectedness:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the mapping from edges to their thickness.
       * <p>
       * The specified non-negative thickness is considered when calculating minimum distances so that there are no overlaps
       * between edges and other graph elements. By default, each edge has thickness <code>0</code>.
       * </p>
       * @see yfiles.hierarchic.HierarchicLayout#EDGE_THICKNESS_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      edgeThickness:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the mapping that provides a {@link yfiles.layout.PreferredPlacementDescriptor} instance for edge
       * {@link yfiles.graph.ILabel}s.
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>}
       */
      edgeLabelPreferredPlacement:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Provides different ways to define a mapping from model items to incremental hints for use with
     * {@link yfiles.hierarchic.HierarchicLayoutData}.
     * <p>
     * Only one of the provided ways to define a mapping can be used at the same time, therefore only one of the properties of
     * this class should be set on an instance.
     * </p>
     * @class
     * @extends {yfiles.layout.ContextItemMapping.<yfiles.graph.IModelItem,Object,yfiles.hierarchic.IIncrementalHintsFactory>}
     * @final
     */
    export interface IncrementalHintItemMapping extends yfiles.layout.ContextItemMapping<yfiles.graph.IModelItem,Object,yfiles.hierarchic.IIncrementalHintsFactory>{}
    export class IncrementalHintItemMapping {
      constructor();
      /**
       * Gets or sets an item collection of nodes that are mapped to {@link yfiles.hierarchic.IIncrementalHintsFactory#createLayerIncrementallyHint incremental layering hints}.
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      incrementalLayeringNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets an item collection of nodes that are mapped to {@link yfiles.hierarchic.IIncrementalHintsFactory#createSequenceIncrementallyHint incremental sequencing hints}.
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IModelItem>}
       */
      incrementalSequencingItems:yfiles.layout.ItemCollection<yfiles.graph.IModelItem>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.hierarchic.SelfLoopCalculator}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface SelfLoopCalculatorData extends yfiles.layout.LayoutData{}
    export class SelfLoopCalculatorData {
      constructor();
      /**
       * Gets or sets the collection of edges that are octilinear.
       * @see yfiles.hierarchic.SelfLoopCalculator#OCTILINEAR_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      octilinearEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the mapping from nodes to their minimum distance to selfloop edges.
       * @see yfiles.hierarchic.SelfLoopCalculator#MINIMUM_NODE_DISTANCE_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      minimumNodeDistances:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      /**
       * Gets or sets the mapping from edges to their minimum distance to the next edge and to node sides.
       * @see yfiles.hierarchic.SelfLoopCalculator#MINIMUM_EDGE_DISTANCE_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      minimumEdgeDistances:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the mapping from edges to the minimum length of their first segment.
       * @see yfiles.hierarchic.SelfLoopCalculator#MINIMUM_FIRST_SEGMENT_LENGTH_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      minimumFirstSegmentLengths:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the mapping from edges to the minimum length of their last segment.
       * @see yfiles.hierarchic.SelfLoopCalculator#MINIMUM_LAST_SEGMENT_LENGTH_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      minimumLastSegmentLengths:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the mapping from edges to the minimum length of their octilinear segments.
       * @see yfiles.hierarchic.SelfLoopCalculator#MINIMUM_OCTILINEAR_SEGMENT_LENGTH_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      minimumOctilinearSegmentLengths:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      static $class:yfiles.lang.Class;
    }
  }export namespace labeling{
    export enum OptimizationStrategy{
      /**
       * An optimization strategy aiming at a good balance between the available optimization options.
       * @see yfiles.labeling.MISLabelingBase#optimizationStrategy
       */
      BALANCED,
      /**
       * An optimization strategy that especially reduces overlaps between labels and nodes as well as labels and node halos.
       * @see yfiles.labeling.MISLabelingBase#optimizationStrategy
       */
      NODE_OVERLAP,
      /**
       * An optimization strategy that especially reduces overlaps between labels.
       * @see yfiles.labeling.MISLabelingBase#optimizationStrategy
       */
      LABEL_OVERLAP,
      /**
       * An optimization strategy that especially reduces overlaps between labels and edges.
       * @see yfiles.labeling.MISLabelingBase#optimizationStrategy
       */
      EDGE_OVERLAP,
      /**
       * An optimization strategy that mainly tries to satisfy the preferences described by a {@link yfiles.layout.PreferredPlacementDescriptor}
       * associated with edge labels.
       * @see yfiles.labeling.MISLabelingBase#optimizationStrategy
       */
      PREFERRED_PLACEMENT,
      /**
       * Use no optimization strategy.
       */
      NONE
    }
    /**
     * A generic labeling algorithm for placing the labels of a graph.
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1,"c":1,"lblFlip":1,"lblB":1},"n":[[249.44,602.63,[[212.75,626.98,32.69,18.7,"Node"]]],[306.92,686.94,[[270.24,711.29,32.69,18.7,"Node"]]],[481.7,787.85,[[515.7,812.2,32.69,18.7,"Node"]]],[219.36,505.12,[[182.67,529.47,32.69,18.7,"Node"]]],[386.71,750.57,[[350.02,774.92,32.69,18.7,"Node"]]],[885,353.19,[[883.66,405.89,32.69,18.7,"Node"]]],[840.72,261.25,[[839.38,257.25,32.69,18.7,"Node"]]],[583.46,112.72,[[582.11,108.72,32.69,18.7,"Node"]]],[249.44,305.56,[[212.75,329.91,32.69,18.7,"Node"]]],[682.94,135.42,[[681.6,131.42,32.69,18.7,"Node"]]],[481.7,120.34,[[480.35,116.34,32.69,18.7,"Node"]]],[771.32,186.45,[[769.97,182.45,32.69,18.7,"Node"]]],[219.36,403.07,[[182.67,427.42,32.69,18.7,"Node"]]],[306.92,221.25,[[270.24,245.6,32.69,18.7,"Node"]]],[386.71,157.62,[[385.36,153.62,32.69,18.7,"Node"]]]],"e":[[0,1,[],[[259.45,643.51,32.03,18.7,0.826238774315996,-0.5633200580636204,"Edge"]]],[3,0,[],[[221.61,542.21,32.03,18.7,0.9555728057861415,-0.294755174410902,"Edge"]]],[1,4,[],[[349.3,723.77,32.03,18.7,0.6234898018587343,-0.7818314824680294,"Edge"]]],[4,2,[],[[418.05,771.98,32.03,18.7,0.36534102436639637,-0.9308737486442037,"Edge"]]],[3,5,[],[[262.64,513.66,32.03,18.7,-0.2225209339563144,-0.9749279121818236,"Edge"]]],[2,5,[],[[534.29,789.82,32.03,18.7,-0.7330518718298267,-0.680172737770919,"Edge"]]],[4,5,[],[[427.6,744.91,32.03,18.7,-0.6234898018587336,-0.7818314824680299,"Edge"]]],[1,5,[],[[350.46,685.47,32.03,18.7,-0.5000000000000003,-0.8660254037844386,"Edge"]]],[0,5,[],[[298.89,624.2,32.03,18.7,-0.3653410243663947,-0.9308737486442044,"Edge"]]],[5,6,[],[[878.37,323.28,32.03,18.7,0.9009688679024184,-0.4338837391175598,"Edge"]]],[7,5,[],[[611.91,162.36,32.03,18.7,0.6234898018587336,-0.7818314824680299,"Edge"]]],[8,5,[],[[292.35,341.41,32.03,18.7,0.07473009358642455,-0.9972037971811801,"Edge"]]],[9,7,[],[[645.8,157.7,32.03,18.7,0.2225209339563144,-0.9749279121818236,"Edge"]]],[7,10,[],[[549.55,131.38,32.03,18.7,-0.07473009358642278,-0.9972037971811804,"Edge"]]],[10,5,[],[[515.14,167.59,32.03,18.7,0.5000000000000003,-0.8660254037844386,"Edge"]]],[6,11,[],[[796.42,239.83,32.03,18.7,0.7330518718298267,-0.680172737770919,"Edge"]]],[11,9,[],[[733.51,192.55,32.03,18.7,0.4999999999999995,-0.866025403784439,"Edge"]]],[8,12,[],[[267.76,373.25,32.03,18.7,-0.9555728057861401,-0.29475517441090626,"Edge"]]],[12,3,[],[[253.06,467.1,32.03,18.7,-1.0,-2.7257675990577433E-15,"Edge"]]],[12,5,[],[[265.18,415.76,32.03,18.7,-0.07473009358642366,-0.9972037971811802,"Edge"]]],[13,5,[],[[347.6,261.29,32.03,18.7,0.22252093395631525,-0.9749279121818234,"Edge"]]],[13,8,[],[[284.16,291.63,32.03,18.7,-0.826238774315993,-0.5633200580636247,"Edge"]]],[10,14,[],[[450.54,153.46,32.03,18.7,-0.3653410243663931,-0.930873748644205,"Edge"]]],[14,13,[],[[349.3,214.42,32.03,18.7,-0.6234898018587314,-0.7818314824680315,"Edge"]]],[14,5,[],[[424.24,201.56,32.03,18.7,0.36534102436639554,-0.930873748644204,"Edge"]]]],"vp":[182.0,90.0,735.0,728.0]}} Node labels and edge labels placed by the labeling algorithm
     * </p>
     * <p>
     * This algorithm can switch between two internal implementations. By default, it generates high quality label placements
     * even for difficult instances. This especially holds if the label models allow a large number of different
     * {@link yfiles.layout.LabelCandidate}s, i.e., there is a high potential for optimizations.
     * </p>
     * <p>
     * If the duration of the calculation is of utmost importance, the internal algorithm can be changed to a simpler
     * implementation by setting {@link yfiles.labeling.GenericLabeling#maximumDuration} to <code>0</code>. On the downside, the results will not be as good as the ones of the default mode.
     * </p>
     * <p>
     * In default mode, this algorithm reduces the labeling problem to the maximum independent set (MIS) problem and solves the
     * problem using simulated annealing. It is inspired by the article from Christensen, Marks and Shieber: <i>A General
     * Cartographic Labelling Algorithm</i>.
     * </p>
     * <p>
     * It is recommended to use {@link yfiles.layout.IEdgeLabelLayoutModel}s and
     * {@link yfiles.layout.INodeLabelLayoutModel}s which allow free positioning of labels to achieve best results with this generic
     * labeling algorithm.
     * </p>
     * <p>
     * This algorithm works according to the general labeling concept defined by {@link yfiles.labeling.LabelingBase}.
     * </p>
     * @see yfiles.labeling.LabelingBase
     * @class
     * @extends {yfiles.labeling.MISLabelingBase}
     */
    export interface GenericLabeling extends yfiles.labeling.MISLabelingBase{}
    export class GenericLabeling {
      /**
       * Creates a new instance of {@link yfiles.labeling.GenericLabeling} with default settings.
       */
      constructor();
      /**
       * Gets or sets whether or not this algorithm behaves deterministically.
       * <p>
       * If enabled, it produces the same results given the same input graph and settings.
       * </p>
       * @type {boolean}
       */
      deterministic:boolean;
      /**
       * Gets or sets the time limit for this algorithm in milliseconds.
       * <p>
       * <code>0x7FFFFFFF</code> denotes that there is no time limit. Values have to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * An abstract base class for generic labeling algorithms which place labels of graphs.
     * <p>
     * Labeling algorithms compute positions for labels in a given graph. The main goal is to place labels such that they do
     * not overlap with other graph elements. Labeling algorithms do not modify positions of nodes and edges.
     * </p>
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * Generic labeling algorithms use the model associated with a label (i.e. {@link yfiles.layout.INodeLabelLayout#labelModel node label model} for nodes and {@link yfiles.layout.IEdgeLabelLayout#labelModel edge label model}
     * for edges). The models are necessary to compute a set of {@link yfiles.layout.LabelCandidate}s, i.e., candidate positions for a
     * label. Then, one <i>best matching</i> candidate from the set will be selected. This selection depends on several preferences, e.g., if {@link yfiles.labeling.LabelingBase#removeNodeOverlaps overlaps with nodes}
     * are not allowed, candidates encoding overlapping positions will be ignored.
     * </p>
     * <p>
     * After one {@link yfiles.layout.LabelCandidate} has been chosen, the label model's parameter is used for encoding the label
     * position. Therefore, the result of a generic labeling algorithm is one model parameter per label. The parameter is {@link yfiles.layout.ILabelLayout#modelParameter stored with the label}
     * and expresses the label position with respect to the underlying label model.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * This class provides a general frame, offering functionality which can be shared by other generic labeling algorithms.
     * </p>
     * <p>
     * To only place labels or a subset of labels, the main labeling methods {@link yfiles.labeling.LabelingBase#label}, {@link yfiles.labeling.LabelingBase#label}
     * and {@link yfiles.labeling.LabelingBase#label} are offered.
     * </p>
     * <p>
     * As this class represents an {@link yfiles.layout.LayoutStageBase}, it allows usage as a generic labeling stage. By combining it
     * with a
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layouter}, the labeling will take place after the core layouter was executed
     * ({@link yfiles.labeling.LabelingBase#applyLayout}). This is especially useful if the core layouter does not support label
     * handling.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface LabelingBase extends yfiles.layout.LayoutStageBase{}
    export class LabelingBase {
      /**
       * Creates a new {@link yfiles.labeling.LabelingBase} instance with default settings.
       */
      constructor();
      /**
       * Returns the profit for placing a {@link yfiles.layout.LabelCandidate} with respect to the current {@link yfiles.labeling.LabelingBase#profitModel profit model}.
       * <p>
       * Method {@link yfiles.layout.IProfitModel#getProfit} on the current profit model will be invoked to compute the actual profit
       * value.
       * </p>
       * <p>
       * The higher the profit (rank) of a candidate is, the more likely it will be chosen as actual position by the algorithm.
       * </p>
       * @param candidate a label candidate
       * @returns the profit value between <code>0</code> and <code>1</code>
       */
      getProfit(candidate:yfiles.layout.LabelCandidate):number;
      /**
       * Places the labels of the input graph restricting the placement to labels contained in the given lists.
       * <p>
       * The {@link yfiles.labeling.LabelingBase#affectedLabelsDpKey selection} will not be considered; the given lists will define the selection of labels that are to be placed.
       * </p>
       * @param graph the input graph
       * @param nodeLabels a list of {@link }s defining the set of node labels that will be placed
       * @param edgeLabels a list of {@link }s defining the set of edge labels that will be placed
       */
      label(graph:yfiles.layout.LayoutGraph,nodeLabels:yfiles.algorithms.YList,edgeLabels:yfiles.algorithms.YList):void;
      /**
       * Places the labels of the input graph using a {@link yfiles.algorithms.IDataProvider} registered to the input graph with the
       * given key for determining which labels to place.
       * <p>
       * Instead of considering the current {@link yfiles.labeling.LabelingBase#affectedLabelsDpKey selection key}, the {@link yfiles.algorithms.IDataProvider} registered with the given key will
       * be used for selecting labels. Invoking this method has the same effect as first {@link yfiles.labeling.LabelingBase#affectedLabelsDpKey specifying the key} as selection key and
       * then invoking {@link yfiles.labeling.LabelingBase#label}.
       * </p>
       * @param graph the input graph
       * @param [key=null] the {@link } key for label selection
       */
      label(graph:yfiles.layout.LayoutGraph,key?:Object):void;
      /**
       * Data provider key for mapping models to edge/node labels
       * <p>
       * The {@link yfiles.layout.INodeLabelLayoutModel}s and {@link yfiles.layout.IEdgeLabelLayoutModel}s linked to
       * {@link yfiles.layout.INodeLabelLayout}s and {@link yfiles.layout.IEdgeLabelLayout}s by this {@link yfiles.algorithms.IDataProvider} are taken as
       * replacement models. This means that the original model is internally replaced by another model when computing valid
       * positions for the label. A model parameter that fits the calculated position of the replacement model best is set for
       * the original label model.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.ILabelLayoutDpKey.<Object>}
       */
      static LABEL_MODEL_DP_KEY:yfiles.algorithms.ILabelLayoutDpKey<Object>;
      /**
       * Gets or sets the {@link yfiles.layout.IProfitModel} for ranking the {@link yfiles.layout.LabelCandidate}s for labels.
       * <p>
       * The profit model is used when {@link yfiles.labeling.LabelingBase#getProfit calculating the profit} of a candidate.
       * </p>
       * <p>
       * The higher the profit (rank) of a candidate is, the more likely it will be chosen as actual position by the algorithm.
       * </p>
       * @see yfiles.layout.IProfitModel
       * @see yfiles.layout.SimpleProfitModel
       * @see yfiles.labeling.LabelingBase#getProfit
       * @type {yfiles.layout.IProfitModel}
       */
      profitModel:yfiles.layout.IProfitModel;
      /**
       * Gets or sets whether or not <em>internal</em> node labels are allowed to move.
       * <p>
       * A node label is <em>internal</em> if the label's boundaries are completely within the node's boundaries.
       * </p>
       * @type {boolean}
       */
      moveInternalNodeLabels:boolean;
      /**
       * Gets or sets whether or not a post-processing step to reduce the number of label overlaps is applied.
       * @type {boolean}
       */
      reduceLabelOverlaps:boolean;
      /**
       * Gets or sets whether or not {@link yfiles.layout.LabelCandidate label candidates} that overlap with nodes are removed.
       * <p>
       * If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
       * that an overlapping candidate is finally chosen.
       * </p>
       * <p>
       * The detection and removal of labels that overlap with nodes may increase the runtime of this algorithm.
       * </p>
       * @type {boolean}
       */
      removeNodeOverlaps:boolean;
      /**
       * Gets or sets whether or not {@link yfiles.layout.LabelCandidate label candidates} that overlap with edges are removed.
       * <p>
       * If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
       * that an overlapping candidate is finally chosen.
       * </p>
       * <p>
       * The detection and removal of labels that overlap with edges may increase the runtime of this algorithm.
       * </p>
       * @type {boolean}
       */
      removeEdgeOverlaps:boolean;
      /**
       * Gets or sets whether or not labels assigned to nodes are placed.
       * @see yfiles.labeling.LabelingBase#affectedLabelsDpKey
       * @type {boolean}
       */
      placeNodeLabels:boolean;
      /**
       * Gets or sets whether or not labels assigned to edges are placed.
       * @see yfiles.labeling.LabelingBase#affectedLabelsDpKey
       * @type {boolean}
       */
      placeEdgeLabels:boolean;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key to mark labels as selected for placement.
       * <p>
       * If a {@link yfiles.algorithms.IDataProvider} is registered with this key, only the selected labels will be placed, while all
       * other labels are considered fixed.
       * </p>
       * <p>
       * The registered {@link yfiles.algorithms.IDataProvider} needs to map from {@link yfiles.layout.ILabelLayout} to {@link boolean} where <code>true</code> indicates
       * that a label should be placed and <code>false</code> indicates that a label should be ignored.
       * </p>
       * @type {Object}
       */
      affectedLabelsDpKey:Object;
      /**
       * Gets or sets whether or not edge labels are automatically flipped if otherwise they would be upside-down.
       * @type {boolean}
       */
      autoFlipping:boolean;
      /**
       * Gets or sets whether or not edge labels may overlap with edges belonging to the same edge group as the label's edge.
       * <p>
       * An edge group is a set of edges which share a common bus near the target or source node. They are defined using
       * {@link yfiles.algorithms.IDataProvider}s registered with {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} and
       * {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {boolean}
       */
      edgeGroupOverlapAllowed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A base class for generic labeling algorithms which solve the labeling problem by reducing it to the maximum independent
     * set (MIS) problem.
     * <p>
     * Different optimization strategies are available; an optimization strategy defines which criteria the algorithm should
     * try to optimize. For example, reducing the number of overlaps between labels and nodes may be considered more important
     * than reducing the number of overlaps between labels and edges.
     * </p>
     * @see yfiles.labeling.GenericLabeling
     * @class
     * @extends {yfiles.labeling.LabelingBase}
     */
    export interface MISLabelingBase extends yfiles.labeling.LabelingBase{}
    export class MISLabelingBase {
      /**
       * Creates a new {@link yfiles.labeling.MISLabelingBase} instance with default settings.
       */
      constructor();
      /**
       * Returns a {@link yfiles.algorithms.INodeMap} which assigns a profit value to each node in the {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph}.
       * <p>
       * As the conflict graph's nodes represent {@link yfiles.layout.LabelCandidate}s, this mapping gives the profit value of label
       * candidates. The assigned value is defined as the difference between the {@link yfiles.labeling.LabelingBase#getProfit profit induced by the profit model} and the
       * candidate's {@link yfiles.layout.LabelCandidate#overlapPenalty overlap penalty}.
       * </p>
       * <p>
       * The returned map is a mapping from each {@link yfiles.algorithms.Node} (representing a label candidate) in the {@link yfiles.labeling.MISLabelingBase#conflictGraph}
       * to a {@link number} representing the profit value of the candidate.
       * </p>
       * @returns a mapping from nodes (i.e. label candidates) to their profit value
       */
      assignProfit():yfiles.algorithms.INodeMap;
      /**
       * Creates the edges in the conflict graph, i.e., one edge between two nodes if the corresponding
       * {@link yfiles.layout.LabelCandidate}s intersect.
       * <p>
       * The nodes of the {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph} represent
       * {@link yfiles.layout.LabelCandidate}s. An edge between candidates signals that they overlap. A maximum independent set will be
       * computed on the conflict graph to choose candidates such that no two candidates overlap.
       * </p>
       * <p>
       * This method may be overridden to change the structure of the {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph}. Edges between two
       * {@link yfiles.layout.LabelCandidate}s in the conflict graph signal that the two candidates should not be selected together. By
       * overriding this method, arbitrary reasons for indicating that two label candidates should not be chosen at the same time
       * can be modeled.
       * </p>
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @see yfiles.labeling.MISLabelingBase#nodesToBoxes
       * @see yfiles.labeling.MISLabelingBase#boxesToNodes
       * @see yfiles.labeling.MISLabelingBase#nodesToID
       * @protected
       */
      createEdges():void;
      /**
       * Indicates that an overlap between a {@link yfiles.layout.LabelCandidate} and an {@link yfiles.algorithms.Edge} of the input graph has
       * been found.
       * <p>
       * This method is called when finding overlaps while {@link yfiles.labeling.MISLabelingBase#createEdges creating edges} of the {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph}. It will store a factor
       * indicating how much the two elements overlap. The factor influences the {@link yfiles.labeling.LabelingBase#getProfit profit} assigned to the given label candidate.
       * </p>
       * <p>
       * This method may be overridden to realize a custom strategy for reacting to overlaps between label candidates and edges.
       * </p>
       * @param labelCandidate the {@link } overlapping with the given {@link }
       * @param edge the {@link } overlapping with the given {@link }
       * @param eSegment the {@link } of the given edge overlapping with the given candidate
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @see yfiles.labeling.MISLabelingBase#createEdges
       */
      foundEdgeOverlap(labelCandidate:yfiles.layout.LabelCandidate,edge:yfiles.algorithms.Edge,eSegment:yfiles.algorithms.LineSegment):void;
      /**
       * Indicates that an overlap between a {@link yfiles.layout.LabelCandidate} and a {@link yfiles.layout.NodeHalo} of the input graph has
       * been found.
       * <p>
       * This method is called when finding overlaps while {@link yfiles.labeling.MISLabelingBase#createEdges creating edges} of the {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph}. It will store a factor
       * indicating how much the two elements overlap. The factor influences the {@link yfiles.labeling.LabelingBase#getProfit profit} assigned to the given label candidate.
       * </p>
       * <p>
       * This method may be overridden to realize a custom strategy for reacting to overlaps between label candidates and node
       * halos.
       * </p>
       * @param labelCandidate the {@link } overlapping with a node halo
       * @param node the {@link } whose {@link } is overlapping with the given label candidate
       * @param haloRect the bounding box of the {@link } overlapping with the given label candidate
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @see yfiles.labeling.MISLabelingBase#createEdges
       */
      foundHaloOverlap(labelCandidate:yfiles.layout.LabelCandidate,node:yfiles.algorithms.Node,haloRect:yfiles.algorithms.YRectangle):void;
      /**
       * Indicates that an overlap between two {@link yfiles.layout.LabelCandidate}s has been found.
       * <p>
       * This method is called when finding overlaps while {@link yfiles.labeling.MISLabelingBase#createEdges creating edges} of the
       * {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph}. It will store a factor indicating how much the two candidates overlap. The factor influences the penalty
       * assigned when both candidates are chosen, i.e., the penalty for the corresponding overlap.
       * </p>
       * <p>
       * This method may be overridden to realize a custom strategy for reacting to overlaps among
       * {@link yfiles.layout.LabelCandidate}s.
       * </p>
       * @param candidate1 the first overlapping {@link }
       * @param candidate2 the second overlapping {@link }
       * @param edge the {@link } in {@link #conflictGraph} representing the found overlap
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @see yfiles.labeling.MISLabelingBase#createEdges
       */
      foundLabelOverlap(candidate1:yfiles.layout.LabelCandidate,candidate2:yfiles.layout.LabelCandidate,edge:yfiles.algorithms.Edge):void;
      /**
       * Indicates that an overlap between a {@link yfiles.layout.LabelCandidate} and a {@link yfiles.algorithms.Node} of the input graph has
       * been found.
       * <p>
       * This method is called when finding overlaps while {@link yfiles.labeling.MISLabelingBase#createEdges creating edges} of the {@link yfiles.labeling.MISLabelingBase#conflictGraph conflict graph}. It will store a factor
       * indicating how much the two elements overlap. The factor influences the {@link yfiles.labeling.LabelingBase#getProfit profit} assigned to the given label candidate.
       * </p>
       * <p>
       * This method may be overridden to realize a custom strategy for reacting to overlaps between label candidates and nodes.
       * </p>
       * @param labelCandidate the {@link } overlapping with the given node
       * @param node the {@link } overlapping with the given label candidate
       * @param nodeBox the bounding box of the given node
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @see yfiles.labeling.MISLabelingBase#createEdges
       */
      foundNodeOverlap(labelCandidate:yfiles.layout.LabelCandidate,node:yfiles.algorithms.Node,nodeBox:yfiles.algorithms.YRectangle):void;
      /**
       * The input graph that will be labeled.
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * The conflict graph modeling
       * {@link yfiles.layout.LabelCandidate}s as nodes and edges between them as conflicts, i.e., overlaps among candidates.
       * @see yfiles.labeling.MISLabelingBase#createEdges
       * @type {yfiles.algorithms.Graph}
       */
      conflictGraph:yfiles.algorithms.Graph;
      /**
       * The mapping from each node in the {@link yfiles.labeling.MISLabelingBase#conflictGraph} to the corresponding {@link yfiles.layout.LabelCandidate}
       * instance.
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @type {yfiles.algorithms.INodeMap}
       */
      nodesToBoxes:yfiles.algorithms.INodeMap;
      /**
       * The mapping from the {@link yfiles.layout.LabelCandidate}s to the corresponding nodes in the
       * {@link yfiles.labeling.MISLabelingBase#conflictGraph}.
       * @see yfiles.labeling.MISLabelingBase#conflictGraph
       * @type {yfiles.collections.IMap.<Object,Object>}
       */
      boxesToNodes:yfiles.collections.IMap<Object,Object>;
      /**
       * The mapping from nodes in the {@link yfiles.labeling.MISLabelingBase#conflictGraph} to a corresponding integer value (ID).
       * <p>
       * The ID denotes the actual label which a {@link yfiles.layout.LabelCandidate} (i.e. a node in the conflict graph) belongs to.
       * </p>
       * @type {yfiles.algorithms.INodeMap}
       */
      nodesToID:yfiles.algorithms.INodeMap;
      /**
       * Gets or sets the ratio between the internal profit (ip) and the profit computed using the specified {@link yfiles.labeling.LabelingBase#profitModel profit model} (sp).
       * <p>
       * This ratio defines how to weight the two profit values (ip) and (sp). The overall ratio is then computed as
       * <code>ratio * sp + (1 - ratio) * ip</code>. The profit of a {@link yfiles.layout.LabelCandidate} defines how likely it is that the candidate
       * will be chosen as actual label position.
       * </p>
       * <p>
       * The ratio is defined to be a value from the interval <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given ratio is negative or larger than <code>1</code>
       * @see yfiles.labeling.LabelingBase#profitModel
       * @see yfiles.labeling.MISLabelingBase#optimizationStrategy
       * @type {number}
       */
      customProfitModelRatio:number;
      /**
       * Gets or sets the optimization strategy which defines the importance of criteria when optimizing labeling results.
       * <p>
       * Depending on the strategy, criteria like label-node overlaps, label-label overlaps and others are more or less
       * important. For example, if the number of overlaps between labels and nodes is the most important criterion for the
       * result, strategy {@link yfiles.labeling.OptimizationStrategy#NODE_OVERLAP} should be chosen.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given strategy is unknown
       * @type {yfiles.labeling.OptimizationStrategy}
       */
      optimizationStrategy:yfiles.labeling.OptimizationStrategy;
      /**
       * Gets or sets whether or not {@link yfiles.layout.LabelCandidate label candidates} that overlap with nodes are removed.
       * <p>
       * If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
       * that an overlapping candidate is finally chosen.
       * </p>
       * <p>
       * The detection and removal of labels that overlap with nodes may increase the runtime of this algorithm.
       * </p>
       * @type {boolean}
       */
      removeNodeOverlaps:boolean;
      /**
       * Gets or sets whether or not {@link yfiles.layout.LabelCandidate label candidates} that overlap with edges are removed.
       * <p>
       * If overlapping candidates are not removed, they will be considered but get a penalty. Therefore, it is still less likely
       * that an overlapping candidate is finally chosen.
       * </p>
       * <p>
       * The detection and removal of labels that overlap with edges may increase the runtime of this algorithm.
       * </p>
       * @type {boolean}
       */
      removeEdgeOverlaps:boolean;
      /**
       * Gets or sets whether or not the number of ambiguous label placements is reduced by applying an additional optimization
       * step.
       * <p>
       * A label position is considered to be ambiguous if it might not be possible to identify to which graph element the label
       * belongs. For example, an edge label placed in between two edges is ambiguous.
       * </p>
       * <p>
       * Enabling this reduction step does not guarantee that no ambiguous placements are selected. The algorithm will try to
       * avoid them if other good positions without ambiguity are available. Other aspects like {@link yfiles.layout.PreferredPlacementDescriptor preferred placement} for edge
       * labels will still be more important than the reduction of ambiguity.
       * </p>
       * @type {boolean}
       */
      reduceAmbiguity:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.labeling.GenericLabeling}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface LabelingData extends yfiles.layout.LayoutData{}
    export class LabelingData {
      constructor();
      /**
       * Gets or sets the collection of labels that shall be placed by the algorithm.
       * <p>
       * Note that the {@link yfiles.layout.DpKeyItemCollection.<V>#dpKey} has to be set to the key assigned to {@link yfiles.labeling.LabelingBase#affectedLabelsDpKey}.
       * </p>
       * @see yfiles.labeling.LabelingBase#affectedLabelsDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.ILabel>}
       */
      affectedLabels:yfiles.layout.DpKeyItemCollection<yfiles.graph.ILabel>;
      /**
       * Gets or sets the mapping from node labels to an {@link yfiles.layout.INodeLabelLayoutModel} used by the labeling algorithm
       * instead of the original label model.
       * <p>
       * If a label has no mapping, the original model is used as normal.
       * </p>
       * @see yfiles.labeling.LabelingBase#LABEL_MODEL_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.INodeLabelLayoutModel>}
       */
      nodeLabelModels:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.INodeLabelLayoutModel>;
      /**
       * Gets or sets the mapping from edge labels to an {@link yfiles.layout.IEdgeLabelLayoutModel} used by the labeling algorithm
       * instead of the original label model.
       * <p>
       * If a label has no mapping, the original model is used as normal.
       * </p>
       * @see yfiles.labeling.LabelingBase#LABEL_MODEL_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.IEdgeLabelLayoutModel>}
       */
      edgeLabelModels:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.IEdgeLabelLayoutModel>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping that provides a {@link yfiles.layout.PreferredPlacementDescriptor} instance for edge
       * {@link yfiles.graph.ILabel}s.
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>}
       */
      edgeLabelPreferredPlacement:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>;
      static $class:yfiles.lang.Class;
    }
  }export namespace organic{
    export enum InitialPlacement{
      /**
       * Initial placement strategy for starting with randomly chosen node locations.
       * @see yfiles.organic.ClassicOrganicLayout#initialPlacement
       */
      RANDOM,
      /**
       * Initial placement strategy for starting with all node location coordinates set to <code>0</code>.
       * <p>
       * This strategy is best if the initial placement should be maximally neutral, ignoring the existing layout of the input
       * graph.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#initialPlacement
       */
      ZERO,
      /**
       * Initial placement strategy for starting with the node locations as they are in the input graph.
       * <p>
       * This strategy is best if the existing layout should change as little as possible.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#initialPlacement
       */
      AS_IS
    }
    export enum GroupsPolicy{
      /**
       * Group handling policy for considering and arranging the nodes contained in a group such that their togetherness is
       * highlighted.
       * @see yfiles.organic.ClassicOrganicLayout#groupNodePolicy
       */
      LAYOUT,
      /**
       * Group handling policy for keeping the relative positions of nodes that belong to the same group fixed.
       * <p>
       * The fixed groups will be taken into account when placing the other nodes of the graph.
       * </p>
       * <p>
       * By registering a {@link yfiles.algorithms.IDataProvider} with key
       * {@link yfiles.organic.ClassicOrganicLayout#FIXED_GROUP_NODES_DP_KEY}, the individual group nodes can be marked as fixed or not
       * fixed. If there is no data provider registered with the mentioned key, then all group nodes of the graph will be treated
       * as fixed.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#groupNodePolicy
       */
      FIXED,
      /**
       * Group handling policy for completely ignoring group nodes during the layout process.
       * @see yfiles.organic.ClassicOrganicLayout#groupNodePolicy
       */
      IGNORE
    }
    export enum Scope{
      /**
       * Scope mode indicating that the algorithm should place <em>all</em> nodes of the graph.
       * @see yfiles.organic.ClassicOrganicLayout#scope
       */
      ALL,
      /**
       * Scope mode indicating that the algorithm should <em>mainly</em> place a subset of nodes.
       * <p>
       * The subset is determined by a boolean value returned by a {@link yfiles.algorithms.IDataProvider} registered with key
       * {@link yfiles.organic.ClassicOrganicLayout#AFFECTED_NODES_DP_KEY}. Nodes that don't belong to the subset may be moved to a
       * certain degree but will keep their relative positions to the other nodes outside the subset.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#scope
       * @see yfiles.organic.ClassicOrganicLayout#AFFECTED_NODES_DP_KEY
       */
      MAINLY_SUBSET,
      /**
       * Scope mode indicating that the algorithm should <em>only</em> place a subset of nodes.
       * <p>
       * The subset is determined by a boolean value returned by a {@link yfiles.algorithms.IDataProvider} registered with key
       * {@link yfiles.organic.ClassicOrganicLayout#AFFECTED_NODES_DP_KEY}. Nodes not being part of the subset will remain at their
       * location.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#scope
       * @see yfiles.organic.ClassicOrganicLayout#AFFECTED_NODES_DP_KEY
       */
      SUBSET
    }
    export enum HorizontalOverlapCriterion{
      /**
       * Horizontal overlap criterion defining an overlap as horizontal if the overlapping area is greater in height than in
       * width.
       * <p>
       * Otherwise, if the overlap area's width is greater than or equal to its height, an overlap will be categorized as
       * vertical.
       * </p>
       * <p>
       * The area of an overlap is defined as the rectangle where two nodes intersect with each other.
       * </p>
       * @see yfiles.organic.ShuffleLayout#horizontalOverlapCriterion
       */
      INTERSECTION_BOX,
      /**
       * Horizontal overlap criterion categorizing an overlap as horizontal if the center-to-center difference between the
       * overlapping nodes is greater in horizontal direction (x-coordinates) than in vertical direction (y-coordinates).
       * <p>
       * Otherwise, if the center-to-center difference between two overlapping nodes is greater in vertical direction
       * (y-coordinates), the corresponding overlap is categorized as vertical. The same applies if the differences in vertical
       * and horizontal direction are equal.
       * </p>
       * @see yfiles.organic.ShuffleLayout#horizontalOverlapCriterion
       */
      NODE_CENTER,
      /**
       * Horizontal overlap criterion categorizing an overlap as horizontal if the required movement for solving the overlap is
       * shorter in horizontal direction than in vertical direction.
       * <p>
       * Otherwise, an overlap will be categorized as vertical.
       * </p>
       * <p>
       * This criterion tries to avoid moving nodes too much because the direction for resolving overlaps will be chosen such
       * that the shorter movement is preferred.
       * </p>
       * @see yfiles.organic.ShuffleLayout#horizontalOverlapCriterion
       */
      LESS_MOVEMENT
    }
    export enum ChainSubstructureStyle{
      /**
       * Substructure style specifier for chains that indicates that the algorithm does not handle such structures in a special
       * way.
       * @see yfiles.organic.OrganicLayout#chainSubstructureStyle
       */
      NONE,
      /**
       * Substructure style specifier for chains that leads to a compact (rotated) rectangular layout style for chains.
       * @see yfiles.organic.OrganicLayout#chainSubstructureStyle
       */
      RECTANGULAR,
      /**
       * Substructure style specifier for chains that leads to a straight-line layout style for chains.
       * @see yfiles.organic.OrganicLayout#chainSubstructureStyle
       */
      STRAIGHT_LINE
    }
    export enum StarSubstructureStyle{
      /**
       * Substructure style specifier for stars that indicates that the algorithm does not handle such structures in a special
       * way.
       * @see yfiles.organic.OrganicLayout#starSubstructureStyle
       */
      NONE,
      /**
       * Substructure style specifier for stars that leads to a radial layout style for stars where the root is placed in the
       * middle.
       * @see yfiles.organic.OrganicLayout#starSubstructureStyle
       */
      RADIAL,
      /**
       * Substructure style specifier for stars that leads to a radial layout style for stars where the root is placed on the
       * outside (separated from the nodes with degree one).
       * @see yfiles.organic.OrganicLayout#starSubstructureStyle
       */
      SEPARATED_RADIAL,
      /**
       * Substructure style specifier for stars that leads to a circular layout style for stars where the root is placed in the
       * middle. In contrast to the radial layout style, all nodes (except the root) are placed on a single cycle.
       * @see yfiles.organic.OrganicLayout#starSubstructureStyle
       */
      CIRCULAR
    }
    export enum ParallelSubstructureStyle{
      /**
       * Substructure style specifier for parallel structures that indicates that the algorithm does not handle such structures
       * in a special way.
       * @see yfiles.organic.OrganicLayout#parallelSubstructureStyle
       */
      NONE,
      /**
       * Substructure style specifier for parallel structures that leads to a (rotated) rectangular layout style for the inner
       * nodes.
       * @see yfiles.organic.OrganicLayout#parallelSubstructureStyle
       */
      RECTANGULAR,
      /**
       * Substructure style specifier for parallel structures that leads to a radial layout style for the inner nodes.
       * @see yfiles.organic.OrganicLayout#parallelSubstructureStyle
       */
      RADIAL,
      /**
       * Substructure style specifier for parallel structures that places the inner nodes on a straight-line.
       * @see yfiles.organic.OrganicLayout#parallelSubstructureStyle
       */
      STRAIGHT_LINE
    }
    export enum CycleSubstructureStyle{
      /**
       * Substructure style specifier for cycle structures that indicates that the algorithm does not handle such structures in a
       * special way.
       * @see yfiles.organic.OrganicLayout#cycleSubstructureStyle
       */
      NONE,
      /**
       * Substructure style specifier for cycle structures that leads to a circular layout style.
       * @see yfiles.organic.OrganicLayout#cycleSubstructureStyle
       */
      CIRCULAR
    }
    export enum GroupNodeMode{
      /**
       * A mode constant for placing and resizing the group nodes and their content by the layout algorithm. The size of the
       * group is adjusted to the space that is needed by the placement of its contained nodes.
       * @see yfiles.organic.OrganicLayout#GROUP_NODE_MODE_DP_KEY
       */
      NORMAL,
      /**
       * A mode constant for placing the group nodes and their content with respect to the original bounds of the group node. The
       * bounds of the group nodes are treated as fixed and cannot be exceeded by the content of the group nodes.
       * @see yfiles.organic.OrganicLayout#GROUP_NODE_MODE_DP_KEY
       */
      FIX_BOUNDS,
      /**
       * A mode constant for placing group nodes with fixed content. Although these particular group nodes can move like all
       * other ordinary group nodes, their content remains fixed relative to the position of the group node.
       * @see yfiles.organic.OrganicLayout#GROUP_NODE_MODE_DP_KEY
       */
      FIX_CONTENTS
    }
    /**
     * This layout algorithm removes overlaps between nodes in a graph.
     * <p>
     * <em>Note:</em> The usage of {@link yfiles.organic.RemoveOverlapsStage} instead of this class is recommended for most use cases involving the
     * mere removal of overlaps. That stage offers a more powerful strategy to do the task.
     * </p>
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * The style of results often resembles the look of tiles which have been dropped onto each other, where tiles correspond
     * to nodes of the graph. The reason is that overlapping nodes are moved in order to resolve the overlap. During this
     * process, several nodes moving in the same direction may be stacked next to or above each other.
     * </p>
     * <p>
     * This algorithm does not route the edges of the input graph - edges might although be stretched due to the node
     * movement.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[88,86],"d":0},"n":[[10,161.5,100,157],[-14,262.5],[-44,292.5],[-74,322.5],[-104,352.5],[-134,382.5],[46,262.5],[76,292.5],[106,322.5],[136,352.5],[166,382.5],[-100,198.5,320,46],[37.5,139,45,45],[195,181.5,30,73.5],[-105,181.5,30,73.5],[537.5,56,100,157],[513.5,219],[483.5,311],[453.5,403],[423.5,495],[393.5,587],[607.5,219],[603.5,311],[633.5,403],[663.5,495],[693.5,587],[458.5,4,320,46],[565,-55.5,45,45],[784.5,-13.25,30,73.5],[422.5,-13.25,30,73.5]],"e":[],"vp":[-134.0,-56.0,949.0,729.0]}} Example with overlaps (left) and after executing the shuffle layout (right) {@graph {"ann":{"s":[65.2,39.6],"d":0},"n":[[360.9,272.2],[360.9,345.2],[575.9,369.2],[378.9,401.2],[494.4,369.2],[528.4,257.7],[538.4,313.2],[657.4,369.2],[619.9,313.2],[-2.6,293],[-2.6,365.8],[173.4,369.2],[15.4,391],[130.6,383],[165.2,278.6],[174.8,296.2],[203.8,383],[191.4,309.2]],"e":[[6,0],[6,3],[1,6],[1,0],[6,5],[8,2],[15,9],[15,12],[10,15],[10,9],[15,14],[17,11]],"vp":[-3.0,257.0,726.0,184.0]}} Example with overlaps (left) and after
     * executing the shuffle layout (right) <b>Concept</b> Nodes overlapping with other nodes will be moved in horizontal or vertical
     * direction in order to remove overlaps. The concept behind this removal step is based on a famous Russian arcade game. <b>Features</b>
     * </p>
     * <p>
     * This layout stage can also be {@link yfiles.organic.ShuffleLayout#applyLayout executed} on its own, without specifying a {@link yfiles.organic.ShuffleLayout#coreLayout core layout algorithm}.
     * </p>
     * <p>
     * A minimum distance between nodes can be specified such that not only overlaps will be removed but nodes will keep this
     * specified distance to other nodes. This distance can be defined separately for each node using a {@link yfiles.algorithms.IDataProvider}
     * registered with key {@link yfiles.organic.ShuffleLayout#MINIMUM_DISTANCE_DP_KEY}. The minimum distance can also be specified
     * globally via {@link yfiles.organic.ShuffleLayout#minimumNodeDistance}.
     * </p>
     * <p>
     * To specify that specific nodes should not be moved, they can be marked as fixed using a {@link yfiles.algorithms.IDataProvider}
     * registered with key {@link yfiles.organic.ShuffleLayout#FIXED_NODE_DP_KEY}.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface ShuffleLayout extends Object,yfiles.layout.ILayoutStage{}
    export class ShuffleLayout {
      /**
       * Creates a new instance of {@link yfiles.organic.ShuffleLayout} with default settings.
       */
      constructor();
      /**
       * Performs the overlap removal (shuffle) algorithm on the given graph, after the {@link yfiles.organic.ShuffleLayout#coreLayout core layout algorithm} was applied to it.
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Data provider key for specifying a minimum distance for each node
       * <p>
       * The default minimum distance specified by {@link yfiles.organic.ShuffleLayout#minimumNodeDistance} will be ignored for a node if the {@link yfiles.algorithms.IDataProvider} registered
       * with this key contains a valid minimum distance for that node.
       * </p>
       * <p>
       * Minimum distance values need to be greater than <code>0</code>.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static MINIMUM_DISTANCE_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data provider key for marking nodes as fixed
       * <p>
       * A node marked as fixed will not be moved by this algorithm but stay at its current position.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static FIXED_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * 
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the criterion for marking an overlap as horizontal.
       * <p>
       * This criterion influences how overlaps will be resolved. If an overlap is considered horizontal, it will <em>preferably</em> be solved by
       * moving nodes horizontally, else vertically.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given criterion is unknown
       * @type {yfiles.organic.HorizontalOverlapCriterion}
       */
      horizontalOverlapCriterion:yfiles.organic.HorizontalOverlapCriterion;
      /**
       * Gets or sets the default minimum distance that has to be obeyed between any two nodes.
       * <p>
       * This default distance will be considered for a node if the {@link yfiles.algorithms.IDataProvider} registered with the graph
       * with key {@link yfiles.organic.ShuffleLayout#MINIMUM_DISTANCE_DP_KEY} does not contain a valid, positive distance for that node.
       * If there is no {@link yfiles.algorithms.IDataProvider} registered with the mentioned key, then this default distance will be
       * applied to all nodes.
       * </p>
       * <p>
       * The minimum distance needs to be a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance is negative
       * @see yfiles.organic.ShuffleLayout#MINIMUM_DISTANCE_DP_KEY
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets whether or not the simple, fast layout mode of this algorithm is active.
       * <p>
       * Enabling this mode, the overlap removal step will be executed using a simpler and less sophisticated approach. All
       * overlaps will only be solved by moving nodes vertically. The algorithm will not try to figure out which direction might
       * be better for overlap removal.
       * </p>
       * <p>
       * The runtime will improve, but results may be of lower quality when using this mode.
       * </p>
       * @type {boolean}
       */
      simpleMode:boolean;
      /**
       * Gets or sets whether or not the barycenter mode is used for node shuffling when removing overlaps.
       * <p>
       * If this mode is active, the overlap removal step will be executed two times for each direction, once with the normal
       * node ordering and once with the reversed ordering. Finally, the barycenter between both results will be used for
       * assigning the node coordinates.
       * </p>
       * <p>
       * Activating the barycenter mode allows more symmetric results for some graphs. However, the runtime might increase.
       * </p>
       * @type {boolean}
       */
      barycenterMode:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A layout algorithm that arranges graphs in an organic fashion, using a force-directed drawing approach.
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * The organic layout style is characterized by a natural distribution of nodes. It is well suited to exhibit clusters and
     * symmetric properties of a graph. Nodes are placed in a space-saving manner, close to their adjacent nodes. Distances
     * between neighbors and edge lengths are highly uniform and edges are drawn as straight-line segments without bends.
     * </p>
     * <p>
     * Organic diagrams are well suited for visualizing relations in large networks, for example, in bioinformatics, enterprise
     * networking, social networks visualization, mesh visualization or system management.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0,"c":1},"n":[[595.38,758.73,90,90],[1117.49,809.3],[536.78,520.64],[1003.62,223.83,60,60],[144.24,1098.18,60,60],[821.57,320.27,60,60],[1021.74,95.55],[546.05,952.52],[339.03,1117.6],[619.72,286.05,60,60],[459.77,255.26],[819.8,180.32],[352.63,758.67],[602.52,1268.98],[418.51,994.58],[1203,194.02],[474.19,1194.71],[1369.52,207.86],[1229.77,66.55],[1130.53,480.76],[1026.12,377.23],[416.54,865.26],[1405.72,431.59],[742.7,1277.43],[576.4,127.48],[818.41,57.66],[609.01,0],[107.77,1267.45],[468.43,35.76],[168.05,858.48],[1479.05,113.81],[347.41,29.93],[363.19,151.7],[1317.66,551.96],[648.62,1164.37],[1213.89,683.56],[1463.15,308.66],[343.86,303.87],[1583.67,28.36],[221.25,1242],[551.17,1091.78],[652.84,482.26,90,90],[257.56,950.62],[1317.11,333.14],[1249.57,431.4],[1180.69,326.71],[1213.95,579.36],[987.92,896.99],[798.79,926.46,60,60],[766.38,1113.01,60,60],[1027.6,1013.7],[921.45,1059.25],[900.59,1191.83],[863.5,1315.51],[0,1086.46],[73.01,996.02],[17.49,1193.73],[1112.66,599.54],[1026.84,506.48]],"e":[[5,3,-2.34,-4.42,-2.34,-4.42],[5,3,-0,5.66,0,5.66],[6,3],[0,7],[8,4],[5,9,-0.84,4.93,-0.84,4.93],[5,9,0,-5.07,0,-5.07],[9,10],[5,11],[7,14],[14,8],[3,15],[8,16],[13,16],[15,17],[15,18],[5,20],[3,20],[20,19],[14,21],[0,21,1.55,4.23,1.55,4.23],[0,21,0,-4.79,0,-4.79],[21,12],[23,13],[9,24],[11,25],[24,26],[4,27],[24,28],[17,30],[28,31],[10,32],[32,31],[22,33],[34,13],[33,35],[35,1],[17,36],[36,22],[10,37],[30,38],[4,39],[7,40],[34,40],[40,16],[0,41,-4.9,-1.02,-4.9,-1.02],[0,41,15.32,0,15.32,0],[0,41,5.11,0,5.11,0],[0,41,-15.32,0,-15.32,0],[41,2],[41,9,-4.88,1.11,-4.88,1.11],[41,5],[41,9,5.13,0,5.13,0],[41,5,0,15.25,13.25,0],[41,5,0,-15.25,-13.25,0],[4,42],[21,42],[42,29],[17,43],[22,43],[33,44],[43,44],[44,19],[15,45],[43,45],[45,44],[45,20],[33,46],[46,19],[46,35],[47,1],[0,48],[0,48,0,-12.87,0,-12.87],[48,47],[48,0,0,12.87,0,12.87],[49,34],[49,23],[49,48,-4.93,-0.86,-4.93,-0.86],[49,48,5.07,0,5.07,0],[50,47],[50,51],[51,48],[51,52],[52,49],[52,53],[53,23],[54,4],[55,4],[4,56],[19,57],[57,46],[57,35],[20,58],[58,19],[58,57]],"vp":[0.0,0.0,1614.0,1346.0]}} Organic layout obtained using default settings {@graph {"ann":{"s":[30,30],"d":0,"c":1},"n":[[791.09,783.74],[562.01,0],[0,578.93],[654.52,397.12],[398.71,661.82],[289.49,301.35],[607.55,181.65],[486.39,286.91],[431.7,133.95],[585.61,83.22],[526.07,132.08],[499.31,59.31],[643.49,286.85],[577.45,334.93],[570.46,256.07],[384.63,282.87],[349.44,208.68],[428.89,217.67],[735.86,604.59],[609.97,735.23],[576.81,577.73],[708.83,495.34],[664.78,560.98],[627.87,490.62],[768.83,700.17],[706.56,764.55],[692.13,688.28],[499.4,712.48],[490.84,632.17],[563.39,666.04],[178.72,621.75],[126.84,449.31],[282.43,497.82],[287.3,654.44],[253.43,582.88],[333.1,587.05],[74.97,603.05],[57.42,516.13],[124.98,543.53],[198.43,365.41],[274.03,396.79],[209.79,443.73]],"e":[[1,9],[9,6],[6,10],[10,9],[10,8],[9,11],[8,11],[11,1],[11,10],[6,12],[12,3],[3,13],[13,12],[13,7],[12,14],[7,14],[14,6],[14,13],[7,15],[15,5],[5,16],[16,15],[16,8],[15,17],[8,17],[17,7],[17,16],[3,21],[21,18],[18,22],[22,21],[22,20],[21,23],[20,23],[23,3],[23,22],[18,24],[24,0],[0,25],[25,24],[25,19],[24,26],[19,26],[26,18],[26,25],[19,27],[27,4],[4,28],[28,27],[28,20],[27,29],[20,29],[29,19],[29,28],[4,33],[33,30],[30,34],[34,33],[34,32],[33,35],[32,35],[35,4],[35,34],[30,36],[36,2],[2,37],[37,36],[37,31],[36,38],[31,38],[38,30],[38,37],[31,39],[39,5],[5,40],[40,39],[40,32],[39,41],[32,41],[41,31],[41,40]],"vp":[0.0,0.0,822.0,814.0]}} Organic layout exposing symmetries of the graph structure<b>Concept</b>
     * </p>
     * <p>
     * This algorithm uses a force-directed approach to place the nodes of the input graph. The graph is modeled as a physical
     * system with forces acting on it. Nodes are considered as electrically charged particles with mutually repulsive forces.
     * Edges are modeled as springs that attract adjacent nodes. A good diagram is obtained from an equilibrium state of the
     * system, i.e., the nodes are rearranged based on the physical forces until the system reaches a (local) minimum of the
     * sum of the forces.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The {@link yfiles.organic.ClassicOrganicLayout#activateTreeBeautifier tree beautifier} feature activates a special layout style for subgraphs with a tree structure. These subgraphs will
     * be arranged using an algorithm dedicated to tree graphs and incorporated into the main organic diagram.
     * </p>
     * <p>
     * Several parameters for the internal force-directed drawing algorithm can be modified. For example, the {@link yfiles.organic.ClassicOrganicLayout#repulsion repulsion}, {@link yfiles.organic.ClassicOrganicLayout#attraction attraction}
     * and {@link yfiles.organic.ClassicOrganicLayout#initialTemperature initial temperature} can be controlled. This allows experts to configure and fine-tune the algorithm with respect to
     * their needs.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface ClassicOrganicLayout extends yfiles.layout.MultiStageLayout{}
    export class ClassicOrganicLayout {
      /**
       * Creates a new {@link yfiles.organic.ClassicOrganicLayout} with default settings.
       */
      constructor();
      /**
       * Frees internal resources which were in use during a previous algorithm run.
       */
      dispose():void;
      /**
       * Data provider key for marking the nodes that are part of the current scope
       * @see yfiles.organic.ClassicOrganicLayout#scope
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static AFFECTED_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for marking group nodes as fixed
       * <p>
       * The content of a fixed group node will not be changed by this algorithm but be treated as fixed.
       * </p>
       * <p>
       * The groups marked as fixed will be considered for the {@link yfiles.organic.ClassicOrganicLayout#groupNodePolicy group node policy} settings {@link yfiles.organic.GroupsPolicy#FIXED} and
       * {@link yfiles.organic.GroupsPolicy#LAYOUT}. When {@link yfiles.organic.GroupsPolicy#IGNORE} is chosen, the data provider registered with
       * this key will be ignored.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#groupNodePolicy
       * @see yfiles.organic.GroupsPolicy#FIXED
       * @see yfiles.organic.GroupsPolicy#LAYOUT
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static FIXED_GROUP_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for defining an individual preferred length for each edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static PREFERRED_EDGE_LENGTH_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Gets or sets the node repulsion value.
       * <p>
       * The repulsion influences the forces between nodes. A higher repulsion value can lead to more compact drawings, whereas a
       * lower repulsion value potentially enlarges drawings.
       * </p>
       * <p>
       * The repulsion is defined to range from <code>0</code> to <code>2</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the repulsion value is negative or greater than <code>2</code>
       * @type {number}
       */
      repulsion:number;
      /**
       * Gets or sets the edge attraction value.
       * <p>
       * The attraction influences the attractive force associated with edges. A higher attraction value can lead to more compact
       * drawings, whereas a lower value potentially enlarges drawings.
       * </p>
       * <p>
       * The attraction is defined to range from <code>0</code> to <code>2</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the attraction value is negative or greater than <code>2</code>
       * @type {number}
       */
      attraction:number;
      /**
       * Gets or sets the {@link yfiles.layout.IGroupBoundsCalculator} instance used for calculating the size of group nodes.
       * @type {yfiles.layout.IGroupBoundsCalculator}
       */
      groupBoundsCalculator:yfiles.layout.IGroupBoundsCalculator;
      /**
       * Gets or sets the compactness of group nodes.
       * <p>
       * The compactness ranges from <code>0</code> to <code>1</code> where <code>0</code> results in group nodes not affecting the overall layout too much while <code>1</code>
       * forces nodes in the same group to be clustered tightly.
       * </p>
       * <p>
       * The compactness values needs to lie within <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if compactness value does not lie within the interval <code>[0,1]</code>
       * @type {number}
       */
      groupNodeCompactness:number;
      /**
       * Gets or sets the policy for the handling of group nodes.
       * <p>
       * The group policy defines whether this algorithm {@link yfiles.organic.GroupsPolicy#IGNORE ignores} groups, leaves them {@link yfiles.organic.GroupsPolicy#FIXED fixed} or actively {@link yfiles.organic.GroupsPolicy#LAYOUT arranges} them.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given policy is unknown
       * @type {yfiles.organic.GroupsPolicy}
       */
      groupNodePolicy:yfiles.organic.GroupsPolicy;
      /**
       * Gets or sets the initial temperature for the force-directed layout process.
       * <p>
       * The algorithm starts with an initial temperature. During the iterations of the layout process, this temperature
       * decreases and if it reaches a certain limit, the process terminates. This limit can be controlled using {@link yfiles.organic.ClassicOrganicLayout#finalTemperature}.
       * </p>
       * <p>
       * The initial temperature will be multiplied by the {@link yfiles.organic.ClassicOrganicLayout#preferredEdgeLength preferred edge length} and then assigned as the actual initial heat of
       * nodes.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#finalTemperature
       * @type {number}
       */
      initialTemperature:number;
      /**
       * Gets or sets the absolute, final temperature value, which will cause the algorithm to stop if the average temperature
       * falls under this value.
       * <p>
       * The algorithm starts with an initial temperature. During the iterations of the layout process, this temperature
       * decreases.
       * </p>
       * @see yfiles.organic.ClassicOrganicLayout#initialTemperature
       * @type {number}
       */
      finalTemperature:number;
      /**
       * Gets or sets whether or not the deterministic mode of the layout algorithm is enabled.
       * <p>
       * In deterministic mode, the layout algorithm will yield the same results if the exact same input and same settings are
       * given as input.
       * </p>
       * @type {boolean}
       */
      deterministic:boolean;
      /**
       * Gets or sets whether or not to arrange subtrees of the input graph in a special, optimized tree style.
       * <p>
       * Subtrees will be arranged in a {@link yfiles.tree.BalloonLayout balloon-like fashion}.
       * </p>
       * <p>
       * If this feature is disabled, subgraphs with a tree structure are arranged using the same style as for the rest of the
       * graph.
       * </p>
       * <p>
       * In case that the input graph is a {@link yfiles.algorithms.Trees#isRootedTree tree} and this feature is enabled, the whole graph will be drawn in a tree-like style.
       * </p>
       * @type {boolean}
       */
      activateTreeBeautifier:boolean;
      /**
       * Gets or sets the gravity factor whose magnitude determines the strength of the force towards the barycenter of the
       * graph.
       * <ul>
       * <li>A high gravity factor value leads to layouts with nodes clustered around the barycenter of the graph.</li>
       * <li>A low gravity factor value stretches the outskirts of the graph far away from the barycenter.</li>
       * </ul>
       * @type {number}
       */
      gravityFactor:number;
      /**
       * Gets or sets the scope that determines which nodes are placed by this algorithm.
       * @throws {Stubs.Exceptions.ArgumentError} if the given scope is unknown
       * @see yfiles.organic.ClassicOrganicLayout#AFFECTED_NODES_DP_KEY
       * @type {yfiles.organic.Scope}
       */
      scope:yfiles.organic.Scope;
      /**
       * Gets or sets the strategy for the initial placement of nodes.
       * <p>
       * The initial placement has an influence on the final result, because this force-directed algorithm starts with some
       * positions and then uses forces between nodes to finally aim for a stable result drawing.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given initial placement mode is unknown
       * @type {yfiles.organic.InitialPlacement}
       */
      initialPlacement:yfiles.organic.InitialPlacement;
      /**
       * Gets or sets the maximum duration in milliseconds that this layout algorithm is allowed to run.
       * <p>
       * The duration needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified duration has a negative value
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the iteration factor which influences the maximum number of optimization iterations the algorithm may
       * perform.
       * @type {number}
       */
      iterationFactor:number;
      /**
       * Gets or sets the default preferred edge length.
       * <p>
       * The default preferred edge length will be used for edges that don't have an individual length preference defined via a {@link yfiles.algorithms.IDataProvider}
       * registered with key {@link yfiles.organic.ClassicOrganicLayout#PREFERRED_EDGE_LENGTH_DP_KEY}.
       * </p>
       * <p>
       * The preferred edge length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified edge length is negative
       * @type {number}
       */
      preferredEdgeLength:number;
      /**
       * Gets or sets whether or not to consider node sizes during layout calculation.
       * <p>
       * If this feature is disabled, overlaps between nodes (e.g. due to large node sizes) may occur.
       * </p>
       * @type {boolean}
       */
      considerNodeSizes:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.organic.InteractiveOrganicLayoutExecutionContext} provides control over the layout calculation in the case of
     * single-threaded algorithm execution.
     * @see yfiles.organic.InteractiveOrganicLayout#startLayout
     * @interface
     */
    export interface InteractiveOrganicLayoutExecutionContext extends Object{
      /**
       * Starts the layout calculation, continuing where it has stopped before and allowing a certain maximum duration in
       * milliseconds.
       * @param duration the duration in milliseconds the layout calculation is allowed to run
       * @abstract
       */
      continueLayout(duration:number):void;
      /**
       * Stops the layout calculation.
       * @abstract
       */
      stopLayout():void;
    }
    var InteractiveOrganicLayoutExecutionContext:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This algorithm arranges graphs in an organic fashion and offers special functionality to interact with the graph while
     * the algorithm is running.
     * <p>
     * Besides the organic graph arrangement, this algorithm enables to immediately visualize changes made to a graph. Changes
     * can be committed and the layout will be locally updated. That way, live interactions with an adapting graph layout are
     * possible. Another advantage is that it is not necessary to compute a completely new layout if only small changes were
     * made.
     * </p>
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * This algorithm supports the organic layout style which is characterized by a natural distribution of nodes. It is well
     * suited to exhibit clusters and symmetric properties of a graph. Nodes are placed in a space-saving manner and distances
     * between neighbors are highly uniform. Edges maintain uniform lengths too and are routed with straight-line segments
     * without bends.
     * </p>
     * <p>
     * Organic diagrams are well suited for visualizing relations in large networks, for example, in bioinformatics, enterprise
     * networking, social networks visualization, mesh visualization or system management.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[48,48],"d":0,"c":1},"n":[[-2901.1,82.94],[-2776.3,150.06],[-3044.96,-84.47],[-3100.49,54.95],[-3341.59,-232.28],[-3150.07,-159.49],[-2593.59,-25.02],[-2797.37,369.15],[-2792.54,582.9],[-2408.44,-110.43],[-3646.34,-1285.02],[-2406.36,-423.92],[-2508.48,535.38],[-3061.33,-181.39],[-3576.11,-153.25],[-3291.19,-446.62],[-3232.03,168.75],[-2991.88,23.83],[-2875.69,-19.77],[-3311.02,255.49],[-3211.58,273.66],[-3238.43,-589.82],[-3371.25,-336.26],[-3780.22,-96.13],[-2438.22,451.91],[-2348.8,552.52],[-2524.48,-582.24],[-2397.54,-234.83],[-3717.44,-1204.02],[-3539.82,-1255.44],[-2280.66,-209.75],[-2305.78,-69.98],[-2710.41,513.62],[-2747.74,704.16],[-2823.23,749.08],[-2354.53,12.96],[-2918.36,-219.22],[-2746.37,809.37],[-2657.24,754.28],[-3054.29,380.79],[-2235.49,-319.69],[-3521.75,-1120.87],[-3498.17,-1358.62],[-3729.27,-1087.89],[-2661.13,-725.91],[-2220.36,485.66],[-2577.32,481.65],[-2278.76,391.75],[-3911.01,-117.89],[-3889.39,-12.86],[-3449.65,-560.78],[-3236.89,-238.15],[-3271.16,-341.32],[-3062.01,-674.62],[-3372.21,-829.48],[-3343.54,161.81],[-3403.91,-648.81],[-2650.26,75.72],[-2655.19,191.31],[-2482.07,33.2],[-2545.18,138.92],[-3389.07,735.04],[-3233.93,-696.37],[-3571.23,-672.16],[-3451.64,-420.88],[-3550.73,-949.47],[-3331.67,-944.96],[-3058.81,-774.77],[-2903.06,-735.09],[-3508.46,-749.04],[-3962.39,59.51],[-4014.76,-33.26],[-4162.59,-51.88],[-2213.5,331.67],[-2143.12,434.04],[-2082.3,582.85],[-2563.73,-859.16],[-3872.43,-1134.67],[-2276.55,-427.33],[-3065.02,511.6],[-3216.65,13.18],[-3228.93,507],[-2666.53,647.27],[-2828.44,878.41],[-2888.55,-350.13],[-2803.46,-224.75],[-2274.13,227.36],[-2702.86,-255.45],[-2938.6,-442.63],[-2801.51,-416.97],[-2850.68,986.11],[-3315.58,637.75],[-3335,498.26],[-3322.98,-0.08],[-2999.45,592.68],[-3121.13,611.43],[-2191.35,-498.52],[-3933.14,-1230.72],[-3975.46,-1097.6],[-2618.48,-953.77],[-2458.11,-831.59],[-2490.11,-956.72],[-2049.18,482.82],[-1967.47,616.36],[-2076.61,696.43],[-4236.41,-130.92],[-4253.91,12.07],[-4056.39,65.42],[-2834.31,-815.64],[-2994.48,-872.19],[-3118.99,-869],[-3291.19,-1046.13],[-3606.59,-1027.04],[-3652.5,-907.63],[-3531.64,-338.57],[-3562.98,-455.05],[-3659.45,-608.85],[-3667.52,-730.4],[-3453.32,820.54],[-2454.91,197.69],[-2634.6,286.15]],"e":[[0,1],[0,2],[2,3],[2,4],[2,5],[1,6],[1,7],[7,8],[6,9],[5,13],[13,2],[4,14],[4,15],[3,16],[3,17],[17,18],[18,2],[16,19],[16,20],[20,19],[15,21],[22,4],[14,23],[12,24],[12,25],[11,26],[11,27],[10,28],[10,29],[9,30],[9,31],[8,32],[32,12],[8,33],[8,34],[34,33],[31,35],[27,35],[36,2],[34,37],[37,33],[33,38],[38,37],[32,39],[30,40],[29,41],[29,42],[28,43],[26,44],[25,45],[24,46],[46,12],[46,32],[24,47],[47,45],[23,48],[23,49],[22,50],[22,51],[51,5],[21,52],[21,53],[21,54],[19,55],[55,16],[50,56],[18,57],[57,58],[57,59],[59,35],[57,60],[56,62],[62,21],[56,63],[52,64],[50,64],[54,65],[43,65],[41,65],[54,66],[53,67],[62,67],[53,68],[44,68],[68,67],[50,69],[69,63],[69,65],[49,70],[49,71],[48,71],[70,71],[72,71],[47,73],[73,45],[47,74],[45,74],[45,75],[44,76],[43,77],[40,78],[78,11],[39,79],[39,80],[80,52],[39,81],[38,82],[82,33],[82,32],[37,83],[83,34],[36,84],[36,85],[35,86],[86,47],[86,73],[85,87],[84,88],[84,89],[83,90],[81,91],[61,91],[81,92],[80,93],[79,94],[79,95],[78,96],[77,97],[77,98],[76,99],[76,100],[76,101],[75,102],[74,102],[75,103],[75,104],[72,105],[72,106],[71,107],[70,107],[68,108],[67,109],[67,110],[66,111],[65,112],[65,113],[64,114],[64,115],[63,116],[63,117],[61,118],[60,119],[58,120]],"vp":[-4254.0,-1359.0,2335.0,2394.0]}} Organic layout obtained using this algorithm<b>Concept</b>
     * </p>
     * <p>
     * The internal basis for computing actual layouts is a force-directed approach placing the nodes of the input graph. The
     * graph is modeled as a physical system with appropriate forces acting on it. Nodes are considered as electrically charged
     * particles with mutually repulsive forces. Edges are modeled as forces that attract adjacent nodes. A good diagram is
     * obtained from an equilibrium state of the system, i.e., the nodes are rearranged based on the physical forces until the
     * system reaches a (local) minimum of the sum of the forces.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * Several update methods allow to indicate <em>lightweight</em> changes on the graph, for example, {@link yfiles.organic.InteractiveOrganicLayout#setCenter}
     * for node location updates. These updates will be scheduled and executed at a specific point within the life-cycle of the
     * layout algorithm. They are ideally suited for usage in interactive scenarios, where users, for example, change node
     * positions via mouse-dragging.
     * </p>
     * <p>
     * Method {@link yfiles.organic.InteractiveOrganicLayout#addStructureUpdate} allows to schedule a custom event handler instance
     * that is executed in a synchronized context and can safely update the structure of the graph.
     * </p>
     * <p>
     * <b>Usage Hints</b>
     * </p>
     * <p>
     * It is easiest to start the algorithm using {@link yfiles.organic.InteractiveOrganicLayout#startLayout}. Furthermore, it is
     * strongly recommended to start it by passing a copy of the original graph (i.e. a {@link yfiles.layout.CopiedLayoutGraph}
     * instance) as parameter to the mentioned method.
     * </p>
     * <p>
     * Importantly, <b>changes will not be automatically applied</b> to the input graph. Updates can be scheduled via various methods (e.g.
     * {@link yfiles.organic.InteractiveOrganicLayout#setCenter}). The whole layout will then internally be adjusted accordingly and
     * changes are stored as intermediate results. Such intermediate results can be {@link yfiles.organic.InteractiveOrganicLayout#commitPositions committed} in order to apply them to the
     * actual graph.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutAlgorithm}
     */
    export interface InteractiveOrganicLayout extends Object,yfiles.layout.ILayoutAlgorithm{}
    export class InteractiveOrganicLayout {
      /**
       * Creates a new instance of the {@link yfiles.organic.InteractiveOrganicLayout} with default settings.
       */
      constructor();
      /**
       * Schedules an update for the structure of the graph, which will automatically be executed at a later point in the
       * life-cycle of this algorithm.
       * <p>
       * The given event handler will be queued and executed at a specific time. The event handler can make structural changes
       * (e.g. removal/creation of edges or nodes). They will be synchronized with the rest of the layout algorithm.
       * </p>
       * @param handler The handler delegate that will be invoked using <code>null</code> as the sender and <code>args</code> as the event arguments
       * @param evt The event argument that will be piped to the <code>handler</code> invocation.
       */
      addStructureUpdate(handler:(sender:Object,evt:yfiles.lang.EventArgs)=>void,evt:yfiles.lang.EventArgs):void;
      /**
       * Calculates an organic layout for the given input graph, however, the layout is <b>not automatically applied</b> to the graph.
       * <p>
       * Changes have to be {@link yfiles.organic.InteractiveOrganicLayout#commitPositions committed} to update the graph with the actual calculated positions.
       * </p>
       * @param graph the input graph
       * @see yfiles.organic.InteractiveOrganicLayout#startLayout
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Writes calculated intermediate locations of nodes and edges to the actual graph.
       * <p>
       * Update methods like {@link yfiles.organic.InteractiveOrganicLayout#setCenter} schedule changes which cause that the whole layout
       * will internally be adjusted. All adjustments are stored as intermediate results. This method immediately transfers all
       * these intermediate results to the actual input graph.
       * </p>
       * @see yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly
       */
      commitPositions():void;
      /**
       * Writes calculated intermediate locations of nodes and edges to the actual graph and returns the largest movement value.
       * <p>
       * Update methods like {@link yfiles.organic.InteractiveOrganicLayout#setCenter} schedule changes which cause that the whole layout
       * will internally be adjusted. All adjustments are stored as intermediate results. This method smoothly transfers all
       * these intermediate results to the actual input graph.
       * </p>
       * <p>
       * Positions are, however, <b>not transferred directly</b> (use {@link yfiles.organic.InteractiveOrganicLayout#commitPositions} if that is intended). Instead, the
       * nodes are moved towards the calculated position. The movement is restricted to the given maximum distance.
       * </p>
       * <p>
       * The movement will be calculated as <em>(movement) = (factor) * (distance between calculated and actual location)</em>
       * </p>
       * <p>
       * The returned largest movement can be used for estimating the difference between calculated layout and actual positions.
       * If the return value is <code>0</code>, the calculated layout was completely transferred.
       * </p>
       * @param maxMovement the maximum distance a node will be moved
       * @param factor a factor that determines the node movement
       * @returns the largest movement value or <code>0</code>, if the calculated layout has been transferred completely
       * @see yfiles.organic.InteractiveOrganicLayout#commitPositions
       */
      commitPositionsSmoothly(maxMovement:number,factor:number):number;
      /**
       * Disables all predefined {@link yfiles.layout.ILayoutStage}s so that upon {@link yfiles.organic.InteractiveOrganicLayout#applyLayout}
       * only the internal organic layout algorithm will be executed.
       * <p>
       * This method is called upon construction of this class so that by default additional
       * {@link yfiles.layout.ILayoutStage}s are deactivated. This method may be overridden if the additional stages should stay active
       * (e.g. override the method and return silently).
       * </p>
       * @see yfiles.layout.MultiStageLayout#disableAllStages
       */
      disableAllStages():void;
      /**
       * Polls the current coordinates of the center of the given node.
       * <p>
       * The returned coordinates do not necessarily correspond to the actual location of the node in the input graph. They may
       * only be intermediate results stored in the algorithm. This will be the case if scheduled updates were not yet completely
       * {@link yfiles.organic.InteractiveOrganicLayout#commitPositions committed} to the actual graph.
       * </p>
       * @param node the node for which the center should be polled
       * @returns the {@link } representing the center location of the given node, or <code>null</code> if nothing about the node is
       * known
       * @see yfiles.organic.InteractiveOrganicLayout#setCenter
       */
      getCenter(node:yfiles.algorithms.Node):yfiles.algorithms.YPoint;
      /**
       * Polls the current x-coordinate of the center location of the given node.
       * <p>
       * The returned coordinate is not necessarily the actual x-coordinate of the node in the input graph but only an
       * intermediate result stored in the algorithm. This will be the case if scheduled updates were not yet completely {@link yfiles.organic.InteractiveOrganicLayout#commitPositions committed}
       * to the actual graph.
       * </p>
       * @param node the node for which the x-coordinate should be polled
       * @returns the x-coordinate of the center location of the given node
       * @see yfiles.organic.InteractiveOrganicLayout#setCenterX
       */
      getCenterX(node:yfiles.algorithms.Node):number;
      /**
       * Polls the current y-coordinate of the center location of the given node.
       * <p>
       * The returned coordinate is not necessarily the actual y-coordinate of the node in the input graph but only an
       * intermediate result stored in the algorithm. This will be the case if scheduled updates were not yet completely {@link yfiles.organic.InteractiveOrganicLayout#commitPositions committed}
       * to the actual graph.
       * </p>
       * @param node the node for which the y-coordinate should be polled
       * @returns the y-coordinate of the center location of the given node
       * @see yfiles.organic.InteractiveOrganicLayout#setCenterY
       */
      getCenterY(node:yfiles.algorithms.Node):number;
      /**
       * Polls the current stress value of a given node.
       * <p>
       * The stress value indicates how far a node will possibly move. The higher the stress of a node is, the farther it may
       * move.
       * </p>
       * <p>
       * The stress value is defined to be a value from the interval <code>[0,1]</code>.
       * </p>
       * @param node the node for which the stress value should be polled
       * @returns the stress value of the given node
       * @see yfiles.organic.InteractiveOrganicLayout#setStress
       */
      getStress(node:yfiles.algorithms.Node):number;
      /**
       * Schedules an update for the center location of the given node.
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * @param node the node that should be updated
       * @param x the desired x-coordinate of the given node
       * @param y the desired y-coordinate of the given node
       */
      setCenter(node:yfiles.algorithms.Node,x:number,y:number):void;
      /**
       * Schedules an update for the center location's x-coordinate of the given node.
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * @param node the node that should be updated
       * @param x the desired x-coordinate of the given node
       * @see yfiles.organic.InteractiveOrganicLayout#setCenterY
       * @see yfiles.organic.InteractiveOrganicLayout#setCenter
       */
      setCenterX(node:yfiles.algorithms.Node,x:number):void;
      /**
       * Schedules an update for the center location's y-coordinate of the given node.
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * @param node the node that should be updated
       * @param y the desired y-coordinate of the given node
       * @see yfiles.organic.InteractiveOrganicLayout#setCenterX
       * @see yfiles.organic.InteractiveOrganicLayout#setCenter
       */
      setCenterY(node:yfiles.algorithms.Node,y:number):void;
      /**
       * Schedules an update for the inertia of the given node.
       * <p>
       * The inertia is defined to be a value from the interval <code>[0,1]</code>.
       * </p>
       * <ul>
       * <li><code>1.0</code>: The node will not move.</li>
       * <li><code>0.5</code>: The node will only move half as far as it would with an inertia of <code>0.0</code>.</li>
       * <li><code>0.0</code>: The node will move as fast as possible.</li>
       * </ul>
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * @param node the node whose inertia to set
       * @param inertia an inertia value between <code>0</code> and <code>1</code>
       * @throws {Stubs.Exceptions.ArgumentError} if the given inertia value is negative or greater than <code>1</code>
       */
      setInertia(node:yfiles.algorithms.Node,inertia:number):void;
      /**
       * Schedules an update for the preferred length of the given edge.
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * @param edge the edge whose preferred length should be updated
       * @param newEdgeLength the new preferred edge length
       */
      setPreferredEdgeLength(edge:yfiles.algorithms.Edge,newEdgeLength:number):void;
      /**
       * Schedules an update for the radius of the given node.
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * @param node the node whose radius should be updated
       * @param radius the desired radius for the given node
       */
      setRadius(node:yfiles.algorithms.Node,radius:number):void;
      /**
       * Schedules an update for the stress value of the given node.
       * <p>
       * The stress value indicates how far a node will possibly move. The higher the stress of a node is, the farther it may
       * move.
       * </p>
       * <p>
       * This method can be used while layout calculation is in progress (e.g. for interactive layout scenarios). However, the
       * change will <b>not directly be applied</b> to the graph itself but only stored internally as an intermediate result. Scheduled updates can be
       * committed to the graph while the algorithm is running using methods {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * or {@link yfiles.organic.InteractiveOrganicLayout#commitPositionsSmoothly}.
       * </p>
       * <p>
       * The stress value is defined to be a value from the interval <code>[0,1]</code>.
       * </p>
       * @param node the node whose stress value should be updated
       * @param stress a stress value from the interval <code>[0,1]</code>
       * @throws {Stubs.Exceptions.ArgumentError} if the given stress value is negative or greater than <code>1</code>
       */
      setStress(node:yfiles.algorithms.Node,stress:number):void;
      /**
       * Creates a {@link yfiles.organic.InteractiveOrganicLayoutExecutionContext context object} that provides methods to continue and {@link yfiles.organic.InteractiveOrganicLayoutExecutionContext#stopLayout stop} the layout calculation for running this layout
       * algorithm in a single-threaded environment.
       * <p>
       * <em>Usage:</em> Call doLayout(long) on the created instance to run the actual layout calculation for some specified period of time,
       * whenever the layout should be recalculated. To actually transfer the changes, {@link yfiles.organic.InteractiveOrganicLayout#commitPositions}
       * should be called subsequently.
       * </p>
       * @param graph the input graph
       * @returns a {@link  context instance} to control layout calculation
       * @see yfiles.organic.InteractiveOrganicLayoutExecutionContext
       * @see yfiles.organic.InteractiveOrganicLayout#applyLayout
       */
      startLayout(graph:yfiles.layout.LayoutGraph):yfiles.organic.InteractiveOrganicLayoutExecutionContext;
      /**
       * Stops the layout algorithm.
       * <p>
       * In contrast to
       * {@link yfiles.organic.InteractiveOrganicLayout#stopAndWait}, the algorithm terminates immediately and will not wait until the
       * ongoing layout calculation is finished.
       * </p>
       * @see yfiles.organic.InteractiveOrganicLayout#stopped
       * @see yfiles.organic.InteractiveOrganicLayout#stopAndWait
       */
      stop():void;
      /**
       * Stops a previously {@link yfiles.organic.InteractiveOrganicLayout#startLayout started} algorithm and then blocks until the current layout calculation is completed.
       * @see yfiles.organic.InteractiveOrganicLayout#stopped
       * @see yfiles.organic.InteractiveOrganicLayout#stop
       */
      stopAndWait():void;
      /**
       * Synchronizes the structure of the graph copy with the original graph.
       * @throws {Stubs.Exceptions.InvalidOperationError} if the currently handled graph is not of type {@link yfiles.layout.CopiedLayoutGraph}
       */
      syncStructure():void;
      /**
       * Wakes up the algorithm with the effect that it will restart/continue the layout calculation.
       * <p>
       * This method is useful if the layouter is {@link yfiles.organic.InteractiveOrganicLayout#sleeping sleeping} but should be notified of changes (e.g. due to user interaction).
       * </p>
       * @see yfiles.organic.InteractiveOrganicLayout#sleeping
       */
      wakeUp():void;
      /**
       * Gets the time when the last wake-up, that is, call to {@link yfiles.organic.InteractiveOrganicLayout#wakeUp}, occurred.
       * <p>
       * The time is defined in terms of the difference between the current time and midnight, January 1, 1970 UTC, measured in
       * milliseconds.
       * </p>
       * @type {number}
       */
      lastWakeupTime:number;
      /**
       * Gets whether or not this layout algorithm has stopped.
       * <p>
       * If the algorithm has stopped, it terminated all its layout calculations. It is not {@link yfiles.organic.InteractiveOrganicLayout#running running} anymore and can not be
       * restarted/continued by calling {@link yfiles.organic.InteractiveOrganicLayout#wakeUp}.
       * </p>
       * @type {boolean}
       */
      stopped:boolean;
      /**
       * Gets whether or not this layout algorithm is currently sleeping.
       * <p>
       * Sleeping indicates that the algorithm has not {@link yfiles.organic.InteractiveOrganicLayout#stopped stopped} yet but is waiting (i.e. doing nothing). It can be notified to
       * continue its work using {@link yfiles.organic.InteractiveOrganicLayout#wakeUp}.
       * </p>
       * @see yfiles.organic.InteractiveOrganicLayout#wakeUp
       * @type {boolean}
       */
      sleeping:boolean;
      /**
       * Gets whether or not this layout algorithm is currently running.
       * <p>
       * The algorithm is running if the layout process is still active, the algorithm has not {@link yfiles.organic.InteractiveOrganicLayout#stopped stopped} yet and is not {@link yfiles.organic.InteractiveOrganicLayout#sleeping sleeping}.
       * </p>
       * @type {boolean}
       */
      running:boolean;
      /**
       * Gets or sets the working ratio which defines the amount of processor time this algorithm tries to get.
       * <p>
       * A working ratio value of <code>1</code> means that the algorithm will try to run as fast as possible. Lower values will lead to small
       * breaks after each internal round.
       * </p>
       * <p>
       * The ratio needs to be a value in <code>(0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given ratio is not in <code>(0,1]</code>.
       * @type {number}
       */
      workingRatio:number;
      /**
       * Gets or sets the default preferred edge length.
       * <p>
       * This length does not define the actual absolute length of edges, but the layout algorithm considers the specified
       * preference where possible.
       * </p>
       * <p>
       * The preferred edge length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified edge length is negative
       * @see yfiles.organic.InteractiveOrganicLayout#setPreferredEdgeLength
       * @type {number}
       */
      preferredEdgeLength:number;
      /**
       * Gets or sets the preferred distance between nodes.
       * <p>
       * The minimum node distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified minimum node distance is negative
       * @type {number}
       */
      preferredNodeDistance:number;
      /**
       * Gets or sets the maximum duration in milliseconds that this algorithm is allowed to run.
       * <p>
       * The duration needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified maximum duration has a negative value
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the ratio of layout quality versus running time.
       * <p>
       * The larger the ratio, the better the quality of the resulting layout but the longer it may take to perform the layout
       * calculation.
       * </p>
       * <p>
       * The value needs to lie within <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified ratio is outside the interval <code>[0,1]</code>
       * @type {number}
       */
      qualityTimeRatio:number;
      /**
       * Gets or sets an {@link yfiles.organic.OutputRestriction} which restricts the area for the layout result of this algorithm.
       * @see yfiles.organic.OutputRestriction
       * @type {yfiles.organic.OutputRestriction}
       */
      outputRestriction:yfiles.organic.OutputRestriction;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm arranges graphs in an organic fashion.
     * <p>
     * <b>Layout Style</b> The organic layout style is characterized by a natural distribution of nodes that exhibits clusters and symmetric
     * properties of the graph. Nodes are placed space-saving, close to their adjacent nodes. Edges maintain uniform lengths
     * and are routed with straight-line segments without bends.
     * </p>
     * <p>
     * Organic diagrams are commonly used for visualizing relations in large networks for example in bioinformatics, enterprise
     * networking, visualizing social networks, mesh visualization or system management.
     * </p>
     * <p>
     * {@graph {"ann": {"s": [30,30], "d": 1},"n": [[623,552], [719,500], [460,606], [461,312], [650,830], [467,417], [399,245], [710,595], [712,756], [423,492], [336,473], [387,373], [545,645], [883,702], [688,675], [507,226], [796,722], [667,62], [572,158], [470,158], [594,354], [520,352], [615,651], [687,222], [731,111], [950,654], [361,553], [309,325], [365,631], [722,895], [282,570], [496,729], [554,76], [211,540], [308,103], [256,489], [703,312], [850,621], [720,405], [333,181], [658,141], [281,421], [159,261], [791,555], [541,0], [574,864], [777,646], [513,518], [590,745], [611,221], [630,289], [557,278], [663,366], [237,274], [809,470], [874,520], [923,583], [889,446], [958,478], [999,550], [1020,624], [722,835], [626,922], [678,895], [251,867], [406,695], [235,779], [63,939], [130,979], [238,810], [355,820], [166,734], [147,828], [317,756], [94,705], [194,852], [127,898], [195,927], [68,863], [761,971], [693,970], [518,920], [637,1005], [714,1036], [255,203], [330,260], [976,402], [1039,447], [1078,551], [1105,411], [893,782], [958,741], [826,393], [0,980], [734,1109]], "e": [[0,1], [5,3], [6,3], [0,7], [8,4], [5,9,3.45,2.02,3.45,2.02], [5,9,-4.64,0,-4.64,0], [9,10], [5,11], [7,14], [14,8], [3,15], [8,16], [13,16], [15,18], [15,19], [5,21], [3,21], [21,20], [14,22], [0,22], [22,12], [25,13], [9,26], [11,27], [26,28], [4,29], [26,30], [18,32], [30,33], [10,35], [35,33], [23,36], [37,13], [36,38], [38,1], [6,39], [39,34], [18,40], [40,17], [40,24], [40,23], [10,41], [37,43], [43,1], [43,7], [32,44], [4,45], [7,46], [37,46], [46,16], [0,47], [0,47,0,-7.33,0,-7.33], [0,47,0,7.33,0,7.33], [47,2], [47,9,-1.25,4.32,-1.25,4.32], [47,5,-4.1,1.87,-4.1,1.87], [47,9,0,-4.68,0,-4.68], [47,5,4.94,0,4.94,0], [4,48], [22,48], [48,31], [18,49], [23,49], [36,50], [49,50], [50,20], [15,51], [49,51], [51,50], [51,21], [36,52], [52,20], [42,53], [53,27], [54,1], [55,43], [55,54], [56,37], [56,55], [56,25], [57,54], [57,58], [58,55], [58,59], [59,56], [59,60], [60,25], [61,4], [62,4], [4,63], [63,61], [2,65], [31,65], [28,65], [64,66], [68,67], [66,71], [69,72], [66,73], [73,65], [73,70], [73,69], [71,74], [66,75], [69,75], [72,76], [75,76], [76,67], [64,77], [75,77], [77,76], [77,68], [72,78], [78,67], [29,79], [63,80], [45,81], [62,82], [79,83], [80,83], [83,82], [39,84], [84,53], [85,6], [85,27], [58,86], [58,87], [59,88], [87,89], [13,90], [13,91], [54,92], [67,93], [83,94]], "vp": [0.0,0.0,1135.0,1139.0]}} Organic Layout obtained with default settings {@graph {"ann": {"s": [30,30], "d": 1},"n": [[563.5,1043.5], [1196.5,36.5], [-0.5,15.5], [871.5,537.5], [598.5,41.5], [315.5,513.5], [717.5,813.5], [590.5,599.5], [439.5,797.5], [635.5,935.5], [576.5,835.5], [502.5,928.5], [597.5,992.5], [568.5,943.5], [532.5,989.5], [682.5,878.5], [649.5,826.5], [622.5,877.5], [507.5,821.5], [464.5,867.5], [525.5,873.5], [809.5,683.5], [737.5,582.5], [688.5,691.5], [773.5,756.5], [750.5,704.5], [714.5,751.5], [846.5,612.5], [807.5,565.5], [788.5,622.5], [663.5,586.5], [641.5,642.5], [699.5,631.5], [446.5,567.5], [364.5,660.5], [482.5,680.5], [520.5,577.5], [478.5,619.5], [535.5,637.5], [378.5,544.5], [334.5,587.5], [391.5,602.5], [391.5,735.5], [451.5,736.5], [419.5,685.5], [1050.5,277.5], [915.5,36.5], [799.5,257.5], [974.5,416.5], [926.5,304.5], [851.5,397.5], [926.5,481.5], [910.5,424.5], [867.5,467.5], [1022.5,351.5], [991.5,300.5], [965.5,354.5], [861.5,283.5], [820.5,328.5], [877.5,341.5], [1127.5,150.5], [1065.5,34.5], [1011.5,140.5], [1095.5,217.5], [1070.5,162.5], [1035.5,209.5], [1165.5,90.5], [1134.5,34.5], [1107.5,87.5], [990.5,29.5], [966.5,84.5], [1024.5,79.5], [756.5,25.5], [706.5,139.5], [823.5,125.5], [837.5,19.5], [804.5,65.5], [863.5,73.5], [676.5,29.5], [655.5,86.5], [713.5,74.5], [748.5,202.5], [807.5,191.5], [766.5,147.5], [282.5,15.5], [142.5,256.5], [392.5,237.5], [442.5,8.5], [368.5,105.5], [486.5,125.5], [522.5,18.5], [481.5,59.5], [538.5,77.5], [360.5,-0.5], [332.5,51.5], [392.5,46.5], [383.5,171.5], [442.5,185.5], [425.5,128.5], [132.5,15.5], [64.5,127.5], [184.5,121.5], [207.5,9.5], [173.5,58.5], [231.5,64.5], [63.5,14.5], [28.5,68.5], [86.5,65.5], [97.5,194.5], [158.5,188.5], [122.5,141.5], [215.5,393.5], [339.5,373.5], [265.5,282.5], [168.5,329.5], [225.5,332.5], [200.5,278.5], [261.5,457.5], [321.5,442.5], [279.5,401.5], [371.5,307.5], [330.5,261.5], [313.5,318.5]], "e": [[0,12], [12,9], [9,13], [13,12], [13,11], [12,14], [11,14], [14,0], [14,13], [9,15], [15,6], [6,16], [16,15], [16,10], [15,17], [10,17], [17,9], [17,16], [10,18], [18,8], [8,19], [19,18], [19,11], [18,20], [11,20], [20,10], [20,19], [6,24], [24,21], [21,25], [25,24], [25,23], [24,26], [23,26], [26,6], [26,25], [21,27], [27,3], [3,28], [28,27], [28,22], [27,29], [22,29], [29,21], [29,28], [22,30], [30,7], [7,31], [31,30], [31,23], [30,32], [23,32], [32,22], [32,31], [7,36], [36,33], [33,37], [37,36], [37,35], [36,38], [35,38], [38,7], [38,37], [33,39], [39,5], [5,40], [40,39], [40,34], [39,41], [34,41], [41,33], [41,40], [34,42], [42,8], [8,43], [43,42], [43,35], [42,44], [35,44], [44,34], [44,43], [3,51], [51,48], [48,52], [52,51], [52,50], [51,53], [50,53], [53,3], [53,52], [48,54], [54,45], [45,55], [55,54], [55,49], [54,56], [49,56], [56,48], [56,55], [49,57], [57,47], [47,58], [58,57], [58,50], [57,59], [50,59], [59,49], [59,58], [45,63], [63,60], [60,64], [64,63], [64,62], [63,65], [62,65], [65,45], [65,64], [60,66], [66,1], [1,67], [67,66], [67,61], [66,68], [61,68], [68,60], [68,67], [61,69], [69,46], [46,70], [70,69], [70,62], [69,71], [62,71], [71,61], [71,70], [46,75], [75,72], [72,76], [76,75], [76,74], [75,77], [74,77], [77,46], [77,76], [72,78], [78,4], [4,79], [79,78], [79,73], [78,80], [73,80], [80,72], [80,79], [73,81], [81,47], [47,82], [82,81], [82,74], [81,83], [74,83], [83,73], [83,82], [4,90], [90,87], [87,91], [91,90], [91,89], [90,92], [89,92], [92,4], [92,91], [87,93], [93,84], [84,94], [94,93], [94,88], [93,95], [88,95], [95,87], [95,94], [88,96], [96,86], [86,97], [97,96], [97,89], [96,98], [89,98], [98,88], [98,97], [84,102], [102,99], [99,103], [103,102], [103,101], [102,104], [101,104], [104,84], [104,103], [99,105], [105,2], [2,106], [106,105], [106,100], [105,107], [100,107], [107,99], [107,106], [100,108], [108,85], [85,109], [109,108], [109,101], [108,110], [101,110], [110,100], [110,109], [85,114], [114,111], [111,115], [115,114], [115,113], [114,116], [113,116], [116,85], [116,115], [111,117], [117,5], [5,118], [118,117], [118,112], [117,119], [112,119], [119,111], [119,118], [112,120], [120,86], [86,121], [121,120], [121,113], [120,122], [113,122], [122,112], [122,121]], "vp": [-1.0,-1.0,1228.0,1075.0]}} Organic Layout exhibiting symmetric properties<b>Concept</b>{@link yfiles.organic.OrganicLayout}
     * uses a force-directed approach to place the nodes of the input graph. According to this approach, the graph is modeled
     * as a physical system with appropriate forces acting on it. Nodes are considered as electrically charged particles with
     * mutually repulsive forces. Edges are modeled as springs that attract adjacent nodes. A good diagram is obtained from an
     * equilibrium state of the system, i.e., the nodes are rearranged based on the physical forces until the system reaches a
     * (local) minimum of the sum of the forces. <b>Features</b>
     * </p>
     * <p>
     * The ratio between the layout quality and the running time is conveniently adjustable.
     * </p>
     * <p>
     * This layout algorithm is able to consider a {@link yfiles.layout.PartitionGrid} structure. However, for common nodes (i.e.
     * non-group nodes) it only considers single partition cells. Furthermore, the layout algorithm will throw a {@link yfiles.lang.Exception}
     * if there is a partition grid and the descendants of a group node are assigned to different partition grid cells or if
     * there are group nodes that are associated with a group node mode other than {@link yfiles.organic.GroupNodeMode#NORMAL}.
     * </p>
     * <p>
     * Although overlaps between edges and nodes are avoided, this layout algorithm doesn't guarantee that they won't appear in
     * the resulting layout. To avoid overlaps, it is possible to route the edges afterwards using an edge routing algorithm,
     * e.g., {@link yfiles.router.OrganicEdgeRouter}.
     * </p>
     * <p>
     * The algorithm is also able to detect certain types of substructures in a graph (i.e., {@link yfiles.organic.OrganicLayout#chainSubstructureStyle chains}, {@link yfiles.organic.OrganicLayout#starSubstructureStyle stars}, {@link yfiles.organic.OrganicLayout#cycleSubstructureStyle cycles} and
     * {@link yfiles.organic.OrganicLayout#parallelSubstructureStyle parallel structures}) and arrange these structures with special-purpose layout styles. Using substructure styles ensures
     * that the corrsponding structures are easily recognized in the graph.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[50,50],"d":0, "c":1},"n":[[651.02,746.2],[499.25,376.34],[757.43,840.9],[574.93,802.49],[772.23,780.83],[712.08,729.69],[591.5,342.9],[630.22,404.23],[511.3,574.88],[356.15,353.31],[488.74,447.78],[566.25,739.99],[832.31,791.39],[152.43,610.4],[689.57,879.35],[248.1,596.17],[120.25,697.57],[639.18,467.72],[427.77,458.88],[408.48,519.6],[691.9,669.45],[818.01,882.83],[539.8,660.71],[585.22,875.17],[636.28,814.7],[714.64,608.38],[656.9,343.12],[511.04,217.28],[470.75,277.59],[422.59,338.16],[565.7,413.55],[531.15,295.7],[348.61,180.63],[402.12,271.28],[284.34,91.79],[321.78,0],[381.55,66.78],[214.95,158.82],[187.52,306.65],[287.7,278.04],[92.76,309.88],[631.56,613.97],[513.18,814.11],[775.66,700.72],[601.82,679.52],[504.9,731.91],[697.17,813.09],[606.93,266.97],[450.55,583.13],[728.3,327.65],[717.42,253.29],[543.5,154.51],[614.76,150.96],[500.57,513.16],[360.14,445.09],[293.67,694],[348.53,611.5],[206.29,734.57],[571.37,553.51],[0,313.72],[233.88,6.24],[182.4,76.67],[420.47,398.21],[561.25,487.65],[680.21,183.49]],"e":[[0,2],[0,5],[0,3],[0,4],[1,7],[1,6],[1,8],[1,10],[1,9],[0,12],[0,14],[0,11],[15,13],[16,13],[1,17],[1,19],[1,18],[0,24],[0,22],[0,20],[0,23],[0,21],[1,58],[0,25],[0,58],[1,29],[1,31],[1,26],[1,27],[1,28],[1,30],[32,34],[1,33],[32,33],[36,34],[35,34],[37,34],[38,40],[38,39],[0,44],[0,41],[0,46],[0,43],[0,45],[0,42],[1,47],[48,8],[48,58],[48,10],[48,18],[48,19],[50,47],[51,47],[52,47],[49,47],[48,53],[48,54],[1,54],[1,53],[55,56],[56,15],[16,57],[57,55],[56,48],[39,33],[59,40],[60,34],[61,34],[1,62],[48,62],[1,63],[48,63],[64,47]],"vp":[0.0,0.0,883.0,933.0]}} Organic Layout without applying specific layout styles to substructures {@graph {"ann":{"s":[50,50],"d":0,"c":1},"n":[[222.41,751.98],[933.19,715.06],[333.12,751.98],[167.05,656.1],[111.7,751.98],[277.76,656.1],[1075.74,818.63],[987.64,882.64],[826.53,1248.81],[1109.39,715.06],[923.05,1325.6],[250.02,979.37],[167.05,847.86],[283.81,1327.29],[277.76,847.86],[357.8,1251.78],[312.21,1429.13],[1075.74,611.5],[946.75,1221.23],[879.92,1137.62],[451.47,751.98],[352.53,563.47],[425.23,858.43],[425.23,645.53],[352.53,940.5],[250.02,524.59],[790.64,611.5],[756.99,715.06],[790.64,818.63],[987.64,547.49],[878.74,882.64],[878.74,547.49],[1282.21,484.89],[1243.62,401.08],[1488.63,603.97],[1399.06,538.9],[1522.84,498.68],[1399.06,669.05],[1287.59,215.47],[1262.1,323.21],[1313.08,107.74],[50.95,600.08],[141.18,966.16],[0,806.8],[141.18,537.8],[50.95,903.88],[0,697.16],[765.65,208.84],[592.77,1142.94],[799.86,314.13],[799.86,103.55],[676.08,143.76],[676.08,273.91],[772.89,1137.75],[706.25,1221.5],[488.59,1379.94],[460.19,1278.11],[414.6,1455.45],[619.49,1010.84],[1338.57,0],[1522.84,709.26],[1599.34,603.97],[730.18,1325.82],[826.67,1372.15],[876.36,208.84]],"e":[[0,2],[0,5],[0,3],[0,4],[1,7],[1,6],[1,8],[1,10],[1,9],[0,12],[0,14],[0,11],[15,13],[16,13],[1,17],[1,19],[1,18],[0,24],[0,22],[0,20],[0,23],[0,21],[1,58],[0,25],[0,58],[1,29],[1,31],[1,26],[1,27],[1,28],[1,30],[32,34],[1,33],[32,33],[36,34],[35,34],[37,34],[38,40],[38,39],[0,44],[0,41],[0,46],[0,43],[0,45],[0,42],[1,47],[48,8],[48,58],[48,10],[48,18],[48,19],[50,47],[51,47],[52,47],[49,47],[48,53],[48,54],[1,54],[1,53],[55,56],[56,15],[16,57],[57,55],[56,48],[39,33],[59,40],[60,34],[61,34],[1,62],[48,62],[1,63],[48,63],[64,47]],"vp":[0.0,0.0,1650.0,1506.0]}} Organic Layout of the same graph applying
     * specific layout styles to the detected substructures
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface OrganicLayout extends yfiles.layout.MultiStageLayout{}
    export class OrganicLayout {
      /**
       * Creates a new {@link yfiles.organic.OrganicLayout} instance with default settings.
       */
      constructor();
      /**
       * Configures the given {@link yfiles.layout.ComponentLayout} to take fixed nodes in components into account.
       * <p>
       * Components that contain fixed nodes will not be rearranged.
       * </p>
       * <p>
       * This method is called by {@link yfiles.organic.OrganicLayout#applyLayout} before the actual layout is calculated. It may be
       * overridden in order to manually configure the {@link yfiles.layout.ComponentLayout}.
       * </p>
       * @param graph the input graph
       * @param layouter the {@link } instance to reconfigure
       * @see yfiles.organic.OrganicLayout#smartComponentLayout
       * @protected
       */
      configureComponentLayout(graph:yfiles.layout.LayoutGraph,layouter:yfiles.layout.ComponentLayout):void;
      /**
       * Disposes of the {@link yfiles.layout.ComponentLayout} instance.
       * <p>
       * This method is called by {@link yfiles.organic.OrganicLayout#applyLayout} after the actual layout is calculated. It may be
       * overridden in order to revert a custom configuration made in {@link yfiles.organic.OrganicLayout#configureComponentLayout}.
       * </p>
       * @param graph the input graph
       * @param layouter the {@link } to reset
       * @see yfiles.organic.OrganicLayout#smartComponentLayout
       * @protected
       */
      disposeComponentLayout(graph:yfiles.layout.LayoutGraph,layouter:yfiles.layout.ComponentLayout):void;
      /**
       * Data provider key for specifying the directedness of edges
       * <p>
       * Generally, the organic layout algorithm doesn't consider the edge direction. Nevertheless, this {@link yfiles.algorithms.IDataProvider}
       * allows the user to specify hints on the directedness of edges. More precisely, a value of <code>1</code> indicates that the edge
       * should be considered to be directed from source to target, a value of <code>-1</code> that it is directed from target to source, and
       * a value of <code>0</code> means that it is undirected.
       * </p>
       * <p>
       * Currently, the specified values are only considered during the detection of special substructures, see {@link yfiles.organic.OrganicLayout#chainSubstructureStyle}, {@link yfiles.organic.OrganicLayout#cycleSubstructureStyle}, {@link yfiles.organic.OrganicLayout#parallelSubstructureStyle} and {@link yfiles.organic.OrganicLayout#starSubstructureStyle}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static EDGE_DIRECTEDNESS_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for marking the nodes that are part of the relevant subset
       * @see yfiles.organic.OrganicLayout#scope
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static AFFECTED_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for assigning individual modes for all group nodes
       * <p>
       * The modes specify how a group's content is handled and if it is resized during layout calculation.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.organic.GroupNodeMode>}
       */
      static GROUP_NODE_MODE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.organic.GroupNodeMode>;
      /**
       * Data provider key for defining an individual preferred length for each edge
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static PREFERRED_EDGE_LENGTH_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * The group node compactness factor.
       * <p>
       * Values should be in the range [0..1], where 0 results in group nodes not affecting the overall layout too much, whereas
       * a value of 1 forces nodes in the same group to be clustered tightly.
       * <br />
       * Note: the specified value is only considered if option AutomaticGroupNodeCompactionEnabled is disabled and if there is
       * no partition grid structure.
       * </p>
       * <p>
       * By default this value is <code>0.4</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if compactness value does not lie in [0..1]
       * @type {number}
       */
      groupNodeCompactness:number;
      /**
       * Gets or sets whether or not group nodes are compacted automatically.
       * <p>
       * When enabled, the compactness factor is determined automatically, i.e. it only depends on the general compactness
       * specified by option {@link yfiles.organic.OrganicLayout#compactnessFactor}.
       * </p>
       * @see yfiles.organic.OrganicLayout#groupNodeCompactness
       * @see yfiles.organic.OrganicLayout#compactnessFactor
       * @type {boolean}
       */
      automaticGroupNodeCompaction:boolean;
      /**
       * Gets or sets whether or not a clustering algorithm should be applied to the input graph.
       * <p>
       * When this option is enabled, the following steps are performed during the layout:
       * </p>
       * <ol>
       * <li>A clustering algorithm is applied to the input graph.</li>
       * <li>All nodes of the same cluster are put into a newly created group node.</li>
       * <li>The layout is applied to the modified graph.</li>
       * <li>Group nodes denoting clusters (inserted during step <code>2</code>) are removed.</li>
       * </ol>
       * <p>
       * The user can also specify customized clusters by defining appropriate groups.
       * </p>
       * @see yfiles.organic.OrganicLayout#maximumDuration
       * @type {boolean}
       */
      clusterNodes:boolean;
      /**
       * Gets or sets the quality measure of the clustering algorithm.
       * <p>
       * The higher the value, the higher the clustering quality.
       * </p>
       * <p>
       * The value needs to lie in <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified quality measure is outside the interval <code>[0,1]</code>
       * @see yfiles.organic.OrganicLayout#clusterNodes
       * @see yfiles.organic.OrganicLayout#maximumDuration
       * @type {number}
       */
      clusteringQuality:number;
      /**
       * Gets or sets whether or not to reserve space for node labels during layout calculation.
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets the {@link yfiles.layout.IGroupBoundsCalculator} instance used for calculating the size of group nodes.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.layout.IGroupBoundsCalculator} is <code>null</code>
       * @type {yfiles.layout.IGroupBoundsCalculator}
       */
      groupBoundsCalculator:yfiles.layout.IGroupBoundsCalculator;
      /**
       * Gets or sets whether or not this instance should configure the {@link yfiles.layout.ComponentLayout} to respect subsets of
       * nodes.
       * <p>
       * When only a subset of nodes is placed by the layout algorithm, the fixed nodes will keep their locations even if they
       * reside in different components.
       * </p>
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.organic.OrganicLayout#scope
       * @type {boolean}
       */
      smartComponentLayout:boolean;
      /**
       * Gets or sets whether or not the layout algorithm tries to avoid node/edge overlaps.
       * @see yfiles.organic.OrganicLayout#nodeOverlapsAllowed
       * @type {boolean}
       */
      nodeEdgeOverlapAvoided:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      /**
       * Gets or sets the ratio of layout quality versus running time.
       * <p>
       * The larger the ratio, the better the quality of the resulting layout but the longer it may take to perform the layout
       * calculation.
       * </p>
       * <p>
       * The value needs to lie within <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified ratio is outside the interval <code>[0,1]</code>
       * @type {number}
       */
      qualityTimeRatio:number;
      /**
       * Gets or sets the maximum duration in milliseconds that this layout algorithm is allowed to run.
       * <p>
       * The duration needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified duration has a negative value
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the scope that determines which nodes are placed by this algorithm.
       * @throws {Stubs.Exceptions.ArgumentError} if the given scope is unknown
       * @see yfiles.organic.OrganicLayout#AFFECTED_NODES_DP_KEY
       * @type {yfiles.organic.Scope}
       */
      scope:yfiles.organic.Scope;
      /**
       * Gets or sets the style specifier for chain substructures.
       * <p>
       * A chain is a simple edge path where the degree of the nodes is less than or equal to <code>2</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given style is unknown
       * @see yfiles.organic.OrganicLayout#scope
       * @see yfiles.organic.OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
       * @type {yfiles.organic.ChainSubstructureStyle}
       */
      chainSubstructureStyle:yfiles.organic.ChainSubstructureStyle;
      /**
       * Gets or sets the style specifier for cycle substructures.
       * <p>
       * A cycle is a simple edge path where the first and last node are identical. The algorithm only considers cycles where the
       * number of edges connecting nodes of the cycle with the remaining nodes is less than or equal to <code>2</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given style is unknown
       * @see yfiles.organic.OrganicLayout#scope
       * @see yfiles.organic.OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
       * @type {yfiles.organic.CycleSubstructureStyle}
       */
      cycleSubstructureStyle:yfiles.organic.CycleSubstructureStyle;
      /**
       * Gets or sets the style specifier for parallel substructures.
       * <p>
       * A parallel structure consists of a set of nodes (called the inner nodes) such that all nodes have degree two and are
       * connected to the same pair of neighbors (called the outer nodes).
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given style is unknown
       * @see yfiles.organic.OrganicLayout#scope
       * @see yfiles.organic.OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
       * @type {yfiles.organic.ParallelSubstructureStyle}
       */
      parallelSubstructureStyle:yfiles.organic.ParallelSubstructureStyle;
      /**
       * Gets or sets the style specifier for star substructures.
       * <p>
       * A star consists of a set of degree one nodes that are all connected to the same node (called the root of the star).
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given style is unknown
       * @see yfiles.organic.OrganicLayout#scope
       * @see yfiles.organic.OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
       * @type {yfiles.organic.StarSubstructureStyle}
       */
      starSubstructureStyle:yfiles.organic.StarSubstructureStyle;
      /**
       * Gets or sets the compactness factor for the layout algorithm.
       * <p>
       * Smaller values result in less compact drawings, greater values result in more compact drawings.
       * </p>
       * <p>
       * The compactness value needs to lie in <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified value is outside the interval <code>[0,1]</code>
       * @type {number}
       */
      compactnessFactor:number;
      /**
       * Gets or sets the default preferred edge length.
       * <p>
       * If there is no specific preferred edge length assigned to an edge, this default preferred edge length is used.
       * </p>
       * <p>
       * The preferred edge length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified edge length is negative
       * @see yfiles.organic.OrganicLayout#PREFERRED_EDGE_LENGTH_DP_KEY
       * @type {number}
       */
      preferredEdgeLength:number;
      /**
       * Gets or sets the minimum preferred distance between nodes and edges when node-edge overlaps are not allowed.
       * <p>
       * The minimum preferred distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      preferredMinimumNodeToEdgeDistance:number;
      /**
       * Gets or sets whether or not to consider node sizes during layout calculation.
       * <p>
       * If this option is enabled, the circumcircles of the nodes are used. If it is disabled, points will be used instead.
       * </p>
       * @type {boolean}
       */
      considerNodeSizes:boolean;
      /**
       * Gets or sets whether or not the deterministic mode of this algorithm is enabled.
       * <p>
       * In deterministic mode, the layout algorithm will yield the same results if the exact same input and same settings are
       * given as input.
       * </p>
       * @type {boolean}
       */
      deterministic:boolean;
      /**
       * Gets or sets the minimum node distance that this algorithm should enforce between all pairs of nodes.
       * <p>
       * The minimum node distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified minimum node distance is negative
       * @see yfiles.organic.OrganicLayout#nodeOverlapsAllowed
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets whether or not overlaps between nodes are allowed.
       * @see yfiles.organic.OrganicLayout#minimumNodeDistance
       * @see yfiles.organic.OrganicLayout#scope
       * @type {boolean}
       */
      nodeOverlapsAllowed:boolean;
      /**
       * Gets or sets the area restriction for the result of the layout algorithm.
       * @type {yfiles.organic.OutputRestriction}
       */
      outputRestriction:yfiles.organic.OutputRestriction;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout stage enables support for handling a graph with a {@link yfiles.layout.PartitionGrid} structure.
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * Before executing the
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}, this stage prepares the grid structure of the graph accordingly, e.g., by hiding top-level grid
     * nodes. Then, the core layout algorithm is applied to the graph. Finally, the grid is restored, configured and positioned
     * such that it is consistent with the layout computed by the core algorithm.
     * </p>
     * <p>
     * Information about the {@link yfiles.layout.PartitionGrid} structure is retrieved from a {@link yfiles.algorithms.IDataProvider}
     * registered with the graph using key {@link yfiles.layout.PartitionGrid#PARTITION_GRID_DP_KEY}.
     * </p>
     * <p>
     * <b>Usage</b>
     * </p>
     * <p>
     * When using {@link yfiles.organic.OrganicLayout}, this stage will automatically be used, if required. It is not necessary to
     * append this stage manually to {@link yfiles.organic.OrganicLayout}.
     * </p>
     * <p>
     * Appending this stage can be useful to add support for {@link yfiles.layout.PartitionGrid partition grid} structures to other
     * {@link yfiles.layout.ILayoutAlgorithm}s.
     * </p>
     * <p>
     * The stage can be appended to all algorithms of type {@link yfiles.layout.MultiStageLayout} by using method
     * {@link yfiles.layout.MultiStageLayout#appendStage}.
     * </p>
     * @see yfiles.layout.PartitionGrid
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface OrganicPartitionGridLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class OrganicPartitionGridLayoutStage {
      /**
       * Creates a new instance of {@link yfiles.organic.OrganicPartitionGridLayoutStage} with the given layout algorithm as
       * {@link yfiles.layout.LayoutStageBase#coreLayout core algorithm}.
       * @param core the core layout algorithm
       */
      constructor(core:yfiles.layout.ILayoutAlgorithm);
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout stage removes node overlaps while changes to the original layout structure are avoided and a specified
     * minimum node distance is guaranteed.
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * Unlike the overlap removal approach implemented by
     * {@link yfiles.organic.RemoveOverlapsStage}, this stage tries to keep the original layout structure, i.e., it tries to keep the
     * geometric distances between the nodes.
     * </p>
     * <p>
     * The strategy for removing overlaps is similar to the layout strategy used by organic (force-directed) layout algorithms
     * (e.g.
     * {@link yfiles.organic.OrganicLayout}). Therefore, this stage is especially well suited to remove node overlaps in undirected
     * graph drawings with straight-line edges, as produced by organic layout approaches.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0,"c":1},"n":[[-264.13,89.64],[-109.95,153.19],[-68.91,-95.12],[-178.12,-35.61,60,60],[-88.42,-54.43],[-134.12,41.2],[-214.95,40.96,60,60],[-96.74,81.55,60,60],[-145.17,111.32],[-255.82,6.45],[-272.91,49.11],[-47.42,-39.94,60,60],[-645,65.04],[-507.21,54.41],[-586.74,40.54,60,60],[-488.66,89.42,60,60],[-539.17,115.13],[-488.66,132.42],[-636.26,24.59],[-645,46.48],[-468.9,-54.43],[-546.97,19.76,60,60],[-456.97,-40.24],[-447.21,-40.24,60,60],[-320.5,-180,1,1,4],[-320.5,251.5,1,1,4]],"e":[[2,3],[8,7],[5,7],[9,6],[6,5],[8,6],[0,6],[10,6],[3,6],[8,5],[7,6],[3,4],[4,2],[7,1],[8,1],[7,11],[20,21],[16,15],[13,15],[18,14],[14,13],[16,14],[12,14],[19,14],[21,14],[16,13],[15,14],[21,22],[22,20],[15,17],[16,17],[15,23],[24,25]],"vp":[-645.0,-182.0,658.0,436.0]}} A graph containing overlaps (left) and the result after applying this layout stage<b>Features</b>
     * </p>
     * <p>
     * It is possible to define a {@link yfiles.organic.OrganicRemoveOverlapsStage#minimumNodeDistance minimum distance} between nodes. This feature allows to not only remove overlaps but forces
     * nodes to keep the specified distance to other nodes.
     * </p>
     * <p>
     * Nodes can be marked as <em>fixed</em>. Such nodes will not be moved during the overlap removal process. A {@link yfiles.algorithms.IDataProvider}
     * with key {@link yfiles.organic.OrganicRemoveOverlapsStage#FIXED_NODE_DP_KEY} can be registered to mark the fixed nodes.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface OrganicRemoveOverlapsStage extends yfiles.layout.LayoutStageBase{}
    export class OrganicRemoveOverlapsStage {
      /**
       * Creates a new instance of {@link yfiles.organic.OrganicRemoveOverlapsStage} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The core layout algorithm.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Data provider key for marking nodes as fixed
       * <p>
       * A node marked as fixed will not be moved by this algorithm but stay at its current position.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static FIXED_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets or sets the maximum duration in milliseconds that this layout stage is allowed to run.
       * <p>
       * This maximum time does not include the time required for the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * </p>
       * <p>
       * The duration needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified duration has a negative value
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the minimum distance to be kept between any two nodes of the graph.
       * <p>
       * Due to this distance not only nodes that overlap will be moved to resolve the overlap, but also nodes which are too
       * close to each other with respect to this distance will be displaced.
       * </p>
       * <p>
       * The distance needs to be a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum distance is negative
       * @type {number}
       */
      minimumNodeDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.organic.OutputRestriction}s restrict the output area and the shape of a layout in conjunction with organic layout
     * algorithms such as {@link yfiles.organic.OrganicLayout} or {@link yfiles.organic.InteractiveOrganicLayout}.
     * <p>
     * In consequence, all nodes need to be placed inside a specific restricted area.
     * </p>
     * @see yfiles.organic.OrganicLayout#outputRestriction
     * @see yfiles.organic.InteractiveOrganicLayout#outputRestriction
     * @class
     */
    export interface OutputRestriction extends Object{}
    export class OutputRestriction {
      /**
       * Creates an {@link yfiles.organic.OutputRestriction} that confines the layout result to a rectangular area which roughly complies
       * with the given <em>aspect ratio</em>.
       * <p>
       * The aspect ratio is defined as the ratio between the width and the height of a layout.
       * </p>
       * <ul>
       * <li>aspect ratio 1: width and height of the layout should be equal</li>
       * <li>aspect ratio between 0 and 1: the height of a layout should be greater than its width</li>
       * <li>aspect ratio larger than 1: the width of a layout should be greater than its height</li>
       * </ul>
       * <p>
       * The aspect ratio needs to be strictly positive.
       * </p>
       * @param ratio the preferred aspect ratio of the output
       * @returns an {@link } imposing the given aspect ratio on the layout
       * @throws {Stubs.Exceptions.ArgumentError} if the given aspect ratio is <code>0</code> or negative
       * @see yfiles.organic.OrganicLayout#outputRestriction
       * @see yfiles.organic.InteractiveOrganicLayout#outputRestriction
       * @static
       */
      static createAspectRatioRestriction(ratio:number):yfiles.organic.OutputRestriction;
      /**
       * Creates an {@link yfiles.organic.OutputRestriction} that confines the layout result to a circle.
       * <p>
       * The actual <em>restriction circle</em> is defined by the given coordinates and radius.
       * </p>
       * <p>
       * The radius of the restriction circle needs to be greater than zero.
       * </p>
       * @param x the x-coordinate of the restriction circle's center
       * @param y the y-coordinate of the restriction circle's center
       * @param radius the radius of the restriction circle
       * @returns the {@link } confining the layout to a circular area
       * @throws {Stubs.Exceptions.ArgumentError} if the given radius is less than or equal to zero
       * @see yfiles.organic.OrganicLayout#outputRestriction
       * @see yfiles.organic.InteractiveOrganicLayout#outputRestriction
       * @static
       */
      static createCircularCageRestriction(x:number,y:number,radius:number):yfiles.organic.OutputRestriction;
      /**
       * Creates an {@link yfiles.organic.OutputRestriction} that confines the layout result to an elliptical area.
       * <p>
       * The <em>restriction ellipse</em> is defined by its bounding box specified by the given coordinates and size values.
       * </p>
       * <p>
       * The main axes of the ellipse are parallel to the coordinate axes.
       * </p>
       * <p>
       * The values for height and width of the ellipse's bounding box need to be greater than zero.
       * </p>
       * @param x the x-coordinate of the bounding box's upper left corner
       * @param y the y-coordinate of the bounding box's upper left corner
       * @param w the width of the bounding box
       * @param h the height of the bounding box
       * @returns the {@link } confining the layout to an elliptical area
       * @throws {Stubs.Exceptions.ArgumentError} if the given width or height is <code>0</code> or negative
       * @see yfiles.organic.OrganicLayout#outputRestriction
       * @see yfiles.organic.InteractiveOrganicLayout#outputRestriction
       * @static
       */
      static createEllipticalCageRestriction(x:number,y:number,w:number,h:number):yfiles.organic.OutputRestriction;
      /**
       * Creates an {@link yfiles.organic.OutputRestriction} that confines the layout to a rectangular area.
       * <p>
       * This <em>restriction rectangle</em> is defined by the given coordinates and size values.
       * </p>
       * <p>
       * The values for height and width of the rectangle need to be greater than zero.
       * </p>
       * @param x the x-coordinate of the restriction rectangle's upper-left corner
       * @param y the y-coordinate of the restriction rectangle's upper-left corner
       * @param w width of the restriction rectangle
       * @param h height of the restriction rectangle
       * @returns the {@link } confining the layout to a rectangular area
       * @throws {Stubs.Exceptions.ArgumentError} if the given width or height is less than or equal to zero
       * @see yfiles.organic.OrganicLayout#outputRestriction
       * @see yfiles.organic.InteractiveOrganicLayout#outputRestriction
       * @static
       */
      static createRectangularCageRestriction(x:number,y:number,w:number,h:number):yfiles.organic.OutputRestriction;
      /**
       * This {@link yfiles.organic.OutputRestriction} does not restrict the output by any means.
       * <p>
       * It is a no-op that can be used when the output area and shape of a layout should be unrestricted.
       * </p>
       * @see yfiles.organic.OrganicLayout#outputRestriction
       * @see yfiles.organic.InteractiveOrganicLayout#outputRestriction
       * @const
       * @static
       * @type {yfiles.organic.OutputRestriction}
       */
      static NONE:yfiles.organic.OutputRestriction;
      static $class:yfiles.lang.Class;
    }
    /**
     * This algorithm recursively removes node overlaps from a graph with a hierarchic group structure.
     * <p>
     * Basically, this algorithm combines a {@link yfiles.layout.RecursiveGroupLayout} with a specific customizable {@link yfiles.organic.RecursiveShuffleLayout#shuffleLayout shuffle algorithm}
     * as its
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. The shuffle algorithm will be recursively applied to all groups of the input graph to remove node
     * overlaps.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[50,50],"d":1},"n":[[158,104.94,384.5,224.75,2,[[158,127.31,384.5,22.38,"Group 1"]]],[173,142.31,175,172.38,2,[[173,164.69,175,22.38,"Group 2"]]],[205,187.69,60,30],[235,179.69,60,30],[225,249.69,60,30],[188,239.69,60,30],[407.5,177.69],[445,187.69,60,30],[432.5,203.38],[382.5,218.75],[295,242.31,155,177.88,2,[[295,264.69,155,22.38,"Group 3"]]],[310,279.69],[335,305.19],[360,329.69],[385,355.19],[469,265.25,50,154.94],[737.5,104.94,386,204.75,2,[[737.5,127.31,386,22.38,"Group 1"]]],[752.5,142.31,170,152.38,2,[[752.5,164.69,170,22.38,"Group 2"]]],[781.5,187.69,60,30],[847.5,179.69,60,30],[833.5,249.69,60,30],[767.5,239.69,60,30],[987,169.5],[1048.5,176.5,60,30],[1042,212.5],[974,225.5],[889.5,315.62,155,270.38,2,[[889.5,338,155,22.38,"Group 3"]]],[904.5,353],[929.5,409],[954.5,465],[979.5,521],[1063.5,324.84,50,154.94],[569.5,329.5,1,1,4],[706.5,329.5,1,1,4]],"e":[[32,33]],"vp":[158.0,104.0,966.0,482.0]}} Input graph (left) and result (right) after executing this algorithm with default settings
     * </p>
     * <p>
     * Since this algorithm is a {@link yfiles.layout.ILayoutStage}, a {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} can be specified. In consequence, this
     * algorithm will perform its work after the core algorithm was executed.
     * </p>
     * @see yfiles.organic.ShuffleLayout
     * @see yfiles.layout.RecursiveGroupLayout
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface RecursiveShuffleLayout extends yfiles.layout.LayoutStageBase{}
    export class RecursiveShuffleLayout {
      /**
       * Creates a new {@link yfiles.organic.RecursiveShuffleLayout} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The core layout algorithm.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Gets or sets the {@link yfiles.layout.IGroupBoundsCalculator} instance that computes the sizes of all group nodes.
       * @type {yfiles.layout.IGroupBoundsCalculator}
       */
      groupBoundsCalculator:yfiles.layout.IGroupBoundsCalculator;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutAlgorithm} instance that is used for removing node overlaps.
       * <p>
       * This instance will be applied recursively to all group nodes of the graph.
       * </p>
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      shuffleLayout:yfiles.layout.ILayoutAlgorithm;
      static $class:yfiles.lang.Class;
    }
    /**
     * This {@link yfiles.layout.ILayoutStage} removes node overlaps and guarantees a certain minimum distance between nodes.
     * <p>
     * <b>Layout Style</b> This stage only removes overlaps between nodes of the input graph. To do so, overlapping nodes will be moved apart.
     * Results often look like a <em>stretched</em> version of the original graph. {@graph {"ann":{"s":[30,30],"d":0,"c":1},"n":[[206,563.54],[343.79,552.91],[264.26,539.04,60,60],[362.34,587.92,60,60],[311.83,613.63],[362.34,630.92],[214.74,523.09],[206,544.98],[382.1,444.07],[304.03,518.26,60,60],[394.03,458.26],[403.79,458.26,60,60],[530.5,318.5,1,1,4],[530.5,750,1,1,4],[587.5,574.26],[819.5,549.26],[677.5,536.26,60,60],[847.5,609.26,60,60],[757.5,647.26],[838.5,680.26],[601.5,492.26],[587.5,533.26],[870.5,365.26],[748.5,503.26,60,60],[881.5,406.26],[922.5,406.26,60,60]],"e":[[8,9],[4,3],[1,3],[6,2],[2,1],[4,2],[0,2],[7,2],[9,2],[4,1],[3,2],[9,10],[10,8],[3,5],[4,5],[3,11],[12,13],[22,23],[18,17],[15,17],[20,16],[16,15],[18,16],[14,16],[21,16],[23,16],[18,15],[17,16],[23,24],[24,22],[17,19],[18,19],[17,25]],"vp":[206.0,317.0,777.0,436.0]}} A graph containing overlaps (left) and the result after the
     * overlap removal <b>Features</b>
     * </p>
     * <p>
     * It is possible to define a {@link yfiles.organic.RemoveOverlapsStage#minimumNodeDistance minimum distance} between nodes. This feature allows to not only remove overlaps but forces
     * nodes to keep a specified distance to other nodes.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface RemoveOverlapsStage extends Object,yfiles.layout.ILayoutStage{}
    export class RemoveOverlapsStage {
      /**
       * Creates a new {@link yfiles.organic.RemoveOverlapsStage} instance with a given {@link yfiles.organic.RemoveOverlapsStage#minimumNodeDistance minimum node distance}.
       * <p>
       * The minimum node distance needs to be non-negative.
       * </p>
       * @param minimumNodeDistance a non-negative minimum distance value
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance is negative
       */
      constructor(minimumNodeDistance:number);
      /**
       * Removes node overlaps and displaces nodes of the given graph such that they keep the specified {@link yfiles.organic.RemoveOverlapsStage#minimumNodeDistance minimum distance} to each
       * other.
       * <p>
       * If a {@link yfiles.organic.RemoveOverlapsStage#coreLayout core layout algorithm} is specified, it will be executed first.
       * </p>
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Gets or sets the minimum distance to be kept between any two nodes of the graph.
       * <p>
       * Due to this distance not only nodes that overlap will be moved to resolve the overlap, but also nodes which are too
       * close to each other with respect to this distance will be displaced.
       * </p>
       * <p>
       * The minimum node distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum distance is negative
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * 
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout stage replaces edges by a path of <em>split-edges</em> with a specified length before it calls a {@link yfiles.organic.SplitEdgeStage#coreLayout core layout algorithm}.
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * Each edge segment (or only segments that belong to a particular subset of edges) in the input graph will be split up
     * into several segments such that all segments have a maximum length defined by the specified
     * {@link yfiles.organic.SplitEdgeStage#splitSegmentLength split segment length}. If an edge segment is already shorter than the split segment length, then it will not be split
     * up.
     * </p>
     * <p>
     * At each split location, a dummy node (the so-called <em>split-node</em>) will be inserted. Split-nodes inserted for the same original edge
     * and the actual source and target node of the original edge will be connected by so called
     * <em>split-edges</em>. In consequence, the original source and target node are not connected by a single edge but by a path consisting of
     * split-edges. Furthermore, all edges are simple straight lines.
     * </p>
     * <p>
     * After this replacement step, the {@link yfiles.organic.SplitEdgeStage#coreLayout core layout algorithm} will be invoked.
     * </p>
     * <p>
     * When the core layout algorithm has finished, the split-nodes will be removed from the graph and the original edge will
     * be restored. The coordinates of the bends will be the center-coordinates of the corresponding split-node.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The {@link yfiles.algorithms.IDataProvider} registered with key {@link yfiles.organic.SplitEdgeStage#splitNodesDpKey} can be used by the {@link yfiles.organic.SplitEdgeStage#coreLayout core layout algorithm} to determine whether
     * or not a certain node is a split-node that was created by this stage. This information may be useful when providing a
     * custom core layout algorithm which should handle split-nodes in a different way than normal nodes. If no {@link yfiles.algorithms.IDataProvider}
     * is registered with this key, this information is dropped.
     * </p>
     * <p>
     * By registering a {@link yfiles.algorithms.IDataProvider} with key {@link yfiles.organic.SplitEdgeStage#affectedEdgesDpKey}, the subset of edges that should be split by this layout
     * stage can be specified. If no {@link yfiles.algorithms.IDataProvider} with this key is registered, all edges of the input graph
     * will be split by the layout stage.
     * </p>
     * <p>
     * The amount of split-nodes replacing a single original edge can be controlled via the
     * {@link yfiles.organic.SplitEdgeStage#splitSegmentLength split segment length}. The smaller the value, the more split-nodes this stage will insert.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface SplitEdgeStage extends Object,yfiles.layout.ILayoutStage{}
    export class SplitEdgeStage {
      /**
       * Creates a new {@link yfiles.organic.SplitEdgeStage} with default settings.
       */
      constructor();
      /**
       * Replaces the edges of the given graph by a path of split-edges, executes the {@link yfiles.organic.SplitEdgeStage#coreLayout core layout algorithm} and restores the
       * original edges, introducing bends at the locations of the split-nodes.
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key to determine which edges are to be split by this layout stage.
       * <p>
       * If no {@link yfiles.algorithms.IDataProvider} is registered with this key, all edges will be split.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key for marking the nodes that were created by splitting edges, i.e.,
       * the split-nodes.
       * <p>
       * The information provided by the {@link yfiles.algorithms.IDataProvider} registered with this key is useful for the {@link yfiles.organic.SplitEdgeStage#coreLayout core layout algorithm}
       * to know whether or not some node is a (dummy) split-node. After {@link yfiles.organic.SplitEdgeStage#applyLayout} finishes, the
       * split-nodes will have been removed and this information will not be available anymore.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @type {Object}
       */
      splitNodesDpKey:Object;
      /**
       * Gets or sets the maximum length for a <em>split-edge</em>.
       * <p>
       * The length of the original edge path divided by the given split segment length yields the number of nodes on the new
       * path.
       * </p>
       * <p>
       * The split segment length must be a strictly positive value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given split segment length is smaller than or equal to zero
       * @type {number}
       */
      splitSegmentLength:number;
      /**
       * Gets or sets the size of the split-nodes that are inserted when splitting edges.
       * <p>
       * The given size will be used for defining the height and the width of the split-nodes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given size is negative
       * @type {number}
       */
      splitNodeSize:number;
      /**
       * 
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.organic.OrganicLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface OrganicLayoutData extends yfiles.layout.LayoutData{}
    export class OrganicLayoutData {
      constructor();
      /**
       * Gets or sets the collection of nodes that is laid out by the layout.
       * @see yfiles.organic.OrganicLayout#AFFECTED_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      affectedNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the mapping from edges to their preferred edge length.
       * @see yfiles.organic.OrganicLayout#PREFERRED_EDGE_LENGTH_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      preferredEdgeLengths:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      /**
       * Gets or sets the mapping from nodes to their minimum distance to other nodes.
       * <p>
       * Note that the distances are only considered by the internally used {@link yfiles.organic.ShuffleLayout} when {@link yfiles.organic.OrganicLayout#nodeOverlapsAllowed} is <code>true</code> and {@link yfiles.organic.OrganicLayout#scope} is
       * not {@link yfiles.organic.Scope#ALL}.
       * </p>
       * @see yfiles.organic.ShuffleLayout#MINIMUM_DISTANCE_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      minimumNodeDistances:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      /**
       * Gets or sets the mapping from group nodes to a mode constant describing how to handle the group node.
       * <p>
       * The algorithm expects for each group node to find one of the following constants or <code>null</code>:
       * <ul>
       * <li>
       * {@link yfiles.organic.GroupNodeMode#NORMAL}
       * </li>
       * <li>
       * {@link yfiles.organic.GroupNodeMode#FIX_BOUNDS}
       * </li>
       * <li>
       * {@link yfiles.organic.GroupNodeMode#FIX_CONTENTS}
       * </li>
       * </ul>
       * </p>
       * @see yfiles.organic.OrganicLayout#GROUP_NODE_MODE_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.organic.GroupNodeMode>}
       */
      groupNodeModes:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.organic.GroupNodeMode>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the partition grid layout data.
       * @type {yfiles.layout.PartitionGridData}
       */
      partitionGridData:yfiles.layout.PartitionGridData;
      /**
       * Gets or sets the mapping from edges to their directedness.
       * <p>
       * Generally, the organic layout algorithm doesn't consider the edge direction. Nevertheless, this property allows the user
       * to specify hints on the directedness of edges. More precisely, a value of <code>1</code> indicates that the edge should be considered
       * to be directed from source to target, a value of <code>-1</code> that it is directed from target to source, and a value of <code>0</code> means
       * that it is undirected.
       * </p>
       * <p>
       * Currently, the specified values are only considered during the detection of special substructures, see {@link yfiles.organic.OrganicLayout#chainSubstructureStyle}, {@link yfiles.organic.OrganicLayout#cycleSubstructureStyle}, {@link yfiles.organic.OrganicLayout#parallelSubstructureStyle} and {@link yfiles.organic.OrganicLayout#starSubstructureStyle}.
       * </p>
       * @see yfiles.organic.OrganicLayout#EDGE_DIRECTEDNESS_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,number>}
       */
      edgeDirectedness:yfiles.layout.ItemMapping<yfiles.graph.IEdge,number>;
      static $class:yfiles.lang.Class;
    }
  }export namespace router{
    /**
     * {@link yfiles.router.ParallelEdgeRouter} is a {@link yfiles.layout.ILayoutStage} that routes parallel edges which connect the same pair
     * of nodes in a graph.
     * <p>
     * <b>Layout Style</b> The edge paths of parallel edges consist of parallel segments. At the end points, the edges can either still be
     * parallel or {@link yfiles.router.ParallelEdgeRouter#joinEnds joined in one point}. {@graph {"ann":{"s":[30,30],"d":0,"c":1},"n":[[198,216],[54,226],[0,93],[110,0],[232,76],[119,122]],"e":[[1,0],[2,1],[3,2],[0,4],[4,3],[2,5,[43.96,125.35,100.3,139.08]],[2,5,[48.7,105.92,105.04,119.65]],[2,5,[51.07,96.2,107.4,109.93]],[3,5,[147.32,45.67,151.57,103.38]],[1,5,[77.62,208.34,108.42,159.06]],[1,5,[69.14,203.04,99.94,153.76]],[0,5,[184.58,212.72,147.11,168.14]],[0,5,[176.92,219.16,139.46,174.58]],[5,3,[141.6,104.12,137.34,46.41]],[5,3,[121.66,105.59,117.4,47.88]],[5,3,[111.68,106.33,107.43,48.62]],[5,4,[171.36,143.38,224.72,121.66]],[5,4,[167.59,134.12,220.95,112.4]],[5,4,[160.05,115.6,213.41,93.88]],[5,2,[97.93,148.8,41.6,135.07]],[5,4,[156.28,106.34,209.64,84.62]],[5,0,[162.42,155.28,199.89,199.86]],[5,0,[170.08,148.84,207.54,193.42]],[5,1,[125.38,169.66,94.58,218.94]],[5,1,[133.86,174.96,103.06,224.24]],[1,5,1],[2,5,1],[0,5,1],[5,3,1],[5,4,1]],"vp":[0.0,0.0,262.0,256.0]}}
     * </p>
     * <p>
     * The master edge determining the routing is highlighted
     * </p>
     * <p>
     * <b>Concept</b> {@link yfiles.router.ParallelEdgeRouter} performs four basic steps:
     * </p>
     * <ol>
     * <li>Remove all parallel edges from the input graph, leaving one master edge in the graph</li>
     * <li>Invoke the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} on the graph without parallel edges</li>
     * <li>Reinsert all previously removed edges</li>
     * <li>Route the parallel edges</li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * {@link yfiles.router.ParallelEdgeRouter} can either be used as a {@link yfiles.layout.ILayoutStage} wrapping a {@link yfiles.layout.ILayoutAlgorithm layout algorithm} which
     * cannot handle parallel edges. Then it will hide the parallel edges from this {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} and take over the
     * routing of them.
     * <br />
     * If no {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} is specified, {@link yfiles.router.ParallelEdgeRouter} can work alone. It will route only the
     * parallel edges and keep the remaining graph unchanged.
     * </p>
     * <p>
     * Parallel edges can either be all edges between the same two nodes, ignoring the direction, or edges are only considered
     * parallel if they share the same source node and target node. Hence, they share the same {@link yfiles.router.ParallelEdgeRouter#directedMode direction}.
     * </p>
     * <p>
     * To make sure that the ports of all parallel edges are on the end nodes, the first and last segments can be
     * {@link yfiles.router.ParallelEdgeRouter#joinEnds joined}. However, these segments won't be parallel.
     * </p>
     * <p>
     * There are two ways to define the distance between two parallel edges. First, an absolute distance can be {@link yfiles.router.ParallelEdgeRouter#lineDistance defined}.
     * Parallel segments will keep this distance. Second, the distance can be
     * {@link yfiles.router.ParallelEdgeRouter#adaptiveLineDistances determined relative to the node bounds}. Depending on the size of the nodes and the number of parallel edges between
     * them, the distance between parallel edges is adjusted. This will keep the edges straight in most cases.
     * </p>
     * <p>
     * It is possible to select
     * {@link yfiles.router.ParallelEdgeRouter#LEADING_EDGE_DP_KEY custom master edges}. All edges that are parallel to the selected edges will be temporarily removed.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface ParallelEdgeRouter extends yfiles.layout.LayoutStageBase{}
    export class ParallelEdgeRouter {
      /**
       * Creates a new {@link yfiles.router.ParallelEdgeRouter} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] The core layout algorithm.
       * @see yfiles.layout.LayoutStageBase#coreLayout
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Hides all parallel edges leaving a master edge in the graph.
       * <p>
       * {@link yfiles.router.ParallelEdgeRouter} detects parallel edges of the given graph. From each set of parallel edges it hides all
       * but one edge from the given graph.
       * </p>
       * <p>
       * This method is called before invoking the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to change the selection of leading
       * edges. Hidden edges should be stored in {@link yfiles.router.ParallelEdgeRouter#hiddenEdges} to enable un-hiding later on.
       * </p>
       * @param graph the input graph
       * @see yfiles.router.ParallelEdgeRouter#directedMode
       * @see yfiles.router.ParallelEdgeRouter#AFFECTED_EDGES_DP_KEY
       * @protected
       */
      findAndHideParallelEdges(graph:yfiles.algorithms.Graph):void;
      /**
       * Assigns a layout to all parallel edges.
       * <p>
       * The layout for each parallel edge follows the layout of the master edge which was not removed from the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * </p>
       * <p>
       * This method is called when the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} is finished. It may be overridden to introduce a custom routing for
       * parallel edges.
       * </p>
       * @param graph the graph after the core layout
       * @param parallelEdges the map that provides all parallel edges for each master edge
       * @see yfiles.router.ParallelEdgeRouter#joinEnds
       * @see yfiles.router.ParallelEdgeRouter#adaptiveLineDistances
       * @see yfiles.router.ParallelEdgeRouter#adjustLeadingEdge
       * @protected
       */
      layoutParallelEdges(graph:yfiles.layout.LayoutGraph,parallelEdges:yfiles.algorithms.IEdgeMap):void;
      /**
       * Data provider key for marking edges that will be routed
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Data provider key for specifying the master edges
       * <p>
       * Marked edges won't be removed for the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. All edges parallel to the master edges will get parallel paths.
       * </p>
       * <p>
       * If no leading edge is specified for a bundle of parallel edges, it will be determined automatically. If there is more
       * than one leading edge, the first one is chosen.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static LEADING_EDGE_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Holds the list of parallel edges that are hidden from the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @protected
       * @type {yfiles.algorithms.EdgeList}
       */
      hiddenEdges:yfiles.algorithms.EdgeList;
      /**
       * {@link yfiles.algorithms.IEdgeMap} that associates a hidden edge with the unique parallel edge not hidden from the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @type {yfiles.algorithms.IEdgeMap}
       */
      parallelEdges:yfiles.algorithms.IEdgeMap;
      /**
       * Gets or sets whether or not the direction of edges should be considered.
       * <p>
       * In directed mode, only edges that share the same source and target node will be routed in parallel. Edges that connect
       * to the same nodes but in different directions won't be considered parallel.
       * </p>
       * <p>
       * In undirected mode, all edges connecting the same pair of nodes will be routed in parallel.
       * </p>
       * @type {boolean}
       */
      directedMode:boolean;
      /**
       * Gets or sets whether or not the distances between parallel edges should be determined automatically according to the
       * sizes of their source and target nodes.
       * <p>
       * If enabled, the distances between edges are chosen such that all parallel edges can be drawn as straight lines and still
       * connect to the nodes' bounds.
       * </p>
       * @see yfiles.router.ParallelEdgeRouter#lineDistance
       * @type {boolean}
       */
      adaptiveLineDistances:boolean;
      /**
       * Gets or sets the distance between two adjacent edge paths that run in parallel.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is less than <code>0</code>
       * @see yfiles.router.ParallelEdgeRouter#adaptiveLineDistances
       * @type {number}
       */
      lineDistance:number;
      /**
       * Gets or sets whether or not to join end points of parallel edges.
       * <p>
       * If enabled, all edges end with non-parallel segments that connect at the same point to the node. Otherwise, all segments
       * of the edges are parallel. In this case, some edges may not connect to the nodes' bounds.
       * </p>
       * @see yfiles.router.ParallelEdgeRouter#absJoinEndDistance
       * @see yfiles.router.ParallelEdgeRouter#relJoinEndDistance
       * @type {boolean}
       */
      joinEnds:boolean;
      /**
       * Gets or sets the absolute distance from the end point of the joined lines to the first parallel segments.
       * <p>
       * The absolute distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @see yfiles.router.ParallelEdgeRouter#joinEnds
       * @see yfiles.router.ParallelEdgeRouter#relJoinEndDistance
       * @type {number}
       */
      absJoinEndDistance:number;
      /**
       * Gets or sets the relative distance from the end point of the joined lines to the first parallel segments.
       * <p>
       * The length is relative to the corresponding original segment's length. It needs to have a value in the interval <code>[0,1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified relative distance is less than <code>0</code> or greater than <code>1</code>
       * @see yfiles.router.ParallelEdgeRouter#joinEnds
       * @see yfiles.router.ParallelEdgeRouter#absJoinEndDistance
       * @type {number}
       */
      relJoinEndDistance:number;
      /**
       * Gets or sets whether or not to adjust the leading edge to obtain more symmetric results.
       * @type {boolean}
       */
      adjustLeadingEdge:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum Scope{
      /**
       * A scope specifier which defines that all edges of the input graph will be routed.
       * @see yfiles.router.BusRouter#scope
       */
      ROUTE_ALL_EDGES,
      /**
       * A scope specifier which defines that only the selected edges of the input graph will be routed.
       * <p>
       * The selection state of an edge is determined by a boolean value returned by a {@link yfiles.algorithms.IDataProvider} registered
       * with key {@link yfiles.router.BusRouter#affectedEdgesDpKey}.
       * </p>
       * <p>
       * All other non-selected edges will be considered to have fixed routes.
       * </p>
       * @see yfiles.router.BusRouter#scope
       * @see yfiles.router.BusRouter#affectedEdgesDpKey
       * @see yfiles.router.BusRouter#DEFAULT_AFFECTED_EDGES_DP_KEY
       */
      ROUTE_AFFECTED_EDGES,
      /**
       * A scope specifier which defines that only edges incident to selected nodes will be routed.
       * <p>
       * The selection state of a node is determined by a boolean value returned by a {@link yfiles.algorithms.IDataProvider} registered
       * with key {@link yfiles.router.EdgeRouter#affectedNodesDpKey}.
       * </p>
       * <p>
       * All other edges that are incident to non-selected nodes will be considered to have fixed routes.
       * </p>
       * @see yfiles.router.EdgeRouter#scope
       * @see yfiles.router.EdgeRouter#affectedNodesDpKey
       */
      ROUTE_EDGES_AT_AFFECTED_NODES
    }
    export enum MonotonicPathRestriction{
      /**
       * A constant specifying that there are no monotonic edge path restrictions.
       * @see yfiles.router.EdgeLayoutDescriptor#monotonicPathRestriction
       */
      NONE,
      /**
       * A constant specifying monotonic edge path restrictions for the vertical direction.
       * <p>
       * This implies that each vertical edge segment is directed from the source to the target.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#monotonicPathRestriction
       */
      VERTICAL,
      /**
       * A constant specifying monotonic edge path restrictions for the horizontal direction.
       * <p>
       * This implies that each horizontal edge segment is directed from the source to the target.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#monotonicPathRestriction
       */
      HORIZONTAL,
      /**
       * A constant specifying monotonic edge path restrictions for the horizontal and vertical direction.
       * <p>
       * This implies that each horizontal as well as each vertical edge segment is directed from the source to the target.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#monotonicPathRestriction
       */
      BOTH
    }
    /**
     * This edge routing algorithm routes edges as straight-lines considering the specified {@link yfiles.layout.PortConstraint}s.
     * <p>
     * <b>Layout Style</b> The edges connect to their ports with straight-lines. This may lead to edges crossing nodes. {@graph {"ann":{"s":[30,30],"d":1},"n":[[591,590],[157,454],[558,361],[665,476],[466,502],[335,663],[230,267],[350,325],[17,519],[187,590],[248,387],[145,310],[464,337],[310,506],[670,151],[220,106],[697,285],[522,187],[457,2],[111,0],[305,5],[375,139],[0,260]],"e":[[0,3],[3,2],[4,0],[1,6],[6,7],[7,2],[1,8],[1,9],[10,7],[10,11],[11,7],[12,4],[12,7],[5,13],[9,13],[1,13],[7,13],[4,13],[15,6],[14,16],[16,2],[12,17],[17,14],[15,19],[15,20],[18,21],[20,21],[15,21],[17,21],[7,21],[11,22]],"vp":[0.0,0.0,727.0,693.0]}}<b>Concept</b>{@link yfiles.router.StraightLineEdgeRouter}
     * will first invoke the {@link yfiles.layout.LayoutStageBase#coreLayout} if one exists. Then, it will remove all bends from the edges and will place the ports either on
     * the center of the nodes or at a location specified by a {@link yfiles.layout.PortConstraint}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface StraightLineEdgeRouter extends yfiles.layout.LayoutStageBase{}
    export class StraightLineEdgeRouter {
      /**
       * Creates a new instance of {@link yfiles.router.StraightLineEdgeRouter} with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The core layout algorithm.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Gets or sets the (sub-)set of edges to be routed.
       * <p>
       * {@link yfiles.router.StraightLineEdgeRouter} will only route those edges and leave the remaining edges unchanged.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given argument is an unknown specifier
       * @see yfiles.router.StraightLineEdgeRouter#affectedEdgesDpKey
       * @see yfiles.router.StraightLineEdgeRouter#affectedNodesDpKey
       * @type {yfiles.router.Scope}
       */
      scope:yfiles.router.Scope;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that provides the selected state of the nodes of the
       * graph.
       * <p>
       * If the scope only includes {@link yfiles.router.Scope#ROUTE_EDGES_AT_AFFECTED_NODES edges at selected nodes}, only edges of the nodes marked in this {@link yfiles.algorithms.IDataProvider}
       * are routed while all other edges are considered to have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @see yfiles.router.StraightLineEdgeRouter#scope
       * @type {Object}
       */
      affectedNodesDpKey:Object;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that provides the selected state of the edges of the
       * graph.
       * <p>
       * If the scope only includes {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES selected edges}, only edges of the nodes marked in this {@link yfiles.algorithms.IDataProvider} are
       * routed while all other edges are considered to have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @see yfiles.router.StraightLineEdgeRouter#scope
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class describes the location and the direction from which a given {@link yfiles.router.PartitionCell partition cell} has been entered during a path
     * search.
     * <p>
     * The path that was followed in order to reach a given {@link yfiles.router.PartitionCell partition cell} can be reconstructed by traversing all
     * {@link yfiles.router.CellEntrance#previousEntrance previous entrances}. The {@link yfiles.router.CellEntrance#enterInterval enter interval} and {@link yfiles.router.CellEntrance#enterDirection enter direction} determine how the {@link yfiles.router.PartitionCell partition cell} was entered.
     * </p>
     * @class
     */
    export interface CellEntrance extends Object{}
    export class CellEntrance {
      /**
       * Creates a new {@link yfiles.router.CellEntrance} instance that is a copy of a given other {@link yfiles.router.CellEntrance}.
       * @param other the {@link } to be copied
       */
      constructor(other:yfiles.router.CellEntrance);
      /**
       * Creates a new {@link yfiles.router.CellEntrance} instance for a given {@link yfiles.router.PartitionCell}.
       * @param cell the {@link } that is entered
       */
      constructor(cell:yfiles.router.PartitionCell);
      /**
       * Gets the {@link yfiles.router.PartitionCell partition cell} of this {@link yfiles.router.CellEntrance} instance.
       * @type {yfiles.router.PartitionCell}
       */
      cell:yfiles.router.PartitionCell;
      /**
       * Gets or sets the previous {@link yfiles.router.CellEntrance} along the path that was traversed in order to reach the {@link yfiles.router.CellEntrance#cell cell}.
       * @type {yfiles.router.CellEntrance}
       */
      previousEntrance:yfiles.router.CellEntrance;
      /**
       * Gets or sets the {@link yfiles.router.OrthogonalInterval} that was used for entering this {@link yfiles.router.CellEntrance#cell cell}.
       * @type {yfiles.router.OrthogonalInterval}
       */
      enterInterval:yfiles.router.OrthogonalInterval;
      /**
       * Gets or sets the direction from which this {@link yfiles.router.CellEntrance#cell cell} was entered.
       * @type {yfiles.layout.Direction}
       */
      enterDirection:yfiles.layout.Direction;
      /**
       * Gets or sets the costs for the implicit path given by the {@link yfiles.router.CellEntrance#previousEntrance previous entrances}.
       * <p>
       * The costs must be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the costs are negative
       * @type {number}
       */
      costs:number;
      /**
       * Gets or sets the heuristic costs for the implicit path given by the {@link yfiles.router.CellEntrance#previousEntrance previous entrances}.
       * <p>
       * Heuristic costs must be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the heuristic costs are negative
       * @type {number}
       */
      heuristicCosts:number;
      /**
       * Gets or sets the {@link yfiles.router.EdgeCellInfo} describing how the previous {@link yfiles.router.PartitionCell partition cell} was traversed.
       * @type {yfiles.router.EdgeCellInfo}
       */
      previousEdgeCellInfo:yfiles.router.EdgeCellInfo;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class stores information about the part of an edge segment that lies inside a specified {@link yfiles.router.PartitionCell partition cell}.
     * @class
     * @extends {yfiles.router.SegmentInfoBase}
     */
    export interface CellSegmentInfo extends yfiles.router.SegmentInfoBase{}
    export class CellSegmentInfo {
      /**
       * Creates a new {@link yfiles.router.CellSegmentInfo} instance.
       * @param edge the edge to which the segment of this info belongs
       * @param segmentIndex the index of the segment to which this info belongs
       * @param direction the direction to which the segment of this info points
       * @param locationRange the range within which the common location of this segment has to lie
       * @param minExtension the minimum interval in extension direction this segment part is known to intersect
       * @param maxExtension the maximum interval in extension direction this segment part will span
       * @param cell the partition cell within which this part of the edge segment lies
       */
      constructor(edge:yfiles.algorithms.Edge,segmentIndex:number,direction:yfiles.layout.Direction,locationRange:yfiles.router.Interval,minExtension:yfiles.router.Interval,maxExtension:yfiles.router.Interval,cell:yfiles.router.PartitionCell);
      /**
       * Gets the direction to which the segment of this info points.
       * @type {yfiles.layout.Direction}
       */
      direction:yfiles.layout.Direction;
      /**
       * Gets the {@link yfiles.router.PartitionCell partition cell} within which this part of the edge segment lies.
       * @type {yfiles.router.PartitionCell}
       */
      cell:yfiles.router.PartitionCell;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class describes a channel, i.e., a set of vertical or horizontal edge segments whose locations depend on each
     * other.
     * <p>
     * Channels are used by the {@link yfiles.router.ChannelBasedPathRouting} to route edges using
     * {@link yfiles.router.SegmentInfo}s describing their path.
     * </p>
     * @class
     */
    export interface Channel extends Object{}
    export class Channel {
      /**
       * Creates a new {@link yfiles.router.Channel} that contains the given {@link yfiles.router.SegmentInfo segment info}.
       * @param segment the given segment info for which the {@link } is created
       * @throws {Stubs.Exceptions.ArgumentError} if the given segment has a different orientation than the other segments already contained in the channel
       */
      constructor(segment:yfiles.router.SegmentInfo);
      /**
       * Adds a new {@link yfiles.router.SegmentInfo segment info} to the {@link yfiles.router.Channel channel}.
       * @param segment the segment info to be added to this channel
       * @throws {Stubs.Exceptions.ArgumentError} if the given segment has a different orientation than the other segments already contained in the channel
       */
      addSegment(segment:yfiles.router.SegmentInfo):void;
      /**
       * Adds a {@link yfiles.router.SegmentGroup segment group} to this {@link yfiles.router.Channel channel}.
       * @param group the segment group to be added
       */
      addSegmentGroup(group:yfiles.router.SegmentGroup):void;
      /**
       * Returns the current location that the given {@link yfiles.router.SegmentInfo segment info} has been assigned within its {@link yfiles.router.SegmentInfoBase#locationRange location range}.
       * @param segmentInfo the segment info for which the current location will be returned
       * @returns the current location of the segment info or {@link #NaN} if no location has been assigned before
       * @see yfiles.router.Channel#setCurrentLocation
       */
      getCurrentLocation(segmentInfo:yfiles.router.SegmentInfo):number;
      /**
       * Returns the {@link yfiles.router.SegmentInfo segment info} with the given index in the list of segments of this {@link yfiles.router.Channel channel}.
       * @param index the index of the segment in the list of segments
       * @returns the {@link  segment info} with the given index in the list of segments
       */
      getSegment(index:number):yfiles.router.SegmentInfo;
      /**
       * Returns the {@link yfiles.router.SegmentGroup segment group} with the given index in this {@link yfiles.router.Channel channel}.
       * @param index the index of the segment group
       * @returns the {@link  segment group} with the given index in this channel
       */
      getSegmentGroup(index:number):yfiles.router.SegmentGroup;
      /**
       * Returns the number of {@link yfiles.router.SegmentInfo segment infos} in this channel.
       * @returns the number of segment infos in this channel
       */
      segmentCount():number;
      /**
       * Returns the number of {@link yfiles.router.SegmentGroup segment groups} in this {@link yfiles.router.Channel channel}.
       * @returns the number of segment groups in this channel
       */
      segmentGroupCount():number;
      /**
       * Specifies the current location that the given {@link yfiles.router.SegmentInfo segment info} has been assigned within its {@link yfiles.router.SegmentInfoBase#locationRange location range}.
       * <p>
       * During the distribution of the segments in the channel, different locations can be assigned but only the last one is
       * finally assigned to the {@link yfiles.router.SegmentInfo segment info}.
       * </p>
       * @param segmentInfo the segment info for which the current location will be set
       * @param location the location of the segment info within its location range
       * @see yfiles.router.SegmentInfoBase#location
       */
      setCurrentLocation(segmentInfo:yfiles.router.SegmentInfo,location:number):void;
      /**
       * Gets an {@link yfiles.router.Interval interval} spanning the union of the location ranges of the {@link yfiles.router.SegmentInfo segment infos}.
       * @see yfiles.router.SegmentInfoBase#locationRange
       * @type {yfiles.router.Interval}
       */
      widthInterval:yfiles.router.Interval;
      /**
       * Gets an {@link yfiles.router.Interval interval} spanning the union of the maximum extension of the {@link yfiles.router.SegmentInfo segment infos}.
       * @see yfiles.router.SegmentInfoBase#maxExtension
       * @type {yfiles.router.Interval}
       */
      lengthInterval:yfiles.router.Interval;
      /**
       * Gets whether or not the segments in this {@link yfiles.router.Channel channel} are vertical.
       * @type {boolean}
       */
      vertical:boolean;
      /**
       * Gets the bounds of this {@link yfiles.router.Channel channel}.
       * <p>
       * These bounds are determined by the channel's {@link yfiles.router.Channel#widthInterval width} and {@link yfiles.router.Channel#lengthInterval length}.
       * </p>
       * @see yfiles.router.Channel#widthInterval
       * @see yfiles.router.Channel#lengthInterval
       * @type {yfiles.algorithms.YRectangle}
       */
      bounds:yfiles.algorithms.YRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class routes edges using their pre-calculated {@link yfiles.router.Path} information.
     * <p>
     * The concept of this algorithm is to bundle edge {@link yfiles.router.SegmentInfo segment}s into so-called {@link yfiles.router.Channel}s and route each {@link yfiles.router.Channel}
     * individually.
     * </p>
     * <p>
     * This class is initialized using {@link yfiles.router.ChannelBasedPathRouting#initialize} which must have been called before the
     * edges are {@link yfiles.router.ChannelBasedPathRouting#route routed}.
     * </p>
     * <p>
     * After a {@link yfiles.router.ChannelBasedPathRouting} instance is no longer in use, the {@link yfiles.router.Channel} information and
     * the reference to the {@link yfiles.router.PathSearchConfiguration configuration} must be reset using method {@link yfiles.router.ChannelBasedPathRouting#cleanup}.
     * </p>
     * @see yfiles.router.EdgeInfo#getSegmentInfo
     * @see yfiles.layout.LayoutGraph#setPath
     * @class
     */
    export interface ChannelBasedPathRouting extends Object{}
    export class ChannelBasedPathRouting {
      /**
       * Creates a new instance of {@link yfiles.router.ChannelBasedPathRouting}.
       */
      constructor();
      /**
       * Adjusts the location assigned to a {@link yfiles.router.SegmentInfo segment info} in the {@link yfiles.router.Channel channel} considering the preferred alignment of the segment.
       * <p>
       * This method is called after the segments have been distributed in the channel but before the final locations of the
       * segments are assigned to the {@link yfiles.router.SegmentInfo segment info}s.
       * </p>
       * @param channel the channel that contains the segment infos for which the alignment should be considered
       * @see yfiles.router.Channel#getCurrentLocation
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       * @protected
       */
      adjustSegmentLocations(channel:yfiles.router.Channel):void;
      /**
       * Calculates the locations of the {@link yfiles.router.SegmentInfo segment infos} in the given {@link yfiles.router.Channel channel}.
       * <p>
       * This method is called after the order of the {@link yfiles.router.SegmentInfo segment infos} within a channel has been {@link yfiles.router.ChannelBasedPathRouting#optimizeSegmentOrder optimized}.
       * </p>
       * @param channel the channel that contains the segment infos for which the locations will be calculated
       * @see yfiles.router.Channel#setCurrentLocation
       * @protected
       */
      calculateSegmentLocations(channel:yfiles.router.Channel):void;
      /**
       * Resets all {@link yfiles.router.Channel channel} information and the reference to the {@link yfiles.router.PathSearchConfiguration configuration}.
       * <p>
       * Afterwards, {@link yfiles.router.ChannelBasedPathRouting} is ready to calculate paths for a new layout.
       * </p>
       */
      cleanup():void;
      /**
       * Creates a {@link yfiles.collections.IComparer.<T> comparator} for sorting the {@link yfiles.router.SegmentInfo}s in a {@link yfiles.router.Channel} from left to right (for
       * vertical segments) or top to bottom (for horizontal segments), respectively.
       * <p>
       * This method is called in {@link yfiles.router.ChannelBasedPathRouting#route} before edge segments are distributed within the
       * channels.
       * </p>
       * @param channels the channels whose segments will be sorted by the {@link  comparator}
       * @param pathSearchResult the {@link  path search result} containing the {@link }s with the
       * {@link }s for which the comparator will be used
       * @param configuration the {@link  configuration} used during this path search
       * @returns a {@link  comparator} used for sorting the {@link }s in a {@link  channel}
       * @protected
       */
      createSegmentInfoComparer(channels:yfiles.algorithms.YList,pathSearchResult:yfiles.router.PathSearchResult,configuration:yfiles.router.PathSearchConfiguration):yfiles.collections.IComparer<Object>;
      /**
       * Initializes this {@link yfiles.router.ChannelBasedPathRouting} instance for routing paths that have been found with the given
       * {@link yfiles.router.PathSearchConfiguration configuration}.
       * <p>
       * This method must be called before the edges are {@link yfiles.router.ChannelBasedPathRouting#route routed}.
       * </p>
       * @param configuration the {@link  configuration} used for the following path routing
       * @see yfiles.router.ChannelBasedPathRouting#route
       */
      initialize(configuration:yfiles.router.PathSearchConfiguration):void;
      /**
       * Optimizes the order of the {@link yfiles.router.SegmentInfo segments} in the given {@link yfiles.router.Channel channel}.
       * <p>
       * After the segments in the channel have been sorted using the comparator created by method
       * {@link yfiles.router.ChannelBasedPathRouting#createSegmentInfoComparer}, segments having a common {@link yfiles.router.SegmentGroup} are
       * replaced by their common {@link yfiles.router.SegmentGroup#commonSegmentInfo representative segment info}.
       * </p>
       * <p>
       * This method further improves the order of the resulting segment info list to make sure that the subsequent {@link yfiles.router.ChannelBasedPathRouting#calculateSegmentLocations location calculation}
       * is able to respect the {@link yfiles.router.SegmentInfoBase#locationRange location ranges} of the
       * {@link yfiles.router.SegmentInfo}s and, if possible, the minimum edge to edge distances and possible grid constraints for the
       * segments.
       * </p>
       * @param channel the channel for which the order of the {@link }s is optimized
       * @protected
       */
      optimizeSegmentOrder(channel:yfiles.router.Channel):void;
      /**
       * Routes the given edges.
       * <p>
       * Before calling this method, method {@link yfiles.router.ChannelBasedPathRouting#initialize} must have been called.
       * </p>
       * <p>
       * As a first step, the {@link yfiles.router.SegmentInfo}s of all edge paths are distributed to {@link yfiles.router.Channel}s. After that,
       * the segments in each of these
       * {@link yfiles.router.Channel}s are sorted and locations are assigned respecting the segments' location ranges by calling the
       * subsequent methods:
       * </p>
       * <ol>
       * <li>{@link yfiles.router.ChannelBasedPathRouting#optimizeSegmentOrder}</li>
       * <li>{@link yfiles.router.ChannelBasedPathRouting#calculateSegmentLocations}</li>
       * <li>{@link yfiles.router.ChannelBasedPathRouting#adjustSegmentLocations}</li>
       * </ol>
       * <p>
       * Finally, the control points for the paths are calculated.
       * </p>
       * @param edges the {@link  list of edges} to be routed
       * @param pathSearchResult the {@link  path search result} containing the {@link } objects for the edges
       * @throws {Stubs.Exceptions.InvalidOperationError} if the locations are not within the segment's location range
       * @see yfiles.router.PathSearchResult#getPath
       * @see yfiles.router.SegmentInfoBase#locationRange
       * @see yfiles.router.SegmentInfoBase#location
       */
      route(edges:yfiles.algorithms.EdgeList,pathSearchResult:yfiles.router.PathSearchResult):void;
      /**
       * Sorts the given list of {@link yfiles.router.SegmentInfo}s using the given {@link yfiles.collections.IComparer.<T> comparator}.
       * <p>
       * This method is called in {@link yfiles.router.ChannelBasedPathRouting#route} when the edge segments are distributed within the
       * channels.
       * </p>
       * @param segmentInfos the list of {@link }s to be sorted
       * @param segmentInfoComparer the comparator used for sorting the list of {@link }s
       * @protected
       */
      sortSegmentInfos(segmentInfos:yfiles.collections.IList<Object>,segmentInfoComparer:yfiles.collections.IComparer<Object>):void;
      /**
       * Gets the {@link yfiles.router.PathSearchConfiguration path search configuration} with which the {@link yfiles.router.ChannelBasedPathRouting} has been {@link yfiles.router.ChannelBasedPathRouting#initialize initialized}.
       * @see yfiles.router.ChannelBasedPathRouting#initialize
       * @protected
       * @type {yfiles.router.PathSearchConfiguration}
       */
      configuration:yfiles.router.PathSearchConfiguration;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class describes an {@link yfiles.router.IObstaclePartition} that decomposes its area dynamically.
     * <p>
     * The partitioning strategy is based on binary space partitioning. It divides the partition space recursively in two {@link yfiles.router.PartitionCell cells}
     * until each cell is completely covered by one or more {@link yfiles.router.Obstacle}s or completely empty.
     * </p>
     * @class
     * @implements {yfiles.router.IObstaclePartition}
     * @implements {yfiles.router.IDynamicDecomposition}
     */
    export interface DynamicObstacleDecomposition extends Object,yfiles.router.IObstaclePartition,yfiles.router.IDynamicDecomposition{}
    export class DynamicObstacleDecomposition {
      /**
       * Constructs a new instance of {@link yfiles.router.DynamicObstacleDecomposition}.
       */
      constructor();
      /**
       * Adds the given {@link yfiles.router.IDecompositionListener dynamic decomposition listener} to receive {@link yfiles.router.PartitionCell} subdivision and creation events
       * from this decomposition.
       * <p>
       * These events occur when the decomposition changes the partition by subdividing cells into sub-cells or when new cells
       * are created.
       * </p>
       * @param listener the dynamic decomposition listener to add
       * @see yfiles.router.IDecompositionListener
       */
      addDynamicDecompositionListener(listener:yfiles.router.IDecompositionListener):void;
      /**
       * Clears the partition data such that the {@link yfiles.router.DynamicObstacleDecomposition} can be reused and {@link yfiles.router.DynamicObstacleDecomposition#init initialized} with
       * new {@link yfiles.router.Obstacle}s.
       * @see yfiles.router.DynamicObstacleDecomposition#init
       */
      clear():void;
      /**
       * Notifies all registered {@link yfiles.router.IDecompositionListener dynamic decomposition listeners} that the given {@link yfiles.router.PartitionCell partition cell} has been created.
       * <p>
       * This method is also called in {@link yfiles.router.DynamicObstacleDecomposition#init}.
       * </p>
       * @param createdCell the newly created cell
       * @see yfiles.router.IDecompositionListener
       * @protected
       */
      fireCreateCellEvent(createdCell:yfiles.router.PartitionCell):void;
      /**
       * Notifies all registered {@link yfiles.router.IDecompositionListener dynamic decomposition listeners} that the given {@link yfiles.router.PartitionCell partition cell} has been finalized.
       * @param finalizedCell the cell that has been finalized
       * @see yfiles.router.IDecompositionListener
       * @protected
       */
      fireFinalizeCellEvent(finalizedCell:yfiles.router.PartitionCell):void;
      /**
       * Notifies all registered {@link yfiles.router.IDecompositionListener dynamic decomposition listeners} of a subdivision of a given {@link yfiles.router.PartitionCell partition cell}.
       * @param cell the cell that has been subdivided
       * @param subCells the new sub-cells resulting from the subdivision of the given cell
       * @see yfiles.router.IDecompositionListener
       * @protected
       */
      fireSubdividedEvent(cell:yfiles.router.PartitionCell,subCells:yfiles.collections.IList<Object>):void;
      /**
       * Returns a list of all {@link yfiles.router.PartitionCell}s that intersect or cover the given rectangle.
       * @param rect the rectangular area whose (partially) covered cells will be returned
       * @returns a list of {@link }s that (partially) cover the given rectangular area
       */
      getCells(rect:yfiles.algorithms.YRectangle):yfiles.collections.IList<Object>;
      /**
       * Returns all {@link yfiles.router.PartitionCell partition cells} that are completely covered by the given {@link yfiles.router.Obstacle}.
       * @param obstacle the obstacle for which the covered cells will be returned
       * @returns an unmodifiable list of {@link } instances that are completely covered by the given obstacle
       */
      getCellsForObstacle(obstacle:yfiles.router.Obstacle):yfiles.collections.IList<Object>;
      /**
       * Calculates the cost of a cut with respect to the geometry of the sub-cells.
       * <p>
       * The cost can take values between <code>0</code> and <code>1</code>.
       * </p>
       * <p>
       * This method is called while a {@link yfiles.router.PartitionCell} is divided into upper and lower or left and right child cells
       * depending on the cut costs (during {@link yfiles.router.DynamicObstacleDecomposition#getCellsForObstacle}, {@link yfiles.router.DynamicObstacleDecomposition#getCells}
       * and {@link yfiles.router.DynamicObstacleDecomposition#getNeighbors} methods).
       * </p>
       * @param cut the coordinate of the cut
       * @param min the left side of the subdivided cell
       * @param max the right side of the subdivided cell
       * @param orthogonalMin the upper side of the subdivided cell
       * @param orthogonalMax the lower side of the subdivided cell
       * @returns the cost of a cut with respect to the geometry of the sub-cells
       * @protected
       */
      getGeometricCutCosts(cut:number,min:number,max:number,orthogonalMin:number,orthogonalMax:number):number;
      /**
       * Returns the neighbor {@link yfiles.router.PartitionCell partition cells} of the given cell.
       * @param cell the cell whose neighbors will be returned
       * @returns the neighbor cells of the given cell
       */
      getNeighbors(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Calculates the cost of a cut with respect to the subdivided obstacles.
       * <p>
       * The cost can take values between <code>0</code> and <code>1</code>.
       * </p>
       * <p>
       * This method is called while a {@link yfiles.router.PartitionCell} is divided into upper and lower or left and right child cells
       * depending on the cut costs (during {@link yfiles.router.DynamicObstacleDecomposition#getCellsForObstacle}, {@link yfiles.router.DynamicObstacleDecomposition#getCells}
       * and {@link yfiles.router.DynamicObstacleDecomposition#getNeighbors} methods).
       * </p>
       * @param numObstaclesInFirstHalf the number of obstacles that lie completely in the first half
       * @param numObstaclesInSecondHalf the number of obstacles that lie completely in the second half
       * @param numObstaclesOnCut the number of obstacles that lie on the cut
       * @returns the cost of a cut with respect to the subdivided obstacles
       * @protected
       */
      getObstacleCutCosts(numObstaclesInFirstHalf:number,numObstaclesInSecondHalf:number,numObstaclesOnCut:number):number;
      /**
       * Returns all {@link yfiles.router.Obstacle}s that cover the given {@link yfiles.router.PartitionCell partition cell}.
       * @param cell the partition cell for which the obstacles will be returned
       * @returns an unmodifiable list of {@link } instances that cover the given cell
       */
      getObstacles(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Initializes this {@link yfiles.router.DynamicObstacleDecomposition} instance with the given obstacles and partition bounds.
       * <p>
       * This method must be called before any other method is invoked.
       * </p>
       * @param obstacles a list of {@link } objects
       * @param partitionBounds the bounds of the partition
       */
      init(obstacles:yfiles.collections.IList<Object>,partitionBounds:yfiles.algorithms.YRectangle):void;
      /**
       * Removes the given {@link yfiles.router.IDecompositionListener dynamic decomposition listener} such that it no longer receives {@link yfiles.router.PartitionCell} subdivision
       * and creation events from this decomposition.
       * @param listener the dynamic decomposition listener to remove
       * @see yfiles.router.IDecompositionListener
       */
      removeDynamicDecompositionListener(listener:yfiles.router.IDecompositionListener):void;
      /**
       * Gets or sets the costs incurred for every {@link yfiles.router.Obstacle} that must be cut in a subdivision.
       * <p>
       * Values need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the cost is negative
       * @type {number}
       */
      cutObstacleCost:number;
      /**
       * Gets or sets the costs incurred if the distribution after a subdivision of obstacles is unbalanced in sub-cells.
       * <p>
       * Values need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the cost is negative
       * @type {number}
       */
      unbalancedObstaclesCost:number;
      /**
       * Gets or sets the costs incurred if the subdivision produces unbalanced rectangles.
       * <p>
       * Values need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the cost is negative
       * @type {number}
       */
      unbalancedRatioCost:number;
      /**
       * Gets the bounds of the original rectangular area that is being partitioned.
       * @type {yfiles.algorithms.YRectangle}
       */
      bounds:yfiles.algorithms.YRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class encapsulates the information that determines the route of a specific edge within a specific {@link yfiles.router.PartitionCell partition cell}.
     * @class
     */
    export interface EdgeCellInfo extends Object{}
    export class EdgeCellInfo {
      /**
       * Creates a new {@link yfiles.router.EdgeCellInfo} instance.
       * @param edge the edge whose routing in the cell is described
       * @param cell the cell in which the edge is routed
       * @param enterInterval the interval that the edge uses to enter the cell
       * @param exitInterval the interval that the edge uses to exit the cell
       * @param enterDirection the direction in which the edge enters the cell
       * @param exitDirection the direction in which the edge exits the cell
       * @param enterSegmentNo the index of the edge segment that enters the cell
       */
      constructor(edge:yfiles.algorithms.Edge,cell:yfiles.router.PartitionCell,enterInterval:yfiles.router.OrthogonalInterval,exitInterval:yfiles.router.OrthogonalInterval,enterDirection:yfiles.layout.Direction,exitDirection:yfiles.layout.Direction,enterSegmentNo:number);
      /**
       * Gets the number of bends of the edge within this cell.
       * @type {number}
       */
      bendCount:number;
      /**
       * Gets the routing type of the edge within this cell.
       * @type {yfiles.router.RoutingType}
       */
      type:yfiles.router.RoutingType;
      /**
       * Gets or sets the index of the edge segment that enters this cell.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given index is negative
       * @type {number}
       */
      enterSegmentIndex:number;
      /**
       * Gets or sets the index of the edge segment that exits this cell.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given index is negative
       * @type {number}
       */
      exitSegmentIndex:number;
      /**
       * Gets or sets the {@link yfiles.router.SegmentGroup segment group} containing the entering segment.
       * <p>
       * When specifying the segment group, the {@link yfiles.router.EdgeCellInfo#enterInterval enter interval} is updated to use the segment group's common enter interval.
       * </p>
       * @see yfiles.router.SegmentGroup#commonLocationRange
       * @type {yfiles.router.SegmentGroup}
       */
      enterSegmentGroup:yfiles.router.SegmentGroup;
      /**
       * Gets or sets the {@link yfiles.router.SegmentGroup segment group} containing the exiting segment.
       * <p>
       * When specifying the segment group, the {@link yfiles.router.EdgeCellInfo#exitInterval exit interval} is updated to use the segment group's common exit interval.
       * </p>
       * @see yfiles.router.SegmentGroup#commonLocationRange
       * @type {yfiles.router.SegmentGroup}
       */
      exitSegmentGroup:yfiles.router.SegmentGroup;
      /**
       * Gets the edge that is routed within the {@link yfiles.router.EdgeCellInfo#cell partition cell}.
       * @type {yfiles.algorithms.Edge}
       */
      edge:yfiles.algorithms.Edge;
      /**
       * Gets the {@link yfiles.router.PartitionCell partition cell} within which the edge is routed.
       * @type {yfiles.router.PartitionCell}
       */
      cell:yfiles.router.PartitionCell;
      /**
       * Gets the {@link yfiles.router.OrthogonalInterval interval} that the edge uses to enter this cell.
       * @type {yfiles.router.OrthogonalInterval}
       */
      enterInterval:yfiles.router.OrthogonalInterval;
      /**
       * Gets the {@link yfiles.router.OrthogonalInterval interval} that the edge uses to exit this cell.
       * @type {yfiles.router.OrthogonalInterval}
       */
      exitInterval:yfiles.router.OrthogonalInterval;
      /**
       * Gets the {@link yfiles.layout.Direction direction} in which the edge enters this cell.
       * @type {yfiles.layout.Direction}
       */
      enterDirection:yfiles.layout.Direction;
      /**
       * Gets the {@link yfiles.layout.Direction direction} in which the edge exits this cell.
       * @type {yfiles.layout.Direction}
       */
      exitDirection:yfiles.layout.Direction;
      /**
       * Calculates the list of {@link yfiles.router.CellSegmentInfo}s for the segment parts of this edge that lie within the associated
       * {@link yfiles.router.EdgeCellInfo#cell partition cell}.
       * @type {yfiles.algorithms.YList}
       */
      cellSegmentInfos:yfiles.algorithms.YList;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class encapsulates the information required to route an edge with a path routing algorithm.
     * @see yfiles.router.ChannelBasedPathRouting
     * @class
     */
    export interface EdgeInfo extends Object{}
    export class EdgeInfo {
      /**
       * Creates a new {@link yfiles.router.EdgeInfo} instance.
       * @param path the path that will be used for routing the edge
       * @param configuration the {@link  configuration} of the path search algorithm
       */
      constructor(path:yfiles.router.Path,configuration:yfiles.router.PathSearchConfiguration);
      /**
       * Calculates an array of {@link yfiles.algorithms.LineSegment}s created by the {@link yfiles.router.EdgeInfo#calculatePathPoints calculated path points}.
       * @returns an array of {@link }s created by the calculated path points
       */
      calculateLineSegments():yfiles.algorithms.LineSegment[];
      /**
       * Calculates a list of {@link yfiles.algorithms.YPoint}s containing the source port, bend and target port locations.
       * @returns a list of {@link }s containing the source port, bend and target port locations
       */
      calculatePathPoints():yfiles.algorithms.YList;
      /**
       * Returns the {@link yfiles.router.EdgeCellInfo} of the given {@link yfiles.router.CellEntrance} in the path.
       * @param entrance the entrance for which the {@link } will be returned
       * @returns the {@link } of the given {@link } in the path
       */
      getEdgeCellInfo(entrance:yfiles.router.CellEntrance):yfiles.router.EdgeCellInfo;
      /**
       * Returns the {@link yfiles.router.SegmentInfo} of the segment following the segment of the given {@link yfiles.router.SegmentInfo}.
       * @param segment the segment info for which the next segment info will be returned
       * @returns the segment info of the segment following the segment of the given segment info
       */
      getNextSegment(segment:yfiles.router.SegmentInfo):yfiles.router.SegmentInfo;
      /**
       * Returns the {@link yfiles.router.SegmentInfo} of the segment preceding the segment of the given {@link yfiles.router.SegmentInfo}.
       * @param segment the segment info for which the previous segment info will be returned
       * @returns the segment info of the segment preceding the segment of the given segment info
       */
      getPreviousSegment(segment:yfiles.router.SegmentInfo):yfiles.router.SegmentInfo;
      /**
       * Returns the {@link yfiles.router.SegmentInfo} of the edge segment with the given index.
       * @param segmentIndex the index of the edge segment for which the segment info will be returned
       * @returns the {@link } of the edge segment with the given index
       */
      getSegmentInfo(segmentIndex:number):yfiles.router.SegmentInfo;
      /**
       * Returns the number of segments of the edge.
       * @returns the number of segments of the edge
       */
      segmentCount():number;
      /**
       * Gets whether the path of this {@link yfiles.router.EdgeInfo#edge edge} is fixed or should be routed by the path search algorithm.
       * @type {boolean}
       */
      fixed:boolean;
      /**
       * Gets the edge whose route is described by this {@link yfiles.router.EdgeInfo} instance.
       * @type {yfiles.algorithms.Edge}
       */
      edge:yfiles.algorithms.Edge;
      /**
       * Gets a list of {@link yfiles.router.EdgeCellInfo}s describing how the edge traverses each {@link yfiles.router.PartitionCell} on its
       * path.
       * @type {yfiles.collections.IList.<Object>}
       */
      edgeCellInfos:yfiles.collections.IList<Object>;
      /**
       * Gets or sets the location of the strong source port that the edge uses.
       * <p>
       * Defining a new strong source port also updates the first {@link yfiles.router.SegmentInfo} to use this fixed port location.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the first segment is already fixed and the given port location contradicts the segment's fixed location
       * @see yfiles.router.SegmentInfoBase#location
       * @see yfiles.router.SegmentInfo#atStrongPortConstraint
       * @type {yfiles.algorithms.YPoint}
       */
      strongSourcePort:yfiles.algorithms.YPoint;
      /**
       * Gets or sets the location of the strong target port that the edge uses.
       * <p>
       * Defining a new strong target port also updates the last {@link yfiles.router.SegmentInfo} to use this fixed port location.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the last segment is already fixed and the given port location contradicts the segment's fixed location
       * @see yfiles.router.SegmentInfoBase#location
       * @see yfiles.router.SegmentInfo#atStrongPortConstraint
       * @type {yfiles.algorithms.YPoint}
       */
      strongTargetPort:yfiles.algorithms.YPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is used by {@link yfiles.router.EdgeRouter} to provide routing details for the edges of the graph.
     * <p>
     * An {@link yfiles.router.EdgeLayoutDescriptor} instance can be specified individually for single edges using a {@link yfiles.algorithms.IDataProvider}
     * that returns an {@link yfiles.router.EdgeLayoutDescriptor} instance for each edge of the graph, or <code>null</code> if no {@link yfiles.router.EdgeLayoutDescriptor}
     * is bound to an edge. The {@link yfiles.algorithms.IDataProvider} is registered with the graph using key
     * {@link yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
     * </p>
     * <p>
     * This class is designed such that it allows future additions of new getter methods.
     * </p>
     * @see yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
     * @class
     */
    export interface EdgeLayoutDescriptor extends Object{}
    export class EdgeLayoutDescriptor {
      /**
       * Creates a new instance of an {@link yfiles.router.EdgeLayoutDescriptor} with the default settings.
       */
      constructor();
      /**
       * Creates a copy of this {@link yfiles.router.EdgeLayoutDescriptor} instance.
       * @returns a copy of this {@link } instance
       */
      createCopy():yfiles.router.EdgeLayoutDescriptor;
      /**
       * Gets or sets the minimum length of the first segment of the edge path (at the source node).
       * <p>
       * The minimum length should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value, i.e.,
       * <code>5.0</code>, will be used instead.
       * </p>
       * @type {number}
       */
      minimumFirstSegmentLength:number;
      /**
       * Gets or sets the minimum length of the last segment of the edge path (at the target node).
       * <p>
       * The minimum length should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value, i.e.,
       * <code>10.0</code>, will be used instead.
       * </p>
       * @type {number}
       */
      minimumLastSegmentLength:number;
      /**
       * Gets or sets the minimum distance between a pair of edges.
       * <p>
       * The minimum distance should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value,
       * i.e., <code>3.0</code>, will be used instead.
       * </p>
       * @type {number}
       */
      minimumEdgeToEdgeDistance:number;
      /**
       * Gets or sets the minimum distance that the edge should maintain from node corners when entering or leaving the node.
       * <p>
       * The minimum distance should be greater than or equal to <code>0</code>. If a negative value is given as input, the default value,
       * i.e., <code>3.0</code>, will be used instead.
       * </p>
       * @type {number}
       */
      minimumNodeCornerDistance:number;
      /**
       * Gets or sets the monotonic path restrictions that should be applied.
       * <p>
       * Monotonic path restrictions imply that (ideally) the vertical and/or horizontal segments of an edge path are directed
       * from the source node towards the target node without ever changing their direction back towards the source node.
       * </p>
       * <p>
       * If an unknown restriction is specified, {@link yfiles.router.MonotonicPathRestriction#NONE} will be used instead.
       * </p>
       * @type {yfiles.router.MonotonicPathRestriction}
       */
      monotonicPathRestriction:yfiles.router.MonotonicPathRestriction;
      /**
       * Gets or sets the {@link yfiles.router.PenaltySettings} used for this edge.
       * <p>
       * Besides specifying a completely customized setting, the user can choose between four predefined optimization strategies:
       * {@link yfiles.router.PenaltySettings#OPTIMIZATION_BALANCED}, {@link yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_BENDS}, {@link yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_CROSSINGS}
       * and {@link yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_LENGTHS}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given penalty settings are <code>null</code>
       * @see yfiles.router.PenaltySettings#OPTIMIZATION_BALANCED
       * @see yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_BENDS
       * @see yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_CROSSINGS
       * @see yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_LENGTHS
       * @type {yfiles.router.PenaltySettings}
       */
      penaltySettings:yfiles.router.PenaltySettings;
      static $class:yfiles.lang.Class;
    }
    /**
     * This edge routing algorithm applies polyline routes to the edges of the graph.
     * <p>
     * <b>Layout Style</b> Edges are normally routed in an orthogonal fashion, i.e., they only consist of horizontal and vertical segments. There
     * is also a {@link yfiles.router.EdgeRouter#polylineRouting routing style} in which, between horizontal and vertical segments, additional segments with other slopes are
     * inserted.
     * </p>
     * <p>
     * During the routing process, the positions of the nodes are considered to be fixed and the routing algorithm will not
     * modify their locations or their sizes in any way.
     * </p>
     * <p>
     * The edge routing algorithm can be applied wherever it is needed to route the edges as polyline or orthogonal segments
     * without crossing any nodes, while keeping the positions of the nodes in the diagram fixed. Some potential applications
     * include electric circuit design, floor planning and navigation maps.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[80,100],"d":1},"n":[[217.75,668.75],[447.12,732.5,135,30],[582,230.25],[582,530.75],[738,668.75],[497.5,372],[770.38,245.25,30,70],[674,419.5],[447.12,525.12,30,30],[357.12,253.38,30,82.5],[369.62,441.38,30,60]],"e":[[0,1,40,28.75,-67.5,0],[0,2,-9.25,-50,-40,-41.94,[248.5,238.31]],[1,3,67.5,-6,3.56,50,[625.56,741.5]],[0,4,40,-8.94,-40,-8.94],[1,4,67.5,6,-40,34.75],[0,5,27.75,-50,-40,-4.72,[285.5,417.28]],[0,6,-27.75,-50,0,-35,[230,220.25,785.38,220.25]],[4,6,7.38,-50,0,35],[2,6,40,0,-15,0],[5,7,40,44.95,-40,-2.55],[4,7,-32,-50,32,50],[7,2,-40,-17.6,19.12,50,[641.12,451.9]],[0,8,40,-34.31,0,15,[462.12,684.44]],[0,9,9.25,-50,-15,0,[267,294.62]],[5,9,-40,-32.91,0,41.25,[372.12,389.09]],[9,2,15,-2.81,-40,11.56],[10,8,15,21.54,0,-15,[462.12,492.92]],[10,3,15,10.62,-16.62,-50,[605.38,482]],[10,5,15,-19.48,-40,29.9]],"vp":[217.0,220.0,601.0,549.0]}} Sample output of the edge routing algorithm with default settings {@graph {"ann":{"s":[80,100],"d":1},"n":[[-80.25,263.5,30,30],[-309.62,407.12],[-170.25,-8.25,30,82.5],[-80.25,470.88,135,30],[54.62,-31.38],[54.62,269.12],[-157.75,179.75,30,60],[210.62,407.12],[-29.88,110.38],[243,-16.38,30,70],[146.62,157.88]],"e":[[1,0,40,28.75,-15,0,[-207.88,485.88,-199.94,477.94,-199.94,300.94,-177.5,278.5]],[1,2,40,28.75,-6,41.25,[-207.88,485.88,-199.94,477.94,-199.94,152.94,-188.56,141.56,-172.56,141.56,-161.25,130.25]],[1,3,40,28.75,-67.5,0],[1,4,40,28.75,0,-50,[-207.88,485.88,-199.94,477.94,-199.94,-19.06,-177.62,-41.38,94.62,-41.38]],[2,4,15,-2.81,-40,11.56],[3,5,67.5,0,3.56,50,[86.12,485.88,98.19,473.81]],[6,0,15,-14.69,0,-15,[-80.94,195.06,-65.25,210.75]],[6,5,15,-14.69,-18.5,-50,[-80.94,195.06,-65.25,210.75,-65.25,233.75,-49.25,249.75,70.75,249.75,76.12,255.12]],[1,7,40,28.75,34.69,-50,[-207.88,485.88,-199.94,477.94,-199.94,-19.06,-177.62,-41.38,268.62,-41.38,285.31,-24.69]],[3,7,67.5,0,-40,28.75],[6,8,15,-14.69,-40,34.69],[1,8,40,28.75,-40,-18.81,[-207.88,485.88,-199.94,477.94,-199.94,152.94,-188.56,141.56]],[8,2,40,47,6,41.25,[60.12,207.38,60.12,100.12,52.31,92.31,-144.69,92.31,-149.25,87.75]],[1,9,40,28.75,0,-35,[-207.88,485.88,-199.94,477.94,-199.94,-19.06,-177.62,-41.38,250.62,-41.38,258,-34]],[7,9,-32,-50,0,35,[218.62,334.38,229.5,323.5,246.5,323.5,258,312]],[4,9,40,0,-15,0],[8,10,40,47,-40,-0.5],[7,10,-32,-50,32,50],[10,4,-40,-29.22,18.5,50,[122.65,178.65,113.12,169.12]]],"vp":[-310.0,-42.0,601.0,550.0]}} Sample output of the edge routing algorithm with
     * polyline routing and grouped edges {@graph {"ann":{"s":[80,100],"d":1},"n":[[556.12,477.62],[648.12,366.38],[471.62,318.88],[316.25,181.25,150,335.75,2],[421.25,472,30,30],[331.25,200.25,30,82.5],[343.75,388.25,30,60],[191.88,615.62],[421.25,679.38,135,30],[712.12,615.62],[541.12,158.12,248.38,134,2],[556.12,177.12],[744.5,192.12,30,70]],"e":[[2,1,40,36.2,-40,-11.3],[2,5,-7.94,-50,15,19.12,[503.69,277.69,486.62,260.62]],[6,4,15,21.54,0,-15,[426.79,439.79,436.25,449.25]],[6,0,15,10.62,-16.62,-50,[564.88,428.88,579.5,443.5]],[6,2,15,-14.69,-40,34.69],[7,4,40,-38.54,0,15,[413.92,627.08,436.25,604.75]],[7,2,40,-21.62,-0.44,50,[489,644,511.19,621.81]],[7,5,24.67,-50,-15,0,[256.54,263.46,278.5,241.5]],[7,8,40,28.75,-67.5,0],[8,0,67.5,-6,3.56,50,[587.62,688.38,599.69,676.31]],[7,9,40,-4.71,-40,-4.71],[8,9,67.5,6,-40,34.75],[9,1,-32,-50,32,50],[7,11,0,-50,0,-50,[231.88,189.12,253.88,167.12,596.12,167.12]],[5,11,15,-19.12,-40,-4.75],[1,11,-40,-35.1,19.12,50,[624.27,381.27,615.25,372.25]],[7,12,-24.67,-50,0,-35,[207.21,186.79,229.87,164.12,751.12,164.12,759.5,172.5]],[9,12,7.38,-50,0,35],[11,12,40,0,-15,0]],"vp":[191.0,158.0,602.0,558.0]}} Sample output of the edge routing algorithm with polyline routing and group
     * nodes<b>Concept</b>The edge routing algorithm basically performs three (four) steps to achieve an orthogonal (polyline) edge routing.
     * </p>
     * <ol>
     * <li>Creating a {@link yfiles.router.IPartition} which divides the area of the graph area into several {@link yfiles.router.PartitionCell}s.</li>
     * <li>Finding the shortest/cheapest paths for all edges through the {@link yfiles.router.IPartition} using {@link yfiles.router.PathSearch}.</li>
     * <li>
     * Assigning coordinates to the segments of the edges based on the paths that were calculated before with
     * {@link yfiles.router.ChannelBasedPathRouting}.
     * </li>
     * <li>Inserting non-orthogonal segments where horizontal and vertical segments meet (only if {@link yfiles.router.EdgeRouter#polylineRouting polyline routing} is enabled).</li>
     * </ol>
     * <p>
     * The first two steps are customizable. {@link yfiles.router.IGraphPartitionExtension}s are able to influence how the {@link yfiles.router.IPartition}
     * is created. They add {@link yfiles.router.PartitionCell}s and/or mark them for adding costs later in the process. The currently
     * used {@link yfiles.router.EdgeRouter#registeredPartitionExtensions partition extensions} can be dropped or extended by custom implementations.
     * </p>
     * <p>
     * For example, the extension 'Node Partition' adds a {@link yfiles.router.PartitionCell} to the {@link yfiles.router.IPartition} for each
     * node and marks it as belonging to a node. During {@link yfiles.router.PathSearch}, the extension 'Node Crossing' recognizes
     * these
     * {@link yfiles.router.PartitionCell}s and adds costs that penalizes crossing a node. The edge will be routed around the nodes.
     * </p>
     * <p>
     * {@link yfiles.router.PathSearchExtension}s influence the {@link yfiles.router.PathSearch} by adding costs for traversing
     * {@link yfiles.router.PartitionCell}s or narrowing their intervals to allow a less expensive traversal of a
     * {@link yfiles.router.PartitionCell}. The currently used {@link yfiles.router.EdgeRouter#registeredPathSearchExtensions partition extensions} can be dropped or extended by custom
     * implementations.
     * </p>
     * <p>
     * Using {@link yfiles.router.EdgeLayoutDescriptor}s, it is possible to add individual layout settings like routing styles to
     * edges. They are registered with the graph with key {@link yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If no
     * descriptor is provided for an edge, a {@link yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor default edge layout descriptor} is used as fallback value.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The routing algorithm supports two approaches to connect edges on a specific side or even on an exact location to a
     * node. {@link yfiles.layout.PortConstraint}s define a single constraint for the ports of an edge. To realize more complex port
     * restrictions, several {@link yfiles.layout.PortCandidate}s or {@link yfiles.layout.PortCandidateSet}s can be assigned to edges or nodes.
     * If an edge with registered {@link yfiles.layout.PortCandidate}s connects to nodes with {@link yfiles.layout.PortCandidateSet}s, the edge
     * router will try to match both collections in order to find an appropriate port. In case there is no matching port
     * candidate, a {@link yfiles.layout.PortCandidate} specified for the edge is preferred. Since their simultaneous existence at the
     * same node may be ambiguous, it is not recommended to use a combination of {@link yfiles.layout.PortConstraint}s and
     * {@link yfiles.layout.PortCandidate}s in the same diagram.
     * </p>
     * <p>
     * Edges can be grouped so that they share common segments at the beginning or end of their routes. Although a graph may
     * contain source and target grouped edges, an edge can only either be part of a source or a target group. Edge groups are
     * specified using {@link yfiles.algorithms.IDataProvider}s that provide the same ID object for all edges in the same group. Those
     * {@link yfiles.algorithms.IDataProvider}s are registered with the graph with key {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
     * for source groups or key {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY} for target groups.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface EdgeRouter extends yfiles.layout.LayoutStageBase{}
    export class EdgeRouter {
      /**
       * Creates a new {@link yfiles.router.EdgeRouter} instance with an optional {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [core=null] The core layout algorithm.
       */
      constructor(core?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Checks the sizes of the nodes to be non-zero.
       * @param g The graph to check.
       * @protected
       */
      checkNodeSize(g:yfiles.layout.LayoutGraph):void;
      /**
       * Removes all registered {@link yfiles.router.IGraphPartitionExtension}s from a given {@link yfiles.router.GraphPartition} instance.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} after the edge routes are calculated. It may be overridden
       * in order to provide a custom implementation for cleaning up a {@link yfiles.router.GraphPartition} instance.
       * </p>
       * @param partition the given {@link } instance
       * @see yfiles.router.EdgeRouter#configureGraphPartition
       * @see yfiles.router.EdgeRouter#registeredPartitionExtensions
       * @protected
       */
      cleanUpGraphPartition(partition:yfiles.router.GraphPartition):void;
      /**
       * Adds all registered {@link yfiles.router.IGraphPartitionExtension}s instances to a given {@link yfiles.router.GraphPartition} instance.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to adjust the configuration of the {@link yfiles.router.GraphPartition} instance.
       * </p>
       * @param partition the given {@link } instance
       * @see yfiles.router.EdgeRouter#registeredPartitionExtensions
       * @see yfiles.router.EdgeRouter#cleanUpGraphPartition
       * @protected
       */
      configureGraphPartition(partition:yfiles.router.GraphPartition):void;
      /**
       * Adds all registered {@link yfiles.router.PathSearchExtension}s to a given {@link yfiles.router.PathSearch} instance.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to adjust the configuration of a {@link yfiles.router.PathSearch} instance.
       * </p>
       * @param pathSearch a {@link } instance
       * @see yfiles.router.EdgeRouter#createPathSearch
       * @see yfiles.router.EdgeRouter#registeredPathSearchExtensions
       * @protected
       */
      configurePathSearch(pathSearch:yfiles.router.PathSearch):void;
      /**
       * Creates a {@link yfiles.router.PathSearchConfiguration} that is used during the path searching process.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to create a new {@link yfiles.router.PathSearchConfiguration} object with custom configuration.
       * </p>
       * @param graph the input graph
       * @param grouping the grouping structure of the graph
       * @returns a {@link } instance
       * @protected
       */
      createConfiguration(graph:yfiles.layout.LayoutGraph,grouping:yfiles.layout.GroupingSupport):yfiles.router.PathSearchConfiguration;
      /**
       * Creates a default {@link yfiles.collections.IComparer.<T>} instance to determine the order of the edges according to which they will be routed.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to create a new {@link yfiles.collections.IComparer.<T>} object with a custom configuration.
       * </p>
       * <p>
       * By default, this method returns an instance of the default implementation.
       * </p>
       * @param graph the input graph
       * @param configuration the given configuration for the path searching process
       * @returns a {@link } instance
       * @protected
       */
      createDefaultEdgeOrderComparer(graph:yfiles.layout.LayoutGraph,configuration:yfiles.router.PathSearchConfiguration):yfiles.collections.IComparer<Object>;
      /**
       * Creates a {@link yfiles.router.GraphPartition} instance that divides the area of the graph into several rectangles.
       * <p>
       * This implementation creates a {@link yfiles.router.GraphPartition} using the current {@link yfiles.router.IObstaclePartition} instance.
       * </p>
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to create a new {@link yfiles.router.GraphPartition} object with a custom configuration.
       * </p>
       * @param decomposition the current {@link }
       * @returns a {@link } instance
       * @see yfiles.router.EdgeRouter#configureGraphPartition
       * @see yfiles.router.EdgeRouter#registeredPartitionExtensions
       */
      createGraphPartition(decomposition:yfiles.router.IObstaclePartition):yfiles.router.GraphPartition;
      /**
       * Creates a {@link yfiles.router.DynamicObstacleDecomposition} that is used by the {@link yfiles.router.GraphPartition} to divide the
       * graph area in rectangles.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to create a new {@link yfiles.router.DynamicObstacleDecomposition} object with custom configuration.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.router.EdgeRouter#createGraphPartition
       * @protected
       */
      createObstacleDecomposition():yfiles.router.DynamicObstacleDecomposition;
      /**
       * Creates a {@link yfiles.router.ChannelBasedPathRouting} instance that routes the edges using pre-calculated {@link yfiles.router.Path}
       * objects.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to create a new {@link yfiles.router.ChannelBasedPathRouting} object with custom configuration.
       * </p>
       * @returns a {@link } instance
       * @protected
       */
      createPathRouting():yfiles.router.ChannelBasedPathRouting;
      /**
       * Creates a {@link yfiles.router.PathSearch} instance that finds the paths of the edges through the {@link yfiles.router.GraphPartition}.
       * <p>
       * This method may be overridden in order to create a new {@link yfiles.router.PathSearch} object with custom configuration.
       * </p>
       * @returns a {@link } instance
       * @see yfiles.router.EdgeRouter#configurePathSearch
       * @see yfiles.router.EdgeRouter#registeredPathSearchExtensions
       * @protected
       */
      createPathSearch():yfiles.router.PathSearch;
      /**
       * Creates a {@link yfiles.router.PathSearchContext} that provides context information for the path searching algorithm.
       * <p>
       * This method is called by {@link yfiles.router.EdgeRouter#applyLayout} before the edge routes are calculated. It may be
       * overridden in order to create a new {@link yfiles.router.PathSearchContext} object with custom configuration.
       * </p>
       * @param pathSearch a given {@link } instance
       * @param configuration a given configuration for the path searching process
       * @returns a {@link } instance
       * @protected
       */
      createPathSearchContext(pathSearch:yfiles.router.PathSearch,configuration:yfiles.router.PathSearchConfiguration):yfiles.router.PathSearchContext;
      /**
       * Returns the {@link yfiles.router.EdgeLayoutDescriptor} instance for a given edge that is provided by a {@link yfiles.algorithms.IDataProvider}
       * which is registered with the graph with key {@link yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
       * <p>
       * For all those edges that do not have a specific layout descriptor assigned, the default layout descriptor returned by {@link yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor}
       * will be assigned.
       * </p>
       * <p>
       * This method may be overridden in order to create an {@link yfiles.router.EdgeLayoutDescriptor} with custom configuration.
       * </p>
       * @param edge the given edge
       * @returns the current {@link } instance for a given edge
       * @see yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor
       * @see yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       */
      getEdgeLayoutDescriptor(edge:yfiles.algorithms.Edge):yfiles.router.EdgeLayoutDescriptor;
      /**
       * Returns whether or not a given edge is selected.
       * <p>
       * If all the edges of the graph will be routed by {@link yfiles.router.EdgeRouter}, i.e., the {@link yfiles.router.EdgeRouter#scope scope} is set to
       * {@link yfiles.router.Scope#ROUTE_ALL_EDGES}, this utility method returns <code>true</code> for all edges.
       * </p>
       * <p>
       * This method may be overridden in order to determine differently whether or not a given edge is considered to be
       * selected.
       * </p>
       * @param edge the given edge
       * @param graph the input graph
       * @returns <code>true</code> if the given edge is selected, <code>false</code> otherwise
       */
      isAffected(edge:yfiles.algorithms.Edge,graph:yfiles.algorithms.Graph):boolean;
      /**
       * Data provider key for specifying individual edge layout information
       * <p>
       * If this {@link yfiles.algorithms.IDataProvider} does not contain an {@link yfiles.router.EdgeLayoutDescriptor} for an edge, then the
       * layout algorithm will use the {@link yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor default descriptor}.
       * </p>
       * @see yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.router.EdgeLayoutDescriptor>}
       */
      static EDGE_LAYOUT_DESCRIPTOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.router.EdgeLayoutDescriptor>;
      /**
       * Data provider key for weighting the costs for crossing each label individually
       * <p>
       * If the factor for a label is <code>0</code> then it is allowed to cross it. Very important labels should get a high factor.
       * </p>
       * <p>
       * This factor is multiplied by the basic penalty arising when an edge must cross a {@link yfiles.router.PenaltySettings#nodeLabelCrossingPenalty node label} or an {@link yfiles.router.PenaltySettings#edgeLabelCrossingPenalty edge label} in order to
       * determine the final costs arising when this label is crossed.
       * </p>
       * @see yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor
       * @see yfiles.router.EdgeLayoutDescriptor#penaltySettings
       * @const
       * @static
       * @type {yfiles.algorithms.ILabelLayoutDpKey.<number>}
       */
      static LABEL_CROSSING_PENALTY_FACTOR_DP_KEY:yfiles.algorithms.ILabelLayoutDpKey<number>;
      /**
       * Gets or sets the time limit (in milliseconds) set for the edge routing algorithm.
       * <p>
       * The maximum duration has to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets the {@link yfiles.router.EdgeLayoutDescriptor} instance used for all those edges that do not have a specific edge layout
       * descriptor assigned.
       * @see yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.router.EdgeLayoutDescriptor}
       */
      defaultEdgeLayoutDescriptor:yfiles.router.EdgeLayoutDescriptor;
      /**
       * Gets or sets whether or not the routing algorithm will route the edges of the graph with (non-orthogonal) polyline
       * segments.
       * @see yfiles.router.EdgeRouter#preferredPolylineSegmentLength
       * @see yfiles.router.EdgeRouter#maximumPolylineSegmentRatio
       * @type {boolean}
       */
      polylineRouting:boolean;
      /**
       * Gets or sets the preferred length of (non-orthogonal) polyline segments.
       * <p>
       * If there is not enough space to use this preferred length, polyline segments may also be shorter.
       * </p>
       * <p>
       * The preferred length of (non-orthogonal) polyline segments has to be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the preferred polyline segment length is negative
       * @see yfiles.router.EdgeRouter#maximumPolylineSegmentRatio
       * @type {number}
       */
      preferredPolylineSegmentLength:number;
      /**
       * Gets or sets the maximum ratio between the horizontal/vertical part of a segment and the (non-orthogonal) polyline
       * part.
       * <p>
       * When polyline segments are added to an edge path, corners between horizontal and vertical segments are cut and replaced
       * by the new segment. This ratio describes the cutting points on a segment. If it is zero, the route stays orthogonal. The
       * value cannot be larger than <code>0.5</code> because there may be another polyline segment at the other side of the segment. For long
       * orthogonal segments the length of the polyline segment is determined by the value returned by {@link yfiles.router.EdgeRouter#preferredPolylineSegmentLength}.
       * </p>
       * <p>
       * The maximum polyline segment ratio must be between <code>0</code> and <code>0.5</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum segment length is negative or greater than <code>0.5</code>
       * @type {number}
       */
      maximumPolylineSegmentRatio:number;
      /**
       * Gets or sets whether or not the routing algorithm uses an additional step to reroute the edges that are considered to
       * have the worst paths.
       * @type {boolean}
       */
      rerouting:boolean;
      /**
       * Gets or sets a (sub-)set of edges that shall be routed.
       * @throws {Stubs.Exceptions.ArgumentError} if the given scope is unknown
       * @see yfiles.router.EdgeRouter#affectedEdgesDpKey
       * @type {yfiles.router.Scope}
       */
      scope:yfiles.router.Scope;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key to look up the selection state of the nodes.
       * <p>
       * If the {@link yfiles.router.EdgeRouter#scope scope} is set to
       * {@link yfiles.router.Scope#ROUTE_EDGES_AT_AFFECTED_NODES}, only the edges that are incident to selected nodes will be routed,
       * while all other edges will be considered to have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.algorithms.IDataProvider} key is <code>null</code>
       * @see yfiles.router.EdgeRouter#scope
       * @type {Object}
       */
      affectedNodesDpKey:Object;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key to look up the selection state of the edges.
       * <p>
       * If the {@link yfiles.router.EdgeRouter#scope scope} is set to
       * {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES}, only the selected edges will be routed, while all other edges will be
       * considered to have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.algorithms.IDataProvider} key is <code>null</code>
       * @see yfiles.router.EdgeRouter#scope
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      /**
       * Gets or sets a custom {@link yfiles.collections.IComparer.<T>} to define the processing order of the edges.
       * @see yfiles.router.EdgeRouter#createDefaultEdgeOrderComparer
       * @type {yfiles.collections.IComparer.<Object>}
       */
      edgeComparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets whether or not the routing algorithm considers the labels of the nodes as obstacles when calculating the
       * edge routes to avoid overlaps.
       * @see yfiles.router.PenaltySettings#nodeLabelCrossingPenalty
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not this routing algorithm ignores node labels that are inside the bounds of their owner as
       * obstacles for edge routes.
       * @see yfiles.router.EdgeRouter#considerNodeLabels
       * @see yfiles.router.PenaltySettings#nodeLabelCrossingPenalty
       * @see yfiles.router.EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY
       * @type {boolean}
       */
      ignoreInnerNodeLabels:boolean;
      /**
       * Gets or sets whether or not the routing algorithm considers as obstacles the edge labels that do not belong to the
       * (sub-)set of edges to be routed when calculating the edge routes.
       * @see yfiles.router.EdgeRouter#scope
       * @see yfiles.router.EdgeRouter#affectedEdgesDpKey
       * @see yfiles.router.PenaltySettings#nodeLabelCrossingPenalty
       * @type {boolean}
       */
      considerEdgeLabels:boolean;
      /**
       * Gets or sets the {@link yfiles.router.Grid} instance on which the routing algorithm places the orthogonal segments.
       * @type {yfiles.router.Grid}
       */
      grid:yfiles.router.Grid;
      /**
       * Gets or sets the minimum distance between edges and node bounds.
       * <p>
       * The minimum distance should have a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum node-to-edge distance is negative
       * @see yfiles.router.PenaltySettings#minimumNodeToEdgeDistancePenalty
       * @type {number}
       */
      minimumNodeToEdgeDistance:number;
      /**
       * Gets a list of all registered {@link yfiles.router.IGraphPartitionExtension}s.
       * <p>
       * {@link yfiles.router.IGraphPartitionExtension}s can be added to a {@link yfiles.router.GraphPartition} in order to create new
       * {@link yfiles.router.Obstacle}s or can be removed from a {@link yfiles.router.GraphPartition} instance.
       * </p>
       * <p>
       * By default, the following {@link yfiles.router.IGraphPartitionExtension}s are registered with a given {@link yfiles.router.GraphPartition}
       * instance:
       * </p>
       * <ul>
       * <li>Minimum Node To Edge Distance Partition</li>
       * <li>Node Partition</li>
       * <li>Partition Grid Partition</li>
       * <li>Node Label Partition</li>
       * <li>Edge Label Partition</li>
       * <li>Fixed Edges Partition</li>
       * <li>External Strong Port Restriction Partition</li>
       * </ul>
       * @see yfiles.router.EdgeRouter#createGraphPartition
       * @see yfiles.router.EdgeRouter#configureGraphPartition
       * @type {yfiles.collections.IList.<Object>}
       */
      registeredPartitionExtensions:yfiles.collections.IList<Object>;
      /**
       * Gets a list of all registered {@link yfiles.router.PathSearchExtension}s.
       * <p>
       * {@link yfiles.router.PathSearchExtension}s can be added to a {@link yfiles.router.PathSearch} instance in order to influence the path
       * searching process or can be removed from a {@link yfiles.router.PathSearch} instance.
       * </p>
       * <p>
       * By default, the following {@link yfiles.router.PathSearchExtension}s are registered with a {@link yfiles.router.PathSearch} instance:
       * </p>
       * <ul>
       * <li>Fixed Grouped Edges</li>
       * <li>Node Crossing</li>
       * <li>Minimum Node To Edge Distance</li>
       * <li>Group Node Crossing</li>
       * <li>Minimum Group Node To Edge Distance</li>
       * <li>Node Label Crossing</li>
       * <li>Edge Label Crossing</li>
       * <li>Bends In Node To Edge Distance</li>
       * <li>Bend</li>
       * <li>Monotonic Route</li>
       * <li>Edge Length</li>
       * <li>Partition Grid</li>
       * <li>Port Restriction</li>
       * <li>Edge Grouping</li>
       * <li>Minimum Node Corner Distance</li>
       * <li>Interval Based Crossing</li>
       * <li>Minimum Edge To Edge Distance And Grid</li>
       * <li>Minimum First Last Segment Length</li>
       * <li>Intersecting Source And Target</li>
       * </ul>
       * @see yfiles.router.EdgeRouter#createPathSearch
       * @see yfiles.router.EdgeRouter#configurePathSearch
       * @type {yfiles.collections.IList.<Object>}
       */
      registeredPathSearchExtensions:yfiles.collections.IList<Object>;
      /**
       * Gets the {@link yfiles.router.GraphPartition} instance used during the routing process.
       * @type {yfiles.router.GraphPartition}
       */
      partition:yfiles.router.GraphPartition;
      static $class:yfiles.lang.Class;
    }
    export enum Alignment{
      /**
       * A constant specifying that the segment prefers to be placed close to the lower bound of its location range.
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       */
      MIN,
      /**
       * A constant specifying that the segment prefers to be placed close to the upper bound of its location range.
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       */
      MAX,
      /**
       * A constant specifying that the segment can be placed anywhere within its location range.
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       * @see yfiles.router.SegmentInfoBase#preferredAlignment
       */
      ANY
    }
    export enum RoutingType{
      /**
       * A routing type representing a straight crossing of the
       * {@link yfiles.router.PartitionCell partition cell}, i.e., a single orthogonal edge segment enters and exits the cell.
       * @see yfiles.router.EdgeCellInfo#type
       */
      STRAIGHT,
      /**
       * A routing type representing a crossing of the {@link yfiles.router.PartitionCell partition cell} with three segments, i.e., the edge enters and exits the
       * cell using the same direction but has an orthogonal middle segment between the entering and exiting segment.
       * @see yfiles.router.EdgeCellInfo#type
       */
      STRAIGHT_BENDING,
      /**
       * A routing type representing a crossing of the {@link yfiles.router.PartitionCell partition cell} with a single bend, i.e., the edge enters the cell in one
       * direction, makes a bend and exits the cell in an orthogonal direction.
       * @see yfiles.router.EdgeCellInfo#type
       */
      BENDING,
      /**
       * A routing type representing a crossing of the {@link yfiles.router.PartitionCell partition cell} by making a U-turn, i.e., the edge enters and exits the
       * cell on the same side using two turns and an orthogonal middle segment.
       * @see yfiles.router.EdgeCellInfo#type
       */
      U_TURN,
      /**
       * A routing type representing the source or target end of an edge being in the cell, i.e., the enter and/or exit interval
       * is <code>null</code>.
       * @see yfiles.router.EdgeCellInfo#type
       */
      END
    }
    /**
     * A {@link yfiles.router.GraphPartition} partitions the bounding box of a graph into rectangles using the graph elements as
     * {@link yfiles.router.Obstacle}s.
     * <p>
     * All {@link yfiles.router.Obstacle}s used during the partitioning are {@link yfiles.router.GraphPartition#createObstacle created} by {@link yfiles.router.IGraphPartitionExtension}s.
     * {@link yfiles.router.IGraphPartitionExtension}s have been {@link yfiles.router.GraphPartition#addDynamicDecompositionListener registered} before using method {@link yfiles.router.IGraphPartitionExtension#preparePartition}
     * which is called during the {@link yfiles.router.GraphPartition#init initialization} of a {@link yfiles.router.GraphPartition}.
     * </p>
     * <p>
     * The core partitioning is delegated to an {@link yfiles.router.IObstaclePartition} that is {@link yfiles.router.IObstaclePartition#init initialized} with these
     * {@link yfiles.router.Obstacle}s.
     * </p>
     * <p>
     * If the {@link yfiles.router.IObstaclePartition} is of type {@link yfiles.router.IDynamicDecomposition}, {@link yfiles.router.GraphPartition}
     * re-dispatches the decomposition notifications to its extensions.
     * </p>
     * @see yfiles.router.IGraphPartitionExtension
     * @class
     * @implements {yfiles.router.IPartition}
     * @implements {yfiles.router.IDynamicDecomposition}
     * @implements {yfiles.router.IDecompositionListener}
     */
    export interface GraphPartition extends Object,yfiles.router.IPartition,yfiles.router.IDynamicDecomposition,yfiles.router.IDecompositionListener{}
    export class GraphPartition {
      /**
       * Creates a new {@link yfiles.router.GraphPartition} instance using the given {@link yfiles.router.GraphPartition#partition delegate inner obstacle partition}.
       * @param partition the delegate inner obstacle partition
       */
      constructor(partition:yfiles.router.IObstaclePartition);
      /**
       * Adds the given listener to the {@link yfiles.router.GraphPartition}.
       * <p>
       * The listener will be notified upon dynamic decomposition events. If it is a {@link yfiles.router.IGraphPartitionExtension},
       * method {@link yfiles.router.IGraphPartitionExtension#preparePartition} is called during {@link yfiles.router.GraphPartition#init initialization}.
       * </p>
       * @param listener the listener to be added
       */
      addDynamicDecompositionListener(listener:yfiles.router.IDecompositionListener):void;
      /**
       * Clears the {@link yfiles.router.GraphPartition partition} data such that the {@link yfiles.router.GraphPartition} can be reused and {@link yfiles.router.GraphPartition#init initialized} with a new
       * configuration.
       * <p>
       * All {@link yfiles.router.IGraphPartitionExtension}s are {@link yfiles.router.IGraphPartitionExtension#cleanup cleaned up}; partition listeners and obstacles are {@link yfiles.router.GraphPartition#removeDynamicDecompositionListener removed}.
       * </p>
       * @see yfiles.router.GraphPartition#init
       */
      clear():void;
      /**
       * Creates and returns a new {@link yfiles.router.Obstacle} using the given bounds and data.
       * <p>
       * All obstacles created via this method are delegated to the {@link yfiles.router.GraphPartition#partition inner partition} upon {@link yfiles.router.GraphPartition#init initialization}.
       * </p>
       * @param bounds the bounds of the obstacle
       * @param data the additional data that should be associated with the obstacle
       * @returns a new {@link } using the given bounds and data
       */
      createObstacle(bounds:yfiles.algorithms.YRectangle,data:Object):yfiles.router.Obstacle;
      /**
       * Returns a list of all {@link yfiles.router.PartitionCell}s that intersect or cover the given rectangle by delegating to the
       * corresponding method of the {@link yfiles.router.GraphPartition#partition inner partition}.
       * @param rect the rectangular area whose (partially) covered cells will be returned
       * @returns a list containing (partially) covered {@link }s
       * @see yfiles.router.IPartition#getCells
       */
      getCells(rect:yfiles.algorithms.YRectangle):yfiles.collections.IList<Object>;
      /**
       * Returns all {@link yfiles.router.PartitionCell}s that are completely covered by the bounds of the given node.
       * @param node the node for which the covered cells will be returned
       * @returns an unmodifiable list of {@link } instances that are completely covered by the bounds of the given
       * node
       * @throws {Stubs.Exceptions.ArgumentError} if the node is unknown
       */
      getCellsForNode(node:yfiles.algorithms.Node):yfiles.collections.IList<Object>;
      /**
       * Returns a list of all
       * {@link yfiles.router.PartitionCell}s that are neighbors of the given cell, i.e., those cells that have a common border segment
       * with the given cell.
       * @param cell the cell whose neighbors will be returned
       * @returns a list of {@link }s that are neighbors of the given cell
       */
      getNeighbors(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Returns a list of all {@link yfiles.algorithms.Node}s whose bounds intersect or cover the bounds of the given cell.
       * @param cell the cell
       * @returns a list of {@link }s that are intersected by the given cell
       */
      getNodes(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Returns the {@link yfiles.router.Obstacle} that has been created earlier for the given data object.
       * @param data the given data for which the obstacle will be returned
       * @returns the {@link } that has been created earlier for the given data object
       */
      getObstacle(data:Object):yfiles.router.Obstacle;
      /**
       * Returns all {@link yfiles.router.Obstacle}s covering the given {@link yfiles.router.PartitionCell} by delegating to the corresponding
       * method of the {@link yfiles.router.GraphPartition#partition inner partition}.
       * @param cell the cell whose obstacles will be returned
       * @returns a list of {@link }s that cover the given cell
       * @see yfiles.router.IObstaclePartition#getObstacles
       */
      getObstacles(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Initializes the {@link yfiles.router.GraphPartition} with the given graph instance based on the given {@link yfiles.router.PathSearchConfiguration configuration} used by path
       * search algorithms.
       * <p>
       * Before calling this method, any old partitioning information must be {@link yfiles.router.GraphPartition#clear cleared}.
       * </p>
       * <p>
       * This initialization method calls {@link yfiles.router.IGraphPartitionExtension#preparePartition} of all registered extensions.
       * Then, the {@link yfiles.router.GraphPartition#partition inner delegate partition} is initialized passing all {@link yfiles.router.GraphPartition#createObstacle added} obstacles as well as the given graph's bounds
       * extended by a small padding.
       * </p>
       * <p>
       * In the case where the {@link yfiles.router.GraphPartition#partition delegate partition} is of type {@link yfiles.router.IDynamicDecomposition}, this {@link yfiles.router.GraphPartition}
       * will also be added as {@link yfiles.router.IDecompositionListener} to it.
       * </p>
       * @param configuration the configuration used by path search algorithms
       * @see yfiles.router.GraphPartition#clear
       * @see yfiles.router.IGraphPartitionExtension#preparePartition
       * @see yfiles.router.GraphPartition#addDynamicDecompositionListener
       */
      init(configuration:yfiles.router.PathSearchConfiguration):void;
      /**
       * Re-dispatches the event of the creation of a new {@link yfiles.router.PartitionCell} to all registered
       * {@link yfiles.router.IDecompositionListener}s.
       * <p>
       * This method is a callback after a {@link yfiles.router.PartitionCell} has been created.
       * </p>
       * @param createdCell the newly created {@link }
       */
      onCellCreated(createdCell:yfiles.router.PartitionCell):void;
      /**
       * Re-dispatches the event of having completed the process of subdivisions of a given {@link yfiles.router.PartitionCell} to all
       * registered {@link yfiles.router.IDecompositionListener}s.
       * <p>
       * This method is a callback after a {@link yfiles.router.PartitionCell} has been finalized and won't be further subdivided.
       * </p>
       * @param finalizedCell the cell that has been finalized
       */
      onCellFinalized(finalizedCell:yfiles.router.PartitionCell):void;
      /**
       * Re-dispatches the event of the subdivision of a given {@link yfiles.router.PartitionCell} into several sub-cells to all
       * registered {@link yfiles.router.IDecompositionListener}s.
       * <p>
       * This method is a callback after a {@link yfiles.router.PartitionCell} has been subdivided into several sub-cells.
       * </p>
       * @param cell the cell that has been subdivided
       * @param subCells the sub-cells resulting from the subdivision of the given {@link }
       */
      onCellSubdivided(cell:yfiles.router.PartitionCell,subCells:yfiles.collections.IList<Object>):void;
      /**
       * Removes the given listener from the {@link yfiles.router.GraphPartition}.
       * <p>
       * The listener will not be notified of dynamic decomposition events anymore.
       * </p>
       * @param listener the listener to be removed
       */
      removeDynamicDecompositionListener(listener:yfiles.router.IDecompositionListener):void;
      /**
       * Gets the delegate inner obstacle partition.
       * @protected
       * @type {yfiles.router.IObstaclePartition}
       */
      partition:yfiles.router.IObstaclePartition;
      /**
       * Gets the bounds of the original rectangular area that is being partitioned.
       * <p>
       * This method delegates to the corresponding method of the {@link yfiles.router.GraphPartition#partition inner partition}.
       * </p>
       * @see yfiles.router.IPartition#bounds
       * @type {yfiles.algorithms.YRectangle}
       */
      bounds:yfiles.algorithms.YRectangle;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.router.GraphPartitionExtensionAdapter} provides an empty implementation of
     * {@link yfiles.router.IGraphPartitionExtension}.
     * <p>
     * The only real functionality is contained in method {@link yfiles.router.IGraphPartitionExtension#preparePartition} which stores
     * the given {@link yfiles.router.PathSearchConfiguration} and {@link yfiles.router.GraphPartition} in properties.
     * </p>
     * @see yfiles.router.GraphPartitionExtensionAdapter#configuration
     * @see yfiles.router.GraphPartitionExtensionAdapter#graphPartition
     * @class
     * @implements {yfiles.router.IGraphPartitionExtension}
     */
    export interface GraphPartitionExtensionAdapter extends Object,yfiles.router.IGraphPartitionExtension{}
    export class GraphPartitionExtensionAdapter {
      /**
       * Creates a new instance of {@link yfiles.router.GraphPartitionExtensionAdapter}.
       */
      constructor();
      /**
       * 
       */
      cleanup():void;
      /**
       * 
       * @param createdCell
       */
      onCellCreated(createdCell:yfiles.router.PartitionCell):void;
      /**
       * 
       * @param finalizedCell
       */
      onCellFinalized(finalizedCell:yfiles.router.PartitionCell):void;
      /**
       * 
       * @param cell
       * @param subCells
       */
      onCellSubdivided(cell:yfiles.router.PartitionCell,subCells:yfiles.collections.IList<Object>):void;
      /**
       * Stores the given {@link yfiles.router.PathSearchConfiguration} and {@link yfiles.router.GraphPartition} such that they are accessible
       * through {@link yfiles.router.GraphPartitionExtensionAdapter#configuration} and {@link yfiles.router.GraphPartitionExtensionAdapter#graphPartition}, respectively.
       * @param configuration the configuration used for the path search
       * @param graphPartition the graph partition that uses this extension
       */
      preparePartition(configuration:yfiles.router.PathSearchConfiguration,graphPartition:yfiles.router.GraphPartition):void;
      /**
       * Gets the {@link yfiles.router.GraphPartition} passed to {@link yfiles.router.IGraphPartitionExtension#preparePartition}.
       * @protected
       * @type {yfiles.router.GraphPartition}
       */
      graphPartition:yfiles.router.GraphPartition;
      /**
       * Gets the configuration of the {@link yfiles.router.PathSearch} passed to
       * {@link yfiles.router.IGraphPartitionExtension#preparePartition}.
       * @protected
       * @type {yfiles.router.PathSearchConfiguration}
       */
      configuration:yfiles.router.PathSearchConfiguration;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides support for routing edges on equidistant grid lines.
     * <p>
     * The grid is defined by its origin and the spacing between the equidistant grid lines.
     * </p>
     * @class
     * @final
     */
    export interface Grid extends Object{}
    export class Grid {
      /**
       * Creates a new {@link yfiles.router.Grid} instance.
       * <p>
       * The spacing between the grid lines must be at least <code>1</code>. If the given spacing is smaller, value <code>1</code> is assumed.
       * </p>
       * @param originX the x-coordinate of the origin of the grid
       * @param originY the y-coordinate of the origin of the grid
       * @param spacing the spacing between the grid lines
       */
      constructor(originX:number,originY:number,spacing:number);
      /**
       * Gets the x-coordinate of the origin.
       * @type {number}
       */
      originX:number;
      /**
       * Gets the y-coordinate of the origin.
       * @type {number}
       */
      originY:number;
      /**
       * Gets the spacing between the horizontal and vertical grid lines.
       * @type {number}
       */
      spacing:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This interface should be implemented by classes that dispatch notifications about changes in {@link yfiles.router.IPartition}s.
     * <p>
     * Such classes can be notified when {@link yfiles.router.PartitionCell}s are created, divided, and/or finalized.
     * </p>
     * @see yfiles.router.IDecompositionListener
     * @interface
     */
    export interface IDynamicDecomposition extends Object{
      /**
       * Adds the given {@link yfiles.router.IDecompositionListener} that will be notified of dynamic decomposition events to the
       * {@link yfiles.router.IPartition}.
       * @param listener the listener to be added
       * @abstract
       */
      addDynamicDecompositionListener(listener:yfiles.router.IDecompositionListener):void;
      /**
       * Removes the given {@link yfiles.router.IDecompositionListener} from the {@link yfiles.router.IPartition}.
       * <p>
       * The {@link yfiles.router.IDecompositionListener} will be no more notified of dynamic decomposition events.
       * </p>
       * @param listener the listener to be removed
       * @abstract
       */
      removeDynamicDecompositionListener(listener:yfiles.router.IDecompositionListener):void;
    }
    var IDynamicDecomposition:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface is implemented by classes that require to be notified of changes in {@link yfiles.router.IPartition}s.
     * <p>
     * Implementations are notified when {@link yfiles.router.PartitionCell}s are created, divided, and/or finalized.
     * </p>
     * @interface
     */
    export interface IDecompositionListener extends Object{
      /**
       * Callback after the new given {@link yfiles.router.PartitionCell} has been created.
       * @param createdCell the newly created {@link }
       * @abstract
       */
      onCellCreated(createdCell:yfiles.router.PartitionCell):void;
      /**
       * Callback after the given {@link yfiles.router.PartitionCell} has been finalized and won't be further subdivided.
       * @param finalizedCell the cell that has been finalized
       * @abstract
       */
      onCellFinalized(finalizedCell:yfiles.router.PartitionCell):void;
      /**
       * Callback after the given {@link yfiles.router.PartitionCell} has been subdivided into several sub-cells.
       * <p>
       * {@link yfiles.router.IDecompositionListener} may not modify the list of sub-cells.
       * </p>
       * @param cell the cell that has been subdivided
       * @param subCells the sub-cells resulting from the subdivision of the given {@link }
       * @abstract
       */
      onCellSubdivided(cell:yfiles.router.PartitionCell,subCells:yfiles.collections.IList<Object>):void;
    }
    var IDecompositionListener:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides additional intervals in order to move from one {@link yfiles.router.PartitionCell partition cell} into an adjacent one.
     * <p>
     * To find a {@link yfiles.router.Path} for an edge, the {@link yfiles.router.PathSearch} algorithm moves from the current partition cell
     * to an adjacent partition cell. By default, the orthogonal interval shared by these two cells is used. Implementations of
     * this interface may be added to the {@link yfiles.router.PathSearch} to calculate additional enter intervals that might
     * accumulate lower costs for entering the adjacent cell.
     * </p>
     * @see yfiles.router.PartitionCell
     * @see yfiles.router.PathSearch#addAdditionalEnterIntervalCalculator
     * @interface
     */
    export interface IEnterIntervalCalculator extends Object{
      /**
       * Appends additional enter intervals to the given list of enter intervals for moving from the {@link yfiles.router.PartitionCell partition cell} of the
       * current {@link yfiles.router.CellEntrance} to the neighboring cell.
       * @param currentEntrance the current {@link } that is exited
       * @param enteredCell the neighboring cell that will be entered
       * @param commonInterval the common interval of the left and entered partition cells
       * @param allEnterIntervals the list to which additional enter intervals have to be appended
       * @param context the context of the path search
       * @abstract
       */
      appendEnterIntervals(currentEntrance:yfiles.router.CellEntrance,enteredCell:yfiles.router.PartitionCell,commonInterval:yfiles.router.OrthogonalInterval,allEnterIntervals:yfiles.collections.IList<Object>,context:yfiles.router.PathSearchContext):void;
    }
    var IEnterIntervalCalculator:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface provides methods for extending the functionality of a {@link yfiles.router.GraphPartition}.
     * <p>
     * A {@link yfiles.router.IGraphPartitionExtension} can be used for {@link yfiles.router.GraphPartition#createObstacle creating} new {@link yfiles.router.Obstacle}s that will be given to the
     * initializer of the {@link yfiles.router.IObstaclePartition} where they can react to the {@link yfiles.router.IObstaclePartition}'s
     * decomposition events if it implements the {@link yfiles.router.IDynamicDecomposition} interface.
     * </p>
     * <p>
     * A {@link yfiles.router.IGraphPartitionExtension} has to be {@link yfiles.router.GraphPartition#addDynamicDecompositionListener added} to the {@link yfiles.router.GraphPartition} which will call method {@link yfiles.router.IGraphPartitionExtension#preparePartition}
     * before the {@link yfiles.router.IObstaclePartition} is initialized.
     * </p>
     * @interface
     * @implements {yfiles.router.IDecompositionListener}
     */
    export interface IGraphPartitionExtension extends Object,yfiles.router.IDecompositionListener{
      /**
       * Cleans up the extension from the last partitioning with the current {@link yfiles.router.PathSearchConfiguration configuration} and {@link yfiles.router.GraphPartition}.
       * @abstract
       */
      cleanup():void;
      /**
       * Adds a {@link yfiles.router.IGraphPartitionExtension} to the {@link yfiles.router.GraphPartition}.
       * <p>
       * It is called by the given {@link yfiles.router.GraphPartition} upon initialization before its {@link yfiles.router.IObstaclePartition}
       * is initialized. Implementations may use the graph upon which the given {@link yfiles.router.GraphPartition} is based to
       * initialize themselves and/or to {@link yfiles.router.GraphPartition#createObstacle add obstacles} to the list with which {@link yfiles.router.IObstaclePartition} is initialized.
       * </p>
       * @param configuration the configuration used for the path search
       * @param graphPartition the graph partition that uses this extension
       * @abstract
       */
      preparePartition(configuration:yfiles.router.PathSearchConfiguration,graphPartition:yfiles.router.GraphPartition):void;
    }
    var IGraphPartitionExtension:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class implements a {@link yfiles.router.IPartition} that takes a list of {@link yfiles.router.Obstacle}s as input to subdivide an
     * area into {@link yfiles.router.PartitionCell}s.
     * <p>
     * Implementations have to satisfy the following condition for each pair of {@link yfiles.router.Obstacle} and
     * {@link yfiles.router.PartitionCell}:
     * </p>
     * <ul>
     * <li>
     * {@link yfiles.router.Obstacle}s either completely cover {@link yfiles.router.PartitionCell}s or do not intersect
     * {@link yfiles.router.PartitionCell}s at all.
     * </li>
     * </ul>
     * <p>
     * {@graph {"ann":{"s":[40,40],"d":1},"styles":{"6":{"s":"#0000ff", "f":null}},"n":[[380,280,[[394.66,309.35,10.67,18.7,"1"]]],[580,400,[[594.66,429.35,10.67,18.7,"2"]]],[240.2,280,139.8,40,6],[240.2,159.2,179.8,120.8,6],[240.2,320,179.95,280,6],[420.15,440,200,160,6],[420.15,400,159.85,40,6],[420.15,159.2,319.2,240.8,6],[620,400,119.35,200,6]],"e":[[0,1,[400,420]]],"vp":[240.0,159.0,500.0,441.0]}} Example of an {@link yfiles.router.IObstaclePartition} of a graph consisting of two nodes connected by one edge. The two nodes
     * are considered to be obstacles. The colored partition cells subdivide the area such that each obstacle either completely
     * covers a partition cell or does not intersect with a partition cell.
     * </p>
     * @interface
     * @implements {yfiles.router.IPartition}
     */
    export interface IObstaclePartition extends Object,yfiles.router.IPartition{
      /**
       * Clears the partition data such that the {@link yfiles.router.IObstaclePartition} can be reused and {@link yfiles.router.IObstaclePartition#init initialized} with new
       * {@link yfiles.router.Obstacle}s.
       * @see yfiles.router.IObstaclePartition#init
       * @abstract
       */
      clear():void;
      /**
       * Returns all {@link yfiles.router.PartitionCell}s that are completely covered by the given {@link yfiles.router.Obstacle}.
       * @param obstacle the obstacle for which the covered cells will be returned
       * @returns an unmodifiable list of {@link } instances that are completely covered by the given obstacle
       * @abstract
       */
      getCellsForObstacle(obstacle:yfiles.router.Obstacle):yfiles.collections.IList<Object>;
      /**
       * Returns all {@link yfiles.router.Obstacle}s that cover the given {@link yfiles.router.PartitionCell}.
       * @param cell the partition cell for which the obstacles will be returned
       * @returns an unmodifiable list of {@link } instances that cover the given cell
       * @abstract
       */
      getObstacles(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Initializes a new {@link yfiles.router.IObstaclePartition} of the area with the given bounds using the given list of
       * {@link yfiles.router.Obstacle}s.
       * <p>
       * Unused partitions must be cleared using method {@link yfiles.router.IObstaclePartition#clear}.
       * </p>
       * @param obstacles a list of {@link }s that have to be considered by the partition
       * @param bounds the bounds of the partition
       * @see yfiles.router.IObstaclePartition#clear
       * @abstract
       */
      init(obstacles:yfiles.collections.IList<Object>,bounds:yfiles.algorithms.YRectangle):void;
    }
    var IObstaclePartition:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class divides a rectangular area into one or more {@link yfiles.router.PartitionCell}s.
     * <p>
     * {@link yfiles.router.PartitionCell}s have to be disjoint. The union of all
     * {@link yfiles.router.PartitionCell}s covers the original area.
     * </p>
     * @interface
     */
    export interface IPartition extends Object{
      /**
       * Returns a list of all {@link yfiles.router.PartitionCell}s that intersect or cover the given rectangle.
       * @param rect the rectangular area whose (partially) covered cells will be returned
       * @returns a list of {@link }s that (partially) cover the given rectangular area
       * @abstract
       */
      getCells(rect:yfiles.algorithms.YRectangle):yfiles.collections.IList<Object>;
      /**
       * Returns a list of all
       * {@link yfiles.router.PartitionCell}s that are neighbors of the given cell, i.e., those cells that have a common border segment
       * with the given cell.
       * @param cell the cell whose neighbors will be returned
       * @returns a list of {@link }s that are neighbors of the given cell
       * @abstract
       */
      getNeighbors(cell:yfiles.router.PartitionCell):yfiles.collections.IList<Object>;
      /**
       * Gets the bounds of the original rectangular area that is partitioned.
       * @abstract
       * @type {yfiles.algorithms.YRectangle}
       */
      bounds:yfiles.algorithms.YRectangle;
    }
    var IPartition:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class represents a one-dimensional closed interval.
     * @class
     */
    export interface Interval extends Object{}
    export class Interval {
      /**
       * Creates a new {@link yfiles.router.Interval} between the given bounds.
       * <p>
       * The upper and lower bound of the two given values are determined automatically, i.e., the smaller one will be treated as
       * the lower bound and the larger one as the upper bound.
       * </p>
       * @param bound1 the first interval bound
       * @param bound2 the second interval bound
       */
      constructor(bound1:number,bound2:number);
      /**
       * Returns the bridging {@link yfiles.router.Interval interval} between the given {@link yfiles.router.Interval intervals}.
       * <p>
       * The bridging interval is <code>null</code> if the given intervals intersect each other. Otherwise, the bridging interval is
       * <code>[minimum of upper bounds, maximum of lower bounds]</code>.
       * </p>
       * @param i1 the first interval
       * @param i2 the second interval
       * @returns the bridging interval between the given intervals or <code>null</code> if the intervals intersect
       * @static
       */
      static calculateBridge(i1:yfiles.router.Interval,i2:yfiles.router.Interval):yfiles.router.Interval;
      /**
       * Returns the intersection of the given {@link yfiles.router.Interval intervals}.
       * <p>
       * The intersection is <code>null</code> if the given intervals do not intersect each other. Otherwise, the intersection is
       * <code>[maximum of lower bounds, minimum of upper bounds]</code>.
       * </p>
       * @param i1 the first interval
       * @param i2 the second interval
       * @returns the intersection of the given intervals or <code>null</code> if the intervals do not intersect
       * @static
       */
      static calculateIntersection(i1:yfiles.router.Interval,i2:yfiles.router.Interval):yfiles.router.Interval;
      /**
       * Returns the spanning rectangle between the given {@link yfiles.router.Interval intervals}.
       * <p>
       * The first interval spans the rectangle in horizontal dimension, the second interval in vertical dimension.
       * </p>
       * @param horizontal the interval that defines the location and length of the rectangle in horizontal dimension
       * @param vertical the interval that defines the location and length of the rectangle in vertical dimension
       * @returns the spanning rectangle between the given intervals
       * @static
       */
      static calculateSpanningRectangle(horizontal:yfiles.router.Interval,vertical:yfiles.router.Interval):yfiles.algorithms.YRectangle;
      /**
       * Returns the union of the given {@link yfiles.router.Interval intervals} and the given value.
       * <p>
       * The union is between the minimum of the lower bound of the given interval and the given value and the maximum of the
       * upper bound of the given interval and the given value.
       * </p>
       * @param interval the interval to be extended
       * @param value the value to be in the union
       * @returns the union of the given interval and the given value
       * @static
       */
      static calculateUnion(interval:yfiles.router.Interval,value:number):yfiles.router.Interval;
      /**
       * Returns the union of the given {@link yfiles.router.Interval intervals}.
       * <p>
       * The union is <code>[minimum of lower bounds, maximum of upper bounds]</code>.
       * </p>
       * @param i1 the first interval
       * @param i2 the second interval
       * @returns the union of the given intervals
       * @static
       */
      static calculateUnion(i1:yfiles.router.Interval,i2:yfiles.router.Interval):yfiles.router.Interval;
      /**
       * Checks whether or not the given value lies within the interval.
       * @param value the value to check
       * @returns <code>true</code> if the value lies within the interval, <code>false</code> otherwise
       */
      contains(value:number):boolean;
      /**
       * Checks whether or not the given {@link yfiles.router.Interval interval} fully covers this {@link yfiles.router.Interval interval} given a small epsilon value.
       * <p>
       * The given interval covers this interval, if <code>other lower bound - epsilon &lt;= this lower bound</code> and
       * <code>this upper bound &lt;= other upper bound + epsilon</code>.
       * </p>
       * @param other the other interval
       * @param [eps=1E-06] the epsilon value that determines the extended range of the other interval
       * @returns <code>true</code> if this interval is fully covered by the other interval, <code>false</code> otherwise
       */
      coveredBy(other:yfiles.router.Interval,eps?:number):boolean;
      /**
       * Returns the distance between this {@link yfiles.router.Interval interval} and the given {@link yfiles.router.Interval interval}.
       * <p>
       * If the intervals {@link yfiles.router.Interval#intersects intersect}, the distance is <code>0</code>. Otherwise, it is equal to
       * <code>dist(minimum of upper bounds, maximum of lower bounds)</code>.
       * </p>
       * @param other the other interval to which the distance is calculated
       * @returns the distance between this interval and the given interval
       */
      distanceTo(other:yfiles.router.Interval):number;
      /**
       * Returns the distance between the value and this {@link yfiles.router.Interval interval}.
       * <p>
       * If this interval contains the specified value, the distance is <code>0</code>. Otherwise, it is the minimum of <code>dist(value, lower bound)</code>
       * and <code>dist(value, upper bound)</code>.
       * </p>
       * @param value the value to which the distance is calculated
       * @returns the distance between the value and the interval
       */
      distanceTo(value:number):number;
      /**
       * Returns the {@link yfiles.router.Interval interval} bound that is closest to the given value.
       * @param value the value for which the closest interval bound will be returned
       * @returns 
       * <ul>
       * <li>the <code>value</code>, if the given value lies within the interval, or</li>
       * <li>the lower bound, if the given value is smaller than or equal to the lower bound, or</li>
       * <li>the upper bound, if the given value is greater than or equal to the upper bound</li>
       * </ul>
       */
      getClosest(value:number):number;
      /**
       * Checks whether or not this {@link yfiles.router.Interval interval} and the given {@link yfiles.router.Interval interval} cover the same range.
       * <p>
       * The intervals are considered to cover the same range, if their minimum values as well as their maximum values differ
       * only in the given epsilon value.
       * </p>
       * @param other the other interval
       * @param [eps=1E-06] the maximum distance between the two minimum and the two maximum values
       * @returns <code>true</code> if the intervals cover the same range, <code>false</code> otherwise
       */
      hasSameRange(other:yfiles.router.Interval,eps?:number):boolean;
      /**
       * Checks whether or not this {@link yfiles.router.Interval interval} intersects with the given
       * {@link yfiles.router.Interval interval}, considering the given minimum intersection length.
       * @param other the other interval to check
       * @param [minIntersection=0] the minimum length of the intersection of the intervals
       * @returns <code>true</code> if the intervals intersect at least by the given minimum length, <code>false</code> otherwise
       */
      intersects(other:yfiles.router.Interval,minIntersection?:number):boolean;
      /**
       * Checks whether or not the lower bound of this {@link yfiles.router.Interval interval} is greater than the upper bound of the given interval.
       * @param other the interval to compare
       * @returns <code>true</code> if the current interval is on the right of the given one, <code>false</code> otherwise
       */
      isGreaterThan(other:yfiles.router.Interval):boolean;
      /**
       * Checks whether or not the upper bound of this {@link yfiles.router.Interval interval} is smaller than the lower bound of the given interval.
       * @param other the interval to compare
       * @returns <code>true</code> if the current interval is on the left of the given one, <code>false</code> otherwise
       */
      isLessThan(other:yfiles.router.Interval):boolean;
      /**
       * Gets the lower bound of the interval.
       * @type {number}
       */
      min:number;
      /**
       * Gets the upper bound of the interval.
       * @type {number}
       */
      max:number;
      /**
       * Gets the midpoint of the interval.
       * @type {number}
       */
      center:number;
      /**
       * Gets the size of the interval, i.e., the difference between the {@link yfiles.router.Interval#max upper bound} and the {@link yfiles.router.Interval#min lower bound}.
       * @type {number}
       */
      size:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class describes the obstacles that are used by implementations of {@link yfiles.router.IObstaclePartition} to subdivide an
     * area into {@link yfiles.router.PartitionCell}s.
     * @class
     */
    export interface Obstacle extends Object{}
    export class Obstacle {
      /**
       * Creates a new {@link yfiles.router.Obstacle} instance with the given bounds and additional data.
       * @param bounds the bounds of the obstacle
       * @param data the additional data associated to the obstacle
       */
      constructor(bounds:yfiles.algorithms.YRectangle,data:Object);
      /**
       * Gets the bounds of the obstacle.
       * @type {yfiles.algorithms.YRectangle}
       */
      bounds:yfiles.algorithms.YRectangle;
      /**
       * Gets the additional data related to the obstacle.
       * @type {Object}
       */
      data:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A horizontal or vertical {@link yfiles.router.Interval} that additionally stores a y- (in the horizontal case) or x-coordinate
     * (in the vertical case).
     * <p>
     * During a {@link yfiles.router.PathSearch}, orthogonal intervals are used for specifying the location where an edge enters or
     * exits a {@link yfiles.router.PartitionCell partition cell}.
     * </p>
     * @see yfiles.router.PathSearch
     * @class
     */
    export interface OrthogonalInterval extends Object{}
    export class OrthogonalInterval {
      /**
       * Creates a new {@link yfiles.router.OrthogonalInterval} instance using the <code>from</code> and <code>to</code> value as interval bounds.
       * @param from the start of the interval bounds
       * @param to the end of the interval bounds
       * @param location the location in the two-dimensional coordinate system
       * @param isVertical <code>true</code> if the orthogonal interval is vertical, <code>false</code> if it is horizontal
       * @throws {Stubs.Exceptions.ArgumentError} if the given range is <code>null</code>
       */
      constructor(from:number,to:number,location:number,isVertical:boolean);
      /**
       * Creates a new {@link yfiles.router.OrthogonalInterval} instance with the given interval bounds.
       * @param range the given range
       * @param location the location in the two-dimensional coordinate system
       * @param isVertical <code>true</code> if the orthogonal interval is vertical, <code>false</code> if it is horizontal
       * @throws {Stubs.Exceptions.ArgumentError} if the given range is <code>null</code>
       */
      constructor(range:yfiles.router.Interval,location:number,isVertical:boolean);
      /**
       * Calculates the intersection of the given {@link yfiles.router.OrthogonalInterval orthogonal intervals}.
       * <p>
       * If the returned orthogonal interval is vertical, it will have the same horizontal location as <code>i2</code> and the shared vertical
       * range of both intervals. If it is horizontal, it will have the same vertical location as <code>i2</code> and the shared horizontal
       * range of both intervals. The orthogonal intervals must have same orientation.
       * </p>
       * @param i1 the first orthogonal interval
       * @param i2 the second orthogonal interval
       * @returns the shared orthogonal interval of both given orthogonal intervals
       * @throws {Stubs.Exceptions.ArgumentError} if the orthogonal intervals have different orientations
       * @see yfiles.router.OrthogonalInterval#intersects
       * @static
       */
      static calculateIntersection(i1:yfiles.router.OrthogonalInterval,i2:yfiles.router.OrthogonalInterval):yfiles.router.OrthogonalInterval;
      /**
       * Calculates the union of the given {@link yfiles.router.OrthogonalInterval orthogonal intervals}.
       * <p>
       * If the returned orthogonal interval is vertical, it will have the same horizontal location as <code>i2</code> and the shared vertical
       * range of both orthogonal intervals. If it is horizontal, it will have the same vertical location as <code>i2</code> and the shared
       * vertical range of both orthogonal intervals. The orthogonal intervals must have same orientation.
       * </p>
       * @param i1 the first orthogonal interval
       * @param i2 the second orthogonal interval
       * @returns the union of both given orthogonal intervals
       * @throws {Stubs.Exceptions.ArgumentError} if the orthogonal intervals have different orientations
       * @see yfiles.router.OrthogonalInterval#intersects
       * @static
       */
      static calculateUnion(i1:yfiles.router.OrthogonalInterval,i2:yfiles.router.OrthogonalInterval):yfiles.router.OrthogonalInterval;
      /**
       * Checks whether or not this {@link yfiles.router.OrthogonalInterval orthogonal interval} is fully covered by the given {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * <p>
       * The given orthogonal interval covers this orthogonal interval, if this interval's range is covered by the given
       * interval's range. The location of both orthogonal intervals is ignored. The orthogonal intervals must have the same
       * orientation.
       * </p>
       * @param other the other orthogonal interval
       * @returns <code>true</code> if this orthogonal interval is fully covered by the other orthogonal interval, <code>false</code> otherwise
       * @throws {Stubs.Exceptions.ArgumentError} if the orthogonal intervals have different orientations
       * @see yfiles.router.PartitionCell
       */
      coveredBy(other:yfiles.router.OrthogonalInterval):boolean;
      /**
       * Determines whether or not this {@link yfiles.router.OrthogonalInterval orthogonal interval} crosses the given one.
       * <p>
       * The orthogonal intervals must have different orientations.
       * </p>
       * @param other the other orthogonal interval
       * @returns <code>true</code> if this orthogonal interval crosses the given one, <code>false</code> otherwise
       * @throws {Stubs.Exceptions.ArgumentError} if the given orthogonal intervals have the same orientation
       */
      crosses(other:yfiles.router.OrthogonalInterval):boolean;
      /**
       * Returns the distance of the {@link yfiles.router.OrthogonalInterval#range ranges} of this {@link yfiles.router.OrthogonalInterval orthogonal interval} and the given {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * <p>
       * The orthogonal intervals must have the same orientation.
       * </p>
       * @param other the other orthogonal interval
       * @returns <code>0</code> if the orthogonal intervals {@link #intersects intersect} each other, otherwise the minimal distance between the orthogonal intervals is
       * returned
       * @throws {Stubs.Exceptions.ArgumentError} if the orthogonal intervals have different orientations
       */
      distanceTo(other:yfiles.router.OrthogonalInterval):number;
      /**
       * Checks whether or not this {@link yfiles.router.OrthogonalInterval orthogonal interval} and the given {@link yfiles.router.OrthogonalInterval orthogonal interval} cover the same range.
       * <p>
       * If both orthogonal intervals cover the same range, they are considered the same. The location of both orthogonal
       * intervals is ignored. The orthogonal intervals must have the same orientation.
       * </p>
       * @param other the other orthogonal interval
       * @returns <code>true</code> if the orthogonal intervals cover the same range, <code>false</code> otherwise
       * @throws {Stubs.Exceptions.ArgumentError} if the orthogonal intervals have different orientations
       * @see yfiles.router.PartitionCell
       */
      hasSameRange(other:yfiles.router.OrthogonalInterval):boolean;
      /**
       * Checks whether or not this {@link yfiles.router.OrthogonalInterval orthogonal interval} intersects the given {@link yfiles.router.OrthogonalInterval orthogonal interval} and the intersection has a given
       * minimum intersection size.
       * <p>
       * Orthogonal intervals intersect each other, if their ranges intersect each other by at least the given minimum
       * intersection size. The location of both orthogonal intervals is ignored. The orthogonal intervals must have the same
       * orientation.
       * </p>
       * @param other the other orthogonal interval
       * @param [minIntersection=0] the minimum size of the intersection
       * @returns <code>true</code> if this interval intersects the given other orthogonal interval, <code>false</code> otherwise
       * @throws {Stubs.Exceptions.ArgumentError} if the orthogonal intervals have different orientations
       * @see yfiles.router.PartitionCell
       */
      intersects(other:yfiles.router.OrthogonalInterval,minIntersection?:number):boolean;
      /**
       * Returns the Manhattan distance of this {@link yfiles.router.OrthogonalInterval orthogonal interval} and the given {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * <p>
       * If both intervals have the same orientation, the Manhattan distance is the sum of the distance between the intervals'
       * ranges and the distance between the intervals' locations.
       * </p>
       * <p>
       * If the intervals have different orientations, the Manhattan distance is the sum of the distances between one interval's
       * range and the other interval's location.
       * </p>
       * @param other the other orthogonal interval
       * @returns the Manhattan distance between this interval and the given one
       */
      manhattanDistanceTo(other:yfiles.router.OrthogonalInterval):number;
      /**
       * Gets whether or not the {@link yfiles.router.OrthogonalInterval orthogonal interval} is oriented vertically.
       * @type {boolean}
       */
      vertical:boolean;
      /**
       * Gets the lower bound of the {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * @type {number}
       */
      min:number;
      /**
       * Gets the upper bound of the {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * @type {number}
       */
      max:number;
      /**
       * Gets the length of the {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * @see yfiles.router.Interval#size
       * @type {number}
       */
      size:number;
      /**
       * Gets the range of this {@link yfiles.router.OrthogonalInterval orthogonal interval}.
       * @type {yfiles.router.Interval}
       */
      range:yfiles.router.Interval;
      /**
       * Gets the vertical coordinate for horizontal intervals or the horizontal coordinate for vertical intervals,
       * respectively.
       * @type {number}
       */
      location:number;
      /**
       * Gets the center of this {@link yfiles.router.OrthogonalInterval orthogonal interval} in the 2D-coordinate system.
       * @type {yfiles.algorithms.YPoint}
       */
      center:yfiles.algorithms.YPoint;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class is a type-safe enumeration used for defining the border of a {@link yfiles.router.PartitionCell}.
     * @see yfiles.router.PartitionCell#createBorderInterval
     * @class
     */
    export interface PartitionCellBorder extends Object{}
    export class PartitionCellBorder {
      /**
       * Returns the type of the border that is passed through when entering a {@link yfiles.router.PartitionCell} in the given
       * direction.
       * @param direction the direction in which a {@link } has been entered
       * @returns the type of the border that is passed through when entering a {@link }
       * @see yfiles.router.PartitionCell
       * @static
       */
      static valueOfEnterDirection(direction:yfiles.layout.Direction):yfiles.router.PartitionCellBorder;
      /**
       * Returns the type of the border that is passed through when exiting a {@link yfiles.router.PartitionCell} in the given
       * direction.
       * @param direction the direction in which a {@link } has been exited
       * @returns the type of the border that is passed through when exiting a {@link }
       * @see yfiles.router.PartitionCell
       * @static
       */
      static valueOfExitDirection(direction:yfiles.layout.Direction):yfiles.router.PartitionCellBorder;
      /**
       * Returns the type of the border that is defined by the given {@link yfiles.layout.PortCandidate}.
       * <p>
       * The border type corresponds to the {@link yfiles.layout.PortCandidate#direction direction} of the given {@link yfiles.layout.PortCandidate}.
       * </p>
       * @param pc the port candidate for which the border type is returned
       * @returns the type of the border that is defined by the given port candidate
       * @see yfiles.layout.PortCandidate
       * @static
       */
      static valueOfPortCandidate(pc:yfiles.layout.PortCandidate):yfiles.router.PartitionCellBorder;
      /**
       * Returns the type of the border that is defined by the given {@link yfiles.layout.PortConstraint}.
       * <p>
       * The border type corresponds to the {@link yfiles.layout.PortConstraint#side side} of the given {@link yfiles.layout.PortConstraint}.
       * </p>
       * @param pc the port constraint for which the border type is returned
       * @returns the type of the border that is defined by the given port constraint
       * @see yfiles.layout.PortConstraint
       * @static
       */
      static valueOfPortConstraint(pc:yfiles.layout.PortConstraint):yfiles.router.PartitionCellBorder;
      /**
       * A type constant that defines the upper border of a {@link yfiles.router.PartitionCell}.
       * @see yfiles.router.PartitionCell#createBorderInterval
       * @const
       * @static
       * @type {yfiles.router.PartitionCellBorder}
       */
      static NORTH:yfiles.router.PartitionCellBorder;
      /**
       * A type constant that defines the bottom border of a {@link yfiles.router.PartitionCell}.
       * @see yfiles.router.PartitionCell#createBorderInterval
       * @const
       * @static
       * @type {yfiles.router.PartitionCellBorder}
       */
      static SOUTH:yfiles.router.PartitionCellBorder;
      /**
       * A type constant that defines the right border of a {@link yfiles.router.PartitionCell}.
       * @see yfiles.router.PartitionCell#createBorderInterval
       * @const
       * @static
       * @type {yfiles.router.PartitionCellBorder}
       */
      static EAST:yfiles.router.PartitionCellBorder;
      /**
       * A type constant that defines the left border of a {@link yfiles.router.PartitionCell}.
       * @see yfiles.router.PartitionCell#createBorderInterval
       * @const
       * @static
       * @type {yfiles.router.PartitionCellBorder}
       */
      static WEST:yfiles.router.PartitionCellBorder;
      /**
       * Gets the type of the border that is on the opposite side of the border defined by this type.
       * @type {yfiles.router.PartitionCellBorder}
       */
      mirrorBorder:yfiles.router.PartitionCellBorder;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.router.PartitionCell} represents a rectangular part of a {@link yfiles.router.IPartition} as result of the decomposition
     * process.
     * <p>
     * Each instance of {@link yfiles.router.PartitionCell} can hold arbitrary additional information (e.g. what element of the graph
     * is covered by a {@link yfiles.router.PartitionCell}) which can be provided using method {@link yfiles.router.PartitionCell#putData}.
     * Accessing and removing this information can be done using {@link yfiles.router.PartitionCell#getData} and
     * {@link yfiles.router.PartitionCell#removeData}, respectively.
     * </p>
     * @see yfiles.router.PartitionCellKeys
     * @class
     */
    export interface PartitionCell extends Object{}
    export class PartitionCell {
      /**
       * Creates a new {@link yfiles.router.PartitionCell} instance of the given {@link yfiles.router.IPartition} with the given bounds.
       * @param x the x-coordinate of the upper-left corner of the partition cell
       * @param y the y-coordinate of the upper-left corner of the partition cell
       * @param width the width of the partition cell
       * @param height the height of the partition cell
       * @param partition the partition to which the partition cell belongs
       * @see yfiles.router.IPartition
       */
      constructor(x:number,y:number,width:number,height:number,partition:yfiles.router.IPartition);
      /**
       * Creates a new {@link yfiles.router.PartitionCell} of the given {@link yfiles.router.IPartition} with the location and the size of the
       * given rectangle.
       * @param bounds the bounds describing the size and location of the partition cell
       * @param partition the partition to which the partition cell belongs
       * @see yfiles.router.IPartition
       */
      constructor(bounds:yfiles.algorithms.YRectangle,partition:yfiles.router.IPartition);
      /**
       * Clears all additional data for this {@link yfiles.router.PartitionCell}.
       * @see yfiles.router.PartitionCell#getData
       * @see yfiles.router.PartitionCell#putData
       * @see yfiles.router.PartitionCell#removeData
       */
      clearData():void;
      /**
       * Returns an {@link yfiles.router.OrthogonalInterval} that defines the location, the size and the orientation of the given
       * {@link yfiles.router.PartitionCellBorder}.
       * <p>
       * The values defining the border are:
       * </p>
       * <ul>
       * <li>{@link yfiles.router.PartitionCellBorder#WEST}</li>
       * <li>{@link yfiles.router.PartitionCellBorder#EAST}</li>
       * <li>{@link yfiles.router.PartitionCellBorder#NORTH}</li>
       * <li>{@link yfiles.router.PartitionCellBorder#SOUTH}</li>
       * </ul>
       * @param border a border of this partition cell
       * @returns an {@link } that defines the location, the size and the orientation of the given border
       * @see yfiles.router.PartitionCellBorder
       */
      createBorderInterval(border:yfiles.router.PartitionCellBorder):yfiles.router.OrthogonalInterval;
      /**
       * Returns the additional data associated with the given key for this {@link yfiles.router.PartitionCell}.
       * @param key the key whose associated data will be returned
       * @returns the additional data associated with the given key or <code>null</code> if there is no data associated with the given key
       * @see yfiles.router.PartitionCell#putData
       * @see yfiles.router.PartitionCell#removeData
       * @see yfiles.router.PartitionCell#clearData
       * @see yfiles.router.PartitionCellKeys
       */
      getData(key:Object):Object;
      /**
       * Stores the additional data associated with the given key for this {@link yfiles.router.PartitionCell}.
       * @param key the key with which the additional data will be associated
       * @param data the additional data
       * @returns the previous data associated with the given key or <code>null</code> if there was no data associated with the given key
       * @see yfiles.router.PartitionCell#getData
       * @see yfiles.router.PartitionCell#removeData
       * @see yfiles.router.PartitionCell#clearData
       * @see yfiles.router.PartitionCellKeys
       */
      putData(key:Object,data:Object):Object;
      /**
       * Removes the additional data associated with the given key for this {@link yfiles.router.PartitionCell}.
       * @param key the key for which the associated data will be removed
       * @returns the additional data associated with the given key or <code>null</code> if there is no data associated with the given key
       * @see yfiles.router.PartitionCell#getData
       * @see yfiles.router.PartitionCell#putData
       * @see yfiles.router.PartitionCell#clearData
       * @see yfiles.router.PartitionCellKeys
       */
      removeData(key:Object):Object;
      /**
       * Gets the {@link yfiles.router.IPartition partition} to which this {@link yfiles.router.PartitionCell} belongs.
       * @see yfiles.router.IPartition
       * @type {yfiles.router.IPartition}
       */
      partition:yfiles.router.IPartition;
      /**
       * Gets a unique identifier of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      id:number;
      /**
       * Gets the bounds of this {@link yfiles.router.PartitionCell}.
       * @type {yfiles.algorithms.YRectangle}
       */
      bounds:yfiles.algorithms.YRectangle;
      /**
       * Gets the x-coordinate of the left border of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      minX:number;
      /**
       * Gets the y-coordinate of the upper border of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      minY:number;
      /**
       * Gets the x-coordinate of the right border of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      maxX:number;
      /**
       * Gets the y-coordinate of the bottom border of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      maxY:number;
      /**
       * Gets the width of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      width:number;
      /**
       * Gets the height of this {@link yfiles.router.PartitionCell}.
       * @type {number}
       */
      height:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class defines keys to obtain additional information about a {@link yfiles.router.PartitionCell}.
     * <p>
     * In order to get the data of the {@link yfiles.router.PartitionCell} that is associated with the given key, method {@link yfiles.router.PartitionCell#getData}
     * should be used.
     * </p>
     * @class
     * @static
     */
    export interface PartitionCellKeys extends Object{}
    export class PartitionCellKeys {
      /**
       * The key references a {@link yfiles.collections.IList.<T>} of nodes whose bounds are covering a certain {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static NODES_KEY:Object;
      /**
       * The key references a {@link yfiles.collections.IList.<T>} of nodes being in node to edge distance to a certain {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static NODES_IN_NODE_TO_EDGE_DISTANCE_KEY:Object;
      /**
       * The key references a {@link yfiles.collections.IList.<T>} of {@link yfiles.layout.INodeLabelLayout}s covering a certain {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static NODE_LABEL_LAYOUTS_KEY:Object;
      /**
       * The key references a {@link yfiles.collections.IList.<T>} of {@link number} values representing factors that are multiplied by an edge's {@link yfiles.router.PenaltySettings#nodeLabelCrossingPenalty node label crossing costs} in case
       * the edge crosses the corresponding {@link yfiles.layout.INodeLabelLayout node label} registered with key
       * {@link yfiles.router.PartitionCellKeys#NODE_LABEL_LAYOUTS_KEY}.
       * @const
       * @static
       * @type {Object}
       */
      static NODE_LABEL_CROSSING_COST_FACTORS_KEY:Object;
      /**
       * The key references a {@link yfiles.collections.IList.<T>} of {@link yfiles.layout.IEdgeLabelLayout}s covering a certain {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static EDGE_LABEL_LAYOUTS_KEY:Object;
      /**
       * The key references a {@link yfiles.collections.IList.<T>} of {@link number} values representing factors that are multiplied by an edge's {@link yfiles.router.PenaltySettings#edgeLabelCrossingPenalty edge label crossing costs} in case
       * the edge crosses the corresponding {@link yfiles.layout.IEdgeLabelLayout edge label} registered with key
       * {@link yfiles.router.PartitionCellKeys#EDGE_LABEL_LAYOUTS_KEY}.
       * @const
       * @static
       * @type {Object}
       */
      static EDGE_LABEL_CROSSING_COST_FACTORS_KEY:Object;
      /**
       * The key references the {@link yfiles.layout.PartitionCellId} of the {@link yfiles.layout.PartitionGrid} cell covering a certain
       * {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static PARTITION_GRID_CELL_ID_KEY:Object;
      /**
       * The key references the row index of the {@link yfiles.layout.PartitionGrid} cell covering a certain
       * {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static PARTITION_GRID_ROW_INDEX_KEY:Object;
      /**
       * The key references the column index of the {@link yfiles.layout.PartitionGrid} cell covering a certain
       * {@link yfiles.router.PartitionCell}.
       * @const
       * @static
       * @type {Object}
       */
      static PARTITION_GRID_COLUMN_INDEX_KEY:Object;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.router.Path} represents the result of a path search as a list of consecutive {@link yfiles.router.CellEntrance}s.
     * <p>
     * A path starts with a {@link yfiles.router.CellEntrance} for a {@link yfiles.router.PartitionCell} of the source node and ends with one
     * for a {@link yfiles.router.PartitionCell} of the target node.
     * </p>
     * @see yfiles.router.PathSearch
     * @see yfiles.router.PathSearchResult
     * @class
     */
    export interface Path extends Object{}
    export class Path {
      /**
       * Creates a new instance of {@link yfiles.router.Path} for the given edge, {@link yfiles.router.CellEntrance cell entrances} and cost.
       * @param edge the edge for which the path was calculated
       * @param entrances the list of {@link } objects describing the edge path from source node to target node
       * @param cost the overall cost of this path
       */
      constructor(edge:yfiles.algorithms.Edge,entrances:yfiles.collections.IList<Object>,cost:number);
      /**
       * Returns the {@link yfiles.router.CellEntrance} at the given position of this path.
       * @param position the position of the {@link } to return
       * @returns the {@link } at the given position of this path
       * @see yfiles.router.Path#setEntrance
       */
      getEntrance(position:number):yfiles.router.CellEntrance;
      /**
       * Returns the number of {@link yfiles.router.CellEntrance} objects in this path.
       * @returns the number of {@link } objects in this path
       */
      length():number;
      /**
       * Returns the position of the first occurrence of the given {@link yfiles.router.CellEntrance} in this path, or <code>-1</code> if this path
       * does not contain this {@link yfiles.router.CellEntrance}.
       * @param entrance the {@link } for which the position is retrieved
       * @returns the position of the first occurrence of the given {@link } in this path, or <code>-1</code> if this path does not
       * contain this {@link }
       */
      positionOf(entrance:yfiles.router.CellEntrance):number;
      /**
       * Replaces the {@link yfiles.router.CellEntrance} at the given position of this path with the given {@link yfiles.router.CellEntrance}.
       * @param position the position of the {@link } to replace
       * @param entrance the {@link } to be stored at the given position
       * @see yfiles.router.Path#getEntrance
       */
      setEntrance(position:number,entrance:yfiles.router.CellEntrance):void;
      /**
       * Gets the edge for which this path was calculated.
       * @type {yfiles.algorithms.Edge}
       */
      edge:yfiles.algorithms.Edge;
      /**
       * Gets the overall cost of this path.
       * @type {number}
       */
      cost:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This is a pathfinding algorithm that calculates the shortest (i.e., the cheapest) paths for a set of edges through a
     * {@link yfiles.router.GraphPartition}.
     * <p>
     * It is based on an <a href="http://en.wikipedia.org/wiki/A*_search_algorithm">A*-algorithm</a> and uses {@link yfiles.router.PartitionCell}s as steps between source node and target node.
     * </p>
     * <p>
     * In each step, the algorithm takes a {@link yfiles.router.CellEntrance}, that consists mainly of a {@link yfiles.router.PartitionCell}
     * and from where it was entered, from a queue with all seen {@link yfiles.router.CellEntrance}s, determines all possible neighbor
     * cells and their enter intervals and enqueues the resulting {@link yfiles.router.CellEntrance}. To influence the order in which
     * the {@link yfiles.router.CellEntrance}s will be processed, the path search assigns real costs (like Dijkstra) as well as
     * heuristic costs (A*-algorithm's heuristic) to the enqueued {@link yfiles.router.CellEntrance}s. The real costs arise from
     * entering a neighbor cell, e.g., a bend has to be created, while the heuristic costs are an estimation of how expensive
     * it will be to reach the target node continuing the path with this neighboring cell. Therefore, the path search prefers
     * searching in the direction in which the target node lies. The {@link yfiles.router.CellEntrance} with the lowest combined costs
     * is processed next until the target cell is reached.
     * </p>
     * <p>
     * {@link yfiles.router.PathSearchExtension}s modify the path search as they are able to add start entrances and weight them with
     * costs. They also add real and heuristic costs to {@link yfiles.router.CellEntrance}s that are created for the currently entered
     * {@link yfiles.router.PartitionCell} and calculate shorter enter intervals that are less expensive to pass.
     * </p>
     * <p>
     * The algorithm gets a {@link yfiles.router.PathSearchContext} which provides information about the graph and the currently routed
     * edge. It stores the results of the path search that can then be retrieved calling {@link yfiles.router.PathSearchContext#pathSearchResult}.
     * </p>
     * @see yfiles.router.PathSearch#addPathSearchExtension
     * @see yfiles.router.PathSearch#addAdditionalEnterIntervalCalculator
     * @class
     */
    export interface PathSearch extends Object{}
    export class PathSearch {
      /**
       * Creates a new instance of {@link yfiles.router.PathSearch}.
       */
      constructor();
      /**
       * Adds a new interval calculator to the list of registered {@link yfiles.router.IEnterIntervalCalculator}s.
       * <p>
       * {@link yfiles.router.IEnterIntervalCalculator}s may add
       * {@link yfiles.router.CellEntrance}s with narrowed intervals to the queue. The interval is more specific and will be judged with
       * lower costs.
       * </p>
       * @param enterIntervalCalculator the calculator to add
       * @returns <code>true</code> if the calculator was successfully added, <code>false</code> otherwise
       */
      addAdditionalEnterIntervalCalculator(enterIntervalCalculator:yfiles.router.IEnterIntervalCalculator):boolean;
      /**
       * Adds the given extension to the list of {@link yfiles.router.PathSearchExtension}s.
       * <p>
       * An extension can influence the pathfinding process by adding costs for entering certain {@link yfiles.router.CellEntrance}s.
       * </p>
       * @param extension the extension to add to this path search
       * @returns <code>true</code> if the extension has been added, <code>false</code> otherwise
       */
      addPathSearchExtension(extension:yfiles.router.PathSearchExtension):boolean;
      /**
       * Calculates the costs for moving from the current {@link yfiles.router.CellEntrance} to the neighboring {@link yfiles.router.PartitionCell}
       * using different enter intervals.
       * <p>
       * It is called by {@link yfiles.router.PathSearch#handleNeighbor} to determine the costs for all
       * {@link yfiles.router.CellEntrance}s that it will create and enqueue afterwards.
       * </p>
       * <p>
       * The costs for the given enter intervals are retrieved from all registered
       * {@link yfiles.router.PathSearchExtension}s. The calculation stops when it reaches the given maximum cost value.
       * </p>
       * @param currentEntrance the current cell entrance
       * @param enteredCell the partition cell to enter
       * @param enterIntervals the different entering intervals of the entered cell
       * @param lastEdgeCellInfos the information about how the last cell was crossed
       * @param context the context information
       * @param costs the array in which the calculated costs for entering the neighbor cell via the according enter intervals shall be
       * written
       * @param maxAllowedCosts the maximum costs an enter interval may induce. If this cost is exceeded, no further additional costs for this interval
       * are calculated. Note that the entries in this array get modified during cost calculation
       * @see yfiles.router.PathSearchExtension#calculateCosts
       * @protected
       */
      calculateCosts(currentEntrance:yfiles.router.CellEntrance,enteredCell:yfiles.router.PartitionCell,enterIntervals:yfiles.router.OrthogonalInterval[],lastEdgeCellInfos:yfiles.router.EdgeCellInfo[],context:yfiles.router.PathSearchContext,costs:number[],maxAllowedCosts:number[]):void;
      /**
       * Returns the estimated costs for the rest of the path when using the given {@link yfiles.router.CellEntrance} for the next step
       * in the path search.
       * <p>
       * It is called by {@link yfiles.router.PathSearch#handleNeighbor} to determine the heuristic part of the costs with which the
       * entrance will be enqueued.
       * </p>
       * <p>
       * The heuristic costs for the given entrance are retrieved from all registered {@link yfiles.router.PathSearchExtension}s.
       * </p>
       * @param entrance the current entrance
       * @param context the context information
       * @returns the heuristic costs for the rest of the path if the given entrance is used
       * @see yfiles.router.PathSearchExtension#calculateHeuristicCosts
       * @protected
       */
      calculateHeuristicCosts(entrance:yfiles.router.CellEntrance,context:yfiles.router.PathSearchContext):number;
      /**
       * Resets all registered {@link yfiles.router.PathSearchExtension}s and {@link yfiles.algorithms.DataProviders} added to this
       * {@link yfiles.router.PathSearch}.
       * <p>
       * So, {@link yfiles.router.PathSearch} is ready to calculate paths for a new layout.
       * </p>
       * @see yfiles.router.PathSearchExtension#cleanUp
       */
      clear():void;
      /**
       * Decreases the given penalty settings for the current edge.
       * <p>
       * If finding a path for the current edge takes too long according to the {@link yfiles.router.EdgeRouter#maximumDuration maximum duration} of the {@link yfiles.router.EdgeRouter edge routing algorithm},
       * the path search for the current edge is canceled and restarted using decreased penalties. The <code>decreaseFactor</code> indicates,
       * how much the penalties shall be reduced.
       * </p>
       * <p>
       * This method is called by
       * {@link yfiles.router.PathSearch#findPathsForCurrentEdge}. If overriding this method, note that the penalty for creating bends
       * should not be reduced as this results in more possible turns of the edge path and, therefore, a longer runtime of the
       * path search. Furthermore, not all penalties should be decreased equally as these decreases would neutralize each other.
       * </p>
       * <p>
       * The <code>decreaseFactor</code> takes values from <code>[0,1]</code>, where <code>0</code> means no reduction while <code>1</code> means the strongest reduction.
       * </p>
       * @param penaltySettings the penalty settings whose penalties shall be reduced
       * @param decreaseFactor the factor with values between <code>0</code> and <code>1</code> that indicates how strong to reduce the penalties
       * @param context the context information of the current path search
       * @protected
       */
      decreasePenaltySettings(penaltySettings:yfiles.router.PenaltySettings,decreaseFactor:number,context:yfiles.router.PathSearchContext):void;
      /**
       * Informs all registered path search extensions about completing a path by calling their {@link yfiles.router.PathSearchExtension#finalizePath finalizePath(Path)} method.
       * <p>
       * That way, extensions can collect data about this path to use it later during path search.
       * </p>
       * <p>
       * This method is called by {@link yfiles.router.PathSearch#findPathsForCurrentEdge} and may be overridden to use a custom
       * finalization step.
       * </p>
       * @param path the path to finalize
       * @protected
       */
      finalizePath(path:yfiles.router.Path):void;
      /**
       * Finds paths for the edges in the given context and stores them in its {@link yfiles.router.PathSearchContext#pathSearchResult PathSearchResult}.
       * <p>
       * This is the main method of {@link yfiles.router.PathSearch}.
       * </p>
       * <p>
       * It initializes its extensions using {@link yfiles.router.PathSearchExtension#initializeEdges} and delegates the path search for
       * each edge to {@link yfiles.router.PathSearch#findPathsForCurrentEdge}.
       * </p>
       * <p>
       * The path calculations for all edges are finalized by calling the extensions' {@link yfiles.router.PathSearchExtension#finalizeEdges}
       * method and, after that, the path search result is filled with the path for each edge.
       * </p>
       * <p>
       * At last, the extensions are asked to finalize the path search result using their {@link yfiles.router.PathSearchExtension#finalizePathSearchResult}
       * callback.
       * </p>
       * @param context the context to use during the path search
       * @see yfiles.router.PathSearchContext#edges
       * @see yfiles.router.PathSearchContext#pathSearchResult
       */
      findPaths(context:yfiles.router.PathSearchContext):void;
      /**
       * Finds the path for the current edge in the given context.
       * <p>
       * This method:
       * </p>
       * <ul>
       * <li>Calls {@link yfiles.router.PathSearchExtension#initializeCurrentEdge} for all extensions.</li>
       * <li>Collects and enqueues all start entrances.</li>
       * <li>Iteratively processes the next cheapest cell entrance and</li>
       * <li>Calls {@link yfiles.router.PathSearchExtension#finalizeCurrentEdge} for all extensions.</li>
       * </ul>
       * <p>
       * It is called by {@link yfiles.router.PathSearch#findPaths} and may be overridden to skip certain edges or implement a custom
       * path search.
       * </p>
       * @param context the context information needed for finding a path
       * @protected
       */
      findPathsForCurrentEdge(context:yfiles.router.PathSearchContext):void;
      /**
       * Returns the path for the given edge if it has already been finalized.
       * <p>
       * The path is finalized if the {@link yfiles.router.PathSearch} chose it as the best result for the edge.
       * </p>
       * @param edge the edge for which the path is returned
       * @returns the finalized path for the given edge or <code>null</code> if no path has been found and finalized
       */
      getFinalizedPath(edge:yfiles.algorithms.Edge):yfiles.router.Path;
      /**
       * Adds
       * {@link yfiles.router.CellEntrance}s for every interval through which the neighboring cell can be entered from the current
       * entrance to the queue.
       * <p>
       * The algorithm calls this method in every step for each neighbor of the current cell to collect all next possible
       * entrances for the current path. This path consists of several entrances where each knows the entrance through which it
       * has been entered.
       * </p>
       * <p>
       * After calculating all possible enter intervals to the given neighboring cell, each interval gets rated with costs. If
       * there is already an entrance for the neighboring cell whose interval is the same with one of these intervals, this
       * entrance will be used and re-enqueued, so that the path search can still reach it. The current entrance is set as its
       * predecessor within the current path and its enter interval and costs will be updated.
       * </p>
       * <p>
       * If there is an entrance for the neighboring cell whose interval is intersected by a current interval, new entrances will
       * be created with the new enter intervals and enqueued. The same happens if there is no entrance that has been matched
       * with one of the current intervals, yet. Costs will be added. If there are some entries afterwards that are intersected
       * by the current interval and have higher costs, they will be removed from the queue.
       * </p>
       * <p>
       * This method is called during cost calculation. It may be overridden to change the interval handling for the
       * {@link yfiles.router.CellEntrance}s.
       * </p>
       * @param currentEntrance the current cell entrance
       * @param neighborCell the neighboring cell that is handled.
       * @param context the context information
       * @see yfiles.router.PathSearch#calculateCosts
       * @see yfiles.router.PathSearch#calculateHeuristicCosts
       * @protected
       */
      handleNeighbor(currentEntrance:yfiles.router.CellEntrance,neighborCell:yfiles.router.PartitionCell,context:yfiles.router.PathSearchContext):void;
      /**
       * Initializes the fields of this {@link yfiles.router.PathSearch}.
       * <p>
       * This method also calls {@link yfiles.router.PathSearchExtension#initialize} for all registered path search extensions.
       * </p>
       * @param configuration the configuration that the path search shall use
       */
      init(configuration:yfiles.router.PathSearchConfiguration):void;
      /**
       * Removes the given interval calculator from the list of registered {@link yfiles.router.IEnterIntervalCalculator}s.
       * @param enterIntervalCalculator the calculator to remove
       * @returns <code>true</code> if an interval calculator was removed as a result of this call, <code>false</code> if the given calculator was not part of the
       * list
       */
      removeAdditionalEnterIntervalCalculator(enterIntervalCalculator:yfiles.router.IEnterIntervalCalculator):boolean;
      /**
       * Removes the given extension from the list of {@link yfiles.router.PathSearchExtension}s.
       * @param extension the extension to remove from the path search
       * @returns <code>true</code> if an extension was removed as a result of this call, <code>false</code> if the given extension was not contained in the list
       */
      removePathSearchExtension(extension:yfiles.router.PathSearchExtension):boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class contains the basic configuration used by path search algorithms.
     * @class
     */
    export interface PathSearchConfiguration extends Object{}
    export class PathSearchConfiguration {
      /**
       * Creates a new instance of {@link yfiles.router.PathSearchConfiguration} used for a path search.
       * @param graph the graph containing the edges whose path will be searched
       * @param grouping the grouping information of the graph
       * @param edgeRouter the edge routing algorithm using this configuration
       */
      constructor(graph:yfiles.layout.LayoutGraph,grouping:yfiles.layout.GroupingSupport,edgeRouter:yfiles.router.EdgeRouter);
      /**
       * Gets the graph containing the edges whose path is searched.
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * Gets the grouping information of the graph.
       * @type {yfiles.layout.GroupingSupport}
       */
      grouping:yfiles.layout.GroupingSupport;
      /**
       * Gets the edge routing algorithm using this configuration.
       * @type {yfiles.router.EdgeRouter}
       */
      edgeRouter:yfiles.router.EdgeRouter;
      /**
       * Gets the remaining time (in milliseconds) within which the algorithm should complete its calculation.
       * @type {number}
       */
      remainingTime:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides context information that is useful for the {@link yfiles.router.PathSearch} algorithm.
     * <p>
     * Most notably, the context provides access to the edge that is currently being routed (see {@link yfiles.router.PathSearchContext#currentEdge}).
     * </p>
     * @class
     */
    export interface PathSearchContext extends Object{}
    export class PathSearchContext {
      /**
       * Creates a new instance of {@link yfiles.router.PathSearchContext}.
       * @param pathSearch the path search that uses this context
       * @param configuration the configuration used for the path search
       */
      constructor(pathSearch:yfiles.router.PathSearch,configuration:yfiles.router.PathSearchConfiguration);
      /**
       * Adds an additional source cell of the {@link yfiles.router.PathSearchContext#currentEdge current edge} if it is not already contained in the list of source cells.
       * @param cell the new source cell to add
       * @see yfiles.router.PathSearchContext#sourceCellCount
       * @see yfiles.router.PathSearchContext#getSourceCell
       */
      addSourceCell(cell:yfiles.router.PartitionCell):void;
      /**
       * Adds an additional target cell of the {@link yfiles.router.PathSearchContext#currentEdge current edge} if it is not already contained in the list of target cells.
       * @param cell the new target cell to add
       * @see yfiles.router.PathSearchContext#targetCellCount
       * @see yfiles.router.PathSearchContext#getTargetCell
       */
      addTargetCell(cell:yfiles.router.PartitionCell):void;
      /**
       * Returns the source cell with the given index in the list of all cells that are covered by the source node of the
       * {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @param index the index of the source cell
       * @returns the source cell with the given index in the list of all source cells
       */
      getSourceCell(index:number):yfiles.router.PartitionCell;
      /**
       * Returns the target cell with the given index in the list of all cells that are covered by the target node of the
       * {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @param index the index of the target cell
       * @returns the target cell with the given index in the list of all cells that are covered by the target node of the {@link #currentEdge current edge}
       */
      getTargetCell(index:number):yfiles.router.PartitionCell;
      /**
       * Determines whether or not the given cell is a source cell of the {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @param cell the cell
       * @returns <code>true</code> if the given cell is in the list of source cells, <code>false</code> otherwise
       * @see yfiles.router.PathSearchContext#sourceCellCount
       * @see yfiles.router.PathSearchContext#getSourceCell
       */
      isSourceCell(cell:yfiles.router.PartitionCell):boolean;
      /**
       * Determines whether or not the given cell is a target cell of the {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @param cell the cell
       * @returns <code>true</code> if the given cell is in the list of target cells, <code>false</code> otherwise
       * @see yfiles.router.PathSearchContext#targetCellCount
       * @see yfiles.router.PathSearchContext#getTargetCell
       */
      isTargetCell(cell:yfiles.router.PartitionCell):boolean;
      /**
       * Specifies the edges that shall be routed.
       * @param edges the list of edges that shall be routed
       */
      setEdges(edges:yfiles.algorithms.EdgeList):void;
      /**
       * Returns the number of all cells that are covered by the source node of the {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @returns the number of all cells that are covered by the source node
       */
      sourceCellCount():number;
      /**
       * Returns the number of all cells that are covered by the target node of the {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @returns the number of all cells that are covered by the target node
       */
      targetCellCount():number;
      /**
       * Gets the {@link yfiles.router.PathSearch} that uses this context.
       * @type {yfiles.router.PathSearch}
       */
      pathSearch:yfiles.router.PathSearch;
      /**
       * Gets the configuration used for the {@link yfiles.router.PathSearch}.
       * @type {yfiles.router.PathSearchConfiguration}
       */
      configuration:yfiles.router.PathSearchConfiguration;
      /**
       * Gets the {@link yfiles.algorithms.IEdgeCursor} to iterate over the edges that are routed.
       * @type {yfiles.algorithms.IEdgeCursor}
       */
      edges:yfiles.algorithms.IEdgeCursor;
      /**
       * Gets or sets the edge that is currently routed.
       * <p>
       * This edge is one of those returned by {@link yfiles.router.PathSearchContext#edges}.
       * </p>
       * @type {yfiles.algorithms.Edge}
       */
      currentEdge:yfiles.algorithms.Edge;
      /**
       * Gets the {@link yfiles.router.EdgeLayoutDescriptor} for the {@link yfiles.router.PathSearchContext#currentEdge current edge} containing edge specific settings for the path search.
       * @type {yfiles.router.EdgeLayoutDescriptor}
       */
      currentEdgeLayoutDescriptor:yfiles.router.EdgeLayoutDescriptor;
      /**
       * Gets an artificial {@link yfiles.router.PartitionCell partition cell} with the size of the bounding box of all source cells of the {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @see yfiles.router.PathSearchContext#getSourceCell
       * @type {yfiles.router.PartitionCell}
       */
      combinedSourceCell:yfiles.router.PartitionCell;
      /**
       * Gets an artificial {@link yfiles.router.PartitionCell partition cell} with the size of the bounding box of all target cells of the {@link yfiles.router.PathSearchContext#currentEdge current edge}.
       * @see yfiles.router.PathSearchContext#getTargetCell
       * @type {yfiles.router.PartitionCell}
       */
      combinedTargetCell:yfiles.router.PartitionCell;
      /**
       * Gets the results of the {@link yfiles.router.PathSearch}.
       * @type {yfiles.router.PathSearchResult}
       */
      pathSearchResult:yfiles.router.PathSearchResult;
      static $class:yfiles.lang.Class;
    }
    /**
     * Extensions are added to a {@link yfiles.router.PathSearch} to influence the search process.
     * <p>
     * An extension contains several callback methods that are used by the {@link yfiles.router.PathSearch} to, e.g., calculate the
     * costs for the next possible steps (i.e., entering the next partition cell) and to decide when the target is reached.
     * </p>
     * <p>
     * The {@link yfiles.router.PathSearch} uses the callbacks in the following order:
     * </p>
     * <ol>
     * <li>{@link yfiles.router.PathSearchExtension#initialize}</li>
     * <li>{@link yfiles.router.PathSearchExtension#initializeEdges}</li>
     * <li>{@link yfiles.router.PathSearchExtension#initializeCurrentEdge}</li>
     * <li>{@link yfiles.router.PathSearchExtension#appendStartEntrances}</li>
     * <li>{@link yfiles.router.PathSearchExtension#calculateStartEntranceCost}</li>
     * <li>{@link yfiles.router.PathSearchExtension#calculateCosts}</li>
     * <li>{@link yfiles.router.PathSearchExtension#calculateHeuristicCosts}</li>
     * <li>{@link yfiles.router.PathSearchExtension#isValidTargetEntrance}</li>
     * <li>{@link yfiles.router.PathSearchExtension#finalizePath}</li>
     * <li>{@link yfiles.router.PathSearchExtension#finalizeCurrentEdge} or {@link yfiles.router.PathSearchExtension#cancelCurrentEdge}</li>
     * <li>{@link yfiles.router.PathSearchExtension#finalizeEdges}</li>
     * <li>{@link yfiles.router.PathSearchExtension#finalizePathSearchResult}</li>
     * <li>{@link yfiles.router.PathSearchExtension#cleanUp}</li>
     * </ol>
     * @see yfiles.router.PathSearch#addPathSearchExtension
     * @see yfiles.router.PathSearchContext#currentEdge
     * @class
     */
    export interface PathSearchExtension extends Object{}
    export class PathSearchExtension {
      /**
       * Creates a new instance of {@link yfiles.router.PathSearchExtension}.
       */
      constructor();
      /**
       * Appends additional start {@link yfiles.router.CellEntrance entrances} for the path search of the current edge to the given list of all previously generated
       * entrances.
       * <p>
       * This method is called before starting the actual path search to determine possible starting points. The current
       * implementation doesn't add any {@link yfiles.router.CellEntrance}s. It may be overridden to add {@link yfiles.router.CellEntrance}s with
       * specific enter intervals, e.g., to consider {@link yfiles.layout.PortConstraint}s.
       * </p>
       * @param allStartEntrances a list of all previously generated entrances
       */
      appendStartEntrances(allStartEntrances:yfiles.collections.IList<Object>):void;
      /**
       * Calculates the costs for entering the given {@link yfiles.router.PartitionCell} via the given enter
       * {@link yfiles.router.OrthogonalInterval}.
       * <p>
       * The given {@link yfiles.router.EdgeCellInfo} describes how the partition cell that has been entered by the current entrance
       * would be traversed if this enter interval to the neighboring cell would be chosen.
       * </p>
       * <p>
       * The <code>maxAllowedCosts</code> describe the maximum costs that entering the neighboring cell via the given enter interval may
       * induce. If the maximum cost is exceeded, calculations that may further increase its cost may be skipped because this
       * interval won't be chosen anyway.
       * </p>
       * <p>
       * This implementation doesn't add any costs. It may be overridden to add costs, e.g., if entering through the given
       * interval would create a bend.
       * </p>
       * @param currentEntrance the entrance that was used for entering the current cell
       * @param enteredCell the neighboring cell that shall be entered
       * @param enterInterval the interval that shall be used for entering the neighboring cell
       * @param edgeCellInfo the information about how the current cell would be traversed if the neighboring cell would be entered by this enter
       * interval
       * @param maxAllowedCosts the maximum allowed costs for this enter interval
       * @returns the costs for entering the neighboring cell via the enter interval
       * @protected
       */
      calculateCosts(currentEntrance:yfiles.router.CellEntrance,enteredCell:yfiles.router.PartitionCell,enterInterval:yfiles.router.OrthogonalInterval,edgeCellInfo:yfiles.router.EdgeCellInfo,maxAllowedCosts:number):number;
      /**
       * Calculates the heuristic costs for the given {@link yfiles.router.CellEntrance} that describe the minimal costs that will arise
       * to finish the path if the given cell entrance is used as next step.
       * <p>
       * After evaluating the costs for each enter interval to a neighboring cell, this method is called for each of the
       * resulting {@link yfiles.router.CellEntrance}s.
       * </p>
       * <p>
       * This implementation doesn't add any costs. It may be overridden to add some heuristic costs, e.g., if the edge would
       * have to bend to reach the target node from the given {@link yfiles.router.CellEntrance}.
       * </p>
       * @param entrance the entrance to calculate the heuristic costs for the rest of the path
       * @returns the minimal costs that will arise to finish the path if the given cell entrance is used as next step
       */
      calculateHeuristicCosts(entrance:yfiles.router.CellEntrance):number;
      /**
       * Returns the cost for starting the path in the given entrance.
       * <p>
       * This method is called for each start entrance of the current edge.
       * </p>
       * <p>
       * The current implementation doesn't add any costs for the given {@link yfiles.router.CellEntrance}. It may be overridden to make
       * start entrances with less specific intervals more expensive. For example, intervals that allow more than just the {@link yfiles.layout.PortConstraint}
       * location to connect to the source node become more expensive.
       * </p>
       * @param startEntrance the start entrance for which the cost is calculated
       * @returns the cost for starting the path in the given entrance
       */
      calculateStartEntranceCost(startEntrance:yfiles.router.CellEntrance):number;
      /**
       * Handles the cancellation of the path search for the current edge.
       * <p>
       * This callback notifies the extension when the path search for the current edge is canceled. Then, the path search will
       * proceed with altering the penalties of the current edge and reinitializing it again.
       * </p>
       * <p>
       * This implementation does nothing. It may be overridden to reset some edge specific information, e.g., removing
       * previously added listeners.
       * </p>
       * @param context the context containing the current edge for which the path search has been canceled
       * @see yfiles.router.PathSearchExtension#initializeCurrentEdge
       * @see yfiles.router.PathSearchExtension#finalizeEdges
       */
      cancelCurrentEdge(context:yfiles.router.PathSearchContext):void;
      /**
       * Cleans up the extension from the path searches with the current {@link yfiles.router.PathSearchExtension#configuration configuration} and {@link yfiles.router.PathSearchExtension#context context}.
       */
      cleanUp():void;
      /**
       * Handles the completion of the path search for the current edge.
       * <p>
       * This callback notifies the extension when enough paths are found for the current edge. After that, the path search
       * either proceeds with initializing the next current edge in the context's edge list or calls {@link yfiles.router.PathSearchExtension#finalizeEdges}
       * if all edges in this list have been handled.
       * </p>
       * <p>
       * This implementation does nothing. It may be overridden to add/reset some edge specific information, e.g., removing
       * previously added listeners or adding new start entrances.
       * </p>
       * @param context the context containing the current edge for which the path search has been finished
       * @see yfiles.router.PathSearchExtension#initializeCurrentEdge
       * @see yfiles.router.PathSearchExtension#finalizeEdges
       */
      finalizeCurrentEdge(context:yfiles.router.PathSearchContext):void;
      /**
       * Handles the completion of all marked edges.
       * <p>
       * This callback notifies the extension if enough paths have been found for all edges in the context's edge list.
       * </p>
       * @param context the context containing the list of edges for which paths have been found
       * @see yfiles.router.PathSearchContext#edges
       */
      finalizeEdges(context:yfiles.router.PathSearchContext):void;
      /**
       * After finding a valid target entrance and creating a {@link yfiles.router.Path}, the extension is notified of the found path.
       * <p>
       * If the path search is configured to calculate several possible paths for an edge, the path search proceeds with choosing
       * another unhandled {@link yfiles.router.CellEntrance}.
       * </p>
       * <p>
       * This implementation does nothing. It may be overridden to collect some information about the given path that may
       * influence the path search of later edges.
       * </p>
       * @param path the path found for the current edge in the context
       * @see yfiles.router.PathSearchContext#currentEdge
       */
      finalizePath(path:yfiles.router.Path):void;
      /**
       * Completes the path search result.
       * <p>
       * This callback notifies the extensions of the paths chosen for the edges in the current context. After calling
       * {@link yfiles.router.PathSearchExtension#finalizeEdges finalizeEdges}, the {@link yfiles.router.PathSearch} decides, which of the found paths to use for each edge and adds them to the
       * {@link yfiles.router.PathSearchResult}.
       * </p>
       * <p>
       * With this callback, the registered extensions are notified of this result before the path search either initializes the
       * next list of edges to route or ends the path search by calling
       * {@link yfiles.router.PathSearchExtension#cleanUp}. This implementation does nothing.
       * </p>
       * @param pathSearchResult the path search result for the edge in the current context's edge list
       * @see yfiles.router.PathSearchExtension#initializeEdges
       * @see yfiles.router.PathSearchExtension#cleanUp
       */
      finalizePathSearchResult(pathSearchResult:yfiles.router.PathSearchResult):void;
      /**
       * Initializes this extension for path searches using the given configuration.
       * <p>
       * This method is the first one to be called by the {@link yfiles.router.PathSearch}. It may be overridden to initialize this
       * extension with necessary settings like {@link yfiles.layout.GroupingSupport}.
       * </p>
       * @param configuration the configuration to use for the following path searches
       */
      initialize(configuration:yfiles.router.PathSearchConfiguration):void;
      /**
       * Initializes this extension with the current edge set in the given context.
       * <p>
       * This method is called for each edge of the {@link yfiles.router.PathSearchExtension#context context}. Each call will be balanced by a call to either {@link yfiles.router.PathSearchExtension#finalizeCurrentEdge}
       * or {@link yfiles.router.PathSearchExtension#cancelCurrentEdge}.
       * </p>
       * <p>
       * It may be overridden to initialize/reset the settings for the current edge (e.g. minimum distances).
       * </p>
       * @param context the context containing the current edge
       * @see yfiles.router.PathSearchContext#currentEdge
       */
      initializeCurrentEdge(context:yfiles.router.PathSearchContext):void;
      /**
       * Initializes this extension with the context that contains the list of edges for which paths are calculated.
       * <p>
       * This method may be called several times during a path search. Each call will be balanced by calls to {@link yfiles.router.PathSearchExtension#finalizeEdges}
       * and {@link yfiles.router.PathSearchExtension#finalizePathSearchResult}.
       * </p>
       * <p>
       * It may be overridden to initialize/reset the settings for the routed edges (e.g. minimum distances).
       * </p>
       * @param context the context containing the list of edges for which paths shall be found
       * @see yfiles.router.PathSearchContext#edges
       */
      initializeEdges(context:yfiles.router.PathSearchContext):void;
      /**
       * Determines whether or not this extension considers the given {@link yfiles.router.CellEntrance} as a valid target entrance,
       * i.e., the path may end with the given entrance.
       * <p>
       * Each time a {@link yfiles.router.CellEntrance} is chosen as next step, all registered extensions are asked if this entrance is a
       * valid target entrance. Only if none of the extensions returns <code>false</code>, a {@link yfiles.router.Path} is created.
       * </p>
       * <p>
       * This implementation accepts all
       * {@link yfiles.router.CellEntrance}s as valid targets. It may be overridden to only activate the target if the route fulfills a
       * certain condition.
       * </p>
       * @param entrance the entrance to decide if it is a valid target entrance
       * @returns <code>true</code> if the path may end with this entrance, <code>false</code> otherwise
       */
      isValidTargetEntrance(entrance:yfiles.router.CellEntrance):boolean;
      /**
       * Gets the configuration used for the path search.
       * <p>
       * This property is initialized in {@link yfiles.router.PathSearchExtension#initialize}.
       * </p>
       * @type {yfiles.router.PathSearchConfiguration}
       */
      configuration:yfiles.router.PathSearchConfiguration;
      /**
       * Gets the current context of the path search.
       * <p>
       * This property is initialized in {@link yfiles.router.PathSearchExtension#initializeEdges}.
       * </p>
       * @protected
       * @type {yfiles.router.PathSearchContext}
       */
      context:yfiles.router.PathSearchContext;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class manages the best {@link yfiles.router.Path}s found for each edge and provides according {@link yfiles.router.EdgeInfo}s.
     * <p>
     * These are used by the {@link yfiles.router.ChannelBasedPathRouting} algorithm to calculate the final segment locations for the
     * segments of an edge path.
     * </p>
     * @class
     */
    export interface PathSearchResult extends Object{}
    export class PathSearchResult {
      /**
       * Creates a new instance of {@link yfiles.router.PathSearchResult}.
       * @param configuration the configuration used for the path search
       */
      constructor(configuration:yfiles.router.PathSearchConfiguration);
      /**
       * Returns an {@link yfiles.router.EdgeInfo} for the given edge.
       * <p>
       * If the given edge shall be routed but no path has been set for it, yet, <code>null</code> is returned. If no {@link yfiles.router.EdgeInfo}
       * has been created for the path by this context before, a new one is instantiated, otherwise the stored one is returned.
       * </p>
       * @param edge the edge for which the {@link } will be returned
       * @returns an {@link } describing the path of the edge
       * @see yfiles.router.PathSearchResult#setPath
       */
      getEdgeInfo(edge:yfiles.algorithms.Edge):yfiles.router.EdgeInfo;
      /**
       * Returns an {@link yfiles.router.EdgeInfo} for the given path.
       * <p>
       * If no {@link yfiles.router.EdgeInfo} has been created for this path by this context before, a new one is instantiated, otherwise
       * the stored one is returned.
       * </p>
       * @param path the path for which the {@link } will be returned
       * @returns an {@link } describing the given path
       */
      getEdgeInfo(path:yfiles.router.Path):yfiles.router.EdgeInfo;
      /**
       * Returns the previously registered path found for the edge.
       * @param edge the edge for which the path is provided
       * @returns the previously registered path found for the edge or <code>null</code> if no path has been registered, yet
       */
      getPath(edge:yfiles.algorithms.Edge):yfiles.router.Path;
      /**
       * Specifies a found path for an edge.
       * @param edge the edge for which the path is set
       * @param path the found path
       */
      setPath(edge:yfiles.algorithms.Edge,path:yfiles.router.Path):void;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides the cost penalties that are applied for violating restrictions during a {@link yfiles.router.PathSearch}.
     * <p>
     * The {@link yfiles.router.PathSearch} calculates the <em>shortest</em> path from the source to the target, where <em>shortest</em> path means the path with the
     * lowest costs. Costs result from violations of restrictions that are defined by the different
     * {@link yfiles.router.PathSearchExtension}s. This class defines penalties for the various violations. The {@link yfiles.router.PathSearch}
     * can be adapted to specific needs by varying these penalties. In order to avoid a certain violation, the appropriate
     * penalty must be increased.
     * </p>
     * <p>
     * There are some predefined penalty settings which set a different focus for the path search:
     * {@link yfiles.router.PenaltySettings#OPTIMIZATION_BALANCED}, {@link yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_BENDS}, {@link yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_LENGTHS}
     * and {@link yfiles.router.PenaltySettings#OPTIMIZATION_EDGE_CROSSINGS}.
     * </p>
     * @class
     */
    export interface PenaltySettings extends Object{}
    export class PenaltySettings {
      /**
       * Creates a new instance of {@link yfiles.router.PenaltySettings}.
       */
      constructor();
      /**
       * Creates a copy of this {@link yfiles.router.PenaltySettings} instance.
       * <p>
       * This implementation uses method {@link yfiles.router.PenaltySettings#newInstance} to obtain a new instance which adopts all
       * settings.
       * </p>
       * @returns a copy of this instance
       */
      createCopy():yfiles.router.PenaltySettings;
      /**
       * Creates a new instance of {@link yfiles.router.PenaltySettings}.
       * <p>
       * This factory method is used by
       * {@link yfiles.router.PenaltySettings#createCopy}. Subclasses can override this method in order to return a more specialized
       * version.
       * </p>
       * @returns a specialized {@link }
       * @protected
       */
      newInstance():yfiles.router.PenaltySettings;
      /**
       * Optimization strategy that balances bends and edge crossings in the edge routes.
       * <p>
       * By default, this strategy is used for the path search.
       * </p>
       * @const
       * @static
       * @type {yfiles.router.PenaltySettings}
       */
      static OPTIMIZATION_BALANCED:yfiles.router.PenaltySettings;
      /**
       * Optimization strategy that minimizes bends in the edge routes.
       * <p>
       * Edges may cross other edges to prevent bends occurring around them.
       * </p>
       * @const
       * @static
       * @type {yfiles.router.PenaltySettings}
       */
      static OPTIMIZATION_EDGE_BENDS:yfiles.router.PenaltySettings;
      /**
       * Optimization strategy that minimizes edge crossings in the edge routes.
       * <p>
       * This may cause more bends.
       * </p>
       * @const
       * @static
       * @type {yfiles.router.PenaltySettings}
       */
      static OPTIMIZATION_EDGE_CROSSINGS:yfiles.router.PenaltySettings;
      /**
       * Optimization strategy that minimizes the length of the edge routes.
       * <p>
       * In consequence, there may be more edge crossings.
       * </p>
       * @const
       * @static
       * @type {yfiles.router.PenaltySettings}
       */
      static OPTIMIZATION_EDGE_LENGTHS:yfiles.router.PenaltySettings;
      /**
       * Gets or sets the penalty for the edge length.
       * <p>
       * This penalty will make long routes more expensive than short routes so edges will preferably be kept short.
       * </p>
       * <p>
       * Increasing the edge length penalty will raise the importance of short edges in relation to all other penalties while
       * decreasing this penalty will raise the importance of all other restrictions. A high edge length penalty will result in
       * routes that have less bends and more edge crossings to maintain the edge as short as possible.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @type {number}
       */
      edgeLengthPenalty:number;
      /**
       * Gets or sets the penalty for an edge bend.
       * <p>
       * This penalty will make routes with many bends more expensive than routes with few or no bends. Hence, edges will have
       * preferably few bends.
       * </p>
       * <p>
       * Increasing bend penalty will raise the importance of avoiding bends in relation to other penalties while decreasing this
       * penalty will raise the importance of all other restrictions. A high bend penalty will result in routes that will
       * preferably cross other edges instead of bending to avoid other edges. When increasing the edge bend penalty, the
       * resulting routes will have more edge crossings.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @type {number}
       */
      bendPenalty:number;
      /**
       * Gets or sets the penalty for a crossing of two edges.
       * <p>
       * This penalty will make routes that cross many other edges more expensive than routes that cross few other edges or no
       * edges at all. Hence, edges preferably won't cross other edges.
       * </p>
       * <p>
       * Increasing the edge crossing penalty will raise the importance of avoiding edge crossings in relation to other
       * penalties, while decreasing this penalty will raise the importance of all other restrictions. When increasing the edge
       * crossing penalty, the resulting route will be longer and have more bends because crossing other edges needs to be
       * avoided.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @type {number}
       */
      edgeCrossingPenalty:number;
      /**
       * Gets or sets the penalty for an edge crossing a regular node.
       * <p>
       * This penalty will make routes that cross normal nodes more expensive than routes that avoid crossing nodes. Hence, edges
       * won't preferably cross any nodes.
       * </p>
       * <p>
       * Increasing the node crossing penalty will raise the importance of avoiding node crossings in relation to other
       * penalties, while decreasing this penalty will raise the importance of all other restrictions.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @type {number}
       */
      nodeCrossingPenalty:number;
      /**
       * Gets or sets the penalty for an edge crossing a group node.
       * <p>
       * This penalty will make routes that cross group nodes more expensive than routes that avoid crossing group nodes. Hence,
       * edges won't preferably cross any group nodes.
       * </p>
       * <p>
       * Increasing the group node crossing penalty will raise the importance of avoiding group node crossings in relation to
       * other penalties, while decreasing this penalty will raise the importance of all other restrictions.
       * </p>
       * @type {number}
       */
      groupNodeCrossingPenalty:number;
      /**
       * Gets or sets the penalty for an edge crossing a node label.
       * <p>
       * This penalty will make routes that cross node labels more expensive than routes that avoid crossing node labels. Hence,
       * edges won't preferably cross any node labels.
       * </p>
       * <p>
       * Increasing the node label crossing penalty will raise the importance of avoiding node labels crossings in relation to
       * other penalties, while decreasing this penalty will raise the importance of all other restrictions. A high label
       * crossing penalty will cause edges to bend more often to avoid passing straight through labels.
       * </p>
       * <p>
       * The crossing penalty can be weighted for each label individually using the
       * {@link yfiles.router.EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY}.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeRouter#considerNodeLabels} to set whether node labels should be considered when routing the edges.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeRouter#considerNodeLabels
       * @type {number}
       */
      nodeLabelCrossingPenalty:number;
      /**
       * Gets or sets the penalty for an edge crossing an edge label.
       * <p>
       * This penalty will make routes that cross edge labels of fixed edges more expensive than routes that avoid crossing edge
       * labels, so edges won't preferably cross any edge labels that belong to fixed edges.
       * </p>
       * <p>
       * Increasing the edge label crossing penalty will raise the importance of avoiding edge labels crossings in relation to
       * other penalties, while decreasing this penalty will raise the importance of all other restrictions. A high label
       * crossing penalty will cause edges to bend more often to avoid passing straight through labels.
       * </p>
       * <p>
       * The crossing penalty can be weighted for each label individually using the
       * {@link yfiles.router.EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY}.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeRouter#considerEdgeLabels} to set whether edge labels should be considered when routing the edges.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeRouter#considerEdgeLabels
       * @type {number}
       */
      edgeLabelCrossingPenalty:number;
      /**
       * Gets or sets the penalty for violations of the minimum distance between any edge and any normal node side.
       * <p>
       * This penalty will make routes that pass too close to normal nodes more expensive than routes that maintain the specified
       * minimum node to edge distance.
       * </p>
       * <p>
       * Increasing the minimum node to edge distance penalty will raise the importance of maintaining the minimum distance to
       * nodes, while decreasing this penalty will raise the importance of all other restrictions. A high penalty for violating
       * the minimum distance between edges and nodes will produce edges that rather use a long route around nodes to maintain
       * the specified distance than pass through a small channel between two nodes.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeRouter#minimumNodeToEdgeDistance
       * @type {number}
       */
      minimumNodeToEdgeDistancePenalty:number;
      /**
       * Gets or sets the penalty for violations of the minimum distance between any edge and any group node side.
       * <p>
       * This penalty will make routes that pass group nodes too close more expensive than routes that maintain the specified
       * minimum node to edge distance.
       * </p>
       * <p>
       * Increasing the minimum group node to edge distance penalty will raise the importance of maintaining the minimum distance
       * to group nodes, while decreasing this penalty will raise the importance of all other restrictions. A high penalty for
       * violating the minimum distance between edges and group nodes will produce edges that rather use a long route around
       * group nodes to maintain the specified distance than pass through a small channel between two group nodes.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeRouter#minimumNodeToEdgeDistance} to set the minimum distance.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeRouter#minimumNodeToEdgeDistance
       * @type {number}
       */
      minimumGroupNodeToEdgeDistancePenalty:number;
      /**
       * Gets or sets the penalty for violations of the minimum distance between any two edges.
       * <p>
       * This penalty will make routes that pass other edges too close more expensive than routes that maintain the specified
       * minimum edge to edge distance.
       * </p>
       * <p>
       * Increasing the minimum edge to edge distance penalty will raise the importance of maintaining the minimum distance to
       * other edges, while decreasing this penalty will raise the importance of all other restrictions. When this penalty has a
       * high value, edges will maintain the specified distance. This may also reduce the number of edges that pass through a
       * small channel between nodes. So, if such a channel is part of the shortest routes for many edges, some of them will take
       * a longer way.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeLayoutDescriptor#minimumEdgeToEdgeDistance} to set the minimum distance for a certain edge to any other edge.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#minimumEdgeToEdgeDistance
       * @type {number}
       */
      minimumEdgeToEdgeDistancePenalty:number;
      /**
       * Gets or sets the penalty for violations of the minimum distance the edge shall maintain from node corners when entering
       * or leaving the node.
       * <p>
       * This penalty will make routes that end too close to the corner of their source or target node more expensive than routes
       * that maintain the specified minimum node corner distance.
       * </p>
       * <p>
       * Increasing the node corner distance penalty will raise the importance of maintaining the minimum distance to the corner
       * of the adjacent nodes, while decreasing this penalty will raise the importance of all other restrictions.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeLayoutDescriptor#minimumNodeCornerDistance} to set the minimum node corner distance for a certain edge.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#minimumNodeCornerDistance
       * @type {number}
       */
      minimumNodeCornerDistancePenalty:number;
      /**
       * Gets or sets the penalty for violations of the minimum length of the first and last segment of an edge.
       * <p>
       * This penalty will make routes whose first/last bend is too close to their source or target node more expensive than
       * routes that maintain the specified minimum first/last segment length.
       * </p>
       * <p>
       * Increasing the minimum first last segment length penalty will raise the importance of maintaining the first and last
       * segment as long as they fit the minimum first/last segment length, while decreasing this penalty will raise the
       * importance of all other restrictions.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeLayoutDescriptor#minimumFirstSegmentLength} to set the minimum length of the first segment and {@link yfiles.router.EdgeLayoutDescriptor#minimumLastSegmentLength} to set the minimum length of the last segment of a certain
       * edge.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#minimumFirstSegmentLength
       * @see yfiles.router.EdgeLayoutDescriptor#minimumLastSegmentLength
       * @type {number}
       */
      minimumFirstLastSegmentLengthPenalty:number;
      /**
       * Gets or sets the penalty for an edge bend being closer to the border of the source node, the target node or one of their
       * ancestor group nodes than the minimum node to edge distance.
       * <p>
       * This penalty will make routes that bend too close to their source or target node more expensive than routes that
       * maintain the specified minimum node to edge distance.
       * </p>
       * <p>
       * Increasing the bends in node to edge distance penalty will raise the importance of avoiding bends that violate the
       * minimum node to edge distance, while decreasing this penalty will raise the importance of all other restrictions. This
       * penalty supports
       * {@link yfiles.router.EdgeRouter#minimumNodeToEdgeDistance}. In case the path search finally reaches the target node, the minimum node to edge distance for this node must be
       * violated. Penalizing bends that are too close to the node forces the edge to directly cross this distance and connect to
       * the node.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeRouter#minimumNodeToEdgeDistance
       * @type {number}
       */
      bendsInNodeToEdgeDistancePenalty:number;
      /**
       * Gets or sets the penalty for violating the monotonic path restrictions of an edge.
       * <p>
       * This penalty will make routes with detours m ore expensive than routes that stay monotone.
       * </p>
       * <p>
       * Increasing the monotony violation penalty will raise the importance of avoiding detours in the specified direction,
       * while decreasing this penalty will raise the importance of all other restrictions.
       * </p>
       * <p>
       * Use {@link yfiles.router.EdgeLayoutDescriptor#monotonicPathRestriction} to set the monotonic path restrictions of a certain edge.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.router.EdgeLayoutDescriptor#monotonicPathRestriction
       * @type {number}
       */
      monotonyViolationPenalty:number;
      /**
       * Gets or sets the penalty for an edge leaving and reentering in the same {@link yfiles.layout.PartitionGrid partition grid} cell.
       * <p>
       * This penalty will make routes that re-enter the same partition grid cell more expensive than routes taking a more direct
       * way.
       * </p>
       * <p>
       * Increasing the partition grid reentrance penalty will raise the importance of taking the most direct way through the
       * partition grid, while decreasing this penalty will raise the importance of all other restrictions.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.layout.PartitionGrid
       * @type {number}
       */
      partitionGridCellReentrancePenalty:number;
      /**
       * Gets or sets the penalty if an edge does not comply with its {@link yfiles.layout.PortCandidate}s or
       * {@link yfiles.layout.PortConstraint}s.
       * <p>
       * This penalty will make routes that start or end at a point that doesn't comply with its port constraints more expensive
       * than routes that obey them.
       * </p>
       * <p>
       * Increasing the port violation penalty will raise the importance of maintaining port constraints, while decreasing this
       * penalty will raise the importance of all other restrictions. It is recommended to have high penalties for port violation
       * since the constraints lose purpose if they are disregarded.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.layout.PortCandidate
       * @see yfiles.layout.PortConstraint
       * @type {number}
       */
      portViolationPenalty:number;
      /**
       * Gets or sets the penalty for an edge with a group ID that is not grouped at source or target side.
       * <p>
       * This penalty will make routes for grouped edges that don't use partly the same path as the other edges in this group
       * more expensive than routes that stick to the group.
       * </p>
       * <p>
       * Increasing the invalid edge grouping penalty will raise the importance of grouping edges with the same group ID, while
       * decreasing this penalty will raise the importance of all other restrictions. As edge groups require that the route of an
       * edge belonging to an edge group stays partly identical to the other edges of the group, the route may not be optimal
       * with respect to the other restrictions. To maintain the edges together as long as possible, this penalty should be
       * higher than most others.
       * </p>
       * <p>
       * Grouped edges have the same source or target group ID assigned in the data provider registered with the graph with the {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY}
       * or {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY} key.
       * </p>
       * <p>
       * The value must be greater than or equal to <code>0</code>, otherwise the default value will be assigned.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {number}
       */
      invalidEdgeGroupingPenalty:number;
      /**
       * Gets or sets the penalty for bends of a self-loop edge.
       * <p>
       * This penalty will make self-loops leave and enter the node at different sides unless they are forced to one side by
       * {@link yfiles.layout.PortConstraint}s or {@link yfiles.layout.PortCandidate}s.
       * </p>
       * <p>
       * Increasing this penalty will stronger enforce routing the self-loops between two sides of the node but it will also
       * require more time for determining the edge path. Decreasing the penalty may result in self-loops that only connect to
       * one side of the node.
       * </p>
       * <p>
       * By default this value is set to <code>9</code>. The value must be <code>>= 0</code>, otherwise the default value will be assigned.
       * </p>
       * @type {number}
       */
      singleSideSelfLoopPenalty:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.router.PolylineLayoutStage} extends the orthogonal edge paths with polyline segments.
     * <p>
     * The current implementation only adds octilinear segments to all orthogonal edges. Non-orthogonal edges are completely
     * ignored.
     * </p>
     * <p>
     * An octilinear segment is added between two consecutive vertical and horizontal segments. The length of an octilinear
     * segment can be influenced by the {@link yfiles.router.PolylineLayoutStage#maximumNonOrthogonalSegmentRatio maximum non-orthogonal segment ratio} and the {@link yfiles.router.PolylineLayoutStage#preferredPolylineSegmentLength preferred polyline segment length}. The new
     * octilinear segments maintain a {@link yfiles.router.PolylineLayoutStage#minimumNodeToEdgeDistance minimum distance} between the edges and all nodes in the graph if possible.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface PolylineLayoutStage extends yfiles.layout.LayoutStageBase{}
    export class PolylineLayoutStage {
      /**
       * Creates a new instance of {@link yfiles.router.PolylineLayoutStage} using the given {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * @param [coreLayouter=null] the core layout routine
       */
      constructor(coreLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key used for looking up the selected state of the nodes of the graph to
       * be laid out.
       * <p>
       * If the {@link yfiles.router.PolylineLayoutStage#scope scope} is set to
       * {@link yfiles.router.Scope#ROUTE_EDGES_AT_AFFECTED_NODES}, only edges of selected nodes are routed, while all other edges are
       * considered to have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @see yfiles.router.PolylineLayoutStage#scope
       * @type {Object}
       */
      affectedNodesDpKey:Object;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key used for looking up the selected state of the edges of the graph to
       * be laid out.
       * <p>
       * If the {@link yfiles.router.PolylineLayoutStage#scope scope} is set to
       * {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES}, only the selected edges are routed, while all other edges are considered to
       * have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @see yfiles.router.PolylineLayoutStage#scope
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      /**
       * Gets or sets the (sub-)set of edges to be routed in a polyline fashion.
       * @throws {Stubs.Exceptions.ArgumentError} if the given argument is none of the predefined scope values
       * @see yfiles.router.PolylineLayoutStage#affectedEdgesDpKey
       * @see yfiles.router.PolylineLayoutStage#affectedNodesDpKey
       * @type {yfiles.router.Scope}
       */
      scope:yfiles.router.Scope;
      /**
       * Gets or sets the minimum distance between edges and node bounds.
       * <p>
       * Values should be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the distance is negative
       * @type {number}
       */
      minimumNodeToEdgeDistance:number;
      /**
       * Gets or sets the maximum segment length ratio at each end of an orthogonal segment that may get converted into a
       * (non-orthogonal) polyline segment.
       * <p>
       * The ratio must be between <code>0</code> and <code>0.5</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the ratio does not lie within <code>[0,0.5]</code> interval
       * @type {number}
       */
      maximumNonOrthogonalSegmentRatio:number;
      /**
       * Gets or sets the preferred segment length for (non-orthogonal) polyline segments.
       * <p>
       * Values should be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the length is negative
       * @type {number}
       */
      preferredPolylineSegmentLength:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Represents a group of segments of different edges that shall be combined at their common source or target.
     * <p>
     * Grouped edges have the same source or target group ID assigned in the {@link yfiles.algorithms.IDataProvider} registered with
     * the graph with {@link yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY} or {@link yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY}
     * keys.
     * </p>
     * <p>
     * The {@link yfiles.router.SegmentGroup} contains the {@link yfiles.router.SegmentGroup#commonLocationRange common location range} for the edge group and provides a {@link yfiles.router.SegmentGroup#commonSegmentInfo common segment info}
     * representing the grouped segments.
     * </p>
     * @class
     */
    export interface SegmentGroup extends Object{}
    export class SegmentGroup {
      /**
       * Creates a new instance of {@link yfiles.router.SegmentGroup} that contains the given segments and a range to place them.
       * @param commonLocationRange the location range of the given {@link }s
       * @param segmentInfos the list containing the grouped {@link }s
       */
      constructor(commonLocationRange:yfiles.router.Interval,segmentInfos:yfiles.collections.IList<Object>);
      /**
       * Gets the common location range for the {@link yfiles.router.SegmentGroup#segmentInfos segments} in this group.
       * <p>
       * For vertical grouped segments, this interval describes the horizontal range where this segment can be placed. Horizontal
       * grouped segments are placed in a vertical range.
       * </p>
       * @type {yfiles.router.Interval}
       */
      commonLocationRange:yfiles.router.Interval;
      /**
       * Gets the list of {@link yfiles.router.SegmentInfo}s contained in this group.
       * @type {yfiles.collections.IList.<Object>}
       */
      segmentInfos:yfiles.collections.IList<Object>;
      /**
       * Gets the common {@link yfiles.router.SegmentInfo} representing the grouped segments.
       * <p>
       * The returned segment info is used for placing the common segment. The coordinates are applied to all edges in the bundle
       * later.
       * </p>
       * @type {yfiles.router.SegmentInfo}
       */
      commonSegmentInfo:yfiles.router.SegmentInfo;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.router.SegmentInfo} holds information about the possible location of an orthogonal edge segment.
     * <p>
     * Most notably, it stores the {@link yfiles.router.SegmentInfoBase#direction direction} of the segment and the {@link yfiles.router.SegmentInfoBase#locationRange interval} that restricts the location of the segment.
     * </p>
     * @class
     * @extends {yfiles.router.SegmentInfoBase}
     */
    export interface SegmentInfo extends yfiles.router.SegmentInfoBase{}
    export class SegmentInfo {
      /**
       * Creates a new instance of {@link yfiles.router.SegmentInfo} with the given information.
       * @param edge the edge to which the segment of this info belongs
       * @param segmentIndex the index of the segment to which this info belongs
       * @param direction the direction to which the segment of this info points
       * @param locationRange the range within which the common location of this segment has to lie
       * @param minExtension the minimum interval in extension direction this segment is known to intersect
       * @param maxExtension the maximum interval in extension direction this segment will span
       * @param cellSegmentInfos the list of {@link } objects upon which this segment info is built
       */
      constructor(edge:yfiles.algorithms.Edge,segmentIndex:number,direction:yfiles.layout.Direction,locationRange:yfiles.router.Interval,minExtension:yfiles.router.Interval,maxExtension:yfiles.router.Interval,cellSegmentInfos:yfiles.collections.IList<Object>);
      /**
       * Creates a new instance of {@link yfiles.router.SegmentInfo} using a {@link yfiles.algorithms.LineSegment} to describe the edge segment.
       * @param edge the edge to which the segment of this info belongs
       * @param segmentIndex the index of the segment to which this info belongs
       * @param segment a line segment describing the edge segment
       */
      constructor(edge:yfiles.algorithms.Edge,segmentIndex:number,segment:yfiles.algorithms.LineSegment);
      /**
       * Returns the number of {@link yfiles.router.CellSegmentInfo} of this segment info.
       * @returns the number of {@link } of this segment info
       */
      cellSegmentInfoCount():number;
      /**
       * Returns the {@link yfiles.router.CellSegmentInfo} at the given index of this segment info.
       * @param index the index of the {@link } to return
       * @returns the {@link } at the given index of this segment info
       */
      getCellSegmentInfo(index:number):yfiles.router.CellSegmentInfo;
      /**
       * Sets the {@link yfiles.router.SegmentGroup segment group} for this segment info and its associated {@link yfiles.router.CellSegmentInfo}s.
       * <p>
       * The {@link yfiles.router.SegmentInfoBase#locationRange location ranges} are adjusted to use the {@link yfiles.router.SegmentGroup#commonLocationRange}.
       * </p>
       * @see yfiles.router.SegmentGroup
       * @type {yfiles.router.SegmentGroup}
       */
      segmentGroup:yfiles.router.SegmentGroup;
      /**
       * Gets or sets whether or not this edge segment has the restriction of a strong {@link yfiles.layout.PortConstraint}.
       * @type {boolean}
       */
      atStrongPortConstraint:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class stores location information for orthogonal edge segments.
     * <p>
     * Most notably, {@link yfiles.router.SegmentInfoBase} stores the {@link yfiles.router.SegmentInfoBase#direction direction} to which the segment points and the {@link yfiles.router.SegmentInfoBase#locationRange interval} describing
     * possible locations and extensions of a segment.
     * </p>
     * @class
     */
    export interface SegmentInfoBase extends Object{}
    export class SegmentInfoBase {
      /**
       * Creates a new instance of {@link yfiles.router.SegmentInfoBase}.
       * @param edge the edge to which the segment of this info belongs
       * @param segmentIndex the index of the segment to which this info belongs
       * @param direction the direction to which the segment of this info points
       * @param locationRange the range within which the common location of this segment has to lie
       * @param minExtension the minimum interval in extension direction this segment is known to intersect
       * @param maxExtension the maximum interval in extension direction this segment will span
       * @protected
       */
      constructor(edge:yfiles.algorithms.Edge,segmentIndex:number,direction:yfiles.layout.Direction,locationRange:yfiles.router.Interval,minExtension:yfiles.router.Interval,maxExtension:yfiles.router.Interval);
      /**
       * Creates a new instance of {@link yfiles.router.SegmentInfoBase} using a line segment to describe the edge segment.
       * <p>
       * This constructor is meant to be used for fixed orthogonal edge segments.
       * </p>
       * @param edge the edge to which this segment info belongs
       * @param segmentIndex the index of the segment to which this info belongs
       * @param segment a line segment describing the edge segment
       * @protected
       */
      constructor(edge:yfiles.algorithms.Edge,segmentIndex:number,segment:yfiles.algorithms.LineSegment);
      /**
       * Gets the minimum known extension of the segment, i.e., the minimum interval that this segment will cover in any case.
       * <p>
       * For horizontal intervals, this is the minimum horizontal extension; for vertical intervals, this is the minimum vertical
       * extension.
       * </p>
       * @type {yfiles.router.Interval}
       */
      minExtension:yfiles.router.Interval;
      /**
       * Gets the maximum extension that this segment can span.
       * <p>
       * For horizontal intervals, this is the maximum horizontal extension; for vertical intervals, this is the maximum vertical
       * extension.
       * </p>
       * @type {yfiles.router.Interval}
       */
      maxExtension:yfiles.router.Interval;
      /**
       * Gets the range, i.e., the interval within which the segment should be placed.
       * <p>
       * For horizontal segments, this is the range containing the vertical location; for vertical segments, this is the range
       * containing the horizontal location.
       * </p>
       * @type {yfiles.router.Interval}
       */
      locationRange:yfiles.router.Interval;
      /**
       * Gets or sets the preferred placement of this segment within its location range.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown alignment is given
       * @type {yfiles.router.Alignment}
       */
      preferredAlignment:yfiles.router.Alignment;
      /**
       * Gets or sets the {@link yfiles.router.SegmentInfoBase#fixed fixed} location within the {@link yfiles.router.SegmentInfoBase#locationRange location range} for this segment info.
       * <p>
       * For horizontal segments, this is the vertical location; for vertical segments, this is the horizontal location.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if the segment info already has a fixed location
       * @throws {Stubs.Exceptions.ArgumentError} if the given location does not lie within the location range
       * @see yfiles.router.SegmentInfoBase#fixed
       * @see yfiles.router.SegmentInfoBase#locationRange
       * @type {number}
       */
      location:number;
      /**
       * Gets whether or not this segment info is fixed, i.e., a {@link yfiles.router.SegmentInfoBase#location fixed location} has been determined within its {@link yfiles.router.SegmentInfoBase#locationRange location range}.
       * @type {boolean}
       */
      fixed:boolean;
      /**
       * Gets whether or not the associated segment is vertical.
       * @type {boolean}
       */
      vertical:boolean;
      /**
       * Gets the {@link yfiles.layout.Direction} to which the segment of this info points.
       * @type {yfiles.layout.Direction}
       */
      direction:yfiles.layout.Direction;
      /**
       * Gets the edge to which the segment of this info belongs.
       * @type {yfiles.algorithms.Edge}
       */
      edge:yfiles.algorithms.Edge;
      /**
       * Gets the index of the segment of this info.
       * @type {number}
       */
      segmentIndex:number;
      /**
       * Gets or sets the segment group to which this segment info belongs.
       * <p>
       * When defining the segment group, the {@link yfiles.router.SegmentInfoBase#locationRange location range} will be adjusted to use the common interval of the segment group.
       * </p>
       * @see yfiles.router.SegmentGroup#commonLocationRange
       * @type {yfiles.router.SegmentGroup}
       */
      segmentGroup:yfiles.router.SegmentGroup;
      static $class:yfiles.lang.Class;
    }
    /**
     * A descriptor for bus-related information to be associated with an edge.
     * <p>
     * It consists of a bus ID defining the bus of the associated edge, two optional edge group IDs for specifying the edge
     * grouping at the source and target side, respectively, and an optional boolean flag to mark the edge as fixed.
     * </p>
     * <p>
     * Every object except <code>null</code> can be used as a valid bus ID. Every edge of ID <code>null</code> forms a group of its own. Also, every
     * object except <code>null</code> can be used as a valid edge group ID. If there is no group ID defined for an edge or if <code>null</code> is set,
     * a default group ID is used. Therefore, all such edges ending at the same node belong to the same group.
     * </p>
     * @see yfiles.router.BusRouter#EDGE_DESCRIPTOR_DP_KEY
     * @class
     */
    export interface BusDescriptor extends Object{}
    export class BusDescriptor {
      /**
       * Creates a new instance of {@link yfiles.router.BusDescriptor} for the specified parameters.
       * @param busID the ID of the bus to which the associated edge belongs
       * @param fixed <code>true</code> if associated edge is fixed, <code>false</code> otherwise
       * @param sourceGroupID the group ID of the source side
       * @param targetGroupID the group ID of the target side
       */
      constructor(busID:Object,fixed:boolean,sourceGroupID:Object,targetGroupID:Object);
      /**
       * Creates a new instance of {@link yfiles.router.BusDescriptor} for the specified parameters and default group IDs.
       * @param busID the ID of the bus to which the associated edge belongs
       * @param fixed <code>true</code> if associated edge is fixed, <code>false</code> otherwise
       */
      constructor(busID:Object,fixed:boolean);
      /**
       * Creates a new instance of {@link yfiles.router.BusDescriptor} for the specified parameters and marks it as not fixed.
       * @param busID the ID of the bus to which the associated edge belongs
       * @param sourceGroupID the group ID of the source side
       * @param targetGroupID the group ID of the target side
       */
      constructor(busID:Object,sourceGroupID:Object,targetGroupID:Object);
      /**
       * Creates a new instance of {@link yfiles.router.BusDescriptor} for the specified bus ID which is marked as not fixed and uses the
       * default edge group IDs.
       * @param busID the ID of the bus to which the associated edge belongs
       */
      constructor(busID:Object);
      /**
       * Gets or sets the bus ID.
       * <p>
       * Edges associated with {@link yfiles.router.BusDescriptor descriptor instances} having the same bus ID belong to a common bus. Every edge associated with ID
       * <code>null</code> forms a group of its own.
       * </p>
       * @type {Object}
       */
      busId:Object;
      /**
       * Gets or sets the group ID for the source side of the edge.
       * <p>
       * Edges incident to the same node <code>v</code> but associated with different group IDs at this endpoint, use separate bus connections
       * at <code>v</code>.
       * </p>
       * @type {Object}
       */
      sourceGroupId:Object;
      /**
       * Gets or sets the group ID for the target side of the edge.
       * <p>
       * Edges incident to the same node <code>v</code> but associated with different group IDs at this endpoint, use separate bus connections
       * at <code>v</code>.
       * </p>
       * @type {Object}
       */
      targetGroupId:Object;
      /**
       * Gets or sets whether or not the associated edge is fixed.
       * <p>
       * The algorithm doesn't change the route of fixed edges but tries to prevent overlaps with non-fixed edges.
       * </p>
       * @type {boolean}
       */
      fixed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class provides helper methods to convert between buses given by complete subgraphs and buses represented by
     * specific <em>hub</em> nodes.
     * <p>
     * In a representation by means of hubs, each connected subgraph induced by hubs establishes a bus. All regular nodes
     * (non-hubs) which are connected to hubs of the same component are end-nodes of the same bus. In other words, nodes which
     * are reachable on paths consisting of only hubs belong to the same bus. Of course, a node can be an end-node of more than
     * one bus.
     * </p>
     * <p>
     * Since a bus models a group of nodes in which every node is connected to every other node, it can be represented as a
     * complete subgraph of these nodes. To distinguish buses in this representation, each edge must be associated with a
     * unique ID which identifies the bus to which it is associated. This is the representation expected by
     * {@link yfiles.router.BusRouter}.
     * </p>
     * @class
     */
    export interface BusRepresentations extends Object{}
    export class BusRepresentations {
      /**
       * Changes the representation of buses from hubs to complete subgraphs by creating new edges between regular nodes and
       * removing the hubs.
       * <p>
       * This method respects multiple connections of nodes and creates appropriate
       * {@link yfiles.router.BusDescriptor}s consisting of the bus ID as well as source and target group IDs.
       * </p>
       * <p>
       * The path of each new edge follows the route defined by the component of the hubs. If the subgraph of hubs is not a tree,
       * these paths are ambiguous. In this case, an arbitrary tree is computed from the hub's subgraph which defines all the
       * paths.
       * </p>
       * @param graph the graph of the bus
       * @param hubEdgesLists an array of {@link }s each containing the edges of a bus
       * @param hubMarker a {@link } that marks hub nodes
       * @param fixedMarker a {@link } that marks fixed edges
       * @param descriptorAcceptor a {@link } for storing the created bus descriptors
       * @returns a list of the created edges
       * @static
       */
      static replaceHubsBySubgraph(graph:yfiles.layout.LayoutGraph,hubEdgesLists:yfiles.algorithms.EdgeList[],hubMarker:yfiles.algorithms.IDataProvider,fixedMarker:yfiles.algorithms.IDataProvider,descriptorAcceptor:yfiles.algorithms.IDataAcceptor):yfiles.algorithms.EdgeList;
      /**
       * Changes the representation of buses from hubs to complete subgraphs.
       * <p>
       * This method delegates to {@link yfiles.router.BusRepresentations#replaceHubsBySubgraph}.
       * </p>
       * @param graph the graph of the bus
       * @param hubMarker a {@link } that marks hub nodes
       * @param descriptorAcceptor a {@link } for storing the created bus descriptors
       * @returns a list of the created edges
       * @see yfiles.router.BusRepresentations#replaceHubsBySubgraph
       * @see yfiles.router.BusRepresentations#toEdgeLists
       * @static
       */
      static replaceHubsBySubgraph(graph:yfiles.layout.LayoutGraph,hubMarker:yfiles.algorithms.IDataProvider,descriptorAcceptor:yfiles.algorithms.IDataAcceptor):yfiles.algorithms.EdgeList;
      /**
       * Changes the representation of buses from complete subgraphs to hubs by replacing intersection points by hubs.
       * <p>
       * The edges of the given graph must form an orthogonal, cycle-free bus, otherwise an {@link yfiles.lang.Exception} is thrown.
       * </p>
       * @param graph the graph where each bus is represented by a complete subgraph of the corresponding nodes
       * @param edgeCursor an {@link } of the regular edges
       * @param descriptorProvider a {@link } that provides a {@link } for each edge
       * @param busIDAcceptor an optional {@link } that stores the bus ID for each new edge
       * @throws {Stubs.Exceptions.InvalidOperationError} if the path of an edge is not orthogonal or if some paths form a cycle
       * @static
       */
      static replaceSubgraphByHubs(graph:yfiles.layout.LayoutGraph,edgeCursor:yfiles.algorithms.IEdgeCursor,descriptorProvider:yfiles.algorithms.IDataProvider,busIDAcceptor:yfiles.algorithms.IDataAcceptor):void;
      /**
       * Calculates for every bus represented by hubs a list of all of its edges.
       * @param graph the graph where buses are represented by means of hubs
       * @param hubMarker a {@link } which marks hub nodes
       * @returns an array of {@link }s where each list contains all edges of a bus
       * @static
       */
      static toEdgeLists(graph:yfiles.algorithms.Graph,hubMarker:yfiles.algorithms.IDataProvider):yfiles.algorithms.EdgeList[];
      /**
       * Data acceptor key to transfer a user-defined object from each original edge to the respective newly created edge
       * <p>
       * Since there is no one-to-one mapping between original and new edges, the data provider of this key is responsible for
       * the edges whose source endpoint is a regular node.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static SOURCE_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data acceptor key to transfer a user-defined object from each original edge to the respective newly created edge
       * <p>
       * Since there is no one-to-one mapping between original and new edges, the data provider of this key is responsible for
       * the edges whose target endpoint is a regular node.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static TARGET_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      static $class:yfiles.lang.Class;
    }
    /**
     * An edge routing algorithm which routes edges of a graph in an orthogonal bus-style.
     * <p>
     * Carefully observe that the resulting representation, with many edge segments drawn on top of each other, leaves little
     * room for a sensible interpretation of edge direction.
     * </p>
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * Edge segments are bundled to buses. A bus is a segment shared by multiple edges to which shorter segments that connect
     * to actual nodes are attached. Buses and all other segments are routed orthogonally.
     * </p>
     * <p>
     * A bus can, for example, be created in parts of a diagram where each node is connected to each other node. There are no
     * cycles induced by any two edge paths of the same bus, that is, the combination of all edge routes looks like an
     * orthogonal tree.
     * </p>
     * <p>
     * The algorithm tries to produce routes where the edges share as much of their paths as possible. It yields long line
     * segments (so-called <b>backbone segments</b>) where ideally all but the first and last segments of all edge paths are drawn on top of each other
     * (forming a
     * <b>bus</b>), with short connections branching off to the nodes
     * (<b>bus connections</b>). These short connections bundle the respective first or last segments of a node's incident edges.
     * </p>
     * <p>
     * This algorithm will not modify positions or sizes of nodes in any way, but will route the edges of the graph.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0},"n":[[375,165],[375,255],[585,165],[585,255],[585,345],[705,345],[705,165],[705,255],[825,165],[495,435],[585,435],[585,525]],"e":[[2,1,-15,0,0,15,[555,180,555,315,390,315]],[2,3,-15,0,-15,0,[555,180,555,270]],[3,1,-15,0,0,15,[555,270,555,315,390,315]],[2,4,-15,0,-15,0,[555,180,555,360]],[3,4,-15,0,-15,0,[555,270,555,360]],[1,4,0,15,-15,0,[390,315,555,315,555,360]],[5,4,0,-15,-15,0,[720,315,555,315,555,360]],[5,1,0,-15,0,15,[720,315,390,315]],[5,3,0,-15,-15,0,[720,315,555,315,555,270]],[5,2,0,-15,-15,0,[720,315,555,315,555,180]],[5,7,0,-15,0,15],[7,4,0,15,-15,0,[720,315,555,315,555,360]],[7,1,0,15,0,15,[720,315,390,315]],[7,3,0,15,-15,0,[720,315,555,315,555,270]],[7,2,0,15,-15,0,[720,315,555,315,555,180]],[5,9,0,-15,15,0,[720,315,555,315,555,450]],[7,9,0,15,15,0,[720,315,555,315,555,450]],[2,9,-15,0,15,0,[555,180,555,450]],[3,9,-15,0,15,0,[555,270,555,450]],[1,9,0,15,15,0,[390,315,555,315,555,450]],[4,9,-15,0,15,0,[555,360,555,450]],[5,10,0,-15,-15,0,[720,315,555,315,555,450]],[7,10,0,15,-15,0,[720,315,555,315,555,450]],[2,10,-15,0,-15,0,[555,180,555,450]],[3,10,-15,0,-15,0,[555,270,555,450]],[1,10,0,15,-15,0,[390,315,555,315,555,450]],[4,10,-15,0,-15,0,[555,360,555,450]],[9,10,15,0,-15,0],[5,11,0,-15,-15,0,[720,315,555,315,555,540]],[7,11,0,15,-15,0,[720,315,555,315,555,540]],[2,11,-15,0,-15,0,[555,180,555,540]],[3,11,-15,0,-15,0,[555,270,555,540]],[1,11,0,15,-15,0,[390,315,555,315,555,540]],[4,11,-15,0,-15,0,[555,360,555,540]],[9,11,15,0,-15,0,[555,450,555,540]],[10,11,-15,0,-15,0,[555,450,555,540]],[0,1,0,15,0,-15,1],[0,2,0,15,0,15,1,[390,225,600,225]],[1,2,0,-15,0,15,1,[390,225,600,225]],[0,3,0,15,0,-15,1,[390,225,600,225]],[1,3,0,-15,0,-15,1,[390,225,600,225]],[3,2,0,-15,0,15,1],[0,6,0,15,0,15,1,[390,225,720,225]],[1,6,0,-15,0,15,1,[390,225,720,225]],[3,6,0,-15,0,15,1,[600,225,720,225]],[6,2,0,15,0,15,1,[720,225,600,225]],[0,7,0,15,0,-15,1,[390,225,720,225]],[1,7,0,-15,0,-15,1,[390,225,720,225]],[3,7,0,-15,0,-15,1,[600,225,720,225]],[7,2,0,-15,0,15,1,[720,225,600,225]],[7,6,0,-15,0,15,1],[0,8,0,15,0,15,1,[390,225,840,225]],[1,8,0,-15,0,15,1,[390,225,840,225]],[3,8,0,-15,0,15,1,[600,225,840,225]],[7,8,0,-15,0,15,1,[720,225,840,225]],[8,2,0,15,0,15,1,[840,225,600,225]],[8,6,0,15,0,15,1,[840,225,720,225]]],"vp":[375.0,165.0,480.0,390.0]}} Bus-style edge routing with default settings {@graph {"ann":{"s":[30,30],"d":0},"n":[[529,405,120,120],[765,405],[765,495],[945,405],[945,495],[255,405],[255,495],[435,405],[345,495],[435,495],[345,405],[765,212],[765,302],[945,212],[855,302],[945,302],[855,212],[675,302],[900,302],[900,212],[810,212],[810,302],[511,212],[435,212],[359,212],[283,212],[555,105],[622,105],[555,30],[622,30],[622,-45]],"e":[[0,1,60,0,0,15,[780,465]],[0,2,60,0,0,-15,[780,465]],[0,3,60,0,0,15,[960,465]],[0,4,60,0,0,-15,[960,465]],[0,5,-60,0,0,15,[270,465]],[0,6,-60,0,0,-15,[270,465]],[0,7,-60,0,0,15,[450,465]],[0,8,-60,0,0,-15,[360,465]],[0,9,-60,0,0,-15,[450,465]],[0,10,-60,0,0,15,[360,465]],[0,11,0,-60,0,15,[589,272,780,272]],[0,12,0,-60,0,-15,[589,272,780,272]],[0,13,0,-60,0,15,[589,272,960,272]],[0,14,0,-60,0,-15,[589,272,870,272]],[0,15,0,-60,0,-15,[589,272,960,272]],[0,16,0,-60,0,15,[589,272,870,272]],[0,17,0,-60,0,-15,[589,272,690,272]],[0,18,0,-60,0,-15,[589,272,915,272]],[0,19,0,-60,0,15,[589,272,915,272]],[0,20,0,-60,0,15,[589,272,825,272]],[0,21,0,-60,0,-15,[589,272,825,272]],[0,22,0,-60,0,15,[589,272,526,272]],[0,23,0,-60,0,15,[589,272,450,272]],[0,24,0,-60,0,15,[589,272,374,272]],[0,25,0,-60,0,15,[589,272,298,272]],[0,26,15,-60,15,0,[604,120]],[0,27,15,-60,-15,0,[604,120]],[0,28,15,-60,15,0,[604,45]],[0,29,15,-60,-15,0,[604,45]],[0,30,15,-60,-15,0,[604,-30]]],"vp":[255.0,-45.0,720.0,570.0]}} Edge routing sample including four different buses<b>Concept</b>
     * </p>
     * <p>
     * The algorithm uses a two-phase process:
     * </p>
     * <ol>
     * <li><b>Backbone Selection</b>: a set of suitable initial backbone segments is determined.</li>
     * <li>
     * <b>Routing and Recombination</b>: each initial backbone segment is connected to all other backbone segments and to each node by using orthogonal edge
     * paths. Then, the resulting structure is reduced to the most optimal structure where backbone segments are long and
     * connections to the nodes are short.
     * </li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * To determine which edges belong to a common bus, a mapping that assigns a bus ID to each edge must be specified using
     * {@link yfiles.router.BusDescriptor}s. A {@link yfiles.algorithms.IDataProvider} holding {@link yfiles.router.BusDescriptor} instances is
     * expected to be registered with the graph using the
     * {@link yfiles.router.BusRouter#EDGE_DESCRIPTOR_DP_KEY descriptor key}. In the absence of an individual bus ID for an edge, a bus consisting only of the single edge is
     * created.
     * </p>
     * <p>
     * This algorithm supports {@link yfiles.layout.PortConstraint}s as well as
     * {@link yfiles.layout.PortCandidate}s to control where edges should connect to nodes.
     * </p>
     * <p>
     * Note that if edges of the same bus connect to a common node but have inconsistent or contradicting port
     * constraints/candidates, then any of these constraints/candidates can determine the actual location of the common port.
     * The same applies for edges that, in addition, belong to the same edge group.
     * </p>
     * <p>
     * Also, the cardinality defined with a {@link yfiles.layout.PortCandidateSet} object is interpreted in terms of different bus IDs
     * (group IDs) instead of number of edges.
     * </p>
     * <p>
     * This algorithm supports incremental edge routing, that is, extending or updating an already existing bus-style
     * representation. This is useful to rearrange existing edges or to include additional edges in an existing bus.
     * <br />
     * Incremental routing is supported by denoting so-called fixed edges using the
     * {@link yfiles.router.BusDescriptor#fixed corresponding BusDescriptor property}. The paths of edges which are not marked as fixed are calculated by the algorithm.
     * The structure induced by the fixed edges must be orthogonal and cycle-free.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface BusRouter extends yfiles.layout.LayoutStageBase{}
    export class BusRouter {
      /**
       * Creates a new instance of {@link yfiles.router.BusRouter} with default settings.
       */
      constructor();
      /**
       * Data provider key for specifying a bus descriptor object for each edge
       * <p>
       * The {@link yfiles.router.BusDescriptor} for an edge provides the edge's bus ID, its optional group IDs and whether or not the
       * edge is fixed.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.router.BusDescriptor>}
       */
      static EDGE_DESCRIPTOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.router.BusDescriptor>;
      /**
       * Data provider key for specifying the edge subset to be routed
       * <p>
       * This key is used if no custom key for specifying the subset is defined using {@link yfiles.router.BusRouter#affectedEdgesDpKey}.
       * </p>
       * @see yfiles.router.BusRouter#scope
       * @see yfiles.router.BusRouter#affectedEdgesDpKey
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DEFAULT_AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
       * <p>
       * Positive values greater than <code>2</code> are allowed. Positive values less than <code>2</code> are ignored, while negative values are mapped to
       * their absolute value.
       * </p>
       * @see yfiles.router.BusRouter#gridRouting
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets or sets whether or not to route edge segments on grid lines only.
       * @type {boolean}
       */
      gridRouting:boolean;
      /**
       * Gets or sets the minimum distance between edge segments and nodes.
       * <p>
       * Positive values greater than <code>2</code> are allowed. Positive values less than <code>2</code> are ignored, while negative values are mapped to
       * their absolutes.
       * </p>
       * @type {number}
       */
      minimumDistanceToNode:number;
      /**
       * Gets or sets the minimum distance between any two edge segments.
       * <p>
       * The edge routing algorithm adheres to this value if possible, but reduces the distance value selectively, i.e., only for
       * a currently processed edge, when there is not enough space to find a path with the proper value.
       * </p>
       * <p>
       * Positive values greater than <code>4</code> are allowed. Positive values less than <code>4</code> are ignored, while negative values are mapped to
       * their absolute values.
       * </p>
       * @type {number}
       */
      minimumDistanceToEdge:number;
      /**
       * Gets or sets the cost for each edge crossing.
       * <p>
       * A cost value of <code>n</code> means that it is more profitable for a path to change its direction <code>n</code> times rather than crossing the
       * path of an edge. If the cost value is set to <code>0.0</code>, no global crossing optimization is performed.
       * </p>
       * <p>
       * The cost is defined to be a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given cost value is negative
       * @type {number}
       */
      crossingCost:number;
      /**
       * Gets or sets whether or not to perform an additional step to reroute the edges such that the number of edge crossings is
       * reduced.
       * <p>
       * This features does not guarantee that the number of crossings will be the minimal.
       * </p>
       * @see yfiles.router.BusRouter#crossingCost
       * @type {boolean}
       */
      rerouting:boolean;
      /**
       * Gets or sets the maximum number of selected backbone segments with the same orientation.
       * <p>
       * This setting defines the number of backbone segments of the same orientation which are computed by the backbone
       * selection phase. The final number of backbone segments may be different due to changes in the routing and recombination
       * phase.
       * </p>
       * <p>
       * The number must be a value greater than or equal to <code>1</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given preferred number is smaller than <code>1</code>
       * @type {number}
       */
      preferredBackboneSegmentCount:number;
      /**
       * Gets or sets the preferred minimum length of a backbone segment.
       * <p>
       * This number defines the minimum length of backbone segments which are computed by the backbone selection phase. Some of
       * the final backbone segments may be shorter due to changes in the routing and recombination phase.
       * </p>
       * <p>
       * The minimum length is defined to be a value greater than or equal to <code>1.0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum length is smaller than <code>1.0</code>
       * @type {number}
       */
      minimumBackboneSegmentLength:number;
      /**
       * Gets or sets the minimum number of bus connections a backbone segment must have.
       * <p>
       * If a backbone segment has less connections, it is removed and the affected nodes connect to another backbone segment.
       * </p>
       * <p>
       * The minimum connection count must be a value greater than or equal to <code>1</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum count is smaller than <code>1</code>
       * @type {number}
       */
      minimumBusConnectionsCount:number;
      /**
       * Gets or sets whether or not collinear bends are removed from the layout.
       * <p>
       * A collinear bend is a bend that lies on a common line with its predecessor bend and successor bend.
       * </p>
       * <p>
       * If an edge has a collinear bend, there is another edge which has a real bend at this point, i.e., the bend location is
       * an intersection of the bus. Therefore, it may be advantageous for some applications to keep such bends.
       * </p>
       * @type {boolean}
       */
      removeCollinearBends:boolean;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} for marking edges as <em>selected</em>.
       * <p>
       * If the {@link yfiles.router.BusRouter#scope scope} is set to {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES}, only the edges for which the registered {@link yfiles.algorithms.IDataProvider}
       * returns <code>true</code> will be routed. All other edges will be considered to have fixed routes.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      /**
       * Gets or sets the scope for this routing algorithm that determines which edges are routed.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown scope is given
       * @type {yfiles.router.Scope}
       */
      scope:yfiles.router.Scope;
      static $class:yfiles.lang.Class;
    }
    /**
     * This edge routing algorithm generates orthogonal routes for the edges of the graph.
     * <p>
     * Compared to the routing algorithm implemented by class
     * {@link yfiles.router.EdgeRouter}, this implementation is usually faster but supports less constraints and, by default, may
     * produce node-edge overlaps.
     * </p>
     * <p>
     * <b>Layout Style</b> Edges are routed in an orthogonal fashion, i.e., edge paths consist only of vertical and horizontal segments.
     * </p>
     * <p>
     * During the routing process, the positions of the nodes are considered to be fixed and the routing algorithm will not
     * modify their locations or their sizes in any way.
     * </p>
     * <p>
     * The edge routing algorithm can be applied wherever it is required to route the edges as orthogonal segments, while
     * keeping the positions of the nodes in the diagram fixed. Some potential applications include electric circuit design,
     * floor planning, UML class diagrams/inheritance diagrams and navigation maps.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[45,30],"d":0},"n":[[210,270,60,30],[405,165,60,30],[120,270,60,30],[-7.5,270],[75,375,60,30],[502.5,165],[615,270,60,30],[630,330,30,30],[292.5,225,75,30],[232.5,60],[112.5,165,75,30],[630,60,30,30],[225,0,60,30],[712.5,330],[-7.5,375],[712.5,0],[120,330,60,30],[307.5,330],[712.5,270],[585,165,30,30],[412.5,225],[502.5,225],[232.5,120],[465,105,75,30]],"e":[[7,6,0,-15,0,15],[8,17,0,15,0,-15],[9,3,-22.5,0,-5,-15,[10,75]],[1,9,0,-15,22.5,0,[435,75]],[11,6,0,15,0,-15],[3,2,22.5,0,-30,0],[18,6,-22.5,0,30,0],[3,14,0,15,0,-15],[12,10,-30,0,-5,-15,[145,15]],[10,1,37.5,-5,-30,-5],[10,2,0,15,0,-15],[6,0,-30,0,30,0],[5,19,22.5,0,-15,0],[1,8,-8.75,15,5,-15,[426.25,205,335,205]],[9,11,5,-15,0,-15,[260,50,645,50]],[10,3,-37.5,0,5,-15,[20,180]],[8,0,-37.5,0,0,-15,[240,240]],[18,15,0,-15,0,15],[13,7,-22.5,0,15,0],[2,0,30,0,-30,0],[10,8,37.5,5,-5,-15,[325,185]],[16,17,30,0,-22.5,0],[13,18,0,-15,0,15],[1,5,30,0,-22.5,0],[12,15,30,0,-22.5,0],[12,9,-5,15,-5,-15],[14,4,22.5,0,-30,0],[16,2,0,-15,0,15],[8,20,37.5,0,-22.5,0],[20,1,1.25,-15,1.25,15],[21,5,0,-15,0,15],[22,9,0,-15,0,15],[22,10,-22.5,0,5,-15,[155,135]],[21,20,-22.5,0,22.5,0],[7,17,-15,0,22.5,0],[13,4,0,15,30,0,[735,390]],[23,11,0,-15,-15,0,[502.5,75]],[23,19,37.5,0,0,-15,[600,120]]],"vp":[-8.0,0.0,766.0,405.0]}} Sample output of the orthogonal edge routing algorithm with default settings<b>Concept</b> This edge routing algorithm combines two
     * strategic steps of edge routing and executes them one after the other. The first strategy is called path finder strategy
     * and will route the edges, potentially with edge overlaps. The second strategy will then split overlapping edge segments
     * inside their channels and distribute them according to the specific distribution strategy.
     * </p>
     * <p>
     * The default path finding strategy is {@link yfiles.router.OrthogonalPatternEdgeRouter}. The default edge distribution strategy
     * is {@link yfiles.router.OrthogonalSegmentDistributionStage}.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * This class itself has no special routing options except from specifying the {@link yfiles.router.ChannelEdgeRouter#pathFinderStrategy path finding} and {@link yfiles.router.ChannelEdgeRouter#edgeDistributionStrategy edge distribution} strategy.
     * Most of the features like minimum element distances, grid spacing and path restrictions have to be set on the used
     * instances for the two routing steps.
     * </p>
     * <p>
     * For incremental edge routing, {@link yfiles.algorithms.IDataProvider} key {@link yfiles.router.ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY}
     * has to be registered with the graph to mark all edges that should be routed by the algorithm. The routes of the other
     * edges remain unchanged. If there is no such key registered, the algorithm routes all edges.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface ChannelEdgeRouter extends yfiles.layout.LayoutStageBase{}
    export class ChannelEdgeRouter {
      /**
       * Creates a new {@link yfiles.router.ChannelEdgeRouter} instance with default settings.
       */
      constructor();
      /**
       * Checks the sizes of the nodes to be non-zero.
       * @param g The graph to check.
       * @protected
       */
      checkNodeSize(g:yfiles.layout.LayoutGraph):void;
      /**
       * Data provider key for marking the edges that should be routed
       * <p>
       * If this key is not registered with the graph, the algorithm will route all edges.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the strategy for the path finding step.
       * <p>
       * This step routes the edges and may produce edge overlaps which are later resolved in the {@link yfiles.router.ChannelEdgeRouter#edgeDistributionStrategy edge distribution} step.
       * </p>
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      pathFinderStrategy:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the strategy for the edge distribution step.
       * <p>
       * This step distributes overlapping edge segments in their channel. The channel in which the segments can be distributed
       * is defined by the surrounding graph elements.
       * </p>
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      edgeDistributionStrategy:yfiles.layout.ILayoutAlgorithm;
      static $class:yfiles.lang.Class;
    }
    /**
     * This edge routing algorithm applies organic routes to the edges of the graph.
     * <p>
     * <b>Layout Style</b> Edges are routed organically, i.e. in smooth curves around the nodes observing a {@link yfiles.router.OrganicEdgeRouter#minimumDistance minimum distance} to the nodes.
     * </p>
     * <p>
     * During the routing process, the positions of the nodes are considered to be fixed and the router will not modify their
     * locations or their sizes in any way.
     * </p>
     * <p>
     * The edge routing algorithm can be applied whenever edge paths should avoid crossing any nodes in organic or cyclic
     * layout styles.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0,"c":1},"n":[[339,332],[314,663],[246,391],[112,365],[280,143],[309,245],[521,184],[519,462],[638,200],[118,200],[289,600],[419,153],[194,257],[564,349],[39,419],[266,76],[637,149],[247,23],[663,356],[420,517],[583,496],[183,525],[406,271],[498,406],[519,95],[250,263],[634,471],[340,459],[93,504],[368,110],[488,592],[177,154],[266,548],[470,161],[180,365],[606,400],[0,309],[130,256],[154,631],[60,311],[397,568],[16,241],[421,51],[22,368],[463,102],[67,224],[139,45],[196,95],[238,606],[347,510],[471,535],[365,631],[199,314],[125,96],[99,445],[421,463],[457,281],[497,35],[351,161],[572,190],[288,340],[400,220],[119,147],[570,98],[340,569],[526,513],[416,636],[196,35],[447,412],[242,496],[559,292],[150,416],[348,403],[38,477],[539,581],[228,146],[594,554],[610,280],[297,194],[329,0],[406,0],[181,206],[163,467],[317,66],[661,305],[55,150],[508,307],[180,576],[689,231],[96,555],[240,657],[543,241],[111,308],[555,405],[467,643],[449,352],[232,445],[666,416]],"e":[[0,1,[335.66,406.18,333.37,416.82,331.63,427.45,326.91,469.44,325.45,574.7,326.08,585.88,329.61,608.76,330.67,620.69]],[0,2,[322.18,373.25,310.13,381.03]],[0,3,[324.5,334.9,309.02,329.8,293.4,330.34,236.16,351.02,222.05,355.29,206.97,355.02,192.08,353.11,178.03,357.07]],[0,4,[351.49,268.72,349.88,254.78,343.6,242.42,331.57,234.82,303.36,234.17,292.35,226.33,286.5,215.59,286.02,203.04]],[0,5],[0,6,[428.62,311.49,440.22,302.84,454.41,274.85]],[0,7,[446.52,392.65,474.45,404,484.01,413.8,488.6,427.79,494.98,440.48]],[0,8,[426.01,313.31,439.52,303.87,453.15,277.95,464.34,271.34,476.96,270.23,500.5,273.67,544.26,281.36,555.61,282.65,567.08,280.79,577.87,275.33]],[0,9,[308.77,326.66,262.87,308.22,248.01,300.72,238.93,288.48,235.94,272.58,231.01,257.56,219.51,247.31,188.2,245.93,173.56,238.97]],[0,10,[338.79,396.59,334.49,413.65,328.25,447.31,320.83,497.58,312.44,564.66]],[0,11,[389.21,229.75,395.29,218.04]],[0,12,[317.61,329.41,260.47,307.45,250.73,302.73,241.57,296.79]],[0,13,[455.92,337.29,471.02,338.02,485.35,341.25,511.94,349.47]],[0,14,[322.04,373.66,309.3,379.72,294.9,380.89,266.5,377.98,252.08,379.76,238.85,386.12,213.64,401.5,199.41,406.7,183.85,406.68,167.87,404.99,151.09,407.31,134.98,411.26]],[0,15,[353.71,330.4,350.89,263.48,348.57,246.47,341.24,213.46,336.58,196.79,325.52,164.96,319.31,148.97,310.63,133.49]],[0,16,[414.67,318.85,438.09,305.77,445.29,294.3,450.39,282.07,458.9,273.15,469,266.59,548.02,222.11,556.91,213.74,565.88,190.33,574.87,182.39,586.14,178.03]],[0,17,[351.18,260.4,349.73,248.97,342.55,215.84,336.08,193.14,323.88,159.03,318.91,147.37,310.73,137.35,299.46,130.24,274.92,118.04,263.72,110.5,256.06,98.75,255.2,83.29]],[0,18,[449.23,337.24,460.44,336.84,472.05,337.94,483.25,340.2,515.57,347.93,527.01,348.23,538.21,346.05,570.17,336.94,581.97,336.14,593.53,338.61]],[0,19,[386.75,401.74,391.67,414.2,406.76,475.79,410.7,488.12,416.28,499.76,422.43,510.76,429.28,521.1]],[0,20,[441.16,390.65,454.09,396.14,467.28,401.18,479.65,408.23,486.46,419.97,490.73,433.62,499.82,443.19,513.09,447.19,527.26,449.42,540.69,452.14,552.84,459.14,562.64,469.33]],[0,21,[346.85,359.59,309.35,420.4,300.65,431.95,292.74,444.04,275.05,467.54,264.99,478.94,237.08,493.01,226.61,504.8]],[0,22],[0,23,[441.87,390.83,452.09,395.12,474.02,402.99]],[0,24,[394.9,278.08,405.38,265.78,420.33,260.03,433.54,253.67,440.93,239.63,453.72,194.88,457.41,179.4,462.73,163.54,473.96,151.35,489.87,142.62,505.68,131.98]],[0,25],[0,26,[443.76,391.94,454.52,396.46,465.49,400.4,475.94,405.45,483.96,413.69,487.9,425.26,492.62,436.58,501.97,444.03,513.85,447.54,538.88,451.1,551.26,454.29]],[0,27,[335.47,409.13,335.91,425.89,341.76,442.39]],[0,28,[304.63,401.81,279.42,427.37,264.93,433.6,249.25,434.04,233.94,436,177.31,454.17,162.81,461.08,150.78,471.69]],[0,29,[391.99,183.49,392.17,168.35]],[0,30,[379.7,389.49,387.21,404.3,392.85,419.75,407.6,477.96,412.89,491.58,422.65,501.66,437.13,505.23,451.35,510.41,461.14,523.17,460.71,539.45,459.43,556.24,467.8,571.02]],[0,31,[345.1,335.21,335.76,323.75,326.96,311.7,289.98,263.58,279.75,251.93]],[0,32,[343.55,373.14,315.75,453.99]],[0,33,[395.41,277.39,404.28,266.36,430.24,255.67,439.95,245.33,447.72,233.23,455.38,222.46,462.6,210.62,470.24,199.31]],[0,34,[320.5,374.57,310.3,379.58,299.25,381.01,267,377.27,256.18,376.82]],[0,35,[447.26,393.05,464.75,396.92,517.71,393.94,552.46,393.96,570.56,394.48,588.29,399.05]],[0,36,[324.51,334.8,313.86,330.99,302.52,329.37,291.4,331,229.75,353.05,218.61,355.85,206.54,355.04,194.57,352.53,183.37,351.65,140.67,350.4,74.42,351.92,63.14,348.81,43.25,339.26]],[0,37,[315.65,330.79,302.24,326.14,221.99,302.44,208.11,298.95,194.21,294.54]],[0,38,[291.97,443.38,271.04,472.33,261.99,480.94,249.87,486.16,238.48,492.54,232.04,503.67,229.86,516.59,221.46,592.14,216.53,604.24,208.21,614.3]],[0,39,[322.7,334.1,305.86,329.69,288.77,331.39,222.92,355.38,204.68,355.06,185.55,351.32,166.46,350.31,147.42,349.96,127.91,348.98,109.08,344.26]],[0,40,[385.83,399.93,391.14,415,405.18,536.96]],[0,41,[315.73,330.32,285.46,320.65,224.16,303.07,212.95,301.44,150.12,299.63,138.87,297.96,128.22,294.68]],[0,42,[392.06,206.63]],[0,43,[327.11,335.97,312.77,330.73,298.03,329.65,284.06,333.44,230.99,352.58,217.24,355.83,202.62,354.37,187.97,351.92,130.42,351.25,115.65,353.54]],[0,44,[389.02,229.97,397.45,216.12,410.94,207.76,438.95,194.72,452.19,186.4,459.38,173.33,463.33,158.21]],[0,45,[319.72,332.86,301.9,326.53,232.62,305.39,198.66,296.17,186.46,286.52,170.91,259.6,160.55,248.12,145.88,243.01]],[0,46,[286.25,258.53,248.79,216.04,242.43,206.94,235,197.31,228.43,187.39,221.82,176.16,217.01,162.99,210.26,150.25,198.66,139.63,188.34,129.15,180.36,115.63,173.55,101.92]],[0,47,[280.19,249.76,224.45,181.39,217.36,164.22]],[0,48,[285.9,512.16,276.58,527.31,263.6,539.9,254.29,553.79,252.08,570.91]],[0,49,[337.85,400.86,335.22,412.8,335.86,425.11,341.55,436.2,351.73,443.92,363.37,449.51,373.84,456.56,379.94,467.39,380.17,479.84,376.25,491.95,371.73,503.11]],[0,50,[363.53,357.76,373.66,368.31,383.38,379.43,393.44,390.12,433.18,435.74,454.89,458.54,463.23,472.48]],[0,51,[338.15,399.57,334.71,417.74,339.34,434.34,353.71,444.87,370.07,453.14,380.21,466.15,383.35,483.6,387.37,519.13,387.54,537.38,384.43,573.7]],[0,52,[319.73,332.96,301.78,329.04,283.75,328.69]],[0,53,[301.52,278.38,291.52,264.42,231.87,190.09,222.09,176.91,216.51,161.08,208.24,148.12,194.61,139.94]],[0,54,[328.75,368.24,314.97,377.91,298.88,381.42,264.15,377.59,247.67,381.71,234.12,393.32,188.56,445.62,174.64,454.91,158.8,458.39]],[0,55,[385.81,395.38]],[0,56,[442.78,309.64]],[0,57,[378.17,262.12,383.23,246.33,389.07,230.06,397.76,216.16,410.81,208.08,437.76,195.53,450.07,188.37,458.21,177.78,459.5,164.06,457.17,151.5,451.13,127.97,450.06,115.65,453.12,104.07,460.33,94.22]],[0,58],[0,59,[421.29,315.21,437.79,306.24,446.01,291.77,453.41,277.31,467.69,267.94]],[0,60],[0,61,[391.19,276.52]],[0,62,[343.32,333.1,332.31,319.71,321.89,305.25,289,262.94,276.51,250.23,222.11,206.93,207.08,197.5,173.74,189.82]],[0,63,[397.1,274,408.19,264.17,424.19,258.19,439.83,249.88,494.17,203.53]],[0,64,[336.87,404.08,333.8,419.28,329.35,464.32,329.41,479.8,333.11,509.98,335.6,525.23,339.44,540.66]],[0,65,[440.17,441.81,452.51,453.34]],[0,66,[384.08,396.29,390.86,414.32,393.85,433.39,403.11,506.26,405.93,524.42,409.95,542.54,421.55,555.65,435.45,567.44,440.34,583.83,438.64,600.97]],[0,67,[306.34,281.88,300.22,271.55,295.69,260.61,279.48,207.91,267.29,152.86,263.63,142.28,239.06,94.7]],[0,68],[0,69],[0,70,[469.45,323.87,482.71,319.47,494.32,311.46,505.19,301.93,517.88,296.03,532.51,296.41]],[0,71,[323.13,372.77,310.98,379.24,297.29,381.04,270.49,378.25,256.85,378.74,244.01,383.32,199,411.8]],[0,72],[0,73,[325.78,370.71,315.11,377.72,302.8,380.91,290.09,380.37,265.1,378.01,251.79,380.05,241.04,386.57,231.83,395.9,189.43,444.63,178.23,453.01,164.62,459.94,151.89,468.02,140.08,477.64,127.82,485.49,113.2,489.69,97.78,490.87]],[0,74,[453.85,457.25,464.04,469.46]],[0,75,[306.38,281.75,298.95,269.69]],[0,76,[440.85,442.45,452.16,452.04,509.45,492.6,521.63,499.85,535.67,502.44,550.32,503.97,562.01,512.5,580.43,536.15]],[0,77,[474.36,323,489.4,315.19,502.42,304.25,516.41,296.03,531.84,291.13,562.99,282.6,579.4,281.42,595.17,285.3]],[0,78,[353.21,311.51,351.75,274.78,349.92,255.71,341.42,238.46]],[0,79,[352.32,266.07,350.2,244.99,339.48,182.86,339.24,171.66,340.97,160.66,357.4,88.15,357.65,76.64,355.99,65.74]],[0,80,[375.1,267.1,384.99,225.91,396.91,184.78,409.57,132.2,410.69,121.28,408.94,67.51,410.21,56.22,412.62,45.46]],[0,81,[291.41,265.52,281.99,255.89,270.2,249.04]],[0,82,[289.83,417.4,279.44,426.99,267.12,432.95,253.03,433.84,239.16,435.43,227.3,441.61]],[0,83,[351.51,265.09,349.93,248.44,339.25,182.25,337.18,165.16]],[0,84,[455.7,337.03,469.05,337.55,481.96,339.78,505.92,345.81,518.43,348.12,531.41,347.66,567.71,337.38,580.54,334.73]],[0,85,[316.07,329.16,263.32,308.39,250.39,302.01,240.77,292.16,236.59,279.05,234.5,265.5,228.38,254.17,217.84,247.14,204.95,246.22,191.96,246.62,180.26,242.04,172.3,232.48,166.36,221.23,160.82,209.17,153.71,198.91,143.37,190.93,113.51,182.34]],[0,86],[0,87,[306.22,421.73,276.69,464.63,265.95,478.34,238.46,492.28,231.78,504.76,228.46,518.2,223.15,544.77,217.77,557.47]],[0,88,[476.59,322,487.46,316.77,506.01,301.68,516.62,295.9,628.38,263.9]],[0,89,[304.58,402.64,286.25,421.15,275.66,429.7,262.67,433.71,248.08,433.9,234.11,437.41,223.91,447.62,217.54,460.12,206.11,485.16,198.82,497.66,189.34,507.33]],[0,90,[348.03,366.59,344.43,376.17,335.23,405.79,332.53,416.36,309.42,567.16,304.71,578.23,296.24,587.44,286.64,595.91,279.81,606.85,278.36,620.58,275.34,634.69]],[0,91,[428.72,311.52,440.09,302.99,453.32,277.82,465.13,270.94,478.69,267.74]],[0,92,[327.03,335.9,312.92,330.7,298.41,329.58,284.16,333.41,230.95,352.66,216.96,355.85,202.5,353.87]],[0,93,[444.66,391.69,459.07,395.94,474.15,396.91,503.46,395.07,518.07,396.29,531.89,400.88]],[0,94,[386.41,400.52,390.51,410.75,392.91,421.24,405.97,527.76,408.98,540.16,416.51,551.43,428.68,560.5,438.65,572.18,453.33,600.22]],[0,95],[0,96,[288.85,419.25,275.08,433.27]],[0,97,[435.13,387.04,451.62,394.24,468.82,397.13,501.49,394.44,533.26,393.31,563.5,393.53,578.12,395.81,590.16,403.49,595.24,416.31,599.59,429.12,608.91,437.7,621.12,440.69,633.66,439.71,669.2,433.82]]],"vp":[0.0,0.0,719.0,693.0]}}<b>Concept</b> The edge routing algorithm uses a force-directed approach to calculate the edge paths. Nodes are considered to be
     * repulsive forces while edges will try to become as short as possible.
     * </p>
     * <p>
     * Each edge is routed separately and is influenced by the nodes in a certain area around it. The algorithm will add bends
     * to the edge path that are placed by balancing the forces.
     * </p>
     * <p>
     * The quality of the result highly depends on how much space there is between the nodes. More precisely, the distance
     * between each pair of nodes should be at least twice the specified {@link yfiles.router.OrganicEdgeRouter#minimumDistance minimum distance}. If it is not necessary that the
     * nodes keep their locations, this can be ensured using a combination of {@link yfiles.organic.RemoveOverlapsStage} and
     * {@link yfiles.router.OrganicEdgeRouter#createNodeEnlargementStage node enlargement stage}.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * {@link yfiles.router.OrganicEdgeRouter#minimumDistance} will make edges keep a custom distance to the nodes. However, if there is not enough space between the nodes, this
     * distance may be undershot (i.e. edges will be closer to nodes).
     * </p>
     * <p>
     * {@link yfiles.router.OrganicEdgeRouter} is able to
     * {@link yfiles.router.OrganicEdgeRouter#keepExistingBends reuse existing bends}. Edges will contain those bends along with other bends added by the layout algorithm.
     * </p>
     * <p>
     * This edge routing algorithm is realized as a {@link yfiles.layout.ILayoutStage} which can be applied to a graph directly or
     * using a {@link yfiles.router.OrganicEdgeRouter#coreLayout core layout algorithm}.
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface OrganicEdgeRouter extends Object,yfiles.layout.ILayoutStage{}
    export class OrganicEdgeRouter {
      /**
       * Creates a new {@link yfiles.router.OrganicEdgeRouter} with the given {@link yfiles.router.OrganicEdgeRouter#coreLayout core layout algorithm}.
       * @param core the core layout algorithm
       */
      constructor(core:yfiles.layout.ILayoutAlgorithm);
      /**
       * Creates a new {@link yfiles.router.OrganicEdgeRouter} instance with the default settings.
       */
      constructor();
      /**
       * Performs the organic routing of the edges of the input graph.
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Checks the sizes of the nodes to be non-zero.
       * @param g The graph to check.
       * @protected
       */
      checkNodeSize(g:yfiles.layout.LayoutGraph):void;
      /**
       * Returns a {@link yfiles.layout.ILayoutStage} which temporarily increases the sizes of the nodes to avoid overlaps.
       * <p>
       * The edges will keep a greater distance to the nodes. Therefore, they won't cross them.
       * </p>
       * @returns the {@link } that resizes the nodes
       */
      createNodeEnlargementStage():yfiles.layout.ILayoutStage;
      /**
       * Data provider key for selecting edges that should be routed
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets whether or not edges are allowed to cross nodes.
       * <p>
       * Allowing edges to overlap with nodes will produce smoother edges, because the edge paths can be closer to the nodes.
       * </p>
       * @see yfiles.router.OrganicEdgeRouter#createNodeEnlargementStage
       * @see yfiles.router.OrganicEdgeRouter#minimumDistance
       * @type {boolean}
       */
      edgeNodeOverlapAllowed:boolean;
      /**
       * Gets or sets the core layout algorithm which arranges the graph before edge routing.
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the minimum distance the algorithm should guarantee between nodes and non-incident edges.
       * <p>
       * The distance also influences how many bends are added to the path (a higher distance leads to less bends).
       * </p>
       * <p>
       * The minimum distance is defined to be a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum distance is negative
       * @type {number}
       */
      minimumDistance:number;
      /**
       * Gets or sets whether or not the initial bend coordinates are kept when determining the edge path.
       * <p>
       * The bends are considered as fixed nodes and stay part of the path.
       * </p>
       * @type {boolean}
       */
      keepExistingBends:boolean;
      /**
       * Gets or sets whether a rerouting step is performed on all edges or just on a subset where distances are violated.
       * <p>
       * If only a subset of edges is rerouted, only those edges which cross nodes or come too close to a node are included.
       * During rerouting, more bends are added to the edges that will be influenced by the repulsive forces.
       * </p>
       * @type {boolean}
       */
      routeAllEdges:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.router.OrthogonalPatternEdgeRouter} routes edges orthogonally such that the resulting layout of the edges consists
     * only of vertical and horizontal segments.
     * <p>
     * Note that the location and size of the nodes in a diagram remains unchanged.
     * </p>
     * <p>
     * The edge router will not try to find a perfect route from source to edge (unlike to what {@link yfiles.router.EdgeRouter} does)
     * but chooses the best path out of a set of fixed paths. The best path out of these possible paths is determined by its
     * cost. The costs may be influenced by setting several cost factors. The distance that an edge will have from its source
     * and target node is determined by {@link yfiles.router.OrthogonalPatternEdgeRouter#minimumDistance}.
     * </p>
     * <p>
     * The edges whose paths have to be routed can be defined registering a {@link yfiles.algorithms.IDataProvider} with key
     * {@link yfiles.router.OrthogonalPatternEdgeRouter#DEFAULT_AFFECTED_EDGES_DP_KEY}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface OrthogonalPatternEdgeRouter extends yfiles.layout.LayoutStageBase{}
    export class OrthogonalPatternEdgeRouter {
      /**
       * Creates a new instance of {@link yfiles.router.OrthogonalPatternEdgeRouter} with default settings.
       */
      constructor();
      /**
       * Calculates the costs for all bends of the given path.
       * <p>
       * This method is called by {@link yfiles.router.OrthogonalPatternEdgeRouter#calculateCost}. The default implementation multiplies
       * the number of bends with the according {@link yfiles.router.OrthogonalPatternEdgeRouter#bendCost costs}. It may be overridden to change the calculation of these costs.
       * </p>
       * @param edge the edge for which the costs are calculated
       * @param path the path of the given edge
       * @param spc the source {@link } for this edge
       * @param tpc the target {@link } for this edge
       * @returns the costs for the bends of this path
       * @protected
       */
      calculateBendCost(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList,spc:yfiles.layout.PortCandidate,tpc:yfiles.layout.PortCandidate):number;
      /**
       * Calculates the edge cost of a possible edge path, in order to determine which path is the best, i.e., the cheapest.
       * <p>
       * This method is called by {@link yfiles.router.OrthogonalPatternEdgeRouter#applyLayout} in order to decide which path is the
       * best. The default implementation considers the edge length, the number of bends, edge crossings,
       * {@link yfiles.layout.PortCandidate}s and monotonic path restrictions. It may be overridden to apply a different set of costs or
       * a different weighting.
       * </p>
       * @param edge the edge whose costs to calculate
       * @param path the edge's path
       * @param spc the source {@link } for this edge
       * @param tpc the target {@link } for this edge
       * @returns the sum of all costs for this edge's path
       * @protected
       */
      calculateCost(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList,spc:yfiles.layout.PortCandidate,tpc:yfiles.layout.PortCandidate):number;
      /**
       * Calculates the overall crossing costs of the given path, including edge crossings, edge overlaps and node crossings.
       * <p>
       * This method is called by
       * {@link yfiles.router.OrthogonalPatternEdgeRouter#calculateCost}. The default implementation will determine overlaps and
       * crossings between edge segments and nodes and sum up the costs. It may be overridden to use a different combination of
       * costs.
       * </p>
       * @param edge the edge for which the costs are calculated
       * @param path the path of the given edge
       * @param spc the source {@link } for this edge
       * @param tpc the target {@link } for this edge
       * @returns the overall crossing costs of the given path, including edge crossings, edge overlaps and node crossings
       * @protected
       */
      calculateCrossingCosts(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList,spc:yfiles.layout.PortCandidate,tpc:yfiles.layout.PortCandidate):number;
      /**
       * Calculates the costs for the length of the given path.
       * <p>
       * This method is called by {@link yfiles.router.OrthogonalPatternEdgeRouter#calculateCost}. The default implementation returns
       * costs between <code>0</code> for short paths and <code>1</code> for long paths. Hence, it has relatively little impact on the overall costs. The
       * method may be overridden to introduce a different weighting of the edge length.
       * </p>
       * @param edge the edge for which the costs are calculated
       * @param path the path of the given edge
       * @param spc the source {@link } for this edge
       * @param tpc the target {@link } for this edge
       * @returns the costs for the length of the given path
       * @protected
       */
      calculateEdgeLength(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList,spc:yfiles.layout.PortCandidate,tpc:yfiles.layout.PortCandidate):number;
      /**
       * Calculates the costs for the chosen {@link yfiles.layout.PortCandidate}s.
       * <p>
       * This method is called by {@link yfiles.router.OrthogonalPatternEdgeRouter#calculateCost}. The default implementation will return
       * the {@link yfiles.layout.PortCandidate#cost candidates' costs}. It may be overridden to change the calculation of these costs.
       * </p>
       * @param edge the edge for which the costs are calculated
       * @param path the path of the given edge
       * @param spc the source {@link } for this edge
       * @param tpc the target {@link } for this edge
       * @returns the costs for the {@link }s
       * @protected
       */
      calculatePortCandidateCost(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList,spc:yfiles.layout.PortCandidate,tpc:yfiles.layout.PortCandidate):number;
      /**
       * Calculates the extra penalty that is added to the path's cost if the edge is a self-loop and source and target ports
       * have the same direction.
       * <p>
       * This method is called by
       * {@link yfiles.router.OrthogonalPatternEdgeRouter#calculateCost}. The default implementation adds costs for another bend if
       * source and target ports share the same node side. In this manner, paths with different source and target directions are
       * cheaper and thus preferred. The method may be overridden to change the calculation of this penalty.
       * </p>
       * @param edge the edge for which the costs are calculated
       * @param path the path of the given edge
       * @param spc the source {@link } for this edge
       * @param tpc the target {@link } for this edge
       * @returns an extra penalty for specific self-loop paths
       * @protected
       */
      calculateSelfLoopSelfSidePenaltyCost(edge:yfiles.algorithms.Edge,path:yfiles.algorithms.YList,spc:yfiles.layout.PortCandidate,tpc:yfiles.layout.PortCandidate):number;
      /**
       * Checks the sizes of the nodes to be non-zero.
       * @param g The graph to check.
       * @protected
       */
      checkNodeSize(g:yfiles.layout.LayoutGraph):void;
      /**
       * Data provider key for determining which edges are routed
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DEFAULT_AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the monotonic path restriction that should be applied.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified path restriction is unknown
       * @type {yfiles.router.MonotonicPathRestriction}
       */
      monotonicPathRestriction:yfiles.router.MonotonicPathRestriction;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} which determines the edges that shall be routed by
       * this algorithm.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.algorithms.IDataProvider} key is <code>null</code>
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      /**
       * Gets or sets the minimum distance that an edge will maintain from its source and target node.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      minimumDistance:number;
      /**
       * Gets or sets the spacing of the grid on which edges are routed.
       * <p>
       * The spacing between two grid lines must be at least <code>2</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified grid spacing is less than <code>2</code>
       * @see yfiles.router.OrthogonalPatternEdgeRouter#gridRouting
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets or sets the origin of the grid.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified point is <code>null</code>
       * @see yfiles.router.OrthogonalPatternEdgeRouter#gridRouting
       * @type {yfiles.algorithms.YPoint}
       */
      gridOrigin:yfiles.algorithms.YPoint;
      /**
       * Gets or sets whether or not to route the edges on a grid.
       * <p>
       * The grid can be defined using methods {@link yfiles.router.OrthogonalPatternEdgeRouter#gridOrigin} and {@link yfiles.router.OrthogonalPatternEdgeRouter#gridSpacing}.
       * </p>
       * @see yfiles.router.OrthogonalPatternEdgeRouter#gridOrigin
       * @see yfiles.router.OrthogonalPatternEdgeRouter#gridSpacing
       * @type {boolean}
       */
      gridRouting:boolean;
      /**
       * Gets or sets the costs for a crossing between two edges.
       * <p>
       * These costs are used for finding the best path out of the predefined paths from which the router can choose.
       * </p>
       * <p>
       * The costs need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified costs are negative
       * @type {number}
       */
      edgeCrossingCost:number;
      /**
       * Gets or sets the costs for edges that cross nodes.
       * <p>
       * These costs are used for finding the best path out of the predefined paths from which the router can choose.
       * </p>
       * <p>
       * The costs need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified costs are negative
       * @type {number}
       */
      nodeCrossingCost:number;
      /**
       * Gets or sets the costs for creating a bend on the edge's path.
       * <p>
       * These costs are used for finding the best path out of the predefined paths from which the router can choose.
       * </p>
       * <p>
       * The costs need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified costs are negative
       * @type {number}
       */
      bendCost:number;
      /**
       * Gets or sets the costs for overlapping edge paths.
       * <p>
       * These costs are used for finding the best path out of the predefined paths from which the router can choose.
       * </p>
       * <p>
       * The costs need to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified costs are negative
       * @type {number}
       */
      edgeOverlapCost:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.router.OrthogonalSegmentDistributionStage} distributes overlapping edge segments of orthogonally routed edges.
     * <p>
     * The edge segments are assigned to the so-called <i>channels</i> within which they are distributed. Those channels are defined by the
     * surrounding graph elements.
     * </p>
     * <p>
     * The algorithm will not distribute the segments, whose endpoints connect to a node using a port that has a strong {@link yfiles.layout.PortConstraint}
     * or a fixed {@link yfiles.layout.PortCandidate}.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * It is possible to define a {@link yfiles.router.OrthogonalSegmentDistributionStage#preferredDistance preferred distance} between distributed edge segments. This distance will only be applied if
     * there is enough space in the according channel. In the case where there is not enough space, the preferred distance may
     * be {@link yfiles.router.OrthogonalSegmentDistributionStage#flexiblePreferredDistance reduced} for the edges. If the layout algorithm is not allowed to reduce the preferred distance, these edges won't be
     * distributed.
     * </p>
     * <p>
     * Edges can be distributed on {@link yfiles.router.OrthogonalSegmentDistributionStage#gridRouting grid coordinates}. The grid can only be considered if there is enough space in the according
     * channel for at least as many grid lines as the number of segments. In the case where there is not enough space, the {@link yfiles.router.OrthogonalSegmentDistributionStage#gridSpacing grid spacing}
     * may be {@link yfiles.router.OrthogonalSegmentDistributionStage#flexibleGridSpacing reduced} in this channel. If the layout algorithm is not allowed to reduce the grid spacing, these edges won't be
     * distributed.
     * </p>
     * <p>
     * Method {@link yfiles.router.OrthogonalSegmentDistributionStage#lockFirstAndLastSegment} can be used to guarantee that the first and last segment of an edge won't be distributed. So, ports can easily
     * be maintained without setting explicit strong {@link yfiles.layout.PortConstraint}s or fixed {@link yfiles.layout.PortCandidate}s.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface OrthogonalSegmentDistributionStage extends yfiles.layout.LayoutStageBase{}
    export class OrthogonalSegmentDistributionStage {
      /**
       * Creates a new instance of {@link yfiles.router.OrthogonalSegmentDistributionStage} with default settings.
       */
      constructor();
      /**
       * Specifies the coordinates of the origin of the grid.
       * <p>
       * The grid coordinates will be multiples of the {@link yfiles.router.OrthogonalSegmentDistributionStage#gridSpacing grid spacing} added to this origin.
       * </p>
       * @param offsetX the x-coordinate of the grid origin
       * @param offsetY the y-coordinate of the grid origin
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridRouting
       */
      setGridOrigin(offsetX:number,offsetY:number):void;
      /**
       * Data provider key for determining which edges are distributed
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DEFAULT_AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} which determines the edges that shall be routed by
       * this algorithm.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.algorithms.IDataProvider} key is <code>null</code>
       * @type {Object}
       */
      affectedEdgesDpKey:Object;
      /**
       * Gets or sets whether or not the preferred distance between edges can be reduced if there is not enough space.
       * <p>
       * The space between the nodes restricts the distribution of the orthogonal segments while considering the
       * {@link yfiles.router.OrthogonalSegmentDistributionStage#preferredDistance preferred distance}. In case this option is enabled, the layout algorithm may reduce the preferred distance, otherwise
       * the algorithm will only distribute segments if there is enough space.
       * </p>
       * @type {boolean}
       */
      flexiblePreferredDistance:boolean;
      /**
       * Gets or sets whether or not the grid spacing can be reduced if there is not enough space.
       * <p>
       * The space between the nodes limits the number of grid lines on which edges are placed. In case this option is enabled,
       * the layout algorithm will divide the given grid spacing by <code>2</code> as long as there are not enough grid lines on which the
       * segments can be distributed, otherwise the grid spacing remains unchanged.
       * </p>
       * @type {boolean}
       */
      flexibleGridSpacing:boolean;
      /**
       * Gets or sets whether or not the orthogonal segments are distributed on grid coordinates.
       * @see yfiles.router.OrthogonalSegmentDistributionStage#setGridOrigin
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridSpacing
       * @see yfiles.router.OrthogonalSegmentDistributionStage#flexibleGridSpacing
       * @type {boolean}
       */
      gridRouting:boolean;
      /**
       * Gets or sets the spacing between two grid lines.
       * <p>
       * The spacing should be at least <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified spacing is negative
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridRouting
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridRouting
       * @see yfiles.router.OrthogonalSegmentDistributionStage#flexibleGridSpacing
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets the x-coordinate of the origin of the grid.
       * <p>
       * The grid coordinates will be multiples of the {@link yfiles.router.OrthogonalSegmentDistributionStage#gridSpacing grid spacing} added to this origin.
       * </p>
       * @see yfiles.router.OrthogonalSegmentDistributionStage#setGridOrigin
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridRouting
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridOriginY
       * @type {number}
       */
      gridOriginX:number;
      /**
       * Gets the y-coordinate of the origin of the grid.
       * <p>
       * The grid coordinates will be multiples of the {@link yfiles.router.OrthogonalSegmentDistributionStage#gridSpacing grid spacing} added to this origin.
       * </p>
       * @see yfiles.router.OrthogonalSegmentDistributionStage#setGridOrigin
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridRouting
       * @see yfiles.router.OrthogonalSegmentDistributionStage#gridOriginX
       * @type {number}
       */
      gridOriginY:number;
      /**
       * Gets or sets whether or not the first and last segment of an edge will be distributed.
       * @type {boolean}
       */
      lockFirstAndLastSegment:boolean;
      /**
       * Gets or sets the preferred distance between each two segments.
       * <p>
       * This value also applies to the distance between a segment and the border of the containing channel.
       * </p>
       * <p>
       * The preferred distance should be at least <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @see yfiles.router.OrthogonalSegmentDistributionStage#flexiblePreferredDistance
       * @type {number}
       */
      preferredDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.router.SnapOuterPortsToNodeBorderStage} moves edge ports that are outside the corresponding area of the nodes to
     * their border.
     * <p>
     * Thus, it guarantees that ports are always in the interior of the nodes or on their borders.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface SnapOuterPortsToNodeBorderStage extends yfiles.layout.LayoutStageBase{}
    export class SnapOuterPortsToNodeBorderStage {
      /**
       * Creates a new instance of {@link yfiles.router.SnapOuterPortsToNodeBorderStage}.
       */
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.BusRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface BusRouterData extends yfiles.layout.LayoutData{}
    export class BusRouterData {
      constructor();
      /**
       * Gets or sets the collection of edges affected by this router.
       * <p>
       * This collection is only used by the router if the {@link yfiles.router.BusRouter#scope} is set to {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES}. The {@link yfiles.layout.DpKeyItemCollection.<V>#dpKey} only has to
       * be specified if the {@link yfiles.router.BusRouter#affectedEdgesDpKey} is not {@link yfiles.router.BusRouter#DEFAULT_AFFECTED_EDGES_DP_KEY}
       * </p>
       * @see yfiles.router.BusRouter#affectedEdgesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.DpKeyItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the mapping of edges to their {@link yfiles.router.BusDescriptor}
       * @see yfiles.router.BusRouter#EDGE_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.router.BusDescriptor>}
       */
      edgeDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.router.BusDescriptor>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from nodes to their {@link yfiles.layout.PortCandidateSet}.
       * @see yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.PortCandidateSet>}
       */
      nodePortCandidateSets:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.PortCandidateSet>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.ChannelEdgeRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface ChannelEdgeRouterData extends yfiles.layout.LayoutData{}
    export class ChannelEdgeRouterData {
      constructor();
      /**
       * Gets or sets the collection of edges affected by this router.
       * @see yfiles.router.ChannelEdgeRouter#AFFECTED_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.OrganicEdgeRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface OrganicEdgeRouterData extends yfiles.layout.LayoutData{}
    export class OrganicEdgeRouterData {
      constructor();
      /**
       * Gets or sets the collection of edges routed by this router.
       * @see yfiles.router.OrganicEdgeRouter#AFFECTED_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.OrthogonalPatternEdgeRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface OrthogonalPatternEdgeRouterData extends yfiles.layout.LayoutData{}
    export class OrthogonalPatternEdgeRouterData {
      constructor();
      /**
       * Gets or sets the collection of edges affected by this router.
       * @see yfiles.router.OrthogonalPatternEdgeRouter#affectedEdgesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.DpKeyItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from nodes to their {@link yfiles.layout.PortCandidateSet}.
       * @see yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.PortCandidateSet>}
       */
      nodePortCandidateSets:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.PortCandidateSet>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.ParallelEdgeRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface ParallelEdgeRouterData extends yfiles.layout.LayoutData{}
    export class ParallelEdgeRouterData {
      constructor();
      /**
       * Gets or sets the collection of edges affected by this router.
       * @see yfiles.router.ParallelEdgeRouter#AFFECTED_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the collection of leading edges.
       * <p>
       * A leading edge is the edge whose layout is used to obtain the layout of the corresponding parallel edges.
       * </p>
       * @see yfiles.router.ParallelEdgeRouter#LEADING_EDGE_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      leadingEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.EdgeRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface PolylineEdgeRouterData extends yfiles.layout.LayoutData{}
    export class PolylineEdgeRouterData {
      constructor();
      /**
       * Gets or sets the collection of affected edges.
       * <p>
       * This collection is only used by the router if the {@link yfiles.router.EdgeRouter#scope} is set to {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES}.
       * </p>
       * @see yfiles.router.EdgeRouter#affectedEdgesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.DpKeyItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the collection of affected nodes.
       * <p>
       * This collection is only used by the router if the {@link yfiles.router.EdgeRouter#scope} is set to {@link yfiles.router.Scope#ROUTE_EDGES_AT_AFFECTED_NODES}.
       * </p>
       * @see yfiles.router.EdgeRouter#affectedNodesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.INode>}
       */
      affectedNodes:yfiles.layout.DpKeyItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the collection of labels of nodes or fixed edges that are ignored by the router.
       * @see yfiles.layout.LabelLayoutKeys#IGNORED_LABELS_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.ILabel>}
       */
      ignoredLabels:yfiles.layout.ItemCollection<yfiles.graph.ILabel>;
      /**
       * Gets or sets the mapping of edges to their {@link yfiles.router.EdgeLayoutDescriptor}
       * <p>
       * If an edge is mapped to <code>null</code>, the {@link yfiles.router.EdgeRouter#defaultEdgeLayoutDescriptor default descriptor} is used.
       * </p>
       * @see yfiles.router.EdgeRouter#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.router.EdgeLayoutDescriptor>}
       */
      edgeLayoutDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.router.EdgeLayoutDescriptor>;
      /**
       * Gets or sets a mapping from labels to a crossing penalty factor.
       * <p>
       * The crossing penalty factor of a label is multiplied with the basic penalty for an edge crossing the label so crossing a
       * label with a higher cost factor is stronger avoided then crossing one with a lower factor.
       * </p>
       * @see yfiles.router.PenaltySettings#edgeLabelCrossingPenalty
       * @see yfiles.router.PenaltySettings#nodeLabelCrossingPenalty
       * @see yfiles.router.EdgeRouter#LABEL_CROSSING_PENALTY_FACTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,number>}
       */
      labelCrossingPenaltyFactors:yfiles.layout.ItemMapping<yfiles.graph.ILabel,number>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from nodes to their {@link yfiles.layout.PortCandidateSet}.
       * @see yfiles.layout.PortCandidateSet#NODE_PORT_CANDIDATE_SET_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.PortCandidateSet>}
       */
      nodePortCandidateSets:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.PortCandidateSet>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the partition grid layout data.
       * @type {yfiles.layout.PartitionGridData}
       */
      partitionGridData:yfiles.layout.PartitionGridData;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.router.StraightLineEdgeRouter}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface StraightLineEdgeRouterData extends yfiles.layout.LayoutData{}
    export class StraightLineEdgeRouterData {
      constructor();
      /**
       * Gets or sets the collection of affected edges.
       * <p>
       * This collection is only used by the router if the {@link yfiles.router.StraightLineEdgeRouter#scope} is set to {@link yfiles.router.Scope#ROUTE_AFFECTED_EDGES}.
       * </p>
       * @see yfiles.router.StraightLineEdgeRouter#affectedEdgesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.DpKeyItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the collection of affected nodes.
       * <p>
       * This collection is only used by the router if the {@link yfiles.router.StraightLineEdgeRouter#scope} is set to {@link yfiles.router.Scope#ROUTE_EDGES_AT_AFFECTED_NODES}.
       * </p>
       * @see yfiles.router.StraightLineEdgeRouter#affectedNodesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.INode>}
       */
      affectedNodes:yfiles.layout.DpKeyItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      static $class:yfiles.lang.Class;
    }
  }export namespace partial{
    export enum EdgeRoutingStrategy{
      /**
       * A routing strategy that produces orthogonal routes for partial edges and inter-edges.
       * <p>
       * A route of an edge is called <i>orthogonal</i> if it only consists of vertical and horizontal segments.
       * </p>
       * <p>
       * <i>Inter-edges</i> are edges between fixed and partial nodes as well as edges between different subgraph components.
       * </p>
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       */
      ORTHOGONAL,
      /**
       * A routing strategy that produces straight-line routes for partial edges and inter-edges.
       * <p>
       * <i>Inter-edges</i> are edges between fixed and partial nodes as well as edges between different subgraph components.
       * </p>
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       */
      STRAIGHTLINE,
      /**
       * Automatically chooses a suitable routing strategy for partial edges and inter-edges by analyzing the routes of the fixed
       * edges. If, for example, all fixed edges have orthogonal edge routes, inter-edges will be routed orthogonally, too.
       * <p>
       * <i>Inter-edges</i> are edges between fixed and partial nodes as well as edges between different subgraph components.
       * </p>
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       */
      AUTOMATIC,
      /**
       * A routing strategy that produces organic routes for partial edges and inter-edges.
       * <p>
       * <i>Inter-edges</i> are edges between fixed and partial nodes as well as edges between different subgraph components.
       * </p>
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       */
      ORGANIC,
      /**
       * A routing strategy that produces octilinear routes for partial edges and inter-edges.
       * <p>
       * A route of an edge is called <i>octilinear</i> if the slope of each segment is a multiple of <code>45</code> degrees.
       * </p>
       * <p>
       * <i>Inter-edges</i> are edges between fixed and partial nodes as well as edges between different subgraph components.
       * </p>
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       */
      OCTILINEAR
    }
    export enum ComponentAssignmentStrategy{
      /**
       * A component assignment strategy that assigns each partial node to a separate subgraph component.
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       */
      SINGLE,
      /**
       * A component assignment strategy where the subgraph components correspond to the connected components of the graph
       * induced by the partial elements.
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       */
      CONNECTED,
      /**
       * A component assignment strategy where the subgraph components correspond to the clusters computed by a clustering
       * algorithm based on {@link yfiles.algorithms.Groups#edgeBetweennessClustering edge betweenness centrality}.
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       */
      CLUSTERING,
      /**
       * A component assignment strategy where the subgraph components are defined by the user. Two partial nodes are considered
       * to be in the same component if the {@link yfiles.algorithms.IDataProvider} associated with key {@link yfiles.partial.PartialLayout#COMPONENT_ID_DP_KEY}
       * returns the same object for both of them.
       * @see yfiles.partial.PartialLayout#COMPONENT_ID_DP_KEY
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       */
      CUSTOMIZED
    }
    export enum SubgraphPlacement{
      /**
       * A positioning strategy which tries to place each subgraph component close to the barycenter of its graph neighbors.
       * @see yfiles.partial.PartialLayout#subgraphPlacement
       */
      BARYCENTER,
      /**
       * A positioning strategy which tries to place each subgraph component close to its original position.
       * @see yfiles.partial.PartialLayout#subgraphPlacement
       */
      FROM_SKETCH
    }
    export enum LayoutOrientation{
      /**
       * Layout orientation specifier where the algorithm tries to place partial nodes (subgraph components) such that each
       * predecessor of a node <code>v</code> is placed above <code>v</code> and each successor below <code>v</code>.
       * @see yfiles.partial.PartialLayout#layoutOrientation
       */
      TOP_TO_BOTTOM,
      /**
       * Layout orientation specifier where the algorithm tries to place partial nodes (subgraph components) such that each
       * predecessor of a node <code>v</code> is placed below <code>v</code> and each successor above <code>v</code>.
       * @see yfiles.partial.PartialLayout#layoutOrientation
       */
      BOTTOM_TO_TOP,
      /**
       * Layout orientation specifier where the algorithm tries to place partial nodes (subgraph components) such that each
       * predecessor of a node <code>v</code> is placed to the left of <code>v</code> and each successor to the right of <code>v</code>.
       * @see yfiles.partial.PartialLayout#layoutOrientation
       */
      LEFT_TO_RIGHT,
      /**
       * Layout orientation specifier where the algorithm tries to place partial nodes (subgraph components) such that each
       * predecessor of a node <code>v</code> is placed to the right of <code>v</code> and each successor to the left of <code>v</code>.
       * @see yfiles.partial.PartialLayout#layoutOrientation
       */
      RIGHT_TO_LEFT,
      /**
       * Layout orientation specifier where the algorithm automatically detects the layout orientation.
       * <p>
       * The algorithm analyzes the flow direction of fixed edges in the current drawing. If there is no common flow direction,
       * the results are the same as for {@link yfiles.partial.LayoutOrientation#NONE}. Otherwise, the layout orientation is either
       * {@link yfiles.partial.LayoutOrientation#TOP_TO_BOTTOM}, {@link yfiles.partial.LayoutOrientation#BOTTOM_TO_TOP}, {@link yfiles.partial.LayoutOrientation#LEFT_TO_RIGHT}
       * or {@link yfiles.partial.LayoutOrientation#RIGHT_TO_LEFT}.
       * </p>
       * @see yfiles.partial.PartialLayout#layoutOrientation
       */
      AUTO_DETECT,
      /**
       * Layout orientation specifier where the layout orientation is completely ignored.
       * @see yfiles.partial.PartialLayout#layoutOrientation
       */
      NONE
    }
    /**
     * This class represents a partial layout algorithm which changes the coordinates for a given set of graph elements (called
     * partial elements) only.
     * <p>
     * The location and size of the remaining elements (called fixed elements) is not allowed to be changed.<b>Layout Style</b>
     * </p>
     * <p>
     * This partial layout algorithm offers a kind of generic partial layout support for other existing layout algorithms.
     * Hence, its layout style heavily depends on the selected {@link yfiles.partial.PartialLayout#coreLayout core layout algorithm} as well as the specified {@link yfiles.partial.PartialLayout#edgeRouter edge router} or
     * {@link yfiles.partial.PartialLayout#edgeRoutingStrategy edge routing strategy}.
     * </p>
     * <p>
     * The partial layout is suitable for applications where users may incrementally add new elements to an existing drawing.
     * The added elements should be arranged so that they fit best possible into the given diagram without making any changes
     * to the already existing layout. Hence, the so-called mental map of the existing drawing is preserved.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[60,30],"d":1},"n":[[49.74,127.62,295.26,342.38,2,[[49.74,150,295.26,22.38,"Group"]]],[270,425,[[291.33,449.35,17.35,18.7,"16"]]],[270,375,[[291.33,399.35,17.35,18.7,"18"]]],[200,375,[[221.33,399.35,17.35,18.7,"19"]]],[64.74,405.17,1,[[86.07,429.52,17.35,18.7,"13"]]],[200,265,[[221.33,289.35,17.35,18.7,"20"]]],[100,215,[[124.66,239.35,10.67,18.7,"2"]]],[100,165,[[124.66,189.35,10.67,18.7,"3"]]],[100,265,[[121.33,289.35,17.35,18.7,"22"]]],[-420.63,317.87,375.21,302.93,3,[[-420.63,340.25,375.21,22.38,"Group"]]],[-179.98,534.8,1,[[-155.31,559.15,10.67,18.7,"6"]]],[-163.42,363.25,1,[[-142.09,387.6,17.35,18.7,"12"]]],[-371.63,364.65,1,[[-346.96,389,10.67,18.7,"8"]]],[-333.29,532.76,1,[[-308.62,557.11,10.67,18.7,"9"]]],[-176.17,453.25,1,[[-154.84,477.6,17.35,18.7,"11"]]],[370,215,[[394.66,239.35,10.67,18.7,"1"]]],[370,165,[[394.66,189.35,10.67,18.7,"4"]]],[-130.4,215,1,[[-105.74,239.35,10.67,18.7,"5"]]],[-224.19,264.44,1,[[-199.52,288.79,10.67,18.7,"7"]]],[370,425,[[391.33,449.35,17.35,18.7,"10"]]],[370,265,[[391.33,289.35,17.35,18.7,"14"]]],[370,375,[[391.33,399.35,17.35,18.7,"15"]]],[270,505,[[291.33,529.35,17.35,18.7,"17"]]],[440,165,[[461.33,189.35,17.35,18.7,"21"]]]],"e":[[13,12,0,-15,0,15],[13,14],[10,13,-30,4,0,0],[14,11],[6,7,0,-15,0,15],[5,8,-30,0,30,0],[1,2,0,-15,0,15],[2,3,-30,0,30,0],[4,3],[15,6,-30,0,30,0],[15,16,0,-15,0,15],[7,16,30,0,-30,0],[6,17],[17,18],[18,12],[10,19],[15,20,0,15,0,-15],[19,21,0,-15,0,15],[21,20,0,-15,0,15],[19,1,-30,0,30,0],[1,22,0,15,0,-15],[20,5,-30,0,30,0],[16,23,30,0,-30,0],[11,4,-30,0,0,0]],"vp":[-421.0,127.0,921.0,494.0]}} The input graph where marked nodes denote the partial elements that were incrementally added to the existing diagram. {@graph {"ann":{"s":[60,30],"d":1},"n":[[67.37,127.62,260,342.38,2,[[67.37,150,260,22.38,"Group"]]],[252.37,425,[[273.7,449.35,17.35,18.7,"16"]]],[252.37,375,[[273.7,399.35,17.35,18.7,"18"]]],[182.37,375,[[203.7,399.35,17.35,18.7,"19"]]],[182.37,321,1,[[203.7,345.35,17.35,18.7,"13"]]],[182.37,265,[[203.7,289.35,17.35,18.7,"20"]]],[82.37,215,[[107.03,239.35,10.67,18.7,"2"]]],[82.37,165,[[107.03,189.35,10.67,18.7,"3"]]],[82.37,265,[[103.7,289.35,17.35,18.7,"22"]]],[-147.05,387.62,180,185.56,3,[[-147.05,410,180,22.38,"Group"]]],[-42.05,425,1,[[-17.38,449.35,10.67,18.7,"6"]]],[-42.05,474.19,1,[[-20.72,498.54,17.35,18.7,"12"]]],[-132.05,425,1,[[-107.38,449.35,10.67,18.7,"8"]]],[-132.05,474.19,1,[[-107.38,498.54,10.67,18.7,"9"]]],[-42.05,528.19,1,[[-20.72,552.54,17.35,18.7,"11"]]],[352.37,215,[[377.03,239.35,10.67,18.7,"1"]]],[352.37,165,[[377.03,189.35,10.67,18.7,"4"]]],[-13.63,215,1,[[11.03,239.35,10.67,18.7,"5"]]],[-13.63,265,1,[[11.03,289.35,10.67,18.7,"7"]]],[352.37,425,[[373.7,449.35,17.35,18.7,"10"]]],[352.37,265,[[373.7,289.35,17.35,18.7,"14"]]],[352.37,375,[[373.7,399.35,17.35,18.7,"15"]]],[252.37,505,[[273.7,529.35,17.35,18.7,"17"]]],[422.37,165,[[443.7,189.35,17.35,18.7,"21"]]]],"e":[[13,12,0,-15,0,15],[13,14,0,15,-30,0,[-102.05,543.19]],[10,13,-30,0,30,0,[-54.55,440,-54.55,489.19]],[14,11,0,-15,0,15],[6,7,0,-15,0,15],[5,8,-30,0,30,0],[1,2,0,-15,0,15],[2,3,-30,0,30,0],[4,3,0,15,0,-15],[15,6,-30,0,30,0],[15,16,0,-15,0,15],[7,16,30,0,-30,0],[6,17,-30,0,30,0],[17,18,0,15,0,-15],[18,12,-30,0,0,-15,[-102.05,280]],[10,19,30,0,0,15,[27.95,440,27.95,474,382.37,474]],[15,20,0,15,0,-15],[19,21,0,-15,0,15],[21,20,0,-15,0,15],[19,1,-30,0,30,0],[1,22,0,15,0,-15],[20,5,-30,0,30,0],[16,23,30,0,-30,0],[11,4,30,5.2,-30,0,[119.87,494.39,119.87,336]]],"vp":[-148.0,127.0,631.0,447.0]}}
     * The result of a partial layout run with {@link yfiles.partial.PartialLayout#componentAssignmentStrategy component assignment strategy} set to {@link yfiles.partial.ComponentAssignmentStrategy#SINGLE}
     * and with {@link yfiles.partial.PartialLayout#considerNodeAlignment node alignment} enabled.<b>Features</b>
     * </p>
     * <p>
     * Similar to the layout style, the supported feature set mainly depends on the features supported by the specified {@link yfiles.partial.PartialLayout#coreLayout core layout algorithm}
     * as well as the specified {@link yfiles.partial.PartialLayout#edgeRouter edge router} or {@link yfiles.partial.PartialLayout#edgeRoutingStrategy edge routing strategy}. The internal step that {@link yfiles.partial.PartialLayout#placeSubgraphs places the components} can handle
     * group nodes as well as minimum distance constraints. Furthermore, it is able to consider node labels.
     * </p>
     * <p>
     * <b>Concept</b> The layout algorithm tries to place the partial elements such that the resulting drawing (including the fixed elements)
     * has a good quality with respect to common graph drawing aesthetics.
     * </p>
     * <p>
     * The layout algorithm handles each selected graph element as partial element. For this, it looks up the {@link yfiles.algorithms.IDataProvider}
     * keys {@link yfiles.partial.PartialLayout#AFFECTED_NODES_DP_KEY} and {@link yfiles.partial.PartialLayout#AFFECTED_EDGES_DP_KEY}. Partial
     * node elements can be assigned to the so-called <i>subgraph components</i>. During the layout process each subgraph induced by the nodes of a
     * component is first laid out using the specified {@link yfiles.partial.PartialLayout#coreLayout core layout algorithm}. Then, the different components are placed
     * one-by-one onto the drawing area such that the number of overlaps among graph elements is small. The user can specify
     * different objectives for finding 'good' positions for subgraph components (see {@link yfiles.partial.PartialLayout#subgraphPlacement}), e.g., {@link yfiles.partial.SubgraphPlacement#BARYCENTER}
     * specifies that the component should be placed close to the barycenter of its graph neighbors and {@link yfiles.partial.SubgraphPlacement#FROM_SKETCH}
     * specifies that the component should be placed close to its original position.
     * </p>
     * <p>
     * Method {@link yfiles.partial.PartialLayout#componentAssignmentStrategy} allows to specify the strategy that assigns partial nodes to subgraph components. Possible values are
     * {@link yfiles.partial.ComponentAssignmentStrategy#CLUSTERING}, {@link yfiles.partial.ComponentAssignmentStrategy#CONNECTED}, {@link yfiles.partial.ComponentAssignmentStrategy#SINGLE}
     * and
     * {@link yfiles.partial.ComponentAssignmentStrategy#CUSTOMIZED}. The last value allows to use a customized component assignment.
     * Note that nodes of a component cannot be assigned to different group nodes.
     * </p>
     * <p>
     * Furthermore, the user can specify the edge routing strategy (see {@link yfiles.partial.PartialLayout#edgeRoutingStrategy}) that is used for routing partial edges and edges
     * between different subgraph components (so-called <i>inter-edges</i>). Possible values are {@link yfiles.partial.EdgeRoutingStrategy#ORGANIC},
     * {@link yfiles.partial.EdgeRoutingStrategy#ORTHOGONAL}, {@link yfiles.partial.EdgeRoutingStrategy#STRAIGHTLINE}, {@link yfiles.partial.EdgeRoutingStrategy#OCTILINEAR}
     * and {@link yfiles.partial.EdgeRoutingStrategy#AUTOMATIC}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface PartialLayout extends yfiles.layout.LayoutStageBase{}
    export class PartialLayout {
      /**
       * Creates a new instance of {@link yfiles.partial.PartialLayout} which uses the specified {@link yfiles.layout.ILayoutAlgorithm} instance
       * as the core layout algorithm.
       * <p>
       * This instance is applied to each subgraph component, see {@link yfiles.partial.PartialLayout#componentAssignmentStrategy}.
       * </p>
       * @param [subgraphLayouter=null] the layout algorithm that is applied to the subgraph components
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       */
      constructor(subgraphLayouter?:yfiles.layout.ILayoutAlgorithm);
      /**
       * This method is called each time when edges are routed with an edge router.
       * <p>
       * Subclasses may modify the configuration of the given edge router instance.
       * </p>
       * <p>
       * The type of the given instance depends on the edge routing strategy, i.e., if the routing strategy is set to {@link yfiles.partial.EdgeRoutingStrategy#OCTILINEAR}
       * or {@link yfiles.partial.EdgeRoutingStrategy#ORTHOGONAL} it's an instance of {@link yfiles.router.EdgeRouter}, if the routing strategy
       * is set to {@link yfiles.partial.EdgeRoutingStrategy#ORGANIC} it's an instance of {@link yfiles.organic.OrganicLayout}, and, if the
       * routing strategy is set to {@link yfiles.partial.EdgeRoutingStrategy#STRAIGHTLINE} it's an instance of a private
       * {@link yfiles.router.StraightLineEdgeRouter}. If the edge routing strategy is set to
       * {@link yfiles.partial.EdgeRoutingStrategy#AUTOMATIC}, the layout algorithm chooses one of the above strategies that best fits
       * the routing style of the fixed edges.
       * </p>
       * @param edgeRouter the instance used for routing the edges
       * @see yfiles.partial.PartialLayout#edgeRouter
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       * @protected
       */
      configureEdgeRouter(edgeRouter:yfiles.layout.ILayoutAlgorithm):void;
      /**
       * This method is called during the {@link yfiles.partial.PartialLayout#applyLayout layout process} and calculates the layout for the given subgraph component using the
       * specified {@link yfiles.partial.PartialLayout#coreLayout core layout algorithm}.
       * <p>
       * It is called once for each subgraph component.
       * </p>
       * <p>
       * Subclasses may implement a custom layout strategy or add some additional data.
       * </p>
       * @param subGraph the subgraph component
       * @protected
       */
      layoutSubgraph(subGraph:yfiles.layout.LayoutGraph):void;
      /**
       * This method is called during the {@link yfiles.partial.PartialLayout#applyLayout layout process} and places the subgraph components one-by-one onto the drawing area.
       * <p>
       * Therefore, it considers the {@link yfiles.partial.PartialLayout#subgraphPlacement specified objective} for finding a suitable position.
       * </p>
       * <p>
       * Subclasses may implement a custom placement strategy or add some additional data. However, they must not modify the
       * given subgraph component lists.
       * </p>
       * @param graph the input graph
       * @param subgraphComponents each entry contains a {@link } that induces a subgraph component
       * @protected
       */
      placeSubgraphs(graph:yfiles.layout.LayoutGraph,subgraphComponents:yfiles.algorithms.NodeList[]):void;
      /**
       * This method is called during the {@link yfiles.partial.PartialLayout#applyLayout layout process} and routes all partial edges that connect two fixed elements.
       * <p>
       * It either uses a {@link yfiles.partial.PartialLayout#edgeRouter custom edge router} or an internal edge router that produces routes according to the specified
       * {@link yfiles.partial.PartialLayout#edgeRoutingStrategy routing strategy}.
       * </p>
       * <p>
       * Subclasses may implement a custom routing strategy or add some additional data.
       * </p>
       * @param graph the subgraph of the input graph induced by the fixed nodes
       * @param partialEdges the {@link  list of partial edges} to be routed
       * @protected
       */
      routeEdgesBetweenFixedElements(graph:yfiles.layout.LayoutGraph,partialEdges:yfiles.algorithms.EdgeList):void;
      /**
       * This method is called during the {@link yfiles.partial.PartialLayout#applyLayout layout process} and routes all inter-edges.
       * <p>
       * <i>Inter-edges</i> are edges between different subgraph components including edges between fixed and partial elements. For the routing,
       * this method uses the edge router instance set with method {@link yfiles.partial.PartialLayout#edgeRouter}. If no edge router was specified by the user, it uses an
       * internal edge router with routing strategy {@link yfiles.partial.PartialLayout#edgeRoutingStrategy}.
       * </p>
       * <p>
       * Subclasses may implement a custom routing strategy or add some additional data.
       * </p>
       * @param graph the relevant subgraph
       * @param interEdges the {@link  list of inter-edges} to be routed
       * @protected
       */
      routeInterEdges(graph:yfiles.layout.LayoutGraph,interEdges:yfiles.algorithms.EdgeList):void;
      /**
       * Data provider key for marking partial nodes
       * @see yfiles.partial.PartialLayout#AFFECTED_EDGES_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static AFFECTED_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for specifying the edges that should be considered to be directed
       * <p>
       * If a layout orientation is specified (i.e., {@link yfiles.partial.PartialLayout#layoutOrientation} is not
       * {@link yfiles.partial.LayoutOrientation#NONE}), the algorithm tries to route directed edges such that they adhere to that
       * orientation.
       * </p>
       * @see yfiles.partial.PartialLayout#layoutOrientation
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DIRECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Data provider key for marking partial edges
       * @see yfiles.partial.PartialLayout#AFFECTED_NODES_DP_KEY
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static AFFECTED_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Data provider key for obtaining the edges that should be routed by the edge router
       * @see yfiles.partial.PartialLayout#edgeRouter
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static ROUTE_EDGE_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Data provider key for defining custom subgraph components
       * @see yfiles.partial.ComponentAssignmentStrategy#CUSTOMIZED
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static COMPONENT_ID_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutAlgorithm} instance that is applied to each subgraph component.
       * <p>
       * More precisely, during the layout process each subgraph induced by the (partial) nodes of a component (see
       * {@link yfiles.partial.PartialLayout#componentAssignmentStrategy}) is first laid out using this instance.
       * </p>
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the preferred time limit (in milliseconds) for the layout algorithm.
       * <p>
       * The specified value has to be greater than or equal to <code>0</code>. If the value is <code>0x7FFFFFFF</code>, the time is not limited.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the maximum duration is negative
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets whether or not edges between different subgraph components should be routed immediately.
       * <p>
       * If this option is enabled, edges are routed during the placement of the subgraph components, i.e., immediately after a
       * component is placed, its edges to other already placed components are routed. Otherwise, these edges are routed in a
       * separate step after placing all subgraph components. Hence, while enabling this option usually leads to shorter edge
       * routes, the placement of subgraph components is less compact.
       * </p>
       * @see yfiles.partial.PartialLayout#routeInterEdges
       * @see yfiles.partial.PartialLayout#placeSubgraphs
       * @type {boolean}
       */
      routeInterEdgesImmediately:boolean;
      /**
       * Gets or sets whether or not a subgraph component may be placed within another subgraph component.
       * <p>
       * Enabling this option leads to more compact layout results but requires more runtime.
       * </p>
       * @type {boolean}
       */
      componentCompaction:boolean;
      /**
       * Gets or sets whether or not fixed (non-partial) group nodes may be resized.
       * <p>
       * Enabling this option may lead to better results if there are fixed group nodes, since there is more space for the
       * partial elements.
       * </p>
       * @type {boolean}
       */
      resizeFixedGroups:boolean;
      /**
       * Gets or sets the objective used for finding 'good' positions for subgraph components.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified strategy does not match one of the predefined positioning strategies
       * @type {yfiles.partial.SubgraphPlacement}
       */
      subgraphPlacement:yfiles.partial.SubgraphPlacement;
      /**
       * Gets or sets the minimum distance between two adjacent nodes.
       * <p>
       * The specified value has to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the distance is negative
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets whether or not partial nodes should be aligned.
       * <p>
       * If this option is enabled, the algorithm tries to align the center of partial nodes with other nodes.
       * </p>
       * @type {boolean}
       */
      considerNodeAlignment:boolean;
      /**
       * Gets or sets the strategy that assigns partial nodes to subgraph components.
       * <p>
       * The specified core layouter (see {@link yfiles.partial.PartialLayout#coreLayout}) independently calculates the layout for each such subgraph component.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified strategy does not match one of the predefined assignment strategies.
       * @see yfiles.partial.PartialLayout#coreLayout
       * @type {yfiles.partial.ComponentAssignmentStrategy}
       */
      componentAssignmentStrategy:yfiles.partial.ComponentAssignmentStrategy;
      /**
       * Gets or sets whether or not a postprocessing step should be applied to reduce the number of directed edges that do not
       * comply with the specified layout orientation.
       * @see yfiles.partial.PartialLayout#layoutOrientation
       * @see yfiles.partial.PartialLayout#DIRECTED_EDGES_DP_KEY
       * @type {boolean}
       */
      optimizeOrientation:boolean;
      /**
       * Gets or sets the custom edge router instance that is used for partial edges and edges between different subgraph
       * components (so-called inter-edges).
       * @see yfiles.partial.PartialLayout#ROUTE_EDGE_DP_KEY
       * @see yfiles.partial.PartialLayout#edgeRoutingStrategy
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      edgeRouter:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the routing strategy that is used for partial edges and edges between different subgraph components
       * (so-called inter-edges).
       * @throws {Stubs.Exceptions.ArgumentError} if the specified strategy does not match one of the predefined routing strategies
       * @see yfiles.partial.PartialLayout#edgeRouter
       * @type {yfiles.partial.EdgeRoutingStrategy}
       */
      edgeRoutingStrategy:yfiles.partial.EdgeRoutingStrategy;
      /**
       * Gets or sets the layout orientation that is considered during the placement of partial elements.
       * <p>
       * More precisely, the algorithm tries to place each {@link yfiles.partial.PartialLayout#componentAssignmentStrategy subgraph component} such that each predecessor of a component's node <code>v</code>
       * is placed before <code>v</code> and each successor after <code>v</code> with respect to the layout orientation.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified orientation does not match one of the predefined orientations
       * @see yfiles.partial.PartialLayout#DIRECTED_EDGES_DP_KEY
       * @type {yfiles.partial.LayoutOrientation}
       */
      layoutOrientation:yfiles.partial.LayoutOrientation;
      /**
       * Gets or sets whether or not subgraph components are mirrored to improve the layout quality.
       * <p>
       * If enabled, the algorithm checks for each component which of the four possible mirrorings minimizes the edge length.
       * </p>
       * @type {boolean}
       */
      allowMirroring:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.partial.PartialLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface PartialLayoutData extends yfiles.layout.LayoutData{}
    export class PartialLayoutData {
      constructor();
      /**
       * Gets or sets the collection of nodes placed by the layout.
       * @see yfiles.partial.PartialLayout#AFFECTED_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      affectedNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the collection of edges placed by the layout.
       * @see yfiles.partial.PartialLayout#AFFECTED_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      affectedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the collection of edges that are considered as directed by the layout.
       * <p>
       * Only if the {@link yfiles.partial.PartialLayout#layoutOrientation} is not {@link yfiles.partial.LayoutOrientation#NONE} the directedness of edges has an effect.
       * </p>
       * @see yfiles.partial.PartialLayout#DIRECTED_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      directedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the mapping from partial nodes to an object defining their component assignment.
       * <p>
       * All partial nodes associated with the same object are assigned to the same subgraph component.
       * </p>
       * @see yfiles.partial.PartialLayout#COMPONENT_ID_DP_KEY
       * @see yfiles.partial.ComponentAssignmentStrategy#CUSTOMIZED
       * @see yfiles.partial.PartialLayout#componentAssignmentStrategy
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,Object>}
       */
      componentIds:yfiles.layout.ItemMapping<yfiles.graph.INode,Object>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to a collection of their source port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#SOURCE_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      sourcePortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets a mapping from edges to a collection of their target port {@link yfiles.layout.PortCandidate candidates}.
       * @see yfiles.layout.PortCandidate#TARGET_PORT_CANDIDATE_COLLECTION_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.collections.ICollection.<yfiles.layout.PortCandidate>>}
       */
      targetPortCandidates:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.collections.ICollection<yfiles.layout.PortCandidate>>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the partition grid layout data.
       * @type {yfiles.layout.PartitionGridData}
       */
      partitionGridData:yfiles.layout.PartitionGridData;
      static $class:yfiles.lang.Class;
    }
  }export namespace tree{
    /**
     * Encapsulates information bound to a node while the algorithm calculates a layout.
     * @class
     */
    export interface BalloonLayoutNodeInfo extends Object{}
    export class BalloonLayoutNodeInfo {
      /**
       * The upper angle of the subtree wedge rooted at the node.
       * @type {number}
       */
      upperAngle:number;
      /**
       * The lower angle of the subtree wedge rooted at the node.
       * @type {number}
       */
      lowerAngle:number;
      /**
       * The angle of the gap to the wedge of the previous child node.
       * @type {number}
       */
      gapAngle:number;
      /**
       * The distance of the node to its root node.
       * @type {number}
       */
      dist:number;
      /**
       * Gets the sum of all wedge angles of the subtree rooted at this node.
       * @type {number}
       */
      angleSum:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A tree layout algorithm that arranges the subtrees of the tree in a balloon-like fashion.
     * <p>
     * <b>Layout Style</b> {@link yfiles.tree.BalloonLayout} is designed to arrange directed and undirected tree graphs. Subtrees rooted at a node are
     * placed in a radial fashion around their root node. All direct children of one node can be placed on a common circle
     * around their parent node (depending on the {@link yfiles.tree.BalloonLayout#childAlignmentPolicy alignment policy}). Therefore, subtrees look like balloons or stars,
     * especially if subtrees have similar sizes. The edges of the tree are drawn as straight lines. {@graph {"ann":{"s":[30,30],"d":1,"c":1},"n":[[765.94,466.2],[616.19,1035.44],[858.96,1192.81],[289.45,1092.35],[983.21,685.48],[390.63,1191.59],[1141.69,322.61],[1215.22,473.98],[599.06,923.71],[924.21,1082.3],[291.41,844.85],[1077.22,396.06],[999.14,1284.78],[391.43,1274.01],[324.89,753.04],[236.05,1223.63],[930.33,1330.15],[1013.36,1203.58],[1165.73,539.9],[1084.13,551.52],[612.56,192.18],[1004.03,1061.74],[901.39,675.54],[684.7,152.3],[1022.85,458.01],[190.18,1336.4],[767.32,1282.66],[473.05,1190.8],[684.09,233.13],[385.33,871.88],[1139.17,442.18],[925.13,116.39],[353.13,675.6],[334.78,932.43],[685.33,1291.17],[152.25,1056.81],[427.86,640.81],[1239.04,331.19],[1006.49,764.55],[934.86,198.24],[732.68,1169.96],[1046.6,632.79],[412.14,1021.39],[1266.98,538.13],[716.32,76.19],[1068.06,1329.99],[76.94,1090.32],[792.51,44.73],[576.22,1141.17],[944.59,280.09],[718.72,987.86],[132.38,1277.64],[1007.52,113.83],[318.35,600.87],[686.08,946.6],[0,1060.76],[621.34,1239.22],[684.87,0],[47.38,1167.26],[929.76,34.1],[866.88,1320.91],[425.22,944.01],[1181.1,233.18],[374.69,793.72],[118.75,981.5],[903.65,1002.48],[532.13,493.99],[200.85,981.73],[760.81,183.93],[633.38,1355.17],[1177.35,621.5],[775.83,1364.64],[784.35,1446.63],[1237.23,394.55],[272.59,1334.67],[974.02,603.57],[457.46,831.99],[159.13,1412.75],[260.6,752.11],[1069.67,256.55],[1044.67,310.87],[1121.63,226.96],[1225.82,272.87],[597.56,111.13],[530.37,198.46],[610.06,274.57],[555.63,251.79],[547.23,141.92],[1062.07,709.49],[929.94,748.38],[843.55,104.61],[857.02,162.82],[1295.93,457.26],[193.97,837.29],[246.66,931.74],[206.06,892.47],[214.43,784.65],[979.61,1149.06],[838.99,1066.04],[786.55,1086.86],[172.43,1172.3],[317.44,1231.28],[428.9,1117.65],[872.95,52.59],[984.14,58.85],[987.6,170.17],[521.61,289.78],[410.77,300.73],[412.02,412.1],[523.07,420.56],[465.9,437.92],[389.75,356.66],[464.06,273.7],[553.17,340.51],[472.17,355.73]],"e":[[0,1],[1,2],[1,3],[0,4],[3,5],[0,6],[6,7],[1,8],[2,9],[3,10],[6,11],[2,12],[5,13],[10,14],[3,15],[12,16],[12,17],[7,18],[18,19],[0,20],[9,21],[4,22],[20,23],[11,24],[15,25],[2,26],[5,27],[20,28],[10,29],[7,30],[0,31],[14,32],[10,33],[26,34],[3,35],[32,36],[6,37],[4,38],[31,39],[2,40],[4,41],[3,42],[7,43],[23,44],[12,45],[35,46],[44,47],[1,48],[39,49],[1,50],[25,51],[31,52],[32,53],[1,54],[46,55],[34,56],[44,57],[46,58],[31,59],[2,60],[29,61],[6,62],[10,63],[35,64],[9,65],[0,66],[3,67],[23,68],[34,69],[18,70],[26,71],[71,72],[7,73],[25,74],[4,75],[29,76],[25,77],[10,78],[6,79],[6,80],[6,81],[6,82],[20,83],[20,84],[20,85],[20,86],[20,87],[4,88],[4,89],[31,90],[31,91],[7,92],[10,93],[10,94],[10,95],[10,96],[2,97],[2,98],[2,99],[3,100],[3,101],[3,102],[31,103],[31,104],[31,105],[0,114],[114,106],[114,107],[114,108],[114,109],[114,110],[114,111],[114,112],[114,113]],"vp":[0.0,0.0,1326.0,1477.0]}} Sample balloon-drawing of
     * a large tree obtained with default settings {@graph {"ann":{"s":[20,20],"d":1,"c":1},"n":[[302.82,325.51,70,70,[[323.15,369.86,29.35,18.7,"Root"]]],[467.87,433.16,35,35],[555.28,457.63,[[581.09,480.55,27.36,18.7,0.20775573281806076,-0.978180737635551,"Leaf"]]],[539.95,490.69,[[558.57,519.19,27.36,18.7,0.6124052098749311,-0.7905440271851036,"Leaf"]]],[511.78,513.79,[[521.49,544.2,27.36,18.7,0.8952174454926904,-0.4456295830457645,"Leaf"]]],[617.42,438.44,[[645.7,457.5,27.36,18.7,-0.01562900399598557,-0.9998778596579152,"Leaf"]]],[604.25,500.42,[[626.78,526.54,27.36,18.7,0.42067550054279435,-0.9072111789672069,"Leaf"]]],[168.96,426.6,35,35],[122.16,495.14,[[108.91,545.35,27.36,18.7,-0.7471310564613752,-0.6646767518658294,"Leaf"]]],[101.02,465.46,[[72.6,501.56,27.36,18.7,-0.38380727069642384,-0.9234132222144982,"Leaf"]]],[94.89,429.54,[[58.95,446.33,27.36,18.7,0.0558744858236695,-0.9984378006836183,"Leaf"]]],[108.08,558.63,[[104.38,613.01,27.36,18.7,-0.8765491095440469,-0.48131243341257923,"Leaf"]]],[60.74,516.51,[[39.11,560.52,27.36,18.7,-0.5800788870754927,-0.8145603014935467,"Leaf"]]],[472.4,260.75,35,35],[534.73,207.69,[[563.84,210.51,27.36,18.7,-0.7413022827299817,-0.6711713086979497,"Leaf"]]],[555.61,237.55,[[585.94,249.4,27.36,18.7,-0.3757344582282841,-0.9267273692407589,"Leaf"]]],[561.43,273.52,[[588.93,294.02,27.36,18.7,0.06458526120504259,-0.997912192547559,"Leaf"]]],[549.36,144.32,[[576.39,143.07,27.36,18.7,-0.8723155432038684,-0.48894334343044304,"Leaf"]]],[596.34,186.85,[[626.56,194.12,27.36,18.7,-0.5729485100623841,-0.8195913645331399,"Leaf"]]],[316.53,515.88,35,35],[349.51,601,[[356.29,631.15,27.36,18.7,0.9501038773669919,-0.3119336823945245,"Leaf"]]],[313.23,604.36,[[326.48,660.69,27.36,18.7,-0.9912225732921417,-0.1322036693745865,"Leaf"]]],[279.09,591.61,[[271.88,644.75,27.36,18.7,-0.8351387927609661,-0.5500392684397689,"Leaf"]]],[397.33,645.07,[[408.68,675.44,27.36,18.7,0.8566078613378385,-0.5159679950289693,"Leaf"]]],[337.13,664.84,[[339.49,693.76,27.36,18.7,0.9957417304784997,-0.09218680048511825,"Leaf"]]],[52.71,198.2,[[17.59,197.15,27.36,18.7,0.42857629306653433,-0.9035055954565794,"Leaf"]]],[39,260.06,[[3.57,279.73,27.36,18.7,-0.006902939255060647,-0.9999761744309916,"Leaf"]]],[145.3,185.63,[[126.97,158.82,27.36,18.7,0.8990721607310399,-0.43780046801987216,"Leaf"]]],[116.92,208.49,[[85.4,197.66,27.36,18.7,0.6192806018773118,-0.7851697498875478,"Leaf"]]],[101.3,241.41,[[64.86,250.7,27.36,18.7,0.21628395104715628,-0.9763305037329477,"Leaf"]]],[173.56,251.58,35,35],[321.26,36.12,[[318.14,1.56,27.36,18.7,0.9965082870714412,-0.08349391473599899,"Leaf"]]],[260.88,55.36,[[239.7,30.94,27.36,18.7,0.861077857359917,-0.5084731296385823,"Leaf"]]],[378.65,109.85,[[406.52,110,27.36,18.7,-0.8303070560332068,-0.557306192950939,"Leaf"]]],[344.63,96.81,[[366.44,90.16,27.36,18.7,-0.9900311505600138,-0.14084857443657478,"Leaf"]]],[308.31,99.85,[[295.59,69.34,27.36,18.7,0.9527898006751772,-0.30363068970273144,"Leaf"]]],[325.62,170.19,35,35],[620.15,245.57,[[649.55,261.91,27.36,18.7,-0.15959409597274285,-0.9871827209441234,"Leaf"]]],[565.45,550.51,[[579.72,580.47,27.36,18.7,0.7732871466592579,-0.6340559823955481,"Leaf"]]],[36.42,458,[[3.15,484.87,27.36,18.7,-0.1682027041762925,-0.9857524285071697,"Leaf"]]],[91.95,148.45,[[66.13,128.88,27.36,18.7,0.7787908142925284,-0.6272837217508362,"Leaf"]]],[383.99,45.04,[[409.22,41.45,27.36,18.7,-0.933684654259895,-0.3580963088326663,"Leaf"]]],[274.32,656.46,[[277.16,712.35,27.36,18.7,-0.9367740425600954,-0.34993484134280844,"Leaf"]]]],"e":[[0,1,[],[[405.66,401.96,61.37,18.7,0.5213485166954306,-0.8533438487148509,"Root-Edge"]]],[1,2],[1,3],[1,4],[1,5],[1,6],[0,7,[],[[268.23,398.95,61.37,18.7,0.4834440722369951,0.8753752504034548,"Root-Edge"]]],[7,8],[7,9],[7,10],[7,11],[7,12],[0,13,[],[[407.74,322.69,61.37,18.7,-0.4757866709710364,-0.8795607106540735,"Root-Edge"]]],[13,14],[13,15],[13,16],[13,17],[13,18],[0,19,[],[[336.08,439.99,61.37,18.7,0.999759704932882,0.02192104907427582,"Root-Edge"]]],[19,20],[19,21],[19,22],[19,23],[19,24],[0,30,[],[[270.35,318.48,61.37,18.7,-0.5287754007295448,0.8487617896579165,"Root-Edge"]]],[30,29],[30,28],[30,27],[30,26],[30,25],[0,36,[],[[340.26,281.05,61.37,18.7,-0.9995303423338449,-0.030644652942837143,"Root-Edge"]]],[36,35],[36,34],[36,33],[36,32],[36,31],[13,37],[1,38],[7,39],[30,40],[36,41],[19,42]],"vp":[-1.0,0.0,678.0,722.0]}} Sample balloon-drawing featuring interleaved child placement and ray-like
     * node labels <b>Concept</b>The algorithm executes the following steps:
     * </p>
     * <ol>
     * <li>Select a root node according to the specified {@link yfiles.tree.BalloonLayout#rootNodePolicy root policy}.</li>
     * <li>Determine the placement of subtrees around the root using a bottom-up recursive approach (starting with leaf nodes).</li>
     * <li>Assign the actual coordinates of nodes, again using a recursive approach (starting with the root node).</li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The algorithm features integrated {@link yfiles.tree.BalloonLayout#integratedEdgeLabeling edge labeling} as well as {@link yfiles.tree.BalloonLayout#integratedNodeLabeling node labeling}. Edge labels and node labels are placed
     * automatically without generating overlaps with other labels or graph elements. There are different {@link yfiles.tree.BalloonLayout#nodeLabelingPolicy ways} to place node
     * labels. Edge labeling will take the settings of {@link yfiles.layout.PreferredPlacementDescriptor} into account.
     * </p>
     * <p>
     * Defining a {@link yfiles.tree.BalloonLayout#preferredChildWedge preferred wedge angle} has a great influence on the layout style. Subtrees rooted at a node get a certain
     * amount of radial space to be placed around the parent node, such that a preferred angle close to <code>360</code> degrees will
     * generate drawings where subtrees look like balloons, while an angle close to <code>180</code> degrees could be chosen to get drawings
     * where subtrees look like semicircles.
     * </p>
     * <p>
     * Since it is computationally not very complex, {@link yfiles.tree.BalloonLayout} is very well suited for large tree graphs. It
     * performs well even for huge graphs.
     * </p>
     * <p>
     * This layout algorithm can only handle graphs with a tree structure. To apply it to a general graph, a {@link yfiles.tree.TreeReductionStage}
     * can be appended. This stage will temporarily remove some edges of the input graph until a tree is obtained. These edges
     * will later be reinserted and routed separately.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface BalloonLayout extends yfiles.layout.MultiStageLayout{}
    export class BalloonLayout {
      /**
       * Creates a new {@link yfiles.tree.BalloonLayout} instance with default settings.
       */
      constructor();
      /**
       * Calculates the wedge angle that has to be reserved for the subtree rooted at the given node scaling the {@link yfiles.tree.BalloonLayoutNodeInfo#dist distance} with
       * the given scale factor.
       * <p>
       * Given some {@link yfiles.tree.BalloonLayoutNodeInfo#dist distance} the {@link yfiles.tree.BalloonLayoutNodeInfo#upperAngle upper angle} and {@link yfiles.tree.BalloonLayoutNodeInfo#lowerAngle lower angle} of the wedge belonging to the subtree rooted at <code>root</code> will be
       * calculated and stored in the {@link yfiles.tree.BalloonLayoutNodeInfo} instance associated with <code>root</code>
       * ({@link yfiles.tree.BalloonLayout#getInfo}).
       * </p>
       * <p>
       * This method may be overridden to perform a custom wedge angle assignment scheme. The method is called when
       * {@link yfiles.tree.BalloonLayout#calculateChildArrangement arranging child nodes}. Large edge labels on the incoming edge to <code>root</code> need to be considered, if {@link yfiles.tree.BalloonLayout#integratedEdgeLabeling integrated edge labeling}
       * should still work properly.
       * </p>
       * @param root the node for which the wedge angles are calculated
       * @param [scaleFactor=1.0] a factor to be applied to the {@link #dist distance} of <code>root</code>
       * @returns the sum of the upper and lower wedge angle of the subtree rooted at the given root node
       * @see yfiles.tree.BalloonLayout#calculateChildArrangement
       * @protected
       */
      calculateAngles(root:yfiles.algorithms.Node,scaleFactor?:number):number;
      /**
       * Calculates a child node arrangement for a given root node of the tree.
       * <p>
       * During the arrangement, child nodes of <code>root</code> will be {@link yfiles.tree.BalloonLayout#sortChildNodes sorted}. Furthermore, distances of the child nodes will be chosen
       * such that the wedge of the subtree of <code>root</code> fits into the preferred wedge angle, which is either defined via {@link yfiles.tree.BalloonLayout#preferredRootWedge} or {@link yfiles.tree.BalloonLayout#preferredChildWedge}.
       * Calculated distances are stored in {@link yfiles.tree.BalloonLayoutNodeInfo#dist}.
       * </p>
       * <p>
       * The angle values - upper and lower wedge angle - may also be updated during this process and stored in {@link yfiles.tree.BalloonLayoutNodeInfo#upperAngle}
       * and {@link yfiles.tree.BalloonLayoutNodeInfo#lowerAngle}, respectively. To compute the angles of wedges, method {@link yfiles.tree.BalloonLayout#calculateAngles}
       * is used.
       * </p>
       * <p>
       * This method may be overridden to perform a custom child node arrangement. If support for available features like {@link yfiles.tree.BalloonLayout#interleavedMode interleaving}
       * should be maintained, then these features need to be carefully considered during the arrangement.
       * </p>
       * @param root the node for whose children to compute an arrangement
       * @protected
       */
      calculateChildArrangement(root:yfiles.algorithms.Node):void;
      /**
       * Determines the root node of {@link yfiles.tree.BalloonLayout#graph} according to the chosen {@link yfiles.tree.BalloonLayout#rootNodePolicy root node policy}.
       * <p>
       * This method may be overridden to implement some other strategy for choosing the root node of the input graph. It is
       * called before the rest of the layout process within {@link yfiles.tree.BalloonLayout#applyLayoutCore}.
       * </p>
       * @returns the root node of {@link #graph}
       * @see yfiles.tree.BalloonLayout#rootNodePolicy
       * @protected
       */
      determineRoot():yfiles.algorithms.Node;
      /**
       * Returns the {@link yfiles.tree.BalloonLayoutNodeInfo} object associated with the given node while the layout algorithm is
       * active.
       * <p>
       * The returned object contains detailed information describing the placement of a node in the layout being computed, e.g.,
       * a node's distance to its parent or the wedge angle of the subtree rooted at a node.
       * </p>
       * <p>
       * Subclasses may want to override this method to realize another node information setup. This method is called throughout
       * the algorithm, each time some information associated with a node needs to be retrieved or stored.
       * </p>
       * @param node the node whose information object should be retrieved
       * @returns the {@link } instance associated to the given node
       * @see yfiles.tree.BalloonLayoutNodeInfo
       */
      getInfo(node:yfiles.algorithms.Node):yfiles.tree.BalloonLayoutNodeInfo;
      /**
       * Returns the preferred radial amount (wedge) in degrees that child nodes may in total occupy around the given node.
       * <p>
       * The wedge angle controls the degree to which the child nodes may radiate from the center of layout. A value close to <code>360</code>
       * means that the child nodes may radiate in (almost) any direction from their parent node, edge lengths can in consequence
       * stay rather small. On the other hand, a small value means that children are restricted to a small angle; thus, edges
       * lengths (and drawings) may become large.
       * </p>
       * <p>
       * This method returns the {@link yfiles.tree.BalloonLayout#preferredRootWedge preferred root wedge} if node <code>root</code> was selected as global root node ({@link yfiles.tree.BalloonLayout#rootNodePolicy}). Otherwise, it either
       * returns {@link yfiles.tree.BalloonLayout#preferredChildWedge} or if the given node has an outdegree equal to <code>2</code>, it returns the minimum of {@link yfiles.tree.BalloonLayout#preferredChildWedge} and <code>180</code>.
       * </p>
       * <p>
       * This method may be overridden to provide a custom child wedge function.
       * </p>
       * @param root the node to get the preferred wedge angle for
       * @returns the preferred wedge angle for <code>root</code> in degrees
       * @see yfiles.tree.BalloonLayout#preferredChildWedge
       * @see yfiles.tree.BalloonLayout#preferredRootWedge
       * @protected
       */
      getPreferredChildWedge(root:yfiles.algorithms.Node):number;
      /**
       * Sorts the child nodes (successors) of the given node.
       * <p>
       * This implementation uses the original node coordinates if {@link yfiles.tree.BalloonLayout#fromSketchMode From Sketch mode} is enabled. Otherwise it uses {@link yfiles.tree.BalloonLayout#comparer the specified comparator}
       * to sort the outgoing edges and thus the children of <code>root</code>. If there is no such comparator, then the sorting depends on
       * whether or not the child nodes are placed in an {@link yfiles.tree.BalloonLayout#interleavedMode interleaved} fashion:
       * </p>
       * <ul>
       * <li>Normal: Children are sorted according to the chosen {@link yfiles.tree.BalloonLayout#childOrderingPolicy child ordering policy}.</li>
       * <li>
       * Interleaved: Children are sorted such that the resulting interleaved node placement is compact, while children inducing
       * larger subgraphs are placed next to smaller ones.
       * </li>
       * </ul>
       * <p>
       * This method may be overridden to realize a custom child node ordering. It gets called in method {@link yfiles.tree.BalloonLayout#calculateChildArrangement}
       * before coordinates are assigned and just after the wedge sizes for all subtrees rooted at <code>root</code> are determined.
       * </p>
       * @param root the node whose child nodes will be sorted
       * @protected
       */
      sortChildNodes(root:yfiles.algorithms.Node):void;
      /**
       * Data provider key for marking nodes whose child nodes should be placed in an interleaved fashion
       * <p>
       * This {@link yfiles.algorithms.IDataProvider} allows to individually configure the interleaving feature for each node in the
       * graph.
       * </p>
       * @see yfiles.tree.BalloonLayout#interleavedMode
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static INTERLEAVED_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Data provider key for marking the node that will be used as root node of the tree
       * @see yfiles.tree.BalloonLayout#rootNodePolicy
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static SELECTED_ROOT_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * The layout graph being acted upon.
       * @protected
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * Gets or sets the {@link yfiles.collections.IComparer.<T>} instance that determines the order of the outgoing edges for each node of the tree.
       * <p>
       * If no {@link yfiles.collections.IComparer.<T>} is specified (i.e. the {@link yfiles.collections.IComparer.<T>} is <code>null</code>), the outgoing edges will be sorted according to the current
       * {@link yfiles.tree.BalloonLayout#childOrderingPolicy child ordering policy}.
       * </p>
       * @see yfiles.tree.BalloonLayout#childOrderingPolicy
       * @see yfiles.tree.BalloonLayout#fromSketchMode
       * @type {yfiles.collections.IComparer.<Object>}
       */
      comparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets the child ordering policy for sorting the child nodes around their parents.
       * <p>
       * The sorting policy can affect the compactness of drawings. Advantageous orderings allow adjacent subtrees to be close
       * together and can thus make the whole layout more compact.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown ordering policy is given
       * @see yfiles.tree.BalloonLayout#fromSketchMode
       * @see yfiles.tree.BalloonLayout#comparer
       * @type {yfiles.tree.ChildOrderingPolicy}
       */
      childOrderingPolicy:yfiles.tree.ChildOrderingPolicy;
      /**
       * Gets or sets the minimum distance to be kept between the nodes in the tree.
       * <p>
       * The distance needs to be a non-negative integer value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum distance is negative
       * @see yfiles.tree.BalloonLayout#allowOverlaps
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets whether or not to consider the given coordinates of the input diagram when arranging the tree.
       * <p>
       * Enabling this feature, the original circular order of child nodes around their parent nodes will be maintained according
       * to the initial coordinates when determining the order of the child nodes.
       * </p>
       * @type {boolean}
       */
      fromSketchMode:boolean;
      /**
       * Gets or sets the root node selection policy of this layout algorithm.
       * <p>
       * The policy determines which node is chosen as (virtual) tree root during the layout process.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown root node policy is given
       * @type {yfiles.tree.RootNodePolicy}
       */
      rootNodePolicy:yfiles.tree.RootNodePolicy;
      /**
       * Gets or sets the preferred radial amount (wedge) in degrees that child nodes may in total occupy around their parent
       * node.
       * <p>
       * The wedge angle controls the degree to which the child nodes may radiate from the center of layout. A value close to <code>360</code>
       * means that the child nodes may radiate in (almost) any direction from their parent node, edge lengths can in consequence
       * stay rather small. On the other hand, a small value means that children are restricted to a small angle; thus, edge
       * lengths (and drawings) may become large.
       * </p>
       * <p>
       * The minimum allowed wedge angle is <code>1</code> and the maximum allowed value is <code>359</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given angle is smaller than <code>1</code> or larger than <code>359</code>
       * @see yfiles.tree.BalloonLayout#preferredRootWedge
       * @type {number}
       */
      preferredChildWedge:number;
      /**
       * Gets or sets the preferred radial amount (wedge) in degrees that child nodes may in total occupy around the global
       * root.
       * <p>
       * This property allows to separately control the wedge angle for the designated root node of the tree, while {@link yfiles.tree.BalloonLayout#preferredChildWedge} controls the
       * angles of all child nodes. The root node will be determined depending on the {@link yfiles.tree.BalloonLayout#rootNodePolicy root node policy}.
       * </p>
       * <p>
       * The minimum allowed root wedge angle is <code>1</code> and the maximum allowed value is <code>360</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given angle is smaller than <code>1</code> or larger than <code>360</code>
       * @see yfiles.tree.BalloonLayout#preferredChildWedge
       * @type {number}
       */
      preferredRootWedge:number;
      /**
       * Gets or sets whether or not (partially) overlapping nodes are allowed.
       * <p>
       * If overlaps are allowed, the resulting layouts can become significantly more compact. Overlaps will mostly occur at the
       * borders of nodes. Nodes will not be totally covered by other nodes.
       * </p>
       * @type {boolean}
       */
      allowOverlaps:boolean;
      /**
       * Gets or sets the factor defining how compact layout results will potentially be, where a smaller factor produces
       * potentially more compact layouts.
       * <p>
       * The algorithm tries to optimize the child node arrangement around each tree node such that each subtree is as close to
       * its root as possible, while still fitting into the {@link yfiles.tree.BalloonLayout#preferredChildWedge preferred wedge angle} and not overlapping with adjacent subtrees.
       * </p>
       * <p>
       * High compactness factor values induce the optimization procedure to be less strict and accept less optimal results,
       * while low factor values mean that the optimization will only stop when being nearly optimal. Thus, lower values lead to
       * a potentially higher runtime.
       * </p>
       * <p>
       * The minimum factor value is <code>0.05</code> and the maximum is <code>1.0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the factor is smaller than <code>0.05</code> or greater than <code>1.0</code>
       * @type {number}
       */
      compactnessFactor:number;
      /**
       * Gets or sets the minimum length that this layout algorithm assigns to edges of the graph.
       * <p>
       * A lower minimum edge length allows generally more compact layouts. It has the highest effect if most nodes of the graph
       * have a low degree, as the minimum can potentially be met for such graphs.
       * </p>
       * <p>
       * The minimum length must be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given length is negative
       * @type {number}
       */
      minimumEdgeLength:number;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for node labels.
       * <p>
       * The size of nodes will temporarily be adjusted such that the label is included. Therefore, result drawings may get
       * significantly larger if this feature is enabled.
       * </p>
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets the mode for child node arrangement.
       * <p>
       * Child nodes are either placed interleaved or on a single layer around their parent node. Interleaved placement means
       * that child nodes are placed around their common parent in two different layers in an alternating fashion. For example,
       * the first child is on the inner layer, the second child on the outer layer, the third one again on the inner layer,
       * etc.
       * </p>
       * <p>
       * Independent of this mode, the alignment of child nodes on the same layer is still defined by the {@link yfiles.tree.BalloonLayout#childAlignmentPolicy alignment policy}.
       * However, {@link yfiles.tree.ChildAlignmentPolicy#SMART} is only supported for non-interleaved arrangement.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown mode for interleaved arrangement is given
       * @see yfiles.tree.BalloonLayout#childAlignmentPolicy
       * @type {yfiles.tree.InterleavedMode}
       */
      interleavedMode:yfiles.tree.InterleavedMode;
      /**
       * Gets or sets the child alignment policy for this layout algorithm.
       * <p>
       * This policy influences the distance of child nodes to their parent nodes and the alignment of children with the same
       * parent.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown policy is given
       * @type {yfiles.tree.ChildAlignmentPolicy}
       */
      childAlignmentPolicy:yfiles.tree.ChildAlignmentPolicy;
      /**
       * Gets or sets whether or not the layout algorithm automatically places node labels.
       * <p>
       * If enabled, this layout algorithm will calculate the positions for the node labels assuring that no overlaps occur.
       * </p>
       * <p>
       * Different labeling strategies may be selected using {@link yfiles.tree.BalloonLayout#nodeLabelingPolicy}.
       * </p>
       * @see yfiles.tree.BalloonLayout#nodeLabelingPolicy
       * @type {boolean}
       */
      integratedNodeLabeling:boolean;
      /**
       * Gets or sets whether or not the layout algorithm automatically places edge labels.
       * <p>
       * If enabled, this layout algorithm will calculate the positions for the edge labels assuring that no overlaps occur.
       * </p>
       * @type {boolean}
       */
      integratedEdgeLabeling:boolean;
      /**
       * Gets or sets the policy defining how node labels are placed by the integrated node labeling mechanism (for example, the
       * desired label orientation).
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown labeling policy is given
       * @see yfiles.tree.BalloonLayout#integratedNodeLabeling
       * @type {yfiles.tree.NodeLabelingPolicy}
       */
      nodeLabelingPolicy:yfiles.tree.NodeLabelingPolicy;
      /**
       * Gets or sets the distance between node labels belonging to the same node.
       * <p>
       * It also defines the distance between labels and the node they belong to in case of label placement outside of the node
       * (e.g. for {@link yfiles.tree.NodeLabelingPolicy#RAY_LIKE ray-like label placement}).
       * </p>
       * <p>
       * The spacing must have a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given spacing value is negative
       * @see yfiles.tree.BalloonLayout#integratedNodeLabeling
       * @type {number}
       */
      nodeLabelSpacing:number;
      /**
       * Gets or sets the distance between edge labels belonging to the same edge as well as the distance of the edge labels to
       * the target node of the edge.
       * <p>
       * The spacing must have a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given label spacing value is negative
       * @see yfiles.tree.BalloonLayout#integratedEdgeLabeling
       * @type {number}
       */
      edgeLabelSpacing:number;
      /**
       * Gets or sets whether or not chains are drawn straight or not.
       * <p>
       * A chain is defined as a tree node with exactly one child node. If this feature is enabled, then the incoming edge and
       * outgoing edge of the chain will have the same orientation, i.e., the whole chain looks straight.
       * </p>
       * <p>
       * Straightening all chains can lead to smoother, more symmetric results.
       * </p>
       * @type {boolean}
       */
      chainStraighteningMode:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for hiding group nodes is activated.
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @see yfiles.layout.MultiStageLayout#hideGroupsStage
       * @see yfiles.layout.HideGroupsStage
       * @type {boolean}
       */
      hideGroupsStageEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing parallel edges is activated.
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouter
       * @see yfiles.router.ParallelEdgeRouter
       * @type {boolean}
       */
      parallelEdgeRouterEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    export enum FillStyle{
      /**
       * Style specifier which defines that rows/columns are aligned with their leading side.
       * @see yfiles.tree.AspectRatioNodePlacer#fillStyle
       */
      LEADING,
      /**
       * Style specifier which defines that rows/columns are aligned with their center.
       * @see yfiles.tree.AspectRatioNodePlacer#fillStyle
       */
      CENTERED,
      /**
       * Style specifier which defines that rows/columns are justified to have the same width/height.
       * @see yfiles.tree.AspectRatioNodePlacer#fillStyle
       */
      JUSTIFY,
      /**
       * Style specifier which defines that rows/columns are aligned with their trailing side.
       * @see yfiles.tree.AspectRatioNodePlacer#fillStyle
       */
      TRAILING
    }
    export enum RootPlacement{
      /**
       * A root placement specifier for placing the root centered above its subtree, depending on the actual {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}.
       * @see yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
       */
      TOP,
      /**
       * A root placement specifier for placing the root in the upper left corner of the subtree bounds with respect to the
       * actual {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}.
       * <p>
       * Whether the root is placed entirely beside the subtree with no horizontal overlaps or entirely above the subtree without
       * vertical overlaps will be determined by the layout of the subtree. The layout algorithm tries to minimize the bounds of
       * the subtree.
       * </p>
       * @see yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
       * @see yfiles.tree.RootPlacement#CORNER_TOP
       * @see yfiles.tree.RootPlacement#CORNER_SIDE
       */
      CORNER,
      /**
       * A root placement specifier for placing the root in the upper left corner of the subtree bounds with respect to the
       * actual {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}.
       * <p>
       * The root is placed entirely beside the subtree with no horizontal overlaps. This might by important if the root's height
       * is very large while its width is small.
       * </p>
       * @see yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
       */
      CORNER_SIDE,
      /**
       * A root placement specifier for placing the root of a subtree in the upper left corner of the subtree bounds with respect
       * to the actual {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}.
       * <p>
       * The root is placed entirely above the subtree with no vertical overlaps. This might by important if the root's width is
       * very large while its height is small.
       * </p>
       * @see yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
       */
      CORNER_TOP
    }
    export enum SubtreeArrangement{
      /**
       * A direction specifier for placing the child nodes next to each other in direction of the actual
       * {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}, with the edges connecting in the direction of flow.
       * @see yfiles.tree.AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
       */
      HORIZONTAL,
      /**
       * A direction specifier for placing the child nodes above each other in direction of the actual
       * {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation}, with the edges connecting orthogonally to the direction of flow.
       * @see yfiles.tree.AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
       */
      VERTICAL
    }
    export enum InterleavedMode{
      /**
       * A child placement mode where all child nodes are placed around their parent with equal distances.
       * @see yfiles.tree.BalloonLayout#interleavedMode
       */
      OFF,
      /**
       * A child placement mode where nodes are placed in an interleaved fashion, that is, with two different, alternating
       * distances to the parent node.
       * @see yfiles.tree.BalloonLayout#interleavedMode
       */
      ALL_NODES,
      /**
       * A child placement mode where child nodes of selected nodes are placed in an interleaved fashion, that is, with two
       * different, alternating distances to the parent node.
       * <p>
       * All other nodes are placed without interleaving. The node selection can be specified by registering an appropriate {@link yfiles.algorithms.IDataProvider}
       * for key {@link yfiles.tree.BalloonLayout#INTERLEAVED_NODES_DP_KEY}.
       * </p>
       * @see yfiles.tree.BalloonLayout#interleavedMode
       */
      MARKED_NODES
    }
    export enum NodeLabelingPolicy{
      /**
       * Node labeling policy for ray-like label placement at nodes with zero or one child node.
       * <p>
       * Labels belonging to leaf nodes and nodes with exactly one successor (thus forming a sort of chain) will not be oriented
       * horizontal but ray-like; they get the same orientation as their nodes' incoming edge. The labels of the other nodes will
       * be oriented horizontally and placed at the center of the corresponding node.
       * </p>
       * @see yfiles.tree.BalloonLayout#nodeLabelingPolicy
       */
      RAY_LIKE,
      /**
       * Node labeling policy for ray-like label placement at leaf nodes.
       * <p>
       * Labels corresponding to leaf nodes get the same orientation as their nodes' incoming edge and are placed outside the
       * node (without overlaps). The labels of the other nodes will be oriented horizontally and placed at the center of the
       * corresponding node.
       * </p>
       * @see yfiles.tree.BalloonLayout#nodeLabelingPolicy
       */
      RAY_LIKE_LEAVES,
      /**
       * Node labeling policy for horizontal label placement at all nodes.
       * <p>
       * The node labels are placed at the center of the corresponding node.
       * </p>
       * @see yfiles.tree.BalloonLayout#nodeLabelingPolicy
       */
      HORIZONTAL
    }
    export enum ChildAlignmentPolicy{
      /**
       * Alignment policy to align child nodes rooted at the same parent such that the border of their convex hull has the same
       * distance to the parent node's center.
       * @see yfiles.tree.BalloonLayout#childAlignmentPolicy
       */
      PLAIN,
      /**
       * Alignment policy to align child nodes rooted at the same parent such that each child has the same center-to-center
       * distance to the parent node.
       * <p>
       * All nodes will be placed on a common radius around their parent, respective to their center coordinates.
       * </p>
       * @see yfiles.tree.BalloonLayout#childAlignmentPolicy
       */
      SAME_CENTER,
      /**
       * Alignment policy for aligning child nodes such that the distances to their parent are kept short and drawings can
       * potentially become more compact.
       * <p>
       * This policy realizes the exact same child alignment as {@link yfiles.tree.ChildAlignmentPolicy#PLAIN} if there are no edge
       * labels associated with edges going to child nodes or if {@link yfiles.tree.BalloonLayout#integratedEdgeLabeling integrated edge labeling} is disabled.
       * </p>
       * <p>
       * If edge labels need to be considered, edge labels will be added to the convex hull of the child node to which the edge
       * connects. This means that child nodes connected with a labeled edge will be aligned at the border where the edge label
       * begins. In consequence, edge labels will be aligned with neighboring edge labels as well as neighboring child nodes if a
       * child node has no edge label at its incoming edge.
       * </p>
       * <p>
       * This alignment strategy is especially effective if large edge labels need to be considered while drawings should still
       * be compact and distances from parent to child nodes short.
       * </p>
       * @see yfiles.tree.BalloonLayout#childAlignmentPolicy
       */
      COMPACT,
      /**
       * Alignment policy to align child nodes rooted at the same parent using a mixture of the other policies and aiming to
       * achieve symmetry and compactness at the same time.
       * <p>
       * Similar to
       * {@link yfiles.tree.ChildAlignmentPolicy#SAME_CENTER}, this policy aims to achieve alignments featuring equal distances between
       * parent and child node centers. However, a much larger distance for all child nodes caused by single large subtrees is
       * avoided by this policy. In such cases, the smaller subtrees may be placed on a common radius and the large subtrees are
       * placed with different distances.
       * </p>
       * @see yfiles.tree.BalloonLayout#childAlignmentPolicy
       */
      SMART
    }
    export enum ChildOrderingPolicy{
      /**
       * Child ordering policy which orders child nodes depending on their subtree size such that large subtrees are placed next
       * to small ones in order to save space.
       * <p>
       * The actual area of a subtree is used as the subtree size. The smallest subtree is placed next to the largest subtree,
       * followed by the second smallest subtree and so on. This way, drawings can become more compact, because the adjacent
       * placement of large subtrees is avoided (which would require edges to become longer for subtrees to fit next to each
       * other).
       * </p>
       * @see yfiles.tree.BalloonLayout#childOrderingPolicy
       */
      COMPACT,
      /**
       * Child ordering policy which sorts the child nodes according to their wedge angles.
       * <p>
       * The node with the greatest angle gets median position in the sorting order. The smaller the wedges, the farther away a
       * node will be placed from the median position. Using this policy, the produced results are often more symmetric.
       * </p>
       * @see yfiles.tree.BalloonLayout#childOrderingPolicy
       */
      SYMMETRIC
    }
    export enum RootNodePolicy{
      /**
       * Root node policy for choosing a node with indegree <code>0</code> as root node of the tree.
       * <p>
       * The selection of the root node will happen according to {@link yfiles.algorithms.Trees#getRoot}.
       * </p>
       * @see yfiles.tree.BalloonLayout#rootNodePolicy
       * @see yfiles.algorithms.Trees#getRoot
       */
      DIRECTED_ROOT,
      /**
       * Root node policy for choosing the center node as root node of the tree.
       * <p>
       * A {@link yfiles.algorithms.Trees#getCenterRoot center node} induces a minimum depth tree when being used as the root of that tree.
       * </p>
       * @see yfiles.tree.BalloonLayout#rootNodePolicy
       * @see yfiles.algorithms.Trees#getCenterRoot
       */
      CENTER_ROOT,
      /**
       * Root node policy for choosing a weighted center node as root node of the tree.
       * <p>
       * A {@link yfiles.algorithms.Trees#getWeightedCenterNode weighted center node} is a node which is part of the greatest number of all undirected paths in a graph.
       * </p>
       * @see yfiles.tree.BalloonLayout#rootNodePolicy
       */
      WEIGHTED_CENTER_ROOT,
      /**
       * Root node policy for choosing a custom node as the root node of the tree.
       * <p>
       * The root node is specified using a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.tree.BalloonLayout#SELECTED_ROOT_DP_KEY}.
       * </p>
       * @see yfiles.tree.BalloonLayout#rootNodePolicy
       */
      SELECTED_ROOT
    }
    export enum ChildPlacement{
      /**
       * Placement specifier which defines that subtrees are placed one below the other and left of the root node.
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       */
      VERTICAL_TO_LEFT,
      /**
       * Placement specifier which defines that subtrees are placed one below the other and right of the root node.
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       */
      VERTICAL_TO_RIGHT,
      /**
       * Placement specifier which defines that subtrees are placed horizontally next to each other and above the root node.
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       */
      HORIZONTAL_UPWARD,
      /**
       * Placement specifier which defines that subtrees are placed horizontally next to each other and below the root node.
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       */
      HORIZONTAL_DOWNWARD
    }
    export enum RootAlignment{
      /**
       * Alignment specifier which defines that the root node is placed ahead of all its child subtrees.
       * <p>
       * Depending on the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, the root node is placed ahead (e.g.
       * {@link yfiles.tree.ChildPlacement#HORIZONTAL_DOWNWARD}: left) of the combined bounding box of all subtrees without labels.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      LEADING_OFFSET,
      /**
       * Alignment specifier which defines that the root node is placed aligned with its first subtree.
       * <p>
       * Depending on the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, the root node is placed aligned with the bounding box of its first (e.g.
       * {@link yfiles.tree.ChildPlacement#HORIZONTAL_DOWNWARD}: left) subtree.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      LEADING,
      /**
       * Alignment specifier which defines that the root node is placed aligned with the center of its subtrees.
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      CENTER,
      /**
       * Alignment specifier which defines that the root node is placed at the median of the connection points to its subtrees.
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      MEDIAN,
      /**
       * Alignment specifier which defines that the root node is placed aligned with its last subtree.
       * <p>
       * Depending on the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, the root node is placed aligned with the bounding box of its last (e.g.
       * {@link yfiles.tree.ChildPlacement#HORIZONTAL_DOWNWARD}: right) subtree.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      TRAILING,
      /**
       * Alignment specifier which defines that the root is placed after all its subtree.
       * <p>
       * Depending on the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, the root node is placed after (e.g.
       * {@link yfiles.tree.ChildPlacement#HORIZONTAL_DOWNWARD}: right) of the combined bounding box of all subtrees without labels.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      TRAILING_OFFSET,
      /**
       * Alignment specifier which defines that the root is placed after all its subtrees, centered on the bus.
       * <p>
       * Depending on the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, the root node is placed after (e.g.
       * {@link yfiles.tree.ChildPlacement#HORIZONTAL_DOWNWARD}: right) of the combined bounding box of all subtrees without labels. The
       * bus leaves directly from the root node.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      TRAILING_ON_BUS,
      /**
       * Alignment specifier which defines that the root is placed ahead of all its subtrees, centered on the bus.
       * <p>
       * Depending on the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, the root node is placed ahead (e.g.
       * {@link yfiles.tree.ChildPlacement#HORIZONTAL_DOWNWARD}: left) of the combined bounding box of all subtrees without labels. The
       * bus leaves directly from the root node.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       */
      LEADING_ON_BUS
    }
    export enum RoutingStyle{
      /**
       * Routing style specifier which defines that edge paths are routed orthogonally with the bends located in the channel
       * between the root node and the child nodes.
       * @see yfiles.tree.DefaultNodePlacer#routingStyle
       */
      FORK,
      /**
       * Routing style specifier which defines that edge paths are routed orthogonally with just one bend.
       * <p>
       * Edges will leave the root node at its sides and bend exactly above the according child node. If the child node is placed
       * directly below the root node, the edge will bend inside of the root node and leave it at the bottom.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#routingStyle
       */
      FORK_AT_ROOT,
      /**
       * Routing style specifier which defines that edge paths are routed with a straight segment to the center of the child
       * nodes.
       * @see yfiles.tree.DefaultNodePlacer#routingStyle
       */
      STRAIGHT,
      /**
       * Routing style constant which defines that edge paths are routed with a straight segment to the connector of the {@link yfiles.tree.SubtreeShape}
       * of the child nodes.
       * @see yfiles.tree.DefaultNodePlacer#routingStyle
       */
      POLYLINE
    }
    export enum PortAssignmentMode{
      /**
       * A port assignment specifier that defines that
       * {@link yfiles.layout.PortConstraint}s are considered. Ports are placed using the following rules:
       * <ul>
       * <li>edges without a {@link yfiles.layout.PortConstraint} are anchored at the center of the node</li>
       * <li>edges with a weak {@link yfiles.layout.PortConstraint} are anchored at the center of their specified side</li>
       * <li>edges with a strong {@link yfiles.layout.PortConstraint} are anchored at their initial coordinates</li>
       * </ul>
       * <p>
       * If edges with
       * {@link yfiles.layout.PortConstraint}s are grouped, all edges in the same group will use the port coordinates of the first edge
       * in the group.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      PORT_CONSTRAINT,
      /**
       * A port assignment specifier which defines that edges are distributed evenly at the northern side of their nodes.
       * <p>
       * Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
       * distribution.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      DISTRIBUTED_NORTH,
      /**
       * A port assignment specifier which defines that edges are distributed evenly at the southern side of their nodes.
       * <p>
       * Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
       * distribution.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      DISTRIBUTED_SOUTH,
      /**
       * A port assignment specifier which defines that edges are distributed evenly at the eastern side of their nodes.
       * <p>
       * Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
       * distribution.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      DISTRIBUTED_EAST,
      /**
       * A port assignment specifier which defines that edges are distributed evenly at the western side of their nodes.
       * <p>
       * Grouped edges will use the same port coordinate and will be considered as a single edge when calculating the
       * distribution.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      DISTRIBUTED_WEST,
      /**
       * A port assignment specifier which defines that all ports are reset to the center of their nodes.
       */
      NONE
    }
    export enum ConnectorDirection{
      /**
       * A direction constant that describes that the last connector segment points up. The edge from the parent will connect to
       * the north of the {@link yfiles.tree.SubtreeShape}.
       * @see yfiles.tree.SubtreeShape#connectorDirection
       */
      NORTH,
      /**
       * A direction constant that describes that the last connector segment points to the right. The edge from the parent will
       * connect to the east of the {@link yfiles.tree.SubtreeShape}.
       * @see yfiles.tree.SubtreeShape#connectorDirection
       */
      EAST,
      /**
       * A direction constant that describes that the last connector segment points down. The edge from the parent will connect
       * to the south of the {@link yfiles.tree.SubtreeShape}.
       * @see yfiles.tree.SubtreeShape#connectorDirection
       */
      SOUTH,
      /**
       * A direction constant that describes that the last connector segment points to the left. The edge from the parent will
       * connect to the west of the {@link yfiles.tree.SubtreeShape}.
       * @see yfiles.tree.SubtreeShape#connectorDirection
       */
      WEST
    }
    export enum BorderLineSide{
      /**
       * Side constant that defines the index of the northern border line.
       * @see yfiles.tree.SubtreeShape#getBorderLine
       */
      NORTH,
      /**
       * Side constant that defines the index of the eastern border line.
       * @see yfiles.tree.SubtreeShape#getBorderLine
       */
      EAST,
      /**
       * Side constant that defines the index of the southern border line.
       * @see yfiles.tree.SubtreeShape#getBorderLine
       */
      SOUTH,
      /**
       * Side constant that defines the index of the western border line.
       * @see yfiles.tree.SubtreeShape#getBorderLine
       */
      WEST
    }
    export enum LayeredRoutingStyle{
      /**
       * Routing style specifier which defines that edges consist of one straight segment.
       * @see yfiles.tree.LayeredNodePlacer#routingStyle
       */
      STRAIGHT,
      /**
       * Routing style specifier which defines that edges are routed in an orthogonal, bus-like fashion.
       * @see yfiles.tree.LayeredNodePlacer#routingStyle
       */
      ORTHOGONAL
    }
    export enum MultiParentRoutingStyle{
      /**
       * Routing style for routing non-shared edge segments in a polyline fashion.
       * <p>
       * The edges connected to a multi-parent structure will share their paths until shortly above/below the multi-parent nodes.
       * This style only applies to the part of the edges where they split to connect to different multi-parents.
       * </p>
       * @see yfiles.tree.MultiParentDescriptor#edgeStyle
       */
      POLYLINE,
      /**
       * Routing style for routing non-shared edge segments in an orthogonal fashion.
       * <p>
       * The edges connected to a multi-parent structure will share their paths until shortly above/below the multi-parent nodes.
       * This style only applies to the part of the edges where they split to connect to different multi-parents.
       * </p>
       * @see yfiles.tree.MultiParentDescriptor#edgeStyle
       */
      ORTHOGONAL,
      /**
       * Routing style for routing non-shared edge segments straight. This style will route the part of the edges that doesn't
       * share the same path directly from the center of the multi-parent to the common point of the edges in the multi-parent
       * structure.
       * <p>
       * The edges connected to a multi-parent structure will share their paths until shortly above/below the multi-parent nodes.
       * This style only applies to the part of the edges where they split to connect to different multi-parents.
       * </p>
       * @see yfiles.tree.MultiParentDescriptor#edgeStyle
       */
      STRAIGHT
    }
    export enum ParentConnectorDirection{
      /**
       * Direction specifier which indicates that no connector should be calculated.
       * @see yfiles.tree.INodePlacer#placeSubtree
       */
      NONE,
      /**
       * Direction specifier which indicates that any direction can be used for the connector to the parent node.
       * @see yfiles.tree.INodePlacer#placeSubtree
       */
      ANY,
      /**
       * Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
       * by a segment that goes north.
       * @see yfiles.tree.INodePlacer#placeSubtree
       */
      NORTH,
      /**
       * Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
       * by a segment that goes east.
       * @see yfiles.tree.INodePlacer#placeSubtree
       */
      EAST,
      /**
       * Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
       * by a segment that goes south.
       * @see yfiles.tree.INodePlacer#placeSubtree
       */
      SOUTH,
      /**
       * Direction specifier which indicates that the connector to the parent node should end in a segment that can be extended
       * by a segment that goes west.
       * @see yfiles.tree.INodePlacer#placeSubtree
       */
      WEST
    }
    export enum PortStyle{
      /**
       * A constant defining that ports are placed at the center of the nodes.
       * <p>
       * This constant uses <code>(0,0)</code> as port offset.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#portStyle
       */
      NODE_CENTER,
      /**
       * A constant defining that ports are placed at the center of the border of the nodes.
       * @see yfiles.tree.ClassicTreeLayout#portStyle
       */
      BORDER_CENTER,
      /**
       * A constant defining that ports are evenly distributed along the border of the nodes.
       * @see yfiles.tree.ClassicTreeLayout#portStyle
       */
      BORDER_DISTRIBUTED,
      /**
       * A constant defining that ports are assigned based on given {@link yfiles.layout.PortConstraint port constraints}.
       * @see yfiles.tree.ClassicTreeLayout#portStyle
       */
      PORT_CONSTRAINTS_AWARE
    }
    export enum EdgeRoutingStyle{
      /**
       * A constant for routing the edges as straight-line segments.
       * @see yfiles.tree.ClassicTreeLayout#edgeRoutingStyle
       */
      PLAIN,
      /**
       * A constant for routing the edges orthogonally in a bus-like fashion.
       * @see yfiles.tree.ClassicTreeLayout#edgeRoutingStyle
       */
      ORTHOGONAL
    }
    export enum LeafPlacement{
      /**
       * A policy for placing the leaf nodes in a stack-like fashion with balanced stack heights.
       * <p>
       * In this context, a stack-like fashion means that leaf nodes that connect to the same parent node are placed one upon the
       * other, resulting in horizontally compact layouts.
       * </p>
       * <p>
       * This policy tries to balance stack heights, i.e., for each subtree that consists only of leaf nodes, {@link yfiles.tree.LeafPlacement#LEAVES_STACKED_RIGHT}
       * or {@link yfiles.tree.LeafPlacement#LEAVES_STACKED_LEFT_AND_RIGHT} will be used depending on the number of leaves in the
       * subtree.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       */
      LEAVES_STACKED,
      /**
       * A policy for placing the leaf nodes in a stack-like fashion using a single stack left.
       * <p>
       * In this context, a stack-like fashion means that leaf nodes that connect to the same root node are placed one upon the
       * other, resulting in horizontally compact layouts.
       * </p>
       * <p>
       * This policy uses a single stack that lies to the left relative to the center of the parent node.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       */
      LEAVES_STACKED_LEFT,
      /**
       * A policy for placing the leaf nodes in a stack-like fashion using a single stack right.
       * <p>
       * In this context, a stack-like fashion means that leaf nodes that connect to the same root node are placed one upon the
       * other, resulting in horizontally compact layouts.
       * </p>
       * <p>
       * This policy uses a single stack that lies to the right relative to the center of the parent node.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       */
      LEAVES_STACKED_RIGHT,
      /**
       * A policy for placing the leaf nodes in a stack-like fashion using two stacks.
       * <p>
       * In this context, a stack-like fashion means that leaf nodes that connect to the same root node are placed one upon the
       * other, resulting in horizontally compact layouts.
       * </p>
       * <p>
       * This policy distributes leaf nodes among two stacks, one to the left and one to the right relative to the center of the
       * parent node.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       */
      LEAVES_STACKED_LEFT_AND_RIGHT,
      /**
       * A policy for placing the leaf nodes in a Dendrogram-like fashion.
       * <p>
       * According to this policy, all leaf nodes are placed in one layer (i.e. all leaves are placed on one horizontal line in a
       * top-to-bottom or in a bottom-to-top layout).
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       */
      ALL_LEAVES_ON_SAME_LAYER,
      /**
       * A policy for placing the leaf nodes with the same parent in the same layer.
       * <p>
       * For example, siblings are being placed on a horizontal line in a top-to-bottom or in a bottom-to-top layout.
       * </p>
       * <p>
       * This policy produces vertically compact layouts.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       */
      SIBLINGS_ON_SAME_LAYER
    }
    /**
     * This {@link yfiles.tree.INodePlacer} places the subtrees such that the overall aspect ratio of the arranged subtree will be
     * close to a preferred aspect ratio.
     * <p>
     * <b>Layout Style</b> The aspect ratio describes the relation between width and height of the subtree. To achieve the desired aspect ratio,
     * this {@link yfiles.tree.INodePlacer} divides the child {@link yfiles.tree.SubtreeShape}s into as much rows as needed. The edges are
     * either routed along the rows or along the columns. {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,0],[70,70],[140,70],[210,70],[280,70],[70,140],[140,140],[210,140],[280,140],[70,210],[140,210],[210,210],[280,210],[70,280],[140,280],[210,280],[280,280]],"e":[[0,1,[50,15,50,85]],[0,2,[120,15,120,85]],[0,3,[190,15,190,85]],[0,4,[260,15,260,85]],[0,5,[50,15,50,155]],[0,6,[120,15,120,155]],[0,7,[190,15,190,155]],[0,8,[260,15,260,155]],[0,9,[50,15,50,225]],[0,10,[120,15,120,225]],[0,11,[190,15,190,225]],[0,12,[260,15,260,225]],[0,13,[50,15,50,295]],[0,14,[120,15,120,295]],[0,15,[190,15,190,295]],[0,16,[260,15,260,295]]],"vp":[0.0,0.0,310.0,310.0]}}
     * </p>
     * <p>
     * Aspect ratio <code>1</code>
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,0],[70,70],[140,70],[210,70],[70,140],[140,140],[210,140],[70,210],[140,210],[210,210],[70,280],[140,280],[210,280],[70,350],[140,350],[210,350],[70,420]],"e":[[0,1,[50,15,50,85]],[0,2,[120,15,120,85]],[0,3,[190,15,190,85]],[0,4,[50,15,50,155]],[0,5,[120,15,120,155]],[0,6,[190,15,190,155]],[0,7,[50,15,50,225]],[0,8,[120,15,120,225]],[0,9,[190,15,190,225]],[0,10,[50,15,50,295]],[0,11,[120,15,120,295]],[0,12,[190,15,190,295]],[0,13,[50,15,50,365]],[0,14,[120,15,120,365]],[0,15,[190,15,190,365]],[0,16,[50,15,50,435]]],"vp":[0.0,0.0,240.0,450.0]}}
     * </p>
     * <p>
     * Aspect ratio <code>0.5</code>
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,0],[70,70],[140,70],[210,70],[280,70],[350,70],[420,70],[490,70],[560,70],[70,140],[140,140],[210,140],[280,140],[350,140],[420,140],[490,140],[560,140]],"e":[[0,1,[50,15,50,85]],[0,2,[120,15,120,85]],[0,3,[190,15,190,85]],[0,4,[260,15,260,85]],[0,5,[330,15,330,85]],[0,6,[400,15,400,85]],[0,7,[470,15,470,85]],[0,8,[540,15,540,85]],[0,9,[50,15,50,155]],[0,10,[120,15,120,155]],[0,11,[190,15,190,155]],[0,12,[260,15,260,155]],[0,13,[330,15,330,155]],[0,14,[400,15,400,155]],[0,15,[470,15,470,155]],[0,16,[540,15,540,155]]],"vp":[0.0,0.0,590.0,170.0]}}
     * </p>
     * <p>
     * Aspect ratio <code>2</code>
     * </p>
     * @class
     * @extends {yfiles.tree.NodePlacerBase}
     * @implements {yfiles.tree.IFromSketchNodePlacer}
     */
    export interface AspectRatioNodePlacer extends yfiles.tree.NodePlacerBase,yfiles.tree.IFromSketchNodePlacer{}
    export class AspectRatioNodePlacer {
      /**
       * Creates a new instance of {@link yfiles.tree.AspectRatioNodePlacer} with default settings.
       */
      constructor();
      /**
       * Creates an {@link yfiles.collections.IComparer.<T> edge comparator} which takes the initial coordinates of the nodes into account.
       * <p>
       * This {@link yfiles.collections.IComparer.<T>} defines the order of the outgoing edges of a node before the actual placement of the node's subtree is calculated.
       * It also considers whether or not the rows are placed {@link yfiles.tree.AspectRatioNodePlacer#horizontal horiontally}.
       * </p>
       * @returns the {@link } for the edges
       */
      createFromSketchComparer():yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets whether child nodes are distributed in horizontal rows or vertical columns.
       * @type {boolean}
       */
      horizontal:boolean;
      /**
       * Gets or sets the vertical distance between the {@link yfiles.tree.SubtreeShape}s.
       * <p>
       * The nodes will be placed below each other, keeping this distance.
       * </p>
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      verticalDistance:number;
      /**
       * Gets or sets the horizontal distance between the {@link yfiles.tree.SubtreeShape}s.
       * <p>
       * The nodes will be placed next to each other, keeping this distance.
       * </p>
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      horizontalDistance:number;
      /**
       * Gets or sets the preferred aspect ratio (width/height) for the subtrees.
       * <p>
       * This ratio determines among how many rows/columns the nodes will be distributed.
       * </p>
       * <p>
       * The aspect ratio needs to be positive.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified aspect ratio is <code>0</code> or negative
       * @type {number}
       */
      aspectRatio:number;
      /**
       * Gets or sets how rows/columns are aligned with each other.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown fill style is specified
       * @type {yfiles.tree.FillStyle}
       */
      fillStyle:yfiles.tree.FillStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.AspectRatioTreeLayout} arranges tree graphs in a compact fashion.
     * <p>
     * <b>Layout Style</b> This layout algorithm tries to generate compact tree layouts with a certain preferred aspect ratio. {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,0],[40,740],[360,740],[480,810],[400,810],[240,810],[80,810],[160,810],[560,810],[200,880],[200,940],[200,1000],[200,1060],[200,1120],[120,880],[120,940],[120,1000],[120,1060],[120,1120],[280,880],[280,940],[320,880],[320,940],[600,880],[600,940],[600,1000],[520,880],[520,940],[520,1000],[440,880],[440,940],[440,1000],[440,1060],[440,1120],[40,70],[390,70],[80,450],[80,640],[200,140],[200,450],[80,140],[150,680],[190,680],[240,210],[240,270],[240,330],[120,520],[120,580],[160,520],[160,580],[270,490],[270,550],[310,490],[310,550],[350,490],[350,550],[120,210],[120,270],[120,330],[120,390],[160,210],[160,270],[160,330],[430,610],[580,510],[430,510],[430,140],[510,140],[590,140],[470,610],[650,550],[500,550],[540,550],[630,210],[630,270],[630,330],[470,210],[470,270],[470,330],[470,390],[470,450],[550,210],[550,270],[550,330],[550,390],[550,450]],"e":[[0,1,-10,0,0,0,[5,50,5,720,55,720]],[0,2,-10,0,0,0,[5,50,5,720,375,720]],[2,3,-10,0,0,0,[365,790,495,790]],[2,4,-10,0,0,0,[365,790,415,790]],[1,5,-10,0,0,0,[45,790,255,790]],[1,6,-10,0,0,0,[45,790,95,790]],[1,7,-10,0,0,0,[45,790,175,790]],[2,8,-10,0,0,0,[365,790,575,790]],[7,9,-10,0,0,0,[165,860,215,860]],[7,10,-10,0,0,0,[165,860,165,920,215,920]],[7,11,-10,0,0,0,[165,860,165,980,215,980]],[7,12,-10,0,0,0,[165,860,165,1040,215,1040]],[7,13,-10,0,0,0,[165,860,165,1100,215,1100]],[6,14,-10,0,0,0,[85,860,135,860]],[6,15,-10,0,0,0,[85,860,85,920,135,920]],[6,16,-10,0,0,0,[85,860,85,980,135,980]],[6,17,-10,0,0,0,[85,860,85,1040,135,1040]],[6,18,-10,0,0,0,[85,860,85,1100,135,1100]],[5,19,-10,0,0,0,[245,860,295,860]],[5,20,-10,0,0,0,[245,860,245,920,295,920]],[5,21,-10,0,0,0,[245,860,335,860]],[5,22,-10,0,0,0,[245,860,245,920,335,920]],[8,23,-10,0,0,0,[565,860,615,860]],[8,24,-10,0,0,0,[565,860,565,920,615,920]],[8,25,-10,0,0,0,[565,860,565,980,615,980]],[3,26,-10,0,0,0,[485,860,535,860]],[3,27,-10,0,0,0,[485,860,485,920,535,920]],[3,28,-10,0,0,0,[485,860,485,980,535,980]],[4,29,-10,0,0,0,[405,860,455,860]],[4,30,-10,0,0,0,[405,860,405,920,455,920]],[4,31,-10,0,0,0,[405,860,405,980,455,980]],[4,32,-10,0,0,0,[405,860,405,1040,455,1040]],[4,33,-10,0,0,0,[405,860,405,1100,455,1100]],[0,34,-10,0,0,0,[5,50,55,50]],[0,35,-10,0,0,0,[5,50,405,50]],[34,36,-10,0,0,0,[45,120,45,430,95,430]],[34,37,-10,0,0,0,[45,120,45,620,95,620]],[34,38,-10,0,0,0,[45,120,215,120]],[34,39,-10,0,0,0,[45,120,45,430,215,430]],[34,40,-10,0,0,0,[45,120,95,120]],[37,41,0,-10,0,0,[130,645,130,660,165,660]],[37,42,0,-10,0,0,[130,645,130,660,205,660]],[38,43,-10,0,0,0,[205,190,255,190]],[38,44,-10,0,0,0,[205,190,205,250,255,250]],[38,45,-10,0,0,0,[205,190,205,310,255,310]],[36,46,-10,0,0,0,[85,500,135,500]],[36,47,-10,0,0,0,[85,500,85,560,135,560]],[36,48,-10,0,0,0,[85,500,175,500]],[36,49,-10,0,0,0,[85,500,85,560,175,560]],[39,50,0,-10,0,0,[250,455,250,470,285,470]],[39,51,0,-10,0,0,[250,455,250,530,285,530]],[39,52,0,-10,0,0,[250,455,250,470,325,470]],[39,53,0,-10,0,0,[250,455,250,530,325,530]],[39,54,0,-10,0,0,[250,455,250,470,365,470]],[39,55,0,-10,0,0,[250,455,250,530,365,530]],[40,56,-10,0,0,0,[85,190,135,190]],[40,57,-10,0,0,0,[85,190,85,250,135,250]],[40,58,-10,0,0,0,[85,190,85,310,135,310]],[40,59,-10,0,0,0,[85,190,85,370,135,370]],[40,60,-10,0,0,0,[85,190,175,190]],[40,61,-10,0,0,0,[85,190,85,250,175,250]],[40,62,-10,0,0,0,[85,190,85,310,175,310]],[35,63,-10,0,0,0,[395,120,395,590,445,590]],[35,64,-10,0,0,0,[395,120,395,490,595,490]],[35,65,-10,0,0,0,[395,120,395,490,445,490]],[35,66,-10,0,0,0,[395,120,445,120]],[35,67,-10,0,0,0,[395,120,525,120]],[35,68,-10,0,0,0,[395,120,605,120]],[35,69,-10,0,0,0,[395,120,395,590,485,590]],[64,70,0,-10,0,0,[630,515,630,530,665,530]],[65,71,0,-10,0,0,[480,515,480,530,515,530]],[65,72,0,-10,0,0,[480,515,480,530,555,530]],[68,73,-10,0,0,0,[595,190,645,190]],[68,74,-10,0,0,0,[595,190,595,250,645,250]],[68,75,-10,0,0,0,[595,190,595,310,645,310]],[66,76,-10,0,0,0,[435,190,485,190]],[66,77,-10,0,0,0,[435,190,435,250,485,250]],[66,78,-10,0,0,0,[435,190,435,310,485,310]],[66,79,-10,0,0,0,[435,190,435,370,485,370]],[66,80,-10,0,0,0,[435,190,435,430,485,430]],[67,81,-10,0,0,0,[515,190,565,190]],[67,82,-10,0,0,0,[515,190,515,250,565,250]],[67,83,-10,0,0,0,[515,190,515,310,565,310]],[67,84,-10,0,0,0,[515,190,515,370,565,370]],[67,85,-10,0,0,0,[515,190,515,430,565,430]]],"vp":[0.0,0.0,680.0,1150.0]}}
     * </p>
     * <p>
     * Example layout with an aspect ratio of <code>0.5</code>
     * </p>
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * The layout algorithm starts from the root and recursively assigns coordinates to all tree nodes. In this manner, leaf
     * nodes will be placed first, while each parent node is placed centered above its child nodes.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The aspect ratio of each subtree can be specified individually. A {@link yfiles.algorithms.IDataProvider} registered with {@link yfiles.tree.AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY}
     * returns the aspect ratio for each local root node.
     * </p>
     * <p>
     * A custom node can be defined as root of the tree using a {@link yfiles.algorithms.IDataProvider} registered with the graph with
     * key {@link yfiles.tree.AspectRatioTreeLayout#SELECTED_ROOT_DP_KEY}.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface AspectRatioTreeLayout extends yfiles.layout.MultiStageLayout{}
    export class AspectRatioTreeLayout {
      /**
       * Creates a new {@link yfiles.tree.AspectRatioTreeLayout} instance with default settings.
       */
      constructor();
      /**
       * Adds bends to the given {@link yfiles.layout.IEdgeLayout}.
       * <p>
       * This method is called by {@link yfiles.tree.AspectRatioTreeLayout#applyLayoutCore} for each edge, after the nodes are placed, to
       * route the path of the edges. It may be overridden to apply a custom routing style.
       * </p>
       * @param edgeLayout the layout of the edge that is routed
       * @param localRoot the local root of the subtree
       * @param child the child connected to the local root with the given edge
       * @param rootPlacement the placement specifier of the local root
       * @param routingPolicy the direction specifier for the routing
       * @protected
       */
      createBends(edgeLayout:yfiles.layout.IEdgeLayout,localRoot:yfiles.algorithms.Node,child:yfiles.algorithms.Node,rootPlacement:yfiles.tree.RootPlacement,routingPolicy:yfiles.tree.SubtreeArrangement):void;
      /**
       * Retrieves the aspect ratio for the subtree that is rooted at the given node.
       * <p>
       * If there is an individual aspect ratio for the subtree provided by a {@link yfiles.algorithms.IDataProvider} registered with key
       * {@link yfiles.tree.AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY}, that ratio will be returned. Otherwise, the {@link yfiles.tree.AspectRatioTreeLayout#aspectRatio default aspect ratio}
       * will be returned.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.AspectRatioTreeLayout#applyLayoutCore} before a subtree is arranged. It may be
       * overridden to use another approach to define the aspect ratio for subtrees.
       * </p>
       * @param localRoot the root node of the subtree
       * @returns the aspect ratio of the subtree rooted at the local root node
       * @see yfiles.tree.AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY
       * @see yfiles.tree.AspectRatioTreeLayout#aspectRatio
       * @protected
       */
      getAspectRatio(localRoot:yfiles.algorithms.Node):number;
      /**
       * Retrieves the root placement for the subtree that is rooted at the given node.
       * <p>
       * Either an individual root placement for the subtree (defined via a {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY}), or if there is none, the {@link yfiles.tree.AspectRatioTreeLayout#rootPlacement default root placement} is returned.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.AspectRatioTreeLayout#applyLayoutCore} after a subtree is arranged to align the
       * local root node. It may be overridden to use another approach to define the root placement for a subtree.
       * </p>
       * @param localRoot the root of the subtree
       * @returns the root placement of the subtree
       * @see yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
       * @see yfiles.tree.AspectRatioTreeLayout#rootPlacement
       * @protected
       */
      getRootPlacement(localRoot:Object):yfiles.tree.RootPlacement;
      /**
       * Retrieves the routing for the subtree that is rooted at the given node.
       * <p>
       * Either an individual routing for the subtree (defined via a {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.tree.AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY}), or if there is none, the {@link yfiles.tree.AspectRatioTreeLayout#subtreeArrangement default routing} is returned.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.AspectRatioTreeLayout#applyLayoutCore} to determine the direction of the subtrees.
       * It may be overridden to use another approach to define the routing for a subtree.
       * </p>
       * @param localRoot the root of the subtree
       * @returns the routing policy for the subtree
       * @see yfiles.tree.AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
       * @see yfiles.tree.AspectRatioTreeLayout#subtreeArrangement
       * @protected
       */
      getSubtreeArrangement(localRoot:Object):yfiles.tree.SubtreeArrangement;
      /**
       * Retrieves all children of the given local root.
       * <p>
       * This method is called by {@link yfiles.tree.AspectRatioTreeLayout#applyLayoutCore} to arrange the subtrees below the local
       * root.
       * </p>
       * @param localRoot the root of a subtree
       * @returns all successors of the given node
       * @protected
       */
      getSuccessors(localRoot:yfiles.algorithms.Node):yfiles.algorithms.INodeCursor;
      /**
       * Data provider key for specifying a target aspect ratio for each subtree
       * <p>
       * The aspect ratio needs to be greater than <code>0</code>.
       * </p>
       * <ul>
       * <li>aspect ratio <code>1</code>: width and height of the layout should be the same</li>
       * <li>aspect ratio between <code>0</code> and <code>1</code>: the height of the layout should be greater than its width</li>
       * <li>aspect ratio greater than <code>1</code>: the width of the layout should be greater than its height</li>
       * </ul>
       * <p>
       * If no specific ratio is defined for a subtree, the layout algorithm falls back to the default {@link yfiles.tree.AspectRatioTreeLayout#aspectRatio aspect ratio}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static SUBTREE_ASPECT_RATIO_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data provider key for specifying the placement of each subtree root
       * <p>
       * If no root placement is specified for one of the subtree roots, the default {@link yfiles.tree.AspectRatioTreeLayout#rootPlacement root placement} will be used.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.tree.RootPlacement>}
       */
      static ROOT_PLACEMENT_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.tree.RootPlacement>;
      /**
       * Data provider key for specifying the routing direction for each subtree root
       * <p>
       * The children in a subtree are arranged either horizontally or vertically. The edges are routed to the top of the child
       * nodes or at the side, respectively. Directions depend on the {@link yfiles.layout.MultiStageLayout#layoutOrientation layout orientation} and refer to {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}
       * for this description.
       * </p>
       * <p>
       * If no specific routing policy is specified for a subtree root, the layout algorithm uses the default {@link yfiles.tree.AspectRatioTreeLayout#subtreeArrangement routing direction}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.tree.SubtreeArrangement>}
       */
      static SUBTREE_ROUTING_POLICY_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.tree.SubtreeArrangement>;
      /**
       * Data provider key for marking the node that will be used as root node of the tree
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static SELECTED_ROOT_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * The input graph this algorithm is handling.
       * @protected
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * Gets or sets the horizontal distance between adjacent nodes.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      horizontalDistance:number;
      /**
       * Gets or sets the vertical distance between adjacent nodes.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      verticalDistance:number;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for hiding group nodes is activated.
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @see yfiles.layout.MultiStageLayout#hideGroupsStage
       * @see yfiles.layout.HideGroupsStage
       * @type {boolean}
       */
      hideGroupsStageEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing parallel edges is activated.
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouter
       * @see yfiles.router.ParallelEdgeRouter
       * @type {boolean}
       */
      parallelEdgeRouterEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      /**
       * Gets or sets the {@link yfiles.collections.IComparer.<T>} that will be used for sorting the {@link yfiles.algorithms.Node#sortOutEdges outgoing edges} of each local root in the tree before they are being
       * arranged.
       * @type {yfiles.collections.IComparer.<Object>}
       */
      comparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets the default aspect ratio for this {@link yfiles.tree.AspectRatioTreeLayout}.
       * <p>
       * This aspect ratio is used for all subtrees for which there is no specific aspect ratio defined in a {@link yfiles.algorithms.IDataProvider}
       * registered with {@link yfiles.tree.AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY}.
       * </p>
       * <p>
       * The aspect ratio needs to have a positive value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified aspect ratio is <code>0</code> or negative
       * @type {number}
       */
      aspectRatio:number;
      /**
       * Gets or sets the desired placement of the tree's root node.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown placement specifiers is set
       * @type {yfiles.tree.RootPlacement}
       */
      rootPlacement:yfiles.tree.RootPlacement;
      /**
       * Gets or sets how the children of a local root are arranged and how the edges between them are routed.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified routing policy is unknown
       * @type {yfiles.tree.SubtreeArrangement}
       */
      subtreeArrangement:yfiles.tree.SubtreeArrangement;
      /**
       * Gets or sets the preferred distance between any two bends of an edge.
       * <p>
       * Additionally, the preferred bend distance governs the distance between the first and last edges and the corresponding
       * ports.
       * </p>
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      bendDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.AssistantNodePlacer} places nodes that are marked as assistants left and right of their parents and all
     * other nodes below the assistant nodes.
     * <p>
     * <b>Layout Style</b> {@graph {"ann":{"s":[30,30],"d":1},"n":[[75,0],[100,171],[50,171],[0,171],[150,171],[110,50,1],[40,100,1],[40,50,1]],"e":[[0,1,[90,150.5,115,150.5]],[0,2,[90,150.5,65,150.5]],[0,3,[90,150.5,15,150.5]],[0,4,[90,150.5,165,150.5]],[0,5,[90,65]],[0,6,[90,115]],[0,7,[90,65]]],"vp":[0.0,0.0,180.0,201.0]}}
     * </p>
     * <p>
     * Assistants are marked
     * </p>
     * <p>
     * <b>Concept</b> This {@link yfiles.tree.INodePlacer} delegates the placement of all assistant nodes to an instance of
     * {@link yfiles.tree.LeftRightNodePlacer}. The non-assistant nodes are handled by another {@link yfiles.tree.AssistantNodePlacer#childNodePlacer node placer}. To be able to do this, {@link yfiles.tree.AssistantNodePlacer}
     * uses a {@link yfiles.tree.IProcessor} which prepares the graph for node placing.
     * </p>
     * <p>
     * A {@link yfiles.algorithms.IDataProvider} that is registered using {@link yfiles.tree.AssistantNodePlacer#ASSISTANT_NODE_DP_KEY} returns
     * <code>true</code> for assistant nodes and <code>false</code> for all other nodes.
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface AssistantNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class AssistantNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.AssistantNodePlacer} instance that uses the transformation defined by the given matrix.
       * @param modificationMatrix the transformation matrix
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a new {@link yfiles.tree.AssistantNodePlacer} instance with default settings.
       */
      constructor();
      /**
       * Creates a {@link yfiles.collections.IComparer.<T>} that compares outgoing edges connecting to assistant nodes and edges connecting to non-assistant nodes
       * separately.
       * @param assistantComparer the {@link } for edges connecting to <em>assistant</em> nodes
       * @param childComparer the {@link } for edges connecting to <em>non-assistant</em> nodes
       * @returns a {@link } that compares edges to assistant nodes separately from edges to non-assistant nodes
       * @static
       */
      static createCompoundComparer(assistantComparer:yfiles.collections.IComparer<Object>,childComparer:yfiles.collections.IComparer<Object>):yfiles.collections.IComparer<Object>;
      /**
       * Data provider key for marking which nodes are placed as assistants
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static ASSISTANT_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets or sets the spacing between subtrees for the delegate {@link yfiles.tree.INodePlacer}.
       * <p>
       * The spacing needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given spacing is negative
       * @type {number}
       */
      spacing:number;
      /**
       * Gets or sets the {@link yfiles.tree.INodePlacer} instance that places the non-assistant children.
       * <p>
       * These non-assistant child nodes will be placed below the assistant child nodes and are arranged by the given
       * {@link yfiles.tree.INodePlacer}. Non-assistant child nodes are the ones that are not marked in the {@link yfiles.algorithms.IDataProvider}
       * registered with {@link yfiles.tree.AssistantNodePlacer#ASSISTANT_NODE_DP_KEY}.
       * </p>
       * @see yfiles.tree.AssistantNodePlacer#ASSISTANT_NODE_DP_KEY
       * @type {yfiles.tree.INodePlacer}
       */
      childNodePlacer:yfiles.tree.INodePlacer;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.BusNodePlacer} creates a bus to which all child nodes and the local root node are connected.
     * <p>
     * <b>Layout Style</b> {@graph {"ann":{"s":[30,30],"d":1},"n":[[40,0],[140,0],[190,0],[50,70],[0,70],[100,70],[150,70],[90,0,1]],"e":[[7,0,[105,50,55,50]],[7,1,[105,50,155,50]],[7,2,[105,50,205,50]],[7,3,[105,50,65,50]],[7,4,[105,50,15,50]],[7,5,[105,50,115,50]],[7,6,[105,50,165,50]]],"vp":[0.0,0.0,220.0,100.0]}}
     * </p>
     * <p>
     * The root node is marked
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface BusNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class BusNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.BusNodePlacer} instance that uses the transformation defined by the given matrix.
       * @param modificationMatrix the transformation matrix
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a new instance of {@link yfiles.tree.BusNodePlacer} with default settings.
       */
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm arranges graphs with a tree structure.
     * <p>
     * <b>Layout Style</b> {@link yfiles.tree.ClassicTreeLayout} is designed to arrange directed and undirected trees that have a unique root node. All
     * children are placed below their parent in relation to the main layout direction. The edges of the graph are routed as
     * straight-line segments or in an orthogonal bus-like fashion.
     * </p>
     * <p>
     * Tree layout algorithms are commonly used for visualizing relational data and for producing diagrams of high quality that
     * are able to reveal possible hierarchic properties of the graph. More precisely, they find applications in dataflow
     * analysis, software engineering, bioinformatics and business administration.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[215.62,70],[365.62,0],[706.25,210],[25,140],[331.25,140],[100,280],[512.5,210],[325,280],[762.5,280],[200,210],[150,350],[400,350],[825,350],[350,350],[25,210],[900,420],[700,420],[100,350],[575,280],[575,350],[600,420],[200,280],[150,420],[25,280],[550,420],[300,420],[850,420],[800,420],[400,420],[200,350],[350,420],[50,350],[500,420],[100,420],[750,420],[450,420],[0,420],[50,420],[200,420],[650,280],[250,420],[650,420],[650,350]],"e":[[1,0,0,15,0,-15,[230.62,50]],[1,2,0,15,0,-15,[721.25,50]],[1,3,0,15,0,-15,[40,50]],[0,4,0,15,0,-15,[346.25,120]],[0,5,0,15,0,-15,[115,120]],[4,6,0,15,0,-15,[527.5,190]],[4,7,0,15,0,-15,[340,190]],[2,8,0,15,0,-15,[777.5,260]],[4,9,0,15,0,-15,[215,190]],[4,10,0,15,0,-15,[165,190]],[7,11,0,15,0,-15,[415,330]],[8,12,0,15,0,-15,[840,330]],[7,13,0,15,0,-15,[365,330]],[3,14,0,15,0,-15],[12,15,0,15,0,-15,[915,400]],[8,16,0,15,0,-15,[715,330]],[5,17,0,15,0,-15],[6,18,0,15,0,-15,[590,260]],[18,19,0,15,0,-15],[19,20,0,15,0,-15,[615,400]],[9,21,0,15,0,-15],[10,22,0,15,0,-15],[14,23,0,15,0,-15],[19,24,0,15,0,-15,[565,400]],[7,25,0,15,0,-15,[315,330]],[12,26,0,15,0,-15,[865,400]],[12,27,0,15,0,-15,[815,400]],[11,28,0,15,0,-15],[21,29,0,15,0,-15],[13,30,0,15,0,-15],[23,31,0,15,0,-15,[65,330]],[6,32,0,15,0,-15,[515,260]],[17,33,0,15,0,-15],[12,34,0,15,0,-15,[765,400]],[6,35,0,15,0,-15,[465,260]],[23,36,0,15,0,-15,[15,330]],[31,37,0,15,0,-15],[29,38,0,15,0,-15],[2,39,0,15,0,-15,[665,260]],[7,40,0,15,0,-15,[265,330]],[39,42,0,15,0,-15],[42,41,0,15,0,-15]],"vp":[0.0,0.0,930.0,450.0]}} A Dendrogram {@graph {"ann":{"s":[30,30],"d":1},"n":[[983.12,0],[495,85],[495,170],[151.25,170],[495,240],[42.5,255],[85,325],[0,325],[520,310],[470,310],[520,380],[135,395],[470,380],[85,395],[520,450],[35,395],[185,240,265,270,2],[260,255],[320,325],[405,395],[320,395],[370,465],[320,465],[235,395],[270,465],[200,325],[570,155,560,510,2],[670,170],[1060,240],[670,240],[1060,310],[670,325],[585,240],[1085,380],[670,395],[670,465],[620,325],[1035,380],[720,310,295,340,2],[810,325],[920,395],[835,395],[920,465],[785,395],[835,465],[970,535],[920,535],[735,395],[735,465],[870,535],[920,605],[1150,70,440,495,2],[1471.25,85],[1545,155],[1397.5,155],[1510,240],[1435,240],[1460,310],[1360,240],[1460,380],[1485,450],[1410,310],[1360,310],[1435,450],[1510,310],[1360,380],[1435,520],[1165,225,175,230,2],[1230,240],[1280,310],[1230,310],[1180,310],[1265,380,60,60,2],[1280,395]],"e":[[0,1,[998.12,50,510,50]],[1,2],[1,3,[510,135,166.25,135]],[2,4],[3,5,[166.25,220,57.5,220]],[5,6,[57.5,305,100,305]],[5,7,[57.5,305,15,305]],[4,8,[510,290,535,290]],[4,9,[510,290,485,290]],[8,10],[6,11,[100,375,150,375]],[9,12],[6,13],[10,14],[6,15,[100,375,50,375]],[3,17,[166.25,220,275,220]],[17,18,[275,305,335,305]],[18,19,[335,375,420,375]],[18,20],[20,21,[335,445,385,445]],[20,22],[18,23,[335,375,250,375]],[20,24,[335,445,285,445]],[17,25,[275,305,215,305]],[1,27,[510,135,685,135]],[27,28,[685,220,1075,220]],[27,29],[28,30],[29,31],[27,32,[685,220,600,220]],[30,33,[1075,360,1100,360]],[31,34],[34,35],[29,36,[685,290,635,290]],[30,37,[1075,360,1050,360]],[29,39,[685,290,825,290]],[39,40,[825,375,935,375]],[39,41,[825,375,850,375]],[40,42],[39,43,[825,375,800,375]],[41,44],[42,45,[935,515,985,515]],[42,46],[39,47,[825,375,750,375]],[47,48],[42,49,[935,515,885,515]],[46,50],[0,52,[998.12,50,1486.25,50]],[52,53,[1486.25,135,1560,135]],[52,54,[1486.25,135,1412.5,135]],[54,55,[1412.5,205,1525,205]],[54,56,[1412.5,205,1450,205]],[56,57,[1450,290,1475,290]],[54,58,[1412.5,205,1375,205]],[57,59],[59,60,[1475,430,1500,430]],[56,61,[1450,290,1425,290]],[58,62],[59,63,[1475,430,1450,430]],[55,64],[62,65],[63,66],[54,68,[1412.5,205,1245,205]],[68,69,[1245,290,1295,290]],[68,70],[68,71,[1245,290,1195,290]],[69,73]],"vp":[0.0,0.0,1590.0,665.0]}} A graph containing group nodes {@graph {"ann":{"s":[30,30],"d":1},"n":[[4.33,440.22],[74.33,418.34,73.78,73.78],[188.11,41,48,48],[197.11,417.95],[298,245.33],[298,445.67],[389.89,393.78,73.78,73.78],[411.78,146.89],[298,25],[411.78,243.78],[411.78,293.78],[298,590.56],[503.67,100,73.78,73.78],[197.11,708.56],[188.11,821.45,48,48],[525.56,415.67],[525.56,243.78],[399.84,13.06,53.88,53.88],[276.11,661.67,73.78,73.78],[411.78,343.78],[411.78,497.56],[402.78,547.56,48,48],[402.78,615.56,48,48],[298,755.45],[525.56,472.56],[525.56,293.78],[629.39,243.78],[525.56,0],[525.56,50],[525.56,522.56],[298,805.45],[617.45,485.62,53.88,53.88],[629.39,50],[411.78,755.45],[711.33,38.06,53.88,53.88],[629.39,559.5],[298,75],[411.78,683.56],[525.56,599.56],[411.78,805.45],[723.27,559.5],[629.39,293.78],[525.56,193.78],[723.27,497.56],[298,855.45],[525.56,649.56]],"e":[[0,1],[1,2,[168.11,455.22,168.11,65]],[1,3,[168.11,455.22,168.11,432.95]],[3,4,[256.11,432.95,256.11,260.33]],[3,5,[256.11,432.95,256.11,460.67]],[5,6,[369.89,460.67,369.89,430.67]],[4,7,[369.89,260.33,369.89,161.89]],[2,8,[256.11,65,256.11,40]],[4,9,[369.89,260.33,369.89,258.78]],[4,10,[369.89,260.33,369.89,308.78]],[3,11,[256.11,432.95,256.11,605.56]],[7,12,[483.67,161.89,483.67,136.89]],[1,13,[168.11,455.22,168.11,723.56]],[1,14,[168.11,455.22,168.11,845.45]],[6,15],[9,16],[8,17],[13,18,[256.11,723.56,256.11,698.56]],[4,19,[369.89,260.33,369.89,358.78]],[5,20,[369.89,460.67,369.89,512.56]],[11,21,[369.89,605.56,369.89,571.56]],[11,22,[369.89,605.56,369.89,639.56]],[13,23,[256.11,723.56,256.11,770.45]],[20,24,[483.67,512.56,483.67,487.56]],[10,25],[16,26],[17,27,[483.67,40,483.67,15]],[17,28,[483.67,40,483.67,65]],[20,29,[483.67,512.56,483.67,537.56]],[14,30,[256.11,845.45,256.11,820.45]],[29,31,[597.45,537.56,597.45,512.56]],[28,32],[23,33],[32,34],[29,35,[597.45,537.56,597.45,574.5]],[2,36,[256.11,65,256.11,90]],[18,37],[22,38,[483.67,639.56,483.67,614.56]],[30,39],[35,40],[25,41],[7,42,[483.67,161.89,483.67,208.78]],[31,43],[14,44,[256.11,845.45,256.11,870.45]],[22,45,[483.67,639.56,483.67,664.56]]],"vp":[4.0,0.0,762.0,886.0]}} Left-to-right layout with global layering<b>Concept</b>
     * </p>
     * <p>
     * The layout algorithm starts from the root and recursively assigns coordinates to all tree nodes. In this manner, leaf
     * nodes will be placed first, while each parent node is placed centered above its children.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The layout algorithm supports custom sorting of sibling nodes by using a specific {@link yfiles.tree.NodeOrderComparer}. Each
     * tree node may have an individual {@link yfiles.tree.NodeOrderComparer} that is stored in a {@link yfiles.algorithms.IDataProvider}
     * registered with the graph with {@link yfiles.tree.NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY}. If this {@link yfiles.tree.NodeOrderComparer}
     * uses the initial coordinates of the nodes, it allows the incremental insertion of child nodes while keeping the relative
     * locations of the other nodes.
     * </p>
     * <p>
     * A custom node can be defined as root of the tree using a {@link yfiles.algorithms.IDataProvider} registered with the graph with
     * key {@link yfiles.tree.ClassicTreeLayout#SELECTED_ROOT_DP_KEY}.
     * </p>
     * <p>
     * {@link yfiles.tree.ClassicTreeLayout} can be configured to {@link yfiles.tree.ClassicTreeLayout#considerNodeLabels reserve space for node labels}. It can also {@link yfiles.tree.ClassicTreeLayout#integratedEdgeLabeling place edge labels along edges}
     * such that the labels won't overlap with other graph elements. Edge labels are placed according to the information stored
     * in a {@link yfiles.layout.PreferredPlacementDescriptor} instance. However, the placement along the edge will only affect the
     * order of multiple labels at the same edge. The algorithm will always place the labels close to the target node.
     * </p>
     * <p>
     * Grouping of nodes can also be handled by this layout algorithm. It is important that a group node contains a whole
     * subtree. Otherwise, the group nodes may overlap with each other or with other nodes. Furthermore, the user may specify
     * minimum size constraints for each group node using {@link yfiles.algorithms.IDataProvider} key
     * {@link yfiles.layout.GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
     * </p>
     * <p>
     * This layout algorithm can only handle graphs with a tree structure. To apply it to a general graph, a {@link yfiles.tree.TreeReductionStage}
     * can be appended. This stage will temporarily remove some edges of the input graph until a tree is obtained. After the
     * layout calculation, the stage will reinsert the edges that were removed and route them separately.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface ClassicTreeLayout extends yfiles.layout.MultiStageLayout{}
    export class ClassicTreeLayout {
      /**
       * Creates a new {@link yfiles.tree.ClassicTreeLayout} instance with default settings.
       */
      constructor();
      /**
       * Data provider key for marking the node that will be used as root node of the tree
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static SELECTED_ROOT_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets or sets whether or not a global layering is enforced, that is, each node spans exactly one layer.
       * <p>
       * If it is enabled, the algorithm ensures that nodes never span more than one layer. Otherwise, nodes with larger size
       * might span two or more layers such that a more compact layout is produced.
       * </p>
       * <p>
       * This setting can be advantageous, if the hierarchical structure of the tree should be emphasized.
       * </p>
       * @see yfiles.tree.ClassicTreeLayout#verticalAlignment
       * @see yfiles.tree.ClassicTreeLayout#verticalAlignment
       * @type {boolean}
       */
      enforceGlobalLayering:boolean;
      /**
       * Gets or sets the layout policy for leaf nodes.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified policy is unknown
       * @type {yfiles.tree.LeafPlacement}
       */
      leafPlacement:yfiles.tree.LeafPlacement;
      /**
       * Gets or sets whether or not group nodes are handled by the layout algorithm.
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @type {boolean}
       */
      groupingSupported:boolean;
      /**
       * Gets or sets the {@link yfiles.collections.IComparer.<T>} instance that is used for sorting the outgoing edges of each node of the tree.
       * <p>
       * This comparator defines the relative order of the child nodes in the layout.
       * </p>
       * @type {yfiles.collections.IComparer.<Object>}
       */
      comparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets the port assignment policy that will be applied.
       * @type {yfiles.tree.PortStyle}
       */
      portStyle:yfiles.tree.PortStyle;
      /**
       * Gets or sets the edge routing style used by this layout algorithm.
       * <p>
       * If {@link yfiles.tree.EdgeRoutingStyle#PLAIN} is set, the edges will be routed as straight-line segments. If {@link yfiles.tree.EdgeRoutingStyle#ORTHOGONAL}
       * is set, all edges will be routed orthogonally in a bus-like fashion.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an invalid layout style is provided
       * @type {yfiles.tree.EdgeRoutingStyle}
       */
      edgeRoutingStyle:yfiles.tree.EdgeRoutingStyle;
      /**
       * Gets or sets the minimum horizontal distance between adjacent nodes within the same layer.
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance is negative
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets the minimum distance between two adjacent layers.
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance is negative
       * @type {number}
       */
      minimumLayerDistance:number;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for node labels.
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for edge labels and places them.
       * <p>
       * To define the desired placement for each label add a {@link yfiles.layout.PreferredPlacementDescriptor} on
       * {@link yfiles.layout.IEdgeLabelLayout}.
       * </p>
       * <p>
       * This method is a convenience method that assures that the {@link yfiles.layout.MultiStageLayout#labeling label layouter} is of type {@link yfiles.layout.LabelLayoutTranslator}
       * and {@link yfiles.layout.LabelLayoutTranslator#translateEdgeLabels} is set to <code>true</code>.
       * </p>
       * @type {boolean}
       */
      integratedEdgeLabeling:boolean;
      /**
       * Gets or sets the vertical bus alignment for orthogonally routed edge buses.
       * <p>
       * The bus alignment determines the relative position of an edge bus between two subsequent layers. The value of this
       * property must lie within the interval <code>[0,1]</code> where:
       * </p>
       * <ul>
       * <li>A value of <code>0</code> places the bus at the top, directly below the parent node.</li>
       * <li>A value of <code>0.5</code> places the bus in the middle between parent and child nodes.</li>
       * <li>A value of <code>1</code> places the bus at the bottom, directly above the child nodes.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if <code>busAlignment</code> is not in <code>[0,1]</code>
       * @see yfiles.tree.ClassicTreeLayout#edgeRoutingStyle
       * @see yfiles.tree.ClassicTreeLayout#enforceGlobalLayering
       * @see yfiles.tree.ClassicTreeLayout#leafPlacement
       * @type {number}
       */
      busAlignment:number;
      /**
       * Gets or sets the vertical alignment of the nodes within their corresponding layers.
       * <p>
       * The value for the vertical alignment is considered as relative to the height of the corresponding layer, which is
       * determined by the maximum height of the nodes that belong to the particular layer.
       * </p>
       * <p>
       * The value of this property must lie within the interval <code>[0,1]</code>, where:
       * </p>
       * <ul>
       * <li>A value of <code>0</code> corresponds to top-alignment.</li>
       * <li>A value of <code>0.5</code> corresponds to center-alignment.</li>
       * <li>A value of <code>1</code> corresponds to bottom-alignment.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if vertical alignment value does not lie within <code>[0,1]</code>
       * @see yfiles.tree.ClassicTreeLayout#enforceGlobalLayering
       * @type {number}
       */
      verticalAlignment:number;
      /**
       * Gets or sets the modification matrix used for rotating / mirroring the layout.
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing parallel edges is activated.
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouter
       * @see yfiles.router.ParallelEdgeRouter
       * @type {boolean}
       */
      parallelEdgeRouterEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.tree.DefaultNodePlacer} is the default implementation of interface {@link yfiles.tree.INodePlacer}.
     * <p>
     * <b>Layout Style</b> The layout style can be customized in several ways. Subtrees sharing the same local root node are either placed above
     * or below each other. {@graph {"ann":{"s":[30,30],"d":1},"n":[[135,0],[0,70],[70,70],[120,149],[170,149],[220,149],[270,149],[70,199],[120,278],[170,278],[220,278],[270,278]],"e":[[0,1,0,15,0,0,[150,50,15,50]],[1,2],[2,3,-11.25,15,0,0,[73.75,116,135,133]],[2,4,-3.75,15,0,0,[81.25,116,185,133]],[2,5,3.75,15,0,0,[88.75,116,235,133]],[2,6,11.25,15,0,0,[96.25,116,285,133]],[1,7,[50,85,50,214]],[7,8,-11.25,15,0,0,[73.75,245,135,262]],[7,9,-3.75,15,0,0,[81.25,245,185,262]],[7,10,3.75,15,0,0,[88.75,245,235,262]],[7,11,11.25,15,0,0,[96.25,245,285,262]]],"vp":[0.0,0.0,300.0,308.0]}}
     * </p>
     * <p>
     * Each layer in the tree uses different configurations of {@link yfiles.tree.DefaultNodePlacer}
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * {@link yfiles.tree.SubtreeShape}s can be arranged in each main direction specified in
     * {@link yfiles.tree.DefaultNodePlacer#childPlacement}. That way, the subtrees of the same graph can have different directions.
     * </p>
     * <p>
     * This node placer aligns the local root node according to a chosen {@link yfiles.tree.DefaultNodePlacer#rootAlignment alignment}.
     * </p>
     * <p>
     * There are various {@link yfiles.tree.DefaultNodePlacer#routingStyle routing styles} that result in different edge routes.
     * </p>
     * @class
     * @extends {yfiles.tree.NodePlacerBase}
     * @implements {yfiles.tree.IFromSketchNodePlacer}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface DefaultNodePlacer extends yfiles.tree.NodePlacerBase,yfiles.tree.IFromSketchNodePlacer,yfiles.lang.ICloneable{}
    export class DefaultNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.DefaultNodePlacer} instance with custom settings.
       * @param childPlacement the child placement specifier describing the style of the arrangement
       * @param rootAlignment the alignment specifier describing how the root node is aligned with its child nodes
       * @param routingStyle the routing style specifier
       * @param verticalDistance the vertical distance between the {@link }s
       * @param horizontalDistance the horizontal distance between the {@link }s
       * @param minFirstSegmentLength the minimum length of the first edge segment (connected to the root node)
       * @param minLastSegmentLength the minimum length of the last edge segment (connected to the child node)
       * @param minSlope the minimum slope between the root node and the {@link }s
       * @param minSlopeHeight the minimum vertical height of sloped edge segments
       * @throws {Stubs.Exceptions.ArgumentError} if the placement specifier or the root alignment or the routing style is unknown or if the horizontal/vertical distance
       * or the minimum first/last segment length or the minimum slope or the minimum vertical height is negative
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       * @see yfiles.tree.DefaultNodePlacer#routingStyle
       * @see yfiles.tree.DefaultNodePlacer#verticalDistance
       * @see yfiles.tree.DefaultNodePlacer#horizontalDistance
       * @see yfiles.tree.DefaultNodePlacer#minimumFirstSegmentLength
       * @see yfiles.tree.DefaultNodePlacer#minimumLastSegmentLength
       * @see yfiles.tree.DefaultNodePlacer#minimumSlope
       * @see yfiles.tree.DefaultNodePlacer#minimumSlopeHeight
       */
      constructor(childPlacement:yfiles.tree.ChildPlacement,rootAlignment:yfiles.tree.RootAlignment,routingStyle:yfiles.tree.RoutingStyle,verticalDistance:number,horizontalDistance:number,minFirstSegmentLength:number,minLastSegmentLength:number,minSlope:number,minSlopeHeight:number);
      /**
       * Creates a new {@link yfiles.tree.DefaultNodePlacer} instance with customized {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, {@link yfiles.tree.DefaultNodePlacer#rootAlignment root alignment}, {@link yfiles.tree.DefaultNodePlacer#routingStyle routing style}, {@link yfiles.tree.DefaultNodePlacer#verticalDistance vertical}
       * and {@link yfiles.tree.DefaultNodePlacer#horizontalDistance horizontal distance}.
       * @param childPlacement the child placement specifier describing the style of the arrangement
       * @param rootAlignment the alignment specifier describing how the root node is aligned with its child nodes
       * @param routingStyle the routing style specifier
       * @param verticalDistance the vertical distance between the {@link }s
       * @param horizontalDistance the horizontal distance between the {@link }s
       * @throws {Stubs.Exceptions.ArgumentError} if the placement specifier or the root alignment or the routing style is unknown or if the horizontal/vertical distance
       * is negative
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       * @see yfiles.tree.DefaultNodePlacer#routingStyle
       * @see yfiles.tree.DefaultNodePlacer#verticalDistance
       * @see yfiles.tree.DefaultNodePlacer#horizontalDistance
       */
      constructor(childPlacement:yfiles.tree.ChildPlacement,rootAlignment:yfiles.tree.RootAlignment,routingStyle:yfiles.tree.RoutingStyle,verticalDistance:number,horizontalDistance:number);
      /**
       * Creates a new {@link yfiles.tree.DefaultNodePlacer} instance with customized {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, {@link yfiles.tree.DefaultNodePlacer#rootAlignment root alignment}, {@link yfiles.tree.DefaultNodePlacer#verticalDistance vertical} and
       * {@link yfiles.tree.DefaultNodePlacer#horizontalDistance horizontal distance}.
       * @param childPlacement the child placement specifier describing the style of the arrangement
       * @param rootAlignment the alignment specifier describing how the root node is aligned with its child nodes
       * @param verticalDistance the vertical distance between the {@link }s
       * @param horizontalDistance the horizontal distance between the {@link }s
       * @throws {Stubs.Exceptions.ArgumentError} if the placement specifier or the root alignment specifier is unknown or if the horizontal/vertical distance is
       * negative
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       * @see yfiles.tree.DefaultNodePlacer#rootAlignment
       * @see yfiles.tree.DefaultNodePlacer#verticalDistance
       * @see yfiles.tree.DefaultNodePlacer#horizontalDistance
       */
      constructor(childPlacement:yfiles.tree.ChildPlacement,rootAlignment:yfiles.tree.RootAlignment,verticalDistance:number,horizontalDistance:number);
      /**
       * Creates a new {@link yfiles.tree.DefaultNodePlacer} instance with customized {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}, {@link yfiles.tree.DefaultNodePlacer#verticalDistance vertical} and {@link yfiles.tree.DefaultNodePlacer#horizontalDistance horizontal distance}.
       * @param childPlacement the child placement specifier describing the style of the arrangement
       * @param verticalDistance the vertical distance between the {@link }s
       * @param horizontalDistance the horizontal distance between the {@link }s
       * @throws {Stubs.Exceptions.ArgumentError} if the placement specifier is unknown or if the horizontal/vertical distance is negative
       * @see yfiles.tree.DefaultNodePlacer#childPlacement
       * @see yfiles.tree.DefaultNodePlacer#verticalDistance
       * @see yfiles.tree.DefaultNodePlacer#horizontalDistance
       */
      constructor(childPlacement:yfiles.tree.ChildPlacement,verticalDistance:number,horizontalDistance:number);
      /**
       * Creates a new {@link yfiles.tree.DefaultNodePlacer} instance with default settings.
       */
      constructor();
      /**
       * Calculates the bends for the connector to the parent node.
       * <p>
       * This implementation adds bends to the connector for edges that route the edges around the child node in case there are
       * {@link yfiles.layout.PortConstraint}s that conflict with the direction of the {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}.
       * </p>
       * <p>
       * This method is called in {@link yfiles.tree.DefaultNodePlacer#placeSubtreeOfNode} to route the edges after the
       * {@link yfiles.tree.SubtreeShape}s are arranged. It may be overridden to customize the routing style.
       * </p>
       * @param graph the input graph
       * @param localRoot the local root node whose connector is calculated and stored in the given subtree shape
       * @param rootLayout the {@link } of the root node
       * @param subtreeShape the shape of the whole subtree of the local root
       * @param parentEdge the incoming edge of the local root node that will be the connector for the given subtree shape
       * @param parentEdgeLayout the current {@link } of the connector edge
       * @param direction the direction specifier as defined in the {@link } interface which should be used for the connector
       * @see yfiles.tree.NodePlacerBase#determineChildConnectors
       * @protected
       */
      calculateParentConnector(graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,rootLayout:yfiles.layout.INodeLayout,subtreeShape:yfiles.tree.SubtreeShape,parentEdge:yfiles.algorithms.Edge,parentEdgeLayout:yfiles.layout.IEdgeLayout,direction:number):void;
      /**
       * Calculates the routing of the source side of the edge to the given child node.
       * <p>
       * This method delegates to {@link yfiles.tree.DefaultNodePlacer#calculateSourceEdgeLayout} if the {@link yfiles.tree.DefaultNodePlacer#routingStyle routing style} is not
       * {@link yfiles.tree.RoutingStyle#FORK}. It may be overridden to implement a custom routing style.
       * </p>
       * @param rootLayout the {@link } of the local root node
       * @param childShape the {@link } instance of the corresponding child's subtree
       * @param edge the edge
       * @param rootForkCoordinate the y-coordinate of the first bend of the edge if the placement is horizontal, the x-coordinate otherwise
       * @param childForkCoordinate the y-coordinate of the second (= last) bend of the edge if the placement is horizontal, the x-coordinate otherwise
       * @protected
       */
      calculateSlopedSourceEdgeLayout(rootLayout:yfiles.layout.INodeLayout,childShape:yfiles.tree.SubtreeShape,edge:yfiles.algorithms.Edge,childForkCoordinate:number,rootForkCoordinate:number):void;
      /**
       * Calculates the routing of the source side of the edge to the given child node.
       * <p>
       * This implementation adds bends to the {@link yfiles.layout.IEdgeLayout} of the given edge according to the specified {@link yfiles.tree.DefaultNodePlacer#routingStyle routing style}
       * and {@link yfiles.tree.DefaultNodePlacer#childPlacement child placement}. It also updates the target points in the {@link yfiles.tree.SubtreeShape}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.DefaultNodePlacer#calculateSlopedSourceEdgeLayout} to route the edges. It may be
       * overridden to implement a custom edge routing style.
       * </p>
       * @param rootLayout the {@link } of the local root node
       * @param childShape the {@link } of the child's subtree that is connected through the given edge
       * @param edge the edge
       * @protected
       */
      calculateSourceEdgeLayout(rootLayout:yfiles.layout.INodeLayout,childShape:yfiles.tree.SubtreeShape,edge:yfiles.algorithms.Edge):void;
      /**
       * Calculates the routing of the target side of the edge to the given child node.
       * <p>
       * This implementation just resets the current {@link yfiles.layout.IEdgeLayout}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.DefaultNodePlacer#placeSubtreeOfNode} after the {@link yfiles.tree.SubtreeShape shapes} of the local root's children
       * are placed. It may be overridden to add some {@link yfiles.tree.SubtreeShape#addTargetPoint target points} to the edge.
       * </p>
       * @param rootLayout the {@link } of the local root node
       * @param childShape the {@link } of the child's subtree that is connected through the given edge
       * @param edge the edge
       * @protected
       */
      calculateTargetEdgeLayout(rootLayout:yfiles.layout.INodeLayout,childShape:yfiles.tree.SubtreeShape,edge:yfiles.algorithms.Edge):void;
      /**
       * Creates a {@link yfiles.collections.IComparer.<T>} for outgoing edges which takes the initial coordinates of the edges' targets into account.
       * <p>
       * Child nodes are sorted in a way that the layout will resemble the current arrangement of the nodes. This is especially
       * useful for interactive or incremental layout calculation.
       * </p>
       * @returns a {@link } that considers the initial coordinates of the nodes
       */
      createComparer():yfiles.collections.IComparer<Object>;
      /**
       * Delegates to {@link yfiles.tree.DefaultNodePlacer#createComparer}.
       * @returns the From Sketch {@link }
       * @see yfiles.tree.DefaultNodePlacer#createComparer
       */
      createFromSketchComparer():yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets the direction in which the child nodes are placed with respect to their parent node.
       * <p>
       * Subtrees can be placed below, above, left or right of their roots.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown placement specifier is set
       * @type {yfiles.tree.ChildPlacement}
       */
      childPlacement:yfiles.tree.ChildPlacement;
      /**
       * Gets or sets how the root node is aligned with its children and their {@link yfiles.tree.SubtreeShape}s.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown root alignment specifier is set
       * @type {yfiles.tree.RootAlignment}
       */
      rootAlignment:yfiles.tree.RootAlignment;
      /**
       * Gets or sets the relative vertical alignment of nodes with the same parent.
       * <p>
       * All values between <code>0</code> and <code>1</code> result in a valid alignment.
       * </p>
       * <ul>
       * <li><code>0</code> - nodes are top-aligned</li>
       * <li><code>0.5</code> - nodes are center-aligned</li>
       * <li><code>1</code> - nodes are bottom-aligned</li>
       * </ul>
       * <p>
       * Values outside the interval <code>[0,1]</code> will result in a more compact node placement with unaligned nodes.
       * </p>
       * @type {number}
       */
      verticalAlignment:number;
      /**
       * Gets or sets the style in which edge paths are routed.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified routing style is unknown
       * @type {yfiles.tree.RoutingStyle}
       */
      routingStyle:yfiles.tree.RoutingStyle;
      /**
       * Gets or sets the vertical distance between nodes or {@link yfiles.tree.SubtreeShape}s.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      verticalDistance:number;
      /**
       * Gets or sets the minimum length for the first segment of an edge.
       * <p>
       * During layout calculation, the edges of the tree are directed. Hence, the first segment is always the segment that is
       * attached to the source, which is the local root node.
       * </p>
       * <p>
       * The length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum segment length is negative
       * @type {number}
       */
      minimumFirstSegmentLength:number;
      /**
       * Gets or sets the minimum length for the last segment of an edge.
       * <p>
       * During layout calculation, the edges of the tree are directed. Hence, the last segment is always the segment that is
       * attached to the target, which is a child node.
       * </p>
       * <p>
       * The length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum segment length is negative
       * @type {number}
       */
      minimumLastSegmentLength:number;
      /**
       * Gets or sets the minimum slope of the edge segments.
       * <p>
       * These sloped segments are between the vertical first and last segment of the edge.
       * </p>
       * <p>
       * The segments may become steeper but they will have at least the specified slope. Negative minimum slopes can lead to
       * edge routes looking like spider legs.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#minimumSlopeHeight
       * @type {number}
       */
      minimumSlope:number;
      /**
       * Gets or sets the minimum height for sloped edge segments.
       * <p>
       * These sloped segments are between the vertical first and last segment of the edge.
       * </p>
       * <p>
       * The segments may become steeper to reach the specified height.
       * </p>
       * @see yfiles.tree.DefaultNodePlacer#minimumSlope
       * @type {number}
       */
      minimumSlopeHeight:number;
      /**
       * Gets or sets the horizontal distance between nodes or {@link yfiles.tree.SubtreeShape}s.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      horizontalDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.DefaultPortAssignment} provides some simple {@link yfiles.tree.DefaultPortAssignment#mode port assignment styles}.
     * <p>
     * Ports are either {@link yfiles.tree.PortAssignmentMode#NONE centered} on the node, distributed at one side of the node or placed considering
     * {@link yfiles.layout.PortConstraint}s.
     * </p>
     * @class
     * @implements {yfiles.tree.IPortAssignment}
     */
    export interface DefaultPortAssignment extends Object,yfiles.tree.IPortAssignment{}
    export class DefaultPortAssignment {
      /**
       * Creates a new {@link yfiles.tree.DefaultPortAssignment} instance using the given port assignment mode and ratio between the
       * border and the ports and the gap between the ports themselves.
       * @param mode one of the predefined port assignment specifiers
       * @param [ratio=0.5] the ratio of the distance between outer ports and the border to the distance between any two ports
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown mode is specified or if the specified ratio is negative
       */
      constructor(mode:yfiles.tree.PortAssignmentMode,ratio?:number);
      /**
       * Creates a new {@link yfiles.tree.DefaultPortAssignment} instance with default settings.
       */
      constructor();
      /**
       * Assigns relative coordinates to the source port of the outgoing edge to a child of the given node.
       * <p>
       * This method is called by {@link yfiles.tree.DefaultPortAssignment#assignPorts} for each outgoing edge of the given node. It may
       * be overridden to assign a custom location to the source port of outgoing edges.
       * </p>
       * @param graph the input graph
       * @param node the node
       * @param edge the edge to the child node
       * @param index the zero-based index of the child edge
       * @see yfiles.layout.IEdgeLayout#sourcePoint
       * @protected
       */
      assignChildEdgeSourcePort(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node,edge:yfiles.algorithms.Edge,index:number):void;
      /**
       * Assigns relative coordinates to the target port of the incoming edge from the parent of the given node.
       * <p>
       * This method is called by {@link yfiles.tree.DefaultPortAssignment#assignPorts} if the given node has a parent node. It may be
       * overridden to assign a custom location to the target port of the incoming edge.
       * </p>
       * @param graph the input graph
       * @param node the node
       * @param edge the edge from the parent node
       * @see yfiles.layout.IEdgeLayout#targetPoint
       * @protected
       */
      assignParentEdgeTargetPort(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node,edge:yfiles.algorithms.Edge):void;
      /**
       * Places the ports of edges connecting to the given node according to the {@link yfiles.tree.DefaultPortAssignment#mode port assignment mode}.
       * @param graph the input graph
       * @param node the node whose adjacent edges' ports should be placed
       * @see yfiles.tree.DefaultPortAssignment#mode
       */
      assignPorts(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):void;
      /**
       * Determines the distance between the outer ports and the border of the node.
       * <p>
       * This implementation calculates the gap according to the number of edges at one side of the node. It also includes the
       * {@link yfiles.tree.DefaultPortAssignment#borderGapToPortGapRatio ratio between border gap and port gap}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.DefaultPortAssignment#assignChildEdgeSourcePort} to calculate the location of the
       * first port. It may be overridden to implement a different distribution of ports on the node's side.
       * </p>
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges that connect to this side
       * @returns the absolute gap to be used on both sides of the ports
       * @see yfiles.tree.DefaultPortAssignment#assignChildEdgeSourcePort
       * @see yfiles.tree.DefaultPortAssignment#borderGapToPortGapRatio
       * @protected
       */
      getPortBorderGap(sideLength:number,edgeCount:number):number;
      /**
       * Determines the distance between two adjacent ports.
       * <p>
       * This implementation calculates the gap according to the number of edges at one side of the node. The edges are
       * distributed keeping the given gap to the border of the node.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.DefaultPortAssignment#assignChildEdgeSourcePort} to calculate the distance between
       * two ports. It may be overridden to implement a different distribution of ports on the node's side.
       * </p>
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges/ports that connect to this side
       * @param portBorderGap the port border gap (i.e. calculated by {@link #getPortBorderGap})
       * @returns the absolute distance to be used between two adjacent ports
       * @see yfiles.tree.DefaultPortAssignment#assignChildEdgeSourcePort
       * @see yfiles.tree.DefaultPortAssignment#getPortBorderGap
       * @protected
       */
      getPortDistanceDelta(sideLength:number,edgeCount:number,portBorderGap:number):number;
      /**
       * Retrieves the source {@link yfiles.layout.PortConstraint} for the given edge.
       * <p>
       * This implementation accesses the {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.DefaultPortAssignment#assignChildEdgeSourcePort} in
       * {@link yfiles.tree.PortAssignmentMode#PORT_CONSTRAINT}.
       * </p>
       * @param graph the input graph
       * @param edge the edge
       * @param index the index of the child that is the target of the given edge
       * @returns the source {@link } or <code>null</code> if no source {@link } is specified
       * @see yfiles.layout.PortConstraint
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @see yfiles.tree.DefaultPortAssignment#assignChildEdgeSourcePort
       * @protected
       */
      getSourcePortConstraint(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge,index:number):yfiles.layout.PortConstraint;
      /**
       * Retrieves the target {@link yfiles.layout.PortConstraint} for the given edge.
       * <p>
       * This implementation accesses the {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.DefaultPortAssignment#assignParentEdgeTargetPort} in
       * {@link yfiles.tree.PortAssignmentMode#PORT_CONSTRAINT}.
       * </p>
       * @param graph the input graph
       * @param edge the edge
       * @returns the target {@link } or <code>null</code> if no target {@link } is specified
       * @see yfiles.layout.PortConstraint
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @see yfiles.tree.DefaultPortAssignment#assignParentEdgeTargetPort
       * @protected
       */
      getTargetPortConstraint(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):yfiles.layout.PortConstraint;
      /**
       * Gets or sets the mode that describes how ports are placed.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown mode is specified
       * @type {yfiles.tree.PortAssignmentMode}
       */
      mode:yfiles.tree.PortAssignmentMode;
      /**
       * Gets or sets the ratio of the <em>border gap</em> (distance between the outer ports to the node border) to the <em>port gap</em> (distance between adjacent
       * ports).
       * <p>
       * The ratio must have a positive value. When the ratio is <code>0</code> the ports will be distributed along the whole side of the node
       * without a gap between the last port and the corner.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified ratio is negative
       * @type {number}
       */
      borderGapToPortGapRatio:number;
      /**
       * Gets or sets whether or not the port assignment should reverse the order of the ports.
       * <p>
       * The normal order is from left to right and from top to bottom. It will be reversed if this option is enabled.
       * </p>
       * @type {boolean}
       */
      reversedPortOrder:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.tree.DelegatingNodePlacer} delegates the arrangement of the children to two different {@link yfiles.tree.INodePlacer}s.
     * <p>
     * <b>Layout Style</b> This {@link yfiles.tree.INodePlacer} can, for example, be used for creating mind maps or a playing schedule of a sports
     * competition. {@graph {"ann":{"s":[130,30],"d":1},"n":[[662,400,[[669.97,424.35,114.05,18.7]]],[662,175,[[669.97,199.35,114.05,18.7]]],[662,625,[[685.65,649.35,82.7,18.7]]],[812,175,[[829.98,199.35,94.04,18.7]]],[982,275,[[999.98,299.35,94.04,18.7]]],[812,625,[[835.65,649.35,82.7,18.7]]],[512,175,[[519.97,199.35,114.05,18.7]]],[982,725,[[1010.64,749.35,72.71,18.7]]],[1152,775,[[1180.64,799.35,72.71,18.7]]],[1322,800,[[1341.97,824.35,90.05,18.7]]],[982,75,[[1004.32,99.35,85.37,18.7]]],[1152,675,[[1163.65,699.35,106.71,18.7]]],[1152,325,[[1177.32,349.35,79.36,18.7]]],[982,525,[[1005.65,549.35,82.7,18.7]]],[1152,125,[[1190.32,149.35,53.37,18.7]]],[1152,575,[[1175.65,599.35,82.7,18.7]]],[1322,350,[[1347.32,374.35,79.36,18.7]]],[1322,150,[[1343.65,174.35,86.7,18.7]]],[1322,300,[[1347.32,324.35,79.36,18.7]]],[1152,225,[[1169.98,249.35,94.04,18.7]]],[1152,25,[[1174.32,49.35,85.37,18.7]]],[342,75,[[372.66,99.35,68.69,18.7]]],[1152,475,[[1165.65,499.35,102.7,18.7]]],[1322,500,[[1326,524.35,122.06,18.7]]],[1322,250,[[1339.98,274.35,94.04,18.7]]],[1322,600,[[1330.31,624.35,113.38,18.7]]],[172,25,[[188.64,49.35,96.72,18.7]]],[342,275,[[349.97,299.35,114.05,18.7]]],[2,0,[[26.32,24.35,81.36,18.7]]],[1322,550,[[1345.65,574.35,82.7,18.7]]],[172,125,[[202.66,149.35,68.69,18.7]]],[1322,200,[[1344.32,224.35,85.36,18.7]]],[2,100,[[32.32,124.35,69.37,18.7]]],[1322,450,[[1335.65,474.35,102.7,18.7]]],[1322,750,[[1350.64,774.35,72.71,18.7]]],[512,625,[[532.65,649.35,88.69,18.7]]],[172,225,[[179.97,249.35,114.05,18.7]]],[1322,700,[[1329.63,724.35,114.74,18.7]]],[172,325,[[200.98,349.35,72.03,18.7]]],[1322,50,[[1344.32,74.35,85.37,18.7]]],[2,300,[[4.98,324.35,124.05,18.7]]],[2,50,[[18.64,74.35,96.72,18.7]]],[2,150,[[32.66,174.35,68.69,18.7]]],[1322,650,[[1333.65,674.35,106.71,18.7]]],[1322,0,[[1356.66,24.35,60.69,18.7]]],[1322,100,[[1360.32,124.35,53.37,18.7]]],[342,525,[[347.31,549.35,119.39,18.7]]],[172,475,[[171.97,499.35,130.06,18.7]]],[2,350,[[30.98,374.35,72.03,18.7]]],[2,450,[[1.97,474.35,130.06,18.7]]],[342,725,[[362.65,749.35,88.69,18.7]]],[2,200,[[25.31,224.35,83.37,18.7]]],[172,575,[[177.31,599.35,119.39,18.7]]],[172,675,[[197.32,699.35,79.35,18.7]]],[2,500,[[15.98,524.35,102.04,18.7]]],[2,550,[[14.63,574.35,104.73,18.7]]],[2,600,[[7.31,624.35,119.39,18.7]]],[2,250,[[9.97,274.35,114.05,18.7]]],[172,775,[[192.65,799.35,88.69,18.7]]],[2,750,[[22.65,774.35,88.69,18.7]]],[2,800,[[16.64,824.35,100.72,18.7]]],[2,650,[[11.97,674.35,110.07,18.7]]],[2,700,[[27.32,724.35,79.35,18.7]]]],"e":[[0,1],[0,2],[1,3],[3,4,[954,190,954,290]],[2,5],[1,6],[5,7,[954,640,954,740]],[7,8,[1124,740,1124,790]],[8,9,[1294,790,1294,815]],[3,10,[954,190,954,90]],[7,11,[1124,740,1124,690]],[4,12,[1124,290,1124,340]],[5,13,[954,640,954,540]],[10,14,[1124,90,1124,140]],[13,15,[1124,540,1124,590]],[12,16,[1294,340,1294,365]],[14,17,[1294,140,1294,165]],[12,18,[1294,340,1294,315]],[4,19,[1124,290,1124,240]],[10,20,[1124,90,1124,40]],[6,21,[500,190,500,90]],[13,22,[1124,540,1124,490]],[22,23,[1294,490,1294,515]],[19,24,[1294,240,1294,265]],[15,25,[1294,590,1294,615]],[21,26,[330,90,330,40]],[6,27,[500,190,500,290]],[26,28,[160,40,160,15]],[15,29,[1294,590,1294,565]],[21,30,[330,90,330,140]],[19,31,[1294,240,1294,215]],[30,32,[160,140,160,115]],[22,33,[1294,490,1294,465]],[8,34,[1294,790,1294,765]],[2,35],[27,36,[330,290,330,240]],[11,37,[1294,690,1294,715]],[27,38,[330,290,330,340]],[20,39,[1294,40,1294,65]],[38,40,[160,340,160,315]],[26,41,[160,40,160,65]],[30,42,[160,140,160,165]],[11,43,[1294,690,1294,665]],[20,44,[1294,40,1294,15]],[14,45,[1294,140,1294,115]],[35,46,[500,640,500,540]],[46,47,[330,540,330,490]],[38,48,[160,340,160,365]],[47,49,[160,490,160,465]],[35,50,[500,640,500,740]],[36,51,[160,240,160,215]],[46,52,[330,540,330,590]],[50,53,[330,740,330,690]],[47,54,[160,490,160,515]],[52,55,[160,590,160,565]],[52,56,[160,590,160,615]],[36,57,[160,240,160,265]],[50,58,[330,740,330,790]],[58,59,[160,790,160,765]],[58,60,[160,790,160,815]],[53,61,[160,690,160,665]],[53,62,[160,690,160,715]]],"vp":[1.0,0.0,1451.0,830.0]}}
     * </p>
     * <p>
     * There are two subtrees that are placed above and below the root; Both subtrees consist of two subtrees that are placed
     * left and right of their local root
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface DelegatingNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class DelegatingNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.DefaultNodePlacer} instance with default settings using the given modification matrix and the
       * given node placers as delegates.
       * @param modificationMatrix the transformation matrix
       * @param placerUpperLeft the first delegate {@link } which places the left subtrees
       * @param placerLowerRight the second delegate {@link } which places the right subtrees
       * @see yfiles.tree.DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix,placerUpperLeft:yfiles.tree.INodePlacer,placerLowerRight:yfiles.tree.INodePlacer);
      /**
       * Returns all upper-left children of the local root node.
       * <p>
       * Which children belong to this list is determined using the {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.tree.DelegatingNodePlacer#PRIMARY_NODES_DP_KEY}. If no such {@link yfiles.algorithms.IDataProvider} is registered, the
       * first half of the children is returned.
       * </p>
       * <p>
       * These children are placed using the {@link yfiles.tree.INodePlacer} returned by {@link yfiles.tree.DelegatingNodePlacer#primaryPlacer}.
       * </p>
       * @param localRoot the local root node
       * @param graph the input graph
       * @returns the list of upper-left child {@link }s
       * @see yfiles.tree.DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
       */
      primaryChildren(localRoot:yfiles.algorithms.Node,graph:yfiles.layout.LayoutGraph):yfiles.algorithms.NodeList;
      /**
       * Returns all lower-right children of the local root node.
       * <p>
       * Which children belong to this list is determined using the {@link yfiles.algorithms.IDataProvider} registered with
       * {@link yfiles.tree.DelegatingNodePlacer#PRIMARY_NODES_DP_KEY}. If no such {@link yfiles.algorithms.IDataProvider} is registered, the
       * second half of the children is returned.
       * </p>
       * <p>
       * These children are placed using the {@link yfiles.tree.INodePlacer} returned by {@link yfiles.tree.DelegatingNodePlacer#secondaryPlacer}.
       * </p>
       * @param localRoot the local root node
       * @param graph the input graph
       * @returns the list of lower-right child {@link }s
       * @see yfiles.tree.DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
       */
      secondaryChildren(localRoot:yfiles.algorithms.Node,graph:yfiles.layout.LayoutGraph):yfiles.algorithms.NodeList;
      /**
       * Data provider key for dividing the children into two sets
       * <p>
       * If there is no {@link yfiles.algorithms.IDataProvider} registered with this key, half of the nodes are placed left of the root
       * and the remaining nodes are placed right of the root.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static PRIMARY_NODES_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets the {@link yfiles.tree.INodePlacer} instance that will arrange the lower-right part of the graph.
       * @see yfiles.tree.DelegatingNodePlacer#primaryPlacer
       * @see yfiles.tree.DelegatingNodePlacer#secondaryChildren
       * @type {yfiles.tree.INodePlacer}
       */
      secondaryPlacer:yfiles.tree.INodePlacer;
      /**
       * Gets the {@link yfiles.tree.INodePlacer} instance that will arrange the upper-left part of the graph.
       * @see yfiles.tree.DelegatingNodePlacer#secondaryPlacer
       * @see yfiles.tree.DelegatingNodePlacer#primaryChildren
       * @type {yfiles.tree.INodePlacer}
       */
      primaryPlacer:yfiles.tree.INodePlacer;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.DendrogramNodePlacer} arranges subtrees as dendrograms.
     * <p>
     * <b>Layout Style</b> This {@link yfiles.tree.INodePlacer} arranges nodes from top to bottom. It arranges subtrees in such a way that all subtrees of
     * a single local root are aligned with their bottom border. {@graph {"ann":{"s":[30,30],"d":1},"n":[[212.5,0],[400,70],[450,140],[300,210],[400,210],[450,210],[150,140],[25,70],[200,280],[150,280],[50,210],[450,280],[350,280],[25,140],[100,210],[400,280],[0,280],[300,280],[50,280],[100,280],[250,280]],"e":[[0,1,[227.5,50,415,50]],[1,2,[415,120,465,120]],[0,3,[227.5,50,315,50]],[1,4],[2,5],[0,6,[227.5,50,165,50]],[0,7,[227.5,50,40,50]],[6,8,[165,190,215,190]],[6,9],[5,11],[1,12,[415,120,365,120]],[7,13],[13,10,[40,190,65,190]],[6,14,[165,190,115,190]],[4,15],[13,16,[40,190,15,190]],[3,17],[10,18],[14,19],[0,20,[227.5,50,265,50]]],"vp":[0.0,0.0,480.0,310.0]}}
     * </p>
     * <p>
     * Note: This implementation does not support {@link yfiles.layout.PortConstraint}s.
     * </p>
     * @class
     * @implements {yfiles.tree.IFromSketchNodePlacer}
     */
    export interface DendrogramNodePlacer extends Object,yfiles.tree.IFromSketchNodePlacer{}
    export class DendrogramNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.DendrogramNodePlacer} instance with default settings.
       */
      constructor();
      /**
       * Creates a {@link yfiles.collections.IComparer.<T>} that sorts outgoing edges according to the x-coordinate of their target nodes.
       * <p>
       * This {@link yfiles.collections.IComparer.<T>} can be used for achieving a From Sketch layout.
       * </p>
       * @returns the {@link }
       * @see yfiles.tree.DendrogramNodePlacer#createFromSketchComparer
       */
      createComparer():yfiles.collections.IComparer<Object>;
      /**
       * Creates a {@link yfiles.collections.IComparer.<T>} for the edges which takes the initial x-coordinates of the nodes into account.
       * <p>
       * This implementation delegates to {@link yfiles.tree.DendrogramNodePlacer#createComparer}.
       * </p>
       * @returns the {@link } for the edges
       * @see yfiles.tree.DendrogramNodePlacer#createComparer
       */
      createFromSketchComparer():yfiles.collections.IComparer<Object>;
      /**
       * Creates an optional {@link yfiles.tree.IProcessor} for pre- and post-processing.
       * <p>
       * This implementation returns <code>null</code>. Hence, no pre- or post-processing is done.
       * </p>
       * @param layouter the current {@link } instance
       * @param graph the input graph
       * @param currentRoot the root node handled by this {@link }
       * @returns <code>null</code>
       */
      createProcessor(layouter:yfiles.tree.TreeLayout,graph:yfiles.layout.LayoutGraph,currentRoot:yfiles.algorithms.Node):yfiles.tree.IProcessor;
      /**
       * Provides {@link yfiles.tree.ParentConnectorDirection#NORTH} for all children.
       * <p>
       * Edges will always connect to the top of the child nodes.
       * </p>
       * @param localRoot the local root node
       * @param connectorMap the map that is used for storing the direction specifiers of the child nodes
       */
      determineChildConnectors(localRoot:yfiles.algorithms.Node,connectorMap:yfiles.algorithms.IDataMap):void;
      /**
       * Arranges the {@link yfiles.tree.SubtreeShape SubtreeShapes} of the local root and its children as a dendrogram.
       * @param nodeShapeProvider the {@link } for obtaining an initial shape of the root node
       * @param subtreeShapeProvider the {@link } for accessing the pre-calculated shapes of the subtrees
       * @param graph the input graph
       * @param localRoot the root of the subtree that should be arranged by this method
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the combined shape of the whole subtree rooted at the given local root node
       */
      placeSubtree(nodeShapeProvider:yfiles.algorithms.IDataProvider,subtreeShapeProvider:yfiles.algorithms.IDataProvider,graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
      /**
       * Gets or sets the minimum distance between the {@link yfiles.tree.SubtreeShape} of a root node and the bus connecting to all of
       * its children.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      minimumRootDistance:number;
      /**
       * Gets or sets the minimum horizontal distance between adjacent {@link yfiles.tree.SubtreeShape SubtreeShapes}.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      minimumSubtreeDistance:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.DoubleLineNodePlacer} arranges the children of a local root alternating in two lines.
     * <p>
     * <b>Layout Style</b> This {@link yfiles.tree.INodePlacer} arranges nodes from top to bottom. It places the subtrees in two horizontal lines below
     * the root. They are distributed in an alternating fashion, i.e., adjacent subtrees are in different lines.
     * </p>
     * <p>
     * This node placer is especially useful for leaves with a large width.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[141.67,30],"d":1},"n":[[298.33,0,60,30],[0,60],[85.83,99.9],[171.67,60],[257.5,99.9],[343.33,60],[429.17,99.9],[515,60]],"e":[[0,1,[70.83,15]],[0,2,[156.67,15]],[0,3,[242.5,15]],[0,4],[0,5,[414.17,15]],[0,6,[500,15]],[0,7,[585.83,15]]],"vp":[0.0,0.0,657.0,130.0]}}
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface DoubleLineNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class DoubleLineNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.DoubleLineNodePlacer} instance whose direction is specified by the given modification matrix.
       * @param modificationMatrix the transformation matrix for this {@link }
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a new {@link yfiles.tree.DoubleLineNodePlacer} instance with the default settings.
       */
      constructor();
      /**
       * Gets or sets the {@link yfiles.tree.RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes.
       * @type {yfiles.tree.RootNodeAlignment}
       */
      rootAlignment:yfiles.tree.RootNodeAlignment;
      /**
       * Gets or sets the ratio of the {@link yfiles.tree.RotatableNodePlacerBase#spacing spacing} between the root and the children in the upper line and the spacing between the
       * two lines of subtrees.
       * @type {number}
       */
      doubleLineSpacingRatio:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.tree.FreeNodePlacer} is a simple <em>pseudo</em>-placer.
     * <p>
     * The children of local root nodes are not moved. The {@link yfiles.tree.INodePlacer} just merges their rotated shapes and routes
     * the edges.
     * </p>
     * <p>
     * This {@link yfiles.tree.INodePlacer} may be useful if a local root node does not have any children or you want to place them
     * manually.
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface FreeNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class FreeNodePlacer {
      /**
       * Creates a new instance of {@link yfiles.tree.FreeNodePlacer} with default settings.
       */
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.GridNodePlacer} arranges the shapes of the children of a local root in a grid.
     * <p>
     * <b>Layout Style</b> The grid consists of a number of rows. The subtrees are assigned to the rows using a {@link yfiles.algorithms.IDataProvider}
     * that is registered with the graph with {@link yfiles.tree.GridNodePlacer#ROW_INDEX_DP_KEY}. {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,0],[50,140],[50,210],[50,70],[100,140],[100,210],[100,70],[150,140],[150,210],[150,70],[200,140],[200,210],[200,70]],"e":[[0,1,[40,15,40,120,65,120]],[0,2,[40,15,40,190,65,190]],[0,3,[40,15,40,50,65,50]],[0,4,[40,15,40,120,115,120]],[0,5,[40,15,40,190,115,190]],[0,6,[40,15,40,50,115,50]],[0,7,[40,15,40,120,165,120]],[0,8,[40,15,40,190,165,190]],[0,9,[40,15,40,50,165,50]],[0,10,[40,15,40,120,215,120]],[0,11,[40,15,40,190,215,190]],[0,12,[40,15,40,50,215,50]]],"vp":[0.0,0.0,230.0,240.0]}}
     * </p>
     * <p>
     * Child nodes placed in a grid using three rows and four columns
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface GridNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class GridNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.GridNodePlacer} instance with the given {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix} and {@link yfiles.tree.GridNodePlacer#rootAlignment root alignment}.
       * @param modificationMatrix the transformation matrix for this {@link }
       * @param rootAlignment the alignment of the local root with its subtrees
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix,rootAlignment:yfiles.tree.RootNodeAlignment);
      /**
       * Creates a new {@link yfiles.tree.GridNodePlacer} instance whose direction is specified by the given modification matrix.
       * @param modificationMatrix the transformation matrix for this {@link }
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a new {@link yfiles.tree.GridNodePlacer} with default settings.
       */
      constructor();
      /**
       * Data provider key for assigning nodes to the rows in the grid
       * <p>
       * If no {@link yfiles.algorithms.IDataProvider} is registered with this key, all nodes will be placed in the same row.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static ROW_INDEX_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Gets or sets the {@link yfiles.tree.RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes.
       * @type {yfiles.tree.RootNodeAlignment}
       */
      rootAlignment:yfiles.tree.RootNodeAlignment;
      static $class:yfiles.lang.Class;
    }
    /**
     * This {@link yfiles.tree.INodePlacer} places the children of a local root in groups.
     * <p>
     * The groups are determined by the grouping of the edges that connect the children to its root. The placement of the
     * groups is specified by a {@link yfiles.tree.INodePlacer} which will consider grouped children as one child. The children within
     * a group are placed by another {@link yfiles.tree.INodePlacer}.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[70.15,0,91.19,30],[0,91],[111,91],[201.5,91],[0,141],[111,141],[201.5,141]],"e":[[0,1,-30,15,0,0,[85.75,50,50,50,50,106]],[0,2,0,15,0,0,[115.75,50,91,50,91,106]],[0,3,30,15,0,0,[145.75,50,181.5,50,181.5,106]],[0,4,-30,15,0,0,[85.75,50,50,50,50,156]],[0,5,0,15,0,0,[115.75,50,91,50,91,156]],[0,6,30,15,0,0,[145.75,50,181.5,50,181.5,156]]],"vp":[0.0,0.0,232.0,171.0]}} There are three edge groups whose target nodes are placed above each other
     * </p>
     * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
     * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
     * @class
     * @implements {yfiles.tree.INodePlacer}
     */
    export interface GroupedNodePlacer extends Object,yfiles.tree.INodePlacer{}
    export class GroupedNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.GroupedNodePlacer} instance.
       * @param groupPlacer the {@link } which places the child groups
       * @param childPlacer the {@link } which places the children within their group
       */
      constructor(groupPlacer:yfiles.tree.INodePlacer,childPlacer:yfiles.tree.INodePlacer);
      /**
       * Creates a {@link yfiles.tree.IProcessor} that prepares the graph for edge grouping.
       * <p>
       * In a pre-processing step, the {@link yfiles.tree.IProcessor} inserts a dummy node for each edge group and connects it to the
       * local root and the edges of the according group. The child node placer is assigned to those dummy nodes and will arrange
       * them during the layout calculation.
       * </p>
       * <p>
       * The changes to the graph are restored in a post-processing step.
       * </p>
       * @param layouter the current {@link } instance
       * @param graph the input graph
       * @param currentRoot the root node handled by this {@link }
       * @returns the {@link } responsible for preparing the graph
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      createProcessor(layouter:yfiles.tree.TreeLayout,graph:yfiles.layout.LayoutGraph,currentRoot:yfiles.algorithms.Node):yfiles.tree.IProcessor;
      /**
       * Delegates to the {@link yfiles.tree.INodePlacer} responsible for arranging the nodes in one group.
       * @param localRoot the local root node
       * @param connectorMap the map used for storing the direction specifiers of the child nodes
       */
      determineChildConnectors(localRoot:yfiles.algorithms.Node,connectorMap:yfiles.algorithms.IDataMap):void;
      /**
       * Places the groups of {@link yfiles.tree.SubtreeShape}s according to their group IDs.
       * @param nodeShapeProvider the {@link } for obtaining an initial shape of the root node
       * @param subtreeShapeProvider the {@link } for accessing the pre-calculated shapes of the subtrees
       * @param graph the input graph
       * @param localRoot the root of the subtree that should be arranged by this method
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the merged {@link }s of the local root and its children
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      placeSubtree(nodeShapeProvider:yfiles.algorithms.IDataProvider,subtreeShapeProvider:yfiles.algorithms.IDataProvider,graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.tree.IFromSketchNodePlacer} arranges the subtrees while taking the initial locations of the nodes into account.
     * <p>
     * It provides a {@link yfiles.collections.IComparer.<T>} which sorts the outgoing edges of a node according to the initial coordinates of their target nodes.
     * Keeping the original order of nodes, this {@link yfiles.collections.IComparer.<T>} allows to incrementally add new nodes to the graph.
     * </p>
     * @interface
     * @implements {yfiles.tree.INodePlacer}
     */
    export interface IFromSketchNodePlacer extends Object,yfiles.tree.INodePlacer{
      /**
       * Creates an {@link yfiles.collections.IComparer.<T> edge comparator} which takes the initial coordinates of the nodes into account.
       * <p>
       * This {@link yfiles.collections.IComparer.<T>} defines the order of the outgoing edges of a node before the actual placement of the node's subtree is
       * calculated.
       * </p>
       * @returns the {@link } for the edges
       * @abstract
       */
      createFromSketchComparer():yfiles.collections.IComparer<Object>;
    }
    var IFromSketchNodePlacer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.tree.INodePlacer} is responsible for the arrangement of a local root node and all of its subtrees.
     * <p>
     * Subtrees are represented as {@link yfiles.tree.SubtreeShape SubtreeShape}s with a connecting edge to the root of the subtree.
     * </p>
     * <p>
     * Implementations of this interface have to:
     * </p>
     * <ul>
     * <li>
     * arrange the {@link yfiles.tree.SubtreeShape SubtreeShapes} and the root node (only the relative position matters, shapes can be placed at any absolute
     * location, {@link yfiles.tree.TreeLayout} will perform the final alignment).
     * </li>
     * <li>
     * perform the edge routing from the root node to all child nodes with respect to the connector provided by the
     * {@link yfiles.tree.SubtreeShape SubtreeShapes}
     * </li>
     * <li>calculate the union of the {@link yfiles.tree.SubtreeShape SubtreeShapes} and the shape of the root node as well as any added edges</li>
     * <li>initialize the connector of the root node with respect to the preferred connection direction</li>
     * <li>return the {@link yfiles.tree.SubtreeShape SubtreeShape} for the subtree including the local root</li>
     * </ul>
     * @interface
     */
    export interface INodePlacer extends Object{
      /**
       * Creates an optional {@link yfiles.tree.IProcessor} for pre- and post-processing.
       * <p>
       * This method is called by {@link yfiles.tree.TreeLayout} before arranging the {@link yfiles.tree.SubtreeShape SubtreeShapes}. If no pre- or post-processing is
       * needed, this method may return <code>null</code>.
       * </p>
       * @param layouter the current {@link } instance
       * @param graph the input graph
       * @param currentRoot the root node handled by this {@link }
       * @returns a {@link } instance or <code>null</code>
       * @abstract
       */
      createProcessor(layouter:yfiles.tree.TreeLayout,graph:yfiles.layout.LayoutGraph,currentRoot:yfiles.algorithms.Node):yfiles.tree.IProcessor;
      /**
       * Provides the direction of the connector to the {@link yfiles.tree.SubtreeShape SubtreeShape} for each child node.
       * <p>
       * This method is called by {@link yfiles.tree.TreeLayout} before the {@link yfiles.tree.SubtreeShape SubtreeShapes} for the children of the local root node are
       * calculated. The direction specifiers are stored in the given map for all child nodes of the local root.
       * </p>
       * @param localRoot the local root node
       * @param connectorMap the map that must be used for storing the direction specifiers of the child nodes
       * @abstract
       */
      determineChildConnectors(localRoot:yfiles.algorithms.Node,connectorMap:yfiles.algorithms.IDataMap):void;
      /**
       * Arranges the {@link yfiles.tree.SubtreeShape SubtreeShapes} of the local root and its children and routes the edges that connect them.
       * <p>
       * This method is the main method of the tree layout algorithm. It is called by {@link yfiles.tree.TreeLayout} in each recursion
       * step to place the local root in relation to its children. Their shapes will be merged into one
       * {@link yfiles.tree.SubtreeShape SubtreeShape}, which is returned by this method.
       * </p>
       * <p>
       * The {@link yfiles.tree.SubtreeShape SubtreeShape} for the local root can be obtained from the <code>nodeShapeProvider</code>. <code>subtreeShapeProvider</code> contains the {@link yfiles.tree.SubtreeShape SubtreeShapes}
       * assigned to the child nodes.
       * </p>
       * <p>
       * Due to the order in which the tree nodes are processed, it is guaranteed that at the time of the invocation of this
       * method the {@link yfiles.tree.SubtreeShape SubtreeShape} of every descendant node has already been calculated.
       * </p>
       * @param nodeShapeProvider the {@link } for obtaining an initial shape of the root node
       * @param subtreeShapeProvider the {@link } for accessing the pre-calculated shapes of the subtrees
       * @param graph the input graph
       * @param localRoot the root of the subtree that should be arranged by this method
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the combined shape of the whole subtree
       * @abstract
       */
      placeSubtree(nodeShapeProvider:yfiles.algorithms.IDataProvider,subtreeShapeProvider:yfiles.algorithms.IDataProvider,graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
    }
    var INodePlacer:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.tree.IPortAssignment} places the ports of edges connecting to the same node.
     * <p>
     * Classes implementing this interface are responsible for the assignment of the edges' ports. {@link yfiles.tree.INodePlacer}
     * instances need to obey the currently set ports.
     * </p>
     * @interface
     */
    export interface IPortAssignment extends Object{
      /**
       * Places the ports of edges connecting to the given node.
       * <p>
       * This method is called by {@link yfiles.tree.TreeLayout} before the actual layout calculation takes place. It assigns both the
       * single incoming edge's target port as well as all source ports of all outgoing child edges.
       * </p>
       * @param graph the graph
       * @param node the node whose adjacent edges' ports should be set
       * @abstract
       */
      assignPorts(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):void;
    }
    var IPortAssignment:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.tree.IProcessor} provides the possibility for each {@link yfiles.tree.INodePlacer} to prepare (and clean up) the graph
     * for the children of the local root.
     * <p>
     * It is created by {@link yfiles.tree.INodePlacer#createProcessor} and called by the {@link yfiles.tree.TreeLayout} before and after the
     * actual arrangement of the nodes.
     * </p>
     * <p>
     * A {@link yfiles.tree.IProcessor} may change values within the
     * {@link yfiles.algorithms.IDataMap}s for its children. It can also temporarily change the graph by hiding nodes or adding dummy
     * nodes.
     * </p>
     * @interface
     */
    export interface IProcessor extends Object{
      /**
       * This method is called by the {@link yfiles.tree.TreeLayout} after the layout has finished.
       * <p>
       * It can be used for cleanup tasks.
       * </p>
       * <p>
       * Changes on the graph structure that have been made in {@link yfiles.tree.IProcessor#preProcess} can be undone here.
       * </p>
       * @abstract
       */
      postProcess():void;
      /**
       * Performs preparations for the actual tree layout in a pre-processing step.
       * <p>
       * This method is called by {@link yfiles.tree.TreeLayout} before the {@link yfiles.tree.INodePlacer}s are called recursively. A {@link yfiles.tree.IProcessor}
       * may change some settings within this method. This can be used for specifying values for the children of the local root,
       * manipulate the graph structure or do anything else that needs to be done.
       * </p>
       * @param nodePlacerDataProvider the map that stores a {@link } instance for each node
       * @param portAssignmentDataProvider The map that stores a {@link } instance for each node
       * @param childComparatorProvider the map that stores a {@link } instance for each node
       * @abstract
       */
      preProcess(nodePlacerDataProvider:yfiles.algorithms.IDataMap,portAssignmentDataProvider:yfiles.algorithms.IDataMap,childComparatorProvider:yfiles.algorithms.IDataMap):void;
    }
    var IProcessor:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A {@link yfiles.tree.LayeredNodePlacer} arranges the nodes of a subtree respecting layers.
     * <p>
     * <b>Layout Style</b> Nodes that have the same depth in the tree will be placed in the same horizontal layer. They will be vertically
     * {@link yfiles.tree.LayeredNodePlacer#verticalAlignment aligned}. {@graph {"ann":{"s":[30,30],"d":1},"n":[[245.52,0],[69.46,77.5],[7.5,162.5],[181.42,240,45,45],[57.5,155,45,45],[0,240,45,45],[246.42,77.5],[406.58,70,45,45],[181.42,155,45,45],[246.42,155,45,45],[375.88,247.5],[7.5,332.5],[122.5,158.04,38.92,38.92],[409.62,158.04,38.92,38.92],[311.42,162.5],[65,247.5],[443.38,243.04,38.92,38.92],[138.92,332.5],[338.92,325,45,45],[311.42,247.5],[403.92,328.04,38.92,38.92],[188.92,332.5],[238.92,332.5]],"e":[[0,1,[260.52,42,84.46,42]],[1,2,[84.46,127,22.5,127]],[1,4,[84.46,127,80,127]],[2,5],[0,6,[260.52,42,261.42,42]],[0,7,[260.52,42,429.08,42]],[6,8,[261.42,127,203.92,127]],[8,3],[6,9,[261.42,127,268.92,127]],[5,11],[1,12,[84.46,127,141.96,127]],[7,13],[13,10,[429.08,212,390.88,212]],[6,14,[261.42,127,326.42,127]],[4,15],[13,16,[429.08,212,462.83,212]],[3,17,[203.92,297,153.92,297]],[10,18,[390.88,297,361.42,297]],[14,19],[10,20,[390.88,297,423.38,297]],[3,21],[3,22,[203.92,297,253.92,297]]],"vp":[0.0,0.0,483.0,370.0]}}
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface LayeredNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class LayeredNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.LayeredNodePlacer} instance whose direction is specified by the given modification matrix.
       * <p>
       * The given ID identifies different {@link yfiles.tree.LayeredNodePlacer} instances that share the same layer heights.
       * </p>
       * @param modificationMatrix the transformation matrix for this {@link }
       * @param id the {@link } ID
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix,id:Object);
      /**
       * Creates a new {@link yfiles.tree.LayeredNodePlacer} instance with the default settings.
       */
      constructor();
      /**
       * Gets or sets whether or not the layering is done dendrogram-like.
       * <p>
       * When enabled, leaves are placed in the same layer and remaining tree nodes are assigned to a layer as close to their
       * children as possible.
       * </p>
       * @type {boolean}
       */
      dendrogramStyle:boolean;
      /**
       * Gets or sets the {@link yfiles.tree.RootNodeAlignment RootAlginment} that defines how the root node is aligned with its children and their shapes.
       * @type {yfiles.tree.RootNodeAlignment}
       */
      rootAlignment:yfiles.tree.RootNodeAlignment;
      /**
       * Gets or sets the relative vertical alignment of nodes within their respective layers.
       * <p>
       * A value of <code>0</code> means that nodes are top-aligned; a value of <code>0.5</code> means that nodes are center-aligned; a value of <code>1</code> means
       * that nodes are bottom-aligned;
       * </p>
       * <p>
       * Values for a valid vertical alignment are in the range <code>[0..1]</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified alignment is outside <code>[0..1]</code>
       * @type {number}
       */
      verticalAlignment:number;
      /**
       * Gets the ID of this {@link yfiles.tree.LayeredNodePlacer}.
       * <p>
       * The ID is used for identifying
       * {@link yfiles.tree.LayeredNodePlacer}s that share information about the layer height. Using different IDs allows aligned layouts
       * within different subtrees.
       * </p>
       * @type {Object}
       */
      id:Object;
      /**
       * Gets or sets the distance between two adjacent layers.
       * <p>
       * The spacing needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified spacing is negative
       * @type {number}
       */
      layerSpacing:number;
      /**
       * Gets or sets the routing style for the edges in the subtree.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified routing style specifier is unknown
       * @type {yfiles.tree.LayeredRoutingStyle}
       */
      routingStyle:yfiles.tree.LayeredRoutingStyle;
      /**
       * Gets or sets the vertical bus alignment for orthogonally routed edges.
       * <p>
       * The bus alignment determines the relative position of an edge bus in between two subsequent layers of nodes.
       * </p>
       * <p>
       * Values for a valid vertical alignment are in the range <code>[0..1]</code>, such that:
       * </p>
       * <ul>
       * <li>A value of <code>0.0</code> places the bus at the top, right below the parent node.</li>
       * <li>A value of <code>0.5</code> places the bus in the middle between parent and child nodes.</li>
       * <li>A value of <code>1.0</code> places the bus at the bottom, right above the child nodes.</li>
       * </ul>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified alignment is outside <code>[0..1]</code>
       * @type {number}
       */
      busAlignment:number;
      /**
       * Gets or sets whether or not straight edges are extended to avoid label overlaps.
       * <p>
       * If enabled, polyline connectors between the parent and its children are added to the according subtree shape. Node or
       * edge labels will not be crossed.
       * </p>
       * @type {boolean}
       */
      polylineLabeling:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.tree.LeafNodePlacer} creates a {@link yfiles.tree.SubtreeShape SubtreeShape} for a leaf node and routes its incoming edge at the target side.
     * @class
     * @implements {yfiles.tree.INodePlacer}
     */
    export interface LeafNodePlacer extends Object,yfiles.tree.INodePlacer{}
    export class LeafNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.LeafNodePlacer} instance with default settings.
       */
      constructor();
      /**
       * Creates an optional {@link yfiles.tree.IProcessor} for pre- and post-processing.
       * <p>
       * This implementation returns <code>null</code>. Hence, no pre- or post-processing is done.
       * </p>
       * @param layouter the current {@link } instance
       * @param graph the input graph
       * @param currentRoot the root node handled by this {@link }
       * @returns <code>null</code>
       */
      createProcessor(layouter:yfiles.tree.TreeLayout,graph:yfiles.layout.LayoutGraph,currentRoot:yfiles.algorithms.Node):yfiles.tree.IProcessor;
      /**
       * Does nothing because leaf nodes don't have children so there are no child connectors.
       * @param localRoot the local root node
       * @param connectorMap the map that must be used for storing the direction specifiers of the child nodes
       */
      determineChildConnectors(localRoot:yfiles.algorithms.Node,connectorMap:yfiles.algorithms.IDataMap):void;
      /**
       * Creates a {@link yfiles.tree.SubtreeShape} for the local root node (a leaf node in this case) and adds target points to the
       * incoming edge.
       * <p>
       * The incoming edge is routed according to possibly assigned {@link yfiles.layout.PortConstraint}s.
       * </p>
       * @param nodeShapeProvider the {@link } for obtaining an initial shape of the root node
       * @param subtreeShapeProvider the {@link } for accessing the pre-calculated shapes of the subtrees
       * @param graph the input graph
       * @param localRoot the root of the subtree that should be arranged by this method
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the {@link } of the leaf node including the connector
       */
      placeSubtree(nodeShapeProvider:yfiles.algorithms.IDataProvider,subtreeShapeProvider:yfiles.algorithms.IDataProvider,graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.LeftRightNodePlacer} arranges the children in a subtree on the left and the right of a vertical bus.
     * <p>
     * <b>Layout Style</b> {@graph {"ann":{"s":[30,30],"d":1},"n":[[35,0],[0,50],[70,50],[0,100],[70,100],[0,150],[70,150],[0,200],[70,200],[35,250]],"e":[[0,1,[50,65]],[0,2,[50,65]],[0,3,[50,115]],[0,4,[50,115]],[0,5,[50,165]],[0,6,[50,165]],[0,7,[50,215]],[0,8,[50,215]],[0,9]],"vp":[0.0,0.0,100.0,280.0]}}
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface LeftRightNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class LeftRightNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.LeftRightNodePlacer} instance using the given modification matrix.
       * @param modificationMatrix the transformation matrix for this {@link }
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a new {@link yfiles.tree.LeftRightNodePlacer} instance with default settings.
       */
      constructor();
      /**
       * Creates a special {@link yfiles.algorithms.IDataProvider} that determines whether a given node is placed left or right of the
       * parent node.
       * <p>
       * This {@link yfiles.algorithms.IDataProvider} can be registered with the graph with key
       * {@link yfiles.tree.LeftRightNodePlacer#LEFT_RIGHT_DP_KEY}. It calculates the side with respect to the initial coordinates of the
       * nodes in the graph, considering the orientation of the subtree.
       * </p>
       * <p>
       * If a node is assigned to the left side, this {@link yfiles.algorithms.IDataProvider} will return <code>true</code>. For the right side, it
       * will return <code>false</code>.
       * </p>
       * @param nodePlacerMap a {@link } containing the corresponding {@link }s for the node
       * @returns 
       * @static
       */
      static createLeftRightDataProvider(nodePlacerMap:yfiles.algorithms.IDataProvider):yfiles.algorithms.IDataProvider;
      /**
       * Data provider key for assigning nodes to the left or the right side of the bus
       * <p>
       * If no {@link yfiles.algorithms.IDataProvider} is specified using this key, the nodes will be placed alternating from left to
       * right.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static LEFT_RIGHT_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * Gets or sets the horizontal distance between nodes or subtrees.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      horizontalDistance:number;
      /**
       * Gets or sets the vertical distance between nodes or subtrees.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is negative
       * @type {number}
       */
      verticalDistance:number;
      /**
       * Sets a uniform horizontal and vertical spacing between subtrees.
       * <p>
       * This method sets the given distance value for the {@link yfiles.tree.LeftRightNodePlacer#horizontalDistance horizontal spacing} and for the {@link yfiles.tree.LeftRightNodePlacer#verticalDistance vertical spacing}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified spacing value is negative
       * @see yfiles.tree.LeftRightNodePlacer#horizontalDistance
       * @see yfiles.tree.LeftRightNodePlacer#verticalDistance
       * @type {number}
       */
      spacing:number;
      /**
       * Gets or sets whether or not the last child is placed centered, aligned with the bus and below the other child nodes.
       * <p>
       * When enabled, the last child will always be centered independent of the number of siblings.
       * </p>
       * @type {boolean}
       */
      placeLastOnBottom:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.MultiParentDescriptor} provides style information for multi-parent structures.
     * <p>
     * This class is used by {@link yfiles.tree.TreeLayout} to determine the desired layout of nodes that constitute a multi-parent
     * structure. All nodes of such a structure are placed side by side and the incident edges are routed over common points
     * for incoming edges and for outgoing edges.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[40,40],"d":1},"n":[[411.25,0],[682.5,85],[210,155],[612.5,165],[35,225],[577.5,235],[385,320.5],[682.5,155,40,60],[70,305],[210,85],[0,295,40,60],[0,385],[210,225],[700,320.5],[175,295],[245,295],[315,225],[752.5,165],[420,405.5],[647.5,235],[752.5,235],[770,406],[140,380.5,40,60],[210,390.5],[315,295,40,60],[490,405.5],[175,485.5],[455,320.5],[481.25,0],[700,406],[560,406],[630,406],[560,491],[630,491],[525,320.5],[280,380.5,40,60],[70,390.5]],"e":[[0,1,[431.25,55.5,466.25,55.5,466.25,70,702.5,70]],[1,3,[702.5,140,632.5,140]],[2,4,[230,210,55,210]],[3,5,[632.5,220,597.5,220]],[5,6,[597.5,290,475,290,475,305,405,305]],[1,7],[4,8,[55,280,90,280]],[0,9,[431.25,55.5,466.25,55.5,466.25,70,230,70]],[9,2],[4,10,[55,280,20,280]],[10,11],[2,12],[5,13,[597.5,290,720,290]],[12,14,[230,280,195,280]],[12,15,[230,280,265,280]],[2,16,[230,210,335,210]],[1,17,[702.5,140,772.5,140]],[6,18,[405,376,475,376,475,390.5,440,390.5]],[3,19,[632.5,220,667.5,220]],[17,20],[13,21,[720,375.5,790,375.5]],[14,22,[195,365,160,365]],[14,23,[195,365,230,365]],[16,24],[6,25,[405,376,475,376,475,390.5,510,390.5]],[22,26,[160,456,195,456]],[23,26,[230,456,195,456]],[5,27,[597.5,290,475,290]],[27,18,[475,390.5,440,390.5]],[27,25,[475,390.5,510,390.5]],[28,9,[501.25,55.5,466.25,55.5,466.25,70,230,70]],[28,1,[501.25,55.5,466.25,55.5,466.25,70,702.5,70]],[13,29],[13,30,[720,375.5,615,375.5,615,390.5,580,390.5]],[13,31,[720,375.5,615,375.5,615,390.5,650,390.5]],[30,32,[580,461.5,615,461.5,615,476,580,476]],[31,32,[650,461.5,615,461.5,615,476,580,476]],[30,33,[580,461.5,615,461.5,615,476,650,476]],[31,33,[650,461.5,615,461.5,615,476,650,476]],[5,34,[597.5,290,475,290,475,305,545,305]],[34,18,[545,376,475,376,475,390.5,440,390.5]],[34,25,[545,376,475,376,475,390.5,510,390.5]],[14,35,[195,365,300,365]],[35,26,[300,456,195,456]],[14,36,[195,365,90,365]],[36,26,[90,456,195,456]]],"vp":[0.0,0.0,810.0,531.0]}}
     * </p>
     * <p>
     * A tree graph containing multi-parent structures
     * </p>
     * @see yfiles.tree.TreeLayout#MULTI_PARENT_DESCRIPTOR_DP_KEY
     * @class
     */
    export interface MultiParentDescriptor extends Object{}
    export class MultiParentDescriptor {
      /**
       * Creates a new {@link yfiles.tree.MultiParentDescriptor} instance with default settings.
       */
      constructor();
      /**
       * Gets or sets the minimum distance between two nodes belonging to the same multi-parent structure.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given node distance is negative
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets the minimum distance between the nodes of a multi-parent structure and the bus connecting these nodes.
       * <p>
       * In case the edge style is not orthogonal, the bus only consists of a common point.
       * </p>
       * <p>
       * The minimum distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given bus distance is negative
       * @type {number}
       */
      minimumBusDistance:number;
      /**
       * Gets or sets the relative vertical alignment of nodes belonging to the same multi-parent structure.
       * <p>
       * A value of <code>0</code> means that nodes are top-aligned; a value of <code>1</code> means that nodes are bottom-aligned; a value of <code>0.5</code> means
       * that nodes are center-aligned.
       * </p>
       * <p>
       * Values outside the interval <code>[0,1]</code> will result in a compact node placement with unaligned nodes, especially when labels
       * are considered.
       * </p>
       * @type {number}
       */
      verticalAlignment:number;
      /**
       * Gets or sets the routing style for edges that connect to nodes which constitute a multi-parent structure.
       * @throws {Stubs.Exceptions.ArgumentError} if the given routing style is unknown
       * @type {yfiles.tree.MultiParentRoutingStyle}
       */
      edgeStyle:yfiles.tree.MultiParentRoutingStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.NodeOrderComparer} compares edges according to a specified order.
     * <p>
     * It obtains the order from a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
     * {@link yfiles.tree.NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY}.
     * </p>
     * <p>
     * Various tree layout algorithms can use this {@link yfiles.collections.IComparer.<T>} to influence the resulting layout. To achieve this, use this comparator and
     * assign it to the comparator property of the layout algorithm classes.
     * </p>
     * @see yfiles.tree.ClassicTreeLayout#comparer
     * @see yfiles.tree.TreeLayout#defaultOutEdgeComparer
     * @see yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY
     * @see yfiles.tree.BalloonLayout#comparer
     * @see yfiles.tree.AspectRatioTreeLayout#comparer
     * @see yfiles.tree.NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY
     * @class
     * @implements {yfiles.collections.IComparer.<Object>}
     */
    export interface NodeOrderComparer extends Object,yfiles.collections.IComparer<Object>{}
    export class NodeOrderComparer {
      /**
       * Creates a new {@link yfiles.tree.NodeOrderComparer} instance.
       */
      constructor();
      /**
       * Compares two edges according to their index in the order specified using a {@link yfiles.algorithms.IDataProvider}.
       * <p>
       * Target nodes for which the {@link yfiles.algorithms.IDataProvider} registered with {@link yfiles.tree.NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY}
       * returns <code>null</code> or an {@link Object} that does not implement {@link yfiles.lang.IComparable} are treated as being smaller than other target
       * nodes.
       * </p>
       * @param edge1 the first edge
       * @param edge2 the second edge
       * @returns 
       * @see yfiles.tree.NodeOrderComparer#NODE_ORDER_COMPARABLE_DP_KEY
       */
      compare(edge1:Object,edge2:Object):number;
      /**
       * Data provider key for registering an order for each node in the graph
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.lang.IComparable>}
       */
      static NODE_ORDER_COMPARABLE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.lang.IComparable>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Abstract base class for implementations of the {@link yfiles.tree.INodePlacer} interface.
     * <p>
     * This class provides convenience methods for sub-tasks that are often used during the arrangement of a local root and its
     * children. It also offers prepared implementations for the methods of the {@link yfiles.tree.INodePlacer} interface.
     * </p>
     * <p>
     * Subclasses just need to override the following two abstract methods: {@link yfiles.tree.NodePlacerBase#placeSubtreeOfNode} and
     * {@link yfiles.tree.NodePlacerBase#determineChildConnectors}.
     * </p>
     * @class
     * @implements {yfiles.tree.INodePlacer}
     * @implements {yfiles.lang.ICloneable}
     */
    export interface NodePlacerBase extends Object,yfiles.tree.INodePlacer,yfiles.lang.ICloneable{}
    export class NodePlacerBase {
      /**
       * Creates a new instance of {@link yfiles.tree.NodePlacerBase}.
       */
      constructor();
      /**
       * Returns a clone of this {@link yfiles.tree.INodePlacer} instance.
       * @returns an exact copy of this {@link } instance
       */
      clone():Object;
      /**
       * Creates an optional {@link yfiles.tree.IProcessor} for pre- and post-processing.
       * <p>
       * This implementation returns <code>null</code>. Hence, no pre- or post-processing will be done.
       * </p>
       * @param layouter the current {@link } instance
       * @param graph the input graph
       * @param currentRoot the root node handled by this {@link }
       * @returns <code>null</code>
       */
      createProcessor(layouter:yfiles.tree.TreeLayout,graph:yfiles.layout.LayoutGraph,currentRoot:yfiles.algorithms.Node):yfiles.tree.IProcessor;
      /**
       * Determines a connector direction for the given child node.
       * <p>
       * This method is used in {@link yfiles.tree.NodePlacerBase#determineChildConnectors} and must be implemented by subclasses.
       * </p>
       * @param child the child node
       * @returns a direction specifier as defined in the {@link } interface
       * @protected
       * @abstract
       */
      determineChildConnector(child:yfiles.algorithms.Node):yfiles.tree.ParentConnectorDirection;
      /**
       * Provides the direction of the connector to the {@link yfiles.tree.SubtreeShape SubtreeShape} for each child node.
       * <p>
       * This implementation uses {@link yfiles.tree.NodePlacerBase#determineChildConnector} to retrieve the direction for the
       * connectors.
       * </p>
       * @param localRoot the local root node
       * @param connectorMap the map that must be used for storing the direction specifiers of the child nodes
       */
      determineChildConnectors(localRoot:yfiles.algorithms.Node,connectorMap:yfiles.algorithms.IDataMap):void;
      /**
       * Returns the {@link yfiles.tree.SubtreeShape} for the given node.
       * <p>
       * This is a convenience method that retrieves a {@link yfiles.tree.SubtreeShape} for a single node from the corresponding
       * {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * <p>
       * This method is used in {@link yfiles.tree.NodePlacerBase#placeSubtreeOfNode} for retrieving the {@link yfiles.tree.SubtreeShape}s for
       * the local root nodes. It may be overridden to return a custom implementation of {@link yfiles.tree.SubtreeShape}.
       * </p>
       * @param node the node
       * @returns the {@link } for the given node
       * @protected
       */
      getNodeShape(node:yfiles.algorithms.Node):yfiles.tree.SubtreeShape;
      /**
       * Returns the {@link yfiles.tree.SubtreeShape} for the subtree rooted at the given node.
       * <p>
       * This is a convenience method that retrieves a {@link yfiles.tree.SubtreeShape} of a complete subtree from the corresponding
       * {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * <p>
       * This method is used in {@link yfiles.tree.NodePlacerBase#placeSubtreeOfNode} for retrieving the {@link yfiles.tree.SubtreeShape}s for
       * the subtrees that are arranged. It may be overridden to return a custom implementation of {@link yfiles.tree.SubtreeShape}.
       * </p>
       * @param node the local root node
       * @returns the {@link } of the subtree rooted at the given local root node
       * @protected
       */
      getSubtreeShape(node:yfiles.algorithms.Node):yfiles.tree.SubtreeShape;
      /**
       * Initializes the local data structures and then delegates the work to the abstract variant of this method
       * ({@link yfiles.tree.NodePlacerBase#placeSubtreeOfNode})
       * @param nodeShapeProvider the {@link } for obtaining an initial shape of the root node
       * @param subtreeShapeProvider the {@link } for accessing the pre-calculated shapes of the subtrees
       * @param graph the input graph
       * @param localRoot the root of the subtree that should be arranged by this method
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the merged {@link } for the subtree rooted at the local root node
       * @see yfiles.tree.NodePlacerBase#placeSubtreeOfNode
       */
      placeSubtree(nodeShapeProvider:yfiles.algorithms.IDataProvider,subtreeShapeProvider:yfiles.algorithms.IDataProvider,graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
      /**
       * Arranges the local root relative to the {@link yfiles.tree.SubtreeShape} of its children.
       * <p>
       * This is the main method that must be implemented by subclasses. It is called by
       * {@link yfiles.tree.NodePlacerBase#placeSubtree}.
       * </p>
       * @param localRoot the local root node
       * @param parentConnectorDirection the direction of the connector
       * @returns a {@link } instance that describes the shape of the whole subtree rooted at the given local root
       * @protected
       * @abstract
       */
      placeSubtreeOfNode(localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
      /**
       * The graph instance this class is working on.
       * @protected
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.tree.RootNodeAlignment} describes how a local root node is arranged in relation to its children.
     * <p>
     * The actual placement depends on the orientation of the subtree rooted at the current local root node. All descriptions
     * of the provided alignments refer to the top-to-bottom orientation described by
     * {@link yfiles.tree.RotatableNodePlacerMatrix#DEFAULT}. In this case, the subtrees are placed below the local root and the local
     * root is aligned horizontally above them.
     * </p>
     * @class
     */
    export interface RootNodeAlignment extends Object{}
    export class RootNodeAlignment {
      /**
       * Calculates the common bounds of the given subtree shapes.
       * <p>
       * This method can be used when placing the local root above its subtrees in method
       * {@link yfiles.tree.RootNodeAlignment#placeParentHorizontal}.
       * </p>
       * @param shapes a list of rotated subtree shapes
       * @returns the rectangle describing the bounds containing all given shapes
       * @protected
       * @static
       */
      static getBounds(shapes:yfiles.collections.IList<Object>):yfiles.algorithms.Rectangle2D;
      /**
       * Moves the shape of the local root to a suitable position relative to the children's shapes.
       * <p>
       * The movement should only be done on the x-axis (horizontally).
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.RotatableNodePlacerBase}s when they {@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode place the subtrees}.
       * </p>
       * @param rootShape the shape of the local root node that will be moved
       * @param shapes a list of rotated subtree shapes, representing the shapes of the child nodes
       * @param shapeBounds accumulated bounds of the shapes of all child nodes
       * @param spacing the spacing value that should be used
       * @protected
       * @abstract
       */
      placeParentHorizontal(rootShape:yfiles.tree.RotatedSubtreeShape,shapes:yfiles.collections.IList<Object>,shapeBounds:yfiles.algorithms.Rectangle2D,spacing:number):void;
      /**
       * Horizontal {@link yfiles.tree.RootNodeAlignment} at the center of the subtrees.
       * <p>
       * The root node is placed centered above its whole subgraph. To determine the alignment, only {@link yfiles.layout.INodeLayout}s
       * are considered. Node labels are not included although they are also contained in {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static CENTER:yfiles.tree.RootNodeAlignment;
      /**
       * Horizontal {@link yfiles.tree.RootNodeAlignment} at the median child node.
       * <p>
       * The root node is placed above the median of its children. To determine the alignment, only {@link yfiles.layout.INodeLayout}s
       * are considered. Node labels are not included although they are also contained in {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static MEDIAN:yfiles.tree.RootNodeAlignment;
      /**
       * Horizontal {@link yfiles.tree.RootNodeAlignment} left of the children.
       * <p>
       * The local root node leaves a distance specified by the given spacing to the leftmost child. To determine the alignment,
       * only {@link yfiles.layout.INodeLayout}s are considered. Node labels are not included although they are also contained in
       * {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static LEADING:yfiles.tree.RootNodeAlignment;
      /**
       * Horizontal {@link yfiles.tree.RootNodeAlignment} right of the children.
       * <p>
       * The root node leaves a distance specified by the given spacing to the rightmost child. To determine the alignment, only
       * {@link yfiles.layout.INodeLayout}s are considered. Node labels are not included although they are also contained in
       * {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static TRAILING:yfiles.tree.RootNodeAlignment;
      /**
       * Horizontal {@link yfiles.tree.RootNodeAlignment} at the left side.
       * <p>
       * The root node is placed left aligned with its leftmost child. To determine the alignment, only
       * {@link yfiles.layout.INodeLayout}s are considered. Node labels are not included although they are also contained in
       * {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static LEFT:yfiles.tree.RootNodeAlignment;
      /**
       * Horizontal {@link yfiles.tree.RootNodeAlignment} at the right side.
       * <p>
       * The root node is placed right aligned with its rightmost child. To determine the alignment, only
       * {@link yfiles.layout.INodeLayout}s are considered. Node labels are not included although they are also contained in
       * {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static RIGHT:yfiles.tree.RootNodeAlignment;
      /**
       * Horizontal alignment at the center of the children.
       * <p>
       * The root node is placed centered over its direct children. To determine the alignment, only {@link yfiles.layout.INodeLayout}s
       * are considered. Node labels are not included although they are also contained in {@link yfiles.tree.SubtreeShape}s.
       * </p>
       * @const
       * @static
       * @type {yfiles.tree.RootNodeAlignment}
       */
      static CENTER_OVER_CHILDREN:yfiles.tree.RootNodeAlignment;
      /**
       * An array containing all supported {@link yfiles.tree.RootNodeAlignment}s.
       * @const
       * @static
       * @type {Array.<yfiles.tree.RootNodeAlignment>}
       */
      static ALL:yfiles.tree.RootNodeAlignment[];
      static $class:yfiles.lang.Class;
    }
    /**
     * A matrix describes the rotation of the subtree handled by {@link yfiles.tree.RotatableNodePlacerBase}.
     * <p>
     * Subtrees may be rotated or mirrored. Also, matrices offering a combination of rotation and mirroring are provided.
     * </p>
     * @class
     */
    export interface RotatableNodePlacerMatrix extends Object{}
    export class RotatableNodePlacerMatrix {
      /**
       * Compares the values of the current {@link yfiles.tree.RotatableNodePlacerMatrix} with the values of the given
       * {@link yfiles.tree.RotatableNodePlacerMatrix}.
       * @param other the matrix
       * @returns <code>true</code> if this matrix and the given other matrix contain the same values, <code>false</code> otherwise
       */
      equalValues(other:yfiles.tree.RotatableNodePlacerMatrix):boolean;
      /**
       * Multiplies this {@link yfiles.tree.RotatableNodePlacerMatrix} with the given other {@link yfiles.tree.RotatableNodePlacerMatrix}.
       * @param other the other matrix
       * @returns the product of both matrices
       */
      multiply(other:yfiles.tree.RotatableNodePlacerMatrix):yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * The identity matrix which does not apply any rotation or mirroring.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static DEFAULT:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a counter-clockwise rotation by 90 degrees.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static ROT90:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a rotation by 180 degrees.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static ROT180:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a counter-clockwise rotation by 270 degrees.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static ROT270:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a horizontal mirroring.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static MIR_HOR:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a vertical mirroring.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static MIR_VERT:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a horizontal mirroring and a counter-clockwise rotation by 90 degrees.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static MIR_HOR_ROT90:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * A matrix which applies a vertical mirroring and a counter-clockwise rotation by 90 degrees.
       * @const
       * @static
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      static MIR_VERT_ROT90:yfiles.tree.RotatableNodePlacerMatrix;
      static $class:yfiles.lang.Class;
    }
    /**
     * Abstract base class for {@link yfiles.tree.INodePlacer}s that support rotations of the subtrees.
     * <p>
     * The direction specified in this {@link yfiles.tree.RotatableNodePlacerBase} (defined by a
     * {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix transformation matrix}) rotates the subtree that is placed. This is the final direction of this subtree. Rotations are
     * not accumulated.
     * </p>
     * <p>
     * {@link yfiles.tree.INodePlacer}s of this type will only implement the default direction (i.e. top-to-bottom). The other
     * directions are calculated using the specified modification {@link yfiles.tree.RotatableNodePlacerMatrix}. The direction used by
     * the implementing
     * {@link yfiles.tree.INodePlacer}s is called the model direction while the actual direction in the graph is the real direction.
     * </p>
     * <p>
     * Note that using rotatable {@link yfiles.tree.INodePlacer}s has some pitfalls. Calculations must be aware of the rotations.
     * Especially operations on {@link yfiles.algorithms.BorderLine}s should not be called directly (e.g.
     * {@link yfiles.algorithms.BorderLine#mergeWithMin}, {@link yfiles.algorithms.BorderLine#mergeWithMax}). The corresponding methods of {@link yfiles.tree.RotatableNodePlacerBase}
     * already support the different directions.
     * </p>
     * <p>
     * Subclasses just need to override the following two abstract methods: {@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode}
     * and {@link yfiles.tree.RotatableNodePlacerBase#determineChildConnectors}.
     * </p>
     * @class
     * @implements {yfiles.tree.IFromSketchNodePlacer}
     */
    export interface RotatableNodePlacerBase extends Object,yfiles.tree.IFromSketchNodePlacer{}
    export class RotatableNodePlacerBase {
      /**
       * Creates a new instance of {@link yfiles.tree.RotatableNodePlacerBase} whose direction is specified by the given modification
       * matrix.
       * @param modificationMatrix the transformation matrix for the {@link }
       * @protected
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a {@link yfiles.collections.IComparer.<T>} for outgoing edges that takes the rotation into account.
       * @returns the {@link } that considers the rotation
       */
      createComparer():yfiles.collections.IComparer<Object>;
      /**
       * Delegates to {@link yfiles.tree.RotatableNodePlacerBase#createComparer}.
       * @returns the From Sketch {@link }
       * @see yfiles.tree.RotatableNodePlacerBase#createComparer
       */
      createFromSketchComparer():yfiles.collections.IComparer<Object>;
      /**
       * Creates an optional {@link yfiles.tree.IProcessor} for pre- and post-processing.
       * <p>
       * This implementation returns <code>null</code>. Hence, no pre- or post-processing is done.
       * </p>
       * @param layouter the current {@link } instance
       * @param graph the input graph
       * @param currentRoot the root node handled by this {@link }
       * @returns <code>null</code>
       */
      createProcessor(layouter:yfiles.tree.TreeLayout,graph:yfiles.layout.LayoutGraph,currentRoot:yfiles.algorithms.Node):yfiles.tree.IProcessor;
      /**
       * Determines a connector direction for the given child node.
       * <p>
       * This method is called by {@link yfiles.tree.RotatableNodePlacerBase#determineChildConnectors} and must be implemented by
       * subclasses. Implementations can assume the default direction (top-to-bottom) for their arrangement.
       * </p>
       * @param child the child node
       * @returns a direction specifier as defined in the {@link } interface
       * @protected
       * @abstract
       */
      determineChildConnector(child:yfiles.algorithms.Node):yfiles.tree.ParentConnectorDirection;
      /**
       * Provides the direction of the connector to the {@link yfiles.tree.SubtreeShape SubtreeShape} for each child node.
       * <p>
       * This implementation uses {@link yfiles.tree.RotatableNodePlacerBase#determineChildConnector} and the {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix} to
       * retrieve the direction for the connectors.
       * </p>
       * @param localRoot the local root
       * @param connectorMap the map that must be used for storing the direction specifiers of the child nodes
       * @see yfiles.tree.RotatableNodePlacerBase#determineChildConnector
       */
      determineChildConnectors(localRoot:yfiles.algorithms.Node,connectorMap:yfiles.algorithms.IDataMap):void;
      /**
       * Returns the rotated subtree shape for the given node.
       * <p>
       * This is a convenience method that retrieves a rotated subtree shape for a single node from the corresponding
       * {@link yfiles.algorithms.IDataProvider}. This shape may be modified and returned by
       * {@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode}.
       * </p>
       * @param node the node
       * @returns the subtree shape for the given node
       * @protected
       */
      getNodeShape(node:yfiles.algorithms.Node):yfiles.tree.RotatedSubtreeShape;
      /**
       * Returns the target {@link yfiles.layout.PortConstraint} for the local root's incoming edge.
       * <p>
       * This method translates the {@link yfiles.layout.PortConstraint} according to the rotation described by the {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix}.
       * It should be used instead of {@link yfiles.layout.PortConstraint#getTPC}.
       * </p>
       * <p>
       * This method may be used in {@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode} to route the edges to the according
       * port. It may be overridden to provide constant {@link yfiles.layout.PortConstraint}s for each node.
       * </p>
       * @param localRoot the local root
       * @returns the {@link } for the incoming edge of the local root or {@link #ANY} if no {@link }
       * is assigned for that edge
       * @protected
       */
      getPortConstraint(localRoot:yfiles.algorithms.Node):yfiles.layout.PortConstraint;
      /**
       * Returns the absolute source point of the given edge depending on the rotation of this {@link yfiles.tree.INodePlacer}.
       * <p>
       * This method translates the source point according to the rotation described by the {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix}. It should be
       * used instead of {@link yfiles.layout.LayoutGraph#getSourcePointAbs}.
       * </p>
       * <p>
       * It can be used in an implementation of {@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode}.
       * </p>
       * @param edge the edge whose absolute source point should be calculated
       * @returns the translated absolute source point for the given edge and current modification matrix
       * @protected
       */
      getSourcePointAbs(edge:yfiles.algorithms.Edge):yfiles.algorithms.YPoint;
      /**
       * Returns the rotated subtree shape for the subtree rooted at the given node.
       * <p>
       * This is a convenience method that retrieves a subtree shape of a complete subtree from the corresponding
       * {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * @param node the local root node
       * @returns the rotated subtree shape of the subtree rooted at the given local root node
       * @protected
       */
      getSubtreeShape(node:yfiles.algorithms.Node):yfiles.tree.RotatedSubtreeShape;
      /**
       * Initializes the local data structures and then delegates the work to the abstract variant of this method
       * ({@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode}).
       * @param nodeShapeProvider the {@link } for obtaining an initial shape of the root node
       * @param subtreeShapeProvider the {@link } for accessing the pre-calculated shapes of the subtrees
       * @param graph the input graph
       * @param localRoot the root of the subtree that should be arranged by this method
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the merged {@link } for the subtree rooted at the local root node
       * @see yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode
       */
      placeSubtree(nodeShapeProvider:yfiles.algorithms.IDataProvider,subtreeShapeProvider:yfiles.algorithms.IDataProvider,graph:yfiles.layout.LayoutGraph,localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.SubtreeShape;
      /**
       * Arranges the local root relative to the rotated subtree shapes of its children.
       * <p>
       * This is the main method that must be implemented by subclasses. For calculations and locations subclasses should use the
       * corresponding methods of {@link yfiles.tree.RotatableNodePlacerBase} which already consider the {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix}.
       * </p>
       * @param localRoot the local root node
       * @param parentConnectorDirection the direction specifier for the connector of the local root node to its parent node
       * @returns the merged subtree shape that contains the local root node and all its children
       * @protected
       * @abstract
       */
      placeSubtreeOfNode(localRoot:yfiles.algorithms.Node,parentConnectorDirection:yfiles.tree.ParentConnectorDirection):yfiles.tree.RotatedSubtreeShape;
      /**
       * Translates the given real direction of the resulting layout to the model direction of this
       * {@link yfiles.tree.RotatableNodePlacerBase}.
       * <p>
       * This method is used in {@link yfiles.tree.RotatableNodePlacerBase#placeSubtreeOfNode} to be able to retrieve the model direction
       * for the connector
       * </p>
       * @param realDirection the real direction
       * @returns the model direction
       * @throws {Stubs.Exceptions.ArgumentError} if translation for the given real direction is not implemented
       * @protected
       */
      translateDirectionToModel(realDirection:yfiles.tree.ParentConnectorDirection):number;
      /**
       * Translates the given model direction to the real direction in the resulting layout.
       * <p>
       * This method is used in {@link yfiles.tree.RotatableNodePlacerBase#determineChildConnectors} to be able to return the real
       * connector direction.
       * </p>
       * @param modelDirection the model direction
       * @returns the real direction
       * @throws {Stubs.Exceptions.ArgumentError} if translation for the given model direction is not implemented
       * @protected
       */
      translateDirectionToReal(modelDirection:yfiles.tree.ParentConnectorDirection):number;
      /**
       * Translates a point from the real layout direction to the model direction using the given modification matrix.
       * @param modificationMatrix the modification matrix to use for the translation
       * @param realWorldPoint the point with coordinates from the real world
       * @returns the translated point with coordinates from the model
       * @static
       */
      static translatePoint(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix,realWorldPoint:yfiles.algorithms.YPoint):yfiles.algorithms.YPoint;
      /**
       * The graph instance this {@link yfiles.tree.INodePlacer} is working on.
       * @protected
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * The provider of {@link yfiles.tree.SubtreeShape}s for whole subtrees.
       * @protected
       * @type {yfiles.algorithms.IDataProvider}
       */
      subtreeShapeProvider:yfiles.algorithms.IDataProvider;
      /**
       * The provider of {@link yfiles.tree.SubtreeShape}s for single nodes.
       * @protected
       * @type {yfiles.algorithms.IDataProvider}
       */
      nodeShapeProvider:yfiles.algorithms.IDataProvider;
      /**
       * The list containing the created children.
       * @protected
       * @type {yfiles.collections.IList.<Object>}
       */
      createdChildren:yfiles.collections.IList<Object>;
      /**
       * Gets the modification matrix that defines the direction of the subtree.
       * @type {yfiles.tree.RotatableNodePlacerMatrix}
       */
      modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix;
      /**
       * Gets or sets the spacing between subtrees.
       * <p>
       * The spacing needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given spacing is negative
       * @type {number}
       */
      spacing:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Decorates a SubtreeShape and provides rotated access on it.
     * @class
     */
    export interface RotatedSubtreeShape extends Object{}
    export class RotatedSubtreeShape {
      /**
       * For testing purposes only
       * @protected
       */
      constructor();
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.SimpleNodePlacer} arranges all children of a local root in a single row.
     * <p>
     * <b>Layout Style</b> {@graph {"ann":{"s":[30,30],"d":1},"n":[[0,0],[50,50],[235,50],[370,50],[720,50],[100,100],[770,100],[285,100],[420,50],[520,50],[570,100],[670,100],[150,150],[720,150],[470,100],[335,100],[620,150],[820,150],[520,150],[185,100]],"e":[[0,1,[65,15]],[0,2,[250,15]],[0,3,[385,15]],[0,4,[735,15]],[1,5,[115,65]],[4,6,[785,65]],[2,7,[300,65]],[0,8,[435,15]],[0,9,[535,15]],[9,10,[585,65]],[9,11,[685,65]],[5,12,[165,115]],[11,13,[735,115]],[8,14,[485,65]],[2,15,[350,65]],[10,16,[635,115]],[6,17,[835,115]],[14,18,[535,115]],[1,19,[200,65]]],"vp":[0.0,0.0,850.0,180.0]}}
     * </p>
     * @class
     * @extends {yfiles.tree.RotatableNodePlacerBase}
     */
    export interface SimpleNodePlacer extends yfiles.tree.RotatableNodePlacerBase{}
    export class SimpleNodePlacer {
      /**
       * Creates a new {@link yfiles.tree.SimpleNodePlacer} instance that uses the given {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix} and {@link yfiles.tree.SimpleNodePlacer#rootAlignment root alignment}.
       * @param modificationMatrix the transformation matrix for this {@link }
       * @param rootAlignment the alignment for the local root in relation to its subtrees
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix,rootAlignment:yfiles.tree.RootNodeAlignment);
      /**
       * Creates a new {@link yfiles.tree.SimpleNodePlacer} instance that uses the given {@link yfiles.tree.RotatableNodePlacerBase#modificationMatrix modification matrix}.
       * @param modificationMatrix the transformation matrix for this {@link }
       */
      constructor(modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix);
      /**
       * Creates a new {@link yfiles.tree.SimpleNodePlacer} instance with default settings.
       */
      constructor();
      /**
       * Gets or sets the {@link yfiles.tree.RootNodeAlignment RootAlignment} that defines how the root node is aligned with its children and their shapes.
       * @type {yfiles.tree.RootNodeAlignment}
       */
      rootAlignment:yfiles.tree.RootNodeAlignment;
      /**
       * Gets or sets the relative vertical alignment of nodes with the same parent.
       * <p>
       * All values between <code>0</code> and <code>1</code> result in a valid alignment.
       * </p>
       * <ul>
       * <li><code>0</code> - nodes are top-aligned</li>
       * <li><code>0.5</code> - nodes are center-aligned</li>
       * <li><code>1</code> - nodes are bottom-aligned</li>
       * </ul>
       * <p>
       * Values outside the interval <code>[0,1]</code> may result in a more compact node placement with unaligned nodes, especially when
       * labels are considered.
       * </p>
       * @type {number}
       */
      verticalAlignment:number;
      /**
       * Gets or sets whether or not a bus is created in the space between the parent and its children.
       * @type {boolean}
       */
      createBus:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.TreeComponentLayout} arranges tree-like subgraph structures in a mixed layout style.
     * <p>
     * {@graph {"ann":{"s":[37.5,37.5],"d":1},"n":[[-18.75,231.25],[56.25,231.25],[131.25,231.25],[-18.75,631.25],[631.25,231.25],[631.25,631.25],[131.25,631.25],[-18.75,-18.75],[631.25,-18.75],[631.25,131.25],[631.25,56.25],[131.25,406.25],[131.25,556.25],[447.5,75,1],[560,162.5,1],[497.5,162.5,1],[391.25,343.75,1],[328.75,343.75,1],[247.5,500,1],[247.5,587.5,1],[310,500,1],[310,587.5,1],[266.25,343.75,1],[272.5,75,1],[385,162.5,1],[322.5,162.5,1],[97.5,75,1],[210,162.5,1],[147.5,162.5,1]],"e":[[0,1,18.75,-0,-18.75,-0],[1,2,18.75,-0,-18.75,-0],[0,3,-0,18.75,-0,-18.75],[2,4,18.75,-0,-18.75,-0],[4,5,-0,18.75,-0,-18.75],[5,6,-18.75,-0,18.75,-0],[6,3,-18.75,-0,18.75,-0],[0,7,-0,-18.75,-0,18.75],[7,8,18.75,-0,-18.75,-0],[4,9,-0,-18.75,-0,18.75],[8,10,-0,18.75,-0,-18.75],[10,9,-0,18.75,-0,-18.75],[11,2,-0,-18.75,-0,18.75],[11,12,-0,18.75,-0,-18.75],[6,12,-0,-18.75,-0,18.75],[7,13,18.75,12.5,0,0,[52.5,12.5,52.5,50,466.25,50]],[13,14,-13.75,0,-0,-0,[452.5,137.5,578.75,137.5]],[13,15,-13.75,0,-0,-0,[452.5,137.5,516.25,137.5]],[2,16,18.75,12.5,0,0,[221.25,262.5,221.25,318.75,410,318.75]],[2,17,18.75,12.5,0,0,[221.25,262.5,221.25,318.75,347.5,318.75]],[11,18,18.75,-0,0,0,[202.5,425,202.5,475,266.25,475]],[11,19,18.75,-0,0,0,[202.5,425,202.5,562.5,266.25,562.5]],[11,20,18.75,-0,0,0,[202.5,425,202.5,475,328.75,475]],[11,21,18.75,-0,0,0,[202.5,425,202.5,562.5,328.75,562.5]],[2,22,18.75,12.5,0,0,[221.25,262.5,221.25,318.75,285,318.75]],[7,23,18.75,12.5,0,0,[52.5,12.5,52.5,50,291.25,50]],[23,24,-13.75,0,-0,-0,[277.5,137.5,403.75,137.5]],[23,25,-13.75,0,-0,-0,[277.5,137.5,341.25,137.5]],[7,26,18.75,12.5,0,0,[52.5,12.5,52.5,50,116.25,50]],[26,27,-13.75,0,-0,-0,[102.5,137.5,228.75,137.5]],[26,28,-13.75,0,-0,-0,[102.5,137.5,166.25,137.5]]],"vp":[-19.0,-19.0,688.0,688.0]}}
     * </p>
     * <p>
     * The tree components are marked
     * </p>
     * @class
     * @implements {yfiles.layout.ILayoutStage}
     */
    export interface TreeComponentLayout extends Object,yfiles.layout.ILayoutStage{}
    export class TreeComponentLayout {
      /**
       * Creates a new {@link yfiles.tree.TreeComponentLayout} instance using the specified layouter for arranging the subtrees.
       * @param treeCoreLayouter the layout algorithm used for arranging the tree components
       * @throws {Stubs.Exceptions.ArgumentError} if the specified layout algorithm is <code>null</code>
       */
      constructor(treeCoreLayouter:yfiles.layout.ILayoutAlgorithm);
      /**
       * Finds the tree components inside the given graph and applies a {@link yfiles.tree.TreeComponentLayout#treeComponentCoreLayout tree layout} to them.
       * @param graph the input graph
       */
      applyLayout(graph:yfiles.layout.LayoutGraph):void;
      /**
       * Delegates the layout calculation to the {@link yfiles.tree.TreeComponentLayout#coreLayout core layout algorithm} and handles the dummy nodes.
       * <p>
       * The dummy nodes are marked using the given {@link yfiles.algorithms.IDataProvider}.
       * </p>
       * <p>
       * This implementation just registers the dummy node {@link yfiles.algorithms.IDataProvider} with the graph with key
       * {@link yfiles.tree.TreeComponentLayout#DUMMY_NODE_DP_KEY}, executes the core layout algorithm and finally removes the data
       * provider again.
       * </p>
       * <p>
       * This method is called by {@link yfiles.tree.TreeComponentLayout#applyLayout} to invoke the core layout algorithm. It may be
       * overridden to customize the handling of the dummy nodes.
       * </p>
       * @param graph the input graph
       * @param dummyDp the {@link } which marks the dummy nodes
       * @protected
       */
      applyLayoutUsingDummies(graph:yfiles.layout.LayoutGraph,dummyDp:yfiles.algorithms.IDataProvider):void;
      /**
       * Data provider key for identifying dummy nodes during the layout calculation
       * <p>
       * This {@link yfiles.algorithms.IDataProvider} is registered and also removed in
       * {@link yfiles.tree.TreeComponentLayout#applyLayoutUsingDummies}.
       * </p>
       * @see yfiles.tree.TreeComponentLayout#applyLayoutUsingDummies
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static DUMMY_NODE_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * 
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      coreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the layout algorithm that is applied to tree components.
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      treeComponentCoreLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets whether or not tree component arrangement is improved by changing the orientation of components.
       * @type {boolean}
       */
      optimizeOrientation:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * A {@link yfiles.tree.SubtreeShape} represents the position and the borders of a subtree.
     * <p>
     * {@link yfiles.tree.INodePlacer}s will move {@link yfiles.tree.SubtreeShape}s and use the borders to calculate distances between the
     * subtrees. When two subtrees are placed in relation to their common root, their
     * {@link yfiles.tree.SubtreeShape}s are merged to represent the new subtree.
     * </p>
     * <p>
     * The borders on each side of the {@link yfiles.tree.SubtreeShape} are modeled with
     * {@link yfiles.algorithms.BorderLine}s. The resulting area includes the borders of each node and edge in the corresponding
     * subtree. It can also contain the bounds of labels belonging to those nodes and edges.
     * </p>
     * <p>
     * The {@link yfiles.tree.SubtreeShape} also describes a connector defining the location to which the edge from its parent is
     * attached. It is possible to add some bends (target points) at the end of this edge and include them in the
     * {@link yfiles.tree.SubtreeShape}. During layout calculation, the last target point is the connection point for the edge.
     * </p>
     * @see yfiles.tree.SubtreeShape#getBorderLine
     * @see yfiles.tree.SubtreeShape#connectorX
     * @see yfiles.tree.SubtreeShape#connectorY
     * @see yfiles.tree.SubtreeShape#addBoundsToShape
     * @see yfiles.tree.SubtreeShape#addTargetPoint
     * @class
     * @final
     */
    export interface SubtreeShape extends Object{}
    export class SubtreeShape {
      /**
       * Extends this {@link yfiles.tree.SubtreeShape} instance by the given rectangle.
       * <p>
       * The border lines are updated with this segment.
       * </p>
       * @param x the x-coordinate of the upper-left corner of the rectangle
       * @param y the y-coordinate of the upper-left corner of the rectangle
       * @param width the width of the rectangle
       * @param height the height of the rectangle
       * @see yfiles.tree.SubtreeShape#getBorderLine
       */
      addBoundsToShape(x:number,y:number,width:number,height:number):void;
      /**
       * Adds all edge segments of the given edge to this {@link yfiles.tree.SubtreeShape} instance.
       * <p>
       * The border lines are updated.
       * </p>
       * @param graph the input graph
       * @param edge the given edge
       * @see yfiles.tree.SubtreeShape#getBorderLine
       * @see yfiles.tree.SubtreeShape#addLineSegment
       */
      addEdgeSegments(graph:yfiles.layout.LayoutGraph,edge:yfiles.algorithms.Edge):void;
      /**
       * Extends this {@link yfiles.tree.SubtreeShape} instance by the given segment.
       * <p>
       * The border lines are updated with this segment.
       * </p>
       * @param x1 the x-coordinate of the first point of the segment
       * @param y1 the y-coordinate of the first point of the segment
       * @param x2 the x-coordinate of the second point of the segment
       * @param y2 the y-coordinate of the second point of the segment
       * @see yfiles.tree.SubtreeShape#getBorderLine
       * @see yfiles.tree.SubtreeShape#addEdgeSegments
       * @see yfiles.tree.SubtreeShape#updateConnectorShape
       */
      addLineSegment(x1:number,y1:number,x2:number,y2:number):void;
      /**
       * Adds another point to the connector.
       * <p>
       * The points will appear as bend of the incoming edge. Points are added in reverse direction of the edge. Hence, the last
       * target point is further away from the target node than the points before.
       * </p>
       * @param x the current x-coordinate of the point
       * @param y the current y-coordinate of the point
       */
      addTargetPoint(x:number,y:number):void;
      /**
       * Appends all target points that have previously been added to this shape to the given {@link yfiles.layout.IEdgeLayout}
       * instance.
       * <p>
       * This method is used by {@link yfiles.tree.INodePlacer} instances for the final edge paths.
       * </p>
       * @param edgeLayout the {@link } that will be modified
       * @see yfiles.tree.SubtreeShape#addTargetPoint
       */
      appendTargetPoints(edgeLayout:yfiles.layout.IEdgeLayout):void;
      /**
       * Copies the state of this {@link yfiles.tree.SubtreeShape} instance to the given shape applying a modification matrix.
       * @param toShape the given shape that gets the assigned values
       * @param modificationMatrix the matrix that may add geometrical modifications
       */
      assignValuesTo(toShape:yfiles.tree.SubtreeShape,modificationMatrix:yfiles.tree.RotatableNodePlacerMatrix):void;
      /**
       * Creates a clone of this {@link yfiles.tree.SubtreeShape} instance, modifying it using the provided modification matrix.
       * @param matrix the matrix that defines the modification for the new instance
       * @returns the clone of this {@link } instance
       * @see yfiles.tree.RotatableNodePlacerMatrix#DEFAULT
       */
      createCopy(matrix:yfiles.tree.RotatableNodePlacerMatrix):yfiles.tree.SubtreeShape;
      /**
       * Returns the {@link yfiles.algorithms.BorderLine} instance that describes the {@link yfiles.tree.SubtreeShape} at the given side.
       * @param index one of the predefined borderline directions
       * @returns the {@link } instance for the given side
       * @throws {Stubs.Exceptions.ArgumentError} if the given index is invalid
       */
      getBorderLine(index:number):yfiles.algorithms.BorderLine;
      /**
       * Merges the given {@link yfiles.tree.SubtreeShape} instance with this {@link yfiles.tree.SubtreeShape} instance.
       * <p>
       * The border lines are updated.
       * </p>
       * @param other the {@link } to be merged with this {@link }
       */
      mergeWith(other:yfiles.tree.SubtreeShape):void;
      /**
       * Moves this {@link yfiles.tree.SubtreeShape} instance and its connector by the given offsets.
       * <p>
       * Border lines are updated.
       * </p>
       * @param dx the delta x-offset by which this shape will be moved
       * @param dy the delta y-offset by which this shape will be moved
       */
      move(dx:number,dy:number):void;
      /**
       * Adds the line segments of the connector to the bounds of this {@link yfiles.tree.SubtreeShape} instance.
       * <p>
       * The line segments result from the target points.
       * </p>
       * @see yfiles.tree.SubtreeShape#addTargetPoint
       */
      updateConnectorShape():void;
      /**
       * Gets the bounds of the {@link yfiles.layout.INodeLayout} of the root node of this {@link yfiles.tree.SubtreeShape} instance.
       * @type {yfiles.algorithms.Rectangle2D}
       */
      coreBounds:yfiles.algorithms.Rectangle2D;
      /**
       * Gets the current bounds of this {@link yfiles.tree.SubtreeShape} instance.
       * @type {yfiles.algorithms.Rectangle2D}
       */
      bounds:yfiles.algorithms.Rectangle2D;
      /**
       * Gets the current x-coordinate of the connection point to which the layout algorithm connects the ingoing edge.
       * <p>
       * The connection point can change when target points are added to the {@link yfiles.tree.SubtreeShape} instance.
       * </p>
       * @see yfiles.tree.SubtreeShape#addTargetPoint
       * @see yfiles.tree.SubtreeShape#appendTargetPoints
       * @type {number}
       */
      connectorX:number;
      /**
       * Gets the current y-coordinate of the connection point to which the layout algorithm connects the ingoing edge.
       * <p>
       * The connection point can change when target points are added to the {@link yfiles.tree.SubtreeShape} instance.
       * </p>
       * @see yfiles.tree.SubtreeShape#addTargetPoint
       * @see yfiles.tree.SubtreeShape#appendTargetPoints
       * @type {number}
       */
      connectorY:number;
      /**
       * Gets the direction of the last connector segment.
       * @type {yfiles.tree.ConnectorDirection}
       */
      connectorDirection:yfiles.tree.ConnectorDirection;
      /**
       * Gets the current minimum x-coordinate of this {@link yfiles.tree.SubtreeShape} instance.
       * @type {number}
       */
      minX:number;
      /**
       * Gets the current minimum y-coordinate of this {@link yfiles.tree.SubtreeShape} instance.
       * @type {number}
       */
      minY:number;
      /**
       * Gets the current maximum x-coordinate of this {@link yfiles.tree.SubtreeShape} instance.
       * @type {number}
       */
      maxX:number;
      /**
       * Gets the current maximum y-coordinate of this {@link yfiles.tree.SubtreeShape} instance.
       * @type {number}
       */
      maxY:number;
      /**
       * Gets the current x-coordinate of the origin of this {@link yfiles.tree.SubtreeShape} instance.
       * <p>
       * The origin is defined as the upper left corner of the {@link yfiles.layout.INodeLayout} of the local root of this shape.
       * </p>
       * @type {number}
       */
      originX:number;
      /**
       * Gets the current y-coordinate of the origin of this {@link yfiles.tree.SubtreeShape} instance.
       * <p>
       * The origin is defined as the upper left corner of the {@link yfiles.layout.INodeLayout} of the local root of this shape.
       * </p>
       * @type {number}
       */
      originY:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm arranges graphs with a tree structure.
     * <p>
     * <b>Layout Style</b> {@link yfiles.tree.TreeLayout} provides multiple different arrangements of trees and subtrees. It is easy to customize the
     * {@link yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY order of edges}, the {@link yfiles.tree.TreeLayout#PORT_ASSIGNMENT_DP_KEY port assignment} and the {@link yfiles.tree.TreeLayout#NODE_PLACER_DP_KEY arrangement of the nodes} for each subtree.
     * </p>
     * <p>
     * Tree layout algorithms are commonly applied to visualize relational data and produce diagrams of high quality that are
     * able to reveal possible hierarchic properties of the graph. More precisely, they find applications in dataflow analysis,
     * software engineering, bioinformatics and business administration.
     * </p>
     * <p>
     * {@graph {"ann": {"s": [37.5,37.5], "d": 1},"n": [[209.96,0], [71.88,77.5], [348.04,77.5], [348.04,155], [202.77,232.5], [172.5,310], [71.88,232.5], [115,387.5], [115,310], [233.04,310], [71.88,155], [348.04,232.5], [319.29,310], [376.79,310], [463.04,232.5], [491.79,310], [434.29,310], [28.75,310], [57.5,387.5], [0,387.5], [376.79,387.5], [261.79,387.5], [204.29,387.5]], "e": [[0,1], [0,2], [2,3], [3,4], [4,5], [6,8], [8,7], [4,9], [1,10], [10,6], [3,11], [11,12], [11,13], [3,14], [14,15], [14,16], [6,17], [17,18], [17,19], [13,20], [9,21], [9,22]], "vp": [0.0,0.0,530.0,425.0]}} Example graph with a layout calculated by {@link yfiles.tree.TreeLayout} using default settings {@graph {"ann": {"s": [60,30], "d": 1},"n": [[1190,0], [0,50], [160,120], [80,120], [240,120], [120,190], [200,190], [280,190], [160,310], [80,310], [240,310], [120,240], [200,240], [280,240], [330,50], [460,256.6], [410,306.6], [510,306.6], [410,356.6], [510,356.6], [460,406.6], [650,441.6], [410,491.6], [490,491.6], [570,491.6], [710,50], [790,100], [870,170], [950,170], [1030,170], [1110,170], [1190,170], [870,240], [950,240], [1030,240], [1110,240], [1190,240], [870,310], [950,310], [1030,310], [1110,310], [1190,310], [870,380], [950,380], [1030,380], [1110,380], [1190,380], [430,120], [500,170], [540,206.6], [580,170], [620,206.6], [660,170], [80,520], [160,430], [160,480], [226.6,405], [226.6,455], [160,380], [790,465], [870,430], [950,430], [950,500], [1030,465], [870,500]], "e": [[0,1,[30,15]], [1,2,[30,100,190,100]], [2,3,[190,170,110,170]], [2,4,[190,170,270,170]], [2,5,[190,170,150,170]], [2,6,[190,170,230,170]], [2,7,[190,170,310,170]], [1,8,[30,360,190,360]], [8,9,[190,290,110,290]], [8,10,[190,290,270,290]], [8,11,[190,290,150,290]], [8,12,[190,290,230,290]], [8,13,[190,290,310,290]], [0,14,[360,15]], [14,15,[360,271.6]], [15,16,[490,321.6]], [15,17,[490,321.6]], [15,18,[490,371.6]], [15,19,[490,371.6]], [15,20], [14,21,[360,456.6]], [21,22,[440,456.6]], [21,23,[520,456.6]], [21,24,[600,456.6]], [0,25,[740,15]], [25,26,[740,115]], [26,27,[860,115,860,150,900,150]], [26,28,[860,115,860,150,980,150]], [26,29,[860,115,860,150,1060,150]], [26,30,[860,115,860,150,1140,150]], [26,31,[860,115,860,150,1220,150]], [26,32,[860,115,860,220,900,220]], [26,33,[860,115,860,220,980,220]], [26,34,[860,115,860,220,1060,220]], [26,35,[860,115,860,220,1140,220]], [26,36,[860,115,860,220,1220,220]], [26,37,[860,115,860,290,900,290]], [26,38,[860,115,860,290,980,290]], [26,39,[860,115,860,290,1060,290]], [26,40,[860,115,860,290,1140,290]], [26,41,[860,115,860,290,1220,290]], [26,42,[860,115,860,360,900,360]], [26,43,[860,115,860,360,980,360]], [26,44,[860,115,860,360,1060,360]], [26,45,[860,115,860,360,1140,360]], [26,46,[860,115,860,360,1220,360]], [14,47,[360,100,410,100,410,135]], [47,48,[530,135]], [47,49,[570,135]], [47,50,[610,135]], [47,51,[650,135]], [47,52,[690,135]], [1,53,[30,535]], [53,54,[110,445]], [53,55,[110,495]], [53,56,[110,420]], [53,57,[110,470]], [53,58,[110,395]], [25,59,[740,480]], [59,60,[900,480]], [59,61,[980,480]], [59,62,[980,480]], [59,63], [59,64,[900,480]]], "vp": [0.0,0.0,1250.0,550.0]}} Tree layout using different
     * {@link yfiles.tree.INodePlacer}s and rotations<b>Concept</b>
     * </p>
     * <p>
     * The layout algorithm starts from the leaves and continues with their parents, then with the parents of the parents and
     * so on. When a node is processed, the algorithm will use the corresponding {@link yfiles.tree.INodePlacer} instance to move its
     * children (along with their subtrees) to a suitable position and to route the outgoing edges of this node. Then, the next
     * local root node will be processed.
     * </p>
     * <p>
     * To avoid moving all nodes several times and to know the area that the subtrees occupy, the layout algorithm uses
     * {@link yfiles.tree.SubtreeShape}s. These {@link yfiles.tree.SubtreeShape}s are moved and merged during layout calculation.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * Each subtree can have a different style of node placement. {@link yfiles.tree.INodePlacer}s are responsible for arranging
     * subtrees and their common root node. They can be specified separately for each local root with a {@link yfiles.algorithms.IDataProvider}
     * registered with the graph using key {@link yfiles.tree.TreeLayout#NODE_PLACER_DP_KEY}.
     * </p>
     * <p>
     * A custom node can be defined as root of the tree using a {@link yfiles.algorithms.IDataProvider} registered with the graph with
     * key {@link yfiles.tree.TreeLayout#SELECTED_ROOT_DP_KEY}.
     * </p>
     * <p>
     * The layout algorithm can be configured to reserve space for node labels and place the edge labels along edges such that
     * the labels won't overlap with other graph elements. Edge labels are placed according to the information stored in a {@link yfiles.layout.PreferredPlacementDescriptor}
     * instance. However, the placement along the edge will only affect the order of multiple labels at the same edge. The
     * algorithm will always place the labels close to the target node.
     * </p>
     * <p>
     * Grouping of nodes can also be handled by this layout algorithm. It is important that a group node contains a whole
     * subtree. Otherwise, the group nodes may overlap with each other or with other nodes. Furthermore, the user may specify
     * minimum size constraints for each group node using {@link yfiles.algorithms.IDataProvider} key
     * {@link yfiles.layout.GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
     * </p>
     * <p>
     * {@link yfiles.tree.TreeLayout} supports custom sorting of the outgoing edges of a node. For example, a {@link yfiles.tree.INodePlacer}
     * instance that implements {@link yfiles.tree.IFromSketchNodePlacer} provides a comparator that keeps the current order of
     * siblings, allowing to extend the graph incrementally.
     * </p>
     * <p>
     * This layout algorithm can only handle graphs with a tree structure. To apply it to a general graph, a {@link yfiles.tree.TreeReductionStage}
     * can be appended. This stage will temporarily remove some edges of the input graph until a tree is obtained. After the
     * layout calculation, the stage will reinsert the edges that were removed and route them separately.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface TreeLayout extends yfiles.layout.MultiStageLayout{}
    export class TreeLayout {
      /**
       * Creates a new {@link yfiles.tree.TreeLayout} instance with default settings.
       */
      constructor();
      /**
       * Creates a {@link yfiles.tree.SubtreeShape} for the given node that consists only of the bounds of this particular node.
       * <p>
       * It may also include node labels or {@link yfiles.layout.NodeHalo}s.
       * </p>
       * <p>
       * This method may be overridden to return a custom {@link yfiles.tree.SubtreeShape} for the given node.
       * </p>
       * @param node the given node
       * @returns a {@link } instance
       */
      createNodeShape(node:yfiles.algorithms.Node):yfiles.tree.SubtreeShape;
      /**
       * Returns a list of edges that need to be reversed in order to obtain a valid rooted and directed tree from the input
       * graph.
       * <p>
       * The root node of the tree is either a node marked by a {@link yfiles.algorithms.IDataProvider} registered with the graph with
       * key {@link yfiles.tree.TreeLayout#SELECTED_ROOT_DP_KEY} or is defined according to {@link yfiles.algorithms.Trees#getRoot}.
       * </p>
       * <p>
       * This method is called initially to calculate a rooted tree from the input graph. It may be overridden to apply a custom
       * algorithm that determines which edges need to be reversed.
       * </p>
       * @returns a list of edges that need to be reversed
       * @see yfiles.tree.TreeLayout#reverseEdges
       * @protected
       */
      directTree():yfiles.algorithms.EdgeList;
      /**
       * Returns the {@link yfiles.tree.INodePlacer} instance that is used for the placement of the local root node and the
       * {@link yfiles.tree.SubtreeShape}s.
       * <p>
       * The method may be overridden to return custom {@link yfiles.tree.INodePlacer} instances. It is possible to return a shared
       * instance for multiple different nodes because the instances are not used after subsequent calls to this method.
       * </p>
       * <p>
       * The current implementation returns the {@link yfiles.tree.INodePlacer} defined by the {@link yfiles.algorithms.IDataProvider} registered
       * with key {@link yfiles.tree.TreeLayout#NODE_PLACER_DP_KEY}. It falls back to the default {@link yfiles.tree.INodePlacer} if there is no
       * specific {@link yfiles.tree.INodePlacer} for the given node.
       * </p>
       * @param localRoot the root of the local subtree
       * @returns the {@link } instance that places the subtree below the given local root node
       * @see yfiles.tree.TreeLayout#NODE_PLACER_DP_KEY
       * @see yfiles.tree.TreeLayout#defaultNodePlacer
       * @protected
       */
      getNodePlacer(localRoot:yfiles.algorithms.Node):yfiles.tree.INodePlacer;
      /**
       * Returns the {@link yfiles.collections.IComparer.<T>} instance that will sort the outgoing edges connecting to the given node.
       * <p>
       * The {@link yfiles.collections.IComparer.<T>} can be <code>null</code> in case the initial edge order shall be used.
       * </p>
       * <p>
       * The method may be overridden to return custom {@link yfiles.collections.IComparer.<T>} instances. It is possible to return a shared instance for multiple
       * different nodes because the instances are not used after subsequent calls to this method.
       * </p>
       * <p>
       * The current implementation returns the {@link yfiles.collections.IComparer.<T>} defined in the {@link yfiles.algorithms.IDataProvider} registered with key
       * {@link yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY}. It falls back to the default {@link yfiles.collections.IComparer.<T>} if there is no specific {@link yfiles.collections.IComparer.<T>} for the given
       * node.
       * </p>
       * @param localRoot the root of the local subtree
       * @returns the {@link } or <code>null</code> that sorts the outgoing edges of the given node
       * @see yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY
       * @see yfiles.tree.TreeLayout#defaultOutEdgeComparer
       */
      getOutEdgeComparer(localRoot:yfiles.algorithms.Node):yfiles.collections.IComparer<Object>;
      /**
       * Returns the {@link yfiles.tree.IPortAssignment} instance that places the ports of the connecting edges of the given node.
       * <p>
       * The method may be overridden to return customized {@link yfiles.tree.IPortAssignment} instances. It is possible to return a
       * shared instance for multiple different nodes because the instances are not used after subsequent calls to this method.
       * </p>
       * <p>
       * The current implementation returns the {@link yfiles.tree.IPortAssignment} defined by the {@link yfiles.algorithms.IDataProvider}
       * registered with key {@link yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY}. It falls back to the default {@link yfiles.tree.IPortAssignment}
       * if there is no specific {@link yfiles.tree.IPortAssignment} for the given node.
       * </p>
       * @param localRoot the root of the local subtree
       * @returns the {@link } instance that assigns the ports of the edges at the given node
       * @see yfiles.tree.TreeLayout#PORT_ASSIGNMENT_DP_KEY
       * @see yfiles.tree.TreeLayout#defaultPortAssignment
       * @protected
       */
      getPortAssignment(localRoot:yfiles.algorithms.Node):yfiles.tree.IPortAssignment;
      /**
       * Returns an array of the nodes that will be laid out.
       * <p>
       * The given node will be considered as the root of the tree. The order of the elements ensures that no parent is processed
       * before one of its successors.
       * </p>
       * <p>
       * This method may be overridden to change the order in which the nodes (and their subtrees) are handled. However, it is
       * important to keep the parents after the successors. Only siblings can change places.
       * </p>
       * @param root the node to be considered as root of the tree
       * @returns an array of nodes that will be laid out
       * @protected
       */
      getRootsArray(root:yfiles.algorithms.Node):yfiles.algorithms.Node[];
      /**
       * Provides access to the {@link yfiles.tree.SubtreeShape} for the given node.
       * <p>
       * The {@link yfiles.tree.SubtreeShape} contains information about the current extent and location of the subtree rooted at the
       * node. It should only be modified during the layout of the parent node. Also, it won't be initialized before the layout
       * of the corresponding subtree is calculated.
       * </p>
       * <p>
       * This method updates the {@link yfiles.tree.SubtreeShape} in {@link yfiles.tree.TreeLayout#layoutRoot}.
       * </p>
       * @param localRoot the root of the subtree
       * @returns the {@link } instance if it has already been calculated, <code>null</code> otherwise
       * @protected
       */
      getSubtreeShape(localRoot:yfiles.algorithms.Node):yfiles.tree.SubtreeShape;
      /**
       * Calculates the layout for the given root node and its subtrees.
       * <p>
       * This method is invoked for each node in the tree exactly once. Thus, children are always handled before their parents.
       * In this manner, the subtrees already have a layout and can be arranged with their parent.
       * </p>
       * <p>
       * To retrieve the shapes of the subtrees of all children of the local root, this method uses
       * {@link yfiles.tree.TreeLayout#getSubtreeShape}.
       * </p>
       * <p>
       * The method may be overridden to add configuration code.
       * </p>
       * @param localRoot the root of the subtree that whose layout is calculated
       * @returns the combined {@link } of the local root node and all of its children and connecting edges
       * @protected
       */
      layoutRoot(localRoot:yfiles.algorithms.Node):yfiles.tree.SubtreeShape;
      /**
       * Reverses the direction of given edges.
       * <p>
       * This method will be called initially after the {@link yfiles.tree.TreeLayout#directTree} method and finally after the layout has
       * been calculated. It may be overridden to additionally reverse data that depends on the direction of the edges.
       * </p>
       * @param reversedEdges the edges that will be reversed
       * @protected
       */
      reverseEdges(reversedEdges:yfiles.algorithms.EdgeList):void;
      /**
       * Data provider key for specifying a child node placer for each node
       * @see yfiles.tree.TreeLayout#defaultNodePlacer
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.tree.INodePlacer>}
       */
      static NODE_PLACER_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.tree.INodePlacer>;
      /**
       * Data provider key for assigning ports to nodes
       * @see yfiles.tree.TreeLayout#defaultPortAssignment
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.tree.IPortAssignment>}
       */
      static PORT_ASSIGNMENT_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.tree.IPortAssignment>;
      /**
       * Data provider key for specifying the comparator for the outgoing edges
       * @see yfiles.tree.TreeLayout#defaultOutEdgeComparer
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.collections.IComparer.<Object>>}
       */
      static OUT_EDGE_COMPARER_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.collections.IComparer<Object>>;
      /**
       * Data provider key for defining the priority of critical edges
       * <p>
       * The layout algorithm tries to align each node pair that is connected by a critical edge (integer value >
       * <code>0</code>). This feature can, for example, be used for highlighting different edge paths that are important for a user.
       * Conflicts between different critical edges are always resolved in favor of the higher priority.
       * </p>
       * <p>
       * Critical edges will always align the centers of source and target node, thus replacing the current root alignment of the
       * {@link yfiles.tree.INodePlacer}.
       * </p>
       * <p>
       * The critical edge may not be straight if subtrees are rotated or port constraints are assigned.
       * </p>
       * <p>
       * This feature is only supported by the following {@link yfiles.tree.INodePlacer}s:
       * </p>
       * <ul>
       * <li>{@link yfiles.tree.DefaultNodePlacer}</li>
       * <li>{@link yfiles.tree.SimpleNodePlacer}</li>
       * <li>{@link yfiles.tree.LayeredNodePlacer}</li>
       * <li>{@link yfiles.tree.DoubleLineNodePlacer}</li>
       * <li>{@link yfiles.tree.DendrogramNodePlacer}</li>
       * </ul>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<number>}
       */
      static CRITICAL_EDGE_DP_KEY:yfiles.algorithms.EdgeDpKey<number>;
      /**
       * Data provider key for retrieving descriptors for nodes in multi-parent structures
       * <p>
       * The returned descriptor affects the layout of the multi-parent structure.
       * </p>
       * @see yfiles.tree.TreeLayout#multiParentAllowed
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.tree.MultiParentDescriptor>}
       */
      static MULTI_PARENT_DESCRIPTOR_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.tree.MultiParentDescriptor>;
      /**
       * Data provider key for marking the node that will be used as root node of the tree
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<boolean>}
       */
      static SELECTED_ROOT_DP_KEY:yfiles.algorithms.NodeDpKey<boolean>;
      /**
       * The input graph for which a layout is calculated.
       * @protected
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * Gets or sets whether or not group nodes are handled by the layout algorithm.
       * @type {boolean}
       */
      groupingSupported:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing parallel edges is activated.
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouter
       * @see yfiles.router.ParallelEdgeRouter
       * @type {boolean}
       */
      parallelEdgeRouterEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      /**
       * Gets a {@link yfiles.algorithms.IDataAcceptor} that can overwrite the source {@link yfiles.layout.PortConstraint port contraint} temporarily used during the layout.
       * <p>
       * This method sets temporary source {@link yfiles.layout.PortConstraint}s, e.g., after edge reversal. It may be overridden to
       * handle those {@link yfiles.layout.PortConstraint}s separately.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.algorithms.IDataAcceptor}
       */
      sourcePortConstraintDataAcceptor:yfiles.algorithms.IDataAcceptor;
      /**
       * Gets a {@link yfiles.algorithms.IDataAcceptor} that can overwrite the target {@link yfiles.layout.PortConstraint port contraint} temporarily used during the layout.
       * <p>
       * This method sets temporary target {@link yfiles.layout.PortConstraint}s, e.g., after edge reversal. It may be overridden to
       * handle those {@link yfiles.layout.PortConstraint}s separately.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @protected
       * @type {yfiles.algorithms.IDataAcceptor}
       */
      targetPortConstraintDataAcceptor:yfiles.algorithms.IDataAcceptor;
      /**
       * Gets a {@link yfiles.algorithms.IDataAcceptor} that can be used for temporarily overwriting the source group information used
       * during the layout.
       * <p>
       * This method may be overridden to handle those edge groups separately.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @protected
       * @type {yfiles.algorithms.IDataAcceptor}
       */
      sourceGroupDataAcceptor:yfiles.algorithms.IDataAcceptor;
      /**
       * Gets a {@link yfiles.algorithms.IDataAcceptor} that can be used for temporarily overwriting the target group information used
       * during the layout.
       * <p>
       * This method may be overridden to handle those edge groups separately.
       * </p>
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @protected
       * @type {yfiles.algorithms.IDataAcceptor}
       */
      targetGroupDataAcceptor:yfiles.algorithms.IDataAcceptor;
      /**
       * Gets or sets the default {@link yfiles.tree.INodePlacer} instance that arranges all subtrees that do not have a specific {@link yfiles.tree.INodePlacer}
       * assigned using a {@link yfiles.algorithms.IDataProvider}.
       * @throws {Stubs.Exceptions.ArgumentError} if the default {@link yfiles.tree.INodePlacer} is set to <code>null</code>
       * @see yfiles.tree.TreeLayout#NODE_PLACER_DP_KEY
       * @type {yfiles.tree.INodePlacer}
       */
      defaultNodePlacer:yfiles.tree.INodePlacer;
      /**
       * Gets or sets the default {@link yfiles.tree.INodePlacer} instance that places the leaf nodes of the tree.
       * @throws {Stubs.Exceptions.ArgumentError} if the default {@link yfiles.tree.INodePlacer} is set to <code>null</code>
       * @see yfiles.tree.TreeLayout#defaultNodePlacer
       * @type {yfiles.tree.INodePlacer}
       */
      defaultLeafPlacer:yfiles.tree.INodePlacer;
      /**
       * Gets or sets the default {@link yfiles.tree.IPortAssignment} instance for all subtrees that do not have a specific {@link yfiles.tree.IPortAssignment}
       * assigned using a {@link yfiles.algorithms.IDataProvider}.
       * @throws {Stubs.Exceptions.ArgumentError} if the default {@link yfiles.tree.IPortAssignment} is set to <code>null</code>
       * @see yfiles.tree.TreeLayout#PORT_ASSIGNMENT_DP_KEY
       * @type {yfiles.tree.IPortAssignment}
       */
      defaultPortAssignment:yfiles.tree.IPortAssignment;
      /**
       * Gets or sets the default {@link yfiles.collections.IComparer.<T>} instance that sorts the outgoing edges in all subtrees that do not have a specific {@link yfiles.collections.IComparer.<T>} assigned
       * using a {@link yfiles.algorithms.IDataProvider}.
       * @see yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY
       * @type {yfiles.collections.IComparer.<Object>}
       */
      defaultOutEdgeComparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for node labels.
       * <p>
       * If this feature is enabled, the {@link yfiles.tree.SubtreeShape} of the nodes will also hold the shapes of its labels.
       * </p>
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not edge labels are placed by the layout algorithm.
       * <p>
       * If this is enabled, the {@link yfiles.tree.SubtreeShape}s of the subtrees are extended by the edge labels.
       * </p>
       * <p>
       * The layout algorithm uses the information of {@link yfiles.layout.PreferredPlacementDescriptor} for an edge label to determine
       * the corresponding placement. However, edge labels are always placed at the target side of the edge. The placement along
       * the edge only affects the order of the edge labels at the same edge.
       * </p>
       * @type {boolean}
       */
      integratedEdgeLabeling:boolean;
      /**
       * Gets or sets whether or not multi-parent structures are allowed for this tree layout.
       * <p>
       * If multi-parent structures are allowed, the directed input graph may contain structures of multiple nodes that share the
       * same predecessors as well as the same successors. All nodes belonging to such a structure are placed side by side and
       * the incident edges are routed in a bus-style manner.
       * </p>
       * @type {boolean}
       */
      multiParentAllowed:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * The {@link yfiles.tree.TreeReductionStage} temporarily reduces general graphs to trees.
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * This stage prepares a non-tree graph such that it can be processed by a tree layout algorithm.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[150,0],[260,70],[210,70],[260,140],[310,210],[335,280],[75,70],[285,280],[25,70],[25,140],[75,140],[150,70],[25,210],[185,280],[175,140],[50,280],[0,280],[210,210],[235,280],[125,140]],"e":[[0,1,[165,50,275,50]],[0,2,[165,50,225,50]],[1,3],[3,4,[275,190,325,190]],[4,5,[325,260,350,260]],[0,6,[165,50,90,50]],[4,7,[325,260,300,260]],[0,8,[165,50,40,50]],[8,9],[6,10],[0,11],[9,12],[11,14,[165,120,190,120]],[12,15,[40,260,65,260]],[12,16,[40,260,15,260]],[3,17,[275,190,225,190]],[17,13,[225,260,200,260]],[17,18,[225,260,250,260]],[11,19,[165,120,140,120]],[2,11,1],[12,13,1],[17,14,1]],"vp":[0.0,0.0,365.0,310.0]}}
     * </p>
     * <p>
     * The non-tree edges are marked
     * </p>
     * <p>
     * <b>Concept</b> This {@link yfiles.layout.ILayoutStage} works in three steps:
     * </p>
     * <ol>
     * <li>Remove some edges to get a tree or forest from the graph</li>
     * <li>Invoke the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} which supports tree graph handling</li>
     * <li>Reinsert all previously removed edges</li>
     * <li>
     * Optionally route these edges using the non-tree edge router and place their labels using the non-tree edge labeling
     * algorithm.
     * </li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The routing of the temporarily hidden non-tree edges can be customized by specifying an {@link yfiles.tree.TreeReductionStage#nonTreeEdgeRouter edge routing algorithm} for those
     * edges. Similarly, the placement of edge labels of non-tree edges can be delegated to a custom
     * {@link yfiles.tree.TreeReductionStage#nonTreeEdgeLabelingAlgorithm edge labeling algorithm}. As default, both custom algorithms are not specified and, thus, non-tree edges are not routed
     * and edge labels not placed.
     * </p>
     * <p>
     * This stage is also able to handle multi-parent structures, i.e., structures of multiple nodes that share the same
     * predecessors as well as the same successors. More precisely, if the specified {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} supports
     * multi-parent structures (see {@link yfiles.tree.TreeLayout#multiParentAllowed}) and option {@link yfiles.tree.TreeReductionStage#multiParentAllowed} is enabled, this stage does not hide such structures, i.e., the multi-parent
     * structures are passed to the {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
     * </p>
     * <p>
     * Non-tree edges can also be bundled together such that their common parts are to some degree merged into a bundled part.
     * The edge bundling can be specified by means of method {@link yfiles.tree.TreeReductionStage#edgeBundling}.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface TreeReductionStage extends yfiles.layout.LayoutStageBase{}
    export class TreeReductionStage {
      /**
       * Creates a new {@link yfiles.tree.TreeReductionStage} instance with the given core layout algorithm and default settings.
       * @param core the core layout algorithm
       */
      constructor(core:yfiles.layout.ILayoutAlgorithm);
      /**
       * Creates a new {@link yfiles.tree.TreeReductionStage} instance with default settings.
       */
      constructor();
      /**
       * Creates a routing algorithm that routes edges as a single straight segment.
       * <p>
       * The created instance can be used for {@link yfiles.tree.TreeReductionStage#nonTreeEdgeRouter routing non-tree edges}.
       * </p>
       * @returns an edge routing algorithm that produces straight-line edges
       * @see yfiles.tree.TreeReductionStage#nonTreeEdgeRouter
       */
      createStraightLineRouter():yfiles.layout.ILayoutAlgorithm;
      /**
       * Routes all edges that do not belong to the chosen spanning tree.
       * <p>
       * This method is called by {@link yfiles.tree.TreeReductionStage#applyLayout} after the tree was arranged by the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. It may be overridden to apply custom edge routes.
       * </p>
       * @param graph the graph containing tree and non-tree edges
       * @param nonTreeEdgeMap the {@link } that marks all non-tree edges in the graph
       * @protected
       */
      routeNonTreeEdges(graph:yfiles.layout.LayoutGraph,nonTreeEdgeMap:yfiles.algorithms.IEdgeMap):void;
      /**
       * Data provider key for explicitly marking (some) edges that should not be considered for the tree
       * @see yfiles.tree.TreeReductionStage#nonTreeEdgeRouter
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static NON_TREE_EDGES_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets whether or not multi-parent structures (structures of multiple nodes that share the same predecessors as
       * well as the same successors) are allowed.
       * <p>
       * More precisely, if this option is enabled and the specified {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} can handle multi-parent structures
       * (see {@link yfiles.tree.TreeLayout#multiParentAllowed}), this stage does not hide such structures, but the multi-parent structures are passed on to the {@link yfiles.layout.LayoutStageBase#coreLayout} core layout
       * algorithm}.
       * </p>
       * @type {boolean}
       */
      multiParentAllowed:boolean;
      /**
       * Gets or sets the edge routing algorithm that is applied to all non-tree edges.
       * @see yfiles.tree.TreeReductionStage#nonTreeEdgeSelectionKey
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      nonTreeEdgeRouter:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that will be used by the {@link yfiles.tree.TreeReductionStage#nonTreeEdgeRouter non-tree edge routing algorithm}
       * to determine the edges that need to be routed.
       * @see yfiles.tree.TreeReductionStage#nonTreeEdgeRouter
       * @type {Object}
       */
      nonTreeEdgeSelectionKey:Object;
      /**
       * Gets or sets the labeling algorithm that is applied to all edge labels that belong to non-tree edges.
       * <p>
       * It is required that a suitable {@link yfiles.tree.TreeReductionStage#nonTreeEdgeLabelSelectionKey edge label selection key} is set. Otherwise, the edge labeling algorithm might also place
       * labels of tree edges.
       * </p>
       * @see yfiles.tree.TreeReductionStage#nonTreeEdgeLabelSelectionKey
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      nonTreeEdgeLabelingAlgorithm:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that is used by the {@link yfiles.tree.TreeReductionStage#nonTreeEdgeLabelingAlgorithm non-tree edge labeling algorithm}
       * to determine which edge labels it should place.
       * @see yfiles.tree.TreeReductionStage#nonTreeEdgeLabelingAlgorithm
       * @type {Object}
       */
      nonTreeEdgeLabelSelectionKey:Object;
      /**
       * Gets the {@link yfiles.layout.EdgeBundling} instance that defines the settings of the edge bundling feature.
       * <p>
       * The specified {@link yfiles.layout.EdgeBundling} defines global bundling properties. Settings for individual edges can be
       * defined by assigning an {@link yfiles.layout.EdgeBundleDescriptor} to an edge using a {@link yfiles.algorithms.IDataProvider} registered
       * with key {@link yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}. To enable bundling for all non-tree edges, set a {@link yfiles.layout.EdgeBundling#defaultBundleDescriptor default bundle descriptor}
       * which has {@link yfiles.layout.EdgeBundleDescriptor#bundled bundling} enabled.
       * </p>
       * <p>
       * If the graph contains self-loops that belong to the set of non-tree edges, the stage will invoke the router defined for
       * the non-tree edges in order to route them. In the case where this is not desired, the user has to hide them from the
       * stage and route them afterwards as desired.
       * </p>
       * @type {yfiles.layout.EdgeBundling}
       */
      edgeBundling:yfiles.layout.EdgeBundling;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.tree.AspectRatioTreeLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface AspectRatioTreeLayoutData extends yfiles.layout.LayoutData{}
    export class AspectRatioTreeLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from nodes to the aspect ratio of their sub tree.
       * @see yfiles.tree.AspectRatioTreeLayout#SUBTREE_ASPECT_RATIO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      subtreeAspectRatios:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      /**
       * Gets or sets the mapping from nodes with sub trees to their placements.
       * @see yfiles.tree.AspectRatioTreeLayout#ROOT_PLACEMENT_DP_KEY
       * @see yfiles.tree.AspectRatioTreeLayout#getRootPlacement
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.tree.RootPlacement>}
       */
      rootPlacements:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.tree.RootPlacement>;
      /**
       * Gets or sets the mapping from nodes to the routing policies of their sub trees.
       * @see yfiles.tree.AspectRatioTreeLayout#SUBTREE_ROUTING_POLICY_DP_KEY
       * @see yfiles.tree.AspectRatioTreeLayout#getSubtreeArrangement
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.tree.SubtreeArrangement>}
       */
      subtreeRoutingPolicies:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.tree.SubtreeArrangement>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.tree.BalloonLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface BalloonLayoutData extends yfiles.layout.LayoutData{}
    export class BalloonLayoutData {
      constructor();
      /**
       * Gets or sets the collection of nodes that should be arranged in an interleaved fashion.
       * @see yfiles.tree.BalloonLayout#INTERLEAVED_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      interleavedNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping for marking the node that will be used as root node of the tree.
       * <p>
       * The custom root node selection induced by this property is only considered if the {@link yfiles.tree.BalloonLayout#rootNodePolicy root node policy} is set to
       * {@link yfiles.tree.RootNodePolicy#SELECTED_ROOT}.
       * </p>
       * @see yfiles.tree.BalloonLayout#SELECTED_ROOT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,boolean>}
       */
      treeRoot:yfiles.layout.ItemMapping<yfiles.graph.INode,boolean>;
      /**
       * Gets or sets the mapping that provides a {@link yfiles.layout.PreferredPlacementDescriptor} instance for edge
       * {@link yfiles.graph.ILabel}s.
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>}
       */
      edgeLabelPreferredPlacement:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.tree.TreeLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface TreeLayoutData extends yfiles.layout.LayoutData{}
    export class TreeLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.tree.INodePlacer}.
       * @see yfiles.tree.TreeLayout#NODE_PLACER_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.tree.INodePlacer>}
       */
      nodePlacers:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.tree.INodePlacer>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.tree.IPortAssignment}.
       * @see yfiles.tree.TreeLayout#PORT_ASSIGNMENT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.tree.IPortAssignment>}
       */
      portAssignments:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.tree.IPortAssignment>;
      /**
       * Gets or sets the mapping from nodes to comparison functions used to sort the nodes' outgoing edges.
       * @see yfiles.tree.TreeLayout#OUT_EDGE_COMPARER_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,Function>}
       */
      outEdgeComparers:yfiles.layout.ItemMapping<yfiles.graph.INode,Function>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets a mapping from edges to their source {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      sourcePortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets a mapping from edges to their target {@link yfiles.layout.PortConstraint}.
       * @see yfiles.layout.PortConstraintKeys#TARGET_PORT_CONSTRAINT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.PortConstraint>}
       */
      targetPortConstraints:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.PortConstraint>;
      /**
       * Gets or sets the collection of nodes the {@link yfiles.tree.AssistantNodePlacer} considers as assistants.
       * <p>
       * This collection is only considered if {@link yfiles.tree.TreeLayoutData#nodePlacers} returns any {@link yfiles.tree.AssistantNodePlacer} .
       * </p>
       * @see yfiles.tree.AssistantNodePlacer#ASSISTANT_NODE_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      assistantNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the collection of nodes the {@link yfiles.tree.DelegatingNodePlacer} places with its {@link yfiles.tree.DelegatingNodePlacer#primaryPlacer}.
       * <p>
       * This collection is only considered if {@link yfiles.tree.TreeLayoutData#nodePlacers} returns any {@link yfiles.tree.DelegatingNodePlacer} .
       * </p>
       * @see yfiles.tree.DelegatingNodePlacer#PRIMARY_NODES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.INode>}
       */
      delegatingNodePlacersPrimaryNodes:yfiles.layout.ItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets a mapping from nodes to a boolean value that specifies whether a node should be placed left of the bus
       * (<code>true</code>) or right of the bus (<code>false</code>).
       * <p>
       * This mapping is only considered if {@link yfiles.tree.TreeLayoutData#nodePlacers} returns a {@link yfiles.tree.LeftRightNodePlacer} .
       * </p>
       * @see yfiles.tree.LeftRightNodePlacer#LEFT_RIGHT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,boolean>}
       */
      leftRightPlacersLeftNodes:yfiles.layout.ItemMapping<yfiles.graph.INode,boolean>;
      /**
       * Gets or sets the mapping from nodes to the index of the row the {@link yfiles.tree.GridNodePlacer} shall place the node in.
       * <p>
       * This collection is only considered if {@link yfiles.tree.TreeLayoutData#nodePlacers} returns any {@link yfiles.tree.GridNodePlacer} .
       * </p>
       * @see yfiles.tree.GridNodePlacer#ROW_INDEX_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,number>}
       */
      gridNodePlacerRowIndices:yfiles.layout.ItemMapping<yfiles.graph.INode,number>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping for marking the node that will be used as root node of the tree.
       * @see yfiles.tree.TreeLayout#SELECTED_ROOT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,boolean>}
       */
      treeRoot:yfiles.layout.ItemMapping<yfiles.graph.INode,boolean>;
      /**
       * Gets or sets the mapping that provides a {@link yfiles.layout.PreferredPlacementDescriptor} instance for edge
       * {@link yfiles.graph.ILabel}s.
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>}
       */
      edgeLabelPreferredPlacement:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.tree.TreeReductionStage}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface TreeReductionStageData extends yfiles.layout.LayoutData{}
    export class TreeReductionStageData {
      constructor();
      /**
       * Gets or sets the collection of edges explicitly marked as not belonging to a tree.
       * @see yfiles.tree.TreeReductionStage#NON_TREE_EDGES_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      nonTreeEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets the mapping of edges to their {@link yfiles.layout.EdgeBundleDescriptor}.
       * <p>
       * Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
       * source and target point, the edges are again clearly split.
       * </p>
       * <p>
       * If an edge is mapped to <code>null</code>, the {@link yfiles.layout.EdgeBundling#defaultBundleDescriptor default descriptor} is used.
       * </p>
       * @see yfiles.layout.EdgeBundleDescriptor
       * @see yfiles.tree.TreeReductionStage#edgeBundling
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.EdgeBundleDescriptor>}
       */
      edgeBundleDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.EdgeBundleDescriptor>;
      static $class:yfiles.lang.Class;
    }
  }export namespace genealogy{
    /**
     * This layout algorithm arranges genealogical graphs (family trees).
     * <p>
     * <b>Layout Style</b> The family trees consist of nodes that represent individuals and nodes that represent families. Individuals are
     * connected via these family nodes. The graph is arranged in layers that describe generations. {@graph {"ann":{"s":[122.27,30],"d":1},"styles":{"5":{"c":1}},"n":[[364.55,210,[[405.01,234.35,41.35,18.7,"Mother"]]],[364.55,140,[[406.35,164.35,38.68,18.7,"Father"]]],[182.27,385,[[206.06,409.35,74.7,18.7,"Grandmother"]]],[182.27,315,[[209.39,339.35,68.04,18.7,"Grandfather"]]],[182.27,175,[[206.06,199.35,74.7,18.7,"Grandmother"]]],[182.27,105,[[209.39,129.35,68.04,18.7,"Grandfather"]]],[0,490,[[6.78,514.35,108.71,18.7,"Great-Grandmother"]]],[0,420,[[10.11,444.35,102.04,18.7,"Great-Grandfather"]]],[0,350,[[6.78,374.35,108.71,18.7,"Great-Grandmother"]]],[0,280,[[10.11,304.35,102.04,18.7,"Great-Grandfather"]]],[0,210,[[6.78,234.35,108.71,18.7,"Great-Grandmother"]]],[0,140,[[10.11,164.35,102.04,18.7,"Great-Grandfather"]]],[0,70,[[6.78,94.35,108.71,18.7,"Great-Grandmother"]]],[0,0,[[10.11,24.35,102.04,18.7,"Great-Grandfather"]]],[56.14,45,10,10,5],[56.14,185,10,10,5],[56.14,325,10,10,5],[56.14,465,10,10,5],[238.41,360,10,10,5],[238.41,150,10,10,5],[420.69,185,10,10,5],[556.82,245,[[597.95,269.35,40.02,18.7,"Sibling"]]],[556.82,315,[[597.95,339.35,40.02,18.7,"Sibling"]]],[556.82,175,[[602.29,199.35,31.35,18.7,"Child"]]]],"e":[[12,14],[13,14],[14,5,5,0,-61.14,0,[132.27,50,132.27,120]],[11,15],[10,15],[15,4,5,0,-61.14,0],[8,16],[9,16],[16,3,5,0,-61.14,0],[6,17],[7,17],[17,2,5,0,-61.14,0,[132.27,470,132.27,400]],[2,18],[3,18],[18,0,5,0,-61.14,0,[314.55,365,314.55,225]],[4,19],[5,19],[19,1,5,0,-61.14,0],[0,20],[1,20],[20,21,5,0,-61.14,0,[536.82,190,536.82,260]],[20,22,5,0,-61.14,0,[536.82,190,536.82,330]],[20,23,5,0,-61.14,0]],"vp":[0.0,0.0,680.0,520.0]}}
     * </p>
     * <p>
     * Basic Family Tree
     * </p>
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * In order to determine whether a node represents an individual or a partnership, a {@link yfiles.algorithms.IDataProvider} has to
     * be registered with key {@link yfiles.genealogy.FamilyTreeLayout#FAMILY_TYPE_DP_KEY}. If this {@link yfiles.algorithms.IDataProvider} is
     * missing, an {@link yfiles.lang.Exception} will be thrown.
     * </p>
     * <p>
     * This {@link yfiles.algorithms.IDataProvider} provides:
     * </p>
     * <ul>
     * <li>{@link yfiles.genealogy.FamilyType#MALE} for a node representing a male individual.</li>
     * <li>{@link yfiles.genealogy.FamilyType#FEMALE} for a node representing a female individual.</li>
     * <li>{@link yfiles.genealogy.FamilyType#FAMILY} for a node representing a family.</li>
     * </ul>
     * <p>
     * All other values will be interpreted as if the node represents an individual.
     * </p>
     * <p>
     * A {@link yfiles.genealogy.FamilyType#FAMILY} node links all members of a family. It can only have two incoming edges from the
     * parents. Two nodes of the same type (families or individuals) cannot be directly connected. If the input graph violates
     * these specifications, a {@link yfiles.lang.Exception} exception will be thrown.
     * </p>
     * <p>
     * The layout is calculated basically in two steps:
     * </p>
     * <ul>
     * <li>The families are laid out by the inner layouter in a compact way.</li>
     * <li>
     * The relations between these "family groups" and their children and other families are laid out by the
     * {@link yfiles.genealogy.FamilyTreeLayout#topLayout top layout algorithm}.
     * </li>
     * </ul>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The distances between nodes {@link yfiles.genealogy.FamilyTreeLayout#spacingBetweenFamilyMembers in the same generation} as well as between {@link yfiles.genealogy.FamilyTreeLayout#offsetForFamilyNodes individuals and family nodes} can be adjusted.
     * </p>
     * <p>
     * Nodes in the same generation can be {@link yfiles.genealogy.FamilyTreeLayout#alignment aligned} at three different coordinates: top, center, bottom.
     * </p>
     * <p>
     * The location of {@link yfiles.genealogy.FamilyType#FAMILY} nodes can be {@link yfiles.genealogy.FamilyTreeLayout#familyNodesAlwaysBelow specified} in relation to the partners. This placement
     * can be set independently for {@link yfiles.genealogy.FamilyTreeLayout#partnerlessBelow parents}.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface FamilyTreeLayout extends yfiles.layout.MultiStageLayout{}
    export class FamilyTreeLayout {
      /**
       * Creates a new instance of {@link yfiles.genealogy.FamilyTreeLayout} with default settings.
       */
      constructor();
      /**
       * Data provider key for assigning a node type for each node
       * <p>
       * The layout algorithm will handle the nodes according to this type.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.genealogy.FamilyType>}
       */
      static FAMILY_TYPE_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.genealogy.FamilyType>;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutAlgorithm layout algorithm} that is responsible for the layout between family groups.
       * <p>
       * If the specified layout algorithm is <code>null</code>, fallback algorithm {@link yfiles.hierarchic.HierarchicLayout} is used.
       * </p>
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      topLayout:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the spacing between two nodes representing members of the same family in one generation.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given spacing is negative
       * @type {number}
       */
      spacingBetweenFamilyMembers:number;
      /**
       * Gets or sets the offset by which a {@link yfiles.genealogy.FamilyType#FAMILY} node is shifted down if its parents are not direct
       * neighbors.
       * <p>
       * Values should be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given offset is negative
       * @type {number}
       */
      offsetForFamilyNodes:number;
      /**
       * Gets or sets whether or not {@link yfiles.genealogy.FamilyType#FAMILY} nodes of individuals that have only children but no
       * partners are placed below this individual or next to it.
       * @type {boolean}
       */
      partnerlessBelow:boolean;
      /**
       * Gets or sets whether or not {@link yfiles.genealogy.FamilyType#FAMILY} nodes are placed between or below the partners.
       * @see yfiles.genealogy.FamilyType#FAMILY
       * @type {boolean}
       */
      familyNodesAlwaysBelow:boolean;
      /**
       * Gets or sets the vertical alignment of the individual nodes in the same generation.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified alignment is unknown
       * @type {yfiles.genealogy.VerticalNodeAlignment}
       */
      alignment:yfiles.genealogy.VerticalNodeAlignment;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing parallel edges is activated.
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouterEnabled
       * @see yfiles.layout.MultiStageLayout#parallelEdgeRouter
       * @see yfiles.router.ParallelEdgeRouter
       * @type {boolean}
       */
      parallelEdgeRouterEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      /**
       * Gets or sets the order of individuals in the same generation (siblings, partners) based on their sex.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified order is unknown
       * @type {yfiles.genealogy.FamilyMembersSortingPolicy}
       */
      familyMembersSortingPolicy:yfiles.genealogy.FamilyMembersSortingPolicy;
      static $class:yfiles.lang.Class;
    }
    export enum FamilyType{
      /**
       * Type specifier for nodes that connect partners with each other and their children.
       * @see yfiles.genealogy.FamilyTreeLayout#FAMILY_TYPE_DP_KEY
       */
      FAMILY,
      /**
       * Type specifier for nodes that represent men.
       * @see yfiles.genealogy.FamilyTreeLayout#FAMILY_TYPE_DP_KEY
       */
      MALE,
      /**
       * Type specifier for nodes that represent women.
       * @see yfiles.genealogy.FamilyTreeLayout#FAMILY_TYPE_DP_KEY
       */
      FEMALE
    }
    export enum VerticalNodeAlignment{
      /**
       * Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their top
       * coordinate.
       * <p>
       * The descriptions of the directions are based on {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}.
       * </p>
       * @see yfiles.genealogy.FamilyTreeLayout#alignment
       */
      TOP,
      /**
       * Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their center
       * coordinate.
       * <p>
       * The descriptions of the directions are based on {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}.
       * </p>
       * @see yfiles.genealogy.FamilyTreeLayout#alignment
       */
      CENTER,
      /**
       * Alignment specifier that defines that all nodes in the same layer (generation) are aligned based on their bottom
       * coordinate.
       * <p>
       * The descriptions of the directions are based on {@link yfiles.layout.LayoutOrientation#TOP_TO_BOTTOM}.
       * </p>
       * @see yfiles.genealogy.FamilyTreeLayout#alignment
       */
      BOTTOM
    }
    export enum FamilyMembersSortingPolicy{
      /**
       * Order specifier that disables sorting of individuals according to their sex.
       * @see yfiles.genealogy.FamilyTreeLayout#familyMembersSortingPolicy
       */
      NO_SORTING,
      /**
       * Order specifier that places female individuals before their male siblings and partners, if possible.
       * @see yfiles.genealogy.FamilyTreeLayout#familyMembersSortingPolicy
       */
      FEMALE_FIRST,
      /**
       * Order specifier that places female individuals before their male siblings and partners.
       * @see yfiles.genealogy.FamilyTreeLayout#familyMembersSortingPolicy
       */
      FEMALE_ALWAYS_FIRST,
      /**
       * Order specifier that places male individuals before their female siblings and partners, if possible.
       * @see yfiles.genealogy.FamilyTreeLayout#familyMembersSortingPolicy
       */
      MALE_FIRST,
      /**
       * Order specifier that places male individuals before their female siblings and partners.
       * @see yfiles.genealogy.FamilyTreeLayout#familyMembersSortingPolicy
       */
      MALE_ALWAYS_FIRST
    }
    /**
     * Specifies custom data for the {@link yfiles.genealogy.FamilyTreeLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface FamilyTreeLayoutData extends yfiles.layout.LayoutData{}
    export class FamilyTreeLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from nodes to their family type.
       * @see yfiles.genealogy.FamilyTreeLayout#FAMILY_TYPE_DP_KEY
       * @see yfiles.genealogy.FamilyType
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.genealogy.FamilyType>}
       */
      familyTypes:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.genealogy.FamilyType>;
      static $class:yfiles.lang.Class;
    }
  }export namespace multipage{
    /**
     * This class is a default implementation of interface {@link yfiles.multipage.IElementFactory}.
     * @see yfiles.multipage.IElementFactory
     * @class
     * @implements {yfiles.multipage.IElementFactory}
     */
    export interface DefaultElementFactory extends Object,yfiles.multipage.IElementFactory{}
    export class DefaultElementFactory {
      /**
       * Creates a new instance of {@link yfiles.multipage.DefaultElementFactory}.
       */
      constructor();
      /**
       * 
       * @param context
       * @param connector
       * @param opposite
       * @param origEdgeId
       * @param atTarget
       * @returns 
       */
      createConnectorEdge(context:yfiles.multipage.LayoutContext,connector:yfiles.algorithms.Node,opposite:yfiles.algorithms.Node,origEdgeId:Object,atTarget:boolean):yfiles.algorithms.Edge;
      /**
       * 
       * @param context
       * @param edgeIds
       * @param representedNodeId
       * @returns 
       */
      createConnectorNode(context:yfiles.multipage.LayoutContext,edgeIds:yfiles.algorithms.YList,representedNodeId:Object):yfiles.algorithms.Node;
      /**
       * 
       * @param context
       * @param proxyNode
       * @param opposite
       * @param replacingEdgeId
       * @param origNodeId
       * @returns 
       */
      createProxyEdge(context:yfiles.multipage.LayoutContext,proxyNode:yfiles.algorithms.Node,opposite:yfiles.algorithms.Node,replacingEdgeId:Object,origNodeId:Object):yfiles.algorithms.Edge;
      /**
       * 
       * @param context
       * @param origNodeId
       * @returns 
       */
      createProxyNode(context:yfiles.multipage.LayoutContext,origNodeId:Object):yfiles.algorithms.Node;
      /**
       * 
       * @param context
       * @param proxyReference
       * @param opposite
       * @param referencingCopyId
       * @returns 
       */
      createProxyReferenceEdge(context:yfiles.multipage.LayoutContext,proxyReference:yfiles.algorithms.Node,opposite:yfiles.algorithms.Node,referencingCopyId:Object):yfiles.algorithms.Edge;
      /**
       * 
       * @param context
       * @param referringProxyId
       * @returns 
       */
      createProxyReferenceNode(context:yfiles.multipage.LayoutContext,referringProxyId:Object):yfiles.algorithms.Node;
      /**
       * Returns the default size for nodes created by this factory.
       * <p>
       * This method is called from {@link yfiles.multipage.DefaultElementFactory#createConnectorNode},
       * {@link yfiles.multipage.DefaultElementFactory#createProxyReferenceNode}, and/or
       * {@link yfiles.multipage.DefaultElementFactory#createProxyNode}. It must return a non-<code>null</code> size with positive width and height.
       * </p>
       * <p>
       * Subclasses may implement this method to use custom size values.
       * </p>
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param id the element ID
       * @param type the type of the node to be created
       * @returns the default size of the new node
       * @protected
       */
      getDefaultNodeSize(context:yfiles.multipage.LayoutContext,id:Object,type:number):yfiles.algorithms.YDimension;
      static $class:yfiles.lang.Class;
    }
    export enum EdgeType{
      /**
       * An edge type that specifies a normal edge.
       * <p>
       * An edge is called <i>normal</i> if it does not belong to any other type.
       * </p>
       * @see yfiles.multipage.IEdgeInfo#type
       */
      NORMAL,
      /**
       * An edge type that specifies a connector edge.
       * <p>
       * An edge is called <i>connector</i> if it is connected to a connector node (a node of {@link yfiles.multipage.INodeInfo#type type}
       * {@link yfiles.multipage.NodeType#CONNECTOR}), i.e., it represents an edge of the input graph whose endpoints lie on different
       * pages.
       * </p>
       * @see yfiles.multipage.IEdgeInfo#representedEdge
       * @see yfiles.multipage.IEdgeInfo#type
       * @see yfiles.multipage.NodeType#CONNECTOR
       */
      CONNECTOR,
      /**
       * An edge type that specifies a proxy reference edge.
       * <p>
       * An edge is called <i>proxy reference</i> if it is connected to a proxy reference node (a node of {@link yfiles.multipage.INodeInfo#type type}
       * {@link yfiles.multipage.NodeType#PROXY_REFERENCE}), i.e., a node that refers to a proxy of an original node lying on another
       * page.
       * </p>
       * @see yfiles.multipage.IEdgeInfo#type
       * @see yfiles.multipage.NodeType#PROXY_REFERENCE
       */
      PROXY_REFERENCE,
      /**
       * An edge type that specifies a proxy edge.
       * <p>
       * An edge is called <i>proxy</i> if it is connected to a proxy node (a node of {@link yfiles.multipage.INodeInfo#type type}
       * {@link yfiles.multipage.NodeType#PROXY}), i.e., a node that is a proxy of an original node lying on another page.
       * </p>
       * @see yfiles.multipage.IEdgeInfo#type
       * @see yfiles.multipage.NodeType#PROXY
       */
      PROXY
    }
    export enum GroupingMode{
      /**
       * Node grouping specifier used when group nodes should be ignored.
       * @see yfiles.multipage.MultiPageLayout#groupingMode
       */
      IGNORE,
      /**
       * Node grouping specifier used when only original nodes should be assigned to group nodes. Hence, connector and proxy
       * nodes (see {@link yfiles.multipage.NodeType#CONNECTOR} and
       * {@link yfiles.multipage.NodeType#PROXY}) are never assigned to group nodes even though the associated node lies within a group.
       * @see yfiles.multipage.MultiPageLayout#groupingMode
       */
      ORIGINAL_NODES_ONLY,
      /**
       * Node grouping specifier used when special nodes like, e.g., connector and proxy nodes (see {@link yfiles.multipage.NodeType#CONNECTOR}
       * and {@link yfiles.multipage.NodeType#PROXY}) should be assigned to group nodes, too.
       * @see yfiles.multipage.MultiPageLayout#groupingMode
       */
      ALL_NODES
    }
    export enum EdgeBundleModes{
      /**
       * Edge bundle mode constant that specifies that all multi-edges (edges with same endpoints) should be distinguished.
       * Hence, if such edges connect two nodes on different pages, a separate connector pair (see
       * {@link yfiles.multipage.NodeType#CONNECTOR}) is used for each of them.
       * @see yfiles.multipage.MultiPageLayout#edgeBundleModeMask
       */
      DISTINGUISH_MULTI_EDGES,
      /**
       * Edge bundle mode constant that specifies that multi-edges (edges with same endpoints) should be distinguished if they
       * have different directions. Hence, two edges with the same endpoints may share a connector pair (see
       * {@link yfiles.multipage.NodeType#CONNECTOR}) if they point in the same direction (i.e., both have the same source and target
       * node).
       * @see yfiles.multipage.MultiPageLayout#edgeBundleModeMask
       */
      DISTINGUISH_DIRECTIONS,
      /**
       * Edge bundle mode constant that specifies that multi-edges (edges with same endpoints) should be distinguished if they
       * are of different (user specified) type, see {@link yfiles.multipage.MultiPageLayout#EDGE_TYPE_DP_KEY}. Hence, two edges with the
       * same endpoints and same type may share a connector pair (see {@link yfiles.multipage.NodeType#CONNECTOR}).
       * @see yfiles.multipage.MultiPageLayout#edgeBundleModeMask
       * @see yfiles.multipage.MultiPageLayout#EDGE_TYPE_DP_KEY
       */
      DISTINGUISH_TYPES
    }
    export enum NodeType{
      /**
       * A node type that specifies a normal node.
       * <p>
       * A node is called <i>normal</i> if it does not belong to any other type.
       * </p>
       * @see yfiles.multipage.INodeInfo#type
       */
      NORMAL,
      /**
       * A node type that specifies a group node.
       * <p>
       * A <i>group</i> node is a node that may contain other nodes.
       * </p>
       * @see yfiles.multipage.INodeInfo#type
       */
      GROUP,
      /**
       * A node type that specifies a proxy node.
       * <p>
       * A <i>proxy</i> node is a copy of a {@link yfiles.multipage.NodeType#NORMAL normal node}. It is created by the algorithm if a normal node and its neighbors or associated {@link yfiles.multipage.NodeType#CONNECTOR connectors}
       * do not fit onto a single page. Hence, in such cases, the neighbors/connectors associated with a normal node may be
       * distributed among different proxy nodes associated with this node.
       * </p>
       * @see yfiles.multipage.INodeInfo#type
       * @see yfiles.multipage.NodeType#NORMAL
       * @see yfiles.multipage.NodeType#PROXY_REFERENCE
       */
      PROXY,
      /**
       * A node type that specifies a proxy reference node.
       * <p>
       * For each {@link yfiles.multipage.NodeType#PROXY proxy node} there is exactly one matching proxy reference node. The reference node refers to the proxy and is
       * either connected to the {@link yfiles.multipage.NodeType#NORMAL normal node} associated with this proxy or one of its other proxy nodes (a normal node may be
       * associated with multiple proxy nodes). Hence, for a node connected to a proxy reference node there always exists a copy
       * located on another page.
       * </p>
       * @see yfiles.multipage.INodeInfo#type
       * @see yfiles.multipage.NodeType#PROXY
       */
      PROXY_REFERENCE,
      /**
       * A node type that specifies a connector node.
       * <p>
       * Each edge of the input graph that connects two nodes <code>v</code> and <code>w</code> that are placed on different pages is split by a pair of
       * matching connector nodes (one for each of the two pages). One of the connector nodes is connected to <code>v</code> and the other to
       * <code>w</code>. Hence, each connector represents a jump mark to the matching connector and guarantees that no information is lost as
       * it would be the case if such edges are simply removed.
       * </p>
       * @see yfiles.multipage.INodeInfo#type
       */
      CONNECTOR
    }
    /**
     * A class that holds all information associated with an edge.
     * @interface
     */
    export interface IEdgeInfo extends Object{
      /**
       * Gets the unique ID of the associated edge.
       * @see yfiles.multipage.MultiPageLayoutResult#getEdgeInfo
       * @see yfiles.multipage.IElementInfoManager#getEdgeInfo
       * @abstract
       * @type {Object}
       */
      id:Object;
      /**
       * Gets the type of the associated edge.
       * @abstract
       * @type {yfiles.multipage.EdgeType}
       */
      type:yfiles.multipage.EdgeType;
      /**
       * Gets the original edge associated with this {@link yfiles.multipage.IEdgeInfo}.
       * <p>
       * More precisely, for edges of {@link yfiles.multipage.IEdgeInfo#type type} {@link yfiles.multipage.EdgeType#CONNECTOR}, this method returns the edge of the input graph
       * that was split by the connector edge. Otherwise, it returns <code>null</code>.
       * </p>
       * @see yfiles.multipage.EdgeType#CONNECTOR
       * @abstract
       * @type {yfiles.algorithms.Edge}
       */
      representedEdge:yfiles.algorithms.Edge;
    }
    var IEdgeInfo:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A class that holds all information related to an edge label.
     * @interface
     */
    export interface IEdgeLabelInfo extends Object{
      /**
       * Gets the unique ID of the associated edge label.
       * @see yfiles.multipage.MultiPageLayoutResult#getEdgeLabelInfo
       * @see yfiles.multipage.IElementInfoManager#getEdgeLabelInfo
       * @abstract
       * @type {Object}
       */
      id:Object;
    }
    var IEdgeLabelInfo:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Factory used by class {@link yfiles.multipage.MultiPageLayout} to create special nodes and edges.
     * @interface
     */
    export interface IElementFactory extends Object{
      /**
       * Callback method for creating an edge of type {@link yfiles.multipage.EdgeType#CONNECTOR}.
       * <p>
       * Such an edge connects the given {@link yfiles.multipage.NodeType#CONNECTOR connector} and opposite node.
       * </p>
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param connector the endpoint of the edge that represents the connector node
       * @param opposite the other endpoint of the edge
       * @param origEdgeId the ID of the edge that is split by the connector edge
       * @param atTarget <code>true</code> if the connector node is the target of the edge, <code>false</code> otherwise
       * @returns the created connector edge
       * @see yfiles.multipage.EdgeType#CONNECTOR
       * @see yfiles.multipage.NodeType#CONNECTOR
       * @see yfiles.multipage.LayoutContext#createEdge
       * @abstract
       */
      createConnectorEdge(context:yfiles.multipage.LayoutContext,connector:yfiles.algorithms.Node,opposite:yfiles.algorithms.Node,origEdgeId:Object,atTarget:boolean):yfiles.algorithms.Edge;
      /**
       * Callback method for creating a node of type {@link yfiles.multipage.NodeType#CONNECTOR}.
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param edgesIds a {@link  list} that contains the IDs of edges that are split by this connector
       * @param representedNodeId the ID of the node that is represented by this connector
       * @returns the created connector node
       * @see yfiles.multipage.NodeType#CONNECTOR
       * @see yfiles.multipage.LayoutContext#createNode
       * @see yfiles.multipage.INodeInfo#representedNode
       * @abstract
       */
      createConnectorNode(context:yfiles.multipage.LayoutContext,edgesIds:yfiles.algorithms.YList,representedNodeId:Object):yfiles.algorithms.Node;
      /**
       * Callback method for creating an edge of type {@link yfiles.multipage.EdgeType#PROXY}.
       * <p>
       * Such an edge should connect the given {@link yfiles.multipage.NodeType#PROXY proxy} and opposite node.
       * </p>
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param proxyNode the endpoint of the edge that represents the proxy node
       * @param opposite the other endpoint of the new edge
       * @param replacingEdgeId the ID of the related edge that is connected to the original node
       * @param origNodeId the ID of the original node to which the proxy node refers
       * @returns the created proxy edge
       * @see yfiles.multipage.EdgeType#PROXY
       * @see yfiles.multipage.NodeType#PROXY
       * @see yfiles.multipage.LayoutContext#createEdge
       * @abstract
       */
      createProxyEdge(context:yfiles.multipage.LayoutContext,proxyNode:yfiles.algorithms.Node,opposite:yfiles.algorithms.Node,replacingEdgeId:Object,origNodeId:Object):yfiles.algorithms.Edge;
      /**
       * Callback method for creating a node of type {@link yfiles.multipage.NodeType#PROXY}.
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param origNodeId the ID of the node for which a proxy has to be created
       * @returns the created proxy node
       * @see yfiles.multipage.LayoutContext#createNode
       * @see yfiles.multipage.NodeType#PROXY_REFERENCE
       * @see yfiles.multipage.NodeType#PROXY
       * @abstract
       */
      createProxyNode(context:yfiles.multipage.LayoutContext,origNodeId:Object):yfiles.algorithms.Node;
      /**
       * Callback method for creating an edge of type {@link yfiles.multipage.EdgeType#PROXY_REFERENCE}.
       * <p>
       * Such an (undirected) edge should connect the given {@link yfiles.multipage.NodeType#PROXY_REFERENCE proxy reference} and opposite node.
       * </p>
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param proxyReference the endpoint of the edge that represents the proxy reference node
       * @param opposite the other endpoint of the new edge
       * @param referencingCopyId the ID of the proxy node that is referenced by the proxy reference node
       * @returns the created proxy reference edge
       * @see yfiles.multipage.EdgeType#PROXY_REFERENCE
       * @see yfiles.multipage.NodeType#PROXY_REFERENCE
       * @see yfiles.multipage.LayoutContext#createEdge
       * @abstract
       */
      createProxyReferenceEdge(context:yfiles.multipage.LayoutContext,proxyReference:yfiles.algorithms.Node,opposite:yfiles.algorithms.Node,referencingCopyId:Object):yfiles.algorithms.Edge;
      /**
       * Callback method for creating a node of type {@link yfiles.multipage.NodeType#PROXY_REFERENCE}.
       * @param context an object providing relevant layout information, for example, the current graph as well as various information about the
       * graph elements
       * @param referringProxyId the ID of the proxy to which the created node refers
       * @returns the new proxy reference node
       * @see yfiles.multipage.NodeType#PROXY_REFERENCE
       * @see yfiles.multipage.LayoutContext#createNode
       * @see yfiles.multipage.NodeType#PROXY
       * @abstract
       */
      createProxyReferenceNode(context:yfiles.multipage.LayoutContext,referringProxyId:Object):yfiles.algorithms.Node;
    }
    var IElementFactory:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Interface that offers access to the information objects of graph elements.
     * @interface
     */
    export interface IElementInfoManager extends Object{
      /**
       * Returns the edge information object of the given edge.
       * @param edge the edge for which the information object should be returned
       * @returns the edge information object of the given edge
       * @abstract
       */
      getEdgeInfo(edge:yfiles.algorithms.Edge):yfiles.multipage.IEdgeInfo;
      /**
       * Returns the edge label information object of the given edge label layout.
       * @param edgeLabelLayout the edge label layout for which the information object should be returned
       * @returns the edge label information object of the given edge label layout
       * @abstract
       */
      getEdgeLabelInfo(edgeLabelLayout:yfiles.layout.IEdgeLabelLayout):yfiles.multipage.IEdgeLabelInfo;
      /**
       * Returns the node information object of the given node.
       * @param node the node for which the information object should be returned
       * @returns the node information object of the given node
       * @abstract
       */
      getNodeInfo(node:yfiles.algorithms.Node):yfiles.multipage.INodeInfo;
      /**
       * Returns the node label layout information object of the given node label layout.
       * @param nodeLabelLayout the node label layout for which the information object should be returned
       * @returns the node label information object of the given node label layout
       * @abstract
       */
      getNodeLabelInfo(nodeLabelLayout:yfiles.layout.INodeLabelLayout):yfiles.multipage.INodeLabelInfo;
    }
    var IElementInfoManager:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * Callback that is invoked when a {@link yfiles.multipage.MultiPageLayout} has calculated a new multi-page layout.
     * @interface
     */
    export interface ILayoutCallback extends Object{
      /**
       * Invoked from {@link yfiles.multipage.MultiPageLayout#applyLayout} when a new multi-page layout has been calculated.
       * @param result the result of the layout calculation
       * @see yfiles.multipage.MultiPageLayout#applyLayout
       * @abstract
       */
      layoutDone(result:yfiles.multipage.MultiPageLayoutResult):void;
    }
    var ILayoutCallback:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This interface offers access to information associated with a node.
     * <p>
     * The information can be either obtained from {@link yfiles.multipage.MultiPageLayoutResult#getNodeInfo} or
     * {@link yfiles.multipage.LayoutContext#getNodeInfo}.
     * </p>
     * @interface
     */
    export interface INodeInfo extends Object{
      /**
       * Gets the node of the input graph that is represented by the node associated with this {@link yfiles.multipage.INodeInfo info object}.
       * <p>
       * Note that, for the special nodes inserted to guarantee that no information is lost, there are no corresponding nodes in
       * the input graph. Hence, special nodes are mapped to common nodes in the following way:
       * </p>
       * <ul>
       * <li>A {@link yfiles.multipage.NodeType#CONNECTOR connector node} is mapped to the common node connected to the matching connector.</li>
       * <li>A {@link yfiles.multipage.NodeType#PROXY proxy node} is mapped to the associated common node (i.e., the proxy is a copy of that node).</li>
       * <li>A {@link yfiles.multipage.NodeType#PROXY_REFERENCE proxy reference node} is not mapped to any node, and, thus this method returns <code>null</code> for such nodes.</li>
       * </ul>
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      representedNode:yfiles.algorithms.Node;
      /**
       * Gets the page number of the {@link yfiles.multipage.MultiPageLayoutResult#getPage page graph} that contains the node associated with this {@link yfiles.multipage.INodeInfo info object}.
       * @abstract
       * @type {number}
       */
      pageNo:number;
      /**
       * Gets the type of the node associated with this {@link yfiles.multipage.INodeInfo info object}.
       * @abstract
       * @type {yfiles.multipage.NodeType}
       */
      type:yfiles.multipage.NodeType;
      /**
       * Gets the unique ID of the node associated with this {@link yfiles.multipage.INodeInfo info object}.
       * @see yfiles.multipage.MultiPageLayout#NODE_ID_DP_KEY
       * @abstract
       * @type {Object}
       */
      id:Object;
      /**
       * Gets the node that refers to the one associated with this {@link yfiles.multipage.INodeInfo info object}.
       * <p>
       * More precisely, if the node associated with this info object
       * </p>
       * <ul>
       * <li>is a {@link yfiles.multipage.NodeType#CONNECTOR connector node}, this method returns the matching connector node,</li>
       * <li>is a {@link yfiles.multipage.NodeType#PROXY proxy node}, it returns the corresponding {@link yfiles.multipage.NodeType#PROXY_REFERENCE proxy reference node},</li>
       * <li>is a {@link yfiles.multipage.NodeType#PROXY_REFERENCE proxy reference node}, it returns the corresponding {@link yfiles.multipage.NodeType#PROXY proxy node},</li>
       * <li>or otherwise, it returns <code>null</code>.</li>
       * </ul>
       * @see yfiles.multipage.NodeType#CONNECTOR
       * @see yfiles.multipage.NodeType#PROXY
       * @see yfiles.multipage.NodeType#PROXY_REFERENCE
       * @abstract
       * @type {yfiles.algorithms.Node}
       */
      referencingNode:yfiles.algorithms.Node;
    }
    var INodeInfo:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * A class that holds all information related to a node label.
     * @interface
     */
    export interface INodeLabelInfo extends Object{
      /**
       * Gets the unique ID of the associated node label.
       * @see yfiles.multipage.MultiPageLayoutResult#getNodeLabelInfo
       * @see yfiles.multipage.IElementInfoManager#getNodeLabelInfo
       * @abstract
       * @type {Object}
       */
      id:Object;
    }
    var INodeLabelInfo:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This class represents the layout context.
     * <p>
     * It provides several methods to access data as well as adding and removing elements.
     * </p>
     * @class
     * @implements {yfiles.multipage.IElementInfoManager}
     */
    export interface LayoutContext extends Object,yfiles.multipage.IElementInfoManager{}
    export class LayoutContext {
      /**
       * Adds a label to a given edge.
       * @param label the label that should be added to the given edge
       * @param labeledElement the edge to which the label should be added
       */
      addEdgeLabel(label:yfiles.layout.IEdgeLabelLayout,labeledElement:yfiles.algorithms.Edge):void;
      /**
       * Adds a label to a given node.
       * @param label the label that should be added to the given node
       * @param labeledElement the node to which the label should be added
       */
      addNodeLabel(label:yfiles.layout.INodeLabelLayout,labeledElement:yfiles.algorithms.Node):void;
      /**
       * All methods of the {@link yfiles.multipage.IElementFactory} have to use this method to create an edge.
       * @param source the source of the edge
       * @param target the target of the edge
       * @param sameDataElementId the ID of the edge whose data should be copied to the created edge (or <code>null</code> if no data should be copied)
       * @returns the created edge
       * @see yfiles.multipage.IElementFactory
       */
      createEdge(source:yfiles.algorithms.Node,target:yfiles.algorithms.Node,sameDataElementId:Object):yfiles.algorithms.Edge;
      /**
       * All methods of the {@link yfiles.multipage.IElementFactory} have to use this method to create a node.
       * @param sameDataElementId the ID of the node whose data should be copied to the created node (or <code>null</code> if no data should be copied)
       * @returns the created node
       * @see yfiles.multipage.IElementFactory
       */
      createNode(sameDataElementId:Object):yfiles.algorithms.Node;
      /**
       * 
       * @param edge
       * @returns 
       */
      getEdgeInfo(edge:yfiles.algorithms.Edge):yfiles.multipage.IEdgeInfo;
      /**
       * 
       * @param edgeLabelLayout
       * @returns 
       */
      getEdgeLabelInfo(edgeLabelLayout:yfiles.layout.IEdgeLabelLayout):yfiles.multipage.IEdgeLabelInfo;
      /**
       * 
       * @param node
       * @returns 
       */
      getNodeInfo(node:yfiles.algorithms.Node):yfiles.multipage.INodeInfo;
      /**
       * 
       * @param nodeLabelLayout
       * @returns 
       */
      getNodeLabelInfo(nodeLabelLayout:yfiles.layout.INodeLabelLayout):yfiles.multipage.INodeLabelInfo;
      /**
       * Returns the edge of the input graph with the given ID or <code>null</code> if there is no such edge.
       * @param id the ID of the edge that should be returned
       * @returns the edge of the input graph with the given ID or <code>null</code> if there is no such edge
       */
      getOriginalEdge(id:Object):yfiles.algorithms.Edge;
      /**
       * Returns the node of the input graph with the given ID or <code>null</code> if there is no such node.
       * @param id the ID of the node that should be returned
       * @returns the node of the input graph with the given ID or <code>null</code> if there is no such node
       */
      getOriginalNode(id:Object):yfiles.algorithms.Node;
      /**
       * Returns the edge of a page graph (a graph that represents the result of the multi-page layout) with the given ID or <code>null</code>
       * if there is no such edge.
       * @param id the ID of the edge that should be returned
       * @returns the edge of a page graph with the given ID or <code>null</code> if there is no such edge
       */
      getPageEdge(id:Object):yfiles.algorithms.Edge;
      /**
       * Returns the node of a page graph (a graph that represents the result of the multi-page layout) with the given ID or <code>null</code>
       * if there is no such node.
       * @param id the ID of the node that should be returned
       * @returns the node of a page graph with the given ID or <code>null</code> if there is no such node
       */
      getPageNode(id:Object):yfiles.algorithms.Node;
      /**
       * Removes a label from a given edge.
       * @param label the label that should be removed
       * @param labeledElement the edge owning the label
       * @returns 
       */
      removeEdgeLabel(label:yfiles.layout.IEdgeLabelLayout,labeledElement:yfiles.algorithms.Edge):boolean;
      /**
       * Removes a label from a given node.
       * @param label the label that should be removed
       * @param labeledElement the node owning the label
       * @returns 
       */
      removeNodeLabel(label:yfiles.layout.INodeLabelLayout,labeledElement:yfiles.algorithms.Node):boolean;
      /**
       * Gets the relevant graph.
       * @type {yfiles.layout.LayoutGraph}
       */
      graph:yfiles.layout.LayoutGraph;
      /**
       * Gets the layouter object that belongs to this layout context.
       * @type {yfiles.multipage.MultiPageLayout}
       */
      layout:yfiles.multipage.MultiPageLayout;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm subdivides the input graph into several {@link yfiles.layout.LayoutGraph}s (called page graphs) such that
     * the layout (calculated by the specified {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}) of each graph fits the {@link yfiles.multipage.MultiPageLayout#maximumPageSize specified maximum page size}.
     * <p>
     * The algorithm adds special nodes to guarantee that no information is lost when splitting edges that connect nodes placed
     * on different pages (for more details take a look at the concept described below). {@graph {"ann":{"s":[45,30],"d":1},"n":[[165,210,60,30,[[187.94,234.96,14.11,19.93,"G"]]],[271.88,210,30,30,[[280.18,234.96,13.39,19.93,"H"]]],[148.12,450,60,30,[[171.79,474.96,12.67,19.93,"V"]]],[166.88,375,[[183.04,399.96,12.67,19.93,"P"]]],[391.88,510,60,30,[[415.9,534.96,11.94,19.93,"Z"]]],[23.12,510,[[39.29,534.96,12.67,19.93,"Y"]]],[436.88,375,[[453.4,399.96,11.94,19.93,"T"]]],[376.88,375,30,30,[[385.54,399.96,12.67,19.93,"S"]]],[455.62,210,30,30,[[466.82,234.96,7.61,19.93,"I"]]],[30,450,60,30,[[53.31,474.96,13.39,19.93,"U"]]],[436.88,450,[[453.04,474.96,12.67,19.93,"X"]]],[15,375,30,30,[[23.31,399.96,13.39,19.93,"N"]]],[455.62,285,30,30,[[463.21,309.96,14.83,19.93,"M"]]],[350.62,285,75,30,[[382.51,309.96,11.23,19.93,"L"]]],[165,285,[[182.25,309.96,10.5,19.93,"J"]]],[166.88,135,75,30,[[198.04,159.96,12.67,19.93,"E"]]],[241.88,375,30,30,[[249.82,399.96,14.11,19.93,"Q"]]],[75,375,30,30,[[82.94,399.96,14.11,19.93,"O"]]],[120,60,60,30,[[143.31,84.96,13.39,19.93,"C"]]],[33.75,0,[[49.91,24.96,12.67,19.93,"A"]]],[358.12,450,[[372.49,474.96,16.27,19.93,"W"]]],[75,135,[[90.81,159.96,13.39,19.93,"D"]]],[256.88,285,60,30,[[280.54,309.96,12.67,19.93,"K"]]],[301.88,375,[[317.68,399.96,13.39,19.93,"R"]]],[45,60,[[61.16,84.96,12.67,19.93,"B"]]],[271.88,135,30,30,[[280.9,159.96,11.94,19.93,"F"]]]],"e":[[11,9,0,15,-20,-15,[30,420,40,420]],[13,6,25,15,-11.25,-15,[413.12,330,448.12,330]],[7,20,0,15,11.25,-15],[25,1,0,15,0,-15],[13,23,-25,15,11.25,-15,[363.12,330,335.62,330]],[10,4,0,15,20,-15,[459.38,495,441.88,495]],[1,22,0,15,0,-15],[22,16,-20,15,0,-15,[266.88,330,256.88,330]],[14,3,11.25,15,-11.25,-15,[198.75,345,178.12,345]],[0,14,0,15,11.25,-15,[195,255,198.75,255]],[17,9,0,15,20,-15,[90,420,80,420]],[3,2,-11.25,15,0,-15],[24,9,-11.25,15,0,-15,[56.25,105,60,105]],[3,20,11.25,15,-11.25,-15,[200.62,420,369.38,420]],[18,15,20,15,0,-15,[170,105,204.38,105]],[15,0,-9.38,15,0,-15],[9,5,0,15,11.25,-15,[60,495,56.88,495]],[15,2,-28.12,15,-20,-15,[176.25,180,150,180,150,420,158.12,420]],[12,6,0,15,11.25,-15],[0,13,20,15,-18.75,-15,[215,255,369.38,255]],[14,17,-11.25,15,0,-15,[176.25,330,90,330]],[15,3,9.38,15,11.25,-15,[213.75,195,240,195,240,360,200.62,360]],[24,21,11.25,15,-11.25,-15,[78.75,105,86.25,105]],[19,11,-11.25,15,0,-15,[45,45,30,45]],[15,13,28.12,15,18.75,-15,[232.5,180,406.88,180]],[22,23,20,15,-11.25,-15,[306.88,330,313.12,330]],[19,24,11.25,15,0,-15],[0,5,-20,15,-11.25,-15,[175,255,0,255,0,495,34.38,495]],[8,12,0,15,0,-15],[18,21,-20,15,11.25,-15,[130,105,108.75,105]],[18,14,0,15,-11.25,-15,[150,120,135,120,135,270,176.25,270]],[20,4,0,15,-20,-15,[380.62,495,401.88,495]],[13,4,0,15,0,-15,[388.12,345,421.88,345]],[22,2,0,15,20,-15,[286.88,435,198.12,435]]],"vp":[0.0,0.0,486.0,540.0]}}
     * </p>
     * <p>
     * The input graph.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"styles":{"1":{"c": 1}},"n":[[523.12,293.69,500,500,2,[[523.12,316.06,500,22.38,"Page 2"]]],[-80.94,293.69,500,500,2,[[-80.94,316.06,500,22.38,"Page 1"]]],[261.56,588.69,45,30,[[277.73,613.65,12.67,19.93,"X"]]],[239.06,528.69,[[247.73,553.65,12.67,19.93,"S"]]],[186.56,588.69,45,30,[[200.93,613.65,16.27,19.93,"W"]]],[164.06,528.69,45,30,[[180.23,553.65,12.67,19.93,"P"]]],[15.94,458.69,75,30,[[47.1,483.65,12.67,19.93,"E"]]],[99.06,528.69,1,[[107.4,553.04,13.33,18.7,"G"]]],[29.06,528.69,1,[[38.73,553.04,10.67,18.7,"L"]]],[38.44,398.69,1,[[47.1,423.04,12.67,18.7,"C"]]],[322.81,458.69,45,30,[[340.06,483.65,10.5,19.93,"J"]]],[271.56,398.69,1,[[279.9,423.04,13.33,18.7,"G"]]],[341.56,398.69,1,[[350.23,423.04,12.67,18.7,"C"]]],[179.06,648.69,60,30,[[203.09,673.65,11.94,19.93,"Z"]]],[121.56,588.69,1,[[131.23,613.04,10.67,18.7,"L"]]],[341.56,528.69,[[349.51,553.65,14.11,19.93,"O"]]],[341.56,588.69,1,[[350.23,613.04,12.67,18.7,"U"]]],[-40.94,528.69,1,[[-31.94,553.04,12,18.7,"K"]]],[-20.94,588.69,60,30,[[2.73,613.65,12.67,19.93,"V"]]],[961.88,408.69,[[973.07,433.65,7.61,19.93,"I"]]],[961.88,468.69,[[969.46,493.65,14.83,19.93,"M"]]],[943.12,528.69,45,30,[[959.65,553.65,11.94,19.93,"T"]]],[855.62,468.69,75,30,[[887.51,493.65,11.23,19.93,"L"]]],[878.12,528.69,1,[[887.46,553.04,11.33,18.7,"Z"]]],[896.88,408.69,1,[[905.87,433.04,12,18.7,"E"]]],[775.62,408.69,60,30,[[798.57,433.65,14.11,19.93,"G"]]],[790.62,468.69,1,[[800.62,493.04,10,18.7,"J"]]],[790.62,358.69,1,[[799.62,383.04,12,18.7,"E"]]],[798.12,528.69,45,30,[[813.93,553.65,13.39,19.93,"R"]]],[620.62,468.69,60,30,[[644.29,493.65,12.67,19.93,"K"]]],[570.62,528.69,1,[[579.62,553.04,12,18.7,"V"]]],[710.62,468.69,45,30,[[726.79,493.65,12.67,19.93,"Y"]]],[635.62,408.69,[[643.93,433.65,13.39,19.93,"H"]]],[635.62,528.69,[[643.57,553.65,14.11,19.93,"Q"]]],[635.62,358.69,[[644.65,383.65,11.94,19.93,"F"]]],[615.62,728.69,60,30,[[638.93,753.65,13.39,19.93,"U"]]],[630.62,668.69,1,[[638.96,693.04,13.33,18.7,"O"]]],[695.62,668.69,45,30,[[711.79,693.65,12.67,19.93,"B"]]],[565.62,668.69,[[573.93,693.65,13.39,19.93,"N"]]],[718.12,728.69,45,30,[[733.93,753.65,13.39,19.93,"D"]]],[783.12,668.69,60,30,[[806.43,693.65,13.39,19.93,"C"]]],[798.12,728.69,1,[[807.12,753.04,12,18.7,"E"]]],[868.12,728.69,1,[[878.12,753.04,10,18.7,"J"]]],[569.38,608.69,45,30,[[585.54,633.65,12.67,19.93,"A"]]]],"e":[[9,6,0,15,0,-15],[4,13,0,15,0,-15],[15,16,0,15,0,-15],[3,4,0,15,11.25,-15,[254.06,568.69,220.31,568.69]],[12,10,0,15,11.25,-15],[6,7,9.38,15,0,-15,[62.81,508.69,114.06,508.69]],[10,15,11.25,15,0,-15],[5,4,11.25,15,-11.25,-15],[11,10,0,15,-11.25,-15,[286.56,438.69,334.06,438.69]],[17,18,0,15,-20,-15,[-25.94,568.69,-10.94,568.69]],[6,8,-9.38,15,0,-15],[14,13,0,15,-20,-15,[136.56,628.69,189.06,628.69]],[2,13,0,15,20,-15,[284.06,628.69,229.06,628.69]],[6,5,28.12,15,-11.25,-15,[81.56,498.69,175.31,498.69]],[5,18,-11.25,15,20,-15,[175.31,568.69,29.06,568.69]],[10,5,-11.25,15,11.25,-15,[334.06,498.69,197.81,498.69]],[6,18,-28.12,15,0,-15,[25.31,498.69,9.06,498.69]],[27,25,0,15,0,-15],[34,32,0,15,0,-15],[19,20,0,15,0,-15],[22,23,0,15,0,-15],[25,26,0,15,0,-15],[32,29,0,15,0,-15],[29,33,0,15,0,-15],[36,35,0,15,0,-15],[40,41,0,15,0,-15],[20,21,0,15,11.25,-15],[43,38,-11.25,15,0,-15],[24,22,0,15,18.75,-15],[37,39,11.25,15,-11.25,-15],[29,30,-20,15,0,-15,[630.62,508.69,585.62,508.69]],[38,35,0,15,-20,-15,[580.62,708.69,625.62,708.69]],[40,42,20,15,0,-15,[833.12,708.69,883.12,708.69]],[22,21,25,15,-11.25,-15,[918.12,508.69,954.38,508.69]],[22,28,-25,15,11.25,-15,[868.12,508.69,831.88,508.69]],[25,31,-20,15,11.25,-15,[785.62,448.69,744.38,448.69]],[37,35,-11.25,15,20,-15,[706.88,708.69,665.62,708.69]],[40,39,-20,15,11.25,-15,[793.12,708.69,751.88,708.69]],[25,22,20,15,-18.75,-15,[825.62,448.69,874.38,448.69]],[43,37,11.25,15,0,-15,[603.12,648.69,718.12,648.69]],[29,28,20,15,-11.25,-15,[670.62,508.69,809.38,508.69]],[35,31,-30,0,-14.5,-15,[555.62,743.69,555.62,348.69,718.62,348.69]]],"vp":[-81.0,293.0,1105.0,501.0]}}
     * </p>
     * <p>
     * The input graph is subdivided into two pages with size <code>500x500</code>. Circular nodes denote the so-called {@link yfiles.multipage.NodeType#CONNECTOR connectors} that
     * split edges to nodes placed on other pages. The label of a connector corresponds to that of the opposite node of the
     * split edge.
     * </p>
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * This multi-page layout algorithm subdivides the input graph into several smaller ones and applies existing layout
     * algorithms to each of them. Hence, its layout style heavily depends on the selected {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm} used for each
     * single page. Furthermore, the algorithm uses several refinement steps to produce more compact results.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * Similar to the layout style, the supported feature set mainly depends on the features supported by the specified
     * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}. Note that due to the underlying approach of the {@link yfiles.multipage.MultiPageLayout} it doesn't
     * support {@link yfiles.layout.PartitionGrid}s.
     * </p>
     * <p>
     * <b>Concept</b>
     * </p>
     * <p>
     * To guarantee that no information is lost, this layout algorithm replaces edges between nodes on different pages by
     * so-called {@link yfiles.multipage.NodeType#CONNECTOR connector nodes}. Furthermore, it may replicate nodes (the clones are called {@link yfiles.multipage.NodeType#PROXY proxy nodes}) and insert special
     * nodes to refer to such nodes (so-called {@link yfiles.multipage.NodeType#PROXY_REFERENCE proxy reference nodes}).
     * </p>
     * <p>
     * Unlike other yFiles layout algorithms, {@link yfiles.multipage.MultiPageLayout} does not modify its input graph but returns its
     * result as a {@link yfiles.multipage.MultiPageLayoutResult}. To be able to profit as much as possible from existing layout
     * support, this layout algorithm implements the {@link yfiles.layout.ILayoutAlgorithm} interface, although method {@link yfiles.layout.ILayoutAlgorithm#applyLayout doLayout} does
     * not specify a return value. Therefore, client code has to {@link yfiles.multipage.MultiPageLayout#layoutCallback register} a {@link yfiles.multipage.ILayoutCallback} that is invoked
     * when the algorithm has calculated a new multi-page result.
     * </p>
     * @class
     * @extends {yfiles.layout.LayoutStageBase}
     */
    export interface MultiPageLayout extends yfiles.layout.LayoutStageBase{}
    export class MultiPageLayout {
      /**
       * Creates a new {@link yfiles.multipage.MultiPageLayout} instance.
       * @param core the layout algorithm used for a single page
       */
      constructor(core:yfiles.layout.ILayoutAlgorithm);
      /**
       * This method is called to further improve the layout results.
       * <p>
       * The marked nodes have to be placed without changing the coordinates of the fixed (non-marked) elements and without
       * exceeding the specified maximum page size. Furthermore, the basic layout properties should be maintained.
       * </p>
       * <p>
       * Subclasses may implement a custom layout strategy for this step.
       * </p>
       * @param graph the input graph
       * @param incrementalNodesDP a {@link } that returns a boolean value indicating whether or not a node is marked; the positions
       * of the non-marked nodes are not allowed to change
       * @param context the current layout context
       * @see yfiles.multipage.LayoutContext
       * @protected
       */
      applyIncrementalLayout(graph:yfiles.layout.LayoutGraph,incrementalNodesDP:yfiles.algorithms.IDataProvider,context:yfiles.multipage.LayoutContext):void;
      /**
       * Calculates a new multi-page layout for the specified graph.
       * <p>
       * Unlike method {@link yfiles.multipage.MultiPageLayout#applyLayout}, this method ignores the registered {@link yfiles.multipage.MultiPageLayout#layoutCallback layout callback} and
       * directly returns the calculated {@link yfiles.multipage.MultiPageLayoutResult}.
       * </p>
       * @param graph the input graph
       * @returns an instance of {@link } that represents the result of the layout run
       * @throws {Stubs.Exceptions.ArgumentError} if for one of the keys {@link yfiles.multipage.MultiPageLayout#NODE_ID_DP_KEY}, {@link yfiles.multipage.MultiPageLayout#EDGE_ID_DP_KEY},
       * {@link yfiles.multipage.MultiPageLayout#NODE_LABEL_ID_DP_KEY} and {@link yfiles.multipage.MultiPageLayout#EDGE_LABEL_ID_DP_KEY} no {@link yfiles.algorithms.IDataProvider}
       * is registered with the given graph
       * @see yfiles.multipage.MultiPageLayoutResult
       */
      calculateLayout(graph:yfiles.layout.LayoutGraph):yfiles.multipage.MultiPageLayoutResult;
      /**
       * This method creates the {@link yfiles.multipage.IElementFactory element factory} for multi-page layouts.
       * <p>
       * It is called once from {@link yfiles.multipage.MultiPageLayout#elementFactory} if no factory has been explicitly set using {@link yfiles.multipage.MultiPageLayout#elementFactory}. By default, this method returns an instance of
       * {@link yfiles.multipage.DefaultElementFactory}.
       * </p>
       * <p>
       * Subclasses may create a custom implementation of the element factory.
       * </p>
       * @returns the created element factory
       * @see yfiles.multipage.MultiPageLayout#elementFactory
       * @see yfiles.multipage.MultiPageLayout#elementFactory
       * @see yfiles.multipage.IElementFactory
       * @protected
       */
      createElementFactory():yfiles.multipage.IElementFactory;
      /**
       * This method is called during a postprocessing step that reduces the number of connectors.
       * <p>
       * A pair of connectors can be removed if both connector nodes are placed on the same page. The method removes both
       * connector nodes and restores the original edges by calling {@link yfiles.multipage.MultiPageLayout#routeRestoredEdges}.
       * </p>
       * <p>
       * Subclasses may implement a custom remove/restore strategy.
       * </p>
       * @param connector1 the first connector of the connector pair
       * @param connector2 the second connector of the connector pair
       * @param originalEdgeIds the IDs of the original edges that have to be restored
       * @param context the current layout context
       * @returns <code>true</code> if the removal of the connector pair was successful, <code>false</code> otherwise
       * @see yfiles.multipage.MultiPageLayout#routeRestoredEdges
       * @protected
       */
      removeConnectorPair(connector1:yfiles.algorithms.Node,connector2:yfiles.algorithms.Node,originalEdgeIds:yfiles.algorithms.YList,context:yfiles.multipage.LayoutContext):boolean;
      /**
       * This method is called whenever some single edges have to be routed without changing the position of nodes (e.g., by {@link yfiles.multipage.MultiPageLayout#removeConnectorPair}
       * to route the restored edges).
       * <p>
       * It has to guarantee that the routes are fully contained within the given bounding rectangle.
       * </p>
       * <p>
       * Subclasses may implement a custom routing strategy for the restored edges.
       * </p>
       * @param graph the relevant graph
       * @param selectedEdgesDP a {@link } that returns a boolean value indicating whether or not an edge should be rerouted
       * @param boundingRectangle the edge routes should be fully contained within this rectangle
       * @see yfiles.multipage.MultiPageLayout#removeConnectorPair
       * @protected
       */
      routeRestoredEdges(graph:yfiles.layout.LayoutGraph,selectedEdgesDP:yfiles.algorithms.IDataProvider,boundingRectangle:yfiles.algorithms.YRectangle):void;
      /**
       * Data provider key for assigning a cluster ID to common (non-group) nodes
       * @see yfiles.multipage.MultiPageLayout#strictClusterSeparation
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static NODE_CLUSTER_ID_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Data provider key for mapping each node of the input graph to a unique ID
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static NODE_ID_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Data provider key for mapping each edge of the input graph to a unique ID
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static EDGE_ID_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Data provider key for mapping each edge label of the input graph to a unique ID
       * @const
       * @static
       * @type {yfiles.algorithms.IEdgeLabelLayoutDpKey.<Object>}
       */
      static EDGE_LABEL_ID_DP_KEY:yfiles.algorithms.IEdgeLabelLayoutDpKey<Object>;
      /**
       * Data provider key for mapping each node label of the input graph to a unique ID
       * @const
       * @static
       * @type {yfiles.algorithms.INodeLabelLayoutDpKey.<Object>}
       */
      static NODE_LABEL_ID_DP_KEY:yfiles.algorithms.INodeLabelLayoutDpKey<Object>;
      /**
       * Data provider key for specifying the type of multi-edges
       * <p>
       * If {@link yfiles.multipage.MultiPageLayout#edgeBundleModeMask} &amp; {@link yfiles.multipage.EdgeBundleModes#DISTINGUISH_TYPES} == 1, multi-edges (edges with same endpoints) associated with
       * different types (objects) are distinguished, i.e., they are split by different connector pairs (see
       * {@link yfiles.multipage.NodeType#CONNECTOR}).
       * </p>
       * @see yfiles.multipage.EdgeBundleModes#DISTINGUISH_TYPES
       * @see yfiles.multipage.MultiPageLayout#edgeBundleModeMask
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<Object>}
       */
      static EDGE_TYPE_DP_KEY:yfiles.algorithms.EdgeDpKey<Object>;
      /**
       * Gets or sets whether or not the algorithm should separate nodes with different {@link yfiles.multipage.MultiPageLayout#NODE_CLUSTER_ID_DP_KEY cluster IDs}.
       * <p>
       * More precisely, if this option is enabled, the algorithm doesn't place nodes with different cluster IDs onto the same
       * page. Otherwise, such a placement is possible.
       * </p>
       * @see yfiles.multipage.MultiPageLayout#NODE_CLUSTER_ID_DP_KEY
       * @type {boolean}
       */
      strictClusterSeparation:boolean;
      /**
       * Gets or sets whether or not the given {@link yfiles.multipage.MultiPageLayout#labeling labeling algorithm} places the labels of the input graph.
       * <p>
       * If this option is disabled, the labels are either ignored or placed by the integrated labeling approach of the
       * {@link yfiles.layout.LayoutStageBase#coreLayout core layout algorithm}.
       * </p>
       * @see yfiles.multipage.MultiPageLayout#labeling
       * @type {boolean}
       */
      labelingEnabled:boolean;
      /**
       * Gets or sets the {@link yfiles.multipage.IElementFactory element factory} for creating special nodes and edges in a multi-page layout.
       * <p>
       * If no element factory is set explicitly (see method {@link yfiles.multipage.MultiPageLayout#elementFactory}), a new instance is created and set by method {@link yfiles.multipage.MultiPageLayout#createElementFactory}
       * during the first layout run.
       * </p>
       * @see yfiles.multipage.MultiPageLayout#createElementFactory
       * @see yfiles.multipage.IElementFactory
       * @type {yfiles.multipage.IElementFactory}
       */
      elementFactory:yfiles.multipage.IElementFactory;
      /**
       * Gets or sets the {@link yfiles.multipage.ILayoutCallback callback} that is notified upon completion of multi-page layout calculation runs.
       * @see yfiles.multipage.ILayoutCallback
       * @type {yfiles.multipage.ILayoutCallback}
       */
      layoutCallback:yfiles.multipage.ILayoutCallback;
      /**
       * Gets or sets the bit mask for defining edge bundles.
       * <p>
       * All multi-edges (edges with same endpoints) belonging to the same edge bundle are split by the same connector pair (see
       * {@link yfiles.multipage.NodeType#CONNECTOR}).
       * </p>
       * @see yfiles.multipage.NodeType#CONNECTOR
       * @type {yfiles.multipage.EdgeBundleModes}
       */
      edgeBundleModeMask:yfiles.multipage.EdgeBundleModes;
      /**
       * Gets or sets how to handle special nodes (like connector and proxy nodes) with respect to groups.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified group mode is unknown
       * @see yfiles.multipage.NodeType#CONNECTOR
       * @see yfiles.multipage.NodeType#PROXY
       * @type {yfiles.multipage.GroupingMode}
       */
      groupingMode:yfiles.multipage.GroupingMode;
      /**
       * Gets or sets the preferred time limit (in milliseconds) for the layout algorithm.
       * <p>
       * The specified value has to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the preferred time limit is negative
       * @type {number}
       */
      maximumDuration:number;
      /**
       * Gets or sets the {@link yfiles.layout.ILayoutStage} that places the labels of the input graph.
       * @see yfiles.multipage.MultiPageLayout#labelingEnabled
       * @see yfiles.labeling.GenericLabeling
       * @type {yfiles.layout.ILayoutStage}
       */
      labeling:yfiles.layout.ILayoutStage;
      /**
       * Gets or sets the maximum size of a single page.
       * <p>
       * The layout algorithm subdivides the input graph such that each part is placed on a different page that fits the
       * specified maximum size.
       * </p>
       * <p>
       * Both the specified width and height have to be positive.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified width or height is not positive
       * @see yfiles.multipage.MultiPageLayout#maximumDuration
       * @type {yfiles.algorithms.YDimension}
       */
      maximumPageSize:yfiles.algorithms.YDimension;
      static $class:yfiles.lang.Class;
    }
    /**
     * This class represents the result of a layout run of {@link yfiles.multipage.MultiPageLayout}.
     * <p>
     * A layout run subdivides the input graph into several {@link yfiles.layout.LayoutGraph}s (called page graphs) such that the
     * layout of each graph fits the {@link yfiles.multipage.MultiPageLayout#maximumPageSize specified maximum page size}. This class provides access to the different page graphs (see
     * method {@link yfiles.multipage.MultiPageLayoutResult#getPage}).
     * </p>
     * <p>
     * An instance of this class can be either obtained by calling {@link yfiles.multipage.MultiPageLayout#calculateLayout} or
     * {@link yfiles.multipage.MultiPageLayout#applyLayout}. In the latter case, a {@link yfiles.multipage.ILayoutCallback} has to be {@link yfiles.multipage.MultiPageLayout#layoutCallback set}
     * before starting the layout run.
     * </p>
     * @class
     * @implements {yfiles.multipage.IElementInfoManager}
     */
    export interface MultiPageLayoutResult extends Object,yfiles.multipage.IElementInfoManager{}
    export class MultiPageLayoutResult {
      /**
       * 
       * @param edge
       * @returns 
       */
      getEdgeInfo(edge:yfiles.algorithms.Edge):yfiles.multipage.IEdgeInfo;
      /**
       * 
       * @param edgeLabelLayout
       * @returns 
       */
      getEdgeLabelInfo(edgeLabelLayout:yfiles.layout.IEdgeLabelLayout):yfiles.multipage.IEdgeLabelInfo;
      /**
       * 
       * @param node
       * @returns 
       */
      getNodeInfo(node:yfiles.algorithms.Node):yfiles.multipage.INodeInfo;
      /**
       * 
       * @param nodeLabelLayout
       * @returns 
       */
      getNodeLabelInfo(nodeLabelLayout:yfiles.layout.INodeLabelLayout):yfiles.multipage.INodeLabelInfo;
      /**
       * Returns the page graph associated with the given page number.
       * <p>
       * A <i>page graph</i> is a subgraph of the input graph whose layout fits the {@link yfiles.multipage.MultiPageLayout#maximumPageSize specified maximum page size}.
       * </p>
       * @param pageNo the given page number
       * @returns the page graph associated with the given page number
       * @throws {Stubs.Exceptions.IndexOutOfRangeError} if <code>pageNo >= pageCount</code> or <code>pageNo &lt; 0</code>
       * @see yfiles.multipage.MultiPageLayoutResult#pageCount
       */
      getPage(pageNo:number):yfiles.layout.LayoutGraph;
      /**
       * Returns the number of pages available in this {@link yfiles.multipage.MultiPageLayoutResult}.
       * <p>
       * Each page contains a {@link yfiles.multipage.MultiPageLayoutResult#getPage page graph} whose layout fits the {@link yfiles.multipage.MultiPageLayout#maximumPageSize specified maximum page size}.
       * </p>
       * @returns the number of available pages
       * @see yfiles.multipage.MultiPageLayoutResult#getPage
       */
      pageCount():number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.multipage.MultiPageLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface MultiPageLayoutData extends yfiles.layout.LayoutData{}
    export class MultiPageLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from nodes to their unique IDs.
       * @see yfiles.multipage.MultiPageLayout#NODE_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,Object>}
       */
      nodeIds:yfiles.layout.ItemMapping<yfiles.graph.INode,Object>;
      /**
       * Gets or sets the mapping from edges to their unique IDs.
       * @see yfiles.multipage.MultiPageLayout#EDGE_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      edgeIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the mapping from node labels to their unique IDs.
       * @see yfiles.multipage.MultiPageLayout#NODE_LABEL_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,Object>}
       */
      nodeLabelIds:yfiles.layout.ItemMapping<yfiles.graph.ILabel,Object>;
      /**
       * Gets or sets the mapping from edge labels to their unique IDs.
       * @see yfiles.multipage.MultiPageLayout#EDGE_LABEL_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,Object>}
       */
      edgeLabelIds:yfiles.layout.ItemMapping<yfiles.graph.ILabel,Object>;
      /**
       * Gets or sets the mapping from nodes to their cluster id.
       * <p>
       * Nodes with the same cluster id should preferably be placed on the same page.
       * </p>
       * @see yfiles.multipage.MultiPageLayout#NODE_CLUSTER_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,Object>}
       */
      nodeClusterIds:yfiles.layout.ItemMapping<yfiles.graph.INode,Object>;
      /**
       * Gets or sets the mapping from edges to their type.
       * <p>
       * Multi-edges (edges with same endpoints) of different kind are distinguished if {@link yfiles.multipage.MultiPageLayout#edgeBundleModeMask} &amp; {@link yfiles.multipage.EdgeBundleModes#DISTINGUISH_TYPES}
       * == 1.
       * </p>
       * @see yfiles.multipage.MultiPageLayout#EDGE_TYPE_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      edgeTypes:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      static $class:yfiles.lang.Class;
    }
  }export namespace circular{
    /**
     * This layout algorithm arranges graphs in a circular fashion.
     * <p>
     * <b>Layout Style</b> The nodes are arranged in circles and stars which emphasize group and tree structures inside a graph. Circular layout
     * algorithms find applications in many areas such as social networking, network management, WWW visualization, eCommerce,
     * telecommunications. {@graph {"ann": {"s": [30,30], "d": 0, "c": 1},"n": [[558.09,362.1], [95.35,12.18], [0,148.17], [441.83,243.3], [540.68,299.26], [142.21,0], [298.88,449.09], [347.79,497.99], [232.08,466.99], [143.67,101.87], [100.64,154.2], [137.11,211.3], [206.74,126.62], [202.69,194.26], [263.08,582.7], [329.89,564.8], [214.18,533.79], [453.2,487.29], [528.46,21.5], [587.42,54.88], [478.49,67.27], [573.89,121.27], [506.57,128.93], [603.52,281.85], [620.93,344.69], [285.81,262.6], [259.33,689.12], [213.12,676.73], [311.63,331.4], [358.3,274.64]], "e": [[4,0], [4,3], [6,7], [6,8], [8,7], [1,9], [9,5], [9,10], [10,2], [10,11], [12,9], [11,13], [13,12], [15,14], [15,7], [14,16], [16,8], [16,7], [7,17], [18,19], [20,18], [19,21], [21,20], [20,22], [22,21], [22,3], [4,23], [0,24], [24,23], [25,13], [14,26], [14,27], [25,28], [28,6], [28,29], [29,25], [29,3]], "vp": [0.0,0.0,651.0,720.0]}} Circular layout with default settings {@graph {"ann": {"s": [30,30], "d": 0,  "c" : 1},"n": [[452.67,672.8], [684.06,272.02], [661.72,203.26], [518.72,643.4], [29.9,484.57], [238.95,672.8], [661.72,484.57], [238.95,15.03], [0,343.92], [309.66,0], [114.42,600.9], [29.9,203.26], [452.67,15.03], [7.56,415.81], [114.42,86.93], [309.66,687.83], [381.96,0], [625.58,140.65], [172.91,643.4], [577.2,86.93], [381.96,687.83], [691.62,343.92], [7.56,272.02], [577.2,600.9], [625.58,547.18], [66.04,140.65], [684.06,415.81], [66.04,547.18], [518.72,44.44], [172.91,44.44]], "e": [[1,0], [1,2], [3,0], [1,5], [5,4], [4,6], [8,4], [9,7], [1,12], [13,4], [4,15], [5,15], [10,18], [18,7], [18,5], [18,1], [17,19], [19,2], [15,20], [20,0], [4,21], [8,21], [21,6], [11,22], [22,8], [20,23], [23,3], [23,13], [23,4], [23,24], [6,24], [14,25], [25,11], [25,22], [22,26], [2,26], [17,26], [26,24], [26,21], [1,27], [27,10], [27,18], [12,28], [28,2], [28,19], [7,29], [12,29], [29,14], [29,16], [29,18]], "vp": [0.0,0.0,722.0,718.0]}} Circular layout with {@link yfiles.circular.LayoutStyle#SINGLE_CYCLE}
     * layout style <b>Concept</b>The layout algorithm performs three steps when calculating a circular arrangement for a graph:
     * </p>
     * <ol>
     * <li>
     * It searches for partitions in the input graph depending on connectivity and on the given
     * {@link yfiles.circular.CircularLayout#layoutStyle layout style}. Considering each partition as a node, the resulting graph has a tree-like structure.
     * </li>
     * <li>The partitions are laid out as circles using the selected {@link yfiles.circular.CircularLayout#partitionStyle partition style}.</li>
     * <li>
     * The algorithm delegates the layout calculation for the underlying tree (in which each node corresponds to a partition)
     * to a specialized layout algorithm accessible by {@link yfiles.circular.CircularLayout#singleCycleLayout} or {@link yfiles.circular.CircularLayout#balloonLayout}. The partitions are moved to their final location.
     * </li>
     * </ol>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The layout algorithm places the nodes in circles that represent a partition. There are several ways to find partitions
     * in the input graph. Which one is applied is defined using {@link yfiles.circular.CircularLayout#layoutStyle}.
     * </p>
     * <p>
     * The nodes in a partition can either lie on or in the interior of a circle. The placement of the nodes affects the
     * compactness of the layout and can be specified using {@link yfiles.circular.CircularLayout#partitionStyle}.
     * </p>
     * <p>
     * Since edges are routed as straight lines, they may overlap with nodes or node labels. To resolve these overlaps, an edge
     * routing algorithm (e.g. {@link yfiles.router.EdgeRouter} or {@link yfiles.router.OrganicEdgeRouter}) can be appended.
     * </p>
     * <p>
     * This layout algorithm supports {@link yfiles.circular.CircularLayout#edgeBundling edge bundling}. In order to bundle the edges, the nodes of the graph are clustered in
     * groups. Edge bundling is supported only if {@link yfiles.circular.CircularLayout#partitionStyle partition layout style} is set to {@link yfiles.circular.PartitionStyle#CYCLE} and {@link yfiles.circular.CircularLayout#layoutStyle layout style}
     * is other than {@link yfiles.circular.LayoutStyle#BCC_ISOLATED}.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface CircularLayout extends yfiles.layout.MultiStageLayout{}
    export class CircularLayout {
      /**
       * Creates a new {@link yfiles.circular.CircularLayout} instance with the default settings.
       */
      constructor();
      /**
       * Data acceptor key for publishing the final circle information
       * <p>
       * For each node, the ID of the circle on which it was placed will be stored.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<number>}
       */
      static CIRCLE_ID_DP_KEY:yfiles.algorithms.NodeDpKey<number>;
      /**
       * Data provider key for defining custom node partitions
       * <p>
       * The {@link yfiles.algorithms.IDataProvider} needs to provide a mapping from {@link yfiles.algorithms.Node}s to unique {@link Object} IDs for each
       * group of nodes. These groups will form the circles in the layout.
       * </p>
       * @see yfiles.circular.LayoutStyle#CUSTOM_GROUPS
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<Object>}
       */
      static CUSTOM_GROUPS_DP_KEY:yfiles.algorithms.NodeDpKey<Object>;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for node labels preventing possible overlaps.
       * <p>
       * Nodes get temporarily enlarged such that they contain their labels.
       * </p>
       * <p>
       * This might result in layouts that need much space.
       * </p>
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not, in the underlying tree, the children of a tree node are placed on a common radius.
       * <p>
       * Disabling this feature may produce more compact layout results.
       * </p>
       * @type {boolean}
       */
      placeChildrenOnCommonRadius:boolean;
      /**
       * Gets or sets whether or not to take the coordinates of the input diagram into account when arranging the nodes of the
       * partitions and the partitions themselves.
       * <p>
       * If enabled:
       * </p>
       * <ul>
       * <li>
       * For complex partitions (those consisting of more than one node), the layout algorithm tries to keep peripheral nodes and
       * maintain their circular order around the center of the disk/circle. Other partitions that connect to this node are moved
       * accordingly, if possible.
       * </li>
       * <li>
       * For multiple partitions that connect to the same node, the layout algorithm tries to keep their circular order around
       * this node. This only works as expected for {@link yfiles.circular.LayoutStyle#BCC_COMPACT} as layout style, since otherwise the
       * underlying tree structure is not well defined.
       * </li>
       * </ul>
       * @type {boolean}
       */
      fromSketchMode:boolean;
      /**
       * Gets or sets the maximum deviation angle allowed for an edge.
       * <p>
       * The deviation angle for an edge is the difference between its optimal angle away from the parent cycle and its actual
       * angle.
       * </p>
       * @type {number}
       */
      maximumDeviationAngle:number;
      /**
       * Gets or sets the global layout style for this layout algorithm.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown layout style is set
       * @type {yfiles.circular.LayoutStyle}
       */
      layoutStyle:yfiles.circular.LayoutStyle;
      /**
       * Gets the {@link yfiles.circular.SingleCycleLayout} instance used for laying out nodes on a single cycle.
       * <p>
       * Configuring this layout algorithm only affects layouts with only one partition.
       * </p>
       * @see yfiles.circular.CircularLayout#layoutStyle
       * @type {yfiles.circular.SingleCycleLayout}
       */
      singleCycleLayout:yfiles.circular.SingleCycleLayout;
      /**
       * Gets the {@link yfiles.tree.BalloonLayout} instance used for arranging multiple partitions.
       * <p>
       * Configuring this layout algorithm affects the arrangement of the partitions within the overall layout.
       * </p>
       * @see yfiles.circular.CircularLayout#layoutStyle
       * @type {yfiles.tree.BalloonLayout}
       */
      balloonLayout:yfiles.tree.BalloonLayout;
      /**
       * Gets or sets the style for the arrangement of each partition.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown style is given
       * @type {yfiles.circular.PartitionStyle}
       */
      partitionStyle:yfiles.circular.PartitionStyle;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for hiding group nodes is activated.
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @see yfiles.layout.MultiStageLayout#hideGroupsStage
       * @see yfiles.layout.HideGroupsStage
       * @type {boolean}
       */
      hideGroupsStageEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      /**
       * Gets the {@link yfiles.layout.EdgeBundling} instance that defines the settings of the edge bundling feature.
       * <p>
       * The specified {@link yfiles.layout.EdgeBundling} defines global bundling properties. Settings for individual edges can be
       * defined by assigning an {@link yfiles.layout.EdgeBundleDescriptor} to an edge using a {@link yfiles.algorithms.IDataProvider} registered
       * with key {@link yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
       * </p>
       * @type {yfiles.layout.EdgeBundling}
       */
      edgeBundling:yfiles.layout.EdgeBundling;
      static $class:yfiles.lang.Class;
    }
    export enum LayoutStyle{
      /**
       * The specifier for a layout style where the nodes within a biconnected component of the input graph are treated as
       * separate partitions.
       * <p>
       * If a node belongs to more than one biconnected component, it will be assigned to the one that seems to be more
       * suitable.
       * </p>
       * @see yfiles.circular.CircularLayout#layoutStyle
       */
      BCC_COMPACT,
      /**
       * The specifier for a layout style where the nodes within a biconnected component of the input graph are treated as
       * separate partitions.
       * <p>
       * If a node belongs to more than one biconnected component, it will form an isolated partition and will be laid out in
       * between of all its biconnected component cycles.
       * </p>
       * @see yfiles.circular.CircularLayout#layoutStyle
       */
      BCC_ISOLATED,
      /**
       * The specifier for a layout style where the user can specify groups that will form separate partitions. To define the
       * groups, a {@link yfiles.algorithms.IDataProvider} with key {@link yfiles.circular.CircularLayout#CUSTOM_GROUPS_DP_KEY} must be
       * registered.
       * @see yfiles.circular.CircularLayout#layoutStyle
       */
      CUSTOM_GROUPS,
      /**
       * The specifier for a layout style where all nodes of the input graph will be placed on or in the interior of a single
       * cycle.
       * @see yfiles.circular.CircularLayout#layoutStyle
       */
      SINGLE_CYCLE
    }
    export enum PartitionStyle{
      /**
       * Specifier for a partition layout style where all nodes of a partition are placed on the boundary of a circle.
       * @see yfiles.circular.CircularLayout#partitionStyle
       */
      CYCLE,
      /**
       * Specifier for a partition style with a semi-compact layout for each partition.
       * <p>
       * This style places nodes in the interior and on the border of a circle and arranges them in an organic manner, leading to
       * a more compact layout. Only nodes with connections to other partitions are guaranteed to lie on the boundary of the
       * resulting disk.
       * </p>
       * @see yfiles.circular.CircularLayout#partitionStyle
       */
      DISK,
      /**
       * Specifier for a partition style with a compact layout for each partition.
       * <p>
       * This style creates an organic circular layout for each partition. Even nodes that connect to different partitions may be
       * placed in the interior of the resulting disk.
       * </p>
       * @see yfiles.circular.CircularLayout#partitionStyle
       */
      ORGANIC
    }
    /**
     * This layout algorithm places all nodes of a graph on a single cycle.
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * All nodes of a graph are placed on one common circle, generating <em>circular layouts</em>. The edges are drawn as simple, straight lines. The
     * resulting layouts look like a ring. Edges connecting nodes which are <em>not</em> next to each other on the ring are routed <em>inside</em> of the
     * circle. If a graph contains many edges, the number of edge crossings inside the circle may be quite high.
     * </p>
     * <p>
     * Single-cycle circular layouts are suitable for the visualization of networks such as ring networks or smaller social
     * networks. They are also useful to layout cyclic subgraphs of a larger graph.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[40,40],"d":1,"c":1},"n":[[39.45,439.2],[459.2,19.45],[336.26,-5],[39.45,193.32],[109.09,543.42],[563.42,89.09],[633.06,193.32],[15,316.26],[213.32,613.06],[563.42,543.42],[336.26,637.52],[459.2,613.06],[633.06,439.2],[657.52,316.26],[213.32,19.45],[109.09,89.09]],"e":[[2,1,0,-4.59,0,-4.59],[4,0,-6.01,0,-6.01,0],[0,4,4.16,-2.78,4.16,-2.78],[1,5],[5,6],[1,2,-0.88,4.41,-0.88,4.41],[11,9],[10,11],[9,12,-4.16,-2.78,-4.16,-2.78],[13,12],[12,9,6.01,0,6.01,0],[14,15],[15,13,0,-4.87,0,-4.87],[7,12,0.88,-4.41,0.88,-4.41],[6,8],[8,14,-5,0,-5,0],[14,8,5,0,5,0],[7,3],[3,15,-4.16,-2.78,-4.16,-2.78],[15,3,6.01,0,6.01,0],[12,7,0,4.59,0,4.59],[0,7],[10,8],[6,13],[4,8]],"vp":[15.0,-5.0,683.0,683.0]}} An example of a single cycle layout<b>Concept</b>First, the nodes are sorted by applying either a custom or some default
     * {@link yfiles.algorithms.INodeSequencer}. Then, all nodes are placed on a circle with an appropriate radius according to the
     * calculated order. <b>Features</b>
     * </p>
     * <p>
     * The order in which the nodes are placed on the circle is a crucial aspect of this algorithm. By specifying a {@link yfiles.algorithms.INodeSequencer}
     * instance via {@link yfiles.circular.SingleCycleLayout#nodeSequencer}, the order can be customized.
     * </p>
     * <p>
     * There are two options to determine the radius of the circle on which nodes are placed:
     * </p>
     * <ol>
     * <li>The radius can be determined automatically. A specified {@link yfiles.circular.SingleCycleLayout#minimumRadius minimum radius} will be obeyed when choosing the radius.</li>
     * <li>The radius can be chosen manually. To do so, disable the {@link yfiles.circular.SingleCycleLayout#automaticRadius automatic radius selection} and specify the custom radius via {@link yfiles.circular.SingleCycleLayout#fixedRadius}.</li>
     * </ol>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface SingleCycleLayout extends yfiles.layout.MultiStageLayout{}
    export class SingleCycleLayout {
      /**
       * Creates a new {@link yfiles.circular.SingleCycleLayout} instance with default settings.
       */
      constructor();
      /**
       * Gets or sets the angle that describes the location on the circle of the first node.
       * <p>
       * The angle is measured in radians and world coordinates, e.g., <code>0</code> means east, <code>Pi/2</code> means south.
       * </p>
       * @type {number}
       */
      initialAngle:number;
      /**
       * Gets or sets whether or not to consider the coordinates of the input diagram when arranging the graph.
       * <p>
       * If this feature is enabled, the original circular order of peripheral nodes around an estimated circle center will be
       * preserved.
       * </p>
       * @type {boolean}
       */
      fromSketchModeEnabled:boolean;
      /**
       * Gets or sets the minimum distance between nodes that are next to each other on the circle.
       * @throws {Stubs.Exceptions.ArgumentError} if the given distance is negative
       * @type {number}
       */
      minimumNodeDistance:number;
      /**
       * Gets or sets a fixed radius for the circle on which the nodes will be placed.
       * @throws {Stubs.Exceptions.ArgumentError} if the given radius is negative
       * @see yfiles.circular.SingleCycleLayout#minimumRadius
       * @see yfiles.circular.SingleCycleLayout#automaticRadius
       * @type {number}
       */
      fixedRadius:number;
      /**
       * Gets or sets the minimum radius for the circle on which the nodes are placed.
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum radius is negative
       * @see yfiles.circular.SingleCycleLayout#fixedRadius
       * @see yfiles.circular.SingleCycleLayout#automaticRadius
       * @type {number}
       */
      minimumRadius:number;
      /**
       * Gets or sets whether or not the radius of the circle should be determined automatically.
       * <p>
       * If enabled, the radius will be chosen such that the distance between nodes which are next to each other on the circle is
       * approximately the same as the {@link yfiles.circular.SingleCycleLayout#minimumNodeDistance minimum node distance}.
       * </p>
       * <p>
       * If this feature is disabled, the radius specified via {@link yfiles.circular.SingleCycleLayout#fixedRadius} will be applied.
       * </p>
       * @see yfiles.circular.SingleCycleLayout#minimumRadius
       * @see yfiles.circular.SingleCycleLayout#fixedRadius
       * @type {boolean}
       */
      automaticRadius:boolean;
      /**
       * Gets or sets the custom {@link yfiles.algorithms.INodeSequencer} that will be used for determining the order of the nodes for
       * the circle placement.
       * @type {yfiles.algorithms.INodeSequencer}
       */
      nodeSequencer:yfiles.algorithms.INodeSequencer;
      /**
       * Gets the radius that was applied to the last layout calculated by this algorithm.
       * <p>
       * This method is useful if one wants to know the actual radius of a layout calculated by this algorithm.
       * </p>
       * @type {number}
       */
      lastAppliedRadius:number;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.circular.CircularLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface CircularLayoutData extends yfiles.layout.LayoutData{}
    export class CircularLayoutData {
      constructor();
      /**
       * Gets or sets the mapper from nodes to their circle id.
       * <p>
       * If this property is set, the mapper is filled by the layout with the circle information.
       * </p>
       * @see yfiles.circular.CircularLayout#CIRCLE_ID_DP_KEY
       * @type {yfiles.collections.IMapper.<yfiles.graph.INode,number>}
       */
      circleIds:yfiles.collections.IMapper<yfiles.graph.INode,number>;
      /**
       * Gets or sets the mapping from nodes to their custom group object.
       * <p>
       * The custom groups are only considered if {@link yfiles.circular.CircularLayout#layoutStyle} is set to {@link yfiles.circular.LayoutStyle#CUSTOM_GROUPS}
       * </p>
       * @see yfiles.circular.CircularLayout#CUSTOM_GROUPS_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,Object>}
       */
      customGroups:yfiles.layout.ItemMapping<yfiles.graph.INode,Object>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping of edges to their {@link yfiles.layout.EdgeBundleDescriptor}.
       * <p>
       * Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
       * source and target point, the edges are again clearly split.
       * </p>
       * <p>
       * If an edge is mapped to <code>null</code>, the {@link yfiles.layout.EdgeBundling#defaultBundleDescriptor default descriptor} is used.
       * </p>
       * @see yfiles.layout.EdgeBundleDescriptor
       * @see yfiles.circular.CircularLayout#edgeBundling
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.EdgeBundleDescriptor>}
       */
      edgeBundleDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.EdgeBundleDescriptor>;
      static $class:yfiles.lang.Class;
    }
  }export namespace orthogonal{
    /**
     * This class provides layout information for the edges of the graph.
     * <p>
     * {@link yfiles.orthogonal.EdgeLayoutDescriptor} is used by {@link yfiles.orthogonal.OrthogonalLayout} to determine the routing details
     * for the edges (e.g. the segment length).
     * </p>
     * <p>
     * An {@link yfiles.orthogonal.EdgeLayoutDescriptor} can be specified individually for each edge by using a {@link yfiles.algorithms.IDataProvider}
     * which maps from an {@link yfiles.algorithms.Edge} to an {@link yfiles.orthogonal.EdgeLayoutDescriptor} instance. The {@link yfiles.algorithms.IDataProvider}
     * is registered with the graph with key {@link yfiles.orthogonal.OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}.
     * </p>
     * @see yfiles.orthogonal.OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
     * @class
     */
    export interface EdgeLayoutDescriptor extends Object{}
    export class EdgeLayoutDescriptor {
      /**
       * Creates a new instance of {@link yfiles.orthogonal.EdgeLayoutDescriptor} with the default settings.
       */
      constructor();
      /**
       * Creates a copy of this {@link yfiles.orthogonal.EdgeLayoutDescriptor} instance.
       * @returns a copy of this {@link } instance
       */
      createCopy():yfiles.orthogonal.EdgeLayoutDescriptor;
      /**
       * Gets or sets the minimum length of the first segment of the edge path (at the source node).
       * <p>
       * The minimum length must be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length of the first segment is negative
       * @type {number}
       */
      minimumFirstSegmentLength:number;
      /**
       * Gets or sets the minimum length of the last segment of the edge path (at the target node).
       * <p>
       * The minimum length must be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length of the last segment is negative
       * @type {number}
       */
      minimumLastSegmentLength:number;
      /**
       * Gets or sets the minimum segment length of an edge.
       * <p>
       * The minimum length must be greater than or equal to <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum length is negative
       * @type {number}
       */
      minimumSegmentLength:number;
      static $class:yfiles.lang.Class;
    }
    export enum LayoutStyle{
      /**
       * A layout style that produces orthogonal edges while maintaining the original node size.
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      NORMAL,
      /**
       * A layout style that produces orthogonal edges and nodes of uniform size.
       * <p>
       * Before calculating the layout, all nodes will be assigned equal sizes.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      UNIFORM,
      /**
       * A layout style that modifies the size of nodes to minimize the overall number of bends.
       * <p>
       * The layout algorithm will enlarge nodes based on the number and position of their neighbors in order to minimize the
       * number of required bends. The segments of all edges are either horizontal or vertical.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      BOX,
      /**
       * A layout style that produces non-orthogonal edges and resizes the nodes to a small uniform size.
       * <p>
       * This layout style resembles {@link yfiles.orthogonal.LayoutStyle#BOX}, but it resizes all nodes to equal size.
       * </p>
       * <p>
       * It introduces additional bends into some edges and routes the last edge segment of these edges non-orthogonally to their
       * adjacent nodes.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      MIXED,
      /**
       * A layout style similar to {@link yfiles.orthogonal.LayoutStyle#NORMAL} that is best-suited for tree-like structures.
       * <p>
       * Larger subtrees are processed using a specialized tree layout algorithm which arranges all directed subtree structures
       * in an optimized way.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      NORMAL_TREE,
      /**
       * A layout style similar to {@link yfiles.orthogonal.LayoutStyle#MIXED} that maintains the original node size.
       * <p>
       * This style introduces additional bends into some edges and routes the last edge segment of these edges non-orthogonally
       * to their adjacent nodes.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      FIXED_MIXED,
      /**
       * A layout style similar to {@link yfiles.orthogonal.LayoutStyle#BOX} that maintains the original node size.
       * <p>
       * Nodes will be placed at the same positions as with
       * {@link yfiles.orthogonal.LayoutStyle#FIXED_MIXED}, but edges will be routed strictly orthogonally.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       */
      FIXED_BOX
    }
    /**
     * This layout algorithm arranges graphs in an orthogonal fashion.
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * This layout algorithm arranges the nodes of a given graph such that each edge is drawn as an alternating sequence of
     * horizontal and vertical segments.
     * </p>
     * <p>
     * It produces compact drawings with no overlapping nodes, few crossings and few bends and is well suited for small and
     * medium-sized sparse graphs.
     * </p>
     * <p>
     * Orthogonal drawings are common in engineering applications since they are able to provide clear representations of
     * complex networks that can also be optimal with respect to diverse objective functions such as bend minimization or area
     * minimization. Application domains of orthogonal drawings include software engineering, database schema representation,
     * system management, knowledge representation, VLSI circuits and floor planning applications.
     * </p>
     * <p>
     * {@graph {"ann": {"s": [30,30], "d": 0},"n": [[135,160], [210,385], [210,60], [285,160], [360,60], [435,285], [60,110], [135,385], [60,435], [210,160], [-15,110], [285,335], [360,235], [360,160], [435,385]], "e": [[3,4,0,-15,-15,10,[300,85]], [0,4,0,-15,0,-15,[150,25,375,25]], [2,4,15,0,-15,0], [3,5,5,15,-15,0,[305,300]], [4,5,15,0,0,-15,[450,75]], [0,6,-15,0,10,15,[85,175]], [5,6,15,0,0,-15,[500,300,500,0,75,0]], [7,1,15,0,-15,0], [7,0,0,-15,0,15], [1,8,0,15,15,0,[225,450]], [8,6,0,-15,0,15], [0,9,15,0,-15,0], [9,6,-10,-15,15,0,[215,125]], [9,3,15,0,-15,0], [9,2,0,-15,0,15], [9,1,0,15,0,-15], [10,8,0,15,-15,0,[0,450]], [10,6,15,0,-15,0], [3,11,0,15,0,-15], [11,1,0,15,15,-10,[300,390]], [12,5,15,0,-10,-15,[440,250]], [12,3,-15,0,10,15,[310,250]], [4,13,0,15,0,-15], [13,12,0,15,0,-15], [13,3,-15,0,15,0], [14,5,0,-15,0,15], [14,1,-15,0,15,0]], "vp": [-15.0,0.0,515.0,465.0]}} Orthogonal layout with default settings
     * </p>
     * <p>
     * This algorithm also optionally supports {@link yfiles.orthogonal.OrthogonalLayout#DIRECTED_EDGE_DP_KEY directed edge drawings} (although not in combination with hierarchically nested
     * graphs or the non-default
     * {@link yfiles.orthogonal.OrthogonalLayout#layoutStyle layout styles}). Application domains of directed orthogonal drawings include, for example, software engineering, database
     * schema and system management.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[165,-15],[205,305],[45,345],[45,65],[205,145],[105,105],[45,405],[-15,405],[285,-15],[205,225],[125,185]],"e":[[5,0,15,-0,-5,15,[175,120]],[5,4,0,15,-15,-0,[120,160]],[5,2,-15,-0,10,-15,[70,120]],[6,4,15,-0,15,-0,[280,420,280,160]],[6,2,0,-15,0,15],[6,7,-15,-0,15,-0],[7,3,0,-15,-15,-0,[0,80]],[8,0,-15,-0,15,-0],[8,7,0,15,0,15,[300,460,0,460]],[2,9,15,-0,15,-0,[260,360,260,240]],[4,10,-10,15,15,-0,[210,200]],[10,1,0,15,-15,-0,[140,320]],[0,1,0,15,-10,-15,1,[180,280,210,280]],[0,3,-10,15,0,-15,1,[170,40,60,40]],[3,2,0,15,0,-15,1],[0,4,10,15,0,-15,1,[190,120,220,120]],[4,9,0,15,0,-15,1],[9,1,0,15,0,-15,1]],"vp":[-15.0,-15.0,330.0,475.0]}} Sample output of {@link yfiles.orthogonal.OrthogonalLayout} with default settings. Marked edges are considered as directed and
     * the layout orientation is top-to-bottom. {@graph {"ann":{"s":[30,30],"d":1},"n":[[85,45],[265,105],[185,-15],[-15,165],[485,165],[265,225],[365,165],[85,225]],"e":[[0,1,15,-0,-15,-0,[160,60,160,120]],[0,2,15,-0,-15,-0,[160,60,160,0]],[2,1,15,10,-15,-10,[240,10,240,110]],[3,0,15,-0,-15,-0,[60,180,60,60]],[2,4,15,-0,-15,-0,[440,0,440,180]],[5,4,15,10,-15,-0,[340,250,340,280,440,280,440,180]],[1,6,15,-0,-15,-0,[320,120,320,180]],[5,6,15,-0,-15,-0,[320,240,320,180]],[6,4,15,-0,-15,-0],[3,7,15,-0,-15,-0,[60,180,60,240]],[7,5,15,-0,-15,-0]],"vp":[-15.0,-15.0,530.0,295.0]}} Sample output of {@link yfiles.orthogonal.OrthogonalLayout} with edge grouping. All
     * edges are considered directed and the layout orientation is left-to-right.
     * </p>
     * <p>
     * Also this algorithm supports hierarchically nested graphs (although not in combination with the directed edge drawings,
     * the {@link yfiles.orthogonal.OrthogonalLayout#fromSketchMode from sketch} option, or the non-default {@link yfiles.orthogonal.OrthogonalLayout#layoutStyle layout styles}) feature:
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0},"n":[[165,585],[0,220,240,340,2,[[0,242.38,240,22.38,"Group 2"]]],[165,385],[165,485],[45,485],[45,385],[105,385],[260,200,200,260,2,[[260,222.38,200,22.38,"Group 3"]]],[305,325],[305,265],[365,325],[305,385],[0,0,240,200,2,[[0,22.38,240,22.38,"Group 1"]]],[165,125],[105,125],[45,125],[45,65],[165,65],[265,485]],"e":[[3,2,0,-15,0,15],[3,0,0,15,0,-15],[4,3,15,0,-15,0],[5,4,0,15,0,-15],[2,6,-15,0,15,0],[6,5,-15,0,15,0],[8,9,0,-15,0,15],[2,9,0,-15,-15,0,[180,280]],[8,10,15,0,-15,0],[10,9,0,-15,15,10,[380,290]],[9,11,15,0,15,0,[420,280,420,400]],[8,11,0,15,0,-15],[2,11,15,0,-15,0],[11,10,15,-10,0,15,[380,390]],[6,14,0,-15,0,15],[14,13,15,0,-15,0],[5,15,0,-15,0,15],[15,14,15,0,-15,0],[16,15,0,15,0,-15],[13,17,0,-15,0,15],[17,16,-15,0,15,0],[3,18,15,0,-15,0]],"vp":[0.0,0.0,460.0,615.0]}} Orthogonal layout of a grouped graph with default settings {@graph {"ann":{"s":[30,30],"d":1},"n":[[-15,-17.38,670,794.75,2,[[-15,5,670,22.38,"Group 6"]]],[0,20,620,680,2,[[0,42.38,620,22.38,"Group 5"]]],[40,460,240,200,2,[[40,482.38,240,22.38,"Group 2"]]],[205,585,[[214.66,609.35,10.67,18.7,"9"]]],[205,525,[[211.33,549.35,17.35,18.7,"10"]]],[85,525,[[91.33,549.35,17.35,18.7,"11"]]],[85,585,[[91.33,609.35,17.35,18.7,"10"]]],[120,80,460,360,2,[[120,102.38,460,22.38,"Group 4"]]],[160,140,180,260,2,[[160,162.38,180,22.38,"Group 1"]]],[265,325,[[274.66,349.35,10.67,18.7,"1"]]],[205,325,[[214.66,349.35,10.67,18.7,"2"]]],[205,205,[[214.66,229.35,10.67,18.7,"3"]]],[265,205,[[274.66,229.35,10.67,18.7,"4"]]],[265,265,[[274.66,289.35,10.67,18.7,"5"]]],[360,160,180,240,2,[[360,182.38,180,22.38,"Group 3"]]],[405,325,[[411.33,349.35,17.35,18.7,"12"]]],[405,265,[[411.33,289.35,17.35,18.7,"13"]]],[465,265,[[471.33,289.35,17.35,18.7,"14"]]],[85,725,[[91.33,749.35,17.35,18.7,"19"]]],[205,725,[[211.33,749.35,17.35,18.7,"18"]]]],"e":[[4,3,0,15,0,-15],[4,5,-15,0,15,0],[3,6,-15,0,15,0],[6,5,0,-15,0,15],[9,10,-15,0,15,0],[10,5,-15,0,0,-15,[100,340]],[10,4,0,15,0,-15],[10,11,0,-15,0,15],[11,12,15,0,-15,0],[12,13,0,15,0,-15],[13,9,0,15,0,-15],[15,16,0,-15,0,15],[16,13,-15,0,15,0],[16,12,0,-15,15,0,[420,220]],[16,17,15,0,-15,0],[6,18,0,15,0,-15],[3,19,0,15,0,-15]],"vp":[-15.0,-18.0,670.0,796.0]}} Orthogonal layout of a graph with nested
     * groups<b>Concept</b>The orthogonal layout algorithm is based on the topology-shape-metrics approach and runs in three phases:
     * </p>
     * <ul>
     * <li><b>Planarization</b> – A planar embedding is computed.</li>
     * <li><b>Orthogonalization</b> – The bends and the angles are computed.</li>
     * <li><b>Compaction</b> – The coordinates for the nodes and edges are determined.</li>
     * </ul>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * There exist several different layout styles that the orthogonal layout algorithm supports depending on whether or not
     * the edges should be routed completely orthogonal, or whether or not the original size of the nodes should be maintained.
     * Such layout styles are {@link yfiles.orthogonal.LayoutStyle#NORMAL}, {@link yfiles.orthogonal.LayoutStyle#UNIFORM},
     * {@link yfiles.orthogonal.LayoutStyle#BOX}, {@link yfiles.orthogonal.LayoutStyle#NORMAL_TREE}, {@link yfiles.orthogonal.LayoutStyle#MIXED},
     * {@link yfiles.orthogonal.LayoutStyle#FIXED_BOX}, {@link yfiles.orthogonal.LayoutStyle#FIXED_MIXED} and can be applied using {@link yfiles.orthogonal.OrthogonalLayout#layoutStyle}.
     * </p>
     * <p>
     * {@link yfiles.orthogonal.OrthogonalLayout} is able to consider edge label data when arranging a graph. This means that the
     * layout algorithm will determine the positions of the nodes and edges such that the edge labels do not overlap with the
     * rest of the layout. Integrated edge labeling can be activated using method {@link yfiles.orthogonal.OrthogonalLayout#integratedEdgeLabeling}.
     * </p>
     * <p>
     * {@link yfiles.orthogonal.EdgeLayoutDescriptor} instances can be used for specifying individual information (e.g. distances) for
     * each edge in the graph. The descriptors are bound to the graph using {@link yfiles.algorithms.IDataProvider}s registered with
     * key {@link yfiles.orthogonal.OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If there is no descriptor assigned to some edges,
     * a default descriptor will be used. To set default descriptors, use {@link yfiles.orthogonal.OrthogonalLayout#edgeLayoutDescriptor}.
     * </p>
     * <p>
     * {@link yfiles.orthogonal.OrthogonalLayout} tries to optimize diverse objective functions such as bend minimization, number of
     * edge crossings, edge length minimization or face maximization. These settings can be enabled using the corresponding
     * methods {@link yfiles.orthogonal.OrthogonalLayout#optimizePerceivedBends}, {@link yfiles.orthogonal.OrthogonalLayout#crossingReduction}, {@link yfiles.orthogonal.OrthogonalLayout#edgeLengthReduction}, and
     * {@link yfiles.orthogonal.OrthogonalLayout#faceMaximization}, respectively. The drawback when these settings are enabled is that the running time of the algorithm may be
     * drastically increased.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface OrthogonalLayout extends yfiles.layout.MultiStageLayout{}
    export class OrthogonalLayout {
      /**
       * Creates a new {@link yfiles.orthogonal.OrthogonalLayout} instance with default settings.
       */
      constructor();
      /**
       * Returns a new {@link yfiles.orthogonal.EdgeLayoutDescriptor} instance that will be used during the various phases of the layout
       * algorithm to determine the drawing details of the edges of the graph.
       * <p>
       * This method may be overridden to create a new {@link yfiles.orthogonal.EdgeLayoutDescriptor} instance with different
       * configuration settings.
       * </p>
       * @returns a new {@link } instance
       * @protected
       */
      createEdgeLayoutDescriptor():yfiles.orthogonal.EdgeLayoutDescriptor;
      /**
       * Data provider key for providing layout information for each edge
       * <p>
       * If no edge layout descriptor is mapped for an edge, a default edge layout descriptor will be obtained using method
       * {@link yfiles.orthogonal.OrthogonalLayout#createEdgeLayoutDescriptor}.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#layoutStyle
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.orthogonal.EdgeLayoutDescriptor>}
       */
      static EDGE_LAYOUT_DESCRIPTOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.orthogonal.EdgeLayoutDescriptor>;
      /**
       * Data provider key for marking edges which should be routed such that they point to the main layout orientation
       * <p>
       * The main layout orientation can be set using method {@link yfiles.layout.MultiStageLayout#layoutOrientation}.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<boolean>}
       */
      static DIRECTED_EDGE_DP_KEY:yfiles.algorithms.EdgeDpKey<boolean>;
      /**
       * Gets or sets the {@link yfiles.orthogonal.EdgeLayoutDescriptor} instance used for all those edges that do not have a specific
       * layout descriptor assigned.
       * <p>
       * By default, this method will return an {@link yfiles.orthogonal.EdgeLayoutDescriptor} instance created with
       * {@link yfiles.orthogonal.OrthogonalLayout#createEdgeLayoutDescriptor}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.orthogonal.EdgeLayoutDescriptor} is <code>null</code>
       * @see yfiles.orthogonal.OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.orthogonal.EdgeLayoutDescriptor}
       */
      edgeLayoutDescriptor:yfiles.orthogonal.EdgeLayoutDescriptor;
      /**
       * Gets or sets whether or not the layout algorithm considers node labels when calculating node positions to avoid
       * overlaps.
       * <p>
       * This method is a convenience method that assures that the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm} is of type {@link yfiles.layout.LabelLayoutTranslator}
       * and {@link yfiles.layout.LabelLayoutTranslator#translateNodeLabels} is set to <code>true</code>.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if no properly configured {@link yfiles.layout.LabelLayoutTranslator} is registered even though this property was enabled
       * earlier (can happen when manually specifying the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm}).
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not the layout algorithm preserves space and places edge labels.
       * <p>
       * To define the desired placement for each label add a {@link yfiles.layout.PreferredPlacementDescriptor} on
       * {@link yfiles.layout.IEdgeLabelLayout}.
       * </p>
       * <p>
       * This method also assures that the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm} is of type {@link yfiles.layout.LabelLayoutTranslator} and {@link yfiles.layout.LabelLayoutTranslator#translateEdgeLabels} is set to
       * <code>true</code>.
       * </p>
       * @throws {Stubs.Exceptions.InvalidOperationError} if no properly configured {@link yfiles.layout.LabelLayoutTranslator} is registered even though integrated labeling was enabled
       * earlier (can happen when manually specifying the {@link yfiles.layout.MultiStageLayout#labeling labeling algorithm}).
       * @type {boolean}
       */
      integratedEdgeLabeling:boolean;
      /**
       * Gets or sets whether or not a randomization strategy should be performed.
       * <p>
       * The randomization will increase the overall layout quality with high probability.
       * </p>
       * @type {boolean}
       */
      randomization:boolean;
      /**
       * Gets or sets whether or not degree-one nodes that have the same neighbor should be aligned.
       * @type {boolean}
       */
      alignDegreeOneNodes:boolean;
      /**
       * Gets or sets whether or not one face of the embedding of the graph should be maximized.
       * <p>
       * An embedding of a graph is uniquely specified by the cyclic order of edges incident to the same node in a graph
       * drawing.
       * </p>
       * <p>
       * In any planar drawing of a planar graph, the edges divide the plane into different regions called faces. If one of these
       * faces (the outer face) gets maximized, all other faces will be more compact.
       * </p>
       * @see yfiles.orthogonal.OrthogonalLayout#randomization
       * @type {boolean}
       */
      faceMaximization:boolean;
      /**
       * Gets or sets whether or not the number of edge crossings should be reduced.
       * <p>
       * The layout algorithm will try to significantly reduce the number of crossings in a post-processing step.
       * </p>
       * @type {boolean}
       */
      crossingReduction:boolean;
      /**
       * Gets or sets whether or not the number of perceived bends should be minimized.
       * <p>
       * A perceived bend is not an actual bend but it is created when a helical arrangement of chains of nodes occurs.
       * </p>
       * @type {boolean}
       */
      optimizePerceivedBends:boolean;
      /**
       * Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
       * <p>
       * Each node will be placed on a grid point. Edges will be routed such that all segments but the first and last one lie on
       * grid lines. (The first and last segments of an edge may or may not lie on grid lines.) Edges consisting of a single
       * segment always lie on grid lines.
       * </p>
       * <p>
       * The grid spacing has to be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the grid spacing is negative
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets or sets the layout style for this layout algorithm.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown layout style is given
       * @type {yfiles.orthogonal.LayoutStyle}
       */
      layoutStyle:yfiles.orthogonal.LayoutStyle;
      /**
       * Gets or sets whether or not the overall edge length should be optimized.
       * <p>
       * The algorithm will try to significantly reduce the length of some edges.
       * </p>
       * @type {boolean}
       */
      edgeLengthReduction:boolean;
      /**
       * Gets or sets whether or not the existing drawing should be used as a sketch of the resulting orthogonal layout.
       * <p>
       * The layout algorithm will try to orthogonalize the given sketch without making too many modifications with respect to
       * the original drawing.
       * </p>
       * @type {boolean}
       */
      fromSketchMode:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for hiding group nodes is activated.
       * @see yfiles.layout.MultiStageLayout#hideGroupsStageEnabled
       * @see yfiles.layout.MultiStageLayout#hideGroupsStage
       * @see yfiles.layout.HideGroupsStage
       * @type {boolean}
       */
      hideGroupsStageEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm arranges a graph in a compact, orthogonal fashion.
     * <p>
     * <b>Layout Style</b>
     * </p>
     * <p>
     * This layout algorithm is a variant of {@link yfiles.orthogonal.OrthogonalLayout} that tries to arrange nodes in a more compact
     * way. It allows to specify a desired aspect ratio for the bounds of the calculated layout. Nodes of a given graph are
     * arranged such that each edge is drawn as an alternating sequence of horizontal and vertical segments.
     * </p>
     * <p>
     * {@link yfiles.orthogonal.CompactOrthogonalLayout} is well suited for small and medium-sized sparse graphs and produces compact
     * drawings with no overlapping nodes, few crossings and few bends. It is also especially suitable for printing the diagram
     * on a paper of predefined size. As a disadvantage, this algorithm will usually produce less elegant edge routes than the
     * ones of {@link yfiles.orthogonal.OrthogonalLayout}.
     * </p>
     * <p>
     * Application domains of orthogonal drawings are software engineering, project management, function call graphs and
     * object-oriented class diagrams.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":0},"n":[[165,65],[285,405],[285,125],[225,285],[25,285],[225,365],[105,365],[285,205],[105,285],[165,205],[165,-15],[365,205]],"e":[[0,2,15,0,-5,-15,[295,80]],[4,1,-5,15,-15,0,[35,420]],[5,3,0,-15,0,15],[5,6,-15,5,15,5],[6,4,-15,0,5,15,[45,380]],[1,7,0,-15,0,15],[3,7,5,-15,-15,5,[245,225]],[2,7,0,15,0,-15],[4,8,15,0,-15,0],[3,8,-15,0,15,0],[6,8,0,-15,0,15],[4,9,-0,-15,-15,-5,[40,215]],[0,9,-5,15,-5,-15],[3,9,-5,-15,5,15,[235,261,185,261]],[8,9,-0,-15,-15,5,[120,225]],[9,5,-5,15,-15,-5,[175,375]],[9,7,15,-5,-15,-5],[9,2,5,-15,-15,0,[185,140]],[0,10,0,-15,0,15],[4,10,-15,0,-15,0,[23,300,23,0]],[10,3,15,-5,15,0,[397,-5,397,300]],[10,2,15,5,5,-15,[305,5]],[2,11,15,0,0,-15,[380,140]],[11,5,0,15,15,0,[380,380]],[11,7,-15,0,15,0]],"vp":[23.0,-15.0,374.0,450.0]}} Sample output of the layout algorithm with aspect ratio <code>1.0</code><b>Concept</b>
     * </p>
     * <p>
     * This algorithm is implemented as a {@link yfiles.layout.PartitionLayout} and applies the following steps:
     * </p>
     * <ul>
     * <li>Divides the graph into smaller partitions.</li>
     * <li>Arranges the nodes of each graph partition using {@link yfiles.orthogonal.OrthogonalLayout}.</li>
     * <li>Arranges the partitions themselves in a compact manner.</li>
     * <li>Routes the edges between partitions (inter-edges) using an edge router.</li>
     * </ul>
     * <p>
     * Each of these steps can be individually configured and/or replaced. To do so, use methods {@link yfiles.layout.PartitionLayout#partitionFinder}, {@link yfiles.layout.LayoutStageBase#coreLayout}, {@link yfiles.layout.PartitionLayout#partitionPlacer} and {@link yfiles.layout.PartitionLayout#interEdgeRouter}, respectively.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * The aspect ratio of the resulting layout bounds, i.e., the ratio of width/height of the layout, can be configured using
     * {@link yfiles.orthogonal.CompactOrthogonalLayout#aspectRatio}. {@link yfiles.orthogonal.CompactOrthogonalLayout} tries to reach this preferred value. However, depending on the actual graph
     * structure, this may not always be possible.
     * </p>
     * <p>
     * Octilinear routing of edges can be supported by appending {@link yfiles.router.PolylineLayoutStage}.
     * </p>
     * @class
     * @extends {yfiles.layout.PartitionLayout}
     */
    export interface CompactOrthogonalLayout extends yfiles.layout.PartitionLayout{}
    export class CompactOrthogonalLayout {
      /**
       * Creates a new {@link yfiles.orthogonal.CompactOrthogonalLayout} instance with default settings.
       */
      constructor();
      /**
       * Gets or sets the equidistant spacing between the horizontal and vertical grid lines.
       * <p>
       * Each node will be placed on a grid point. Edges will be routed such that their segments lie on grid lines, if the
       * endpoints of the edges allow to place the ports accordingly.
       * </p>
       * <p>
       * The grid spacing has to be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the grid spacing is negative or zero
       * @type {number}
       */
      gridSpacing:number;
      /**
       * Gets or sets the desired aspect ratio (i.e.
       * <p>
       * the quotient width/height) of the resulting layout bounds.
       * </p>
       * <p>
       * The aspect ratio needs to be greater than <code>0</code>.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given aspect ratio is less than or equal to zero
       * @see yfiles.layout.PartitionLayout#partitionPlacer
       * @type {number}
       */
      aspectRatio:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.orthogonal.OrthogonalLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface OrthogonalLayoutData extends yfiles.layout.LayoutData{}
    export class OrthogonalLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from edges to their {@link yfiles.orthogonal.EdgeLayoutDescriptor}.
       * @see yfiles.orthogonal.EdgeLayoutDescriptor
       * @see yfiles.orthogonal.OrthogonalLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.orthogonal.EdgeLayoutDescriptor>}
       */
      edgeLayoutDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.orthogonal.EdgeLayoutDescriptor>;
      /**
       * Gets or sets the collection of edges that should be routed in a way that point in the main layout direction.
       * @see yfiles.orthogonal.OrthogonalLayout#DIRECTED_EDGE_DP_KEY
       * @type {yfiles.layout.ItemCollection.<yfiles.graph.IEdge>}
       */
      directedEdges:yfiles.layout.ItemCollection<yfiles.graph.IEdge>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping that provides a {@link yfiles.layout.PreferredPlacementDescriptor} instance for edge
       * {@link yfiles.graph.ILabel}s.
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>}
       */
      edgeLabelPreferredPlacement:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>;
      static $class:yfiles.lang.Class;
    }
  }export namespace radial{
    export enum CenterNodesPolicy{
      /**
       * Specifier for a center nodes policy which chooses all nodes with in-degree equal to <code>0</code> as center nodes. If no node with
       * this property is present, an arbitrary node will be selected.
       * @see yfiles.radial.RadialLayout#centerNodesPolicy
       * @see yfiles.algorithms.Trees#getRoot
       */
      DIRECTED,
      /**
       * Specifier for a center nodes policy which chooses the node with the highest centrality as center node.
       * @see yfiles.radial.RadialLayout#centerNodesPolicy
       * @see yfiles.algorithms.Trees#getCenterRoot
       */
      CENTRALITY,
      /**
       * Specifier for a center nodes policy which chooses the node with the highest weighted centrality as center node.
       * @see yfiles.radial.RadialLayout#centerNodesPolicy
       * @see yfiles.algorithms.Trees#getWeightedCenterNode
       */
      WEIGHTED_CENTRALITY,
      /**
       * Specifier for a center nodes policy which uses center nodes specified by the user. The center nodes have to be specified
       * using a {@link yfiles.algorithms.IDataProvider} registered with the {@link yfiles.radial.RadialLayout#centerNodesDpKey according key}. The {@link yfiles.algorithms.IDataProvider} returns <code>true</code>
       * for nodes that should be placed in the center.
       * <p>
       * If there exists more than one center node, they will be placed on the innermost circle while the center position stays
       * empty.
       * </p>
       * @see yfiles.radial.RadialLayout#centerNodesPolicy
       * @see yfiles.radial.RadialLayout#centerNodesDpKey
       */
      CUSTOM
    }
    export enum LayeringStrategy{
      /**
       * Specifier for a layering strategy that uses breadth first search (BFS) to determine a layering for the graph. All edges
       * will span at most one layer in the resulting drawing. Edges between nodes that belong to the same layer are possible.
       * @see yfiles.radial.RadialLayout#layeringStrategy
       */
      BFS,
      /**
       * Specifier for a layering strategy that uses an optimal hierarchical layering strategy. The layer distance of an edge is
       * the absolute difference between the layer numbers of its source and target node. Layer assignment will be done such that
       * the overall sum of the layer distances of all edges in the layout is minimal.
       * @see yfiles.radial.RadialLayout#layeringStrategy
       */
      HIERARCHICAL
    }
    export enum EdgeRoutingStrategy{
      /**
       * Specifier for an edge routing strategy that will route edges with polyline segments from source to target with one bend
       * on each spanned circle.
       * @see yfiles.radial.RadialLayout#edgeRoutingStrategy
       */
      POLYLINE,
      /**
       * Specifier for an edge routing strategy that will route edges as arcs. The source and target of an edge as well as one
       * bend on each spanned circle are used as main control points. An easing function is applied in order to calculate
       * additional control points between the polar coordinates of the main control points.
       * @see yfiles.radial.RadialLayout#edgeRoutingStrategy
       */
      ARC
    }
    /**
     * Info object used for retrieving information about each node from the layout algorithm.
     * <p>
     * The information can be retrieved by registering with the graph a {@link yfiles.algorithms.IDataProvider} (which should also
     * implement the {@link yfiles.algorithms.IDataAcceptor} interface) using key {@link yfiles.radial.RadialLayout#NODE_INFO_DP_KEY}. The
     * layout algorithm will return a {@link yfiles.radial.RadialLayoutNodeInfo} object with appropriately filled fields for each {@link yfiles.algorithms.Node}
     * when the layout is calculated.
     * </p>
     * @class
     */
    export interface RadialLayoutNodeInfo extends Object{}
    export class RadialLayoutNodeInfo {
      /**
       * Creates a new {@link yfiles.radial.RadialLayoutNodeInfo} instance.
       */
      constructor();
      /**
       * Gets the index of the circle the node is placed on.
       * <p>
       * The circles used by the {@link yfiles.radial.RadialLayout} are indexed from the innermost to the outermost circle. Index <code>0</code>
       * represents the center, i.e. the circle with radius <code>0</code>.
       * </p>
       * @type {number}
       */
      circleIndex:number;
      /**
       * Gets the radius of the circle the node is placed on.
       * @type {number}
       */
      radius:number;
      /**
       * Gets the offset from the center of the circle, the node is placed on, to the center of the node.
       * @type {yfiles.algorithms.YPoint}
       */
      centerOffset:yfiles.algorithms.YPoint;
      /**
       * Gets the start angle in degrees of the sector the node is placed in.
       * <p>
       * The angle refers to a counter-clockwise rotation starting at the 3&nbsp;o'clock position.
       * </p>
       * @see yfiles.radial.RadialLayoutNodeInfo#sectorSize
       * @type {number}
       */
      sectorStart:number;
      /**
       * Gets the size in degrees of the sector the node is placed in.
       * @see yfiles.radial.RadialLayoutNodeInfo#sectorStart
       * @type {number}
       */
      sectorSize:number;
      static $class:yfiles.lang.Class;
    }
    /**
     * This layout algorithm arranges graphs in a radial fashion.
     * <p>
     * <b>Layout Style</b> The nodes are distributed into circles (layers) around a common center based on predefined {@link yfiles.radial.RadialLayout#layeringStrategy layering strategies}. Some
     * potential applications include visualization of social networks, data clustering and bioinformatics. {@graph {"ann":{"s":[30,30],"d":1, "c":1, "bg": 0}, "styles" : {"6" : {"a" : 0.5,"c" : 1,"bg" : 1,"f" : null}},"n":[[-15,-15,1],[13.93,106.61],[65.6,80.55],[109.84,-8.76],[64.87,-111.16],[-81.77,-120.67],[-13.62,-139.99],[-133.34,-55.24],[-132.35,28.06],[-74.13,95.13],[-51.74,-262.29],[27.21,-261.41],[144.74,-207.31],[217.89,-105.89],[279.81,-246.76],[224.61,-303.47],[158.73,-347.33],[207.67,98.65],[230.47,32.36],[233.97,-37.66],[-114.67,214.27],[-185.45,167.88],[-53,232.09],[-295.24,234.18],[-243.87,282.06],[-220.04,-158.04],[-251.69,-95.49],[-264.73,-26.61],[-375,-381.52,750,750,6],[-255.95,-247.77,500,500,6],[-120,-120,240.76,240,6]],"e":[[0,1],[0,2],[0,3],[0,4],[0,5],[0,6],[0,7],[0,8],[0,9],[9,8,[-70.52,125.91,-86.2,136.85,-105.19,140.56,-124.87,135.74,-141.6,122.91,-151.95,104.54,-153.96,84.38,-147.61,66.1,-134.58,52.11]],[6,10,[-1.65,-142.85,-15.61,-195.81,-28.91,-230.34]],[6,11,[4.82,-142.78,19.94,-195.41,34,-229.64]],[4,11,[82.38,-111.82,83.32,-128.06,82.62,-144.75,80.23,-161.75,76.13,-178.92,70.29,-196.12,62.69,-213.2,53.33,-230.01]],[4,12],[4,13,[94.6,-101.69,110.18,-105.84,126.49,-108.53,143.41,-109.7,160.83,-109.28,178.62,-107.22,214.79,-98.06]],[3,13,[138.89,0.42,165.98,-15.1,191.14,-35.68,213.67,-61.05,223.74,-75.44]],[12,14,[175.06,-199.78,223.92,-217.98,276.56,-229.49]],[12,15],[12,16,[164.26,-208.75,173.19,-260.12,174.85,-313.97]],[3,17,[138.24,13.42,163.65,31.57,186.64,54.55,206.52,82.04,215.1,97.36]],[3,18,[142.51,9.99,211.87,32.11,228.78,39.39]],[3,19,[142.79,4.26,214.01,-10.83,231.56,-16.39]],[9,20,[-67.21,129.42,-94.25,208.89]],[9,21,[-72.16,123.29,-101.29,147.07,-134.19,167.07,-151.93,175.53]],[21,7,[-170.54,166.68,-181.84,45.73,-180.63,25.53,-175.63,6.64,-167.31,-9.76,-156.5,-22.85,-144.15,-32.21,-131.17,-37.89]],[9,22,[-61.2,126.61,-61.38,160.54,-56.17,195.2,-45.44,229.93]],[21,23,[-185.24,193.48,-263.69,240.87]],[21,24,[-179.99,198.37,-221.74,279.97]],[7,25,[-131.46,-49.93,-155.99,-72.16,-177.9,-98.03,-196.82,-127.26]],[7,26],[7,27,[-134.65,-40.57,-167.65,-37.89,-200.78,-30.75,-233.59,-19.11]]],"vp":[-375.0,-382.0,750.0,751.0]}} Sample output of
     * the layout algorithm with default settings and one center node {@graph {"ann":{"s":[30,30],"d":1, "c":1, "bg": 0}, "styles" : {"6" : {"a" : 0.5,"c" : 1,"bg" : 1,"f" : null}},"n":[[-168.28,-179.72],[-227.35,-89.37],[195.73,63.85],[208.34,-42.28],[-239.94,-9.73],[-217.86,82.33],[154.4,133.08],[-136.84,174.15],[96.65,-210.34],[-286.03,-236.46],[-80.8,-230.16],[-216.41,-301.24],[-345.33,-130.69],[-330.56,136.4],[-204.53,279.24],[-48.25,207.53],[170.9,-141.76],[-0.72,-239.55],[59.22,197.41],[153.75,291.63],[64.99,325.74],[93.56,447.43],[158.68,-318.87],[-415.1,241.02],[-465.07,136.87],[-272.22,384.33],[332.42,-57.43],[0.83,-113.74,1],[49.15,61.71,1],[80.56,-44.46,1],[-89.99,51.16,1],[-97,-72.24,1],[-465.42,-448.52,920,920,6],[-355.1,-344.73,700,700,6],[-226.03,-218.04,445,445,6],[-97,-100.15,194,194,6]],"e":[[0,9],[0,10],[0,11],[1,12],[1,13],[13,5],[7,14],[19,18],[20,18],[20,21],[8,22],[13,23],[23,14],[13,24],[14,25],[3,26],[10,27],[16,27],[27,17],[27,8],[28,18],[28,6],[28,2],[29,28],[29,16],[29,27],[29,3],[7,30],[30,28],[30,15],[30,4],[30,5],[31,27],[31,0],[31,1],[31,30]],"vp":[-466.0,-449.0,921.0,927.0]}} Sample output of the layout algorithm with straight-line
     * edges and five center nodes using {@link yfiles.radial.CenterNodesPolicy#CUSTOM} policy<b>Concept</b> The layout algorithm creates a virtual
     * tree structure for the input graph whose root node is placed in the center of the circles. All children of a tree node
     * are placed on a higher layer (i.e. on a circle of larger radius) than their parent. A sector is calculated for each node
     * such that the sector of each child node in the virtual tree is a sub-sector of its parent's sector. The radii of the
     * circles are calculated based on the sizes of the sectors required to accommodate each whole subtree. <b>Features</b>
     * </p>
     * <p>
     * There are different {@link yfiles.radial.RadialLayout#centerNodesPolicy center nodes policies} that can be applied in order to determine the nodes that will be placed in the
     * center (single center node) or on the innermost circle (more than one center node). The distribution of the nodes to the
     * circles is specified by a {@link yfiles.radial.RadialLayout#layeringStrategy layering strategy}.
     * </p>
     * <p>
     * The edges are routed based on the
     * {@link yfiles.radial.RadialLayout#edgeRoutingStrategy edge routing strategy}. Note that the edge routing cannot always prevent node-edge overlaps.
     * </p>
     * <p>
     * The minimum distance between nodes of the same circle is specified using
     * {@link yfiles.radial.RadialLayout#minimumNodeToNodeDistance}. If there are too many nodes assigned to a circle, the radius of the particular circle will be increased until all
     * nodes fit onto it.
     * </p>
     * <p>
     * The distance between consecutive layers is specified using {@link yfiles.radial.RadialLayout#layerSpacing}. The radii of the circles are a multiple of this spacing
     * value. If the layer spacing is <code>0</code>, the radii are chosen such that all nodes fit on their corresponding circles. A minimum
     * layer distance is specified by {@link yfiles.radial.RadialLayout#minimumLayerDistance}.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface RadialLayout extends yfiles.layout.MultiStageLayout{}
    export class RadialLayout {
      /**
       * Creates a new {@link yfiles.radial.RadialLayout} instance with the default settings.
       */
      constructor();
      /**
       * Data acceptor key for publishing the circle ID and sector data
       * <p>
       * For each node, the ID of the circle and data regarding the circular sector on which it was placed will be stored.
       * </p>
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.radial.RadialLayoutNodeInfo>}
       */
      static NODE_INFO_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.radial.RadialLayoutNodeInfo>;
      /**
       * Gets or sets the minimum distance between two neighboring nodes of the same circle.
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum distance is negative
       * @type {number}
       */
      minimumNodeToNodeDistance:number;
      /**
       * Gets or sets the spacing between two consecutive circles (layers).
       * <p>
       * The spacing is a non-negative value, where <code>0</code> means that no spacing is considered.
       * </p>
       * <p>
       * The radius of each circle will be a multiple of this value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the spacing value is negative
       * @type {number}
       */
      layerSpacing:number;
      /**
       * Gets or sets the minimum distance between two circles.
       * @throws {Stubs.Exceptions.ArgumentError} if the minimum layer distance is negative
       * @type {number}
       */
      minimumLayerDistance:number;
      /**
       * Gets or sets the maximum sector angle around a node within which the node's children will be placed.
       * <p>
       * Based on the location of a given node, the sector of its children that lie on the next outer layer is described by this
       * angle. The vertex of the angle is located in the center of the node while its sides point away from the inner circles.
       * The child nodes are placed inside this angle. If there is not enough space, the radius of the child circle is
       * increased.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the sector angle is negative or greater than <code>360</code> degrees
       * @type {number}
       */
      maximumChildSectorAngle:number;
      /**
       * Gets or sets the minimum bend angle (in degrees) between two adjacent edge segments.
       * <p>
       * The minimum bend angle is defined to be a value between <code>0</code> and <code>90</code> degrees.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given minimum bend angle is negative or greater than <code>90</code>
       * @type {number}
       */
      minimumBendAngle:number;
      /**
       * Gets or sets the policy that determines which nodes are selected as roots of the (virtual) tree for the layout process
       * and are set in the center of the circles.
       * @throws {Stubs.Exceptions.ArgumentError} if an invalid center node policy is given
       * @type {yfiles.radial.CenterNodesPolicy}
       */
      centerNodesPolicy:yfiles.radial.CenterNodesPolicy;
      /**
       * Gets or sets the {@link yfiles.algorithms.IDataProvider} key used for looking up the selected nodes that should be placed in the
       * center.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified key is <code>null</code>
       * @type {Object}
       */
      centerNodesDpKey:Object;
      /**
       * Gets or sets the layering strategy that determines how nodes are distributed into different layers.
       * <p>
       * All nodes of a given layer will be placed on the same circle.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if an invalid layering strategy is set
       * @type {yfiles.radial.LayeringStrategy}
       */
      layeringStrategy:yfiles.radial.LayeringStrategy;
      /**
       * Gets or sets the edge routing strategy.
       * @throws {Stubs.Exceptions.ArgumentError} if an invalid edge routing strategy is set
       * @type {yfiles.radial.EdgeRoutingStrategy}
       */
      edgeRoutingStrategy:yfiles.radial.EdgeRoutingStrategy;
      /**
       * Gets or sets whether or not node labels are taken into account when calculating node positions.
       * <p>
       * Thereby, possible node/node label or node label/node label overlaps are prevented.
       * </p>
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} that modifies the orientation of the layout is activated.
       * @see yfiles.layout.MultiStageLayout#orientationLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#orientationLayout
       * @see yfiles.layout.MultiStageLayout#layoutOrientation
       * @see yfiles.layout.OrientationLayout
       * @type {boolean}
       */
      orientationLayoutEnabled:boolean;
      /**
       * Gets the {@link yfiles.layout.EdgeBundling} instance that defines the settings of the edge bundling feature.
       * <p>
       * The specified {@link yfiles.layout.EdgeBundling} defines global bundling properties. Settings for individual edges can be
       * defined by assigning an {@link yfiles.layout.EdgeBundleDescriptor} to an edge using a {@link yfiles.algorithms.IDataProvider} registered
       * with key {@link yfiles.layout.EdgeBundling#EDGE_BUNDLE_DESCRIPTOR_DP_KEY}.
       * </p>
       * @type {yfiles.layout.EdgeBundling}
       */
      edgeBundling:yfiles.layout.EdgeBundling;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.radial.RadialLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface RadialLayoutData extends yfiles.layout.LayoutData{}
    export class RadialLayoutData {
      constructor();
      /**
       * Gets or sets the collection of nodes that shall be placed in the center with {@link yfiles.radial.RadialLayout#centerNodesPolicy center node policy} set to {@link yfiles.radial.CenterNodesPolicy#CUSTOM}
       * .
       * @see yfiles.radial.RadialLayout#centerNodesDpKey
       * @type {yfiles.layout.DpKeyItemCollection.<yfiles.graph.INode>}
       */
      centerNodes:yfiles.layout.DpKeyItemCollection<yfiles.graph.INode>;
      /**
       * Gets or sets the mapping from nodes to a {@link yfiles.radial.RadialLayoutNodeInfo} describing on which circle and in which
       * sector the node is placed.
       * <p>
       * If the mapper contains a {@link yfiles.radial.RadialLayoutNodeInfo} for a node, the layout will update its property during the
       * layout.
       * </p>
       * @see yfiles.radial.RadialLayout#NODE_INFO_DP_KEY
       * @type {yfiles.collections.IMapper.<yfiles.graph.INode,yfiles.radial.RadialLayoutNodeInfo>}
       */
      nodeInfos:yfiles.collections.IMapper<yfiles.graph.INode,yfiles.radial.RadialLayoutNodeInfo>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroups:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroups:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.layout.NodeHalo}.
       * @see yfiles.layout.NodeHalo#NODE_HALO_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.layout.NodeHalo>}
       */
      nodeHalos:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.layout.NodeHalo>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping of edges to their {@link yfiles.layout.EdgeBundleDescriptor}.
       * <p>
       * Bundling together multiple edges means that their common parts are to some degree merged into a bundled part. At the
       * source and target point, the edges are again clearly split.
       * </p>
       * <p>
       * If an edge is mapped to <code>null</code>, the {@link yfiles.layout.EdgeBundling#defaultBundleDescriptor default descriptor} is used.
       * </p>
       * @see yfiles.layout.EdgeBundleDescriptor
       * @see yfiles.radial.RadialLayout#edgeBundling
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.layout.EdgeBundleDescriptor>}
       */
      edgeBundleDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.layout.EdgeBundleDescriptor>;
      static $class:yfiles.lang.Class;
    }
  }export namespace seriesparallel{
    /**
     * {@link yfiles.seriesparallel.DefaultOutEdgeComparer} is the default implementation for the out-edge comparator used in
     * {@link yfiles.seriesparallel.SeriesParallelLayout}.
     * <p>
     * It keeps the order of the edges in the graph for edges without {@link yfiles.layout.PortConstraint}s and considers the
     * {@link yfiles.layout.PortConstraint}s such that the order of the target nodes of the outgoing edges is similar to the port
     * location at the source node. Note that this won't work for all cases because the target nodes may not be completely
     * independent.
     * </p>
     * @see yfiles.seriesparallel.SeriesParallelLayout#defaultOutEdgeComparer
     * @class
     * @implements {yfiles.collections.IComparer.<Object>}
     */
    export interface DefaultOutEdgeComparer extends Object,yfiles.collections.IComparer<Object>{}
    export class DefaultOutEdgeComparer {
      /**
       * Creates a {@link yfiles.seriesparallel.DefaultOutEdgeComparer} instance with default settings.
       */
      constructor();
      /**
       * Compares the two given edges considering their {@link yfiles.layout.PortConstraint}s.
       * <p>
       * The order of edges is kept unless there are
       * {@link yfiles.layout.PortConstraint}s assigned. Then the edges will be sorted to avoid edge crossings.
       * </p>
       * @param o1 the first edge
       * @param o2 the second edge
       * @returns the order of the given two edges
       */
      compare(o1:Object,o2:Object):number;
      static $class:yfiles.lang.Class;
    }
    /**
     * {@link yfiles.seriesparallel.DefaultPortAssignment} implements the default port assignment strategy.
     * <p>
     * Ports are either placed at the {@link yfiles.seriesparallel.PortAssignmentMode#CENTER center} or get {@link yfiles.seriesparallel.PortAssignmentMode#DISTRIBUTED distributed} at the sides of the node.
     * </p>
     * <p>
     * If {@link yfiles.layout.PortConstraint}s are specified, the strong {@link yfiles.layout.PortConstraint}s are handled separately and keep
     * their initial location relative to the node while weak
     * {@link yfiles.layout.PortConstraint}s just influence on which side of the node the port is placed.
     * </p>
     * <p>
     * If there are edge groups specified, the edges which belong to the same group get the same port locations.
     * </p>
     * @see yfiles.layout.PortConstraint
     * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
     * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
     * @class
     * @implements {yfiles.seriesparallel.IPortAssignment}
     */
    export interface DefaultPortAssignment extends Object,yfiles.seriesparallel.IPortAssignment{}
    export class DefaultPortAssignment {
      /**
       * Creates a new {@link yfiles.seriesparallel.DefaultPortAssignment} instance using the given {@link yfiles.seriesparallel.DefaultPortAssignment#mode mode} and a given value for the {@link yfiles.seriesparallel.DefaultPortAssignment#borderGapToPortGapRatio ratio}
       * of the gap between the border and the ports to the gap between the ports themselves.
       * @param mode one of the predefined port assignment modes
       * @param [ratio=0.5] the ratio of the distance between outer ports and the border to the distance between any two ports
       * @throws {Stubs.Exceptions.ArgumentError} if the given mode is unknown or the given ratio is negative
       */
      constructor(mode:yfiles.seriesparallel.PortAssignmentMode,ratio?:number);
      /**
       * Creates a new {@link yfiles.seriesparallel.DefaultPortAssignment} with default settings.
       */
      constructor();
      /**
       * Places the ports of edges connecting to the given node according to the {@link yfiles.seriesparallel.DefaultPortAssignment#mode port assignment mode}.
       * @param graph the input graph
       * @param node the node whose adjacent edges' ports should be placed
       * @see yfiles.seriesparallel.DefaultPortAssignment#mode
       */
      assignPorts(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):void;
      /**
       * Determines the distance between the outer ports and the border of the node.
       * <p>
       * This implementation calculates the gap according to the number of edges at one side of the node. It also includes the
       * {@link yfiles.seriesparallel.DefaultPortAssignment#borderGapToPortGapRatio ratio between border gap and port gap}.
       * </p>
       * <p>
       * This method is called by {@link yfiles.seriesparallel.DefaultPortAssignment#assignPorts} to calculate the location of the first
       * port. It may be overridden to implement a different distribution of ports on the node's side.
       * </p>
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges that connect to this side
       * @returns the absolute gap to be used on both sides of the ports
       * @see yfiles.seriesparallel.DefaultPortAssignment#assignPorts
       * @see yfiles.seriesparallel.DefaultPortAssignment#borderGapToPortGapRatio
       * @protected
       */
      getPortBorderGap(sideLength:number,edgeCount:number):number;
      /**
       * Determines the distance between two adjacent ports.
       * <p>
       * This implementation calculates the gap according to the number of edges at one side of the node. The edges are
       * distributed keeping the given gap to the border of the node.
       * </p>
       * <p>
       * This method is called by {@link yfiles.seriesparallel.DefaultPortAssignment#assignPorts} to calculate the distance between two
       * ports. It may be overridden to implement a different distribution of ports on the node's side.
       * </p>
       * @param sideLength the width/height of the side
       * @param edgeCount the number of edges/ports that connect to this side
       * @param portBorderGap the port border gap (i.e. calculated by {@link #getPortBorderGap})
       * @returns the absolute distance to be used between two adjacent ports
       * @see yfiles.seriesparallel.DefaultPortAssignment#assignPorts
       * @see yfiles.seriesparallel.DefaultPortAssignment#getPortBorderGap
       * @protected
       */
      getPortDistanceDelta(sideLength:number,edgeCount:number,portBorderGap:number):number;
      /**
       * Gets or sets the mode that describes how ports are placed.
       * @throws {Stubs.Exceptions.ArgumentError} if an unknown mode is specified
       * @type {yfiles.seriesparallel.PortAssignmentMode}
       */
      mode:yfiles.seriesparallel.PortAssignmentMode;
      /**
       * Gets or sets the ratio of the <em>border gap</em> (distance between the outer ports and the node border) to the <em>port gap</em> (distance between adjacent
       * ports).
       * <p>
       * The ratio must have a positive value. If the ratio is
       * <code>0</code>, the ports will be distributed along the whole side of the node without a gap between the last port and the corner.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified ratio is negative
       * @type {number}
       */
      borderGapToPortGapRatio:number;
      /**
       * Gets or sets the fork style which influences the port assignment.
       * <p>
       * If the fork style is set to
       * {@link yfiles.seriesparallel.ForkStyle#OUTSIDE_NODE}, edges without port constraints will be assigned in flow direction.
       * </p>
       * <p>
       * If the fork style is set to
       * {@link yfiles.seriesparallel.ForkStyle#AT_NODE}, the ports of edges in flow direction also get distributed to the sides unless
       * they are without bends. Due to large minimum first/last segment lengths edges may overlap.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified fork style is unknown
       * @type {yfiles.seriesparallel.ForkStyle}
       */
      forkStyle:yfiles.seriesparallel.ForkStyle;
      static $class:yfiles.lang.Class;
    }
    /**
     * An {@link yfiles.seriesparallel.EdgeLayoutDescriptor} provides routing details for an edge that are considered by
     * {@link yfiles.seriesparallel.SeriesParallelLayout}.
     * <p>
     * Such an {@link yfiles.seriesparallel.EdgeLayoutDescriptor} can either be specified for a single edge or shared by several edges.
     * It is assigned by registering a {@link yfiles.algorithms.IDataProvider} with key
     * {@link yfiles.seriesparallel.SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY}. If no {@link yfiles.algorithms.IDataProvider} is
     * registered with this key, {@link yfiles.seriesparallel.SeriesParallelLayout#defaultEdgeLayoutDescriptor a default descriptor} is used.
     * </p>
     * @see yfiles.seriesparallel.SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
     * @see yfiles.seriesparallel.SeriesParallelLayout#defaultEdgeLayoutDescriptor
     * @class
     */
    export interface EdgeLayoutDescriptor extends Object{}
    export class EdgeLayoutDescriptor {
      /**
       * Creates a new instance of {@link yfiles.seriesparallel.EdgeLayoutDescriptor} with default settings.
       */
      constructor();
      /**
       * Creates a copy of this {@link yfiles.seriesparallel.EdgeLayoutDescriptor} instance.
       * @returns a copy of this {@link } instance
       */
      createCopy():yfiles.seriesparallel.EdgeLayoutDescriptor;
      /**
       * Gets or sets the minimum length for the associated edge.
       * <p>
       * The minimum length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} for negative edge lengths
       * @type {number}
       */
      minimumLength:number;
      /**
       * Gets or sets the minimum length for the first segment of the associated edge.
       * <p>
       * The minimum first segment length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} for negative minimum segment lengths
       * @type {number}
       */
      minimumFirstSegmentLength:number;
      /**
       * Gets or sets the minimum length for the last segment of the associated edge.
       * <p>
       * The minimum last segment length needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} for negative minimum segment lengths
       * @type {number}
       */
      minimumLastSegmentLength:number;
      static $class:yfiles.lang.Class;
    }
    export enum PortAssignmentMode{
      /**
       * A port assignment specifier which defines that all ports are reset to the center of their nodes.
       * @see yfiles.layout.PortConstraint
       */
      CENTER,
      /**
       * A port assignment specifier which defines that the ports of incoming and outgoing edges are distributed evenly at the
       * side of their nodes.
       * <p>
       * The source ports of edges without {@link yfiles.layout.PortConstraint}s will be distributed at the sides of their source and
       * target nodes. Edges with weak {@link yfiles.layout.PortConstraint}s are distributed at the side of the nodes which is specified
       * in the {@link yfiles.layout.PortConstraint}.
       * </p>
       * <p>
       * Grouped edges will get the same port location.
       * </p>
       * @see yfiles.layout.PortConstraint
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       */
      DISTRIBUTED
    }
    export enum ForkStyle{
      /**
       * Fork style specifier that defines that the edges will fork outside a node. Edges leave/enter the nodes south/north and
       * bend between their source and target.
       */
      OUTSIDE_NODE,
      /**
       * Fork style specifier that defines that the edges will fork on a node. Edges leave/enter the nodes south/north if they
       * are straight (no bends) or east and west of the node.
       */
      AT_NODE
    }
    export enum RoutingStyle{
      /**
       * Style constant describing an orthogonal edge style. If this style is used, edges will contain only horizontal and
       * vertical segments.
       * @see yfiles.seriesparallel.SeriesParallelLayout#routingStyle
       */
      ORTHOGONAL,
      /**
       * Style constant describing an octilinear edge style. If this style is used, edges will contain horizontal, vertical and
       * <code>45</code>-degree sloped segments.
       * @see yfiles.seriesparallel.SeriesParallelLayout#routingStyle
       */
      OCTILINEAR,
      /**
       * Style constant describing a polyline edge style. If this style is used, edges will contain vertical and other sloped
       * segments.
       * @see yfiles.seriesparallel.SeriesParallelLayout#routingStyle
       */
      POLYLINE
    }
    /**
     * A {@link yfiles.seriesparallel.IPortAssignment} is used by {@link yfiles.seriesparallel.SeriesParallelLayout} for determining the
     * locations of the edges' ports at their incident nodes.
     * @interface
     */
    export interface IPortAssignment extends Object{
      /**
       * Assigns both the incoming edges' target ports as well as all source ports for the outgoing edges.
       * <p>
       * Outgoing edges are sorted according to the specified out-edge comparator from left to right and incoming edges are in
       * the order of the subgraphs they are coming from.
       * </p>
       * <p>
       * This method is called by {@link yfiles.seriesparallel.SeriesParallelLayout} before the actual layout of the graph takes place.
       * </p>
       * @param graph the graph instance the node is part of
       * @param node the node whose incident edges' ports are assigned
       * @see yfiles.seriesparallel.SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY
       * @see yfiles.seriesparallel.SeriesParallelLayout#defaultOutEdgeComparer
       * @abstract
       */
      assignPorts(graph:yfiles.layout.LayoutGraph,node:yfiles.algorithms.Node):void;
    }
    var IPortAssignment:{
      $class:yfiles.lang.Class;
      isInstance(o:Object):boolean;
    };
    /**
     * This layout algorithm arranges series-parallel graphs.
     * <p>
     * <b>Layout Style</b> Series-parallel graphs are directed graphs with a single source (node without incoming edges) and a single sink (node
     * without outgoing edges). The layout algorithm highlights the main layout direction (from source to sink). It also
     * emphasizes the paths through the graph because edges are routed with few bends.
     * </p>
     * <p>
     * {@link yfiles.seriesparallel.SeriesParallelLayout} is suitable for the visualization of circuits, call trees or flowcharts.
     * </p>
     * <p>
     * {@graph {"ann":{"s":[30,30],"d":1},"n":[[226,0],[226,480],[278,60],[278,420],[0,270],[360,300],[300,300],[240,300],[240,180],[60,240],[0,330],[180,120],[120,240],[0,150],[180,270],[180,210],[338,180],[300,240],[436,240],[0,210],[240,240],[180,360],[360,240]],"e":[[0,1,[241,45,481.5,45,481.5,465,241,465]],[0,2,[241,45,293,45]],[3,1,[293,465,241,465]],[5,3,[375,405,293,405]],[6,3,[315,405,293,405]],[0,9,[241,45,75,45]],[9,1,[75,465,241,465]],[4,10],[10,1,[15,465,241,465]],[2,11,[293,105,195,105]],[11,8,[195,165,255,165]],[11,12,[195,165,135,165]],[0,13,[241,45,15,45]],[11,15],[15,14],[2,16,[293,105,353,105]],[16,3,[353,225,405.5,225,405.5,405,293,405]],[16,17,[353,225,315,225]],[17,6],[2,18,[293,105,451,105]],[18,3,[451,405,293,405]],[13,19],[19,4],[8,20],[20,7],[7,21,[255,345,195,345]],[14,21],[12,21,[135,345,195,345]],[21,3,[195,405,293,405]],[16,22,[353,225,375,225]],[22,5]],"vp":[0.0,0.0,482.0,510.0]}}<b>Concept</b> Series-parallel graphs are directed graphs with a single source (node without incoming edges) and a single sink (node
     * without outgoing edges) that are built using only the following two rules:
     * </p>
     * <ol>
     * <li><b>Series composition:</b> The source of a subgraph is merged with the sink of a second subgraph.</li>
     * <li><b>Parallel composition:</b> The sources and sinks of two subgraphs are merged.</li>
     * </ol>
     * <p>
     * From the recursive structure of series-parallel graphs, the layout algorithm retrieves a decomposition tree where each
     * node represents one of the decomposition types. Then, this tree is traversed recursively from bottom to top, aligning
     * subgraphs above (series) or next to (parallel) each other until the whole graph is arranged. The edges are routed when
     * both end nodes are placed. Different {@link yfiles.seriesparallel.SeriesParallelLayout#routingStyle routing styles} can be used.
     * </p>
     * <p>
     * To avoid moving all nodes several times and to be aware of the area that the subtrees occupy, the layout algorithm keeps
     * track of the shape of the subtrees. These shapes are moved and merged during the layout calculation. The layout
     * algorithm also stores the connections between nodes and nodes that haven't already been placed in these shapes.
     * </p>
     * <p>
     * <b>Features</b>
     * </p>
     * <p>
     * {@link yfiles.seriesparallel.SeriesParallelLayout} can take strong
     * {@link yfiles.layout.PortConstraint}s into account. It will connect the edges to the specified locations, the directions,
     * however, will be ignored.
     * </p>
     * <p>
     * Grouping of nodes can also be handled by this layout algorithm. It is important that a group node contains a whole
     * series-parallel subgraph. Otherwise, the group nodes may overlap with each other or with other nodes. Edges which are
     * connected to non-empty group nodes are not allowed. Furthermore, the user may specify minimum size constraints for each
     * group node using {@link yfiles.algorithms.IDataProvider} key {@link yfiles.layout.GroupingKeys#MINIMUM_NODE_SIZE_DP_KEY}.
     * </p>
     * <p>
     * The layout algorithm can be configured to reserve space for node labels and to place the edge labels along the edge such
     * that the labels won't overlap with other graph elements. Edge labels are placed according to the information stored in a
     * {@link yfiles.layout.PreferredPlacementDescriptor} instance. However, labels that should be centered between source and target
     * are placed close to the target node, unless this edge connects to the local source and sink of a subgraph.
     * </p>
     * <p>
     * Parallel subgraphs can be aligned in different ways. Depending on the node sizes, a different alignment can increase the
     * compactness of the layout.
     * </p>
     * <p>
     * The way in which edges are distributed around their incident nodes is computed by an instance of
     * {@link yfiles.seriesparallel.IPortAssignment}. The {@link yfiles.seriesparallel.DefaultPortAssignment default assignment} is able to consider
     * {@link yfiles.layout.PortConstraint}s and edge groups.
     * </p>
     * <p>
     * The {@link yfiles.seriesparallel.SeriesParallelLayout#fromSketchMode From Sketch mode} allows to take the initial locations of the nodes into account. However, the layout algorithm won't
     * insert crossings because it maintains the order of children of each node.
     * </p>
     * <p>
     * {@link yfiles.seriesparallel.SeriesParallelLayout} supports custom sorting of the outgoing edges of a node. A {@link yfiles.collections.IComparer.<T>} can be assigned
     * individually for the nodes using a {@link yfiles.algorithms.IDataProvider} registered with key
     * {@link yfiles.seriesparallel.SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY}. For all nodes for which the {@link yfiles.algorithms.IDataProvider}
     * returns <code>null</code>, the layout algorithm falls back to the {@link yfiles.seriesparallel.SeriesParallelLayout#defaultOutEdgeComparer default comparator}.
     * </p>
     * <p>
     * By default, this layout algorithm can only handle graphs with a series-parallel structure. To apply it to a general
     * graph, {@link yfiles.seriesparallel.SeriesParallelLayout#generalGraphHandling general graph handling} needs to be activated. Then, the layout algorithm will temporarily add and/or remove some
     * edges from the input graph until a series-parallel graph is obtained. The edges that were removed will be routed
     * separately afterwards.
     * </p>
     * @class
     * @extends {yfiles.layout.MultiStageLayout}
     */
    export interface SeriesParallelLayout extends yfiles.layout.MultiStageLayout{}
    export class SeriesParallelLayout {
      /**
       * Creates a new {@link yfiles.seriesparallel.SeriesParallelLayout} instance with default settings.
       */
      constructor();
      /**
       * Determines whether or not the given graph has a series-parallel structure.
       * <p>
       * The current implementation detects the series-parallel graph structure in linear time.
       * </p>
       * @param graph the input graph
       * @returns <code>true</code> if the given graph is series-parallel, <code>false</code> otherwise
       * @static
       */
      static isSeriesParallelGraph(graph:yfiles.algorithms.Graph):boolean;
      /**
       * Data provider key for storing individual settings for edges
       * <p>
       * If no edge layout descriptor is mapped for an edge, a default edge layout descriptor will be obtained by method {@link yfiles.seriesparallel.SeriesParallelLayout#defaultEdgeLayoutDescriptor}.
       * </p>
       * @see yfiles.seriesparallel.SeriesParallelLayout#defaultEdgeLayoutDescriptor
       * @const
       * @static
       * @type {yfiles.algorithms.EdgeDpKey.<yfiles.seriesparallel.EdgeLayoutDescriptor>}
       */
      static EDGE_LAYOUT_DESCRIPTOR_DP_KEY:yfiles.algorithms.EdgeDpKey<yfiles.seriesparallel.EdgeLayoutDescriptor>;
      /**
       * Data provider key for assigning different orderings for outgoing edges of the nodes
       * <p>
       * If the {@link yfiles.collections.IComparer.<T>} associated with a node is <code>null</code>, the outgoing edges maintain their initial order.
       * </p>
       * @see yfiles.seriesparallel.SeriesParallelLayout#defaultOutEdgeComparer
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.collections.IComparer.<Object>>}
       */
      static OUT_EDGE_COMPARER_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.collections.IComparer<Object>>;
      /**
       * Data provider key for providing an individual port distribution at nodes
       * <p>
       * If there is no {@link yfiles.seriesparallel.IPortAssignment} specified for a node, the layout algorithm uses the
       * {@link yfiles.seriesparallel.SeriesParallelLayout#defaultPortAssignment default assignment}.
       * </p>
       * @see yfiles.seriesparallel.SeriesParallelLayout#defaultPortAssignment
       * @const
       * @static
       * @type {yfiles.algorithms.NodeDpKey.<yfiles.seriesparallel.IPortAssignment>}
       */
      static PORT_ASSIGNMENT_DP_KEY:yfiles.algorithms.NodeDpKey<yfiles.seriesparallel.IPortAssignment>;
      /**
       * Data provider key for marking edge labels of non-series-parallel edges
       * @see yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey
       * @see yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm
       * @const
       * @static
       * @type {yfiles.algorithms.IEdgeLabelLayoutDpKey.<boolean>}
       */
      static NON_SERIES_PARALLEL_EDGE_LABELS_DP_KEY:yfiles.algorithms.IEdgeLabelLayoutDpKey<boolean>;
      /**
       * Gets or sets whether or not the layout algorithm reserves space for node labels to avoid overlaps.
       * @type {boolean}
       */
      considerNodeLabels:boolean;
      /**
       * Gets or sets whether or not the layout algorithm will place edge labels and reserve space for them.
       * @type {boolean}
       */
      integratedEdgeLabeling:boolean;
      /**
       * Gets or sets the vertical alignment of parallel subgraphs.
       * <p>
       * The alignment is defined by a ratio:
       * </p>
       * <ul>
       * <li>Ratio <code>0</code> means that nodes are top-aligned</li>
       * <li>Ratio <code>0.5</code> means that nodes are center-aligned</li>
       * <li>Ratio <code>1</code> means that nodes are bottom-aligned</li>
       * </ul>
       * @type {number}
       */
      verticalAlignment:number;
      /**
       * Gets or sets whether or not the layout algorithm can handle general graphs.
       * <p>
       * General graphs are required to be transformed before being laid out (by adding or removing some edges) such that they
       * satisfy the criteria of a series-parallel graph. After the layout, the graph will be restored and the initial edges
       * which weren't included in the series-parallel graph are routed by a separate {@link yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeRouter edge routing algorithm}. Furthermore, the
       * labels of these edges are placed by a customizable {@link yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm edge labeling algorithm}.
       * </p>
       * @type {boolean}
       */
      generalGraphHandling:boolean;
      /**
       * Gets or sets the edge routing algorithm used for the edges of a general graph that are not part of the series-parallel
       * subgraph whose layout is calculated.
       * @throws {Stubs.Exceptions.ArgumentError} if <code>null</code> is specified
       * @see yfiles.seriesparallel.SeriesParallelLayout#generalGraphHandling
       * @see yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgesDpKey
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      nonSeriesParallelEdgeRouter:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that is used for marking non-series-parallel edges.
       * <p>
       * This key is used for determining the edges that are not part of the series-parallel structure in a general graph, such
       * that the specified non-series-parallel edge router only routes marked edges.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the given key is set to <code>null</code>
       * @see yfiles.seriesparallel.SeriesParallelLayout#generalGraphHandling
       * @see yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeRouter
       * @type {Object}
       */
      nonSeriesParallelEdgesDpKey:Object;
      /**
       * Gets or sets the labeling algorithm that is applied to all edge labels that belong to non-series-parallel edges.
       * @see yfiles.seriesparallel.SeriesParallelLayout#generalGraphHandling
       * @see yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeLabelSelectionKey
       * @type {yfiles.layout.ILayoutAlgorithm}
       */
      nonSeriesParallelEdgeLabelingAlgorithm:yfiles.layout.ILayoutAlgorithm;
      /**
       * Gets or sets the key to register a {@link yfiles.algorithms.IDataProvider} that is used by the {@link yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm non-series-parallel edge labeling algorithm}
       * to determine which edge labels it should place.
       * <p>
       * During the layout, a {@link yfiles.algorithms.IDataProvider} with this key will be registered with the graph. It will mark all
       * {@link yfiles.layout.IEdgeLabelLayout}s that belong to non-series-parallel edges. A specified custom non-series-parallel edge
       * labeling algorithm needs to obey this selection. If using {@link yfiles.labeling.GenericLabeling} as labeling algorithm, set
       * this key as value of property {@link yfiles.labeling.LabelingBase#affectedLabelsDpKey}.
       * </p>
       * <p>
       * The labeling algorithm set as default is already configured such that it uses the correct selection key.
       * </p>
       * @see yfiles.seriesparallel.SeriesParallelLayout#nonSeriesParallelEdgeLabelingAlgorithm
       * @type {Object}
       */
      nonSeriesParallelEdgeLabelSelectionKey:Object;
      /**
       * Gets or sets the default {@link yfiles.seriesparallel.IPortAssignment} used for those nodes that do not have their own specific
       * instance.
       * <p>
       * A {@link yfiles.seriesparallel.IPortAssignment} instance is defined using a {@link yfiles.algorithms.IDataProvider} registered with the
       * graph with key {@link yfiles.seriesparallel.SeriesParallelLayout#PORT_ASSIGNMENT_DP_KEY}.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if <code>null</code> is specified
       * @type {yfiles.seriesparallel.IPortAssignment}
       */
      defaultPortAssignment:yfiles.seriesparallel.IPortAssignment;
      /**
       * Gets or sets the default {@link yfiles.collections.IComparer.<T>} used for sorting the outgoing edges incident to nodes that do not have a specific {@link yfiles.collections.IComparer.<T>}.
       * <p>
       * Such a comparator is defined using a {@link yfiles.algorithms.IDataProvider} registered with the graph with key
       * {@link yfiles.seriesparallel.SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY}.
       * </p>
       * @type {yfiles.collections.IComparer.<Object>}
       */
      defaultOutEdgeComparer:yfiles.collections.IComparer<Object>;
      /**
       * Gets or sets the currently used routing style for edges.
       * @type {yfiles.seriesparallel.RoutingStyle}
       */
      routingStyle:yfiles.seriesparallel.RoutingStyle;
      /**
       * Gets or sets the minimum vertical distance of the edge segments that are not orthogonal.
       * <p>
       * The distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified length is smaller than <code>0</code>
       * @see yfiles.seriesparallel.SeriesParallelLayout#minimumSlope
       * @see yfiles.seriesparallel.SeriesParallelLayout#routingStyle
       * @see yfiles.seriesparallel.RoutingStyle#POLYLINE
       * @type {number}
       */
      minimumPolylineSegmentLength:number;
      /**
       * Gets or sets the minimum slope which a non-orthogonal edge segment should have.
       * <p>
       * A higher minimum slope prevents those segments in very wide graphs from becoming nearly horizontal. The slope needs to
       * have a non-negative value.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified slope is smaller than <code>0</code>
       * @see yfiles.seriesparallel.SeriesParallelLayout#minimumPolylineSegmentLength
       * @see yfiles.seriesparallel.SeriesParallelLayout#routingStyle
       * @see yfiles.seriesparallel.RoutingStyle#POLYLINE
       * @type {number}
       */
      minimumSlope:number;
      /**
       * Gets or sets the preferred length for non-orthogonal segments in octilinear edge routes.
       * <p>
       * If there is not enough space, those segments may be shorter than the specified length. The length needs to be
       * non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified length is smaller than <code>0</code>
       * @see yfiles.seriesparallel.SeriesParallelLayout#routingStyle
       * @see yfiles.seriesparallel.RoutingStyle#OCTILINEAR
       * @type {number}
       */
      preferredOctilinearSegmentLength:number;
      /**
       * Gets or sets the {@link yfiles.seriesparallel.EdgeLayoutDescriptor} instance used for all those edges that do not have a
       * specific layout descriptor assigned.
       * @throws {Stubs.Exceptions.ArgumentError} if the specified {@link yfiles.seriesparallel.EdgeLayoutDescriptor} is <code>null</code>
       * @see yfiles.seriesparallel.SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.seriesparallel.EdgeLayoutDescriptor}
       */
      defaultEdgeLayoutDescriptor:yfiles.seriesparallel.EdgeLayoutDescriptor;
      /**
       * Gets or sets the minimum distance between nodes.
       * <p>
       * This distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is smaller than <code>0</code>
       * @type {number}
       */
      minimumNodeToNodeDistance:number;
      /**
       * Gets or sets the minimum distance between nodes and edges.
       * <p>
       * This distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is smaller than <code>0</code>
       * @type {number}
       */
      minimumNodeToEdgeDistance:number;
      /**
       * Gets or sets the minimum distance between edges.
       * <p>
       * This distance needs to be non-negative.
       * </p>
       * @throws {Stubs.Exceptions.ArgumentError} if the specified distance is smaller than <code>0</code>
       * @type {number}
       */
      minimumEdgeToEdgeDistance:number;
      /**
       * Gets or sets whether or not to take the coordinates of the input diagram into account when arranging the nodes.
       * <p>
       * The order of edges incident to the same source node will remain the same as in the original layout.
       * </p>
       * @type {boolean}
       */
      fromSketchMode:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for arranging the components of the graph is activated.
       * @see yfiles.layout.MultiStageLayout#componentLayoutEnabled
       * @see yfiles.layout.MultiStageLayout#componentLayout
       * @see yfiles.layout.ComponentLayout
       * @type {boolean}
       */
      componentLayoutEnabled:boolean;
      /**
       * Sets whether or not the {@link yfiles.layout.ILayoutStage} used for routing self-loops is activated.
       * @see yfiles.layout.MultiStageLayout#selfLoopRouterEnabled
       * @see yfiles.layout.MultiStageLayout#selfLoopRouter
       * @see yfiles.layout.SelfLoopRouter
       * @type {boolean}
       */
      selfLoopRouterEnabled:boolean;
      static $class:yfiles.lang.Class;
    }
    /**
     * Specifies custom data for the {@link yfiles.seriesparallel.SeriesParallelLayout}.
     * @class
     * @extends {yfiles.layout.LayoutData}
     */
    export interface SeriesParallelLayoutData extends yfiles.layout.LayoutData{}
    export class SeriesParallelLayoutData {
      constructor();
      /**
       * Gets or sets the mapping from edges to their {@link yfiles.seriesparallel.EdgeLayoutDescriptor}.
       * @see yfiles.seriesparallel.EdgeLayoutDescriptor
       * @see yfiles.seriesparallel.SeriesParallelLayout#EDGE_LAYOUT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,yfiles.seriesparallel.EdgeLayoutDescriptor>}
       */
      edgeLayoutDescriptors:yfiles.layout.ItemMapping<yfiles.graph.IEdge,yfiles.seriesparallel.EdgeLayoutDescriptor>;
      /**
       * Gets or sets the mapping from nodes to their {@link yfiles.seriesparallel.IPortAssignment}.
       * @see yfiles.seriesparallel.SeriesParallelLayout#PORT_ASSIGNMENT_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,yfiles.seriesparallel.IPortAssignment>}
       */
      portAssignments:yfiles.layout.ItemMapping<yfiles.graph.INode,yfiles.seriesparallel.IPortAssignment>;
      /**
       * Gets or sets the mapping from nodes to comparison functions used to sort the nodes' outgoing edges.
       * @see yfiles.seriesparallel.SeriesParallelLayout#OUT_EDGE_COMPARER_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.INode,Function>}
       */
      outEdgeComparers:yfiles.layout.ItemMapping<yfiles.graph.INode,Function>;
      /**
       * Gets or sets a mapping from edges to an object representing their source edge group.
       * @see yfiles.layout.PortConstraintKeys#SOURCE_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      sourceGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets a mapping from edges to an object representing their target edge group.
       * @see yfiles.layout.PortConstraintKeys#TARGET_GROUP_ID_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.IEdge,Object>}
       */
      targetGroupIds:yfiles.layout.ItemMapping<yfiles.graph.IEdge,Object>;
      /**
       * Gets or sets the AbortHandler used during the layout.
       * @see yfiles.algorithms.AbortHandler#ABORT_HANDLER_DP_KEY
       * @type {yfiles.algorithms.AbortHandler}
       */
      abortHandler:yfiles.algorithms.AbortHandler;
      /**
       * Gets or sets the mapping that provides a {@link yfiles.layout.PreferredPlacementDescriptor} instance for edge
       * {@link yfiles.graph.ILabel}s.
       * @see yfiles.layout.PreferredPlacementDescriptor
       * @see yfiles.layout.LayoutGraphAdapter#EDGE_LABEL_LAYOUT_PREFERRED_PLACEMENT_DESCRIPTOR_DP_KEY
       * @type {yfiles.layout.ItemMapping.<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>}
       */
      edgeLabelPreferredPlacement:yfiles.layout.ItemMapping<yfiles.graph.ILabel,yfiles.layout.PreferredPlacementDescriptor>;
      static $class:yfiles.lang.Class;
    }
  }}

type yfiles_namespace = typeof yfiles;
declare module 'yfiles/algorithms' { export = yfiles; }
declare module 'yfiles/complete' { export = yfiles; }
declare module 'yfiles/lang' { export = yfiles.lang; }
declare module 'yfiles/layout' { export = yfiles; }
declare module 'yfiles/layout-familytree' { export = yfiles; }
declare module 'yfiles/layout-hierarchic' { export = yfiles; }
declare module 'yfiles/layout-multipage' { export = yfiles; }
declare module 'yfiles/layout-organic' { export = yfiles; }
declare module 'yfiles/layout-orthogonal' { export = yfiles; }
declare module 'yfiles/layout-orthogonal-compact' { export = yfiles; }
declare module 'yfiles/layout-partial' { export = yfiles; }
declare module 'yfiles/layout-radial' { export = yfiles; }
declare module 'yfiles/layout-seriesparallel' { export = yfiles; }
declare module 'yfiles/layout-tree' { export = yfiles; }
declare module 'yfiles/router-other' { export = yfiles; }
declare module 'yfiles/router-polyline' { export = yfiles; }
declare module 'yfiles/view' { export = yfiles; }
declare module 'yfiles/view-component' { export = yfiles; }
declare module 'yfiles/view-editor' { export = yfiles; }
declare module 'yfiles/view-folding' { export = yfiles; }
declare module 'yfiles/view-graphml' { export = yfiles; }
declare module 'yfiles/view-layout-bridge' { export = yfiles; }
declare module 'yfiles/view-table' { export = yfiles; }